--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...,"Base.BaseModel")
local MainChapterConfig = require("Metadatas.MainChapterConfig")
local GamePlayer = require("Player.GamePlayer")
local TerrBuildManager = require("Territory.TerrBuildManager")

function This:init(id)
	self.m_id = id
	self.m_achieveList = {}
	self.m_config = MainChapterConfig[id]
	self.m_achieveKey = {}
	for i , id in ipairs(self.m_config) do
		self.m_achieveKey[id] = true
	end
	self.m_isOpen = self:isLevelEnough()
	self.m_preChapter = nil
end

function This:getId()
	return self.m_id
end

function This:getAchieveList()
	local result = {}
	for i , achieve in ipairs(self.m_achieveList) do
		table.insert(result,achieve)
	end
	table.sort(result,function(a,b)
		local as = a:getState()
		local bs = b:getState()
		if as == AchieveState.kHaseFinish and bs ~= AchieveState.kHaseFinish then
			return true
		end
		if as == AchieveState.kHaseFinish and bs == AchieveState.kHaseFinish and a:getId() < b:getId() then
			return true
		end

		if as == AchieveState.kUpdating and bs == AchieveState.kHaseReward then
			return true
		end
		if as == AchieveState.kUpdating and bs == AchieveState.kUpdating and a:getId() < b:getId() then
			return true
		end
		if as == AchieveState.kHaseReward and bs == AchieveState.kHaseReward and a:getId() < b:getId() then
			return true
		end 
		return false
	end)
	return result
end

function This:getAchieveIdList()
	return self.m_config.missions
end

function This:isLevelEnough()
	if self.m_config["require"] == 1 then
		local building = TerrBuildManager:getBuildByType(TerrBuildType.kGuanfu)
		return building:getLevel() >= self.m_config.require_lv
	else
		return GamePlayer:getLevel() >= self.m_config.require_lv
	end
end

function This:getOpenDesc()
	if self.m_config["require"] == 1 then
		return string.format(Serialize[60085],self.m_config.require_lv)
	else
		return string.format(Serialize[60086],self.m_config.require_lv)
	end
end

function This:getRewardWithAchieve()
	local result = {}
	local tmp = {}
	for i , info in ipairs(self.m_config.reward) do
		local t = {id = info[1],num = info[2]}
		table.insert(result,t)
		tmp[t.id] = t
	end
	for i , achieve in ipairs(self.m_achieveList) do
		local state = achieve:getState()
		if state == AchieveState.kHaseFinish then
			local rewardList = achieve:getRewardList()
			for _, info in ipairs(rewardList) do
				if tmp[info.id] then
					tmp[info.id].num = tmp[info.id].num + info.num
				else
					table.insert(result,info)
					tmp[info.id] = info
				end
			end
		end
	end
	return result
end

function This:getRewardList()
	local result = {}
	for i , info in ipairs(self.m_config.reward) do
		table.insert(result,{id = info[1],num = info[2]})
	end
	return result
end

function This:isAchieveInChapter(id)
	return self.m_achieveKey[id] ~= nil
end

function This:getName()
	return self.m_config.name
end

function This:getTitle()
	return self.m_config.des_title
end

function This:getDesc()
	return self.m_config.des_content
end

function This:addAchieve(achieve)
	for i , ac in ipairs(self.m_achieveList) do
		if ac:getId() == achieve:getId() then
			return
		end
	end
	table.insert(self.m_achieveList,achieve)
end

function This:getUpdatingAchieveCount()
	local count = 0
	for i , achieve in ipairs(self.m_achieveList) do
		local state = achieve:getState()
		if state == AchieveState.kUpdating then
			count = count + 1
		end
	end
	return count
end

function This:getAchieve(index)
	return self.m_achieveList[index]
end

function This:getAchieveCount()
	return #self.m_achieveList
end

function This:setIsRecieveReward(isReward)
	self.m_isRecieveReward = isReward
end

function This:isRecieveReward()
	return self.m_isRecieveReward
end

function This:isFinish()
	for i , achieve in pairs(self.m_achieveList) do
		local state = achieve:getState()
		if state == AchieveState.kUpdating then
			return false
		end
	end
	return true
end

function This:isCanReward()
	local isAllFinish = true
	for i , achieve in pairs(self.m_achieveList) do
		local state = achieve:getState()
		if state == AchieveState.kHaseFinish then
			return true
		elseif state == AchieveState.kUpdating then
			isAllFinish = false
		end
	end
	if not self.m_isRecieveReward and isAllFinish then
		return true
	end
	return false
end

function This:setPreChapter(preChapter)
	if preChapter then
		self.m_preChapter = preChapter
		self.m_isOpen = self.m_preChapter:isRecieveReward() and self.m_isOpen
	end
end

function This:isOpen()
	return self.m_isOpen
end

function This:setIsOpen(isOpen)
	return self.m_isOpen
end

function This:checkOpen()
	if not self.m_isOpen then
		if not self:isLevelEnough() then
			return false
		end
		if self.m_preChapter then
			if not self.m_preChapter:isRecieveReward() then
				return false
			end
		end
		self.m_isOpen = true
		return true
	end
end

function This:getProgress()
	local progress , total = 0, 0
	for i, achieve in ipairs(self.m_achieveList) do
		local state = achieve:getState()
		if state == AchieveState.kHaseFinish or state == AchieveState.kHaseReward then
			progress = progress + 1
		end
		total = total + 1
	end
	return progress , total
end

function This:getGuideCheckCount()
	local count = 0
	for i, achieve in ipairs(self.m_achieveList) do
		local state = achieve:getState()
		if state == AchieveState.kUpdating and achieve:isNeedGuideCheck() then
			count = count + 1
		end
	end
	return count
end

--endregion
