-- @Author: superm
-- @Date:   2018-01-23 19:46:29
-- @Last Modified by:   superm
-- @Last Modified time: 2018-10-29 15:58:06
-- @File Name: AchieveFirstWinBtn.lua
-- @Description: 


local This, super = class(..., "Home.MenuButton")
local AchieveManager = require("Achieve.AchieveManager")
local GuideManager = require("Guide.GuideManager")

function This:init(config)
    super.init(self, config)
    self.m_subView = nil
    self.m_achieveIdList = config.uiclass.indices
    self.m_achieveType = config.uiclass.achieve_type
    self.m_achieves =  {}
    if self.m_achieveIdList then
        for _, id in ipairs(self.m_achieveIdList) do
            self.m_achieves[id] = AchieveManager:getAchieve(self.m_achieveType, id)
        end
    end
end

function This:addCustomListener()
    if self:hasGuideTouch() then
        GuideManager:addObserver(self)
    end
    AchieveManager:addObserver(self)
    self:refreshRedDot()
end

function This:removeCustomListener()
    AchieveManager:removeObserver(self)
    GuideManager:removeObserver(self)
end

function This:initTouchEvent()
    self:setTouch({
        isScale = false, 
        isDynamic = true, 
        onEnded = function()
	        if not self.m_subView then
	            local panel = g_scene:openDialog("Achieve.AchieveFirstWinPanel", {indices = self.m_achieveIdList, achieves = self.m_achieves})
	            self.m_subView = panel
				panel:addCloseCallback(function ()
	                self.m_subView = nil
	            end)
	        end
        end
    }) 
end

function This:getRedDotValue()
    local value = false
    if self.m_achieveIdList and self.m_achieves then
        for _, id in ipairs(self.m_achieveIdList) do
            local achieve = self.m_achieves[id]
            if achieve then
                local state = achieve:getState()
                if state == AchieveState.kHaseFinish then
                    value = true
                    break
                elseif state ~= AchieveState.kHaseReward then
                    value = nil
                end
            end
        end
    end
    return value
end

function This:onGetAchieveReward( achieve )
    local achieveId = 0
    if achieve then
        achieveId = achieve:getId()
    end

    if self.m_achieveIdList and table.contain(self.m_achieveIdList, achieveId) then
        self:refreshByEventHandler(function()
            self:refreshRedDot()
        end)
        if achieve:getState() == AchieveState.kHaseReward and self.m_subView then
            if self:getRedDotValue() == false then
                self.m_subView:close()
                self.m_subView = nil
            else
                self.m_subView:refresh()
            end
        end
    end
end

function This:onRefreshAchieve(achieveType)
    if achieveType == self.m_achieveType then
        if self.m_achieveIdList then
            for _, id in ipairs(self.m_achieveIdList) do
                self.m_achieves[id] = AchieveManager:getAchieve(self.m_achieveType, id)
            end
        end
        self:refreshByEventHandler(function()
            self:refreshRedDot()
        end)
    end
end

function This:onAchieveChanged( achieveType, gongxunList )
    if achieveType == self.m_achieveType then
        for _, achieve in ipairs(gongxunList) do
            local achieveId = achieve:getId()
            if self.m_achieveIdList and table.contain(self.m_achieveIdList, achieveId) then
                self.m_achieves[achieveId] = achieve
                self:refreshByEventHandler(function()
                    self:refreshRedDot()
                end)
            end
        end
    end
end
