-- @Author: superm
-- @Date:   2018-01-23 20:01:29
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-28 11:19:48
-- @File Name: AchieveFirstWinPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ItemNode = require("UI.ItemNode")
local AchieveManager = require("Achieve.AchieveManager")

function This:initData(info)
	if info then
        self.m_achieves = info.achieves
		self.m_achieveIdList = info.indices
		self.m_inited = false
	end
end

function This:initNode()
	self.m_curViews = {}
 	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MatchingFirstWin.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self:initReward()
    self.m_inited = true
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.lingqu:setTouchEnded(function()
        self:pickUpReward(1)
    end)

    if csNode.lingqu2 then 
        csNode.lingqu2:setTouchEnded(function()
            self:pickUpReward(2)
        end)
    end

    csNode.close:setTouchEnded(function()
		self:close()
    end) 
end

function This:initReward()
    local csNode = self.m_csNode
    local itemList
    local pkbtnList = {csNode.lingqu, csNode.lingqu2}
    local pktxtList = {csNode.yilingqu1, csNode.yilingqu2}
    for index, achieveId in ipairs(self.m_achieveIdList) do
        local achieve = self.m_achieves[achieveId]
        local pkbtn = pkbtnList[index]
        local pktxt = pktxtList[index]
        if achieve then
            local state = achieve:getState()
            local visiblity = state ~= AchieveState.kHaseReward
            pkbtn:setVisible(visiblity)
            pktxt:setVisible(not visiblity)
            if visiblity then
                if state ~= AchieveState.kHaseFinish then
                    pkbtn:setGrayShader()
                else
                    pkbtn:clearShader()
                end
            end

            local rewardList = achieve:getRewardList()
            if not itemList then
        	   itemList = rewardList
            else
                for _,v in ipairs(rewardList) do
                    table.insert(itemList, v)
                end
            end
        end
    end

    if itemList then
    	for i, item in ipairs(itemList) do
     		local itemNode = csNode["item" .. i]
     		if itemNode then
            	itemNode:setVisible(item ~= nil)
            end

            if item and itemNode then
                if not itemNode.node then
                    itemNode.node = ItemNode:create(item.id, item.num)
                    itemNode.node:setAnchorPoint(0.5, 0.5)
                    itemNode:addChild(itemNode.node)
                else
                    itemNode.node:init(item.id, item.num)
                end
            end
    	end
    end
end

function This:pickUpReward(index)
    local achieve
    local id = self.m_achieveIdList[index]
    local ac = self.m_achieves[id]
    if ac and ac:getState() == AchieveState.kHaseFinish then
        achieve = ac
    end

    if not achieve then
        g_scene:showTips(Serialize[60089])
        return
    else
        AchieveManager:onGetRewardRequest(achieve)
    end
end

function This:refresh()
    local csNode = self.m_csNode
    local pkbtnList = {csNode.lingqu, csNode.lingqu2}
    local pktxtList = {csNode.yilingqu1, csNode.yilingqu2}
    for index, achieveId in ipairs(self.m_achieveIdList) do
        local achieve = self.m_achieves[achieveId]
        local pkbtn = pkbtnList[index]
        local pktxt = pktxtList[index]
        if achieve then
            local state = achieve:getState()
            local visiblity = state ~= AchieveState.kHaseReward
            pkbtn:setVisible(visiblity)
            pktxt:setVisible(not visiblity)
            if visiblity then
                if state ~= AchieveState.kHaseFinish then
                    pkbtn:setGrayShader()
                else
                    pkbtn:clearShader()
                end
            end
        end
    end
end


