-- @Author: zheng
-- @Date:   2019-01-30 14:57:13
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-06-17 20:12:09
-- @File Name: AchieveTimePreferentialBtn.lua


local This, super = class(..., "Home.MenuButton")
local AchieveManager = require("Achieve.AchieveManager")
local PopDialogManager = require("UI.PopDialogManager")
local NewActivityManager = require("NewActivity.NewActivityManager")
local GuideManager = require("Guide.GuideManager")
local Application = require("cc.Application")
local GamePlayer = require("Player.GamePlayer")

function This:init(config)
    super.init(self, config)
    self.m_subView = nil
    self.m_achieveType = config.uiclass.achieve_type
    self.m_activityId = config.uiclass.id
    self.m_activity = NewActivityManager:getActivity(self.m_activityId)
    self.time:setVisible(false)
    self.m_uiName = "Achieve.LimitedTimePreferentialPanel"
    self.m_escape = 0
    self:scheduleUpdate()
    self:showPanel()
end

function This:refreshTime(onlyRefreshTime)
    local achieve = self:getCurentAchieve()
    if achieve then
        local duration = achieve:getDuration()
        local str,leftTime = self.m_activity:getLeftTime(duration)
        if leftTime <=0 and not onlyRefreshTime then
             self:refreshByEventHandler(function()
				self:refreshRedDot()
			end)
        end
        self.time:setString(str)
        self.time:setVisible(leftTime>0)
    end
end

function This:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape >= 1 then
        self.m_escape = self.m_escape - 1
        self:refreshTime()
    end
end

function This:addCustomListener()
    AchieveManager:addObserver(self)
    if self.m_activity then
        self.m_activity:addObserver(self)
    end
    self:refreshRedDot()
end

function This:removeCustomListener()
    if self.m_activity then
        self.m_activity:removeObserver(self)
    end
    AchieveManager:removeObserver(self)
end

function This:showPanel()
    if not self:getCurentAchieve() or not self.m_activity or self.m_activity:getIsHaveShow() and self.m_subView then
        return
    end
    if GamePlayer:isAuditVersion() then
        return
    end
    PopDialogManager:addPopDialog({
        bCheck = true,
        name = self.m_uiName,
        level = DialogLevel.kForLevel,
        func = function()
            local achieve = self:getCurentAchieve()
            if achieve and not self.m_activity:getIsHaveShow() then 
                self.m_activity:setIsHaveShow(true)
                local panel = g_scene:openDialog(self.m_uiName,{achieve = achieve,activityId = self.m_activityId})
                self.m_subView = panel
                panel:addCloseCallback(function ()
                    self.m_subView = nil
                end)
                return panel
            end
        end
    })
end

function This:initTouchEvent()
    self:setTouch({
        isScale = false, 
        isDynamic = true, 
        onEnded = function()
            local achieve = self:getCurentAchieve()
            if not self.m_subView and achieve and not GamePlayer:isAuditVersion() then
                local panel = g_scene:openDialog(self.m_uiName,{achieve = achieve,activityId = self.m_activityId})
                self.m_subView = panel
                panel:addCloseCallback(function ()
                    self.m_subView = nil
                end)
            end
        end
    }) 
end

function This:getDefaultVisible()
    if self:isHaveTask() then
        return true
    end

    return false
end

function This:getCurentAchieve()
    if not self.m_activity then
        return false
    end
    local achieve = AchieveManager:getAchieve(self.m_achieveType, self.m_activity:getCurentAchieveId())
    if achieve and achieve:getState()~=AchieveState.kHaseReward then
        return achieve
    end
    return false
end

function This:isHaveTask()
    local result = false
    local achieve = self:getCurentAchieve()
    if achieve then
        result = true
    end
    return result
end

function This:getRedDotValue()
    local value = super.getRedDotValue(self)
    if self:isHaveTask() then
        local achieves = AchieveManager:getAllAchieves(self.m_achieveType)
        for _, achieve in pairs(achieves) do
            if achieve then
                local state = achieve:getState()
                if state == AchieveState.kHaseFinish then
                    value = true
                    break
                elseif state ~= AchieveState.kHaseReward then
                    value = nil
                end
            end
        end
    end
    return value
end

--需要
function This:onGetAchieveReward( achieve )
    local achieveId = 0
    if achieve then
        achieveId = achieve:getId()
    end
    local achieves = AchieveManager:getAllAchieves(self.m_achieveType)
    if achieves and table.contain(achieves, achieveId) then
        self:refreshByEventHandler(function()
            self:refreshRedDot()
        end)
        if achieve:getState() == AchieveState.kHaseReward and self.m_subView then
            if self:getRedDotValue() == false then
                self.m_subView:close()
                self.m_subView = nil
            else
                self.m_subView:refresh()
            end
        end
    end
end

function This:onRefreshAchieve(achieveType)
    if achieveType == self.m_achieveType then
        self:refreshByEventHandler(function()
            self:refreshRedDot()
        end)
    end
end

function This:onAchieveChanged( achieveType, gongxunList )
    if achieveType == self.m_achieveType then
        local achieves = AchieveManager:getAllAchieves(self.m_achieveType)
        for _, achieve in pairs(gongxunList) do
            local achieveId = achieve:getId()
            if achieves and table.contain(achieves, achieveId) then
                self:refreshByEventHandler(function()
                    self:refreshRedDot()
                end)
            end
        end
    end
end

function This:onGetActivityPushGiftInfo(actQueueId)
    if actQueueId and actQueueId == self.m_activityId then
        self:showPanel()
        self:refreshByEventHandler(function()
            self:refreshTime(true)
            self:refreshRedDot()
        end)
    end
end
