-- @Author: zheng
-- @Date:   2018-12-08 11:36:15
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-12-24 20:58:08
-- @File Name: CrusadeAchieveView.lua
local ItemNode = require("UI.ItemNode")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")

local This , super = class(...,"Achieve.DailyAchieveView")
This.fileName = "MenuScene/NodeUI/TaskCrusadeItem.csb"

function This:init(achieve)
    self.m_achieve = achieve
    self:getData()
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self:addChild(self.m_csNode)
    local w , h = self.m_csNode.desBG:getContentSize()
    self:setContentSize(w,h)
    self.m_csNode:setPosition(w/2,h/2)
    self:setAnchorPoint(0.5,0.5)

    local scrollview = self.m_csNode.scrollView
    scrollview:setDirection("horizontal")
    scrollview.m_direction = "right"
    scrollview:setSwallowTouch(false)
end

function This:refreshReward()
	local scrollview = self.m_csNode.scrollView
    scrollview:removeAllItems()
    local function createItemView(model)
        local node = ItemNode:create(model[1],model[2])
        node:setAnchorPoint(0.5,0.57)
        return node
    end
    local rewardList = self.m_achieve:getRewardList()
    for i,itemConfig in ipairs(rewardList) do
        local item = createItemView(itemConfig)
        local scale = 0.9
        item:setScale(scale)
        local w , h = item:getContentSize()
        scrollview:pushBackCustomItem(item,w*scale,h,0.5)
    end
end

function This:setTime(str)
    self.m_csNode.time.time:setString(str)
end

function This:refresh()
    super.refresh(self)
   self.m_csNode.time:setVisible(self.m_csNode.tasksituation1.taskbegin:isVisible()) 
end

function This:onAchieveUpdate(leftTime)
    if leftTime ==0 then
        local achieve = self.m_achieve
        achieve:setState(DailyAchieveState.kFinish)
        self:refresh()
    else
        self:refreshProgress()
    end
end

function This:btnCallbackBegin()
    local data = self.m_data
    if data:getCrusadeNum()<1 then
        g_scene:showTips(Serialize["achieve_CrusadeAchieve_1_c"])
        return
    end
    local info ={crusadeItemId = self.m_achieve:getId() }
    local panel = g_scene:openDialog("Achieve.CrusadeHeroPanel",info)
end

function This:btnCallbackFinish()
    DailyAchieveManager:onGetDailyAchieveRewardRequest(self.m_achieve:getId(),DailyAchieveType.kCrusade)
end

function This:btnCallbackAccelerate()
    local achieve = self.m_achieve
    local leftTime = achieve:getLeftTime()
    if leftTime > 5 then 
        local cost = achieve:getAccelerateMoney()
        local progressNum = achieve:getProgress()*100
        local heroList = achieve:getHeroList()
        g_scene:openDialog("Achieve.CrusadeFinish", {heroNodeList = heroList,numList = {cost},progressNum = progressNum,
            btnCallBack = function ()
                -- print("btnCallBackbtnCallBackbtnCallBackbtnCallBack")
               DailyAchieveManager:onFinishAchieveFastlyRequest(self.m_achieve:getId(),DailyAchieveType.kCrusade) 
            end})
    elseif leftTime > 0 then
        g_scene:showTips(Serialize[60102])
    end
end

function This:getData()
    self.m_data = DailyAchieveManager:getCrusadeData()
end


