-- @Author: zheng
-- @Date:   2018-12-10 10:25:35
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-01 00:53:12
-- @File Name: CrusadeHeroPanel.lua

local This , super = class(...,"Dialog")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")
local CrusadeHeroItemView = require("Achieve.CrusadeHeroItemView")
local CrusadeHeroSelectionItemView = require("Achieve.CrusadeHeroSelectionItemView")
local HeroManager = require("Hero.HeroManager")
local UpdateLayer = require("UI.UpdateLayer")

This.fileName = "MenuScene/NodeUI/TaskCrusadeHero.csb"

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self:getData()
    self.m_selectHero ={}
    self.m_selectHeroKey ={}
    if info then
       self.m_crusadeItemData = self.m_data:getAchieve(info.crusadeItemId)
    end
    local scrollview = self.m_csNode.scrollView
    self.m_listLayer = UpdateLayer:create({
        itemWidth = 87,
        itemHeight = 80,
        column = 7,
        spaceX =5,
        spaceY =5,
        scrollView = scrollview
    })
    local heroList = self:getHeroList()
    self.m_listLayer:setDataList(heroList,function(troop)
        local view = self:createItemView(troop)
        
        return view
    end)
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    scrollview:pushBackCustomItem(self.m_listLayer, w,h,0)

    self:refresh()
end


function This:refresh()
    local itemData = self.m_crusadeItemData
    local attrId,attrNum,name = itemData:getHeroAttr()
    self.m_csNode.miaoshu:setString(itemData:getDesc2(), (not AppText:isChinese()) and (670/self.m_csNode.miaoshu:getScale()) or nil)
    self.m_csNode.yaoqiu:setString(string.format(Serialize["achieve_CrusadeTab_3"],name,attrNum))
    self:refreshScrollSelectedHero()
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    self.m_csNode.button:setTouchEnded(function()
        if #self.m_selectHero~= self:getHeroLimit() then
            g_scene:showTips(Serialize["achieve_CrusadeHero_4"])
            return
        end
        local data = self.m_data
        data:crusadeModify(-1)
        DailyAchieveManager:onGetCrusadeTaskRequest(self.m_crusadeItemData:getId(),self.m_selectHero)
        self:close()
    end)
end

function This:getHeroList()
   if not self.m_heroList then
        self.m_heroList ={}
        local data = self.m_data
        local itemData = self.m_crusadeItemData
        local attrId,attrNum,name = itemData:getHeroAttr()
        local heroList,alreadySelected = data:getAllHeros(attrId)
        for i, hero in pairs(heroList) do
            local heroId = hero:getId()
            local info = {hero = hero,isSelected = alreadySelected[heroId],attrId = attrId,attrValue =attrNum,attrStr = name ,gouFun = function (heroId2)
                    return self.m_selectHeroKey[heroId2]
                end,
            btnCallBack = function (itemInfor)
                local heroId2 = itemInfor.heroId
                local itemNode = itemInfor.itemNode
                local isAdd = self:addSelectHero(heroId2)
                if isAdd then
                    itemNode:refreshGou(heroId2,true)
                else
                    g_scene:showTips(Serialize["achieve_CrusadeTab_2"])
                end
            end }
            table.insert(self.m_heroList,info)
        end
   end 
   return self.m_heroList
end

function This:createItemView(info)
    local node = CrusadeHeroItemView:create(info)
    return node
end

function This:addSelectHero(heroId)
    local res = false
    if heroId then
        if #self.m_selectHero<self:getHeroLimit() then
            res = true
            table.insert(self.m_selectHero, heroId)
            self.m_selectHeroKey[heroId]= true
            self:refreshScrollSelectedHero()
        end
    end
    return res
end

function This:removeSelectHero(heroId)
    for index,heroId2 in ipairs(self.m_selectHero) do
        if heroId2 == heroId then
            table.remove(self.m_selectHero, index)
            self.m_selectHeroKey[heroId]= nil
            self:refreshScrollSelectedHero()
            break
        end
    end
end

function This:getHeroLimit()
    return self.m_crusadeItemData:getHeroNum()
end

function This:refreshScroolItems(heroId)
    -- local scroolView = self.m_csNode.scrollView
    if heroId then
        local allItems = self.m_listLayer:getAllViews()
        for _,item1 in ipairs(allItems) do
            if item1.refreshGou then
                item1:refreshGou(heroId)
            end
        end
        self:removeSelectHero(heroId)
    end
end

function This:refreshScrollSelectedHero()
    local heros = self.m_csNode.hero
    if not self.m_heroUiList then
        self.m_heroUiList={}
        for uiIndex=1,5 do
            local item1 = heros["herobg"..uiIndex]
            if uiIndex>self:getHeroLimit() then
                item1:setVisible(false)
            end
            self.m_heroUiList[uiIndex] = item1
        end
    end
    for _,heroUi in ipairs(self.m_heroUiList) do
        heroUi:removeAllChildren()
    end
    local index =1
    for _,heroId in pairs(self.m_selectHero) do
        local hero = HeroManager:getHero(heroId)
        local heroParent = self.m_heroUiList[index]
        heroParent:removeAllChildren()
        local info = {hero = hero,btnCallBack = function (heroId)
               self:refreshScroolItems(heroId)
            end }
        local view = CrusadeHeroSelectionItemView:create(info)
        local w,h = heroParent:getContentSize()
        view:setPosition(w/2,h/2)
        heroParent:addChild(view)
        index = index + 1
    end
end

function This:getData()
	self.m_data = DailyAchieveManager:getCrusadeData()
end
