-- @Author: zheng
-- @Date:   2018-12-08 11:20:01
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-12-15 11:43:17
-- @File Name: CrusadeTabNode.lua

local This , super = class(...,"NodeUI")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")
local CrusadeAchieveView = require("Achieve.CrusadeAchieveView")
This.fileName = "MenuScene/NodeUI/TaskCrusade.csb"

function This:initNode()
    self:getData()
    self:refreshAll()
    DailyAchieveManager:onGetCrusadeInfoRequest()
    self.m_isUpdating = false
    self.m_escape = 0
    self:refreshAll()
    self:scheduleUpdate()
end


function This:refreshAll()
    self:refreshBaseInfo()
    self:refreshScroolView()
end

function This:refreshScroolView()
    self.scrollView:removeAllItems()
    local data = self.m_data
    local achieveList = data:getAchieveWithSort()
    local achieveCount = #achieveList

    for _, achieve in ipairs(achieveList) do
        local view = self:getAchieveView(achieve)
        local w , h = view:getContentSize()
        self.scrollView:pushBackCustomItem(view,w,h,0.5)
    end

    self.empty:setVisible(achieveCount <= 0)
end

function This:getAchieveView(achieve)
    local view = CrusadeAchieveView:create(achieve)
    return view
end

function This:refreshBaseInfo()
    local data = self.m_data
    local numLimit = data:getCrusadeConfigNum(AchieveCrusadeLimitType.kCompletenum)
    local refreshCrusadeLeftTime = data:getRefreshCrusadeLeftTime()
    local refresLeftTime = data:getRefreshLeftTime()

    local function refreshTime(visibleNode ,timeLabel,time)
        local GameUtil = require("Util.GameUtil")
        local isEqual = false
        local visible = false
        local leftTime = data:calculateLeftTime(time)
        if leftTime==0 then
            isEqual = true
        elseif leftTime>0 then
            timeLabel:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
            visible =true
        end
        visibleNode:setVisible(visible)
        return isEqual
    end
    
    local item1 = self.item
    local num = data:getCrusadeNum()
    item1.num:setString(string.format("（%d/%d）",num,numLimit))
    local isEqual = refreshTime(item1.time_node,item1.time_node.time,refreshCrusadeLeftTime)
    if isEqual then
        data:setRefreshCrusadeLeftTime(0)
        data:setCrusadeNum(num+1)
    end
    item1.time_node:setVisible(num<numLimit)

    local refresh = self.refresh
    isEqual = refreshTime(refresh.refreshtime,refresh.refreshtime.time,refresLeftTime)
    if isEqual then
        data:setRefreshLeftTime(0)
    end
    refresh.text_free:setVisible(not refresh.refreshtime:isVisible())
end

function This:initTouchEvent()
	
    self.yijian.kuang:setTouchEnded(function()

        local PrivilegeManager = require("Vip.PrivilegeManager")
        if PrivilegeManager:hasPrivilege(Privilege.kCrusadeUnlock, true ) then
            local data = self.m_data
            local rusadeNum = data:getCrusadeNum()
            if rusadeNum<1 then
                g_scene:showTips(Serialize["achieve_CrusadeAchieve_1_c"])
    
            else
                local oneMoney = data:getCrusadeConfigNum(AchieveCrusadeLimitType.kCompleteMoney)
                g_scene:openDialog("Achieve.CrusadeFinish", {fileName = "MenuScene/NodeUI/TaskCrusadeProblem.csb",
                    numListName = "finish",numList = {oneMoney*rusadeNum,rusadeNum},
                    btnCallBack = function ()

                        -- print("btnCallBackbtnCallBackbtnCallBackbtnCallBack")
                       DailyAchieveManager:onGetCrusadeSSvipRequest() 
                       self.m_data:setCrusadeNum(0)
                    end})
            end

        end
    end)
	self.refresh:setTouchEnded(function()

        local data = self.m_data
        if not data:isCanStartAchieve() then
            g_scene:showTips(Serialize["achieve_CrusadeTab_1"])
           return false 
        end
        if self.refresh.refreshtime:isVisible() then
            local cost = data:getCrusadeConfigNum(AchieveCrusadeLimitType.kRefreshMoney)
            g_scene:openDialog("Achieve.CrusadeFinish", {fileName = "MenuScene/NodeUI/TaskCrusadeProblem.csb",
                numListName = "refresh",numList = {cost},
                btnCallBack = function ()
                    -- print("btnCallBackbtnCallBackbtnCallBackbtnCallBack")
                   DailyAchieveManager:onGetCrusadeRefreshRequest() 
                end})
        else
            DailyAchieveManager:onGetCrusadeRefreshRequest()
        end
    end)
end

function This:onEnter()
    super.onEnter(self)
    DailyAchieveManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    DailyAchieveManager:removeObserver(self)
    self:unscheduleUpdate()
end

function This:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape > 1 then
        self.m_escape = self.m_escape - 1
        self:refreshBaseInfo()
    end
end

function This:onGetCrusadeInfo()
    self:refreshAll()
end

function This:onGetCrusadeRefresh()
    self:refreshAll()
end

function This:onFinishAchieveFastly(achieve)
    self:refreshScroolView()
end

function This:onGetCrusadeAchieveReward()
    self:refreshScroolView()
end

function This:onGetCrusadeTask()
    self:refreshAll()
end

function This:onFinishAchieveFastly(achieve)
    self:refreshScroolView()
end

function This:onGetCrusadeSSvip()
    self:refreshBaseInfo()
end

function This:getData()
	self.m_data = DailyAchieveManager:getCrusadeData()
end