--region *.lua
--Date
--此文件由[BabeLua]插件自动生成
-- DailyAchieveManager.lua
local This , super = class(...,"Base.BaseModel")
local DailyAchieve = require("Achieve.DailyAchieve")
local Socket = require("Net.Socket")
local EventDispatcher = require("EventDispatcher")
local GamePlayer = require("Player.GamePlayer")
local GameCrusadeData = require("Achieve.GameCrusadeData")

DailyAchieveType =
{
	kCrusade = 1,--每日讨伐
}
DailyAchieveState = 
{
	kNotStart = 1, --未开始，可接
	kUpdating = 2, --正在进行中
	kFinish = 3, --已完成，可领取
	kHasReward = 4, --已领取
}

kRefreshDailyAchieveCD = 3600

function This:init()
	super.init(self)
	self.m_refreshTime = 0
	self.m_refreshEndTime = 0
	self.m_achieves = {}
	self.m_totalCount = 0
	self.m_finishCount = 0
	self.m_crusadeData = GameCrusadeData:create(0)
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self,10500,self.onGetDailyAchieveReply)
	EventDispatcher:addListener(self,10501,self.onStartDailyAchieveReply)
	EventDispatcher:addListener(self,10502,self.onFinishAchieveFastlyReply)
	EventDispatcher:addListener(self,10503,self.onRefreshDailyAchieveReply)
	EventDispatcher:addListener(self,10504,self.onGetDailyAchieveRewardReply)
	EventDispatcher:addListener(self, 10505, self.onGetCrusadeInfoReply)--每日讨伐列表
	EventDispatcher:addListener(self, 10506, self.onGetCrusadeTaskReply)--讨伐
	EventDispatcher:addListener(self, 10507, self.onGetCrusadeRefreshReply)--讨伐刷新任务
	EventDispatcher:addListener(self, 10508, self.onGetCrusadeSSvipReply)--一键讨伐

	EventDispatcher:addListener(self, "day_passed",self.onDayPassed)
end

function This:getCrusadeData()
	return self.m_crusadeData
end

--isNeedCheckNum 不满足返回true
function This:getArgConfigLv(keyStr,isNeedCheckNum,lv)
	local lvLimit = self.m_crusadeData:getCrusadeConfigNum(keyStr)
	if not isNeedCheckNum then
		return lvLimit
	end

	local curentLv = lv or GamePlayer:getLevel()
	return curentLv<lvLimit,lvLimit
end

function This:getRefreshCDEndTime()
	return self.m_refreshEndTime
end

function This:getLeftRefreshTime()
	local curTime = GamePlayer:getServerTime()
	return math.max(0,self.m_refreshEndTime - curTime)
end

function This:isCanStartAchieve()
	for i , achieve in pairs(self.m_achieves) do
		if achieve:getState() == DailyAchieveState.kUpdating then
			return false
		end
	end
	return true
end

function This:getFinishCount()
	return self.m_finishCount
end

function This:getMaxFinishCount()
	return self.m_totalCount
end

function This:getAchieve(id)
	return self.m_achieves[id]
end

function This:getRefreshCost(leftTime, isAll)
    if leftTime > 0 then
        local diamond = math.ceil(0.015 * (leftTime ^ 0.8))
        return math.max(diamond, isAll and 10 or 1)
    else
        return 0
    end
end

function This:addAchieve(info)
	local achieve = self.m_achieves[info.id]
	if not achieve then
		achieve = DailyAchieve:create(info)
		self.m_achieves[info.id] = achieve
		self:notifyObservers("onAddAchieve",achieve)
	else
		achieve:updateInfo(info)
		self:notifyObservers("onUpdateAchieve,achieve")
	end
	return achieve
end

function This:removeAchieve(id)
	if self.m_achieves[id] then
		local achieve = self.m_achieves[id]
		self.m_achieves[id] = nil
		self:notifyObservers("onRemoveAchieve",achieve)
	end
end

function This:getAchieveWithSort()
	local result = {}
	for i , achieve in pairs(self.m_achieves) do
		table.insert(result,achieve)
	end
	table.sort(result,function(a,b)
		local aq, bq = a:getQuality(), b:getQuality()
		if aq ~= bq then
			return aq > bq
		end
		
		local as = a:getState()
		local bs = b:getState()
		if as ~= bs then
			return as > bs
		end

		return a:getId() < b:getId()
	end)
	return result
end

function This:getAllAchieves()
	return self.m_achieves
end

function This:refreshAllAchieves(items)
	local flag = {}
	for id, achieve in pairs(self.m_achieves) do
		flag[id] = true
	end
	for i , info in ipairs(items) do
		self:addAchieve(info)
		flag[info.id] = nil
	end
	for id , _ in pairs(flag) do
		self:removeAchieve(id)
	end
end

function This:achieveHasReward()
	local red = false
	local data = self:getCrusadeData()
	local res = data:getAllAchieves()
	for _,item in pairs (res) do
		if item:getState() == DailyAchieveState.kFinish then
			red = true
		end
	end
	return  red 
end
----------------------------------------------------------------

function This:onGetDailyAchieveRequest()
	Socket:sendAndLock(10500, {})
end

function This:onGetDailyAchieveReply(msg)
	self.m_totalCount = msg.totalAchieveCount
	self.m_finishCount = msg.achieveCount
	self.m_refreshEndTime = msg.refreshSecondInterval + GamePlayer:getServerTime()
	self:refreshAllAchieves(msg.dailyQuests)

	self:notifyObservers("onDailyAchieveRefresh")
	EventDispatcher:dispatcher("daily_achieve_refresh")
end

function This:onStartDailyAchieveRequest(id)
	self.m_requestStartId = id
	Socket:sendAndLock(10501,{id = id})
end

function This:onStartDailyAchieveReply(msg)
	if msg.err_code == 0 then
		local achieve = self:getAchieve(self.m_requestStartId)
		self:notifyObservers("onStartDailyAchieve",achieve)
		EventDispatcher:dispatcher("daily_achieve_changed",{achieve})
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onFinishAchieveFastlyRequest(id,achieveType)
	self.m_requestFinishId = id
	Socket:sendAndLock(10502,{id = id,type = achieveType})
end

function This:onFinishAchieveFastlyReply(msg)
	if msg.err_code == 0 then
		local requestId = self.m_requestFinishId
		if msg.type and msg.type ==DailyAchieveType.kCrusade then
			local data = self.m_crusadeData
			local achieve = data:getAchieve(requestId)
			achieve:fastEnd()
			self:notifyObservers("onFinishAchieveFastly",achieve)
		else
			local achieve = self:getAchieve(requestId)
			self:notifyObservers("onFinishAchieveFastly",achieve)
			EventDispatcher:dispatcher("daily_achieve_changed",{achieve})
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onRefreshDailyAchieveRequest()
	Socket:sendAndLock(10503,{})
end

function This:onRefreshDailyAchieveReply(msg)
	if msg.err_code == 0 then
		g_scene:showReward(msg.item)
		self.m_refreshEndTime = GamePlayer:getServerTime() + kRefreshDailyAchieveCD
		self:notifyObservers("onRefreshDailyAchieve")
		EventDispatcher:dispatcher("daily_achieve_refresh")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetDailyAchieveRewardRequest(id,achieveType)
	self.m_requestRewardId = id
	Socket:sendAndLock(10504,{id = id,type = achieveType})
end

function This:onGetDailyAchieveRewardReply(msg)
	if msg.err_code == 0 then
		local requestId = self.m_requestRewardId
		g_scene:showReward(msg.item)
		if msg.type and msg.type ==DailyAchieveType.kCrusade then
			local data = self.m_crusadeData
			data:removeAchieve(requestId)
			self:notifyObservers("onGetCrusadeAchieveReward")
			EventDispatcher:dispatcher("crusade_info_refresh")
		else
			local achieve = self:getAchieve(requestId)
			-- self.m_finishCount = self.m_finishCount + 1
			self:notifyObservers("onGetDailyAchieveReward",achieve)
			EventDispatcher:dispatcher("daily_achieve_changed",{achieve})
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--------------------------------------------------------
function This:onDayPassed()
	self:onGetDailyAchieveRequest()
end
	
function This:onGetCrusadeTaskRequest(id,heroList)
	Socket:sendAndLock(10506,{id = id, heroid_list = heroList})--讨伐
end

function This:onGetCrusadeTaskReply( msg )
	if msg.err_code == 0 then
		local crusadeItemData = msg.crusade_info
        local data = self.m_crusadeData
        local crusadeItem = data:getAchieve(crusadeItemData.id)
        if crusadeItem then
        	crusadeItem:updateInfo(crusadeItemData)
        end
        data:setCrusadeNum(msg.crusade_card_num)
        data:setRefreshCrusadeLeftTime(msg.resume_time)
		self:notifyObservers("onGetCrusadeTask")
		EventDispatcher:dispatcher("onGetCrusadeTask")	
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end
	
	
function This:onGetCrusadeRefreshRequest()
	Socket:sendAndLock(10507)--讨伐刷新任务
end

function This:onGetCrusadeRefreshReply( msg )
    if msg.err_code == 0 then
    	g_scene:showReward(msg.item)
        local data = self.m_crusadeData
		data:refreshAllAchieves(msg.crusade_list or {})
		data:setRefreshLeftTime(msg.reflesh_time or 0)
        self:notifyObservers("onGetCrusadeRefresh")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
	
	
function This:onGetCrusadeSSvipRequest()
	Socket:sendAndLock(10508)--一键讨伐
end

function This:onGetCrusadeSSvipReply( msg )
    if msg.err_code == 0 then
        g_scene:showReward(msg.item)
        local data = self.m_crusadeData
        data:setCrusadeNum(0)
		self:notifyObservers("onGetCrusadeSSvip")
		EventDispatcher:dispatcher("onGetCrusadeSSvip")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetCrusadeInfoRequest()
	local crusadeData = self:getCrusadeData()
	local levelLimit = crusadeData:getCrusadeConfigNum(AchieveCrusadeLimitType.kOpenLevel)
	local level = GamePlayer:getLevel()
	if level >= levelLimit then
		Socket:sendAndLock(10505)--每日讨伐列表
	end
end

function This:onGetCrusadeInfoReply( msg )
	local data = self.m_crusadeData
	data:refreshAllAchieves(msg.crusade_list or {})
	data:setCrusadeNum(msg.crusade_card_num or 0)
	data:setRefreshLeftTime(msg.reflesh_time or 0)
	data:setRefreshCrusadeLeftTime(msg.resume_time or 0)
	self:notifyObservers("onGetCrusadeInfo")
	EventDispatcher:dispatcher("crusade_info_refresh")
	   
end
--endregion
