--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"NodeUI")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")
local DailyAchieveView = require("Achieve.DailyAchieveView")

function This:initNode()
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/DailyQuest.csb")
	self:addChild(self.m_csNode)
	self.m_isUpdating = false
	self.m_escape = 0
end

function This:refreshBaseInfo()
	local curTime = GamePlayer:getServerTime()
	local leftTime = DailyAchieveManager:getRefreshCDEndTime() - curTime

	local finishTime = DailyAchieveManager:getFinishCount()
	local maxFinishTime = DailyAchieveManager:getMaxFinishCount()
	if finishTime < maxFinishTime then
		self.m_csNode.taskcount.refreshCount:setString(string.format("%d/%d",finishTime,maxFinishTime))
		self.m_csNode.taskcount:setVisible(true)
    	self.m_csNode.refresh:setVisible(true)
    	self.m_csNode.timecount:setVisible(true)
    	self.m_csNode.cost:setVisible(true)
        self.m_csNode.emptyNote:setVisible(false)
	else
        self.m_csNode.empty:setString(Serialize[60114])
        self.m_csNode.emptyNote:setVisible(true)
		self.m_csNode.taskcount:setVisible(false)
    	self.m_csNode.refresh:setVisible(false)
    	self.m_csNode.timecount:setVisible(false)
    	self.m_csNode.cost:setVisible(false)
    	if self.m_isUpdating then
			self.m_isUpdating = false
			self.m_escape = 0
			self:unscheduleUpdate()
		end
		return
	end

	if leftTime > 0 and not self.m_isUpdating then
		self.m_isUpdating = true
		self.m_escape = 0
		self:scheduleUpdate()
	elseif leftTime <= 0 and self.m_isUpdating then
		self.m_isUpdating = false
		self.m_escape = 0
		self:unscheduleUpdate()
	end

	if leftTime > 0 then
		self.m_csNode.timecount.time:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
	end
	self.m_csNode.timecount:setVisible(leftTime > 0)

	local cost = DailyAchieveManager:getRefreshCost(leftTime)
	if cost > 0 then
		self.m_csNode.cost.value:setString(cost)
	end
	self.m_csNode.cost:setVisible(cost > 0)
end

function This:initTouchEvent()
	self.m_csNode.refresh:setTouchEnded(function()
		if not DailyAchieveManager:isCanStartAchieve() then
			g_scene:showTips(Serialize[60111])
			return
		end

		local leftTime = DailyAchieveManager:getLeftRefreshTime()
		local cost = DailyAchieveManager:getRefreshCost(leftTime)
		if cost <= 0 then
			DailyAchieveManager:onRefreshDailyAchieveRequest()
		else
			local leftTime = DailyAchieveManager:getLeftRefreshTime()
			local panel = g_scene:openDialog("UI.CostConfirmPanel", {
				cost = cost, 
				desc = Serialize[60115], 
				notips = "DailyQuest", 
				note = GameUtil:formatTime(Serialize[60116],leftTime),
				okHandler = function()
                	DailyAchieveManager:onRefreshDailyAchieveRequest()
			    end 
			})
		end
	end)
end

function This:getAchieveView(achieve)
	local view = DailyAchieveView:create(achieve)
	return view
end

function This:refreshItems()
	self.m_csNode.scrollView:removeAllItems()

	local achieveList = DailyAchieveManager:getAchieveWithSort()
	local achieveCount = #achieveList

	for _, achieve in ipairs(achieveList) do
		local view = self:getAchieveView(achieve)
		local w , h = view:getContentSize()
		self.m_csNode.scrollView:pushBackCustomItem(view,w,h,0.5)
	end

    if achieveCount <= 0 then
        self.m_csNode.empty:setVisible(true)
    else
        self.m_csNode.empty:setVisible(false)
    end 
end

function This:onEnter()
	super.onEnter(self)
	self:refreshBaseInfo()
	self:refreshItems()
	DailyAchieveManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	DailyAchieveManager:removeObserver(self)
end

function This:onDailyAchieveRefresh()
	self:refreshBaseInfo()
	self:refreshItems()
end

function This:onStartDailyAchieve(achieve)
	self:refreshItems()
end

function This:onFinishAchieveFastly(achieve)
	self:refreshBaseInfo()
	self:refreshItems()
end

function This:onGetDailyAchieveReward()
	self:refreshBaseInfo()
	self:refreshItems()
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshBaseInfo()
	end
end


--endregion
