-- @Author: zheng
-- @Date:   2018-12-06 19:56:44
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-09-02 16:37:06
-- @File Name: GameCrusadeData.lua
local ArgConfig = require("Metadatas.ArgConfig")

local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")
local GameCrusadeItem = require("Achieve.GameCrusadeItem")
-- local DailyAchieveManager = require("Achieve.DailyAchieveManager")

local This = class(...)

AchieveCrusadeLimitType ={
	kRefreshTime   = "refresh_time",          	--讨伐（李嘉茜）刷新时间
	kRefreshMoney  = "refresh_money",         	--讨伐（李嘉茜）刷新花费
	kCompletenum   = "complete_num",          	--每日讨伐（李嘉茜）讨伐令上限次数
	kCompleteTime  = "complete_time",         	--每日讨伐（李嘉茜）讨伐令恢复周期
	kCompleteMoney  = "complete_money",         	--每日讨伐（李嘉茜）一键讨伐花费
	kTaskNum       = "task_num",             	--每日讨伐（李嘉茜）任务数量
	-- kLimit1        = "limit_1",             		--每日讨伐（李嘉茜）免费刷新品质上限
	-- kGuarantees    = "guarantees",             	--每日讨伐（李嘉茜）付费刷新品质保底
	-- kLimit2        = "limit_2",             		--每日讨伐（李嘉茜）付费刷新品质上限
	kOpenLevel     = "open_level",             	--每日讨伐（李嘉茜）任务开启等级
	kBoxOpenLevel  = "achievements_box_open_level",	--政绩宝箱（余林峰）：开启等级
	
}

function This:create(id)
    local Morality = {}
    setmetatable(Morality,This)
    Morality:init(id)
    return Morality
end

function This:init(id)
    self.m_achieves = {}
end

function This:getAllHeros(attrId)
	local alreadySelected = {}
	for id, achieve in pairs(self.m_achieves) do
		local heroList = achieve:getHeroList()
		for _,heroId in ipairs(heroList) do
			alreadySelected[heroId] = true
		end
	end
	local res = HeroManager:getAllHeroGotWithSort()
	table.sort(res, function( a, b )
        local state1, state2 = a:getAttr(attrId) , b:getAttr(attrId)
        return state1>state2
	end)
	return res ,alreadySelected
end

function This:getCrusadeConfigNum(crusadeType)
	local res =0
	local item1 = ArgConfig[crusadeType]
	if item1 then
		local temNum = item1.arg[1]
		if type(temNum) == 'number' then
			res = temNum
		end
	end
	return res
end

function This:getCrusadeConfig(crusadeType)
	local item1 = ArgConfig[crusadeType]
	if item1 then
		return item1.arg
	end
	return nil
end

--讨伐令数量
function This:setCrusadeNum(CrusadeNum)
	self.m_CrusadeNum = CrusadeNum or self.m_CrusadeNum
end

function This:getCrusadeNum()
	return self.m_CrusadeNum or 0
end

function This:crusadeModify(modifyNum)
	if modifyNum then
		local crusadeNum = self.m_CrusadeNum+ modifyNum
		if crusadeNum< 0 then
			crusadeNum = 0
		end
		self:setCrusadeNum(crusadeNum)
	end
end

--刷新倒计时
function This:setRefreshLeftTime(RefreshLeftTime)
	self.m_RefreshLeftTime = RefreshLeftTime
end

function This:getRefreshLeftTime()
	local temTime = self.m_RefreshLeftTime or 0
	return temTime--self:calculateLeftTime(temTime)
end
--讨伐令恢复倒计时
function This:setRefreshCrusadeLeftTime(RefreshCrusadeLeftTime)
	self.m_RefreshCrusadeLeftTime = RefreshCrusadeLeftTime
end

function This:getRefreshCrusadeLeftTime()
	local temTime = self.m_RefreshCrusadeLeftTime or 0
	return temTime--self:calculateLeftTime(temTime)
end

function This:calculateLeftTime(temTime)
	return temTime - GamePlayer:getServerTime()
end

function This:getAchieve(id)
	return self.m_achieves[id]
end

function This:getAllAchieves()
	return self.m_achieves
end

function This:refreshAllAchieves(items)
	for id, achieve in pairs(self.m_achieves) do
		self:removeAchieve(id)
	end
	for i , info in ipairs(items) do
		self:addAchieve(info)
	end
end

function This:addAchieve(info)
	local achieve = self.m_achieves[info.id]
	if not achieve then
		achieve = GameCrusadeItem:create(info)
		self.m_achieves[info.id] = achieve
		-- DailyAchieveManager:notifyObservers("onAddAchieve",achieve)
	else
		achieve:updateInfo(info)
		-- DailyAchieveManager:notifyObservers("onUpdateAchieve,achieve")
	end
	return achieve
end

function This:removeAchieve(id)
	if self.m_achieves[id] then
		local achieve = self.m_achieves[id]
		self.m_achieves[id] = nil
		-- DailyAchieveManager:notifyObservers("onRemoveAchieve",achieve)
	end
end

function This:isCanStartAchieve()
	for i , achieve in pairs(self.m_achieves) do
		if achieve:getState() == DailyAchieveState.kUpdating then
			return false
		end
	end
	return true
end

function This:getAchieveWithSort()
	local result = {}
	for i , achieve in pairs(self.m_achieves) do
		table.insert(result,achieve)
	end
	table.sort(result,function(a,b)
		local aq, bq = a:getQuality(), b:getQuality()
		if aq ~= bq then
			return aq > bq
		end
		
		local as = a:getState()
		local bs = b:getState()
		if as ~= bs then
			return as > bs
		end

		return a:getId() < b:getId()
	end)
	return result
end