-- @Author: zheng
-- @Date:   2019-05-14 16:54:02
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-02 11:18:19
-- @File Name: GiftDayTabNode.lua
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local This , super = class(...,"NodeUI")

This.fileName = "MenuScene/NodeUI/GiftMainBg.csb"
-- local WeekGiftLevelConfig = require("Metadatas.WeekGiftLevelConfig")

local GiftDayTabItemView = require("Achieve.GiftDayTabItemView")
local AchieveManager = require("Achieve.AchieveManager")


function This:getEndTime()
    local function addDay(time,day)
        return time+86400*day
    end
    local curTime = GamePlayer:getServerTime()
    local tmp = os.serverdate("*t",curTime)
    local year,month,day = tmp.year,tmp.month,tmp.day
    local wDay = tmp.wday
    local endTime = os.time({year = year,month = month, day = day, hour = 0, min = 0, sec = 0})
    if self.m_type== GiftMainPanelTabName.kDayGift then
        return addDay(endTime,1)
    elseif self.m_type== GiftMainPanelTabName.kWeekGift then
        local temDay =6-wDay +2+1
        return addDay(endTime,temDay)
    else
        local month = month+1
        local year = year
        if month>12 then
            year = year+1
            month= 1
        end
        return os.time({year = year,month = month, day = 1, hour = 0, min = 0, sec = 0})
    end
end

function This:initNode(info)
    self.m_type = info.tabType
    -- self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    -- self.m_csNode:setToCenter()
    -- self:addChild(self.m_csNode)
    self.m_isUpdating = false
    self.m_escape = 0
    -- GameUtil:hideNodeList(self,{"timedi","time"})
    self.m_endTime = self:getEndTime()
    self:scheduleUpdate()
    self.scrollView:setDirection("horizontal")
    self.scrollView.m_direction = "right"
    self:refresh()
    self:refreshTime()
end

function This:refreshTime()
    if self.m_endTime then
        local timeStr = GameUtil:getFormatDtimeStr(self.m_endTime-GamePlayer:getServerTime())
        self.time:setString(Serialize[60123]..timeStr)
    end
end

function This:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape > 1 then
        self.m_escape = self.m_escape - 1
        self:refreshTime()
    end
end

function This:refresh()
    self:refreshBoxState()
    local scrollView = self.scrollView
    scrollView:removeAllItems()
    local itemList = self:getData()
    local acheveType = self:getAcheveType()
    if acheveType== AchieveType.kWeekGift then
        self.act_title:setImage("MenuScene/act_title_33.jpg")
    elseif acheveType== AchieveType.kMonthGift then
        self.act_title:setImage("MenuScene/act_title_39.jpg")
    end
    local isLock = false
    local lastNum= #itemList
    for index,achieve in ipairs(itemList) do
        local info ={acheveType = acheveType,achieve = achieve,isLock = isLock,isLast= lastNum==index}
        local item = self:createItemView(info)
        local w , h = item:getContentSize()
        scrollView:pushBackCustomItem(item,w,h,0.5)
        if not isLock then
            isLock = achieve:getState() == AchieveState.kUpdating
        end
    end
end

function This:initTouchEvent()
   self.box_node.box:setTouchEnded(function()
        if AchieveManager:isHaveGiftDayBox() then
            AchieveManager:onGetGiftDayBoxRequest()
        else
            g_scene:showTips(Serialize["achieve_GiftTab_2_c"])
        end
    end) 
end

function This:refreshBoxState()
    local box_node = self.box_node
    if self.m_type== GiftMainPanelTabName.kDayGift then
        local isHaveBox = AchieveManager:isHaveGiftDayBox()
        GameUtil:createBoxEffect(isHaveBox,box_node.box,17,not isHaveBox)
        box_node.keling:setVisible(isHaveBox)
        box_node.yilingqu:setVisible(not isHaveBox)
    end
    box_node:setVisible(self.m_type== GiftMainPanelTabName.kDayGift)
end

function This:createItemView(info)
    return GiftDayTabItemView:create(info)
end

function This:getAcheveType()
    local convertTable ={
        [GiftMainPanelTabName.kWeekGift] = AchieveType.kWeekGift,
        [GiftMainPanelTabName.kDayGift] = AchieveType.kDayGift,
        [GiftMainPanelTabName.kMonthGift] = AchieveType.kMonthGift,
    }
    return convertTable[self.m_type]
end

function This:getData()
    local acheveType = self:getAcheveType()
    local result = AchieveManager:getAchieveWithSort(acheveType)
    local temResult = {}
    local myLv = GamePlayer:getLevel()
    for _,achieve in ipairs(result) do
        if achieve:getOpenLv()<=myLv then
            table.insert(temResult,achieve)
        end
    end
    return temResult
end

function This:onEnter()
    super.onEnter(self)
    AchieveManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    AchieveManager:removeObserver(self)
end

-- function This:onRefreshAchieve(typeId)
--     local acheveType = self:getAcheveType()
--    if acheveType == AchieveType.kDayGift or acheveType == AchieveType.kWeekGift then
--         self:refresh()
--    end
-- end

-- function This:onAchieveChanged(typeId,list)
--     self:onRefreshAchieve(typeId)
-- end



This.onGetGiftDayBox = This.refreshBoxState
This.onRefreshAchieve = This.refreshBoxState