-- @Author: Su
-- @Date:   2019-01-29 9:44:30
-- @Last Modified by:   Su
-- @Last Modified time: 2019-01-29 9:44:30
-- @File Name: TaskContendBoxItem.lua

local This , super = class(...,"NodeUI")
local MatchManager = require("Match.MatchManager")

local GameUtil = require("Util.GameUtil")


local picList = {
    "MenuScene/kuang105.png",
    "MenuScene/kuang104.png",
    "MenuScene/kuang101.png",
}


function This:initData(info)
    self.m_index = info.index
    self.m_escape = 0
    self.m_hasSetTouch = false
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaskContendBoxItem.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:playCSAnimation("animation0",true)
    self:refresh()
end

function This:refresh()
    self.m_box = MatchManager:getBox(self.m_index)
    local node = self.m_csNode
    local state = MatchManager:getIfCanOpen()
    self.m_csNode.bg:setImage("MenuScene/kuang102.png")
    if self.m_box then
        local ImageTex = "boxLock"
        if self.m_box:getState() == BoxState.kReceive then
            ImageTex = "boxGet"
        end
        self.m_csNode.bg:setImage(picList[self.m_box:getState() + 1])
        if self.m_box:getState() == BoxState.kLock and state then
            self.m_csNode.bg:setImage("MenuScene/kuang103.png" )
        elseif self.m_box:getState() == BoxState.kLock and not state then
            self.m_csNode.bg:setImage("MenuScene/kuang105.png" )
        end
        node.box:setImage(string.format( "MenuScene/%s%d.png",ImageTex,self.m_box:getConfig().quality))

        node.jiesuo.time:setString(GameUtil:formatTime( Serialize[60148],self.m_box:getUnlockTime()))
        node.weijiesuo.time:setString(GameUtil:formatTime( Serialize[60148],self.m_box:getUnlockTime()))

        node.fastopen.time:setString(GameUtil:formatTime( Serialize[60149],self.m_box:getLeftTime()))
        node.fastopen.num:setString(self.m_box:getRecieveCost())

        if self.m_box:getState() == BoxState.kOpening then
            self:scheduleUpdate()
        else
            self:unscheduleUpdate()
        end
    end
    node.box:setVisible(self.m_box ~= nil)
    node.jiesuo:setVisible(self.m_box ~= nil and self.m_box:getState() == BoxState.kLock and state)
    node.weijiesuo:setVisible(self.m_box ~= nil and self.m_box:getState() == BoxState.kLock and not state)  
    node.fastopen:setVisible(self.m_box ~= nil and self.m_box:getState() == BoxState.kOpening)
    node.open:setVisible(self.m_box ~= nil and self.m_box:getState() == BoxState.kReceive)
    node.weizi:setVisible( self.m_box == nil)
    self:refreshEffect()
end

function This:refreshTimeInfo()
    local node = self.m_csNode
    node.fastopen.time:setString(GameUtil:formatTime( Serialize[60149],self.m_box:getLeftTime()))
    node.fastopen.num:setString(self.m_box:getRecieveCost())
    if self.m_box:getState() ~= BoxState.kOpening then
        self:refresh()
    end
end

function This:initTouchEvent()
    self.m_csNode.box:setTouch({
        isScale = false, 
        isDynamic = true, 
        onEnded = function()
            if self.m_box:getState() == BoxState.kReceive then
                MatchManager:onOpenBoxRequest(self.m_index)
                return 
            else
                print("打开解锁宝箱界面")
                g_scene:openDialog("Achieve.TaskContendBoxItemInfoPanel", {box = self.m_box})
            end
        end
    })
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshTimeInfo()
	end
end


function This:moveCenterAnimation( param, factor, callback)
    cc.Audio:playEffect("GetItem.mp3")
    local parent = param.parent
    local target = cc.Sprite:create(param.name)
    local x, y = parent:convertToNodeSpace(cc.visible.center.x, cc.visible.center.y)

    local posx, posy = self.m_csNode.box:getPosition()
    local effectParent = parent
    local effect = nil
    effect = GameUtil:createEffect("XinGongNeng", false, nil, {
        onAnimationComplete = function ()
            effect:removeFromParent()
            local particle = cc.Particle:create("tuowei.plist", self)
            particle:setPosition(x, y)

            local action = nil
            action = cc.Sequence(cc.MoveTo(factor * 0.85, posx, posy, "SineInOut"),
                cc.DelayTime(factor * 0.3),
                cc.CallFunc(function()
                    if param.next then
                        self:playAnimation2( param.next, factor, callback)
                    elseif callback then
                        callback()
                    end
                end),
                cc.RemoveAction()
            )
            particle:runAction(action)
            effectParent:addChild(particle)
            particle:setLocalZOrder(101)  

            action = cc.Sequence(cc.MoveTo(factor * 0.6 , posx, posy, "SineIn"), cc.RemoveAction())
            target:runAction(action)
        end     
    }, false, "animation2")
    effect:setPosition(x, y)
    effectParent:addChild(effect)
    effect:setLocalZOrder(120)

    target:setPosition(x, y)
    target:setScale(1)
    effectParent:addChild(target)
    target:setLocalZOrder(121)
end

function This:refreshEffect()
    if self.m_box ~= nil then
        if self.m_box:getState() == BoxState.kOpening and not self.m_effectLock then
            self.m_effectLock = GameUtil:createEffect("UnlockLight",true)
            local x , y = self.m_csNode.bg:getPosition()
            self.m_effectLock:setPosition(x+57, y+75)
            self.m_csNode.bg:addChild(self.m_effectLock)
        elseif self.m_box:getState() == BoxState.kReceive and not self.m_effectOpen then
            if self.m_effectLock then
                self.m_csNode.bg:removeAllChildren()
                self.m_effectLock = nil
            end
            self.m_effectOpen = GameUtil:createEffect("OpenLight",true)
            local x , y = self.m_csNode.bg:getPosition()
            self.m_effectOpen:setPosition(x+55, y+75)
            self.m_csNode.bg:addChild(self.m_effectOpen)
        else
            -- skeletonNode:playAnimation("animation0",true)
        end
    else
        self.m_csNode.bg:removeAllChildren()
        self.m_effectLock = nil
    end
end

function This:addBoxAnim()
    self.m_box = MatchManager:getBox(self.m_index)
    self:moveCenterAnimation({name= string.format("MenuScene/boxLock%d.png",self.m_box:getConfig().quality),parent = self.m_csNode}, 0.5, function()
        self:refresh()
    end)
end
