-- @Author: Su
-- @Date:   2019-01-29 16:51:30
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-19 16:20:35
-- @File Name: TaskContendBoxItemInfoPanel.lua

local This , super = class(...,"Dialog")
local MatchManager = require("Match.MatchManager")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")


function This:initData( info )
    self.m_box = info.box
    self.m_config = self.m_box:getConfig()
    self.m_escape = 0
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaskContendBoxItemInfo.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    -- self:initTouchEvent()
    self:refresh()
end

function This:refresh()
    local canOpen,hasVip = MatchManager:getIfCanOpen()

    local node = self.m_csNode.beijing
    node.title:setString(self.m_config.name)
    node.text:setStringWithColor(self.m_config.desc)
    node.box:setImage(string.format( "MenuScene/boxLock%d.png",self.m_config.quality))

    node = self.m_csNode.jindutiao
    node:setVisible(self.m_box:getState() == BoxState.kOpening)
    node.num:setString(GameUtil:formatTime(Serialize["H_M_S"],self.m_box:getLeftTime()))
    local w = (self.m_box:getUnlockTime() - self.m_box:getLeftTime())/self.m_box:getUnlockTime()
    node.jindu:setPercent(w*100)

    node = self.m_csNode
    node.button1:clearShader()
    if not canOpen or self.m_box:getState() ~= BoxState.kLock then
        node.button1:setGrayShader()
    end
    node.button1.text1:setString(Serialize[60151])
    node.button3:setVisible(not canOpen and not hasVip)
    node.text1:setVisible(not hasVip and not canOpen and self.m_box:getState() ~= BoxState.kOpening)
    node.text2:setVisible(hasVip and not canOpen and self.m_box:getState() ~= BoxState.kOpening)

    node = self.m_csNode.spend
    node.num:setString(self.m_box:getRecieveCost())

    if self.m_box:getState() == BoxState.kOpening then
        self:scheduleUpdate()    
    else
        self:unscheduleUpdate()        
    end
end

function This:onEnter()
	super.onEnter(self)
    MatchManager:addObserver(self)    
end

function This:onExit()
	super.onExit(self)
    MatchManager:removeObserver(self)    
end

function This:initTouchEvent()
    self.m_csNode.button1:setTouchEnded(function ()
        local canOpen,hasVip = MatchManager:getIfCanOpen()
        if not canOpen or self.m_box:getState() ~= BoxState.kLock then
            return 
        end
        MatchManager:onUnlockBoxRequest(self.m_box:getIndex())
        self:close()
        --解锁动画
        --xxx
    end)

    self.m_csNode.button2:setTouchEnded(function ()
        g_scene:openDialog("UI.CostConfirmPanel", {
            cost = self.m_box:getRecieveCost(),
            desc = Serialize[60152],
            okHandler = function()
                if GamePlayer:isEnoughDiamond(self.m_box:getRecieveCost()) then
                    MatchManager:onFinishBoxCDRequest(self.m_box:getIndex())        
                else
                    g_scene:showTips(Serialize[60153])
                end
                self:close()
            end
        })
    end)

    self.m_csNode.button3:setTouchEnded(function ()
		g_scene:openDialog("Vip.VipPanel")
    end)

    self.m_csNode.close:setTouchEnded(function ()
        self:close()
    end)
end

function This:refreshTimeInfo()
    local node = self.m_csNode.jindutiao
    node:setVisible(self.m_box:getState() == BoxState.kOpening)
    node.num:setString(GameUtil:formatTime(Serialize["H_M_S"],self.m_box:getLeftTime()))
    local w = (self.m_box:getUnlockTime() - self.m_box:getLeftTime())/self.m_box:getUnlockTime()
    node.jindu:setPercent(w*100)
    node = self.m_csNode.spend
    node.num:setString(self.m_box:getRecieveCost())
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshTimeInfo()
	end
end

function This:onFinishBoxCD(box)
    if self.m_box:getIndex() == box:getIndex() then
        self:refresh()
    end
end