-- @Author: Su
-- @Date:   2019-01-29 9:44:30
-- @Last Modified by:   Su
-- @Last Modified time: 2019-01-29 9:44:30
-- @File Name: TaskContendBoxPanel.lua

local This , super = class(...,"NodeUI")
local GamePlayer = require("Player.GamePlayer")
local MatchManager = require("Match.MatchManager")
local AchieveManager = require("Achieve.AchieveManager")

local TaskContendBoxItem = require("Achieve.TaskContendBoxItem")
local TaskContendBoxTaskItem = require("Achieve.TaskContendBoxTaskItem")

function This:initData()
    self.m_boxList = {}
    self.m_taskList = {}
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaskContendBox.csb")
    self:addChild(self.m_csNode)
    -- self.m_csNode:setToCenter()
    self:initBox()
    self:initTask()
end

function This:initBox()
    -- local scrollView = self.m_csNode.scrollView2
    -- scrollView:removeAllItems()
    local scrollView =self.m_csNode.boxList
    for i = 1, 6 do
        local node = TaskContendBoxItem:create({index = i})
        self.m_boxList[i] = node
        -- node:setAnchorPoint(0.5,0.5)
        local w,h = node:getContentSize()
        scrollView:addChild(node)
        node:setPosition(120*i-40, -70)
        -- scrollView:pushNode(node,122,114,6)
    end
end

function This:initTask()
    local achievesList = AchieveManager:getAllAchieves(AchieveType.kZhengJi)
    local achieveList = {}
    for id,info in pairs(achievesList) do
        achieveList[math.floor(id/100)] = info
    end
    table.sort(achieveList,function (a,b)
        local as = a:getState()
		local bs = b:getState()
		if as ~= bs then
			return as < bs
		end

        local aq,bq = a:getQuality(),b:getQuality()
        if aq ~= bq then
            return aq > bq
        end

        return a.m_id > b.m_id
    end)
    local scrollView = self.m_csNode.scrollView1
    scrollView:removeAllItems()
    if achievesList then
        for _,achieve in ipairs(achieveList) do
            local item = TaskContendBoxTaskItem:create({achieve = achieve})
            table.insert(self.m_taskList,item)
            item:setAnchorPoint(0,0)
            local w,h = item:getContentSize()
            scrollView:pushNode(item,379,90,2)
        end
    end
end

function This:onEnter()
	super.onEnter(self)
    MatchManager:addObserver(self)   
    AchieveManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
    MatchManager:removeObserver(self)
    AchieveManager:removeObserver(self)
end

function This:initTouchEvent()
    self.m_csNode.wenhao:setTouchEnded(function ()
        g_scene:openDialog("UI.HelpTips",{text = Serialize[60155],node=self.m_csNode.wenhao})
    end)
end

function This:onRefreshAchieve(achieveType)
    if achieveType == AchieveType.kZhengJi then
        self:initTask()
        if self.m_boxList then 
            for _,box in pairs(self.m_boxList) do
                box:refresh()
            end
        end
    end
end

function This:onFinishBoxCD(box)
    for _,box in pairs(self.m_boxList) do
        box:refresh()
    end
end

function This:onUnlockBox(box)
    -- self.m_boxList[box:getIndex()]:refresh()
    for _,box in pairs(self.m_boxList) do
        box:refresh()
    end
end

function This:onOpenBox(box)
    self.m_boxList[box:getIndex()]:refresh()
end

function This:onAddBox(box)
    local index = box:getIndex()
    self.m_boxList[index]:addBoxAnim()
end

function This:onGetAchieveReward()
    self:initTask()
    --for _,task in pairs(self.m_taskList) do
    --    task:refresh()
    --end
end
