-- @Author: superm
-- @Date:   2017-11-28 19:33:25
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-11-22 11:27:37
-- @File Name: TianXiaAchieveView.lua
-- @Description: 

local This , super = class(...,"NodeView")
This.fileName = "MenuScene/NodeUI/GreatEventsItem.csb"

local AchieveManager = require("Achieve.AchieveManager")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

function This:init(achieve)
	self.m_inited = false
	self.m_achieve = achieve
	local w , h = self.bg:getContentSize()
	self:setContentSize(w,h)
	self:setCascadeOpacityEnabled(true)
	self.m_escape = 0
	self:getData()
end

function This:setModel( achieve, index )
	self:unscheduleUpdate()
	self.m_achieve:setView(nil)
	self.m_achieve = achieve
	self.m_achieve:setView(self)
    self:refresh()
end

function This:refresh()
	self.title:setString(self.m_achieve:getName())
	self.describe:setString(self.m_achieve:getDesc())
	self.target:setString(self.m_achieve:getTargetDesc())
	self.number:setString(tostring(self.m_achieve:getIndex()))
	self.litem:setVisible(false)
	local state = self.m_achieve:getState()
	local myId = self.m_achieve:getId()
	local curentId = self:getTxdsCurIndex()
	if myId > curentId then
		self.underway:setVisible(false)
		self.lock:setVisible(true)
		self.receive:setVisible(false)
		self.unfinish:setVisible(false)
		self.finish:setVisible(false)
	else
		self.lock:setVisible(false)
		self.underway:setVisible(false)
		self.m_leftTime = self.m_achieve:getLeftTime()
		local progress, maxValue = self.m_achieve:getProgress(), self.m_achieve:getMaxProgress()
		if state ~= AchieveState.kHaseReward then
			if self.m_achieve:hasStarted() and self.m_leftTime > 0 then
				self.underway:setVisible(true)
				self.underway.renwujindu:setPercent(progress / maxValue * 100)
				self.underway.num:setString(string.format("%d/%d", progress, maxValue))
			end
		end
		if state == AchieveState.kUpdating then
			local visible = self.m_leftTime > 0 and progress < maxValue
			if visible then
				self:scheduleUpdate()
			end
			self.underway.time:setVisible(self.m_leftTime > 0)
			self:updateTime(visible)
			self.unfinish:setVisible(not self.m_achieve:hasStarted())
			self.receive:setVisible( self.m_achieve:hasStarted() and progress >= maxValue)
		else
			self.unfinish:setVisible(false)
			self.underway.time:setVisible(false)
			self.receive:setVisible(state == AchieveState.kHaseFinish)
		end
		self.finish:setVisible(state == AchieveState.kHaseReward)

		--军团大势的积分展示
		--local numjifen = self.underway.num1
		--local jifen = self.underway.jifen
		--local isJunTuanDaShi = self.m_uiType == AchieveType.kJunTuanDaShi
		--if isJunTuanDaShi and (state == AchieveState.kUpdating or state == AchieveState.kHaseFinish) then
		--	numjifen:setString(string.format("%d", self.m_achieve:getJiFen()))
		--end
		--numjifen:setVisible(isJunTuanDaShi)
		--jifen:setVisible(isJunTuanDaShi)
	end
	self:refreshReward()
end

function This:refreshReward()
	local w = -20

	
	local itemList, count = self.m_achieve:getFuncReward()
	if count > 0 then
		if not self.m_funcItemList then
			self.m_funcItemList = {}
		end

		for i , info in ipairs(itemList) do
			local node = self.m_funcItemList[i]
			if not node then
				node = ItemNode:create(info.id, info.num)
				node:setAnchorPoint(0, 0.5)
				node:setPositionX(w)
				w = w + node:getContentWidth() + 10
				self.item:addChild(node)
				self.m_funcItemList[i] = node 
			else
				node:init(info.id, info.num)
				node:setPositionX(w)
				w = w + node:getContentWidth() + 10
				node:setVisible(true)
			end
			funcItemListCount = i
		end
		for i=count + 1, #self.m_funcItemList do
			self.m_funcItemList[i]:setVisible(false)
		end

		local visible = self.m_leftTime and self.m_leftTime <= 0
		visible = visible or self.m_achieve:getState() == AchieveState.kHaseReward
		self.litem:setVisible(true)
		self.litem.open:setVisible(visible)
		self.litem.lock:setVisible(not visible)
	elseif self.m_funcItemList then
		for _, node in ipairs(self.m_funcItemList) do
			node:setVisible(false)
		end
	end


	local itemList = self.m_achieve:getRewardList()
	if not self.m_itemList then
		self.m_itemList = {}
	end
	local itemListCount = 0
	for i , info in ipairs(itemList) do
		local node = self.m_itemList[i]
		if not node then
			node = ItemNode:create(info.id,info.num)
			node:setAnchorPoint(0, 0.5)
			node:setPositionX(w)
			w = w + node:getContentWidth() + 10
			self.item:addChild(node)
			self.m_itemList[i] = node 
		else
			node:init(info.id, info.num)
			node:setPositionX(w)
			w = w + node:getContentWidth() + 10
			node:setVisible(true)
		end	
		itemListCount = i
	end
	for i=itemListCount + 1, #self.m_itemList do
		self.m_itemList[i]:setVisible(false)
	end
end

function This:initTouchEvent()
	if not self.m_inited then
		self.m_inited = true
		self.receive:setTouch({
			onEnded = function()
				self:btnCallBack()
				self.m_achieve:setState(AchieveState.kHaseReward)
				-- self:refreshQuest()
			end,
			isScale = false,
			isContainer = true
		})
	end
end

function This:onEnter()
	super.onEnter(self)
	self:refresh()
	self.m_achieve:setView(self)
end

function This:onExit()
	super.onExit(self)
	self.m_achieve:setView(nil)
end

function This:update(dt)
	super.update(self, dt)
    if self.m_leftTime and self.m_leftTime > 0 then
        self.m_escape = self.m_escape + dt
        if self.m_escape >= 1 then
            self.m_escape = self.m_escape - 1
            self.m_leftTime = self.m_leftTime - 1
            self:updateTime(true)
        end
    else
    	self:unscheduleUpdate()
    	self:updateTime()
    	print("update print")
    	self:refreshQuest()
    end
end

function This:updateTime(update)
	if update then
		self.underway.time:setString(Serialize[60161] .. GameUtil:formatDTime(Serialize["activity_BaseTabs_1_c"], self.m_leftTime))
	else
		self.underway.time:setVisible(false)
	end
end

function This:getTxdsCurIndex()
    return AchieveManager:getTxdsCurIndex()
end

function This:btnCallBack()
   AchieveManager:onGetRewardRequest(self.m_achieve)
end

function This:refreshQuest()
    AchieveManager:onGetAchieveInfoRequest(self.m_uiType)
end

function This:getData()
	self.m_uiType = AchieveType.kTianXiaDaShi
end