-- @Author: superm
-- @Date:   2017-10-30 14:46:23
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-05 15:13:05
-- @File Name: VitalityManager.lua
-- @Description: 活跃度任务管理器

local This , super = class(...,"Base.BaseModel")
local WeeklyAchieve = require("Achieve.WeeklyAchieve")
local FamilyAchieve = require("Achieve.FamilyAchieve")
-- local FamilyManager = require("Family.FamilyManager")
local WeekMissionBoxConfig = require("Metadatas.WeekMissionBoxConfig")
local DayMissionBoxConfig = require("Metadatas.DayMissionBoxConfig")
local LegionTaskBoxConfig = require("Metadatas.LegionTaskBoxConfig")
local GamePlayer = require("Player.GamePlayer")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")

kWeeklyBoxNum = 4
kDailyBoxNum = 2

VitalityBoxType = {
	kDaily = 1,
	kWeekly = 2,
	kMember = 1,
	kFamily = 2,
}

function This:init()
	super.init(self)
    self.m_inited = false

	self.m_weeklyAchieves = {}
	self.m_weeklyVitality = 0  --周活跃度
	self.m_dailyVitality = 0  --日活跃度
	self.m_boxIsReward = {false,false,false,false}
	self.m_dailyBoxIsReward = {false,false}

    self.m_familyAchieves = {}
	self.m_familyVitality = 0
	self.m_memberVitality = 0
	self.m_familyBoxIsReward = {}

	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self,"init_role_info", self.onGetAllAchieveInfoRequest)
	EventDispatcher:addListener(self, 11900, self.onGetAchieveInfoReply)
	EventDispatcher:addListener(self, 11901, self.onAchieveInfoChangeReply)
	EventDispatcher:addListener(self, 11902, self.onGetAchieveRewardReply)
	EventDispatcher:addListener(self, 11906, self.onGetBoxRewardReply)
	EventDispatcher:addListener(self, 12128, self.onGetFamilyTaskInfoReply)
    EventDispatcher:addListener(self, 12129, self.onGetFamilyBoxRewardReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,"init_role_info")
	EventDispatcher:removeListener(self,11900)
	EventDispatcher:removeListener(self,11901)
	EventDispatcher:removeListener(self,11902)
	EventDispatcher:removeListener(self,11906)
	EventDispatcher:removeListener(self,12128)
    EventDispatcher:removeListener(self,12129)
end

function This:isInited()
	if self.m_lastRefreshTime then
		local curTime = GamePlayer:getServerTime()
		local curDay = os.serverdate("*t", curTime).yday
		self.m_inited = curDay == self.m_lastRefreshTime
	end
	return self.m_inited
end

function This:getAchieve(id, type)
	if type == AchieveType.kWeekly then
		return self.m_weeklyAchieves[id]
	else
		return self.m_familyAchieves[id]
	end
end

function This:addAchieve(info, type)
	local achieve 
	if type == AchieveType.kWeekly then
		local achieveList = self.m_weeklyAchieves
		achieve = achieveList[info.id]
		if achieve then
			achieve:updateInfo(info)
		else
			achieve = WeeklyAchieve:create(info)
			achieveList[info.id] = achieve
		end
	else
		local achieveList = self.m_familyAchieves
		achieve = achieveList[info.id]
		if achieve then
			achieve:updateInfo(info)
		else
			achieve = FamilyAchieve:create(info)
			achieveList[info.id] = achieve
		end
	end

	self:notifyObservers("onAddAchieve",achieve)
	return achieve
end

function This:removeAchieve(id, type)
	local mission 
	if type == AchieveType.kWeekly then
		mission = self.m_weeklyAchieves[id]
		self.m_weeklyAchieves[id] = nil
	else 
		mission = self.m_familyAchieves[id]
		self.m_familyAchieves[id] = nil
	end

	if mission then
		self:notifyObservers("onRemoveAchieve",mission)
	end
end

function This:updateAchieve(info)
	local achieve = self:getAchieve(info.id)
	if achieve then
		achieve:updateInfo(info)
		self:notifyObservers("onUpdateAchieve",achieve)
	end
end

function This:isBoxReward(index, type)
	if type == AchieveType.kWeekly then
		return self.m_boxIsReward[index]
	else
		return self.m_familyBoxIsReward[index]
	end
end

function This:isDailyBoxReward(index, type)
	if type == AchieveType.kWeekly then
		return self.m_dailyBoxIsReward[index]
	else
		return self.m_familyBoxIsReward[index]
	end
end

function This:getVitalityInWeek()
	return self.m_weeklyVitality
end

function This:getVitalityInDay()
	return self.m_dailyVitality
end

function This:getVitalityInFamily()
	return self.m_familyVitality
end

function This:getVitalityInFamilyMember()
	return self.m_memberVitality
end

function This:getAllAchieves(type)
	if type == AchieveType.kWeekly then
		return self.m_weeklyAchieves
    else
        return self.m_familyAchieves
	end
end

function This:getAchieveWithSort(type)
	local achieveList = {}
	local achieves = self:getAllAchieves(type)
	for i , achieve in pairs(achieves) do
		table.insert(achieveList,achieve)
	end
	table.sort(achieveList, function(a,b)
		local state1, state2 = a:getState(), b:getState()
		if state1 ~= state2 then
			return state1 < state2
		end
		return a:getId() < b:getId()
	end)
	return achieveList
end

function This:selectAchieve(selector, type)
	local result = {}
	local achieves = self:getAllAchieves(type)
	for i , achieve in pairs(achieves) do
		if not selector or selector and selector(achieve) then
			table.insert(result,achieve)
		end
	end
	return result
end

function This:isValidAchieveType( atype )
	return AchieveManageType[atype] == false
end

function This:cleanAllFamilyAchieves()
    self.m_familyAchieves = {}
	self.m_familyVitality = 0
	self.m_memberVitality = 0
	self.m_familyBoxIsReward = {}
end

--------------------------------------------

function This:onGetAllAchieveInfoRequest()
    self:onGetAchieveInfoRequest()
end

function This:onGetAchieveInfoRequest()
	Socket:send(11900, {type = AchieveType.kWeekly})
end

function This:onGetAchieveInfoReply( msg )
    if msg then
        local achieveType = msg.achievement.type
		if not self:isValidAchieveType(achieveType) then
			return 
		end
		
		if achieveType == AchieveType.kWeekly then
			self.m_lastRefreshTime = os.date("*t", GamePlayer:getServerTime()).yday 
			self.m_weeklyVitality = msg.achievement.week_arg.active_num
            self.m_dailyVitality = 0
	        if msg.achievement.week_arg.week_box_list then
			    for i , boxId in ipairs(msg.achievement.week_arg.week_box_list) do
				    self.m_boxIsReward[boxId] = true
			    end
	        end

	        if msg.achievement.week_arg.day_box_list then
			    for i , boxId in ipairs(msg.achievement.week_arg.day_box_list) do
				    self.m_dailyBoxIsReward[boxId] = true
			    end
	        end
	    end

		local tmp = {}

		for i , info in ipairs(msg.achievement.progress_list) do
			tmp[info.id] = info	
		end

        local dataList
        if achieveType == AchieveType.kWeekly then
            dataList = msg.achievement.week_arg.mission_list
        else
            dataList = self.m_sourceAchieveList
        end

        if dataList then
			for i , id in ipairs(dataList) do
				if tmp[id] then
					local achieve = self:addAchieve(tmp[id], achieveType)
					achieve:setState(AchieveState.kUpdating)
					tmp[id] = nil
				end
			end
		end

		for i , id in ipairs(msg.achievement.complete_list) do
			local achieve = self:getAchieve(id, achieveType)
			if achieve then
				achieve:setState(AchieveState.kHaseFinish)
			end
		end

		
		for i , id in ipairs(msg.achievement.reward_list) do
			local achieve = self:getAchieve(id, achieveType)
			if achieve then
				achieve:setState(AchieveState.kHaseReward)
				if msg.achievement.type == AchieveType.kWeekly then
					local delta = achieve:getVitality()
                	self.m_dailyVitality = self.m_dailyVitality + delta
                end
			end
		end

    	self.m_inited = true
        self:notifyObservers("onGetAchieveInfo", msg.achievement.type)
		EventDispatcher:dispatcher("achieve_change")
        if achieveType == AchieveType.kWeekly then
			EventDispatcher:dispatcher("weekly_achieve_refresh")
		else
			EventDispatcher:dispatcher("family_achieve_changed")
		end
		SignalSystem.distributeSignal(Signals.FamilyPanelVitalityTabRedDot)
    else
        g_scene:handleErrorCode("")
    end
end

function This:onAchieveInfoChangeReply( msg )
	local hasWeeklyAchieve = false
	local hasFamilyAchieve = false

	for i , info in ipairs(msg.progress_list) do
		if info.type == AchieveType.kWeekly or info.type == AchieveType.kFamilyActive then 
			local achieve = self:getAchieve(info.id, info.type)
			if achieve then
				achieve:updateInfo(info)
				if achieve:getState() == AchieveState.kHaseFinish then
					if not hasWeeklyAchieve and info.type == AchieveType.kWeekly then
						hasWeeklyAchieve = true
					elseif not hasFamilyAchieve and info.type == AchieveType.kWeekly then
						hasFamilyAchieve = true
					end
				end
				self:notifyObservers("onAchieveChanged", achieve)
			end
		end
	end

	if hasWeeklyAchieve then
		EventDispatcher:dispatcher("weekly_achieve_changed")
	end

	if hasFamilyAchieve then
		EventDispatcher:dispatcher("family_achieve_changed")
	end

	SignalSystem.distributeSignal(Signals.FamilyPanelVitalityTabRedDot)
end

function This:onGetAchieveRewardRequest(achieve)
	Socket:sendAndLock(11902, {type = achieve:getType(), id = achieve:getId()})
end

function This:onGetAchieveRewardReply( msg )
    if msg.err_code == 0 then
		if not self:isValidAchieveType(msg.type) then
			return 
		end
		print_r(msg)
		local achieve = self:getAchieve(msg.id, msg.type)
		achieve:setState(AchieveState.kHaseReward)

		local delta = 0
		if AchieveType.kWeekly == achieve:getType() then
			self.m_weeklyVitality = msg.gongxun
			delta = achieve:getVitality()
			self.m_dailyVitality = self.m_dailyVitality + delta
			EventDispatcher:dispatcher("weekly_achieve_changed",{achieve})
		elseif AchieveType.kFamilyActive ==  achieve:getType() then
			self.m_familyVitality = msg.group_active_num
			delta = achieve:getVitality()
			self.m_memberVitality = self.m_memberVitality + delta
			EventDispatcher:dispatcher("family_achieve_changed",{achieve})
		end
		
		g_scene:showTips(Serialize[60163] .. tostring(delta))
		local itemList = achieve:getRewardList()
		g_scene:showReward(itemList)
        self:notifyObservers("onGetAchieveReward", achieve)
		EventDispatcher:dispatcher("achieve_change")
		SignalSystem.distributeSignal(Signals.FamilyPanelVitalityTabRedDot)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetBoxRewardRequest(index, boxType)
	Socket:sendAndLock(11906, {box_id = index, type = boxType})
end

function This:onGetBoxRewardReply( msg )
    if msg.err_code == 0 then
		local config
		if msg.type == VitalityBoxType.kDaily then
			config = DayMissionBoxConfig[msg.box_id]
			self.m_dailyBoxIsReward[msg.box_id] = true
		else
			config = WeekMissionBoxConfig[msg.box_id]
			self.m_boxIsReward[msg.box_id] = true
		end
		
		if config then
			g_scene:showReward(config.reward)
		end
	    self:notifyObservers("onGetBoxReward", msg.box_id, msg.type)
		EventDispatcher:dispatcher("achieve_change")
		SignalSystem.distributeSignal(Signals.FamilyPanelVitalityTabRedDot)
	else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetFamilyBoxRewardRequest(index, boxType)
	Socket:sendAndLock(12129, {box_id = index, type = boxType})
end

function This:onGetFamilyBoxRewardReply( msg, achieveType )
    if msg.err_code == 0 then
		self.m_familyBoxIsReward[msg.box_id] = true
		local config = LegionTaskBoxConfig[msg.box_id]
		
		if config then
			g_scene:showReward(config.reward)
		end
		-- FamilyManager:addFamilyFunds(config.reward)
	    self:notifyObservers("onGetFamilyBoxReward", msg.box_id, msg.type)
		EventDispatcher:dispatcher("achieve_change")
		SignalSystem.distributeSignal(Signals.FamilyPanelVitalityTabRedDot)
	else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetFamilyTaskInfoRequest()
    self.m_sourceAchieveList = nil
	Socket:send(12128)
end

function This:onGetFamilyTaskInfoReply( msg )
	if msg.err_code == 0 then

		self.m_familyVitality = msg.group_task_num or 0
		self.m_memberVitality = msg.personal_task_num or 0
        if msg.group_reward_list then
		    for i , boxId in ipairs(msg.group_reward_list) do
			    self.m_familyBoxIsReward[boxId] = true
		    end
        end

        if msg.reward_list then
		    for i , boxId in ipairs(msg.reward_list) do
			    self.m_familyBoxIsReward[boxId] = true
		    end
        end
        
        self.m_sourceAchieveList = msg.mission_list
        self:notifyObservers("onGetFamilyTaskInfo")
        Socket:send(11900, {type = AchieveType.kFamilyActive})
	else
        g_scene:handleErrorCode(msg.err_code)
	end
end

--endregion
