-- @Author: superm
-- @Date:   2017-10-30 12:50:45
-- @Last Modified by:   superm
-- @Last Modified time: 2018-07-10 21:20:11
-- @File Name: WeeklyAchieveView.lua
-- @Description: 

local This , super = class(...,"NodeView")
This.fileName = "MenuScene/NodeUI/TakeForWeekItem.csb"

local VitalityManager = require("Achieve.VitalityManager")
local ItemNode = require("UI.ItemNode")

function This:init(achieve)
	self.m_achieve = achieve
	local w, h = self.bg:getContentSize()
	self:setContentSize(w , h)
	self.scrollview:setDirection("horizontal")
	self.scrollview.m_direction = "right"
	self:setCascadeOpacityEnabled(true)
end

function This:refresh()
	self.title:setString(self.m_achieve:getName())
	self.desc:setString(self.m_achieve:getDesc())
	self.renwujindu:setPercent(self.m_achieve:getProgress() / self.m_achieve:getMaxProgress() * 100)
	self.num:setString(string.format("%d/%d",self.m_achieve:getProgress(),self.m_achieve:getMaxProgress()))
	self.num1:setString(tostring(self.m_achieve:getVitality()))

	local state = self.m_achieve:getState()
	local hasGuider = self.m_achieve:hasGuider()
	self.notfinish:setVisible(false)
	self.finish:setVisible(state == AchieveState.kHaseReward)
	self.taskfinish:setVisible(state == AchieveState.kHaseFinish)
	if self.goto then
		self.goto:setVisible(state == AchieveState.kUpdating and hasGuider)
	end

	if self.help then
		self.help:setVisible(state == AchieveState.kUpdating and not hasGuider)
	end
	self:refreshReward()
end

function This:setModel(achieve)
    if achieve ~= self.m_achieve then
		self.m_achieve:setView(nil)
		self.m_achieve = achieve
		self.m_achieve:setView(self)
		self:refresh()
    end
end

function This:refreshReward()
	self.scrollview:removeAllItems()
	local itemList = self.m_achieve:getRewardList()
	for i , info in ipairs(itemList) do
		local item = ItemNode:create(info.id,info.num)
		local w , h = item:getContentSize()
		item:setAnchorPoint(0.5,0.5)
		self.scrollview:pushBackCustomItem(item,w,h,0.5)
	end
end

function This:initTouchEvent()
	if not self.m_inited then
		self.taskfinish:setTouch({
			onEnded = function()
				VitalityManager:onGetAchieveRewardRequest(self.m_achieve)
			end,
			isScale = false,
			isContainer = true
		})
		
		if self.goto then
			self.goto:setTouch({
				onEnded = function()
					if self.m_gotoHandler then
						self.m_gotoHandler(self.m_achieve)
					end
				end,
				isScale = false,
				isContainer = true
			})
		end

		if self.help then
			self.help:setTouch({
				onEnded = function()
					if self.m_gotoHandler then
						self.m_gotoHandler(self.m_achieve)
					end
				end,
				isScale = false,
				isContainer = true
			})
		end

		self.scrollview:setSwallowTouch(false)
		self.m_inited = true
	end
end

function This:onEnter()
	super.onEnter(self)
	self:refresh()
	self.m_achieve:setView(self)
end

function This:onExit()
	super.onExit(self)
	self.m_achieve:setView(nil)
end

function This:setGotoHandler(handler)
	self.m_gotoHandler = handler
end
