-- @Author: superm
-- @Date:   2017-10-27 15:57:01
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-26 18:21:54
-- @File Name: WeeklyTabNode.lua
-- @Description: 

local This , super = class(..., "NodeUI")
local WeekMissionBoxConfig = require("Metadatas.WeekMissionBoxConfig")
local DayMissionBoxConfig = require("Metadatas.DayMissionBoxConfig")
local WeeklyAchieveView = require("Achieve.WeeklyAchieveView")
local VitalityManager = require("Achieve.VitalityManager")
local GameUtil = require("Util.GameUtil")
local EventDispatcher = require("EventDispatcher")

local BoxImage = 
{
	[1] = {
		"MenuScene/boxLock1.png",
		"MenuScene/boxGet1.png",
	},
	[2] = {
		"MenuScene/boxLock2.png",
		"MenuScene/boxGet2.png",
	},
	[3] = {
		"MenuScene/boxLock3.png",
		"MenuScene/boxGet3.png",
	},
	[4] = {
		"MenuScene/boxLock4.png",
		"MenuScene/boxGet4.png",
	}
}

function This:initNode()
	self.m_curViews = {}
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TakeForWeekBg.csb")
	self:addChild(self.m_csNode)
	self.m_boxEffect={}

	self:initBoxInfo()
end

function This:initBoxInfo()
	self.m_maxVitality = WeekMissionBoxConfig[kWeeklyBoxNum].vitality
	local posx, width = self.m_csNode.jindubg:getPositionX(), self.m_csNode.jindubg:getContentWidth()
	for i = 1 , kWeeklyBoxNum do
		local node = self.m_csNode["bg"..i]
		local vitality = WeekMissionBoxConfig[i].vitality
		node.text:setString(tostring(vitality))

		local curPos = posx + width * vitality / self.m_maxVitality
		self.m_csNode["boxLock" .. i]:setPositionX(curPos)
		node:setPositionX(curPos)
	end

	for i=1, kDailyBoxNum do 
		local config = DayMissionBoxConfig[i]
		if config then
			self.m_csNode["num" .. i]:setString(tostring(config.vitality))
		else
			self.m_csNode["dailybox" .. i]:setVisible(false)
			self.m_csNode["num" .. i]:setVisible(false)
			self.m_csNode["text" .. i]:setVisible(false)
		end
	end

	if VitalityManager:isInited() then
		self:refresh()
		self:refreshItem()
	else
		VitalityManager:onGetAchieveInfoRequest()
	end
end

function This:refresh()
	local achieveType = AchieveType.kWeekly
	local vitality = VitalityManager:getVitalityInWeek() 
	self.m_csNode.jindubg.jindutiao:setPercent(vitality / self.m_maxVitality * 100)
	self.m_csNode.weekvitality:setString(tostring(vitality))

	local dailyVitality = VitalityManager:getVitalityInDay()
	self.m_csNode.dailyvitality:setString(tostring(dailyVitality))
	
	for i = 1 , kWeeklyBoxNum do
		local node = self.m_csNode["boxLock"..i]
		if VitalityManager:isBoxReward(i, achieveType) then
			node:setGrayShader()
			node:setImage(BoxImage[i][2])
			if self.m_boxEffect[i] then
				for i , effect in ipairs(self.m_boxEffect[i]) do
					effect:removeFromParent()
				end
				self.m_boxEffect[i] = nil
			end
		elseif vitality >= WeekMissionBoxConfig[i].vitality then
            node:clearShader()
			node:setImage(BoxImage[i][2])
			if not self.m_boxEffect[i] then
				local w ,h = node:getContentSize()
				local effect1 = GameUtil:createEffect("BoxLing",true)
				effect1:setPosition(w/2, h/2)
				node:addChild(effect1, -1)
				local effect2 = GameUtil:createEffect("BoxLing2",true)
				node:addChild(effect2, -1)
				effect2:setPosition(w/2,h)
				self.m_boxEffect[i] = {effect1,effect2}
			end
		else
            node:clearShader()
			node:setImage(BoxImage[i][1])
		end
	end

	for i = 1 , kDailyBoxNum do
		local node = self.m_csNode["dailybox"..i]
		local index = i + kWeeklyBoxNum
		if node:isVisible() then
			if VitalityManager:isDailyBoxReward(i, achieveType) then
				node:setGrayShader()
			    node:setImage(BoxImage[4][2])
				if self.m_boxEffect[index] then
					for i , effect in ipairs(self.m_boxEffect[index]) do
						effect:removeFromParent()
					end
					self.m_boxEffect[index] = nil
				end
			elseif dailyVitality >= DayMissionBoxConfig[i].vitality then
                node:clearShader()
				node:setImage(BoxImage[4][2])
				if not self.m_boxEffect[index] then
					local w ,h = node:getContentSize()
					local effect1 = GameUtil:createEffect("BoxLing",true)
					effect1:setPosition(w/2, h/2)
					node:addChild(effect1, -1)
					local effect2 = GameUtil:createEffect("BoxLing2",true)
					node:addChild(effect2, -1)
					effect2:setPosition(w/2, h)
					self.m_boxEffect[index] = {effect1,effect2}
				end
			else
                node:clearShader()
				node:setImage(BoxImage[4][1])
			end
		end
	end
end


function This:refreshItem()
	self.m_csNode.scrollView:removeAllItems()
	self:removeAllTimer()
	local achieveList = VitalityManager:getAchieveWithSort(AchieveType.kWeekly)
	for i , achieve in ipairs(achieveList) do
		self:setTimer(i * 0.01, function()
			local id = achieve:getId()
			local view = self.m_curViews[id]
			if not view then
				view = WeeklyAchieveView:create(achieve)
				self.m_curViews[id] = view
				view:setGotoHandler(function (ac)
				    if ac:hasGuider() then
						local panel = g_scene:getDialog("AchieveMainPanel")
						if not ac:triggerAncGuider() then
							panel:close()
						end
				    else
				        g_scene:openDialog("UI.ConfirmPanel", {richText = ac:getWarming()})
				    end
				end)
				view:retain()
			else
				view:refresh()
			end
			local w , h = view:getContentSize()
			self.m_csNode.scrollView:pushBackCustomItem(view, w, h)
		end)
	end
end

function This:initTouchEvent()
	for i = 1 , kWeeklyBoxNum do
		self.m_csNode["boxLock"..i]:setTouch({
			onEnded = function()
				-- if not VitalityManager:isBoxReward(i) then
					local vitality = VitalityManager:getVitalityInWeek()
					g_scene:openDialog("Achieve.WeeklyAchieveBoxPanel", {vitality = vitality, type = VitalityBoxType.kWeekly, index = i, config = WeekMissionBoxConfig[i]})
				-- end
			end,
			isScale = false,
		})
	end

	for i = 1 , kDailyBoxNum do
		self.m_csNode["dailybox" .. i]:setTouch({
			onEnded = function()
				-- if not VitalityManager:isDailyBoxReward(i) then
					local vitality = VitalityManager:getVitalityInDay()
					g_scene:openDialog("Achieve.WeeklyAchieveBoxPanel", {vitality = vitality, type = VitalityBoxType.kDaily, index = i, config = DayMissionBoxConfig[i]})
				-- end
			end,
			isScale = false,
		})
	end

	self.m_csNode.help:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text = Serialize["weekly_help"],node = self.m_csNode.help})
	end)
end

function This:onEnter()
	super.onEnter(self)
	VitalityManager:addObserver(self)
end

function This:onExit()
	if self.m_curViews then
		for _, view in pairs(self.m_curViews) do
			if view and view[0] ~= 0 then
				view:release()
			end
		end
	end
	super.onExit(self)
	VitalityManager:removeObserver(self)
end

function This:onGetAchieveInfo()
	self:refresh()
	self:refreshItem()
end

function This:onGetAchieveReward()
	self:refresh()
	self:refreshItem()
end