--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local TouchAllListener = require("cc.TouchAllListener")
local Vec = require("Vec")
local Timer = require("Timer")

function This:create(...)
	local o = {}
	setmetatable(o,self)
	o:init(...)
	return o
end

function This:init(info)
	self.m_node = info.node
	self.m_minWidth = info.minWidth
	self.m_minHeight = info.minHeight
	self.m_maxWidth = info.maxWidth
	self.m_maxHeight = info.maxHeight
	self.m_initScale = info.scale
	local width , height = self.m_node:getContentSize()
	self.m_maxScale = math.min(self.m_maxWidth / width ,self.m_maxHeight / height)
	self.m_minScale = math.max(self.m_minWidth / width ,self.m_minHeight / height)

	self.m_distance = 0
	self.m_curScale = self.m_initScale or self.m_maxScale
	self.m_node:setScale(self.m_curScale)
	print("dragnode initscale = ",self.m_curScale)
	
	self.m_timer = Timer:create(self)
	self.m_isEnable = true
	self.m_touchList = {}
	if self.m_initScale then
		self:refreshNode(cc.Vec(0,0),self.m_initScale)
	end
end

function This:isEnable()
	return self.m_isEnable
end

function This:setEnable(isEnable)
	self.m_isEnable = isEnable
end

function This:setScaleHandler(handler)
	self.m_scaleHandler = handler
end

function This:addIndependentListener()
	self.m_allListener = TouchAllListener:create({
		onMoved = function(_,touchList)
			self:onTouchesMoved(touchList)
		end,
		onEnded = function(_,touchList)
			self:onTouchesEnded(touchList)
		end,
		onBegan = function(_,touchList)
			self:onTouchesBegan(touchList)
		end
	})
	self.m_node:addEventListener(self.m_allListener)
end

function This:getCenter(touchList)
	local x , y , count = 0 , 0 , 0
	for i , p in pairs(touchList) do
		x = x + p.x
		y = y + p.y
		count = count + 1
	end
	if count > 0 then
		x = x / count
		y = y / count
	end
	return {x = x , y = y}
end

function This:getDistance(center , touchList)
	local dist = 0
	for i , p in ipairs(touchList) do
		dist = dist + Vec.getDistance(p,center)
	end	
	return math.sqrt(dist)
end

function This:setSlideHandler(handler)
	self.m_slideHandler = handler
end

function This:refreshNode(touchCenter,newScale)
	local curScale = self.m_node:getScale()
	if curScale == newScale then return end

	local boundBox = self.m_node:getBoundingBox()
	local x , y = self.m_node:convertToNodeSpace(touchCenter.x,touchCenter.y)
	local scalex = x / boundBox.width
	local scaley = y / boundBox.height
	x = (curScale - newScale) * boundBox.width * scalex + self.m_node:getPositionX()
	y = (curScale - newScale) * boundBox.height * scaley + self.m_node:getPositionY()

	local width , height = self.m_node:getContentSize()
	width = width * newScale
	height = height * newScale

	if x > 0 then x = 0 end
	if y > 0 then y = 0 end
	if x < self.m_minWidth - width then x = self.m_minWidth - width end
	if y < self.m_minHeight - height then y = self.m_minHeight - height end
	self.m_node:setScale(newScale)
	self.m_node:setPosition(x,y)
	self.m_curScale = newScale
	print("newScale = ",newScale)
	if self.m_scaleHandler then
		self.m_scaleHandler(newScale)
	end
end

function This:onTouchesBegan(touchList)
	if table.count(self.m_touchList) >= 2 then
		return
	end

	for i , touch in ipairs(touchList) do
		if i <= 2 then
			self.m_touchList[touch.id] = {x = touch.x , y = touch.y}
		end
	end

	if table.count(self.m_touchList) > 1 then
		local center = self:getCenter(self.m_touchList)
		self.m_distance = self:getDistance(center,self.m_touchList)
		self.m_center = self:getCenter(self.m_touchList)
		print("onTouchesBegan self.m_distance = ",self.m_distance)
	end
end

function This:onTouchesMoved(touchList)
	if table.count(self.m_touchList) < 2 or not self.m_isEnable or self.m_distance <= 0 then 
		return
	end

	for i , touch in ipairs(touchList) do
		if self.m_touchList[touch.id] then
			self.m_touchList[touch.id] = {x = touch.x , y = touch.y}
		end
	end

	local dist = self:getDistance(self.m_center,self.m_touchList)
	local scale = self.m_curScale * dist / self.m_distance
	print(string.format("m_curScale = %f , dist = %f, m_distance = %f , scale = %f",self.m_curScale,dist,self.m_distance,scale))
	
	if scale < self.m_minScale then
		scale = self.m_minScale
	end
	if scale > self.m_maxScale then
		scale = self.m_maxScale
	end
	self:refreshNode(self.m_center,scale)
	self.m_distance = dist
end

function This:onTouchesEnded(touchList)
	for i , touch in pairs(touchList) do
		self.m_touchList[touch.id] = nil
	end
end

local touchList = {}
function This:updateTouch()
	if #touchList <= 0 then
		for i = 1, 3 do
			local x = math.random(1,cc.visible.width)
			local y = math.random(1,cc.visible.height)
			table.insert(touchList,{x = x, y = y})
		end
		self:onTouchesBegan(touchList)
	end
	local center = self:getCenter(touchList)
	for i , touch in ipairs(touchList) do
		touch.x = (center.x - touch.x) / 500 + touch.x
		touch.y = (center.y - touch.y) / 500 + touch.y
	end
	self:onTouchesMoved(touchList)
end

function This:test()
	for i = 1 , 1000 do
		self.m_timer:setTimer(i * 0.05,function ()
			self:updateTouch()
		end,"test"..i)
	end
end

function This:update(dt)
	self.m_timer:update(dt)
end



--endregion
