--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local TouchListener = require("cc.TouchListener")

function This:create(info)
	local o = {}
	setmetatable(o,self)
	o:init(info)
	return o
end

function This:init(info)
	self.m_node = info.node
	self.m_curPoint = cc.Vec()
	self.m_prePoint = cc.Vec()
	self.m_originalPoint = cc.Vec()
	self.m_isEnable = true
	self.m_onTouchHandler = nil
end

function This:isEnable()
	return self.m_isEnable
end

function This:setEnable(isEnable)
	self.m_isEnable = isEnable
end

function This:setTouchHandler(handler)
	self.m_onTouchHandler = handler
end

function This:setSlideHandler(handler)
	self.m_slideHandler = handler
end

function This:addIndependentListener()
	self.m_listener = TouchListener:create({
		onBegan = function(_,touch)
			return self:onTouchBegan(touch)
		end,

		onMoved = function(_,touch)
			self:onTouchMoved(touch)
		end,

		onEnded = function(_,touch)
			self:onTouchEnded(touch)
		end,
		isSwallow = false,
	})
	self.m_node:addEventListener(self.m_listener)
end

function This:refreshNode(x,y)
	local boundBox = self.m_node:getBoundingBox()
	if x > 0 then x = 0 end
	if x < cc.visible.width - boundBox.width then
		x = cc.visible.width - boundBox.width 
	end

	if y > 0 then y = 0 end
	if y < cc.visible.height - boundBox.height then
		y = cc.visible.height - boundBox.height 
	end
	self.m_node:setPosition(x, y)
	if self.m_slideHandler then
		self.m_slideHandler(x,y)
	end
end

function This:onTouchBegan(touch)
	if not self.m_isEnable then
		return false
	end
	self.m_curPoint.x = touch.x
	self.m_curPoint.y = touch.y
	self.m_prePoint.x = touch.x
	self.m_prePoint.y = touch.y
	self.m_isTouched = true
	self.m_curTouchTime = os.clock()
	self.m_preTouchTime = os.clock()
	local x , y = self.m_node:getPosition()
	self.m_originalPoint.x = x
	self.m_originalPoint.y = y
	return true
end

function This:onTouchMoved(touch)
	if not self.m_isEnable then return end
	local dx = touch.x - self.m_curPoint.x
	local dy = touch.y - self.m_curPoint.y
	local x , y = self.m_node:getPosition()
	self:refreshNode(dx + x, dy + y)
	self.m_prePoint.x = self.m_curPoint.x
	self.m_prePoint.y = self.m_curPoint.y
	self.m_curPoint.x = touch.x
	self.m_curPoint.y = touch.y
	self.m_preTouchTime = self.m_curTouchTime
	self.m_curTouchTime = os.clock()
end

function This:onTouchEnded(touch)
	self.m_isTouched = false
	if not self.m_isEnable or not self.m_preTouchTime then return end
	
	local x , y = self.m_node:getPosition()
	if math.abs(x - self.m_originalPoint.x) < 1 and math.abs(y - self.m_originalPoint.y) < 1 then
		return
	end

	local dt = os.clock() - self.m_preTouchTime
	if dt <= 0.000001 then
		print(string.format("cur = %f, pre = %f",os.clock(),self.m_preTouchTime))
		self.m_delatPoint = nil
		return
	end

	local dx = self.m_curPoint.x - self.m_prePoint.x
	local dy = self.m_curPoint.y - self.m_prePoint.y
	self.m_delatPoint = {}
	self.m_delatPoint.x = dx > 0 and math.sqrt(dx / dt) or -math.sqrt(-dx / dt)
	self.m_delatPoint.y = dy > 0 and math.sqrt(dy / dt) or -math.sqrt(-dy / dt)	
	print(string.format("惯性:(%f,%f)",self.m_delatPoint.x, self.m_delatPoint.y))
end

function This:refreshNodePosition(newx, newy)
	local boundBox = self.m_node:getBoundingBox()
	local x , y = self.m_node:getPosition()
	local minx = cc.visible.width - boundBox.width
	local rate = 20
	if newx < minx then
		--newx = minx + (minx - newx)
		newx = minx
		self.m_delatPoint.x = -self.m_delatPoint.x / rate
	end
	if newx > 0 then
		--newx = -newx
		newx = 0
		self.m_delatPoint.x = -self.m_delatPoint.x / rate
	end

	local miny = cc.visible.height - boundBox.height
	if newy < miny then
		--newy = miny + (miny - newy)
		newy = miny
		self.m_delatPoint.y = -self.m_delatPoint.y / rate
	end

	if newy > 0 then
		--newy = -newy
		newy = 0
		self.m_delatPoint.y = -self.m_delatPoint.y / rate
	end
	self.m_node:setPosition(newx,newy)
	if self.m_slideHandler then
		self.m_slideHandler(newx,newy)
	end
end

function This:update(dt)
	if not self.m_isTouched and self.m_delatPoint then
		local dx = self.m_delatPoint.x * dt
		local dy = self.m_delatPoint.y * dt
		local x , y = self.m_node:getPosition()
		self:refreshNodePosition(x + dx, y + dy)

		local rate = 100
		local av = {x = -self.m_delatPoint.x / rate, y = -self.m_delatPoint.y / rate}
		self.m_delatPoint.x = self.m_delatPoint.x + av.x
		if math.abs(self.m_delatPoint.x) <= 1 then
			self.m_delatPoint.x = 0
		end

		self.m_delatPoint.y = self.m_delatPoint.y + av.y
		if math.abs(self.m_delatPoint.y) <= 1 then
			self.m_delatPoint.y = 0
		end
		if self.m_delatPoint.x == 0 or self.m_delatPoint.y == 0 then
			self.m_delatPoint = nil
		end
	end
end

--endregion
