-- @Author: superm
-- @Date:   2018-01-10 16:33:08
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-09 11:16:27
-- @File Name: ActAddOfficeQueueModel.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")

local This , super = class(..., "Activity.ActivityQueueBaseModel")
local ActAddOfficeModel = class( This._NAME .. ".ActAddOfficeModel", "Activity.ActivityBaseModel")

local MatchingConfig = require("Metadatas.MatchingConfig")
local MatchManager = require("Match.MatchManager")

function ActAddOfficeModel:getCondition()
	return self.m_config.office
end

function ActAddOfficeModel:getLimitNum()
    return tostring(self:getCondition())
end

function ActAddOfficeModel:getCurDescNum()
    local curNum = self:getQueue():getCurNum(self.m_config.office) or 0
    local limitNum = self:getCondition()
    if limitNum <= curNum then
        return tostring(limitNum)
    else
        return tostring(curNum)
    end
end

function ActAddOfficeModel:updateState(isInit, info)
    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end
    local ret = false
    local queue = self:getQueue()
    if queue then
        local office = queue:getCurNum() or 1
        ret = office >= self:getCondition()
    end
    self.m_state = ret and ActObjectState.kNormal or ActObjectState.kLocked
end

function This:initBeforeUpdateState( info, createHandler )
	if not createHandler then
		createHandler = function( actQueueId, activityIndex, actCfg, queue )
			local activity = ActAddOfficeModel:create({type = actQueueId, id = activityIndex, config = actCfg})
			activity:setQueue(queue)
			return activity
		end
	end

	super.initBeforeUpdateState( self, info, createHandler )
end

function This:getCurNum(index)
    return MatchManager:getMaxRankLvInHistory()
end

function This:getProgressByIndex(index)
    return self:getAllObjectState()
end

function This:getCSFileName()
    return "MenuScene/NodeUI/ActivityOffice.csb"
end

function This:getAllObjectState(update)
    if update then
        local record, backup
        self.m_allObejectState = nil
        for id, act in pairs(self.m_objects) do
            if act:isOpening() then
                record = act
                self.m_allObejectState = true
                break
            elseif not backup and act:isLocked() then
                backup = act
            end
        end

        if not record and not backup then
            self.m_allObejectState = false
        end
    end
    return self.m_allObejectState  
end

function This:getConditionDesc()
    local targetRankLv
    local score = 0
    for id, act in pairs(self.m_objects) do
        if not act:isCompleted() then
            targetRankLv = act:getCondition()
            break
        end
    end
    local rankLv = MatchManager:getMaxRankLvInHistory()
    if targetRankLv and targetRankLv > rankLv then
        for i=rankLv, targetRankLv - 1 do
            score = score + MatchingConfig[i].score
        end
        score = score - MatchManager:getScore()
    end
    if AppText:isEnglish() then
        return "" .. score
    else
        return "" .. score .. Serialize[60008]
    end
end

function This:showTimer()
    return true
end

function This:showCondition()
    return true
end