-- @Author: superm
-- @Date:   2018-03-06 11:02:15
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 15:59:52
-- @File Name: ActImpactChampionQueueModel.lua
-- @Description: 冲冠

local GamePlayer = require("Player.GamePlayer")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")

local This , super = class(..., "Activity.ActivityQueueBaseModel")
local ActImpactChampionModel = class( This._NAME .. ".ActImpactChampionModel", "Activity.ActivityBaseModel")

-- function ActImpactChampionModel:isFinish()
--     local limitNum = self:getRewardLimitNum()
--     local selfIndex = self:getRewardIndex()
--     local ret = self:getRewardNumber() >= limitNum
--     if not ret and selfIndex > 0  then
--         ret = selfIndex <= limitNum
--     end
--     print_r({selfIndex = selfIndex})
--     return ret
-- end

function ActImpactChampionModel:getUIFileName()
    if self:getConfigType() == ActivityImpactType.kRank then
        return "MenuScene/NodeUI/ActivityRankOfficeItem.csb"
    else
        return "MenuScene/NodeUI/ActivityRankItem.csb"
    end
end

function ActImpactChampionModel:getCurNum(index)
	if self.m_config.type == ActivityImpactType.kLevel then
		return GamePlayer:getLevel()
	elseif self.m_config.type == ActivityImpactType.kRank then
        local ret = math.max(MatchManager:getRankLv(), MatchManager:getMaxRankLvInHistory())
    	return ret
    end
    return 0
end

function ActImpactChampionModel:getCondition()
	return self.m_config.num
end

function ActImpactChampionModel:getDesc()
    local limitNum = self:getCondition() or 0
    if self.m_config.type == ActivityImpactType.kLevel then
        return tostring(limitNum) .. Serialize[60187]
    elseif self.m_config.type == ActivityImpactType.kRank then
        local rank = limitNum > 0 and limitNum or 1
        return MatchingConfig[rank].rank_level
    end
end

function ActImpactChampionModel:getLeftRewardNumber()
    local ret = self:getRewardLimitNum() - self:getRewardNumber()
    return ret < 0 and 0 or ret
end

function ActImpactChampionModel:updateState(isInit, info)
    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end

    local ret = self:getCurNum() >= self:getCondition()
    if ret then
        ret = self:getRewardIndex() > 0 and self:getRewardIndex() <= self:getRewardLimitNum()
    end
    self.m_state = ret and ActObjectState.kNormal or ActObjectState.kLocked
end

function This:initBeforeUpdateState( info, createHandler )
	if not createHandler then
		createHandler = function( actQueueId, activityIndex, actCfg, queue )
			local activity = ActImpactChampionModel:create({type = actQueueId, id = activityIndex, config = actCfg})
			activity:setQueue(queue)
			return activity
		end
	end

	super.initBeforeUpdateState( self, info, createHandler )
    if info.reward_list then
		for _, activityId in ipairs(info.reward_list) do
			local act = self.m_objects[activityId]
            if act then
                act:setCompleted()
            end
		end
	end
end

function This:getUITypeName()
    return "Activity.ActivityImpactChampionUI"
end

function This:showTimer()
    return true
end