-- @Author: superm
-- @Date:   2018-01-04 21:19:08
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-07 16:20:56
-- @File Name: ActInvestmentPlanQueueModel.lua
-- @Description: 投资计划

local This , super = class(..., "Activity.ActivityQueueBaseModel")
local ActInvestmentPlanModel = class( This._NAME .. ".ActInvestmentPlanModel", "Activity.ActivityBaseModel")

local ActivityManager = require("Activity.ActivityManager")
local EventDispatcher = require("EventDispatcher")

local function isTable( obj )
    if obj and type(obj) == "table" then
        return true
    end
    return false
end

local ActInvestCostType = {
    gold = 1,
    coins = 2,  
}

function ActInvestmentPlanModel:getCondition()
	return self.m_config.day
end

function ActInvestmentPlanModel:getLimitNum()
    return tostring(self:getCondition())
end

function ActInvestmentPlanModel:getCurDescNum()
    local curNum = self:getQueue():getCurNum(self.m_config.Investment_type) or 0
    local limitNum = self:getCondition()
    if limitNum <= curNum then
        return tostring(limitNum)
    else
        return tostring(curNum)
    end
end

function ActInvestmentPlanModel:updateState(isInit, info)
    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end
    local ret = false
    local queue = self:getQueue()
    if queue then
        local days = queue:getCurNum(self.m_config.Investment_type) or 0
        ret = days >= self:getCondition()
    end
	local state = ret and ActObjectState.kNormal or ActObjectState.kLocked
	self.m_state = state
end

function This:initBeforeUpdateState( info, createHandler )
	if not createHandler then
		createHandler = function( actQueueId, activityIndex, actCfg, queue )
			local activity = ActInvestmentPlanModel:create({type = actQueueId, id = activityIndex, config = actCfg})
			activity:setQueue(queue)
			return activity
		end
	end

	super.initBeforeUpdateState( self, info, createHandler )
end

function This:getUITypeName()
    return "Activity.ActivityBaseTabsUI"
end

function This:initGroupsInfo( config )
    self:getGroupNameList( config, "Investment_type", function ( value )
        return value == ActInvestCostType.gold and Serialize["activity_ActInvestment_1"] or Serialize["activity_ActInvestment_2"]
    end )
end

function This:setCurNum(info)
    if not info or not info.type then
        return
    end

    if not self.m_curNumArr then
        self.m_curNumArr = {}
    end
    if isTable(info.type) then
        for i, key in ipairs(info.type) do
            if ActInvestCostType.gold == key then
                self.m_curNumArr[key] = info.gold_days
            else
                self.m_curNumArr[key] = info.coins_days
            end
        end
    else
        if ActInvestCostType.gold == info.type then
            self.m_curNumArr[info.type] = info.gold_days
        else
            self.m_curNumArr[info.type] = info.coins_days
        end
    end
end

function This:getProgressByIndex(index)
    return self:getCurNum(index)
end

function This:getCurNum(index)
    if self.m_curNumArr then
	   return self.m_curNumArr[index]
    end
end

function This:getCSFileName()
    return "MenuScene/NodeUI/ActivityInvestment.csb"
end

function This:getCost( ctype )
    if ActInvestCostType.gold == ctype then
        return ActFormula.kInvestmentPlanCostGold(), 102
    elseif ActInvestCostType.coins == ctype then
        return ActFormula.kInvestmentPlanCostGoins(), 101
    end
    return 0, false
end

function This:requestRecharge( index )
    ActivityManager:onDoActivateActRequest(self:getId(), index)
end

function This:getAllObjectState(update)
    if update then
        local record, backup
        self.m_allObejectState = nil
        for id, act in pairs(self.m_objects) do
            if act:isOpening() then
                record = act
                self.m_allObejectState = true
                break
            elseif not backup and act:isLocked() then
                backup = act
            end
        end

        if not record and not backup then
            self.m_allObejectState = false
        end
    end
    return self.m_allObejectState  
end

function This:getActivateDesc()
    return Serialize["activity_ActInvestment_3"]
end

function This:getCostDesc()
    return Serialize["activity_ActInvestment_4"]
end