-- @Author: superm
-- @Date:   2018-03-07 17:40:33
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:22:32
-- @File Name: ActMonthlyCardQueueModel.lua
-- @Description: 月卡活动

local This , super = class(..., "Activity.ActivityQueueBaseModel")
local ActMonthlyCardModel = class( This._NAME .. ".ActMonthlyCardModel", "Activity.ActivityBaseModel")

local GamePlayer = require("Player.GamePlayer")
local ActivityManager = require("Activity.ActivityManager")

function ActMonthlyCardModel:getCondition()
    return self.m_config.days
end

function ActMonthlyCardModel:getCost()
    return self.m_config.price
end

function ActMonthlyCardModel:getLimitNum()
    return tostring(self:getCondition())
end

function ActMonthlyCardModel:getCurDescNum()
    local curNum = self:getQueue():getCurNum(self:getId())
    local curTime = GamePlayer:getServerTime()
    if not curNum or curNum <= 0 or curNum > curTime then
        return 0
    end 

    local startDate = os.serverdate("*t", curNum)
    local startYear, startDay = startDate.year , startDate.yday
    local curDay = os.serverdate("*t", curTime).yday
    if curDay < startDay then
        if (startYear % 4 == 0 and startYear % 100 ~= 0) or (startYear % 400 == 0) then
            curDay = curDay + 366
        else
            curDay = curDay + 365
        end
    end

    local limitNum = self:getCondition()
    local ret = limitNum - curDay + startDay
    if ret > limitNum then
        return limitNum
    elseif ret > 0 then
        return ret
    else
        return 0
    end
end

function ActMonthlyCardModel:getContextDesc()
    return self.m_config.context
end

function ActMonthlyCardModel:getTypeDesc()
    return self.m_config.type_desc
end

function ActMonthlyCardModel:getLeftDays()
    return self.m_config.days
end

function ActMonthlyCardModel:requestPickUp()
    if self:isOpening() then
        ActivityManager:onPickupActRewardRequest(self:getQueueConfigType(), self:getType(), self:getId())
        return true
    elseif self:isLocked() then
        -- g_scene:pause()
        local SdkManager = require("Sdk.SdkManager")
        SdkManager:onApplyBillIDRequest(self.m_config.id)        
        return true
    end
    return false
end

function ActMonthlyCardModel:updateState(isInit, info)
    if not self:getQueue() then
        return 
    end 

    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end

    local ret = self:getCurDescNum() > 0
	local state = ret and ActObjectState.kNormal or ActObjectState.kLocked
	self.m_state = state
end

function This:initBeforeUpdateState( info, createHandler )
	if not createHandler then
		createHandler = function( actQueueId, activityIndex, actCfg, queue )
			local activity = ActMonthlyCardModel:create({type = actQueueId, id = activityIndex, config = actCfg})
			activity:setQueue(queue)
			return activity
		end
	end

	super.initBeforeUpdateState( self, info, createHandler )
end

function This:setCurNum(info)
    if not info then
        return
    end

    if not self.m_curNumArr then
        self.m_curNumArr = {}
    end
end

function This:getCurNum(index)
    if self.m_curNumArr then
	   return self.m_curNumArr[index]
    end
end

function This:getAllObjectState(update)
    if update then
        local record, backup
        self.m_allObejectState = nil
        for id, act in pairs(self.m_objects) do
            if act:isOpening() then
                record = act
                self.m_allObejectState = true
                break
            elseif not backup and act:isLocked() then
                backup = act
            end
        end

        if not record and not backup then
            self.m_allObejectState = false
        end
    end
    return self.m_allObejectState  
end

function This:getUITypeName()
    return "Activity.ActivityMonthlyCardUI"
end

function This:setCurNum(info)
    if info.activity_list then
        self.m_curNumArr = {}
        for _, act in pairs(self.m_objects) do
            if act then
                act:setState(ActObjectState.kLocked)
            end
        end
        for i, actInfo in ipairs(info.activity_list) do
            local activityId = actInfo.id
            self.m_curNumArr[ activityId ] = actInfo.start_time
            local act = self.m_objects[activityId]
            if act and actInfo.is_reward then
                act:setCompleted()
            end
        end
    end
end

function This:getCurNum(index)
    if self.m_curNumArr then
        return self.m_curNumArr[index or 1] or 0
    end
    return 0
end
