-- @Author: superm
-- @Date:   2018-02-28 14:35:21
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:21:50
-- @File Name: ActNewYearQueueModel.lua
-- @Description: 新年活动

local This , super = class(..., "Activity.ActOpenTaskQueueModel")
local ActNewYearModel = class( This._NAME .. ".ActNewYearModel", super._NAME .. ".ActOpenTaskModel")
local ActNewYearExchangeModel = class( This._NAME .. ".ActNewYearExchangeModel", super._NAME ..".ActOpenTaskExchangeModel")

function ActNewYearModel:isValid()
    local queue = self:getQueue()
    if not self:isSpecial() and queue then
		local day = queue:getActivityCreateDays()
        return self.m_config.open_num == day
	end
end

function ActNewYearExchangeModel:isValid()
    local queue = self:getQueue()
    if queue then
        local day = queue:getActivityCreateDays()
        return self.m_config.open_num == day
    end
end

function This:getExchangeCreator()
    return ActNewYearExchangeModel
end

function This:initBeforeUpdateState( info, createHandler )
	if not createHandler then
		createHandler = function( actQueueId, activityIndex, actCfg, queue )
			local activity = ActNewYearModel:create({type = actQueueId, id = activityIndex, config = actCfg})
			activity:setQueue(queue)
			return activity
		end
	end

	if info.initing and createHandler then
		self.m_objects = {}
		self.m_eobjects = {}
		local typeName = self:getTypeName()
		local config = require("Metadatas." .. typeName .. "Config")
        local actConfigList = {}
        local queueId = self:getId()
        local originLevel = self:getOriginalLevel()

		for cfgIndex, cfg in pairs(config) do
            if self:isSuitable(cfg, queueId, originLevel) then
				local act = createHandler(self:getId(), cfgIndex, cfg, self)
		    	self.m_objects[act:getId()] = act
                table.insert(actConfigList, cfg)
			end
		end

		local finishHandler = self.initGroupsInfo
		if finishHandler then
			finishHandler(self, actConfigList)
		end

        actConfigList = self:initExchangeObjList(typeName)
        local finishHandler = self.initExchangeGroupsInfo
        if finishHandler and actConfigList then
            finishHandler(self, actConfigList)
        end
	end
end

function This:initExchangeGroupsInfo( config )
    self:getGroupNameList( config, "open_num", function ( value )
        return string.format(Serialize["activity_ActOpen_1_c"], value) 
    end, true)
end

function This:getAllExchangeObjectState(update)
	if self:hasExchangeBehaviour() then
        if update then
            self.m_allExchangeObejectState = false
            for id, act in pairs(self.m_eobjects) do
                if act:isOpening() and act:isValid() then
                    self.m_allExchangeObejectState = true
                    break
                end
            end
        end
        return self.m_allExchangeObejectState  
    end
end

function This:getUITypeName(index)
	local nameIndex = index
	if self.m_uiconfig then
		config = self.m_uiconfig[index]
		if config then
			nameIndex = config.ui_type
		end
	end

	if nameIndex == 1 then
		return "NewActivity.ActivityNewYearUI"
	else
		return "NewActivity.ActivityNewYearExchangeUI"
	end
end

function This:showTimer()
    return false
end