-- @Author: superm
-- @Date:   2017-12-01 14:43:29
-- @Last Modified by:   superm
-- @Last Modified time: 2018-07-26 16:35:01
-- @File Name: ActOnlineTimeBtn.lua
-- @Description: 

local This, super = class(..., "Home.MenuButton")
local ActivityManager = require("Activity.ActivityManager")
-- local ViewUtil = require("UI.ViewUtil")
local PopDialogManager = require("UI.PopDialogManager")

function This:init(config)
    super.init(self, config)
    self.m_queueId = config.uiclass.id
    self.m_levelUnlocked = config.uiclass.unlocked
    self.m_queue = ActivityManager:getQueueById(self.m_queueId)
    self.lingqu:setVisible(true)
    self.time:setVisible(false)
end

function This:addCustomListener()
    if self.m_queue then
        local activity = self.m_queue:getObjectBy(function(act)
            return not act:isCompleted()
        end)
        
        if activity then
            self.m_queue:setView(self)
            self.m_queue:getAllObjectState(true)
        end
    end
    ActivityManager:addObserver(self)
end


function This:removeCustomListener()
    if self.m_queue then
        self.m_queue:setView(nil)
    end
    ActivityManager:removeObserver(self)
end

function This:initTouchEvent()
    if not self.m_inited then
        self:setTouchEnded(function()
            self:onButtonClick()
        end)
        self.m_inited = true    
    end
end

function This:refresh()
	local curNum = self.m_queue:getCurDescNum()
	if curNum then
        self.time:setVisible(true)
        self.lingqu:setVisible(false)
        self.time:setString(curNum)
        if self.m_subView then
            self.m_subView:refresh()
        end
    else
        self.lingqu:setVisible(true)
        self.time:setVisible(false)
        self.m_queue:setView(nil)
        if self.m_subView then
            self.m_subView:refreshBtn()
        end
	end
end

function This:onButtonClick()
    local visible = self.time:isVisible()
    if not visible and self.m_queue then
        if self.m_levelUnlocked then
            self.m_queue:requestPickUp()
            return
        end
    end

    return self:openActivityPanel( not self.m_levelUnlocked and not visible )
end

function This:openActivityPanel(value, delay)
    if not self.m_subView then
        local uiName = "Activity.Activity" .. self.m_queue:getPanelName() .. "Panel"
        if not delay then
            local panel = g_scene:openDialog(uiName, {queue = self.m_queue, param = value})
            self.m_subView = panel
            panel:addCloseCallback(function ()
                self.m_subView = nil
            end)
            return panel
        else
            PopDialogManager:addPopDialog({
                bCheck = true,
                name = uiName,
                level = DialogLevel.kForLevel,
                func = function()
                    local panel = g_scene:openDialog(uiName, {queue = self.m_queue, param = value})
                    self.m_subView = panel
                    panel:addCloseCallback(function ()
                        self.m_subView = nil
                    end)
                    return panel
                end
            })
        end
    end
end

function This:onPickupActReward(actQueueId, id, act)
	if self.m_queueId ~= actQueueId then
		return
	end

    if self.m_subView then
        self.m_subView:close()
    end

    self.m_queue:setView(self)
    self.m_queue:getAllObjectState(true)
	if act then
		local itemList = act:getRewardList(1)
		if itemList and #itemList > 0 then
			g_scene:showReward(itemList)
		end
	end
    self:refresh()
end

function This:onGetActivityInfo( actQueueId, actType )
    if actQueueId == self.m_queueId then
        if not self.m_levelUnlocked then
            local activity = self.m_queue:getObjectBy(function(act)
                return not act:isCompleted()
            end)

            if activity then
                self:setNodeVisible(true)
                self:setVisible(true)
                self.m_queue:setView(self)
                self.m_queue:getAllObjectState(true)
            end
        end
    end
end

function This:getDefaultVisible()
    if not self.m_queue or not self.m_queue:isInited() then
        return false
    end

    return super.getDefaultVisible(self)
end

function This:onCreateActivity( actQueueId )
    if actQueueId and actQueueId > 0 and self.m_queueId == actQueueId then
        self.m_queue = ActivityManager:getQueueById(self.m_queueId)
        if self.m_queue then
            if self.m_levelUnlocked then
                self:openActivityPanel(true, true)
            end
            local activity = self.m_queue:getObjectBy(function(act)
                return not act:isCompleted()
            end)
            
            if activity then
                self.m_queue:setView(self)
                self.m_queue:getAllObjectState(true)
            end
        end
    end
end
