-- @Author: superm
-- @Date:   2017-10-31 14:29:56
-- @Last Modified by:   superm
-- @Last Modified time: 2019-09-26 20:13:21
-- @File Name: ActOnlineTimeQueueModel.lua
-- @Description: 在线游戏

local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ActivityManager = require("Activity.ActivityManager")

local This , super = class(..., "Activity.ActivityQueueBaseModel")
local ActOnlineTimeModel = class( This._NAME .. ".ActOnlineTimeModel", "Activity.ActivityBaseModel")

function ActOnlineTimeModel:getCondition()
    return self.m_config.online_time
end

function ActOnlineTimeModel:getLimitNum()
    return tostring(math.floor(self:getCondition() / SECONDS_PER_MIN)) .. Serialize[60259]
end

function ActOnlineTimeModel:getCurDescNum()
    local curNum = self:getQueue():getCurNum()
    local limitNum = self:getCondition()
    if limitNum > curNum then
        return GameUtil:formatTime(Serialize["H_M_S"], limitNum - curNum)
    end
end

function ActOnlineTimeModel:updateState(isInit, info)
    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end

    local ret = false
    local queue = self:getQueue()
    if queue then
        ret = (queue:getCurNum() or 0) >= self:getCondition()
    end
    self.m_state = ret and ActObjectState.kNormal or ActObjectState.kLocked
end

function ActOnlineTimeModel:updateStateAndNotify(queue)
    if self:isLocked() and self:getCondition() <= queue:getCurNum() then
        self:updateState()
        queue:getAllObjectState(true)
        ActivityManager:notifyActState(queue, true)
    end 
end

function This:initBaseData( info )
    super.initBaseData( self, info )
    self.m_allObejectState = nil
end

function This:initBeforeUpdateState( info, createHandler )
    self.m_selfEscape = 0
    if not createHandler then
        createHandler = function( actQueueId, activityIndex, actCfg, queue )
            local activity = ActOnlineTimeModel:create({type = actQueueId, id = activityIndex, config = actCfg})
            activity:setQueue(queue)
            return activity
        end
    end

    super.initBeforeUpdateState( self, info, createHandler )
end

function This:setCurNum(info)
    self.m_curNum = info.online_seconds
    self.m_recordNum = GamePlayer:getServerTime()
    self.m_showObj = nil
    self.m_selfEscape = 0
    if not self.m_upIsUpdating then
        cc.Application:addToUpdateList(self)
    end
end

function This:update( dt )
    super.update(self, dt)
    self:updateTimer(dt)
end

function This:updateTimer( dt )
    self.m_selfEscape = self.m_selfEscape + dt
    if self.m_selfEscape >= 1 then
        self.m_selfEscape = self.m_selfEscape - 1
        self:updateCurNum(1)
        if self.m_view then
            self.m_view:refresh()
        end
    end
end

function This:updateCurNum(delta)
    if not self.m_curNum then
        return
    end

    if not delta then
        local curTime = GamePlayer:getServerTime()
        self.m_curNum = self.m_curNum + curTime - self.m_recordNum 
        self.m_recordNum = curTime
    else
        self.m_curNum = self.m_curNum + delta
        self.m_recordNum = self.m_recordNum + delta
        if self.m_showObj then
            self.m_showObj:updateStateAndNotify(self)
        end
    end
end

function This:updateObjectByTime()
    local queueId = self:getId()
    local ret = self:getObjectBy(function( obj )
        return self.m_curNum < obj:getCondition()
    end)

    if not ret then
        print("[Online] endUpdate")
        if not self.m_upIsUpdating then
            cc.Application:removeFromUpdateList(self)
        end

        if self.m_showObj then
            self.m_showObj:updateStateAndNotify(self)
        end
    end 
end

function This:getUITypeName()
    return "Activity.ActivityBtnView"
end

function This:getAllObjectState(update)
    if update then
        local record, backup
        self.m_allObejectState = nil
        for id, act in pairs(self.m_objects) do
            if act:isOpening() then
                record = act
                self.m_allObejectState = true
                break
            elseif not backup and act:isLocked() then
                backup = act
            end
        end
        if self.m_curNum then
            self.m_showObj = record or backup
            self:updateObjectByTime()
        end

        if not record and not backup then
            self.m_allObejectState = false
            if self.m_view then
                self.m_view:setVisible(false)
                self.m_view = nil
            end
        end
    end
    return self.m_allObejectState
end

function This:getCurDescNum()
    if self.m_showObj then
        return self.m_showObj:getCurDescNum()
    end
    return
end

function This:requestPickUp()
    if self.m_showObj then
        return self.m_showObj:requestPickUp()
    end
    return
end

function This:getCurRewardList()
    if self.m_showObj then
        return self.m_showObj:getRewardList()
    end
end

function This:getCurObj()
    return self.m_showObj
end
