-- @Author: superm
-- @Date:   2018-01-19 10:21:05
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:21:19
-- @File Name: ActOpenTaskExchangeBtn.lua
-- @Description: 对话按妞

local This , super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/ActivitySevenDaysItem.csb"

local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local ActivityManager = require("Activity.ActivityManager")

function This:initData(model)
    self.m_inited = false
    self.m_exchangeModel = model
    local w , h = self.bg:getContentSize()
    self:setContentSize(w,h)
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    ActivityManager:addObserver(self)
end

function This:onExit()
    ActivityManager:removeObserver(self)
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:initNode()
    if not self.m_clippingNode then
        local clippingNode = cc.ClippingNode:create(self.iconbg)
        clippingNode:setAlphaThreshold(self.iconbg)
        self.iconbg:switchParent(clippingNode)
        self.icon:switchParent(clippingNode)
        self.m_clippingNode = clippingNode
        self:addChild(clippingNode)
        self.light:setLocalZOrder(10)
    end

    self:initItemUI()
    self:setItemEnable(not self.m_exchangeModel:isCompleted())
end

function This:initTouchEvent()
    if not self.m_inited then
        self.exchange:setTouch({
            getRect = function (touch)
                local rect = touch.sizeNode:getBoundingBox()
                return rect
            end,
		    onEnded = function()
                if self.m_touch then
                    self.m_touch.onEnded()
                end
                self:onExchangeBtnClick()
            end,
        })    
        self.m_inited = true
    end
end

function This:initItemUI()
    if not self.m_rewardItemList then
        self.m_rewardItemList = self.m_exchangeModel:getRewardList(1)
        self.m_needItemList = self.m_exchangeModel:getNeedItemList(1)
    end

    local _, rewardItemInfo = next(self.m_rewardItemList)
    if rewardItemInfo.id then
        self.icon:setImage(ItemUtil.getIconImage(rewardItemInfo.id))
        self.title:setString(ItemUtil.getName(rewardItemInfo.id))
        self.title:setColorByID("Q".. ItemUtil.getQuality(rewardItemInfo.id))
        self.shuliang:setString(tostring(rewardItemInfo.count or 0))
    end
    self.day:setString(Serialize[60247] .. string.numberToWord(self.m_exchangeModel:getConditionOpenDay()) .. Serialize[60011], 32)

    self:refreshItemUI()
end

function This:refresh()
    self:refreshItemUI()
    self:setItemEnable(not self.m_exchangeModel:isCompleted())
end

function This:refreshItemUI()
    local _, needItemInfo = next(self.m_needItemList)
    if needItemInfo.id then
        local curNum = DepotManager:getItemCount(needItemInfo.id)
        local enough = curNum >= needItemInfo.count
        local format = "%s ( " ..  (enough and "<A0>" or "<R0>") .. "%s<A0>/%d )"
        self.num:setStringWithColor(string.format(format, ItemUtil.getName(needItemInfo.id), ItemUtil.formatItemCount(curNum), needItemInfo.count))
        return enough
    end
end

function This:setItemEnable(enble)
    if self:isLocked() then
        self.exchange:setVisible(false)
        self.num:setVisible(false)
    else
        self.exchange:setVisible(true)
        if enble then
            self.exchange:clearShader()
            self.exchange.text:setString(Serialize["activity_OpenTask_1"])
        else
            self.exchange:setGrayShader()
            self.exchange.text:setString(Serialize["activity_OpenTask_2"])
        end
    end
end

function This:onExchangeBtnClick()
    local _, needItemInfo = next(self.m_needItemList)
    if needItemInfo.id then
        local curNum = DepotManager:getItemCount(needItemInfo.id)
        local enough = curNum >= needItemInfo.count
        if enough then
            self.m_exchangeModel:requestExchange()
        else
            g_scene:showTips(Serialize["activity_AssassinsPanel_1_c"])
        end
    end
end

function This:isLocked()
    return not self.m_exchangeModel:isValid()
end

function This:showLockWarming(dialog)
    -- g_scene:showTips(Serialize[60261])
    local day = self.m_exchangeModel:getConditionOpenDay()
    local queue = self.m_exchangeModel:getQueue()
    local reward = queue:getGroupRewardListBy(day)
    dialog:openDialog("Activity.ActivityPreviewPanel", {reward=reward, title = Serialize["activity_OpenTask_3"]})
end

function This:itemUpdate(itemList)
    if itemList and #itemList then
        self:refreshItemUI()
    end
end

function This:onExchangeReward(queueId, id, act)
    local ret = false
    if act then
        ret = (act == self.m_exchangeModel)
    else 
        ret = (id == self.m_exchangeModel:getId() and queueId == self:getQueueType())
    end

    if ret then
        local itemList = self.m_exchangeModel:getRewardList(1)
        if itemList and #itemList > 0 then
            g_scene:showReward(itemList)
        end
        self:refresh()
    end
end