-- @Author: superm
-- @Date:   2017-11-04 18:21:56
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:20:46
-- @File Name: ActOpenTaskQueueModel.lua
-- @Description: 开服7天活动

local GamePlayer = require("Player.GamePlayer")
local This , super = class(..., "Activity.ActivityQueueBaseModel")
local ActOpenTaskModel = class( This._NAME .. ".ActOpenTaskModel", "Activity.ActivityBaseModel")
local ActOpenTaskExchangeModel = class( This._NAME .. ".ActOpenTaskExchangeModel", "Activity.ActivityExchangeModel")

local GameUtil = require("Util.GameUtil")
local ViewUtil = require("UI.ViewUtil")
local GuideManager = require("Guide.GuideManager")
local ActivityUtil = require("Activity.ActivityUtil")

function ActOpenTaskModel:getCondition()
    return self.m_config.progress
end

function ActOpenTaskModel:getLimitNum()
    return tostring(self:getCondition())
end

function ActOpenTaskModel:getCurDescNum()
    local curNum = self:getQueue():getCurNum(self:getId())
    local limitNum = self:getCondition()
    local x
    if limitNum <= curNum then
        x = limitNum
    else
        x = curNum
    end
    return string.format("%d", x)
end

function ActOpenTaskModel:getWholeDescNum()
    local curNum = self:getQueue():getCurNum(self:getId())
    local limitNum = self:getCondition()
    local x
    if limitNum <= curNum then
        x = limitNum
    else
        x = curNum
    end
    return string.format("(%d/%d)", x, limitNum)
end

function ActOpenTaskModel:getQueueCurNum()
    local curNum = self:getQueue():getCurNum(self:getId())
    local limitNum = self:getCondition()
    local x
    if limitNum <= curNum then
        return limitNum
    else
        return curNum
    end
end

function ActOpenTaskModel:isValid()
    if not self:isSpecial() then
        local day = self:getQueue():getActivityCreateDays()
        return self.m_config.open_num <= day 
    end
end

function ActOpenTaskModel:isSpecial()
    local id = self:getId()
	return ActivityUtil.isSpecialAchieve(id)
end

function ActOpenTaskModel:updateState(isInit, info)
    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end

    local ret = false
    local queue = self:getQueue()
    if queue then
        ret = queue:getCurNum(self:getId()) >= self:getCondition()
    end
    self.m_state = ret and ActObjectState.kNormal or ActObjectState.kLocked
end

function ActOpenTaskModel:hasGuider()
    if self.m_config.guide_data and self.m_config.guide_data ~= "" then
        return true
    end

    if self.m_config.panel and self.m_config.panel ~= "" then
        return true
    end
    return false
end

function ActOpenTaskModel:hasGotoPanel()
    if self.m_config.panel and self.m_config.panel ~= "" then
        return true
    end
    return false
end

function ActOpenTaskModel:getGotoPanel()
    return self.m_config.panel
end

function ActOpenTaskModel:getGuideSession()
    return self.m_config.guide
end

function ActOpenTaskModel:getWarming()
    return self.m_config.warming
end

function ActOpenTaskModel:getConditionOpenDay()
    return self.m_config.open_num  
end

function ActOpenTaskModel:getProgressType()
    return self.m_config.progress_type  
end

function ActOpenTaskModel:triggerAncGuider()
    if self.m_config.guide_data and self.m_config.guide_data ~= "" then
        GuideManager:triggerAncGuider(self.m_config.guide_data)
    elseif self.m_config.panel then
        local panel = self.m_config.panel
        ViewUtil:jumpToPanel(panel.name,panel.arg)
    end
end

------------------------------------------------------------------------
function ActOpenTaskExchangeModel:getCondition()
    if self.m_exchangeConfig then
        local ret = {}
        if self.m_config.open_num > 0 then
            table.insert(ret, {6, self.m_config.open_num})
        end

        for i,v in ipairs(self.m_exchangeConfig.need_item) do
            table.insert(ret, {5, v})
        end
        return ret
    end
end

function ActOpenTaskExchangeModel:getConditionOpenDay()
    return self.m_config.open_num  
end

function ActOpenTaskExchangeModel:isValid()
    local queue = self:getQueue()
    if queue then
        local day = queue:getActivityCreateDays()
        return self.m_config.open_num <= day
    end
end

------------------------------------------------------------------------
function This:getExchangeCreator()
    return ActOpenTaskExchangeModel
end

function This:getTaskCreator()
    return ActOpenTaskModel  
end

function This:getConfigName(exchange)
    return self:getTypeName()
end

function This:initBeforeUpdateState( info, createHandler )
    if not createHandler then
        createHandler = function( actQueueId, activityIndex, actCfg, queue )
            local creator = self:getTaskCreator()
            local activity = creator:create({type = actQueueId, id = activityIndex, config = actCfg})
            activity:setQueue(queue)
            return activity
        end
    end

    if info.initing and createHandler then
        self.m_objects = {}
        self.m_eobjects = {}
        local queueId = self:getId()
        local originLevel = self:getOriginalLevel()
        local typeName = self:getConfigName()
        local config = require("Metadatas." .. typeName .. "Config")
        local actConfigList = {}
        for cfgIndex, cfg in pairs(config) do
            if self:isSuitable(cfg, queueId, originLevel) then
                local act = createHandler(queueId, cfgIndex, cfg, self)
                self.m_objects[act:getId()] = act
                table.insert(actConfigList, cfg)
            end
        end

        local otherTypeName = self:getConfigName(true)
        self:initExchangeObjList(otherTypeName)

        local finishHandler = self.initGroupsInfo
        if self:hasGroupsBehaviour() and finishHandler and actConfigList then
            finishHandler(self, actConfigList)
        end

        if self:hasRankBehaviour() then
            self:initRankObjList(otherTypeName)
        end
    end
end

function This:selfUpdateInfo(info, createHandler)
    -- print_tmp(info)
    if info.task_list then
        for _, act in pairs(self.m_objects) do
            act:setState(ActObjectState.kLocked)
        end
    end

    if info.complete_list then 
        local data = info.complete_list
        if info.reward_list then
            local t1, t2 = GameUtil:getValueMap(info.complete_list), GameUtil:getValueMap(info.reward_list)
            local data = GameUtil:diffByKey(t1, t2)
            for activityId, _ in pairs(data) do
                local act = self.m_objects[activityId]
                if act then
                    act:setOpening()
                    if act:isSpecial() then
                        self:showRookieActivity(act)
                    end
                end
            end
        else
            for _, activityId in ipairs(info.complete_list) do
                local act = self.m_objects[activityId]
                if act then
                    act:setOpening()
                    if act:isSpecial() then
                        self:showRookieActivity(act)
                    end
                end
            end
        end
    end

    if info.progress_list then
        for _, info in ipairs(info.progress_list) do
            local act = self.m_objects[info.id]
            if act then
                act:setState(info.state - 1)
                if act:isSpecial() then
                    self:showRookieActivity(act)
                end
            end
        end         
    end

    if info.reward_list then
        for _, activityId in ipairs(info.reward_list) do
            local act = self.m_objects[activityId]
            if act then
                act:setCompleted()
            end
        end
    end

    self:selfUpdateExchangeInfo(info)
end

function This:selfUpdateExchangeInfo(info)
    if info.exchange_list then
        for _, act in pairs(self.m_eobjects) do
            if act then
                act:updateState()
            end
        end

        for _, exchangeInfo in ipairs(info.exchange_list) do

            local act = self.m_eobjects[exchangeInfo.id]
            if act then
                act:setCompleteTimes(exchangeInfo.num)
            end
        end
    end
end

function This:showRookieActivity(act)
    if GuideManager:isInGuide() and GuideManager:getGuideStep() <= kActRookieOpenGuideId then
        if GamePlayer:getLevel() <= kActRookieOpenLevel  then
            return
        end
    end

    local delayMode = true
    local sceneType = g_scene:getType()
    if sceneType == SceneType.kWorldScene or sceneType == SceneType.kHomeScene then
        delayMode = not g_scene:isInit()
    end

    if not delayMode then
        g_scene:openDialog("Activity.ActivityRookiePanel", act)
    else
        g_scene:addDelayDialog({name = "Activity.ActivityRookiePanel", arg = act})
    end
end

function This:isAllObjectAutoUpdate()
    return false
end

function This:hasExchangeBehaviour()
    return true
end

function This:showExchangeBehaviour()
    return true
end

function This:getOriginalLevel()
    return self.m_originalLevel
end

function This:setOriginalLevel(value)
    self.m_originalLevel = value
end

function This:hasGroupsBehaviour()
    return true
end

function This:initGroupsInfo( config )
    table.sort(config, function(cfgA, cfgB)
		if cfgA.open_num ~= cfgB.open_num then
		    return cfgA.open_num < cfgB.open_num 
		end
		return cfgA.id < cfgB.id
    end)
    self:getGroupNameList( config, "open_num", function ( value )
        return string.format(Serialize["activity_ActOpen_1_c"], value) 
    end )
end

function This:getDefaultGroup()
    -- print( "开服活动:" .. tostring(math.min(self:getActivityCreateDays(), self:getGroupListCount())))
    return tostring(math.min(self:getActivityCreateDays(), self:getGroupListCount()))
end

function This:setCurNum(info)
    if info.task_list then
        self.m_curNumArr = {}
        for i, actInfo in ipairs(info.task_list) do
            self.m_curNumArr[ actInfo.id ] = actInfo.num
        end
    end

    if info.progress_list then
        if not self.m_curNumArr then
            self.m_curNumArr = {}
        end
        for i, actInfo in ipairs(info.progress_list) do
            self.m_curNumArr[ actInfo.id ] = actInfo.num
        end
    end
end

function This:getCurNum(index)
    if self.m_curNumArr then
        return self.m_curNumArr[index or 1] or 0
    end
    return 0
end

function This:setHightestNum(info)
    if info.id and info.name then
        if not self.m_hightestNumArr then
            self.m_hightestNumArr = {}
        end
        self.m_hightestNumArr[ info.id ] = info
    end
end

function This:getHightestNum(id)
    if self.m_hightestNumArr then
        return self.m_hightestNumArr[id]
    end
end

function This:getUITypeName()
    if self:getGroupListCount() > 1 then
        return "Activity.ActivityBaseTabsUI"
    end
    return "Activity.ActivityBaseUI"
end

function This:getCSFileName()
    if self.m_config.panel then
        return "MenuScene/NodeUI/ActivitySevenDays.csb"
    end
end

function This:getAllObjectState(update)
    if update then
        local record, backup
        self.m_allObejectState = nil
        for id, act in pairs(self.m_objects) do
            if act:isOpening() and act:isValid() then
                record = act
                self.m_allObejectState = true
                break
            elseif not backup and (not act:isValid() or act:isLocked()) then
                backup = act
            end
        end

        if not record and not backup then
            self.m_allObejectState = false
        end
    end
    return self.m_allObejectState  
end

function This:getObjectStateByGroupIndex( num, exchange  )
    local ret = false
    local objects = self:getGroupListBy(num, exchange )
    for _, act in ipairs(objects) do
        if act:isOpening() and act:isValid() then
            ret = true
            break
        end
    end
    return ret  
end

function This:getObjectStateByRuleIndex( index )
    if not index or not self.m_uiconfig then
        return self:getAllObjectState()
    end

    local filterIdsList = self:getFilterIdsList(index)
    if not filterIdsList or #filterIdsList <= 0 then
        return self:getAllObjectState()
    end

    local ret = table.find(self.m_objects, function(id, activity)
        if not activity:isOpening() or not activity:isValid() then
            return false
        end

        local fid = activity:getProgressType()
        if not fid then
            return false
        end
        return table.find(filterIdsList, function(_, v) return fid == v end)
    end)
    
    return ret 
end

function This:showTimer()
    return true
end

function This:getCostNum()
    local maxNum = 0
    if self.m_curNumArr then
        for taskId, num in pairs(self.m_curNumArr) do
            if self.m_objects[taskId] and maxNum < num then
                maxNum = num
            end
        end
    end
    return maxNum 
end
This.getPayNum = This.getCostNum