-- @Author: superm
-- @Date:   2019-03-01 17:11:17
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-03 05:40:26
-- @File Name: ActSecondWeekQueueModel.lua
-- @Description: 

local This, super = class(..., "Activity.ActOpenTaskQueueModel")
local ActSecondRechargeModel = class( This._NAME .. ".ActSecondRechargeModel", super._NAME .. ".ActOpenTaskModel")
local ActSecondExchangeModel = class( This._NAME .. ".ActSecondExchangeModel", super._NAME ..".ActOpenTaskExchangeModel")
local DepotManager = require("Depot.DepotManager")
local ActivityManager = require("Activity.ActivityManager")
local ItemUtil = require("Bag.ItemUtil")

ActSecondWeek = {
	Task = 1,
	Recharge = 2,
	Exchange = 3,
	Rank = 4,
}

kSecondWeekKeyItemId1 = 277
kSecondWeekKeyItemId2 = 278


function ActSecondRechargeModel:getCondition()
    if self.m_config then
        local ret = {}
        if self.m_config.happy_count > 0 then
            table.insert(ret, {5, {kSecondWeekKeyItemId1, self.m_config.happy_count}})
        end

        table.insert(ret, {2, self.m_config.rmb_count})
        return ret
    end
end

function ActSecondRechargeModel:getLimitNum()
    return self.m_config.rmb_count
end

function ActSecondRechargeModel:getCurDescNum()
    local curNum = self:getQueue():getCurNum() 
    local limitNum = self:getLimitNum()
    if limitNum <= curNum then
        return tostring(limitNum)
    else
        return tostring(curNum)
    end
end

function ActSecondRechargeModel:getWholeDescNum()
    return string.format(Serialize[60264], self.m_config.rmb_count)
end

function ActSecondRechargeModel:updateState(isInit, info)
    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end

    local conds = self:getCondition()
    if not conds or conds == 0 then
        self.m_state = ActObjectState.kNormal
    else
        local ret = true
        if not isNestedObject(conds) then
            if #conds >= 2 then
                ret = self:isNotLockedBy(conds[1], conds[2], 1)
            end
        else
            for i, cond in ipairs(conds) do
                ret = ret and self:isNotLockedBy(cond[1], cond[2], i)
                if not ret then
                    self.m_state = ActObjectState.kLocked
                    break
                end
            end
        end
        self.m_state = ret and ActObjectState.kNormal or ActObjectState.kLocked
    end
end

function ActSecondRechargeModel:isValid()
    local happyCount = DepotManager:getItemOrResCount(kSecondWeekKeyItemId1)
    return self.m_config.happy_count <= happyCount
end

function ActSecondRechargeModel:getDesc()
    if not os.isWin32 then
        return string.format(Serialize[60265], ItemUtil.getName(kSecondWeekKeyItemId1), self.m_config.happy_count)
    else
        return tostring(self:getId()) .. ":" .. string.format(Serialize[60265], ItemUtil.getName(kSecondWeekKeyItemId1), self.m_config.happy_count)
    end
end

function ActSecondRechargeModel:requestPickUp()
    if self:isOpening() then
        ActivityManager:onPickupNewActRewardRequest(self:getQueueConfigSubCmd(), self:getType(), self:getId())
        return true
    end
    return false
end

------------------------------------------------------------------------------
function ActSecondExchangeModel:getRewardList(dataType)
	local config = self.m_config.reward
    if dataType then
        if dataType == 1 then
            local t = {}
            for i, itemInfo in ipairs(config) do
                table.insert(t, {id = itemInfo[1], count = itemInfo[2]})
            end
            return t
        end
    end
    return config
end

function ActSecondExchangeModel:getNeedItemList(dataType)
	local config = self.m_config.cost
    if dataType then
        if dataType == 1 then
            local t = {}
            for i, itemInfo in ipairs(config) do
                table.insert(t, {id = itemInfo[1], count = itemInfo[2]})
            end
            return t
        end
    end
    return config
end

function ActSecondExchangeModel:getCondition()
    if self.m_config then
        local ret = {}
        if self.m_config.happy_count > 0 then
            table.insert(ret, {5, {kSecondWeekKeyItemId1, self.m_config.happy_count}})
        end

        for _, v in ipairs(self.m_config.cost) do
            table.insert(ret, {5, v})
        end
        return ret
    end
end

function ActSecondExchangeModel:isValid()
    local happyCount = DepotManager:getItemOrResCount(kSecondWeekKeyItemId1)
    return self.m_config.happy_count <= happyCount
end

function ActSecondExchangeModel:getDesc()
    if not os.isWin32 then
        return string.format(Serialize[60266], ItemUtil.getName(kSecondWeekKeyItemId1), self.m_config.happy_count)
    else
        return tostring(self:getId()) .. ":" .. string.format(Serialize[60266], ItemUtil.getName(kSecondWeekKeyItemId1), self.m_config.happy_count)
    end
end

function ActSecondExchangeModel:getWholeDescNum()
    local curNum = self:getCurNum()
    local limitNum = self:getLimitNum()
    local x
    if limitNum <= curNum then
        x = 0
    else
        x = limitNum - curNum
    end
    return string.format("%d/%d", x, limitNum)
end

function ActSecondExchangeModel:getTotalCompleteTimes()
    if self.m_config then
        return self.m_config.times_limit or 1
    end
    return 0
end

-------------------------------------------------------------------------
function This:setCurNum(info)
    self.m_curNum = info.recharge_price or 0
    self.m_curRank = info.happy_rank or 0
end

function This:getCurRank()
    return self.m_curRank
end

function This:getCurNum(index)
    if self.m_curNumArr then
        return self.m_curNumArr[index or 1] or 0
    end
    return self.m_curNum
end

function This:getExchangeCreator()
    return ActSecondExchangeModel
end

function This:getTaskCreator()
    return ActSecondRechargeModel  
end

function This:getConfigName(exchange)
	local ret = self:getTypeName()
	if not exchange then
    	ret = ret .. "Receive" 
    end
	return ret
end

function This:hasGroupsBehaviour()
    return false
end

function This:selfUpdateExchangeInfo(info)
    -- print_tmp(info)
    if info.exchange_id_list and info.exchange_count_list then
        for _, act in pairs(self.m_eobjects) do
            if act then
                act:updateState()
            end
        end

        local numList = info.exchange_count_list
        for index, exchangeId in ipairs(info.exchange_id_list) do
            local act = self.m_eobjects[exchangeId]
            if act then
                -- print(exchangeId, numList[index])
                act:setCompleteTimes(numList[index])
            end
        end
    end
end

function This:getCSFileName(indice)
	return string.format("MenuScene/NodeUI/PlayerCompeteNode%d.csb", indice)
end


function This:getUITypeName(index)
    if self.m_uiconfig and self.m_uiconfig[index] then
        local name = self.m_uiconfig[index].source
        if name then
            return name
        end
    end
    return "Activity.ActivitySecondWeekBaseUI"
end

function This:isAllObjectAutoUpdate()
    return true
end

function This:hasExchangeBehaviour()
    return true
end

function This:hasRankBehaviour()
    return true
end