-- @Author: superm
-- @Date:   2019-03-02 11:43:42
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:19:47
-- @File Name: ActSecondWeekTaskQueueModel.lua
-- @Description: 次周活动模型

local This , super = class(..., "Activity.ActOpenTaskQueueModel")
local ActSecondWeekTaskModel = class( This._NAME .. ".ActSecondWeekTaskModel", super._NAME .. ".ActOpenTaskModel")
local SecondWeekDayRankingConfig = require("Metadatas.SecondWeekDayRankingConfig")
local ActivityManager = require("Activity.ActivityManager")
local MatchingConfig = require("Metadatas.MatchingConfig")

function ActSecondWeekTaskModel:requestRefresh()
    ActivityManager:onGetActivityHighestInfoRequest(self:getId())
end

function ActSecondWeekTaskModel:getHighestDesc()
	local info = self:getQueue():getHightestNum(self:getId())
	if not info then
		return
	end

	local name, value = info.name, info.value
    if self.m_config.day == 7 and name and value then
    	local config = MatchingConfig[value]
    	if config then
    		return name, config.rank_level
    	end
    end
    return name, value
end

function This:getTypeName()
    return "OpenTask"
end

function This:getUITypeName()
    if self:getGroupListCount() > 1 then
        return "Activity.ActivitySecondWeekTabsUI"
    end
    return "Activity.ActivityBaseUI"
end

function This:getCSFileName()
    return "MenuScene/NodeUI/PlayerCompeteNode.csb"
end

function This:initGroupsInfo( config )
	self.m_groupCfgInfoList = {}
	for _, cfg in pairs(SecondWeekDayRankingConfig) do
		self.m_groupCfgInfoList[cfg.day] = cfg
	end

    table.sort(config, function(cfgA, cfgB)
		if cfgA.open_num ~= cfgB.open_num then
		    return cfgA.open_num < cfgB.open_num 
		end
		return cfgA.id < cfgB.id
    end)
    self:getGroupNameList( config, "open_num", function ( value )
        return self.m_groupCfgInfoList[value].title
    end)

    self:initHighestObjects()
end

function This:initHighestObjects()
    if not self.m_hobjects then
        local createHandler
	    if not createHandler then
	        createHandler = function( actQueueId, activityIndex, actCfg, queue )
	            local activity = ActSecondWeekTaskModel:create({type = actCfg.activity_id, id = actCfg.activity_id, config = actCfg})
	            activity:setQueue(queue)
	            return activity
	        end
	    end

	    if createHandler then
    		self.m_hobjects = {}
    		self.m_hobjectMap = {}
        	local queueId = self:getId()
	        for cfgIndex, cfg in pairs(self.m_groupCfgInfoList) do
	            local act = createHandler(queueId, cfgIndex, cfg, self)
	            local actId = act:getId()
	            self.m_hobjects[act:getId()] = act
	            self.m_hobjectMap[cfgIndex] = actId
	        end
	    end 	
    end
end

function This:getHighestObject(index)
	if self.m_hobjects and self.m_hobjectMap then
		local actId = self.m_hobjectMap[index]
		if actId then
    		return self.m_hobjects[actId]
    	end
    end
end

function This:getAllObjectState(update)
    if update then
    	local defaultIndex = math.min(self:getActivityCreateDays(), self:getGroupListCount())
        self.m_allObejectState = self:getObjectStateByGroupIndex(defaultIndex)
    end
    return self.m_allObejectState  
end