-- @Author: superm
-- @Date:   2018-01-23 16:19:10
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:02:28
-- @File Name: ActivityAssassinsPanel.lua
-- @Description: 铜雀台

local This, super = class(...,"Activity.ActivitySeparatedPanel")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local ActivityManager = require("Activity.ActivityManager")

function This:initTabsNode( groupCount )
	for i=1, 2 do
		local button = self.m_csNode["tab" .. i]
		self:addTabButton(tostring(i), button, function()
	    	if self.m_curTabIndex == i then
	    		return false
	    	end
	        self.m_curTabIndex = i
			self:refresh()
	        return true
	    end)

	    if not button.m_touch then
            button:setTouch({
	            getRect = function (touch)
	                local rect = touch.sizeNode.light:getBoundingBox()
	                return rect
	            end,
                onEnded = function ()
                    self:selectTabButton(tostring(i))
                end,
                isScale = false,
            })
        end
	end
end

function This:initTouchEvent()
    super.initTouchEvent(self)
    self.m_csNode.change.exchange:setTouchEnded(function()
		local _, needItemInfo = next(self.m_needItemList)
	    if needItemInfo.id then
	        local curNum = DepotManager:getItemCount(needItemInfo.id)
	        local enough = curNum >= needItemInfo.count
	        if enough then
	            self.m_exchangeModel:requestExchange()
	        else
	            g_scene:showTips(Serialize["activity_AssassinsPanel_1_c"])
	        end
	    end
	end)
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    ActivityManager:addObserver(self)
	self:selectTabButton("1")
end

function This:onExit()
    ActivityManager:removeObserver(self)
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:refreshItemUI()
    self.m_needItemList = self.m_exchangeModel:getNeedItemList(1)

    local _, needItemInfo = next(self.m_needItemList)
    if needItemInfo.id then
        self.m_csNode.change.icon:setImage(ItemUtil.getIconImage(needItemInfo.id))
        -- self.m_csNode.title:setString(ItemUtil.getName(needItemInfo.id))
        local curNum = DepotManager:getItemCount(needItemInfo.id)
        local enough = curNum >= needItemInfo.count
        local format = "" ..  (enough and "<A0>" or "<R0>") .. "%s<A0>/%d"
        self.m_csNode.change.num:setStringWithColor(string.format(format, ItemUtil.formatItemCount(curNum), needItemInfo.count))
        return enough
    end

end

function This:refresh()
	self.m_exchangeModel = self.m_queue:getEObjectBy(function ( obj )
		return obj:getId() == (3 - self.m_curTabIndex)
	end)
	
	for i=1,2 do
		self.m_csNode["info" .. i]:setVisible(self.m_curTabIndex == i)
	end

	local visible = not self.m_exchangeModel:isCompleted()
    self.m_csNode.change:setVisible(visible)
    self.m_csNode.convertibility:setVisible(not visible)

    if visible then
		self:refreshItemUI()
	end
end

function This:itemUpdate(itemList)
    if itemList and #itemList then
        self:refreshItemUI()
    end
end

function This:onExchangeReward(queueId, id, act)
    local ret = false
    if act then
        ret = (act == self.m_exchangeModel)
    else 
        ret = (id == self.m_exchangeModel:getId() and queueId == self:getQueueType())
    end

    if ret then
         self:refresh()
    end
end
