-- @Author: superm
-- @Date:   2018-01-16 15:41:28
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:02:55
-- @File Name: ActivityBaseBtn.lua
-- @Description: 单活动的入口按钮

local This, super = class(..., "Home.MenuButton")
local ActivityManager = require("Activity.ActivityManager")

function This:init(config)
    super.init(self, config)
    self.m_autoClose = config.uiclass.auto_close
    self.m_queueId = config.uiclass.id
    if self.m_queueId and self.m_queueId > 0 then
        self.m_queue = ActivityManager:getQueueById(self.m_queueId)
    end
end

function This:addCustomListener()
    super.addCustomListener(self)
    if self.m_autoClose then
        ActivityManager:addObserver(self)
        if self.m_queue then
            self.m_queue:addObserver(self)
        end
    end
end

function This:removeCustomListener()
    if self.m_queue then
        self.m_queue:removeObserver(self)
    end
    ActivityManager:removeObserver(self)
    super.removeCustomListener(self)
end

function This:initTouchEvent()
    self:setTouchEnded(function()
        local name = self.m_queue:getPanelName() or "Separated"
        local panel = g_scene:openDialog("Activity.Activity" .. name .. "Panel", {queue = self.m_queue})
        self.m_subView = panel
        panel:addCloseCallback(function ()
            self.m_subView = nil
        end)
    end) 
end

function This:close()
	if self.m_queue and self.m_queue:isInited() then
        if self.m_subView then
            self.m_subView:close()
        end
		ActivityManager:notifyActState(self.m_queue, false)
	end
end

function This:onQueueUpdateEnd( id )
    if self.m_autoClose and id == self.m_queueId then
    	self:close()
    end 
end

function This:getDefaultVisible()
	if not self.m_queue or not self.m_queue:isInited() then
		return false
	end

    return super.getDefaultVisible(self)
end

function This:onCreateActivity( id )
    if id and id > 0 and self.m_queueId == id then
        self.m_queue = ActivityManager:getQueueById(self.m_queueId)
        if self.m_autoClose and self.m_queue then
            self.m_queue:addObserver(self)
        end
    end
end