-- @Author: superm
-- @Date:   2017-10-09 17:53:42
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:03:30
-- @File Name: ActivityBaseModel.lua
-- @Description: 基础活动

local This , super = class(..., "Base.BaseModel")
local ActivityUtil = require("Activity.ActivityUtil")
local GamePlayer = require("Player.GamePlayer")
local ActivityManager = require("Activity.ActivityManager")
local DepotManager = require("Depot.DepotManager")

local function isTable( obj )
    if obj and type(obj) == "table" then
        return true
    end
    return false
end

function This:init( info )
    super.init(self)
    if info then
        self:initSelf( info )
        self:updateState(true, info)
    else
        self.m_ID = 0
        self.m_state = ActObjectState.kLocked
    end
end

function This:initId( info )
    self.m_type = info.type
    self.m_ID = info.id or 0
end

function This:initSelf( info )
    self:initId( info )
    self.m_state = ActObjectState.kLocked
    self.m_config = info.config
    self:updateState()
end

function This:updateState(isInit, info)
    local continue = not self:isCompleted()
    if isInit then
        self:initSelf(info)
        continue = true
    end

    if not continue then
        return
    end

    local conds = self:getCondition()
    if not conds or conds == 0 then
        self.m_state = ActObjectState.kNormal
    else
        local ret = true
        if not isNestedObject(conds) then
            if #conds >= 2 then
                ret = self:isNotLockedBy(conds[1], conds[2], 1)
            end
        else
            for i, cond in ipairs(conds) do
                ret = ret and self:isNotLockedBy(cond[1], cond[2], i)
                if not ret then
                    self.m_state = ActObjectState.kLocked
                    break
                end
            end
        end
        self.m_state = ret and ActObjectState.kNormal or ActObjectState.kLocked
    end
end

function This:isNotLockedBy( cond1, cond2, index )
    if not self.m_config then
        return false
    end

    local switch = {
        [1] = function(arg, index)  
            return self:getCurNum(index) >= arg
        end,  
        [2] = function(arg, index)  
            local queue = self:getQueue()
            if queue then
                local number = queue:getCurNum(index) or 0
                -- print("::  " .. index , number , arg )
                return number >= arg 
            else
                return false
            end
        end, 
        [3] = function(arg, index)  
            local queue = self:getQueue()
            if queue then
                local number = queue:getCurNum() or false
                return number == arg 
            else
                return false
            end
        end, 
        [4] = function(arg, index) 
            local hasTypeRefered = isTable(arg)
            local queue = self:getQueue()
            if queue and hasTypeRefered then
                local number = queue:getCurNum(arg[1]) or 0
                return number == arg[2] 
            elseif queue then
                local number = queue:getCurNum() or 0
                return number >= arg 
            else
                return false
            end
        end, 
        [5] = function( arg, index )
            if isTable(arg) then
                local curNum = DepotManager:getItemOrResCount(arg[1])
                return curNum >= arg[2]
            end
            return false
        end,
        [6] = function( arg, index )
            local queue = self:getQueue()
            if queue then
                local number = queue:getActivityCreateDays() or 0
                return number >= arg 
            else
                return false
            end
        end,
        [7] = function(arg, index)  
            local queue = self:getQueue()
            if queue then
                local number = queue:getCurNum(index)
                if number then
                    return number <= arg 
                end
            end
            return false
        end
    }  
      
    local fSwitch = switch[cond1] --switch func  
      
    if fSwitch then --key exists  
        local result = fSwitch(cond2, index) --do func  
        return result
    else
        return false
    end 
end

function This:getQueue()
    return self.m_queue
end

function This:getQueueType()
    if self.m_queue then
        return self.m_queue:getType()
    end
    return 0
end

function This:getQueueConfigType()
    if self.m_queue then
        return self.m_queue:getConfigType()
    end
    return 0
end

function This:setQueue( queue )
    self.m_queue = queue
end

function This:getCondition()
    return self.m_hasPay
end

function This:setId( id )
    self.m_ID = id
end

function This:getId()
    return self.m_ID
end

function This:getConfigType()
    return self.m_config.type
end

function This:getConfigId()
    return self.m_config.id
end

function This:getConfig()
    return self.m_config
end

function This:isSameType( info )
    return self.m_ID == info.id and self.m_type== info.type
end

function This:getType( )
    return self.m_type
end

function This:isLocked()
    return (self.m_state == ActObjectState.kLocked) or (not self.m_config)
end

function This:isOpening()
    return self.m_state == ActObjectState.kNormal
end

function This:isCompleted()
    return self.m_state == ActObjectState.kCompleted
end

function This:setCompleted()
    self.m_state = ActObjectState.kCompleted
end

function This:setCompleteTimes(times)
    self.m_completedTimes = times or 1
end

function This:setOpening()
    self.m_state = ActObjectState.kNormal
end

function This:isFinish()
    return self:getRewardNumber() >= self:getRewardLimitNum()
end

function This:getState()
    return self.m_state
end

function This:setState(value)
    self.m_state = value
end

function This:setRewardNumber(num)
    self.m_rewardNum = num
end

function This:increaseRewardNumber()
    self.m_rewardNum = (self.m_rewardNum or 0) + 1
end

function This:getRewardNumber()
    return self.m_rewardNum or 0
end

function This:setRewardIndex(index)
    self.m_rewardIndex = index
end

function This:getRewardIndex()
    return self.m_rewardIndex or 0
end

function This:finish()
    self.m_rewardNum = self:getRewardLimitNum()
end

function This:requestPickUp()
    if self:isOpening() then
        ActivityManager:onPickupActRewardRequest(self:getQueueConfigType(), self:getType(), self:getId())
        return true
    end
    return false
end

function This:getCurNum()
    return false
end

function This:getUpdateInterval()
    return nil
end

function This:setView(view)
    self.m_view = view
end

function This:getView()
    return self.m_view
end

--获取配置信息region begin
function This:getDesc()
    if not os.isWin32 then
        return self.m_config.desc
    else
        return tostring(self:getId()) .. ":" .. self.m_config.desc
    end
end

function This:getRewardLimitNum()
    return self.m_config.people
end

-- nil 原生 1奖励
function This:getRewardList(dataType)
    if dataType then
        if dataType == 1 then
            local t = {}
            for i, itemInfo in ipairs(self.m_config.reward) do
                table.insert(t, {id = itemInfo[1], count = itemInfo[2]})
            end
            return t
        end
    end
    return self.m_config.reward
end

function This:getQueueConfigSubCmd()
    if self.m_queue then
        return self.m_queue:getConfigSubCmd()
    end
    return 12004
end

function This:getQueueConfigOtherCmd()
    if self.m_queue then
        return self.m_queue:getConfigOtherCmd()
    end
    return 12014
end



--获取配置信息region end

--endregion