  -- @Author: superm
-- @Date:   2017-10-09 17:53:42
-- @Last Modified by:   superm
-- @Last Modified time: 2020-10-14 12:03:43
-- @File Name: ActivityBaseUI.lua
-- @Description: 基本的活动页

local This, super = class(..., "NodeUI")
This.csfileName = "MenuScene/NodeUI/ActivityBg.csb"

local ActivityManager = require("Activity.ActivityManager")
local ActivityItemView = require("Activity.ActivityItemView")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

function This:initData(activity, index)
	self.m_activity = activity
	if index and type(index) == "number" then
    	self.m_curSelection = index
    end
	self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
	local filename = self.m_activity:getCSFileName(self.m_curSelection)
    self.m_csNode = cc.Node:createFromCSFile(filename or self.csfileName)
    self:addChild(self.m_csNode)
end

function This:getAllItemViews()
	local cacheId = self.m_activity:getId() * 10000 + (self.m_curSelection or 1)
	return ActivityItemView.getAllViews(cacheId)
end

function This:initNode()
	self.m_curViews = self:getAllItemViews()

    local csNode = self.m_csNode
	if csNode.act_title and not self.m_activity:hasResTypeWith(ActResType.kNotShowTitle) then
	    csNode.act_title:setVisible(true)
		csNode.act_title:setImage("MenuScene/act_title_" .. tostring(self.m_activity:getTitleType()) .. ".jpg")
	end
	
	if csNode.chongzhi then
		if self.m_activity:isRechargeDirtyType() and not self.m_activity:hasResTypeWith(ActResType.kNotShowPay) then
			csNode.chongzhi:setVisible(true)
			csNode.bg1:setVisible(true)
			csNode.button:setVisible(true)
			if not self.m_activity:hasResTypeWith(ActResType.kNotShowPayDetail) then
				self:refreshPayNumNode()
			end
		elseif self.m_activity:hasResTypeWith(ActResType.kNotShowDesc) then
			csNode.chongzhi:setVisible(true)
			csNode.bg1:setVisible(true)
			csNode.button:setVisible(false)
			self:refreshCostDiamondNumNode()
		else
			csNode.chongzhi:setVisible(false)
			csNode.bg1:setVisible(false)
			csNode.button:setVisible(false)
		end
	end

	local hasTimer = self.m_activity:showTimer() 
	if csNode.time then 
		csNode.time:setVisible(hasTimer)
		csNode.timedi_7:setVisible(hasTimer)
		if hasTimer then
			csNode.time:setString(Serialize[60123] .. GameUtil:formatTime(Serialize["activity_BaseTabs_1_c"], self.m_activity:getLeftTime()))
		end
	end
    self:refreshListView()
end

function This:onEnter()
    super.onEnter(self)
    ActivityManager:addObserver(self)
    self.m_activity:setView(self)
	if self.m_activity:showTimer() then
		self.m_activity:addObserver(self)
	end 
    if self.m_activity:isTimeDirtyType() then
    	self.m_activity:updateCurNum()
		self:refreshInner()
    end
end

function This:onExit()
	self.m_activity:removeObserver(self)
	self.m_activity:setView()
    ActivityManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
	if not self.m_inited then
    	local csNode = self.m_csNode
		if csNode.button then
			csNode.button:setTouchEnded(function()
				g_scene:openDialog("System.RechargeMainUI")
			end)
		end
		self:initCustomTouchEvent(csNode)
	    self.m_inited = true
	end
end

function This:initCustomTouchEvent()
end

function This:refreshPayNumNode()
	local number = self.m_activity:getPayNum()
    if number then
		self.m_csNode.chongzhi:setString(Serialize[60194] .. Serialize[60195] .. number)
	    return number > 0
	end

    return false
end

function This:refreshCostDiamondNumNode()
	local number = self.m_activity:getCostNum()
    if number then
	    self.m_csNode.chongzhi:setString(Serialize[60196] .. number .. Serialize[60197])
	    return number > 0
	end
    return false
end

function This:initTimeNode()
	local startTime = self.m_activity:getStartTime()
	local startStr = GameUtil:formatDate(Serialize["activity_BaseUI_1_c"],startTime)	
	local endTime = self.m_activity:getEndTime()
	local endStr = GameUtil:formatDate(Serialize["activity_BaseUI_1_c"],endTime)

	self.m_csNode.time:setString(string.format("%s ~ %s",startStr,endStr))
end

function This:refreshListView()
    self:refreshItemList()
end

function This:refreshItemList(index)
	local activitys = GameUtil:shallowChange(self.m_activity:getAllObjects())

	local hasShowLimit = self.m_activity:hasShowLimit()
	if hasShowLimit then
		local activityList = {}
		for _, activity in ipairs(activitys) do
			local state = activity:getState()
			local ret = activityList[state]
			if not ret then
				ret = {}
				activityList[state] = ret
			end
			table.insert(ret, activity)
		end

		local limitType, limitNum = ActObjectState.kLocked, hasShowLimit
		local ret = activityList[limitType] or {}
		table.sort(ret, function( a, b )
			return a:getId() < b:getId()
		end)
		local newList = {}
		for i, v in ipairs( ret ) do
			if i > limitNum then
				break
			end
			table.insert(newList, v)
		end
		activityList[limitType] = newList

		local newActivitys = {}
		for _, vList in pairs( activityList ) do
			for __, v in ipairs( vList ) do
				table.insert(newActivitys, v)
			end
		end
		activitys = newActivitys
    end

    self:refreshItemListByData(activitys)
end

function This:getNewItemView( ... )
    return ActivityItemView:create( ... )
end

function This:refreshItemListByData( activitys )
	local csNode = self.m_csNode
	if not csNode or not csNode.scrollView then
		return
	end

    csNode.scrollView:removeAllItems()
    csNode.scrollView:setTouchEnabled(true)
	table.sort(activitys, function( a, b )
        local state1, state2 = a:getState() , b:getState()
        if state1 ~= state2 then
            return state1 < state2
        else
            return a:getId() < b:getId()
        end
	end)

	for index, activity in ipairs(activitys) do
        local id = activity:getId()
        local view = self.m_curViews[id]
        if not view then
			view = self:getNewItemView(activity, self.m_curSelection)
			self.m_curViews[id] = view
			view:retain()
		else
			view:refresh()
		end
	    local w , h = view:getContentSize()
	    csNode.scrollView:pushNode(view, w, h)
	end  
end

function This:refresh()
	if self.m_inited then
		self:refreshListView()
		if self.m_csNode.chongzhi then
			local visible = self.m_activity:isRechargeDirtyType()
			if visible then
	        	self:refreshPayNumNode()
	        end
	    end
	end
end

function This:refreshInner()
	if self.m_inited then
		local itemViewList = self.m_csNode.scrollView.m_itemList
		for _, view in ipairs(itemViewList) do
			view:refresh()
		end
	end 
end

function This:onPickupActReward(actQueueId, id, act)
	if self.m_activity:getId() ~= actQueueId then
		return
	end

	local view = self.m_curViews[id]
	if view then
		view:showRewardList()
		self:refreshListView()
	end
end

function This:updateQueueTime( info )
	local csNode = self.m_csNode
	if csNode.time then
    	csNode.time:setString(Serialize[60123] .. GameUtil:formatDTime(Serialize["activity_BaseTabs_1_c"] ,info.time))
    end
end