-- @Author: superm
-- @Date:   2019-01-21 16:50:14
-- @Last Modified by:   superm
-- @Last Modified time: 2019-02-01 16:52:11
-- @File Name: ActivityBoxView.lua
-- @Description: 

local This , super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")

function This:initData(act, filename)
    self.m_activity = act
    self.m_csfileName = filename
    self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
    self.m_csNode = cc.Node:createFromCSFile(self.m_csfileName)
    self:addChild(self.m_csNode)
    local w , h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)
end

function This:onEnter()
    super.onEnter(self)
    self.m_activity:setView(self)
    self.m_activity:addObserver(self)
end

function This:onExit()
    self.m_dialog = nil
    self.m_activity:setView(nil)
    self.m_activity:removeObserver(self)
    super.onExit(self)
end

function This:initNode()
    self:refreshItemUI()
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        local csNode = self.m_csNode
        if csNode.button then
            csNode.button:setTouch({
                getRect = function (touch)
                    local rect = touch.sizeNode:getBoundingBox()
                    return rect
                end,
    		    onEnded = function()
                    if not self:onPickUpBtnClick() then
                        if self:isLocked() then
                            local limitNum, visible = self.m_activity:getLimitNum()
                            if limitNum and limitNum > 0 then
                                if visible then
                                    g_scene:openDialog("UI.CostConfirmPanel", {
                                        cost = limitNum,
                                        desc = Serialize[60201], 
                                        okHandler = function()
                                            self.m_activity:requestUnlock()
                                        end }
                                    )
                                else
                                    g_scene:showTips(string.format(Serialize[60202], limitNum))
                                end
                            end
                        end
                    end
                end,
                isContainer = true,
                isDynamic = true,
                isScale = false,
            })
        end

        if csNode.box then
            csNode.box:setTouch({
                getRect = function (touch)
                    local rect = touch.sizeNode:getBoundingBox()
                    return rect
                end,
                onEnded = function()
                    if not csNode.button then
                        if not self:onPickUpBtnClick() then
                            self:onPreviewBtnClick()
                        end
                    else
                        self:onPreviewBtnClick()
                    end
                end,
                isContainer = true,
                isDynamic = true,
                isScale = false,
            })
        end
        self.m_inited = true
    end
end

function This:refreshItemUI()
    -- local rewardList = self.m_activity:getRewardList()
    local csNode = self.m_csNode
    local image = self.m_activity:getIconIamge()
    if csNode.box and image then
    	csNode.box:setImage(image)
    end

    self:initStateUI()
end

function This:initStateUI()
    local csNode = self.m_csNode

    if csNode.cost then
        local limitNum, visible = self.m_activity:getLimitNum()
        if self:isLocked() and limitNum and visible and limitNum > 0 then
            csNode.cost:setVisible(true)
            csNode.cost.num:setString(limitNum)
            if csNode.cost.days then
                csNode.cost.days:setString(string.format(Serialize[60203], self.m_activity:getQueueCreateDays()))
            end
            if csNode.button then
                csNode.button.text:setVisible(false)
            end    
        else
            csNode.cost:setVisible(false)
            if csNode.button then
                csNode.button.text:setVisible(true)
            end
        end
    end
end

function This:refreshStateUI()
    local csNode = self.m_csNode
    self:setItemEnable(self.m_activity:isOpening())
end

function This:setItemEnable(enble)
    local csNode = self.m_csNode
    local isLocked = self:isLocked()
    if csNode.suo then
        local isLockedByCond = self.m_activity:isLockedByCond()
        csNode.suo:setVisible(isLockedByCond)
        if isLockedByCond and csNode.button then
            csNode.button.text:setVisible(false)
        end
    end

    if isLocked then
        if csNode.button then
            csNode.button:setGrayShader()
            csNode.button:setVisible(true)
        end
        
        if csNode.pickuptext then 
            csNode.pickuptext:setVisible(false)
        end
        
        if csNode.unlocktips then
            csNode.unlocktips:setVisible(false)
        end
    else
        if csNode.button then
            if enble then
                csNode.button:clearShader()
                local num = self.m_activity:getRewardLimitNum()
                if num > 1 then
                    csNode.button.text:setString(string.format(Serialize[60204], num))
                else
                    csNode.button.text:setString(Serialize[60205])
                end
            else
                csNode.button:setVisible(false)
            end
        end

        if csNode.cost:isVisible() and not enble then
            csNode.cost:setVisible(enble)
        end

        if csNode.pickuptext then 
            csNode.pickuptext:setVisible(not enble)
        end

        if csNode.unlocktips then
            csNode.unlocktips:setVisible(enble)
        end
    end

    local isIniting = false
    if not csNode.button then
        local hasEffect = not isLocked and enble
        if hasEffect and not self.m_effect then
            local node = csNode.effect
            self.m_effect = GameUtil:createEffect("ChunJie", true, node)
            node:addChild(self.m_effect)         
            isIniting = true
        elseif not hasEffect and self.m_effect then
            self.m_effect:removeFromParent()
            self.m_effect = nil
        end
    end

    if self.m_effect and not isIniting then
        self.m_effect:scheduleUpdate()
    end
end

function This:isLocked()
    return self.m_activity:isLocked()
end

function This:refresh()
    self:setItemEnable(self.m_activity:isOpening())
    self:refreshStateUI()
end

function This:onPickUpBtnClick()
    if self.m_activity then
        return self.m_activity:requestPickUp()
    end
end

function This:onPreviewBtnClick()
    if self.m_activity then
        local reward = self.m_activity:getRewardList()
        g_scene:openDialog("Activity.ActivityPreviewPanel", {reward=reward, rtype = 1, title = Serialize[60206]})
    end
end

function This:showRewardList()
	local itemList = self.m_activity:getRewardList(1)
	if itemList and #itemList > 0 then
		g_scene:showReward(itemList)
	end
end
