-- @Author: superm
-- @Date:   2018-01-16 17:44:14
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:08:26
-- @File Name: ActivityExchangeModel.lua
-- @Description: 兑换活动项的数据模型

local This, super = class(..., "Activity.ActivityBaseModel")
local TargetConfig = require("Metadatas.TargetConfig")
local ActivityManager = require("Activity.ActivityManager")
local DepotManager = require("Depot.DepotManager")

function This:initSelf( info )
    self:initId( info )
    self.m_state = ActObjectState.kLocked
    self.m_config = info.config
    self.m_exchangeConfig = TargetConfig[self.m_config.exchange_id]
    self:updateState()
end

function This:getRewardList(dataType)
    if dataType then
        if dataType == 1 then
            local t = {}
            for i, itemInfo in ipairs(self.m_exchangeConfig.target_item) do
                table.insert(t, {id = itemInfo[1], count = itemInfo[2]})
            end
            return t
        end
    end
    return self.m_exchangeConfig.target_item
end

function This:getNeedItemList(dataType)
    if dataType then
        if dataType == 1 then
            local t = {}
            for i, itemInfo in ipairs(self.m_exchangeConfig.need_item) do
                table.insert(t, {id = itemInfo[1], count = itemInfo[2]})
            end
            return t
        end
    end
    return self.m_exchangeConfig.need_item
end

function This:getUITypeName()
    return "Activity.ActOpenTaskExchangeBtn"
end

function This:setCurNum(info)
end

function This:getCurNum()
    return self.m_completedTimes or 0
end

function This:getLimitNum()
    return self:getTotalCompleteTimes()
end

function This:getWholeDescNum()
    local curNum = self:getCurNum()
    local limitNum = self:getLimitNum()
    local x
    if limitNum <= curNum then
        x = limitNum
    else
        x = curNum
    end
    return string.format("%d/%d", x, limitNum)
end

function This:getCurDescNum()
    local curNum = self:getQueue():getCurNum(self:getId())
    local limitNum = self:getCondition()
    local x
    if limitNum <= curNum then
        x = limitNum
    else
        x = curNum
    end
    return string.format("%d", x)
end

function This:requestExchange()
    self:updateState()
    if self:isOpening() then
        local actQueueId = self.m_queue:getId()
        ActivityManager:onExchangeRewardRequest(self:getQueueConfigOtherCmd(), actQueueId, self.m_ID)
    end
end

function This:getExchangeId()
    return self.m_config.exchange_id
end

function This:setCompleteTimes( times )
    self.m_completedTimes = times or 1
    if self.m_completedTimes >= self:getTotalCompleteTimes() then
        self:setCompleted(true)
    end
end

function This:increaseCompleteTimes()
    self.m_completedTimes = (self.m_completedTimes or 0) + 1
    if self.m_completedTimes >= self:getTotalCompleteTimes() then
        self:setCompleted(true)
    end 
end

function This:getTotalCompleteTimes()
    if self.m_config then
        return self.m_config.exchange_times or 1
    end
    return 0
end