-- @Author: superm
-- @Date:   2017-12-29 16:00:12
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:08:47
-- @File Name: ActivityFamilyUI.lua
-- @Description: 军团活动


local This, super = class(..., "NodeUI")
This.csfileName = "MenuScene/NodeUI/ActivityfamilyBg.csb"

local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")

local ActivityFamilyItemView = class(This._NAME .. ".ActivityFamilyItemView", "NodeView")
ActivityFamilyItemView.csfileName = "MenuScene/NodeUI/ActivityFamilyrank.csb"

function ActivityFamilyItemView:initData(config, index)
    self.m_config = config
    self.m_inited = false
    self:initCsNode()
end

function ActivityFamilyItemView:initCsNode()
    self.m_csNode = cc.Node:createFromCSFile(self.csfileName)
    self:addChild(self.m_csNode)
    local w , h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)
end

function ActivityFamilyItemView:initNode()
    self:refreshItemUI()
end

function ActivityFamilyItemView:refreshIndexNode()
    local csNode = self.m_csNode
	self.m_index = self.m_config.conditionlist[1]
	local topVisible = self.m_index < 4 and self.m_index > 0
	csNode.rank:setVisible(not topVisible)
	csNode.top:setVisible(topVisible)
	if topVisible then
		csNode.top:setImage("MenuScene/top" .. self.m_index .. ".png")
	-- elseif self.m_index > 0 then
	-- 	csNode.rank:setString(tostring(self.m_index))
	else
		csNode.rank:setString(Serialize["activity_FamilyUI_2"])
	end 
end

function ActivityFamilyItemView:refreshItemUI()
    local csNode = self.m_csNode
	self:refreshIndexNode()
	local rewardMapList = {self.m_config.rewardlist, self.m_config.add_reward}
	-- local rewardNodeList = {self.mem_txt, self.leader_txt}
	local rewardNameList = {"item", "reward"}
	local rewardNumList = {3, 2}
    for index=1, 2 do
		local rewardList = rewardMapList[index]
	    local name = rewardNameList[index]
	    local count = #rewardList
	    
	    -- rewardNodeList[index]:setVisible(count > 0)

	    for i, info in ipairs(rewardList) do
	    	local itemId, num = info[1], info[2]
	    	local itemNode = csNode[name .. tostring(i)]
	    	if itemNode then
	    		itemNode:setVisible(true)
	    		local node = itemNode.node
				if not node then
			        node = ItemNode:create(itemId, num)
			        node:setAnchorPoint(0.5, 0.5)
			        itemNode:addChild(node)
			        itemNode.node = node
			    else
			        node:init(itemId, num)
			    end
	    	end
	    end

	    for i=count + 1, rewardNumList[index] do
	    	local itemNode = csNode[name .. tostring(i)]
	    	if itemNode then
	    		itemNode:setVisible(false)
	    	end
	    end
    end
end

function This:initData(activity)
	self.m_activity = activity
	self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
    self.m_csNode = cc.Node:createFromCSFile(self.csfileName)
    self:addChild(self.m_csNode)
end

function This:initNode()
	local startTime = self.m_activity:getStartTime()
	local startStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],startTime)	
	local endTime = self.m_activity:getEndTime()
	local endStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],endTime)
	self.m_csNode.time:setString(string.format("%s ~ %s",startStr,endStr))
    self:initRewardList()
end

function This:initTouchEvent()
	if not self.m_inited then
    	local csNode = self.m_csNode
		if csNode.button then
			csNode.button:setTouchEnded(function()
				g_scene:openDialog("Family.FamilyPanel")
			end)
		end
	    self.m_inited = true
	end
end

function This:initRewardList()
    local csNode = self.m_csNode
	local configList = self.m_activity:getConfigList()
	local rankRewardList = configList[1]
    csNode.scrollView:removeAllItems()
    csNode.scrollView:setTouchEnabled(true)
    if rankRewardList then
		for _, config in ipairs(rankRewardList) do
			local view = ActivityFamilyItemView:create(config)
		    local w , h = view:getContentSize()
		    csNode.scrollView:pushNode(view, w, h)
		end  
	end

    for i=1, 5 do
    	local itemNode = csNode.memberBg["item" .. tostring(i)]
    	if itemNode then
    		itemNode:setVisible(false)
    	end
    end
    
	local secondGroup = configList[2]
	if not secondGroup then
		return 
	end

	local _, membweRewardList = next(secondGroup)
	if not membweRewardList then
		return
	end

	local rewardlist = membweRewardList.rewardlist
	if not rewardlist then
		return
	end
	
	local count = #rewardlist
	for i, info in ipairs(rewardlist) do
    	local itemId, num = info[1], info[2]
    	local itemNode = csNode.memberBg["item" .. tostring(i)]
    	if itemNode then
    		itemNode:setVisible(true)
    		local node = itemNode.node
			if not node then
		        node = ItemNode:create(itemId, num)
		        node:setAnchorPoint(0.5, 0.5)
		        itemNode:addChild(node)
		        itemNode.node = node
		    else
		        node:init(itemId, num)
		    end
    	end
    end

end
