-- @Author: superm
-- @Date:   2018-04-17 19:55:05
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-06 14:43:47
-- @File Name: ActivityFestivalPanel.lua
-- @Description: 节日活动面板

local This, super = class(...,"Dialog")
local ActivityManager = require("Activity.ActivityManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ViewUtil = require("UI.ViewUtil")
local NewActivityManager = require("NewActivity.NewActivityManager")


function This:initData(info)
    self.m_freshType = ActFreshMode.kFestive
    self.m_title = info and info.title
    self.m_csFileName = info and info.file
    
    self.m_activityList = info.activityList
    self.m_hasUnusualTabButton = info.unusualTabs
    self.m_runningAniantion = false
    self.m_activityNodes = {}
    self.m_activityIndexMap = {}
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/" .. self.m_csFileName .. ".csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    if self.m_title then
       self.m_csNode.title:setString(self.m_title)
    end

    local index = 1
    for i, actQueueIdInfo in ipairs(self.m_activityList) do
    	local btn = self.m_csNode["tep" .. index]
    	if btn then
    		local tabName = tostring(actQueueIdInfo)
			local queue, nameIndex, actQueueId, newAct = self:getTabNodeData(tabName, true)
    		if queue then
    			local indexInfo = {index = index, nameIndex = nameIndex}
    			self.m_activityIndexMap[i] = indexInfo
    			self:setReddotVisible(indexInfo, actQueueId, true)
    			if self.m_csNode.tabbg then
	    			local textNode = self.m_csNode.tabbg["text" .. index]
	    			if not textNode then
	    				if btn.light and btn.light.light then
	    					btn.light.light:setString(queue:getName(nameIndex))
							btn.normal.normal:setString(queue:getName(nameIndex))
	    				end
	    			else
	    				textNode:setString(queue:getName(nameIndex))
	    			end
	    		end

    			index = index + 1
    			
    			self:addTabButton(tabName, btn, function()
					if self.m_runningAniantion then
						return false
					end

					if not self:showQueueTips(queue, newAct) then
						return false
					end

	                self.m_curActQueueId = actQueueId
	                self:selectTabById(tabName)
				end)

	            if self.m_isOpen and not btn.m_touch then
	                btn:setTouch({
	                    isAlpha = false,
	                    onEnded = function ()
	                        self:selectTabButton(tabName)
	                    end
	                })
	            end
    		end
		end
    end
    for i=index, #self.m_activityList do
		self:setTabVisible(i, false)
	end
end

function This:showQueueTips( queue, newAct )
	if not queue then
		g_scene:showTips(Serialize["activity_FestivalPanel_1_c"])
		return false
	end

	if not newAct then
		local visible = queue:isInited() and queue:isUpdating()
		if not visible then
			g_scene:showTips(Serialize["activity_FestivalPanel_1_c"])
            return false
		end
	else
		if queue:getState() == NewActivityState.kNotOpen then
			g_scene:showTips(Serialize["activity_FestivalPanel_2_c"],activity:getOpenLevel())
			return false
		end

		if not queue or queue:getState() == NewActivityState.kEnded then
			g_scene:showTips(Serialize["activity_FestivalPanel_3"])
			return false
		end

		if queue:getState() == NewActivityState.kNotStart then
			g_scene:showTips(Serialize["activity_FestivalPanel_4"])
			return false
		end
	end
	return true
end

function This:getTabNodeData(tabName, needUpdate)
	local newAct = false
	local result = string.split(tabName, "@")
	local name, index = result[1], tonumber(result[2])
	local actQueue = ActivityManager:getActivityQueue(tonumber(name), ActRequestMode.kRequestInDirty) 
	if not actQueue then
		actQueue = NewActivityManager:getActivity(tonumber(name))
        if actQueue and actQueue:isNotStart()  then
            actQueue = nil
        end
		if actQueue and actQueue:isLevelOpen() then
			newAct = true
			if actQueue:isUpdating() and needUpdate then
				actQueue:updateInfo()
			end
		end
	end
	return actQueue, index, name, newAct
end

function This:isTabOpen(tabName)
    local queue = self:getTabNodeData(tabName)
    if not queue then
    	return false
    elseif queue.isInited then
    	return queue:isInited() and queue:isUpdating()
    else
    	return queue:isUpdating()
    end
end

function This:isTabExit(tabName)
    local queue = self:getTabNodeData(tabName)
    if not queue then
    	return false
    elseif queue.isInited then
    	return not queue:isInited() or (queue:isInited() and queue:isUpdating())
    else
    	local state = queue:getState() 
    	return state == NewActivityState.kNotStart or state == NewActivityState.kUpdating
    end
end

function This:createTabNode(tabName)
    local queue, index, name = self:getTabNodeData(tabName)
    if not queue then
    	return
    end

    local uiName = queue:getUITypeName(index)
    if uiName then
    	print(uiName)
        local tabNode = require(uiName):create(queue, index)
		if index then
			if not self.m_activityNodes[name] then
				self.m_activityNodes[name] = {}
			end
			self.m_activityNodes[name][index] = tabNode
		end
        return tabNode
    end
end

function This:selectTabById( tabName )
    self.m_activity = self:getTabNodeData(tabName) 
    if self.m_activity then
        self:openTabNode(tabName)
    end
end

function This:setTabVisible( index, visible )
    if self.m_csNode["tep" .. index] then
	    self.m_csNode["tep" .. index]:setVisible(visible)
    end
    
    if self.m_csNode.tabbg then
		local textNode = self.m_csNode.tabbg["text" .. index]
		if textNode then
			textNode:setVisible(visible)
		end
	end
end

function This:setReddotVisible( info, actId, openable )
	if not info then
		return
	end

	local btn = self.m_csNode["tep" .. info.index]
	if not btn then
		return
	end

    local result = string.split(tostring(actId), "@")
	local actQueueId = result[1]

	local reddotName = "reddot" .. actQueueId
	if info.nameIndex then
		reddotName = reddotName  .. "_" .. info.nameIndex
	end
	local reddot = self.m_csNode[reddotName]
	if reddot then
		reddot:setVisible(openable)
	else
		reddot = cc.Node:create()
		-- print_tmp("reddot create:" .. reddotName)
		reddot:setName(reddotName)
		self.m_csNode:addChild(reddot)
		self.m_csNode[reddotName] = reddot
		reddot:setVisible(openable)
	end

	if openable then
		local x, y = btn:getPosition()
		local w, h = btn:getContentSize()
        if w <= 0 then
            w = 147
        else
            w = w * 0.5
        end

        if h <= 0 then
            h = 76
        else
            h = h * 0.5
        end
		reddot:setPosition(x + w, y + h)
	end
end

function This:selectTabButton(name)
    if self.m_tabButtons then
        if self.m_selectTabButton == name then
            return
        end

        local btn = self.m_tabButtons[name]
        if not btn then
        	return
        end

        if btn.onSelected() ~= false then
            self:lightTabButton(name)
        end
    end
end

function This:refresh()
	if self.m_activityList then
		local defaultTab
		for i, actQueueId in ipairs(self.m_activityList) do
			local indexInfo = self.m_activityIndexMap[i]
			if indexInfo then
				local visible = self:isTabExit(tostring(actQueueId))
				self:setTabVisible(indexInfo.index, visible)
				local openable = self:isTabOpen(tostring(actQueueId))
				self:setReddotVisible(indexInfo, actQueueId, openable)
				if not defaultTab and visible and openable then
					defaultTab = tostring(actQueueId)
				end
			end
		end

		local tabName = self:getCurSelectTab()
		if tabName and self:isTabExit(tabName) and self:isTabOpen(tabName) then
			return
		end

		if defaultTab then
			self:selectTabButton(defaultTab)
		end
	end
end

function This:jumpTabByIndex(index)
	if self.m_activityList then
		local actQueueId = self.m_activityList[index]
		if not actQueueId then return end
		local tabName = tostring(actQueueId)
		if tabName and self:isTabExit(tabName) and self:isTabOpen(tabName) then
			self:selectTabButton(tabName)
		end
	end
end

function This:setIsRunning( value )
    self.m_runningAniantion = value
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
		if self.m_runningAniantion then
			return false
		end
        self:close()
    end)
end

function This:onEnter()
    super.onEnter(self)
    ActivityManager:addObserver(self)
	NewActivityManager:removeObserver(self)
	self:refresh()
end

function This:onExit()
	NewActivityManager:removeObserver(self)
    ActivityManager:removeObserver(self)
    super.onExit(self)
end

function This:lightTabButton(name)
	if self.m_hasUnusualTabButton then
	    self.m_selectTabButton = name
	    if self.m_tabButtons then
	        for k,node in pairs(self.m_tabButtons) do
	            if k == name then
	                node:setOpacity(255)
	            else
	                node:setOpacity(0)
	            end
	        end
	    end
	else
		super.lightTabButton(self, name)
	end
end

-----------------------------------------
function This:onGetActivityInfo( actQueueId )
    local node = self.m_tabNodes[tostring(actQueueId)]
    if node and node.refresh then
        node:refresh()
    end

    local nodeList = self.m_activityNodes[tostring(actQueueId)]
    if nodeList then
	    for _, node in pairs(nodeList) do
		    if node and node.refresh then
		        node:refresh()
		    end
	    end
	end
end

function This:onActivityStart(activity)
	if activity:getId() == NewActivityType.kLimitRecruit then
		self:refresh()
	end
end

function This:onActivityEnded(activity)
	if activity:getId() == NewActivityType.kLimitRecruit then
		self:refresh()
	end
end