-- @Author: superm
-- @Date:   2017-09-23 16:00:00
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-03 00:54:10
-- @File Name: ActivityItemView.lua

local This , super = class(..., "NodeView")
This.csfileName = "MenuScene/NodeUI/ActivityItem.csb"

local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

This.s_itemViews = {}

function This.getAllViews(queueId)
    local ret = This.s_itemViews[queueId]
    if not ret then
        ret = {}
        This.s_itemViews[queueId] = ret
    end
    return ret
end

function This.cacheView(queueId, actId, view)
    if not This.s_itemViews[queueId] then
        This.s_itemViews[queueId] = {}
    end
    This.s_itemViews[queueId][actId] = view
    view:retain()
end

function This.releaseAllViews()
    for _, views in pairs(This.s_itemViews) do
        for _, view in pairs(views) do
            view:release()
        end
    end
    This.s_itemViews={}
end

function This:initData(act, selectionIndex)
    self.m_activity = act
    self.m_selection= selectionIndex
    self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
    self.m_csNode = cc.Node:createFromCSFile(self.csfileName)
    self:addChild(self.m_csNode)
    local w , h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)
end

function This:onEnter()
    super.onEnter(self)
    self.m_activity:setView(self)
    self.m_activity:addObserver(self)
end

function This:onExit()
    self.m_dialog = nil
    self.m_activity:setView(nil)
    self.m_activity:removeObserver(self)
    super.onExit(self)
end

function This:initNode()
    self:refreshItemUI()
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        if self.m_csNode.button then
            self.m_csNode.button:setTouch({
                getRect = function (touch)
                    local rect = touch.sizeNode:getBoundingBox()
                    return rect
                end,
    		    onEnded = function()
                    self:onPickUpBtnClick()
                end,
                isContainer = true,
                isDynamic = true,
                isScale = false,
            })  
        end
        self.m_inited = true
    end
end

function This:refreshItemUI()
    local rewardList = self.m_activity:getRewardList()
    local csNode = self.m_csNode
    local count = #rewardList
    for i, info in ipairs(rewardList) do
    	local itemId, num = info[1], info[2]
    	local itemNode = csNode["reward" .. tostring(i)]
    	if itemNode then
    		itemNode:setVisible(true)
    		local node = itemNode.node
			if not node then
		        node = ItemNode:create(itemId, num)
		        node:setAnchorPoint(0.5, 0.5)
		        itemNode:addChild(node)
		        itemNode.node = node
		    else
		        node:init(itemId, num)
		    end
    	end
    end

    for i=count + 1, 7 do
    	local itemNode = csNode["reward" .. tostring(i)]
    	if itemNode then
    		itemNode:setVisible(false)
    	end
    end

    self:initStateUI()
end

function This:initStateUI()
    local csNode = self.m_csNode

    csNode.condition:setString(self.m_activity:getDesc())
    local limitNum = self.m_activity:getLimitNum()
    csNode.limit:setString(limitNum) 
end

function This:refreshStateUI()
    local csNode = self.m_csNode
    local curNum = self.m_activity:getCurDescNum()
    csNode.cur:setString(curNum)
    -- self:setItemEnable(self.m_activity:isOpening())
    GameUtil:linearLayoutNodes(2, {csNode.text, csNode.cur, csNode.text2, csNode.limit}, false, nil, 184)
end

function This:setDialog( dialog )
    self.m_dialog = dialog
end

function This:setItemEnable(enble)
    local csNode = self.m_csNode

    if not csNode.button then
        return
    end
    if self:isLocked() then
        csNode.button:setGrayShader()
        csNode.button:setVisible(true)
        
        if csNode.pickuptext then 
            csNode.pickuptext:setVisible(false)
        end
    else
        if enble then
            csNode.button:clearShader()
        else
            csNode.button:setVisible(false)
        end

        if csNode.pickuptext then 
            csNode.pickuptext:setVisible(not enble)
        end
    end
end

function This:isLocked()
    return self.m_activity:isLocked()
end

function This:refresh()
    self:setItemEnable(self.m_activity:isOpening())
    self:refreshStateUI()
end

function This:onPickUpBtnClick()
    if self.m_activity then
        self.m_activity:requestPickUp()
    end
end

function This:showRewardList()
	local itemList = self.m_activity:getRewardList(1)
	if itemList and #itemList > 0 then
		g_scene:showReward(itemList)
	end
end


--endregion