-- @Author: superm
-- @Date:   2018-02-08 10:31:58
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-02 13:52:49
-- @File Name: ActivityListBaseBtn.lua
-- @Description: 

local This, super = class(..., "Home.MenuButton")
local ActivityManager = require("Activity.ActivityManager")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ActivityUtil = require("Activity.ActivityUtil")


function This:init(config)
    super.init(self, config)
    self.m_autoClose = config.uiclass.auto_close
    self.m_configIdsList = config.uiclass.ids
    self.m_hasRules = config.uiclass.has_rules
    if config.uiclass.csnode then
        self.m_queueIdList = {}
        self.m_queueList = {}
        self.m_configIdsList = {}
        for _, str in ipairs(config.uiclass.ids) do
            local result = string.split(str, "@")
            local queueId = tonumber(result[1])
            if not self.m_hasRules then
                table.insert(self.m_queueIdList, queueId)
                table.insert(self.m_configIdsList, str)
            else 
                local groupList = ActivityUtil:getUIStrList(queueId)
                if groupList then
                    for _, groupStr in ipairs(groupList) do
                        table.insert(self.m_queueIdList, queueId)
                        table.insert(self.m_configIdsList, groupStr)
                    end
                else
                    table.insert(self.m_queueIdList, queueId)
                    table.insert(self.m_configIdsList, str)
                end
            end
            
            local actQueue = ActivityManager:getQueueById(queueId) or NewActivityManager:getActivity(queueId)
            if actQueue and actQueue:isLevelOpen() then
                self.m_queueList[queueId] = actQueue
            end
        end
    else
        self.m_queueIdList = config.uiclass.ids or {}
        self.m_queueList = {}
        for _, queueId in ipairs(self.m_queueIdList) do
            local actQueue = ActivityManager:getQueueById(queueId) or NewActivityManager:getActivity(queueId)
            if actQueue and actQueue:isLevelOpen() then
                self.m_queueList[queueId] = actQueue
            end
        end
    end
end

function This:addCustomListener()
    super.addCustomListener(self)
    if self.m_autoClose then
        ActivityManager:addObserver(self)
    	if self.m_queueIdList then
    		for id, queue in pairs(self.m_queueList) do
    			queue:addObserver(self)
    		end
    	end
    end
end

function This:removeCustomListener()
	if self.m_queueIdList then
		for id, queue in pairs(self.m_queueList) do
			queue:removeObserver(self)
		end
	end
    ActivityManager:removeObserver(self)
    super.removeCustomListener(self)
end

function This:initTouchEvent()
    if self.m_config.uiclass.csnode then
        local panelName = "ActivityFestival"
        if self.m_config.uiclass.panel_name then
            panelName = self.m_config.uiclass.panel_name
        end

        self:setTouch({
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
                local panel = g_scene:openDialog("Activity." .. panelName .. "Panel", {
                    file = self.m_config.uiclass.csnode, 
                    unusualTabs = self.m_config.uiclass.unusual_tabs,
                    activityList = self.m_configIdsList,
                })
                self.m_subView = panel
                panel:addCloseCallback(function ()
                    self.m_subView = nil
                end)
            end
        })
    elseif self.m_config.uiclass.common_panel then
        self:setTouch({
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
                local panel = g_scene:openDialog("Activity.ActivityPanel", self.m_config.uiclass.common_panel)
                self.m_subView = panel
                panel:addCloseCallback(function ()
                    self.m_subView = nil
                end)
            end
        })
    end
end

function This:close()
    if self.m_subView and self.m_subView[0] ~= 0 then
        self.m_subView:close()
    end
    self:setVisible(false)
end

function This:open()
    self:setVisible(true)
end

function This:getDefaultVisible()
	if self.m_queueIdList then
		for id, queue in pairs(self.m_queueList) do
			if (not queue.isInited or queue:isInited()) and queue:isUpdating() then
				return true
			end
		end
	end

    return super.getDefaultVisible(self)
end

function This:onQueueUpdateEnd( id )
    if self.m_autoClose then
        local found = false
        for _, queueId in ipairs(self.m_queueIdList) do
            if id == queueId then
                found = true
                break
            end
        end

        if found then
            local isAllClose = true 
            for id, queue in pairs(self.m_queueList) do
                if (not queue.isInited or queue:isInited()) and queue:isUpdating() then
                    isAllClose = false
                    break
                end
            end

            if isAllClose then
                self:close()
            end
        end
    end 
end

function This:onQueueUpdateStart( id )
    if self.m_autoClose then
        for _, queueId in ipairs(self.m_queueIdList) do
            if id == queueId then
                self:open()
                break
            end
        end
    end
end

function This:onActivityStart( activity )
    self:onQueueUpdateStart(activity:getId())  
end

function This:onActivityEnded( activity )
    self:onQueueUpdateEnd(activity:getId())  
end

function This:onRedDotChange( name, value )
    if name == self:getName() then
        local ret = value
        if self.m_autoClose then
            ret = value and true or nil
        end
        super.onRedDotChange(self, name, ret)
    end
end

function This:getRedDotValue()
    local value = super.getRedDotValue(self)
    local ret = value
    if self.m_autoClose then
        ret = value and true or nil
    end
    return ret
end

function This:onCreateActivity( id )
    if self.m_queueIdList then
        local ret 
        if self.m_config and self.m_config.uiclass.csnode then
            for _, str in ipairs(self.m_config.uiclass.ids) do
                local result = string.split(str, "@")
                local queueId = tonumber(result[1])
                if queueId == id then
                    ret = str
                    break
                end
            end
        else
            if not self.m_config or not self.m_config.uiclass.ids then
                return
            end

            for _, str in ipairs(self.m_config.uiclass.ids) do
                local queueId = tonumber(str)
                if queueId == id then
                    ret = str
                    break
                end
            end
        end

        if ret then
            local actQueue = ActivityManager:getQueueById(id) or NewActivityManager:getActivity(id)
            if actQueue and actQueue:isLevelOpen() then
                self.m_queueList[id] = actQueue
                if self.m_autoClose then
                    actQueue:addObserver(self)
                end
            end
        end
    end
end
