-- @Author: superm
-- @Date:   2017-11-20 16:35:53
-- @Last Modified by:   superm
-- @Last Modified time: 2019-09-26 18:29:19
-- @File Name: ActivityManager.lua
-- @Description: 常规活动

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local ActivityTypeConfig = require("Metadatas.ActivityTypeConfig")
local ActivityUtil = require("Activity.ActivityUtil")

local Timer = require("Timer")
local NewActivityManager = require("NewActivity.NewActivityManager")

-- local bbb = print_r

-- local function print_r( ... )
-- end
-- local function bbb( ... )
-- end

function This:isInited()
    return self.m_inited
end

function This:init()
	super.init(self)
    self:initListener()
    self.m_actQueue = {}
    self.m_actTypeQueue = {}
    self.m_actFreshList = {}
	self.m_actSubTypeMap = {}
	self.m_actOtherTypeMap = {}
    self.m_actFreshMap = {}
    self.m_cmdList = {}
	self:initActivitys()
end

function This:initListener()
    -- self.m_updateList = ActivityNodeList:create()
	
	EventDispatcher:addListener(self, "player_attr_changed", self.onRefreshAfterAttrsChanged)
	EventDispatcher:addListener(self, "menu_scene_exit", self.clearAllCache)
	EventDispatcher:addListener(self, "init_role_info", self.requestAllActivityInfo)
	EventDispatcher:addListener(self, "day_passed_delay_31s_next",self.onDayPassed)
	EventDispatcher:addListener(self, "after_recharge", self.onRefreshAfterRecharge)
	EventDispatcher:addListener(self, "rank_level_history_change", self.onRankLevelChanged)
	EventDispatcher:addListener(self, "equip_changed", self.onItemUpdate)
	
	for _, cmd in pairs(ActType) do
		EventDispatcher:addListener(self, cmd, self.onGetActivityInfoReply)
	end
	EventDispatcher:addListener(self, 12004, self.onPickupActRewardReply)
	EventDispatcher:addListener(self, 12042, self.onUnlockActReply)
	EventDispatcher:addListener(self, 12045, self.onUnlockActReply)
	EventDispatcher:addListener(self, 12043, self.onPickupActRewardReply)
	EventDispatcher:addListener(self, 12046, self.onPickupActRewardReply)
	EventDispatcher:addListener(self, 12049, self.onPickupActRewardReply)
	EventDispatcher:addListener(self, 12009, self.onGetActivityOrderReply)
	EventDispatcher:addListener(self, 11901, self.onOpenTaskInfoChangeReply)
	EventDispatcher:addListener(self, 12010, self.onDoActivateActReply)
	EventDispatcher:addListener(self, 12014, self.onExchangeRewardReply)
	EventDispatcher:addListener(self, 12050, self.onExchangeRewardReply)
	EventDispatcher:addListener(self, 12024, self.onBuyMonthlyReply)
	EventDispatcher:addListener(self, 12051, self.onGetActivityHighestInfoReply)
	EventDispatcher:addListener(self, 10217, self.onGetPhoneCardReply)
	cc.Application:addListener(self)
end

function This:removeListener()
	cc.Application:removeListener(self)
	EventDispatcher:removeListener(self, "after_recharge")
	EventDispatcher:removeListener(self, "day_passed_delay_31s_next")
	EventDispatcher:removeListener(self, "init_role_info")
	EventDispatcher:removeListener(self, "player_attr_changed")
	EventDispatcher:removeListener(self, "equip_changed")
	for _, cmd in pairs(ActType) do
		EventDispatcher:removeListener(self, cmd)
	end
	for _, cmd in pairs(ActUnlockType) do
		EventDispatcher:removeListener(self, cmd)
	end
	EventDispatcher:removeListener(self, 12004)
	EventDispatcher:removeListener(self, 12009)
	EventDispatcher:removeListener(self, 11901)
	EventDispatcher:removeListener(self, 12010)
	EventDispatcher:removeListener(self, 12014)
	EventDispatcher:removeListener(self, 12042)
	EventDispatcher:removeListener(self, 12045)
	EventDispatcher:removeListener(self, 12043)
	EventDispatcher:removeListener(self, 12046)
	EventDispatcher:removeListener(self, 12049)
	EventDispatcher:removeListener(self, 12050)
	EventDispatcher:removeListener(self, 12051)
	EventDispatcher:removeListener(self,10217)

    -- self.m_updateList = nil
end

function This:onGetActivityHighestInfoRequest(activityId)
	Socket:sendAndLock(12051, {activity_id = activityId})
end

function This:onGetActivityHighestInfoReply( msg )
    if msg.err_code == 0 then
    	local actQueueId = 57
	    local actQueue = self:getActivityQueue(actQueueId)
	    if actQueue then
	    	msg.id = msg.activity_id
	    	actQueue:setHightestNum(msg)
		    self:refreshActState(actQueue, actQueueId)
	    end
		self:notifyObservers("onGetActivityHighestInfo", msg.activity_id)
    elseif msg.err_code == 1 then
		self:notifyObservers("onGetActivityHighestInfo", msg.activity_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:initActivitys()
	for i, queueCfg in ipairs(ActivityTypeConfig) do
		if queueCfg.type_name then 
			local actId, actFreshMode = queueCfg.id, queueCfg.fresh
			local actType, actSubType, actOtherType = queueCfg.cmd, queueCfg.subcmd, queueCfg.othercmd
			self.m_actFreshMap[actId] = actFreshMode
            if actType and actType > 0 then
            	if not self.m_actTypeQueue[actType] then
			        self.m_actTypeQueue[actType] = {}
		        end
            	if actSubType then
			        self.m_actSubTypeMap[actSubType] = actType
		        end
            	if actOtherType then
			        self.m_actOtherTypeMap[actOtherType] = actType
		        end
				table.insert(self.m_actTypeQueue[actType], actId)
            end
		end
	end
end

function This:createActivity(info)
	if not info then return end
	local infoId = info.activity_id

	if not infoId or infoId <= 0 then
		return 
	end
--测试
	local queueCfg = ActivityTypeConfig[infoId]
	if not self.m_actQueue[infoId] and queueCfg then
		local level = GamePlayer:getLevel()
		if queueCfg.open_level > level then
			return
		end

		if queueCfg.type_name then
		    local activity
		    local fileName = nil
		    if queueCfg.sub_type then
		    	fileName = "Act" .. queueCfg.sub_type .. "QueueModel"
		    else
		    	fileName = "Act" .. queueCfg.type_name .. "QueueModel"
		    end

		    local class = require("Activity." .. fileName)
			local actId, actType = queueCfg.id, queueCfg.cmd
            
            if class and class ~= true then
            	local param = {
            		id = actId,
					initing = true,
					start_level = info.start_level,
	            	start_time = info.start_time,
					end_time = info.end_time,
					config = queueCfg,
	            }
		        activity = class:create(param)
            end

		    if activity then
			    self.m_actQueue[actId] = activity
			    EventDispatcher:dispatcher("onCreateActivity", actId)
                self:notifyObservers("onCreateActivity", actId)
		    	print(string.format("创建活动, id:%d, %s", actId, activity:getName()))
		    end
		    return activity
        end
	end
end

-- function This:addMsg( ... )
-- 	EventDispatcher:addMsg( ... )
-- end

-- function This:setTimer( ... )
--     self.m_timer:setTimer( ... )
-- end

-- function This:update(dt)
-- 	if not self.m_isRunning then
--         return false
--     end
--     -- self.m_timer:update(dt)
-- 	self.m_updateList:update(dt)
-- end

-- function This:removeTimer(key)
--     self.m_timer:removeTimer(key)
-- end

-- function This:removeAllTimer()
--     self.m_timer:removeAllTimer()
-- end

-- function This:addToUpdateList(listener)
-- 	self.m_updateList:addNode(listener)
-- end

-- function This:removeFromUpdateList(listener)
-- 	self.m_updateList:removeNode(listener)
-- 	if self.m_updateList:getSize() <= 0 then
--         self:endUpdate()
-- 	end
-- end

-- function This:startUpdate()
-- 	if not self.m_isRunning then
-- 		self.m_isRunning = true
--     	cc.Application:addToUpdateList(self)
--     end
-- end

-- function This:endUpdate()
-- 	if self.m_isRunning then
-- 		self.m_isRunning = false
-- 		cc.Application:removeFromUpdateList(self) 
-- 	end
-- end

function This:requestAllActivityInfo()
	if not self.m_inited then
		self:onGetActivityOrderRequest()
	end
end

function This:getQueueIdByType( actType, index )
	local queueIdsList = self.m_actTypeQueue[actType]
	if not queueIdsList then
		return nil
	end

	return queueIdsList[index or 1]
end

function This:getQueueByType( actType, index )
	local actId = self:getQueueIdByType( actType, index )
	if actId and actId > 0 then
		return self.m_actQueue[actId]
	end
end

function This:getQueueBySubType( actSubType, index )
	local actType = self.m_actSubTypeMap[actSubType]
	if not actType then
		return
	end

	return self:getQueueByType( actType, index )
end

function This:getQueueByOtherType( actOtherType, index )
	local actType = self.m_actOtherTypeMap[actOtherType]
	if not actType then
		return
	end

	return self:getQueueByType( actType, index )
end

function This:getQueueById( actId )
	if actId and actId > 0 then
		return self.m_actQueue[actId]
	end
end

function This:getQueueListByType( actType )
	local queueIdsList = self.m_actTypeQueue[actType]
	if not queueIdsList then
		return nil
	end

	local ret = {}
	for i, actId in ipairs(queueIdsList) do
		ret[i] = self.m_actQueue[actId]
	end
	return ret
end

function This:notifyActState(queue, state, operate)
	local param = {name = queue:getRedDotKey()}
	if not operate then
		param.value = state
	else
		param.operate = state and "+" or "-"
		param.value = {queue:getId()}
        -- print_r(param)
	end
	print("活动变化通知 " .. "activity_" .. queue:getRedDotKey() .. "_state_changed")
    EventDispatcher:dispatcher("activity_" .. queue:getRedDotKey() .. "_state_changed")
    EventDispatcher:dispatcher("on_menu_visible_change",param)
end

function This:refreshActState( actQueue, actQueueId )
	if not actQueue then
		return
	end

	if actQueue and not actQueueId then
		actQueueId = actQueue:getId()
	end
	local ret = actQueue:getAllObjectState(true)
	local freshMode = actQueue:getFreshMode() or ActFreshMode.kNormal
	if freshMode > 0 then
        if freshMode ~= ActFreshMode.kIdle then
	    	local redDotList = self.m_actFreshList[freshMode]
	    	if redDotList then
	    		for i, info in ipairs(redDotList) do
	    			if info.id == actQueue:getId() then
	    				info.state = ret 
	    				break
	    			end
	        	end
	        end		
		    self:notifyObservers("onActRedDotChange", actQueueId, ret)
    		self:notifyActState(actQueue, ret, true)
    	else
		    self:notifyActState(actQueue, ret)
		end	
	end 
end

function This:getActivityQueueByInfo(info, requestMode, predicateHandler)
	local id = info.activity_id
	local ret = self.m_actQueue[id]
	if not ret then
		ret = self:createActivity(info)
		if not ret then return end
	else
		ret:updateTimeInfo(info)
	end

	requestMode = requestMode or ActRequestMode.kNone
	local switch = {  
        [ActRequestMode.kNormal] = function( ... )
        	local needRequst = false
            if not ret:isInited() or ret:isDirty() then
            	needRequst = true
        	elseif predicateHandler then
    			local obj = ret:getObjectBy(predicateHandler)
            	needRequst = obj ~= nil
            end  

            if needRequst then
            	self:requestActivityInfoBy( ... )
            end
        end,  
        [ActRequestMode.kForce] = function( ... )  
        	self:requestActivityInfoBy( ... )
        end,  
        [ActRequestMode.kRequestInDirty] = function( ... )
        	local needRequst = not ret or ret:isDirty() or ret:isAllwaysDirty()
			if needRequst and predicateHandler then
    			local obj = ret:getObjectBy(predicateHandler)
            	needRequst = obj ~= nil
            end

            if needRequst then
            	self:requestActivityInfoBy( ... )
            end
        end
    }  
      
    local fSwitch = switch[requestMode]  
    if fSwitch and ret then 
        local result = fSwitch(ret:getType(), id)
    end  
    return ret
end

function This:getActivityQueue(id, requestMode, predicateHandler)
	local param = {
		activity_id = id,
	}
	return self:getActivityQueueByInfo(param, requestMode, predicateHandler)
end

function This:setDirty( reason, noRequest )
    for actQueueId, queue in pairs(self.m_actQueue) do
    	if queue:isInited() and not queue:isLocked() then
			queue:setDirty(true, reason)
			if queue:isDirty() and noRequest then
				queue:updateState()
				self:refreshActState(queue, actQueueId)
			end
		end
	end

	if not noRequest then
		self:refreshIfBeDirty()
	end
end

function This:refreshIfBeDirty()
    for actQueueId, queue in pairs(self.m_actQueue) do
    	if queue:isInited() and not queue:isLocked() and queue:isDirty() then
			self:requestActivityInfoBy(queue:getType(), actQueueId)
		end
	end    
end

function This:getActivityOrderList(freshMode)
	local curTime = GamePlayer:getServerTime()
	local actInfoList = self.m_actFreshList[freshMode]
	local list = {}
	for index, actInfo in ipairs(actInfoList) do
		if actInfo.invalid <= 0 then
			table.insert(list, actInfo)
		else
			local info = actInfo.info
			if info.end_time >= curTime and info.start_time <= curTime then
	        	local actQueueId = info.activity_id
				local oldQueue = self:getActivityQueueByInfo(info, ActRequestMode.kNormal)
	        	local newQueue = NewActivityManager:getActivity(actQueueId)
	        	local queue = oldQueue or newQueue
	        	if queue then
		        	local freshMode = queue:getFreshMode()
		        	if freshMode > 0 and queue:getTitleType() > 0 then
		        		local state = queue:getAllObjectState(true)
		        		local newActInfo = {
	    					invalid = 0,
		        			id = actQueueId, 
		        			time = queue:hasResTypeWith(ActResType.kTimeLimitPic), 
		        			name = queue:getName(), 
		        			priority = queue:getPriority(), 
		        			state = state
		        		}
		        		actInfoList[index] = newActInfo
						table.insert(list, newActInfo)
			        end
			    end	
			end
		end
	end

    table.sort(list, function( infoA, infoB )
    	if infoA.priority ~= infoB.priority then
    		return infoA.priority < infoB.priority 
    	end
    	return infoA.id < infoB.id
    end)
	return list
end

function This:loadLocalActivity( dataList )
	-- if dataList then
	-- 	if not GamePlayer:checkChanelId(GamePlayerChanelId.k51Gamer) and not GamePlayer:isAuditVersion() then
	-- 		table.insert(dataList, {activity_id = 80, activity_type=0, start_time=-1, end_time=-1, start_level=1})
	-- 	end
	-- end
	-- table.insert(dataList, {activity_id = 54, activity_type=32, start_time=-1, end_time=-1, start_level = 1})
end

------------------------------------------------------------------
function This:requestActivityInfoBy( actType, actQueueId )
	print_r({ name=Serialize["activity_ActivityManager_1"], actType=actType, activity_id = actQueueId})
	Socket:send(actType, {activity_id = actQueueId})
end

function This:onGetActivityInfoReply( msg )
	if not msg then
		g_scene:handleErrorCode("")
		return 
	end

    print_r(msg)
	if msg.err_code == 0 then
		local actType = tonumber(msg.cmd or 0)
		if actType <= 0 then
			return
		end
		local actQueueId = tonumber(msg.activity_id or 0)
		if actQueueId <= 0 then
			actQueueId = self:getQueueIdByType( actType )
			if not actQueueId or actQueueId <= 0 then
				return
			end
		end

		local actQueue = self:getActivityQueue(actQueueId)
		if actQueue then
			actQueue:markDataTimeBy()
			actQueue:updateInfo(msg)
			self:refreshActState(actQueue, actQueueId)
		end
		self:notifyObservers("onGetActivityInfo", actQueueId, actType)
	-- else
	-- 	g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetActivityOrderRequest()
	Socket:send(12009)
end

function This:onGetActivityOrderReply( msg )
	print_r(msg)
    self:loadLocalActivity(msg.activity_time_list)
	if not self.m_inited then
	    if msg.err_code == 0 then
	        local curTime = GamePlayer:getServerTime()
	        for i, info in ipairs(msg.activity_time_list) do
	        	if info.end_time < 0 or info.end_time >= curTime and info.start_time <= curTime then
		        	local actQueueId = info.activity_id
					local queue = self:getActivityQueueByInfo(info)
					if queue and queue:getType() > 0 then
			        	self:requestActivityInfoBy(queue:getType(), actQueueId)
			        end
			    end
	        end
	    end
	    self:onGetActivityOrderRequest()
		self.m_inited = true
		return
	end

    if msg.err_code == 0 then

        local curTime = GamePlayer:getServerTime()
		for _, mode in pairs(ActFreshMode) do
			self.m_actFreshList[mode] = {}
		end
        for i, info in ipairs(msg.activity_time_list) do
	        local actQueueId = info.activity_id
	        if info.end_time < 0 or info.end_time >= curTime and info.start_time <= curTime then
	        	local oldQueue = self:getActivityQueueByInfo(info, ActRequestMode.kNormal)
	        	local newQueue = NewActivityManager:getActivity(actQueueId)
	        	local queue = oldQueue or newQueue
	        	if queue then
		        	local freshMode = queue:getFreshMode()
		        	if freshMode > 0 and queue:getTitleType() > 0 then
		        		local state = queue:getAllObjectState(true)
		        		table.insert(self.m_actFreshList[freshMode], {
        					invalid = 0,
		        			id = actQueueId, 
		        			time = queue:hasResTypeWith(ActResType.kTimeLimitPic), 
		        			name = queue:getName(), 
		        			priority = queue:getPriority(), 
		        			state = state
		        		})
			        end
			    end
			else
				local freshMode = self.m_actFreshMap[actQueueId]
				if freshMode and freshMode > 0 then
	        		table.insert(self.m_actFreshList[freshMode], {
	        			invalid = 1,
						info = info,
	        		})
	        	end
			end
        end
		self:notifyObservers("onGetActivityOrder")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onPickupActRewardRequest(actType, actQueueId, actId)
	Socket:sendAndLock(12004, {type = actType, activity_id = actQueueId, id = actId, reward_id = actId})
end

function This:onPickupNewActRewardRequest(cmd, actQueueId, actId)
	Socket:sendAndLock(cmd, {activity_id = actQueueId, reward_id = actId})
end

function This:onPickupActRewardReply( msg )
    if msg.err_code == 0 then
    	local actId = msg.id or msg.reward_id
    	local actQueueId = msg.activity_id
        if not actQueueId then
        	actQueue = self:getQueueBySubType(msg.cmd or 0)
        else
            actQueue = self:getActivityQueue(actQueueId)
        end
        
        local activityCompleted = nil
        if actQueue then
        	activityCompleted = actQueue:getObjectBy(function( activity )
        		return activity:getId() == actId
        	end)
            if activityCompleted and actQueueId then
        		activityCompleted:setCompleted()
        	end
            if not actQueueId then
            	actQueueId = actQueue:getId()
            end
        	self:refreshActState(actQueue, actQueueId)
        end
		self:notifyObservers("onPickupActReward", actQueueId, actId, activityCompleted)
    else
    	if msg.err_code == 12011 then
    		local actQueue = self:getActivityQueue(msg.activity_id)
	        local activityCompleted = nil
	        if actQueue then
	        	activityCompleted = actQueue:getObjectBy(function( activity )
	        		return activity:getId() == msg.id
	        	end)

	        	if activityCompleted then
	        		activityCompleted:finish()
	        	end
	        end
			self:notifyObservers("onPickupActFinish", msg.activity_id, msg.id, activityCompleted)
			
    	end
        g_scene:handleErrorCode(msg.err_code)
    end
end


function This:onUnlockActRequest(cmd, actQueueId, actId)
	local param
	if actQueueId then
		param = {activity_id = actQueueId, reward_id = actId}
	end
	Socket:sendAndLock(cmd, param)
end

function This:onUnlockActReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onUnlockAct")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 开服活动的推送
function This:onOpenTaskInfoChangeReply( msg )
	local _, info = next(msg.progress_list)
	if info and info.type == AchieveType.kOpenTask then 
		local queueList = self:getQueueListByType(ActType.kOpenTask)
		if queueList then
			for _, queue in pairs(queueList) do
				if queue and queue:isInited() and not queue:isLocked() then
					local actQueueId = queue:getId()
					queue:updateInfo(msg)
					self:refreshActState(queue, actQueueId)
				end
			end
		end
	end  
end

-- 活动激活
function This:onDoActivateActRequest(id, ctype)
	Socket:sendAndLock(12010, {activity_id = id, type = ctype})
end

function This:onDoActivateActReply( msg )
    if msg.err_code == 0 then
    	self:onGetActivityInfoReply( msg )
        self:notifyObservers("onDoActivateAct", msg.activity_id, msg.type)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onDoRechargeActReply( msg )
    if msg.err_code == 0 then
    	self:onGetActivityInfoReply( msg )
        self:notifyObservers("onDoRechargeActReply", msg.activity_id, msg.type)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 道具兑换
function This:onExchangeRewardRequest(cmd, actQueueId, actId)
	print_r({activity_id = actQueueId, id = actId})
	Socket:sendAndLock(cmd, {activity_id = actQueueId, id = actId, exchange_id = actId})
end

function This:onExchangeRewardReply( msg )
    if msg.err_code == 0 then
    	local actId = msg.id or msg.exchange_id
    	local actQueueId = msg.activity_id
        if not actQueueId then
        	actQueue = self:getQueueByOtherType(msg.cmd or 0)
        else
            actQueue = self:getActivityQueue(actQueueId)
        end

        local activityCompleted = nil
        if actQueue then
        	activityCompleted = actQueue:getEObjectBy(function( activity )
        		return activity:getId() == actId
        	end)

            if not actQueueId then
            	actQueueId = actQueue:getId()
            end

        	if activityCompleted then
                if msg.time then
        		    activityCompleted:setCompleteTimes(msg.time)
                else
					activityCompleted:increaseCompleteTimes()
                end
        	end
        	
        	self:refreshActState(actQueue, actQueueId)
        	self:notifyObservers("onExchangeReward", actQueueId, actId, activityCompleted)
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetPhoneCardRequest(operate)--// 0 查看; 1 领取
	if not GamePlayer:checkChanelId(GamePlayerChanelId.k51Gamer) then
		self.m_onGetPhoneCardRequestOperate = operate
		Socket:sendAndLock(10217,{operate = operate})
	end
end

function This:onGetPhoneCardReply( msg )
	if not GamePlayer:checkChanelId(GamePlayerChanelId.k51Gamer) then
    -- if msg.err_code == 0 then
        -- print(msg)
		local actQueueId = NewActivityType.kPhoneCardAd
        local is_click = msg.is_click or 0--// 是否点击过广告 0 否, 1 是
        local is_received = msg.is_received or 0--// 是否领取了奖励 0 否, 1 是
        local actQueue = self:getActivityQueue(actQueueId)
        if actQueue then
        	actQueue:setIsCanGetCard(is_click)
        	actQueue:setIsAlreadyHaveCard(is_received)
	        if self.m_onGetPhoneCardRequestOperate and self.m_onGetPhoneCardRequestOperate ==1 and is_received ==1 then
	        	local itemList = actQueue:getReward()
				g_scene:showReward(itemList)
	        end
        end
		self:refreshActState(actQueue, actQueueId)
        self:notifyObservers("onGetPhoneCard")
    -- else
    --     g_scene:handleErrorCode(msg.err_code)
    -- end
	end
end
function This:onBuyMonthlyReply( msg )
    if msg.err_code == 0 then
    	print_r(msg)
        local actQueue = self:getActivityQueue(msg.activity_id)
        local activityOpening = nil
        if actQueue then
        	activityOpening = actQueue:getObjectBy(function( activity )
        		return activity:getId() == msg.type_id
        	end)

        	if activityOpening then
        		activityOpening:setOpening()
        	end
        	
        	self:refreshActState(actQueue, actQueueId)
        	self:notifyObservers("onBuyMonthlyReply", msg.activity_id, msg.type_id, activityOpening)
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end 
end

--------------------活动按类型刷新-------------------------------
function This:onRefreshAfterRecharge(rechargeInfo)
	local price = 0
	if rechargeInfo then
		local price = rechargeInfo.price or 0
		local activity = self:getQueueById(NewActivityType.kHonorPay)
		if not activity then
			local queueList = self:getQueueListByType(ActType.kTotalPay)
			if queueList then
				for _, queue in pairs(queueList) do
					if queue then
						local payNum = (queue:getPayNum() or 0) + price
						if payNum >= 10000 and payNum <= 10648 then
							self:onGetActivityOrderRequest()
							break
						end
					end
				end
			end
		end
	end
	self:setDirty(ActDirtyReasonType.kRecharge)
end

function This:onRefreshAfterAttrsChanged( info )
	if info.attrId == PlayerAttr.level and info.old ~= info.new then
		for actQueueId, queue in pairs(self.m_actQueue) do
	    	if queue:isOpening() or queue:isUpdating() then
				queue:setDirty(true, ActDirtyReasonType.kLevel)
				if queue:isDirty() then
					queue:updateState()
					self:refreshActState(queue, actQueueId)
					self:requestActivityInfoBy(queue:getType(), actQueueId)
					queue:setDirty(false)
				end
			elseif queue:isLocked() and not queue:isSpecialUnlocked() then
				queue:setSpecialUnlocked(true)
				self:requestActivityInfoBy(queue:getType(), actQueueId)
			end
		end

		self:onGetActivityOrderRequest()
	elseif info.attrId == PlayerAttr.diamond and info.old > info.new then
		for actQueueId, queue in pairs(self.m_actQueue) do
	    	if queue:isOpening() or queue:isUpdating() then
				queue:setDirty(true, ActDirtyReasonType.kCostDiamond)
				if queue:isDirty() then
					queue:updateState()
					self:refreshActState(queue, actQueueId)
					self:requestActivityInfoBy(queue:getType(), actQueueId)
					queue:setDirty(false)
				end
			end
		end
	end
end

function This:onEnterForeground()
	if self.m_actQueue then
	    for actQueueId, queue in pairs(self.m_actQueue) do
	    	if queue:isInited() and not queue:isLocked() and queue:isDirty() or queue:isAllwaysDirty() then
	    		self:requestActivityInfoBy(queue:getType(), actQueueId)
			end
		end
	end
end

function This:onEnterBackground()
	if self.m_actQueue then
	    for actQueueId, queue in pairs(self.m_actQueue) do
	    	if queue:isInited() and not queue:isLocked() and not queue:isDirty() and not queue:isAllwaysDirty() then
	    		queue:setDirty(true, ActDirtyReasonType.kTime)
			end
		end
	end    
end

function This:onDayPassed()
	self:setDirty(ActDirtyReasonType.kDay, true)
	self:onGetActivityOrderRequest()
end

function This:onRankLevelChanged()
	self:setDirty(ActDirtyReasonType.kRankLevel)
end

function This:onItemUpdate()
	self:setDirty(ActDirtyReasonType.kItemUpdate, true)
end

function This:clearAllCache()
    require("Activity.ActivityItemView").releaseAllViews()
end

function This:setPalyerActInfo(key, value, dispatch)
	local GameUtil = require("Util.GameUtil")
    local info = cc.FileUtils:loadLuaFile("activity") or {}
    local id = GameUtil:int64ToString(GamePlayer:getPlayerID())
    local actinfo = info[id]
    if not actinfo then
    	actinfo = {}
    	info[id] = actinfo
    end
	actinfo[key] = value
    cc.FileUtils:saveToLua(info,"activity")

    if dispatch then
		EventDispatcher:addMsg("activity_" .. key .. "_state_changed")
    end
end

function This:getPalyerActInfo(key, defaultValue)
    local info = cc.FileUtils:loadLuaFile("activity")
    if not info then
    	return defaultValue
    end

	local GameUtil = require("Util.GameUtil")
    local id = GameUtil:int64ToString(GamePlayer:getPlayerID())
    local actinfo = info[id]
    if not actinfo then
    	return defaultValue
    end

    local value = actinfo[key]
    if value ~= nil then
    	return value
    else
    	return defaultValue
    end
end

function This:isFirstPayFinish()
	local queue = self:getQueueById(1)
	if queue then
		if queue:getCurNum() then
			return true
		end
	end
	return false
end