-- @Author: superm
-- @Date:   2019-03-04 16:01:29
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-10 10:00:46
-- @File Name: ActivityNewBtn.lua
-- @Description: 


local This, super = class(..., "Home.MenuButton")
local NewActivityManager = require("NewActivity.NewActivityManager")

function This:init(config)
    super.init(self, config)
    self.m_autoClose = config.uiclass.auto_close
    self.m_queueId = config.uiclass.id
    if self.m_queueId and self.m_queueId > 0 then
        self.m_queue = NewActivityManager:getActivity(self.m_queueId)
    end
end

function This:addCustomListener()
    super.addCustomListener(self)
    if self.m_autoClose then
        NewActivityManager:addObserver(self)
        if self.m_queue then
            self.m_queue:addObserver(self)
        end
    end
end

function This:removeCustomListener()
    if self.m_queue then
        self.m_queue:removeObserver(self)
    end
    NewActivityManager:removeObserver(self)
    super.removeCustomListener(self)
end

function This:initTouchEvent()
    self:setTouchEnded(function()
        local name = self.m_queue:getPanelName() or "Activity.ActivitySeparatedPanel"
        local panel = g_scene:openDialog(name, {queue = self.m_queue})
        self.m_subView = panel
        panel:addCloseCallback(function ()
            self.m_subView = nil
        end)
    end) 
end

function This:close()
	if self.m_queue and not self.m_queue:isUpdating() then
        if self.m_subView then
            self.m_subView:close()
        end
	end
end

function This:onActivityStart( activity )
    if self.m_autoClose and activity:getId() == self.m_queueId then
    	self:open()
    end 
end


function This:onActivityEnded( activity )
    if self.m_autoClose and activity:getId() == self.m_queueId then
    	self:close()
    end 
end

function This:getDefaultVisible()
	if self.m_queue and self.m_queue:isUpdating() then
		return true
	end

    return super.getDefaultVisible(self)
end