-- @Author: superm
-- @Date:   2019-01-21 16:10:44
-- @Last Modified by:   superm
-- @Last Modified time: 2020-01-14 14:29:14
-- @File Name: ActivityNewSpringUI.lua
-- @Description: 

local This, super = class(..., "Activity.ActivityBaseUI")
local ViewUtil = require("UI.ViewUtil")
local ArgConfig = require("Metadatas.ArgConfig")

function This:initData(activity, index)
	self.m_activity = activity
    self.m_curSelection = index
    self.m_itemName, self.m_itemFileName = activity:getItemName()
	self.m_inited = false
    self:initCsNode()
end

function This:initNode()
	super.initNode(self)
	local csNode = self.m_csNode
	if csNode.look then
		csNode.look:setVisible(false)
    	ViewUtil:blinkView(csNode.look, true)
	end
end

function This:initCustomTouchEvent(csNode)
	if csNode.wenhao then
		csNode.wenhao:setTouchEnded(function()
			local helpTips = self.m_activity:getHelpTips()
			g_scene:openDialog("UI.HelpTips", {text = Serialize[helpTips], node = csNode.wenhao})
		end)
	end

	if csNode.jump then
		csNode.jump:setTouchEnded(function()
			local panel = g_scene:getDialog("ActivityFestivalPanel")
			if panel then
				panel:jumpTabByIndex(ArgConfig.activity_54_ui.arg[1])
			end
		end)
	end

	if csNode.songfu then
		csNode.songfu:setTouchEnded(function()
			self.m_activity:requestUnlock()
		end)
	end
end

function This:getAllItemViews()
    return {}
end

function This:getNewItemView( ... )
    return require(self.m_itemName):create(..., self.m_itemFileName)
end

function This:refreshResource()
	local csNode = self.m_csNode
	if csNode.zhufuzhi then
		csNode.zhufuzhi.num:setString(self.m_activity:getCurNum())
    end

    if csNode.songfu then
		csNode.songfu.text:setString(string.format(Serialize[60222], self.m_activity:getCurDescNum()))
    end

    for _, view in pairs(self.m_curViews) do
    	view:refresh()
    end
end

function This:refreshItemListByData( activitys )
	local csNode = self.m_csNode
    csNode.scrollView:removeAllItems()
    csNode.scrollView:setTouchEnabled(true)
	table.sort(activitys, function( a, b )
        return a:getId() < b:getId()
	end)

	local count = #activitys
	local width = csNode.scrollView:getContentWidth()
	for index, activity in ipairs(activitys) do
        local id = activity:getId()
        local view = self.m_curViews[id]
        if not view then
			view = self:getNewItemView(activity)
			self.m_curViews[id] = view
			view:retain()
		else
			view:refresh()
		end
	    local w , h = view:getContentSize()
		if not csNode.scrollView.m_itemMargin then
			print(width / count - w)
			csNode.scrollView.m_itemMargin = {width / 4 - w, 5}
		end
	    csNode.scrollView:pushNode(view, w, h, 4) 
	end  

	if csNode.zhufuzhi then
		csNode.zhufuzhi.num:setString(self.m_activity:getCurNum())
    end

    if csNode.songfu then
		csNode.songfu.text:setString(string.format(Serialize[60222], self.m_activity:getCurDescNum()))
    end
end

function This:onUnlockAct()
	self:refreshResource()
end

function This:refresh()
	super.refresh(self)
	self:refreshResource()
end