-- @Author: superm
-- @Date:   2017-09-23 10:36:25
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:11:45
-- 活动面板
--region ActivityPanel.lua

local This, super = class(...,"Dialog")
local ActivityManager = require("Activity.ActivityManager")
-- local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ViewUtil = require("UI.ViewUtil")
local NewActivityManager = require("NewActivity.NewActivityManager")
local DepotManager = require("Depot.DepotManager")

function This:initData(info)
    self.m_freshType = info and info.fresh or ActFreshMode.kNormal
    self.m_freshType = self.m_freshType or ActFreshMode.kNormal
    self.m_title = info and info.title
    self.m_csFileName = info and info.file or "Activity"
    self.m_csFileName = self.m_csFileName or "Activity"
    self.m_buttonList = {}

    local orderList = ActivityManager:getActivityOrderList(self.m_freshType)
    self.m_orderList = orderList
    if not orderList then
        ActivityManager:onGetActivityOrderRequest()
    end
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/" .. self.m_csFileName .. ".csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    if self.m_title then
       self.m_csNode.title:setString(self.m_title)
    end

    local bgNode = self.m_csNode.typeBg
    local bgPosx, bgPosy = bgNode:getPosition()
    local bgw, bgh = bgNode:getContentSize()
    self.m_bgNodeInfo = {
        width = bgw,
        high = bgh,
        posx = bgPosx,
        posy = bgPosy,
    }
    
    self.m_btnListView = self.m_csNode.buttonList
    if self.m_btnListView then
        self.m_btnListView:setSwallowTouch(false)
    end
    self:onGetActivityOrder(self.m_orderList)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        if self.m_runningAniantion then
            return false
        end
        self:close()
    end)
end

function This:onEnter()
    super.onEnter(self)
    ActivityManager:addObserver(self)
end

function This:onExit()
    if self.m_buttonList then
        for _, view in pairs(self.m_buttonList) do
            view:release()
        end
        self.m_buttonList = {}
    end

    ActivityManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
    for name, node in pairs(self.m_tabNodes) do
        if node and node.refresh then
            node:refresh()
        end
    end
end

function This:getTabNodeData(name)
    local actQueue = ActivityManager:getActivityQueue(tonumber(name)) or NewActivityManager:getActivity(tonumber(name)) 
    if actQueue and actQueue:isLevelOpen() then
       return actQueue
    end
end

function This:createTabNode(name)
    local queue = self:getTabNodeData(name)
    local uiName = queue:getUITypeName()
    if uiName then
        local tabNode = require(uiName):create(queue, self)
        return tabNode
    end
end

function This:selectTabById( actQueueId, forceRequest )
    local mode = ActRequestMode.kNormal
    if forceRequest then
        mode = ActRequestMode.kRequestInDirty
    end
    local activity = ActivityManager:getActivityQueue(actQueueId, mode) 
    if activity then
        self:openTabNode(tostring(actQueueId))
    else
        activity = NewActivityManager:getActivity(actQueueId)
        if activity and activity:isLevelOpen() then
            activity:updateInfo()
            self:openTabNode(tostring(actQueueId))
        end
    end
end

function This:addActTabBtn( index, info, hasParent )
    local actBtnName = "act" .. tostring(info.id)
    local rootNode = self.m_buttonList[actBtnName]
    if not rootNode then
        rootNode = cc.Sprite:create("MenuScene/huodongweidianji.png") 
        rootNode:setAnchorPoint(0.5, 0.5)
        rootNode:retain()
        self.m_buttonList[actBtnName] = rootNode
        
        local lightSprite = cc.Sprite:create("MenuScene/huodongxuanzhong.png") 
        lightSprite:setAnchorPoint(0.5, 0.5)
        lightSprite:setPosition(80, 30)
        rootNode.light = lightSprite
        rootNode:addChild(lightSprite)

        local timeLimitSprite = cc.Sprite:create("MenuScene/huodongxianshi.png") 
        timeLimitSprite:setAnchorPoint(0, 1)
        timeLimitSprite:setPosition(-5, 64)
        rootNode.timeLimit = timeLimitSprite
        timeLimitSprite:setVisible(info.time)
        rootNode:addChild(timeLimitSprite)
        
        local normalabel = cc.Label:createNormalTTF(info.name, 20)
        normalabel:setAnchorPoint(0.5, 0.5)
        normalabel:setPosition(81, 32)
        normalabel:setColor(136, 152, 190)
        rootNode.normal = normalabel
        rootNode:addChild(normalabel)

        local lightlabel = cc.Label:createNormalTTF(info.name, 20)
        lightlabel:setAnchorPoint(0.5, 0.5)
        lightlabel:setPosition(83, 34)
        lightlabel:setColor(255, 255, 255)
        rootNode.light.light = lightlabel
        rootNode.light:addChild(lightlabel)
        if not hasParent then
            self.m_csNode:addChild(rootNode)
        end
    else
        rootNode.normal:setString(info.name)
        rootNode.light.light:setString(info.name)
    end

    if hasParent then
        if self.m_btnListView then
            local w, h = rootNode:getContentSize()
            self.m_btnListView:pushNode(rootNode, w, h-6) 
            -- self.m_btnListView:pushBackCustomItem(rootNode, w, h, 0.5, 0) 
        end
    end
    rootNode:setName(actBtnName)

    local hongdian = self.m_csNode["hongdian" .. tostring(info.id)]
    if not hongdian then
        hongdian = cc.Sprite:create("LoginScene/baoman.png") 
        hongdian:setPosition(147, 42)
        hongdian:setScale(0.65)
        self.m_csNode["hongdian" .. tostring(info.id)] = hongdian
        self.m_reddotList[info.id] = hongdian
        rootNode:addChild(hongdian)
        hongdian:setVisible(false)
        if info.id == NewActivityType.kFabaoDraw then
            -- ViewUtil:refreshRedDot(hongdian, DepotManager:getItemOrResCount(261, {itemUid = 0}) > 0)
        else
            ViewUtil:refreshRedDot(hongdian, info.state)
        end
    end

    if not hasParent then
        local w, h = rootNode:getContentSize()
        local posy = self.m_bgNodeInfo.posy + self.m_bgNodeInfo.high  - (h - 1) * ( index - 0.5 ) - 7
        rootNode:setPosition(self.m_bgNodeInfo.posx + w / 2 + 5, posy)
    end
    rootNode:setVisible(true)
    return rootNode
end

function This:addTabButton(name, button, onSelected, isContainer)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end
    if self.m_tabButtons[name] then
        return
    end
    self.m_tabButtons[name] = button
    if not button.m_touch then
        button:setTouch({
            isContainer = isContainer,
            isDynamic = true,
            isScale = not isContainer,
            onEnded = function ()
                self:selectTabButton(name)
            end
        })
    end
    button.onSelected = onSelected
end

function This:addTabButtonList(orderList)
    if self.m_tabButtons then
        for _, view in pairs(self.m_tabButtons) do
            view:setVisible(false)
        end
    end

    if self.m_reddotList then
        for _, view in pairs(self.m_reddotList) do
            view:setVisible(false)    
        end    
    else
        self.m_reddotList = {}
    end

    local _, defaultActQueueInfo = next(orderList)
    if not defaultActQueueInfo then
        return
    end
    local defaultActQueueId = defaultActQueueInfo.id
    
    local hasParent = self.m_btnListView ~= nil
    for index, actQueueInfo in ipairs(orderList) do
        local actQueueId = actQueueInfo.id
        local tabName = tostring(actQueueId)
        local btn = self:addActTabBtn( index, actQueueInfo, hasParent )
        if btn then
            self:addTabButton(tostring(actQueueId), btn, function()
                if self.m_runningAniantion then
                    return false
                end
                
                self.m_curActQueueId = actQueueId
                self:selectTabById(actQueueId, true)
                return true
            end, hasParent)

            -- if index > 1 then
            --     self.m_timer:setTimer(0.3 + 0.05 * index, function()
            --         self:addTabNode(tabName)
            --     end, "ActivityPanelTab" .. tabName)
            -- end
        end
    end

    self:selectTabButton(tostring(defaultActQueueId))
end

-------------------------------------
function This:onGetActivityOrder(orderList)
    if self.m_btnListView then
        self.m_btnListView:removeAllItems()
    end
    self.m_orderList = orderList or ActivityManager:getActivityOrderList(self.m_freshType)
    self:addTabButtonList(self.m_orderList)
end

function This:onGetActivityInfo( actQueueId )
    local node = self.m_tabNodes[tostring(actQueueId)]
    if node and node.refresh then
        node:refresh()
    end
end

function This:onActRedDotChange( actQueueId, value )   
    local node = self.m_csNode["hongdian" .. tostring(actQueueId)]
    if node then
        ViewUtil:refreshRedDot(node, value)  
    end
end

function This:setIsRunning( value )
    self.m_runningAniantion = value
end

-- function This:itemUpdate(itemList)
--     if itemList and #itemList > 0 then
--         g_scene:showReward(itemList)
--     end
-- end

--endregion