-- @Author: superm
-- @Date:   2017-12-18 16:44:04
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-07 15:21:46
-- @File Name: ActivityRookiePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ActivityManager = require("Activity.ActivityManager")
local ItemNode = require("UI.ItemNode")
local ViewUtil = require("UI.ViewUtil")

function This:initNode(activity)
    self.m_inited = true
	self.m_activity = activity
 	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ActivityRecover.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
    cc.Audio:playEffect("ZuiQiangZhanLi_Decline2.mp3")	
end

function This:onEnter()
    super.onEnter(self)
    ActivityManager:addObserver(self)
    self:setLocalZOrder(g_scene:getMaxLocalZOrder() + 1)
    self:refresh()
end

function This:onExit()
	self.m_activity:removeObserver(self)
    ActivityManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.lingqu:setTouchEnded(function()
    	self.m_activity:requestPickUp()
    end)
end

function This:refreshBtn()
	self.m_activity:updateState()
	local canPickup = self.m_activity:isOpening()
    local limitNum = self.m_activity:getLimitNum()

    local curNum = self.m_activity:getQueueCurNum()
    local preNum = curNum - 1
    local leftTime = limitNum - curNum
    local preLeftTime = limitNum - preNum
    self.m_csNode.lingqu:setVisible(false)
	self.m_csNode.text:setVisible(false)
	self.m_csNode.jiantou:setVisible(false)
	self.m_csNode.num:setString(preLeftTime)
	self.m_csNode.jindu:setString(preNum .. "/" .. limitNum)
	self.m_csNode.jindutiao:setPercent(preNum / limitNum * 100)

	self.m_csNode.jindutiao:runPercentAction(curNum / limitNum * 100, 5)
	self.m_csNode.num:runAction(cc.Sequence(cc.FadeOut(0.8), cc.CallFunc(function()
		self.m_csNode.num:setString(leftTime)
	end), cc.FadeIn(0.2)))

	self.m_csNode.jindu:runAction(cc.Sequence(cc.FadeOut(0.8), cc.CallFunc(function()
    	self.m_csNode.jindu:setString(curNum .. "/" .. limitNum)
    	self.m_csNode.lingqu:setVisible(canPickup)
    	ViewUtil:blinkView(self.m_csNode.text, not canPickup)
		ViewUtil:blinkView(self.m_csNode.jiantou, not canPickup)
	end), cc.FadeIn(0.2)))

end

function This:refreshItemList()
	local rewardList = self.m_activity:getRewardList()
	local node = self.m_csNode
	if rewardList then
		for i, item in ipairs(rewardList) do
	        local itemNode = node["item" .. i]
	        itemNode:setVisible(item ~= nil)
	        if item then
	            if not itemNode.node then
	                itemNode.node = ItemNode:create(item[1], item[2])
	                itemNode.node:setAnchorPoint(0.5, 0.5)
	                itemNode:addChild(itemNode.node)
	            else
	                itemNode.node:init(item[1], item[2])
	            end
	        end
	    end    
    end
end

function This:refresh()
	if self.m_inited then
    	self:refreshBtn()
	    -- self:refreshItemList()
	end
end

function This:onPickupActReward(actQueueId, id, act)
	local queue = self.m_activity:getQueue()
	if queue:getId() ~= actQueueId then
		return
	end
	
    self.m_activity:setCompleted()

	local itemList = act:getRewardList(1)
    if itemList and #itemList > 0 then
        g_scene:showReward(itemList)
    end
	self:close()
end

function This:onTouchMask()
	if self.m_csNode.text:isVisible() then
		self:close()
	end
end

