-- @Author: superm
-- @Date:   2019-03-02 21:27:08
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-09 16:16:42
-- @File Name: ActivitySecondWeekExchangeUI.lua
-- @Description: 次周活動兌換界面的

local This, super = class(..., "Activity.ActivitySecondWeekBaseUI")

local ActivitySecondWeekExchangeItemView = require("Activity.ActivitySecondWeekExchangeItemView")
local GameUtil = require("Util.GameUtil")

function This:initNode()
	self.m_curViews = {}
    self:initTimeNode()
    self:refreshListView()
    self.m_inited = true
end

function This:onExit()
    for _, v in pairs(self.m_curViews) do
        if v and v[0] ~= 0 then
            v:release()
        end
    end
    super.onExit(self)
end

function This:refreshItemList(index)
	local activitys = GameUtil:shallowChange(self.m_activity:getAllExchangeObjects())
    self:refreshItemListByData(activitys)
end

function This:refreshItemListByData( activitys )
    local csNode = self.m_csNode
    csNode.scrollView:removeAllItems()
    csNode.scrollView:setTouchEnabled(true)
	table.sort(activitys, function( a, b )
        local state1, state2 = a:getState() , b:getState()
        if state1 ~= state2 then
            return state1 < state2
        else
            return a:getId() < b:getId()
        end
	end)

	for index, activity in ipairs(activitys) do
        activity:updateState()
        local id = activity:getId()
        local view = self.m_curViews[id]
        if not view then
			view = self:getNewItemView(activity, self.m_curSelection)
			self.m_curViews[id] = view
			view:retain()
		else
			view:refresh()
		end
	    local w , h = view:getContentSize()
	    csNode.scrollView:pushNode(view, w, h)
	end  
end

function This:initTimeNode()
end

function This:getNewItemView( ... )
    return ActivitySecondWeekExchangeItemView:create( ... )
end

function This:onExchangeReward(queueId, id, act)
    if self.m_activity:getId() ~= queueId then
        return
    end

    local view = self.m_curViews[id]
    if view then
        view:showRewardList()
        self:refreshListView()
    end
end

function This:itemUpdate()
    self:refresh()
end
