-- @Author: superm
-- @Date:   2019-03-02 10:37:32
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-03 05:54:43
-- @File Name: ActivitySecondWeekTabsUI.lua
-- @Description: 

local This, super = class(..., "Activity.ActivityBaseTabsUI")
local ActivitySecondWeekItemView = require("Activity.ActivitySecondWeekItemView")

function This:getNewItemView( ... )
    return ActivitySecondWeekItemView:create( ... )
end

function This:initNode()
    local csNode = self.m_csNode
    if not self.m_btnListView then
        local scrollview = csNode.btnList
        scrollview:setDirection("horizontal")
        scrollview.m_direction = "right"
        self.m_btnListView = scrollview
    end
    super.initNode(self)
end

function This:addActTabBtn( index )
    local nameList = self.m_activity:getGroupNameList()
    local groupBtnName = "day" .. tostring(index)
    local rootNode = self.m_buttonList[groupBtnName]
    if not rootNode then
        local showName = nameList[index]
        print(showName)
        rootNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PlayerCompetePaging.csb")
        local w, h = rootNode.bg:getContentSize()
        rootNode:setContentSize(w, h)
        self.m_buttonList[groupBtnName] = rootNode
        rootNode.normal.day:setString(Serialize[60247] .. index .. Serialize[60011])
        rootNode.normal.text:setString(showName)
        rootNode.light.day:setString(Serialize[60247] .. index .. Serialize[60011])
        rootNode.light.text:setString(showName)
        rootNode:setAnchorPoint(0.5,0.5)
    end

    if self.m_btnListView then
        local w, h = rootNode:getContentSize()
        self.m_btnListView:pushBackCustomItem(rootNode, w, h - 5, 0.5) 
    end
    rootNode:setName(groupBtnName)
    return rootNode
end

function This:addActivitySubTab()
	if not self.m_buttonList then
		self.m_buttonList = {}
	end

    local csNode = self.m_csNode    
    self.m_btnListView:removeAllItems()
	local nameList = self.m_activity:getGroupNameList()
	for index=1, self.m_groupCount do
        local btn = self:addActTabBtn( index )
        if btn then
            self:addTabButton(tostring(index), btn, function()
		    	if self.m_curTabIndex == index or tonumber(self.m_activity:getDefaultGroup()) ~= index then
		    		return false
		    	end
                self.m_highestActModel = self.m_activity:getHighestObject(index)
                self.m_highestActModel:requestRefresh()
		        self:refreshItemList(index)
		        self.m_curTabIndex = index
		        return true
            end, true)
        end

	end
    local defaultGroup = self.m_activity:getDefaultGroup()
    self:selectTabButton(defaultGroup)
    if tonumber(defaultGroup) > math.ceil(self.m_groupCount / 2) then
        self.m_btnListView:scrollToRight(0.1)
    end
end


function This:addTabButton(name, button, onSelected, isContainer)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end
    if self.m_tabButtons[name] then
        return
    end
    self.m_tabButtons[name] = button
    if not button.m_touch then
        button:setTouch({
            isContainer = isContainer,
            isDynamic = true,
            isScale = not isContainer,
            onEnded = function ()
                self:selectTabButton(name)
            end
        })
    end
    button.onSelected = onSelected
end

function This:lightTabButton(name)
    self.m_selectTabButton = name
    if self.m_tabButtons then
        for k,node in pairs(self.m_tabButtons) do
            local visible = k == name
            node.light:setVisible(visible)
            node.normal:setVisible(not visible)
        end
    end
end

function This:refreshItemListByData( activitys )
    self.m_highestActView = nil
    local csNode = self.m_csNode
    csNode.scrollView:removeAllItems()
    csNode.scrollView:setTouchEnabled(true)
    table.sort(activitys, function( a, b )
        local state1, state2 = a:getState() , b:getState()
        if state1 ~= state2 then
            return state1 < state2
        else
            return a:getId() < b:getId()
        end
    end)

    local hview = ActivitySecondWeekItemView:create(self.m_highestActModel, "winer")
    local w , h = hview:getContentSize()
    csNode.scrollView:pushNode(hview, w, h)
    self.m_highestActView = hview

    for index, activity in ipairs(activitys) do
        local id = activity:getId()
        local view = self.m_curViews[id]
        if not view then
            view = self:getNewItemView(activity, self.m_curSelection)
            self.m_curViews[id] = view
            view:retain()
        else
            view:refresh()
        end
        local w , h = view:getContentSize()
        csNode.scrollView:pushNode(view, w, h)
    end  
end

function This:onGetActivityHighestInfo(actQueueId)
    if self.m_highestActView and self.m_highestActModel and actQueueId == self.m_highestActModel:getId() then
        self.m_highestActView:refresh()
    end
end