-- @Author: superm
-- @Date:   2018-01-05 15:37:47
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-07 16:19:54
-- @File Name: ActivitySeparatedPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ActivityManager = require("Activity.ActivityManager")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

function This:initData(info)
	if info then
		if info.queue then
			self.m_queue = info.queue
		else
			self.m_queue = ActivityManager:getQueueByType(info.actType, info.index)
		end
		self.m_inited = false
	end
end

function This:initNode()
	self.m_curViews = {}
	local filename = self.m_queue:getCSFileName()
 	self.m_csNode = cc.Node:createFromCSFile(filename)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    
    local groupCount = self.m_queue:getGroupListCount()
	self:refreshExchangeItemList(groupCount)
	self:initTabsNode(groupCount)

	if self.m_queue:showTimer() then
		self.m_csNode.time:setString(Serialize[60249] .. GameUtil:formatTime(Serialize["H_M_S"], self.m_queue:getLeftTime()))
	end

	if self.m_queue:showCondition() then
		self.m_csNode.condition:setString(self.m_queue:getConditionDesc())
	end
    self.m_inited = true
end

function This:onEnter()
    super.onEnter(self)
    ActivityManager:addObserver(self)
    if self.m_groupCount then
    	self:selectTabButton(self.m_queue:getDefaultGroup())
    else
    	self:refreshBtn()
    end

	if self.m_queue:showTimer() then
		self.m_queue:addObserver(self)
	end 
end

function This:onExit()
	if self.m_curViews then
		for _, view in pairs(self.m_curViews) do
			if view and view[0] ~= 0 then
				view:release()
			end
		end
	end

	self.m_queue:removeObserver(self)
    ActivityManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    if self.m_csNode.lingqu then
	    self.m_csNode.lingqu:setTouchEnded(function()
	    	self.m_queue:requestPickUp()
	    end)
	end

	if self.m_csNode.Recharge then
		self.m_csNode.Recharge:setTouchEnded(function()
			if not self.m_curTabIndex then
				g_scene:openDialog("System.RechargeMainUI")
			else
				if self.m_queue.getCost then
					local cost, itemId = self.m_queue:getCost(self.m_curTabIndex)
					g_scene:openDialog("UI.CostConfirmPanel", {
		                cost = cost,
		                costId = itemId,
		                desc = self.m_queue:getCostDesc(), 
		                okHandler = function()
		                    self.m_queue:requestRecharge(self.m_curTabIndex)
		                end }
		            )
				else
					self.m_queue:requestRecharge(self.m_curTabIndex)
				end
			end
		end)		
	end
end

function This:initTabsNode( groupCount )
	if groupCount > 0 then
		for i=1, groupCount do
			local button = self.m_csNode["tab" .. i]
		    self:addTabButton(tostring(i), button, function()
		    	if button.isLocked and button:isLocked(self.m_queue) then
		    		button:showLockWarming(self)
		    		return false
		    	end

		    	if self.m_curTabIndex == i then
		    		return false
		    	end
		        self.m_curTabIndex = i
				self:refreshItemList(i)
                self:refreshBtn()
		        return true
		    end)
		    if not button.m_touch then
	            button:setTouch({
	                onEnded = function ()
	                    self:selectTabButton(tostring(i))
	                end,
	                isScale = false,
	            })
	        end
		end		
		self.m_groupCount = groupCount

	else
    	self:refreshRewardItemList()
	end
end

function This:refreshBtn()
	local number = self.m_queue:getProgressByIndex(self.m_curTabIndex)
	if number then
		if self.m_csNode.Recharge then 
			self.m_csNode.Recharge:setVisible(false)
		end
		
		if self.m_csNode.lingqu then 
			self.m_csNode.lingqu:setVisible(true)
		end
	else
		if self.m_csNode.Recharge then 
			self.m_csNode.Recharge:setVisible(true)
		end

		if self.m_csNode.lingqu then   
			self.m_csNode.lingqu:setVisible(false)
		end
	end
end

function This:refreshExchangeItemList(groupCount)
	if groupCount <= 0 then
		return
	end

	if not self.m_queue:hasExchangeBehaviour() then
		return
	end

	if not self.m_queue:showExchangeBehaviour() then
		return 
	end

	local exchangeModelList = self.m_queue:getAllExchangeObjects()
	if exchangeModelList then
		local actType = self.m_queue:getType()
		for i, model in ipairs(exchangeModelList) do
			local class = require(model:getUITypeName())
			if class and class ~= true then
				local node = class:create(model)
				local w = node:getContentWidth()
				node:setPositionX(( w + 17 ) * ( i - 1 ))
				self.m_csNode.typeBg:addChild(node)
				self.m_csNode.typeBg["tab" .. tostring(actType) .. tostring(i)] = node
    			node:setName("tab" .. tostring(actType) .. tostring(i))
				self.m_csNode["tab" .. tostring(i)] = node
			end
		end
	end
end

function This:refreshRewardItemList()
	local rewardList = self.m_queue:getRewardList()
	local node = self.m_csNode
	if rewardList then
		for i, item in ipairs(rewardList) do
	        local itemNode = node["item" .. i]
	        itemNode:setVisible(item ~= nil)
	        if item then
	            if not itemNode.node then
	                itemNode.node = ItemNode:create(item[1], item[2])
	                itemNode.node:setAnchorPoint(0.5, 0.5)
	                itemNode:addChild(itemNode.node)
	            else
	                itemNode.node:init(item[1], item[2])
	            end
	        end
	    end    
    end
end

function This:refreshItemList( index )
	if index then
		if self.m_curTabIndex == nil then
			return 
		end
	    local groupList = self.m_queue:getGroupListBy( index )
	    self:refreshItemListByData(groupList)
	else
		local activitys = GameUtil:shallowChange(self.m_queue:getAllObjects())
		if table.count(activitys) > 0 then
			self:refreshRewardItemList()
	    else
	    	self:refreshItemListByData(activitys)
	    end
	end
end

function This:refreshItemListByData( activitys )
    self.m_csNode.scrollView:removeAllItems()
    self.m_csNode.scrollView:setTouchEnabled(true)
	table.sort(activitys, function( a, b )
        local state1, state2 = a:getState() , b:getState()
        if state1 ~= state2 then
            return state1 < state2
        else
            return a:getId() < b:getId()
        end
	end)

	local uiName = "Activity.Activity" .. self.m_queue:getTypeName() .. "ItemView"
	for index, activity in ipairs(activitys) do
        local id = activity:getId()
        local view = self.m_curViews[id]
        if not view then
        	local class = require(uiName)
        	if class and class ~= true then
				view = class:create(activity)
				self.m_curViews[id] = view
				view:retain()
			end
		else
			view:refresh()
		end
		view:setDialog(self)
	    local w , h = view:getContentSize()
	    self.m_csNode.scrollView:pushNode(view, w, h)
	end  
end

function This:onPickupActReward(actQueueId, id, act)
	if self.m_queue:getId() ~= actQueueId then
		return
	end

	if self.m_curTabIndex then
		local view = self.m_curViews[id]
		if view then
			view:showRewardList()
			self:refreshItemList(self.m_curTabIndex)
		end
	else
	    self.m_queue:setCompleted()
        if act then
    	    local itemList = act:getRewardList(1)
	        if itemList and #itemList > 0 then
	            g_scene:showReward(itemList)
	        end
        end
	    ActivityManager:notifyActState(self.m_queue, false)
		self:close()		
	end
end

function This:onDoActivateAct( queueId, ctype )
	g_scene:showTips(self.m_queue:getActivateDesc())
	self:refreshItemList(self.m_curTabIndex)
    self:refreshBtn()    
end

function This:updateQueueTime( info )
    self.m_csNode.time:setString(Serialize[60249] .. GameUtil:formatDTime(Serialize["activity_BaseTabs_1_c"] ,info.time))
end

function This:onGetActivityInfo( actQueueId, actType )
    if actType == self.m_queue:getType() and actQueueId == self.m_queue:getId() then
	    if self.m_groupCount then
	    	self:selectTabButton(self.m_queue:getDefaultGroup())
	    else
	    	self:refreshBtn()
	    end
    end
end