
local GameSetting = require("GameSetting")

AppLanguage = {
	zh_CN = 'zh_CN', --简体中文
	zh_TW = 'zh_TW', --繁体中文
	en_US = 'en_US', --美式英语
}

LangTypeText = {
	zh_CN = '简体中文',
	zh_TW = '繁體中文',
	en_US = 'English',
}

AppText = {
	m_language = AppLanguage.zh_TW,
	m_replaceDict = {},
	m_texts = nil,
}


function AppText:init()
	self.m_language = GameSetting:get("language") or getNativeLanguage()

	local dict = {}
	for _, config in pairs( self:getTextConfig() ) do
		for key, value in pairs(config) do
			local fileName = 'Metadatas.' .. value.path[1]
			if not dict[fileName] then
				dict[fileName] = {}
			end
			table.insert(dict[fileName], value)
		end
	end
	self.m_replaceDict = dict
end

function AppText:getTextConfig()
	if not self.m_texts then
		self.m_texts = {
			[1] = require("Metadatas.TextConfig1"),
			[2] = require("Metadatas.TextConfig2"),
			[3] = require("Metadatas.TextConfig3"),
		}
	end
	return self.m_texts
end

-- 设置语言
function AppText:setLanguage(langType)
	self.m_language = langType or AppLanguage.zh_CN
	GameSetting:set("language", langType)
	GameSetting:save()
	require("Login.LoginManager"):switchAccount()
end

-- 获取语言
function AppText:getLanguage()
	return self.m_language
end

-- 获取本地化文字
function AppText:getText(key)
	local texts = self:getTextConfig()
	for _, config in pairs(texts) do
		if config[key] then
			local str = config[key][self.m_language]
			if not str or str == "" then
				str = config[key][AppLanguage.zh_CN]
			end
			return str
		end
	end
end

-- 替换配置表文字
function AppText:replaceConfigText(filePath, config)
	local list = self.m_replaceDict[filePath]
	if list then
		for i, v in ipairs(list) do
			local parent = config
			local path = v.path
			local length = #path

			for i=2, length do
				if i >= length then
					break
				end
				parent = parent[path[i]]
				if not parent then
					break
				end
			end

			if parent then
				local str = v[self.m_language]
				if not str or str == "" then
					str = v[AppLanguage.zh_CN]
				end
				parent[path[length]] = str
			end
		end
	end
end

-- 替换已加载的配置表文字
function AppText:replaceLoadedConfigText()
	for filePath, list in pairs(self.m_replaceDict) do
		if package.loaded[filePath] then
			self:replaceConfigText(filePath, package.loaded[filePath])
		end
	end
end

function AppText:onConfigLoaded(filePath, config)
	self:replaceConfigText(filePath, config)
end

-- 翻译服务器发下来的文字
function AppText:getServerText(name)
	name = name or ""
	local config = require("Metadatas.ServerTextConfig")
	if config[name] then
		return config[name][self.m_language]
	end
	return name
end

-- 是否为中文
function AppText:isChinese()
	return self.m_language == AppLanguage.zh_CN or self.m_language == AppLanguage.zh_TW
end

-- 是否为英文
function AppText:isEnglish()
	return self.m_language == AppLanguage.en_US
end

-- 语言选项
function AppText:getLanguageOptions(isRemoveCur)
	local list = {}

	for _, id in pairs(AppLanguage) do
		table.insert(list, {id = id, text = LangTypeText[id]})
	end

	if isRemoveCur then
		for i, v in ipairs(list) do
			if v.id == self.m_language then
				table.remove(list, i)
			end
		end
	end

	return list
end

-- 选项文字
function AppText:getLangTypeText(id)
	return LangTypeText[id]
end


return AppText