-- @Author: superm
-- @Date:   2018-05-08 14:17:09
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-22 14:53:13
-- @File Name: ArenaHelpTipsPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local ArenaManager = require("Arena.ArenaManager")
local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")
local ArenaRewardConfig = require("Metadatas.ArenaRewardConfig")

local kShowRewardRankLevel = 2000

function This:initNode()
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ArenaMainHelp.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
end

function This:refreshBaseInfo()
	local maxRankIndex = ArenaManager:getRankIndexInHistory()
	local topVisible = maxRankIndex < 4 and maxRankIndex > 0
	self.m_csNode.rank.number:setVisible(not topVisible)
	self.m_csNode.rank.top3:setVisible(topVisible)
	if topVisible then
		self.m_csNode.rank.top3:setImage("MenuScene/top" .. maxRankIndex .. ".png")
	elseif maxRankIndex > 0 then
		self.m_csNode.rank.number:setString(tostring(maxRankIndex))
	else
		self.m_csNode.rank.number:setString(Serialize["Arena_HelpTips_1_c"])
	end 

	local rankIndex = ArenaManager:getRankIndex()
	if rankIndex > 0 and rankIndex <= kShowRewardRankLevel  then
		self.m_csNode.text:setString(string.format("%d", rankIndex))
		self.m_csNode.jiangli:setString(Serialize[60280])
	else
		if rankIndex <= 0 then
			self.m_csNode.text:setString(Serialize[60238])
		else
			self.m_csNode.text:setString(string.format("%d", rankIndex))
		end
		self.m_csNode.jiangli:setString(string.format(Serialize[60281], kShowRewardRankLevel))
	end
	self.m_csNode.rank:setVisible(rankIndex > 0)
	GameUtil:linearLayoutNodes(10, {self.m_csNode.paiming, self.m_csNode.text, self.m_csNode.jiangli}, false, self.m_csNode.Image:getContentWidth())

	local index, lastId
	for _, info in pairs(ArenaRewardConfig) do
		-- print_tmp({rankIndex = rankIndex, id = _, imin = info.min, imax = info.max})
		if not index and info.min >= kShowRewardRankLevel then
			index = info.id
			lastId = info.id
		end
		
		if info.max <= rankIndex and info.min >= rankIndex then
			index = info.id
			break
		end
	end

	local rewardList = ArenaRewardConfig[index or lastId].rewardlist
	local count = 0
	for i, info in ipairs(rewardList) do
		local id, num = info[1], info[2]
		local node = self.m_csNode["reward" .. i]

		if not node then break end

		if not node.itemNode then
			local itemNode = ItemNode:create(id, num, false, true)
			itemNode:setAnchorPoint(0.5, 0.5)
			itemNode:setScale(0.9)
			node:addChild(itemNode)
			node.itemNode = itemNode
		else
			node.itemNode:initItem(id, num, true)
		end 
		node:setVisible(true)
		count = count + 1
	end
	for i= count + 1, 6 do
		local node = self.m_csNode["reward" .. i]
		node:setVisible(false)
	end

	self.m_csNode.rule:setString(Serialize["Arena_HelpTips_2"])
end

function This:initTouchEvent()	
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
end

function This:onEnter()
	super.onEnter(self)
	self:refreshBaseInfo()
	ArenaManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	ArenaManager:removeObserver(self)
end

function This:onRefreshArenaInfo()
	self:refreshBaseInfo()
end
