--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local BasePlayer = require("Player.BasePlayer")
local GamePlayer = require("Player.GamePlayer")
local EventDispatcher = require("EventDispatcher")
local HeroManager = require("Hero.HeroManager")
local ArenaTimesConfig = require("Metadatas.ArenaTimesConfig")
local BattleDataManager = require("Battle.BattleDataManager")
local GameSetting = require("GameSetting")
local GameUtil = require("Util.GameUtil")

local PrivilegeManager = require("Vip.PrivilegeManager")

local kMaxFreeChallengeTimes = 5
-- kMaxBuyChallengeTimes = 5
local kBuyOnceTimesNum = 5
kMaxRankIndex = 2000 --竞技场最大排名
kCostClearChallengeCD = 30 --清除CD价格 1元宝/10秒

function This:init()
	super.init(self)
	self.m_rankIndex = 0
	self.m_maxRankIndex = 0
	self.m_cardList = {}
	self.m_power = 0
	self.m_challengeTimes =0
	self.m_buyTimes = 0
	self.m_nextChanlengeTime = 0
	self.m_challengePlayers = {}
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self,12200,self.onGetArenaInfoReply)
	EventDispatcher:addListener(self,12201,self.onRefreshChallengePlayerReply)
	EventDispatcher:addListener(self,12202,self.onSetCardListReply)
	EventDispatcher:addListener(self,12203,self.onBuyChallengeTimesReply)
	EventDispatcher:addListener(self,12204,self.onGetArenaRankInfoReply)
	EventDispatcher:addListener(self,12205,self.onChallengeArenaReply)
	EventDispatcher:addListener(self,12211,self.onClearChallengeCDReply)
	EventDispatcher:addListener(self,12212,self.onGetFirstRankRoleReply)
	-- EventDispatcher:addListener(self,"day_passed",self.onDayPassNotify)
end

function This:getRankIndex()
	return self.m_rankIndex
end

function This:getRankIndexInHistory()
	self.m_maxRankIndex = math.min(self.m_rankIndex, self.m_maxRankIndex)
    return self.m_maxRankIndex
end

function This:getPower()
	return self.m_power
end

function This:hasDefendCards()
	for i , id in ipairs(self.m_cardList) do
		if id > 0 then
			return true;
		end
	end
	return false
end

function This:getLeftChallengeTimes()
	local left = kMaxFreeChallengeTimes + self.m_buyTimes * kBuyOnceTimesNum  - self.m_challengeTimes
	return math.max(0,left)
end

function This:getCostForBuyChallengeTime()
	local times = math.min(self.m_buyTimes+1,#ArenaTimesConfig)
	return ArenaTimesConfig[times].reset_cost
end

function This:getBuyTimes()
	return self.m_buyTimes or 0
end

function This:getLeftChallengeCd()
	local curTime = GamePlayer:getServerTime()
	return math.max(0,self.m_nextChanlengeTime - curTime)
end

function This:getChallengePlayer(index)
	return self.m_challengePlayers[index]
end

function This:getDefenderCardList()
	return self.m_cardList
end

function This:getDefenderCardIds()
	local result = {}
	for pos , id in pairs(self.m_cardList) do
		if id > 0 then
			local hero = HeroManager:getHeroByUid(id)
			result[pos] = hero:getId()
		end
	end
	return result
end

function This:getFirstPlayer()
	return self.m_firstPlayer
end

function This:isArenaOpened( withWarming )
    return PrivilegeManager:hasPrivilege( Privilege.kArenaUnlock, withWarming )
end

-------------------------------------------------------------
--查看竞技场排名
function This:onGetArenaInfoRequest()
	Socket:sendAndLock(12200,{})
end

function This:onGetArenaInfoReply(msg)
	if msg.err_code == 0 then
		-- 本地化机器人名字
		if msg.challenge_member_list then
			for i, v in ipairs(msg.challenge_member_list) do
				v.name = AppText:getServerText(v.name)
			end
		end

		self.m_rankIndex = msg.rank_id
		self.m_maxRankIndex = msg.max_rank_id
		self.m_power = msg.power
		self.m_cardList = msg.card_list
		self.m_buyTimes = msg.buy_times 
		self.m_challengeTimes = msg.challenge_times
		self.m_nextChanlengeTime = msg.next_refresh_time
		self.m_challengePlayers = {}
		for i , info in ipairs(msg.challenge_member_list) do
			local player = BasePlayer:create(info)
			table.insert(self.m_challengePlayers,player)
		end

		--[[local playerId = GamePlayer:getPlayerID()
		local key = GameUtil:int64ToString(playerId)
		local attackList = cc.FileUtils:loadLuaFile("arena") or {}]]

		self:notifyObservers("onRefreshArenaInfo")
		EventDispatcher:dispatcher("refresh_arena_info")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--刷新NPC
function This:onRefreshChallengePlayerRequest()
	Socket:sendAndLock(12201,{})
end

function This:onRefreshChallengePlayerReply(msg)
	if msg.err_code == 0 then
		-- 本地化机器人名字
		if msg.challenge_member_list then
			for i, v in ipairs(msg.challenge_member_list) do
				v.name = AppText:getServerText(v.name)
			end
		end

		self.m_challengePlayers = {}
		for i , info in ipairs(msg.challenge_member_list) do
			local player = BasePlayer:create(info)
			table.insert(self.m_challengePlayers,player)
		end
		self:notifyObservers("onRefreshChallengePlayer")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--设置阵容
function This:onSetCardListRequest(cardList)
	Socket:sendAndLock(12202,{card_list = cardList})
end

function This:onSetCardListReply(msg)
	if msg.err_code == 0 then
		self.m_cardList = msg.card_list
		self.m_rankIndex = msg.rank_id
		self.m_maxRankIndex = msg.max_rank_id
		self:notifyObservers("onSetCardList",self.m_cardList)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--购买挑战次数
function This:onBuyChallengeTimesRequest()
	Socket:sendAndLock(12203,{buy_time = 1})
end

function This:onBuyChallengeTimesReply(msg)
	if msg.err_code == 0 then
		self.m_buyTimes = msg.buy_time
		self.m_challengeTimes = msg.challenge_time
		self:notifyObservers("onBuyChallengeTimes")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--查看竞技排行榜
function This:onGetArenaRankInfoRequest(startIndex, endIndex)
	print( "竞技排行榜 startIndex:" .. startIndex, "endIndex:" .. endIndex )
	Socket:sendAndLock(12204,{start_index = startIndex, end_index = endIndex})
end

function This:onGetArenaRankInfoReply(msg)
	if msg.err_code == 0 then
		self:notifyObservers("onGetArenaRank", msg.start_index, msg.end_index, msg.challenge_member_list)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--挑战竞技场

function This:onChallengeArenaRequest(id,rankIndex,cardList)
	Socket:sendAndLock(12205,{rank_id = rankIndex, role_id = id, card_list = cardList})
end

function This:onChallengeArenaReply(msg)
	if msg.err_code == 0 then
		-- 本地化机器人名字
		if msg.challenge_member_list then
			for i, v in ipairs(msg.challenge_member_list) do
				v.name = AppText:getServerText(v.name)
			end
		end

		self.m_battleResult = {
			oldRank = self.m_rankIndex,
			oldMaxRank = self.m_maxRankIndex,
			curRank = msg.rank,
			maxRank = msg.max_rank,
			rewardList = msg.reward_list,
		}
		self.m_rankIndex = msg.rank
		self.m_maxRankIndex = msg.max_rank
		self.m_challengeTimes = msg.challenge_time 
		self.m_nextChanlengeTime = msg.next_refresh_time
		self.m_challengePlayers = {}
		for i , info in ipairs(msg.challenge_member_list) do
			local player = BasePlayer:create(info)
			table.insert(self.m_challengePlayers,player)
		end
		cc.SceneManager:switchTo("Arena.ArenaLoadingScene",nil,{
			msg = msg
		})   
		BattleDataManager:addObserver(self)
		self:notifyObservers("onChallengeArena")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onClearChallengeCDRequest()
	Socket:sendAndLock(12211,{})
end

function This:onClearChallengeCDReply(msg)
	if msg.err_code == 0 then
		self.m_nextChanlengeTime = msg.next_challenge_time
		self:notifyObservers("onClearChallenge")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetFristRankRoleRequest()
	Socket:sendAndLock(12212,{rank_id = 1})
end

function This:onGetFirstRankRoleReply(msg)
	if msg.err_code == 0  then
		local info = msg.challenge_member_list[1]
		self.m_firstPlayer = BasePlayer:create(info)
		self:notifyObservers("onGetArenaPlayerInfo",self.m_firstPlayer)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-------------------------------------------------------------

function This:onBattleVictory()
	BattleDataManager:removeObserver(self)
	g_scene:openDialog("Arena.ArenaVictoryPanel",self.m_battleResult)
end

function This:onBattleFailed()
	BattleDataManager:removeObserver(self)
	g_scene:openDialog("Arena.ArenaFailedPanel",self.m_battleResult)
end

function This:onBattleSkip()
	BattleDataManager:removeObserver(self)
end

function This:onDayPassNotify()
	self:onGetArenaInfoRequest()
	self:onGetFristRankRoleRequest()
end

--endregion
