--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local ArenaManager = require("Arena.ArenaManager")
local Application = require("cc.Application")
local GameUtil = require("Util.GameUtil")
local ArenaPlayerView = require("Arena.ArenaPlayerView")
local ArenaHeroView = require("Arena.ArenaHeroView")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local ArenaTimesConfig = require("Metadatas.ArenaTimesConfig")
local ReportManager = require("Report.ReportManager")
local ArenaReportPanel = require("Report.ArenaReportPanel")
local ReportManager = require("Report.ReportManager")
local PrivilegeManager = require("Vip.PrivilegeManager")

local LastRefreshTime = 0
local kRefreshPlayerCD = 5

function This:initNode(info)
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ArenaMain.csb")
	if GamePlayer:isAuditVersion() then
		GameUtil:hideNodeList(self.m_csNode, {"button3"});
	end
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self.m_cardNodes = {}
	for i = 1 , 6 do
		local view = ArenaHeroView:create(nil)
		self.m_csNode["hero"..i]:addChild(view)
		self.m_cardNodes[i] = view
	end

	self.m_playerNodes = {}
	self.m_escape = 0
	self.m_isUpdating = false
	ArenaManager:onRefreshChallengePlayerRequest()

	if info and info.subview then
		self.m_subDialogInfo = info.subview
		self:addOpenCallBack(function()
			self:openDialog(self.m_subDialogInfo.name, self.m_subDialogInfo.arg)
		end)
	end
end

function This:refreshBaseInfo()
	local rankIndex = ArenaManager:getRankIndex()
	if rankIndex > 0 then
		self.m_csNode.rank:setString(rankIndex)
	end
	self.m_csNode.noin:setVisible(rankIndex <= 0)
	self.m_csNode.rank:setVisible(rankIndex > 0)

	local leftChallengeTimes = ArenaManager:getLeftChallengeTimes()
	self.m_csNode.num:setString(leftChallengeTimes)
end

function This:refreshChallengeCD()
	local challengeCd = ArenaManager:getLeftChallengeCd()
	if challengeCd > 0 and not self.m_isUpdating then
		self.m_isUpdating = true
	elseif challengeCd <= 0 and self.m_isUpdating then
		self.m_isUpdating = false
	end
	if challengeCd > 0 then
		self.m_csNode.time:setString(GameUtil:formatTime(Serialize["Arena_ArenaPanel_1"],challengeCd))
	end
	self.m_csNode.time:setVisible(challengeCd > 0)
end

function This:getPlayerNode(index)
	local player = ArenaManager:getChallengePlayer(index)
	local view = ArenaPlayerView:create(player)
	view:setChallengeHandler(function(playerId)
		
	end)
	return view
end

function This:refreshPlayerInfo()
	for i = 1 , 3 do
		if not self.m_playerNodes[i] then
			local view = self:getPlayerNode(i)
			self.m_csNode["player"..i]:addChild(view)
			self.m_playerNodes[i] = view
		else
			local player = ArenaManager:getChallengePlayer(i)
			self.m_playerNodes[i]:setModel(player)
		end
	end
end

function This:refreshCardInfo()
	local cardList = ArenaManager:getDefenderCardList()
	local fightPower = 0
	for i = 1 , 6 do
		if #cardList >= i and cardList[i] > 0 then
			local hero = HeroManager:getHeroByUid(cardList[i])
			local info = {id = hero:getUid(), level = hero:getLevel(), star = hero:getStarLv() * 5 + hero:getPhase(), heroId = hero:getId(),awake = hero:isAwakeFull()}
			self.m_cardNodes[i]:setModel(info)
			fightPower = fightPower + hero:getFightPower()
		else
			self.m_cardNodes[i]:setModel(nil)
		end
	end
	self.m_csNode.allfight:setString(math.floor(fightPower))
end

function This:initTouchEvent()	
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.button1:setTouchEnded(function()
		g_scene:openDialog("Arena.ArenaHelpTipsPanel")
	end)

	self.m_csNode.button2:setTouchEnded(function()
		-- print("竞技商店")
		g_scene:openDialog("Store.StoreArenaPanel")
	end)
	self.m_csNode.button3:setTouchEnded(function()
		-- print("经济排行")
        g_scene:openDialog("Arena.ArenaRankListPanel")
	end)
	self.m_csNode.button4:setTouchEnded(function()
		print("战斗记录")
		ReportManager:onGetArenaBattleReportRequest()
	end)
	self.m_csNode.add:setTouchEnded(function()
		if ArenaManager:getLeftChallengeTimes() > 0 then
			g_scene:showTips(Serialize["Arena_ArenaPanel_2"])
			return
		end
		local buyTimes = ArenaManager:getBuyTimes()
		if PrivilegeManager:isPrivilegeNumEnough(Privilege.kArenaBuyTimes, buyTimes, true) then
			local cost = ArenaManager:getCostForBuyChallengeTime()
			GamePlayer:costDiamond({
				cost = cost,
				okHandler = function()
					ArenaManager:onBuyChallengeTimesRequest()
				end,
				desc = Serialize["Arena_ArenaPanel_3"]
			})
		end
	end)
	self.m_csNode.adjustment:setTouchEnded(function()
		print("阵容调整")
		g_scene:openDialog("Arena.ArenaEmbattlePanel",{isDefender = true})
	end)
	self.m_csNode.change:setTouchEnded(function()
		ArenaManager:onRefreshChallengePlayerRequest()
	end)
end

function This:onEnter()
	super.onEnter(self)
	self:refreshBaseInfo()
	self:refreshChallengeCD()
	self:refreshPlayerInfo()
	self:refreshCardInfo()
	ArenaManager:addObserver(self)
	ReportManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	ArenaManager:removeObserver(self)
	ReportManager:removeObserver(self)
end

function This:onRefreshArenaInfo()
	self:refreshBaseInfo()
	self:refreshChallengeCD()
	self:refreshPlayerInfo()
	self:refreshCardInfo()
end

function This:onRefreshChallengePlayer()
	self:refreshPlayerInfo()
	g_scene:showTips(Serialize["Arena_ArenaPanel_4"])
end

function This:onSetCardList()
	self:refreshCardInfo()
	self:refreshBaseInfo()
end

function This:onBuyChallengeTimes()
	self:refreshBaseInfo()
	self:refreshChallengeCD()
end

function This:onChallengeArena()
	self:refreshBaseInfo()
	self:refreshChallengeCD()
end

function This:onClearChallenge()
	self:refreshChallengeCD()
end

function This:onGetArenaBattleReport(reportList)
	g_scene:openDialog("Report.ArenaReportPanel",{reportList = reportList})
end

function This:update(dt)
	super.update(self,dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return 
	end
	self.m_escape = self.m_escape - 1
	if self.m_isUpdating then
		self:refreshChallengeCD()
	end
end

--endregion
