-- @Author: superm
-- @Date:   2018-03-29 22:07:56
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-03 14:19:32
-- @File Name: ArenaRankListPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local ArenaManager = require("Arena.ArenaManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local ArenaRankListItemView = require("Arena.ArenaRankListItemView")

function This:initData()
	self.m_inited = false
	self:resetRankListData()
end

function This:initNode()
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ArenaMainRanking.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 748,
        itemHeight = 57,
        spaceX = 0,
        spaceY = 2,
        scrollView = self.m_csNode.ScrollView,
		dataCounter = function ()
			local count = 0
	        if self.m_endIndex then
	        	count = self.m_endIndex
	        end
	        return count
        end,
        viewCreator =  function(index, node)
        	if not self.m_allRankItemList then
        		return
        	end

        	local count, data = 0, nil
	        for pageIndex, num in pairs(self.m_allRankItemCountList) do
	        	local newCount = count + num
	        	if newCount >= index then
	        		data = self.m_allRankItemList[pageIndex][index - count]
	        		break
	        	end
	        	count = newCount
	        end
	        
	        if data then
		        if not node then
			        return ArenaRankListItemView:create(data, index)
			    else
		        	node:setModel(data, index)
		        end
		    end
	    end
    })
    
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.m_csNode.ScrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)
end

function This:initTouchEvent()
	if not self.m_inited then
		self.m_csNode.close:setTouchEnded(function()
			self:close()
		end)
		self.m_inited = true
	end
	ArenaManager:onGetArenaRankInfoRequest(kRankItemStartIndex, kRankItemEndIndex * 5)
end

function This:onEnter()
	super.onEnter(self)
	ArenaManager:addObserver(self)
end

function This:onExit()
	ArenaManager:removeObserver(self)
	super.onExit(self)
end

function This:resetRankListData()
	self.m_allRankItemList = {}
	self.m_allRankItemCountList = {}
	self.m_curPageNum = 0
	self.m_startIndex = 0
	self.m_endIndex = 0
	self.m_selfIndex = 0
end

function This:refresh()
	if self.m_inited then
		self:refreshTitleView()
		self:refreshSelfRankView()
		self:refreshRankListView()
	end
end

function This:refreshTitleView()
end

function This:refreshSelfRankView()
end

function This:refreshRankListView()
    self.m_listLayer:setDataList()
end

function This:onGetArenaRank( startIndex, endIndex, memberListData )
    self.m_locked = false
	local needClear = false
	if startIndex <= kRankItemStartIndex then
		needClear = true
		self:resetRankListData()
	elseif startIndex == self.m_startIndex then
		return 
	end
	-- print_r(memberListData)
	local count = #memberListData
	if count > 0 then
		local newStartIndex = self.m_endIndex or kRankItemStartIndex
		local curEndIndex = self.m_endIndex or 0
		
		self.m_startIndex = newStartIndex
		self.m_endIndex = curEndIndex + count
		local curPage = math.ceil(self.m_endIndex/kRankItemNum)
		if curPage ~= self.m_curPageNum then
			self.m_curPageNum = curPage
		end
		self.m_allRankItemList[self.m_curPageNum] = memberListData
		self.m_allRankItemCountList[self.m_curPageNum] = count

		if not needClear then
			self.m_listLayer:updateByDataList()
		else
			self:refresh()
		end
	end
end