

local This, super = class(...,"Dialog")
local ArmyTacticalItemView = require("ArmyTactical.ArmyTacticalItemView")
local ArmyTacticalManager = require("ArmyTactical.ArmyTacticalManager")
local ArmyTactAttrItemView = require("ArmyTactical.ArmyTactAttrItemView")
local RankPageListLayer = require("Rank.RankPageListLayer")

function This:initData()
	self.m_array = {}
	self.m_arrayNodes = {}
	self.m_curAttr = {}
	self.m_maxAttr = {}
	self.m_listLayers = {}

	self:initArmyTactList()
	self:initCurAttrList()
	self:initMaxAttrList()
end

function This:initArmyTactList()
	local all = ArmyTacticalManager:getAllArmyTactical()
	self.m_array = {}

	for k, at in pairs(all) do
		table.insert(self.m_array, at)
	end

	table.sort(self.m_array, function (a, b)
		return a:getId() < b:getId()
	end)
end

function This:initCurAttrList()
	local curAttr = ArmyTacticalManager:getAttr()
	self.m_curAttr = self:formatAttr(curAttr, "A1")
end

function This:initMaxAttrList()
	local all = ArmyTacticalManager:getAllArmyTactical()
	local maxAttr = {}

	local mergeAttr = function(mainTb, subTb)
		for k, v in pairs(subTb) do
			if not mainTb[k] then
				mainTb[k] = 0
			end
			mainTb[k] = mainTb[k] + v
		end
		return mainTb
	end

	for _, at in pairs(all) do
		local levelConfig = at:getLevelConfig( at:getMaxLevel() )
		local stageConfig = at:getStageConfig( at:getMaxStage() )
		local rate = stageConfig.promote_percentage/100
		local attr = {}

		for i, v in ipairs(levelConfig.levelup_arg) do
			attr[v[1]] = v[2] * rate
		end

		mergeAttr(maxAttr, attr)
	end

	self.m_maxAttr = self:formatAttr(maxAttr, "New18")
end

function This:formatAttr(attribute, colorId)
	local temp = {}
	for k, v in pairs(attribute) do
		table.insert(temp, {k, v})
	end
	if #temp > 1 then
		table.sort(temp, function (a, b)
			return a[1] < b[1]
		end)
	end

	local list = {}

	for i, attr in ipairs(temp) do
		local attrId = attr[1]
		local attrVal = attr[2]
		local desc = ""

		if AttrInfo[attrId].valueType == ValueType.kPercent then
			desc = string.format(Serialize[60297], colorId, AttrName[attrId], attrVal*100)
		else
			desc = string.format(Serialize[60298], colorId, AttrName[attrId], math.floor(attrVal))
		end

		table.insert(list, desc)
	end

	return list
end

function This:initNode()
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/kingWayArrayValue.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

	local scrollview = csNode.scrollview
    scrollview:setDirection("horizontal")
	scrollview.m_direction = "right"
	self:refreshArray()


	local listConfig = {
		[1] = {
			scrollview = csNode.scrollview1,
			listData = self.m_curAttr,
		},
		[2] = {
			scrollview = csNode.scrollview2,
			listData = self.m_maxAttr,
		}
	}

	for i, v in ipairs(listConfig) do
		local scrollview = v.scrollview
		local listData = v.listData

		scrollview:setDirection("vertical")

	    local listLayer = RankPageListLayer:create({
	        itemWidth = 200,
	        itemHeight = 28,
	        spaceX = 0,
	        spaceY = 0,
	        scrollView = scrollview,
			dataCounter = function ()	 
		        return #listData
	        end,
	        viewCreator =  function(index, node) 
	        	local data = listData[index]
	        	if data then     
				    if not node then
					    return ArmyTactAttrItemView:create(data, index)
					else
				        node:setModel(data, index)
				    end
				end
		    end
	    })
		listLayer:setCascadeOpacityEnabled(true)
	    listLayer:setAnchorPoint(0.5, 0)
	    local w, h = listLayer:getContentSize()
	    scrollview:pushBackCustomItem(listLayer, w, h, 0)
	    listLayer:setDataList()
		self.m_listLayers[i] = listLayer		
	end
end

function This:initTouchEvent()
	if not self.m_inited then
		self.m_inited = true

		local csNode = self.m_csNode
        local scrollview = csNode.scrollview

		csNode.close:setTouchEnded(function()
        	self:close()
    	end)
		
		-- 左箭头
        self.m_move1 = false
        csNode.jiantou1:setTouchEnded(function ()
        	if self.m_move1 then
        		return
        	end

        	local x, y = scrollview:getInnerContainerPosition()
        	if x >= 0 then
        		return
        	end

        	local w, h = scrollview:getContentSize()
        	local iw, ih = scrollview:getInnerContainerSize()
        	
        	x = x + w
        	if x > 0 then
        		x = 0
        	end

        	local percent = math.abs(x)/(iw-w)*100
            scrollview:scrollToPercentHorizontal(percent, 0.2, false)
            self.m_move1 = true

            self:setTimer(0.2, function ()
            	self.m_move1 = false
            end, "army_tact_move_1")
        end)

        -- 右箭头
        self.m_move2 = false
        csNode.jiantou2:setTouchEnded(function ()
            if self.m_move2 then
        		return
        	end

        	local x, y = scrollview:getInnerContainerPosition()
        	local w, h = scrollview:getContentSize()
        	local iw, ih = scrollview:getInnerContainerSize()
        	local minX = w-iw
        	if x <= w-iw then
        		return
        	end

        	x = x - w
        	if x < minX then
        		x = minX
        	end

        	local percent = math.abs(x)/(iw-w)*100
            scrollview:scrollToPercentHorizontal(percent, 0.2, false)
            self.m_move2 = true

            self:setTimer(0.2, function ()
            	self.m_move2 = false
            end, "army_tact_move_2")
        end)		
	end
end

function This:refreshArray()
	local scrollview = self.m_csNode.scrollview
	for i, data in ipairs(self.m_array) do
		local node = self.m_arrayNodes[i]
		if not node then
			node = ArmyTacticalItemView:create(data)
			node:setAnchorPoint(0.5, 0.5)
			node:setCheckRedDot(false)

			local w, h = node:getContentSize()
			scrollview:pushBackCustomItem(node, w, h, 0.5, 10)
			self.m_arrayNodes[i] = node
		else
			node:setModel(data)
		end
	end
end

function This:refresh()
	
end

