

local This, super = class(...,"Dialog")
local ArmyTactDetailsItemView = require("ArmyTactical.ArmyTactDetailsItemView")
local RankPageListLayer = require("Rank.RankPageListLayer")


function This:initData(info)
	self.m_armyTact = info
	self.m_list = {}

	local maxLv = self.m_armyTact:getMaxLevel()
	local curLv = self.m_armyTact:getLevel()
	for lv = 0, maxLv do
		local config = self.m_armyTact:getLevelConfig(lv)
		local t = {}

		t.lv = lv
		t.curLv = curLv
		t.fightPower = config.levelup_fightpower_min
		t.attrList = self:formatAttrList(config.levelup_arg)

		table.insert(self.m_list, t)
	end
end

function This:formatAttrList(attribute)
	local list = {}

	for i, attr in ipairs(attribute) do
		local attrId = attr[1]
		local attrVal = attr[2]
		local desc = ""

		if AttrInfo[attrId].valueType == ValueType.kPercent then
			desc = string.format(Serialize[60301], AttrName[attrId], attrVal*100)
		else
			desc = string.format(Serialize[60302], AttrName[attrId], math.floor(attrVal))
		end

		table.insert(list, desc)
	end

	return list
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/kingWayArrayInfo.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

	local scrollView = self.m_csNode.scrollView
	scrollView:setDirection("vertical")

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 720,
        itemHeight = 94,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
		dataCounter = function ()	 
	        return #self.m_list
        end,
        viewCreator =  function(index, node) 
        	local data = self.m_list[index]
        	if data then     
			    if not node then
				    return ArmyTactDetailsItemView:create(data, index)
				else
			        node:setModel(data, index)
			    end
			end
	    end,
	    hint = self.m_armyTact:getLevel()+1,
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setDataList()
end

function This:initTouchEvent()
	if not self.m_inited then
		self.m_inited = true

		self.m_csNode.close:setTouchEnded(function()
        	self:close()
    	end)
		
	end
end