

local This, super = class(...,"Dialog")
local ArmyTactQueueMenuBtn = require("ArmyTactical.ArmyTactQueueMenuBtn")
local ArmyTacticalManager = require("ArmyTactical.ArmyTacticalManager")
local ArmyTacticalSlotNode = require("ArmyTactical.ArmyTacticalSlotNode")
local HeroManager = require("Hero.HeroManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local ArmyTactQueueHeroNode = require("ArmyTactical.ArmyTactQueueHeroNode")



function This:initData(armyTact)
	self.m_array = {}
	self.m_arrayNodes = {}
	self.m_arrayIdx = 0
	self.m_slotNodes = {}
	self.m_selectHeros = {}
	self.m_allHero = {}
	self.m_countryHero = {}
	self.m_countryTab = {}
	self.m_countryIdx = 0
	self.m_heroList = {}
	self.m_isSaveing = false

	self:initArmyTactList()
	self:initQueue()
	self:initHeroList()
end

function This:initArmyTactList()
	local all = ArmyTacticalManager:getAllArmyTactical()
	self.m_array = {}

	for k, at in pairs(all) do
		table.insert(self.m_array, at)
	end

	table.sort(self.m_array, function (a, b)
		return a:getId() < b:getId()
	end)
end

function This:initQueue()
	local all = ArmyTacticalManager:getAllArmyTactical()
	self.m_selectHeros = {}

	for k, at in pairs(all) do
		local queue = at:initQueue()
		for i, hero in ipairs(queue) do
			self:setHeroArmyTact(hero:getId(), at)
		end
	end
end

function This:initHeroList()
	local selector = function (hero)
		if hero:isDeputy() then
			return false
		end
		return true
	end

	self.m_allHero = HeroManager:getAllHeroGotWithSort(selector)
	self.m_countryHero = {}

	for i, hero in ipairs(self.m_allHero) do 
		local country = hero:getCountry()
		if not self.m_countryHero[country] then
			self.m_countryHero[country] = {}
		end
		table.insert(self.m_countryHero[country], hero)
	end
end

function This:initNode(armyTact)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/kingWayArrayArrangement.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

    --军阵
    local svArray = csNode.svArray
    svArray:setDirection("vertical")
	self:refreshArray()

	--6个位
	local num = ArmyTacticalManager:getHeroSlotNum()
	for i=1, num do
		local node = ArmyTacticalSlotNode:create()
		node:setStyle(2)
		node:setScale(0.8)
		node:setIndex(i)
		node:setClick(function ()
			self:onSlotClick(node)
		end)
		csNode:addChild(node)
		self.m_slotNodes[i] = node

		local posNode = csNode[string.format("hero%d", i)]
		node:setPosition( posNode:getPosition() )
	end

	--武将列表
	self.m_countryTab = {
		{btn = csNode.country5, country = 5},
		{btn = csNode.country1, country = 1},
		{btn = csNode.country2, country = 2},
		{btn = csNode.country3, country = 3},
		{btn = csNode.country4, country = 4},
	}
	for i, v in ipairs(self.m_countryTab) do
		v.btn:setTouchEnded(function ()
			self:setCountrySelect(i)
		end)
	end

	local svHero = csNode.svHero
	svHero:setDirection("vertical")

    self.m_heroListLayer = RankPageListLayer:create({
        itemWidth = 92,
        itemHeight = 94,
        spaceX = 0,
        spaceY = 0,
        column = 6,
        scrollView = svHero,
		dataCounter = function ()	 
	        return #self.m_heroList
        end,
        viewCreator =  function(index, node) 
        	local hero = self.m_heroList[index]
        	if hero then   
        		local data = {
        			hero = hero, 
        			armyTact = self:getHeroArmyTact( hero:getId() )
        		}  
        		
			    if not node then
			    	node = ArmyTactQueueHeroNode:create(data, index)
			    	node:setClick(function ()
			    		self:onHeroNodeClick(node)
			    	end)
				    return node 
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })
    self.m_heroListLayer:setCascadeOpacityEnabled(true)
    self.m_heroListLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_heroListLayer:getContentSize()
    svHero:pushBackCustomItem(self.m_heroListLayer, w, h, 0)
    self:setCountrySelect(1)

	if armyTact then
		self:setArraySelectById( armyTact:getId() )
	else
		self:setArraySelectIdx(1)
	end
end

function This:onEnter()
    super.onEnter(self)
    ArmyTacticalManager:addObserver(self)
end

function This:onExit()
    ArmyTacticalManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
	if not self.m_inited then
		self.m_inited = true

		local csNode = self.m_csNode

		csNode.close:setTouchEnded(function()
        	self:close()
    	end)

    	csNode.btnSave:setTouchEnded(function()
    		if self.m_isSaveing then
    			return
    		end

        	local armyTact = self:getCurArmyTactical()
			if not armyTact then
				return
			end

			local id = armyTact:getId()
			local state = self:isChange(id)
			if not state then
				print("The queue is not change !!!")
				return
			end

			local uidList = armyTact:getQueueUidList()
			ArmyTacticalManager:onArmyTactAdjustRequest(id, uidList)
			self.m_isSaveing = true

			self:setTimer(0.1, function ()
            	self.m_isSaveing = false
            end, "army_tact_save")
    	end)

   	end
end

function This:refreshArray()
	local svArray = self.m_csNode.svArray
	for i, data in ipairs(self.m_array) do
		local node = self.m_arrayNodes[i]
		if not node then
			node = ArmyTactQueueMenuBtn:create(data)
			node:setAnchorPoint(0.5, 0.5)
			node:setClick(function ()
				self:onArrayNodeClick(node) 
			end)

			local w, h = node:getContentSize()
			svArray:pushBackCustomItem(node, w, h, 0.5, 1)
			self.m_arrayNodes[i] = node
		else
			node:setModel(data)
		end
	end
end

function This:onArrayNodeClick(node)
	local model = node:getModel()
	if model:isLock() then
		g_scene:showTips( model:getLockStr() )
		return
	end
	self:setArraySelectById( model:getId() )
end

function This:getArrayIndex(id)
	for i, data in ipairs(self.m_array) do
		if data:getId() == id then
			return i
		end
	end
	return 0
end

function This:setArraySelectById(id)
	local idx = self:getArrayIndex(id)
	self:setArraySelectIdx(idx)
end

function This:setArraySelectIdx(index)
	if self.m_arrayIdx == index then
		return
	end

	self.m_arrayIdx = index
	for i, node in ipairs(self.m_arrayNodes) do
		node:setSelect(i == index)
	end

	self:refreshQueue()
end

function This:refreshQueue()
	local armyTact = self:getCurArmyTactical()
	if not armyTact then
		return
	end

	local csNode = self.m_csNode
	local queue = armyTact:getQueue()
	local fightPower = 0

	for i, node in ipairs(self.m_slotNodes) do
		local hero = queue[i]
		if hero then
			fightPower = fightPower + hero:getFightPower()
			node:setModel({hero = hero})
		else
			node:setModel(nil)
		end
	end

	csNode.power:setString(fightPower)

	-- 保存按钮
	local state = self:isChange(armyTact:getId())
	if state then
		csNode.btnSave:clearShader()
	else
		csNode.btnSave:setGrayShader()
	end

end

function This:getCurArmyTactical()
	return self.m_array[self.m_arrayIdx]
end

function This:onSlotClick(node)
	local armyTact = self:getCurArmyTactical()
	if not armyTact then
		return
	end

	local idx = node:getIndex()
	local queue = armyTact:getQueue()
	local hero = queue[idx]

	if not hero then
		return
	end

	--移除上阵的武将
	armyTact:removeHeroFromQueue(hero)
	self:setHeroArmyTact(hero:getId(), nil)
	self:refreshQueue()
	self:refreshArray()
	self.m_heroListLayer:setDataList()
end

function This:setCountrySelect(idx)
	if idx == self.m_countryIdx then
		return
	end

	self.m_countryIdx = idx

	for i, v in ipairs(self.m_countryTab) do
		v.btn.light:setVisible(i == idx)
	end

	local config = self.m_countryTab[idx]
	if config.country == 5 then
		self.m_heroList = self.m_allHero
	else
		self.m_heroList = self.m_countryHero[config.country] or {}
	end
	self.m_heroListLayer:setDataList()
end

function This:setHeroArmyTact(heroId, armyTact)
	if armyTact then
		self.m_selectHeros[heroId] = {
			heroId = heroId,
			armyTact = armyTact,
		}
	else
		self.m_selectHeros[heroId] = nil
	end
end

function This:getHeroArmyTact(heroId)
	local data = self.m_selectHeros[heroId]
	if data then
		return data.armyTact
	end
	return nil
end

function This:onHeroNodeClick(node)
	local hero = node:getHero()
	local heroId = hero:getId()
	local armyTact = node:getArmyTact()

	if armyTact then
		--取消选中（不做这个功能）
		-- armyTact:removeHeroFromQueue(hero)
		-- self:setHeroArmyTact(heroId, nil)
		-- node:setModel({hero = hero})
		-- self:refreshQueue()
		-- self:refreshArray()
		g_scene:showTips(Serialize[60322])
		return
	end

	--选中
	armyTact = self:getCurArmyTactical()
	local ret = armyTact:addHeroFromQueue(hero)
	if ret then
		self:setHeroArmyTact(heroId, armyTact)
		node:setModel({hero = hero, armyTact = armyTact})
		self:refreshQueue()
		self:refreshArray()
	end
end

--队列是否改变
function This:isChange(id)
	local armyTact = ArmyTacticalManager:getArmyTactical(id)
	local oldList = armyTact:getHeroList()
	local newList = armyTact:getQueue()

	if #oldList ~= #newList then
		return true
	end

	for i, newHero in ipairs(newList) do
		local oldHero = oldList[i]
		if newHero:getId() ~= oldHero:getId() then
			return true
		end
	end

	return false
end


function This:onArmyTactAdjustReply(armyTact)
	print("------- onArmyTactAdjustReply")
	self:refreshQueue()
end

