

local This , super = class(..., "Base.BaseModel")
local ArmyTacticalConfig = require("Metadatas.ArmyTacticalConfig")
local ArmyTacticalLevelConfig = require("Metadatas.ArmyTacticalLevelConfig")
local ArmyTacticalStageConfig = require("Metadatas.ArmyTacticalStageConfig")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")


function This:init(id)
    super.init(self)

    self.m_id = id
    self.m_heroUidList = {}
    self.m_fightPower = 0
    self.m_level = 0
    self.m_stage = 0
    self.m_queue = {} -- 布阵的临时数据
end

function This:setModel(info)
	if type(info) ~= "table" then
		return
	end

	if info.card_id_list then
		self.m_heroUidList = info.card_id_list		
	end

	if info.fight_power then
		self.m_fightPower = info.fight_power
	end

	if info.level then
		self.m_level = info.level
	end

	if info.stage then
		self.m_stage = info.stage
	end
end

function This:getId()
	return self.m_id
end

function This:getLevel()
	return self.m_level
end

function This:getStage()
	return self.m_stage
end

function This:getCurFightPower()
	return self.m_fightPower
end

function This:getMaxFightPower()
	local config = self:getLevelConfig()
	if config then
		return config.levelup_fightpower_max + 1
	end
	return 1
end

function This:isMaxLevel()
	return self:getLevel() >= self:getMaxLevel()
end

--最大等级
function This:getMaxLevel()
	local config = ArmyTacticalConfig[self.m_id]
	if config then
		return config.max_level
	end
	return 50
end

function This:isMaxStage()
	return self:getStage() >= self:getMaxStage()
end

--最大品阶
function This:getMaxStage()
	local config = ArmyTacticalConfig[self.m_id]
	if config then
		return config.max_stage
	end
	return 5	
end

--开启等级
function This:getOpenLevel()
	local config = ArmyTacticalConfig[self.m_id]
	if config then
		return config.open_level or 0
	end
	return 0
end

--军阵配置
function This:getConfig()
	return ArmyTacticalConfig[self.m_id]
end

--等级配置
function This:getLevelConfig(level)
	level = level or self.m_level
	local key = self.m_id * 1000 + level
	return ArmyTacticalLevelConfig[key]
end

--品阶配置
function This:getStageConfig(stage)
	stage = stage or self.m_stage
	local key = self.m_id * 1000 + stage
	return ArmyTacticalStageConfig[key]
end

--是否锁
function This:isLock()
	return self:getOpenLevel() > GamePlayer:getLevel()
end

function This:getLockStr()
	return string.format(Serialize[60304], self:getOpenLevel())
end

--上阵的武将列表
function This:getHeroList()
	local list = {}

	for i, uid in ipairs(self.m_heroUidList) do
		local hero = HeroManager:getHeroByUid(uid)
		if hero then
			table.insert(list, hero)
		end
	end

	return list
end

function This:getAttrListByLevel(level)
	local list = {}
	local config = self:getLevelConfig(level)

	if config then
		for i, v in ipairs(config.levelup_arg) do
			table.insert(list, {v[1], v[2]})
		end
	end

	return list
end

--等级属性
function This:getLevelAttr()
	local ret = {}
	if self:isLock() then
		return ret
	end

	local config = self:getLevelConfig()
	if config then		
		for i, v in ipairs(config.levelup_arg) do
			ret[v[1]] = v[2]
		end
	end

	return ret
end

--总属性（等级+品阶）
function This:getTotalAttr()
	local ret = {}
	if self:isLock() then
		return ret
	end

	local config = self:getStageConfig()
	if config then
		local attr = self:getLevelAttr()
		local rate = config.promote_percentage/100
		for id, val in pairs(attr) do
			ret[id] = val*rate
		end
	end

	return ret
end

function This:isCanUpLevel()
	if self:isMaxLevel() then
		return false
	end
	return self:getCurFightPower() >= self:getMaxFightPower()
end

function This:isCanUpStage()
	if self:isMaxStage() then
		return false
	end

	local needLv = self:getUpStageNeedLevel()
	if GamePlayer:getLevel() < needLv then
		return false
	end

	local cost = self:getUpStageNeedCost()
	local needId, needNum, count = 0, 0, 0
	for i, v in ipairs(cost) do
		needId = v[1]
		needNum = v[2]
		count = DepotManager:getItemOrResCount(needId)
		if count < needNum then
			return false
		end
	end
	
	return true
end

function This:getUpStageNeedCost()
	if self:isMaxStage() then
		return {}
	end

	local config = self:getStageConfig(self.m_stage+1)
	return config.stageup_cost
end

function This:getUpStageNeedLevel()
	if self:isMaxStage() then
		return 0
	end	

	local config = self:getStageConfig(self.m_stage+1)
	return config.stageup_level
end

function This:getSlotNum()
	return 6
end

----------------布阵用的----------------

function This:initQueue()
	self.m_queue = self:getHeroList()	
	return self.m_queue
end

function This:getQueue()
	return self.m_queue or {}
end

function This:setQueue(queue)
	self.m_queue = queue
end

function This:addHeroFromQueue(hero)
	if #self.m_queue >= self:getSlotNum() then
		return false
	end

	local heroId = hero:getId()
	local isExist = false
	for i, v in ipairs(self.m_queue) do
		if v:getId() == heroId then
			isExist = true
			break
		end
	end

	if isExist then
		return false
	end

	table.insert(self.m_queue, hero)
	return true
end

function This:removeHeroFromQueue(hero)
	local idx = 0
	local heroId = hero:getId()

	for i, v in ipairs(self.m_queue) do
		if v:getId() == heroId then
			idx = i
			break
		end
	end

	if idx ~= 0 then
		table.remove(self.m_queue, idx)
	end
end

function This:getQueueUidList()
	local list = {}

	for i, hero in ipairs(self.m_queue) do
		table.insert(list, hero:getUid())
	end

	return list
end

----------------------------------------------