

local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local ArmyTacticalConfig = require("Metadatas.ArmyTacticalConfig")
local ArmyTactical = require("ArmyTactical.ArmyTactical")
local HeroManager = require("Hero.HeroManager")
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")


function This:init()
	super.init(self)

	self:initArmyTactical()
	self:initListener()
end

function This:initArmyTactical()
	self.m_ats = {}
	for id, v in pairs(ArmyTacticalConfig) do
		self.m_ats[id] = ArmyTactical:create(id)
	end
end

function This:initListener()
	EventDispatcher:addListener(self, 14400, self.onArmyTactInfoReply)
	EventDispatcher:addListener(self, 14401, self.onArmyTactAdjustReply)
	EventDispatcher:addListener(self, 14402, self.onArmyTactUpStageReply)
	EventDispatcher:addListener(self, 14403, self.onArmyTactUpLevelReply)
	GamePlayer:addObserver(self)
end

--军阵信息
function This:onArmyTactInfoRequest()
	if not self:checkOpen() then
		return
	end

	Socket:send(14400, {})
end

function This:onArmyTactInfoReply(msg)
	if msg.err_code == 0 then
		for _, data in ipairs(msg.armytactical_list) do
			local at = self:getArmyTactical(data.armytactical_id)
			at:setModel(data)
		end
		self:updateHeroAttr()
		self:notifyObservers("onArmyTactInfoReply")
	else
        g_scene:handleErrorCode(msg.err_code)
	end	
end

--军阵调整
function This:onArmyTactAdjustRequest(armytactical_id, card_id_list)
	Socket:sendAndLock(14401, {armytactical_id = armytactical_id, card_id_list = card_id_list})
end

function This:onArmyTactAdjustReply(msg)
	--print(msg)
	if msg.err_code == 0 then
		local at = self:getArmyTactical(msg.armytactical_id)
		at:setModel({
			card_id_list = msg.card_id_list,
			fight_power = msg.fight_power,
			level = msg.level,
		})
		self:updateHeroAttr()
		self:notifyObservers("onArmyTactAdjustReply", at)
	else
        g_scene:handleErrorCode(msg.err_code)
	end	
end

--军阵升阶
function This:onArmyTactUpStageRequest(armytactical_id)
	Socket:sendAndLock(14402, {armytactical_id = armytactical_id})
end

function This:onArmyTactUpStageReply(msg)
	--print(msg)
	if msg.err_code == 0 then
		local at = self:getArmyTactical(msg.armytactical_id)
		at:setModel({
			fight_power = msg.fight_power,
			level = msg.level,
			stage = msg.stage,
		})
		self:updateHeroAttr()
		self:notifyObservers("onArmyTactUpStageReply", at)
	else
        g_scene:handleErrorCode(msg.err_code)
	end		
end

--军阵升级
function This:onArmyTactUpLevelRequest(armytactical_id)
	Socket:sendAndLock(14403, {armytactical_id = armytactical_id})
end

function This:onArmyTactUpLevelReply(msg)
	--print(msg)
	if msg.err_code == 0 then
		local at = self:getArmyTactical(msg.armytactical_id)
		at:setModel({
			fight_power = msg.fight_power,
			level = msg.level,
		})
		self:updateHeroAttr()
		self:notifyObservers("onArmyTactUpLevelReply", at)
	else
        g_scene:handleErrorCode(msg.err_code)
	end		
end

--上阵位数量
function This:getHeroSlotNum()
	return 6
end

function This:getArmyTactical(id)
	return self.m_ats[id]
end

function This:getAllArmyTactical()
	return self.m_ats
end

function This:getOpenLevel()
	if not self.m_openLv then
		self.m_openLv = ArgConfig.armytactical_openlevel.arg[1]
	end
	return self.m_openLv
end

--检查开启，true开启，false关闭
function This:checkOpen()
	return  GamePlayer:getLevel() >= self:getOpenLevel()
end

function This:getOpenTips()
	return string.format(Serialize[60309], self:getOpenLevel())
end

function This:getAttr()
	local attrs = {}
	if not self:checkOpen() then
		return attrs
	end

	for id, at in pairs(self.m_ats) do
		local atAtrr = at:getTotalAttr()
		for k, v in pairs(atAtrr) do
			attrs[k] = (attrs[k] or 0) + v
		end
	end	

	return attrs
end

function This:updateHeroAttr()
	if not self:checkOpen() then
		return
	end

	local attrs = self:getAttr()
	local heros = HeroManager:getAllHero()
	for _, hero in pairs(heros) do
		if hero:isGot() then
			hero:updateArmyTacticalAttr(attrs)
		end
	end
	HeroManager:calcSaintFightPower()	
end

function This:onPlayerAttrChanged(field, lastValue, value)
	--主公等级检查
	if field == PlayerAttr.level and lastValue ~= value then
		local lv = self:getOpenLevel()
		if value == lv then
			self:onArmyTactInfoRequest()
		end
	end
end

