

local This , super = class(..., "NodeUI")
local ArenaHeroView = require("Arena.ArenaHeroView")


function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/kingWayArrayItem.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local w, h = csNode.bg:getContentSize()
    csNode:setPosition(w/2, h/2)
    csNode.close:setVisible(false)
    csNode.add:setVisible(false)

    self:setAnchorPoint(0.5, 0.5)
    self:setPosition(w/2, h/2)
    self:setContentSize(w, h)

    self:setModel(info)
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        self.m_csNode.bg:setTouch({
            isScale = false,
            onEnded = function ()
                if type(self.m_clickCb) == "function" then
            		self.m_clickCb(self)
            	end 
            end,
        })
    end
end

function This:setModel(info)
	info = info or {}
	self.m_hero = info.hero
	self:refresh()
end

function This:setStyle(style)
	self.m_style = style -- 1军阵主界面，2军阵选择界面
	self:refreshStyle()
end

function This:setClick(func)
	self.m_clickCb = func
end

function This:setIndex(idx)
	self.m_index = idx
end

function This:getIndex()
	return self.m_index
end

function This:refresh()
	local hero = self.m_hero

	if hero then
		local heroId = hero:getId()
		local level = hero:getLevel()
		local star = hero:getTotalPhase()
		local awake = hero:isAwakeFull()

		if not self.m_heroNode then
			self.m_heroNode = ArenaHeroView:create({
				heroId = heroId,
				level = level,
				star = star,
				awake = awake,
			})
			self.m_csNode.item:addChild(self.m_heroNode)
		else
			self.m_heroNode:setModel({
				heroId = heroId,
				level = level,
				star = star,
				awake = awake,
			})
			self.m_heroNode:setVisible(true)
		end
	else
		if self.m_heroNode then
			self.m_heroNode:setVisible(false)
		end
	end

	self:refreshStyle()
end

function This:refreshStyle()
	local csNode = self.m_csNode

	if self.m_style == 1 then
		csNode.add:setVisible( (self.m_hero == nil) and true or false )
		csNode.close:setVisible(false)
	elseif self.m_style == 2 then
		csNode.add:setVisible(false)
		csNode.close:setVisible( self.m_hero and true or false )
	end
end
