

local This , super = class(...,"NodeUI")
local ArmyTacticalManager = require("ArmyTactical.ArmyTacticalManager")
local ArmyTacticalItemView = require("ArmyTactical.ArmyTacticalItemView")
local ArmyTacticalSlotNode = require("ArmyTactical.ArmyTacticalSlotNode")
local ItemNode = require("UI.ItemNode")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")

function This:initData(parent)
    self.m_parent = parent
	self.m_array = {}
	self.m_selectIdx = 0
	self.m_arrayNodes = {}
	self.m_slotNodes = {}

    self:initArmyTactList()

    --重新请求数据
    ArmyTacticalManager:onArmyTactInfoRequest()
end

function This:initArmyTactList()
	local all = ArmyTacticalManager:getAllArmyTactical()
	self.m_array = {}

	for k, at in pairs(all) do
		table.insert(self.m_array, at)
	end

	table.sort(self.m_array, function (a, b)
		return a:getId() < b:getId()
	end)
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/kingWayArray.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local scrollview = csNode.scrollview
    scrollview:setDirection("horizontal")
	scrollview.m_direction = "right"
	self:refreshArray()

	local num = ArmyTacticalManager:getHeroSlotNum()
	for i=1, num do
		local node = ArmyTacticalSlotNode:create()
		node:setStyle(1)
		node:setScale(0.8)
		node:setClick(function ()
			local armyTact = self:getCurArmyTactical()
			if not armyTact then
				return
			end

			g_scene:openDialog("ArmyTactical.ArmyTactQueuePanel", armyTact)
		end)
		csNode:addChild(node)
		self.m_slotNodes[i] = node

		local posNode = csNode[string.format("hero%d", i)]
		node:setPosition( posNode:getPosition() )
	end

	self:setSelectIdx(1)
	self:scheduleUpdate()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        local csNode = self.m_csNode
        local scrollview = csNode.scrollview

        -- 左箭头
        self.m_move1 = false
        csNode.arrow1:setTouchEnded(function ()
        	if self.m_move1 then
        		return
        	end

        	local x, y = scrollview:getInnerContainerPosition()
        	if x >= 0 then
        		return
        	end

        	local w, h = scrollview:getContentSize()
        	local iw, ih = scrollview:getInnerContainerSize()
        	
        	x = x + w
        	if x > 0 then
        		x = 0
        	end

        	local percent = math.abs(x)/(iw-w)*100
            scrollview:scrollToPercentHorizontal(percent, 0.2, false)
            self.m_move1 = true

            self:setTimer(0.2, function ()
            	self.m_move1 = false
            end, "army_tact_move_1")
        end)

        -- 右箭头
        self.m_move2 = false
        csNode.arrow2:setTouchEnded(function ()
            if self.m_move2 then
        		return
        	end

        	local x, y = scrollview:getInnerContainerPosition()
        	local w, h = scrollview:getContentSize()
        	local iw, ih = scrollview:getInnerContainerSize()
        	local minX = w-iw
        	if x <= w-iw then
        		return
        	end

        	x = x - w
        	if x < minX then
        		x = minX
        	end

        	local percent = math.abs(x)/(iw-w)*100
            scrollview:scrollToPercentHorizontal(percent, 0.2, false)
            self.m_move2 = true

            self:setTimer(0.2, function ()
            	self.m_move2 = false
            end, "army_tact_move_2")
        end)

        -- 属性一览
        csNode.previewBg:setTouchEnded(function ()
            g_scene:openDialog("ArmyTactical.ArmyTactAttrPanel")
        end)

        -- 升级
        csNode.btnRefresh:setTouchEnded(function ()
            local armyTact = self:getCurArmyTactical()
			if not armyTact then
				return
			end

			if armyTact:isCanUpLevel() then
				ArmyTacticalManager:onArmyTactUpLevelRequest( armyTact:getId() )
			end
        end)

        -- 升阶
        csNode.btnStage:setTouchEnded(function ()
            local armyTact = self:getCurArmyTactical()
			if not armyTact then
				return
			end

			if armyTact:isCanUpStage() then
				ArmyTacticalManager:onArmyTactUpStageRequest( armyTact:getId() )
			end
        end)

        -- 详细信息
        csNode.look:setTouchEnded(function ()
        	local armyTact = self:getCurArmyTactical()
			if not armyTact then
				return
			end

            g_scene:openDialog("ArmyTactical.ArmyTactDetailsPanel", armyTact)
        end)
    end

end

function This:onEnter()
    super.onEnter(self)
    ArmyTacticalManager:addObserver(self)
    GamePlayer:addObserver(self)
	DepotManager:addObserver(self)
end

function This:onExit()
    ArmyTacticalManager:removeObserver(self)
    GamePlayer:removeObserver(self)
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:getArrayIndex(id)
	for i, data in ipairs(self.m_array) do
		if data:getId() == id then
			return i
		end
	end
	return 0
end

function This:setSelectById(id)
	local idx = self:getArrayIndex(id)
	self:setSelectIdx(idx)
end

function This:setSelectIdx(index)
	if self.m_selectIdx == index then
		return
	end

	self.m_selectIdx = index
	for i, node in ipairs(self.m_arrayNodes) do
		node:setSelect(i == index)
	end
	self:refresh()
end

function This:getCurArmyTactical()
	return self.m_array[self.m_selectIdx]
end

function This:refresh()
	local armyTact = self:getCurArmyTactical()
	local csNode = self.m_csNode

	if not armyTact then
		return
	end

	local config = armyTact:getConfig()
	csNode.lbTitle:setString(config.name)

	--武将位
	local heroList = armyTact:getHeroList()
	for i, node in ipairs(self.m_slotNodes) do
		local hero = heroList[i]
		node:setModel({hero = hero})
	end

	--进度
	local isMaxLv = armyTact:isMaxLevel()
	local curExp = armyTact:getCurFightPower()
	local maxExp = armyTact:getMaxFightPower()
	csNode.progress.tiao:setPercent(curExp/maxExp*100)
	csNode.progress.num:setString(string.format("%d/%d", curExp, maxExp))

	--属性
	local curLv = armyTact:getLevel()
	local nexLv = isMaxLv and curLv or (curLv + 1)		
	local curAttr = armyTact:getAttrListByLevel(curLv)
	local nexAttr = isMaxLv and {} or armyTact:getAttrListByLevel(nexLv)
	local curAttrDesc = self:formatAttrDesc(curAttr, "A1")
	local nexAttrDesc = self:formatAttrDesc(nexAttr, "A5")

	local updateAttr = function (node, lv, attrs, isMaxLv)
		if isMaxLv then
			node:setVisible(false)
			return
		end

		node:setVisible(true)
		node.lbLevel:setString(string.format(Serialize[60312], lv))

		for i=1, 3 do
			local lbAttr = node[string.format("lbAttr%d", i)]
			local desc = attrs[i]
			if desc then
				lbAttr:setVisible(true)
				lbAttr:setStringWithColor(desc)
			else
				lbAttr:setVisible(false)
			end
		end
	end

	updateAttr(csNode.value1, curLv, curAttrDesc, false)
	updateAttr(csNode.value2, nexLv, nexAttrDesc, isMaxLv)
	csNode.yimanji:setVisible(isMaxLv)


	--升级按钮
	if armyTact:isCanUpLevel() then
		csNode.btnRefresh:clearShader()
	else
		csNode.btnRefresh:setGrayShader()
	end

	--品阶
	local isMaxStage = armyTact:isMaxStage()
	csNode.lbMaxStage:setVisible(isMaxStage)
	csNode.stage2:setVisible(not isMaxStage)
	csNode.btnStage:setVisible(not isMaxStage)
	csNode.lbStageLock:setVisible(not isMaxStage)
	if self.m_costNode then
		self.m_costNode:setVisible(not isMaxStage)
	end

	local curStage = armyTact:getStage()
	local curConfig = armyTact:getStageConfig(curStage)
	csNode.stage1.jie:setString(string.format(Serialize[60313], curStage))
	csNode.stage1.value1:setString(string.format(Serialize[60314], curConfig.promote_percentage))

	if #curAttr == 1 then
		local val = curAttr[1][2]
		curAttr[1][2] = curConfig.promote_percentage/100 *val

		local curStageDesc = self:formatAttrDesc(curAttr)
		csNode.stage1.value2:setString( string.format("（%s）", curStageDesc[1]) )
	else
		csNode.stage1.value2:setString("")
	end

	if not isMaxStage then
		local nexStage = curStage + 1
		local nexConfig = armyTact:getStageConfig(nexStage)
		csNode.stage2.jie:setString(string.format(Serialize[60315], nexStage))
		csNode.stage2.value1:setString(string.format(Serialize[60314], nexConfig.promote_percentage))

		if #nexAttr == 1 then
			local val = nexAttr[1][2]
			nexAttr[1][2] = nexConfig.promote_percentage/100 *val

			local nexStageDesc = self:formatAttrDesc(nexAttr)
			csNode.stage2.value2:setString( string.format("（%s）", nexStageDesc[1]) )
		else
			csNode.stage2.value2:setString("")
		end

		-- 消耗
		local cost = armyTact:getUpStageNeedCost()[1]
		if self.m_costNode then
			self.m_costNode:initItem(cost[1], cost[2], true)
			self.m_costNode:showNeedNum(cost[2])
		else
			local costNode = ItemNode:create(cost[1], cost[2], true)
			costNode:setAnchorPoint(0.5, 0.5)
			costNode:setScale(0.8)
			costNode:setPosition(csNode.cost:getPosition())
			csNode:addChild(costNode)
			self.m_costNode = costNode
		end

		-- 解锁等级
		local needLv = armyTact:getUpStageNeedLevel()
		local playerLv = GamePlayer:getLevel()
		csNode.lbStageLock:setString(string.format(Serialize[60316], needLv))

		if playerLv >= needLv then
			csNode.lbStageLock:setVisible(false)
		else
			csNode.lbStageLock:setVisible(true)
			csNode.lbStageLock:setColor(255, 0, 0)
		end

		-- 升阶按钮
		if armyTact:isCanUpStage() then
			csNode.btnStage:clearShader()
		else
			csNode.btnStage:setGrayShader()
		end
	end

end

function This:onArrayNodeClick(node)
	local model = node:getModel()
	if model:isLock() then
		g_scene:showTips( model:getLockStr() )
		return
	end
	self:setSelectById( model:getId() )
end

function This:formatAttrDesc(attribute, colorId)
	local list = {}
	local count = #attribute

	if colorId == nil then
		colorId = ""
	else
		colorId = string.format("<%s>", colorId)
	end

	for i, attr in ipairs(attribute) do
		local attrId = attr[1]
		local attrVal = attr[2]
		local desc = ""

		if AttrInfo[attrId].valueType == ValueType.kPercent then
			desc = string.format(Serialize[60317], colorId, AttrName[attrId], attrVal*100)
		else
			desc = string.format(Serialize[60318], colorId, AttrName[attrId], math.floor(attrVal))
		end

		table.insert(list, desc)
	end

	return list
end

function This:refreshArray()
	local scrollview = self.m_csNode.scrollview
	for i, data in ipairs(self.m_array) do
		local node = self.m_arrayNodes[i]
		if not node then
			node = ArmyTacticalItemView:create(data)
			node:setAnchorPoint(0.5, 0.5)
			node:setClick(function ()
				self:onArrayNodeClick(node)
			end)

			local w, h = node:getContentSize()
			scrollview:pushBackCustomItem(node, w, h, 0.5, 10)
			self.m_arrayNodes[i] = node
		else
			node:setModel(data)
		end
	end
end

function This:onArmyTactInfoReply()
	self:initArmyTactList()
	self:refreshArray()
	self:refresh()	
end

function This:onArmyTactAdjustReply()
	self:initArmyTactList()
	self:refreshArray()
	self:refresh()
end

function This:onArmyTactUpStageReply()
	self:initArmyTactList()
	self:refreshArray()
	self:refresh()	
end

function This:onArmyTactUpLevelReply()
	self:initArmyTactList()
	self:refreshArray()
	self:refresh()	
end

function This:onPlayerAttrChanged(field, lastValue, value)
	--主公等级检查
	if field == PlayerAttr.level and lastValue ~= value then
		self:refreshArray()
		self:refresh()
	end
end

function This:itemUpdate(itemList)
	self:refreshArray()
	self:refresh()
end

