-- @Author: zheng
-- @Date:   2018-12-26 14:55:55
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-02 16:01:38
-- @File Name: ArmsAwakePanel.lua

local This , super = class(...,"Dialog")
local GameUtil = require("Util.GameUtil")
local HeroManager = require("Hero.HeroManager")
local TerrBuildManager = require("Territory.TerrBuildManager")
local DepotManager = require("Depot.DepotManager")

This.fileName = "GameScene/NodeUI/AwakeArm.csb"

function This:initNode(info)
    self.m_awakeSoilderId = info.awakeSoilderId
    self.m_hero = HeroManager:getHero(info.heroId)
    self.m_awakeSoilder = self.m_hero:getAwakeSoilder(self.m_awakeSoilderId)

    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self:refresh()
end


function This:refresh()
    self:refreshBaseInfo()
    self:refreshScroolView()
end

function This:initTouchEvent()
    local normal = self.m_csNode.normal
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    normal.goumai:setTouchEnded(function()--购  买
        g_scene:openDialog("Awake.ArmsBuyPanel", {})  
    end)
    normal.culture:setTouchEnded(function()--觉  醒
        if not GameUtil:checkNeedItem(self.m_csNode.normal.scrollview) then
            local cardId = self.m_hero:getUid()
            local forceType = self.m_awakeSoilder:getId()
            HeroManager:onGetForceAwakeRequest(cardId,forceType)
        end
    end)
end

function This:onEnter()
    super.onEnter(self)
    HeroManager:addObserver(self)
    DepotManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    HeroManager:removeObserver(self)
    DepotManager:removeObserver(self)
end

function This:refreshBaseInfo()
    
end

function This:refreshAwakeSoilder()
    local soilderId = self.m_awakeSoilder:getId()
    self.m_awakeSoilder = self.m_hero:getAwakeSoilder(soilderId)
end

function This:refreshScroolView()
    self:refreshAwakeSoilder()
    local function refreshJue(node,level)
        local str = string.numberToWord(level)..Serialize["Awake_ArmsAwake_1"]
        node.name:setString(str)
    end
    local normal = self.m_csNode.normal
    local awakeSoilder = self.m_awakeSoilder
    local nextSoilderConfig = awakeSoilder:getNextConfig()
    local levelId = awakeSoilder:getPhase()

    local isFull = false

    local str1s = {}
    local str2s = {}
    if nextSoilderConfig then
        str1s= awakeSoilder:getAttrName()
        str2s= awakeSoilder:getAttrName(nil,nil,nil,nextSoilderConfig.num)
    else
        str1s= awakeSoilder:getAttrName()
        isFull = true
    end
    self.m_csNode.manji:setVisible(isFull)
    normal:setVisible(not isFull)
    local item1 =self.m_csNode.item1
    GameUtil:resetTexts(item1,str1s)
    refreshJue(item1,levelId)

    if not isFull then
        GameUtil:resetTexts(normal.item2,str2s)
        refreshJue(normal.item2,nextSoilderConfig.level)

        local needs = awakeSoilder:getNeed()
        GameUtil:resetScoolItemNode(normal.scrollview,needs,nil,true,30)
        normal.text1:setVisible(false)

        local upLimit = awakeSoilder:getLimit()
        local buildId = upLimit[1]
        local buildLimit = upLimit[2]
        local building = TerrBuildManager:getBuildBy(buildId)
        local buildLv = building:getLevel()
        -- local buildLv = TerrBuildManager:getBuildByType(buildId)

        local text1 = normal.text1
        local culture = normal.culture
        culture:setVisible(buildLv>=buildLimit)
        text1:setVisible(not culture:isVisible())
        if buildLv<buildLimit then
            text1:setString(string.format(Serialize["Awake_ArmsAwake_2"],SoilderTypeName[awakeSoilder:getId()],buildLimit))
        end
    end
end

function This:onReceiveUniversalStoreItemInfo()
    self:refresh()
end

function This:onGetForceAwake()
    self:refresh()
end
