-- @Author: zheng
-- @Date:   2018-12-26 20:02:18
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-23 11:01:16
-- @File Name: ArmsDetailsPanel.lua

local This , super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local GameUtil = require("Util.GameUtil")
local StarUpConfig = require("Metadatas.StarUpConfig")


This.fileName = "GameScene/NodeUI/AwakeMain.csb"

function This:initNode(info)
    self.m_awakeSoilderId = info.awakeSoilderId
    self.m_hero = HeroManager:getHero(info.heroId)
    self.m_awakeSoilder = self.m_hero:getAwakeSoilder(self.m_awakeSoilderId)

    self.m_soilder = self.m_hero:getSoilder(self.m_awakeSoilderId)
    self.m_detailId = info.detailId
    self.m_detailConfig = self.m_awakeSoilder:getDetailsConfig(info.detailId)

    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()


    self:addChild(self.m_csNode)
    self:refresh()
end


function This:refresh()
    self:refreshBaseInfo()
    self:refreshScroolView()
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    self.m_csNode.tejiitem:setTouchEnded(function()
        local res,level = self.m_awakeSoilder:getAwakeSkillDesc(self.m_detailId)
           g_scene:openDialog("Fabao.FabaoSkillInfoPanel", {
               skillName = self.m_detailConfig.arm_skill_name,
               skillDescriptionList = res,
               highlightIndex = level,
           })
    end)

    for i = 1 , 3 do
        g_scene:handleHelpTips(self.m_csNode["tep"..i].icon,function()
            local featureList = self.m_soilder:getAttrFeatures()
            local nextFeatureList = self.m_soilder:getNextPhaseAttrFeatures()
            local featureId = featureList[i] <= 0 and nextFeatureList[i] or featureList[i]
            if featureId > 0 then
                local info = {}
                info.text = string.format("%s:\n%s",StarUpConfig[featureId].name,StarUpConfig[featureId].desc)
                return info
            end
        end) 
    end
end


function This:getSoilderAttrName()
    local result = {}
    local soilder = self.m_soilder
    local attrList = {AttrType.kPower, AttrType.kDfs, AttrType.kIntelligence, AttrType.kChiefValue}
    for attrIndex , attrId in pairs(attrList) do
        local value = soilder:getAttr(attrId)
        local name ,valueStr = GameUtil:getAttrStr( attrId,value )
        local str = name..valueStr
        table.insert(result,str)
    end
    return result
end

function This:refreshBaseInfo()
    local data = self.m_data
    local str1s = self.m_awakeSoilder:getAttrName(nil,"")
    local str2s = self:getSoilderAttrName()
    
    GameUtil:resetTexts(self.m_csNode.jxsx,str1s)
    GameUtil:resetTexts(self.m_csNode.pysx,str2s)

    self.m_csNode.title:setString(self.m_detailConfig.arm_name)
    self.m_csNode.tejiitem:setImage( GameUtil:getSkillIcon(self.m_detailConfig.arm_skill_icon))
    self.m_csNode.text2:setString(self.m_detailConfig.arm_des,683)
    self.m_csNode.bz:setImage(string.format("Icons/Arm/%d.png",self.m_detailConfig.arm_card))
    local featureList = self.m_soilder:getAttrFeatures()
        local nextFeatureList = self.m_soilder:getNextPhaseAttrFeatures()

        for i = 1 , 3 do
            local node = self.m_csNode["tep"..i]
            if featureList[i] <= 0 and nextFeatureList[i] > 0 then
                local featureId = nextFeatureList[i]
                local image = GameUtil:getSkillIcon(StarUpConfig[featureId].icon)
                node.icon:setImage(image)
                node:setVisible(true)
                node.text:setVisible(true)
                node.level:setString(string.format("LV:%d",StarUpConfig[featureId].level or 1))
            elseif featureList[i] > 0 then
                local featureId = featureList[i]
                local image = GameUtil:getSkillIcon(StarUpConfig[featureId].icon)
                node.icon:setImage(image)
                node:setVisible(true)
                node.text:setVisible(false)
                node.level:setString(string.format("LV:%d",StarUpConfig[featureId].level or 1))
            else
                node:setVisible(false)
            end
        end
    
    

end

function This:refreshScroolView()
    local str1s = self.m_awakeSoilder:getAttrName(nil,"")
    local str2s = self:getSoilderAttrName()
    GameUtil:resetTexts(self.m_csNode.jxsx,str1s)
    GameUtil:resetTexts(self.m_csNode.pysx,str2s)
end