-- @Author: zheng
-- @Date:   2018-12-29 11:08:08
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-08-09 10:35:14
-- @File Name: AwakeSoilser.lua

local GameUtil = require("Util.GameUtil")
local HeroManager = require("Hero.HeroManager")

local ArmAwakePromoteConfig = require("Metadatas.ArmAwakePromoteConfig")
local ArmDetailsConfig = require("Metadatas.ArmDetailsConfig")

local This , super = class(...,"Base.BaseModel")

function This:init(info)
    super.init(self)
    self.m_id = info.id
    self.m_heroId = 0
    self.m_contry = info.contry
    self:setPhase(info.phase or 0)
    self:setSelectSkill(1)
end

function This:getCountryId()
    return self.m_contry
end

function This:getId()
    return self.m_id
end

function This:isUnlock()
    return true
end

--获取觉醒技能描述
function This:getAwakeSkillDesc(detailId)
    local result ={}
    local forceType = math.floor(detailId / 1000)
    
    local temDetailId = detailId%1000
    local contry = math.floor(temDetailId / 100)
    local phase = temDetailId%100
    local skillIds = {}
    local awakeCount = 0
    for lv=1,kSoilderAwakeMaxPhase do
       detail = forceType*1000+ contry*100+lv
       local detailItem = self:getDetailsConfig(detail)
       local skillId = detailItem.skill_id
       if not skillIds[skillId] then
           local skillDesc = detailItem.skill_des
           local awakeLevel = detailItem.awake_level
           skillIds[skillId]= skillId
           local level = #result+1
           local str = "Lv."..level..":"..skillDesc
           if lv<=phase then
                awakeCount= awakeCount +1
               str = str..Serialize["Awake_AwakeSoilser_1_c"]
           else
               str = str..string.format(Serialize["Awake_AwakeSoilser_2"],awakeLevel)
           end
           table.insert(result,str)
           -- count = count + 1
       end
    end
    return result,awakeCount
end

function This:setPhase(phase)
    self.m_phase = phase
    local key = self.m_contry*1000+ self.m_id * 100 + self.m_phase
    self.m_config = ArmAwakePromoteConfig[key]
    self:updateAttr()
    -- local nextSoilderConfig = self:getNextConfig()
    -- if not nextSoilderConfig then
    --     nextSoilderConfig = self:getConfig()
    -- end
    -- self:setSelectSkill(nextSoilderConfig.unlock_arm[1])
end

function This:getPhase()
    return self.m_phase
end

function This:getPhase()
    return self.m_phase
end

-- 觉醒的技能id
function This:setSelectSkill(selectSkill)
    if selectSkill and selectSkill~=0 then
        self.m_selectSkill = selectSkill
    end
end

function This:getSelectSkill()
    return self.m_selectSkill
end

--觉醒配置
function This:getConfig()
    return self.m_config
end

function This:getNextConfig()
    local key = self.m_contry*1000+ self.m_id * 100 + self.m_phase+1
    return ArmAwakePromoteConfig[key]
end

function This:getDetailsConfig(detailId)
    return ArmDetailsConfig[detailId]
end

function This:getPhase()
    return self.m_phase
end

function This:getHeroId()
    return self.m_heroId
end

function This:getName()
    return self.m_config.arms_name
end

function This:updateAttr(attrs)
    if not attrs then
        attrs = GameUtil:convertData(self:getAttr())
    end
    -- self.m_attrValue = attrs
    if self.m_heroId > 0 then
        local hero = HeroManager:getHero(self.m_heroId)
        self.m_backupAttr  = self.m_backupAttr or {}
        attrs = hero:resetEmptyAttrs(attrs,self.m_backupAttr)
        local addattrs = {}
        for attrId , value in pairs(attrs) do
            addattrs[attrId] = value - (self.m_backupAttr[attrId] or 0)
            hero:addAttr(attrId,addattrs[attrId])
            self.m_backupAttr[attrId] = value
        end
        hero:updateFightPower()
        -- local primaryId = hero:getPrimaryHero()
        -- if primaryId > 0 then
        --     local primary = HeroManager:getHero(primaryId)
        --     primary:updateDeputy(hero)
        -- end
    end
end

function This:attachHero(hero)
    if hero:getId() ~= self.m_heroId then
        self:disattachHero()
    end
    self.m_heroId = hero:getId()
    self:updateAttr()
end

function This:disattachHero()
    if self.m_heroId <= 0 then
        return
    end
    self:updateAttr({})
    self.m_heroId = 0
end

function This:reset()
    self.m_phase = 0
    self.m_heroId = 0
    self.m_backupAttr = {}
    self.m_addValueForHero = {}
    self.m_attrPoint = {}
end

function This:hasAwake()
    return self.m_phase > 0
end

----------------------------------------------------------读取配表
-- 募兵消耗
function This:getRecruitCost()
    local cost = self:getLockArms()
    local selectSkill = self:getSelectSkill()
    local detailConfig = ArmDetailsConfig[cost[selectSkill]]
    return detailConfig.arm_cost
end

function This:getLockArms()
    return self.m_config.unlock_arm
end

function This:getNeed()
    return self.m_config.lock_cost
end

function This:getAttr()
    return self.m_config.num
end

function This:getLimit()
    return self.m_config.unlock_condition
end

--觉醒的属性
function This:getAttr()
    return self.m_config.num
end

function This:getTotalTrainCost()
    return self.m_config.item_return
end

--获取觉醒获得的属性的名字
function This:getAttrName(value,spaceStr,colorStr,attr)
    if not colorStr then
        colorStr =""
    end
    if not spaceStr then
        spaceStr = "  "
    end
    if not attr then
        attr = self:getAttr()
    end
    local result = {}
    for _,item1 in ipairs(attr) do
        local itemKey = item1[1]
        local itemValue = value or item1[2]
        local name,desc = GameUtil:getAttrStr( itemKey,itemValue )
        local str = colorStr..name..spaceStr..desc
        table.insert(result,str)
    end
    return result
end
