-- @Author: zheng
-- @Date:   2019-01-24 14:30:27
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-24 17:10:10
-- @File Name: FameUpTab.lua
local RolePrestigeConfig = require("Metadatas.RolePrestigeConfig")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ArgConfig = require("Metadatas.ArgConfig")

local This , super = class(...,"NodeUI")

function This:initNode(node)
    self.m_csNode = node.m_csNode.mingwang
    self:getData()
end

function This:setVisible(isVisible)
	super.setVisible(self,isVisible)
	self.m_csNode:setVisible(isVisible)
end

function This:getAttrStr(configItem)
	local result ={}
	if not configItem then
		return result
	end
	local attrs = configItem.attribute
	for attrIndex,itemConfig in ipairs(attrs) do
		local name,value = GameUtil:getAttrStr(itemConfig[1],itemConfig[2])
		local str = name..value
		table.insert(result,str)
	end
	return result
end

function This:refresh()
	self:getData()
	local function updateAttr(node,configItem,isHaveColor)
		local sttrStrs = self:getAttrStr(configItem)
		local color = "<New3>"
		if isHaveColor then
			color = "<New9>"
		end
		color = ""
		for attrIndex=1,6 do
			local des= node["des"..attrIndex]
			local valueTxt = node["value"..attrIndex]
			local dianSp = node["dian"..attrIndex]
			local attrStr = sttrStrs[attrIndex]
			local isVisible = false
			if attrStr then
				isVisible = true
				if des then
					des:setString(color..Serialize["Awake_FameUp_3_c"])
				end
				valueTxt:setString(color..attrStr)
			end
			if des then
				des:setVisible(isVisible)
				dianSp:setVisible(isVisible)
			end
			valueTxt:setVisible(isVisible)
		end
	end
	local function updateLv(node,configItem)
		local id = configItem.id
        local number1 = math.floor(id / 10)
		local number2 = id % 10
		local nums = {number1,number2}
		local numFormat = "Fonts/Repute/%d.png"
		for index,num in ipairs(nums) do
			local temNum = num
			local txt = node["text"..index]	
			local visible = true
			if number1 == 0 then
				if index==2 then
					visible = false
				else
					temNum = number2
				end
			end
			if visible then
				txt:setImage(string.format(numFormat,temNum))
			end
			txt:setVisible(visible)
		end
	end
	
	GameUtil:setPlayerFeltNodeResource(self.m_csNode.zhugong, GamePlayer:getIcon(),false)
	local lvConfig = self.m_lvConfig
	local needMingWang = lvConfig.value
	local needLv = lvConfig.level

	local nextLvConfig = self.m_nextLvConfig
	local isCanUp = g_redDot:mingWangIsCanUp()
	local mingWang = GamePlayer:getMingWangValue()
	local fulllevel = lvConfig.value ==-1

	local manji = self.m_csNode.manji
	local upNode = self.m_csNode.weiwang
	local jinshengNode = upNode.jinsheng
	if fulllevel then
		manji:setVisible(true)
		upNode:setVisible(false)
		updateLv(manji.weiwang1,lvConfig)
		updateAttr(manji.full_attr,lvConfig,true)
	else
		manji:setVisible(false)
		upNode:setVisible(true)
		updateLv(upNode.weiwang1,lvConfig)
		updateAttr(upNode.before1,lvConfig,lvConfig.id~=0)

		updateLv(upNode.weiwang2,nextLvConfig)
		updateAttr(upNode.before2,nextLvConfig,false)

		local jinsheng = jinshengNode.jinshengbutton
		local lockTxt = jinshengNode.tips
		jinshengNode.jindu2:setPercent(mingWang/needMingWang*100)
		jinshengNode.num:setString(string.format("%d/%d",mingWang,needMingWang))
		local showLock = GamePlayer:getLevel()<needLv
		if showLock then
			lockTxt:setString(string.format(Serialize["Awake_FameUp_1"],needLv))	
		end
		lockTxt:setVisible(showLock)
		GameUtil:btnGray(jinsheng, not isCanUp)
	end

end

function This:chackIsCanup()
	local function checkLv()
		local level = GamePlayer:getLevel()
		local lv = GamePlayer:getMingWangLv()
		local lvConfig = RolePrestigeConfig[lv]
		local lwevelLimit = lvConfig.level
		if level< lwevelLimit then
			return false
		end
		return true
	end
	local function checkNeed()
		local mingWang = GamePlayer:getMingWangValue()
		local lv = GamePlayer:getMingWangLv()
		local lvConfig = RolePrestigeConfig[lv]
		if lvConfig then
			local needMingWang = lvConfig.value
			if needMingWang~=-1 and mingWang<needMingWang then
				return false
			end
		end
		return true
	end
	local isNeed = checkNeed()
	local isLv = checkLv()
	if not isNeed and not isLv then
		return false,Serialize["Awake_FameUp_5"]
	elseif not isNeed then
		return false,Serialize["Awake_FameUp_7"]
	elseif not isLv then
		return false,Serialize["Awake_FameUp_6"]
	end
	return true,""
end

function This:initTouchEvent()
	local jinsheng = self.m_csNode.weiwang.jinsheng
	jinsheng.jinshengbutton:setTouchEnded(function()
		local isCanUp,tipStr = self:chackIsCanup()
		if not isCanUp then
			-- print("me  ",tipStr)
			g_scene:showTips(tipStr)
			return false
		end
        GamePlayer:onGetMingWangUpRequest()
    end)
    local wenhao = jinsheng.bangzhu
    if wenhao then
        wenhao:setTouchEnded(function()
            local text = Serialize["Awake_FameUp_4"]
            g_scene:openDialog("UI.HelpTips", {text = text,node = wenhao})
        end)
    end
end

function This:onEnter()
    super.onEnter(self)
    GamePlayer:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    GamePlayer:removeObserver(self)
end

function This:getData()
	self.m_mingWang = GamePlayer:getMingWangValue()
	self.m_lv = GamePlayer:getMingWangLv()
    self.m_lvConfig = RolePrestigeConfig[self.m_lv]
    self.m_nextLvConfig = RolePrestigeConfig[self.m_lv+1]
end

function This:createEffect()
	local effect = nil
	effect = GameUtil:createEffect("KaPaiShengJi",false,nil,{
		onAnimationComplete = function()
			effect:removeFromParent()
		end
	})
	local animParent = self.m_csNode.anim
	if animParent then
		effect:setScale(0.96,0.77)
		animParent:addChild(effect)
	end
	cc.Audio:playEffect("Win.mp3")
end

function This:onGetMingWangUp()
	self:createEffect()
	self:refresh()
end
