-- @Author: zheng
-- @Date:   2018-08-20 14:50:08
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-09-03 17:07:06
-- @File Name: GameKingWay.lua
local MoralityLevelConfig = require("Metadatas.MoralityLevelConfig")
local KinglyWayConfig = require("Metadatas.KinglyWayConfig")

local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local strList= {
	endTitle = Serialize[60345],
}
KingWayAttrStr =Serialize[60346]
KingWayType ={
	kRen =1,
	kZhong =2,
	kZhi =3,
	kXin =4,
	kYi =5,
	kSheng =6,
}
KingImage ={}
KingImage[KingWayType.kRen] = "MenuScene/wordren3.png"
KingImage[KingWayType.kZhong] = "MenuScene/wordzhong3.png"
KingImage[KingWayType.kZhi] = "MenuScene/wordzhi3.png"
KingImage[KingWayType.kXin] = "MenuScene/wordxin3.png"
KingImage[KingWayType.kYi] = "MenuScene/wordyi3.png"
KingImage[KingWayType.kSheng] = "MenuScene/wordsheng3.png"

local This = class(...)

function This:create(id)
	local Morality = {}
	setmetatable(Morality,This)
	Morality:init(id)
	return Morality
end

function This:init(id)
	self.m_id = id
	self.m_isPass = false
	self.m_preKingWay = nil
	self.m_NexKingWay = nil
	self.m_maxUpdate = nil
	self.m_config = KinglyWayConfig[id]
	self.m_CurentData ={0,0,0,0,0,0}
	self:initCoinfig()
end

function This:initCoinfig( )
	local id = self.m_id
	local data = {}
	for k,config in pairs(MoralityLevelConfig) do
		local configId = config.stage
		local moralityId = config.morality_id
		local moralityLv = config.morality_level
		if configId == id then
			if not data[moralityId] then
				data[moralityId] = {}
			end
			local item = data[moralityId]
			item[moralityLv] = config
		end
	end
	self.m_data = data
	local dataCount = {}
	for index,item in ipairs(self.m_data) do
		dataCount[index] = table.count(item)
	end
	self.m_dataCount = dataCount
end

function This:getId()
	return self.m_id
end

function This:setPreKingWay(KingWay)
	self.m_preKingWay = KingWay
end

function This:getPreKingWay()
	return self.m_preKingWay
end

function This:setNexKingWay(KingWay)
	self.m_NexKingWay = KingWay
end

function This:getNexKingWay()
	return self.m_NexKingWay
end

function This:getName()
	return self.m_config.stage_name
end

function This:getPlayerLevelLimit()
	return self.m_config.open_level
end

function This:getNeed()
	return self.m_config.need_item
end

function This:getTitle(isAddColor)
	local lockStr = self.m_config.unlook
	local title = self.m_config.talent_title
	local res =""
	if isAddColor then
		if self:isPassed() then
			res = string.format("%s(%s)",title,strList.endTitle)
		else
			res = string.format("%s(%s)",title,lockStr)
		end
	else
		if self:isPassed() then
			res = string.format("%s(%s)",title,strList.endTitle)
		else
			res = string.format("%s(%s)",title,lockStr)
		end
	end
	return res,title
end

function This:getDes()
	return self.m_config.talent_des
end

function This:isCanUp()
	local res = false
	for kingType=1, KingWayType.kSheng do
		if self:getlvCanUpdate(kingType) then
			res = true
			break
		end
	end
	if self:isCanPo() then
		local temRes = true
		local needs = self:getNeed()
		for k,item in pairs(needs) do
			local resCount = self:getResCount(item[1])
			if resCount < item[2] then
				temRes = false
				break
			end
		end
		res = temRes
	end
	return res
end

--是否突破
function This:isCanPo()
	local res = true
	for KingWay,v in ipairs(self.m_CurentData) do
		if not self:getLvIsPass(KingWay) then
			res = false
			break
		end
	end
	return res
end

function This:getAttr()
	if self.m_attr then
		return self.m_attr,self.m_attrLv
	end
	local res ={}
	for KingWay,v in ipairs(self.m_CurentData) do
		local item =self:getLvAccumulateAttr(KingWay)
		self:tableAdd(item, res)
	end
	self.m_attrLv = table.clone(res)
	local item = self:convertData(self.m_config.arg)
	self:tableAdd(item, res)
	item = self:convertData(self.m_config.talent_arg)
	self:tableAdd(item, res)
	self.m_attr = res
	return self.m_attr,self.m_attrLv
end

function This:getCurentAtrr()
	local res ={}
	for KingWay,curentLv in ipairs(self.m_CurentData) do
		if curentLv>0 then
			local item =self:getLvAccumulateAttr(KingWay,curentLv)
			self:tableAdd(item, res)
		end
	end
	return res
end

function This:getPlayerTexture()
	return GamePlayer:getIcon()
end

function This:getResCount(id)
	local info = {itemUid = 0}
	local total = DepotManager:getItemOrResCount(id, info)
	return total
end

-----------------------------------------

function This:getLvMaxCanLock( )
	local max = 1
	if self.m_maxUpdate then
		return self.m_maxUpdate
	end
	max = nil
	for kingType=1,KingWayType.kSheng do
		if self:getLvIsLock(kingType) then
			max = kingType
			break
		end
	end
	if not max then
		max = KingWayType.kSheng
	end
	self.m_maxUpdate = max
	return self.m_maxUpdate
end

function This:getlvCanUpdate(KingWay)
	local res =false
	local maxCanLock = self:getLvMaxCanLock( )
	if KingWay>maxCanLock or (KingWay>1 and self:getLvCurrent(KingWay) == self:getLvCurrent(KingWay-1)) then
		return res
	end
	local curent,nextLv = self:getLvCurrent( KingWay )
	if curent == nextLv then
		return false
	end
	res =true
	local needs = self:getLvNeed(KingWay,nextLv)
	for k,item in pairs(needs) do
		local resCount = self:getResCount(item[1])
		if resCount < item[2] then
			res = false
			break
		end
	end
	return res
end

function This:getLvIsLock( KingWay)
	return self:getLvCurrent(KingWay)==0	
end

function This:getLvIsPass(KingWay)
	local curent,nextLv = self:getLvCurrent( KingWay )
	return curent == self:getLvMaxLv( KingWay )
end

function This:getLvData(KingWay,lv)
	return self.m_data[KingWay][lv]
end

function This:getLvName( KingWay ,lv )
	if not lv then
		lv = 1
	end
	local item = self:getLvData(KingWay,lv)
	return item.morality_name
end

function This:getLvNeed( KingWay ,lv )
	local item = self:getLvData(KingWay,lv)
	return item.need_item
end

function This:getLvDes( KingWay ,lv )
	local item = self:getLvData(KingWay,lv)
	return item.des
end

function This:getLvAtrr( KingWay ,lv )
	local item = self:getLvData(KingWay,lv)
	return self:convertData(item.arg)
end

function This:getLvMaxLv( KingWay )
	local res = self.m_dataCount[KingWay]
	if res then
		return res
	end
	return 0

end

function This:getLvCurrent( KingWay )
	local curent = self.m_CurentData[KingWay]
	if not curent then
		return 0,0
	end
	local nextLv =  curent+1
	local maxLv = self:getLvMaxLv(KingWay)
	if nextLv>maxLv then
		nextLv = maxLv
	end
	return curent,nextLv
end

function This:getLvAccumulateAttr(KingWay ,lv)
	local res ={}
	if not lv then
		lv = self:getLvMaxLv( KingWay )
	end
	if lv==1 then
		return self:getLvAtrr(KingWay ,lv)
	end
	for i=1,lv do
		local item = self:getLvAtrr(KingWay ,i)
		self:tableAdd(item, res)
	end
	return res
end

-------------------------------------------------
function This:convertDataToStr(srcTable, isSpace)
	local function getKey(data)
		local res = {}
		for k,v in pairs(data) do
			table.insert(res,k)
		end
		return res
	end
	local tableKey =getKey(srcTable)
	table.sort(tableKey,function(a,b)
		return a < b
	end)
	if isSpace then
		local str = ""
		for i,v in ipairs(tableKey) do
			local name,desc = self:getAttrStr(v, srcTable[v])
			str = str .. KingWayAttrStr .. name .. desc.."\n"
		end
		return str
	else
		local str1 = ""
		local str2 = ""
		for i,v in ipairs(tableKey) do
			local name,desc = self:getAttrStr(v, srcTable[v])
			str2 = str2 .. desc.."\n"
			str1 = str1 .. KingWayAttrStr .. name .."\n"
		end
		return {str1,str2},#tableKey
	end
end

function This:convertDataToStr2(srcTable, isSpace)
	local function getKey(data)
		local res = {}
		for k,v in pairs(data) do
			table.insert(res,k)
		end
		return res
	end
	local tableKey =getKey(srcTable)
	table.sort(tableKey,function(a,b)
		return a < b
	end)
	if isSpace then
		local str = ""
		for i,v in ipairs(tableKey) do
			local name,desc = self:getAttrStr(v, srcTable[v])
			str = str .. "<New1>".. KingWayAttrStr .. name .. "  " .. "<New9>" .. desc.."\n"
		end
		return str
	end

end


function This:getAttrStr( key,value )
	local name = AttrName[key]
	local attr = "+%d"
	if AttrInfo[key] and AttrInfo[key].valueType == ValueType.kPercent then
		value = value * 100
		attr = "+%d%%"
	end
	return name,string.format(attr, value)
end

function This:resetScoolColor2( node,colors,awake )
	if node and node.txt then
		if node.txt then
			local color = colors[2]
			if awake then
				color = colors[1]
			end
			node.txt:setColor(color.r,color.g,color.b)
		end
	end
end

function This:resetScoolSize2(node,widthLimit,_height ,text)
	if node and not node.txt then
		node.txt = node:getChildByName("txt")
	end
    if not _height or not node or not node.txt  then
        return
    end
    local height = 0

    local texRule1 = node.txt
    texRule1:setString(text,widthLimit/texRule1:getScaleX())
    local w ,h = texRule1:getContentSize()
    h = h*texRule1:getScaleY()
    height = height + h
    if height < _height then
        height = _height
    end
    texRule1:setPositionY(height)
    node:setInnerContainerSize(widthLimit,height)
end

function This:resetScoolSize(node,widthLimit,_height ,texts)
    if node and not node.txt then
    	local txt ={}
    	for i=1,2 do
    		local item = node["txt"..i]
    		txt[i]= item
    	end
      	node.txt =txt
    end
    if not _height or not node or not node.txt  then
        return
    end
    local txt =node.txt
    for i,v in ipairs(txt) do
    	v:setString(texts[i])
    end
    local height = 0

    local texRule1 = txt[1]
    local w ,h = texRule1:getContentSize()
    h = h*texRule1:getScaleY()
    height = height + h
    if height < _height then
        height = _height
    end
    for i,v in ipairs(txt) do
    	v:setPositionY(height)
    end
    node:setInnerContainerSize(widthLimit,height)
end

--两个表相加
function This:tableAdd( table1,res )
	for k,v in pairs(table1) do
		if res[k] then
			res[k] = res[k] + v
		else
			res[k] = v
		end
	end 
end


-- table转换
function This:convertData( tableData )
	return GameUtil:convertData( tableData )
end

function This:isLock()
	return self:getPlayerLevelLimit()>GamePlayer:getLevel()
end

function This:isPassed()
	return self.m_isPass
end

function This:getLevelConfig(id)
	return MoralityLevelConfig[id]
end

function This:setData(data)
	if data then
		self.m_maxUpdate = nil
		local function resetCurentData(  )
			local wayCount = table.count(KingWayType)
			for i=1,wayCount do
				self.m_CurentData[i] = self:getLvMaxLv(i)
			end
		end
		local list = data.id_list
		local listCount = table.count(list)
		local stage = data.stage
		if listCount ==0 then
			stage= stage+1
			if not self:getNexKingWay() then
				stage= stage-1
			end
		end
		local breakStage = data.break_stage
		if breakStage then
			self.m_isPass = breakStage >= self:getId()
			if self.m_isPass then
				resetCurentData()
			end
		end
		if stage == self:getId() then
			if listCount>0 then
				for i,configId in ipairs(list) do
					local item = self:getLevelConfig(configId)
					local moralityId = item.morality_id
					local moralityLv = item.morality_level
					self.m_CurentData[moralityId] = moralityLv
				end
			end
		end
	end
end
