-- @Author: zheng
-- @Date:   2018-08-20 15:10:19
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-25 14:40:13
-- @File Name: GameKingWayManager.lua

local This = class(...)
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")

local KinglyWayConfig = require("Metadatas.KinglyWayConfig")
local GameKingWay = require("Awake.GameKingWay")

local HeroManager = require("Hero.HeroManager")
local Socket = require("Net.Socket")
local EventDispatcher = require("EventDispatcher")
local Serialize = require("Serialize.Serialize")


function This:init()
	self.m_kingWay ={}
	self.m_kingData = nil
	for id , config in pairs(KinglyWayConfig) do
		self.m_kingWay[id]= GameKingWay:create(id)
	end
	for id , kingWay in pairs(self.m_kingWay) do
		local preStage = self.m_kingWay[id-1]
		kingWay:setPreKingWay(preStage)
		local nexChapter = self.m_kingWay[id + 1]
		if nexChapter then
			kingWay:setNexKingWay(nexChapter)
		end
	end

	self.m_observers = {}
	self:getAllAttr()
	self:initEventListener()
end

function This:getKingData()
	return self.m_kingWay
end

function This:getKingWay(index)
	if index then
		return self.m_kingWay[index]
	end
	if not index then
		if self.m_kingData then
			return self.m_kingData
		end
		for i,kingWay in ipairs(self.m_kingWay) do
			if not kingWay:isPassed() then
				self.m_kingData = kingWay
				break
			end
		end
		if not self.m_kingData then
			self.m_kingData = self.m_kingWay[#self.m_kingWay]
		end
		return self.m_kingData
	end
end

function This:isHaveLevelLimit()
	local level = GamePlayer:getLevel()
	local levelLimit = ArgConfig.kingcraft_open_level.arg[1]
	return level < levelLimit,levelLimit
end

function This:isLock(  )
	local kingData = self:getKingWay()
	return kingWay:isLock()
end

function This:getNexKingWay( kingWay )
	local res = true
	if not kingWay then
		kingWay = self:getKingWay()
	end
	local nextKingWay = kingWay:getNexKingWay()
	if not nextKingWay then
		res = false
		nextKingWay = kingWay
	end
	return nextKingWay,res
end

function This:getPreKingWay( kingWay )
	local res = true
	if not kingWay then
		kingWay = self:getKingWay()
	end
	local preKingWay = kingWay:getPreKingWay()
	if not preKingWay then
		res = false
		preKingWay = kingWay
	end
	return preKingWay,res
end

function This:getAllAttr()
	local res ={}
	for i,kingWay in ipairs(self.m_kingWay) do
		local item = kingWay:getAttr()
		kingWay:tableAdd(item, res)
	end
	return res
end

-- 当前
function This:getCurentAtrr()
	local res ={}
	for i,kingWay in ipairs(self.m_kingWay) do
		local item = kingWay:getAttr()
		if kingWay:isPassed() then
			local item = kingWay:getAttr()
			kingWay:tableAdd(item, res)
		else
			local item =kingWay:getCurentAtrr()
			kingWay:tableAdd(item, res)
		end
	end
	return res
end

-- 当前 and 当前满阶（只有突破用到）
function This:getCurentBreakAtrr()
	local res ={}
	local kingData = self:getKingWay(index)
	local id = kingData:getId()
	local isPasse = kingData:isPassed()
	if kingData:isCanPo() then
		local res2 = {}
		for kingIndex=1,id do
			local kingWay = self.m_kingWay[kingIndex]
			if kingWay then
				if kingWay:isPassed() then
					local item = kingWay:getAttr()
					kingWay:tableAdd(item, res)
				else
					res2 = table.clone(res)
					local item = kingWay:getAttr()
					kingWay:tableAdd(item, res2)

					item =kingWay:getCurentAtrr()
					kingWay:tableAdd(item, res)
					break
				end
			end
		end
		return res,res2,isPasse
	end
	return res,res,isPasse
end

function This:setKingWayData(data )
	for i,kingWay in ipairs(self.m_kingWay) do
		kingWay:setData(data)
	end
	self.m_kingData = nil
	self:getKingWay()
	HeroManager:updateWangDao()
end

function This:isHaveKingWay()
	local data = self:getKingWay(1)
	return not data:getLvIsLock(1)
end

----------------------------------------------------------
function This:initEventListener()
	EventDispatcher:addListener(self, 12701, self.onGetKingWayReply)
	EventDispatcher:addListener(self, 12700, self.onGetKingInfoReply)
end

function This:removeEventListener()
	EventDispatcher:removeListener(self,12701)
	EventDispatcher:removeListener(self,12700)
end

function This:addObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			return 
		end
	end
	table.insert(self.m_observers,observer)
end

function This:removeObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			table.remove(self.m_observers,i)
			return
		end
	end
end

function This:notifyObserver(funcName,...)
	local tmp = {}
	for i, observer in pairs(self.m_observers) do
		tmp[i] = observer
	end

	for i , observer in pairs(tmp) do
		if observer[funcName] then
			observer[funcName](observer,...)
		end
	end
end
----------------------------------------------------------------------
--op_type  操作类型 - 1:激活/升级(传六德ID) 2:一键升级(传六德ID)  3:突破
function This:onGetKingWayRequest(op_type,kingWay)
	Socket:sendAndLock(12701,{op_type = op_type,morality_id = kingWay})
end

function This:onGetKingWayReply( msg )
    if msg.err_code == 0 then
    	local kingInfor = msg.kingly_way_info
        if kingInfor then
        	self:setKingWayData(kingInfor)
        end
        self:notifyObserver("onGetKingWay",msg.op_type==3)
        EventDispatcher:dispatcher("kingway_refresh")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetKingInfoRequest()
	Socket:sendAndLock(12700)
end

function This:onGetKingInfoReply( msg )
    local kingInfor = msg.kingly_way_info
    if kingInfor then
    	self:setKingWayData(kingInfor)
    end
    self:notifyObserver("kingway_refresh")
end