-- @Author: zheng
-- @Date:   2019-05-09 15:46:27
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-15 15:39:28
-- @File Name: GameLordEquip.lua

local This = class(...)
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")
local LordEquipManager = require("Awake.LordEquipManager")
local DepotManager = require("Depot.DepotManager")

local LordEquipConfig = require("Metadatas.LordEquipConfig")
local ItemConfig = require("Metadatas.ItemConfig")
function This:create(...)
    local chapter = setmetatable({},self)
    chapter:init(...)
    return chapter
end

function This:init(id)
    self.m_id = id
    self.m_config = LordEquipConfig[id]
end

function This:getId()
    return self.m_id
end

function This:getPos()
    return self.m_config.solt_id
end

function This:getAttr()
    return GameUtil:convertData(self.m_config.attribute)
end

-- function This:getBacks()--分解返还
--    return self.m_config.decompose_get
-- end

function This:getLvLimit()
   return self.m_config.level
end

function This:getUpTargetData()
    return LordEquipManager:getEquip(self.m_config.target_item)
end

function This:isCanUp()
    return DepotManager:getItemOrResEnought(self:getUpNeed())
end

function This:getUpNeed()
    return self.m_config.advanced_item
end


function This:getSuitCondition()
    return self.m_config.condition or {}
end

function This:getSuitId()
   return self.m_config.suit_id 
end

function This:getSuitInfo()
    local result = LordEquipManager:getSuitItemAttr(self:getSuitId())
    local haveSuits = {}
    local count = 0
    if self:isHaveSuit() then
        local removeEquipKeyList ={}
        local maxPos = LordEquipManager:getMaxPos()
        for pos=1,maxPos do
            local posData = LordEquipManager:getPos(pos)
            local equipId = posData:getEquipId()
            if equipId then
                removeEquipKeyList[equipId] = equipId
            end
        end
        local conditionList = self:getSuitCondition()
        for _,equipId in ipairs(conditionList) do
            if removeEquipKeyList[equipId] then
                count = count+1
                haveSuits[equipId]= equipId
            end
        end
    end
    return result,haveSuits,count
end

function This:isHaveSuit()
    return self:getSuitId() ~=0
end


function This:isLock()
    return self:getLvLimit()>GamePlayer:getLevel()
end

function This:getAttrScore()
    if not self.m_score then
        self.m_score = HeroManager:getAttrKeyFightPower(self:getAttr())
    end
    return self.m_score or 0
end

function This:getName()
    if self.m_name then
       return self.m_name
    end
    local item1 = ItemConfig[self:getId()]
    if item1 then
        self.m_name  = item1.name
        return self.m_name
    end
    return ""
end

function This:getQuality()
    local item1 = ItemConfig[self:getId()]
    if item1 then
        return item1.quality
    end
    return 1
end

function This:getDesc()
    local item1 = ItemConfig[self:getId()]
    if item1 then
        return item1.comment
    end
    return ""
end

