-- @Author: zheng
-- @Date:   2019-05-09 15:46:27
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-12-23 14:47:01
-- @File Name: GameLordEquipPos.lua
local This = class(...)
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")
local LordEquipManager = require("Awake.LordEquipManager")

local LordEquipSlotUnlockConfig = require("Metadatas.LordEquipSlotUnlockConfig")
function This:create(...)
    local chapter = setmetatable({},self)
    chapter:init(...)
    return chapter
end

function This:init(id)
    self.m_id = id
    self.m_config = LordEquipSlotUnlockConfig[id]
    self.m_inscriptionKeyList ={}
    self.m_equipUid = 0
    -- self.m_lv = 5
    -- self.m_breakLv = 2
    self.m_lv = 0
    self.m_breakLv = 0
end

function This:getId()
    return self.m_id
end

function This:getPos()
    return self:getId()
end

--默认道具ID
function This:getNormalEquipId()
    return self.m_config.item_id
end

function This:getEquipUid()
    -- return 110015
    if self.m_equipUid == 0 then
        return nil
    end
    return self.m_equipUid
end

function This:getEquipId(uid)
    local temUid = uid or self:getEquipUid()
    local bagItem = LordEquipManager:getBagItemWithUid(temUid)
    if bagItem then
        return bagItem:getId()
    end
    return 0
end

function This:getEquipData(uid)
    local temUid = uid or self:getEquipUid()
    return LordEquipManager:getBagItemWithUid(temUid,true)
end

function This:getLv()
   return self.m_lv 
end

function This:getBreakLv()
   return self.m_breakLv
end

function This:isCanBreak()
    if self:isLock() then
        return false
    end
    local temCanUp,tipStr,tipCode = self:getBreak():isCanUp()
    if not self:getBreak(self:getBreakLv()+1) then
        return false,Serialize[60353],LorderEquipTip.kFull
    end
    -- print("breakTipStr ",tipStr, "   ",tipCode)
    return temCanUp,tipStr,tipCode
end

function This:isCanUp(checkOpenLv)
    if self:isLock() then
        if checkOpenLv then
             GamePlayer:checkOpenlvLimit(50,nil,true) 
             return false
        end
        return false
    end
    local temCanUp,tipStr,tipCode = self:getEquipLv():isCanUp()
    if not self:getBreak(self:getBreakLv()+1) then
        return false,Serialize[60353],LorderEquipTip.kFull
    end
    -- print("lvTipStr ",tipStr, "   ",tipCode)
    return temCanUp,tipStr,tipCode
end

function This:getBreak(lv)
    local temLv = lv or self:getBreakLv()
    return LordEquipManager:getBreak(self:getPos(),temLv)
end

function This:getEquipLv(lv)
    local temLv = lv or self:getLv()
    return LordEquipManager:getequipLv(self:getPos(),temLv)
end

function This:getAttr(equipId,lv,breakLv)
    local temEquipId = equipId or self:getEquipId()

    local temLvData = self:getEquipLv(lv)
    local lvAttrList = {}
    if temLvData then
        lvAttrList = temLvData:getAttr()
    end
    local temBreakData = self:getBreak(breakLv)
    local attrRate = 1
    if temBreakData then
        attrRate = temBreakData:getRate()
    end
    for key,v in pairs(lvAttrList) do
        lvAttrList[key] = (1+attrRate)*v
    end
    local equipAttr ={}
    local temCondition = lv or breakLv
    if not temCondition then
        local equipItem = LordEquipManager:getEquip(temEquipId)
        if equipItem then
            equipAttr = equipItem:getAttr()
        end
    end
    lvAttrList = GameUtil:tableAdd(equipAttr,lvAttrList)
    return lvAttrList
end

function This:getName()
   return self.m_config.name 
end

function This:getLvLimit()
    -- return 1
   return self.m_config.unlock_level
end

function This:isLock()
    local res = self:getLvLimit()>GamePlayer:getLevel()
    if self:getEquipUid() then
       res = false 
    end
    return res
end

function This:getOverviewRed()
    local res = self:isHaveOtherBetterEquip()
    return res
end

function This:getStrengthenRed()
    local isCanReak = self:isCanBreak()
    local isCanUp = self:isCanUp()
    return isCanReak or isCanUp
end

function This:getUpRed()
    local equipData = self:getEquipData()
    if equipData then
       if not equipData:getUpTargetData() or not equipData:isCanUp()  then
            return false
        end 
    end
    return equipData
end

function This:getAttrScore(equipId)
    local inscriptionAttr = {}--位置铭文的属性
    local lvBreakAttr =self:getAttr(equipId)
    lvBreakAttr = GameUtil:tableAdd(inscriptionAttr,lvBreakAttr)
    return HeroManager:getAttrKeyFightPower(lvBreakAttr)
end

function This:getAllMyTypeEquipBagItem(isSort)
    return LordEquipManager:getAllEquipBagItemWithSort(isSort,self:getPos())
end

function This:getBetterEquip()
    local function compareFun(aData,bData)
        local aQualityNum,bQualityNum  =aData:getQuality(),bData:getQuality()
        local score= aQualityNum>bQualityNum
        if aQualityNum== bQualityNum then
            return aData:getLvLimit()>bData:getLvLimit()
        end
        return score
    end
    local equipItem = LordEquipManager:getBagItemWithUid(self:getEquipUid(),true)
    local myEquipList =self:getAllMyTypeEquipBagItem()
    if #myEquipList ==0 then
       return equipItem,equipItem 
    end
    local maxEquip = equipItem
    for _,bagEquipItem in ipairs(myEquipList) do
        local equipData = LordEquipManager:getEquipDataWithBagItem(bagEquipItem)
        -- if not maxEquip or equipData:getAttrScore()>maxEquip:getAttrScore()  then
        if not maxEquip or compareFun(equipData,maxEquip)  then
            maxEquip = equipData
        end
    end
    return maxEquip,equipItem
end

function This:isHaveOtherBetterEquip()
    local maxEquip, equipItem =self:getBetterEquip()
    if not equipItem or not maxEquip then
        return maxEquip
    end
    return maxEquip ~= equipItem
end

function This:updateInfo(info)
    local infoPos = info.pos
    if infoPos and infoPos == self:getId()  then
        self.m_lv = info.strength_level or self.m_lv
        self.m_breakLv = info.breakthrough_level or self.m_breakLv
        self.m_equipUid = info.item_uid or self.m_equipUid
    end
end

function This:initInscriptionKeyList(indexList,valueList)
    if indexList and valueList then
        for index,key in ipairs(indexList) do
            self:setInscriptionKeyItem(key,valueList[index])
        end
    end
end

function This:setInscriptionKeyItem(key,value)
    if key and value then
        self.m_inscriptionKeyList[key] = value
    end
end

function This:getInscriptionKeyItem(key)
    return self.m_inscriptionKeyList[key]
end

function This:getInscriptionKeyList()
   return self.m_inscriptionKeyList 
end
