-- @Author: zheng
-- @Date:   2018-07-19 17:30:53
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-03 17:06:38
-- @File Name: HeroAwak.lua

AwakdValue= {
	startLimit = 20,--最大星位数量
	ParagraphLimit = 3,--最大觉醒阶段
}

local This = class(...)

local AwakeConfig = require("Metadatas.AwakeConfig")
local ParagraphConfig = require("Metadatas.ParagraphConfig")
local ParagraphLevelConfig = require("Metadatas.ParagraphLevelConfig")
local SkillConfig = require("Metadatas.SkillConfig")

local GameUtil = require("Util.GameUtil")

function This:create(hero)
	local o = {}
	setmetatable(o,self)
	o:init(hero)
	return o
end

function This:init(hero)
	self.m_hero = hero
	self.m_heroId = self.m_hero:getId()
	self:initData( )
end

function This:initData( )
	local heroId = self.m_heroId
	local function getAwake( ) 
		local res ={}
		for k,v in pairs(AwakeConfig) do
			local hero_id = v.hero_id
			if heroId == hero_id then
				if not res[hero_id] then
					res[hero_id] = {}
				end
				local item = res[hero_id]
				item[v.stage] = v
			end
		end
		return res
	end
	local function getParagraphLevel()--把ParagraphLevelConfig 根据英雄id进行分组
		local res ={}
		for k,v in pairs(ParagraphLevelConfig) do
			local hero_id = v.hero_id
			if heroId == hero_id then
				local paragraph = v.paragraph
				local paragraph_level = v.paragraph_level
				if not res[hero_id] then
					res[hero_id] = {}
				end
				local item = res[hero_id]
				if not item[paragraph] then
					item[paragraph] ={}
				end
				local temItem = item[paragraph]
				temItem[paragraph_level]= v
			end
		end
		return res
	end
	self.m_paragraphLevelConfig = getParagraphLevel() --星位获得的属性列表,使用hero索引
	self.m_paragraphData = {} --三个阶段完全觉醒获得属性加成
	self.m_awakeConfig = getAwake() --优化后的awakeconfig
end

--获得星位信息
function This:getStarInfo( starIndex )
	local data = ParagraphConfig[starIndex]
	if data then
		return data
	end
	return {}
end

--获取某个星位从1到 levelLimit 累加的属性加成
function This:getStarAccumulateAttr(heroId, starIndex,levelLimit )
	local res ={}
	if not levelLimit then
		levelLimit = self:getStarMaxLevel( starIndex )
	end
	if levelLimit==1 then
		return self:getAwakeStarAttr(heroId, starIndex, levelLimit)
	end
	for i=1,levelLimit do
		local item = self:getAwakeStarAttr(heroId, starIndex, i)
		self:tableAdd(item, res)
	end
	return res
end

--获取某个星位从1到 levelLimit 所需返还的材料
function This:getStarAccumulateBack(heroId, starIndex,levelLimit )
	local res ={}
	if not levelLimit then
		levelLimit = self:getStarMaxLevel( starIndex )
	end
	if levelLimit==1 then
		return self:getAwakeStarBack(heroId, starIndex, levelLimit)
	end
	for i=1,levelLimit do
		local item = self:getAwakeStarBack(heroId, starIndex, i)
		self:tableAdd(item, res)
	end
	return res
end


--星位升级获得属性加成
function This:getAwakeStarAttr(heroId, starIndex, level)
	return self:convertData(self.m_paragraphLevelConfig[heroId][starIndex][level].arg)
end


function This:getAwakeStarBack( heroId, starIndex, level )
	return self:convertData(self.m_paragraphLevelConfig[heroId][starIndex][level].reback_item)
end


--星位升级消耗
function This:getAwakeStarNeed(heroId, starIndex, level)
	return self:convertData(self.m_paragraphLevelConfig[heroId][starIndex][level].need_item)
end

function This:getAwakeStarLuck( heroId, starIndex, level)
	return self.m_paragraphLevelConfig[heroId][starIndex][level].luckycount
end
--初始幸运值
function This:getInitLuck( heroId, starIndex, level)
	return self.m_paragraphLevelConfig[heroId][starIndex][level].probability
end

--某一阶段的觉醒配表数据
function This:getParagraphInfor(heroId,paragraph)
	local data = self.m_awakeConfig[heroId][paragraph]
	if data then
		return data
	end
	return {}
end

--获取某一阶段的星位列表
function This:getParagraphStarList( heroId,paragraph )
	local data = self:getParagraphInfor(heroId,paragraph)
	return data.paragraph_array
end

--获取某一阶段的所有属性加成
function This:getParagraphData(heroId, paragraph )
	local data = self.m_paragraphData[heroId]
	if data and self.m_paragraphData[heroId][paragraph] then
		data = self.m_paragraphData[heroId][paragraph]
		return table.clone(data)
	elseif self.m_awakeConfig[heroId] then
		if not self.m_paragraphData[heroId] then
			self.m_paragraphData[heroId] ={}
		end
		local temData = self:getParagraphAccumulateAttr(heroId, paragraph )
		self.m_paragraphData[heroId][paragraph]= temData
		return table.clone(temData)
	end
	return {}
end

--获取某一阶段的所有可能的属性加成
function This:getParagraphAccumulateAttr(heroId, paragraph )
	local data = self:getParagraphInfor(heroId,paragraph)
	local res = {}
	local starList = data.paragraph_array
	for k,v in pairs(starList) do
		if v then
			local maxStarLevel = self:getStarMaxLevel( v )
			local item = self:getStarAccumulateAttr(heroId,v,maxStarLevel)
			self:tableAdd(item, res)
		end
	end
	local canUse ,perfectAttrs = self:getParagraphPerfectInfor(heroId,paragraph)
	self:tableAdd(perfectAttrs, res)
	local temRes = table.clone(res)
	local awakeAttrs ,needitem = self:getParagraphAwakeInfor(heroId,paragraph)
	self:tableAdd(awakeAttrs, res)
	--技能觉醒属性
	local awakeAttr = self:convertData(data.awake_skill_arg)
	self:tableAdd(awakeAttr, res)
	return res,temRes
end

--计算某一阶段所有的属性消耗
function This:getParagraphAccumulateBack( heroId, paragraph )
	local data = self:getParagraphInfor(heroId,paragraph)
	local res = {}
	local starList = data.paragraph_array
	for k,v in pairs(starList) do
		if v then
			local maxStarLevel = self:getStarMaxLevel( v )
			local item = self:getStarAccumulateBack(heroId,v,maxStarLevel)
			self:tableAdd(item, res)
		end
	end
	local awakeAttrs ,needitem ,rebackNeed= self:getParagraphAwakeInfor(heroId,paragraph)
	self:tableAdd(rebackNeed, res)
	return res
end

--获取某一阶段的觉醒属性，觉醒消耗
function This:getParagraphAwakeInfor(heroId, paragraph )
	local data = self:getParagraphInfor(heroId,paragraph)
	local needitem = self:convertData(data.need_item)
	local attrs = self:convertData(data.arg)
	local rebackNeed = self:convertData(data.awake_reback_item)
	return attrs,needitem,rebackNeed
end

--获取某一阶段的完美度属性 perfectNum 喂num时 获取某一个完美度属性
function This:getParagraphPerfectInfor(heroId, paragraph,perfectNum )
	local data = self:getParagraphInfor(heroId,paragraph)
	local perfect = data.perfect
	local perfect_arg = data.perfect_arg
	if perfectNum then --获取某一个完美度属性
		for i,v in ipairs(perfect) do
			if perfectNum == v then
				local temAttrs = self:convertData(perfect_arg[i])
				return true ,temAttrs 
			end
		end
		return false,{}
	else--获取所有的完美度属性累加
		local res ={}
		for i,v in ipairs(perfect_arg) do
			local temAttrs = self:convertData(v)
			self:tableAdd(temAttrs, res)
		end
		return true ,res
	end
end
----------------------------------------------------------------------

--某一阶段真实的属性加成,isAwake 是否已经觉醒
function This:getParagraphCurrentAttr(heroId, starDatas,paragraph,isAwake )
	if isAwake then
		return self:getParagraphData(heroId, paragraph )
	end
	local res = {}
	if table.count(starDatas)==0 then
		return res
	end
	local data = self:getParagraphInfor(heroId, paragraph)
	local curentStarLevel = self:getCurrentLevelList(heroId, starDatas, paragraph)
	for i,v in pairs(curentStarLevel) do
		if v>0 then--过滤掉0级的星位
			local item = self:getStarAccumulateAttr(heroId, i, v )
			self:tableAdd(item, res)
		end
	end
	local perfect ,arrsResult,arrs = self:calculationPerFect(heroId, starDatas,paragraph )
	self:tableAdd(arrsResult, res)
	return res
end

--某一阶段所有要返还的材料
function This:getParagraphCurrentBack(heroId, starDatas,paragraph,isAwake )
	if isAwake then
		return self:getParagraphAccumulateBack( heroId, paragraph )
	end
	local res = {}
	if table.count(starDatas)==0 then
		return res
	end
	local data = self:getParagraphInfor(heroId, paragraph)
	local curentStarLevel = self:getCurrentLevelList(heroId, starDatas, paragraph)
	for i,v in pairs(curentStarLevel) do
		if v>0 then--过滤掉0级的星位
			local item = self:getStarAccumulateBack(heroId, i, v )
			self:tableAdd(item, res)
		end
	end
	return res
end


--计算觉醒的完美度 return 觉醒完美度，觉醒完美度累加，觉醒的属性列表
function This:calculationPerFect(heroId, starDatas, paragraph )
	local data = self:getParagraphInfor(heroId,paragraph)
	local res = {}
	local temAttrs = {}
	local starList = data.paragraph_array
	local maxLvCount =0
	local curentLvCount = 0
	for k,v in ipairs(starList) do
		if v then
			curentLvCount = curentLvCount + self:getStarLevel(starDatas, v)
			maxLvCount = maxLvCount + self:getStarMaxLevel( v )
		end
	end
	local perfectNum = curentLvCount/maxLvCount
	if perfectNum>0 then
		local perfect = data.perfect
		for i,v in ipairs(perfect) do
			if perfectNum>=v then
				local canUse ,perfectAttrs = self:getParagraphPerfectInfor(heroId, paragraph, v)
				table.insert(temAttrs,perfectAttrs)
			end
		end
		for i,v in ipairs(temAttrs) do
			self:tableAdd(v, res)
		end
		return perfectNum ,res,temAttrs
	else
		return 0 ,res,temAttrs
	end

end

--某一阶段星位是否满足条件星位是否满足解锁条件
--return res 某一阶段解锁星位列表，curentStarLevel 某一阶段真实的等级列表
function This:isStarUnlockList(heroId,starDatas,paragraph)
	local res = {}
	local data = self:getParagraphInfor(heroId,paragraph)
	local starList = data.paragraph_array
	local curentStarLevel = self:getCurrentLevelList(heroId,starDatas,paragraph)
	for k,v in pairs(curentStarLevel) do
		local starId = k
		local lastStarLv = curentStarLevel[k-1]-- 上一个星位的真实等级
		local starInfor = self:getStarInfo( starId )
		if starInfor.unlock_Paragraphlevel ==-1 or (lastStarLv and lastStarLv >=starInfor.unlock_Paragraphlevel ) then
			res[k] = true
		else
			res[k] = false
		end
	end
	return res,curentStarLevel
end

--某一阶段是否达到开启条件
function This:getParagraphUnlockLimit(paragraphSelect)
	local hero = self.m_hero
	local heroId = hero:getId()
	local starDatas,paragraph,fulllevel = self:getAwakeInfor()
	if paragraphSelect then
		paragraph = paragraphSelect
	end
	local data= self:getParagraphInfor(heroId,paragraph)
	local awakeLevelLimit = data.awake_level-- 觉醒等级限制
	local heroLevel = hero:getLevel()
	if heroLevel< awakeLevelLimit then
		return true ,string.format(Serialize["Awake_HeroAwake_1"],awakeLevelLimit)
	end
	return false ,""
end

--某一阶段真实的等级列表
function This:getCurrentLevelList(heroId, starDatas, paragraph)
	local data = self:getParagraphInfor(heroId,paragraph)
	local curentStarLevel = {}
	local starList = data.paragraph_array
	for k,v in ipairs(starList) do
		if v then
			curentStarLevel[v] = self:getStarLevel(starDatas, v)
		end
	end
	return curentStarLevel
end

--获取星位的实际星级
function This:getStarLevel(starDatas,starIndex)
	local res= starDatas[starIndex]
	if res then
		return res.star_level
	end
	return 0
end

--获取星位的实际幸运值
function This:getStarLuck(starDatas,starIndex)
	local res= starDatas[starIndex]
	if res then
		return res.star_lucky
	end
	return 0
end

function This:getStarMaxLevel(starIndex, curentStarLve)
	local starInfor = self:getStarInfo( starIndex )
	local maxlevel = starInfor.maxlevel
	if curentStarLve then
		if curentStarLve <=0 then
			return 0
		else
			return curentStarLve/maxlevel*100
		end
	end
	return maxlevel
end

--获取某一阶段觉醒描述
function This:getAwakeDes(heroId,paragraph)
	if paragraph<AwakdValue.ParagraphLimit then
		local data = self:getParagraphInfor(heroId,paragraph)
		return data.awake_skill_des
	end
	return ""
end

--获取所有阶段完全觉醒的技能描述
function This:getSkillDes(skillId)
	local skill = SkillConfig[skillId]
	if skill then
		return skill.name,skill.desp
	end
	return "",""
end

--两个表相加
function This:tableAdd( table1,res )
	for k,v in pairs(table1) do
		if res[k] then
			res[k] = res[k] + v
		else
			res[k] = v
		end
	end 
end

-- table转换
function This:convertData( tableData )
	return GameUtil:convertData( tableData )
end

function This:convertDataToStr(srcTable, isSpace)
	local function getKey(data)
		local res = {}
		for k,v in pairs(data) do
			table.insert(res,k)
		end
		return res
	end
	local tableKey =getKey(srcTable)
	table.sort(tableKey,function(a,b)
		return a < b
	end)
	if isSpace then
		local str = ""
		for i,v in ipairs(tableKey) do
			local name,desc = self:getAttrStr(v, srcTable[v])
			str = str .. name .. desc.."\n"
		end
		return str
	else
		local str1 = ""
		local str2 = ""
		for i,v in ipairs(tableKey) do
			local name,desc = self:getAttrStr(v, srcTable[v])
			str2 = str2 .. desc.."\n"
			str1 = str1 .. name .."\n"
		end
		return {str1,str2},#tableKey
	end
end

function This:getAttrStr( key,value )
	return GameUtil:getAttrStr( key,value )
end

function This:getAwakeInfor()
	local awakeData = self.m_hero:getAwakeData()
	return awakeData.starDatas,awakeData.level,awakeData.awake
end
------------------------------ui
function This:getAwakeTitle(heroId,paragraph)
	local paragraphData = self:getParagraphInfor(heroId,paragraph)
    local titleName = paragraphData.awake_skill_title
    if paragraph == AwakdValue.ParagraphLimit then
        local awakeSkillId = self.m_hero:getAwakenSkillId()
        local name ,des = self:getSkillDes(awakeSkillId)
        return string.format("%s%s",titleName,name),des
    else
        desc = self:getAwakeDes(heroId, paragraph)
        return titleName,desc
    end
end

function This:resetScoolColor2( node,colors,awake )
	if node and node.txt then
		if node.txt then
			local color = colors[2]
			if awake then
				color = colors[1]
			end
			node.txt:setColor(color.r,color.g,color.b)
		end
	end
end

function This:resetScoolSize2(node,widthLimit,_height ,text)
	GameUtil:resetScoolSize2(node,widthLimit,_height ,text)
end

function This:resetScoolSize(node,widthLimit,_height ,texts)
	GameUtil:resetScoolSize(node,widthLimit,_height ,texts)
end

--endregion
