-- @Author: zheng
-- @Date:   2018-12-15 16:56:52
-- @Last Modified by:   superm
-- @Last Modified time: 2020-10-30 15:45:32
-- @File Name: LordEquipManager.lua
local DepotManager = require("Depot.DepotManager")
local This , super = class(..., "Base.BaseModel")
-- local FamilyManager = require("Family.FamilyManager")
local HeroManager = require("Hero.HeroManager")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local Hero = require("Hero.Hero")
local InscriptionConfig = require("Metadatas.InscriptionConfig")
LorderEquipTip ={
    kEmpty = 0,
    kLevel = 1,
    kNeed = 2,
    kCondition = 3,
    kFull = 4,
}
LordEquipPanelTabName ={
    kOverview    = 1,--总览
    kStrengthen  = 2,--强化
    kInscription = 3,--铭文
    kUp          = 4,--进阶
    kCount       = 4,
}

LordEquipMasterPanelTabName ={
    kStrengthen  = 1,--强化
    kInscription = 2,--铭文
    kCount       = 2,
}
-- local GameScienceData = require("Family.GameScienceData")
-- local Signals = require("SignalSystem.Signals")
-- local SignalSystem = require("SignalSystem.SignalSystem")
local GameLordEquipPos = require("Awake.GameLordEquipPos")
local GameLordEquip = require("Awake.GameLordEquip")
local GameLordEquipBreak = require("Awake.GameLordEquipBreak")
local GameLordEquipLv = require("Awake.GameLordEquipLv")
local GameLordEquipInscriptionAttr = require("Awake.GameLordEquipInscriptionAttr")

local LordEquipSlotStrengthenConfig = require("Metadatas.LordEquipSlotStrengthenConfig")
local LordEquipSlotUnlockConfig = require("Metadatas.LordEquipSlotUnlockConfig")
local LordEquipSlotBreachConfig = require("Metadatas.LordEquipSlotBreachConfig")
local LordEquipConfig = require("Metadatas.LordEquipConfig")
local LordEquipResonanceConfig = require("Metadatas.LordEquipResonanceConfig")
local SlotStrengthenResonanceConfig = require("Metadatas.SlotStrengthenResonanceConfig")
local InscriptionAttributeValueConfig = require("Metadatas.InscriptionAttributeValueConfig")

function This:init()
    super.init(self)
    self:initListener()
    local function getItem(configs,gameItemData,isUsePos)
        local res ={}
        for id,_ in pairs(configs) do
            local stage = gameItemData:create(id)
            local pos = stage:getPos()
            if stage.getLv and not isUsePos then
                local lv = stage:getLv()
                if not res[pos] then
                    res[pos] = {}
                end
                local item1 = res[pos]
                item1[lv] = stage
            elseif isUsePos then
                res[pos] = stage
            else
                res[id] = stage
            end
        end
        return res
    end
    self.m_lvKeyList = getItem(LordEquipSlotStrengthenConfig,GameLordEquipLv)
    self.m_breakKeyList = getItem(LordEquipSlotBreachConfig,GameLordEquipBreak)
    self.m_equipKeyList = getItem(LordEquipConfig,GameLordEquip)
    self.m_posKeyList = getItem(LordEquipSlotUnlockConfig,GameLordEquipPos,true)
    self.m_inscriptionAttrForServerList = {}
    self.m_alabaster_list = {}
    self.m_inscriptionAddList ={} --在线增肌的铭文
    -- self.m_allEquip = {110114,110214,110314,110414,110115,110215,110315,110415,110116,110216}
    -- self.m_allEquip = {}--道具id

end

function This:initListener()
    EventDispatcher:addListener(self, 10448, self.onModifyLordEquipReply)--主公装备穿戴、卸载
    EventDispatcher:addListener(self, 10450, self.onUpPosLvReply)--强化槽位
    EventDispatcher:addListener(self, 10451, self.onUpPosBreakReply)--突破槽位
    EventDispatcher:addListener(self, 10458, self.onOneKeyBreakReply)--一键强化
    EventDispatcher:addListener(self, 10459, self.onGetInforReply)--主公装备总览
    EventDispatcher:addListener(self, 10460, self.onEquipUpReply)--主公装备进阶
    EventDispatcher:addListener(self, 10610, self.onMwenLockReply)--主公装备符文加解锁
    EventDispatcher:addListener(self, "equip_changed_add",self.onBagChangeAdd)
    EventDispatcher:addListener(self,10447,self.onModifyLordEquipInScriptionReply)--铭文穿戴，卸载
end

function This:removeListener()
    EventDispatcher:removeListener(self,10448)--主公装备穿戴、卸载
    EventDispatcher:removeListener(self,10450)--强化槽位
    EventDispatcher:removeListener(self,10451)--突破槽位
    EventDispatcher:removeListener(self,10458)--一键强化
    EventDispatcher:removeListener(self,10459)--主公装备总览
    EventDispatcher:removeListener(self,10460)--主公装备进阶
    EventDispatcher:removeListener(self,10610)--主公装备铭文加解锁
end

function This:getAllEquipBagItemWithSort(isSort,posIndex)
    local removeEquipKeyList ={}
    local maxPos = self:getMaxPos()
    for pos=1,maxPos do
        local posData = self:getPos(pos)
        local equipId = posData:getEquipUid()
        if equipId then
            removeEquipKeyList[equipId] = equipId
        end
    end
    local res ={}
    local allEquipId = self:getAllHaveBagEquip()
    for _,equip in ipairs(allEquipId) do
        local equipId= equip:getUid()
        local equipData = self:getEquipDataWithBagItem(equip)
        if removeEquipKeyList[equipId] then
            removeEquipKeyList[equipId] = nil
        elseif equipData then
            local isAdd = not posIndex
            if posIndex and equipData:getPos() == posIndex then
               isAdd = true
            end
            if isAdd then
                table.insert(res,equip)
            end
        end
    end
    if isSort then
       table.sort(res,function(a,b)
            local aData,bData = self:getEquipDataWithBagItem(a),self:getEquipDataWithBagItem(b)
            -- local score = aData:getAttrScore()>bData:getAttrScore()
            local aQualityNum,bQualityNum  =aData:getQuality(),bData:getQuality()
            local score= aQualityNum>bQualityNum
            if aQualityNum== bQualityNum then
                return aData:getId()>bData:getId()
            end
            return score
        end) 
    end
    return res
end

function This:getInscriptionWithSolt(solt)
    local res = {}
    local tmpList = DepotManager:getPropsItemListForInScription()
    for i,item1 in ipairs(tmpList) do
        if solt == item1:getSolt() and not item1:isEquipped() then
            table.insert(res,item1)
        end
    end
    table.sort(res,function(a,b)
        -- local score = aData:getAttrScore()>bData:getAttrScore()
        local aQualityNum,bQualityNum  =a:getQuality(),b:getQuality()
        local aRed ,bRed = self:inscriptHaveRed(solt,a:getUid()),self:inscriptHaveRed(solt,b:getUid())
        local score= aQualityNum>bQualityNum
        if aRed and not bRed then
            return true
        end
        if bRed  and not aRed then
            return false
        end
        return score
    end) 
    return res
    
end

-- 获取强化共鸣等级
function This:getStrengthenLv(lv)
    local attrs = {}
    local temLvItem = SlotStrengthenResonanceConfig[lv]
    if lv then
        if temLvItem then
           return GameUtil:convertData(temLvItem.attribute),lv,temLvItem.dec
        end
        return attrs,nil,""
    end
    local temLv = 0
    local lvKeyList ={}
    local maxPos = self:getMaxPos()
    for pos=1,maxPos do
        local posData = self:getPos(pos)
        lvKeyList[pos] = posData:getLv()
    end
    local function checkLvLimit(conditionList)
        for _, limitConfig in ipairs(conditionList) do
            local pos ,limitValue= limitConfig[1],limitConfig[2]
            if limitValue>lvKeyList[pos] then
                return false
            end
        end
        return true
    end
    for lvIndex,lvItem in ipairs(SlotStrengthenResonanceConfig) do
        if lvIndex>0 then
            if checkLvLimit(lvItem.condition) then
                temLv = lvIndex
            else
                break
            end
        end
    end
    -- temLv =1
    temLvItem = SlotStrengthenResonanceConfig[temLv]
    return GameUtil:convertData(temLvItem.attribute),temLv,temLvItem.dec
end

function This:getAllHaveBagEquip()
    return DepotManager:getItemListByType(ItemCommonType.kLordEquip)
    -- return self.m_allEquip
end

function This:getInscriptionAttr(pos,attrType,quality)
    self:initInscriptionAttr()
    if pos then
        if attrType and quality then
            return self.m_inscriptionAttrList[pos][attrType][quality]
        else
            return self.m_inscriptionAttrList[pos]
        end
    end
    return nil
end

function This:initInscriptionAttr()
    if not self.m_inscriptionAttrList then
        local res = {}
        for id,itemConfig in pairs(InscriptionAttributeValueConfig) do
            local stage = GameLordEquipInscriptionAttr:create(id)
            local pos = stage:getPos()
            local attrType = stage:getAttrType()
            local quality = stage:getQuality()
            if not res[pos] then
                res[pos] ={}
            end
            local posT = res[pos]
            if not posT[attrType] then
                posT[attrType] ={}
            end
            local attrT = posT[attrType]
            attrT[quality] = stage
        end
        self.m_inscriptionAttrList = res
    end
end

--铭文是不是完美的
function This:isPerfectItemId(itemId)
    if not self.m_PerfectKeyList then
        local res ={}
        for id,itemConfig in pairs(InscriptionConfig) do
            if #itemConfig.attribute>0 then
                res[itemConfig.item_id]= id
            end
        end
        self.m_PerfectKeyList = res
    end
    local perfectItem = self.m_PerfectKeyList[itemId]
    return perfectItem
end

-- --套装相关
-- function This:isHaveSuit(equipId)
--     if not self.m_equipSuit then
--         local equipSuitKey = {}
--         for index,configItem in pairs(LordEquipResonanceConfig) do
--             local conditionList = configItem.condition
--             for _,temEquipId in ipairs(conditionList) do
--                 equipSuitKey[temEquipId] = index
--             end
--         end
--         self.m_equipSuit = equipSuitKey
--     end
--     return self.m_equipSuit[equipId]
-- end

function This:getSuitAttr()
   local suitIdCounts ={}
    local maxPos = self:getMaxPos()
    for pos=1,maxPos do
        local posData = self:getPos(pos)
        local equipdata = posData:getEquipData(uid)
        if equipdata then
            local suitId = equipdata:getSuitId()
            if not suitIdCounts[suitId] then
                suitIdCounts[suitId] = 1
            else
                suitIdCounts[suitId] = suitIdCounts[suitId] +1
            end
        end
    end 
    local res = {}
    for suitid,suitIdItemCount in pairs(suitIdCounts) do
       local temAttr =  self:getSuitItemAttr(suitid,suitIdItemCount)
       res = GameUtil:tableAdd(temAttr,res)
    end
    return res
end

function This:getSuitItemAttr(suitId,suitNum)
    if not self.m_suitAttr then
        local suitattr = {}
        for id,itemConfig in pairs(LordEquipResonanceConfig) do
            local suitId = itemConfig.suit_id
            local num = itemConfig.num
            if not suitattr[suitId] then
                suitattr[suitId] = {}
            end
            suitIdT = suitattr[suitId]
            suitIdT[num] = itemConfig.attribute
        end
        self.m_suitAttr = suitattr
    end
    local allAtr = table.clone(self.m_suitAttr[suitId] or {})
    if suitId and suitNum then
        local res ={}
        for temSuitNum,temAttr in pairs(allAtr) do
            local temAttr2 = GameUtil:convertData(temAttr)
            if suitNum>=temSuitNum then
                res = GameUtil:tableAdd(temAttr2,res)
            end
        end
        return res
    end
    return allAtr
end

function This:getBagItemWithUid(uid,isGetEquipData)
    local bagItem = DepotManager:getItemByUid(uid)
    if bagItem and isGetEquipData then
       return self:getEquipDataWithBagItem(bagItem) 
    end
    return bagItem
end

function This:getEquipDataWithBagItem(bagEquipItem)
    if bagEquipItem then
        local equipId = bagEquipItem:getId()
        return self:getEquip(equipId)
    end
    return nil
end

function This:getEquip(equipId)
    return self.m_equipKeyList[equipId]
end

function This:getBreak(pos,lv)
    local item1 = self.m_breakKeyList[pos]
    if item1 then
        return item1[lv]
    end
    return nil
end

function This:getequipLv(pos,lv)
   local item1 = self.m_lvKeyList[pos]
    if item1 then
        return item1[lv]
    end
    return nil 
end

function This:updatePosInfo(pos,lv,breakLv,equipUid)
    local info ={pos =pos,strength_level = lv,breakthrough_level = breakLv,item_uid = equipUid}
    local posData = self:getPos(pos)
    if posData then
        posData:updateInfo(info)
    end
end

function This:updateAllPosInfo(allInfo)
    for _,posInfo in ipairs(allInfo) do
        local pos = posInfo.pos
        local posData = self:getPos(pos)
        if posData then
            posData:updateInfo(posInfo)
        end
    end
end

function This:updateAllPosInscription(allInfo)
    for _,posInfo in pairs(allInfo) do
        local pos = posInfo.pos
        local posData = self:getPos(pos)
        if posData then
            posData:initInscriptionKeyList(posInfo.index_list or {},posInfo.item_uid_list or {})
        end
    end
end


function This:toSetAlabasterList(info)
    self.m_alabaster_list={}
    local alabaster_list = info.alabaster_list
    --local indexList = alabaster_list[1].index_list
    for i,v in ipairs(alabaster_list) do
        self.m_alabaster_list[v.pos] = self.m_alabaster_list[v.pos] or {}
        self.m_alabaster_list[v.pos][1] = v.item_uid_list
        self.m_alabaster_list[v.pos][2] = v.index_list
    end

    self.m_alabaster_list = alabaster_list
    -- for heroId ,heroData in pairs(HeroManager:getAllHero()) do
    --     --print("heroId========="..heroId)
    --     local playerHero = HeroManager:getHero(heroId)
    --     local hero = Hero:create({
    --         heroId = heroId,
    --         level = playerHero:getLevel(),
    --         starLv = playerHero:getStarLv(),
    --         quality = playerHero:getQuality(),
    --         deputy = table.clone(playerHero:getDeputyWithPos())
    --     })
    --     if hero then
    --         hero:updateInscription(alabaster_list)
    --     end

    print("self.m_alabaster_listself.m_alabaster_listself.m_alabaster_listself.m_alabaster_list")
    --print_tmp(self.m_alabaster_list)

    local heros = HeroManager:getAllHero()
    for id , hero in pairs(heros) do
        if hero:isGot() then
            hero:updateInscription(alabaster_list)
        end
    end

    HeroManager:calcSaintFightPower()   


end

function This:toGetAlabsterList()
    return self.m_alabaster_list
end

function This:getAlabasterList(pos)
    if self.m_alabaster_list then
        return self.m_alabaster_list[pos]
    end
end

function This:getAllBasterList()
    return self.m_alabaster_list
end

function This:getPos(pos)
    return self.m_posKeyList[pos]
end

function This:getMaxPos()
    return #self.m_posKeyList
end

function This:onBagChangeAdd(list)
    local isAdd = false
    for _,item1 in pairs(list) do
        if item1:isInscription() then
            isAdd= true
            local solt = item1:getSolt()
            local soltList = self.m_inscriptionAddList[solt]
            if not soltList then
                soltList = {}
                self.m_inscriptionAddList[solt] = soltList
            end
            soltList[item1:getUid()] = solt
            -- table.insert(soltList,item1)
        end
    end
    if isAdd then
        EventDispatcher:dispatcher("lord_quip_change")
        self:notifyObservers("onEquipUp")
    end
end

function This:inscriptHaveRed(solt,uid)
    local res = false
   local soltList = self.m_inscriptionAddList[solt]
   if soltList and solt then
        if uid and not DepotManager:getItemByUid(uid) then
            self:removeRedItem(solt,uid)
            return res
        end
        if uid then
            res = soltList[uid] and DepotManager:getItemByUid(uid)
        else
            for index,item1 in pairs(soltList) do
               res = DepotManager:getItemByUid(index)
                if res then
                    break
                end
            end
        end
    end
    return res
end

function This:removeRedItem(solt,uid)
   local soltList = self.m_inscriptionAddList[solt]
   if soltList and uid then
        soltList[uid]= nil
        EventDispatcher:dispatcher("lord_quip_change")
        self:notifyObservers("onEquipUp")
   end
end

function This:isHaveRedTabType(tabType,posIndex)
    local function checkRed(checkFun)
        local maxPos = self:getMaxPos()
        for pos=1, maxPos do
            local posData = self:getPos(pos)
            if posData:getLvLimit() <= GamePlayer:getLevel() and checkFun and checkFun(posData) then
                return true
            end
        end
        return false
    end
    local switch ={
        [LordEquipPanelTabName.kOverview]=function (posData)-- for case 1
                    return posData:getOverviewRed()
                end,
        [LordEquipPanelTabName.kStrengthen]=function (posData)
                    return posData:getStrengthenRed()
                end,
        [LordEquipPanelTabName.kUp]=function (posData)
                    return posData:getUpRed()
                end,
        [LordEquipPanelTabName.kInscription]=function (posData)
                    return self:inscriptHaveRed(posData:getPos())
                end,
    }
    local switchFun = switch[tabType]
    if tabType and switchFun then
        local posData = self:getPos(posIndex)
        if posData then
            if posData:getLvLimit() > GamePlayer:getLevel() then
                return false
            end
            return switchFun(posData)
        else
           return checkRed(switchFun) 
        end
    end
    return false
end

function This:isHaveRed()
    local res = false
    local tabTypes = {LordEquipPanelTabName.kOverview,LordEquipPanelTabName.kStrengthen,LordEquipPanelTabName.kUp,LordEquipPanelTabName.kInscription}
    for _,tabType in ipairs(tabTypes) do
        res = self:isHaveRedTabType(tabType,posIndex)
        if res then
            return res
        end
    end
    return res
end

function This:updateLordAttr()
    HeroManager:updateMingWang()
end

--自己评级的军衔属性加成
function This:getLordAttr()
    local res = {}
    local maxPos = self:getMaxPos()
    for pos=1,maxPos do
        local posData = self:getPos(pos)
        local posAttr = posData:getAttr()
        res = GameUtil:tableAdd(posAttr,res)
    end
    local suitAttr = self:getSuitAttr()
    res = GameUtil:tableAdd(suitAttr,res)
    local curentAttr,curentLv,desc = self:getStrengthenLv()
    res = GameUtil:tableAdd(curentAttr,res)
    return res
end

--根据自己的评级的刷新所有武将属性点
function This:updateAllHeroLordAttr()
    -- HeroManager:updateRating()
end 
    
function This:onUpPosBreakRequest(pos)
    Socket:sendAndLock(10451,{pos = pos})--突破槽位
end

function This:onUpPosBreakReply( msg )
    if msg.err_code == 0 then
        self:updatePosInfo(msg.pos,nil,msg.breakthrough_level)
        self:notifyObservers("onUpPosBreak")
        EventDispatcher:dispatcher("lord_quip_change")
        self:updateLordAttr()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onUpPosLvRequest(pos)
    Socket:sendAndLock(10450,{pos = pos})--强化槽位
end

function This:onUpPosLvReply( msg )
    if msg.err_code == 0 then
        self:updatePosInfo(msg.pos,msg.strength_level)
        self:notifyObservers("onUpPosLv")
        EventDispatcher:dispatcher("lord_quip_change")
        self:updateLordAttr()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onModifyLordEquipRequest(pos,equipUid)
    Socket:sendAndLock(10448,{pos = pos,item_id = equipUid})--主公装备穿戴、卸载
end

function This:onModifyLordEquipReply( msg )
    if msg.err_code == 0 then
        local itemUid = msg.item_id
        self:updatePosInfo(msg.pos,nil,nil,itemUid)
        if itemUid and itemUid~=0 then
            g_scene:showTips(Serialize["Awake_LordEquipManager_1"])
        end
        EventDispatcher:dispatcher("lord_quip_change")
        self:notifyObservers("onModifyLordEquip",itemUid)
        self:updateLordAttr()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onModifyLordEquipInScriptionRequest(item_id,pos,index)
    print("item_id====="..item_id .. "======pos====="..pos .. "======index===="..index)
    Socket:sendAndLock(10447,{item_id = item_id,pos = pos,index = index})
end

function This:onModifyLordEquipInScriptionReply(msg)

    -- optional int32 err_code = 1;
    -- optional int64 item_id = 2;     // 传0表示卸载
    -- optional int32 pos = 3;
    -- optional int32 index = 4;   
    if msg.err_code == 0 then
        local posData = self:getPos(msg.pos)
        if posData then
            posData:setInscriptionKeyItem(msg.index,msg.item_id)
        end
        self:notifyObservers("onModifyLordEquipInScription",msg)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onOneKeyBreakRequest()
    Socket:sendAndLock(10458)--一键强化
end

function This:onOneKeyBreakReply( msg )
    if msg.err_code == 0 then
        self:updateAllPosInfo(msg.open_pos_list or {})
        self:notifyObservers("onOneKeyBreak")
        EventDispatcher:dispatcher("lord_quip_change")
        self:updateLordAttr()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onEquipUpRequest(equipUid)
    Socket:sendAndLock(10460,{equip_uid = equipUid})--主公装备进阶
end

function This:onEquipUpReply( msg )
    if msg.err_code == 0 then
        
        self:notifyObservers("onEquipUp")
        self:updateLordAttr()
        EventDispatcher:dispatcher("lord_quip_change")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetInforRequest()
    Socket:sendAndLock(10459)--主公装备总览
end

function This:setInscriptionAttrForSerever(info)--获取服务器下发的铭文属性
    local msg_alabaster_info = info.msg_alabaster_info
    local uid = info.id
    self.m_inscriptionAttrForServerList[uid] = self.m_inscriptionAttrForServerList[uid] or msg_alabaster_info
end

function This:getInscriptionAttrForSerever(uid)--获取服务器下发的铭文属性
    return self.m_inscriptionAttrForServerList[uid]
end

function This:getInscriptionAttrForSereverAll()
    return self.m_inscriptionAttrForServerList
end

function This:onGetInforReply( msg )
    if msg.err_code == 0 then
        local allInfo = msg.msg_role_equip_info
        local alabasterList = allInfo.alabaster_list or {}
        self:updateAllPosInfo(allInfo.open_pos_list or {})
        self:updateAllPosInfo(allInfo.equip_list or {})
        self:updateAllPosInscription(alabasterList)
        self:toSetAlabasterList(allInfo)
        self:notifyObservers("onGetInfor")
        EventDispatcher:dispatcher("lord_quip_change")
        self:updateLordAttr()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onMwenLockRequest(id)
    local item = DepotManager:getItemByUid(id)
    local lock = not item:isLocked()
    Socket:sendAndLock(10610,{item_id = id,lock_type = lock,type = 7})--主公装备总览
end

function This:onMwenLockReply(msg)
    if msg.err_code == 0 then
        local item = DepotManager:getItemByUid(msg.item_id)
        item:setLocked()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end