-- @Author: zheng
-- @Date:   2019-07-08 11:50:48
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-18 15:43:19
-- @File Name: LordEquipMasterNameplate.lua
local GameUtil = require("Util.GameUtil")
local LordEquipManager = require("Awake.LordEquipManager")
local LordEquipNameAttributeNode = require("Awake.LordEquipNameAttributeNode")
local InscriptionConfig = require("Metadatas.InscriptionConfig")
local LordEquipSlotBreachConfig = require("Metadatas.LordEquipSlotBreachConfig")
local GameLordEquipPos = require("Awake.GameLordEquipPos")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")

-- local This , super = class(...,"UI.HelpTips")
local This , super = class(...,"Dialog")
This.fileName = "MenuScene/NodeUI/LordEquipMasterNameplate.csb"


function This:onEnter()
    super.onEnter(self)
    LordEquipManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    LordEquipManager:removeObserver(self)
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    local w,h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w,h)
    local id = info.itemId
    self.m_isReplace = info.isReplace
    self.m_inScriptItemid = info.inScriptItemid
    self.m_equipId = info.equipId
    self.m_selectPos = info.m_selectPos
    self.m_equipMakeSelectPos = info.m_equipMakeSelectPos
    self.m_isPutOut = info.isPutOut

    self.m_id = id
    local isPerfect = LordEquipManager:isPerfectItemId(id)
    self.m_qualitys = {}
    self.m_attrs = {}
    self.m_pos = 1
    if isPerfect then
        local itemConfig = InscriptionConfig[isPerfect]
        local temAttr = itemConfig.attribute
        local res = {}
        for index,attr in ipairs(temAttr) do
            local attrId = attr[1]
            res[attrId] = {attr[1],attr[2],5}
        end
        self.m_attrs = res
        self.m_pos = itemConfig.solt
    else
        local tinscriptionAttr = LordEquipManager:getInscriptionAttrForSerever(self.m_inScriptItemid)
        self.m_pos = self.m_selectPos
        local typeList = tinscriptionAttr.attr_type_list
        local value_list =tinscriptionAttr.attr_value_list
        local quality_list =tinscriptionAttr.attr_quality_list
        local tmpList = {}
        for index,types in ipairs(typeList) do
            local attrId = types
            tmpList[attrId] = {types,value_list[index],quality_list[index]}
        end
        self.m_attrs = tmpList

        -- print_tmp(tmpList)
    end
	self:refresh()
end

function This:refresh()
    local csNode = self.m_csNode
    local clearNode = csNode.clear
    csNode.equipname:setString(ItemUtil.getName(self.m_id))
    -- csNode.equipname:setPositionX(-30)
    GameUtil:creatItemNode(csNode.fuwen,{self.m_id},false,-1)
    local posData = LordEquipManager:getPos(self.m_pos)
    csNode.equipdes:setString(Serialize[60375]..posData:getName(),170)
    local function resetItem(attrItem,index)
        local txt = clearNode["txt"..index]
        if not txt then
            return 
        end
        -- local star = clearNode["star"..index]
        if attrItem then
            local name,value = GameUtil:getAttrStr(attrItem[1],attrItem[2],true)
            local str = string.format("%s   %s",name,value)

            if txt then
                txt:setString(str)
                txt:setColorByID("Q".. attrItem[3])
            end
        end
        local isVisible = false or attrItem
        txt:setVisible(isVisible)
        -- star:setVisible(isVisible)
        if txt then
            txt:setVisible(isVisible)
        end
        if star then
            --star:setVisible(isVisible)
        end
    end
    local count = 0
    for index,attrItem in pairs(self.m_attrs) do
        count = count+1
        resetItem(attrItem,count)
    end
    if count<5 then
        for index=count+1,5 do
            resetItem(nil,index)
        end
    end
    csNode.genghuan:setVisible(self.m_isReplace)
    if self.m_isPutOut then
        csNode.genghuan.text:setString(Serialize[60376])
    end
    self.m_item = DepotManager:getItemByUid(self.m_inScriptItemid)
    csNode.suo:setVisible(self.m_item:isLocked())
    if self.m_item:isLocked() then
        csNode.suoding.text:setString(Serialize[60377])
    else
        csNode.suoding.text:setString(Serialize[60378])
    end
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.bg:setTouch({
            onEnded = function()            
                self:checkHideAttrNode()
            end,
            isScale = false,
            isDynamic = true,
            isSwallow = true,
        })
    csNode.xiangqing.xiangqingline:setTouch({
            onEnded = function()            
                -- print("cloose")
                local attrNode= self:createAttrNode()
                attrNode:setVisible(true)
            end,
            isScale = false,
            isDynamic = true,
            isSwallow = true,
        })
    csNode.fuwen.touch:setTouch({
        onEnded = function()
            
        end,
        isSwallow = true,
        isDynamic = true,
        -- name = "city"..self.m_city:getCityId(),
        isScale = false,
        isAlpha = false
    })
    csNode.genghuan:setTouch({
        onEnded = function()

            if self.m_isPutOut then
                self:toPutItOut()
            else
                self:toPutItOn()
            end
        end,
        isSwallow = true,
        isDynamic = true,
        -- name = "city"..self.m_city:getCityId(),
        isScale = false,
        isAlpha = false
    })
    csNode.suoding:setTouch({
        onEnded = function()
            LordEquipManager:onMwenLockRequest(self.m_inScriptItemid)
            self:close()
        end,
        isSwallow = true,
        isDynamic = true,
        -- name = "city"..self.m_city:getCityId(),
        isScale = false,
        isAlpha = false
    })
end

function This:toPutItOut()
    LordEquipManager:onModifyLordEquipInScriptionRequest(0,self.m_selectPos,self.m_equipMakeSelectPos)
end

function This:toPutItOn()
        if self.m_equipMakeSelectPos ~= 0 then
        local gameLordEquipPos = LordEquipManager:getPos(self.m_selectPos)
        local breaklv = gameLordEquipPos:getBreakLv()

        local isCanPutIt = true
        local isCanPutIt2 = true
        local tEquipSlotBreachConfig = nil
        local tEquipSlotBreachConfig2 = nil
        local index = 0
        local index2 = 0


        local tNewSlotList = {}
        for i,v in pairs(LordEquipSlotBreachConfig) do
            if v.solt_id == self.m_selectPos then
                table.insert(tNewSlotList,v)
            end
        end

        table.sort(tNewSlotList,function(a,b)
            return a.id < b.id
        end)

        for i,v in ipairs(tNewSlotList) do
            if v.solt_id == self.m_selectPos then
                if v.equip_num == 2 then
                    index = i
                    tEquipSlotBreachConfig = v
                    break
                end
            end
        end
        for i,v in ipairs(tNewSlotList) do
            if v.solt_id == self.m_selectPos then
                if v.equip_num == 3 then
                    index2 = i
                    tEquipSlotBreachConfig2 = v
                    break
                end
            end
        end

        for i=1,tEquipSlotBreachConfig.equip_num do
           if breaklv < tEquipSlotBreachConfig.level and i == 2 and self.m_equipMakeSelectPos == 2 then
                isCanPutIt = false
            end

            if tEquipSlotBreachConfig2 and breaklv < tEquipSlotBreachConfig2.level and i == 3 and self.m_equipMakeSelectPos == 3 then
                isCanPutIt2 = false
            end
        end

        if isCanPutIt then
            if self.m_equipId ~= 0 then
                if isCanPutIt2 then
                    LordEquipManager:onModifyLordEquipInScriptionRequest(self.m_inScriptItemid,self.m_selectPos,self.m_equipMakeSelectPos)
                else
                    g_scene:showTips(string.format(Serialize[60379],tEquipSlotBreachConfig2.level))
                end
            else
                g_scene:showTips(Serialize[60380])
            end
        else
            g_scene:showTips(string.format(Serialize[60379],tEquipSlotBreachConfig.level))
        end
    else
        g_scene:showTips(Serialize[60381])
    end
end

function This:createAttrNode()
    if not self.m_attrNode then
        local info= {attr =self.m_attrs,pos =self.m_pos}
        local attrNode = LordEquipNameAttributeNode:create(info)
        attrNode:setVisible(false)
        self.m_csNode:addChild(attrNode)
        local w,h = self:getContentSize()
        attrNode:setPositionX(w/2)
        self.m_attrNode = attrNode
    end
    return self.m_attrNode,self.m_attrNode:isVisible()
end

function This:checkHideAttrNode()
    local attrNode ,isVisible = self:createAttrNode()
    if isVisible then
        attrNode:setVisible(false)
        return true
    end
    return false
end

function This:onTouchMask()
    if not self:checkHideAttrNode() then
        self:close()
    end
end

function This:onModifyLordEquipInScription(msg)
    self:close()
end