-- @Author: zheng
-- @Date:   2019-01-24 20:01:08
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-02-20 15:50:20
-- @File Name: LordPanel.lua
local GameKingWayManager = require("Awake.GameKingWayManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local KingWayTab = require("Awake.KingWayTab")
local FameUpTab = require("Awake.FameUpTab")
local NobilityView = require("Nobility.NobilityView")
local FunctionNoticeManager = require("Function.FunctionNoticeManager")
local NobilityManager = require("Nobility.NobilityManager")
local ArmyTacticalManager = require("ArmyTactical.ArmyTacticalManager")
local ArmyTacticalView = require("ArmyTactical.ArmyTacticalView")
local DrumManager = require("Drum.DrumManager")
local DrumTab = require("Drum.DrumTab")
local This , super = class(...,"Dialog")

This.fileName = "MenuScene/NodeUI/KingWay.csb"
local TabName = 
{
    kKing = 1,--王道
    kFame = 2,--名望
    kNobility = 3, --爵位
	kArray = 4, --军阵
	kDrum = 5, -- 战鼓
}

local TabUnlock = {
	[TabName.kKing] = function ()
		local isOpen = FunctionNoticeManager:checkOpen(FunctionOpenId.kWangDao)
		local tips = FunctionNoticeManager:getOpenTips(FunctionOpenId.kWangDao)
		return isOpen, tips
	end,
	[TabName.kFame] = function ()
		local isOpen = FunctionNoticeManager:checkOpen(FunctionOpenId.kMingWang)
		local tips = FunctionNoticeManager:getOpenTips(FunctionOpenId.kMingWang)
		return isOpen, tips
	end,
	[TabName.kNobility] = function ()
		local isOpen = FunctionNoticeManager:checkOpen(FunctionOpenId.kJueWei)
		local tips = FunctionNoticeManager:getOpenTips(FunctionOpenId.kJueWei)
		return isOpen, tips
	end,
	[TabName.kArray] = function ()
		local isOpen = ArmyTacticalManager:checkOpen()
		local tips = ArmyTacticalManager:getOpenTips()
		return isOpen, tips
	end,
	[TabName.kDrum] = function ()
		local isOpen = DrumManager:checkOpen()
		local tips = DrumManager:getOpenTips()
		return isOpen, tips
	end,
}

function This:initNode(info)
	info = info or {}
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    self.m_kingNode = KingWayTab:create(self)
	self:addChild(self.m_kingNode)

	self.m_fameNode = FameUpTab:create(self)
	self:addChild(self.m_fameNode)

	self.m_nobilityNode = NobilityView:create(self)
	self.m_csNode:addChild(self.m_nobilityNode)

	self.m_arrayNode = ArmyTacticalView:create(self)
	self.m_csNode:addChild(self.m_arrayNode)

	self.m_drumNode = DrumTab:create(self)
	self.m_csNode:addChild(self.m_drumNode)

	self.m_inited = false
	for tabKey = 1, TabName.kDrum do
		local item = self.m_csNode["tep"..tabKey]
		self:addTabButton(tabKey, item, function()

			local func = TabUnlock[tabKey]
			if type(func) == "function" then
				local isOpen, tips = func()
				if not isOpen then
					g_scene:showTips(tips)
					return false
				end
			end
			
			self:lightTabButton(tabKey)
			self.m_selectIndex = tabKey
			self:refresh()
	    end)
	end
	self.m_tabNodes = {self.m_kingNode, self.m_fameNode, self.m_nobilityNode, self.m_arrayNode, self.m_drumNode}

	self.m_selectIndex = TabName.kNobility
	if info.tab then
		self.m_selectIndex = info.tab
	end

	self:selectTabButton(self.m_selectIndex)
	self:refresh()
	self:refreshLock()
	self:refreshDot()
	self:refreshNobilityRedDot()
	self.m_inited = true
end

function This:refreshDot()
	local csNode = self.m_csNode
	local function refreshDot(index,isHaveRed)
		local tep = csNode["tep"..index]
		if tep then
			local normal = tep
			GameUtil:refreshRedDot(normal,isHaveRed) 
		end
	end
	for tabKey=1,TabName.kFame do
		local isHaveRed = false
		if tabKey == TabName.kKing then
			isHaveRed = g_redDot:kingWayCanUp()
		elseif tabKey == TabName.kFame then
			isHaveRed = g_redDot:mingWangIsCanUp()
		end
		refreshDot(tabKey,isHaveRed)
	end
end

function This:refresh()
	for tabIndex,tabNode in ipairs(self.m_tabNodes) do
		local isRefresh = self.m_selectIndex == tabIndex
		tabNode:setVisible(isRefresh)
		if isRefresh then
			tabNode:refresh()
		end
	end

end

function This:refreshLock()
	local csNode = self.m_csNode

	for k = 1, TabName.kDrum do
		local lock = csNode[string.format("tep%d", k)].lock
		local func = TabUnlock[k]
		local isOpen = true

		if type(func) == "function" then
			isOpen = func()
		end

		if lock then
			lock:setVisible(not isOpen)
		end
	end
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.close:setTouchEnded(function()
		self:close()
	end)
end

function This:onEnter()
    super.onEnter(self)
    GamePlayer:addObserver(self)
    GameKingWayManager:addObserver(self)
    NobilityManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    GamePlayer:removeObserver(self)
    GameKingWayManager:removeObserver(self)
    NobilityManager:removeObserver(self)
end
This.onGetMingWangUp = This.refreshDot
This.onGetKingInfo = This.refreshDot
This.onGetKingWay = This.refreshDot


function This:refreshNobilityRedDot()
	local node = self.m_csNode.tep3
	if node then
		local isShow = NobilityManager:isCanUpgradeNobility() or NobilityManager:isHasReward()
		GameUtil:refreshRedDot(node, isShow)
	end
end

function This:onNobilityAchieveChange()
	self:refreshNobilityRedDot()
end

function This:onNobilityUpgradeReply()
	self:refreshNobilityRedDot()
end

function This:onNobilityRewardReply()
	self:refreshNobilityRedDot()
end