--region *.lua
--Date
--此文件由[BabeLua]插件自动生成
-- local ItemUtil = require("Bag.ItemUtil")
local This = class(...)

local ItemConfig = require("Metadatas.ItemConfig")
local EquipConfig = require("Metadatas.EquipConfig")
local TargetConfig = require("Metadatas.TargetConfig")
local OptionConfig = require("Metadatas.OptionConfig")
local BaseItemModel = require("Bag.BaseItemModel")
local EquipItemModel = require("Bag.EquipItemModel")
local FabaoItemModel = require("Bag.FabaoItemModel")
local SpallItemModel = require("Bag.SpallItemModel")
local PlanItemModel = require("Bag.PlanItemModel")
local PropsItemModel = require("Bag.PropsItemModel")
local LordEquipItemModel = require("Bag.LordEquipItemModel")
local RuneItemModel = require("Bag.RuneItemModel")
local InscriptionItemModel = require("Bag.InscriptionItemModel")
local TacticsItemModel = require("Bag.TacticsItemModel")

-- 道具分类，内部使用
ItemType = {
	kGameRes = 1,
	kProps = 2,
	kHero = 3,
	kHeroSpall = 4,
    kEquipSpall = 5,
    kEquip = 6,
    kPropsSpall = 7, --道具碎片
    kPlan = 8, --计策
	kPsvSkill = 9,  --被动技能
    kFaBao = 10, -- 法宝
}

-- 道具的通用逻辑分类
ItemCommonType = {
    kGameRes = 1,
    kEquip = 2,
    kProps = 3,
    kSpall = 4,
    kHero = 5,
    kPropPack = 6, --礼包
    kFaBao = 7,
    kSkin = 8,--皮肤相关
    kShenWen = 9, --神纹
    kLordEquip = 11,--主公装备
    kInscription = 12,--装备铭文(程序使用)
    kSpecialItem = 14,-- 使用触发特殊事件道具
    kRune = 15,--符石
    kTactics = 16, --战法
    kWarfare = 17, --战意令
    kHammer = 18, --神锤
}

-- 道具使用场景类型
ItemUsageType = {
    kNone = 0,
    kInTerritory = 1,
    kInWorld = 2,
    kInCity = 3,
    kInvisible = 4,
    kInEverywhere = 9,
    kUnknown = 10,
}

ItemOperation = {
    kReborn = 98,
    kSell = 99,
}

RecycleTargetType = {
    kHero = 1,
    kEquip = 2,
    kFabao = 3,
    kLordEquip = 4,
}

RecycleType = {
    kReborn = 1,
    kResolve = 2,
}

-- 需要统计数量的不叠加物品
NeedAloneCountType = {
    [ItemCommonType.kTactics] = true,
}

function This.createSimpleItemModel(id, count)
    return BaseItemModel.New({type_id=id, num=count})
end

function This.createItemModel(data)
    local itype = This.getType(data.type_id)
    if itype == ItemCommonType.kProps or itype == ItemCommonType.kPropPack then
        if This.isPlan(data.type_id) then
            return PlanItemModel.New(data)
        end
        return PropsItemModel.New(data)
    elseif itype == ItemCommonType.kEquip then
        return EquipItemModel.New(data, data.equip)
    elseif itype == ItemCommonType.kFaBao then
        return FabaoItemModel.New(data)
    elseif itype == ItemCommonType.kSpall then
        return SpallItemModel.New(data)
    elseif itype == ItemCommonType.kLordEquip then
        return LordEquipItemModel.New(data)
    elseif itype == ItemCommonType.kRune then
        return RuneItemModel.New(data)
    elseif itype == ItemCommonType.kInscription then
        return InscriptionItemModel.New(data)
    elseif itype == ItemCommonType.kTactics then
        return TacticsItemModel.New(data)
    end
    return BaseItemModel.New(data)
end

--方法类

function This.getType(id)
    local item_cfg = ItemConfig[id]
    if not item_cfg then
        print(id .. "道具不存在")
        return 0
    end
    return item_cfg.itype
end

function This.getComment(id)
    local item_cfg = ItemConfig[id]
    return  item_cfg.comment
end

function This.getItemGetWays(id)
    local item_cfg = ItemConfig[id]
    return  item_cfg.get_way
end

function This.getName(id)
    local item_cfg = ItemConfig[id]
    return  item_cfg.name
end

function This.getPrice(id)
    local item_cfg = ItemConfig[id]
    return  item_cfg.price
end

function This.getQuality(id)
    local item_cfg = ItemConfig[id]
    return  item_cfg and item_cfg.quality or 1
end

function This.getPriceMoneyType(id)
    local item_cfg = ItemConfig[id]
    return  item_cfg.mtype
end

function This.getPriceImage(id)
    local item_cfg = ItemConfig[id]
    return  This.getIconImage(item_cfg.mtype)
end  
    
function This.getTargetCost(id)
    local target_cfg = TargetConfig[id]
    if target_cfg then
        return target_cfg.need_item
    end   
end
    
function This.getTargetInfo(id)
    local target_cfg = TargetConfig[id]
    if target_cfg then
        return target_cfg.target_item
    end
end

function This.getIconImage(id)
    if id <= 0 then
        return
    end
    
    local item_cfg = ItemConfig[id]
    if item_cfg then
        return "Icons/Item/".. item_cfg.icon .. ".png"
    else
        return "Icons/Item/".. id .. ".png"
    end
end

function This.getQualityImage(id)
    if id == 0 then
        return "MenuScene/zbkong.png"
    else
        local quality = This.getQuality(id)
        if This.getType(id) == ItemCommonType.kFaBao then
            return "MenuScene/itemfabao" .. quality .. ".png"
        else
            return "MenuScene/itemQuality" .. quality .. ".png"
        end
    end
end

function This.isPlan(id)
    return id > 2000 and id < 3000
end

function This.isPassiveSkill(id)
    return id > 60000 and id < 60999
end

function This.isAnimal(id)
    return id > 100000 and id <1010000
end

function This.isAnimalSpall(id)
    return id > 101000 and id <102000
end


function This.isInscriptionBooks(id)
    return id > 1100 and id < 1110
end

function This.isInscription(id)
    return id >= 120011 and id <= 120085
    -- return itype == 12
end

function This.isItemVisible(id)
    local item_cfg = ItemConfig[id]
    if item_cfg.is_use then
        if item_cfg.is_use[1] and item_cfg.is_use[1] < ItemUsageType.kNone then
            return false
        end
    end
    return true
end

function This.getEquipAttrList( id )
    local config = EquipConfig[id]
    local attr = {}
    if config then
        for i = 1 , 4 do
            local attrId = config["attrId"..i]
            local value = config["value"..i][1] or 0
            if attrId > 0 and value then
                table.insert(attr,{attrId = attrId, value = value})
            end
        end
    end
    return attr  
end

function This.getPackageList( id )
    local config = OptionConfig[id]
    if config then
        return config.reward  
    end
end

--使用货币类型转化为道具id 
function This.getItemIdByMoneyType(money_type)
    if money_type == 8 then
        return 101
    elseif money_type == 9 then
        return 101
    elseif money_type == 10 then
        return 102
    elseif money_type == 12 then
        return 107
    elseif money_type == 13 then
        return 110
    elseif money_type == 5 then
        return 103
    elseif money_type == 26 then
        return 176
    elseif money_type == 35 then
        return 302
    else
        error("invalid money_type:" .. money_type)
    end
end

--获取道具分类
function This.getItemCategory(id)
    local config = ItemConfig[id]
   local itype = config.itype

   if itype and itype > 0 then
        if itype == ItemCommonType.kGameRes then
            return ItemType.kGameRes
        elseif itype == ItemCommonType.kEquip then
            return ItemType.kEquip
        elseif itype == ItemCommonType.kFaBao then
            return ItemType.kFaBao
        elseif itype == ItemCommonType.kProps or itype == ItemCommonType.kPropPack then
            if This.isPlan(id) then
                return ItemType.kPlan
            elseif This.isPassiveSkill(id) then
                return ItemType.kPsvSkill
            end
            return ItemType.kProps
        elseif itype == ItemCommonType.kSpall then
            local targetCfg = This.getTargetInfo(id)
            local targetId = 0
            if targetCfg then
                local _, targetItem = next(targetCfg)
                if targetItem then
                    targetId = targetItem[1]
                end
            end
            if targetId == 0 then
                return ItemType.kProps
            end

            local targetItemType = This.getType(targetId)
            if targetItemType == ItemCommonType.kEquip then
                return ItemType.kEquipSpall
            elseif targetItemType == ItemCommonType.kHero  then
                return ItemType.kHeroSpall
            else
                return ItemType.kPropsSpall
            end
        elseif itype == ItemCommonType.kHero then
            return ItemType.kHero
        end
    end
end



--数字格式输出千和万
function This.formatItemCount(count)
    local countStr = count

    if AppText:isChinese() then
        if count > 1000 and count < 10000 then
            countStr = math.floor(count / 1000 * 10) / 10 .. Serialize[60406]
        elseif count >= 10000 then
            countStr = math.floor(count / 10000 * 10) / 10 .. Serialize[60407]
        end
    else
        -- 英文版: K(千) - M(百万) - B(十亿) 
        if count >= 1000 and count < 1000000 then
            countStr = math.floor(count / 1000 * 10) / 10 .. "K"
        elseif count >= 1000000 and count < 1000000000 then
            countStr = math.floor(count / 1000000 * 10) / 10 .. "M"
        elseif count >= 1000000000 then
            countStr = math.floor(count / 1000000 * 10) / 10 .. "B"
        end
    end

    return countStr
end

function This.customFormatCount( count, max )
    local countStr = count

    if AppText:isChinese() then
        if count > max and max >= 10000 then
            countStr = math.floor(count / 10000 * 10) / 10 .. Serialize[60407]
        end
    else
        if count > max then 
            if count >= 1000 and count < 1000000 then
                countStr = math.floor(count / 1000 * 10) / 10 .. "K"
            elseif count >= 1000000 and count < 1000000000 then
                countStr = math.floor(count / 1000000 * 10) / 10 .. "M"
            elseif count >= 1000000000 then
                countStr = math.floor(count / 1000000 * 10) / 10 .. "B"
            end
        end
    end

    return countStr
end

function This.canUsed( itemId )
    local item_cfg = ItemConfig[itemId]
    if item_cfg.itype ~= ItemCommonType.kProps and item_cfg.itype ~= ItemCommonType.kPropPack  then
        return false, ItemUsageType.kUnknown
    end 

    if item_cfg.is_use then
        if item_cfg.is_use[1] and item_cfg.is_use[1] <= ItemUsageType.kNone then
            return false, item_cfg.is_use
        else
            return true, item_cfg.is_use
        end
    end
    return false, ItemUsageType.kUnknown
end

--endregion
