-- @Author: superm
-- @Date:   2018-12-27 11:40:47
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-05 11:27:58
-- @File Name: RecycleBaseNode.lua
-- @Description: 
local This, super = class(..., "NodeView")

local GamePlayer = require("Player.GamePlayer")
local EquipManager = require("Equip.EquipManager")
local DepotManager = require("Depot.DepotManager")
local FabaoManager = require("Fabao.FabaoManager")
local HeroManager = require("Hero.HeroManager")

local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local ViewUtil = require("UI.ViewUtil")

function This:initData(info)
	local target 
	if info then
		target = info.ttype
	end
	self.m_targetType = target or 1
	-- print(self.m_targetType)
    local csNode = cc.Node:createFromCSFile(self.filename)
    self.m_csNode = csNode
    self:addChild(csNode)
    self.m_fileterList = {false, false, false, false}
    self.m_fileterInfoList = {}
	self.m_leftNumber = 0
	self.m_inited = false
end

function This:initNode()
	local csNode = self.m_csNode
	csNode.miaoshu:setVisible(false)
	ViewUtil:blinkView(csNode.miaoshu, true)
	self:initResource(csNode.resource)
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    HeroManager:addObserver(self)
    GamePlayer:addObserver(self)
	self:refresh()
end

function This:onExit()
    DepotManager:removeObserver(self)
    HeroManager:removeObserver(self)
    GamePlayer:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
	if not self.m_inited then
    	local csNode = self.m_csNode

		csNode.wenhao:setTouchEnded(function()
        	g_scene:openDialog("UI.HelpTips",{text = Serialize["recycle_help_tips"], node = csNode.button or csNode.button1})
		end)

		csNode.resource.jiahaodi1:setTouchEnded(function()
            GamePlayer:checkGold({nocheck = true, nowarming = true})
		end)

		csNode.resource.jiahaodi2:setTouchEnded(function()
			g_scene:openDialog("System.RechargeMainUI")
		end)
		self:addTouchEvent()
	    self.m_inited = true
	end
end

function This:initResource(node)
	self:refreshResource(node)
end

function This:refreshResource(node)
    local resNode = node or self.m_csNode.resource
	if resNode.number1 then
		resNode.number1:setString(GamePlayer:getGold())
	end
	resNode.number2:setString(GamePlayer:getDiamond())
	if resNode.number3 then
		resNode.number3:setString(GamePlayer:getSoul())
	end
end

function This:refresh()
	if self.m_inited then
		for index, item in ipairs(self.m_fileterList) do
			if item then
				self:revokeChosen(index)
			end
		end
		self:refreshInner()
	end
end

function This:chooseProp(pos, checkHandler, valueHandler)
	local panel = g_scene:openDialog("Depot.PropChoosePanel",{
		filename = "MenuScene/NodeUI/RebornChooseHero.csb",
		spaceX = 6,
		filter = self.m_fileterList,
		valueHandler = valueHandler,
		checkHandler = checkHandler,
	})
	panel:setSelectHandler(function (...)
		return self:putInChosen(pos, ...)
	end)
	return panel
end

function This:chooseEquip(pos, isFaBao)
	local etype
	if isFaBao then
		etype = ItemCommonType.kFaBao
	end
	local panel = g_scene:openDialog("Equip.EquipChoosePanel", {type= etype })
	panel:setSelectHandler(function(equipUid)
		local item = DepotManager:getItemByUid(equipUid)
		return self:putInChosen(pos, item)
	end)
	return panel
end

function This:chooseHero(pos)
	local haveSelected = {}
	local panel = g_scene:openDialog("Bag.RecycleHeroChoosePanel",{
		spaceX = 8,
		haveSelected = haveSelected,
	})
	panel:setSelectHandler(function(hero)
		return self:putInChosen(pos, hero)
	end)
	return panel
end

function This:putInChosen(pos, item)
	-- print("putin index:" .. pos)
	local itemlistNode = self.m_csNode.itemtipslist
	local node = itemlistNode["item" .. pos]
	if item then
		local itemId, itemNum = item:getId(), item.num
		self.m_fileterList[pos] = item
		itemlistNode["need" .. pos]:setVisible(false)
		if not node.itemNode then
			local itemNode = ItemNode:create(itemId, itemNum, false, true)
			itemNode:setAnchorPoint(0.5, 0.5)
			node:addChild(itemNode)
			node.itemNode = itemNode
			node.close:setLocalZOrder(10)
			node.jianhao:setLocalZOrder(11)
		else
			node.itemNode:initItem(itemId, itemNum, false)
		end 
		node:setVisible(true)
	else
		node:setVisible(false)
	end
	node.close:setVisible(true)
	node.jianhao:setVisible(true)
	return true
end

function This:revokeChosen(pos)
	-- print("revoke index:" .. pos)
	self.m_quicklyPutIn = false
	local itemListNode = self.m_csNode.itemtipslist
    self.m_fileterList[pos] = false
	-- self.m_fileterInfoList[pos] = { quality = 0, uid = 0 }
    itemListNode["item" .. pos]:setVisible(false)
    itemListNode["need" .. pos]:setVisible(true)
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.gold or field == PlayerAttr.diamond or field == PlayerAttr.zhangong or field == PlayerAttr.soul or field ==  PlayerAttr.skill_reel then
		local csNode = self.m_csNode
		self:initResource(csNode.resource)
    end
end

This.onHeroReset = This.refresh
This.onSellItemNew = This.refresh