-- @Author: superm
-- @Date:   2018-12-27 11:41:30
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-12-23 15:26:23
-- @File Name: RecycleMultiNode.lua
-- @Description: 

local This, super = class(..., "Bag.RecycleBaseNode")

local GamePlayer = require("Player.GamePlayer")
local EquipManager = require("Equip.EquipManager")
local DepotManager = require("Depot.DepotManager")
local FabaoManager = require("Fabao.FabaoManager")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local PropChoosePanel = require("Depot.PropChoosePanel")

This.filename = "MenuScene/NodeUI/RebornMultiple.csb"

local tipsText = {
	Serialize[60416],
	Serialize[60417],
	Serialize[60418],
	Serialize[60419],
	Serialize[60420],
	Serialize[60421],
	Serialize[60422],
}

local jumpText = {
	Serialize[60423],
	Serialize[60424],
	Serialize[60425],
	Serialize[60424],
	Serialize[60426],
	Serialize[60427],
	"",
}


local jumpPanel = {
	{"Store.StoreMainPanel", {tabs = StoreType.kJianghun}},
	{"Store.StoreMainPanel", {tabs = StoreType.kZhangong}},
	{"Depot.DepotMainUI", {tabs = 2}},
	{"Store.StoreMainPanel", {tabs = StoreType.kZhangong}},
	{"Store.StoreMainPanel", {tabs = StoreType.kHeroEquipShop}},
	{"Awake.LordEquipPanel", {tabs = 3}},
	false, --战法没有跳转
}

function This:initData(info)
	super.initData(self,info)
	self.m_closeFun = info.closeFun
end

function This:addTouchEvent()
	local csNode = self.m_csNode
	
	local jumpName = jumpText[self.m_targetType]
	csNode.peiyang:setVisible(jumpName ~= "")
	csNode.peiyang.text:setString(jumpName)
	csNode.miaoshu:setString(tipsText[self.m_targetType])
	csNode.link:setTouch({
		onEnded = function()
			local info = jumpPanel[self.m_targetType]
			if not info then
				return
			end
			
			if self.m_targetType == 5 then
				local level = GamePlayer:getLevel()
				if level < 55 then
					g_scene:showTips(Serialize[60428])
					return
				end
			end
			local isClose = false
			if self.m_targetType == 6 then
				isClose = true
				if GamePlayer:checkOpenlvLimit(50,nil,true) then
					return false
				end
			end
			g_scene:openDialog(unpack(info))
			if isClose and self.m_closeFun then
				self.m_closeFun()
			end
		end,
		isAlpha = false
	})

	-- 自动添加
	csNode.button1:setTouchEnded(function()
		local alones = {
			[2] = true, [3] = true, [5] = true, [6] = true, [7] = true
		}
		self:quicklyPutIn(function ( item )
			if self.m_targetType == 1 then
				return item:isSpall() and item:isHeroSpall()
			elseif self.m_targetType == 2 then
				return item:isEquip() and not item:isEquipped()
			elseif self.m_targetType == 3 then
				return item:isFabao() and not item:isEquipped()
			elseif self.m_targetType == 4 then
				return item:isSpall() and item:isEquipSpall()
			elseif self.m_targetType == 5 then
				return item:isLordEquip() and not item:isEquipped()
			elseif self.m_targetType == 6 then
				return item:isInscription() and not item:isEquipToEquip() and not item:isLocked()
			elseif self.m_targetType == 7 then
				return item:checkCommonType(ItemCommonType.kTactics) and (item:getEquipCardId() == 0) and (item:getQuality() < Quality.kOrange)
			end
		end, alones[self.m_targetType] and true or false)
	end)

	-- 分解
	csNode.button2:setTouchEnded(function()
		local itemList, idList = {}, {}
		for _, item in ipairs(self.m_fileterList) do
			if item then
				table.insert(itemList, item)
				if self.m_targetType == 1 or self.m_targetType == 4 then
					table.insert(idList, {item:getUid(), item:getCount()})
				else
					table.insert(idList, item:getUid())
				end
			end
		end

		if #itemList <= 0 then return end
		g_scene:openDialog("Bag.RecyclePreviewPanel", {
			reborn = false,
			target = itemList,
			handler	= function ()
				local operate
				if self.m_targetType == 3 or self.m_targetType == 5 or self.m_targetType == 6 or self.m_targetType == 7 then
					 operate = ItemOperation.kSell
				end
				DepotManager:onSellItemNewRequest(idList, operate)
			end	
		})
	end)


	local itemListNode = csNode.itemtipslist
	for index=1, 4 do
		itemListNode["need" .. index].add:setTouch({
			onEnded = function()
				if self.m_targetType == 7 then
					self:chooseTactics(index)
				else
					self:chooseProp(index, function ( item )
						if self.m_targetType == 1 then
							return item:isSpall() and item:isHeroSpall()
						elseif self.m_targetType == 2 then
							return item:isEquip() and not item:isEquipped()
						elseif self.m_targetType == 3 then
							return item:isFabao() and not item:isEquipped()
						elseif self.m_targetType == 4 then
							return item:isSpall() and item:isEquipSpall()
						elseif self.m_targetType == 5 then
							return item:isLordEquip() and not item:isEquipped()
						elseif self.m_targetType == 6 then
							return item:isInscription() and not item:isEquipToEquip() and not item:isLocked()
						end
					end, function (t)
						local ret = {}
						for _, v in ipairs(t) do
							if v then
								if self.m_targetType == 1 or self.m_targetType == 4 then
									local id = v.id
									ret[id] = (ret[id] or 0) + v.num
								else
									local id = v._id
									ret[id] = (ret[id] or 0) + v.num
								end
							end
						end
						return ret			
					end)
				end
			end,
			isDynamic = true,
			isScale = false,
			isAlpha = false,
		})
		local curItemNode = itemListNode["item" .. index]
		curItemNode:setVisible(false)
		curItemNode.close:setTouch({
			onEnded = function()
				self:revokeChosen(index)
			end,
			isDynamic = true,
			isScale = false,
			isAlpha = false,
		})
		-- curItemNode.jianhao:setTouchEnded({
		-- 	onEnded = function()
		-- 		self:revokeChosen(index)
		-- 	end,
		-- 	isDynamic = true,
		-- 	isScale = false,
		-- 	isAlpha = false,
		-- })
	end
end

function This:refreshInner()
	if self.m_inited then
		local csNode = self.m_csNode
		csNode.peiyang.text:setString(jumpText[self.m_targetType])
		csNode.tips:setVisible(table.contain({2,3}, self.m_targetType))		
	end 
end

function This:quicklyPutIn(checkHandler, aloneMode)
	local slotCount = 4
	local posList = {}
	for index, item in ipairs(self.m_fileterList) do
		if not item then
			table.insert(posList, index)
		end
	end

	local leftNumber = #posList
	if leftNumber > 0 then
		local propList, _, count = PropChoosePanel:getItemList({
			filter = self.m_fileterList,
			valueHandler = function (t)
				local ret = {}
				for _, v in ipairs(t) do
					if v then
						local id
						if aloneMode then
							id = v._id
						else
							id = v.id
						end
						ret[id] = (ret[id] or 0) + v.num
					end
				end
				return ret			
			end,
			checkHandler = checkHandler,
			alone = aloneMode
		})
		if leftNumber > count then
			if count <= 0 then
				g_scene:showTips(Serialize[60429])
			end
			self.m_quicklyPutIn = false
		end

		for i, index in ipairs(posList) do
			local newProp = propList[index]
			if newProp then
				local chosenItem = newProp
            	if aloneMode then
            		chosenItem = newProp.item
            	end
				super.putInChosen(self, index, chosenItem)
			else
				return
			end
		end
	else
		g_scene:showTips(Serialize[60430])
	end
	self.m_quicklyPutIn = true
end

function This:putInChosen(pos, _, selectInfo, itemList)
	local slotCount = 4
	local posList = {}
	for index, item in ipairs(self.m_fileterList) do
		if not item then
			table.insert(posList, index)
		end
	end

	local leftNumber = #posList
	local count = slotCount - leftNumber
	-- print("剩余:" .. leftNumber)
	if selectInfo then
		if not selectInfo.selected and selectInfo.index >= leftNumber then
			return false, Serialize[60431]
		end
	end
	-- print_tmp(selectInfo)
	if not itemList then
		return false
	elseif itemList then
		count = #itemList + count
		if slotCount < count then
			return false, Serialize[60431]
		end
	end
	-- print_tmp(itemList)
	local items = {}
	for _, info in pairs(itemList) do
		items[info.pos] = info.item
	end
	for index, posIndex in ipairs(posList) do
		local item = items[index]
		if item then
			super.putInChosen(self, posIndex, item)
		end
	end
	return true
end

-- 选择战法
function This:chooseTactics(pos)
	local checkHandler = function (item)
		return item:checkCommonType(ItemCommonType.kTactics) and (item:getEquipCardId() == 0)
	end

	local valueHandler = function (t)
		local ret = {}
		for _, v in ipairs(t) do
			if v then
				local id = v._id
				ret[id] = (ret[id] or 0) + v.num
			end
		end
		return ret			
	end

	local compare = function (item1, item2)
		local quality1, quality2 = item1:getQuality(), item2:getQuality()
		if quality1 ~= quality2 then
			return quality1 < quality2
		end

		local star1, star2 = item1:getStar(), item2:getStar()
		if star1 ~= star2 then
			return star1 < star2
		end

		local id1, id2 = item1:getId(), item2:getId()
		return id1 < id2
	end

	local panel = g_scene:openDialog("Depot.PropChoosePanel",{
		filename = "MenuScene/NodeUI/RebornChooseHero.csb",
		spaceX = 6,
		filter = self.m_fileterList,
		valueHandler = valueHandler,
		checkHandler = checkHandler,
		compare = compare
	})

	panel:setSelectHandler(function (prop, param, selectedItems)
		if selectedItems then
			local isTips = false
			for _, info in pairs(selectedItems) do
				if info.item:getQuality() >= Quality.kOrange then
					isTips = true
					break
				end
			end
			if isTips then
				g_scene:openDialog("UI.CostConfirmPanel", {
                    simple = Serialize[60432],
                    notips = "tactics_recycle_tips", 
                    okHandler = function()
                    	self:putInChosen(pos, nil, nil, selectedItems)
                        panel:close()
                    end}
                )
                return false
			else
				return self:putInChosen(pos, prop, param, selectedItems)
			end
		else
			return self:putInChosen(pos, prop, param, selectedItems)
		end
	end)

	return panel	
end