-- @Author: superm
-- @Date:   2018-12-27 11:41:18
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-31 21:53:59
-- @File Name: RecycleOneNode.lua
-- @Description: 


local This, super = class(..., "Bag.RecycleBaseNode")

local GamePlayer = require("Player.GamePlayer")
local EquipManager = require("Equip.EquipManager")
local DepotManager = require("Depot.DepotManager")
local FabaoManager = require("Fabao.FabaoManager")
local HeroManager = require("Hero.HeroManager")

local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

This.filename = "MenuScene/NodeUI/RebornOne.csb"

local tipsText = {
	Serialize[60434],
	Serialize[60435],
	Serialize[60436],
}

local jumpText = {
	Serialize[60437],
	Serialize[60425],
	Serialize[60425],
}

local jumpPanel = {
	{"Hero.HeroListPanel"},
	{"Depot.DepotMainUI", {tabs = 2}},
	{"Depot.DepotMainUI", {tabs = 2}},
}

function This:addTouchEvent()
	local csNode = self.m_csNode
	
	csNode.peiyang.text:setString(jumpText[self.m_targetType])
	csNode.miaoshu:setString(tipsText[self.m_targetType])
	csNode.link:setTouch({
		onEnded = function()
			g_scene:openDialog(unpack(jumpPanel[self.m_targetType]))
		end,
		isAlpha = false
	})

	csNode.button:setTouchEnded(function()
		local target = self.m_fileterList[1]
		if not target then return end
        local cost, errorStr = DepotManager:getRecycleCost(self.m_targetType, RecycleType.kReborn) 
        if not cost and errorStr then
        	return g_scene:showTips(errorStr)
        end

		if not target:canReborn() then
			return g_scene:showTips(Serialize[60438])
		end

		g_scene:openDialog("Bag.RecyclePreviewPanel", {
			reborn = true,
			isHero = self.m_targetType == 1,
			cost = cost,
			target = target,
			handler	= function (costNum)
				if costNum > 0 then
					GamePlayer:costDiamond({
						cost = cost, 
						title = Serialize[60439],
						desc = Serialize[60439],
						okHandler = function()
							if self.m_targetType == 1 then
								HeroManager:onResetHeroRequest(target:getId())
							else
								DepotManager:onSellItemNewRequest({target:getUid()}, ItemOperation.kReborn)
							end
						end
					})
				else
					if self.m_targetType == 1 then
						HeroManager:onResetHeroRequest(target:getId())
					else
						DepotManager:onSellItemNewRequest({target:getUid()}, ItemOperation.kReborn)
					end
				end
			end	
		})
	end)

	local itemListNode = csNode.itemtipslist
	for index=1, 1 do
		itemListNode["need" .. index].add:setTouch({
			onEnded = function()
				if self.m_targetType == 1 then
					self:chooseHero(index)
				else
					self:chooseEquip(index, self.m_targetType == 3)
				end
			end,
			isDynamic = true,
			isScale = false,
			isAlpha = false,
		})
		local curItemNode = itemListNode["item" .. index]
		curItemNode:setVisible(false)
		curItemNode.close:setTouch({
			onEnded = function()
				self:revokeChosen(index)
			end,
			isDynamic = true,
			isScale = false,
			isAlpha = false,
		})
		-- curItemNode.jianhao:setTouchEnded({
		-- 	onEnded = function()
		-- 		self:revokeChosen(index)
		-- 	end,
		-- 	isDynamic = true,
		-- 	isScale = false,
		-- 	isAlpha = false,
		-- })
	end
end

function This:refreshInner()
	if self.m_inited then
		local csNode = self.m_csNode
        local cost, errorStr = DepotManager:getRecycleCost(self.m_targetType, RecycleType.kReborn) 
        if not cost and errorStr then
        	csNode.jiagebg:setVisible(false)
        	csNode.button:setVisible(false)
        elseif cost then
        	local visible = cost > 0
			csNode.jiagebg:setVisible(true)
			csNode.button:setVisible(true)
			csNode.jiagebg.chongzhika:setVisible(false)
			csNode.jiagebg.yuanbao:setVisible(false)
			csNode.jiagebg.mianfei:setVisible(not visible)
			csNode.jiagebg.number:setVisible(visible)
			if visible then
				local resetCardNum = 0
				if self.m_targetType == 1 then
					resetCardNum = DepotManager:getItemCount(123)
				end
				if resetCardNum > 0 then
					csNode.jiagebg.chongzhika:setVisible(true)
					csNode.jiagebg.yuanbao:setVisible(false)
					csNode.jiagebg.number:setString(resetCardNum)
				else
					csNode.jiagebg.chongzhika:setVisible(false)
					csNode.jiagebg.yuanbao:setVisible(true)
					csNode.jiagebg.number:setString(cost)
				end
			end

        end
	end
end