-- @Author: superm
-- @Date:   2018-12-27 11:41:56
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-16 15:07:42
-- @File Name: RecyclePreviewPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local ItemNode = require("UI.ItemNode")
local HeroManager = require("Hero.HeroManager")
local DepotManager = require("Depot.DepotManager")
local EquipManager = require("Equip.EquipManager")
local ItemUtil = require("Bag.ItemUtil")
local GamePlayer = require("Player.GamePlayer")
local ItemReturnConfig = require("Metadatas.ItemReturnConfig")
local AffinationMasterConfig = require("Metadatas.AffinationMasterConfig")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")

function This:initNode(info)
	self.m_reborn = info.reborn
	self.m_target = info.target
	self.m_handler = info.handler
	self.m_isHero = info.isHero
	self.m_costNum = info.cost

	local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RebornGet.csb")
	self.m_csNode = csNode
	self:addChild(csNode)
	csNode:setToCenter()


	local rebackList, hasHighItem, isEquip
	if self.m_reborn then
		csNode.title:setString(Serialize[60442])
		csNode.button.text:setString(Serialize[60443])
		hasHighItem = self.m_target:getQuality() >= Quality.kOrange
		local cost = info.cost
		if cost and cost > 0 then
        	local visible = cost > 0
			csNode.jiagebg:setVisible(true)
			csNode.button:setVisible(true)
			csNode.jiagebg.chongzhika:setVisible(false)
			csNode.jiagebg.yuanbao:setVisible(false)
			csNode.jiagebg.mianfei:setVisible(not visible)
			csNode.jiagebg.number:setVisible(visible)
			if visible then
				local resetCardNum = 0
				if self.m_isHero then
					resetCardNum = DepotManager:getItemCount(123)
				end
				if resetCardNum > 0 then
					self.m_costNum = 0
					csNode.jiagebg.chongzhika:setVisible(true)
					csNode.jiagebg.yuanbao:setVisible(false)
					csNode.jiagebg.number:setString(string.format("%d / 1", resetCardNum))
				else
					csNode.jiagebg.chongzhika:setVisible(false)
					csNode.jiagebg.yuanbao:setVisible(true)
					csNode.jiagebg.number:setString(cost)
				end
			end
		end
		csNode.tips:setVisible(not self.m_reborn and hasHighItem) 
		if not self.m_isHero and self.m_target:isEquip() then
			isEquip = true
			EquipManager:onReqSeeEquipClearData(self.m_target:getUid(),self.m_target:getId())
			return
		else
			rebackList = self.m_target:getRebornReturn()
		end
	else
		csNode.title:setString(Serialize[60444])
		csNode.button.text:setString(Serialize[60445])
		rebackList = {}
		for _, item in ipairs(self.m_target) do
			local itemId = item:getId()
			local num = item:getCount()
			local id, count
			if not item:isFaBaoItemReturnConfig() and not item:getType() == 12  then
				id, count = ItemUtil.getPriceMoneyType(itemId), ItemUtil.getPrice(itemId) * num
				rebackList[id] = (rebackList[id] or 0) + count
			else
				local config = ItemReturnConfig[itemId]
				if config and config.return_id then
					for _, returnInfo in ipairs(config.return_id) do
						id, count = unpack(returnInfo)
						rebackList[id] = (rebackList[id] or 0) + count
					end
				end
			end
		    if not hasHighItem then
		    	hasHighItem = item:getQuality() >= Quality.kOrange
		    end
		end
		csNode.tips:setVisible(not self.m_reborn and hasHighItem) 
	end

	if rebackList then
		self.m_costList = {}
	    for id, number in pairs(rebackList) do
	    	if number > 0 then
		        local view = ItemNode:create(id, number)
				view:setAnchorPoint(0.5,0.5)
		        csNode.scrollView:pushNode(view, 95, 85, 6)
		        table.insert(self.m_costList, {id = id, count = number})
		    end
	    end
	end
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.closebutton:setTouchEnded(function()
		self:close()
	end)

	csNode.button:setTouchEnded(function()
		if self.m_isHero  then

			local rank = MatchManager:getRankLv()
			if not (MatchingConfig[rank] and MatchingConfig[rank].reset > 0) then
				local openRank = MatchManager:getHeroResetOpenRankLv()
				g_scene:showTips(string.format(Serialize[60446],MatchingConfig[openRank].rank_level))
				return
			end
			if self.m_target:getState() ~= HeroState.kInFree then
				g_scene:showTips(Serialize[60447])
				return
			end
			if self.m_target:getDeputyCount() > 0 then
				g_scene:showTips(Serialize[60448])
				return
			end
			for i = EquipPos.kClothes , EquipPos.kWeapons do
				if self.m_target:isWearEquip(i) then
					g_scene:showTips(Serialize[60449])
					return
				end
			end
			if #self.m_costList <= 0 then
				g_scene:showTips(Serialize[60450])
				return
			end
		end

		if self.m_handler then
			self.m_handler(self.m_costNum)
		end
	end)
end

function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
	DepotManager:addObserver(self)
	EquipManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	DepotManager:removeObserver(self)
	HeroManager:removeObserver(self)
	EquipManager:removeObserver(self)
end

function This:updateEquipClearMaster( masterData )
	local csNode = self.m_csNode
	local curExp = masterData.exp or 0
    for i , v in ipairs(AffinationMasterConfig) do 
        if v.level < masterData.level then
            curExp = curExp + v.exp
        else
        	break
        end
    end
    print(curExp)
	local rebackList = self.m_target:getRebornReturn(curExp)
	self.m_costList = {}
    for id, number in pairs(rebackList) do
    	if number > 0 then
	        local view = ItemNode:create(id, number)
			view:setAnchorPoint(0.5,0.5)
	        csNode.scrollView:pushNode(view, 95, 85, 4)
	        table.insert(self.m_costList, {id = id, count = number})
	    end
    end
end

function This:onHeroReset()
	if self.m_costList and #self.m_costList > 0 then
		g_scene:showReward(self.m_costList)
	end
	self:close()
end

This.onSellItemNew = This.onHeroReset
