
-- 战法

local This, super = BaseClass(..., "Bag.BaseItemModel")
local TacticsConfig = require("Metadatas.TacticsConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local DepotManager = require("Depot.DepotManager")


function This:getTacticsConfigVal(key, default)
	if self.id > 0 then
		local config = TacticsConfig[self.id]
		if config and config[key] ~= nil then
			return config[key]
		end
	end
	return default
end

-- 战法类型
function This:getTacticsType()
	return self:getTacticsConfigVal("num", 0)
end

-- 星级
function This:getStar()
	return self:getTacticsConfigVal("star", 0)
end

-- 最大星级
function This:getMaxStar()
	return ArgConfig.tactics_starlevel.arg[1]
end

-- 是否满星
function This:isMaxStar()
	return self:getStar() == self:getMaxStar()
end

-- 升星消耗
function This:getUpgradeStarCost()
	local cost = {}
	local config = TacticsConfig[self:getId()]
	if config then
		cost[1] = config.star_up_tactics
		cost[2] = config.star_up_item
	end
	return cost
end

-- 下一星目标
function This:getNextStarTargetId()
	return self.id + 1
end

-- 属性
function This:getAttrs()
	local list = self:getTacticsConfigVal("arg", {})
	local ret = {}

	for _, v in ipairs(list) do
		local id = v[1]
		ret[id] = (ret[id] or 0) + v[2]
	end

	return ret 
end

function This:isWear()
	return self.equip_card_id ~= 0
end

-- 返回更好的那件
function This:compareTactics(other)
	if not other then
		return self
	end

	local a, b = self, other

	local aQuality, bQuality = a:getQuality(), b:getQuality()
	if aQuality ~= bQuality then
	    return (aQuality > bQuality) and a or b
	end

	local aStar, bStar = a:getStar(), b:getStar()
	if aStar ~= bStar then
	    return (aStar > bStar) and a or b
	end

	local aId, bId = a:getId(), b:getId()
	if aId ~= bId then
	    return (aId > bId) and a or b
	end

	return a
end

function This:canUsed( showWarming )
	return false
end