--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...)
local BattleData = require("Battle.BattleData")
local TroopsActor = require("Battle.TroopsActor")
local Timer = require("Timer")

kCountPerTroops = 6

function This:create()
	local creator = {}
	setmetatable(creator,self)
	creator:init()
	return creator
end

function This:init()
	self.m_friendTroops={}
	self.m_enemyTroops={}

	self.m_lastFriendAttackIndex = 0
	self.m_lastEnemyAttackIndex = 0
	
	self.m_enemyWallTroopsLeftHp = 0
	self.m_enemyTroopsTotalHp = self.m_enemyWallTroopsLeftHp
	self.m_friendTroopsTotalHp = 0

	self.m_isAttackWall = false
	self.m_isWarEnd = false
	self.m_battleData = BattleData:create()
	self.m_battleIndex = 0
	self.m_roundCount = 1
	self.m_speedFlag = 0
	self.m_lastAttackTroops = nil
	self.m_isPause = false
	self.m_observers = {}
end

function This:addObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			return
		end
	end
	table.insert(self.m_observers,observer)
end

function This:removeObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			table.remove(self.m_observers,i)
			break
		end
	end
end

function This:notifyObserver(funcName,...)
	local observers = {}
	for k , v in pairs(self.m_observers) do
		observers[k] = v
	end
	for k , o in pairs(observers) do
		if o[funcName] then
			o[funcName](o,...)
		end
	end
end

function This:getRoundIndex()
	return self.m_battleIndex
end

function This:getRoundCount()
	return self.m_roundCount
end

function This:getBattleData()
	return self.m_battleData
end

function This:setBattleData(data)
	self.m_battleData = data
end

function This:isWarEnd()
	return self.m_isWarEnd
end

function This:isAttackWall()
	return self.m_isAttackWall
end

function This:getTroops(index,isEnemy)
	if isEnemy then
		return self.m_enemyTroops[index]
	else
		return self.m_friendTroops[index]
	end	
end

function This:getTroopsById(id)
	local isEnemy = id > 100
	local index = isEnemy and id - 100 or id
	return self:getTroops(index,isEnemy)
end

function This:getAllTroops(isEnemy)
	return (isEnemy and self.m_enemyTroops) or self.m_friendTroops
end

function This:getEnemyTroopsLeftHp()
	local totalHp = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() then
			totalHp = totalHp + troops:getAttr(AttrType.kHp)
		end
	end
	return totalHp + self.m_enemyWallTroopsLeftHp
end

function This:getFriendTroopsLeftHp()
	local totalHp = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() then
			totalHp = totalHp + troops:getAttr(AttrType.kHp)
		end
	end
	return totalHp
end

function This:getEnemyTroopsTotalHp()
	return self.m_enemyTroopsTotalHp
end

function This:getFriendTroopsTotalHp()
	return self.m_friendTroopsTotalHp
end

function This:getEnemyTroopsCount()
	local count = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() then
			count = count  + 1
		end
	end
	return count
end

function This:getFriendTroopsCount()
	local count = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() then
			count = count  + 1
		end
	end
	return count
end

function This:addTroops(troops)
	local isEnemy = troops:isEnemy()
	local index = troops:getIndex()
	if isEnemy then
		self.m_enemyTroops[index] = troops
		self.m_enemyTroopsTotalHp = self.m_enemyTroopsTotalHp + troops:getAttr(AttrType.kHp)
	else
		self.m_friendTroops[index] = troops
		self.m_friendTroopsTotalHp = self.m_friendTroopsTotalHp + troops:getAttr(AttrType.kHp)
	end
	self.m_battleData:readTroops(troops)
end

function This:removeTroops(troops)
	local isEnemy = troops:isEnemy()
	local index = troops:getIndex()
	if isEnemy then
		if self.m_enemyTroops[index] then
			self.m_enemyTroops[index] = nil
		end
	else
		if self.m_friendTroops[index] then
			self.m_friendTroops[index] = nil
		end
	end
end

-------------------------------------------------------------

function This:exitBattle()
	for id , troops in pairs(self.m_enemyTroops) do
		self:removeTroops(troops)
	end
	for id , troops in pairs(self.m_friendTroops) do
		self:removeTroops(troops)
	end

	self.m_friendTroops = {}
	self.m_enemyTroops = {}

	self.m_enemyTroopsTotalHp = 0
	self.m_enemyWallTroopsLeftHp = 0
	self.m_friendTroopsTotalHp = 0

	self.m_lastFriendAttackIndex = 0
	self.m_lastEnemyAttackIndex = 0

	self.m_isAttackWall = false
	self.m_isWarEnd = false
	self.m_speedFlag = 0
	self.m_lastAttackTroops = nil
	self.m_isPause = false
	self.m_observers = {}

	print("*********************战斗生成器退出战斗********************")
end

function This:pauseBattle()
	self.m_isPause = true
end

function This:resumeBattle()
	self.m_isPause = false
	self:nextRound()
end

function This:getFriendTroopsOnNextAttack()
	self.m_lastFriendAttackIndex = self.m_lastFriendAttackIndex  + 1
	local troops = self.m_friendTroops[self.m_lastFriendAttackIndex]
	while (not troops or not troops:isAlive()) and self.m_lastFriendAttackIndex <= kCountPerTroops do
		self.m_lastFriendAttackIndex = self.m_lastFriendAttackIndex  + 1
		troops = self.m_friendTroops[self.m_lastFriendAttackIndex]
	end
	return troops
end

function This:getEnemyTroopsOnNextAttack()
	self.m_lastEnemyAttackIndex = self.m_lastEnemyAttackIndex + 1
	local troops = self.m_enemyTroops[self.m_lastEnemyAttackIndex]
	while (not troops or not troops:isAlive()) and self.m_lastEnemyAttackIndex <= kCountPerTroops do
		self.m_lastEnemyAttackIndex = self.m_lastEnemyAttackIndex + 1
		troops = self.m_enemyTroops[self.m_lastEnemyAttackIndex]
	end
	return troops
end

function This:getTroopsOnNextAttack()
	local findNextTroops = function(isFriendFrist)
		local troops = nil
		if isFriendFrist then
			troops = self:getFriendTroopsOnNextAttack()
			if troops then return troops end
			troops = self:getEnemyTroopsOnNextAttack()
		else
			troops = self:getEnemyTroopsOnNextAttack()
			if troops then return troops end
			troops = self:getFriendTroopsOnNextAttack()
		end
		return troops
	end

	local isFriendFirst = true
	if self.m_lastAttackTroops == nil then
		isFriendFirst = self.m_speedFlag == 0
	else
		isFriendFirst = self.m_lastAttackTroops:isEnemy()
	end

	local troops = findNextTroops(isFriendFirst)
	if troops == nil then
		self.m_lastEnemyAttackIndex = 0
		self.m_lastFriendAttackIndex = 0
		self.m_roundCount = self.m_roundCount + 1
		troops = findNextTroops(isFriendFirst)
	end
	return troops
end

function This:getSpeedValue(friendSpeed,enemySpeed)
	if friendSpeed > enemySpeed then
		return 0
	elseif friendSpeed < enemySpeed then
		return 1
	else
		local friendTotalHp = self:getFriendTroopsTotalHp()
		local enemyTotalHp = self:getEnemyTroopsTotalHp()
		if friendTotalHp > enemyTotalHp then
			return 0
		else
			return 1
		end
	end
end

function This:enterBattle(battleInitData)
	print("*********************战斗生成器进入战斗********************")
	self.m_battleData:clear()
	for i , info in pairs(battleInitData.troopsInfo) do
		local troops = TroopsActor:create({
			id = info.id,
			heroId = info.heroId,
			hero = info.hero
		})
		troops:setGameScene(self)
		self:addTroops(troops)
	end

	local attackerInfo = battleInitData.attackerBaseInfo
	self.m_battleData:setAttackerBaseInfo(attackerInfo)
	local defenderInfo = battleInitData.defenderBaseInfo
	self.m_battleData:setDefenderBaseInfo(defenderInfo)

	self.m_speedFlag = self:getSpeedValue(attackerInfo.speed,defenderInfo.speed)
	self.m_battleIndex = 0
	self.m_roundCount = 1
	self.m_leftEnemyTroopsHp = self.m_enemyTroopsTotalHp
end

function This:nextRound()
	if self.m_isPause then
		return
	end
	
	local friendCount = self:getFriendTroopsCount()
	if friendCount <= 0 then
		self:battleFailed()
		return
	end 

	local enemyCount = self:getEnemyTroopsCount()
	if enemyCount <= 0 then
		self:battleVictory()
		return
	end

	local troops = self:getTroopsOnNextAttack()
	assert(troops)

	self.m_battleIndex = self.m_battleIndex + 1
	self.m_lastAttackTroops = troops
	troops:enterAttack()
end

function This:battleVictory()
	self.m_isWarEnd = true 
	self.m_battleData:battleEnd(true)
	self.m_battleData:print()
	print("战斗胜利")
end

function This:battleFailed()
	self.m_isWarEnd = true 
	self.m_battleData:battleEnd(false)
	self.m_battleData:print()
	print("战斗失败")
end

-----------------------------------------------------------

function This:onTroopsEnterAttack(troops)
	self.m_battleData:generateOneRoundData()
	self.m_battleData:setAttacker(troops)
end

function This:onTroopsActiveSkill(troops,skill)
	self.m_battleData:setAttackerSkill(skill)
end

function This:onTroopsActiveSkillEnd(troops) end
function This:onTroopsForwardEnd(troops) end
function This:onTroopsAttack(troops,target) end
function This:onTroopsAttackEnd(troops) end

function This:onTroopsHitted(troops,damage)
	if troops:getId() == 0 then
		assert(false)
	end
	self.m_battleData:addHitter(troops,damage)
end

function This:onTroopsHittedEnd(troops)end

function This:onTroopsBackEnd(troops)
	self.m_battleData:pushBattleData()
end

function This:onWallHitted(damage)
	self.m_enemyWallTroopsLeftHp = self.m_enemyWallTroopsLeftHp - damage
	print("城墙受击",self.m_enemyWallTroopsLeftHp+damage,"->",self.m_enemyWallTroopsLeftHp)
	self.m_battleData:addWallHitter()
end

function This:onTroopsDead(troops,damage)
	if troops:isEnemy() then
		--[[local enemyCount = self:getEnemyTroopsCount()
		if enemyCount <= 0 then
			self.m_isAttackWall = true
		end]]
	end
	self:removeTroops(troops)
end

function This:onTroopsCrit(troops)
	self.m_battleData:setAttackerBehavior(troops,BehaviorType.kCrit)
end

function This:onTroopsDodge(troops)
	self.m_battleData:setHitterBehavior(troops,BehaviorType.kDodge)
end

function This:onTroopsPenetrate(troops)
	self.m_battleData:setHitterBehavior(troops,BehaviorType.kPenetrate)
end

function This:onTroopsImmuneDamage(troops)
	self.m_battleData:setHitterBehavior(troops,BehaviorType.kImmuneDamage)
end

function This:onTroopsImmuneDebuff(troops)
end

function This:onCreateTroopsBuff(buff)
	local receiver = buff:getReceiver()
	self.m_battleData:addBuffWithHitter(receiver,buff)
end



--endregion
