--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local AttributeHelper = require("Battle.AttributeHelper")
local BattleManager = require("Battle.BattleManager")

function This:create()
	local data = {}
	setmetatable(data,This)
	data:init()
	return data
end

function This:init()
	self.m_troopsInitData = {}  --部队初始数据
	self.m_battleRoundData = {} --战斗回合数据
	self.m_battleResultData = {} --战斗结果数据
	self.m_battleResultData.isWin = false
	self.m_battleResultData.enemyDeadCount = 0
	self.m_battleResultData.friendDeadCount = 0
	self.m_attackerBaseInfo = {}  --进攻方信息
	self.m_defenderBaseInfo = {}  --防守方信息
	self.m_curRoundData = {} --当前回合数据
	self.m_curRoundIndex = 0
end

function This:clear()
	self.m_troopsInitData = {}  --部队初始数据
	self.m_battleRoundData = {} --战斗回合数据
	self.m_battleResultData.isWin = false
	self.m_battleResultData.enemyDeadCount = 0
	self.m_battleResultData.friendDeadCount = 0
	self.m_attackerBaseInfo = {}  --进攻方信息
	self.m_defenderBaseInfo = {}  --防守方信息
	self.m_curRoundData = {} --当前回合数据
	self.m_curRoundIndex = 0
end

function This:readTroops(troops)
	local t = {}
	local attribute = troops:getAttribute()
	local attr = attribute:getAllAttr()
	t.id = troops:getId()
	t.attr = table.clone(attr)
	t.heroId = troops:getHeroId()
	t.buffs = {}
	local buffs = troops:getBuff()
	for id , buff in pairs(buffs) do
		local leftRoundCount = buff:getRoundCount()
		local attackInfo = buff:getAttackInfo()
		local attackerId = attackInfo.applyer:getId()
		local receiverId = attackInfo.receiver:getId()
		local hitterIds = {}
		for i , hitter in pairs(attackInfo.hitters) do
			table.insert(hitterIds,hitter:getId())
		end
		local buff = {id = id ,roundCount = leftRoundCount,attackerId = attackerId,receiverId = receiverId,hitterIds = hitterIds}
		table.insert(t.buffs,buff)
	end
	self.m_troopsInitData[troops:getId()] = t
end

function This:writeTroops(troops)
	local t = self.m_troopsInitData[troops:getId()]
	assert(t,Serialize[60462])
	local player = BattleManager:getBattlePlayer()
	local attribute = troops:getAttribute()
	attribute:setAllAttr(t.attr)
	local buffs = t.buffs
	for i , info in pairs(buffs) do
		local applyer = player:getTroopsById(info.attackerId)
		local receiver = player:getTroopsById(info.receiverId)
		local hitters = {}
		for i , id in pairs(info.hitterIds) do
			local hitter = player:getTroopsById(id)
			table.insert(hitters,hitter)
		end

		local buffCfg = BuffConfig[info.id]
		local buffInfo = BuffClass[buffCfg.type]
		local class = require(buffInfo.class)
		local buff = class:create({
			id = info.id,
			applyer = applyer,
			receiver = receiver,
			hitters = hitters
		})
		buff:work()
	end
end

function This:getInitTroopsData()
	return self.m_troopsInitData
end

function This:getBattleRoundData()
	return self.m_battleRoundData
end

function This:getAttackerBaseInfo()
	return self.m_attackerBaseInfo
end

function This:setAttackerBaseInfo(info)
	self.m_attackerBaseInfo = info
end

function This:getDefenderBaseInfo()
	return self.m_defenderBaseInfo
end

function This:setDefenderBaseInfo(info)
	self.m_defenderBaseInfo = info
end

function This:generateOneRoundData()  --生成一场空的回合数据
	self.m_curRoundData = {}
	self.m_curRoundData.hitters = {}
end

function This:setAttacker(troops)
	self.m_curRoundData.attackerId = troops:getId()
	self.m_curRoundData.behavior = BehaviorType.kNone
	self.m_curRoundData.heroId = troops:getHeroId()
	self.m_curRoundData.skillId = -1
	self.m_curRoundData.buffs = {}
	self.m_curRoundData.hitters = {}
end

function This:setAttackerSkill(skill)
	if skill then
		self.m_curRoundData.skillId = skill:getId()
	else
		self.m_curRoundData.skillId = -1
	end
end

function This:setAttackerBehavior(troops , behavior)
	assert(troops:getId() == self.m_curRoundData.attackerId)
	self.m_curRoundData.behavior = behavior
end

function This:addHitter(troops,damage)
	for i , hitter in pairs(self.m_curRoundData.hitters) do
		if hitter.id == troops:getId() then
			return hitter
		end
	end
	local hitter = {id = troops:getId(), behavior = BehaviorType.kNone, damage = damage or 0}
	table.insert(self.m_curRoundData.hitters,hitter)
	return hitter
end

function This:addWallHitter(damage)
	local hitter = {id = 0, behavior = BehaviorType.kNone, damage = damage}
	table.insert(self.m_curRoundData.hitters,hitter)
	return hitter
end

function This:setHitterBehavior(troops,behavior)
	for i , hitter in pairs(self.m_curRoundData.hitters) do
		if hitter.id == troops:getId() then
			hitter.behavior = behavior
			return
		end
	end
	local hitter = self:addHitter(troops)
	hitter.behavior = behavior
end

function This:addBuffWithHitter(troops,buff)
	for i , info in pairs(self.m_curRoundData.buffs) do
		if info.id == buff:getId() then
			table.insert(info.targets,troops:getId())
			return
		end
	end
	local t = {id = buff:getId(),targets = {troops:getId()}}
	table.insert(self.m_curRoundData.buffs,t)
end

function This:pushBattleData()
	table.insert(self.m_battleRoundData,self.m_curRoundData)
	self.m_curRoundData = nil
end

---------------------------------------------------------

function This:nextRoundData()
	self.m_curRoundIndex = self.m_curRoundIndex + 1
	if self.m_curRoundIndex <= #self.m_battleRoundData then
		return self.m_battleRoundData[self.m_curRoundIndex]
	end
end

function This:isExistHitter(troops)
	if self.m_curRoundIndex > 0 and self.m_curRoundIndex <= #self.m_battleRoundData then
		local roundData = self.m_battleRoundData[self.m_curRoundIndex]
		for i , hitter in pairs(roundData.hitters) do
			if hitter.id == troops:getId() then
				return true
			end
		end
	end
	return false
end

function This:isExistWallHitter()
	if self.m_curRoundIndex > 0 and self.m_curRoundIndex <= #self.m_battleRoundData then
		local roundData = self.m_battleRoundData[self.m_curRoundIndex]
		for i , hitter in pairs(roundData.hitters) do
			if hitter.id == 0 then
				return true
			end
		end
	end
	return false
end

function This:getAttackerBehavior()
	if self.m_curRoundIndex > 0 and self.m_curRoundIndex <= #self.m_battleRoundData then
		local roundData = self.m_battleRoundData[self.m_curRoundIndex]
		return roundData.behavior
	end
end

function This:getAttackerSkillId()
	if self.m_curRoundIndex > 0 and self.m_curRoundIndex <= #self.m_battleRoundData then
		local roundData = self.m_battleRoundData[self.m_curRoundIndex]
		return roundData.skillId
	end
end

function This:getHitterBehavior(troops)
	if self.m_curRoundIndex > 0 and self.m_curRoundIndex <= #self.m_battleRoundData then
		local roundData = self.m_battleRoundData[self.m_curRoundIndex]
		for i , hitter in pairs(roundData.hitters) do
			if hitter.id == troops:getId() then
				return hitter.behavior
			end
		end
	end
	assert(false,Serialize[60463])
end

function This:getHitterDamage(troops)
	if self.m_curRoundIndex > 0 and self.m_curRoundIndex <= #self.m_battleRoundData then
		local roundData = self.m_battleRoundData[self.m_curRoundIndex]
		for i , hitter in pairs(roundData.hitters) do
			if hitter.id == troops:getId() then
				return hitter.damage
			end
		end
	end
	assert(false,Serialize[60463])
end

function This:getBuffTargets(buffId)
	if self.m_curRoundIndex > 0 and self.m_curRoundIndex <= #self.m_battleRoundData then
		local roundData = self.m_battleRoundData[self.m_curRoundIndex]
		for i , info in pairs(roundData.buffs) do
			if info.id == buffId then
				return info.targets
			end
		end
	end
	return {}
end

-------------------------------------------------------------

function This:troopsDeadth(troops)
	if troops:isEnemy() then
		self.m_battleResultData.enemyDeadCount = self.m_battleResultData.enemyDeadCount + 1
	else
		self.m_battleResultData.friendDeadCount = self.m_battleResultData.friendDeadCount + 1
	end
end

function This:battleEnd(isWin)
	self.m_battleResultData.isWin = isWin
end

function This:getTroopsDeadth(isEnemy)
	if isEnemy then
		return self.m_battleResultData.enemyDeadCount
	else
		return self.m_battleResultData.friendDeadCount
	end
end

function This:isWin()
	return self.m_battleResultData.isWin
end

function This:print()
	do return end
	for id , data in pairs(self.m_troopsInitData) do
		print(string.format("\n部队%d的信息如下：",data.id))
		print(string.format("部队武将ID：%d",data.heroId))
		print(string.format("部队属性信息如下："))
		local attr = AttributeHelper:create()
		attr:setAllAttr(data.attr)
		attr:print()

		print("部队BUFF信息如下：")
		local str = ""
		for i , info in pairs(data.buffs) do
			local str = string.format(Serialize[60464])
			for k , id in pairs(info.hitterIds) do
				str = str .. id .. ","
			end
			print(str)
		end
	end

	for i , info in ipairs(self.m_battleRoundData) do
		print(string.format("\n第%d回合：",i))
		print(string.format("攻击者ID：%d",info.attackerId))
		print(string.format("攻击者的行为(没有行为1、暴击2、闪避3、识破4)为:%d",info.behavior))
		print(string.format("攻击者的武将ID:%d",info.heroId))
		print(string.format("攻击者的技能ID(0表示普攻技能):%d",info.skillId))
		print("受击者信息如下:")
		for j , hitterInfo in pairs(info.hitters) do
			print(string.format("受击者ID:%d(0表示城墙)",hitterInfo.id))
			print(string.format("受击者行为:%d",hitterInfo.behavior))
			print(string.format("受击者所受到的伤害:%f",hitterInfo.damage))
		end
		local str = Serialize[60465]
		for j , buffInfo in pairs(info.buffs) do
			str = "BUFF ID:".. buffInfo.id .. Serialize[60466]
			for k , targetId in pairs(buffInfo.targets) do
				str = str .. targetId .. ","
			end
		end
		print(str)
	end

	print(string.format("战斗结果：isWin:%s,敌军死亡数量:%d,我方死亡数量:%d",self.m_battleResultData.isWin,
		self.m_battleResultData.enemyDeadCount,self.m_battleResultData.friendDeadCount))
end

--endregion
