--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local BattleDataManager = require("Battle.BattleDataManager")
local GameEvent = require("Event.GameEvent")
local AsynchEvent = require("Event.AsynchEvent")
local Socket = require("Net.Socket")
local EventDispatcher = require("EventDispatcher")

function This:init(info)
	--self.noMask = true
	super.init(self,info)
end

function This:initNode(info)
	self.m_reportId = info.reportId
	self.m_offline = info.offline
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/BattleLoad.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self.m_csNode.text:setString("")
	self.m_curIndex = 1
	self.m_escape = 0

	self.m_tasks = {}
	if self.m_reportId and self.m_reportId > 0 then
		self:initTask()
		self.m_waiting = false
	else
		self.m_waiting = true
	end
end

function This:onEnter()
	super.onEnter(self)
	EventDispatcher:addListener(self,10302,self.onGetReportDetailReply)
end

function This:onExit()
	super.onExit(self)
	EventDispatcher:removeListener(self,10302)
end

function This:setReportId(reportId)
	assert(reportId > 0)
	self.m_reportId = reportId
	self:initTask()
	self.m_waiting = false
end

function This:addTask(task)
	table.insert(self.m_tasks,task)
end

function This:initTask()
	if not self.m_offline then
		local task1 = AsynchEvent:create(function ()
			self:onGetReportDetailRequest()
		end,10302)
		self:addTask(task1)
	else
		local task1 = GameEvent:create(function()
			self.m_battleReport = self.m_offline
			self.m_attackerHeroAttrList = {}
			self.m_defenderHeroAttrList = {}
		end)
		self:addTask(task1)
	end

	local task2 = GameEvent:create(function()
		BattleDataManager:setAttackerInfo(self.m_battleReport.attack)
	end)
	self:addTask(task2)

	local task3 = GameEvent:create(function()
		BattleDataManager:setDefenderInfo(self.m_battleReport.defence)
	end)
	self:addTask(task3)

	local task4 = GameEvent:create(function()
		for i, info in ipairs(self.m_battleReport.attack.card_list) do
			BattleDataManager:addAttackerHero(info)
		end
	end)
	self:addTask(task4)

	local task5 = GameEvent:create(function()
		for i, info in ipairs(self.m_battleReport.defence.card_list) do
			BattleDataManager:addDefenderHero(info)
		end
	end)
	self:addTask(task5)

	local task9 = GameEvent:create(function()
		if #self.m_battleReport.prepare > 0 then
			for i , data in ipairs(self.m_battleReport.prepare) do
				BattleDataManager:addRoundData(data.round,data.action_list)
			end
		end
	end)
	self:addTask(task9)

	local task6 = GameEvent:create(function()
		for i , data in ipairs(self.m_battleReport.fight) do
			BattleDataManager:addRoundData(data.round,data.action_list)
		end
	end)
	self:addTask(task6)

	local task7 = GameEvent:create(function()
		for i , info in ipairs(self.m_attackerHeroAttrList) do
			BattleDataManager:setAttackerHeroAttrList(info.pos,info.index_list,info.value_list)
		end
	end)
	self:addTask(task7)

	local task8 = GameEvent:create(function()
		for i , info in ipairs(self.m_defenderHeroAttrList) do
			BattleDataManager:setDefenderHeroAttrList(info.pos,info.index_list,info.value_list)
		end
	end)
	self:addTask(task8)

	local task9 = GameEvent:create(function()
		BattleDataManager:setBattleResult(self.m_battleReport.state)
		BattleDataManager:setStartTime(self.m_battleReport.start_time)
		BattleDataManager:setEndTime(self.m_battleReport.end_time)
		BattleDataManager:setCityId(self.m_battleReport.city_id)
	end)
	self:addTask(task9)
end

function This:setFinishHandler(handler)
	self.m_finishHandler = handler
end

function This:onGetReportDetailRequest()
	Socket:sendAndLock(10302,{report_id = self.m_reportId})
end

function This:onGetReportDetailReply(msg)
	self.m_battleReport = msg.report
	self.m_attackerHeroAttrList = msg.attack_list or {}
	self.m_defenderHeroAttrList = msg.defence_list or {}
end

function This:loadFinish()
	if not self.m_isFinish then
		self:close()
		if self.m_finishHandler then
			self.m_finishHandler(self.m_reportId)
		end
		self.m_isFinish = true
	end
end

function This:update(dt)
	super.update(self,dt)
	if self.m_waiting then
		return 
	end
	if #self.m_tasks <= 0 then
		self:loadFinish()
		return
	end
	self.m_escape = self.m_escape + dt

	if self.m_curIndex > #self.m_tasks then
		if self.m_escape > 2 then
			self:loadFinish()
			self.m_escape = self.m_escape - 1
		end
	else
		local task = self.m_tasks[self.m_curIndex]
		task:execute()
		if task:isFinish() then
			self.m_curIndex = self.m_curIndex + 1
		end
	end
end

--endregion
