--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local Timer = require("Timer")
local BattleDataManager = require("Battle.BattleDataManager")
local BattlePerformer = require("Battle.BattlePerformer")

function This:create()
	local player = {}
	setmetatable(player, self)
	player:init()
	return player
end

function This:init()
	self.m_timer = Timer:create(self)
	self.m_enemyTroops = {}
	self.m_friendTroops = {}
	self.m_observers = {}
	self.m_roundData = nil
	self.m_actionIndex = 0
	self.m_nextRoundHandler = nil
	self.m_isInPrepare = false
end

function This:clear()
	self.m_enemyTroops = {}
	self.m_friendTroops = {}
	self.m_observers = {}
	self.m_roundData = nil
	self.m_actionIndex = 0
	self.m_nextRoundHandler = nil
	self.m_isInPrepare = false
end

function This:addObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			return
		end
	end
	table.insert(self.m_observers,observer)
end

function This:removeObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			table.remove(self.m_observers,i)
			break
		end
	end
end

function This:notifyObserver(funcName,...)
	local observers = {}
	for k , v in pairs(self.m_observers) do
		observers[k] = v
	end
	for k , o in pairs(observers) do
		if o[funcName] then
			o[funcName](o,...)
		end
	end
end

function This:addTroops(troops)
	local isEnemy = troops:isEnemy()
	local index = troops:getIndex()
	if isEnemy then
		self.m_enemyTroops[index] = troops
	else
		self.m_friendTroops[index] = troops
	end
	troops:setBattlePlayer(self)
	self:notifyObserver("onAddTroops",troops)
end

function This:removeTroops(troops)
	local isEnemy = troops:isEnemy()
	local index = troops:getIndex()
	if isEnemy then
		if self.m_enemyTroops[index] then
			self.m_enemyTroops[index] = nil
		end
	else
		if self.m_friendTroops[index] then
			self.m_friendTroops[index] = nil
		end
	end
	self:notifyObserver("onRemoveTroops",troops)
end

function This:getTroops(index,isEnemy)
	if isEnemy then
		return self.m_enemyTroops[index]
	else
		return self.m_friendTroops[index]
	end	
end

function This:getTroopsByHeroId(isEnemy, heroId)
	if isEnemy then
		for index , troops in pairs(self.m_enemyTroops) do
			if troops:getHeroId() == heroId then
				return troops
			end
		end
	else
		-- print_tmp(self.m_friendTroops)
		for index , troops in pairs(self.m_friendTroops) do
			if troops:getHeroId() == heroId then
				return troops
			end
		end
	end
end


function This:getEnemyTroopsLeftHp()
	local totalHp = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() then
			totalHp = totalHp + troops:getAttr(AttrType.kHp)
		end
	end
	return totalHp
end

function This:getEnemyTroopsMaxHp()
	local maxHp = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() then
			maxHp = maxHp + troops:getAttr(AttrType.kMaxHp)
		end
	end
	return maxHp
end

function This:getFriendTroopsLeftHp()
	local totalHp = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() then
			totalHp = totalHp + troops:getAttr(AttrType.kHp)
		end
	end
	return totalHp
end

function This:getFriendTroopsMaxHp()
	local totalHp = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() then
			totalHp = totalHp + troops:getAttr(AttrType.kMaxHp)
		end
	end
	return totalHp
end

function This:getEnemyTroopsCount()
	local count = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() then
			count = count  + 1
		end
	end
	return count
end

function This:getFriendTroopsCount()
	local count = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() then
			count = count  + 1
		end
	end
	return count
end

function This:pauseBattle()
	self.m_isPause = true
end

function This:resumeBattle()
	self.m_isPause = false
end

function This:isInPrepare()
	return self.m_isInPrepare
end

function This:playAction(action)
	local troops = self:getTroopsByHeroId(action.isEnemy,action.heroId)
	if troops == nil then
		assert(false)
	end
	if troops then
		troops:performAction(action)
	else
		print("缺少的heroid11111======"..action.heroId)
	end
end

function This:enterBattle(attackerList,defenderList)
	for pos , info in pairs(attackerList) do
		local troops = BattlePerformer:create(info)
		self:addTroops(troops)
	end
	for pos , info in pairs(defenderList) do
		local troops = BattlePerformer:create(info)
		self:addTroops(troops)
	end
end

function This:playBattle(RoundData,roundIndex, nextRoundHandler)
	print(string.format("\n第%d回合开始",roundIndex))
	self.m_actionIndex = 0
	self.m_isInPrepare = (roundIndex == 0)
	self.m_roundData = RoundData
	self.m_nextRoundHandler = nextRoundHandler
	self:playNext()
	self:notifyObserver("onNextRound",roundIndex)
end

function This:playNext()
	if self.m_actionIndex < #self.m_roundData then
		self.m_actionIndex = self.m_actionIndex + 1
		print(string.format("\n第%d次攻击...",self.m_actionIndex))
		self:playAction(self.m_roundData[self.m_actionIndex])
		return false
	else
		self.m_nextRoundHandler()
	end
	return true
end

function This:onTroopsForward(troops)
	self:notifyObserver("onTroopsForward",troops)
end

function This:onTroopsAttack(troops,skill)
	self:notifyObserver("onTroopsAttack",troops,skill)
end

function This:onTroopsHitted(troops)
	self:notifyObserver("onTroopsHitted",troops)
end

function This:onTroopsBack(troops)
	self:notifyObserver("onTroopsBack",troops)
end

function This:onTroopsDead(troops)
	self:notifyObserver("onTroopsDead",troops)
end

function This:onSkillActived(troops,skill)
	self:notifyObserver("onSkillActived",troops,skill)
end

function This:isAllTroopsFree()
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() and troops:getState() ~= TroopsState.kFree then
			return false
		end
	end
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() and troops:getState() ~= TroopsState.kFree then
			return false
		end
	end
	return true
end

function This:update(dt)
	local isAllFree = self:isAllTroopsFree()
	if isAllFree then
		self:playNext()
	end
end


--endregion
