--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local BattleUtil = require("Battle.BattleUtil")
local Timer = require("Timer")
local BattleDataManager = require("Battle.BattleDataManager")
local BattlePerformer = require("Battle.BattlePerformer")
local WallSoilder = require("Battle.WallSoilder")
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local MythicalAnimalsAttributeConfig = require("Metadatas.MythicalAnimalsAttributeConfig")

function This:create()
	local player = {}
	setmetatable(player, self)
	player:init()
	return player
end

function This:init()
	super.init(self)
	self.m_timer = Timer:create(self)
	self.m_enemyTroops = {}
	self.m_friendTroops = {}
	self.m_roundData = nil
	self.m_actionIndex = 0
	self.m_isInPrepare = false
	self.m_escape = 0
	self.m_isPause = false
	self.m_isPlaying = false
	self.m_attackerInfo = {}
	self.m_defencerInfo = {}
	self.m_playMode = 0
	self.m_escape = 0
	self.m_mineTroops = {}
	self.m_notMineTroops = {}
end

function This:clear()
	self.m_enemyTroops = {}
	self.m_friendTroops = {}
	self.m_roundData = nil
	self.m_actionIndex = 0
	self.m_isInPrepare = false
	self.m_isPlaying = false
	self.m_isPause = false
	self.m_attackerInfo = {}
	self.m_defencerInfo = {}
	self.m_playMode = 0
	self.m_escape = 0
	self.m_endTime = 0
	self.m_startTime = 0
	self.m_mineTroops = {}
	self.m_notMineTroops = {}
end

function This:prepare(isImmediate,playMode,mapInfoId)
	self.m_enemyTroops = {}
	self.m_friendTroops = {}	
	self.m_roundData = nil
	self.m_actionIndex = 0
	self.m_isInPrepare = true
	self.m_isPlaying = false
	self.m_isPause = false
	self.m_startTime = 0
	self.m_endTime = 0
	self.m_isImmediate = isImmediate
	self.m_playMode = playMode
	self.m_mapInfoId = mapInfoId
	self.m_attackerInfo = {}
	self.m_defencerInfo = {}
	self.m_mineTroops = {}
end

function This:setBattleTime(startTime,endTime)
	self.m_startTime = startTime
	self.m_endTime = endTime
end

function This:getMapInfoId()
	return self.m_mapInfoId
end

function This:addTroops(troops)
	local isEnemy = troops:isEnemy()
	local index = troops:getIndex()
	local heroId = troops:getHeroId()
	local temTroop = self:getTroopsByHeroId(isEnemy, heroId)
	if isEnemy then
		--[[if self.m_enemyTroops[index] then
			local oldTroop = self.m_enemyTroops[index]
			self.m_enemyTroops[index] = nil
			self:notifyObservers("onRemoveTroops",oldTroop)
		end]]
		if temTroop then
			local temIndex = temTroop:getIndex()
			self.m_enemyTroops[temIndex] = nil
		end
		table.insert(self.m_notMineTroops,troops)
		self.m_enemyTroops[index] = troops
		self:notifyObservers("onAddTroops",troops)
	else
		--[[if self.m_friendTroops[index] then
			local oldTroop = self.m_friendTroops[index]
			self.m_friendTroops[index] = nil
			self:notifyObservers("onRemoveTroops",oldTroop)
		end]]
		if temTroop then
			local temIndex = temTroop:getIndex()
			self.m_friendTroops[temIndex] = nil
		end

		if heroId ~= 1000 then  --不要城墙
			table.insert(self.m_mineTroops,troops)
			self.m_friendTroops[index] = troops
			self:notifyObservers("onAddTroops",troops)
		end
	end
end

function This:removeTroops(troops)
	local isEnemy = troops:isEnemy()
	local index = troops:getIndex()
	if isEnemy then
		if self.m_enemyTroops[index] then
			--self.m_enemyTroops[index] = nil
			self.m_enemyTroops[index]:setState(TroopsState.kDead)
		end
	else
		if self.m_friendTroops[index] then
			--self.m_friendTroops[index] = nil
			self.m_friendTroops[index]:setState(TroopsState.kDead)
		end
	end
	self:notifyObservers("onRemoveTroops",troops)
end

function This:getTroops(index,isEnemy)
	if isEnemy then
		local tmpTroop = self.m_enemyTroops[index]

		if not tmpTroop then
			for i,troops in pairs(self.m_notMineTroops) do
				if troops and troops:getIndex() == index then
					return troops
				end
			end
		end
		return tmpTroop
	else
		local tmpTroop = self.m_friendTroops[index]
		if not tmpTroop then
			for i,troops in pairs(self.m_mineTroops) do
				if troops and troops:getIndex() == index then
					return troops
				end
			end
		end

		return tmpTroop
	end	
end

function This:troopsResurgence(troops)
	troops:resurgence()
	self:addTroops(troops)
	self:notifyObservers("onResurgenceTroops",troops)
end

function This:getTroopsByHeroId(isEnemy, heroId)
	if isEnemy then
		-- print_tmp(self.m_enemyTroops)
		if MythicalAnimalsAttributeConfig[heroId] then  --如果是神兽
			local tmpTroop = nil
			for index , troops in pairs(self.m_friendTroops) do
				local hp = troops:getAttr(AttrType.kHp)
				if troops:getHeroId() == heroId then
					tmpTroop = troops
				end
			end
			if not tmpTroop then
				for index,_troops in pairs(self.m_notMineTroops) do
					if _troops:getHeroId() == heroId then
						return _troops
					end
				end
			else
				return tmpTroop
			end
			return tmpTroop
		else
			for index , troops in pairs(self.m_enemyTroops) do
				if troops:getHeroId() == heroId then
					return troops
				end
			end
		end
		for index , troops in pairs(self.m_enemyTroops) do
			if troops:getHeroId() == heroId then
				return troops
			end
		end
	else
		local tmpTroop = nil
		for index , troops in pairs(self.m_friendTroops) do
			local hp = troops:getAttr(AttrType.kHp)
			if troops:getHeroId() == heroId then
				tmpTroop = troops
			end
		end
		if not tmpTroop then
			for index,_troops in pairs(self.m_mineTroops) do
				if _troops:getHeroId() == heroId then
					return _troops
				end
			end
		else
			return tmpTroop
		end
		return tmpTroop
	end
end


function This:getEnemyTroopsLeftHp()
	local totalHp = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() and troops:isMonster() ~= 1 then
			totalHp = totalHp + troops:getAttr(AttrType.kHp)
		end
	end
	return totalHp
end

function This:getEnemyTroopsMaxHp()
	local maxHp = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() and troops:isMonster() ~= 1 then
			maxHp = maxHp + troops:getAttr(AttrType.kMaxHp)
		end
	end
	return maxHp
end

function This:getFriendTroopsLeftHp()
	local totalHp = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() and troops:isMonster() ~= 1 then
			totalHp = totalHp + troops:getAttr(AttrType.kHp)
		end
	end
	return totalHp
end

function This:getFriendTroopsMaxHp()
	local totalHp = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() and troops:isMonster() ~= 1 then
			totalHp = totalHp + troops:getAttr(AttrType.kMaxHp)
		end
	end
	return totalHp
end

function This:getEnemyTroopsCount()
	local count = 0
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() and troops:getIndex() < 7 then
			count = count  + 1
		end
	end
	return count
end

function This:getFriendTroopsCount()
	local count = 0
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive()and troops:getIndex() < 7 then
			count = count  + 1
		end
	end
	return count
end

function This:getAttackerInfo()
	return self.m_attackerInfo
end

function This:setAttackerInfo(info)
	self.m_attackerInfo = info
end

function This:getDefenderInfo()
	return self.m_defenderInfo
end

function This:setDefenderInfo(info)
	self.m_defenderInfo = info
end

function This:getRoundIndex()
	return self.m_roundIndex
end

function This:getActionIndex()
	return self.m_actionIndex
end

function This:pauseBattle()
	self.m_isPause = true
end

function This:resumeBattle()
	self.m_isPause = false
end

function This:isInPrepare()
	return self.m_isInPrepare
end

function This:isImmediate()
	return self.m_isImmediate
end

function This:isPlaying()
	return self.m_isPlaying
end

function This:getPlayMode()
	return self.m_playMode
end

function This:playNext()
	if (self.m_roundIndex > #self.m_battleData) or (self.m_roundIndex == #self.m_battleData and self.m_actionIndex > #self.m_battleData[self.m_roundIndex]) then
		BattleDataManager:nextBattle()
	elseif self.m_actionIndex <= #self.m_battleData[self.m_roundIndex] then
		local action = self.m_battleData[self.m_roundIndex][self.m_actionIndex]
		print(string.format("\n第%d次攻击, 查询武将ID:%d, 是否是敌人:%s ...",self.m_actionIndex,action.heroId,action.isEnemy))
		local troops = self:getTroopsByHeroId(action.isEnemy,action.heroId)
		if troops == nil then
			assert(false)
		end
		if troops then
			troops:performAction(action)
		else
			print("缺少的heroid222222======"..action.heroId)
		end
		self.m_actionIndex = self.m_actionIndex + 1
	elseif self.m_roundIndex < #self.m_battleData then
		self.m_actionIndex = 1
		self.m_roundIndex = self.m_roundIndex + 1
		self:notifyObservers("onNextRound",self.m_roundIndex)

		local action = self.m_battleData[self.m_roundIndex][self.m_actionIndex]
		print(string.format("\n第%d回合，第1动作，查询武将%d，是否是敌人：%s...",self.m_roundIndex,action.heroId,action.isEnemy))
		local troops = self:getTroopsByHeroId(action.isEnemy,action.heroId)
		if troops == nil then
			assert(false)
		end
		if troops then
			troops:performAction(action)
		else
			print("缺少的heroid33333======"..action.heroId)
		end
		self.m_actionIndex = self.m_actionIndex + 1
	end
end

function This:startPlay(startTime,endTime)
	self.m_isInPrepare = false
	self.m_isPlaying = true
	if self.m_startTime < startTime then
		self.m_startTime = startTime
	end
	self.m_endTime = endTime
	self.m_escape = 0
end

function This:continuePlay(startTime,endTime)
	self:resumeBattle()
	self.m_isPlaying = true
	self.m_isInPrepare = false
	self.m_escape = 0
	self.m_startTime = startTime
	self.m_endTime = endTime
end

function This:deducteBattle(battleData,attackerList,defenderList,isImmediate)
	self.m_isInPrepare = true
	self.m_isImmediate = isImmediate
	self.m_battleData = battleData

	local str = Serialize[60476]
	for id , troop in pairs(self.m_friendTroops) do
		str = str .. troop:getHeroId() .. ","
	end
	print(str)

	str = Serialize[60477]
	for id , troop in pairs(self.m_enemyTroops) do
		str = str .. troop:getHeroId() .. ","
	end
	print(str)

	str = Serialize[60478]
	for pos , info in pairs(attackerList) do
		local troops = BattlePerformer:create(info)
		self:addTroops(troops)
		str = str .. info.heroId .. ","
	end
	print(str)
	str = Serialize[60479]
	for pos , info in pairs(defenderList) do
		if pos == kWallSoilderIndex then
			local troops = WallSoilder:create(info)
			self:addTroops(troops)
		else
			local troops = BattlePerformer:create(info)
			self:addTroops(troops)
		end
		str = str .. info.heroId .. ","
	end
	print(str)

	local curTime = GamePlayer:getServerTime() * 1000
	--[[for i , action in ipairs(battleData[0]) do
		local troops = self:getTroopsByHeroId(action.isEnemy,action.heroId)
		troops:performAction(action)
	end]]

	self.m_roundIndex = 1
	self.m_actionIndex = 1
	if battleData[0] then
		self.m_roundIndex = 0
	end
	local isFind = false

	while isImmediate and self.m_roundIndex <= #battleData do
		self.m_actionIndex = 1
		for i , action in ipairs(battleData[self.m_roundIndex]) do
			if action.startTime >= curTime then
				self.m_startTime = action.startTime / 1000
				self.m_actionIndex = i
				isFind = true
				break
			end
			self.m_actionIndex = i
			--print(string.format("\n-----------------推演第%d回合,第%d动作----------------------",self.m_roundIndex,self.m_actionIndex))
			local troops = self:getTroopsByHeroId(action.isEnemy,action.heroId)
			if troops then
				troops:performAction(action)
			else
				print("缺少的heroid111111======"..action.heroId)
			end
		end
		if isFind then
			break
		end
		self.m_roundIndex = self.m_roundIndex + 1
	end
	if isImmediate and not isFind then
		self.m_startTime = self.m_endTime
	end
	self:notifyObservers("onStartPlayBattle",self.m_roundIndex)
end

function This:exitBattle()
	self.m_isPlaying = false
	self.m_endTime = 0
	self.m_startTime = 0
end

-----------------------------------------------------------
function This:onTroopsForward(troops)
	self:notifyObservers("onTroopsForward",troops)
end

function This:onTroopsAttack(troops,skill)
	self:notifyObservers("onTroopsAttack",troops,skill)
end

function This:onMonsterAttack(troops,toDoWorkHandler)
	self:notifyObservers("onMonsterAttack",troops,toDoWorkHandler)
end

function This:onReturnPostion(troops)
	self:notifyObservers("onReturnPostion",troops)
end

function This:onTroopsHitted(troops)
	self:notifyObservers("onTroopsHitted",troops)
end

function This:onTroopsBack(troops)
	self:notifyObservers("onTroopsBack",troops)
end

function This:onTroopsDead(troops)
	self:notifyObservers("onTroopsDead",troops)
end

function This:onSkillActived(troops,skill,effectList)
	self:notifyObservers("onSkillActived",troops,skill,effectList)
end

function This:onBuffActived(troops,buff,effectList)
	self:notifyObservers("onBuffActived",troops,buff,effectList)
end

function This:onBuffActiveEnded(buff)
	self:notifyObservers("onBuffActiveEnded",buff)
end

function This:isAllTroopsFree()
	for i , troops in pairs(self.m_friendTroops) do
		if troops:isAlive() and troops:getState() ~= TroopsState.kFree then
			return false
		end
	end
	for i , troops in pairs(self.m_enemyTroops) do
		if troops:isAlive() and troops:getState() ~= TroopsState.kFree then
			return false
		end
	end
	return true
end

function This:playAction(et)
	for i = self.m_roundIndex , #self.m_battleData do
		for j = self.m_actionIndex , #self.m_battleData[i] do
			local action = self.m_battleData[i][j]
			if action.startTime > et then
				return
			end
			if not self:isAllTroopsFree() then
				return
			end
			print(string.format("\n第%d回合，第%d动作，查询武将%d，是否是敌人：%s,时间%d",i,j,action.heroId,tostring(action.isEnemy),action.startTime))
			local troops = self:getTroopsByHeroId(action.isEnemy,action.heroId)
--			if i==1 and action.heroId == 10004 then
--				print("kkIsMe")
--				local view = troops:getView()
--			end
			
			if troops then
				troops:performAction(action)
			else

				for index,_troops in pairs(self.m_mineTroops) do
					-- print("indexiiiiiiiiiii======"..index..">>>>>>===="..troops:getHeroId())
					troops = _troops
					if troops then
						troops:performAction(action)
					else
						print("缺少的heroid44444======"..action.heroId)
					end
				end

			end
			self.m_actionIndex = j + 1
		end
		self.m_roundIndex = i + 1
		self.m_actionIndex = 1
		if self.m_roundIndex <= #self.m_battleData then
			self:notifyObservers("onNextRound",self.m_roundIndex)
		end
	end
end

--[[function This:update(dt)
	if self.m_isPlaying then
		local isAllFree = self:isAllTroopsFree()
		if isAllFree then
			self.m_escape = self.m_escape + dt
			if not self.m_isPause and self.m_escape >= 0.1 then
				self:playNext()
				self.m_escape = 0
			end
		end
	end
end]]

function This:isFinish()
	if not self:isAllTroopsFree() then
		return false
	end
	if self.m_roundIndex > #self.m_battleData then
		return true
	end
	if self.m_roundIndex == #self.m_battleData and self.m_actionIndex >= #self.m_battleData[self.m_roundIndex] then
		return true
	end
	return false
end

function This:update(dt)
	if self.m_isPlaying and not self.m_isPause then
		self.m_escape = self.m_escape + dt
		if self.m_roundIndex <= #self.m_battleData then
			local effectTime = 2
			if self:getAttackerInfo().drumData.level == 0 and self:getDefenderInfo().drumData.level == 0 then
				effectTime = 0
			end
			self:playAction((self.m_startTime-effectTime + self.m_escape) * 1000)
		elseif self.m_escape + self.m_startTime > self.m_endTime then
			if self:isFinish() then
				BattleDataManager:nextBattle()
			end
		end
	end
end


function This:print()
	print("我方：")
	for i , troop in pairs(self.m_friendTroops) do
		print(string.format("heroId:%d",troop:getHeroId()))
	end
	print("敌方:")
	for i , troop in pairs(self.m_enemyTroops) do
		print(string.format("heroId:%d",troop:getHeroId()))
	end
end

--endregion
