--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This,super = class(...,"Scene")
local TroopsView = require("Battle.TroopsView")
local WallSoilderView = require("Battle.WallSoilderView")
local BattleUI = require("Battle.BattleUI")
local BattleDataManager = require("Battle.BattleDataManager")
local Timer = require("Timer")
local BattlePlayer = require("Battle.BattlePlayer")
local ShooterView = require("Battle.ShooterView")
local StoneThrowerView = require("Battle.StoneThrowerView")
local CavalryView = require("Battle.CavalryView")
local BattleSceneConfig = require("Metadatas.BattleSceneConfig")
local GameUtil = require("Util.GameUtil")

BattleZorder = 
{
	kMapLayer = 0,
	kEffectLayer = 1,
	kUiLayer = 2,
	--------------
	kMaskLayer = 100,
	kZhanHuoLayer = 120,
}

function This:init()
	super.init(self)
	cc.SpriteFrameCache:addPlist("Fonts")
	self.m_mapInfoId = BattlePlayer:getMapInfoId()
	self.m_config = BattleSceneConfig[self.m_mapInfoId]
	self.m_troopsViews = {}
	self.m_accelerateFactor = 1.0
	self.m_skillCount = 0
	self.m_isPause = false
	self.name = "BattleScene"
	g_weakTable["BattleScene"] = self
end

function This:initLayer()
	self.m_mapLayer = cc.Node:create()
	self:addChild(self.m_mapLayer)
	self.m_mapLayer:setLocalZOrder(BattleZorder.kMapLayer)

	local bg = cc.Sprite:create(self.m_config.image.file)
	bg:setPosition(cc.visible.center.x + self.m_config.image.x,cc.visible.center.y + self.m_config.image.y)
	bg:setScale(self.m_config.image.scalex,self.m_config.image.scaley)
	self.m_mapLayer:addChild(bg)

	if self.m_config.image.dixing and self.m_config.image.dixing ~= "" then
		local bg1 = cc.Sprite:create(self.m_config.image.dixing)
		bg1:setPosition(cc.visible.center.x + self.m_config.image.x,cc.visible.center.y + self.m_config.image.y)
		bg1:setScale(self.m_config.image.scalex,self.m_config.image.scaley)
		self.m_mapLayer:addChild(bg1)
	end

	self.m_maskLayer = cc.LayerColor:create(cc.Color(0,0,0,210))
	self.m_maskLayer:setContentSize(cc.visible.width,cc.visible.height)
	self.m_mapLayer:addChild(self.m_maskLayer)
	self.m_maskLayer:setLocalZOrder(BattleZorder.kMaskLayer)
	self.m_maskLayer:setVisible(false)

	self.m_effectLayer = cc.Node:create()
	self:addChild(self.m_effectLayer)
	self.m_effectLayer:setLocalZOrder(BattleZorder.kEffectLayer)

	local effect = GameUtil:createEffect("ZhanDou",true)
	effect:setLocalZOrder(BattleZorder.kZhanHuoLayer)
	effect:setPosition(cc.visible.center.x,cc.visible.center.y)
	self:addEffect(effect)

	if BattlePlayer:getPlayMode() == PlayMode.kStage then
		effect:playAnimation("animation",true)
	else
		effect:playAnimation("animation2",true)
	end

	self.m_ui = BattleUI:create()
	self:addChild(self.m_ui)
	self.m_ui:setLocalZOrder(BattleZorder.kUiLayer)
	--self.m_ui:playDrumEffect()
end

function This:initTroops()
	for i = 1 , kWallSoilderIndex - 1 do
		local troop = BattlePlayer:getTroops(i,false)
		if troop then
			self:onAddTroops(troop)
		end
	end
	for i = 1 , kWallSoilderIndex do
		local troop = BattlePlayer:getTroops(i,true)
		if troop then
			self:onAddTroops(troop)
		end
	end
end

function This:onEnter()
	super.onEnter(self)
	self:initLayer()
	self:initTroops()
	self:scheduleUpdate()
	BattlePlayer:addObserver(self)
	cc.Audio:playBackground("Bg_Fighting.mp3")
end

function This:onExit()
	super.onExit(self)
	self:unscheduleUpdate()
	BattlePlayer:removeObserver(self)
	BattleDataManager:clear()
	cc.Audio:stopBackground()
end

function This:openDialog(name, info)
	local panel = super.openDialog(self,name,info)
	panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	return panel
end

function This:getMapConfig()
	return self.m_config
end

function This:getTimer()
	return self.m_timer
end

function This:pause()
	self.m_isPause = true
end

function This:resume()
	self.m_isPause = false
end

function This:getTroopsPosition(x, y)
	local spacex , spacey = 150 , 150
	local dir = x > 0 and 1 or -1
	local firstx , firsty = cc.visible.center.x + 130 * dir , cc.visible.center.y + spacey
	return firstx + (x - dir) * spacex , firsty - (y - 1) * spacey
end

function This:getTroopsConfig(troopId)
	return self.m_config.troop[troopId]
end

function This:createTroopsView(model)

	if model:getIndex() < 7 then
		local soilderId = model:getSoilderId()
		if soilderId == SoilderType.kArcher then
			return ShooterView:create(model)
		elseif soilderId == SoilderType.kSapperCar then
			return StoneThrowerView:create(model)
		elseif soilderId == SoilderType.kCavalry then
			return CavalryView:create(model)
		else
			return TroopsView:create(model)
		end
	else
		return WallSoilderView:create(model)
	end
end

function This:getTroopsView(model)
	local id = model:getId()
	return self.m_troopsViews[id]
end

function This:onAddTroops(model)
	if model:isAlive() then
		local id = model:getId()
		if self.m_troopsViews[id] then
			self:onRemoveTroops(model)
		end

		local config = self:getTroopsConfig(model:getId())
		local view = self:createTroopsView(model)
		self.m_mapLayer:addChild(view)
		
		if config then
			view:setPosition(cc.visible.center.x + config.x - 480, cc.visible.center.y + config.y - 270)
			view:setScale(config.scale)
			view:setLocalZOrder(config.zorder)
		else
			print("缺少的model:getId()===="..model:getId())
		end
		view:setAnchorPoint(0.5,0.5)
		view:setActionRunWith(self)
		self.m_troopsViews[id] = view
	end
end

function This:onRemoveTroops(model)
	local id = model:getId()
	local view = self.m_troopsViews[id]
	if view then
		view:removeFromParent()
		self.m_troopsViews[id] = nil
	end
end

function This:setAccelerateFactor(factor)
	self.m_accelerateFactor = factor
end

function This:getAccelerateFactor()
	return self.m_accelerateFactor
end

function This:addEffect(effect)
	self.m_effectLayer:addChild(effect)
	local scale = effect:getScale()
	effect:setScale(self.m_config.effect.scale * scale)
	effect:setActionRunWith(self)
end

function This:onSkillActived(troop,skill,effectList)
	if skill:isPassive() then
		return
	end
	if skill:getId() > 1 then
		self.m_maskLayer:setVisible(true)
		self.m_skillCount = self.m_skillCount + 1	
		local config = self.m_config.troop
		for i , effect in ipairs(effectList) do
			for i , hitInfo in ipairs(effect.target) do         
				local target = BattlePlayer:getTroopsByHeroId(hitInfo.isEnemy,hitInfo.heroId)
				if target then
					local targetView = target:getView()
					if targetView then
						targetView:setLocalZOrder(BattleZorder.kMaskLayer + config[target:getId()].zorder)
					end
				end
			end
		end
		local view = troop:getView()
		if view then
			view:setLocalZOrder(BattleZorder.kMaskLayer + config[troop:getId()].zorder)
		end
	else
		self.m_skillCount = 0
		local config = self.m_config.troop
		self.m_maskLayer:setVisible(false)
		for i , view in pairs(self.m_troopsViews) do
			local model = view:getModel()
			if config[model:getId()] then
				view:setLocalZOrder(config[model:getId()].zorder)
			else
				print("model:getId()===========")
			end
		end
	end
end

function This:onBuffActived(troop,buff,effectList)
	--[[if buff:hasActiveEffect() then
		self.m_skillCount = self.m_skillCount + 1	
		local hasTarget = false
		local config = self.m_config.troop
		for i , effect in ipairs(effectList) do
			for i , hitInfo in ipairs(effect.target) do  
				if hitInfo.effectResult == 0 then       
					local target = BattlePlayer:getTroopsByHeroId(hitInfo.isEnemy,hitInfo.heroId)
					if target then
						local targetView = target:getView()
						targetView:setLocalZOrder(BattleZorder.kMaskLayer + config[target:getId()].zorder)
						hasTarget = true
					end
				end
			end
		end
		self.m_maskLayer:setVisible(hasTarget)
		local view = troop:getView()
		view:setLocalZOrder(BattleZorder.kMaskLayer + config[troop:getId()].zorder)
	end]]
end

function This:onBuffActiveEnded(buff)
	if buff:hasActiveEffect() then
		self.m_skillCount = self.m_skillCount - 1
		local config = self.m_config.troop
		self.m_maskLayer:setVisible(false)
		for i , view in pairs(self.m_troopsViews) do
			local model = view:getModel()
			view:setLocalZOrder(config[model:getId()].zorder)
		end
	end
end

function This:onTroopsBack(troop)
	local skill = troop:getSkill()
	if skill and skill:getId() > 1 then
		self.m_skillCount = self.m_skillCount - 1
		local config = self.m_config.troop
		self.m_maskLayer:setVisible(self.m_skillCount > 0)
		for i , view in pairs(self.m_troopsViews) do
			local model = view:getModel()
			view:setLocalZOrder(config[model:getId()].zorder)
		end
	end
end

function This:shake(x,y,time)
	self.m_mapLayer:shake(x,y,time)
end

function This:update(dt)
	local t = dt * self.m_accelerateFactor
	super.update(self,t)
	if not self.m_isPause then
		BattlePlayer:update(t)
	end
end

--endregion
