--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local HeroUtil = require("Hero.HeroUtil")
local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")
local BattleDataManager = require("Battle.BattleDataManager")
local BattlePlayer = require("Battle.BattlePlayer")
local GamePlayer = require("Player.GamePlayer")
local ReportManager = require("Report.ReportManager")
local BattleArmyItem = require("Battle.BattleArmyItem")
local WorldMapManager = require("World.WorldMapManager")
local PrivilegeManager = require("Vip.PrivilegeManager")
local HeroManager = require("Hero.HeroManager")
local DrumManager = require("Drum.DrumManager")
local kCostIconVisible = false
local kDefaultAccFactorInWarfare = 2

local kPlayModeNormalSpeed ={--某个模式的默认速度
	[PlayMode.kRescu] = 2,--勤王战
}

local drumIconConfig={
    "MenuScene/bgzhangu2.png",
    "MenuScene/guchui.png",
}

function This:init()
    self.m_acceFactorRecord = false
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/GameScene.csb")
	self.m_csNode:fitScreen(1)
	self.m_MonsterSkeleton = {}
	self:addChild(self.m_csNode)
	local mode = BattlePlayer:getPlayMode()
	self.m_acceFactor = kPlayModeNormalSpeed[mode] or 1
	self:refreshAccelerateFactor(self.m_acceFactor)
	self:initNode()
end

function This:getTotalHp(heroInfo)
	local hp = 0
	for i , info in pairs(heroInfo) do
		if info.heroId > 0 and info.fight_card_type ~= 1 then
			hp = hp + info.hp
		end
	end
	return hp
end

function This:getHeroNum(heroInfo)
	local count = 0
	for i , info in pairs(heroInfo) do
		if info.pos < 7 and info.heroId >0 then
			count  = count + 1
		end
	end
	return count
end

function This:initNode()
	local attackerInfo = BattleDataManager:getAttackerInfo()
	local attackHeros = BattleDataManager:getAttackerHero()

	--队列面板状态
	self.m_startTeamX, self.m_startTeamY = self.m_csNode.teamMessage:getPosition()
	self.m_isShowTeam = true
	
	self.m_csNode.role.name:setString(attackerInfo.name or "")
	self.m_csNode.role.level:setString(attackerInfo.level or "")
	self.m_csNode.role.speedNum:setString(attackerInfo.speed or 0)
	GameUtil:setPlayerIconNodeImage(self.m_csNode.role.icon, attackerInfo.icon)
	self.m_csNode.role.heroNum:setString(self:getHeroNum(attackHeros))
	self.m_csNode.role.hpbat:setPercentStyle(1)

	local attackTotalHp = self:getTotalHp(attackHeros)
	self.m_csNode.role.troopNum:setString(math.floor(attackTotalHp))
	self.m_attackTotalHp = attackTotalHp
	self:refreshAttackerInfo(true)

	local defenderInfo = BattleDataManager:getDefenderInfo()
	local defenderHeros = BattleDataManager:getDefenderHero()

	self.m_csNode.boss.name:setString(defenderInfo.name or "")
	self.m_csNode.boss.level:setString(defenderInfo.level or "")
	self.m_csNode.boss.speedNum:setString(defenderInfo.speed or 0)
	GameUtil:setPlayerIconNodeImage(self.m_csNode.boss.icon, defenderInfo.icon)
	self.m_csNode.boss.heroNum:setString(self:getHeroNum(defenderHeros))
	
	local defendTotalHp = self:getTotalHp(defenderHeros)
	self.m_csNode.boss.troopsNum:setString(math.floor(defendTotalHp))
	self.m_defendTotalHp = defendTotalHp
	self:refreshDefenderInfo(true)

	local mode = BattlePlayer:getPlayMode()
	self.m_csNode.round.text:setString(BattlePlayer:getRoundIndex())
	self.m_csNode.acceleratebutton:setVisible(BattlePlayAccelerate[mode])
	self.m_csNode.skip:setVisible(BattlePlaySkip[mode])
	self.m_csNode.icon:setVisible(kCostIconVisible and mode == PlayMode.kStage)
	self.m_csNode.cost:setVisible(kCostIconVisible and mode == PlayMode.kStage)

	self.m_csNode.quitbutton:setVisible(mode == PlayMode.kWorld)

	self.m_effectLayer = cc.Node:create()
	self:addChild(self.m_effectLayer)
	self.m_effectLayer:setLocalZOrder(BattleZorder.kEffectLayer)
	self:playDrumEffect()
	self:initArmyQueue()
end

function This:refreshAttackerInfo(isInit)
	local friendLeftHp = BattlePlayer:getFriendTroopsLeftHp()
	self.m_csNode.role.hp:setString(string.format("%d/%d",friendLeftHp,self.m_attackTotalHp))
	if not isInit then
		self.m_csNode.role.hpbat:runPercentAction(friendLeftHp / self.m_attackTotalHp * 100,100)
	else
		self.m_csNode.role.hpbat:setPercent(friendLeftHp / self.m_attackTotalHp * 100)
	end
	local attackerInfo = BattlePlayer:getAttackerInfo()
	local defenderInfo = BattlePlayer:getDefenderInfo()
	self.m_csNode.role.name:setString(attackerInfo.name or "")
	self.m_csNode.role.level:setString(attackerInfo.level or "")
	self.m_csNode.role.speedNum:setString(attackerInfo.speed or 0)
	GameUtil:setPlayerIconNodeImage(self.m_csNode.role.icon, attackerInfo.icon)

	local troopNum = BattlePlayer:getFriendTroopsCount()
	self.m_csNode.role.heroNum:setString(troopNum)
	self.m_csNode.role.troopNum:setString(math.floor(friendLeftHp))

	local mode = BattlePlayer:getPlayMode()
	if mode == PlayMode.kWorld then
		WorldMapManager:addHanXianDiSkillInfo(self.m_csNode.buff1,attackerInfo.id)
	end
	local drumData = attackerInfo.drumData
	local defendDrumData = defenderInfo.drumData
	local isWin = drumData.level > defendDrumData.level
	if drumData then
		self.m_csNode.drum1.drumstar:setVisible(DrumManager:getNowHaloLevel(drumData.level)>0)
		if drumData.level > 0 then
			local pic = DrumManager:getNowHaloImage(DrumManager:getNowHaloLevel(drumData.level))
			self.m_csNode.drum1.drumstar:setImage(pic)
			if isWin and not self.m_attackerEffect then
				self.m_attackerEffect = GameUtil:createEffect("Itemnb", true)
				self.m_attackerEffect:setScale(0.6)
				local x , y = self.m_csNode.drum1.drumdi:getPosition()
				self.m_attackerEffect:setPosition(x,y+5)
				self.m_csNode.drum1:addChild(self.m_attackerEffect)
			end
		else
			self.m_csNode.drum1.drumdi:setGrayShader()
			self.m_csNode.drum1.drum:setGrayShader()
			self.m_csNode.drum1.drumstar:setGrayShader()
		end
	end
end

function This:refreshDefenderInfo(isInit)
	local enemyLeftHp = BattlePlayer:getEnemyTroopsLeftHp()
	self.m_csNode.boss.hp:setString(string.format("%d/%d",enemyLeftHp,self.m_defendTotalHp))
	if not isInit then
		self.m_csNode.boss.hpbar:runPercentAction(enemyLeftHp / self.m_defendTotalHp * 100,100)
	else
		self.m_csNode.boss.hpbar:setPercent(enemyLeftHp / self.m_defendTotalHp * 100)
	end
	local defenderInfo = BattlePlayer:getDefenderInfo()
	local attackerInfo = BattlePlayer:getAttackerInfo()
	self.m_csNode.boss.name:setString(defenderInfo.name or "")
	self.m_csNode.boss.level:setString(defenderInfo.level or "")
	self.m_csNode.boss.speedNum:setString(defenderInfo.speed or 0)
	GameUtil:setPlayerIconNodeImage(self.m_csNode.boss.icon, defenderInfo.icon)

	local troopNum = BattlePlayer:getEnemyTroopsCount()
	self.m_csNode.boss.heroNum:setString(troopNum)
	self.m_csNode.boss.troopsNum:setString(math.floor(enemyLeftHp))

	local mode = BattlePlayer:getPlayMode()
	if mode == PlayMode.kWorld then
		WorldMapManager:addHanXianDiSkillInfo(self.m_csNode.buff2,defenderInfo.id)
	end

	local drumData = defenderInfo.drumData
	local attackerDrumData = attackerInfo.drumData
	local isWin = drumData.level > attackerDrumData.level

	if drumData then
		self.m_csNode.drum2.drumstar:setVisible(DrumManager:getNowHaloLevel(drumData.level)>0)
		if drumData.level > 0 then
			local pic = DrumManager:getNowHaloImage(DrumManager:getNowHaloLevel(drumData.level))
			self.m_csNode.drum2.drumstar:setImage(pic)
			if isWin and not self.m_defenderEffect then
				self.m_defenderEffect = GameUtil:createEffect("Itemnb", true)
				self.m_defenderEffect:setScale(0.6)
				local x , y = self.m_csNode.drum2.drumdi:getPosition()
				self.m_defenderEffect:setPosition(x,y+5)
				self.m_csNode.drum2:addChild(self.m_defenderEffect)
			end
		else
			self.m_csNode.drum2.drumdi:setGrayShader()
			self.m_csNode.drum2.drum:setGrayShader()
			self.m_csNode.drum2.drumstar:setGrayShader()
		end
	end
end

function This:onEnter()
	super.onEnter(self)
	self:initTouchEvent()
	BattlePlayer:addObserver(self)
	BattleDataManager:addObserver(self)
	self:setAccelerateFactorFromFile()
end

function This:onExit()
	self.m_MonsterSkeleton = {}
	super.onExit(self)
	BattlePlayer:removeObserver(self)
	BattleDataManager:removeObserver(self)
end

function This:initTouchEvent()
	local teamNode = self.m_csNode.teamMessage
	--local x,y = teamNode:getPosition()

	--local isHide = false
	self.m_csNode.teamMessage.bg:setTouch({
		onEnded = function()
			self:setTeamMessageShow(not self.m_isShowTeam, true)
			-- local isVisible = self.m_csNode.teamMessage.ScrollView:isVisible()
			-- if not isHide then
			-- 	teamNode:runAction(cc.MoveTo(0.3,x-230,y,"SineOut"))
			-- 	isHide = true
			-- 	teamNode.jiantou:setAngle(180)
			-- else
			-- 	teamNode:runAction(cc.MoveTo(0.3,x,y,"SineOut"))
			-- 	isHide = false
			-- 	teamNode.jiantou:setAngle(0)
			-- end
			-- self.m_csNode.teamMessage.ScrollView:setVisible(true)
		end,
		isDynamic = true
	})

	self.m_csNode.acceleratebutton:setTouchEnded(function()
        self:refreshAccelerateFactor(self.m_acceFactor % 2 + 1)
		self:saveAccelerateFactor()
	end)

	self.m_csNode.quitbutton:setTouchEnded(function()
		BattleDataManager:exitBattle(true)
		if g_scene:getName() == "BattleTestSceneView" then
			g_scene:resume()
		end
	end)

	local cost = PrivilegeManager:getPrivilegeNumBy(Privilege.kFightSkipDiamond)
	if kCostIconVisible then
		if cost <= 0 then
			self.m_csNode.cost:setVisible(false)
			self.m_csNode.icon:setVisible(false)
		else
			self.m_csNode.icon:setVisible(BattlePlayer:getPlayMode() == PlayMode.kStage)
			self.m_csNode.cost:setVisible(BattlePlayer:getPlayMode() == PlayMode.kStage)
			self.m_csNode.cost:setString(tostring(cost))
		end
	end
	
	self.m_csNode.skip:setTouchEnded(function()
		if BattlePlayer:getPlayMode() ~= PlayMode.kReport and cost > 0 then
			GamePlayer:costDiamond({
				cost = cost,
				title = Serialize["Battle_BattleUI_2_c"],
				desc = Serialize["Battle_BattleUI_2_c"],
                note = Serialize["Battle_BattleUI_3"],
				okHandler = function()
					BattleDataManager:exitBattle(true)
				end
			})
		else
			BattleDataManager:exitBattle(true)
		end
	end)

	self.m_csNode.buff1.bg:setTouchEnded(function ()
		g_scene:openDialog("Map.HanXianDiSkillPanel",{})
	end)

	self.m_csNode.buff2.bg:setTouchEnded(function ()
		g_scene:openDialog("Map.HanXianDiSkillPanel",{})
	end)
end

function This:refreshAccelerateFactor(factor)
    self.m_acceFactor = factor
    g_scene:setAccelerateFactor(factor)
	print("scene %s accelerate = %d", g_scene._NAME, self.m_acceFactor)
	self.m_csNode.acceleratebutton.accelerate:setString(string.format("x%d",factor))
end

function This:initArmyQueue()
	self:refreshArmyQueue()
	local mode = BattlePlayer:getPlayMode()
	if mode == PlayMode.kWorld or mode == PlayMode.kWarfare or mode == PlayMode.kWarSingleMultiQueue then
		-- self.m_csNode.teamMessage.ScrollView:setVisible(false)
		-- self.m_csNode.teamMessage.bg2:setVisible(false)
		self:setTeamMessageShow(false, false)
	else
		self.m_csNode.teamMessage:setVisible(false)
	end
end

function This:refreshArmyQueue()
	local playMode = BattlePlayer:getPlayMode()
	if playMode ~= PlayMode.kWorld and playMode ~= PlayMode.kWarfare and mode ~= PlayMode.kWarSingleMultiQueue then
		return
	end
	local defenceQueue = BattleDataManager:getDefenceQueue()
	local attackQueue = BattleDataManager:getAttackQueue()

	local isAttacker = nil
	for i , army  in ipairs(attackQueue) do
		if army:isOur() then
			isAttacker = true
			break
		end
	end
	for i , army in ipairs(defenceQueue) do
		if army:isOur() then
			isAttacker = false
			break
		end
	end

	if not isAttacker and BattleDataManager:getDefenderInfo().id == GamePlayer:getPlayerID() then
		isAttacker = false
	end

	if not isAttacker and BattleDataManager:getAttackerInfo().id == GamePlayer:getPlayerID() then
		isAttacker = true
	end

	if isAttacker == nil then
		self.m_csNode.teamMessage.ScrollView:removeAllItems()
		self.m_csNode.teamMessage:setVisible(false)
		return
	end

	local ownerIds = {}
	local roleList = {}

	local viewName = ""
	local itemList = self.m_csNode.teamMessage.ScrollView:getItemList()
	local armyList = isAttacker and attackQueue or defenceQueue

	local roleSource
	if playMode == PlayMode.kWorld then
		roleSource = WorldMapManager
	elseif playMode == PlayMode.kWarfare then
		roleSource = require("Warfare.WarfareManager")
	elseif playMode == PlayMode.kWarSingleMultiQueue then
		--todo
	end

	for i , army in ipairs(armyList) do
		local item = nil
		if itemList[i] then
			item = itemList[i]
			item:setArmy(army,i)
		else
			item = BattleArmyItem:create(army, i, playMode)
			local w , h = item:getContentSize()
			self.m_csNode.teamMessage.ScrollView:pushBackCustomItem(item,w,h)
		end
		local ownerId = army:getOwnerId()
		if not ownerIds[ownerId] then
			local role = roleSource:getRole(ownerId)
			table.insert(roleList,role)
		end
		ownerIds[ownerId] = true
	end

	for i = #itemList , #armyList + 1, -1 do
		self.m_csNode.teamMessage.ScrollView:removeItem(i)
	end
	
	if #roleList > 1 then
		for i , role in ipairs(roleList) do
			viewName = viewName .. "/" .. GameUtil:getSubString(role:getName(),1,2)
		end
	elseif #roleList == 1 then
		viewName = roleList[1]:getName()
	end
	-- self.m_csNode.teamMessage.nameList:setString(viewName)
	-- if isAttacker then
	-- 	self.m_csNode.teamMessage:setPositionX(10)
	-- else
	-- 	self.m_csNode.teamMessage:setPositionX(727)
	-- end
	--self.m_csNode.teamMessage:setVisible(#itemList > 0)
	print("队列可见性:%s",tostring(#itemList > 0))
end

function This:playDrumEffect()
	local attackerInfo = BattlePlayer:getAttackerInfo()
	local defenderInfo = BattlePlayer:getDefenderInfo()
	local attackerDrumData = attackerInfo.drumData
	local defenderDrumData = defenderInfo.drumData
	if attackerDrumData then
		if attackerDrumData.level == 0 and defenderDrumData.level == 0 then
			return
		end
		local drum = self.m_drumEffect
		if not drum then
			drum = GameUtil:createEffect("ZhanGu", false, self.m_csNode, nil, nil, nil, true)
			drum:setToCenter()
			-- drum:setFlippedX(true)
			self.m_csNode:addChild(drum)
			drum:setAnimationListener({
				onAnimationComplete = function (animationName,animationEvent)
					drum:setVisible(false)
					self:runDrumEffect(self.m_csNode.drum1,self.m_csNode.drum1,attackerDrumData)
					self:runDrumEffect(self.m_csNode.drum2,self.m_csNode.drum2,defenderDrumData)
				end
			})
			if attackerDrumData.level > defenderDrumData.level then
				--播放攻击方压制动画
				drum:playAnimation("animation", false)
			elseif attackerDrumData.level < defenderDrumData.level then
				--播放防守方压制动画
				drum:playAnimation("animation2", false)
			else
				--播放平局动画
				drum:playAnimation("animation3", false)
			end
		end
	end
end

function This:runDrumEffect(node1,node2,drumData)
	-- if drumData.level == 0 then
	-- 	return
	-- end
	local nodeX,nodeY = self.m_csNode.texiao:getPosition()
    local effect = nil
	local drumData = drumData
	effect = GameUtil:createEffect("ChuFa5", false, nil, {
        onAnimationComplete = function ()
            effect:removeFromParent()
            local particle = cc.Particle:create("tuowei.plist", self)
            particle:setPosition(nodeX, nodeY)

            local action = nil
			local x,y = node1:getPosition()
			local func = cc.CallFunc(function () 
				node2:setVisible(drumData.level >= 0)
				cc.RemoveAction()
				particle:removeFromParent()
			end)
			action = cc.Sequence(cc.MoveTo(1,x,y),func)
            cc.Audio:playEffect("Recruit_Change.mp3")

			particle:runAction(action)
			self.m_csNode:addChild(particle)
    end})
    effect:setPosition(0,0)
    effect:setVisible(false)
    self.m_csNode:addChild(effect)
end

------------------------------------------------------------------
function This:onTroopsDead(troops)
	if troops:isEnemy() then
		self:refreshDefenderInfo()
	else
		self:refreshAttackerInfo()
	end
end

function This:onTroopsHitted(troops)
	if troops:isEnemy() then
		self:refreshDefenderInfo()
	else
		self:refreshAttackerInfo()
	end
end

function This:onNextRound(roundIndex)
	self.m_csNode.round.text:setString(roundIndex)
end

function This:onStartPlayBattle(roundIndex)
	self.m_csNode.round.text:setString(roundIndex)
end

function This:onTroopsAttack(troops,skill)
	if skill == nil or skill:getId() <= 1 or skill:isPassive() then
		return 
	end
	local node = nil
	if troops:isEnemy() then
		node = cc.Node:createFromCSFile("GameScene/NodeUI/SkillZhanShiRight.csb")
	else
		node = cc.Node:createFromCSFile("GameScene/NodeUI/SkillZhanShiLeft.csb")
	end
	local name = skill:getName()
	for i = 1 , 4 do
		node.char["char"..i]:setString(GameUtil:getChar(name,i))
	end
	if HeroManager:isRedHero(troops:getHeroId()) then
		node.icon.icon:setImage(GameUtil:getRedHeroIcon(troops:getHeroId()))

	else
		node.icon.icon:setImage(GameUtil:getHeroIcon(troops:getHeroId()))
	end

	--node.icon.icon:setImage(GameUtil:getHeroIcon(troops:getHeroId()))
	node:playCSAnimation(nil,false,function()
		node:removeFromParent()
	end)
	node:setCSAnimationSpeed(g_scene:getAccelerateFactor())
	node:setCascadeOpacityEnabled(true)
	node:setActionRunWith(g_scene)
	g_scene:addChild(node)
end

function This:onMonsterAttack(troops,endHandler)
	self.m_MonsterEffectLayer = cc.Node:create()
	local skeletonNode = cc.SkeletonNode:createCharacter("ZhanShen")
    skeletonNode:scheduleUpdate()
    self.m_MonsterEffectLayer:addChild(skeletonNode)

	local duration = 0.2

	local tmpX,tmpY = 350,50

	local moveTo = cc.MoveTo(duration,tmpX,tmpY,"SineInOut")
	local endFunc = cc.CallFunc(function ()
		if endHandler then
			endHandler()
		end
	end)
	skeletonNode:stopAllActions()
	skeletonNode:runAction(cc.Sequence(moveTo,endFunc))
	self.m_MonsterSkeleton[troops:getHeroId()] = skeletonNode
    self:addChild(self.m_MonsterEffectLayer)	
end

function This:onReturnPostion(troops)
	local duration = 0.2

	local tmpX,tmpY = -500,50

	if troops:isEnemy() then
		tmpX,tmpY = 1200,50
	end

	local moveTo = cc.MoveTo(duration,tmpX,tmpY,"SineInOut")
	local finishHandler = cc.CallFunc(function ()
	end)

	local skeletonNode = self.m_MonsterSkeleton[troops:getHeroId()]
	if skeletonNode then
		skeletonNode:stopAllActions()
		skeletonNode:runAction(cc.Sequence(moveTo,finishHandler))
		-- skeletonNode:removeFromParent()
	end	
end

function This:onBattleQueueChanged()
	self:refreshArmyQueue()
end

function This:onLoadBattleFinish()
	self.m_attackTotalHp = BattlePlayer:getFriendTroopsMaxHp()
	self.m_defendTotalHp = BattlePlayer:getEnemyTroopsMaxHp()
	self:refreshAttackerInfo()
	self:refreshDefenderInfo()
	self:refreshArmyQueue()
end

function This:onBuffActiveEnded(troops)
	self:refreshAttackerInfo()
	self:refreshDefenderInfo()
end

function This:setAccelerateFactorFromFile()
    local factorLists = cc.FileUtils:loadLuaFile("accelerate") or {}
	local key = GameUtil:int64ToString(GamePlayer:getPlayerID())
	local value = factorLists[key]

	local valueString={
		[PlayMode.kStage] = "stage",
		[PlayMode.kArena] = "arena",
		[PlayMode.kWarSingle] = "warSingle",
		[PlayMode.kWarfare] = "warfare",
		[PlayMode.kWarSingleMultiQueue] = "warSingle",
	}
	for playerMode,playerStr in pairs(valueString) do
		if BattlePlayer:getPlayMode() == playerMode then
			if playerMode == PlayMode.kWarfare then
				self:refreshAccelerateFactor(kDefaultAccFactorInWarfare)
			else
				if value and type(value) == "number" then
					self:refreshAccelerateFactor(value)
				elseif value and value[playerStr] then
		            self:refreshAccelerateFactor(value[playerStr])
				end 
			end
		end
	end
end

function This:saveAccelerateFactor()
    local factorLists = cc.FileUtils:loadLuaFile("accelerate") or {}
	local key = GameUtil:int64ToString(GamePlayer:getPlayerID())
	if type(factorLists[key]) ~= "table" then
		factorLists[key] = {}
	end
	if BattlePlayer:getPlayMode() == PlayMode.kStage then
		factorLists[key].stage = self.m_acceFactor
	elseif BattlePlayer:getPlayMode() == PlayMode.kArena then
		factorLists[key].arena = self.m_acceFactor
	elseif BattlePlayer:getPlayMode() == PlayMode.kWarSingle then
		factorLists[key].warSingle = self.m_acceFactor
	elseif BattlePlayer:getPlayMode() == PlayMode.kWarSingleMultiQueue then
		factorLists[key].warSingle = self.m_acceFactor
	elseif BattlePlayer:getPlayMode() == PlayMode.kWarfare then
		factorLists[key].warfare = self.m_acceFactor
	end
    cc.FileUtils:saveToLua(factorLists, "accelerate")
end

function This:setTeamMessageShow(isShow, isAni)
	if self.m_isShowTeam == isShow then
		return 
	end

	self.m_isShowTeam = isShow
	local node = self.m_csNode.teamMessage
	node:stopAllActions()

	if isShow then
		if isAni then
			node:runAction(cc.MoveTo(0.3, self.m_startTeamX, self.m_startTeamY,"SineOut"))
		else
			node:setPosition(self.m_startTeamX, self.m_startTeamY)
		end
		node.jiantou:setAngle(0)
	else
		if isAni then
			node:runAction(cc.MoveTo(0.3, self.m_startTeamX-230, self.m_startTeamY, "SineOut"))
		else
			node:setPosition(self.m_startTeamX-230, self.m_startTeamY)
		end
		node.jiantou:setAngle(180)
	end
end


--endregion
