--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)

local AttributeHelper = require("Battle.AttributeHelper")
local SkillSpecial = require("Skill.SkillSpecial")
local SkillCommon = require("Skill.SkillCommon")
local HeroManager = require("Hero.HeroManager")
local HeroConfig = require("Metadatas.HeroConfig")
local BuffConfig = require("Metadatas.BuffConfig")
local SkillConfig = require("Metadatas.SkillConfig")
local SearchTargetHelper = require("Battle.SearchTargetHelper")
local MythicalAnimalsAttributeConfig = require("Metadatas.MythicalAnimalsAttributeConfig")
local MythicalAnimalsStarConfig = require("Metadatas.MythicalAnimalsStarConfig")
local GameBeast = require("GodBeast.GameBeast")

function This:create(...)
	local actor = {}
	setmetatable(actor,self)
	actor:init(...)
	return actor
end

function This:init(info)
	self.m_id = info.id
	self.m_heroId = info.heroId
	if info.id == nil then
		assert(false)
	end
	self.m_index = info.id > 100 and info.id - 100 or info.id
	self.m_isEnemy = info.id > 100

	self.m_y = (self.m_index - 1) % 3 + 1
	local tmp = math.floor((self.m_index - 1) / 3 + 1)
	self.m_x = self.m_isEnemy and tmp or -tmp

	if info.hero then
		self.m_attribute = info.hero:getAttribute():copy()
	else
		self.m_attribute = AttributeHelper:create()
	end

	self.m_gameScene = nil
	self.m_searchHelper = SearchTargetHelper:create()
	self.m_state = TroopsState.STAND

	self.m_buffs = {}
	self:initSkills()
end

function This:initSkills()
	self.m_skills = {}

	local skillId = 0
	if HeroConfig[self.m_heroId] then
		skillId = HeroConfig[self.m_heroId].skill_id
	else
		local godBeast = GameBeast:create(self.m_model:getHeroId())
		local godBeastData = godBeast:getStar()
		if godBeastData then
			skillId = godBeastData:getSkillId()
		else
			skillId = 1002
		end 
	end
	self.m_skills[1] = SkillSpecial:create(skillId,self)
	self.m_skills[0] = SkillCommon:create(0,self)
end


function This:removeSkills()
	for i , skill in pairs(self.m_skills) do
		skill:stop()
	end
	self.m_skills = {}
end

function This:addBuff(buff)
	local id = buff:getId()
	local bf = self.m_buffs[id]
	if bf then
		bf:addRoundCount(buff:getRoundCount())
	else
		self.m_buffs[id] = buff
	end
end

function This:removeBuff(buff)
	local id = buff:getId()
	local buff = self.m_buffs[id]
	if buff then
		if buff:isRunning() then
			buff:stop()
		end
		self.m_buffs[id] = nil
	end
end

function This:removeAllBuffs()
	for i = #self.m_buffs , 1 , -1 do
		local buff = self.m_buffs[i]
		buff:stop()
	end
end

function This:getBuff(id)
	if id then
		return self.m_buffs[id]
	else
		return self.m_buffs
	end
end

function This:getId()
	return self.m_id
end

function This:getIndex()
	return self.m_index
end

function This:getHeroId()
	return self.m_heroId
end

function This:getX()
	return self.m_x
end

function This:getY()
	return self.m_y
end

function This:getXY()
	return self.m_x , self.m_y	
end

function This:getView()
	return nil
end

function This:isEnemy()
	return self.m_isEnemy
end

function This:getAttribute()
	return self.m_attribute
end

function This:setGameScene(gameScene)
	self.m_gameScene = gameScene
	self.m_skills[0]:setGameScene(self.m_gameScene)
	self.m_skills[1]:setGameScene(self.m_gameScene)
end

function This:getGameScene()
	return self.m_gameScene
end

function This:isAlive()
	return self.m_state ~= TroopsState.DEAD
end

function This:getAttr(attrId)
	return self.m_attribute:getAttr(attrId)
end

function This:setAttr(attrId,value)
	local oldValue = self.m_attribute:getAttr(attrId)
	local curValue = self.m_attribute:setAttr(attrId,value)
	if self.m_view then
		self.m_view:onAttrChanged(attrId,oldValue,curValue,curValue - oldValue)
	end
	return value
end

function This:addAttr(attrId,value)
	local oldValue = self.m_attribute:getAttr(attrId)
	local curValue = self.m_attribute:addAttr(attrId,value)
	if self.m_view then
		self.m_view:onAttrChanged(attrId,oldValue,curValue,value)
	end
	return curValue
end

function This:reduceAttr(attrId,value)
	local oldValue = self.m_attribute:getAttr(attrId)
	local curValue = self.m_attribute:reduceAttr(attrId,value)
	if self.m_view then
		self.m_view:onAttrChanged(attrId,oldValue,curValue,-value)
	end
	return curValue
end

function This:crit()
	print(self.m_id, "暴击...")
	self.m_gameScene:onTroopsCrit(self)
end

function This:dodge()
	print(self.m_id, "闪避...")
	self.m_gameScene:onTroopsDodge(self)
end

function This:penetrate()
	print(self.m_id, "识破...")
	self.m_gameScene:onTroopsPenetrate(self)
end

function This:immuneDamage()
	print(self.m_id, "免疫伤害...")
	self.m_gameScene:onTroopsImmuneDamage(self)
end

function This:immuneDebuff()
	print(self.m_id, "免疫debuff...")
	self.m_gameScene:onTroopsImmuneDebuff(self)
end

function This:getSkillTargets(searchId)
	local x , y = self:getXY()
	return self.m_searchHelper:search(searchId,x,y,self.m_gameScene)
end

function This:getSkillBuffs()
	local buffs = {}
	local skillId = self.m_curSkill:getId()
	if skillId > 0 then
		for i , buffId in pairs(SkillConfig[skillId].buffId) do
			if BuffConfig[buffId] == nil then
				assert(false)
			end
			if BuffConfig[buffId] and math.random(1,100) <= BuffConfig[buffId].trigger_ratio * 100 then
				table.insert(buffs,buffId)
			end
		end
	end
	return buffs
end

function This:getBuffTargets(buffId)
	local config = BuffConfig[buffId]
	local searchId = config.target
	if searchId < 0 then
		local skillId = self.m_curSkill:getId()
		searchId = SkillConfig[skillId].damage_target_id
	end

	local x , y = self:getXY()
	local targets = self.m_searchHelper:search(searchId,x,y,self.m_gameScene)
	local t = {}
	for i , target in pairs(targets) do
		if config.is_buff == 1 then
			table.insert(t,target)
		else
			local attribute = target:getAttribute()
			if not attribute:IsImmuneDebuff() and math.random(1,100) <= config.trigger_ratio * 100 then
				table.insert(t,target)
			end
		end
	end
	return t
end

------------------------------------------------------------

function This:getDamageToWall()
	return self.m_attribute:getFinalPhyAttack()
end

function This:checkSkill()
	local count = self.m_gameScene:getEnemyTroopsCount()
	if not self.m_isEnemy and count <= 0 then
		return self.m_skills[0]
	end
	if self.m_skills[1]:isCanActive() then
		return self.m_skills[1]
	end
	if self.m_skills[0]:isCanActive() then
		return self.m_skills[0]
	end
end

function This:enterAttack()
	local roundIndex = self.m_gameScene:getRoundIndex()
	print(string.format("\n第%d次攻击，部队%d开始进入攻击属性如下：",roundIndex, self.m_id))
	self.m_attribute:print()

	self.m_gameScene:onTroopsEnterAttack(self)
	for id , buff in pairs(self.m_buffs) do
		buff:receiverEnterAttack(self)
		print(string.format("附加的BUFF:%d,剩余回合数:%d",id,buff:getRoundCount()))
	end

	if self:getAttr(AttrType.kHp) <= 0 then
		self:back()
		self:dead()
		return
	end
	local skill = self:checkSkill()
	self:useSkill(skill)
end

function This:useSkill(skill)
	self.m_curSkill = skill
	if skill then
		print(string.format("部队%d激活技能%d(0表示普攻)",self.m_id,skill:getId()))
		self.m_gameScene:onTroopsActiveSkill(self,skill)
		skill:active(function ()
			skill:work()
		end)
	else
		self.m_gameScene:onTroopsBackEnd(self)
	end
end

function This:forward(x, y, endHandler)
	self.m_gameScene:onTroopsForwardEnd(self)
	if endHandler then endHandler() end
end

function This:attack(target,hitHandler,attackEndHandler)
	self.m_gameScene:onTroopsAttack(self,target)
	if hitHandler then 
		hitHandler() 
	end
	if attackEndHandler then 
		attackEndHandler() 
	end
	self.m_gameScene:onTroopsAttackEnd(self)
end

function This:getDamage(damage,damageType)
	if damage < 0 then damage = 0 end
	if damageType == DamageType.kPhyDamage then
		self:setAttr(AttrType.kPhyDamageValue,damage)
	elseif damageType == DamageType.kMagicDamage then
		self:setAttr(AttrType.kMagicDamageValue,damage)
	end

	for id , buff in pairs(self.m_buffs) do
		buff:receiverHitted(self)
	end

	local finalDamage = 0
	if damageType == DamageType.kPhyDamage then
		finalDamage = self:getAttr(AttrType.kPhyDamageValue)
		self:setAttr(AttrType.kPhyDamageValue,0)
	elseif damageType == DamageType.kMagicDamage then
		finalDamage = self:getAttr(AttrType.kMagicDamageValue)
		self:setAttr(AttrType.kMagicDamageValue,0)
	end
	return finalDamage
end

function This:checkCrit()
	if self.m_attribute:checkCrit() then
		self:crit()
		return true
	end
	return false
end

function This:checkHitted(damageType)  --检查是否被技能命中
	if self.m_attribute:IsImmuneDamage() then
		self:immuneDamage()
		return false
	else
		if damageType == DamageType.kPhyDamage and self.m_attribute:checkDodge() then
			self:dodge()
			return false
		end

		if damageType == DamageType.kMagicDamage and self.m_attribute:checkPenetrate() then
			self:penetrate()
			return false
		end
	end
	return true
end

function This:getHitted(damage,damageType)
	if not self:checkHitted(damageType) then
		local hp = self:getAttr(AttrType.kHp)
		return false , 0, hp
	end

	for id , buff in pairs(self.m_buffs) do
		buff:receiverHitted(self)
	end

	print(string.format("部队%d受到%f伤害",self.m_id,damage))
	local finalDamage = self:getDamage(damage,damageType)
	local hp = self:getAttr(AttrType.kHp)
	local leftHp = self:reduceAttr(AttrType.kHp,finalDamage)
	print(string.format("部队%d受到的最终伤害为:%f,hp:%f->%f",self.m_id,finalDamage,hp,leftHp))
	
	return true , finalDamage, leftHp
end

function This:hitted(damage,damageType,endHandler)
	local isHitted , finalDamage, leftHp = self:getHitted(damage,damageType)
	if isHitted then
		self.m_gameScene:onTroopsHitted(self,finalDamage,damageType)
	end
	if isHitted and leftHp <= 0 then
		self:dead(finalDamage + leftHp)
	end
	if endHandler then
		endHandler()
	end
end

function This:dead(damage,endHandler)
	self:removeSkills()
	self:removeAllBuffs()
	self.m_state = TroopsState.DEAD
	print(string.format("部队%d死亡",self.m_id))
	self.m_gameScene:onTroopsDead(self)
end

function This:back(isToOrigin,endHandler)
	self.m_gameScene:onTroopsBackEnd(self)
	if endHandler then endHandler() end
end

--endregion
