 --region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local HpFigureChangeView = require("UI.HpFigureChangeView")
local GameUtil = require("Util.GameUtil")
local HeroConfig = require("Metadatas.HeroConfig")
local SkillConfig = require("Metadatas.SkillConfig")
local Timer = require("Timer")
local BattlePlayer = require("Battle.BattlePlayer")
local ArmConfig = require("Metadatas.ArmConfig")
local HeroUtil = require("Hero.HeroUtil")
local EffectTimeConfig = require("Metadatas.EffectTimeConfig")
local BattleSceneConfig = require("Metadatas.BattleSceneConfig")
local BattleSoilderConfig = require("Metadatas.BattleSoilderConfig")

local MythicalAnimalsAttributeConfig = require("Metadatas.MythicalAnimalsAttributeConfig")
local MythicalAnimalsStarConfig = require("Metadatas.MythicalAnimalsStarConfig")
local GameBeast = require("GodBeast.GameBeast")
local EffectConfig = require("Metadatas.EffectConfig")

StateName = 
{
	STAND = "Stand",
    ATTACK = "Attack",
    WALK = "Move",
    DEAD = "Dead",
    HITTED = "Hit"
}

EventName =
{
    FIRE = "Fire"
}

function This:init(troopModel)
	self.m_model = troopModel
	self.m_scene = g_scene
	self:setContentSize(130,130)
	self:setAnchorPoint(0.5,0.5)
	local soilderId = self.m_model:getSoilderId()
	local sceneConfig = self.m_scene:getMapConfig()
	local configId = sceneConfig.soilder[soilderId]

	self.m_config = BattleSoilderConfig[configId]

	self.m_soilders = {}
	self.m_effectNodes = {}
	self.m_isBacked = true
	self.m_timer = Timer:create(self)
end

function This:onEnter()
	super.onEnter(self)
	self:initCharactor()
	self:initBuff()
	self.m_model:setView(self)
end

function This:onExit()
	for i , soilder in pairs(self.m_soilders) do
		soilder:unscheduleUpdate()
	end
	super.onExit(self)
	self.m_model:setView(nil)
end

function This:getModel()
	return self.m_model
end

function This:getConfig()
	return self.m_config
end

function This:getSoilderCount()
	return #self.m_config.stand_format
end

function This:getConfigPosition()
	local sceneConfig = self.m_scene:getMapConfig()
	local id = self.m_model:getId()
	return cc.visible.center.x + sceneConfig.troop[id].x - 480 , cc.visible.center.y + sceneConfig.troop[id].y - 270
end

function This:getSoilderStandFormat(index)
	local config = self.m_config.stand_format[index]
	if self.m_model:isEnemy() then
		return -config.x , config.y 
	end
	return config.x , config.y 
end

function This:getSoilderAttackFormat(index)
	local config = self.m_config.attack_format[index]
	if self.m_model:isEnemy() then
		return -config.x , config.y 
	end
	return config.x, config.y
end

function This:getSoilderCenter()
	local count , cx , cy = 0 , 0 ,0
	for i , soilder in pairs(self.m_soilders) do
		local x , y = soilder:getPosition()
		cx = cx + x
		cy = cy + y
		count = count + 1
	end

	cx = cx / count
	cy = cy / count

	local x , y = self:getPosition()
	return x + cx , y + cy
end

function This:getOrderSoiderByRandom()
	local random = {}
	for i = 1 , #self.m_soilders do
		table.insert(random,i)
	end
	for i = 1 , #random do
		local k = math.random(1,#random)
		local tmp = random[i]
		random[i] = random[k]
		random[k] = tmp
	end
	return random
end

function This:getHittedPoinit()
	local x , y = self:getPosition()
	return x + self.m_config.hitted_point.x , y + self.m_config.hitted_point.y
end

function This:getAttackPoint(attackerView)
	local sceneConfig = self.m_scene:getMapConfig()
	local config = sceneConfig.troop[self.m_model:getId()]
	local x , y = self:getConfigPosition()

	local maxX = 0
	for i , info in ipairs(self.m_config.stand_format) do
		if info.x > maxX then
			maxX = info.x
		end
	end
	maxX = self:getScaleX() * maxX
	if self.m_model:isEnemy() then
		x = x - maxX
	else
		x = x + maxX
	end

	maxX = 0
	local config = nil
	if attackerView then
		config = attackerView:getConfig()
		for i , info in ipairs(config.stand_format) do
			if info.x > maxX then
				maxX = info.x
			end
		end
		maxX = attackerView:getScale() * maxX
		if attackerView:getModel():isEnemy() then
			x = x + maxX
		else 
			x = x - maxX
		end
	end
	if self.m_model:isEnemy() then
		return x - self.m_config.attack_point.x , y + self.m_config.attack_point.y
	else
		return x + self.m_config.attack_point.x , y + self.m_config.attack_point.y
	end
end


function This:initCharactor()
	local sizew , sizeh = self:getContentSize()
	local layer = cc.LayerColor:create(cc.Color(255,255,0,128))
	layer:setContentSize(sizew,sizeh)
	layer:ignoreAnchorPointForPosition(false)
	layer:setAnchorPoint(0.5,0.5)
	layer:setPosition(sizew/2,sizeh/2)
	--self:addChild(layer)

	local x , y = sizew / 2 , sizeh / 2	
	local soilderId = self.m_model:getSoilderId()

	local modelName = ArmConfig[self.m_model:getSoilderConfigId()].arms_model

	self.m_bottomEffectLayer = cc.Node:create()
	self:addChild(self.m_bottomEffectLayer)
	self.m_bottomEffectLayer:setLocalZOrder(-1)

	self.m_soilderNode = cc.Node:create()
	self.m_soilderNode:setPosition(x,y)
	self:addChild(self.m_soilderNode)
	self.m_soilderNode:setCascadeOpacityEnabled(true)

	for i  , info in ipairs(self.m_config.stand_format) do

		local skeletonNode = cc.SkeletonNode:createCharacter(modelName)
		skeletonNode:scheduleUpdate()
		skeletonNode:playAnimation(StateName.STAND, true)
		if i == #self.m_config.stand_format then
			skeletonNode:setAnimationListener(self)
		end
		self.m_soilderNode:addChild(skeletonNode)
		skeletonNode:setScale(info.scale)
		skeletonNode:setLocalZOrder(i)

		if self.m_model:isEnemy() then
			skeletonNode:setFlippedX(true)
			local x , y = self:getSoilderStandFormat(i)
			skeletonNode:setPosition(x , y)
		else
			skeletonNode:setPosition(info.x , info.y)
		end
		skeletonNode:setActionRunWith(self.m_scene)
		self.m_soilders[i] = skeletonNode
	end

	self.m_infoNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroItem.csb")
	self.m_infoNode:setPosition(x,y)
	self.m_infoNode:setScale(0.8)
	self:addChild(self.m_infoNode)

	local hpBarImg = self.m_model:isEnemy() and "GameScene/jindutiao18.png" or "GameScene/jindutiao18lv.png"
	self.m_infoNode.hpbar:setImage(hpBarImg)
	
	local heroId = self.m_model:getHeroId()
    local quality = self.m_model:getHeroQuality()

    if HeroConfig[heroId] then
		self.m_infoNode.heroname:setString(HeroConfig[heroId].name)
	else
		self.m_infoNode.heroname:setString(MythicalAnimalsAttributeConfig[heroId].name)
	end
    self.m_infoNode.heroname:setColorByID("QQ" .. quality)
	self.m_hpBar = self.m_infoNode.hpbar
	self.m_hpText = self.m_infoNode.hp

	self.m_topEffectLayer = cc.Node:create()
	self:addChild(self.m_topEffectLayer)
	self.m_topEffectLayer:setLocalZOrder(100)

	self:refresh(true)
	self:refreshAnger()
end

function This:initBuff()
	local buffList = self.m_model:getBuffs()
	for i , buff in pairs(buffList) do
		local config = buff:getConfig()
		if config.effect_name ~= "" then
			self:addBuffEffect(buff,config.effect_name,config.timetype > 0)
		end
	end
end

function This:refresh(isInit)
	local hp = self.m_model:getAttr(AttrType.kHp)
	local maxHp = self.m_model:getAttr(AttrType.kMaxHp)
	local percent = hp / maxHp * 100
	if percent < 0 then  percent = 0 end
	if not isInit then
		self.m_hpBar:runPercentAction(percent,100)
	else
		self.m_hpBar:setPercent(percent)
	end
	self.m_hpText:setString(math.floor(hp))
	BattlePlayer:onTroopsHitted(self.m_model)
end

function This:refreshAnger(value)

	local skillId = 0

	if HeroConfig[self.m_model:getHeroId()] then
		skillId = HeroConfig[self.m_model:getHeroId()].skill_id
	else
		local godBeast = GameBeast:create(self.m_model:getHeroId())
		local godBeastData = godBeast:getStar()
		if godBeastData then
			skillId = godBeastData:getSkillId()
		else
			skillId = 1002
		end 
	end

	local cost = SkillConfig[skillId].anger_cost

	local anger = self.m_model:getAttr(AttrType.kAngerValue)
	local lastNode = nil
	for i = 1, 4 do
		local isVisible = anger >= i and cost >= i
		local node = self.m_infoNode.energlist["energ"..i]
		node:setVisible(isVisible)

		self.m_infoNode.energlist["energbg"..i]:setVisible(cost >= i)
		if isVisible then
			lastNode = node
		end
	end

	if lastNode and anger > cost then
		local numTxt = self.m_infoNode.energlist.number
		numTxt:setString("X"..anger)
		numTxt:setPositionX(lastNode:getPositionX() + 10)
		numTxt:setVisible(true)
	else
		self.m_infoNode.energlist.number:setVisible(false)
	end
end

function This:showCrit()
	--[[local sprite = cc.Sprite:create("GameScene/crit.png")
	local w , h = self:getContentSize()
	sprite:setPosition(w/2,h + 40)
	sprite:setScale(self.m_config.effectNumberScale)
	self:addChild(sprite)
	sprite:runAction(cc.Sequence(cc.DelayTime(0.5),cc.CallFunc(function()
		sprite:removeFromParent()
	end)))]]
end

function This:showFantan() --反弹
	local sprite = cc.Sprite:create("GameScene/fantan.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 40)
	sprite:runAction(cc.Sequence(cc.DelayTime(0.5),cc.RemoveAction()))
	self:addEffect(sprite,false)	
end

function This:showFenTan() --分摊
	local sprite = cc.Sprite:create("GameScene/fentan.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 40)
	sprite:runAction(cc.Sequence(cc.DelayTime(0.5),cc.RemoveAction()))
	self:addEffect(sprite,false)
end

function This:showLianHuan() --连环
	local sprite = cc.Sprite:create("GameScene/lianhuan.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 20)
	sprite:runAction(cc.Sequence(cc.DelayTime(0.5),cc.RemoveAction()))
	self:addEffect(sprite,false)
end

function This:showFanji()
	local sprite = cc.Sprite:create("GameScene/fanji.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 40)
	sprite:setScale(1.2)
	sprite:runAction(cc.Sequence(cc.DelayTime(1),cc.RemoveAction()))
	self:addEffect(sprite,false)	
end

function This:showLianJi()
	local sprite = cc.Sprite:create("GameScene/lianji.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 40)
	sprite:setScale(1.2)
	sprite:runAction(cc.Sequence(cc.DelayTime(1),cc.RemoveAction()))
	self:addEffect(sprite,false)
end

function This:showFuHuo()
	local sprite = cc.Sprite:create("GameScene/fuhuo.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 40)
	sprite:setScale(1.2)
	sprite:runAction(cc.Sequence(cc.DelayTime(1),cc.RemoveAction()))
	self:addEffect(sprite,false)
end

function This:showImmune()
	local sprite = cc.Sprite:create("GameScene/immune.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 20)
	sprite:runAction(cc.Sequence(cc.DelayTime(0.5),cc.RemoveAction()))
	self:addEffect(sprite,false)
end

function This:showDodge()
	local sprite = cc.Sprite:create("GameScene/dodge.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 20)
	sprite:runAction(cc.Sequence(cc.DelayTime(0.5),cc.RemoveAction()))
	self:addEffect(sprite,false)	
end

function This:showPenetrate()
	local sprite = cc.Sprite:create("GameScene/penetrate.png")
	local w , h = self:getContentSize()
	sprite:setPosition(0,h/2 + 20)
	sprite:runAction(cc.Sequence(cc.DelayTime(0.5),cc.RemoveAction()))
	self:addEffect(sprite,false)	
end

function This:showAddHp(value,effect)
	self:refresh()
	local hpFigure = HpFigureChangeView:create({number = value,figureType = FigureType.kAddHp,effect = effect})
	local w , h = self:getContentSize()
	local x , y = self:getPosition()
	local scale = self:getScale()
	hpFigure:setPosition(x-10, y * scale + h/2)
	self.m_scene:addEffect(hpFigure)
end

function This:showReduceHp(damage,effectType,effect)
	self:refresh()
	local figureType = FigureType.kDamage
	if effectType == EffectType.kCrit then
		figureType = FigureType.kCrit
	end
	local hpFigure = HpFigureChangeView:create({number = -damage,figureType = figureType,effect = effect})
	local w , h = self:getContentSize() 
	local scale = self:getScale()
	local x , y = self:getPosition()
	hpFigure:setPosition(x, y + h * scale /2)
	self.m_scene:addEffect(hpFigure)
end

function This:showKeZhi()
	local w , h = self:getContentSize()
	local sprite = cc.Sprite:create("Fonts/kezhi.png")
	sprite:setPosition(-50, h/2)
	sprite:runAction(cc.Sequence(
		cc.MoveBy(0.2,0,20,"SineInOut"),
		cc.DelayTime(0.3),
		cc.Spawn(cc.FadeOut(0.3),cc.MoveBy(0.3,0,20,"SineInOut")),
		cc.RemoveAction()
	))
	self:addEffect(sprite,false)		
end

function This:addEffect(effect,bCenter,sound)
	local w , h = self:getContentSize()
	if bCenter then
		-- effect:setPosition(w/2, h/2)
		local tmpX,tmpY = 0,0
		if EffectConfig[effect.effectId] then
			tmpX = EffectConfig[effect.effectId].offset[1] or 0
			tmpY = EffectConfig[effect.effectId].offset[2] or 0
		end
		effect:setPosition(w/2 + tmpX, h/2 + tmpY)
	else
		local x ,y = effect:getPosition()
		effect:setPosition(w/2 + x, h/2 + y)
	end
	if sound and sound ~= "" then
		local rate = self.m_scene:getAccelerateFactor()
		if rate > 1 then
			rate = 1 + rate / 5
		end
		cc.Audio:playEffect(sound,false,rate)
	end
	local scale = effect:getScale()
	local config = self.m_scene:getMapConfig()
	effect:setScale(scale * config.effect.scale)
	local zorder = effect:getLocalZOrder()
	if zorder < 0 then
		if not self.m_bottomEffectLayer then
			self.m_bottomEffectLayer = cc.Node:create()
			self:addChild(self.m_bottomEffectLayer)
			self.m_bottomEffectLayer:setLocalZOrder(-1)
		end
		self.m_bottomEffectLayer:addChild(effect)
	else
		if not self.m_topEffectLayer then
			self.m_topEffectLayer = cc.Node:create()
			self:addChild(self.m_topEffectLayer)
			self.m_topEffectLayer:setLocalZOrder(100)
		end
		self.m_topEffectLayer:addChild(effect)
	end
end

function This:removeEffect(effect)
	self.m_topEffectLayer:removeChild(effect)
end

function This:playAnimation(animationName,bLoop,timeScale)
	if self.m_playAnimation ~= animationName then
		for i , soilder in ipairs(self.m_soilders) do
            if soilder and soilder[0] ~= 0 then
			    soilder:playAnimation(animationName,bLoop)
				soilder:setTimeScale(timeScale or 1)
            end
		end
		self.m_playAnimation = animationName
	end
end

function This:onAnimationComplete(animationName,eventName)
	if animationName == StateName.ATTACK then
		if self.m_attackEndHandler then
			self.m_attackEndHandler()
			self.m_attackEndHandler = nil
		end
		self:playAnimation(StateName.STAND,true)
	elseif animationName == StateName.HITTED then
		if self.m_hitEndHandler then
			self.m_hitEndHandler()
			self.m_hitEndHandler = nil
		end
		self:playAnimation(StateName.STAND,true)
	end
end

function This:onAnimationEvent(animationName,eventName)
    if animationName == StateName.ATTACK and eventName == EventName.FIRE then
		self.m_model.m_forwardEndTime = os.clock()
        if self.m_hitHandler then
			self.m_hitHandler()
			self.m_hitHandler = nil
		end
    end
end

function This:skillActived(skill,endHandler)
	if not skill or skill:getId() <= 1  then
		if endHandler then
			endHandler()
		end
		return
	end

	local config = EffectConfig["BaoQi"]
	local skeletonNode = cc.SkeletonNode:createEffect("BaoQi")
	skeletonNode:scheduleUpdate()
	skeletonNode:playAnimation("animation", false)
	skeletonNode:setAnimationListener({
		onAnimationComplete = function (_,animationName,animationEvent)
			skeletonNode:removeFromParent()
			if endHandler then
				endHandler()
			end	
		end
	})
	if config then
		skeletonNode:setPosition(config.offset[1],config.offset[2] )
	end

	self:addEffect(skeletonNode,true,"Skill_Zb.mp3")
end

function This:forward(x,y,endHandler)
	local duration = kCommonAttackTime / 2
	local moveTo = cc.MoveTo(duration,x,y,"SineInOut")
	local endFunc = cc.CallFunc(function ()
		if endHandler then
			endHandler()
		end
	end)
	self:stopAllActions()
	self:runAction(cc.Sequence(moveTo,endFunc))
	self:playAnimation(StateName.WALK)
	cc.Audio:playEffect("Move.mp3",false)
end

function This:back(endHandler)
	local duration = kCommonAttackTime / 2
	local x , y = self:getConfigPosition()
	local moveTo = cc.MoveTo(duration,x,y,"SineInOut")
	local finishHandler = cc.CallFunc(function ()
		if endHandler then 
			endHandler()
		end
		self:playAnimation(StateName.STAND,true)
	end)
	self:stopAllActions()
	self:runAction(cc.Sequence(moveTo,finishHandler))
	self:playAnimation(StateName.WALK,true)
	cc.Audio:playEffect("Walkback.mp3",false)
end

function This:attack(hitHandler,attackEndHandler,targetList)
	if self.m_playAnimation == StateName.HITTED then
		if self.m_hitEndHandler then
			self.m_hitEndHandler()
			self.m_hitEndHandler = nil
		end
	end
	self.m_hitHandler = hitHandler
	self.m_attackEndHandler = attackEndHandler
	self:playAnimation(StateName.ATTACK,false)

	local soilderId = self.m_model:getSoilderId()
	cc.Audio:playEffect(SoilderAttackSound[soilderId])
end

function This:hitted(attacker)
	if self.m_playAnimation ~= StateName.ATTACK then
		self.m_hitEndHandler = hitEndHandler
		self:playAnimation(StateName.HITTED,false)
	elseif hitEndHandler then
		hitEndHandler()
	end

	self:refresh()
	local function setColor(r,g,b)
		for i , view in pairs(self.m_soilders) do
			view:setColor(r,g,b)
		end
	end
	setColor(200,0,0)
	self.m_timer:setTimer(0.2,function()
		setColor(255,255,255)
	end)
end

function This:dead(deadEndHandler)
	self:refresh()
	self:setCascadeOpacityEnabled(true)
	self:runAction(cc.Sequence(cc.DelayTime(0.5), cc.FadeOut(1.0 ), cc.CallFunc(function ()
		if deadEndHandler then
			deadEndHandler()
		end
	end)))
	self:playAnimation(StateName.DEAD,false)
end

function This:addSkillEffect(troops,effectId,hitHandler,endHandler)
	local name = EffectTimeConfig[effectId].effect_name
	local isFlip = not troops:isEnemy()
	local skeletionNode, config = GameUtil:createEffect(name, false, self,nil,isFlip)
	if config and config.isFlip == 1 then
		skeletionNode:setFlippedX(isFlip)
	end

	local fireCount,fireIndex = 0 , 0
	fireCount = skeletionNode.m_skeletonData.events.count

	if fireCount < 1 then
		print(string.format("技能%d没有事件帧Fire",skill:getId()))
		assert(false)
	end

	skeletionNode:setAnimationListener({
		onAnimationComplete = function (_,animationName,animationEvent)
			skeletionNode:removeFromParent()
			if endHandler then
				endHandler()
			end
		end,
		onAnimationEvent = function (_,animationName,animationEvent)
			fireIndex = fireIndex + 1
			if hitHandler then
				hitHandler(fireCount , fireIndex)
			end
			local factor = EffectTimeConfig[effectId].shake_factor
			self.m_scene:shake(10 * factor, 10 * factor, 0.06 * factor)
		end
	})
	local sound = EffectTimeConfig[effectId].sound
	self:addEffect(skeletionNode,true,sound)
end

function This:addBuffEffect(buff,name,isLoop)
	local skeleteNode = GameUtil:createEffect(name,isLoop,nil,nil,self.m_model:isEnemy())
	if isLoop == false then
		skeleteNode:setAnimationListener({
			onAnimationComplete = function (animationName,animationEvent)
				skeleteNode:removeFromParent()
			end
		})
	else
		self.m_effectNodes[buff] = skeleteNode
	end
	self:addEffect(skeleteNode,false)
end

function This:removeBuffEffect(buff)
	print(string.format("移除BUFF %d",buff:getId()))
	if self.m_effectNodes[buff] then
		self.m_effectNodes[buff]:removeFromParent()
		self.m_effectNodes[buff] = nil
	end
end

function This:update(dt)
	self.m_timer:update(dt)
end
--endregion
