-- @Author: Su
-- @Date:   2019-06-29 15:20:32
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-29 15:20:32
-- @File Name: KingBossListItem.lua
-- @Desc: BOSS

local This , super = class(..., "NodeView")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")
local PrincipalBossManager = require("Boss.PrincipalBossManager")
local TerrBuildManager = require("Territory.TerrBuildManager")
local GameUtil = require("Util.GameUtil")


function This:initData(info)
    if info then
        self.m_type = info.tabIndex
        self.m_config = info.config
        self.m_nodeIndex = info.nodeIndex
    end
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingBossListItem.csb")
    self.m_escape = 0
    self.m_csNode = csNode
    self.m_isRequest = false
    self:addChild(csNode)
    self:showLight(1)
    self:refresh()
    self:scheduleUpdate()
end

function This:onEnter()
    super.onEnter(self)
    PrincipalBossManager:addObserver(self)

end

function This:onExit()
    super.onExit(self)
	PrincipalBossManager:removeObserver(self)
end

function This:refresh()
    local node = self.m_csNode
    if self.m_type == 1 then
        node["5"].jiesuo:setString(string.format( Serialize[60498],self.m_config.level ))
    else
        node["5"].jiesuo:setString(string.format( Serialize[60499],self.m_config.level ))
    end
    node.boss.zhezhao:setVisible(self:getIsLocked())
    node.boss.suo:setVisible(self:getIsLocked())
    -- node.jiesuo:setVisible(self:getIsLocked())

    node.boss.icon:setImage(ItemUtil.getIconImage(self.m_config.icon))
    node.boss.biaoqian:setVisible(false)
    node.boss.chongsheng:setVisible(false)
    node.boss.jisha:setVisible(false)
    node.boss.taopao:setVisible(false)

    for i = 1,6 do
        node[tostring(i)]:setVisible(false)
        node[tostring(i)].level:setString(string.format( "Lv.%d",self.m_config.boss_level))
        node[tostring(i)].name:setString(self.m_config.name)
    end

    node["1"]:setVisible(self.m_type == 1 and not self:getIsLocked())
    node["3"]:setVisible(self.m_type == 2 and not self:getIsLocked())
    node["5"]:setVisible(self:getIsLocked())
    if self.m_type == 1 then
        local wholeBossList = PrincipalBossManager:getWholeBossList()
        local leftChllengeTime =self.m_config.challenge_time -wholeBossList[self.m_config.id].challenge_times
        node["1"].jindutiao.jindutiao:setPercent(leftChllengeTime/self.m_config.challenge_time *100)
        node["1"].jindutiao.num:setString(string.format( "%d/%d",leftChllengeTime,self.m_config.challenge_time ))

        node["2"].time:setString(GameUtil:formatTime(Serialize[60500],0))

        node["1"]:setVisible(leftChllengeTime > 0 and not self:getIsLocked())
        node["2"]:setVisible(self.m_type == 1 and leftChllengeTime == 0 and not self:getIsLocked())
        node.boss.biaoqian:setVisible(leftChllengeTime == 0)
        node.boss.chongsheng:setVisible(leftChllengeTime == 0)
    else

        local exclusiveBossList = PrincipalBossManager:getExclusiveBossList()
        local bossInfo = exclusiveBossList[self.m_config.id]
        local roleInfo = bossInfo.roleinfo
        node["3"]:setVisible(roleInfo.role_id == 0 and not self:getIsLocked() and not bossInfo.dead ) 
        node["4"]:setVisible(roleInfo.role_id ~= 0 and not self:getIsLocked() and not bossInfo.dead ) 
        node["6"]:setVisible(bossInfo.dead and not self:getIsLocked())
        -- formatDate
        node["6"].chuxian:setString(GameUtil:formatDate(Serialize[60501],bossInfo.lift_time))

        node["3"].jindutiao.jindutiao:setPercent(bossInfo.hp)
        node["3"].jindutiao.num:setString(Serialize[60502]..bossInfo.hp.."%")
        if roleInfo.role_id ~= 0 then
            node["4"].player:setString(string.format( Serialize[60503],roleInfo.role_name))
        end

    end

    -- node["4"]:setVisible(self.m_type == 2)
    -- node["4"].player:setString(string.format( Serialize[60504], ))


end

function This:refreshTime()
    local node = self.m_csNode

    if self.m_type == 1 then
        local wholeBossList = PrincipalBossManager:getWholeBossList()
        local deadTime = wholeBossList[self.m_config.id].deadtime
        local leftTime = deadTime + self.m_config.resurrection_time -GamePlayer:getServerTime()
        if leftTime < 0 then
            leftTime =0
            -- self:unscheduleUpdate()
            -- if deadTime > 0 then
            --     -- PrincipalBossManager:onGetBossListRequest()

            -- end
        end
        node["2"].time:setString(GameUtil:formatTime(Serialize[60500],leftTime))
    else
        local exclusiveBossList = PrincipalBossManager:getExclusiveBossList()
        local bossInfo = exclusiveBossList[self.m_config.id]
        local roleInfo = bossInfo.roleinfo
        if roleInfo.role_id ~= 0 and not self:getIsLocked() and not bossInfo.dead then

            local leftTime = roleInfo.value + self.m_config.occupy_time -GamePlayer:getServerTime()
            if leftTime <= 0 then
                leftTime = 0
                if not self.m_isRequest then
                    self.m_isRequest = true
                    PrincipalBossManager:onGetBossListRequest()
                end
            end
            node["4"].time:setString(GameUtil:formatTime(Serialize[60505],leftTime))
        end
    end
end

function This:initTouchEvent()
    self.m_csNode.bg:setTouch({
        onEnded = function()
            if self.m_clickHandler then
                self.m_clickHandler()
            end
        end,
        isScale = false, 
        isDynamic = true, 
        isContainer  = true,
    })

end

function This:setClickHandler(handler)
	self.m_clickHandler = handler
end

function This:getNodeIndex()
    return self.m_nodeIndex
end

function This:showLight(index)
    self.m_csNode.xuanzhong:setVisible(index==self.m_nodeIndex)
end

function This:getSize()
    local x,y=self.m_csNode.bg:getContentSize()
    return x,y
end

function This:getIsLocked()
    if self.m_type == 1 then
        return GamePlayer:getLevel() < self.m_config.level
    else
        return TerrBuildManager:getBuildLevel(TerrBuildType.kGuanfu) < self.m_config.level
    end
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshTime()
	end
end

function This:updateInfo(info)
    if info then
        self.m_type = info.tabIndex
        self.m_config = info.config
        self.m_nodeIndex = info.nodeIndex
        self.m_targetId = info.lastId or 1
    end
    self:showLight(self.m_targetId)
    self:refresh()
end