-- @Author: Su
-- @Date:   2019-06-29 15:30:53
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-29 15:30:53
-- @File Name: PrincipalBossManager.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local BattleDataManager = require("Battle.BattleDataManager")
local WholeNationBossBoxConfig = require("Metadatas.WholeNationBossBoxConfig")
local MenuScene = require("Home.MenuScene")

function This:init()
	super.init(self)
	-- self.m_bossList = nil
	self.m_selfWholeBossChallengeTime = 0
	self.m_wholeBossFightAllTimes = 0
	self.m_WholeBossBuyTime = 0
	self.m_bossRankInfo = {}
	self.m_WholeBossList = {}
	self.m_exclusiveBossList = {}
	self.m_rankData = {}
	self.m_rewardHasReceiveList = {}
	self.m_boxList = {}
	for i,boxInfo in ipairs(WholeNationBossBoxConfig) do
		self.m_boxList[i] = boxInfo
	end
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 12407, self.onGetSelfBossInfoReply)
	EventDispatcher:addListener(self, 12408, self.onGetBossListReply)
	EventDispatcher:addListener(self, 12409, self.onBuyFightTimeReply)
	EventDispatcher:addListener(self, 12410, self.onFightBossReply)
	EventDispatcher:addListener(self, 12411, self.onGetBossRankReply)
	EventDispatcher:addListener(self, 12412, self.onFightExclusiveBossReply)
	EventDispatcher:addListener(self, 12413, self.onBuyChallegeOrderReply)
	EventDispatcher:addListener(self, 12414, self.onReceiveBoxReply)

	-- EventDispatcher:addListener(self, "day_passed", self.onDayPassNotify)
end

function This:removeListener()
	EventDispatcher:removeListener(self, 12407)
	EventDispatcher:removeListener(self, 12408)
	EventDispatcher:removeListener(self, 12409)
	EventDispatcher:removeListener(self, 12410)
	EventDispatcher:removeListener(self, 12411)
	EventDispatcher:removeListener(self, 12412)
	EventDispatcher:removeListener(self, 12413)
	EventDispatcher:removeListener(self, 12414)
	-- EventDispatcher:removeListener(self, "day_passed")
end

function This:getBoxList()
	table.sort(self.m_boxList,function (a,b)
		if self:isReceive(a.id) ~=self:isReceive(b.id) then
			return not self:isReceive(a.id)
		end
		if a.id ~= b.id then
			return a.id < b.id
		end
	end )
	return self.m_boxList
end

function This:getLastState()
	return self.m_lastIndex or 0,self.m_lastBossId or 0
end

function This:setLastState(index,bossIndex)
	self.m_lastIndex = index
	self.m_lastBossId = bossIndex
end

function This:getSelfRankInfo(id)
	return self.m_rankData[id]
end

function This:getChallengeTime()
	return self.m_selfWholeBossChallengeTime
end

function This:getRankList(id)
	return self.m_bossRankInfo[id]
end

function This:getWholeBossList()
	return self.m_WholeBossList
end

function This:getExclusiveBossList()
	return self.m_exclusiveBossList
end

function This:getLastRefreshTime()
	return self.m_refreshTime
end

function This:onBattleSkip(result)
	if self.m_battleInfo then
		local isBeat
		if self.m_battleInfo.boss_info then
			isBeat = self.m_battleInfo.boss_info.roleinfo.role_id
		end
		if isBeat and isBeat > 0 and result == BattleResult.kDefenderVictory and isBeat ~= GamePlayer:getPlayerID() then
			self:onBattleFailed(result)
			BattleDataManager:clear()
			return
		end
	end
	self:onBattleVictory()
	BattleDataManager:clear()
end


function This:onBattleVictory()
	if self.m_battleInfo then
		BattleDataManager:removeObserver(self)
		-- g_scene:openDialog("WorldBoss.WorldBossOverPanel", self.m_battleInfo)
		g_scene:openDialog("Stage.StageVictoryPanel",{
        -- stageId = self.m_robberInBattle:getBattleId(),
        isRobber = true,
		dropInfo = self.m_battleInfo.raward_list,
		isBoss = true,
		callback = function ()
			MenuScene:addDelayDialog({name ="Boss.PrincipalBossPanel" })
		end,
    })
    BattleDataManager:removeObserver(self)

		self.m_battleInfo = nil
	end
end

function This:getHeroCardList()
	return self.m_heroCardList
end

function This:onBattleFailed(result)
	if self.m_battleInfo then
		local isBeat
		if self.m_battleInfo.boss_info then
			isBeat= self.m_battleInfo.boss_info.roleinfo.role_id
		end
		if isBeat and isBeat > 0 then
			g_scene:openDialog("Stage.StageFailedPanel", {
				isRobber = true,
				callback = function ()
					BattleDataManager:removeObserver(self)
					MenuScene:addDelayDialog({name ="Boss.PrincipalBossPanel" })
				end
			})
		else
			self:onBattleVictory()
		end
	else
		self:onBattleVictory()
	end
end

function This:isReceive(id)
	if self.m_rewardHasReceiveList then
		for _,boxId in pairs(self.m_rewardHasReceiveList) do
			if id==boxId then
				return true
			end
		end
	end
	return false
end

function This:isCanReceive(id)
	if not self.m_wholeBossFightAllTimes then
		self.m_wholeBossFightAllTimes = 0
	end
	return self.m_wholeBossFightAllTimes >= WholeNationBossBoxConfig[id].times
end

function This:getWholeBossChallegeTime()
	return self.m_wholeBossFightAllTimes or 0
end

function This:getExclusiveBossChallegeTime()
	return self.m_exclusiveBuyTime or 0
end

function This:getLestReceiveTimes()
	for i, boxData in ipairs(WholeNationBossBoxConfig) do
		if not self:isReceive(boxData.id) or i ==#WholeNationBossBoxConfig then
			return boxData.times
		end
	end
	return 0
end



---------------------------------------------------------------------------------
---
function This:onGetSelfBossInfoRequest()
	Socket:sendAndLock(12407,{})
end

function This:onGetSelfBossInfoReply(msg)
    if msg.err_code == 0 then
		self.m_selfWholeBossChallengeTime = msg.challenge_times
		self.m_WholeBossBuyTime = msg.buy_time
		self.m_exclusiveBuyTime = msg.ex_boss_buy_times
		self.m_heroCardList = msg.card_list
		self.m_refreshTime = msg.refresh_time
		self.m_wholeBossFightAllTimes=msg.all_times
		self.m_rewardHasReceiveList = msg.reward_list
        self:notifyObservers("onGetSelfWholeBossInfo")

	else
        g_scene:handleErrorCode(msg.err_code)

	end
end

function This:onGetBossListRequest()
	Socket:sendAndLock(12408,{})
end

function This:onGetBossListReply(msg)
    if msg.err_code == 0 then
		self.m_WholeBossList = msg.wn_boss_list
		self.m_exclusiveBossList = msg.ex_boss_list
		for _,bossInfo in pairs(msg.wn_boss_list) do
			self:onGetBossRankRequest(bossInfo.boss_id)
		end
        self:notifyObservers("onGetWholeBossListInfo")
	else
        g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onBuyFightTimeRequest()
	Socket:sendAndLock(12409,{})
end

function This:onBuyFightTimeReply(msg)
	if msg.err_code == 0 then
		self.m_selfWholeBossChallengeTime = msg.challenge_times
		self.m_WholeBossBuyTime = msg.buy_times
		self:notifyObservers("onBuyWholeBossInfo")
	else
        g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onFightBossRequest(bossId,heroList)
	self:setLastState(1,bossId)
	Socket:sendAndLock(12410,{id=bossId,card_list = heroList})
end

function This:onFightBossReply(msg)
	if msg.err_code == 0 then
		self.m_refreshTime=msg.refresh_time
		self.m_WholeBossList[msg.id].role_challenge_times = self.m_WholeBossList[msg.id].role_challenge_times +1
		self:onGetBossRankRequest(self.m_lastBossId)
		self.m_battleInfo = msg
		cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {
			msg = msg,
			bgId = 2,
			mode = PlayMode.kReport,
		})
		BattleDataManager:addObserver(self)
		self:notifyObservers("onChallengeWholeBoss")

	else
        g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetBossRankRequest(bossId)
	Socket:sendAndLock(12411,{id=bossId})
end

function This:onGetBossRankReply(msg)
	if msg.err_code == 0 then
		self.m_rankData[msg.id] = {}
		self.m_bossRankInfo[msg.id] = msg.rankinfo
		table.sort(self.m_bossRankInfo[msg.id],function(a,b)
			if a.value ~= b.value then
				return a.value > b.value
			end
		end)
		for i=1 , #self.m_bossRankInfo[msg.id] do
			if self.m_bossRankInfo[msg.id][i].role_id == GamePlayer:getPlayerID() then
				self.m_rankData[msg.id] = self.m_bossRankInfo[msg.id][i]
			end
			self.m_bossRankInfo[msg.id][i].rank = i
		end
		self:notifyObservers("onGetBossRankInfo")

	else
        g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onFightExclusiveBossRequest(bossId,heroList)
	self:setLastState(2,bossId)
	Socket:sendAndLock(12412,{id=bossId,card_list = heroList})
end

function This:onFightExclusiveBossReply(msg)
	if msg.err_code == 0 then
		-- self.m_refreshTime=msg.refresh_time
		self.m_exclusiveBossList[msg.boss_info.boss_id] = msg.boss_info
		self.m_battleInfo = msg
		cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {
			msg = msg,
			bgId = 2,
			mode = PlayMode.kReport,
		})
		BattleDataManager:addObserver(self)
		self:notifyObservers("onChallengeExclusiveBoss")

	else
        g_scene:handleErrorCode(msg.err_code)
	end
end 

function This:onBuyChallegeOrderRequest(itemId,itemNum)
	Socket:sendAndLock(12413,{id=itemId,num = itemNum})

end

function This:onBuyChallegeOrderReply(msg)
	if msg.err_code == 0 then
		self.m_exclusiveBuyTime = msg.ex_boss_buy_times
		self:notifyObservers("onBuyExBossFight")
	else
        g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onReceiveBoxRequest(id)
	Socket:sendAndLock(12414,{id=id})
	
end

function This:onReceiveBoxReply(msg)
	if msg.err_code == 0 then
		self.m_rewardHasReceiveList = msg.reward_list
		self:notifyObservers("onReceiveWholeBossBox")
		local reward = WholeNationBossBoxConfig[msg.id].reward
		g_scene:showReward(reward)
	else
        g_scene:handleErrorCode(msg.err_code)

	end
end