-- @Author: Su
-- @Date:   2019-06-29 15:20:32
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-29 15:20:32
-- @File Name: PrincipalBossPanel.lua
-- @Desc: BOSS

local This, super = class(...,"Dialog")
local PrincipalBossManager = require("Boss.PrincipalBossManager")
local PrincipalBoss = require("Boss.PrincipalBoss")
local WholeNationBossConfig = require("Metadatas.WholeNationBossConfig")
local ExclusiveBossConfig =require("Metadatas.ExclusiveBossConfig")
local KingBossListItem = require("Boss.KingBossListItem")
local ItemNode = require("UI.ItemNode")
local ItemUtil = require("Bag.ItemUtil")
local DropConfig = require("Metadatas.DropConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local DepotManager = require("Depot.DepotManager")


local BossName = {
    [1] = "kAllBoss",
    [2] = "kExclusiveBoss",
}

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingBoss.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    self.m_bossListNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingBossList.csb")
    self:addChild(self.m_bossListNode)
    self.m_bossListNode:setToCenter()

    self.m_bossAllNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingBossAll.csb")
    self:addChild(self.m_bossAllNode)
    self.m_bossAllNode:setToCenter()

    self.m_bossDetailNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingBossMe.csb")
    self:addChild(self.m_bossDetailNode)
    self.m_bossDetailNode:setToCenter()

    self.m_bossDetailNode:setVisible(false)
    self.m_tabIndex = 1
    self.m_escape = 0               
    self.m_crusadeId = ArgConfig.exclusive_boss_item_id.arg[1]
    self:refreshBoxEffect()
    self.m_csNode.tep2.light:setVisible(false)
    self:addTabButton(BossName[1],self.m_csNode.tep1,function ()
        self.m_tabIndex = 1
        self:refreshBossList()
        -- self:openTabNode("Boss.AnimalsTurnTableNode",self.m_tabIndex)        
    end)

    self:addTabButton(BossName[2],self.m_csNode.tep2,function ()
        self.m_tabIndex = 2
        self:refreshBossList()
        -- self:openTabNode("Boss.AnimalsTurnTableNode",self.m_tabIndex)        
    end)
end

function This:onEnter()
    super.onEnter(self)
    PrincipalBossManager:addObserver(self)
    PrincipalBossManager:onGetBossListRequest()
    PrincipalBossManager:onGetSelfBossInfoRequest()
end

function This:onExit()
    super.onExit(self)
	PrincipalBossManager:removeObserver(self)
    self:unscheduleUpdate()
    self.m_bossItemList = {}
end

function This:onGetBossRankInfo()

end

function This:refresh()
    local index,bossId = PrincipalBossManager:getLastState()
    if index == 0 then
        self:selectTabButton(BossName[self.m_tabIndex])
    else
        local bossConfig
        if index==1 then
            bossConfig = WholeNationBossConfig[bossId]
        else
            bossConfig =ExclusiveBossConfig[bossId]
        end
        self.m_tabIndex = index
        self:selectTabButton(BossName[self.m_tabIndex])
        for index,node in ipairs(self.m_bossItemList) do
            node:showLight(bossId)
        end
        self:refreshBossInfo(bossConfig)
        local pos = self.m_bossItemList[bossId]:getPositionY()
        self.m_bossListNode.ScrollView:jumpToAtInnerPosY(pos+40)

    end
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self.m_bossAllNode.look:setTouchEnded(function()
        g_scene:openDialog("Boss.PrincipalBossRankPanel",{id =self.m_selectBossId })
    end)

    self.m_bossAllNode.jiahao:setTouchEnded(function()
        local cost = ArgConfig.whole_nation_boss_times_cost.arg[1]
        local panel = g_scene:openDialog("UI.CostConfirmPanel",{cost = cost,desc = Serialize[60512],})
        panel:setOkHandler(function ()
            if PrincipalBossManager:getChallengeTime() > 0 then
                g_scene:showTips(Serialize[60513])
                return
            end
			PrincipalBossManager:onBuyFightTimeRequest()
		end)
    end)

    self.m_bossAllNode.button:setTouchEnded(function()
        self.m_challengeCoroutine = coroutine.create(function ()
            self:challenge()
            self.m_challengeCoroutine = nil
        end)
        coroutine.resume(self.m_challengeCoroutine)
    end)

    self.m_bossAllNode.boxLock:setTouchEnded(function ()
        g_scene:openDialog("Boss.PrincipalRewardPanel",{})
    end)

    self.m_bossAllNode.wenhao:setTouchEnded(function ()
        g_scene:openDialog("UI.HelpTips",{text = Serialize[60514],node=self.m_bossAllNode.wenhao})
    end)

    self.m_bossDetailNode.wenhao:setTouchEnded(function ()
        g_scene:openDialog("UI.HelpTips",{text = Serialize[60515],node=self.m_bossDetailNode.wenhao})
    end)

    self.m_bossDetailNode.jia:setTouchEnded(function ()
        g_scene:openDialog("Boss.ItemBuyPanel",{})
    end)

    self.m_bossDetailNode.button:setTouchEnded(function ()
        self.m_challengeCoroutine = coroutine.create(function ()
            self:challenge()
            self.m_challengeCoroutine = nil
        end)
        coroutine.resume(self.m_challengeCoroutine)
    end)
end

function This:refreshBossList()
    self.m_bossAllNode:setVisible(self.m_tabIndex == 1)
    self.m_bossDetailNode:setVisible(self.m_tabIndex == 2)
    if not self.m_bossItemList then
        self.m_bossItemList = {}
    end
    local scrollView = self.m_bossListNode.ScrollView
    --scrollView:removeAllItems()
    local config
    if self.m_tabIndex == 1 then
        config = WholeNationBossConfig
        self:refreshBoxEffect()
    else
        config = ExclusiveBossConfig
    end
    if #self.m_bossItemList > #config then
        for i=#self.m_bossItemList,#config+1,-1 do
            scrollView:removeItem(i)
            self.m_bossItemList[i]=nil
        end
    end
    for i,bossConfig in ipairs(config) do
        local item = self.m_bossItemList[i]
        if not item then
            item = KingBossListItem:create({tabIndex = self.m_tabIndex,config = bossConfig,nodeIndex = i})
            item:setClickHandler(function ()
                if item:getIsLocked() then
                    return
                end
                for index,node in ipairs(self.m_bossItemList) do
                    node:showLight(i)
                end
                self:refreshBossInfo(bossConfig)
            end)
            -- item
            local x,y = item:getSize()
            item:setAnchorPoint(0,0)
            scrollView:pushBackCustomItem(item,x,y-5,0.5)
            self.m_bossItemList[i] = item
        else
            item:updateInfo({tabIndex = self.m_tabIndex,config = bossConfig,nodeIndex = i})
            item:setClickHandler(function ()
                if item:getIsLocked() then
                    return
                end
                for index,node in ipairs(self.m_bossItemList) do
                    node:showLight(i)
                end
                self:refreshBossInfo(bossConfig)
            end)
        end
    end

    self:refreshBossInfo(config[1])
end

function This:refreshBossInfo(config)
    local node,leftChllengeTime
    self.m_selectBossId=config.id
    self.m_selectBoss =PrincipalBoss:create(config)
    self.m_bossAllNode.bosstu:setImage(string.format("Hero/%d.png",config.icon))
    self.m_bossDetailNode.bosstu:setImage(string.format("Hero/%d.png",config.icon))
    local bossInfo = PrincipalBossManager:getRankList(self.m_selectBossId)
    local wholeBossList = PrincipalBossManager:getWholeBossList()
    local exclusiveBossList = PrincipalBossManager:getExclusiveBossList()
    if self.m_tabIndex == 2 then
        node = self.m_bossDetailNode
        local exBossInfo = exclusiveBossList[self.m_selectBossId]
        node.jindutiao.jindutiao:setPercent(exBossInfo.hp)
        node.jindutiao.num:setString(Serialize[60516]..tostring(exBossInfo.hp).."%")
        node.time:setVisible(exBossInfo.roleinfo.value ~= 0)
        if exBossInfo.roleinfo.value ~= 0 then
            node.playername:setString(string.format( Serialize[60503],exBossInfo.roleinfo.role_name))
        else
            node.playername:setString(Serialize[60517])

        end
        node.zhengtaoling:setString( string.format("%d/%d",DepotManager:getItemCount(self.m_crusadeId),config.item_num[2]))
        node.daoju:setImage(ItemUtil.getIconImage(self.m_crusadeId))
    else
        node = self.m_bossAllNode
        local selfBossInfo = PrincipalBossManager:getSelfRankInfo(self.m_selectBossId)
        local reward = DropConfig[config.first_drop].drop_info
        for i = 5,4+#reward do
            node["item"..i]:removeAllChildren()
            local item = ItemNode:create(reward[i-4][1],reward[i-4][2])
            item:setAnchorPoint(0.5,0.5)
            item:setScale(0.7)
            node["item"..i]:addChild(item)
        end
        leftChllengeTime =config.challenge_time -wholeBossList[self.m_selectBossId].challenge_times
        node.num:setString(string.format("%d/2",2-wholeBossList[self.m_selectBossId].role_challenge_times))

        node.tiaozhancishu:setString(string.format( Serialize[60518],PrincipalBossManager:getChallengeTime(),10))
        node.boxnumg:setVisible(true)
        node.boxnumg:setString(string.format("%d/%d",PrincipalBossManager:getWholeBossChallegeTime(),PrincipalBossManager:getLestReceiveTimes()))
        if selfBossInfo and selfBossInfo.rank then
            node.paiming:setString(string.format( Serialize[60519],selfBossInfo.rank ))
        else
            node.paiming:setString(Serialize[60520])
        end
    end
    node.lv:setString(string.format( "Lv.%d",config.boss_level))
    node.bossname:setString(config.name)

    local reward
    if config.challenge_reward then
        reward = config.challenge_reward
    elseif config.challenge_drop then
        reward = DropConfig[config.challenge_drop].drop_info
    else
        reward = DropConfig[config.drop_id].drop_info
    end
    for i = 1,#reward do
        node["item"..i]:removeAllChildren()
        local item = ItemNode:create(reward[i][1],reward[i][2])
        item:setAnchorPoint(0.5,0.5)
        item:setScale(0.7)
        node["item"..i]:addChild(item)
    end

end

function This:refreshTime()
    local time = PrincipalBossManager:getLastRefreshTime()
    local intervalTime = ArgConfig.whole_nation_boss_times_recovery_interval.arg[1]
    if self.m_tabIndex == 1 then
        local leftTime = time+intervalTime - GamePlayer:getServerTime()
        if leftTime then
            if leftTime <= 0.1 then
                leftTime = 0
                if PrincipalBossManager:getChallengeTime() < 10 then
                    PrincipalBossManager:onGetSelfBossInfoRequest()
                    -- self:unscheduleUpdate()
                end
            end
            if PrincipalBossManager:getChallengeTime() == 10 then
                self.m_bossAllNode.time:setString(Serialize[60521])
            else
                self.m_bossAllNode.time:setString(GameUtil:formatTime(Serialize[60522],leftTime))
            end
        end
    else
        local exclusiveBossList = PrincipalBossManager:getExclusiveBossList()
        local bossInfo = exclusiveBossList[self.m_selectBossId]
        local roleInfo = bossInfo.roleinfo
        if bossInfo.dead then
            self.m_bossDetailNode.taopaotime:setString(GameUtil:formatTime(Serialize[60523],0))
        else
            local leftTime = bossInfo.lift_time + self.m_selectBoss:getConfig().continued_time - GamePlayer:getServerTime()
            if leftTime <= 0 then
                leftTime = 0
            end
            self.m_bossDetailNode.taopaotime:setString(GameUtil:formatTime(Serialize[60523],leftTime))
            if roleInfo.role_id ~= 0 and not bossInfo.dead then
                local time =roleInfo.value + self.m_selectBoss:getConfig().occupy_time -GamePlayer:getServerTime()
                if time <= 0 then
                    time = 0
                end
                self.m_bossDetailNode.time:setString(GameUtil:formatTime(Serialize[60524],time))
            end
        end
    end
end

function This:refreshBoxEffect()
    local node = self.m_bossAllNode.boxLock
    local boxList = PrincipalBossManager:getBoxList()
    local isAdd = false
    for i,boxInfo in pairs(boxList) do
        if PrincipalBossManager:isCanReceive(boxInfo.id) and not PrincipalBossManager:isReceive(boxInfo.id) then
            isAdd = true
        end
    end
    GameUtil:createBoxEffect(isAdd,node,3)
    self.m_bossAllNode.boxnumg:setString(string.format("%d/%d",PrincipalBossManager:getWholeBossChallegeTime(),PrincipalBossManager:getLestReceiveTimes()))
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshTime()
	end
end

function This:challenge()
        local bossInfo = PrincipalBossManager:getRankList(self.m_selectBossId)
        if bossInfo.deadtime and bossInfo.deadtime ~= 0 then
            g_scene:showTips(Serialize[60525])
            return
        end
        if PrincipalBossManager:getChallengeTime() == 0 and self.m_tabIndex == 1 then
            g_scene:showTips(Serialize[60526])
            return
        end
        local cardList
        g_scene:openDialog("Boss.PrincipalBossEmbattlePanel",{
            battleHandler = function(param)
                coroutine.resume(self.m_challengeCoroutine, param)
            end,
            stage = self.m_selectBoss
        })
        --local panel = self:openDialog("Boss.PrincipalBossEmbattlePanel",{
		--	battleHandler = function(param)
        --    	coroutine.resume(self.m_challengeCoroutine, param)
		--	end,
		--	stage = self.m_selectBossConfig
		--})
        cardList = coroutine.yield()
        if cardList then
            if self.m_tabIndex == 1 then
                PrincipalBossManager:onFightBossRequest(self.m_selectBossId,cardList)
            else
                PrincipalBossManager:onFightExclusiveBossRequest(self.m_selectBossId,cardList)
            end
        end
end

function This:onGetSelfWholeBossInfo()
    self.m_bossAllNode.tiaozhancishu:setString(string.format( Serialize[60518],PrincipalBossManager:getChallengeTime(),10))
    self:scheduleUpdate()
    self:refreshBossList()
    self:refresh()
end

function This:onGetWholeBossListInfo()
    self:selectTabButton(BossName[self.m_tabIndex])
    self:refresh()
end

function This:onBuyExBossFight()
    self.m_bossDetailNode.zhengtaoling:setString( string.format("%d/%d",DepotManager:getItemCount(self.m_crusadeId),self.m_selectBoss:getConfig().item_num[2]))
end

function This:onChallengeWholeBoss()
    self:refresh()
    self:refreshBoxEffect()
end

function This:onChallengeExclusiveBoss()
    -- self:refresh()
    self:refreshBossList()
end

function This:onReceiveWholeBossBox()
    self:refreshBoxEffect()    
end

function This:onBuyWholeBossInfo()
    -- self:refresh()
    self.m_bossAllNode.tiaozhancishu:setString(string.format( Serialize[60518],PrincipalBossManager:getChallengeTime(),10))
end