--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local Chat = require("Chat.Chat")
local UpdateList = require("Base.UpdateList")
local GamePlayer = require("Player.GamePlayer")
local FriendManager = require("Friend.FriendManager")
local GameUtil = require("Util.GameUtil")
local MatchManager = require("Match.MatchManager")

ChatChannelType = 
{
	kWorld = 1,  --世界
	kPrivate = 2, --私人
	kCurMap = 3, --当前
	kSystem = 4, --系统
	kServer = 5, --区
	kFamily = 6, --军团
	kRescue = 7, --勤王战
	
}

ChatType = 
{
	kNormal = 0,
	kAddFriend = 1,
	kAgreeAdd = 2,  
	kRefuseAdd = 3,
	kShareReport = 4, --分享战报
	kApplyGroup = 5, --邀请加入军团
	kApplyMatch = 6, --邀请加入组队匹配
	kVoice = 7, --语音
	kApplyTeamWarfare = 8, --邀请加入组队战役
	kApplyTower = 9, --
}

ChatChannelName = 
{
	[ChatChannelType.kWorld] = Serialize["chat_ChatManager_12"],
	[ChatChannelType.kCurMap] = Serialize["chat_ChatManager_1"],
	[ChatChannelType.kPrivate] = Serialize["chat_ChatManager_2"],
	[ChatChannelType.kSystem] = Serialize["chat_ChatManager_3"],
	[ChatChannelType.kServer] = Serialize["chat_ChatManager_4"],
	[ChatChannelType.kFamily] = Serialize["chat_ChatManager_5_c"],
}

VoiceState = 
{
	kNone = 0,
	kRecording = 1,
	kCancel = 2,
}

kMaxChatItem = 30  --最大发言容量
kChatLevelLimit = 17  --发言等级限制
kChatRankLimit = 4 --发言官职等级限制
kChatCDTime = 5  --发言CD
kChatChatLimit = 50  --发言字数限制

function This:init()
	super.init(self)
	self.m_map = UpdateList:create()
	self.m_world = UpdateList:create()
	self.m_private = UpdateList:create()
	self.m_sysytem = UpdateList:create()
	self.m_family = UpdateList:create()
	self.m_server = UpdateList:create()
    self.m_worldandserver = UpdateList:create()
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self,10800,self.onSendMessageReply)
	EventDispatcher:addListener(self,10801,self.onGetHistoryChatReply)
	EventDispatcher:addListener(self,10802,self.onRemoveHistoryReply)
	EventDispatcher:addListener(self,10803,self.onShareBattleReportReply)
    EventDispatcher:addListener(self,10804,self.onGetWorldChatNum)
end

function This:removeListener()
	EventDispatcher:removeListener(self,10800)
	EventDispatcher:removeListener(self,10801)
	EventDispatcher:removeListener(self,10802)
	EventDispatcher:removeListener(self,10803)
    EventDispatcher:removeListener(self,10804)
end

function This:addChat(msg)
	if msg.channel == ChatChannelType.kServer and msg.from_role.id <= 0 then
		msg.channel = ChatChannelType.kSystem
	end

	local chat = Chat:create(msg)
	local function addHandler(updateList)
		if updateList:getSize() < kMaxChatItem then
			updateList:addNode(chat)
		else
			local oldChat = updateList:getFrontNode()
			self:removeChat(oldChat)
			updateList:addNode(chat)
		end
	end

	local channel = chat:getChannel()
	if channel == ChatChannelType.kWorld and not GamePlayer:isMergeServerStart() then
		return
	end

	local sendPlayerId = chat:getFromRole().id
	local friend = FriendManager:getFriend(sendPlayerId)
	if friend and friend:isBlackFriend() then
		return 
	end

	local isAdding = true
	if channel == ChatChannelType.kWorld then
		addHandler(self.m_world)
        addHandler(self.m_worldandserver)
	elseif channel == ChatChannelType.kCurMap then
		addHandler(self.m_map)
	elseif channel == ChatChannelType.kPrivate then
		local fromRole = FriendManager:getFriend()
		if chat:getFromRole().id ~= GamePlayer:getPlayerID() then
			local friend = FriendManager:getFriend(chat:getFromRole().id)
			if friend and friend:isBlackFriend() then
				return
			end
		elseif chat:getToRole().id ~= GamePlayer:getPlayerID() then
			local friend = FriendManager:getFriend(chat:getToRole().id)
			if friend and friend:isBlackFriend() then
				return
			end
		end
		addHandler(self.m_private)
	elseif channel == ChatChannelType.kFamily then
		if not chat:isNotShowInChatList() then
			addHandler(self.m_family)
		else
			isAdding = false
		end
	elseif channel == ChatChannelType.kSystem then
		addHandler(self.m_sysytem)
	elseif channel == ChatChannelType.kServer then
		addHandler(self.m_server)
        addHandler(self.m_worldandserver)
    elseif channel == ChatChannelType.kRescue then
		isAdding = false
	end

	if isAdding then
		self:notifyObservers("onAddChat",chat)
	end
	
	if chat:isNeedBroadCast() then
		if not chat:isEmperExclusive() then
			g_scene:addNoticeBoard(chat:getContent())
		else
			g_scene:addNoticeBoard(chat:getContent(),0,true)
		end
	end
	return chat
end

function This:removeChat(chat)
	local channel = chat:getChannel()
	if channel == ChatChannelType.kWorld then
		self.m_world:removeNode(chat)
        self.m_worldandserver:removeNode(chat)
	elseif channel == ChatChannelType.kPrivate then
		self.m_private:removeNode(chat)
	elseif channel == ChatChannelType.kCurMap then
		self.m_map:removeNode(chat)
	elseif channel == ChatChannelType.kFamily then
		self.m_family:removeNode(chat)
	elseif channel == ChatChannelType.kSystem then
		self.m_sysytem:removeNode(chat)
	elseif channel == ChatChannelType.kServer then
		self.m_server:removeNode(chat)
        self.m_worldandserver:removeNode(chat)
	end

	if not chat:isNotShowInChatList() then
		self:notifyObservers("onRemoveChat",chat)
	end
end

function This:getChatList(channel)
	local function selectHandler(updateList)
		local list = updateList:getNodeList(function(chat)
			if chat:getChannel() == channel then
				return true
			end
			return false
		end)
		return list
	end

	if channel == ChatChannelType.kWorld then
		return selectHandler(self.m_world)
	elseif channel == ChatChannelType.kCurMap then
		return selectHandler(self.m_map)
	elseif channel == ChatChannelType.kPrivate then
		return selectHandler(self.m_private)
	elseif channel == ChatChannelType.kFamily then
		return selectHandler(self.m_family)
	elseif channel == ChatChannelType.kSystem then
		return selectHandler(self.m_sysytem)
	elseif channel == ChatChannelType.kServer then
		return selectHandler(self.m_server)
	end
end

function This:geWorldAndServerChatList()
	local function selectHandler(updateList)
		local list = updateList:getNodeList(function(chat)
			if chat:getChannel() == ChatChannelType.kWorld or  chat:getChannel() == ChatChannelType.kServer then
				return true
			end
			return false
		end)
		return list
	end
    return selectHandler(self.m_worldandserver)
	
end

function This:isCanChatInWorld()
	if GamePlayer:getLevel() < kChatLevelLimit then
		return false, Serialize["chat_ChatManager_6"]
	end
	if MatchManager:getRankLv() < kChatRankLimit then
		return false, string.format(Serialize["chat_ChatManager_7"],MatchManager:getRankName(kChatRankLimit))
	end
	return true
end

---------------------------------------------------------------
--发送聊天信息
function This:onSendMessageRequest(channel,msg,roleId)
	roleId = roleId or GamePlayer:getPlayerID()
	if os.isWin32 then
		Socket:send(10800,{channel = 1, content = "command_off",role_id = roleId})
	end
	Socket:send(10800,{channel = channel,content = msg, role_id = roleId, type = ChatType.kNormal })
end

function This:onSendMessageReply(msg)
	if msg.err_code == 0 then
		self:addChat(msg.chat)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
	self:notifyObservers("onSendChat")
end
--同步世界聊天条数
function This:onGetWorldChatNum(msg)
	GamePlayer:setWorldChatNum(msg.world_chat_num)
end

--近期聊天记录
function This:onGetHistoryChatRequest()
	Socket:sendAndLock(10801,{})
end
function This:onGetHistoryChatReply(msg)
	table.sort(msg.chat_list,function(a,b)
		return a.time < b.time
	end)
	for i , msg in ipairs(msg.chat_list) do
		self:addChat(msg)
	end
end

--删除近期聊天记录
function This:onRemoveHistoryRequest(roleId)
	Socket:sendAndLock(10802,{role_id = roleId})
end
function This:onRemoveHistoryReply(msg)
	if msg.err_code == 0 then
		local list = self.m_private:getNodeList(function(chat)
			if chat:getFromRole().id == msg.role_id then
				return true
			end
			return false
		end)
		for i , chat in ipairs(list) do
			self:removeChat(chat)
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--分享战报协议
function This:onShareBattleReportRequest(reportId)
	local channel = ChatChannelType.kServer
	if GamePlayer:isMergeServerStart() then
		channel = ChatChannelType.kWorld
	end 
	Socket:sendAndLock(10803,{channel = channel, type = ChatType.kShareReport, content = GameUtil:int64ToString(reportId), role_id = 0})
end

function This:onApplyJoinGroupRequest(groupId,groupName,channel,roleId)
	local t = {}
	t.roleName = GamePlayer:getRoleName()
	t.level = GamePlayer:getLevel()
	t.rank = MatchManager:getRankLv()
	t.icon = GamePlayer:getIcon()
	t.id = GamePlayer:getPlayerID()
	t.groupName = groupName
	t.groupId = GameUtil:int64ToString(groupId)
	-- t.groupId = groupId
	local str = table.tostring(t)
	channel = channel or ChatChannelType.kWorld
	roleId = roleId or 0
	Socket:sendAndLock(10803,{channel = ChatChannelType.kServer, type = ChatType.kApplyGroup, content =  str, role_id = roleId})
end

function This:onApplyJoinMatchRequest(roomRole, room)
	local role = {}
	local ss =  GameUtil:int64ToString(roomRole:getId())
	role.role_id = ss
	role.role_name = roomRole:getName()
	role.role_icon = roomRole:getIcon()
	role.role_level = roomRole:getLevel()
	role.rank_level = roomRole:getRank()
	local t = {}
	t.role = role
	t.relation = room:getRelationType()
	t.matchType = room:getMatchType()
	local str = table.tostring(t)

	local channel = ChatChannelType.kServer
	if GamePlayer:isMergeServerStart() then
		channel = ChatChannelType.kWorld
	end 
	Socket:sendAndLock(10803,{channel = channel, type = ChatType.kApplyMatch, content =  str, role_id = 0})
end

function This:onShareBattleReportReply(msg)
	if msg.err_code == 0 then
		if msg.type == ChatType.kShareReport then
			g_scene:showTips(Serialize["chat_ChatManager_8"])
		elseif msg.type == ChatType.kApplyGroup then
			g_scene:showTips(Serialize["chat_ChatManager_9"])
			self:notifyObservers("onInviteFamily", msg)
		elseif msg.type == ChatType.kApplyMatch then
			g_scene:showTips(Serialize["chat_ChatManager_10"])
		elseif msg.type == ChatType.kApplyTeamWarfare then
			g_scene:showTips(Serialize["chat_ChatManager_10"])
		elseif msg.type == ChatType.kVoice then
			g_scene:showTips(Serialize["chat_ChatManager_11"])
		end
		self:notifyObservers("onShareBattle")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onSendVoiceRequest(channel,voiceId,size,roleId)
	local t = {
		voiceId = voiceId,
		size = size,
		name = GamePlayer:getRoleName()
	}
	local str = table.tostring(t)
	if channel == ChatChannelType.kWorld then
		if not GamePlayer:isMergeServerStart() then
			channel = ChatChannelType.kServer
		end 
	end
	Socket:sendAndLock(10803,{channel = channel,type = ChatType.kVoice, content =  str, role_id = roleId or 0})
end

function This:onApplyJoinkWarfareRequest(team)
	local channel 
	local chatType = team:getChatType()
	if chatType == ChatType.kApplyTeamWarfare then
		channel = ChatChannelType.kServer
		if GamePlayer:isMergeServerStart() then
			channel = ChatChannelType.kWorld
		end
	elseif chatType == ChatType.kApplyTower then
		channel = ChatChannelType.kFamily
	end
	
	Socket:sendAndLock(10803,{channel = channel, type = chatType, content = team:tostring(), role_id = 0})
end

function This:onSendGmRequest(content)
	print("发送GM命令",content)
	local playerId = GamePlayer:getPlayerID()
	if os.isWin32 then
		Socket:send(10800,{channel = 1, content = "command_on",role_id = playerId})
	end
	Socket:send(10800,{channel = 1,content = content,role_id = playerId})
end



--endregion
