local This, super = class(..., "Dialog")
This.openSound = "Open_Ui.mp3"
This.isQuickExit = true

local ArgConfig = require("Metadatas.ArgConfig")
local ChatManager = require("Chat.ChatManager")
local ChatView = require("Chat.ChatView")
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local GuideManager = require("Guide.GuideManager")
local FamilyManager = require("Family.FamilyManager")
local VoiceChatManager = require("VoiceChat")
local GameUtil = require("Util.GameUtil")
local VoiceView = require("Chat.VoiceView")
local VoiceStateView = require("Chat.VoiceStateView")
local MatchManager = require("Match.MatchManager")
local ReportManager = require("Report.ReportManager")
local EventDispatcher = require("EventDispatcher")

This.lastChannel = ChatChannelType.kServer

local ChatMode = 
{
	kText = 1,
	kVoice = 2,
	kTranslate = 3,
	kCount = 2
}

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ChatMain.csb")
    self.m_csNode:setPosition(-190, cc.visible.center.y)
    self:addChild(self.m_csNode)
    
    self.m_isRequesting = false
    self.m_curChannel = 0
    self.m_csNode.history.m_direction = "down"
	self.m_escape = 0
    self.m_chatnum = 0
	self.m_csNode.suo:setVisible(not GamePlayer:isMergeServerStart())

	self:addTabButton(ChatChannelName[ChatChannelType.kWorld],self.m_csNode.world,function()
		if GamePlayer:isMergeServerStart() then
			self:onSelectChannel(ChatChannelType.kWorld)
		else
			g_scene:showTips(Serialize["family_BuildNode_1_c"])
			return false
		end
	end)

	self:addTabButton(ChatChannelName[ChatChannelType.kCurMap],self.m_csNode.now,function()
		self:onSelectChannel(ChatChannelType.kCurMap)
	end)

	self:addTabButton(ChatChannelName[ChatChannelType.kFamily],self.m_csNode.family,function()
		self:onSelectChannel(ChatChannelType.kFamily)
	end)

	self:addTabButton(ChatChannelName[ChatChannelType.kSystem],self.m_csNode.system,function()
		self:onSelectChannel(ChatChannelType.kSystem)
	end)

	self:addTabButton(ChatChannelName[ChatChannelType.kServer],self.m_csNode.server,function()
		self:onSelectChannel(ChatChannelType.kServer)
	end)

	self.m_mode = ChatMode.kText
	self:refreshSendBtn()
	self:refreshMode()
	self:refreshWorldBtn()
    self:refreshChatNum()
end


function This:initTouchEvent()
     local eventfun = function()
		local content = self.m_csNode.editor:getString()
		local set1, set2 = string.match(content,"%s*($)%s*(.+)")
		if set1 ~= "$" then
			if self.m_curChannel == ChatChannelType.kSystem then
				g_scene:showTips(Serialize["chat_ChatPanel_1"])
			elseif self.m_curChannel == ChatChannelType.kFamily and not FamilyManager:hasFamily() then
				g_scene:showTips(Serialize["chat_ChatPanel_2"])
			elseif self.m_curChannel == ChatChannelType.kCurMap and not GamePlayer:isInWorld() then
				g_scene:showTips(Serialize["chat_ChatPanel_3"])
			elseif (self.m_curChannel == ChatChannelType.kWorld or self.m_curChannel == ChatChannelType.kServer) and GamePlayer:getLevel() < kChatLevelLimit then
				g_scene:showTips(string.format(Serialize["chat_ChatPanel_4"]))--kChatLevelLimit
			elseif (self.m_curChannel == ChatChannelType.kWorld or self.m_curChannel == ChatChannelType.kServer) and MatchManager:getRankLv() < kChatRankLimit then
				g_scene:showTips(string.format(Serialize["chat_ChatPanel_5"],MatchManager:getRankName(kChatRankLimit)))--kChatLevelLimit
			elseif self.m_escape > 0 then
				local str = string.format(Serialize["chat_ChatPanel_6_c"], math.ceil(self.m_escape))
				g_scene:showTips(str)
			else
				local len = string.utf8Len(content)
				if len <= 0 then
					g_scene:showTips(Serialize["chat_ChatPanel_7"])
				elseif len > kChatChatLimit then
					g_scene:showTips(Serialize["chat_ChatPanel_8"])
				else
                    if self.m_curChannel == ChatChannelType.kWorld and ArgConfig["cose_free_frequency"].arg[1] - GamePlayer:getWorldChatNum() <= 0 then
                        local goldNum = ArgConfig["cose_price"].arg[2]
                        g_scene:openDialog("UI.CostConfirmPanel", {
                                            cost = goldNum,
                                            desc = Serialize[60588] , 
                                            okHandler = function()
                                                 ChatManager:onSendMessageRequest(self.m_curChannel,content)
					                             self.m_isRequesting = true
                                    end })
                    else
					    ChatManager:onSendMessageRequest(self.m_curChannel,content)
					    self.m_isRequesting = true
                    end
				end
			end
		else
			if set2 == "skip_guide" then
				GuideManager:onUpdateGuideProgressRequest(100,100)
			else
				ChatManager:onSendGmRequest(set2)
			end
		end
    end
    self.m_csNode.sendbig.bg:setTouchEnded(eventfun)
    self.m_csNode.sendsmall.bg:setTouchEnded(eventfun)
    self.m_csNode.hideButton:setTouchEnded(function ()
        self:close()
    end)
	
	self.m_csNode.change:setTouchEnded(function()
		if  self.m_curChannel ~= ChatChannelType.kWorld or (GamePlayer:getLevel() >= kChatLevelLimit and MatchManager:getRankLv() >= kChatRankLimit) then
			self.m_mode = self.m_mode % ChatMode.kCount + 1
			self:refreshMode()
		else
			g_scene:showTips(Serialize["chat_ChatPanel_9"])
		end
	end)
	self.m_csNode.talk:setTouch({
		onBegan = function()
			local playerId = GamePlayer:getPlayerID()
			local str = GameUtil:int64ToString(playerId)
			self.m_isRecordingVoice = true
			if self.m_mode == ChatMode.kVoice then
				VoiceChatManager:recordVoice(str)
			elseif self.m_mode == ChatMode.kTranslate then
				VoiceChatManager:speechText()
			end
			self:refreshMode()
			g_scene:showVoiceState(VoiceState.kRecording)
			if self.m_playView then
				self.m_playView:showPlayVoice(false)
				self.m_playView = nil
			end
		end,
		onEnded = function()
			self.m_isRecordingVoice = false
			if self.m_mode == ChatMode.kVoice then
				VoiceChatManager:stopRecord()
			elseif self.m_mode == ChatMode.kTranslate then
				VoiceChatManager:stopSpeech()
			end
			self:refreshMode()
			g_scene:showVoiceState(VoiceState.kNone)
		end,
		onCanceled = function()
			if self.m_mode == ChatMode.kTranslate then
				VoiceChatManager:cancelRecord()
			else
				VoiceChatManager:cancelSpeech()
			end
			self.m_isRecordingVoice = false
			self:refreshMode()
			g_scene:showVoiceState(VoiceState.kNone)
		end,
		onMoved = function(_,touchPoint)
			local rect = self.m_csNode.talk.speak:getBoundingBox()
			if not rect:isContainsPoint(touchPoint) then
				g_scene:showVoiceState(VoiceState.kCancel)
			else
				g_scene:showVoiceState(VoiceState.kRecording)
			end
		end,
		sizeNode = self.m_csNode.talk.speak,
		isScale = false
	})

    self.m_csNode.num:setTouchEnded(function()
	    self.m_csNode.history:jumpToBottom()
        self.m_chatnum = 0
        self:refreshChatNum()
	end)
end

function This:onEnter()
	super.onEnter(self)
	ReportManager:addObserver(self)
	ChatManager:addObserver(self)
	Application:addListener(self)
	self:selectTabButton(ChatChannelName[This.lastChannel])
	VoiceChatManager:setRecordCallback(function(voiceId,size)
		ChatManager:onSendVoiceRequest(self.m_curChannel, voiceId,size)
	end)
	VoiceChatManager:setPlayCallback(function()
		if self.m_playView then
			self.m_playView:showPlayVoice(false)
		end
		self.m_playView = nil
	end)
	VoiceChatManager:setSpeechCallback(function(msg)
		ChatManager:onSendMessageRequest(self.m_curChannel,msg,GamePlayer:getPlayerID())
	end)

	EventDispatcher:addListener(self,"ios_keyboard_show",self.onIosKeyboardShow)
    EventDispatcher:addListener(self,"ios_keyboard_hide",self.onIosKeyboardHide)
end

function This:onExit()
    super.onExit(self)
    ChatManager:removeObserver(self)
	Application:removeListener(self)
	ReportManager:removeObserver(self)
	VoiceChatManager:setRecordCallback(nil)
	VoiceChatManager:setPlayCallback(nil)
	VoiceChatManager:setSpeechCallback(nil)
	EventDispatcher:removeListener(self, "ios_keyboard_show")
    	EventDispatcher:removeListener(self, "ios_keyboard_hide")
end

function This:onIosKeyboardShow(height)
    if not self.m_originY then
        self.m_originY = self.m_csNode:getPositionY()
    end
    self.m_csNode:setPositionY(self.m_originY + height + 55)
end

function This:onIosKeyboardHide()
    self.m_csNode:setPositionY(self.m_originY)
end

function This:addChat(chat)
	local view = nil
	if chat:getChatType() == ChatType.kVoice then
		view = VoiceView:create(chat)
		view:setPlayHandler(function(playView)
			if self.m_playView then
				self.m_playView:showPlayVoice(false)
			end
			self.m_playView = playView
		end)
	else
		view = ChatView:create(chat)
	end
	view:setAnchorPoint(0.5,0)
	local w , h = view:getContentSize()
    self.m_csNode.history:pushBackCustomItem(view, w, h+5, 0)
	view:setTouch()
 
    return view
end

function This:refreshMode()
	self.m_csNode.change.yuyin:setVisible(self.m_mode == ChatMode.kVoice)
	self.m_csNode.change.wenzi:setVisible(self.m_mode == ChatMode.kText)
	self.m_csNode.change.fanyi:setVisible(self.m_mode == ChatMode.kTranslate)

	self.m_csNode.editor:setVisible(self.m_mode == ChatMode.kText)
	self.m_csNode.talk:setVisible(self.m_mode ~= ChatMode.kText)
	self.m_csNode.talk.speak:setVisible(not self.m_isRecordingVoice)
	self.m_csNode.talk["end"]:setVisible(self.m_isRecordingVoice)
	
	if self.m_mode == ChatMode.kVoice then
		self.m_csNode.talk.speak.text:setString(Serialize["chat_ChatPanel_10_c"])
	elseif self.m_mode == ChatMode.kTranslate then
		self.m_csNode.talk.speak.text:setString(Serialize["chat_ChatPanel_11_c"])
	end
    if self.m_curChannel == ChatChannelType.kWorld then
        self.m_csNode.sendbig:setVisible(false)
        self.m_csNode.sendsmall:setVisible(self.m_mode == ChatMode.kText)
    else
         self.m_csNode.sendbig:setVisible(self.m_mode == ChatMode.kText)
        self.m_csNode.sendsmall:setVisible(false)
    end
end

function This:refreshWorldBtn()
	local curTime = GamePlayer:getServerTime()
	if curTime >= GamePlayer:getMergeServTime() then
		self.m_csNode.world.light:clearShader()
	else
		self.m_csNode.world.light:setGrayShader()
	end
end

function This:refreshSendBtn()
    local sendBtn = nil
    local BtnText = nil
    if self.m_curChannel == ChatChannelType.kWorld then
        self.m_csNode.sendbig:setVisible(false)
        sendBtn = self.m_csNode.sendsmall
        local freenum = ArgConfig["cose_free_frequency"].arg[1] - GamePlayer:getWorldChatNum()
        if freenum > 0 then
            self.m_csNode.sendsmall.text:setVisible(true)
            self.m_csNode.sendsmall.buy:setVisible(false)
            BtnText = self.m_csNode.sendsmall.text
        else
             BtnText = self.m_csNode.sendsmall.buy.text
             self.m_csNode.sendsmall.text:setVisible(false)
             self.m_csNode.sendsmall.buy:setVisible(true)
             freenum = 0
        end
         self.m_csNode.sendsmall.freetext:setString(Serialize[60589] .. freenum.."）")

    else
        self.m_csNode.sendsmall:setVisible(false)
        sendBtn = self.m_csNode.sendbig
        BtnText = self.m_csNode.sendbig.text
    end
    sendBtn:setVisible(true)
	if self.m_escape > 0 then
		sendBtn.bg:setGrayShader()
		local str = string.format(Serialize["chat_ChatPanel_12_c"], math.ceil(self.m_escape))
		BtnText:setString(str)
	else
		if self.m_curChannel == ChatChannelType.kSystem then
			sendBtn.bg:setGrayShader()
		elseif self.m_curChannel == ChatChannelType.kCurMap and not GamePlayer:isInWorld() then
			sendBtn.bg:setGrayShader()
		elseif self.m_curChannel == ChatChannelType.kFamily and not FamilyManager:hasFamily() then
			sendBtn.bg:setGrayShader()
		elseif (self.m_curChannel == ChatChannelType.kWorld or self.m_curChannel == ChatChannelType.kServer) and (GamePlayer:getLevel() < kChatLevelLimit or MatchManager:getRankLv() < kChatRankLimit) then
			sendBtn.bg:setGrayShader()
		else
			sendBtn.bg:resetToNormalShader()
		end
		BtnText:setString(Serialize["chat_ChatPanel_13_c"])
	end
end

function This:refreshChatNum()

    if self.m_chatnum <= 0 then
         self.m_csNode.num:setVisible(false)
    else
         self.m_csNode.num:setVisible(true)
         self.m_csNode.num.text:setString(self.m_chatnum)
    end
end


function This:refreshItems()
	self.m_csNode.history:removeAllItems()
	local chatList = ChatManager:getChatList(self.m_curChannel)
    
	for i, chat in ipairs(chatList) do
		self:addChat(chat)
    end
    self.m_csNode.history:jumpToBottom()
end

function This:onSelectChannel(channel)
	if self.m_curChannel ~= channel then
		self.m_curChannel = channel
		self:refreshItems()
		self:refreshSendBtn()
		This.lastChannel = channel
	end
end

function This:runOpenAnimation()
	local isIphonex = GamePlayer:isIPhioneX()
	local x = 380
	if isIphonex then
		x = x + 60
	end
    self.m_csNode:runAction(cc.Sequence(cc.MoveBy(0.2, x, 0, "ExponentialIn"),cc.CallFunc(function()
		self:onOpen()
	end)))
end

function This:runCloseAnimation()
	local isIphonex = GamePlayer:isIPhioneX()
	local x = -380
	if isIphonex then
		x = x - 60
	end
    self.m_csNode:runAction(cc.MoveBy(0.2, x, 0, "ExponentialOut"))
	self:runAction(cc.Sequence(cc.FadeOut(0.2), cc.CallFunc(function ()
        self:removeFromParent()
    end)))
end

function This:getQuickRect()
    local rect = self.m_csNode.bg:getBoundingBox()
    rect:centerExtend(20, 60)
    return rect
end

function This:onAddChat(chat)
	if not chat then
		return
	end
	
	if self.m_curChannel == chat:getChannel() then
        local x,y =  self.m_csNode.history:getInnerContainerPosition()
		self:addChat(chat)
        if y > -100 then
            self.m_csNode.history:jumpToBottom()
            self.m_chatnum = 0
        else
            self.m_chatnum = self.m_chatnum + 1
        end
        self:refreshChatNum()
        --self.m_csNode.history:jumpToBottom()
	end
end



function This:onRemoveChat(chat)
	if not chat then
		return
	end
	
	local itemList = self.m_csNode.history:getItemList()
	for i , view in ipairs(itemList) do
		if chat == view:getChat() then
			view:removeFromParent()
			table.remove(itemList,i)
			break
		end
	end
end

function This:onSendChat()
	if self.m_isRequesting then
		self.m_csNode.editor:setString("")
		self.m_isRequesting = false
		self.m_escape = kChatCDTime
		self:scheduleUpdate()
	end
end

function This:onKeyDown(keyCode)
	if lib2d.isControlDown() and keyCode == "V" then
		local text = lib2d.readFromClipboard()
		self.m_csNode.editor:setString("$"..text)
	end
end

function This:onGetBattleSimpleReport(report)
	local panel = g_scene:openDialog("Report.ReportDetailPanel",report)
	panel:setToShared()
end

function This:update(dt)
	super.update(self,dt)
	if self.m_escape > 0 then
		self.m_escape = self.m_escape - dt
		self:refreshSendBtn()
		self:refreshWorldBtn()
		if self.m_escape <= 0 then
			self:unscheduleUpdate()
		end
	end
end

