-- @Author: zheng
-- @Date:   2019-08-12 16:09:01
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-09-05 17:14:04
-- @File Name: BetsPanel.lua

local CompetitionManager = require("Competition.CompetitionManager")
local GameUtil = require("Util.GameUtil")

local This , super = class(...,"Dialog")
This.fileName = "GameScene/NodeUI/CompetitionBets.csb"
function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    local teamIndex = nil
    local playerDatas = {}
    local playerIdKeyList = {}
    for _,playerInfo in ipairs(info.playerDatas) do
        local roleBase = playerInfo.roleData
        playerIdKeyList[roleBase.id] = true
        teamIndex = roleBase.index
        table.insert(playerDatas,roleBase)
    end
    self.m_battleIndex = CompetitionManager:getBattleIndex()
    self.m_teamIndex = teamIndex
    self.m_playerDatas = playerDatas
    self.m_betInfo = CompetitionManager:getBetInfoWithRoleIds(playerIdKeyList,CompetitionManager:getBateState())
    self.m_selectIndex = nil
    self:refresh()
end

function This:refresh()
    local csNode = self.m_csNode
    local function setPlayerData(data,index)
        local playerId = data.id
        local icon = csNode["player"..index]
        local name = csNode["name"..index]
        local jingcai = csNode["button"..index]
        local tip =csNode["tips"..index]
        GameUtil:setPlayerFeltNodeResource(icon,data.icon)
        name:setString(data.name,27)
        local max = CompetitionManager:getMaxBetBuyNum()
        tip:setString(string.format(Serialize[60609],max-CompetitionManager:getCurentBetNum(),max))
        local jingcaiIsGray = false
        if self.m_betInfo then
            if self.m_betInfo.role_id == playerId then
                self.m_selectIndex = index
                jingcaiVisible = true
                jingcai.text:setString(Serialize[60610])
            else
                jingcaiIsGray = true
            end
        end
        tip:setVisible(not jingcaiIsGray)
        GameUtil:btnGray(jingcai, jingcaiIsGray)
    end
    for index,baseinfo in ipairs(self.m_playerDatas) do
        setPlayerData(baseinfo,index)
    end
end

-- function This:refreshXuanzhong(selectIndex)
--     self.m_selectIndex = selectIndex
--     local csNode = self.m_csNode
--     for index,baseinfo in ipairs(self.m_playerDatas) do
--        local xuanzhong = csNode["xuanzhong"..index]
--        xuanzhong:setVisible(selectIndex ==index)
--     end
-- end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.close:setTouchEnded(function()
        self:close()
    end)
    -- csNode.look1:setTouchEnded(function()
    --     g_scene:openDialog("Player.RoleInfoPanel",{id = self.m_leftData.id})
    -- end)
    -- csNode.look2:setTouchEnded(function()
    --     g_scene:openDialog("Player.RoleInfoPanel",{id = self.m_rightData.id})
    -- end)
    -- csNode.button1:setTouchEnded(function()
    --     -- g_scene:openDialog("Player.RoleInfoPanel",{id = self.m_rightData.id})
    -- end)
    -- csNode.button2:setTouchEnded(function()
    --     -- g_scene:openDialog("Player.RoleInfoPanel",{id = self.m_rightData.id})
    -- end)
    local wenhao = csNode.wenhao
    if wenhao then
        wenhao:setTouchEnded(function()
            local text = Serialize["Competition_BetsPanel_1"]
            g_scene:openDialog("UI.HelpTips", {text = text,node = wenhao})
        end)
    end
    for index,baseInfo in ipairs(self.m_playerDatas) do
        -- local icon = csNode["player"..index]
        local look = csNode["look"..index]
        local button = csNode["button"..index]
        -- icon:setTouch({
        --     onEnded = function()
        --         if not self.m_betInfo then
        --             self:refreshXuanzhong(index)
        --         end
        --     end,
        --     isScale = false, 
        -- })
        look:setTouchEnded(function()
            g_scene:openDialog("Player.RoleInfoPanel",{id = baseInfo.id})
        end)
        button:setTouchEnded(function()
            if self.m_betInfo and self.m_betInfo.role_id~=baseInfo.id then
                g_scene:showTips(Serialize[60611])
                return
            end
            if CompetitionManager:checkIsCanBuy() then
                local info = {id = baseInfo.id,name = baseInfo.name,betInfo = self.m_betInfo,teamIndex= self.m_teamIndex,battleIndex = self.m_battleIndex }
                g_scene:openDialog("Competition.BetsBuyPanel",info)
            end
        end)
    end
end

function This:onBetPlayer(temBetInfo)
    local isFind = false
    for _,playerInfo in ipairs(self.m_playerDatas) do
        if temBetInfo.role_id == playerInfo.id then
            self.m_betInfo = temBetInfo
            self:refresh()
            break
        end
    end
end

--function This:onBetPlayer()
--    self:getData()
--    self:refresh()
--end

function This:onEnter()
    super.onEnter(self)
    CompetitionManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    CompetitionManager:removeObserver(self)
end