-- @Author: Su
-- @Date:   2019-08-22 15:17:05
-- @Last Modified by:   Su
-- @Last Modified time: 2019-08-22 15:17:05
-- @File Name: CompetitionArrayPanel.lua
-- @Description: 演武争霸布阵
local This, super = class(...,"PeakContest.PeakContestArrayPanel")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local CompetitionManager = require("Competition.CompetitionManager")
local PeakContestUtil = require("PeakContest.PeakContestUtil") 
local HeroManager = require("Hero.HeroManager")
local UpdateLayer = require("UI.UpdateLayer")

function This:initNode(info)
	self.m_troopType = info.type
	local troopList = CompetitionManager:getMyTroop()
	self.m_troopList = {}

	if troopList then 
		for i , v in ipairs(troopList) do 
			self.m_troopList[i] = v
		end
	end
	self.m_cardTroopList = {}
	for i , v in ipairs(self.m_troopList) do 
		for j , uid in ipairs(v) do 
			self.m_cardTroopList[uid] = i
		end
	end


    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PeakContestArray.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
	self.m_heroLayer = UpdateLayer:create({
		itemWidth = 95,
		itemHeight = 85,
		column = 6,
		spaceY = 0,
		spaceX = 0,
		scrollView = self.m_csNode.ScrollView,
	})
	self.m_heroLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_heroLayer:getContentSize()
	self.m_csNode.ScrollView:pushBackCustomItem(self.m_heroLayer, w,h,0)
	self.m_heroLayer:setCascadeOpacityEnabled(true)
	self.m_heroList = CompetitionManager:getHeroList()
	self.m_selectCountry = 5
	self.m_selectTroop = 1
	self.m_troopListItem = {}
	for i = 1 , 5 do 
		self.m_csNode["troop"..i]:setVisible(true)
	end
	self.m_csNode.jiantou3:setVisible(true)
	self.m_csNode.jiantou4:setVisible(true)
	self:refresh()
end

function This:refresh()
	self.m_csNode.save:setVisible(self.m_selectTroop == 5) 
	self.m_csNode.next:setVisible(self.m_selectTroop ~= 5) 
	self:updateTroopList()
	self:updateHeroList()
	self:updateTroopNum()
end

function This:updateTroopList()

	for i = 1 , 5 do 
		self.m_csNode["troop"..i].light:setVisible( self.m_selectTroop == i )
	end

	for i , v in ipairs(self.m_troopListItem) do 
		v:setVisible(false)
	end
	local heroList = self.m_troopList[self.m_selectTroop] or {}
	local width = 80
	for i = 1, 6 do  
		local hero = HeroManager:getHeroByUid(heroList[i])
		local awake = false
		if hero then
			awake = hero:isAwakeFull()
		end
		local item = self.m_troopListItem[i] 
		if not item then 
			item = PeakContestTeamItemView:create(hero,i,nil,awake)
			item:setSelectHandler(function(hero)
				self:delTroopList(hero)
			end)
			item:setPosition(width * (i - 3.5), 0 )
			item:setScale(0.78)
			self.m_csNode.team:addChild(item)
		else
			item:setModel(hero,nil,nil,awake)
		end
		item:setVisible(true)
		self.m_troopListItem[i] = item
	end

	local power = 0 
	for i , uid in ipairs(heroList) do 
		local hero = HeroManager:getHeroByUid(uid)
		power = power + hero:getFightPower()
	end
	self.m_csNode.power:setString(power)
end

function This:initTouchEvent()
	for i = 1 , 5 do 
		self.m_csNode["botton"..i]:setTouchEnded(function()
			self.m_selectCountry = i
			self:refresh()
		end)
	end

	for i = 1 , 5 do 
		self.m_csNode["troop"..i]:setTouchEnded(function()
			self.m_selectTroop = i
			self:refresh()
		end)
	end

	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.next:setTouchEnded(function()
		if self.m_selectTroop < 5 then 
			self.m_selectTroop = self.m_selectTroop + 1 
			self:refresh()
		end
	end)

	self.m_csNode.save:setTouchEnded(function()
		local function insufficientTroop()
			local t = 0
			for i = 1 , 5 do 
				if not self.m_troopList[i] then 
					return -1
				end
				for j = 1 , 6 do 
					if not self.m_troopList[i][j] then 
						if j == 1 then 
							return -1 
						else
							if t == 0 then
								t = i
							end
						end
					end
				end
			end		
			return t	
		end		
		local troop = insufficientTroop() 
		if troop == -1 then
			g_scene:showTips(Serialize[60618])
		elseif troop ~= 0 then  
			local panel = g_scene:openDialog("UI.ConfirmPanel", {text = string.format(Serialize[60619],troop)})		
			panel:setOkHandler(function()
				cc.Audio:playEffect("TopArenaSave.mp3")
				CompetitionManager:reqSaveTroopData(self.m_troopType,self.m_troopList)
				self:close()
			end)	
		else
			cc.Audio:playEffect("TopArenaSave.mp3")
			CompetitionManager:reqSaveTroopData(self.m_troopType,self.m_troopList)
			self:close()
		end		
	end)
end

function This:updateTroopNum()
	for i = 1 , 5 do
		local num = self.m_troopList[i] and #self.m_troopList[i] or 0
		self.m_csNode["troop"..i].num:setString(string.format("(%d/%d)",num,6))
	end
end