-- @Author: Su
-- @Date:   2019-08-24 15:17:05
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-09-05 11:38:17
-- @File Name: CompetitionBattleInfoItem.lua
-- @Description:

local This , super = class(...,"NodeUI")
local GamePlayer = require("Player.GamePlayer")
local ReportManager = require("Report.ReportManager")
local CompetitionManager = require("Competition.CompetitionManager")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local GameUtil = require("Util.GameUtil")
local ChatManager = require("Chat.ChatManager")
local Hero = require("Hero.Hero")
local GodBeastManager = require("GodBeast.GodBeastManager")


local LastShareReportTime = 0

This.fileName = "GameScene/NodeUI/CompetitionBattleInfoItem.csb"
local order = {Serialize[60012],Serialize[60013],Serialize[60014],Serialize[60015],Serialize[60016],Serialize[60097],Serialize[60018],Serialize[60019],Serialize[60020],Serialize[60021]}

function This:initNode(...)
	self.m_myTroopItemList = {}
	self.m_roleTroopItemList = {}
	self.m_isHideBtn = false
	self:setData(...)
end

function This:refresh()
	local xiangqing = self.xiangqing
	local lunkong = self.lunkong
	local lunkongVisible,xiangqingVisible = false,false
	if self.data.state ==0 then
		lunkongVisible = true
	else
		local function checkId(data)
			local playerId = GamePlayer:getPlayerID()
			return playerId == data.id
		end
		self.m_attack = self.data.attack or {}
		self.m_defence = self.data.defence or {}
		self.m_AttackSucess = self.data.state == 1
		self.m_defenceSucess = self.data.state ~= 1
		if checkId(self.m_defence) then
			self.m_attack = self.data.defence or {}
			self.m_defence = self.data.attack or {}
			self.m_AttackSucess = self.data.state ~= 1
			self.m_defenceSucess = self.data.state == 1
		end
		if self.data.state == 3 then
			local data = self.m_attack
			local attack = data.force_num_result / data.force_num_init
			data = self.m_defence
			local defence = data.force_num_result / data.force_num_init
			self.m_AttackSucess = attack >= defence
			self.m_defenceSucess = attack < defence
		end
		xiangqingVisible = true
		self:updateMyTroop()
		self:updateRoleTroop()
	end
	lunkong:setVisible(lunkongVisible)
	xiangqing:setVisible(xiangqingVisible)
	self.zhan:setString(string.format(Serialize[60621],order[self.m_index]))
	xiangqing.round:setString(string.format(Serialize[60622],self.data.roundcount or 0))
	
	xiangqing.round:setVisible(self.m_isHideBtn)
	xiangqing.videotape:setVisible(not self.m_isHideBtn)
	-- xiangqing.share:setVisible(not self.m_isHideBtn)
end

function This:updateMyTroop()
	local xiangqing = self.xiangqing
	local data = self.m_attack
	xiangqing.levelNode1.level:setString(data.level or 0)
	GameUtil:setPlayerIconNodeImage(xiangqing.pic1.icon, data.icon)
	xiangqing.name1:setString(data.name or "")
	xiangqing.troops1.num:setString(string.format("%d/%d",data.force_num_result,data.force_num_init))
	xiangqing.troops1.pro:setPercent(data.force_num_result / data.force_num_init * 100)
	if self.m_AttackSucess then 
		xiangqing.result1:setImage("MenuScene/shengli1.png")
	else
		xiangqing.result1:setImage("MenuScene/shibai1.png")
	end
	local width = 65

	local temCardList = GodBeastManager:removeListBeastId(data.card_list)
	for i = 1 , 6 do 
		local card = temCardList[i]
		local hero
		if card then  
			hero = Hero:create({
				heroId = card.card_type or 1000,
				starLv = card.star_level or 0
			})
		end
		local item = self.m_myTroopItemList[i]
		-- local awake = card.awake
		if not item then 
			item = PeakContestTeamItemView:create(hero,nil,nil,awake)
			item:setPosition(width * (i - 3.5), 0)
			item:setScale(0.6)
			xiangqing.troop1:addChild(item)
		else
			item:setModel(hero,nil,nil,awake)
		end		
		self.m_myTroopItemList[i] = item
	end
end

function This:updateRoleTroop()
	local data = self.m_defence

	local xiangqing = self.xiangqing

	xiangqing.levelNode2.level:setString(data.level or 0)
	GameUtil:setPlayerIconNodeImage(xiangqing.pic2.icon, data.icon)
	xiangqing.name2:setString(data.name or "")
	xiangqing.troops2.num:setString(string.format("%d/%d",data.force_num_result,data.force_num_init))
	xiangqing.troops2.pro:setPercent(data.force_num_result / data.force_num_init * 100)
	if self.m_defenceSucess then 
		self.xiangqing.result2:setImage("MenuScene/shengli1.png")
	else
		self.xiangqing.result2:setImage("MenuScene/shibai1.png")
	end
	local width = 65
	local temCardList = GodBeastManager:removeListBeastId(data.card_list)
	for i = 1 , 6 do 
		local card = temCardList[i]
		local hero
		if card then  
			hero = Hero:create({
				heroId = card.card_type or 1000,
				starLv = card.star_level or 0
			})
		end
		-- local awake = card.awake
		local item = self.m_roleTroopItemList[i]
		if not item then 
			item = PeakContestTeamItemView:create(hero,nil,nil,awake)
			item:setPosition(width * (i - 3.5), 0)
			item:setScale(0.6)
			self.xiangqing.troop2:addChild(item)
		else
			if hero then
				item:setModel(hero,nil,nil,awake)
			end
		end		
		self.m_roleTroopItemList[i] = item
	end
end

function This:onEnter()
	super.onEnter(self)
	ReportManager:addObserver(self)
	CompetitionManager:addObserver(self)
	self:refresh()
end

function This:onExit()
	ReportManager:removeObserver(self)
	CompetitionManager:removeObserver(self)
	super.onExit(self)
end

function This:initTouchEvent()
	self.xiangqing.videotape:setTouch({
        isScale = false, 
        isDynamic = true, 
        isContainer  = true,
		onEnded = function ()
			local type = self.m_type
			if not type then
				type = 1
			end
			CompetitionManager:onGetRankReportDataRequest(self.data.id,type)
			
	end})
end

function This:onGetReportDetail(msg)
	cc.SceneManager:switchTo("Report.ReportLoadingScene",nil,msg)
end

function This:setData(data,index,type)
	self.data = data
	print('ZYY:PeakContestBattleInfoListView.lua [start:142] :data',data)
	self.m_index = index
	self.m_type = type
	self:refresh()
end

function This:hide()
	self.m_isHideBtn = true
end