-- @Author: Su
-- @Date:   2019-08-13 15:17:05
-- @Last Modified by:   Su
-- @Last Modified time: 2019-08-13 15:17:05
-- @File Name: CompetitionDefenseFightPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local CompetitionManager = require("Competition.CompetitionManager")
local PeakContestUtil = require("PeakContest.PeakContestUtil") 
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")

function This:initNode()
    local troopList = CompetitionManager:getMyTroop()
    self.m_troopList = {}
    if troopList then
        for i ,v in ipairs(troopList) do
            self.m_troopList[i] = v
        end
    end

    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/CompetitionDefenseFight.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    
    self.m_beforTroopItemList = {}
	self.m_afterTroopItemList = {}
	self.m_selectTroop = 0
    self.m_csNode.changeNode:setVisible(false)
    self:updateJiantouAction()
	self:refresh()
end

function This:refresh()
	self:updateRoleTroop(1)
	self:updateTroop()
end

function This:updateRoleTroop(troop)
	for i = 1 , 5 do 
		self.m_csNode["troopBtn"..i].light:setVisible(troop == i)
	end
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
		self:close()
    end)

    self.m_csNode.change:setTouchEnded(function ()
        self.m_csNode.changeNode:setVisible(true) 
    end)

    self.m_csNode.replace:setTouchEnded(function ()
		-- 换将
		g_scene:openDialog("Competition.CompetitionArrayPanel",{})
    end)

    self.m_csNode.pk:setTouchEnded(function ()
		-- 保存整容出战
		local type = 1
		if CompetitionManager:getIsSignUp() == 1 then
			type = 2
		end
		CompetitionManager:onSignUpRequest(type,self.m_troopList)
    end)


    self.m_csNode.changeNode.cancelChange:setTouchEnded(function ()
        self.m_csNode.changeNode:setVisible(false)
    end)

    self.m_csNode.changeNode.save:setTouchEnded(function ()
        CompetitionManager:reqSaveTroopData(self.m_troopType,self.m_troopList)	
        cc.Audio:playEffect("TopArenaSave.mp3")
        self.m_csNode.changeNode:setVisible(false)
    end)

	for i = 1 , 5 do 
		self.m_csNode.changeNode["kuang"..i]:setTouchEnded(function()
			self.m_selectTroop = i
			for j = 1 , 5 do 
				self.m_csNode.changeNode["kuang"..j]:setVisible(false)
				self.m_csNode.changeNode["jiantou"..j]:setVisible(j ~= i)
				self.m_csNode.changeNode["zhezhao"..j]:setVisible(j ~= i)
			end
		end)
	end

	for i = 1 , 5 do 
		self.m_csNode["troopBtn"..i]:setTouchEnded(function()
			self:updateRoleTroop(i)		
		end)
	end

	for i = 1 , 5 do 
		self.m_csNode.changeNode["jiantou"..i]:setTouchEnded(function()
			if self.m_selectTroop ~= 0 then 
				local t = table.clone(self.m_troopList[self.m_selectTroop])
				self.m_troopList[self.m_selectTroop] = table.clone(self.m_troopList[i])
				self.m_troopList[i] = table.clone(t) 
				for j = 1 , 5 do 
					self.m_csNode.changeNode["kuang"..j]:setVisible(true)
					self.m_csNode.changeNode["jiantou"..j]:setVisible(false)					
					self.m_csNode.changeNode["zhezhao"..j]:setVisible(false)
				end
				self.m_selectTroop = 0
				self:updateTroop()
				cc.Audio:playEffect("SwitchTeam.mp3")
			end
		end)		
	end

end

function This:updateTroop()
	local width = 80
	for i = 1 , 5 do 
		local num = 0
		self.m_beforTroopItemList[i] = self.m_beforTroopItemList[i] or {}
		self.m_afterTroopItemList[i] = self.m_afterTroopItemList[i] or {}
		for j = 1 , 6 do 	
			local uid = 0 
			if self.m_troopList[i] then 
				uid = self.m_troopList[i][j] or 0
			end
			if uid ~= 0 then 
				num = num + 1 
			end
			local hero = HeroManager:getHeroByUid(uid)
			local awake = false
			if hero then
				awake = hero:isAwakeFull()
			end
			local item = self.m_beforTroopItemList[i][j]
			if not item then 
				item = PeakContestTeamItemView:create(hero,nil,nil,awake)
				item:setPosition(width * (j - 3.5), 0)
				item:setScale(0.75)
				self.m_csNode["troop" .. i]:addChild(item)
			else
				item:setModel(hero,nil,nil,awake)
			end
			self.m_beforTroopItemList[i][j] = item
			
			local item2 = self.m_afterTroopItemList[i][j]
			if not item2 then 
				item2 = PeakContestTeamItemView:create(hero,nil,nil,awake)
				item2:setPosition(width * (j - 3.5), 0)
				item2:setScale(0.75)
				self.m_csNode["troop" .. i]:addChild(item2)
			else
				item2:setModel(hero,nil,nil,awake)
			end	
			self.m_afterTroopItemList[i][j] = item2
		end
		self.m_csNode["troopBtn"..i].num:setString(string.format("(%d/%d)",num,6))
	end		
end

function This:updateTroopData(type,troopList) 
	self.m_troopList = troopList 
	self:refresh()

end

function This:onEnter()
	super.onEnter(self)
	CompetitionManager:addObserver(self)
	self:refresh()
end

function This:onExit()
	CompetitionManager:removeObserver(self)
	super.onExit(self)
end

function This:onSignUp()
	self:close()
end

function This:updateJiantouAction()
	local x = 10
	local time = 0.5
	for i = 1 , 5 do 
		local action = cc.Sequence(cc.MoveBy(time, -x, 0) , cc.MoveBy(time, x, 0))
		self.m_csNode.changeNode["jiantou"..i]:runAction(cc.RepeatForever(action))
	end
end