-- @Author: Su
-- @Date:   2019-08-20 15:30:53
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-09-05 17:14:11
-- @File Name: CompetitionManager.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")
local CompetitionTimeConfig = require("Metadatas.CompetitionTimeConfig")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")
local Application = require("cc.Application")
local BattleDataManager = require("Battle.BattleDataManager")
local MenuScene = require("Home.MenuScene")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local ArgConfig = require("Metadatas.ArgConfig")

CompetitionManagerBetState ={
  kjueSaiCompetition = 2,    --决赛竞猜阶段
  kBanjueSaiCompetition = 4, --半决赛竞猜阶段
  kSiQiangCompetition =8,     --四强竞猜阶段
  kBaQiangCompetition =16,     --八强竞猜阶段
}
CompetitionManagerFightType ={
    kWiner = 1,
    kFight2 =2,
    kFight4 =4,
    kFight8 =8,
    kFight16 =16,
}

function This:init()
    super.init(self)
    
    self.m_isSignUp = 0
    self.m_isShow = false
    self.m_isRefreshBtn = false
    self.m_teamIndex = 0
    self.m_nowStageId = 0
    self.m_nowStage={}
    self.m_serverId = 0
    self.m_stageTimeList = {}
    self.m_cardList = {}
    self.m_serverInfo = {}--战斗信息
    
    self.m_otherHonor = {}--其他区服荣誉
    self.m_myHonor = {}--本区荣誉
    self.m_betInfos = {}--竞猜信息
    self.m_betBuyTimes = 0 --购买次数
    
    self:initListener()
    -- self:onGetSelfInfoRequest()
end

function This:initListener()
    EventDispatcher:addListener(self, 14600, self.onGetSelfInfoReply)
    EventDispatcher:addListener(self, 14601, self.onSignUpReply)
    EventDispatcher:addListener(self, 14602, self.onGetTeamInfoReply)
    EventDispatcher:addListener(self, 14603, self.onGetRoleReportInfoReply)
    EventDispatcher:addListener(self, 14604, self.onGetTeamFightDataReply)
    EventDispatcher:addListener(self, 14605, self.onGetTeamFightShortReportDataReply)
    EventDispatcher:addListener(self, 14606, self.onGetRankReportDataReply)
    EventDispatcher:addListener(self, 14607, self.onGetWarZoneReply)

    EventDispatcher:addListener(self, 14500, self.onGetBatInfoReply)----竞猜信息
    EventDispatcher:addListener(self, 14501, self.onBetPlayerReply)--竞猜
    EventDispatcher:addListener(self, 14502, self.onHonorInfoReply)--荣誉
    EventDispatcher:addListener(self, 14503, self.onHonorInfosReply)--荣誉
    EventDispatcher:addListener(self, 14504, self.onSupportPlayerReply)--点赞
end

function This:removeListener()
  	EventDispatcher:removeListener(self, 14600)
  	EventDispatcher:removeListener(self, 14601)
  	EventDispatcher:removeListener(self, 14602)
  	EventDispatcher:removeListener(self, 14603)
    EventDispatcher:removeListener(self, 14604)
    EventDispatcher:removeListener(self, 14605)
    EventDispatcher:removeListener(self, 14606)

    EventDispatcher:removeListener(self,14500)
    EventDispatcher:removeListener(self,14501)
    EventDispatcher:removeListener(self,14502)
    EventDispatcher:removeListener(self,14503)
    EventDispatcher:removeListener(self,14504)
    
    self:endUpdate()
end

function This:refreshStageTime()
    if  self.m_stageTimeDataList then
       return 
    end
    self:startUpdate()
    self.m_stageTimeDataList = {}
    local curTime = GamePlayer:getServerTime()
    local tmp = os.serverdate("*t",curTime)
    self.m_today = tmp
    self.m_week = (tmp.wday + 5) % 7 + 1
    for index,data in pairs(CompetitionTimeConfig) do
        local startDayTime =curTime + 86400*(data.start_time[1]-self.m_week)
        local endDayTime =curTime + 86400*(data.end_time[1]-self.m_week)
        local startYear,startMonth,startDay = tmp.year,tmp.month,tmp.day
        local endYear,endMonth,endDay = tmp.year,tmp.month,tmp.day
        if startDayTime ~= curTime then
            local cmp = os.serverdate("*t",startDayTime)
            startYear,startMonth,startDay = cmp.year,cmp.month,cmp.day
        end
        if endDayTime ~= curTime then
            local dmp = os.serverdate("*t",endDayTime)
            endYear,endMonth,endDay = dmp.year,dmp.month,dmp.day
        end

        local startTime = os.time({year = startYear,month = startMonth, day = startDay, hour = data.start_time[2], min = data.start_time[3], sec = 0})
        local endTime = os.time({year = endYear,month = endMonth, day = endDay, hour = data.end_time[2], min = data.end_time[3], sec = 0})
        data.startTime = startTime
        data.endTime = endTime
        self.m_stageTimeList[index] = data
        if not self.m_stageTimeDataList[data.type] then
            self.m_stageTimeDataList[data.type]={}
        end
        self.m_stageTimeDataList[data.type][data.stage_id] = data
        if self.m_week >= data.start_time[1] then
            if curTime >= startTime and curTime <= endTime then
                if data.id ~= 201 and data.id ~= 117 then
                --if curTime >= startTime then
                    self.m_nowStage = data
                    self.m_nowStageId = data.stage_id
                end
            end
        end
    end
    if self.m_nowStageId == 0 then
        self.m_nowStage = self.m_stageTimeDataList[1][17]
        self.m_nowStageId = 17
    end
end

function This:isOpen(type)
    local curTime = GamePlayer:getServerTime()
    local openTime = GamePlayer:getOpenServerTime() + 21*86400
    if not self.m_stageTimeDataList then
        return false
    end
    if type == 1 then
        return curTime >= self.m_stageTimeDataList[1][1].startTime and curTime >= openTime
    else
        return curTime >= self.m_stageTimeDataList[2][2].startTime and curTime >= openTime
    end
end

function This:isHaveHonorRed()
    local res = false
    local function checkRoleData(honorInfo)
        if honorInfo and honorInfo.role_id and honorInfo.role_id ~=0 and honorInfo.reward_count ==0 then
            return true
        end
        return false
    end
    if checkRoleData(self.m_myHonor) then
        return true
    end
    if self.m_otherHonor then
        for i,honorInfo in ipairs(self.m_otherHonor) do
            if checkRoleData(honorInfo) then
                return true
            end
        end
    end
    return res
end

function This:getHonors()
    return self.m_myHonor or {}
end

function This:getOtherHonors()
   return self.m_otherHonor or {}
end

function This:getBetInfo()
    -- return {1,2,3,4,2,3,4,2,3,4,2,3,4}
   return self.m_betInfos  
end

function This:checkIsCanBuy()
    if self:getMaxBetBuyNum()==self:getCurentBetNum() then
        g_scene:showTips(Serialize[60642])
        return false
    end
    return true
end

function This:getCurentBetNum()
    return self.m_betBuyTimes or 0
end

function This:getMaxBetBuyNum()
   return DailyAchieveManager:getCrusadeData():getCrusadeConfigNum("competition_guess_max_times")
end

function This:getSucessBetReturn(betnum)
    local configItem= DailyAchieveManager:getCrusadeData():getCrusadeConfig("competition_guess_right_return_integral")
    return GameUtil:formatCount(configItem[2]*(betnum or 1)),ItemUtil.getName(configItem[1])
end

function This:getFailBetReturn(betnum)
    local configItem= DailyAchieveManager:getCrusadeData():getCrusadeConfig("competition_guess_worse_return_integral")
    return GameUtil:formatCount(configItem[2]*(betnum or 1)),ItemUtil.getName(configItem[1])
end

function This:getBetOneCost(betnum)
    return DailyAchieveManager:getCrusadeData():getCrusadeConfigNum("competition_guess_integral_cost")*(betnum or 1)
end

function This:getBetInfoWithRoleIds(roleKeyList,index)
    if index then 
        for _,betInfo in ipairs(self.m_betInfos) do
            if roleKeyList[betInfo.role_id] and betInfo.betType == index then
                return betInfo
            end
        end
    end
    return nil
end

function This:setServerType(type1)
    if type1 then
        self.m_serverTypeBet =type1
    end
end

function This:getBateState()
    local temInfo ={
      [CompetitionState.kjueSaiCompetition] = CompetitionManagerBetState.kjueSaiCompetition,
      [CompetitionState.kBanjueSaiCompetition] = CompetitionManagerBetState.kBanjueSaiCompetition,
      [CompetitionState.kSiQiangCompetition] = CompetitionManagerBetState.kSiQiangCompetition,
      [CompetitionState.kBaQiangCompetition] = CompetitionManagerBetState.kBaQiangCompetition,
    }
    return temInfo[self:getNowStage(self.m_serverTypeBet)]
end

function This:getNowStage(type)
    if not type then
        type = 1
    end
    if type == 1 and self:isCrossSever() or self.m_nowStageId == 0 then
        return 17
    end
    return self.m_nowStageId
end

function This:getNowStageFormat()
    local res = ""
    if self.m_nowStage and self.m_nowStage.stage_name then
        res = self.m_nowStage.stage_name or ""
    end
    return res..Serialize[60643]
end

function This:getStageFarmatByIndex(index)
    local res = ""
    local stage = self.m_stageTimeList[index]
    if stage and stage.stage_name then
        res = stage.stage_name or ""
    end
    return res..Serialize[60643]
end

function This:getStageByIndex(index)
    local stage = self.m_stageTimeList[index]
    return stage or {}
end

function This:getBattleIndex()
    return self.m_teamIndex
end

function This:setNowBattleIndex(index)
    if self.m_teamIndex ~= index then
        self.m_teamIndex = index
        self:notifyObservers("battlIndexChange")
    end
end

function This:isCrossSever()
    return self.m_nowStage and self.m_nowStage.type == 2
end

function This:getStageLeftTime()
    local leftTime 
    local curTime = GamePlayer:getServerTime()
    local endTime = self.m_nowStage.endTime or 0
    leftTime= endTime - curTime
    return leftTime 
    -- return 0
end

function This:getIsSignUp()
    return self.m_isSignUp
end

function This:getMyTroop()
    if self.m_cardList then
        return table.clone(self.m_cardList) 
    else
        return {}
    end
end

function This:getHeroList()
	local heros = HeroManager:getAllHeroGotWithSort(function(hero) 
			if hero:isDeputy() then
				return false
			end
			return true
		end
	)
	return heros
end

function This:reqSaveTroopData(type,troopList)
    self.m_cardList = troopList
    g_scene:showTips(Serialize[60644])
    self:notifyObservers("updateTroopData",type,table.clone(self.m_cardList))
end

function This:getTroopData()
    return self.m_cardList
end

function This:getRankList()
    return self.m_rankDataList or {}
end

function This:getCompetionRankList16()
    return self.m_teamRankList16 or{}
end

function This:getCompetionRankList8()
    return self.m_teamRankList8 or {}
end

function This:getCompetionRankList4()
    return self.m_teamRankList4 or {}
end

function This:getCompetionRankList2()
    return self.m_teamRankList2 or {}
end

function This:showInvite()
    if not self.m_isShow and self:getNowStage() == 1 and self.m_isSignUp == 0 and not self:isCrossSever() then
        if self:isOpen(1) and GamePlayer:getLevel() >= ArgConfig.competition_open_level.arg[1] then
            self.m_isShow = true
            g_scene:openDialog("Competition.CompetitionNoticePanel", {})
        end
    end
end

-- function This:getCompetionChamtion()
--     return self.m_teamChamTion or {}
-- end

--某个战区2，4,8,16强战况可能不准确,不能拿到冠军数据
function This:getBattleInfoWithFightType(battleIndex,fightType)
    local res = {}
    local rankData = self:getBattleRank(battleIndex)
    if not fightType and not rankData then
        return res
    end
    local temItem = rankData[fightType] or {}
    return temItem.rank or {}
end

-----------------------------------------------------------------------------
--点赞 rewardType--1、本区，2，跨区
function This:onSupportPlayerRequest(playerUid,rewardType)
    Socket:sendAndLock(14504,{roleId = playerUid,rewardType =rewardType })
end

function This:onSupportPlayerReply( msg )
    if msg.err_code == 0 then
        local roleId = msg.roleId or 0
        local rewardType = msg.rewardType
        local temInfo = nil
        local reward = DailyAchieveManager:getCrusadeData():getCrusadeConfig("cross_server_praise_reward")
        if rewardType ==1 then
            reward = DailyAchieveManager:getCrusadeData():getCrusadeConfig("local_server_praise_reward")
            self.m_myHonor.reward_count =1
            temInfo = self.m_myHonor
        elseif rewardType ==2 then
            for _,info in ipairs(self.m_otherHonor) do
                if info.role_id == roleId then
                    info.reward_count =1
                    temInfo = info
                    break
                end
            end
        end

        g_scene:showReward(reward or {})
        self:notifyObservers("onSupportPlayer",rewardType,temInfo)
        EventDispatcher:dispatcher("competition_honor_refresh")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

--荣誉
function This:onHonorInfoRequest()
    Socket:sendAndLock(14502)
end

function This:onHonorInfoReply( msg )
    self.m_myHonor = msg.myhonorinfo
    self:notifyObservers("onHonorInfo")
    EventDispatcher:dispatcher("competition_honor_refresh")
end

function This:onHonorInfosReply(msg)
    self.m_otherHonor = msg.myhonorinfo
    self:notifyObservers("onHonorInfos")
    EventDispatcher:dispatcher("competition_honor_refresh")
end

--竞猜  区服，决赛半决赛，分组，竞猜数量，
function This:onBetPlayerRequest(battleIndex,battleType,index,betNum,roleId,roleName,orderId)
    local info = {battle_index = battleIndex or 0,index = index,betType = battleType,role_id = roleId,role_name= roleName,betNum = betNum,order_id = orderId or 0}
    Socket:sendAndLock(14501,info)
end

function This:onBetPlayerReply( msg )
    if msg.err_code == 0 then
        local temBetInfo =  msg.betsinfo or {}
        local roleId = temBetInfo.role_id or 0
        local betType = temBetInfo.betType
        local isFind = false
        for index,betInfo in ipairs(self.m_betInfos) do
             if betInfo.role_id == roleId and betType == betInfo.betType then
                 self.m_betInfos[index] = temBetInfo
                 isFind = true
                 break
             end
        end
        if not isFind and temBetInfo.role_id then
            table.insert(self.m_betInfos,temBetInfo)
        end
        g_scene:showTips(Serialize[60645])
        self.m_betBuyTimes = msg.betBuyTime or self.m_betBuyTimes
        -- self.m_betInfos = msg.betsinfos or {}
        self:notifyObservers("onBetPlayer",temBetInfo)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetBatInfoRequest(serverType,isLock)
    local info = {type = serverType}
    if isLock then
        Socket:sendAndLock(14500,info)
    else
        Socket:send(14500,info)
    end
end

--竞猜信息
function This:onGetBatInfoReply( msg )
    if msg.err_code == 0 then
        -- print(msg)
        self.m_betInfos = msg.betsinfos or {}
        self.m_betBuyTimes = msg.betBuyTimes
        self:notifyObservers("onGetBatInfo")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetSelfInfoRequest()
	Socket:sendAndLock(14600,{})
end

function This:onGetSelfInfoReply(msg)
    if msg.err_code == 0 then
        self:refreshStageTime()
        self.m_isSignUp = msg.is_signup
        self.m_teamIndex = msg.battle_index
        for i,card in ipairs(msg.formation_list) do
            self.m_cardList[i] = card.card_list
        end
        --if self.m_teamIndex > 0 then
        --    self:onGetTeamInfoRequest(self.m_teamIndex)
        --end

        self:notifyObservers("onGetSelfCompetitionInfo")
    else
        g_scene:handleErrorCode(msg.err_code)
    end

end

function This:onSignUpRequest(type,cardList)
    Socket:sendAndLock(14601,{type = type,card_list1 =cardList[1],card_list2 = cardList[2],
    card_list3 = cardList[3],card_list4 =cardList[4],card_list5 =cardList[5] })
end

function This:onSignUpReply(msg)
    if msg.err_code == 0 then
        for i,card in ipairs(msg.formation_list) do
            self.m_cardList[i] = card.card_list
        end
        self.m_isSignUp = 1
        self:notifyObservers("onSignUp")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetTeamInfoRequest(teamIndex)
	Socket:sendAndLock(14602,{battle_index = teamIndex})

end

function This:onGetTeamInfoReply(msg)
    if msg.err_code == 0 then
        self.m_rankDataList = msg.rank_list
        self:notifyObservers("onGetRankListData",msg.rank_list)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetRoleReportInfoRequest()
    local index = self:getBattleIndex()
    if index == 0 then
        index = 1
    end
	Socket:sendAndLock(14603,{battle_index = index,role_id=GamePlayer:getPlayerID()})
    
end

function This:onGetRoleReportInfoReply(msg)
    if msg.err_code == 0 then
        self.m_roleReportList = msg.rolereport_list
         self:notifyObservers("onGetRoleReportListData",msg.rolereport_list)
    else
       g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetTeamFightDataRequest(teamIndex,type)
	Socket:sendAndLock(14604,{battle_index = teamIndex,battle_type =type })
    
end

function This:getBattleRank(battleIndex,state)
    local temState = state
    local temInfo = self.m_serverInfo[battleIndex]
    if temInfo and state and temInfo.state ~= state then
        self.m_serverInfo = {}
        return nil
    end
    return temInfo or{}
end

function This:onGetTeamFightDataReply(msg)
    if msg.err_code == 0 then
        local function convetTeamInfo(rankList,winerKeyList)
           winerKeyList[0]= nil
           local res ={}
           local roleKey ={}
           if rankList then
               for _,rankInfo in ipairs(rankList) do
                   local index = rankInfo.index
                   local roleId1 = rankInfo.role1.id
                   local roleId2 = rankInfo.role2.id
                   roleKey[roleId1] = true
                   roleKey[roleId2] = true
                   if winerKeyList[roleId1] then
                       rankInfo.winId = roleId1
                   end
                   if winerKeyList[roleId2] then
                       rankInfo.winId = roleId2
                   end
                   res[index] = rankInfo
               end
               roleKey[0] = nil
           end
           return res,roleKey
       end
       local function convertRank(rankMax,temRank)
           local res ={}
           if temRank then
               for index=1,rankMax do
                   local rankInfo = temRank[index] or {}
                   local winCount1,winCount2 = rankInfo.role1_win_count,rankInfo.role2_win_count
                   local roleInfo1,roleInfo2 ={count =0,index = index},{count = 0,index = index}
                   if winCount1 then
                       local winId = rankInfo.winId
                       roleInfo1 = {count= winCount1,roleData= rankInfo.role1,winId = winId,index = index}
                       roleInfo2 = {count= winCount2,roleData= rankInfo.role2,winId = winId,index = index}
                   end
                   table.insert(res,roleInfo1)
                   table.insert(res,roleInfo2)
               end
           end
           return res
       end
       local winRole = msg.chamption
       local winRank = {count= 0,roleData= winRole,winId = winRole.id,index = 1}
       local winRoleKey = {} 
       winRoleKey[winRole.id] =true
       local rank2 ,winRoleKey2 = convetTeamInfo(msg.rank_2_list,winRoleKey)
       local rank4 ,winRoleKey4 = convetTeamInfo(msg.rank_4_list,winRoleKey2)
       local rank8 ,winRoleKey8 = convetTeamInfo(msg.rank_8_list,winRoleKey4)
       local rank16 ,winRoleKey16 = convetTeamInfo(msg.rank_16_list,winRoleKey8)
       local info = {
           [CompetitionManagerFightType.kFight2] = {rank = convertRank(1,rank2)},
           [CompetitionManagerFightType.kFight4] = {rank = convertRank(2,rank4)},
           [CompetitionManagerFightType.kFight8] = {rank = convertRank(4,rank8)},
           [CompetitionManagerFightType.kFight16] = {rank = convertRank(8,rank16)},
           [CompetitionManagerFightType.kWiner] = {rank = {winRank}},--冠军
           --state = 5,
       }
       self.m_serverInfo[msg.battle_index]= info
       self.m_teamIndex = msg.battle_index
       self:notifyObservers("onGetFightInfoWithBattleIndex")
   else
       g_scene:handleErrorCode(msg.err_code)
   end
end

function This:onGetTeamFightShortReportDataRequest(teamIndex,reportType,type)
    local index = self:getBattleIndex()
    if index == 0 then
        index = 1
    end
	Socket:sendAndLock(14605,{team_index = teamIndex,battle_index=index,weed_out_type =reportType*2,battle_type=type})
    
end

function This:onGetTeamFightShortReportDataReply(msg)
    if msg.err_code == 0 then
        self.m_teamShortReportList = msg.simplereport_list
        self:notifyObservers("getTeamShortReportInfo",msg.simplereport_list)
    else
        g_scene:handleErrorCode(msg.err_code)

    end
end

function This:onGetRankReportDataRequest(reportId,type)
    local index = self:getBattleIndex()
    if index == 0 then
        index = 1
    end
	Socket:sendAndLock(14606,{battle_index = index,report_id = reportId,battle_type =type })
    
end

function This:onGetRankReportDataReply(msg)
    -- if msg.err_code == 0 then
    --     self.m_nowReport = msg.report_detail
    -- else
    --     g_scene:handleErrorCode(msg.err_code)
    -- end

    if msg.err_code == 0 then
        BattleDataManager:addObserver(self)
        local data={report = msg.report_detail}
		self:notifyObservers("onGetReportDetail",data)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetWarZoneRequest()
	Socket:sendAndLock(14607,{})
    
end

function This:onGetWarZoneReply(msg)
    if msg.err_code == 0 then
        self.m_nowWarZone = msg.teamcount
		self:notifyObservers("onGetCrossServer",msg.teamcount)

    else
        g_scene:handleErrorCode(msg.err_code)
    end
    
end
------------------------------
 
function This:startUpdate()
	self.m_escape = 0
	Application:addToUpdateList(self)
end

function This:endUpdate()
	Application:removeFromUpdateList(self)
end

function This:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape >= 1 then
        self.m_escape = self.m_escape - 1
        local oldStage = self.m_nowStageId
        local newStage 
        local curTime = GamePlayer:getServerTime()
        for i,data in pairs(self.m_stageTimeList) do
            if curTime >= data.startTime and curTime <= data.endTime and data.id ~= 201 and data.id ~= 117 then
                newStage = data
            end
        end
        if not newStage then
            if oldStage == 16 then
                newStage = self.m_stageTimeList[117]
            end
        end
        if newStage and newStage.stage_id ~= oldStage then
            self.m_nowStageId = newStage.stage_id
            self.m_nowStage = newStage
            self:notifyObservers("onCompetionStageChange",newStage)
        end
        if newStage and newStage.stage_id == 1 and not self.m_isRefreshBtn then
            self.m_isRefreshBtn = true
            MenuScene:refreshBtnHelp()
        end
    end
end
------------------------------
 
function This:onBattleSkip()
    cc.SceneManager:switchTo("Home.MenuScene")
    MenuScene:addDelayDialog({name = "Competition.CompetitionAllBattleInfoPanel",arg={type=self.m_serverTypeBet}})
	BattleDataManager:removeObserver(self)
end

function This:onBattleVictory()
    cc.SceneManager:switchTo("Home.MenuScene")
    MenuScene:addDelayDialog({name = "Competition.CompetitionAllBattleInfoPanel",arg={type=self.m_serverTypeBet}})
	BattleDataManager:removeObserver(self)
end

function This:onBattleFailed(isSkip)
    cc.SceneManager:switchTo("Home.MenuScene")
    MenuScene:addDelayDialog({name = "Competition.CompetitionAllBattleInfoPanel",arg={type=self.m_serverTypeBet}})
	BattleDataManager:removeObserver(self)
end
