-- @Author: zheng
-- @Date:   2019-08-12 16:09:01
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-08-30 18:07:24
-- @File Name: CompetitionResultPanel.lua

local CompetitionManager = require("Competition.CompetitionManager")
local GameUtil = require("Util.GameUtil")
local UpdateLayer = require("Rank.RankPageListLayer")
local GamePlayer = require("Player.GamePlayer")
local CompetitionResultItemView = require("Competition.CompetitionResultItemView")
local This , super = class(...,"Dialog")
This.fileName = "GameScene/NodeUI/CompetitionResult.csb"
function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    self.m_hasSelf = false
    self:getData()
    self:initUpdateLayer()
    -- self:refresh()
end

function This:refresh()
    self.m_listLayer:setDataList()
    local node = self.m_csNode.me
    node:setVisible(self.m_hasSelf)
    self.m_csNode.lookbg:setVisible(self.m_hasSelf)
    if self.m_hasSelf then
        node.paiming:setString(self.m_myRankData.rank)
        node.level:setString(string.format( "Lv%d",self.m_myRankData.level))
        node.name:setString(self.m_myRankData.name)
        node.hurt:setString(self.m_myRankData.score)
    end
end

function This:initUpdateLayer()
    local scrollView = self.m_csNode.scrollView
    if scrollView then
         self.m_listLayer = UpdateLayer:create({
            itemWidth = 540,
            itemHeight = 51,
            spaceX = 0,
            spaceY = 2,
            scrollView = scrollView,
            dataCounter = function ()
                return #self.m_configlist
            end,
         viewCreator =  function(index, node)
                local item = self.m_configlist[index]
                if item then
                    if not node then
                        return self:getScroolItem(item,index)
                    else
                        node:setModel(item, index)
                    end
                end
            end,
--            hint = self.m_selectIndex,
        })
        self.m_listLayer:setCascadeOpacityEnabled(true)
        self.m_listLayer:setAnchorPoint(0.5, 0)
        local w , h = self.m_listLayer:getContentSize()
        scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    end
end

function This:getScroolItem(data,index)
    data.rank = index
    local view = CompetitionResultItemView:create({data = data,refreshFun = self.refreshItemView})
    return view 
end

function This:refreshItemView(view,data)
    local meNode = view.me
    if meNode then
        meNode.name:setString(data.name)
        meNode.level:setString(data.level)
        meNode.hurt:setString(data.score)
        meNode.paiming:setString(data.rank)
    end
end

function This:getData()
    -- local item1 = {name = "jack",level =55,score =463,rank = 0}
    -- self.m_configlist = {item1,item1,item1,item1,item1,item1,item1,item1}
    self.m_configlist ={}
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.beijing.close:setTouchEnded(function()
        self:close()
    end)

    csNode.lookbg:setTouchEnded(function()
        if not self.m_hasSelf then
            g_scene:showTips(Serialize[60657])
            return
        end
        CompetitionManager:onGetRoleReportInfoRequest()
        g_scene:openDialog("Competition.CompetitionMyBattleInfoPanel",{})
        
    end)
end

-- function This:onBetPlayer()
--     self:getData()
--     self:refresh()
-- end

function This:onEnter()
    super.onEnter(self)
    CompetitionManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    CompetitionManager:removeObserver(self)
end

function This:onGetRankListData(list)
    self.m_configlist=list
    for i ,item in ipairs(list) do
        if item.id == GamePlayer:getPlayerID() then
            self.m_hasSelf = true
            self.m_myRankData = item
            self.m_myRankData.rank = i
        end
    end
    self:refresh()
end