-- @Author: Su
-- @Date:   2019-09-05 20:17:05
-- @Last Modified by:   Su
-- @Last Modified time: 2019-09-05 20:17:05
-- @File Name: CompetitionRuleRewardNode.lua
-- @Description: 

local This, super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")
local CompetitionRewardConfig = require("Metadatas.CompetitionRewardConfig")
local ItemNode = require("UI.ItemNode")

local RewardTypeTitle ={
    [1] = Serialize[60661],
    [2] = Serialize[60662],
}

function This:initData(info)
	self.m_rules = {}
	self.m_itemList = {}

	for k, v in pairs(CompetitionRewardConfig) do
		if not self.m_rules[v.type] then
			self.m_rules[v.type] = {}
		end
		table.insert(self.m_rules[v.type], v)
	end

	for reward_type, list in pairs(self.m_rules) do
		table.sort(list, function (a, b)
			return a.id < b.id
		end)
	end
end

function This:initNode(info)
	local scrollview = cc.ScrollView:create()
	scrollview:setContentSize(750, 410)
	self:addChild(scrollview)

	local node = self:createRewardPanel()
	local w, h = node:getContentSize()
	scrollview:pushBackCustomItem(node, w, h, 0)
end

function This:createRewardPanel()
	local node = cc.Node:create()

	for rewardType = 1, 2 do
		local titleNode = self:createTitleNode(RewardTypeTitle[rewardType])
		node:addChild(titleNode)
		table.insert(self.m_itemList, titleNode)

		local list = self.m_rules[rewardType]
		for i, v in ipairs(list) do
			local rewardNode = self:createRewardNode(v.name, v.reward_list)
			node:addChild(rewardNode)
			table.insert(self.m_itemList, rewardNode)
		end
	end

	local h = 10
	for i=#self.m_itemList, 1, -1 do
		local node = self.m_itemList[i]
		local nodeW, nodeH = node:getContentSize()
		node:setPosition(0, h)
		if i == 1 then
			h = h + nodeH
		else
			h = h + nodeH + 10
		end
	end

	node:setContentSize(750, h)
	node:setAnchorPoint(0.5, 0)

	return node
end

function This:createRewardNode(condition, reward)
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorRuleRewardItem.csb")
	node:setAnchorPoint(0, 0)
	node:setContentSize(740, 60)

	node.num:setString(condition)

	for i, v in ipairs(reward) do
		local posNode = node[string.format("item%d", i)]
		if posNode then
			local item = ItemNode:create(v[1], v[2])
			item:setAnchorPoint(0.5, 0.5)
			item:setPosition( posNode:getPosition() )
			item:setScale(0.75)
			node:addChild(item)
		end
	end

	return node
end

function This:createTitleNode(text)
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorRuleReward.csb")
	local w, h = node.bg1:getContentSize()
	node:setAnchorPoint(0, 0)
	node:setContentSize(740, h)
	node.name1:setString(text)
	return node
end