-- @Author: Su
-- @Date:   2019-09-17 16:18:33
-- @Last Modified by:   Su
-- @Last Modified time: 2019-09-17 16:18:33
-- @File Name: CrossServerBoss.lua
-- @Description: 

local This ,super = class(...,"Base.BaseModel")
local HeroManager = require("Hero.HeroManager")
local BattleDataManager = require("Battle.BattleDataManager")
local MenuScene = require("Home.MenuScene")
local DropConfig = require("Metadatas.DropConfig")

function This:create(info)
	local ret = {}
	setmetatable(ret, self)
	ret:init(info)
	return ret
end

function This:init(info)
    self.m_config = info
	self.m_challengeCards = {}
	self.m_challengeReward ={}
	local dropId = self.m_config.challenge_reward
	if DropConfig[dropId] then
		local reward = DropConfig[dropId].drop_info
		if reward then
			for i, item in ipairs( reward ) do
				self.m_challengeReward[i]={id=item[1],num = item[2]}
			end
		end
	end
end

function This:getId()
	return self.m_config.BossID
end

function This:setCardList(cardList)
	if #cardList > 0 then
		for i,cardUid in ipairs(cardList) do
			self.m_challengeCards[cardUid] = cardUid
		end
	end
end

function This:isCardCanChallenge(uid)
	if not self.m_challengeCards then
		return true
	end
	
	return not self.m_challengeCards[uid]
end

function This:getCanEmbattleHeroList()
	local heroList = HeroManager:getAllHeroGotWithSort(function(hero)
		if not self:isCardCanChallenge(hero:getUid()) then
			return false
        end
        if hero:getTotalPhase() < self.m_config.star then
            return false
        end
		return true
	end)
	return heroList
end

function This:getName()
    return self.m_config.name
end

function This:getCurHp()

end

function This:getBuySkillCost()
    return self.m_config.buff_price
end

function This:getConfig()
	return self.m_config
end

function This:getReward()
	return self.m_challengeReward or {}
end

function This:onFight(info)
	self.m_battleInfo = info
	self.m_fightPlayerList={}
	for i,report in ipairs(info.report_list) do
		self.m_fightPlayerList[i] = report.attack
	end
	info.fight_player = self.m_fightPlayerList
	cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {
		msg = info,
		bgId = self.m_config.bgId,
		mode = PlayMode.kCrossServBoss,
	})
	BattleDataManager:addObserver(self)
end

function This:onBattleVictory()
	if self.m_battleInfo then
		-- BattleDataManager:removeObserver(self)
		-- g_scene:openDialog("WorldBoss.WorldBossOverPanel", self.m_battleInfo)
		-- self.m_battleInfo = nil
		local itemList = {}
		for i,item in ipairs(self.m_battleInfo.item_list) do
			if not itemList[item.id] then
				itemList[item.id] =item
			else
				itemList[item.id].num = itemList[item.id].num + item.num
			end
		end
		BattleDataManager:removeObserver(self)
		-- g_scene:openDialog("WorldBoss.WorldBossOverPanel", self.m_battleInfo)
		g_scene:openDialog("Stage.StageVictoryPanel",{
        -- stageId = self.m_robberInBattle:getBattleId(),
        isRobber = true,
		dropInfo = itemList,
		isBoss = true,
		callback = function ()
			MenuScene:addDelayDialog({name ="CrossServerBoss.CrossServerBossPanel" })
		end,
    	})
    	BattleDataManager:removeObserver(self)

		self.m_battleInfo = nil
	end
end

function This:onBattleFailed()
	self:onBattleVictory()
end

function This:onBattleSkip(result)
	self:onBattleVictory()
end