-- @Author: Su
-- @Date:   2019-11-12 15:17:05
-- @Last Modified by:   Su
-- @Last Modified time: 2019-11-12 15:17:05
-- @File Name: CrossServerBossArrayPanel.lua
-- @Description: 跨服boss布阵

local This, super = class(...,"Competition.CompetitionArrayPanel")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local CrossServerBossManager = require("CrossServerBoss.CrossServerBossManager")
local PeakContestUtil = require("PeakContest.PeakContestUtil") 
local HeroManager = require("Hero.HeroManager")
local UpdateLayer = require("UI.UpdateLayer")

function This:initNode(info)
	self.m_troopList = {}
	local troopList = CrossServerBossManager:reqGetTroopData()

	if troopList then 
		for i , v in ipairs(troopList) do 
			self.m_troopList[i] = v
		end
	end
	self.m_cardTroopList = {}
	for i , v in ipairs(self.m_troopList) do 
		for j , uid in ipairs(v) do 
			self.m_cardTroopList[uid] = i
		end
	end

    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/CrossServerBossArray.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
	self.m_heroLayer = UpdateLayer:create({
		itemWidth = 95,
		itemHeight = 85,
		column = 6,
		spaceY = 0,
		spaceX = 0,
		scrollView = self.m_csNode.ScrollView,
	})
	self.m_heroLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_heroLayer:getContentSize()
	self.m_csNode.ScrollView:pushBackCustomItem(self.m_heroLayer, w,h,0)
	self.m_heroLayer:setCascadeOpacityEnabled(true)
	self.m_boss = CrossServerBossManager:getBoss()
	self.m_heroList = self.m_boss:getCanEmbattleHeroList()
	self.m_selectCountry = 5
	self.m_selectTroop = 1
	self.m_troopListItem = {}
	for i = 1 , 10 do 
		self.m_csNode.ListView["troop"..i]:setVisible(true)
	end
	self.m_csNode.jiantou3:setVisible(true)
	self.m_csNode.jiantou4:setVisible(true)
	self:refresh()
end

function This:refresh()
	self.m_csNode.save:setVisible(true) 
	self.m_csNode.next:setVisible(false) 
	self:updateTroopList()
	self:updateHeroList()
	self:updateTroopNum()
end

function This:updateTroopList()

	for i = 1 , 10 do 
		self.m_csNode.ListView["troop"..i].light:setVisible( self.m_selectTroop == i )
	end

	for i , v in ipairs(self.m_troopListItem) do 
		v:setVisible(false)
	end
	local heroList = self.m_troopList[self.m_selectTroop] or {}
	local width = 80
	for i = 1, 6 do  
		local hero = HeroManager:getHeroByUid(heroList[i])
		local awake = false
		if hero then
			awake = hero:isAwakeFull()
		end
		local item = self.m_troopListItem[i] 
		if not item then 
			item = PeakContestTeamItemView:create(hero,i,nil,awake)
			item:setSelectHandler(function(hero)
				self:delTroopList(hero)
			end)
			item:setPosition(width * (i - 3.5), 0 )
			item:setScale(0.78)
			self.m_csNode.team:addChild(item)
		else
			item:setModel(hero,nil,nil,awake)
		end
		item:setVisible(true)
		self.m_troopListItem[i] = item
	end

	local power = 0 
	for i , uid in ipairs(heroList) do 
		local hero = HeroManager:getHeroByUid(uid)
		power = power + hero:getFightPower()
	end
	self.m_csNode.power:setString(power)
end

function This:initTouchEvent()
	for i = 1 , 5 do 
		self.m_csNode["botton"..i]:setTouchEnded(function()
			self.m_selectCountry = i
			self:refresh()
		end)
	end

	for i = 1 , 10 do
		self.m_csNode.ListView["troop"..i].dark:setTouch({
			isScale = false, 
			isDynamic = true, 
			isContainer  = true,	
			isSwallow = false,
			onEnded = function()
			self.m_selectTroop = i
			self:refresh()
		end})
	end

	self.m_csNode.close:setTouchEnded(function()
		self:close()
		local list = CrossServerBossManager:reqGetTroopData()
		if #list > 0 then
			g_scene:openDialog("CrossServerBoss.CrossServerBossDefensePanel",{cardList = list })
		end
	end)

	self.m_csNode.next:setTouchEnded(function()
		if self.m_selectTroop < 10 then 
			self.m_selectTroop = self.m_selectTroop + 1 
			self:refresh()
		end
	end)

	self.m_csNode.save:setTouchEnded(function()
		local function insufficientTroop()
			local t={}
			for i = 1 , 10 do 
				--if not self.m_troopList[i] then
				--	t[i] = 0
				--end
				for j = 1 , 6 do
					if self.m_troopList[i] then
						if  not self.m_troopList[i][j] then
							t[i] = t[i] or 0
						else
							t[i] = j
						end
					else
						t[i] = 0
					end
				end
			end		
			return t	
		end	
		local function ifSaveTroop(t)
			local o = 0
			for i,num in ipairs(t) do
				if num > o then
					o = num
				end
				if num > 0 and num ~= 6 then
					return i
				end
				if i >= 2 then
					if t[i-1] == 0 and num > 0 then
						return -1
					end
				end
			end
			if o == 0 then
				return -2
			end
			return 0
		end
		local list = insufficientTroop() 
		local troop = ifSaveTroop(list)
		if troop > 0 then
			g_scene:showTips(string.format( Serialize[60670],troop))
		elseif troop == -1 then
			g_scene:showTips(Serialize[60671])

			-- local panel = g_scene:openDialog("UI.ConfirmPanel", {text = string.format(Serialize[60619],troop)})		
			-- panel:setOkHandler(function()
			-- 	cc.Audio:playEffect("TopArenaSave.mp3")
			-- 	CompetitionManager:reqSaveTroopData(self.m_troopType,self.m_troopList)
			-- 	self:close()
			-- end)	
		elseif troop == -2 then
			g_scene:showTips(Serialize[60672])
		elseif troop == 0 then
			cc.Audio:playEffect("TopArenaSave.mp3")
			local list = table.clone(self.m_troopList)
			CrossServerBossManager:reqSaveTroopData(list)
			g_scene:showTips(Serialize[60673])

			-- self:close()
		end		
	end)
end

function This:updateTroopNum()
	for i = 1 , 10 do
		local num = self.m_troopList[i] and #self.m_troopList[i] or 0
		self.m_csNode.ListView["troop"..i].num:setString(string.format("(%d/%d)",num,6))
	end
end