-- @Author: Su
-- @Date:   2019-09-11 15:30:53
-- @Last Modified by:   Su
-- @Last Modified time: 2019-09-11 15:30:53
-- @File Name: CrossServerBossManager.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local CrossServerBossConfig = require("Metadatas.CrossServerBossConfig")
local GamePlayer = require("Player.GamePlayer")
local CrossServerBoss = require("CrossServerBoss.CrossServerBoss")

CrossServerBossState = {
	kNotOpen = 0,
	kOpen = 1,
	kEnd = 2
}

function This:init()
	super.init(self)
	self.m_bossList = nil
	self.m_hurtList = {}
	self.m_bossId = 1
	self.m_boss = CrossServerBoss:create(CrossServerBossConfig[self.m_bossId])
	self.m_changeTime = 0
	self.m_useGoldList = {}
	self:initTime()
	self:initListener()
end

function This:initListener()
    EventDispatcher:addListener(self, 12415, self.onGetBossInfoReply)
    EventDispatcher:addListener(self, 12416, self.onFightBossReply)
end

function This:removeListener()
  	EventDispatcher:removeListener(self, 12415)
  	EventDispatcher:removeListener(self, 12416)
    
end

function This:initTime()
    local curTime = GamePlayer:getServerTime()
    local tmp = os.serverdate("*t",curTime)
    self.m_today = tmp
	self.m_week = (tmp.wday + 5) % 7 + 1
	local startTime = os.time({year = tmp.year,month = tmp.month, day = tmp.day, hour = 0, min = 0, sec = 0})
	self.m_openTime = startTime + self:getBossConfig().open_time
	self.m_endTime = startTime + self:getBossConfig().close_time
end

function This:getBossConfig()
	return CrossServerBossConfig[self.m_bossId]
end

function This:setIsUseGold(index,isUseGold)
	if isUseGold then
		self.m_useGoldList[index] = true
	else
		self.m_useGoldList[index] = false
	end
end

function This:getHurtList()
	return self.m_hurtList
end

function This:getChangeTime()
	return self.m_changeTime
end

function This:getBoss()
	return self.m_boss
end

function This:getTime()
    local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_openTime then
		if curTime <= self.m_endTime then
			return self.m_endTime - curTime,CrossServerBossState.kOpen
		else
			return self.m_endTime - curTime,CrossServerBossState.kEnd
		end
	else
		return self.m_openTime - curTime,CrossServerBossState.kNotOpen
	end
end

function This:isShow()
	local curTime = GamePlayer:getServerTime()
    local tmp = os.serverdate("*t",curTime)
    self.m_today = tmp
	self.m_week = (tmp.wday + 5) % 7 + 1
	if self.m_week == 2 or self.m_week == 4 or self.m_week == 6 then
		return true
	end
	return false
end

function This:getSelfDamage()
	return self.m_selfHurt or 0
end

function This:reqGetTroopData()
	return self.m_cardList or {}
end

function This:reqSaveTroopData(cardList)
	for i,list in ipairs(cardList) do
		if not list or #list == 0 then
			cardList[i] = nil
		end
	end
	self.m_cardList = cardList
end

-----------------------------------------------------
function This:onGetBossInfoRequest()
	Socket:sendAndLock(12415,{id = self.m_bossId})
end

function This:onGetBossInfoReply(msg)
	-- if msg.err_code == 0 then
		self.m_hurtList = msg.damege_list
		self.m_useCardList =msg.card_list
		self.m_boss:setCardList(msg.card_list)
		self.m_changeTime = msg.change_times
		self.m_selfHurt = msg.self_damege
        self:notifyObservers("onGetBossInfo")
    -- else
    --    g_scene:handleErrorCode(msg.err_code)
    -- end
end

function This:onFightBossRequest(id,cardList)
	local list = {}
	for i,cardIdList in ipairs(cardList) do
		for _,card in ipairs(cardIdList) do
			table.insert(list,card)
		end
	end
	Socket:sendAndLock(12416,{id = id,card_list = list, user_gold = self.m_useGoldList})
end

function This:onFightBossReply(msg)
	if msg.err_code == 0 then
		self.m_boss:onFight(msg)
		self:onGetBossInfoRequest()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end