-- @Author: Su
-- @Date:   2019-09-11 15:30:53
-- @Last Modified by:   Su
-- @Last Modified time: 2019-09-11 15:30:53
-- @File Name: CrossServerBossPanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local CrossServerBossManager = require("CrossServerBoss.CrossServerBossManager")
local CrossServerBossListItem = require("CrossServerBoss.CrossServerBossListItem")
local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager") 
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local RescuePalaceManager = require("Palace.RescuePalaceManager")


function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/CrossServerBoss.csb")
    self.m_csNode = csNode
    
    self:addChild(csNode)
    csNode:setToCenter()

    self.m_bossConfig = CrossServerBossManager:getBossConfig()
    self.m_boss = CrossServerBossManager:getBoss()
    self.m_escape = 0
    self.m_haveRank = false
    self.m_itemList = {}
    self:initInterface()
    self:refreshList()
end

function This:initInterface()
    local node = self.m_csNode
    local reward = self.m_boss:getReward()
    node.bosstu:setImage(self.m_bossConfig.image)
    node.me:setString(Serialize[60682])
    node.hurt:setString(Serialize[60529])
    local index = 1
    for _,itemData in pairs(reward) do
        local item = ItemNode:create(itemData.id,itemData.num)
        item:setAnchorPoint(0.5,0.5)
        node["item"..index]:addChild(item)
        index = index + 1
    end
end

function This:refresh()
    local node = self.m_csNode
    node.number2:setVisible(false)
    node.number2:setString(string.format( Serialize[60683],self.m_bossConfig.challenge_time-CrossServerBossManager:getChangeTime(),self.m_bossConfig.challenge_time))
    local hurt = CrossServerBossManager:getSelfDamage()
    if not self.m_haveRank and hurt > 0 then
        node.me:setString(Serialize[60684])
    end
    node.hurt:setString(string.format( Serialize[60530],hurt))
    local leftTime,bossState = CrossServerBossManager:getTime()

    local area = RescuePalaceManager:getCompetitionAreaInfo()
	if not area then
		node.beijing.qufu1:setString(Serialize[60685])
	else
		local serverList = area.serverList
		local minServer = serverList[1]
		local maxServer = serverList[#serverList]
		local str = ""

		if minServer.id == maxServer.id then
			str = string.format(Serialize[60686], minServer.tag, minServer.indexInGroup)
		else
			str = string.format(Serialize[60687], minServer.tag, minServer.indexInGroup, maxServer.tag, maxServer.indexInGroup)
		end

		node.beijing.qufu1:setString(str)
	end

    -- node.qufu1:setString()
    if bossState ~= CrossServerBossState.kOpen or CrossServerBossManager:getChangeTime() ~= 0 then
        node.button2:setGrayShader()
    else
        node.button2:clearShader()
    end
end

function This:refreshList()
    local node = self.m_csNode
    local ScrollView = node.ScrollView
    local hurtList = CrossServerBossManager:getHurtList()
    -- local hurtList = {[1]={rank = 1,name = Serialize[60688],hurt = 23333},
    -- [2]={rank = 2,name = Serialize[60688],hurt = 23333}}
    for i,hurtData in ipairs(hurtList) do
        if not self.m_itemList[i] then
            hurtData.rank=i
            if GamePlayer:getPlayerID() == hurtData.role_id then
                node.me:setString(string.format(Serialize[60689],hurtData.rank))
                self.m_haveRank = true
                -- node.hurt:setString(string.format(Serialize[60530],hurtData.damege))
            end
            local item = CrossServerBossListItem:create(hurtData)
            item:setScaleX(0.85)
            local w,h = item:getSize()
            ScrollView:pushBackCustomItem(item,448,36,0.5)
            self.m_itemList[i] = item
        else
            local item = self.m_itemList[i]
            hurtData.rank=i
            item:refreshData(hurtData)
        end
    end

end

function This:onEnter()
    -- super:onEnter(self)
    CrossServerBossManager:onGetBossInfoRequest()
    self:scheduleUpdate()
	CrossServerBossManager:addObserver(self)

end

function This:onExit()
    -- super:onExit(self)
	CrossServerBossManager:removeObserver(self)

end

function This:initTouchEvent()

    self.m_csNode.beijing.close:setTouchEnded(function()
        self:close() 
    end)

    self.m_csNode.button1:setTouchEnded(function()
        g_scene:openDialog("CrossServerBoss.CrossServerBossRewardPanel", {})
    end)

    self.m_csNode.button2:setTouchEnded(function()
        -- self.m_challengeCoroutine = coroutine.create(function ()
        --     self:challenge()
        --     self.m_challengeCoroutine = nil
        -- end)
        -- coroutine.resume(self.m_challengeCoroutine)
        local leftTime,bossState = CrossServerBossManager:getTime()
        if bossState ~= CrossServerBossState.kOpen then
            g_scene:showTips(Serialize[60690])
            return
        end
        if CrossServerBossManager:getChangeTime() ~= 0 then
            g_scene:showTips(Serialize[60526])
            return
        end
        local cardList = CrossServerBossManager:reqGetTroopData()
        if cardList and #cardList > 0 then
			g_scene:openDialog("CrossServerBoss.CrossServerBossDefensePanel",{cardList = cardList })
        else
            g_scene:openDialog("CrossServerBoss.CrossServerBossArrayPanel", {})
        end
    end)
    self.m_csNode.wenhao:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips", {text=Serialize[60691], node = self.m_csNode.wenhao})
    end)
end

function This:checkItem(itemId, costNum)
    local result, errorCode = DepotManager:isItemOrResEnough(itemId, costNum)
    if not result then
        if errorCode then
            g_scene:handleErrorCode(errorCode)
        else
            g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(itemId)))
        end
    end
    return result
end


function This:challenge()
	--local level, canFight = self.m_Boss:getBattleLevel()
	--if not canFight then
	--	g_scene:showTips(string.format(Serialize[60692], level))
	--	return
	--end
    local leftTime,bossState = CrossServerBossManager:getTime()
    if bossState ~= CrossServerBossState.kOpen then
        g_scene:showTips(Serialize[60690])
        return
    end
	self.m_challengeTimes = self.m_bossConfig.challenge_time-CrossServerBossManager:getChangeTime()
    if self.m_challengeTimes <= 0 then
		g_scene:showTips(Serialize[60693])
    	return
    end

	local cardList, buyBuff
	if self.m_challengeTimes > 0 then
		local panel = self:openDialog("CrossServerBoss.BossCrossEmbattlePanel",{
			battleHandler = function(param)
            	coroutine.resume(self.m_challengeCoroutine, param)
			end,
			stage = self.m_boss
		})
    	cardList = coroutine.yield()
    else
    	return
	end
	if cardList then
		local costNum = self.m_boss:getBuySkillCost()
	    self:openDialog("UI.CostConfirmPanel",{
	    	fileName = "MenuScene/NodeUI/WorldBossMorale.csb",
	    	cost = costNum,
	        okHandler = function()
	        	if self:checkItem(102, costNum) then
	        	    coroutine.resume(self.m_challengeCoroutine, true)
	        	end
	        end,
	        cancelHandler = function()
	            coroutine.resume(self.m_challengeCoroutine, false)
	        end
	    })
	    buyBuff = coroutine.yield()

        -- WorldMonsterManager:onChallengeWorldBossRequest(self.m_selectBoss:getId(), cardList, buyBuff)
        CrossServerBossManager:onFightBossRequest(self.m_boss:getId(),cardList,buyBuff)
	end
end

function This:refreshTime()
    local node = self.m_csNode
    local leftTime,bossState = CrossServerBossManager:getTime()
    if leftTime <= 0 then
        leftTime = 0
    end
    if bossState == CrossServerBossState.kOpen then
        node.number:setString(string.format(Serialize[60694],GameUtil:getFormatDtimeStr(leftTime)))
    elseif bossState == CrossServerBossState.kNotOpen then
        node.number:setString(string.format(Serialize[60695],GameUtil:getFormatDtimeStr(leftTime)))
    else
        node.number:setStringWithColor(Serialize[60696])
    end
end

function This:onGetBossInfo()
    self:refreshList()
    self:refresh()
end

function This:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshTime()
	end
end