-- @Author: superm
-- @Date:   2018-05-18 10:08:20
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-16 15:28:39
-- @File Name: DepotBaseTabNode.lua
-- @Description: 

local This = class(...)
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local DepotItemView = require("Depot.DepotItemView")
local DepotQueue = require("Depot.DepotQueue")

function This:create(...)
	local o = {}
	setmetatable(o,self)
	o:init(...)
	return o
end

function This:init(info)
    self.m_csNode = info.node
    self.m_csParentNode = info.parent
    self.m_listLayer = info.list
    self.m_views = DepotQueue:create()
    self.m_selectItemUID = 0
end

function This:isProps()
    return true
end

function This:isSpall()
    return false
end

function This:isEquip()
    return false
end

function This:isRune()
    return false
end

function This:getDepotItemList()
    return DepotManager:getPropsItemList()
end

function This:getSelectedItem()
    return self.m_item
end

function This:getSelectedIndex()
	if self.m_itemNode then
    	return self.m_itemNode:getIndex()
    end
    return 1
end

function This:setNodesVisible(nameList, isVisible, rootNode)
    local csNode
    if rootNode then
        csNode = rootNode
    else
        csNode = self.m_csNode.backpack
    end
    for _, names in ipairs(nameList) do
        for _, name in ipairs(names) do
            csNode[name]:setVisible(isVisible)
        end
    end
end

function This:resetNodes()
    local basePartNames = {"quality", "icon", "equiplevel", "pieceicon", "namebg", "name", "counttxt", "count"}
    local descPartNames = {"descbg", 
	    "propname1", "propvalue1", "propname2", "propvalue2", "propname3", "propvalue3", "propname4", "propvalue4", 
	    "propdesc", "normaldesc", "propnote"
	}
    local spallPartNames = {"spalltxt", "spall", "spallnote", "spalllacknote"}
    local pricePartNames = {"pricetxt", "priceicon", "price","price2"}
    local buttonPartNames = {"sell", "use", "strengthen", "synthesis", "ascend"}
    local nameList = {basePartNames, descPartNames, spallPartNames, pricePartNames, buttonPartNames}
    self:setNodesVisible(nameList, false)
    self.m_csNode.choose:setVisible(false)
    self.m_csNode.choose:switchParent(self.m_csNode)
    self.m_itemNode = nil
end

function This:clear()
    self.m_views:clear(function( view )
        view:release()
    end)
end

----------------------------------------------------------------
function This:refreshInfoUI(visible)
    self:refreshItemUI(visible)
    self:refreshDescUI(visible)
    self:refreshButtonUI(visible)
end

function This:refreshItemUI(visible)
    local csNode = self.m_csNode.backpack
    csNode.quality:setVisible(visible)
    csNode.icon:setVisible(visible)
    csNode.namebg:setVisible(visible)
    csNode.name:setVisible(visible)
    csNode.counttxt:setVisible(visible)
    csNode.count:setVisible(visible)

    csNode.pricetxt:setVisible(visible)
    csNode.priceicon:setVisible(visible)
    csNode.price:setVisible(visible)

    csNode.equiplevel:setVisible(false)
    csNode.pieceicon:setVisible(false)

    if not visible then
        return
    end

    csNode.quality:setImage(ItemUtil.getQualityImage(self.m_item:getId()))
    csNode.icon:setImage(ItemUtil.getIconImage(self.m_item:getId()))
    csNode.name:setString(self.m_item:getName(), (not AppText:isChinese()) and 520 or nil)
    csNode.count:setString(self.m_item:getCount())

    if self.m_item:canBeSold() then
        csNode.priceicon:setImage(ItemUtil.getPriceImage(self.m_item:getId()))
        csNode.price:setString(self.m_item:getPrice())
        csNode.priceicon:setVisible(true)
        csNode.price:setVisible(true)
        csNode.price2:setVisible(false)
    else
        csNode.priceicon:setVisible(false)
        csNode.price:setVisible(false)
        csNode.price2:setVisible(true)
    end
end

function This:getDescNode()
    return self.m_csNode.backpack.normaldesc
end

function This:refreshDescUI(visible)
	local descNode = self:getDescNode()
    self.m_csNode.backpack.descbg:setVisible(visible)
    descNode:setVisible(visible)
    if not visible then
        return
    end
    local strCommon = self.m_item:getComment()
    if os.isWin32 then
        strCommon = "ID:".. tostring(self.m_item:getId()) .. "," .. strCommon
    end
    descNode:setString(strCommon, 270/descNode:getScale())
end

function This:refreshButtonUI(visible)
    if self.m_item:isEquip() or self.m_item:isSpall() then
    	return
    end

    self.m_csNode.backpack.sell:setVisible(visible and self.m_item:canBeSold())
    self.m_csNode.backpack.use:setVisible(visible)

    local state = self.m_item:canUsed()
    if state then
        self.m_csNode.backpack.use:clearShader()
    else
        self.m_csNode.backpack.propnote:setVisible(true)
        self.m_csNode.backpack.use:setGrayShader()
    end
end

function This:refresh(cleanup)
    if not (self.isEquip and self:isEquip()) then
        self.m_csNode.fabao:setVisible(false)
        self.m_csNode.fushi:setVisible(false)
    end

    self:cacheItemViews()
    self.m_listLayer:clearNode(true, false)
    self.m_itemNode = nil
    self.m_item = nil

    self:resetNodes()

    self.m_itemList = self:getDepotItemList()
    local itemCount = #self.m_itemList
    if itemCount == 0 then -- 没有道具
        self.m_csNode.empty:setVisible(true)
        self.m_csNode.fabao:setVisible(false)
        self.m_csNode.scrollView:setVisible(false)
        self.m_csParentNode.bg1:setVisible(false)
        self.m_csParentNode.bg2:setVisible(false)
        self.m_csParentNode.xian1:setVisible(false)
        self.m_csParentNode.xian2:setVisible(false)
        return
    else
        self.m_csNode.empty:setVisible(false)
        self.m_csNode.scrollView:setVisible(true)
        self.m_csParentNode.bg1:setVisible(true)
        self.m_csParentNode.bg2:setVisible(true)
        self.m_csParentNode.xian1:setVisible(true)
        self.m_csParentNode.xian2:setVisible(true)
    end

    self.m_csNode.scrollView:setTouchEnabled(true)
    self.m_csNode.choose:switchParent(self.m_csNode.scrollView)
    self.m_csNode.choose:setLocalZOrder(10)

    self.m_listLayer:setDataList(
        self.m_itemList, 
        function(item, index)
            local info = {item = item, index = index, handler = function(node)
                self:onSelectItem(node)
            end}
            return self:getItemView(info)
        end,
        function() 
            self:updateItemSelect() 
        end,
        self.m_hintIndex
    )
end

function This:getItemView(info)
    local ret = self.m_views:popBack()
    if not ret then
        ret = DepotItemView:create(info)
    else
        ret:setInfo(info)
    end    
    return ret
end

function This:cacheItemViews()
    local viewList = self.m_listLayer.m_views or {}
    for _, node in ipairs(viewList) do
        node:retain()
        self.m_views:pushBack(node)
    end
end

function This:updateItemSelect()
    local nodeItemList = self.m_listLayer.m_views or {}
    local _id = self.m_selectItemUID

    for i, node in ipairs(nodeItemList) do
        if node:isVisible() and node.item and node.item._id == _id then
            self:onSelectItem(node, true)
            return
        end
    end
    self:onSelectItem(nodeItemList[1], true)
end

function This:onSelectItem(node, delayShow)
    if self.m_itemNode ~= node then
		local visible 	= false
		self.m_itemNode = node
	    self.m_item 	= node and node.item or nil
        self.m_hintIndex = node.index

	    if self.m_item then
	    	visible = true
	    	self.m_selectItemUID = self.m_item:getUid()
            if self.m_item:isRune() then
                self.m_csNode.backpack:setVisible(false)
                self.m_csNode.fabao:setVisible(false)
                self.m_csNode.fushi:setVisible(true)
                self:refreshInfoUI(visible)
            elseif self.m_item.isFabao and self.m_item:isFabao() then -- 如果选中的是法宝
                self.m_csNode.backpack:setVisible(false)
                self.m_csNode.fabao:setVisible(true)
                self.m_csNode.fushi:setVisible(false)
                if self.refreshFabaoInfo then
                    self:refreshFabaoInfo()
                end
            else -- 选中的不是法宝
                self.m_csNode.fushi:setVisible(false)
                self.m_csNode.backpack:setVisible(true)
                self.m_csNode.fabao:setVisible(false)
                self:refreshInfoUI(visible)
            end
	    else
			self.m_selectItemUID = 0
            self:refreshInfoUI(visible)
	    end
        self:updateSelectNode(delayShow)
    else
        self:updateSelectNode(delayShow)
    end
end

function This:_updateSelectNode()
    local isVisible = false
    if self.m_itemNode and self.m_itemNode:isVisible() then
        isVisible = true
    end
    self.m_csNode.choose:setVisible(isVisible)
    if not isVisible then
        return
    end
    for i, node in ipairs(self.m_listLayer.m_views) do
        if node == self.m_itemNode then
            local x, y = node:getPosition()
            self.m_csNode.choose:setPosition(x-1.2, y-3.1)
            return
        end
    end
    self.m_csNode.choose:setVisible(false)
end

function This:updateSelectNode(delayShow)
	local found = false
    if delayShow and self.m_itemNode then
        local scrollView = self.m_csNode.scrollView
        for i, node in ipairs(self.m_listLayer.m_views) do
            if node == self.m_itemNode then
                scrollView:runAction(cc.Sequence(cc.DelayTime(0.1), cc.CallFunc(function()
                    self:_updateSelectNode()
                end)))
                found = true
                break
            end
        end
    end

    -- print(found and "found" or "not found")
    if not found then
    	self:_updateSelectNode()
    end
end

function This:setVisible(value)
    self.m_csNode:setVisible(value)
end

