-- @Author: superm
-- @Date:   2018-05-21 09:56:58
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-07-13 14:45:04
-- @File Name: DepotBoxView.lua
-- @Description: 

local This , super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/DepotPropBoxItem.csb"

local MatchManager = require("Match.MatchManager")
local GamePlayer = require("Player.GamePlayer")
local VipManager = require("Vip.VipManager")
local BoxSlotConfig = require("Metadatas.BoxSlotConfig")
local GameUtil = require("Util.GameUtil")

local StateDesc = {[BoxState.kLock]=Serialize["depot_BoxView_1"], [BoxState.kOpening]=Serialize["depot_BoxView_2"], [BoxState.kReceive]=Serialize["depot_BoxView_3"]}
local StateImg = {[BoxState.kLock]="boxLock", [BoxState.kOpening]="boxLock", [BoxState.kReceive]="boxGet"}

function This:initData(info)
	if info then
	    self.m_index = info.index
	    self.m_parent = info.parent
	end
	self.m_selected = false
end

function This:initNode()
    self.m_config = BoxSlotConfig[self.m_index]
    if self:doNeedVip() then
        self.m_vipModel = VipManager:getNewestModel()
    end
    self.m_box = MatchManager:getBox(self.m_index)
    self:refresh()
end

function This:initTouchEvent()
	self.quxiao:setTouchEnded(function()
		g_scene:openDialog("Vip.VipPanel")
	end)

	self.baoxiang1:setTouch({
        onEnded = function()
            if self.m_slotState then
                self.m_parent:setSelectedIndex(self.m_index)
            end
        end,
        isScale = false,
	})
end

function This:onEnter()
    super.onEnter(self)
    if self.m_box then
        self.m_box:addObserver(self)
    end

    if self.m_vipModel then
        self.m_vipModel:addObserver(self)
    end
    GamePlayer:addObserver(self)
    VipManager:addObserver(self)
end

function This:onExit()
    VipManager:removeObserver(self)
    GamePlayer:removeObserver(self)
    if self.m_box then
        self.m_box:removeObserver(self)
    end
    
    if self.m_vipModel then
    	self.m_vipModel:removeObserver(self)
    end
    super.onExit(self)
end

function This:doNeedVip()
    return self.m_config.condition == TerrConditionType.kVipLevel
end

function This:doNeedLevel()
    return self.m_config.condition == TerrConditionType.kPlayerLev
end

function This:isNotLockedBy( cond1, cond2 )
    local switch = {
        [TerrConditionType.kNone] = function()
            return true
        end,
        [TerrConditionType.kPlayerLev] = function(arg)
        -- 君主等级达
            return  arg[1] <= GamePlayer:getLevel(), "" .. arg[1] .. Serialize[60705]
        end,
        [TerrConditionType.kVipLevel] = function(args)   
            local state = VipManager:getMemberState()
            local ret, min = false, nil
            for _, vlev in ipairs(args) do 
                ret = ret or state[vlev]
                if not min or min > vlev then
                    min = vlev
                end
            end
            return ret, VipName[min] .. Serialize[60706]
        end,
    }

    return switch[cond1](cond2)
end

function This:getSlotState()
    return self.m_slotState
end

function This:initBoxSlot()
    local state, desc = self:isNotLockedBy(self.m_config.condition, self.m_config.num)
    self.m_slotState = state
    self.dengjijiesuo:setVisible(not state)
    self.baoxiang1.xian:setVisible(not state)
    self.lock:setVisible(not state)
    if self:doNeedVip() then
    	self.quxiao:setVisible(not state)
    else
    	self.quxiao:setVisible(false)
    end
    self.boxicon:setVisible(false)
    self.baoxiang1.time:setVisible(false)

    if not state then
        self.dengjijiesuo:setString(desc)
    else
        self.depodengdai.text:setString(Serialize["in_free"])
    end

    local isVipOpen = self:doNeedVip()
    -- self.daojishizhuangshi:setVisible(state and isVipOpen)
    -- self.daojishizhuangshi2:setVisible(state and isVipOpen)
    self.daojishi:setVisible(state and isVipOpen)
end

function This:initBox( box )
	local node = self
    if (box ~= nil) and (self.m_slotState or (self:doNeedVip() and box:isOpening())) then
        local config = box:getConfig()
        local state = box:getState()
        node.boxicon:setVisible(true)
        node.baoxiang1.time:setVisible(state == BoxState.kOpening)
        
        -- node.quality:setImage("MenuScene/itemQuality" .. config.quality .. ".png")
        node.boxicon:setImage("MenuScene/" .. StateImg[state] .. config.quality .. ".png")
        node.depodengdai:setVisible(true)
        node.depodengdai.text:setString(StateDesc[state])
    else
        node.boxicon:setVisible(false)
        node.depodengdai:setVisible(self.m_slotState)
        node.quality:setImage("MenuScene/itemQuality1.png")
    end
end

function This:refresh()
    self:initBoxSlot()
	self:initBox(self.m_box)
end

function This:refreshTipUI()
    if self.m_box and self.m_slotState then
        local state = self.m_box:getState()
        self.depodengdai.text:setString(StateDesc[state])
    end
end

function This:setBox(box)
	if self.m_box then
    	self.m_box:removeObserver(self)
    end

    self.m_box = box 
    if self.m_box then
    	self.m_box:addObserver(self)
    end
    self:refresh()
end

function This:setSelected(value)
    if value ~= self.m_selected then
    	self.m_selected = value
    end
end

function This:updateTime(leftTime)
	if leftTime > 0 then 
		if self.m_selected then
			self.m_parent:updateTime(leftTime)
		end
		local hour = math.floor(leftTime / 3600)
	    local minute = math.floor(leftTime % 3600 / 60)
	    local second = math.floor(leftTime % 60)
	    self.baoxiang1.time:setString(string.format(Serialize["depot_BoxView_4"], hour, minute, second))
    end
end

function This:onBoxUpdateEnd( box )
    if self.m_box == box then
    	if self.m_selected then
			self.m_parent:refreshTimerUI(false)
            self.m_parent:refreshButtonUI(true)
		end
		self:refresh()
    end
end

function This:onBoxUpdateStart( box )
    if self.m_box == box then
    	if self.m_selected then
			self.m_parent:refreshTimerUI(true)
		end
		self.baoxiang1.time:setVisible(true)
    end
end

function This:onVipActivated( maxLevel )
    if self:doNeedVip() then
    	self:refresh()
    end
end

function This:onGetVipInfo()
    if self:doNeedVip() then
        if self.m_vipModel then
            self.m_vipModel:removeObserver(self)
        end
        self.m_vipModel = VipManager:getNewestModel()
        if self.m_vipModel then
            self.m_vipModel:addObserver(self)
        end
    	self:refresh()
    end
end

function This:updateVipTime( info )
    if info and info.time > 0 then 
        local leftTime = info.time
        self.daojishi:setString(GameUtil:formatDTime(Serialize["activity_BaseTabs_1_c"], leftTime))
    end 
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if self:doNeedLevel() then
        if field == PlayerAttr.level and not self.m_slotState then
            local state, desc = self:isNotLockedBy(self.m_config.condition, self.m_config.num)
            if self.m_slotState then
                self:initBoxSlot()
            end
        end
    end
end
