-- @Author: superm
-- @Date:   2018-05-19 11:37:52
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-07-13 14:53:56
-- @File Name: DepotBoxesTabNode.lua
-- @Description: 

local This = class(..., "Depot.DepotBaseTabNode")
local MatchManager = require("Match.MatchManager")
local DepotBoxView = require("Depot.DepotBoxView")
local GameUtil = require("Util.GameUtil")

function This:init(info)
    self.m_csNode = info.node
    self.m_csParentNode = info.parent
    self.m_selectItemId = 0
    self.m_selectItemIndex = 0
    self.m_boxView = {}
end

function This:isProps()
    return false
end

function This:isSpall()
    return false
end

function This:isEquip()
    return false
end

function This:getDepotItemList()
    return MatchManager:getPropsItemList()
end

function This:onEnter()
    self:initNode()
    self:initTouchEvent()
    MatchManager:addObserver(self)
end

function This:onExit()
    MatchManager:removeObserver(self)
end

function This:initNode()
    for i=1, 4 do
        local node = DepotBoxView:create({index = i, parent = self})
        self.m_boxView[i] = node
        self.m_csNode["box" .. i]:addChild(node)
    end
end

function This:initTouchEvent()
    if self.m_csNode.help then
        self.m_csNode.help:setTouchEnded(function ()
            g_scene:openDialog("UI.HelpTips", {text = Serialize["depot_box_help"], node = self.m_csNode.help})
        end)
    end

    self.m_csNode.open:setTouchEnded(function()
        if not self.m_item then
            return
        end

        local state = self.m_item:getState()
        if state == BoxState.kLock then
            local count, index = MatchManager:getBoxCountInOpening()
            print(count, index, self.m_selectItemIndex)
            if count <= 0 then
                MatchManager:onUnlockBoxRequest(self.m_item:getIndex())
            elseif index and index > 0 and index ~= self.m_selectItemIndex then 
                self:openBoxImmediately(self.m_item)
            end
        elseif state == BoxState.kOpening then
            self:openBoxImmediately(self.m_item)
        else
            MatchManager:onOpenBoxRequest(self.m_item:getIndex())
        end        
    end)
end

function This:openBoxImmediately( box )
    if box and box:getId() > 0 then
        local cost = box:getRecieveCost()
        local index = box:getIndex()
        if cost > 0 and index > 0 then
            g_scene:openDialog("UI.CostConfirmPanel", {
                cost = cost,
                desc = box:isOpening() and Serialize["depot_BoxesTab_1"] or Serialize["depot_BoxesTab_2"], 
                okHandler = function()
                    local box = MatchManager:getBox(index)
                    if box and box:getId() > 0 then
                        local state = box:getState()
                        if box:isOpening() then
                            MatchManager:onFinishBoxCDRequest(index)
                        elseif box:getState() == BoxState.kReceive then
                            MatchManager:onOpenBoxRequest(index)
                        elseif self.m_boxView[index]:getSlotState() then
                            MatchManager:onUnlockBoxRequest(index, true)
                            MatchManager:onFinishBoxCDRequest(index, true)
                            MatchManager:onOpenBoxRequest(index, true)
                        end
                    end
                end 
            })  
        end
    end
end

----------------------------------------------------------------

function This:setSelectedIndex( index )
    local box = MatchManager:getBox(index)
    self.m_item = box
    self.m_selectItemId = box and box:getId() or 0
    if self.m_selectItemIndex > 0 then
        self.m_boxView[self.m_selectItemIndex]:setSelected(false)
    end
    self.m_selectItemIndex = index
    self.m_boxView[index]:setSelected(true)

    self:refreshUI()
end

function This:resetNodes(index, visible)
    local basePartNames = {"quality", "icon", "name", "des" }
    local descPartNames = {"timeDesc", "timebg", "time", "cost", "costnum"}
    local buttonPartNames = {"open"}
    local nameList = {basePartNames, descPartNames, buttonPartNames}
    if not index or index <= 0 then
        self:setNodesVisible(nameList, false, self.m_csNode)
    else
        self:setNodesVisible({nameList[index]}, visible, self.m_csNode)
    end
--    self.m_csNode.choose:switchParent(self.m_csNode)
--    self.m_itemNode = nil
end

function This:refreshInfoUI(visible)
    self:refreshItemUI(visible)
    self:refreshDescUI(visible)
    self:refreshButtonUI(visible)
end

function This:refreshItemUI(visible)
    local csNode = self.m_csNode
    csNode.quality:setVisible(visible)
    csNode.icon:setVisible(visible)
    csNode.name:setVisible(visible)

    if not visible then
        return
    end
    local config = self.m_item:getConfig()
    -- csNode.quality:setImage("MenuScene/itemQuality" .. config.quality .. ".png")
    csNode.icon:setImage("MenuScene/boxLock" .. config.quality .. ".png")
    csNode.name:setString(config.name)
end

function This:refreshTimerUI(visible)
    self:resetNodes(2, visible)
end

function This:getDescNode()
    return self.m_csNode.des
end

function This:refreshDescUI(visible)
	local descNode = self:getDescNode()
    descNode:setVisible(visible)
    if not visible then
        return
    end
    local config = self.m_item:getConfig()
    local strCommon = config.desc
    if os.isWin32 then
        strCommon = "ID:".. tostring(self.m_item:getId()) .. "," .. strCommon
    end
    descNode:setStringWithColor(strCommon, 500)
end

function This:refreshButtonUI(visible)
    self.m_csNode.open:setVisible(visible)
    if not visible then
        return
    end

    local state = self.m_item:getState()
    if state == BoxState.kLock then
        self.m_csNode.open.text:setString(Serialize["depot_BoxesTab_3"])
    else
        if state == BoxState.kOpening then
            self.m_csNode.open.text:setString(Serialize["depot_BoxesTab_4"])
        else
            self.m_csNode.open.text:setString(Serialize["depot_BoxesTab_5"])
        end
    end
end

-- 默认选 
function This:refresh(cleanup)
    local count, index = MatchManager:getBoxCountInOpening()
    index = index or 1
    self:setSelectedIndex(index)
end

function This:refreshUI()
    self:resetNodes()
    self:refreshInfoUI(self.m_item ~= nil)
    self:refreshTimerUI(self.m_item and self.m_item:isOpening()) 
end

function This:updateTime(leftTime)
    self.m_csNode.time:setPercent(self.m_item:getProgress() * 100)
    self.m_csNode.timeDesc:setString(GameUtil:formatTime(Serialize["depot_BoxesTab_6"], leftTime))
    self.m_csNode.costnum:setString(self.m_item:getRecieveCost())
end

function This:updateItemSelect()
end

function This:onSelectItem(node, delayShow)
end

function This:_updateSelectNode()
    -- no need
end

function This:updateSelectNode(delayShow)
    -- no need
end

function This:setVisible(value)
    self.m_csNode:setVisible(value)
end


---------------------------------------------------------
function This:onUnlockBox( box )
    local index = box:getIndex()
    self.m_boxView[index]:refresh()
    if self.m_item == box then
        self:refreshButtonUI(self.m_item ~= nil)
    end

    for i=1, 4 do
        if i ~= index then
            self.m_boxView[i]:refreshTipUI()
        end
    end
end

function This:onFinishBoxCD( box )
    if self.m_item == box then
        self:refreshButtonUI(self.m_item ~= nil)
    end
end

function This:onOpenBox( box )
    if self.m_item == box then
        self:refresh()
    end
end

-- function This:onAddBox( box )
--     -- no need
-- end

function This:onRemoveBox( box )
    local index = box:getIndex()
    self.m_boxView[index]:setBox(nil)
end
