-- @Author: superm
-- @Date:   2018-05-18 10:25:08
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-01 02:01:06
-- @File Name: DepotEquipTabNode.lua
-- @Description: 

local This, super = class(..., "Depot.DepotBaseTabNode")
local EquipManager = require("Equip.EquipManager")
local FabaoManager = require("Fabao.FabaoManager")
local ItemUtil = require("Bag.ItemUtil")
local FabaoSkillView = require("Fabao.FabaoSkillView")
local GameUtil = require("Util.GameUtil")
local FabaoUtil = require("Fabao.FabaoUtil")
local SkillConfig = require("Metadatas.SkillConfig")
local FabaoConstants = require("Fabao.FabaoConstants")
local ItemNode = require("UI.ItemNode")

function This:init(info)
    super.init(self, info)
    --self:initNode()
    self:initTouchEvent()
end

--function This:initNode()
--    -- TODO
--end

function This:initTouchEvent()
    -- 法宝出售按钮
    self.m_csNode.fabao.sell:setTouchEnded(function()
        g_scene:openDialog("Depot.ItemSellUI", {item = self.m_item})
    end)

    -- 法宝培养按钮
    self.m_csNode.fabao.culture:setTouchEnded(function()
        g_scene:openDialog("Fabao.FabaoCultivatePanel", {fabaoUid = self.m_item:getUid(), sourceType = FabaoConstants.SourceType.Backpack})
    end)
end

function This:isProps()
    return false
end

function This:isSpall()
    return false
end

function This:isEquip()
    return true
end

local ChineseNumber = {
    Serialize[60012], Serialize[60013], Serialize[60014], Serialize[60015], Serialize[60016], Serialize[60097], Serialize[60018], Serialize[60019], Serialize[60020],
}

function This:getDepotItemList()
    local equipItemList = EquipManager:getEquipItemList()
    local fabaoItemList = FabaoManager:getAllUnequippedFabaoItemList()
    for _, fabao in ipairs(fabaoItemList) do
        table.insert(equipItemList, fabao)
    end
    table.sort(equipItemList, FabaoUtil.equipmentAndFabaoSortFunc)

    return equipItemList
end

function This:refreshInfoUI( visible )
    if self.m_item and not self.m_item:isEquip() then
    	return
    end
    super.refreshInfoUI( self, visible )
end

function This:refreshItemUI(visible)
	super.refreshItemUI(self, visible)
    self.m_csNode.backpack.equiplevel:setVisible(visible)
end

function This:getDescNode()
    return self.m_csNode.backpack.propdesc
end

function This:refreshDescUI(visible)
    super.refreshDescUI(self, visible)
    for i=1, 4 do
        local nameNode = self.m_csNode.backpack["propname" .. i]
        local valueNode = self.m_csNode.backpack["propvalue" .. i]
        nameNode:setVisible(visible)
        valueNode:setVisible(visible)
    end

    if not visible then
        return
    end

    self.m_csNode.backpack.equiplevel:setString("+" .. self.m_item:getStrengthLv())

    local attrList = self.m_item:getAddAttr()
    for i = 1, 4 do
        local nameNode = self.m_csNode.backpack["propname" .. i]
        local valueNode = self.m_csNode.backpack["propvalue" .. i]
        local attr = attrList[i] or {}
        local attrId, value = attr.attrId, attr.value
        if attrId and attrId > 0 then
            nameNode:setString(AttrName[attrId])
            valueNode:setString("+" .. math.floor(value))
            local width = nameNode:getContentWidth()
            local x, y = nameNode:getPosition()
            valueNode:setPosition(x + width, y)
        else
            nameNode:setVisible(false)
            valueNode:setVisible(false)
        end
    end

    local x, y = self.m_csNode.backpack.normaldesc:getPosition()
    self.m_csNode.backpack.propdesc:setPosition(x, y - (#attrList * 30))
end

function This:refreshButtonUI(visible)
    self.m_csNode.backpack.sell:setVisible(visible)
    self.m_csNode.backpack.strengthen:setVisible(visible)
    self.m_csNode.backpack.ascend:setVisible(false)
    if not visible then
        return
    end

    local state = self.m_item:canUsedForStrength()
    if state then
        self.m_csNode.backpack.strengthen:clearShader()
    else
        self.m_csNode.backpack.strengthen:setGrayShader()
    end    

    state = self.m_item:canUsedForAscend()
    if state then
        self.m_csNode.backpack.ascend:clearShader()
    else
        self.m_csNode.backpack.ascend:setGrayShader()
    end 
end

function This:refreshFabaoInfo()
    local fabaoRootNode = self.m_csNode.fabao

    -- 法宝名
    fabaoRootNode.fabaoname:setString(self.m_item:getName())
    fabaoRootNode.fabaoname:setColorByID("Q".. ItemUtil.getQuality(self.m_item:getId()))

    -- 左上角图标
    local tag = 1545726081;
    local fabaoItemView = fabaoRootNode.fabao:getChildByTag(tag)
    if not fabaoItemView then
        fabaoItemView = ItemNode:create(self.m_item:getId(), nil, nil, false)
        fabaoItemView:setAnchorPoint(0.5,0.5)
        fabaoItemView:setTag(tag)
        fabaoRootNode.fabao:addChild(fabaoItemView)
    end
    local fabao = FabaoManager:getFabao(self.m_item:getUid())
    fabaoItemView:initItem(self.m_item:getId())
    fabaoItemView:setToFabao(fabao:getLevel(), fabao:getJinglianDegree(), fabao:getStarLevel())

    -- 法宝描述
    fabaoRootNode.text1:setString(ItemUtil.getComment(self.m_item:getId()), 255)

    -- 法宝技能
    local tag = 1545723785
    local fabaoSkillView = fabaoRootNode.teji:getChildByTag(tag)
    if not fabaoSkillView then
        fabaoSkillView = FabaoSkillView:create()
        fabaoSkillView:setFabao(self.m_item:getUid())
        fabaoSkillView:setTag(tag)
        fabaoRootNode.teji:addChild(fabaoSkillView)
    else
        fabaoSkillView:setFabao(self.m_item:getUid())
    end

    -- 基础属性
    local levelAttributeList = fabao:getAdditionalAttributeListOnLevel()
    for index = 1, 3 do
        local attributeNameView = fabaoRootNode.scrollView["name"..index]
        local attributeValueView = fabaoRootNode.scrollView["num"..index]
        local attribute = levelAttributeList[index]
        if attribute then
            local attributeName, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeName)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end

    -- 精炼属性
    local jinglianDegreeAttributeList = fabao:getAdditionalAttributeListOnJinglianDegree()
    for index = 1, 3 do
        local attributeNameView = fabaoRootNode.scrollView["name"..(index+3)]
        local attributeValueView = fabaoRootNode.scrollView["num"..(index+3)]
        local attribute = jinglianDegreeAttributeList[index]
        if attribute then
            local attributeName, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeName)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end

    -- 升星效果
    local currentStarLevel = fabao:getStarLevel()
    for index = 1, 5 do
        local color
        if index <= currentStarLevel then
            color = "<Q5>"
        else
            color = "<Study1>"
        end

        local formatStr = Serialize[60708]
        local newEffectText
        local newEffect, effectType = FabaoUtil.getDifferentBetweenStarLevel(fabao:getItemId(), index-1, index)
        if effectType == FabaoUtil.Type.Attribute then
            local attributeName, attributeValueStr = GameUtil:getAttrStr(newEffect[1], newEffect[2])
            newEffectText = string.format("%s%s", attributeName, attributeValueStr)
        elseif effectType == FabaoUtil.Type.Skill then
            local skillConfig = SkillConfig[newEffect]
            newEffectText = string.format(Serialize[60709], skillConfig.name, newEffect % 10)
        else
            formatStr = Serialize[60710]
            newEffectText = ""
        end

        local view = fabaoRootNode.scrollView[""..index.."xing"]
        view:setStringWithColor(color..string.format(
                formatStr,
                ChineseNumber[index],
                FabaoUtil.getStarLevelAttributesEnhanciveFactor(fabao:getItemId(), index)*100, -- TODO 这里显示的是全量，而不是差异，是否要修改？
                newEffectText
        ))
    end

    -- 出售价格
    if self.m_item:canBeSold() then
        fabaoRootNode.priceicon:setImage(ItemUtil.getPriceImage(self.m_item:getId()))
        fabaoRootNode.price:setString(self.m_item:getPrice())
        fabaoRootNode.priceicon:setVisible(true)
        fabaoRootNode.price:setVisible(true)
    else
        fabaoRootNode.priceicon:setVisible(false)
        fabaoRootNode.price:setVisible(false)
    end

    --
    -- TODO
end
