-- @Author: superm
-- @Date:   2017-10-19 20:25:00
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-10-23 18:26:31
-- @File Name: DepotItemView.lua

local This , super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/DepotPropTips.csb"

local ItemUtil = require("Bag.ItemUtil")
local FabaoManager = require("Fabao.FabaoManager")

function This:init(info)
    self.m_inited = false
    self:setInfo(info)
    local w , h = self.quality:getContentSize()
    self:setContentSize(w, h)
end

function This:initNode()
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self:setTouch({
            getRect = function (touch)
                return touch.sizeNode.quality:getBoundingBox()
            end,
            onEnded = function ()
                if self.m_handler then
                    self:m_handler(self.item)
                end
            end,
            isContainer = true,
            isDynamic = true,
            isScale = false,
        })  
        self.m_inited = true
    end
end

function This:onEnter()
    super.onEnter(self)
    FabaoManager:addObserver(self)
end

function This:onExit()
    FabaoManager:removeObserver(self)
    super.onExit(self)
end

function This:setInfo(info)
    if info then
        self.item = info.item
        self.m_index = info.index
        self.m_fcount = info.fcount or 0
        self.m_aloneMode = info.aloneMode
        self.m_selected = info.is_selected
        self.m_handler = info.handler
    end
end

function This:getTipInfo()
   return {fileName = "UI.ItemTips", itemId = self.item:getId(), itemType = self.item:getType()} 
end

function This:setModel(item, index, fcount, selected, isForceUpdate)
	if item:getUid() ~= self.item:getUid() or isForceUpdate then
        self.m_fcount = fcount or 0
        self.m_index = index
        self.m_selected = selected
		self.item = item
		self:refresh()
	end
end

function This:refresh(cleanup)
    if cleanup then
        print(tostring(self.item:getId()))
    end
    self.quality:setImage(ItemUtil.getQualityImage(self.item:getId()))
    self.icon:setImage(ItemUtil.getIconImage(self.item:getId()))
    local itemCount = self.item:getCount() - self.m_fcount
    if not self.m_aloneMode and itemCount > 1 then
        self.count:setString(itemCount)
        self.count:setVisible(true)
        self.daojushuliangdi_1:setVisible(true)
    else
        self.count:setVisible(false)
        self.daojushuliangdi_1:setVisible(false)
    end
    self.equiplv:setVisible(self.item:isEquip())
    self.pieceicon:setVisible(self.item:isSpall())
    self.state:setVisible(self.item:isSpall())
    self.equiplevel:setVisible(self.item:isEquip())
    self.icon:setVisible(true)
    self.quality:setVisible(true)
    self.fabao:setVisible(false)
    self.star:setVisible(false)
    if self.item:isEquip() then
        local strengthLev = self.item:getStrengthLv()
        local level = self.item:getLevel()
        if strengthLev then
            self.equiplevel:setString("+" .. tostring(strengthLev))
        else
            self.equiplevel:setString("")
        end
        if level then
            self.equiplv:setString(tostring(level))
        else
            self.equiplv:setString("")
        end
    elseif self.item:isSpall() then
        local state, isGotHero = self.item:canUsed()
        if not state then
            if isGotHero then
                self.state.propstate1:setVisible(false)
                self.state.propstate2:setVisible(true)
            else
                self.state:setVisible(false)
            end
        else
            self.state.propstate1:setVisible(true)
            self.state.propstate2:setVisible(false)
        end
    elseif self.item.isFabao and self.item:isFabao() then
        self.fabao:setVisible(true)
        local fabao = FabaoManager:getFabao(self.item:getUid())
        self.fabao.jinglian:setString("+"..fabao:getJinglianDegree())
        self.fabao.dengji:setString(fabao:getLevel())
        self.fabao.xingshu:setString(fabao:getStarLevel())
    elseif self.item:checkCommonType(ItemCommonType.kTactics) then
        local starLv = self.item:getStar()
        self.star:setVisible(true)
        for i=1, self.item:getMaxStar() do
            local node = self.star[string.format("star%d", i)]
            if node then
                node:setVisible(i <= starLv)
            end
        end
    end

    self.choose:setVisible(self.m_selected)
end

function This:setSelected( value )
    self.m_selected = value
    self.choose:setVisible(self.m_selected)
end

function This:getIndex()
    return self.m_index
end

function This:onFabaoDataChanged(fabaoUid)
    if self.item and self.item.isFabao and self.item:isFabao() and (fabaoUid == self.item:getUid()) then
        self:refresh()
    end
end


--endregion