local This, super = class(..., "Dialog")
local PlanShopManager = require("PlanShop.PlanShopManager") 
local DepotManager = require("Depot.DepotManager")
local EquipManager = require("Equip.EquipManager")
local FabaoManager = require("Fabao.FabaoManager")
local RankListLayer = require("Rank.RankListLayer")
local RuneManager = require("Equip.RuneManager")
local ItemUtil = require("Bag.ItemUtil")

local DepotBaseTabNode = require("Depot.DepotBaseTabNode")
local DepotEquipTabNode = require("Depot.DepotEquipTabNode")
local DepotSpallTabNode = require("Depot.DepotSpallTabNode")
local DepotRuneTabNode = require("Depot.DepotRuneTabNode")
-- local DepotBoxesTabNode = require("Depot.DepotBoxesTabNode")

This.openSound = "Open_Ui.mp3"
local kBagLineCount = 4

function This:initData()
    self.m_typeName = nil
end

function This:initNode(params)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/DepotProp.csb")
    self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

    self.m_listLayer = RankListLayer:create({
        itemWidth = 87,
        itemHeight = 87,
        spaceX = 6,
        spaceY = 12,
        column = 4,
        scrollView = self.m_csNode.beibao.scrollView,
    })
    
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.m_csNode.beibao.scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)
    
    local info = { node = self.m_csNode.beibao, parent = self.m_csNode,  list = self.m_listLayer }
    self.m_propsTabNode= DepotBaseTabNode:create(info)
    self.m_equipTabNode= DepotEquipTabNode:create(info)
    self.m_spallTabNode= DepotSpallTabNode:create(info)
    self.m_runeTabNode= DepotRuneTabNode:create(info)

    info.node = self.m_csNode.box
    info.node:setVisible(false)
    -- self.m_csNode.tep4:setVisible(false)
    -- self.m_boxesTabNode= DepotBoxesTabNode:create(info)
    -- self.m_boxesTabNode:setVisible(false)

    local tabList = {
        {name = "Props", node = self.m_propsTabNode}, 
        {name = "Equip", node = self.m_equipTabNode}, 
        {name = "Spall", node = self.m_spallTabNode},
        {name = "Rune", node = self.m_runeTabNode},
        -- {name = "Boxes", node = self.m_boxesTabNode},
    }

    for i, info in ipairs(tabList) do
        self:addTabButton(info.name, self.m_csNode["tep" .. i], function ()
            self.m_curTab:setVisible(false)
            self.m_curTab = info.node
            self.m_curTab:setVisible(true)
            self:onSwitchType(info.name)
        end)
    end

    if params then
        self.m_curSelectIndex = params.tabs or 1
    else
        self.m_curSelectIndex = 1
    end
    local info = tabList[self.m_curSelectIndex]
    self.m_curTab = info.node 
    self:selectTabButton(info.name)
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    local backpack = csNode.beibao.backpack
    local fushi = csNode.beibao.fushi
    csNode.close:setTouchEnded(function ()
        self:close()
    end)
    backpack.sell:setTouchEnded(function ()
        local curItem = self.m_curTab:getSelectedItem()
        if curItem then
            g_scene:openDialog("Depot.ItemSellUI", {item = curItem})
        end
    end)
    backpack.use:setTouchEnded(function ()
        local curItem = self.m_curTab:getSelectedItem()

        if curItem then
            if curItem:canUsed(true) then
                local packList = ItemUtil.getPackageList(curItem:getId())
                if curItem:isPack() and packList then
                    g_scene:openDialog("Depot.ItemSelectPanel", {item=curItem, itemList = packList})
                else
                    g_scene:openDialog("Depot.ItemUseUI", {item=curItem})
                end
            end
        end
    end)
    backpack.strengthen:setTouchEnded(function ()
        local curItem = self.m_curTab:getSelectedItem()
        if curItem and curItem:canUsedForStrength(true) then
            local index = self.m_curTab:getSelectedIndex()
            g_scene:openDialog("Equip.EquipStrengthPanel", {equipUid=curItem:getUid(), tab = "backpack", hint = index})
        end
    end)
    backpack.ascend:setTouchEnded(function ()
        local curItem = self.m_curTab:getSelectedItem()
        if curItem and curItem:canUsedForAscend(true) then
            local index = self.m_curTab:getSelectedIndex()
            g_scene:openDialog("Equip.EquipStrengthPanel", {equipUid=curItem:getUid(), tab = "backpack", subTab = "Ascend", hint = index})
        end
    end)
    backpack.synthesis:setTouchEnded(function ()
        local curItem = self.m_curTab:getSelectedItem()
        if curItem then
            local result, isGotHero = curItem:canUsed()
            if result then
                if curItem:isHeroSpall() then
                    DepotManager:synthesizeSpall(curItem)
                else
                    g_scene:openDialog("Depot.ItemSynthesizeUI", {item = curItem})
                end
            elseif not isGotHero then
                g_scene:showTips(backpack.spalllacknote:getString())
            else
                g_scene:showTips(backpack.spallnote:getString())
            end
        end
    end)

    fushi.culture:setTouchEnded(function ()
        local curItem = self.m_curTab:getSelectedItem()
        local function closeFun()
            self:close()
        end
        local info = {data = curItem,type =RuneSkillStart.kBag,closeFun = closeFun}
        g_scene:openDialog("Equip.RuneSkillPanel",info)
    end)
    fushi.sell:setTouchEnded(function ()
        local curItem = self.m_curTab:getSelectedItem()
        RuneManager:runeSell(curItem)
    end)

end

function This:initListener()
    -- self.m_boxesTabNode:onEnter()
    
    RuneManager:addObserver(self)
    EquipManager:addObserver(self)
    DepotManager:addObserver(self, {
        itemUpdate = function(self, itemList)
            local cleanup = false
            if itemList and #itemList > 0 then
                cleanup = true
            end
            self.m_curTab:refresh(true)
        end,
        sellItem = function(self, itemList)
            if itemList and #itemList > 0 then
                g_scene:showReward(itemList)
            end
        end,
        useItem = function(self, rewardList)
            if rewardList and #rewardList > 0 then
                g_scene:showReward(rewardList)
            end
        end,
        synthesizeSpall = function(self, rewardList)
            g_scene:showTips(Serialize["depot_MainUI_1_c"])
            if rewardList and #rewardList > 0 then
                g_scene:showReward(rewardList)
            end
            self.m_curTab:refresh()
        end,
    })
    PlanShopManager:addObserver(self)
    FabaoManager:addObserver(self)
end

function This:onStrengthLevel(uid)
    self.m_curTab:refresh()
end

function This:onEquipUpLevel()
    self.m_curTab:refresh()    
end

function This:onExit()
    self.m_propsTabNode:clear()
    self.m_equipTabNode:clear()
    self.m_spallTabNode:clear()
    self.m_runeTabNode:clear()

    -- self.m_boxesTabNode:onExit()
    super.onExit(self)
    FabaoManager:removeObserver(self)
    PlanShopManager:removeObserver(self)
    --有顺序问题，不能乱
    DepotManager:removeObserver(self)
    RuneManager:removeObserver(self)
    EquipManager:removeObserver(self)
end

function This:onSwitchType(typeName)
    if self.m_typeName ~= typeName then
        self.m_typeName = typeName
        self.m_curTab:refresh()
    end
end

function This:onUsePlan(msg)
    if msg.planId == JiCeType.kCaiYuanGunGun or msg.planId == JiCeType.kWuGuFengDeng then
        if msg.rewardList and #msg.rewardList > 0 then
            g_scene:showReward(msg.rewardList)
        end
    end
end

function This:onUnequippedFabaosChanged()
    if self.m_curTab == self.m_equipTabNode then
        self.m_curTab:refresh()
    end
end