-- @Author: superm
-- @Date:   2018-05-19 14:52:07
-- @Last Modified by:   superm
-- @Last Modified time: 2018-05-19 15:34:02
-- @File Name: DepotQueue.lua
-- @Description: 

local This = class(...)

function This:create()
	local o = {}
	setmetatable(o,self)
	o:init()
	return o
end

function This:init()
	self.m_first = 0
	self.m_last = -1
	self.m_list = {}
end

function This:size()
	if self.m_first > self.m_last then
		return 0
	else
		return self.m_last + 1 - self.m_first
	end
end

function This:empty()
	return self.m_first > self.m_last
end

function This:get(index)
	local self = self.m_first + index
	return self.m_list[index]	
end

function This:pushFront(value)
	local first = self.m_first - 1
	self.m_first = first
	self.m_list[first] = value
end

function This:pushBack(value)
	local last = self.m_last + 1
	self.m_last = last
	self.m_list[last] = value
end

function This:popFront()
	local first = self.m_first
	if first > self.m_last then return end
	local value = self.m_list[first]
	self.m_list[first] = nil		-- to allow garbage collection
	self.m_first = first + 1
	return value
end

function This:popBack()
	local last = self.m_last
	if self.m_first > last then return end
	local value = self.m_list[last]
	self.m_list[last] = nil		-- to allow garbage collection
	self.m_last = last - 1
	return value
end

function This:clear(handler)
	if not handler then
		for index = self.m_first, self.m_last do
			self.m_list[index] = nil
		end
	else
		for index = self.m_first, self.m_last do
			handler(self.m_list[index])
			self.m_list[index] = nil
		end
	end
	self.m_first = self.m_last + 1
end

function This:getBack()
	local last = self.m_last
	return self.m_list[last]	
end

function This:getFront()
	local first = self.m_first
	return self.m_list[first]
end
