-- @Author: superm
-- @Date:   2018-03-08 21:03:52
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-25 18:05:28
-- @File Name: ItemSelectPanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local ViewUtil = require("UI.ViewUtil")

This.openSound = "Open_Ui.mp3"

function This:initData(info)
    self.m_useCount = 0
    self.m_item = info.item
    self.m_itemList = info.itemList
    self.m_itemCount = #info.itemList
	self.m_okHandler = info.okHandler
	self.m_inited = false
end

function This:initNode()
 	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/OptionBox.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    self.m_csNode.condition:setVisible(false)
    self.m_csNode.button:setVisible(false)
    ViewUtil:blinkView(self.m_csNode.condition, true)
    self.m_itemNodeList = {}
    self:refreshItemList()
    self:updateUseCount(1)
end

function This:onEnter()
    super.onEnter(self)
end

function This:refreshItemList()
    if self.m_itemCount <= 4 then
        self.m_csNode.scrollView:removeFromParent()
        self.m_csNode.scrollView = nil
        local w = 10
        for i , info in ipairs(self.m_itemList) do
            local node = ItemNode:create(info[1], info[2], nil, false)
            node:setCascadeOpacityEnabled(true)
            node:setAnchorPoint(0,0.5)
            node:setPositionX(w)
            local nodeW, nodeH = node:getContentSize()
            w = w + nodeW + 10

            local selectedBg = cc.Sprite:create("MenuScene/choose1.png")
            node:addChild(selectedBg)
            selectedBg:setVisible(false)
            node.m_selectedBg = selectedBg
            selectedBg:setPosition(nodeW/2, nodeH/2)

            self.m_itemNodeList[i] = node
            self.m_csNode.itemtipslist:addChild(node)
        end
        self.m_csNode.itemtipslist:setPositionX(- w / 2)
    else
        self.m_csNode.scrollView:setDirection("horizontal")
        self.m_csNode.scrollView.m_direction = "right"
        self.m_csNode.scrollView:setTouchEnabled(true)
        for i , info in ipairs(self.m_itemList) do
            local node = ItemNode:create(info[1], info[2], nil, false)
            node:setAnchorPoint(0.5,0.5)
            local w , h = node:getContentSize()            
            local selectedBg = cc.Sprite:create("MenuScene/choose1.png")
            node:addChild(selectedBg)
            selectedBg:setVisible(false)
            node.m_selectedBg = selectedBg
            selectedBg:setPosition(w/2, h/2)

            self.m_itemNodeList[i] = node
            self.m_csNode.scrollView:pushBackCustomItem(node, w, h, 0.5)
        end
    end
end


function This:handleHelpTips(node, getInfoHandler)
    local tips = nil
    node:setTouch({
        onBegan = function(_,touch)
            local rect = node:getBoundingBox()
            if rect:isContainsPoint(touch) then
                local info = getInfoHandler()
                if info then
                	if not info.node then
                		info.node = node
                	end
                    tips = g_scene:openDialog(info.fileName or "UI.HelpTips", info)
                    return true
                end
            end
            return false
        end,
        onEnded = function()
            if tips and tips[0] ~= 0 then
                tips:close()
                tips = nil
            end
            -- self:setTimer(0.1, function ()
                local info = getInfoHandler()
                self:selectItemByIndex( info.index )
            -- end)
        end,
        isScale = false,
        isDynamic = true,
		isSwallow = false,
        onCanceled = function()
            if tips and tips[0] ~= 0 then
                tips:close()
                tips = nil
            end
        end,
    })
end

function This:selectItemByIndex( index )
    if self.m_selectedIndex == nil then
        ViewUtil:blinkView(self.m_csNode.condition, false)
        self.m_csNode.button:setVisible(true)
    end

    if self.m_selectedIndex ~= index then
        self.m_selectedIndex = index
        for i, view in ipairs(self.m_itemNodeList) do
            view.m_selectedBg:setVisible(i == self.m_selectedIndex)
        end        
    end

end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_csNode.button:setTouch({
            -- getRect = function (touch)
            --     local rect = touch.sizeNode:getBoundingBox()
            --     return rect
            -- end,
            onEnded = function()
                if self.m_selectedIndex and  self.m_selectedIndex > 0 and self.m_useCount>0 then
                    DepotManager:useItem(self.m_item, self.m_useCount, self.m_itemList[self.m_selectedIndex][1])
                    self:close()
                end
            end,
            -- isContainer = true,
            isDynamic = true,
            isScale = false,
        })

        self.m_csNode.close:setTouchEnded(function ()
            self:close()
        end)

        for i, view in ipairs(self.m_itemNodeList) do
            self:handleHelpTips(view, function ()
                return {fileName = "UI.ItemTips", itemId = view.m_id, itemType = view.m_itemType, index = i}
            end)
        end
        self.m_csNode.reduce:setTouch({
            onBegan = function()
                self.m_csNode.reduce.hasAlter = false
                self.m_csNode.reduce:runAction(cc.RepeatForever(cc.Sequence(cc.DelayTime(0.25), cc.CallFunc(function()
                    self.m_csNode.reduce.hasAlter = true
                    self:updateUseCount(self.m_useCount - 1)
                end))))
            end,
            onEnded = function()
                self.m_csNode.reduce:stopAllActions()
                if not self.m_csNode.reduce.hasAlter then
                    self:updateUseCount(self.m_useCount - 1)
                end
                self.m_csNode.reduce.hasAlter = false
            end,
        })
        self.m_csNode.increase:setTouch({
            onBegan = function()
                self.m_csNode.increase.hasAlter = false
                self.m_csNode.increase:runAction(cc.RepeatForever(cc.Sequence(cc.DelayTime(0.25), cc.CallFunc(function()
                    self.m_csNode.increase.hasAlter = true
                    self:updateUseCount(self.m_useCount + 1)
                end))))
            end,
            onEnded = function()
                self.m_csNode.increase:stopAllActions()
                if not self.m_csNode.increase.hasAlter then
                    self:updateUseCount(self.m_useCount + 1)
                end
                self.m_csNode.increase.hasAlter = false
            end,
        })
        self.m_csNode.max:setTouchEnded(function()
            self:updateUseCount(self.m_item:getCount())
        end)
        self.m_csNode.addten:setTouchEnded(function()
            self:updateUseCount(self.m_useCount + 10)
        end)
        self.m_csNode.reduceten:setTouchEnded(function()
            self:updateUseCount(self.m_useCount -10)
        end)
        self.m_inited = true
    end
end

function This:updateUseCount(count)
    count = math.min(count, self.m_item:getCount())
    count = math.max(count, 1)
    if count ~= self.m_useCount then
        self.m_useCount = count
        self.m_csNode.sellcount:setString(self.m_useCount)
    end
end