local This, super = class(..., "Dialog")
local ItemUtil = require("Bag.ItemUtil")

This.openSound = "Open_Ui.mp3"
This.isQuickExit = true

function This:initData(info)
    self.m_manager = require("Depot.DepotManager")
    self.m_item = info.item
    self.m_sellCount = 0
    self.m_sellPrice = self.m_item:getPrice()
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/DepotPropSell.csb")
    self.m_csNode:setPosition(cc.visible.center.x, cc.visible.center.y)
    self:addChild(self.m_csNode)
    local item = self.m_item
    local csNode = self.m_csNode
    csNode.quality:setImage(ItemUtil.getQualityImage(item:getId()))
    csNode.icon:setImage(ItemUtil.getIconImage(item:getId()))
    csNode.priceicon:setImage(ItemUtil.getPriceImage(item:getId()))
    csNode.price:setString(self.m_sellPrice)
    csNode.name:setString(item:getName())
    csNode.count:setString(item:getCount())
    csNode.pieceicon:setVisible(item:isSpall())
    csNode.equiplevel:setVisible(item:isEquip())
    if item:isEquip() then
        csNode.equiplevel:setString("+" .. item:getStrengthLv())
    end
    self:updateSellCount(1)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    self.m_csNode.sellbotton:setTouchEnded(function()
        if self.m_sellCount > 0 then
            self.m_manager:sellItem(self.m_item, self.m_sellCount)
            self:close()
        end
    end)
    self.m_csNode.reduce:setTouch({
        onBegan = function()
            self.m_csNode.reduce.hasAlter = false
            self.m_csNode.reduce:runAction(cc.RepeatForever(cc.Sequence(cc.DelayTime(0.25), cc.CallFunc(function()
                self.m_csNode.reduce.hasAlter = true
                self:updateSellCount(self.m_sellCount - 1)
            end))))
        end,
        onEnded = function()
            self.m_csNode.reduce:stopAllActions()
            if not self.m_csNode.reduce.hasAlter then
                self:updateSellCount(self.m_sellCount - 1)
            end
            self.m_csNode.reduce.hasAlter = false
        end,
    })
    self.m_csNode.increase:setTouch({
        onBegan = function()
            self.m_csNode.increase.hasAlter = false
            self.m_csNode.increase:runAction(cc.RepeatForever(cc.Sequence(cc.DelayTime(0.25), cc.CallFunc(function()
                self.m_csNode.increase.hasAlter = true
                self:updateSellCount(self.m_sellCount + 1)
            end))))
        end,
        onEnded = function()
            self.m_csNode.increase:stopAllActions()
            if not self.m_csNode.increase.hasAlter then
                self:updateSellCount(self.m_sellCount + 1)
            end
            self.m_csNode.increase.hasAlter = false
        end,
    })
    self.m_csNode.max:setTouchEnded(function()
        self:updateSellCount(self.m_item:getCount())
    end)
end

function This:updateSellCount(count)
    count = math.min(count, self.m_item:getCount())
    count = math.max(count, 1)
    if count ~= self.m_sellCount then
        self.m_sellCount = count
        self.m_csNode.sellcount:setString(self.m_sellCount)
        self.m_csNode.price:setString(self.m_sellPrice * self.m_sellCount)
    end
end