local This, super = class(..., "Dialog")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
This.openSound = "Open_Ui.mp3"
This.isQuickExit = true

function This:initData(info)
    self.m_item = info.item
    self.m_useCount = 0
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/DepotPropUse.csb")
    self.m_csNode:setPosition(cc.visible.center.x, cc.visible.center.y)
    self:addChild(self.m_csNode)
    local item = self.m_item
    local csNode = self.m_csNode
    csNode.quality:setImage(ItemUtil.getQualityImage(item:getId()))
    csNode.icon:setImage(ItemUtil.getIconImage(item:getId()))
    csNode.name:setString(item:getName())
    csNode.count:setString(item:getCount())
    csNode.pieceicon:setVisible(item:isSpall())
    csNode.equiplevel:setVisible(item:isEquip())
    if item:isEquip() then
        csNode.equiplevel:setString("+" .. item:getStrengthLv())
    end
    csNode.desc:setString(item:getComment(), AppText:isChinese() and 360 or 560)
    self:updateUseCount(1)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self:initUseButtonTouchEvent()

    self.m_csNode.reduce:setTouch({
        onBegan = function()
            self.m_csNode.reduce.hasAlter = false
            self.m_csNode.reduce:runAction(cc.RepeatForever(cc.Sequence(cc.DelayTime(0.25), cc.CallFunc(function()
                self.m_csNode.reduce.hasAlter = true
                self:updateUseCount(self.m_useCount - 1)
            end))))
        end,
        onEnded = function()
            self.m_csNode.reduce:stopAllActions()
            if not self.m_csNode.reduce.hasAlter then
                self:updateUseCount(self.m_useCount - 1)
            end
            self.m_csNode.reduce.hasAlter = false
        end,
    })
    self.m_csNode.increase:setTouch({
        onBegan = function()
            self.m_csNode.increase.hasAlter = false
            self.m_csNode.increase:runAction(cc.RepeatForever(cc.Sequence(cc.DelayTime(0.25), cc.CallFunc(function()
                self.m_csNode.increase.hasAlter = true
                self:updateUseCount(self.m_useCount + 1)
            end))))
        end,
        onEnded = function()
            self.m_csNode.increase:stopAllActions()
            if not self.m_csNode.increase.hasAlter then
                self:updateUseCount(self.m_useCount + 1)
            end
            self.m_csNode.increase.hasAlter = false
        end,
    })
    self.m_csNode.max:setTouchEnded(function()
        self:updateUseCount(self.m_item:getCount())
    end)
    self.m_csNode.addten:setTouchEnded(function()
        self:updateUseCount(self.m_useCount + 10)
    end)
    self.m_csNode.reduceten:setTouchEnded(function()
        self:updateUseCount(self.m_useCount -10)
    end)

end

function This:initUseButtonTouchEvent()
    self.m_csNode.usebotton:setTouchEnded(function()
        if self.m_useCount > 0 then
            if not self.m_item:isPlan() then
                DepotManager:useItem(self.m_item, self.m_useCount)
            else
                self.m_item:useAsPlan(self.m_useCount)
            end
            self:close()
        end
    end)
end

function This:updateUseCount(count)
    count = math.min(count, self.m_item:getCount())
    count = math.max(count, 1)
    if count ~= self.m_useCount then
        self.m_useCount = count
        self.m_csNode.usecount:setString(self.m_useCount)
    end
end