-- @Author: superm
-- @Date:   2018-06-22 15:51:46
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-10-23 18:27:43
-- @File Name: PropChoosePanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local RankPageListLayer = require("Rank.RankPageListLayer")
local DepotManager = require("Depot.DepotManager")
local DepotItemView = require("Depot.DepotItemView")
local GameUtil = require("Util.GameUtil")


function This:getValueMap(t)
	local ret = {}
	for _, v in ipairs(t) do
		if v ~= 0 then
			ret[v] = (ret[v] or 0) + 1
		end
	end
	return ret
end

function This:getItemList( info )
	local checkHandler = info.checkHandler
	if not checkHandler then
		checkHandler = function (item)
			return item:isPassiveSkill()
		end
	end
	local filterList
	if not info.valueHandler then
		filterList = self:getValueMap(info.filter)
	else
		filterList = info.valueHandler(info.filter)
	end

	local idsList, count, icount = nil, 0, 0
	if not info.alone then
		idsList, count, icount = DepotManager:getItemListByFilter(function (item)
			local ret = checkHandler(item)
			local num = 0
			if ret then 
				local itemId = item:getId()
				if item:isEquip() or item:isFabao() or item:isLordEquip() or item:isInscription() or item:isRune() or item:checkCommonType(ItemCommonType.kTactics) then
					itemId = item:getUid()
				end
				num = filterList[itemId] or 0
			end
			return ret, num
		end, info.compare)
	else
		idsList, count, icount = DepotManager:getItemIdsListByFilter(function (item)
			local ret = checkHandler(item)
			local num = 0
			if ret then 
				local itemId = item:getId()
				if item:isEquip() or item:isFabao() or item:isLordEquip() or item:isRune() or item:checkCommonType(ItemCommonType.kTactics) then
					itemId = item:getUid()
				end
				num = filterList[itemId] or 0
			end
			return ret, num
		end, info.compare)
	end
	return idsList, count, icount
end

function This:initByInfo( info )
	self.m_aloneMode = info.alone
	self.m_spaceX = info.spaceX or 5

	local filterList
	if not info.valueHandler then
		filterList = self:getValueMap(info.filter)
	else
		filterList = info.valueHandler(info.filter)
	end
	self.m_filterList = filterList
	self.m_itemList, self.m_count = self:getItemList(info)


	self.m_filename = info.filename or "MenuScene/NodeUI/BlackSmithPassiveSkill.csb"
	self.m_selectedItems = {}
	self.m_selectIndex = 0
end

function This:initNode(info)

	self:initByInfo(info)
	self.m_csNode = cc.Node:createFromCSFile(self.m_filename)
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	local totalWidth = self.m_csNode.scrollView:getContentWidth()
	self.m_layerLayer = RankPageListLayer:create({
		itemWidth = 80,
		itemHeight = 80,
        spaceX = self.m_spaceX,
        spaceY = 5,
        column = math.floor(totalWidth / (80 + self.m_spaceX)),
        scrollView = self.m_csNode.scrollView,
		dataCounter = function ()
	        return self.m_count
        end,
        viewCreator =  function(index, node)
        	if self.m_count < index then
        		return
        	end

        	local item = self.m_itemList[index]
	        if item then
	        	local fcount = 0
	        	local selected = false
	        	if not self.m_aloneMode then
		        	local itemId = item:getId()
    				if item:isEquip() or item:isFabao() or item:isInscription() or item:checkCommonType(ItemCommonType.kTactics) then
						itemId = item:getUid()
					end
	        		selected = self.m_selectedItems[itemId] ~= nil
		        	fcount = self.m_filterList[itemId] or 0
		        end

		        if not node then
			        return self:getChooseItemView(item, index, fcount, selected)
			    else
		        	node:setModel(item, index, fcount, selected)
		        end
		    end
	    end,
    })

	self.m_layerLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_layerLayer:getContentSize()
	self.m_csNode.scrollView:pushBackCustomItem(self.m_layerLayer, w, h,0)
	self.m_layerLayer:setDataList()

	if self.m_csNode.text then
		self.m_csNode.text:setVisible(self.m_count <= 0)
	end
	if self.m_csNode.select then
		self.m_select = false
		self.m_csNode.select.selected:setVisible(self.m_select)
	end
end

function This:getChooseItemView(itemModel, index, fcount)
    local info = {item = itemModel, index = index, aloneMode = self.m_aloneMode, fcount = fcount, handler = function(node, prop)
		if self.m_selectHandler then
			local itemId = prop:getId()
			if prop:isLordEquip() or prop:isEquip() or prop:isFabao() or prop:isInscription() or prop:checkCommonType(ItemCommonType.kTactics) then
				itemId = prop:getUid()
			end
			local param = {
				index = self.m_selectIndex, 
				selected = self.m_selectedItems[itemId]
			}
			local result, errorStr = self.m_selectHandler(prop, param)
			if result == true then
				return self:close()
			else
				if errorStr then
					return g_scene:showTips(errorStr)
				end
				if not self.m_filterList[itemId] then
					self.m_filterList[itemId] = prop:getCount()
					self.m_selectIndex = self.m_selectIndex + 1
					self.m_selectedItems[itemId] = {pos = self.m_selectIndex, item = prop}
					node:setSelected(true)
				else
					self.m_filterList[itemId] = nil
					self.m_selectedItems[itemId] = nil
					self.m_selectIndex = self.m_selectIndex - 1
					node:setSelected(false)
				end
			end
		end
    end}
    local node = DepotItemView:create(info)
	g_scene:handleHelpTips(node.icon, function ()
		return node:getTipInfo()
    end)
	node:setScale(0.92)
	return node
end

function This:setSelectHandler(handler)
	self.m_selectHandler = handler
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.closebutton:setTouchEnded(function()
        self:close()
    end)

    if csNode.button then
    	csNode.button:setTouchEnded(function()
			if self.m_selectHandler(nil, nil, self.m_selectedItems) ~= false then
				return self:close()
			end
	    end)
	end
	
	if csNode.select then
		csNode.select:setTouchEnded(function()
			self.m_select = not self.m_select
			self.m_csNode.select.selected:setVisible(self.m_select)
			table.sort(self.m_itemList,function (a,b)
				if a:getQuality() ~= b:getQuality() then
					if self.m_select then
						return a:getQuality() > b:getQuality()
					else
						return a:getQuality() < b:getQuality()
					end
				end
			end)
			self.m_layerLayer:updateItemView()

		end)


	end
end