local This, super = class(..., "cc.Node")
This.openSound = "Open_Ui.mp3"
This.closeSound = "Close_Ui.mp3"
This.maskOpacity = 180
local Timer = require("Timer")

--[[function This:create(...)
    if self.fileName then
        local node = super.createFromCSFile(self, self.fileName, self._MNAME)
        node:init(...)
        return node
    else
        return super.create(self, ...)
    end
    return node
end]]

function This:init(info)
    self.m_dialogList = {}
    self.m_timer = Timer:create(self)
    self.m_level = 0xfffff
    self:scheduleUpdate()
    if not self.noMask then
        self:initMaskLayer()
    end
    self:initData(info)
    self:initNode(info)
    self:initListener()
end

function This:setLevel(level)
    self.m_level = level
end

function This:getLevel()
    return self.m_level
end

function This:onEnter()
    super.onEnter(self)
end

local EventDispatcher = require("EventDispatcher")
function This:addListener(key, func)
    self.m_isRegisterEvent = true
    EventDispatcher:addListener(self, key, func)
end

function This:callMsg(msg, args)
    EventDispatcher:dispatcher(msg, args)
end

function This:onExit()
    super.onExit(self)
    if self.m_isRegisterEvent then
        EventDispatcher:removeAllListener(self)
    end
	local parent = self:getParent()
	for index, _dialog in ipairs(parent.m_dialogList) do
        if dialog == _dialog then
            table.remove(parent.m_dialogList, index)
        end
    end
end

function This:addOpenCallBack(callback)
    self.m_openCallback = self.m_openCallback or {}
    self.m_openCallback[#self.m_openCallback + 1] = callback
end

function This:addCloseCallback(callback)
    self.m_closeCallbcak = self.m_closeCallbcak or {}
    self.m_closeCallbcak[#self.m_closeCallbcak + 1] = callback
end

function This:initTouchEvent()
    
end

function This:onOpen()
    self.m_isOpen = true
    self:initTouchEvent()
    if self.m_tabButtons then
        for name,button in pairs(self.m_tabButtons) do
            if not button.m_touch then
                button:setTouch({
                    onEnded = function ()
                        self:selectTabButton(name)
                    end
                })
            end
        end
    end
    if self.m_openCallback then
        for _, callback in ipairs(self.m_openCallback) do
            callback()
        end
    end
    EventDispatcher:dispatcher("dialog_open",self)
	self.name = "dialog"
	g_weakTable[tostring(self)] = self
end

function This:open(delayTime)
    delayTime = delayTime or 0

    -- 这段代码有问题，定时器会没了，问题不明, 心跳包来的时候概率大
    -- self:setTimer(delayTime, function (self)
    --     if self.openSound then
    --         cc.Audio:playEffect(self.openSound)
    --     end
    --     self:runOpenAnimation()
    -- end)

    --发现要下一帧再调，不然新手引导会卡住
    if self[0] ~= 0 then
        self:runAction(cc.Sequence(
            cc.DelayTime(delayTime),
            cc.CallFunc(function()
                if self[0] ~= 0 then 
                    if self.openSound then
                        cc.Audio:playEffect(self.openSound)
                    end
                    self:runOpenAnimation()
                end
            end)
        ))
    end
end

function This:initNode(info)
    
end

function This:initData(info)

end

function This:initListener()
    
end

function This:removerDialog(dialog)
    for index, _dialog in ipairs(self.m_dialogList) do
        if dialog == _dialog then
            table.remove(self.m_dialogList, index)
            return true
        end
    end
    return false
end

function This:onClose()
    if self.m_isClose then
        return
    end
    self.m_isClose = true
    self.m_isOpen = false
	if self.closeSound then
		cc.Audio:playEffect(self.closeSound)
	end
    local node = cc.Node:create()
    self:addChild(node, 99999)
    node:setTouchForMask({})

    self:getParent():removerDialog(self)
    self:runCloseAnimation()
    if self.m_closeCallbcak then
        for _, callback in ipairs(self.m_closeCallbcak) do
            callback(self)
        end
    end
    EventDispatcher:dispatcher("dialog_close",self)
end

function This:close()
    print("================== close")
    local lastDialog = self.m_dialogList[#self.m_dialogList]
    if lastDialog then
        self:setTimer(0.2, self.close, "Close")
        lastDialog:close()
    else
        self:onClose()
    end
end

function This:setTimer(delayTime, listener, key, triggerTotal)
    self.m_timer:setTimer(delayTime, listener, key, triggerTotal)
end

function This:update(deltaTime)
    self.m_timer:update(deltaTime)
end

function This:runCloseAnimation()
    if self.closeAnimation then
        self.m_csNode:playCSAnimation(self.closeAnimation, false, function ()
            self:removeFromParent()
        end)
    else
		self:removeFromParent()
    end
end

function This:runOpenAnimation()
    if self.openAnimation then
        self.m_csNode:playCSAnimation(self.openAnimation, false, function ()
            self:onOpen()
        end)
    else
		self:onOpen()
    end
end

function This:onTouchMask()
    
end

function This:initMaskLayer()
    self.m_maskLayer = cc.LayerColor:create(cc.Color(0, 0, 0, self.maskOpacity))
    self:addChild(self.m_maskLayer, -1)
    if self.isQuickExit then
        local touch = {}
        touch.onEnded = function (_, point)
            print("------------------ 1")
            self:onTouchMask()
            if self.m_csNode and self.m_isOpen then
                local rect = self.getQuickRect and self:getQuickRect() or self.m_csNode.bg:getBoundingBox()
                if not rect:isContainsPoint(point) then
                    self:close()
                end
            end
        end
        self.m_maskLayer:setTouchForMask(touch)
    else
        self.m_maskLayer:setTouchForMask({
            onEnded = function (touch,location)
                print("------------------ 2")
                self:onTouchMask(touch,location)
            end
        })
    end
end

local function getSubDialog(dialog, name)
    for _, dialog in ipairs(dialog.m_dialogList) do
        if dialog._NAME == name then
            return dialog
        end

        local result = getSubDialog(dialog, name)

        if result then
            return result
        end
    end

    return false
end

function This:getDialog(name)
    return getSubDialog(self, name)
end

function This:getTopDialog()
    if self.m_dialogList then
        return self.m_dialogList[#self.m_dialogList]
    end
end

function This:openDialog(name, info)
    local dialog = self:getDialog(name)
    if dialog and not dialog.isMultDialog then
        dialog:close()
    end
    local delayTime = info and info.delayTime or 0
    local dialog = require(name):create(info)
    print("openDialog", name)
    self.m_dialogList[#self.m_dialogList + 1] = dialog
    EventDispatcher:dispatcher("add_dialog",self._NAME)
    self:addChild(dialog)
    dialog:open(delayTime)
    return dialog
end

function This:getTabNode(name)
    return self.m_tabNodes and self.m_tabNodes[name]
end

function This:createTabNode(name)
    return require(name):create(self:getTabNodeData(name))
end

function This:addTabNode(name)
    if not self.m_tabNodes then
        self.m_tabNodes = {}
    end
    if self.m_tabNodes[name] then
        return self.m_tabNodes[name]
    end
    local node = self:createTabNode(name)
    node:setVisible(false)
    EventDispatcher:dispatcher("add_tab_node",name)
    self.m_tabNodes[name] = node
    local parent = self.m_tabNodeParent or self.m_csNode or self
    parent:addChild(node)
    self:onAddTabNode(node, name)
    return node
end

function This:getTabNodeData(name)
    
end

function This:onAddTabNode(node, name)
    
end

function This:onTabNodeSwitchIn(node, name)
    
end

function This:onTabNodeSwitchOut(node, name)
    
end

function This:canOpenTabNode(name)
    return true
end

function This:openTabNode(name)
    local result, errorCode = self:canOpenTabNode(name)
    if not result then
        if errorCode then
            g_scene:handleErrorCode(errorCode)
        end
        return false
    end
    if not self.m_tabNodes or not self.m_tabNodes[name] then
        self:addTabNode(name)
    end
    if self.m_selectTabName then
        local lastNode = self.m_tabNodes[self.m_selectTabName]
        if lastNode then
            self:onTabNodeSwitchOut(lastNode, self.m_selectTabName)
        end
    end
    self.m_selectTabName = name
    local curNode = self.m_tabNodes[name]
    if curNode then
        self:onTabNodeSwitchIn(curNode, name)
        EventDispatcher:dispatcher("open_tab_node",name)
    end
    for k,tabNode in pairs(self.m_tabNodes) do
        tabNode:setVisible(k == name)
    end
    return curNode
end

function This:addTabButton(name, button, onSelected)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end
    if self.m_tabButtons[name] then
        return
    end
    self.m_tabButtons[name] = button
    button.m_touch = false
    button.onSelected = onSelected
end

function This:getCurSelectTab()
    return self.m_selectTabButton
end

function This:selectTabButton(name)
    if self.m_tabButtons then
        if self.m_selectTabButton == name then
            return
        end
        if self.m_tabButtons[name].onSelected() ~= false then
            self:lightTabButton(name)
        end
    end
end

function This:lightTabButton(name)
    self.m_selectTabButton = name
    if self.m_tabButtons then
        for k,node in pairs(self.m_tabButtons) do
            if k == name then
                if node.light then
                    node.light:setVisible(true)
                end
                if node.normal then
                    node.normal:setVisible(false)
                end
            else
                if node.light then
                    node.light:setVisible(false)
                end
                if node.normal then
                    node.normal:setVisible(true)
                end
            end
        end
    end
end