-- @Author: Su
-- @Date:   2019-11-28 14:50:53
-- @Last Modified by:   Su
-- @Last Modified time: 2019-11-28 14:50:53
-- @File Name: BattleDrumGodHammerColumnItem.lua
-- @Description: 

local This , super = class(...,"NodeUI")
local DrumManager = require("Drum.DrumManager")
local ItemNode = require("UI.ItemNode")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local DepotManager = require("Depot.DepotManager")

This.fileName = "MenuScene/NodeUI/BattleDrumGodHammerColumn.csb"

function This:initNode(info)
    self.m_data = info
    self.m_escape = 0
    self:initView()
    self:refresh()
    self:scheduleUpdate()
end

function This:initView()
    for i=1,4 do
        local attributeData = self.m_data.attribute[i]
        if attributeData then
            self["text"..i]:setString(string.format( "%s",AttrName[attributeData[1]]))
            self["num"..i]:setString(string.format( "+%d",attributeData[2]))
            self["num"..i]:setAnchorPoint(0, 0.5)
        end

    end

end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
	DrumManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	DrumManager:removeObserver(self)
	DepotManager:removeObserver(self)
end

function This:refresh()

    local data = DrumManager:getHammerById(self.m_data.id)
    local item = ItemNode:create(self.m_data.item_id)
    item:setAnchorPoint(0.5,0.5)
    self.Node_2:addChild(item)
    self.hammername:setString(self.m_data.name)
    -- self.description:setString(self.m_data.description)
    self.btnadd:setVisible(data and data.state == GodHammerState.kUse)
    self.status_locked:setVisible(not data or data.state == GodHammerState.kLock )
    self.status:setVisible(data and data.state > GodHammerState.kLock)
    self.msg:setVisible(data and data.state == GodHammerState.kUse)
    self.btnuse:setVisible(data and data.state == GodHammerState.kActivated)
    self.btnactive:setVisible(not data or data.state == GodHammerState.kLock)
    self.time:setVisible(data and data.state >= GodHammerState.kActivated)
    self.txtime:setVisible(data and data.state >= GodHammerState.kActivated)
end

function This:getSize()
    local w,h = self.bg:getContentSize()
    return w,h
end

function This:initTouchEvent()
    self.btnactive:setTouch({
        isScale = false, 
        isDynamic = true, 
        onEnded = function()
            g_scene:openDialog("Drum.BattleDrumHammerSelectionPanel",{itemData = self.m_data.item[1]})

        end
    })

    self.btnuse:setTouch({
        isScale = false, 
        isDynamic = true, 
        onEnded = function()
            DrumManager:onUseGodHammerRequest(self.m_data.id,self.m_data.pos)
        end
    })

    self.btnadd:setTouch({
        isScale = false, 
        isDynamic = true, 
        onEnded = function()
            g_scene:openDialog("Drum.BattleDrumHammerSelectionPanel",{itemData = self.m_data.item[1]})

        end
    })
end

function This:onGetHammerList()
    self:refresh()
end

function This:refreshTime()
    local data = DrumManager:getHammerById(self.m_data.id)
    if data then
        local leftTime = data.time - GamePlayer:getServerTime()
        self.time:setString(GameUtil:formatDTime(Serialize[60728],leftTime))
    end
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshTime()
	end
end