-- @Author: Su
-- @Date:   2019-11-27 15:30:53
-- @Last Modified by:   Su
-- @Last Modified time: 2019-11-27 15:30:53
-- @File Name: BattleDrumRankPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local DrumManager = require("Drum.DrumManager")
local BattleDrumRankColumnItem = require("Drum.BattleDrumRankColumnItem")
local GamePlayer = require("Player.GamePlayer")

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/BattleDrumRank.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self.m_itemList = {}
    self.m_DrumId = DrumManager:getDrumLevel()
    self:initView()
end

function This:onEnter()
    super.onEnter(self)
    DrumManager:onGetRankListRequest()
	DrumManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	DrumManager:removeObserver(self)
end


function This:initView()
    local node = self.m_csNode
    local data = DrumManager:getRankList()
    local scrollView = node.Scroll
    scrollView:removeAllItems()
    for i = 1,10 do
        local rankData = {}
        if data[i] then
            rankData = data[i]
        else
            rankData.rank = i
        end
        local item = BattleDrumRankColumnItem:create(rankData) 
        local w,h= item:getSize()
        item:setAnchorPoint(0.5,0.5)
        scrollView:pushBackCustomItem(item,w,h,0.5)

    end
    local myData = DrumManager:getSelfRankData()
    node.inrank:setVisible(myData.rank)
    node.outrank:setVisible(not myData.rank)
    node.outrank.userid:setString(GamePlayer:getRoleName())
    if myData then
        node.inrank.ranknum:setString(myData.rank)
        node.inrank.userid:setString(myData.name)
    end
end

function This:initTouchEvent()
    local node = self.m_csNode
    node.close:setTouchEnded(function()
        self:close()
    end)

    node.help:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize[60738],node = self.m_csNode.help})
    end)

    node.inrank.reward:setTouchEnded(function()
        local myData = DrumManager:getSelfRankData()
        
        g_scene:openDialog("Drum.BattleDrumRankRewardPanel",{rank = myData.rank})
        
    end)
end

function This:onGetRankList()
    self:initView()
end