-- @Author: Su
-- @Date:   2019-11-21 15:30:53
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-09 11:49:33
-- @File Name: DrumManager.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local ArgConfig = require("Metadatas.ArgConfig")
local DrumConfig = require("Metadatas.DrumConfig")
local DrumHammerConfig = require("Metadatas.DrumHammerConfig") 
local DrumHaloConfig = require("Metadatas.DrumHaloConfig")
local HeroManager = require("Hero.HeroManager")
local GameUtil = require("Util.GameUtil")

GodHammerState = {
	kLock = 0,		--未激活
	kActivated = 1, --激活
	kUse = 2,		--使用中
}


function This:init()
	super.init(self)
	self.m_drumLevel = 1
	self.m_fightingValue = 0
	self.m_drumList = {}
	self.m_godHammerPos={0,0,0}
	self:initListener()
	-- self:onGetDrumRequest()
end

function This:initListener()
	EventDispatcher:addListener(self, 14700, self.onGetDrumReply)
	EventDispatcher:addListener(self, 14701, self.onUseGodHammerReply)
	EventDispatcher:addListener(self, 14702, self.onGetRankListReply)

end

function This:removeListener()
	EventDispatcher:removeListener(self,14700)
	EventDispatcher:removeListener(self,14701)
	EventDispatcher:removeListener(self,14702)

end

function This:getOpenLevel()
	if not self.m_openLv then
		self.m_openLv = ArgConfig.drum_open_level.arg[1]
	end
	return self.m_openLv
end

function This:checkOpen()
    return  GamePlayer:getLevel() >= self:getOpenLevel()
end

function This:getOpenTips()
	return string.format(Serialize[60309], self:getOpenLevel())
end

function This:getDrumLevel()
	return self.m_drumLevel
end

function This:getFightingValue()
	return self.m_fightingValue
end

function This:getNextFightingValue()
	return DrumConfig[self.m_drumLevel].spirit
end

function This:isMaxDrum()
	return self.m_drumLevel >= #DrumConfig
end

function This:getMaxDrum()
	return #DrumConfig
end

function This:getNowDrumConfig()
	local drum = DrumConfig[self.m_drumLevel]
	return drum
end

function This:getDrumConfigById(id)
	local drum = DrumConfig[id]
	return drum
end

function This:getNextDrumConfig()
	local drum = DrumConfig[self.m_drumLevel + 1]
	return drum
end

function This:getRankList()
	return self.m_rankList or {}
end

function This:getSelfRankData()
	return self.m_rankData or {}
end

function This:getDrumList()
	return self.m_drumList
end

function This:getHammerList()
	return self.m_godHammerPos
end

function This:getHammerById(id)
	for _,hammer in pairs(self.m_drumList) do
		if hammer.id == id then
			return hammer
		end
	end
	return nil
end

-- 获取当前光环等级
function This:getNowHaloLevel(drumlevel)
	if not drumlevel then
		drumlevel = self.m_drumLevel
	end
	local level = 0
	for i,haloData in ipairs(DrumHaloConfig) do
		if drumlevel >= haloData.drum_id then
			level = haloData.id
		end
	end
	return level
end

function This:getNowHaloImage(id)
	local config = DrumHaloConfig[id]
	return config.halo_star
end

-- 获取下一等级光环限制战鼓等级
function This:getNextHaloLimitLevel(level)
	if level < 4 then
		return DrumHaloConfig[level + 1].drum_id,false
	else
		return DrumHaloConfig[level].drum_id,true
	end
end

function This:getDrumAttr()
	 --战鼓属性
	local result = GameUtil:convertData(self:getNowDrumConfig().attribute)
	 --神锤属性
	for _, hammer in pairs(self.m_drumList) do
		if hammer.state == GodHammerState.kUse then
			local temAttr = GameUtil:convertData(DrumHammerConfig[hammer.id].attribute)
			result = GameUtil:tableAdd(temAttr, result)
		end
	end
	 --光环属性
	local haloLevel = self:getNowHaloLevel()
	local temAttr1 = GameUtil:convertData(DrumHaloConfig[haloLevel].halo)
	result = GameUtil:tableAdd(temAttr1, result)
	return result
end

function This:getHammerConfigById(id)
	return DrumHammerConfig[id]
end

--------------------------------

function This:onGetDrumRequest()
	if not self:checkOpen() then
		return
	end
	Socket:sendAndLock(14700,{})
end

function This:onGetDrumReply(msg)
    if msg.err_code == 0 then
		self.m_drumLevel = msg.level
		self.m_fightingValue = msg.value
		self.m_drumList = msg.drum_list
		if #msg.drum_list > 0 then
			for _,hammer in pairs(msg.drum_list) do
				if hammer.pos > 0 then
					self.m_godHammerPos[hammer.pos] = hammer.id
				end
			end
		end
		HeroManager:updateDrum()
        self:notifyObservers("onGetHammerList")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onUseGodHammerRequest(id, pos)
	-- -- 有装备卸下
	-- if self.m_godHammerPos[pos] > 0 then
	-- 	pos = 0
	-- end
	Socket:sendAndLock(14701,{id = id,pos = pos})
end

function This:onUseGodHammerReply(msg)
	if msg.err_code == 0 then
		self.m_drumList = msg.drum_list
		if #msg.drum_list > 0 then
			for _,hammer in pairs(msg.drum_list) do
				if hammer.pos > 0 then
					self.m_godHammerPos[hammer.pos] = hammer.id
				end
			end
		end
		HeroManager:updateDrum()
        self:notifyObservers("onGetHammerList")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetRankListRequest()
	Socket:sendAndLock(14702,{})
end

function This:onGetRankListReply(msg)
	if msg.err_code == 0 then
		self.m_rankList = msg.rank_list
		table.sort(self.m_rankList,function(a,b)
			if a.rank ~= b.rank then
				return a.rank < b.rank
			end
			
		end)
		for _,rankData in pairs(self.m_rankList) do
			if GamePlayer:getPlayerID() == rankData.roleid then
				self.m_rankData = rankData
			end
		end
        self:notifyObservers("onGetRankList")
    else
        g_scene:handleErrorCode(msg.err_code)
    end

end