-- @Author: Su
-- @Date:   2019-11-21 15:30:53
-- @Last Modified by:   Su
-- @Last Modified time: 2019-11-21 15:30:53
-- @File Name: DrumTab.lua
-- @Description: 

local This , super = class(...,"NodeUI")
local DrumManager = require("Drum.DrumManager")
local DrumCostConfig = require("Metadatas.DrumCostConfig")
local DrumHammerConfig = require("Metadatas.DrumHammerConfig") 
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local DrumAttributeTxt =require("Drum.DrumAttributeTxt")
local DrumItemModel = require("Drum.DrumItemModel")
local ViewUtil = require("UI.ViewUtil")
local GameUtil = require("Util.GameUtil")

function This:initNode(node)
    self.m_parent = node
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/BattleDrumMain.csb")
    self:addChild(csNode)
    self.m_csNode = csNode
    self.m_selected= false
    self.m_itemList = {}
    self:initCostView()
end

function This:onEnter()
    super.onEnter(self)
    DrumManager:onGetDrumRequest()
    DepotManager:addObserver(self)
	DrumManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	DrumManager:removeObserver(self)
	DepotManager:removeObserver(self)
end

function This:initCostView()
    local node = self.m_csNode
    for i,itemData in pairs(DrumCostConfig) do
        local index = itemData.id - 341
        if not self.m_itemList[i] then
            local item = GameUtil:creatItemNode(node["cost"..index].iconitem,{itemData.id,1},true)
            item.m_qualityBg:setTouch({
                onEnded = function()
                    local item = DepotManager:getItem(itemData.id)
                    local count = 1
                    local num = DepotManager:getItemCount(itemData.id)
                    if item then
                        local count = 1
                        if self.m_selected then
                            count = 100
                            if num < count then
                                count = num
                            end
                        end
                        DepotManager:useItem(item,count)  
                    end      
                end,
                isScale = false,
                isDynamic = true,
                isSwallow = false,
            })
            item:setAnchorPoint(0.5,0.5)
            item:setScale(0.8)
            self.m_itemList[i] = item
        else
            self.m_itemList[i]:initItem(itemData.id,DepotManager:getItemCount(itemData.id))
        end
    end
end

function This:refreshHummerList ()
    local node = self.m_csNode
    local list = DrumManager:getHammerList()
    for i=1,3 do
        if list[i] > 0 then
            node["item"..i]:removeAllChildren()
            local item = ItemNode:create(DrumHammerConfig[list[i]].item_id)
            item:setAnchorPoint(0.5,0.5)
            node["item"..i]:addChild(item)
        else
            node["item"..i]:removeAllChildren()
        end
    end

end



function This:refresh()
    local drumConfig = DrumManager:getNowDrumConfig()
    local nextDrumConfig = DrumManager:getNextDrumConfig()
    local node = self.m_csNode
    node.drumname:setImage(drumConfig.drum_icon)
    node.drumname:setScale(0.8)
    node.drumlevel:setImage(drumConfig.drum_icon2)
    node.drumkuang:setImage(drumConfig.drum_icon3)

    node.drumlevel:setVisible(not DrumManager:isMaxDrum())
    node.maxtxt:setVisible(DrumManager:isMaxDrum())
    node.property.next:setVisible(not DrumManager:isMaxDrum())
    node.property.scroll2:setVisible(not DrumManager:isMaxDrum())
    node.value:setString(string.format( "%d/%d",DrumManager:getFightingValue(),DrumManager:getNextFightingValue()))
    if DrumManager:isMaxDrum() then
        node.value:setString(string.format( "%d/∞",DrumManager:getFightingValue(),DrumManager:getNextFightingValue()))

    end
    node.exp:setPercent(DrumManager:getFightingValue()/DrumManager:getNextFightingValue()*100)
    node.note:setVisible(false)
    node.select.selected:setVisible(self.m_selected)
    ViewUtil:blinkView(node.note, true)

    local scrollview = node.property.scroll
    scrollview:removeAllItems()
    scrollview:setDirection("vertical")
    for i,attribute in pairs(drumConfig.attribute) do
        local str = string.format( "%s",AttrName[attribute[1]])
        local value = string.format( "+%d",attribute[2] ) 
        local item = DrumAttributeTxt:create({text = str,num =value,isGreen = false})
        item:setAnchorPoint(0.5,0.5)
        scrollview:pushBackCustomItem(item,118,20,0.5)
    end

    if not DrumManager:isMaxDrum() then
        local Scrollview = node.property.scroll2
        Scrollview:removeAllItems()
        for i,attribute in pairs(nextDrumConfig.attribute) do
            local str = string.format( "%s",AttrName[attribute[1]])
            local value = string.format( "+%d",attribute[2] ) 
            local item = DrumAttributeTxt:create({text = str,num =value,isGreen = true})
            item:setAnchorPoint(0.5,0.5)
            Scrollview:pushBackCustomItem(item,118,20,0.5)
        end
    end
end

function This:initTouchEvent()
    local node = self.m_csNode
    node.btnpreview:setTouchEnded(function()
        g_scene:openDialog("Drum.BattleDrumPreviewPanel",{})

    end)

    node.lightcircle.bg:setTouchEnded(function()
        g_scene:openDialog("Drum.BattleDrumLightCirclePanel",{})

    end)

    node.rank.bg:setTouchEnded(function()
        g_scene:openDialog("Drum.BattleDrumRankPanel",{})
    end)

    node.underline.tx:setTouchEnded(function()
        g_scene:openDialog("SingleDaily.WarTowerMainPanel",{})
    end)

    node.help:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize[60744],node = self.m_csNode.help})
    end)

    node.select:setTouchEnded(function()
        self.m_selected = not self.m_selected 
        node.select.selected:setVisible(self.m_selected)
    end)
    for i = 1,3 do
        node["bgitem"..i].greenicon:setTouch({
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
                g_scene:openDialog("Drum.BattleDrumGodHammerPanel",{pos = i})
            end
        })

    end

end

function This:onGetHammerList()
    self:refreshHummerList()
    self:refresh()
    self:initCostView()
end

function This:useItem()
    DrumManager:onGetDrumRequest()
end

