local ffi = require("ffi")
ffi.cdef[[
typedef unsigned int uint;
typedef struct Rect{float x,y,width,height;} Rect;
typedef struct Rect ccRect;
typedef struct Vec{float x,y;}Vec;
typedef struct Vec Vec2;
typedef struct Vec ccVec2;
typedef struct Size{float width,height;} Size;
typedef struct Size ccSize;
typedef struct AABB{float minX,minY,maxX,maxY;} AABB;
]]

function ffi.metaclass(typeName, classObject)
    local ffiobj = nil
    if classObject then
        classObject.__index = classObject
        ffiobj = ffi.metatype(typeName, classObject)
    else
        ffiobj = ffi.typeof(typeName)
    end
    package.loaded[typeName] = ffiobj
    return ffiobj
end

-- math extend start
local random = math.random
function math.clamp(v, a, b)
    if v < a then
        v = a
    end
    if v > b then
        v = b
    end
    return v
end
function math.round(number)
    return math.floor(number + 0.5000001)
end
function math.fmod(x, y)
    local result = x % y
    if x > 0 and result == 0 then
        return y
    else
        return result
    end
end
function math.lerpingconst(from, to, d)
    local s = to - from
    if s < -d then
        s = -d
    end
    if s > d then
        s = d
    end
    return from + s
end
function math.isPointInAABB(minX, minY, maxX, maxY, px, py)
    return px >= minX and px <= maxX and py >= minY and py <= maxY
end
function math.isPointInCenterAABB(centerX, centerY, width, height, px, py)
    local minX = centerX - width * 0.5
    local maxX = centerX + width * 0.5
    local minY = centerY - height * 0.5
    local maxY = centerY + height * 0.5
    return px >= minX and px <= maxX and py >= minY and py <= maxY
end
function math.isPointInCircle(centerX, centerY, radius, px, py)
    local x = px - centerX
    local y = py - centerY
    local distance2 = x * x + y * y
    return distance2 <= radius * radius
end
local sqrt = math.sqrt
function math.getPointOnCircle(x0, y0, r, x1, y1)
    local d = r*r*(-2*y0*y1-2*x0*x1+y1*y1+y0*y0+x0*x0-r*r+x1*x1)
    if d < 0 then
        return nil
    end
    local a = y0*x0+y1*x1-y0*x1-y1*x0
    local b = sqrt(d)
    local c = -r*r+x0*x0-2*x0*x1+x1*x1
    if c * c <= 0.00001 then
        return x1, y1
    end
    local k1=(a+b)/c
    local k2= (a-b)/c
    local x_1=(-k1*y1+x0+k1*k1*x1+y0*k1)/(1+k1*k1)
    local y_1 =-(-y1-k1*x0-y0*k1*k1+k1*x1)/(1+k1*k1)
    local x_2=(-k2*y1+x0+k2*k2*x1+y0*k2)/(1+k2*k2)
    local y_2 =-(-y1-k2*x0-y0*k2*k2+k2*x1)/(1+k2*k2)
    return x_1, y_1, x_2, y_2
end
function math.getAngleTwoVec(x1, y1, x2, y2)
    local d = sqrt(x1*x1+y1*y1) * sqrt(x2*x2+y2*y2)
    return math.deg(math.acos((x1*x2 + y1*y2) / d))
end
function math.dotVec(x1, y1, x2, y2)
    return x1 * x2 + y1 * y2
end
function math.secondToHour(allSeconds)
    local hours = math.floor(allSeconds / 3600)
    local minutes = math.floor(allSeconds % 3600 / 60)
    local second = allSeconds % 3600 % 60
    return hours, minutes, second
end
function math.secondToDay(second)
    local d = math.floor(second / 86400)
    local h = math.floor((second - d * 86400) / 3600)
    local m = math.ceil((second - d * 86400 - h * 3600) / 60)
    return d, h, m
end
function math.randomFloat(a, b)
    return a + random(10000) * (b - a) * 0.0001
end
function math.randomAtoB(a, b)
    return a + random(10000) * (b - a) * 0.0001
end
function math.isRandom(ratio)
    if ratio >= 1 then
        return true
    elseif ratio > 0 and ratio * 10000 >= random(10000) then
        return true
    end
    return false
end
function math.weightRandom(weights)
    local segments = {}
    local sum = 0
    for i = 1, #weights do
        sum = sum + weights[i][2]
        segments[i] = sum
    end
    local num = random(sum)
    for i = 1, #segments do
        if num <= segments[i] then
            return weights[i][1]
        end
    end
end

function math.isSegmentIntersect(ax, ay, bx, by, cx, cy, dx, dy)
    local area_abc = (ax - cx) * (by - cy) - (ay - cy) * (bx - cx)
    local area_abd = (ax - dx) * (by - dy) - (ay - dy) * (bx - dx)

    if area_abc*area_abd >= 0 then
        return false
    end

    local area_cda = (cx - ax) * (dy - ay) - (cy - ay) * (dx - ax)
    local area_cdb = area_cda + area_abc - area_abd
    if area_cda * area_cdb >= 0 then
        return false
    end
    local t = area_cda / ( area_abd- area_abc )
    return true, ax + t*(bx - ax), ay + t*(by - ay)
end

local rad = math.rad
local cos = math.cos
local sin = math.sin
function math.rotate(px, py, angle, cx, cy)
    angle = rad(angle)
    local c = cos(angle)
    local s = sin(angle)
    local x = (px - cx) * c - (py - cy) * s + cx
    local y = (px - cx) * s + (py - cy) * c + cy
    return x, y
end

function math.getVector(angle, length)
    length = length or 1
    angle = rad(angle)
    return cos(angle) * length, sin(angle) * length
end
function math.mirrorVector(x,y, nx,ny)
    local d = 2*(x*nx + y*ny)/(nx*nx + ny* ny)
    return -d*nx + x, -d*ny + y
end

-- math extend end

-- table extend start
local type = type
local tostring = tostring
function table.find( tb, filter_func )
    for k, v in pairs(tb) do
        if filter_func(k, v) then
            return true
        end
    end
    return false
end

function table.ifilter(tb, filter_func)
    local t = {}
    local count = #tb
    for i = 1, count do
        if filter_func(i, tb[i]) then
            t[#t + 1] = tb[i]
        end
    end

    return t
end

function table.filter(tb, filter_func)
    local t = {}

    for k, v in pairs(tb) do
        if filter_func(k, v) then
            t[k] = v
        end
    end

    return t
end

function table.getKeyList(t)
    local key_list = {}
    for k, _ in pairs(t) do
        key_list[#key_list + 1] = k
    end
    return key_list
end

local function dumpStringList(t, mainDepth, depth, keyCmp)
    local stringList = {}
    if #t > 0 then
        local n = 1
        local isArray = true
        for k, v in pairs(t) do
            if k ~= n then
                isArray = false
                break
            end
            n = n + 1
        end
        if isArray then
            for _,v in ipairs(t) do
                local value = v
                local valueType = type(v)
                if valueType == "table" then
                    -- value = tostring(v)
                    value = table.__dump(v, mainDepth, 0, keyCmp)
                elseif valueType == "string" then
                    value = '"' .. value  .. '"'
                else
                    value = tostring(v)
                end
                table.insert(stringList, value)
            end
            return stringList
        end
    end

    local keyList = {}
    for k,v in pairs(t) do
        table.insert(keyList,k)
    end
    if keyCmp then
        table.sort(keyList, keyCmp)
    end
    for i = 1, #keyList do
        local k = keyList[i]
        local v = t[k]
        local value = v
        local valueType = type(v)
        local key = type(k) == "number" and "["..k.."]" or k
        if valueType == "table" then
            --value = tostring(v)
             value = table.__dump(v, mainDepth, 0, keyCmp)
        elseif valueType == "string" then
            value = '"' .. value  .. '"'
        else
            value = tostring(v)
        end
        table.insert(stringList, key .. "=" .. value)
    end
    return stringList
end

function table.__dump(t, mainDepth, depth, keyCmp)
    local str = nil
    mainDepth = mainDepth + 1
    depth = depth + 1
    
    if mainDepth == 1 then
        return table.concat(dumpStringList(t, mainDepth, depth, keyCmp), ",\n    ")
    elseif mainDepth == 2 and depth == 1 then
        return "{\n        " .. table.concat(dumpStringList(t, mainDepth, depth, keyCmp), ",\n        ") .. "\n    }"
    elseif mainDepth == 2 then
        return "{\n" .. table.concat(dumpStringList(t, mainDepth, depth, keyCmp), ", ") .. "}\n"
    elseif mainDepth == 3 and depth == 1 then
        return "{" .. table.concat(dumpStringList(t, mainDepth, depth, keyCmp), ", ") .. "}"
    elseif mainDepth == 3 and depth == 2 then
        return "        {" .. table.concat(dumpStringList(t, mainDepth, depth, keyCmp), ", ") .. "}"
    else
        return "{" .. table.concat(dumpStringList(t, mainDepth, depth, keyCmp), ", ") .. "}"
    end
end

function table.dump(t, keyCmp)
    return "{\n    " .. table.__dump(t, 0, 0, keyCmp) .. "\n}\n"
end

local function table_tostring(t)
    local stringList = {}
    if #t > 0 then
        local n = 1
        local isArray = true
        for k, v in pairs(t) do
            if k ~= n then
                isArray = false
                break
            end
            n = n + 1
        end
        if isArray then
            for _,v in ipairs(t) do
                local valueType = type(v)
                if valueType ~= "function" then
                    local value = v
                    if valueType == "table" then
                        value = table_tostring(v)
                    elseif valueType == "string" then
                        value = '["' .. value  .. '"]'
                    else
                        value = tostring(v)
                    end
                    stringList[#stringList + 1] = value
                end
            end
            return "{" .. table.concat(stringList, ", ") .. "}"
        end
    end
    for k,v in pairs(t) do
        local valueType = type(v)
        if valueType ~= "function" then
            local value = v
            local key = type(k) == "number" and "["..k.."]" or '["'..k..'"]'
            if valueType == "table" then
                value = table_tostring(v)
            elseif valueType == "string" then
                value = '"' .. value  .. '"'
            else
                value = tostring(v)
            end
            stringList[#stringList + 1] = key .. "=" .. value
        end
    end
    return "{" .. table.concat(stringList, ",") .. "}"
end
table.tostring = table_tostring

local function table_copy(t, target)
    for k,v in pairs(target) do
        if type(v) == "table" then
            t[k] = t[k] or {}
            table_copy(t[k], v)
        else
            t[k] = v
        end
    end
    return t
end
table.copy = table_copy

function table.clone(t)
    return table_copy({}, t)
end

function table.contain(t, value)
    for k,v in pairs(t) do
        if v == value then
            return true
        end
    end
    return false
end

function table.keyOfItem(t, item)
    for k, v in pairs(t) do
        if v == item then return k end
    end
    return nil
end

function table.count(t)
    local count = 0
    for k, v in pairs(t) do
        count = count + 1
    end
    return count
end

local function addNode(self, node)
    if not self.m_head then
        self.m_head = node
        self.m_tail = node
        self.m_count = 1
    else
        node.m_prev = self.m_tail
        self.m_tail.m_next = node
        self.m_tail = node
        self.m_count = self.m_count + 1
    end
end

local function removeNode(self, node)
    local prev = node.m_prev
    local next = node.m_next
    if prev then
        prev.m_next = next
        node.m_prev = false
    end
    if next then
        next.m_prev = prev
        node.m_next = false
    end
    if self.m_head == node then
        self.m_head = next
    end
    if self.m_tail == node then
        self.m_tail = prev
    end
    self.m_count = self.m_count - 1
end

local function moveToTail(self, node)
    if self.m_tail ~= node then
        removeNode(self, node)
        addNode(self, node)
    end
end
table.addNode = addNode
table.removeNode = removeNode
table.moveToTail = moveToTail
-- table extend start

-- string extend start
local string = string

function string.trimLeft(str)
    return (string.gsub(str, "^%s+", ""))
end

function string.trimRight(str)
    return (string.gsub(str, "%s+$", ""))
end

function string.trim(str)
    return string.trimRight(string.trimLeft(str))
end

function string.trimAllSpace(str)
    return (string.gsub(str, "%s+", ""))
end

function string.trimAllNewline(str)
    return (string.gsub(str, "[\r\n]", ""))
end

function string.split(str, delimiter)
    if str then
        delimiter = delimiter or "|"
        local result = {}
        for match in string.gmatch(str, "[^" .. delimiter .. "]+") do
            result[#result + 1] = match
        end
        return result
    end
end
function string.split_char(str, delimiter)
    local b = string.byte(delimiter)
    local result = {}
    local n = #str
    local pos = 1
    for i = 1, n do
        if string.byte(str, i) == b then
            result[#result + 1] = string.sub(str, pos, i - 1)
            pos = i + 1
        end
    end
    if pos <= n then
        result[#result + 1] = string.sub(str, pos, -1)
    end
    return result
end
function string.formatTable(str, t)
    local ret = string.gsub(str, "{([^{}]+)}", function (name) 
        local n = tonumber(name)
        if n then
            return t[n]
        else
            if name:byte(-1) == 37 then -- 37 == %
                local k = name:sub(1, -2)
                k = tonumber(k) or k
                return (t[k] * 100) .. "%"
            else
                return t[name]
            end
        end
    end)
    return ret
end
function string.formatSecond(second)
    local h, m, s = math.secondToHour(second)
    if h == 0 then
        return m .. ":" .. s
    else
        return h .. ":" .. m .. ":" .. s
    end
end
function string.formatSecondInCn(second, noSecond)
    local h, m, s = math.secondToHour(second)
    if noSecond then
        if h == 0 then
            return AppText:isChinese() and (m .. "分") or string.format("%s minute", m)
        else
            return AppText:isChinese() and (h .. "时" .. m .. "分") or string.format("%s:%s", h, m)
        end
    else
        if h == 0 then
            return AppText:isChinese() and (m .. "分" .. s .. "秒") or string.format("%s:%s", m, s)
        else
            return AppText:isChinese() and (h .. "时" .. m .. "分" .. s .. "秒") or string.format("%s:%s:%s", h, m, s)
        end
    end
end
function string.secondToDay(second)
    local d, h, m = math.secondToDay(second)
    if AppText:isChinese() then
        return d .. "天" .. h .. "时" .. m .. "分"
    end
    return string.format("%s-%s:%s", d, h, m)
end

function string.numberToWord(number)
    local words = {"一", "二", "三", "四", "五", '六', "七", "八", "九", "十"}

    if AppText:isChinese() then
        if number <= 10 then
            if number ==0 then
                return "零"
            else
                return words[number]
            end
        else
            if number > 99 then
                return number
            else
                number1 = math.floor(number / 10)
                number2 = number % 10
                if number < 20 then
                    return words[10]..words[number2]
                else
                    local word = words[number1]..words[10]
                    if number2 > 0 then
                        word = word..words[number2]
                    end
                    return word
                end
            end
        end
    else
        return number
    end
end
function string.startWith(str, s)
    return string.find(str,s) == 1
end
function string.firstCharUpper(s)
    return string.upper(string.sub(s,1,1)) .. string.sub(s,2)
end

function string.trim(s)
    return string.gsub(s, "^%s*(.-)%s*$", "%1")
end

function string.formatFloat(float, digit) --精确到小数点后digit位
    local formatPat = "%." .. digit .. "f"
    float = string.format(formatPat, float)
    float = string.gsub(float, "0+$", "")
    float = string.gsub(float, "%.$", "")
    return float
end

function string.ceilOrFloor(value,isCeil)
    local function float_Eaual(x,v)
        local EPSILON = 0.000001  
        return ((v - EPSILON) < x) and (x <( v + EPSILON))  
    end
    local mathmod1,mathmod2  = math.modf(value)
    if float_Eaual(value,mathmod1) then
        return mathmod1
    elseif isCeil then
        return math.ceil(value)--向上
    else
        return math.floor(value)--向下
    end
end

function string.colorIdAndContent(str)
    local colorID, s = string.match(str, "#([^#]+)#([^#]+)")
    if colorID and s then
        return colorID, s
    else
        return nil
    end
end

function string.utf8Len(str)
    local count , i = 0, 1
    local strlen = string.len(str)
    while i <= strlen do
        local byte = string.byte(str,i,i)
        if byte >= 0xFC then
            len = 6
        elseif byte >= 0xF8 then
            len = 5
        elseif byte >= 0xF0 then
            len = 4
        elseif byte >= 0xE0 then
            len = 3
        elseif byte >= 0xC0 then
            len = 2
        else
            len = 1
        end
        i = i + len
        count = count + 1
    end
    return count
end
-- string extend end

-- bit extend begin
function bit.packBitArray(t)
    local bitList = {}
    local bor = bit.bor
    local lshift = bit.lshift
    local ceil = math.ceil
    for i = 1, #t do
        local mainIndex = ceil(i / 15)
        local subIndex = i - (mainIndex - 1) * 15
        local b = bitList[mainIndex] or 0
        bitList[mainIndex] = bor(b, lshift(t[i], (subIndex - 1) * 2))
    end
    return bitList
end

function bit.unpackBitArray(bitList)
    local t = {}
    local band = bit.band
    local rshift = bit.rshift
    for i, v in ipairs(bitList) do
        for i = 1, 15 do
            t[#t + 1] = band(v, 3)
            v = rshift(v, 2)
            if v == 0 then
                break
            end
        end
    end
    return t
end
-- bit extend end

-- os date extend begin
function os.serverdate( format, timeStamp )
    local function getTimezone()
        local now = os.time()
        return os.difftime(now, os.time(os.date("!*t", now)))
    end
    
    local timeZoneOffset = 8 * 3600 - getTimezone()  --计算出服务端时区与客户端时区差值
    return os.date(format, timeStamp + timeZoneOffset)
end
-- os date extend end

cc = {}
ffi.metaclass("Size")
--require("luaj")
require("Class")
require("BaseClass")
require("Rect")
require("Range")
require("Vec")
require("AABB")
require("cc.Base")
require("b2.Base")