--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local ItemUtil = require("Bag.ItemUtil")
local EquipConfig = require("Metadatas.EquipConfig")
local HeroManager = require("Hero.HeroManager")
local EquipUpConfig = require("Metadatas.EquipUpConfig")
local FoundationAttributeConfig = require("Metadatas.FoundationAttributeConfig")
local EquipmentQualityConfig = require("Metadatas.EquipmentQualityConfig")
local EquipLevelUpConfig = require("Metadatas.EquipLevelUpConfig")
local GameUtil = require("Util.GameUtil")

EquipCalcMode = {
	kNone = 0,
	kStrength = 1,
	kAscend = 2,
	kWash = 3,
	kLevel = 4
}

StrengthenType = {
	kLow = 1, -- 低级强化
	kIntermediate = 2, -- 中级强化
	kPerfect = 3, -- 完美强化
}

-- local print = print_r

function This:create(info)
	local o = {}
	setmetatable(o,self)
	o:init(info)
	return o
end

function This:init(info, itemInfo)
	assert(info.equipId and info.uid)
	self.m_uid = info.uid
	self.m_equipId = info.equipId
	self.m_progress = info.progress or 0
	self.m_strengthLv = info.strengthLv or 0
	self.m_level = info.level or 0
	self.m_config = EquipConfig[self.m_equipId]
	self.m_heroId = info.heroId or 0
	self.m_item = self:createItem(itemInfo)
	self.m_clearList = {}
	self:updateClearAttr(info.clearList)
end

function This:getUid()
	return self.m_uid
end

function This:getEquipId()
	return self.m_equipId
end

function This:getWearPos()
	return self.m_config.type
end

function This:getProgress()
	return self.m_progress
end

function This:getStrengthLv()
	return self.m_strengthLv
end

function This:getLevel()
	return self.m_level
end

function This:getHeroId()
	return self.m_heroId
end

function This:getQuality()
	return ItemUtil.getQuality(self.m_equipId)
end

function This:getQualityImage()
	return ItemUtil.getQualityImage(self.m_equipId)
end

function This:getConfig()
	return self.m_config
end

function This:getClearList()
	return self.m_clearList
end

-- 等级加成系数
function This:getLevelCoefficient()
	return self.m_config.coefficient
end

function This:getStrengthUpConfigId(strengthLv)
	if strengthLv > kMaxStrengthLv then
		strengthLv = kMaxStrengthLv
	end
	return strengthLv + self.m_config.type * 1000
end

function This:getPlayerLvLimit()
	local nextLv = self:getStrengthUpConfigId(self.m_strengthLv + 1)
	return EquipUpConfig[nextLv].player_level
end

function This:getUpgradePlayerLvLimit()
	return self.m_config.player_level
end

function This:getIcon()
	local file = string.format("Icons/Item/%s.png",self.m_config.icon)
	if cc.FileUtils:isFileExist(file) then
		return file
	end
	return "Icons/Item/48005.png"
end

function This:getCostItemList(strengthenType)
	local result = {}
	if self.m_strengthLv < kMaxStrengthLv then
		local nextId = self:getStrengthUpConfigId(self.m_strengthLv + 1)
		local config = EquipUpConfig[nextId]
		local itemList

		if strengthenType then
			if strengthenType == StrengthenType.kLow then
				itemList = config.cost1
			elseif strengthenType == StrengthenType.kIntermediate then
				itemList = config.cost2
			elseif strengthenType == StrengthenType.kPerfect then
				itemList = config.cost3
			else
				error("error!")
			end
		else
			itemList = config.cost1
		end

		for i = 1 , #itemList do
			local item = itemList[i]
			table.insert(result,{id = item[1], num = item[2]})
		end
	end
	return result
end

function This:strengthSecceed()
	self:updateData({progress = 0,strengthLv = self.m_strengthLv + 1,heroId = self.m_heroId})
end

function This:strengthFailed()
	self:updateData({progress = self.m_progress + 1,strengthLv = self.m_strengthLv,heroId = self.m_heroId})
end

function This:updateData(info)

	if self.m_heroId ~= info.heroId then
		self:setHeroId(info.heroId)
	end
	print("装备进度更新",info.progress)
	self.m_progress = info.progress
	if self.m_strengthLv ~= info.strengthLv then
		self:updateAttr(self.m_strengthLv , info.strengthLv)
	end

	if self.m_equipId ~= info.equipId then
		self:updateAttrByUpgradeQuality(info.equipId)
	end

	if self.m_level ~= info.level then
		self:updateUpLevelAttr(self.m_level,info.level)
	end
	self:updateClearAttr(info.clearList)
end

function This:updateAttr(oldLv,newLv)
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		local oldAttr = self:getAddAttr(EquipCalcMode.kStrength, {level = oldLv})
		print(string.format("装备属性更新:"))
		for i , info in pairs(oldAttr) do
			hero:reduceAttr(info.attrId,info.value)
			print(string.format("%s减少%d",AttrName[info.attrId],info.value))
		end
		local newAttr = self:getAddAttr(EquipCalcMode.kStrength, {level = newLv})
		for i , info in pairs(newAttr) do
			hero:addAttr(info.attrId,info.value)
			print(string.format("%s增加%d",AttrName[info.attrId],info.value))
		end
		hero:updateEquipClear()
		hero:updateFightPower()
	end
	self.m_strengthLv = newLv
	self.m_item.strength_level = newLv

end

function This:updateUpLevelAttr(oldLv,newLv)
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		local oldAttr = self:getUpLevelAttr(oldLv)
		print(string.format("装备属性更新:"))
		for id , value in pairs(oldAttr) do
			hero:reduceAttr(id,value)
			print(string.format("%s减少%d",AttrName[id],value))
		end
		local newAttr = self:getUpLevelAttr(newLv)
		for id , value in pairs(newAttr) do
			hero:addAttr(id,value)
			print(string.format("%s增加%d",AttrName[id],value))
		end
		hero:updateEquipClear()
		hero:updateFightPower()
	end
	self.m_level = newLv
	self.m_item.level = newLv
end

function This:updateAttrByUpgradeQuality(newEquipId)
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		local oldAttr = self:getAddAttr()
		print(string.format("装备属性更新:"))
		for i , info in pairs(oldAttr) do
			hero:reduceAttr(info.attrId,info.value)
			print(string.format("%s减少%d",AttrName[info.attrId],info.value))
		end
		local newAttr = self:getAddAttr(EquipCalcMode.kAscend)
		for i , info in pairs(newAttr) do
			hero:addAttr(info.attrId,info.value)
			print(string.format("%s减少%d",AttrName[info.attrId],info.value))
		end
		hero:updateEquipClear()
		hero:updateFightPower()
	end
	self:upgradeQuality(newEquipId)
end

function This:getAddAttr( mode, info )
	local mode = mode or EquipCalcMode.kNone
	local calcBasicAttr = true
	local config = self.m_config
	local level = self.m_strengthLv
	local clearList = self.m_clearList
	if mode == EquipCalcMode.kWash then
		calcBasicAttr = false
	end
	if mode == EquipCalcMode.kStrength then
		if info then
			level = info.level or self.m_strengthLv
		end
	elseif mode == EquipCalcMode.kAscend then
		config = EquipConfig[self.m_equipId + 1]
	elseif mode == EquipCalcMode.kWash then
		if info and info.clearList then 
			clearList = {}
			for _ ,v in pairs(info.clearList) do 
				clearList[v.index] = v.wash_id
			end
		end
	end

	local attr = {}
	if config then
		for i = 1 , 4 do
			local attrId = config["attrId"..i]
			local value = config["value"..i][level + 1] or 0
			if attrId > 0 and value then
				table.insert(attr,{attrId = attrId, value = value})
			end
		end
		
		if not calcBasicAttr then
			for _ , v in pairs(clearList) do 
				local info = FoundationAttributeConfig[v]
				local attrId, value = info.attribute_type or 0, info.attribute_value or 0
				if attrId > 0 and value then
					table.insert(attr,{attrId = attrId, value = value})
				end
			end
			local upLevelAttr = self:getUpLevelAttr()
			for id,value in pairs(upLevelAttr) do
				table.insert(attr,{attrId = id, value = value})
			end
		end
	end
	return attr
end

function This:getUpLevelAttr(level)
	if not level then
		level = self:getLevel()
	end
	local attr = {}
	local levelCoefficient = self:getLevelCoefficient()
	if level < 1 then
		for  _,data in pairs(EquipLevelUpConfig[ self:getWearPos()*1000 + 1].num) do
			attr[data[1]] = 0
		end
		return attr
	end
	-- for id = self:getWearPos()*1000 + 1,self:getWearPos()*1000+level do
	-- 	for _,data in pairs(EquipLevelUpConfig[id].num) do
	-- 		if attr[data[1]] == nil then
	-- 			attr[data[1]] = data[2]*levelCoefficient
	-- 		else
	-- 			attr[data[1]] = attr[data[1]]*levelCoefficient
	-- 		end
	-- 	end
	-- end
	if level >100 then
		return attr
	end
	for _,data in pairs(EquipLevelUpConfig[self:getWearPos()*1000+level].num) do
		attr[data[1]] = data[2]*levelCoefficient
	end
	return attr
end

function This:getAscendCost()
    return self.m_config.max_times
end

function This:initHeroId(heroId)
	if not heroId or heroId <= 0 then
		return 
	end
	self.m_heroId = heroId
	local hero = HeroManager:getHero(self.m_heroId)
	local attr = self:getAddAttr(EquipCalcMode.kWash)
	print(string.format("装备初始化宿主:%s,初始属性如下:", hero:getName()))
	for i , info in pairs(attr) do
		hero:addAttr(info.attrId,info.value)
		print(string.format("%s增加:%d",AttrName[info.attrId],info.value))
	end
end

function This:setHeroId(heroId)
	if self.m_heroId == heroId then
		return
	end
	if self.m_heroId and self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		local attr = self:getAddAttr(EquipCalcMode.kWash)
		--local levelAttr = self:getUpLevelAttr(self.m_level)
		--for id,value in pairs(levelAttr) do
		--	table.insert(attr,{attrId = id,value = value})
		--end
		print(string.format("\n宿主:%s,减少属性如下:",hero:getName()))
		for i , info in pairs(attr) do
			hero:reduceAttr(info.attrId,info.value)
			print(string.format("%s减少:%d",AttrName[info.attrId],info.value))
		end
	end
	self.m_heroId = heroId
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		local attr = self:getAddAttr(EquipCalcMode.kWash)
		--local levelAttr = self:getUpLevelAttr(self.m_level)
		--for id,value in pairs(levelAttr) do
		--	table.insert(attr,{attrId = id,value = value})
		--end
		print(string.format("宿主:%s,增加属性如下:",hero:getName()))
		for i , info in pairs(attr) do
			hero:addAttr(info.attrId,info.value)
			print(string.format("%s增加:%d",AttrName[info.attrId],info.value))
		end
	end
end

function This:createItem(info)
    local itemInfo = info
    if not itemInfo then
    	itemInfo = {id=self.m_uid, type_id=self.m_equipId, num=1, equip=self}
    else
    	itemInfo.equip=self
    end
    return ItemUtil.createItemModel(itemInfo)
end

function This:getItem()
    return self.m_item
end

function This:upgradeQuality( newEquipId )
    if newEquipId and newEquipId ~= self:getEquipId() then
		self.m_equipId = newEquipId
		self.m_config = EquipConfig[self.m_equipId]
		self.m_item:setEquipId(self.m_equipId)
    end
end

function This:updateClearAttr(clearList)
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		for _ , id in pairs(self.m_clearList) do 
			local info = FoundationAttributeConfig[id]
			print(string.format("%s减少:%f",AttrName[info.attribute_type],info.attribute_value))
			hero:reduceAttr(info.attribute_type,info.attribute_value)
		end
		for _ , v in pairs(clearList) do 
			local info = FoundationAttributeConfig[v.wash_id]
			print(string.format("%s增加:%f",AttrName[info.attribute_type],info.attribute_value))
			hero:addAttr(info.attribute_type,info.attribute_value)
		end
		self.m_clearList = {}
		for _ ,v in pairs(clearList) do 
			self.m_clearList[v.index] = v.wash_id
		end
		hero:updateEquipClear()
		hero:updateFightPower()
	else
		self.m_clearList = {}
		for _ ,v in pairs(clearList) do 
			self.m_clearList[v.index] = v.wash_id
		end
	end

end

function This:getClearTotalLevel()
	local level = 0
	if self.m_clearList then
	    for _, clearId in pairs(self.m_clearList) do
	    	local config = FoundationAttributeConfig[clearId]
	    	level = level + config.star_level
	    end
	end
	return level
end

function This:canReborn()
	if self.m_heroId <= 0 then
	    if (self.m_strengthLv > 0) or (self.m_clearList and #self.m_clearList > 0) or (self.m_level > 0) then
	    	return true
	    end
    end
    return false
end

function This:getRebornReturn(curExp)
	local ret = {}
	local list = {}
	local levelList = {}
	if self.m_strengthLv > 0 then
        local configId = self:getStrengthUpConfigId(self.m_strengthLv)
		local config = EquipUpConfig[configId].return_item
		table.insert(list, config)
	end
	if self.m_level >0 then
		local configId = self:getWearPos()*1000+self:getLevel()
		local config = EquipLevelUpConfig[configId].return_item
		table.insert(levelList, config)
	end
	for _, config in ipairs(list) do
		for _, info in ipairs(config) do
			local id, num = unpack(info)
			ret[id] = num + (ret[id] or 0)
		end
	end
	for _,config in ipairs(levelList) do
		for _, info in ipairs(config) do
			local id, num = unpack(info)
			ret[id] = num + (ret[id] or 0)
		end
	end

	if curExp and curExp > 0 then
		local affConfig = EquipmentQualityConfig[self:getQuality()]
		if affConfig then
			local times = math.ceil(curExp / affConfig.exp)
			local config = affConfig.return_item
			for _, info in ipairs(config) do
				local id, num = unpack(info)
				print(id, num)
				ret[id] = num * times + (ret[id] or 0)
			end
		end
	end
	return ret
end

--endregion
