-- @Author: superm
-- @Date:   2018-01-17 14:22:09
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-09-16 14:16:37
-- @File Name: EquipAscendNode.lua
-- @Description: 

local This, super = class(..., "NodeView")
This.fileName = "GameScene/NodeUI/EquipAscendItem.csb"

local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local EquipManager = require("Equip.EquipManager")
local Serialize = require("Serialize.Serialize")
local EquipConfig = require("Metadatas.EquipConfig")
local EquipUpConfig = require("Metadatas.EquipUpConfig")
local FoundationAttributeConfig = require("Metadatas.FoundationAttributeConfig")
local ItemNode = require("UI.ItemNode")
local ItemUtil = require("Bag.ItemUtil")

function This:initData(dialog)
	self.m_cache = {}
    self.m_dialog = dialog
end

function This:initNode()
    self.m_inited = false
end

function This:initTouchEvent()
    if not self.m_inited then
		self.ascendbotton:setTouchEnded(function()
			self:upgradeQuality()
		end)

		self.putin:setTouchEnded(function()
			self:quicklyPutIn()
		end)

		self.help:setTouchEnded(function()
			g_scene:openDialog("UI.HelpTips", {text = Serialize["equip_ascend_help_tips"], node = self.help})
		end)

		for i=1, 5 do
			self.itemtipslist["need" .. i].add:setTouch({
				onEnded = function()
					if self.m_subView then
						return
					else
		        		self.m_subView = true
					end

					local panel = self:chooseEquip(i)
			        panel:addCloseCallback(function ()
			            self.m_subView = nil
			        end)
				end,
				isDynamic = true,
				isScale = false,
			})

			self.itemtipslist["item" .. i].close:setTouch({
				onEnded = function()
					self:revokeChosen(i)
				end,
				isDynamic = true,
				isScale = false,
				isAlpha = false,
			})
		end
        self.m_inited = true
    end
end

function This:refresh()
	if self.m_inited then
		self:refreshAttr()
	end
end

function This:getEquipUid()
    return self.m_dialog:getEquipUid()
end

function This:refreshAttr()
	local equip = EquipManager:getEquip(self:getEquipUid())
	if equip then 
		self:setVisible(true)
	else
		self:setVisible(false)
		return 
	end
	local config = equip:getAscendCost()
	local equipId = equip:getEquipId()
	print("装备界面更新", equipId)
	self:resetCostNodeList( equip, equipId )
	local curLv = equip:getStrengthLv()
	local level = equip:getLevel()
	local curQuality = equip:getQuality()

	local curAttr = equip:getAddAttr()
	if self.m_itemNode == nil then
		local itemNode = ItemNode:create(equipId)
		itemNode:setAnchorPoint(0.5,0.5)
		itemNode:setToEquip(curLv,level)
		self.EquipItem:addChild(itemNode)
		self.m_itemNode = itemNode
	else
		self.m_itemNode:initItem(equipId)
		self.m_itemNode:setToEquip(curLv,level)
	end

	local afterEquipId = equipId + 1
	local nextAttr
	if curQuality < kMaxQuality and #config > 0 then
		nextAttr = equip:getAddAttr(EquipCalcMode.kAscend)
	end

	if nextAttr and next(nextAttr) ~= nil then
		for i , info in ipairs(curAttr) do
			self.before["property"..i]:setVisible(true)
			self.before["property"..i.."num"]:setVisible(true)

			self.before["property"..i]:setString(AttrName[info.attrId])
			self.before["property"..i.."num"]:setString(string.format("+%d",info.value))
		end
		for i = #curAttr + 1, 4 do
			self.before["property"..i]:setVisible(false)
			self.before["property"..i.."num"]:setVisible(false)
		end

		for i , info in ipairs(nextAttr) do
			self.after["property"..i]:setVisible(true)
			self.after["property"..i.."num"]:setVisible(true)	
					
			self.after["property"..i]:setString(AttrName[info.attrId])
			self.after["property"..i.."num"]:setString(string.format("+%d",info.value))
		end
		for i = #nextAttr + 1, 4 do
			self.after["property"..i]:setVisible(false)
			self.after["property"..i.."num"]:setVisible(false)
		end

		local curLevel, eLevel = GamePlayer:getLevel(), equip:getUpgradePlayerLvLimit()
		local visible = curLevel >= eLevel
		self.ascendbotton:setVisible(visible)
		self.putin:setVisible(visible)
		self.itemtipslist:setVisible(visible)
		local str = string.format(Serialize["equip_AscendNode_1_c"], eLevel)
		self.tixing:setString(str)
		self.tixing:setVisible(not visible)
	else
		afterEquipId = equipId
		if self.m_afterItemNode then
			self.m_afterItemNode:setVisible(false)
		end

		for i , info in ipairs(curAttr) do
			local name = AttrName[info.attrId]
			self.before["property"..i]:setVisible(true)
			self.before["property"..i.."num"]:setVisible(true)
			self.after["property"..i]:setVisible(true)
			self.after["property"..i.."num"]:setVisible(true)	

			self.before["property"..i]:setString(name)
			self.before["property"..i.."num"]:setString(string.format("+%d",info.value))
			self.after["property"..i]:setString(name)
			self.after["property"..i.."num"]:setString(Serialize["equip_AscendNode_2_c"])
		end
		for i = #curAttr + 1, 4 do
			self.before["property"..i]:setVisible(false)
			self.before["property"..i.."num"]:setVisible(false)
			self.after["property"..i]:setVisible(false)
			self.after["property"..i.."num"]:setVisible(false)
		end
		self.ascendbotton:setVisible(false)
		self.putin:setVisible(false)
		self.tixing:setString(Serialize["equip_AscendNode_2_c"])
		self.tixing:setVisible(true)
	end

	if self.m_afterItemNode == nil then
		local itemNode = ItemNode:create(afterEquipId)
		itemNode:setAnchorPoint(0.5,0.5)
		itemNode:setToEquip(curLv,level)
		self.EquipItemafter:addChild(itemNode)
		self.m_afterItemNode = itemNode
	else
		self.m_afterItemNode:setVisible(true)
		self.m_afterItemNode:initItem(afterEquipId)
		self.m_afterItemNode:setToEquip(curLv,level)
	end
end

function This:getCostInfo(equip, equipId)
	local record = self.m_cache[equipId]
	if not record then
		local config = equip:getAscendCost()
		self.m_costItemList, self.m_costEquipList = {}, {}
		self.m_costEquipCount, self.m_costItemCount = 0, 0

		if config then
			for i, itemInfo in ipairs(config) do
				local id, num = unpack(itemInfo)
				local itype = ItemUtil.getType(id)
				if itype == ItemCommonType.kEquip then
					table.insert(self.m_costEquipList, itemInfo)
					self.m_costEquipCount = self.m_costEquipCount + 1
				else
					table.insert(self.m_costItemList, itemInfo)
					self.m_costItemCount = self.m_costItemCount + 1
				end
			end

			self.m_cache[equip:getEquipId()] = {
				item = self.m_costItemList,
				equip = self.m_costEquipList,
				equipCount = self.m_costEquipCount,
				itemCount = self.m_costItemCount,
			} 
			print("装备界面更新", self.m_costEquipList, self.m_costItemList)
		end
	else
		self.m_costEquipList, self.m_costItemList = record.equip, record.item
		self.m_costEquipCount, self.m_costItemCount = record.equipCount, record.itemCount
	end
	self.m_fileterList = {0,0,0,0,0, equip:getUid()}
end

function This:resetCostNodeList(equip, equipId)
	self.m_quicklyPutIn = nil
	self:getCostInfo(equip, equipId)

	for i = 1, 5 do
		self.itemtipslist["item" .. i]:setVisible(false)
	end
	local viewCount = self.m_costEquipCount + self.m_costItemCount
	local itemWidth = self.itemtipslist.need1:getContentWidth() * self.itemtipslist.need1:getScaleX()
	local w = itemWidth / 2
	for i= 1, viewCount do
		local needNode = self.itemtipslist["need" .. i]
		local node = self.itemtipslist["item" .. i]
		needNode:setVisible(true)
		needNode:setPositionX(w)
		node:setVisible(false)
		node:setPositionX(w)
		w = w + itemWidth + 10
	end

	self.itemtipslist:setPositionX( 170 - (itemWidth + 10) * viewCount / 2 )

	for i= viewCount + 1, 5 do
		self.itemtipslist["need" .. i]:setVisible(false)
	end

	self:putInCostItem()
end

function This:quicklyPutIn()
	local indexList = {}
	local slotCount = self.m_costEquipCount
	for i, uid in ipairs(self.m_fileterList) do
		if i > slotCount then
			break
		end

		if uid <= 0 then
			table.insert(indexList, i)
		end
	end

	local emptyCount = #indexList
	if emptyCount > 0 then
		local equipUid = self:getEquipUid()
		local equip = EquipManager:getEquip(self:getEquipUid())
		
		local equipList, count = EquipManager:getEquipUnWearedById(equip:getEquipId(), self.m_fileterList)
		if emptyCount > count then
			if count <= 0 then
				g_scene:showTips(Serialize["equip_AscendNode_3_c"])
			end
			self.m_quicklyPutIn = false
		end
		for i, index in ipairs(indexList) do
			local newEquip = equipList[i]
			if newEquip then
				self:putInChosen(newEquip, index)
			else
				return
			end
		end
	else
		g_scene:showTips(Serialize["equip_AscendNode_4"])
	end
	self.m_quicklyPutIn = true
end

function This:chooseEquip(pos)
	local equipUid = self:getEquipUid()
	local equip = EquipManager:getEquip(self:getEquipUid())
	local panel = g_scene:openDialog("Equip.EquipAscendChoosePanel",{filter = self.m_fileterList, id = equip:getEquipId()})
	panel:setSelectHandler(function(Uid)
		self:putInChosen(Uid, pos)
	end)
	return panel
end

function This:putInCostItem()
	for i, itemInfo in ipairs(self.m_costItemList) do
        local id, num = unpack(itemInfo)
		local pos = self.m_costEquipCount + i 
		self.itemtipslist["need" .. pos]:setVisible(false)
		local node = self.itemtipslist["item" .. pos]
		if not node.itemNode then
			local itemNode = ItemNode:create(id, num, true, true)
			itemNode:setAnchorPoint(0.5, 0.5)
			node:addChild(itemNode)
			node.itemNode = itemNode
			node.close:setLocalZOrder(10)
		else
			node.itemNode:initItem(id, num, true)
		end 
		node.itemNode:showNeedNum()
		node.close:setVisible(false)
		node:setVisible(true)
	end
end

function This:putInChosen(Uid, pos)
	local node = self.itemtipslist["item" .. pos]
	if Uid and Uid > 0 then
		self.m_fileterList[pos] = Uid
		local chosen = EquipManager:getEquip(Uid)
		self.itemtipslist["need" .. pos]:setVisible(false)
		if not node.itemNode then
			local itemNode = ItemNode:create(chosen:getEquipId(), nil, false, true)
			itemNode:setAnchorPoint(0.5, 0.5)
			itemNode:setToEquip(chosen:getStrengthLv(),chosen:getLevel())
			node:addChild(itemNode)
			node.itemNode = itemNode
			node.close:setLocalZOrder(10)
		else
			node.itemNode:initItem(chosen:getEquipId(), nil, false)
			node.itemNode:setToEquip(chosen:getStrengthLv(),chosen:getLevel())
		end 
		node.close:setVisible(true)
		node:setVisible(true)
	else
		node:setVisible(false)
	end
end

function This:revokeChosen(pos)
	if pos <= self.m_costEquipCount then
		self.m_quicklyPutIn = false

	    self.m_fileterList[pos] = 0
	    self.itemtipslist["item" .. pos]:setVisible(false)
	    self.itemtipslist["need" .. pos]:setVisible(true)
	end   	
end

function This:upgradeQuality()
	local paramList = {}
	for i, uid in ipairs(self.m_fileterList) do
		if i > self.m_costEquipCount then
			break
		end

		if uid <= 0 then
			g_scene:showTips(Serialize["activity_AssassinsPanel_1_c"])
			return 
		end
		table.insert(paramList, uid)
	end

	for i, itemInfo in ipairs(self.m_costItemList) do
		local pos = self.m_costEquipCount + i 
		local node = self.itemtipslist["item" .. pos]
		local total, num, uid = node.itemNode:showNeedNum()
		if total >= num then
			if uid > 0 then
				table.insert(paramList, uid)
			end
		else
			g_scene:showTips(Serialize["activity_AssassinsPanel_1_c"])
			return
		end 
	end

	local equip = EquipManager:getEquip(self:getEquipUid())
	if equip:getQuality() >= kMaxQuality then
		g_scene:showTips(Serialize["equip_AscendNode_5"])
		return
	elseif GamePlayer:getLevel() < equip:getUpgradePlayerLvLimit() then
		g_scene:showTips(string.format(Serialize["equip_AscendNode_1_c"], equip:getUpgradePlayerLvLimit()))
		return
	end

	self.clearAttrList = {}
	self.paramList = paramList
	for _ , uid in ipairs(paramList) do 
		local equip = EquipManager:getEquip(uid)
		if equip then 
			local equipId = equip:getEquipId()
			EquipManager:onReqSeeEquipClearData(uid,equipId,2)
		end
	end
	-- EquipManager:onEquipUpgradeQualityRequest(self.m_fileterList[6], paramList)
end

function This:updateClearLevel(attributeList)
	table.insert(self.clearAttrList,attributeList)
	if #self.clearAttrList >= 3 then
		for _ , attrList in ipairs(self.clearAttrList) do 
			if #attrList ~= 0 then 
	            local panel = g_scene:openDialog("Equip.EquipClearProblemPanel", {
	                notips = "Ascend", 
            		type = 2,
	                okHandler = function()
	                    EquipManager:onEquipUpgradeQualityRequest(self.m_fileterList[6], self.paramList)
	                end 
	            })					
				return 
			end
		end
		EquipManager:onEquipUpgradeQualityRequest(self.m_fileterList[6], self.paramList)
	end
end

function This:onEnter()
	super.onEnter(self)
	EquipManager:addObserver(self)
	self:refresh()
end

function This:onExit()
	EquipManager:removeObserver(self)
	super.onExit(self)
end

function This:onEquipUpgradeQuality()
	cc.Audio:playEffect("Recruit_Change.mp3")
	local equip = EquipManager:getEquip(self:getEquipUid())
	local curAttr = equip:getAddAttr()
	for i = 1,#curAttr do
		self.effect["effect"..i]:removeAllChildren()
		if self["m_effect"..i] then
			self["m_effect"..i] = nil
		end
		self["m_effect"..i] = GameUtil:createEffect("ShuXing")
		self["m_effect"..i]:setScale(0.5)
		self.effect["effect"..i]:addChild(self["m_effect"..i])
	end
end