-- @Author: ZYY
-- @Date:   2018-08-21 10:36:02
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-16 15:58:45
-- @File Name: EquipClearNode.lua
-- @Desc: 
local GameUtil = require("Util.GameUtil")
local This , super = class(...,"NodeUI")
local FoundationAttributeConfig = require("Metadatas.FoundationAttributeConfig")
local EquipmentQualityConfig = require("Metadatas.EquipmentQualityConfig")
local AffinationMasterConfig = require("Metadatas.AffinationMasterConfig")
local AffinationFactorConfig = require("Metadatas.AffinationFactorConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local StarColorConfig = require("Metadatas.StarColorConfig")
local EquipManager = require("Equip.EquipManager")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")

This.fileName = "GameScene/NodeUI/EquipClear.csb"

function This:initNode(dialog)
	self.m_dialog = dialog
    -- self.itemCost = {}
    self.selectAttr = 1
    self.m_selectFull = false
    self.m_masterLevel = 0
end

function This:refresh()
    self.m_equipUid = self:getEquipUid()
    if self.m_equipUid and self.m_equipUid > 0 then
        self:setVisible(true)
    else
        self:setVisible(false)
        return 
    end    
    local equip = EquipManager:getEquip(self.m_equipUid)
    local equipId = equip:getEquipId()
    self.selectAttr = 1
    self.m_selectFull = false
	EquipManager:onReqSeeEquipClearData(self.m_equipUid,equipId,3)
    self:updateTick()
    self:refreshFullItem()
    self:updateCost()
    self:updateArrow(0)
end

function This:onEnter()
	super.onEnter(self)
	EquipManager:addObserver(self)
	self:refresh()
end

function This:onExit()
	EquipManager:removeObserver(self)
	super.onExit(self)
end

function This:getEquipUid()
    return self.m_dialog:getEquipUid()
end

function This:getEquipHeroId()
    return self.m_dialog:getEquipHeroId()
end


function This:updateEquipClearMaster(masterData)
    local data = {}
    for i , v in ipairs(AffinationMasterConfig) do 
        if v.level == masterData.level then 
            data = v
            break
        end
    end
    local maxExp = data.exp
    local minStar = data.min_star
    local maxStar = data.max_star
    local minQuality = StarColorConfig[minStar].star_color
    local maxQuality = StarColorConfig[maxStar].star_color
    if masterData.exp >= maxExp then 
        self.clear.value:setString(Serialize[60769])
    else
        self.clear.value:setString(string.format("%d/%d",masterData.exp , maxExp))
    end
    self.clear.jindutiao:setPercent(masterData.exp  / maxExp * 100)
    self.clear.masterLevel:setString(masterData.level)
    self.m_masterLevel = masterData.level
    self.clear.masterMin:setString(minStar)
    self.clear.masterMax:setString(maxStar)
    self["clear"]["masterMin"]:setColorByID("Q"..minQuality) 
    self["clear"]["masterMax"]:setColorByID("Q"..maxQuality)        
    self["clear"]["minStar"]:setImage(string.format("Common/star%d.png",minQuality + 1)) 
    self["clear"]["maxStar"]:setImage(string.format("Common/star%d.png",maxQuality + 1)) 

end

function This:updateEquipClearAttribute(uid,attributeList)
    self.m_equipUid = uid
    self.attributeList = {}
    for i ,v in pairs(attributeList) do 
        self.attributeList[v.index] = v.wash_id
    end
    local equip = EquipManager:getEquip(uid)
    if self.m_itemNode == nil then
        local itemNode = ItemNode:create(equip:getEquipId())
        itemNode:setAnchorPoint(0.5,0.5)
        itemNode:setToEquip(equip:getStrengthLv(),equip:getLevel())
        self.clear.itemEquip:addChild(itemNode)
        self.m_itemNode = itemNode
    else
        self.m_itemNode:initItem(equip:getEquipId())
        self.m_itemNode:setToEquip(equip:getStrengthLv(),equip:getLevel())
    end
    local lv = equip:getStrengthLv()
    local fator = AffinationFactorConfig[lv].fator
    for i = 1 , 4 do 
        if self.attributeList[i] then 
            self:updateAttribute(self.m_equipUid,i,self.attributeList[i])
        else
            self["clear"]["attr"..i]:setString(Serialize[60770])
            self["clear"]["attr"..i]:setColorByID("New8")
            if i <= fator then 
                self["clear"]["maxAttr"..i]:setString(Serialize[60771])
                self["clear"]["maxAttr"..i]:setColorByID("G2")
            else
                local openLv = 1
                for _, v in ipairs(AffinationFactorConfig) do 
                    if v.fator == i then 
                        openLv = v.id
                        break
                    end
                end
                self["clear"]["maxAttr"..i]:setString(string.format(Serialize[60772],openLv))
                self["clear"]["maxAttr"..i]:setColorByID("New8")
            end
            self["clear"]["attrStar"..i]:setVisible(false)
        end
        self["clear"]["gou"..i]:setVisible(i <= fator)
    end
    self:updateArrow(0)

end

function This:initTouchEvent()
    for i = 1 , 4 do 
        self["clear"]["gou"..i]:setTouchEnded(function()
            self.selectAttr = i
            self:updateTick()
        end)
    end
    self["clear"]["selectgou"]:setTouchEnded(function()
        self.m_selectFull = not self.m_selectFull
        self:refreshFullItem()
    end)

    self.clear.bg.info:setTouchEnded(function()
        g_scene:openDialog("Equip.EquipClearMasterInfoPanel",{level = self.m_masterLevel})
    end)

    self.clear.cost1.buttonClear:setTouchEnded(function()
        local equip = EquipManager:getEquip(self.m_equipUid)
        local function clearFun(isUse)
            local equiptype = equip:getWearPos()   
            EquipManager:onReqEquipClear(self.m_equipUid,self.selectAttr,equiptype,isUse)            
        end
        local function tipFun(isUse)
            local washId = self.attributeList[self.selectAttr]
            if not washId or FoundationAttributeConfig[washId].star_level < ClearOpenTipsLevel then 
                clearFun(isUse)
            else
                local panel = g_scene:openDialog("Equip.EquipClearProblemPanel", {
                    notips = "Clear", 
                    type = 1,
                    okHandler = function()
                        clearFun(isUse)
                    end 
                })
            end
        end
        local quality = equip:getQuality()
        local costList = EquipmentQualityConfig[quality].cost
        local enoughSelectFull = not self.m_selectFull
        for i , info in ipairs(costList) do 
            local id = info[1] 
            local count = DepotManager:getItemOrResCount(id)
            if count < info[2] then 
                if id == 101 then 
                    local panel = g_scene:openDialog("Territory.TerrCommonConfirmPanel")
                    panel:setString(Serialize[60773])
                    panel:setOkHandler(function()
                        g_scene:openDialog("System.GlodChangePanel", info)
                    end)
                else
                    g_scene:showTips(Serialize[60774])
                    g_scene:openDialog("Equip.EquipClearStonePanel")
                end
                return 
            end
        end
        if enoughSelectFull  then
            tipFun()
        elseif self.m_selectFull and DepotManager:getItemOrResEnought({self:getFullItem()},true) then
            tipFun(self.m_selectFull)
        end
    end)

    self.clear.cost2.cancel:setTouchEnded(function()
        EquipManager:onReqClearReplace(self.m_equipUid,self.replaceIndex,self.replaceId,0)
    end)

    self.clear.cost2.preservation:setTouchEnded(function()
        EquipManager:onReqClearReplace(self.m_equipUid,self.replaceIndex,self.replaceId,1)
    end)

    self.clear.help:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize["clear_help_tips"],node = self.clear.help})
    end)

    self.clear.bg.buy:setTouchEnded(function()
        g_scene:openDialog("Equip.EquipClearStonePanel")
    end)
    
    self.clear.dashi.kuang:setTouchEnded(function()
        local heroId = self:getEquipHeroId()
        if not heroId or heroId <= 0 then
            g_scene:showTips(Serialize[60775])
            return
        end
        if GamePlayer:getLevel() < MasterOpenLevel then 
            g_scene:showTips(Serialize[60758]..MasterOpenLevel ..Serialize[60759])
            return 
        end
        g_scene:openDialog("Equip.EquipClearMasterPanel",{heroId = heroId,tabType = EquipClearMasterSubTabType.Clear})
    end)
end

function This:updateTick()
    for i = 1 , 4 do 
        self["clear"]["gou"..i]["light"]:setVisible(i == self.selectAttr)
    end
end

function This:updateArrow(value)
    for i = 1 , 4 do 
        self["clear"]["arrow"..i]:setVisible(value == i)
        self["clear"]["maxStar"..i]:setVisible(value == i)
    end
end

function This:getFullItem()
    return ArgConfig.equipment_cost.arg
end

function This:refreshFullItem()
    local selectlight = self["clear"]["selectgou"]["light"]
    selectlight:setVisible(self.m_selectFull)
end

function This:updateCost()
    local equip = EquipManager:getEquip(self.m_equipUid)
    local quality = equip:getQuality()
    local costList = table.clone(EquipmentQualityConfig[quality].cost)
    table.insert(costList,self:getFullItem())
    local max =#costList
    for j = 1, max do
        local parent = self["clear"]["itemCost"..j]
        GameUtil:creatItemNode(parent,costList[j],true)   
    end
end

function This:updateReplaceAttribute(index,washId)
    self.replaceIndex = index
    self.replaceId = washId
    local attr = FoundationAttributeConfig[washId] 
    local star = attr.star_level
    local desc = attr.attribute_desc
    local quality = StarColorConfig[star].star_color
    if star < 10 then
        self["clear"]["maxAttr"..index]:setString(string.format(" %d     ：%s",star,desc))
    else
        self["clear"]["maxAttr"..index]:setString(string.format("%d    ：%s",star,desc))
    end    
    self["clear"]["maxStar"..index]:setImage(string.format("Common/star%d.png",quality + 1)) 
    self["clear"]["maxAttr"..index]:setColorByID("Q"..quality)        
    self:updateArrow(index)
end

function This:updateAttribute(uid,index,washId)
    if uid ~= self.m_equipUid then return end 
    local attr = FoundationAttributeConfig[washId] 
    local star = attr.star_level
    local desc = attr.attribute_desc
    local quality = StarColorConfig[star].star_color
    if star < 10 then
        self["clear"]["attr"..index]:setString(string.format(" %d     ：%s",star,desc))
    else
        self["clear"]["attr"..index]:setString(string.format("%d    ：%s",star,desc))
    end
    self["clear"]["attr"..index]:setColorByID("Q"..quality)
    self["clear"]["attrStar"..index]:setImage(string.format("Common/star%d.png",quality + 1)) 
    self["clear"]["attrStar"..index]:setVisible(true)
    local maxId = washId
    for i = washId + 1, #FoundationAttributeConfig do 
        if FoundationAttributeConfig[i].equip_type == attr.equip_type and FoundationAttributeConfig[i].attribute_type == attr.attribute_type then 
            maxId = i
        else
            break
        end
    end
    local attr1 = FoundationAttributeConfig[maxId] 
    local star1 = attr1.star_level
    local value1 = attr1.attribute_value
    local quality1 = StarColorConfig[star1].star_color
    local attrId1 = attr1.attribute_type
    local desc1 = ""
    if AttrInfo[attrId1].valueType == ValueType.kPercent then
        desc1 = string.format("+%.2f%%",value1 * 100)
    else
        desc1 = string.format("+%d",value1)
    end
    if maxId == washId then 
        self["clear"]["maxAttr"..index]:setString(string.format(Serialize[60776],desc1))
    else
        self["clear"]["maxAttr"..index]:setString(string.format(Serialize[60777],desc1))
    end
    self["clear"]["maxAttr"..index]:setColorByID("Q"..quality1)
end

function This:updateCostState(state)
    self.clear.cost1:setVisible(state)
    self.clear.cost2:setVisible(not state)
    self:updateCost()
end

function This:clearEffect(index,state)
    local x , y 
    if state == 1 then 
        x = 270 +50
        cc.Audio:playEffect("Recruit_Change.mp3")
    else --洗炼
        x = 40 +50
        cc.Audio:playEffect("KeepValue.mp3")
    end
    y = 83.22 - 27.83 * (index - 1)
    local effectClear = GameUtil:createEffect("KaPaiShengJi", false, self)
    effectClear:setPosition(x, y)
    effectClear:setRotation(90)
    effectClear:setScale(0.08,0.45)
    self.clear:addChild(effectClear)             

    self:setTimer(1.5, function()
        effectClear:removeFromParent()
    end, "effectClear")
end
