--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local Equip = require("Equip.Equip")
local EquipConfig = require("Metadatas.EquipConfig")
local Socket = require("Net.Socket")
local EventDispatcher = require("EventDispatcher")
local EquipUpConfig = require("Metadatas.EquipUpConfig")
local ItemConfig = require("Metadatas.ItemConfig")
local HeroManager = require("Hero.HeroManager")
local AffinationStoreConfig = require("Metadatas.AffinationStoreConfig")

local ErrorCode = 
{
	kToMaxStrlengthLv = "to_max_strength_lv",
	kGreaterPlayerLevel = "greater_player_level", 
	kBaseEquipNotEnough = "base_equip_not_enough",
	kGoldNotEnough = "goldLack",
	kDataError = "data_error",
}

-- local function print(...)
-- 	print_r(...)
-- end

function This:init()
	self.m_equips = {}
	self.m_observers = {}
	self:initListener()
end

function This:addObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			return 
		end
	end
	table.insert(self.m_observers,observer)
end

function This:removeObserver(observer)
	for i , o in pairs(self.m_observers) do
		if o == observer then
			table.remove(self.m_observers,i)
			return
		end
	end
end

function This:notifyObserver(funcName,...)
	local tmp = {}
	for i, observer in pairs(self.m_observers) do
		tmp[i] = observer
	end

	for i , observer in pairs(tmp) do
		if observer[funcName] then
			observer[funcName](observer,...)
		end
	end
end

function This:initListener()
	EventDispatcher:addListener(self,10406,self.onStrengthLevelReply)
	EventDispatcher:addListener(self,10600,self.onGetEquipNotify)
	EventDispatcher:addListener(self,10601,self.onEquipChangedNotify)
	EventDispatcher:addListener(self,10415,self.onEquipUpgradeQualityReply)
	EventDispatcher:addListener(self,10445,self.onEquipUpLevelReply)


	EventDispatcher:addListener(self,12800,self.onAnsSeeEquipClearData)
	EventDispatcher:addListener(self,12801,self.onAnsEquipClear)
	EventDispatcher:addListener(self,12802,self.onAnsClearReplace)
	EventDispatcher:addListener(self,12803,self.onAnsStoneData)
	EventDispatcher:addListener(self,12804,self.onAnsStoneBuy)
	EventDispatcher:addListener(self,12805,self.onAnsEquipMaster)
end

function This:removeListener()
	EventDispatcher:removeListener(self,10406)
	EventDispatcher:removeListener(self,10600)
	EventDispatcher:removeListener(self,10601)
	EventDispatcher:removeListener(self,10415)
end

function This:getAllEquip()
	return self.m_equips
end

local function equipItemSortComp(item1, item2)
    local quality1, quality2 = item1:getQuality(), item2:getQuality()
    if quality1 ~= quality2 then
        return quality1 > quality2
    end

    local lev1, lev2 = item1:getStrengthLv(), item2:getStrengthLv()
    if lev1 ~= lev2 then
        return lev1 > lev2
    end

    if item1.id ~= item2.id then
        return item1.id < item2.id
    else
        return item1._id < item2._id
    end
end

function This:getEquipUnWeared(pos)
	local tmp = {}
	for i , equip in pairs(self.m_equips) do
		if (pos == nil or equip:getWearPos() == pos) and equip:getHeroId() <= 0 then
			table.insert(tmp,equip)
		end
	end
	table.sort(tmp,function(a, b)
		local qualityA = a:getQuality() 
		local qualityB = b:getQuality()
		if qualityA ~= qualityB then
			return qualityA > qualityB
		end
		if a:getStrengthLv() ~= b:getStrengthLv() then
			return a:getStrengthLv() > b:getStrengthLv() 
		end
		return a:getUid() < b:getUid()
	end)

	local result = {}
	for i , equip in ipairs(tmp) do
		table.insert(result,equip:getUid())
	end
	return result
end

function This:getEquipUnWearedById(equipId, filterList)
	local tmp, count = {}, 0
	if not filterList then
		for i , equip in pairs(self.m_equips) do
			local equipUid = equip:getUid()
			if equip:getEquipId() == equipId and equip:getHeroId() <= 0 then
				table.insert(tmp,equip)
				count = count + 1
			end
		end		
	else
		for i , equip in pairs(self.m_equips) do
			local equipUid = equip:getUid()
			if equip:getEquipId() == equipId and not table.contain(filterList, equipUid) and equip:getHeroId() <= 0 then
				table.insert(tmp,equip)
				count = count + 1
			end
		end
	end

	table.sort(tmp,function(a, b)
		if a:getStrengthLv() ~= b:getStrengthLv() then
			return a:getStrengthLv() < b:getStrengthLv() 
		end
		return a:getUid() < b:getUid()
	end)

	table.sort(tmp,function(a, b)
		if a:getLevel() ~= b:getLevel() then
			return a:getLevel() < b:getLevel() 
		end
		return a:getUid() < b:getUid()
	end)

	local result = {}
	for i , equip in ipairs(tmp) do
		table.insert(result,equip:getUid())
	end
	return result, count
end

function This:getEquipUnWearedByIdList( equipIdList, filterList )
	local tmp = {}
	for i, equip in pairs(self.m_equips) do
		local equipId = equip:geEquipId()
		local equipUid = equip:geUid()
		if not filterList then
			if table.contain(equipIdList, equipId) and equip:getHeroId() <= 0 then
				table.insert(tmp, equip)
			end
		else
			if table.contain(equipIdList, equipId) and not table.contain(filterList, equipUid) and equip:getHeroId() <= 0 then
				table.insert(tmp, equip)
			end
		end
	end
	table.sort(tmp, function(a, b)
		local qualityA = a:getQuality() 
		local qualityB = b:getQuality()
		if qualityA ~= qualityB then
			return qualityA < qualityB
		end
		if a:getStrengthLv() ~= b:getStrengthLv() then
			return a:getStrengthLv() < b:getStrengthLv() 
		end
		return a:getUid() < b:getUid()
	end)

	local result = {}
	for i , equip in ipairs(tmp) do
		table.insert(result,equip:getUid())
	end
	return result
end

function This:getBaseEquipCount(equip)
	local equipId = equip:getEquipId()
	local count = 0
	for i , eq in pairs(self.m_equips) do
		if eq:getStrengthLv() ==0 and eq:getEquipId() == equipId and equip:getUid() ~= eq:getUid() and eq:getHeroId() <= 0 then
			count = count + 1
		end
	end
	return count
end

function This:getEquip(uid, log)
	local ret = self.m_equips[uid]
	if not ret and log then
		local GameUtil = require("Util.GameUtil")
		print("equipment not found, uid:" .. GameUtil:int64ToString(uid))
		print("==========================dump equipment list:")
		for key in pairs(self.m_equips) do
			print(GameUtil:int64ToString(key))
		end	
		print("==========================dump end")	
	end
	return ret
end

function This:addEquip(info, itemInfo)
	local equip = Equip:create(info, itemInfo)
	self.m_equips[info.uid] = equip
	self:notifyObserver("onAddEquip",info.uid)
end

function This:removeEquip(uid)
	if self.m_equips[uid] then
		self.m_equips[uid] = nil
		self:notifyObserver("onRemoveEquip",uid)
	end
end

function This:updateEquip(info)
	if self.m_equips[info.uid] then
		local equip = self.m_equips[info.uid]
		equip:updateData(info)
		self:notifyObserver("onUpdateEquip",info.uid)
	end
end

function This:checkEquip(info, itemInfo)
	local equip = self:getEquip(info.uid)
	if equip then
		if info.num <= 0 then
			self:removeEquip(info.uid)
		else
			self:updateEquip(info)
		end
	else
		self:addEquip(info, itemInfo)
	end
end

function This:onGetEquipRequest()
	Socket:sendAndLock(10600, {})
end

function This:onGetEquipNotify(message)
	--print_tmp(message)
	local list = message.bag_item_list
	for i , info in pairs(list) do
		if  ItemConfig[info.type_id].itype == 2 then
			local hero = HeroManager:getHeroByUid(info.equip_card_id)
			local heroId = hero and hero:getId() or 0
			local t = {
				uid = info.id , 
				equipId = info.type_id, 
				progress = info.process, 
				strengthLv = info.strength_level, 
				heroId = heroId,
				num = info.num, 
				clearList = info.wash_attribute,
				level = info.level
			}
			self:checkEquip(t)
			-- print("装备信息：",table.tostring(t))
		elseif ItemConfig[info.type_id].itype == 12 then
			require("Awake.LordEquipManager"):setInscriptionAttrForSerever(info)	
		end
	end
	EventDispatcher:dispatcher("equip_init",message)
end

function This:onEquipChangedNotify(message)
	-- print_tmp(message)
	local list = message.bag_item_list
	for i , info in pairs(list) do
		if  ItemConfig[info.type_id].itype == 2 then
			local hero = HeroManager:getHeroByUid(info.equip_card_id)
			local heroId = hero and hero:getId() or 0
			local t = {
				uid = info.id, 
				equipId = info.type_id, 
				progress = info.process, 
				strengthLv = info.strength_level, 
				heroId = heroId,
				num = info.num, 
				clearList = info.wash_attribute,
				level = info.level
			}
			self:checkEquip(t, info)
			-- print("装备信息：",table.tostring(t))
		elseif ItemConfig[info.type_id].itype == 12 then
			require("Awake.LordEquipManager"):setInscriptionAttrForSerever(info)	
		end
	end
	EventDispatcher:dispatcher("equip_changed",message)
end

function This:onStrengthLevelRequest(uid, strengthenType)
	Socket:sendAndLock(10406,{id = uid, strengthen_type = strengthenType})
end

function This:onStrengthLevelReply(message)
	if message.err_code == 0 then
		if message.succ then
			if message.card then
				HeroManager:calcSaintFightPower()
			end
			g_scene:showTips(Serialize["equip_EquipManager_1"])
		else
			g_scene:showTips(Serialize["equip_EquipManager_2"])
		end
		self:notifyObserver("onStrengthLevel",message.id,message.succ)
	else
		g_scene:handleErrorCode(message.err_code)
	end
end

function This:onEquipUpLevelRequest(itemId,level)
	Socket:sendAndLock(10445,{item_id = itemId, up_level = level})
end

function This:onEquipUpLevelReply( message )
	if message.err_code == 0 then
		HeroManager:calcSaintFightPower()
		self:notifyObserver("onEquipUpLevel",message.item_id)
	else
		g_scene:handleErrorCode(message.err_code)
	end
end
--------------------------------------------------

function This:onAddEquipRequest(equipId)
	local tmp = {}
	for id , config in pairs(EquipConfig) do
		table.insert(tmp,{equipId = id ,strengthLv = 0})
	end
	Socket:sendAndLock("Equip_addEquip",{data = tmp})
end

function This:onAddEquipReply(message)
	if message.result then
		print("添加成功")
	end
end

function This:onRemoveEquipRequest(equipId)
	local tmp={}
	for i , equip in pairs(self.m_equips) do
		table.insert(tmp,equip:getUid())
	end
	Socket:sendAndLock("Equip_removeEquip",{data = tmp})
end

function This:onRemoveEquipReply(message)
	if message.result then
		print("删除成功")
	end
end

function This:onUpdateEquipNotify(message)
	local equip = self:getEquip(message.uid)
	equip:updateData(message)
	self:notifyObserver("onEquipChanged",message.uid)
end

-- function This:onRemoveEquipNotify(message)
-- 	print("删除装备",table.tostring(message))
-- 	self.m_equips[message.uid] = nil
-- 	self:notifyObserver("onRemoveEquip",message.uid)
-- end

-- function This:onAddEquipNotify(message)
-- 	if not self.m_equips[message.uid] then
-- 		print("添加装备",table.tostring(message))
-- 		local equip = Equip:create(message)
-- 		self.m_equips[message.uid] = equip
-- 		self:notifyObserver("onAddEquip",message.uid)
-- 	end
-- end

function This:getEquipItemList()
	local itemList = {}
	for k,equip in pairs(self.m_equips) do
		if equip:getHeroId() <= 0 then
			table.insert(itemList, equip:getItem())
		end
	end
	if #itemList > 1 then
		table.sort(itemList, equipItemSortComp)
	end
	return itemList
end

--function This:test1()
--	self:onAddEquipRequest()
--end
--
--function This:test2()
--	self:onRemoveEquipRequest()
--end
--
--function This:test3()
--	local tmp = {}
--	for i , equip in pairs(self.m_equips) do
--		table.insert(tmp,equip:getUid())
--	end
--	local index = math.random(1,#tmp)
--	self:onStrengthLevelRequest(tmp[1])
--end

function This:onEquipUpgradeQualityRequest(sourceUId, resourceUIdList)
	print("装备升阶: sourceUId=", sourceUId, "resourceUId=" .. table.tostring(resourceUIdList))
	Socket:sendAndLock(10415, {source_item_id = sourceUId, cailiao_item_id_list = resourceUIdList})
end

function This:onEquipUpgradeQualityReply( msg )
    if msg.err_code == 0 then
    	print("装备升阶成功", table.tostring(msg))
        HeroManager:calcSaintFightPower()
        self:notifyObserver("onEquipUpgradeQuality", msg.source_item_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onReqSeeEquipClearData(uid,pos,typeId)
	Socket:sendAndLock(12800,{item_uid = uid , item_id = pos , type_id = typeId or 1})
end

function This:onAnsSeeEquipClearData(msg)
    if msg.err_code == 0 then
    	if msg.type_id == 1 or msg.type_id ==3 then --查看
	        self:notifyObserver("updateEquipClearMaster", msg.washmaster)
	        self:notifyObserver("updateEquipClearAttribute", msg.item_uid,msg.wash_attribute)
	        if msg.current_wash_attribute.index ~= 0 then 
		        self:notifyObserver("updateReplaceAttribute", msg.current_wash_attribute.index,msg.current_wash_attribute.wash_id)
		        self:notifyObserver("updateCostState", false)
		    else	        	
		        self:notifyObserver("updateCostState", true)	        	
	        end
	    elseif msg.type_id == 2 then --升阶
	        self:notifyObserver("updateClearLevel", msg.wash_attribute)
	    end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onReqEquipClear(uid,index,pos,userItem)
	Socket:sendAndLock(12801,{item_uid = uid , index = index, type = pos,user_item = userItem or false})
end

function This:onAnsEquipClear(msg)
    if msg.err_code == 0 then
        self:notifyObserver("updateEquipClearMaster", msg.washmaster)
        self:notifyObserver("updateReplaceAttribute", msg.index,msg.wash_id)
        self:notifyObserver("updateCostState", false)
        self:notifyObserver("clearEffect", msg.index,1)
    else
        g_scene:handleErrorCode(msg.err_code)
    end	
end

function This:onReqClearReplace(uid,index,wathId,state)
	Socket:sendAndLock(12802,{item_uid = uid , index = index, wash_id = wathId,type = state})

end

function This:onAnsClearReplace(msg)
    if msg.err_code == 0 then
	    local equip = self:getEquip(msg.item_uid)
	    local equipId = equip:getEquipId()
		self:onReqSeeEquipClearData(msg.item_uid,equipId)
        -- self:notifyObserver("updateAttribute", msg.item_uid,msg.index,msg.wash_id)
        self:notifyObserver("updateCostState", true)
        if msg.wash_id ~= 0 then 
        	self:notifyObserver("clearEffect", msg.index,2)
        	local heroId = equip:getHeroId()
        	if heroId and heroId ~= 0 then 
				self:notifyObserver("onEquipChanged",heroId)
			end
        	HeroManager:calcSaintFightPower()
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end		
end

function This:onReqStoneData()
	Socket:sendAndLock(12803,{})
end

function This:onAnsStoneData(msg)
	print('ZYY:EquipManager.lua [start:478] :data',msg)
    self:notifyObserver("updateBuyNumber", msg.buytimeinfolist)
end

function This:onReqStoneBuy(buyId)
	Socket:sendAndLock(12804,{buy_type = buyId})
end

function This:onAnsStoneBuy(msg)
	if msg.err_code == 0 then
		local list = {buy_type = msg.buy_type, buy_time = msg.buy_time}
    	self:notifyObserver("updateBuyNumber", {list})
    	self:notifyObserver("updateCost")
    	local rewards = AffinationStoreConfig[msg.buy_type].reward
    	g_scene:showReward(rewards)
    else
        g_scene:handleErrorCode(msg.err_code)
    end	
end

function This:onReqEquipMaster(cardUid,type)
	Socket:sendAndLock(12805,{card_id = cardUid,type = type})
end

function This:onAnsEquipMaster(msg)
	if msg.err_code == 0 then
    	self:notifyObserver("updateEquipMasterAttr", msg)
    else
        g_scene:handleErrorCode(msg.err_code)
    end		
end
--endregion
