-- @Author: superm
-- @Date:   2018-01-03 20:49:04
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-12 15:42:06
-- @File Name: EquipStrengthItemView.lua
-- @Description: 

local This , super = class(..., "NodeView")
This.csfileName = "GameScene/NodeUI/EquipStrengthenHero.csb"

local ItemUtil = require("Bag.ItemUtil")
local EquipManager = require("Equip.EquipManager")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")

function This:initData(hero, inedx, handler)
    self.m_inited = false
    self.m_hero = hero
    self.m_pos = EquipPos.kWeapons
    self.m_handler = handler
    self:initCsNode()
end

function This:initCsNode()
    self.m_csNode = cc.Node:createFromCSFile(self.csfileName)
    self:addChild(self.m_csNode)
    local w , h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)
end

function This:initNode()
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	local nodeList = {csNode.weaponslot, csNode.clothesslot, csNode.horseslot}
		for pos = EquipPos.kWeapons, EquipPos.kHorse do
			local node = nodeList[pos]
    		node:setTouch({
				onEnded = function()
					self:onEquipPosClicked(pos)
				end,
				isScale = false,
				isContainer = true,
			})

			node.jianhao:setTouch({
				onEnded = function()
					self:onEquipUnload(pos)
					self:setSelected(false)
				end,
				isScale = false,
				isContainer = true,
			})
    	end
        self.m_inited = true
    end
end

function This:setModel(hero, index)
	if hero:getUid() ~= self.m_hero:getUid() then
		self.m_hero = hero
		self:refresh()
	end
end

function This:getHeroId()
    return self.m_hero:getId()
end

function This:onEquipPosClicked(pos)
	self.m_pos = pos
	local isWear = self.m_hero:isWearEquip(pos)
	if not isWear then
		self:chooseEquip(pos)
	else
		self:showEquipTips(pos)
		self:setSelected(pos)
	end
end

function This:onEquipUnload(pos)
	self.m_pos = pos
	local isWear = self.m_hero:isWearEquip(pos)
	if isWear then 
		local panel = g_scene:openDialog("UI.CostConfirmPanel", {
			cost = 0, 
			simple = Serialize[60785], 
			notips = "EquipUnload", 
			okHandler = function()
				HeroManager:onUpdateEquipRequest(self:getHeroId(),pos,0)
		    end 
		})		
	end
end


function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
end

function This:onExit()
	HeroManager:removeObserver(self)
	super.onExit(self)
end

function This:onEquipChanged(heroId)
	if heroId == self.m_hero:getId() then
		self:refreshEquipment()
		self:showEquipTips(self.m_pos)
	end 
end

function This:chooseEquip(pos)
	local panel = g_scene:openDialog("Equip.EquipChoosePanel",{pos = pos})
	panel:setSelectHandler(function(uid)
		HeroManager:onUpdateEquipRequest(self.m_hero:getId() ,pos,uid)
	end)
end

function This:showEquipTips(pos)
	local equipUid = self.m_hero:getEquipUid(pos)
	if self.m_handler then
		self.m_handler(equipUid, pos, self)
	end
end

function This:refresh()
    local csNode = self.m_csNode
	csNode.icon:setImage(ItemUtil.getIconImage(self.m_hero:getId()))
	csNode.name:setString(self.m_hero:getName())
	csNode.name:setColorByID("Q".. self.m_hero:getQuality())
	self:refreshEquipment()
end

function This:refreshEquipment()
	local image = "MenuScene/3003.png"
	local csNode = self.m_csNode
	local nodeList = {csNode.weaponslot, csNode.clothesslot, csNode.horseslot}
	for i = EquipPos.kWeapons , EquipPos.kHorse do
		local equipUid = self.m_hero:getEquipUid(i)
		local node = nodeList[i]
		if equipUid and equipUid > 0 then
			local equip = EquipManager:getEquip(equipUid)
			if not node.itemNode then
				local itemNode = ItemNode:create(equip:getEquipId(), nil, false, false)
				itemNode:setAnchorPoint(0.5,0.5)
				itemNode:setToEquip(equip:getStrengthLv(),equip:getLevel())
				node.icon:addChild(itemNode)
				node.itemNode = itemNode
			else
				node.itemNode:initItem(equip:getEquipId())
				node.itemNode:setToEquip(equip:getStrengthLv(),equip:getLevel())
			end 
			node.jianhao:setVisible(self.m_selectedPos == i)
		else
			if node.itemNode then
				node.itemNode:removeFromParent()
				node.itemNode = nil
			end
			node.jianhao:setVisible(false)
		end
	end
end

function This:setSelected(pos)
	local csNode = self.m_csNode
	local nodeList = {csNode.weaponslot, csNode.clothesslot, csNode.horseslot}
	for i = EquipPos.kWeapons , EquipPos.kHorse do
		nodeList[i].jianhao:setVisible(pos == i)
	end
	self.m_selectedPos = pos
end