-- @Author: superm
-- @Date:   2018-01-16 20:19:52
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-09-17 15:43:09
-- @File Name: EquipStrengthNode.lua
-- @Description: 


local This, super = class(..., "NodeView")
This.fileName = "GameScene/NodeUI/EquipStrengthenItem.csb"

local GamePlayer = require("Player.GamePlayer")
local EquipManager = require("Equip.EquipManager")
local Serialize = require("Serialize.Serialize")
local EquipConfig = require("Metadatas.EquipConfig")
local EquipUpConfig = require("Metadatas.EquipUpConfig")
local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")
local TabGroup = require("TabGroup")

function This:initData(dialog)
    self.m_dialog = dialog
	self.m_strengthenType = StrengthenType.kPerfect
end

function This:initNode()
    self.m_inited = false

	self.m_tabGroup = TabGroup:new()
	self.m_tabGroup:addTabNode(StrengthenType.kLow, self.fenye.di)
	self.m_tabGroup:addTabNode(StrengthenType.kIntermediate, self.fenye.zhong)
	self.m_tabGroup:addTabNode(StrengthenType.kPerfect, self.fenye.wan)
	self.m_tabGroup:selectTab(StrengthenType.kPerfect)
	self.m_tabGroup:setOnCurrentTabChangedListener(function(currentTabKey)
		self.m_strengthenType = currentTabKey
		self:refresh()
	end)
end

function This:initTouchEvent()
    if not self.m_inited then
		-- 强化按钮
		self.strengthenbotton:setTouchEnded(function()
			local equip = EquipManager:getEquip(self:getEquipUid())
			if equip:getStrengthLv() >= kMaxStrengthLv then
				g_scene:showTips(Serialize["to_max_strength_lv"])
			elseif GamePlayer:getLevel() < equip:getPlayerLvLimit() then
				g_scene:showTips(string.format(Serialize["equip_AscendNode_1_c"], equip:getPlayerLvLimit()))
			else
				EquipManager:onStrengthLevelRequest(self:getEquipUid(), self.m_strengthenType)
			end
		end)

		self.help:setTouchEnded(function()
			g_scene:openDialog("UI.HelpTips", {text = Serialize["equip_strength_help_tips"], node = self.help})
		end)
		
		self.dashi.kuang:setTouchEnded(function()
	        local heroId = self:getEquipHeroId()
	        if not heroId or heroId <= 0 then
	            g_scene:showTips(Serialize[60787])
	            return
	        end
	        if GamePlayer:getLevel() < MasterOpenLevel then 
	            g_scene:showTips(Serialize[60758]..MasterOpenLevel ..Serialize[60759])
	            return 
	        end
	        g_scene:openDialog("Equip.EquipClearMasterPanel",{heroId = heroId,tabType = EquipClearMasterSubTabType.Strength})
		end)
        self.m_inited = true
    end
end

function This:onEnter()
    super.onEnter(self)
	EquipManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
	EquipManager:removeObserver(self)
end


function This:refresh()
	if self.m_inited then
        local equipUid = self:getEquipUid()
        if equipUid and equipUid > 0 then
		    self:refreshAttr()
            self:setVisible(true)
        else
            self:setVisible(false)
        end
	end
end

function This:getEquipUid()
    return self.m_dialog:getEquipUid()
end

function This:getEquipHeroId()
    return self.m_dialog:getEquipHeroId()
end

function This:refreshAttr()
	local equip = EquipManager:getEquip(self:getEquipUid())
	local curLv = equip:getStrengthLv()
	local level = equip:getLevel()
	local curAttr = equip:getAddAttr()
	if self.m_itemNode == nil then
		local itemNode = ItemNode:create(equip:getEquipId())
		itemNode:setAnchorPoint(0.5,0.5)
		itemNode:setToEquip(curLv,level)
		self.EquipItem:addChild(itemNode)
		self.m_itemNode = itemNode
	else
		self.m_itemNode:initItem(equip:getEquipId())
		self.m_itemNode:setToEquip(curLv,level)
	end

	self.before.strengthen:setString(string.format("+%d",curLv))
	
	if curLv < kMaxStrengthLv then
		for i , info in ipairs(curAttr) do
			self.before["property"..i]:setVisible(true)
			self.before["property"..i.."num"]:setVisible(true)

			self.before["property"..i]:setString(AttrName[info.attrId])
			self.before["property"..i.."num"]:setString(string.format("+%d",info.value))
		end
		for i = #curAttr + 1, 4 do
			self.before["property"..i]:setVisible(false)
			self.before["property"..i.."num"]:setVisible(false)
		end

		local nextLv = curLv + 1
		local nextAttr = equip:getAddAttr(EquipCalcMode.kStrength, {level = nextLv})
		self.after.strengthen:setString(string.format("+%d",nextLv))
		for i , info in ipairs(nextAttr) do
			self.after["property"..i]:setVisible(true)
			self.after["property"..i.."num"]:setVisible(true)	
					
			self.after["property"..i]:setString(AttrName[info.attrId])
			self.after["property"..i.."num"]:setString(string.format("+%d",info.value))
		end
		for i = #nextAttr + 1, 4 do
			self.after["property"..i]:setVisible(false)
			self.after["property"..i.."num"]:setVisible(false)
		end

		self:refreshItems(equip)
		local nextLv = math.min(kMaxStrengthLv,equip:getStrengthLv() + 1) + equip:getConfig().type * 1000
		local baseCount = EquipManager:getBaseEquipCount(equip)
		local costCount = EquipUpConfig[nextLv].cost_count
		local maxCount = EquipUpConfig[nextLv].max_times
		local luckyValue
		local ratio
		local config = EquipUpConfig[nextLv]
		if self.m_strengthenType == StrengthenType.kLow then
			ratio = config.ratio1
			luckyValue = config.luck1
		elseif self.m_strengthenType == StrengthenType.kIntermediate then
			ratio = config.ratio2
			luckyValue = config.luck2
		elseif self.m_strengthenType == StrengthenType.kPerfect then
			ratio = config.ratio3
			luckyValue = config.luck3
		else
			error("error!")
		end

		self.lucky.completionnumber:setString(string.format("%d/%d",equip:getProgress(),maxCount))
		self.chenggonglv:setVisible(true)
		self.chenggonglv:setString(string.format(Serialize["equip_StrengthNode_1_c"],ratio * 100))
		self.lucky.completionbar:setPercent(equip:getProgress() / maxCount * 100)
		-- self.cost:setString(equip:getStrengthGoldCost())

		local curLevel, eLevel = GamePlayer:getLevel(), equip:getPlayerLvLimit()
		local visible = curLevel >= eLevel
		self.strengthenbotton:setVisible(visible)
		local str = string.format(Serialize["equip_AscendNode_1_c"], eLevel)
		self.tixing:setString(str)
		self.tixing:setVisible(not visible)
		self.lucky:setVisible(visible)
		self.strengthenbotton:setVisible(true)

		-- self.icon:setVisible(true)
		-- self.iconbg:setVisible(true)
		-- self.cost:setVisible(true)
		self.xingyunzhi:setVisible(true)
		self.xingyunzhi:setString(Serialize[60788]..luckyValue)

		self.fenye:setVisible(true)
		self.kuang:setVisible(true)
	else
		self.after.strengthen:setString("")
		for i , info in ipairs(curAttr) do
			local attrName = AttrName[info.attrId]
			self.before["property"..i]:setString(attrName)
			self.before["property"..i]:setVisible(true)
			self.before["property"..i.."num"]:setVisible(true)
			self.after["property"..i]:setVisible(true)
			self.after["property"..i.."num"]:setVisible(true)

			self.before["property"..i.."num"]:setString(string.format("+%d",info.value))
			self.after["property"..i.."num"]:setString(Serialize["equip_AscendNode_2_c"])
		end
		for i = #curAttr + 1, 4 do
			self.before["property"..i]:setVisible(false)
			self.before["property"..i.."num"]:setVisible(false)
			self.after["property"..i]:setVisible(false)
			self.after["property"..i.."num"]:setVisible(false)
		end	
		self.chenggonglv:setVisible(false)
		self.items:setVisible(false)
		self.tixing:setVisible(false)
		self.lucky:setVisible(false)
		self.tixing:setVisible(true)
		self.tixing:setString(Serialize["equip_AscendNode_2_c"])
		self.strengthenbotton:setVisible(false)
		self.xingyunzhi:setVisible(false)

		self.fenye:setVisible(false)
		self.kuang:setVisible(false)
	end
end

-- 刷新消耗品
function This:refreshItems(equip)
	local itemList = equip:getCostItemList(self.m_strengthenType)
	local viewCount, tw = #itemList, 0
	local offsetX = 120 / viewCount
	local itemWidth

	self.items:setVisible(true)
	self.items:removeAllChildren()
	for i , info in ipairs(itemList) do
		local itemNode = ItemNode:create(info.id, info.num, true)
		itemNode:setAnchorPoint(0,0.5)
		itemNode:showNeedNum(info.num)
		self.items:addChild(itemNode)
		if not itemWidth then
			itemWidth = itemNode:getContentWidth()
		end
		itemNode:setPositionX(tw)
		tw = tw + itemWidth + offsetX
	end

	self.items:setPositionX(170-tw/2+67)
end

function This:onStrengthLevel(uid,succ)
	if succ then
		cc.Audio:playEffect("Recruit_Change.mp3")
		local equip = EquipManager:getEquip(self:getEquipUid())
		local curAttr = equip:getAddAttr()
		for i = 1,#curAttr do
			self.effect["effect"..i]:removeAllChildren()
			if self["m_effect"..i] then
				self["m_effect"..i] = nil
			end
			self["m_effect"..i] = GameUtil:createEffect("ShuXing")
			self["m_effect"..i]:setScale(0.5)
			self.effect["effect"..i]:addChild(self["m_effect"..i])
		end
	end
end