-- @Date:   2017-11-09 16:22:34
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-02 10:30:55
-- @File Name: EquipStrengthPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local EquipManager = require("Equip.EquipManager")
local HeroManager = require("Hero.HeroManager")

local RankPageListLayer = require("Rank.RankPageListLayer")
local DepotItemView = require("Depot.DepotItemView")
local EquipStrengthItemView = require("Equip.EquipStrengthItemView")
local GamePlayer = require("Player.GamePlayer")

EquipClearMasterSubTabType = {
	Strength = 1,	--强化
	Clear = 2,		--洗炼
}

local EquipTabType = {
	hero = 1,
	backpack = 2,
}

local EquipSubTabType = {
	Strength = 1,	--强化
	Ascend = 2,		--升阶
	Clear = 3,		--洗炼
	UpLevel = 4,    --升级
}

function This:initData( info )
	if info then
		if info.tab then
			self.m_curType = EquipTabType[info.tab]
		end
		if info.subTab then
			self.m_curSubType = EquipSubTabType[info.subTab]
		end
		self.m_equipType = info.equipType
		self.m_equipUid = info.equipUid
		self.m_heroId = info.heroId 
		self.m_hintIndex = info.hint
		self.m_isShowEquipHero = false
		self:initHeroOrder()
    end
end

function This:initNode(info)
	self.m_listChoose = {}
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/EquipStrengthen.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self:initListViews()
	self:initTabs()
	self:selectTips()
end

function This:initHeroOrder()
	self.m_noDeputyHeros = HeroManager:getNotDeputyHeros()
	table.sort(self.m_noDeputyHeros,function(a,b)
		if a:getQuality() > b:getQuality() then
			return true
		elseif a:getQuality() == b:getQuality() and a:getFightPower() > b:getFightPower() then
			return true
		elseif a:getQuality() == b:getQuality() and a:getFightPower() == b:getFightPower() and a:getId() < b:getId() then
			return true
		end
		return false
	end)
		
	local count = 0
	self.m_allHeros = {}
	for i , hero in ipairs(self.m_noDeputyHeros) do
		if self.m_isShowEquipHero or hero:hasWearEquip() then
			table.insert(self.m_allHeros,hero)
			count = count + 1
			if self.m_heroId and not self.m_hintIndex and hero:getId() == self.m_heroId then
				self.m_hintIndex = count
			end
		end
		local deputy = hero:getDeputyHeros()
		for j , id in pairs(deputy) do
			local o = HeroManager:getHero(id)
			if self.m_isShowEquipHero or o:hasWearEquip() then
				table.insert(self.m_allHeros,o)
				count = count + 1
				if self.m_heroId and not self.m_hintIndex and o:getId() == self.m_heroId then
					self.m_hintIndex = count
				end
			end
		end
	end

	if self.m_heroId and self.m_hintIndex and not self.m_equipType and not self.m_equipUid then
		local hero = self.m_allHeros[self.m_hintIndex]
		if hero then
			self.m_equipType = EquipPos.kWeapons
			self.m_equipUid = hero:getEquipUid(EquipPos.kWeapons)
		end
	end
	--if not self.m_isShowEquipHero then
	--	local res = self.m_allHeros
	--	for index,hero in pairs(res)do
	--		if not hero:hasWearEquip() then
	--			self.m_allHeros[index] = nil
	--		end
	--	end
	--end
end

function This:initListViews()
	local function createListLayer(param)
		local container = param.scrollView
		local chooseView = container.choose
		chooseView:setLocalZOrder(1)

	    local listLayer = RankPageListLayer:create(param)
	    listLayer:setAnchorPoint(0.5,0)
	    local w , h = listLayer:getContentSize()
	    container:pushBackCustomItem(listLayer, w, h, 0)
	    listLayer:setAnchorPoint(0.5,0)
	    listLayer:setCascadeOpacityEnabled(true)
	    return listLayer, chooseView
	end

    self.m_bagListLayer, self.m_bagChoose = createListLayer({
        itemWidth = 70,
        itemHeight = 70,
        spaceX = 10,
        spaceY = 5,
        column = 4,
        scrollView = self.m_csNode.bagScrollView,
        dataCounter = function ()
        	return #self.m_itemList
        end,
        viewCreator = function ( index, node )
        	local itemInfo = self.m_itemList[index]
        	if not itemInfo then return end
        	if not node then
		        local view = DepotItemView:create({
		        	item = itemInfo, 
		    		index = index, 
		    		handler = function( itemNode )
			            self.m_equipUid = itemNode.item:getUid()
			        	self.m_curItemView = itemNode
			            local x, y = itemNode:getPosition()
			            self.m_bagChoose:setPosition(x - 2, y - 3)
			            self:refreshTabUI()
		        	end
		        })
		        if self.m_equipUid == itemInfo:getUid() then
		            self.m_curItemView = view
		        end
		        view:setScale(0.81)
		        return view
        	else
		        node:setModel(itemInfo)
        	end 
        end,
        finishHandler = function()    	
	        self.m_bagChoose:setVisible(true)
	        if self.m_curItemView then
	            local x, y = self.m_curItemView:getPosition()
	            self.m_bagChoose:setPosition(x - 2, y - 3)
	        end
	        self:refreshTabUI()
	    end,
        hint = self.m_hintIndex
    })

    self.m_heroListLayer, self.m_heroChoose = createListLayer({
        itemWidth = 325,
        itemHeight = 95,
        spaceX = 2,
        spaceY = 5,
        column = 1,
        scrollView = self.m_csNode.heroScrollView,
        dataCounter = function ()
        	return #self.m_allHeros
        end,
        viewCreator = function ( index, node )
        	local ret
        	local heroInfo = self.m_allHeros[index]
        	if not heroInfo then return end
        	if not node then
		        ret = EquipStrengthItemView:create(heroInfo, index, function( equipUid, pos, itemNode )
		        	if self.m_curItemView ~= itemNode then
		        		if self.m_curItemView then
		        			self.m_curItemView:setSelected(false)
		        		end
			        	self.m_curItemView = itemNode
			        end
		            self.m_equipUid = equipUid 
		            self.m_equipType = pos
		            self.m_heroId = itemNode:getHeroId()
		            local x, y = itemNode:getPosition()
		            self.m_heroChoose:setPosition(x + (pos - 2.56) * 78, y + 4)
		            self:refreshTabUI()
		        end)
        	else
		        node:setModel(heroInfo)
        	end

		    if not self.m_curItemView and self.m_heroId and self.m_heroId == heroInfo:getId() then
		        self.m_curItemView = ret or node
		    end
        	return ret
        end,
        finishHandler = function()
			self.m_heroChoose:setVisible(true)
	        if self.m_curItemView then
	        	if self.m_equipUid and self.m_equipType then
		        	if self.m_equipUid > 0 then
			            local x, y = self.m_curItemView:getPosition()
			        	self.m_curItemView:setSelected(self.m_equipType)
			            self.m_heroChoose:setPosition(x + (self.m_equipType - 2.56) * 78, y + 4)
		       		else
						self.m_curItemView:chooseEquip(self.m_equipType)
					end
	       		end
	        end
	        self:refreshTabUI()
	    end,
        hint = self.m_hintIndex
    })
end

function This:getTabNodeData(name)
	return self
end

function This:getEquipUid()
    return self.m_equipUid
end

function This:getEquipHeroId()
	return self.m_heroId
end

function This:initTabs()
	for name, tab in pairs(EquipTabType) do
		local button = self.m_csNode[name]
	    self:addTabButton(tostring(tab), button, function(subTabHandler)
	    	if self.m_curType == tab then
	    		return false
	    	end

	    	if subTabHandler then
	    		subTabHandler()
	    	end
	        self.m_curType = tab
			self:selectTabById(tab)
	        return true
	    end)
	end

	for _, value in pairs(EquipSubTabType) do
		self:addSubTabButton(tostring(value), self.m_csNode["tep" .. value], function ()
	    	if self.m_curSubType == value then
	    		return false
	    	end
	    	if value == EquipSubTabType.Clear and GamePlayer:getLevel() < ClearOpenLevel then 
	    		g_scene:showTips(Serialize[60758]..ClearOpenLevel ..Serialize[60759])
	    		return false
	    	end
		    self.m_curSubType = value
	    	self:refreshTabUI()
			return true
		end, function (name)
		    self.m_curSubType = value
			return true
		end)
	end
	self.m_csNode.tep3.suo:setVisible(GamePlayer:getLevel() < ClearOpenLevel)
end

function This:refreshTabUI()
	local node
	if self.m_curSubType == EquipSubTabType.Strength then
	    node = self:openTabNode("Equip.EquipStrengthNode")
	elseif self.m_curSubType == EquipSubTabType.Ascend then 
	    node = self:openTabNode("Equip.EquipAscendNode")
	elseif self.m_curSubType == EquipSubTabType.Clear then 
		node = self:openTabNode("Equip.EquipClearNode")
	elseif self.m_curSubType == EquipSubTabType.UpLevel then
		node = self:openTabNode("Equip.EquipUpLevelNode") 
	end
	local tipsType = {Serialize[60790],Serialize[60791],Serialize[60792],Serialize[60793]}
	self.m_csNode.tips:setString(string.format(Serialize[60794],tipsType[self.m_curSubType]))

	if node then
		node:refresh()
	end
end

function This:refreshBagList(cleanup)
	local csNode = self.m_csNode
	if self.m_curItemView then
		if cleanup then
			self.m_curItemView:refresh()
		else
			self.m_curItemView:setSelected(false)
		end
	end
	self.m_curItemView = nil
	csNode.bagScrollView:setVisible(true)
	csNode.heroScrollView:setVisible(false)
	self.m_bagChoose:setVisible(false)
    self.m_itemList = EquipManager:getEquipItemList()
	self:refreshTabUI()
    self.m_bagListLayer:setDataList() 
end

function This:refreshHeroList(cleanup)
	local csNode = self.m_csNode
	if self.m_curItemView then
		if cleanup then
			self.m_curItemView:refresh()
		end
	end
	self.m_curItemView = nil
	csNode.bagScrollView:setVisible(false)
	csNode.heroScrollView:setVisible(true)

	self:refreshTabUI()
    self.m_heroListLayer:setDataList()
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.see.weigouxuan:setTouchEnded(function ()
		self.m_isShowEquipHero = false
		self:selectTips()
	end)
	self.m_csNode.see.gouxuan:setTouchEnded(function ()
		self.m_isShowEquipHero = true
		self:selectTips()
	end)
end

function This:selectTips( )
	local node = self.m_csNode.see
	node.weigouxuan:setVisible(self.m_isShowEquipHero)
	node.gouxuan:setVisible(not self.m_isShowEquipHero)
	self:initHeroOrder()
	if self.m_curTabId == EquipTabType.hero then
		self:refreshHeroList(true)
	end
end

function This:selectTabById( index, cleanup )
	if self.m_curTabId ~= index then
		if index then
			self.m_curTabId = index
		end

		if self.m_curTabId == EquipTabType.hero then
			self:refreshHeroList(cleanup)
		else
			self:refreshBagList(cleanup)
		end
	end
end

function This:onEnter()
	super.onEnter(self)
	EquipManager:addObserver(self)
	local tab = self.m_curType or EquipTabType.hero
	self.m_curType = nil
    self.m_curSubType = self.m_curSubType or EquipSubTabType.UpLevel
	self:selectTabButton(tostring(tab), tostring(self.m_curSubType))
end	

function This:onExit()
	super.onExit(self)
	EquipManager:removeObserver(self)
end

function This:onStrengthLevel(equipUid)
	if self.m_equipUid == equipUid then
		self:refreshTabUI()
		if self.m_curItemView then
			self.m_curItemView:refresh()
		end
	end
end

function This:onEquipUpLevel(equipUid)
	if self.m_equipUid == equipUid then
		self:refreshTabUI()
		if self.m_curItemView then
			self.m_curItemView:refresh()
		end
	end
end

function This:onEquipUpgradeQuality(equipUid)
	if self.m_equipUid == equipUid or equipUid == 0 then
		self:selectTabById(nil, true)
	end 
end

-----------------------------
function This:selectTabButton(name, defaultSubTabName)
    if self.m_tabButtons then
        if self.m_selectTabButton == name then
            return
        end

		local handler
	    if self.m_subTabButtons then
	        local subTabName, button
	        if defaultSubTabName then
	        	subTabName, button = defaultSubTabName, self.m_subTabButtons[defaultSubTabName]
	        end

	        if not button then
	        	subTabName, button = next(self.m_subTabButtons)
	        end

	        handler = function()
	        	if button.onReseted() ~= false then
		            self:lightSubTabButton(subTabName)
		        end
	        end
	    end

	    if self.m_tabButtons[name].onSelected(handler) ~= false then
            self:lightTabButton(name)
        end
    end
end


function This:selectSubTabButton(name)
    if self.m_subTabButtons then
        if self.m_selectSubTabButton == name then
            return
        end
        if self.m_subTabButtons[name] then
            if self.m_subTabButtons[name].onSelected(self.m_selectTabButton) ~= false then
                self:lightSubTabButton(name)
            end
        end
    end
end

function This:addSubTabButton(name, button, onSelected, onReseted)
    if not self.m_tabButtons then
        return
    end

    if not self.m_subTabButtons then
    	self.m_subTabButtons = {}
	elseif self.m_subTabButtons[name] then
        return
    end
    self.m_subTabButtons[name] = button

    button:setTouch({
        onEnded = function ()
            self:selectSubTabButton(name)
        end
    })
    button.onSelected = onSelected
    button.onReseted = onReseted
end


function This:lightSubTabButton(name)
    self.m_selectSubTabButton = name
    if self.m_subTabButtons[name] then
        for k, node in pairs(self.m_subTabButtons) do
            if k == name then
                if node.light then
                    node.light:setVisible(true)
                end
                if node.normal then
                    node.normal:setVisible(false)
                end
            else
                if node.light then
                    node.light:setVisible(false)
                end
                if node.normal then
                    node.normal:setVisible(true)
                end
            end
        end
    end
end

--endregion
