-- @Author: zheng
-- @Date:   2019-09-10 20:11:04
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-09 14:11:41
-- @File Name: RuneMakePanel.lua
local ArgConfig = require("Metadatas.ArgConfig")
local This , super = class(...,"Dialog")
local RuneManager = require("Equip.RuneManager")
local GamePlayer = require("Player.GamePlayer")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
This.fileName = "GameScene/NodeUI/HeroSkillRuneSynthesis.csb"
local ItemUtil = require("Bag.ItemUtil")
local RuneProbabilityConfig = require("Metadatas.RuneProbabilityConfig")
local RuneSynthesisConfig = require("Metadatas.RuneSynthesisConfig")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")
local makeAnim1 ={
    HeCheng ="FSHC_HeCheng",
    Icon ="FSHC_Icon",
    ShuSong ="FSHC_ShuSong",
    ZhongXin ="FSHC_ZhongXin",
}
function This:initNode(info)
    if info then
       self.m_family = info.family
    end
    self.m_isWillId =nil
    self.m_fileterList = {false, false, false, false,false}
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self:resetQuality()
    self:removeAllChose()
    self:getData()
    self:refresh()
end

function This:refresh()
    local data = self.m_data
    local maxNum = DailyAchieveManager:getArgConfigLv("rune_melting_max_integral")
    local curent = GamePlayer:getPlayerAttrWithType(PlayerAttr.runnestone_composite_return)
    local csNode = self.m_csNode
    local ronglianzhi = csNode.ronglianzhi
    ronglianzhi.num:setString(curent)
    ronglianzhi.line:setPercent(curent/maxNum*100,true)
    
    local rewards = ArgConfig.rune_melting_max_return_item.arg
    local a = rewards[1]
    local b = rewards[1][1]
    ronglianzhi.icon:setImage(ItemUtil.getIconImage(b))

    local function createAnim( parent,anmStr )
        -- local GameUtil = require("Util.GameUtil")
        local effect1 = GameUtil:createEffect(anmStr,true)
        parent:addChild(effect1,-1)
        return effect1
    end
   
    local ronglianzhiEffect = ronglianzhi.kuang
    ronglianzhiEffect:removeAllChildren()
    if curent>=maxNum then
        local effect1 = createAnim(ronglianzhiEffect,"BoxLing")
        local w ,h = ronglianzhiEffect:getContentSize()
        effect1:setPosition(w/2, h/2)
        local effect2 = createAnim(ronglianzhiEffect,"BoxLing2")
        effect2:setPosition(w/2, h/2)
    end
    self:refreshSucessRate()
end

function This:createAnim(parent,anmStr,isLoop,isEnd)
    local name ="anim"
    if parent.name then
        parent.name:removeFromParent(true)
        parent.name = nil
    end
    if not anmStr then
        return 
    end
    local isEndFun ={
                    onAnimationComplete = function()
                        self:removeAllChose()
                        self:refresh()

                        if self.m_makeResult.result==1 then
                           g_scene:showTips(Serialize["depot_MainUI_1_c"]) 
                        else
                            g_scene:showTips(Serialize["depot_MainUI_2_c"]) 
                            local runeId = self.m_makeResult.material_list[1]
                            local reward = RuneManager:getRuneConfigInfo(runeId,"per_synthesis_failure")
                            local temReward ={}
                            local useCount = #self.m_makeResult.material_list
                --            if useCount >1 then
                                for _,item1 in pairs(reward) do
                                    local temItem = {item1[1],item1[2]*useCount}
                                    table.insert(temReward,temItem)
                                end
                --            end
                            g_scene:showReward(temReward)
                        end
                        self.m_makeResult = nil
                    end
                }
    if not  isEnd then
        isEndFun = nil
    end
    local effect1 = GameUtil:createEffect(anmStr,isLoop,parent,isEndFun)
    parent:addChild(effect1)
    if parent.getContentSize then
        local w ,h = parent:getContentSize()
        effect1:setPosition(w/2, h/2)
    end
    parent.name = effect1
    return effect1
end

function This:addAnim(iconType,pos,isAdd,isLoop,isEnd)
    local itemlistNode = self.m_csNode
    local node = itemlistNode["item" .. pos]
    if iconType == makeAnim1.ShuSong then
        node = node.line
    end
    if not isAdd then
        iconType = nil
    end
    self:createAnim(node,iconType,isLoop,isEnd)
end

function This:refreshSucessRate()
    local count = 0
    local item1 = nil
    for index, item in ipairs(self.m_fileterList) do
         if item then
            count = count+1
            item1 = item
         end
    end 
    local rate =0
    local rateItem = RuneProbabilityConfig[count]
    if rateItem then
        rate = rateItem.radio
    end
    self.m_csNode.num1:setString(string.format(Serialize["equip_StrengthNode_1_c"],rate * 100))
    local isHave = item1
    local node = self.m_csNode.item6
    if isHave then
        local itemId = item1:getId() +1
        if not node.itemNode then
            local itemNode = ItemNode:create(itemId, nil, false, false)
            itemNode:setAnchorPoint(0.5, 0.5)
            node:addChild(itemNode)
            node.itemNode = itemNode
        else
            node.itemNode:initItem(itemId, nil, false,false)
        end 
        node.itemNode:setVisible(true)
        self.m_isWillId =itemId
        self:addAnim(makeAnim1.ZhongXin,6,isHave,true)
    else
        self.m_isWillId =nil
        if node.itemNode then
            node.itemNode:setVisible(false)
        end
        self:addAnim(makeAnim1.ZhongXin,6,isHave)
    end
end

function This:resetQuality()
    self.m_quality = 0
    for index, item in ipairs(self.m_fileterList) do
        if item then
            self.m_quality = item:getQuality()
            break
        end
    end
end

function This:choseItem(item)
    local quality = item:getQuality()
    return item:isRune() and  quality ~=KMaxRuneQualityRuneManager and  not item:isEquipped() and 
    (self.m_quality ==0 or 
        quality == self.m_quality)
end

function This:chooseProp(pos)
    local function checkHandler(item)
        return self:choseItem(item)
    end
    local function valueHandler(t)
        local ret = {}
        for _, v in ipairs(t) do
            if v then
                local id = v._id
                ret[id] = (ret[id] or 0) + v.num
            end
        end
        return ret          
    end
    local panel = g_scene:openDialog("Depot.PropChoosePanel",{
        filename = "MenuScene/NodeUI/RebornChooseHero.csb",
        spaceX = 6,
        filter = self.m_fileterList,
        valueHandler = valueHandler,
        checkHandler = checkHandler,
    })
    panel:setSelectHandler(function (...)
        return self:putInChosen(pos, ...)
    end)
    return panel
end

function This:removeAllChose()
    for index, item in ipairs(self.m_fileterList) do
        if item then
            self:revokeChosen(index)
        end
    end
end

function This:revokeChosen(pos)
    -- print("revoke index:" .. pos)
    -- self.m_quicklyPutIn = false
    local itemListNode = self.m_csNode
    self.m_fileterList[pos] = false
    self:resetQuality()
    self:refreshSucessRate()
    -- self.m_fileterInfoList[pos] = { quality = 0, uid = 0 }
    itemListNode["item" .. pos]:setVisible(false)
    itemListNode["need" .. pos]:setVisible(true)
    self:addAnim(makeAnim1.ShuSong,pos)
end

function This:quicklyPutIn()
    local slotCount = #self.m_fileterList
    local propQualityKeyList = {}
    local posList = {}
    local emptyList ={}
    for index, item in ipairs(self.m_fileterList) do
        if item then
            table.insert(posList, index)
        else
            table.insert(emptyList, index)
        end
        table.insert(propQualityKeyList,{})
    end

    local leftNumber = #emptyList
    if leftNumber > 0 then
        local propList = RuneManager:getAllHaveBagRunes()
        for _,item1 in ipairs(propList) do
            if not item1:isEquipped() then
                local quality = item1:getQuality()
                if quality<KMaxRuneQualityRuneManager then
                    local res1 = propQualityKeyList[quality]
                    table.insert(res1,item1)
                end
            end
        end
        local temItemList = propQualityKeyList[self.m_quality]
        if not temItemList then
            for _,itemList in ipairs(propQualityKeyList) do
                if #itemList>0 then
                    temItemList = itemList
                    break
                end
            end
        end

        if not temItemList or #posList == #temItemList then
            g_scene:showTips(Serialize["equip_AscendNode_3_c"])
            return
        end
        for index,haveIndex in ipairs(posList) do
            self:putInChosen(haveIndex, temItemList[index])
        end
        local haveCount = #posList
        for index=haveCount+1,slotCount do
            local itemData = temItemList[index]
            local emptyIndex = emptyList[index-haveCount]
            if itemData then
                self:putInChosen(emptyIndex, itemData)
            else
                break
            end
        end
    else
        g_scene:showTips(Serialize["Warfare_WarfareInfo_55"])
    end
end

function This:putInChosen(pos, item)
    print("putin index:" .. pos)
    local itemlistNode = self.m_csNode
    local node = itemlistNode["item" .. pos]
    if item then
        local itemId, itemNum = item:getId(), nil
        self.m_fileterList[pos] = item
        itemlistNode["need" .. pos]:setVisible(false)
        if not node.itemNode then
            local itemNode = ItemNode:create(itemId, itemNum, false, true)
            itemNode:setAnchorPoint(0.5, 0.5)
            node:addChild(itemNode)
            node.itemNode = itemNode
            node.close:setLocalZOrder(10)
            node.jianhao:setLocalZOrder(11)
            -- self:createAnim(node.line,makeAnim1.ShuSong,true)
        else
            node.itemNode:initItem(itemId, itemNum, false)
        end 
        self:addAnim(makeAnim1.ShuSong,pos,true,true)
        node:setVisible(true)
    else
        node:setVisible(false)
        self:addAnim(makeAnim1.ShuSong,pos,false)
    end
    self:resetQuality()
    self:refreshSucessRate()
    node.close:setVisible(true)
    node.jianhao:setVisible(true)
    return true
end

function This:initTouchEvent()
    
	local csNode = self.m_csNode
	csNode.close:setTouchEnded(function()
        if self.m_makeResult then
            return false
        end
        self:close()
    end)
    csNode.deleteButton:setTouchEnded(function()--add
        if self.m_makeResult then
            return false
        end
        self:quicklyPutIn()
    end)
    csNode.allreceive:setTouchEnded(function()--made
        if self.m_makeResult then
            return false
        end
        local uidList ={}
        for index, item in ipairs(self.m_fileterList) do
            if item then
                table.insert(uidList,item:getUid())
            end
        end
        if #uidList>0 then
            cc.Audio:playEffect("Skill_Hua.mp3")
            self:runAction(cc.Sequence(
            cc.DelayTime(1),
            cc.CallFunc(function()
                cc.Audio:playEffect("Build_MakeOver.mp3")
            end)))
            RuneManager:onRuneMakeRequest(uidList)
        end
        -- self:onRuneMake()
    end)
    csNode.look:setTouchEnded(function()
        if self.m_makeResult then
            return false
        end
        g_scene:openDialog("Equip.RunePreviewPanel")
    end)
    
    csNode.item6.item:setTouch({
            isScale = false,
            -- isDynamic = true,
            isAlpha = false,
            onEnded = function()
            if self.m_makeResult then
                return false
            end
                if self.m_isWillId then
                    g_scene:openDialog("Equip.RuneSkillPanel",{fileName = "GameScene/NodeUI/HeroSkillRuneInfo.csb",runeId = self.m_isWillId})
                end
            end
        })
    csNode.ronglianzhi.kuang:setTouch({
            isScale = false,
            -- isDynamic = true,
            isAlpha = false,
            onEnded = function()
                if self.m_makeResult then
                    return false
                end
                local maxNum = DailyAchieveManager:getArgConfigLv("rune_melting_max_integral")
                local curent = GamePlayer:getPlayerAttrWithType(PlayerAttr.runnestone_composite_return)
                if curent>=maxNum then
                    RuneManager:onGetRuneMakeValueRewardRequest()
                end
            end
        })
    local wenhao = csNode.wenhao
    wenhao:setTouchEnded(function()
        if self.m_makeResult then
            return false
        end
        local text = Serialize["Awake_RuneMake_1"]
        g_scene:openDialog("UI.HelpTips", {text = text,node = wenhao})
    end)
    for index, item in ipairs(self.m_fileterList) do
        csNode["need" .. index].add:setTouch({
            isScale = false,
            isDynamic = true,
            onEnded = function()
                if self.m_makeResult then
                    return false
                end
                 self:chooseProp(index)
            end
        })
        local curItemNode = csNode["item" .. index]
        curItemNode:setVisible(false)
        curItemNode.close:setTouch({
            onEnded = function()
                if self.m_makeResult then
                    return false
                end
                self:revokeChosen(index)
            end,
            isDynamic = true,
            isScale = false,
            isAlpha = false,
        })
    end
end

function This:onEnter()
    super.onEnter(self)
    RuneManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    RuneManager:removeObserver(self)
end

function This:onRuneMake(makeCode)
    for index, item in ipairs(self.m_fileterList) do
        if item then
            self:addAnim(makeAnim1.ShuSong,index)
            self:addAnim(makeAnim1.ShuSong,index,true)
            self:addAnim(makeAnim1.Icon,index,true)
        end
    end
    self.m_makeResult = makeCode
    self:addAnim(makeAnim1.HeCheng,6,true,nil,true)
end

function This:getData()
	
end

function This:onGetRuneMakeValueReward()
    self:refresh()
end
