-- @Author: zheng
-- @Date:   2019-09-11 14:44:43
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-10-21 14:12:45
-- @File Name: RuneResetPanel.lua
local GameUtil = require("Util.GameUtil")
local This , super = class(...,"Equip.RuneSkillPanel")
local RuneManager = require("Equip.RuneManager")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")
This.fileName = "GameScene/NodeUI/HeroSkillRuneRemanufacturing.csb"
function This:initNode(info)
    self.m_before = info.before
    self.m_after = nil
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    local isSkip = self:getSkipAnim(  )
    self:setSkipAnim(not isSkip )
    -- self:getData()
    self:isInitButtonVisible(true)
    self:refresh()
end

function This:refresh()
    local csNode = self.m_csNode
    local itemNode = GameUtil:creatItemNode(csNode.tubiao,{self.m_before:getId()})
    itemNode:setTouchEnable(false)
    self:refreshCommon(csNode.before,self.m_before,357)
    local haveNode = csNode.after.have
    local emptyNode = csNode.after.empty
    if self.m_after then
        self:refreshCommon(haveNode,self.m_after)
    else

    end
    haveNode:setVisible(self.m_after)
    emptyNode:setVisible(not self.m_after)

    local function resetItem(node,index,itemModel)
        local itemId = itemModel[1]
        local needNum = itemModel[2]
        local icon = node["icon"..index]
        local cost = node["cost"..index]
        icon:setVisible(itemId)
        cost:setVisible(itemId)
        if not itemId then
            return
        end
        local curentCount = DepotManager:getItemOrResCount(itemId)
        icon:setImage(ItemUtil.getIconImage(itemId))
        cost:setString(GameUtil:formatCount(curentCount).."/"..GameUtil:formatCount(needNum))
    end
    local cost1,cost2 = self:getCost()
    for index,item1 in ipairs(cost1) do
        resetItem(csNode,index,item1)
    end
    resetItem(csNode,3,cost2[1] or {})
    csNode.isUse:setVisible(cost2[1])
end

function This:getCost()
    local runeId = self.m_before:getId()
    return RuneManager:getRuneConfigInfo(runeId,"cost"),RuneManager:getRuneConfigInfo(runeId,"rare_item")
end

function This:checkIsEnought()
    local cost1,cost2= self:getCost()
    local temCost = table.clone(cost1)
    local isSelect = self:getSkipAnim()
    if isSelect and cost2[1] then
        table.insert(temCost,cost2[1])
    end
    return DepotManager:getItemOrResEnought(temCost,true)
end

function This:isInitButtonVisible(isInist)
    local initVisible = isInist
    local csNode = self.m_csNode
    for index=1,3 do
        local button = csNode["button"..index]
        if index==1 then
            button:setVisible(initVisible)
        else
            button:setVisible(not initVisible)
        end
    end
end

function This:setSkipAnim( isBool )
    self.m_csNode.isUse.gouxuan:setVisible(isBool)
end

function This:getSkipAnim(  )
    return self.m_csNode.isUse.gouxuan:isVisible()
end

function This:initTouchEvent()
    local function resetRequest()
        if self:checkIsEnought() then
            local isUse = 0
            if self:getSkipAnim(  ) then
                isUse =1
            end
            RuneManager:onRuneResetRequest(self.m_before:getUid(),isUse)
        end
    end
    local csNode = self.m_csNode
    csNode.close:setTouchEnded(function()
        self:close()
    end)
    csNode.button1:setTouchEnded(function()
        resetRequest()
    end)
    csNode.button2:setTouchEnded(function()
        local after =self.m_after
        if after then
           RuneManager:onRuneSaveRequest(self.m_before:getUid(),1)
        end
    end)
    csNode.button3:setTouchEnded(function()
        resetRequest()
    end)
    csNode.isUse.weigouxuan:setTouchEnded(function()
        local isSkip = self:getSkipAnim(  )
        self:setSkipAnim(not isSkip )
    end)
    local wenhao = csNode.wenhao
    wenhao:setTouchEnded(function()
        local text = Serialize["Awake_RuneMake_2"]
        g_scene:openDialog("UI.HelpTips", {text = text,node = wenhao})
    end)
    local uiNames ={"before","after"}
    for index,name in ipairs(uiNames) do
        local itemNode = csNode[name]
        local lookNode = itemNode.look
        if lookNode then
            lookNode:setTouchEnded(function()
                -- print("index",index)
                g_scene:openDialog("Equip.RunePreviewPanel")
            end)
        end
    end
end

function This:onEnter()
    super.onEnter(self)
    RuneManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    RuneManager:removeObserver(self)
end

function This:onTouchMask() 
    
end

function This:onRuneSave(rune)
    if self.m_before:getUid()~= rune:getUid() then
        return
    end
    self.m_before =  rune
    self.m_after = nil
    self:refresh()
end

function This:onRuneReset(rune)
    if self.m_before:getUid()~= rune:getUid() then
        return
    end
    self:isInitButtonVisible()
    self.m_after = rune
    self:refresh()
end

