local This = class(...)
This.m_msgList = {} -- [msg] = args
This.m_listeners = setmetatable({}, {
    __index = function (s, k)
        local v = {}
        s[k] = v
        return v
    end
})-- [listener]{[msg] = func, [listener] = func}

function This:init(application)
    application:addToUpdateList(self)
end

function This:update()
    local msgList = self.m_msgList
    if #msgList > 0 then
        self.m_msgList = {}
        local listeners = self.m_listeners
        for i = 1, #msgList do
            local msg = msgList[i]
            local key = msg[1]
            local callbackList = {}
            for listener, events in pairs(listeners) do
                if events[key] then
                    callbackList[#callbackList + 1] = listener
                end
            end
            if #callbackList > 0 then
                for i, listener in ipairs(callbackList) do
                    listeners[listener][key](listener, msg[2])
                end
            end
        end
    end
end

function This:addListener(listener, key, func)
    self.m_listeners[listener][key] = func
end

function This:removeListener(listener, key)
    self.m_listeners[listener][key] = nil
	if not next(self.m_listeners[listener]) then
		self.m_listeners[listener] = nil
	end
end

function This:removeAllListener(listener)
    self.m_listeners[listener] = nil
end

function This:addMsg(key, args)
    self.m_msgList[#self.m_msgList + 1] = {key, args}
end

function This:dispatcher(key, args)
    local callbackList = {}
    local listeners = self.m_listeners
    for listener, events in pairs(listeners) do
        if events[key] then
            callbackList[#callbackList + 1] = listener
        end
    end
    if #callbackList > 0 then
        for i, listener in ipairs(callbackList) do
			if listeners[listener] and listeners[listener][key] then
				listeners[listener][key](listener, args)
			end
        end
    end
end
