-- @Author: superm
-- @Date:   2019-11-26 15:20:18
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-12 17:36:45
-- @File Name: ExpeditionAchievePanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ExpeditionAchieveView = require("Expedition.ExpeditionAchieveView")

function This:initData(info)
    self.m_viewList = {}
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsAchievement.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

	local scrollview = self.m_csNode.scrollview
    local achievelList = ExpeditionManager:getAchieveListInSort()
    for _, achieve in ipairs( achievelList ) do
        local view = ExpeditionAchieveView:create(achieve)
        self.m_viewList[achieve.config.id] = view
        local w, h = view:getContentSize()
        view:retain()
        scrollview:pushBackCustomItem(view, w, h, 0.5)
    end
end

function This:onEnter()
    super.onEnter(self)
    ExpeditionManager:addObserver(self)
end

function This:onExit()
    local views = self.m_viewList
    if views then
        self.m_viewList = nil
        for _, view in pairs( views ) do
            view:release()
        end
    end
    ExpeditionManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
end

function This:refresh()
    local scrollview = self.m_csNode.scrollview
    scrollview:removeAllItems()
    local achievelList = ExpeditionManager:getAchieveListInSort()
    for _, achieve in ipairs( achievelList ) do
        local view = self.m_viewList[achieve.config.id]
        if view then
            local w, h = view:getContentSize()
            scrollview:pushBackCustomItem(view, w, h, 0.5)
        end
    end
end

function This:onPickupAchieveReward(id)
    local view = self.m_viewList[id]
    if view then
        view:showReward()
        view:refresh()
        self:refresh()
    end
end