-- @Author: superm
-- @Date:   2019-11-26 16:26:24
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-12 15:45:09
-- @File Name: ExpeditionAchieveView.lua
-- @Description: 

local This, super = class(..., "NodeView")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ItemNode = require("UI.ItemNode")

function This:initData(achieve)
	self.m_achieve = achieve
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsAchievementItem.csb")
	self:addChild(self.m_csNode)
	local w , h = self.m_csNode.bg:getContentSize()
	self:setContentSize(w,h)
end

function This:onEnter()
    super.onEnter(self)
	self:refresh()
end
    
function This:initNode()
	local csNode = self.m_csNode

	local curNum = ExpeditionManager:getNpcKilledCount()
	local targetNum = self.m_achieve.config.count
	local state = self.m_achieve.state
	csNode.text:setString(string.format(Serialize[60822], targetNum, curNum, targetNum))
	csNode.yilingqu:setVisible(state)
	csNode.gotobtn:setVisible(not state and curNum < targetNum)
	csNode.pickup:setVisible(not state and curNum >= targetNum)

	local scrollview = self.m_csNode.scrollview
    scrollview:setDirection("horizontal")
    scrollview.m_direction = "right"
	scrollview:removeAllItems()
	local itemList = self.m_achieve.config.reward
	for i, info in ipairs(itemList) do
		local item = ItemNode:create(info[1], info[2])
		local scale = 0.8
		local w, h = item:getContentSize()
		item:setAnchorPoint(0.5, 0.5)
		item:setScale(scale)
		scrollview:pushBackCustomItem(item, w * scale, h * scale, 0.5)
	end

	self.m_curNum = curNum
	self.m_targetNum = targetNum
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	if not self.m_inited then
		csNode.pickup:setTouch({
			onEnded = function()
				ExpeditionManager:onPickupAchieveRewardRequest(self.m_achieve.config.id)
			end,
			isContainer = true,
			isScale = false
		})

		csNode.gotobtn:setTouch({
			onEnded = function()
				local panel = g_scene:getDialog("ExpeditionAchievePanel")
				if panel then
					panel:close()
				end				
			end,
			isScale = false,
			isContainer = true
		})

		csNode.scrollview:setSwallowTouch(false)
		self.m_inited = true
	end
end

function This:refresh()
	local csNode = self.m_csNode
	local curNum = ExpeditionManager:getNpcKilledCount()
	local targetNum = self.m_achieve.config.count
	csNode.text:setString(string.format(Serialize[60823], targetNum, curNum, targetNum))

	local state = self.m_achieve.state
	csNode.gotobtn:setVisible(not state and curNum < targetNum)
	csNode.pickup:setVisible(not state and curNum >= targetNum)
	self.m_curNum = curNum
	self.m_targetNum = targetNum
end

function This:showReward()
	local rewardList = self.m_achieve.config.reward
	if rewardList then
		g_scene:showRewardInExp(rewardList)
	end
end