-- @Author: superm
-- @Date:   2019-11-26 14:55:47
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-10 16:35:33
-- @File Name: ExpeditionChallengePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ExpeditionUtil = require("Expedition.ExpeditionUtil")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ExpeditionHeroSlotView = require("Expedition.ExpeditionHeroSlotView")
local ItemNode = require("UI.ItemNode")
local ItemUtil = require("Bag.ItemUtil")

function This:initData(info)
	self.m_level = info.level
	self.m_levelId = info.levelId
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsChallenge.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode
    self.m_itemList = self.m_level:getReward()

    local view = csNode.hero2
    local heroId = self.m_level:getImageId()
    view.icon:setImage(ItemUtil.getIconImage(heroId))
    view.bg:setImage(ItemUtil.getQualityImage(heroId))
    csNode.hero1:setVisible(false)
    self.m_heroView = ExpeditionHeroSlotView:create()
    csNode.hero1:addChild(self.m_heroView)
    csNode.hero1.jianhao:setLocalZOrder(10)
    local list = ExpeditionUtil:filterAlive(ExpeditionManager:getSlotList())
    local heroUid = ExpeditionManager:getSkpFightState(nil,ExpeditionSaveData.fightHero)
    if heroUid ~=1 then
        for _,slot in pairs(list) do
            if slot:getImageId()==heroUid then
                csNode.selectbg:setVisible(false)
                csNode.hero1:setVisible(true)
                self.m_selected = slot
                self.m_heroView:setModel(slot)
                break
            end
        end
        if not self.m_selected then
            ExpeditionManager:setSkpFightState(nil,ExpeditionSaveData.fightHero)
        end
    end
    self:refresh()
end

function This:initTouchEvent()
	local csNode = self.m_csNode
    csNode.close:setTouchEnded(function()
        self:close()
    end)

    local selectFunc = function()
        g_scene:openDialog("Expedition.ExpeditionHeroChoosePanel", {
            list = ExpeditionUtil:filterAlive(ExpeditionManager:getSlotList()),
            selectHandler = function( slot )
                csNode.selectbg:setVisible(false)
                csNode.hero1:setVisible(true)
                self.m_selected = slot
                self.m_heroView:setModel(slot)
                ExpeditionManager:setSkpFightState(slot:getImageId(),ExpeditionSaveData.fightHero)
            end,
            selected = self.m_selected,
        })
    end
    csNode.selectbg:setTouchEnded(selectFunc)
    csNode.hero1.jianhao:setTouchEnded(selectFunc)

    csNode.automatic:setTouchEnded(function()
        local gou =1
        if ExpeditionManager:getSkpFightState(true) then
            gou =0
        end
        ExpeditionManager:setSkpFightState(gou)
        self:refreshGou()
    end)
    csNode.button:setTouchEnded(function()
    	if not self.m_selected then
    		return g_scene:showTips(Serialize[60825])
    	end
        local hero = self.m_selected:getHero()
    	ExpeditionManager:onChallengeLevelRequest(hero:getUid(), self.m_levelId)
    end)
end

function This:onEnter()
    super.onEnter(self)
    ExpeditionManager:addObserver(self)
end

function This:onExit()
    ExpeditionManager:removeObserver(self)
    super.onExit(self)
end

function This:refreshGou()
    local csNode = self.m_csNode
    csNode.automatic.gou:setVisible(ExpeditionManager:getSkpFightState(true))
end

function This:refresh()
    local csNode = self.m_csNode

    local itemlist = csNode.itemtipslist
	local count = 0
	local w = 10
	for i, info in ipairs(self.m_itemList) do
		local node = ItemNode:create(unpack(info))
		node:setCascadeOpacityEnabled(true)
		node:setAnchorPoint(0, 0.5)
		node:setPositionX(w)
		w = w + node:getContentWidth() + 10
		itemlist:addChild(node)
		itemlist:setPositionX(- w / 2)
	end
    self:refreshGou()
end

function This:onChallengeLevel()
    if ExpeditionManager:getSkpFightState(true) then
        self:close()
        return  
    end
    g_scene:closeAllDialog()
end
