-- @Author: superm
-- @Date:   2019-11-28 17:51:13
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-12 17:16:44
-- @File Name: ExpeditionChapter.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local RemainsLevelConfig = require("Metadatas.RemainsLevelConfig")
local ExpeditionMonster = require("Expedition.ExpeditionMonster")

function This:init(info)
	super.init(self)
	if not info then
		self.m_id = 0
		self.m_levelList = {}
		self.m_levelIds = {}
	else
	    self:updateInfo( info )
	end
end

function This:updateInfo( param )
	if self.m_id ~= param.chapter_index then
		self.m_id = param.chapter_index
		self.m_config = RemainsLevelConfig[self.m_id]
		self.m_levelIds = {}
		self.m_levelList = {}
	end

	for index, info in ipairs( param.normal_level_list ) do
		local id = info.config_id
		local indice = self.m_levelIds[id] 
		if not indice then
			level = ExpeditionMonster:create()
			self.m_levelList[index] = level
		end
		self.m_levelIds[id] = index
		level:updateInfo(info)
	end

	local boss = self.m_boss
	if not boss then
		boss = ExpeditionMonster:create()
		self.m_boss = boss
	end
	boss:updateInfo(param.boss_level, true)
end

function This:setData(info)
    self:updateInfo(info)
end

function This:getId()
    return self.m_id
end

function This:removeEvent(levelId, eventId)
	local indice = self.m_levelIds[levelId]
	if indice and self.m_levelList[indice] then
	    local level = self.m_levelList[indice] 
    	local ret = level:removeEvent(eventId)
    	return ret
	end
end 

function This:selectEvent(levelId, eventId, state, data)
	local event = self:getEvent(levelId, eventId)
	if event then
		event:setState(state)
		event:setContent(data)
	end
end

function This:getEvent(levelId, eventId)
	local indice = self.m_levelIds[levelId]
	if indice and self.m_levelList[indice] then
	    local level = self.m_levelList[indice] 
    	return level:getEventById(eventId)
	end
end

function This:getLevelList()
	local ret = {}
    for _, v in ipairs( self.m_levelList ) do
    	if not v:isDead() then
    		table.insert(ret, v)
    	end
    end
    if not ret[1] and not self.m_boss:isDead() then
		table.insert(ret, self.m_boss)
    end
    return ret
end

function This:getBoss()
    return self.m_boss
end

function This:getAllLevels()
    return self.m_levelList 
end

function This:getLevelByIndex(index)
    return self.m_levelList[index] 
end

function This:getLevelReward(levelId)
	if levelId == 0 then
		return self.m_boss:getReward()
	end

	local indice = self.m_levelIds[levelId]
	if indice and self.m_levelList[indice] then
	    level = self.m_levelList[indice] 
    	return level:getReward()
	end
end

function This:isFinish()
	return self.m_boss:isDead()
end

function This:hasNext()
    return RemainsLevelConfig[self.m_id + 1] ~= nil
end



