-- @Author: superm
-- @Date:   2019-12-03 10:54:00
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-13 11:57:00
-- @File Name: ExpeditionEventContent.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")

function This:init(info)
	super.init(self)
    self:updateInfo(info)
end


function This:updateInfo( param )
	self.m_id = param.id
	self.m_type = param.type
	self.m_subType = param.subType

	local indices = param.indices
	local config = param.config
	if not indices then
		if config then
			self.m_config = config
		end
	else
		self.m_configList = {}
		for index, v in ipairs( indices ) do
			local id = v.id
			self.m_configList[index] = config[id]
		end
		self.m_indices = indices
	end
	if param.data then
		self.m_shopData = param.data
	end
end

function This:setData(info)
    self:updateInfo(info)
end

function This:getImage()
    return string.format("Hero/%d.png", self.m_config.resource)
end

function This:getDesc()
    return self.m_config.desc
end

function This:getCondition()
    return self.m_config.turn_in or self.m_config.hero_condition
end

function This:setContentType( value )
    self.m_subType = value
end

function This:getContentType()
    return self.m_subType
end

function This:completeQuestionById(id, result)
	for _, v in ipairs( self.m_indices ) do
		if v.id == id then
			v.result = result
			break
		end
	end
end

function This:getFirstUnCompletedQuestion()
	local index
	for i, v in ipairs( self.m_indices ) do
		if v.result == ExpQuestionResultType.kNotAnswer then
			index = i
			break
		end
	end
	if not index then return end

	local config = self.m_configList[index]
	if config then
		return { id = config.id, question = config.question, answer = config.correct_answer,
		optlist = { [1] = config.option_1, [2] = config.option_2, [3] = config.option_3, [4] = config.option_4 }, }
	end
end

function This:isAllCompleted()
	for _, v in ipairs( self.m_indices ) do
		if v.result == ExpQuestionResultType.kNotAnswer then
			return false
		end
	end
	return true
end

function This:isAllNewQuestions()
	if #self.m_indices < ExpQuestionCount then
		return false
	else
		local _, info = next(self.m_indices)
		if info.result == ExpQuestionResultType.kNotAnswer then
			return true
		end
		return false
	end
end

function This:getQuestionProgress( param )
	local ret = ExpQuestionCount - #self.m_indices
	for i, v in ipairs( self.m_indices ) do
		ret = ret + 1
		if v.result == ExpQuestionResultType.kNotAnswer then
			break
		end
	end
	return ret
end

function This:getReward(index)
	if self.m_configList then
		return self.m_configList[option or 1].reward
	else
		local reward = self.m_config.reward
		if not reward then
    		if self.m_subType == CaveSubEvent.kRecluse then
    			reward = self.m_config.super_reward
    		elseif self.m_subType == CaveSubEvent.kReward then
	    		reward = self.m_config.nomal_reward
    		end
    	end
		return reward
	end
end

function This:getStoreData()
	return self.m_shopData
end