-- @Author: superm
-- @Date:   2019-11-26 20:39:19
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-13 14:16:49
-- @File Name: ExpeditionEventPanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local ExpeditionUtil = require("Expedition.ExpeditionUtil")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ItemNode = require("UI.ItemNode")
local ItemUtil = require("Bag.ItemUtil")

function This:initData(info)
	self.m_event = info.event
	self.m_step = nil
	self.m_selected = 0
	self.m_showAnswer = false
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsEvent.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode
	self.m_eType = self.m_event:getType()
	self.m_eContent = self.m_event:getContent()
	csNode.name:setString(self.m_event:getName())
	csNode.eventicon:setVisible(false)
	if not self.m_eContent then
		self:resetUI()
	else
		self:refresh()
	end
end

function This:onEnter()
    super.onEnter(self)
    ExpeditionManager:addObserver(self)
    if not self.m_eContent then
		ExpeditionManager:onGetEventInfoRequest(self.m_event:getLevelId(), self.m_event:getId())
	end
end

function This:onExit()
    ExpeditionManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
	local csNode = self.m_csNode
    csNode.close:setTouchEnded(function()
        self:close()
    end)

	-- 洞穴事件
	if self.m_eType == ExpEventTarget.kCave then
	    csNode.operation1.textbg:setTouchEnded(function()
	    	-- 前往获取
			g_scene:openDialog("Recruit.RecruitPanel")
	    	-- self:close()
	    end)

	    csNode.operation2.button:setTouchEnded(function()
	    	-- 前往探索
	    	if self.m_eventType == CaveSubEvent.kRecluse then
				ExpeditionManager:onPickupCaveRewardRequest(self.m_event:getLevelId(), self.m_event:getId())
	    	else
		    	if not self.m_selectedHero then
		    		return g_scene:showTips(Serialize[60830])
				end
				local hero = self.m_selectedHero:getHero()
				ExpeditionManager:onCaveEventRequest(self.m_event:getLevelId(), self.m_event:getId(), hero:getUid())
			end
	    end)

	    local selectFunc = function()
	    	local condition = self.m_eContent:getCondition()
	    	local filterList = ExpeditionUtil:filterAliveAnCond(ExpeditionManager:getSlotList(), condition)
	        g_scene:openDialog("Expedition.ExpeditionHeroChoosePanel", {
	            list = filterList,
	            selectHandler = function( model )
	                csNode.daoju2.item:setVisible(false)
	                csNode.daoju2.jiahao:setVisible(false)
	                csNode.daoju2.jianhao:setVisible(true)
	                self.m_selectedHero = model
			    	if not self.m_heroView then 
						local ExpeditionHeroSlotView = require("Expedition.ExpeditionHeroSlotView")
				    	local heroView = ExpeditionHeroSlotView:create({slot = model})
				    	heroView:setAnchorPoint(0.5,0.5)
				    	heroView:setPositionY(-20)
					    csNode.daoju2:addChild(heroView)
					    csNode.daoju2.jianhao:setLocalZOrder(10)
					    self.m_heroView = heroView
					else
	                	self.m_heroView:setModel(model)
	                end
	    			csNode.operation2:setVisible(true)
	            end,
	            selected = self.m_selectedHero,
	        })
	    end
	    csNode.daoju2.jiahao:setTouchEnded(selectFunc)
        csNode.daoju2.jianhao:setVisible(false)
	    csNode.daoju2.jianhao:setTouchEnded(selectFunc)
	end

	-- 推送 或者 問答
	if self.m_eType == ExpEventTarget.kQuestion or self.m_eType == ExpEventTarget.kPush then
	    csNode.operation3.button1:setTouchEnded(function()
	    	-- 有詐不買
	    	if self.m_eType == ExpEventTarget.kPush then
				ExpeditionManager:onDialogEventRequest(self.m_event:getLevelId(), self.m_event:getId(), 0)
			end
	    	self:close()
	    end)

	    csNode.operation3.button2:setTouchEnded(function()
	    	if self.m_eType == ExpEventTarget.kQuestion then
	    		--速速来战
	    		if self.m_step == 0 then
	    			self.m_step = 1
	    			self:resetUI()
	    			self:refreshQuestion()
	    		end
	    	elseif self.m_eType == ExpEventTarget.kPush then
	    		-- 買買買
				ExpeditionManager:onDialogEventRequest(self.m_event:getLevelId(), self.m_event:getId(), 1)
	    	end
	    end)

	    if self.m_eType == ExpEventTarget.kQuestion then
	    	for index = 1, 4 do
				local button = csNode.operation4["button" .. index]
				button.x:setVisible(false)
				button.gou:setVisible(false)
				button.xuanzhong:setVisible(false)
	    		button:setTouch({
	    			onEnded = function(  )
						if self.m_isRunningAction then
		    				return
		    			end

		    			self.m_selected = index
		    			button.xuanzhong:setVisible(true)
		    			local correctIdx = self.m_question.answer
		    			if correctIdx ~= index then
							local correctBtn = csNode.operation4["button" .. correctIdx]
							correctBtn.gou:setVisible(true)
							button.x:setVisible(true)
						else
							button.gou:setVisible(true)
		    			end
		    			ExpeditionManager:onQuestionEventRequest(self.m_event:getLevelId(), self.m_event:getId(), self.m_question.id, index)
		    		end,
					isScale = false,    		
		    	})
	    	end
	    end
	end
end

function This:refresh()
	self:resetUI()
	if self.m_eType == ExpEventTarget.kCave then
		self:refreshCave()
	elseif self.m_eType == ExpEventTarget.kQuestion then
		self:refreshQuestion()
	elseif self.m_eType == ExpEventTarget.kPush then
		self:refreshPush()
	end
	self.m_isRunningAction = false
end

function This:resetUI()
	local csNode = self.m_csNode
	for i = 1, 2 do
		csNode["neirong" .. i]:setVisible(false)
	end
	for i = 1, 2 do
		csNode["daoju" .. i]:setVisible(false)
	end
	for i = 1, 4 do
		csNode["operation" .. i]:setVisible(false)
	end
	csNode.jindu:setVisible(false)
end

-- Question
function This:refreshQuestion()
	local csNode = self.m_csNode
	local content = self.m_eContent
	csNode.eventicon:setVisible(true)
	csNode.neirong1:setVisible(true)
    csNode.neirong1.tips:setVisible(false)

	if not self.m_step then
		if content:isAllNewQuestions() then
			self.m_step = 0
		else
			self.m_step = 1
		end
	end

    if self.m_step == 0 then
    	csNode.daoju1:setVisible(true)

    	local opNode = csNode.operation3
    	opNode:setVisible(true)
    	opNode.button1:setVisible(true)
    	opNode.button1.text:setString(Serialize[60831])

    	local btn2 = opNode.button2
    	btn2:setVisible(true)
    	btn2.icon:setVisible(false)
    	btn2.num:setVisible(false)
    	btn2.text:setString(Serialize[60832])

	    local itemlist = csNode.daoju1.items
		local count = 0
		local w = 5
    	local rewardList = content:getReward()
		for i, info in ipairs(rewardList) do
			local node = ItemNode:create(unpack(info))
			node:setCascadeOpacityEnabled(true)
			node:setAnchorPoint(0, 0.5)
			node:setPositionX(w)
			w = w + node:getContentWidth() + 5
			itemlist:addChild(node)
		end
		itemlist:setPositionX(130 - w / 2)

	    csNode.neirong1.text1:setString(string.format(Serialize[60833], ExpQuestionCount))
    else
		self.m_question = self.m_eContent:getFirstUnCompletedQuestion()
    	if not self.m_question then
    		self:close()
    		return
    	end
    	csNode.daoju1:setVisible(false)
    	csNode.operation4:setVisible(true)
		csNode.jindu:setVisible(true)
		csNode.jindu:setString(string.format("%d/%d", content:getQuestionProgress(), ExpQuestionCount))
    	if self.m_selected == 0 then
	    	for i = 1, 4 do
				local node = csNode.operation4["button" .. i]
				node.text:setString(self.m_question.optlist[i])
    			node.xuanzhong:setVisible(false)
    			node.gou:setVisible(false)
				node.x:setVisible(false)
	    	end
    	end
	    csNode.neirong1.text1:setString(self.m_question.question)
    end
end

-- Push
function This:refreshPush()
	local csNode = self.m_csNode
	local textNode = csNode.neirong2
	local opNode = csNode.operation3

	textNode:setVisible(true)
	opNode:setVisible(true)
	textNode.tips:setVisible(false)
	textNode.text2:setVisible(false)
	csNode.daoju1:setVisible(true)

	local content = self.m_eContent
	csNode.eventicon:setVisible(true)
	csNode.eventicon:setImage(content:getImage())
    textNode.text1:setString(content:getDesc(), 700)

	local rewardList = content:getReward()
    local itemlist = csNode.daoju1.items
	local count = 0
	local w = 5
	local rewardList = content:getReward()
	if rewardList then
		for i, info in ipairs(rewardList) do
			local node = ItemNode:create(unpack(info))
			node:setCascadeOpacityEnabled(true)
			node:setAnchorPoint(0, 0.5)
			node:setPositionX(w)
			w = w + node:getContentWidth() + 10
			itemlist:addChild(node)
		end
	end
	itemlist:setPositionX(130 - w / 2)

	local condList = content:getCondition()
	local cItemId, cNum = unpack(condList[1])
	local btn2 = opNode.button2
    btn2.icon:setImage(ItemUtil.getIconImage(cItemId))
    btn2.num:setString(cNum)
end

-- Cave Explore
function This:refreshCave()
	local csNode = self.m_csNode
	local content = self.m_eContent
	self.m_eventType = content:getContentType()
	csNode.eventicon:setVisible(true)
	csNode.eventicon:setImage(content:getImage())
	csNode.neirong1.text1:setStringWithColor(content:getDesc(), 460)
	csNode.neirong1.tips:setVisible(false)

    if self.m_eventType == CaveSubEvent.kRecluse then
    	csNode.daoju1:setVisible(true)
    	csNode.daoju1.text:setString(Serialize[60834])
    	csNode.neirong1:setVisible(true)
    	csNode.neirong1.text1:setString(Serialize[60835])
    	csNode.neirong1.tips:setVisible(false)
    	csNode.operation2:setVisible(true)
    	csNode.operation2.button.text:setString(Serialize[60836])

	    local itemlist = csNode.daoju1.items
		local count = 0
		local w = 5
    	local rewardList = content:getReward()
    	if rewardList then
			for i, info in ipairs(rewardList) do
				local node = ItemNode:create(unpack(info))
				node:setCascadeOpacityEnabled(true)
				node:setAnchorPoint(0, 0.5)
				node:setPositionX(w)
				w = w + node:getContentWidth() + 5
				itemlist:addChild(node)
			end
		end
		itemlist:setPositionX(130 - w / 2)
    elseif not self.m_eventType or self.m_eventType == CaveSubEvent.kUnknown then
    	csNode.neirong1:setVisible(true)
    	csNode.daoju2:setVisible(true)
    	csNode.operation2.button.text:setString(Serialize[60837])

    	local heroId = ExpeditionUtil:checkHeroCondition(content:getCondition())
    	if heroId then 
			csNode.operation1:setVisible(true)
			csNode.daoju2:setVisible(false)
			csNode.operation1.tips:setString(string.format(Serialize[60838], ItemUtil.getName(heroId)))
		end
    end
end

function This:onGetQuestionInfo()
	self.m_eContent = self.m_event:getContent()
    self:refresh()
end

function This:onGetDialogInfo()
	self.m_eContent = self.m_event:getContent()
    self:refresh()
end

function This:onGetCaveInfo()
	self.m_eContent = self.m_event:getContent()
	self.m_eventType = self.m_eContent:getContentType()
    self:refresh()
end

function This:onQuestionEvent(eventId, result)
    if eventId == self.m_event:getId() then
    	if result == ExpQuestionResultType.kRight then
    		self:showReward(ExpEventTarget.kQuestion)
    	end
    	self.m_selected = 0
		self.m_isRunningAction = true
		self:runAction(cc.Sequence(cc.DelayTime(1),cc.CallFunc(function()
			if self.m_eContent:isAllCompleted() then return self:close() end
    		self:refresh() 
		end)))
		
    end
end

function This:showReward(option)
	local rewardList = self.m_eContent:getReward(option)
	if rewardList then
		g_scene:showRewardInExp(rewardList)
	end
end

function This:onDialogEvent(eventId)
    if eventId == self.m_event:getId() then
    	self:showReward()
    	self:close()
    end
end

function This:onPickupCaveReward(eventId)
    if eventId == self.m_event:getId() then
    	self:showReward()
    	self:close()
    end
end

function This:onCaveEvent(eventId, eventType, closeAction)
    if eventId == self.m_event:getId() then
    	self.m_eventType = eventType
	    if self.m_eventType == CaveSubEvent.kRecluse then
    		self:refresh()
    	elseif self.m_eventType == CaveSubEvent.kReward then
    		self:showReward()
	    end

	    if closeAction then
    		self:close()
	    end
    end
end