-- @Author: Su
-- @Date:   2019-12-10 19:54:42
-- @Last Modified by:   Su
-- @Last Modified time: 2019-12-10 19:54:42
-- @File Name: ExpeditionEventStorePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ExpeditionEventStoreItem = require("Expedition.ExpeditionEventStoreItem")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local ArgConfig = require("Metadatas.ArgConfig")

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsMysteryStore.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self.m_shopItemList = {}
    self.m_event = info.event
    self:refreshItemList()
    self:refresh()
end

function This:onEnter()
    super.onEnter(self)
    ExpeditionManager:addObserver(self)
    self.m_content= self.m_event:getContent()
    if not self.m_content then
        ExpeditionManager:onGetEventInfoRequest(self.m_event:getLevelId(), self.m_event:getId())
    end
end

function This:onExit()
    super.onExit(self)
    ExpeditionManager:removeObserver(self) 
end

function This:refresh()
    local node = self.m_csNode
    if not self.m_content then
        return
    end
    local data = self.m_content:getStoreData()
    local maxValue = ArgConfig.remains_favorable.arg[1]
    node.text1:setString(GamePlayer:getDiamond())
    node.text2:setString(GamePlayer:getMonumentCoin())
    node.jindutiao.num:setString(string.format( "%d/%d", data.favorable_value,maxValue))
    node.jindutiao.jindu:setPercent(data.favorable_value/maxValue * 100)
    node.box:setVisible(data.favorable_value >= maxValue)
end

function This:refreshItemList()
    self.m_content= self.m_event:getContent()
    if not self.m_content then
        return
    end
    local data = self.m_content:getStoreData()
    local node = self.m_csNode
	local scrollView = node.ScrollView

    local itemList = data.shop_item_list
    for i,item in ipairs(itemList) do
        if not self.m_shopItemList[item.id] then
            local itemNode = ExpeditionEventStoreItem:create({item = item,event = self.m_event})
            itemNode:setAnchorPoint(0.5,0.5)
            local w , h = itemNode:getContentSize()
            scrollView:pushNode(itemNode, 360, 120 ,2)
            self.m_shopItemList[item.id] = itemNode
        else
            self.m_shopItemList[item.id]:updateInfo({item = item,event = self.m_event})
        end
    end
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    self.m_csNode.add:setTouchEnded(function()
        g_scene:openDialog("System.RechargeMainUI")
    end)
    self.m_csNode.box:setTouchEnded(function()
        g_scene:openDialog("Expedition.ExpeditionSuperAwardsPanel",{event = self.m_event})
    
    end)
end

function This:onGetExpStoreInfo()
    self:refreshItemList()
    self:refresh()
end

function This:onBuyExpStoreItem(id)
    local reward
    for _,item in pairs(self.m_shopItemList) do

        local itemId,num =item:getItemRewardByPos(id)
        if itemId then
            reward = {{itemId,num}}
        end
    end
    if reward then
        g_scene:showReward(reward)
    end
end
