-- @Author: superm
-- @Date:   2019-12-05 14:43:52
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-16 16:00:49
-- @File Name: ExpeditionHeroChoosePanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ExpeditionHeroSlotView = require("Expedition.ExpeditionHeroSlotView")
local RankPageListLayer = require("Rank.RankPageListLayer")

function This:initData(info)
	if info then
		self.m_heroList = info.list
		self.m_selected = info.selected 
		self.m_selectHandler = info.selectHandler
	end
end

function This:initNode(info)
	local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsChooseHero.csb")
	self:addChild(csNode)
	csNode:setToCenter()
	self.m_csNode = csNode

	self.m_count = #self.m_heroList
	local totalWidth = csNode.ScrollView:getContentWidth()
	local itemWidth = 90
	self.m_layerLayer = RankPageListLayer:create({
		itemWidth = itemWidth,
		itemHeight = 110,
        spaceX = 8,
        spaceY = 8,
        column = math.floor(totalWidth / itemWidth),
        scrollView = csNode.ScrollView,
		dataCounter = function ()
	        return self.m_count
        end,
        viewCreator =  function(index, node)
        	if self.m_count < index then return end
        	local slot = self.m_heroList[index]
	        if not slot then return end
	        if not node then
				local view = self:getHeroView(slot)
				if self.m_selected == slot then
					self.m_selectedView = view
				end
		        return view
		    else
	        	node:setModel(slot)
	        end
	    end,
    })

	self.m_layerLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_layerLayer:getContentSize()
	csNode.ScrollView:pushBackCustomItem(self.m_layerLayer, w, h,0)
	self.m_layerLayer:setDataList()
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.close:setTouchEnded(function()
		self:close()
	end)

	csNode.button2:setTouchEnded(function()
		if not self.m_selected then
			return g_scene:showTips(Serialize[60845])
		end

		if self.m_selectHandler then
			self.m_selectHandler(self.m_selected)
		end
		self:close()
	end)
end

function This:getHeroView(slot)
	local view = ExpeditionHeroSlotView:create({
		slot = slot,
		touch = true,
		isSwallow = false,
		isContanier = true,
	})
	view:setSelectHandler(function(obj)
		if self.m_selected == obj then
	    	return 
		end
		self.m_selected = obj
		if self.m_selectedView then
			self.m_selectedView:refreshSelectState()
		end
		self.m_selectedView = view
	end, function(obj)
		return self.m_selected == obj
	end)
	return view
end