local This, super = class(..., "Base.BaseModel")
local Socket = require("Net.Socket")
local EventDispatcher = require("EventDispatcher")
local ExpeditionUtil = require("Expedition.ExpeditionUtil")
local ExpeditionChapter = require("Expedition.ExpeditionChapter")
local ExpeditionHeroSlot = require("Expedition.ExpeditionHeroSlot")
local ExpeditionTreasure = require("Expedition.ExpeditionTreasure")
local BattleDataManager = require("Battle.BattleDataManager")
local NewActivityManager = require("NewActivity.NewActivityManager")
local PopDialogManager = require("UI.PopDialogManager")
local SingleManager = require("SingleDaily.SingleManager")

function This:init()
	super.init(self)
	self.m_npcKilledCount = 0
	self.m_slotList = {}
	self.m_heroMap = {}
	self.m_itemUseInfo = ExpeditionUtil:genExpItemInfoList()
	self.m_records = {}
    self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 14900, self.onGetExpeditionInfoReply)
	EventDispatcher:addListener(self, 14901, self.onChallengeLevelReply)
	EventDispatcher:addListener(self, 14902, self.onJumpNextChapterReply)
	EventDispatcher:addListener(self, 14903, self.onUseExpItemReply)
	EventDispatcher:addListener(self, 14904, self.onGetEventInfoReply)
	EventDispatcher:addListener(self, 14905, self.onGetQuestionInfoReply)
	EventDispatcher:addListener(self, 14906, self.onGetDialogInfoReply)
	EventDispatcher:addListener(self, 14907, self.onGetCaveInfoReply)
	EventDispatcher:addListener(self, 14908, self.onGetExpStoreInfoReply)
	EventDispatcher:addListener(self, 14910, self.onGetTreasureBoxInfoReply)
	EventDispatcher:addListener(self, 14911, self.onDialogEventReply)
	EventDispatcher:addListener(self, 14912, self.onQuestionEventReply)
	EventDispatcher:addListener(self, 14913, self.onCaveEventReply)
	EventDispatcher:addListener(self, 14914, self.onPickupCaveRewardReply)
	EventDispatcher:addListener(self, 14915, self.onBuyExpStoreItemReply)
	EventDispatcher:addListener(self, 14916, self.onPickStorePrizeReply)
	EventDispatcher:addListener(self, 14917, self.onOpenTreasureBoxReply)
	EventDispatcher:addListener(self, 14918, self.onRemoveEventReply)
	EventDispatcher:addListener(self, 14919, self.onPickupAchieveRewardReply)
	EventDispatcher:addListener(self, 14920, self.onSyncSlotInfoReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self, 14900)
	EventDispatcher:removeListener(self, 14901)
	EventDispatcher:removeListener(self, 14902)
	EventDispatcher:removeListener(self, 14903)
	EventDispatcher:removeListener(self, 14904)
	EventDispatcher:removeListener(self, 14905)
	EventDispatcher:removeListener(self, 14906)
	EventDispatcher:removeListener(self, 14907)
	EventDispatcher:removeListener(self, 14908)
	EventDispatcher:removeListener(self, 14910)
	EventDispatcher:removeListener(self, 14911)
	EventDispatcher:removeListener(self, 14912)
	EventDispatcher:removeListener(self, 14913)
	EventDispatcher:removeListener(self, 14914)
	EventDispatcher:removeListener(self, 14915)
	EventDispatcher:removeListener(self, 14916)
	EventDispatcher:removeListener(self, 14917)
	EventDispatcher:removeListener(self, 14918)
	EventDispatcher:removeListener(self, 14919)
	EventDispatcher:removeListener(self, 14920)
end
ExpeditionSaveData ={
	skipState	= "ExpeditionSkpFightState",
	fightHero = "ExpeditionFightHero",
}
function This:setSkpFightState(state,key)
	SingleManager:setPalyerActInfo(key or ExpeditionSaveData.skipState,state or 1)
end

function This:getSkpFightState(check,key)
	local state = SingleManager:getPalyerActInfo(key or ExpeditionSaveData.skipState,1)
	if not key and check then
		return state==1
	end
	return state
end

function This:onPickupAchieveRewardRequest(rewardId)
	Socket:sendAndLock(14919, {reward_id = rewardId})
end

function This:onPickupAchieveRewardReply( msg )
    if msg.err_code == 0 then
    	local achieveId = msg.reward_id
		self:updateAchieveList({achieveId})
        self:notifyObservers("onPickupAchieveReward", msg.reward_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onRemoveEventReply( msg )
    if msg.err_code == 0 then
    	self.m_chapter:removeEvent(msg.level_config_id, msg.event_index)
        self:notifyObservers("onRemoveEvent")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onOpenTreasureBoxRequest(levelId, eventIndex, boxId)
	Socket:sendAndLock(14917, {
		level_config_id = levelId,
		event_index = eventIndex,
		box_index = boxId,
	})
end

function This:onOpenTreasureBoxReply( msg )
    if msg.err_code == 0 then
        if self.m_boxExpedition then
        	local reward = {self.m_boxExpedition:getReward(msg.reward_id)}
        	g_scene:showRewardInExp(reward)
        end
        self:notifyObservers("onOpenTreasureBox")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onPickStorePrizeRequest(levelId, eventIndex, rewardId)
	Socket:sendAndLock(14916, {
		level_config_id = levelId,
		event_index = eventIndex,
		reward_id = rewardId,
	})
end

function This:onPickStorePrizeReply( msg )
    if msg.err_code == 0 then
		-- print(msg)
		self:onGetEventInfoRequest(msg.level_config_id,msg.event_index)
        self:notifyObservers("onPickStorePrize",msg.reward_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onBuyExpStoreItemRequest(levelId, eventIndex, storeItemId)
	Socket:sendAndLock(14915, {
		level_config_id = levelId,
		event_index = eventIndex,
		shop_item_id = storeItemId,
	})
end

function This:onBuyExpStoreItemReply( msg )
    if msg.err_code == 0 then
		-- print(msg)
		self:onGetEventInfoRequest(msg.level_config_id,msg.event_index)
        self:notifyObservers("onBuyExpStoreItem",msg.shop_item_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 领取山洞大奖
function This:onPickupCaveRewardRequest(levelId, eventIndex)
	Socket:sendAndLock(14914, {
		level_config_id = levelId,
		event_index = eventIndex,
	})
end

function This:onPickupCaveRewardReply( msg )
    if msg.err_code == 0 then
		local event = self.m_chapter:removeEvent(msg.level_config_id, msg.event_index)
		self:notifyObservers("onRemoveEvent")
        self:notifyObservers("onPickupCaveReward", msg.event_index)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onCaveEventRequest(levelId, eventIndex, heroId)
	Socket:sendAndLock(14913, {
		level_config_id = levelId,
		event_index = eventIndex,
		card_id = heroId,
	})
end

function This:onCaveEventReply( msg )
    if msg.err_code == 0 then
		local event = self.m_chapter:getEvent(msg.level_config_id, msg.event_index)
		local closeAction = false
		if event then
			local content = event:getContent()
			content:setContentType(msg.hole_event_id)
			if msg.hole_event_id ~= CaveSubEvent.kRecluse then
    			self.m_chapter:removeEvent(msg.level_config_id, msg.event_index)
				self:notifyObservers("onRemoveEvent")
				closeAction = true
			end
		end
        self:notifyObservers("onCaveEvent", msg.event_index, msg.hole_event_id, closeAction)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onQuestionEventRequest(levelId, eventIndex, questionId, chooseIndex)
	Socket:sendAndLock(14912, {
		level_config_id = levelId,
		event_index = eventIndex,
		question_id = questionId,
		choose = chooseIndex,
	})
end

function This:onQuestionEventReply( msg )
    if msg.err_code == 0 then
		local event = self.m_chapter:getEvent(msg.level_config_id, msg.event_index)
		if event then
			local content = event:getContent()
			content:completeQuestionById(msg.question_id, msg.result)
        	self:notifyObservers("onQuestionEvent", msg.event_index, msg.result)
			if content:isAllCompleted() then
    			self.m_chapter:removeEvent(msg.level_config_id, msg.event_index)
        		self:notifyObservers("onRemoveEvent")
			end
		end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onDialogEventRequest(levelId, eventIndex, chooseIndex)
	Socket:sendAndLock(14911, {
		level_config_id = levelId,
		event_index = eventIndex,
		choose = chooseIndex,
	})
end

function This:onDialogEventReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onDialogEvent", msg.event_index)
		local event = self.m_chapter:removeEvent(msg.level_config_id, msg.event_index)
		self:notifyObservers("onRemoveEvent")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:getBoxExpedition()
	return self.m_boxExpedition
end

function This:onGetTreasureBoxInfoReply( msg )
	if msg then
		local boxInfo = msg.box_info
		if not self.m_boxExpedition then
			self.m_boxExpedition = ExpeditionTreasure:create(boxInfo)
		end
		self.m_boxExpedition:updataeInfo(boxInfo)
        self.m_chapter:selectEvent(msg.level_config_id, msg.event_index, msg.state, msg.box_info)
        self:notifyObservers("onGetTreasureBoxInfo")
	else
		g_scene:handleErrorCode("")
	end
end

-- 神秘商人事件
function This:onGetExpStoreInfoReply( msg )
	if msg then
        self.m_chapter:selectEvent(msg.level_config_id, msg.event_index, msg.state, msg.shop_info)
        self:notifyObservers("onGetExpStoreInfo")
	else
		g_scene:handleErrorCode("")
	end
end

function This:onGetCaveInfoReply( msg )
	if msg then
        -- print(msg)
    	-- optional msg_event_arg_hole hole_info = 4;
        self.m_chapter:selectEvent(msg.level_config_id, msg.event_index, msg.state, msg.hole_info)
        self:notifyObservers("onGetCaveInfo")
	else
		g_scene:handleErrorCode("")
	end
end

function This:onGetDialogInfoReply( msg )
	if msg then
		-- optional int32 talk_id = 4; // 对话配置ID
        self.m_chapter:selectEvent(msg.level_config_id, msg.event_index, msg.state, msg.talk_id)
        self:notifyObservers("onGetDialogInfo")
	else
		g_scene:handleErrorCode("")
	end
end

function This:onGetQuestionInfoReply( msg )
	if msg then
	    -- msg_event_question question_list = 4;      // 题目列表(当前，答完删除)
        self.m_chapter:selectEvent(msg.level_config_id, msg.event_index, msg.state, msg.question_list)
        self:notifyObservers("onGetQuestionInfo")
	else
		g_scene:handleErrorCode("")
	end
end

function This:onSyncSlotInfoReply( msg )
	if msg then
	    local info = self.m_heroMap[msg.card_id]
	    local index, preHp = info.index, info.hp
    	local slot = self.m_slotList[index]
    	local hero
	    if slot then
	    	slot:setHp(msg.hp_percent)
	    	hero = slot:getHero()
	    end
	    if not hero then hero = HeroManager:getHeroByUid(msg.card_id) end
	    if not hero then return end

	    local delta = math.floor(hero:getAttr(AttrType.kMaxHp) * (msg.hp_percent - preHp))
	    g_scene:showHeroInfoInExp(hero:getId(), delta)
        self:notifyObservers("onSyncSlotInfo")
	else
		g_scene:handleErrorCode("")
	end
end

function This:onGetEventInfoRequest(levelId, eventIndex)
	Socket:sendAndLock(14904, {level_config_id = levelId, event_index = eventIndex})
end

function This:onGetEventInfoReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onGetEventInfo")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onUseExpItemRequest(itemId, target)
	Socket:sendAndLock(14903, {item_id = itemId, target = target})
end

function This:onUseExpItemReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onUseExpItem", msg.item_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetExpeditionInfoRequest()
	Socket:send(14900)
end

function This:onGetExpeditionInfoReply( msg )
	if msg then
		self.m_chapter = ExpeditionChapter:create(msg.chapter_info)
		self.m_npcKilledCount = msg.kill_npc_count 

		self:updateUseItemInfo(msg.item_use_info)
		for index, v in ipairs(msg.card_info) do
			self.m_heroMap[v.id] = {index = index, hp = v.hp}
			local slot = self.m_slotList[index]
			if not slot then
				slot = ExpeditionHeroSlot:create(v)
				self.m_slotList[index] = slot
			else
			    slot:updateInfo(v)
			end
		end
		self:updateAchieveList(msg.achievement_reward_list)
		print( "[message]onGetExpeditionInfo" )
        self:notifyObservers("onGetExpeditionInfo")
	else
		g_scene:handleErrorCode("")
	end
end

function This:onChallengeLevelRequest(heroId, levelId)
	self.m_curLevelId = levelId
	Socket:sendAndLock(14901, {card_id = heroId, level_config_id = levelId})
end

function This:onChallengeLevelReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onChallengeLevel")
        self.m_report = msg.fight_report
        if self.m_curLevelId then
        	self.m_rewardList = self.m_chapter:getLevelReward(self.m_curLevelId)
        end
        if self:getSkpFightState(true) then
        	self:onShowBattleResult(self.m_report.state==1,true)
        	
        else
	    	self:startBattle()
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onJumpNextChapterRequest()
	Socket:sendAndLock(14902)
end

function This:onJumpNextChapterReply( msg )
	if msg.err_code == 0 then
		self.m_boxExpedition = nil
	    self.m_report = nil
	    self.m_rewardList = nil
	    self.m_curLevelId = nil
        self:notifyObservers("onJumpNextChapter")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:getCurChapter()
    return self.m_chapter
end

function This:getNpcKilledCount()
    return self.m_npcKilledCount or 0
end

function This:getSlotList()
	local ret = {}
	for i, v in ipairs( self.m_slotList ) do
		ret[i] = v
	end

	table.sort(ret, function( slotA, slotB )
		local hpA, hpB = slotA:getHp(), slotB:getHp()
		if hpA > 0 and hpB > 0 then
			local powerA, powerB = slotA:getFightPower(), slotB:getFightPower() 
			if powerA ~= powerB then
				return powerA > powerB
			end
			if hpA ~= hpB then
				return hpA > hpB
			end
			return slotA:getHeroUid() > slotB:getHeroUid() 
		elseif hpA > 0 then
			return true
		else
			return false
		end
	end)
    return ret
end

function This:getAchieveList()
	local list = self.m_achieveList
	if not list then
		list = ExpeditionUtil:genAchieveList()
		self.m_achieveList = list
	end
    return list
end

function This:getAchieveListInSort()
	local achieveList = self:getAchieveList()
    local ret = {}
	for _, achieve in ipairs(achieveList) do
		table.insert(ret, achieve)
	end
	table.sort(ret, function( a, b )
		if a.state ~= b.state then
			return not a.state
		end
		return a.config.id < b.config.id
	end)
	return ret
end

function This:updateAchieveList(param)
	local achieveList = self:getAchieveList()
	if param then
		for _, achieveId in ipairs(param) do
			local achieve = achieveList[achieveId]
			if achieve then
				achieve.state = true
			end
		end
	end
end

function This:canAchievePickup()
	local achieveList = self:getAchieveList()
	for _, achieve in ipairs(achieveList) do
		local config = achieve.config
		if (not achieve.state) and config.count <= self:getNpcKilledCount() then
			return true
		end
	end
	return false
end

function This:getItemUseInfo()
    return self.m_itemUseInfo
end

function This:updateUseItemInfo( param )
	local itemUseInfo = self.m_itemUseInfo
	local indices = itemUseInfo.indices
	for _, v in ipairs( param ) do
		itemUseInfo[indices[v.id]].use = v.use_times
	end
end

function This:getCurChapter()
    return self.m_chapter
end

function This:startBattle()
	local params = { mode = PlayMode.kStage, bgId = 2, msg = {report = self.m_report}}
    BattleDataManager:addObserver(self)
    cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, params)
end

function This:onShowBattleResult(victory, stay) 
    if self.m_report then
        BattleDataManager:removeObserver(self)
        if victory then
            g_scene:openDialog("Stage.StageVictoryPanel",{
                isRobber = true,
                dropInfo = self.m_rewardList,
                keepScene = stay,
                callback = function( ... )
					PopDialogManager:addPopDialog({
						scene = "MenuScene", 
						name = "Expedition.ExpeditionPanel", 
						arg = {queue = NewActivityManager:getActivity(NewActivityType.kExpedition)},
						level = DialogLevel.kForLevel,
					})
                end
            })
        else
            g_scene:openDialog("Stage.StageFailedPanel", {
                isRobber = true,
                keepScene = stay,
                callback = function( ... )
					PopDialogManager:addPopDialog({
						scene = "MenuScene", 
						name = "Expedition.ExpeditionPanel", 
						arg = {queue = NewActivityManager:getActivity(NewActivityType.kExpedition)},
						level = DialogLevel.kForLevel,
					})
                end
            })
        end
	    self.m_report = nil
	    self.m_rewardList = nil
	    self.m_curLevelId = nil
    end
end

function This:onBattleSkip(result)
    self:onShowBattleResult(result == BattleResult.kAttackerVictory)
end

function This:onBattleVictory()
    self:onShowBattleResult(true)
end

function This:onBattleFailed()
    self:onShowBattleResult()
end

function This:hasFinRecord(levelId)
	local chapterId = self.m_chapter:getId()
    return self.m_records[string.format("%d,%d", chapterId, levelId)] 
end

function This:recordFin(levelId)
	local chapterId = self.m_chapter:getId()
    self.m_records[string.format("%d,%d", chapterId, levelId)] = true
end

