-- @Author: superm
-- @Date:   2019-11-28 17:54:17
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-11 11:55:16
-- @File Name: ExpeditionMonster.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local RemainsConfig = require("Metadatas.RemainsConfig")
local ExpeditionUtil = require("Expedition.ExpeditionUtil")
local ExpeditionEvent = require("Expedition.ExpeditionEvent")

function This:init(info)
	super.init(self)
	if not info then
		self.m_id = 0
		self.m_hpPercent = 0
		self.m_events = {}
	else
	    self:updateInfo( info )
	end
end

function This:updateInfo( param, isBoss )
	if self.m_id ~= param.config_id then
		self.m_id = param.config_id
		self.m_config = RemainsConfig[self.m_id]
	end
	self.m_hpPercent = param.npc_hp_left
	local events = {}
	for _, v in ipairs( param.event_list ) do
		v.level_id = self.m_id
		local event = self.m_events[v.index]
		if not event then
			event = ExpeditionEvent:create(v)
		else
			event:updateInfo(v)
		end
		events[v.index] = event
	end
	self.m_isBoss = isBoss
	self.m_events = events
end

function This:setData( info )
    self:updateInfo( info )
end

function This:getId()
    return self.m_id
end

function This:getImage()
    return string.format("Icons/Item/%d.png", self.m_config.icon)
end

function This:getHero()
	return    
end

function This:isBoss()
    return self.m_isBoss
end

function This:isDead()
    return self.m_hpPercent <= 0
end

function This:getImageId()
    return self.m_config.icon
end

function This:getReward()
    return self.m_config.reward
end

function This:getEventList()
	local ret, events = {}, self.m_events
	if events then
		for _, v in pairs( events ) do
			table.insert(ret, v)
		end
	end
    return ret
end

function This:removeEvent(id)
	local events = self.m_events
	local ret = events[id]
	if ret then
		events[id] = nil
	end
	return ret
end

function This:getEventById(id)
	return self.m_events[id]
end
