-- @Author: superm
-- @Date:   2019-11-25 10:57:28
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-17 17:27:43
-- @File Name: ExpeditionPanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local DepotManager = require("Depot.DepotManager")
local ExpeditionHeroSlotView = require("Expedition.ExpeditionHeroSlotView")
local ExpeditionEventView = require("Expedition.ExpeditionEventView")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")

function This:initData(info)
    self.m_slotViewList = {}
    self.m_eventViews= {}
    self.m_itemUseInfo = ExpeditionManager:getItemUseInfo()
    self.m_actQueue = info.queue
    self.m_player = info.play
end

function This:initNode()
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsMain.csb")
    csNode:setToCenter()
    self:addChild(csNode)
	self.m_csNode = csNode

    for index, info in ipairs( self.m_itemUseInfo ) do
    	local node = csNode.ListView["item" .. index]
    	node.icon:setImage(ItemUtil.getIconImage(info.id))
        node.icon:setTouch({
            onEnded = function()
                local checkHandler = info.check
                if checkHandler then
                    local ret, tip = checkHandler(ExpeditionManager)
                    if ret then
                        return g_scene:showTips(tip)
                    end
                end
                self:openDialog("Expedition.UseExpeditionItemPanel", info)
            end,
            isDynamic = true,
            isContanier = true,
            isSwallow = false,
            isScale = false
        })
    end

    local scrollview = csNode.scrollview
	scrollview:setDirection("horizontal")
	scrollview.m_direction = "right"
    local slotList = ExpeditionManager:getSlotList()
    for i, slot in ipairs( slotList ) do
        local view = ExpeditionHeroSlotView:create({slot = slot})
        self.m_slotViewList[i] = view
        local w, h = view:getContentSize()
        view:setAnchorPoint(0.5, 0.5)
        scrollview:pushBackCustomItem(view, w, h, 0.5)
    end
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.ListView.achieve.icon:setTouch({
        onEnded = function()
            self:openDialog("Expedition.ExpeditionAchievePanel")
        end,
        isDynamic = true,
        isContanier = true,
        isSwallow = false,
        isScale = false
    })

    csNode.ListView.shop.icon:setTouch({
        onEnded = function()
            self:openDialog("Expedition.ExpeditionStorePanel")
        end,
        isDynamic = true,
        isContanier = true,
        isSwallow = false,
        isScale = false
    })

    csNode.buttonnext:setTouchEnded(function()
        local panel = g_scene:openDialog("UI.ConfirmPanel", {
            text = Serialize[60853]
        })        
        panel:setOkHandler(function()
            ExpeditionManager:onJumpNextChapterRequest()
        end)
	end)

    csNode.close:setTouchEnded(function()
        self:close()
    end)

    for i = 1, 4 do
        csNode["boss" .. i].boss:setTouchEnded(function()
            local level = self.m_chapter:getLevelByIndex(i)
            if level:isDead() then
                return g_scene:showTips(Serialize[60854])
            end
            self:openDialog("Expedition.ExpeditionChallengePanel", {
                level = level,
                levelId = level:getId(),
            })
        end)
    end

    csNode.boss.boss:setTouchEnded(function()
        local level = self.m_chapter:getBoss()
        if level:isDead() then
            return g_scene:showTips(Serialize[60854])
        end
        self:openDialog("Expedition.ExpeditionChallengePanel", {
            level = level,
            levelId = 0,
        })
    end)
end

function This:onEnter()
    super.onEnter(self)
	ExpeditionManager:addObserver(self)    
    DepotManager:addObserver(self)    
    if self.m_actQueue then
        self.m_actQueue:addObserver(self)    
    end
    ExpeditionManager:onGetExpeditionInfoRequest()
end

function This:onExit()
    if self.m_actQueue then
    	self.m_actQueue:removeObserver(self)
    end
    DepotManager:removeObserver(self)
    ExpeditionManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
	self:refreshItemList()
	self:refreshSlotList()
end

function This:refreshTimeInfo()
    local csNode = self.m_csNode
    local leftTime = self.m_actQueue:getLeftTime()
	csNode.time:setString(Serialize[60855] .. GameUtil:formatDTime(Serialize[60856], leftTime))
end

function This:refreshItemList()
    local csNode = self.m_csNode
    for i, itemInfo in ipairs( self.m_itemUseInfo ) do
        local node = csNode.ListView["item" .. i]
        local num = DepotManager:getItemCount(itemInfo.id)
        node.num:setString(num)
    end
end

function This:refreshSlotList()
    local scrollview = self.m_csNode.scrollview
    local slotList = ExpeditionManager:getSlotList()

    for index, slot in ipairs( slotList ) do
        local view = self.m_slotViewList[index]
        if not view then
            view = ExpeditionHeroSlotView:create({slot = slot})
            self.m_slotViewList[index] = view
            local w, h = view:getContentSize()
            view:setAnchorPoint(0.5, 0.5)
            scrollview:pushBackCustomItem(view, w, h, 0.5)
        else
            view:setModel(slot)
        end
    end
end

function This:clearAllEventViews()
    local ret = self.m_eventViews
    self.m_eventViews = {}
    for _, list in pairs( ret ) do
        for __, view in pairs( list ) do
            view:removeFromParent()
        end
    end
end

function This:refreshAchieveReddot()
    local isShow = ExpeditionManager:canAchievePickup()
    if not self.m_reddot then
        local node = self.m_csNode.ListView.achieve.icon
        self.m_reddot = cc.Sprite:create("LoginScene/baoman.png")
        local w , h = node:getContentSize()
        local w1 , h1 = self.m_reddot:getContentSize()
        self.m_reddot:setPosition(w - w1/2, h - h1/2)
        node:addChild(self.m_reddot)
    end
    self.m_reddot:setVisible(isShow)
end

function This:refreshMap()
    self:clearAllEventViews()
    local csNode = self.m_csNode
    if self.m_chapter and self.m_chapter:isFinish() and self.m_chapter:hasNext() then
        csNode.buttonnext:setVisible(true)
    else
        csNode.buttonnext:setVisible(false)
    end

    local showBoss = true
    for i, level in ipairs( self.m_chapter:getAllLevels() ) do
        local levelId = level:getId()
        local bnode, zhezhao, kuang = csNode["boss" .. i], csNode["zhezhao" .. i], csNode["zjkuang" .. i]
        local yun = csNode["yun" .. i]
        if not yun then
            yun = GameUtil:createEffect("ZhengBaYun", false, kuang, nil, nil, nil, true)
            yun:setVisible(false)
            yun:setScale(0.25)
            yun:setPosition(zhezhao:getPosition())
            csNode:addChild(yun, 1)
            kuang:setLocalZOrder(2)
            bnode:setLocalZOrder(3)
            csNode["yun" .. i] = yun
        end

        if not level:isDead() then
            showBoss = false
            bnode:setVisible(true)
            zhezhao:setVisible(true)
            kuang:setVisible(true)
            yun:setOpacity(150)
            yun:setVisible(true)
            bnode.boss:setImage(level:getImage())
        else
            bnode:setVisible(false)
            zhezhao:setVisible(false)
            kuang:setVisible(false)
            if self.m_player or not ExpeditionManager:hasFinRecord(levelId) then
                yun:setOpacity(255)
                yun:setVisible(true)
                yun:setAnimationListener({
                    onAnimationComplete = function (animationName,animationEvent)
                        yun:setVisible(false)
                        yun:playAnimation("animation", false)
                        self:runAction(cc.Sequence(cc.DelayTime(0.15),cc.CallFunc(function()
                            yun:unscheduleUpdate()
                        end)))
                        ExpeditionManager:recordFin(levelId)
                    end
                })
                yun:scheduleUpdate()
                yun:playAnimation("animation", false)
            end
            local x, y = kuang:getPosition()
            local eventList = level:getEventList()
            local eventViewList = {} 
            for index, event in ipairs( eventList ) do
                local view = ExpeditionEventView:create(event)
                csNode:addChild(view)
                view:setPosition(x + index * 40 , y - index* 20)
                eventViewList[index] = view
            end
            self.m_eventViews[i] = eventViewList
        end

    end

    csNode.boss:setVisible(false)
    if showBoss then 
        local boss = self.m_chapter:getBoss()
        if boss and not boss:isDead() then
            csNode.boss:setVisible(true)
            csNode.boss.boss:setImage(boss:getImage())
        end
    end
    self.m_player = false
end

function This:onGetExpeditionInfo()
    self.m_chapter = ExpeditionManager:getCurChapter()
    self.m_csNode.textzj:setString(string.format(Serialize[60857], self.m_chapter:getId()))
    self:refreshItemList()
    self:refreshSlotList()
    self:refreshMap()
    self:refreshAchieveReddot()
end

function This:onPickupAchieveReward()
    self:refreshAchieveReddot()
end

function This:onRemoveEvent()
    self:refreshMap()
end

function This:onJumpNextChapter()
    self:clearAllEventViews()
end

function This:itemUpdate()
    self:refreshItemList()
end

---------------------
-- function This:onActivityStart()
-- end

function This:onActivityEnded()
    g_scene:showTips(Serialize[60858])
    self:close()
end

function This:onUpdateActivity()
    self:refreshTimeInfo()
end

function This:onSyncSlotInfo()
    self:refreshSlotList()
end
