-- @Author: superm
-- @Date:   2019-12-03 11:58:10
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-08 14:08:47
-- @File Name: ExpeditionStorePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ExpeditionStoreItem = require("Expedition.ExpeditionStoreItem")
local StoreManager = require("Store.StoreManager")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

-- local TabName = 
-- {
--     kBattlefield = 1,
--     kRelic = 2,
-- }

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsShop.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
	self.m_select = StoreType.kBattlefieldShop
    self.m_shopItemList = {}
    self.m_store = StoreManager:getStoreBy(StoreType.kBattlefieldShop)

    self.m_csNode.tips:setVisible(false)
    self.m_csNode.button:setVisible(false)
    self:addTabButton(tostring(StoreType.kBattlefieldShop),self.m_csNode.tep1,function()
        self:selectTabBattle()
        return true
    end)
    self:refresh()
    self:lightTabButton(tostring(self.m_select))
    self:selectTabByType(self.m_select)
end

function This:onEnter()
    super.onEnter(self)
    ExpeditionManager:addObserver(self)
    StoreManager:addObserver(self)
    GamePlayer:addObserver(self)

end

function This:onExit()
    super.onExit(self)
	ExpeditionManager:removeObserver(self)
    StoreManager:removeObserver(self)
    GamePlayer:addObserver(self)
    if self.m_store then
        self.m_store:removeObserver(self)
    end
end

function This:createTabNode(name)
    local store = self:getTabNodeData(name)
    local uiName = store:getUITypeName()
    if uiName  then
        local tabNode = require(uiName):create(store)
        return tabNode
    end
end

function This:getTabNodeData(name)
    return self.m_store
end

function This:refresh()
    self.m_csNode.text1:setString(GamePlayer:getDiamond())
    self.m_csNode.text2:setString(GamePlayer:getMonumentCoin())
end

function This:selectTabByType(storeType)
    self.m_store = StoreManager:getStoreBy(storeType)
    if self.m_store then
        local node = self:openTabNode(tostring(storeType))
        if node and node ~= false then
            node:refresh()
        end
    else
        StoreManager:onGetStoreInfoRequest(StoreType.kBattlefieldShop)
    end

end

function This:selectTabBattle()
    self.m_select = StoreType.kBattlefieldShop
    self:selectTabByType( self.m_select)

end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    -- self.m_csNode.button:setTouchEnded(function()
    --     --刷新 
    -- end)
    self.m_csNode.add:setTouchEnded(function()
        g_scene:openDialog("System.RechargeMainUI")
    end)
end

function This:updateTime(leftTime)
    local node = self.m_csNode
	node.tips:setString(GameUtil:formatTime(Serialize[60864], leftTime))
end

function This:onGetCommonStoreInfo(storeType)
    if storeType == StoreType.kBattlefieldShop then
        self:selectTabByType( StoreType.kBattlefieldShop)
        self:refresh()
    end

end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.diamond or field == PlayerAttr.expedition then
        self:refresh()
    end
end