-- @Author: superm
-- @Date:   2019-11-22 14:52:00
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-17 17:27:04
-- @File Name: ExpeditionUtil.lua
-- @Description: 

local This, super = class(...)
local RemainsQuestionBankConfig = require("Metadatas.RemainsQuestionBankConfig")
local RemainsPushConfig = require("Metadatas.RemainsPushConfig")
local RemainsCaveConfig = require("Metadatas.RemainsCaveConfig")
local RemainsAchievementConfig = require("Metadatas.RemainsAchievementConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local ExpeditionEventContent = require("Expedition.ExpeditionEventContent")
local RemainsBusinessmanConfig = require("Metadatas.RemainsBusinessmanConfig") 

ExpEventTarget = {
	kQuestion = 1, -- 答题
	kPush = 2, -- 推送
	kCave = 3, -- 洞穴
	kMystery = 4, -- 神秘商店
	kTreasure = 5, -- 藏宝阁
}

CaveSubEvent = {
	kUnknown = 0,
	kBleeding = 1,
	kDeath = 2,
	kReward = 3,
	kRecluse = 4,
}

ExpItemType = {
	kCross = 1,
	kHpPotion = 2,
	kExcalibur = 3,
	kWings = 4,
}

ExpCondType = {
	kLevel = 1,--等级，大于等于
	kStar = 2,--星级，大于等于
	kQuality = 3,--品质，大于等于
	kCountry = 4,--国家，等于
	kId = 5,--武将ID，等于
}

ExpQuestionResultType = {
	kNotAnswer = 0,
	kRight = 1,
	kWrong = 1,
}

ExpQuestionCount = 3
if ArgConfig["question_count"] then
	ExpQuestionCount = ArgConfig["question_count"].arg[1]
end

function This:genAchieveList()
	local achieveList = {}
	for id, config in ipairs(RemainsAchievementConfig) do
		achieveList[id] = {
			state = false,
			config = config,
		}
	end
	return achieveList
end

function This:genExpItemInfoList()
	local ret = {
		[ExpItemType.kCross] = {
			use = 0,
			target = function( manager )
			    local ret = {}
			    for _, slot in ipairs( manager.m_slotList ) do
			    	if slot:isDead() then
			    		table.insert( ret, slot )
			    	end
			    end
			    return ret
			end,
			check = function( manager )
				local ret, tip = true, Serialize[60874]
			    for _, slot in ipairs( manager.m_slotList ) do
			    	if ret and slot:isDead() then
			    		ret = false
			    	end
			    end
			    return ret, tip
			end,		
		},
		[ExpItemType.kExcalibur] = {
			use = 0,
			indice = true,
			target = function( manager )
				return manager.m_chapter:getLevelList()
			end,
		},
		[ExpItemType.kHpPotion] = {
			use = 0,
			target = function( manager )
			    local ret = {}
			    for _, slot in ipairs( manager.m_slotList ) do
			    	if slot:isInjured() then
			    		table.insert( ret, slot )
			    	end
			    end
			    return ret
			end,
			check = function( manager )
				local ret1, tip1 = true, Serialize[60875]
				local ret2, tip2 = true, Serialize[60876]
			    for _, slot in ipairs( manager.m_slotList ) do
			    	if ret2 and not slot:isDead() then
			    		ret2 = false
			    	end

			    	if ret1 and not slot:isFullHp() then
			    		ret1 = false
			    	end
			    end
			    return ret1 or ret2, ret2 and tip2 or tip1
			end,
		},
		[ExpItemType.kWings] = {
			use = 0,
			target = false,
			check = function( manager )
				if manager.m_chapter:hasNext() then
					return false
				end
				return true, Serialize[60877]
			end
		},
		indices = {}
	}

    local usetimeList = ArgConfig["remains_item_usetimes"].arg
    for itype, usetimeInfo in ipairs( usetimeList ) do
    	local info = ret[itype]
    	local indices = ret.indices
    	info.id, info.limit = unpack(usetimeInfo)
    	indices[info.id] = itype
    end
    return ret
end

function This:filterAlive( list )
    local ret = {}
    for _, slot in ipairs( list ) do
    	if not slot:isDead() then
    		table.insert( ret, slot )
    	end
    end
    return ret
end

function This:filterAliveAnCond( list, condList )
    local result = {}
    for _, slot in ipairs( list ) do
    	if not slot:isDead()  then
    		local hero = slot:getHero()
    		local ret = false
    		for _, v in ipairs( condList ) do
    			local ctype, num = unpack(v)
    			if ctype == ExpCondType.kLevel then
    				ret = hero:getLevel() >= num
    			elseif ctype == ExpCondType.kStar then
    				ret = hero:getStarLv() >= num
    			elseif ctype == ExpCondType.kQuality then
    				ret = hero:getQuality() >= num
    			elseif ctype == ExpCondType.kCountry then
    				ret = hero:getCountry() == num
    			elseif ctype == ExpCondType.kId then
    				ret = hero:getId() == num
    			end
    			if not ret then break end
    		end

    		if ret then
    			table.insert( result, slot )
    		end
    	end
    end
    return result
end

function This:checkHeroCondition( condList )
	local HeroManager = require("Hero.HeroManager")
	local heroId
	for _, condition in ipairs( condList ) do
		local ctype, num = unpack(condition)
		if condition[1] == ExpCondType.kId then
			heroId = num
			break
		end 
	end
	if heroId and not HeroManager:isGotHero(heroId) then
		return heroId
	end
end

function This:createEventContent(ptype, param, content)
	print( ptype, param )
	local etype, eid = ptype, 0
	local config, indices, subType,data
	if etype == ExpEventTarget.kQuestion then
		indices = param
		config = RemainsQuestionBankConfig
	elseif etype == ExpEventTarget.kPush then
		eid = param
		config = RemainsPushConfig[eid]
	elseif etype == ExpEventTarget.kCave then
		eid = param.hole_id
		subType = param.hole_event_id
		config = RemainsCaveConfig[eid]
	elseif etype == ExpEventTarget.kMystery then
		data = param
		config = RemainsBusinessmanConfig
	end

	if not content then
		content = ExpeditionEventContent:create({
			type = etype, id = eid,
			config = config, indices = indices, subType = subType,data = data
		})
	else
		content:updateInfo({ id = eid, indices = indices, subType = subType, config = config, data = data })
	end
	return content
end


