-- @Author: superm
-- @Date:   2019-11-22 16:00:13
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-17 15:10:18
-- @File Name: UseExpeditionItemPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ExpeditionManager = require("Expedition.ExpeditionManager")
local ExpeditionHeroSlotView = require("Expedition.ExpeditionHeroSlotView")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")

function This:initData(info)
	self.m_itemId = info.id
    self.m_useTimes = info.use
    self.m_useIndice = info.indice
    self.m_limTimes = info.limit
    self.m_filterHandler = info.target
    if self.m_filterHandler then
        self.m_show = true
        self.m_targetList = self.m_filterHandler(ExpeditionManager)
    end
    self.m_slotViews = {}
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RuinsUseProp.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode
    self:refresh()
end

function This:onEnter()
    super.onEnter(self)
    ExpeditionManager:addObserver(self)
end

function This:onExit()
    local views = self.m_slotViews
    if views then
        for _, view in pairs( views ) do
            view:release()
        end
        self.m_slotViews = nil
    end
    ExpeditionManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.close:setTouchEnded(function()
        self:close()
    end)

    csNode.button:setTouchEnded(function()
        if self.m_itemCount <= 0 then
            return g_scene:showTips(Serialize[60879])
        end
        if self.m_leftTimes <= 0 then
            return g_scene:showTips(Serialize[60880])
        end
        if self.m_targetList and not self.m_target then
            return g_scene:showTips(Serialize[60881])
        end

        local targetId = 0
        if self.m_target then
            local hero = self.m_target:getHero()
            if hero then
                targetId = hero:getUid()
            else
                targetId = self.m_target:getId()
                if self.m_target:isBoss() then
                    targetId = 0
                end
            end
        end
        ExpeditionManager:onUseExpItemRequest(self.m_itemId, targetId)
    end)
end

function This:refresh()
    self:refreshInfoNode()
    self:refreshSlotList()
end

function This:refreshInfoNode()
    local csNode = self.m_csNode
    self.m_itemCount = DepotManager:getItemCount(self.m_itemId)
    GameUtil:creatItemNode(csNode.item, {self.m_itemId, self.m_itemCount})
    csNode.name:setString(ItemUtil.getName(self.m_itemId))
    csNode.neirong:setString(ItemUtil.getComment(self.m_itemId), 460)
    local leftTimes = self.m_limTimes - self.m_useTimes
    if leftTimes < 0 then
        leftTimes = 0
    end 
    self.m_leftTimes = leftTimes
    csNode.cishu:setString(string.format(Serialize[60882], leftTimes, self.m_limTimes))

    csNode.tips:setVisible(not self.m_show)
    csNode.button:setVisible(not self.m_show)
    csNode.scrollview:setVisible(self.m_show)
    csNode.jiantou1:setVisible(self.m_show)
    csNode.jiantou2:setVisible(self.m_show)
end

function This:refreshSlotList()
    local csNode = self.m_csNode
    if self.m_show then
        local scrollview = csNode.scrollview
        scrollview:setDirection("horizontal")
        scrollview.m_direction = "right"
        scrollview:removeAllItems()
        for i, slot in ipairs( self.m_targetList ) do
            local hero = slot:getHero() or i
            local view = self.m_slotViews[hero]
            if not view then
                view = ExpeditionHeroSlotView:create({
                    slot = slot, 
                    touch = true,
                    isSwallow = false,
                    isContanier = true,
                })
                view:setSelectHandler(function( obj )
                    if self.m_target ~= obj then
                        self.m_target = obj
                        if self.m_selectView then
                            self.m_selectView:refreshSelectState()
                        end
                        self.m_selectView = view
                    end
                    csNode.button:setVisible(self.m_target ~= nil)
                end, function(obj)
                    return self.m_target == obj
                end)
                self.m_slotViews[hero] = view
                view:retain()
            else
                view:refreshSelectState()
            end
            local w, h = view:getContentSize()
            view:setAnchorPoint(0.5, 0.5)
            scrollview:pushBackCustomItem(view, w, h, 0.5)
        end
    end
end

function This:onUseExpItem(itemId)
    self.m_itemCount = self.m_itemCount - 1
    self.m_useTimes = self.m_useTimes + 1
    if self.m_filterHandler then
        self.m_target = nil
        if self.m_useIndice then
            for i, v in ipairs( self.m_slotViews ) do
                if v == self.m_selectView then
                    table.remove(self.m_slotViews, i)
                    self.m_csNode.scrollview:removeItem(i)
                    break                
                end
            end
        end
        self.m_selectView = nil
        self.m_targetList = self.m_filterHandler(ExpeditionManager)
    end
    self:refresh()
    if self.m_itemCount <= 0 and self.m_leftTimes <= 0 then
        self:close()
        return
    end
end

