---
---
--- Created by aid.
--- DateTime: 2018/12/14 11:08
---
local This = class(...)

local ItemUtil = require("Bag.ItemUtil")
local MagicWeaponStartConfig = require("Metadatas.MagicWeaponStartConfig")
local FabaoUtil = require("Fabao.FabaoUtil")

function This:create(arguments)
    local o = {}
    setmetatable(o,self)
    o:init(arguments)
    return o
end

function This:setRawData(rawData)
    self.m_rawData = rawData
    if self.m_item then
        self.m_item:setFabaoId(self:getItemId())
    else
        self.m_item = ItemUtil.createItemModel({id=self:getUid(), type_id=self:getItemId(), num=1})
    end
end

function This:init(arguments)
    self:setRawData(arguments)
end

function This:asItem()
    return self.m_item
end

function This:getUid()
    return self.m_rawData.id
end

function This:getItemId()
    return self.m_rawData.type_id
end

--function This:getNumber()
--    return self.m_rawData.num
--end

-- 法宝等级
function This:getLevel()
    return self.m_rawData.msg_magic_weapon_info.level
end

-- 精炼程度
function This:getJinglianDegree()
    return self.m_rawData.msg_magic_weapon_info.refine_level
end

-- 精炼进度
function This:getJinglianProgress()
    return self.m_rawData.msg_magic_weapon_info.refine_progress
end

-- 星级
function This:getStarLevel()
    return self.m_rawData.msg_magic_weapon_info.star
end

-- 获取该法宝当前的技能Id
function This:getCurrentSkillId()
    local skillId = MagicWeaponStartConfig[self:getItemId()*1000+self:getStarLevel()].skill
    if skillId ~= 0 then
        return skillId
    else
        return nil
    end
end

-- 获取该法宝当前的技能图标Id
function This:getCurrentSkillIconId()
    local skillIconId = MagicWeaponStartConfig[self:getItemId()*1000+self:getStarLevel()].skill_icon
    if skillIconId ~= 0 then
        return skillIconId
    else
        return nil
    end
end

-- 该法宝是否有技能
function This:hasSkill()
    return self:getCurrentSkillId() ~= nil
end

-- 是否被装备在某个武将身上
function This:isEquipped()
    return self.m_rawData.equip_card_id and self.m_rawData.equip_card_id > 0
end

-- 获得装备了该法宝的武将Uid
function This:getHeroUid()
    return self.m_rawData.equip_card_id
end

-- 法宝等级属性加成
function This:getAdditionalAttributeListOnLevel()
    return FabaoUtil.getLevelAttributeList(self:getItemId(), self:getLevel(), self:getStarLevel())
end

-- 法宝精炼属性加成
function This:getAdditionalAttributeListOnJinglianDegree()
    return FabaoUtil.getJinglianAttributeList(self:getItemId(), self:getJinglianDegree(), self:getStarLevel())
end

-- 法宝星级属性加成
function This:getAdditionalAttributeListOnStarLevel()
    return FabaoUtil.getStarLevelAttributeList(self:getItemId(), self:getStarLevel())
end

-- 总体的法宝属性加成
function This:getAllAdditionalAttributes()
    return FabaoUtil.getAllAttributes(self:getItemId(), self:getLevel(), self:getJinglianDegree(), self:getStarLevel())
end

function This:getRebornReturn()
    local ret = {}
    local list = FabaoUtil.getFabaoRebornConfig(self:getItemId(), self:getLevel(), self:getJinglianDegree(), self:getStarLevel())
    for _, config in ipairs(list) do
        for _, info in ipairs(config) do
            local id, num = unpack(info)
            ret[id] = num + (ret[id] or 0)
        end
    end
    return ret
end

function This:canReborn()
    if self:getLevel() > 0 or self:getStarLevel() > 0 or self:getJinglianDegree() > 0 or self:getJinglianProgress() > 0 then
        return true
    end
    return false
end
