---
---
--- Created by aid.
--- DateTime: 2018/12/20 14:39
---
local This , super = class(...,"cc.Node")
local MagicWeaponRefineConfig = require("Metadatas.MagicWeaponRefineConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local FabaoManager = require("Fabao.FabaoManager")
local GamePlayer = require("Player.GamePlayer")
local FabaoUtil = require("Fabao.FabaoUtil")
local ItemNode = require("UI.ItemNode")

local NeededItemType = {
    kCoin = 1,
    kJinglianStone = 2,
}

function This:init()
    self:initData()
    self:initNode()
    self:initTouchEvent()
end

function This:initData()
    self.m_fabao = nil -- 当前的法宝
    self.m_lastFabao = nil -- 上个法宝
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureRefine.csb")
    self:addChild(self.m_csNode)

    self.m_originalScaleX = self.m_csNode.lucky.completionbar:getScaleX()
    self:setAutoBuyUsedItem(false)
    -- TODO
end

function This:checkJianlianCondition(times, callback)
    -- 检查法宝精炼度
    if self.m_fabao:getJinglianDegree() == self.m_maxRefineDegree then
        g_scene:showTips(Serialize[60886]) -- TODO 国际化
        callback(false)
        return
    end

    -- 检查玩家等级
    if GamePlayer:getLevel() < self.m_currentRefineConfig.need_level then
        g_scene:showTips(Serialize[60887]) -- TODO 国际化
        callback(false)
        return
    end

    -- 检查需要消耗的物品数量是否足够
    --local hasDialog = false
    for neededItemType, itemInfo in ipairs(self.m_neededitemList) do
        if neededItemType == NeededItemType.kCoin then
            local needCount = itemInfo[2]*times
            if DepotManager:getItemOrResCount(itemInfo[1], {itemUid = 0}) < needCount then
                g_scene:showTips(Serialize[60714] .. ItemUtil.getName(itemInfo[1])..Serialize[60800]) -- TODO 国际化
                callback(false)
                return
            end
        elseif neededItemType == NeededItemType.kJinglianStone then
            local needCount = itemInfo[2]*times
            local totalCount = DepotManager:getItemOrResCount(itemInfo[1], {itemUid = 0})
            if totalCount < needCount then
               if self.m_autoBuyUsedItem then
                   --hasDialog = true
                   local neededYuanBaoCount = (needCount-totalCount)*ArgConfig.one_refinery_price.arg[1]
                   local ownedYuanBaoCoun = GamePlayer:getDiamond()
                   if ownedYuanBaoCoun < neededYuanBaoCount then -- 检查玩家持有的元宝数量， 不足的话进入元宝充值界面
                       g_scene:openDialog("System.RechargeMainUI")
                       callback(false)
                       return
                   else
                       callback(true)
                       return
                   end

                   --local dialog = g_scene:openDialog("UI.ConfirmPanel", {
                   --    text = string.format(Serialize[60888], (needCount-totalCount)*ArgConfig.one_refinery_price.arg[1], times)
                   --})
                   --dialog:setOkHandler(function()
                   --    callback(true)
                   --    return
                   --end)
                   --dialog:setCancelHandler(function()
                   --    callback(false)
                   --    return
                   --end)
               else
                   -- 进入精炼石购买界面
                   g_scene:openDialog("Store.GeneralStorePanel", {storeType = StoreType.kFabao})
                   callback(false)
                   return
               end
            end
        else
            error("error!")
        end
    end

    callback(true)
    return

    --if not hasDialog then
    --    callback(true)
    --    return
    --end
end

function This:initTouchEvent()

    -- 帮助按钮
    self.m_csNode.help:setTouch({
        onEnded = function()
            g_scene:openDialog("UI.HelpTips", {text = Serialize[60889] , node = self.m_csNode.help})
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 自动购买勾选
    self.m_csNode.gouxuan:setTouch({
        onEnded = function()
            self:setAutoBuyUsedItem(false)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })
    self.m_csNode.weigouxuan:setTouch({
        onEnded = function()
            self:setAutoBuyUsedItem(true)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })
    self.m_csNode.zidong:setTouch({
        onEnded = function()
            self:setAutoBuyUsedItem(not self.m_autoBuyUsedItem)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 礼包购买按钮
    self.m_csNode.buy:setTouch({
        onEnded = function()
            g_scene:openDialog("Store.GeneralStorePanel", {storeType = StoreType.kFabao})
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 精炼按钮
    self.m_csNode.jinglianbutton:setTouch({
        onEnded = function()
            self:checkJianlianCondition(1, function(ok)
                if ok then
                    cc.Audio:playEffect("Recruit_Change.mp3")
                    FabaoManager:onRefineFabaoRequest(self.m_fabao:getUid(), 1)
                end
            end)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 精炼10次按钮
    --self.m_csNode.jinglianbutton1:setTouch({
    --    onEnded = function()
    --        local times = 10
    --        if self.m_remainingJinglianTimes < times then
    --            times = self.m_remainingJinglianTimes
    --        end
    --        self:checkJianlianCondition(times, function(ok)
    --            if ok then
    --                FabaoManager:onRefineFabaoRequest(self.m_fabao:getUid(), times)
    --            end
    --        end)
    --    end,
    --    isScale = false,
    --    isDynamic = true,
    --    isSwallow = false,
    --})
end

function This:onEnter()
    super.onEnter(self)
    FabaoManager:addObserver(self)
    DepotManager:addObserver(self)
end

function This:onExit()
    DepotManager:removeObserver(self)
    FabaoManager:removeObserver(self)
    super.onExit(self)
end

function This:setFabao(fabao)
    self.m_lastFabao = self.m_fabao
    self.m_fabao = fabao

    -- 法宝图标
    local fabaoRootNode = self.m_csNode.EquipItem
    local tag = 1545379467;
    self.m_fabaoItemView = fabaoRootNode:getChildByTag(tag)
    if not self.m_fabaoItemView then
        self.m_fabaoItemView = ItemNode:create(self.m_fabao:getItemId())
        self.m_fabaoItemView:setAnchorPoint(0.5,0.5)
        self.m_fabaoItemView:setTag(tag)
        fabaoRootNode:addChild(self.m_fabaoItemView)
    end
    self.m_fabaoItemView:initItem(self.m_fabao:getItemId())
    self.m_fabaoItemView:setToFabao(self.m_fabao:getLevel(), self.m_fabao:getJinglianDegree(), self.m_fabao:getStarLevel())

    local itemId = self.m_fabao:getItemId()
    local jinglianDegree = self.m_fabao:getJinglianDegree()

    -- 当前属性
    self.m_csNode.before.strengthen:setString("+"..jinglianDegree)
    self.m_currentRefineConfig = MagicWeaponRefineConfig[itemId*1000+jinglianDegree]
    local attributeRootNode = self.m_csNode.before
    local currentAttributeList = FabaoUtil.getJinglianAttributeList(itemId, jinglianDegree, self.m_fabao:getStarLevel())
    for index = 1, 3 do
        local attributeNameView = attributeRootNode["property"..index]
        local attributeValueView = attributeRootNode["property"..index.."num"]
        local attribute = currentAttributeList[index]
        if attribute then
            local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeNameStr)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end

    -- 下一级属性
    attributeRootNode = self.m_csNode.after
    local nextRefineConfig = MagicWeaponRefineConfig[itemId*1000+jinglianDegree+1]
    if nextRefineConfig then
        local nextAttributeList = FabaoUtil.getJinglianAttributeList(itemId, jinglianDegree+1, self.m_fabao:getStarLevel())
        for index = 1, 3 do
            local attributeNameView = attributeRootNode["property"..index]
            local attributeValueView = attributeRootNode["property"..index.."num"]
            local attribute = nextAttributeList[index]
            if attribute then
                local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
                attributeNameView:setString(attributeNameStr)
                attributeValueView:setString(attributeValueStr)
                attributeNameView:setVisible(true)
                attributeValueView:setVisible(true)
            else
                attributeNameView:setVisible(false)
                attributeValueView:setVisible(false)
            end
        end

        --for index, attribute in ipairs(FabaoUtil.getJinglianAttributeList(itemId, jinglianDegree+1, self.m_fabao:getStarLevel())) do
        --    local attributeName, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
        --    local attributeNameNode = attributeRootNode["property"..index]
        --    local attributeValueStrNode = attributeRootNode["property"..index.."num"]
        --    attributeNameNode:setString(attributeName)
        --    attributeValueStrNode:setString(attributeValueStr)
        --    attributeNameNode:setVisible(true)
        --    attributeValueStrNode:setVisible(true)
        --end
        attributeRootNode.strengthen:setString("+"..(jinglianDegree+1))
    else
        for index = 1, 3 do
            local attributeNameNode = attributeRootNode["property"..index]
            local attributeValueStrNode = attributeRootNode["property"..index.."num"]
            attributeNameNode:setVisible(false)
            attributeValueStrNode:setVisible(false)
        end
        attributeRootNode.strengthen:setString(Serialize[60890])
    end

    -- 法宝最高精炼程度
    self.m_maxRefineDegree = jinglianDegree
    while MagicWeaponRefineConfig[itemId*1000+self.m_maxRefineDegree+1] do
        self.m_maxRefineDegree = self.m_maxRefineDegree + 1
    end

    -- 法宝剩余精炼次数
    local remainingAllRefineProgress = MagicWeaponRefineConfig[itemId*1000+jinglianDegree].progress-self.m_fabao:getJinglianProgress()
    if remainingAllRefineProgress <= 0 then
        remainingAllRefineProgress = 0
    end
    for degree = jinglianDegree+1, self.m_maxRefineDegree-1 do
        remainingAllRefineProgress = remainingAllRefineProgress + MagicWeaponRefineConfig[itemId*1000+degree].progress
    end
    self.m_remainingJinglianTimes = math.ceil(remainingAllRefineProgress/ArgConfig.one_refinery_add_progress.arg[1])

    -- 精炼度
    local jinglianProgress = self.m_fabao:getJinglianProgress()
    local needProgress = self.m_currentRefineConfig.progress
    if jinglianDegree == self.m_maxRefineDegree then
        self.m_csNode.lucky.completionnumber:setString(""..jinglianProgress.."/0")
        self.m_csNode.lucky.completionbar:setScaleX(self.m_originalScaleX)
    else
        self.m_csNode.lucky.completionnumber:setString(""..jinglianProgress.."/"..needProgress)
        self.m_csNode.lucky.completionbar:setScaleX(self.m_originalScaleX*(jinglianProgress/needProgress))
    end

    -- 道具消耗
    self:initNeededItem()

    -- 播放属性变更动画
    if self.m_lastFabao
            and self.m_lastFabao:getUid() == self.m_fabao:getUid()
            and self.m_lastJinglianDegree
            and self.m_fabao:getJinglianDegree() > self.m_lastJinglianDegree
            and self:isVisible()
    then
        for index=1, #currentAttributeList do
            FabaoUtil.showAnimeOn(self.m_csNode.effect["effect"..index])
        end
    end
    self.m_lastJinglianDegree = self.m_fabao:getJinglianDegree()
end

function This:initNeededItem()
    self.m_neededitemList = {}
    self.m_neededitemList[NeededItemType.kCoin] = self.m_currentRefineConfig.cost[2]
    self.m_neededitemList[NeededItemType.kJinglianStone] = self.m_currentRefineConfig.cost[1]
    local nodeList = {"money", "item1"}
    for index, item in ipairs(self.m_neededitemList) do
        self.m_csNode[nodeList[index]]:setImage(ItemUtil.getIconImage(item[1]))
        local ownedCount = DepotManager:getItemOrResCount(item[1], {itemUid = 0})
        local neededCount = item[2]
        local color = "<A0>"
        if ownedCount < neededCount then
            color = "<R0>" -- 数量不足显示红色
        end
        local text = string.format("%s%s<A0>/%s", color, ItemUtil.formatItemCount(ownedCount), ItemUtil.formatItemCount(neededCount))
        self.m_csNode[nodeList[index]].text:setStringWithColor(text)
    end
end

function This:setAutoBuyUsedItem(yes)
    local function set(yes)
        self.m_autoBuyUsedItem = yes
        self.m_csNode.gouxuan:setVisible(self.m_autoBuyUsedItem)
        self.m_csNode.weigouxuan:setVisible(not self.m_autoBuyUsedItem)
    end

    if yes then
        -- 再次确认
        g_scene:openDialog("Fabao.FabaoJinglianAutoBuyMaterialHintPanel", {callback = function(ok)
            if not ok then return end
            set(true)
        end})
    else
        set(false)
    end
end

function This:onFabaoDataChanged(fabaoUid)
    if self.m_fabao:getUid() == fabaoUid then
        self:setFabao(FabaoManager:getFabao(fabaoUid))
    end
end

function This:onFabaoRefineSuccessful(fabaoUid, refineProgressAdd)
    if self.m_fabao:getUid() == fabaoUid then
        g_scene:showTips(Serialize[60891]..refineProgressAdd)
    end
end

function This:onReceiveUniversalStoreItemInfo(data)
    if data.buy_type == StoreType.kFabao then
        self:initNeededItem()
    end
end