local This , super = class(...,"Dialog")
local TabGroup = require("TabGroup")
local RankPageListLayer = require("Rank.RankPageListLayer")
local HeroWithFabaoItemView = require("Fabao.HeroWithFabaoItemView")
local DepotItemView = require("Depot.DepotItemView")
local HeroManager = require("Hero.HeroManager")
local FabaoManager = require("Fabao.FabaoManager")
local FabaoConstants = require("Fabao.FabaoConstants")
local FabaoCultivateUpgradeLevelView = require("Fabao.FabaoCultivateUpgradeLevelView")
local FabaoCultivateJingLianView = require("Fabao.FabaoCultivateJingLianView")
local FabaoCultivateUpgradeStarView = require("Fabao.FabaoCultivateUpgradeStarView")
local FabaoUtil = require("Fabao.FabaoUtil")

local FabaoSourceTabKeys = {
    kHero = 1,
    kBackpack = 2,
}

local FabaoCultivateMethodTabKeys = {
    kUpgradeLevel = 1,
    kJingLian = 2,
    kUpgradeStar = 3,
}

function This:initData(arguments)
    super.initData(self, arguments)

    -- 选中的法宝Uid
    self.m_currentSelectedFabaoUid = arguments.fabaoUid
    self.m_sourceType = arguments.sourceType
    self.m_isOnlyWearFabao = true

    self:initHeroList()

    -- 背包中的法宝
    self.m_fabaoItemList = FabaoManager:getAllUnequippedFabaoItemList()
    table.sort(self.m_fabaoItemList, FabaoUtil.fabaoSortFunc)
end

function This:initHeroList()
     -- 所有武将
    self.m_heroList = {}
    for _, hero in pairs(HeroManager:getAllHero()) do
        if hero:isGot() then
            if self.m_isOnlyWearFabao then
                if hero:isWearFabao(FabaoPos.kFirst) then
                    table.insert(self.m_heroList, hero)
                end
            else
                table.insert(self.m_heroList, hero)
            end
        end
    end
    table.sort(self.m_heroList, function(a,b)
        local aWear = a:isWearFabao(FabaoPos.kFirst) and 1 or 0
        local bWear = b:isWearFabao(FabaoPos.kFirst) and 1 or 0
        if aWear ~= bWear then
            return aWear > bWear
        end

        local aQua = a:getQuality()
        local bQua = b:getQuality()
        if aQua ~= bQua then
            return aQua > bQua
        end

        local aFp = a:getFightPower()
        local bFp = b:getFightPower()
        if aFp ~= bFp then
            return aFp > bFp
        end

        local aId = a:getId()
        local bId = b:getId()
        if aId ~= bId then
            return aId < bId
        end        

        -- if a:getQuality() > b:getQuality() then
        --     return true
        -- elseif a:getQuality() == b:getQuality() and a:getFightPower() > b:getFightPower() then
        --     return true
        -- elseif a:getQuality() == b:getQuality() and a:getFightPower() == b:getFightPower() and a:getId() < b:getId() then
        --     return true
        -- end
        return false
    end)
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureMain.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    self.m_csNode.heroScrollView.choose:setVisible(false)
    self.m_csNode.bagScrollView.choose:setVisible(false)
    self.m_csNode.see.gouxuan:setVisible(self.m_isOnlyWearFabao)

    self:initLeftListViews()
    self:initRightContents()
    -- TODO
end

function This:initRightContents()

    -- 右侧培养手段分页
    self.m_fabaoCultivateMethodTabGroup = TabGroup:new()
    -- 升星
    self.m_fabaoCultivateUpgradeLevelView = FabaoCultivateUpgradeLevelView:create()
    self.m_csNode:addChild(self.m_fabaoCultivateUpgradeLevelView)
    self.m_fabaoCultivateMethodTabGroup:addTabNode(FabaoCultivateMethodTabKeys.kUpgradeLevel, self.m_csNode.tep1, self.m_fabaoCultivateUpgradeLevelView)
    -- 精炼
    self.m_fabaoCultivateJingLianView = FabaoCultivateJingLianView:create()
    self.m_csNode:addChild(self.m_fabaoCultivateJingLianView)
    self.m_fabaoCultivateMethodTabGroup:addTabNode(FabaoCultivateMethodTabKeys.kJingLian, self.m_csNode.tep2, self.m_fabaoCultivateJingLianView)
    -- 升星
    self.m_fabaoCultivateUpgradeStarView = FabaoCultivateUpgradeStarView:create()
    self.m_csNode:addChild(self.m_fabaoCultivateUpgradeStarView)
    self.m_fabaoCultivateMethodTabGroup:addTabNode(FabaoCultivateMethodTabKeys.kUpgradeStar, self.m_csNode.tep3, self.m_fabaoCultivateUpgradeStarView)


    --self.m_fabaoCultivateMethodTabGroup:setTabChangeGuardian(function(sourceTabKey, targetTabKey)
    --    if targetTabKey == FabaoCultivateMethodTabKeys.kUpgradeStar then
    --        -- TODO
    --    end
    --    return true
    --end)

    -- TODO
end

function This:highlightBagItem(itemView)
    local x, y = itemView:getPosition()
    self.m_bagChooseView:setVisible(true)
    self.m_bagChooseView:setPosition(x - 2, y - 3)
end

function This:highlightHeroItem(itemView)
    local x, y = itemView:getPosition()
    self.m_heroChooseView:setVisible(true)
    self.m_heroChooseView:setPosition(x, y)
end

function This:initLeftListViews()

    -- 左侧法宝来源分页
    self.m_fabaoSourceTabGroup = TabGroup:new()
    self.m_fabaoSourceTabGroup:addTabNode(FabaoSourceTabKeys.kHero, self.m_csNode.hero, self.m_csNode.heroScrollView)
    self.m_fabaoSourceTabGroup:addTabNode(FabaoSourceTabKeys.kBackpack, self.m_csNode.backpack, self.m_csNode.bagScrollView)

    local function createListLayer(param)
        local container = param.scrollView
        local chooseView = container.choose
        chooseView:setLocalZOrder(1)

        local listLayer = RankPageListLayer:create(param)
        listLayer:setAnchorPoint(0.5,0)
        local w , h = listLayer:getContentSize()
        container:pushBackCustomItem(listLayer, w, h, 0)
        listLayer:setAnchorPoint(0.5,0)
        listLayer:setCascadeOpacityEnabled(true)
        return listLayer, chooseView
    end

    -- 左侧背包列表
    self.m_bagListView, self.m_bagChooseView = createListLayer({
        itemWidth = 70,
        itemHeight = 70,
        spaceX = 10,
        spaceY = 5,
        column = 4,
        scrollView = self.m_csNode.bagScrollView,
        dataCounter = function()
            return #self.m_fabaoItemList
        end,
        viewCreator = function (index, node)
            local itemInfo = self.m_fabaoItemList[index]
            if not itemInfo then return end
            if not node then
                local view = DepotItemView:create({
                    item = itemInfo,
                    index = index,
                    handler = function( itemNode )
                        --self.m_equipUid = itemNode.item:getUid()
                        --self.m_curItemView = itemNode
                        self:highlightBagItem(itemNode)

                        self:onFabaoHasBeenSelected(itemNode.item:getUid())
                    end
                })
                --if self.m_equipUid == itemInfo:getUid() then
                --    self.m_curItemView = view
                --end
                if itemInfo:getUid() == self.m_currentSelectedFabaoUid then
                    self.m_delayhighlightBagItemView = view
                end

                view:setScale(0.81)
                return view
            else
                node:setModel(itemInfo)
                if itemInfo:getUid() == self.m_currentSelectedFabaoUid then
                    self:highlightBagItem(node)
                end
            end
        end,
        finishHandler = function()
            if self.m_delayhighlightBagItemView then
                self:highlightBagItem(self.m_delayhighlightBagItemView)
                self.m_delayhighlightBagItemView = nil
            end
        end,
        hint = self.m_hintIndex
    })
    self.m_bagListView:setDataList()

    -- 左侧武将列表
    self.m_heroListView, self.m_heroChooseView = createListLayer({
        itemWidth = 325,
        itemHeight = 95,
        spaceX = 2,
        spaceY = 5,
        column = 1,
        scrollView = self.m_csNode.heroScrollView,
        dataCounter = function ()
            return #self.m_heroList
        end,
        viewCreator = function(index, node)
            local hero = self.m_heroList[index]
            local view = node
            local isSelected = hero:getFabaoUid(FabaoPos.kFirst) == self.m_currentSelectedFabaoUid
            if view then
                view:setHeroData(index, hero, isSelected)
            else
                view = HeroWithFabaoItemView:create(index, hero, isSelected)
            end

            if hero:isWearFabao(FabaoPos.kFirst) and isSelected then
                self.m_delayhighlightHeroView = view
            end

            view:setOnClickListener(function(index, view)
                --local x, y = view:getPosition()
                --self.m_heroChooseView:setPosition(x - 2, y - 3)

                if hero:isWearFabao(FabaoPos.kFirst) and hero:getFabaoUid(FabaoPos.kFirst) == self.m_currentSelectedFabaoUid then
                    self:highlightHeroItem(view)
                elseif hero:isWearFabao(FabaoPos.kFirst) then
                    self:highlightHeroItem(view)
                    local all = self.m_heroListView:getAllViews()
                    for _, itemView in pairs(all) do
                        if itemView:isVisible() then
                            itemView:setToBeSelected(itemView == view)
                        end
                    end
                    self:onFabaoHasBeenSelected(hero:getFabaoUid(FabaoPos.kFirst))
                --else
                --    self:onFabaoHasBeenSelected(nil)
                end
            end)
            return view
        end,
        finishHandler = function()
            if self.m_delayhighlightHeroView then
                self:highlightHeroItem(self.m_delayhighlightHeroView)
                self.m_delayhighlightHeroView = nil
            end
        end,
        hint = self.m_hintIndex
    })
    self.m_heroListView:setDataList()
    -- TODO
end

--function This:selectHero(index)
--    self.m_selectedHeroIndex = index
--    if self.m_selectedHeroIndex == 0 then
--        self.m_heroChooseView:setVisible(false)
--    else
--        self.m_heroChooseView:setVisible(true)
--        local x, y = self.m_heroListView:getPosByIndex(self.m_selectedHeroIndex)
--        self.m_heroChooseView:setPosition(x - 2, y - 3)
--        -- TODO
--    end
--end

function This:onEnter()
    super.onEnter(self)
    FabaoManager:addObserver(self)
    HeroManager:addObserver(self)

    self:onFabaoHasBeenSelected(self.m_currentSelectedFabaoUid)
    -- 切换至对应分页
    if self.m_sourceType == FabaoConstants.SourceType.Hero then
        self.m_fabaoSourceTabGroup:selectTab(FabaoSourceTabKeys.kHero)
    elseif self.m_sourceType == FabaoConstants.SourceType.Backpack then
        self.m_fabaoSourceTabGroup:selectTab(FabaoSourceTabKeys.kBackpack)
    else
        error("error!")
    end

    -- 打开法宝升级分页
    self.m_fabaoCultivateMethodTabGroup:selectTab(FabaoCultivateMethodTabKeys.kUpgradeLevel)
end

function This:onExit()
    self.m_allHeroItemViews = nil
    FabaoManager:removeObserver(self)
    HeroManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    -- 关闭按钮
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.see.weigouxuan:setTouchEnded(function()
        self.m_isOnlyWearFabao = not self.m_isOnlyWearFabao
        self.m_csNode.see.gouxuan:setVisible(self.m_isOnlyWearFabao)
        self:initHeroList()
        self.m_heroChooseView:setVisible(false)
        self.m_heroListView:setDataList()
    end)
end

function This:onUnequippedFabaosChanged()
    -- 刷新背包分页
    self.m_fabaoItemList = FabaoManager:getAllUnequippedFabaoItemList()
    table.sort(self.m_fabaoItemList, FabaoUtil.fabaoSortFunc)
    self.m_bagChooseView:setVisible(false)
    self.m_bagListView:setDataList()
end

function This:onFabaoHasBeenSelected(fabaoUid)
    self.m_currentSelectedFabaoUid = fabaoUid
    local fabao = FabaoManager:getFabao(fabaoUid)
    self.m_fabaoCultivateUpgradeLevelView:setFabao(fabao)
    self.m_fabaoCultivateJingLianView:setFabao(fabao)
    self.m_fabaoCultivateUpgradeStarView:setFabao(fabao)
end



function This:onEquipChanged(heroId)
    local hero = HeroManager:getHero(heroId)
    if hero:isWearFabao(FabaoPos.kFirst) then
        local fabaoUid = hero:getFabaoUid(FabaoPos.kFirst)
        self:onFabaoHasBeenSelected(fabaoUid)
    end

    self:initHeroList()
    self.m_heroChooseView:setVisible(false)
    self.m_heroListView:setDataList()
end

function This:onFabaoDataChanged(fabaoUid)
    self:initHeroList()
    self.m_heroChooseView:setVisible(false)
    self.m_heroListView:setDataList()    
end