---
---
--- Created by aid.
--- DateTime: 2018/12/20 14:37
---
local This , super = class(...,"cc.Node")
local MagicWeaponLevelConfig = require("Metadatas.MagicWeaponLevelConfig")
local GameUtil = require("Util.GameUtil")
local FabaoManager = require("Fabao.FabaoManager")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")
local FabaoUtil = require("Fabao.FabaoUtil")
local ItemNode = require("UI.ItemNode")

function This:init()
    self:initData()
    self:initNode()
    self:initTouchEvent()
end

function This:initData()
    self.m_continuousUpgradeLevel = false
    self.m_fabao = nil -- 当前的法宝
    self.m_lastFabao = nil -- 上个法宝
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureUpgrade.csb")
    self:addChild(self.m_csNode)

    --self:setContinuousUpgradeLevel(false)
    self.m_continuousUpgradeLevel = false
    self.m_csNode.gouxuan:setVisible(self.m_continuousUpgradeLevel)
    self.m_csNode.weigouxuan:setVisible(not self.m_continuousUpgradeLevel)
end

function This:initTouchEvent()
    -- 帮助按钮
    self.m_csNode.help:setTouch({
        onEnded = function()
            g_scene:openDialog("UI.HelpTips", {text = Serialize[60894] , node = self.m_csNode.help})
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 升级按钮
    self.m_csNode.ascendbotton:setTouch({
        onEnded = function()
            local fabao = self.m_fabao
            local incrementalLevel = 1
            if self.m_continuousUpgradeLevel then
                incrementalLevel = 10
            end

            -- 检查法宝等级
            if self.m_fabao:getLevel() == self.m_maxLevel then
                g_scene:showTips(Serialize[60895]) -- TODO 国际化
                return
            end

            -- 检查玩家等级
            if GamePlayer:getLevel() < self.m_currentLevelConfig.need_level then
                g_scene:showTips(Serialize[60887]) -- TODO 国际化
                return
            end

            -- 检查物品数量
            local items = self:getCostItems()
            for itemId, needCount in pairs(items) do
                if DepotManager:getItemOrResCount(itemId, {itemUid = 0}) < needCount then
                    g_scene:showTips(Serialize[60714] .. ItemUtil.getName(itemId)..Serialize[60800]) -- TODO 国际化
                    return
                end
            end

            cc.Audio:playEffect("Recruit_Change.mp3")
            -- 发送法宝升级请求
            FabaoManager:onUpgradeFabaoLevelRequest(fabao:getUid(), incrementalLevel)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 连升10级勾选
    self.m_csNode.gouxuan:setTouch({
        onEnded = function()
            self:setContinuousUpgradeLevel(false)
            local items, itemTypeCount = self:getCostItems()
            self:setCostItems(items, itemTypeCount)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })
    self.m_csNode.weigouxuan:setTouch({
        onEnded = function()
            self:setContinuousUpgradeLevel(true)
            local items, itemTypeCount = self:getCostItems()
            self:setCostItems(items, itemTypeCount)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })
    self.m_csNode.liansheng:setTouch({
        onEnded = function()
            self:setContinuousUpgradeLevel(not self.m_continuousUpgradeLevel)
            local items, itemTypeCount = self:getCostItems()
            self:setCostItems(items, itemTypeCount)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

end

function This:getCostItems()
    local incrementalLevel = 1
    if self.m_continuousUpgradeLevel then
        incrementalLevel = 10
    end
    local targetLevel = self.m_fabao:getLevel()+incrementalLevel
    if targetLevel > self.m_maxLevel then
        targetLevel = self.m_maxLevel
    end
    local items = {}
    local count = 0
    for level = self.m_fabao:getLevel(), targetLevel-1 do
        for _, item in ipairs(MagicWeaponLevelConfig[self.m_fabao:getItemId()*1000+level].cost) do
            local itemId = item[1]
            local itemCount = item[2]
            local originalCount = items[itemId]
            if not originalCount then
                originalCount = 0
                count = count + 1
            end
            items[itemId] = originalCount + itemCount
        end
    end
    return items, count
end

function This:setContinuousUpgradeLevel(yes)
    self.m_continuousUpgradeLevel = yes
    self.m_csNode.gouxuan:setVisible(self.m_continuousUpgradeLevel)
    self.m_csNode.weigouxuan:setVisible(not self.m_continuousUpgradeLevel)
    self:showNextLevelAttributes(self.m_fabao:getLevel())
end

function This:onEnter()
    super.onEnter(self)
    FabaoManager:addObserver(self)
end

function This:onExit()
    FabaoManager:removeObserver(self)
    super.onExit(self)
end

function This:setCostItems(items, itemTypeCount)
    local viewCount, tw = itemTypeCount, 0
    local offsetX = 60 / viewCount
    local itemWidth
    local itemsNode = self.m_csNode.itemtipslist
    itemsNode:setVisible(true)
    itemsNode:removeAllChildren()
    for itemId, neededCount in pairs(items) do
        local itemNode = ItemNode:create(itemId, neededCount, true)
        itemNode:setAnchorPoint(0,0.5)
        itemNode:showNeedNum(neededCount)
        itemsNode:addChild(itemNode)
        if not itemWidth then
            itemWidth = itemNode:getContentWidth()
        end
        itemNode:setPositionX(tw)
        tw = tw + itemWidth + offsetX + 30
    end
    itemsNode:setPositionX( 200 - tw / 2 )
end

function This:setFabao(fabao)
    self.m_lastFabao = self.m_fabao
    self.m_fabao = fabao

    local itemId = self.m_fabao:getItemId()
    local level = self.m_fabao:getLevel()

    -- 法宝可升至的最高等级
    self.m_maxLevel = FabaoUtil.getFabaoMaxLevel(itemId)

    -- 法宝图标
    local fabaoRootNode = self.m_csNode.EquipItem
    local tag = 1545290822;
    self.m_fabaoItemView = fabaoRootNode:getChildByTag(tag)
    if not self.m_fabaoItemView then
        self.m_fabaoItemView = ItemNode:create(self.m_fabao:getItemId())
        self.m_fabaoItemView:setAnchorPoint(0.5,0.5)
        self.m_fabaoItemView:setTag(tag)
        fabaoRootNode:addChild(self.m_fabaoItemView)
    end
    self.m_fabaoItemView:initItem(self.m_fabao:getItemId())
    self.m_fabaoItemView:setToFabao(self.m_fabao:getLevel(), self.m_fabao:getJinglianDegree(), self.m_fabao:getStarLevel())

    -- 当前属性
    self.m_csNode.level.text:setString(""..level..Serialize[60187])
    self.m_currentLevelConfig = MagicWeaponLevelConfig[itemId*1000+level]
    local attributeRootNode = self.m_csNode.before
    local currentAttributeList = FabaoUtil.getLevelAttributeList(itemId, level, self.m_fabao:getStarLevel())
    for index=1, 3 do
        local attributeNameView = attributeRootNode["property"..index]
        local attributeValueView = attributeRootNode["property"..index.."num"]
        local attribute = currentAttributeList[index]
        if attribute then
            local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeNameStr)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end

    -- 下一级属性
    self:showNextLevelAttributes(level)

    -- 升至下一级需要消耗的物品
    local items, itemTypeCount = self:getCostItems()
    self:setCostItems(items, itemTypeCount)


    -- 播放属性变更动画
    if self.m_lastFabao
            and self.m_lastFabao:getUid() == self.m_fabao:getUid()
            and self.m_lastFabaoLevel
            and self.m_fabao:getLevel() > self.m_lastFabaoLevel
            and self:isVisible()
    then
        for index=1, #currentAttributeList do
            FabaoUtil.showAnimeOn(self.m_csNode.effect["effect"..index])
        end
    end
    self.m_lastFabaoLevel = self.m_fabao:getLevel()
end

-- 根据连升是否选中显示下次升级提升的属性
function This:showNextLevelAttributes(currentLevel)
    if currentLevel == self.m_maxLevel then
        self:setLevelAttributesToNext(currentLevel+1)
    elseif self.m_continuousUpgradeLevel then
        local incrementalLevel = self.m_maxLevel - currentLevel
        if incrementalLevel > 10 then
            -- 连升10级
            self:setLevelAttributesToNext(currentLevel+10)
        elseif incrementalLevel == 1 then
            -- 没必要连升
            self:setLevelAttributesToNext(currentLevel+1)
        else
            -- 连升incrementalLevel级
            self:setLevelAttributesToNext(currentLevel+incrementalLevel)
        end
    else
        self:setLevelAttributesToNext(currentLevel+1)
    end
end

function This:setLevelAttributesToNext(level)
    local itemId = self.m_fabao:getItemId()
    local attributeRootNode = self.m_csNode.after
    if level <= self.m_maxLevel then
        local nextAttributeList = FabaoUtil.getLevelAttributeList(itemId, level, self.m_fabao:getStarLevel())
        for index=1, 3 do
            local attributeNameView = attributeRootNode["property"..index]
            local attributeValueView = attributeRootNode["property"..index.."num"]
            local attribute = nextAttributeList[index]
            if attribute then
                local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
                attributeNameView:setString(attributeNameStr)
                attributeValueView:setString(attributeValueStr)
                attributeNameView:setVisible(true)
                attributeValueView:setVisible(true)
            else
                attributeNameView:setVisible(false)
                attributeValueView:setVisible(false)
            end
        end
        self.m_csNode.level2.text:setString(""..level..Serialize[60187])
    else
        for index = 1, 3 do
            local attributeNameNode = attributeRootNode["property"..index]
            local attributeValueStrNode = attributeRootNode["property"..index.."num"]
            attributeNameNode:setVisible(false)
            attributeValueStrNode:setVisible(false)
        end
        self.m_csNode.level2.text:setString(Serialize[60353])
    end
end

function This:onFabaoDataChanged(fabaoUid)
    if self.m_fabao:getUid() == fabaoUid then
        self:setFabao(FabaoManager:getFabao(fabaoUid))
    end
end