---
---
--- Created by aid.
--- DateTime: 2018/12/20 14:40
---
local This , super = class(...,"cc.Node")
local MagicWeaponStartConfig = require("Metadatas.MagicWeaponStartConfig")
local MagicWeaponLevelConfig = require("Metadatas.MagicWeaponLevelConfig")
local MagicWeaponRefineConfig = require("Metadatas.MagicWeaponRefineConfig")
local ItemUtil = require("Bag.ItemUtil")
local SkillConfig = require("Metadatas.SkillConfig")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local FabaoUtil = require("Fabao.FabaoUtil")
local FabaoManager = require("Fabao.FabaoManager")
local GamePlayer = require("Player.GamePlayer")


function This:init()
    self:initData()
    self:initNode()
    self:initTouchEvent()
end

function This:initData()
    self.m_fabao = nil -- 当前的法宝
    self.m_lastFabao = nil -- 上个法宝
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureStarup.csb")
    self:addChild(self.m_csNode)

    self.m_before = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureStarupValue.csb")
    local width, height = self.m_before.Image_1:getContentSize()
    self.m_csNode.before.scrollView:pushNode(self.m_before, width, height, 1)

    self.m_after = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureStarupValue.csb")
    self.m_csNode.after.scrollView:pushNode(self.m_after, width, height, 1)
end

function This:initTouchEvent()
    -- 帮助按钮
    self.m_csNode.help:setTouch({
        onEnded = function()
            g_scene:openDialog("UI.HelpTips", {text = Serialize[60897] , node = self.m_csNode.help})
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 升星按钮
    self.m_csNode.ascendbotton:setTouch({
        onEnded = function()

            -- 检查法宝星级
            if self.m_fabao:getStarLevel() == self.m_maxStarLevel then
                g_scene:showTips(Serialize[60898]) -- TODO 国际化
                return
            end

            -- 检查玩家等级
            if GamePlayer:getLevel() < self.m_currentStarLevelConfig.need_level then
                g_scene:showTips(Serialize[60887]) -- TODO 国际化
                return
            end

            -- 检查消耗品是否足够
            local itemList = {}
            for position, fabaoUid in pairs(self.m_neededItems) do
                table.insert(itemList, fabaoUid)
            end
            if #itemList ~= self.m_neededItemCount then
                g_scene:showTips(Serialize[60714] .. ItemUtil.getName(self.m_fabao:getItemId())..Serialize[60800]) -- TODO 国际化
                return
            end

            cc.Audio:playEffect("Recruit_Change.mp3")
            -- 发送升星请求
            FabaoManager:onUpgradeFabaoStarRequest(self.m_fabao:getUid(), itemList)
        end,
        isScale = false,
        isDynamic = true,
        isSwallow = false,
    })

    -- 点击添加消耗品
    for i=1, 5 do
        self.m_csNode.itemtipslist["need"..i].add:setTouch({
            onEnded = function()
                self:onAddNeededItem(i)
            end,
            isScale = false,
            isDynamic = true,
            isSwallow = false,
        })
    end

    -- 点击删除消耗品
    for i=1, 5 do
        self.m_csNode.itemtipslist["need"..i]:setTouch({
            onEnded = function()
                self:onDeleteNeededItem(i)
            end,
            isScale = false,
            isDynamic = true,
            isSwallow = false,
        })
    end
end

function This:onEnter()
    super.onEnter(self)
    FabaoManager:addObserver(self)
end

function This:onExit()
    FabaoManager:removeObserver(self)
    super.onExit(self)
end

local itemViewTag = 1545741452;

function This:onAddNeededItem(position)
    if self.m_neededItems[position] then
        return
    end

    local panel = g_scene:openDialog("Equip.EquipChoosePanel",{
        pos = FabaoPos.kFirst,
        type = ItemCommonType.kFaBao,
        emptyHint = Serialize[60899],
        filter = function(fabaoUid)
            if fabaoUid == self.m_fabao:getUid() then
                return false
            end
            for _, selectedFabaoUid in pairs(self.m_neededItems) do
                if fabaoUid == selectedFabaoUid then
                    return false
                end
            end
            local fabao = FabaoManager:getFabao(fabaoUid)
            if fabao:getItemId() ~= self.m_fabao:getItemId() then
                return false
            end
            if fabao:getStarLevel() ~= self.m_fabao:getStarLevel() then
                return false
            end
            return true
        end,
    })
    panel:setSelectHandler(function(uid)
        self.m_neededItems[position] = uid
        local fabaoRootNode = self.m_csNode.itemtipslist["need"..position]
        local fabao = FabaoManager:getFabao(uid)
        fabaoRootNode.add:setVisible(false)
        local fabaoItemView = fabaoRootNode:getChildByTag(itemViewTag)
        if not fabaoItemView then
            fabaoItemView = ItemNode:create(fabao:getItemId(), nil, nil, false)
            fabaoItemView:setAnchorPoint(0.5, 0.5)
            fabaoItemView:setTag(itemViewTag)
            fabaoItemView:setPosition(46, 46)
            fabaoRootNode:addChild(fabaoItemView)
        end
        fabaoItemView:initItem(fabao:getItemId())
        fabaoItemView:setToFabao(fabao:getLevel(), fabao:getJinglianDegree(), fabao:getStarLevel())

    end)
end

function This:onDeleteNeededItem(position)
    if not self.m_neededItems[position] then
        return
    end
    self.m_neededItems[position] = nil

    local fabaoRootNode = self.m_csNode.itemtipslist["need"..position]
    local fabaoItemView = fabaoRootNode:getChildByTag(itemViewTag)
    fabaoItemView:removeFromParent()
    fabaoRootNode.add:setVisible(true)
end

function This:setFabao(fabao)
    self.m_lastFabao = self.m_fabao
    self.m_fabao = fabao

    -- 法宝图标
    local fabaoRootNode = self.m_csNode.EquipItem
    local tag = 1545473483;
    self.m_fabaoItemView = fabaoRootNode:getChildByTag(tag)
    if not self.m_fabaoItemView then
        self.m_fabaoItemView = ItemNode:create(self.m_fabao:getItemId())
        self.m_fabaoItemView:setAnchorPoint(0.5,0.5)
        self.m_fabaoItemView:setTag(tag)
        fabaoRootNode:addChild(self.m_fabaoItemView)
    end
    self.m_fabaoItemView:initItem(self.m_fabao:getItemId())
    self.m_fabaoItemView:setToFabao(self.m_fabao:getLevel(), self.m_fabao:getJinglianDegree(), self.m_fabao:getStarLevel())

    local starLevel = fabao:getStarLevel()

    -- 当前星级属性
    self.m_csNode.level.text:setString(starLevel)
    self.m_currentStarLevelConfig = MagicWeaponStartConfig[fabao:getItemId()*1000+starLevel]
    local attributeRootNode = self.m_before
    local currentAttributes = {}
    local attributeCount = 0
    for _, attribute in ipairs(FabaoUtil.getLevelAttributeList(fabao:getItemId(), fabao:getLevel(), starLevel)) do
        if currentAttributes[attribute[1]] then
            currentAttributes[attribute[1]] = currentAttributes[attribute[1]] + attribute[2]
        else
            currentAttributes[attribute[1]] = attribute[2]
            attributeCount = attributeCount + 1
        end
    end
    for _, attribute in ipairs(FabaoUtil.getJinglianAttributeList(fabao:getItemId(), fabao:getJinglianDegree(), starLevel)) do
        if currentAttributes[attribute[1]] then
            currentAttributes[attribute[1]] = currentAttributes[attribute[1]] + attribute[2]
        else
            currentAttributes[attribute[1]] = attribute[2]
            attributeCount = attributeCount + 1
        end
    end
    local index = 1
    for attributeId, attributeValue in pairs(currentAttributes) do
        local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attributeId, attributeValue)
        local attributeNameView = attributeRootNode["property"..index]
        local attributeValueView = attributeRootNode["property"..index.."num"]
        attributeNameView:setString(attributeNameStr)
        attributeValueView:setString(attributeValueStr)
        attributeNameView:setVisible(true)
        attributeValueView:setVisible(true)
        index = index + 1
    end
    for i = index, 6 do
        attributeRootNode["property"..i]:setVisible(false)
        attributeRootNode["property"..i.."num"]:setVisible(false)
    end

    -- 下一级星级属性
    local nextStarLevelConfig = MagicWeaponStartConfig[fabao:getItemId()*1000+starLevel+1]
    if nextStarLevelConfig then
        -- 新增效果或技能提升
        local newEffect, effectType = FabaoUtil.getDifferentBetweenStarLevel(fabao:getItemId(), starLevel, starLevel+1)
        if newEffect then
            if effectType == FabaoUtil.Type.Attribute then
                -- 新属性
                local attributeName, attributeValueStr = GameUtil:getAttrStr(newEffect[1], newEffect[2])
                self.m_csNode.xiaoguobg.Text1:setString(Serialize[60900])
                self.m_csNode.xiaoguobg.Text2:setString(attributeName..attributeValueStr, 550/self.m_csNode.xiaoguobg.Text2:getScale())
            elseif effectType == FabaoUtil.Type.Skill then
                -- 新技能
                local skillConfig = SkillConfig[newEffect]
                self.m_csNode.xiaoguobg.Text1:setString(string.format(Serialize[60901], skillConfig.name))
                self.m_csNode.xiaoguobg.Text2:setString(skillConfig.desp, 550/self.m_csNode.xiaoguobg.Text2:getScale())
            else
                error("error!")
            end
        else
            -- 没有新增属性或技能提升的改变
            self.m_csNode.xiaoguobg.Text1:setString(Serialize[60900])
            self.m_csNode.xiaoguobg.Text2:setString(Serialize[60243], 550/self.m_csNode.xiaoguobg.Text2:getScale())
        end

        local attributeRootNode = self.m_after
        local nextAttributes = {}
        for _, attribute in ipairs(FabaoUtil.getLevelAttributeList(fabao:getItemId(), fabao:getLevel(), starLevel+1)) do
            if nextAttributes[attribute[1]] then
                nextAttributes[attribute[1]] = nextAttributes[attribute[1]] + attribute[2]
            else
                nextAttributes[attribute[1]] = attribute[2]
            end
        end
        for _, attribute in ipairs(FabaoUtil.getJinglianAttributeList(fabao:getItemId(), fabao:getJinglianDegree(), starLevel+1)) do
            if nextAttributes[attribute[1]] then
                nextAttributes[attribute[1]] = nextAttributes[attribute[1]] + attribute[2]
            else
                nextAttributes[attribute[1]] = attribute[2]
            end
        end
        local index = 1
        for attributeId, attributeValue in pairs(nextAttributes) do
            local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attributeId, attributeValue)
            local attributeNameView = attributeRootNode["property"..index]
            local attributeValueView = attributeRootNode["property"..index.."num"]
            attributeNameView:setStringWithColor("<GreenAttr>"..attributeNameStr)
            attributeValueView:setStringWithColor("<GreenAttr>"..attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
            index = index + 1
        end
        for i = index, 6 do
            attributeRootNode["property"..i]:setVisible(false)
            attributeRootNode["property"..i.."num"]:setVisible(false)
        end
        self.m_csNode.level2.text:setString(starLevel+1)
    else
        self.m_csNode.level2.text:setString(Serialize[60890])
    end

    -- 升星消耗的道具槽
    self.m_neededItems = {}
    self.m_neededItemCount = self.m_currentStarLevelConfig.cost
    for i=1, 5 do
        local view = self.m_csNode.itemtipslist["need"..i]
        if i > self.m_neededItemCount then
            view:setVisible(false)
        else
            view:setVisible(true)
        end
        view.add:setVisible(true)
        local itemView = view:getChildByTag(itemViewTag)
        if itemView then
            itemView:removeFromParent()
        end
    end

    -- 位置居中
    local centerX = self.m_csNode.title3:getPositionX()
    local view = self.m_csNode.itemtipslist.need1
    local newX = centerX-(view:getContentWidth()*view:getScaleX()*self.m_neededItemCount+(self.m_neededItemCount-1)*10)/2
    self.m_csNode.itemtipslist:setPositionX(newX)

    -- 升星可达到的最高星数
    self.m_maxStarLevel = starLevel
    while MagicWeaponStartConfig[fabao:getItemId()*1000+self.m_maxStarLevel+1] do
        self.m_maxStarLevel = self.m_maxStarLevel + 1
    end

    -- 播放属性变更动画
    if self.m_lastFabao
            and self.m_lastFabao:getUid() == self.m_fabao:getUid()
            and self.m_lastStarLevel
            and self.m_fabao:getStarLevel() > self.m_lastStarLevel
            and self:isVisible()
    then
        local count = attributeCount
        if count > 3 then count = 3 end
        for index=1, count do
            FabaoUtil.showAnimeOn(self.m_csNode.effect["effect"..index])
        end
    end
    self.m_lastStarLevel = self.m_fabao:getStarLevel()
end

function This:onFabaoDataChanged(fabaoUid)
    if self.m_fabao:getUid() == fabaoUid then
        self:setFabao(FabaoManager:getFabao(fabaoUid))
    end
end