---
---
--- Created by aid.
--- DateTime: 2018/12/28 10:27
---
local This , super = class(...,"Dialog")
local ItemUtil = require("Bag.ItemUtil")
local FabaoHandbookItemView = require("Fabao.FabaoHandbookItemView")
local FabaoUtil = require("Fabao.FabaoUtil")
local GameUtil = require("Util.GameUtil")
local SkillConfig = require("Metadatas.SkillConfig")
local FabaoSkillView = require("Fabao.FabaoSkillView")
local ItemNode = require("UI.ItemNode")
local RankPageListLayer = require("Rank.RankPageListLayer")
local MagicWeaponShowConfig = require("Metadatas.MagicWeaponShowConfig")

function This:initData()
    super.initData(self)
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureBook.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    local function createListLayer(param)
        local container = param.scrollView
        local chooseView = container.choose
        chooseView:setLocalZOrder(1)

        local listLayer = RankPageListLayer:create(param)
        listLayer:setAnchorPoint(0.5,0)
        local w , h = listLayer:getContentSize()
        container:pushBackCustomItem(listLayer, w, h, 0)
        listLayer:setAnchorPoint(0.5,0)
        listLayer:setCascadeOpacityEnabled(true)
        return listLayer, chooseView
    end

    self.m_allItemView = {}
    self.m_fabaoListView, self.m_fabaoChooseView = createListLayer({
        itemWidth = 230,
        itemHeight = 100,
        spaceX = 2,
        spaceY = 5,
        column = 1,
        scrollView = self.m_csNode.heroScrollView,
        dataCounter = function ()
            return #MagicWeaponShowConfig
        end,
        viewCreator = function(index, node)
            local config = MagicWeaponShowConfig[index]
            local view = node
            if not view then
                view = FabaoHandbookItemView:create()
                table.insert(self.m_allItemView, view)
            end
            view:setFabaoItemId(index, config.id, config.fabao_type, self.m_selectedIndex)

            view:setOnClickListener(function(index, view)
                self:selectFabao(index, view)
            end)
            return view
        end,
    })
    self.m_fabaoListView:setDataList()
    self.m_fabaoChooseView:setVisible(false)
end

--function This:highlightFabaoItem(index)
--    local x, y = self.m_fabaoListView:getPosByIndex(index)
--    self.m_fabaoChooseView:setPosition(x, y)
--end

function This:selectFabao(index)
    self.m_selectedIndex = index
    self:setRightFabaoInfo(MagicWeaponShowConfig[index].id)
    for _, view in ipairs(self.m_allItemView) do
        view:setToBeSelected(view:getIndex() == index)
    end
end

function This:onEnter()
    super.onEnter(self)
    self:selectFabao(1)
end

function This:onExit()
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
end

local ChineseNumber = {
    Serialize[60012],Serialize[60013],Serialize[60014],Serialize[60015],Serialize[60016],Serialize[60097],Serialize[60018],Serialize[60019],Serialize[60020],
}

function This:setRightFabaoInfo(fabaoItemId)
    -- 法宝名
    self.m_csNode.text1:setString(ItemUtil.getName(fabaoItemId))
    self.m_csNode.text1:setColorByID("Q".. ItemUtil.getQuality(fabaoItemId))
    -- 法宝描述
    self.m_csNode.text2:setString(ItemUtil.getComment(fabaoItemId), 291)
    -- 法宝基础属性
    local levelAttributeList = FabaoUtil.getLevelAttributeList(fabaoItemId, FabaoUtil.getFabaoMaxLevel(fabaoItemId), FabaoUtil.getFabaoMaxStarLevel(fabaoItemId))
    local attributeNode = self.m_csNode.value
    for i=1, 3 do
        local attributeNameView = attributeNode["name"..i]
        local attributeValueView = attributeNode["num"..i]
        local attribute = levelAttributeList[i]
        if attribute then
            local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeNameStr)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end
    -- 法宝精炼属性
    local refineAttributeList = FabaoUtil.getJinglianAttributeList(fabaoItemId, FabaoUtil.getFabaoMaxRefineDegree(fabaoItemId), FabaoUtil.getFabaoMaxStarLevel(fabaoItemId))
    for i=4, 6 do
        local attributeNameView = attributeNode["name"..i]
        local attributeValueView = attributeNode["num"..i]
        local attribute = refineAttributeList[i-3]
        if attribute then
            local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeNameStr)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end

    --for i=4, 6 do
    --    local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attributeList[i-3][1], attributeList[i-3][2])
    --    attributeNode["name"..i]:setString(attributeNameStr)
    --    attributeNode["num"..i]:setString(attributeValueStr)
    --end
    -- 升星效果
    for index = 1, 5 do
        local formatText = Serialize[60708]
        local newEffectText
        local newEffect, effectType = FabaoUtil.getDifferentBetweenStarLevel(fabaoItemId, index-1, index)
        if effectType == FabaoUtil.Type.Attribute then
            local attributeName, attributeValueStr = GameUtil:getAttrStr(newEffect[1], newEffect[2])
            newEffectText = string.format("%s%s", attributeName, attributeValueStr)
        elseif effectType == FabaoUtil.Type.Skill then
            local skillConfig = SkillConfig[newEffect]
            newEffectText = string.format(Serialize[60709], skillConfig.name, newEffect % 10)
        else
            formatText = Serialize[60710]
            newEffectText = ""
        end

        local view = attributeNode[""..index.."xing"]
        view:setString(string.format(
                formatText,
                ChineseNumber[index],
                FabaoUtil.getStarLevelAttributesEnhanciveFactor(fabaoItemId, index)*100, -- TODO 这里显示的是全量，而不是差异，是否要修改？
                newEffectText
        ))
    end

    -- 法宝图标
    if self.m_fabaoIconView then
        self.m_fabaoIconView:removeFromParent()
    end
    self.m_fabaoIconView = ItemNode:create(fabaoItemId)
    self.m_fabaoIconView:setAnchorPoint(0.5, 0.5)
    self.m_fabaoIconView:setToFabao(
            FabaoUtil.getFabaoMaxLevel(fabaoItemId),
            FabaoUtil.getFabaoMaxRefineDegree(fabaoItemId),
            FabaoUtil.getFabaoMaxStarLevel(fabaoItemId)
    )
    self.m_csNode.item:addChild(self.m_fabaoIconView)

    -- 技能图标
    if self.m_fabaoSkillView then
        self.m_fabaoSkillView:removeFromParent()
    end
    self.m_fabaoSkillView = FabaoSkillView:create()
    self.m_fabaoSkillView:setFabaoData(fabaoItemId, FabaoUtil.getFabaoMaxStarLevel(fabaoItemId))
    self.m_fabaoSkillView:hideActivationState()
    self.m_csNode.item1:addChild(self.m_fabaoSkillView)

end