
---
---
--- Created by aid.
--- DateTime: 2018/12/14 10:02
---
local This , super = class(...,"Base.BaseModel")
local Socket = require("Net.Socket")
local EventDispatcher = require("EventDispatcher")
local ItemConfig = require("Metadatas.ItemConfig")
local Fabao = require("Fabao.Fabao")
local HeroManager = require("Hero.HeroManager")

function This:init()
    super.init(self)
    self.m_unequippedFabaos = {}
    self.m_equippedFabaos = {}

    self:initListener()
end

function This:initListener()
    EventDispatcher:addListener(self, 10600, self.onReceiveAllOwnedEquipableItem)
    EventDispatcher:addListener(self, 10601, self.onReceiveChangedEquipableItems)
    EventDispatcher:addListener(self, 10439, self.onReceiveFabaoUpgradeLevelResultReply)
    EventDispatcher:addListener(self, 10440, self.onReceiveFabaoRefineResultReply)
    EventDispatcher:addListener(self, 10441, self.onReceiveFabaoUpgradeStarResultReply)
    EventDispatcher:addListener(self, 10445, self.onReceiveRefineStonePackageResultReply)
    EventDispatcher:addListener(self, 10446, self.onReceiveRefineStonePackageStatusReply)
end

function This:getUnequippedFabaoUidList(position)
    local fabaoUidList = {}
    for fabaoUid, _ in pairs(self.m_unequippedFabaos) do
        table.insert(fabaoUidList, fabaoUid)
    end
    return fabaoUidList
end

function This:getAllUnequippedFabaoItemList()
    local fabaoItemList = {}
    for _, fabao in pairs(self.m_unequippedFabaos) do
        table.insert(fabaoItemList, fabao:asItem())
    end
    return fabaoItemList
end

function This:getFabao(uid)

    local fabao = self.m_unequippedFabaos[uid]
    if fabao then
        return fabao
    end
    fabao = self.m_equippedFabaos[uid]
    if fabao then
        return fabao
    end
    return nil
end

function This:changeFabaoToUnequipped(fabaoUid)
    --moveFabaoBetweenFabaoList(self.m_equippedFabaos, self.m_unequippedFabaos, fabaoUid)
    local fabao = self.m_equippedFabaos[fabaoUid]
    self.m_equippedFabaos[fabaoUid] = nil
    self.m_unequippedFabaos[fabaoUid] = fabao
    self:notifyObservers("onUnequippedFabaosChanged")
end

function This:changeFabaoToEquipped(fabaoUid)
    --moveFabaoBetweenFabaoList(self.m_unequippedFabaos, self.m_equippedFabaos, fabaoUid)
    local fabao = self.m_unequippedFabaos[fabaoUid]
    self.m_unequippedFabaos[fabaoUid] = nil
    self.m_equippedFabaos[fabaoUid] = fabao
    self:notifyObservers("onUnequippedFabaosChanged")
end

function This:onReceiveAllOwnedEquipableItem(message)
    self.m_unequippedFabaos = {}
    self.m_equippedFabaos = {}
    local itemList = message.bag_item_list
    for _, item in ipairs(itemList) do
        if  ItemConfig[item.type_id].itype == ItemCommonType.kFaBao then
            local fabao = Fabao:create(item)
            if item.equip_card_id == 0 then
                self.m_unequippedFabaos[fabao:getUid()] = fabao
            else
                self.m_equippedFabaos[fabao:getUid()] = fabao
            end
        end
    end
    self:notifyObservers("onUnequippedFabaosChanged")
end

-- 处理法宝被删除动作，刷新法宝因为升级，精炼和升星造成的数据变化
function This:onReceiveChangedEquipableItems(message)
    local hasBeenDeleted = false
    for _, item in pairs(message.bag_item_list) do
        if  ItemConfig[item.type_id].itype == 7 then
            if item.num == 0 then -- 法宝被删除
                self.m_unequippedFabaos[item.id] = nil
                hasBeenDeleted = true
            else
                local unequippedFabao = self.m_unequippedFabaos[item.id]
                local equippedFabao = self.m_equippedFabaos[item.id]
                if unequippedFabao or equippedFabao then
                    -- 法宝数据发生变化
                    if equippedFabao then
                        equippedFabao:setRawData(item)
                    end
                    if unequippedFabao then
                        unequippedFabao:setRawData(item)
                    end

                    --self:notifyObservers("onFabaoDataChanged", item.id)
                else
                    -- 新增的法宝
                    local fabao = Fabao:create(item)
                    self.m_unequippedFabaos[fabao:getUid()] = fabao
                    self:notifyObservers("onUnequippedFabaosChanged")
                end
            end
        end
    end
    if hasBeenDeleted then
        -- 由于目前只有在背包里的物品才有可能被删除（例如：出售），所以向外通知的是 onUnequippedFabaosChanged 事件
        self:notifyObservers("onUnequippedFabaosChanged")
    end
end

-- 法宝升级
function This:onUpgradeFabaoLevelRequest(fabaoUid, incrementalLevel)
    Socket:sendAndLock(10439,{item_id=fabaoUid, up_levels=incrementalLevel})
end
-- 法宝升级结果回复
function This:onReceiveFabaoUpgradeLevelResultReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    local fabao = self:getFabao(msg.item_id)
    if fabao:isEquipped() then
        local hero = HeroManager:getHeroByUid(fabao:getHeroUid())
        hero:updateFabao(msg.item_id, msg.item_id)
        HeroManager:calcSaintFightPower()
    end

    self:notifyObservers("onFabaoDataChanged", msg.item_id)
end

-- 法宝精炼
function This:onRefineFabaoRequest(fabaoUid, times)
    Socket:sendAndLock(10440,{item_id=fabaoUid, cost_count=times})
end
-- 法宝精炼结果回复
function This:onReceiveFabaoRefineResultReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    local fabao = self:getFabao(msg.item_id)
    if fabao:isEquipped() then
        local hero = HeroManager:getHeroByUid(fabao:getHeroUid())
        hero:updateFabao(msg.item_id, msg.item_id)
        HeroManager:calcSaintFightPower()
    end

    self:notifyObservers("onFabaoDataChanged", msg.item_id)
    self:notifyObservers("onFabaoRefineSuccessful", msg.item_id, msg.add)
end

-- 法宝升星
function This:onUpgradeFabaoStarRequest(fabaoUid, itemList)
    Socket:sendAndLock(10441,{item_id=fabaoUid, cost_item_list=itemList})
end
-- 法宝升星结果回复
function This:onReceiveFabaoUpgradeStarResultReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    local fabao = self:getFabao(msg.item_id)
    if fabao:isEquipped() then
        local hero = HeroManager:getHeroByUid(fabao:getHeroUid())
        hero:updateFabao(msg.item_id, msg.item_id)
        HeroManager:calcSaintFightPower()
    end

    self:notifyObservers("onFabaoDataChanged", msg.item_id)
end

-- 精炼石礼包购买
function This:onBuyRefineStonePackageRequest(index)
    Socket:sendAndLock(10445,{buy_index=index})
end
function This:onReceiveRefineStonePackageResultReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end
    self:notifyObservers("onBuyRefineStonePackageSuccessful")
end

-- 请求精炼石购买状况
function This:getRefineStonePackageStatus()
    Socket:sendAndLock(10446,{})
end
function This:onReceiveRefineStonePackageStatusReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end
    self:notifyObservers("onRefineStonePackageStatusChanged", msg.buy_info)
end