---
---
--- Created by aid.
--- DateTime: 2018/12/13 18:11
---
local This , super = class(...,"UI.HelpTips")
local HeroManager = require("Hero.HeroManager")
local ItemUtil = require("Bag.ItemUtil")
local FabaoSkillView = require("Fabao.FabaoSkillView")
local FabaoManager = require("Fabao.FabaoManager")
local GameUtil = require("Util.GameUtil")
local FabaoUtil = require("Fabao.FabaoUtil")
local SkillConfig = require("Metadatas.SkillConfig")
local FabaoConstants = require("Fabao.FabaoConstants")
local ItemNode = require("UI.ItemNode")

function This:initData(arguments)
    self.m_fabaoUid = arguments.fabaoUid
    self.m_heroUid = arguments.heroUid
    self.m_pos = arguments.pos
    self.m_itemId = arguments.itemId
end

local ChineseNumber = {
    Serialize[60012],Serialize[60013],Serialize[60014],Serialize[60015],Serialize[60016],Serialize[60097],Serialize[60018],Serialize[60019],Serialize[60020]
}

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureDetails.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    local fabao = FabaoManager:getFabao(self.m_fabaoUid)

    -- 法宝名
    self.m_csNode.fabaoname:setString(ItemUtil.getName(self.m_itemId))
    self.m_csNode.fabaoname:setColorByID("Q".. ItemUtil.getQuality(self.m_itemId))

    -- 法宝描述
    self.m_csNode.text1:setString(ItemUtil.getComment(self.m_itemId), 230/self.m_csNode.text1:getScale())

    -- 法宝图标
    local fabaoItemView = ItemNode:create(self.m_itemId, nil, nil, false)
    fabaoItemView:setAnchorPoint(0.5, 0.5)
    fabaoItemView:setToFabao(fabao:getLevel(), fabao:getJinglianDegree(), fabao:getStarLevel())
    self.m_csNode.fabao:addChild(fabaoItemView)

    -- 法宝技能
    local fabaoSkillView = FabaoSkillView:create()
    fabaoSkillView:setFabao(self.m_fabaoUid)
    self.m_csNode.teji:addChild(fabaoSkillView)

    local fabao = FabaoManager:getFabao(self.m_fabaoUid)

    -- 基础属性
    local levelAttributeList = fabao:getAdditionalAttributeListOnLevel()
    for index = 1, 3 do
        local attributeNameView = self.m_csNode["name"..index]
        local attributeValueView = self.m_csNode["num"..index]
        local attribute = levelAttributeList[index]
        if attribute then
            local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeNameStr)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end

    -- 精炼属性
    local jinglianDegreeAttributeList = fabao:getAdditionalAttributeListOnJinglianDegree()
    for index = 1, 3 do
        local attributeNameView = self.m_csNode["name"..(index+3)]
        local attributeValueView = self.m_csNode["num"..(index+3)]
        local attribute = jinglianDegreeAttributeList[index]
        if attribute then
            local attributeNameStr, attributeValueStr = GameUtil:getAttrStr(attribute[1], attribute[2])
            attributeNameView:setString(attributeNameStr)
            attributeValueView:setString(attributeValueStr)
            attributeNameView:setVisible(true)
            attributeValueView:setVisible(true)
        else
            attributeNameView:setVisible(false)
            attributeValueView:setVisible(false)
        end
    end

    -- 升星效果
    local currentStarLevel = fabao:getStarLevel()
    for index = 1, 5 do
        local color
        if index <= currentStarLevel then
            color = "<Q5>"
        else
            color = "<Study1>"
        end

        local formatText = Serialize[60708]
        local newEffectText
        local newEffect, effectType = FabaoUtil.getDifferentBetweenStarLevel(fabao:getItemId(), index-1, index)
        if effectType == FabaoUtil.Type.Attribute then
            local attributeName, attributeValueStr = GameUtil:getAttrStr(newEffect[1], newEffect[2])
            newEffectText = string.format("%s%s", attributeName, attributeValueStr)
        elseif effectType == FabaoUtil.Type.Skill then
            local skillConfig = SkillConfig[newEffect]
            newEffectText = string.format("%s%sLv.%d",Serialize[61255], skillConfig.name, newEffect % 10)
        else
            formatText = Serialize[60710]
            newEffectText = ""
        end

        local view = self.m_csNode[""..index.."xing"]
        view:setStringWithColor(color..string.format(
                formatText,
                ChineseNumber[index],
                FabaoUtil.getStarLevelAttributesEnhanciveFactor(fabao:getItemId(), index)*100, -- TODO 这里显示的是全量，而不是差异，是否要修改？
                newEffectText
        ))
    end

end

function This:initTouchEvent()
    -- 卸下按钮
    self.m_csNode.xiexia:setTouchEnded(function()
        HeroManager:onTakeOffFabaoRequest(self.m_heroUid, self.m_fabaoUid)
        self:close()
    end)
    -- 替换按钮
    self.m_csNode.tihuan:setTouchEnded(function()
        local panel = g_scene:openDialog("Equip.EquipChoosePanel",{pos = self.m_pos, type = ItemCommonType.kFaBao})
        panel:setSelectHandler(function(uid)
            HeroManager:onEquipFabaoRequest(self.m_heroUid, uid)
        end)
        self:close()
    end)
    -- 培养按钮
    self.m_csNode.peiyang:setTouchEnded(function()
        g_scene:openDialog("Fabao.FabaoCultivatePanel", {fabaoUid = self.m_fabaoUid, sourceType = FabaoConstants.SourceType.Hero})
        self:close()
    end)
end