---
---
--- Created by aid.
--- DateTime: 2018/12/24 20:36
---
local M = {}
package.loaded[...] = M

local MagicWeaponLevelConfig = require("Metadatas.MagicWeaponLevelConfig")
local MagicWeaponRefineConfig = require("Metadatas.MagicWeaponRefineConfig")
local MagicWeaponStartConfig = require("Metadatas.MagicWeaponStartConfig")
local GameUtil = require("Util.GameUtil")
local SkillConfig = require("Metadatas.SkillConfig")
local FabaoManager = require("Fabao.FabaoManager")
local ItemUtil = require("Bag.ItemUtil")

M.Type = {
    Attribute = 1,
    Skill = 2,
}

local function isInteger(number)
    return math.ceil(number) == number
end

local function calculateRealAttributeList(enhanciveFactor, originalAttributeList)
    local attributeList = {}
    for _, attribute in ipairs(originalAttributeList) do
        table.insert(attributeList, {attribute[1], attribute[2]})
    end
    local factor = enhanciveFactor+1
    for _, attribute in ipairs(attributeList) do
        local attributeValue = attribute[2]
        if isInteger(attributeValue) then
            attributeValue = math.floor(attributeValue*factor+0.5)
        else
            attributeValue = math.floor(attributeValue*factor*100+0.5)/100
        end
        attribute[2] = attributeValue
    end
    return attributeList
end

local function getOneNewAttribute(currentConfig, nextConfig)
    local currentAttributes = {}
    for _, attribute in ipairs(currentConfig.start_up_num) do
        currentAttributes[attribute[1]] = attribute[2]
    end
    for _, attribute in ipairs(nextConfig.start_up_num) do
        if not currentAttributes[attribute[1]] then
            return attribute
        end
    end
    return nil
end

local function getNewSkill(currentConfig, nextConfig)
    if currentConfig.skill == nextConfig.skill then
        return nil
    end

    if currentConfig.skill ~= 0 and nextConfig.kill ~= 0 then
        return nextConfig.skill
    else
        -- 原本没有技能的法宝在升星玩法中不会给它添加技能，
        -- 同样，一个原本有技能的法宝在升星玩法中也不会把它的技能删除
        error("error!")
    end
end

-- 受星级影响的等级加成属性，即实际生效的属性
function M.getLevelAttributeList(fabaoItemId, level, starLevel)
    local enhanciveFactor = M.getStarLevelAttributesEnhanciveFactor(fabaoItemId, starLevel)
    return calculateRealAttributeList(enhanciveFactor, MagicWeaponLevelConfig[fabaoItemId*1000+level].num)
end

-- 受星级影响的精炼程度加成属性，即实际生效的属性
function M.getJinglianAttributeList(fabaoItemId, jinglianDegree, starLevel)
    local enhanciveFactor = M.getStarLevelAttributesEnhanciveFactor(fabaoItemId, starLevel)
    return calculateRealAttributeList(enhanciveFactor, MagicWeaponRefineConfig[fabaoItemId*1000+jinglianDegree].num)
end

-- 某星级下生效的星级属性加成
function M.getStarLevelAttributeList(fabaoItemId, starLevel)
    local config = MagicWeaponStartConfig[fabaoItemId*1000+starLevel]
    return config.start_up_num
end

-- 装备该法宝给武将带来的所有真实属性加成
function M.getAllAttributes(fabaoItemId, level, jinglianDegree, starLevel)
    local attributes = {}
    for _, attributeList in ipairs({
            M.getLevelAttributeList(fabaoItemId, level, starLevel),
            M.getJinglianAttributeList(fabaoItemId, jinglianDegree, starLevel),
            M.getStarLevelAttributeList(fabaoItemId, starLevel),
        }) do
        for _, attribute in ipairs(attributeList) do
            if attributes[attribute[1]] then
                attributes[attribute[1]] = attributes[attribute[1]] + attribute[2]
            else
                attributes[attribute[1]] = attribute[2]
            end
        end
    end
    return attributes
end

-- 某星级下对比上一星级额外增加的属性或技能
function M.getDifferentBetweenStarLevel(fabaoItemId, starLevel, nextStarLevel)
    local currentConfig = MagicWeaponStartConfig[fabaoItemId*1000+starLevel]
    local nextConfig = MagicWeaponStartConfig[fabaoItemId*1000+nextStarLevel]
    if (not currentConfig) or (not nextConfig) then
        return nil
    end
    local newAttribute = getOneNewAttribute(currentConfig, nextConfig)
    if newAttribute then
        return newAttribute, M.Type.Attribute
    end
    local newSkill = getNewSkill(currentConfig, nextConfig)
    if newSkill then
        return newSkill, M.Type.Skill
    end
    return nil
end

-- 某星级下生效的全属性（不包括星级属性）加成百分比
function M.getStarLevelAttributesEnhanciveFactor(fabaoItemId, starLevel)
    local config = MagicWeaponStartConfig[fabaoItemId*1000+starLevel]
    return config.num
end

-- 法宝的最高等级
local maxLevels = {}
function M.getFabaoMaxLevel(fabaoItemId)
    if maxLevels[fabaoItemId] then
        return maxLevels[fabaoItemId]
    end

    local maxLevel = 0
    while MagicWeaponLevelConfig[fabaoItemId*1000+maxLevel+1] do
        maxLevel = maxLevel + 1
    end
    maxLevels[fabaoItemId] = maxLevel
    return maxLevel
end

-- 法宝的最高精炼程度
local maxRefineDegrees = {}
function M.getFabaoMaxRefineDegree(fabaoItemId)
    if maxRefineDegrees[fabaoItemId] then
        return maxRefineDegrees[fabaoItemId]
    end

    local maxRefineDegree = 0
    while MagicWeaponRefineConfig[fabaoItemId*1000+maxRefineDegree+1] do
        maxRefineDegree = maxRefineDegree + 1
    end
    maxRefineDegrees[fabaoItemId] = maxRefineDegree
    return maxRefineDegree
end

-- 法宝的最高星级
local maxStarLevels = {}
function M.getFabaoMaxStarLevel(fabaoItemId)
    if maxStarLevels[fabaoItemId] then
        return maxStarLevels[fabaoItemId]
    end

    local maxStarLevel = 0
    while MagicWeaponStartConfig[fabaoItemId*1000+maxStarLevel+1] do
        maxStarLevel = maxStarLevel + 1
    end
    maxStarLevels[fabaoItemId] = maxStarLevel
    return maxStarLevel
end

-- 判断法宝在某星级下是否有技能
function M.hasSkillOnFabao(fabaoItemId, starLevel)
    return M.getFabaoSkillId(fabaoItemId, starLevel) ~= 0
end

-- 获得法宝在某星级下激活的技能
function M.getFabaoSkillId(fabaoItemId, starLevel)
    local skillConfig = MagicWeaponStartConfig[fabaoItemId*1000+starLevel]
    return skillConfig.skill
end

-- 法宝培养属性变化动画
function M.showAnimeOn(view)
    local effectRootView = view
    local tag = 1546139351
    local effectView = effectRootView:getChildByTag(tag)
    if effectView then
        effectView:removeFromParent()
    end
    effectView = GameUtil:createEffect("ShuXing")
    effectView:setScale(0.5)
    effectView:setTag(tag)
    effectRootView:addChild(effectView)
end

local ChineseHanzi = {Serialize[60022], Serialize[60012], Serialize[60013], Serialize[60014], Serialize[60015], Serialize[60016], Serialize[60097], Serialize[60018], Serialize[60019], Serialize[60020],}
local function toHanzi(number)
    if number < 0 or number > 99 then
        error("error!") -- 不支持三位以上的数字，也不支持负数
    end
    if number < 10 then
        return ChineseHanzi[number+1]
    else
        if number%10 ~= 0 then
            return string.format(
                    Serialize[60912],
                    ChineseHanzi[math.floor(number/10)+1],
                    ChineseHanzi[(number%10)+1]
            )
        else
            return string.format(
                    Serialize[60913],
                    ChineseHanzi[math.floor(number/10)+1]
            )
        end
    end
end

function M.getSkillName(skillId)
    return SkillConfig[skillId].name
end

function M.getSkillDescriptionList(fabaoItemId, skillId, hideActivationState)
    -- 法宝技能需要的最低星级
    local minimalStarLevelRequests = {}
    local starLevel = 0
    while true do
        local starConfig = MagicWeaponStartConfig[fabaoItemId*1000+starLevel]
        if not starConfig then break end
        --
        local minimalStarLevel = minimalStarLevelRequests[starConfig.skill]
        if minimalStarLevel then
            if starLevel < minimalStarLevel then
                minimalStarLevel = starLevel
            end
        else
            minimalStarLevel = starLevel
        end
        minimalStarLevelRequests[starConfig.skill] = minimalStarLevel
        --
        starLevel = starLevel + 1
    end

    -- 技能的各级描述
    local skillDescriptionList = {}
    local highlightIndex = 0
    local skillIdBase = math.floor(skillId/10)*10
    for i = 1, 6 do -- 技能1-6级
        local statusText = ""
        if not hideActivationState then
            if skillIdBase+i <= skillId then -- 已激活
                statusText = Serialize[60914]
                highlightIndex = i
            else -- 未激活
                local necessaryMinimalStarLevel = minimalStarLevelRequests[skillIdBase+i]
                statusText = string.format(Serialize[60915], toHanzi(necessaryMinimalStarLevel))
            end
        end

        table.insert(skillDescriptionList, string.format(
                "Lv.%d：%s%s",
                i, -- 技能等级
                SkillConfig[skillIdBase+i].desp, -- 技能描述
                statusText -- 激活状态
        ))
    end
    return skillDescriptionList, highlightIndex
end

function M.getFabaoRebornConfig(fabaoItemId, level, jinglianDegree, starLevel)
    local list = {}
    if level and level > 0 then
        local config = MagicWeaponLevelConfig[fabaoItemId*1000+level]
        table.insert(list, config.return_item)
    end
    if jinglianDegree and jinglianDegree > 0 then
        local config = MagicWeaponRefineConfig[fabaoItemId*1000+jinglianDegree]
        table.insert(list, config.return_item)
    end
    if starLevel and starLevel > 0 then
        local config = MagicWeaponStartConfig[fabaoItemId*1000+starLevel]
        table.insert(list, config.return_item)
    end
    return list
end

function M.fabaoSortFunc(left, right)
    local leftItemUid = left:getUid()
    local rightItemUid = right:getUid()
    local leftItemId = left:getId()
    local rightItemId = right:getId()
    local leftItemQuality = ItemUtil.getQuality(leftItemId)
    local rightItemQuality = ItemUtil.getQuality(rightItemId)
    -- 高品质在前
    if leftItemQuality > rightItemQuality then
        return true
    elseif leftItemQuality < rightItemQuality then
        return false
    end
    -- 高星级法宝在前
    local leftFabao = FabaoManager:getFabao(leftItemUid)
    local rightFabao = FabaoManager:getFabao(rightItemUid)
    if leftFabao:getStarLevel() > rightFabao:getStarLevel() then
        return true
    elseif leftFabao:getStarLevel() < rightFabao:getStarLevel() then
        return false
    end
    -- 道具id小的在前
    if leftItemId < rightItemId then
        return true
    else
        return false
    end
end

function M.equipmentAndFabaoSortFunc(left, right)
    local leftItemUid = left:getUid()
    local rightItemUid = right:getUid()
    local leftItemId = left:getId()
    local rightItemId = right:getId()
    local leftItemType = ItemUtil.getType(leftItemId)
    local rightItemType = ItemUtil.getType(rightItemId)
    local leftItemQuality = ItemUtil.getQuality(leftItemId)
    local rightItemQuality = ItemUtil.getQuality(rightItemId)

    -- 高品质在前
    if leftItemQuality > rightItemQuality then
        return true
    elseif leftItemQuality < rightItemQuality then
        return false
    end
    -- 法宝在前, 法宝的类型为7，装备的类型为2
    if leftItemType > rightItemType then
        return true
    elseif leftItemType < rightItemType then
        return false
    end
    -- 同类型道具根据它们所属的类型分别处理
    if leftItemType == ItemCommonType.kFaBao then
        return M.fabaoSortFunc(left, right)
    elseif leftItemType == ItemCommonType.kEquip then
        local leftStrengthLevel = left:getStrengthLv()
        local rightStrengthLevel = right:getStrengthLv()
        -- 强化等级高的在前
        if leftStrengthLevel > rightStrengthLevel then
            return true
        elseif leftStrengthLevel < rightStrengthLevel then
            return false
        end
        -- 道具id小的在前
        if leftItemId < rightItemId then
            return true
        else
            return false
        end
        -- uid小的在前
        if leftItemUid < rightItemUid then
            return true
        else
            return false
        end
    else
        -- 不支持的排序类型
        error("error!")
    end
end



