---
---
--- Created by aid.
--- DateTime: 2018/12/15 11:15
---
local This , super = class(...,"cc.Node")
local ItemUtil = require("Bag.ItemUtil")
local FabaoManager = require("Fabao.FabaoManager")
local HeroManager = require("Hero.HeroManager")
local FabaoSkillView = require("Fabao.FabaoSkillView")
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")
local ItemNode = require("UI.ItemNode")

function This:init(index, data, isSelected)
    self:initData(index, data, isSelected)
    self:initNode()
    self:initTouchEvent()
end

function This:initData(index, data, isSelected)
    self.m_index = index
    self.m_hero = data
    self.m_isSelected = isSelected
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureHero.csb")
    self:addChild(self.m_csNode)
    self:refreshNode()
end

function This:initTouchEvent()
    --
    self.m_csNode.bg0:setTouch({
        onEnded = function()
            if self.m_hero:isWearFabao(FabaoPos.kFirst) then
                self.m_csNode.jianhao:setVisible(true)
            end

            if self.m_onClickListener then
                self.m_onClickListener(self.m_index, self)
            end
        end,
        isContainer = true,
        isScale = false
    })

    -- 法宝卸载按钮
    self.m_csNode.jianhao:setTouch({
        onEnded = function()
            HeroManager:onTakeOffFabaoRequest(self.m_hero:getUid(), self.m_hero:getFabaoUid(FabaoPos.kFirst))
        end,
        isContainer = true,
        isScale = true
    })

    -- 法宝装备按钮
    self.m_csNode.fabao:setTouch({
        onEnded = function()
            if self.m_hero:isWearFabao(FabaoPos.kFirst) then return end
            local neededPlayerLevel = ArgConfig.fabao_open_level.arg[1]
		    if GamePlayer:getLevel() < neededPlayerLevel then
                g_scene:showTips(tostring(neededPlayerLevel) .. Serialize[60917])
                return
            end
            local panel = g_scene:openDialog("Equip.EquipChoosePanel",{pos = FabaoPos.kFirst, type = ItemCommonType.kFaBao})
            panel:setSelectHandler(function(uid)
                HeroManager:onEquipFabaoRequest(self.m_hero:getUid(), uid)
            end)
        end,
        isContainer = true,
        isScale = false,
    })
end

function This:onEnter()
    super.onEnter(self)
    -- HeroManager:addObserver(self)
    -- FabaoManager:addObserver(self)
end

function This:onExit()
    -- FabaoManager:removeObserver(self)
    -- HeroManager:removeObserver(self)
    super.onExit(self)
end

function This:setOnClickListener(listener)
    self.m_onClickListener = listener
end

function This:getContentSize()
    return self.m_csNode.bg:getContentSize()
end

function This:refreshFabao()
    if self.m_hero:isWearFabao(FabaoPos.kFirst) then
        local fabao = FabaoManager:getFabao(self.m_hero:getFabaoUid(FabaoPos.kFirst))
        --local data = {
        --    itemId = fabao:asItem():getId(),
        --    jinglianDegree = fabao:getJinglianDegree(),
        --    level = fabao:getLevel(),
        --    starCount = fabao:getStarLevel(),
        --}
        self.m_fabaoItemView:initItem(fabao:asItem():getId())
        self.m_fabaoItemView:setToFabao(fabao:getLevel(), fabao:getJinglianDegree(), fabao:getStarLevel())
        self.m_fabaoItemView:setVisible(true)
        self.m_fabaoSkillView:setFabao(fabao:getUid())
        self.m_fabaoSkillView:setVisible(true)
        self.m_csNode.text_0:setVisible(true)
        self.m_csNode.fabao:setVisible(false)
        if self.m_isSelected then
            self.m_csNode.jianhao:setVisible(true)
        else
            self.m_csNode.jianhao:setVisible(false)
        end
        self.m_csNode.fabaoIcon:setVisible(true)
    else
        self.m_fabaoItemView:setVisible(false)
        self.m_fabaoSkillView:setVisible(false)
        self.m_csNode.text_0:setVisible(false)
        self.m_csNode.fabao:setVisible(true)
        self.m_csNode.jianhao:setVisible(false)
        self.m_csNode.fabaoIcon:setVisible(false)
    end
end

function This:refreshNode()
    self.m_csNode.name:setString(self.m_hero:getName()) -- 武将名
    self.m_csNode.icon:setImage(ItemUtil.getIconImage(self.m_hero:getId())) -- 武将头像

    -- 法宝图标
    local fabaoRootNode = self.m_csNode.fabaoIcon
    local tag = 5798299998;
    self.m_fabaoItemView = fabaoRootNode:getChildByTag(tag)
    if not self.m_fabaoItemView then
        self.m_fabaoItemView = ItemNode:create(101, nil, nil, false)
        self.m_fabaoItemView:setAnchorPoint(0.5,0.5)
        self.m_fabaoItemView:setTag(tag)
        fabaoRootNode:addChild(self.m_fabaoItemView)
    end

    -- 法宝技能
    local skillNode = self.m_csNode.Node_1
    local tag = 1545745411
    self.m_fabaoSkillView = skillNode:getChildByTag(tag)
    if not self.m_fabaoSkillView then
        self.m_fabaoSkillView = FabaoSkillView:create()
        self.m_fabaoSkillView:setTag(tag)
        skillNode:addChild(self.m_fabaoSkillView)
    end

    --if self.m_hero:isWearFabao(FabaoPos.kFirst) and self.m_isSelected then
    --    self.m_csNode.jianhao:setVisible(true)
    --else
    --    self.m_csNode.jianhao:setVisible(false)
    --end

    self:refreshFabao()
end

function This:setHeroData(index, heroData, isSelected)
    self.m_index = index
    self.m_hero = heroData
    self.m_isSelected = isSelected
    self:refreshNode()
end

-- function This:onEquipChanged(heroId)
--     if self.m_hero:getId() ~= heroId then
--         return
--     end
--     self:refreshFabao()
--     if self.m_hero:isWearFabao(FabaoPos.kFirst) and self.m_onClickListener then
--         self.m_onClickListener(self.m_index, self)
--     end
-- end

-- function This:onFabaoDataChanged(fabaoUid)
--     if self.m_hero:isWearFabao(FabaoPos.kFirst) then
--         local fabao = FabaoManager:getFabao(self.m_hero:getFabaoUid(FabaoPos.kFirst))
--         if fabao:getUid() == fabaoUid then
--             self:refreshFabao()
--         end
--     end
-- end

function This:setToBeSelected(bool)
    self.m_isSelected = bool
    self:refreshNode()
end