-- @Author: zheng
-- @Date:   2018-09-10 16:10:44
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-28 17:17:41
-- @File Name: FamiltApplyListView.lua

local This , super = class(...,"NodeUI")
local GameUtil = require("Util.GameUtil")
local FamilyManager = require("Family.FamilyManager")
-- local RankListLayer = require("Rank.RankListLayer")
local FamilyApplyItemView = require("Family.FamilyApplyItemView")
local ChatManager = require("Chat.ChatManager")
local GamePlayer = require("Player.GamePlayer")
This.fileName = "MenuScene/NodeUI/FamiltApplyList.csb"

function This:initNode(info)
    if info then
       self.m_family = info.family
    end
    if self.m_family then
    	self.m_family:requestRefreshInfo()
    end
    self:setPosition(cc.visible.center.x, cc.visible.center.y)
    self.m_isSend = false
end

function This:initTouchEvent()
	self.apply.request:setTouchEnded(function()
        g_scene:openDialog("Family.FamilyJoinRequestPanel", {family = self.m_family})
    end)
    self.noapply.request:setTouchEnded(function()
        g_scene:openDialog("Family.FamilyJoinRequestPanel", {family = self.m_family})
    end)
    self.Invitation.request:setTouchEnded(function()
        if os.time() - self:getLastSendTime() >= kFamilyInviteSendApplyCD then
			ChatManager:onApplyJoinGroupRequest(self.m_family:getId(), self.m_family:getName())
			self:setLastSendTime()
			self:scheduleUpdate()
			self.Invitation.request:setGrayShader()
		end
    end)
    self.dissolution.request:setTouchEnded(function()
        self:leftFamily(Serialize["family_InfoTab_5"])
    end)
    self.inherit.request:setTouchEnded(function()
        self.m_isSend = true
        self:modifySendBtn(true,self.m_isSend)
        FamilyManager:onGetSendInheritRequest()
    end)
  
    self.quit.request:setTouchEnded(function()
        self:leftFamily(Serialize["family_InfoTab_4"],true)
    end)

end

function This:leftFamily(tipStr,isQuit)
    if not self.m_family then
        return
    end
    local isMengzhuQuit = self:getSelfInfor()
    if isMengzhuQuit and isQuit then
        tipStr= "          "..tipStr..Serialize[60923]
    end
    local selfMember = self.m_family:getSelfMemberInfo()
    if selfMember then
        g_scene:openDialog("Stage.ChallengeConfirmPanel", {
            simple = tipStr, 
            okHandler = function()
                if isMengzhuQuit and not isQuit then
                    FamilyManager:onMengZhuQuitRequest(GamePlayer:getPlayerID())
                else
                    self.m_family:requestLeft()
                end
            end 
        })
    else
        -- 首次离开失败以后, 直接请求离开
        self.m_family:requestLeft()
    end
end

function This:setLastSendTime()
    FamilyManager:setInviteTimes(os.time())
end

function This:getLastSendTime()
    local lastTime = FamilyManager:getInviteTimes() or 0
	return lastTime
end

function This:refreshInvite()
	local escape = os.time() - self:getLastSendTime()
	if escape < kFamilyInviteSendApplyCD then
        self.Invitation.request:setGrayShader()
        self.Invitation.request.text:setString(string.format(Serialize["family_InfoTab_6_c"], math.ceil(kFamilyInviteSendApplyCD - escape)))
    else
        self:unscheduleUpdate()
        self.Invitation.request.text:setString(Serialize["family_InfoTab_10"])
        self.Invitation.request:clearShader()
    end
end

function This:refresh()
	self:switchTab(self.m_adminisTabName)
	self:refreshScroolView()
	local selfMember = self.m_family:getSelfMemberInfo()
	if selfMember then
		local dismissVisible = selfMember:hasPower(FamilyPower.kDismiss)
		local applyVisible = selfMember:hasPower(FamilyPower.kAgreeApplying)
		self.apply.request:setVisible(dismissVisible or applyVisible)
		self.noapply.request:setVisible(dismissVisible or applyVisible)
		self.Invitation.request:setVisible(dismissVisible or applyVisible)
        self.dissolution.request:setVisible(dismissVisible)
	end
	self:refreshInvite()
end

function This:update(dt)
    super.update(self,dt)
    local escape = os.time() - self:getLastSendTime() 
    self:refreshInvite()
end
function This:onEnter()
    super.onEnter(self)
    FamilyManager:addObserver(self)
    if os.time() - self:getLastSendTime() <= kFamilyInviteSendApplyCD then
		self:scheduleUpdate()
	end
end

function This:onExit()
    super.onExit(self)
    FamilyManager:removeObserver(self)
end

function This:setType(adminisTabName)
	self.m_adminisTabName = adminisTabName
	self:refresh()
end

function This:switchTab(adminisTabName)
	local hideNamelist = {"apply","noapply","Invitation","inherit","dissolution","quit"}
	GameUtil:hideNodeList(self, hideNamelist)

	if adminisTabName == FamilyAdminisTabName.ktep1 then
		local requestList = self.m_family:getAllRequestInfo()
		if #requestList ==0 then
			self.noapply:setVisible(true)
		else
			self.apply:setVisible(true)
		end
	elseif adminisTabName == FamilyAdminisTabName.ktep2 then
		self.Invitation:setVisible(true)
	elseif adminisTabName == FamilyAdminisTabName.ktep3 then
		self.inherit:setVisible(true)
        local isShow = self:getisShowSender()
        self:modifySendBtn(isShow,self.m_isSend)
	elseif adminisTabName == FamilyAdminisTabName.ktep4 then
		self.dissolution:setVisible(true)
	elseif adminisTabName == FamilyAdminisTabName.ktep5 then
		self.quit:setVisible(true)
	end
end

function This:modifySendBtn(showSender,isSender)
	self.inherit.request:setVisible(false)
	self.inherit.yifasong:setVisible(false)
	if showSender then
		if isSender then
			self.inherit.yifasong:setVisible(true)
		else
			self.inherit.request:setVisible(true)
		end
	end
end

function This:getSelfInfor()
    local selfMemberInfo = self.m_family:getSelfMemberInfo()
    return selfMemberInfo:getTitle() == FamilyTitle.kMengZhu ,selfMemberInfo
end

function This:getisShowSender()
    local isMengzhu,selfMemberInfo = self:getSelfInfor()
    if isMengzhu then
        return false
    end

    local lastLineTime = 0
    local mengZhu = self.m_family:getLeaderInfo()
    if mengZhu then
        lastLineTime = mengZhu:getLaseOnlineTime()
    end

    if lastLineTime == 0 then
        return true
    elseif lastLineTime > 0 then
        local serTime = GamePlayer:getServerTime()
        local leftTime = serTime - lastLineTime
        if leftTime > kFamilyHeadOfflineTime2 then
            return true
        elseif leftTime > kFamilyHeadOfflineTime and selfMemberInfo:hasPower(FamilyPower.kInherit) then
            return true
        end
    end
    return false
end

function This:refreshScroolView()
    local scrollView = self.apply.scrollView

    scrollView:removeAllItems()
    local dataList = self.m_family:getAllRequestInfo()
    for i,v in ipairs(dataList) do
        local view = self:getScroolItem(v,i)
        local w , h = view:getContentSize()
        scrollView:pushBackCustomItem(view,w,h,0.5)
    end
end

function This:getScroolItem(itemData,index)
    local view = FamilyApplyItemView:create(itemData,index)
    return view
end

function This:onGetMyFamilyInfo(family)
    self:setTimer(0.3, function ()
        self:refresh()
    end, "refresh_info")
end

function This:onApplyApproval()
    self:refreshScroolView()
end

function This:onGetSendInherit()
    -- self.m_isSend = false
    self:refresh()
end
