-- @Author: ZYY
-- @Date:   2018-07-04 16:45:31
-- @Last Modified time: 2019-01-30 11:18:42
-- @Desc: 军团内务

local This , super = class(...,"NodeUI")
local FamilyMemberItemView = require("Family.FamilyMemberItemView")
local FamilyManager = require("Family.FamilyManager")
local GameUtil = require("Util.GameUtil")
local FamilyModel = require("Family.FamilyModel")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")
local LegionAbossManager = require("LegionAboss.LegionAbossManager")
local FamilyBossManager = require("FamilyBoss.FamilyBossManager")


This.fileName = "MenuScene/NodeUI/FamilyAffairs.csb"

function This:initNode(info)
    if info then
	   self.m_family = info.family
    end

	-- 分红红点
	self.m_bonusRedDot = GameUtil:createRedDotTo(self.fenhong)
	local redDotWidth, redDotHeight = self.m_bonusRedDot:getContentSize()
	local width, height = self.fenhong.fenhongbg:getContentSize()
	self.m_bonusRedDot:setPosition(width/2-redDotWidth/2, height/2-redDotHeight/2)
	self.m_bonusRedDot:setVisible(false)

	-- boss红点
	self.m_bossRedDot = GameUtil:createRedDotTo(self.boss)
	local redDotWidth, redDotHeight = self.m_bossRedDot:getContentSize()
	local width, height = self.boss.bossbg:getContentSize()
	self.m_bossRedDot:setPosition(width/2-redDotWidth/2, height/2-redDotHeight/2)
	self.m_bossRedDot:setVisible(false)

	-- 军团活跃boss
	self.m_legionAbossRedDot = GameUtil:createRedDotTo(self.zhaiyao)
	local redDotWidth, redDotHeight = self.m_legionAbossRedDot:getContentSize()
	local width, height = self.zhaiyao.zhaiyaobg:getContentSize()
	self.m_legionAbossRedDot:setPosition(width/2-redDotWidth/2, height/2-redDotHeight/2)
	self.m_legionAbossRedDot:setVisible(false)	


	self:refreshLock()
end

function This:initTouchEvent()
    self.fenhong:setTouch({
    	onEnded = function()
            if not self.m_family then return end
            local is_normal = FamilyBuildManager:getBuildByType(FamilyBuildType.kYiShiTing):isNormal()
            if not is_normal then
            	g_scene:showTips(Serialize["family_AffairsNode_3"])
            	return false
        	end
    		local bonus_state = self.m_family:getBonusState()
    		if not bonus_state then 
	    		g_scene:openDialog("Family.FamilyBonusView", self.m_family)
	    	else
	    		g_scene:showTips(Serialize["family_AffairsNode_2"])
	    	end
		end,
		sizeNode = self.fenhong.fenhongbg,
		-- isScale = false,
		isAlpha = false
	})

    self.boss:setTouch({
    	onEnded = function()
			local chapter = FamilyBossManager:getChapter()
			if not chapter:isActive() then
				g_scene:showTips( string.format(Serialize[60930], chapter:getOpenWeekStr(), chapter:getOpenTimeStr()) )
				return
			end
			g_scene:openDialog("FamilyBoss.FamilyBossPanel")
		end,
		sizeNode = self.boss.bossbg,
		isAlpha = false
	})

	self.emal:setTouch({
    	onEnded = function()
            if not self.m_family then return end 
    		local selfMember = self.m_family:getSelfMemberInfo()
    		if selfMember then
    			local dismissVisible = selfMember:hasPower(FamilyPower.kSendMail)
    			if dismissVisible then
    				g_scene:openDialog("Family.FamilyMailPanel")
    			else
    				g_scene:showTips(Serialize["family_AffairsNode_1"])
    			end
    		end
		end,
		sizeNode = self.emal.emailbg,
		-- isScale = false,
		isAlpha = false
	})

	self.zhaiyao:setTouch({
    	onEnded = function()
            if not self.m_family then return end
            if LegionAbossManager:getDataModel():checkUnlock(true) then
            	g_scene:openDialog("LegionAboss.LegionAbossPanel")
            end
		end,
		sizeNode = self.zhaiyao.zhaiyaobg,
		-- isScale = false,
		isAlpha = false
	})
	
	self.hongbao:setTouch({
    	onEnded = function()
            if not self.m_family then return end 
            g_scene:showTips(Serialize["family_BuildNode_1_c"])
		end,
		sizeNode = self.hongbao.hongbaobg,
		-- isScale = false,
		isAlpha = false
	})
end

function This:onEnter()
	super.onEnter(self)
	FamilyManager:addObserver(self)

	self.m_signalHandlers = {
		[Signals.FamilyPanelAffairsTabRedDot] = function(data)
			self.m_bonusRedDot:setVisible(g_redDot:checkFamilyAffairsBonus())
			self.m_bossRedDot:setVisible(g_redDot:checkFamilyAffairsBoss())
			self.m_legionAbossRedDot:setVisible(g_redDot:checkLegionAboss())
		end,
	}
	SignalSystem.addSignalHandlers(self.m_signalHandlers)
	SignalSystem.distributeSignal(Signals.FamilyPanelAffairsTabRedDot)
end

function This:onExit()
	SignalSystem.removeSignalHandlers(self.m_signalHandlers)
	super.onExit(self)
	FamilyManager:removeObserver(self)
end

function This:onGetMyFamilyInfo(family)
	self.m_family = family
end

function This:refreshLock()
	local isUnlock = LegionAbossManager:getDataModel():checkUnlock(false)
	self.zhaiyao.jiesuo:setVisible(not isUnlock)
end
