-- @Author: ZYY
-- @Date:   2018-07-05 17:37:39
-- @Last Modified time: 2018-07-14 14:22:23
-- @Desc: 军团分红人物列表
local This, super = class(...,"Dialog")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local UpdateLayer = require("UI.UpdateLayer")
local FamilyUtil = require("Family.FamilyUtil")
local LegionShareConfig = require("Metadatas.LegionShareConfig")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local FamilyBonusListItemView = require("Family.FamilyBonusListItemView")
local FamilyMemberModel = require("Family.FamilyMemberModel")
local FamilyManager = require("Family.FamilyManager")
local FamilyPowerConfig = require("Metadatas.FamilyPowerConfig")

function This:initNode(family)
	self.m_selfFamily = family
	self.role_list = {}
	self.reward = {}
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyBonus.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
	self.m_listLayer = UpdateLayer:create({
        itemWidth = 388,
        itemHeight = 80,
        spaceX = 0,
        spaceY = 0,
        scrollView = self.m_csNode.ScrollView, 
    })
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.m_csNode.ScrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)
	self:refresh()
end

function This:refresh()
	self.yst_level = FamilyBuildManager:getBuildingLevel(FamilyBuildType.kYiShiTing)
	if not self.yst_level or self.yst_level == 0 then 
		self.yst_level = 1
	end
	self:updateRoleReward()
	self:updateMyReward()
end

function This:initTouchEvent()
    self.m_csNode.beijing.closebutton:setTouchEnded(function()
    	self:close()
    end)
    
    self.m_csNode.receive.button:setTouchEnded(function()
		local is_normal = FamilyBuildManager:getBuildByType(FamilyBuildType.kYiShiTing):isNormal()
		if is_normal then 
	    	FamilyManager:onFamilyBonusRequest()
	    else
	    	g_scene:showTips(Serialize["family_BonusView_2"])
	    end
    end)

    self.m_csNode.beijing.wenhao:setTouchEnded(function()
    	g_scene:openDialog("UI.HelpTips", {text = Serialize["family_BonusView_1"] ,node = self.m_csNode.beijing.wenhao})
    end)
end

function This:updateRoleReward()
	local data = {}
    local titleList = {}
    for _, cfg in ipairs(FamilyPowerConfig) do
        titleList[cfg.level] = cfg.sequence
    end

	for i , v in pairs(LegionShareConfig) do 
		local level = math.floor(v.level / 100)
		if level == self.yst_level then 
			table.insert(data,v)
		end
	end
	local function sort_fun(a,b)
		local levelA, levelB = a.level, b.level
		local posA, posB = levelA % 100, levelB % 100
		local posLevA, posLevB = titleList[posA], titleList[posB]
		return posLevA < posLevB
	end

	table.sort(data,sort_fun)
	self.m_listLayer:setDataList(data, function(member, index)
		local item = FamilyBonusListItemView:create(member, index)
		return item
	end)
end

function This:updateMyReward()
	local my_level = self.yst_level * 100 + self.m_selfFamily:getSelfMemberInfo():getTitle()
	if not LegionShareConfig[my_level] then return end 
	for i , v in ipairs(self.reward) do 
		v:setVisible(false)
	end
	self.m_config = LegionShareConfig[my_level]["office"]

	for i, info in ipairs(self.m_config) do
		local item_id = info[1]
		local item_num = info[2]
		local item_str = "item" .. i 
		local node = self.reward[i]
	    if not node then
	        node = ItemNode:create(item_id, item_num)
	        node:setAnchorPoint(0.5, 0.5)
	        node:setScale(0.8)
	        self.m_csNode["receive"][item_str]:addChild(node)
	    else
	        node:init(item_id, item_num)
	    end	
	    node:setVisible(true)
	    self.reward[i] = node
	end
end

function This:onReceiveBonus()
	self.m_csNode.receive.button:setGrayShader()
	self.m_csNode.receive.button.lingqu:setString(Serialize["family_BonusView_3"])
end

function This:onFamilyBonus()
	local rewardList = {}
	if self.m_config then
		g_scene:showReward(self.m_config)
	end	

	self:close()
end

function This:onEnter()
	super.onEnter(self)
	FamilyManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	FamilyManager:removeObserver(self)	
end

