-- @Author: superm
-- @Date:   2018-07-03 14:44:46
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-14 16:09:34
-- @File Name: FamilyBuildNode.lua
-- @Description: 

local This , super = class(...,"NodeView")
local TerrTabGroupBehaviour = require("Territory.TerrTabGroupBehaviour")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")

local function getBuildCondDesc(targetBuild, breakMode)
    local conds = targetBuild:getConditions()
    local condMap = {}
    if conds then
        for i, cond in ipairs(conds) do
            local key, value = cond[1], cond[2]
            if not condMap[value] then
                condMap[value] = {}
            end
            table.insert(condMap[value], key)
        end
    end

    local desc
    for level, idsList in pairs(condMap) do
        local count, suitable, name = #idsList, true, nil
        for i=1, count do
            local build = FamilyBuildManager:getBuildByType(idsList[i])
            if build:getMaxLevel() <= 0 then
                count = count - 1
            else
                local buildLev = build:getLevel()
                if not name then
                    if buildLev >= level then
                        if not breakMode then name = "<A1>" .. build:getName() end
                    else
                        suitable = false
                        name = breakMode and build:getName() or ("<R0>" .. build:getName())
                    end
                else
                    if buildLev >= level then
                        if not breakMode then name = name .. "、<A1>" .. build:getName() end
                    else
                        suitable = false
                        name = name .. "、<R0>" .. build:getName()
                    end
                end
            end
            if not suitable and breakMode then break end
        end

        if name then
            local text
            if not breakMode then
                local textFormat = count > 1 and Serialize[60939] or Serialize[60940]
                text = string.format(textFormat, name, suitable and "<A1>" or "<R0>", level)
            else
                text = string.format(Serialize[60941], name, level)
            end
            desc = desc and (desc .. "，" .. text) or text
        end
    end
    return desc or Serialize[60243]
end

function This:initData(info)
    if info then
	   self.m_family = info.family
    end
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyBuild.csb")
    self:addChild(self.m_csNode)
end

function This:initNode()
    local csNode = self.m_csNode

    self.m_redDotNodes = {}

    self.m_groupBuild = TerrTabGroupBehaviour:create()
    for _, bType in pairs(FamilyBuildType) do
        local node = csNode.build["build" .. bType]
        local redDot = GameUtil:createRedDotTo(node.bg)
        self.m_redDotNodes[bType] = redDot

        local build = FamilyBuildManager:getBuildByType(bType)
        if build then
            local name = build:getName()
            if name then
                node.name:setString(name)
            end
            node.level:setString(string.format("Lv.%d", build:getLevel()))
        	self.m_groupBuild:addTabButton(bType, node, function()
                if build:getMaxLevel() <= 0 then
                    g_scene:showTips(Serialize["family_BuildNode_1_c"])
                    return false  
                end
                self.m_build = build
                self:selectTabByType(bType)
            end, false)
        end
    end
    self.m_groupBuild:selectTabButton(FamilyBuildType.kYiShiTing)
end

function This:initTouchEvent()
    if not self.m_inited then
        local csNode = self.m_csNode
        csNode.spend.button:setTouch({
            onEnded = function()
                self:onUpgradeBtnClick()
            end,
        })    
        self.m_inited = true
    end
end

function This:loadBuildUpEffect(buildNode, posx, posy)
    local buildId = self.m_build:getId()
    local level = self.m_build:getLevel()
    local effectBuildUp = GameUtil:createEffect("BuildUp", false, self)
    effectBuildUp:setPosition(posx, posy)
    buildNode:addChild(effectBuildUp)             

    self:setTimer(1.5, function()
        effectBuildUp:removeFromParent()
    end, "buildUp" .. tostring(buildId) .. "Lev" .. level)
    cc.Audio:playEffect("building.mp3")
end

function This:onEnter()
    super.onEnter(self)
    FamilyBuildManager:addObserver(self)

    self.m_signalHandlers = {}
    self.m_signalHandlers[Signals.FamilyPanelBuildTabRedDot] = function(data)
        for _, buildType in pairs(FamilyBuildType) do
            local building = FamilyBuildManager:getBuildByType(buildType)
            local redDotNode = self.m_redDotNodes[buildType]
            redDotNode:setVisible(g_redDot:checkFamilyBuilding(building))
        end
    end
    SignalSystem.addSignalHandlers(self.m_signalHandlers)
    SignalSystem.distributeSignal(Signals.FamilyPanelBuildTabRedDot)
end

function This:onExit()
    SignalSystem.removeSignalHandlers(self.m_signalHandlers)
    FamilyBuildManager:removeObserver(self)
    super.onExit(self)
end

function This:selectTabByType( buildType )
    self:refresh()
end

function This:refresh()
    local csNode = self.m_csNode
    local infoNode = csNode.info
    local costNode = infoNode.mcost
    local level, bType = self.m_build:getLevel(), self.m_build:getType()
    local maxLevel = self.m_build:getMaxLevel()
    self.m_build:updateUpgradeState()
    self.m_csNode.build.build4.jiesuo:setVisible(false)


    local node = csNode.build["build" .. bType]
    node.level:setString(string.format("Lv.%d", self.m_build:getLevel()))
    local costId, costNum = self.m_build:getMaintainceCost()
    if costId then
        costNode:setVisible(true)
        costNode:setString(tostring(costNum) .. Serialize[60942])
    else
        costNode:setVisible(false)
    end
    infoNode.desc:setStringWithColor(self.m_build:getDescription())

    local button = csNode.spend.button
    local visible = level < maxLevel

    if visible then
        infoNode.cond:setVisible(true)
        if level > 0 then
            button.text:setString(Serialize["family_BuildNode_3"])
        else
            button.text:setString(Serialize["family_BuildNode_4"])
        end 

        infoNode.cond:setStringWithColor(getBuildCondDesc(self.m_build))
        local upcostId, upcostNum = self.m_build:getUpgradeCost()
        if self.m_build:isMaintaince() then
            upcostId, upcostNum = self.m_build:getMaintainceCost()
            button.text:setString(Serialize[60943])
        end
        self.m_upcostNum = upcostNum
        if upcostId then
            local icon = ItemUtil.getIconImage(upcostId)
            csNode.item1.item:setImage(icon)
            csNode.item2.item:setImage(icon)
            csNode.item1.numberitem:setString(self.m_family:getFunds())
            csNode.item2.numberitem:setString(upcostNum)
        end
    else
        self.m_upcostNum = 0
        infoNode.cond:setStringWithColor(Serialize[60944])
    end

    if visible then
        local memberModel = self.m_family:getSelfMemberInfo()
        if level > 0 then 
            visible = memberModel:hasPower(FamilyPower.kUpgradeBuild)
        else
            visible = memberModel:hasPower(FamilyPower.kUnlockBuild)
        end
        if self.m_build:isMaintaince() then
           visible = memberModel:hasPower(FamilyPower.kRelieve) 
        end
    end
    csNode.item1:setVisible(visible)
    csNode.item2:setVisible(visible)
    csNode.spend:setVisible(visible)
    for _, bType in pairs(FamilyBuildType) do
        local node = csNode.build["build" .. bType]
        local build = FamilyBuildManager:getBuildByType(bType)
        if build then
            local jingshikuang = node.jingshikuang
            jingshikuang:stopAllActions()
            jingshikuang:setVisible(false)
            if build:isMaintaince() then
                jingshikuang:setVisible(true)
                jingshikuang:runAction(cc.RepeatForever(cc.Blink(5, 5)))
            end
        end
    end
    
end

function This:onUpgradeBtnClick()
    self.m_build:updateUpgradeState()
    if self.m_build:isMaintaince() then
        local ret, state = self.m_build:isCanChangeMaintaince()  
        if not ret then
            g_scene:showTips(Serialize["fundsLack"])
            return
        end
        FamilyBuildManager:onChangeStateRequest(self.m_build:getType())
    else
        local ret, state, tips = self.m_build:canUpgrade(true)  
        if not ret then
            if TerrResponse.kFailByMaxLevel == state then
                g_scene:showTips(Serialize["family_BuildNode_8"])
                return
            elseif TerrResponse.kFailByCost == state then
                g_scene:showTips(Serialize[tips])
                return
            elseif TerrResponse.kFailByLocked == state then
                local tips = getBuildCondDesc(self.m_build, true)
                g_scene:showTips(tips)
                return
            else
                g_scene:showTips(Serialize["family_BuildNode_10"])
                return
            end
        end
        FamilyBuildManager:onUpgradeFamilyBuildRequest(self.m_build:getType(), self.m_build:getLevel())
    end
end

function This:onUpgradeFamilyBuild()
    if self.m_build then
        self.m_family:costFunds(self.m_upcostNum)
        local level = self.m_build:getLevel()
        if level == 1 then
            g_scene:showTips(Serialize["family_BuildNode_11"])
        elseif level > 0 then
            g_scene:showTips(Serialize["family_BuildNode_12"])
        end
        local bType = self.m_build:getType()
        local node = self.m_csNode.build["build" .. bType]
        local posx, posy = node.jingshikuang:getPosition()
        self:loadBuildUpEffect(node, posx, posy)
    end
    self:refresh()
end

function This:onChangeState()
    if self.m_build then
        self.m_family:costFunds(self.m_upcostNum)
    end
    self:refresh()
end

function This:onGetFamilyBuildInfo()
    self:refresh()
end