-- @Author: superm
-- @Date:   2018-06-28 17:09:02
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-09-11 16:31:41
-- @File Name: FamilyBuilding.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")

local LegionBuildingConfig = require("Metadatas.LegionBuildingConfig")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local FamilyManager = require("Family.FamilyManager")

local FamilyBuildState = {
	kNormal = 0,
	kMaintainaced = 1,
}

function This:create(...)
	local ret = {}
	setmetatable(ret,self)
	ret:init(...)
	return ret
end

function This:createByBuildType(btype)
    local ret = {}
    local info = { level = 0, type= btype }
    setmetatable(ret,self)
    ret:init(info)
    return ret
end

function This:init( info )
	super.init(self)
	self.m_level = info.level or 0
	self.m_state = info.state
	self.m_type = info.type
    self.m_lastPayTime = info.last_deduction_time
	self.m_deductionTimes = info.deduction_time or 0
	self:initIdByInfo(info)
end

function This:initIdByInfo(info)
    if info then
        local id = 10000 + info.type * 100 + info.level
	    self:setId(id)
	    self.m_config = LegionBuildingConfig[id]
    	-- self.m_nextConfig = LegionBuildingConfig[id + 1]
    end
end

function This:updateInfo( info )
	self.m_level = info.level
	self.m_state = info.state
	self.m_lastPayTime = info.last_deduction_time
    self.m_deductionTimes = info.deduction_time or 0
	self:initIdByInfo(info)
end

function This:updateUpgradeState()
    local conds = self:getConditions()
    local ret   = true
    for i, cond in ipairs(conds) do
        ret = ret and self:isNotLockedBy(cond[1], cond[2])
        if not ret then break end
    end
    self.m_upgradeState = ret and TerrObjectState.kNormal or TerrObjectState.kLocked
end

function This:canUpgrade(returnState)
    if self:isMaxLevel() then
        return false, TerrResponse.kFailByMaxLevel
    end

    if self.m_upgradeState ~= TerrObjectState.kNormal then
        if returnState then
            local ret, condition = self:isQualified()
            if not ret then
                return false, TerrResponse.kFailByLocked + TerrResponse.kFailByCost, condition
            end            
        end
        return false, TerrResponse.kFailByLocked
    end

    local ret, condition = self:isQualified()
    if not ret then
        return false, TerrResponse.kFailByCost, condition
    end

    return true, TerrResponse.kOK
end


function This:setId( id )
    self.m_ID = id
end

function This:getId()
    return self.m_ID
end

function This:getType()
    return self.m_type
end

function This:getLevel()
    return self.m_level
end

function This:setLevel( level )
    local max = self:getMaxLevel()
    if level > max then
        self.m_level = max
        return
    end
    self.m_level = level
end

function This:getConfigId()
    return self.m_descConfig.uid
end

function This:isNormal()
    return self.m_state == FamilyBuildState.kNormal
end

--低维护
function This:isMaintaince()
    local res = false
    if self.m_state and self.m_state == FamilyBuildState.kMaintainaced then
        res = true
    end
    return res
end

--低维护
function This:isCanChangeMaintaince()
    if not self:isMaintaince() then
        return false,false
    end
    local config = self:getConfig()
    local costList = config.maintaincost
    if not costList or costList == 0 then
        return true
    end

    local ret = true
    local errCode
    if not isNestedObject(costList) then
        if #costList >= 2 then
            ret, errCode = self:isCostEnoughBy(costList[1], costList[2])
        end
    else
        for _, cond in ipairs(costList) do
            local result, reason = self:isCostEnoughBy(cond[1], cond[2])
            ret = ret and result
            if not ret then
                return false, reason
            end
        end
    end
    return ret, errCode
end

function This:getDescConfig()
    if not self.m_descConfig then
    	local id = 10001 + self.m_type * 100
    	local config = LegionBuildingConfig[id]
    	local maxLevel = 0
    	for index, v in pairs(LegionBuildingConfig) do
    		if v.id == self.m_type then
    			if not maxLevel or maxLevel < index then
    				maxLevel = index
    			end
    		end
    	end

    	self.m_descConfig = { 
	    	config_id = id,
	    	uid = self.m_type,
	    	build_type = self.m_type,
	    	pic = "MenuScene/build_" .. self.m_type .. ".png",
	    	max_level = maxLevel % 100,
	    }
    end
    return self.m_descConfig
end

function This:getConfig()
    return self.m_config 
end

function This:getName()
    local config = self:getConfig()
    if config then
        return config.name
    end
end

function This:getMaxLevel()
	local config = self:getDescConfig()
    if not config or not config.max_level then
        return 0
    end
    return config.max_level 
end

function This:isMaxLevel()
    return self.m_level == self:getMaxLevel()
end

function This:isQualified()
    local config = self:getConfig()
    local costList = config.upcost
    if not costList or costList == 0 then
        return true
    end

    local ret = true
    local errCode
    if not isNestedObject(costList) then
        if #costList >= 2 then
            ret, errCode = self:isCostEnoughBy(costList[1], costList[2])
        end
    else
        for _, cond in ipairs(costList) do
            local result, reason = self:isCostEnoughBy(cond[1], cond[2])
            ret = ret and result
            if not ret then
                return false, reason
            end
        end
    end
    return ret, errCode
end

function This:isCostEnoughBy( id, num )
	local ret = false
	if id == 126 then
		ret = FamilyManager:getMyFamilyFunds() >= num
		if not ret then
			return ret, "fundsLack"
		else
			return ret
		end
	end
	return ret, "itemNotFound"
end

function This:isNotLockedBy( cond1, cond2 )
    return FamilyBuildManager:getBuildingLevel(cond1) >= cond2
end

function This:getConditions()
    local cfg = self:getConfig()
    if self.m_level > 0 then
        return cfg.upcondition
    end
    return cfg.unlocking
end

function This:getUpgradeCost()
    local config = self:getConfig()
    if config then
        local info = config.upcost
        if info[1] then
            local costId, costNum = unpack(info[1])
            return costId, costNum
        end
    end
    return
end

function This:getMaintainceCost()
    local config = self:getConfig()
    if config then
    	local info = config.maintaincost
        if info[1] then
    	    local costId, costNum = unpack(info[1])
            return costId, costNum
        end
    end
end

function This:getDeductionCost()
    local _, num = self:getMaintainceCost()
    return self.m_deductionTimes * num
end

function This:getDescription()
    if self:isNormal() then
        return self:getNormalDescr()
    else
        return self:getMaintainceDescr()
    end
end

function This:getNormalDescr()
    local config = self:getConfig()
    if config then
        return config.desp1
    end
end

function This:getMaintainceDescr()
    local config = self:getConfig()
    if config then
        return config.desp2
    end
end
