-- @Author: superm
-- @Date:   2017-12-15 18:57:11
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-07-10 10:16:14
-- @File Name: FamilyCreatePanel.lua
-- @Description: 

local This , super  = class(...,"Dialog")
local FamilyManager = require("Family.FamilyManager")
local GamePlayer = require("Player.GamePlayer")
local FamilyUtil = require("Family.FamilyUtil")
local GameUtil = require("Util.GameUtil")

function This:initNode(info)
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyCreation.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	if not info.inputNameOnly and not info.inputDescOnly then
		self.m_csNode.inputName.name:setString(info.defaultName or "")
		self.m_csNode.inputDesc.declaration:setString(info.defaultDesc or "")
	elseif info.inputNameOnly then
		self.m_csNode.inputDesc:setVisible(false)
		self.m_csNode.declaration:setVisible(true)
		self.m_csNode.declaration:setString(info.defaultDesc, 300)
		self.m_csNode.inputName.name:setString(info.defaultName or "")
	else
		self.m_csNode.inputName:setVisible(false)
		self.m_csNode.name:setVisible(true)
		self.m_csNode.name:setString(info.defaultName)
		self.m_csNode.inputDesc.declaration:setString(info.defaultDesc or "")
	end

	if info.title then
		self.m_csNode.title:setString(info.title)
	end
	if info.nameMaxLength then
		self.m_csNode.inputName.name:setMaxLength(info.nameMaxLength)
	end

	if info.descMaxLength then
		self.m_csNode.inputDesc.declaration:setMaxLength(info.descMaxLength)
	end

	if info.cost then
		self.m_csNode.cost:setString(info.cost)
	else
		self.m_csNode.cost:setVisible(false)
		self.m_csNode.costdesc:setVisible(false)
		self.m_csNode.costbg:setVisible(false)
		self.m_csNode.icon:setVisible(false)
	end
	self.m_info = info
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.cancel:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.confirm:setTouchEnded(function()
		local name, desc
		if not self.m_info.inputDescOnly then
			name = self.m_csNode.inputName.name:getString()
			local len = string.utf8Len(name)
			if len <= 0 then
				g_scene:showTips(Serialize["family_CreatePanel_1"])
				return
			elseif len < 2 then
				g_scene:showTips(Serialize["tip_1_c"])
				return
			elseif len > 7 then
				g_scene:showTips(Serialize["tip_2_c"]) 
				return
			elseif GameUtil:checkInvalidAscill(name) then
				g_scene:showTips(Serialize["tip_3_c"])
				return
			end
		end

		if not self.m_info.inputNameOnly then
			desc = self.m_csNode.inputDesc.declaration:getString()
			local len = string.utf8Len(desc)
			if len <= 0 then
				g_scene:showTips(Serialize["family_CreatePanel_2"])
				return
			elseif len < 1 then
				g_scene:showTips(Serialize["tip_5_c"])
				return
			elseif len > 50 then
				g_scene:showTips(Serialize["tip_4_c"]) 
				return
			elseif GameUtil:checkInvalidAscill(desc, true) then
				g_scene:showTips(Serialize["tip_3_c"])
				return
			end
		end

		if self.m_info.cost then
			self.m_info.okHandler = function()
				if self.m_renameHandler then
					self.m_renameHandler(name, desc)
				end
				-- self:close()
			end
			self.m_info.desc = self.m_info.title
			GamePlayer:costDiamond(self.m_info)
		else
			if self.m_renameHandler then
				self.m_renameHandler(name, desc)
			end
			-- self:close()
		end
	end)
end

function This:onEnter()
	super.onEnter(self)
	FamilyManager:addObserver(self)
end

function This:onExit()
	FamilyManager:removeObserver(self)
	super.onExit(self)
end

function This:onCreateFamily()
	self:close()
end

function This:onFamilyRename()
	self:close()
end

function This:onSubmitApplyFilter()
	self:close()
end

function This:setTitle(str)
	self.m_csNode.title:setString(str)
end

function This:setRenameHandler(handler)
	self.m_renameHandler = handler
end