﻿-- @Author: superm
-- @Date:   2018-06-28 20:57:19
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-07-17 10:04:46
-- @File Name: FamilyDonatePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local FamilyManager = require("Family.FamilyManager")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")
local GamePlayer = require("Player.GamePlayer")
local ContributeConfig = require("Metadatas.ContributeConfig")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local ContributeBoxConfig = require("Metadatas.ContributeBoxConfig")


local BoxImage = 
{
	[1] = {
		"MenuScene/boxLock1.png",
		"MenuScene/boxGet1.png",
	},
	[2] = {
		"MenuScene/boxLock2.png",
		"MenuScene/boxGet2.png",
	},
	[3] = {
		"MenuScene/boxLock3.png",
		"MenuScene/boxGet3.png",
	},
	[4] = {
		"MenuScene/boxLock4.png",
		"MenuScene/boxGet4.png",
	}
}

function This:initNode(family)
    -- if not FamilyBuildManager:isBuildUnlock(FamilyBuildType.kShangHui) then
    --     g_scene:showTips(Serialize[60948])
    --     return false
    -- end

    self.m_family = family
    self.m_itemList = {}
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyDonate.csb")
    self.m_csNode:setToCenter()
    self.m_sliderX = self.m_csNode.jindubg.jindutiao:getScaleX()
	self:addChild(self.m_csNode)
	self:refresh()
end

function This:getLeftDonateTimes()
    local leftTimes = kFamilyDonateTimes - FamilyManager:getDonateTimes()
    if leftTimes < 0 then
       leftTimes = 0
    end
    return leftTimes
end

function This:refresh()
    local function updateLabel(txt,visible)
        local color = {r=255,g=0,b=0}
        local str = Serialize["family_DonatePanel_2"]
        if visible then
            color = {r=187,g=228,b=97}
            str = Serialize["family_DonatePanel_1"]
        end
        txt:setColor(color.r,color.g,color.b)
        txt:setString(str)
    end
    updateLabel(self.m_csNode.info.zhengchang,FamilyBuildManager:isInNormalState())
    self.m_csNode.info.numberjtzj:setString(self.m_family:getFunds())
    self.m_csNode.beijing.shengyucishu:setString(string.format(Serialize["family_DonatePanel_3"], self:getLeftDonateTimes(), kFamilyDonateTimes))
    local maxScore = FamilyManager:getMaxScore()
    local score = FamilyManager:getMyFamily():getDonationScore()
    local tex = Serialize[60949]
    self.m_csNode.text:setString(string.format( tex,score ))
    if score >= maxScore then
        score = maxScore
    end
    local w = (score/maxScore)*self.m_sliderX
    self.m_csNode.jindubg.jindutiao:setPercent(w*100)
    for i=1,4 do
        self.m_csNode["bgnum"..i].text:setString(ContributeBoxConfig[i].score)
        self.m_csNode["bgnum"..i]:setPositionX(-90+430*ContributeBoxConfig[i].score/maxScore)
        self.m_csNode["boxLock"..i]:setPositionX(-90+430*ContributeBoxConfig[i].score/maxScore)
    end
    self:refreshBoxState()
end

function This:refreshBoxState()
    for i = 1,4 do
        local state = self:getStateById(i)
        local node,index = self.m_csNode["boxLock"..i],2
        GameUtil:createBoxEffect(state==FamilyDonateRewardState.kCanRecive,node,i)
        if state == FamilyDonateRewardState.kNoReward then
            index = 1
        elseif state == FamilyDonateRewardState.kHasRecive then
            node:setGrayShader()
        end
        node:setImage(BoxImage[i][index])
    end
end

function This:onEnter()
    super.onEnter(self)
    FamilyManager:addObserver(self)
    FamilyRatingManager:addObserver(self)
end

function This:onExit()
    FamilyManager:removeObserver(self)
    FamilyRatingManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()

    self.m_csNode.beijing.closebutton:setTouchEnded(function()
        if self.m_runningAniantion then
            return false
        end
        self:close()
    end)
    self.m_csNode.wenhao:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize["family_donateText"],node = self.m_csNode.wenhao})
    end)
    for i, config in ipairs(ContributeConfig) do
    	local node = self.m_csNode["jishi" .. i]
        node.jsname:setString(config.name)
        node.jifen:setString(string.format( Serialize[60950],config.score))
        local itemNode1 = node.item1
        local itemNode2 = node.item2
        itemNode1.item:setImage(ItemUtil.getIconImage(126))
        itemNode2.item:setImage(ItemUtil.getIconImage(127))
        itemNode1.numberitem:setString(string.format("+ %d", config.funds))
        itemNode2.numberitem:setString(string.format("+ %d", config.contribution))
        local costId, costNum = unpack(config.upcost[1])
        node.button.jinbi:setImage(ItemUtil.getIconImage(costId))
        node.button.numberjinbi:setString(" " .. ItemUtil.formatItemCount(costNum))
        node.button.buttonhuang:setTouchEnded(function ()
	    	local ret, errCode = self:isCostEnoughBy( costId, costNum, function ()
                FamilyManager:onFamilyDonateRequest(config.id)
            end )

	    	if not ret then
                if errCode then
                    g_scene:handleErrorCode(errCode)
                end
	    		return
	    	end
    	end)
    end

    for i=1,4 do
        self.m_csNode["boxLock"..i]:setTouch({
            onEnded = function ()
                g_scene:openDialog("Family.FamilyDonateReward",{key=i})
            end,
            isScale = false,
        })
    end
end

function This:isCostEnoughBy( id, num, handler )
    if ItemUtil.getType(id) == ItemCommonType.kGameRes then
        local switch = {
            [101] = function(num)
                -- GamePlayer:costGold({
                --     itemId = 101,
                --     cost = num,
                --     title = Serialize[60951],
                --     desc = Serialize[60951],
                --     okHandler = handler
                -- })
                handler()
                return false
            end,
            [102] = function(num)
                GamePlayer:costDiamond({
                    itemId = 102,
                    cost = num,
                    title = Serialize[60951],
                    desc = Serialize[60951],
                    okHandler = handler
                })
                return false
            end,
        }
        return switch[id](num)
    else
        handler()
        return true
    end
end

function This:getStateById(id)
    local state = FamilyDonateRewardState.kNoReward
    local score = FamilyManager:getMyFamily():getDonationScore()
    if score >= ContributeBoxConfig[id].score then
        state = FamilyDonateRewardState.kCanRecive
    end
    for _,data in ipairs(FamilyManager:getDonatineRewardRecord()) do
        if data == id then
            state=FamilyDonateRewardState.kHasRecive
        end
    end
    return state
end

function This:onFamilyDonate()
    self:refresh()
end

function This:onDonateRewardRecive()
    self:refresh()
end