-- @Author: superm
-- @Date:   2017-12-15 10:29:24
-- @Last Modified by:   superm
-- @Last Modified time: 2017-12-28 18:33:40
-- @File Name: FamilyInfoItemView.lua
-- @Description: 

local This , super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/FamilyListItem.csb"

local FamilyManager = require("Family.FamilyManager")
local FamilyUtil = require("Family.FamilyUtil")

function This:init(familyId, index)
    self.m_inited = false
    self.m_familyId = familyId
    self.m_family = FamilyManager:getFamily(familyId)
    self.m_listIndex = index
    local w , h = self.bg:getContentSize()
    self:setContentSize(w, h)
end

function This:initNode()
	self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.apply:setTouch({
		    onEnded = function()
                self:onBuyBtnClick()
            end,
            isContainer = true,
        })    
        self.m_inited = true
    end
end

function This:onEnter()
    super.onEnter(self)
    self.m_family:setView(self)
end

function This:onExit()
    self.m_family:setView(nil)
    super.onExit(self)
end

function This:setModel( familyId, index )
    if self.m_familyId ~= familyId then
        self.m_family:setView(nil)
        self.m_familyId = familyId
        self.m_family = FamilyManager:getFamily(familyId)
        self.m_listIndex = index     
        self.m_family:setView(self)
        self:refresh()
    end
end

function This:refresh()
    self:refreshItemUI()
    self:setItemEnable()
    self.bg:setVisible(self.m_listIndex % 2 == 1)
end

function This:refreshItemUI()
    self.number:setString(tostring(self.m_listIndex))
    self.name:setString(self.m_family:getName())
    self.level:setString("Lv." .. self.m_family:getLevel())
    self.integral:setString(self.m_family:getScore())
    self.people:setString(self.m_family:getMemberPeple())
end

function This:refreshConditionUI()
    self.condition:setString(self.m_family:getConditionDesc())
end

function This:setItemEnable()
    local isLocked = self:isLocked()
    if isLocked then
        self:refreshConditionUI()
        self.applied:setVisible(false)
        self.apply:setVisible(false)
    else
        local isExist = self.m_family:isInMyRequestList()
        self.applied:setVisible(isExist)
        self.apply:setVisible(not isExist)
    end
    self.condition:setVisible(isLocked)
end

function This:isLocked()
    return not self.m_family:isQualified()
end

function This:onBuyBtnClick()
    if self.m_family and not self:isLocked() then
        self.m_family:requestSubmitApplying()
    end
end

function This:onSubmitApplying( familyId )
    if familyId == self.m_familyId then
        self.applied:setVisible(true)
        self.apply:setVisible(false)
    end
end

function This:onCancelApplying( familyId )
    if familyId == self.m_familyId then
        self.applied:setVisible(false)
        self.apply:setVisible(true)
    end
end

--endregion
 