-- @Author: superm
-- @Date:   2017-12-15 10:25:25
-- @Last Modified by:   superm
-- @Last Modified time: 2018-07-09 10:11:46
-- @File Name: FamilyListTabNode.lua
-- @Description: 全部家族的列表分页

local This, super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/FamilyList.csb"

local RankListLayer = require("Rank.RankListLayer")
local FamilyInfoItemView = require("Family.FamilyInfoItemView")
local FamilyManager = require("Family.FamilyManager")

function This:initData( ... )
end

function This:initNode()
    FamilyManager:onGetAllFamilyInfoRequest()
    self.m_listLayer = RankListLayer:create({
        itemWidth = 920,
        itemHeight = 45,
        spaceX = 0,
        spaceY = 0,
        scrollView = self.scrollView,
        scrollHandler = function ( data, event )
            if event == 1 and not self.m_locked then
                self.m_locked = true
                FamilyManager:onGetAllFamilyInfoRequest()
                return true
            end
            return false
        end
    })
    
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)
	self.m_inited = true
end

function This:initTouchEvent()
    self.createf:setTouchEnded(function()
        local panel = g_scene:openDialog("Family.FamilyCreatePanel",{
            nameMaxLength = 7,
            descMaxLength = 500,
            title = Serialize["family_ListTab_1"],
            cost = kFamilyCreatCost,
        })
        panel:setRenameHandler(function(name, desc)
		    FamilyManager:onCreateFamilyRequest(name, desc)
        end)
    end)

	self.fastjoin:setTouchEnded(function()
        FamilyManager:onJoinFamilyQuicklyRequest()
	end)
end

function This:onEnter()
    super.onEnter(self)
    FamilyManager:addObserver(self)
end

function This:onExit()
    self.m_isClose = true
    super.onExit(self)
    FamilyManager:removeObserver(self)
    FamilyManager:clearFamilyList()
end

function This:refresh()
	if self.m_inited then
		self:refreshListView()
	end
end

function This:refreshListView()
    self.m_listLayer:setDataList(self.m_data, function(familyId, index)
        return FamilyInfoItemView:create(familyId, index)
    end)
end

function This:onGetAllFamilyInfo(listData)
    if self.m_isClose then
        return
    end
	
    self.m_data = listData
    self.m_locked = (#listData % FamilyManager:getPageNum()) ~= 0
	self:refresh()
end