﻿
-- @Author: superm
-- @Date:   2017-12-13 20:58:38
-- @File Name: FamilyManager.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local FamilySimpleModel = require("Family.FamilySimpleModel")
local SubscriberConfig = require("Metadatas.SubscriberConfig")
local FamilyModel = require("Family.FamilyModel")
local FamilyMemberModel = require("Family.FamilyMemberModel")
local FamilyList = require("Family.FamilyList")
local FamilyUtil = require("Family.FamilyUtil")
local GamePlayer = require("Player.GamePlayer")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local EntrantManager = require("LegionEntrant.EntrantManager")
local VitalityManager = require("Achieve.VitalityManager")
local LegionMilitaryConfig = require("Metadatas.LegionMilitaryConfig")
local ContributeBoxConfig = require("Metadatas.ContributeBoxConfig")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")
local DepotManager = require("Depot.DepotManager")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")
local FamilyBossManager = require("FamilyBoss.FamilyBossManager")


FamilyDonateRewardState = {
    kNoReward  = 0,         --无奖励
    kCanRecive = 1,         --可领取
    kHasRecive = 2,         --已领取    
}

FamilyScienceTab = {
    kBuBing = 1,
    kQiBing = 2,
    kGongBing = 3,
    kCheBing = 4
}

function This:getPageNum()
    return 20
end

function This:isInited()
    return self.m_inited
end

function This:init()
	super.init(self)
    self:initListener()

    self.m_selfFamilyId = 0
    self.m_selfFamily = nil
    self.m_selfMember = nil
    self.m_donateTimes = 0
    self.m_isEntranted = 0
    self.m_requestList = {}
    self.m_strategy = {}

    self.m_inited = false
    self.m_isDirty = false

    self:clearFamilyList()
    self.m_familyMap = FamilyList:create()

    FamilyBuildManager:init()
    EntrantManager:init()
    FamilyRatingManager:init()
    FamilyBossManager:init()
    -- self:initStrategy()
end

function This:initListener()
    EventDispatcher:addListener(self, "init_role_info", self.onGetFamilyInfoChangeRequest)
	EventDispatcher:addListener(self,"day_passed",self.onDayPassed)

    EventDispatcher:addListener(self, 12100, self.onGetAllFamilyInfoReply)
    EventDispatcher:addListener(self, 12101, self.onCreateFamilyReply)
    EventDispatcher:addListener(self, 12102, self.onGetMyFamilyInfoReply)
    EventDispatcher:addListener(self, 12103, self.onSubmitApplyingReply)
    EventDispatcher:addListener(self, 12104, self.onCancelApplyingReply)
    EventDispatcher:addListener(self, 12105, self.onApplyApprovalReply)
    EventDispatcher:addListener(self, 12106, self.onMemberKickOffReply)
    EventDispatcher:addListener(self, 12107, self.onMemberAppointReply)
    EventDispatcher:addListener(self, 12108, self.onSubmitApplyFilterReply)
    EventDispatcher:addListener(self, 12109, self.onGetFamilyInfoChangeReply)
    EventDispatcher:addListener(self, 12110, self.onFamilyRenameReply)
    EventDispatcher:addListener(self, 12111, self.onFamilyUpgradeReply)
    EventDispatcher:addListener(self, 12112, self.onJoinFamilyQuicklyReply)
    EventDispatcher:addListener(self, 10800, self.onSendMessageReply)
    EventDispatcher:addListener(self, 12113, self.onReceiveMemberChangeNotify)
    EventDispatcher:addListener(self, 12118, self.onFamilyDonateReply)
    EventDispatcher:addListener(self, 12126, self.onFamilySendMailReply)
    EventDispatcher:addListener(self, 12127, self.onFamilyBonusReply)
    EventDispatcher:addListener(self, 12130, self.onFamilyBonusStateReply)
    EventDispatcher:addListener(self, 12132, self.onGetSendInheritReply)
    EventDispatcher:addListener(self, 12133, self.onMemberKickOffReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self,"init_role_info")
    EventDispatcher:removeListener(self,"day_passed")
    EventDispatcher:removeListener(self,12100)
    EventDispatcher:removeListener(self,12101)
    EventDispatcher:removeListener(self,12102)
    EventDispatcher:removeListener(self,12103)
    EventDispatcher:removeListener(self,12104)
    EventDispatcher:removeListener(self,12105)
    EventDispatcher:removeListener(self,12106)
    EventDispatcher:removeListener(self,12107)
    EventDispatcher:removeListener(self,12108)
    EventDispatcher:removeListener(self,12109)
    EventDispatcher:removeListener(self,12110)
    EventDispatcher:removeListener(self,12111)
    EventDispatcher:removeListener(self,12112)
    EventDispatcher:removeListener(self,10800)
    EventDispatcher:removeListener(self,12113)
    EventDispatcher:removeListener(self,12118)
    EventDispatcher:removeListener(self,12126)
    EventDispatcher:removeListener(self,12127)
    EventDispatcher:removeListener(self,12130)
    EventDispatcher:removeListener(self,12132)
    EventDispatcher:removeListener(self,12133)

    FamilyBuildManager:removeListener()
    EntrantManager:removeListener()
    DepotManager:removeObserver(self)
end

function This:clearFamilyList()
    self.m_curStartIndex = 1
    self.m_curEndIndex = 1
    self.m_curIndex = 0
    self.m_familyList = {} 
    self.m_requestList = {}
end

function This:setDonateTimes( times )
    self.m_donateTimes = times or 0
end

function This:getDonateTimes()
    return self.m_donateTimes  
end

function This:setDonatineRewardRecord( record )
    self.m_donationRewardRecord = record
    self:notifyObservers("onDonatineRewardRecordChange")	
end

function This:getDonatineRewardRecord()
    return self.m_donationRewardRecord or{}
end

function This:donationRewardHasReceive( id )
    local receive = false
    for _,data in pairs(self:getDonatineRewardRecord()) do
        if data == id then
            receive = true
            break
        end
    end
    return receive
end

function This:addSimpleFamily( info, index )
    local id = info.group_id
    self.m_familyList[index] = id
    
    local family = self.m_familyMap:getNode(id)
    if not family then
        self.m_familyMap:addNode(FamilySimpleModel:create(info), id)
    else
        family:updateInfo(info)
    end
end

function This:getFamily( id )
    if self.m_familyMap then
        return self.m_familyMap:getNode(id) 
    end
end

function This:getMyFamilyName()
    if self:hasFamily() and self.m_selfFamily then
        return self.m_selfFamily:getName()
    end
end

function This:getMyFamilyId()
    if self:hasFamily() then
        return self.m_selfFamilyId
    end
end

function This:getMyFamilyFunds()
    if self:hasFamily() and self.m_selfFamily then
        return self.m_selfFamily:getFunds()
    end
    return 0
end

function This:getMyFamilyLoong()
    if self:hasFamily() and self.m_selfFamily then
        return self.m_selfFamily:getLoong()
    end
    return 0
end

function This:setMyFamilyLoong(loong)
    if self:hasFamily() and self.m_selfFamily then
        self.m_selfFamily:setLoong(loong)
    end
end

function This:getMyFamily()
    if self:hasFamily() then
        return self.m_selfFamily 
    end
end

function This:getMySelfMemeberInfo()
    if self:hasFamily() and self.m_selfFamily and self.m_selfMember then
        return self.m_selfMember
    end
end

function This:getMySelfTitle()
    local res = FamilyPower.kChengYuan
    if self:getMySelfMemeberInfo() then
        res = self.m_selfMember:getTitle()
    end
    return res
end

function This:checkMySelfTitlePower(powerType)
    local result = false
    if powerType and self:getMySelfMemeberInfo() then
        result = self.m_selfMember:hasPower(powerType)
    end
    return result
end

function This:isInRequestList( familyId )
    if self.m_requestList then
        return self.m_requestList[familyId]
    end
    return false
end

function This:hasFamily()
    return self.m_selfFamilyId > 0
end

function This:setEntranted(value)
    if self:hasFamily() then
        self.m_isEntranted = value
    end
end

function This:isEntranted()
    if self:hasFamily() then
        return self.m_isEntranted > 0
    end
end

function This:isGroupEntranted()
    if self:hasFamily() then
        return self.m_isEntranted == 1
    end
end

function This:isKingEntranted()
    if self:hasFamily() then
        return self.m_isEntranted > 1
    end
end

function This:getMaxScore()
    local num =0
    for _,config in ipairs(ContributeBoxConfig) do
        if num <= config.score then
            num = config.score
        end
    end
    return num
end

function This:saveFamilyRecord(value)
    if self.m_familyRecord or self:hasFamily() then
        return true
    end

    local params = cc.FileUtils:loadLuaFile("FamilyInfoRecord") or {}
    local key = tostring(GamePlayer:getPlayerID())
    print_r(params)
    if value ~= nil then
        self.m_familyRecord = value
        params[key] = self.m_familyRecord
        cc.FileUtils:saveToLua(params,"FamilyInfoRecord")
    else
        if params then
            self.m_familyRecord = params[key]
        end
    end
    return self.m_familyRecord
end

function This:isDirty()
    return self.m_isDirty
end

function This:notifyFamily( id, funcName )
    local family = self:getFamily(id)
    if family then
        family:notifyView(funcName, id)
    else
        self:notifyObservers(funcName, id)
    end
end

function This:openInvitePanel( info )
    if self:hasFamily() then
        g_scene:handleErrorCode("hadFamily")
        return
    end

    print_r(info)
    local ret = false
    if not info or not info.groupId then
        ret = true
    else
        local groupId = info.groupId
        local valueType = type(info.groupId)
        if valueType == "number" and groupId <= 0 then
            ret = true
        elseif valueType == "string" then
            if string.len(groupId) <= 0 then
                ret = true
            else
                local tmp = tonumber(groupId)
                if not tmp or tmp <= 0 then
                    ret = true
                else
                    info.groupId = tmp
                end
            end
        end
    end
    local isLock,lvLimit = DailyAchieveManager:getArgConfigLv("join_group_level",true)
    if isLock then
        g_scene:showTips(string.format(Serialize["family_Manager_10"],lvLimit))
        return
    end
    if ret then
        g_scene:showTips(Serialize["family_Manager_1"])
        return
    end
    g_scene:openDialog("Family.FamilyInvitePopPanel", info)
end

function This:setSendMailTimes( times )
    self.m_SendMailTimes = times or 0
end

function This:getSendMailTimes()
    return self.m_SendMailTimes
end

function This:setInviteTimes(InviteTimes)
    self.m_InviteTimes = InviteTimes or 0
end

function This:getInviteTimes()
    return self.m_InviteTimes
end

function This:hasFamilyRed()
    if FamilyRatingManager:ratingHaveReward()
            or FamilyRatingManager:scienceHaveReward()
            or g_redDot:checkFamilyVitality()
            or g_redDot:checkFamilyAffairsBonus()
            or g_redDot:checkFamilyAffairsBoss()
            or g_redDot:checkAllFamilyBuilding()
            or g_redDot:checkLegionAboss()
    then
        return true
    end
    return false
    
end

function This:onSendMessageReply( msg )
    if msg and msg.chat and msg.chat.channel == ChatChannelType.kFamily then
        if msg.chat.from_role and msg.chat.from_role.id == 0 then
            if self:hasFamily() then
                self:onGetMyFamilyInfoRequest()
            end
        end
    end
end

-- 查看所有家族信息
function This:onGetAllFamilyInfoRequest()
    -- print_r({start= self.m_curEndIndex, end_index = self:getPageNum() + self.m_curEndIndex - 1})
    Socket:sendAndLock(12100, {start= self.m_curEndIndex, end_index = self:getPageNum() + self.m_curEndIndex - 1})
end

function This:onGetAllFamilyInfoReply( msg )
    if msg and msg.err_code == 0 then
        self.m_curStartIndex = self.m_curEndIndex
        for i, info in ipairs(msg.group_list) do
            self.m_curEndIndex = self.m_curEndIndex + 1
            self:addSimpleFamily(info, self.m_curEndIndex - 1)
        end

        if not self.m_requestList then
            self.m_requestList = FamilyUtil:getValueMap(msg.request_id_list)
        else
            FamilyUtil:getValueMap(msg.request_id_list, self.m_requestList)
        end

        self:notifyObservers("onGetAllFamilyInfo", self.m_familyList)
    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 创建家族
function This:onCreateFamilyRequest(name, desc)
    Socket:sendAndLock(12101, {name = name, desc = desc})
end

function This:onCreateFamilyReply( msg )
    if msg and msg.err_code == 0 then
        self:notifyObservers("onCreateFamily")
		EventDispatcher:dispatcher("family_change")		


    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 查看自身家族信息
function This:onGetMyFamilyInfoRequest(silence)
    if not silence then
        Socket:sendAndLock(12102)
    else
        Socket:send(12102)
    end
end

function This:onGetMyFamilyInfoReply( msg )
    if msg and msg.err_code == 0 then
        self:setDonateTimes(msg.donation_total_time)
        self:setDonatineRewardRecord(msg.donation_reward_record)
        self.m_selfFamilyId = msg.group_info.group_id or 0
        if not self.m_selfFamily then
            self.m_selfFamily = FamilyModel:create(msg.group_info)
        else
            self.m_selfFamily:updateInfo(msg.group_info)
        end
        self.m_selfMember = self.m_selfFamily:getSelfMemberInfo()
        self.m_isDirty = false
        FamilyRatingManager:updateAllHeroRatingAttr()
        FamilyRatingManager:onScienceRequest()
        FamilyRatingManager:updateAllHeroScienceAttr()
        -- self:onStrategyRequest()
        if self:hasFamily() and FamilyBuildManager:isInNormalState() then
            self:onFamilyBonusStateRequest()
        end
        self:notifyObservers("onGetMyFamilyInfo", self.m_selfFamily)
    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 申请加入家族
function This:onSubmitApplyingRequest(id)
    print_r({group_id = id})
    Socket:sendAndLock(12103, {group_id = id})
end

function This:onSubmitApplyingReply( msg )
    print_r(msg)
    if msg and msg.err_code == 0 then
        g_scene:showTips(Serialize["family_Manager_2_c"])
        self.m_requestList[msg.group_id] = true
        self:notifyFamily(msg.group_id, "onSubmitApplying")
    elseif msg and msg.err_code then
        if msg.err_code ~= 1 then
            g_scene:handleErrorCode(msg.err_code)
        else
            g_scene:showTips(Serialize["familyNotFound"])
        end
    end
end

-- 取消加入家族的申请
function This:onCancelApplyingRequest(id)
    Socket:sendAndLock(12104, {group_id = id})
end

function This:onCancelApplyingReply( msg )
    if msg and msg.err_code == 0 then
        g_scene:showTips(Serialize["family_Manager_3"])
        self.m_requestList[msg.group_id] = false
        self:notifyFamily(msg.group_id, "onCancelApplying")
    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 审批加入家族申请
function This:onApplyApprovalRequest(roleId, agree)
    if self.m_selfFamily and self.m_selfFamily:hasRequest(roleId) then
        Socket:sendAndLock(12105, {request_role_id = roleId, agree = agree})
    end
end

function This:onApplyApprovalReply( msg )
    local ret = false
    if msg and msg.err_code == 0 then
        ret = msg.agree
        local opRet = self.m_selfFamily:removeRequest(msg.request_role_id, ret)
        if opRet then
            self.m_isDirty = true
            self:notifyObservers("onApplyApproval", msg)
        end
    elseif msg and msg.err_code == 1 then
        self:onGetMyFamilyInfoRequest()
        g_scene:showTips(Serialize["family_Manager_4"])
    elseif msg then
        if msg.err_code == 12208 then
            self:onApplyApprovalRequest(msg.request_role_id, 2) 
            local opRet = self.m_selfFamily:removeRequest(msg.request_role_id, false)
            if opRet then
                self.m_isDirty = true
                self:notifyObservers("onApplyApproval", msg)
            end  
        end
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 审批家族条件设置
function This:onSubmitApplyFilterRequest( params )
    Socket:sendAndLock(12108, params)
end

function This:onSubmitApplyFilterReply( msg )
    if msg and msg.err_code == 0 then
        if self.m_selfFamily then
            local ret = self.m_selfFamily:updateCondition(msg)
            self:notifyObservers("onSubmitApplyFilter", ret)
        end
    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

--解散
function This:onMengZhuQuitRequest(roleId)
    Socket:sendAndLock(12133, {target_id = roleId})
end

function This:onMengZhuQuitReply( msg )
    if msg and msg.err_code == 0 then
        self.m_selfMember:setMilitary(0)
        FamilyRatingManager:getScienceData():resetAddtion()
        FamilyRatingManager:updateAllHeroRatingAttr()
        FamilyRatingManager:updateAllHeroScienceAttr()
        -- self.m_selfFamily:setMemberMilitary(msg.role_id, msg.military_id)
        self.m_selfFamilyId = 0
        self.m_selfFamily = nil
        self.m_selfMember = nil
        self.m_requestList = {}
        FamilyBuildManager:cleanAllBuilding()
        VitalityManager:cleanAllFamilyAchieves()
        self:notifyObservers("onMemberKickOff")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 家族成员踢人
function This:onMemberKickOffRequest(roleId)
    if self.m_selfFamily then
        Socket:sendAndLock(12106, {target_id = roleId})
    end
end

function This:onMemberKickOffReply( msg )
    if msg and msg.err_code == 0 then
        local _, member = self.m_selfFamily:removeMember(msg.target_id)
        if member then
            g_scene:showTips( member:getName() .. Serialize[60965])
        else
        --     self.m_selfMember:setMilitary(0)
        --     FamilyRatingManager:getScienceData():resetAddtion()
        --     FamilyRatingManager:updateAllHeroRatingAttr()
        --     FamilyRatingManager:updateAllHeroScienceAttr()
        --     -- self.m_selfFamily:setMemberMilitary(msg.role_id, msg.military_id)
        --     self.m_selfFamilyId = 0
        --     self.m_selfFamily = nil
        --     self.m_selfMember = nil
        --     self.m_requestList = {}
        --     FamilyBuildManager:cleanAllBuilding()
        --     VitalityManager:cleanAllFamilyAchieves()
            self:onMengZhuQuitReply(msg)
        end
        self:notifyObservers("onMemberKickOff")
    elseif msg and msg.err_code == 1 then
        self:onGetMyFamilyInfoRequest()
        g_scene:showTips(Serialize["family_Manager_6"])
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 家族成员设置官职
function This:onMemberAppointRequest(roleId, title)
    if self.m_selfFamily then
        Socket:sendAndLock(12107, {target_id = roleId, title = title})
    end
end

function This:onMemberAppointReply( msg )
    if msg and msg.err_code == 0 then
        self.m_selfFamily:setMemberTitle(msg.target_id, msg.title)
        if self.m_selfFamily:isDirty() then
            self:onGetMyFamilyInfoRequest()
        end
        g_scene:showTips(Serialize["family_Manager_7"])
        self:notifyObservers("onMemberAppoint")
    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 角色家族信息变化
function This:onGetFamilyInfoChangeRequest()
    Socket:sendAndLock(12109)
end

function This:onGetFamilyInfoChangeReply( msg )
    if msg then
        local oldstate = self:hasFamily()
        if msg.group_id and msg.group_id > 0 then
            self:saveFamilyRecord(true)
        end
        
        self.m_selfFamilyId = msg.group_id or 0
        local newstate = self:hasFamily()
        if not self:isInited() and newstate then
            self.m_inited = true
            self:onGetMyFamilyInfoRequest()
            FamilyBuildManager:onGetFamilyBuildInfoRequest(true)
            VitalityManager:onGetFamilyTaskInfoRequest()
            require("LegionAboss.LegionAbossManager"):request()

        elseif self:isInited() and oldstate and not newstate then
            
            self.m_selfMember:setMilitary(0)
            FamilyRatingManager:updateAllHeroRatingAttr()
            -- 这里判断并不准确
            self.m_selfFamilyId = 0
            self.m_selfFamily = nil
            self.m_selfMember = nil
            self.m_requestList = {}
            g_scene:showTips(Serialize["family_Manager_5_c"])
            FamilyBuildManager:cleanAllBuilding()
            VitalityManager:cleanAllFamilyAchieves()

        end
        self:notifyObservers("onGetFamilyInfoChange", newstate)
    else
        g_scene:handleErrorCode("")
    end
end

-- 家族改名
function This:onFamilyRenameRequest(name)
    Socket:sendAndLock(12110, {name = name})
end

function This:onFamilyRenameReply( msg )
    if msg and msg.err_code == 0 then
        if self.m_selfFamily then
            self.m_selfFamily:setName(msg.name)
            self:notifyObservers("onFamilyRename", msg.name)
        end
    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 家族升级
function This:onFamilyUpgradeRequest()
    Socket:sendAndLock(12111)
end

function This:onFamilyUpgradeReply( msg )
    if msg and msg.err_code == 0 then
        self.m_selfFamily:increaseLevel()
        self:notifyObservers("onFamilyUpgrade")
    elseif msg then
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 快速加速
function This:onJoinFamilyQuicklyRequest()
    Socket:sendAndLock(12112)
end

function This:onJoinFamilyQuicklyReply( msg )
    if msg.err_code == 0 then
        g_scene:showTips(Serialize["family_Manager_2_c"])
        self:notifyObservers("onJoinFamilyQuickly")
    elseif msg and msg.err_code == 1 then
        self:clearFamilyList()
        self:onGetAllFamilyInfoRequest()
        g_scene:showTips(Serialize["family_Manager_8"])
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onReceiveMemberChangeNotify( msg )
    if msg then
        if self:hasFamily() and self.m_selfFamily then
            local changed = self.m_selfFamily:updateMemberInfo(msg.member) 
            if changed then
                self:onGetMyFamilyInfoRequest(true)
            end
            local ratingData = FamilyRatingManager:getRatingData()
            if not ratingData then
                return
            end
            FamilyRatingManager:updateAllHeroRatingAttr()
            FamilyRatingManager:updateAllHeroScienceAttr()
            self:notifyObservers("onReceiveMemberChange")
        end
    end
end

-- 家族捐献
function This:onFamilyDonateRequest(donateType)
    if self:getDonateTimes() >= kFamilyDonateTimes then
        g_scene:showTips(Serialize["family_Manager_9"])
        return
    end 

    Socket:sendAndLock(12118, { type= donateType })
end

function This:onFamilyDonateReply( msg )
    if msg.err_code == 0 then
        self:setDonateTimes(msg.total_time)
        self.m_selfFamily:addDonationScore(msg.add_donation_score)
        local contributionList = self.m_selfFamily:addDonateReward(msg.type, FamilyBuildManager:getFamilyDonateAddition())
        g_scene:showReward(contributionList)
        self:notifyObservers("onFamilyDonate")
        SignalSystem.distributeSignal(Signals.FamilyPanelSacrifice)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 发送邮件
function This:onFamilySendMailRequest(title,content)
    Socket:sendAndLock(12126, { title = title,content= content })
end

function This:onFamilySendMailReply( msg )
    if msg.err_code == 0 then
        -- print(msg)
        self:notifyObservers("onFamilySendMail")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

--军团分红
function This:onFamilyBonusRequest()
    Socket:sendAndLock(12127)
end

function This:onFamilyBonusReply(msg)
    if msg.err_code == 0 then
        self.m_selfFamily:setBonusState(true)
        g_scene:showTips(Serialize[60966])
        self:notifyObservers("onReceiveBonus")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

--查看军团分红状态
function This:onFamilyBonusStateRequest()
    Socket:sendAndLock(12130)
end

function This:onFamilyBonusStateReply(msg)
    if msg.err_code == 0 then
        self.m_selfFamily:setBonusState(msg.is_reward)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetSendInheritRequest()
    Socket:sendAndLock(12132)
end

function This:onGetSendInheritReply( msg )
    if msg.err_code == 0 then
        
        self:notifyObservers("onGetSendInherit")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

--endregion 
