
-- @Author: superm
-- @Date:   2017-12-26 17:39:12
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-12-17 10:42:57
-- @File Name: FamilyManipulateView.lua
-- @Description: 
local FamilyManager = require("Family.FamilyManager")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")

local This, super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/FamilyAdministration.csb"

local FamilyPowerConfig = require("Metadatas.FamilyPowerConfig")
local ListLayer = require("UI.ListLayer")

local showNode ={
    change  =1, --更改职位
    renming =2, --任命军衔
}

function This:initData(family)
	self.m_family = family
    -- local w , h = self.bg:getContentSize()
    self:setContentSize(314,230)
    -- self:setToCenter()
    self:setAnchorPoint(0, 1)
end

function This:initNode()
    self.m_inited = true
end

function This:initTouchEvent()
    local function hideSelect()
        self.select:setVisible(false)
        self.select1:setVisible(false)
    end
    self.observe:setTouch({
        onEnded = function()
          hideSelect()
		  g_scene:openDialog("Player.RoleInfoPanel",{id = self.m_member:getId()})
          self:hide()
        end,
        isScale = false,
        isDynamic = true,
    })

    self.delete:setTouch({
        onEnded = function()
            hideSelect()
            local selfMember = self.m_family:getSelfMemberInfo()
            if selfMember:hasPower(FamilyPower.kKickOff, {target = self.m_member:getId()}) then
                g_scene:openDialog("UI.CostConfirmPanel", {
                    simple = Serialize[60968] .. self.m_member:getName() .. Serialize[60969], 
                    okHandler = function()
                        local ret = self.m_member:requestBeKickedOff()
                        if not ret then
                            g_scene:showTips(Serialize["family_Manipulate_1"])
                        end
                    end 
                })
            else
                g_scene:showTips(Serialize["family_Manipulate_2"])
            end
            self:hide()
        end,
        isScale = false,
        isDynamic = true,
    })

    self.change:setTouch({
        onEnded = function()
            hideSelect()
            self:updateScollView()
            self.select:setVisible(true)
        end,
        isScale = false,
        isDynamic = true,
    })

    self.renming:setTouch({
        onEnded = function()
            local ratingData = self.m_family:getRatingData()
			local rankScore = ratingData:getRankScoreModel()
            if not rankScore:checkIdPass(1) then
                g_scene:showTips(Serialize["family_Manipulate_9"])
                return
            end
            self:updateScollView()
            hideSelect()
            self.select1:setVisible(true)
        end,
        isScale = false,
        isDynamic = true,
    })
end


function This:updateScollView()
    local function updateScollView1(node,titleList,btnCallback)
        if node and titleList and btnCallback then
            node:removeAllItems()
             table.sort( titleList, function (infoA, infoB )
                -- if infoA.sequence ~= infoB.sequence then
                --     return infoA.sequence < infoB.sequence
                -- end
                return infoA.title < infoB.title
            end )
            for _, info in ipairs(titleList) do
                local level = info.title
                local name = info.name
                local bg = cc.Scale9Sprite:create("MenuScene/jiazexuanzeanniu.png")
                bg:setContentSize(150, 32)
                local text = cc.Label:createWithTTF("Fonts/Normal.ttf", "", AppText:isChinese() and 15 or 12)
                text:setPosition( 75 , 16)
                text:setStringWithColor(string.format("<New6>%s (<New9>%d<New6>/%d)",name,info.curentCount,info.allCount))
                bg:addChild(text)
                bg:setTouch({
                    onEnded = function()
                        btnCallback(level)
                    end,
                    -- onEnded = btnCallback,
                    isContainer = true,
                    isScale = false,
                })
                local w , h = bg:getContentSize()
                node:pushBackCustomItem(bg,w,h,0.5)
            end
            node:scrollToTop(0.3)
        end
    end
    local memeberList = self.m_family:getMemberList()
    local lv = self.m_family:getLevel()
    local militaryList = {}
    local titleList ={}
    for _,member in pairs(memeberList) do
        local title = member:getTitle()
        local titleName =member:getTitleDesc()
        if title>0 then
            local item1= titleList[title]
            if not item1 then
                titleList[title]={name = titleName,curentCount =1}
            else
                item1.curentCount  = item1.curentCount + 1
            end
        end
        local military = member:getMilitary()
        local militaryName = member:getMilitaryDesc()
        if military>0 then
            local item1= militaryList[military]
            if not item1 then
                militaryList[military]={name = militaryName,curentCount =1}
            else
                item1.curentCount  = item1.curentCount + 1
            end 
        end
    end
    local ratingData = self.m_family:getRatingData()
    local titleRes = ratingData:getTitleList(titleList,lv)
    local militaryRes = ratingData:getMilitaryList(militaryList)

    local militaryInfo = {}
    for _,member in ipairs(militaryRes) do
        local title = member.title
        militaryInfo[title]= member
    end
    self.m_militaryInfo = militaryInfo

    local function titleCallBack( level )
        self:appoint( level )
        --self:hide()
    end
    local function militaryCallback( title )
        self:militaryLevel( title )
        self:hide()
    end
    updateScollView1(self.select.scrollView,titleRes,titleCallBack)
    updateScollView1(self.select1.scrollView,militaryRes,militaryCallback)
end

function This:militaryLevel(level)
    local levelInfo = self.m_militaryInfo[level]
    if levelInfo then
        local selfMember = self.m_family:getSelfMemberInfo()
        if not selfMember:hasPower(FamilyPower.kAppointMilitary) then
            g_scene:showTips(Serialize["family_Manipulate_8"])
        elseif levelInfo.allCount == levelInfo.curentCount then
            g_scene:showTips(Serialize["family_Manipulate_7"])
        else
            FamilyRatingManager:onGetMilltaryRequest(self.m_member:getId(),level)
        end
    end
    -- print(level)
end

function This:appoint( title )
    local selfMember = self.m_family:getSelfMemberInfo()
    local ret = selfMember:hasPower(FamilyPower.kAppoint, {target = self.m_member:getId(), params = title})
	if ret then
        if title == FamilyTitle.kMengZhu then
            g_scene:openDialog("UI.CostConfirmPanel", {
                simple = Serialize["family_Manipulate_3"], 
                okHandler = function()
                    self.m_member:requestAppoint(title)
                    self.m_family:setDirty(true)
                end 
            })            
            return
        end

        if self.m_family:isReachTitleLimit(title) then
            g_scene:showTips(Serialize["family_Manipulate_4"])
            return
        end

		self.m_member:requestAppoint(title)
	else
        if ret == false then
            g_scene:showTips(Serialize["family_Manipulate_5"])
        else
		    g_scene:showTips(Serialize["family_Manipulate_6"])
        end
		self:hide()
	end
end

function This:refreshView()
	self.select:setVisible(false)
    self.select1:setVisible(false)
    self.change:setVisible(true)
    self.observe:setVisible(true)
    self.delete:setVisible(true)
    self.renming:setVisible(true)
end

function This:showViewBy( member, parent, pos)
    local oldvalue
    if self.m_member then
        oldvalue = self.m_member:getId()
    end
    self:refreshView()
    self.m_member = member
    local x, y = self:getFixScreenPosition(pos, parent:getBoundingBox(), 150, 210)
    self:stopAllActions()
    self:setVisible(true)
    self:setPosition(x, y)
    return oldvalue
end

function This:hide()
    local parent = self:getParent()
    if parent then
        parent:close()
    --else
    --    self:setVisible(false)
    end
end

function This:getFixScreenPosition(pos, rect, selfw, selfh)
    local worldx, worldy = pos.x, pos.y
    local x , y = worldx, worldy
    if worldx - selfw >= rect.x then
        x = worldx - selfw
    end
    

    if worldy - selfh <= rect.y then
        y = rect.y + selfh
    end
    
    return x , y
end