-- @Author: superm
-- @Date:   2017-12-13 21:29:19
-- @Last Modified by:   superm
-- @Last Modified time: 2020-10-30 15:12:26
-- @File Name: FamilyMemberModel.lua
-- @Description: 

local This , super = class(..., "Family.FamilyBaseModel")
local FamilyManager = require("Family.FamilyManager")
local FamilyPowerConfig = require("Metadatas.FamilyPowerConfig")
local LegionWarPowerConfig = require("Metadatas.LegionWarPowerConfig")
local GamePlayer = require("Player.GamePlayer")
local ContributeConfig = require("Metadatas.ContributeConfig")
local LegionMilitaryConfig = require("Metadatas.LegionMilitaryConfig")
local FamilyUtil = require("Family.FamilyUtil")

function This:doInit( info )
    self.m_id = info.id
    self.m_name = info.name

    self.m_title = info.title or 0  --职务
    self.m_rankLevel = info.rank_level
	self.m_icon = info.icon
    self.m_groupId = 0
    self.m_militaryId = info.military_rank or 0

    self.m_vitalty = info.active_num
    self.m_contribution = info.contribution or 0
    self.m_lastOnlineTime = info.last_offline_time or 0
    self.m_isOnline = info.on_line

    self.m_isMember = info.jifen ~= nil
    self.m_isRequest = not self.m_isMember
end

function This:updateInfo( info, updateView )
    super.updateBaseInfo(self, info)
    self.m_title = info.title or 0
    self.m_rankLevel = info.rank_level
    self.m_groupId = 0
    self.m_militaryId = info.military_rank or 0
    
    self.m_vitalty = info.active_num
    local contribution = self.m_contribution or info.contribution
    self.m_contribution = info.contribution or 0
    self.m_lastOnlineTime = info.last_offline_time or 0
    self.m_isOnline = info.on_line

	self.m_icon = info.icon
    self.m_isMember = info.jifen ~= nil
    self.m_isRequest = not self.m_isMember
    if updateView and self.m_view then
        self:notifyView("refresh")
    end
    return contribution ~= info.contribution
end

function This:getTitle()
    return self.m_title or FamilyTitle.kChengYuan
end

function This:getTitleDesc()
    local title = self:getTitle()
    if title > 0 then
        return FamilyPowerConfig[title].title
    end
    return Serialize["family_BuildNode_7"]
end

function This:setTitle(title)
	self.m_title = title or FamilyTitle.kChengYuan
end

--军衔
function This:setMilitary(militaryId,updateView)
    self.m_militaryId = militaryId
    if updateView and self.m_view then
        self:notifyView("refresh")
    end
end
function This:getMilitary()
    return self.m_militaryId or 0
end

--军衔
function This:getMilitaryDesc()
    local ratingId = self:getMilitary()
    if ratingId>0 then
        return LegionMilitaryConfig[ratingId].name
    end
    return Serialize["family_BuildNode_7"]
end

function This:getRankLv()
    return self.m_rankLevel
end

function This:getIcon()
	return self.m_icon
end

function This:getContribution()
    return self.m_contribution
end

function This:getVitality()
    return self.m_vitalty
end

function This:setVitality(vitalty, updateView)
    self.m_vitalty = vitalty
    if updateView and self.m_view then
        self:notifyView("refresh")
    end
end

function This:getLaseOnlineTime()
    if self.m_isOnline then
        return -1
    end
    return self.m_lastOnlineTime
end

function This:hasPower(powerType, info)
    if info and info.target == GamePlayer:getPlayerID() then
        if powerType == FamilyPower.kKickOff then
            return true
        elseif powerType == FamilyPower.kAppoint then
            return
        end
    end

    if self.m_title > 0 then
        local value = self:getMyPower(powerType)
        if value then
            local vtype = type(value)
            if vtype == "number" then
                return value > 0
            elseif vtype == "table" and info and info.params then
                return table.find(value, function ( _, v)
                    return v == info.params
                end)
            end
        else
            return false
        end
    end
end

function This:getMyPower(powerType, inWar)
    local powerConfig = nil
    if inWar then powerConfig = LegionWarPowerConfig[powerType] end
    
    if powerConfig then
        for _,powerValue in ipairs(powerConfig.power_list) do
            if powerValue == self.m_title then
                return powerValue
            end
        end
        return 0
    else
        return FamilyPowerConfig[self.m_title][powerType]
    end
end

function This:isMengZhu()
    return self.m_title == FamilyTitle.kMengZhu
end

function This:joinFamily( groupId )
    if not self.m_isMember and self.m_isRequest then
    	self.m_isRequest = false
        self.m_isMember = true
        self.m_groupId = groupId
    end
end

function This:donate( dtype )
    local dconfig = ContributeConfig[dtype]
    local contrb = dconfig.contribution
    self.m_contribution = self.m_contribution + contrb
    return dconfig.funds , contrb
end

---------------------------------------------------------------------
function This:requestApplyApproval( result )
    return FamilyManager:onApplyApprovalRequest(self:getId(), result and 1 or 2 )
end

function This:requestBeKickedOff()
    local ret = false
    if self:getId() ~= GamePlayer:getPlayerID() then
        FamilyManager:onMemberKickOffRequest(self:getId())
        ret = true
    end
    return ret
end

function This:requestAppoint( title )
    FamilyManager:onMemberAppointRequest(self:getId(), title)
end

function This:isOnline()
    return self.m_isOnline
end