-- @Author: superm
-- @Date:   2017-12-13 21:33:22
-- @File Name: FamilyModel.lua
-- @Description: 

local This , super = class(..., "Family.FamilySimpleModel")
local FamilyUtil = require("Family.FamilyUtil")
local FamilyList = require("Family.FamilyList")
local FamilyManager = require("Family.FamilyManager")
local FamilyMemberModel = require("Family.FamilyMemberModel")
local FamilySelectedSection = require("Family.FamilySelectedSection")
local GameRatingData = require("FamilyRating.GameRatingData")
local GameScienceData = require("Family.GameScienceData")
local GamePlayer = require("Player.GamePlayer")
local VitalityManager = require("Achieve.VitalityManager")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")
local EventDispatcher = require("EventDispatcher")

function This:create( ... )
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret    
end

function This:init( ... )
    super.init(self, ...)
    self.m_selSection = FamilySelectedSection:create()
    self.m_ratingData = GameRatingData:create(0)
    self.m_gameScienceData = GameScienceData:create(0)
end

function This:doInit( info )
    super.doInit(self, info)
    self.m_desc = info.desc
    self.m_autoJoin = info.auto_join

    self.m_funds = info.money or 0

    self.m_titleNumMap = {0,0,0,0,0}

    if info.member_list then
        self.m_leaderId = 0
        self.m_memberList = FamilyList:create()
        for _, memberInfo in ipairs(info.member_list) do
            self:addMember(memberInfo)
        end
    end

    if info.request_list then
        self.m_requestList = FamilyList:create()
        for _, requestInfo in ipairs(info.request_list) do
            local request = FamilyMemberModel:create(requestInfo)
            self.m_requestList:addNode(request, request:getId())
        end
    end

    if info.donation_score then
        self.m_donationScore=info.donation_score
        SignalSystem.distributeSignal(Signals.FamilyPanelSacrifice)
    end
end

function This:updateInfo(info )
    super.updateInfo( self, info )
    if info.member_list then
        self.m_titleNumMap = {0,0,0,0,0}
        self.m_leaderId = 0
        -- self.m_memberList:clear()

        self.m_memberList:updateAllNodeList(info.member_list, 
            function ( memberInfo )
                return memberInfo.id
            end,
            function ( id, member, memberInfo )
                if member then
                    member:updateInfo(memberInfo)
                else
                    member = FamilyMemberModel:create(memberInfo)
                end

                if member:isMengZhu() then
                    self.m_leaderId = id
                end
                self.m_memberList:addNode(member, id)
            end
        )
    end
    
    if info.money then
        self.m_funds = info.money
    end
    if info.desc then
        self.m_desc = info.desc
    end
    if info.auto_join ~= nil then
        self.m_autoJoin = info.auto_join    
    end
    if info.request_list then
        self.m_requestList:clear()
        for _, requestInfo in ipairs(info.request_list) do
            local request = FamilyMemberModel:create(requestInfo)
            self.m_requestList:addNode(request, request:getId())
        end
    end
    if info.rating then
        self.m_ratingData:init(info.rating)
    end
    if info.junwei then
       self.m_ratingData:setScore(info.junwei) 
    end
    if info.donation_score then
        self.m_donationScore=info.donation_score
        SignalSystem.distributeSignal(Signals.FamilyPanelSacrifice)
    end
end

function This:updateCondition( info )
    local ret = false
    super.updateCondition(self, info)
    if info.desc and string.len(info.desc) > 0 then
        if self.m_desc ~= info.desc then
            self.m_desc = info.desc
            ret = true
        end
    end
    if info.auto_join ~= nil then
        self.m_autoJoin = info.auto_join
    elseif info.auto then
        self.m_autoJoin = info.auto == 1
    end
    return ret
end

function This:setName( name )
    if name then
        self.m_name = name
    end
end

function This:setAutoJoin( value )
   self.m_autoJoin = value
end

function This:getAutoJoin()
   return self.m_autoJoin
end

function This:setDescription( value )
   self.m_desc = value
end

function This:getDescription()
   return self.m_desc
end

function This:getSetting(cond, autoJoin)
    local ret = super.getSetting(self, cond)
    local autoJoinValue = FamilyUtil:NVL(autoJoin, self.m_autoJoin)
    ret.auto = autoJoinValue and 1 or 2
    ret.desc = self.m_desc
    return ret
end

function This:hasRequest( roleId )
    if roleId then
        local node = self.m_requestList:getNode(roleId)
        return node ~= nil
    end
end

function This:removeRequest( roleId, agree )
    local node = self.m_requestList:getNode(roleId)
    local _, ret
    if node then
        _, ret = self.m_requestList:removeNode(roleId)
    end 

    if agree then
        node:joinFamily(self:getId())
        self.m_memberList:addNode(node, roleId)
    elseif ret == nil then
        return false
    end
    return true
end

function This:updateMemberInfo( memberInfo )
    local member = self.m_memberList:getNode(memberInfo.id)
    if not member then
        self:addMember(memberInfo)
        return true
    else
        return member:updateInfo(memberInfo, true)
    end
end

function This:addMember( memberInfo )
    local member = FamilyMemberModel:create(memberInfo)
    if member:isMengZhu() then
        self.m_leaderId = member:getId()
    end
    local title = member:getTitle()
    if title < 5 then
        local num = self.m_titleNumMap[title]
        self.m_titleNumMap[title] = num + 1
    end
    self.m_memberList:addNode(member, member:getId())  
end

function This:removeMember( roleId )
    if GamePlayer:getPlayerID() ~= roleId then
        if self.m_memberList then
            return self.m_memberList:removeNode(roleId)
        end 
    end
end

function This:getSelfMemberInfo() 
    if self.m_memberList then
        local roleId = GamePlayer:getPlayerID()
        return self.m_memberList:getNode(roleId)
    end
end

function This:getMyContribution()
    return GamePlayer:getContributionbi()
end

function This:setMemberTitle( roleId, title )
    local node = self.m_memberList:getNode(roleId)
    if node then
        node:setTitle(title)
        local view = node:getView()
        if view then
            view:refresh()
        end
    end 
    return node
end

-- 修改所有人的军衔并刷新ui
function This:setAllMemberMilitary(title)
    local list = self.m_memberList:getAllNodeList()
    for _,member in pairs(list) do
        if member then
            member:setMilitary(title,true)
        end
    end
end

-- 修改当个人的军衔
function This:setMemberMilitary( roleId, title )
    local node = self.m_memberList:getNode(roleId)
    if node then
        node:setMilitary(title)
        local view = node:getView()
        if view then
            view:refresh()
        end
    end 
    return node
end

function This:getLeaderName()
    if self.m_leaderId > 0 then
        local member = self.m_memberList:getNode(self.m_leaderId)
        if member then
            return member:getName()
        end
    end
    return ""
end

function This:getLeaderInfo()
    if self.m_leaderId > 0 then
        return self.m_memberList:getNode(self.m_leaderId)
    end
end

function This:getMemberList()
    local list = self.m_memberList:getAllNodeList()
    local ret = {}
    for k, v in ipairs(list) do
        ret[k] = v
    end

    table.sort( ret, function ( memberA, memberB )
        local titleA, titleB = memberA:getTitle(), memberB:getTitle() 
        if titleA ~= titleB then
            return titleA < titleB
        end
        local scoreA, scoreB = memberA:getScore(), memberB:getScore() 
        if scoreA ~= scoreB then
            return scoreA > scoreB
        end

        return memberA:getId() > memberB:getId()
    end)

    return ret
end

function This:getAllRequestInfo()
    return self.m_requestList:getAllNodeList()
end

function This:getFunds()
    return self.m_funds
end

function This:addDonationScore( score )
    self.m_donationScore = self.m_donationScore + score
    SignalSystem.distributeSignal(Signals.FamilyPanelSacrifice)
end

function This:getDonationScore()
    return self.m_donationScore
end

function This:costFunds( value )
    if self.m_funds >= value then
        self.m_funds =  self.m_funds - value
    end
end

function This:addDonateReward(dtype, addition)
    if not addition or addition <= 1.0 then
        addition = 1.0
    end

    local selfInfo = self:getSelfMemberInfo()
    if selfInfo then
        local funds, contrb = selfInfo:donate(dtype)
        funds = funds * addition
        self.m_funds =  self.m_funds + funds
        return {{126, funds}, {127, contrb}}
    end
end

function This:setLoong( value )
    self.m_loong = value or 0
end

function This:getSelectSection()
    return self.m_selSection
end

function This:getRatingData()
    return self.m_ratingData
end

function This:getScienceData()
    return self.m_gameScienceData
end

function This:updateRatingWithId(id)
    if id and self.m_ratingData then
        self.m_ratingData:init(id)
    end
end

function This:asynSelfVitality(vitality)
    local value = vitality or VitalityManager:getVitalityInFamilyMember()
    local selfInfo = self:getSelfMemberInfo()
    if selfInfo then
        selfInfo:setVitality(value, true)
    end
end

-- -- -- -- --
function This:requestChangeDesc( desc )
    local params = self:getSetting()
    params.desc = desc
    FamilyManager:onSubmitApplyFilterRequest(params)
end

function This:requestChangeRequestion( level, rankLev, autoJoin )
    local ret = {level or 0, rankLev or 0}
    local params = self:getSetting(ret, autoJoin)
    FamilyManager:onSubmitApplyFilterRequest(params)
end

function This:requestRename( name )
    FamilyManager:onFamilyRenameRequest(name)
end

function This:requestLeft()
    FamilyManager:onMemberKickOffRequest(GamePlayer:getPlayerID())
end

function This:requestUpgrade()
    FamilyManager:onFamilyUpgradeRequest()
end

function This:requestRefreshInfo()
    FamilyManager:onGetMyFamilyInfoRequest()
end

function This:setBonusState(state)
    self.m_bonus_state = state
    SignalSystem.distributeSignal(Signals.FamilyPanelAffairsTabRedDot)
    EventDispatcher:dispatcher("family_bonus_change")
end

function This:getBonusState()
   return self.m_bonus_state ~= false
end


