-- @Author: superm
-- @Date:   2019-09-12 14:56:55
-- @Last Modified by:   superm
-- @File Name: FamilyPlanPanel.lua
-- @Description:  

local This , super = class(..., "Dialog")
local GameUtil = require("Util.GameUtil")
local FamilyManager = require("Family.FamilyManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local FamilyPlanItemNode = require("Family.FamilyPlanItemNode")
local ArgConfig = require("Metadatas.ArgConfig")
local LegionWarPlanConfig = require("Metadatas.LegionWarPlanConfig")
local GamePlayer = require("Player.GamePlayer")

function This:initData()
	self.m_map = require("Map.GameMapManager"):getMap(GameMapType.kLegionMap)
	self.m_family = self.m_map:getMyLegion()

	self.m_startTime = self.m_map:getStartTime()
	self.m_silkBagInitialNum, self.m_silkBagLimitNum = unpack(ArgConfig["legion_war_silkbag_initial_max"].arg)
	self.m_silkBagTimeInterval, self.m_silkBagDeltaNum = unpack(ArgConfig["legion_war_silkbag_add"].arg)
	self.m_escape, self.m_sescape = 0, 0
	self.m_isUpdating = false
	self.m_silkBagNum = self.m_family:getSilkBagNum()
	self.m_itemList = {}
end

function This:initNode(info)
	local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyPlan.csb")
	csNode:setToCenter()
	self:addChild(csNode)
	self.m_csNode = csNode

	self:initPlanInfo()
	self:initPlanItemList()
end

function This:initPlanInfo()
    self.m_csNode.tip.tips:setString(string.format(Serialize["legion_war_silkbag_add"], self.m_silkBagTimeInterval, self.m_silkBagDeltaNum))
	self:refreshSilkBagNum()
	self:refreshSilkBagTime()
end

-- 倒计时
function This:refreshSilkBagTime()
	local node = self.m_csNode.tip
    local leftTime = (GamePlayer:getServerTime() - self.m_startTime) % self.m_silkBagTimeInterval
	node.timer:setString(GameUtil:formatTime(Serialize["H_M_S"], self.m_silkBagTimeInterval - leftTime))
end

-- 锦囊进度条
function This:refreshSilkBagNum()
	local num = self.m_family:getSilkBagNum()
	self.m_silkBagNum = math.min(num, self.m_silkBagLimitNum)
	self.m_isUpdating = self.m_silkBagNum < self.m_silkBagLimitNum

	local csNode = self.m_csNode
	local node = csNode.jindubg
	local tipNode = csNode.tip
	tipNode:setVisible(self.m_isUpdating)
	node.num:setString(self.m_silkBagNum .. "/" .. self.m_silkBagLimitNum)
	node.jindutiao:setPercent(self.m_silkBagNum / self.m_silkBagLimitNum * 100)
end

-- 计策列表
function This:initPlanItemList()
	local scrollView = self.m_csNode.bgxia.ScrollView
    scrollView:removeAllItems()

    local junceList = self.m_map:getJunCeList()
	table.sort(junceList,function(a,b)
		return a:getId() < b:getId()
	end)

	for i, v in ipairs(junceList) do 
		local item = self.m_itemList[i]
		if not item then
			item = FamilyPlanItemNode:create(v)
		else
			item:setData(v)
		end
		local w , h = item.bg:getContentSize()
		scrollView:pushNode(item, w, h,5)
		self.m_itemList[i] = item
	end
end

function This:onEnter()
    self.m_map:addObserver(self)
    super.onEnter(self)
    -- self.m_map:onGetJinNangInfoRequest()
end

function This:onExit()
    self.m_map:removeObserver(self)
    super.onExit(self)
end

function This:update(dt)
	super.update(self,dt)
	-- self.m_timer:update(dt)

	if not self.m_isUpdating then
		return
	end

    self.m_escape = self.m_escape + dt
    self.m_sescape = self.m_sescape + dt
    if self.m_escape >= 1 then
        self.m_escape = self.m_escape - 1
        self:refreshSilkBagTime()
    end

	if self.m_sescape >= self.m_silkBagTimeInterval then
		self.m_sescape = self.m_sescape - self.m_silkBagTimeInterval
        self:refreshSilkBagNum()
	end
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.closebutton:setTouchEnded(function()
		self:close()
	end)
end

function This:onGetFamilyInfoChange()
    if not FamilyManager:hasFamily() then
    	self:close()
    end
	self:refreshSilkBagNum()
end

function This:onGetJinNangInfo()
	self:refreshSilkBagNum()
end





