-- @Author: su
-- @Date:   2018-12-11 15:30:07
-- @Last Modified by: ZYY
-- @Last Modified time: 2018-12-15 14:14:26
-- @File Name: FamilyScienceResearchList.lua
-- @Desc: 

local This, super = class(...,"NodeUI")
This.fileName = "MenuScene/NodeUI/FamilyScienceResearchList.csb"
local LegionWarPowerConfig = require("Metadatas.LegionWarPowerConfig")
local TechnologyUpgradeConfig = require("Metadatas.TechnologyUpgradeConfig")
local FamilyManager = require("Family.FamilyManager")
local FamilyScienceItem = require("Family.FamilyScienceItem")
local FamilyUtil = require("Family.FamilyUtil")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")
local DepotManager = require("Depot.DepotManager")
local GameUtil = require("Util.GameUtil")




function This:initData(info)
    self.m_data = info.data
    self.m_id = info.data.m_config.id
    self.m_image =info.data.m_config.image
    self.m_family = info.family
end

function This:initNode()
    self.m_node = FamilyScienceItem:create({image=self.m_image})
    self.icon:addChild(self.m_node)
    self.skill:setString(self.m_data.m_config.text)
    self:refresh()
end

function This:refresh()
    local scienceData = FamilyRatingManager:getScienceData()
    self.m_view = scienceData:getChapter(self.m_id)
    local MilltaryLevel = self.m_view:getMilitaryLevel()
    local MilitaryLimitLevel = self.m_view:getMilitaryLimitLevel()
    local limitLevel = self.m_view:getLimitLevel()
    local level = self.m_view:getLevel()
    local nextLevel = self.m_view:getNextLevel()
    local levelCost,nextLevelCost=self.m_view:getLevelandNextCost()
    local nowCost,nextCost
    if levelCost == nil then
        nowCost = 0
    else
        nowCost = levelCost[3]
    end
    if nextLevelCost == nil then
        nextCost = nowCost
    else
        nextCost = nextLevelCost[3]
    end
    self.m_cost = nextCost
    self.level:setString(string.format( Serialize[60993],level))
    self.fullnum:setString(string.format( Serialize[60994],nextLevel))
    self.needlevel:setString(string.format( Serialize[60995],MilitaryLimitLevel))
    self.m_state = level >= limitLevel
     self.spend.num:setString(nextCost)
     self.spend:setVisible(level ~= nextLevel and not self.m_state)
     self.fullnum:setVisible(level ~= nextLevel)
     self.yimanji:setVisible(level == nextLevel)
     self.needlevel:setVisible(MilltaryLevel < MilitaryLimitLevel and level~=nextLevel and self.m_state )
     self.button:setVisible(level ~= nextLevel)
     if self.m_state then
         self.button:setGrayShader()
     end
     self.m_node:refresh(level ~= nextLevel and not self.m_state)

end

function This:onEnter()
    super.onEnter(self)
    FamilyManager:addObserver(self)
    FamilyRatingManager:addObserver(self)
end

function This:onExit()
    FamilyManager:removeObserver(self)
    FamilyRatingManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.button:setTouch({
        isScale = false, 
        isDynamic = true, 
        isContainer  = true,
        onEnded = function ()
        if self:hasPower() then
            if not self.m_state then
                local num = self.m_family:getFunds()
                if self.m_family:getFunds() >= self.m_cost then
                    FamilyRatingManager:onScienceResearchRequest(self.m_id)
                    cc.Audio:playEffect("Recruit_Change.mp3")
                    self.m_family:costFunds(self.m_cost)
                    if self.icon and not self.icon.effect then
                        local effect = GameUtil:createEffect("TiHuan2", false, nil,{
                            onAnimationComplete = function()
                                if self.icon.effect then
                                    self.icon.effect:removeFromParent()
                                    self.icon.effect = nil
                                end
                            end
                        })
                        self.icon.effect = effect
                        self.icon:addChild(effect)
                    end
                    if self.icon and not self.icon.effect1 then
                        local effect = GameUtil:createEffect("Task", false, nil,{
                            onAnimationComplete = function()
                                if self.icon.effect1 then
                                    self.icon.effect1:removeFromParent()
                                    self.icon.effect1 = nil
                                end
                            end
                        })
                        self.icon.effect1 = effect
                        self.icon:addChild(effect)
                        self.icon.effect1:setPosition(110,0)
                        self.icon.effect1:setScale(0.7)
                    end
                else
                    g_scene:showTips(string.format( Serialize[60996]))
                end
            end
        else
            g_scene:showTips(Serialize[60997])
        end
    end})
end

function This:onScienceResearch( id)
    if id == self.m_id then
        self:refresh()
    end
end

function This:hasPower()
    local title = FamilyManager:getMySelfTitle()
    for _,Tytle in ipairs(LegionWarPowerConfig["legion_technology_level_up"].power_list) do
        if title == Tytle then
            return true
        end
    end
    return false
end
