-- @Author: su
-- @Date:   2018-12-12 10:30:07
-- @Last Modified by: ZYY
-- @Last Modified time: 2018-12-15 10:58:14
-- @File Name: FamilyScienceUpListItem.lua
-- @Desc: 

local This , super = class(...,"NodeUI")
This.fileName = "MenuScene/NodeUI/FamilyScienceUpList.csb"
local FamilyScienceItem = require("Family.FamilyScienceItem")
local FamilyManager = require("Family.FamilyManager")
local ItemUtil = require("Bag.ItemUtil")
local GamePlayer = require("Player.GamePlayer")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")
local DepotManager = require("Depot.DepotManager")
local GameUtil = require("Util.GameUtil")





function This:initNode()
    self.m_node = FamilyScienceItem:create({image=self.m_image})
    self.icon:addChild(self.m_node)
    self.skill:setString(self.m_data.m_config.text)
    self:refresh()
end

function This:initData(info)
    self.m_data = info.data
    self.m_id = info.data.m_config.id
    self.m_image =info.data.m_config.image
    self.m_costType = info.data.m_config.upgrade_cost[1][2]
    self.m_level = GamePlayer:getLevel()
end

function This:refresh()
    local scienceData = FamilyRatingManager:getScienceData()
    self.m_stageMode = scienceData:getStage(self.m_id)
    self.m_chapter = scienceData:getChapter(self.m_id)
    self.m_limitLevel = self.m_stageMode:getLimitLevel()
    self.m_cost = self.m_stageMode:getNextCost()
    local level = self.m_stageMode:getLevel()
    local nextLevel = self.m_stageMode:getNextLevel()
    local scienceLeve = self.m_chapter:getLevel()
    self.m_state = level < scienceLeve
    local state = level == scienceLeve and level ~= nextLevel
    local attribute,nextAttribute = self.m_stageMode:getAttribute()
    local Text,nextText
    if attribute == nil then
        Text = string.format("%0.2f", 0)
    else
        Text = string.format("%0.2f", attribute[3]*100)
    end
    if nextAttribute == nil then
        nextText = Text
    else
        nextText = string.format("%0.2f", nextAttribute[3]*100)
    end
    self.skill:setString(self.m_stageMode:getName().."    Lv."..level.."/"..scienceLeve)
    self.level:setString(self.m_stageMode:getText()..":    +"..Text.."%")
    self.fullnum:setString(Serialize[61000]..nextText.."%")
    self.spend.num:setString(self.m_stageMode:getNextCost())
    self.need:setString(self:getMyText())

    self.spend.zijin:setImage(ItemUtil.getIconImage(self.m_stageMode:getIconID()))
    self.fullnum:setVisible(level ~= nextLevel)
    self.spend:setVisible(self.m_state and self.m_level >= self.m_limitLevel)
    self.need:setVisible( (not self.m_state or self.m_level < self.m_limitLevel) and level~=nextLevel)

    self.button:setVisible(level ~= nextLevel)
    self.yimanji:setVisible(level == nextLevel)
    if not self.m_state or self.m_level < self.m_limitLevel then
        self.button:setGrayShader()
    end
    self.m_node:refresh(self.m_state and self.m_level >= self.m_limitLevel)
end

function This:onEnter()
    super.onEnter(self)
    FamilyManager:addObserver(self)
    FamilyRatingManager:addObserver(self)
end

function This:onExit()
    FamilyManager:removeObserver(self)
    FamilyRatingManager:removeObserver(self)
    super.onExit(self)
end


function This:initTouchEvent()
    self.button:setTouch({
        isScale = false, 
        isDynamic = true, 
        isContainer  = true,
        onEnded = function ()
        if  self.m_state and self.m_level >= self.m_limitLevel then
            if DepotManager:isItemOrResEnough(self.m_costType,self.m_cost) then
                FamilyRatingManager:onScienceUpRequest(self.m_id)
                cc.Audio:playEffect("Recruit_Change.mp3")
                if self.icon and not self.icon.effect then
                    local effect = GameUtil:createEffect("TiHuan2", false, nil,{
                        onAnimationComplete = function()
                            if self.icon.effect then
                                self.icon.effect:removeFromParent()
                                self.icon.effect = nil
                            end
                        end
                    })
                    self.icon.effect = effect
                    self.icon:addChild(effect)
                end
                if self.icon and not self.icon.effect1 then
                    local effect = GameUtil:createEffect("Task", false, nil,{
                        onAnimationComplete = function()
                            if self.icon.effect1 then
                                self.icon.effect1:removeFromParent()
                                self.icon.effect1 = nil
                            end
                        end
                    })
                    self.icon.effect1 = effect
                    self.icon:addChild(effect)
                    self.icon.effect1:setPosition(110,0)
                    self.icon.effect1:setScale(0.7)

                end
            else
                g_scene:showTips(string.format( Serialize[60334],ItemUtil.getName(self.m_costType)))
            end
        end
    end})
end

function This:onScienceUp(id)
    if id == self.m_id then
        self:refresh()
    end
end

function This:getMyText()
    if not self.m_state then
        return Serialize[61001]
    end
    if self.m_level < self.m_limitLevel then
        return string.format( Serialize[61002],self.m_limitLevel)
    end
    return " "
end