-- @Author: superm
-- @Date:   2017-12-13 21:31:23
-- @Last Modified by:   superm
-- @Last Modified time: 2018-10-11 15:27:45
-- @File Name: FamilySimpleModel.lua
-- @Description: 

local This , super = class(..., "Family.FamilyBaseModel")
local FamilyUtil = require("Family.FamilyUtil")
local GamePlayer = require("Player.GamePlayer")
local MatchingConfig = require("Metadatas.MatchingConfig")
local MatchManager = require("Match.MatchManager")
local FamilyManager = require("Family.FamilyManager")
local FamilyLevelConfig = require("Metadatas.FamilyLevelConfig")

function This:create( ... )
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret    
end

function This:doInit( info )
	self.m_id = info.group_id
    self.m_name = info.group_name
    self.m_memberNum = info.member_num or 0

    if info.condition_type then
        self.m_conditionList = {}
        for index, ctype in ipairs(info.condition_type) do
        	self.m_conditionList[ctype] = info.condition_value[index]
        end
    end
end

function This:updateInfo( info )
	super.updateBaseInfo(self, info)
    self.m_name = info.group_name
    self.m_memberNum = info.member_num or 0
    self:updateCondition(info)
end

function This:updateCondition( info )
    if info.condition_type then
        self.m_conditionList = {}
        for index, ctype in ipairs(info.condition_type) do
            self.m_conditionList[ctype] = info.condition_value[index]
        end
    end
end

function This:getLevel()
    if not self.m_level then
        return 1
    elseif self.m_level > kFamilyMaxLevel then
        self.m_level = kFamilyMaxLevel
        return kFamilyMaxLevel
    end
    return self.m_level
end

function This:getMemberNum()
    return self.m_memberNum
end

function This:getMemberNumLimit()
    return FamilyUtil:getMemberNumLimitByLevel(self:getLevel())
end

function This:isFullMember()
    return self.m_memberNum <= self:getMemberNumLimit()
end

function This:isReachTitleLimit(title)
    if title >= FamilyTitle.kChengYuan then
        return false
    end

    local config = FamilyLevelConfig[self:getLevel()]
    if not config then
        return false
    end

    local numLimit = config.title_limit[title][2]
    if not numLimit then
        return false
    end 
    return self.m_titleNumMap[title] >= numLimit
end

function This:isMaxLevel()
    return self:getLevel() >= kFamilyMaxLevel
end

function This:getNextLevelCond()
    local level = self:getLevel() + 1
    if level > kFamilyMaxLevel then
        level = kFamilyMaxLevel 
    end
    return FamilyLevelConfig[level].type
end

function This:isLevelUpLocked()
    if self:isMaxLevel() then
        return true
    end
    return self:getScore() < self:getNextLevelCond()
end

function This:getMemberPeple()
    return tostring(self:getMemberNum()) .. "/" .. tostring(self:getMemberNumLimit())
end

function This:isQualified( member )
	local ret, stateCode = true, 0
	local value = self.m_conditionList[1] or 0
	if value > 0 then
		local role = member or GamePlayer
		if value > role:getLevel() then 
			ret = false
			stateCode = 1
		end
	end

	value = self.m_conditionList[2] or 0
	if value > 0 then
		local role = member or MatchManager
		if value > role:getRankLv() then 
			ret = false
			stateCode = stateCode + 2
		end
	end
    return ret, stateCode
end

function This:getConditionDesc()
    local desc, count = "", 0
    local value = self.m_conditionList[1] or 0
    if value > 0 then
        count = 1
    	desc = Serialize[61010] .. value
    end

    local value = self.m_conditionList[2] or 0
    if value > 0 then
        if count > 0 then
            desc = desc .. "  "
        end
    	desc = desc .. Serialize[61011] .. self:getRankName(value)
    end
    return desc
end

function This:getConditionValue(ctype)
    local ret = self.m_conditionList[ctype]
    if ret and ret > 0 then
        return ret
    end
end

function This:getSetting(cond)
    local conditionList = cond or self.m_conditionList 
    local ret = {condition_type={}, condition_value={}}
    for i, value in ipairs(conditionList) do
        if value > 0 then
            table.insert(ret.condition_type, i)
            table.insert(ret.condition_value, value)
        end
    end
    return ret
end

function This:getRankName(rank)
	if not rank or rank < 1 then
		rank = 1
	elseif rank >= kMaxMatchRankLv then
		rank = kMaxMatchRankLv
	end
	return MatchingConfig[rank].rank_level
end

function This:isInMyRequestList()
    return FamilyManager:isInRequestList(self:getId())
end

function This:requestSubmitApplying()
    return FamilyManager:onSubmitApplyingRequest(self:getId())
end