-- @Author: superm
-- @Date:   2018-07-03 20:25:08
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-28 16:00:00
-- @File Name: FamilyStoreNode.lua
-- @Description: 

local This , super = class(...,"NodeUI")
This.fileName = "MenuScene/NodeUI/FamilyStore.csb"

local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local StoreManager = require("Store.StoreManager")
local FamilyManager = require("Family.FamilyManager")
local StoreFamilyUI = require("Store.StoreFamilyUI")

local storeTypeList = {
    StoreType.kFamilyContribution, 
    StoreType.kFamilyShare,
    StoreType.kFamilyLevel,
    StoreType.kFamilyFight
}

function This:initData(info)
    if info then
       self.m_family = info.family
    end
end

function This:initNode()
    local default = storeTypeList[1]
    for i, storeType in ipairs(storeTypeList) do
        self:addTabButton(tostring(storeType), self["tep" .. i], function()
            if self.m_curTab == storeType then
                return false
            end
            self.m_curTab = storeType
            self:selectTabByType( self.m_curTab )
            return true
        end)
    end

    self:selectTabButton(tostring(default))
end

function This:cleanUp()
end

function This:refresh(check)
    if check and self.m_store then
        self.m_store:updateState()
    end
end

function This:onEnter()
    super.onEnter(self)
    StoreManager:addObserver(self)
    DepotManager:addObserver(self)
end

function This:onExit()
    self:cleanUp()
    DepotManager:removeObserver(self)
    StoreManager:removeObserver(self)
    super.onExit(self)
end

function This:selectTabByType( shopType )
    if not shopType then
    	shopType = storeTypeList[1]
    end
    self.m_store = StoreManager:getStoreBy(shopType)
    if self.m_store then
        self.m_store:updateState()
        local node = self:openTabNode(tostring(shopType))
        if node then
            node:refresh()
        end
    else
        StoreManager:onGetStoreInfoRequest(shopType)
    end
end

function This:createTabNode(name)
    local tabNode = StoreFamilyUI:create(self.m_store)
    return tabNode
end

function This:onGetFamilyStoreInfo()
    self:selectTabByType( self.m_curTab )
end

function This:itemUpdate(itemList)
    if itemList and #itemList > 0 then
        g_scene:showReward(itemList)
    end
end

