-- @Author: superm
-- @Date:   2018-07-04 15:23:49
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-05 17:45:31
-- @File Name: FamilyTaskNode.lua
-- @Description: 

local This , super = class(...,"NodeView")
This.fileName = "MenuScene/NodeUI/FamilyActive.csb"

local RankPageListLayer = require("Rank.RankPageListLayer")
local VitalityManager = require("Achieve.VitalityManager")
local LegionTaskBoxConfig = require("Metadatas.LegionTaskBoxConfig")
local FamilyAchieveView = require("Family.FamilyAchieveView")
local GameUtil = require("Util.GameUtil")

function This:initData(info)
    if info then
       self.m_family = info.family
    end
end

function This:initNode()
	self.m_curViews = {}
	self.m_maxVitality = LegionTaskBoxConfig[kDailyBoxNum + kWeeklyBoxNum].vitality
	self.m_targetVitalityList = {}
	self:refreshVitality()
	self:initBoxInfo()
end

function This:onEnter()
	super.onEnter(self)
	VitalityManager:addObserver(self)
end

function This:onExit()
	if self.m_curViews then
		for _, view in pairs(self.m_curViews) do
			if view and view[0] ~= 0 then
				view:release()
			end
		end
	end
	VitalityManager:removeObserver(self)
	super.onExit(self)
end

function This:refreshVitality()
	self.m_familyVitality = VitalityManager:getVitalityInFamily()
	self.m_memberVitality = VitalityManager:getVitalityInFamilyMember()
	self.xialan.number:setString(self.m_memberVitality)
	self.jthuoyuenumber:setString(self.m_familyVitality)
	self.shanglan.jindutiao:setPercent(self.m_familyVitality / self.m_maxVitality * 100)
    if self.m_family then
    	self.m_family:asynSelfVitality(self.m_memberVitality)
    end
end

function This:refreshBoxView(node, state, quality)
    if node then
    	if state == false then
    		local icon = string.format("MenuScene/boxGet%d.png", quality)
			node:setGrayShader()
			node:setImage(icon)
			if node.effectLing then
				node.effectLing:removeFromParent()
				node.effectLing = nil
			end
			if node.effectLing2 then
				node.effectLing2:removeFromParent()
				node.effectLing2 = nil
			end
    	elseif state == true then
    		local icon = string.format("MenuScene/boxGet%d.png", quality)
            node:clearShader()
			node:setImage(icon)
			local w ,h = node:getContentSize()
			if not node.effectLing then
				local effect = GameUtil:createEffect("BoxLing",true)
				effect:setPosition(w/2, h/2)
				node:addChild(effect, -1)
				node.effectLing = effect
			end
			if not node.effectLing2 then
				local effect = GameUtil:createEffect("BoxLing2",true)
				effect:setPosition(w/2, h/2)
				node:addChild(effect, -1)
				node.effectLing2 = effect
			end
    	else
    		local icon = string.format("MenuScene/boxLock%d.png", quality) 
            node:clearShader()
			node:setImage(icon)
    	end
    end
end

function This:initBoxInfo()
	local posx = self.shanglan.jindutiao:getPositionX()
	local width = 666
	local snode = self.shanglan
	for i = 1 , kWeeklyBoxNum do

		local vitality = LegionTaskBoxConfig[kDailyBoxNum + i].vitality
		self.m_targetVitalityList[kDailyBoxNum + i] = vitality
		snode["number"..i]:setString(tostring(vitality))
		local curPos = posx + width * vitality / self.m_maxVitality
		snode["boxLock" .. i]:setPositionX(curPos)
		snode["number"..i]:setPositionX(curPos)
		snode["keduxian" .. i]:setPositionX(curPos)
	end

	for i=1, kDailyBoxNum do 
		local config = LegionTaskBoxConfig[i]
		if config then
			self.m_targetVitalityList[i] = config.vitality
			self.xialan["kelingqu" .. i]:setString(tostring(config.vitality) .. Serialize[61015])
		end
	end
end


function This:initTouchEvent()
	for i = 1 , kWeeklyBoxNum do
		local index = kDailyBoxNum + i
		self.shanglan["boxLock"..i]:setTouch({
			onEnded = function()
				local vitality = VitalityManager:getVitalityInFamily()
				g_scene:openDialog("Family.FamilyAchieveBoxPanel", {
					type = VitalityBoxType.kFamily, 
					vitality = vitality, 
					index = index, 
					config = LegionTaskBoxConfig[index]
				})
			end,
			isScale = false,
		})
	end

	for i = 1 , kDailyBoxNum do
		local index = i
		self.xialan["dailybox" .. i]:setTouch({
			onEnded = function()
				local vitality = VitalityManager:getVitalityInFamilyMember()
				g_scene:openDialog("Family.FamilyAchieveBoxPanel", {
					type = VitalityBoxType.kMember, 
					vitality = vitality, 
					index = index, 
					config = LegionTaskBoxConfig[index]
				})
			end,
			isScale = false,
		})
	end

	self.help:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text = Serialize["family_achieve_help"],node = self.help})
	end)
end

function This:refreshItem()
	self.ScrollView:removeAllItems()
	self:removeAllTimer()
	local achieveList = VitalityManager:getAchieveWithSort()
	for i , achieve in ipairs(achieveList) do
		self:setTimer(i * 0.1, function()
			local id = achieve:getId()
			local view = self.m_curViews[id]
			if not view then
				view = FamilyAchieveView:create(achieve)
				self.m_curViews[id] = view
				view:setGotoHandler(function (ac)
				    if ac:hasGuider() then
						local panel = g_scene:getDialog("FamilyPanel")
						if panel then
							panel:close()
						end
						ac:triggerAncGuider()
				    else
				        g_scene:openDialog("UI.ConfirmPanel", {richText = ac:getWarming()})
				    end
				end)
				view:retain()
			else
				view:refresh()
			end
			local w , h = view:getContentSize()
			self.ScrollView:pushBackCustomItem(view, w, h)
		end)
	end

end

function This:refresh()
	self:refreshVitality()
	local index = 1
	for i = 1 , kWeeklyBoxNum do
		index = i + kDailyBoxNum
		local node = self.shanglan["boxLock"..i]
		local state 
		if VitalityManager:isBoxReward(index) then
			state = false
		elseif self.m_familyVitality >= self.m_targetVitalityList[index] then
			state = true
		end
		self:refreshBoxView(node, state, i)
	end

	for i = 1 , kDailyBoxNum do
		index = i
		local node = self.xialan["dailybox"..i]
		local state 
		if VitalityManager:isDailyBoxReward(index) then
			state = false
		elseif self.m_memberVitality >= self.m_targetVitalityList[index] then
			state = true
		end
		self:refreshBoxView(node, state, 4)
	end

end

function This:onGetAchieveInfo(achieveType)
    if achieveType == AchieveType.kFamilyActive then
	    self:refresh()
	    self:refreshItem()
    end
end

function This:onGetFamilyBoxReward(index, boxType)
	self:refresh()
	self:refreshItem()
end

function This:onGetAchieveReward(achieve)
    if achieve and achieve:getType() == AchieveType.kFamilyActive then
	    self:refresh()
    end 
end
