-- @Author: zheng
-- @Date:   2018-10-20 15:22:20
-- @Last Modified by: ZYY
-- @Last Modified time: 2018-12-15 15:19:57
-- @File Name: GameScienceData.lua

local TechnologyUpgradeConfig = require("Metadatas.TechnologyUpgradeConfig")
local TechnologyAdditionConfig = require("Metadatas.TechnologyAdditionConfig")
local TechnologyStructureConfig = require("Metadatas.TechnologyStructureConfig")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local GameResearchChapter = require("Family.GameResearchChapter")
local GameResearchStage = require("Family.GameResearchStage")

local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

FamilyScienceDataType ={
    kRole   = 1,
    kFamily = 2,
}

FamilyScienceType = {
    kBu = 1,
    kQi = 2,
    kGong = 3,
    kChe = 4,
}

local This = class(...)

function This:create(id)
    local Morality = {}
    setmetatable(Morality,This)
    Morality:init()
    return Morality
end

function This:init(id)
	local chapterItems ={}
	local stageItems ={}
	for index,configData in ipairs(TechnologyStructureConfig) do
		local subordinate = configData.subordinate
		for _,stageId in ipairs(subordinate) do
			local chapterData = GameResearchChapter:create(stageId)
			local stageData = GameResearchStage:create(stageId)
			chapterItems[stageId]=chapterData
			stageItems[stageId] = stageData
		end
	end
	self.m_chapterItems=chapterItems
	self.m_stageItems=stageItems
	local num =TechnologyAdditionConfig[1].attribute
	self.m_attributeNum = table.count(num)
end

function This:getChapter( id )
	return self.m_chapterItems[id]
end

function This:getAllChapter()
	return self.m_chapterItems
end

function This:getStage( id )
	return self.m_stageItems[id]
end

function This:getAllStage()
	return self.m_stageItems
end

function This:getAddtiionId( id )
	local stage=self:getStage(id)
	local additionId = stage:getAddtiionId()
	return additionId
end

function This:getAttr(id)
	if self.m_chapterItems[id]:getLevel() < self.m_stageItems[id]:getLevel() then
		return self.m_chapterItems[id]
	else
		return self.m_stageItems[id]
	end

end

function This:resetAddtion()
	for _,tab in pairs(FamilyScienceTab) do
		local idList = self:getScienceIds(tab)
		for _,id in pairs(idList) do
			local chapter = self:getChapter(id)
			chapter:resetAddtion()
		end
	end
end

--根据科技类型获取对应科技id
function This:getScienceIds(scienceType)
    local item1 = TechnologyStructureConfig[scienceType]
    if item1 then
        return item1.subordinate
    end
    return nil
end

function This:setChapterLevel(group)
	for _,data in ipairs(group) do
		local groupData = self:getChapter(data.id)
		groupData:setLevel(data.level)
		groupData:setType(data.main_type)
	end
end

function This:updateChapterLevel(id,level)
	local groupData = self:getChapter(id)
	groupData:setLevel(level)
end

function This:setStageLevel( group )
	for _,data in ipairs(group) do
		local roleData = self:getStage(data.id)
		roleData:setLevel(data.level)
		roleData:setType(data.main_type)
	end
end

function This:updateStageLevel(id,level)
	local roleData = self:getStage(id)
	roleData:setLevel(level)
end

function This:getMinLevel( group )
	local level
	for _,item in pairs(group) do
		if not level or level > item:getLevel() then
			level = item :getLevel()
		end
	end
	return level
end

function This:getScienceLevelByType( mType)
	local res= {}
	local idList = self:getScienceIds(mType)
	for _,id in ipairs(idList) do
		table.insert(res,self:getStage(id) )
	end
	return self:getMinLevel(res)
end

function This:getScienceLimitLevelByAddLevel( level)
	local num = TechnologyAdditionConfig[level].condition[1][2]
	return num
end

-- 获取当前增幅等级
function This:getScienceAddLevel()
	local groupAddLevel = self:getMinLevel(self.m_chapterItems)
	local roleAddLevel = self:getMinLevel(self.m_stageItems)
	if roleAddLevel > groupAddLevel then
		roleAddLevel = groupAddLevel
	end
	local addLevel = 0
	for index,data in ipairs(TechnologyAdditionConfig) do
		if roleAddLevel >= data.condition[1][2] then
			addLevel = index
		end
	end
	return addLevel
end

function This:getScienceNextAddLevel( )
	local level = self:getScienceAddLevel()
	local nextLevel = level
	if level < table.count(TechnologyAdditionConfig) then
		nextLevel = level + 1
	end
	return nextLevel
end

function This:getScienceAddState()
    local state ,text
    if self:getScienceAddLevel() == #TechnologyAdditionConfig then
        state = true
        text = Serialize[61020]
    else
        state = false
        text = Serialize[61021]
    end
    return state,text
end

function This:getAttributeByIndex(index,level)
	local text,num,attrribute
	--local addLevel = self:getScienceAddLevel()
	local data = TechnologyAdditionConfig[level]
	if not data then
		data = TechnologyAdditionConfig[1]
	end
	attrribute =  data.attribute[index]
	if attrribute then
		text,num = GameUtil:getAttrStr(attrribute[1],attrribute[2])
	else
		text,num = " "," "
	end
	return text,num
end

function This:getScienceAddAddtion()
	local level = self:getScienceAddLevel()
	local addition = {}
	if level ~=0 then
		addition = TechnologyAdditionConfig[level].attribute
		addition = self:tableFormat(addition)
		return addition
	else
		addition = TechnologyAdditionConfig[1].attribute
		addition = self:tableFormat(addition)
		for index,data in pairs(addition) do
			addition[index] = 0
		end
	end
	return addition
end


function This:tableFormat(list)
	local res = {}
	for _,data in pairs(list) do
		res[data[1]] = data[2]
	end
	return  res
end

function This:getAttributeNum( )
	return self.m_attributeNum 
end

-- 两个表相加
function This:scienceAddTionTableAdd(scienceAttrList,scienceAddAttrList)
	for _,data in pairs(scienceAttrList) do
		for index,list in pairs(scienceAddAttrList) do
			if data[index] then
				data[index] = data[index] + scienceAddAttrList[index]
			else
				data[index] = scienceAddAttrList[index]
			end
		end
	end
	return scienceAttrList
end