

local This, super = class(..., "Dialog")
local FamilyBossManager = require("FamilyBoss.FamilyBossManager")
local LegionBossRankConfig = require("Metadatas.LegionBossRankConfig")
local PersonalBossRankConfig = require("Metadatas.PersonalBossRankConfig")
local ItemNode = require("UI.ItemNode")


function This:initData(info)
	
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyBossRewardList.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    self:refresh()
end

function This:initTouchEvent()
	local csNode = self.m_csNode

	csNode.closebutton:setTouchEnded(function ()
		self:close()
	end)

	csNode.lingQu:setTouchEnded(function ()
		local chapter = FamilyBossManager:getChapter()
		if not chapter:isExistReward() then
			return
		end

		if chapter:isLingQu() then
			g_scene:showTips(Serialize[61033])
		else
			FamilyBossManager:onFamilyBossRewardRequest(chapter:getId())
		end
	end)
end

function This:onEnter()
	super.onEnter(self)
	FamilyBossManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	FamilyBossManager:removeObserver(self)
end

function This:refresh()
	local csNode = self.m_csNode
	local chapter = FamilyBossManager:getChapter()

	local setReward = function (node, rank, reward, emptyText)
		node.rank:setVisible(false)
		node.top:setVisible(false)
		node.juntuantext:setVisible(false)

		if type(reward) ~= "table" or #reward == 0 then	
			node.juntuantext:setVisible(true)
			node.juntuantext:setString(emptyText)
			return
		end

		if type(rank) == "number" and rank > 0 then
			if rank >= 1 and rank <= 3 then
				node.top:setVisible(true)
				node.top:setImage(string.format("MenuScene/top%d.png", rank))
			else
				node.rank:setVisible(true)
				node.rank:setString(string.format(Serialize[60698], rank))
			end
		end

		for i, v in ipairs(reward) do
			local posNode = node["item"..i]
			if posNode then
				local id, num = v[1], v[2]
				local item = ItemNode:create(id, num)
		    	item:setAnchorPoint(0.5, 0.5)
		    	item:setScale(0.7)
		    	item:setAnchorPoint(0.5, 0.5)
		    	item:setPosition(posNode:getPosition())
		    	node:addChild(item)
		    end
		end
	end

	setReward(csNode.item2, nil, chapter:getChallengeReward(), Serialize[61034])
	setReward(csNode.item1, chapter:getLegionRank(), chapter:getLegionRankReward(), Serialize[61035])
	setReward(csNode.item3, chapter:getRoleRank(), chapter:getRoleRankReward(), Serialize[61036])


	local isReward = chapter:isExistReward()
	csNode.lingQu:setVisible(isReward)

	if isReward then
		if chapter:isLingQu() then
			csNode.lingQu.text:setString(Serialize[61037])
			csNode.lingQu:setGrayShader()
		else
			csNode.lingQu.text:setString(Serialize[61038])
			csNode.lingQu:clearShader()
		end
	end

end

function This:onFamilyBossChange()
	self:close()
end