

local This , super = class(..., "Dialog")
local FamilyBossManager = require("FamilyBoss.FamilyBossManager")
local FamilyManager = require("Family.FamilyManager")
local GameUtil = require("Util.GameUtil")


local StateBtnType = {
	kNone = "",
	kOpen = "open",
	kEnter = "enter",
	kReward = "reward"
}


function This:initData(info)
	self.m_escape = 0
	FamilyBossManager:onFamilyBossInfoRequest()
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/LegionBoss.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    self:refresh()
end

function This:initTouchEvent()
	local csNode = self.m_csNode

	csNode.closebutton:setTouchEnded(function()
		self:close()
	end)

	csNode.btnRank:setTouchEnded(function()
		g_scene:openDialog("FamilyBoss.FamilyBossRankPanel")
	end)

	csNode.btnState:setTouchEnded(function()
		local info = self:getStateBtnInfo()
		local chapter = FamilyBossManager:getChapter()

		if info.btnType == StateBtnType.kOpen then
			-- 开启
			if info.phaseType == FamilyBossPhaseType.kPreview then
				g_scene:showTips( string.format(Serialize[61042], chapter:getOpenTimeStr()) )
			else
				if chapter:isHasPowerOpen() then
					local family = FamilyManager:getMyFamily()
					local funds = family:getFunds()
					local cost = chapter:getOpenFunds()
					g_scene:openDialog("UI.ConfirmPanel",{
						text = string.format(Serialize[61043],cost),
						okHandler = function()
							if funds >= cost then
								FamilyBossManager:onFamilyBossOpenRequest(chapter:getId())
							else
								g_scene:showTips(Serialize[61044])
							end
						end
					})
				else
					g_scene:showTips(Serialize[61045])
				end
			end

		elseif info.btnType == StateBtnType.kEnter then
			-- 进入
			if chapter:getRemainChallengeNum() <= 0 then
				g_scene:showTips(Serialize[61046])
				return
			end
			g_scene:openDialog("FamilyBoss.FamilyBossEmbattlePanel", {
				onConfirm = function (heroList)
					FamilyBossManager:onFamilyBossChallengeRequest(heroList)
				end
			})

		elseif info.btnType == StateBtnType.kReward then
			-- 我的奖励
			g_scene:openDialog("FamilyBoss.FamilyBossMyRewardPanel")

		end
	end)

	csNode.help:setTouchEnded(function()
		g_scene:openDialog("FamilyBoss.FamilyBossHelpPanel")
	end)

	csNode.box:setTouchEnded(function()
		g_scene:openDialog("FamilyBoss.FamilyBossRewardPreviewPanel")
	end)
end

function This:onEnter()
	super.onEnter(self)
	FamilyBossManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	FamilyBossManager:removeObserver(self)
end

function This:refresh()
	local csNode = self.m_csNode
	local chapter = FamilyBossManager:getChapter()
	local phase = chapter:getPhase()

	-- 副本名
	csNode.dungeonbg.name:setString( chapter:getName() )

	-- 描述图
	csNode.desc:setImage( chapter:getDescImage() )
	
	-- boss名
	csNode.name:setString( chapter:getBossName(), AppText:isChinese() and 30 or nil)

	-- 特点
	csNode.info.text:setString( chapter:getBossDesc() )

	-- boss图片
	csNode.bossimg:setImage( chapter:getBossImage() )

	-- 按钮(开启/进入/我的奖励)
	local btnState = csNode.btnState
	local btnInfo = self:getStateBtnInfo()
	btnState:setVisible(btnInfo.isVisible)
	btnState.text:setString(btnInfo.text)
	if btnInfo.isGray then
		btnState:setGrayShader()
	else
		btnState:clearShader()
	end
	GameUtil:refreshRedDot(btnState, btnInfo.isRedDot) 

	-- 未参与
	if phase.type == FamilyBossPhaseType.kReward and chapter:getChallengeNum() <= 0 then
		csNode.lbState:setString(Serialize[61047])
		csNode.lbState:setVisible(true)
	else
		csNode.lbState:setVisible(false)
	end

	-- 挑战次数
	if phase.type == FamilyBossPhaseType.kEnter and chapter:isOpen() then
		local maxNum = chapter:getMaxChallengeNum()
		local remainNum = chapter:getRemainChallengeNum()
		local color = (remainNum <= 0) and "<New11>" or "<A0>"
		local str = string.format(Serialize[61048], color, remainNum, maxNum)
		csNode.lbChallengeNum:setStringWithColor(str)
	else
		csNode.lbChallengeNum:setString("")
	end
	
	self:refreshTimeLabel()
end

function This:refreshTimeLabel()
	local csNode = self.m_csNode

	-- 时间
	local lbTime = csNode.lbTime
	local chapter = FamilyBossManager:getChapter()
	local phase = chapter:getPhase()

	if phase.type == FamilyBossPhaseType.kNotStart then
		lbTime:setString( string.format(Serialize[61049], chapter:getOpenWeekStr(), chapter:getOpenTimeStr()) )

	elseif phase.type == FamilyBossPhaseType.kPreview then
		lbTime:setString( string.format(Serialize[61042], chapter:getOpenTimeStr()) )

	elseif phase.type == FamilyBossPhaseType.kEnter then
		if chapter:isOpen() then
			lbTime:setString( GameUtil:formatTime(Serialize[61050], chapter:getRemainEnterTime()) )
		else
			lbTime:setString("")
		end

	elseif phase.type == FamilyBossPhaseType.kReward then
		lbTime:setString(Serialize[61051])

	else
		lbTime:setString("")

	end
end

function This:getStateBtnInfo()
	local chapter = FamilyBossManager:getChapter()
	local phaseType = chapter:getPhase().type

	local ret = {
		phaseType = phaseType,
		btnType = StateBtnType.kNone,
		text = "",
		isVisible = false,
		isGray = false,
		isRedDot = false,
	}

	if phaseType == FamilyBossPhaseType.kPreview then
		ret.btnType = StateBtnType.kOpen
		ret.text = Serialize[61052]
		ret.isVisible = true
		ret.isGray = true
		
	elseif phaseType == FamilyBossPhaseType.kEnter then
		if chapter:isOpen() then
			ret.btnType = StateBtnType.kEnter
			ret.text = Serialize[61053]
			ret.isVisible = true
			ret.isGray = chapter:getRemainChallengeNum() <= 0
		else
			ret.btnType = StateBtnType.kOpen
			ret.text = Serialize[61052]
			ret.isVisible = true
			ret.isGray = not chapter:isHasPowerOpen()
		end

	elseif phaseType == FamilyBossPhaseType.kReward then
		ret.btnType = StateBtnType.kReward
		ret.text = Serialize[61054]
		ret.isVisible = (chapter:getChallengeNum() > 0)
		ret.isRedDot = chapter:isExistReward() and not chapter:isLingQu()

	end

	return ret
end

function This:update(dt)
	super.update(self, dt)

	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1 then
		self.m_escape = 0
		self:refreshTimeLabel()
	end
end

function This:onFamilyBossChange()
	self:refresh()
end

