

local This , super = class(...,"NodeUI")


function This:initData(data)
    self.m_data = data
end

function This:initNode(data)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/LegionBossRankItem.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local w, h = csNode.bg:getContentSize()
    self:setContentSize(w, h)
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:setModel(data)
    self.m_data = data
    self:refresh()
end

function This:refresh()
    local data = self.m_data
    local csNode = self.m_csNode

    if not data then
        csNode.role:setVisible(false)
        csNode.legion:setVisible(false)
        return
    end

    csNode.role:setVisible(data.rankType == FamilyBossRankType.kMember)
    csNode.legion:setVisible(data.rankType == FamilyBossRankType.kFamily)

    if data.rankType == FamilyBossRankType.kFamily then
        self:refreshLegionRankNode(csNode.legion, data)
    elseif data.rankType == FamilyBossRankType.kMember then
        self:refreshRoleRankNode(csNode.role, data)
    end
end

function This:refreshLegionRankNode(node, info)
    node.name:setString(info.name)
    node.damage:setString(info.damage)
    node.people:setString(info.people)

    if info.rank <= 0 then
        node.top:setVisible(false)
        node.rank:setVisible(false)
        node.weishangbang:setVisible(true)
        node.people:setVisible(info.damage > 0)
        node.weicanyu:setVisible(info.damage <= 0)
    else
        if info.rank >= 1 and info.rank <= 3 then
            node.top:setVisible(true)
            node.top:setImage(string.format("MenuScene/top%d.png", info.rank))
            node.rank:setVisible(false)
        else
            node.top:setVisible(false)
            node.rank:setVisible(true)
            node.rank:setString(info.rank)
        end
        node.weishangbang:setVisible(false)
        node.weicanyu:setVisible(false)
    end    
end

function This:refreshRoleRankNode(node, info)
    node.name:setString(info.name)
    node.damage:setString(info.damage)  

    if info.rank <= 0 then
        node.top:setVisible(false)
        node.rank:setVisible(false)
        node.weishangbang:setVisible(true)
        node.damage:setVisible(info.damage > 0)  
        node.weicanyu:setVisible(info.damage <= 0)
    else
        if info.rank >= 1 and info.rank <= 3 then
            node.top:setVisible(true)
            node.top:setImage(string.format("MenuScene/top%d.png", info.rank))
            node.rank:setVisible(false)
        else
            node.top:setVisible(false)
            node.rank:setVisible(true)
            node.rank:setString(info.rank)
        end
        node.weishangbang:setVisible(false)
        node.weicanyu:setVisible(false)
    end
end