

local This , super = class(..., "Dialog")
local FamilyBossManager = require("FamilyBoss.FamilyBossManager")
local FamilyBossRankItemView = require("FamilyBoss.FamilyBossRankItemView")
local RankPageListLayer = require("Rank.RankPageListLayer")


function This:initData(info)
	self.m_rankList = {}
	self.m_curType = 0
	self.m_request = {}
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyRankingList.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode


    self:addTabButton(FamilyBossRankType.kFamily, csNode.tabs.legionlist, function()
    	return self:setRankType(FamilyBossRankType.kFamily)
    end)

    self:addTabButton(FamilyBossRankType.kMember, csNode.tabs.personlist, function()
    	return self:setRankType(FamilyBossRankType.kMember)
    end)

    local node = FamilyBossRankItemView:create()
    csNode.my:addChild(node)
    csNode.my.item = node

    local scrollView = csNode.scrollView
	self.m_listLayer = RankPageListLayer:create({
        itemWidth = 608,
        itemHeight = 55,
        spaceX = 0,
        spaceY = 3,
        scrollView = scrollView,
		dataCounter = function ()
			local list = self:getRankList()
	        return #list
        end,
        viewCreator =  function(index, node)
			local list = self:getRankList()
        	local data = list[index]

	        if data then
	        	if not node then
	        		node = FamilyBossRankItemView:create(data)
	        		return node
	        	else
	        		node:setModel(data)
	        	end
		    end
	    end,
    })
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w, h = self.m_listLayer:getContentSize()
    self.m_csNode.scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)


    self:selectTabButton(FamilyBossRankType.kFamily)
end

function This:initTouchEvent()
	local csNode = self.m_csNode

	csNode.beijing.closebutton:setTouchEnded(function()
		self:close()
	end)

end

function This:onEnter()
	super.onEnter(self)
	FamilyBossManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	FamilyBossManager:removeObserver(self)
end

function This:getRankList()
	return self.m_rankList[self.m_curType] or {}
end

function This:setRankType(rankType)
    if self.m_curType == rankType then
		return false
	end
    self.m_curType = rankType

    if not self.m_request[rankType] then
    	FamilyBossManager:onFamilyBossRankRequest(rankType)
    	self.m_request[rankType] = true
    end

    self.m_csNode.biaoqian1:setVisible(self.m_curType == FamilyBossRankType.kFamily)
	self.m_csNode.biaoqian2:setVisible(self.m_curType == FamilyBossRankType.kMember)
    self:refresh()
    return true
end

function This:refresh()
	local csNode = self.m_csNode
	local chapter = FamilyBossManager:getChapter()
	local phase = chapter:getPhase()

	if phase.type == FamilyBossPhaseType.kReward then
		local list = self:getRankList()
		local count = #list
		csNode.tips:setVisible(count == 0)
		csNode.tips.tips1:setVisible(false)
		csNode.tips.tips2:setVisible(count == 0 and self.m_curType == FamilyBossRankType.kMember)
		csNode.tips.tips3:setVisible(count == 0 and self.m_curType == FamilyBossRankType.kFamily)
		self.m_listLayer:setDataList()

		csNode.my:setVisible(true)
		csNode.my.myrank:setVisible(self.m_curType == FamilyBossRankType.kMember)
		csNode.my.mylegion:setVisible(self.m_curType == FamilyBossRankType.kFamily)
		csNode.my.item:setModel( self:getMyRankData() )
	else
		csNode.tips:setVisible(true)
		csNode.tips.tips1:setVisible(true)
		csNode.tips.tips2:setVisible(false)
		csNode.tips.tips3:setVisible(false)
		csNode.my:setVisible(false)
		self.m_listLayer:clearNode(true, true)
	end

end

function This:onFamilyBossRankReply(info)
	self.m_rankList[info.type] = info.list
	self:refresh()
end

function This:getMyRankData()
	local chapter = FamilyBossManager:getChapter()
	if self.m_curType == FamilyBossRankType.kMember then
		return chapter:getMyRoleRankInfo()
	elseif self.m_curType == FamilyBossRankType.kFamily then
		return chapter:getMyLegionRankInfo()
	end
end