-- @Author: zheng
-- @Date:   2018-10-23 11:25:22
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-17 20:54:31
-- @File Name: FamilyGradeNode.lua

local This , super = class(...,"NodeUI")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")
local GameUtil = require("Util.GameUtil")

This.fileName = "MenuScene/NodeUI/FamilyGrade.csb"

function This:initNode(info)
    if info then
       self.m_family = info.family
    end
    
    self:getData()
    self:initRatintId()
    self:refresh()
end

function This:initRatintId()
    local data = self.m_data
    local rankScore = data:getRankScoreModel()
    local temId = rankScore:checkId()
    local curentId = rankScore:getCurentId()
    if curentId>temId then
        temId = curentId
    end
    self.m_ratingId = temId

end

function This:refresh()
    local data = self.m_data
    self:refreshRatingInfor(self.m_ratingId)
    self:refreshBoxState()
end

function This:refreshRatingInfor(rankId)
    local data = self.m_data
    local rankScore = data:getRankScoreModel()
    local ratingConfig,ratingNum = data:getRankScoreConfig(rankId)
    local leftVisible =true
    local rightVisible = true
    if ratingConfig then
        if rankId<=1 then
            rankId =1
            leftVisible = false
        elseif rankId>=ratingNum then
            rankId = ratingNum
            rightVisible = false
        end
        self.jiantou1:setVisible(leftVisible)
        self.jiantou2:setVisible(rightVisible)
        local btn = self.Button
        local isPass = rankScore:checkIdPass(rankId)

        local isCanUp = rankScore:checkCanUp(data:getScore())
        self:btnGray(btn,not isCanUp)
        local btnTxt = Serialize["family_GradeNode_6"]
        local btnVisible = true
        local suoVisible = false
        if rankScore:getCurentId() ==0 then
            btnTxt = Serialize["family_GradeNode_5"]
            suoVisible = true
        elseif rankScore:checkIdPass(ratingNum) then
            btnVisible = false
        end
        self.suo:setVisible(suoVisible)
        btn:setVisible(btnVisible)

        btn.text:setString(btnTxt)
        self.icon:setImage(rankScore:getIcon(rankId))
        self.m_ratingId = rankId
        self.text1:setString(rankScore:getLockDesc(rankId))
        data:resetScoolSize2(self.scrollView,250,123,rankScore:getDesc(rankId))
        self.namebg.name:setString(rankScore:getName(rankId))

        local min = rankScore:getMinScore(rankId)
        local max = rankScore:getMaxScore(rankId)
        if ratingNum ==rankId and isPass then
            max =999999999
        end
        local curent = data:getScore()
        local jinDuVisible = false
        local jinDuTitle = string.format("%s:%d",Serialize["family_GradeNode_2"],min)
        if curent>=min then
            jinDuVisible = true
            jinDuTitle = string.format("%s:",Serialize["family_GradeNode_1"])
            self.jindutiao.num:setString(string.format("%d/%d",curent,max))
            local perfect = curent/max
            if perfect>1 then
                perfect =1
            end
            self.jindutiao.pro:setPercent(perfect*100)
        end
        self.junwei:setString(jinDuTitle)
        self.jindutiao:setVisible(jinDuVisible)

        local setSeasonNum = data:getSeasonNum()
        local endTime = data:getEndTime()
        self.saiji:setString(string.format(Serialize["family_GradeNode_8"],setSeasonNum))
        self.time:setString(GameUtil:formatDate(Serialize["family_GradeNode_9"], endTime))
    end
end

function This:createEffect()
    local function createAnim( parent,anmStr )
        local GameUtil = require("Util.GameUtil")
        local effect1 = GameUtil:createEffect(anmStr)
        parent:addChild(effect1)
        return effect1
    end
    cc.Audio:playEffect("PingJiJinShen.mp3")
    local node = self.icon_anim
    node:removeAllChildren()
    local effects = createAnim( node,"JianZhuShengJi" )
end

function This:refreshBoxState()
    local boxIndexs = {
        [FamilyRatingRewardType.kType1] =4,--赛季
        [FamilyRatingRewardType.kType2] =3,--每周
        [FamilyRatingRewardType.kType3] =2,--个人
    }

    for rewardType=1,FamilyRatingRewardType.kType3 do
        local boxIndex= boxIndexs[rewardType]
        local boxNode = self["box"..rewardType]
        local isHaveReward,boxIsGray = self:getBoxIsHaveReward(rewardType)
        GameUtil:createBoxEffect(isHaveReward,boxNode,boxIndex)
        local texture = "MenuScene/boxGet"..boxIndex..".png"
        if boxIsGray then
            boxNode:setImage(texture)
        end
        self:btnGray(boxNode,boxIsGray)
    end
end

--是否有可领取的奖励
function This:getBoxIsHaveReward(rewardType)
    local data = self.m_data
    local res,boxIsGray = data:getSeasonOrPersonRewardInfo(rewardType)
    if rewardType and rewardType ==FamilyRatingRewardType.kType2 then--每周
        res,rewards,boxIsGray = data:getWeekRewardInfo()
    end
    return res,boxIsGray
    -- return true
end

function This:btnGray(btn,isGray)
    local gray = false
    if isGray then
        gray = true
    end
    if btn then
        if gray then
            btn:setGrayShader()
        else
            btn:clearShader()
        end
    end
end

function This:initTouchEvent()
    
    self.help_Info:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips", {text = Serialize["family_GradeNode_11"],node = self.help_Info})
    end)
    self.jiantou1:setTouchEnded(function()
        -- self:close()
        self:refreshRatingInfor(self.m_ratingId-1)
    end)
    --right
    self.jiantou2:setTouchEnded(function()
        self:refreshRatingInfor(self.m_ratingId+1)
    end)
    -- 赛季奖励
    self.box1:setTouchEnded(function()
        local info ={family = self.m_family,types = FamilyRatingRewardType.kType1}--赛季
        g_scene:openDialog("FamilyRating.FamilyRatingRewardPanel",info)
    end)
    -- 每周奖励
    self.box2:setTouchEnded(function()

        local data = self.m_data
        local havaPower,rewards,receive = data:getWeekRewardInfo()
        local function titleCallBack(  )
            if not havaPower then
                return
            end
            FamilyRatingManager:onGetRatingRewardRequest(FamilyRatingRewardType.kType2)
        end
        local emptyRewardStr = nil
        if #rewards ==0 then
            emptyRewardStr = Serialize["family_GradeNode_4"]
        end
        local info ={rewards =rewards,isGray = not havaPower,emptyRewardStr = emptyRewardStr,
        receive = receive ,btnCallBack =titleCallBack,wenhaoStr =Serialize["family_GradeNode_3"] }        
        g_scene:openDialog("FamilyRating.FamilyRatingWeeklyRewardPanel",info)
    end)
    --个人奖励
    self.box3:setTouchEnded(function()
        local info ={family = self.m_family,types = FamilyRatingRewardType.kType3}--个人
        g_scene:openDialog("FamilyRating.FamilyRatingRewardPanel",info)
    end)
    --晋  升
    self.Button:setTouchEnded(function()
        local selfMember = self.m_family:getSelfMemberInfo()
        local dismissVisible = selfMember:hasPower(FamilyPower.kPromotion)
        if not dismissVisible then
            g_scene:showTips(Serialize["family_GradeNode_10_c"])
            return
        end
        
		local data = self.m_data
        local rankScore = data:getRankScoreModel()
        local isCanUp = rankScore:checkCanUp(data:getScore())
        if isCanUp then
            FamilyRatingManager:onRatingUpRequest()
        end
    end)
    -- 军衔一览
    self.junxianicon:setTouchEnded(function()
        local info ={family = self.m_family}
        g_scene:openDialog("FamilyRating.FamilyRatingMilitaryPanel",info)
    end)
end

function This:onEnter()
    super.onEnter(self)
    FamilyRatingManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    FamilyRatingManager:removeObserver(self)
end

function This:getData()
	self.m_data = self.m_family:getRatingData()
end

function This:onGetRatingInfo()
	self:initRatintId()
    self:refresh()
end

function This:onGetRatingReward()
    self:refresh()
end

function This:onRatingUp()
    self:initRatintId()
    self:refresh()
    self:createEffect()
end

function This:onGetMyFamilyInfo()
    self:initRatintId()
    self:refresh()
end
