-- @Author: zheng
-- @Date:   2018-12-15 16:56:52
-- @Last Modified by:   superm
-- @Last Modified time: 2020-10-30 16:49:30
-- @File Name: FamilyRatingManager.lua
local This , super = class(..., "Base.BaseModel")
local FamilyManager = require("Family.FamilyManager")
local HeroManager = require("Hero.HeroManager")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GameScienceData = require("Family.GameScienceData")
local GamePlayer = require("Player.GamePlayer")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")
local DepotManager = require("Depot.DepotManager")

local LegionMilitaryConfig = require("Metadatas.LegionMilitaryConfig")
local LegionWarPowerConfig = require("Metadatas.LegionWarPowerConfig")

function This:init()
    super.init(self)
    self:initListener()
    --self:onGetCastInfoRequest()
end
function This:initListener()
	EventDispatcher:addListener(self,11900,self.onGetCastInfoScheduleReply)--投名状进度查看

    EventDispatcher:addListener(self, 12146, self.onGetRatingInfoReply)--查看军团评级信息
    EventDispatcher:addListener(self, 12147, self.onRatingUpReply)--军团评级晋升
    EventDispatcher:addListener(self, 12148, self.onGetMilltaryReply)--军衔任命
    EventDispatcher:addListener(self, 12149, self.onGetRatingRewardReply)--领取奖励
    EventDispatcher:addListener(self, 12150, self.onGetCastInfoReply)--投名状信息
    EventDispatcher:addListener(self, 12151, self.onSetCastReply)--投名状报名
    EventDispatcher:addListener(self, 12152, self.onCastRewardReply)--投名状领取奖励
    EventDispatcher:addListener(self, 12153, self.onDonateRewardReply)--领取祭祀奖励
    EventDispatcher:addListener(self, 12154, self.onScienceReply) -- 军团科技
    EventDispatcher:addListener(self, 12155, self.onScienceResearchReply)-- 军团科技研究
    EventDispatcher:addListener(self, 12156, self.onScienceUpReply) -- 军团科技升级
end

function This:removeListener()
    EventDispatcher:removeListener(self,11900)--投名状进度查看

    EventDispatcher:removeListener(self,12146)--查看军团评级信息
    EventDispatcher:removeListener(self,12147)--军团评级晋升
    EventDispatcher:removeListener(self,12148)--军衔任命
    EventDispatcher:removeListener(self,12149)--领取奖励
    EventDispatcher:removeListener(self,12150)--投名状信息
    EventDispatcher:removeListener(self,12151)--投名状报名
    EventDispatcher:removeListener(self,12152)--投名状领取奖励
    EventDispatcher:removeListener(self,12153)
    EventDispatcher:removeListener(self,12154)
    EventDispatcher:removeListener(self,12155)
    EventDispatcher:removeListener(self,12156)
end

--自己评级的军衔属性加成
function This:getRatingAttr()
    local res = {}
	local ratingData = self:getRatingData()
    if ratingData then
        local selfMember = FamilyManager:getMySelfMemeberInfo()
        if ratingData and selfMember then
           local militaryId = selfMember:getMilitary()
           res = ratingData:getMilitaryAttr(militaryId)
        end
    end
    return res
end

--根据自己的评级的刷新所有武将属性点
function This:updateAllHeroRatingAttr()
    HeroManager:updateRating()
end

function This:updateAllHeroScienceAttr()
    HeroManager:updateScienceAttr(self:getScienceAttr())
    --HeroManager:updateScienceAdd(self:getScienceAddAttr())
end

--投名状进度
function This:getCastNameProgress()
    return self.m_castNameProgress or 0
end

function This:hasCastRed(num)
    local ratingData = self:getRatingData()
    if ratingData then
        local castId,isRecevice = ratingData:getCastId()
         if num == 1 then
             return ratingData:getCastRewardState() == FamilyRatingWarlordRewardState.kType1 and castId < 100 and isRecevice
         else
             return ratingData:getCastRewardState() == FamilyRatingWarlordRewardState.kType1 and castId > 100
         end
        --return true
    end
    return false
end

function This:getMyFamily()
    return FamilyManager:getMyFamily()
end

function This:getRatingData()
    local result = nil
    local family = self:getMyFamily()
    if family then
        result = family:getRatingData()
    end
    return result
end

function This:getScienceData()
    local result = nil
    local hasFamily = FamilyManager:hasFamily()
    local family = self:getMyFamily()
    if family and hasFamily then
        result = family:getScienceData()
    end
    return result
end

function This:ratingHaveReward()
    local result = false
    local data = self:getRatingData()
    local family = self:getMyFamily()
    if data then
        local selfMember = family:getSelfMemberInfo()
        local dismissVisible = FamilyManager:checkMySelfTitlePower(FamilyPower.kPromotion)
        if dismissVisible then
            local rankScore = data:getRankScoreModel()
            if rankScore:getCurentId() ==0 then
                return true
            end
            local isCanUp = rankScore:checkCanUp(data:getScore())
            if isCanUp then
                return isCanUp
            end
        end
        for rewardType=1,FamilyRatingRewardType.kType3 do
            local isHaveReward,boxIsGray = data:getSeasonOrPersonRewardInfo(rewardType)
            if rewardType and rewardType ==FamilyRatingRewardType.kType2 then--每周
                isHaveReward,rewards,boxIsGray = data:getWeekRewardInfo()
            end
            if isHaveReward then
                return isHaveReward
            end
        end
    end
    return result
end

function This:scienceHaveReward()
    local red = false
    local sceneData = self:getScienceData()
    if sceneData then
        local selfMember = FamilyManager:getMySelfMemeberInfo()
        if not selfMember then return false end
        if selfMember:hasPower(FamilyPower.kTechnology) then
            local family = self:getMyFamily()
            for _,chapter in pairs (sceneData:getAllChapter()) do
                if chapter:getLevel() < chapter:getLimitLevel() and family:getFunds() >= chapter:getNextCost() then
                    red = true
                end
            end
        end
        
        for _,stage in pairs(sceneData:getAllStage()) do
            local chapter = sceneData:getChapter(stage:getId())
            if stage:getLevel() < chapter:getLevel() and GamePlayer:getLevel() >=  stage:getLimitLevel() and DepotManager:isItemOrResEnough(stage:getCostType(),stage:getNextCost()) then
                red = true
            end
        end
    end
    return red
end

function This:scienceResearchRed(itemList)
    local red = false
    local sceneData = self:getScienceData()
    if sceneData then
        local selfMember = FamilyManager:getMySelfMemeberInfo()
        if not selfMember then return false end
        if not itemList and not selfMember:hasPower(FamilyPower.kTechnology) then
            return false
        end
        local family = self:getMyFamily()
        if not itemList then
            itemList = sceneData:getAllChapter()
            if itemList then
                for _,chapter in pairs (itemList) do
                    if chapter:getLevel() < chapter:getLimitLevel() and family:getFunds() >= chapter:getNextCost() then
                        red = true
                    end
                end
            end
        end
    end
    return red
end

function This:scienceUpRed(itemList)
    local red = false
    local sceneData = self:getScienceData()
    if sceneData then
        for _,stage in pairs(itemList) do
            local chapter = sceneData:getChapter(stage:getId())
            if stage:getLevel() < chapter:getLevel() and GamePlayer:getLevel() >=  stage:getLimitLevel() and  DepotManager:isItemOrResEnough(stage:getCostType(),stage:getNextCost()) then
                red = true
            end
        end
    end
    return red
end

function This:getScienceAttr()
    local attrAdditionList = {            
        [SoilderType.kShield] = {},
        [SoilderType.kCavalry] = {},
        [SoilderType.kArcher] = {},
        [SoilderType.kSapperCar] = {},
    }
    local scienceData = self:getScienceData()
    if scienceData then
        for index,addition in pairs(attrAdditionList) do
            local scienceIdList = scienceData:getScienceIds(index)
            for _,id in pairs(scienceIdList) do
                local data = scienceData:getAttr(id)
                local attrId = scienceData:getAddtiionId(id)
                addition[attrId]=data:getAddition()
            end
        end
        attrAdditionList =scienceData:scienceAddTionTableAdd(attrAdditionList,self:getScienceAddAttr())
    end
    return attrAdditionList
end

function This:getScienceAddAttr()
    local attrAdditionList = {}
    local scienceData = self:getScienceData()
    if scienceData then
        attrAdditionList = scienceData:getScienceAddAddtion()
        return attrAdditionList
    end
    return attrAdditionList
end

function This:onGetRatingInfoRequest()
    Socket:sendAndLock(12146)--查看军团评级信息
end

function This:onGetRatingInfoReply( msg )
    if msg.err_code == 0 then
        
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        ratingData:setPersonScore(msg.junwei or 0)
        ratingData:setRatingRewardInfors(msg.reward_info or {})
        ratingData:setSeasonNum(msg.season_num)
        ratingData:setEndTime(msg.end_time)        
        -- ratingData:setRatingRewardInfor({["reward_his"]={[1]=1,[2]=2,},["reward_list"]={[1]=3,[2]=4,},["reward_type"]=3,})
        self:notifyObservers("onGetRatingInfo")
		EventDispatcher:dispatcher("menu_scene_enter")		

    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onRatingUpRequest()
    Socket:sendAndLock(12147)--军团评级晋升
end

function This:onRatingUpReply( msg )
    if msg.err_code == 0 then
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        local rating = msg.rating
        ratingData:init( rating or 1)
        ratingData:setRatingRewardInfors(msg.reward_info or {})
        if rating ==1 then  --把所有人从无变成军卒并刷新ui
            local family = self:getMyFamily()
            family:setAllMemberMilitary(#LegionMilitaryConfig)
        end
        self:notifyObservers("onRatingUp")
		EventDispatcher:dispatcher("rating_change")		

    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetMilltaryRequest( roleId, title )
    Socket:sendAndLock(12148, {role_id = roleId, military_id = title})--军衔任命
end

function This:onGetMilltaryReply(msg)
    if msg and msg.err_code == 0 then
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        local family = self:getMyFamily()
        family:setMemberMilitary(msg.role_id, msg.military_id)
        if family:isDirty() then
            FamilyManager:onGetMyFamilyInfoRequest()
        end
        -- g_scene:showTips(Serialize["family_Manager_7"])
        self:updateAllHeroRatingAttr()
        self:notifyObservers("onGetMilltary")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetRatingRewardRequest(opType,id)
    Socket:sendAndLock(12149,{op_type= opType,id = id})--领取奖励
end

function This:onGetRatingRewardReply( msg )
    if msg.err_code == 0 then
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        ratingData:setRatingRewardInfor(msg.reward_info or {})
        local reward_list = msg.reward_list or {}
        g_scene:showReward(reward_list)
        self:notifyObservers("onGetRatingReward")
		EventDispatcher:dispatcher("rating_change")		

    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
    
function This:onGetCastInfoRequest()
    if not FamilyManager:hasFamily() then
        return
    end
    Socket:sendAndLock(12150)--投名状信息
end

function This:onGetCastInfoReply( msg )
    -- print(#msg.task_list.." task_list")
    if msg.err_code == 0 then
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        ratingData:setCastRewardState(msg.reward_state)
        ratingData:setCastId(msg.id)
        ratingData:setTaskIdList2(msg.task_list2)
        ratingData:setTaskIdList(msg.task_list)
        self:notifyObservers("onGetCastInfo")
        EventDispatcher:dispatcher("science_change")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onSetCastRequest(id)
    Socket:sendAndLock(12151,{id = id})--投名状报名
end

function This:onSetCastReply( msg )
    if msg.err_code == 0 then
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        ratingData:setCastId(msg.id)
        self:notifyObservers("onSetCast")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onCastRewardRequest(id)
    Socket:sendAndLock(12152,{type = id})--投名状领取奖励
end

function This:onCastRewardReply( msg )
    if msg.err_code == 0 then
        print(msg.err_code.."msg")
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        local reward_list = msg.reward_list or {}
        g_scene:showReward(reward_list)
        ratingData:setCastRewardState(msg.reward_state)
        self:notifyObservers("onCastReward")
		EventDispatcher:dispatcher("science_change")		

    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onDonateRewardRequest(id)
    Socket:sendAndLock(12153,{box_id=id})
end

function This:onDonateRewardReply( msg )
    if msg.err_code == 0 then
        FamilyManager:setDonatineRewardRecord(msg.donation_reward_record)
        g_scene:showReward(msg.reward_list)
        self:notifyObservers("onDonateRewardRecive")
        SignalSystem.distributeSignal(Signals.FamilyPanelSacrifice)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end


function This:onScienceRequest()
    if FamilyManager:hasFamily() then
        Socket:sendAndLock(12154)  
    end  
end

function This:onScienceReply( msg )
    if msg.err_code == 0 then
        local ratingData = self:getScienceData()
        ratingData:setChapterLevel(msg.group_technology)
        ratingData:setStageLevel(msg.role_group_technology)
        self:updateAllHeroScienceAttr()
    else
        g_scene:handleErrorCode(msg.err_code)        
    end
end

function This:onScienceResearchRequest( id )
    Socket:sendAndLock(12155,{id = id})        
end

function This:onScienceResearchReply( msg )
    if msg.err_code == 0 then
        local ratingData = self:getScienceData()
        ratingData:updateChapterLevel(msg.id,msg.level)
        self:notifyObservers("onScienceResearch", msg.id)
		EventDispatcher:dispatcher("science_change")		

    else
        g_scene:handleErrorCode(msg.err_code)        
    end
end

function This:onScienceUpRequest( id )
    Socket:sendAndLock(12156,{id = id})        
end

function This:onScienceUpReply( msg )
    if msg.err_code == 0 then
        local ratingData = self:getScienceData()
        ratingData:updateStageLevel(msg.id,msg.level)
        self:updateAllHeroScienceAttr()
        self:notifyObservers("onScienceUp", msg.id)
		EventDispatcher:dispatcher("science_change")		
        -- ratingData:updateScienceRoleData(msg.id,msg.level)
    else
        g_scene:handleErrorCode(msg.err_code)        
    end
end


-- 投名状进度
function This:onGetCastInfoScheduleRequest()
    Socket:sendAndLock(11900,{type = AchieveType.kTouMingZhuang})        
end

function This:onGetCastInfoScheduleReply(msg)
    local achieveType = msg.achievement.type
    if achieveType == AchieveType.kTouMingZhuang then
        local ratingData = self:getRatingData()
        if not ratingData then
            return
        end
        local castId = ratingData:getCastId()

        local progress = msg.achievement.progress_list
        for _,data in pairs(progress) do
            if data.id == castId then
                self.m_castNameProgress =data.num
            end
        end
    end
end