local This , super = class(...,"NodeView")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")
local FamilyManager = require("Family.FamilyManager")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local ItemNode = require("UI.ItemNode") 

local LegionWarPersonalShopConfig = require("Metadatas.LegionWarPersonalShopConfig")
local LegionWarShareShopConfig = require("Metadatas.LegionWarShareShopConfig")


This.fileName = "LegionScene/NodeUI/LegionShopItem.csb"
local kJunTuanGoldType = 126 


function This:init(info)
    local w,h = self.bg:getContentSize()
    self:setContentSize(w,h)
    self.m_map = info.map
    self:initdata(info.m_type,info)
    self.m_cityId = info.cityID
end

function This:initdata(type,info)
    local shopData = self.m_map:getGameWarShopData()
    if type == FamilyWarShopType.kPersonal then
        self.m_config = shopData:getWarPersonShopList()
        self.m_shopConfig = LegionWarPersonalShopConfig
    else
        self.m_config = shopData:getWarShareShopList()
        self.m_shopConfig = LegionWarShareShopConfig
    end
    if info then
        self.m_data = self:getItemData(info.m_shop_id,info.m_index,info.m_type)
        self.m_solt = self.m_data.solt
    end
    self.m_shop_id = shopData:getWarShopId()
    self.m_buyPower = self.m_data.buy_power
    self.m_type = type
    self.m_data = self:getItemData(self.m_shop_id,self.m_solt)
    self.m_family = FamilyManager:getMyFamily()
    self:refresh()
end

function This:initTouchEvent()
    self.unlock.buybutton:setTouch({
        onEnded = function()
            self:requestBuy()
        end,
        isScale = false,
        isContainer = true
    })
end
------------------- 界面刷新 -----------------------

function This:refresh()
    local unlockNode = self.unlock 
    self:createItemNode(unlockNode.item,self.m_data.item_id,self.m_data.item_num)
    self.condition:setVisible(false)
    self.name:setString(ItemUtil.getName(self.m_data.item_id))
    unlockNode.buyTimes:setString(string.format(Serialize[61081],self:GetLostBuyTime(),self.m_data.limit_time))
    unlockNode.yuanbao:setImage(ItemUtil.getIconImage(self.m_data.cost_type))
    unlockNode.price:setString(self:getBuyPrice())
    local state= false
    if self.m_data.discount ~= 10 then
        state = true
        unlockNode.discount:setString(string.format(Serialize[61082],self.m_data.discount))
    end
    unlockNode.bg5:setVisible(state)
    unlockNode.discount:setVisible(state)
    state = true
    if self:GetLostBuyTime() == 0 then
        state = false
    end
    unlockNode.buybutton:setVisible(state)
    unlockNode.purchased:setVisible(not state)
    state=false
    if self.m_type == FamilyWarShopType.kShare then
        state = true
    end
    unlockNode.gongxiang:setVisible(false)
end

function This:onShopItem(type)
    self:initdata(type)
    print("购买回调")
end

------------------- 工具函数 --------------------

function This:createItemNode(parent,key,needNum)
    local name = "ItemNode"
    local node = parent:getChildByName(name)
    if not node then
        node = ItemNode:create(key,1)
        node:setAnchorPoint(0.5, 0.5)
        parent:addChild(node)
        node:setName(name)
    end
    node:initItem(key,needNum)
end

-- 获取剩余购买次数
function This:GetLostBuyTime()
    local num
    for i,itemBuyState in pairs(self.m_config) do
        if itemBuyState.solt == self.m_solt then
            num = itemBuyState.num
        end
    end
    return self.m_data.limit_time - num or 0
end

-- 获取购买价格
function This:getBuyPrice()
    local res,num,price = {}
    for i,itemBuyState in pairs(self.m_config) do
        if itemBuyState.solt == self.m_solt then
            num = itemBuyState.num
        end
    end
    if num + 1 <= self.m_data.limit_time then
        price = self.m_data.price[num + 1]
    else
        price = self.m_data.price[self.m_data.limit_time]
    end
    return price
end

-- 获取数据
function This:getItemData(shopId,index)
    local res = {}
    for k,shopItemData in pairs(self.m_shopConfig) do
        if shopId == shopItemData.shop_num and index == shopItemData.solt then
            res = shopItemData
        end
    end
    return res
end

function This:onEnter()
    super.onEnter(self)
    self.m_map:addObserver(self)
    FamilyRatingManager:addObserver(self)
end

function This:onExit()
    FamilyRatingManager:removeObserver(self)
    self.m_map:removeObserver(self)
    super.onExit(self)
end

function This:requestBuy()
    if self:GetLostBuyTime() == 0 then
        g_scene:showTips(Serialize[61083])
    end
    local state, id = self:isCostEnoughBy()
    if state then
        if self:isHasTitlelBuy() then
            self.m_map:onWarShopBuyRequest(self.m_data.item_id,self.m_solt,self.m_type,self.m_cityId)
            if self.m_data.cost_type == kJunTuanGoldType then
                self.m_family:costFunds(self:getBuyPrice())
            end
        else
            g_scene:showTips(Serialize[61084])
        end
    else
        if id == 102 then
            GamePlayer:checkDiamond(0, true)
        else
            g_scene:showTips(Serialize[60714] .. ItemUtil.getName(id) .. Serialize[60715])
        end
    end
end

function This:isCostEnoughBy()
    local id, num = self.m_data.cost_type,self:getBuyPrice()  
    if id == kJunTuanGoldType then
        return self.m_family:getFunds() >= num , kJunTuanGoldType
    end
    local state =  DepotManager:isItemOrResEnough(id,num)
    return state,id
end

function This:isHasTitlelBuy()
    if self.m_data.buy_power then
        local canBuy = false
        local ratingData = self.m_map:getGameWarShopData()
        local m_familyTytle = ratingData:getTytle()
        for _, v in pairs(self.m_data.buy_power) do
            if m_familyTytle == v then
                canBuy = true
            end
        end
        return canBuy
    else
        return true
    end
end