


local This,super = class(...,"Dialog")
local Timer = require("Timer")
local ItemUtil = require("Bag.ItemUtil")
local LegionWarPersonalShopConfig = require("Metadatas.LegionWarPersonalShopConfig")
local LegionWarShareShopConfig = require("Metadatas.LegionWarShareShopConfig")
local FamilyWarShopItemView = require("FamilyRating.FamilyWarShopItemView")

local fileName = "LegionScene/NodeUI/LegionShop.csb"
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local DepotManager = require("Depot.DepotManager")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")
local FamilyManager = require("Family.FamilyManager")


local BoxImage = 
{
    "MenuScene/boxLock2.png",
    "MenuScene/boxGet2.png",
}

function This:initData(info)
    self.m_map = info.map
    self.m_shopData = self.m_map:getGameWarShopData()
    self.m_shop_Id = self.m_shopData:getWarShopId()
    self.m_integral = self.m_shopData:getWarShareShopIntegral()
    self.m_nextIntegral = self.m_shopData:getScore(self.m_integral)
    self.m_jifenRewardState = self.m_shopData:getRoleScoreReward()
    self.m_endTime = self.m_shopData:getOverTime()
    self.m_family = FamilyManager:getMyFamily()
    self.m_cityId = info.cityId
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile(fileName)
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    self.m_itemNodeList = {}
    self.m_escape = 0
    self.m_maxIntegral = 200  -- 积分最大值
    self.m_timer = Timer:create(self)
    self.m_sliderX = self.m_csNode.tex.jindutiao.pro:getScaleX()
    self.m_cityId = info.cityId
    self:refresh()
    self:tabClick(FamilyWarShopType.kPersonal,false)
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.close:setTouchEnded(function()
        self:close()
    end)

    local tagNode = csNode.biaoqian
    tagNode.tep1:setTouchEnded(function()
        self:tabClick(FamilyWarShopType.kPersonal,false)
    end)
    tagNode.tep2:setTouchEnded(function()
        self:tabClick(FamilyWarShopType.kShare,true)
    end)

    local infoNode = csNode.info
    infoNode.zijin.add:setTouchEnded(function()
        g_scene:openDialog("System.RechargeMainUI")
    end)
    infoNode.wenhao:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize["family_warlordShop_2"],node = self.m_csNode.info.wenhao})
    end)
    csNode.tex.box:setTouchEnded(function()
        local args= {
            storeNumber = self.m_shop_Id,
            myScore = self.m_integral,
            allAwardStatus = self.m_jifenRewardState,
            map =self.m_map:getMapType()
        }
        g_scene:openDialog("Store.BarracksSoreAwardPanel",args)
    end)
end

function This:initUI(type)
    self.m_MaxSolt = self.m_shopData:getMaxSolt(type)
    local ScrollView = self.m_csNode.info.ScrollView
    ScrollView:removeAllItems()
    self.m_itemNodeList = {}
    local function creatItemView(id,index,type)
        local node = FamilyWarShopItemView:create({m_shop_id = id,m_index = index,m_type = type,cityID=self.m_cityId,map=self.m_map})
        return node
    end
    
    for i = 1, self.m_MaxSolt do
        local item = creatItemView(self.m_shop_Id,i,type) 
        table.insert(self.m_itemNodeList, item)
        local w , h = item:getContentSize()
        ScrollView:pushNode(item,w,h-20,2)
    end
end

function This:refresh()

    local timeTxt = string.format(Serialize["family_WarlordShop_1"],GameUtil:formatTime(Serialize["H_M_S"],self:getLeftTime()))
    local csNode = self.m_csNode
    local texNode = csNode.tex
    local node = csNode.info.zijin
    local node1 = texNode.jindutiao

    node.time:setString(timeTxt)
    texNode.icon1:setImage(ItemUtil.getIconImage(126))
    texNode.number1:setString(self.m_family:getFunds())

    node.number2:setString(GamePlayer:getDiamond())
    node1.num:setString(string.format("%d/%d",self.m_integral,self.m_nextIntegral))
    local w = (self.m_integral/self.m_nextIntegral)*self.m_sliderX
    node1.pro:setScaleX(w)
    if self.m_shopData:hasReward() then
        texNode.box:setImage(BoxImage[2])
    else
        texNode.box:setImage(BoxImage[1])
    end
    if self.m_shopData:hasRewardCompleted() then
        texNode.box:setGrayShader()
    end
    GameUtil:createBoxEffect(self.m_shopData:hasReward(),self.m_csNode.tex.box,2)
end

-- 刷新时间显示
function This:refreshTime()
    local m_text = string.format(Serialize["family_WarlordShop_1"],GameUtil:formatTime(Serialize["H_M_S"],self:getLeftTime()))
    self.m_csNode.info.zijin.time:setString(m_text)
end

function This:tabClick(tab,state)
    self:initUI(tab)
    self:refresh()
    for _, shopItem in pairs(self.m_itemNodeList) do
        shopItem:initdata(tab)
    end
    local node = self.m_csNode.info
    self.m_csNode.biaoqian.tep1.normal:setVisible(state)
    self.m_csNode.biaoqian.tep2.normal:setVisible(not state)
    self.m_csNode.tex:setVisible(state)
end

function This:getLeftTime()
    local leftTime = self.m_endTime - GamePlayer:getServerTime()
    if leftTime <= 0 then
        leftTime = 0
    end
    return leftTime or 0
end

function This:onEnter()
    super.onEnter(self)
    self.m_map:addObserver(self)
    FamilyRatingManager:addObserver(self)
    DepotManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    self.m_map:removeObserver(self)
    FamilyRatingManager:removeObserver(self)
    DepotManager:removeObserver(self)
end

function This:update(dt)
    self.m_timer:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape >= 1 then
        self.m_escape = self.m_escape - 1
        self:refreshTime()
    end
end

function This:onShopItem(type)
    self:initData({cityId=self.m_cityId,map = self.m_map})
    self:refresh()
end

function This:itemUpdate()
    local m_node = self.m_csNode.info.zijin
    m_node.number1:setString(self.m_family:getFunds())
    m_node.number2:setString(GamePlayer:getDiamond())
end

function  This:onReceiveBarracksScoreAwards( data )
    self.m_jifenRewardState = data.role_jifen_rewarded
    self:refresh()
end