-- @Author: zheng
-- @Date:   2018-10-20 15:22:20
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-12-15 15:20:54
-- @File Name: GameRatingData.lua

local LegionRankScoreConfig = require("Metadatas.LegionRankScoreConfig")
local LegionMilitaryConfig = require("Metadatas.LegionMilitaryConfig")
local LegionQuarterlyPersonConfig = require("Metadatas.LegionQuarterlyPersonConfig")
local FamilyPowerConfig = require("Metadatas.FamilyPowerConfig")
local FamilyLevelConfig = require("Metadatas.FamilyLevelConfig")
local RewardConfig = require("Metadatas.RewardConfig")
local TechnologyUpgradeConfig = require("Metadatas.TechnologyUpgradeConfig")
local GameRankScore = require("FamilyRating.GameRankScore")


local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

FamilyRatingRewardType ={--奖励领取类型
    kType1 = 1,--赛季
    kType2 = 2,--每周
    kType3 = 3,--个人
}

FamilyRatingBoxState = {
    kLock = 1,
    kOpen = 2,
    kClose = 0
}

FamilyRatingWarlordRewardState ={--投名状状态
    kType1 = 1,--可领取
    kType2 = 2,--不可领取
    kType3 = 3 --已领取
}
FamilyWarShopType={
    kPersonal = 1,  --个人商店
    kShare = 2      --共享商店
}

local This = class(...)


function This:create(id)
    local Morality = {}
    setmetatable(Morality,This)
    Morality:init(id,true)
    return Morality
end

function This:resetRewardType(rewardType)
    if not rewardType then
        return
    end
    if rewardType == FamilyRatingRewardType.kType3 then--个人
        self.m_personRewardList = {}
        for index, config in ipairs(LegionQuarterlyPersonConfig) do
            self.m_personRewardList[index] = {
                id = config.id, 
                state = FamilyRatingBoxState.kLock, 
                reward = config.reward, 
                min = config.min,
                uiType = FamilyRatingRewardType.kType3,--个人
            }
        end
        self.m_personIsGray = false
    elseif rewardType == FamilyRatingRewardType.kType1 then--赛季
        self.m_seasonRewardList = {}
        for index, config in ipairs(LegionRankScoreConfig) do
            self.m_seasonRewardList[index] = {
                id = config.id, 
                state = FamilyRatingBoxState.kLock, 
                reward = config.reward, 
                name = config.name,
                uiType = FamilyRatingRewardType.kType1,--赛季
            }
        end
        self.m_seasonIsGray = false
    end 
end

function This:init(id,isReset)
    -- self.m_id = id
    local rankScore = self:getRankScoreModel()
    rankScore:init(id)
    if isReset then
        self:resetRewardType(FamilyRatingRewardType.kType3 )--个人
        self:resetRewardType(FamilyRatingRewardType.kType1 )--赛季
        self.m_rewardInfo ={{reward_type = 1,reward_list ={},reward_his ={}}
        ,{reward_type = 2,reward_list ={},reward_his ={}}
        ,{reward_type = 3,reward_list ={},reward_his ={}}}--奖励信息
    end
end

function This:getId()
    local rankScore = self:getRankScoreModel()
    return rankScore:getCurentId()
end

function This:getRankScoreModel()
    if not self.m_randScoreModel then
        self.m_randScoreModel = GameRankScore:create()
    end
    return self.m_randScoreModel
end

-- 当前军威值
function This:setScore(Score)
    self.m_Score = Score
end

function This:getScore()
    return self.m_Score or 0
end

function This:setSeasonNum(SeasonNum)
    self.m_SeasonNum = SeasonNum
end

function This:getSeasonNum()
    return self.m_SeasonNum or 0
end

function This:setEndTime(EndTime)
    self.m_EndTime = EndTime
end

function This:getEndTime()
    return self.m_EndTime or 0
end

-- 设置多个奖励信息
function This:setRatingRewardInfors(rewardInfors)
    if rewardInfors then
        for _,info in ipairs(rewardInfors) do
            self:setRatingRewardInfor(info)
        end
    end
end

-- 获取多个奖励信息
function This:setRatingRewardInfor(rewardInfor)
    local rewardType = rewardInfor.reward_type
    self.m_rewardInfo[rewardType]= rewardInfor
    if rewardType ~= FamilyRatingRewardType.kType2 then--每周
        self:updateRewards(rewardType,rewardInfor.reward_his,rewardInfor.reward_list)
    end
end

function This:getRatingRewardInfor(rewardType)
    return self.m_rewardInfo[rewardType]
end

-- 获取个人赛季是否可以领取
function This:getSeasonOrPersonRewardInfo(rewardType)
    local boxIsGray = self.m_personIsGray
    local rewardInfor = self:getRatingRewardInfor(rewardType)
    if rewardType == FamilyRatingRewardType.kType1 then--赛季
        boxIsGray = self.m_seasonIsGray
    end
    if rewardInfor then
        return rewardInfor.reward_list[1],boxIsGray-- or rewardInfor.reward_his[1]
    end
    return false,boxIsGray
end

-- 获每周奖励是否可以领取
function This:getWeekRewardInfo()
    local rewardInfor = self:getRatingRewardInfor(FamilyRatingRewardType.kType2)--每周
    if rewardInfor then
        local hitId = rewardInfor.reward_his[1] or rewardInfor.reward_list[1]
        local config1 = RewardConfig[hitId]
        if config1 then
            return rewardInfor.reward_list[1],config1.rewardlist,rewardInfor.reward_his[1]
        end
    end
    return false,{},false
end


--投名状ID
function This:setCastId(CastId)
    self.m_CastId = CastId
end

function This:getCastId()
    local castId = 0
    if self.m_CastId then
        castId = self.m_CastId
    end
    print(castId.."castId")
    return castId ,castId~=0 
end

-- 领奖状态
function This:setCastRewardState(CastRewardState)
    self.m_CastRewardState = CastRewardState or FamilyRatingWarlordRewardState.kType2
end

--1可领取 2不可领取 3已领取
function This:getCastRewardState()
    return self.m_CastRewardState or FamilyRatingWarlordRewardState.kType2
end

function This:setTaskIdList( taskIdList )
    self.m_taskIdList = taskIdList or {}
    table.sort(self.m_taskIdList,function (a,b)
        return a < b
    end)
end

function  This:setTaskIdList2( taskIdList )
    self.m_taskIdList2 = taskIdList or {}
    table.sort(self.m_taskIdList2,function (a,b)
        return a < b
    end)
end

function This:getTaskIdList(type)
    if type == 2 then
        return self.m_taskIdList2 or {}
    else
        return self.m_taskIdList or {}
    end
end

-- 军团评级config
function This:getRankScoreConfig(id)
    local item1 = LegionRankScoreConfig[id]
    if item1 then
        return item1,#LegionRankScoreConfig
    end
    return nil
end

--  军衔config
function This:getMilitaryConfig(id)
    local item1 = LegionMilitaryConfig[id]
    if item1 then
        return item1
    end
    return nil
end

-- 个人奖励config
function This:getPersonalRewardConfig(id)
    local item1 = LegionQuarterlyPersonConfig[id]
    if item1 then
        return item1
    end
    return nil
end

--军衔attr
function This:getMilitaryAttr(id)
    local res ={}
    local item1 = self:getMilitaryConfig(id)
    if item1 then
        res = self:convertData(item1.impact)
    end
    return res
end

function This:getMilitaryNameAndDesc(id,isMyMilitary)
    local item1 = self:getMilitaryConfig(id)

    if item1 then
        local text0 = item1.text0
        local text1 = item1.text1
        if isMyMilitary == id then
            return item1.name,text1
        end
        return item1.name,text0
    end
    return "",""
end

-----------------------

function This:getTitleList(memberList,lv)
    local function getPowerName( title )
        return FamilyPowerConfig[title].title
    end
    local res ={}
    local item1= FamilyLevelConfig[lv]
    if item1 then
        local titleLimits= self:convertData( item1.title_limit )
        for title,allCount in pairs(titleLimits) do
            local member = memberList[title]
            if not member then
                local name = getPowerName(title)
                member = {name = name,curentCount =0}
            end
            member.allCount = allCount
            member.title = title
           table.insert(res,member)
        end
    end
    return res
end

--军衔
function This:getMilitaryList(memberList)
    local function getPowerName( title )
        local name,desc = self:getMilitaryNameAndDesc(title)
        return name
    end
    local res ={}
    local rankScore = self:getRankScoreModel()
    local titleLimits= rankScore:getImpact()
    for title,allCount in pairs(titleLimits) do
        local member = memberList[title]
        if not member then
            local name = getPowerName(title)
            member = {name = name,curentCount =0}
        end
        member.allCount = allCount
        member.title = title
       table.insert(res,member)
    end
    return res
end

-- 更新个人赛季奖励
function This:updateRewards(rewardType,receivedData,canReceiveDataList)
    local res =value
    -- rewardTable 本地的奖励列表
    -- curentScore 当前积分
    -- rewardConfig 奖励的配置表
    -- rewardReceivedData 已经领取的奖励列表
    local function getRewardList(rewardTable,curentScore,rewardConfig,rewardReceivedData)
        local rewardHistory = rewardReceivedData
        table.sort(rewardHistory,function(a,b)
            return a < b
        end)
        local starCount = curentScore
        local isCanReward = false
        local maxCanRewardIndex =0
        local minCanRewardIndex = 1
        for i,itemId in ipairs(canReceiveDataList) do
                if itemId>maxCanRewardIndex then
                    isCanReward = true
                    maxCanRewardIndex = itemId
                end
        end
        for index=1,maxCanRewardIndex do
            rewardTable[index].state = FamilyRatingBoxState.kOpen
        end
        for index1=1,#rewardConfig do
            local isFind = table.find(rewardHistory, function (index, value)
                return index1 == value
            end)
            if not isFind then
                minCanRewardIndex = index1
                break
            end
        end
        for _,configIndex in ipairs(rewardHistory) do
            if rewardTable[configIndex] then
                rewardTable[configIndex].state = FamilyRatingBoxState.kClose
            elseif maxCanRewardIndex >= configIndex then
                isCanReward = true
                rewardTable[configIndex].state = FamilyRatingBoxState.kOpen
            else
                rewardTable[configIndex].state = FamilyRatingBoxState.kLock
            end
        end

        if maxCanRewardIndex ==table.count(rewardHistory) then
            isCanReward = false
            if #rewardConfig == maxCanRewardIndex then
                minCanRewardIndex = maxCanRewardIndex
            end
        end
        local minChallengeStarLv = rewardConfig[minCanRewardIndex].min 
        return rewardTable,isCanReward,minChallengeStarLv
    end
    local rankScore = self:getRankScoreModel()
    local minScore = rankScore:getMinScore()
    local rewardCofnigs ={
        [FamilyRatingRewardType.kType1] = {reward = self.m_seasonRewardList,minScore = minScore,configTable =LegionRankScoreConfig },--赛季
        [FamilyRatingRewardType.kType3] = {reward = self.m_personRewardList,minScore = self:getPersonScore(),configTable =LegionQuarterlyPersonConfig },--个人
    }
    local rewardData = rewardCofnigs[rewardType]
    if rewardData then
        local function calcCloseCount(rewardTable)
            local res =0
            for id,reward in pairs(rewardTable) do
                if reward and reward.reward and #reward.reward>0 then
                    res  = res + 1
                end
            end
            return res
        end
        self:resetRewardType(rewardType)
        if rewardType == FamilyRatingRewardType.kType3 then--个人
            self.m_personRewardList = getRewardList(table.clone(self.m_personRewardList),rewardData.minScore,rewardData.configTable,receivedData)
            self.m_personIsGray = calcCloseCount(self.m_personRewardList) == #receivedData
        elseif rewardType == FamilyRatingRewardType.kType1 then--赛季
            self.m_seasonRewardList = getRewardList(table.clone(self.m_seasonRewardList),rewardData.minScore,rewardData.configTable,receivedData)
            self.m_seasonIsGray = calcCloseCount(self.m_seasonRewardList)== #receivedData
        end
    end
    return res
end

--获取个人赛季奖励列表
function This:getSeasonOrPersonalRewards(rewardType)
    
    local res = nil
    if rewardType then
        if rewardType == FamilyRatingRewardType.kType3 then--个人
            res = self.m_personRewardList
        elseif rewardType == FamilyRatingRewardType.kType1 then--赛季
            res = self.m_seasonRewardList
        end
    end
    return res
end


--赛季积分
function This:setSeasonScore(SeasonScore)
    self.m_SeasonScore = SeasonScore
end

function This:getSeasonScore()
    return self.m_SeasonScore or 0
end

-- 设置自己的积分
function This:setPersonScore(PersonScore)
    self.m_PersonScore = PersonScore
end

function This:getPersonScore()
    return self.m_PersonScore or 0
end

-- table转换
function This:convertData( tableData )
    return GameUtil:convertData( tableData )
end

function This:resetScoolSize2(node,widthLimit,_height ,text)
    GameUtil:resetScoolSize3(node,widthLimit,_height ,text)
end