local This , super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local ChatManager = require("Chat.ChatManager")

local Friend = require("Friend.Friend")
local Chat = require("Chat.Chat")
local UpdateList = require("Base.UpdateList")
local VipManager = require("Vip.VipManager")

ResponseType = 
{
	kAgree = 1,
	kRefuse = 2,
}

function This:init()
	super.init(self)
	self.m_recvRequest = {}
	self.m_chatRecord = {}
	self.m_chatMessage = {}
	self.m_chatFriend = UpdateList:create()
    self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self,10900,self.onInitFriendInfoNotify)
	EventDispatcher:addListener(self,10800,self.onFriendChatNotify)
	EventDispatcher:addListener(self,10801,self.onInitHistoryNotify)

    EventDispatcher:addListener(self, 10901,self.onSearchPlayerReply)
    EventDispatcher:addListener(self, 10902,self.onApplyAddFriendReply)
    EventDispatcher:addListener(self, 10903,self.onRespondApplyReply)

    EventDispatcher:addListener(self, 10802,self.onDeleteHistoryReply)
    EventDispatcher:addListener(self, 10904,self.onDeleteFriendReply)
	EventDispatcher:addListener(self, 10906,self.onFriendResponseNotify)
	EventDispatcher:addListener(self, 10907,self.onGetFriendDetailReply)
	EventDispatcher:addListener(self, 10909,self.onOnlineStateChangedNotify)
	EventDispatcher:addListener(self, 10910,self.onAddBlackFriendReply)
	EventDispatcher:addListener(self, 10911,self.onRemoveBlackFriendReply)
end

function This:notifyObservers(funcName,...)
	if not self.m_blocked then
		super.notifyObservers(self, funcName,...)
	end
end

function This:getFriend(id)
	return self.m_chatFriend:getNode(id)
end

function This:addFriend(info)
	local friend = self.m_chatFriend:getNode(info.id)
	if not friend then
		friend = Friend:create(info)
		self.m_chatFriend:addNode(friend,info.id)
	else
		friend:updateInfo(info)
	end
	friend:setIsFriend(true)
	self:notifyObservers("onAddFriend",friend)
	return friend
end

function This:removeFriend(id)
	if self.m_chatFriend:hasNode(id) then
		local friend = self.m_chatFriend:getNode(id)
		friend:setIsFriend(false)
		self:notifyObservers("onRemoveFriend",friend)
	end
end

function This:getFriendList()
	local list = self.m_chatFriend:getNodeList(function(friend)
		return friend:isFriend()
	end)
	table.sort(list,function(a,b)
		if a:isOnline() and not b:isOnline() then
			return true
		end
		if a:isOnline() and b:isOnline() then
			if a:getLastSendTime() ~= b:getLastSendTime() then
				return a:getLastSendTime() > b:getLastSendTime()
			end
			if a:getId() < b:getId() then
				return true
			end
		end
		if not a:isOnline() and not b:isOnline() then
			if a:getLastSendTime() ~= b:getLastSendTime() then
				return a:getLastSendTime() > b:getLastSendTime()
			end
			if a:getId() < b:getId() then
				return true
			end
		end
		return false
	end)
	return list
end

function This:getBlackFriend()
	local list = self.m_chatFriend:getNodeList(function(friend)
		return friend:isBlackFriend()
	end)
	return list
end

function This:getAllRecvRequest()
	local result = {}
	for i , id in pairs(self.m_recvRequest) do
		local friend = self.m_chatFriend:getNode(id)
		if friend and friend:getLastSendTime() > 0 then
			result[id] = true
		end
	end
	return result
end

function This:addSendRequest(info)
	local friend = self.m_chatFriend:getNode(info.id)
	if not friend then
		friend = Friend:create(info)
		friend:setIsSendInvite(true)
		self.m_chatFriend:addNode(friend,info.id)
	else
		friend:setIsSendInvite(true)
	end
	--[[local chat = self:createChat(friend,ChatType.kAddFriend)
	friend:addChatMessage(chat)]]

	self:notifyObservers("onAddSendReqeust",friend)
	return friend
end

function This:removeSendRequest(id)
	local friend = self.m_chatFriend:getNode(id)
	if friend then
		friend:setIsSendInvite(false)
		self:notifyObservers("onRemoveSendRequest",friend)
	end
end

function This:createChat(friend,chatType)
	local from_role = {}
	local to_role = {}
	if chatType == ChatType.kAddFriend then
		from_role = {id = friend:getId() , name = friend:getName(), icon = friend:getIcon(), cur_rank_level = friend:getRankLv(), vip_list = friend:getVipList()}
		to_role = {id = GamePlayer:getPlayerID() , name = GamePlayer:getRoleName(), icon = GamePlayer:getIcon(), cur_rank_level = 0 , vip_list = VipManager:getMaxMemberLevel()}
	elseif chatType == ChatType.kAgreeAdd then
		to_role = {id = friend:getId() , name = friend:getName(), icon = friend:getIcon(), cur_rank_level = friend:getRankLv(), vip_list = friend:getVipList()}
		from_role = {id = GamePlayer:getPlayerID() , name = GamePlayer:getRoleName(), icon = GamePlayer:getIcon(), cur_rank_level = 0 , vip_list = VipManager:getMaxMemberLevel()}
	elseif chatType == ChatType.kRefuseAdd then
		to_role = {id = friend:getId() , name = friend:getName(), icon = friend:getIcon(), cur_rank_level = friend:getRankLv(), vip_list = friend:getVipList()}
		from_role = {id = GamePlayer:getPlayerID() , name = GamePlayer:getRoleName(), icon = GamePlayer:getIcon(), cur_rank_level = 0 , vip_list = VipManager:getMaxMemberLevel()}
	end
	local chat = Chat:create({
		from_role = from_role,
		to_role = to_role,
		content = "",
		channel = ChatChannelType.kPrivate,
		time = GamePlayer:getServerTime(),
		type = chatType,
		pos = 0,
	})
	return chat
end

function This:addRecvRequest(info)
	local friend = self.m_chatFriend:getNode(info.id)
	if not friend then
		friend = Friend:create(info)
		friend:setIsRecvInvite(true)
		self.m_chatFriend:addNode(friend,info.id)
	else
		friend:setIsRecvInvite(true)
	end
	self.m_recvRequest[info.id] = true

	self:notifyObservers("onAddRecvReqeust",friend)
	EventDispatcher:dispatcher("friend_recv_request",friend)
	return friend
end

function This:removeRecvRequest(id)
	local friend = self.m_chatFriend:getNode(id)
	if friend then
		friend:setIsRecvInvite(false)
		self.m_recvRequest[id] = nil
		self:notifyObservers("onRemoveRecvRequest",friend)
		EventDispatcher:dispatcher("friend_remove_request",friend)
	end
end

function This:haveRecvRequest(friendId)
	if not friendId then
		for friendId , _ in pairs(self.m_recvRequest) do
			local friend = self:getFriend(friendId)
			if friend:getChatMessageCount() > 0 then
				return true
			end
		end
	else
		if self.m_recvRequest[friendId] then
			local friend = self:getFriend(friendId)
			return friend:getChatMessageCount() > 0 
		end
	end
	return false
end

function This:addChatMessage(msgchat)
	local friendInfo = nil
	if msgchat.from_role.id ~= GamePlayer:getPlayerID() then
		friendInfo = msgchat.from_role
	else
		friendInfo = msgchat.to_role
	end
	if not self.m_chatFriend:hasNode(friendInfo.id) then
		local friend = self:getFriend(friendInfo.id)
		if not friend then
			friend = Friend:create(friendInfo)
		end
		self.m_chatFriend:addNode(friend,friend:getId())
	end

	local chat = Chat:create(msgchat)
	local friend = self.m_chatFriend:getNode(friendInfo.id)
	friend:addChatMessage(chat)

	self:notifyObservers("onAddChatMessage",chat)
	EventDispatcher:dispatcher("friend_chat_changed")
	return chat
end

function This:getChatFriendList()
	local list = self.m_chatFriend:getNodeList(function(friend)
		if not friend:isBlackFriend() and friend:getLastSendTime() > 0 then
			return true
		end
		return false
	end)
	table.sort(list,function(a,b)
		return a:getLastSendTime() > b:getLastSendTime()
	end)
	return list
end

function This:removeChatMessage(id)
	local friend = self.m_chatFriend:getNode(id)
	if friend then
		friend:removeChatMessage()
		friend:setIsRecvInvite(false)
		self:notifyObservers("onRemoveChat",friend)
	end 
end

function This:addBlackFriend(info)
	local friend = self.m_chatFriend:getNode(info.id)
	if not friend then
		friend = Friend:create(info)
		friend:setIsBlackFriend(true)
		self.m_chatFriend:addNode(friend,info.id)
	else
		friend:setIsBlackFriend(true)
	end
	self:notifyObservers("onAddBlackFriend",friend)
	return friend
end

function This:removeBlackFriend(id)
	local friend = self.m_chatFriend:getNode(id)
	if friend then
		friend:setIsBlackFriend(false)
		self:notifyObservers("onRemoveBlackFriend",friend)
	end
end

function This:updateFriendOnlineState(id,isOnline)
	local friend = self.m_chatFriend:getNode(id)
	if friend then
		friend:setIsOnline(isOnline)
	end
end

function This:readFriendChatMessage(friendId)
	self.m_chatRecord[friendId] = nil
	EventDispatcher:dispatcher("friend_chat_changed")
end

function This:haveNoReadChatMessage()
	return next(self.m_chatRecord) ~= nil
end

function This:haveFriendNoReadChatMessage(friendId)
	if friendId then
		return self.m_chatRecord[friendId] ~= nil
	else
		for id , _ in pairs(self.m_chatRecord) do
			local friend = self.m_chatFriend:getNode(id)
			if friend:isFriend() then
				return true
			end
		end
	end
end

-------------------------------------------------------
--初始化好友信息
function This:onInitFriendInfoRequest()
	self.m_blocked = true
	Socket:sendAndLock(10900, {})
end 

function This:onInitFriendInfoNotify(msg)
	for i , info in pairs(msg.friend_list) do
		self:addFriend(info)
	end

	for i , info in pairs(msg.request_list) do
		self:addRecvRequest(info)
	end

	for i , info in pairs(msg.send_request_list) do
		self:addSendRequest(info)
	end

	for i , info in pairs(msg.black_list) do
		self:addBlackFriend(info)
	end

	for i , pid in pairs(msg.friend_online_list) do
		self:updateFriendOnlineState(pid,true)
	end

	for i , pid in pairs(msg.friend_offline_list) do
		self:updateFriendOnlineState(pid,false)	
	end
	self.m_blocked = false
	self:notifyObservers("onRefreshFriendUi")
	self:notifyObservers("onFriendChanged")
end

--好友聊天通知
function This:onFriendChatNotify(msg)
	if msg.chat.channel == ChatChannelType.kPrivate then
		if msg.chat.from_role.id ~= GamePlayer:getPlayerID() then
			local friend = self:getFriend(msg.chat.from_role.id)
			if friend and friend:isBlackFriend() then
				return
			end
			self.m_chatRecord[msg.chat.from_role.id] = true
		else
			local friend = self:getFriend(msg.chat.to_role.id)
			if friend and friend:isBlackFriend() then
				return
			end
			self.m_chatRecord[msg.chat.to_role.id] = true
		end
		self:addChatMessage(msg.chat)
	end
end

--好友历史消息通知
function This:onInitHistoryNotify(msg)
	for i , info in pairs(msg.chat_list) do
		if info.channel == ChatChannelType.kPrivate then
			self:addChatMessage(info)
		end
	end
end

--搜索好友
function This:onSearchPlayerRequest(keyword)
    keyword = string.trim(keyword)
    if keyword == "" then
        g_scene:showTips(Serialize["friend_FriendManager_1"])
        return
    end
    Socket:sendAndLock(10901, {name=keyword})
end

function This:onSearchPlayerReply(msg)
	if msg.err_code == 0 then
		local friendList = {}
        for i , player in ipairs(msg.role) do 
			local friend = Friend:create(player)
			if not friend:isBlackFriend() then
				table.insert(friendList,friend)
			end
        end
        self:notifyObservers("onSearchPlayer", friendList)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

--添加好友
function This:onApplyAddFriendRequest(pid)
	local friend = self.m_chatFriend:getNode(pid)
	if friend then
		if friend:isFriend() then
			g_scene:showTips(Serialize["friend_FriendManager_2"])
			return
		elseif friend:isBlackFriend() then
			g_scene:showTips(Serialize["friend_FriendManager_3"])
			return
		end
	end
	Socket:sendAndLock(10902, {role_id=pid})
end

function This:onApplyAddFriendReply(msg)
	if msg.err_code == 0 then
		self:addSendRequest(msg.role)
		self:notifyObservers("onApplyAddFriend")
		g_scene:showTips(Serialize["friend_FriendManager_4"])
    else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--好友要求响应
function This:onRespondApplyRequest(pid, response)
    Socket:sendAndLock(10903, {role_id=pid, agree = response})
end

function This:onRespondApplyReply(msg)  --添加好友回应
	if msg.err_code == 0 then
		local friend = self:getFriend(msg.target_id)
		if not friend then
			return
		end
		self:removeRecvRequest(msg.target_id)
		--[[if msg.agree == ResponseType.kAgree then
			local chat = self:createChat(friend,ChatType.kAgreeAdd)
			friend:addChatMessage(chat)
		else
			local chat = self:createChat(friend,ChatType.kRefuseAdd)
			friend:addChatMessage(chat)
		end]]
		self:notifyObservers("onResponseApply",msg.agree)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--发送好友聊天消息
function This:onSendChatMessageRequest(pid, content)
    content = string.trim(content)
    if #content == 0 then
        g_scene:handleErrorCode("nullChatContent")
        return
    end
    -- if not cc.Label.hasLetter(content) then
    --     g_scene:handleErrorCode("invalidChatContent")
    --     return
    -- end
    ChatManager:onSendMessageRequest(ChatChannelType.kPrivate ,content , pid)
end

--删除历史聊天
function This:onDeleteHistoryRequest(pid)
    Socket:sendAndLock(10802, {role_id = pid})
end

function This:onDeleteHistoryReply(msg)
	if msg.err_code == 0 then
		self:removeChatMessage(msg.role_id)
		self:notifyObservers("onDeleteChatMessage",msg.role_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

--删除好友
function This:onDeleteFriendRequest(pid)
    Socket:sendAndLock(10904, {role_id = pid})
end

function This:onDeleteFriendReply(msg)
    if msg.err_code == 0 then
		self:notifyObservers("onDeleteFriend")
    else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--在线状态变化
function This:onOnlineStateChangedNotify(msg)
   for i , pid in pairs(msg.online_change_list) do
		local friend = self.m_chatFriend:getNode(pid)
		if friend then
			friend:setIsOnline(true)
			self:notifyObservers("onFriendStateChanged",friend)
		end
   end
   for i , pid in pairs(msg.offline_change_list) do
		local friend = self.m_chatFriend:getNode(pid)
		if friend then
			friend:setIsOnline(false)
			self:notifyObservers("onFriendStateChanged",friend)
		end
   end
end

function This:onFriendResponseNotify(msg)
	for i , info in pairs(msg.add_friend_list) do
		local friend = self:addFriend(info)
		friend:setIsOnline(true)
	end

	for i , info in pairs(msg.del_friend_list) do
		self:removeFriend(info.id)
	end

	for i , info in pairs(msg.add_request_list) do
		local friend = self:addRecvRequest(info)
		friend:setIsOnline(true)
	end

	for i , info in pairs(msg.del_request_list) do
		self:removeRecvRequest(info.id)
	end

	for i , info in pairs(msg.add_send_list) do
		local friend = self:addSendRequest(info)
		friend:setIsOnline(true)
	end

	for i , info in pairs(msg.del_send_list) do
		self:removeSendRequest(info.id)
	end

	for i , info in pairs(msg.add_black_list) do
		local friend = self:addBlackFriend(info)
		friend:setIsOnline(true)
	end

	for i , info in pairs(msg.del_black_list) do
		self:removeBlackFriend(info.id)
	end

	self:notifyObservers("onFriendChanged")
end

--添加黑名单

function This:onAddBlackFriendRequest(id)
	Socket:sendAndLock(10910,{role_id = id})
end

function This:onAddBlackFriendReply(msg)
	if msg.err_code == 0 then
		g_scene:showTips(Serialize["friend_FriendManager_5"])
		self:notifyObservers("onAddBlack")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--删除黑名单
function This:onRemoveBlackFriendRequest(id)
	Socket:sendAndLock(10911,{role_id = id})
end

function This:onRemoveBlackFriendReply(msg)
	if msg.err_code == 0 then
		self:notifyObservers("onRemoveBlack")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--查看好友详细信息
function This:onGetFriendDetailRequest(id)
	Socket:sendAndLock(10907,{role_id = id})
end

function This:onGetFriendDetailReply(msg)
	if msg.err_code == 0 then
		self:notifyObservers("onGetFriendDetail",msg)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end