
local This , super = class(...,"NodeUI")
local FunctionNoticeConfig = require("Metadatas.FunctionNoticeConfig")
local GameUtil = require("Util.GameUtil")
local FunctionNoticeManager = require("Function.FunctionNoticeManager")


function This:initData(data)
    self.m_data = data
end

function This:initNode(data)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ZuiQiangZhanLiPromoteNoticeItem.csb")
    self:addChild(self.m_csNode)
    self:setCascadeOpacityEnabled(true)

    local w, h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)
    self.m_csNode:setPosition(w/2, h/2) 
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        self.m_csNode.boxLock:setTouch({
            isDynamic = true, 
            isContainer  = true,
            onEnded = function ()
                self:onBoxClick()
            end,
        })
    end
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:setModel(data)
    self.m_data = data
    self:refresh()
end

function This:refresh()
    local data = self.m_data
    local csNode = self.m_csNode
    local config = FunctionNoticeConfig[self.m_data.id]
    local img = data.isOpen and "MenuScene/boxGet4.png" or "MenuScene/boxLock4.png"

    csNode.boxLock:setImage(img)
    csNode.tubiao:setImage(config.icon)
    csNode.name:setString(config.name)
    csNode.text:setString(config.desc)
    csNode.tiaojian:setVisible(not data.isOpen)
    csNode.yikaiqi:setVisible(data.isOpen)
    csNode.yikaiqi:setPositionY( csNode.tiaojian:getPositionY() )
    csNode.tiaojian:setString(data.openTips)

    local effectList = {"BoxLing","BoxLing2"}
    local box = csNode.boxLock

    if data.isOpen then
        if data.rewardState == 0 then
            --未领
            for _, name in ipairs(effectList) do
                if not box[name] then
                    local eff = GameUtil:createEffect(name, true, box)
                    eff:setScale(1.2)
                    box:addChild(eff, -1)
                    box[name] = eff
                end
                box[name]:setVisible(true)
            end
            box:clearShader()
        else
            --已领
            box:setGrayShader()
            GameUtil:hideNodeList(box, effectList)
        end
    else
        box:clearShader()
        GameUtil:hideNodeList(box, effectList)
    end

end

function This:onBoxClick()
    local data = self.m_data

    if data.isOpen and data.rewardState == 1 then
        return
    end

    local state = 0
    if data.isOpen and data.rewardState == 0 then
        state = 1
    end

    g_scene:openDialog("Function.FunctionRewardBoxPanel", {
        title = Serialize[60659],
        state = state, -- 0不可领取，1可领取
        text = data.openTips,
        reward = data.reward,
        onBtnClick = function ( ... )
            FunctionNoticeManager:onSystemOpenRewardRequest(data.id)
        end
    })
end

