

local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local FunctionNoticeConfig = require("Metadatas.FunctionNoticeConfig")
local GamePlayer = require("Player.GamePlayer")
local TerrBuildManager = require("Territory.TerrBuildManager")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local PopDialogManager = require("UI.PopDialogManager")
local ViewUtil = require("UI.ViewUtil")
local HeroManager = require("Hero.HeroManager")
local GuideManager = require("Guide.GuideManager")
local Application = require("cc.Application")


FunctionOpenCondition = {
	kRoleLv 			= 1, --主公等级
	kGuanFuLv 			= 2, --官府等级
	kGuanZhiLv 			= 3, --官职等级
	kOpenServerDay 		= 4, --开服天数
}

FunctionOpenId = {
	kJunTuan = 1, --军团
	kRiChang = 2, --日常
	kJingJiChang = 3, --竞技场
	kMingWang = 4, --名望
	kZhanYi = 5, --战役
	kSuMing = 6, --宿命
	kZhengJiBaoXiang = 7, --政绩宝箱
	kWuDaoShangDian = 8, --武道商店
	kZhenFa = 9, --阵法
	kJiNengRongHe = 10, --技能融合
	kWangDao = 11, --王道
	kZhuangBeiXiLian = 12, --装备洗练
	kJueXing = 13, --觉醒
	kShiJieBoss = 14, --世界boss
	kPeiDian = 15, --配点
	kZhenHeTa = 16, --镇河塔
	kFaBao = 17, --法宝
	kDianFengJingJiChang = 18, --巅峰竞技场
	kMeiRiTaoFa = 19, --每日讨伐
	kTieJiangPu = 20, --铁匠铺
	kJiaoChang = 21, --校场
	kQiBingYing = 22, --骑兵营
	kGongBingYing = 23, --弓兵营
	kCheBingYing = 24, --车兵营
	kJueWei = 25, --爵位
	kJunZhen = 26, --军阵
	kShenShou = 27, --神兽
}

function This:init()
	super.init(self)
	self.m_popList = {}
	self:initListener()
end

function This:initListener()
    EventDispatcher:addListener(self, 10216, self.onSystemOpenRewardReply)
    MatchManager:addObserver(self)
    GamePlayer:addObserver(self)
    TerrBuildManager:addObserver(self)
end

------------------------------------------------------------------------------------------
--网络协议
------------------------------------------------------------------------------------------

function This:onSystemOpenRewardRequest(id)
	Socket:sendAndLock(10216, {id = id})
end

function This:onSystemOpenRewardReply(msg)
	if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    local id = msg.id

    local reward = GamePlayer:getSystemOpenReward()
    table.insert(reward, id)

    g_scene:showReward(FunctionNoticeConfig[id].reward)
    self:notifyObservers("onSystemOpenRewardSuccess", {id = id})
    EventDispatcher:dispatcher("system_open_reward_success", {id = id})
end

------------------------------------------------------------------------------------------
--业务逻辑
------------------------------------------------------------------------------------------

function This:getFunctionList()
	local list = {}

	for k, v in pairs(FunctionNoticeConfig) do
		local t = {}
		t.id 				= v.id
		t.isOpen 			= self:checkOpen(v.id)
		t.openLv 			= v.open_level
		t.openTips 			= self:getOpenTips(v.id)
		t.rewardState 		= self:getRewardState(v.id) --0未领，1已领
		t.reward 			= v.reward
		table.insert(list, t)
	end	

	table.sort(list, function (a, b)
		if a.rewardState ~= b.rewardState then
			return a.rewardState < b.rewardState
		else
			if a.isOpen ~= b.isOpen then
				local aOpen = a.isOpen == true and 1 or 0
				local bOpen = b.isOpen == true and 1 or 0
				return aOpen > bOpen
			end
			
			if a.openLv ~= b.openLv then
				return a.openLv < b.openLv
			end

			return a.id < b.id
		end
	end)

	return list
end

function This:checkOpen(id)
	local config = FunctionNoticeConfig[id]
	local condition = config.open_condition
	local openVal = config.open_level

	if condition == FunctionOpenCondition.kRoleLv then
		local lv = GamePlayer:getLevel() or 1
		return lv >= openVal

	elseif condition == FunctionOpenCondition.kGuanFuLv then
		local lv = TerrBuildManager:getBuildLevel(TerrBuildType.kGuanfu) or 1
		return lv >= openVal

	elseif condition == FunctionOpenCondition.kGuanZhiLv then
		local lv = MatchManager:getRankLv() or 1
		return lv >= openVal

	elseif condition == FunctionOpenCondition.kOpenServerDay then
		local days = GamePlayer:getOpenServerDays()
		return days >= openVal

	end

	return false
end

--奖励的状态，0未领，1已领
function This:getRewardState(id)
	local list = GamePlayer:getSystemOpenReward() or {}
	for i, sysId in ipairs(list) do
		if sysId == id then
			return 1
		end
	end
	return 0
end

function This:getOpenTips(id)
	local config = FunctionNoticeConfig[id]
	local condition = config.open_condition
	local openVal = config.open_level 

	if condition == FunctionOpenCondition.kRoleLv then
		return string.format(Serialize[60309], openVal)
	elseif condition == FunctionOpenCondition.kGuanFuLv then
		return string.format(Serialize[61104], openVal)
	elseif condition == FunctionOpenCondition.kGuanZhiLv then
		return string.format(Serialize[61105], MatchingConfig[openVal].rank_level)
	elseif condition == FunctionOpenCondition.kOpenServerDay then
		return string.format(Serialize[61106], openVal)
	end

	return ""
end

--主界面功能预告
function This:getFunctionNoticeForHomeUI()
	local list = {}

	for k, v in pairs(FunctionNoticeConfig) do
		if v.open_condition == FunctionOpenCondition.kRoleLv and v.is_notice == 1 then
			if not self:checkOpen(v.id) then
				table.insert(list, v)
			end
		end
	end

	if #list > 1 then
		table.sort(list, function (a, b)
			return a.open_level < b.open_level
		end)
	end

	return list[1]
end

--前往
function This:gotoFunction(id)
	local config = FunctionNoticeConfig[id]
	-- if config.guide_data and config.guide_data.id ~= nil then
	-- 	GuideManager:triggerAncGuider(config.guide_data)
	-- 	return
	-- end

	local ui = config.ui_path
	if type(ui) == "table" then
		local name = ui.name
		local arg = ui.arg
		if name == "HeroFate" then
			--武将宿命
			local heroList = HeroManager:getAllHeroGotWithSort(nil, false)
			if #heroList > 0 then
				g_scene:openDialog("Hero.HeroPanel", {heroId = heroList[1]:getId(), tab = "fate"})
			else
				g_scene:showTips(Serialize[61107])
			end
		elseif name == "HeroAwake" then
			--武将觉醒
			local heroList = HeroManager:getAllHeroGotWithSort(function (hero)
				return hero:isAwaken()
			end, false)
			if #heroList > 0 then
				g_scene:openDialog("Hero.HeroPanel", {heroId = heroList[1]:getId(), tab = "equip"})
			else
				g_scene:showTips(Serialize[61108])
			end
		elseif name == "HeroFaBao" then
			--武将法宝
			local heroList = HeroManager:getAllHeroGotWithSort(nil, false)
			if #heroList > 0 then
				g_scene:openDialog("Hero.HeroPanel", {heroId = heroList[1]:getId(), tab = "equip"})
			else
				g_scene:showTips(Serialize[61107])
			end			
		elseif name == "EquipClear" then
			--装备洗练
			local heroList = HeroManager:getAllHeroGotWithSort(nil, false)
			if #heroList > 0 then
				g_scene:openDialog("Hero.HeroPanel", {heroId = heroList[1]:getId(), tab = "equip"})
				g_scene:openDialog("Equip.EquipStrengthPanel",{heroId = heroList[1]:getId(), tab = "hero", subTab = "Clear"})
			else
				g_scene:showTips(Serialize[61107])
			end
		elseif name == "NewActivity.WudaoShopPanel" then
			--武道商店
			local build = TerrBuildManager:getBuildByType(TerrBuildType.kWuDaoShangDian, 1)
			local func = build:getRequest()
			if func then
				func()
			end
		elseif name == "HeroPeiDian" then
			--武将配点
			local openLv = FunctionNoticeConfig[FunctionOpenId.kPeiDian].open_level
			local heroList = HeroManager:getAllHeroGotWithSort(function (hero)
				return hero:getLevel() >= openLv
			end, false)
			if #heroList > 0 then
				g_scene:openDialog("Hero.HeroPanel", {heroId = heroList[1]:getId(), tab = "equip"})
				g_scene:openDialog("Hero.AllotAttrPointPanel", heroList[1])
			end
		elseif name == "Awake.LordPanel" then
			g_scene:openDialog("Awake.LordPanel", arg)
		else
			ViewUtil:jumpToPanel(ui.name, ui.arg)
		end
	end
end

--某些条件改变时检查
function This:onConditionChangeCheck(condition, compareVal)
	local openList = {}
	local popList = {}

	for k, v in pairs(FunctionNoticeConfig) do
		if v.open_condition == condition and v.open_level == compareVal then
			table.insert(openList, v)
			if v.is_notice == 1 then
				table.insert(popList, v)
			end
		end
	end	

	if #popList > 1 then
		table.sort(popList, function (a, b)
			return a.id < b.id
		end)
	end

	for i, v in ipairs(popList) do
		table.insert(self.m_popList, v)
	end

	--延时弹窗
	local timer = cc.Application:getTimer()
	timer:setTimer(1, function()
		self:onNoticePopDialog()
	end, "notice_pop_dialog")
	

	if #openList > 0 then
		EventDispatcher:dispatcher("new_system_open", openList)
	end
end

function This:onPlayerAttrChanged(field, lastValue, value)
	--主公等级检查
	if field == PlayerAttr.level and lastValue ~= value then
		self:onConditionChangeCheck(FunctionOpenCondition.kRoleLv, value)
	end
end

function This:onBuildUpFinish(id)
	--官府等级检查
	if id == TerrBuildType.kGuanfu then
		local lv = TerrBuildManager:getBuildLevel(TerrBuildType.kGuanfu) or 1
		self:onConditionChangeCheck(FunctionOpenCondition.kGuanFuLv, lv)
	end
end

function This:onMatchRankLvChanged()
	--官职等级检查
	local lv = MatchManager:getRankLv() or 1
	self:onConditionChangeCheck(FunctionOpenCondition.kGuanZhiLv, lv)
end

function This:onDayPassed()
	--开服天数检查
	local lv = GamePlayer:getOpenServerDays()
	self:onConditionChangeCheck(FunctionOpenCondition.kOpenServerDay, lv)	
end

--是否有未领取的奖励
function This:isExistReward()
	for k, v in pairs(FunctionNoticeConfig) do
		local isOpen = self:checkOpen(v.id)
		if isOpen then
			if self:getRewardState(v.id) == 0 then
				return true
			end
		end
	end		
	return false
end

function This:onNoticePopDialog()
	if #self.m_popList == 0 then
		return
	end

	for i, v in ipairs(self.m_popList) do
		PopDialogManager:addPopDialog({
			bCheck = true,
			scene = "MenuScene", 
			name = "Function.FunctionOpenPanel", 
			arg = {id = v.id},
			level = DialogLevel.kForLevel
		})
	end
	self.m_popList = {}
end

function This:getOpenLevel(id)
	local config = FunctionNoticeConfig[id]
	if config then
		return config.open_level
	end
	return 0
end