-- @Author: Su
-- @Date:   2019-12-27 15:12:05
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-17 15:46:47
-- @File Name: AnimalsExchangePanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")

function This:initNode(data)
    self.m_animalId = data.id
    self.m_isRed =  data.red
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroExchange.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    if self.m_isRed then
        self.m_activity = NewActivityManager:getActivity(NewActivityType.kRedHeroTable)
        self.m_itemId = self.m_activity:getShopMoney()
    else
        self.m_activity = NewActivityManager:getActivity(NewActivityType.kAnimalTable)
        self.m_itemId = self.m_activity:getLuckBestReward() + 1000
      
    end
    self.m_num = 0
    self.m_maxNum = DepotManager:getItemCount(self.m_itemId)
    self:refresh()
end
function This:onEnter()
    super.onEnter(self)
	 self.m_activity:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_activity:removeObserver(self)
end

function This:refresh()

    local node = self.m_csNode
    node.wannengbg.icon1:setVisible(true)
    node.wannengbg.name:setColor(QualityColor[Quality.kOrange].r, QualityColor[Quality.kOrange].g, QualityColor[Quality.kOrange].b)
    node.wannengbg.icon:setImage(ItemUtil.getIconImage(self.m_itemId))
    node.wannengbg.bg:setImage(ItemUtil.getQualityImage(self.m_itemId))
    node.wannengbg.num:setString(string.format( "%d/%d",self.m_num,self.m_maxNum))
    node.wannengbg.name:setString(ItemUtil.getName(self.m_itemId))
    node.text:setString(string.format( Serialize[61126],ItemUtil.getName(self.m_itemId),ItemUtil.getName(self.m_animalId)))

    node.wujiangbg.icon:setImage(ItemUtil.getIconImage(self.m_animalId))
    node.wujiangbg.name:setString(ItemUtil.getName(self.m_animalId))
    node.wujiangbg.bg:setImage(ItemUtil.getQualityImage(self.m_itemId))
    node.wujiangbg.num:setString(self.m_num)
    node.changecount:setString(self.m_num)

    if self.m_maxNum == 0 then
        self.m_csNode.exchange:setGrayShader()  
    else
        self.m_csNode.exchange:clearShader()
    end
end

function This:getInfo(id)
    return {fileName = "UI.ItemTips", itemId = id, itemType = ItemUtil.getType(id)} 
end

function This:getTipStr()
    if self.m_isRed then
          return Serialize[61127]
    end
    return Serialize[61128]
end

function This:initTouchEvent()

  local node = self.m_csNode
    
    g_scene:handleHelpTips(node.wannengbg.bg, function ()
    return self:getInfo(self.m_itemId)
    end)
    g_scene:handleHelpTips(node.wujiangbg.bg, function ()
    return self:getInfo(self.m_animalId)
    end)

    self.m_csNode.close:setTouchEnded(function()
      self:close()
    end)
    
    self.m_csNode.reduce:setTouchEnded(function()
      if self.m_num > 0 then
        self.m_num = self.m_num - 1
        self:refresh()
      end
    end)
    
    self.m_csNode.increase:setTouchEnded(function()
      if self.m_num < self.m_maxNum then
        self.m_num = self.m_num + 1
        self:refresh()
      elseif self.m_maxNum == 0 then
        g_scene:showTips(self:getTipStr())
      else
        g_scene:showTips(Serialize[61129])
      end
    end)
    
    self.m_csNode.jia10:setTouchEnded(function()
      if self.m_maxNum == 0 then
        g_scene:showTips(self:getTipStr())
        return
      end
      if self.m_num >= self.m_maxNum then
        g_scene:showTips(Serialize[61129])
        return
      end
      if self.m_num + 10 > self.m_maxNum then
        self.m_num = self.m_maxNum
        --g_scene:showTips(Serialize[61130])
      else
        self.m_num = self.m_num + 10
      end
      self:refresh()
    end)  


    self.m_csNode.Max:setTouchEnded(function()
      if self.m_maxNum == 0 then
        g_scene:showTips(self:getTipStr())
        return
      end
      if self.m_num >= self.m_maxNum then
        g_scene:showTips(Serialize[61129])
      else
        self.m_num = self.m_maxNum
      end
      self:refresh()
    end)
    
    self.m_csNode.exchange:setTouchEnded(function()
      if self.m_maxNum == 0 then
        return
      end
      if self.m_num > 0 then
              self.m_activity:onExchangeRequest(self.m_animalId,self.m_num)
      else
        g_scene:showTips(Serialize[61131])
      end
    end)
end

function This:onExchangeAnimal()
    self:close()
end

