-- @Author: zheng
-- @Date:   2019-05-09 15:46:27
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-08-30 17:20:12
-- @File Name: GameBeast.lua

local This = class(...)
local GameUtil = require("Util.GameUtil")
local ItemConfig = require("Metadatas.ItemConfig")
local MythicalAnimalsAttributeConfig = require("Metadatas.MythicalAnimalsAttributeConfig")
local GodBeastManager = require("GodBeast.GodBeastManager")
local GamePlayer = require("Player.GamePlayer")
function This:create(...)
    local chapter = setmetatable({},self)
    chapter:init(...)
    return chapter
end

function This:init(id)
    self.m_id = id
    -- self.m_stages = {}
    self.m_config = MythicalAnimalsAttributeConfig[id]
    self.m_starData = {}
    self.m_lv = 0
    self.m_starLv = 0
    self.m_exp = 0
    -- self.m_totalStar = 0
    -- self.m_starReward = {false,false,false}
    -- self.m_maxStar = 0
    -- self.m_preChapter = nil
    -- self.m_nexChapter = nil
    -- self:initVisitInfo()
end

function This:getId()
    return self.m_id
end

function This:getAnim()
    return self.m_config.path
end

function This:setStarLv(starlv)
    self.m_starLv = starlv
end

function This:getId()
    return self.m_id
end

function This:getStarLv()
    return self.m_starLv or 0
end

function This:getMaxStarLv()
    return 5
end

function This:getMadeNeedNum()
    return self.m_config.need_item[2]
end
-- function This:setPos(pos)
--     self.m_pos = pos
-- end

-- function This:getPos()
--     return self.m_pos or 0
-- end
--修改经验
function This:setExp(exp)
    self.m_exp = exp
end

function This:getExp()
    return self.m_exp
end

function This:getMaxExp()
    local expRadio =self:getEXP()
    local _,exp2 = GodBeastManager:getMaxLv(self:getLv())
    local max = expRadio*exp2
    return max
end

function This:isCanUpLv()
    local lvLimit,lvNeedExp = GodBeastManager:getMaxLv(self:getLv(),true)
    
    if lvNeedExp==0 or lvLimit>GamePlayer:getLevel() then
        return false
    end
    local myExp = GodBeastManager:getMyAllExp()
    local curent = self:getExp()
    if myExp>=(lvNeedExp-curent) then
        return true
    end
    return false
end

function This:isCanUpStar()
    local starData = self:getStar()
    if starData and self.m_starLv ~= self:getMaxStarLv() then
        return starData:isCanUp()
    end
    return false
end


function This:setLv(lv)
    self.m_lv = lv
end

function This:getLv()
    return self.m_lv
end

function This:addStar(beastStar)
    if beastStar then
        self.m_starData[beastStar:getStar()]=beastStar
    end
end

function This:getStar(starLv)
    return self.m_starData[starLv or self:getStarLv()]
end

function This:getName()
    return self.m_config.name
end

function This:getNamePic()
    local str = string.format("Icons/Animals/%s.png", self.m_config.name_picture )
   return str
end

function This:getEXP()--经验系数
    return self.m_config.exp_coefficient
end

function This:getQuality()
    return ItemConfig[self.m_id].quality
end

function This:getMixNeed()--合成道具
    return self.m_config.need_item
end

function This:getDesc()
    return self.m_config.introduction
end

function This:getConvertRadio()--转换比例
    return self.m_config.ratio or 1
end

function This:getSpeed()--速度
    local temSpeed = self.m_config.speed
    local starUp = self.m_config.star_up
    return temSpeed+starUp*self:getStarLv()
end

function This:getAnger()--初始怒气值
    return self.m_config.anger
end

function This:getAttr(lv,starLv,notOtherAttr)
    local temLv = lv or self:getLv()
    local temStarlv = starLv or self:getStarLv()
    local star = self:getStar(temStarlv)
    local result ={}
    if star then
        local starParameter = star:getAttrParameter()
        local attrList = self.m_config.attribute
        for _,attrItem in pairs(attrList) do
            local attrId = attrItem[1]
            local attrValue = attrItem[2]
            local attrParameter = attrItem[3]
            local temAttrValue = (attrValue+temLv*attrParameter)*(1+starParameter)
            if result[attrId] then
               temAttrValue = result[attrId] + temAttrValue
            end
            result[attrId] = temAttrValue
        end
        if not notOtherAttr then
            result[AttrType.kFirst] =self:getSpeed()
            result[AttrType.kAngerValue] =self:getAnger()
        end
    end
    return result
end

function This:getConverAttr()
    local result = self:getAttr(nil,nil,true)
    local radio = self:getConvertRadio()
    for id,value in pairs(result) do
        result[id]= result[id]*radio
    end
    -- result[AttrType.kFirst] =self:getSpeed()
    -- result[AttrType.kAngerValue] =self:getAnger()
    return result
end

function This:getAttrStr(lv,starLv)
    local attrList = self:getAttr(lv,starLv)
    local result = {}
    for key,value in pairs(attrList) do
        local name,attrrValue = GameUtil:getAttrStr( key,value,true )
        -- result = result ..name..attrrValue
        local str = "<New1>"..name.."   <New9>"..attrrValue
        table.insert(result,str)
    end
    return result
end

function This:getCustomDescs()
    local result =""
    local starLv = self:getStarLv()
    for index,star in ipairs(self.m_starData) do
        local desStr = star:getCustomDesc()
        if index<= starLv then
            desStr = star:getCustomDesc1()
        end
        result = result.. desStr.."\n"
    end
    return result
end
