-- @Author: zheng
-- @Date:   2019-05-09 10:54:27
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-27 11:02:45
-- @File Name: GodBeastManager.lua

local This , super = class(..., "Base.BaseModel")
local MythicalAnimalsAttributeConfig = require("Metadatas.MythicalAnimalsAttributeConfig")
local MythicalAnimalsStarConfig = require("Metadatas.MythicalAnimalsStarConfig")
local MythicalAnimalsExpConfig = require("Metadatas.MythicalAnimalsExpConfig")
local MythicalAnimalsPositionConfig = require("Metadatas.MythicalAnimalsPositionConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local GameBeast = require("GodBeast.GameBeast")
local GameBeastStar = require("GodBeast.GameBeastStar")
local HeroManager = require("Hero.HeroManager")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local PopDialogManager = require("UI.PopDialogManager")
local DepotManager = require("Depot.DepotManager")

GodBeastManagerTabType={
  kLv =1,
  kStar =2,
  kCount =2,
}

function This:init()
    super.init(self)
    local chapters = {}
    local maxPos = 0
    for id,config in pairs(MythicalAnimalsAttributeConfig) do
    	chapters[id] = GameBeast:create(id)
    	maxPos = maxPos +1
    end
    for id,config in pairs(MythicalAnimalsStarConfig) do
    	local beastType = config.type
    	local chapter = chapters[beastType]
    	local star = GameBeastStar:create(id)
    	if chapter then
    		chapter:addStar(star)
    	end
    end
	self.m_chapters = chapters
	self.m_haveChapters = {}
	self.m_allHaveChaptersId ={}

	-- self.m_haveChapters[1]= 100003
	-- self.m_haveChapters[3]= 100001
	-- self.m_allHaveChaptersId ={100001,100002,100003}

	self.m_maxPos = maxPos
    self:initListener()
end

function This:initListener()
    EventDispatcher:addListener(self, 14300, self.onGetInforReply)--神兽信息总览
	EventDispatcher:addListener(self, 14301, self.onGetUpLvReply)--升级
	EventDispatcher:addListener(self, 14302, self.onGetUpLvReply)--升星
	EventDispatcher:addListener(self, 14303, self.onGetModifyBeastReply)--上阵、卸下
	EventDispatcher:addListener(self, 14304, self.onGetUpdateBeastReply)	--单个神兽信息刷新
	EventDispatcher:addListener(self, 14305, self.onGetMadeBeastReply)	--合成神兽
	EventDispatcher:addListener(self, 14306, self.onGetNewBeastReply)	--获取新神兽
end

function This:removeListener()
    EventDispatcher:removeListener(self,14300)--神兽信息总览
	EventDispatcher:removeListener(self,14301)--升级
	EventDispatcher:removeListener(self,14302)--升星
	EventDispatcher:removeListener(self,14303)--上阵、卸下
	EventDispatcher:removeListener(self,14304)--单个神兽信息刷新
	EventDispatcher:removeListener(self,14305)--合成神兽
	EventDispatcher:removeListener(self,14306)--获取新神兽
end

function This:createAttrEffect(node,count,offsetY,countLimit)
    if not node or not count then
        return
    end
    if not offsetY then
        offsetY = 31
    end
    local function createAnim( parent,anmStr )
        -- local GameUtil = require("Util.GameUtil")
        local effect1 = GameUtil:createEffect(anmStr)
        parent:addChild(effect1)
        return effect1
    end
    local maxCount = count
    if maxCount and countLimit and countLimit>maxCount then
        maxCount = countLimit
    end
    node:removeAllChildren()
    for index=1,maxCount do
        local effects = createAnim( node,"ShuXing" )
        local y= (index-1)*offsetY
        effects:setPositionY(-y)
    end
end

function This:createEffectCharacter(parentNode,beastId)
    local function createAnim( parent,anmStr )
        -- local GameUtil = require("Util.GameUtil")
        local skeletonNode = cc.SkeletonNode:createCharacter(anmStr)
        skeletonNode:scheduleUpdate()
        skeletonNode:playAnimation("animation2", true)
        skeletonNode:setScale(0.75)
        -- skeletonNode:setPositionY(0)
        -- skeletonNode:setPositionX(250)
        -- local effect1 = GameUtil:createEffectCharacter(anmStr,true)
        parent:addChild(skeletonNode)
        return skeletonNode
    end
    local beastName = ""
    if beastId then
        beastName= string.format("beastId%d",beastId)
        local beastNode = parentNode:getChildByName(beastName)
        if beastNode then
            return
        end
    end
    local node = parentNode
    node:removeAllChildren()
    local chapter= self:getBeast(beastId)
    if chapter then
    	local anim = chapter:getAnim()
    	local beastNode = createAnim(node,anim)
        beastNode:setName(beastName)
    end
end

function This:getCurentAtrr()
	local res ={}
	for pos,beastId in pairs(self.m_haveChapters) do
		local beast = self:getBeast(beastId)
		if beast then
			local attrs = beast:getConverAttr()
			res = GameUtil:tableAdd(attrs,res)
		end
	end
	return res
end

function This:removeListBeastId(cardList)
    local result = {}
    if cardList then
        for i,heroInfo in ipairs(cardList) do
            local heroId = heroInfo
            if heroInfo.card_type then
                heroId = heroInfo.card_type
            end
            if not self:getBeast(heroId) then
                table.insert(result,heroInfo)
            end
        end
    end
    return result
end

function This:getBeast(id)
	return self.m_chapters[id]
end

function This:getBeasts()
	return self.m_chapters
end

function This:getAllHaveBeastsIdWithSort()
    local idList = table.clone(self.m_allHaveChaptersId or{})
    table.sort(idList,function (a,b)
		local as,bs = self:getBeast(a),self:getBeast(b)
		if as:getQuality() ~= bs:getQuality() then
			return as:getQuality() > bs:getQuality()
		elseif as:getStarLv() ~= bs:getStarLv() then
			return as:getStarLv() > bs:getStarLv()
		elseif as:getLv() ~= bs:getLv() then
			return as:getLv() > bs:getLv()
		else
			return as:getId() > bs:getId()
		end
    end)
    return idList
end

function This:getAllHaveBeastsId() 
	return self.m_allHaveChaptersId
end

function This:isGotHero(id)
    return self:isHaveBeast(id)
end

function This:isHaveBeast(id)
    if id then
    	for _,beastId in pairs(self.m_allHaveChaptersId) do
    		if id == beastId then
    			return true
    		end
    	end
    end
	return false
end

function This:isHavePos(id)
	local temId = id or 0
	for pos,beastId in pairs(self.m_haveChapters) do
		if beastId == temId then
			return true, pos
		end
	end
	return false,0
end

function This:getUpBestList()
	return self.m_haveChapters
end

--GodBeastManagerTabType.kStar
function This:isHaveRed(redType)
    local function checkRed(checkFun)
        local maxPos = self:getMaxPos()
        for pos=1,maxPos do
            local posData = self:getBeastPos(pos)
            if posData and checkFun and checkFun(posData) then
                return true
            end
        end
        return false
    end
    local temRedType = redType or GodBeastManagerTabType.kLv
    local switch ={
        [GodBeastManagerTabType.kLv]=function (posData)-- for case 1
                    return posData:isCanUpLv()
                end,
        [GodBeastManagerTabType.kStar]=function (posData)
                    return posData:isCanUpStar()
                end,
    }
    local switchFun = switch[temRedType]
    if temRedType and switchFun then
        -- local posData = self:getPos(posIndex)
        -- if posData then
        --     return switchFun(posData)
        -- else
           return checkRed(switchFun) 
        -- end
    end
    return false
end

function This:getBeastPos(pos)
	local chapterId = self.m_haveChapters[pos]
	if chapterId and chapterId~=0 then
		return self.m_chapters[chapterId]
	end
	return nil
end

function This:setBeastInfo(info)
	local chapter = self:getBeast(info.type_id)
	if chapter then
		chapter:setStarLv(info.star)
		chapter:setLv(info.level)
		chapter:setExp(info.exp)
	end
end

function This:getLvUpNeed()--神兽升级消耗
	return ArgConfig.mythical_animals_level_up_item.arg
end

--获取自己可以转化的经验
function This:getMyAllExp()
    local res = 0
    local needs =self:getLvUpNeed()
    for _,item1 in ipairs(needs) do
        local itemCount = DepotManager:getItemCount(item1[1])
        res = res+ itemCount*item1[2]
    end
    return res
end

function This:getMaxLv(lv,isAdd)---等级和经验
	local playerLv = lv or  GamePlayer:getLevel()
	if isAdd then
		playerLv = playerLv+1
	end
	local configItem = MythicalAnimalsExpConfig[playerLv]
	if configItem then
		return configItem.need_level,configItem.exp
	end
	return false,0
end

function This:getPosLevelLimit(pos)--所需爵位等级
	return MythicalAnimalsPositionConfig[pos or 1].need_level
end

function This:getMaxPos()
	return self.m_maxPos
end
	
function This:onGetMadeBeastRequest(id)
	Socket:sendAndLock(14305,{type_id = id})--合成神兽
end

function This:onGetMadeBeastReply( msg )
    if msg.err_code == 0 then
        if not self:isHaveBeast(msg.type_id) then
            table.insert(self.m_allHaveChaptersId,msg.type_id)
            self:notifyObservers("onGetMadeBeast")
        end

        -- PopDialogManager:addPopDialog({
        --     name = "GodBeast.MythicalAnimalsGetPanel",
        --     arg = {id = msg.type_id, isGot = false},
        --     level = DialogLevel.kTopLevel,
        --     onlyone = true,
        -- })
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetUpdateBeastRequest(type_id)
	Socket:sendAndLock(14304,{type_id =type_id})--单个神兽信息刷新
end

function This:onGetUpdateBeastReply( msg )
    if msg.err_code == 0 then
        local info = msg.monster or {}
        local beastId = info.type_id
        if beastId then
            self:setBeastInfo(info)
            self:notifyObservers("onGetUpdateBeast",beastId)
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetModifyBeastRequest(type_id,pos)
	Socket:sendAndLock(14303,{type_id =type_id,pos= pos})
end

function This:onGetModifyBeastReply( msg )
    if msg.err_code == 0 then
    	local pos = msg.pos
    	local typeId = msg.type_id
    	if pos and typeId then
        	self.m_haveChapters[pos] = typeId
    	end
    	HeroManager:updateMingWang()
        EventDispatcher:dispatcher("god_beast_change")
        self:notifyObservers("onGetModifyBeast",typeId)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetUpStarRequest(type_id)
	Socket:sendAndLock(14302,{type_id =type_id})--升星
end

function This:onGetUpLvRequest(type_id)
	Socket:sendAndLock(14301,{type_id =type_id})--升级
end

function This:onGetUpLvReply( msg )
    if msg.err_code == 0 then
        HeroManager:updateMingWang()
        self:notifyObservers("onGetUpLv",msg.type_id)
        EventDispatcher:dispatcher("god_beast_change")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetInforRequest()
	Socket:sendAndLock(14300)--神兽信息总览
end

function This:onGetInforReply( msg )
    if msg.err_code == 0 then
		self.m_allHaveChaptersId={}
    	local monster_list = msg.monster_list
        self.m_haveChapters = msg.pos_list or {}
        for _,info in ipairs(monster_list) do
        	self:setBeastInfo(info)
        	table.insert(self.m_allHaveChaptersId,info.type_id)
        end
        HeroManager:updateMingWang()
        self:notifyObservers("onGetInfor")
        EventDispatcher:dispatcher("god_beast_change")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetNewBeastReply(msg)
    if msg.err_code == 0 then
        table.insert(self.m_allHaveChaptersId,msg.monster.type_id)
        self:notifyObservers("onGetMadeBeast")

        PopDialogManager:addPopDialog({
            name = "GodBeast.MythicalAnimalsGetPanel",
            arg = {id = msg.monster.type_id, isGot = false},
            level = DialogLevel.kTopLevel,
            onlyone = true,
        })
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end