

local This, super = class(..., "Dialog")
local MythicalAnimalsAttributeConfig = require("Metadatas.MythicalAnimalsAttributeConfig")
local GodBeastManager = require("GodBeast.GodBeastManager")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")


function This:initData(info)
	self.m_id = info.id
	self.m_list = info.list or self:getListData()
	self:initIndex()
end

function This:getListData()
	local list = {}
	local have = {}

	for _, id in ipairs(GodBeastManager.m_allHaveChaptersId) do
		have[id] = true
	end

	for _, v in pairs(MythicalAnimalsAttributeConfig) do
		local t = {}
		t.id = v.id
		t.config = v
		t.isHave = (have[v.id] and true) or false
		table.insert(list, t)
	end

	table.sort(list, function (a, b)
		return a.id < b.id
	end)

	return list
end

function This:initIndex()
	self.m_index = 1

	for i, v in ipairs(self.m_list) do
		if v.id == self.m_id then
			self.m_index = i
			break
		end
	end	
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MythicalAnimalsBookDetails.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode
end

function This:initTouchEvent()
    if not self.m_inited then
    	self.m_inited = true

        local csNode = self.m_csNode

        csNode.close:setTouchEnded(function()
            self:close()
        end)

        csNode.jiantou1:setTouchEnded(function()
            if self.m_index <= 1 then
            	return
            end

            self.m_index = self.m_index - 1
            self.m_id = self.m_list[self.m_index].id
            self:refresh()
        end)

		csNode.jiantou2:setTouchEnded(function()
            if self.m_index >= #self.m_list then
            	return
            end

            self.m_index = self.m_index + 1
            self.m_id = self.m_list[self.m_index].id
            self:refresh()
        end)

        csNode.ListView.jineng.info:setTouch({
            isScale = false, 
            isDynamic = true, 
            isContainer  = true,
            onEnded = function()
            	g_scene:openDialog("GodBeast.MythicalAnimalsSkillPanel", {beastId = self.m_id})
            end
        })
    end
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:onExit()
    super.onExit(self)
end

function This:refresh()
	local csNode = self.m_csNode
	local data = GodBeastManager:getBeast(self.m_id)

	csNode.jiantou1:setVisible(self.m_index > 1)
	csNode.jiantou2:setVisible(self.m_index < #self.m_list)

	local name = data:getName()
	csNode.name:setString(name)
	csNode.name:setColorByID("Q".. ItemUtil.getQuality(data:getId()))
	GodBeastManager:createEffectCharacter(csNode.animal,data:getId())

	local listView = csNode.ListView 
	local shuxing = listView.shuxing
	local attrStr = self:getAttrStr(data)

	for index = 1, 6 do
    	local txt = shuxing["text"..index]
    	if txt then
            local str = attrStr[index]
            txt:setStringWithColor(str)
            txt:setVisible(str)
    	end
    end
    shuxing.level:setString(string.format("Lv.%d", 1))

    local star = data:getStar()
    local skillName,skillDes = star:getSkillDes()
    local jineng = listView.jineng
    jineng.skill_icon:setImage(star:getSkillIcon())
    jineng.skillname:setString(skillName)
    jineng.skill:setStringWithColor(skillDes,220)

    listView.texing.text:setStringWithColor(data:getCustomDescs(),310)
    listView.jianjie.text:setString(data:getDesc(),500)
end

function This:getAttrStr(data)
	local attr = data:getAttr(1, 0)
	local ret = {}
	local ignore = {
		[AttrType.kAngerValue] = true,
	}

	for key, value in pairs(attr) do
		if not ignore[key] then
        	local name, attrValue = GameUtil:getAttrStr(key, value,true)
        	local str = string.format("<New1>%s   <New9>%s", name, attrValue)
        	table.insert(ret, str)
        end
    end

	return ret
end