-- @Author: zheng
-- @Date:   2019-05-10 11:44:58
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-23 15:40:05
-- @File Name: MythicalAnimalsMeInfoPane.lua

local MythicalAnimalsMeInfoLvTab = require("GodBeast.MythicalAnimalsMeInfoLvTab")
local MythicalAnimalsMeInfoStarTab = require("GodBeast.MythicalAnimalsMeInfoStarTab")
local GodBeastManager = require("GodBeast.GodBeastManager")
local TerrTabGroupBehaviour = require("Territory.TerrTabGroupBehaviour")

local This , super = class(...,"GodBeast.MythicalAnimalsItemInfoPanel")
This.fileName = "MenuScene/NodeUI/MythicalAnimalsMeInfo.csb"
function This:initNode(info)
    self.m_tabNodes ={}
    self.m_selectTabName = info.tabType or GodBeastManagerTabType.kLv
    local beastId = info.beastId or 0
    local allBeastId = GodBeastManager:getAllHaveBeastsId()
    local selectPos = 1
    for pos,id in ipairs(allBeastId) do
        if id ==beastId then
            selectPos = pos
            break
        end
    end
    info.selectPos = selectPos
    -- super.initNode(self,info)
    if info then
       self.m_selectPos = info.selectPos
    else
      self.m_selectPos = 1
    end
    -- self.m_selectPos = 1
    self.m_csNode = cc.Node:createFromCSFile(self.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self:refresh(self.m_selectPos)

    local function refreshGroup(index)
        if self.m_inited then
            self.m_selectTabName = index
            self:refresh()
        end
    end
    local group = TerrTabGroupBehaviour:create(param)
    for index=1,GodBeastManagerTabType.kCount do
        local node = self.m_csNode["tep" .. index]
        group:addTabButton(index, node, function ()
            refreshGroup(index)
        end, false)
    end
    self.m_tabGroup = group
    group:selectTabButton(self.m_selectTabName)
    self.m_inited = true
end

function This:getSelectTabName()
    return self.m_selectTabName
end

function This:createTabNode(data,refreshAttrEffect)
    local selectTabName = self:getSelectTabName()
    local tabNode = self.m_tabNodes[selectTabName]
    for _,node in pairs(self.m_tabNodes) do
        node:setVisible(false)
    end
    if not tabNode then
        local info = {beastData= data}
        if GodBeastManagerTabType.kLv == selectTabName then
            tabNode = MythicalAnimalsMeInfoLvTab:create(info)
        else
            tabNode = MythicalAnimalsMeInfoStarTab:create(info)
        end
        self:addChild(tabNode)
        self.m_tabNodes[selectTabName]= tabNode
    else
        tabNode:setVisible(true)
        tabNode:refresh(data,refreshAttrEffect)
    end

end

function This:refresh(pos,refreshAttrEffect)
    local temPos = pos or self.m_selectPos
    local allBeastId = GodBeastManager:getAllHaveBeastsId()
    local data = GodBeastManager:getBeast(allBeastId[temPos])
    if data then
        self:refreshCommon(data)
        self:createTabNode(data,refreshAttrEffect)
        local selectTabName = self:getSelectTabName()
        local csNode = self.m_csNode
        local lv_node= csNode.lv_node
        local star_node= csNode.star_node

        local lvVisible = false
        if GodBeastManagerTabType.kLv == selectTabName then
            local curLevel = data:getLv()
            local nextLevel = GodBeastManager:getMaxLv(curLevel, true)
            lvVisible = nextLevel
            if lvVisible then
                local expRadio = data:getEXP()
                local _, curNeedExp = GodBeastManager:getMaxLv(curLevel)
                local max = expRadio * curNeedExp
                local curent = data:getExp()
                local jindutiao = lv_node.jindutiao
                jindutiao.bar:setRatio(curent / max)
                jindutiao.num:setString(string.format("%d/%d", curent, max))
                jindutiao.lv:setString(string.format("Lv.%d", curLevel))
            end
    --     else
    --         local starLv = data:getStarLv()
    --         local fullStar = starLv==5
    --         if not fullStar then
    --             local nextStar = starLv+1
				-- local str = data:getStar(nextStar):getDesc()
    --             star_node.xiaoguo:setString(str,329)
    --         end
    --         star_node.xiaoguo:setVisible(not fullStar)
    --         star_node.all:setVisible(fullStar)
        end
        local isHavePos = GodBeastManager:isHavePos(data:getId())
        csNode.zhenNode:setVisible(isHavePos)

        lv_node:setVisible(lvVisible)
        star_node:setVisible(not lvVisible)
    end
end

function This:initTouchEvent()
 --    item1:setTouchEnded(function()
	-- 	self:btnCallBack(btnType)
	-- end,item1.bg)
    
 --    self.zhengduo:setTouch({
 --            isScale = false,
 --            isDynamic = true,
 --            isContainer = true,
 --            onEnded = function()
 --                 self:close()
 --            end
 --        })
	super.initTouchEvent(self)
end

function This:getRight()
    local result =false
    local allBeastIds = GodBeastManager:getAllHaveBeastsId()
    local temPos = self.m_selectPos+1
    if temPos>#allBeastIds then
        temPos = nil
    end
    return result or temPos
end

function This:getLeft()
    local result =false
    local temPos = self.m_selectPos-1
    if temPos<1 then
        temPos = nil
    end
    return result or temPos
end

function This:checkRefresh(beastId,refreshAttrEffect)
    local temPos = self.m_selectPos
    local allBeastId = GodBeastManager:getAllHaveBeastsId()
    local data = GodBeastManager:getBeast(allBeastId[temPos])
    if beastId and data and data:getId() then
        self:refresh(nil,refreshAttrEffect)
    end
end

function This:onGetUpLv(beastId)
    self:checkRefresh(beastId,true)
end     

-- function This:onGetUpdateBeastReply(beastId)
--     self:checkRefresh(beastId)
-- end

-- function This:onGetModifyBeastReply(beastId)
--     self:checkRefresh(beastId)
-- end
