-- @Author: Su
-- @Date:   2019-05-10 18:00:21
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-09 11:49:10
-- @File Name: MythicalAnimalsMeItem.lua
local This , super = class(...,"NodeUI")
local GodBeastManager = require("GodBeast.GodBeastManager")
local ItemNode = require("UI.ItemNode")
local ItemUtil = require("Bag.ItemUtil")

function This:initNode(info)
    if info.animal then
        self.m_animal = info.animal
        self.m_id = self.m_animal:getId()
    end
    if info.item then
        self.m_item = info.item
        self.m_id = self.m_item:getId()
        self.m_isHave = info.isHave
    end
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MythicalAnimalsMeItem.csb")
    local w,h = self.m_csNode.bg:getContentSize()
    self.m_csNode:setContentSize(w,h)
    self:addChild(self.m_csNode)

    self:refresh()
end

function This:onEnter()
    super.onEnter(self)
    GodBeastManager:addObserver(self)
end

function This:onExit()
    GodBeastManager:removeObserver(self)
    super.onExit(self)

end

function This:refresh()
    local isHava = self.m_isHave
    local csNode = self.m_csNode
    if not self.m_animalItem then
        local itemNode= ItemNode:create(self.m_id)
        itemNode:setAnchorPoint(0.5,0.5)

        csNode.item:addChild(itemNode)
        self.m_animalItem = itemNode
    end
    local hechengVisible = false
    local button = csNode.button
    local animalPartVisible = false
    local itemId = 0
    if self.m_animal then
        csNode.itemName:setString(self.m_animal:getName())
        csNode.lv:setString(string.format(Serialize[60312],self.m_animal:getLv()))
        csNode.biaoqian:setVisible(GodBeastManager:isHavePos(self.m_animal:getId()))
        local starLv = self.m_animal:getStarLv()
        for i = 1,5 do
            csNode["star"..i].light:setVisible(starLv >= i)
        end
        itemId= self.m_animal:getId()
    else
        itemId= self.m_item:getId()
        local animal = GodBeastManager:getBeast(self.m_item:getId()-1000)
        self.m_limitNum = animal:getMadeNeedNum()

        animalPartVisible = true
        if self.m_item:getCount() >= self.m_limitNum and not isHava then
            hechengVisible = true
        end

        csNode.itemName:setString(string.format( "%s(%d/%d)",self.m_item:getName(),self.m_item:getCount(),self.m_limitNum))
        csNode.biaoqian:setVisible(false)
        csNode.text1:setVisible(false)
        csNode.lv:setVisible(false)
        csNode.text2:setVisible(false)
        for i = 1,5 do
            csNode["star"..i]:setVisible(false)
        end
    end
    csNode.itemName:setColorByID("Q".. ItemUtil.getQuality(itemId))
    button.qianghua:setVisible(not animalPartVisible)
    button.hecheng:setVisible(animalPartVisible and hechengVisible)
    button.huoqu:setVisible(animalPartVisible and not hechengVisible)
end

function This:getSize()
    local w,h = self.m_csNode.bg:getContentSize()
    return w,h
end

function This:initTouchEvent()
	local csNode = self.m_csNode
    csNode.button:setTouch({
        isScale = false, 
        isDynamic = true, 
        isContainer  = true,    
        onEnded = function()
        if self.m_animal then
            g_scene:openDialog("GodBeast.MythicalAnimalsMeInfoPane",{tabType = GodBeastManagerTabType.kLv,beastId =self.m_animal:getId()})
        
        else
            if self.m_item:getCount() < self.m_limitNum or self.m_isHave then
                g_scene:openDialog(unpack({"Store.StoreMainPanel", {tabs = StoreType.kAnimalShop}}))
                return
            end
            GodBeastManager:onGetMadeBeastRequest(self.m_item:getId()-1000)
        end
        
    end})
end

function This:onGetUpLv(id)
    if id == self.m_id then
        self:refresh()
    end

end