--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local GuideEntryConfig = require("Metadatas.GuideEntryConfig")
local GuideUnit = require("Guide.GuideUnit")
local GuideManager = require("Guide.GuideManager")
local GuideCity = require("Guide.GuideCity")
local GuideUnitConfig = require("Metadatas.GuideUnitConfig")
local EventDispather = require("EventDispatcher")
local GuideManager = require("Guide.GuideManager")
local GuideUtil = require("Guide.GuideUtil")
local GuideChapterOpen = require("Guide.GuideChapterOpen")
local GamePlayer = require("Player.GamePlayer")

UnitType = 
{
	kUnitFinger = 1,
	kSession = 2,
	kGuideCity = 3,
	kGuideChapter = 4,
}

function This:create(...)
	local guide = {}
	setmetatable(guide,self)
	guide:init(...)
	return guide
end

function This:init(id)
	self.m_id = id
	self.m_config = GuideEntryConfig[id]
	self.m_isWaitRequest = false
	self.m_curIndex = 0
	self.m_maxIndex = #self.m_config.unit_list
	assert(#self.m_config.unit_list >= 1)
end

function This:getGuideUnit(unitId,index)
	local info = {
		unitId = unitId,
		father = self,
		index = index,
	}
	if not GuideUnitConfig[unitId] then
		assert(false)
	end
	local unitType = GuideUnitConfig[unitId].type
	if unitType == UnitType.kSession or unitType == UnitType.kUnitFinger then
		return GuideUnit:create(info)
	elseif unitType == UnitType.kGuideCity then
		return GuideCity:create(info)
	elseif unitType == UnitType.kGuideChapter then
		return GuideChapterOpen:create(info)
	end
end

function This:getId()
	return self.m_id
end

function This:getType()
	return self.m_config.type
end

function This:getDesp()
	return self.m_config.desp
end

function This:getGuideDespList()
	return self.m_config.guide_desp
end

function This:setIndex(index)
	self.m_isWaitRequest = false
	if self.m_curIndex == 0 and self:isCanSkip() then
		self:cancel()
	elseif self.m_curIndex ~= index then
		self.m_curIndex = index
		if self.m_curIndex <= 1 then
			self:initGuide()
		end
		self.m_oldUnit = self.m_curUnit
		if self.m_oldUnit then
			self.m_oldUnit:removeView()
		end
		self.m_curUnit = self:getGuideUnit(self.m_config.unit_list[self.m_curIndex],self.m_curIndex)
		if self.m_curUnit:check() then
			self.m_curUnit:guide()
		end
	end
end

function This:getMaxIndex()
	return self.m_maxIndex
end

function This:getIndex()
	return self.m_curIndex
end

function This:isFinish()
	return self.m_curIndex > self.m_maxIndex
end

function This:getNextGuide()
	return self.m_config.next_guide
end

function This:isRunning()
	if self.m_curIndex <= self.m_maxIndex then
		if not self.m_isWaitRequest then
			return self.m_curUnit:isRunning()
		end
	end
end

function This:hasCancelEvent()
	if type(self.m_config.cancel_event) == "table" then
		return true
	end
	return false	
end

function This:isCanSkip()
	if not GamePlayer:isEnterGame() then
		return
	end
	print(string.format("Entry %d 检查跳过",self.m_id))
	if type(self.m_config.skip.func) == "function" then
		return self.m_config.skip.func()
	end	
	return false
end

function This:initGuide()
	for i , event in ipairs(self.m_config.cancel_event) do
		EventDispather:addListener(self,event,function()
			print("Entry %d 收到取消事件：",self.m_id, event)
			if not self.m_isWaitRequest and self:isCanSkip() then
				self:cancel()
			end
		end)
	end
end

function This:next()
	self.m_isWaitRequest = false
	self:setIndex(self.m_curIndex + 1)
end

function This:finish()
	if self.m_isWaitRequest then
		return
	end
	self.m_isWaitRequest = true
	if self.m_curIndex >= self.m_maxIndex then
		for i , event in ipairs(self.m_config.cancel_event) do
			EventDispather:removeListener(self,event)
		end
	end
	GuideManager:onUpdateGuideProgressRequest(self.m_id,self.m_curIndex)
end

function This:cancel()
	print("-----------entry cancel----------",self.m_id)
	if self.m_curUnit then
		print("-----------entry unit cancel----------",self.m_curIndex)
		self.m_curUnit:cancel()
		self.m_curUnit = nil
	end
	self.m_curIndex = self.m_maxIndex
	self:finish()
end

function This:clear()
	print("-----------entry clear----------",self.m_id)
	if self.m_curUnit then
		self.m_curUnit:removeView()
		self.m_curUnit = nil
	end
end

--endregion
