--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local GuideUnitConfig = require("Metadatas.GuideUnitConfig")
local EventDispathcer = require("EventDispatcher")
local GuideSessionView = require("Guide.GuideSessionView")
local GuideFingerView = require("Guide.GuideFingerView")
local GuideUtil = require("Guide.GuideUtil")
local Application = require("cc.Application")
local GamePlayer = require("Player.GamePlayer")

function This:create(...)
	local guide = {}
	setmetatable(guide,self)
	guide:init(...)
	return guide
end

function This:init(info)
	self.m_id = info.unitId
	self.m_father = info.father
	self.m_index = info.index
	self.m_isRunning = false
	self.m_config = GuideUnitConfig[self.m_id]
	self.m_checkEventList = {"dialog_open","dialog_close","exit_city","city_menu_open","menu_scene_enter","world_scene_enter"}
	if self.m_config.arg.check_event then
		for i , event in ipairs(self.m_config.arg.check_event) do
			table.insert(self.m_checkEventList,event)
		end
	end
	self:initGuide()
end

function This:getId()
	return self.m_id
end

function This:getIndex()
	return self.m_index
end

function This:getSessionTips()
	return self.m_config.arg.tips
end

function This:getFinishDelayTime()
	return self.m_config.arg.finishDelay or 0
end

function This:initGuide()
	local keyList = {}
    for match in string.gmatch(self.m_config.arg.node, "[^%.]+") do
        keyList[#keyList + 1] = match
    end
	self.m_keyList = keyList
	for i , event in ipairs(self.m_checkEventList) do
		EventDispathcer:addListener(self,event,function()
			print(string.format("收到检测事件%s",event))
			if not self:isRunning() and self:check() then
				self:guide()
			elseif self:isRunning() and self:isCanSkip() then
				self:finish()
			end
		end)
	end
end

function This:getNode(node,keyList,fromIndex)
	if type(self.m_config.arg.node_func) == "function" then
		self.m_config.arg.node_func()
	end

	assert(#keyList >= fromIndex)
	local curKey = keyList[fromIndex]
	local curNode = node
	local isVisible = node:isVisible()
	while fromIndex <= #keyList do
		local name = keyList[fromIndex]
		if name == "container" then
			print("---------------------------find the container-------------------------------")
		end
 		curNode = curNode:getChildByName(name)
		if not curNode then
			return nil
		end
		if not curNode:isVisible() then
			isVisible = false
		end
		fromIndex = fromIndex + 1
	end
	return curNode,isVisible
end

function This:check()
	local checkHandler = function()
		if not GamePlayer:isEnterGame() then
			return false
		end
		if g_scene:getName() ~= self.m_keyList[1] then
			return false
		end
	
		if #self.m_keyList > 1 then
			local node , isVisible = self:getNode(g_scene,self.m_keyList,2)
			self.m_node = node
			if not node or not isVisible then
				return false
			end
			if self.m_config.type == UnitType.kUnitFinger and not node.m_touch then
				return false
			end
		end

		local funcList = self.m_config.arg.check_func
		if self.m_config.arg.check_func then
			return self.m_config.arg.check_func()
		end
		return true
	end

	if self:isCanSkip() then
		self:finish()
	elseif not self.m_isRunning then
		local result = checkHandler()
		return result
	end
	return false
end

function This:isRunning()
	return self.m_isRunning
end

function This:isCanSkip()
	if self.m_config.arg.cancel_func then
		return self.m_config.arg.cancel_func()
	end
	return false
end

function This:guide()
	self.m_isRunning = true
	local guideHandler = function()
		local GuideManager = require("Guide.GuideManager")
		GuideManager:cancelAncGuider()

		if self.m_config.arg.guide_func then
			self.m_config.arg.guide_func()
		end

		if self.m_config.arg.cancel_event then
			for i , name in ipairs(self.m_config.arg.cancel_event) do
				EventDispathcer:addListener(self,name,function()
					self:finish()
				end)
			end
		end

		if self.m_config.type == UnitType.kSession then
			print("---------------------create view GuideSessionView",self:getId())
			self.m_view = GuideSessionView:create({
				text = self.m_config.arg.text,
				model = self,
				isForce = self.m_config.is_force == 1,
				dir = self.m_config.arg.dir,
				icon = self.m_config.arg.icon,
				name = self.m_config.arg.name,
				desp = self.m_config.arg.desp,
			})
			if g_scene:getName() == "MenuScene" or g_scene:getName() == "WorldScene" then
				g_scene:setUIVisible(false)
			end
		elseif self.m_config.type == UnitType.kUnitFinger then
			print("---------------------create view GuideFingerView",self:getId())
			self.m_view = GuideFingerView:create({
				node = self.m_node,
				model = self,
				isForce = self.m_config.is_force == 1,
				dir = self.m_config.arg.dir,
				offset = self.m_config.arg.offset,
				fingerEvent = self.m_config.arg.finger_event,
			})
		end
		local zorder = g_scene:getMaxLocalZOrder()
		g_scene:addChild(self.m_view,zorder + 1)

		if self.m_config.sound and self.m_config.sound ~= "" then
			GuideManager:playSound(self.m_config.sound)
		end
		EventDispathcer:dispatcher("guide_start",self)
	end

	if self.m_config.arg.delay then
		self.m_timerKey = "guide_unit" .. self.m_id
		local timer = Application:getTimer()
		timer:setTimer(self.m_config.arg.delay, function()
			guideHandler()
		end,self.m_timerKey)
	else
		guideHandler()
	end
end

function This:finish()
	function finishHandler()
		self.m_isRunning = false
		for i , event in ipairs(self.m_checkEventList) do
			EventDispathcer:removeListener(self,event)
		end
		if self.m_config.arg.cancel_event then
			for i , name in ipairs(self.m_config.arg.cancel_event) do
				EventDispathcer:removeListener(self,name)
			end
		end
		local timer = Application:getTimer()
		timer:removeTimer(self.m_timerKey)

		if self.m_config.arg.finish_func then
			self.m_config.arg.finish_func()
		end
		self.m_father:finish()
	end
	finishHandler()
end

function This:cancel()
	self.m_isRunning = false
	for i , event in ipairs(self.m_checkEventList) do
		EventDispathcer:removeListener(self,event)
	end
	if self.m_config.arg.cancel_event then
		for i , name in ipairs(self.m_config.arg.cancel_event) do
			EventDispathcer:removeListener(self,name)
		end
	end
	self:removeView()
end

function This:removeView()
	print("-----------------removeView------------",self:getId(),tostring(self.m_view))
	if self.m_view and self.m_view[0] ~= 0 then
		self.m_view:disappear()
		self.m_view = nil
		if g_scene:getName() == "MenuScene" or g_scene:getName() == "WorldScene" then
			g_scene:setUIVisible(true)
		end
	end
end

--endregion
