--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local WorldMapManager = require("World.WorldMapManager")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local TerrBuildManager = require("Territory.TerrBuildManager")
local AchieveManager = require("Achieve.AchieveManager")

function This:getNodeByPath(path,isIgnoreScene)
	local keyList = {}
    for match in string.gmatch(path, "[^%.]+") do
        keyList[#keyList + 1] = match
    end

	if #keyList <= 0 then
		return
	end

	if not isIgnoreScene and g_scene:getName() ~= keyList[1] then
		return
	end

	local fromIndex = 2
	local curNode = g_scene
	local isVisible = curNode:isVisible()
	while fromIndex <= #keyList do
		local name = keyList[fromIndex]
		curNode = curNode:getChildByName(name)
		if not curNode then
			return nil
		end
		if not curNode:isVisible() then
			isVisible = false
		end
		fromIndex = fromIndex + 1
	end
	return curNode,isVisible
end

function This:hasDialog(arg)
	local dialogList = g_scene:getAllDialogs()
	local dialogs = {}
	for i , dialog in ipairs(dialogList) do
		dialogs[dialog._NAME] = dialog
	end
	if #arg <= 0 then
		if #dialogList > 0 then
			return false
		else
			if g_scene:getName() == "WorldScene" and g_scene:getShowMode() == ShowMode.kBuild then
				 return false
			end
		end
	end
	local count = 0
	for i , name in ipairs(arg) do
		if dialogs[name] then
			count = count + 1
		end
	end
	return count >= #arg 
end

function This:isTopDialog(name)
	local dialogList = g_scene:getAllDialogs()
	if #dialogList > 0 then
		local dialog = dialogList[#dialogList]
		return dialog._NAME == name
	end 
	return false
end

function This:closeDialog(name)
	local panel = g_scene:getDialog(name)
	if panel then
		panel:close()
	end
end

function This:isMyCity(param)
	if GamePlayer:isInWorld() then
		local cityId = param.city or param[1]
		local city = WorldMapManager:getCity(cityId)
		return city:isMy()
	end
	return false
end

function This:isCityInFighting(arg)
	local cityId = arg.city
	assert(cityId)
	local city = WorldMapManager:getCity(cityId)
	return city:isFighting()
end

function This:isRecruitViewDisappear()
	local view = g_scene:getDialog("RecruitView")
	return not view
end

function This:isShowHeroCardFinish()
	return g_scene:isShowHeroCardFinish()
end

function This:moveHomeBuildToCenter(arg)
	if g_scene:getName() == "MenuScene" then
		g_scene:focuseBuildByType(arg[1],arg[2],nil,false)
	end
end

function This:moveRobberToCenter(arg)
	if g_scene:getName() == "MenuScene" then
		g_scene:focuseRobber(arg[1])
	end
end

function This:isTopDialog(arg)
	local panel = g_scene:getTopDialog()
	return panel and panel:getName() == arg[1]
end

function This:hasHero(arg)
	assert(arg.heroId)
	local hero = HeroManager:getHero(arg.heroId)
	return hero:isGot()
end

function This:isPlayerLevelHighter(arg)
	return GamePlayer:getLevel() > arg
end

function This:isHasHeros(arg)
	for i , heroId in ipairs(arg) do
		local hero = HeroManager:getHero(heroId)
		if not hero:isGot() then
			return false
		end
	end
	return true
end

function This:isHeroNumHighter(arg)
	return HeroManager:getHeroGotCount() > arg
end

function This:isHasHeroWithLvAndNum(arg)
	assert(arg.level and arg.num)
	local total = 0
	local heros = HeroManager:getAllHero()
	for i , hero in pairs(heros) do
		if hero:isGot() and hero:getLevel() >= arg.level then
			total = total + 1
		end
	end
	return total >= arg.num
end

function This:isBuildingLevelHighter(arg)
	assert(arg.type and arg.pos and arg.level)
	return TerrBuildManager:getBuildLevel(arg.type,arg.pos) >= arg.level
end

function This:hasCityBuilding(arg)
	local city = WorldMapManager:getCity(arg)
	if city then
		for id, pos in pairs(BuildPos) do
			if city:getBuild(pos) then
				return true
			end
		end
	end
	return false
end

function This:moveCityToCenter(arg)
	assert(arg.city)
	if g_scene:getName() == "WorldScene" then
		g_scene:focuseCity(arg.city)
	end
end

function This:isHeroLevelHighter(arg)
	assert(arg.heroId and arg.level)
	local hero = HeroManager:getHero(arg.heroId)
	if hero:isGot() and hero:getLevel() > arg.level then
		return true
	end
	return false
end

function This:isChapterFinish(arg)
	assert(arg.id)
	local chapter = AchieveManager:getChapter(arg.id)
	return chapter and chapter:isFinish()
end

function This:isChapterUpdating(arg)
	assert(arg.id)
	local chapter = AchieveManager:getChapter(arg.id)
	return chapter and not chapter:isFinish()
end

function This:isInNewPlayerMap()
	if GamePlayer:isInWorld() and WorldMapManager:getMapType() <= 0 then
		return true
	end
	return false
end

function This:isInWorld()
	return GamePlayer:isInWorld()
end

function This:isWorldSceneInit()
	if g_scene:getName() == "WorldScene" and g_scene:isInit() then
		return true
	end
	return false
end

function This:isOutNewPlayerMap()
	if GamePlayer:isInWorld() and WorldMapManager:getMapType() <= 0 then
		return false
	end
	return true
end

function This:isMainAchieveHasReward(arg)
	local achieve = AchieveManager:getAchieve(AchieveType.kMainLine,arg.achieveId)
	if achieve then
		return achieve:getState() == AchieveState.kHaseReward
	end
end

function This:isMainAchieveCanReward(arg)
	local achieve = AchieveManager:getAchieve(AchieveType.kMainLine,arg.achieveId)
	if achieve then
		local mainAchieves = AchieveManager:getAllMainAchieves()
		for id , _ in pairs(mainAchieves) do
			local ac = AchieveManager:getAchieve(AchieveType.kMainLine,id)
			if ac:getId() < arg.achieveId and ac:getState() ~= AchieveState.kHaseReward then
				return false
			end
		end
		return achieve:getState() == AchieveState.kHaseFinish
	end
end

function This:isMainAchieveUpdating(arg)
	local achieve = AchieveManager:getAchieve(AchieveType.kMainLine,arg.achieveId)
	if achieve then
		return achieve:getState() == AchieveState.kUpdating
	end
end

function This:isMainAchieveFinish(arg)
	local achieve = AchieveManager:getAchieve(AchieveType.kMainLine,arg.achieveId)
	if achieve then
		return achieve:getState() == AchieveState.kHaseFinish
	end
end

function This:isMainAchieveSelectedOnPanel()
	local panel = g_scene:getDialog("MainLineAchievePanel")
	if panel then
		return panel:isMainAchieveSelected()
	end
end

function This:hasArmyToCity(arg)
	assert(arg.city)
	local armyList = WorldMapManager:getMarchArmyList(GamePlayer:getPlayerID())
	for i , army in ipairs(armyList) do
		if army:getToCity() == arg.city then
			return true
		end
	end
	return false
end

function This:isJiceInUseCd(arg)
	if GamePlayer:isInWorld() then
		local jiceItem = WorldMapManager:getStgItem(arg.jice)
		local leftTime = jiceItem:getLeftUseCD()
		return leftTime > 0
	end
end

function This:isCityUsedDRGH(arg)
	if GamePlayer:isInWorld() then
		local city = WorldMapManager:getCity(arg.city)
		local stgList = city:getStgList()
		for i , stg in ipairs(stgList) do
			if stg.state == JiCeState.kDRGH then
				return true
			end
		end
	end
	return false
end

function This:cityUIMoveToCenter()
	if g_scene:getName() == "WorldScene" then
		local cityUI =  g_scene.m_worldUI.m_cityUI
		if cityUI then
			cityUI:moveToCenter()
		end
	end
end

function This:isRoleInWorldMap(arg)
	if g_scene:getName() == "WorldScene" then
		local role = WorldMapManager:getRole(arg.roleId)
		return role ~= nil
	end
end

function This:mapLayerStopMove()
	if g_scene:getName() == "WorldScene" then
		g_scene.m_mapLayer:stopAllActions()
	end
end

function This:showRecommend()
	require("Guide.GuideManager"):showRecommend()
end

--endregion
