local This, super = class(..., "cc.Node")
local GuideManager = require("GuideManager")

function This:onExit()
    if self.m_owner then
        self.m_owner.m_guideNode = false
    end
    GuideManager:removeListener(self)
end

function This:init(isLock)
    self.m_isTouchToEnd = true
    self.m_isLockMode = isLock
    self:setCascadeOpacityEnabled(true)
    if isLock then
        local layer = cc.LayerColor:create()
        layer:setOpacity(100)

        local drawNode = cc.DrawNode:create()
        local clippingNode = cc.ClippingNode:create(drawNode)
        clippingNode:setInverted(true)
        clippingNode:addChild(layer)
        self:addChild(clippingNode)
        self.m_drawNode = drawNode
    end
    self:addTouchListener({
        isSwallow = false,
        onEnded = function (touch, point)
            -- if self.m_maskTouch.targetTouchRect and self.m_isTouchToEnd then
            --     if self.m_maskTouch.targetTouchRect:isContainsPoint(point) then
            --         GuideManager:touchGuideNode(self)
            --     end
            -- end
        end,
        onMoved = function (touch, delta, point)
            if self.m_stepID == 1002 then
                local touchRect = cc.Rect(0, 0, 480, 230)
                if touchRect:isContainsPoint(point) then
                    GuideManager:touchGuideNode(self)
                end
            end
        end,
    })
    local maskTouch = self:addTouchListener({
        isSwallow = false,
        onBegan = function (touch, point)
            if touch.targetTouchRect and touch.lightRect then
                if touch.targetTouchRect:isContainsPoint(point) then
                    -- return false
                else
                    self:highLightRect(touch.lightRect)
                end
            end
        end
    })
    self.m_maskTouch = maskTouch
end

function This:disappear(isRemoveNow)
    if self.m_isLock then
        self.m_isLock = false
        g_scene.m_lockGuide = g_scene.m_lockGuide - 1
    end
    if self.m_lockTouch then
        self.m_lockTouch.m_unlock = 0
        self.m_lockTouch.onEnded = self.m_lockTouch._onEnded
        self.m_lockTouch._onEnded = nil
        self.m_lockTouch = nil
    end
    if isRemoveNow then
        self:removeFromParent()
    else
        self:runAction(cc.Sequence(cc.FadeOut(0.5), cc.RemoveAction()))
    end
    if not self.m_owner then
        return
    end
    self.m_owner.m_guideNode = false
    self.m_owner = false
    -- self.m_maskTouch:setEnabled(false)
end

function This:showSmallTips(text, x, y)
    if self.m_smallTips then
        self.m_smallTips:runActon(cc.Sequence(cc.FadeOut(0.3), cc.RemoveAction()))
        self.m_smallTips = false
    end
    local node = cc.Node:createFromCSFile("Common/NodeUI/GuideSmallTips.csb")
    node.text:setString(text, 400)

    local width, height = node.text:getContentSize()
    width = width
    if width < 340 then
        width = 340
    end
    local w, h = node.bg:getContentSize()
    node.bg:setContentSize(width, h)
    -- local width, height=node.textbg:getContentSize()
    -- node.textbg:setContentSize(width - 90, height)

    node:setPosition(x, y)
    node:runActionByID("Appear")
    self:addChild(node)
    self.m_smallTips = node
end

function This:highLightRect(targetRect)
    local lightBoxSprite = cc.Scale9Sprite:create("Common/Guide/gaoliangkuang.png")
    lightBoxSprite:setContentSize(cc.visible.width, cc.visible.height)
    self:addChild(lightBoxSprite)
    local width, height = lightBoxSprite:getContentSize()
    local rect = cc.Rect(targetRect)
    rect:centerExtend(50, 50)
    local anchorY = rect.y / (height - rect:getMaxY() + rect.y)
    local anchorX = rect.x / (width - rect:getMaxX() + rect.x)

    lightBoxSprite:setPosition(width * 0.5, height * 0.5)
    lightBoxSprite:applyAnchorPoint(anchorX, anchorY)
    lightBoxSprite:setOpacity(0)
    lightBoxSprite:runAction(cc.Sequence(cc.Spawn(cc.FadeIn(0.2), cc.SizeTo(0.3, rect.width, rect.height, "CubicIn")), cc.DelayTime(0.2), cc.FadeOut(0.4), cc.RemoveAction()))
    if self.m_drawNode then
        self.m_drawNode:clear()
        self.m_drawNode:drawSolidRect(targetRect, cc.Color.WHITE)
    end
end

function This:setHighLightButton(button)
    self.m_maskTouch.targetTouchRect = button.m_touch:getTouchRect()
    local lightNode = button.m_touch.sizeNode
    if button.m_touch.getRect and button.bg then
        lightNode = button.bg
    end
    self.m_maskTouch.lightRect = lightNode:getBoundingBox()
    if self.m_maskTouch.lightRect.width < 20 then
        local centerX, centerY = self.m_maskTouch.lightRect:getCenterXY()
        local width, height = 130, 60
        self.m_maskTouch.lightRect = cc.Rect(centerX - width * 0.5, centerY - height * 0.5, width, height)
    end
end

function This:showFinger(node)
    if not self.m_finger then
        self.m_finger = cc.Node:createFromCSFile("Common/NodeUI/Finger.csb")
        self.m_finger:playCSAnimation()
        self:addChild(self.m_finger)
    end
    if self.m_maskTouch.lightRect then
        local x, y = self.m_maskTouch.lightRect:getCenterXY()
        if y > 480 then
            self.m_finger:setRotation(90)
        end
        self.m_finger:setPosition(x, y)
    else
        self.m_finger:setPosition(node:convertToNodeSpaceAR(0, 0))
    end
end

function This:setStep(stepID, stepInfo, button)
    self.m_stepID = stepID
    self:setHighLightButton(button)
    if self.m_isLockMode then
        self.m_isLock = true
        button.m_touch.m_unlock = 1
    end
    local onEnded = button.m_touch.onEnded
    button.m_touch._onEnded = onEnded
    self.m_lockTouch = button.m_touch
    button.m_touch.onEnded =  function (touch, touchPoint)
        if onEnded then
            onEnded(touch, touchPoint)
        end
        touch.onEnded = touch._onEnded
        self.m_lockTouch = nil
        touch._onEnded = nil
        touch.m_unlock = 0
        if self.m_isLock then
            self.m_isLock = false
            g_scene.m_lockGuide = g_scene.m_lockGuide - 1
        end
        if self.m_drawNode then
            self.m_drawNode:clear()
        end
        GuideManager:touchGuideNode(self)
    end
    local wx, wy = button:convertToWorldSpace(0, 0)
    local ax, ay = wx / cc.visible.width, wy / cc.visible.height
    -- if ax > 0.5 then
    --     wx = wx - 100
    -- else
    --     wx = wx + 100
    -- end

    -- if ay > 0.5 then
    --     wy = wy - 150
    -- else
    --     wy = wy + 150
    -- end
    if wx < 180 then
        wx = 180
    end
    if wx > 450 then
        wx = 450
    end
    self:showSmallTips(stepInfo.text, wx, wy)
    self:showFinger(button)
    self:highLightRect(self.m_maskTouch.lightRect)

    button:runActionByID("Appear")
    if stepInfo and stepInfo.endOrder ~= 0 then
        self.m_isTouchToEnd = false
        local keyList = GuideManager:parseOrder(stepInfo.endOrder)
        local function checkOrder(self, name)
            if name == keyList[2] then
                GuideManager:removeListener(self)
                GuideManager:touchGuideNode(self)
            end
        end
        if keyList[1] == "Dialog" then
            self:addListener("add_dialog", checkOrder)
        elseif keyList[1] == "TabNode" then
            self:addListener("add_tapNode", checkOrder)
        elseif keyList[1] == "Event" then
            self:addListener("on_event", checkOrder)
        end
    end
end

function This:addListener(key, func)
    GuideManager:addListener(key, self, func)
end