-- @Author: su
-- @Date:   2018-12-19 16:52:20
-- @Last Modified by: su
-- @Last Modified time: 2018-12-19 17:11:54
-- @File Name: CountryArrayPanel.lua

local This,super = class(...,"Dialog")
local ArrayManager = require("Hero.ArrayManager")
local HeroManager = require("Hero.HeroManager")
local GameUtil = require("Util.GameUtil")


-- local HeroCountryName = {
--     Serialize[61202],
--     Serialize[61203],
--     Serialize[61204],
--     Serialize[61205]
-- }

local SoulTips=
{
    Serialize[61206],
    Serialize[61207],
    Serialize[61208]
}

local awakeLevelText=
{
    Serialize[61209],
    Serialize[61210],
    Serialize[61211],
    Serialize[61212],
    Serialize[61213]
}

local ArrayPic = {
    "MenuScene/wordzhenql.png",
    "MenuScene/wordzhenbh.png",
    "MenuScene/wordzhenzq.png",
    "MenuScene/wordzhenxw.png",
}

local ArrayBoxPic = {
    "MenuScene/zhenfaql.png",
    "MenuScene/zhenfabh.png",
    "MenuScene/zhenfazq.png",
    "MenuScene/zhenfaxw.png",
}

function This:initData( info )
    self.m_arrayData = ArrayManager:getArrayData()
    ArrayManager:onGetArrayInfoRequest()
end

function This:initNode( info )
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/CountryArray.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self:refreshTab(CountryArrayTab.kQingLong)
end

function This:refreshTab(Tab)
    if self.m_countryId and self.m_countryId == Tab then
        return 
    end
    self.m_countryId = Tab
    -- self.m_stage = self.m_arrayData:getArrayStage(Tab)
    self:refreshButtn(Tab)
	self:refresh()
end

function This:refreshButtn(Tab)
    for i=1,4 do
       self.m_csNode["troop"..i].light:setVisible(Tab == i)
    end
end

function This:refresh()
    local node = self.m_csNode
    local id = self.m_countryId
    local text=string.format( Serialize[61214],self.m_arrayData:getName(id),Serialize["Array_Tips_"..id])
    node.name:setImage(ArrayPic[id])
    node.num:setString(string.format( Serialize[61215],self.m_arrayData:getSoulNum(id)))
    node.describe:setString(self.m_arrayData:getDesc(id))
    node.tips:setStringWithColor(text)
    node.icon:setImage(ArrayBoxPic[self.m_countryId])

    node=node.value
    local number =self.m_arrayData:getMatrixSoulNum()
    local MatrixSoulNum = 0
    for i=1,5 do
        if i > number then
            for j=i,5 do
                node["value"..j]:setVisible(false)
            end
            break
        end
        local state = self.m_arrayData:getAddtionState(id,i)
        local text,num = self.m_arrayData:getStageArg(id,i)
        local tips = Serialize[61216]
        local color = "<A0>"
        if state then
            color = "<A1>"
            MatrixSoulNum = MatrixSoulNum + 1
        elseif i - MatrixSoulNum >= 2 then
            tips = Serialize[61217]
        else
            tips = string.format( Serialize[61218],self.m_arrayData:getStageNum(id,i))
        end
        node["value"..i]:setStringWithColor(string.format( Serialize[61219],color,i,CountryName[id],text,num,tips))
    end
    local scrollView = self.m_csNode.soul
    self.m_soulList = self.m_arrayData:getSoulItemList(id)
    scrollView:removeAllChildren()
    self.m_soulItem = {}
    for i,data in ipairs(self.m_soulList) do
        local item = cc.Node:createFromCSFile("MenuScene/NodeUI/CountryArraySoul.csb")
        self.m_soulItem[i] = item
        item.button2:setTouch({
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
                g_scene:openDialog("Hero.HeroListPanel")
            end
        })
        item.button1:setTouch({
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
                local data = self.m_arrayData:getDataByType(self.m_countryId,i)
                print("点击激活")
                ArrayManager:onActivationRequest(data.id)
                self:runActiveAction(item.button1)
            end
        })
        item.level:setStringWithColor(SoulTips[data.type])
        local level = self.m_arrayData:getStage(id,data.type)
        local maxLevel = self.m_arrayData:getMaxStage(id,data.type)
        -- local data = self.m_arrayData:getDataByType(id,data.type)
        item.num1:setStringWithColor(string.format( Serialize[61220],data.deployment_soul))
        local state,num = self.m_arrayData:getConditionState(id,data.type,data.parameter[1],data.parameter[2])
        local color = "<A0>"
        local text = self:getText(data.type,data.parameter[2])
        local addtionText,addtionNum = {},{}
        for index=1,table.count(data.arg) do
            addtionText[index],addtionNum[index] = self.m_arrayData:getConditionalArg(data.arg[index])
        end
        if state or level == maxLevel then
            color = "<New2>"
        end
        item.num2:setStringWithColor(string.format( Serialize[61221],color,data.parameter[1],CountryName[id],text,num,data.parameter[1]))
        if addtionText[2] then
            item.num3:setStringWithColor(string.format( Serialize[61222],color,CountryName[id],addtionText[1],addtionNum[1],addtionText[2],addtionNum[2]))
        else
            item.num3:setStringWithColor(string.format( Serialize[61223],color,CountryName[id],addtionText[1],addtionNum[1]))            
        end
        item.button1:setVisible(state and level ~= maxLevel)
        item.button2:setVisible(not state and level ~= maxLevel)
        item.num4:setVisible(level == maxLevel)
        scrollView:addChild(item)
        self.m_soulItem[i]:setPosition(253,120-(i-1)*90)
    end
end

function This:onEnter()
    super.onEnter(self)
    ArrayManager:addObserver(self)
    HeroManager:addObserver(self)
end

function This:onExit()
    ArrayManager:removeObserver(self)
    HeroManager:removeObserver(self)
    super.onExit(self)
end


function This:initTouchEvent()
    local node = self.m_csNode
    node.close:setTouchEnded(function ()
        self:close()
    end)
    node.button:setTouchEnded(function ()
        print("打开增幅")
        g_scene:openDialog("Hero.CountryArrayResonatePanel")
    end)
    node.details:setTouchEnded(function ()
        print("打开增幅")
        g_scene:openDialog("Hero.CountryValuePanel",{coutryId=self.m_countryId})
    end)

    for i=1,4 do
        node["troop"..i]:setTouch({ 
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
               self:refreshTab(i)
            end})
    end

    -- for i,node in pairs(self.m_soulItem) do
    --     node.button2:setTouch({
    --         isScale = false, 
    --         isDynamic = true, 
    --         onEnded = function()
    --             g_scene:openDialog("Hero.HeroListPanel")
    --         end
    --     })
    --     node.button1:setTouch({
    --         isScale = false, 
    --         isDynamic = true, 
    --         onEnded = function()
    --             local data = self.m_arrayData:getDataByType(self.m_countryId,i)
    --             print("点击激活")
    --             ArrayManager:onActivationRequest(data.id)
    --             self:runActiveAction(node.button1)
    --         end
    --     })
    -- end

end


function This:runActiveAction(node)
    local nodeX,nodeY = node:getPosition()
    local effect = nil
    effect = GameUtil:createEffect("ChuFa5", false, nil, {
        onAnimationComplete = function ()
            effect:removeFromParent()
            local particle = cc.Particle:create("tuowei.plist", self)
            particle:setPosition(nodeX, nodeY)

            local action = nil
            local x,y = self.m_csNode.num:getPosition()
            action = cc.Sequence(cc.MoveTo(1,x+20,y),
            cc.RemoveAction())
            cc.Audio:playEffect("Recruit_Change.mp3")

            particle:runAction(action)
            self.m_csNode:addChild(particle)
    end})
    effect:setPosition(0,0)
    effect:setVisible(false)
    self.m_csNode:addChild(effect)

end


function This:getText(type,num)
    local text
    if type == 1 then
        text = string.format( Serialize[60312],num )
    elseif type == 2 then
        local star = math.modf( num/5 )
        local rank = num%5
        text = string.format( Serialize[61224],star,rank )
    else
        text = awakeLevelText[num]
    end
    return text
end

function This:onGetArrayInfo()
    self:refresh()
end

function This:onActivationArray()
    self:refresh()
end

function This:onUpgradeLevel()
    ArrayManager:onGetArrayInfoRequest()
end

function This:onUpgradeStarLv()
    ArrayManager:onGetArrayInfoRequest()
end

function This:onAddAttrPoint()
    ArrayManager:onGetArrayInfoRequest()
end