-- @Author: superm
-- @Date:   2018-09-15 14:30:55
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-08-09 09:43:17
-- @File Name: FateCondition.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local HeroManager = require("Hero.HeroManager")
function This:init(info)
    super.init(self)
    self.m_fate = info.fate
    self.m_type = info.type
    self.m_heroOrOtherId = info.heroOrOtherId
    if self:checkType(FateCondType.kHero) then
        self.m_hero = HeroManager:getHero(self.m_heroOrOtherId)
    else
        self.m_hero = info.hero
    end
    self.m_heroId = self.m_hero:getId()
    self.m_lvType = info.lvType
    self.m_levelInfo = info.levelInfo
end

function This:getHero()
    return self.m_hero
end

function This:getName()
    return self.m_hero:getName()
end

function This:getFate()
    return self.m_fate
end

function This:getHeroId()
    return self.m_heroId 
end

function This:getSoilderInfo()
    if not self:isSoilderCondType()then
    	return
    end
    local isAwakeSoilser = self:checkLvType(FateSoilderType.kAwakeLv)
	local soilder = self.m_hero:getSoilder(self.m_heroOrOtherId)
    if isAwakeSoilser then
        soilder = self.m_hero:getAwakeSoilder(self.m_heroOrOtherId)
    end
    if type(self.m_heroOrOtherId) ~= "table" then
		return {[1] = soilder}
    else
    	local ret = {}
    	for index, sid in ipairs(self.m_heroOrOtherId) do
    		ret[index] = soilder
    	end
		return ret
    end
end

function This:getValue()
	if not self.m_hero:isGot() then
		return -1
	end

	if self:checkType(FateCondType.kHero) then
        if self:checkLvType(FateHeroType.kPhase) then
            return self.m_hero:getTotalPhase()
        elseif self:checkLvType(FateHeroType.kLv) then
            return self.m_hero:getLevel()
        elseif self:checkLvType(FateHeroType.kAwakeLv) then
            return self.m_hero:getAwakeLevel( )
        end
		return self.m_hero:getTotalPhase()
    elseif self:checkType(FateCondType.kSoilder) then
		local soilderList = self:getSoilderInfo()
		local soilder = soilderList[1]
        local phase = 0
        if soilder:isUnlock() then
            phase = soilder:getPhase()
        end
		return phase
    end
end

function This:getLevelInfo()
    return self.m_levelInfo 
end

function This:isItemType()
    return false
end


function This:checkType(fateCondType,customType)
    local temType = customType or self:getCondType()
    if fateCondType and temType then
        return temType == fateCondType
    end
    return false
end

function This:getCondType()
    return self.m_type
end

function This:getLvType()
    return self.m_lvType
end

function This:checkLvType(lvType)
   return self:checkType(lvType,self:getLvType()) 
end

function This:isSoilderCondType()
    return self:checkType(FateCondType.kSoilder)
end

function This:isStarType()
    if self:checkType(FateCondType.kHero) and self:checkLvType(FateHeroType.kPhase) then
        return true
    end
    return false
end

function This:isLevelType()
    if self:checkType(FateCondType.kHero) and self:checkLvType(FateHeroType.kLv) then
        return true
    end
    return false
end

