-- @Author: superm
-- @Date:   2018-09-11 16:05:42
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-09-21 17:20:03
-- @File Name: FateInfo.lua
-- @Description: 
local This = class(...)
local HeroFate, BaseModel = class(This._NAME .. ".HeroFate", "Base.BaseModel")
local FateCondition = require("Hero.FateCondition")
local FateConditionItemType = require("Hero.FateConditionItemType")
local FatalismNewConfig = require("Metadatas.FatalismNewConfig")
local ConditionNewConfig = require("Metadatas.ConditionNewConfig")
local HeroManager = require("Hero.HeroManager")
local HeroConfig = require("Metadatas.HeroConfig")

FateCondType = {
	kHero = 1,  
	kEquip = 2,
	kFabao = 3,
	kSoilder = 4, 
}
FateHeroType = {
	kPhase = 1,--阶数(5阶为1星)
	kLv =2,--等级
	kAwakeLv=3,--武将觉醒等级
}
FateEquipType = {
	kLv = 1,
	kStrengthen = 2,
}
FateFabaoType = {
	kLv = 1,
	kRefineLv   = 2,--法宝精炼等级
	kStarlv = 3,
}
FateSoilderType = {
	kPhase = 1,--阶数
	kAwakeLv=2,--兵种觉醒等级
}

function HeroFate:init(info)
    BaseModel.init(self)
    self.m_id = info.id
    self.m_level = info.level or 0
    -- self.m_state = info.state
	self.m_config = info.config or FatalismNewConfig[self.m_id]
	self.m_heroId = info.heroId
	-- self.m_ownerIds = {}
end

function HeroFate:updateInfo(info)
    if self.m_id == info.id then
	    self.m_level = info.level
	    -- self.m_state = info.state
	end
end

function HeroFate:reset()
    self.m_level = 0
    -- self.m_state = nil
end

function HeroFate:getId()
    return self.m_id
end

--ownerId
function HeroFate:getHeroId()
	return self.m_heroId
end

function HeroFate:getLevel()
	-- if self.m_state == 2 then
    	return self.m_level
	-- end
	-- return 0
end

function HeroFate:getDescByLevel(level)
	local config = self:getConfig()
    local attrId, attrValueList = config.attr_id, config.numeral
    local value = attrValueList[level]
    local isPercent = AttrInfo[attrId].valueType == ValueType.kPercent
    local formatStr
    if level > 1 then
    	formatStr = string.numberToWord(level-1) .. Serialize[61232]
    else
    	formatStr = Serialize[61233]
    end
    if isPercent then 
    	return string.format(formatStr .. Serialize[61234], AttrName[attrId], value * 100)
    else 
    	return string.format(formatStr .. Serialize[61235], AttrName[attrId], value)
    end
end

function HeroFate:getDesc()
	return self:getDescByLevel(self:getLevel())
end

function HeroFate:canActivate(heroId)
	if self.m_level <= 0 then
		-- if self.m_state then
  --   		return self.m_state == 1 and self:canUpgrade(0, heroId)
  --   	else
    		return self:canUpgrade(0, heroId)
    	-- end
    else
     	return self:canUpgrade(nil, heroId)
    end
end

function HeroFate:canUpgrade(paramLevel, heroId)
	local curFateLevel = paramLevel
	if not curFateLevel then
		local curLevel = self:getLevel()
		if not curLevel or curLevel <= 0 then
			curFateLevel = 0
		else
			curFateLevel = curLevel
		end
	end
	local condList = self:getAllConditionList() 
	local ret = true 
	for i, cond in ipairs(condList) do

		if heroId and cond:isSoilderCondType()
				and cond:getSoilderInfo()[1]:getId() ~= HeroManager:getHero(heroId):getCurSoilder():getId()
		then
			return false
		end

		local value = cond:getValue()
		local levelInfo = cond:getLevelInfo()
		if type(levelInfo) ~= "table" then
			ret = value >= levelInfo
			if not ret then return false end
		else
			local count = #levelInfo
			local curIndex = curFateLevel + 1
			if curIndex <= count then
				local level = levelInfo[curIndex] or 0
				ret = value >= level
				if not ret then return false end
			end
		end
	end
	return ret
end

-- function HeroFate:getConditionWith(condId)
-- 	local res = nil
-- 	if condId then
-- 		local condList = self:getAllConditionList() 
-- 		for i, cond in ipairs(condList) do
-- 			if cond:getId() == condId then
-- 				return cond
-- 			end
-- 		end
-- 	end
-- 	return nil
-- end

function HeroFate:getCondTypeList(paramLevel)
	local curFateLevel = paramLevel
	if not curFateLevel then
		local curLevel = self:getLevel()
		if not curLevel or curLevel <= 0 then
			curFateLevel = 0
		else
			curFateLevel = curLevel
		end
	end
	local condList = self:getAllConditionList() 
	local ret = {} 
    for i, cond in ipairs(condList) do
    	local levelInfo = cond:getLevelInfo()
		-- if type(levelInfo) ~= "table" then
		-- 	local level = levelInfo
  --   		ret[cond.m_type] = level
		-- else
			local count = #levelInfo
			local curIndex = curFateLevel + 1
			if curIndex <= count then
				local level = levelInfo[curIndex] or 0
				table.insert(ret,{cond = cond,level=level})
    			-- ret[cond.m_type] = {level}
			end
		-- end
    end
    return ret
	
end

function HeroFate:getAllConditionList()
	local config = self.m_config

	local dataObjList = {}
	local conditionList = self.m_conditionList
	if not conditionList then
		conditionList = {}

		self.m_conditionList = conditionList
		local configDataList = config.condition_set
		-- local heroIdList = config.card_set
		local myHero = HeroManager:getHero(self:getHeroId())
		for configIndex, configId in ipairs(configDataList) do
			local configData = ConditionNewConfig[configId]
			local typeId = configData.type
			local heroOrOtherId = configData.type_id
			local lvType = configData.arg_type
			local levelInfo = configData.arg_value
			if table.contain({FateCondType.kHero,FateCondType.kSoilder}, typeId) then
				if typeId == FateCondType.kHero then
					local hero = HeroManager:getHero(heroOrOtherId)
					local param = {
						fate = self,
						type= typeId,
						hero = HeroManager:getHero(heroOrOtherId),
						heroOrOtherId = heroOrOtherId,
						lvType = lvType,
						levelInfo = levelInfo,
					}
					local fcond = FateCondition:create(param)
					-- print("fcond",fcond:getName(),configId,typeId,configData.id)
					table.insert(conditionList, fcond)
				else
					local param = {
						fate = self,
						type= typeId,
						hero = myHero,
						heroOrOtherId = heroOrOtherId,
						lvType = lvType,
						levelInfo = levelInfo,
					}
					local fcond = FateCondition:create(param)
					-- print("fcond",fcond:getName(),configId,typeId,configData.id)
					table.insert(conditionList, fcond)
				end
			else
				local param = {
					fate = self,
					type= typeId,
					hero = myHero,
					heroOrOtherId = heroOrOtherId,
					lvType = lvType,
					levelInfo = levelInfo,
				}
				local fcond = FateConditionItemType:create(param)
				-- print("fcond",fcond:getName(),configId,typeId,configData.id)
				table.insert(conditionList, fcond)
			end
		end
	end
	return conditionList
end

function HeroFate:getName()
    return self.m_config.name
end

function HeroFate:getConfig()
    return self.m_config
end

function HeroFate:getAttrValue()
	local attrId = self.m_config.attr_id
	if self:getLevel() > 0 then
		local attrNumList = self.m_config.numeral
	    return attrId, attrNumList[self:getLevel()]
	else
		return attrId, 0
	end
end

function HeroFate:isContainHeroId( heroId )
    local idList = self:getOwnerHeroList()
    if idList then
    	return table.contain(idList, heroId)
    end
end

function HeroFate:getOwnerHeroList()
	local ret = {self.m_heroId}
	-- for heroId, state in pairs(self.m_ownerIds) do
	-- 	if state then
	-- 		table.insert(ret, heroId)
	-- 	end
	-- end
    return ret
end

function This:create(...)
	local ret = {}
	setmetatable(ret, self)
	ret:init(...)
	return ret
end

function This:init()
	-- local heroMap = {}
	local heroIdMap = {}

	self.m_fateHeroIdList = {}--拥有宿命的武将id
	-- self.m_fateHeroMap = heroMap--宿命对应的武将
	self.m_fates = {}
	-- for fid, config in pairs(FatalismNewConfig) do
	-- 	for _, heroId in ipairs(config.card_set) do
	-- 		heroIdMap[heroId] = true
	-- 	end

	-- 	local fate = HeroFate:create({id = fid, config = config})
	-- 	self.m_fates[fid] = fate
	-- 	heroMap[fid] = fate.m_ownerIds
	-- end

	for heroId, config in pairs(HeroConfig) do
		local heroFateKeyList ={}
		local fidList = config.fatalism
		if fidList and #fidList > 0 then
			for _, fid in ipairs(fidList) do
				local fateConfit = FatalismNewConfig[fid]
				local fate = HeroFate:create({id = fid, config = fateConfit,heroId = heroId})
				heroFateKeyList[fid] = fate
				-- heroMap[fid] = fate.m_ownerIds
				-- heroMap[fid][heroId] = true
			end
			self.m_fates[heroId]= heroFateKeyList
			heroIdMap[heroId] = true
		end
	end

	for heroId, _ in pairs(heroIdMap) do
		table.insert(self.m_fateHeroIdList, heroId)
	end
end

-- function This:getFate(fid)
--     return self.m_fates[fid]
-- end

function This:updateInfo(info)
	-- if not info then
	-- 	return
	-- end
	-- -- print_r(info)
	-- if info.active_list or info.activatable_list then
	-- 	for _, fate in pairs(self.m_fates) do
	-- 		fate:reset()
	-- 	end

	-- 	if info.active_list then
	-- 	    for _, fateData in ipairs(info.active_list) do
	-- 			self:updateFate(fateData)
	-- 	    end
	-- 	end
	--     if info.activatable_list then
	-- 	    for _, fateData in ipairs(info.activatable_list) do
	-- 			self:updateFate(fateData)
	-- 	    end
	-- 	end
	-- end
	-- HeroManager:updateFateAttr()
end

function This:updateFate(info,uid,heroId)
	local fateKeyList = self:getFateByHeroId(heroId,uid)
	local fate = fateKeyList[info.id]
	if fate then
		fate:updateInfo(info)
	end
	return fate
end

function This:calcFateFightPower(withoutCalc, hero)
--     if withoutCalc then
-- 		HeroManager:onGetFateInfoRequest()
-- 	else
	if hero then
		self:updateStateByHero(hero)
	end
end

function This:getFateAttrAdditionList(heroId)
	local idList = self:getFateByHeroId(heroId,uid)
	local attrs = {}
	for _, fate in pairs(idList) do
		local attrid, value = fate:getAttrValue()
		if attrid > 0 then
			attrs[attrid] = (attrs[attrid] or 0) + value
		end
	end
	-- print_tmp(attrs)
	return attrs
end

function This:getFateByHeroId(heroId,uid)
	if heroId then
		return self.m_fates[heroId]
	else
		local hero= HeroManager:getHeroByUid(uid)
		if hero then
			local temHeroId = hero:getId()
			return self.m_fates[temHeroId]
		end
	end
	return {}
end

function This:getFateByHero(hero)
	local fateList = {}
	local idList = self:getFateByHeroId(hero:getId())
	if idList then
	    for _, fate in pairs(idList) do
			table.insert(fateList, fate)
	    end
	end
	table.sort(fateList,function(a,b)
		
		return a:getId() < b:getId()
	end)
    return fateList
end

function This:updateStateByHero(hero)
	if not hero then
		return
	end

	local heroId = hero:getId()
	local idList = hero.m_config.fatalism
	if idList and #idList>0 then
		local fateList = self:getFateByHeroId(heroId)
		local fate = fateList[idList[1]]
		if fate then
	    	fate:notifyObservers("onRefreshHero", heroId)
		end
	end
end

function This:getAllHeroList()
    return self.m_fateHeroIdList
end