-- @Author: superm
-- @Date:   2018-09-12 16:29:45
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-29 12:19:20
-- @File Name: FateInfoListItem.lua
-- @Description: 

local This , super = class(..., "NodeView")
local FateHeroView = require("Hero.FateHeroView")
local HeroManager = require("Hero.HeroManager")
local GameUtil = require("Util.GameUtil")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")

function This:initData(info)
    if info then
        self.m_fate = info.fate
        self.m_hero = info.hero
    end
    self.m_inited = false

    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/HeroFatalismItem.csb")
    self.m_csNode = csNode
    self:addChild(csNode)
    self:setContentSize(530, 130)
end

function This:initNode()
    local scrollView = self.m_csNode.scrollView
	scrollView:setDirection("horizontal")
	scrollView.m_direction = "right"
    self:refresh()
end

function This:onExit()
    if self.m_btnEffect then
        self.m_btnEffect:unscheduleUpdate()
    end
    super.onExit(self)
    HeroManager:removeObserver(self)
    
end

function This:onEnter()
    super.onEnter(self)
    HeroManager:addObserver(self)
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_csNode.chakan:setTouch({
			onEnded = function()
                if not self.m_fate then
                    return
                end
                local param = {hero = self.m_hero, fate = self.m_fate}
                g_scene:openDialog("Hero.FateInfoPanel", param)
            end,
            isContainer = true,
            isScale = false
        })
        self.m_inited = true
    end
	self.m_csNode.scrollView:setSwallowTouch(false)
end

function This:setModel(info)
    if info then
        self.m_fate = info.fate
        self.m_hero = info.hero
        self:refresh()
    end
end

function This:refresh()
    local csNode = self.m_csNode
    local config = self.m_fate:getConfig()
	csNode.name:setString(config.name)
	
	local visible = self.m_fate:getLevel() > 0
	csNode.invalid:setVisible(not visible)
	csNode.state:setVisible(visible)
	if visible then
		csNode.state:setString(self.m_fate:getDesc())
	end

	local scrollView = csNode.scrollView
	scrollView:removeAllItems()
    local condList = self.m_fate:getAllConditionList()
    for index, fcond in ipairs(condList) do
		local view = FateHeroView:create(fcond)
        view:setScale(0.8)
    	local w, h = view:getContentSize()
    	scrollView:pushBackCustomItem(view, w * 0.8, h, 0.5)
	end
    self:refreshBtnEffect(self.m_csNode.chakan)
end

function This:refreshBtnEffect(node)
    local state
    local totalLev = 0

    local level = self.m_fate:getLevel()
    local config = self.m_fate:getConfig()
    if config and config.numeral then
        totalLev = #config.numeral
    end
    if level <= 0 then
        state = self.m_fate:canActivate(self.m_hero:getId())
    else
        if level >= totalLev then
            state = false
        else
            state = self.m_fate:canUpgrade(nil, self.m_hero:getId())
        end
    end
    if state then
        node.text:setString(level <= 0 and Serialize[61237] or Serialize[61238])
    else
        node.text:setString(Serialize[61239])
    end

    SignalSystem.distributeSignal(Signals.HeroPanelFateTabRedDot)

    local isIniting = false
    if state and not self.m_btnEffect then
        self.m_btnEffect = GameUtil:createEffect("Ready",true, node)
        self.m_btnEffect:setScaleX(0.75)
        node:addChild(self.m_btnEffect)
        isIniting = true
    elseif not state and self.m_btnEffect then
        self.m_btnEffect:removeFromParent()
        self.m_btnEffect = nil
    end

    if self.m_btnEffect and not isIniting then
        self.m_btnEffect:scheduleUpdate()
    end
end

function This:onActFateivate( fate )
    if fate and self.m_fate:getId() == fate:getId() then
        local csNode = self.m_csNode
        local visible = self.m_fate:getLevel() > 0
        csNode.invalid:setVisible(not visible)
        csNode.state:setVisible(visible)
        if visible then
            csNode.state:setString(self.m_fate:getDesc())
        end
        self:refreshBtnEffect(csNode.chakan)
    end
end

function This:onAttachSoilder(heroId)
    self:refreshBtnEffect(self.m_csNode.chakan)
end