-- @Author: superm
-- @Date:   2018-09-12 16:28:10
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-08-09 10:13:50
-- @File Name: FateInfoPanel.lua
-- @Description:

local This, super = class(...,"Dialog")
local ViewUtil = require("UI.ViewUtil")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local HeroManager = require("Hero.HeroManager")
local FateHeroView = require("Hero.FateHeroView")

local kMaxAttrNum = 6

function This:initNode(info)
    if info then
        self.m_fate = info.fate
    	self.m_hero = info.hero
    end
	local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/HeroFatalism.csb")
    self.m_csNode = csNode
    csNode:setToCenter()
	self:addChild(csNode)
    self:initCondition()
	self:initDescText()
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.close:setTouchEnded(function()
		self:close()
	end)

    csNode.jihuobutton:setTouchEnded(function()
        local level = self.m_fate:getLevel()
        if level <= 0 then
            if not self.m_fate:canActivate() then
                return g_scene:showTips(Serialize[61241])
            end
        else
            local totalLev = self.m_totalLev
            if level >= totalLev then
                return g_scene:showTips(Serialize[60353])
            end

            if not self.m_fate:canUpgrade() then
                return g_scene:showTips(Serialize[61241])
            end
        end

		local fid = self.m_fate:getId()
		HeroManager:onActivateFateRequest(fid,self.m_hero:getUid())
	end)
end

function This:onEnter()
    super.onEnter(self)
    HeroManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    HeroManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
    self:refreshHeros()
    self:refreshDescText()
end

function This:initDescText()
	local csNode = self.m_csNode
	local config = self.m_fate:getConfig()
    local attrId, attrValueList = config.attr_id, config.numeral
    local isPercent = AttrInfo[attrId].valueType == ValueType.kPercent
    self.m_totalLev = #attrValueList
    for i = 1, self.m_totalLev do
        local desc
        local value = attrValueList[i]

        local formatStr
        if i > 1 then
            formatStr = string.numberToWord(i-1) .. Serialize[61232]
        else
            formatStr = Serialize[61233]
        end
        if isPercent then 
            desc = string.format(formatStr .. Serialize[61234], AttrName[attrId], value * 100)
        else 
            desc = string.format(formatStr .. Serialize[61235], AttrName[attrId], value)
        end
    	local node = csNode["suming" .. i]
		node:setString(desc)
    end
    csNode.title:setString(config.name)
end

function This:initCondition()
    local csNode = self.m_csNode
    local config = self.m_fate:getConfig()

    function createHeros(heroNode, infoList)
        local w = 100
        for i, info in ipairs(infoList) do
            local node = FateHeroView:create(info)
            node:setAnchorPoint(0,0)
            node:setPositionX(w)
            w = w + node:getContentWidth() + 10
            heroNode:addChild(node)
        end
        heroNode:setPositionX(- w * 0.8 / 2) 
    end 
    local ownerId = self.m_hero:getId()
    local condList = self.m_fate:getAllConditionList()
    local mainCondList, otherCondList = {}, {}
    for _, cond in ipairs(condList) do
        if cond:isItemType() then
            table.insert(otherCondList, cond)
        elseif not cond:isSoilderCondType() and cond:getHeroId() == ownerId then
            table.insert(mainCondList, cond)
        else
            table.insert(otherCondList, cond)
        end
    end
    createHeros(csNode.hero, mainCondList)
    createHeros(csNode.cond, otherCondList)
end

function This:refreshHeros()
end

function This:refreshDescText()
    local csNode = self.m_csNode
    local totalLev = self.m_totalLev
    local level = self.m_fate:getLevel()
    for i = 1, totalLev do
        local node = csNode["suming" .. i]
    	if i ~= level then
            node:setColor(136,152,190)
        else
    		node:setColor(184,228,97)
    	end
    	node:setVisible(true)
    end

    for i = totalLev + 1, kMaxAttrNum do
    	local node = csNode["suming" .. i]
    	node:setVisible(false)
    end

    if level < totalLev then
        local lastEquipType = nil
        local ret = self.m_fate:getCondTypeList()
        local descList = {}  
        for index, condInfo in pairs(ret) do
            local conditionData = condInfo.cond
            local condType = conditionData:getCondType()
            local level = condInfo.level
            local name = conditionData:getName()
            local isEquipType = conditionData:isItemType()
            if level <= 0 then
                -- if lastEquipType and lastEquipType == condType then
                --     local desCount = #descList
                --     descList[desCount] = descList[desCount].."、"..name
                -- else
                    if isEquipType then
                        table.insert(descList, Serialize[61242]..name)
                    else
                        table.insert(descList, Serialize[61243]..name)
                    end
                -- end
                -- lastEquipType = condType
            elseif not isEquipType then
                if conditionData:checkType(FateCondType.kHero) then
                    if conditionData:checkLvType(FateHeroType.kPhase) then
                        table.insert(descList, string.format(name..Serialize[61244], level/5))
                    elseif conditionData:checkLvType(FateHeroType.kLv) then
                        table.insert(descList, string.format(name..Serialize[61245], level))
                    elseif conditionData:checkLvType(FateHeroType.kAwakeLv) then
                        table.insert(descList, string.format(name..Serialize[61246], level))
                    end
                elseif conditionData:checkType(FateCondType.kSoilder) then
                    if conditionData:checkLvType(FateSoilderType.kAwakeLv) then
                        table.insert(descList, string.format(name..Serialize[61247], level))
                    elseif conditionData:checkLvType(FateSoilderType.kPhase) then
                       table.insert(descList, string.format(name..Serialize[61248], level)) 
                    end
                end
            else
                if conditionData:checkType(FateCondType.kEquip) then
                    if conditionData:checkLvType(FateEquipType.kLv) then
                        table.insert(descList, string.format(name..Serialize[61249], level))
                    elseif conditionData:checkLvType(FateEquipType.kStrengthen) then
                        table.insert(descList, string.format(name..Serialize[61250], level))
                    end
                elseif conditionData:checkType(FateCondType.kFabao) then
                    if conditionData:checkLvType(FateFabaoType.kLv) then
                        table.insert(descList, string.format(name..Serialize[61249], level))
                    elseif conditionData:checkLvType(FateFabaoType.kRefineLv) then
                        table.insert(descList, string.format(name..Serialize[61251], level))
                    elseif conditionData:checkLvType(FateFabaoType.kStarlv) then
                        table.insert(descList, string.format(name..Serialize[61252], level))
                    end
                end
            end
        end
        local str = table.concat(descList, "，")
        csNode.updesc:setString(level <= 0 and (str .. Serialize[61253]) or (str .. Serialize[61254]))
        csNode.jihuobutton.text:setString(level <= 0 and Serialize[61255] or Serialize[61256])

        csNode.jihuobutton:setVisible(true)
        csNode.updesc:setVisible(true)
    else
        csNode.jihuobutton:setVisible(false)
        csNode.updesc:setVisible(false)
    end
end

function This:onActFateivate(fate)
	if fate and self.m_fate:getId() == fate:getId() then
        local level = fate:getLevel() - 1 
        if level > 0 then
            g_scene:showTips(Serialize[61257])
        else
            g_scene:showTips(Serialize[61258])
        end
    	self:refreshDescText()
    end
end