-- @Author: superm
-- @Date:   2018-09-12 16:26:00
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-08-09 16:49:17
-- @File Name: FateTabNode.lua
-- @Description: 

local This , super = class(...,"cc.Node")
local HeroManager = require("Hero.HeroManager")
local EquipManager = require("Equip.EquipManager")
local FabaoManager = require("Fabao.FabaoManager")
local FateInfoListItem = require("Hero.FateInfoListItem")

function This:init(heroId)
    self.m_fateViewList = {}
	self.m_hero = HeroManager:getHero(heroId)
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/HeroFatalismMain.csb")
	self:addChild(self.m_csNode)
end

function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
	EquipManager:addObserver(self)
	FabaoManager:addObserver(self)
	self:initTouchEvent()
	self:refresh()
end

function This:onExit()
    if self.m_fateViewList then
    	for _, view in ipairs(self.m_fateViewList) do
    		view:release()
    	end
        self.m_fateViewList = nil
    end

	HeroManager:removeObserver(self)
	EquipManager:removeObserver(self)
	FabaoManager:removeObserver(self)
	super.onExit(self)
end

function This:initTouchEvent()
end

function This:setModel(heroId)
	if self.m_hero:getId() ~= heroId then
		self.m_hero = HeroManager:getHero(heroId)
		self:refresh()
	end 
end

function This:refresh()
    local csNode = self.m_csNode
	local scrollView = csNode.scrollView

	scrollView:removeAllItems()
	local fateInfo = HeroManager:getFateInfo()
	local fateList = fateInfo:getFateByHero(self.m_hero)
	
	for index, fate in ipairs(fateList) do
		local param = {
			hero = self.m_hero,
			fate = fate,		
		}
    	local view = self.m_fateViewList[index]
    	if view then
    		view:setModel(param)
    	else
    		view = FateInfoListItem:create(param)
    		view:retain()
    		self.m_fateViewList[index] = view
    	end
    	local w, h = view:getContentSize()
    	scrollView:pushBackCustomItem(view, w, h, 0.5)
	end
	csNode.nosuming:setVisible(#fateList <= 0)
end

function This:onHeroReset(hero)
	if hero and self.m_hero and hero:getId() == self.m_hero:getId() then
		self:refresh()
	end
end

function This:onEquipChanged(heroId)
	if heroId and heroId == self.m_hero:getId() then
		self:refresh()
	end
end

function This:onStrengthLevel(equipUid)
	local equip = EquipManager:getEquip(equipUid)
	if equip and equip:getHeroId() == self.m_hero:getId() then
		self:refresh()
	end 
end

This.onTrainSoilder = This.onEquipChanged
This.onAttachSoilder = This.onEquipChanged
This.onUpgradeStarLv = This.onEquipChanged
This.onUpgradeLevel = This.onEquipChanged
This.onEquipUpLevel = This.onStrengthLevel
This.onEquipUpgradeQuality = This.onStrengthLevel
This.onGetAwake = This.refresh
This.onGetForceAwake = This.refresh
This.onUpPhaseSoilder = This.refresh