-- @Author: su
-- @Date:   2018-12-20 10:52:20
-- @Last Modified by:   su
-- @Last Modified time: 2018-12-20 10:52:20
-- @File Name: GameArrayData.lua

local DeploymentAchievementConfig = require("Metadatas.DeploymentAchievementConfig")
local DeploymentDetailsConfig = require("Metadatas.DeploymentDetailsConfig")
local DeploymentResonanceConfig = require("Metadatas.DeploymentResonanceConfig")
local GameUtil = require("Util.GameUtil")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")


local This = class(...)


function This:create(id)
    local res = {}
    setmetatable(res,This)
    res:init(id)
    return res
end

function This:init(id)
    self.m_detailConfig = {}
    self.m_achievementConfig = {}
    self.m_addConfig = DeploymentResonanceConfig
    for _,config in pairs(DeploymentDetailsConfig) do
        if not self.m_detailConfig[config.deployment_id] then
            self.m_detailConfig[config.deployment_id] ={}
        end
        table.insert(self.m_detailConfig[config.deployment_id],config)
    end
    for i = 1, 4 do
        table.sort(self.m_detailConfig[i],function (a,b)
            if a.id < b.id then
                return true
            end
        end ) 
    end
    for _,config in pairs(DeploymentAchievementConfig) do
        --self.m_achievementConfig[config.country]= config
        if not self.m_achievementConfig[config.country] then
            self.m_achievementConfig[config.country] ={}
        end
        if not self.m_achievementConfig[config.country][config.type] then
            self.m_achievementConfig[config.country][config.type]={}
        end
        table.insert(self.m_achievementConfig[config.country][config.type],config)
    end
    for i=1,4 do
        for j=1,3 do
            table.sort(self.m_achievementConfig[i][j],function (a,b)
                if a.id < b.id then
                    return true
                end
            end ) 
        end
    end
end

function This:getMaxResonanceNum()
    return table.count(self.m_addConfig[1].resonance_arg)
end

function This:setTroopsIdList( list )
    local res = {{},{},{},{}}
    local soulNum = {0,0,0,0}
    for _,id in pairs(list) do
        local data = DeploymentAchievementConfig[id]
        table.insert(res[data.country],data)
        soulNum[data.country]=soulNum[data.country] + data.deployment_soul
    end
    self.m_troopsIdList = res
    self.m_soulNum = soulNum
end

function This:updateTroopsIdList(id)
    local data = DeploymentAchievementConfig[id]
    local state = true
    for i,config in pairs (self.m_troopsIdList[data.country]) do
        if config == data then
            state = false
        end
    end
    if state then
        table.insert(self.m_troopsIdList[data.country],data)
    end
end

function This:getTroopsIdList()
    local res = {{},{},{},{}}
    return self.m_troopsIdList or res
end

function This:getSoulNum(countryId)--获取对应阵法阵魂值
    local res = self:getTroopsIdList()
    local soulNum = 0
    for _,data in pairs(res[countryId]) do
        soulNum = soulNum + data.deployment_soul
    end
    return soulNum
end

function This:getName(countryId)  --获取阵法名字
    return  self.m_detailConfig[countryId][1].name
end

function This:getDesc(countryId) -- 获取阵法描述
    return  self.m_detailConfig[countryId][1].desc
end

function This:getAddLevel() -- 获取当前阵法增幅等级
    local res= {}
    for i=1,4 do
        local num = self:getSoulNum(i)
        table.insert(res,num)
    end
    local soulMinNum = math.min( res[1],res[2],res[3],res[4])
    local addLevel = 0
    for level,data in ipairs(self.m_addConfig) do
        if soulMinNum >= data.resonance_requirement[1] then
            addLevel = addLevel + 1
        end
    end
    return addLevel
end

function This:getNextAddLevel()
    local level = self:getAddLevel()
    if level < table.count(DeploymentResonanceConfig) then
        level = level + 1
    end
    return level
end

function This:getAddStageNum(countryId,level)
    if not level then
        level = self:getAddLevel()
    end
    if level == 0 then
        level=1
    end
    return self.m_addConfig[level].resonance_requirement[countryId]
end


function This:getStageArg(countryId,index)  -- 获取属性加成文本
    local info =  self.m_detailConfig[countryId][index].stage_arg[1]
    local text,num = GameUtil:getAttrStr(info[1],info[2])
    return text,num
end

function This:getConditionalArg(info)   -- 获取属性加成文本
    local text,num = GameUtil:getAttrStr(info[1],info[2])
    return text,num
end

function This:getAddArg(index,level)      -- 通过下标获取增幅属性加成文本
    if level == 0 then
        level = 1
    end
    local text,num = " ",0
    local info = self.m_addConfig[level].resonance_arg[index]
    if info then
        text,num = GameUtil:getAttrStr(info[1],info[2])
    end
    return text,num
end

function This:getMatrixSoulNum()    -- 获取阵眼数量
    return table.count(self.m_detailConfig[1])
end

function This:getStageNum(countryId,index)  --获取阶段阵魂值
    if not index then
        index = self:getArrayStage(countryId)
    end
    if index == 0 then
        index = 1
    end
    return self.m_detailConfig[countryId][index].stage
end

function This:getArrayStage(countryId)  --获取阵法阶段
    local soulNum = self:getSoulNum(countryId)
    local index = 0
    for _,config in ipairs (self.m_detailConfig[countryId]) do
        if soulNum >= config.stage then
            index = index + 1
        end
    end
    return index
end

-- 通过国家和类型获取当前阵魂条件阶段
function This:getStage(countryId,type)
    local stage = 0
    local res=self:getTroopsIdList()
    for _,data in pairs(res[countryId]) do
        -- local config = DeploymentAchievementConfig[id]
        if data.country == countryId and data.type == type then
            stage = stage + 1
        end
    end
    return stage
end

function This:getMaxStage(countryId,type)
    return table.count(self.m_achievementConfig[countryId][type]) or 0
end

function This:getDataByType(countryId,type)
    local stage = self:getStage(countryId,type)
    if stage >= table.count(self.m_achievementConfig[countryId][type]) then
        stage = stage
    else
        stage = stage + 1
    end
    return self.m_achievementConfig[countryId][type][stage] or {}
end

-- 获取当前阵魂条件状态
function This:getConditionState(countryId,type,number1,number2)
    local heros = HeroManager:getAllHero()
    local num = 0
    for id,hero in pairs(heros) do
        if type == 1 then
            if hero:isGot() and hero:getLevel() >= number2 and hero:getCountry() == countryId then
                num = num + 1
            end
        elseif type == 2 then
            if hero:isGot() and hero:getStarLv() >= math.floor(number2/5) and hero:getPhase() >= number2%5  and hero:getCountry() == countryId then
                num = num + 1
            end
        else
            if hero:isGot() and hero:getAwakeLevel() >= number2 and hero:getCountry() == countryId then
                num = num + 1
            end
        end
    end
    return num >= number1,num
end

-- 获取当前阵眼条件状态
function This:getAddtionState(countryId,index)
    --local stage = self:getArrayStage(countryId)
    local soulNum = self:getSoulNum(countryId)
    return  soulNum >= self.m_detailConfig[countryId][index].stage
end

-- 是否有可激活阵法
function This:hasArrayActivation(countryId)
    local redDotStage = false
    if not countryId then    
        for i=1,4 do
            for j =1,3 do
                local data = self:getDataByType(i,j)
                if self:getConditionState(i,j,data.parameter[1],data.parameter[2]) then
                    redDotStage = true
                end
                if self:getStage(i,j) == self:getMaxStage(i,j) then
                    redDotStage = false
                end
            end
        end
    else
        for i=1,3 do
            local data = self:getDataByType(countryId,i)
            if self:getConditionState(countryId,i,data.parameter[1],data.parameter[2]) then
                redDotStage = true
            end
            if self:getStage(countryId,i) == self:getMaxStage(countryId,i) then
                redDotStage = false
            end
        end
    end
    return redDotStage
end

function This:getArrayAttr()
    local attrAdditionList = {
        [CountryArrayTab.kQingLong] = {},
        [CountryArrayTab.kBaiHu] = {},
        [CountryArrayTab.kXuanWu] = {},
        [CountryArrayTab.kZhuQue] = {},
    }
    for countryId,attrData in ipairs(attrAdditionList) do
        --已激活的阵魂条件加成
        local data = self:getTroopsIdList()[countryId]
        for _,config in pairs(data) do
            if table.count(config.arg) == 1 then
                self:addAddtionToTable(attrData,config.arg[1])
            else
                for i=1,table.count(config.arg) do
                    self:addAddtionToTable(attrData,config.arg[i])
                end
            end
        end
        -- 阵眼属性加成
        local level = self:getArrayStage(countryId)
        if level > 0 then
            for i= 1,level do
                self:addAddtionToTable(attrData,self.m_detailConfig[countryId][i].stage_arg[1])
            end
        end
        -- 阵法增幅加成
        local addLevel = self:getAddLevel()
        if addLevel > 0 then
            local addtion = self.m_addConfig[addLevel].resonance_arg
            for _,data in pairs(addtion) do
                if attrData[data[1]] then
                    attrData[data[1]] = attrData[data[1]] + data[2]
                else
                    attrData[data[1]] = data[2]
                end
            end
            -- for i = 1,addLevel do
            --     local addtion = self.m_addConfig[i].resonance_arg
            --     for j=1,table.count(addtion) do   
            --         attrData[addtion[j][1]]=  addtion[j][2]
            --     end
            -- end
        end
    end
    return attrAdditionList
end

function This:addAddtionToTable(list, info)
    if list[info[1]] then
        list[info[1]] = list[info[1]] + info[2]
    else
        list[info[1]] = info[2]
    end
end

function This:getDes(countryId)
    local attrData={}
    local data = self:getTroopsIdList()[countryId]
    for _,config in pairs(data) do
        if table.count(config.arg) == 1 then
            self:addAddtionToTable(attrData,config.arg[1])
        else
            for i=1,table.count(config.arg) do
                self:addAddtionToTable(attrData,config.arg[i])
            end
        end
    end
    return attrData 
end

function This:getMaxDes(countryId)
    local res = {}
    local des = self.m_achievementConfig[countryId]
    for _,config in pairs(des) do
        for _,data in pairs(config) do
            for _,addtion in pairs (data.arg) do
                self:addAddtionToTable(res,addtion)

            end

        end
    end
    return res
end

function This:getSoulItemList(countryId)
    local soulList = {}
    for i = 1,3 do
        local data = self:getDataByType(countryId,i)
        local level = GamePlayer:getLevel()
        if level >= data.display_level then
            table.insert(soulList,data)
        end
    end
    return soulList
end