--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local HeroConfig = require("Metadatas.HeroConfig")
local HeroManager = require("Hero.HeroManager")
local ListLayer = require("UI.ListLayer")
local LegionItemView = require("Hero.LegionItemView")
local StageManager = require("Stage.GameStageManager")
local UpdateLayer = require("UI.UpdateLayer")
local GameStageManager = require("Stage.GameStageManager")

function This:initNode(info)
	self.m_heroId = info.heroId
	self.m_selectHandler = nil
	self.m_pos = info.pos

	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroLegionList.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	local hero = HeroManager:getHero(self.m_heroId)
	self.m_deputy = {}
	local deputy = hero:getDeputyHeros()
	for i , id in ipairs(deputy) do
		self.m_deputy[id] = true
	end

	self:initRecommend()
	self:initDeputyList()
end

function This:setSelectHandler(handler)
	self.m_selectHandler = handler
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
end

function This:initRecommend()
	local legion = HeroConfig[self.m_heroId].legion
	local initX = 0
	for i , heroId in ipairs(legion) do
		local hero = HeroManager:getHero(heroId)
		local node = self:getDeputyNode(hero,true)
		node:setAnchorPoint(0.45,0.45)
		local w , h = node:getContentSize()
		node:setPosition(initX + w / 2,h / 2)
		self.m_csNode.recommend:addChild(node)
		initX = initX + w + 8
	end
end

function This:initDeputyList()
	local primary = HeroManager:getHero(self.m_heroId)
	self.m_heros = HeroManager:getAllHeroGotWithSort(function(hero)
		if not primary:isRecommended(hero:getId()) and primary:isCanAddDeputy(hero:getId()) then
			return true
		end
		return false
	end)

	self.m_listLayer = UpdateLayer:create({
		itemWidth = 115,
		itemHeight = 155,
		itemCount = #self.m_heros,
		column = 5,
		scrollView = self.m_csNode.ScrollView
	})

	self.m_listLayer:setDataList(self.m_heros,function(hero)
		return self:getDeputyNode(hero,false)
	end)
	self.m_listLayer:setCascadeOpacityEnabled(true)
	local w , h = self.m_listLayer:getContentSize()
	self.m_listLayer:setAnchorPoint(0.5,0)
	self.m_csNode.ScrollView:pushBackCustomItem(self.m_listLayer,w,h,0,0)
end

function This:getDeputyNode(hero,isRecommand)
	local node = LegionItemView:create(self.m_heroId,hero:getId())
	local primary = HeroManager:getHero(self.m_heroId)
	if isRecommand then
		node:showFlag()
	end
	node:setCascadeOpacityEnabled(true)
	node:setSelectHandler(function(heroId)
		self.m_selectHeroId = heroId
		local deputy = HeroManager:getHero(heroId)
		if primary:isRecommended(heroId) then
			local result , reason = primary:isCanAddDeputy(heroId)
			if result then
				if self.m_selectHandler then
					self.m_selectHandler(deputy,self.m_pos)
				end
			else
				if reason == "in_stage_embattle" then
					local panel = g_scene:openDialog("UI.ConfirmPanel",{text = Serialize[61340]})
					panel:setOkHandler(function()
						GameStageManager:unEmbattleHero(heroId)
					end)
				else
					g_scene:showTips(Serialize[reason])
				end
			end
		else
			if self.m_selectHandler then
				self.m_selectHandler(deputy,self.m_pos)
			end
		end
	end,isRecommand)
	return node
end

function This:onEmbattleHeros()
	local primary = HeroManager:getHero(self.m_heroId)
	if primary:isRecommended(self.m_selectHeroId) and self.m_selectHandler then
		local hero = HeroManager:getHero(self.m_selectHeroId)
		self.m_selectHandler(hero,self.m_pos)
	end
end

function This:onEnter()
	super.onEnter(self)
	GameStageManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	GameStageManager:removeObserver(self)
end


--endregion
