--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local HeroManager = require("Hero.HeroManager")
local LegionSelectPanel = require("Hero.LegionSelectPanel")
local LegionItemView = require("Hero.LegionItemView")
-- local LegionPlusConfig = require("Metadatas.LegionPlusConfig")
local ArgConfig = require("Metadatas.ArgConfig")

function This:init(heroId)
	self.m_heroId = heroId
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroLegion.csb")
	self:addChild(self.m_csNode)
	self.m_sltPos = 1
	self.m_deputy = {{},{}}
end

function This:onEnter()
	super.onEnter(self)
	self:initNode()
	self:initTouchEvent()
	self:refresh()
	HeroManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	HeroManager:removeObserver(self)
end

function This:initNode()
	local hero = HeroManager:getHero(self.m_heroId)
	local deputy = hero:getDeputyWithPos()

	for i = 1 , #deputy do
		if deputy[i] > 0 then
			self.m_deputy[i].heroId = deputy[i]
			self.m_deputy[i].isLock = false
		else
			local isUnlock = hero:isUnlockAtDeputyPos(i)
			self.m_deputy[i].isLock = not isUnlock
			self.m_deputy[i].heroId = 0
		end
		if self.m_deputy[i].node then
			self.m_deputy[i].node:removeFromParent()
			self.m_deputy[i].node = nil
		end
	end
end

function This:initTouchEvent()
	for i = 1 , 2 do
		self.m_csNode["need"..i]:setTouch({
			onEnded = function()
				if not self.m_deputy[i].isLock then
					local dialog = g_scene:openDialog("Hero.LegionSelectPanel",{heroId = self.m_heroId,pos = i})
					dialog:setSelectHandler(function(hero,pos)
						dialog:close()
						if self.m_deputy[pos].heroId == hero:getId() then
							return 
						end

						self.m_sltPos = pos
						HeroManager:onUpdateDeputyRequest(self.m_heroId,pos,hero:getId())
					end)
				else
					local hero = HeroManager:getHero(self.m_heroId)
					local _ , reason = hero:isUnlockAtDeputyPos(i)
					assert(reason)
					g_scene:handleErrorCode(reason)
				end
			end,
			isScale = false
		})
	end
	self.m_csNode.help:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text=Serialize["hero_legion_help_tips"],node = self.m_csNode.help})
	end)
end

function This:setModel(heroId)
	if self.m_heroId ~= heroId then
		self.m_heroId = heroId
		self:initNode()
		self:refresh()
	end
end

function This:refresh()
	self:refreshDeputy(1)
	self:refreshDeputy(2)
	self:refreshAttr()
end

function This:refreshAttr()
	local hero = HeroManager:getHero(self.m_heroId)
	for i = 1 , #self.m_deputy do
		local deputyId = self.m_deputy[i].heroId
		if deputyId > 0 then
			local deputy = HeroManager:getHero(deputyId)
			local name = deputy:getName()
			if hero:isRecommended(deputyId) then
				name = name .. Serialize[61342]
			end
			-- if hero:isFetterHero(deputyId) then
			-- 	name = name .. Serialize[61343]
			-- end
			self.m_csNode["legion"..i].legion1:setString(name)

			local power = deputy:getOriginalAttr(AttrType.kPower)
			local intelligence = deputy:getOriginalAttr(AttrType.kIntelligence)
			local chief = deputy:getOriginalAttr(AttrType.kChiefValue)
			local dfs = deputy:getOriginalAttr(AttrType.kDfs) 

			-- local deputyStar = deputy:getStarLv()
			-- local commonRatio = LegionPlusConfig[deputyStar].commonadd
			local commonRatio = ArgConfig["subordinate_inherit_rate"].arg[1]
			self.m_csNode["legion"..i].chief:setString(string.format("+%.1f",chief * commonRatio))
			self.m_csNode["legion"..i].power:setString(string.format("+%.1f",power * commonRatio))
			self.m_csNode["legion"..i].Intelligence:setString(string.format("+%.1f",intelligence * commonRatio))
			self.m_csNode["legion"..i].defense:setString(string.format("+%.1f",dfs * commonRatio))
			
			if hero:isRecommended(deputyId) then
				self.m_csNode["legion"..i].recommand:setVisible(true)
				-- local recommonRatio = LegionPlusConfig[deputyStar].recommendadd
				local recommonRatio = ArgConfig["subordinate_recommend_rate"].arg[1]
				self.m_csNode["legion"..i].recommand.chief2:setString(string.format("+%.1f",chief * recommonRatio))
				self.m_csNode["legion"..i].recommand.power2:setString(string.format("+%.1f",power * recommonRatio))
				self.m_csNode["legion"..i].recommand.Intelligence2:setString(string.format("+%.1f",intelligence * recommonRatio))
				self.m_csNode["legion"..i].recommand.defense2:setString(string.format("+%.1f",dfs * recommonRatio))
			else
				self.m_csNode["legion"..i].recommand:setVisible(false)
			end
			self.m_csNode["legion"..i]:setVisible(true)
		else
			self.m_csNode["legion"..i]:setVisible(false)
		end
	end
	self.m_csNode.Tips:setVisible(hero:isDeputy())
end

function This:refreshDeputy(pos)
	if not self.m_deputy[pos].isLock then
		if self.m_deputy[pos].node == nil and self.m_deputy[pos].heroId > 0 then
			local x , y = self.m_csNode["need"..pos]:getPosition()
			local node = LegionItemView:create(self.m_heroId,self.m_deputy[pos].heroId)
			node:setPosition(x,y)
			node:setAnchorPoint(0.458,0.47)
			self.m_csNode:addChild(node)

			local w , h = node:getContentSize()

			node:setCancelHandler(function()
				HeroManager:onUpdateDeputyRequest(self.m_heroId, pos ,0)
			end)
			self.m_deputy[pos].node = node
		elseif self.m_deputy[pos].node and self.m_deputy[pos].heroId > 0 then
			self.m_deputy[pos].node:refreshNode(self.m_deputy[pos].heroId)
		elseif self.m_deputy[pos].node and self.m_deputy[pos].heroId <= 0 then
			self.m_deputy[pos].node:removeFromParent()
			self.m_deputy[pos].node = nil
		end
	end
	self.m_csNode["lock"..pos]:setVisible(self.m_deputy[pos].isLock)
end

-------------------------------------------------------------

function This:onDeputyChanged(heroId)
	if self.m_heroId == heroId then
		local hero = HeroManager:getHero(heroId)
		local deputy = hero:getDeputyWithPos()
		for pos , info in pairs(self.m_deputy) do
			info.heroId = deputy[pos]
		end
		self:refresh()		
	end
end

function This:onUpgradeStarLv(heroId)
	if self.m_heroId == heroId then
		local hero = HeroManager:getHero(heroId)
		for i = 1 , 2 do
			local isUnlock = hero:isUnlockAtDeputyPos(i)
			self.m_deputy[i].isLock = not isUnlock
		end
	end
end

function This:onUpgradeQuality(heroId)
	if self.m_heroId == heroId then
		local hero = HeroManager:getHero(heroId)
		for i = 1 , 2 do
			local isUnlock = hero:isUnlockAtDeputyPos(i)
			self.m_deputy[i].isLock = not isUnlock
		end
	end
end


--endregion
 