---
---
--- Created by aid.
--- DateTime: 2018/10/31 11:29
---

local This , super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local PsvSoltView = require("Hero.PsvSoltView")
local PsvSkillSolt = require("Hero.PsvSkillSolt")
local PsvResetHeroItemView = require("Hero.PsvTransHeroItemView")

local PassiveSoltConfig = require("Metadatas.PassiveSoltConfig")
local UniversalConfig = require("Metadatas.UniversalConfig")
local ResetSkillConfig = require("Metadatas.ResetSkillConfig")

function This:initData(info)
    super.initData(self, info)

    self.heroId = nil
    self.heroUid = nil
    self.canBeReset = false
    self.resetIdList = {}
    self.CardNames = {
        Left = "LeftCardView",
        Right = "RightCardView"
    }
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PassvieReset.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    self:setHero(info.heroId)
end

function This:setHero(heroId)
    self:clearHero()

    self.heroId = heroId

    local heroData = HeroManager:getHero(self.heroId)
    self.heroUid = heroData:getUid()

    local leftCardRoot = self.m_csNode.scard_0
    local rightCardRoot = self.m_csNode.tcard

    -- 左边武将卡片
    local maxHp = heroData:getAttr(AttrType.kMaxHp)
    local leftCardView  = PsvResetHeroItemView:create(heroData:getId(), {hp = maxHp, maxHp = maxHp, hero = nil})
    leftCardView:setName(self.CardNames.Left)
    local w, h = leftCardRoot:getContentSize()
    leftCardView:setAnchorPoint(0.485, 0.41)
    leftCardView:setPosition(w/2,h/2)
    leftCardRoot:addChild(leftCardView)

    -- 右边武将卡片
    local rightCardView = PsvResetHeroItemView:create(heroData:getId(), {hp = maxHp, maxHp = maxHp, hero = nil})
    rightCardView:setName(self.CardNames.Right)
    w, h = rightCardRoot:getContentSize()
    rightCardView:setAnchorPoint(0.485, 0.41)
    rightCardView:setPosition(w/2,h/2)
    rightCardRoot:addChild(rightCardView)

    -- 左边的武将卡片需要显示“红叉”
    leftCardView:setCancelHandler(function(heroId)
        self:clearHero()
    end)

    -- 武将技能槽
    for i = 1, 6 do

        -- 左边被动技能槽
        local leftSolt = heroData:getSoltByPos(i)
        local leftSoltView = PsvSoltView:create(leftSolt)
        leftSoltView:setNoLearn(true)
        leftSoltView:setTouchEnabled(false)
        self.m_csNode["skill"..i]:addChild(leftSoltView)

        -- 右边被动技能槽
        local rightSolt = PsvSkillSolt:create({
            quality = leftSolt.m_quality,
            solt = leftSolt.m_solt,
            hero = leftSolt.m_hero,
        })
        rightSolt:setIsUnlock(leftSolt:isUnlock())
        local skill = leftSolt:getPsvSkill()
        if skill then
            rightSolt:setPsvSkill(skill)
        end
        -- 如果左边的技能槽已解锁但没有配置技能，则对应的右边技能槽要显示为未解锁状态
        if leftSolt:isUnlock() and leftSolt:isEmpty() then
            rightSolt:setIsUnlock(false)
            table.insert(self.resetIdList, leftSolt:getQualit()*100+leftSolt:getIndex())
            self.canBeReset = true
        end

        local rightSoltView = PsvSoltView:create(rightSolt)
        rightSoltView:setNoLearn(true)
        rightSoltView:setTouchEnabled(false)
        self.m_csNode["tskill"..i]:addChild(rightSoltView)
    end
end

function This:clearHero()
    local leftCardRoot = self.m_csNode.scard_0
    local rightCardRoot = self.m_csNode.tcard

    -- 清空武将卡片
    local cardView = leftCardRoot:getChildByName(self.CardNames.Left)
    if cardView then
        cardView:removeFromParent()
    end
    cardView = rightCardRoot:getChildByName(self.CardNames.Right)
    if cardView then
        cardView:removeFromParent()
    end

    -- 清空技能槽
    for i=1, 6 do
        self.m_csNode["skill"..i]:removeAllChildren()
        self.m_csNode["tskill"..i]:removeAllChildren()
    end

    self.resetIdList = {}
    self.canBeReset = false
    self.heroUid = nil
    self.heroId = nil
end

function This:onEnter()
    super.onEnter(self)
    HeroManager:addObserver(self)
end

function This:onExit()
    HeroManager:removeObserver(self)
    super.onExit(self)
end

local function addItems(container, itemId, count)
    if count == 0 then return end
    if container[itemId] then
        container[itemId] = container[itemId]+count
    else
        container[itemId] = count
    end
end

function This:getReturnItems()
    local returnItems = {}
    for _, id in ipairs(self.resetIdList) do

        -- 根据配表计算重置该槽位应返还的物品和物品数量
        local quality = math.floor(id/100)
        for _, config in pairs(UniversalConfig) do
            if config.quality == quality then
                --
                local itemId = config.award_ID
                local count = PassiveSoltConfig[id].need_card*config.exchange_count
                addItems(returnItems, itemId, count)
                --
                for _, item in pairs(PassiveSoltConfig[id].need_item) do
                    addItems(returnItems, item[1], item[2])
                end

                break
            end
        end
    end

    return returnItems
end

function This:getNeedYuanBaoCountWhenReset()
    local needYuanBaoCount = 0 -- 重置需要消耗的元宝数
    local hasResetPassvieSkillTimes = HeroManager:getHasResetPassvieSkillTimes()
    if hasResetPassvieSkillTimes == 0 then
        needYuanBaoCount = 0
    elseif hasResetPassvieSkillTimes > #ResetSkillConfig then
        needYuanBaoCount = ResetSkillConfig[#ResetSkillConfig].price
    else
        needYuanBaoCount = ResetSkillConfig[hasResetPassvieSkillTimes].price
    end
    return needYuanBaoCount
end

function This:initTouchEvent()
    -- 选择武将按钮
    self.m_csNode.scard_0.need:setTouch({
        onEnded = function()
            local panel = self:openSelectPanel()
            panel:setSelectHandler(function(heroId)
                self:setHero(heroId)
            end)
        end,
        isScale = false
    })

    -- 对话框关闭按钮
    if self.m_csNode.close then
        self.m_csNode.close:setTouchEnded(function()
            self:close()
        end)
    end

    -- 右下角帮助按钮
    self.m_csNode.bangzhu:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text=Serialize["hero_psvreset_help_tips"], node = self.m_csNode.bangzhu})
    end)

    -- 重置按钮
    self.m_csNode.querenbutton:setTouchEnded(function()
        if not self.canBeReset then
            g_scene:showTips(Serialize["hero_ResetPassvieSkillError"])
            return
        end

        g_scene:openDialog("Hero.PsvResetConfirmPanel", {
            heroUid = self.heroUid,
            returnItems = self:getReturnItems(),
            needYuanBaoCount = self:getNeedYuanBaoCountWhenReset(),
        })
    end)
end

function This:openSelectPanel()
    local haveSelected = {}
    if self.heroId then
        haveSelected[self.heroId]=true
    end
    print_r(haveSelected)
    return g_scene:openDialog("Stage.StageSelectHeroPanel",{haveSelected = haveSelected})
end

-- 重置成功后的回调
function This:onResetPsvSkillSlot(heroId)
    self:setHero(heroId) -- 重新显示武将信息
end