-- @Author: superm
-- @Date:   2018-06-05 16:37:45
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-20 19:04:43
-- @File Name: PsvTransPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local PsvSoltView = require("Hero.PsvSoltView")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")
local PsvTransHeroItemView = require("Hero.PsvTransHeroItemView")
local RecruitView = require("Recruit.RecruitView")

function This:initNode(info)
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PassiveSkillTransfer.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self.m_sourceInfo = {skillTag = "skill", cardTag = "scard", heroId = info.sourceId or 0}
	self.m_targetInfo = {skillTag = "tskill", cardTag = "tcard", heroId = info.targetId or 0}
	self:refreshHeroNode(self.m_sourceInfo)
	self:refreshHeroNode(self.m_targetInfo)
	self:refreshCostItemNode()
end

function This:refreshCostItemNode()
	local state, costId, costNum = self:hasFreeTime()
	if costId then
		self.m_csNode.yuanbao:setVisible(false)
		self.m_csNode.num:setVisible(false)
		self.m_csNode.yuanbao:setImage(ItemUtil.getIconImage(costId))
		self.m_csNode.num:setStringWithColor( (state and "<A1>" or "<R0>") .. costNum)
	else
		self.m_csNode.yuanbao:setVisible(false)
		self.m_csNode.num:setVisible(false)
	end
end

function This:refreshHeroNode( info, inited, stable )
	local node = uiNode or self.m_csNode
	local stag = info.skillTag
	local ctag = info.cardTag
	local hero = nil
	if info.heroId > 0 then
		hero = stable or HeroManager:getHero(info.heroId)
	end

	if not info.viewList and hero then
		local soltViewList, slotList = {}, {}
		info.viewList = soltViewList
		info.slotList = slotList
		for i = 1, 6 do
			local solt = hero:getSoltByPos(i)
			local soltView = PsvSoltView:create(solt)
			soltView:setNoLearn(true)
			node[stag .. i]:addChild(soltView)
			soltViewList[i] = soltView
			slotList[i] = solt
		end
	elseif info.viewList then
		if not hero then
			for i, view in ipairs(info.viewList) do
				view:setVisible(false)
			end
		else
			for i, view in ipairs(info.viewList) do
				local solt = hero:getSoltByPos(i)
				view:setModel(solt)
				view:setVisible(true)
				info.slotList[i] = solt
			end
		end
	end

	local cardRootNode = node[ctag]
	if not inited then
		cardRootNode.need:setTouch({
			onEnded = function()
				if self.m_subView then
					return
				else
	        		self.m_subView = true
				end

				if info.heroId and info.heroId > 0 then
					return
				end

				local panel = self:openSelectPanel()
				panel:setSelectHandler(function(heroId)
					info.heroId = heroId
					self:refreshHeroNode(info, true)
				end)
		        panel:addCloseCallback(function ()
		            self.m_subView = nil
		        end)
			end,
			isScale = false
		})
	end

	if info.card == nil and hero then
		local maxHp = hero:getAttr(AttrType.kMaxHp)
		local cardNode  = PsvTransHeroItemView:create(hero:getId(), {hp = maxHp, maxHp = maxHp, hero = stable})
		cardRootNode:addChild(cardNode)
		local w, h = cardRootNode:getContentSize()
		cardNode:setAnchorPoint(0.485,0.41)
		cardNode:setPosition(w/2,h/2)
		if not stable then
			cardNode:setCancelHandler(function(heroId)
				info.heroId = 0
				self:refreshHeroNode(info, true)
			end)
		end
		info.card = cardNode
	elseif info.card then
		local cardNode = info.card
		if not hero then
			cardNode:setVisible(false)
		else
			cardNode:setModel(hero)
			cardNode:setVisible(true)
		end
	end
end

function This:refreshHeroSlot( info, slot )
	local hero = nil
	if slot then
		hero = slot:getHero()
	end
	
	if info and hero and info.heroId == hero:getId() then
		if info.viewList then
			local index = slot:getIndex()
			info.viewList[index]:setModel(slot)
			info.viewList[index]:setVisible(true)
		end
	end
end

function This:hasFreeTime()
	local ret = HeroManager:getLeftTransFreeTimes() > 0
    if ret then 
    	return true
    elseif DepotManager:getItemOrResCount(130) > 0 then
    	return true, 130, 1
    else
    	local total = DepotManager:getItemOrResCount(102)
    	local ret = HeroManager:getNextPsvSkillTransCost()
    	return total > ret, 102, ret, total
    end
end

function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
end

function This:onExit()
	HeroManager:removeObserver(self)
	super.onExit(self)
end

function This:initTouchEvent()
	if self.m_csNode.close then
	    self.m_csNode.close:setTouchEnded(function()
	        self:close()
	    end)
	end

    if self.m_csNode.bangzhu then 
	   	self.m_csNode.bangzhu:setTouchEnded(function()
			g_scene:openDialog("UI.HelpTips",{text=Serialize["hero_psvtrans_help_tips"],node = self.m_csNode.bangzhu})
		end)
	end

	self.m_csNode.querenbutton:setTouchEnded(function()
		if self.m_sourceInfo and self.m_targetInfo then
			self:openPreviewDialog()
			return
		elseif self.m_resultInfo then
			local state, costId, costNum = self:hasFreeTime()
			if state and not costId then
                HeroManager:onTransPsvSkillRequest(self.m_resultInfo.sourceId, self.m_resultInfo.heroId)
			elseif costId > 0 then
				g_scene:openDialog("UI.CostConfirmPanel", {
	                cost = costNum,
	                costId = costId,
	                desc = Serialize[61374], 
	                okHandler = function()
                		HeroManager:onTransPsvSkillRequest(self.m_resultInfo.sourceId, self.m_resultInfo.heroId)
	                end 
	            })
			end
			return
		end
	end)
end

function This:openPreviewDialog()
    if self.m_sourceInfo.heroId <= 0 or self.m_targetInfo.heroId <= 0 then
		g_scene:showTips(Serialize[60830])
		return
	end

	local ret, count, indexList = true, 0, {}
	for i = 1, 6 do
		local sslot = self.m_sourceInfo.slotList[i]
		local tslot = self.m_targetInfo.slotList[i]
		if sslot and sslot:isUnlock() and not sslot:isEmpty() then
			count = count + 1
			local isUnlock = tslot:isUnlock()
			if not isUnlock then
				table.insert(indexList, i)
			end
			if ret then
				ret = isUnlock
			end
		end
	end

	if count <= 0 then
		ret = false
		g_scene:showTips(Serialize[61375])
		return
	end

	if not ret then
		g_scene:showTips(Serialize[10426])
		if self.m_targetInfo.viewList and #indexList > 0 then
			for _, index in ipairs(indexList) do
				local view = self.m_targetInfo.viewList[index or 1]
				if view then
					view:showChosenEffect()
				end
			end
		end
		return
	end

	g_scene:openDialog("Hero.PsvTransResultPreviewPanel", {
		sourceId = self.m_sourceInfo.heroId,
		targetId = self.m_targetInfo.heroId,
		slotList = self.m_sourceInfo.slotList
	})
end

function This:openSelectPanel()
	local haveSelected = {}
	if self.m_sourceInfo.heroId > 0 then
		haveSelected[self.m_sourceInfo.heroId]=true
	end
	if self.m_targetInfo.heroId > 0 then
		haveSelected[self.m_targetInfo.heroId]=true
	end
	print_r(haveSelected)
	return g_scene:openDialog("Stage.StageSelectHeroPanel",{haveSelected = haveSelected})
end


function This:onTransPsvSkill()
	self.m_sourceInfo.heroId = 0
	self:refreshHeroNode(self.m_sourceInfo, true)
	local id = self.m_targetInfo.heroId
	self.m_targetInfo.heroId = 0
	self:refreshHeroNode(self.m_targetInfo, true)
    self:refreshCostItemNode()
end

function This:onClearHeroResetTime()
    self:refreshCostItemNode()
end

function This:onUnlockPsvSolt(solt)
	self:refreshHeroSlot(self.m_sourceInfo, solt)
	self:refreshHeroSlot(self.m_targetInfo, solt)
end

function This:onEquipPsvSkill(solt)
	self:refreshHeroSlot(self.m_sourceInfo, solt)
	self:refreshHeroSlot(self.m_targetInfo, solt)
end
