-- @Author: superm
-- @Date:   2018-06-06 14:31:30
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-13 11:01:02
-- @File Name: PsvTransResultPreviewPanel.lua
-- @Description: 

-- PassiveTransferTips

local This , super = class(...,"Hero.PsvTransPanel")
local HeroManager = require("Hero.HeroManager")
local PsvSkillSolt = require("Hero.PsvSkillSolt")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")

function This:initNode(info)
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PassiveTransferTips.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	local tHero = GameUtil:deepCopy(HeroManager:getHero(info.targetId))
	local sHero = HeroManager:getHero(info.sourceId)
	self.m_resultInfo = {skillTag = "skill", cardTag = "tcard", heroId = tHero:getUid(), heroQuality = tHero:getQuality(), sourceId = sHero:getUid()}
    for i, sslot in ipairs(info.slotList) do
    	local tslot = tHero.m_psvSolt[i]
    	if tslot:isUnlock() then
			if sslot:isEmpty() and not tslot:isEmpty() then
				tslot.m_psvSkill = nil
			end
			GameUtil:shallowCopy(sslot, tslot)
			tHero.m_psvSolt[i]:setIsUnlock(true)
			tHero.m_psvSolt[i].m_hero = tHero
		end
    end
	tHero:resetPsvSlot()
	self:refreshHeroNode(self.m_resultInfo, true, tHero)
	self:refreshCostItemNode()
end

function This:onTransPsvSkill()
    self:showPsvTransEffect()
end

function This:onTouchMask()
	if not self.m_isRunning then
		self:close()
	end
end

function This:showPsvTransEffect()
	self.m_isRunning = true
	self.m_csNode.yulantext:setVisible(false)
	self.m_csNode.bg:setVisible(false)
	self.m_csNode.bg2:setVisible(false)
	self.m_csNode.skillbg:setVisible(false)
	self.m_csNode.zhuanyifaguang1:setVisible(false)
	self.m_csNode.zhuanyifaguang2:setVisible(false)
	self.m_csNode.querenbutton:setVisible(false)
	self.m_csNode.querentext:setVisible(false)
	self.m_csNode.tcard:setVisible(false)
	self.m_csNode.yuanbao:setVisible(false)
	self.m_csNode.num:setVisible(false)
	self.m_csNode.mianfei:setVisible(false)
	--self.m_csNode.shuliang:setVisible(false)

	self.m_csNode.skill1:setVisible(false)
	self.m_csNode.skill2:setVisible(false)
	self.m_csNode.skill3:setVisible(false)
	self.m_csNode.skill4:setVisible(false)

	self:runAction(cc.Sequence(
		cc.DelayTime(0.5),
		cc.CallFunc(function()
			if self.m_isRemoving then
				return 
			end

			if not self.m_cardEffect then
				self:stopAllActions()
				self:showCardEffect()
				self:runAction(cc.Sequence(
					cc.DelayTime(1.5),
					cc.CallFunc(function()
						self:showEffectFinish()
					end)
				))
			end
		end)
	))
	cc.Audio:playEffect("Recruit_AppearCard.mp3",false)
end

function This:showCardEffect()
	self.m_cardEffect = cc.SkeletonNode:createEffect("ChouKa01")
	self.m_cardEffect:scheduleUpdate()
	self.m_cardEffect:playAnimation("animation",false)
	self.m_cardEffect:setAnimationListener({
		onAnimationEvent = function(_,animationName,eventName)
			if self.m_resultInfo.heroQuality == Quality.kOrange then
				cc.Audio:playEffect("Recruit_ChengKa.mp3",false)
			else
				cc.Audio:playEffect("Recruit_Common.mp3",false)
			end
		end
	})
	self.m_csNode.tcard:setVisible(false)
	self.m_csNode.tcard:addChild(self.m_cardEffect)
	self.m_cardEffect:setLocalZOrder(10)
	self.m_resultInfo.card:setLocalZOrder(11)
	self:runAppear(self.m_csNode.tcard, 2)
	self:runAppear(self.m_csNode.chenggongtext, 2)
	self.m_csNode.skill1:setVisible(true)
	self.m_csNode.skill2:setVisible(true)
	self.m_csNode.skill3:setVisible(true)
	self.m_csNode.skill4:setVisible(true)
	self.m_csNode.bg:setVisible(true)
	self.m_csNode.bg2:setVisible(true)
end

function This:showEffectFinish(time)
	if self.m_isRemoving then
		return
	end
	self.m_isRemoving = true
	self:stopAllActions()
	time = time or 0.2

	self:runAction(cc.Sequence(cc.DelayTime(time),
		cc.FadeOut(0.8),
		cc.CallFunc(function()
			self:close()
			self.m_isRemoving = nil
		end)
	))
end

function This:runAppear( node, times )
	if node then
	    node:stopAllActions()
	    node:setVisible(true)
	    node:setOpacity(0)
	    node:setScale(0.1)
	    node:runAction(cc.FadeIn(0.2 * times))
	    node:runAction(cc.ScaleTo(0.6 * times, 0.58, "BackOut"))
	end
end

function This:refreshCostItemNode()
	local state, costId, costNum, totalNum = self:hasFreeTime()
	if costId then
		self.m_csNode.yuanbao:setVisible(true)
		self.m_csNode.num:setVisible(true)
		self.m_csNode.mianfei:setVisible(false)
		self.m_csNode.yuanbao:setImage(ItemUtil.getIconImage(costId))
		self.m_csNode.num:setStringWithColor( (state and "<A1>" or "<R0>") .. costNum)
		-- if self.m_csNode.shuliang then
		-- 	self.m_csNode.shuliang:setVisible(totalNum ~= nil)
		-- 	if totalNum then
		-- 		self.m_csNode.shuliang:setStringWithColor(ItemUtil.getName(costId) .. ":" .. (state and "<A1>" or "<R0>") .. costNum .. "<A1>/" .. totalNum)
		-- 	end
		--end
	else
		--self.m_csNode.shuliang:setVisible(false)
		self.m_csNode.yuanbao:setVisible(false)
		self.m_csNode.num:setVisible(false)
		self.m_csNode.mianfei:setVisible(true)
	end
end
