-- @Author: superm
-- @Date:   2018-06-11 21:20:58
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 15:31:46
-- @File Name: SaintBoxRewardPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local RewardItemView, NodeView = class(This._NAME .. ".RewardItemView", "NodeView")
RewardItemView.csfileName = "GameScene/NodeUI/WuShengRewardList.csb"

local ItemNode = require("UI.ItemNode")
local HeroManager = require("Hero.HeroManager")
local GameUtil = require("Util.GameUtil")

function This:initNode(info)
    self.m_itemViews = {}
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/WuShengReward.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    self:onPickupSaintLevReward()
end

function This:onEnter()
    super.onEnter(self)
    self:addCustomListener()
end

function This:onExit()
    self:releaseAllViews()
    self:removeCustomListener()
    super.onExit(self)
end

function This:addCustomListener()
    HeroManager:addObserver(self)
end

function This:removeCustomListener()
    HeroManager:removeObserver(self)
end

function This:releaseAllViews()
    for _, view in pairs(self.m_itemViews) do
        view:release()
    end
    self.m_itemViews={}
end

function This:refreshRewardList()
    self.m_csNode.scrollView:removeAllItems()
    self.m_csNode.scrollView:setTouchEnabled(true)
    self:sortDataList()

	for index, data in ipairs(self.m_itemDataList) do
        self.m_timer:setTimer((index - 1) * 0.03, function()
            local itemId = data.id
            local view = self.m_itemViews[itemId]
            if not view then
                view = self:getNewItemView(data)
                self.m_itemViews[itemId] = view
                view:retain()
            else
                view:refresh()
            end
            local w , h = view:getContentSize()
            self.m_csNode.scrollView:pushNode(view, w, h)
        end)
	end  
end

function This:sortDataList()
    table.sort(self.m_itemDataList, function( a, b )
        if a.state ~= b.state then
            return a.state > b.state
        else
            return a.id < b.id
        end
    end)
end

function This:getNewItemView( ... )
    return RewardItemView:create(self, ...)
end

function This:initTouchEvent()
    self.m_csNode.closebg:setTouchEnded(function()
        if self.m_runningAniantion then
            return false
        end
        self:close()
    end)
end

function This:onPickupSaintLevReward()
    local saintInfo = HeroManager:getSaintInfo()
    self.m_itemDataList = saintInfo:getReachRewardList()
    self:refreshRewardList()
end

-----------------------RewardItemView-------------------------------------------

function RewardItemView:initData(dialog, config)
    self.m_dialog = dialog
    self.m_config = config
    self.m_inited = false
    self:initCsNode()
end

function RewardItemView:initCsNode()
    local csNode = cc.Node:createFromCSFile(self.csfileName)
    self.m_csNode = csNode
    self:addChild(csNode)
    local w , h = csNode.beijing.bg:getContentSize()
    self:setContentSize(w, h)
end

function RewardItemView:initNode()
    self:refreshItemUI()
    self:refresh()
end

function RewardItemView:initTouchEvent()
    if not self.m_inited then
        self.m_csNode.button:setTouch({
            getRect = function (touch)
                local rect = touch.sizeNode:getBoundingBox()
                return rect
            end,
		    onEnded = function()
                self:onPickUpBtnClick()
            end,
            isContainer = true,
            isDynamic = true,
            isScale = false,
        })  
        self.m_inited = true
    end
end

function RewardItemView:refreshItemUI()
    local rewardList = self.m_config.reward
    local count = #rewardList
    local csNode = self.m_csNode
    for i, info in ipairs(rewardList) do
    	local itemId, num = info[1], info[2]
    	local itemNode = csNode["item" .. tostring(i)]
    	if itemNode then
    		itemNode:setVisible(true)
    		local node = itemNode.node
			if not node then
		        node = ItemNode:create(itemId, num)
		        node:setAnchorPoint(0.5, 0.5)
		        itemNode:addChild(node)
		        itemNode.node = node
		    else
		        node:init(itemId, num)
		    end
    	end
    end

    for i=count + 1, 4 do
    	local itemNode = csNode["item" .. tostring(i)]
    	if itemNode then
    		itemNode:setVisible(false)
    	end
    end
end

function RewardItemView:onPickUpBtnClick()
    if self:isOpening() then
        HeroManager:onPickupSaintLevRewardRequest(self.m_config.id)
    end
end

function RewardItemView:refresh()
    local csNode = self.m_csNode
    csNode.beijing.level:setString(self.m_config.level .. Serialize["Warfare_WarfareInfo_17"])
    self:setItemEnable(self:isOpening())
end

function RewardItemView:setItemEnable(enble)
    local csNode = self.m_csNode
    if self:isLocked() then
        csNode.button:setGrayShader()
        csNode.button:setVisible(true)
        
        if csNode.yilingqu then 
            csNode.yilingqu:setVisible(false)
        end
    else
        if enble then
            csNode.button:clearShader()
        else
            csNode.button:setVisible(false)
        end

        if csNode.yilingqu then 
            csNode.yilingqu:setVisible(not enble)
        end
    end
end

function RewardItemView:isOpening()
    return self.m_config and self.m_config.state == SaintBoxState.kOpen
end

function RewardItemView:isLocked()
    return self.m_config and self.m_config.state == SaintBoxState.kLock
end