-- @Author: superm
-- @Date:   2018-06-12 10:30:09
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-08 15:56:53
-- @File Name: SaintInfo.lua
-- @Description: 

local This = class(...)

local SaintHeroConfig = require("Metadatas.SaintHeroConfig")
local SaintLevelConfig = require("Metadatas.SaintLevelConfig")
local SaintBoxConfig = require("Metadatas.SaintBoxConfig")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")

local SaintPowerUpEffectView = require("Hero.SaintPowerUpEffectView")

kSaintStrongGroupNum = 6

SaintBoxState = {
	kLock = 1,
	kOpen = 2,
	kClose = 0
}

SaintPowerAction = {
	kLevel = 1,
	kStar = 2,
	kStrenghtLev = 3,
	kSkill = 4,
	kSoilder = 5,
}

SaintPowerActionName = {
    [SaintPowerAction.kLevel] = Serialize[60793],
    [SaintPowerAction.kStar] = Serialize[61380],
    [SaintPowerAction.kStrenghtLev] = Serialize[60790],
    [SaintPowerAction.kSkill] = Serialize[61381],
    [SaintPowerAction.kSoilder] = Serialize[61382],
}

function This:create(...)
	local ret = {}
	setmetatable(ret, self)
	ret:init(...)
	return ret
end

function This:init()
    self.m_saintLev = 0
    self.m_saintPower = 0
    self.m_saintStrongPower = 0

	self.m_saintRewardList = {}
	self.m_rewardedIdList = {}
	-- self.m_totalPower = {}

	for index, config in ipairs(SaintBoxConfig) do
		self.m_saintRewardList[index] = {
			id = config.id, 
			state = SaintBoxState.kLock, 
			reward = config.reward, 
			level = config.level
		}
	end

	self.m_saintTargetLev = nil

	self.m_saintHeroNum = nil
	self.m_saintHeroMaxLevel = nil
	self.m_saintHeroRecordLevel = nil
	self.m_saintHeroList = nil
end

function This:updateInfo( info, newest )
	if newest then
		self.m_saintHeroNum = nil
		self.m_saintHeroMaxLevel = nil
		self.m_saintHeroRecordLevel = nil
		self.m_saintHeroList = nil
	end
	--print_r(info)
	local oldLevel = self.m_saintLev or 0
	if info.saint_level and info.saint_level ~= oldLevel then
	    self.m_saintLev = info.saint_level
    	HeroManager:updateSaintAttr()
	end

	if info.reward_list or info.saint_level then
		if info.reward_list then
			self.m_rewardedIdList = {}
		    for _, id in ipairs(info.reward_list) do
		     	self.m_rewardedIdList[id] = true
		    end
		end

	    local target
	    for _, box in ipairs(self.m_saintRewardList) do
	    	if self.m_rewardedIdList[box.id] then
	    		box.state = SaintBoxState.kClose
	    	elseif self.m_saintLev >= box.level then
	    		box.state = SaintBoxState.kOpen
	    	else
	    		box.state = SaintBoxState.kLock
	    		if not target or target > box.level then
	    			target = box.level
	    		end
	    	end
	    end

    	self.m_saintTargetLev = target
	end

	-- local oldvalue = self.m_saintPower or 0
	-- local oldStrongValue = self.m_saintStrongPower or 0
	-- if info.all_fight_power then
	-- 	self.m_saintPower = info.all_fight_power
	-- end
	-- if info.strongest_fight_power then
	-- 	self.m_saintStrongPower = info.strongest_fight_power
	-- end

	-- if info.all_fight_power and oldvalue ~= self.m_saintPower then
	-- 	print(string.format("武圣战力：%d => %d", oldvalue, self.m_saintPower))
	-- 	HeroManager:notifyObserver("onSaintHeroInfoChange" , oldvalue, self.m_saintPower)
	-- end

	-- if info.strongest_fight_power and oldStrongValue ~= self.m_saintStrongPower then
	-- 	print(string.format("最高战力：%d => %d", oldStrongValue, self.m_saintStrongPower))
	-- 	SaintPowerUpEffectView:showStrongPower(oldStrongValue, self.m_saintStrongPower)
	-- 	HeroManager:notifyObserver("onSaintStrongPowerChange" , oldStrongValue, self.m_saintStrongPower)
	-- end
 	self:calcSaintFightPower()
end

function This:getSaintProgress()
	local totalPower = SaintLevelConfig[self.m_saintLev].militarypower
	-- local totalPower = self.m_totalPower[self.m_saintLev]
	-- if not totalPower then
	-- 	totalPower = 0
	--     for _, config in ipairs(SaintLevelConfig) do
	--     	if config.level <= self.m_saintLev then
	--     		totalPower = totalPower + config.militarypower
	--     	end
	--     end
	--     self.m_totalPower[self.m_saintLev] = totalPower
	-- end

    local config = SaintLevelConfig[self.m_saintLev + 1]
    if config then
    	local levelPower = config.militarypower - totalPower
    	-- local levelPower = config.militarypower
    	local value = self.m_saintPower - totalPower
		print(value, levelPower, self.m_saintPower, totalPower)
    	if value < 0 then
    		value = 0
		elseif value > levelPower then
    		value = levelPower
    	end 
    	return value, levelPower
    else
    	config = SaintLevelConfig[self.m_saintLev]
    	local levelPower = config.militarypower - totalPower
    	-- local levelPower = config.militarypower
    	return levelPower, levelPower
    end
end

function This:getSaintAttrAdditionList(level)
	local attList = {AttrType.kPower, AttrType.kIntelligence, AttrType.kChiefValue, AttrType.kDfs}
	local tmp = {0,0,0,0}
    for _, config in ipairs(SaintLevelConfig) do
    	if config.level <= level then
    		for _, info in ipairs(config.reward) do
    			local attrId = info[1] or 0 
    			if attrId > 0 then
		    		local value = tmp[attrId]
		    		tmp[attrId] = value + info[2]
		    	end
	    	end
    	end
    end

	local ret = {}
    for i, attr in ipairs(attList) do
    	ret[attr] = tmp[i]
    end
    return ret
end

function This:getTargetLevel()
    return self.m_saintTargetLev
end

function This:getSaintPower()
	return self.m_saintPower
end

function This:getSaintLevel()
	return self.m_saintLev
end

function This:getReachRewardList()
	return GameUtil:shallowCopy(self.m_saintRewardList)
end

function This:getRewardListById( id )
    local rewardItem = self.m_saintRewardList[id] 
    if rewardItem and rewardItem.state == SaintBoxState.kOpen then
    	return rewardItem.reward
   	end
end

function This:isFullSaintLevel()
	local level = self:getTargetLevel()
	return level == nil
end

function This:canUpgradeSaintLev()
    local config = SaintLevelConfig[self.m_saintLev + 1]
    if config then
    	-- return self.m_saintPower >= self.m_totalPower[self.m_saintLev] + config.militarypower
    	return self.m_saintPower >= config.militarypower
    else
    	return false
    end
end

function This:hasSanitReward()
	return table.find(self.m_saintRewardList, function (index, value)
		return value.state ~= SaintBoxState.kLock
	end)
end

function This:hasSanitRewardOpen()
	return table.find(self.m_saintRewardList, function (index, value)
		return value.state == SaintBoxState.kOpen
	end)
end

function This:calcSaintFightPower(withoutCalc)
	--print(not withoutCalc and "计算圣武战力..." or "请求圣武战力...")
	if withoutCalc then
		HeroManager:onGetSaintHeroInfoRequest()
		return
	end
	--print("=============================================计算圣武战力 BEGIN")

    local level = GamePlayer:getLevel()
	local maxLevel = self.m_saintHeroMaxLevel
	--print(self.m_saintHeroRecordLevel, level, maxLevel)
	if self.m_saintHeroRecordLevel ~= level and (maxLevel and level > maxLevel) then
		--print("重新计算圣武战力")
		self.m_saintHeroNum = nil
		self.m_saintHeroMaxLevel = nil
		self.m_saintHeroRecordLevel = nil
		maxLevel = nil
	end

	local num = self.m_saintHeroNum
	if not num then 
		for i, config in ipairs(SaintHeroConfig) do
			if config.min_level <= level and config.max_level >= level then
				num = config.hero_num
				maxLevel = config.max_level
				break
			end
		end
	    
	    if not num then
	    	num = SaintHeroConfig[#SaintHeroConfig].hero_num
	    end
	    self.m_saintHeroNum = num
	    self.m_saintHeroMaxLevel = maxLevel
	    self.m_saintHeroRecordLevel = level
		--print("重新圣武战力武将个数：" .. self.m_saintHeroNum)
	end

	local heroList = self.m_saintHeroList
    if not heroList then
    	heroList = GameUtil:shallowChange(HeroManager.m_heros)
		self.m_saintHeroList = heroList
	end

	table.sort(heroList, function(a,b)
		local avalue, bvalue = a:getFightPower() , b:getFightPower()
		if avalue ~= bvalue then
			return avalue > bvalue
		end
		return a:getId() < b:getId()
	end)

	local retPower, retStrongPower, count = 0, 0, 0
	local breakNum = math.max(kSaintStrongGroupNum, num)
	for _, hero in ipairs(heroList) do
		if hero:isGot() and not hero:isDeputy() then
			local heroPower =  hero:getFightPower()
			if count < kSaintStrongGroupNum then
				retStrongPower = retStrongPower + heroPower
			end
			if count < num then
				retPower = retPower + heroPower
			end
			--print(hero:getName() .. ":" .. heroPower)
			count = count + 1
			if count >= breakNum then
				break
			end
		end
	end

	if retPower ~= self.m_saintPower then
		local oldvalue = self.m_saintPower
		self.m_saintPower = retPower
		--print(string.format("calc 武圣战力：%d => %d", oldvalue, self.m_saintPower))
        HeroManager:notifyObserver("onSaintHeroInfoChange" , oldvalue, self.m_saintPower)
    end

    if retStrongPower ~= self.m_saintStrongPower then
		local oldvalue = self.m_saintStrongPower
		self.m_saintStrongPower = retStrongPower
		--print(string.format("calc 最高战力：%d => %d", oldvalue, self.m_saintStrongPower))
		SaintPowerUpEffectView:showStrongPower(oldvalue, self.m_saintStrongPower)
        HeroManager:notifyObserver("onSaintStrongPowerChange" , oldvalue, self.m_saintStrongPower)
    end
	--print("=============================================计算圣武战力 END")
	return retPower, retStrongPower 
end

function This:getStrongestPower()
	return self.m_saintStrongPower
end

function This:getSaintHeroNum()
	local level = GamePlayer:getLevel()
	local num = self.m_saintHeroNum
	if not num then 
		for i, config in ipairs(SaintHeroConfig) do
			if config.min_level <= level and config.max_level >= level then
				num = config.hero_num
				maxLevel = config.max_level
				break
			end
		end
	    
	    if not num then
	    	num = SaintHeroConfig[#SaintHeroConfig].hero_num
	    end
	    self.m_saintHeroNum = num
	    self.m_saintHeroMaxLevel = maxLevel
	    self.m_saintHeroRecordLevel = level
	end
    return self.m_saintHeroNum
end

function This:getStrongestGroup()
	local heroList = self.m_saintHeroList
    if not heroList then
    	heroList = GameUtil:shallowChange(HeroManager.m_heros)
		self.m_saintHeroList = heroList
	end

	table.sort(heroList, function(a,b)
		local avalue, bvalue = a:getFightPower() , b:getFightPower()
		if avalue ~= bvalue then
			return avalue > bvalue
		end
		return a:getId() < b:getId()
	end)

	local retPower, retHeroList, count = 0, {}, 0
	for _, hero in ipairs(heroList) do
		if hero:isGot() and not hero:isDeputy() then
			count = count + 1
			table.insert(retHeroList, hero)
			retPower = retPower + hero:getFightPower()
			print(hero:getName(), hero:getFightPower())
			if count >= kSaintStrongGroupNum then
				break
			end
		end
	end

	table.sort(retHeroList, function(a,b)
		local avalue, bvalue = a:getFightPower() , b:getFightPower()
		if avalue ~= bvalue then
			return avalue > bvalue
		end

		avalue, bvalue = a:getStarLv() , b:getStarLv()
		if avalue ~= bvalue then
			return avalue > bvalue
		end

		avalue, bvalue = a:getQuality() , b:getQuality()
		if avalue ~= bvalue then
			return avalue > bvalue
		end

		return a:getId() < b:getId()
	end)
	return retPower, retHeroList
end
