-- @Author: superm
-- @Date:   2018-06-12 13:33:22
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 15:33:58
-- @File Name: SaintStrongInfoPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local EquipManager = require("Equip.EquipManager")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")
local ArenaHeroView = require("Arena.ArenaHeroView")
local GameUtil = require("Util.GameUtil")

local kAttenuationCoefficient1 = 0.8
local kAttenuationCoefficient2 = 1

function This:initNode(info)
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ZuiQiangZhanLiPromote.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
local scale = self.m_csNode.tips:getScale()
	self.m_csNode.tips:runAction(cc.RepeatForever(cc.Sequence(cc.ScaleTo(1,1.2* scale),cc.ScaleTo(1,scale))))
	self.m_cardNodes = {}
	self.m_progressList = {}
	self.m_progressIndex = {}
	self.m_saintInfo = HeroManager:getSaintInfo()
	self:refresh()
end

function This:getEquipTotalLev(hero)
	local strengthLev = 0
	for i = EquipPos.kWeapons , EquipPos.kHorse do
		local equipUid = hero:getEquipUid(i)
		if equipUid > 0 then
			local equip = EquipManager:getEquip(equipUid)
			if equip then
				strengthLev = strengthLev + equip:getStrengthLv()
			end
		end
	end
	return strengthLev
end

function This:getMaxPhaseSoilder(hero)
	local retPhase, max = 0, 0
    local soilderList = {SoilderType.kShield,SoilderType.kCavalry,SoilderType.kArcher,SoilderType.kSapperCar}
    for i , soilderId in ipairs(soilderList) do
		local soilder = hero:getSoilder(soilderId)
	    local phase = soilder:isUnlock() and soilder:getPhase() or 0
    	if phase > retPhase then
    		retPhase = phase
			max = soilder:getCouldUpgradeMaxPhase()
		end
    end
    -- print(retPhase, max)
    return retPhase, max
end

function This:getHeroPsvTotalQulity(hero)
	local retQuality, maxQuality = 0, 0
	for i = 1, 4 do
		local slot = hero:getSoltByPos(i)
		if slot:isUnlock() then
			maxQuality = maxQuality + 1
			if not slot:isEmpty() then
				local skill = slot:getPsvSkill()
			    if skill then 
			    	if skill:isHighLevel() then
				    	retQuality = retQuality + kAttenuationCoefficient2
				    else
				    	retQuality = retQuality + kAttenuationCoefficient1
				    end
			    end
			end
		elseif slot:isCanUnlock() then
			maxQuality = maxQuality + 1
		end
	end
	return retQuality, maxQuality
end

function This:getProgress(action, heroList, info)
	local progressInfo = info[action]
	if not progressInfo then
		progressInfo = {valueList={}, total=0}
		info[action] = progressInfo
	end

	if action == SaintPowerAction.kLevel then
		local level = 0
		for i, hero in ipairs(heroList) do
			local heroLev = hero:getLevel()
			local heroId = hero:getId()
			progressInfo.valueList[heroId] = { heroId= heroId, value = heroLev}
			level = heroLev + level
		end
		progressInfo.total = level
		return level / (GamePlayer:getLevel() * 6) * 100
	elseif action == SaintPowerAction.kStar then
		local level = 0
		for i, hero in ipairs(heroList) do
			local heroLev = hero:getTotalPhase()
			local heroId = hero:getId()
			progressInfo.valueList[heroId] = { heroId= heroId, value = heroLev}
			level = heroLev + level
		end
		progressInfo.total = level
		return level / (25 * 6) * 100
	elseif action == SaintPowerAction.kStrenghtLev then
		local level = 0
		for i, hero in ipairs(heroList) do
			local heroLev = self:getEquipTotalLev(hero)
			local heroId = hero:getId()
			progressInfo.valueList[heroId] = { heroId= heroId, value = heroLev}
			level = heroLev + level
		end
		progressInfo.total = level
		return level / (30 * 6) * 100
	elseif action == SaintPowerAction.kSkill then
		local level, total = 0, 0
		for i, hero in ipairs(heroList) do
			local quality, max = self:getHeroPsvTotalQulity(hero)
			local heroId = hero:getId()
			progressInfo.valueList[heroId] = { heroId= heroId, value = quality}
			level = quality + level
			total = total + max
		end
		progressInfo.total = level
		if total <= 0 then
			return 0
		end
		return level / total * 100
	elseif action == SaintPowerAction.kSoilder then
		local level, total = 0, 0
		for i, hero in ipairs(heroList) do
			local phase, max = self:getMaxPhaseSoilder(hero)
			local heroId = hero:getId()
			progressInfo.valueList[heroId] = { heroId= heroId, value = phase}
			level = phase + level
			total = total + max
		end
		progressInfo.total = level
		if total <= 0 then
			return 0
		end
		return level / total * 100
	end
	return 0
end

function This:refresh()
	local power, cardList = self.m_saintInfo:getStrongestGroup()
    self.m_csNode.numberzhanli:setString(power)
	GameUtil:setPlayerFeltNodeResource(self.m_csNode.renwutu, GamePlayer:getIcon())
	self.m_csNode.name:setString(GamePlayer:getRoleName())
	self.m_fpowerList = {}
	for i = 1 , 6  do
		local hero, info = cardList[i], nil
		if hero then
			info = {id=i, level=hero:getLevel(), star=hero:getTotalPhase(), heroId=hero:getId()}
			self.m_fpowerList[hero:getId()] = hero:getFightPower()
		end
		if not self.m_cardNodes[i] then
			local view = ArenaHeroView:create(info)
			self.m_csNode["hero" .. i]:addChild(view)
			self.m_cardNodes[i] = view
		else
			self.m_cardNodes[i]:setModel(info)
		end
	end
	self.m_progressList = {}
	for _, index in pairs(SaintPowerAction) do
		local node = self.m_csNode["cond" .. index]
        local progress = self:getProgress(index, cardList, self.m_progressList)
        self.m_progressIndex[index] = progress
        if progress > 100 then
        	progress = 100
        end
        node.jindu:setPercent(progress)
        node.progress:setString( string.format("%.2f", progress) .. "%")
    end
end

function This:initTouchEvent()
	if not self.m_inited then
		self.m_inited = true
	    self.m_csNode.closebg:setTouchEnded(function()
	        if self.m_runningAniantion then
	            return false
	        end
	        self:close()
	    end)

		for _, index in pairs(SaintPowerAction) do
		    self.m_csNode["cond" .. index].button:setTouchEnded(function()
		    	local progress = self.m_progressIndex[index]
		    	if progress >= 100 or not self.m_progressList[index] then
		    		self:showTipsByAction(index)
		    		return
		    	end
		    	local dataList = GameUtil:shallowChange(self.m_progressList[index].valueList)
		    	table.sort(dataList, function (a, b)
		    		if a.value ~= b.value then
		    			return a.value < b.value
		    		end
		    		local apower = self.m_fpowerList[a.heroId] 
		    		local bpower = self.m_fpowerList[b.heroId] 
		    		if apower ~= bpower then
		    			return apower > bpower
		    		end
		    		return a.heroId < b.heroId
		    	end)
		    	local heroId = 	dataList[1].heroId
		    	self:openDialogByAction(index, heroId)
		    end)
		end
	end
end

function This:showTipsByAction( action )
	local tips = ""
	if action == SaintPowerAction.kLevel then 
		tips = Serialize[61117]
	elseif action == SaintPowerAction.kStar then
		tips = Serialize[61118]
	elseif action == SaintPowerAction.kStrenghtLev then
		tips = Serialize[61119]
	elseif action == SaintPowerAction.kSkill then
		tips = Serialize[61120]
	elseif action == SaintPowerAction.kSoilder then
		tips = Serialize[61121]
	end
	g_scene:showTips(tips)
end

function This:openDialogByAction( action, heroId )
	if action == SaintPowerAction.kLevel then 
		g_scene:openDialog("Hero.HeroPanel", {heroId = heroId, tab="equip"})
	elseif action == SaintPowerAction.kStar then
		g_scene:openDialog("Hero.HeroPanel", {heroId = heroId, tab="equip"})
	elseif action == SaintPowerAction.kStrenghtLev then
		g_scene:openDialog("Hero.HeroPanel", {heroId = heroId, tab="equip"})
		-- g_scene:openDialog("Equip.EquipStrengthPanel",{heroId = heroId})
	elseif action == SaintPowerAction.kSkill then
		g_scene:openDialog("Hero.HeroPanel", {heroId = heroId, tab="skill"})
	elseif action == SaintPowerAction.kSoilder then
		g_scene:openDialog("Hero.HeroPanel", {heroId = heroId, tab="armType"})
	end
end

function This:onEnter()
    super.onEnter(self)
	HeroManager:addObserver(self)
end

function This:onExit()
	HeroManager:removeObserver(self)
    super.onExit(self)
end

function This:onSaintStrongPowerChange(oldvalue, strongPower)
    self:refresh()
end