--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local ArmConfig = require("Metadatas.ArmConfig")
local StarUpConfig = require("Metadatas.StarUpConfig")
local HeroManager = require("Hero.HeroManager")
local TerrBuildManager = require("Territory.TerrBuildManager")
local DepotManager = require("Depot.DepotManager")

function This:init(info)
	super.init(self)
	self.m_id = info.id
	self.m_heroId = 0
	self.m_attrValue = {}
	self.m_addValueForHero = {}
	self.m_attrPoint = {}
	self.m_buildAddAttr = {}
	self:setPhase(info.phase or 1)
	self.m_awake = nil
end

function This:getId()
	return self.m_id
end

function This:setPhase(phase)
	self.m_phase = phase
	local key = self.m_id * 1000 + self.m_phase
	self.m_config = ArmConfig[key]
	self:updateAttr()
end

function This:getAwake()
    return self.m_awake
end

function This:setAwake(value)
    self.m_awake = value
end

function This:getConfig()
	return self.m_config
end

function This:getPhase()
	return self.m_phase
end

function This:getHeroId()
	return self.m_heroId
end

function This:isUnlock()
	local building = TerrBuildManager:getBuildBy(self.m_config.build_id)
	local level = building:getLevel()
	return level >= ArmConfig[self.m_id * 1000 + 1].barracks_level
end

function This:isFull()
	return self.m_phase >= kSoilderMaxPhase and self:isToMaxAttrPoint()
end

function This:getAssociatedBuild()
	return TerrBuildManager:getBuildBy(self.m_config.build_id)
end

function This:getName()
	return self.m_config.arms_name
end

function This:getDesc()
	return self.m_config.arms_des
end

function This:getRecruitCost()
	if self.m_awake and self.m_awake:hasAwake()then
		return self.m_awake:getRecruitCost()
	end
	return self.m_config.arms_cost
end

function This:getModel()
	return self.m_config.arms_model
end

function This:getTrainCost()
	local itemList = {}
	for i , item in ipairs(self.m_config.train_cost) do
		table.insert(itemList,{id = item[1], num = item[2]})
	end
	return itemList
end

function This:getMultiTrainTime()
	local total = 0
	for i , value in pairs(self.m_attrPoint) do
		total = total + value
	end
	local max = 0
	for i , value in ipairs(self.m_config.arms_progress) do
		max = max + value
	end
	
	if total >= max then
		return 1
	end

	local ret = math.min(kSoliderMultiTrainTime, max - total)
	for i , item in ipairs(self.m_config.train_cost) do
		local time = math.modf(DepotManager:getItemOrResCount(item[1]) / item[2])
		ret = math.min(ret,  time)
	end
	
	if ret < 1 then
		return 1
	end

	return ret
end

function This:getUpPhaseCost()
	local itemList = {}
	for i , item in ipairs(self.m_config.progress_cost) do
		table.insert(itemList,{id = item[1], num = item[2]})
	end
	return itemList
end

function This:getUnlockBarrackLevel()
	local config = ArmConfig[self.m_id * 1000 + 1]
	return config.barracks_level
end

function This:getUpPhaseBarrackLevel()
	if self.m_phase < kSoilderMaxPhase then
		local config = ArmConfig[self.m_id * 1000 + self.m_phase + 1]
		return config.barracks_level
	end
	return self.m_config.barracks_level
end

function This:getCouldUpgradeMaxPhase()
	local build = self:getAssociatedBuild()
	local buildLevel = build:getLevel()
	local ret = self.m_phase
    for i=self.m_phase+1, kSoilderMaxPhase do
		local config = ArmConfig[self.m_id * 1000 + i]
    	if config and config.barracks_level <= buildLevel then
    		ret = i
    	end
    end
    return ret
end

function This:getTotalTrainCost()
	local cost = {}
	local preSum = 0

	for i = 1, self.m_phase do
		local key = self.m_id * 1000 + i
		local point , sum = 0 , 0 

		if i == self.m_phase then
			for id , num in pairs(self.m_attrPoint) do
				sum = sum + num
			end
		else
			for _ , num in ipairs(ArmConfig[key].arms_progress) do
				sum = sum + num
			end
		end
		point = sum - preSum
		preSum = sum

		for _, info in ipairs(ArmConfig[key].train_cost) do
			cost[info[1]] = cost[info[1]] or 0
			cost[info[1]] = cost[info[1]] + info[2] * point
		end
		if i < self.m_phase then
			for _, info in ipairs(ArmConfig[key].progress_cost) do
				cost[info[1]] = cost[info[1]] or 0
				cost[info[1]] = cost[info[1]] + info[2]
			end
		end
	end
	return cost
end

function This:setAttrPoint(pointList)
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		for attrId , value in pairs(self.m_attrPoint) do
			if value > 0 then
				hero:reduceAttr(attrId, value)
			end
		end
	end
	self.m_attrPoint[AttrType.kPower] = pointList[1] or 0
	self.m_attrPoint[AttrType.kIntelligence] = pointList[2] or 0
	self.m_attrPoint[AttrType.kChiefValue] = pointList[3] or 0
	self.m_attrPoint[AttrType.kDfs] = pointList[4] or 0
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		for attrId , value in pairs(self.m_attrPoint) do
			if value > 0 then
				hero:addAttr(attrId, value)
			end
		end
	end
end

function This:isToMaxAttrPoint()
	local total = 0
	for i , value in pairs(self.m_attrPoint) do
		total = total + value
	end
	local max = 0
	for i , value in ipairs(self.m_config.arms_progress) do
		max = max + value
	end
	return total >= max
end

function This:getAttr(attrId)
	local a1 = self.m_buildAddAttr[attrId] or 0
	local a2 = self.m_attrPoint[attrId] or 0
	return a1 + a2
end

function This:getAttrPoint(attrId)
	if attrId then
		return self.m_attrPoint[attrId] or 0
	else
		return self.m_attrPoint
	end
end

function This:getAttrFeatures()
	return self.m_config.arms_change
end

function This:getNextPhaseAttrFeatures()
	if self.m_phase < kSoilderMaxPhase then
		local config = ArmConfig[self.m_id * 1000 + self.m_phase + 1]
		return config.arms_change
	end
	return {0,0,0}
end

function This:getAttrValue(attrId)
	if attrId then
		return self.m_attrValue[attrId]
	else
		return self.m_attrValue
	end
end

function This:updateAttr()
	local attr = {}
	for i , featureId in ipairs(self.m_config.arms_change) do
		local config = StarUpConfig[featureId]
		if config then
			local attrId = config.attri_id
			if attr[attrId] then
				attr[attrId] = attr[attrId] + config.value + config.value2
			else
				attr[attrId] = config.value + config.value2
			end
		end
	end 

	local building = TerrBuildManager:getBuildBy(self.m_config.build_id)
	local attrs = {}

	if building:getLevel() >= self:getUnlockBarrackLevel() then
		attrs = building:getBuildAtrrNum()
	end

	local oldBuildAttrs = self.m_buildAddAttr
	self.m_buildAddAttr = attrs

	self.m_attrValue = attr
	if self.m_heroId > 0 then
		local hero = HeroManager:getHero(self.m_heroId)
		for attrId , value in pairs(self.m_addValueForHero) do
			hero:reduceAttr(attrId,value)
		end

		self.m_addValueForHero = {}
		for attrId , value in pairs(attr) do
			hero:addAttr(attrId,value)
			self.m_addValueForHero[attrId] = value
		end

		for attrId , value in pairs(oldBuildAttrs) do
			hero:reduceAttr(attrId,value)
		end

		for attrId , value in pairs(self.m_buildAddAttr) do
			hero:addAttr(attrId,value)
		end
		hero:updateFightPower()

		-- local primaryId = hero:getPrimaryHero()
		-- if primaryId > 0 then
		-- 	local primary = HeroManager:getHero(primaryId)
		-- 	primary:updateDeputy(hero)
		-- end
	end
end

function This:attachHero(hero)
	if hero:getId() ~= self.m_heroId then
		self:disattachHero()
	end
	self.m_heroId = hero:getId()
	for attrId , value in pairs(self.m_attrValue) do
		hero:addAttr(attrId,value)
		self.m_addValueForHero[attrId] = value
	end
	for attrId , value in pairs(self.m_attrPoint) do
		hero:addAttr(attrId,value)
	end
	for attrId , value in pairs(self.m_buildAddAttr) do
		hero:addAttr(attrId,value)
	end
end

function This:disattachHero()
	if self.m_heroId <= 0 then
		return
	end
	local hero = HeroManager:getHero(self.m_heroId)
	for attrId , value in pairs(self.m_addValueForHero) do
		hero:reduceAttr(attrId,value)
	end
	for attrId , value in pairs(self.m_attrPoint) do
		hero:reduceAttr(attrId,value)
	end
	for attrId , value in pairs(self.m_buildAddAttr) do
		hero:reduceAttr(attrId,value)
	end
	self.m_heroId = 0
end

function This:reset()
	self.m_phase = 1
	self.m_heroId = 0
	self.m_buildAddAttr = {}
	self.m_addValueForHero = {}
	self.m_attrPoint = {}
end



--endregion
