--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrBuildConfig = require("Metadatas.TerrBuildConfig")
local HeroManager = require("Hero.HeroManager")
local Serialize = require("Serialize.Serialize")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")

function This:initNode(info)
	self.m_hero = info.hero
	self.m_soilder = info.soilder
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroArmsCulture.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self:initBaseInfo()
	self:refresh()
end

function This:initTouchEvent()
	self.m_csNode.cost.culture:setTouchEnded(function()
		local building = self.m_soilder:getAssociatedBuild()
		local level = building:getLevel()
		if level <= 0 then
			g_scene:showTips(Serialize[61400])
			return
		end

		local costList = self.m_soilder:getTrainCost()
		if GamePlayer:checkGold({cost = costList[1].num}) then
			if DepotManager:getItemCount(costList[2].id) < costList[2].num then
				g_scene:showTips(string.format(Serialize[60334],ItemUtil.getName(costList[2].id)))
				return
			end
			HeroManager:onTrainSoilderRequest(self.m_hero:getUid(),self.m_soilder:getId())
		end
	end)

	self.m_csNode.cost.culture_0:setTouchEnded(function()
		local building = self.m_soilder:getAssociatedBuild()
		local level = building:getLevel()
		if level <= 0 then
			g_scene:showTips(Serialize[61400])
			return
		end

		local costList = self.m_soilder:getTrainCost()
		local time = self.m_soilder:getMultiTrainTime()

		for _, itemInfo in ipairs(costList) do
			local result, errorCode = DepotManager:isItemOrResEnough(itemInfo.id, itemInfo.num * time)
			if not result then
				if errorCode then
					g_scene:handleErrorCode(errorCode)
				else
					g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(costList[2].id)))
				end
				return
			end
		end

		HeroManager:onMultiTrainSoilderRequest(self.m_hero:getUid(),self.m_soilder:getId(), time)
	end)

	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	g_scene:handleHelpTips(self.m_csNode.help,function()
		return {text = Serialize["soiler_train_help_tips"], node = self.m_csNode.name}
	end)
end

function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
	GamePlayer:addObserver(self)
end

function This:onExit()
	GamePlayer:removeObserver(self)
	HeroManager:removeObserver(self)
	super.onExit(self)
end

function This:initBaseInfo()
	self.m_csNode.name:setString(self.m_soilder:getName())
	self.m_csNode.icon:setImage(SoilderIcon[self.m_soilder:getId()])
	self.m_csNode.introduce:setString(self.m_soilder:getDesc())
	local itemCost = self.m_soilder:getTrainCost()
	self.m_csNode.cost.icon1:setImage(ItemUtil.getIconImage(itemCost[1].id))
	self.m_csNode.cost.cost1:setString(itemCost[1].num)
	self.m_csNode.cost.icon2:setImage(ItemUtil.getIconImage(itemCost[2].id))
	self.m_csNode.cost.cost2:setString(itemCost[2].num)
end

function This:refresh()
	local attrList = {AttrType.kPower, AttrType.kIntelligence,AttrType.kChiefValue,AttrType.kDfs}
	local total , maxPoint = 0, 0
	local config = self.m_soilder:getConfig()
	for i = 1 , 4 do
		local value = self.m_soilder:getAttrPoint(attrList[i])
		self.m_csNode.Arr["num"..i]:setString(string.format("%d/%d",value,config.arms_progress[i]))
		self.m_csNode.Arr["jindutiao"..i]:setPercent(value / config.arms_progress[i] * 100)
		total = total + value
		maxPoint = maxPoint + config.arms_progress[i]
	end
	self.m_csNode.cost:setVisible(not (self.m_soilder:getPhase() >= kSoilderMaxPhase and total >= maxPoint))
	self.m_csNode.full:setVisible(self.m_soilder:getPhase() >= kSoilderMaxPhase and total >= maxPoint)

	local itemCost = self.m_soilder:getTrainCost()
	if GamePlayer:getGold() < itemCost[1].num then
		self.m_csNode.cost.cost1:setColor(255,0,0)
	else
		self.m_csNode.cost.cost1:setColor(255,255,255)
	end

	if DepotManager:getItemCount(itemCost[2].id) < itemCost[2].num then
		self.m_csNode.cost.cost2:setColor(255,0,0)
	else
		self.m_csNode.cost.cost2:setColor(255,255,255)
	end
end

function This:onTrainSoilder(heroId,soilderId)
	if self.m_hero:getId() == heroId and self.m_soilder:getId() == soilderId then
		if self.m_soilder:isToMaxAttrPoint() then
			self:close()
		else
			self:refresh()
		end
		g_scene:showTips(Serialize[61401])
	end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.gold and self.m_soilder then
		local itemCost = self.m_soilder:getTrainCost()
		if value < itemCost[1].num then
			self.m_csNode.cost.cost1:setColor(255,0,0)
		else
			self.m_csNode.cost.cost1:setColor(255,255,255)
		end 	
    end
end

--endregion
