--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local ArmConfig = require("Metadatas.ArmConfig")
local StarUpConfig = require("Metadatas.StarUpConfig")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")

function This:initNode(info)
	self.m_hero = info.hero
	self.m_soilder = info.soilder
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroArmsAdvanced.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self:initBaseInfo()
	self:refresh()
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	g_scene:handleHelpTips(self.m_csNode.help,function()
		return {text = Serialize["soilder_up_phase_help_tips"]}
	end)
	self.m_csNode.cost.culture:setTouchEnded(function()
		local building = self.m_soilder:getAssociatedBuild()
		if building:getLevel() < self.m_soilder:getUpPhaseBarrackLevel() then
			g_scene:showTips(Serialize[61403])
			return
		end
		
		local costList = self.m_soilder:getUpPhaseCost()
		if GamePlayer:checkGold({cost = costList[1].num}) then
			if DepotManager:getItemCount(costList[2].id) < costList[2].num then
				g_scene:showTips(string.format(Serialize[60334],ItemUtil.getName(costList[2].id)))
				return
			end
			HeroManager:onUpPhaseSoilderRequest(self.m_hero:getUid(),self.m_soilder:getId())
		end
	end)

	for i = 1 , 3 do
		g_scene:handleHelpTips(self.m_csNode.Advanced["tep"..i].icon,function()
			local featureList = self.m_soilder:getAttrFeatures()
			local featureId = featureList[i]
			if featureId > 0 then
				local info = {}
				info.text = string.format("%s:\n%s",StarUpConfig[featureId].name,StarUpConfig[featureId].desc)
				return info
			end
		end) 
	end

	for i = 4 , 6 do
		g_scene:handleHelpTips(self.m_csNode.Advanced["tep"..i].icon,function()
			local featureList = self.m_soilder:getNextPhaseAttrFeatures()
			local featureId = featureList[i-3]
			if featureId > 0 then
				local info = {}
				info.text = string.format("%s:\n%s",StarUpConfig[featureId].name,StarUpConfig[featureId].desc)
				return info
			end
		end) 
	end
end

function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
	GamePlayer:addObserver(self)
end

function This:onExit()
	GamePlayer:removeObserver(self)
	HeroManager:removeObserver(self)
	super.onExit(self)
end

function This:initBaseInfo()
	self.m_csNode.name:setString(self.m_soilder:getName())
	self.m_csNode.icon:setImage(SoilderIcon[self.m_soilder:getId()])
	self.m_csNode.introduce:setString(self.m_soilder:getDesc())
	local items = self.m_soilder:getUpPhaseCost()
	self.m_csNode.cost.icon1:setImage(ItemUtil.getIconImage(items[1].id))
	self.m_csNode.cost.cost1:setString(items[1].num)
	self.m_csNode.cost.icon2:setImage(ItemUtil.getIconImage(items[2].id))
	self.m_csNode.cost.cost2:setString(items[2].num)
	if self.m_soilder:getPhase() < kSoilderMaxPhase then
		local str = string.format(Serialize[61404],SoilderTypeName[self.m_soilder:getId()],self.m_soilder:getUpPhaseBarrackLevel())
		self.m_csNode.Advanced.tixing:setString(str)
	end
	local building = self.m_soilder:getAssociatedBuild()
	self.m_csNode.Advanced.tixing:setVisible(building:getLevel() < self.m_soilder:getUpPhaseBarrackLevel())
	self.m_csNode.cost:setVisible(building:getLevel() >= self.m_soilder:getUpPhaseBarrackLevel() and self.m_soilder:getPhase() < kSoilderMaxPhase)
end

function This:refresh()
	local attrList = {AttrType.kPower,AttrType.kDfs, AttrType.kIntelligence,AttrType.kChiefValue}
	for i = 1, 4 do
		local value = self.m_soilder:getAttrPoint(attrList[i])
		self.m_csNode.Advanced["value"..i]:setString(value)
	end
	
	local key = self.m_soilder:getId() * 1000 + self.m_soilder:getPhase()
	local curConfig = ArmConfig[key]
	self.m_csNode.Advanced.name1:setString(curConfig.arms_name,10)
	for i , featureId in ipairs(curConfig.arms_change) do
		local node = self.m_csNode.Advanced["tep"..i]
		if featureId > 0 then
			local icon = GameUtil:getSkillIcon(StarUpConfig[featureId].icon)
			node.icon:setImage(icon)
			node.level:setString(string.format("LV:%d",StarUpConfig[featureId].level))
		else
			node:setVisible(false)
		end
	end
	self.m_csNode.Advanced.price1:setString(curConfig.arms_cost)

	local nexConfig = ArmConfig[key + 1]
	self.m_csNode.Advanced.name2:setString(nexConfig.arms_name,10)
	for i , featureId in ipairs(nexConfig.arms_change) do
		local node = self.m_csNode.Advanced["tep"..(i+3)]
		if featureId > 0 then
			local icon = GameUtil:getSkillIcon(StarUpConfig[featureId].icon)
			node.icon:setImage(icon)
			node.level:setString(string.format("LV:%d",StarUpConfig[featureId].level))
			node.new:setVisible(curConfig.arms_change[i] <= 0 and featureId > 0)
		else
			node:setVisible(false)
		end
	end
	self.m_csNode.Advanced.price2:setString(nexConfig.arms_cost)
	
	local itemCost = self.m_soilder:getUpPhaseCost()
	if GamePlayer:getGold() < itemCost[1].num then
		self.m_csNode.cost.cost1:setColor(255,0,0)
	else
		self.m_csNode.cost.cost1:setColor(255,255,255)
	end

	if DepotManager:getItemCount(itemCost[2].id) < itemCost[2].num then
		self.m_csNode.cost.cost2:setColor(255,0,0)
	else
		self.m_csNode.cost.cost2:setColor(255,255,255)
	end
end

function This:onUpPhaseSoilder(heroId,soilderId)
	if self.m_hero:getId() == heroId and self.m_soilder:getId() == soilderId then
		self:close()
		g_scene:showTips(Serialize[61405])
	end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.gold and self.m_soilder then
		local itemCost = self.m_soilder:getTrainCost()
		if value < itemCost[1].num then
			self.m_csNode.cost.cost1:setColor(255,0,0)
		else
			self.m_csNode.cost.cost1:setColor(255,255,255)
		end 	
    end
end

--endregion
