

local This = class(...)
local HeroManager = require("Hero.HeroManager")
local RefinementSkillAttributeConfig = require("Metadatas.RefinementSkillAttributeConfig")
local HeroConfig = require("Metadatas.HeroConfig")
local RefinemenNeedtItemConfig = require("Metadatas.RefinemenNeedtItemConfig")
local HeroRefineSkill = require("HeroRefine.HeroRefineSkill")
local HeroRefineSlot = require("HeroRefine.HeroRefineSlot")


function This:create(hero)
	local o = {}
	setmetatable(o,self)
	o:init(hero)
	return o
end

function This:init(hero)
	self.m_hero = hero
	self.m_heroId = self.m_hero:getId()
	self.m_refineLv = 0
	self.m_maxRefineLv = HeroManager:getMaxRefineLv()
	self.m_slots = {}
	self.m_skillList = {}

	self:initSlot()
	self:initSkill()
end

function This:initSlot()
	self.m_slots = {}

	for id=1, 4 do
		local slot = HeroRefineSlot:create({id = id, hero = self.m_hero})
		slot:setRefineLv(self.m_refineLv)
		self.m_slots[id] = slot
	end
end

function This:getSlot(id)
	return self.m_slots[id]
end

function This:initSkill()
	self.m_skillList = {}

	local list = self:getConfigSkillList()
	local country = self.m_hero:getCountry()

	for i, id in ipairs(list) do
		local skill = HeroRefineSkill:create({
			skillId = id,
			country = country,
			refineLv = self.m_refineLv,
			index = i,
		})
		self.m_skillList[i] = skill
	end
end

function This:getSkillList()
	return self.m_skillList
end

function This:getConfigSkillList()
	local heroId = self.m_hero:getId()
	local key = heroId * 100 + self.m_refineLv
	local config = RefinementSkillAttributeConfig[key]

	if config then
		return config.skill
	end

	return {}
end

function This:getRefineLv()
	return self.m_refineLv
end

function This:getMaxRefineLv()
	return self.m_maxRefineLv
end

function This:isMaxRefineLv()
	return self.m_refineLv >= self.m_maxRefineLv
end

--加给【对应武将】的属性
function This:getTotalItemAttr()
	local ret = {}

	for k, slot in pairs(self.m_slots) do
		self:mergeAttr(ret, slot:getTotalAttr())
	end

	return ret
end

--按照加成类型分类的属性
function This:getAddTypeSkillAttr()
	local ret = {}

	for k, skill in pairs(self.m_skillList) do
		local addType = skill:getSkillAttrAddType()
		if not ret[addType] then
			ret[addType] = {}
		end
		self:mergeAttr(ret[addType], skill:getAttr())
	end

	return ret
end

function This:mergeAttr(mainTb, subTb)
	for k, v in pairs(subTb) do
		if not mainTb[k] then
			mainTb[k] = 0
		end
		mainTb[k] = mainTb[k] + v
	end
	return mainTb
end

function This:isCanUpgrade()
	if self:isMaxRefineLv() then
		return false
	end

	for _, slot in pairs(self.m_slots) do
		if not slot:isWearFull() then
			return false
		end
	end
	return true
end

function This:isCanWear()
	if self:isMaxRefineLv() then
		return false, Serialize[61414]
	end

	-- 策划提示需求，倒着来检查
	local maxNum = #self.m_slots
	local tips = ""

	for id = maxNum, 1, -1 do
		local slot = self.m_slots[id]
		if not slot:isWearFull() then
			local ret, str = slot:isCanWear()
			if ret then
				return true, ""
			else
				tips = str
			end
		end
	end

	return false, tips
end

function This:updateInfo(info)
	if not info then
		return
	end
	--print_tmp(info)
	local level = info.level
	self.m_refineLv = level

	--装备位
	local itemList = {}
	for i, v in ipairs(info.item_list) do
		itemList[v.pos] = v
	end

	for i, slot in ipairs(self.m_slots) do
		local item = itemList[i]
		slot:setRefineLv(level)
		if item then
			slot:setItemNum(item.cur_num)
		else
			slot:setItemNum(0)
		end
	end

	--技能
	self:initSkill()

	--属性
	local gAttr = HeroManager:getGlobalRefineAttr()
	gAttr:updateHeroRefineSkillAttr(self.m_heroId, self:getAddTypeSkillAttr())
end

function This:getRebornReturn(cost)
	-- print_tmp(cost)
	cost = cost or {}

	for lv = 0, self.m_refineLv do
		local config = RefinemenNeedtItemConfig[lv]
		for i=1, 4 do
			local slot = self:getSlot(i)
			local item = config[string.format("need_item%d", i)]
			local id = item[1]
			local num = item[2]

			if lv == self.m_refineLv then
				num = slot:getItemNum()
			end

			if id and num then
				if not cost[id] then
					cost[id] = 0
				end
				cost[id] = cost[id] + num
			end
		end
	end

	return cost 
end

function This:isRedDot()
	if self:isMaxRefineLv() then
		return false
	end

	local wearFull = 0
	
	for _, slot in pairs(self.m_slots) do
		if slot:isCanWear() then
			return true --可穿戴
		end

		if slot:isCanSlotMerge() then
			return true --可合成
		end

		if slot:isWearFull() then
			wearFull = wearFull + 1
		end
	end

	--可升级
	if wearFull >= #self.m_slots then
		return true
	end

	return false
end

function This:getAddSkillAttr()
	local ret = {}
	local gAttr = HeroManager:getGlobalRefineAttr()

	-- 全体的
	self:mergeAttr(ret, gAttr:getTotalAttr())

	-- 国家的
	local country = self.m_hero:getCountry()
	self:mergeAttr(ret, gAttr:getCountryAttr(country))

	-- 个人的
	local addTypeAttr = self:getAddTypeSkillAttr()	
	self:mergeAttr(ret, addTypeAttr[HeroRefineSkillAddType.Single] or {})
	
	return ret
end

function This:getTotalAttr()
	local ret = {}

	self:mergeAttr(ret, self:getTotalItemAttr())
	self:mergeAttr(ret, self:getAddSkillAttr())
	
	return ret
end