

local This , super = class(...,"NodeUI")
local ViewUtil = require("UI.ViewUtil")
local StageEliteConfig = require("Metadatas.StageEliteConfig")
local GameStageManager = require("Stage.GameStageManager")
local PopDialogManager = require("UI.PopDialogManager")
local MenuScene = require("Home.MenuScene")


function This:initData(data)
    self.m_data = data
end

function This:initNode(data)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTrainItemFrom.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local w, h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)

    self.m_csNode.button:setTouch({
        isDynamic = true, 
        isContainer = true,
        onEnded = function ()
            local stage = GameStageManager:getStage(self.m_data.stageId, GameStageManagerStageType.kMliteStage)
            if stage:isLock() then
                g_scene:showTips( stage:getLockStr() )
            else
                local wayUI = self.m_data.wayUI
                local slot = wayUI:getSlot()
                local heroId = wayUI:getHeroId()
                local itemId = wayUI:getItemId()

                local from = {
                    fromType = "HeroRefine",
                    recoverUI = function ()
                        local arg = {
                                heroId = heroId,
                                tab = "refine",
                                openDialog = {name = "HeroRefine.HeroRefineMergePanel", arg = slot}
                        }
                        MenuScene:addDelayDialog({name = "Hero.HeroPanel", arg = arg})
                    end
                }
                local view = g_scene:openDialog("Stage.StageConfirmPanel", stage)
                view:setFrom(from)
                GameStageManager:setStageType(GameStageManagerStageType.kMliteStage)
            end
        end,
    })
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:setModel(data)
    self.m_data = data
    self:refresh()
end

function This:refresh()
    local data = self.m_data
    local config = StageEliteConfig[data.stageId]
    local csNode = self.m_csNode

    csNode.where:setString(string.format(Serialize[61420], config.chapter_id, config.name))
    csNode.num:setString(data.dropNum)
end
