

local This, super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")
local ItemConfig = require("Metadatas.ItemConfig")
local RefinementItemAttributeConfig = require("Metadatas.RefinementItemAttributeConfig")
local MaterialSourceConfig = require("Metadatas.MaterialSourceConfig")
local StageEliteConfig = require("Metadatas.StageEliteConfig")
local DepotManager = require("Depot.DepotManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local HeroRefineGetWayItemView = require("HeroRefine.HeroRefineGetWayItemView")


function This:initData(info)
	self.m_itemId = info.itemId
    self.m_slot = info.slot
	self.m_waylist = {}
	self:initWayList()
end

function This:initWayList()
	local config = MaterialSourceConfig[self.m_itemId]
	self.m_waylist = {}

	for i, v in ipairs(config.source) do
		local t = {}
		t.stageId = v[1]
		t.dropNum = v[2]
        t.wayUI = self
		table.insert(self.m_waylist, t)
	end
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTrainItemGet.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

    -- 背景触摸拦截
    csNode.bg:setTouch({
        onEnded = function()
            print("bg click")
        end,
        isScale = false,
        isAlpha = false,
    })

    local scrollView = csNode.scrollView
	scrollView:setDirection("vertical")

	self.m_listLayer = RankPageListLayer:create({
        itemWidth = 460,
        itemHeight = 100,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
		dataCounter = function ()	 
	        return #self.m_waylist
        end,
        viewCreator =  function(index, node) 
        	local data = self.m_waylist[index]
        	if data then     
			    if not node then
                    node = HeroRefineGetWayItemView:create(data, index)
				    return node
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setDataList()

    self:refreshItemInfo()
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
end

function This:onExit()
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:refreshItemInfo()
	local csNode = self.m_csNode
	local config = ItemConfig[self.m_itemId]

	-- 图标
	if not self.m_itemNode then
	    local itemNode = ItemNode:create(self.m_itemId)
	    itemNode:setAnchorPoint(0.5, 0.5)
	   	itemNode:setScale(0.8)
	   	itemNode:setPosition(csNode.item:getPosition())
	    csNode:addChild(itemNode)
	    self.m_itemNode = itemNode
	end

    -- 名字
    csNode.lbName:setString(config.name)
    csNode.lbName:setColorByID("Q".. config.quality)

    -- 数量
    local count = DepotManager:getItemCount(self.m_itemId)
    csNode.lbNum:setString( string.format(Serialize[61422], count) )
    
    -- 描述
    csNode.lbDesc:setString(config.comment)
end

function This:onTouchMask()
    self:close()
end

function This:getSlot()
    return self.m_slot
end

function This:getHeroId()
    local hero = self:getSlot():getHero()
    return hero:getId()
end

function This:getItemId()
    return self.m_itemId
end

function This:itemUpdate(itemList)
    self:setTimer(1.0, function ()
        self:refreshItemInfo()
    end, "refine_way_item_update")
end
