

local This, super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")
local ItemConfig = require("Metadatas.ItemConfig")
local RefinementItemAttributeConfig = require("Metadatas.RefinementItemAttributeConfig")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")
local MaterialSourceConfig = require("Metadatas.MaterialSourceConfig")

--兑换提示
This.IsExchangeTips = true


function This:initData(slot)
	self.m_slot = slot
	self.m_itemNode = nil
	self.m_mergeNode = nil
	self.m_needNodes = {}
end

function This:initNode(slot)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTrainSynthesis.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

    -- 关闭
    csNode.close:setTouchEnded(function ()
        self:close()
    end)

    -- 合成
    csNode.hecheng:setTouchEnded(function ()
        self:onMergeClick()
    end)

	self:refresh()
end

function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
	DepotManager:addObserver(self)
end

function This:onExit()
	HeroManager:removeObserver(self)
	DepotManager:removeObserver(self)
	super.onExit(self)
end

function This:refresh()
	self:refreshItemInfo()
	self:refreshMergeInfo()
end

function This:refreshItemInfo()
	local csNode = self.m_csNode
	local itemId = self.m_slot:getConfigItem().id
	local config = ItemConfig[itemId]

	-- 图标
	if not self.m_itemNode then
	    local node = ItemNode:create(itemId)
	    node:setAnchorPoint(0.5, 0.5)
	   	node:setScale(0.8)
	   	node:setPosition(csNode.item:getPosition())
	    csNode:addChild(node)
	    self.m_itemNode = node
	end

    -- 名字
    csNode.lbName:setString(config.name)
    csNode.lbName:setColorByID("Q".. config.quality)

    -- 数量
    local count = DepotManager:getItemCount(itemId)
    csNode.lbNum:setString( string.format(Serialize[61422], count) )

    -- 属性
    csNode.lbAttr:setString( self:getItemAttrDesc() )
    
    -- 描述
    csNode.lbDesc:setString(config.comment) 

    -- 穿戴等级
    local wearLv = self.m_slot:getWearLv()
    local heroLv = self.m_slot:getHero():getLevel() 
    csNode.lbNeed:setString(string.format(Serialize[61424], wearLv))
    if heroLv >= wearLv then
    	csNode.lbNeed:setColor(144, 152, 166)
    else
    	csNode.lbNeed:setColor(255, 0, 0)
    end

end

function This:getItemAttrDesc()
	local itemId = self.m_slot:getConfigItem().id
	local attribute = RefinementItemAttributeConfig[itemId].attribute
	local desc = ""
	local count = #attribute

	for i, attr in ipairs(attribute) do
		local attrId = attr[1]
		local attrVal = attr[2]

		if AttrInfo[attrId].valueType == ValueType.kPercent then
			desc = desc .. string.format("%s +%.2f%%", AttrName[attrId], attrVal*100)
		else
			desc = desc .. string.format("%s +%d", AttrName[attrId], math.floor(attrVal))
		end

		if i < count then
			desc = desc .. "\n"
		end
	end

	return desc
end

function This:onWearClick()
	local isCan, tips = self.m_slot:isCanWear()
	if isCan then
		local hero = self.m_slot:getHero()
		local item = self.m_slot:getConfigItem()
		HeroManager:onRefineWearRequest(hero:getId(), item.id, self.m_slot:getId())
		self:close()
	else
		if tips then
			g_scene:showTips(tips)
		end
	end
end

function This:onMergeClick()
	local isCan, fragment = self.m_slot:isCanMerge()
	if not isCan then
		g_scene:showTips(Serialize[61425])
		return
	end

	if type(fragment) == "table" and This.IsExchangeTips then
		g_scene:openDialog("HeroRefine.HeroRefineExchangePanel", {slot = self.m_slot, fragment = fragment})
	else
		local itemId = self.m_slot:getConfigItem().id
		HeroManager:onRefineMergeRequest(itemId)
	end
end

function This:refreshMergeInfo()
	local csNode = self.m_csNode
	local itemId = self.m_slot:getConfigItem().id
	local config = ItemConfig[itemId]

	csNode.title1:setString(config.name)
    csNode.title1:setColorByID("Q".. config.quality)

	if not self.m_mergeNode then
		local node = ItemNode:create(itemId)
	    node:setAnchorPoint(0.5, 0.5)
	   	node:setScale(0.8)
	   	node:setPosition(csNode.shenwen:getPosition())
	    csNode:addChild(node)
	    self.m_mergeNode = node
	end

	--消耗列表约定格式 {金钱，道具，道具}
	local needList = self.m_slot:getMergeNeedItemList()
	local money = needList[1]
	local moneyNum = DepotManager:getItemOrResCount(money[1])
	csNode.yuanbao:setImage( ItemUtil.getIconImage(money[1]) )
	csNode.yuanbaonum:setString(money[2])
	if moneyNum >= money[2] then
    	csNode.yuanbaonum:setColor(255, 239, 209)
    else
    	csNode.yuanbaonum:setColor(255, 0, 0)
    end


	for i=2, 3 do
		local needId = needList[i][1]
		local needNum = needList[i][2]
		local posNode = csNode[string.format("item%d", i)]

		if posNode then
			local node = self.m_needNodes[i]
			if not node then
				node = ItemNode:create(needId, needNum, true)
				node:setAnchorPoint(0.5, 0.5)
				node:setScale(0.8)
				node:setPosition(posNode:getPosition())
				node:showNeedNum(needNum)
	    		csNode:addChild(node)
	    		self.m_needNodes[i] = node
	    		self:setCostTouch(node)
			else
				node:initItem(needId, needNum, true)
				node:showNeedNum(needNum)
			end
		end
	end
end

function This:setCostTouch(node)
	node.m_tipsView = nil

	function node:openTipsView()
		if self.m_tipsView then
			return
		end
		local info = {node = self, itemId = self.m_id, itemType = self.m_itemType}
		self.m_tipsView = g_scene:openDialog("UI.ItemTips", info)
	end

	function node:removeTipsView()
		if self.m_tipsView and self.m_tipsView[0] ~= 0 then
            self.m_tipsView:close()
            self.m_tipsView = nil
        end
	end

	-- 有配置获取路径的物品点击时显示获取路径界面，没有就通用显示
	node:setTouch({
		onBegan = function(_, touch)
            local rect = node:getBoundingBox()
            if rect:isContainsPoint(touch) then
            	local config = MaterialSourceConfig[node.m_id]
            	if not config or #config.source == 0 then
            		node:openTipsView()
				end
                return true
            end
            return false
        end,
        onEnded = function()
        	node:removeTipsView()
	        local config = MaterialSourceConfig[node.m_id]
	        if config and #config.source > 0 then
	            g_scene:openDialog("HeroRefine.HeroRefineGetWayPanel", {itemId = node.m_id, slot = self.m_slot})
			end
        end,
        onCanceled = function()
            node:removeTipsView()
        end,
        isScale = false,
        isDynamic = true,
		isSwallow = false,
	})
end

function This:onRefineMergeReply(itemId)
	if self.m_slot:isCanWear() then
		g_scene:openDialog("HeroRefine.HeroRefineWearPanel", self.m_slot)
		self:close()
	end
end

function This:itemUpdate(itemList)
	self:setTimer(0.02, function ()
		self:refresh()
	end, "refine_merge_item_update")
end