

-- 武将精进装备槽

local This , super = class(..., "Base.BaseModel")
local RefinemenNeedtItemConfig = require("Metadatas.RefinemenNeedtItemConfig")
local RefinementItemAttributeConfig = require("Metadatas.RefinementItemAttributeConfig")
local MaterialSourceConfig = require("Metadatas.MaterialSourceConfig")
local DepotManager = require("Depot.DepotManager")
local HeroManager = require("Hero.HeroManager")


function This:init(info)
	super.init(self)

	self.m_id = info.id --槽位id
	self.m_hero = info.hero
	self.m_refineLv = 0
	self.m_maxRefineLv = HeroManager:getMaxRefineLv()
	self.m_curNum = 0
end

function This:getId()
	return self.m_id
end

function This:getHero()
	return self.m_hero
end

function This:setRefineLv(lv)
	self.m_refineLv = lv
end

function This:getRefineLv()
	return self.m_refineLv
end

function This:isMaxRefineLv()
	return self.m_refineLv >= self.m_maxRefineLv
end

function This:setItemNum(num)
	self.m_curNum = num
end

function This:getItemNum()
	return self.m_curNum
end

function This:getConfigItem(lv)
	lv = lv or self.m_refineLv

	local ret = {id = 0, num = 0}
	local needCfg = RefinemenNeedtItemConfig[lv]
	
	if needCfg then
		local item = needCfg[string.format("need_item%d", self.m_id)]
		ret.id = item[1] or 0
		ret.num = item[2] or 0
	end

	return ret
end

--num没有传，默认读取配置的数量
function This:getItemAttr(refineLv, num)
	local ret = {}
	local item = self:getConfigItem(refineLv)
	num = num or item.num

	if item.id > 0 and num > 0 then
		local attr = RefinementItemAttributeConfig[item.id].attribute
		for i, v in ipairs(attr) do
			local id = v[1]
			if not ret[id] then
				ret[id] = 0
			end
			ret[id] = ret[id] + v[2]
		end

		for k, v in pairs(ret) do
			ret[k] = v*num
		end
	end

	return ret
end

function This:getTotalAttr()
	local ret = {}

	for lv = 0, self.m_refineLv do
		local attr = {}

		if lv == self.m_refineLv then
			attr = self:getItemAttr(lv, self.m_curNum)
		else
			attr = self:getItemAttr(lv)
		end

		self:mergeAttr(ret, attr)
	end

	return ret
end

function This:mergeAttr(mainTb, subTb)
	for k, v in pairs(subTb) do
		if not mainTb[k] then
			mainTb[k] = 0
		end
		mainTb[k] = mainTb[k] + v
	end
	return mainTb
end

--是否可穿戴
function This:isCanWear()
	if self:isMaxRefineLv() then
		return false, Serialize[61440]
	end

	local item = self:getConfigItem()
	if self.m_curNum >= item.num then
		return false, Serialize[61441]
	end

	local heroLv = self.m_hero:getLevel() 
	if heroLv < self:getWearLv() then
		return false, Serialize[61442]
	end

	local count = DepotManager:getItemCount(item.id)
	if count <= 0 then
		return false, Serialize[61425]
	end

	return true, ""
end

--是否可合成
function This:isCanMerge()
	local list = self:getMergeNeedItemList()
	if #list == 0 then
		return false
	end

	local count, needId, needNum, fragment = 0, 0, 0, false
	for i, item in ipairs(list) do
		needId = item[1]
		needNum = item[2]
		count = DepotManager:getItemOrResCount(needId)
		if count < needNum then
			fragment = self:getExchangeFragment(needId, needNum - count)
			if fragment.needNum <= 0 or fragment.curNum < fragment.needNum then
				return false
			end
		end
	end

	return true, fragment
end

--是否槽位可合成
function This:isCanSlotMerge()
	if self:isMaxRefineLv() then
		return false
	end

	local item = self:getConfigItem()
	if self.m_curNum >= item.num then
		return false
	end

	return self:isCanMerge()
end

function This:getMergeNeedItemList()
	local ret = {}
	local item = self:getConfigItem()
	local config = RefinementItemAttributeConfig[item.id]

	if config then
		ret = config.need_item
	end

	return ret
end

function This:getWearLv()
	local lv = 0
	local item = self:getConfigItem()
	local config = RefinementItemAttributeConfig[item.id]

	if config then
		lv = config.wear_level
	end

	return lv
end

--是否穿戴满
function This:isWearFull()
	local item = self:getConfigItem()
	if item.id <= 0 then
		return false
	end

	return self.m_curNum >= item.num
end

function This:getCommonFragmentId()
	return 297 --万能碎片
end

-- 万能碎片兑换
function This:getExchangeFragment(itemId, itemNum)
	local fId = self:getCommonFragmentId()
	local fragment = {id = fId, needNum = 0, curNum = 0, itemId = itemId}
	local config = MaterialSourceConfig[itemId]

	if config then
		fragment.needNum = config.need_fragment_num*itemNum
		fragment.curNum = DepotManager:getItemOrResCount(fId)
	end

	return fragment
end

