

local This , super = class(...,"cc.Node")
local HeroManager = require("Hero.HeroManager")
local HeroRefineSlotNode = require("HeroRefine.HeroRefineSlotNode")
local HeroRefineSkillNode = require("HeroRefine.HeroRefineSkillNode")
local DepotManager = require("Depot.DepotManager")


function This:init(heroId)
	self.m_heroId = heroId or 0
	self.m_slots = {}
	self.m_skills = {}
	self:initNode()
end

function This:initNode()
	local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTrain.csb")
	self:addChild(csNode)
	self.m_csNode = csNode

	self.m_slots = {}
	for i=1, 4 do
		local x, y = csNode[string.format("shenwen%d", i)]:getPosition()
		local node = HeroRefineSlotNode:create()
		node:setPosition(x, y)
		csNode:addChild(node)
		self.m_slots[i] = node
	end

	local scrollview = csNode.scrollview
	scrollview:setDirection("horizontal")
	scrollview.m_direction = "right"
	self.m_skills = {}
	for i=1, 4 do
		local node = HeroRefineSkillNode:create()
		local w, h = node:getContentSize()
		node:setAnchorPoint(0.5, 0.5)
		scrollview:pushBackCustomItem(node, w + 10, h, 0.5, 10)
		self.m_skills[i] = node
	end

	self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then

    	-- 一键装备/进阶
		self.m_csNode.button:setTouchEnded(function ()
	        self:onButtonClick()
	    end)

    end
end

function This:onEnter()
	super.onEnter(self)
	self:initTouchEvent()
	HeroManager:addObserver(self)
	DepotManager:addObserver(self)
end

function This:onExit()
	HeroManager:removeObserver(self)
	DepotManager:removeObserver(self)
	super.onExit(self)
end

function This:setModel(heroId)
	if self.m_heroId ~= heroId then
		self.m_heroId = heroId
		self:refresh()
	end
end

function This:refresh()
	self:refreshSlot()
	self:refreshSkill()
end

function This:refreshSlot()
	local hero = HeroManager:getHero(self.m_heroId)
	local refine = hero:getRefine()
	local isMaxLv = refine:isMaxRefineLv()
	local csNode = self.m_csNode

	csNode.lbLevel:setString( string.format(Serialize[61445], refine:getRefineLv()) )

	for i=1, 4 do
		local slot = refine:getSlot(i)
		local node = self.m_slots[i]
		node:setModel(slot)
		node:setVisible(not isMaxLv)
	end

	local button = csNode.button
	if isMaxLv then
		button.text:setString(Serialize[61446])
	else
		if refine:isCanUpgrade() then
			button.text:setString(Serialize[61447])
		else
			button.text:setString(Serialize[61448])
		end
	end
end

function This:refreshSkill()
	local hero = HeroManager:getHero(self.m_heroId)
	local refine = hero:getRefine()
	local scrollview = self.m_csNode.scrollview
	local skillList = refine:getSkillList()

	for i, skill in ipairs(skillList) do
		local node = self.m_skills[i]
		if not node then
			node = HeroRefineSkillNode:create(skill)
			local w, h = node:getContentSize()
			scrollview:pushBackCustomItem(node, w, h, 0.5, 10)
			self.m_skills[i] = node
		else
			node:setModel(skill)
		end
	end

	local nodeNum = #self.m_skills
	local skillNum = #skillList

	if nodeNum > skillNum then
		for i=nodeNum, skillNum+1, -1 do
			scrollview:removeItem(i)
			self.m_skills[i] = nil
		end
	end
end

function This:onButtonClick()
	local hero = HeroManager:getHero(self.m_heroId)
	local refine = hero:getRefine()

	if refine:isCanUpgrade() then
		HeroManager:onRefineUpgradeRequest(self.m_heroId)
		return
	end

	local ret, tips = refine:isCanWear()
	if ret then
		HeroManager:onRefineOneKeyWearRequest(self.m_heroId)
		return
	end

	if tips and tips ~= "" then 
		g_scene:showTips(tips)
	end
end

function This:onRefineUpgradeReply(hero)
	if hero:getId() == self.m_heroId then
		self:refresh()
		g_scene:showTips(Serialize[61405])
	end
end

function This:onRefineWearReply(hero)
	if hero:getId() == self.m_heroId then
		self:refresh()
	end
end

function This:onRefineOneKeyWearReply(hero)
	if hero:getId() == self.m_heroId then
		self:refresh()
	end
end

function This:onRefineMergeReply(itemId)
	self:refresh()
end

function This:onUpgradeLevel(heroId)
	if heroId == self.m_heroId then
		self:refresh()
	end
end

function This:itemUpdate(itemList)
	self:refresh()
end