

local This, super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")
local ItemConfig = require("Metadatas.ItemConfig")
local RefinementItemAttributeConfig = require("Metadatas.RefinementItemAttributeConfig")
local DepotManager = require("Depot.DepotManager")


function This:initData(slot)
	self.m_slot = slot
end

function This:initNode(slot)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroShenWenWear.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

    -- 关闭
    csNode.close:setTouchEnded(function ()
        self:close()
    end)

    -- 穿戴
    csNode.btnWear:setTouchEnded(function ()
        self:onWearClick()
    end)

    -- 物品
	self:refreshItemInfo()
end

function This:refreshItemInfo()
	local csNode = self.m_csNode
	local itemId = self.m_slot:getConfigItem().id
	local config = ItemConfig[itemId]

	-- 图标
	if not self.m_itemNode then
	    local itemNode = ItemNode:create(itemId)
	    itemNode:setAnchorPoint(0.5, 0.5)
	   	itemNode:setScale(0.8)
	   	itemNode:setPosition(csNode.item:getPosition())
	    csNode:addChild(itemNode)
	    self.m_itemNode = itemNode
	end

    -- 名字
    csNode.lbName:setString(config.name)
    csNode.lbName:setColorByID("Q".. config.quality)

    -- 数量
    local count = DepotManager:getItemCount(itemId)
    csNode.lbNum:setString( string.format(Serialize[61422], count) )

    -- 属性
    csNode.lbAttr:setStringWithColor( self:getItemAttrDesc() )
    
    -- 描述
    csNode.lbDesc:setString(config.comment) 

    -- 穿戴等级
    local wearLv = self.m_slot:getWearLv()
    local heroLv = self.m_slot:getHero():getLevel() 
    csNode.lbNeed:setString(string.format(Serialize[61424], wearLv))
    if heroLv >= wearLv then
    	csNode.lbNeed:setColor(255, 239, 209)
    else
    	csNode.lbNeed:setColor(255, 0, 0)
    end

end

function This:getItemAttrDesc()
	local itemId = self.m_slot:getConfigItem().id
	local attribute = RefinementItemAttributeConfig[itemId].attribute
	local desc = ""
	local count = #attribute

	for i, attr in ipairs(attribute) do
		local attrId = attr[1]
		local attrVal = attr[2]

		if AttrInfo[attrId].valueType == ValueType.kPercent then
			desc = desc .. string.format("<A1>%s：<A2>%.2f%%", AttrName[attrId], attrVal*100)
		else
			desc = desc .. string.format("<A1>%s：<A2>%d", AttrName[attrId], math.floor(attrVal))
		end

		if i < count then
			desc = desc .. "\n"
		end
	end

	return desc
end

function This:onWearClick()
	local isCan, tips = self.m_slot:isCanWear()
	if isCan then
		local hero = self.m_slot:getHero()
		local item = self.m_slot:getConfigItem()
		HeroManager:onRefineWearRequest(hero:getId(), item.id, self.m_slot:getId())
		self:close()
	else
		if tips then
			g_scene:showTips(tips)
		end
	end
end