
-- 战法总览

local This, super = class(...,"Dialog")
local ItemNode = require("UI.ItemNode")
local RankPageListLayer = require("Rank.RankPageListLayer")
local TacticsConfig = require("Metadatas.TacticsConfig")
local HeroTacticsItemView = require("HeroTactics.HeroTacticsItemView")


local TabName = {
	kBlue = Quality.kBlue,
	kPurple = Quality.kPurple,
	kOrange = Quality.kOrange,
}


function This:initData(info)
	self.m_items = {}
	self.m_listType = TabName.kBlue
	self:initItemList()
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsAll.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

   	local scrollView = csNode.scrollView
	scrollView:setDirection("vertical")
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 100,
        itemHeight = 100,
        spaceX = 0,
        spaceY = 0,
        column = 4,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getItemList(self.m_listType)	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getItemList(self.m_listType)	   
        	local data = list[index]
        	if data then     
			    if not node then
			    	node = HeroTacticsItemView:create({id = data.id})
			    	node:setClick(function (node)
			    		g_scene:openDialog("HeroTactics.HeroTacticsDetailsPanel", {id = node:getId()})
			    	end, true)
					node:setScale(0.85)
			    	return node
				else
			        node:setModel({id = data.id})
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)


	self:addTabButton(TabName.kBlue, csNode.tep1, function()
		self.m_listType = TabName.kBlue
		self.m_listLayer:setDataList()
	end)

	self:addTabButton(TabName.kPurple, csNode.tep2, function()
		self.m_listType = TabName.kPurple
		self.m_listLayer:setDataList()
	end)

	self:addTabButton(TabName.kOrange, csNode.tep3, function()
		self.m_listType = TabName.kOrange
		self.m_listLayer:setDataList()
	end)

	self:selectTabButton(TabName.kBlue)
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

    	-- 关闭
	    csNode.beijing.close:setTouchEnded(function ()
	        self:close()
	    end)
    end
end	    

function This:refresh()
	
end

function This:initItemList()
	local items = {}
	for id, v in pairs(TacticsConfig) do
		if v.star == 1 then
			if not items[v.quality] then
				items[v.quality] = {}
			end
			table.insert(items[v.quality], v)
		end
	end

	for quality, list in pairs(items) do
		if #list > 1 then
			table.sort(list, function (a, b)
				return a.id < b.id
			end)
		end
	end

	self.m_items = items
end

function This:getItemList(quality)
	return self.m_items[quality] or {}
end

function This:onEnter()
	super.onEnter(self)
	self:addListener("tactics_goto_get", function()
		self:close()
	end)
end