
-- 战法佩戴选择

local This, super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local DepotItemView = require("Depot.DepotItemView")


function This:initData(info)
	self.m_slot = info.slot
	self.m_sortType = 0 -- 0默认，1星级，2类型
	self.m_itemList = {}
	self.m_selectItem = nil
	self.m_itemNode = nil
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsReplace.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

   	local scrollView = csNode.scrollview
	scrollView:setDirection("vertical")
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 100,
        itemHeight = 100,
        spaceX = 0,
        spaceY = 0,
        column = 4,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getItemList()	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getItemList()	   
        	local item = list[index]
        	if item then     
			    if not node then
			    	node = DepotItemView:create({
			    		item = item, 
			    		index = index, 
			    		aloneMode = true,
			    		is_selected = self:checkItemSelect(item),
			    		handler = function ()
			    			if self:checkItemSelect(node.item) then
			    				return
			    			end
			    			self:setSelectItem(node.item)
			    		end
			    	})
			    	node:setScale(0.9)
			    	return node
				else
			        node:setModel(item, index, 0, self:checkItemSelect(item), true)
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)


    self:setSortType(0)
    self:setSelectItem(self.m_itemList[1])
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

    	-- 关闭
	    csNode.closebg:setTouchEnded(function ()
	        self:close()
	    end)

	    -- 佩戴
	    csNode.btnWear:setTouchEnded(function ()
	    	local item = self.m_selectItem
	    	if not item then
	    		return
	    	end

	    	local hero = self.m_slot:getHero()
	    	local tactics = hero:getTactics()
	    	local curSlotId = self.m_slot:getId()
	    	local selectType = item:getTacticsType()
	    	local sameTypeNum = 0

	    	for i, slot in ipairs( tactics:getAllSlots() ) do
	    		if slot:getId() ~= curSlotId then
	    			if slot:getTacticsType() == selectType then
	    				sameTypeNum = sameTypeNum + 1
	    			end
	    		end
	    	end

	    	if sameTypeNum >= 1 then
	    		g_scene:showTips(Serialize[61455])
	    		return
	    	end

	    	HeroManager:onTacticsWearRequest(item:getUid(), hero:getUid(), curSlotId)
	    	self:close()
	    end)

	    -- 升星
	    csNode.btnStar:setTouchEnded(function ()
	    	local item = self.m_selectItem
	    	if not item then
	    		return
	    	end

	    	if item:isMaxStar() then
	    		g_scene:showTips(Serialize[61456])
	    		return
	    	end

	    	g_scene:openDialog("HeroTactics.HeroTacticsStarPanel", {item = item})
	    	self:close()
	    end)

	    -- 星级
	    csNode.select1:setTouchEnded(function ()
	    	if self.m_sortType == 1 then
	    		self:setSortType(0)
	    	else
	    		self:setSortType(1)
	    	end
	    end)

	    -- 类型
	    csNode.select2:setTouchEnded(function ()
	    	if self.m_sortType == 2 then
	    		self:setSortType(0)
	    	else
	    		self:setSortType(2)
	    	end
	    end)
    end
end

function This:initItemList()
	local function filter(item)
		if not item:checkCommonType(ItemCommonType.kTactics) then
			return false, 0
		end
		return (not item:isWear()), 0
	end

	local sorts = {}

	-- 品质＞星级＞类型
	sorts[0] = function (a, b)
		local qualityA, qualityB = a:getQuality(), b:getQuality()
		if qualityA ~= qualityB then
			return qualityA > qualityB
		end

		local starA, starB = a:getStar(), b:getStar()
		if starA ~= starB then
			return starA > starB
		end

		local typeA, typeB = a:getTacticsType(), b:getTacticsType()
		if typeA ~= typeB then
			return typeA > typeB
		end

		return a:getId() < b:getId()
	end

	-- 星级＞品质＞类型
	sorts[1] = function (a, b)
		local starA, starB = a:getStar(), b:getStar()
		if starA ~= starB then
			return starA > starB
		end

		local qualityA, qualityB = a:getQuality(), b:getQuality()
		if qualityA ~= qualityB then
			return qualityA > qualityB
		end

		local typeA, typeB = a:getTacticsType(), b:getTacticsType()
		if typeA ~= typeB then
			return typeA > typeB
		end

		return a:getId() < b:getId()
	end

	-- 类型＞品质＞星级
	sorts[2] = function (a, b)
		local typeA, typeB = a:getTacticsType(), b:getTacticsType()
		if typeA ~= typeB then
			return typeA > typeB
		end

		local qualityA, qualityB = a:getQuality(), b:getQuality()
		if qualityA ~= qualityB then
			return qualityA > qualityB
		end

		local starA, starB = a:getStar(), b:getStar()
		if starA ~= starB then
			return starA > starB
		end

		return a:getId() < b:getId()				
	end

	self.m_itemList = DepotManager:getItemListByFilter(filter, sorts[self.m_sortType])
end

function This:getItemList()
	return self.m_itemList
end

-- 设置排序
function This:setSortType(sortType)
	self.m_sortType = sortType

	local csNode = self.m_csNode
	csNode.select1.selected:setVisible(sortType == 1)
	csNode.select2.selected:setVisible(sortType == 2)

	self:initItemList()
	self.m_listLayer:setDataList()

	local isEmpty = #self.m_itemList <= 0
	csNode.empty:setVisible(isEmpty)
end

-- 设置选中
function This:setSelectItem(item)
	self.m_selectItem = item
	self:refreshListViewSelect()
	self:refreshItemInfo()
end

function This:checkItemSelect(item)
	if self.m_selectItem then
		return self.m_selectItem:getUid() == item:getUid()
	end
	return false
end

-- 列表的选中
function This:refreshListViewSelect()
	local views = self.m_listLayer:getAllViews()
	for k, itemView in pairs(views) do
		itemView:setSelected( self:checkItemSelect(itemView.item) )
	end
end

-- 选中的信息
function This:refreshItemInfo()
	local csNode = self.m_csNode
	local item = self.m_selectItem
	local isSelect = (item ~= nil)

	if self.m_itemNode then
		self.m_itemNode:setVisible(isSelect)
	end
	self:setNodeListVisible(csNode, {"namebg", "itemname", "shuxing", "btnWear", "btnStar"}, isSelect)

	if not isSelect then
		return
	end

	-- 物品
	if not self.m_itemNode then
		local node = ItemNode:create(item:getId())
		local w, h = node:getContentSize()
		node:setScale(82/h)
		node:setAnchorPoint(0.5, 0.5)
		node:setPosition( csNode.item:getPosition() )
		csNode:addChild(node)
		self.m_itemNode = node
	else
		self.m_itemNode:setModel({id = item:getId()})
	end

	-- 名字
	csNode.itemname:setString( item:getName() )
	csNode.itemname:setColorByID("Q".. item:getQuality())

	-- 属性
	local desc = ""
	local attribute = item:getTacticsConfigVal("arg")
	local count = #attribute
	for i, attr in ipairs(attribute) do
		local attrId = attr[1]
		local attrVal = attr[2]

		if AttrInfo[attrId].valueType == ValueType.kPercent then
			desc = desc .. string.format("<New2>%s <New9>+%.2f%%", AttrName[attrId], attrVal*100)
		else
			desc = desc .. string.format("<New2>%s <New9>+%d", AttrName[attrId], math.floor(attrVal))
		end

		if i < count then
			desc = desc .. "\n"
		end
	end
	csNode.shuxing:setStringWithColor(desc)

	if item:isMaxStar() then
		csNode.btnStar:setGrayShader()
	else
		csNode.btnStar:clearShader()
	end
end

function This:refresh()
	
end

function This:setNodeListVisible(parent, list, visible)
	for i, name in ipairs(list) do
		local node = parent[name]
		if node then
			node:setVisible(visible)
		end
	end
end