

local This, super = class(...,"Dialog")
local HeroTacticsItemView = require("HeroTactics.HeroTacticsItemView")
local TacticsConfig = require("Metadatas.TacticsConfig")
local ItemConfig = require("Metadatas.ItemConfig")
local EventDispatcher = require("EventDispatcher")
local ViewUtil = require("UI.ViewUtil")


function This:initData(info)
    self.m_id = info.id
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsDetails.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

        csNode.bg:setTouch({
            isScale = false,
            isAlpha = false,
            onEnded = function ()
                return
            end
        })

        -- 获取
        csNode.go:setTouchEnded(function ()
            ViewUtil:jumpToPanel("ExpeditionPanel")
            EventDispatcher:dispatcher("tactics_goto_get")
            self:close()
        end)
    end
end

function This:onTouchMask()
    self:close()
end	    

function This:refresh()
    local csNode = self.m_csNode
    local itemCfg = ItemConfig[self.m_id]

	if not self.m_itemNode then
        local node = HeroTacticsItemView:create({id = self.m_id})
        node:setScale(0.8)
        node:setAnchorPoint(0.5, 0.5)
        node:setPosition( csNode.item:getPosition() )
        csNode:addChild(node)
        self.m_itemNode = node
    else
        self.m_itemNode:setModel({id = self.m_id})
    end

    csNode.itemname:setString(itemCfg.name)
    csNode.itemname:setColorByID("Q" .. itemCfg.quality)

    local descList = self:getAttrDescList()
    for i, v in ipairs(descList) do
        local node1 = csNode[string.format("star%d", i)]
        local node2 = csNode[string.format("shuxing%d", i)]
        node1:setString(string.format(Serialize[61458], v.star))
        node2:setStringWithColor(v.attrText)
    end
end

function This:getAttrDescList()
    local list = {}
    local minId = self.m_id - TacticsConfig[self.m_id].star + 1

    local getAttrText = function (attrList)
        local desc = ""
        local count = #attrList
        for i, attr in ipairs(attrList) do
            local attrId = attr[1]
            local attrVal = attr[2]
            if AttrInfo[attrId].valueType == ValueType.kPercent then
                desc = desc .. string.format("<New1>%s <New9>+%.2f%%", AttrName[attrId], attrVal*100)
            else
                desc = desc .. string.format("<New1>%s <New9>+%d", AttrName[attrId], math.floor(attrVal))
            end
            if i < count then
                desc = desc .. " "
            end
        end
        return desc
    end

    for i=1, 5 do
        local config = TacticsConfig[minId + i - 1]
        list[i] = {star = i, attrText = getAttrText(config.arg)}
    end

    return list
end