
-- 战法信息

local This, super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")
local TacticsConfig = require("Metadatas.TacticsConfig")
local GameUtil = require("Util.GameUtil")


function This:initData(info)
	self.m_slot = info.slot
	self.m_itemNode = nil
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsValue.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true


	   	csNode.bg:setTouch({
	   		isScale = false,
	   		isAlpha = false,
	   		onEnded = function ()
	   			return
	   		end
	   	})

    	-- 升星
	    csNode.button:setTouchEnded(function ()
	    	g_scene:openDialog("HeroTactics.HeroTacticsStarPanel", {item = self.m_slot:getItem()})
	        self:close()
	   	end)

	   	-- 卸下
	    csNode.buttonxiexia:setTouchEnded(function ()
	    	local slot = self.m_slot 
	    	HeroManager:onTacticsWearRequest(0, slot:getHero():getUid(), slot:getId())
	        self:close()
	   	end)

	    -- 更换
	   	csNode.buttongenghuan:setTouchEnded(function ()
	   		g_scene:openDialog("HeroTactics.HeroTacticsChoosePanel", {slot = self.m_slot})
	        self:close()
	   	end)	
	end
end

function This:onTouchMask()
    self:close()
end

function This:refresh()
	local csNode = self.m_csNode
	local item = self.m_slot:getItem()
	local isHasItem = (item ~= nil)

	self:setNodeListVisible(csNode, {"itemname", "button", "buttonxiexia", "buttongenghuan", "xing1", "shuxing1", "xing2", "shuxing2"}, isHasItem)
	if self.m_itemNode then
		self.m_itemNode:setVisible(isWear)
	end

	if not isHasItem then
		return
	end

	-- 物品
	if not self.m_itemNode then
		local node = ItemNode:create(item:getId())
		local w, h = node:getContentSize()
		node:setScale(76/h)
		node:setAnchorPoint(0.5, 0.5)
		node:setPosition( csNode.item:getPosition() )
		csNode:addChild(node)
		self.m_itemNode = node
	else
		self.m_itemNode:setModel({id = item:getId()})
	end

	-- 名字
	csNode.itemname:setString( item:getName() )
	csNode.itemname:setColorByID("Q".. item:getQuality())

	-- 当前效果
	local curEff = self:getStarUpEffect(item:getId())
	csNode.xing1:setString(curEff.starDesc)
	csNode.shuxing1:setStringWithColor(curEff.attrDesc)

	-- 下一级效果
	local isMaxStar = item:isMaxStar()
	if not isMaxStar then
		local nexEff = self:getStarUpEffect(item:getNextStarTargetId())
		csNode.xing2:setString(nexEff.starDesc)
		csNode.shuxing2:setStringWithColor(nexEff.attrDesc)
	else
		csNode.xing2:setString("")
		csNode.shuxing2:setStringWithColor(Serialize[61460])
	end

	csNode.button:setVisible(not isMaxStar)
	GameUtil:refreshRedDot(csNode.button, self.m_slot:isUpgradeStarRedDot())
end

function This:getStarUpEffect(id)
	local ret = {
		starDesc = "",
		attrDesc = "",
	}

	local config = TacticsConfig[id]
	if not config then
		return ret
	end

	local desc = ""
	local attribute = config.arg
	local count = #attribute
	for i, attr in ipairs(attribute) do
		local attrId = attr[1]
		local attrVal = attr[2]

		if AttrInfo[attrId].valueType == ValueType.kPercent then
			desc = desc .. string.format("<New1>%s <New9>+%.2f%%", AttrName[attrId], attrVal*100)
		else
			desc = desc .. string.format("<New1>%s <New9>+%d", AttrName[attrId], math.floor(attrVal))
		end

		if i < count then
			desc = desc .. "\n"
		end
	end

	ret.starDesc = string.format(Serialize[61458], config.star)
	ret.attrDesc = desc

	return ret
end

function This:setNodeListVisible(parent, list, visible)
	for i, name in ipairs(list) do
		local node = parent[name]
		if node then
			node:setVisible(visible)
		end
	end
end