

local This , super = class(...,"NodeUI")
local ItemUtil = require("Bag.ItemUtil")
local ItemConfig = require("Metadatas.ItemConfig")
local GameUtil = require("Util.GameUtil")


function This:initData(info)
	self.m_id = info.id
end

function This:initNode(data)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsIcon.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local w, h = 100, 100
    csNode:setPosition(w/2, h/2)
    self:setContentSize(w, h)
end

function This:initTouchEvent()
    if not self.m_inited then
    	self.m_inited = true

        self:setTouch({
            onEnded = function ()
                if type(self.m_handler) == "function" then
                    self:m_handler(self)
                end
            end,
            isContainer = self.m_isContainer,
            isScale = false,
        })  
    end	
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:setModel(info)
	self.m_id = info.id
    self:refresh()
end

function This:refresh()
	local csNode = self.m_csNode
    csNode.quality:setImage( ItemUtil.getQualityImage(self.m_id) )
    csNode.icon:setImage( ItemUtil.getIconImage(self.m_id) )
    self:refreshQualityEffect()
end

function This:refreshQualityEffect()
    local config = ItemConfig[self.m_id]
    local hasEffect = config.quality > 4 -- 橙色和红色品质道具有特效

    if hasEffect then
        if not self.m_effectQuality then
            self.m_effectQuality = GameUtil:createEffect("Itemnb", true)
            self.m_effectQuality:setPosition(0, 11)
            self.m_csNode:addChild(self.m_effectQuality)
        else
            self.m_effectQuality:setVisible(true)
        end
    else
        if self.m_effectQuality then
            self.m_effectQuality:setVisible(false)
        end
    end
end

function This:setClick(handler, isContainer)
    self.m_handler = handler
    self.m_isContainer = isContainer
end

function This:getId()
	return self.m_id
end