

local This = class(...)
local HeroManager = require("Hero.HeroManager")
local DepotManager = require("Depot.DepotManager")
local ArgConfig = require("Metadatas.ArgConfig")


function This:create(info)
	local o = {}
	setmetatable(o,self)
	o:init(info)
	return o
end

function This:init(info)
	self.m_hero = info.hero
	self.m_slots = info.slots
	self.m_refineIdx = 1
	self.m_unlockLv = ArgConfig.tactics_position_openlevel.arg[#ArgConfig.tactics_position_openlevel.arg]
	self:updateRefineIdx()
end

function This:getRefineIdx()
	return self.m_refineIdx
end

function This:isMaxRefine()
	local slot = self.m_slots[self.m_refineIdx]
	if slot then
		return slot:isMaxPosLv()
	end
	return false
end

-- 解锁武将等级
function This:getUnlockLv()
	return self.m_unlockLv
end

function This:isUnlock()
	return self.m_hero:getLevel() >= self:getUnlockLv()
end

function This:getCost()
	local ret = {}
	local slot = self.m_slots[self.m_refineIdx]
	if slot and not slot:isMaxPosLv() then
		local cost = slot:getStrengthenCost()
		if cost and cost[1] then
			ret.id = cost[1]
			ret.needNum = cost[2]
			ret.curNum = DepotManager:getItemOrResCount(cost[1]) or 0
		end
	end
	return ret
end

function This:updateRefineIdx()
	local choose = nil
	for i, slot in ipairs(self.m_slots) do
		if not choose then
			choose = slot
		else
			if slot:getPosLv() < choose:getPosLv() then
				choose = slot
				break
			elseif slot:getPosLv() > choose:getPosLv() then
				break
			else 
				if i == #self.m_slots then
					choose = self.m_slots[1]
				else
					choose = slot
				end
			end 
		end
	end
	self.m_refineIdx = choose and choose:getId() or 1
end

function This:updateSlots(slots)
	self.m_slots = slots
	self:updateRefineIdx()	
end

-- 精练红点
function This:isRedDot()
	if not self:isUnlock() then
		return false
	end

	local slot = self.m_slots[self.m_refineIdx]
	if not slot or slot:isMaxPosLv() then
		return false
	end

	local cost = self:getCost()
	if next(cost) then
		return cost.curNum >= cost.needNum
	end
	return false
end