

local This = class(...)
local TacticsSuitConfig = require("Metadatas.TacticsSuitConfig")
local TacticsSkillConfig = require("Metadatas.TacticsSkillConfig")
local TacticsHeroConfig = require("Metadatas.TacticsHeroConfig")
local HeroConfig = require("Metadatas.HeroConfig")


function This:create(info)
	local o = {}
	setmetatable(o,self)
	o:init(info)
	return o
end

function This:init(info)
	self.m_hero = info.hero
	self.m_slots = info.slots
	self.m_skillId = 0
	self.m_isActivate = false

	local heroId = self.m_hero:getId()
	local config = TacticsHeroConfig[heroId]
	if config then
		self.m_skillId = config.skill_id
	end
	self:updateSlots(info.slots)
end

function This:getHero()
	return self.m_hero
end

function This:isActivate()
	return self.m_isActivate
end

function This:getSkillId()
	return self.m_skillId
end

function This:getConditionConfig()
	return TacticsSuitConfig[1001]
end

function This:getSkillConfig()
	return TacticsSkillConfig[self:getSkillId()]
end

function This:getAttr()
	local ret = {}
	local config = TacticsSkillConfig[self:getSkillId()]
	if config then
		for _, v in ipairs(config.arg) do
			ret[v[1]] = v[2]
		end
	end
	return ret	
end

function This:checkCondition(count, quality, star)
	if not self.m_slots then
		return false
	end

	local num = 0
	
	for _, slot in pairs(self.m_slots) do
		local item = slot:getItem()
		if item then
			if item:getQuality() >= quality and item:getStar() >= star then
				num = num + 1
			end
		end
	end

	return num >= count
end

function This:updateSlots(slots)
	self.m_slots = slots
	local config = self:getConditionConfig()
	self.m_isActivate = self:checkCondition(config.count, config.quality, config.star)
end