

local This = class(...)
local ArgConfig = require("Metadatas.ArgConfig")
local TacticsStrengthenConfig = require("Metadatas.TacticsStrengthenConfig")
local TacticsSuitConfig = require("Metadatas.TacticsSuitConfig")
local DepotManager = require("Depot.DepotManager")


function This:create(info)
	local o = {}
	setmetatable(o,self)
	o:init(info)
	return o
end

function This:init(info)
	self.m_id = info.id
	self.m_hero = info.hero

	self.m_itemUid = 0
	self.m_item = nil
	self.m_curPosLv = 0
	self.m_maxPosLv = ArgConfig.tactics_position_level.arg[1]
	self.m_unlockLv = ArgConfig.tactics_position_openlevel.arg[self.m_id]
end

function This:getId()
	return self.m_id
end

function This:getHero()
	return self.m_hero
end

-- 槽位等级
function This:setPosLv(lv)
	self.m_curPosLv = lv or 0
end

function This:getPosLv()
	return self.m_curPosLv
end

function This:isMaxPosLv()
	return self.m_curPosLv >= self.m_maxPosLv
end

-- 解锁武将等级
function This:getUnlockLv()
	return self.m_unlockLv
end

function This:isUnlock()
	return self.m_hero:getLevel() >= self:getUnlockLv()
end

-- 物品
function This:setItemUid(uid)
	uid = uid or 0

	self.m_itemUid = uid
	if uid ~= 0 then
		self.m_item = DepotManager:getItemByUid(uid)
	else
		self.m_item = nil
	end
end

function This:getItemUid()
	return self.m_itemUid
end

function This:getItem()
	return self.m_item
end

function This:getTacticsType()
	if self.m_item then
		return self.m_item:getTacticsType()
	end
	return 0
end

-- 是否穿戴
function This:isWear()
	return self.m_item ~= nil
end

-- 强化加成
function This:getAddition(level)
	level = level or self.m_curPosLv

	if level > self.m_maxPosLv then
		level = self.m_maxPosLv
	end

	local key = self:getStrengthenKey(level)
	local config = TacticsStrengthenConfig[key]
	if config then
		return config.Addition
	end
	return 0
end

-- 强化消耗
function This:getStrengthenCost()
	local key = self:getStrengthenKey(self.m_curPosLv+1)
	local config = TacticsStrengthenConfig[key]
	if config then
		return config.cost
	end	
	return {}
end

function This:getStrengthenKey(level)
	return self.m_hero:getQuality() * 1000 + self.m_id * 100 + level
end

function This:getAllStrengthenCost()
	local all = {}

	for i=1, self.m_curPosLv do
		local key = self:getStrengthenKey(i)
		local config = TacticsStrengthenConfig[key]
		if config then
			local id = config.cost[1]
			all[id] = (all[id] or 0) + config.cost[2]
		end
	end

	return all	
end

function This:isActivate()
	local item = self:getItem()
	if not item then
		return false
	end

	local config = TacticsSuitConfig[1001]
	return item:getQuality() >= config.quality and item:getStar() >= config.star
end

-- 总属性
function This:getTotalAttr()
	if not self.m_item then
		return {}
	end

	local ret = self.m_item:getAttrs()
	local rate = self:getAddition() + 1
	for id, val in pairs(ret) do
		ret[id] = val*rate
	end
	return ret
end

-- 升星红点
function This:isUpgradeStarRedDot()
	local item = self:getItem()
	if not item or item:isMaxStar() then
		return false
	end

	local list = item:getUpgradeStarCost()
	local isEnough = true

	for _, v in ipairs(list) do
		local id = v[1]
		local needNum = v[2]
		local curNum = DepotManager:getItemOrResCount(id) or 0

		if curNum < needNum then
			isEnough = false
			break
		end
	end

	return isEnough
end