
-- 战法升星

local This, super = class(...,"Dialog")
local RankPageListLayer = require("Rank.RankPageListLayer")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")
local DepotItemView = require("Depot.DepotItemView")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")


function This:initData(info)
	self.m_fromItem = info.item
	self.m_itemList = {}
	self.m_selectItem = nil
	self.m_itemNodes = {}

	self:initItemList(info.item)
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsStar.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

   	local scrollView = csNode.scrollview
	scrollView:setDirection("vertical")
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 80,
        itemHeight = 80,
        spaceX = 0,
        spaceY = 0,
        column = 1,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getItemList()	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getItemList()	   
        	local item = list[index]
        	if item then
			    if not node then
			    	node = DepotItemView:create({
			    		item = item, 
			    		index = index, 
			    		aloneMode = true,
			    		is_selected = self:checkItemSelect(item),
			    		handler = function ()
			    			if self:checkItemSelect(node.item) then
			    				return
			    			end
			    			self:setSelectItem(node.item)
			    		end
			    	})
			    	node:setScale(0.8)
			    	return node
				else
			        node:setModel(item, index, 0, self:checkItemSelect(item), true)
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)    
    self.m_listLayer:setDataList()

    self:setSelectItem(self.m_fromItem)
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

    	-- 关闭
	    csNode.close:setTouchEnded(function ()
	        self:close()
	    end)

	    -- 升星
	    csNode.button:setTouchEnded(function ()
	        if not self.m_selectItem then
	        	return
	        end

	        if self.m_selectItem:isMaxStar() then
	        	return
	        end

	        local cost, err = self:getUpgradeStarCost(self.m_selectItem)
	        if not err.isEnough then
	        	g_scene:showTips(err.tips)
	        	return
	        end
				       
	        HeroManager:onTacticsUpgradeStarRequest(self.m_selectItem:getUid())
	    end)
	end

end

function This:initItemList(fromItem)
	self.m_itemList = {fromItem}

	local cardId = fromItem:getEquipCardId()
	if cardId == 0 then		
		return
	end

	local hero = HeroManager:getHeroByUid(cardId)
	if not hero then
		return
	end

	local tactics = hero:getTactics()
	self.m_itemList = {}

	for i, slot in ipairs(tactics:getAllSlots()) do
		local item = slot:getItem()
		if item then
			table.insert(self.m_itemList, item)
		end
	end
end

function This:onEnter()
	super.onEnter(self)
	HeroManager:addObserver(self)
end

function This:onExit()
	HeroManager:removeObserver(self)
	super.onExit(self)
end

function This:getItemList()
	return self.m_itemList
end

-- 设置选中
function This:setSelectItem(item)
	self.m_selectItem = item
	self:refreshListViewSelect()
	self:refreshItemInfo()	
end

function This:checkItemSelect(item)
	if self.m_selectItem then
		return self.m_selectItem:getUid() == item:getUid()
	end
	return false
end

-- 列表的选中
function This:refreshListViewSelect()
	local views = self.m_listLayer:getAllViews()
	for k, itemView in pairs(views) do
		itemView:setSelected( self:checkItemSelect(itemView.item) )
	end
end

-- 选中的信息
function This:refreshItemInfo()
	local csNode = self.m_csNode
	local item = self.m_selectItem
	local isSelect = (item ~= nil)

	for k, node in pairs(self.m_itemNodes) do
		node:setVisible(isSelect)
	end
	self:setNodeListVisible(csNode, {"nowitemnum", "jiantou", "icon", "cost", "button", "tips"}, isSelect)

	if not isSelect then
		return
	end

	local isMaxStar = item:isMaxStar()
	csNode.tips:setVisible(isMaxStar)

	if isMaxStar then
		for k, node in pairs(self.m_itemNodes) do
			node:setVisible(false)
		end
		self:setNodeListVisible(csNode, {"nowitemnum", "icon", "cost", "button"}, false)
		return
	end

	-- 消耗
	local cost = self:getUpgradeStarCost(item)

	local curNode = self.m_itemNodes["curNode"]
	if not curNode then
		curNode = ItemNode:create(cost[1].id)
		curNode:setAnchorPoint(0.5, 0.5)
		curNode:setPosition(csNode.nowitem:getPosition())
		curNode:setScale(0.8)
		csNode:addChild(curNode)
		self.m_itemNodes["curNode"] = curNode
	else
		curNode:setModel({id = cost[1].id})
	end
	csNode.nowitemnum:setString(string.format("%d/%d", cost[1].curNum, cost[1].needNum))
	if cost[1].curNum >= cost[1].needNum then
		csNode.nowitemnum:setColor(255, 255, 255)
	else
		csNode.nowitemnum:setColor(255, 0, 0)
	end

	csNode.icon:setImage( ItemUtil.getIconImage(cost[2].id) )
	csNode.cost:setString(string.format("%s/%s", cost[2].curNum, cost[2].needNum))
	if cost[2].curNum >= cost[2].needNum then
		csNode.cost:setColor(255, 255, 255)
	else
		csNode.cost:setColor(255, 0, 0)
	end	

	local nexNode = self.m_itemNodes["nexNode"]
	local nexId = item:getNextStarTargetId()
	if not nexNode then
		nexNode = ItemNode:create(nexId)
		nexNode:setAnchorPoint(0.5, 0.5)
		nexNode:setPosition(csNode.nextitem:getPosition())
		nexNode:setScale(0.8)
		csNode:addChild(nexNode)
		self.m_itemNodes["nexNode"] = nexNode
	else
		nexNode:setModel({id = nexId})
	end

end

function This:getUpgradeStarCost(item)
	local ret = {}
	local list = item:getUpgradeStarCost()
	local err = {isEnough = true, tips = ""}

	for _, v in ipairs(list) do
		local id = v[1]
		local t = {}
		t.id = id
		t.needNum = v[2]
		t.curNum = DepotManager:getItemOrResCount(id) or 0

		-- 主升装备不能算进去
		if item:getEquipCardId() == 0 and item:getId() == id then
			t.curNum = t.curNum - 1
		end

		table.insert(ret, t)

		if t.curNum < t.needNum then
			err.isEnough = false
			if err.tips == "" then
				err.tips = string.format(Serialize["world_TreasureBox_2"], ItemUtil.getName(t.id))
			end
		end
	end

	return ret, err
end

function This:setNodeListVisible(parent, list, visible)
	for i, name in ipairs(list) do
		local node = parent[name]
		if node then
			node:setVisible(visible)
		end
	end
end

function This:onTacticsUpgradeStarReply(itemUid)
	self.m_fromItem = DepotManager:getItemByUid(self.m_fromItem:getUid())
	if self.m_selectItem then
		self.m_selectItem = DepotManager:getItemByUid(self.m_selectItem:getUid())
	end
	self:initItemList(self.m_fromItem)
	self.m_listLayer:clearNode(true, true)
	self.m_listLayer:setDataList()
	self:refreshItemInfo()
end