

local This = class(...)
local HeroManager = require("Hero.HeroManager")
local TacticsSuitConfig = require("Metadatas.TacticsSuitConfig")
local TacticsHeroConfig = require("Metadatas.TacticsHeroConfig")
local TacticsSuitArgConfig = require("Metadatas.TacticsSuitArgConfig")

local AddTypeAttrText = {
	[1] = Serialize[61472],
	[2] = Serialize[61473],
	[3] = Serialize[61474],
	[4] = Serialize[61475],
	[5] = Serialize[61476],
	[6] = Serialize[61477],
}

function This:create(info)
	local o = {}
	setmetatable(o,self)
	o:init(info)
	return o
end

function This:init(info)
	self.m_hero = info.hero
	self.m_slots = info.slots
	self.m_suits = {}
	self.m_attrs = {}
	self.m_idxDict = {}

	for id, config in pairs(TacticsSuitConfig) do
		if config.index ~= 0 then
			local t = {}

			t.id = id
			t.index = config.index
			t.name = config.name
			t.count = config.count
			t.quality = config.quality
			t.star = config.star
			
			t.argId = 0
			t.attrs = {}
			t.attrList = {}
			t.attrType = 0
			t.attrDesc = ""
			local arg = self:getHeroSuitArgConfig(id)
			if arg then
				for _, v in ipairs(arg.attribute) do
					t.attrs[v[1]] = v[2]
				end
				t.argId = arg.id
				t.attrList = arg.attribute
				t.attrType = arg.type
				t.attrDesc = arg.desc
			end
			t.isActivate = false
			self.m_suits[id] = t

			if not self.m_idxDict[config.index] then
				self.m_idxDict[config.index] = {}
			end
			table.insert(self.m_idxDict[config.index], id)
		end
	end

	for index, idList in pairs(self.m_idxDict) do
		if #idList > 1 then
			table.sort(idList, function (a, b)
				return a < b
			end)
		end
	end

	self:updateSuit()
end

function This:updateSuit()
	self.m_attrs = {}

	for index, idList in pairs(self.m_idxDict) do
		local isAct = false
		for i=#idList, 1, -1 do --优先检查大的id，同个index的只能激活一条
			local t = self:getSuit(idList[i])
			if not isAct then
				t.isActivate = self:checkCondition(t.count, t.quality, t.star)
				if t.isActivate then
					if not self.m_attrs[t.attrType] then
						self.m_attrs[t.attrType] = {}
					end
					self:mergeAttr(self.m_attrs[t.attrType], t.attrs)
					isAct = true
				end
			else
				t.isActivate = false
			end
		end
	end

	local gAttr = HeroManager:getGlobalTacticsAttr()
	gAttr:updateHeroTacticsSuitAttr(self.m_hero:getId(), self.m_attrs)	
end

function This:getSuit(id)
	return self.m_suits[id]
end

function This:getAllSuits()
	return self.m_suits
end

function This:getActivateAttrs()
	return self.m_attrs
end

function This:getHeroSuitArgConfig(suitId)
	local config = TacticsHeroConfig[self.m_hero:getId()]
	if not config then
		return
	end

	local skillList = config.suit
	local index = TacticsSuitConfig[suitId].index
	local level = TacticsSuitConfig[suitId].level
	local id = (skillList[index] or 0) + level - 1

	return TacticsSuitArgConfig[id]
end

function This:checkCondition(count, quality, star)
	if not self.m_slots then
		return false
	end

	local num = 0
	
	for _, slot in pairs(self.m_slots) do
		local item = slot:getItem()
		if item then
			if item:getQuality() == quality and item:getStar() == star then
				num = num + 1
			end
		end
	end

	return num >= count
end

function This:mergeAttr(mainTb, subTb)
	for k, v in pairs(subTb) do
		if not mainTb[k] then
			mainTb[k] = 0
		end
		mainTb[k] = mainTb[k] + v
	end
	return mainTb
end

function This:updateSlots(slots)
	self.m_slots = slots
	self:updateSuit()	
end

function This:getAddSuitAttr()
	local ret = {}
	local gAttr = HeroManager:getGlobalTacticsAttr()

	-- 全体的
	self:mergeAttr(ret, gAttr:getTotalAttr())

	-- 国家的
	local country = self.m_hero:getCountry()
	self:mergeAttr(ret, gAttr:getCountryAttr(country))

	-- 个人的
	self:mergeAttr(ret, self.m_attrs[HeroTacticsSuitAddType.Single] or {})
	
	return ret
end

function This:getAddTypeAttrText(atype)
	return AddTypeAttrText[atype] or ""
end