
-- 战混界面

local This, super = class(...,"Dialog")
local RankPageListLayer = require("Rank.RankPageListLayer")
local HeroTacticsSuitItemView = require("HeroTactics.HeroTacticsSuitItemView")


function This:initData(info)
	self.m_suit = info.suit
	self.m_suitList = {}
	self:initSuitList()
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsSoul.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

   	local scrollView = csNode.scrollview
	scrollView:setDirection("vertical")
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 820,
        itemHeight = 124,
        spaceX = 0,
        spaceY = 10,
        column = 1,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getSuitList()	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getSuitList()	   
        	local data = list[index]
        	if data then
			    if not node then
			    	return HeroTacticsSuitItemView:create(data)
				else
			        node:setModel(data)
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)    
    
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

    	-- 关闭
	    csNode.close:setTouchEnded(function ()
	        self:close()
	   	end)

	end
end

function This:refresh()
	local csNode = self.m_csNode

	csNode.text1:setStringWithColor(self:getTotalAttrDesc())
	self.m_listLayer:setDataList()
end

function This:initSuitList()
	local idxDict = {}
	for id, info in pairs(self.m_suit:getAllSuits()) do
		local idx = info.index
		if idx ~= 0 then
			if not idxDict[idx] then
				idxDict[idx] = {}
			end
			table.insert(idxDict[idx], info)
		end
	end

	local ret = {}
	for idx, list in pairs(idxDict) do
		if #list > 1 then
			table.sort(list, function (a, b)
				return a.id < b.id
			end)
		end
		table.insert(ret, {index = idx, list = list, suit = self.m_suit})
	end

	if #ret > 1 then
		table.sort(ret, function (a, b)
			return a.index < b.index
		end)
	end	
	self.m_suitList = ret
end

function This:getSuitList()
	return self.m_suitList
end

function This:getTotalAttrDesc()
	local suit = self.m_suit
	local attrs = suit:getActivateAttrs()
	local color1, color2 = "New5", "New9"
	local desc = nil
	
	local getAttrDesc = function (dict)	
		local attrDesc = ""
		for attrId, attrVal in pairs(dict) do
			local text = ""
		    if AttrInfo[attrId].valueType == ValueType.kPercent then
		        text = string.format("<%s>%s<%s>+%.2f%%", color1, AttrName[attrId], color2, attrVal*100)
		    else
		        text = string.format("<%s>%s<%s>+%d", color1, AttrName[attrId], color2, math.floor(attrVal))
		    end
		    if attrDesc == "" then
		    	attrDesc = text
		    else
		    	attrDesc = attrDesc .. "    " .. text
		    end
		end
		return attrDesc
	end

	local list = {}
	for atype, dict in pairs(attrs) do
		local text = string.format("<%s>%s：%s", color1, suit:getAddTypeAttrText(atype), getAttrDesc(dict))
		table.insert(list, {atype = atype, text = text})
	end
	if #list > 1 then
		table.sort(list, function (a, b)
			return a.atype < b.atype
		end)
	end

	desc = string.format("<%s>", color1)
	if #list > 0 then
		for i, v in ipairs(list) do
			if i == 1 then
				desc = string.format("%s%s", desc, v.text)
			else
				desc = string.format("%s\n%s", desc, v.text)
			end
		end
	end

	return desc
end