

local This , super = class(...,"cc.Node")
local HeroManager = require("Hero.HeroManager")
local DepotManager = require("Depot.DepotManager")
local HeroTacticsSlotNode = require("HeroTactics.HeroTacticsSlotNode")
local HeroTacticsSkillNode = require("HeroTactics.HeroTacticsSkillNode")
local RankPageListLayer = require("Rank.RankPageListLayer")
local HeroTacticsMainAttrItemView = require("HeroTactics.HeroTacticsMainAttrItemView")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")


function This:init(heroId)
	self.m_heroId = heroId or 0
	self.m_slots = {}
	self.m_skill = nil
	self.m_attrList = {}
	self.m_showType = 0 --0战法，1精练
	self.m_redDot = {}
	self:initAttrList()
	self:initRedDot()
	self:initNode()
end

function This:initAttrList()
	local hero = HeroManager:getHero(self.m_heroId)
	local tactics = hero:getTactics()
	local attrs = tactics:getMainPanelAttr()

	local list = {}
	for id, val in pairs(attrs) do
		table.insert(list, {id, val})
	end
	if #list > 1 then
		table.sort(list, function (a, b)
			return a[1] < b[1]
		end)
	end
	self.m_attrList = list
end

function This:getAttrList()
	return self.m_attrList
end

function This:initNode()
	local csNode = cc.Node:createFromCSFile("GameScene/NodeUI/HeroTacticsMain.csb")
	self:addChild(csNode)
	self.m_csNode = csNode

	-- 槽位
	self.m_slots = {}
	for i = 1, 5 do
		local node = csNode[string.format("item%d", i)]
		local slot = HeroTacticsSlotNode:create({node = node})
		csNode:addChild(slot)
		self.m_slots[i] = slot
	end

	-- 技能
	self.m_skill = HeroTacticsSkillNode:create()
	self.m_skill:setPosition(csNode.skill:getPosition())
	csNode:addChild(self.m_skill)

	-- 属性列表
   	local scrollView = csNode.shuxing.scrollview
	scrollView:setDirection("vertical")
	scrollView:setSwallowTouch(false)
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 320,
        itemHeight = 25,
        spaceX = 0,
        spaceY = 0,
        column = 2,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getAttrList()	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getAttrList()	   
        	local data = list[index]
        	if data then     
			    if not node then
				    return HeroTacticsMainAttrItemView:create(data)
				else
			        node:setModel(data)
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)

    self:setShowType(0)
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

    	-- 战法
		csNode.buttonzf:setTouchEnded(function ()
			if self.m_showType == 0 then
				return
			end
	        self:setShowType(0)
	    end)

		-- 精练
		csNode.buttonjl:setTouchEnded(function ()
			local tactics = self:getTactics()
			local refine = tactics:getRefine()
			if not refine:isUnlock() then
				g_scene:showTips( string.format(Serialize[61452], refine:getUnlockLv()) )
				return
			end

			if self.m_showType == 1 then
				return
			end
	        self:setShowType(1)
	    end)

		-- 战法一览
		csNode.buttonyilan:setTouchEnded(function ()
	        g_scene:openDialog("HeroTactics.HeroTacticsAllPanel")
	    end)
		
		-- 战魂
		csNode.buttonzh:setTouchEnded(function ()
			local hero = HeroManager:getHero(self.m_heroId)
	        local tactics = hero:getTactics()
	        g_scene:openDialog("HeroTactics.HeroTacticsSuitPanel", {suit = tactics:getSuit()})
	    end)
	    
	    -- 一键佩戴
		csNode.buttonpeidai:setTouchEnded(function ()
	        local hero = HeroManager:getHero(self.m_heroId)
	        local tactics = hero:getTactics()
	        local idList, isChange = tactics:getBestWearList()

	        if isChange then
	        	-- print("------ onTacticsOneKeyWearRequest")
	        	-- print(tactics:uidListToString(idList))
	        	HeroManager:onTacticsOneKeyWearRequest(idList, hero:getUid())
	        end    	
	    end)

	    -- 一键卸载
	    csNode.buttonyuxie:setTouchEnded(function ()
	        local hero = HeroManager:getHero(self.m_heroId)
	        local isNeedReq = false
	        local idList = {}

	        for i, slot in ipairs(hero:getTactics():getAllSlots()) do
	        	if slot:isWear() then
	        		isNeedReq = true
	        	end
	        	idList[i] = 0
	        end

	        if isNeedReq then
	    		HeroManager:onTacticsOneKeyWearRequest(idList, hero:getUid())
	    	end
	    end)

	    -- 强化
	    csNode.jinglianhuafei.button:setTouchEnded(function ()
	    	local hero = HeroManager:getHero(self.m_heroId)
	    	local refine = hero:getTactics():getRefine()

	        if refine:isMaxRefine() then
	        	return
	        end

	        local cost = refine:getCost()
	        if cost.curNum < cost.needNum then
	        	g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(cost.id)))
	        	return
	        end

			HeroManager:onTacticsUpgradeSlotRequest(hero:getUid())
	    end)	    
    end
end

function This:onEnter()
	super.onEnter(self)
	self:initTouchEvent()
	HeroManager:addObserver(self)
	DepotManager:addObserver(self)
end

function This:onExit()
	HeroManager:removeObserver(self)
	DepotManager:removeObserver(self)
	super.onExit(self)
end

function This:setModel(heroId)
	if self.m_heroId ~= heroId then
		self.m_heroId = heroId
		self:initAttrList()
		self:initRedDot()
		self:setShowType(0)
	end
end

function This:setShowType(showType)
	self.m_showType = showType
	self:refresh()
end

function This:refresh()
	local csNode = self.m_csNode
	local tactics = HeroManager:getHero(self.m_heroId):getTactics()
	local isShow = true
	local text = ""

	if not tactics:isHasTacticsSystem() then
		isShow = false
		text = Serialize[61483]
	elseif not tactics:isTacticsFuncOpen() then
		isShow = false
		text = string.format(Serialize[61452], tactics:getOpenHeroLv())
	end
	csNode.tips:setVisible(not isShow)
	csNode.tips:setString(text)

	self:setNodeListVisible(
		self.m_csNode, 
		{"tuan", "item1", "item2", "item3", "item4", "item5", "skill", "shuxing", "jinglianhuafei", "buttonzf", "buttonjl", "buttonyilan", "buttonzh", "buttonpeidai", "buttonyuxie"}, 
		isShow
	)

	for k, slot in pairs(self.m_slots) do
		slot:setVisible(isShow)
	end
	self.m_skill:setVisible(isShow)

	if not isShow then
		return
	end

	if self.m_showType == 0 then
		self:showTacticsStyle()
	else
		self:showRefineStyle()
	end

	self:refreshRedDot()
end

-- 战法
function This:showTacticsStyle()
	local hero = HeroManager:getHero(self.m_heroId)
	local tactics = hero:getTactics()
	local csNode = self.m_csNode

	self:setNodeListVisible(csNode, {"jinglianhuafei"}, false)

	csNode.buttonzf.light:setVisible(true)
	csNode.buttonjl.light:setVisible(false)

	-- 槽位
	for id, node in ipairs(self.m_slots) do
		local slot = tactics:getSlot(id)
		node:setShowTyle(self.m_showType)
		node:setSelect(false)
		node:setModel(slot)
	end

	-- 技能
	self.m_skill:setModel(tactics:getSkill())

	-- 属性
	self.m_listLayer:setDataList()
end

-- 精练
function This:showRefineStyle()
	local hero = HeroManager:getHero(self.m_heroId)
	local tactics = hero:getTactics()
	local csNode = self.m_csNode

	self:setNodeListVisible(csNode, {"skill", "shuxing", "buttonyilan", "buttonzh", "buttonpeidai", "buttonyuxie"}, false)
	self.m_skill:setVisible(false)

	csNode.buttonzf.light:setVisible(false)
	csNode.buttonjl.light:setVisible(true)

	-- 槽位
	local refine = tactics:getRefine()
	local idx = refine:getRefineIdx()
	for id, node in ipairs(self.m_slots) do
		local slot = tactics:getSlot(id)
		node:setShowTyle(self.m_showType)
		node:setSelect(idx == id)
		node:setModel(slot)
	end

	-- 加成
	local jinglianhuafei = csNode.jinglianhuafei
	local choose = tactics:getSlot(idx)
	local curLv = choose:getPosLv()
	jinglianhuafei.now:setString( string.format(Serialize[61484], choose:getAddition(curLv)*100) )

	if choose:isMaxPosLv() then
		jinglianhuafei.next:setString(Serialize[61485])
		self:setNodeListVisible(jinglianhuafei, {"cost", "icon", "button"}, false)
	else
		jinglianhuafei.next:setString( string.format(Serialize[61486], choose:getAddition(curLv+1)*100) )
		self:setNodeListVisible(jinglianhuafei, {"cost", "icon", "button"}, true)

		local cost = refine:getCost()
		jinglianhuafei.icon:setImage( ItemUtil.getIconImage(cost.id) )
		jinglianhuafei.cost:setString(string.format("%s/%s", cost.curNum, cost.needNum))
		if cost.curNum >= cost.needNum then
			jinglianhuafei.cost:setColor(250, 194, 83)
		else
			jinglianhuafei.cost:setColor(255, 0, 0)
		end	
	end

end

function This:itemUpdate(itemList)
	self:refresh()
end

function This:setNodeListVisible(parent, list, visible)
	for i, name in ipairs(list) do
		local node = parent[name]
		if node then
			node:setVisible(visible)
		end
	end
end

function This:getTactics()
	return HeroManager:getHero(self.m_heroId):getTactics()
end

function This:onTacticsChanged()
	self:initAttrList()
	self:initRedDot()
	self:refresh()
end

function This:initRedDot()
	local tactics = self:getTactics()
	local redDot = {}

	redDot.oneKeyWear = tactics:isOneKeyWearRedDot()
	redDot.refine = tactics:getRefine():isRedDot()
	for i, slot in ipairs(tactics:getAllSlots()) do
		redDot[string.format("slot%d", i)] = slot:isUpgradeStarRedDot()
	end

	self.m_redDot = redDot
end

function This:getRedDot(key)
	local isRedDot = self.m_redDot[key]
	if isRedDot == nil then
		return false
	end
	return isRedDot
end

function This:refreshRedDot()
	local csNode = self.m_csNode
	GameUtil:refreshRedDot(csNode.buttonpeidai, self:getRedDot("oneKeyWear"))
	GameUtil:refreshRedDot(csNode.buttonjl, self:getRedDot("refine"))
	for id, node in ipairs(self.m_slots) do
		if self.m_showType == 0 then
		 	GameUtil:refreshRedDot(node.m_csNode, self:getRedDot(string.format("slot%d", id)), {30, 30})
		else
			GameUtil:refreshRedDot(node.m_csNode, false, {30, 30})
		end
	end
end

function This:onUpgradeLevel(heroId)
	self:refresh()
end