-- @Author: superm
-- @Date:   2017-12-12 19:37:53
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-03-20 15:02:12
-- @File Name: MenuBtnHelper.lua
-- @Description: 

local This, super= class(..., "Base.BaseModel")
local ActivityUtil = require("Activity.ActivityUtil")
local NewActivityUtil = require("NewActivity.NewActivityUtil")
local MenuButton = require("Home.MenuButton")
local UIButtonListBehaviour = require("UI.UIButtonListBehaviour")
local OpenLevelConfig = require("Metadatas.OpenLevelConfig")

This.instance = nil
This.s_valueList = {}
This.m_observers = {}


local BtnFieldType = 
{
	kPanelBtn = 2, --右下角界面按钮
	kActivityBtn = 3, --右上角活动按钮
}

local ScenceTypeCompose = {
    [10] = {1,2},
    [11] = {1,4},
    [12] = {2,4},
}

function This:create( node )
    local ret = {}
    setmetatable(ret,self)
    ret:init( node )
    This.instance = ret
    return ret
end

function This:init( node )
	self.m_csNode = node.m_csNode
	self.m_behaviours = {}
	self.m_buttons = {}

	if not self.m_configList then
        local function getSceneOrSceneList(configItem,sceneType)
            local sceneNum = configItem.scence
            local priority = configItem.priority
            local result = nil
            if sceneNum then
                if sceneNum ==0 then --通用
                    result = sceneType
                elseif priority>0 then 
                    if sceneNum<10 and sceneType == sceneNum then --单一scene
                        result = sceneNum
                    elseif sceneNum>=10 then --组合group
                        result = ScenceTypeCompose[sceneType]
                    end
                end
            end
            return result
        end
		local sceneType = g_scene:getType()
		local nameList = {}
		local configList = {}

        local function addOneScene(configItem,customGroup)
            local group = customGroup or configItem.group
            local priority = configItem.priority
            if not configList[group] then
                configList[group] = {}
            end
            local temConfig = configList[group][priority]
            if temConfig then
                print("error MenuBtnHelper:init()",temConfig.note,"和",configItem.note,"重复")
            end
            configList[group][priority] = configItem
        end
		for index, config in ipairs(OpenLevelConfig) do
            local sceneOrSceneList = getSceneOrSceneList(config,sceneType)
            if sceneOrSceneList then
                if type(sceneOrSceneList) == "table" then
                    for _,sceneNum in pairs(sceneOrSceneList) do
        				addOneScene(config,sceneNum)
                    end
                elseif type(sceneOrSceneList) == "number" then
                    addOneScene(config)
                end
            end
		end
		self.m_configList = configList
	end

	for group, list in pairs(self.m_configList) do
		local buttonList, info = {}, nil
		if group == BtnFieldType.kPanelBtn then
			info = {switch = MenuButton:createByName("close_bg", self.m_csNode), switchCount = 7, adjustable = "panel_btns"}
		elseif group == BtnFieldType.kActivityBtn then
			info = {switch = MenuButton:createByName("huodong_button", self.m_csNode), switchCount = 5, flexible = true}
		end
		local behaviour = UIButtonListBehaviour:create(self.m_csNode)

        local configArray = list
        -- local configArray = {}
        -- for _, config in pairs(list) do
        --     table.insert(configArray, config)
        -- end
        -- table.sort(configArray, function (configA, configB)
        --     return configA.priority < configB.priority
        -- end)
        for _, config in ipairs(configArray) do
			local creator
			if config.uiclass and string.len(config.uiclass.name) > 0 then
				creator = require(config.uiclass.name)
			end		

			if creator == nil or creator == true then
				creator = MenuButton
			end

            local ret = creator:create(config, self.m_csNode)
            if ret then
                table.insert(buttonList, ret)
                self.m_buttons[ret:getName()] = ret
            end
		end
		behaviour:setup(buttonList, info)
		self.m_behaviours[group] = behaviour
	end
end

function This:refresh(isInited)
    for group, behaviour in ipairs(self.m_behaviours) do
    	behaviour:refreshButtonsVisible(isInited)
    end
end

function This:forceOpen(id)
	local config = OpenLevelConfig[id]
	if config then
		local group = config.group
		local behaviour = self.instance.m_behaviours[group]
		behaviour:setButtonVisible(config.priority, true)
	end
end

function This:getButtonByName( name )
    return self.instance.m_buttons[name]
end

local EventDispatcher = require("EventDispatcher")
EventDispatcher:addListener(This, "on_menu_visible_change", function(This, message)
    local curNode = This.s_valueList[message.name]
    if not curNode then
        curNode = {}
        This.s_valueList[message.name] = curNode
    end
    -- print_r({name= message.name, before=curNode})
    local valueType = type(message.value)
    if valueType == "boolean" then
        curNode.value = message.value
    elseif valueType == "table" then
    	if not message.operate then
    		curNode.data = message.value
    		curNode.value = curNode.data and curNode.data[1] ~= nil
    	elseif message.operate == "+" then
    		local t = curNode.data
    		if not t then
    			t = {}
    			curNode.data = t
    		end
            for _, v in ipairs(message.value) do
                local found = not table.find(t, function(key, value)
                    return value == v
                end)
                if found then
                    table.insert(t, v)
                end
            end  
    		curNode.value = curNode.data and curNode.data[1] ~= nil
    	elseif message.operate == "-" then
    		local t, valueCount = curNode.data, 0
    		if not t then
	    		curNode.value = false
	    	else
                valueCount = #curNode.data
	            if valueCount == 0 then
                    return
                end

                for _, v in ipairs(message.value) do
                    if valueCount > 0 then
                        for k, lv in ipairs(t) do
                            if v == lv then
                                valueCount = valueCount - 1
                                table.remove(t, k)
                                break
                            end
                        end
                    else
                        break
                    end	                
	            end  
    		end
    		curNode.value = curNode.data and curNode.data[1] ~= nil
    	end
    end
    -- print_r({name= message.name, after=curNode})
    This:notifyObservers("onRedDotChange", message.name, curNode.value)
end)

function This:getRedDotValue( name )
    return This.s_valueList[name]
end