-- @Author: superm
-- @Date:   2017-12-20 11:42:14
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-30 21:17:56
-- @File Name: MenuButton.lua
-- @Description: 

local This , super = class(..., "cc.Node")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local MenuBtnHelper = require("Home.MenuBtnHelper")
local GuideManager = require("Guide.GuideManager")

function This.getNodeFromPath(parent, path)
    local name, ret, sparent = path, parent, parent
    for match in string.gmatch(path, "[^%.]+") do
        name = match
        sparent = ret
        ret = ret[match]
        if not ret then break end
    end
    return ret, name, sparent
end

function This:createByName( name, parent )
    local snode = parent[name]
    if not snode then
        return 
    end

    local snodeId = snode[0]
    local ret = cc.registerObject(snodeId, self.getAutoChildClass(snodeId, self._MNAME), {[0]=snodeId})
    if ret:getName() == "" then
        ret:setName(self._NAME)
    end
    parent[name] = ret
    return ret
end

function This:create(config, parent)
    local snode, name, sparent = self.getNodeFromPath(parent, config.name)
    if not snode then
        return 
    end

    local snodeId = snode[0]
    local ret = cc.registerObject(snodeId, self.getAutoChildClass(snodeId, self._MNAME), {[0]=snodeId})
    if ret:getName() == "" then
        ret:setName(self._NAME)
    end

    if ret.init then
        ret:init(config)
    end
    sparent[name] = ret
    return ret
end

function This:init(config)
    self.m_Id = config.id
    self.m_config = config
end

function This:onEnter()
    super.onEnter(self)
    self:initTouchEvent()
    self:addBaseListener()
    if not self:isAuditHidden() then
        self:addCustomListener()
    end
end

function This:initTouchEvent()
end

function This:getRedDotValue()
    local node = MenuBtnHelper:getRedDotValue(self:getName())
    if not node then
        return nil 
    end

    return node.value
end

function This:addBaseListener()
    if self:hasRedDotView() then
        local value = self:getRedDotValue()
        self:onRefreshRedDot(value)
        MenuBtnHelper:addObserver(self)
    end

    if self:hasGuideTouch() then
        GuideManager:addObserver(self)
    end
end

function This:removeBaseListener()
    MenuBtnHelper:removeObserver(self)
    GuideManager:removeObserver(self)
end

function This:addCustomListener()
end

function This:removeCustomListener()
end

function This:onExit()
    self:removeCustomListener()
    self:removeBaseListener()
    super.onExit(self)
end

function This:setConfig( config )
    self.m_config = config
end

function This:setBehaviour( behaviour )
    self.m_behaviour = behaviour
end

function This:setBehaviourSwitchVisible( value )
    if self.m_behaviour then
        self.m_behaviour:setSwitchVisible(value)
    end
end

-- info = {
--     value = false
--     callbackList = { 
--         node_handler=nil, 
--         other_handler=nil
--     }
-- }
function This:playBehaviourSwitchAnimation( info )
    if self.m_behaviour and info then
        self.m_switchState = info.value
        self.m_behaviour:playSwitchAnimation(info.value, info.callbackList, info.hasAnimation)
    end
end

function This:getSwitchState()
    if self:isVisible() then 
        return self.m_switchState ~= false
    end
    return true
end

function This:setRedDotVisible( value )
    if self.hongdian then
        self.hongdian:setVisible(value)
    end
end

function This:setTouch(touch)
    local func = touch.onEnded
    if touch then
        touch.isScale = false
        touch.isDynamic = true
        touch.onEnded = function ( ... )
            if not self:isPlaying() then
                func(...)
            end
        end
    end
    super.setTouch(self, touch)
end

function This:isFlexible()
    return self:hasGuideTouch()
end

function This:hasGuideTouch()
    return self.m_config and self.m_config.guide_id > 0
end

function This:getGuideTouchId()
    return self.m_config.guide_id
end

function This:getPriority()
    return self.m_config.priority
end

function This:isAuditHidden()
    if self.m_config and GamePlayer:isAuditVersion() then
        return self.m_config.audit == 1
    end
    return false
end

function This:getOpenLevel()
    return self.m_config.open_level
end

function This:getGroup()
    return self.m_config.group
end

function This:setNodeVisible(value)
    -- self.m_isVisible = value
    return super.setVisible(self, value)
end

-- function This:isVisible()
--     return self.m_isVisible
-- end

function This:getEffectName()
    return self.m_config and self.m_config.effect
end

function This:loadEffect(name)
    if not self.m_effect then
        self.m_effect = GameUtil:createEffect(name or self:getEffectName(), true, self)
        self:addChild(self.m_effect)
    end
end

function This:hasRedDotView()
    return self.m_config and self.m_config.reddot > 0
end

-- function This:isHighRedDotType()
--     return self.m_config and self.m_config.reddot > 1 
-- end

function This:getDefaultVisible()
    if self.m_config and self.m_config.visible then
        return self.m_config.visible > 0
    end
end

function This:refreshRedDot()
    if self:hasRedDotView() then
        local value = self:getRedDotValue()
        self:onRefreshRedDot(value)
    end
end

function This:onRefreshRedDot(paramValue)
    if self:hasRedDotView() then 
        local value = paramValue

        local visible = value
        local ret 
        if self:hasGuideTouch() then
            if GuideManager:isInGuide() then
                ret = GuideManager:getGuideStep() >= self:getGuideTouchId()
            else
                ret = true
            end
        end

        local level = GamePlayer:getLevel()
        local openlev = self:getOpenLevel() or 0
        if ret or openlev <= level  then
            visible = self:getDefaultVisible()
            if visible == nil then
                self:refreshEffect(value)
                return
            end
            
            if value ~= nil then
                self:setVisible(value)
                visible = value
            else
                self:setVisible(visible)
            end
        end
        
        self:refreshEffect(visible)
    end
end

function This:refreshEffect(visible)
    local effectName = self:getEffectName()
    if effectName and string.len(effectName) > 0 then
        if visible then
            self:loadEffect(effectName)
        else
            self:unloadEffect()
        end
    end
end

function This:unloadEffect()
    if self.m_effect then
        self.m_effect:removeFromParent()
        self.m_effect = nil
    end  
end

function This:setVisible(value)
    if self.m_behaviour then
        if self.m_config then
            self.m_behaviour:setButtonVisible(self:getPriority(), value)
        else
            self.m_behaviour:setSwitchVisible(value)
        end
    else
        super.setVisible(self, value)
    end
end

function This:getCurVisible()
    if self:isAuditHidden() then
        return false
    end

    local openLevel = self:getOpenLevel() or 0
    local hasRedDot = self:hasRedDotView()
    if openLevel <= 1 and not hasRedDot then
        -- print("btn " .. self:getName() .. "计算所得:true")
        return true
    end

    local ret = false
    if self:hasGuideTouch() then
        if GuideManager:isInGuide() then
            ret = GuideManager:getGuideStep() >= self:getGuideTouchId()
        else
            ret = true
        end
    end

    if not ret then
        local level = GamePlayer:getLevel()
        if openLevel <= level then
            local defaultVisible = self:getDefaultVisible()
            if hasRedDot and defaultVisible ~= nil then 
                local value = self:getRedDotValue()
                if value ~= nil then
                    ret = not (value == false)
                else
                    ret = defaultVisible
                end
            else
                ret = true
            end
        end
    end
    -- print("btn " .. self:getName() .. "计算所得:" .. tostring(ret))
    return ret
end

function This:isPlaying()
    return self.m_isPlaying
end

function This:adjustPosition( info )
    local position = info.pos
    local hasAnimation = info.animate
    local hasEffect = info.eff
    local callback = info.callback
    local factor = info.factor or 1
    local visible = info.visible

    if hasAnimation and hasEffect then
        self:playAnimation(factor, position.x, position.y, callback)
    elseif hasAnimation then
        local actionSeq, action
        if position then
            action = cc.MoveTo(0.85 * factor, position.x, position.y, "SineIn")
            if visible then
                local actionExtra = action
                action = cc.Sequence(actionExtra, cc.FadeIn(0.3 * factor))
            end

            if callback then
                actionSeq = cc.Sequence(action, cc.CallFunc(function()
                    callback()
                end))
            end
        else
            if visible then
                action = cc.FadeIn(0.3 * factor)
            else
                action = cc.FadeOut(0.85 * factor)
            end

            if callback then
                actionSeq = cc.Sequence(action, cc.CallFunc(function()
                    callback()
                end))
            end
        end
        self:runAction(actionSeq or action)
    else
        if position then
            self:setPosition(position.x, position.y)
        elseif visible ~= nil then
            self:setNodeVisible(not visible)
        end
        if callback then
            callback()
        end
    end
end

function This:playAnimation(factor, posx, posy, callback)
    self.m_isPlaying = true
    local parent = self:getParent()
    local effectParent = parent
    local x, y = parent:convertToNodeSpace(cc.visible.center.x, cc.visible.center.y)
    local effect = nil
    effect = GameUtil:createEffect("XinGongNeng", false, nil, {
        onAnimationComplete = function ()
            effect:removeFromParent()
            local particle = cc.Particle:create("tuowei.plist", self)
            particle:setPosition(x, y)

            local action = nil
            action = cc.Sequence(cc.MoveTo(factor * 0.85, posx, posy, "SineInOut"),
                cc.DelayTime(factor * 0.3),
                cc.CallFunc(function()
                    if callback then
                        callback()
                    end
                    self.m_isPlaying = false
                end),
                cc.RemoveAction()
            )
            particle:runAction(action)
            effectParent:addChild(particle)
            particle:setLocalZOrder(101)  

            action = cc.MoveTo(factor * 0.7 , posx, posy, "SineIn")
            self:runAction(action)
        end     
    })
    effect:setPosition(x, y)
    effectParent:addChild(effect)
    effect:setLocalZOrder(100)
    self:setNodeVisible(true)
    self:setLocalZOrder(102)
end

function This:playSequenceAnimation( targetList, factor, callback )
    local root = {}
    local pre = root
    for i, v  in ipairs(targetList) do
        local name = "MenuScene/boxLock" .. v .. ".png"
        if pre.name then
            local node = { name = name }
            pre.next = node
            pre = node
        else
            pre.name = name
        end
    end
    self:playAnimation2(root, factor, callback)
end

function This:playAnimation2( param, factor, callback)
    local parent = self:getParent()
    local target = cc.Sprite:create(param.name)
    local x, y = parent:convertToNodeSpace(cc.visible.center.x, cc.visible.center.y)

    local posx, posy = self:getPosition()
    local effectParent = parent
    local effect = nil
    effect = GameUtil:createEffect("XinGongNeng", false, nil, {
        onAnimationComplete = function ()
            effect:removeFromParent()
            local particle = cc.Particle:create("tuowei.plist", self)
            particle:setPosition(x, y)

            local action = nil
            action = cc.Sequence(cc.MoveTo(factor * 0.85, posx, posy, "SineInOut"),
                cc.DelayTime(factor * 0.3),
                cc.CallFunc(function()
                    if param.next then
                        self:playAnimation2( param.next, factor, callback)
                    elseif callback then
                        callback()
                    end
                end),
                cc.RemoveAction()
            )
            particle:runAction(action)
            effectParent:addChild(particle)
            particle:setLocalZOrder(101)  

            action = cc.Sequence(cc.MoveTo(factor * 0.6 , posx, posy, "SineIn"), cc.RemoveAction())
            target:runAction(action)
        end     
    }, false, "animation2")
    effect:setPosition(x, y)
    effectParent:addChild(effect)
    effect:setLocalZOrder(120)

    target:setPosition(x, y)
    target:setScale(0.7)
    effectParent:addChild(target)
    target:setLocalZOrder(121)
end

-- 汉献帝按钮从中间移动到指定位置
function This:newBtnAnimation( param, factor, callback)
    local parent = self:getParent()
    local x, y = parent:convertToNodeSpace(cc.visible.center.x, cc.visible.center.y)

    local posx, posy = self:getPosition()
    local effectParent = parent
    local effect = nil
    effect = GameUtil:createEffect("XinGongNeng", false, nil, {
        onAnimationComplete = function ()
            effect:removeFromParent()
            local particle = cc.Particle:create("tuowei.plist", self)
            particle:setPosition(x, y)

            local action = nil
            action = cc.Sequence(cc.MoveTo(factor * 0.85, posx, posy, "SineInOut"),
                cc.DelayTime(factor * 0.3),
                cc.CallFunc(function()
                    if callback then
                        callback()
                    end
                end),
                cc.RemoveAction()
            )
            particle:runAction(action)
            effectParent:addChild(particle)
            particle:setLocalZOrder(101)  

            action = cc.Sequence(cc.MoveTo(factor * 0.85 , posx, posy, "SineIn"))
            self:runAction(action)
        end     
    }, false, "animation2")
    self:setPosition(x, y)
    effect:setPosition(30, 35)
    self:addChild(effect,-1)
end

function This:refreshByEventHandler( handler )
    if self:hasRedDotView() then
        local openLevel = self:getOpenLevel()
        if openLevel then
            if self:hasGuideTouch() then
                if GuideManager:isInGuide() and GuideManager:getGuideStep() <= self:getGuideTouchId() then
                    local curLevel = GamePlayer:getLevel()
                    if curLevel < openLevel then
                        return
                    end
                end
            else
                local curLevel = GamePlayer:getLevel()
                if curLevel < openLevel then
                    return
                end
            end
        end

        if handler then
            handler()
        end
    end
end

function This:onRedDotChange( name, value )
    if name == self:getName() then
        self:refreshByEventHandler(function()
            self:onRefreshRedDot(value)
        end)
    end
end

function This:onGuideNextStep( entry )
    if GuideManager:isInGuide() and self:hasGuideTouch() and entry then
        if not self:isVisible() and entry:getId() >= self:getGuideTouchId() then
            self:setVisible(true)
        end
    end  
end

function This:getNoteName()
    return self.m_config.note or self:getName()
end
