﻿
local This, super = class(..., "LoadingScene")
local GameUtil = require("Util.GameUtil")
local HomeUI = require("Home.HomeUI")
local GuideManager = require("Guide.GuideManager")
local EventDispatcher = require("EventDispatcher")
local SlideScaleContaner = require("Action.SlideScaleContaner")
local TerritoryManager = require("Territory.TerritoryManager")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrRobberView = require("Territory.TerrRobberView")
local TerrBossRobberView = require("Territory.TerrBossRobberView")
local TerrBusinessManView = require("Territory.TerrBusinessManView")
local RecruitView = require("Recruit.RecruitView")
local TerrBuild = require("Territory.TerrBuild")
local AsynchEvent = require("Event.AsynchEvent")
local GameEvent = require("Event.GameEvent")
local IterateEvent = require("Event.IterateEvent")
local AchieveManager = require("Achieve.AchieveManager")
local GuideManager = require("Guide.GuideManager")
local RedDotManager = require("RedDot.RedDotManager")
local ArenaManager = require("Arena.ArenaManager")
local MatchManager = require("Match.MatchManager")
local WuDaoShopView = require("Territory.WuDaoShopView")
local GamePlayer = require("Player.GamePlayer")
local TerrRollView = require("Territory.TerrRollView")
local PeakContestManager = require("PeakContest.PeakContestManager")
local OfficialManager = require("Official.OfficialManager")
local CompetitionManager = require("Competition.CompetitionManager")

function This:init(info)
    super.init(self)
    self.m_isInit = false
    self.m_isTouchOnView = false
    self.m_curBuild = nil
    if info then
        self.m_initHandler = info.initHandler
    end
    self:setType(SceneType.kHomeScene)
    self.m_robberViews = {}
    self.m_bossRobberViews = {}
    self:initTask()
end

function This:initTask()
    local task = GameEvent:create(function()
        self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/MainBg.csb")
        if not self.m_arenaNode then
           self.m_arenaNode = TerrRollView:create(self.m_csNode.arenaMenu)
        end
        self:addChild(self.m_csNode)
        self.m_csNode:setPosition(-477, -742)       
    end)
    self:addTask(task)
    
    task = AsynchEvent:create(function()
        AchieveManager:onGetAchieveInfoRequest(AchieveType.kTianXiaDaShi)
    end,11900)
    self:addTask(task)

    task = GameEvent:create(function()
         self:showTerrRobbers(true)  --贼寇
         self:showTerrBossRobbers(true) -- boss
    end)
    self:addTask(task)

    task = GameEvent:create(function()
        self:initAction()
    end)
    self:addTask(task)

    local buildList = TerrBuildManager:getAllBuildings()
    for _, build in ipairs(buildList) do
        self:addTask(GameEvent:create(function()
            self:initTerrBuild(build)
        end))
    end

    task = GameEvent:create(function()
        self.m_ui = HomeUI:create()
        self.m_ui.name = "homeui"
        self:addChild(self.m_ui)
        g_weakTable[tostring(self.m_ui)] = self.m_ui
    end)
    self:addTask(task)

    task = GameEvent:create(function()
        TerrBuildManager:showOfflineReward()
    end)
    self:addTask(task)

    task = GameEvent:create(function()
        self.m_csNode.qizhi.name:setString(OfficialManager:getCountryName(),23)
        OfficialManager:showEditPanel()
    end)
    self:addTask(task)

    task = GameEvent:create(function()
        MatchManager:showBattleEndReward()
    end)
    self:addTask(task)

    task = GameEvent:create(function()
		self:refreshArenaPlayerInfo()
	end)
	self:addTask(task)

    task = GameEvent:create(function()
        self:updateFirstGroup()
    end)
    self:addTask(task)    

    task = GameEvent:create(function()
        local count = collectgarbage("count")
        print("垃圾收集:",count)
        collectgarbage("collect")
    end)
    self:addTask(task)

    task = GameEvent:create(function()
        self.m_timer:setTimer(0.2,function ()
            self:checkDelayDialog()
        end,"init_delay_dialog")
    end)
    self:addTask(task)

    task = GameEvent:create(function()
        if GamePlayer:getLevel() >= 50 then 
            CompetitionManager:showInvite()
        end
    end)
    self:addTask(task)

end

function This:loadFinish()
    super.loadFinish(self)
    self:initTouchEvent()
    if self.m_initHandler then
        self.m_initHandler()
    end
    cc.Application:addListener(self)
    TerritoryManager:addObserver(self)
    cc.Audio:playBackground("Bg_Fengdi.mp3")
    EventDispatcher:dispatcher("menu_scene_enter")
    self.m_isInit = true
end

function This:isInit()
    return self.m_isInit
end

function This:onEnter()
    super.onEnter(self)
    TerritoryManager:addObserver(self)
    PeakContestManager:addObserver(self)
end

function This:onExit()
    self.m_isInit = false
    TerritoryManager:removeObserver(self)
    PeakContestManager:removeObserver(self)
    super.onExit(self)
    cc.Application:removeListener(self)
    cc.Audio:stopBackground()
    EventDispatcher:dispatcher("menu_scene_exit")
	GuideManager:cancelAncGuider()
end

function This:removerDialog(dialog)
	local Dialog = require("Dialog")
    if Dialog.removerDialog(self, dialog) then
        self:resumeGame()
    end
	if #self.m_dialogList <= 0 then
		self:checkDelayDialog()
	end
end

function This:onKeyDown(keyCode)
    print(keyCode)
    if keyCode == "F1" then
		-- g_scene:openDialog("Arena.ArenaPanel")
    elseif keyCode == "F2" then
        local node = RecruitView:create()
        self:addChild(node)
        node:showObtainEffect({heroId = 10013 , isGot = true})
    elseif keyCode == "F3" then
        local node = RecruitView:create()
        self:addChild(node)
        node:showObtainEffect({heroId = 10013 , isGot = false})
    elseif keyCode == "F6" then
        if self.m_curBuild then
            self.m_curBuild.m_build:requestUpgrade()
        end
    elseif keyCode == "F4" then
        g_scene:openDialog("Activity.ActivityPlayerPanel")
    end
end

function This:initAction()
    if not self.m_isInit then
        -- local width , height = self.m_csNode.Main:getContentSize()
        local width , height = 4022*0.65, 2482*0.65
        self.m_csNode:setContentSize(width , height)
        self.m_slideScaleContaner = SlideScaleContaner:create({
            node = self.m_csNode,
            maxWidth =  width,
            maxHeight = height,
            minWidth = cc.visible.width,
            minHeight = cc.visible.height,
        })

        self.m_slideScaleContaner:setScaleHandler(function ( scaleValue )
            local value = 1/scaleValue
            for i=1, 14 do
                self.m_csNode["menuView" .. tostring(i)]:setScale(0.85 * value)
            end
        end)
    end
end
function This:initTerrBuild(build)
    if not build then return end
    local index = build:getConfigId()
    local menuView = build:createView()
    local buildView = self.m_csNode["building" .. index]
    local clickView = self.m_csNode["click" .. index]
    if buildView then
        menuView:setPositionBy(buildView)
    end
    menuView:setClickView(clickView)
    menuView:setSelectHandler(function(view)
        self:onTouchView(view)
    end)
    if clickView then
        clickView:setTouch({
            onEnded = function()
                -- self:focuseBuild(index)
                self:onTouchView(menuView)
            end,
            isAlpha = false,
            isScale = false,
            isSwallow = false,
            isDynamic = true,
    		name = "clickview"
        })
    end
    self.m_csNode:addChild(menuView)
    self.m_csNode["menuView" .. tostring(index)] = menuView
    menuView:setName("menuView" .. tostring(index))
end

function This:refreshArenaPlayerInfo()
    if self.m_arenaNode then
        self.m_arenaNode:refresh()
    end
end

function This:onTouchView(view)
    if view then
        if self.m_curBuild then
            self.m_curBuild:setSelect(false)
        end
        view:setSelect(true)
        self.m_curBuild = view
        self.m_isTouchOnView = true
        self.m_isHasSelfFuc = false

        local buildList = TerrBuildManager:getAllBuildings()
        for _,build in ipairs(buildList) do
            local index = build:getConfigId()
            local buildView = self.m_csNode["menuView" .. tostring(index)]
            if self.m_curBuild == buildView then
                self.m_isHasSelfFuc = build:getSelfFuc()
                break
            end
        end
        
        for _, build in ipairs(buildList) do
            local index = build:getConfigId()
            local buildView = self.m_csNode["menuView" .. tostring(index)]
            if buildView and self.m_curBuild ~= buildView and self.m_isHasSelfFuc then
                buildView:hide()
            end
        end
        self.m_isInHidden = true
    else
        if not self.m_isTouchOnView and self.m_curBuild then
            if self.m_isInHidden then
                local buildList = TerrBuildManager:getAllBuildings()
                for _, build in ipairs(buildList) do
                    local index = build:getConfigId()
                    local buildView = self.m_csNode["menuView" .. tostring(index)]
                    if buildView then
                        buildView:show()
                    end
                end
                self.m_isInHidden = false
            end
            self.m_curBuild:setSelect(false)
            self.m_curBuild = nil
        end
        self.m_isTouchOnView = false
    end
end

function This:initTouchEvent()
    self:addTouchListener({
        onMoved = function()
            self:onTouchView()  
        end,
        onEnded = function()
            self:onTouchView()
        end,
        isSwallow = false,
    })

    -- self.m_csNode.arena:setTouch({
    --     onEnded = function()
    --         if ArenaManager:isArenaOpened(true) then
    --             g_scene:openDialog("Arena.ArenaPanel")
    --         end
    --     end,
    --     isDynamic = true,
    --     isSwallow = false,
    --     name = "arena"
    -- })
end

function This:setUIVisible(isVisible)
	if self.m_ui then
		self.m_ui:setVisible(isVisible)
	end
end

function This:stopMove()
	self.m_csNode:stopAllActions()
end

-------------------------------------------------------

function This:getBuildNode(type,index)
    local buildIndex = index or 1
    local buildId = TerrBuild.getConfigIdBy(type, buildIndex)
    local node = self.m_csNode["click" .. tostring(buildId)]
    return node
end

function This:showTerrRobbers(isIniting)
    local shangRen = TerrBusinessManView:create()
    self.m_csNode.shangren:addChild(shangRen)

	local wudaoView = WuDaoShopView:create()
	self.m_csNode.wudaoshangren:addChild(wudaoView)

    local robbers = TerritoryManager:getAllRobbers()
    if table.count(robbers) <= 0 then
        self.m_robberViews = {}
        return
    end

    local removeIndexLists = {}
    for pos, view in pairs(self.m_robberViews) do
        local robber = robbers[pos]
        if not robber then
            table.insert(removeIndexLists, pos)
        else
            view:refresh()
        end
    end

    for _, pos in ipairs(removeIndexLists) do
        self.m_robberViews[pos] = nil
    end

    for pos, robber in pairs(robbers) do
        local view = self.m_robberViews[robber:getId()]
        if not view then
            view = TerrRobberView:create(robber)
            self.m_csNode["zeikou" .. pos]:addChild(view)
            self.m_robberViews[robber:getId()] = view
        end
    end
end

function This:showTerrBossRobbers(isIniting)
    local robbers = TerritoryManager:getAllBossRobbers()
    if table.count(robbers) <= 0 then
        self.m_bossRobberViews = {}
        return
    end

    local removeIndexLists = {}
    for pos, view in pairs(self.m_bossRobberViews) do
        local robber = robbers[pos]
        if not robber then
            table.insert(removeIndexLists, pos)
        else
            view:refresh()
        end
    end

    for _, pos in ipairs(removeIndexLists) do
        self.m_bossRobberViews[pos] = nil
        self.m_csNode.kingboss[tostring(pos)]:removeAllChildren()
    end

    for pos, robber in pairs(robbers) do
        local view = self.m_bossRobberViews[robber:getId()]
        if not view then
            view = TerrBossRobberView:create(robber)
            self.m_csNode.kingboss[tostring(pos)]:addChild(view)
            self.m_bossRobberViews[robber:getId()] = view
        else
            view:updateInfo(robber)
        end
    end
    
end

function This:focuseBuildByType( buildType, index, handler,bAction )
    local buildIndex = index or 1
    local buildId = TerrBuild.getConfigIdBy(buildType, buildIndex)
    self:focuseBuild(buildId, bAction, handler)
end

function This:focuseBuild( buildId, bAction, handler )
    self:focuseNode( "menuView" .. tostring(buildId),buildId, bAction, handler )
end

function This:focuseRobber( posId )
    self:focuseNode( "zeikou" .. tostring(posId), nil,true)
end

function This:focuseNode( name, buildId, bAction, handler )
    local node = self.m_csNode[name]
    if node then
        local x , y = node:convertToWorldSpace(0,0)
        local dx , dy = cc.visible.center.x - x , cc.visible.center.y - y
        local lx , ly = self.m_csNode:getPosition()
        local boundRect = self.m_csNode:getBoundingBox()

        local maxx = 0
        local minx = cc.visible.width - boundRect.width

        if lx + dx > maxx then
            dx = maxx - lx
        elseif lx + dx < minx then
            dx = minx - lx
        end

        local maxy = 0
        local miny = cc.visible.height - boundRect.height

        if ly + dy > maxy then
            dy = maxy - ly
        elseif ly + dy < miny then
            dy = miny - ly
        end

        local distance = dx * dx + dy * dy
        if distance < 200 then
            if handler then handler(node) end
            EventDispatcher:dispatcher("menu_scene_focuse", buildId)
            return 
        end

        if bAction then
            if not handler then
                self.m_csNode:runAction(cc.MoveTo(0.5 * 10000 / distance, lx + dx, ly + dy, "SineIn"))
            else
                self.m_csNode:runAction(cc.Sequence(
                    cc.MoveTo(0.5, lx + dx, ly + dy, "SineIn"), 
                    cc.DelayTime(0.3),
                    cc.CallFunc(function()
                        if handler then handler(node) end
                        EventDispatcher:dispatcher("menu_scene_focuse", buildId)
                    end)))
            end
        else
            self.m_csNode:setPosition(lx + dx, ly + dy)
            if handler then handler(node) end
            EventDispatcher:dispatcher("menu_scene_focuse", buildId)
        end
        
    end
end

function This:update(dt)
    super.update(self,dt)
    if self.m_isInit then
        self.m_slideScaleContaner:update(dt)
    end
end

function This:onGetRobbersInfo()
    self:showTerrRobbers()
    self:showTerrBossRobbers()
    g_scene:showTips(Serialize[61499])
end

function This:updateFirstGroup()
    local groudName = PeakContestManager:getFirstGroupName()
    if not groudName or groudName == "" then
        self.m_csNode.firstGroud.text:setString(Serialize[60736])
    else 
        self.m_csNode.firstGroud.text:setString(groudName)
    end
end

function This:refreshBtnHelp()
    if self.m_ui then
        self.m_ui:refreshBtnHelp()
    end
end