
local This , super = class(...,"NodeUI")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local MatchingConfig = require("Metadatas.MatchingConfig")
local FamilyPowerConfig = require("Metadatas.FamilyPowerConfig")


-- 邀请间隔
local InviteInterval= 10


function This:initData(data)
    self.m_data = data
    self.m_inviteHandler = nil
    self.m_refreshHandler = nil
    self.m_isUpdate = false
end

function This:initNode(data)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TeamMatchingFriendList.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local w, h = csNode.bg:getContentSize()
    csNode:setPosition(w/2, h/2)
    self:setContentSize(w, h)
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        self.m_csNode.button:setTouch({
            isDynamic = true, 
            isContainer = true,
            onEnded = function ()
                if type(self.m_inviteHandler) == "function" then
                    self.m_inviteHandler(self)
                end
            end,
        })
    end    
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
    self:scheduleUpdate()
end

function This:setModel(data)
    self.m_data = data
    self:refresh()
end

function This:refresh()
    if type(self.m_refreshHandler) == "function" then
        self.m_refreshHandler(self)
    else
        local otype = self.m_data.type

        if otype == "friend" then
            self:refreshFriend()
        elseif otype == "family" then
            self:refreshFamily()
        end
    end
end

function This:refreshFriend()
    local csNode = self.m_csNode
    local friend = self.m_data.obj
    
    GameUtil:setPlayerIconNodeImage(csNode.icon, friend:getIcon())
    GameUtil:setPlayerEffectNodeTo(csNode.icon, friend:getEffect())
    
    csNode.yiyaoqing:setVisible(false)
    csNode.gaming:setVisible(false)
    csNode.name:setString( friend:getName() )
    csNode.level:setString("") -- 好友列表没有level
    csNode.online:setVisible( friend:isOnline() )
    csNode.outline:setVisible(not friend:isOnline())

    local cd = self:getInviteCd()
    self.m_isUpdate = (cd > 0)
    csNode.button:setVisible(not self.m_isUpdate)
    csNode.cd:setVisible(self.m_isUpdate)
    csNode.cd:setString(string.format(Serialize[61501], cd))

    local rank = friend:getRankLv()
    rank = rank <= 0 and 1 or rank
    csNode.office:setString(MatchingConfig[rank].rank_level)
end

function This:refreshFamily()
    local csNode = self.m_csNode
    local member = self.m_data.obj

    csNode.icon:removeAllChildren()
    GameUtil:setPlayerIconNodeImage(csNode.icon, member:getIcon())

    csNode.yiyaoqing:setVisible(false)
    csNode.gaming:setVisible(false)
    csNode.name:setString( member:getName() )
    csNode.level:setString( string.format("Lv.%d", member:getLevel()) )
    csNode.online:setVisible( member:isOnline() )
    csNode.outline:setVisible(not member:isOnline())

    local cd = self:getInviteCd()
    self.m_isUpdate = (cd > 0)
    csNode.button:setVisible(not self.m_isUpdate)
    csNode.cd:setVisible(self.m_isUpdate)
    csNode.cd:setString(string.format(Serialize[61501], cd))

    local text = member:getTitleDesc()
    csNode.office:setString(text)        
end

function This:setInviteHandler(func)
    self.m_inviteHandler = func
end

function This:setRefreshHandler(func)
    self.m_refreshHandler = func
end

function This:getInviteCd()
    local lastInviteTime = self.m_data.lastInviteTime or 0
    local curTime = GamePlayer:getServerTime()
    local delta = curTime - lastInviteTime

    if delta >= InviteInterval then
        return 0
    end
    return math.floor(InviteInterval - delta)
end

function This:update(deltaTime)
    super.update(self, deltaTime)

    if not self.m_isUpdate then
        return
    end

    local csNode = self.m_csNode
    local cd = self:getInviteCd()
    if cd > 0 then
        csNode.button:setVisible(false)
        csNode.cd:setVisible(true)
        csNode.cd:setString(string.format(Serialize[61501], cd))
    else
        csNode.button:setVisible(true)
        csNode.cd:setVisible(false)
        csNode.cd:setString("")
        self.m_isUpdate = false
    end
end

function This:getData()
    return self.m_data
end

function This:getId()
    return self.m_data.obj:getId()
end