

local This , super = class(...,"NodeUI")
local FriendManager = require("Friend.FriendManager")
local FamilyManager = require("Family.FamilyManager")
local GameUtil = require("Util.GameUtil")
local RankPageListLayer = require("Rank.RankPageListLayer")
local GamePlayer = require("Player.GamePlayer")


local InviteListState = {
	kClose = 0,
	kOpen = 1,
	kRunAni = 2,
}


function This:initData(info)
    info = info or {}

    self.m_csNode = info.csNode
    self.m_isMaskLayer = info.isMaskLayer
    self.m_itemWidth = info.itemWidth or 255
    self.m_itemHeight = info.itemHeight or 90
    self.m_itemFile = info.itemFile or "Invite.InviteListItemView"
	self.m_onInvite = info.onInvite
	self.m_onWorldInvite = info.onWorldInvite
	self.m_onSelector = info.onSelector
	self.m_onSort = info.onSort
	self.m_onRefresh = info.onRefresh

	self.m_listName = ""
	self.m_maskLayer = false
	self.m_state = InviteListState.kClose
	self.m_record = {}
	self.m_lastFriendReq = 0
	self.m_lastFamilyReq = 0
end

function This:initNode(info)
	local csNode = self.m_csNode
    local node = csNode.invite.node
    csNode.invite:setLocalZOrder(1)

    if self.m_isMaskLayer then
    	self.m_maskLayer = cc.LayerColor:create(cc.Color(0, 0, 0, 0))
    	self.m_maskLayer:ignoreAnchorPointForPosition(false)
    	self.m_maskLayer:setAnchorPoint(0.5, 0.5)
    	self.m_maskLayer:setPosition(0, 0)
    	csNode:addChild(self.m_maskLayer)
    end

    local scrollView = node.scrollView
    local itemView = require(self.m_itemFile)
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = self.m_itemWidth,
        itemHeight = self.m_itemHeight,
        spaceX = 0,
        spaceY = 0,
        column = 1,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getList()	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getList()
        	local data = list[index]
        	if data then
        		local id = data.obj:getId()
        		data.lastInviteTime = self.m_record[id] or 0
			    if not node then
			    	node = itemView:create(data, index)
			    	node:setInviteHandler(function ()
			    		self:onInviteClick(node)
			    	end)
			    	node:setRefreshHandler(self.m_onRefresh)
				    return node
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)


    self:addTabButton("friend", node.friend, function ()
    	self.m_listName = "friend"
    	self.m_listLayer:setDataList()
    	self:onDataRequest()
    end)

    self:addTabButton("family", node.family, function ()
    	self.m_listName = "family"
    	self.m_listLayer:setDataList()
    	self:onDataRequest()
    end)

    self:setState(self.m_state)
end

function This:initTouchEvent()
	local csNode = self.m_csNode

	if csNode.invite.node.worldInvite then
		csNode.invite.node.worldInvite:setTouchEnded(function()
			self:onWorldInviteClick()
		end)
	end

    if self.m_maskLayer then
        self.m_maskLayer:setTouch({
	        isScale = false, 
	        isAlpha = false,
	        onEnded = function()
	            print("click invite mask layer")
	            self:runCloseAni()
	        end
	    })
    end
end

-- 好友列表
function This:initFriendList()
	local ret = {}

	local list = FriendManager.m_chatFriend:getNodeList(function(friend)
		return friend:isFriend()
	end)

	for _, friend in ipairs(list) do
		local isAdd = true
		local t = {}
		t.type = "friend" 
		t.obj = friend -- Friend.lua

		if type(self.m_onSelector) == "function" then
			isAdd = self.m_onSelector(t)
		end

		if isAdd then
			table.insert(ret, t)
		end
	end

	if #ret > 1 then
		if type(self.m_onSort) == "function" then
			table.sort(ret, function (a, b)
				return self.m_onSort("friend", a, b)
			end)
		else
			table.sort(ret, function (a, b)
				local objA = a.obj
				local objB = b.obj

				local onlineA = (objA:isOnline() == true) and 1 or 0
				local onlineB = (objB:isOnline() == true) and 1 or 0
				if onlineA ~= onlineB then
					return onlineA > onlineB
				end

				local levelA = objA:getLevel()
				local levelB = objB:getLevel()
				if levelA ~= levelB then
					return levelA > levelB
				end

				return objA:getId() < objB:getId()
			end)
		end
	end

	self.m_friendList = ret	
end

-- 家族列表
function This:initFamilyList()
	local ret = {}
	local family = FamilyManager:getMyFamily()

	if family then
		local list = family.m_memberList:getAllNodeList()
		for _, member in ipairs(list) do
			local isAdd = true
			local t = {}
			t.type = "family" 
			t.obj = member -- FamilyMemberModel.lua

			if type(self.m_onSelector) == "function" then
				isAdd = self.m_onSelector(t)
			end

			if isAdd then
				table.insert(ret, t)
			end
		end

		if type(self.m_onSort) == "function" then
			table.sort(ret, function (a, b)
				return self.m_onSort("family", a, b)
			end)
		else
			table.sort(ret, function(a, b)
				local objA = a.obj
				local objB = b.obj

			    local onlineA = (objA:isOnline() == true) and 1 or 0
			    local onlineB = (objB:isOnline() == true) and 1 or 0
			    if onlineA ~= onlineB then
			    	return onlineA > onlineB
			    end

			    local titleA = objA:getTitle()
			    local titleB = objB:getTitle()
			    if titleA ~= titleB then
			        return titleA < titleB
			    end

			    return objA:getId() < objB:getId()
			end)
		end
	end

	self.m_familyList = ret	
end

function This:getList()
	if self.m_listName == "friend" then
		return self:getFriendList()
	elseif self.m_listName == "family" then
		return self:getFamilyList()
	end
	return {}
end

function This:getFriendList()
	if not self.m_friendList then
		self:initFriendList()
	end
	return self.m_friendList
end

function This:getFamilyList()
	if not self.m_familyList then
		self:initFamilyList()
	end
	return self.m_familyList
end

function This:onEnter()
    super.onEnter(self)
    FriendManager:addObserver(self)
    FamilyManager:addObserver(self)
end

function This:onExit()
	FriendManager:removeObserver(self)
    FamilyManager:removeObserver(self)
    super.onExit(self)
end

function This:onInviteClick(node)
	if type(self.m_onInvite) == "function" then
		self.m_onInvite({node = node})

		local curTime = GamePlayer:getServerTime()
		local data = node:getData()
		local id = node:getId()
		print("onInviteClick " .. id)
		self.m_record[id] = curTime
		data.lastInviteTime = curTime
		node:refresh()
	end
end

function This:onWorldInviteClick()
	if type(self.m_onWorldInvite) == "function" then
		self.m_onWorldInvite()
	end
end

function This:runOpenAni()
	if self.m_state == InviteListState.kRunAni or self.m_state == InviteListState.kOpen then
		return
	end

	local invite = self.m_csNode.invite
	local w, h = invite:getContentSize()

	invite.node:setPosition(w, 0)
    invite.node:runAction(cc.Sequence(cc.MoveTo(0.2, 0, 0, "ExponentialIn"), cc.CallFunc(function()
		self:setState(InviteListState.kOpen)
	end)))
	self.m_state = InviteListState.kRunAni
end

function This:runCloseAni()
	if self.m_state == InviteListState.kRunAni or self.m_state == InviteListState.kClose then
		return
	end

	local invite = self.m_csNode.invite
	local w, h = invite:getContentSize()

	invite.node:setPosition(0, 0)
    invite.node:runAction(cc.Sequence(cc.MoveTo(0.2, w, 0, "ExponentialIn"), cc.CallFunc(function()
		self:setState(InviteListState.kClose)
	end)))
	self.m_state = InviteListState.kRunAni
end

-- 0关闭，1开启
function This:setState(state)
	local invite = self.m_csNode.invite
	local w, h = invite:getContentSize()

	if state == InviteListState.kClose then -- 关闭
		if self.m_maskLayer then
			self.m_maskLayer:setVisible(false)
		end
		invite.node:setPosition(w, 0)
		self.m_state = InviteListState.kClose

	elseif state == InviteListState.kOpen then -- 开启
		if self.m_maskLayer then
			self.m_maskLayer:setVisible(true)
		end
		invite.node:setPosition(0, 0) 
		self.m_state = InviteListState.kOpen
		self:selectTabButton("friend")
		self:onDataRequest()

	end
end

function This:onDataRequest()
	local curTime = GamePlayer:getServerTime()

	if self.m_listName == "friend" and curTime - self.m_lastFriendReq >= 10 then
		self.m_lastFriendReq = curTime
		FriendManager:onInitFriendInfoRequest()
		
	elseif self.m_listName == "family" and FamilyManager:hasFamily() and curTime - self.m_lastFamilyReq >= 10 then		
		self.m_lastFamilyReq = curTime
		FamilyManager:onGetMyFamilyInfoRequest()

	end
end

function This:onFriendChanged()
	self:initFriendList()
	if self.m_listName == "friend" then
		self.m_listLayer:setDataList()
	end
end

function This:onGetMyFamilyInfo()
	self:initFamilyList()
	if self.m_listName == "family" then
		self.m_listLayer:setDataList()
	end
end

function This:reload()
	self:initFriendList()
	self:initFamilyList()
	self.m_listLayer:setDataList()
end

function This:addTabButton(name, button, onSelected)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end
    if self.m_tabButtons[name] then
        return
    end
    self.m_tabButtons[name] = button
    if not button.m_touch then
        button:setTouch({
            isScale = false,
            isSwallow = true,
            onEnded = function ()
                self:selectTabButton(name)
            end
        })
    end
    button.onSelected = onSelected
end