
-- 军团活跃boss数据

local This , super = class(..., "Base.BaseModel")
local LegionActiveItemConfig = require("Metadatas.LegionActiveItemConfig")
local LegionActivePoolConfig = require("Metadatas.LegionActivePoolConfig")
local LegionActiveBossConfig = require("Metadatas.LegionActiveBossConfig")
local FamilyManager = require("Family.FamilyManager")
local GamePlayer = require("Player.GamePlayer")


function This:init()
	super.init(self)

	self.m_isOpen = false --是否开启
	self.m_curActive = 0 --当前活跃
	self.m_maxActive = 1 --需要的总活跃
	self.m_useOpenNum = 0 --当前已使用的周开启次数
	self.m_maxOpenNum = 1 --最大周开启次数
	self.m_useMyPlayNum = 0 --当前已使用我的挑战次数
	self.m_maxMyPlayNum = 1 --我的最大挑战次数
	self.m_allPeople = 0 --总人数
	self.m_allDamage = 0 --总伤害
	self.m_rewardLv = 1 --奖池等级
	self.m_rewardPool = {} --奖池里面的物品
	self.m_rewardLeftNum = 0 --奖励剩余
	self.m_rewardAllNum = 0	--奖励总数
	self.m_luckyDrawTimesList = {} --抽奖次数排名
	self.m_leftFreeNum = 0 --当前免费抽奖次数
	self.m_maxFreeNum = 0 --最大免费抽奖次数
	self.m_record = {} --抽奖记录
	self.m_usedHeroList = {} --已使用的卡牌列表
	self.m_report = nil --战报
	self.m_rewardId = 0 --抽中的物品
	self.m_phase = self:calculatePhase() --阶段
end

-- 军团的信息
function This:handlerLegionAbossInfoMsg(msg)
	self.m_isOpen = msg.isopen
	self.m_curActive = msg.active_num
	self.m_useOpenNum = msg.week_challeng_times
	self.m_allPeople = msg.challenge_num
	self.m_allDamage = msg.all_damage

	-- 活跃和周挑战次数
	local activePoint = LegionActiveBossConfig[1].active_point
	self.m_maxOpenNum = #activePoint
	self.m_maxActive = activePoint[self.m_useOpenNum+1] or activePoint[self.m_maxOpenNum]

	-- 奖池
	self.m_rewardLv = 1
	self:updateRewardPool(msg.jackpot)
	if #self.m_rewardPool > 0 then
		self.m_rewardLv = self.m_rewardPool[1].rewardLv
	end

	-- 抽奖次数排名
	self:updateLuckyDrawTimesList(msg.luckdarytimes_list)
end

-- 个人的信息
function This:handlerLegionAbossRoleInfoMsg(msg)
    self.m_leftFreeNum = msg.left_free_times
    self.m_useMyPlayNum = msg.challenge_times
    self.m_usedHeroList = msg.used_card_list
    
    -- 最大挑战次数
    local abossConfig = LegionActiveBossConfig[1]
    self.m_maxMyPlayNum = abossConfig.challenge_times

    -- 最大免费抽奖次数
    local myInfo = FamilyManager:getMySelfMemeberInfo()
    self.m_maxFreeNum = 1
    if myInfo then
    	local title = myInfo:getTitle()
    	for i, v in ipairs(abossConfig.position_times) do
    		if v[1] == title then
    			self.m_maxFreeNum = v[2]
    			break
    		end
    	end
    end

    -- 抽奖记录
    self:updateRecord(msg.record)
end

-- 挑战
function This:handlerLegionAbossChallengeMsg(msg)
	self.m_report = msg.report
	self.m_leftFreeNum = msg.left_free_times
	self.m_allPeople = msg.challenge_num
	self.m_allDamage = msg.all_damage	
	self.m_useMyPlayNum = msg.challenge_times
	self.m_usedHeroList = msg.used_card_list
end

-- 抽奖
function This:handlerLegionAbossLuckyDrawMsg(msg)
	self.m_leftFreeNum = msg.left_free_times
	self.m_rewardId = msg.jackpot_id
	self:updateRewardPool(msg.jackpot)
	self:updateRecord(msg.record)
	self:updateLuckyDrawTimesList(msg.luckdarytimes_list)
end

-- 更新奖池
function This:updateRewardPool(jackpot)
	jackpot = jackpot or {}
	self.m_rewardPool = {}
	self.m_rewardLeftNum = 0
	self.m_rewardAllNum = 0

	for i, v in ipairs(jackpot) do
		local id = v.id
		local config = LegionActiveItemConfig[id]
		local t = {}
		t.id = id
		t.remainNum = v.count
		t.totalNum = config.count
		t.itemId = config.item_id	
		t.itemNum = config.num	
		t.rewardLv = config.reward_pool
		t.isRare = (config.if_rare == 1)
		table.insert(self.m_rewardPool, t)

		self.m_rewardLeftNum = self.m_rewardLeftNum + t.remainNum
		self.m_rewardAllNum = self.m_rewardAllNum + t.totalNum
	end
end

-- 更新抽奖记录
function This:updateRecord(record)
	record = record or {}
    self.m_record = {}

    for i, v in ipairs(record) do
    	local config = LegionActiveItemConfig[v.reward_id]
    	local t = {}
    	t.time = v.time
    	t.name = v.name
    	t.itemId = config.item_id
    	t.itemNum = config.num
    	table.insert(self.m_record, t)
    end
    if #self.m_record > 1 then
    	table.sort(self.m_record, function (a, b)
    		return a.time > b.time 
    	end)
    end    
end

-- 更新抽奖次数排名
function This:updateLuckyDrawTimesList(timesList)
	timesList = timesList or {}
	self.m_luckyDrawTimesList = {}

	for i, v in ipairs(timesList) do
		local t = {}
		t.roleId = v.id
		t.name = v.name
		t.rank = v.rank
		t.icon = v.icon
		t.level = v.level
		t.times = v.times
		table.insert(self.m_luckyDrawTimesList, t)
	end
	if #self.m_luckyDrawTimesList > 1 then
		table.sort(self.m_luckyDrawTimesList, function (a, b)
			return a.rank < b.rank
		end)
	end	
end

-- 是否开启
function This:getIsOpen()
	return self.m_isOpen
end

-- 当前活跃
function This:getCurActive()
	return self.m_curActive
end

-- 最大活跃
function This:getMaxActive()
	return self.m_maxActive
end

-- 剩余开启次数(一周多少次)
function This:getLeftOpenNum()
	local num = self.m_maxOpenNum - self.m_useOpenNum
	if num < 0 then
		num = 0
	end
	return num
end

-- 总的开启次数（一周多少次）
function This:getMaxOpenNum()
	return self.m_maxOpenNum
end

-- 我的剩余挑战次数
function This:getMyLeftPlayNum()
	local num = self.m_maxMyPlayNum - self.m_useMyPlayNum
	if num < 0 then
		num = 0
	end
	return num
end

-- 我的最大挑战次数
function This:getMyMaxPlayNum()
	return self.m_maxMyPlayNum
end

-- 奖池等级
function This:getRewardPoolLv()
	return self.m_rewardLv
end

-- 挑战人数
function This:getAllPeople()
	return self.m_allPeople
end

-- 总伤害
function This:getAllDamage()
	return self.m_allDamage
end

-- 奖励剩余
function This:getRewardLeftNum()
	return self.m_rewardLeftNum
end

-- 奖励总数
function This:getRewardAllNum()
	return self.m_rewardAllNum
end

-- 奖品
function This:getRewardPool()
	return self.m_rewardPool
end

-- 剩余免费抽奖次数
function This:getLeftFreeNum()
	return self.m_leftFreeNum
end

-- 最大免费抽奖次数
function This:getMaxFreeNum()
	return self.m_maxFreeNum
end

-- 抽奖记录
function This:getRecord()
	return self.m_record
end

-- 奖品
function This:getRewardId()
	return self.m_rewardId
end

-- 抽奖排名
function This:getLuckyDrawTimesList()
	return self.m_luckyDrawTimesList
end

-- 开启的时间段
function This:getCanOpenTime()
	local curTime =	GamePlayer:getServerTime()
	local date = os.serverdate("*t", curTime)
	local zeroTime = os.time({year = date.year, month = date.month, day = date.day, hour = 0, min = 0, sec = 0})
	local config = LegionActiveBossConfig[1]
	local info = {
		startTime = config.open_time + zeroTime,
		endTime = config.end_time + zeroTime
	}
	return info
end

-- 免费抽奖的时间段
function This:getCanFreeLuckyDrawTime()
	local curTime =	GamePlayer:getServerTime()
	local date = os.serverdate("*t", curTime)
	local zeroTime = os.time({year = date.year, month = date.month, day = date.day, hour = 0, min = 0, sec = 0})
	local config = LegionActiveBossConfig[1]	
	local info = {
		startTime = config.end_time + zeroTime, -- 挑战结束就是抽奖的开始
		endTime = 86400 + zeroTime
	}
	return info
end

-- 付费抽奖的时间段
function This:getCanMoneyLuckyDrawTime()
	local curTime =	GamePlayer:getServerTime()
	local date = os.serverdate("*t", curTime)
	local zeroTime = os.time({year = date.year, month = date.month, day = date.day, hour = 0, min = 0, sec = 0})
	local config = LegionActiveBossConfig[1]	
	local info = {
		startTime = config.yuanbao_cost_time + zeroTime,
		endTime = 86400 + zeroTime
	}
	return info
end

-- 用过的武将
function This:getUsedHeroList()
 	return self.m_usedHeroList
end

function This:formatTimestamp(time)
	local date = os.serverdate("*t", time)
	local zeroTime = os.time({year = date.year, month = date.month, day = date.day, hour = 0, min = 0, sec = 0})
	local second = time - zeroTime
	local hour = math.floor(second/3600)
	local min = math.floor((second%3600)/60)
	return string.format("%d:%02d", hour, min)
end

-- 解锁等级
function This:getUnlockLv()
	local config = LegionActiveBossConfig[1]
	return config.open_level
end

-- 检查解锁
function This:checkUnlock(isTips)
	local isHas = FamilyManager:hasFamily()
	if not isHas then
		if isTips then
			g_scene:showTips(Serialize[61508])
		end
		return false
	end

	local needLv = self:getUnlockLv()
	local isUnlock = GamePlayer:getLevel() >= needLv
	if isTips and not isUnlock then
		g_scene:showTips(string.format(Serialize[61509], needLv))
	end
	return isUnlock
end

-- 购买次数花费的元宝
function This:getMoneyLuckyDrawCost()
	local num = LegionActiveBossConfig[1].yuanbao_cost
	return num
end

function This:getCanOpenTitleList()
	return {FamilyTitle.kMengZhu, FamilyTitle.kFuMengZhu}
end

function This:getOpenTitlePowerTips()
	return Serialize[61510]
end

function This:getOpenTimeTips()
	local config = LegionActiveBossConfig[1]
	local openH = math.floor(config.open_time/3600)
	local openM = math.floor(config.open_time%3600/60)
	local endH = math.floor(config.end_time/3600)
	local endM = math.floor(config.end_time%3600/60)

	return string.format(Serialize[61511], openH, openM, endH, endM)
end

-- 是否有开启权限
function This:isHaveOpenPower()
	local myInfo = FamilyManager:getMySelfMemeberInfo()
	if myInfo then
		local title = myInfo:getTitle()
		local list = self:getCanOpenTitleList()
		for i, v in ipairs(list) do
			if title == v then
				return true
			end
		end
	end
	return false
end

-- 是否在开启的时间段
function This:isAtOpenTime()
	local curTime =	GamePlayer:getServerTime()
	local info = self:getCanOpenTime()

	return (curTime >= info.startTime and curTime <= info.endTime)
end

-- 是否在免费抽奖时间段
function This:isAtFreeLuckyDrawTime()
	local curTime =	GamePlayer:getServerTime()
	local info = self:getCanFreeLuckyDrawTime()

	return (curTime >= info.startTime and curTime <= info.endTime)
end

function This:getFreeLuckyDrawTimeTips()
	local config = LegionActiveBossConfig[1]
	local openH = math.floor(config.end_time/3600)
	local openM = math.floor(config.end_time%3600/60)
	local endH = math.floor(86400/3600)
	local endM = math.floor(86400%3600/60)	

	return string.format(Serialize[61512], openH, openM, endH, endM)
end

function This:isAtMoneyLuckyDrawTime()
	local curTime =	GamePlayer:getServerTime()
	local info = self:getCanMoneyLuckyDrawTime()

	return (curTime >= info.startTime and curTime <= info.endTime)
end

function This:getMoneyLuckyDrawTimeTips()
	local config = LegionActiveBossConfig[1]
	local openH = math.floor(config.yuanbao_cost_time/3600)
	local openM = math.floor(config.yuanbao_cost_time%3600/60)
	local endH = math.floor(86400/3600)
	local endM = math.floor(86400%3600/60)	

	return string.format(Serialize[61513], openH, openM, endH, endM)
end

-- 计算当前在哪个阶段
function This:calculatePhase()
	local curTime =	GamePlayer:getServerTime()

	local openTime = self:getCanOpenTime()
	if curTime >= openTime.startTime and curTime <= openTime.endTime then
		return LegionAbossPhase.kOpen
	end

	local freeTime = self:getCanFreeLuckyDrawTime()
	if curTime >= freeTime.startTime and curTime <= freeTime.endTime then
		return LegionAbossPhase.kFreeLuckyDraw
	end

	local moneyTime = self:getCanMoneyLuckyDrawTime()
	if curTime >= moneyTime.startTime and curTime <= moneyTime.endTime then
		return LegionAbossPhase.kMoneyLuckyDraw
	end

	return LegionAbossPhase.kNotStart
end

-- 更新阶段
function This:updatePhase()
	local oldPhase = self.m_phase
	self.m_phase = self:calculatePhase()
	return self.m_phase, oldPhase
end