

local This, super = class(..., "Dialog")
local LegionAbossManager = require("LegionAboss.LegionAbossManager")
local GameUtil = require("Util.GameUtil")
local LegionActiveBossConfig = require("Metadatas.LegionActiveBossConfig")
local ItemNode = require("UI.ItemNode")
local ItemUtil = require("Bag.ItemUtil")


function This:initData()

end

function This:initNode()
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyActiveBossPlayer.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    self:createFinalReward()

    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        local csNode = self.m_csNode
        self.m_inited = true

        csNode.close:setTouchEnded(function()
            self:close()
        end)

        -- 帮助
        csNode.help:setTouchEnded(function ()
            g_scene:openDialog("UI.HelpTips", {text=Serialize["legion_aboss_rank_tips"], node = csNode.help})
        end)
    end
end

function This:refresh()
	local dataModel = LegionAbossManager:getDataModel()
	local list = dataModel:getLuckyDrawTimesList()
	local csNode = self.m_csNode

	for i=1, 3 do
		local node = csNode[string.format("player%d", i)]
		local info = list[i]
		local isHasData = (info ~= nil)
		if isHasData then
			node.icon:setVisible(true)
			node.name:setString(info.name)
			node.ranknum:setString(info.times)
			GameUtil:setPlayerIconNodeImage(node.icon, info.icon)
		else
			node.icon:setVisible(false)
			node.name:setString(Serialize[61516])
			node.ranknum:setString("0")
		end
		node.top:setImage(string.format("MenuScene/top%d.png", i))
	end
end

function This:createFinalReward()
	local scrollview = self.m_csNode.scrollview
	local sw, sh = scrollview:getContentSize()
	scrollview:setDirection("horizontal")
    scrollview.m_direction = "right"

	local panel = cc.Node:create()
	local pw, ph = 0, sh

	local list = LegionActiveBossConfig[1].final_reward
	local count = #list


	for i, v in ipairs(list) do
		local node = ItemNode:create(v[1], v[2])
        node:setAnchorPoint(0, 0.5)
        node:setPosition(pw, ph/2)
        node:setScale(0.8)
        panel:addChild(node)

        local box = node:getBoundingBox()
        pw = pw + box.width
        if i < count then
        	pw = pw + 10
        end
	end
	panel:setAnchorPoint(0.5, 0.5)
	panel:setContentSize(pw, ph)

	if pw > sw then
		scrollview:pushBackCustomItem(panel, pw, ph, 0.5)
	else
		panel:setPosition( scrollview:getPosition() )
		self.m_csNode:addChild(panel)
	end
end