

local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local HeroManager = require("Hero.HeroManager")
local Application = require("cc.Application")
local BattleDataManager = require("Battle.BattleDataManager")
local MenuScene = require("Home.MenuScene")
local LegionActiveBossConfig = require("Metadatas.LegionActiveBossConfig")
local FamilyManager = require("Family.FamilyManager")
local Signals = require("SignalSystem.Signals")
local SignalSystem = require("SignalSystem.SignalSystem")
local PopDialogManager = require("UI.PopDialogManager")


LegionAbossPhase = {
	kNotStart = 0, --未开始
	kOpen = 1, --开启/挑战
	kFreeLuckyDraw = 2, --免费抽奖
	kMoneyLuckyDraw = 3 --元宝抽奖
}


function This:init()
	super.init(self)
	self.m_escape = 0
	self.m_isUpdate = false
	self.m_sceneName = ""
	self.m_isRedDotDirty = false 
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 12160, self.onLegionAbossInfoReply)
	EventDispatcher:addListener(self, 12161, self.onLegionAbossRoleInfoReply)
	EventDispatcher:addListener(self, 12162, self.onLegionAbossOpenReply)
	EventDispatcher:addListener(self, 12163, self.onLegionAbossChallengeReply)
	EventDispatcher:addListener(self, 12164, self.onLegionAbossLuckyDrawReply)
end

function This:getDataModel()
	if not self.m_dataModel then
		self.m_dataModel = require("LegionAboss.LegionAbossData"):create()
	end
	return self.m_dataModel
end

function This:request()
	if FamilyManager:hasFamily() then
		self:onLegionAbossInfoRequest()
	end
end

-- boss军团信息
function This:onLegionAbossInfoRequest()
	Socket:sendAndLock(12160, {})
end

function This:onLegionAbossInfoReply(msg)
	local dataModel = self:getDataModel()
	dataModel:handlerLegionAbossInfoMsg(msg)
	self:notifyObservers("onLegionAbossInfoReply")
	self:onLegionAbossRoleInfoRequest()
end

-- boss个人信息
function This:onLegionAbossRoleInfoRequest()
	Socket:sendAndLock(12161, {})	
end

function This:onLegionAbossRoleInfoReply(msg)
	local dataModel = self:getDataModel()
	dataModel:handlerLegionAbossRoleInfoMsg(msg)	
	self:notifyObservers("onLegionAbossRoleInfoReply")
	self:setRedDotDirty(true)	
	self:startUpdate()
end

-- 开启boss
function This:onLegionAbossOpenRequest()
	Socket:sendAndLock(12162, {})
end

function This:onLegionAbossOpenReply(msg)
	if msg.err_code == 0 then
		self:notifyObservers("onLegionAbossOpenReply")
		self:setRedDotDirty(true)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 挑战
function This:onLegionAbossChallengeRequest(heros)
	local result = {0,0,0,0,0,0}
	for i , info in pairs(heros) do
		local hero = HeroManager:getHero(info.heroId)
		result[info.pos] = hero:getUid()
	end
	Socket:sendAndLock(12163, {card_list = result})
end

function This:onLegionAbossChallengeReply(msg)
	if msg.err_code == 0 then
		local dataModel = self:getDataModel()
		dataModel:handlerLegionAbossChallengeMsg(msg)
		self:notifyObservers("onLegionAbossChallengeReply")

		-- 战报
		local bgId = LegionActiveBossConfig[1].bgId
		self.m_sceneName = cc.SceneManager:getSceneName()

		cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {
			msg = msg,
			bgId = bgId,
			mode = PlayMode.kReport,
		})
		BattleDataManager:addObserver(self)
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

-- 抽奖 (1位免费抽奖，2为元宝抽奖)
function This:onLegionAbossLuckyDrawRequest(dtype)
	Socket:sendAndLock(12164, {type = dtype})
end

function This:onLegionAbossLuckyDrawReply(msg)
	if msg.err_code == 0 then
		local dataModel = self:getDataModel()
		dataModel:handlerLegionAbossLuckyDrawMsg(msg)
		self:notifyObservers("onLegionAbossLuckyDrawReply")
		self:setRedDotDirty(true)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:startUpdate()
	if self.m_isUpdate then
		return
	end
	self.m_isUpdate = true
	Application:addToUpdateList(self)
end

function This:endUpdate()
	self.m_isUpdate = false
	Application:removeFromUpdateList(self)
end

-- 更新
function This:update(dt)
	self.m_escape = self.m_escape + dt

	if self.m_escape >= 1 then
		self.m_escape = self.m_escape - 1

		-- 阶段
		local dataModel = self:getDataModel()
		local newPhase, oldPhase = dataModel:updatePhase()
		if newPhase ~= oldPhase then
			self:setRedDotDirty(true)
			self:notifyObservers("onLegionAbossPhaseChange", {newPhase = newPhase, oldPhase = oldPhase})
		end

		-- 红点
		if self.m_isRedDotDirty then
			self:refreshRedDot()
		end
	end
end

function This:onBattleVictory()
	BattleDataManager:removeObserver(self)
	g_scene:openDialog("LegionAboss.LegionAbossVictoryPanel", {
	    onTouchMask = function ()
	        PopDialogManager:addPopDialog({
	        	func = function ()
	        		g_scene:openDialog("Family.FamilyPanel", {tab = "neiwu"})
	        		return g_scene:openDialog("LegionAboss.LegionAbossPanel")
	        	end
	        })
	        cc.SceneManager:switchTo(self.m_sceneName)
	    end
	})
end

function This:onBattleFailed(result)
	BattleDataManager:removeObserver(self)
	g_scene:openDialog("LegionAboss.LegionAbossFailedPanel", {
	    onTouchMask = function ()
	        PopDialogManager:addPopDialog({
	        	func = function ()
	        		g_scene:openDialog("Family.FamilyPanel", {tab = "neiwu"})
	        		return g_scene:openDialog("LegionAboss.LegionAbossPanel")
	        	end
	        })
	        cc.SceneManager:switchTo(self.m_sceneName)
	    end
	})
end

function This:onBattleSkip(result)
	if result == BattleResult.kAttackerVictory then
		self:onBattleVictory()
	else
		self:onBattleFailed(result)
	end
	BattleDataManager:clear()
end

-- 开启按钮红点
function This:isOpenAbossRedDot()
	local dataModel = self:getDataModel()
	if not dataModel:isHaveOpenPower() then
		return false
	end

	if dataModel:getIsOpen() then
		return false
	end

	if not dataModel:isAtOpenTime() then
        return false
    end

    if dataModel:getLeftOpenNum() <= 0 then
        return false
    end

    if dataModel:getCurActive() < dataModel:getMaxActive() then
        return false
    end    

	return true	
end

-- 挑战红点
function This:isChallengeRedDot()
	local dataModel = self:getDataModel()
	if not dataModel:getIsOpen() then
		return false
	end

	if not dataModel:isAtOpenTime() then
        return false
    end

    if dataModel:getMyLeftPlayNum() <= 0 then
        return false
    end    	

    return true
end

-- 免费抽奖红点
function This:isFreeDrawRedDot()
	local dataModel = self:getDataModel()
    if not dataModel:isAtFreeLuckyDrawTime() then
        return false
    end

    if dataModel:getLeftFreeNum() <= 0 then
        return false
    end

    if dataModel:getRewardLeftNum() <= 0 then
        return false
    end

    return true
end

function This:isRedDot()
	local dataModel = self:getDataModel()

	if dataModel:checkUnlock(false) then
		if self:isOpenAbossRedDot() or self:isChallengeRedDot() or self:isFreeDrawRedDot() then
			return true
		end
	end

	return false
end

function This:setRedDotDirty(dirty)
	self.m_isRedDotDirty = dirty
end

function This:refreshRedDot()
	self:setRedDotDirty(false)
	SignalSystem.distributeSignal(Signals.FamilyPanelAffairsTabRedDot)
	EventDispatcher:dispatcher("legion_aboss_change")	
end