

local This, super = class(..., "Dialog")
local LegionAbossManager = require("LegionAboss.LegionAbossManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local HeroManager = require("Hero.HeroManager")
local LegionActiveItemConfig = require("Metadatas.LegionActiveItemConfig")


function This:initData()
	self.m_isCloseAni = false
	self.m_escape = 0
	self:initHeroList()
end

function This:initHeroList()
	local dataModel = LegionAbossManager:getDataModel()
	local usedIds = dataModel:getUsedHeroList()
	local usedDict = {}

	for _, uid in ipairs(usedIds) do
		local hero = HeroManager:getHeroByUid(uid)
		local id = hero:getId()
		usedDict[id] = hero
	end

	local heros = HeroManager:getAllHeroGotWithSort(function(hero) 
		if usedDict[hero:getId()] or hero:isDeputy() then
			return false
		end
		return true
	end)
	table.sort(heros,function(a,b)
		return a:getFightPower() > b:getFightPower()
	end)

	self.m_heroList = heros
end

function This:initNode()
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyActiveBoss.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        local csNode = self.m_csNode
        self.m_inited = true

        -- 关闭
        csNode.close:setTouchEnded(function()
            self:close()
        end)

        -- 开启/挑战
        csNode.tiaozhan:setTouchEnded(function()
            local dataModel = LegionAbossManager:getDataModel()
            local isOpen = dataModel:getIsOpen()

            if not isOpen then
            	-- 开启
            	if not dataModel:isHaveOpenPower() then
            		g_scene:showTips( dataModel:getOpenTitlePowerTips() ) -- 权限不够
            		return
            	end

            	if not dataModel:isAtOpenTime() then
            		g_scene:showTips( dataModel:getOpenTimeTips() ) -- 不在开启时间
            		return
            	end

            	if dataModel:getLeftOpenNum() <= 0 then
            		g_scene:showTips(Serialize[61519])
            		return
            	end

            	if dataModel:getCurActive() < dataModel:getMaxActive() then
            		g_scene:showTips(Serialize[61520])
            		return
            	end

            	LegionAbossManager:onLegionAbossOpenRequest()
            else
            	-- 挑战
            	if not dataModel:isAtOpenTime() then
            		g_scene:showTips(Serialize[61521])
            		return
            	end            	

            	if dataModel:getMyLeftPlayNum() <= 0 then
            		g_scene:showTips(Serialize[61046])
            		return
            	end
				
				g_scene:openDialog("Territory.TerrBattleHeroPanel", {
		            targetType = TerrBattleTarget.kLegionAboss,
		            heroSource = {},
		            attacker = "",
		            heroList = self.m_heroList,
		            battleHandler = function (heros)
		                LegionAbossManager:onLegionAbossChallengeRequest(heros)
		                return true
		            end
        		})
            end
        end)

        -- 免费抽奖
        csNode.zhuanpan.button1:setTouchEnded(function()
        	if self.m_play then
        		return
        	end

            local dataModel = LegionAbossManager:getDataModel()
            if not dataModel:isAtFreeLuckyDrawTime() then
            	g_scene:showTips( dataModel:getFreeLuckyDrawTimeTips() )
            	return
            end

            if dataModel:getLeftFreeNum() <= 0 then
            	g_scene:showTips(Serialize[61522])
            	return
            end

            if dataModel:getRewardLeftNum() <= 0 then
            	g_scene:showTips(Serialize[61523])
            	return
            end

            LegionAbossManager:onLegionAbossLuckyDrawRequest(1)
        end)

        -- 元宝抽奖
        csNode.zhuanpan.button2:setTouchEnded(function()
        	if self.m_play then
        		return
        	end

            local dataModel = LegionAbossManager:getDataModel()
            if not dataModel:isAtMoneyLuckyDrawTime() then
            	g_scene:showTips( dataModel:getMoneyLuckyDrawTimeTips() )
            	return
            end

            local cost = dataModel:getMoneyLuckyDrawCost()
            if GamePlayer:checkDiamond(cost) then
            	LegionAbossManager:onLegionAbossLuckyDrawRequest(2)         
            end   
        end)

        -- 查看详情
        csNode.look:setTouchEnded(function()
            g_scene:openDialog("LegionAboss.LegionAbossRewardsPanel")
        end)

        -- 抽奖次数
        csNode.cishu:setTouchEnded(function()
            g_scene:openDialog("LegionAboss.LegionAbossLuckyDrawTimesPanel")
        end)

        -- 抽奖记录
        csNode.jilu:setTouchEnded(function()
            g_scene:openDialog("LegionAboss.LegionAbossRecordPanel")
        end)

        -- 物品
        for i=1, 10 do
        	local node = csNode.zhuanpan[string.format("pot%d", i)]
        	g_scene:handleHelpTips(node, function ()
        		local dataModel = LegionAbossManager:getDataModel()
        		local pool = dataModel:getRewardPool()
        		if pool[i] then
        			local itemId = pool[i].itemId
        			return {fileName = "UI.ItemTips", itemId = itemId, itemType = ItemUtil.getType(itemId)}
        		end
        	end)
        end

        csNode.zhuanpan.weigouxuan:setTouchEnded(function()
            self.m_isCloseAni = not self.m_isCloseAni
            self:refreshCloseAniBtn()
        end)

        -- 帮助
        csNode.help:setTouchEnded(function ()
            g_scene:openDialog("UI.HelpTips", {text=Serialize["legion_aboss_tips"], node = csNode.help})
        end)
    end
end

function This:onEnter()
    super.onEnter(self)
    LegionAbossManager:addObserver(self)
    LegionAbossManager:request()
end

function This:onExit()
	LegionAbossManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
	local dataModel = LegionAbossManager:getDataModel()
	local csNode = self.m_csNode

	-- 本周挑战次数
	local leftOpenNum = dataModel:getLeftOpenNum()
	local maxOpenNum = dataModel:getMaxOpenNum()
	csNode.cishunumber:setString(string.format(Serialize[61524], leftOpenNum, maxOpenNum))

	-- 军团活跃
	local curActive = dataModel:getCurActive()
	local maxActive = dataModel:getMaxActive()
	csNode.juntuanhuoyue:setString(string.format("%d/%d", curActive, maxActive))
	csNode.jindutiaodi.jindutiao:setPercent(curActive/maxActive*100)


	-- 开启/挑战
	if not dataModel:getIsOpen() then
		-- 开启 （军团长/副军团长）
		csNode.tiaozhan.text:setString(Serialize[61525])
	
		if 	dataModel:isAtOpenTime() and 
			dataModel:isHaveOpenPower() and 
			dataModel:getLeftOpenNum() > 0 and
			dataModel:getCurActive() >= dataModel:getMaxActive() then
			csNode.tiaozhan:clearShader()
		else
			csNode.tiaozhan:setGrayShader()
		end
	else
		-- 挑战
		local leftPlayNum = dataModel:getMyLeftPlayNum()
		local maxPlayNum = dataModel:getMyMaxPlayNum()
		csNode.tiaozhan.text:setString(string.format(Serialize[61526], leftPlayNum, maxPlayNum))

		if dataModel:isAtOpenTime() and leftPlayNum > 0 then
			csNode.tiaozhan:clearShader()
		else
			csNode.tiaozhan:setGrayShader()
		end
	end

	-- 奖池等级
	local poolLv = dataModel:getRewardPoolLv()
	csNode.jiangchilv:setString(string.format(Serialize[61527], poolLv))

	-- 挑战人数和伤害
	local people = dataModel:getAllPeople()
	local damage = dataModel:getAllDamage()
	csNode.nextlv:setStringWithColor(string.format(Serialize[61528], people, ItemUtil.formatItemCount(damage) ))

	-- 奖励剩余
	local rwLeftNum = dataModel:getRewardLeftNum()
	local rwAllNum = dataModel:getRewardAllNum()
	local rwStr = string.format(Serialize[61529], rwLeftNum, rwAllNum)
	if rwLeftNum <= 0 then
		rwStr = string.format(Serialize[61530], rwLeftNum, rwAllNum)
	end
	csNode.shengyunum:setStringWithColor(rwStr)

	-- 倒计时
	self:updateCountDown()

	-- 奖池物品
	local pool = dataModel:getRewardPool()
	local zhuanpan = csNode.zhuanpan
	for i=1, 10 do
		local pot = zhuanpan[string.format("pot%d", i)]
		local item = zhuanpan[string.format("item%d", i)]
		local dazhetiao = zhuanpan[string.format("dazhetiao%d", i)]
		local itemInfo = pool[i]
		if pot and item and dazhetiao then
			local isHasData = (itemInfo ~= nil)
			item:setVisible(isHasData)
			dazhetiao:setVisible(isHasData)
			pot.selected:setVisible(false)

			if isHasData then
				dazhetiao:setVisible(itemInfo.isRare)
				item.icon:setImage( ItemUtil.getIconImage(itemInfo.itemId) )

				local numStr = string.format("<New19>%s<A0>（%d/%d）", ItemUtil.formatItemCount(itemInfo.itemNum), itemInfo.remainNum, itemInfo.totalNum)
				if itemInfo.remainNum <= 0 then
					numStr = string.format("<New19>%s<New11>（%d/%d）", ItemUtil.formatItemCount(itemInfo.itemNum), itemInfo.remainNum, itemInfo.totalNum)
				end
				item.num:setStringWithColor(numStr)
			end
		end
	end

	-- 免费抽奖次数
	local freeNum = dataModel:getLeftFreeNum()
	zhuanpan.button1.text:setString(string.format(Serialize[61531], freeNum))

	-- 购买次数的元宝
	local cost = dataModel:getMoneyLuckyDrawCost()
	zhuanpan.lbCost:setString(cost)

	self:refreshCloseAniBtn()
	self:refreshRedDot()
end

function This:refreshCloseAniBtn()
	self.m_csNode.zhuanpan.gouxuan:setVisible( self.m_isCloseAni )
end

function This:refreshRedDot()
	local dataModel = LegionAbossManager:getDataModel()
	local csNode = self.m_csNode

	if dataModel:getIsOpen() then
		GameUtil:refreshRedDot(csNode.tiaozhan, LegionAbossManager:isChallengeRedDot())
	else
		GameUtil:refreshRedDot(csNode.tiaozhan, LegionAbossManager:isOpenAbossRedDot())
	end

	GameUtil:refreshRedDot(csNode.zhuanpan.button1, LegionAbossManager:isFreeDrawRedDot())
end

-- 倒计时
function This:updateCountDown()
	local label = self.m_csNode.time
	local curTime =	GamePlayer:getServerTime()
	local dataModel = LegionAbossManager:getDataModel()

	if dataModel:getIsOpen() then
		local freeTime = dataModel:getCanFreeLuckyDrawTime()
		if curTime <= freeTime.startTime then
			label:setStringWithColor( GameUtil:formatTime(Serialize[61532], freeTime.startTime - curTime) )
			return
		end

		local moneyTime = dataModel:getCanMoneyLuckyDrawTime()
		if curTime <= moneyTime.startTime then
			label:setStringWithColor( GameUtil:formatTime(Serialize[61533], moneyTime.startTime - curTime) )
			return
		end
	end

	label:setString("")
end

function This:update(dt)
	super.update(self, dt)

	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1 then
		self.m_escape = self.m_escape - 1
		self:updateCountDown()
	end
end

function This:onLegionAbossInfoReply()
	self:refresh()
end

function This:onLegionAbossRoleInfoReply()
	self:initHeroList()
	self:refresh()
end

function This:onLegionAbossLuckyDrawReply()
	local dataModel = LegionAbossManager:getDataModel()
	local rid = dataModel:getRewardId()

	local showReward = function ()
		local config = LegionActiveItemConfig[rid]
		g_scene:showReward({{config.item_id, config.num}})
		self:refresh()
	end

	if self.m_isCloseAni then
		showReward()
	else
		self:playAnimation(rid, showReward)
	end
end

function This:onLegionAbossPhaseChange(info)
	self:refresh()
end

function This:playAnimation(rid, finishCb)
	if self.m_play then
		return
	end

	local dataModel = LegionAbossManager:getDataModel()
	local pool = dataModel:getRewardPool()
	local itemIdx = 0
	for i, v in ipairs(pool) do
		if v.id == rid then
			itemIdx = i
			break
		end
	end
	if itemIdx == 0 then
		return
	end

	self.m_play = {
		itemIdx = itemIdx,
		node = nil,
		action = nil,
	}

	local zhuanpan = self.m_csNode.zhuanpan
	local list = {}
	for i=1, 10 do
		local pot = zhuanpan[string.format("pot%d", i)]
		pot.selected:setVisible(false)
	end

	for i = 0, itemIdx+9 do
		local idx = i%10+1
		table.insert(list, cc.DelayTime(0.2))
		table.insert(list, cc.CallFunc(function ()
			print(i, idx)
			local pot = zhuanpan[string.format("pot%d", idx)]
			pot.selected:setVisible(true)

			if self.m_play.node then
				self.m_play.node.selected:setVisible(false)
			end
			self.m_play.node = pot
		end))
	end

	local ac1 = cc.Ease["SineOut"]( cc.Sequence(unpack(list)) )
	local ac2 = cc.CallFunc(function ()
		self.m_play.node:runAction( cc.Sequence(cc.DelayTime(0.2), cc.Blink(0.5, 5)) )
	end)
	local ac3 = cc.DelayTime(0.7)
	local ac4 = cc.CallFunc(function ()
		self.m_play = nil
		if finishCb then
			finishCb()
		end
	end)

	local action = cc.Sequence(ac1, ac2, ac3, ac4)
	self.m_play.action = action

	self:runAction(action)
end