

local This, super = class(..., "Dialog")
local ItemUtil = require("Bag.ItemUtil")
local LegionActivePoolConfig = require("Metadatas.LegionActivePoolConfig")
local RankPageListLayer = require("Rank.RankPageListLayer")
local LegionAbossRewardsItemView = require("LegionAboss.LegionAbossRewardsItemView")


function This:initData(info)
	self.m_poolList = {}
	self.m_selectIdx = 1

	for id, pool in pairs(LegionActivePoolConfig) do
		table.insert(self.m_poolList, pool)
	end

	table.sort(self.m_poolList, function (a, b)
		return a.id < b.id
	end)
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/FamilyActiveBossReward.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    for i, v in ipairs(self.m_poolList) do
    	local node = csNode[string.format("tep%d", i)]
      	self:addTabButton(i, node, function ()
			return self:onTabButtonSelect(i)
		end)
    end

    local scrollView = csNode.ScrollView
	scrollView:setDirection("vertical")

	self.m_listLayer = RankPageListLayer:create({
        itemWidth = 111,
        itemHeight = 127,
        spaceX = 8,
        spaceY = 8,
        column = 5,
        scrollView = scrollView,
		dataCounter = function ()	 
	        return #self:getRewardIds()
        end,
        viewCreator =  function(index, node) 
        	local data = self:getRewardIds()[index]
        	if data then     
			    if not node then
                    node = LegionAbossRewardsItemView:create(data, index)
				    return node
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)    

    self:selectTabButton(1)
end

function This:getRewardIds()
	return self.m_poolList[self.m_selectIdx].reward_id
end

function This:refresh()
	local condition = self.m_poolList[self.m_selectIdx].pool_condition
	local str = string.format(Serialize[61539], condition[1], ItemUtil.formatItemCount(condition[2]))
	self.m_csNode.tiaojian:setStringWithColor(str)

	self.m_listLayer:clearNode(true, true)
	self.m_listLayer:setDataList()
end

function This:initTouchEvent()
    if not self.m_inited then
        local csNode = self.m_csNode
        self.m_inited = true

        csNode.close:setTouchEnded(function()
            self:close()
        end)
    end
end

function This:onEnter()
    super.onEnter(self)
end

function This:onExit()
    super.onExit(self)
end

function This:onTabButtonSelect(name)
	self.m_selectIdx = name
	self:refresh()
	return true
end