
-- 赛区

local This = class(...)
local CompetitionAreaConfig = require("Metadatas.CompetitionAreaConfig")
local LoginManager = require("Login.LoginManager")
local GameUtil = require("Util.GameUtil")
local ServerMixConfig = require("Metadatas.ServerMixConfig")


function This:init()
	local myServerId = self:getMergeServerId()
	self.m_areas = {}
	self.m_myAreaId = 0

	for id, config in pairs(CompetitionAreaConfig) do
		local area = {}
		local minSid = config.serverid_min
		local maxSid = config.serverid_max

		area.id = id
		area.minServerId = minSid
		area.maxServerId = maxSid
		area.serverList = {}
		self.m_areas[id] = area

		if myServerId >= minSid and myServerId <= maxSid then
			self.m_myAreaId = id
		end
	end

	local serverList = LoginManager:getServerListWithSort()
	for _, server in ipairs(serverList) do
		local serverId = server:getId()
		local mergeId = self:getMergeServerId(serverId)
		for _, area in pairs(self.m_areas) do
			if mergeId >= area.minServerId and mergeId <= area.maxServerId then
				local t = {}
				t.id = serverId
				t.indexInGroup = server:getIndexInGroup()
				t.areaName = server:getAreaName(true)
				table.insert(area.serverList, t)
				break
			end
		end
	end
end

function This:getMyArea()
	if not self.m_myAreaId then
		self:init()
	end
	return self.m_areas[self.m_myAreaId]
end

function This:getMyAreaRegionText()
	local text = ""
	local area = self:getMyArea()
	if area then
		if #area.serverList == 1 then
			text = area.serverList[1].areaName
		else
			local minServer = area.serverList[1]
			local maxServer = area.serverList[#area.serverList]
			text = string.format("%s-%s", minServer.areaName, maxServer.areaName)
		end
	end
	return text
end

function This:getMergeServerId(id)
	id = id or LoginManager:getLoginServer():getId()

	if ServerMixConfig[id] then
		return ServerMixConfig[id].target_server
	end

	return id
end