-- @Author: superm
-- @Date:   2018-11-01 20:27:38
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-19 21:12:17
-- @File Name: EntrantBtn.lua
-- @Description: 

local This, super = class(..., "Home.MenuButton")
local EntrantManager = require("LegionEntrant.EntrantManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local PlayTypeConfig = require("Metadatas.PlayTypeConfig")


local EntrantFileName = {
	[GameMapType.kEmperorMap] = "LegionEntrant.KingWarPanel",
	[GameMapType.kLegionMap] = "LegionEntrant.LegionWarPanel",
	[GameMapType.kRescueMap] = "RescueEmperor.RescueWarPanel",
}


function This:init(config)
    super.init(self, config)
    self.m_autoClose = config.uiclass.auto_close
    self.m_playType = config.uiclass.map_type --换成playType
    self.m_mapType = PlayTypeConfig[self.m_playType].map_type

	self:updateTimeInfo()
	self:refreshCountDown()
    self:scheduleUpdate()
end

function This:initTouchEvent()
    self:setTouch({
		onEnded = function()
	        local mapType = self.m_mapType
	        local state, errorTips = EntrantManager:hasFamilyEntrant(mapType) 
	        if state then
		        if not self.m_subView then
		            local panel = g_scene:openDialog(EntrantFileName[mapType], {mapType = mapType, playType = self.m_playType})
	            	self.m_subView = panel
					panel:addCloseCallback(function ()
		                self.m_subView = nil
		            end)
		        end
	        elseif errorTips then
	            g_scene:showTips(errorTips)
	        end
		end,
		isScale = false
	})
end

function This:addCustomListener()
    super.addCustomListener(self)
    if self.m_autoClose then
		EntrantManager:addObserver(self)
    end
end

function This:removeCustomListener()
	EntrantManager:removeObserver(self)
    super.removeCustomListener(self)
end

function This:close()
    if self.m_subView and self.m_subView[0] ~= 0 then
        self.m_subView:close()
    end
    self:setVisible(false)
end

function This:open()
    self:setVisible(true)
end

function This:getDefaultVisible()
	if self.m_nextState then
		if self.m_nextState <= KingWarBtnState.kNotStart then
			return false
		else
			return true
		end
	end

	return super.getDefaultVisible(self)
end

function This:getRedDotValue()
    local value = super.getRedDotValue(self)
    local ret = value
    if self.m_autoClose then
        ret = value and true or nil
    end
    return ret
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return 0
	end
	return self.m_endTime - curTime
end

function This:update(dt)
    if self.m_hasTimeUpdate then
		self.m_escape = self.m_escape + dt
		if self.m_escape >= 1.0 then
			self.m_escape = self.m_escape - 1
			local leftTime = self:getLeftTime()
			if leftTime <= 0 then
    			self:updateTimeInfo()
			end
			self:refreshCountDown()
		end
	end
end

function This:updateTimeInfo()
	local oldState = -1
	if self.m_btnInfo then
		oldState = self.m_btnInfo.state
	end

	self.m_btnInfo = EntrantManager:getButtonInfo(self.m_playType)
	self.m_nextState = self.m_btnInfo.state
	self.m_endTime = self.m_btnInfo.endTime
	self.m_escape = 0
	self.m_hasTimeUpdate = true

	if self.m_autoClose then
		if self.m_nextState <= KingWarBtnState.kNotStart then
			self:close()
		else
			self:open()
		end
	end

	-- 请求数据
	if oldState ~= self.m_btnInfo.state then
		print("/// EntrantBtn require "..self.m_mapType)
		if self.m_mapType == GameMapType.kLegionMap then
			EntrantManager:onLegionWarInfoRequest()
		elseif self.m_mapType == GameMapType.kEmperorMap then
			EntrantManager:onKingWarInfoRequest()
		end
	end

	-- print( string.format("=== EntrantBtn updateTimeInfo %s", self.m_playType) )
	-- print(self.m_btnInfo)	
end

function This:refreshCountDown()
	local state = self.m_btnInfo.state
	local endTime =	self.m_btnInfo.endTime
	local text = self.m_btnInfo.text

	-- 预告、报名、备战、战斗
	local isShow = true
	if state == KingWarBtnState.kNotStart or state == KingWarBtnState.kEnd then
		isShow = false

	elseif state == KingWarBtnState.kFight then
		-- 提前结束不显示
		if self.m_mapType == GameMapType.kLegionMap then
			local data = EntrantManager:getLegionWarData()
			local resultFlag = data:getResultFlag()
			if resultFlag == 4 then
				isShow = false
			end
		elseif self.m_mapType == GameMapType.kEmperorMap then
			local data = EntrantManager:getKingWarData()
			local resultRank = data:getResultRank()
			if #resultRank > 0 then
				isShow = false
			end
		end
	end

	self.timebg:setVisible(isShow)
	self.time:setVisible(isShow)

	if not isShow then
		self.text:setPositionY(1)
		self:setFireEffectVisible(false)
		return
	end

	local curTime = GamePlayer:getServerTime()
	local leftTime = endTime - curTime 
	if leftTime < 0 then
		leftTime = 0
	end

	local formatTime = GameUtil:formatTime(Serialize["H_M_S"], leftTime)
	self.time:setString( string.format("%s %s", text, formatTime) )
	self:setFireEffectVisible( state == KingWarBtnState.kFight )
	self.text:setPositionY(12)
end

function This:setFireEffectVisible(visible)
	if visible then
		if not self.m_fireEff then
			self.m_fireEff = GameUtil:createEffect("ZhengBa", true, self)
			self.m_fireEff:setScale(0.8)
			self:addChild(self.m_fireEff)
		else
			self.m_fireEff:setVisible(true)
		end
	else
		if self.m_fireEff then
			self.m_fireEff:setVisible(false)
		end
	end
end