-- @Author: superm
-- @Date:   2018-10-10 18:11:40
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-21 15:30:10
-- @File Name: EntrantManager.lua
-- @Description: 6L+Z5YWs5Y+45b6I5Z2R5b+r6LWw5ZCn77yM5rKh5pyJ5aWW6YeR

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local FamilyManager = require("Family.FamilyManager")
local PlayTypeConfig = require("Metadatas.PlayTypeConfig")
local LegionEnrollConfig = require("Metadatas.LegionEnrollConfig")
local GameUtil = require("Util.GameUtil")
local EventListConfig = require("Metadatas.EventListConfig")
local ActivityTrailer = require("NewActivity.ActivityTrailer")
local RedDotManager = require("RedDot.RedDotManager")
local InviteManager = require("Invite.InviteManager")
local LoginManager = require("Login.LoginManager")


-- 按钮的状态：未开始 -> 预告 -> 报名 -> 准备中 -> 战斗中 -> 结束（消失倒计时）
KingWarBtnState = {
	kNotStart = 0,
	kPreView = 1,
	kSignUp = 2,
	kPrepare = 3,
	kFight = 4,
	kEnd = 5,
}

EntrantState = {
	kNotStart = 0, --开服未满N天
	kPreView = 1, -- 预告时间
	kEntrant = 2, --报名时间
	kCheerOrDeclare = 3, --押注，宣战时间
	kGroup = 4, --分组
	kInFight = 5, --战斗时间
	kFinish = 6, --结算
}

--勤王战排行榜
RescueRankType = {
	kTuLu = 5, -- 屠戮
	kShaDi = 4, --杀敌
	kLianZhan = 6, --连斩
}

local kSystemOpenTime = 1542643200
local EntrantConfigField = {
	[EntrantState.kPreView] = "preview_start_time",
	[EntrantState.kEntrant] = "sign_up_start_time",
	[EntrantState.kCheerOrDeclare] = "sign_up_end_time",
	[EntrantState.kGroup] = "create_group",
	[EntrantState.kInFight] = "fight_start_time",
	[EntrantState.kFinish] = "fight_end_time",
}

function This:isInited()
    return self.m_inited
end

function This:init()
	super.init(self)
    self:initListener()
	self.m_timeInfo = {}
	self.m_cheerNum = 0
	self.m_allCheerNum = 0
	self:initConfig()
    self.m_inited = true
end

function This:initConfig()
	local config = LegionEnrollConfig[1]
    self.m_cheerMaxNum = config.upper_limit
	self.m_cheerPrice = config.encourage_price
	self.m_entrantPrice = config.Enroll_cost
	self.m_declarePrice = config.declarewar_cost
	self.m_auctionMinNum = config.auction_price
end

--入口开关
function This:getEntrantOpenState(mapType)
	if mapType == GameMapType.kEmperorMap then
		return true --皇帝战
	elseif mapType == GameMapType.kLegionMap then
		return true --军团战
	elseif mapType == GameMapType.kRescueMap then
		return true --勤王战
	end
	return false
end

-----------------Time Tools -------------------
-- 获取下个阶段，mapType:地图类型, currentState:当前阶段
local function getNextState(mapType, currentState)
	if EntrantState.kFinish < currentState then
		return
	end

	if mapType == GameMapType.kEmperorMap and (currentState + 1) == EntrantState.kGroup then
		return EntrantState.kInFight
	end

	if mapType == kRescueMap and (currentState + 1) == EntrantState.kCheerOrDeclare then
		return EntrantState.kGroup
	end

	return currentState + 1
end

-- 计算今天星期几， 星期一：1, 星期天：7
local function calculateWeekDay(paramDate)
	return (paramDate.wday + 5) % 7 + 1
end

local function isPassed(currentDate, configTime, paramOffsetDays)
	local offsetDays = paramOffsetDays
	if not offsetDays then
		offsetDays = configTime[1] - calculateWeekDay(currentDate)
	end

	local bePostponed
	if (offsetDays < 0) then
		bePostponed = true
	elseif offsetDays == 0 then
		local offsetHours = configTime[2] - currentDate.hour
		if offsetHours < 0 then
			bePostponed = true
		elseif offsetHours == 0 and configTime[3] < currentDate.min then
			bePostponed = true
		end
	end
	return bePostponed
end

-- 计算某个阶段的时候，curDate:日期, cfgTime:某个阶段的配置信息, postponed:允许倒回去计算
local function getPhaseTime(curDate, cfgTime, postponed)
	local offsetDays = cfgTime[1] - calculateWeekDay(curDate)

	if not postponed and isPassed(curDate, cfgTime, offsetDays) then
		offsetDays = offsetDays + 7
	end
	local ret = os.time({day = curDate.day, month = curDate.month, year = curDate.year, hour = cfgTime[2], min = cfgTime[3], sec = cfgTime[4]})
	ret = ret + offsetDays * SECONDS_PER_DAY
	return ret
end

function This:initListener()
	--EventDispatcher:addListener(self, "init_role_info", self.requestAllEntrantInfo)
	EventDispatcher:addListener(self, 13514, self.onRescueInfoReply)
	EventDispatcher:addListener(self, 13515, self.onRescueResponseReply)
	EventDispatcher:addListener(self, 13516, self.onRescueResultReply)
	EventDispatcher:addListener(self, 12057, self.onRescueAreaRankReply)

	EventDispatcher:addListener(self, 13517, self.onLegionWarInfoReply)
	EventDispatcher:addListener(self, 13518, self.onLegionWarResponseReply)
	EventDispatcher:addListener(self, 13519, self.onLegionWarResultReply)

	EventDispatcher:addListener(self, 13520, self.onKingWarInfoReply)
	EventDispatcher:addListener(self, 13521, self.onKingWarResultNotify)
	EventDispatcher:addListener(self, 13522, self.onKingWarApplyTeamReply)
	EventDispatcher:addListener(self, 13523, self.onKingWarDisposeApplyReply)
	EventDispatcher:addListener(self, 13524, self.onKingWarInviteTeamReply)
	EventDispatcher:addListener(self, 13525, self.onKingWarInviteTeamPushNotify)
	EventDispatcher:addListener(self, 13526, self.onKingWarAgreeInviteReply)
	EventDispatcher:addListener(self, 13527, self.onKingWarKickOutTeamReply)
	EventDispatcher:addListener(self, 13528, self.onKingWarEnterReply)	

end

function This:removeListener()
	--EventDispatcher:removeListener(self, "init_role_info")
	EventDispatcher:removeListener(self,13514)
	EventDispatcher:removeListener(self,13515)
	EventDispatcher:removeListener(self,13516)
	EventDispatcher:removeListener(self,12057)

	EventDispatcher:removeListener(self,13517)
	EventDispatcher:removeListener(self,13518)
	EventDispatcher:removeListener(self,13519)	

	EventDispatcher:removeListener(self,13520)
	EventDispatcher:removeListener(self,13521)
	EventDispatcher:removeListener(self,13522)
	EventDispatcher:removeListener(self,13523)
	EventDispatcher:removeListener(self,13524)
	EventDispatcher:removeListener(self,13525)
	EventDispatcher:removeListener(self,13526)
	EventDispatcher:removeListener(self,13527)		
end

function This:getOpenServerTime()
    return self.m_openServerTime or 0
end

function This:hasPower( ... )
	if not FamilyManager:hasFamily() then
		return
	end

	local family = FamilyManager:getMyFamily()
	if family then
		local selfMember = family:getSelfMemberInfo()
		if selfMember then
	    	return selfMember:hasPower( ... )
	    end
	end
end

function This:requestAllEntrantInfo()
	local legionWarData = self:getLegionWarData()
	if legionWarData:getState() ~= KingWarBtnState.kNotStart then
		self:onLegionWarInfoRequest()
	end

	local kingWarData = self:getKingWarData()
	if kingWarData:getState() ~= KingWarBtnState.kNotStart then
		self:onKingWarInfoRequest()
	end
end

function This:hasFamilyEntrant(mapType, nocheck)
	if nocheck or FamilyManager:hasFamily() then
  		return true
    else
    	return false, Serialize[61508]
    end
end

-- 获取本次战斗的开始时间、结束时间
function This:getFightTime(mapType, finishPhase)
	local curTime = GamePlayer:getServerTime()
	local curDate = os.serverdate("*t", curTime)
	local config =  PlayTypeConfig[mapType]
	local phase = finishPhase and EntrantState.kFinish or EntrantState.kInFight
	local cfgTime = config[EntrantConfigField[phase]]
    return getPhaseTime(curDate, cfgTime, not finishPhase)
end

---------------------------------------------------------------------------------------------
--勤王战


-- 勤王战数据
function This:getRescueWarData()
	local playType = self:selectPlayType(GameMapType.kRescueMap)

	if self.m_rescueWarData then
		if playType ~= self.m_rescueWarData:getPlayType() then
			self.m_rescueWarData = nil
		end
	end

	if not self.m_rescueWarData then
		self.m_rescueWarData = require("RescueEmperor.RescueWarData"):create(playType)
	end

	return self.m_rescueWarData
end

-- 勤王战信息
function This:onRescueInfoRequest()
	Socket:sendAndLock(13514, {})
end

function This:onRescueInfoReply(msg)
	-- print("====== onRescueInfoReply")
	-- print_tmp(msg)
	
	local data = self:getRescueWarData()
	data:handlerRescueInfoMsg(msg)
	self:notifyObservers("onRescueInfoReply", data)
end

-- 勤王战响应
function This:onRescueResponseRequest()
	Socket:sendAndLock(13515, {})
end

function This:onRescueResponseReply(msg)
	-- print("====== onRescueResponseReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getRescueWarData()
		data:handlerRescueResponseMsg(msg)
		self:notifyObservers("onRescueResponseReply", data)
    else
        g_scene:handleErrorCode(msg.err_code)
        self:onRescueInfoRequest()
    end
end

-- 勤王战结果
function This:onRescueResultRequest()
	Socket:sendAndLock(13516, {})
end

function This:onRescueResultReply(msg)
	-- print("====== onRescueResultReply")
	-- print_tmp(msg)	

	if msg.err_code == 0 then
		local data = self:getRescueWarData()
		data:handlerRescueResultMsg(msg)
		self:notifyObservers("onRescueResultReply", data)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 勤王战赛区排名
function This:onRescueAreaRankRequest()
	Socket:sendAndLock(12057, {})
end

function This:onRescueAreaRankReply(msg)
	-- print("====== onRescueAreaRankReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getRescueWarData()
		data:handlerRescueAreaRankMsg(msg)
		self:notifyObservers("onRescueAreaRankReply", data)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end


function This:hasCountEventFignt()
	local hasEvent = false
	local hasEvent2 = false
	for id, config in ipairs(EventListConfig) do
		local trailer = ActivityTrailer:create(id)
		if id == 9 then
			if trailer:isRunning() then
				hasEvent = true
			end
			if trailer:getState() == 1 and trailer:getId() == 9 then
				hasEvent = true
			end
		elseif id == 10 then
			if trailer:isRunning() then
				hasEvent2 = true
			end
		end
	end
	return hasEvent,hasEvent2
end

-- 开服0点的时间
function This:getOpenServerZeroTime()
	local time = GamePlayer:getOpenServerTime()
	if time <= 0 then
		return 0
	end

	local date = os.serverdate("*t", time)
	local zeroTime = os.time({year = date.year, month = date.month, day = date.day, hour = 0, min = 0, sec = 0})
	return zeroTime
end

-- 周一0点时间
function This:getMondayZeroTime(time)
	time = time or GamePlayer:getServerTime()
	local date = os.serverdate("*t", time)
	local wday = (date.wday + 5) % 7 + 1 --星期一：1, 星期天：7
	local offset = wday - 1
	local monDate = os.serverdate("*t", time - offset * SECONDS_PER_DAY)

	return os.time({year = monDate.year, month = monDate.month, day = monDate.day, hour = 0, min = 0, sec = 0})
end

-- 转换配置的时间为秒 configTime = {周几，时，分，秒}
function This:convertToSecond(configTime)
	return (configTime[1] - 1) * 86400 + configTime[2] * 3600 + configTime[3] * 60 +  configTime[4]
end

-- 下一次预告时间
function This:getNextPreviewStartTime(playType)
	local config =  PlayTypeConfig[playType]
	local previewStartTime = config.preview_start_time
	local curTime =	GamePlayer:getServerTime()

	-- 首次预告
	local openSysTime = SECONDS_PER_DAY * config.open_day + self:getOpenServerZeroTime()
	if curTime < openSysTime then
		local monTime = self:getMondayZeroTime(openSysTime)
		local preTime = monTime + self:convertToSecond(previewStartTime)
		if preTime < openSysTime then
			return preTime + 7 * SECONDS_PER_DAY
		end
		return preTime
	end

	local monTime = self:getMondayZeroTime(curTime)
	local preTime = monTime + self:convertToSecond(previewStartTime)

	-- 临界点：预告比开启时间小
	if preTime < openSysTime then
		return preTime + 7 * SECONDS_PER_DAY
	end

	-- 下周的预告
	if curTime >= preTime then
		return preTime + 7 * SECONDS_PER_DAY
	end

	return preTime
end

-- 按钮状态（皇帝、军团、勤王1、勤王2）
function This:getButtonInfo(playType)
	local ret = {
		playType = playType,
		state = KingWarBtnState.kNotStart,
		endTime = 0,
		text = Serialize[61544]
	}

	local config =  PlayTypeConfig[playType]
	local mapType = config.map_type

	-- 测试代码
	-- if mapType == GameMapType.kEmperorMap then
	-- 	ret.state = KingWarBtnState.kSignUp
	-- 	ret.endTime = GamePlayer:getServerTime() + 3600
	-- 	ret.text = Serialize[61545]
	-- 	return ret
	-- end

	-- 功能关闭
	if not self:getEntrantOpenState(mapType) then
		return ret
	end

	local openSvrTime = self:getOpenServerZeroTime()
	if openSvrTime <= 0 then
		return ret
	end	

	local curTime = GamePlayer:getServerTime()

	-- 开服第几天开启
	local openTime = SECONDS_PER_DAY * config.open_day + openSvrTime
	if curTime < openTime then
		-- print("开服不足" .. config.open_day)
		ret.endTime = self:getNextPreviewStartTime(playType)
		return ret
	end

	-- 预告
	local monTime = self:getMondayZeroTime()
	local preStart = monTime + self:convertToSecond(config.preview_start_time)
	if preStart < openTime or curTime < preStart then
		ret.endTime = self:getNextPreviewStartTime(playType)
		return ret
	end

	local preEnd = monTime + self:convertToSecond(config.preview_end_time)
	if curTime < preEnd then
		ret.state = KingWarBtnState.kPreView
		ret.endTime = preEnd
		ret.text = Serialize[61546]
		return ret
	end

	-- 报名
	local signUpEnd = monTime + self:convertToSecond(config.sign_up_end_time)
	if curTime < signUpEnd then
		ret.state = KingWarBtnState.kSignUp
		ret.endTime = signUpEnd
		ret.text = Serialize[60632]
		return ret
	end 

	-- 准备
	local fightStart = monTime + self:convertToSecond(config.fight_start_time)
	if curTime < fightStart then
		ret.state = KingWarBtnState.kPrepare
		ret.endTime = fightStart
		ret.text = Serialize[61547]
		return ret
	end

	-- 战斗
	local fightEnd = monTime + self:convertToSecond(config.fight_end_time)
	if curTime < fightEnd then
		ret.state = KingWarBtnState.kFight
		ret.endTime = fightEnd
		ret.text = Serialize[61548]
		return ret
	end

	-- 结束
	local disTime = monTime + self:convertToSecond(config.button_disappear_time)
	if curTime < disTime then
		ret.state = KingWarBtnState.kEnd
		ret.endTime = disTime
		ret.text = Serialize[61549]
		return ret
	end

	ret.endTime = self:getNextPreviewStartTime(playType)

	return ret
end

-- 事件薄的预告时间
function This:getEventBookPreviewTime(mapType)
	local list = {}

	-- 相同的mapType一周会有多场，需要判断哪个先开
	for k, v in pairs(PlayTypeConfig) do
		if v.map_type == mapType then
			local t = {}
			t.playType = v.type
			t.startTime = 0
			t.endTime = 0
			table.insert(list, t)
		end
	end

	local target = nil
	for i, v in ipairs(list) do
		local config =  PlayTypeConfig[v.playType]
		local openSysTime = SECONDS_PER_DAY * config.open_day + self:getOpenServerZeroTime()
		local monTime = self:getMondayZeroTime(openSysTime)
		local preTime = monTime + self:convertToSecond(config.preview_start_time)
		if preTime < openSysTime then
			preTime = preTime + 7 * SECONDS_PER_DAY
		end

		v.startTime = preTime - 7 * SECONDS_PER_DAY
		v.endTime = preTime

		if not target then
			target = v
		else
			if v.startTime < target.startTime then
				target = v
			end
		end
	end

	local startDate = os.serverdate("*t", target.startTime)
	local endDate = os.serverdate("*t", target.endTime)
	print( string.format("【map_type = %d】事件簿预告时间：%d-%d-%d %02d:%02d:%02d 到 %d-%d-%d %02d:%02d:%02d", mapType, 
		startDate.year, startDate.month, startDate.day, startDate.hour, startDate.min, startDate.sec,
		endDate.year, endDate.month, endDate.day, endDate.hour, endDate.min, endDate.sec) )


	return target.startTime, target.endTime
end

-- 根据mapType选取一个正在开或即将开的战斗
function This:selectPlayType(mapType)
	local list = {}
	for k, v in pairs(PlayTypeConfig) do
		if v.map_type == mapType then
			table.insert(list, v)
		end
	end

	if #list == 1 then
		return list[1].type
	end

	table.sort(list, function (a, b)
		-- {星期，时，分，秒}
		local aPreview = a.preview_start_time
		local bPreview = b.preview_start_time
		local aTime = aPreview[2]*3600 + aPreview[3]*60 + aPreview[4]
		local bTime = bPreview[2]*3600 + bPreview[3]*60 + bPreview[4]
		if aPreview[1] ~= bPreview[1] then
			return aPreview[1] < bPreview[1]
		else
			return aTime < bTime
		end
	end)
	
	local target = nil
	for i, v in ipairs(list) do
		local info = self:getButtonInfo(v.type)
		if info.state > KingWarBtnState.kNotStart then
			return v.type
		else
			if target == nil then
				target = info
			else
				if info.endTime < target.endTime then
					target = info
				end 
			end
		end
	end

	return target.playType
end


---------------------------------------------------------------------------------------------
--军团战

function This:getLegionWarData()
	local playType = self:selectPlayType(GameMapType.kLegionMap)

	if self.m_legionWarData then
		if playType ~= self.m_legionWarData:getPlayType() then
			self.m_legionWarData = nil
		end
	end

	if not self.m_legionWarData then
		self.m_legionWarData = require("LegionEntrant.LegionWarData"):create(playType)
	end

	return self.m_legionWarData
end

-- 军团战信息
function This:onLegionWarInfoRequest()
	Socket:sendAndLock(13517, {})
end

function This:onLegionWarInfoReply(msg)
	-- print("====== onLegionWarInfoReply")
	-- print_tmp(msg)

	local data = self:getLegionWarData()
	data:handlerLegionWarInfoMsg(msg)
	self:notifyObservers("onLegionWarInfoReply", data)
end

-- 军团战响应
function This:onLegionWarResponseRequest()
	Socket:sendAndLock(13518, {})
end

function This:onLegionWarResponseReply(msg)
	-- print("====== onLegionWarResponseReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getLegionWarData()
		data:handlerLegionWarResponseMsg(msg)
		self:notifyObservers("onLegionWarResponseReply", data)
    else
        g_scene:handleErrorCode(msg.err_code)
        self:onLegionWarInfoRequest()
    end
end

-- 军团战结果
function This:onLegionWarResultRequest()
	Socket:sendAndLock(13519, {})
end

function This:onLegionWarResultReply(msg)
	-- print("====== onLegionWarResultReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getLegionWarData()
		data:handlerLegionWarResultMsg(msg)
		self:notifyObservers("onLegionWarResultReply", data)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end


---------------------------------------------------------------------------------------------
--皇帝战

function This:getKingWarData()
	local playType = self:selectPlayType(GameMapType.kEmperorMap)

	if self.m_kingWarData then
		if playType ~= self.m_kingWarData:getPlayType() then
			self.m_kingWarData = nil
		end
	end

	if not self.m_kingWarData then
		self.m_kingWarData = require("LegionEntrant.KingWarData"):create(playType)
	end

	return self.m_kingWarData
end

-- 皇帝战信息
function This:onKingWarInfoRequest()
	Socket:sendAndLock(13520, {})
end

function This:onKingWarInfoReply(msg)
	-- print("====== onKingWarInfoReply")
	-- print_tmp(msg)

	local data = self:getKingWarData()

	if msg.err_code == 0 then	
		data:handlerKingWarInfoMsg(msg)
	elseif msg.err_code == 13518 then
		data:handlerKingWarErrCode13518()
	else
		g_scene:handleErrorCode(msg.err_code)
	end

	self:notifyObservers("onKingWarInfoReply", data)
	EventDispatcher:dispatcher("king_war_change")
end

-- 皇帝战结果
function This:onKingWarResultNotify(msg)
	-- print("====== onKingWarResultNotify")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getKingWarData()
		data:handlerKingWarResultMsg(msg)
		self:notifyObservers("onKingWarResultNotify", data)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 申请加入
function This:onKingWarApplyTeamRequest(teamId)
	Socket:sendAndLock(13522, {team_id = teamId})
	self:notifyObservers("onKingWarApplyTeamRequest", teamId)
end

function This:onKingWarApplyTeamReply(msg)
	-- print("====== onKingWarApplyTeamReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getKingWarData()
		data:handlerKingWarApplyTeamMsg(msg)
		self:notifyObservers("onKingWarApplyTeamReply", data)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 处理请求 agree = 1同意，0拒绝
function This:onKingWarDisposeApplyRequest(roleId, agree)
	Socket:sendAndLock(13523, {role_id = roleId, agree = agree})
end

-- (处理请求回复/有人申请推送)
function This:onKingWarDisposeApplyReply(msg)
	-- print("====== onKingWarDisposeApplyReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getKingWarData()
		local team = data:handlerKingWarDisposeApplyMsg(msg)
		self:notifyObservers("onKingWarDisposeApplyReply", data)
		EventDispatcher:dispatcher("king_war_change")
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

-- 邀请组队
function This:onKingWarInviteTeamRequest(id)
	Socket:sendAndLock(13524, {firend_id = id})
end

function This:onKingWarInviteTeamReply(msg)
	-- print("====== onKingWarInviteTeamReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		self:notifyObservers("onKingWarInviteTeamReply")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 收到邀请组队消息
function This:onKingWarInviteTeamPushNotify(msg)
	-- print("====== onKingWarInviteTeamPushNotify")
	-- print_tmp(msg)

	local data = self:getKingWarData()
	if data:isHasTeam() then
		return
	end

	local team = require("LegionEntrant.KingWarTeamModel"):create(msg.teaminfo)
	local leader = team:getLeaderInfo()
	
	InviteManager:addInvite({
		name = leader.name,
		level = leader.level,
		icon = leader.icon,
		rankLv = leader.rankLv,
		inviteText = Serialize[61550],
		onCheck = function (invite)
			if data:getState() ~= KingWarBtnState.kSignUp then
				return false
			end
			if data:isHasTeam() then
				return false
			end
			if GamePlayer:isInWorld() then
				return false
			end
		end,
		onSure = function (invite)
			if data:getState() ~= KingWarBtnState.kSignUp then
				g_scene:showTips(Serialize[61551])
				return
			end

			local info = invite:getInfo()
			self:onKingWarAgreeInviteRequest(info.teamId)
		end,
		onRefuse = function (invite)
			-- 不处理
		end,
		teamId = team:getId()			
	})	
end

-- 好友同意邀请
function This:onKingWarAgreeInviteRequest(teamId)
	Socket:sendAndLock(13526, {team_id = teamId})
end

function This:onKingWarAgreeInviteReply(msg)
	-- print("====== onKingWarAgreeInviteReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getKingWarData()
		local team = data:handlerKingWarAgreeInviteMsg(msg)		
		self:notifyObservers("onKingWarTeamChange", team)
		EventDispatcher:dispatcher("king_war_change")
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

-- 踢出队伍
function This:onKingWarKickOutTeamRequest(roleId)
	Socket:sendAndLock(13527, {role_id = roleId})
end

function This:onKingWarKickOutTeamReply(msg)
	-- print("====== onKingWarKickOutTeamReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		local data = self:getKingWarData()
		local team = data:handlerKingWarKickOutTeamMsg(msg)		
		self:notifyObservers("onKingWarTeamChange", team)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 进入皇帝战地图
function This:onKingWarEnterRequest()
	Socket:sendAndLock(13528)
end

function This:onKingWarEnterReply(msg)
	-- print("====== onKingWarEnterReply")
	-- print_tmp(msg)

	if msg.err_code == 0 then
		require("Map.GameMapManager"):enterEmperorMap()
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end


-- 申请红点（队长才有）
function This:isKingWarApplyRedDot()
	local data = self:getKingWarData()
	if not data:isHasTeam() then
		return false
	end

	if not data:isLeader() then
		return false
	end

	if data:getState() ~= KingWarBtnState.kSignUp then
		return false
	end

	local team = data:getMyTeam()
	local list = team:getRequestList()
	if #list > 0 then
		return true
	end

	return false
end

-- 是否参加了皇帝战
function This:isJoinKingWar()
	local data = self:getKingWarData()
	local state = data:getState()

	if state == KingWarBtnState.kSignUp or state == KingWarBtnState.kPrepare then
		return data:isHasTeam()
	end

	if state == KingWarBtnState.kFight then
		local team = data:getMyTeam()
		if not team then
			return false
		end

		if team:isSelfOut() or team:getIsOut() then
			return false
		end

		local resultRank = data:getResultRank()
		if #resultRank > 0 then
			return false
		end

		return true
	end

	return false
end