

-- 皇帝战申请列表

local This, super = class(..., "Dialog")
local GameUtil = require("Util.GameUtil")
local EntrantManager = require("LegionEntrant.EntrantManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local KingWarApplyListItemView = require("LegionEntrant.KingWarApplyListItemView")
local GamePlayer = require("Player.GamePlayer")


function This:initData(info)
	self.m_data = EntrantManager:getKingWarData()
	self.m_applyList = self.m_data:getApplyList()
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWarApply.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

   	local scrollView = csNode.apply.scrollView
	scrollView:setDirection("vertical")

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 624,
        itemHeight = 44,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getListData()	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getListData()	   
        	local data = list[index]
        	if data then     
			    if not node then
			    	node = KingWarApplyListItemView:create(data, index)

			    	node:setSureCb(function (item)
			    		local team = self.m_data:getMyTeam()
			    		if not team:isHasPos() then
			    			g_scene:showTips(Serialize[61554])
			    			return
			    		end

			    		local itemData = item:getData()
			    		EntrantManager:onKingWarDisposeApplyRequest(itemData.id, 1)
			    	end)

			    	node:setRefuseCb(function (item)
			    		local itemData = item:getData()
			    		EntrantManager:onKingWarDisposeApplyRequest(itemData.id, 0)
			    	end)

				    return node
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    
    self:refresh()
end

function This:refresh()
	self.m_listLayer:setDataList()
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

        csNode.close:setTouchEnded(function()
	    	self:close()
		end)

        csNode.apply.request:setTouchEnded(function()
	    	self:close()
		end)		
    end
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
    self:refresh()
end

function This:onExit()
	EntrantManager:removeObserver(self)
    super.onExit(self)
end

function This:getListData()
	return self.m_applyList
end

function This:onKingWarDisposeApplyReply(data)
	self.m_data = data
	self.m_applyList = self.m_data:getApplyList()
	self:refresh()
end