
-- 皇帝战数据

local This , super = class(..., "Base.BaseModel")
local ArgConfig = require("Metadatas.ArgConfig")
local EntrantManager = require("LegionEntrant.EntrantManager")
local PlayTypeConfig = require("Metadatas.PlayTypeConfig")
local GamePlayer = require("Player.GamePlayer")
local FamilyManager = require("Family.FamilyManager")
local LoginManager = require("Login.LoginManager")
local FriendManager = require("Friend.FriendManager")
local CompetitionArea = require("LegionEntrant.CompetitionArea")
local KingWarTeamModel = require("LegionEntrant.KingWarTeamModel")
local InviteManager = require("Invite.InviteManager")


-- 预告
-- 组队阶段
-- 进入地图倒计时
-- 开战阶段
-- 结果展示阶段


function This:init(playType)
	super.init(self)
   
   	self.m_playType = playType or 3
   	self.m_mapType = PlayTypeConfig[self.m_playType].map_type
	self.m_state = 0
	self.m_endTime = 0

	self.m_teams = {}
	self.m_myTeamId = 0
	self.m_loongRank = {}	
	self.m_resultRank = {}

	self:initState()
end

function This:initState()
	local info = EntrantManager:getButtonInfo(self.m_playType)
	self.m_state = info.state
	self.m_endTime = info.endTime

	if self.m_state == KingWarBtnState.kNotStart then
		self.m_teams = {}
		self.m_myTeamId = 0
		self.m_loongRank = {}	
		self.m_resultRank = {}
	end
end

function This:handlerKingWarInfoMsg(msg)
	
	self.m_myTeamId = msg.team_id

	-- 队伍列表
	local newTeams = {}
	local playerId = GamePlayer:getPlayerID()
	local found = false
	for _, info in ipairs(msg.team_list) do
		local team = self:updateTeam(info)
		newTeams[info.team_id] = team

		--这里做下容错，防止msg.team_id没传
		if team:isHasMember(playerId) then
			found = true
			self.m_myTeamId = team:getId() 
		end
	end
	self.m_teams = newTeams
	if not found then
		print( "the info of my team is not found" )
	end

	-- 龙气排行
	self.m_loongRank = {}
	for i, v in ipairs(msg.loong_rank_list) do
		local role = {}
		role.id = v.id
		role.name = v.name
		role.level = v.level
		role.icon = v.icon
		role.rankLv = v.rank
		role.title = v.title
		role.effect = v.effect
		role.serverId = v.server_id
		role.serverName = v.server_name
		role.loongVal = v.loong_value
		role.loongRank = v.loong_rank
		role.areaName = self:getServerAreaName(v.server_id)		
		table.insert(self.m_loongRank, role)
	end
	if #self.m_loongRank > 1 then
		table.sort(self.m_loongRank, function (a, b)
			if a.loongRank ~= b.loongRank then
				return a.loongRank < b.loongRank
			end
			return a.id < b.id
		end)
	end

	self:updateApplyTeams(msg.request_teamid_list)
end

-- 错误码13518：没有参战队伍信息
function This:handlerKingWarErrCode13518()
	self.m_myTeamId = 0
	self.m_teams = {}
	self.m_loongRank = {}
	self.m_resultRank = {}
end

function This:handlerKingWarResultMsg(msg)
	self.m_myTeamId = msg.team_id

	local playerId = GamePlayer:getPlayerID()
	for tid, team in pairs(self.m_teams) do
		if team:isHasMember(playerId) then
			self.m_myTeamId = team:getId() 
		end
	end

	self.m_resultRank = {}
	for i, v in ipairs(msg.king_war_rank_list) do
		local role = {}
		role.teamId = v.team_id
		role.leaderId = v.leader_id
		role.name = v.name
		role.level = v.level
		role.icon = v.icon
		role.rank = v.rank
		role.serverId = v.server_id
		role.serverName = v.server_name
		role.leftMember = v.left_member
		role.cityNum = v.city_num
		role.areaName = self:getServerAreaName(v.server_id)
		table.insert(self.m_resultRank, role)
	end

	if #self.m_resultRank > 1 then
		table.sort(self.m_resultRank, function (a, b)
			if a.rank ~= b.rank then
				return a.rank < b.rank
			end
			return a.id < b.id
		end)
	end
end

function This:handlerKingWarApplyTeamMsg(msg)
	self:updateApplyTeams(msg.request_teamid_list)
end

function This:handlerKingWarDisposeApplyMsg(msg)
	local team = self:updateTeam(msg.team)
	return team
end

function This:handlerKingWarAgreeInviteMsg(msg)
	local team = self:updateTeam(msg.teaminfo)

	local playerId = GamePlayer:getPlayerID()
	if msg.role_id == playerId then
		g_scene:showTips(Serialize[61558])
	end

	return team
end

function This:handlerKingWarKickOutTeamMsg(msg)
	local team = self:updateTeam(msg.teaminfo)

	local playerId = GamePlayer:getPlayerID()
	if msg.role_id == playerId then
		self.m_myTeamId = 0
		g_scene:showTips(Serialize[61559])
	end

	return team
end

function This:updateTeam(info)
	local id = info.team_id
	local team = self:getTeam(id)

	if not team then
		team = KingWarTeamModel:create(info)
		self.m_teams[id] = team
	else
		team:updateInfo(info)
	end

	return team
end

function This:updateApplyTeams(teamIdList)
	local dict = {}

	for _, id in ipairs(teamIdList) do
		dict[id] = true
	end

	for id, team in pairs(self.m_teams) do
		if dict[id] then
			team:setIsApply(true)
		else
			team:setIsApply(false)
		end
	end
end

function This:getState()
	local curTime = GamePlayer:getServerTime()
	if curTime > self.m_endTime then
		self:initState()
	end

	return self.m_state
end

function This:getStateEndTime()
	return self.m_endTime
end

function This:getStateLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return 0
	end
	return self.m_endTime - curTime	
end

function This:getPlayType()
	return self.m_playType
end

function This:getMapType()
	return self.m_mapType
end

function This:getServerAreaName(serverId)
	if not serverId then
		return ""
	end

	local ret = ""
	local server = LoginManager:getServer(serverId)
	if server then
		ret = server:getAreaName(true)
	end
	return ret
end

function This:getMyAreaRegionText()
	return CompetitionArea:getMyAreaRegionText()
end

function This:getMyMergeServerId()
	return CompetitionArea:getMergeServerId()
end

function This:getTeam(id)
	return self.m_teams[id]
end

function This:getTeamList()
	local list = {}

	for k, v in pairs(self.m_teams) do
		table.insert(list, v)
	end

	if #list > 1 then
		table.sort(list, function (a, b)
			local roleA = a:getLeaderInfo()
			local roleB = b:getLeaderInfo()
			if roleA.loongRank ~= roleB.loongRank then
				return roleA.loongRank < roleB.loongRank
			end
			return roleA.id < roleB.id
		end)
	end

	return list
end

function This:getMyTeamId()
	return self.m_myTeamId
end

function This:getMyTeam()
	return self:getTeam(self.m_myTeamId)
end

function This:isLeader()
	local teamId = self:getMyTeamId()
	if teamId <= 0 then
		return false
	end

	local team = self:getTeam(teamId)
	if not team then
		return false
	end
	local playerId = GamePlayer:getPlayerID()
	return team:getLeaderId() == playerId
end

function This:isHasTeam(roleId)
	if roleId then
		for _, team in pairs(self.m_teams) do
			if team:isHasMember(roleId) then
				return true
			end
		end
		return false
	else
		-- 检查自己 roleId = nil
		if self:getMyTeam() then
			return true
		end
		return false
	end
end

function This:getLoongRank()
	return self.m_loongRank
end

function This:getResultRank()
	return self.m_resultRank
end

function This:getApplyList()
	local team = self:getTeam(self.m_myTeamId)
	if team then
		return team:getRequestList()
	end
	return {}
end