

-- 皇帝战龙气排行

local This, super = class(..., "Dialog")
local GameUtil = require("Util.GameUtil")
local EntrantManager = require("LegionEntrant.EntrantManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local KingWarLoongRankItemView = require("LegionEntrant.KingWarLoongRankItemView")
local GamePlayer = require("Player.GamePlayer")


function This:initData(info)
	self.m_data = EntrantManager:getKingWarData()
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWarIntegralRank.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

   	local scrollView = csNode.scrollView
	scrollView:setDirection("vertical")

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 565,
        itemHeight = 47,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
		dataCounter = function ()
	        local rank = self:getRankData()	 
	        return #rank
        end,
        viewCreator =  function(index, node)
        	local rank = self:getRankData()	   
        	local data = rank[index]
        	if data then     
			    if not node then
				    return KingWarLoongRankItemView:create(data, index)
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    
    self:refresh()
end

function This:refresh()
	self.m_listLayer:setDataList()

	local csNode = self.m_csNode
	local myRank = self:getMyRank()
	local me = csNode.me
	
	if not myRank then
		me.paiming:setString(Serialize[60243])
		me.level:setString(Serialize[60243])
		me.name:setString(Serialize[60243])
		me.qufu:setString(Serialize[60243])
		me.hurt:setString(Serialize[60243])
	else
		me.paiming:setString(myRank.loongRank)
		me.level:setString(string.format("Lv%d", myRank.level))
		me.name:setString(myRank.name)
		me.qufu:setString(myRank.areaName)
		me.hurt:setString(myRank.loongVal)
	end
end

function This:initTouchEvent()
    if not self.m_inited then
    	self.m_inited = true

        self.m_csNode.beijing.close:setTouchEnded(function()
	    	self:close()
		end)
    end
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:onExit()
    super.onExit(self)
end

function This:getRankData()
	return self.m_data:getLoongRank()
end

function This:getMyRank()
	local list = self:getRankData()
	local playerId = GamePlayer:getPlayerID()

	for i, role in ipairs(list) do
		if role.id == playerId then
			return role
		end
	end

	return nil
end
