
-- 皇帝战主界面

local This, super = class(..., "Dialog")
local RulesPanel = require("UI.RulesPanel")
local EntrantManager = require("LegionEntrant.EntrantManager")


-- 未开始 KingWarBeforeNode
-- 预告 KingWarPreviewNode
-- 报名 KingWarSignUpNode
-- 开战倒计时 KingWarPrepareNode
-- 战斗中 KingWarFightNode
-- 结果排名 KingWarResultNode

function This:initData(info)	
	self.m_playType = info.playType
	self.m_mapType = info.mapType
	self.m_nodeView = nil
	self.m_nodeName = ""
	
	self.m_data = EntrantManager:getKingWarData()
	self.m_state = 0
	self.m_endTime = 0
	self.m_hasTimeUpdate = false
	self.m_escape = 0
	self:updateState()
	
	EntrantManager:onKingWarInfoRequest()
end

function This:refresh()
	local data = self.m_data
	local arg = {mainUI = self, data = data}

	if self.m_state == KingWarBtnState.kNotStart then
		-- 未开始
		self:openNodeView("LegionEntrant.KingWarBeforeNode", arg)

	elseif self.m_state == KingWarBtnState.kPreView then
		-- 预告
		self:openNodeView("LegionEntrant.KingWarPreviewNode", arg)

	elseif self.m_state == KingWarBtnState.kSignUp then
		-- 报名
		self:openNodeView("LegionEntrant.KingWarSignUpNode", arg)

	elseif self.m_state == KingWarBtnState.kPrepare then
		-- 备战
		self:openNodeView("LegionEntrant.KingWarPrepareNode", arg)

	elseif self.m_state == KingWarBtnState.kFight then
		-- 战斗，可能出现：进入地图 / 提前结束，出战斗结果
		local resultRank = data:getResultRank()
		if #resultRank > 0 then
			self:openNodeView("LegionEntrant.KingWarResultNode", arg)
		else
			self:openNodeView("LegionEntrant.KingWarFightNode", arg)
		end

	elseif self.m_state == KingWarBtnState.kEnd then
		-- 结束
		self:openNodeView("LegionEntrant.KingWarResultNode", arg)

	else
		self:openNodeView("LegionEntrant.KingWarBeforeNode", arg)
	end
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    EntrantManager:removeObserver(self)
end

function This:openNodeView(name, info)
	if self.m_nodeName == name then
		return
	end
	
	print("openNodeView", name)
	self:closeNodeView()

	local view = require(name):create(info)
	self:addChild(view)
	self.m_nodeView = view
	self.m_nodeName = name
	
	return view
end

function This:closeNodeView()
	if self.m_nodeView then
		self.m_nodeView:removeFromParent(true)
		self.m_nodeView = nil
	end
	self.m_nodeName = ""
end

function This:callNodeViewFunc(funcName, ...)
	if not self.m_nodeView then
		return
	end

	local func = self.m_nodeView[funcName]
    if type(func) == "function" then
		return func(self.m_nodeView, ...)
	end
end

function This:getLeftTime()
	return self.m_data:getStateLeftTime()
end

function This:update(dt)
    super.update(self, dt)
    if self.m_hasTimeUpdate then
		self.m_escape = self.m_escape + dt
		if self.m_escape >= 1.0 then
			self.m_escape = self.m_escape - 1
			local leftTime = self:getLeftTime()
			if leftTime > 0 then
				self:callNodeViewFunc("updateLeftTime", leftTime)
			else
				self.m_hasTimeUpdate = false
				self.m_escape = 0
				self:callNodeViewFunc("updateLeftTime", 0)
				EntrantManager:onKingWarInfoRequest()
			end
		end
	end
end

-- 皇帝战信息
function This:onKingWarInfoReply(data)
	self.m_data = data
	self:updateState()
	self:refresh()
end

-- 皇帝战结果
function This:onKingWarResultNotify(data)
	self.m_data = data
	self:updateState()
	self:refresh()
end

function This:updateState()
	local data = self.m_data
	self.m_state = data:getState()
	self.m_endTime = data:getStateEndTime()
	self.m_hasTimeUpdate = (self.m_state < KingWarBtnState.kEnd)
end