

-- 皇帝战结果排行

local This, super = class(..., "Dialog")
local GameUtil = require("Util.GameUtil")
local EntrantManager = require("LegionEntrant.EntrantManager")
local RankPageListLayer = require("Rank.RankPageListLayer")
local KingWarResultRankItemView = require("LegionEntrant.KingWarResultRankItemView")
local GamePlayer = require("Player.GamePlayer")


function This:initData(info)
	self.m_data = EntrantManager:getKingWarData()
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWarRank.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

   	local scrollView = csNode.scrollView
	scrollView:setDirection("vertical")

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 650,
        itemHeight = 47,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
		dataCounter = function ()
	        local rank = self:getRankData()	 
	        return #rank
        end,
        viewCreator =  function(index, node)
        	local rank = self:getRankData()	   
        	local data = rank[index]
        	if data then     
			    if not node then
				    return KingWarResultRankItemView:create(data, index)
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    
    self:refresh()
end

function This:refresh()
	self.m_listLayer:setDataList()

	local csNode = self.m_csNode
	local myRank = self:getMyRank()
	local me = csNode.me
	
	if not myRank then
		me.paiming:setString(Serialize["family_BuildNode_7"])
		me.level:setString(Serialize["family_BuildNode_7"])
		me.name:setString(Serialize["family_BuildNode_7"])
		me.qufu:setString(Serialize["family_BuildNode_7"])
		me.renshu:setString(Serialize["family_BuildNode_7"])
		me.city:setString(Serialize["family_BuildNode_7"])
	else
		me.paiming:setString(myRank.rank)
		me.level:setString(string.format("Lv%d", myRank.level))
		me.name:setString(myRank.name)
		me.qufu:setString(myRank.areaName)
		me.renshu:setString(myRank.leftMember)
		me.city:setString(myRank.cityNum)
	end
end

function This:initTouchEvent()
    if not self.m_inited then
    	self.m_inited = true

        self.m_csNode.beijing.close:setTouchEnded(function()
	    	self:close()
		end)
    end
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:onExit()
    super.onExit(self)
end

function This:getRankData()
	return self.m_data:getResultRank()
end

function This:getMyRank()
	local list = self:getRankData()
	local teamId = self.m_data:getMyTeamId()

	for i, v in ipairs(list) do
		if v.teamId == teamId then
			return v
		end
	end

	return nil
end
