

-- 皇帝战规则奖励节点

local This, super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")
local LegionMapKingRuleRewardConfig = require("Metadatas.LegionMapKingRuleRewardConfig")
local ItemNode = require("UI.ItemNode")


local RewardTypeTitle = {
	[0] = Serialize[61574],
	[1] = Serialize[61575],
}


function This:initData(info)
	self.m_rules = {}
	self.m_itemList = {}

	for k, v in pairs(LegionMapKingRuleRewardConfig) do
		if not self.m_rules[v.master] then
			self.m_rules[v.master] = {}
		end
		table.insert(self.m_rules[v.master], v)
	end

	for reward_type, list in pairs(self.m_rules) do
		table.sort(list, function (a, b)
			return a.id < b.id
		end)
	end
end

function This:initNode(info)
	local scrollview = cc.ScrollView:create()
	scrollview:setContentSize(750, 410)
	self:addChild(scrollview)

	local node = self:createRewardPanel()
	local w, h = node:getContentSize()
	scrollview:pushBackCustomItem(node, w, h, 0)
end

function This:createRewardPanel()
	local node = cc.Node:create()

	local addEmptyNode = function (w, h)
		local emptyNode = self:createEmptyNode(w, h)
		node:addChild(emptyNode)
		table.insert(self.m_itemList, emptyNode)
	end	

	local typeList = {1, 0}
	for _, rewardType in ipairs(typeList) do
		local titleNode = self:createTitleNode(RewardTypeTitle[rewardType])
		node:addChild(titleNode)
		table.insert(self.m_itemList, titleNode)

		addEmptyNode(740, 20)

		local list = self.m_rules[rewardType]
		for i, v in ipairs(list) do
			local rewardNode = self:createRewardNode(v.condition_list, v.reward_list)
			node:addChild(rewardNode)
			table.insert(self.m_itemList, rewardNode)

			if i == #list then
				addEmptyNode(740, 30)
			else
				addEmptyNode(740, 20)
			end
		end
	end

	local h = 10
	for i=#self.m_itemList, 1, -1 do
		local node = self.m_itemList[i]
		local nodeW, nodeH = node:getContentSize()
		node:setPosition(0, h)
		h = h + nodeH
	end

	node:setContentSize(750, h)
	node:setAnchorPoint(0.5, 0)

	return node
end

function This:createTitleNode(text)
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorRuleReward.csb")
	local w, h = node.bg1:getContentSize()
	node:setAnchorPoint(0, 0)
	node:setContentSize(740, h)
	node.name1:setString(text)

	node.m_type = "TitleNode"	
	return node
end

function This:createRewardNode(condition, reward)
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorRuleRewardItem.csb")
	node:setAnchorPoint(0, 0)
	node:setContentSize(740, 60)

	if condition[1] == condition[2] then
		node.num:setString( string.format(Serialize[60698], condition[1]) )
	else
		node.num:setString( string.format(Serialize[60699], condition[1], condition[2]) )
	end

	for i, v in ipairs(reward) do
		local posNode = node[string.format("item%d", i)]
		if posNode then
			local item = ItemNode:create(v[1], v[2])
			item:setAnchorPoint(0.5, 0.5)
			item:setPosition( posNode:getPosition() )
			item:setScale(0.75)
			node:addChild(item)
		end
	end

	node.m_type = "RewardNode"
	return node
end

function This:createEmptyNode(w, h)
	local node = cc.Node:create()
	node:setContentSize(w, h)
	node.m_type = "EmptyNode"
	return node
end

