

-- 皇帝战报名界面

local This, super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")
local RankPageListLayer = require("Rank.RankPageListLayer")
local KingWarTeamItemView = require("LegionEntrant.KingWarTeamItemView")
local InviteListNode = require("Invite.InviteListNode")
local GameMapManager = require("Map.GameMapManager")
local EntrantManager = require("LegionEntrant.EntrantManager")
local EventDispatcher = require("EventDispatcher")
local GamePlayer = require("Player.GamePlayer")


function This:initData(info)
	self.m_mainUI = info.mainUI
	self.m_data = info.data
	self.m_mapType = info.data:getMapType()
	self.m_leftTime = info.data:getStateLeftTime()
	self.m_teamList = info.data:getTeamList()
	self.m_inviteNode = false
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWarTeam.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    -- 队伍列表
   	local scrollView = csNode.ScrollView
	scrollView:setDirection("vertical")
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 143,
        itemHeight = 212,
        spaceX = 15,
        spaceY = 0,
        column = 6,
        scrollView = scrollView,
		dataCounter = function ()
	        local list = self:getTeamList()	 
	        return #list
        end,
        viewCreator =  function(index, node)
        	local list = self:getTeamList()	   
        	local team = list[index]
        	if team then     
			    if not node then
				    return KingWarTeamItemView:create({team = team, kingWarData = self.m_data}, index)
				else
			        node:setModel({team = team, kingWarData = self.m_data}, index)
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)

    -- 邀请列表
    self.m_inviteNode = InviteListNode:create({
    	csNode = csNode,
    	isMaskLayer = true,
        itemWidth = 255,
        itemHeight = 90,
        itemFile = "Invite.InviteListItemView",
    	onInvite = function (info)
    		local id = info.node:getId()
    		EntrantManager:onKingWarInviteTeamRequest(id)
    	end,
    	onSelector = function (item)
            local otype = item.type
            if otype == "family" then
                if item.obj:getId() == GamePlayer:getPlayerID() then
                    return false
                end
            end
    		return item.obj:isOnline()
    	end,
    	onSort = function (listType, a, b)
    		local objA = a.obj
    		local objB = b.obj

    		if listType == "friend" then
    			local hasTeamA = self.m_data:isHasTeam( objA:getId() ) and 1 or 0
    			local hasTeamB = self.m_data:isHasTeam( objB:getId() ) and 1 or 0
    			if hasTeamA ~= hasTeamB then
    				return hasTeamA < hasTeamB
    			end

    			local levelA = objA:getLevel()
				local levelB = objB:getLevel()
				if levelA ~= levelB then
					return levelA > levelB
				end

				return objA:getId() < objB:getId()

    		elseif listType == "family" then
    			local hasTeamA = self.m_data:isHasTeam( objA:getId() ) and 1 or 0
    			local hasTeamB = self.m_data:isHasTeam( objB:getId() ) and 1 or 0
    			if hasTeamA ~= hasTeamB then
    				return hasTeamA < hasTeamB
    			end

    			local titleA = objA:getTitle()
			    local titleB = objB:getTitle()
			    if titleA ~= titleB then
			        return titleA < titleB
			    end

			    return objA:getId() < objB:getId()
    		end
    		
    		return false
    	end,
    	onRefresh = function (item)
    		local csNode = item.m_csNode
    		local otype = item.m_data.type

    		if otype == "friend" then
            	item:refreshFriend()
        	elseif otype == "family" then
            	item:refreshFamily()
        	end

    		if self.m_data:isHasTeam( item:getId() ) then
    			csNode.gaming:setString(Serialize[61577])
    		 	csNode.gaming:setVisible(true)
    		 	csNode.online:setVisible(false)
    			csNode.outline:setVisible(false)
    			csNode.button:setVisible(false)
    			csNode.cd:setVisible(false)
    			item.m_isUpdate = false
    		end
    	end
    })
    self.m_inviteNode:setState(0)
    self:addChild(self.m_inviteNode)


	self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        local csNode = self.m_csNode

		-- 关闭
		csNode.close:setTouchEnded(function()
		    self.m_mainUI:close()
		end)    

		-- 龙气排行
		csNode.btnRank:setTouchEnded(function()
			g_scene:openDialog("LegionEntrant.KingWarLoongRankPanel")
		end)

		-- 申请列表
		csNode.btnApply:setTouchEnded(function()
			g_scene:openDialog("LegionEntrant.KingWarApplyListPanel")
		end)

		-- 邀请好友
		csNode.btnInvite:setTouchEnded(function()
			self.m_inviteNode:runOpenAni()
		end)

		-- 进入地图
		csNode.btnEnter:setTouchEnded(function()
			EntrantManager:onKingWarEnterRequest()
		end)

        if csNode.info then
            csNode.info:setTouchEnded(function()
                g_scene:openDialog("UI.RulesPanel", {rulesType = RulesType.kKing})
            end)
        end
    end	 
end

function This:onEnter()
	super.onEnter(self)
	EntrantManager:addObserver(self)
	--GameMapManager:getEmperorMap():addObserver(self)
	EventDispatcher:addListener(self, "king_war_show_invite_list", self.onShowInviteList)
end

function This:onExit()
    EntrantManager:removeObserver(self)
    --GameMapManager:getEmperorMap():removeObserver(self)
    EventDispatcher:removeListener(self, "king_war_show_invite_list")
	super.onExit(self)
end

function This:refresh()
	local csNode = self.m_csNode
	local data = self.m_data
	local isLeader = data:isLeader()

	csNode.qufu:setString( data:getMyAreaRegionText() )
	csNode.zhandouzhong:setVisible(false)
	csNode.btnEnter:setVisible(false)
	csNode.btnRank:setVisible(true)
	csNode.btnApply:setVisible(isLeader)
	csNode.btnInvite:setVisible(isLeader)

	self.m_listLayer:setDataList()
	self:updateLeftTime(self.m_leftTime)
    self:refreshRedDot()
end

function This:refreshRedDot()
    GameUtil:refreshRedDot(self.m_csNode.btnApply, EntrantManager:isKingWarApplyRedDot())
end

function This:updateLeftTime(leftTime)
	self.m_leftTime = leftTime
	self.m_csNode.time:setString( GameUtil:formatDTime(Serialize[61578], leftTime) )
end

function This:getTeamList()
	return self.m_teamList
end

function This:onShowInviteList()
	self.m_inviteNode:runOpenAni()
end

function This:onKingWarDisposeApplyReply(data)
	self.m_data = data
	self.m_teamList = self.m_data:getTeamList()	
	self:refresh()
end

function This:onKingWarTeamChange(team)
	self.m_teamList = self.m_data:getTeamList()	
	self:refresh()
    self.m_inviteNode:reload()
end