

-- 皇帝战队伍信息

local This, super = class(..., "Dialog")
local GameUtil = require("Util.GameUtil")
local EntrantManager = require("LegionEntrant.EntrantManager")
local GamePlayer = require("Player.GamePlayer")
local EventDispatcher = require("EventDispatcher")


local PanelType = {
    kInfo = "info",
    kApply = "apply",
}

local ErrorType = {
	kNone = 0,
	kNoLeader = 1,
	kNoSignUp = 2,
	kNoHasPos = 3,
	kSelf = 4,
}

function This:initData(info)
	self.m_team = info.team
	self.m_data = EntrantManager:getKingWarData()
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWarTeamPeople.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
    	local csNode = self.m_csNode
    	self.m_inited = true

    	-- 关闭
    	csNode.closebutton:setTouchEnded(function()
			self:close()
		end)

		-- 中间的确定
		csNode.button:setTouchEnded(function()
			self:close()
		end)

    	-- 取消
        csNode.button1:setTouchEnded(function()
	    	self:close()
		end)

		-- 申请
		csNode.button2:setTouchEnded(function()
			if GamePlayer:isInMatchMap() then
				g_scene:showTips(Serialize[61580])
				return
			end

			local curTime = GamePlayer:getServerTime()
			self.m_team:setLastApplyTime(curTime)
			self.m_team:setIsApply(true)
			EntrantManager:onKingWarApplyTeamRequest( self.m_team:getId() )
			self:close()
		end)

		-- 队员
		local nodeList = {csNode.item1, csNode.item2, csNode.item3}
		for i=1, 3 do
			local node = nodeList[i]

			-- 邀请
			node.jiahao:setTouchEnded(function()
				local ret, err = self:isCanInvite()
				if not ret then
					if err == ErrorType.kSignUp then
						g_scene:showTips(Serialize[61581])
					end
					return
				end

				EventDispatcher:dispatcher("king_war_show_invite_list")
	    		self:close()
			end)

			-- 踢出
			node.jianhao:setTouchEnded(function()
				local role = self.m_team:getMemberList()[i]
				if not role then
					return
				end

				local ret, err = self:isCanKickOut(role)
				if not ret then
					if err == ErrorType.kSignUp then
						g_scene:showTips(Serialize[61582])
					end
					return
				end

				g_scene:openDialog("UI.CostConfirmPanel", {
                    simple = string.format(Serialize[61583], role.name), 
                    okHandler = function()
                        EntrantManager:onKingWarKickOutTeamRequest(role.id)
                    end}
                )
			end)
		end
    end
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    EntrantManager:removeObserver(self)	
    super.onExit(self)
end

function This:getPanelType()
    local team = self.m_team
    local kwData = self.m_data
    local state = kwData:getState()
    local mySid = kwData:getMyMergeServerId()
    local leaderSid = team:getLeaderServerId()

    if state == KingWarBtnState.kSignUp and team:isHasPos() and not kwData:isHasTeam() and mySid == leaderSid then
        return PanelType.kApply
    end

    return PanelType.kInfo
end

function This:refresh()
	local ptype = self:getPanelType()

	if ptype == PanelType.kApply then
		self:refreshApplyPanel()
	else
		self:refreshInfoPanel()
	end
end

--  显示信息
function This:refreshInfoPanel()
	local csNode = self.m_csNode
	local team = self.m_team

	local memberList = team:getMemberList()
	local nodeList = {csNode.item1, csNode.item2, csNode.item3}
	local isCanInvite = self:isCanInvite()
	for i=1, 3 do
		local node = nodeList[i]
		local role = memberList[i]
		local isHas = (role ~= nil)

		node.kong:setVisible(not isHas)
		node.jiahao:setVisible(not isHas)
		node.zhugong:setVisible(isHas)
		node.zhugongdi:setVisible(isHas)
		node.name:setVisible(isHas)
		node.level:setVisible(isHas)
		node.levelbg:setVisible(isHas)
		node.icon:setVisible(isHas)
		node.jianhao:setVisible(isHas)
		
		if isHas then
	    	GameUtil:setPlayerIconNodeImage(node.icon, role.icon)
	    	GameUtil:setPlayerEffectNodeTo(node.icon, role.effect)
	    	node.level:setString(role.level)
	    	node.name:setString(role.name)
	    	node.zhugong:setVisible(role.isLeader)
			node.zhugongdi:setVisible(role.isLeader)
			node.jianhao:setVisible( self:isCanKickOut(role) )

			if role.isOut then
				node.icon:setGrayShader()
			else
				node.icon:clearShader()
			end
		else
			node.jiahao:setVisible(isCanInvite)
		end
	end

	csNode.title:setString(Serialize[61584])
	csNode.tips:setVisible(false)
	csNode.button1:setVisible(false)
	csNode.button2:setVisible(false)
	csNode.button:setVisible(true)
end

-- 显示申请
function This:refreshApplyPanel()
	local csNode = self.m_csNode
	local team = self.m_team

	local memberList = team:getMemberList()
	local nodeList = {csNode.item1, csNode.item2, csNode.item3}
	for i=1, 3 do
		local node = nodeList[i]
		local role = memberList[i]
		local isHas = (role ~= nil)

		node.kong:setVisible(not isHas)
		node.jiahao:setVisible(false)
		node.zhugong:setVisible(isHas)
		node.zhugongdi:setVisible(isHas)
		node.name:setVisible(isHas)
		node.level:setVisible(isHas)
		node.levelbg:setVisible(isHas)
		node.icon:setVisible(isHas)
		node.jianhao:setVisible(false)

		if isHas then
	    	GameUtil:setPlayerIconNodeImage(node.icon, role.icon)
	    	GameUtil:setPlayerEffectNodeTo(node.icon, role.effect)
	    	node.level:setString(role.level)
	    	node.name:setString(role.name)
	    	node.zhugong:setVisible(role.isLeader)
			node.zhugongdi:setVisible(role.isLeader)
		end
	end

	csNode.title:setString(Serialize[61585])
	csNode.tips:setVisible(true)
	csNode.button1:setVisible(true)
	csNode.button2:setVisible(true)
	csNode.button:setVisible(false)
end

function This:isSignUpPhase()
	return self.m_data:getState() == KingWarBtnState.kSignUp
end

function This:isLeader()
	return GamePlayer:getPlayerID() == self.m_team:getLeaderId()
end

function This:isCanInvite()
	if not self:isLeader() then
		return false, ErrorType.kNoLeader
	end

	if not self:isSignUpPhase() then
		return false, ErrorType.kNoSignUp
	end

	if not self.m_team:isHasPos() then
		return false, ErrorType.kNoHasPos
	end

	return true, ErrorType.kNone
end

function This:isCanKickOut(role)
	if not self:isLeader() then
		return false, ErrorType.kNoLeader
	end

	if not self:isSignUpPhase() then
		return false, ErrorType.kNoSignUp
	end

	if GamePlayer:getPlayerID() == role.id then
		return false, ErrorType.kSelf
	end

	return true, ErrorType.kNone	
end

function This:onKingWarTeamChange(team)
	if self.m_team:getId() == team:getId() then
		self.m_team = team
		self:refresh()
	end	
end