

local This , super = class(...,"NodeUI")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local EntrantManager = require("LegionEntrant.EntrantManager")

local ButtonState = {
    kInfo = "info",
    kApply = "apply",
    kRequest = "request",
    kOut = "out"
}

-- 申请间隔
local ApplyInterval= 10


function This:initData(data)
    self.m_data = data
    self.m_isUpdate = false
end

function This:initNode(data)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWarTeamItem.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local box = csNode.bg0:getBoundingBox()
    self:setContentSize(box.width, box.height+10)
end

function This:initTouchEvent()
    if not self.m_isInit then
        local csNode = self.m_csNode
        self.m_isInit = true

        csNode.button:setTouch({
            isDynamic = true, 
            isContainer = true,
            onEnded = function ()
                local btnState = self:getButtonState()
                if btnState == ButtonState.kRequest then
                    return
                end

                g_scene:openDialog("LegionEntrant.KingWarTeamInfoPanel", {team = self.m_data.team})
            end,
        })
    end
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
    self:scheduleUpdate()
    self:refresh()
end

function This:onExit()
    EntrantManager:removeObserver(self)
    super.onExit(self)
end

function This:setModel(data)
    self.m_data = data
    self:refresh()
end

function This:refresh()
    local node = self.m_csNode
    local team = self.m_data.team
    local kingWarData = self.m_data.kingWarData
    local memberList = team:getMemberList()
    local leader = memberList[1] -- 队长

    GameUtil:setPlayerIconNodeImage(node.icon, leader.icon)
    GameUtil:setPlayerEffectNodeTo(node.icon, leader.effect)
    node.level:setString(leader.level)
    node.name:setString(leader.name)
    node.qufu:setString(leader.areaName)
    node.rank:setString(leader.loongRank)
    
    local curNum = #memberList
    local maxNum = team:getMaxMemberNum()
    node.member:setString(string.format("%d/%d", curNum, maxNum))
    node.member:setVisible(curNum < maxNum)
    node.membertext:setVisible(curNum < maxNum)
    node.yiman:setVisible(curNum >= maxNum)

    -- 我的队伍
    node.qizhi:setVisible( team:getIsMyTeam() )
    node.myteam:setVisible( team:getIsMyTeam() )

    -- 按钮
    self:refreshButton()
end

function This:refreshButton()
    local btnState = self:getButtonState()
    local button = self.m_csNode.button
    self.m_isUpdate = false

    if btnState == ButtonState.kInfo then
        button:setImage("Common/3jibuttonyellow.png")
        button.text:setString(Serialize[61587])

    elseif btnState == ButtonState.kApply then
        button:setImage("Common/3jibuttonblue.png")
        button.text:setString(Serialize[61588])

    elseif btnState == ButtonState.kRequest then
        button:setImage("Common/3jibuttonblue.png")
        local cd = self:getApplyCd()
        if cd > 0 then
            self.m_isUpdate = true
            button.text:setString(string.format(Serialize[61501], cd))
        else
            button.text:setString(Serialize[61588])
        end

    elseif btnState == ButtonState.kOut then   
        button:setImage("Common/3jibuttonred.png")
        button.text:setString(Serialize[61561])

    end
end

function This:getButtonState()
    local team = self.m_data.team
    local kwData = self.m_data.kingWarData
    local state = kwData:getState()
    local mySid = kwData:getMyMergeServerId()
    local leaderSid = team:getLeaderServerId()

    if state == KingWarBtnState.kSignUp and team:isHasPos() and not kwData:isHasTeam() and mySid == leaderSid then
        if team:getIsApply() and self:getApplyCd() > 0 then
            return ButtonState.kRequest
        end
        return ButtonState.kApply
    end

    if state == KingWarBtnState.kFight and team:getIsOut() then
        return ButtonState.kOut
    end

    return ButtonState.kInfo
end

function This:getApplyCd()
    local lastApplyTime = self.m_data.team:getLastApplyTime() or 0
    local curTime = GamePlayer:getServerTime()
    local delta = curTime - lastApplyTime

    if delta >= ApplyInterval then
        return 0
    end
    return math.floor(ApplyInterval - delta)
end

function This:update(deltaTime)
    super.update(self, deltaTime)

    if not self.m_isUpdate then
        return
    end

    local button = self.m_csNode.button
    local cd = self:getApplyCd()
    if cd > 0 then
        button.text:setString(string.format(Serialize[61501], cd))
    else
        self:refreshButton()
    end
end

function This:onKingWarApplyTeamRequest(teamId)
    local team = self.m_data.team
    if team:getId() == teamId then
        self:refresh()
    end      
end