

local This , super = class(..., "Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local LoginManager = require("Login.LoginManager")


function This:init(info)
	super.init(self)

	self.m_id = info.team_id
	self.m_leaderId = 0
	self.m_members = {}
	self.m_requestList = {}
	self.m_isOut = false
	self.m_isLeaderEnter = false
	self.m_isApply = false
	self.m_isMyTeam = false
	self.m_curMemberNum = 0
	self.m_maxMemberNum = 3
	self.m_lastApplyTime = 0

	self:updateInfo(info)
end

function This:updateInfo(info)
	local leaderId = info.leader_id
	local playerId = GamePlayer:getPlayerID()

	self.m_leaderId = leaderId
	self.m_members = {}
	self.m_curMemberNum = #info.member_list
	self.m_isMyTeam = false
	for _, v in ipairs(info.member_list) do
		local role = {}
		role.id = v.id
		role.name = v.name
		role.level = v.level
		role.icon = v.icon
		role.rankLv = v.rank
		role.title = v.title
		role.effect = v.effect
		role.serverId = v.server_id
		role.serverName = v.server_name
		role.loongVal = v.loong_value
		role.loongRank = v.loong_rank
		role.areaName = self:getServerAreaName(v.server_id)
		role.isLeader = (v.id == leaderId)
		role.isOut = (v.out == 1)
		self.m_members[v.id] = role

		if v.id == playerId then
			self.m_isMyTeam = true
		end
	end

	self.m_requestList = {}
	for _, v in ipairs(info.request_list) do
		local role = {}
		role.id = v.id
		role.name = v.name
		role.level = v.level
		role.icon = v.icon
		role.rankLv = v.rank
		role.title = v.title
		role.effect = v.effect
		role.serverId = v.server_id
		role.serverName = v.server_name
		role.loongVal = v.loong_value
		role.loongRank = v.loong_rank
		role.areaName = self:getServerAreaName(v.server_id)			
		table.insert(self.m_requestList, role)
	end

	self.m_isOut = (info.out == 2) -- 0未进入地图，1已进入地图，2已淘汰
	self.m_isLeaderEnter = (info.out >= 1)
end

function This:getId()
	return self.m_id
end

function This:getLeaderId()
	return self.m_leaderId
end

function This:getLeaderInfo()
	for _, role in pairs(self.m_members) do
		if role.isLeader then
			return role
		end
	end
end

function This:getLeaderServerId()
	local role = self:getLeaderInfo()
	if role then
		return role.serverId
	end
	return 0
end

function This:getMemberList()
	local list = {}

	for _, role in pairs(self.m_members) do
		table.insert(list, role)
	end

	if #list > 1 then
		table.sort(list, function (a, b)
			local leaderA = a.isLeader and 1 or 0
			local leaderB = b.isLeader and 1 or 0
			if leaderA ~= leaderB then
				return leaderA > leaderB
			end
			return a.id < b.id
		end)
	end

	return list
end

function This:getRequestList()
	return self.m_requestList
end

function This:getIsOut()
	return self.m_isOut
end

function This:setIsApply(isApply)
	self.m_isApply = isApply
end

function This:getIsApply()
	return self.m_isApply	
end	

function This:getIsMyTeam()
	return self.m_isMyTeam
end

function This:getCurMemberNum()
	return self.m_curMemberNum
end

function This:getMaxMemberNum()
	return self.m_maxMemberNum
end

function This:setLastApplyTime(time)
	self.m_lastApplyTime = time
end

function This:getLastApplyTime()
	return self.m_lastApplyTime
end

function This:getServerAreaName(serverId)
	if not serverId then
		return ""
	end

	local ret = ""
	local server = LoginManager:getServer(serverId)
	if server then
		ret = server:getAreaName(true)
	end
	return ret
end

function This:isHasPos()
	return self:getCurMemberNum() < self:getMaxMemberNum()
end

function This:isHasMember(roleId)
	return self.m_members[roleId] and true or false
end

function This:isSelfOut()
	local playerId = GamePlayer:getPlayerID()
	local role = self.m_members[playerId]
	if role then
		return role.isOut
	end
	return false
end