
-- 军团战数据

local This , super = class(..., "Base.BaseModel")
local ArgConfig = require("Metadatas.ArgConfig")
local EntrantManager = require("LegionEntrant.EntrantManager")
local PlayTypeConfig = require("Metadatas.PlayTypeConfig")
local GamePlayer = require("Player.GamePlayer")
local FamilyManager = require("Family.FamilyManager")
local LoginManager = require("Login.LoginManager")


function This:init(playType)
	super.init(self)
   
   	self.m_playType = playType or 3
   	self.m_mapType = PlayTypeConfig[self.m_playType].map_type
	self.m_state = 0
	self.m_endTime = 0
	self.m_fightFlag = 0

	self.m_curResponse = 0
	self.m_maxResponse = ArgConfig["legion_war_personal"].arg[1]
	self.m_isResponse = false

	-- 本场结果
	self.m_resultFlag = 0
	self.m_groupList = {}
	self.m_groupRank = {}
	self.m_roleRank = {}
	self.m_myRank = false


	self:initState()
end

function This:initState()
	local info = EntrantManager:getButtonInfo(self.m_playType)
	self.m_state = info.state
	self.m_endTime = info.endTime

	if self.m_state == KingWarBtnState.kNotStart then
		self.m_fightFlag = 0
		self.m_curResponse = 0
		self.m_isResponse = false
		self.m_resultFlag = 0
		self.m_groupList = {}
		self.m_groupRank = {}
		self.m_roleRank = {}
	end
end

function This:getPlayType()
	return self.m_playType
end

function This:getMapType()
	return self.m_mapType
end

function This:handlerLegionWarInfoMsg(msg)
	self.m_fightFlag = msg.fight_flag --0没有参加，1有参加，2还没有参战数据（预告）

	if msg.fight_flag == 2 then
		self.m_fightFlag = 0
	end
end

function This:handlerLegionWarResponseMsg(msg)
	self.m_curResponse = msg.times
	self.m_isResponse = (msg.signupflag == 1)

	if self.m_curResponse >= self.m_maxResponse then
		self.m_fightFlag = 1
	else
		self.m_fightFlag = 0
	end
end

function This:handlerLegionWarResultMsg(msg)
	self.m_resultFlag = msg.can_fight -- 0没报名，1等待分组，2分组成功，3本团被淘汰，4战斗结束

	self.m_groupList = {} -- 本场报名列表
	self.m_groupRank = {} -- 本场军团排名

	for i, v in ipairs(msg.group_list) do
    	local t = {}
    	t.index 		= i
    	t.rank 			= v.rank -- 没被淘汰的时候排名是0
    	t.groupId 		= v.group_id
    	t.groupName 	= v.group_name
    	t.serverId 		= v.server_id
    	t.serverName	= v.server_name
    	t.indexInGroup 	= self:getServerIndexInGroup(v.server_id)
    	table.insert(self.m_groupList, t)

    	if v.rank > 0 then
    		table.insert(self.m_groupRank, t)
    	end
	end

	if #self.m_groupRank > 1 then
		table.sort(self.m_groupRank, function (a, b)
			return a.rank < b.rank
		end)
	end

	-- 本场角色排名
	local playerId = GamePlayer:getPlayerID()
	self.m_roleRank = {}
	self.m_myRank = false
	for i, v in ipairs(msg.role_rank_info.role_list) do
		if v.rank > 0 then
			local t = {}
			t.id 			= v.id
			t.name 			= v.name
			t.rank 			= v.rank
			t.groupName 	= v.group_name
			t.icon 			= v.icon
			t.value 		= v.value
			t.serverId 		= v.server_id
			t.serverName 	= v.server_name
			t.indexInGroup  = self:getServerIndexInGroup(v.server_id)
			t.level 		= v.level
			table.insert(self.m_roleRank, t)

			if playerId == v.id then
				self.m_myRank = t
			end
		end
	end
	if #self.m_roleRank > 1 then
		table.sort(self.m_roleRank, function (a, b)
			return a.rank < b.rank
		end)
	end
	
end

function This:getState()
	local curTime = GamePlayer:getServerTime()
	if curTime > self.m_endTime then
		self:initState()
	end

	return self.m_state
end

function This:getStateEndTime()
	return self.m_endTime
end

function This:getStateLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return 0
	end
	return self.m_endTime - curTime	
end

function This:getCurResponse()
	return self.m_curResponse
end

function This:getMaxResponse()
	return self.m_maxResponse
end

function This:getIsResponse()
	return self.m_isResponse
end

function This:getFightFlag()
	return self.m_fightFlag
end

function This:getGroupList()
	return self.m_groupList
end

function This:getServerIndexInGroup(serverId)
	if not serverId then
		return 0
	end

	local idx = serverId
	local server = LoginManager:getServer(serverId)
	if server then
		idx = server:getIndexInGroup()
	end
	return idx
end

function This:getGroupRank()
	return self.m_groupRank
end

function This:getRoleRank()
	return self.m_roleRank
end

function This:getMyRank()
	return self.m_myRank
end

function This:getResultFlag()
	return self.m_resultFlag
end

function This:getFamilyId()
	return FamilyManager:getMyFamilyId() or 0
end

-- 是否被淘汰
function This:isOut()
	return (self.m_resultFlag >= 3)
end