

-- 军团战开战界面


local This, super = class(...,"NodeView")
local RankPageListLayer = require("Rank.RankPageListLayer")
local LegionWarPrepareItemView = require("LegionEntrant.LegionWarPrepareItemView")
local GameUtil = require("Util.GameUtil")
local EntrantManager = require("LegionEntrant.EntrantManager")
local GameMapManager = require("Map.GameMapManager")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")


function This:initData(info)
	self.m_mainUI = info.mainUI
	self.m_data = info.data
	self.m_mapType = info.data:getMapType()
    self.m_isInitListLayer = false
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/LegionWarGroup.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    csNode.bg1:adapterFullScreen()
	csNode.close:adapterBorder("right", "top", -cc.visible.designWidth/2, -cc.visible.designHeight/2)
	csNode.guize:adapterBorder("right", "bottom", -cc.visible.designWidth/2, -cc.visible.designHeight/2)
	csNode.toumingzhuang:adapterBorder("right", "bottom", -cc.visible.designWidth/2, -cc.visible.designHeight/2 )
	csNode.into:setVisible(true)
	csNode.time:setVisible(false)

	-- 关闭
	csNode.close:setTouchEnded(function()
	    self.m_mainUI:close()
	end) 

	-- 规则
	csNode.guize:setTouchEnded(function()
        g_scene:openDialog("UI.RulesPanel", {rulesType = RulesType.kLegion})
	end)

	-- 投名状
	csNode.toumingzhuang:setTouchEnded(function()
		g_scene:openDialog("FamilyRating.FamilWarlordPanel",{warType = self.m_mapType})
	end)

	-- 进入
	csNode.into:setTouchEnded(function()
		GameMapManager:enterLegionMap()
	end)


   	local scrollView = self.m_csNode.ScrollView
	scrollView:setDirection("vertical")

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 440,
        itemHeight = 39,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
		dataCounter = function ()
	        local rank = self:getRankData()	 
	        return #rank
        end,
        viewCreator =  function(index, node)
        	local rank = self:getRankData()	   
        	local data = rank[index]
        	if data then     
			    if not node then
				    return LegionWarPrepareItemView:create(data, index)
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)

end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
    GameMapManager:getLegionMap():addObserver(self)
	FamilyRatingManager:addObserver(self)
	self:initRed()    

    if not self.m_isInitListLayer then
    	self.m_isInitListLayer = true
    	self.m_listLayer:setDataList()
    end    
end

function This:onExit()
    EntrantManager:removeObserver(self)
    GameMapManager:getLegionMap():removeObserver(self)
    FamilyRatingManager:removeObserver(self)    
    super.onExit(self)
end

function This:getRankData()
	return self.m_data:getGroupList()
end

function This:onLegionWarResultReply(data)
	self.m_data = data

	local resultFlag = data:getResultFlag()
	if resultFlag == 4 then
		return
	end

	self.m_isInitListLayer = true
	self.m_listLayer:setDataList()
end

function This:onGetMapInfoFail(msg)
	EntrantManager:onLegionWarInfoRequest()
	
	if self.m_data:isOut() then
		g_scene:showTips(Serialize[61593])
		return
	end
	
	g_scene:handleErrorCode(msg.err_code)
end

function This:initRed()
	local node = self.m_csNode.toumingzhuang
	if node then
		GameUtil:refreshRedDot(node,FamilyRatingManager:hasCastRed(self.m_mapType),{80,75},1)
	end
end

function This:onCastReward()
	self:initRed()
end

function This:onGetCastInfo()
	self:initRed()
end
