
-- 军团战主界面

local This, super = class(..., "Dialog")
local RulesPanel = require("UI.RulesPanel")
local EntrantManager = require("LegionEntrant.EntrantManager")
local FamilyRatingManager = require("FamilyRating.FamilyRatingManager")

-- 未开始 LegionWarBeforeNode
-- 预告 LegionWarPreviewNode
-- 报名 LegionWarResponseNode
-- 开战倒计时 LegionWarPrepareNode
-- 战斗中 LegionWarFightNode
-- 结果排名 LegionWarResultNode



function This:initData(info)	
	self.m_playType = info.playType
	self.m_mapType = info.mapType
	self.m_nodeView = nil
	self.m_nodeName = ""
	
	self.m_data = EntrantManager:getLegionWarData()
	self.m_state = 0
	self.m_endTime = 0
	self.m_hasTimeUpdate = false
	self.m_escape = 0
	self:updateState()
	
	EntrantManager:onLegionWarInfoRequest()
	FamilyRatingManager:onGetCastInfoRequest()
	FamilyRatingManager:onGetCastInfoScheduleRequest()	
end

function This:initNode()
	local data = self.m_data
	local arg = {mainUI = self, data = data}

	if self.m_state == KingWarBtnState.kNotStart then
		-- 未开始
		self:openNodeView("LegionEntrant.LegionWarBeforeNode", arg)

	elseif self.m_state == KingWarBtnState.kPreView then
		-- 预告
		self:openNodeView("LegionEntrant.LegionWarPreviewNode", arg)

	elseif self.m_state == KingWarBtnState.kSignUp then
		-- 报名
		self:openNodeView("LegionEntrant.LegionWarResponseNode", arg)

	elseif self.m_state == KingWarBtnState.kPrepare then
		-- 备战，可能出现：未报名 / 分组中 / 开战倒计时
		local fightFlag = data:getFightFlag()
		local groupList = data:getGroupList()
		if fightFlag == 0 then
			self:openNodeView("LegionEntrant.LegionWarNotSignUpNode", arg)
		elseif #groupList == 0 then
			self:openNodeView("LegionEntrant.LegionWarGroupNode", arg)
		else
			self:openNodeView("LegionEntrant.LegionWarPrepareNode", arg)
		end

	elseif self.m_state == KingWarBtnState.kFight then
		-- 战斗，可能出现：未报名 / 进入地图 / 提前结束，出战斗结果
		local fightFlag = data:getFightFlag()
		local resultFlag = data:getResultFlag()
		if fightFlag == 0 then
			self:openNodeView("LegionEntrant.LegionWarNotSignUpNode", arg)
		elseif resultFlag == 4 then
			self:openNodeView("LegionEntrant.LegionWarResultNode", arg)
		else
			self:openNodeView("LegionEntrant.LegionWarFightNode", arg)
		end

	elseif self.m_state == KingWarBtnState.kEnd then
		-- 结束，可能出现：未报名 / 战斗结果
		local fightFlag = data:getFightFlag()
		if fightFlag == 0 then
			self:openNodeView("LegionEntrant.LegionWarNotSignUpNode", arg)
		else
			self:openNodeView("LegionEntrant.LegionWarResultNode", arg)
		end

	else
		self:openNodeView("LegionEntrant.LegionWarBeforeNode", arg)

	end
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    EntrantManager:removeObserver(self)
end

function This:openNodeView(name, info)
	if self.m_nodeName == name then
		return
	end
	
	print("openNodeView", name)
	self:closeNodeView()

	local view = require(name):create(info)
	self:addChild(view)
	self.m_nodeView = view
	self.m_nodeName = name
	
	return view
end

function This:closeNodeView()
	if self.m_nodeView then
		self.m_nodeView:removeFromParent(true)
		self.m_nodeView = nil
	end
	self.m_nodeName = ""
end

function This:callNodeViewFunc(funcName, ...)
	if not self.m_nodeView then
		return
	end

	local func = self.m_nodeView[funcName]
    if type(func) == "function" then
		return func(self.m_nodeView, ...)
	end
end

function This:getLeftTime()
	return self.m_data:getStateLeftTime()
end

function This:update(dt)
    super.update(self, dt)
    if self.m_hasTimeUpdate then
		self.m_escape = self.m_escape + dt
		if self.m_escape >= 1.0 then
			self.m_escape = self.m_escape - 1
			local leftTime = self:getLeftTime()
			if leftTime > 0 then
				self:callNodeViewFunc("updateLeftTime", leftTime)
			else
				self.m_hasTimeUpdate = false
				self.m_escape = 0
				self:callNodeViewFunc("updateLeftTime", 0)
				EntrantManager:onLegionWarInfoRequest()
				FamilyRatingManager:onGetCastInfoRequest()
				FamilyRatingManager:onGetCastInfoScheduleRequest()	
			end
		end
	end
end

-- 信息
function This:onLegionWarInfoReply(data)
	self.m_data = data
	self:updateState()
	self:initNode()
end

-- 结果
function This:onLegionWarResultReply(data)
	self.m_data = data
	self:initNode()
end

function This:updateState()
	local data = self.m_data
	self.m_state = data:getState()
	self.m_endTime = data:getStateEndTime()
	self.m_hasTimeUpdate = (self.m_state < KingWarBtnState.kEnd)
end