

-- 军团战规则奖励节点

local This, super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")
local LegionMapRuleRewardConfig = require("Metadatas.LegionMapRuleRewardConfig")
local LegionMapPersonRuleRewardConfig = require("Metadatas.LegionMapPersonRuleRewardConfig")
local LegionMapGuarRuleRewardConfig = require("Metadatas.LegionMapGuarRuleRewardConfig")
local ItemNode = require("UI.ItemNode")


function This:initData(info)
	self.m_legionReward = {} 
	self.m_personReward = {} 
	self.m_scoreReward = {} 
	self.m_itemList = {}

	-- 军团排名奖励
	for k, v in pairs(LegionMapRuleRewardConfig) do
		table.insert(self.m_legionReward, v)
	end
	table.sort(self.m_legionReward, function (a, b)
		return a.id < b.id
	end)

	-- 个人排名奖励
	for k, v in pairs(LegionMapPersonRuleRewardConfig) do
		table.insert(self.m_personReward, v)
	end
	table.sort(self.m_personReward, function (a, b)
		return a.id < b.id
	end)

	-- 积分累计奖励
	self.m_scoreReward = LegionMapGuarRuleRewardConfig[1]
end

function This:initNode(info)
	local scrollview = cc.ScrollView:create()
	scrollview:setContentSize(750, 410)
	self:addChild(scrollview)

	local node = self:createRewardPanel()
	local w, h = node:getContentSize()
	scrollview:pushBackCustomItem(node, w, h, 0)
end

function This:createRewardPanel()
	local node = cc.Node:create()

	local addEmptyNode = function (w, h)
		local emptyNode = self:createEmptyNode(w, h)
		node:addChild(emptyNode)
		table.insert(self.m_itemList, emptyNode)
	end

	-- 军团排名奖励
	local titleNode1 = self:createTitleNode1(Serialize[61063])
	node:addChild(titleNode1)
	table.insert(self.m_itemList, titleNode1)

	addEmptyNode(740, 20)

	for i, v in ipairs(self.m_legionReward) do
		local condition = v.condition_list
		local text = ""
		if condition[1] == condition[2] then
			text = string.format(Serialize[61614], condition[1], v.personal_score)
		else
			text = string.format(Serialize[61615], condition[1], condition[2], v.personal_score)
		end
		local titleNode2 = self:createTitleNode2(text)
		node:addChild(titleNode2)
		table.insert(self.m_itemList, titleNode2)

		addEmptyNode(740, 10)

		local rewardNode = self:createRewardNode(nil, v.reward_list)
		node:addChild(rewardNode)
		table.insert(self.m_itemList, rewardNode)

		if i == #self.m_legionReward then
			addEmptyNode(740, 30)
		else
			addEmptyNode(740, 20)
		end
	end

	-- 个人排名奖励
	local titleNode1 = self:createTitleNode1(Serialize[61062])
	node:addChild(titleNode1)
	table.insert(self.m_itemList, titleNode1)

	addEmptyNode(740, 20)

	for i, v in ipairs(self.m_personReward) do
		local rewardNode = self:createRewardNode(v.condition_list, v.reward_list)
		node:addChild(rewardNode)
		table.insert(self.m_itemList, rewardNode)

		if i == #self.m_personReward then
			addEmptyNode(740, 30)
		else
			addEmptyNode(740, 20)
		end
	end

	-- 积分累计奖励
	local titleNode1 = self:createTitleNode1(Serialize[61616])
	node:addChild(titleNode1)
	table.insert(self.m_itemList, titleNode1)

	addEmptyNode(740, 20)

	local text = string.format(Serialize[61617], self.m_scoreReward.personal_score, self.m_scoreReward.condition_list)
	local titleNode2 = self:createTitleNode2(text)
	node:addChild(titleNode2)
	table.insert(self.m_itemList, titleNode2)

	addEmptyNode(740, 10)

	local rewardNode = self:createRewardNode(nil, self.m_scoreReward.reward_list)
	node:addChild(rewardNode)
	table.insert(self.m_itemList, rewardNode)

	addEmptyNode(740, 20)	

	-----------------------------------------------------------------------

	local h = 10
	for i=#self.m_itemList, 1, -1 do
		local node = self.m_itemList[i]
		local nodeW, nodeH = node:getContentSize()
		node:setPosition(0, h)
		h = h + nodeH
	end

	node:setContentSize(750, h)
	node:setAnchorPoint(0.5, 0)

	return node
end

function This:createTitleNode1(text)
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorRuleReward.csb")
	local w, h = node.bg1:getContentSize()
	node:setAnchorPoint(0, 0)
	node:setContentSize(740, h)
	node.name1:setString(text)

	node.m_type = "TitleNode1"
	return node
end

function This:createTitleNode2(text)
	local node = cc.Label:createNormalTTF("", 16)
    node:setStringWithColor(text, 750)
    node:setAnchorPoint(0, 0)

    node.m_type = "TitleNode2"
    return node
end

function This:createRewardNode(condition, reward)
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorRuleRewardItem.csb")
	node:setAnchorPoint(0, 0)
	node:setContentSize(740, 60)

	if type(condition) == "table" then
		if condition[1] == condition[2] then
			node.num:setString( string.format(Serialize[60698], condition[1]) )
		else
			node.num:setString( string.format(Serialize[60699], condition[1], condition[2]) )
		end
	else
		node.num:setVisible(false)
		for i=1, 6 do
			local posNode = node[string.format("item%d", i)]
			local x, y = posNode:getPosition()
			posNode:setPosition(x - 80, y)
		end
	end

	for i, v in ipairs(reward) do
		local posNode = node[string.format("item%d", i)]
		if posNode then
			local item = ItemNode:create(v[1], v[2])
			item:setAnchorPoint(0.5, 0.5)
			item:setPosition( posNode:getPosition() )
			item:setScale(0.75)
			node:addChild(item)
		end
	end

	node.m_type = "RewardNode"
	return node
end

function This:createEmptyNode(w, h)
	local node = cc.Node:create()
	node:setContentSize(w, h)
	node.m_type = "EmptyNode"
	return node
end

