--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local SwapLayer = require("UI.SwapLayer")
local StageHeroItemView = require("Stage.StageHeroItemView")
local HeroManager = require("Hero.HeroManager")
local Timer = require("Timer")
local HeroListManager = require("List.HeroListManager")
local UpdateLayer = require("UI.UpdateLayer")
local UpdateList = require("Base.UpdateList")
local kMaxHeroCountInQueue = 30

function This:initNode(queue) 
	self.m_queue = queue
	self.m_timer = Timer:create(self)
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/ListEditor.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self.m_heroList = self.m_queue:getHeroList() 
	self.m_updateList = {}
	for i , heroId in ipairs(self.m_heroList) do
		self.m_updateList[heroId] = true
		self.m_heroList[i] = HeroManager:getHero(heroId)
	end

	self.m_listLayer = SwapLayer:create({
		itemWidth = 124,
		itemHeight = 160,
		itemCount = #self.m_heroList,
		column = 6,
		spaceX = 0,
		scrollView = self.m_csNode.ScrollView1,
	})
	self.m_listLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_listLayer:getContentSize()
	self.m_csNode.ScrollView1:pushBackCustomItem(self.m_listLayer, w,h,0)
	self.m_listLayer:setCascadeOpacityEnabled(true)

	self.m_listLayer:setDragSwapNode(true)
	self.m_listLayer:setSwapHandler(function(pos1,pos2)
		local tmp = self.m_heroList[pos1]
		self.m_heroList[pos1] = self.m_heroList[pos2]
		self.m_heroList[pos2] = tmp
	end,function(pos1,pos2)
		return true
	end)

	self.m_listLayer:setDataList(self.m_heroList,function(model)
		local view = self:getListItem(model)
		return view
	end)

	self.m_csNode.ScrollView2:setDirection("horizontal")
	self.m_csNode.ScrollView2.m_direction = "right"
	local heroList = HeroManager:getAllHeroGotWithSort(function(hero)
		return not hero:isDeputy() and not self.m_queue:hasHero(hero:getId())
	end)
	self.m_heroLayer = UpdateLayer:create({
		itemWidth = 90,
		itemHeight = 147,
		itemCount = #heroList,
		column = 1,
		isVertical = false,
		scrollView = self.m_csNode.ScrollView2,
	})
	self.m_heroLayer:setAnchorPoint(0,0.5)
	w , h = self.m_heroLayer:getContentSize()
	self.m_csNode.ScrollView2:pushBackCustomItem(self.m_heroLayer, w,h,0)
	self.m_heroLayer:setCascadeOpacityEnabled(true)
	self.m_heroLayer:setDataList(heroList,function(model)
		return self:getHeroItem(model)
	end)

	self:scheduleUpdate()
	self.m_csNode.title:setString(self.m_queue:getName())
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.empty:setTouchEnded(function()
		self.m_heroList = {}
		self.m_updateList = {}
		self:refreshListItems()
		self:refreshHeroItems()
	end)
	self.m_csNode.cancel:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.preservation:setTouchEnded(function()
		self.m_isRequesting = true
		local heroIdList = {}
		for i , hero in ipairs(self.m_heroList) do
			table.insert(heroIdList,hero:getId())
		end
		HeroListManager:onEditListHeroRequest(self.m_queue:getIndex(),heroIdList)
	end)
	self.m_csNode.leftbutton:setTouchEnded(function()
		self:onRightScroll()
	end)
	self.m_csNode.rightbutton:setTouchEnded(function()
		self:onLeftScroll()
	end)
	self.m_csNode.rename:setTouchEnded(function()
		local panel = g_scene:openDialog("List.QueueRenamePanel",{
			defaultStr = self.m_queue:getName(),
		})
		panel:setRenameHandler(function(str)
			HeroListManager:onEditListNameRequest(self.m_queue:getIndex(),str)
		end)
	end)
end

function This:refreshListItems()
	self.m_listLayer:setDataList(self.m_heroList,function(model)
		local view = self:getListItem(model)
		return view
	end)
end

function This:refreshHeroItems()
	local heroList = HeroManager:getAllHeroGotWithSort(function(hero)
		return not hero:isDeputy() and not self.m_updateList[hero:getId()]
	end)
	self.m_heroLayer:setDataList(heroList,function(model)
		return self:getHeroItem(model)
	end)
end

function This:getListItem(hero)
	local view = StageHeroItemView:create(hero:getId(),{hp = hero:getAttr(AttrType.kMaxHp)})
	view:setCancelHandler(function(id)
		self:removeListHero(id)
	end,true)
	view:setScale(0.95)
	view:setCascadeOpacityEnabled(true)
	return view
end

function This:getHeroItem(hero)
	local view = StageHeroItemView:create(hero:getId(),{hp = hero:getAttr(AttrType.kMaxHp)})
	view:setSelectHandler(function(id)
		if #self.m_heroList < kMaxHeroCountInQueue then
			self:addListHero(id)
		else
			g_scene:showTips(Serialize[61625])
		end
	end,true)
	local scale = 0.8
	view:setScale(scale)
	view:setAnchorPoint(0.5,0.5)
	view:setCascadeOpacityEnabled(true)
	return view
end

function This:removeListHero(id)
	for i , hero in ipairs(self.m_heroList) do
		if id == hero:getId() then
			table.remove(self.m_heroList,i)
			self.m_updateList[id] = nil
			break
		end
	end
	self:refreshListItems()
	self:refreshHeroItems()
end

function This:addListHero(heroId)
	local hero = HeroManager:getHero(heroId)
	table.insert(self.m_heroList,hero)
	self.m_updateList[heroId] = true
	self:refreshListItems()
	self:refreshHeroItems()
	self.m_csNode.ScrollView1:jumpToBottom()
end

function This:onLeftScroll()
	local w , h = self.m_csNode.ScrollView2:getContentSize()
	local node = self.m_csNode.ScrollView2:getInnerContainer()
	local posx = node:getPositionX()
	local width = node:getContentWidth()
	if posx > w - width then
		local percent = (-posx + 300) / (width - w) * 100
		self.m_csNode.ScrollView2:scrollToPercentHorizontal(percent,0.3,false)
	end
end

function This:onRightScroll()
	local w , h = self.m_csNode.ScrollView2:getContentSize()
	local node = self.m_csNode.ScrollView2:getInnerContainer()
	local posx = node:getPositionX()
	local width = node:getContentWidth()
	if posx < 0 then
		local percent = (-posx - 300) / (width - w) * 100
		self.m_csNode.ScrollView2:scrollToPercentHorizontal(percent,0.3,false)
	end
end

function This:onEditList()
	g_scene:showTips(Serialize[61626])
	self.m_csNode.title:setString(self.m_queue:getName())
	if self.m_isRequesting then
		self.m_isRequesting = false
		self:close()
	end
end

function This:onEnter()
	super.onEnter(self)
	HeroListManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	HeroListManager:removeObserver(self)
end

function This:update(dt)
	super.update(self,dt)
	self.m_timer:update(dt)
end

--endregion
