﻿local This, super = class(..., "Scene")
local GamePlayer = require("Player.GamePlayer")
local GameSetting = require("GameSetting")

local LoadText =
{
	[1] = Serialize[60030],
	[2] = Serialize[60031],
	[3] = Serialize[60032],
	[4] = Serialize[60033],
	[5] = Serialize[60034],
	[6] = Serialize[60035],
	[7] = Serialize[60036],
	[8] = Serialize[60037],
	[9] = Serialize[60038],
	[10] = Serialize[60039],
	[11] = Serialize[60040],
}

local kUseWarmingUI = false

function This:init()
    super.init(self)
	self:setType(SceneType.kLoadingScene)
	self.m_tasks = {}

	local kUseAudit = self:getName() ~= "LegionEditScene"
	if kUseAudit then
		self.m_isAudit = GamePlayer:isAuditVersion()
		local key = GamePlayer:getPlayerID() .. "_loading"
		if kUseWarmingUI then
			self.m_isNoWarning = GameSetting:get(key) == true
			self.m_isPause = not self.m_isNoWarning
		else
			self.m_isNoWarning = true
			self.m_isPause = false
		end
	end
	self.m_curIndex = 1
end

function This:onEnter()
    super.onEnter(self)
	self.m_csLoadingNode = cc.Node:createFromCSFile("MenuScene/NodeUI/LoadingScene.csb")
	self:addChild(self.m_csLoadingNode, 0xfffff)
	self.m_csLoadingNode:fitScreen(1)
	self.m_csLoadingNode.bg:fitScreen()
	self.m_csLoadingNode.bg2:fitScreenX()
	self.m_csLoadingNode.progress.bar:setPercent(0)

	cc.Audio:playBackground("Bg_Main.mp3")
	--self:lockTouchWithoutWaiting()
	if self.m_isAudit then
		self.m_csLoadingNode.progress.zhuangshi2:setVisible(false)
		self.m_csLoadingNode.progress.zhuangshi1:setVisible(false)
		self.m_csLoadingNode.progress.bg:setVisible(false)
		self.m_csLoadingNode.progress.bar:setVisible(false)
		self.m_csLoadingNode.progress.state:setVisible(false)
		self.m_csLoadingNode.progress.light:setVisible(false)
		self.m_csLoadingNode.progress.text:setVisible(self.m_isNoWarning)
		self.m_csLoadingNode.progress.text1:setVisible(self.m_isNoWarning)
		self.m_csLoadingNode.select:setVisible(not self.m_isNoWarning)
		self.m_csLoadingNode.select.selected:setVisible(self.m_isNoWarning)
		self.m_csLoadingNode.tishi:setVisible(not self.m_isNoWarning)
		self.m_csLoadingNode.btn:setVisible(not self.m_isNoWarning)
		if self.m_isNoWarning then
			self:showAuditEffect()
		end
	else
		local index = math.random(1,#LoadText)
		self.m_csLoadingNode.progress.text:setString(LoadText[index])
		self.m_csLoadingNode.progress.state:setString("0%")
		self.m_csLoadingNode.btn:setVisible(false)
		self.m_csLoadingNode.select:setVisible(false)
		self.m_csLoadingNode.tishi:setVisible(false)
		self.m_csLoadingNode.progress.text1:setVisible(false)
	end
	self.m_csLoadingNode.btn:setTouch({
		onEnded = function()
			self.m_isPause = false
			self.m_csLoadingNode.btn:setVisible(false)
			self.m_csLoadingNode.select:setVisible(false)
			self.m_csLoadingNode.tishi:setVisible(false)
			self.m_csLoadingNode.progress.text:setVisible(true)
			self:showAuditEffect()
		end,
		isDynamic = true
	})
	self.m_csLoadingNode.select:setTouch({
		onEnded = function()
			self.m_isNoWarning = not self.m_isNoWarning
			self.m_csLoadingNode.select.selected:setVisible(self.m_isNoWarning)
		end,
		isDynamic = true
	})

	self.m_csLoadingNode.bg:setTouch({
		onBegan = function()
			return true
		end,
		onEnded = function()
			print("ddd")
		end,
		isDynamic = true,
		isScale = false,
		isAlpha = false
	})

end

function This:showAuditEffect()
	if cc.FileUtils:isFileExist("Audit/zhuanquan.png") then
		local sprite = cc.Sprite:create("Audit/zhuanquan.png")
		sprite:runAction(cc.RepeatForever(cc.RotateBy(2.5,360)))
		self.m_csLoadingNode:addChild(sprite)
		sprite:setScale(0.3)
		sprite:setColor(0,255,0)
		sprite:setPosition(cc.visible.center.x,cc.visible.center.y)
	end
end

function This:onExit()
	super.onExit(self)
	cc.Audio:stopBackground()
	if kUseWarmingUI and self.m_isAudit then
		local key = GamePlayer:getPlayerID() .. "_loading"
		GameSetting:set(key,self.m_isNoWarning)
	end
end

function This:addTask(task)
	table.insert(self.m_tasks,task)
end

function This:updateProgress()
	if self.m_csLoadingNode then
		local progress = self.m_curIndex / #self.m_tasks
		self.m_csLoadingNode.progress.bar:setPercent(progress * 100)
		self.m_csLoadingNode.progress.state:setString(string.format("%d",progress*100).."%")
		local posx = self.m_csLoadingNode.progress.bg:getPositionX() 
		local width = self.m_csLoadingNode.progress.bg:getContentWidth() * progress
		self.m_csLoadingNode.progress.light:setPositionX(posx + width)
	end
end

function This:loadFinish()
	self:unlockTouch()
	if self.m_csLoadingNode then
		self.m_csLoadingNode:removeFromParent()
        self.m_csLoadingNode = nil
	end
	self.m_tasks = {}
    self.m_curIndex = 1
end

function This:update(dt)
	super.update(self,dt)
	if self.m_isAudit and self.m_isPause then
		return
	end

	if #self.m_tasks <= 0 then
		return
	end

	if self.m_curIndex > #self.m_tasks then
		self:loadFinish()
	else
		local task = self.m_tasks[self.m_curIndex]
		task:execute()
		if task:isFinish() then
			self:updateProgress()
			self.m_curIndex = self.m_curIndex + 1
		end
	end
end