local This, super = class(..., "Scene")
local EventDispatcher = require("EventDispatcher")
local LoginManager = require("Login.LoginManager")
local PlayerName = require("Player.PlayerName")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local DragSelectContainer = require("UI.DragSelectContainer")
local DragFastSelectContainer =require("UI.DragFastSelectContainer")
local IconBtnView = require("Hero.IconBtnView")
local RoleList = {1,2,3}
local MythicalAnimalsBookListView = require("GodBeast.MythicalAnimalsBookListView")

function This:initUI()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/CreateRoleScene1.csb")
    self.m_csNode:setOpacity(0)
    self.m_csNode:runAction(cc.FadeIn(0.3))
    
	--[[local frameScale = cc.visible.frameHeight / cc.visible.frameWidth
	local scale = cc.visible.designHeight / cc.visible.designWidth
	if frameScale > scale then
		self.m_csNode:setScale(1 / frameScale * scale)
	end]]
	
    local bgSprite = cc.Sprite:create("MenuScene/createbg.jpg")
    self:addChild(bgSprite)
	self:addChild(self.m_csNode)
	self.m_csNode:fitScreen(1)
	self.m_csNode:recoveryToFixedWidth("center")
    bgSprite:setAnchorPoint(0, 0)
    bgSprite:setScaleX(cc.visible.frameWidth / bgSprite:getContentWidth());
    self.m_csNode.bg:setVisible(false)
    
    local w , h = self.m_csNode.juanzhou2:getContentSize()
	local x , y = self.m_csNode.juanzhou2:getPosition()
    
	self.m_container = DragFastSelectContainer:create({
		width = w+10,
		height = h+34,
		space = 170,
		maxScale = 1.0,
		minScale = 1.0,
		initSpeed = 300,
    })
    self.m_container:setAnchorPoint(0.5,0.5)
	self.m_container:setPosition(x,y)
    self.m_csNode:addChild(self.m_container)
    for i ,iconId in ipairs(RoleList) do
        local view = IconBtnView:create(iconId)
        view:setSelectHandler(function()
            self:selectSoilder(iconId)
        end)
        self.m_container:addItemNode(view)
    end
    self.m_container:setItemToCenter(1)
    local order = self.m_csNode.playerdi1:getLocalZOrder()
    self.m_container:setLocalZOrder(order)
    self.m_csNode.playerdi1:setLocalZOrder(order + 1)
    self.m_csNode.effect:setLocalZOrder(order + 2)
    

    local name = self:getRandomName()
    self.m_csNode.name:setString(name)
    self.m_csNode.nameText:setString(name)
    self.m_csNode.name:setVisible(true)
    self.m_csNode.nameText:setVisible(false)
    self:updateIcon(1)
    self:focusePageOnIndex()
end

function This:onEnter()
    super.onEnter(self)
    self:initUI()
    self:initTouchEvent()
    self.m_container:showItem(false)
    self.m_csNode.player1:setVisible(true)
    self.m_csNode.player2:setVisible(true)
    GameUtil:setPlayerIconNodeImage(self.m_csNode.player2.touxiang1.icon,3)
    LoginManager:addObserver(self)
    self.m_csNode:playCSAnimation("start",false,function()
        self.m_csNode.player1:setVisible(false)
        self.m_container:showItem(true)
        self.m_csNode.player2:setVisible(false)
    end)

    EventDispatcher:addListener(self,"ios_keyboard_show",self.onIosKeyboardShow)
    EventDispatcher:addListener(self,"ios_keyboard_hide",self.onIosKeyboardHide)
end

function This:onIosKeyboardShow(height)
    if not self.m_originY then
        self.m_originY = self.m_csNode:getPositionY()
    end
    self.m_csNode:setPositionY(self.m_originY + height)
    self.m_csNode.name:setString("")
end

function This:onIosKeyboardHide()
    self.m_csNode:setPositionY(self.m_originY)
end

function This:onExit()
    super.onExit(self)
    LoginManager:removeObserver(self)
    EventDispatcher:removeListener(self, "ios_keyboard_show")
    EventDispatcher:removeListener(self, "ios_keyboard_hide")
end

function This:getRandomName()
    -- 英文随机名
    if AppText:getLanguage() == AppLanguage.en_US then
        local name, temp = "", ""

        temp = PlayerName.first[ math.random(1,#PlayerName.first) ]
        temp = string.gsub(temp, "^%l", string.upper)
        name = name .. temp

        temp = PlayerName.second[ math.random(1,#PlayerName.second) ]
        temp = string.gsub(temp, "^%l", string.upper)
        name = name .. temp

        name = string.gsub(name, " ", "")
        return name
    end

	local first = math.random(1,#PlayerName.first)
	local firstName = PlayerName.first[first]
	local second = math.random(1,#PlayerName.second)
	local secondName = PlayerName.second[second]
    local thirdly = math.random(1,#PlayerName.thirdly)
    local thirdlyName = PlayerName.thirdly[thirdly]

	return firstName .. secondName .. thirdlyName
end

function This:initTouchEvent()
    self.m_csNode.button:setTouchEnded(function()
        local name = self.m_csNode.name:getString()
        local info = self:getInputInfo(name)

        print(string.format("当前名字为：%s", name))

        if info.singleCharCount > 0 and info.multipCharCount > 0 then
            -- 中英混合
            local len = info.count
            if len < 2 then
                g_scene:showTips(Serialize[61629])
                return
            elseif len > 7 then
                g_scene:showTips(Serialize[61630]) 
                return
            end
        elseif info.singleCharCount > 0 then
            -- 只有字母或数字
            local len = info.singleCharCount
            if len > 20 then
                g_scene:showTips( string.format(Serialize["Login_CreateRoleScene_1"], 20) )
                return
            end
        elseif info.multipCharCount > 0 then
            -- 只有中文
            local len = info.multipCharCount
            if len < 2 then
                g_scene:showTips(Serialize[61629])
                return
            elseif len > 7 then
                g_scene:showTips(Serialize[61630]) 
                return
            end
        else
            -- 空字符串
            g_scene:showTips(Serialize[61628])
            return
        end

        if GameUtil:checkInvalidAscill(name) then
            g_scene:showTips(Serialize[61631])
            return
        end

		LoginManager:onRegisterRoleRequest(name, RoleList[self.m_icon])
    end)
    self.m_csNode.random:setTouchEnded(function()
        local name = self:getRandomName()
        self.m_csNode.name:setString(name)
        self.m_csNode.name:setVisible(true)
        self.m_csNode.nameText:setVisible(false)
    end)

    self.m_csNode.contentBg:setTouch({
        onEnded = function()
            local name = self:getRandomName()
            self.m_csNode.name:setVisible(true)
            self.m_csNode.nameText:setVisible(false)
        end,
        isScale = false
    })

    -- self.m_csNode.zuojiantou:setTouchEnded(function()
    --     self:updateIcon(self.m_icon - 1, "Left")
    -- end)

    -- self.m_csNode.youjiantou:setTouchEnded(function()
    --     self:updateIcon(self.m_icon + 1, "Right")
    -- end)

    -- for i, id in ipairs(RoleList) do
    --     local node = self.m_csNode["head" ..id]
    --     node:setTouch({
    --         onEnded = function()
    --             if self.m_icon == i then
    --                 return 
    --             end
    --             self:updateIcon(i, self.m_icon > i and "Left" or "Right")
    --         end,
    --         isDynamic = true,
    --     })
    -- end
end

function This:focusePageOnIndex()
    self:updateIconChoose()

    -- if self.m_icon == 1 then
    --     self.m_csNode.zuojiantou:setGrayShader()
    -- else
    --     self.m_csNode.zuojiantou:clearShader()
    -- end

    -- if self.m_icon == #RoleList then
    --     self.m_csNode.youjiantou:setGrayShader()
    -- else
    --     self.m_csNode.youjiantou:clearShader()
    -- end
    -- self.m_csNode.win_1:setImage(string.format("Icons/Player/choose%d.png", RoleList[self.m_icon]))
    self.m_csNode.effect:removeAllChildren()
    local skeletonNode = cc.SkeletonNode:createCharacter(string.format( "XuanRen%d",self.m_icon))
    skeletonNode:scheduleUpdate()
    skeletonNode:playAnimation("animation", true)
    skeletonNode:setPosition(0,-200)
    -- self.m_roleNode:setAnimationListener(self)
    self.m_csNode.win_1:setVisible(false)

    self.m_csNode.effect:addChild(skeletonNode)
end

function This:selectSoilder(iconId)
    self:updateIcon(iconId)
end

function This:updateIcon(icon, animation)
    if icon >= 1 and icon <= #RoleList and self.m_icon ~= icon then
        self.m_icon = icon
        self:focusePageOnIndex()
        self.m_csNode:playCSAnimation(animation or "Start",false,function()
            self.m_csNode:playCSAnimation("Start2",true)
        end)
    end
end

function This:updateIconChoose()
    local id = RoleList[self.m_icon]
    local node = self.m_csNode["head" .. id]
    if node then
        self.m_csNode.choose:setPosition(node:getPosition())
    end
end

function This:updateName(name)
    self.m_csNode.nameText:setVisible(false)
    self.m_csNode.name:setVisible(true)
    self.m_csNode.name:setString(name)
    self.m_csNode.nameText:setString(name)
end

function This:handleErrorCode(errorCode, info)
    super.handleErrorCode(self, errorCode, info)
    if errorCode and errorCode > 0 and info then
        local str = self.m_csNode.name:getString()
        local name = str
        for _, src in ipairs(info.minganci) do
            print(src)
            local dst = "<R0>" .. src .. "<A0>"
            local content, count = string.gsub(name, src, dst)
            print(content)
            name = content
        end
        self.m_isEmpty = true
        self.m_csNode.name:setString("")
        self.m_csNode.name:setVisible(false)
        self.m_csNode.nameText:setStringWithColor(name)
        self.m_csNode.nameText:setVisible(true)
    end
end

function This:getInputInfo(str)
    str = str or ""

    local ret = {
        singleCharCount = 0, --字母或数字
        multipCharCount = 0, --汉字
        count = 0,
    }

    local i = 1
    local strlen = string.len(str)
    while i <= strlen do
        local byte = string.byte(str,i,i)
        if byte >= 0xFC then
            len = 6
        elseif byte >= 0xF8 then
            len = 5
        elseif byte >= 0xF0 then
            len = 4
        elseif byte >= 0xE0 then
            len = 3
        elseif byte >= 0xC0 then
            len = 2
        else
            len = 1
        end
        i = i + len

        ret.count = ret.count + 1
        if len > 1 then
            ret.multipCharCount = ret.multipCharCount + 1
        elseif len == 1 then
            ret.singleCharCount = ret.singleCharCount + 1
        end
    end

    return ret    
end