--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GameSetting = require("GameSetting")
local ServerConfig = require("Net.ServerConfig")
local GamePlayer = require("Player.GamePlayer")
local Serialize = require("Serialize.Serialize")
local AchieveManager = require("Achieve.AchieveManager")
local SdkManager = require("Sdk.SdkManager")
local LoginServer = require("Login.LoginServer")
local LoginUtil = require("Login.LoginUtil")
local EntrantManager =  require("LegionEntrant.EntrantManager")


function This:init()
	super.init(self)
	self:initListener()
	self.m_observers = {}
	self.m_config = nil
	self.m_state = LoginState.kFree
	self.m_isRegisting = false
	self.m_servers = {}
	self.m_serverList = {}
	self.m_reconnnectCount = 0
	self:initUserAccount()
end

function This:initListener()
    self.m_listeners = 
    {
		connect_login_server = self.onConnectLoginServerReply,
		connect_game_server = self.onConnectGameServerReply,
        connect_lost = self.onConnectLostNotify,

        [10000] = self.onLoginLoginServerReply,
        [10001] = self.onLoginGameServerReply,
        [10002] = self.onRegisterRoleReply,
		[10004] = self.onAccountConflictNotify,
		[10005] = self.onReloginServerReply,
		[10006] = self.onRegisterAccountReply,
		[10008] = self.onSystemAnnouncementReply,
    }
    for eventKey, eventHander in pairs(self.m_listeners) do
        EventDispatcher:addListener(self, eventKey, eventHander)
    end
end

function This:removeListener()
    for eventKey, _ in pairs(self.m_listeners) do
        EventDispatcher:removeListener(self, eventKey)
    end
end

function This:isHasLoginLoginServer()
	print("self.m_state = ",self.m_state)
	return self.m_state > LoginState.kLoginingLoginServer
end

function This:getUserAccount()
    if self.m_account and self.m_account.username then
        return self.m_account
    else
        local account = {}
		if os.isWin32 or os.isIosSimulator then
			-- local computerName = lib2d.getComputerName()
			local computerName = "superm"
			account.username= string.gsub(computerName,"[^%w]","")
			account.password = "aaaaaa"
		else
			account.username= ""
			account.password = ""
		end
        
        return account
    end
end

function This:initUserAccount()
	local account = cc.FileUtils:loadLuaFile("account") or {}
	self.m_account = {
		username = account.username,
		password = account.password,
		serverId = account.serverId
	}
end

function This:saveUserAccount(t)
	if self.m_selectServer then
		self.m_account.serverId = self.m_selectServer:getId()
	end
	cc.FileUtils:saveToLua(self.m_account, "account")
end

function This:getServerList()
	local list = {}
	for i , info in pairs(self.m_servers) do
		table.insert(list,info)
	end
	table.sort(list,function(a,b)
		return a.id <= b.id
	end)
	return list
end

function This:getServer(serverId)
	return self.m_servers[serverId]
end

function This:getLoginServer()
	return self.m_selectServer
end

function This:getServerAreaName(serverId,haveServerName)
	local server = self:getServer(serverId) or self:getLoginServer()
	if server then
		return server:getAreaName(haveServerName)
	end
	return ""
end

function This:switchAccount(logout)
	self.m_state = LoginState.kFree
	if not logout then
		SdkManager:loginOut()
	end
	cc.Application:reload("Full")
end

function This:getServerListWithSort()
	return self.m_serverList
end

------------------------------------------------------------

function This:onConnectLoginServerRequest(connectLoginServerHandler)
	local ip = ServerConfig.loginServerHost
	local port = ServerConfig.loginServerPort
    print(string.format("开始连接登录服：ip=%s,port=%s",ip,port))
    Socket:connectLoginServer(ip,port)
    self.m_state = LoginState.kConnectingLoginServer
	self.connectLoginServerHandler = connectLoginServerHandler
end

function This:onConnectLoginServerReply(result)
    if result then
        print("------------连接登录服成功---------------")
		if self.connectLoginServerHandler then
			self.connectLoginServerHandler()
		end
    else
        self:notifyObservers("onConnectLoginServer",false)
		g_scene:showTips(Serialize.connect_error)
        self.m_state = LoginState.kFree
    end
end

function This:onLoginLoginServerRequest(loginInfo)
	if self.m_state > LoginState.kLoginingLoginServer then
		print("已经成功登录登录服！！！")
		return
	end
    self:onConnectLoginServerRequest(function()
		local message = {}
		message.sdkuid = loginInfo.sdkuid or ""
		message.channel = loginInfo.gameId or ""
		message.login_channel = loginInfo.loginChannel or ""
		message.passwd = loginInfo.password or ""
		message.mac = loginInfo.mac or ""
		message.agent = loginInfo.agent or ""
		message.serial = loginInfo.serial or ""
		message.imei = loginInfo.imei or ""
		message.imei2 = loginInfo.imei2 or ""
		message.idfa = loginInfo.idfa or ""
		message.idfv = loginInfo.idfv or ""
		message.systemId = loginInfo.systemId or ""
		message.systemInfo = loginInfo.systemInfo or ""
		message.screen = loginInfo.screen or ""
		message.type = tonumber(loginInfo.type or "0")
		message.ip = loginInfo.ip or ""
		message.ver = loginInfo.ver or ""
		message.city = loginInfo.city or ""
		message.province = loginInfo.province or ""
		message.udid = loginInfo.udid or ""
		message.login_token = loginInfo.token or ""
		message.pfId = loginInfo.pfId or 0
		message.packageId = LoginUtil.getPackageId()

		print("--------onLoginLoginServerRequest--------")
		print(table.tostring(message))
		self.m_account.username = message.sdkuid
		self.m_account.password = message.passwd
		self.m_account.channel = message.channel
		self.m_account.loginChannel = message.login_channel
		self.m_loginInfo = message

		Socket:send(10000, message)
		self.m_state = LoginState.kLoginingLoginServer
	end)
end

function This:onLoginLoginServerReply(msg)
	if msg.err_code == 0 then
		print("--------------登录登录服成功------------------")

		self.m_serverList = {}
		self.m_servers = {}
		for i , info in ipairs(msg.server_list) do
			local server = LoginServer:create(info)
			table.insert(self.m_serverList,server)
			self.m_servers[info.id] = server
			print( i , info.id )
		end
		table.sort(self.m_serverList,function(a,b)
			return a:getId() < b:getId()
		end)

		local groupList = {}
		for i , server in ipairs(self.m_serverList) do
			server:setIndex(i)
			local group = server:getGroup()
			if not groupList[group] then
				groupList[group] = {}
			end
			print("selectServer: " .. group, server:getId(), server:getName())
			table.insert(groupList[group], server)
		end

		for _, slist in pairs(groupList) do
			local count = #slist
			for serverIndex = 1, count do
				local server = slist[serverIndex]
				if server then
					server:setGroupIndex(serverIndex)
					server:setIsRecommand(serverIndex == count)
				end
			end
		end

		local serverId = self.m_account.serverId
		if serverId and self.m_servers[serverId] then
			self.m_selectServer = self.m_servers[serverId]
		else
			self.m_selectServer = self.m_serverList[#self.m_serverList]
		end

		local isModifySelect = true
	    if os.isIos and ServerConfig.isAudit then
	    	isModifySelect = false
	    end
	    
	    if isModifySelect then
			local config = LoginUtil.getGroupConfig()
			local selectGroup = self.m_selectServer:getGroup()
		    if selectGroup <= 0 or not table.contain(config.name, selectGroup) then
		    	local list = groupList[config.name[1]]
		    	if list then
		    		self.m_selectServer = list[#list]
		    	else
		    		self.m_selectServer = nil
		    	end
		    end
		end

		if self.m_selectServer then
			print("selectServer: " .. self.m_selectServer:getId(), self.m_selectServer:getName())
		else
			print("selectServer: None", serverId)
		end
		print("---------------onLoginLoginServer----------------")
		self:notifyObservers("onLoginLoginServer", msg.err_code)
		EventDispatcher:dispatcher("login_login_server")
        self:onSystemAnnouncementRequest()
	else
		g_scene:handleErrorCode(msg.err_code)
		self:notifyObservers("onLoginLoginServer", msg.err_code)
		if msg.err_code == 8 then
			if os.isWin32 or os.isIosSimulator then
				g_scene:openDialog("Login.LoginUI")
			else
				-- 重新登陸
				require("Sdk.SdkManager"):warm("无法登录账号，请再次尝试", function()
					require("Sdk.SdkManager"):login()
				end)
			end
		end
	end
end

function This:onSystemAnnouncementRequest()
    local message = {}
    message.channel = self.m_loginInfo.channel or "0"
    message.login_channel = self.m_loginInfo.login_channel or "0"
    message.platform = tostring(self.m_loginInfo.type or "0")
    Socket:sendAndLock(10008, message)
end

function This:onSystemAnnouncementReply(msg)
	if not GamePlayer:isAuditVersion() then
    	g_scene:openDialog("Login.SystemAnnouncePanel", {text = msg.content})
    end
end

function This:onRegisterAccountRequest(username,password)
	self.m_account.username = username
	self.m_account.password = password
	self:onConnectLoginServerRequest(function(isSecceed)
		Socket:sendAndLock(10006,{
			account = username,
			passwd = password,
			channel = "",
			login_channel = "",
		})
		self.m_state = LoginState.kRegisterAccount	
	end)
end

function This:onRegisterAccountReply(msg)
	if msg.err_code == 0 then
		self:onLoginLoginServerRequest({sdkuid = self.m_account.username,password = self.m_account.password})
	else
		g_scene:handleErrorCode(msg.err_code)
	end
	self:notifyObservers("onRegisterAccount",msg.err_code)
end

function This:onLoginGameServerRequest(serverId)
	if self.m_state >= LoginState.kEnterGame then
		print("已经登录成功！！！")
		return
	end

	self.m_selectServer = self:getServer(serverId)
    if self.m_state < LoginState.kConnectingGameServer then
        self:onConnectGameServerRequest(serverId)
    else
		self.m_loginInfo.role_id = self.m_selectServer:getRoleId()
		self.m_state = LoginState.kLoginingGameServer
		Socket:sendAndLock(10001, self.m_loginInfo)
    end
end

function This:onConnectGameServerRequest(serverId)
	self.m_selectServer = self:getServer(serverId)
	-- todo
    if self.m_selectServer then
        local ip = self.m_selectServer:getIp()
        local port = self.m_selectServer:getPort()
		print("开始连接游戏服,ip:%s, port:%d",ip,port)
        Socket:connectGameServer(ip,port)
        self.m_state = LoginState.kConnectingGameServer
    end
end

function This:onConnectGameServerReply(result)
    if result then
        print("-------------连接游戏服成功------------------")
		if self.m_selectServer:hasRole() then
			self:onLoginGameServerRequest(self.m_selectServer:getId())
		else
			cc.SceneManager:switchTo("Login.CreateRoleScene")
		end
    else
        self:notifyObservers("onConnectGameServerFailed")
		g_scene:showTips(Serialize.connect_error)
        self.m_state = LoginState.kFree
		if g_scene:getName() ~= "LoginScene" then
			local panel = g_scene:openDialog("Login.ReloginUI")
			panel:setToConnectLost()
		end
    end
end

function This:onLoginGameServerReply(msg)
    if msg.err_code  <= 0 then
        print("---------------登录游戏服成功-----------------------")
        GamePlayer:setPlayerID(msg.role.id)
        GamePlayer:setRoleName(msg.role.name)
        GamePlayer:setIcon(msg.role.icon)
		GamePlayer:setAccount(self.m_account.username)
		GamePlayer:setOpenServerTime(msg.open_server_time)
        self:saveUserAccount()
        self.m_state = LoginState.kEnterGame

		cc.SceneManager:switchTo("Player.InitDataScene")

    	EventDispatcher:dispatcher("activity_trailer_state_changed")

		self:notifyObservers("onEnterGameServer")
		EventDispatcher:dispatcher("enter_game_server")

	elseif msg.err_code == 12 then
		g_scene:openDialog("UI.ConfirmBox", {
            tips="該版本不可用，請重新下載游戲",
            onConfirm = function ( )
				local updateDir = cc.FileUtils:getWritablePath().."/Update/"
				if cc.FileUtils:isDirectoryExist(updateDir) then
					print("删除更新包...")
					cc.FileUtils:removeDirectory(updateDir)
				end
            end,
        })
	else
		if msg.err_code == 10001 then
			self.m_state = LoginState.kFree
		end
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onRegisterRoleRequest(roleName, icon, serverId)
    local message = {}
    message.sdkuid = self.m_account.username
    message.channel = self.m_account.channel
    message.login_channel = self.m_account.loginChannel
    message.name = roleName
    message.icon = icon
    local curServerId = serverId
    if not curServerId and self.m_selectServer then
		curServerId = self.m_selectServer:getId()
    end
    message.chooseserverid = curServerId or self.m_account.serverId
    Socket:sendAndLock(10002, message)
end

function This:onRegisterRoleReply(message)
    if message.err_code == 0 then
        self.m_selectServer:addRole({id = message.role.id,name = message.role.name})
        self:onLoginGameServerRequest(self.m_selectServer:getId())
		GamePlayer:setRoleCreateTime(message.role.create_time)
		GamePlayer:setPlayerID(message.role.id)
		GamePlayer:setRoleName(message.role.name)
        self:notifyObservers("onRegisterAccount", message.role)
		EventDispatcher:dispatcher("create_role")
    else
    	g_scene:handleErrorCode(message.err_code, message)
    end
end

function This:onReloginServerRequest()
    Socket:send(10005, {role_id=GamePlayer:getPlayerID() , ver = self.m_loginInfo.ver})
end

function This:onReloginServerReply(msg)
	if msg.err_code == 0 then
		
	end
end

function This:onConnectLostNotify()
	if self.m_state > LoginState.kConnectingGameServer then
		local dialog = g_scene:getDialog("ReloginUI")
		if dialog then
			dialog:close()
		end
		local panel = g_scene:openDialog("Login.ReloginUI")
		panel:setToConnectLost()
	end
end

function This:onAccountConflictNotify(message)
    if message.not_used == 0 then
        self:notifyObservers("onAccountConflict")
		print("你的账号被顶掉...")
		local panel = g_scene:getDialog("ReloginUI")
		if not panel then
			g_scene:openDialog("Login.ReloginUI")
		end
    end
end

This:init()
