local This, super = class(..., "Dialog")
local LoginManager = require("Login.LoginManager")
local Application = require("cc.Application")

function This:initNode()
    print("----------LoginUI:initNode-------------")
    self.m_csNode = cc.Node:createFromCSFile("LoginScene/NodeUI/LoginUI.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    
    local account = LoginManager:getUserAccount()
    self:resetAccount(account)
end

function This:resetAccount(account)
    self.m_userAccount = account
    local account = self.m_userAccount
    if account and account.username then
        self.m_csNode.account:setString(account.username)
        self.m_csNode.password:setString(account.password)
    end
end

function This:onEnter()
    super.onEnter(self)
    LoginManager:addObserver(self)
    Application:addListener(self)
end

function This:onExit()
    super.onExit(self)
    LoginManager:removeObserver(self)
    Application:removeListener(self)
end


function This:initTouchEvent()
    self.m_csNode.register:setTouchEnded(function()
        g_scene:openDialog("Login.RegisterUI")
    end)
    self.m_csNode.close:setTouchEnded(function()
        if LoginManager:isHasLoginLoginServer() then
            self:close()
        end
    end)
    self.m_csNode.login:setTouchEnded(function()
        local result, errorCode = self:canLogin()
        if result then
            self:login()
        else
            g_scene:handleErrorCode(errorCode)
        end
    end)
end

local function checkAscill(str)
    for i = 1, #str do
        local byte = string.byte(str, i)
        local isValid = (byte >= 48 and byte <= 57) or (byte >= 65 and byte <= 90) or (byte >= 97 and byte <= 122)
        if not isValid then
            return false
        end
    end
    return true
end

function This:canLogin()
    do return true end
    local username = self.m_csNode.account:getString()
    if #username == 0 then
        return false, "userNameNil"
    elseif #username < 2 then
        return false, "userNameShort"
    elseif #username > 15 then
        return false, "userNameLong"
    elseif not checkAscill(username) then
      return false, "userInvalid"
    end

    local password = self.m_csNode.password:getString()
    if #password == 0 then
        return false, "passwordNil"
    elseif #password < 6 then
        return false, "passwordShort"
    elseif #password > 20 then
        return false, "passwordLong"
    elseif not checkAscill(password) then
        return false, "passwordInvalid"
    end
    return true
end

function This:login()
    if self.m_isLogining then
        return
    end
    local username = self.m_csNode.account:getString()
    local password = self.m_csNode.password:getString()
    local channel = self.m_csNode.channel:getString()
    local loginChannel = self.m_csNode.login_channel:getString()
    local mainVersin , subVersion = cc.Application:getVersion()
    if os.isWin32 and g_setting.innerUsed then
        local ServerConfig = require("Net.ServerConfig")
        loginChannel = ServerConfig.channelId  
        channel = ServerConfig.gameId  
        g_setting.innerUsed = false
    end
    LoginManager:onLoginLoginServerRequest({
        sdkuid = username,
        password = password, 
        gameId = channel, 
        loginChannel = loginChannel,
        ip = "192.168.1.130", --lib2d.getComputerIpAddress(),
        mac = lib2d.getComputerMacAddress(),
        ver = mainVersin .. "." .. subVersion,
        type = "0"
    })
    self.m_isLogining = true
end

function This:onLoginLoginServer(errorCode,serverList)
    if errorCode == 0 then
        self:close()
    end
    self.m_isLogining = false
end

function This:onKeyDown(keyCode)
    if lib2d.isControlDown() and keyCode == "V" then
        local text = lib2d.readFromClipboard()
        self.m_csNode.account:setString(text)
    end
end