local This, super = class(..., "Dialog")
local LoginManager = require("Login.LoginManager")

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("LoginScene/NodeUI/RegisterUI.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self:initUsername()
end

function This:initTouchEvent()
    self.m_csNode.submitBtn:setTouchEnded(function()
        local result, errorCode = self:canRegister()
        if result then
            self:register()
        else
            g_scene:handleErrorCode(errorCode)
        end
    end)
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
end

function This:onEnter()
	super.onEnter(self)
	LoginManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	LoginManager:removeObserver(self)
end

function This:initUsername()
    local username = os.time()
    self.m_csNode.account:setString(username)
end

local function checkAscill(str)
    for i = 1, #str do
        local byte = string.byte(str, i)
        local isValid = (byte >= 48 and byte <= 57) or (byte >= 65 and byte <= 90) or (byte >= 97 and byte <= 122)
        if not isValid then
            return false
        end
    end
    return true
end

function This:canRegister()
    local username = self.m_csNode.account:getString()
    if #username == 0 then
        return false, "userNameNil"
    elseif #username < 2 then
        return false, "userNameShort"
    elseif #username > 15 then
        return false, "userNameLong"
    elseif not checkAscill(username) then
        return false, "userInvalid"
    end

    local password = self.m_csNode.password1:getString()
    local password2 = self.m_csNode.password2:getString()
    if password ~= password2 then
        return false, "passwordNotEqual"
    elseif #password == 0 then
        return false, "passwordNil"
    elseif #password < 6 then
        return false, "passwordShort"
    elseif #password > 20 then
        return false, "passwordLong"
    elseif not checkAscill(password) then
        return false, "passwordInvalid"
    end

    return true
end

function This:register()
    if self.m_isRegistering then
        return
    end
    local username = self.m_csNode.account:getString()
    local password = self.m_csNode.password1:getString()
    LoginManager:onRegisterAccountRequest(username, password, "test")
    --self.m_isRegistering = true
end

function This:onRegisterAccount(errorCode)
	if errorCode == 0 then
		self:close()
	end
	self.m_isRegistering = false
end