-- @Author: superm
-- @Date:   2018-04-24 21:24:40
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-01-16 19:17:18
-- @File Name: ServerListPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local LoginManager = require("Login.LoginManager")
local ServerTabNode = require("Login.ServerTabNode")
local ServerItemView = require("Login.ServerItemView")
local LoginUtil = require("Login.LoginUtil")

local reXueGroupId =2
local kNumPerTab = 10  --每一行item数量

function This:initNode(serverList)
	self.m_roleServerList = {}
	self.m_recommandList = {}
	self.m_serverList = {}

	local csNode = cc.Node:createFromCSFile("LoginScene/NodeUI/SelectBigZoneUI.csb")
	self:addChild(csNode)
	csNode:setToCenter()
	self.m_csNode = csNode
end

function This:initServerListInfo(groupId)
	self.m_serverList = {}
	self.m_roleServerList = {}
	self.m_recommandList = {}
	self.m_lastServer = nil
	local allServerList = LoginManager:getServerListWithSort()
	for _, server in ipairs(allServerList) do
		local group = server:getGroup()
		if group == groupId then
			if DaYanStartGroupIndex and LoginUtil.getPackageId() ==reXueGroupId and groupId ==reXueGroupId
			and LoginManager.checkServerStartIndex then
				local groupIndex = server:getIndexInGroup(true)
				if groupIndex>DaYanStartGroupIndex then
					table.insert(self.m_serverList, server)
				end
			else
				table.insert(self.m_serverList, server)
			end
		end
	end

	self.m_lastServer = LoginManager:getLoginServer()
	for i , server in ipairs(self.m_serverList) do
		if server:isRecommand() then
			table.insert(self.m_recommandList,server)
		end
		if server:hasRole() then
			table.insert(self.m_roleServerList,server)
		end
	end

	if self.m_lastServer and self.m_lastServer:getGroup() ~= groupId then
		local server = self.m_roleServerList[#self.m_roleServerList]
		server = server or self.m_recommandList[#self.m_recommandList]
		server = server or self.m_serverList[#self.m_serverList]
		self.m_lastServer = server
	end
end

function This:initTabs()
	local tabConfig = LoginUtil.getGroupConfig()
	local tabNames = tabConfig.name
	local groupCount = #tabNames
	print("groupCount:" .. groupCount)
	
	local tabFunction = function (groupId)
		self.m_selectGroupId = groupId
		print("select id:" .. groupId)
    	self:initServerListInfo(groupId)
		self:initTabNodeList()
		self.m_recommandNode:setSelect(true)
		self.m_selectTabNode = self.m_recommandNode
    end
	local loginServer = LoginManager:getLoginServer()
    local defaultGroup = loginServer:getGroup()

	local csNode = self.m_csNode
    local tabNode = csNode.tabs
	if groupCount <= 1 then
		self.m_fullMode = true
		local srollNode = csNode.typelist
		local w, h = srollNode:getContentSize()
		srollNode:setContentSize(150, h)
		csNode.tabs:setVisible(false)
		return tabFunction(defaultGroup)
	end

    print("==============> default:" .. defaultGroup)
	local TerrTabGroupBehaviour = require("Territory.TerrTabGroupBehaviour")
    local group = TerrTabGroupBehaviour:create(param)
    local tabTextList = tabConfig.text
    for index, gid in ipairs(tabNames) do
    	local name = tabNames[index]
    	local node = tabNode["fenye" .. index]
    	node.light.name:setString(tabTextList[index], 5)
    	node.normal.name:setString(tabTextList[index], 5)
    	group:addTabButton(name, node, function ()
	    	tabFunction(name)
	    end, false)
    end

    if defaultGroup > 0 then
	    group:selectTabButton(defaultGroup)
	end
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.close:setTouchEnded(function()
		self:close()
	end)
end

function This:onEnter()
	super.onEnter(self)
	self:initTabs()
end

function This:getServerItemView(server,isContainer)

	local view = ServerItemView:create(server,self.m_selectGroupId ==reXueGroupId)
	view:setSelectHandler(function(sr)
		if self.m_selectHandler then
			self.m_selectHandler(sr)
			self:close()
		end
	end)
	view:setTouchIsContanier(isContainer)
	return view
end

function This:selectServer(serverList)
	local srollNode = self.m_csNode.zoneScrollView
	local itemList = srollNode:getItemList()
	for i , server in ipairs(serverList) do
		if i <= #itemList then
			itemList[i]:setModel(server)
		else
			local view = self:getServerItemView(server,true)
			srollNode:pushNode(view,256,70,3)			
		end
	end
	for i = #itemList , #serverList + 1, -1 do
		srollNode:removeItem(i)
	end
end

function This:getRecommandTabNode()
	local tabNode = ServerTabNode:create({name = Serialize[61645], fullMode = self.m_fullMode})
	tabNode:setTouchHandler(function()
		if self.m_selectTabNode then
			self.m_selectTabNode:setSelect(false)
		end
		self.m_selectTabNode = tabNode
		self.m_selectTabNode:setSelect(true)
	end)
	tabNode:setSelectHandler(function()
		self:selectServer(self.m_recommandList)
	end)
	return tabNode
end

function This:getRoleTabNode()
	local tabNode = ServerTabNode:create({name = Serialize[61646], fullMode = self.m_fullMode})
	tabNode:setTouchHandler(function()
		if self.m_selectTabNode then
			self.m_selectTabNode:setSelect(false)
		end
		self.m_selectTabNode = tabNode
		self.m_selectTabNode:setSelect(true)
	end)
	tabNode:setSelectHandler(function()
		self:selectServer(self.m_roleServerList)
	end)
	return tabNode
end

function This:initTabNodeList()
	local function getZoneTabNode(index,name)
		local tabNode = ServerTabNode:create({index = index, name = name, fullMode = self.m_fullMode})
		tabNode:setTouchHandler(function(index)
			if self.m_selectTabNode then
				self.m_selectTabNode:setSelect(false)
			end
			self.m_selectTabNode = tabNode
			self.m_selectTabNode:setSelect(true)
		end)
		tabNode:setSelectHandler(function(index)
			local serverList = {}
			for i = (index -1) * kNumPerTab + 1, index * kNumPerTab do
				if i <= #self.m_serverList then
					table.insert(serverList,self.m_serverList[i])
				end
			end
			self:selectServer(serverList)
		end)
		return tabNode
	end

	local function addSrollItem(sv, node)
		local w, h = node:getContentSize()
		sv:pushBackCustomItem(node, w, h, 0.5)
	end

	local csNode = self.m_csNode
	local srollNode = csNode.typelist
	srollNode:removeAllItems()
	self.m_roleTabNode = self:getRoleTabNode()
	addSrollItem(srollNode, self.m_roleTabNode)

	self.m_recommandNode = self:getRecommandTabNode()
	addSrollItem(srollNode, self.m_recommandNode)

	local count = #self.m_serverList
	local tabNum = math.ceil(count / kNumPerTab) 
	for i = tabNum , 1 , -1 do
		local endIndex = math.min(i * kNumPerTab, #self.m_serverList)
		local name = string.format(Serialize[61647],(i-1) * kNumPerTab + 1, endIndex)
		local node = getZoneTabNode(i,name)
		addSrollItem(srollNode, node)
	end

	if self.m_lastServer then
		local lastView = self:getServerItemView(self.m_lastServer)
		csNode.lastZone.zone:addChild(lastView)
	end
end

function This:setSelectHandler(handler)
	self.m_selectHandler = handler
end

--endregion
