local This, super = class(..., "Scene")
local GameSetting = require("GameSetting")
local GameChannel = require("GameChannel")

This.fadeTime = 0.03

function This:init()
    super.init(self)
	self:initUI()
end

function This:initUI()
	print("---------------LoginScene-----------------")
    cc.SpriteFrameCache:addPlist("LoginScene")
    self.m_csNode = cc.Node:createFromCSFile("LoginScene/NodeUI/LoginScene.csb")
    self.m_csNode:setOpacity(0)
    self.m_csNode:runAction(cc.FadeIn(0.3))
    self:addChild(self.m_csNode)

    self.m_csNode:fitScreen(1)
    self.m_csNode.bg:fitScreen()

	self.m_csNode.progress.state:setString(Serialize[60042])

    self.m_loginVisible = false
    self.m_csNode.login:setVisible(not os.isIos)
    self.m_csNode.selectZone:setVisible(false)
    self.m_csNode.switchAccount:setVisible(false)
	self.m_csNode.progress:setVisible(false)
	self.m_csNode.SystemPopup:setVisible(false)
	self.m_csNode.logo:setVisible(false)

	self:initLangUI()
end

function This:initLangUI()
	local csNode = self.m_csNode
	csNode.language:setVisible(false)
	csNode.langMask:setVisible(false)
	csNode.langMask:setContentSize(cc.visible.width, cc.visible.height)
	self:setLangSwitch(false)
end

function This:setLangSwitch(isOpen)
	local csNode = self.m_csNode
	--csNode.en_US:setVisible(isOpen)
	--csNode.zh_TW:setVisible(isOpen)
	--csNode.zh_CN:setVisible(isOpen)
	self.m_langState = isOpen and 1 or 0
end

function This:initLangTouch()
	local csNode = self.m_csNode

	local setLangType = function (langType)
		if AppText:getLanguage() == langType then
			self:setLangSwitch(false)
		else
			AppText:setLanguage(langType)
		end
	end

	csNode.language:setTouchEnded(function()
		-- if self.m_langState == 0 then
		-- 	self:setLangSwitch(true)
		-- else
		-- 	self:setLangSwitch(false)
		-- end
		g_scene:openDialog("Login.LoginSceneLanguagePanel",{})
	end)

	csNode.langMask:setTouchEnded(function()
		self:setLangSwitch(false)
	end)

	--csNode.en_US:setTouchEnded(function()
	--	setLangType(AppLanguage.en_US)
	--end)
	--
	--csNode.zh_TW:setTouchEnded(function()
	--	setLangType(AppLanguage.zh_TW)
	--end)
	--
	--csNode.zh_CN:setTouchEnded(function()
	--	setLangType(AppLanguage.zh_CN)
	--end)
end

function This:refreshWithAudit()
	self.m_csNode.logo:setImage("MenuScene/logo1.png")
	self.m_csNode.logo:setVisible(true)
end

function This:onEnter()
    super.onEnter(self)
	local function loadFinishHandler()
        local ServerConfig = require("Net.ServerConfig")
		if os.isIos and ServerConfig.isAudit then
			cc.FileUtils:addSearchPath("zh_CN/Audit/", true)
			cc.FileUtils:clearCache()
			cc.TextureCache:releaseAllTextures()
		end
		self:refreshWithAudit()
		cc.Application:initGame()
		require("Login.LoginManager"):addObserver(self)
		self:initTouchEvent()
		if os.isWin32 or os.isIosSimulator then
			self:setTimer(0.3, function ()
				self:openDialog("Login.LoginUI")
			end)
		else
			self:setTimer(0.3, function ()
				require("Sdk.SdkManager"):login()
			end)
		end
		print("--------loadFinishHandler---------")
		self:updateServerInfo()	
	end
	if not os.isWin32 and not os.isIosSimulator then
		self:checkResourceUpdate(loadFinishHandler)
	else
		if g_setting.isCheckUpdate then
			self:checkResourceUpdate(loadFinishHandler)
		else
			loadFinishHandler()
		end
	end
end

function This:onExit()
    super.onExit(self)
    require("Login.LoginManager"):removeObserver(self)
end

function This:initTouchEvent()
	local LoginManager = require("Login.LoginManager")
    self.m_csNode.login:setTouchEnded(function()
    	if not self.m_loginVisible then
			require("Sdk.SdkManager"):login()
			return
    	end

		if self.m_server:getState() == ServerState.kMaintenance then
			g_scene:showTips(Serialize[60043])
		else
			local ServerConfig = require("Net.ServerConfig")
			if os.isIos and ServerConfig.isAudit then
				os.native_call("showPrivacyStatementView")
			end
			LoginManager:onLoginGameServerRequest(self.m_server:getId())
		end
    end)
    self.m_csNode.switchAccount:setTouchEnded(function()
		if os.isWin32 or os.isIosSimulator or g_setting.isInnerLogin then
			g_scene:openDialog("Login.LoginUI")
		else
			require("Sdk.SdkManager"):login()
		end
    end)
    self.m_csNode.selectZone.changeBtn:setTouchEnded(function ()
		local panel = g_scene:openDialog("Login.ServerListPanel")
		panel:setSelectHandler(function(server)
			self:updateServerInfo(server)
		end)
    end)
	self.m_csNode.SystemPopup:setTouchEnded(function()
		LoginManager:onSystemAnnouncementRequest()
	end)

	-- self.m_csNode.bg:setTouch({
 --        isScale = false, 
 --        isDynamic = true, 
 --        onEnded = function()
	-- 		require("Sdk.SdkManager"):login()
 --        end
 --    })

    self:initLangTouch()
end

function This:updateServerInfo(server)
	local LoginManager = require("Login.LoginManager")
	if not server then
		server = LoginManager:getLoginServer()
	end
	if server then
		self.m_server = server
		self.m_csNode.selectZone.area:setString(server:getIndexInGroup(true) .. Serialize[60044])
		self.m_csNode.selectZone.name:setString(server:getName())
		self.m_csNode.selectZone.state:setImage(ServerStateImage[self.m_server:getState()])
    	self.m_loginVisible = true
		-- self.m_csNode.login:setVisible(true)
		self.m_csNode.selectZone:setVisible(true)
		self.m_csNode.switchAccount:setVisible(true)
		self.m_csNode.language:setVisible(true)
		self.m_csNode.SystemPopup:setVisible(not require("Player.GamePlayer"):isAuditVersion())	
	else
    	self.m_loginVisible = false
		-- self.m_csNode.login:setVisible(false)
		self.m_csNode.selectZone:setVisible(false)
		self.m_csNode.switchAccount:setVisible(false)
		self.m_csNode.SystemPopup:setVisible(false)
		self.m_csNode.language:setVisible(false)
	end
end

------------------------热更-------------------------
function This:checkResourceUpdate(onUpdateFinish)
	print("---------------checkResourceUpdate---------------------")
    require("ResourceUpdate"):create({
        owner = self,
        onInvalidVersion = function (self, _ , url)
			if url and url ~= "" then
				self:openDialog("UI.ConfirmBox", {
					tips=Serialize[60045],
					onConfirm = function ( )
						print("url = " , url)
						local updateDir = cc.FileUtils:getWritablePath().."/Update/"
						if cc.FileUtils:isDirectoryExist(updateDir) then
							print("删除更新包...")
							cc.FileUtils:removeDirectory(updateDir)
						end
						if url and url ~= "" then
							os.native_call("openUrl",url)
						end
						os.native_call("exitGame")
					end,
				})
			else
				self:openDialog("UI.ConfirmBox", {
					tips=Serialize[60046],
				})
			end
        end,
		onConfigLost = function(self)
        	local count = GameSetting:get("UpdateServerSiteFail") or 0
        	count = count + 1
        	if count > 2 and count < 4 then
        		GameSetting:set("UpdateServerSite", "http://newupdate.51gamer.cn")
        	elseif count > 4 then
        		count = 0
        		GameSetting:set("UpdateServerSite", nil)
            end
        	GameSetting:set("UpdateServerSiteFail", count)
			GameSetting:save()
			self:openDialog("UI.ConfirmBox", {
                tips=Serialize[60047],
                onConfirm = function ()
                	cc.Application:reload("Full")
                end,
            })
		end,
        onNewVersion = function (self, resourceUpdate, versionInfo)
            for i, dialog in ipairs(self.m_dialogList) do
                dialog:close()
            end
            self:openDialog("UI.ConfirmBox", {
                tips=string.format(Serialize[60048], math.max(versionInfo.totalSize / 1024 / 1024, 0.01)),
                onConfirm = function ( )
                    resourceUpdate:stepState("DowloadStart")
    				self.m_loginVisible = false
                    -- self.m_csNode.login:setVisible(false)
					self.m_csNode.selectZone:setVisible(false)
					self.m_csNode.switchAccount:setVisible(false)
					self.m_csNode.language:setVisible(false)
                    self.m_csNode.progress:setVisible(true)
                    self.m_csNode.progress.bar:setPercent(0)
					local posx = self.m_csNode.progress.bar:getPositionX() 
					self.m_csNode.progress.light:setPositionX(posx)
                    self.m_csNode.progress.text:setString("0%")
                end,
            })
        end,
        onDownloadStart = function (self, _)
			print("开始现在热更包...")
			self.m_loginVisible = false
            -- self.m_csNode.login:setVisible(false)
			self.m_csNode.selectZone:setVisible(false)
			self.m_csNode.switchAccount:setVisible(false)
			self.m_csNode.language:setVisible(false)
            self.m_csNode.progress:setVisible(true)
            self.m_csNode.progress.bar:setPercent(0)
			local posx = self.m_csNode.progress.bar:getPositionX() 
			self.m_csNode.progress.light:setPositionX(posx)
            self.m_csNode.progress.bar.m_isRunning = false
            self.m_csNode.progress.state:setString(Serialize[60049])
            self.m_csNode.progress.text:setString("0%")
        end,
        onDownloading = function (self, _, name, index, count, size, totalSize)
        	local curPrgress = size / totalSize
            local percent = curPrgress * 100
			print(string.format("正在下载%s, %d%%",name,percent))
            self.m_csNode.progress.bar:setPercent(percent)
			local posx = self.m_csNode.progress.bar:getPositionX() 
    		local width = self.m_csNode.progress.bar:getContentWidth()
			self.m_csNode.progress.light:setPositionX(posx + width * curPrgress)
			print(string.format("posx = %d",posx + width))
            self.m_csNode.progress.state:setString(string.format(Serialize[60050], name, index, count))
            size = size / 1024 / 1024
            totalSize = totalSize / 1024 / 1024
            self.m_csNode.progress.text:setString(string.format("%.1f%%  (%.1f/%.1fM)", percent, size, totalSize))
        end,
        onUnzipStart = function (self, _)
			local posx = self.m_csNode.progress.bar:getPositionX() 
			self.m_csNode.progress.light:setPositionX(posx)
            self.m_csNode.progress.bar:setPercent(0)
			print(string.format("开始解压..."))
            self.m_csNode.progress.bar.m_isRunning = false
            self.m_csNode.progress.text:setString("0%")
            self.m_csNode.progress.state:setString(Serialize[60051])
            self.m_csNode.progress:setVisible(true)
			self.m_loginVisible = false
            -- self.m_csNode.login:setVisible(false)
			self.m_csNode.selectZone:setVisible(false)
			self.m_csNode.switchAccount:setVisible(false)
			self.m_csNode.language:setVisible(false)
        end,
        onUnziping = function (self, _, name, index, count, size, totalSize)
        	local curPrgress = size / totalSize
            local percent = curPrgress * 100
			print(string.format("正在解压%d%%",percent))
            self.m_csNode.progress.state:setString(string.format(Serialize[60052], name, index, count))
            self.m_csNode.progress.text:setString(string.format("%.1f%%", percent))
            self.m_csNode.progress.bar:setPercent(percent)
			local posx = self.m_csNode.progress.bar:getPositionX() 
    		local width = self.m_csNode.progress.bar:getContentWidth()
			self.m_csNode.progress.light:setPositionX(posx + width * curPrgress)
        end,
        onUpdateFinish = function (self, _, reason)
			print("热更完成...")
    		GameSetting:set("UpdateServerSite", nil)
        	GameSetting:set("UpdateServerSiteFail", 0)
			GameSetting:save()
            self.m_csNode.progress:setVisible(false)
			self.m_loginVisible = true
            -- self.m_csNode.login:setVisible(true)
			self.m_csNode.selectZone:setVisible(true)
			self.m_csNode.switchAccount:setVisible(true)
			self.m_csNode.language:setVisible(true)
            if reason == "UnzipAll" then
                cc.FileUtils:clearCache()
                cc.Application:reload("Full")
            else
                if onUpdateFinish then
                    onUpdateFinish(self, reason)
                end
            end
        end,
		onFileLost = function(self,_, fileName)
			if not self:getDialog("ConfirmBox") then
				self:openDialog("UI.ConfirmBox", {
					tips = string.format(Serialize[60053],fileName),
					onConfirm = function ( )
						cc.Application:reload("Full")
					end,
				})
			end
		end,
    })
end

----------------------------------------------------------------

function This:onLoginLoginServer(errCode)
	if errCode == 0 then
		print("----------LoginScene:onLoginLoginServer---------")
		self:updateServerInfo()
		self.m_csNode.login:setVisible(true)
	end
end

function This:autoRegister(account)
	local LoginManager = require("Login.LoginManager")
    if self:canAutoRegister(account) then
        LoginManager:onRegisterAccountRequest(account.username, account.password, account.sdkID)
    end
end

function This:autoLoginAccountServer(account)
	local LoginManager = require("Login.LoginManager")
    if self:canAutoRegister(account) then
        LoginManager:onLoginAccountRequest(account.username, account.password, account.sdkID)
    end
end

--[[function This:update(dt)
	self.m_timer:update(dt)
	self:updateNodeList(delta)
end]]
