
local This , super = class(...,"NodeUI")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")


function This:initData(data)
    self.mail = data
    self.refreshCb = nil
end

function This:initNode(data)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/EmailItem.csb")
    self:addChild(self.m_csNode)
    self:setCascadeOpacityEnabled(true)
    self:setContentSize(self.m_csNode.bg:getContentSize())
    self.m_csNode:setPosition(147, 55.5) 
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:setModel(data)
    self.mail = data
    self:refresh()
end

function This:refresh()
	local mail = self.mail
	local node = self.m_csNode

    if mail.itemList and mail.itemList[1] and not mail.isReceive then
        local mainItemInfo = mail.itemList[1]
        node.icon:setImage(ItemUtil.getIconImage(mainItemInfo.id))
    elseif mail.isRead then
        node.icon:setImage("MenuScene/emailicon.png")
    else
        node.icon:setImage("MenuScene/emailicon1.png")
    end

    if mail.ctype ~= MailContentType.kFamily then
        node.title:setString(mail.title)
    else
        node.title:setString(Serialize["mail_juntuan"] .. mail.title)
    end
        
    local dt = GamePlayer:getServerTime() - mail.startTime
    if dt < 60 then
        node.time:setString(Serialize["mail_in_one_minute"])
    elseif dt < 3600 then
        node.time:setString(math.floor(dt / 60) .. Serialize["mail_minute_ago"])
    elseif dt < 3600 * 24 then
        node.time:setString(math.floor(dt / 3600) .. Serialize["mail_hour_ago"])
    else
        local date = os.date("*t", mail.startTime)
        node.time:setString(string.format(Serialize["mail_date"], date.year, date.month, date.day))
    end

    if type(self.refreshCb) == "function" then
    	self.refreshCb(self)
    end
end

function This:setRefreshCb(cb)
	self.refreshCb = cb
end

function This:setSelectState(isSelect)
    self.m_csNode.bg:setImage(isSelect and "MenuScene/youjianxuanzhog.png" or "MenuScene/youjianitmebg.png")
end
