

-- 重写require
local oldRequire = require
require = function (path)
    local isFirstLoad = (package.loaded[path] == nil)
    local file = oldRequire(path)
    local pos = string.find(path, "Metadatas.")
    local ignores = {
        ["Metadatas.SerializeConfig"] = true,
        ["Metadatas.TextConfig1"] = true,
        ["Metadatas.TextConfig2"] = true,
        ["Metadatas.TextConfig3"] = true,
    }

    if not ignores[path] and pos and pos > 0 and isFirstLoad then
        if _G["AppText"] then
            AppText:onConfigLoaded(path, file)
        end
    end

    return file
end

-- 默认语言（全局函数）
function getNativeLanguage()
    return "zh_TW"
end

-- 文件搜索路径
local function initSearchPath()
    local ccFileUtils = lib2d.ccFileUtils
    local this = ccFileUtils.getInstance()
    local writablePath = ccFileUtils.getWritablePath(this)
    local setting = ccFileUtils.loadLuaDataFromFile(this, writablePath .. "Data/GameSetting", false)
    local language = getNativeLanguage() --默认

    if setting and setting.language then
        language = setting.language
    end

    ccFileUtils.clearSearchPath(this)

    if language == 'zh_CN' then
        ccFileUtils.addSearchPath(this, string.format("%sUpdate/%s/", writablePath, "zh_CN"), false)
        ccFileUtils.addSearchPath(this, "zh_CN/", false)
    else
        ccFileUtils.addSearchPath(this, string.format("%sUpdate/%s/", writablePath, language), false)
        ccFileUtils.addSearchPath(this, string.format("%sUpdate/%s/", writablePath, "zh_CN"), false)
        ccFileUtils.addSearchPath(this, string.format("%s/", language), false)
        ccFileUtils.addSearchPath(this, "zh_CN/", false)
    end
end

local errorCount = 0

if not buglyReportLuaException then
    buglyReportLuaException = function(id,content)
        print(string.format("id = %s \n content = %s",id,content))
    end
end

function logError(...)
    errorCount = errorCount + 1
    local traceDebug = debug.traceback()
    local argCount = select("#", ...)
    if argCount > 0 then
        traceDebug = traceDebug .. "\n"
        for i=1,argCount do
            local arg = select(i, ...)
            traceDebug = traceDebug .. "\t" .. tostring(arg)
        end
    end

    print("----------------------------------------")
    print(traceDebug)
    print("----------------------------------------")

    local titleInfo = "gameplayer is not init"
    local GamePlayer = require("Player.GamePlayer")
    if GamePlayer:isInit() then
        titleInfo = string.format("ID:%s,name:%s",tostring(GamePlayer:getPlayerID()),tostring(GamePlayer:getRoleName()))
        local LoginManager = require("Login.LoginManager")
        if LoginManager then
            local server = LoginManager:getLoginServer()
            if server then
                titleInfo = titleInfo .. string.format(",server:%s",tostring(server:getName()))
            end
        end     
        local cacheMainVer , cacheSubVer = lib2d.ccApplication.getVersion(lib2d.ccApplication.getInstance())
        cacheMainVer , cacheSubVer = cacheMainVer or "1", cacheSubVer or "0"

        local version = os.native_call("getVersion") 
        titleInfo = titleInfo .. string.format(",version:(%s.%s)-(%s)",tostring(cacheMainVer),tostring(cacheSubVer),tostring(version))
    end

    if g_scene then
        if g_scene.pauseGame then
            g_scene:pauseGame()
        else
            g_scene:unscheduleUpdate()
        end
    end

    buglyReportLuaException(titleInfo,traceDebug)
end


local function main()
    initSearchPath()
    require("Setting")
    require("Serialize.Serialize")
    require("Engine")
    require("Proto.Protocal")
    require("NativeCall")
    require("AppText"):init()
    require("GameChannel"):init()
    require("cc.Application"):init()
end

xpcall(main, logError)
