-- @Author: superm
-- @Date:   2018-11-13 09:46:38
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-21 16:23:28
-- @File Name: GameMapManager.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local GameMapUtil = require("Map.GameMapUtil")
local GamePlayer = require("Player.GamePlayer")

function This:init()
	super.init(self)
	self.m_maps = {}
	self.m_maps[GameMapType.kLegionMap] = require("Map.LegionMapManager"):create(GameMapType.kLegionMap)
	self.m_maps[GameMapType.kEmperorMap] = require("World.WorldMapManager")
	self.m_maps[GameMapType.kRescueMap] = require("RescueEmperor.RescueMapManager"):create(GameMapType.kRescueMap)
end

function This:getMap(mapType)
	return self.m_maps[mapType]
end

function This:getMapType()
    for _, mapType in pairs(GameMapType) do
    	if mapType == GameMapType.kEmperorMap and GamePlayer:isInKingMap() then
			return mapType
    	else    	    
	    	local map = self.m_maps[mapType]
	    	if map and map:isInit() then
    			return mapType
	    	end
    	end
    end
end

function This:isInMap(mapType)
	local map = self.m_maps[mapType]
	return map:isInit()
end

function This:getLegionMap()
	return self.m_maps[GameMapType.kLegionMap]
end

function This:getEmperorMap()
	return self.m_maps[GameMapType.kEmperorMap]
end

function This:enterLegionMap()
	local map = self.m_maps[GameMapType.kLegionMap]
	if map then
		map:enter()
	end
end

function This:enterEmperorMap()
	local map = self.m_maps[GameMapType.kEmperorMap]
	if map then
		map:enter()
	end
end

function This:enterRescueMap()
	local map = self.m_maps[GameMapType.kRescueMap]
	if map then
		map:enter()
	end
end

function This:enterMap(mapId)
	local foundMap
	for _, map in pairs(self.m_maps) do
		if map:getId() == mapId then
			foundMap = map
		end
	end
	
	if foundMap then
		foundMap:enter()
	else
		cc.SceneManager:switchTo("Home.MenuScene")
	end
end

function This:getRescueMap()
	return self.m_maps[GameMapType.kRescueMap]
end


--endregion
