-- @Author: zheng
-- @Date:   2018-10-20 15:22:20
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-03-20 14:51:03
-- @File Name: GameWarShopData.lua

local LegionWarShopTypeConfig = require("Metadatas.LegionWarShopTypeConfig")
local LegionWarShopJifenConfig = require("Metadatas.LegionWarShopJifenConfig")
local FamilyManager = require("Family.FamilyManager")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

FamilyWarShopType={--Map
    kPersonal = 1,  --个人商店
    kShare = 2      --共享商店
}

local This = class(...)

function This:create(id)
    local Morality = {}
    setmetatable(Morality,This)
    Morality:init(id)
    return Morality
end

function This:init(id)
    self.m_id = id
    self.m_legionPersonShopList = {}
    self.m_legionShareShopList = {}
    self:initList()
end


function This:initList()
    local personShopList = require("Metadatas.LegionWarPersonalShopConfig")
    local shareShopList = require("Metadatas.LegionWarShareShopConfig")
    self.m_legionPersonShopList = self:formateShopList(personShopList)
    self.m_legionShareShopList = self:formateShopList(shareShopList)
end

-- 工具函数
function This:formateShopList(list,types)

    local key = 1
    local res = {}
    local mList = {}
    if types==1  then
        for _,shopItem in pairs(list) do
            local shopNum = shopItem.shop_num
            local solt = shopItem.solt
            local temSolt = mList[shopNum] or 0
            if solt> temSolt then
                mList[shopNum] = solt
            end
        end
        return mList
    end

    local function getMaxNum( m_list )
        local shopNum = 0
        local num = 0
        for _,data in pairs(m_list) do
            shopNum = data.shop_num
            if data.solt > num then
                num = data.solt
            end
        end
        return shopNum, num
    end
    for _,v in pairs(list) do
        if v.shop_num == key then
            if not res[key] then
                res[key]={}
                table.insert(res[key],v)
            else
                table.insert(res[key],v)
            end
        else
            key = v.shop_num
            if not res[key] then
                res[key]={}
                table.insert(res[key],v)
            else
                table.insert(res[key],v)
            end
        end
    end

    for _,v in pairs(res) do
        local key,maxSolt=getMaxNum(v)
        mList[key]=maxSolt
    end
    return mList
end



-- 积分奖励状态列表处理
function This:rewardListFormat(rewardList,type)
    local list = rewardList
    for i=1,self:getMaxSolt(type) do
        local hasInsert = false
        for _,v in pairs(list) do
            if i == v.solt then
                hasInsert = true
            end
        end
        if hasInsert == false then
            table.insert( list,{solt = i,num = 0})
        end
    end
    return list
end

-- 获取下次奖励所需积分
function This:getScore( num )
    local list=self:getRewardStageList()
    local nextNum = list[1]
    for _,scoreValue in ipairs(list) do
        if scoreValue>num then
            nextNum = scoreValue
            break
        end
    end
    if num >=list[#list] then
        nextNum = list[#list]
    end
    return nextNum
end

-- 军营商店主键id
function This:setWarShopkey(shopKey)
    self.m_shopKey = shopKey 
end

function This:getWarShopkey()
    return self.m_shopKey or 0
end

-- 获取商店编号
function This:getWarShopId()
    if not self.m_shopId then
        for i,v in pairs(LegionWarShopTypeConfig) do
            local s = self:getWarShopkey()
            if v.id == self:getWarShopkey() then
                self.m_shopId = v.personal_shop_num
                break
            end
        end
    end
    return self.m_shopId or 1
end

-- 获取商店最大槽位数
function This:getMaxSolt(type)
    -- local res 
    -- if type == FamilyWarShopType.kPersonal then
    --     res =  require("Metadatas.LegionWarPersonalShopConfig")
    -- else
    --     res =  require("Metadatas.LegionWarShareShopConfig")
    -- end
    -- local maxSolt = 0
    -- for _,v in pairs(res) do 
    --     if self:getWarShopId() == v.shop_num then
    --         if v.solt > maxSolt then
    --             maxSolt = v.solt
    --         end
    --     end
    -- end
    local res
    if type == FamilyWarShopType.kPersonal then
        res = self.m_legionPersonShopList
    else
        res = self.m_legionShareShopList
    end
    local redd = res[self:getWarShopId()]
    return res[self:getWarShopId()]
end

-- 个人商店已购买信息
function This:setWarPersonShopList(rolesoltList)
    self.m_rolesoltList = self:rewardListFormat(rolesoltList,FamilyWarShopType.kPersonal)
end

function This:getWarPersonShopList()
    return self.m_rolesoltList 
end
 

-- 共享商店已购买信息
function This:setWarShareShopList(shareSoltList)
    self.m_shareSoltList = self:rewardListFormat(shareSoltList,FamilyWarShopType.kShare)
end

function This:getWarShareShopList()
    return self.m_shareSoltList 
end
 
-- 共享商店积分
function This:setWarShareShopIntegral(integral)
    local m_maxIntegral = 200
    self.m_integral = integral
    if self.m_integral > m_maxIntegral then
        self.m_integral = m_maxIntegral
    end
    --self.m_integral = 100 --测试
end

function This:getWarShareShopIntegral()
    return self.m_integral or 0
end

-- 积分奖励状态列表
function This:setRoleScoreReward(m_roleScoreRewarded)
    self.m_role_jifen_rewarded = m_roleScoreRewarded
end

function This:getRoleScoreReward()
    return self.m_role_jifen_rewarded or {}
end

-- 商店下次开启时间/ 商店关闭时间
function This:setOverTime(overTime)
    self.m_overTime = overTime
end

function This:getOverTime()
    return self.m_overTime or GamePlayer:getServerTime() + 3600
end


function This:getTytle()
    return FamilyManager:getMySelfTitle()
end

-- 获取积分阶段奖励列表
function This:getRewardStageList()
    if not self.RewardList then
        local res,des={},{}
        for _,data in pairs(LegionWarShopJifenConfig) do
            if not des[data.jifen] then
                table.insert(res,data.jifen)
                des[data.jifen] = data
            end
        end
        table.sort(res)
        self.RewardList = res
    end
    return self.RewardList
end

-- 是否有奖励未领取
function This:hasReward()
    local hasReward = false
    local index =1
    for _,v in pairs(self.m_role_jifen_rewarded) do
        local score = self:getRewardStageList()[index]
        if self.m_integral >= self:getRewardStageList()[index] and not v.is_get then
            hasReward = true
        end
        index = index+1
    end
    return hasReward
end

-- 积分奖励是否领取完毕
function This:hasRewardCompleted()
    local completed = true
    if not self:getRoleScoreReward() then
        completed = false
    end
    for i,reward in pairs (self:getRoleScoreReward()) do
        if not reward.is_get then
            completed = false
        end
    end
    return completed
end