-- @Author: ZYY
-- @Date:   2018-07-25 14:53:28
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-16 16:22:48
-- @File Name: LegionBianDuiEditItemView.lua
-- @Desc: 队伍英雄头像

local This , super = class(...,"NodeUI")
local GameUtil = require("Util.GameUtil")
local HeroConfig = require("Metadatas.HeroConfig")
local ItemUtil = require("Bag.ItemUtil")

function This:initData(info)
    self.m_filename = "LegionScene/NodeUI/LegionTroopEditItem.csb"
end

function This:initNode(info)
	self.m_hero = info.hero
	self.m_debuffInfo = info.debuffInfo
	self.m_isInAllTeam = false
	self.m_isInCurTeam = false
	self.m_isSwallow = info.isSwallow or false
	self.m_isContanier = info.isContanier or false

	self.m_csNode = cc.Node:createFromCSFile(self.m_filename)
	self:addChild(self.m_csNode)
end

function This:refresh()
	local csNode = self.m_csNode
	if self.m_hero then
		local heroId = self.m_hero:getId()
		csNode.hero:setImage(ItemUtil.getIconImage(heroId))
		csNode.bg1:setImage(ItemUtil.getQualityImage(heroId))

		local star = self.m_hero:getStarLv()
		for i = 1 , 5 do
			csNode["star"..i]:setVisible(star >= i)
		end
		csNode.hero:setVisible(true)
		csNode.level:setVisible(true)
		csNode.starbg:setVisible(star > 0)
		csNode.level:setString(self.m_hero:getLevel())

		local debuffInfo = self.m_debuffInfo
		if debuffInfo then
			local debuffNum = debuffInfo[heroId]
			local visible = debuffNum and debuffNum > 0
			csNode.ceng:setVisible(visible)
			if visible then
				csNode.ceng:setString(debuffNum)
			end
		end
	else
		csNode.bg1:setImage("MenuScene/itemQuality3.png")
		for i = 1 , 5 do
			csNode["star"..i]:setVisible(false)
		end
		csNode.hero:setVisible(false)
		csNode.starbg:setVisible(false)
		csNode.level:setVisible(false)
		if csNode.ceng then
			csNode.ceng:setVisible(false)
		end
	end
	self:refreshSelectState()
end

function This:refreshSelectState()
	local csNode = self.m_csNode
	if self.m_hero then
		self.m_isInAllTeam = self.m_judgeInAllTeamHandler and self.m_judgeInAllTeamHandler(self.m_hero)
		csNode.xuanzhong:setVisible(self.m_isInAllTeam)
		self.m_isInCurTeam = self.m_judgeInCurTeamHandler and self.m_judgeInCurTeamHandler(self.m_hero)
		csNode.xuanzhong.gou:setVisible(self.m_isInCurTeam)
	else
		csNode.xuanzhong:setVisible(false)
		csNode.xuanzhong.gou:setVisible(false)
	end
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.bg1:setTouch({
		onEnded = function()
			if not self.m_isInAllTeam then
				if self.m_selectHandler and self.m_hero then
					self.m_selectHandler(self.m_hero)
					self:refresh()
				end
			else
				g_scene:showTips(Serialize[61678])
			end
		end,
		isDynamic = self.m_isContanier,
		isContanier = self.m_isContanier,
		isSwallow = self.m_isSwallow,
		isScale = false
	})
end

function This:onEnter()
	super.onEnter(self)
	self:refresh()
end

function This:onEixt()
	super.onExit(self)
end

function This:setSelectHandler(handler)
	self.m_selectHandler = handler
end

function This:setJudgeIsInAllTeamHandler(handler)
	self.m_judgeInAllTeamHandler = handler
end

function This:setJudgeIsInCurTeamHandler(handler)
	self.m_judgeInCurTeamHandler = handler
end

function This:setModel(hero)
	if self.m_hero ~= hero then
		self.m_hero = hero
		self:refresh()
	else
		self:refreshSelectState()
	end
end

function This:getModel()
	return self.m_hero
end