-- @Author: ZYY
-- @Date:   2018-07-24 11:09:58
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-28 10:10:31
-- @File Name: LegionBianDuiEditPanel.lua
-- @Desc: 编辑部队界面

local This, super = class(...,"Dialog")
local UpdateLayer = require("UI.UpdateLayer")
local LegionBianDuiEditItemView = require("Map.LegionBianDuiEditItemView")
local HeroManager = require("Hero.HeroManager")

local kMaxBianDuiCount =3
local TroopName = 
{
	Serialize[61680],
	Serialize[61681],
	Serialize[61682],
	Serialize[61683],
	Serialize[61684]
}

function This:initData(info)
	self.m_filename = "LegionScene/NodeUI/LegionTroopsEdit.csb"
	if info then
		self.m_map = info.map
		self.m_debuffInfo = self.m_map:getDebuffInfo()
	end
end

function This:initNode(info)
	local pos = info.pos

	self.m_bianDuiCards = {}
	self.m_allBianDuiCards = {}
	self.m_itemViewList = {}
	self.m_curBianDui = {}
	
	local csNode = cc.Node:createFromCSFile(self.m_filename)
    self.m_csNode = csNode
    csNode:setToCenter()
	self:addChild(csNode)

	for i = 1, kMaxBianDuiCount do
		local bianduiTab = csNode["tab"..i]
		self:addTabButton(i , bianduiTab,function(tab)
			return self:selectBianDui(i)
		end)
		local biandui = self.m_map:getBianDui(i)
		local name = TroopName[i]
		if biandui and not biandui:isEmpty() then 
			name = biandui:getName()
		end
		bianduiTab.text:setString(name)
		bianduiTab.light.text:setString(name)
	end

	self.m_heroLayer = UpdateLayer:create({
		itemWidth = 58,
		itemHeight = 75,
		column = 7,
		spaceY = 0,
		scrollView = self.m_csNode.ScrollView,
	})
	self.m_heroLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_heroLayer:getContentSize()
	csNode.ScrollView:pushBackCustomItem(self.m_heroLayer, w,h,0)
	self.m_heroLayer:setCascadeOpacityEnabled(true)
	self.m_heroList = HeroManager:getAllHeroGotWithSort()
	
	self:initBianDuiCardNode()
	self:selectTabButton(pos)
end

function This:initBianDuiCardNode()
	for i = 1, 6 do
		local view = self:createCardItemView()
		self.m_csNode["team"..i]:addChild(view)
		self.m_itemViewList[i] = view
	end
end

function This:initTouchEvent()
	self.m_csNode.baocunbutton:setTouchEnded(function()
		local biandui = self.m_map:getBianDui(self.m_index)
		if not biandui:isEmpty() then
			local troop = biandui:getTroop()
			local state = troop:getState()
			if state ~= LegionTroopState.kIdle then
				g_scene:showTips(Serialize[61685])
				return
			end
		end
		local uidList = {}
		for i , heroId in ipairs(self.m_curBianDui) do 
			local hero = HeroManager:getHero(heroId)
			table.insert(uidList,hero:getUid())
		end
		if #uidList > 0 and #uidList < 6 then
			g_scene:showTips(Serialize[61686])
			return
		end
		local name = self.m_csNode.name:getString()
		if name == "" then
			g_scene:showTips(Serialize[61687])
			return 
		end
		self.m_map:onUpdateBianDuiRequest(self.m_index,uidList,name)
	end)

	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.changename:setTouchEnded(function()
		local panel = g_scene:openDialog("List.QueueRenamePanel",{
			defaultStr = "",
			maxLength = 5,
		})
		panel:setRenameHandler(function(str)
			self.m_csNode["tab" .. self.m_index]["text"]:setString(str)
			self.m_csNode["tab" .. self.m_index].light["text"]:setString(str)
			self.m_csNode.name:setString(str)
		end)
	end)
end

function This:onEnter()
	super.onEnter(self)
	self.m_map:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_map:removeObserver(self)
end

function This:createHeroItemView(hero)
	local view = LegionBianDuiEditItemView:create({
		hero = hero,
		debuffInfo = self.m_debuffInfo,
		isContanier = true
	})

	view:setSelectHandler(function(o)
		local isFind = false
		for i = 1 , 6 do
			if not self.m_curBianDui[i] then
				self:addCardToCurBianDui(i,o:getId())
				isFind = true
				break
			end
		end
		return isFind
	end)

	view:setJudgeIsInAllTeamHandler(function(o)
		local heroId = o:getId()
		return self.m_allBianDuiCards[heroId]
	end)

	view:setJudgeIsInCurTeamHandler(function(o)
		local heroId = o:getId()
		return self.m_bianDuiCards[self.m_index][heroId]
	end)
	return view
end

function This:createCardItemView(hero)
	local view = LegionBianDuiEditItemView:create({
		hero = hero,	
		debuffInfo = self.m_debuffInfo,
		isSwallow = true,
		isContanier = true
	})
	view:setSelectHandler(function(o)
		local biandui = self.m_map:getBianDui(self.m_index)
		if biandui:getState() ~= LegionTroopState.kIdle then
			g_scene:showTips(Serialize[61688])
			return
		end
		local isFind = false
		for i = 1 , 6 do
			if self.m_curBianDui[i] == o:getId() then
				self:removeCardFromCurBianDui(i,o:getId())
				isFind = true
				break
			end
		end
		return isFind
	end)
	return view
end

function This:updateBianDuiData()
	self.m_allBianDuiCards = {}
	self.m_curBianDui = {}
	local maxCount = self.m_map:getMaxBianDuiCount()
	for i  = 1, maxCount do
		self.m_bianDuiCards[i] = {}
		local biandui = self.m_map:getBianDui(i)
		if not biandui:isEmpty() then
			local troop = biandui:getTroop()
			for _, card in ipairs(troop:getCardList()) do
				local heroId = card:getHeroId()
				self.m_bianDuiCards[i][heroId] = true
				self.m_allBianDuiCards[heroId] = i
				if i == self.m_index then
					self.m_curBianDui[card:getPos()] = heroId
				end
			end
		end
		if i == self.m_index then
			local name = biandui:isEmpty() and TroopName[i] or biandui:getName()
			self.m_csNode.name:setString(name)
		end
	end
end

function This:selectBianDui(index)
	if self.m_index ~= index then
		self.m_index = index
		self:updateBianDuiData()
		self.m_heroLayer:setDataList(self.m_heroList,function(hero,index)
			local view = self:createHeroItemView(hero)
			return view
		end)
		self:refreshBianDuiCard()
	end
end

function This:refreshBianDuiCard(hero)
	local total = 0
	for i = 1 , 6 do
		if self.m_curBianDui[i] then
			local hero = HeroManager:getHero(self.m_curBianDui[i])
			self.m_itemViewList[i]:setModel(hero)
			total = total + hero:getFightPower()
		else
			self.m_itemViewList[i]:setModel(nil)
		end
	end
	self.m_csNode.zhanli:setString(total)
end

function This:addCardToCurBianDui(pos,heroId)
	print(pos,heroId)
	self.m_curBianDui[pos] = heroId
	if not self.m_bianDuiCards[self.m_index] then
		self.m_bianDuiCards[self.m_index] = {}
	end
	self.m_bianDuiCards[self.m_index][heroId] = true
	self.m_allBianDuiCards[heroId] = self.m_index
	local views = self.m_heroLayer:getAllViews()
	for i , view in pairs(views) do
		if view:getModel():getId() == heroId then
			view:refresh()
		end
	end	
	self:refreshBianDuiCard()
end

function This:removeCardFromCurBianDui(pos,heroId)
	self.m_curBianDui[pos] = nil
	self.m_bianDuiCards[self.m_index][heroId] = false
	self.m_allBianDuiCards[heroId] = false
	local views = self.m_heroLayer:getAllViews()
	for i , view in pairs(views) do
		view:refresh()
	end	
	self:refreshBianDuiCard()
end


---------------------------------------------------

function This:onUpdateBianDuiTroop(index,troop)
	self:updateBianDuiData()
	g_scene:showTips(Serialize[60673])
end
