-- @Author: ZYY
-- @Date:   2018-08-14 11:01:40
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-08 11:51:58
-- @File Name: LegionBianDuiView.lua
-- @Desc: 编辑部队列表
local This , super = class(...,"NodeUI")
local ItemUtil = require("Bag.ItemUtil")
local LegionCityConfig = require("Metadatas.LegionCityConfig")

-- This.fileName = ""

local StateImage = {
	[LegionTroopState.kAddHp] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kBattle] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kDefensive] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kIdle] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kMarch] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kTechAddSpeed] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kTransport] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kReadyBattle] = "MenuScene/zhengzhan.png"
}

local StateColor = 
{
	-- [LegionTroopState.kAddHp] = {255,0,255},
	-- [LegionTroopState.kBattle] = {255,0,255},
	-- [LegionTroopState.kDefensive] = {255,0,255},
	-- [LegionTroopState.kIdle] = {255,0,255},
	-- [LegionTroopState.kMarch] = {255,0,255},
	-- [LegionTroopState.kTechAddSpeed] = {255,0,255},
	-- [LegionTroopState.kTransport] = {255,0,255},
	-- [LegionTroopState.kReadyBattle] = {255,0,255}
	[LegionTroopState.kAddHp] = "<New9>",
	[LegionTroopState.kBattle] = "<New11>",
	[LegionTroopState.kDefensive] = "<New5>",
	[LegionTroopState.kIdle] = "<New9>",
	[LegionTroopState.kMarch] = "<New5>",
	[LegionTroopState.kTechAddSpeed] = "<New5>",
	[LegionTroopState.kTransport] = "<New5>",
	[LegionTroopState.kReadyBattle] = "<New11>"
}

function This:initNode(info)
	local biandui = info.biandui
	self.m_map = info.map
	self.m_index = info.index

	self:initCsNode()
	self:setModel(biandui)
end

function This:initCsNode()
    local csNode = cc.Node:createFromCSFile(self.m_fileName or "LegionScene/NodeUI/LegionPreviewlist.csb")
    self.m_csNode = csNode
    self:addChild(csNode)
    self:setContentSize(250, 75)
end

function This:getCityName(cityId)
	if cityId > 0 then
		return LegionCityConfig[cityId].city_name
	end
	return ""
end

function This:refresh()
	local contentNode = self.m_csNode.node
	if not self.m_biandui or self.m_biandui:isEmpty() then 
		contentNode:setVisible(false)
	else
		local troop = self.m_biandui:getTroop()
		local cardList = troop:getCardList()
		local firstCard
		local maxHp , hp = 0, 0
		for i , card in ipairs(cardList) do
			maxHp = maxHp + card:getMaxHp()
			hp = hp + card:getHp()
			if card:getPos() == 1 then
				firstCard = card
			end
		end
		if not firstCard then
			firstCard = cardList[1]
		end
		local heroId = firstCard:getHeroId()
		contentNode.icon:setImage(ItemUtil.getIconImage(heroId))
		contentNode.name:setString(self.m_biandui:getName())
		contentNode.iconbg:setImage(ItemUtil.getQualityImage(heroId))
		self:refreshState(troop)
		contentNode.hpbat:setPercent(hp / maxHp * 100)
		contentNode:setVisible(true)
	end
end

function This:refreshState(troop)
	local contentNode = self.m_csNode.node

	local state = troop:getState()
	contentNode.pic:setImage(StateImage[state])

	local cityId = troop:getCityId()
	local statestr = LegionTroopStateText[state]
	if cityId > 0 then
		 statestr = self:getCityName(cityId) .. statestr
	end
	-- contentNode.state2:setString(statestr)
	contentNode.state2:setStringWithColor(StateColor[state]..statestr)
	-- contentNode.state2:setColor(StateColor[state][1],StateColor[state][2],StateColor[state][3])
end

function This:onEnter()
	super.onEnter(self)
	self:refresh()
	self.m_map:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_map:removeObserver(self)
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.biandui:setTouch({
		onEnded = function()
			g_scene:openDialog("Map.LegionBianDuiEditPanel",{
				pos = self.m_biandui:getPos(),
				map = self.m_map
			})
		end,
		isDynamic = true
	})

	csNode.node.iconbg:setTouch({
		onEnded = function()
			if self.m_map and self.m_biandui then
				local troop = self.m_biandui:getTroop()
				if troop then
					local state = troop:getState()
					if state == LegionTroopState.kMarch then
						local marchId = troop:getMarchId()
						if marchId and marchId > 0 then
							g_scene:focuseMarch(marchId)
						end
					else
						local cityId = troop:getCityId()
						if cityId and cityId > 0 then
							g_scene:focuseToCity({cityId = cityId})
						else
							g_scene:showTips(Serialize[61691])
						end
					end
				end
			end
		end,
		isDynamic = true
	})
end

function This:setModel(biandui)
	if self.m_biandui ~= biandui then
		self.m_biandui = biandui 
	end
	self:refresh()
end

function This:getModel()
	return self.m_biandui
end

---------------------------------------------------------

function This:onUpdateTroop(troop)
	if self.m_biandui:getTroop() == troop then
		self:refresh()
	end
end