--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local LegionCityConfig = require("Metadatas.LegionCityConfig")
local LegionMapRoomConfig = require("Metadatas.LegionMapRoomConfig")
local LegionChangeLevelConfig = require("Metadatas.LegionChangeLevelConfig")
local LegionTech = require("Map.LegionTech")
local FamilyManager = require("Family.FamilyManager")
local LegionMapUtil = require("Map.LegionMapUtil")
local LegionChengFang = require("Map.LegionChengFang")
local GamePlayer = require("Player.GamePlayer")
local ArgConfig = require("Metadatas.ArgConfig")

function This:init(info)
	self.m_id = info.id
	self.m_map = info.map
	self.m_level = 0

	self.m_oreNum = 0
	self.m_hpNum = 0
	self.m_damage = 0
	self.m_state = 0
	self.m_legionId = 0

	self.m_guardTroops = {}
	self.m_addHpTroops = {}
	self.m_chengfang = {}
	self.m_cfNextTime = 0

	self.m_jianshedu = 0
	self.m_buildState = LegionBuildState.kNormal
	self.m_addSpeedTroops = {}

    self:initByConfig()
end

function This:initByConfig()
	local initLevel = self.m_map:getCityInitLevel()
	self.m_cityConfig = LegionCityConfig[self.m_id]
	self.m_buildType = self.m_cityConfig.initial_change_tpye

	local levelConfigId = self:getLevelConfigId(initLevel)
	self.m_levelConfig = LegionChangeLevelConfig[levelConfigId]
	if self.m_levelConfig == nil then
		assert(false)
	end
	self.m_technologys = {}
	self:setLevel(initLevel)
end

function This:isBarrack()
	return self.m_levelConfig.change_type == 2
end

function This:getId()
	return self.m_id
end

function This:getCityId()
	return self.m_id
end

function This:getType()
	return self.m_cityConfig.city_type
end

function This:getName()
	return self.m_cityConfig.city_name
end

function This:getLevel()
	return self.m_level
end

function This:getMaxLevel()
	return self.m_map:getCityMaxLevel()
end

function This:getBuildType()
	return self.m_buildType
end

function This:getBuildName()
	return self.m_levelConfig.change_name
end

function This:isBarracks()
	return self.m_buildType == LegionBuildType.kBarracks
end

function This:isWarehouse()
	return self.m_buildType == LegionBuildType.kWarehouse
end

function This:isMintage()
	return self.m_buildType == LegionBuildType.kMintage
end

function This:getLevelConfigId(level)
	return self.m_buildType * 100 + level
end

function This:getDesc()
	return self.m_levelConfig.change_dec
end

--是否能改造
function This:isCanRebuild()
	return self.m_cityConfig.if_change == 1
end

function This:getNeighborhood()
	local result = {}
	for i , id in ipairs(self.m_cityConfig.neighborhood) do
		local city = self.m_map:getCity(id)
		table.insert(result,city)
	end
	return result
end

function This:getImage()
	if self.m_levelConfig.picture1 == "" then
		return "LegionScene/map/maincity.png"
	end
	return self.m_levelConfig.picture1
end

--获取升级花费
function This:getLevelUpCost()
	return self.m_levelConfig.levelup_cost
end

--获取改造花费
function This:getRemouldCost()
	return self.m_levelConfig.change_cost
end

function This:getConfigNextLevel()
	local maxLevel = self:getMaxLevel()  
	local level = self.m_level < maxLevel and (self.m_level + 1) or maxLevel
	local config = LegionChangeLevelConfig[self.m_buildType * 100 + level]
	return config
end

function This:getBuildNameNextLevel()
	local config = self:getConfigNextLevel()
    return config.change_name
end

function This:getTechNextLevel( techType )
	local config = self:getConfigNextLevel()
	for i , info in ipairs(config.technology) do
		if info[1] == techType then
			return info[2]
		end
	end
end

function This:isUnlock(level)
	-- local configId = self.m_buildType * 100 + level
	-- if LegionChangeLevelConfig[configId] then
	-- 	return self.m_techTotalLevel >= LegionChangeLevelConfig[configId].unlock_condition
	-- end
	return true
end

function This:getTech(techType)
	return self.m_technologys[techType]
end

function This:getMaxOre()
	if self:isWarehouse() then
		local tech = self:getTech(6)       --- 6 城池科技仓库id
		return tech:getTechAttrValue(7)		--- 7 矿石储蓄值对应id
	else
		return 0
	end
end

function This:getTechList()
	local techList = {}
	for i , tech in pairs(self.m_technologys) do
		table.insert(techList,tech)
	end
	table.sort(techList,function(a,b)
		return a:getId() < b:getId()
	end)
	return techList
end

function This:setLevel(level)
	self.m_level = math.min(level, self.m_map:getCityMaxLevel())
	local levelConfigId = self:getLevelConfigId(self.m_level)
	self.m_levelConfig = LegionChangeLevelConfig[levelConfigId]
	for i , info in ipairs(self.m_levelConfig.technology) do
		local techId = info[1]
		local maxLevel = info[2]
		if not self.m_technologys[techId] then
			self.m_technologys[techId] = LegionTech:create({
				maxLevel = maxLevel,
				initLevel = self.m_map:getCityInitLevel(),
				buildType = self.m_buildType,
				techType = techId
			})
		end
		self.m_technologys[techId]:setMaxLevel(maxLevel)
	end
end

function This:getResNum()
	if self.m_buildType == LegionBuildType.kBarracks then
		return self.m_hpNum
	elseif self.m_buildType == LegionBuildType.kWarehouse then
		return self.m_oreNum
	end
	return 0
end

function This:isAlertBuilding()
	return self.m_buildType == LegionBuildType.kArrowTower or self.m_buildType == LegionBuildType.kBoltTower
end

function This:isAlertMarch(march)
	if march:getToCity() == self.m_id and march:getTransType() == LegionTransType.kHp then
		local fromCity = self.m_map:getCity(march:getFromCity())
		local toCity = self.m_map:getCity(march:getToCity())
		if fromCity:isOur() == toCity:isOur() or (march:isOur() and toCity:isOur()) then
			return false
		else
			return true
		end
	end
	return false
end

function This:getHp()
	if self.m_buildType == LegionBuildType.kBarracks then
		return self.m_hpNum
	end
	return 0
end

function This:getMaxHp()
	if self.m_buildType == LegionBuildType.kBarracks then
		return self.m_technologys[LegionTechType.kAddMaxHp]:getBarracksMaxHp()
	end
	return 0
end

function This:getMineral()
	if self.m_buildType == LegionBuildType.kMintage then
		return self.m_oreNum
	end
	return 0
end

function This:getMaxMineral()
	if self.m_buildType == LegionBuildType.kMintage then
		local tech = self.m_technologys[LegionTechType.kAddMaxMineral]
		return tech:getMineralMaxLimit()
	end
	return 0
end

function This:getDamage()
	if self.m_buildType == LegionBuildType.kArrowTower or self.m_buildType == LegionBuildType.kBoltTower then
		return self.m_damage
	end
	return 0
end

function This:getState()
	return self.m_state
end

function This:setState(state)
	self.m_state = state
end

function This:isProtected()
	return self.m_state == LegionCityState.kProtected
end

function This:isOur()
	local myLegionId = FamilyManager:getMyFamilyId()
	if myLegionId then
		return self.m_legionId == myLegionId
	end
	return false
end

function This:isEmpty()
	return self.m_legionId <= 0
end

function This:isFighting()
	return self.m_state == LegionCityState.kFighting
end

function This:isImprisoned()
	local junce = self.m_map:getJunCe(LegionStrategyType.kBaGua)
	if junce:isUsedOnCity(self:getId()) then
		return true
	end
end

function This:getGuardTroops()
	return self.m_guardTroops
end

function This:getGuardTroopCount()
	local count = 0
	for i , troop in pairs(self.m_guardTroops) do
		count = count + 1
	end
	return count
end

function This:getAttackTroops()
	return self.m_attackTroops
end

function This:getAttackTroopCount()
	local count = 0
	for i , troop in pairs(self.m_attackTroops) do
		count = count + 1
	end
	return count
end

function This:getAddHpTroops()
	return self.m_addHpTroops
end

function This:getLegionId()
	return self.m_legionId
end

function This:getCfNextCreateTime()
	return self.m_cfNextTime
end

function This:getLeftCfNextTime()
	local curTime = GamePlayer:getServerTime()
	return math.max(0,self.m_cfNextTime - curTime)
end

function This:getCostWithClearCfTime()
	local curTime = GamePlayer:getServerTime()
	local time = math.max(0,self.m_cfNextTime - curTime)
	return math.ceil(time * ArgConfig["legion_war_city_npc_coefficient"].arg[1])
end

function This:getCfQualityWithOnceKey()
	return ArgConfig["legion_war_city_npc_train_quality"].arg[1]
end

function This:getCfTrainCostWithOnceKey()
	local total = 0
	for i , chengfang in ipairs(self.m_chengfang) do
		total = total + chengfang:getCfTrainCostWithOnceKey()
	end
	return total
end

function This:getCFMaxCount()
	if self.m_technologys then
		local tech = self.m_technologys[LegionTechType.kAddWallSoilderSolt]
		if tech then
			return tech:getTechAttrValue(LegionTechAttrType.kChengFangSoltMaxLimit)
		end
	end
	return 0
end

function This:getCFCount()
	return #self.m_chengfang
end

function This:getGuardCountAndHp()
	local total = 0
	local count = 0
	for i , troop in pairs(self.m_guardTroops) do
		if troop:isOur() then
			total = total + troop:getTotalHp()
			count = count + 1
		end
	end
	return count, total
end

function This:getChengFangList()
	return self.m_chengfang
end

function This:getAddHpSpeed()
	local tech = self.m_technologys[LegionTechType.kSuppleHpSpeed]
	return tech:getTechAttrValue(LegionTechAttrType.kBarracksSuppleHpSpeed)
end

function This:updateTech(info)
	local tech = self.m_technologys[info.tech_type]
	tech:updateInfo(info)
	return tech
end

function This:isInProtected()
	local curTime = GamePlayer:getServerTime()
	if self.m_map:getFightTime() > curTime and self.m_cityConfig.city_type == LegionCityType.kProtected then
		return true
	end
	return self.m_state == LegionCityState.kProtected
end

function This:isEmptyCity()
	--  if self.m_state == LegionCityState.kProtected then
		local junce = self.m_map:getJunCe(LegionStrategyType.kKongChengJi)
		if junce:isUsedOnCity(self:getId()) then
			return true
		end
	--  end
	return false
end

function This:isInFighting()
	return self.m_state == LegionCityState.kFighting
end

function This:updateInfo(info)
	local oldLegionId = self.m_legionId
	local oldBuildType = self.m_buildType
	self.m_buildType = info.build_type
	self.m_hpNum = info.force_num
	self.m_oreNum = info.ore_num
	self.m_legionId = info.group_id
	self.m_damage = info.damage
	self.m_state = info.state

	if self.m_buildType == 0 then
		assert(true)
	elseif oldBuildType and oldBuildType ~= self.m_buildType then
		self.m_technologys = {}
	end

	self:setLevel(info.level)

	self:updateAddSpeedTroopList(info)

	self:updateTechList(info.tech_list)

	self.m_guardTroops = {}
	for i , id in ipairs(info.guard_troop_list) do
		self.m_guardTroops[id] = self.m_map:getTroop(id)
	end
	self.m_attackTroops = {}
	for i , id in ipairs(info.attack_troop_list) do
		self.m_attackTroops[id] = self.m_map:getTroop(id)
	end
	self.m_addHpTroops = {}
	for i , id in ipairs(info.fill_force_troop_list) do
		self.m_addHpTroops[id] = self.m_map:getTroop(id)
	end
	for i , o in ipairs(info.cf_list) do
		local cf = self.m_chengfang[i]
		if not cf then
			cf = LegionChengFang:create({
				map = self.m_map,
				pos = i
			})
			self.m_chengfang[i] = cf
		end
		cf:updateInfo(o)
	end
	for i = #info.cf_list + 1, #self.m_chengfang do
		self.m_chengfang[i] = nil
	end
	self.m_cfNextTime = info.cf_next_time

	if oldLegionId > 0 then
		local legion = self.m_map:getLegion(oldLegionId)
		if legion then
			legion:removeCity(self)
		end
	end

	if self.m_legionId > 0 then
		local legion = self.m_map:getLegion(self.m_legionId)
		if legion then
			legion:addCity(self)
		end
	end
end

function This:isInBuilding()
	return self.m_buildState == LegionBuildState.kUpgrading
end

function This:setBuildState(state)
	self.m_buildState = state
end

-- 获取pos的加速部队信息
function This:getTroopInfo(pos)
	return self.m_addSpeedTroops[pos]
end

function This:hasTroopAddSpeed(pos)
	return self.m_addSpeedTroops[pos] ~= nil
end

function This:getCurJianSheDu()
	return self.m_jianshedu
end

function This:getAddSpeedEndTime(troopId)
	for _, info in pairs(self.m_addSpeedTroops) do
		if info.troop_id == troopId then
			return info.endtime
		end
	end
	return 0
end

function This:getMaxJianSheDu()
	return self.m_levelConfig.degree_num
end

function This:updateTechList( info )
	for i , techInfo in ipairs(info) do
		local tech = self.m_technologys[techInfo.tech_type]
		if tech then
			tech:updateInfo(techInfo)
		end
	end
end

function This:updateAddSpeedTroopList( info )
	self.m_jianshedu = info.jianshedu
	self.m_buildState = info.tech_state
	local tmp = {}
	for pos , troop in pairs(self.m_addSpeedTroops) do
		tmp[pos] = true
	end
	
	for i , o in pairs(info.troop_list) do
		local addspeed = self.m_addSpeedTroops[o.pos]
		if addspeed then
			addspeed.troop_id = o.troop_id
			addspeed.pos = o.pos
			addspeed.add_speed_id = o.add_speed_id
			addspeed.endtime = o.endtime
			addspeed.is_rate = o.is_rate
		else
			self.m_addSpeedTroops[o.pos] = {
				troop_id = o.troop_id,
				pos = o.pos,
				add_speed_id = o.add_speed_id,
				endtime = o.endtime,
				is_rate = o.is_rate
			}
		end
		tmp[o.pos] = nil
	end
	for pos , _ in pairs(tmp) do
		self.m_addSpeedTroops[pos] = nil
	end
end

--endregion
