-- @Author: zheng
-- @Date:   2019-03-21 17:46:32
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-03 17:28:34
-- @File Name: LegionEditMinMapNode.lua

local This , super = class(...,"NodeUI")
local ColorList = 
{
    cc.Color.WHITE,
    cc.Color.YELLOW,
    cc.Color.GREEN,
    cc.Color.BLUE,
    cc.Color.RED,
    cc.Color.MAGENTA,
    cc.Color.BLACK,
    cc.Color.ORANGE,
    cc.Color.GRAY,
    cc.ColorList.New14,
    cc.ColorList.New12,
    cc.ColorList.New11,
    cc.ColorList.New10,
    cc.ColorList.New9,
    cc.ColorList.New4,
    cc.ColorList.QQ6,
    cc.ColorList.QQ5,
    cc.ColorList.QQ3,
    cc.ColorList.PetPhase2,
    cc.ColorList.PetPhase1,
    cc.ColorList.PetPhase5
}

local CityFlag = {
    [LegionBuildType.kArrowTower] = "WarScene/town_0.png",
    [LegionBuildType.kBarracks] = "WarScene/town_0.png",
    [LegionBuildType.kBoltTower] = "WarScene/town_0.png",
    [LegionBuildType.kFarmland] = "WarScene/town_0.png",
    [LegionBuildType.kMainCity] = "WarScene/town_0.png",
    [LegionBuildType.kMarket] = "WarScene/town_0.png",
    [LegionBuildType.kMintage] = "WarScene/town_0.png",
    [LegionBuildType.kNone] = "WarScene/town_0.png",
    [LegionBuildType.kWarehouse] = "WarScene/town_0.png",
}

function This:initNode(info)
    if info then
        local map = info.map
        local mapNode = info.mapNode.img or info.mapNode
        local scaleRateX = info.scaleRate or info.scaleRateX
        local scaleRateY = info.scaleRate or info.scaleRateY
        local mapRadioX, mapRadioY = info.mapRadioX, info.mapRadioY

        self.m_scene = g_scene
        self.m_map = map
        self.m_mapNode = mapNode
        self.m_mapContainNode = info.mapNode
        self.m_liangNode = cc.Node:create()
        mapNode:addChild(self.m_liangNode)
        self.m_ceilsNode = cc.Node:create()
        mapNode:addChild(self.m_ceilsNode)
        self.m_scaleRateX = scaleRateX
        self.m_scaleRateY = scaleRateY
        self.m_cityInfor ={}

        -- self.m_ceilConfig = map:getCityCeilConfig()
        local w , h = mapNode:getContentSize()
        local dx , dy = 20 * mapRadioX * scaleRateX, 12 * mapRadioY * scaleRateY
        local f = dy / dx
        local nx , ny = math.ceil(w / dx), math.ceil(h / dy)
        self.m_nx = nx
        self.m_ny = ny
        self.m_dx = dx
        self.m_dy = dy
        self.m_f = f
        self.m_vy = cc.Vec(self.m_dx,self.m_dy):normalize()
        self.m_vx = cc.Vec(-self.m_dx,self.m_dy):normalize()
    end

end

-- function This:getMyDx()
--     local map = self.m_map
--     if map then
--         return self.m_dx
--     end
-- end

-- function This:getMyDy()
--     local map = self.m_map
--     if map then
--         return self.m_dy
--     end
-- end

-- function This:getMyF()
--     local map = self.m_map
--     if map then
--         return self.m_f
--     end
-- end


function This:refresh()
   self:initCitys()
   self:initCeils()
end

--色块
function This:initCeils()
    -- local map = self.m_map
    -- if map then
        -- self.m_ceilsNode:removeAllChildren()
        self.m_cityNum = {}
        local citys = self.m_scene:getAllCitys()
        for cityId , city in pairs(citys) do
            self:createCityCeils(cityId,city)
        end
    -- end
end

function This:createCityCeils(cityId,configs)
    local cityCeils = self.m_cityInfor[cityId] or {}
    for j , ceilKey in ipairs(configs.ceilList) do
        local nx = math.floor(ceilKey / 1000)
        local ny = ceilKey % 1000
        local cityName = ""..nx..ny
        local sprite = cityCeils[cityName]
        local ceilColor = ColorList[configs.color] or cc.Color(0,0,0,0)
        if not sprite then
            sprite = self:createCeil(nx,ny,ceilColor)
            sprite:setName(cityName)
        end
        sprite:setColor(ceilColor.r,ceilColor.g,ceilColor.b)
    end
    self.m_cityInfor[cityId] = cityCeils
end

--白点
function This:initCitys()
    -- local map = self.m_map
    -- if map then
        local liangNode = self.m_liangNode
        liangNode:removeAllChildren()
        local citys = self.m_scene:getAllCitys()
        for cityId, city in pairs(citys) do
            local rx , ry = self.m_scene:getCityNodeRadio(cityId)
            local buildType = LegionBuildType.kNone--city:getBuildType()
            local sprite = cc.Sprite:create(CityFlag[buildType])
            liangNode:addChild(sprite)
            sprite:setScale(self.m_scaleRateX, self.m_scaleRateY)
            local w , h = self.m_mapNode:getContentSize()
            sprite:setPosition(w * rx, h * ry)
        end
    -- end
end

function This:createCeil(nx,ny,color)
    local b = cc.Vec(self.m_dx, self.m_dy):length() / 2
    local t = self.m_vy * b * (ny + 0.5) + self.m_vx * b * (nx + 0.5)
    local layer = cc.Sprite:create("LegionScene/lingxing.png")
    layer:setPosition(t.x,t.y)
    layer:setColor(color.r,color.g,color.b)
    layer:setOpacity(128)
    layer:setScale(self.m_dx / 250)
    
    self.m_ceilsNode:addChild(layer)
    return layer
end

function This:getLegionColor(legionId)
    return self.m_map:getLegionColor(legionId)
end

function This:getLegionCityNums()
    return self.m_cityNum or {0,0,0,0}
end
