-- @Author: superm
-- @Date:   2018-12-15 17:58:31
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-27 16:54:18
-- @File Name: LegionEditSearchCityPanel.lua
-- @Description: 


local This , super = class(...,"Dialog")
local UpdateLayer = require("UI.UpdateLayer")
local GameUtil = require("Util.GameUtil")
local CityConfig = false

function This:initNode(info)
	self.m_scene = g_scene
	CityConfig = g_scene:getCityConfig()
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/SearchCity.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self.m_searchContent = ""

	self.m_listLayer = UpdateLayer:create({
		itemWidth = 390,
		itemHeight = 40,
		spaceY = 0,
		column = 1,
		scrollView = self.m_csNode.scrollView,
	})
	self.m_listLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_listLayer:getContentSize()
	self.m_csNode.scrollView:pushBackCustomItem(self.m_listLayer, w,h,0)

	self:searchCity(self.m_searchContent)
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.search:setTouchEnded(function()
		local str = self.m_csNode.editor:getString()
		if self.m_searchContent == "" and str == "" then
			g_scene:showTips(Serialize["world_SearchCity_1"])
			return
		end
		self.m_searchContent = str
		self:searchCity(str)
		if self.m_searchContent ~= "" and #self.m_cityList > 0 then
			g_scene:showTips(Serialize["world_SearchCity_2"])
		end
	end)
end

function This:isNumberString(str)
	if str == "" then
		return false
	end
	local minByte = string.byte('0')
	local maxByte = string.byte('9')
	for i = 1 , #str do
		local byte = string.byte(str,i)
		if not (byte >= minByte and byte <= maxByte) then
			return false
		end
	end 
	return true
end

function This:getCityList(str)
	str = string.trimAllSpace(str)
	local result = {}
	if self:isNumberString(str) then
		local id = tonumber(str)
		if CityConfig[id] then
			table.insert(result,id)
		end
	elseif str == "" then
		local citys = self.m_scene.m_cityInfo
		for id , _ in pairs(citys) do
			table.insert(result,id)
		end
	else
		local format = {}
		table.insert(format,str)
		local len = string.utf8Len(str)
		if len > 1 then
			for i = 1 , len do
				local char = GameUtil:getChar(str,i)
				table.insert(format,char)
			end
		end
		local mohu = {}
		local citys = self.m_scene.m_cityInfo
		for id , city in pairs(citys) do
			local config = CityConfig[id]
			for i , ft in ipairs(format) do
				ft = string.format("(.*%s.*)",ft)
				local rt = string.match(config.city_name,ft)
				if rt then
					if rt == str then
						table.insert(result,id)
					else
						table.insert(mohu,id)
					end
					break
				end
			end				
		end
		table.sort(mohu,function(a,b)
			return a < b
		end)

		for i , id in ipairs(mohu) do
			table.insert(result,id)
		end
	end
	return result
end

function This:getCityItem(id,index)
	local item = cc.Node:createFromCSFile("GameScene/NodeUI/SearchCityItem.csb")
	item.goto:setTouch({
		onEnded = function()
			g_scene:focuseCity(id)
			self:close()
			g_scene:showTips(Serialize["world_SearchCity_3"])
		end,
		isScale = false,
		isContainer = true
	})

	local refresh = function()
		item.number:setString(id)
		item.cityname:setString(CityConfig[id].city_name)	
		item.bg:setVisible(index % 2 == 1)
	end
	refresh()
	
	item.setModel = function(_,cityId, cityIndex)
		id = cityId 
		index = cityIndex
		refresh()
	end
	
	local w , h = item.bg:getContentSize()
	return item , w, h
end

function This:searchCity(str)
	self.m_cityList = self:getCityList(str)
	self.m_listLayer:setDataList(self.m_cityList,function(id,index)
		return self:getCityItem(id,index)
	end)
	self.m_csNode.destip:setVisible(#self.m_cityList <= 0)
end

