--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Stage.EmbattleHeroPanel")
local Dialog = require("Dialog")
local MapPath = require("Map.MapPath")
local GameUtil = require("Util.GameUtil")
local LegionMapUtil = require("Map.LegionMapUtil")
local HeroManager = require("Hero.HeroManager")
local StageHeroItemView = require("Stage.StageHeroItemView")

function This:initNode(info)
	self.m_biandui = info.biandui
	self.m_fromCity = info.fromCity
	self.m_toCity = info.toCity
	self.m_map = info.map
	self.m_transType = info.transType
	self.m_resList = info.resList
	self.m_scene = g_scene

	self.m_heros = {}
	self.m_fetters = {}
	self.m_fetterInfo = {}

	self.m_extraJiceList = self:getExtraJiceList()

	local junce = self.m_map:getJunCe(LegionStrategyType.kQiangXi)
	if not self.m_toCity:isOur() and junce:isUsedOnCity(self.m_toCity:getId()) then
		self.m_path = MapPath:getMapJicePath(self.m_fromCity:getId(),self.m_toCity:getId(),self.m_map,self.m_transType)
		self.m_isUseJice = true
	else
		self.m_path = MapPath:getMapPath(self.m_fromCity:getId(),self.m_toCity:getId(),self.m_map,self.m_transType)
		self.m_isUseJice = false
	end

	self.m_csNode = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionLineUp.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self:initBaseInfo()
    self:initListLayer(info)
	self:initItems()
end

function This:initBaseInfo()
	self.m_csNode.bg3.taget:setString(self.m_toCity:getName())
	local legion = self.m_map:getMyLegion()
	self.m_csNode.bg3.name:setString(legion:getName())

	local res = self:getTotalHp()
	if self.m_transType == LegionTransType.kHp then
		self.m_csNode.bg3.totalhptext:setString(Serialize[61712])
	else
		self.m_csNode.bg3.totalhptext:setString(Serialize[61713])
	end
	self.m_csNode.bg3.totalhp:setString(res)

	local food = self.m_path:getCostFood(res, self.m_isUseJice, self.m_extraJiceList)
	self.m_csNode.bg3.consume:setString(Serialize[61714])
	self.m_csNode.bg3.cost:setString(math.ceil(food))

	local time = self.m_path:getCostTime(self.m_isUseJice, self.m_extraJiceList)
	if self.m_map:isToJueShengTime() and self.m_transType == LegionTransType.kHp then
		local fator = self.m_map:getMarchTimeFactorInVictory()
		time = math.ceil(time * fator)
	end
	self.m_csNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"],time))
	self.m_cost = food
end

function This:getHeroSource()
	local result = {}
	local troop = self.m_biandui:getTroop()
	local cardList = troop:getCardList()
	for _, card in ipairs(cardList) do
		local pos = card:getPos()
		result[pos] = card:getHeroId()
	end
	return result
end

function This:onEnter()
	Dialog.onEnter(self)
	self.m_map:addObserver(self)
end

function This:onExit()
	Dialog.onExit(self)
	self.m_map:removeObserver(self)
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.Battlebutton:setTouchEnded(function()
		if self.m_transType == LegionTransType.kMineral and not self.m_toCity:isOur() then
			g_scene:showTips(Serialize[61715])
			return
		end

		if self.m_map:getFood() < self.m_cost then
			g_scene:showTips(Serialize[61716])
			return
		end

		self.m_map:onCreateMarchRequest({
			index = self.m_biandui:getPos(),
			res_type = self.m_transType,
			res_list = self.m_resList,
			from_city = self.m_fromCity:getId(),
			to_city = self.m_toCity:getId()
		})
	end)
end

function This:getTotalHp()
	local total = 0
	for i , num in ipairs(self.m_resList) do
		total = total + num
	end
	return total
end

function This:getHeroView(heroId)
	local hp, maxHp = 0, 0
	local cardList = self.m_biandui:getTroop():getCardList()
	for _, card in ipairs(cardList) do
		if card:getHeroId() == heroId then
			local pos = card:getPos()
			hp = self.m_resList[pos]
			if self.m_transType == LegionTransType.kMineral then
				maxHp = self.m_map:getTransMaxMineral(card:getMaxHp(),card:getHeroId())
			else
				maxHp = card:getMaxHp()
			end
			break
		end
	end
	local node = StageHeroItemView:create(heroId,{
		hp = hp,
		maxHp = maxHp
	})
	return node
end

function This:addHeroView(heroId,pos)
	local node = self:getHeroView(heroId)
	node:setAnchorPoint(0.5,0.5)
	self.m_csNode["hero"..pos]:addChild(node)
	self.m_heros[pos].node = node
end

------------------------------------------------------
function This:onCreateMarch(march)
	self:close()
	self.m_scene:exitSelectMode()
end

function This:getExtraJiceList()
	local ret = {}
	local list = {LegionStrategyType.kPoFu, LegionStrategyType.kQuanJunChuJi}
    for _, v in ipairs(list) do
    	if self.m_map:isJunceEffectInLegion(v) then
	    	ret[v] = true
    	end
    end
    return ret
end

--endregion
