-- @Author: superm
-- @Date:   2019-09-16 14:24:56
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-11 15:32:39
-- @File Name: LegionIntegralItemView.lua
-- @Description: 

local This , super = class(...,"NodeView")
local GameUtil = require("Util.GameUtil")

function This:initData(box)
    self.m_box = box
    self.m_map = require("Map.GameMapManager"):getMap(GameMapType.kLegionMap)
    self.m_family = self.m_map:getMyLegion()

    local csNode = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionBuildIntegralItem.csb")
    self.m_csNode = csNode
    self:addChild(csNode)
    local w , h = csNode.bg:getContentSize()
    self:setContentSize(w + 39, h)
end

function This:initNode(info)

	local csNode = self.m_csNode
	local config = self.m_box:getConfig()
	csNode.title:setString(config.title)
	local groupIntegral, prestigeIntegral, jiceAdd = config.need_group_integral, config.need_prestige_integral, config.silkbag_add
	csNode.des.text1:setString(string.format(Serialize["legion_group_integeral_task"], groupIntegral))
	csNode.des.text2:setString(string.format(Serialize["legion_prestige_integeral_task"], prestigeIntegral))
	csNode.jinnang:setString(string.format(Serialize["legion_integeral_jinnang"], jiceAdd))

	csNode.buttonList:setVisible(true)
    -- csNode.scrollview:setSwallowTouch(false)
    GameUtil:resetScoolItemNode(csNode.scrollview, config.reward_list, nil, nil, nil, 3)
    self.m_groupIntegral, self.m_prestigeIntegral = groupIntegral, prestigeIntegral

    self:refresh()
end

function This:onEnter()
    super.onEnter(self)
    self.m_box:addObserver(self)
end

function This:onExit()
    self.m_box:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
    local csNode = self.m_csNode
    local hasPickup = self.m_box:getState()
    local pkbtn = csNode.buttonList.button
    if hasPickup then
        pkbtn:clearShader()
        pkbtn:setVisible(false)
        csNode.buttonList.yilingqu:setVisible(true)
    else
        pkbtn:setVisible(true)
        csNode.buttonList.yilingqu:setVisible(false)
        if self.m_groupIntegral <= self.m_family:getJiFen() and self.m_prestigeIntegral <= self.m_map:getJiFen() then
            pkbtn:clearShader()
        else
            pkbtn:setGrayShader()
        end
    end
end

function This:initTouchEvent()
	self.m_csNode.buttonList.button:setTouch({
        isContainer  = true,
        onEnded = function()
            local hasPickup = self.m_box:getState()
            if hasPickup then
                return
            end

            if self.m_groupIntegral <= self.m_family:getJiFen() and self.m_prestigeIntegral <= self.m_map:getJiFen() then
                self.m_map:onPickupJifenBoxRequest(self.m_box:getId())
            elseif self.m_prestigeIntegral > self.m_map:getJiFen() then
                g_scene:showTips(string.format(Serialize["legion_prestige_integeral_task"], self.m_prestigeIntegral))
            else
                g_scene:showTips(string.format(Serialize["legion_group_integeral_task"], self.m_groupIntegral))
            end
        end,
    })
end

function This:onPickupJifenBox( id )
    if self.m_id == id then
        self:refresh()
    end
end