-- @Author: superm
-- @Date:   2019-09-16 11:54:01
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-14 19:48:53
-- @File Name: LegionIntegralPanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local LegionIntegralItemView = require("Map.LegionIntegralItemView")
local AchieveManager = require("Achieve.AchieveManager")
local ArgConfig = require("Metadatas.ArgConfig")
local GameUtil = require("Util.GameUtil")

function This:initData()
	self.m_map = require("Map.GameMapManager"):getMap(GameMapType.kLegionMap)
	self.m_family = self.m_map:getMyLegion()

	local taskConfig = ArgConfig["legion_war_special_task_arg"]
	self.m_targetScore, self.m_peopleNum = unpack(taskConfig.arg)
end

function This:initNode(info)
	local csNode = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionBuildIntegral.csb")
	self.m_csNode = csNode
	self:addChild(csNode)
	csNode:setToCenter()

	local rewardsv = csNode.reward
    rewardsv:setDirection("horizontal")
    rewardsv.m_direction = "right"
    rewardsv:setSwallowTouch(false)

    local itemList = csNode.list
    itemList:setDirection("horizontal")
    itemList.m_direction = "right"
    itemList:setSwallowTouch(false)

	local rewardConfig = ArgConfig["legion_war_special_task_reward"]
    GameUtil:resetScoolItemNode(rewardsv,rewardConfig.arg,nil,nil,nil,3)
	csNode.renwu:setString(string.format(Serialize["legion_integeral_special_task"], self.m_targetScore, self.m_peopleNum))
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.close:setTouchEnded(function()
		self:close()
	end)

	csNode.button:setTouchEnded(function()
		local hasPickUp = self.m_map:IsJifenSpecialBoxRewarded()
		if not hasPickUp then
			local finishNum = self.m_family:getRoleCount(function( role ) return role:getJiFen() >= self.m_targetScore end)
			local canPickUp = finishNum >= self.m_peopleNum 
		    if canPickUp then
		    	self.m_map:onPickupJifenSpecialBoxRequest()
		    -- else
		    -- 	g_scene:showTips(string.format(, self.m_targetScore))
		    end
		end
	end)

	self.m_csNode.look:setTouchEnded(function()
		-- 贡献界面
		g_scene:openDialog("Map.LegionContributionPanel")
	end)

	if csNode.wenhao then
		csNode.wenhao:setTouchEnded(function()
			g_scene:openDialog("UI.HelpTips", {text = Serialize["legion_integeral_help"], node = csNode.wenhao})
		end)
	end
end

function This:onEnter()
	super.onEnter(self)
	self.m_map:addObserver(self)
	self:refresh()
end

function This:onExit()
	super.onExit(self)
	self.m_map:removeObserver(self)
end

function This:refresh()
	self:refreshPersonalInfoNode()
	self:refreshSpecialTask()
end

function This:refreshPersonalInfoNode()
	local csNode = self.m_csNode
	csNode.jifenjuntuan:setString(self.m_family:getJiFen())
    csNode.jifenme:setString(self.m_map:getJiFen())
    csNode.paihang:setString(self.m_map:getJiFenRank())

    local itemList = csNode.list
    itemList:removeAllItems()

    local boxList = self.m_map:getJifenBoxList()
    for _, box in ipairs(boxList) do
        local item = LegionIntegralItemView:create(box)
        local w, h = item:getContentSize()
        item:setAnchorPoint(0.5, 0.5)
        itemList:pushBackCustomItem(item, w, h, 0.5)
    end
end

function This:refreshSpecialTask()
	local csNode = self.m_csNode
	local finishNum = self.m_family:getRoleCount(function( role ) return role:getJiFen() >= self.m_targetScore end)
	local node = csNode.jindutiao
	node.num:setString(finishNum .. "/" .. self.m_peopleNum)
	node.jindutiao:setPercent(finishNum / self.m_peopleNum * 100)

	local pkbtn = csNode.button
	local hasPickUp = self.m_map:IsJifenSpecialBoxRewarded()
	pkbtn:setVisible(not hasPickUp)
	if not hasPickUp then
		local canPickUp = finishNum >= self.m_peopleNum 
	    if canPickUp then
	        pkbtn:clearShader()
	    else
	        pkbtn:setGrayShader()
	    end
	end
end

function This:onPickupJifenSpecialBox()
    self:refresh()
end

function This:onGetIntegral(score)
    self:refresh()
end

function This:onGeRenInfoChanged()
    self:refresh()
end

function This:onPickupJifenBox()
	self:refresh()
end

