﻿-- @Author: ZYY
-- @Date:   2018-08-09 11:07:57
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-15 11:31:48
-- @File Name: LegionMarchListItemView.lua
-- @Desc: 

local This , super = class(...,"NodeUI")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

This.fileName = "LegionScene/NodeUI/LegionMarchingListItem.csb"

function This:initNode(info)
	self.m_march = info.march
	self.m_map = info.map

	self.state:setImage(self.m_march:getMarchFlag())
	local fromCity = self.m_map:getCity(self.m_march:getFromCity())
	local toCity = self.m_map:getCity(self.m_march:getToCity())
	self.start:setString(fromCity:getName())
	self.target:setString(toCity:getName())
	
	local transType = self.m_march:getTransType()
	local troop = self.m_march:getTroop()
	local cardList = troop:getCardList()
	for _, card in ipairs(cardList) do
		local pos = card:getPos()
		local quality = card:getQuality()
		self["hero" .. pos]:setColor(QualityColor[quality].r,QualityColor[quality].g,QualityColor[quality].b)
		self["hero" .. pos]:setString(card:getName())
		if transType == LegionTransType.kHp then
			self["hp" .. pos]:setString(card:getHp())
		else
			self["hp" .. pos]:setString(card:getTransNum())
		end
	end

	local leftTime = self.m_march:getLeftTime()
	self.timecost:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
	self.m_escape = 0
end

function This:refresh()
	local toCity = self.m_map:getCity(self.m_march:getToCity())
	local addSpeedTime = self.m_march:getAddSpeedTime()
	local maxTime = self.m_map:getMaxMarchAddSpeedTime()

	-- self.accelerate:setVisible(not toCity:isFighting())
	self.gotowar:setVisible(false)
	self.accelerate.text:setString(string.format(Serialize[61782], maxTime - addSpeedTime, maxTime))
	if addSpeedTime >= maxTime then
		self.accelerate:setGrayShader()
	end
end

function This:onEnter()
	super.onEnter(self)
	self:refresh()
	self.m_map:addObserver(self)
	self:scheduleUpdate()
end

function This:onExit()
	super.onExit(self)
	self.m_map:removeObserver(self)
end

function This:getModel()
	return self.m_march
end

function This:initTouchEvent()
	self.gotowar:setTouchEnded(function()
		
	end)
	self.accelerate:setTouchEnded(function()
		local addSpeedTime = self.m_march:getAddSpeedTime()
		local maxTime = self.m_map:getMaxMarchAddSpeedTime()
		if addSpeedTime < maxTime then
			local cost = self.m_map:getMarchAddSpeedCost()
			GamePlayer:costDiamond({
				cost = cost,
				okHandler = function()
					self.m_map:onMarchAddSpeedRequest(self.m_march:getId())
				end,
				desc = Serialize[61783]
			})
		end
	end)
end

--------------------------------------------------
function This:onCityInfoChanged(city)
	if self.m_march:getToCity() == city:getId() then
		self:refresh()
	end
end

function This:onMarchAddSpeed(march)
	if self.m_march == march then
		self:refresh()
	end
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return 
	end
	self.m_escape = self.m_escape - 1
	local leftTime = self.m_march:getLeftTime()
	self.timecost:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
end