-- @Author: superm
-- @Date:   2018-11-13 09:46:38
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-13 16:59:43
-- @File Name: LegionMiniMapPanel.lua
-- @Description:

local This , super = class(...,"Dialog")
-- local CeilsConfig = require("Map.CeilsConfig")
local LegionMiniMapItem = require("Map.LegionMiniMapItem")
local LegionMinMapNode = require("Map.LegionMinMapNode")

-- local ColorList = 
-- {
-- 	cc.Color.WHITE,
-- 	cc.Color.YELLOW,
-- 	cc.Color.GREEN,
-- 	cc.Color.BLUE,
-- 	cc.Color.RED,
-- 	cc.Color.MAGENTA,
-- 	cc.Color.BLACK,
-- 	cc.Color.ORANGE,
-- 	cc.Color.GRAY,
-- 	cc.ColorList.New14,
-- 	cc.ColorList.New12,
-- 	cc.ColorList.New11,
-- 	cc.ColorList.New10,
-- 	cc.ColorList.New9,
-- 	cc.ColorList.New4,
-- 	cc.ColorList.QQ6,
-- 	cc.ColorList.QQ5,
-- 	cc.ColorList.QQ3,
-- 	cc.ColorList.PetPhase2,
-- 	cc.ColorList.PetPhase1,
-- 	cc.ColorList.PetPhase5
-- }

local kMapRadioX, kMapRadioY = 1.3, 1.3

function This:initData(info)
	self.m_map = info.map
	self.m_scene = g_scene
	kMapRadioX = info.mapRadioX or kMapRadioX
	kMapRadioY = info.mapRadioY or kMapRadioY
	if self.m_map:getMapType() <= GameMapType.kEmperorMap then
		filename = "LegionScene/NodeUI/LegionWorldMap.csb"
	else
		filename = "RescueEmperor/RescueEmperorWorldMap.csb"
	end
	print(self.m_map:getMapType())
	self.m_csFileName = filename
end

function This:initNode()
	self.m_csNode = cc.Node:createFromCSFile(self.m_csFileName)
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self.m_escape = 0
	self.m_cityList = {}
	
	local info = {
		map = self.m_map, 
		scaleRate = 1,
		mapRadioX = kMapRadioX,
		mapRadioY = kMapRadioY,
		mapNode = self.m_csNode.map, 
	}
	local liangDianNode = LegionMinMapNode:create(info)
	self:addChild(liangDianNode)
	self.m_liangDianNode = liangDianNode
	self:initCeils()
end

function This:initCeils()
	local w , h = self.m_csNode.map:getContentSize()
	local dx , dy = 20 * kMapRadioX, 12 * kMapRadioY
	local f = dy / dx
	local nx , ny = math.ceil(w / dx), math.ceil(h / dy)

	self.m_nx = nx
	self.m_ny = ny
	self.m_dx = dx
	self.m_dy = dy
	self.m_top = top
	self.m_f = f
	self.m_vy = cc.Vec(self.m_dx,self.m_dy):normalize()
	self.m_vx = cc.Vec(-self.m_dx,self.m_dy):normalize()
	
	local liangDianNode = self.m_liangDianNode
	liangDianNode:refresh()
	local cityNums = liangDianNode:getLegionCityNums()
	local index = 1
	local legions = self.m_map:getAllLegions()
	for i , legion in pairs(legions) do
		local legionId = legion:getId()
		local itemNode = LegionMiniMapItem:create({
			legion = legion,
			color = liangDianNode:getLegionColor(legionId),
			num = cityNums[legionId]
		})
		self.m_csNode.legion["legion"..index]:addChild(itemNode)
		index = index + 1
		if index > 4 then
			break
		end
	end

	self.m_rectW , self.m_rectH = 150,100
	self.m_rectNode = cc.DrawNode:create()
	self.m_csNode.map:addChild(self.m_rectNode)
	self.m_rectNode:drawRect(cc.Rect(0,0,self.m_rectW,self.m_rectH),cc.Color(255,255,0,255))
	
	local dx , dy = g_scene:getViewportCenterRatio()
	x = w * dx - self.m_rectW/2
	y = h * dy - self.m_rectH/2
	x = math.min(math.max(0,x),w - self.m_rectW)
	y = math.min(math.max(0,y),h - self.m_rectH)
	self.m_rectNode:setPosition(x,y)

	local drawNode = cc.DrawNode:create()
	x , y = self.m_csNode.map:getPosition()
	w , h = self.m_csNode.map:getContentSize()
	local scale = self.m_csNode:getScale()
	drawNode:drawSolidRect(cc.Rect(0, 0 , w, h))

	local clippingNode = cc.ClippingNode:create(drawNode)
	clippingNode:setAnchorPoint(0,0)
	clippingNode:setPosition(x,y)
	self.m_csNode.map:setPosition(0,0)
	self.m_csNode.map:switchParent(clippingNode)
	self.m_csNode:addChild(clippingNode)
end

function This:createCeil(nx,ny,color)
	return self.m_liangDianNode:createCeil()
end

function This:addCeils(nx,ny,cityId)
	local key = nx * 1000 + ny
	if not self.m_ceils[key] then
		local config = self.m_cityInfo[cityId]
		local layer = self:createCeil(nx,ny,config.color)
		self.m_ceils[key] = {node = layer, color = config.color, cityId = cityId}
		table.insert(config.ceilList, key)
	end
end

function This:removeCeils(nx,ny)
	local key = nx * 1000 + ny
	if self.m_ceils[key] then
		local sprite = self.m_ceils[key].node
		sprite:removeFromParent()
		local cityId = self.m_ceils[key].cityId
		for i , id in ipairs(self.m_cityInfo[cityId].ceilList) do
			if id == key then
				table.remove(self.m_cityInfo[cityId].ceilList,i)
				break
			end
		end
		self.m_ceils[key] = nil
	end
end

function This:getCeilsPosition(x,y)
	local vy = self.m_vy
	local vx = self.m_vx
	 
	local v = cc.Vec(x,y)
	local l = v:length()
	local nv = v:getNormalize()
	local v1 = vy * nv:dot(vy) * l
	local flag = nv:dot(self.m_vx) > self.m_vy:dot(self.m_vx) and 1 or -1
	local v2 = v1:getNormalize() * (v - v1):length() * (( 1 - self.m_f * self.m_f) / (2 * self.m_f)) * flag
			
	local v3 = v1 + v2
	local v4 = v - v3
	 
	local b = cc.Vec(self.m_dx, self.m_dy):length() / 2
	local b1 = v3:getNormalize():dot(vy) * v3:length()
	local b2 = v4:getNormalize():dot(vx) * v4:length()
	local ny = math.floor( b1 / b)
	local nx = math.floor( b2 / b)
	return nx , ny
end

function This:onTouchMapLayer(x,y)
	local nx , ny = self:getCeilsPosition(x,y)
	local key = nx * 1000 + ny
	print(string.format("(%d,%d)-->(%d,%d)",x,y,nx,ny))
	if not self.m_ceils[key] and self.m_isSelected then	
		self:addCeils(nx,ny,self.m_curCityId)
	elseif self.m_ceils[key] and self.m_isDelete then
		self:removeCeils(nx,ny)
	end
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	local vy = cc.Vec(1, self.m_f):getNormalize()
	local vx = cc.Vec(-1, self.m_f):getNormalize()

	self.m_csNode.map:setTouch({
		onEnded = function(_,p)
			local w , h = self.m_csNode.map:getContentSize()
			local rx , ry = p.x / w , p.y/h
			g_scene:focusePoint(rx,ry)
			self:focusePoint(p.x,p.y)
		end,
		isScale = false,
		isAlpha = false
	})
end

function This:focusePoint(x,y)
	x , y = self.m_csNode.map:convertToNodeSpace(x,y)
	local mapW , mapH = self.m_csNode.map:getContentSize()
	x = x- self.m_rectW / 2
	y = y - self.m_rectH / 2
	x = math.min(math.max(0,x),mapW - self.m_rectW)
	y = math.min(math.max(0,y),mapH - self.m_rectH)
	self.m_rectNode:setPosition(x,y)
	g_scene:focusePoint((x + self.m_rectW / 2)/mapW,(y + self.m_rectH / 2)/mapH)
end


function This:update(dt)
	super.update(self,dt)
	--[[self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return 
	end
	self.m_escape = self.m_escape - 1]]
	--[[if self.m_index <= #self.m_cityList then
		for i = self.m_index , self.m_index + 4 do
			if i <= #self.m_cityList then
				local cityId = self.m_cityList[i]:getId()
				local config = CeilsConfig[cityId]
				for j , ceilKey in ipairs(config.ceilList) do
					local nx = math.floor(ceilKey / 1000)
					local ny = ceilKey % 1000
					local sprite = self:createCeil(nx,ny,config.color)
				end
			end
		end
		self.m_index = self.m_index + 4
	end]]
end

--endregion
