-- @Author: superm
-- @Date:   2019-09-12 10:29:23
-- @Last Modified by:   superm
-- @Last Modified time: 2019-11-20 10:49:11
-- @File Name: LegionPersonalRankingController.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local LegionWarPrestigeIntegralConfig = require("Metadatas.LegionWarPrestigeIntegralConfig")

function This:init(node)
	super.init(self)
    self.m_itemListCache = {}
    self.m_map = require("Map.GameMapManager"):getMap(GameMapType.kLegionMap)

    self.m_mapId = self.m_map:getMapType() *10000 + self.m_map:getMapLevel() *100

    self.m_csNode = node
    self:initNode()
end

function This:initNode()
    self:setMyRankData()
end

function This:onEnter()
    self.m_map:addObserver(self)

    self.m_csNode.mine.xiangqing:setTouch({
        onEnded = function()
            self:onClickInfoButton(self.m_csNode.mine.xiangqing)
        end,
        isDynamic = true
    })

end

function This:onExit()
	self:clearAllItems()
    self.m_map:removeObserver(self)
end

function This:setRankingData( data )
	local scrollview = self.m_csNode.scrollview
    scrollview:removeAllItems()

    local function updateItemView(node, index, info)
    	node.num1:setString(index)
    	node.name:setString(info.role_name)
    	node.num2:setString(info.value)
    end
    for i, rankInfo in ipairs(data) do
        if rankInfo.value > 0 then
        	local item = self:getItemView(index)
        	updateItemView(item, i, rankInfo)
        	local w , h = item.bg:getContentSize()
            scrollview:pushNode(item, w, h-5)
        end
    end
end

function This:setMyRankData( info )
    local node = self.m_csNode.mine
	if info and info.value > 0 then
		node.num1:setVisible(true)
		node.name:setVisible(true)
		node.num2:setVisible(true)
		node.num4:setVisible(false)
		node.num5:setVisible(false)
		node.num1:setString(info.index)
		node.name:setString(info.role_name)
		node.num2:setString(info.value)
	else
		node.num1:setVisible(false)
		node.name:setVisible(false)
		node.num2:setVisible(false)
		node.num4:setVisible(true)
		node.num5:setVisible(true)
		node.num5:setString(GamePlayer:getRoleName())
	end
    node.xiangqing:setVisible(true)
end

function This:getItemView(index)
    local item = self.m_itemListCache[index]
    if not item then
		item = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionBuildMainRankItem.csb")
		table.insert(self.m_itemListCache, item)
		item:retain()
	end
	return item
end

function This:clearAllItems()
    for _, v in ipairs( self.m_itemListCache ) do
    	v:release()
    end
    self.m_itemListCache = {}
end

function This:onGetIntegralRankInfo(data, mine)
    if data then
        self:setRankingData( data )
        self:setMyRankData(mine)
    end
end

function This:onClickInfoButton(node)
    local detailList = self.m_map:getJifenDetail()
    local details = {}
    for _, info in ipairs( detailList ) do
        details[info.type] = info.score
    end

    local ret = {}
    local desc = {
        [LegionIntegralType.kLegionTransport] = Serialize[61791],
        [LegionIntegralType.kLegionBuild] = Serialize[61792],
        [LegionIntegralType.kLegionKill] = Serialize[61793],
        [LegionIntegralType.kLegionDamage] = Serialize[61794],
    }
    for _, itype in pairs( LegionIntegralType ) do
        local value = details[itype] or 0
        local limit = LegionWarPrestigeIntegralConfig[self.m_mapId + itype].integral_max
        table.insert(ret, string.format(desc[itype], value, limit))
    end
    g_scene:openDialog("UI.HelpTips", {text = table.concat(ret, '\n'), node = node})
end