---
---
--- Created by aid.
--- DateTime: 2018/11/19 22:05
---
local This , super = class(...,"cc.Node")
local LegionMapManager = require("Map.GameMapManager"):getMap(GameMapType.kLegionMap)

function This:init(info)
    self.m_csNode = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionRanklittle.csb")
    self:addChild(self.m_csNode)

    self:initNode()
end

function This:initNode()
    self.m_csNode.tips:setTouch({
        onEnded = function()
            g_scene:openDialog("UI.HelpTips", {text = Serialize[61796], node = self.m_csNode.tips})
        end,
        isDynamic = true
    })
end

function This:onEnter()
    super.onEnter(self)
    LegionMapManager:addObserver(self)
end

function This:onExit()
    LegionMapManager:removeObserver(self)
    super.onExit(self)
end

function This:setLegionRankingData(groupList)
    local quittedLegionList = LegionMapManager:getQuitLegionList()

    for index = 1, 4 do
        local group = groupList[index]
        local quittedLegion = quittedLegionList[index]

        local rankingNumberView = self.m_csNode["num"..index] -- 名次
        local legionNameView = self.m_csNode["top"..index] -- 军团名
        local scoreView = self.m_csNode["jifen"..index] -- 积分

        rankingNumberView:setVisible(true)
        legionNameView:setVisible(true)
        scoreView:setVisible(true)

        if group then
            rankingNumberView:setString(""..index)
            legionNameView:setString(group.m_name)
            scoreView:setString(tostring(group.m_jifen))
        elseif quittedLegion then
            rankingNumberView:setString(""..index)
            legionNameView:setString(quittedLegion.group_name)
            scoreView:setString(Serialize.legion_battle_eliminated)
        else
            rankingNumberView:setVisible(false)
            legionNameView:setVisible(false)
            scoreView:setVisible(false)
        end
    end
end

function This:refreshLegionRanking()
    local groupList = {}
    for _, group in pairs(LegionMapManager:getAllLegions()) do
        table.insert(groupList, group)
    end
    table.sort(groupList, function(a, b)
        return a.m_jifen > b.m_jifen
    end)
    self:setLegionRankingData(groupList)
end

function This:onGetLegionMapInfo()
    self:refreshLegionRanking()
end

-- 未淘汰的军团积分发生改变
function This:onLegionScoreChanged(legionId, newScore)
    self:refreshLegionRanking()
end

-- 有军团被淘汰出局
function This:onLegionExitMapAfter()
    self:refreshLegionRanking()
end