-- @Author: superm
-- @Date:   2019-09-26 16:15:06
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-19 18:01:54
-- @File Name: LegionStateEntry.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")
local LegionCityConfig = require("Metadatas.LegionCityConfig")
local LegionWarPlanConfig = require("Metadatas.LegionWarPlanConfig")


local calcEntryId = function( info )
    local etype = info.etype or LegionStateEntryType.kStrategy
    local stype = info.type or 0
    local cityId = info.city_id or 0
    return string.format("%d-%d-%d", etype, stype , cityId)
end

function This:init(info)
    super.init(self)
    self.m_id = calcEntryId(info)
    self:setContent(info)
end

function This:updateInfo( info )
    local id = calcEntryId(info)
    if self.m_id == id then
        local preValue = self.m_holdEndTime
    	self:setContent(info)
        return preValue ~= self.m_holdEndTime
    elseif self.m_stateType == info.type then
    	self.m_id = id
    	self:setContent(info)
        return true
    end
end

function This:setContent(info)
	self.m_sortTime = GamePlayer:getServerTime()
    self.m_entryType = info.etype or LegionStateEntryType.kStrategy
	self.m_stateType = info.type
	self.m_cityId = info.city_id
	self.m_holdEndTime = info.hold_endtime
	self.m_effectTimes = info.effect_times
	self.m_legionId = info.group_id
end

function This:getId()
    return self.m_id
end

function This:getType()
    return self.m_stateType
end

function This:getLegionId()
    return self.m_legionId
end

function This:getCityId()
    return self.m_cityId
end

function This:getCityName()
    if self.m_cityId > 0 then
        return LegionCityConfig[self:getCityId()].city_name
    end
    return ""
end

function This:getMaxHoldTime()
    if self.m_entryType == LegionStateEntryType.kSeizedCity then
        return 1
    end
    return LegionWarPlanConfig[self:getType()].hold_time
end

function This:getLeftHoldTime()
    if self.m_entryType == LegionStateEntryType.kSeizedCity then
        return 1
    end
	local curTime = GamePlayer:getServerTime()
	return math.max(self.m_holdEndTime - curTime, 0)
end

function This:getIcon()
	return ItemUtil.getIconImage(self.m_stateType)
end

function This:getName()
    if self.m_entryType == LegionStateEntryType.kSeizedCity then
        return Serialize[61806]
    end
    return ItemUtil.getName(self.m_stateType)
end

function This:getSortTime()
    return self.m_sortTime
end

function This:link(t)
    super.link(self, t)
    print("LegionStateEntry link:" .. self.index)
end

function This:unlink()
    super.link(self)
    print("LegionStateEntry unlink:" .. self.index)
end

function This:setView( view )
    self.m_view = view
end

function This:getView()
    return self.m_view
end