-- @Author: superm
-- @Date:   2019-10-15 13:01:50
-- @Last Modified by:   superm
-- @Last Modified time: 2019-11-02 11:47:21
-- @File Name: LegionStateEntryController.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local UpdateList = require("Base.UpdateList")
local LegionStateEntry = require("Map.LegionStateEntry")

LegionStateEntryType = {
	kSeizedCity = 1,
	kStrategy = 2,
}

function This:init(map)
	super.init(self)
	self.m_map = map

	self.m_entryCount = 6
	self.m_list = UpdateList:create()

	self.m_junces = {}
	self.m_resident = {}
	for _ , id in pairs(LegionStrategyType) do
		self.m_junces[id] = {}
	end
	self.m_upEscape = 0
end

function This:addOtherEntry( etype )
    local switch = {
        [LegionStateEntryType.kSeizedCity] = function()
            return {
            	etype = LegionStateEntryType.kSeizedCity,
            	group_id = self.m_map:getMyLegionId(),
            	type = 51009,
            	city_id = 149,
            }
        end,
    }
    if not self.m_resident[etype] then
    	self.m_resident[etype] = LegionStateEntry:create(switch[etype]())
		self:notifyObservers("onLegionStateEntryChanged")
    end
end

function This:removeOtherEntry( etype )
	if self.m_resident[etype] then
	    self.m_resident[etype] = nil
		self:notifyObservers("onLegionStateEntryChanged")
	end
end

function This:addEntry(info)
	local legionId = self.m_map:getMyLegionId()

	local groupId, cityId, jcType
	if info then
		groupId = info.group_id
		cityId = info.city_id
		jcType = info.type 
	end

	local container = self.m_list
	if groupId ~= legionId then
		if cityId <= 0 then
			return
		end

		local city = self.m_map:getCity(cityId)
		local cityOwnerId = city:getLegionId()
		if cityOwnerId ~= legionId then
			return
		end
	end

	local entry = self.m_junces[jcType][cityId]
	if not entry then
		entry = LegionStateEntry:create(info)
		self.m_junces[jcType][cityId] = entry
		container:addNode(entry)
	else
		local replaceAction = entry:updateInfo(info)
		if replaceAction then
			if container:hasNode(entry) then
				container:removeNode(entry)
			end
			container:addNode(entry)
		else
			entry = nil
		end
	end
	if entry then
		if not self.m_isUpdating then
			cc.Application:addToUpdateList(self)
			self.m_isUpdating = true
		end
		self:notifyObservers("onLegionStateEntryChanged")
		return true
	end
end

function This:removeEntry( info )
	local jcType, cityId
	if info then
		cityId = info.cityId
		jcType = info.type 
	end
	local entry = self.m_junces[jcType][cityId]
	if entry then
		self.m_list:removeNode(entry)
		self.m_junces[jcType][cityId] = nil
		self:notifyObservers("onLegionStateEntryChanged")
		return true
	end
end

function This:getEntryList()
    local ret = {}
    local count = 1
    for _, entry in pairs(self.m_resident) do
		ret[count] = entry
		count = count +1
    end

	local entry = self.m_list:getBackNode()
    for index = count, 6 do
        if entry then
    		ret[index] = entry 
            entry = self.m_list:getPreNode(entry)
        else
    		ret[index] = false 
        end
    end
    return ret
end

function This:cleanup()
	if self.m_isUpdating then
		self.m_isUpdating = false
		cc.Application:removeFromUpdateList(self)
	end

	self.m_list:clear()
	for _ , id in pairs(LegionStrategyType) do
		self.m_junces[id] = {}
	end
end

function This:removeInvalidEntries()
	local list = self:getEntryList()
	local removeList = {}
    for _, entry in ipairs( list ) do
    	if entry and entry:getLeftHoldTime() <= 0 then
    		table.insert(removeList, entry)
    	end
    end

    local head = next(removeList)
    if head then
	    local entries = self.m_list
	    for _, v in ipairs( removeList ) do
			entries:removeNode(v)
	    end
	    
		self:notifyObservers("onLegionStateEntryChanged")
	end

	if self.m_list:getSize() <= 0 and self.m_isUpdating then
		self.m_isUpdating = false
		cc.Application:removeFromUpdateList(self)
	end
end

function This:update( dt )
	if not self.m_isUpdating then
		return
	end
	self.m_upEscape = self.m_upEscape + dt
	if self.m_upEscape >= 1.0 then
		self.m_upEscape = self.m_upEscape - 1
		self:removeInvalidEntries()
	end
end

function This:isJunceEffectInLegion( jcType )
	local entries = self.m_junces[jcType]
	if entries then
		for cityId, entry in pairs(entries) do
	    	if entry and entry:getLeftHoldTime() > 0 then
	    		return true
	    	end
		end
	end
end
