-- @Author: superm
-- @Date:   2019-09-26 20:18:20
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-18 10:36:14
-- @File Name: LegionStateEntryView.lua
-- @Description: 

local This , super = class(..., "cc.Node")

function This:createByName(info)
    local parent, name = info.node, info.name
    local snode = parent[name]
    if not snode then
        return 
    end

    local snodeId = snode[0]
    local ret = cc.registerObject(snodeId, self.getAutoChildClass(snodeId, self._MNAME), {[0]=snodeId})
    if ret:getName() == "" then
        ret:setName(self._NAME)
    end
    parent[name] = ret
    if ret.init then
        ret:init(info)
    end
    return ret
end

function This:init(info)
    self.m_entry = info.entry
    self.m_escape = 0
    self.m_updating = false
    self:initNode()
end

function This:onEnter()
    super.onEnter(self)
    self:initTouchEvent()
    self:scheduleUpdate()
end

function This:initNode()
    self:refresh()
end

function This:onExit()
    self:unscheduleUpdate()
    super.onExit(self)
end

function This:initTouchEvent()
	self.bg:setTouch({
        isDynamic = true,
        onEnded = function()
            local cityId = self.m_entry:getCityId()
            if cityId and cityId > 0 then
                g_scene:focuseToCity({cityId = cityId})
            end
        end
    })
end

function This:setEntry(entry)
    if entry ~= self.m_entry then
        self.m_entry = entry
        self:refresh()
    end
end

function This:refresh()
    local csNode = self
    local entry = self.m_entry
    if entry then
        csNode:setVisible(true)
        csNode.item:setImage(entry:getIcon())
        local cityId = self.m_entry:getCityId()
        if cityId and cityId > 0 then
            csNode.citybg:setVisible(true)
            csNode.title:setVisible(true)
            csNode.title:setString(entry:getCityName())
        else
            csNode.citybg:setVisible(false)
            csNode.title:setVisible(false)
        end

        self:refreshTime()
    else
        self.m_updating = false
        csNode:setVisible(false)
    end
end

function This:refreshTime()
    local node = self.progress
    local leftTime = self.m_entry:getLeftHoldTime()
    if leftTime > 0 then
        self.m_updating = true
        if not self.m_progressTimer then
            self.m_progressTimer =  cc.ProgressTimer:create(node.cooldowncd)
            node:addChild(self.m_progressTimer)
        end
        local maxTime = self.m_entry:getMaxHoldTime()
        self.m_progressTimer:setPercent(100 - (leftTime / maxTime * 100))
    else
        self.m_updating = false
        self:setVisible(false)
    end
end

function This:update(dt)
    if not self.m_updating then
        return
    end

    self.m_escape = self.m_escape + dt
    if self.m_escape < 1 then
        return
    end
    self.m_escape = self.m_escape - 1
    self:refreshTime()
end

