-- @Author: superm
-- @Date:   2019-09-27 16:43:16
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-16 17:28:01
-- @File Name: LegionStateUI.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local LegionStateEntryView = require("Map.LegionStateEntryView")

function This:init(node)
    super.init(self)
    self.m_csNode = node
    self:initNode()
end

function This:initNode()
    local container, posList, csNode = {}, {}, self.m_csNode
    for i = 1, 6 do
        local name = "item" .. i
        local node = LegionStateEntryView:createByName({node = csNode, name = name})
    	table.insert(container, node)
        table.insert(posList, node:getPositionX())
    	node:setVisible(false)
    end
    self.m_itemList = container
    self.m_posList = posList
    self.m_map = require("Map.GameMapManager"):getMap(GameMapType.kLegionMap)
    self.m_controller = self.m_map:getStateEntryController()
    self.m_entryIndices = {}

    local indices = self.m_entryIndices
    local entryList = self.m_map:getStateEntryList()
    for index, entry in ipairs( entryList ) do
        local view = self.m_itemList[index]
        view:setEntry(entry)
        if entry then
            indices[entry:getId()] = index
        end
    end
end

function This:onEnter()
    self.m_controller:addObserver(self)
end

function This:onExit()
    self.m_controller:removeObserver(self)
end

function This:refresh()
    local preIndices = self.m_entryIndices
    local indices = {}
    self.m_entryIndices = indices

    local preItemList = self.m_itemList
    local itemList = {}
    self.m_itemList = itemList

    local entryList = self.m_map:getStateEntryList()
    for index, entry in ipairs( entryList ) do
        itemList[index] = false
        if entry then
            local entryId = entry:getId()
            indices[entryId] = index
            local indice = preIndices[entryId]
            if indice then
                itemList[index] = preItemList[indice]
                preItemList[indice] = false
            end
        end        
    end

    local changeItemList = {}
    for i, view in ipairs( preItemList ) do
        if view then
            table.insert(changeItemList, view)
        end
    end

    local counter = 1
    for i, view in ipairs( itemList ) do
        if not view then
            view = changeItemList[counter]
            view:setEntry(entryList[i])
            itemList[i] = view
            counter = counter + 1
        end
        if view then
            view:setPositionX(self.m_posList[i])
        end
    end
end

function This:onLegionStateEntryChanged()
    self:refresh()
end




