--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local LegionWarPlanConfig = require("Metadatas.LegionWarPlanConfig")
local GamePlayer = require("Player.GamePlayer")
local MapPath = require("Map.MapPath")
local ArgConfig = require("Metadatas.ArgConfig")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local FamilyManager = require("Family.FamilyManager")

function This:init(info)
	super.init(self, info)
	self.m_map = info.map
	self.m_type = info.type
	self.m_config = LegionWarPlanConfig[self.m_type]
	self.m_cdEndTime = 0
	self.m_useCount = 0
	self.m_count = 0
	self.m_entrys = {}
end

function This:getId()
	return self.m_type
end

function This:isUseTimeArrived()
	local curTime = GamePlayer:getServerTime()
	local startTime = self.m_map:getStartTime() + self.m_config.can_use_time
	return curTime >= startTime
end

function This:getName()
	return self.m_config.name
end

function This:getCost()
	return self.m_config.cost
end

function This:getIcon()
	return ItemUtil.getIconImage(self.m_type)
end

function This:getHoldTime()
	return self.m_config.hold_time
end

function This:canShowEffect()
    return self.m_config.is_effect == 1
end

function This:getLeftHoldTime(cityId)
	local info = self.m_entrys[cityId]
	if info then
		local curTime = GamePlayer:getServerTime()
		return math.max(info.holdEndTime - curTime,0)
	end
	return 0
end

function This:getUseCDTime()
	return self.m_config.cd_time
end

function This:getLeftUseCDTime()
	local curTime = GamePlayer:getServerTime()
	return math.max(self.m_cdEndTime - curTime,0)
end

function This:getCount()
    return self.m_count or 0
end

function This:getUseCount()
	return self.m_useCount or 0
end

function This:getUseMaxTime()
	return self.m_config.max_time
end

function This:isUsedOnCity(cityId)
	if self.m_entrys[cityId] then
		local curTime = GamePlayer:getServerTime()
		return curTime < self.m_entrys[cityId].holdEndTime
	end
	return false
end

function This:isUsedOnCityWithMyLegion(cityId)
	if self.m_entrys[cityId] and self.m_entrys[cityId].legionId == self.m_map:getMyLegionId() then
		local curTime = GamePlayer:getServerTime()
		return curTime < self.m_entrys[cityId].holdEndTime
	end
	return true
end

function This:getUsedLegionId(cityId)
	if self.m_entrys[cityId] then
		return self.m_entrys[cityId].legionId
	end
	return 0
end

function This:getUsedLegionIdIsOur(cityId)
	local myLegionId = FamilyManager:getMyFamilyId()
	if myLegionId then
		local temLegion = self:getUsedLegionId(cityId)
		return temLegion == myLegionId
	end
	return false
end

function This:getClearCDCost()
	local curTime = GamePlayer:getServerTime()
	local leftTime = math.max(0, self.m_cdEndTime - curTime)
	return math.ceil(leftTime * ArgConfig["legion_war_clean_cd_coefficient"].arg[1])
end

function This:isPowerEnough()
	local role = self.m_map:getRole(GamePlayer:getPlayerID())
	local office = role:getLegionOffice()
	for i , of in ipairs(self.m_config.use_power) do
		if of == office then
			return true
		end
	end
	return false
end

function This:isCanUseOnCity(city)
	local curTime = GamePlayer:getServerTime()
	local startTime = self.m_map:getStartTime() + self.m_config.can_use_time
	if not self:isPowerEnough() then
		return false, Serialize[61811]
	end
	if curTime < startTime then
		return false, Serialize[61812]
	end
	
	if curTime < self.m_cdEndTime then
		return false, Serialize[61813]
	end
	if self.m_count <= 0 then
		return false, Serialize[61814]
	end
	-- if self.m_count <= 0 then
	-- 	return false, Serialize[61815]
	-- end

	if self.m_type == LegionStrategyType.kTuCheng then
		--[[local guardCount = city:getGuardTroopCount()
		local attackCount = city:getAttackTroopCount()
		local ret = guardCount >= self.m_config.arg[2] and attackCount >= self.m_config.arg[1]
		if not ret  then
			return false, Serialize[61816]
		end]]
		if not city:isInFighting() then
			return false, Serialize[61817]
		end
	elseif self.m_type == LegionStrategyType.kQiangXi then
		if city:isOur() then
			return false, Serialize[61818]
		end
		if city:isInProtected() then
			return false, Serialize[61819]
		end

		local path = MapPath:create()
		local cityList = path:getAroundCityList(self.m_map,city,self.m_config.arg[1],function(ct)
			return ct:isOur()
		end)
		local ret = #cityList > 0
		if not ret then
			return false, string.format(Serialize[61820],self.m_config.arg[1])
		end
	elseif self.m_type == LegionStrategyType.kKongChengJi then
		if city:isFighting() then
			return false, Serialize[61821]
		end
		if city:isInProtected() then
			return false, Serialize[61819]
		end

		if not city:isOur() then
			if self:isUsedOnCity(city:getId()) then
				return true
			end
			return false, Serialize[61822]
		end
	elseif self.m_type == LegionStrategyType.kPoZhen then
		if city:isInProtected() then
			return false, Serialize[61819]
		end

		if city:isOur() and city:isImprisoned() then
			local path = MapPath:create()
			local cityList = path:getAroundCityList(self.m_map,city,1,function(ct)
				return not ct:isOur()
			end)

			if #cityList <= 1 then
				return false, Serialize[61823]
			end
			return true
		end
		return false, Serialize[61822]
	elseif self.m_type == LegionStrategyType.kBaGua then
		if city:isInProtected() then
			return false, Serialize[61819]
		end
		
		if not city:isOur() then
			local path = MapPath:create()
			local cityList = path:getAroundCityList(self.m_map,city,1,function(ct)
				return not ct:isOur()
			end)

			if #cityList > 1 then
				return false, Serialize[61824]
			end
			return true
		end
		return false, Serialize[61822]
	elseif not city:isOur() then
		return false, Serialize[61825]
	end
	return true
end

function This:addEntry(info)
	local cityId = info.city_id
	if cityId and cityId > 0  then
		self.m_entrys[cityId] = {
			id = info.id,
			cityId = cityId,
			holdEndTime = info.hold_endtime,
			effectTimes = info.effect_times,
			legionId = info.group_id,
		}
	end
end

function This:removeEntry(info)
	local cityId = info.city_id
	if cityId and cityId > 0 and self.m_entrys[cityId] then
		self.m_entrys[cityId] = nil
	end
end

function This:updateInfo(info)
	self.m_cdEndTime = info.cd_endtime
	self.m_useCount = info.use_count
	self.m_count = info.count
end

--endregion
