-- @Author: ZYY
-- @Date:   2018-07-31 17:43:01
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-30 10:24:41
-- @File Name: LegionTroopsPanel.lua
-- @Desc: 部队总界面

local This, super = class(...,"Dialog")
local LegionTroopsView = require("Map.LegionTroopsView")
local LegionChengFangView = require("Map.LegionChengFangView")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

local TabName = 
{
	kTroop = 1,
	kDefense = 2
}

function This:initNode(info)
	self.m_map = info.map
	self.m_city = info.city
	self.m_escape = 0

    self.m_csNode = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionTroopMain.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)

	self:addTabButton(TabName.kTroop,self.m_csNode.tep1,function()
		self:selectTab(TabName.kTroop)
	end)

	self:addTabButton(TabName.kDefense,self.m_csNode.tep2,function()
		self:selectTab(TabName.kDefense)
	end)

	self:selectTabButton(TabName.kTroop)
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.defence.reducetime:setTouchEnded(function()
		print("消除时间...")
		local cost = self.m_city:getCostWithClearCfTime()
		if cost > 0 then
			GamePlayer:costDiamond({
				cost = cost,
				okHandler = function()
					self.m_map:onClearChengFangCreateTimeRequest(self.m_city:getId())
				end
			})
		else
			local quality = self.m_city:getCfQualityWithOnceKey()
			g_scene:showTips(string.format(Serialize[61872]))
		end
	end)
	self.m_csNode.defence.xunlian:setTouchEnded(function()
		local isCan = false
		for i , chengfang in ipairs(self.m_chengfangList) do
			if chengfang:getQuality() < self.m_city:getCfQualityWithOnceKey() then
				isCan = true
			end
		end

		if not isCan then
			g_scene:showTips(Serialize[61873])
			return
		end

		local cost = self.m_city:getCfTrainCostWithOnceKey()
		GamePlayer:costDiamond({
			cost = cost,
			okHandler = function()
				self.m_map:onTrainChengFangOnceKeyRequest(self.m_city:getId())
			end
		})
	end)
	self.m_csNode.defence.help:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips", {text = Serialize.legion_troop_help ,node = self.m_csNode.defence.help})
	end)
end

function This:onEnter()
	super.onEnter(self)
	self:scheduleUpdate()
	self.m_map:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_map:removeObserver(self)
end

function This:updateTroopList()
	if self.m_tab == TabName.kTroop then
		self.m_bianduiList = {}
		local bianduiList = self.m_map:getAllBianDui()
		local buildType = self.m_city:getBuildType()
		for i , biandui in ipairs(bianduiList) do
			if not biandui:isEmpty() then
				local state = biandui:getState()
				if (state == LegionTroopState.kDefensive and  biandui:getCityId() == self.m_city:getId() )then
					table.insert(self.m_bianduiList,biandui)
				elseif state == LegionTroopState.kIdle and (buildType == LegionBuildType.kBarracks or buildType == LegionBuildType.kMintage) then
					table.insert(self.m_bianduiList,biandui)
				elseif state == LegionTroopState.kAddHp and biandui:getCityId() == self.m_city:getId() then
					table.insert(self.m_bianduiList,biandui)
				end
			end
		end
	elseif self.m_tab == TabName.kDefense then
		self.m_chengfangList = {}
		local chengfangList = self.m_city:getChengFangList()
		for i , cf in ipairs(chengfangList) do
			table.insert(self.m_chengfangList,cf)
		end
	end
end

function This:selectTab(tab)
	if self.m_tab ~= tab then
		self.m_tab = tab
		self:updateTroopList()
		if self.m_tab == TabName.kTroop then
			self:switchToTroop()
		elseif self.m_tab == TabName.kDefense then
			self:switchToChengFang()
		end
	end
end

function This:switchToTroop()
	self.m_csNode.ScrollView:removeAllItems()
	for i , biandui in ipairs(self.m_bianduiList) do
		local view = LegionTroopsView:create({
			biandui = biandui, 
			city = self.m_city,
			map = self.m_map
		})
		self.m_csNode.ScrollView:pushBackCustomItem(view,630,150,0.5)
	end
	self.m_csNode.defence:setVisible(false)
end

function This:switchToChengFang()
	self.m_csNode.ScrollView:removeAllItems()
	for i , chengfang in ipairs(self.m_chengfangList) do
		local view = LegionChengFangView:create({
			map = self.m_map,
			city = self.m_city,
			chengfang = chengfang
		})
		self.m_csNode.ScrollView:pushNode(view,90,100,6)
	end
	local view = LegionChengFangView:create({
		map = self.m_map,
		city = self.m_city
	})
	self.m_csNode.ScrollView:pushNode(view,90,100,6)
	self.m_csNode.defence:setVisible(true)
	self:refreshTime()
end

function This:refreshChengFang()
	if self.m_tab == TabName.kDefense then
		local itemList = self.m_csNode.ScrollView:getItemList()
		for i , chengfang in ipairs(self.m_chengfangList) do
			local view = itemList[i]
			if view then
				view:setModel(chengfang)
			else
				view = LegionChengFangView:create({
					map = self.m_map,
					city = self.m_city,
					chengfang = chengfang
				})
				self.m_csNode.ScrollView:pushNode(view,90,100,6)
			end
		end
		if #itemList > #self.m_chengfangList then
			for i = #itemList, #self.m_chengfangList + 2 , -1 do
				self.m_csNode.ScrollView:removeItem(i)
			end
			local view = itemList[#itemList]
			view:setModel(nil)
		else
			local view = LegionChengFangView:create({
				map = self.m_map,
				city = self.m_city
			})
			self.m_csNode.ScrollView:pushNode(view,90,100,6)
		end
	end
end

function This:refresh()
	local itemList = self.m_csNode.ScrollView:getAllItem()
	for i , view in ipairs(itemList) do
		view:refresh()
	end
end

function This:refreshTime()
	local leftTime = self.m_city:getLeftCfNextTime()
	self.m_csNode.defence.time:setString(GameUtil:formatTime(Serialize[61874],leftTime))
	local count = self.m_city:getCFCount()
	local maxCount = self.m_city:getCFMaxCount()
	self.m_csNode.defence.time:setVisible(count < maxCount)
	self.m_csNode.defence.timebg:setVisible(count < maxCount)
	self.m_csNode.defence.reducetime:setVisible(count < maxCount)
	self.m_csNode.defence.tip:setVisible(count < maxCount)
end

function This:update(dt)
	super.update(self,dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return 
	end
	self.m_escape = self.m_escape - 1
	self:refreshTime()
end

---------------------------------------------------------

function This:onAddHpFinish(biandui)
	if self.m_biandui == biandui then
		self:refresh()
	end
end

function This:onClearChengFangTime(city)
	if self.m_city == city then
		self:updateTroopList()
		self:refreshChengFang()
	end
end

function This:onTrainChengFangOnceKey(city)
	if self.m_city == city then
		self:refreshChengFang()
	end
end

function This:onCityInfoChanged(city)
	if self.m_city == city then
		self:refreshChengFang()
	end
end

function This:onTrainChengFang(cityId,pos)
	self:refreshChengFang()
end