-- @Author: ZYY
-- @Date:   2018-07-31 20:33:12
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-25 16:56:32
-- @File Name: LegionTroopsView.lua
-- @Desc: 补队列表界面

local This , super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")
local LegionCardView = require("Map.LegionCardView")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")

function This:initData(info)
	self.m_biandui = info.biandui
	self.m_city = info.city
	self.m_map = info.map
	self.m_scene = g_scene

	self.m_escape = 0
    self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
	self.m_fileName = "LegionScene/NodeUI/LegionTroopsList.csb"
    self.m_csNode = cc.Node:createFromCSFile(self.m_fileName)
    self:addChild(self.m_csNode)
end

function This:initNode(info)
	local csNode = self.m_csNode
	csNode.team1:setString(self.m_biandui:getName())
	self.m_itemList = {}
	local troop = self.m_biandui:getTroop()
	for _, card in ipairs(troop:getCardList()) do
		local pos = card:getPos()
		local view = LegionCardView:create(card)
		csNode["hero".. pos]:addChild(view)
		self.m_itemList[pos] = view
	end

	if self.m_city:isMintage() then
		csNode.bubing.text:setString(Serialize[61884])
	end
    self:scheduleUpdate()
end

function This:initTouchEvent()
	if not self.m_inited then
		local csNode = self.m_csNode
		if csNode.bubing then
			csNode.bubing:setTouch({
				onEnded = function()
					print("补兵...")
					if self.m_city:isBarracks() then
						g_scene:openDialog("Map.LegionTroopsAddHpPanel",{
							biandui = self.m_biandui,
							city = self.m_city,
							map = self.m_map
						})
					elseif self.m_city:isMintage() then
						g_scene:openDialog("Map.LegionTroopsAddMinePanel",{
							biandui = self.m_biandui,
							city = self.m_city,
							map = self.m_map
						})
					end
				end,
				isScale = false,
				isContainer = true
			})
		end

		csNode.chuzheng:setTouch({
			onEnded = function()
				print("出征...")
				self.m_scene:closeAllDialog()
				local cardList = self.m_biandui:getCardList()
				local hpList = {0,0,0,0,0,0}
				local total = 0
				for _, card in ipairs(cardList) do
					local pos = card:getPos()
					hpList[pos] = card:getHp()
					total = total + card:getHp()
				end
				if total <= 0 then
					g_scene:showTips(Serialize[61691])
					return
				end
				self.m_scene:enterSelectMode(self.m_city,self.m_biandui, LegionTransType.kHp,hpList)
			end,
			isScale = false,
			isContainer = true
		})

		self.m_inited = true
	end
end

function This:refresh()
	local csNode = self.m_csNode
	local troop = self.m_biandui:getTroop()
	local state = troop:getState()
	self.m_isAddingHp = (state == LegionTroopState.kAddHp)
	self.m_csNode.Text_1:setVisible(self.m_isAddingHp)
	local isCanAddHp = state == LegionTroopState.kIdle or state == LegionTroopState.kDefensive or state == LegionTroopState.kAddHp
	if isCanAddHp then
		if self.m_city:isBarracks() then
			isCanAddHp = true
		elseif self.m_city:isMintage() then
			isCanAddHp = true
			for _, card in ipairs(troop:getCardList()) do
				if card:getHp() > 0 then
					isCanAddHp = false
					break
				end
			end
		else
			isCanAddHp = false
		end
	end

	csNode.bubing:setVisible(isCanAddHp)
	csNode.chuzheng:setVisible(state == LegionTroopState.kDefensive)

	csNode.state:setString(LegionTroopStateText[state])
	for i , view in ipairs(self.m_itemList) do
		view:refresh()
	end
end

function This:onEnter()
	super.onEnter(self)
	self:refresh()
	self.m_map:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_map:removeObserver(self)
end

---------------------------------------
function This:update(dt)
	super.update(self,dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return
	end
	self.m_escape = self.m_escape - 1
	if self.m_isAddingHp then
		local endTime = self.m_biandui:getTroop():getAddHpEndTime() 
		local curTime = GamePlayer:getServerTime()
		self.m_csNode.Text_1:setString( GameUtil:formatTime(Serialize["H_M_S"],math.max(0,endTime - curTime)))
	end
end

function This:onAddBianDuiRes(biandui)
	if self.m_biandui == biandui then
		self:refresh()
	end
end

function This:onUndoAddBianDuiHp(biandui)
	if self.m_biandui == biandui then
		self:refresh()
	end
end

function This:onClearBianDuiHp(biandui)
	if self.m_biandui == biandui then
		self:refresh()
	end
end

function This:onAddHpFinish(biandui)
	if self.m_biandui == biandui then
		self:refresh()
	end
end