-- @Author: zheng
-- @Date:   2019-01-21 10:41:02
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-11-07 20:08:42
-- @File Name: LegionWarBoxTip.lua
-- 军团战宝箱
local GameUtil = require("Util.GameUtil")
local GameMapManager = require("Map.GameMapManager")
local EventDispatcher = require("EventDispatcher")
local PopDialogManager = require("UI.PopDialogManager")

local This , super = class(...,"NodeUI")
-- local FamilyManager = require("Official.FamilyManager")
-- This.fileName = "GameScene/NodeUI/TreasurePeerless.csb"
function This:initNode(node)
    self.m_csNode = node.m_csNode.boxtip
    self:getData()
    self:refresh()
    self.m_isUpdating = false
    self.m_escape = 0
    self:scheduleUpdate()
    self:createEffect(true,self.m_csNode.box.item)
    EventDispatcher:addListener(self,"legionWarBoxTipClose",self.closeAnim)
end

function This:refresh()
    local function getCityName(cityId)
        local LegionCityConfig = require("Metadatas.LegionCityConfig")
        local cityItem = LegionCityConfig[cityId]
        if cityItem then
            return cityItem.city_name
        end
        return ""
    end
    local csNode = self.m_csNode.box
    local mMap = self.m_map
    local boxState,leftTime = mMap:getWarBoxLeftTime()
    local _,warBoxItem = mMap:getWarBoxId()
    local isShow = false
    if warBoxItem then
        local cityIds = self:getAppearCity()
        if boxState == LegionBoxState.kAppear or boxState == LegionBoxState.kFuture then
            isShow = mMap:getBoxIsHaveAppear()
            local appearVisible = false
            if boxState == LegionBoxState.kAppear then
                appearVisible = true
                
                for cityIndex,cityId in ipairs(cityIds) do
                    local cityName = ""
                    if cityIndex~=1 then
                        cityName = cityName.."、"
                    end
                    cityName = cityName.."["..getCityName(cityId).."]"
                    local name = csNode.node2["name"..cityIndex].txt
                    name:setString(cityName)
                end
            end
            csNode.node2:setVisible(appearVisible)
            csNode.time1:setVisible(not appearVisible)
        end
    end
    self.m_isUpdating = isShow
    csNode:setVisible(isShow)
    self.m_csNode.anim:setVisible(not isShow)
    self.m_lastRefrshState = boxState
end

function This:refreshTime()
    local mMap = self.m_map
    local boxState,leftTime = mMap:getWarBoxLeftTime()
    local boxNode = self.m_csNode.box
    local labelNode = boxNode.time1
    local strFormat = Serialize["Map_BoxPanel_6_c"].."<New9>:"..Serialize["H_M_S"]
    if boxState == LegionBoxState.kAppear then
        strFormat = Serialize["Map_BoxPanel_5_c"].."<New9>:"..Serialize["H_M_S"]
        labelNode = boxNode.node2.time2
    end
    local str = GameUtil:formatTime(strFormat,leftTime)
    labelNode:setStringWithColor(str)
    if self.m_lastRefrshState and self.m_lastRefrshState ~= boxState then
        self.m_lastRefrshState = boxState
        self:refresh()
    end
end

function This:initTouchEvent()
    local item1 = self.m_csNode.box
    item1.item:setTouchEnded(function()
        g_scene:openDialog("Map.LegionWarBoxPanel")
    end)
     local appearNode = item1.node2
     for nameIndex=1,3 do
         local nameNode = appearNode["name"..nameIndex]
         if nameNode then
             local bg= nameNode.bg
             bg:setTouch({
                        isAlpha = false,
                        isScale = false, 
                        isDynamic = true, 
                        onEnded = function ()
                            local cityId = self:getAppearCity()
                            -- print(cityId[nameIndex])
                            g_scene:focuseCity(cityId[nameIndex])
                            g_scene:showTips(Serialize["world_SearchCity_3"])
                        end
                    })
         end
     end
end
--cityId列表
function This:getAppearCity()
	return self.m_map:getAppearCitys()
end

function This:createEffect(isAdd,node)
    local function createAnim( parent,anmStr )
        local GameUtil = require("Util.GameUtil")
        local effect1 = GameUtil:createEffect(anmStr,true)
        parent:addChild(effect1)
        return effect1
    end
    if node then
        node:removeAllChildren()
        if isAdd then
            local effect1 = createAnim( node,"FaBaoLight" )
            effect1:setPosition(42,42)
        end
    end
end

function This:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape > 1 then
        self.m_escape = self.m_escape - 1
        local mMap= self.m_map
        if mMap and mMap:isInit() then
            if not mMap:getBoxIsHaveAppear() and not self.m_isShowPanel then
                local boxState,leftTime = mMap:getWarBoxLeftTime()
                if boxState == LegionBoxState.kAppear or boxState == LegionBoxState.kFuture then
                    self.m_isShowPanel = true
                    local sceneName = g_scene:getName()
                    local info = {name = "Map.LegionWarBoxPanel",level =DialogLevel.kForLevel,
                        scene = sceneName,bCheck = true}
                    PopDialogManager:addPopDialog(info)
                end
            end
        end
        self:refreshTime()
    end
end

function This:getData()
	-- local mapType = GameMapManager:getMapType()
    self.m_map = GameMapManager:getMap(GameMapType.kLegionMap)
end

function This:closeAnim()
    
    self:moveCenterAnimation({
        name="Icons/Item/264.png",
        parent = self.m_csNode.anim
    }, 0.5, function() self:refresh() end)
end

--参考 DepotBoxEffect
function This:moveCenterAnimation( param, factor, callback)
    local parent = param.parent
    local target = cc.Sprite:create(param.name)
    local x, y = parent:convertToNodeSpace(cc.visible.center.x, cc.visible.center.y)

    local posx, posy = self:getPosition()
    local effectParent = parent
    local effect = nil
    effect = GameUtil:createEffect("XinGongNeng", false, nil, {
        onAnimationComplete = function ()
            effect:removeFromParent()
            local particle = cc.Particle:create("tuowei.plist", self)
            particle:setPosition(x, y)

            local action = nil
            action = cc.Sequence(cc.MoveTo(factor * 0.85, posx, posy, "SineInOut"),
                cc.DelayTime(factor * 0.3),
                cc.CallFunc(function()
                    if param.next then
                        self:playAnimation2( param.next, factor, callback)
                    elseif callback then
                        callback()
                    end
                end),
                cc.RemoveAction()
            )
            particle:runAction(action)
            effectParent:addChild(particle)
            particle:setLocalZOrder(101)  

            action = cc.Sequence(cc.MoveTo(factor * 0.6 , posx, posy, "SineIn"), cc.RemoveAction())
            target:runAction(action)
        end     
    }, false, "animation2")
    effect:setPosition(x, y)
    effectParent:addChild(effect)
    effect:setLocalZOrder(120)

    target:setPosition(x, y)
    target:setScale(0.7)
    effectParent:addChild(target)
    target:setLocalZOrder(121)

end
function This:onEnter()
    super.onEnter(self)
    if self.m_map then
        self.m_map:addObserver(self)
    end
end

function This:onExit()
    super.onExit(self)
    if self.m_map then
        self.m_map:removeObserver(self)
    end
end

function This:onLegionBoxInfo()
    self:refresh()
end
