--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local MatchManager = require("Match.MatchManager")
local GameUtil = require("Util.GameUtil")

local kMaxUnlockBoxNum = 1

function This:initNode(box)
	self.m_box = box
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MatchingBoxOpen.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self:initInfo()
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.openbutton:setTouchEnded(function()
		self:openDirectly()
	end)
	self.m_csNode.beginbutton:setTouchEnded(function()
		self:unlockBox()
	end)
end

function This:initInfo()
	local config = self.m_box:getConfig()
	self.m_csNode.credit:setString(string.format("%d-%d",config.min_merit,config.max_merit))
	self.m_csNode.cardpiece:setString(string.format("X%d",config.card_total))

	local qualitList = {}
	for i , quality in ipairs(config.card_quality) do
		if quality > 2 then
			table.insert(qualitList,{id = quality , num = config.card_num[i]})
		end
	end

	for i = 1 , #qualitList do
		self.m_csNode["quality_num"..i]:setString(string.format("X%d",qualitList[i].num))
	end
	for i = #qualitList + 1 , 2 do
		self.m_csNode["quality_image"..i]:setVisible(false)
		self.m_csNode["quality_num"..i]:setVisible(false)
	end	
	self:refresh()
end

function This:onEnter()
	super.onEnter(self)
	self.m_box:addObserver(self)
	MatchManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_box:removeObserver(self)
	MatchManager:removeObserver(self)
end

function This:refresh()
	local state = self.m_box:getState()
	local count = MatchManager:getBoxCountInOpening()

	self.m_csNode.beginbutton:setVisible(state == BoxState.kLock and count < kMaxUnlockBoxNum )
	self.m_csNode.countdownbg:setVisible(state == BoxState.kOpening)

	if state == BoxState.kOpening then
		self.m_csNode.countdownbg.time:setString(GameUtil:formatTime(Serialize["H_M_S"],self.m_box:getLeftTime()))
		self.m_csNode.countdownbg.timebar:setPercent(self.m_box:getLeftTime() / self.m_box:getCDTotalTime() * 100)
	elseif state == BoxState.kLock and count < kMaxUnlockBoxNum then
		self.m_csNode.beginbutton.tiem:setString(GameUtil:formatTime(Serialize["H_M_S"],self.m_box:getCDTotalTime()))
	end
	self.m_csNode.openbutton.cost:setString("X"..self.m_box:getRecieveCost())
end

function This:openDirectly()
	MatchManager:onOpenBoxRequest(self.m_box:getIndex())
end

function This:unlockBox()
	MatchManager:onUnlockBoxRequest(self.m_box:getIndex())
end

function This:updateTime(leftTime)
	self:refresh()
end

function This:onInfoChanged(box)
	self:refresh()
end

function This:onRemoveBox(box)
	if box == self.m_box then
		self:close()
	end
end


--endregion
