--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local MatchManager = require("Match.MatchManager")
local RoomRoleView = require("Match.RoomRoleView")
local InviteView = require("Match.InviteView")
local Timer = require("Timer")
local GamePlayer = require("Player.GamePlayer")
local Serialize = require("Serialize.Serialize")
local FriendManager = require("Friend.FriendManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local MatchingRoomConfig = require("Metadatas.MatchingRoomConfig")
local ChatManager = require("Chat.ChatManager")
local VoiceChatManager = require("VoiceChat")
local GameUtil = require("Util.GameUtil")

local LastSendApplyTime = 0
local kSendApplyCD = 60

function This:initNode(relationType)
	FriendManager:onInitFriendInfoRequest()
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TeamMatching.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self.m_viewList = {}
	self.m_recvList = {}
	self.m_timer = Timer:create(self)
	self:scheduleUpdate()

	self:addTabButton("friend",self.m_csNode.friend,function()
		self:refreshListItem(InviteType.kFriend)
	end)

	self:addTabButton("tongmeng",self.m_csNode.family,function()
		self:refreshListItem(InviteType.kGroup)
	end)
	
	self.m_maxRoleCount = 0
	self:initRoom()
	self:selectTabButton("friend")

	local escape = os.time() - LastSendApplyTime
	if escape < kSendApplyCD then
		self.m_csNode.invitation.text:setString(string.format(Serialize[61501],math.ceil(kSendApplyCD - escape)))
		self.m_csNode.invitation:setGrayShader()
		self.m_isUpdateCD = true
	end
end
 
function This:initTouchEvent()
	local room = MatchManager:getRoom()
	local role = room:getMainRole()
	self.m_isShowFriend = true
	self.m_isRecvRefresh = true

	self.m_csNode.button1:setTouchEnded(function()
		if role:isRoomOwner() then
			if room:isReady() then
				MatchManager:onMatchBattleRequest()
			else
				g_scene:showTips(Serialize["room_roles_not_ready"])
			end
		elseif not role:isReady() then
			MatchManager:onPrepareMatchRequest(true)
		else
			MatchManager:onPrepareMatchRequest(false)
		end
	end)
	self.m_csNode.close:setTouchEnded(function()
		if MatchManager:isInRoom(role:getId()) then
			MatchManager:onExitTeamMatchRequest(role:getId())
		end
		self:close()
	end)

	self.m_csNode.invitation:setTouchEnded(function()
		if MatchManager:getRankLv() < kChatRankLimit then
			g_scene:showTips(string.format(Serialize["chat_ChatPanel_5"],MatchManager:getRankName(kChatRankLimit)))--kChatLevelLimit
			return
		end
		if os.time() - LastSendApplyTime >= kSendApplyCD then
			ChatManager:onApplyJoinMatchRequest(role,room)
			self.m_csNode.invitation:setGrayShader()
			self.m_isUpdateCD = true
			LastSendApplyTime = os.time()
		end
	end)
	self.m_isInVoiceRoom = false
	self.m_csNode.voice:setTouchEnded(function()
		if not self.m_isInVoiceRoom then
			local key = "voice_" .. GameUtil:int64ToString(room:getId())
			VoiceChatManager:joinTeamRoom(key)
		else
			VoiceChatManager:exitRoom()
		end
	end)
end

function This:getLevelRange()
	local room = MatchManager:getRoom()
	local matchType = room:getMatchType()
	return MatchingRoomConfig[matchType].Minlevel , MatchingRoomConfig[matchType].Maxlevel
end

function This:initRoom()
	local space = 80
	local room = MatchManager:getRoom()
	local roomType = room:getRelationType()
	if roomType == MatchRelationType.kJun then
		self.m_csNode.lianmen:setVisible(false)
		self.m_maxRoleCount = 3
		space = 200
	elseif roomType == MatchRelationType.kAlly then
		self.m_csNode.juncheng:setVisible(false)
		space = 250
		self.m_maxRoleCount = 2
	end

	for i = 1 , self.m_maxRoleCount do
		local view = RoomRoleView:create()
		view:setPositionX(-space * self.m_maxRoleCount / 2 + (i-1) * space + space/2)
		self.m_csNode.palyer:addChild(view)
		self.m_viewList[i] = view
	end
	local minlv , maxlv = self:getLevelRange()
	self.m_csNode.levellimit:setString(string.format(Serialize[61900],minlv,maxlv))
	
	self:refresh()
end

function This:onEnter()
	super.onEnter(self)
	MatchManager:addObserver(self)
	FriendManager:addObserver(self)
	VoiceChatManager:setJoinRoomCallback(function(memeberId)
		if memeberId > 0 then
			g_scene:showTips(Serialize[61901])
			self.m_csNode.voice.text:setString(Serialize[61902])
			self.m_isInVoiceRoom = true
			cc.Audio:pauseBackground()
		else
			cc.Audio:resumeBackground()
			g_scene:showTips(Serialize[61903])
			self.m_csNode.voice.text:setString(Serialize[61904])
			self.m_isInVoiceRoom = false
		end
	end)
end

function This:onExit()
	super.onExit(self)
	cc.Audio:resumeBackground()
	MatchManager:removeObserver(self)
	FriendManager:removeObserver(self)
	VoiceChatManager:setJoinRoomCallback(nil)
	VoiceChatManager:exitRoom()
end

function This:refreshListItem(inviteType)
	self.m_csNode.scrollView:removeAllItems()
	self.m_timer:removeAllTimer()
	self.m_itemViews = {}

	local list = MatchManager:getCanInviteList(inviteType)
	for i , invite in ipairs(list) do
		self.m_timer:setTimer(0.01 * (i - 1),function()
			local view = InviteView:create(invite)
			local w , h = view:getContentSize()
			self.m_csNode.scrollView:pushBackCustomItem(view,w,h-3,0.5)
			self.m_itemViews[invite:getId()] = view
		end,"invite"..i)
	end
end

function This:refresh()
	local room = MatchManager:getRoom()
	local roleList = room:getRoleList()
	for i , role in ipairs(roleList) do
		local view = self.m_viewList[i]
		view:setModel(role)
	end
	for i = #roleList + 1 , self.m_maxRoleCount do
		local view = self.m_viewList[i]
		view:setModel(nil)
	end

	local role = room:getMainRole()
	if role:isRoomOwner() then
		self.m_csNode.button1.begin:setVisible(room:isReady())
		self.m_csNode.button1.zhunbei:setVisible(not room:isReady())
		self.m_csNode.button1.quxiao:setVisible(false)
	else
		self.m_csNode.button1.begin:setVisible(false)
		self.m_csNode.button1.zhunbei:setVisible(not role:isReady())
		self.m_csNode.button1.quxiao:setVisible(role:isReady())
	end
	self.m_csNode.invitation:setVisible(room:getOwnerId() == GamePlayer:getPlayerID())
end

function This:onExitTeamBattle(targetId,sourceId)
	if targetId == GamePlayer:getPlayerID() then
		self:close()
		if sourceId ~= GamePlayer:getPlayerID() then
			local room = MatchManager:getRoom()
			local role = room:getRole(sourceId)
			local str = string.format(Serialize["exit_room"],role:getName())
			g_scene:showTips(str)
		end
	else
		local room = MatchManager:getRoom()
		local role = room:getRoleRemoved(targetId)
		local str = string.format(Serialize["exit_room1"],role:getName())
		g_scene:showTips(str)
	end
	local view = self.m_itemViews[targetId]
	if view then
		view:refresh()
	end
end

function This:onRoomChanged()
	self:refresh()
	local room = MatchManager:getRoom()
	local roles = room:getAllRole()
	for i , role in pairs(roles) do
		local view = self.m_itemViews[role:getId()]
		if view then
			view:refresh()
		end
	end
end

function This:onDissolveTeam(targetId,sourceId)
	self:close()
end

function This:onPrepareMatch(roleId)
	for i , view in ipairs(self.m_viewList) do
		local role = view:getModel()
		if role and role:getId() == roleId then
			view:refresh()
		end
	end
	local view = self.m_itemViews[roleId]
	if view then
		view:refresh()
	end
end

function This:update(dt)
	super.update(self,dt)
	self.m_timer:update(dt)
	if self.m_isUpdateCD then
		local escape = os.time() - LastSendApplyTime
		if escape < kSendApplyCD then
			self.m_csNode.invitation.text:setString(string.format(Serialize[61501],math.ceil(kSendApplyCD - escape)))
		else
			self.m_csNode.invitation.text:setString(Serialize[61905])
			self.m_csNode.invitation:clearShader()
			self.m_isUpdateCD = false
		end
	end
end

--endregion
