--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local MatchManager = require("Match.MatchManager")
local WarBoxConfig = require("Metadatas.WarBoxConfig")
local DropConfig = require("Metadatas.DropConfig")

-- 消除CD的公式
local kBoxFormula = function ( time )
	return time / 240
end

BoxState = 
{
	kLock = 0,	--为开锁
	kOpening = 1, --正开启
	kReceive = 2,  --带领取
}

function This:create(...)
	local box = {}
	setmetatable(box,self)
	box:init(...)
	return box
end

function This:init(info)
	super.init(self)
	print_r(info)
	self.m_index = info.box_index
	self.m_state = info.state
	self.m_endTime = info.end_time
	self.m_boxId = info.box_id
	self.m_config = WarBoxConfig[self.m_boxId]
	self.m_escape = 0
	self.m_isUpdating = false
	local curTime = GamePlayer:getServerTime()
	if self.m_state ~= BoxState.kLock then
		if curTime < self.m_endTime then
			self.m_state = BoxState.kOpening
			self:startUpdate()
		elseif curTime >= self.m_endTime then
			self.m_state = BoxState.kReceive
		end
	end
end

function This:getId()
	return self.m_boxId
end

function This:getState()
	return self.m_state
end

function This:isOpening()
	return self.m_state == BoxState.kOpening
end

function This:getConfig()
	return self.m_config
end

function This:getIndex()
	return self.m_index
end

function This:getCDTotalTime()
	return self.m_config.need_time
end

function This:updateInfo(info)
	if info.box_index ~= self.m_index then
		return
	end

	if self.m_isUpdating then
		self:endUpdate()
	end

	self.m_index = info.box_index
	self.m_state = info.state
	self.m_endTime = info.end_time
	self.m_boxId = info.box_id
	self.m_config = WarBoxConfig[self.m_boxId]
	self.m_escape = 0

	local curTime = GamePlayer:getServerTime()
	if self.m_state ~= BoxState.kLock then
		if curTime < self.m_endTime then
			self.m_state = BoxState.kOpening
			self:startUpdate()
		elseif curTime >= self.m_endTime then
			self.m_state = BoxState.kReceive
		end
	end
end

function This:getRecieveCost()
	-- if self.m_state == BoxState.kLock then
	-- 	return math.ceil(kBoxFormula(self.m_config.need_time))
	-- elseif self.m_state == BoxState.kOpening then
	-- 	local leftTime = self:getLeftTime()
	-- 	return math.ceil(kBoxFormula(leftTime))
	-- end
	-- return 0
	return self.m_config.cost
end

function This:getUnlockTime()
	return self.m_config.need_time
end

function This:getProgress()
    return self:getLeftTime() / self:getUnlockTime()
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return 0
	end
	return self.m_endTime - curTime
end

function This:setEndTime(endTime)
	self.m_endTime = endTime
	local curTime = GamePlayer:getServerTime()
	if curTime < self.m_endTime then
		self.m_state = BoxState.kOpening
		self:startUpdate()
	end
	self:notifyObservers("onInfoChanged",self)
end

function This:startUpdate()
	Application:addToUpdateList(self)
	self:notifyObservers("onBoxUpdateStart",self)
	self.m_isUpdating = true
end

function This:endUpdate()
	Application:removeFromUpdateList(self)
	self.m_state = BoxState.kReceive
	self:notifyObservers("onBoxUpdateEnd",self)
	self.m_isUpdating = false
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			self:notifyObservers("updateTime",leftTime)
		else
			self:notifyObservers("updateTime",0)
			self:endUpdate()
		end
	end
end

function This:getComment()
    return self.m_config.desc
end



--endregion
