-- @Author: superm
-- @Date:   2017-10-24 18:00:18
-- @Last Modified by:   su
-- @Last Modified time: 2019-03-05 17:00:11
-- @File Name: MatchOfficeBoxPanel.lua

local This, super = class(...,"Dialog")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local ItemNode = require("UI.ItemNode")

function This:initNode(info)
    self.m_initLevel = MatchManager:getRankLv()
    self.m_mode = info.mode
    self.m_level = info.level
    self.m_state = info.state

    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MatchingLevelBox.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
    self:addChild(self.m_csNode)

    if self.m_mode then
    	self.m_csNode.title:setString(Serialize[61933])
	else
    	self.m_csNode.title:setString(Serialize[61934])
    end

	self.m_csNode.left:setVisible(self.m_mode)
	self.m_csNode.right:setVisible(self.m_mode)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.button:setTouchEnded(function()
    	if not self.m_state then
    		if self.m_mode then
				MatchManager:onPickUpDailyRewardRequest()
			else
				MatchManager:onPickUpLevelRewardRequest(self.m_level)
			end
    	end
    end)

    self.m_csNode.right:setTouchEnded(function()
		local count = kMaxMatchRankLv
	    local maxIndex = self.m_initLevel + 1 >= count and count or self.m_initLevel + 1
	    if self.m_level + 1 <= maxIndex then
	        self.m_level = self.m_level + 1
	        self:refresh()
	    end
    end)

    self.m_csNode.left:setTouchEnded(function()
	    local minIndex = self.m_initLevel - 1 <= 0 and 1 or self.m_initLevel - 1
	    if self.m_level - 1 >= minIndex then
	        self.m_level = self.m_level - 1
	        self:refresh()
	    end
    end)
end

function This:onEnter()
    super.onEnter(self)
    MatchManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    MatchManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
	self:refreshList(self.m_level, self.m_csNode)
end

function This:refreshList(level, page)
	if self.m_mode then
		local count = kMaxMatchRankLv
	    local minIndex = self.m_initLevel - 1 <= 0 and 1 or self.m_initLevel - 1
	    local maxIndex = self.m_initLevel + 1 >= count and count or self.m_initLevel + 1

	    if level == minIndex then
	        self.m_csNode.left:setGrayShader()
	    elseif level == minIndex + 1 then
	        self.m_csNode.left:clearShader()
	    end

	    if level == maxIndex then
	        self.m_csNode.right:setGrayShader()
	    elseif level == maxIndex - 1 then
	        self.m_csNode.right:clearShader()
	    end
	end

	local config = MatchingConfig[level]
	local rewardList = self.m_mode and config.per_diem or config.office_box

	local scrollView = self.m_csNode.scrollView
	scrollView:removeAllChildren()
	scrollView.m_direction = "right"
	if rewardList then
		for i , info in ipairs(rewardList) do
			local item = ItemNode:create(info[1], info[2])
			item:setCascadeOpacityEnabled(true)
			-- item:setAnchorPoint(0.5, 0.5)
			local w , h = item:getContentSize()
			if #rewardList < 4 then
				scrollView:setTouchEnabled(false)
				item:setPosition(-(#rewardList-2)*50+100*i,0)
				scrollView:addChild(item)
			else
				item:setAnchorPoint(0.5, 0.5)
				scrollView:pushBackCustomItem(item, w + 4, h,0.5)
			end
		end
	end

    local curState = false
    if not self.m_mode then
    	curState = MatchManager:getMaxRankLvInHistory() >= level
	else 
    	curState = level == self.m_initLevel
    end

    if curState then
    	page.condition:setVisible(false)
    	page.button:setVisible(true)
		if self.m_state then
			page.button:setGrayShader()
			page.button.text:setColor(0, 0, 0)
			page.button.text:setString(Serialize[61037])
		else
			page.button:clearShader()
			page.button.text:setColor(255, 255, 255)
			page.button.text:setString(Serialize[61935])
		end
	else
		local name = MatchingConfig[level].rank_level
    	page.condition:setVisible(true)
    	page.button:setVisible(false)
    	if self.m_mode then
    		page.condition:setString(name .. Serialize[61936])
		else
    		page.condition:setString( Serialize[61937] .. name .. Serialize[61015])
		end
    end
end

function This:afterPickUp()
	local config = MatchingConfig[self.m_level]
	local rewardList = self.m_mode and config.per_diem or config.office_box
	local itemList = {}
	for _, itemInfo in ipairs(rewardList) do
		table.insert(itemList, {id=itemInfo[1], num=itemInfo[2]})
	end
	g_scene:showReward(itemList)
	self:close()
end

function This:onPickUpLevelReward(level)
	self:afterPickUp()
end

function This:onPickUpDailyReward()
	self:afterPickUp()
end