-- @Author: superm
-- @Date:   2017-10-24 14:46:52
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-06 11:46:06
-- @File Name: MatchOfficeItemView.lua

local This , super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/MatchingLevelListItem.csb"

local GameUtil = require("Util.GameUtil")
local MatchUtil = require("Match.MatchUtil")
local MatchManager = require("Match.MatchManager")
local WarfareUtil = require("Warfare.WarfareUtil")

local BuildTypeText = {
	[BuildType.kDwelling] = Serialize[61939],
	[BuildType.kFarmland] = Serialize[61940],
	[BuildType.kMarket] = Serialize[61941],
}

local PrivObj = {
	kBuildTime = 1,
	kProduction = 2,
	kBuildCost = 3,
}

local PrivObjText = {
	[PrivObj.kBuildTime] = Serialize[61942],
	[PrivObj.kProduction] = Serialize[61943],
	[PrivObj.kBuildCost] = Serialize[61944],
}

local BoxImage = 
{
	"MenuScene/boxLock4.png",
	"MenuScene/boxGet4.png",
}

function This:initData(lev, config)
    self.m_level = lev or 1
    self.m_config = config
    self.m_inited = false
    local w , h = self.bg:getContentSize()
    self:setContentSize(w,h)
    self.duanwei:setImage(RankIconImage[self.m_level])
end

function This:onEnter()
    super.onEnter(self)
    MatchManager:addObserver(self)
end

function This:onExit()
	MatchManager:removeObserver(self)
    super.onExit(self)
end

function This:initNode()
    self:refreshItemUI()
    self:refreshStateUI()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.Box:setTouch({
            getRect = function (touch)
                local rect = touch.sizeNode:getBoundingBox()
                return rect
            end,
		    onEnded = function()
                self:onPickUpBtnClick()
            end,
            isContainer = true,
            isDynamic = true,
            isScale = false,
        })  
        self.m_inited = true
    end
end

function This:refreshItemUI()
	self.title:setString(self.m_config.rank_level)
    local comment1 = self.m_config.comment1
    self.describe1:setStringWithColor(comment1)
    local comment2 = self.m_config.comment2
    if comment2 and comment2 ~= 0 and comment2 ~= "" then
    	self.describe2:setStringWithColor(comment2)
    else
    	self.describe2:setVisible(false)
    	self.dtitle:setVisible(false)
    end
end

function This:refreshStateUI()
	local curLevel = MatchManager:getMaxRankLvInHistory()
	self.m_levelListGotReward = MatchManager:getLevelListGotReward()
	self.m_mode, self.m_state, self.m_canPickUp = false, self.m_levelListGotReward[self.m_level], self.m_level <= curLevel
	-- if self.m_state then
	-- 	self.m_mode = true
	-- 	self.m_canPickUp = self.m_level == curLevel
	-- 	self.m_state = MatchManager:isGotDailyReward()
	-- end
	local hasEff = false
	local visible = self.m_config.office_box and #self.m_config.office_box > 0 
	self.Box:setVisible(visible)
	if self.m_canPickUp then
		self.Box:setImage(BoxImage[2])
		if not self.m_state then
			self.Box:clearShader()
			hasEff = true
		else
			self.Box:setGrayShader()
		end
	else
		self.Box:setImage(BoxImage[1])
	end
	self.m_effectList = self.m_effectList or {}
	WarfareUtil.createBoxEffect(self.Box, hasEff, self.m_effectList)
	self.gaoguang:setVisible(self.m_level == MatchManager:getRankLv())
end


function This:onPickUpBtnClick()
	g_scene:openDialog("Match.MatchOfficeBoxPanel", {level = self.m_level, mode = self.m_mode, state = self.m_state})
end

function This:onPickUpLevelReward(level)
	if self.m_level == level then
		self:refreshStateUI()
	end
end

-- function This:onPickUpDailyReward()
-- 	self:refreshStateUI()
-- end


--endregion