-- @Author: superm
-- @Date:   2017-10-24 15:44:24
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-21 14:47:04
-- @File Name: MatchOfficePanel.lua

local This, super = class(...,"Dialog")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local MatchOfficeItemView = require("Match.MatchOfficeItemView")
local SeasonConfig = require("Metadatas.SeasonConfig")
local SeasonRewardConfig = require("Metadatas.SeasonRewardConfig")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

local BoxImage = 
{
    "MenuScene/boxLock4.png",
    "MenuScene/boxGet4.png",
}

local MatchTab = {
    kOffice = "tep1",
    kWelfare = "tep2",
    kExhaustion = "tep3",
}

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MatchingLevelList.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
    self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
    self.m_scrollView = self.m_csNode.ScrollView

    -- self.m_progressTimer = cc.ProgressTimer:create(self.m_csNode.jifenbar)
    -- self.m_csNode.jifenbg:addChild(self.m_progressTimer)
    -- self.m_progressTimer:setScaleY(-1)
    -- local w , h = self.m_csNode.jifenbg:getContentSize()
    -- self.m_progressTimer:setPosition(w/2,h/2)
    for key,val in pairs(MatchTab) do
        self:addTabButton(val, self.m_csNode[val], function ( ... )
			return self:onTabButtonSelect(val)
		end)
    end
	self.m_initSelectTab = MatchTab.kOffice
    

    -- self:refreshBaseInfo()
    -- self:initSeasonReward()
	GamePlayer:onGetSeasonInfoRequest()
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:setVisible(false)
        self:setTimer(0.05, function ()
            self:close()
        end)
    end)
    -- self.m_csNode.boxGet:setTouchEnded(function()
    --     g_scene:openDialog("Match.MatchOfficeBoxPanel", {level = self.m_level, mode = self.m_mode, state = self.m_state})
    -- end)
    -- self.m_csNode.help:setTouchEnded(function()
    --     g_scene:openDialog("UI.HelpTips",{text = Serialize["guanzhi_help"], node = self.m_csNode.help})
    -- end)
end

function This:onEnter()
    super.onEnter(self)
    MatchManager:addObserver(self)
    GamePlayer:addObserver(self)
    self:selectTabButton(self.m_initSelectTab)
    
    -- self:refresh()
end

function This:onExit()
    MatchManager:removeObserver(self)
	GamePlayer:removeObserver(self)
    super.onExit(self)
end

function This:onTabButtonSelect(name)
    self:openTabNode(name)
	return true
end

function This:createTabNode(name)
	if name == MatchTab.kOffice then
		return require("Match.MatchingLevelListLookPanel"):create()
	elseif name == MatchTab.kWelfare then
		return require("Match.MatchingLevelListRewardPanel"):create()
	elseif name == MatchTab.kExhaustion then
		return require("Match.MatchingLevelListRechargePanel"):create()
	end
end

-- function This:refreshBaseInfo()
--     local rankLv = MatchManager:getRankLv()
--     local config = MatchingConfig[rankLv]
--     local score = MatchManager:getScore()
--     local jifenStr = score
--     if rankLv < kMaxMatchRankLv then
--         jifenStr = string.format("%d/%d",score,config.score)
--     end
--     self.m_csNode.jifen:setString(jifenStr)
--     local star = MatchManager:getStar()
--     local maxStar = MatchingConfig[rankLv].star
--     self.m_progressTimer:fromToTimer(1,0,score / config.score)
-- end

-- function This:initSeasonReward()
--     local seasonId = MatchManager:getSeasonId()
--     self.m_csNode.winreward.text_0:setString(string.format("第%d赛季",seasonId))
--     local endTime = MatchManager:getSeasonEndTime()

--     local date = os.serverdate("*t",endTime)
--     self.m_csNode.winreward.text_0_0:setString(string.format("%d年%d月%d日",date.year,date.month,date.day))
--     local ssconfig = SeasonConfig[seasonId]
--     if ssconfig then
--         local limitRank = ssconfig.rank_limit
--         local limitRankName = MatchingConfig[limitRank].rank_level
--         self.m_csNode.winreward.desc:setString(string.format("官职达到%s", limitRankName))

--         local maxRank = MatchManager:getCurSeasonMaxRankLv()
--         local item = SeasonRewardConfig[seasonId * 1000 + maxRank].office_reward[1]
--         local node = ItemNode:create(item[1],item[2])
--         node:setAnchorPoint(0.5,0.5)
--         self.m_csNode.seasonreward.rank_item:addChild(node) 
        
--         item = ssconfig.reward
--         node = ItemNode:create(item[1],item[2])
--         node:setAnchorPoint(0.5,0.5)
--         self.m_csNode.winreward.season_item:addChild(node)
--         self.m_csNode.seasonreward.des:setString(string.format("%s奖励", MatchingConfig[maxRank].rank_level))
--     end
-- 	self.m_csNode.winreward.finished:setVisible(false)
-- end

-- function This:refreshDailyReward()
--     self.m_mode = true
--     self.m_level = MatchManager:getRankLv()
--     self.m_state = MatchManager:isGotDailyReward()
--     local hasEff = not self.m_state
--     if hasEff then
--         if not self.m_csNode.effect.effectList then
--             local w ,h = self.m_csNode.boxGet:getContentSize()
--             local scale = self.m_csNode.boxGet:getScale()
--             local effect1 = GameUtil:createEffect("BoxLing",true)
--             effect1:setPosition(w/2,50)
--             self.m_csNode.effect:addChild(effect1)
--             local effect2 = GameUtil:createEffect("BoxLing2",true)
--             self.m_csNode.effect:addChild(effect2)
--             effect2:setPosition(w/2,h)
--             self.m_csNode.effect.effectList = {effect1,effect2}
--         end
--     else
--         if self.m_csNode.effect.effectList then
--             for _ , effect in ipairs(self.m_csNode.effect.effectList) do
--                 effect:removeFromParent()
--             end
--             self.m_csNode.effect.effectList = nil
--         end
--         self.m_csNode.boxGet:setGrayShader()
--     end
-- end

-- function This:refresh()
--     self:refreshDailyReward()
--     self.m_csNode.position:setString(MatchingConfig[self.m_level].rank_level)
--     self.m_scrollView:removeAllItems()
--     self.m_scrollView:setTouchEnabled(true)
--     self:setTimer(0.05, function ()
--         for i=1, kMaxMatchRankLv do
--             local index = kMaxMatchRankLv - i + 1
--             local view = MatchOfficeItemView:create(index, MatchingConfig[index])
--             local w , h = view:getContentSize()
--             self.m_scrollView:pushNode(view, w, h, nil)
--             if self.m_level == index then
--                 self.m_scrollView:jumpToBottom()
--             end
--         end
--     end)
-- end

-- function This:onGetRewardInfo()
--     self:refresh()
-- end

-- function This:onPickUpDailyReward()
--     self:refreshDailyReward()
-- end

-- function This:onGetSeasonInfo(info)
-- 	self.m_csNode.winreward.finished:setVisible(info.win_num >= kSecceedFinishNumLimit)
-- end

-------------------------------------