--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local Timer = require("Timer")
local GamePlayer = require("Player.GamePlayer")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local LoginManager = require("Login.LoginManager")
local RankItemView = require("Match.RankItemView")
local GameUtil = require("Util.GameUtil")

local RankListType = 
{
	kRankWholeServer = "whole_server",
	kRankServerSelected = "server_selected",
}

function This:initNode(info)
	self.m_startIndex = info.start_index
	self.m_endIndex = info.end_index
	self.m_rankList = info.rank_list
	self.m_myRank = info.self_rank
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RankingList.csb")
	self:addChild(self.m_csNode)
	self.m_initPosX = -168
	self.m_initPosY = cc.visible.rect.height / 2
	self.m_csNode:setPosition(self.m_initPosX,self.m_initPosY)
	self.m_timer = Timer:create(self)
	self.m_curSelected = RankListType.kRankWholeServer
	self:scheduleUpdate()

	self:refreshMyInfo()
	self:refreshItems()
end

function This:initTouchEvent()
	self.m_csNode.rankinglist.rankinglistbg:setTouch({
		onEnded = function()
			self:close()
		end,
		isScale = false
	})

	self:addTabButton(RankListType.kRankWholeServer,self.m_csNode.teb1,function()
		self:onSelectRankList(RankListType.kRankWholeServer)
	end)

	self:addTabButton(RankListType.kRankServerSelected,self.m_csNode.teb2,function()
		self:onSelectRankList(RankListType.kRankServerSelected)
	end)
	self:selectTabButton(RankListType.kRankWholeServer)
end

function This:refreshMyInfo()
	if self.m_myRank >= self.m_startIndex and self.m_myRank <= self.m_endIndex then
		self.m_csNode.rankinglistnum:setString(self.m_myRank)
	else
		self.m_csNode.rankinglistnum:setString(Serialize[60238])
	end

	GameUtil:setPlayerIconNodeImage(self.m_csNode.pic.icon, GamePlayer:getIcon())
	self.m_csNode.name:setString(GamePlayer:getRoleName())
	self.m_csNode.level:setString(Serialize[61950]..GamePlayer:getLevel())
	self.m_csNode.marquis:setString(MatchingConfig[MatchManager:getRankLv()].rank_level)
	self.m_csNode.starnumber:setString(MatchManager:getStar())
	self.m_csNode.marquisicon:setImage(RankIconImage[self.m_rank] or RankIconImage[1])
end

function This:refreshItems()
	self.m_csNode.ScrollView:removeAllItems()
	self.m_timer:removeAllTimer()
	for i , info in ipairs(self.m_rankList) do
		self.m_timer:setTimer(0.1 * i,function()
			local view = RankItemView:create(info,i)
			local w , h = view:getContentSize()
			view:setAnchorPoint(0,0)
			self.m_csNode.ScrollView:pushBackCustomItem(view,w,h,0.5)
		end,"rank_list"..i)
	end
end

function This:onSelectRankList(index)
	if self.m_curSelected ~= index then
		local server = 0 
		if index == RankListType.kRankServerSelected then
			local config = LoginManager:getLoginServer()
            if config then
                server = config:getId() or 0
            end
		end
		MatchManager:onGetRankListRequest(server)
		self.m_curSelected = index
	end
end

function This:runOpenAnimation()
    self:setCascadeOpacityEnabled(true)
    self:setOpacity(0)
    self.m_csNode:runAction(cc.MoveTo(0.2, self.m_initPosX + 327, self.m_initPosY, "ExponentialIn"))
    self:runAction(cc.Sequence(cc.FadeIn(0.2), cc.CallFunc(function ()
        self:onOpen()
    end)))
end

function This:runCloseAnimation()
    self:setCascadeOpacityEnabled(true)
    self:setOpacity(255)
    self.m_csNode:runAction(cc.MoveTo(0.2, self.m_initPosX, self.m_initPosY, "ExponentialOut"))
    self:runAction(cc.Sequence(cc.FadeOut(0.2), cc.CallFunc(function ()
        self:removeFromParent()
    end)))
end

function This:onGetRankListInfo(msg)
	self.m_startIndex = msg.start_index
	self.m_endIndex = msg.end_index
	self.m_rankList = msg.rank_list
	self.m_myRank = msg.self_rank
	self:refreshMyInfo()
	self:refreshItems()
end

function This:onEnter()
	super.onEnter(self)
	MatchManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	MatchManager:removeObserver(self)
end

function This:update(dt)
	super.update(self,dt)
	self.m_timer:update(dt)
end

--endregion
