--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local RoomRole = require("Match.RoomRole")
local GamePlayer = require("Player.GamePlayer")

function This:init(info)
	super.init(self, info)
	self:initInfo(info)
end

function This:initInfo(info)
	local roomId = info.room_id or 0
	self.m_id = tonumber(roomId)

	self.m_matchType = info.match_type
	self.m_type = info.relation_type
	self.m_masterId = info.master[1] or 0
	self.m_allyList = info.ally
	self.m_collList = info.colleague
	
	local ownerId = info.owner_id or 0
	self.m_ownerId = tonumber(ownerId)

	self.m_roleRemoved = {}
	self.m_roleList = {}
	for i , roleInfo in ipairs(info.role_list) do
		self:addRole(roleInfo)
	end
	
	self:updateRelation()
	self:setOwnerId(self.m_ownerId)
	self.m_matching = false
end

function This:getId()
	return self.m_id
end

function This:getRelationType()
	return self.m_type
end

function This:getMatchType()
	return self.m_matchType
end

function This:getOwnerId()
	return self.m_ownerId
end

function This:setOwnerId(ownerId)
	self.m_ownerId = tonumber(ownerId or 0)
	for i , role in ipairs(self.m_roleList) do
		role:setIsRoomOwner(role:getId() == ownerId)
	end
end

function This:getOwner()
	for i , role in ipairs(self.m_roleList) do
		if role:getId() == self.m_ownerId then
			return role
		end
	end
end

function This:startMatch()
	self.m_matching = true
end

function This:getAllRole()
	return self.m_roleList
end

function This:getRoleList()
	local roleList = {}
	for i , role in ipairs(self.m_roleList) do
		table.insert(roleList,role)
	end
	--[[table.sort(roleList,function(a,b)
		if a:isRoomOwner() then
			return true
		elseif b:isRoomOwner() then
			return false
		elseif a:getId() < b:getId() then
			return true
		end
		return false
	end)]]
	return roleList
end

function This:getRole(id)
	for i , role in ipairs(self.m_roleList) do
		if role:getId() == id then
			return role
		end
	end
end

function This:getRoleRemoved(roleId)
	return self.m_roleRemoved[roleId]
end

function This:isInRoom(id)
	for i , role in ipairs(self.m_roleList) do
		if role:getId() == id then
			return true
		end
	end
	return false
end

function This:getMainRole()
	local playerId = GamePlayer:getPlayerID()
	return self:getRole(playerId)
end

function This:addRole(roleInfo)
	local role = RoomRole:create(roleInfo)
	table.insert(self.m_roleList,role)
	role:setIsRoomOwner(roleInfo.role_id == self.m_ownerId)
end

function This:removeRole(id)
	for i , role in ipairs(self.m_roleList) do
		if role:getId() == id then
			table.remove(self.m_roleList,i)
			self.m_roleRemoved[id] = role
			return 
		end
	end
end

function This:removeAllRole()
	for i ,role in ipairs(self.m_roleList) do
		table.remove(self.m_roleList,i)
		self.m_roleRemoved[role:getId()] = role
	end
end

function This:updateRelation()
	for i , role in ipairs(self.m_roleList) do
		local id = role:getId()
		if id == self.m_masterId then
			role:setIsMaster(true)
		end
		for j , allyId in ipairs(self.m_allyList) do
			if id == allyId then
				role:setIsAlly(true)
			end
		end
		for j , collId in ipairs(self.m_collList) do
			if id == collId then
				role:setIsColl(true)
			end
		end
	end
end

function This:updateRole(info)
	for i , role in ipairs(self.m_roleList) do
		if role:getId() == info.role_id then
			role:updateInfo(info)
			break
		end
	end
end

function This:updateInfo(info)
	self.m_id = info.room_id 
	self.m_matchType = info.match_type
	self.m_type = info.relation_type
	self.m_ownerId = info.owner_id
	self.m_masterId = info.master[1] or 0
	self.m_allyList = info.ally
	self.m_collList = info.colleague

	local updateList = {}
	for i , roleInfo in ipairs(info.role_list) do
		updateList[roleInfo.role_id] = roleInfo
	end

	for i = #self.m_roleList, 1 , -1 do
		local role = self.m_roleList[i]
		if not updateList[role:getId()] then
			table.remove(self.m_roleList,i)
			self.m_roleRemoved[role:getId()] = role
		end
	end

	for i , roleInfo in ipairs(info.role_list) do
		local role = self:getRole(roleInfo.role_id)
		if role then
			role:updateInfo(roleInfo)
		else
			self:addRole(roleInfo)
		end
	end

	self:updateRelation()
	self:setOwnerId(self.m_ownerId)
end

function This:isReady()
	for i , role in ipairs(self.m_roleList) do
		if not role:isReady() then
			return false
		end
	end
	return true
end


--endregion
