-- @Author: Su
-- @Date:   2020-04-15 15:15:24
-- @Last Modified by:   Su
-- @Last Modified time: 2020-04-15 15:15:24
-- @File Name: MatchingLevelListLookPanel.lua

local This, super = class(..., "NodeUI")
local MatchManager = require("Match.MatchManager")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local MatchingConfig = require("Metadatas.MatchingConfig")
local MatchOfficeItemView = require("Match.MatchOfficeItemView")

function This:initData(info)

end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MatchingLevelListLook.csb")
    self:addChild(self.m_csNode)

    self.m_scrollView = self.m_csNode.ScrollView
end

function This:onEnter()
    super.onEnter(self)
    MatchManager:addObserver(self)
	GamePlayer:addObserver(self)
    self:refresh()
end

function This:onExit()
    MatchManager:removeObserver(self)
	GamePlayer:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()

    self.m_csNode.boxGet:setTouchEnded(function()
        g_scene:openDialog("Match.MatchOfficeBoxPanel", {level = self.m_level, mode = self.m_mode, state = self.m_state})
    end)
    self.m_csNode.help:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize["guanzhi_help"], node = self.m_csNode.help})
    end)

end

function This:refreshDailyReward()

    self.m_mode = true
    self.m_level = MatchManager:getRankLv()
    self.m_state = MatchManager:isGotDailyReward()
    local hasEff = not self.m_state
    if hasEff then
        if not self.m_csNode.effect.effectList then
            local w ,h = self.m_csNode.boxGet:getContentSize()
            local scale = self.m_csNode.boxGet:getScale()
            local effect1 = GameUtil:createEffect("BoxLing",true)
            effect1:setPosition(w/2,50)
            self.m_csNode.effect:addChild(effect1)
            local effect2 = GameUtil:createEffect("BoxLing2",true)
            self.m_csNode.effect:addChild(effect2)
            effect2:setPosition(w/2,h)
            self.m_csNode.effect.effectList = {effect1,effect2}
        end
    else
        if self.m_csNode.effect.effectList then
            for _ , effect in ipairs(self.m_csNode.effect.effectList) do
                effect:removeFromParent()
            end
            self.m_csNode.effect.effectList = nil
        end
        self.m_csNode.boxGet:setGrayShader()
    end
end

function This:refresh()
    self:refreshDailyReward()
    self.m_csNode.position:setString(MatchingConfig[self.m_level].rank_level)
    self.m_scrollView:removeAllItems()
    self.m_scrollView:setTouchEnabled(true)
    self:setTimer(0.05, function ()
        for i=1, kMaxMatchRankLv do
            local index = kMaxMatchRankLv - i + 1
            local view = MatchOfficeItemView:create(index, MatchingConfig[index])
            local w , h = view:getContentSize()
            self.m_scrollView:pushNode(view, w, h, nil)
            if self.m_level == index then
                self.m_scrollView:jumpToBottom()
            end
        end
    end)
end

function This:onPickUpDailyReward()
    self:refreshDailyReward()
end

function This:onGetRewardInfo()
    self:refresh()
end