-- @Author: Su
-- @Date:   2020-04-16 14:15:24
-- @Last Modified by:   Su
-- @Last Modified time: 2020-04-16 14:15:24
-- @File Name: MatchingLevelListRechargeItemView.lua

local This , super = class(..., "NodeView")
This.csfileName = "MenuScene/NodeUI/MatchingLevelListRechargeItem.csb"

local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local NewActivityManager = require("NewActivity.NewActivityManager")

function This:initData(act,payNum)
    self.m_chapter = act
    self.m_payNum = payNum
    self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
    self.m_csNode = cc.Node:createFromCSFile(self.csfileName)
    self:addChild(self.m_csNode)
    local w , h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)

    self:refesh()
end

function This:refesh()
    local node = self.m_csNode
    node.condition:setString(self.m_chapter:getDesc())
    local limitNum = self.m_chapter:getProgress()
    node.limit:setString(limitNum) 

    local payNum = self.m_payNum
    if payNum >= limitNum then
        node.button:clearShader()
        node.cur:setString(limitNum)
    else
        node.button:setGrayShader()
        node.cur:setString(payNum)
    end
    node.button:setVisible(self.m_chapter:getState() == SevenDayActivityTaskState.kNotReceive )
    node.pickuptext:setVisible(self.m_chapter:getState() ~= SevenDayActivityTaskState.kNotReceive )
    GameUtil:linearLayoutNodes(2, {node.text, node.cur, node.text2, node.limit}, false, nil, 184)

    local rewardList = self.m_chapter:getRewardList()
    local count = #rewardList
    for i, info in ipairs(rewardList) do
    	local itemId, num = info[1], info[2]
    	local itemNode = node["reward" .. tostring(i)]
        if itemNode then
            itemNode:setVisible(true)
    		local node = itemNode.node
			if not node then
		        node = ItemNode:create(itemId, num)
		        node:setAnchorPoint(0.5, 0.5)
		        itemNode:addChild(node)
		        itemNode.node = node
		    else
		        node:init(itemId, num)
            end
        end
    end

    for i=count + 1, 7 do
    	local itemNode = node["reward" .. tostring(i)]
    	if itemNode then
    		itemNode:setVisible(false)
    	end
    end
end

function This:initTouchEvent()
    self.m_csNode.button:setTouch({
        isContainer = true,
        isDynamic = true,
        isScale = false,
        onEnded = function()
            local activity = NewActivityManager:getActivity(NewActivityType.kSeasonRecharge)
            if self.m_payNum >= self.m_chapter:getProgress() and self.m_chapter:getState() == SevenDayActivityTaskState.kNotReceive then
                activity:onPickupActRewardRequest(self.m_chapter:getId())
            end
        end,

    })  

end