-- @Author: Su
-- @Date:   2020-04-15 15:15:24
-- @Last Modified by:   Su
-- @Last Modified time: 2020-04-15 15:15:24
-- @File Name: MatchingLevelListRewardPanel.lua

local This, super = class(..., "NodeUI")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local SeasonRewardConfig = require("Metadatas.SeasonRewardConfig")
local SeasonConfig = require("Metadatas.SeasonConfig")
local GamePlayer = require("Player.GamePlayer")
local ItemNode = require("UI.ItemNode")
local MatchingLevelListRewardItemView = require("Match.MatchingLevelListRewardItemView")

function This:initData(info)

end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MatchingLevelListReward.csb")
    self:addChild(self.m_csNode)

    self.m_scrollView = self.m_csNode.ScrollView

    self.m_progressTimer = cc.ProgressTimer:create(self.m_csNode.jifenbar)
    self.m_csNode.jifenbg:addChild(self.m_progressTimer)
    self.m_progressTimer:setScaleY(-1)
    local w , h = self.m_csNode.jifenbg:getContentSize()
    self.m_progressTimer:setPosition(w/2,h/2)

    self.m_level = MatchManager:getRankLv()
    self:refreshBaseInfo()
    self:initSeasonReward()
    self:refresh()

end

function This:onEnter()
    super.onEnter(self)
    MatchManager:addObserver(self)
    GamePlayer:addObserver(self)
end

function This:onExit()
    MatchManager:removeObserver(self)
	GamePlayer:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
    self.m_scrollView:removeAllItems()
    self.m_scrollView:setTouchEnabled(true)
    self:setTimer(0.05, function ()
        for i=1, kMaxMatchRankLv do
            local index = kMaxMatchRankLv - i + 1
            
            local view = MatchingLevelListRewardItemView:create(index, MatchingConfig[index])
            local w , h = view:getContentSize()
            self.m_scrollView:pushNode(view, w, h, nil)
            if self.m_level == index then
                self.m_scrollView:jumpToBottom()
            end
        end
    end)
end

function This:refreshBaseInfo()
    local rankLv = MatchManager:getRankLv()
    local config = MatchingConfig[rankLv]
    local score = MatchManager:getScore()
    local jifenStr = score
    if rankLv < kMaxMatchRankLv then
        jifenStr = string.format("%d/%d",score,config.score)
    end
    self.m_csNode.jifen:setString(jifenStr)
    local star = MatchManager:getStar()
    local maxStar = MatchingConfig[rankLv].star
    self.m_progressTimer:fromToTimer(1,0,score / config.score)
end

function This:initSeasonReward()
    local seasonId = MatchManager:getSeasonId()
    self.m_csNode.winreward.text_0:setString(string.format(Serialize[61927],seasonId))
    local endTime = MatchManager:getSeasonEndTime()

    local date = os.serverdate("*t",endTime)
    self.m_csNode.winreward.text_0_0:setString(string.format(Serialize[61928],date.year,date.month,date.day))
    local ssconfig = SeasonConfig[seasonId]
    if ssconfig then
        local limitRank = ssconfig.rank_limit
        local limitRankName = MatchingConfig[limitRank].rank_level
        self.m_csNode.winreward.desc:setString(string.format(Serialize[61929], limitRankName))

        local maxRank = MatchManager:getCurSeasonMaxRankLv()
        local item = SeasonRewardConfig[seasonId * 1000 + maxRank].office_reward[1]
        local node = ItemNode:create(item[1],item[2])
        -- node:setAnchorPoint(0.5,0.5)
        -- self.m_csNode.seasonreward.rank_item:addChild(node) 
        
        -- item = ssconfig.reward
        -- node = ItemNode:create(item[1],item[2])
        node:setAnchorPoint(0.5,0.5)
        self.m_csNode.winreward.season_item:addChild(node)
        -- self.m_csNode.seasonreward.des:setString(string.format(Serialize[61925], MatchingConfig[maxRank].rank_level))
    end
	self.m_csNode.winreward.finished:setVisible(false)
end

function This:onGetSeasonInfo(info)
	self.m_csNode.winreward.finished:setVisible(info.win_num >= kSecceedFinishNumLimit)
end