﻿local This = {
    m_maskKey = false,
	m_debugInfo = ""
}
-- print(debug.traceback())
local protobuf = require("protobuf")

local crypt = lib2d.crypt -- loading luaclib/crypt.so

local mode -- socket模式 size,  flag
local state = "invalid"

local challenge
local clientkey
local serverkey
local tokenResult
local token = {}
local secret
local isRegister = false

local function encrypt(message) --加密
    message = crypt.desencode(secret, message)
    return crypt.base64encode(message)
end

local function decrypt(message) --解密
    message = crypt.base64decode(message)
    return crypt.desdecode(secret, message)
end

local session = 0 
local loginIndex = 1
local session2cmd = {}
local convertor = require("Net.MessageConvertor")
local EventDispatcher = require("EventDispatcher")
local serverConfig = require("Net.ServerConfig")


function This:connectLoginServer(ip,port)
    self:lockNetMask("connect_login_server")
    lib2d.socket.connect(ip,port,1)
    mode = "size"
end

function This:connectGameServer(ip,port)
    self:lockNetMask("connect_game_server")
    lib2d.socket.connect(ip,port,2)
    mode = "flag"
end

function This:connectServerSecceed()
    if mode == "size" then
		self:unlockNetMask("connect_login_server")
        EventDispatcher:dispatcher("connect_login_server",true)
    elseif mode == "flag" then
		self:unlockNetMask("connect_game_server")
        EventDispatcher:dispatcher("connect_game_server",true)
    end
end

function This:connectFailed()
    if mode == "size" then
		self:unlockNetMask("connect_login_server")
        EventDispatcher:dispatcher("connect_login_server",false)
    elseif mode == "flag" then
		self:unlockNetMask("connect_game_server")
        EventDispatcher:dispatcher("connect_game_server",false)
    end
end

function This:connectLost()
    EventDispatcher:dispatcher("connect_lost")
end

function This:lockNetMask(key)
	if self.m_maskKey then
		self.m_maskKey = key
	else
		self.m_maskKey = key
		g_scene:lockTouch()
	end
    print("locked net mask :" .. key)
    return true
end

function This:unlockNetMask(key)
    if key == nil or self.m_maskKey == key then
        print("unlock net mask :" .. tostring(key))
        self.m_maskKey = false
        g_scene:unlockTouch()
    end
end

function This:printMsg(package)
	for k , v in pairs(package._fields) do
		print("key=",k.name)
		if type(v) == "table" and  v._fields then
			self:printMsg(v)
		else
			print("value=",v)
		end
	end
end

local function fillSendPackage(descriptor,package,message)
	for i , field_descriptor in ipairs(descriptor.fields) do
		local name = field_descriptor.name
		if field_descriptor.label == 3 then
			for descriptor , value in ipairs(message[name]) do
				if field_descriptor.message_type then
					local v = field_descriptor.message_type._concrete_class()
					table.insert(package[name], v)
					fillSendPackage(field_descriptor.message_type,package[name][descriptor],value)
				else
					table.insert(package[name],value)
				end
			end
		else
			if field_descriptor.message_type then
				fillSendPackage(field_descriptor.message_type,package[name],message[name])
			elseif message[name] ~= nil then
				package[name] = message[name]
			end
		end
	end
end

local function fillRecvMessage(descriptor, message, package)
	for i, field_descriptor in ipairs(descriptor.fields) do
		local name = field_descriptor.name
		if field_descriptor.label == 3 then
			message[name] = {}
			for descriptor , value in ipairs(package[name]) do
				if field_descriptor.message_type then
					message[name][descriptor] = {}
					fillRecvMessage(field_descriptor.message_type,message[name][descriptor],value)
				else
					message[name][descriptor] = value
				end
			end
		else
			if field_descriptor.message_type then
				message[name] = {}
				fillRecvMessage(field_descriptor.message_type,message[name],package[name])
			else
				message[name] = package[name]
			end
		end
	end
end

function This:handleCmd(cmd,data,size)
	self:unlockNetMask(cmd)
	local moduleName = Protocal.getModuleName(cmd)
    if not moduleName then
        print(string.format("协议号:%d所属模块没有写进Protocal",cmd))
        return
    end

	local package, descriptor = Protocal[moduleName]["msgs"..cmd]()
	package:ParseFromString(data)
	local message = {cmd = cmd}
    fillRecvMessage(descriptor, message, package)
	EventDispatcher:dispatcher(cmd, message)
	return message
end

function This:testCmd(cmd,msg)
    local moduleName = Protocal.getModuleName(cmd)
    local package, decriptor = Protocal[moduleName]["msgs"..cmd]()
	fillSendPackage(decriptor,package,msg)
    local data = package:SerializeToString()
    local str = ""
	for i = 1, #data do
		str = str .. " " .. string.byte(data,i)
	end
	print(str)
end

function This:send(cmd, message)
	print(string.format("********************************send cmd : %d*************************",cmd))
    local moduleName = Protocal.getModuleName(cmd)
    local package, decriptor = Protocal[moduleName]["msgc"..cmd]()
	fillSendPackage(decriptor,package,message)
    local data = package:SerializeToString()
    lib2d.socket.sendCmd(cmd, data)
end

function This:sendAndLock(cmd, message)
	assert(type(cmd) == "number")
    self:lockNetMask(cmd)
    self:send(cmd, message)
end

---------------------------------------------------------------
function This:close()
    lib2d.socket.close()
end

return This
