-- @Author: superm
-- @Date:   2018-02-06 21:23:44
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-10 10:07:34
-- @File Name: ActivityBase.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local ActivityTimeConfig = require("Metadatas.ActivityTimeConfig")
local ActivityTypeConfig = require("Metadatas.ActivityTypeConfig")
local EventDispatcher = require("EventDispatcher")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ActivityUtil = require("Activity.ActivityUtil")

function This:init( id )
	super.init(self)
	self.m_escape = 0
    self.m_id = id
	self.m_state = NewActivityState.kNotOpen
    self.m_startTime = 0
    self.m_endTime = 0

	self.m_config = ActivityTimeConfig[self.m_id]
	self.m_descConfig = ActivityTypeConfig[self.m_id]
	self.m_uiconfig = ActivityUtil:getUIConfigList(self.m_id)
	self:initListener()
end

function This:initListener()
	
end

function This:removeListener()
	
end

function This:initData()
	
end

function This:getId()
	return self.m_id
end

function This:getType()
	return self.m_config.type
end

function This:getState()
	return self.m_state
end

function This:getStartTime()
	return self.m_startTime
end

function This:getEndTime()
	return self.m_endTime
end

function This:getOpenLevel()
	return self.m_config.level_min
end

function This:preWork()
	
end

function This:setTime(startTime,endTime)
	self.m_startTime = startTime
	self.m_endTime = endTime
	self:updateState()

	if self.m_endTime > 0 then
		local curTime = GamePlayer:getServerTime()
		--[[if curTime < self.m_endTime and not self.m_isUpdating then
			self.m_upIsUpdating = true
			self.m_escape = 0
			cc.Application:addToUpdateList(self)
		end]]

		if curTime > endTime and self.m_state == NewActivityState.kUpdating then
			self:stop()
		elseif curTime > startTime and (self.m_state == NewActivityState.kNotOpen or self.m_state == NewActivityState.kNotStart) then
			self:start()
		end
	end
end

function This:updateTimeInfo(info)
    local activityId = info.activity_id
    local endTime = info.end_time
    local startTime = info.start_time
    self.m_originalLevel = info.start_level
    self:setTime(startTime,endTime,activityId)
end

function This:work()
	
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	local leftTime = self.m_endTime - curTime
	return math.max(0,leftTime)
end

function This:getState()
	if self.m_endTime >= 0 and self.m_startTime >= 0 then
		local curTime = GamePlayer:getServerTime()
		if curTime > self.m_endTime then
			return NewActivityState.kEnded
		elseif curTime < self.m_startTime then
			return NewActivityState.kNotStart
		end
	end
	return NewActivityState.kUpdating
end

function This:isUpdating()
    return NewActivityState.kUpdating == self:getState()
end

function This:isNotStart()
    return self.m_upIsUpdating == nil
end

function This:updateState()
	if self.m_endTime < 0 then
		self.m_state = NewActivityState.kUpdating
	else
		local state = self:getState()
		if state == NewActivityState.kEnded and self.m_state == NewActivityState.kUpdating then
			self:stop()
		elseif state == NewActivityState.kUpdating and (self.m_state == NewActivityState.kNotStart or self.m_state == NewActivityState.kNotOpen) then
			self:start()
		end 
		self.m_state = state
	end
end

function This:start()
	 if not self.m_upIsUpdating then
		self.m_upIsUpdating = true
		self.m_escape = 0
        cc.Application:addToUpdateList(self)
    end
	NewActivityManager:activityStart(self)
	self:notifyObservers("onActivityStart",self)
end

function This:stop()
	if self.m_upIsUpdating then
        self.m_upIsUpdating = false
        cc.Application:removeFromUpdateList(self)
    end
	self:removeListener()
	NewActivityManager:activityEnded(self)
	self:notifyObservers("onActivityEnded",self)
end

function This:clear()
	self:removeListener()
end

function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1.0
		self:updateState()
		self:notifyObservers("onUpdateActivity",self)
	end
end

function This:getUITypeName(indice)
	if self.m_uiconfig then
		local index = indice or 1
		local config = self.m_uiconfig[index]
		if not config or not config.source then
			return
		end
		return config.source
	end
end

function This:getCSFileName(indice)
	if self.m_uiconfig then
		local index = indice or 1
		local config = self.m_uiconfig[index]
		if not config or not config.csfile then
			return
		end
		return config.csfile
	end
end

function This:getFreshMode()
	if not self.m_descConfig then
		return 0
	end
    return self.m_descConfig.fresh
end

function This:getTitleType()
	if not self.m_descConfig then
		return 0
	end
    return self.m_descConfig.title_type
end

function This:getName()
	if not self.m_descConfig then
		return ""
	end
    return self.m_descConfig.name
end

function This:getPriority()
	if not self.m_descConfig then
		return 0
	end
    return self.m_descConfig.priority
end

function This:hasResTypeWith( value )
	if not self.m_descConfig then
		return false
	end
	if not self.m_descConfig.res_type then
		return false
	end

    return self.m_descConfig.res_type == value
end

function This:isLevelOpen()
	if not self.m_descConfig then
		return false, Serialize[61978]
	end
	if self.m_descConfig.audit and GamePlayer:isAuditVersion() then
		return false, Serialize[61979]
	end

	local level = GamePlayer:getLevel()
    return self.m_descConfig.open_level <= level , string.format(Serialize[61980],level)
end

function This:isSuitable(config, actId, orgLevel)
    if config.activity_id and actId ~= config.activity_id then
        return false
    end
    if orgLevel and orgLevel > 0 and config.level_limit then
        local startLev, endLev = unpack(config.level_limit)
        if startLev > 0 and endLev > 0 and startLev <= endLev then
            return orgLevel >= startLev and orgLevel <= endLev
        end
    end
    return true
end

function This:getOriginalLevel()
    return self.m_originalLevel
end

function This:setOriginalLevel(value)
    self.m_originalLevel = value
end

function This:getAllObjectState()
    return false
end

function This:updateInfo()

end

