--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local ContinuousPayConfig = require("Metadatas.ContinuousPayConfig")
local Socket = require("Net.Socket")

function This:init(id)
	super.init(self, id)
	self.m_continueDay = 0
	self.m_rewardDayList = {}
	self.m_payDayList = {}
	self.m_curRewardDay = 0
	self.m_activityDay = 0
	self.m_totalPayTime = 0

	self.m_configList = {}
	self.m_maxPayNum = 0
end

function This:initListener()
	EventDispatcher:addListener(self, 12029, self.onGetActivityInfoReply)
	EventDispatcher:addListener(self, 12004, self.onGetRewardActivityReply)
	EventDispatcher:addListener(self, "day_passed", self.onDayPassNotify)
end

function This:removeListener()
	EventDispatcher:removeListener(self, 12029)
	EventDispatcher:removeListener(self, 12004)
	EventDispatcher:removeListener(self, "day_passed")
end

function This:updateTimeInfo( info )
	super.updateTimeInfo(self, info)

	self.m_configList = {}
	self.m_maxPayNum = 0
	for _, config in ipairs(ContinuousPayConfig) do
		if self:isSuitable(config, self.m_id, self.m_originalLevel) then
			if self.m_maxPayNum < config.pay_num then
				self.m_maxPayNum = config.pay_num
			end
			table.insert(self.m_configList, config)
		end
	end
end

function This:getTotalPayTime()
	return self.m_totalPayTime
end

function This:getMaxPayTime()
	return self.m_maxPayNum
end

function This:isPay(index)
	return self.m_payDayList[index]
end

function This:getCurRewardDay()
	return self.m_curRewardDay
end

function This:getActivityDay()
	return self.m_activityDay
end

function This:isCanReward()
	return self:getCurRewardDay() <= self:getTotalPayTime()
end

function This:getRewardList(index)
	local result = {}
	if self.m_configList[index] then
		for i , info in ipairs(self.m_configList[index].everyday_reward) do
			table.insert(result,{id = info[1],num = info[2]})
		end
	end
	return result
end

function This:hasBoxReward(index)
	if self.m_configList[index] then
		return #self.m_configList[index].extra_reward > 0
	end
end

function This:hasRewardBox(index)
	return self.m_rewardDayList[index]
end

function This:getBoxRewardList(index)
	local result = {}
	if self.m_configList[index] then
		for i , info in ipairs(self.m_configList[index].extra_reward) do
			table.insert(result,{id = info[1],num = info[2]})
		end
	end
	return result
end

--------------------------------------------

function This:onGetActivityInfoRequest()
	Socket:sendAndLock(12029,{activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
	if self:getId() ~= msg.activity_id then
		return
	end

	if msg.err_code == 0 then
		-- msg = {day = {7,6,5,4,3,2,1}, reward_list= {}, activity_day=7}
		self.m_rewardDayList = {}
		self.m_payDayList = {}
		self.m_curRewardDay = 1
		self.m_totalPayTime = 0
		self.m_activityDay = msg.activity_day

		for i , rewardId in ipairs(msg.reward_list) do
			local day = (rewardId - 1) % 7 + 1
			self.m_rewardDayList[day] = true
			self.m_curRewardDay = self.m_curRewardDay + 1
		end
		-- if self.m_curRewardDay > #self.m_configList then
		-- 	self.m_curRewardDay = #self.m_configList
		-- end

		for _, day in ipairs(msg.day) do
			self.m_payDayList[day] = true
			self.m_totalPayTime = self.m_totalPayTime + 1
		end
		self:notifyObservers("onGetActivityInfo",self:getId())
		EventDispatcher:dispatcher("continuous_pay_refresh")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetRewardActivityRequest()
	-- local config = self.m_configList[self.m_curRewardDay]
	-- if not config then return end
	-- local actItemId = config.id
	local actItemId = self.m_curRewardDay
	Socket:sendAndLock(12004,{activity_id = self:getId(), id = actItemId, type = self:getType()})
end

function This:onGetRewardActivityReply(msg)
	if self:getId() ~= msg.activity_id then
		return
	end

	if msg.err_code == 0 then
		self.m_rewardDayList[self.m_curRewardDay] = true
		local result = self:getRewardList(self.m_curRewardDay)
		if self:hasBoxReward(self.m_curRewardDay) then
			local tmp = self:getBoxRewardList(self.m_curRewardDay)
			for i , info in ipairs(tmp) do
				table.insert(result,info)
			end
		end
		g_scene:showReward(result)
		self.m_curRewardDay = self.m_curRewardDay + 1
		self:notifyObservers("onGetRewardActivity")
		EventDispatcher:dispatcher("continuous_pay_reward")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onDayPassNotify(msg)
	self:onGetActivityInfoRequest()
end

function This:getUITypeName()
    return "NewActivity.ContinuePayPanel"
end

-- function This:getUIFileName()
-- 	return "MenuScene/NodeUI/ActivityContinuousPay.csb"
-- end

function This:getRewardPackage()
    return {{113,20},{113,50},{151,20},{151,50},{101,100000},{101,200000},{101,1000000},{121,10},{121,20},{121,100}}
end

--endregion
