--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local DaySignConfig = require("Metadatas.DaySignConfig")
local CumulativeSignConfig = require("Metadatas.CumulativeSignConfig")
local VipManager = require("Vip.VipManager")

function This:init()
	super.init(self,NewActivityType.kSignInDaily)
	self.m_hasReward = false
	self.m_dailyReward = {}
	self.m_rewardCount = 0
end

function This:initListener()
	EventDispatcher:addListener(self,12025,self.onGetActivityInfoReply)
	EventDispatcher:addListener(self,12026,self.onRewardSignInReply)
	EventDispatcher:addListener(self,"day_passed",self.onDayPassNotify)
end

function This:removeListener()
	EventDispatcher:removeListener(self,12025)
	EventDispatcher:removeListener(self,12026)
	EventDispatcher:removeListener(self,"day_passed")
end

function This:getState()
	local state = super.getState(self)
	return state
end

function This:setTime(startTime,endTime)
	super.setTime(self,startTime,endTime)
end

function This:getRewardDays()
	return self.m_rewardCount
end

function This:hasReward(index)
	return index <= self.m_rewardCount
end

function This:isCanReward(index)
	index = index or self.m_rewardCount + 1
	return not self.m_hasReward and index == self.m_rewardCount + 1
end

function This:getRewardConfig(index)
	return DaySignConfig[index] or {}
end

function This:getRewardList(index,bVip)
	local config = DaySignConfig[index]
	local result = {id = config.reward[1],num = config.reward[2]}

	if bVip and self:isVipDouble(index) then
		result.num = result.num * 2
	end
	return result
end

function This:hasBoxReward(index)
	local config = self:getBoxConfig(index)
	return self.m_dailyReward[config.days]
end

function This:getBoxRewardList(index)
	local config = CumulativeSignConfig[index]
	local result = {}
	if config then
		for i , info in ipairs(config.reward) do
			table.insert(result,{id = info[1],num = info[2]})
		end
	end
	return result
end

function This:getBoxConfig(index)
	local config = CumulativeSignConfig[index]
	return config or {}
end

function This:isRewardDouble(index)
	local config = self:getRewardConfig(index)
	for i , vip in ipairs(config.if_double) do
		if vip > 0 then
			return true
		end
	end
end

function This:isVipDouble(index)
	local config = self:getRewardConfig(index)
	local vipState = VipManager:getMemberState()
	for i , vip in ipairs(config.if_double) do
		if vipState[vip] then
			return true
		end
	end
end

-------------------------------------------------------
function This:onGetActivityInfoRequest()
	Socket:send(12025,{activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
	if msg.err_code == 0 then
		self.m_dailyReward = {}
		self.m_rewardCount = 0
		self.m_hasReward = not msg.have_reward
		for i , id in ipairs(msg.reward_list) do
			self.m_dailyReward[id] = true
			self.m_rewardCount = self.m_rewardCount + 1
		end
		self:notifyObservers("onGetSignInfo")
		EventDispatcher:dispatcher("daily_sign_refresh")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onRewardSignInRequest()
	Socket:sendAndLock(12026,{activity_id = self:getId()})
end

function This:onRewardSignInReply(msg)
	if msg.err_code == 0 then
		self.m_rewardCount = self.m_rewardCount + 1
		self.m_hasReward = true

		local result = {}
		local t = {}

		local reward = self:getRewardList(self.m_rewardCount,true)
		t[reward.id] = reward.num
		for i , config in pairs(CumulativeSignConfig) do
			if self.m_rewardCount == config.days and not self.m_dailyReward[config.days] then
				for _,info in ipairs(config.reward) do
					local id = info[1]
					local num = info[2]
					t[id] = (t[id] or 0) + num
				end
			end
		end
		
		for id , num in pairs(t) do
			table.insert(result,{id = id,num = num})
		end
		
		self.m_dailyReward[self.m_rewardCount] = true
		g_scene:showReward(result)
		self:notifyObservers("onRewardDailySign",self.m_rewardCount)
		EventDispatcher:dispatcher("daily_sign_reward")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onDayPassNotify()
	self:onGetActivityInfoRequest()
end

function This:getUITypeName()
    return "NewActivity.DailySignTabNode"
end

--endregion
