-- @Author: Su
-- @Date:   2019-06-05 15:39:14
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-05 15:39:14
-- @File Name: ActivityDecreeTaskItem.lua
-- @Description: 
local This , super = class(...,"NodeUI")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

function This:initNode(info)
    self.m_id = info.id
    self.m_activity = info.activity
    self.m_escape = 0
    self.m_config = self.m_activity:getConfigById(self.m_id)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/DecreeTaskItem.csb")
    self.m_Node = self.m_csNode
    self:addChild(self.m_csNode)
    self.m_csNode.time:setString(string.format(Serialize[61994],self.m_config.time/60))
    self:refresh()


    self.m_csNode.item:removeAllChildren()
    for i,info in ipairs(self.m_config.reward) do
        local item = ItemNode:create(info[1],info[2])
        item:setAnchorPoint(0,0)
        item:setPosition(-45,-50)
        self.m_csNode.item:addChild(item)
    end
end

function This:getSize()
    local w,h = self.m_csNode.bg:getContentSize()
    return w,h
end

--function This:onEnter()
--    super.onEnter(self)
--    self.m_activity:addObserver(self)
--end
--
--function This:onExit()
--    self.m_activity:removeObserver(self)
--    super.onExit(self)
--end

function This:refresh()
    local node =self.m_Node
    local color = QualityColor[self.m_config.quality]
    node.title:setColor(color.r,color.g,color.b)
    node.title:setString(self.m_config.name)
    node.jifen:setString(string.format( Serialize[61995],self.m_config.jifen))
    node.neirong:setString(self.m_config.des)

    local progress = self.m_activity:getTaskProgress(self.m_id)
    if progress > self.m_config.progress then
        progress = self.m_config.progress
    end
    node.jindu:setStringWithColor(string.format( Serialize[61996],progress,self.m_config.progress))

    local isReceive = self.m_activity:isReceive(self.m_id)
    local isReceiveReward = self.m_activity:isTaskRewardReceive(self.m_id)
    local isFinish = self.m_activity:isFinish(self.m_id)
    local isOver = self.m_activity:isTaskOverTime(self.m_id)
    local canReceiveNum = self.m_activity:getTaskCanReceiveNum()
    node.keling:setVisible(isReceive and isFinish and not isReceiveReward)
    node.ing:setVisible(isReceive and not isFinish and not isOver)
    node.jiequ:setVisible(not isReceive)
    node.guoqi:setVisible(isOver and not isFinish)
    node.time:setVisible(not isOver)

    if self.m_activity:isReceive(self.m_id) and not isOver then
        self:scheduleUpdate()
    else
        self:unscheduleUpdate()
    end
end

function This:initTouchEvent()
    local node = self.m_csNode
    node.jiequ:setTouch({
        isScale = false, 
        isDynamic = true, 
        isContainer  = true,
        onEnded = function()   
            local num = self.m_activity:getHasFinishTaskNum()
            if num >= 10 then
                g_scene:showTips(Serialize[61997])
                return
            end

            local canReceiveNum = self.m_activity:getTaskCanReceiveNum()
            local isReceive = self.m_activity:isReceive(self.m_id)
            if canReceiveNum > 0 and not isReceive then
                self.m_activity:onReceiveTaskRequest(self.m_id)
            else
                g_scene:showTips(Serialize[61998])
            end
    end})

    node.keling:setTouch({
        isScale = false, 
        isDynamic = true, 
        isContainer  = true,
        onEnded = function()    
            self.m_activity:onGetTaskRewardRequest(self.m_id)
    end})

end


function This:refreshTimeInfo()
    local node = self.m_csNode
    local lessTime = self.m_activity:getTaskStartTime(self.m_id) + self.m_config.time - GamePlayer:getServerTime()
    if lessTime < 0 then
        self:unscheduleUpdate()
        lessTime = 0
    end
    node.time:setString(string.format(Serialize[61994], math.ceil( lessTime / 60 )))
    local isOver = self.m_activity:isTaskOverTime(self.m_id)
    node.time:setVisible(not isOver)
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape > 1 then
		self.m_escape = self.m_escape - 1
		self:refreshTimeInfo()
	end
end

function This:onDecreeTaskStateRefresh(id)
    if id and id == self.m_id then
        self:refresh()
    end
end
