-- @Author: Su
-- @Date:   2019-06-04 09:59:14
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-04 09:59:14
-- @File Name: ActivityDecreeTaskNode.lua
-- @Description: 

local This , super = class(...,"NodeUI")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ActivityDecreeTaskItem = require("NewActivity.ActivityDecreeTaskItem")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ArgConfig = require("Metadatas.ArgConfig")


function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/DecreeTask.csb")
    self:addChild(self.m_csNode)
    -- self.m_csNode:setToCenter()
    self.m_csNode.jindutiao:setScaleY(-1)
    self.m_activity =NewActivityManager:getActivity(NewActivityType.kDecreeTask)
    self.m_progressTimer = cc.ProgressTimer:create(self.m_csNode.jindutiao)
    self.m_csNode.yuanbg:addChild(self.m_progressTimer)
    self.m_progressTimer:setScaleY(-1)
    local w , h = self.m_csNode.yuanbg:getContentSize()
    self.m_progressTimer:setPosition(w/2,h/2)
    -- self.m_activity:onGetActivityInfoRequest()
    self:refresh()
    -- self:initTouchEvent()
end

 function This:onEnter()
     super.onEnter(self)
     GamePlayer:addObserver(self)
     self.m_activity:addObserver(self)
     self.m_activity:onGetActivityInfoRequest()
     self.m_progressTimer:fromToTimer(1,0,self.m_activity:getOfficeNum() / ArgConfig["decree_office_limit"].arg[1])
 end

function This:onExit()
    GamePlayer:addObserver(self)
    self.m_activity:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    local clickRefresh = function()      
        local refreshTimes, refrshMaxTimes = self.m_activity:getRefreshTimes()
        if refreshTimes <= 0 then
            local cost = self.m_activity:getRefreshCost()
            local panel = g_scene:openDialog("UI.CostConfirmPanel", {
                cost = cost,
                notips = "StoreCommon58",
                desc = Serialize[60860], 
                okHandler = function(build)
                    self.m_activity:onRefreshTaskRequest()
                end } )
        else
            self.m_activity:onRefreshTaskRequest()
        end
    end

    local node = self.m_csNode
    node.shuaxin:setTouchEnded(function ()
        if self.m_activity:hasTaskRewardCanReceive() then
            g_scene:showTips(Serialize[62003])
            return
        end
        if self.m_activity:hasTaskNotFinish() then
            local panel = g_scene:openDialog("UI.CostConfirmPanel", {
                okHandler = function(build)
                    clickRefresh()
                end 

            })
            panel:setSimpleString(Serialize[62004],500)
            return
        end
        clickRefresh()
    end)

    node.gantanhao:setTouchEnded(function ()
        g_scene:openDialog("UI.HelpTips",{text = Serialize[61987],node = node.gantanhao})
    end)

    node.info:setTouchEnded(function()
        g_scene:openDialog("NewActivity.ActivityDecreeTaskLvPanel",{})
    end)
end

function This:refreshTask()
    local cNode = self.m_csNode
    local scrollView = cNode.ScrollView
    scrollView:setDirection("vertical")
    scrollView.m_direction ="down"
    scrollView:removeAllItems()
    local turnDatas = self.m_activity:getTurnDatas()
    for i=1,#turnDatas do
        if not self.m_activity:isTaskRewardReceive(turnDatas[i]) then
            local item = ActivityDecreeTaskItem:create({id =turnDatas[i],activity = self.m_activity})
            local w,h = item:getSize()
            scrollView:pushBackCustomItem(item,w,h,0.5)
        end
    end
end

function This:refreshTime()
    local node = self.m_csNode
    local leftTime = self.m_activity:getEndTime() - GamePlayer:getServerTime()
    local refreshTimes, refrshMaxTimes = self.m_activity:getRefreshTimes()
    local lastTime,intervalTime = self.m_activity:getLastRefresTime()

    node.time:setString(GameUtil:formatDTime(Serialize[60856],leftTime))
    if refreshTimes >= 0 and refreshTimes < refrshMaxTimes then
        local time = intervalTime -(GamePlayer:getServerTime() - lastTime)
        if time < 0.1 then
            time = 0
            self.m_activity:onGetActivityInfoRequest()
        end
        node.shuaxintime:setString(GameUtil:formatTime(Serialize["H_M_S"],intervalTime -(GamePlayer:getServerTime() - lastTime)))
    end
end

function This:refreshTimeNode()
    local cNode = self.m_csNode
    local refreshTimes, refrshMaxTimes = self.m_activity:getRefreshTimes()
    local lastTime,intervalTime = self.m_activity:getLastRefresTime()
    cNode.text2:setVisible(refreshTimes < refrshMaxTimes)
    cNode.shuaxintime:setVisible(refreshTimes < refrshMaxTimes)
    if refreshTimes > 0 then
        cNode.shuaxin.text:setString(Serialize[62005] .. refreshTimes .. ")")
    else
        cNode.shuaxin.text:setString(Serialize[62006])
    end
    cNode.dengjinum:setString(self.m_activity:getDecreeStage())
    cNode.jifen:setStringWithColor(string.format( Serialize[61988],self.m_activity:getDecreeNum(),self.m_activity:getLostNum()))
    cNode.jifennum:setStringWithColor(string.format( "<New5>%d<New8>/%d",self.m_activity:getOfficeNum(),ArgConfig["decree_office_limit"].arg[1]))
    cNode.jindubg.jindutiao:setPercent(self.m_activity:getPercent())
    --cNode.kezhuanjifennum:setString(string.format( Serialize[61991],self.m_activity:getOfficeNum()))
    cNode.renwunum:setStringWithColor(string.format( Serialize[62007],self.m_activity:getHasFinishTaskNum()))
end

function This:refresh()
    self:refreshTimeNode()
    self:refreshTask()
end


function This:onUpdateActivity(activity)
    local type = activity:getId()
    if type == NewActivityType.kDecreeTask then
        self:refreshTime()
    end
end

function This:onDecreeRefresh()
    self:refresh()
end

function This:onDecreeTaskStateRefresh()
    self:refresh()
end