-- @Author: superm
-- @Date:   2018-04-21 11:37:10
-- @Last Modified by:   superm
-- @Last Modified time: 2018-04-25 15:07:47
-- @File Name: ActivityEntireConsume.lua
-- @Description: 


local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local SeverConsumptionConfig = require("Metadatas.SeverConsumptionConfig")
local Socket = require("Net.Socket")

function This:init()
	super.init(self, NewActivityType.kEntireConsume)

	local itemList = {}
	self.m_maxMoney = 0
	for id, config in ipairs(SeverConsumptionConfig) do
		itemList[id] = { 
			id = id,
			state = ActObjectState.kLocked, 
			money = config.money, 
			reward = config.reward_list 
		}
		self.m_maxMoney = config.money
	end
	self.m_rewardItemList = itemList
end

function This:initListener()
	EventDispatcher:addListener(self,12030,self.onGetActivityInfoReply)
	EventDispatcher:addListener(self,12004,self.onGetRewardActivityReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,12030)
	EventDispatcher:removeListener(self,12004)
end

function This:setTime(startTime,endTime)
	super.setTime(self,startTime,endTime)
end


function This:isCanReward()
	for id, act in ipairs(self.m_rewardItemList) do
		if act.state == ActObjectState.kNormal then
			return true
		end
	end

	return false
end

function This:getDefaultItem()
	local ret, backup, last
	for id, act in ipairs(self.m_rewardItemList) do
		last = act
		if act.state == ActObjectState.kNormal then
			ret = act
			break
		elseif not backup and act.state == ActObjectState.kLocked then
			backup = act
		end
	end

	return ret or backup or last
end

function This:isBoxRewarded(index)
	return self.m_rewardItemList[index].state == ActObjectState.kCompleted
end

function This:hasRewardBox(index)
	return self.m_rewardItemList[index].state == ActObjectState.kNormal
end

function This:getBoxRewardList(index)
	return self.m_rewardItemList[index].reward
end

function This:getBoxTargetNum(index)
    return self.m_rewardItemList[index].money
end

function This:getMaxBoxTargetNum()
    return self.m_maxMoney
end

function This:getSelfConsumption()
	return self.m_selfConsumption
end

function This:getTotalConsumption()
	return self.m_totalConsumption
end

function This:getConsumptionProgress()
	return self.m_totalConsumption / self.m_maxMoney
end

function This:updateInfo()
    self:onGetActivityInfoRequest()
end

--------------------------------------------

function This:onGetActivityInfoRequest()
	Socket:sendAndLock(12030,{activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
	if msg.activity_id ~= self:getId() then
		return
	end

	if msg.err_code == 0 then
		self.m_selfConsumption = msg.personal_consumption
		self.m_totalConsumption = msg.all_consumption

		for id, act in ipairs(self.m_rewardItemList) do
			if act.money <= self.m_totalConsumption then
				act.state = ActObjectState.kNormal
			end
		end

		for _, id in ipairs(msg.reward_list) do
			local act = self.m_rewardItemList[id]
			if act then
				act.state = ActObjectState.kCompleted
			end
		end

		self:notifyObservers("onGetActivityInfo",self:getId())
		EventDispatcher:dispatcher("entire_consume_refresh")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetRewardActivityRequest(actItemId)
	Socket:sendAndLock(12004, {activity_id = self:getId(), id = actItemId, type = self:getType()})
end

function This:onGetRewardActivityReply(msg)
	if msg.activity_id ~= self:getId() then
		return
	end

	if msg.err_code == 0 then
		local act = self.m_rewardItemList[msg.id]
		if act then
			act.state = ActObjectState.kCompleted
		end
		g_scene:showReward(act.reward)
		self:notifyObservers("onGetRewardActivity")
		EventDispatcher:dispatcher("entire_consume_reward")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:getUITypeName()
    return "NewActivity.EntireConsumePanel"
end