-- @Author: superm
-- @Date:   2018-03-14 17:23:43
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-10 10:01:08
-- @File Name: ActivityHappyPlay.lua
-- @Description: 


local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local OneArmBoxConfig = require("Metadatas.OneArmBoxConfig")
local OneArmItemConfig = require("Metadatas.OneArmItemConfig")
local GameUtil = require("Util.GameUtil")
HappyTurnBoxState = {
	kLock = 1,
	kOpen = 2,
	kClose = 0
}

function This:init(id)
	super.init(self, id)
	if self.m_descConfig then
	    self.m_startTime = self.m_descConfig.start_time or 0
	    self.m_endTime = self.m_descConfig.end_time or 0
	end
	self.m_playCount = 0
	self.m_msgStr = ""
	self.m_isCanReward = false--是否有宝箱可以领取
	self.m_rewardHistory = {} --已经领取的宝箱列表
	self.m_turnRewardList ={} --
	self.m_turnPlayData ={}--转盘奖励
	self.m_turnPlayResult ={}--转盘结果
	local rewardConfig = self:getRewardConfig()
	for index, config in ipairs(rewardConfig) do
		self.m_turnRewardList[index] = {
			id = config.id, 
			state = HappyTurnBoxState.kLock, 
			reward = config.reward, 
			times = config.times
		}
	end
end

function This:initListener()
	EventDispatcher:addListener(self, 12038, self.onGetActivityInfoReply)
	EventDispatcher:addListener(self, 12039, self.onGetTurnPlayReply)
	EventDispatcher:addListener(self, 12040, self.onGetTurnBoxReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,12038)
	EventDispatcher:removeListener(self,12039)
	EventDispatcher:removeListener(self,12040)
end

--获取转盘数据可能数据
function This:getTurnDatas(  )
	local activityId = self:getId()
	local res ={}
	for i,config in pairs(OneArmItemConfig) do
		if config.activity_id == activityId then
			if not res[config.position] then
				res[config.position] = {}
			end
			local item = res[config.position]
			table.insert(item,config.item)
		end
	end
	return table.clone(res)
end

--获取珍宝预览
function This:getZhenBaoList(  )
	local res ={}
	local activityId = self:getId()
	for i,config in pairs(OneArmItemConfig) do
		if config.activity_id == activityId and config.is_exhibition == 1 then
			table.insert(res,config.item)
		end
	end
	return table.clone(res)
end

function This:isCanReward()
	return self.m_isCanReward
end

function This:getMsgStr( )
	return self.m_msgStr
end

function This:playCount(  )
	return self.m_playCount
end

function This:getRewardHistory(  )
	return self.m_rewardHistory
end

function This:getRewardList( )
	return GameUtil:shallowCopy(self.m_turnRewardList)
end

function This:getTurnPlayResult(  )
	return self.m_turnPlayResult
end

function This:getTurnPlayData(  )
	-- return {{101,90},{101,90},{101,90},{101,90},{101,90},{101,90}}
	return self.m_turnPlayData
end

function This:getPanelName()
	return "Activity.ActivityBingFaPanel"
end

function This:getUITypeName()
    return "NewActivity.ActivityHappyTurnTabNode"
end

--领取宝箱奖励
function This:onGetTurnBoxRequest(box_id)
	Socket:sendAndLock(12040,{activity_id = self:getId(),box_id = box_id})
end

function This:onGetTurnBoxReply( msg )
	if msg and msg.activity_id ~= self:getId() then
		return
	end
    if msg.err_code == 0 then
    	local rewardConfig = self:getRewardConfig()
        local rewardList = rewardConfig[msg.box_id].reward
		g_scene:showReward(rewardList)

		local rewardHistory = msg.reward_list or {}
		self.m_rewardHistory= rewardHistory
		self:refreshturnRewardList()

        self:notifyObservers("onGetTurnBox")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

--请求抽奖
function This:onGetTurnPlayRequest(time)
	Socket:send(12039,{activity_id = self:getId(),draw_times= time})
end

function This:onGetTurnPlayReply( msg )
	if msg and msg.activity_id ~= self:getId() then
		return
	end
    if msg.err_code == 0 then
    	local turnPlayData = {}
    	local temturnPlayData = msg.item_list or {}
		self.m_playCount = msg.draw_times or 0
    	local temTurnPlayResult ={}
    	for i,item in pairs(temturnPlayData) do
    		local temItem ={item.id,item.num}
    		table.insert(turnPlayData,temItem)
    		if temTurnPlayResult[temItem[1]] then
    			temTurnPlayResult[temItem[1]] = temTurnPlayResult[temItem[1]] + temItem[2]
    		else
    			temTurnPlayResult[temItem[1]] = temItem[2]
    		end
    	end
    	local turnPlayResult = {}
    	for key,num in pairs(temTurnPlayResult) do
    		local temItem ={key,num}
    		table.insert(turnPlayResult,temItem)
    	end
    	self.m_turnPlayData = turnPlayData
    	self.m_turnPlayResult = turnPlayResult
    	self:refreshturnRewardList()
        self:notifyObservers("onGetTurnPlay",msg.draw_times)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 获取转盘信息
function This:onGetActivityInfoRequest()
	Socket:send(12038,{activity_id = self:getId()})
end

function This:getRewardConfig()
	return OneArmBoxConfig
end

function This:refreshturnRewardList()
	local rewardHistory = self.m_rewardHistory
    table.sort(rewardHistory,function(a,b)
        return a < b
    end)

    self.m_isCanReward = false
    local maxCanRewardIndex =0
    local rewardConfig = self:getRewardConfig()
    for i,item in ipairs(rewardConfig) do
        if item and item.times<=self.m_playCount then
            maxCanRewardIndex = i
            self.m_isCanReward = true
        end
    end
    for index=1,maxCanRewardIndex do
    	self.m_turnRewardList[index].state = HappyTurnBoxState.kOpen
    end
    for _,configIndex in ipairs(rewardHistory) do
    	if self.m_turnRewardList[configIndex] then
    		self.m_turnRewardList[configIndex].state = HappyTurnBoxState.kClose
		elseif maxCanRewardIndex >= configIndex then
			self.m_isCanReward = true
			self.m_turnRewardList[configIndex].state = HappyTurnBoxState.kOpen
		else
			self.m_turnRewardList[configIndex].state = HappyTurnBoxState.kLock
    	end
    end
    if maxCanRewardIndex ==table.count(rewardHistory) then
    	self.m_isCanReward = false
    end
	EventDispatcher:dispatcher("activity_happyturn_state_changed")
end

function This:onGetActivityInfoReply( msg )
	if msg and msg.activity_id == self:getId() then
		if msg.err_code == 0 then
			self.m_playCount = msg.times or 0
			local strList = msg.msg_list or {}
			local msgStr =""
			for i,str in ipairs(strList) do
				msgStr = msgStr ..str.."\n"
			end
			local rewardHistory = msg.reward_list or {}
		    self.m_rewardHistory= rewardHistory
		    self:refreshturnRewardList()
			self.m_msgStr = msgStr

			self:notifyObservers("onGetHappyInfo", self:getId())
		else
        	g_scene:handleErrorCode(msg.err_code)
		end
	end
end

function This:updateInfo()
    self:onGetActivityInfoRequest()
end