-- @Author: zheng
-- @Date:   2018-08-06 15:24:41
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-03-05 18:03:14
-- @File Name: ActivityHappyTurnTabNode.lua

local This , super = class(...,"NodeUI")
local ActivityHappyTurnNode = require("NewActivity.ActivityHappyTurnNode")
local NewActivityManager = require("NewActivity.NewActivityManager")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local OneArmBoxConfig = require("Metadatas.OneArmBoxConfig")
local playAnimNum =3
local strList ={
    day =Serialize[62017],
    [NewActivityType.kLuckyRoulette] =Serialize[62018],
    [NewActivityType.kBingFaRoulette] =Serialize[62019],
    bingFaCost = Serialize[62020]
}


function This:initData(queue, dialog)
    self.m_dialog = dialog
end

function This:initNode(info)
    local activityType = NewActivityType.kLuckyRoulette
    if info and info.activityType then
        activityType = info.activityType
    end

    self.m_activityType = activityType
    self.m_activity = NewActivityManager:getActivity(activityType)

    local csNode = nil
    if info and info.csNode then
        csNode = info.csNode
        self.m_activity:onGetActivityInfoRequest()
    else
        csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ActivityHappyTurnTable.csb")
        self:addChild(csNode)
    end

    self.m_csNode = csNode
    local turntable = self.m_csNode.turntable
    local turnNodes = {}
    local turnDatas = self.m_activity:getTurnDatas()
    for index=1,playAnimNum do
        local parent = turntable["ItemPanel"..index]
        local infor ={node = parent,itemList = turnDatas[index],callFunc = function (index2)
                 self:turnEndCallBack( index2 )
            end }
        local turnNode = ActivityHappyTurnNode:create(infor)
        turnNode:setPosition(56, 116)
        turnNode:setScale(0.95)
        parent:addChild(turnNode)
        table.insert(turnNodes,turnNode)
    end

    local function getConfigData(configId)
        local DailyAchieveManager = require("Achieve.DailyAchieveManager")
        local crusadeData = DailyAchieveManager:getCrusadeData()
        local levelLimit = crusadeData:getCrusadeConfigNum(configId)
        return levelLimit
    end
    local itemIdStr = string.format("activity_%d_item_id",activityType)
    local itemPrice = string.format("activity_%d_item_worth",activityType)
    self.m_turntableNode = turntable
    self.m_turnNodes = turnNodes
    self.m_usedPrice = getConfigData(itemPrice)
    self.m_useItemId = getConfigData(itemIdStr)
    self:refreshLabel()
    self.m_playCount = 0 --播放了多少次转盘
    self.m_isPlayAnim = false--转盘是否正在转
    if activityType ~= NewActivityType.kLuckyRoulette then
        turntable.jiahaodi1:setVisible(false)
    end
end

function This:refreshLabel( )
    self:updateLiJuanLabel(  )
    self.m_turntableNode.number2:setString("" .. GamePlayer:getDiamond())--当前元宝
    local useDiamond = self.m_usedPrice
    self.m_turntableNode.number4:setString(string.format("%d",useDiamond))--转动1次消耗元宝
    self.m_turntableNode.number6:setString(string.format("%d",useDiamond*10))--转动10次消耗元宝

    local playCount = self.m_activity:playCount()
    local rewardHistory = self.m_activity:getRewardHistory()
    local maxCanRewardIndex =1

    for i,item in ipairs(OneArmBoxConfig) do
        if item and item.times<=playCount then
            maxCanRewardIndex = i
        end
    end
    local allItemCount = table.count(OneArmBoxConfig)
    local rewardCount = table.count(rewardHistory)
    local str = ""
    if allItemCount == rewardCount then
        str = string.format("%s%d",strList.day,playCount)
    elseif maxCanRewardIndex>=1 and maxCanRewardIndex <= allItemCount  then
        local minCanRewardIndex =nil
        for index1=1,maxCanRewardIndex do
            local isFind = table.find(rewardHistory, function (index, value)
                return index1 == value
            end)
            if not isFind then
                minCanRewardIndex = index1
                break
            end
        end
         if not minCanRewardIndex then
             minCanRewardIndex = maxCanRewardIndex+1
         end
        local configItem = OneArmBoxConfig[minCanRewardIndex]
        str = string.format("%s%d/%d",strList.day,playCount,configItem.times)
    end
    self.m_turntableNode.numbercishu:setString(str)

    local msgStr = self.m_activity:getMsgStr( )
    self:resetScoolSize2(self.m_turntableNode.scrollView,200,150,msgStr)
    self:createEffect(self.m_activity:isCanReward())
end

function This:createEffect(isAdd)
    local function createAnim( parent,anmStr )
        local GameUtil = require("Util.GameUtil")
        local effect1 = GameUtil:createEffect(anmStr,true)
        parent:addChild(effect1,-1)
        return effect1
    end
    local node = self.m_turntableNode.box11
    node:removeAllChildren()
    local textures ={"MenuScene/boxLock2.png","MenuScene/boxGet2.png"}
    local texture = textures[1]
    if isAdd then
        texture = textures[2]
        local effect1 = createAnim(node,"BoxLing")
        local w ,h = node:getContentSize()
        effect1:setPosition(w/2, h/2)
        local effect2 = createAnim(node,"BoxLing2")
        effect2:setPosition(w/2, h/2)
    end
    node:setImage(texture)
end

function This:turnEndCallBack( index )
    print("turnEndCallBack",index)
     if index and index ==playAnimNum  then
        for i,item in ipairs(self.m_turnNodes) do
            item:refreshTimes( )
        end
        local turnData = self.m_activity:getTurnPlayData()
        local turnDataCount = #turnData
        local palyCount = self.m_playCount
        if palyCount*playAnimNum == turnDataCount then
            self:showReward(  )
            return
        end
        palyCount = palyCount+1
        self:runAction(cc.Sequence(cc.DelayTime(0.5), cc.CallFunc(function ()
            self:starTurnWithTime(palyCount)
        end)))
    end
end

function This:starTurnWithTime(count)
    local turnData = self.m_activity:getTurnPlayData()
    local res = {}
    local starIndex = count*playAnimNum-3
    local endIndex = starIndex+playAnimNum
    if endIndex>#turnData then
        return
    end
    self.m_playCount = count
    if starIndex<1 then
        starIndex = 1
    end
    for i=starIndex,endIndex do
        local item= table.clone(turnData[i])
        table.insert(res,item)
    end
    self:starTurnAnim(res)
end

function This:starTurnAnim(itemResult)
    local function getTimes()
        local res = true
        for i,itemNodeData in ipairs(itemResult) do
           local itemData = itemResult[1]
           if itemData[1]~=itemNodeData[1] or itemData[2]~=itemNodeData[2] then
               res = false
               break 
           end
        end
        return  res
    end
    local ThreeTime = getTimes()
    for i,item in ipairs(self.m_turnNodes) do
       local itemData = itemResult[i]
       if ThreeTime then
           itemData[2] = itemData[2]/3
           item:start(i,itemData,3)
       else
           item:start(i,itemData)
       end
    end
end


function This:requestplay( num )
    local diamondNum = GamePlayer:getDiamond()
    local juanCount = self:getLiJuanCont()
    local needjuanCount = (num- juanCount)
    local needDiamond = self.m_usedPrice*needjuanCount
    if juanCount< num then
        local myPanel = g_scene:openDialog("UI.CostConfirmPanel", {
                fileName = "MenuScene/NodeUI/ActivityHappyRewardVoucher.csb",
                cost = needDiamond,
                notips = "ActivityHappyPlay"..self.m_activity:getId()..needDiamond,--"StoreCommon" .. tostring(self:getId()),
                descText = string.format(strList[self.m_activityType],needjuanCount), 
                okHandler = function(build)
                        if GamePlayer:checkDiamond(needDiamond) then
                            self.m_activity:onGetTurnPlayRequest(num)                    
                        end
                end } )
        if self.m_activityType == NewActivityType.kBingFaRoulette then
            myPanel:setBtnString(string.format(strList[self.m_activityType],needjuanCount),strList.bingFaCost)
        end
        return
    end

    self.m_activity:onGetTurnPlayRequest(num)
end

function This:initTouchEvent()
    self.m_turntableNode.box11:setTouchEnded(function()
        g_scene:openDialog("NewActivity.ActivityHappyTurnDayPanel",{activityType = self.m_activityType})
        -- print("self.m_csNode.turntable.box1")
    end)
    self.m_turntableNode.button1:setTouchEnded(function()
        if not self.m_isPlayAnim then
            self:requestplay(1)
        end
    end)
    self.m_turntableNode.button2:setTouchEnded(function()
        if not self.m_isPlayAnim then
            self:requestplay(10)
        end
    end)
    self.m_turntableNode.line:setTouchEnded(function()
        local itemList = self.m_activity:getZhenBaoList()
        g_scene:openDialog("NewActivity.ActivityHappyRewardPanel",itemList)
    end)
    self.m_turntableNode.weigouxuan:setTouchEnded(function()
        if not self.m_isPlayAnim then
            local isSkip = self:getSkipAnim(  )
            self:setSkipAnim(not isSkip )
        end
    end)
    self.m_turntableNode.jiahaodi1:setTouchEnded(function()
        if self.m_dialog then
            self.m_dialog:selectTabButton(tostring(NewActivityType.kScoreShop))
        end
    end)
    self.m_turntableNode.jiahaodi2:setTouchEnded(function()
        g_scene:openDialog("System.RechargeMainUI")
    end)
end


function This:onEnter()
    super.onEnter(self)
    self.m_activity:addObserver(self)
    GamePlayer:addObserver(self)
    DepotManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    self.m_activity:removeObserver(self)
    GamePlayer:removeObserver(self)
    DepotManager:removeObserver(self)
end


function This:showReward(  )
    local turnPlayResult = self.m_activity:getTurnPlayResult()
    g_scene:showReward(turnPlayResult)
    self.m_isPlayAnim = false
    self:updateBtn() 

end

function This:updateBtn(  )
    local isEnabale = not self.m_isPlayAnim
    if isEnabale then
        self.m_turntableNode.button1:clearShader()
        self.m_turntableNode.button2:clearShader()
    else
        self.m_turntableNode.button1:setGrayShader()
        self.m_turntableNode.button2:setGrayShader()
    end
end


function This:setSkipAnim( isBool )
    self.m_turntableNode.gouxuan:setVisible(isBool)
end

function This:getSkipAnim(  )
    return self.m_turntableNode.gouxuan:isVisible()
end

function This:diaMondLack( num )
    
end


--礼券数量
function This:getLiJuanCont(  )
    return DepotManager:getItemCount(self.m_useItemId)
end

function This:updateLiJuanLabel(  )
    self.m_turntableNode.number1:setString(self:getLiJuanCont( ))--当前礼券
end

function This:onGetHappyInfo(  )
    self:refreshLabel( )
end

function This:onPlayerAttrChanged(field, oldValue, value)--钻石
    if field == PlayerAttr.diamond then
        self.m_turntableNode.number2:setString(value)
    end
end

function This:itemUpdate(itemList)--元宝
    if itemList and #itemList > 0 then
        self:updateLiJuanLabel(  )
    end
end
function This:onGetTurnBox()
    self:refreshLabel()
end
function This:onGetTurnPlay( times )
    self.m_isPlayAnim = true
    self:updateBtn()
    if self:getSkipAnim() then
        self:showReward( )
    else
        self:starTurnWithTime(1)
    end
    self:refreshLabel()
end

function This:resetScoolSize2(node,widthLimit,_height ,text)
    if node and not node.txt then
        node.txt = node:getChildByName("txt")
    end
    if not _height or not node or not node.txt  then
        return
    end
    local height = 0
    local texRule1 = node.txt
    texRule1:setStringWithColor(text,widthLimit)
    local w ,h = texRule1:getContentSize()
    h = h*texRule1:getScaleY()
    height = height + h
    if height < _height then
        height = _height
    end
    texRule1:setPositionY(height)
    node:setInnerContainerSize(widthLimit,height)
end