--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")
local UniversalConfig = require("Metadatas.UniversalConfig")

kMaxLimitRecruitFavours = 100  --限时招募好感度
kMaxLimitRecruitTime = 5   --显示招募次数
kLimitRecruitCost = 350  --招募花费
kRefreshLimitRecruitCost = 120  --刷新花费

function This:init()
	super.init(self,NewActivityType.kLimitRecruit)
	self.m_heroId = 0
	self.m_favours = 0
	self.m_recruitTime = 0
end

function This:initListener()
	EventDispatcher:addListener(self,12015,self.onGetActivityInfoReply)
	EventDispatcher:addListener(self,12016,self.onRecruitHeroReply)
	EventDispatcher:addListener(self,12017,self.onRefreshLimitRecruitReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,12015)
	EventDispatcher:removeListener(self,12016)
	EventDispatcher:removeListener(self,12017)
end

function This:getHeroId()
	return self.m_heroId
end

function This:getFavours()
	return self.m_favours
end

function This:getRecruitTime()
	return self.m_recruitTime
end

function This:getMaxRecruitTime()
	return kMaxLimitRecruitTime
end

function This:isFavoursFull()
	return self.m_favours >= kMaxLimitRecruitFavours
end

function This:getMaxFavours()
	return kMaxLimitRecruitFavours
end

function This:getRecruitCost()
	return kLimitRecruitCost
end

function This:getState()
	local state = super.getState(self)
	if state == NewActivityState.kUpdating then
		if GamePlayer:getLevel() < self.m_config.level_min then
			return NewActivityState.kNotOpen
		end
		if self.m_recruitTime >= kMaxLimitRecruitTime then
			return NewActivityState.kEnded
		end
	end
	return state
end

----------------------------------------------------
function This:onGetActivityInfoRequest()
	Socket:sendAndLock(12015,{activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
	if msg.err_code == 0 then
		self.m_heroId = msg.show_card_id[1]
		self.m_favours = msg.favours
		self.m_recruitTime = msg.recruit_time
		self:notifyObservers("onRefreshLimitRecruit")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:backUpHeroInfo()
	self.m_backInfo = {}
	local heros = HeroManager:getAllHero()
	for id , hero in pairs(heros) do
		self.m_backInfo[id] = hero:isGot()
	end
end

function This:onRecruitHeroRequest()
	Socket:sendAndLock(12016,{activity_id = self:getId()})
	g_scene:setIsEnableShowTips(false)
	self:backUpHeroInfo()
end

function This:onRecruitHeroReply(msg)
	g_scene:setIsEnableShowTips(true)
	if msg.err_code == 0 then
		self.m_recruitTime = msg.recruit_time

		local old = self.m_favours
		self.m_favours = msg.favours
		if self.m_favours > old then
			g_scene:showTips(string.format(Serialize[62037],self.m_favours - old))
		end

		local hero = HeroManager:getHero(msg.card_id[1])
		if hero:isGot() then
			local key = 121 * 100 + hero:getQuality()
			local num = UniversalConfig[key].conversion_count
			g_scene:showReward({{id = 121,num = num}})
		end

		self:updateState()
		self:notifyObservers("onLimitRecruitHero",{{heroId = hero:getId(), isGot = self.m_backInfo[hero:getId()]}})
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onRefreshLimitRecruitRequest()
	Socket:sendAndLock(12017,{activity_id = self:getId()})
end

function This:onRefreshLimitRecruitReply(msg)
	if msg.err_code == 0 then
		self.m_heroId = msg.card_id[1]
		self.m_favours = msg.favours
		self:notifyObservers("onRefreshLimitRecruit")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:getUITypeName()
    return "NewActivity.LimitRecruitTabNode"
end

function This:getUIFileName()
    -- return "LimitRecruit"
    return "ActivityLabourLimitRecruit"
end

--endregion
