-- @Author: superm
-- @Date:   2018-02-07 19:21:17
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-07 14:39:37
-- @File Name: ActivityNewYearExchangeUI.lua
-- @Description: 

local This, super = class(..., "Activity.ActivityBaseUI")
This.csfileName = "MenuScene/NodeUI/ActivityFestivalTask.csb"

local ActivityItemView = require("Activity.ActivityItemView")
local ActivityNewYearExchangeItemView = require("NewActivity.ActivityNewYearExchangeItemView")
local GameUtil = require("Util.GameUtil")

function This:initNode()
	self.m_curViews = {}
    self:initTimeNode()
    self:refreshListView()
    self.m_inited = true
end

function This:onExit()
    for _, v in pairs(self.m_curViews) do
        if v and v[0] ~= 0 then
            v:release()
        end
    end
    super.onExit(self)
end

function This:refreshListView()
    self.m_curTabIndex = self.m_activity:getActivityCreateDays()
    self:refreshItemList( self.m_curTabIndex )
end

function This:refreshItemList( index )
    if self.m_curTabIndex == nil then
        return 
    end
    local groupList = self.m_activity:getGroupListBy( index, true )
    self:refreshItemListByData(groupList)
end

function This:refreshItemListByData( activitys )
    local csNode = self.m_csNode
    csNode.scrollView:removeAllItems()
    csNode.scrollView:setTouchEnabled(true)
	table.sort(activitys, function( a, b )
        local state1, state2 = a:getState() , b:getState()
        if state1 ~= state2 then
            return state1 < state2
        else
            return a:getId() < b:getId()
        end
	end)

	for index, activity in ipairs(activitys) do
        activity:updateState()
        local id = activity:getId()
        local view = self.m_curViews[id]
        if not view then
			view = self:getNewItemView(activity)
			self.m_curViews[id] = view
			view:retain()
		else
			view:refresh()
		end
	    local w , h = view:getContentSize()
	    csNode.scrollView:pushNode(view, w, h)
	end  
end

function This:initTimeNode()
    local startTime = self.m_activity:getStartTime()
    local startStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],startTime)  
    local endTime = self.m_activity:getEndTime()
    local endStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],endTime)
    local csNode = self.m_csNode

    csNode.time:setString(string.format("%s ~ %s",startStr,endStr))
    if self.m_curSelection then
        csNode.text1:setString( self.m_activity:getUIDesc(self.m_curSelection) )
    end
end

function This:getNewItemView( ... )
    return ActivityNewYearExchangeItemView:create( ... )
end

function This:onExchangeReward(queueId, id, act)
    if self.m_activity:getId() ~= queueId then
        return
    end

    local view = self.m_curViews[id]
    if view then
        view:showRewardList()
        self:refreshListView()
    end
end

function This:itemUpdate()
    self:refresh()
end
