-- @Author: superm
-- @Date:   2018-02-07 17:54:27
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-07 17:11:19
-- @File Name: ActivityNewYearUI.lua
-- @Description: 

local This, super = class(..., "Activity.ActivityOneGroupUI")
This.csfileName = "MenuScene/NodeUI/ActivityFestivalTask.csb"

local NewItemView = class(This._NAME .. ".NewItemView", "Activity.ActivityItemView")

local GameUtil = require("Util.GameUtil")

function NewItemView:initCsNode()
    local filename
    local queue = self.m_activity:getQueue()
    if queue then
        filename = queue:getSubCSFileName()
    end
     
    if not filename then
        filename = "MenuScene/NodeUI/ActivityFestivalTaskItem.csb"
    end
    self.m_csNode = cc.Node:createFromCSFile(filename)
    self:addChild(self.m_csNode)

    local w , h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)
end

function This:initNode()
	self.m_curViews = {}
	self:initTimeNode()
    self:refreshListView()
    local csNode = self.m_csNode
    if csNode.button then
        if self.m_curSelection then
            csNode.text1:setString( self.m_activity:getUIDesc(self.m_curSelection) )
            local visible = self.m_activity:isUIRechargeDirtyType(self.m_curSelection)
            csNode.button:setVisible( visible )
        else
            csNode.button:setVisible(self.m_activity:isRechargeDirtyType())
        end
    end
end

function This:onExit()
	for _, v in pairs(self.m_curViews) do
		if v and v[0] ~= 0 then
			v:release()
		end
	end
    super.onExit(self)
end

function This:getNewItemView( ... )
    return NewItemView:create( ... )
end

function This:initTimeNode()
    local startTime = self.m_activity:getStartTime()
    local startStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],startTime)  
    local endTime = self.m_activity:getEndTime()
    local endStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],endTime)

    self.m_csNode.time:setString(string.format("%s ~ %s",startStr,endStr))
end

function This:refreshItemList( index )
    if not self.m_curSelection then
        super.refreshItemList(self, index)
    else
        local filterIdsList = self.m_activity:getFilterIdsList(self.m_curSelection)
        if not filterIdsList or #filterIdsList <= 0 then
            super.refreshItemList(self, index)
            return
        end

        local count = self.m_activity:getGroupListCount()
        local groupList
        if count > 0 then
            if self.m_curTabIndex == nil then
                return 
            end
            groupList = self.m_activity:getGroupListBy(index, false)
        else
            groupList = self.m_activity:getAllObjects()
        end
        local actList = table.filter(groupList, function(id, activity)
            local fid = activity:getProgressType()
            if not fid then
                return false
            end
            return table.find(filterIdsList, function(_, v) return fid == v end)
        end)
        local activitys = GameUtil:shallowChange(actList)
        self:refreshItemListByData(activitys)
    end
end