-- @Author: superm
-- @Date:   2018-02-28 17:15:28
-- @Last Modified by:   superm
-- @Last Modified time: 2019-11-18 18:50:32
-- @File Name: ActivityPrizeDraw.lua
-- @Description: 

local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")

ActPrizeDrawType = {
	kOnce = 1,
	kFiveTimes = 2, 
	kTenTimes = 3
}

ActPrizeDrawNumType = {
	[ActPrizeDrawType.kOnce] = 1,
	[ActPrizeDrawType.kFiveTimes] = 5, 
	[ActPrizeDrawType.kTenTimes] = 10,	
}

ActPrizeDrawNumDesc = {
	[ActPrizeDrawType.kOnce] = 1,
	[ActPrizeDrawType.kFiveTimes] = 5, 
}

local DrawInfoConfig = {
	[NewActivityType.kYuanxiaoDraw] = {
		keyItemSourceCostNum = 200, 
		keyItemSourceGainNum = 1, 
		keyItemId = 228, 									--道具的Id
		useResource = false, 								--道具的类型是否游戏资源
		keyItemNumLimit = 162, 								--每日获得道具的上限
		diffPrice = false,
		defaultDtype = ActPrizeDrawType.kOnce,			--默认抽奖的类型
		keyItemPrice = function( dType ) 					--每次抽奖消耗道具的个数
		    return 1
		end,
		freeTimes = function( dType )						--是否有免费次数
			return 0
		end, 
		hasFreeTimesPerDay = false, 						--免费次数是否每日重置
		drawConfig =  "Metadatas.YuanxiaoDrawConfig",		--使用的配置文件
		uiType = "NewActivity.ActivityPrizeDrawUI",			--使用的界面
		warmingFile = "ActivityFestivalLanternreward",		--使用提示界面文件
		tipsWord = "activity_prizedraw_help_tips",			--使用提示文字
		csbFileName = "MenuScene/NodeUI/ActivityFestivalLanternTask.csb",
	},
	[NewActivityType.kLuckyDraw] = {
		keyItemSourceCostNum = 200, 
		keyItemSourceGainNum = 1, 
		keyItemId = 240, 									--道具的Id
		useResource = false, 								--道具的类型是否游戏资源
		keyItemNumLimit = 162, 								--每日获得道具的上限
		diffPrice = false,
		defaultDtype = ActPrizeDrawType.kOnce,			--默认抽奖的类型
		keyItemPrice = function( dType ) 					--每次抽奖消耗道具的个数
		    return 1
		end,
		freeTimes = function( dType )						--是否有免费次数
			return 0
		end, 
		hasFreeTimesPerDay = false, 						--免费次数是否每日重置
		drawConfig =  "Metadatas.YuanxiaoDrawConfig",		--使用的配置文件
		uiType = "NewActivity.ActivityPrizeDrawUI",			--使用的界面
		warmingFile = "ActivityFestivalLanternreward",		--使用提示界面文件
		tipsWord = "activity_prizedraw_help_tips",			--使用提示文字
		csbFileName = "MenuScene/NodeUI/ActivityFestivalLabourDraw.csb",
	},
	[NewActivityType.kNormalDraw] = {
		keyItemId = 102, 									--道具的Id
		useResource = true, 								--道具的类型是否游戏资源
		keyItemNumLimit = 100, 								--每日获得道具的上限
		diffPrice = true,
		defaultDtype = ActPrizeDrawType.kOnce,			--默认抽奖的类型
		keyItemPrice = function( dType ) 					--每次抽奖消耗道具的个数
			return dType == ActPrizeDrawType.kOnce and 200 or 190
		end, 
		freeTimes = function( dType )						--是否有免费次数
			return dType == ActPrizeDrawType.kOnce and 1 or 0
		end,
		hasFreeTimesPerDay = true, 							--免费次数是否每日重置
		drawConfig =  "Metadatas.YuanBaoDrawConfig",		--使用的配置文件
		-- uiType = "NewActivity.ActivityPrizeNoramlDrawUI",	--使用的界面
		uiType = "NewActivity.ActivityPrizeDrawUI",			--使用的界面
		warmingFile = "ActivityFestivallabourreward",		--使用提示界面文件
		tipsWord = "activity_normalprizedraw_help_tips",	--使用提示文字
		csbFileName = "MenuScene/NodeUI/ActivityFestivalLabourTask.csb",
	},
	[NewActivityType.kFabaoDraw] = {
		keyItemSourceCostNum = 200, 
		keyItemSourceGainNum = 1, 
		keyItemId = 261, 									--道具的Id
		useResource = false, 								--道具的类型是否游戏资源
		keyItemNumLimit = 162, 								--每日获得道具的上限
		diffPrice = false,
		defaultDtype = ActPrizeDrawType.kOnce,			--默认抽奖的类型
		keyItemPrice = function( dType ) 					--每次抽奖消耗道具的个数
		    return 1
		end,
		freeTimes = function( dType )						--是否有免费次数
			return 0
		end, 
		hasFreeTimesPerDay = false, 						--免费次数是否每日重置
		drawConfig =  "Metadatas.YuanxiaoDrawConfig",		--使用的配置文件
		uiType = "NewActivity.ActivityPrizeDrawUI",			--使用的界面
		warmingFile = "ActivityFestivalLanternreward",		--使用提示界面文件
		tipsWord = "activity_prizedraw_help_tips",			--使用提示文字
		csbFileName = "MenuScene/NodeUI/ActivityTreasure.csb",
	},
	[NewActivityType.kCelebration] = {
		keyItemId = 102, 									--道具的Id
		useResource = true, 								--道具的类型是否游戏资源
		keyItemNumLimit = 100, 								--每日获得道具的上限
		diffPrice = true,
		defaultDtype = ActPrizeDrawType.kOnce,			--默认抽奖的类型
		keyItemPrice = function( dType ) 					--每次抽奖消耗道具的个数
			return dType == ActPrizeDrawType.kOnce and 200 or 190
		end, 
		freeTimes = function( dType )						--是否有免费次数
			return dType == ActPrizeDrawType.kOnce and 1 or 0
		end,
		hasFreeTimesPerDay = true, 							--免费次数是否每日重置
		drawConfig =  "Metadatas.YuanBaoDrawConfig",		--使用的配置文件
		uiType = "NewActivity.ActivitySpendGoldNode",	--使用的界面
		warmingFile = "ActivityFestivallabourreward",		--使用提示界面文件
		tipsWord = "activity_normalprizedraw_help_tips",	--使用提示文字
		csbFileName = "MenuScene/NodeUI/SevenDayCelebrationNode5.csb",
	},
} 


function This:init(id)
	super.init(self, id)
	self.m_id = id
	self.m_todayKeyItemNum = 0
	self.m_drawTimeInHistory = 0
	self.m_customConfig = DrawInfoConfig[self:getId()]
	self.m_useFreeTimes = 0
	-- print(self:getHelpTips())
end

function This:initListener()
	EventDispatcher:addListener(self, 12020, self.onActivityPrizeDrawReply)
	EventDispatcher:addListener(self, 12021, self.onGetActivityInfoReply)
	EventDispatcher:addListener(self, 12032, self.onActivityPrizeDrawReply)
	EventDispatcher:addListener(self, 12031, self.onGetActivityInfoReply)
	EventDispatcher:addListener(self, "after_recharge", self.onAfterRechargeReply)
	EventDispatcher:addListener(self, "equip_changed", self.onGetBagDataChange)
end

function This:removeListener()
	EventDispatcher:removeListener(self, "equip_changed")
	EventDispatcher:removeListener(self, "after_recharge")
	EventDispatcher:removeListener(self, 12020)
	EventDispatcher:removeListener(self, 12021)
	EventDispatcher:removeListener(self, 12031)
	EventDispatcher:removeListener(self, 12032)
end

function This:getState()
	local state = super.getState(self)
	if state == NewActivityState.kUpdating then
		if GamePlayer:getLevel() < self.m_config.level_min then
			return NewActivityState.kNotOpen
		end
	end
	return state
end

function This:getId()
	return self.m_id
end

function This:getCurKeyItemNumDescr()
    return tostring(self:getCurProgress()) .. " / " .. tostring(self:getActKeyItemNumLitmit())
end

function This:getUsedKeyItemNumDescr()
    return tostring(self.m_drawTimeInHistory)
end

function This:isUseResMode()
    return self.m_customConfig.useResource
end

function This:getHelpTips()
	if not self.m_helpTips then
		local keyItemName = ItemUtil.getName(self.m_customConfig.keyItemId)
		local argList = {
			[1] = Serialize[self.m_customConfig.tipsWord],
			[2] = self.m_customConfig.keyItemSourceCostNum, 
			[3] = self.m_customConfig.keyItemSourceGainNum,
		}

		if not self.m_customConfig.diffPrice then
			table.insert(argList, keyItemName)
			table.insert(argList, self.m_customConfig.keyItemNumLimit)
			table.insert(argList, keyItemName)
		else
			for dtype, times in ipairs(ActPrizeDrawNumDesc) do
				local cost = self:getActKeyItemPrice(dtype) * times
				table.insert(argList, cost .. keyItemName)
				table.insert(argList, times)
			end
			local freeTimes = self.m_customConfig.freeTimes(self.m_customConfig.defaultDtype)
			table.insert(argList, freeTimes)
			table.insert(argList, self.m_customConfig.keyItemNumLimit)
		end

		local itemDesList = {}
		local timeDescr, itemDescr = "", ""
		local itemList = self:getDrawItemList()
		for _, info in ipairs(itemList) do
			if info.times > 0 then
				local itemName = ItemUtil.getName(info.item_id)
				local numDesc = info.num .. Serialize[60804]
				if info.num >= 100 then
					GameUtil:formatCount(info.num)
				end
				local desc = string.format(Serialize[62043], info.times, itemName .. numDesc)
				table.insert(itemDesList, {id = info.id, times = info.times, desc = desc})
			end
		end

		table.sort(itemDesList,function(cfgA, cfgB)
			if cfgA.times ~= cfgB.times then
			    return cfgA.times < cfgB.times 
			end
			return cfgA.id < cfgB.id
		end)

		for _, info in ipairs(itemDesList) do
			itemDescr = itemDescr .. info.desc
			timeDescr = timeDescr .. info.times .. "、"
		end

		table.insert(argList, timeDescr)
		table.insert(argList, itemDescr)
		self.m_helpTips = string.format(unpack(argList))
	end
    return self.m_helpTips
end

function This:getCurProgress()
	local keyId = self:getActKeyItemId()
	if not self:isUseResMode() then
		return self.m_todayKeyItemNum
	else
		local times = self:getActKeyItemNumLitmit() - (self.m_drawTimeToday or 0)
		if times < 0 then times = 0 end
		return times
	end
end

function This:isSuitable(config, actId, orgLevel)
    if config.activity_id and actId ~= config.activity_id then
        return false
    end
    if orgLevel and orgLevel > 0 and config.level_min and config.level_max then
        local startLev, endLev = config.level_min, config.level_max
        if startLev > 0 and endLev > 0 and startLev <= endLev then
            return orgLevel >= startLev and orgLevel <= endLev
        end
    end
    return true
end

function This:getActKeyItemId()
    return self.m_customConfig.keyItemId
end

function This:getActKeyItemNumLitmit()
    return self.m_customConfig.keyItemNumLimit
end

function This:getActKeyItemPrice(dType)
    return self.m_customConfig.keyItemPrice(dType)
end

function This:getLeftFreeTimes(dType)
    
	local freeTimes = self.m_customConfig.freeTimes(dType or self.m_customConfig.defaultDtype)
    print(freeTimes, self.m_useFreeTimes)
	local num = freeTimes - self.m_useFreeTimes
	if num < 0 then
        num = 0
    end
    return num or 0
end

function This:getLeftTimes()
	return self:getCurProgress()
end

function This:checkHasLeftTimes( times )
    if not self:isUseResMode() then
    	return true
    end
    
    local leftTime = self:getCurProgress()
    return leftTime >= times
end

function This:getDrawItemList()
	local actConfigList = {}
	local itemList = require(self.m_customConfig.drawConfig)
    local queueId = self:getId()
    local originLevel = self:getOriginalLevel()
    for _, cfg in ipairs(itemList) do
        if self:isSuitable(cfg, queueId, originLevel) then
            table.insert(actConfigList, cfg)
        end
    end

    return actConfigList
end

function This:isCanPrizeDraw()
	if not self:checkHasLeftTimes(1) then
		return false
	end

	if self:getLeftFreeTimes() > 0 or self:getLeftFreeTimes(ActPrizeDrawType.kFiveTimes) > 0 then
		return true
	end

	if not self:isUseResMode() then
    	return DepotManager:getItemOrResCount(self:getActKeyItemId()) >= self:getActKeyItemPrice()
    end
end

function This:onGetActivityInfoRequest()
	local cmd = self.m_descConfig.cmd
	Socket:send(cmd, {activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
	if self:getId() ~= msg.activity_id then
		return
	end

	print_r(msg)
	if msg.err_code == 0 then
		self.m_todayKeyItemNum = msg.get_key_num or 0
		self.m_drawTimeToday = msg.buy_time or msg.draw_time
		self.m_drawTimeInHistory = msg.buy_time or msg.all_draw_time
		local oldFreeTimes = self.m_useFreeTimes or 0
		self.m_useFreeTimes = msg.free_draw_time or 0
		self:notifyObservers("onRefreshPrizeDraw", self:getId())
		if oldFreeTimes ~= self.m_useFreeTimes then
			EventDispatcher:dispatcher("prize_draw_item_changed")		
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onActivityPrizeDrawRequest( drawType )
	if not self:isUpdating() then
		return
	end

	if not drawType then
		drawType = ActPrizeDrawType.kOnce
	end

	local cmd = self.m_descConfig.subcmd
    Socket:sendAndLock(cmd, {activity_id = self:getId(), type = drawType})
end

function This:onActivityPrizeDrawReply( msg )
	if msg.activity_id > 0 and self:getId() ~= msg.activity_id then
		return
	end

    if self.m_descConfig.subcmd ~= msg.cmd then
		return
	end
	if msg.err_code == 0 then
		self.m_todayKeyItemNum = msg.get_key_num or 0
		self.m_drawTimeToday = msg.buy_time or msg.draw_time
		self.m_drawTimeInHistory = msg.buy_time or msg.all_draw_time
		local oldFreeTimes = self.m_useFreeTimes or 0
		self.m_useFreeTimes = msg.free_draw_time or 0
		self:notifyObservers("onActivityPrizeDraw", {id = msg.activity_id, reward = msg.reward_list} )
		EventDispatcher:dispatcher("prize_draw_item_changed")		
		-- if oldFreeTimes ~= self.m_useFreeTimes then
		-- 	EventDispatcher:dispatcher("prize_draw_item_changed")		
		-- end
	else
    	g_scene:handleErrorCode(msg.err_code)
		self:notifyObservers("onActivityPrizeDraw", {id = self:getId()})		
	end
end

function This:onAfterRechargeReply()
    if self:isUpdating() then
    	self:onGetActivityInfoRequest()
    end
end

function This:getUITypeName()
    return self.m_customConfig.uiType
    -- return "NewActivity.ActivityPrizeDrawUI"
end

function This:getCsbFileName()
	local name = self:getCSFileName()
    return name or self.m_customConfig.csbFileName
end

function This:getWarmingUIFileName()
    return self.m_customConfig.warmingFile
end

function This:onGetBagDataChange( message )
-- "item_yuanxiao_changed" "yuanxiaoItemUpdate"
	if not self:isUpdating() then
		return
	end

	local foundKeyItem = false
    for i, pitem in ipairs(message.bag_item_list) do
        if pitem.type_id == self.m_customConfig.keyItemId then
            foundKeyItem = true
        end
    end

	if foundKeyItem then
		local keyItemId = self:getActKeyItemId()
		local keyItemCount = DepotManager:getItemOrResCount(keyItemId)
		local info = {activityId = self:getId(), count = keyItemCount}
		EventDispatcher:dispatcher("prize_draw_item_changed", info)
        self:notifyObservers("prizeDrawItemUpdate", info)
	end
end
