-- @Author: superm
-- @Date:   2018-02-28 21:07:57
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-29 16:33:57
-- @File Name: ActivityPrizeDrawRewardPanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")

local kDefaultActPrizeDrawNumType = ActPrizeDrawNumType[ActPrizeDrawType.kOnce]

function This:initData(info)
	if info then
		self.m_keyItemId = info.keyItemId
		self.m_mode = info.mode
		self.m_dialog = info.dialog
		self.m_itemList = info.itemList
		self.m_leftDrawNum = info.leftTimes
		self.m_fileName = info.fileName or "ActivityFestivalLanternreward"
		self.m_activity = info.activity
	end
	self.m_inited = false
end

function This:initNode()
 	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/" .. self.m_fileName .. ".csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

	self.m_keyItemName = ItemUtil.getName(self.m_keyItemId)
    self:refreshNumNode()
    self:refreshItemList()
end

function This:initTouchEvent()
	if not self.m_inited then
	    self.m_csNode.numbg:setTouchEnded(function()
    		if self.m_keyItemNum > 0 then
		    	if self.m_dialog then
				    local multiTime = #self.m_itemList > kDefaultActPrizeDrawNumType
				    if self.m_keyItemNum < #self.m_itemList then
				    	multiTime = false
				    end
	    			self.m_dialog:startPrizeDraw(multiTime)     
    			end
		        self:close()
    		else
                g_scene:showTips(string.format(Serialize[60334], self.m_keyItemName))
	    	end
	    end)

	    self.m_csNode.surebutton:setTouchEnded(function()
	        self:close()
	    end)

    	self.m_inited = true
    end
end

function This:refreshNumNode()
	if self.m_csNode.lastnum then
		self.m_keyItemNum = DepotManager:getItemOrResCount(self.m_keyItemId)
		self.m_csNode.lastnum:setStringWithColor( (self.m_keyItemNum > 0 and "<A1>" or "<R0>") .. self.m_keyItemNum)
	end

    local times = #self.m_itemList
	local costNum = self.m_dialog:calculateCostByDType(times >= kDefaultActPrizeDrawNumType)
	if self.m_activity  and times > 1 then
		times = ActPrizeDrawNumType[ActPrizeDrawType.kTenTimes]
	end
    if self.m_keyItemNum < costNum then
    	times = kDefaultActPrizeDrawNumType
	end

    self.m_csNode.shengyunum:setString(Serialize[62045] .. self.m_keyItemName .. ":")
    if not self.m_mode then
    	self.m_csNode.numbg.num:setString(Serialize[62046] .. tostring(times) .. Serialize[60804])
    else
    	self.m_csNode.numbg.num:setString(Serialize[62047] .. tostring(times) .. Serialize[60532])
    end
end

function This:refreshItemList()
	local node = self.m_csNode
	if self.m_activity then
		local scrollview = node.scrollview
		scrollview:removeAllChildren()
		scrollview.m_direction = "right"
		scrollview:setDirection("horizontal")
		for i,info in ipairs(self.m_itemList) do
			self:setTimer(i * 0.5 - i*i*0.05, function ()
				local itemNode = ItemNode:create(info.item_id, info.num)
				itemNode:setAnchorPoint(0,0.5)
				local w,h = itemNode:getContentSize()
				scrollview:pushBackCustomItem(itemNode,w,h,0.5)
			end)
		end
	else
		local count = 0
		local w = 10
		for i, info in ipairs(self.m_itemList) do
			self:setTimer(i * 0.5 - i*i*0.05, function ()
				local node = ItemNode:create(info.item_id, info.num)
				node:setCascadeOpacityEnabled(true)
				node:setAnchorPoint(0,0.5)
				node:setPositionX(w)
				w = w + node:getContentWidth() + 10
				self.m_csNode.itemtipslist:addChild(node)
				self.m_csNode.itemtipslist:setPositionX(- w / 2)
			end)
		end
	end

end