-- @Author: superm
-- @Date:   2018-04-24 11:03:07
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-02 23:04:05
-- @File Name: ActivityPrizeDrawUI.lua
-- @Description: 

local This, super = class(..., "NodeUI")

local DepotManager = require("Depot.DepotManager")
local NewActivityManager = require("NewActivity.NewActivityManager")
local HeroManager = require("Hero.HeroManager")
local ItemNode = require("UI.ItemNode")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local CostConfirmPanel = require("UI.CostConfirmPanel")

local kMaxRewardNum = 14
local kRewardRoundNum = 1 * kMaxRewardNum
local kMaxRound = kMaxRewardNum * 200
local kDecelationNum = 1
local kBlinkNum = 6

function This:initData(queue, dialog)
    self.m_activity = queue
    self.m_dialog = dialog

    self.m_lott = {
        iCurIndex = 1,
        iMaxRound = kMaxRound - kDecelationNum ,
        iCurRound = 0, 
        iLuckyID  = 0, 
        iLuckyIndex = 0,
        bIsGot = false,
        bNeedClear = false,
        fDelay = 0.2,
        fDelayFactor = 0.2,
    }

    self.m_runningAniantion = false
    self.m_showAnimation = true 
    self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
    self.m_csNode = cc.Node:createFromCSFile(self.m_activity:getCsbFileName())
    self:addChild(self.m_csNode)
end

function This:hasDesc()
    -- return self.m_activity:hasResTypeWith(ActResType.kTimeLimitPic)
    return self.m_csNode.recharge ~= nil
end

function This:initNode()
    local csNode = self.m_csNode
    self.m_keyItemId = self.m_activity:getActKeyItemId()
    self.m_useResMode = self.m_activity:isUseResMode()
    self.m_itemList = self.m_activity:getDrawItemList() 

    if self:hasDesc() then
        self:initTimeNode()
    end
    for i=1, kMaxRewardNum do
        csNode["pot" .. tostring(i)].selected:setVisible(false)
        local itemInfo = self.m_itemList[i]
        if itemInfo then
            local itemId, num = itemInfo.item_id, itemInfo.num
            local itemNode = csNode["item" .. tostring(i)]
            if itemNode then
                self:initRewardItemNode(itemNode, itemId, num)
                if os.isWin32 then
                    itemNode.num:setString(string.format("id:%d-%s", itemInfo.id, ItemUtil.formatItemCount(num)))
                end
            end
        end
    end

    local keyIcon = ItemUtil.getIconImage(self.m_keyItemId)
    csNode.button1.icon:setImage(keyIcon)
    csNode.button2.icon:setImage(keyIcon)

    self:refreshItemNumNode()
    csNode.gouxuan:setVisible(not self.m_showAnimation)
    if self.m_useResMode then
    	self:updateItemNumNode( GamePlayer:getDiamond() )
    end
end

function This:initRewardItemNode( node, id, num )
    if not node.act_inited then  
        node.icon:setImage(ItemUtil.getIconImage(id))
        g_scene:handleHelpTips(node.icon, function ()
            return {fileName = "UI.ItemTips", itemId = id, itemType = ItemUtil.getType(id)}
        end)
        node.num:setString(ItemUtil.formatItemCount(num))
        node.act_inited = true
    end
end

function This:initTimeNode()
    local startTime = self.m_activity:getStartTime()
    local startStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],startTime)    
    local endTime = self.m_activity:getEndTime()
    local endStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],endTime)

    self.m_csNode.time:setString(string.format("%s ~ %s",startStr,endStr))
end


function This:initTouchEvent()
    if not self.m_inited then
        local csNode = self.m_csNode
        if csNode.recharge then
            csNode.recharge:setTouchEnded(function()
                g_scene:openDialog("System.RechargeMainUI")
            end)
        end

        if csNode.help then
            csNode.help:setTouchEnded(function()
                local tips = self.m_activity:getHelpTips()
                g_scene:openDialog("UI.HelpTips", {text = tips, node = csNode.help})
            end)
        end

        --  法宝图鉴
        if csNode.button then    
            csNode.button:setTouchEnded(function()
                g_scene:openDialog("Fabao.FabaoHandbookPanel", {})
            end)
        end

        if csNode.libao then
            csNode.libao:setTouchEnded(function()
                g_scene:openDialog("Store.GeneralStorePanel", {storeType = 3})
            end)
        end

        csNode.weigouxuan:setTouchEnded(function()
            if not self.m_runningAniantion then
                local visible = csNode.gouxuan:isVisible()
                if self.m_showAnimation == not visible then 
                    self.m_showAnimation = visible
                    csNode.gouxuan:setVisible(not visible)
                end
            end
        end)

        csNode.button1:setTouchEnded(function()
            self:startPrizeDraw()      
        end)

        csNode.button2:setTouchEnded(function()
            self:startPrizeDraw(true)      
        end)

        self.m_inited = true
    end
end

function This:calculateCost( dtype, times )
    local price = self.m_activity:getActKeyItemPrice(dtype) 
    return price * times
end

function This:calculateCostByDType( multiTime )
    local dType = ActPrizeDrawType.kOnce 
    if multiTime then
        dType = ActPrizeDrawType.kFiveTimes
    end
    return self:calculateCost(dType, ActPrizeDrawNumType[dType])
end

function This:startPrizeDraw( multiTime )
    if self.m_runningAniantion or not self.m_activity or not self.m_activity:isUpdating() then
        return
    end

    local dType = ActPrizeDrawType.kOnce
    if multiTime then
        dType = ActPrizeDrawType.kFiveTimes
    end
    local dNumType = ActPrizeDrawNumType[dType]
    if not self.m_activity:checkHasLeftTimes(dNumType) then
        g_scene:showTips(Serialize[11510])
        return
    end

    local reuestDraw = function ()
        if self.m_showAnimation then
            self.m_runningAniantion = true
            if self.m_dialog and type(self.m_dialog) ~="number" then
                self.m_dialog:setIsRunning(true)
            end
            self:runSelectionItem()
        end
        self.m_activity:onActivityPrizeDrawRequest(dType)   
    end

    local costNum = self:calculateCost(dType, dNumType - self.m_activity:getLeftFreeTimes(dType))

    if costNum > 0 then
        local notips = "Activity" .. tostring(self.m_activity:getId()) .. "shuaxin" .. tostring(dNumType)
        local check = function ()
            local result, errorCode = DepotManager:isItemOrResEnough(self.m_keyItemId, costNum)
            if not result then
                if errorCode then
                    g_scene:handleErrorCode(errorCode)
                else
                    g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(self.m_keyItemId)))
                end
            end
            return result
        end
        
        if not CostConfirmPanel:existNoTipsKey(notips) then
            local openDialog = function ()
                    g_scene:openDialog("UI.CostConfirmPanel", {
                        notips = notips,
                        cost = costNum,
                        costId = self.m_keyItemId,
                        desc = string.format(Serialize[62049], dNumType), 
                        okHandler = function()
                            coroutine.resume(self.m_coroutine)
                        end }
                    )
                    return coroutine.yield()
            end

            self.m_coroutine = coroutine.create(function ()
--                openDialog()
                if check() then
                    reuestDraw()
                end
            end)
            coroutine.resume(self.m_coroutine)
        else
            if check() then
                reuestDraw()
            end
        end
    else
        reuestDraw()
    end 
end

function This:refreshItemNumNode()
    local csNode = self.m_csNode
    local leftTime, number = self.m_activity:getLeftFreeTimes(), DepotManager:getItemOrResCount(self.m_keyItemId)
    if not self.m_useResMode then
        csNode.shengyunum:setStringWithColor((number > 0 and "<A7>" or "<R0>") .. tostring(number))
    else
        csNode.shengyunum:setStringWithColor((leftTime > 0 and "<A7>" or "<R0>") .. tostring(leftTime))
    end

    if self:hasDesc() then
        csNode.huodongnum:setString(self.m_activity:getCurKeyItemNumDescr())
        csNode.eatnum:setString(self.m_activity:getUsedKeyItemNumDescr())
    end

    if leftTime > 0 or number >= self:calculateCostByDType(true) then
        csNode.button1:clearShader()
        csNode.button2:clearShader()
    elseif number >= self:calculateCostByDType(false) then
        csNode.button2:setGrayShader()
        csNode.button1:clearShader()
    else
        csNode.button1:setGrayShader()
        csNode.button2:setGrayShader()
    end
end

function This:refresh()
    if self.m_lott.bNeedClear then
        self.m_lott = {
            iCurIndex = 1,
            iMaxRound = kMaxRound - kDecelationNum ,
            iCurRound = 0, 
            iLuckyID  = 0, 
            iLuckyIndex = 0,
            bIsGot = false,
            bNeedClear = false,
            fDelay = 0.2,
            fDelayFactor = 0.2,
        }
    end

    local csNode = self.m_csNode
    for i=1, kMaxRewardNum do
        csNode["pot" .. tostring(i)].selected:setVisible(false)
    end
    self:refreshItemNumNode()
end

function This:runSelectionItem(delay)
    local csNode = self.m_csNode
    local continue = false
    local loopCount = 1

    function runTask( )
        loopCount = loopCount + 1
        if not delay then
            delay = self.m_lott.fDelay
        end

        local count = kMaxRewardNum
        local minIndex = 1
        local maxIndex = kMaxRewardNum

        self.m_lott.iCurRound = self.m_lott.iCurRound + 1
        if self.m_lott.iCurRound < kDecelationNum or self.m_lott.iCurRound > self.m_lott.iMaxRound then
            continue = false
        else
            continue = true
        end

        csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
        self.m_lott.iCurIndex = self.m_lott.iCurIndex >= maxIndex and minIndex or self.m_lott.iCurIndex + 1
        csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(true);


        if self.m_lott.iCurRound > self.m_lott.iMaxRound then
            if self.m_lott.fDelay < self.m_lott.fDelayFactor then
                -- music run
                self.m_lott.fDelay = self.m_lott.fDelay + 0.32 * self.m_lott.fDelayFactor
                self.m_lott.fDelay = self.m_lott.fDelay > self.m_lott.fDelayFactor and self.m_lott.fDelayFactor or self.m_lott.fDelay
                self:runSelectionItem(self.m_lott.fDelay)
            else 
                -- music stop
                if self.m_lott.iCurRound == kMaxRound then
                    -- 移動 kMaxRound 次還沒收到網絡回調,認定為断网的节奏
                    csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    self.m_lott.iCurIndex = minIndex
                    g_scene:showTips(Serialize["act_tip_network_bad"])
                    self.m_runningAniantion = false
                    self.m_lott.bNeedClear = true
                    if self.m_dialog and type(self.m_dialog) ~="number" then
                        self.m_dialog:setIsRunning(false)
                    end 
                else
                    csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    csNode["pot" .. self.m_lott.iCurIndex].selected:runAction(cc.Sequence(cc.Blink(self.m_lott.fDelay * 2, kBlinkNum), cc.CallFunc(function ()
                        self:onLotteryResultShow()
                    end)))
                end
                self.m_lott.iCurRound = 0
                self.m_lott.iMaxRound = kMaxRound - kDecelationNum
            end
        elseif self.m_lott.iCurRound < kDecelationNum then
            -- music run
            self.m_lott.fDelay = self.m_lott.fDelay - 0.32
            self:runSelectionItem(self.m_lott.fDelay)

        else
            -- music run
        end

        if continue then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
        end
    end

    if delay then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
    else
        runTask()
    end
end

function This:onEnter()
    super.onEnter(self)
    self.m_activity:addObserver(self)
    GamePlayer:addObserver(self)
end

function This:onExit()
    GamePlayer:removeObserver(self)
    self.m_activity:removeObserver(self)
    super.onExit(self)
end

function This:onLotteryResultShow(noAnimations)
    if self.m_curRewardList then
        if self.m_showAnimation then
            self:runSelectionItem()
            self:setTimer(0.1, function() self:onActivityPrizeDraw({id=self.m_activity:getId()}) end)
        end
    else
        if not noAnimations then
            g_scene:setIsEnableShowTips(true)
            if self.m_showPrizeRewardList then
                g_scene:openDialog("NewActivity.ActivityPrizeDrawRewardPanel", {
                    mode = self.m_useResMode,
                    keyItemId = self.m_keyItemId, 
                    fileName = self.m_activity:getWarmingUIFileName(),
                    itemList = self.m_showPrizeRewardList, 
                    dialog = self
                })
            end
        end

        self.m_csNode["pot" .. self.m_lott.iLuckyIndex].selected:setVisible(true)
        self.m_lott.bNeedClear = true
        self.m_runningAniantion = false
        if self.m_dialog and type(self.m_dialog) ~="number" then
            self.m_dialog:setIsRunning(false)
        end        
        self:refresh()
    end
end

-------------------------------------
local function getCurIndexNum(num)
    return ((num - 1) % kMaxRewardNum) + 1
end

function This:onActivityPrizeDraw( msg )
    if msg.id == self.m_activity:getId() then
        local info = msg.reward

        self.m_curRewardList = info or self.m_curRewardList
        self.m_curRewardIndex = (self.m_curRewardIndex or 0) + 1
        if self.m_curRewardList and self.m_curRewardIndex <= #self.m_curRewardList then
            local index = getCurIndexNum(self.m_curRewardList[self.m_curRewardIndex] or 1)
            self.m_lott.iLuckyIndex = index 
            self.m_lott.iLuckyID = self.m_itemList[index].item_id

            self.m_lott.bIsGot = ItemUtil.getType(self.m_lott.iLuckyID) == ItemCommonType.kHero and HeroManager:isGotHero(self.m_lott.iLuckyID)

            if self.m_showAnimation then
                if self.m_curRewardIndex == 1 then
                    self.m_lott.iMaxRound = self.m_lott.iLuckyIndex - self.m_lott.iCurIndex
                    self.m_lott.iMaxRound = self.m_lott.iMaxRound > 0 and self.m_lott.iMaxRound or self.m_lott.iMaxRound + kRewardRoundNum
                    self.m_lott.iMaxRound = self.m_lott.iCurRound + self.m_lott.iMaxRound + kRewardRoundNum * 1 - kDecelationNum
                else
                    self.m_lott.iMaxRound = self.m_lott.iLuckyIndex - self.m_lott.iCurIndex
                    self.m_lott.iMaxRound = self.m_lott.iMaxRound > 0 and self.m_lott.iMaxRound or self.m_lott.iMaxRound + kMaxRewardNum
                    self.m_lott.iMaxRound = self.m_lott.iCurRound + self.m_lott.iMaxRound - kDecelationNum            
                end
            else
                self.m_showPrizeRewardList = {}
                for i, rindex in ipairs(self.m_curRewardList) do
                    table.insert(self.m_showPrizeRewardList, self.m_itemList[getCurIndexNum(rindex)])
                end
                self.m_curRewardList = nil
                self.m_curRewardIndex = 0
                self:onLotteryResultShow()
            end

            if self.m_curRewardList and self.m_curRewardIndex == #self.m_curRewardList then
                self.m_showPrizeRewardList = {}
                for i, rindex in ipairs(self.m_curRewardList) do
                    table.insert(self.m_showPrizeRewardList, self.m_itemList[getCurIndexNum(rindex)])
                end
                self.m_curRewardList = nil
                self.m_curRewardIndex = 0
            end
        elseif not info then
            self.m_curRewardList = nil
            self.m_curRewardIndex = 0
            if msg and msg.err_code ~= 0 then
                self.m_lott.iCurRound = kMaxRound - kMaxRewardNum
            end
        end
    end
end

function This:onRefreshPrizeDraw( activityId )
    if activityId == self.m_activity:getId() and self:hasDesc() then
        self.m_csNode.huodongnum:setString(self.m_activity:getCurKeyItemNumDescr()) 
        self.m_csNode.eatnum:setString(self.m_activity:getUsedKeyItemNumDescr()) 
    end
end

function This:prizeDrawItemUpdate( info )
    if info.activityId == self.m_activity:getId() then
        self:refreshItemNumNode()
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if self.m_useResMode and field == PlayerAttr.diamond then
        self:updateItemNumNode( value )
    end
end

function This:updateItemNumNode( value )
    self:refreshItemNumNode()
    if self.m_useResMode then
    	if self.m_csNode.yuanbaonum then
            self.m_csNode.yuanbaonum:setString(value)
    	end
    end
end

