-- @Author: Su
-- @Date:   2019-03-25 10:01:41
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-28 11:10:56
-- @File Name: ActivitySpendGoldNode.lua

local This , super = class(...,"NodeUI")
local NewActivityManager = require("NewActivity.NewActivityManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local SevenDayCelebrationItem = require("NewActivity.SevenDayCelebrationItem")
local CelebrationCostRankConfig = require("Metadatas.CelebrationCostRankConfig")
local OpenTaskConfig = require("Metadatas.OpenTaskConfig")
local CostConfirmPanel = require("UI.CostConfirmPanel")


local kMaxRewardNum = 14
local kRewardRoundNum = 1 * kMaxRewardNum
local kMaxRound = kMaxRewardNum * 200
local kDecelationNum = 1
local kBlinkNum = 6

function This:initData(info)
    if info then
        self.m_tabIndex = info.tab
        self.m_name =info.name
    end
    -- self.m_config = self:getDataByTabIndex(self.m_tabIndex)
    self.m_escape = 0
    
    self.m_lott = {
        iCurIndex = 1,
        iMaxRound = kMaxRound - kDecelationNum ,
        iCurRound = 0, 
        iLuckyID  = 0, 
        iLuckyIndex = 0,
        bIsGot = false,
        bNeedClear = false,
        fDelay = 0.2,
        fDelayFactor = 0.2,
    }

    self.m_runningAniantion = false
    self.m_showAnimation = true 
    self.m_inited = false
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/SevenDayCelebrationNode"..self.m_tabIndex..".csb")
    self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

    self:refreshNode()
    if self.m_tabIndex == SevenDayActivityTab.kBestKing then
        local item = SevenDayCelebrationItem:create({data = {},tabIndex = self.m_tabIndex,activity=self.m_activity})
        self.m_csNode:addChild(item)
    elseif self.m_tabIndex == SevenDayActivityTab.kCelebration then
        self.m_keyItemId = self.m_activity:getActKeyItemId()
        self.m_useResMode = self.m_activity:isUseResMode()
        self.m_itemList = self.m_activity:getDrawItemList()
        local node = self.m_csNode
        for i=1, kMaxRewardNum do
            node["pot" .. tostring(i)].selected:setVisible(false)
            local itemInfo = self.m_itemList[i]
            if itemInfo then
                local itemId, num = itemInfo.item_id, itemInfo.num
                local itemNode = node["item" .. tostring(i)]
                if itemNode then
                    self:initRewardItemNode(itemNode, itemId, num)
                    if os.isWin32 then
                        itemNode.num:setString(string.format("id:%d-%s", itemInfo.id, ItemUtil.formatItemCount(num)))
                    end
                end               
            end
        end

        self:refreshItemNumNode()
        node.gouxuan:setVisible(not self.m_showAnimation)
        if self.m_useResMode then
            self:updateItemNumNode( GamePlayer:getDiamond() )
        end
    end
    self:refreshTime()
end

function This:hasDesc()
    return self.m_csNode.recharge ~= nil
end

function This:initRewardItemNode( node, id, num )
    if not node.act_inited then  
        node.icon:setImage(ItemUtil.getIconImage(id))
        g_scene:handleHelpTips(node.icon, function ()
            return {fileName = "UI.ItemTips", itemId = id, itemType = ItemUtil.getType(id)}
        end)
        node.num:setString(ItemUtil.formatItemCount(num))
        node.act_inited = true
    end
end

function This:refreshItemNumNode()
    local csNode = self.m_csNode
    local leftTime, number = self.m_activity:getLeftFreeTimes(), DepotManager:getItemOrResCount(self.m_keyItemId)
    if not self.m_useResMode then
        csNode.shengyunum:setStringWithColor((number > 0 and "<A1>" or "<R0>") .. tostring(number))
    else
        csNode.shengyunum:setStringWithColor((leftTime > 0 and "<A1>" or "<R0>") .. tostring(leftTime))
    end

    if self:hasDesc() then
        csNode.huodongnum:setString(self.m_activity:getCurKeyItemNumDescr())
        csNode.eatnum:setString(self.m_activity:getUsedKeyItemNumDescr())
    end

    if leftTime > 0 or number >= self:calculateCostByDType(true) then
        csNode.button1:clearShader()
        csNode.button2:clearShader()
    elseif number >= self:calculateCostByDType(false) then
        csNode.button2:setGrayShader()
        csNode.button1:clearShader()
    else
        csNode.button1:setGrayShader()
        csNode.button2:setGrayShader()
    end
end

function This:refresh()
    if self.m_lott.bNeedClear then
        self.m_lott = {
            iCurIndex = 1,
            iMaxRound = kMaxRound - kDecelationNum ,
            iCurRound = 0, 
            iLuckyID  = 0, 
            iLuckyIndex = 0,
            bIsGot = false,
            bNeedClear = false,
            fDelay = 0.2,
            fDelayFactor = 0.2,
        }
    end

    local csNode = self.m_csNode
    for i=1, kMaxRewardNum do
        csNode["pot" .. tostring(i)].selected:setVisible(false)
    end
    self:refreshItemNumNode()
end

function This:refreshNode()
    local activityType = NewActivityType[self.m_name]
    self.m_activityType = activityType
    self.m_activity = NewActivityManager:getActivity(activityType)
    local scrollView = self.m_csNode.scrollView
    if scrollView then
        scrollView:removeAllItems()
        local turnDatas = self.m_activity:getTurnDatas()
        for i=1,#turnDatas do
            local item = SevenDayCelebrationItem:create({index = i,data = turnDatas,tabIndex = self.m_tabIndex,activity=self.m_activity})
            local w,h = item:getContentSize()
            scrollView:pushBackCustomItem(item,743,106,0.5)
        end
    end
end

function This:refreshTime()
    local node = self.m_csNode
    local leftTime = self.m_activity:getEndTime() - GamePlayer:getServerTime()
    if node.number1 then
        if self.m_activity then
            node.number1:setString(self.m_activity:getMyConsumption())
        else
            node.number1:setString("5000")
        end
    end
    if self.m_tabIndex == SevenDayActivityTab.kCelebration then
        node.yuanbaonum:setString(GamePlayer:getDiamond())
    end
    node.time:setString(GameUtil:formatDTime(Serialize[62052],leftTime))
end

function This:initTouchEvent()
    local node = self.m_csNode
    if self.m_tabIndex == SevenDayActivityTab.kSpendGold then
        node.wenhao:setTouchEnded(function ()
            g_scene:openDialog("UI.HelpTips",{text = Serialize["activity_spend_gold_help_tips"],node = node.wenhao})

        end)
        node.bangdan.bg:setTouchEnded(function ()
            g_scene:openDialog("NewActivity.SevenDayCelebrationRankPanel",{activity = self.m_activity})
        end)
    elseif self.m_tabIndex == SevenDayActivityTab.kCelebration then
        node.help:setTouchEnded(function ()
            g_scene:openDialog("UI.HelpTips",{text = Serialize["activity_yuanbao_turntable_tips"],node = node.help})
        end)
        node.recharge:setTouchEnded(function()
            g_scene:openDialog("System.RechargeMainUI")
        end)
        node.weigouxuan:setTouchEnded(function()
            if not self.m_runningAniantion then
                local visible = node.gouxuan:isVisible()
                if self.m_showAnimation == not visible then 
                    self.m_showAnimation = visible
                    node.gouxuan:setVisible(not visible)
                end
            end
        end)

        node.button1:setTouchEnded(function()
            self:startPrizeDraw()      
        end)

        node.button2:setTouchEnded(function()
            self:startPrizeDraw(true)      
        end)
    end
end

function This:calculateCost( dtype, times )
    local price = self.m_activity:getActKeyItemPrice(dtype) 
    return price * times
end

function This:calculateCostByDType( multiTime )
    local dType = ActPrizeDrawType.kOnce 
    if multiTime then
        dType = ActPrizeDrawType.kFiveTimes
    end
    return self:calculateCost(dType, ActPrizeDrawNumType[dType])
end

function This:startPrizeDraw( multiTime )
    if self.m_runningAniantion or not self.m_activity or not self.m_activity:isUpdating() then
       return
    end

    local dType = ActPrizeDrawType.kOnce
    if multiTime then
        dType = ActPrizeDrawType.kFiveTimes
    end
    local dNumType = ActPrizeDrawNumType[dType]
    if not self.m_activity:checkHasLeftTimes(dNumType) then
        g_scene:showTips(Serialize[11510])
        return
    end

    local reuestDraw = function ()
        if self.m_showAnimation then
            self.m_runningAniantion = true
            if self.m_dialog then
                self.m_dialog:setIsRunning(true)
            end
            self:runSelectionItem()
        end
        self.m_activity:onActivityPrizeDrawRequest(dType)   
    end

    local costNum = self:calculateCost(dType, dNumType - self.m_activity:getLeftFreeTimes(dType))

    if costNum > 0 then
        local notips = "Activity" .. tostring(self.m_activity:getId()) .. "shuaxin" .. tostring(dNumType)
        local check = function ()
            local result, errorCode = DepotManager:isItemOrResEnough(self.m_keyItemId, costNum)
            if not result then
                if errorCode then
                    g_scene:handleErrorCode(errorCode)
                else
                    g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(self.m_keyItemId)))
                end
            end
            return result
        end
        
        if not CostConfirmPanel:existNoTipsKey(notips) then
            local openDialog = function ()
                    g_scene:openDialog("UI.CostConfirmPanel", {
                        notips = notips,
                        cost = costNum,
                        costId = self.m_keyItemId,
                        desc = string.format(Serialize[62049], dNumType), 
                        okHandler = function()
                            coroutine.resume(self.m_coroutine)
                        end }
                    )
                    return coroutine.yield()
            end

            self.m_coroutine = coroutine.create(function ()
                openDialog()
                if check() then
                    reuestDraw()
                end
            end)
            coroutine.resume(self.m_coroutine)
        else
            if check() then
                reuestDraw()
            end
        end
    else
        reuestDraw()
    end 
end

function This:runSelectionItem(delay)
    local csNode = self.m_csNode
    local continue = false
    local loopCount = 1

    function runTask( )
        loopCount = loopCount + 1
        if not delay then
            delay = self.m_lott.fDelay
        end

        local count = kMaxRewardNum
        local minIndex = 1
        local maxIndex = kMaxRewardNum

        self.m_lott.iCurRound = self.m_lott.iCurRound + 1
        if self.m_lott.iCurRound < kDecelationNum or self.m_lott.iCurRound > self.m_lott.iMaxRound then
            continue = false
        else
            continue = true
        end

        csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
        self.m_lott.iCurIndex = self.m_lott.iCurIndex >= maxIndex and minIndex or self.m_lott.iCurIndex + 1
        csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(true);


        if self.m_lott.iCurRound > self.m_lott.iMaxRound then
            if self.m_lott.fDelay < self.m_lott.fDelayFactor then
                -- music run
                self.m_lott.fDelay = self.m_lott.fDelay + 0.32 * self.m_lott.fDelayFactor
                self.m_lott.fDelay = self.m_lott.fDelay > self.m_lott.fDelayFactor and self.m_lott.fDelayFactor or self.m_lott.fDelay
                self:runSelectionItem(self.m_lott.fDelay)
            else 
                -- music stop
                if self.m_lott.iCurRound == kMaxRound then
                    -- 移動 kMaxRound 次還沒收到網絡回調,認定為断网的节奏
                    csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    self.m_lott.iCurIndex = minIndex
                    g_scene:showTips(Serialize["act_tip_network_bad"])
                    self.m_runningAniantion = false
                    self.m_lott.bNeedClear = true
                    if self.m_dialog then
                        self.m_dialog:setIsRunning(false)
                    end 
                else
                    csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    csNode["pot" .. self.m_lott.iCurIndex].selected:runAction(cc.Sequence(cc.Blink(self.m_lott.fDelay * 2, kBlinkNum), cc.CallFunc(function ()
                        self:onLotteryResultShow()
                    end)))
                end
                self.m_lott.iCurRound = 0
                self.m_lott.iMaxRound = kMaxRound - kDecelationNum
            end
        elseif self.m_lott.iCurRound < kDecelationNum then
            -- music run
            self.m_lott.fDelay = self.m_lott.fDelay - 0.32
            self:runSelectionItem(self.m_lott.fDelay)

        else
            -- music run
        end

        if continue then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
        end
    end

    if delay then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
    else
        runTask()
    end
end

function This:onEnter()
    super.onEnter(self)
    GamePlayer:addObserver(self)
    self.m_activity:addObserver(self)
    self.m_activity:onGetActivityInfoRequest()
end

function This:onExit()
    GamePlayer:addObserver(self)
    self:unscheduleUpdate()
    self.m_activity:removeObserver(self)
    super.onExit(self)
end

function This:onLotteryResultShow(noAnimations)
    if self.m_curRewardList then
        if self.m_showAnimation then
            self:runSelectionItem()
            self:setTimer(0.1, function() self:onActivityPrizeDraw({id=self.m_activity:getId()}) end)
        end
    else
        if not noAnimations then
            g_scene:setIsEnableShowTips(true)
            if self.m_showPrizeRewardList then
                g_scene:openDialog("NewActivity.ActivityPrizeDrawRewardPanel", {
                    mode = self.m_useResMode,
                    keyItemId = self.m_keyItemId, 
                    fileName = self.m_activity:getWarmingUIFileName(),
                    itemList = self.m_showPrizeRewardList, 
                    dialog = self
                })
            end
        end

        self.m_csNode["pot" .. self.m_lott.iLuckyIndex].selected:setVisible(true)
        self.m_lott.bNeedClear = true
        self.m_runningAniantion = false
        if self.m_dialog then
            self.m_dialog:setIsRunning(false)
        end        
        self:refresh()
    end
end

local function getCurIndexNum(num)
    return ((num - 1) % kMaxRewardNum) + 1
end

function This:onActivityPrizeDraw( msg )
    if msg.id == self.m_activity:getId() then
        local info = msg.reward

        self.m_curRewardList = info or self.m_curRewardList
        self.m_curRewardIndex = (self.m_curRewardIndex or 0) + 1
        if self.m_curRewardList and self.m_curRewardIndex <= #self.m_curRewardList then
            local index = getCurIndexNum(self.m_curRewardList[self.m_curRewardIndex] or 1)
            self.m_lott.iLuckyIndex = index 
            self.m_lott.iLuckyID = self.m_itemList[index].item_id

            self.m_lott.bIsGot = ItemUtil.getType(self.m_lott.iLuckyID) == ItemCommonType.kHero and HeroManager:isGotHero(self.m_lott.iLuckyID)

            if self.m_showAnimation then
                if self.m_curRewardIndex == 1 then
                    self.m_lott.iMaxRound = self.m_lott.iLuckyIndex - self.m_lott.iCurIndex
                    self.m_lott.iMaxRound = self.m_lott.iMaxRound > 0 and self.m_lott.iMaxRound or self.m_lott.iMaxRound + kRewardRoundNum
                    self.m_lott.iMaxRound = self.m_lott.iCurRound + self.m_lott.iMaxRound + kRewardRoundNum * 1 - kDecelationNum
                else
                    self.m_lott.iMaxRound = self.m_lott.iLuckyIndex - self.m_lott.iCurIndex
                    self.m_lott.iMaxRound = self.m_lott.iMaxRound > 0 and self.m_lott.iMaxRound or self.m_lott.iMaxRound + kMaxRewardNum
                    self.m_lott.iMaxRound = self.m_lott.iCurRound + self.m_lott.iMaxRound - kDecelationNum            
                end
            else
                self.m_showPrizeRewardList = {}
                for i, rindex in ipairs(self.m_curRewardList) do
                    table.insert(self.m_showPrizeRewardList, self.m_itemList[getCurIndexNum(rindex)])
                end
                self.m_curRewardList = nil
                self.m_curRewardIndex = 0
                self:onLotteryResultShow()
            end

            if self.m_curRewardList and self.m_curRewardIndex == #self.m_curRewardList then
                self.m_showPrizeRewardList = {}
                for i, rindex in ipairs(self.m_curRewardList) do
                    table.insert(self.m_showPrizeRewardList, self.m_itemList[getCurIndexNum(rindex)])
                end
                self.m_curRewardList = nil
                self.m_curRewardIndex = 0
            end
        elseif not info then
            self.m_curRewardList = nil
            self.m_curRewardIndex = 0
            if msg and msg.err_code ~= 0 then
                self.m_lott.iCurRound = kMaxRound - kMaxRewardNum
            end
        end
    end
end

function This:onRefreshPrizeDraw( activityId )
    if activityId == self.m_activity:getId() and self:hasDesc() then
        self.m_csNode.huodongnum:setString(self.m_activity:getCurKeyItemNumDescr()) 
        self.m_csNode.eatnum:setString(self.m_activity:getUsedKeyItemNumDescr()) 
    end
end

function This:prizeDrawItemUpdate( info )
    if info.activityId == self.m_activity:getId() then
        self:refreshItemNumNode()
    end
end

function This:getTabIndex()
    return self.m_tabIndex
end

function This:updateItemNumNode( value )
    self:refreshItemNumNode()
    if self.m_useResMode then
    	if self.m_csNode.yuanbaonum then
            self.m_csNode.yuanbaonum:setString(value)
    	end
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if self.m_useResMode and field == PlayerAttr.diamond then
        self:updateItemNumNode( value )
    end
end

function This:onUpdateActivity(activity)
    local type = activity:getId()
    if type >= NewActivityType.kSpendGold and type <= NewActivityType.kCelebration then
        self:refreshTime()
    end
end

function This:onRefreshCumulative(index)
    self:refreshNode()
end

function This:onRefreshDailyRecharge(index)
    self:refreshNode()
end