-- @Author: superm
-- @Date:   2018-08-02 18:40:19
-- @Last Modified by:   superm
-- @Last Modified time: 2018-10-29 21:00:28
-- @File Name: ActivityTrailer.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local EventListConfig = require("Metadatas.EventListConfig")

local GamePlayer = require("Player.GamePlayer")
local ActivityManager = require("Activity.ActivityManager")
local NewActivityManager = require("NewActivity.NewActivityManager")
local EntrantManager =  require("LegionEntrant.EntrantManager")

function This:init( id )
	super.init(self)
	self.m_escape = 0
    self.m_id = id
	self.m_config = EventListConfig[self.m_id]

	self.m_eType = 0
	self.m_endTime = 0
	self.m_startTime = 0
	self.m_state = ActTrailerState.kInvalid
	local _state = false

	if self.m_config then
    	self.m_eType = self.m_config.etype
    	self.m_uiConfig = self.m_config.ui_file
		if self:getInterval() > 0 then
            local time = self:getReferenceTime()
            if time then
			    self.m_startTime = time - self:getInterval() * SECONDS_PER_DAY
			    self.m_endTime = time
			end
		else
			if self.m_eType == 2 then
				if self.m_config.act_id > 0 then
					local queue = ActivityManager:getActivityQueue(queueId) or NewActivityManager:getActivity(queueId)
				    self.m_startTime = GamePlayer:getServerTime()
				    self.m_endTime = queue:getStartTime()
				end
			elseif self.m_eType == 1 then
				local startTime, endTime = self:getReferenceTime()
				-- if id == 9 then
				-- 	_state,endTime,startTime = EntrantManager:getFirstPreviewTimeForEm(GameMapType.kEmperorMap)
				-- elseif id == 10 then
				-- 	_state,endTime,startTime = EntrantManager:getFirstPreviewTimeForEm(GameMapType.kRescueMap)
				-- 	if type(endTime) == "number" then
				-- 		endTime = endTime + (7 * 86400)
				-- 	end
				-- end
				-- if type(endTime) == "string" then
				-- 	endTime = 0
				-- end
			    self.m_startTime = startTime or 0 
			    self.m_endTime = endTime or 0
			else
			    self.m_startTime = self.m_config.start_time
			    self.m_endTime = self.m_config.end_time
			end
		end

		self:updateState()
	end
end

function This:getState()
	if self.m_startTime < 0 and self.m_endTime < 0 then
		return ActTrailerState.kUpdating
	end

	local curTime = GamePlayer:getServerTime()
	if curTime > self.m_endTime then
		return ActTrailerState.kInvalid
	elseif curTime < self.m_startTime then
		return ActTrailerState.kNotOpen
	end
	return ActTrailerState.kUpdating
end

function This:updateState()
	self:updateTimeInfo()
	self.m_state = self:getState()
end

function This:updateTimeInfo()
	if self.m_id > 0 and self.m_startTime > 0 then
		local curTime = GamePlayer:getServerTime()
		if curTime < self.m_endTime then
			if not self.m_isUpdating then
				self:startUpdate()
			end
			return
		end
	end

	if self.m_isUpdating then
		self:endUpdate()
	end  
end

function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
	    local curTime = GamePlayer:getServerTime()
	    local leftOpenTime = self.m_startTime - curTime
	    if leftOpenTime > 0 then
	    	self:notifyObservers("updateOpenTime", leftOpenTime)
	    else
	    	if self.m_state == ActTrailerState.kNotOpen then
	    		self:updateState()
	    		self:notifyObservers("onTrailerOpenning", self)
	    	end

	    	local leftTime = self.m_endTime - curTime
	    	if leftTime > 0 then
				self:notifyObservers("updateTime", leftTime)
			else
	            self:updateState()
	        end
		end
	end
end

function This:startUpdate()
	cc.Application:addToUpdateList(self)
	self.m_isUpdating = true
	self:notifyObservers("onTrailerUpdateStart", self:getId())
end

function This:endUpdate()
	cc.Application:removeFromUpdateList(self)
	self.m_isUpdating = false
	self:notifyObservers("onTrailerUpdateEnd", self:getId())
end

function This:isRunning()
	return self.m_state == ActTrailerState.kUpdating
end

function This:getId()
	return self.m_id
end

function This:getName()
    return self.m_config.name
end

function This:getType()
    return self.m_eType
end

function This:getConfig()
    return self.m_config 
end

function This:getUIConfig()
    return self.m_uiConfig
end

function This:getPriority()
    return self.m_config.priority
end

function This:getLeftTime()
    local curTime = GamePlayer:getServerTime()
    local offset = self.m_endTime - curTime
    if offset < 0 then
        return 0
    end
    return offset
end

function This:getInterval()
	return self.m_config.interval_day
end

function This:getStartTime()
	return self.m_startTime
end

function This:getEndTime()
	return self.m_endTime
end

function This:getReferenceTime()
	local checkConfig = self.m_config.interval_check
	if checkConfig then
    	return checkConfig.func()
    end
end

