-- @Author: superm
-- @Date:   2018-03-15 22:04:54
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-20 18:47:40
-- @File Name: ActivityTrailerBaseNode.lua
-- @Description: 

local This, super = class(..., "NodeUI")
local GamePlayer = require("Player.GamePlayer")
local ActivityManager = require("Activity.ActivityManager")

function This:initData(trailer)
	if trailer then
        self.m_config = trailer:getConfig()
		self.m_upEndTime = trailer:getEndTime()
        self.m_eType = trailer:getType()
        self.m_escape = 0
	    self.m_hasTimeUpdate = self:getLeftTime() > 0
	    self.m_inited = false
	    self:initCsNode(trailer)
	end
end

function This:initCsNode(trailer)
	local config = trailer:getUIConfig()
    local fileName = "MenuScene/NodeUI/" .. config.cs .. ".csb"
    self.m_csNode = cc.Node:createFromCSFile(fileName)
    self:addChild(self.m_csNode)
end

function This:initNode()
	if self.m_eType == 1 then
		ActivityManager:setPalyerActInfo("trailer_nocheck", true, true)
		self:initSeparatedNode()
	elseif self.m_eType == 2 or self.m_eType == 3 then
		self:initNormalNode()
	end
end

function This:onEnter()
	super.onEnter(self)
	if self.m_eType == 1 then
		if self.m_hasTimeUpdate then
			self:scheduleUpdate()
		end
	end
end

function This:unscheduleUpdate( ... )
    print("unscheduleUpdate")
    super.unscheduleUpdate(self)
end

function This:initSeparatedNode()
    local csNode = self.m_csNode
	if csNode.guize_title then
		csNode.guize_title:setString(self.m_config.rule_dot_title)
	end
	if csNode.guize_text then
		csNode.guize_text:setString(self.m_config.rule_text, 1200)
	end
	if self.m_hasTimeUpdate then
		local leftTime = self:getLeftTime()
		self:updateQueueTime( leftTime )
	end
end

function This:initNormalNode( ... )
    local csNode = self.m_csNode
	csNode.act_title:setImage(self.m_config.ui_file.bg)
    if csNode.rule then
    	csNode.rule.title:setString(self.m_config.rule_dot_title)
    	csNode.rule_text:setString(self.m_config.rule_text, 1200)
    	csNode.rule_time:setString(self.m_config.rule_time_desc)
    end

    if csNode.reward then
    	csNode.reward.title:setString(self.m_config.reward_dot_title)
    	csNode.reward_desc:setString(self.m_config.reward_desc, 1200)
    end
end

function This:update(dt)
	super.update(self, dt)
	if self.m_hasTimeUpdate then
		self.m_escape = self.m_escape + dt
		if self.m_escape >= 1.0 then
			self.m_escape = self.m_escape - 1
			local leftTime = self:getLeftTime()
			if leftTime > 0 then
				self:updateQueueTime(leftTime)
			else
				self:endUpdate()
			end
		end
	end
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_upEndTime then
		return 0
	end
	return self.m_upEndTime - curTime
end

function This:endUpdate()
    if self.m_hasTimeUpdate then
		self.m_hasTimeUpdate = false
	end
end

function This:updateQueueTime( leftTime )
    local timeNode = self.m_csNode.time
	if timeNode then
		local numList, nodeList = {}, {timeNode.day, timeNode.hour, timeNode.mine, timeNode.second}
	    numList[1] = math.floor(leftTime / 86400)
	    numList[2] = math.floor((leftTime - numList[1] * 86400) / 3600)
	    numList[3] = math.floor((leftTime - numList[1] * 86400 - numList[2] * 3600) / 60)
	    numList[4] = leftTime - numList[1] * 86400 - numList[2] * 3600 - numList[3] * 60

		for i, x in ipairs(numList) do
			local numPicList = {"MenuScene/num_" .. math.floor(x / 10) .. ".png", "MenuScene/num_" .. x % 10 .. ".png" }
			for index, pic in ipairs(numPicList) do
				if nodeList[i] then
					nodeList[i]["bg" .. index].num:setImage(pic)
				else
					print(i .. " not exist")
				end
			end
		end
	end    
end