-- @Author: superm
-- @Date:   2018-03-15 23:10:54
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-02 20:28:54
-- @File Name: ActivityTrailerBtn.lua
-- @Description: 事件簿

local This, super = class(..., "Home.MenuButton")
local EventListConfig = require("Metadatas.EventListConfig")
local GamePlayer = require("Player.GamePlayer")
local ActivityTrailer = require("NewActivity.ActivityTrailer")

function This:init(config)
    super.init(self, config)
    self.m_autoClose = config.uiclass.auto_close
    self:refreshData()
end

function This:initTouchEvent()
    self:setTouch({
        isScale = false, 
        isDynamic = true, 
        onEnded = function()
        	local curTime = GamePlayer:getServerTime()
        	local curOrderList = {}
        	for i, trailer in ipairs(self.m_queueList) do
				local config = trailer:getConfig()
                --屏蔽军团战事件簿
				if trailer:isRunning() then
                    if trailer:getId() ~= 8 then
					   table.insert(curOrderList, trailer)
                    end
				end
                if trailer:getState() == 1 then
                    if trailer:getId() == 9 then--or trailer:getId() == 10 then
                       table.insert(curOrderList, trailer)
                    end
                end
        	end
        	g_scene:openDialog("NewActivity.ActivityTrailerPanel", curOrderList)
        end
    }) 
end

function This:close()
    if self.m_subView and self.m_subView[0] ~= 0 then
        self.m_subView:close()
    end
    self:setVisible(false)
end

function This:open()
    self:setVisible(true)
end

function This:addCustomListener()
    super.addCustomListener(self)
    if self.m_autoClose then
    	if self.m_queueIdList then
    		for id, queue in pairs(self.m_queueList) do
    			queue:addObserver(self)
    		end
    	end
    end
end

function This:removeCustomListener()
	if self.m_queueIdList then
		for id, queue in pairs(self.m_queueList) do
			queue:removeObserver(self)
		end
	end
    super.removeCustomListener(self)
end


function This:refreshData()
	self.m_isAllQueueClose = true
	self.m_allwaysOpen = false

	if not self.m_queueList then
		self.m_queueList = {}
		for id, config in ipairs(EventListConfig) do
			local trailer = ActivityTrailer:create(id)
			self.m_queueList[id] = trailer
			if trailer:getEndTime() < 0 then
				self.m_allwaysOpen = true
				self.m_isAllQueueClose = false
			elseif trailer:isRunning() then
				self.m_isAllQueueClose = false
			end
		end
	end
end

function This:refreshActListState()
    local isAllClose = true

    for id, queue in pairs(self.m_queueList) do
        if queue:isRunning() then
            isAllClose = false
            break
        end
    end

    self.m_isAllQueueClose = isAllClose
    if isAllClose then
    	if not self.m_allwaysOpen then
        	self:close()
        end
    end
end

function This:getDefaultVisible()
	if self.m_allwaysOpen then
		return true
	end 

	if self.m_queueIdList then
		for id, queue in pairs(self.m_queueList) do
			if queue:isUpdating() or queue:isRunning() then
				return true
			end
		end
	end

    return super.getDefaultVisible(self)
end

function This:getRedDotValue()
    local value = super.getRedDotValue(self)
    local ret = value
    if self.m_autoClose then
        ret = value and true or nil
    end
    return ret
end

function This:onTrailerUpdateStart( queueId )
    if self.m_autoClose then
        for _, queue in pairs(self.m_queueList) do
            if queue:getId() == queueId then
                self:open()
                break
            end
        end
    end
end

function This:onTrailerUpdateEnd( queueId )
    if self.m_autoClose then
        local found = false
        for _, queue in pairs(self.m_queueList) do
            if queue:getId() == queueId then
                found = true
                break
            end
        end

        if found then
            self:refreshActListState()
        end
    end 
end