-- @Author: superm
-- @Date:   2018-03-15 20:36:48
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-02 20:08:19
-- @File Name: ActivityTrailerPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ViewUtil = require("UI.ViewUtil")
local EventDispatcher = require("EventDispatcher")
local RedDotUtil = require("RedDot.RedDotUtil")

local redDotBtnList = {}

function This:initData(orderList)
    self.m_orderList = orderList
end

function This:addActTabBtn( index, info )
    local rootNode = self.m_csNode["act" .. tostring(info.id)]
    local posy = 184.23 - 59 * ( index - 1 )
    if not rootNode then
        rootNode = cc.Sprite:create("MenuScene/huodongweidianji.png") 
        rootNode:setAnchorPoint(0.5, 0.5)
        rootNode:setPosition(-305.57, posy)
        self.m_csNode["act" .. tostring(info.id)] = rootNode
        rootNode:setName("act" .. tostring(info.id))
        self.m_csNode:addChild(rootNode)
        
        local lightSprite = cc.Sprite:create("MenuScene/huodongxuanzhong.png") 
        lightSprite:setAnchorPoint(0.5, 0.5)
        lightSprite:setPosition(81, 32)
        rootNode.light = lightSprite
        rootNode:addChild(lightSprite)
        
        local normalabel = cc.Label:createNormalTTF(info.name, 20)
        normalabel:setAnchorPoint(0.5, 0.5)
        normalabel:setPosition(81, 32)
        normalabel:setColor(136, 152, 190)
        rootNode.normal = normalabel
        rootNode:addChild(normalabel)

        local lightlabel = cc.Label:createNormalTTF(info.name, 20)
        lightlabel:setAnchorPoint(0.5, 0.5)
        lightlabel:setPosition(83, 34)
        lightlabel:setColor(255, 255, 255)
        rootNode.light.light = lightlabel
        rootNode.light:addChild(lightlabel)
    else
        rootNode.normal:setString(info.name)
        rootNode.light.light:setString(info.name)
    end

    return rootNode
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/EventList1.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    self:addTabButtonList(self.m_orderList)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
end

function This:addTabButtonList(orderList)
    local defaultTabId = 1

    local ihasClick,ihasClick2 = RedDotUtil:isOpenInOneDay(true)
    
    for index, trialer in ipairs(orderList) do
        local tabName = tostring(index)
        local btn = self:addActTabBtn( index, trialer:getConfig() )
        if btn then
            self:addTabButton(tabName, btn, function()
                self.m_curTabId = trialer:getId()
                self.m_curTabIndex = index
                self:selectTabById(index, true)
                return true
            end)
            if self.m_isOpen and not btn.m_touch then
                btn:setTouch({
                    onEnded = function ()
                        self:selectTabButton(tabName)
                    end
                })
            end

            local m_bonusRedDot = GameUtil:createRedDotTo(btn)
            local redDotWidth, redDotHeight = m_bonusRedDot:getContentSize()
            local width, height = btn:getContentSize()
            m_bonusRedDot:setPosition(width-redDotWidth,height-redDotHeight)
            m_bonusRedDot:setVisible(false)

            if trialer:getId() == 9 then
                if ihasClick > 0 then
                    m_bonusRedDot:setVisible(true)
                end
            elseif trialer:getId() == 10 then
                if ihasClick2 > 0 then
                    m_bonusRedDot:setVisible(true)
                end
            end
            redDotBtnList[index] = m_bonusRedDot
            -- if index > 1 then
            --     self.m_timer:setTimer(0.3 + 0.05 * index, function()
            --         self:addTabNode(tabName)
            --     end, "ActivityPanelTab" .. tabName)
            -- end
        end
    end


    self:selectTabButton(tostring(defaultTabId))
end

function This:createTabNode(name)
    local trailer = self.m_orderList[tonumber(name)]
    return require("NewActivity.ActivityTrailerBaseNode"):create(trailer)
end

function This:selectTabById( id )
    if redDotBtnList[id] then
        if redDotBtnList[id].setVisible then
            redDotBtnList[id]:setVisible(false)
            if self.m_curTabId == 9 then
                cc.FileUtils:saveToLua({time = GamePlayer:getServerTime()},"loginServerTimeKing")
            elseif self.m_curTabId == 10 then
                cc.FileUtils:saveToLua({time = GamePlayer:getServerTime()},"loginServerTimeQW")
            end
            EventDispatcher:dispatcher("activity_trailer_state_changed")
        end
    end
    
    self:openTabNode(tostring(id))
end