-- @Author: superm
-- @Date:   2018-03-14 17:23:43
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-16 10:22:50
-- @File Name: ActivityWestShop.lua
-- @Description: 

local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local WestShopItem = require("NewActivity.WestShopItem")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local WestShopRefreshConfig = require("Metadatas.WestShopRefreshConfig")

function This:getMaxTotalBuyTimes()
    return 10
end

function This:init(id)
	super.init(self, id)
	if self.m_descConfig then
	    self.m_startTime = self.m_descConfig.start_time or 0
	    self.m_endTime = self.m_descConfig.end_time or 0
	end
	self.m_slotList = {}
	self.m_curMaxTotalBuyTime = self:getMaxTotalBuyTimes()
	self.m_curTotalBuyTime = 0
	self.m_curRefreshTime = 0
end

function This:initListener()
	EventDispatcher:addListener(self,12027,self.onGetActivityInfoReply)
	EventDispatcher:addListener(self,12028,self.onBuyActShopItemReply)
	EventDispatcher:addListener(self,12035,self.onRefreshWestShopReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,12027)
	EventDispatcher:removeListener(self,12028)
	EventDispatcher:removeListener(self,12035)
end

function This:getState()
	local curTime = GamePlayer:getServerTime()
	local startDate = os.serverdate("*t", self.m_startTime)
    local curDate = os.serverdate("*t", curTime)
    if startDate.year == curDate.year and startDate.yday == curDate.yday then
		local state = NewActivityState.kUpdating
		if curTime > self.m_endTime then
			state = NewActivityState.kEnded
		elseif curTime < self.m_startTime then
			state = NewActivityState.kNotStart
		end

		if state == NewActivityState.kUpdating then
			if GamePlayer:getLevel() < self.m_config.level_min then
				return NewActivityState.kNotOpen
			end
		end
		return state
	elseif startDate.year > curDate.year or (startDate.year == curDate.year and startDate.yday > curDate.yday) then 
		if startDate.year == curDate.year and startDate.yday - curDate.yday == 7 then
			return NewActivityState.kEnded
		else
			return NewActivityState.kNotOpen
		end
	else
		return super.getState(self)
	end
end

function This:isNotStart()
    return NewActivityState.kNotStart == self:getState()
end

function This:getSlotCount()
    return #self.m_slotList
end

function This:getAllSlotItem()
    return self.m_slotList
end

function This:getSlotItemAtIndex(i)
    return self.m_slotList[i]
end

function This:onGetActivityInfoRequest()
	Socket:sendAndLock(12027,{activity_id = self:getId()})
end

function This:onGetActivityInfoReply( msg )
	if msg and msg.activity_id == self:getId() then
		if msg.err_code == 0 then
			self.m_slotList = self.m_slotList or {}
			self.m_curMaxTotalBuyTime = msg.max_buy_times or self:getMaxTotalBuyTimes()
			self.m_curTotalBuyTime = msg.total_buy_times or 0
			self.m_curRefreshTime = msg.refresh_times or 0
			for i, info in ipairs(msg.shop_item_list) do
				local item = self.m_slotList[info.grid_id]
				if not item then
					item = self:getNewActivityItem(info)
					item:setActivityQueue(self)
					self.m_slotList[item:getId()] = item 
				else
					item:updateInfo(info)
				end
			end
			self:notifyObservers("onGetActShopInfo", self:getId())	
		else
        	g_scene:handleErrorCode(msg.err_code)
		end
	end
end

function This:getNewActivityItem( ... )
    return WestShopItem:create( ... )
end

function This:onBuyActShopItemReply(msg)
	if msg and msg.activity_id == self:getId() then
		if msg.err_code == 0 then
			if msg.gold <= 0 and msg.item_num <= 0 then
				g_scene:showTips(Serialize[62058])
			end

			local item = self.m_slotList[msg.grid_id]
			if item then
				item:increaseBuyTime()
				g_scene:showReward({{id=item:getItemId(), count=item:getItemNum()}})
			end

			self.m_curTotalBuyTime = self.m_curTotalBuyTime + 1
			self:notifyObservers("onBuyActShopItem", self:getId())	
		else
        	g_scene:handleErrorCode(msg.err_code)
		end
	elseif msg.activity_id == 0 and msg.err_code ~= 0 then
		msg.activity_id = self:getId()
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onRefreshWestShopRequest()
	Socket:sendAndLock(12035,{activity_id = self:getId()})
end

function This:onRefreshWestShopReply(msg)
	if msg and msg.activity_id == self:getId() then
		if msg.err_code == 0 then
			self.m_curRefreshTime = msg.refresh_times or 0
			self.m_curTotalBuyTime = msg.total_buy_times or 0
			for i, info in ipairs(msg.shop_item_list) do
				local item = self.m_slotList[info.grid_id]
				if not item then
					item = WestShopItem:create(info)
					item:setActivityQueue(self)
					self.m_slotList[item:getId()] = item 
				else
					item:updateInfo(info)
				end
			end
			self:notifyObservers("onRefreshWestShop", self)	
		else
        	g_scene:handleErrorCode(msg.err_code)
		end
	end
end

function This:getRefreshCost()
	local config = WestShopRefreshConfig[self.m_curRefreshTime + 1]
	if config then
    	return config.price
    else
    	config = WestShopRefreshConfig[#WestShopRefreshConfig]
    	return config.price
    end
end

function This:getLeftTotalBuyTime()
    return self.m_curMaxTotalBuyTime - self.m_curTotalBuyTime
end

function This:canBuy()
    return self:getLeftTotalBuyTime() > 0
end
