--region *.lua
--Date
--此文件由[BabeLua]插件自动生成
--ActivityWuDaoShop.lua
local This , super = class(...,"NewActivity.ActivityBase")
local GamePlayer = require("Player.GamePlayer")
local EventDispatcher = require("EventDispatcher")
local ActivityWuDaoShopItem = require("NewActivity.ActivityWuDaoShopItem")
local ActivityTimeConfig = require("Metadatas.ActivityTimeConfig")
local ActivityTypeConfig = require("Metadatas.ActivityTypeConfig")
local Socket = require("Net.Socket")

local WeekDayOpen = {4,6,7,1}

function This:init()
	super.init(self)
	self.m_id = self:getConfigId()
	self.m_escape = 0
	self.m_state = NewActivityState.kNotOpen
    self.m_startTime = 0
    self.m_endTime = 0
	self.m_itemList = {}
	self.m_activityIdList = {}
	self:initListener()
end

function This:getConfigId()
	local curTime = GamePlayer:getServerTime()
	local t = os.serverdate("*t",curTime)
	for i , weekDay in ipairs(WeekDayOpen) do
		if weekDay == t.wday then
			return NewActivityType.kWuDaoShop1 + i - 1
		end
	end
	return nil
end

function This:initListener()
	EventDispatcher:addListener(self,12033,self.onGetActivityInfoReply)
	EventDispatcher:addListener(self,12034,self.onBuyShowItemReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,12033)
	EventDispatcher:removeListener(self,12034)
end

function This:getItemList()
	return self.m_itemList
end

function This:getItemWithConfigId(configId)
	for i , item in ipairs(self.m_itemList) do
		if configId == item:getConfigId() then
			return item
		end
	end
end

function This:updateInfo(info)
	for i , info in ipairs(info.item_list) do
		if not self.m_itemList[info.grid_id] then
			local item = ActivityWuDaoShopItem:create(info.grid_id)
			item:updateInfo(info)
			self.m_itemList[info.grid_id] = item
		else
			self.m_itemList[info.grid_id]:updateInfo(info)
		end
	end
	for i = #self.m_itemList , #info.item_list + 1 , -1 do
		table.remove(self.m_itemList,i)
	end 
	self:updateState()
end

function This:preWork()
	self.m_activityIdList = {}
end

function This:setTime(startTime,endTime,activityId)
	self.m_activityIdList[activityId] = {startTime = startTime, endTime = endTime}
end

function This:work()
	local curTime = GamePlayer:getServerTime() 
	local isFind = false
	local nextId = nil
	local nextTime = nil
	self.m_id = nil
	self.m_config = nil
	self.m_descConfig = nil
	local configId = self:getConfigId()

	for id , info in pairs(self.m_activityIdList) do
		if curTime >= info.startTime and curTime < info.endTime then
			self.m_id = id
			self.m_config = ActivityTimeConfig[self.m_id]
			self.m_descConfig = ActivityTypeConfig[self.m_id]
			super.setTime(self,info.startTime,info.endTime)
			isFind = true
		end
	end
	if not isFind then
		if  configId and self.m_activityIdList[configId] then
			self.m_id = configId
			self.m_config = ActivityTimeConfig[self.m_id]
			self.m_descConfig = ActivityTypeConfig[self.m_id]
			super.setTime(self,self.m_activityIdList[self.m_id].startTime,self.m_activityIdList[self.m_id].endTime)
		else
			super.setTime(self,0,0)
		end
	end
end

function This:isLevelOpen()
	local limitLevel = ActivityTypeConfig[NewActivityType.kWuDaoShop1].open_level
	local level = GamePlayer:getLevel()
	if level < limitLevel then
		return false, string.format(Serialize[61980],limitLevel)
	end
	local state = self:getState()
	if state ~= NewActivityState.kUpdating then
		if self.m_id  then
			local t1 = os.serverdate("*t",self.m_startTime)
			local t2 = os.serverdate("*t",GamePlayer:getServerTime())
			if t1.day ~= t2.day then
				return false, Serialize[62060]
			else
				if state == NewActivityState.kNotStart then
					return false, Serialize[62061]
				else
					return false, Serialize[62060]
				end
			end
		else
			return false, Serialize[62062]
		end
	end

	if self.m_descConfig.audit and GamePlayer:isAuditVersion() then
		return false, Serialize[61978]
	end
	return true
end

---------------------------------------------------

function This:onGetActivityInfoRequest(activityId)
	if not activityId or activityId == self.m_id then
		Socket:sendAndLock(12033,{activity_id = self.m_id})
	end
end

function This:onGetActivityInfoReply(msg)
	if msg.err_code == 0 then
		self:updateInfo(msg)
		self:notifyObservers("onRefreshActivityInfo")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onBuyShowItemRequest(itemConfigId,costItemId,costItemNum)
	Socket:sendAndLock(12034,{activity_id = self.m_id,
		config_id = itemConfigId,
		price_type = costItemId,
		price_num = costItemNum
	})
end

function This:onBuyShowItemReply(msg)
	if msg.err_code == 0 then
		local item  = self:getItemWithConfigId(msg.config_id)
		item:setCostItem(msg.price_type,msg.price_num)
		local itemId , num = item:getBuyItem()
		g_scene:showReward({{id = itemId , num = num}})
		self:notifyObservers("onBuyShopItem",item)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
	self:onGetActivityInfoRequest()
end	

--endregion
