-- @Author: Su
-- @Date:   2019-05-15 15:05:28
-- @Last Modified by:   Su
-- @Last Modified time: 2019-05-15 15:05:28
-- @File Name: ActivitykAnimalTable.lua
-- @Description: 

local This , super = class(...,"NewActivity.ActivityPrizeDraw")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local AnimalsActivityItemConfig = require("Metadatas.AnimalsActivityItemConfig")
local AnimalsActivityLuckyConfig = require("Metadatas.AnimalsActivityLuckyConfig")
local AnimalsActivityConfig = require("Metadatas.AnimalsActivityConfig")
local AnimalsActivityMailConfig = require("Metadatas.AnimalsActivityMailConfig")

local DrawInfoConfig = {
    [NewActivityType.kAnimalTable] = {
        keyItemId = 102, 									--道具的Id
		useResource = true, 								--道具的类型是否游戏资源
		keyItemNumLimit = 1000, 								--每日获得道具的上限
		diffPrice = true,
		defaultDtype = ActPrizeDrawType.kOnce,			--默认抽奖的类型
		keyItemPrice = function( dType ) 					--每次抽奖消耗道具的个数
			return dType == ActPrizeDrawType.kOnce and AnimalsActivityConfig[1].onetime_cost[2] or AnimalsActivityConfig[1].tentime_cost[2]
		end, 
		freeTimes = function( dType )						--是否有免费次数
			return dType == ActPrizeDrawType.kOnce and 1 or 0
		end,
		hasFreeTimesPerDay = false, 							--免费次数是否每日重置
		drawConfig =  "Metadatas.AnimalsActivityItemConfig",		--使用的配置文件
		uiType = "NewActivity.AnimalsTurnTableNode",	--使用的界面
		warmingFile = "ActivityFestivalLanternreward",		--使用提示界面文件
		tipsWord = "activity_normalprizedraw_help_tips",	--使用提示文字
		csbFileName = "MenuScene/NodeUI/AnimalsTurnTable.csb",
    }
}

AnimalProportionType =
{
    kDevout = 1,
    kBless = 2,
} 

function This:init(id)
	super.init(self, id)
	self.m_id = id
	self.m_rewardPool = 1
	self.m_todayKeyItemNum = 0
	self.m_drawTimeInHistory = 0
	self.m_blessTime = 0
	self.m_jifen = 0
	self.m_customConfig = DrawInfoConfig[self:getId()]
	self.m_useFreeTimes = 0
	self.m_exchangeNum = 0
	self.m_rankRoleList={}
	self.m_itemList = {}
	-- print(self:getHelpTips())


end

function This:initListener()
	EventDispatcher:addListener(self, 12053, self.onActivityPrizeDrawReply)--随机抽奖活动领奖
    EventDispatcher:addListener(self, 12056, self.onGetActivityInfoReply)--查看随机抽奖活动领奖
	EventDispatcher:addListener(self, 12054, self.onGetAnimalRewardReply)--领取箱子奖励
	EventDispatcher:addListener(self, 12055, self.onBlessingReply)--祝福
	EventDispatcher:addListener(self, 12064, self.onExchangeReply)--兑换

	EventDispatcher:addListener(self, "after_recharge", self.onAfterRechargeReply)
	EventDispatcher:addListener(self, "equip_changed", self.onGetBagDataChange)
end

function This:removeListener()
	EventDispatcher:removeListener(self, "equip_changed")
	EventDispatcher:removeListener(self, "after_recharge")
	EventDispatcher:removeListener(self, 12053)
	EventDispatcher:removeListener(self, 12054)
	EventDispatcher:removeListener(self, 12056)
	EventDispatcher:removeListener(self, 12064)
end

function This:getLuckNum()

    return self.m_jifen or 0
end

function This:getLuckBestReward()
	return AnimalsActivityMailConfig[self.m_rewardPool].lucky_reward[1][1]
end

-- 神兽转盘宝箱红点
function This:hasBoxCanReceive()
	if self.m_blessTime > 0 and #self.m_rankRoleList > 0 then
		return true
	end
	for index,boxConfig in ipairs(AnimalsActivityLuckyConfig) do
		if self.m_jifen >= boxConfig.lucky_count and not self:getBoxHaxReceive(boxConfig.id) then
			return true
		end
	end
	return false

end

function This:isBoxCanReceive(id)
	return self.m_jifen >= AnimalsActivityLuckyConfig[id].lucky_count and not self:getBoxHaxReceive(id)
end

-- 获取玩家虔诚值
function This:getDevoutNum()
    return self.m_devoutNum or 30
end

-- 获取玩家祝福值
function This:getBlessNum()
    return self.m_blessNum or 0
end

-- 获取祝福次数
function This:getBlessTimes()
    return self.m_blessTime or 0
end

-- 获取抽奖记录
function This:getRewardStringList()
	return self.m_rewardStringList
end

-- 获取奖池道具信息
function This:getDrawItemList()
	local index = 1
	for id,item in ipairs(AnimalsActivityItemConfig) do
		if item.reward_pool == self.m_rewardPool then
			self.m_itemList[index] = item
			index = index + 1 
		end
	end
	return self.m_itemList
end

-- 根据排行获取虔诚值或祝福值和比例
function This:getProportion(rankNum,type)
	local maxDevoutNum,maxBlessNum = 0,0
	for _,roleData in pairs(self.m_rankRoleList) do
		maxDevoutNum = maxDevoutNum + roleData.value
		maxBlessNum = maxBlessNum + roleData.bened_value
	end
	if #self.m_rankRoleList > 0 and self.m_rankRoleList[rankNum] then
		if type == AnimalProportionType.kDevout then
			return self.m_rankRoleList[rankNum].value,self.m_rankRoleList[rankNum].value/maxDevoutNum*100
		elseif type == AnimalProportionType.kBless then
			return self.m_rankRoleList[rankNum].bened_value,self.m_rankRoleList[rankNum].bened_value/maxBlessNum*100
		end
	end
	return 0,0
end

function This:getRoleInfo(index)
	return self.m_rankRoleList[index]
end

function This:getIndexById(id,num)

	for index,item in ipairs(self.m_itemList)do
		if item.item_id == id and item.num == num then
			return index
		end
	end
end

function This:getLuckyBoxConfigByIndex(index)
	return AnimalsActivityLuckyConfig[index]
end

function This:getBoxHaxReceive(id)
	if self.m_rewardList then
		for _,boxId in pairs(self.m_rewardList) do
			if boxId == id then
				return true
			end
		end
	end
	return false
end

function This:getShowReward()
	return self.m_reward,self.m_rewardCount
end

function This:getLeftTimes()
	return DepotManager:getItemCount(299)
end

---------------------------------------------------------------------------------------------------------------------
function This:onGetActivityInfoRequest()
	Socket:send(12056, {activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
    if msg.err_code == 0 then
		self.m_rankRoleList = msg.rank_role_list
		self.m_blessTime = msg.bened_times
		self.m_jifen = msg.lucky_value
		self.m_rewardStringList = msg.record_list
		self.m_rewardList = msg.reward_list--已领取奖励
		self.m_rewardPool = msg.pool_id
		self.m_devoutNum = msg.buy_times--虔诚值
		self:notifyObservers("onRefreshPrizeDraw", self:getId())
		EventDispatcher:dispatcher("activity_activityanimal_state_changed")        

    else
		g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onActivityPrizeDrawRequest(drawType)
	if not self:isUpdating() then
		return
	end

	if not drawType then
		drawType = ActPrizeDrawType.kOnce
	end
    Socket:sendAndLock(12053, {draw_times= drawType})
end

function This:onActivityPrizeDrawReply(msg)
	if msg.err_code == 0 then
		self:onGetActivityInfoRequest()
		self:notifyObservers("onActivityPrizeDraw", {id = self:getId(),reward = msg.item_list,rewardCount =msg.count_list })
		self.m_reward = msg.item_list
		self.m_rewardCount = msg.count_list
		EventDispatcher:dispatcher("activity_activityanimal_state_changed")        

	else
    	g_scene:handleErrorCode(msg.err_code)
		-- self:notifyObservers("onActivityPrizeDraw", {id = self:getId()})		
	end

end

function This:onBlessingRequest(index)
	if not self.m_rankRoleList[index] then
		g_scene:showTips(Serialize[62022])
		return
	end
	local roleId = self.m_rankRoleList[index].id
	Socket:sendAndLock(12055, {target_role_id= roleId})
end

function This:onBlessingReply(msg)
    if msg.err_code == 0 then
		g_scene:showReward(AnimalsActivityConfig[1].support_reward)
		self:notifyObservers("onRefreshPrizeDraw", self:getId())
		EventDispatcher:dispatcher("activity_activityanimal_state_changed")        
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetAnimalRewardRequest(id)
	Socket:sendAndLock(12054, {box_id= id})
end

function This:onGetAnimalRewardReply(msg)
    if msg.err_code == 0 then
		table.insert(self.m_rewardList,msg.box_id)
		g_scene:showReward(self:getLuckyBoxConfigByIndex(msg.box_id).reward)
		self:notifyObservers("onRefreshPrizeDraw", self:getId())
		EventDispatcher:dispatcher("activity_activityanimal_state_changed")        
	else
		g_scene:handleErrorCode(msg.err_code)
	end

end

function This:onExchangeRequest(itemId,num)
	self.m_exchangeNum = num
	Socket:sendAndLock(12064, {excharge_itemid= itemId,count = num})

end

function This:onExchangeReply(msg)
	if msg.err_code == 0 then
		g_scene:showReward({{id=msg.excharge_itemid,num=self.m_exchangeNum}})
		self:notifyObservers("onExchangeAnimal", self:getId())
	else
		g_scene:handleErrorCode(msg.err_code)

	end
end