-- @Author: Su
-- @Date:   2019-03-27 16:30:43
-- @Last Modified by:   Su
-- @Last Modified time: 2019-03-26 16:30:43
-- @File Name: ActivitykCumulative.lua
-- @Description: 

local This , super = class(...,"NewActivity.ActivityBase")
local NewActivityManager = require("NewActivity.NewActivityManager")
local EventDispatcher = require("EventDispatcher")
local GamePlayer = require("Player.GamePlayer")
local ActivitykSpendGold = require("NewActivity.ActivitykSpendGold")
local Socket = require("Net.Socket")
local OpenTaskConfig = require("Metadatas.OpenTaskConfig")
local SevenDayActivityChapter =require("NewActivity.SevenDayActivityChapter")


function This:init(id)
    super:init(id)
    self.m_id = id
    self:initListener()
    self.m_rewardList = {}
    self.m_completeList = {}
    local curTab = 1
    for _,config in pairs(OpenTaskConfig) do
        if config.activity_id == NewActivityType.kCumulative then
            local task = SevenDayActivityChapter:create(config.id)
            self.m_rewardList[curTab] = task
            curTab = curTab + 1
        end
    end
    table.sort(self.m_rewardList,function (a,b)
        return a:getId() < b:getId()
    end)
end

function This:initListener()
	EventDispatcher:addListener(self, 12008, self.onGetActivityInfoReply)
    EventDispatcher:addListener(self, 12004, self.onPickupActRewardReply)
    GamePlayer:addObserver(self)
    ActivitykSpendGold:addObserver(self)
end

function This:removeListener()
	EventDispatcher:removeListener(self, 12008)
    EventDispatcher:removeListener(self, 12004)
    ActivitykSpendGold:removeListener(self)
    GamePlayer:removeListener(self)
end

function This:getId()
    return self.m_id
end

function This:getUITypeName()
    return "Activity.ActivitySevenDayCelebrationPanel"
end

function This:getTurnDatas()
    -- local res ={}
    -- local res2 ={}
    -- local index,num = 1,1
    -- for id,task in ipairs(self.m_rewardList) do
    --     if not self:hasTaskComplete(task.id) then
    --         res[index] = task
    --         index = index + 1
    --     end
    -- end
    -- for _,task in pairs(self.m_completeList) do
    --     res2[num] = OpenTaskConfig[task]
    --     num = num + 1
    -- end
    -- table.sort(res2,function (a,b)
    --     return a.id<b.id       
    -- end )
    -- for _,task in ipairs(res2) do
    --     res[index] = task
    --     index = index +1
    -- end
    -- return res
    self:refreshItemData()
    return self.m_rewardList
end

function This:refreshItemData()
    table.sort(self.m_rewardList,function(a,b)
        if a:getState() ~= b:getState() then
            return a:getState() < b:getState()
        else
            return a:getId() < b:getId()
        end 
    end)
end

function This:hasRewardCanReceive()
    for index,task in ipairs(self.m_rewardList) do
        if self:getMyConsumption() >= task:getProgress() and not self:hasTaskComplete(task:getId()) then
            return true
        end
    end
    return false
end

function This:getTaskById(id)
    for _,task in ipairs(self.m_rewardList) do
        if task:getId() == id then
            return task
        end
    end

end

function This:getReward(id)
    local reward = {}
    for _,rewardInfo in pairs(self.m_rewardList) do
        if id == rewardInfo.id then
            reward = rewardInfo.reward
            break
        end
    end
    return reward
end

function This:hasTaskComplete(taskId)
    -- if self.m_completeList[taskId] then
    --     return true
    -- else
    --     return false
    -- end
    return self:getTaskById(taskId):getState() == SevenDayActivityTaskState.kReceive
    -- return self.m_rewardList[taskId]:getState() == SevenDayActivityTaskState.kReceive
end

function This:onActivityPrizeDraw()
    self:onGetActivityInfoRequest()
end

------------------------------
function This:getMyConsumption()
    local activity = NewActivityManager:getActivity(NewActivityType.kSpendGold)
    return activity:getMyConsumption()
end

function This:onGetActivityInfoRequest()
	Socket:sendAndLock(12008,{activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
    if self:getId() ~= msg.activity_id then
		return
    end
    if msg.err_code == 0 then
        for _,task in pairs(msg.reward_list) do
            -- self.m_completeList[taskId] = taskId
            local item = self:getTaskById(task)
            item:setState(SevenDayActivityTaskState.kReceive)
        end
        self:notifyObservers("onRefreshCumulative",2)
		EventDispatcher:dispatcher("activity_activitysevenday_state_changed")        
    else
		g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onPickupActRewardRequest(rewardId)
	Socket:sendAndLock(12004,{activity_id = self:getId(),id =rewardId,type=8})
end

function This:onPickupActRewardReply(msg)
    if self:getId() ~= msg.activity_id then
		return
    end
    if msg.err_code == 0 then
        local reward = self:getReward(msg.id)
        -- self.m_completeList[msg.id] = msg.id
        self:getTaskById(msg.id):setState(SevenDayActivityTaskState.kReceive)
        g_scene:showReward(reward)
        self:notifyObservers("onRefreshCumulative",2)
		EventDispatcher:dispatcher("activity_activitysevenday_state_changed")                
    else
		g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.diamond then
        self:onGetActivityInfoRequest()
    end
end

function This:onGetActivityInfo(id)
    if id == NewActivityType.kSpendGold then
        self:refreshItemData()
    end

end

