-- @Author: Su
-- @Date:   2019-03-27 19:17:43
-- @Last Modified by:   Su
-- @Last Modified time: 2019-03-27 19:17:43
-- @File Name: ActivitykDailyRecharge.lua
-- @Description:

local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local GamePlayer = require("Player.GamePlayer")
local Socket = require("Net.Socket")
local GameUtil = require("Util.GameUtil")
local SevenDayActivityChapter =require("NewActivity.SevenDayActivityChapter")
local OpenTaskConfig = require("Metadatas.OpenTaskConfig")

function This:init(id)
    super:init(id)
    self.m_id = id
    self:initListener()
    self.m_payNum = 0
    self.m_rewardInfoList = {}
    self.m_rewardList = {}
    local curTab = 1
    for _,config in pairs(OpenTaskConfig) do
        if config.activity_id == NewActivityType.kDailyRecharge then
            local task = SevenDayActivityChapter:create(config.id)
            self.m_rewardList[curTab] = task
            curTab = curTab + 1
        end
    end
    table.sort(self.m_rewardList,function (a,b)
        return a.id < b.id
    end)
end

function This:initListener()
    EventDispatcher:addListener(self, 12008, self.onGetActivityInfoReply)
    EventDispatcher:addListener(self, 12004, self.onReceiveActivityRewardReply)
    EventDispatcher:addListener(self, "day_passed", self.onDayPassNotify)
    GamePlayer:addObserver(self)
    
end

function This:removeListener()
	EventDispatcher:removeListener(self, 12008)
    EventDispatcher:removeListener(self, 12004)
	EventDispatcher:removeListener(self, "day_passed")
    
end

function This:getId()
    return self.m_id
end

function This:getTaskById(id)
    for _,task in ipairs(self.m_rewardList) do
        if task:getId() == id then
            return task
        end
    end

end

function This:hasTaskComplete(taskId)
    -- if self.m_rewardInfoList[taskId] then
    --     return true
    -- else
    --     return false
    -- end
    return self:getTaskById(taskId):getState() == SevenDayActivityTaskState.kReceive

end

function This:getUITypeName()
    return "Activity.ActivitySevenDayCelebrationPanel"
end

function This:getMyConsumption()
    return self.m_payNum
end

function This:hasRewardCanReceive()
    local taskList = self:getTurnDatas()
    for _,task in ipairs(taskList) do
        if self:getMyConsumption() >= task.progress and not self:hasTaskComplete(task.id) then
            return true
        end
    end
    return false
end

function This:getTurnDatas()
    -- local curTime = GamePlayer:getServerTime() - self:getStartTime()
    -- local res = {}
    -- local res2 ={}
    -- local day = tonumber(GameUtil:formatDTime(Serialize["D"],curTime))+1
    -- local index,num = 1,1
    -- for _,reward in ipairs(self.m_rewardList) do
    --     if reward.open_num == day and not self:hasTaskComplete(reward.id) then
    --         res[index] = reward
    --         index = index + 1
    --     end
    -- end
    -- for _,task in pairs(self.m_rewardInfoList) do
    --     res2[num] = OpenTaskConfig[task]
    --     num = num + 1
    -- end
    -- table.sort(res2,function (a,b)
    --     return a.id<b.id       
    -- end )
    -- for _,task in ipairs(res2) do
    --     res[index] = task
    --     index = index +1
    -- end
    -- return res
    self:refreshItemData()
    return self.m_rewardList
end

function This:refreshItemData()
    local res = {}
    if self:getStartTime() > 0 then
        local curTime = GamePlayer:getServerTime() - self:getStartTime()
        local day = tonumber(GameUtil:formatDTime(Serialize["D"],curTime))+1
        local index = 1
        for _,task in ipairs(self.m_rewardList) do
            if task.open_num == day then
                --self.m_rewardList[index] = nil
                --table.remove(self.m_rewardList,index)
                res[index] = task
                index = index + 1
            end
        end
        --if #res > 0 then
        self.m_rewardList = res
        --end
    end
    table.sort(self.m_rewardList,function(a,b)
        if a:getState() ~= b:getState() then
            return a:getState() < b:getState()
        else
            return a:getId() < b:getId()
        end 
    end)
end


function This:getReward(id)
    local reward = {}
    for _,rewardInfo in pairs(self.m_rewardList) do
        if id == rewardInfo.id then
            reward = rewardInfo.reward
            break
        end
    end
    return reward
end

function This:onGetActivityInfoRequest()
	Socket:sendAndLock(12008,{activity_id = self:getId()})
end

function This:onGetActivityInfoReply(msg)
    if self:getId() ~= msg.activity_id then
        return
    end
    if msg.err_code == 0 then
        self:setTime(msg.start_time,msg.end_time)

        -- 今天的充值金额 
        self.m_payNum = 0
        local taskDict = {}
        for _, data in ipairs(msg.task_list) do
            taskDict[data.id] = data.num
        end
        for _, task in pairs( self:getTurnDatas() ) do
            local id = task:getId()
            if taskDict[id] then
                self.m_payNum = taskDict[id]
                break
            end
        end


        for _,taskId in pairs(msg.reward_list) do
            -- self.m_rewardInfoList[taskId] = taskId
            local item = self:getTaskById(taskId)
            if item then
                item:setState(SevenDayActivityTaskState.kReceive)
            end
        end
        self:notifyObservers("onRefreshDailyRecharge",4)
        EventDispatcher:dispatcher("activity_activitysevenday_state_changed")             
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onPickupActRewardRequest(taskId)
	Socket:sendAndLock(12004,{activity_id = self:getId(),id = taskId, type = 8})
end

function This:onReceiveActivityRewardReply(msg)
    if self:getId() ~= msg.activity_id then
		return
    end
    if msg.err_code == 0 then
        local reward = self:getReward(msg.id)
        -- self.m_rewardInfoList[msg.id] = msg.id
        self:getTaskById(msg.id):setState(SevenDayActivityTaskState.kReceive)
        g_scene:showReward(reward)
        self:notifyObservers("onRefreshDailyRecharge",4)
		EventDispatcher:dispatcher("activity_activitysevenday_state_changed")             
    else
		g_scene:handleErrorCode(msg.err_code)
    end
    
end

function This:onDayPassNotify(msg)
	self:onGetActivityInfoRequest()
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.diamond then
        self:onGetActivityInfoRequest()
    end
end