-- @Author: Su
-- @Date:   2019-06-04 15:44:43
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-04 15:44:43
-- @File Name: ActivitykDecreeTask.lua
-- @Description: 

local This , super = class(...,"NewActivity.ActivityBase")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")
local ActivityTypeConfig = require("Metadatas.ActivityTypeConfig")
local DecreeTaskConfig = require("Metadatas.DecreeTaskConfig")
local DecreeRewardConfig = require("Metadatas.DecreeRewardConfig")
local ActivityTimeConfig = require("Metadatas.ActivityTimeConfig")

function This:init(id)
    super:init(id)
    self.m_id = id
    self.m_descConfig = ActivityTypeConfig[self.m_id]
    self.m_config = ActivityTimeConfig[self.m_id]
    self:initListener()

    self.m_decreeJifen = 0
    self.m_officeJifen = 0
    self.m_finishTaskNum = 0
    self.m_vipNum = 0
    self.m_maxStage = 30
    self.m_taskIdList = {}
    self.m_hasReceiveIdList = {}
    self.m_progressList = {}
    self.m_hasFinishIdList = {}
    self.m_decreeReward ={}
    self.m_hasReceiveRewardIdList = {}
end

function This:initListener()
    EventDispatcher:addListener(self, 11901, self.onTaskChangeReply)
    EventDispatcher:addListener(self, 11912, self.onGetActivityInfoReply)
    EventDispatcher:addListener(self, 11913, self.onRefreshTaskReply)   --刷新
    EventDispatcher:addListener(self, 11902, self.onGetTaskRewardReply)     --领取任务奖励
    EventDispatcher:addListener(self, 11914, self.onGetDecreeRewardReply)   --领取政令奖励
    EventDispatcher:addListener(self, 11915, self.onReceiveTaskReply)       --领取任务
	EventDispatcher:addListener(self,10205,self.onApplyBillIDReply)

    GamePlayer:addObserver(self)
end

function This:removeListener()
    EventDispatcher:removeListener(self, 11901)
    EventDispatcher:removeListener(self, 11912)
	EventDispatcher:removeListener(self, 11913)
	EventDispatcher:removeListener(self, 11902)
	EventDispatcher:removeListener(self, 11914)
	EventDispatcher:removeListener(self, 11915)
    EventDispatcher:removeListener(self, 10205)


end

function This:getId()
    return self.m_id
end

function This:isOpen()
    local openTime =GamePlayer:getServerTime()-GamePlayer:getOpenServerTime()
    local tmp = math.modf(openTime/86400)
    local day =tmp
    return day >= self.m_config.start_time
end

function This:getDecreeNum()
    return self.m_decreeJifen
end

function This:getOfficeNum()
    return self.m_officeJifen
end

function This:getTurnDatas()
    table.sort(self.m_taskIdList,function(a,b)
        if self:isTaskOverTime(a) ~= self:isTaskOverTime(b) then
            return self:isTaskOverTime(a) ~= true
        end
        if (self:isFinish(a) and not self:isTaskRewardReceive(a)) ~= (self:isFinish(b) and not self:isTaskRewardReceive(b)) then
            return self:isFinish(a) and not self:isTaskRewardReceive(a)
        end
        if (not self:isFinish(a) and self:isReceive(a)) ~= (not self:isFinish(b) and self:isReceive(b)) then
            return not self:isFinish(a) and self:isReceive(a)
        end
        if self:getConfigById(a).quality ~= self:getConfigById(b).quality then
            return self:getConfigById(a).quality > self:getConfigById(b).quality
        end
    end)
    return self.m_taskIdList
end

function This:getMaxStage()
    return self.m_maxStage
end

function This:getVipNum()
    return self.m_vipNum
end

function This:isReceive(id)
    if self.m_hasReceiveIdList[id] then
        return true
    end
    return false
end

-- function This:isLevelOpen()
--     --return GamePlayer:getLevel() >= 20
--     super:isLevelOpen()
-- end

function This:getRefreshCost()
    local costList = ArgConfig["decree_task_refresh_cost"].arg
    local num = self.m_hasRefreshFrequency
    if num < costList[2][1]then
        return costList[1][2]
    elseif num >= costList[2][1] and num < costList[3][1] then
        return costList[2][2]
    elseif num >= costList[3][1] and num < costList[4][1] then
        return costList[3][2]
    else
        return costList[4][2]
    end
end

function This:isFinish(id)
    for _,task in pairs (self.m_hasFinishIdList) do
        if id == task then
            return true
        end
    end
    return false
end

function This:getTaskCanReceiveNum()
    local stage = self:getDecreeStage()
    if stage == 0 then
        stage = 1
    end
    local id = 0
    if self.m_vipNum == 1 then
        id = stage + 2000
    else
        id = stage + 1000
    end
    local maxNum = DecreeRewardConfig[id].task_num
    local hasReceiveNum = 0
    --  table.count(self.m_hasReceiveIdList)
    for _,taskId in pairs(self.m_hasReceiveIdList) do
        if not self:isTaskOverTime(taskId.id) and not self:isFinish(taskId.id) then
            hasReceiveNum = hasReceiveNum + 1
        end
    end
    return maxNum - hasReceiveNum
end

function This:getRewardByStage(stage)
    local rewardList = DecreeRewardConfig[stage+1000].reward
    local vipRewardList = DecreeRewardConfig[stage+2000].reward
    return rewardList,vipRewardList
end

function This:getDecreeStage() --当前阶段
    local stage = 0
    for _,config in pairs(DecreeRewardConfig) do
        if self.m_decreeJifen >= config.need_jifen and stage < config.stage then
            stage = config.stage
        end
    end
    return stage
end

function This:getLostNum()-- 下一阶段需要积分
    local stage = self:getDecreeStage()
    if stage >= table.count(DecreeRewardConfig)/2 then
        return  DecreeRewardConfig[1000+table.count(DecreeRewardConfig)/2].need_jifen 
    else
        return DecreeRewardConfig[stage+1+1000].need_jifen
    end

end

function This:getPercent()--进度条进度
    local stage = self:getDecreeStage()
    if stage < table.count(DecreeRewardConfig)/2 then
        stage = stage + 1
    end
    return (self.m_decreeJifen/DecreeRewardConfig[stage+1000].need_jifen)*100
end

function This:getTaskProgress(id)
    if self.m_progressList[id] then
        return self.m_progressList[id].num
    end
    return 0
end

function This:getTaskStartTime(id)
    if self.m_hasReceiveIdList[id] then
        return self.m_hasReceiveIdList[id].time
    end
    return 0
end

function This:getConfigById(id)
    if DecreeTaskConfig[id] then
        return DecreeTaskConfig[id]
    else
        g_scene:showTips(Serialize[62027])
    end
end

function This:isTaskOverTime(id)
    if self.m_progressList[id] then
        local time = self:getTaskStartTime(id) + self:getConfigById(id).time
        local time1 = GamePlayer:getServerTime()
        return self:getTaskStartTime(id) + self:getConfigById(id).time - GamePlayer:getServerTime() < 0
    else
        return false
    end
end

function This:getRefreshTimes()
    local times = self.m_refreshFrequency or 0
    return times,ArgConfig["decree_task_refresh_limit"].arg[1]
end

function This:getLastRefresTime()
    local times = self.m_refreshTime or 0
    return times,ArgConfig["decree_task_refresh_time"].arg[1]
end

function This:isDecreeRewardReceive(id )
    if self.m_decreeReward[id] then
        return true
    end
    return false
end

function This:isTaskRewardReceive(id)
    for _,taskId in pairs(self.m_hasReceiveRewardIdList) do
        if taskId == id then
            return true
        end
    end
    return false
end

function This:getHasFinishTaskNum()
    return self.m_finishTaskNum
end

function This:hasTaskRewardCanReceive()
    for _,id in pairs(self.m_taskIdList) do
        if self:isFinish(id) and not self:isTaskRewardReceive(id) then
            return true
        end
    end
    return false
end

function This:hasDecreeRewardCanReceive()
    for i = 1,self:getDecreeStage() do
        if not self:isDecreeRewardReceive(i+1000) then
            return true
        elseif self.m_vipNum == 1 and not self:isDecreeRewardReceive(i+2000) then
            return true
        end
    end
    return false
end

function This:hasRedPoint()
    return self:hasTaskRewardCanReceive() or self:hasDecreeRewardCanReceive()
end

function This:hasTaskNotFinish()
    for _,taskData in pairs(self.m_hasReceiveIdList) do
        if not self:isTaskOverTime(taskData.id) and not self:isFinish(taskData.id) then
            return true
        end
    end
    return false
end

function This:getUITypeName()
    return "Activity.ActivityDecreeMainPanel"
end
-----------------------------------------------------------------------------------------


function This:onGetActivityInfoRequest()
    Socket:sendAndLock(11912,{})
end

function This:onGetActivityInfoReply(msg)
    if msg.err_code == 0 then
        self.m_decreeJifen = msg.decreearg.decree_jifen
        self.m_officeJifen = msg.decreearg.office_jifen
        self.m_vipNum = msg.decreearg.vip
        self.m_refreshTime = msg.decreearg.refresh_time
        self.m_refreshFrequency = msg.decreearg.left_times
        self.m_hasRefreshFrequency = msg.decreearg.times
        self.m_taskIdList = msg.decreearg.cur_mission
        table.sort(self.m_taskIdList,function(a,b)
            if self:isTaskOverTime(a) ~= self:isTaskOverTime(b) then
                return self:isTaskOverTime(a) ~= true
            end
            if (self:isFinish(a) and not self:isTaskRewardReceive(a)) ~= (self:isFinish(b) and not self:isTaskRewardReceive(b)) then
                return self:isFinish(a) and not self:isTaskRewardReceive(a)
            end
            if self:getConfigById(a).quality ~= self:getConfigById(b).quality then
                return self:getConfigById(a).quality > self:getConfigById(b).quality
            end
        end)
        self.m_hasReceiveIdList={}
        for _,taskData in pairs(msg.decreearg.self_mission_list) do
            self.m_hasReceiveIdList[taskData.id] = taskData
        end
        self.m_hasFinishIdList = {}
        for _,taskData in pairs(msg.achievement.complete_list) do
            self.m_hasFinishIdList[taskData]=taskData
        end
        self.m_hasReceiveRewardIdList = {}
        self.m_hasReceiveRewardIdList = msg.achievement.reward_list
        self.m_progressList={}
        for _,taskData in pairs(msg.achievement.progress_list) do
            if taskData.type == 17 then
                self.m_progressList[taskData.id] = taskData
            end
        end
        self.m_decreeReward={}
        for _,taskData in pairs(msg.decreearg.rewardlist) do
            self.m_decreeReward[taskData] = taskData
        end
        self.m_finishTaskNum = msg.decreearg.finish_task_num
        self:notifyObservers("onDecreeRefresh")
		EventDispatcher:dispatcher("activity_DecreeRefresh")             

    else
		g_scene:handleErrorCode(msg.err_code)

    end
end

function This:onRefreshTaskRequest()
    Socket:sendAndLock(11913,{})
end

function This:onRefreshTaskReply(msg)
    if msg.err_code == 0 then
        self.m_decreeJifen = msg.decreearg.decree_jifen
        self.m_officeJifen = msg.decreearg.office_jifen
        self.m_vipNum = msg.decreearg.vip
        self.m_refreshTime = msg.decreearg.refresh_time
        self.m_refreshFrequency = msg.decreearg.left_times
        self.m_hasRefreshFrequency = msg.decreearg.times
        self.m_taskIdList = msg.decreearg.cur_mission
        table.sort(self.m_taskIdList,function(a,b)
            if self:isTaskOverTime(a) ~= self:isTaskOverTime(b) then
                return self:isTaskOverTime(a) ~= true
            end
            if (self:isFinish(a) and not self:isTaskRewardReceive(a)) ~= (self:isFinish(b) and not self:isTaskRewardReceive(b)) then
                return self:isFinish(a) and not self:isTaskRewardReceive(a)
            end
            if self:getConfigById(a).quality ~= self:getConfigById(b).quality then
                return self:getConfigById(a).quality > self:getConfigById(b).quality
            end
        end)
        self.m_hasReceiveIdList={}
        for _,taskData in pairs(msg.decreearg.self_mission_list) do
            self.m_hasReceiveIdList[taskData.id] = taskData
        end
        self.m_hasFinishIdList = {}
        for _,taskData in pairs(msg.achievement.complete_list) do
            self.m_hasFinishIdList[taskData]=taskData
        end
        self.m_hasReceiveRewardIdList = {}
        self.m_hasReceiveRewardIdList = msg.achievement.reward_list
        self.m_progressList={}
        for _,taskData in pairs(msg.achievement.progress_list) do
            if taskData.type == 17 then
                self.m_progressList[taskData.id] = taskData
            end
        end
        self.m_decreeReward={}
        for _,taskData in pairs(msg.decreearg.rewardlist) do
            self.m_decreeReward[taskData] = taskData
        end
        self.m_finishTaskNum = msg.decreearg.finish_task_num
        self:notifyObservers("onDecreeRefresh")
		EventDispatcher:dispatcher("activity_DecreeRefresh")   
    else
		g_scene:handleErrorCode(msg.err_code)

    end
end

function This:onTaskChangeReply(msg)
    if msg.progress_list[1].type ~= 17 then
        return
    end
    self.m_progressList={}
    for _,taskData in pairs(msg.progress_list) do
        self.m_progressList[taskData.id] = taskData
        if taskData.state == 1 then
            table.insert(self.m_hasFinishIdList,taskData.id)
        end
    end
    local isAction = true
    self:notifyObservers("onDecreeTaskStateRefresh",isAction)
    EventDispatcher:dispatcher("activity_DecreeRefresh")             

end

function This:onGetTaskRewardRequest(taskId)
    Socket:sendAndLock(11902,{type = 17,id = taskId})
end

function This:onGetTaskRewardReply(msg)
    if msg.type ~= 17 then
        return
    end
    if msg.err_code == 0 then
        local itemList = DecreeTaskConfig[msg.id].reward
        g_scene:showReward(itemList)
        table.insert(self.m_hasReceiveRewardIdList,msg.id) 
        self.m_finishTaskNum = self.m_finishTaskNum + 1
        self.m_decreeJifen = self.m_decreeJifen + DecreeTaskConfig[msg.id].jifen
        self:notifyObservers("onDecreeTaskStateRefresh",msg.id)
        EventDispatcher:dispatcher("activity_DecreeRefresh")

    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetDecreeRewardRequest(rewardId)
    Socket:sendAndLock(11914,{reward_id = rewardId})    
end

function This:onGetDecreeRewardReply(msg)
    if msg.err_code == 0 then
        self.m_decreeReward[msg.reward_id] = msg.reward_id
        local rewardList = DecreeRewardConfig[msg.reward_id].reward
        g_scene:showReward(rewardList)
        self:notifyObservers("onDecreeTaskStateRefresh")
        EventDispatcher:dispatcher("activity_DecreeRefresh")

    else
        -- g_scene:handleErrorCode(msg.err_code)
        g_scene:showTips(Serialize[62028])
    end
end

function This:onReceiveTaskRequest(taskId)
    Socket:sendAndLock(11915,{id = taskId})    
end

function This:onReceiveTaskReply(msg)
    if msg.err_code == 0 then
        self.m_hasReceiveIdList[msg.misson.id] = msg.misson
        self:notifyObservers("onDecreeTaskStateRefresh")
		EventDispatcher:dispatcher("activity_DecreeRefresh",msg.misson.id)             
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onApplyBillIDReply(msg)
    if msg.err_code == 0 then
        if msg.productId == ArgConfig["decree_recharge_money"].arg[1] then
            self.m_vipNum = 1
            self:notifyObservers("onDecreeRefresh")
		    EventDispatcher:dispatcher("activity_DecreeRefresh")             

        end
    else
		g_scene:handleErrorCode(msg.err_code)

    end

end