-- @Author: Su
-- @Date:   2019-07-31 16:42:28
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-16 16:01:23
-- @File Name: ActivitykRedHeroTable.lua
-- @Description: 

local This , super = class(...,"NewActivity.ActivitykAnimalTable")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local RedHeroActivityItemConfig = require("Metadatas.RedHeroActivityItemConfig")
local RedHeroCountRewardConfig =require("Metadatas.RedHeroCountRewardConfig")
local AnimalsActivityConfig =require("Metadatas.AnimalsActivityConfig")
local RedHeroShopConfig =require("Metadatas.RedHeroShopConfig")
local ArgConfig = require("Metadatas.ArgConfig")

local DrawInfoConfig = {
    [NewActivityType.kRedHeroTable] = {
        keyItemId = 102, 									--道具的Id
		useResource = true, 								--道具的类型是否游戏资源
		keyItemNumLimit = 1000, 								--每日获得道具的上限
		diffPrice = true,
		defaultDtype = ActPrizeDrawType.kOnce,			--默认抽奖的类型
		keyItemPrice = function( dType ) 					--每次抽奖消耗道具的个数
			return dType == ActPrizeDrawType.kOnce and ArgConfig.red_hero_onetime_cost.arg[1] or ArgConfig.red_hero_tentime_cost.arg[1]
		end, 
		freeTimes = function( dType )						--是否有免费次数
			return dType == ActPrizeDrawType.kOnce and 1 or 0
		end,
		hasFreeTimesPerDay = false, 							--免费次数是否每日重置
		drawConfig =  "Metadatas.RedHeroActivityItemConfig",		--使用的配置文件
		uiType = "NewActivity.RedHeroTurnTableNode",	--使用的界面
		warmingFile = "ActivityFestivalLanternreward",		--使用提示界面文件
		tipsWord = "activity_normalprizedraw_help_tips",	--使用提示文字
		csbFileName = "MenuScene/NodeUI/RedHeroTurnTable.csb",
    }
}


function This:init(id)
	super.init(self, id)
	self.m_id = id
	self.m_exchangeNum = 0
	self.m_rewardPool = 1
	self.m_todayKeyItemNum = 0
	self.m_drawTimeInHistory = 0
	self.m_buyTime = 0
	self.m_customConfig = DrawInfoConfig[self:getId()]
	self.m_useFreeTimes = 0
	self.m_rankRoleList={}
	self.m_itemList = {}
	self.m_rewardList ={}
	-- print(self:getHelpTips())


end

function This:initListener()
	EventDispatcher:addListener(self, 12062, self.onActivityPrizeDrawReply)--随机抽奖活动领奖
    EventDispatcher:addListener(self, 12061, self.onGetActivityInfoReply)--查看随机抽奖活动领奖
	EventDispatcher:addListener(self, 12063, self.onReceiveBoxReply)--领取箱子奖励
	

	EventDispatcher:addListener(self, "after_recharge", self.onAfterRechargeReply)
	EventDispatcher:addListener(self, "equip_changed", self.onGetBagDataChange)
	EventDispatcher:addListener(self, 12065, self.onExchangeRequestReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self, "equip_changed")
	EventDispatcher:removeListener(self, "after_recharge")
	EventDispatcher:removeListener(self, 12061)
	EventDispatcher:removeListener(self, 12062)
	EventDispatcher:removeListener(self, 12063)
	EventDispatcher:removeListener(self,12065)
end

function This:onExchangeRequest(id,num)
	self.m_exchangeNum = num
	Socket:sendAndLock(12065,{excharge_itemid= id,count = num})
end

function This:onExchangeRequestReply( msg )
    if msg.err_code == 0 then
        -- print(msg)
        g_scene:showReward({{id=msg.excharge_itemid,num=self.m_exchangeNum}})
        self:notifyObservers("onExchangeRequest")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:getShopList()
	return RedHeroShopConfig[self.m_rewardPool].item
end

function This:getShopMoney()
	return RedHeroShopConfig[self.m_rewardPool].money
end

-- 获取奖池道具信息
function This:getDrawItemList()
	local index = 1
	for id,item in ipairs(RedHeroActivityItemConfig) do
		if item.reward_pool == self.m_rewardPool then
			self.m_itemList[index] = item
			index = index + 1 
		end
	end
	return self.m_itemList
end

function This:getBuyTime()
	return self.m_buyTime
end

function This:getLeftTimes()
	return DepotManager:getItemCount(ArgConfig.red_hero_cost_item.arg[1])
end

function This:getLuckyBoxConfigByIndex(index)
	return RedHeroCountRewardConfig[index]
end

function This:getBoxList()
	local boxList = {}
	for id,config in pairs(RedHeroCountRewardConfig) do
		if config.reward_pool == self.m_rewardPool then
			boxList[id] = config
		end
	end
	return boxList
end

function This:isReceive(id)
	if self.m_rewardList then
		for _,boxId in pairs(self.m_rewardList) do
			if id == boxId then
				return true
			end
		end
	end
	return false
end

function This:isCanReceive(id)
	local config = RedHeroCountRewardConfig[id]
	if config then
		return self.m_buyTime >=config.lucky_count
	else
		return false
	end
end

function This:getIndexById(id,num)
	for index,item in ipairs(self.m_itemList)do
		if item.item_id == id and item.num == num then
			return index
		end
	end
end

function This:hasRewardCanReceive()
	for _ , config in pairs(RedHeroCountRewardConfig) do
		if self:isCanReceive(config.id) and not self:isReceive(config.id) and config.reward_pool == self.m_rewardPool then
			return true
		end
	end
	return false
end

function This:hasReceiveAllReward()
	return #self.m_rewardList == table.count(RedHeroCountRewardConfig)/2
end

function This:getRewardStringList()
	return self.m_rewardStringList
end

---------------------------------------------------------------------------

function This:onGetActivityInfoRequest()
	Socket:send(12061, {activity_id = self:getId()})	
end

function This:onGetActivityInfoReply( msg )
	if msg.err_code == 0 then
		self.m_rewardPool = msg.pool_id
		self.m_buyTime = msg.buy_times
		self.m_rewardList = msg.reward_list
		self.m_rewardStringList = msg.record_list
		self:notifyObservers("onRefreshPrizeDraw", self:getId())
		EventDispatcher:dispatcher("activity_activityRed_state_changed")
    else
		g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onActivityPrizeDrawRequest(drawType)
	if not self:isUpdating() then
		return
	end

	if not drawType then
		drawType = ActPrizeDrawType.kOnce
	end
    Socket:sendAndLock(12062, {draw_times= drawType})
end

function This:onActivityPrizeDrawReply(msg)
	if msg.err_code == 0 then
		self:onGetActivityInfoRequest()
		self:notifyObservers("onActivityPrizeDraw", {id = self:getId(),reward = msg.item_list,rewardCount =msg.count_list })
		EventDispatcher:dispatcher("activity_activityRed_state_changed")        

	else
    	g_scene:handleErrorCode(msg.err_code)
		-- self:notifyObservers("onActivityPrizeDraw", {id = self:getId()})		
	end

end

function This:onReceiveBoxRequest(id)
	Socket:sendAndLock(12063, {reward_id= id})
end

function This:onReceiveBoxReply(msg)
    if msg.err_code == 0 then
		table.insert(self.m_rewardList,msg.reward_id)
		g_scene:showReward(self:getLuckyBoxConfigByIndex(msg.reward_id).reward)
		self:notifyObservers("onRefreshPrizeDraw", self:getId())
		EventDispatcher:dispatcher("activity_activityRed_state_changed")        
	else
		g_scene:handleErrorCode(msg.err_code)
	end

end


-- // 红将活动预览
-- message msgc12061 {	
-- }

-- message msgs12061 {
-- 	optional int32 err_code = 1;	
-- 	optional int32 pool_id = 2;			// 奖池序列
-- 	optional int32 buy_times = 3; 		// 已抽奖次数
-- 	repeated string record_list = 4; 	// 全服抽奖记录
-- 	repeated int32 reward_list = 5;		// 已领取次数奖励
-- }

-- // 红将活动抽奖
-- message msgc12062 {
-- 	optional int32 draw_times = 1;	
-- }

-- message msgs12062 {
-- 	optional int32 err_code = 1;	
-- 	repeated int32 item_list = 2;
-- 	repeated int32 count_list = 3;
-- }


-- // 红将活动抽奖次数领奖
-- message msgc12063 {
-- 	optional int32 reward_id = 1;
-- }

-- message msgs12063 {
-- 	optional int32 err_code = 1;	
-- 	optional int32 reward_id = 2;
-- }
