-- @Author: Su
-- @Date:   2019-05-15 11:42:41
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-28 11:18:49
-- @File Name: AnimalsTurnTableNode.lua

-- local This , super = class(...,"NodeUI")
local This , super = class(...,"NewActivity.ActivitySpendGoldNode")
local NewActivityManager = require("NewActivity.NewActivityManager")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local ViewUtil = require("UI.ViewUtil")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local CostConfirmPanel = require("UI.CostConfirmPanel")
local AnimalsTurnTableInfo = require("NewActivity.AnimalsTurnTableInfo")
local HeroManager = require("Hero.HeroManager")
local AnimalsActivityConfig = require("Metadatas.AnimalsActivityConfig")
local WarfareUtil = require("Warfare.WarfareUtil")
local GodBeastManager = require("GodBeast.GodBeastManager")
local ArgConfig = require("Metadatas.ArgConfig")



local kMaxRewardNum = 12
local kRewardRoundNum = 1 * kMaxRewardNum
local kMaxRound = kMaxRewardNum * 200
local kDecelationNum = 1
local kBlinkNum = 6
local kMaxJifen = AnimalsActivityConfig[1].max_lucky_count

function This:initData(info)
    if info then
        self.m_tabIndex = info.tab
        self.m_name =info.name
    end
    -- self.m_config = self:getDataByTabIndex(self.m_tabIndex)
    self.m_escape = 0
    
    self.m_lott = {
        iCurIndex = 1,
        iMaxRound = kMaxRound - kDecelationNum ,
        iCurRound = 0, 
        iLuckyID  = 0, 
        iLuckyIndex = 0,
        bIsGot = false,
        bNeedClear = false,
        fDelay = 0.2,
        fDelayFactor = 0.2,
    }

    self.m_runningAniantion = false
    self.m_showAnimation = true 
    self.m_inited = false
end
function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/AnimalsTurnTable.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    local activityType = NewActivityType[self.m_name]
    self.m_activityType = activityType
    self.m_activity = NewActivityManager:getActivity(activityType)
    
    self.m_keyItemId = self.m_activity:getActKeyItemId()
    self.m_useResMode = self.m_activity:isUseResMode()
    self.m_itemList = self.m_activity:getDrawItemList()
    local node = self.m_csNode
    for i=1, kMaxRewardNum do
        node["pot" .. tostring(i)].selected:setVisible(false)
        local itemInfo = self.m_itemList[i]
        if itemInfo then
            local itemId, num, isRale = itemInfo.item_id, itemInfo.num, itemInfo.if_rare
            local itemNode = node["item" .. tostring(i)]
            local rareNode = node["dazhetiao"..tostring(i)]
            rareNode:setVisible(isRale==1)
            if itemNode then
                self:initRewardItemNode(itemNode, itemId, num)
                if os.isWin32 then
                    itemNode.num:setString(string.format("id:%d-%s", itemInfo.id, ItemUtil.formatItemCount(num)))
                end
            end               
        end
    end
    self:refreshItemNumNode()
    if self.m_useResMode then
        self:updateItemNumNode( GamePlayer:getDiamond() )
    end
    ViewUtil:blinkView(self.m_csNode.tips,true)
    self:initView()
    self:initTouchEvent()
end

function This:initView()
    local csNode = self.m_csNode
    csNode.number4:setString(self.m_activity:getActKeyItemPrice(ActPrizeDrawType.kOnce))
    csNode.number6:setString(self.m_activity:getActKeyItemPrice(ActPrizeDrawType.kTenTimes)*10)
    GodBeastManager:createEffectCharacter(csNode.animals,self.m_activity:getLuckBestReward())

    csNode.wordanimal:setImage(string.format("Icons/Animals/wordanimal%d.png" ,self.m_activity:getLuckBestReward()))
    csNode.animals:setScale(0.65)
    csNode.animals:setPosition(-200,90)
end

function This:refresh()
    if self.m_lott.bNeedClear then
        self.m_lott = {
            iCurIndex = 1,
            iMaxRound = kMaxRound - kDecelationNum ,
            iCurRound = 0,
            iLuckyID  = 0,
            iLuckyIndex = 0,
            bIsGot = false,
            bNeedClear = false,
            fDelay = 0.2,
            fDelayFactor = 0.2,
        }
    end

    local csNode = self.m_csNode
    for i=1, kMaxRewardNum do
        csNode["pot" .. tostring(i)].selected:setVisible(false)
    end
    self:refreshItemNumNode()
end

function This:refreshItemNumNode()
    local csNode = self.m_csNode
    local blessTimes = self.m_activity:getBlessTimes()
    self.m_luckNum = self.m_activity:getLuckNum()
    csNode.lucknum:setString(self.m_luckNum)
    csNode.jindutiao.jindu:setPercent(self.m_luckNum/kMaxJifen*100)

    csNode.num1:setString(self.m_activity:getDevoutNum())
    csNode.num2:setString(self.m_activity:getBlessNum())
    
    for i=1,3 do
        local roleInfo = self.m_activity:getRoleInfo(i)
        local num,percent = self.m_activity:getProportion(i,AnimalProportionType.kDevout)
        local blessNum,blessPercent = self.m_activity:getProportion(i,AnimalProportionType.kBless)
        local mNode = csNode[tostring(i)]
        if roleInfo then
            mNode.name:setString(roleInfo.name)
            GameUtil:setPlayerIconNodeImage(mNode.icon, roleInfo.icon)
            -- hero
        else
            mNode.name:setString(Serialize[62068])
        end
        mNode.num:setString(num)
        mNode.top1:setImage(string.format( "MenuScene/top%d.png",i))
        mNode.jindutiao1:setPercent(percent)
        mNode.jindutiao2:setPercent(blessPercent)
        mNode.icon:setVisible(roleInfo)
    end

    local scrollView = csNode.scrollView
    scrollView:removeAllItems()

    local stringList=self.m_activity:getRewardStringList()
    if stringList then
        for _,rewardString in pairs(stringList) do
            local view = AnimalsTurnTableInfo:create({text = rewardString})
            local w,h = view:getSize()
            scrollView:pushBackCustomItem(view,250,h)
        end
    end

    if blessTimes > 0 then
        csNode.button:clearShader()
        csNode.button10:clearShader()
        csNode.button20:clearShader()
    else
        csNode.button:setGrayShader()
        csNode.button10:setGrayShader()
        csNode.button20:setGrayShader()
    end
    self:refreshBox()
end

function This:refreshBox()
    local cnode = self.m_csNode
    for i =1,4 do
        local boxData = self.m_activity:getLuckyBoxConfigByIndex(i)
        -- local x = boxData.lucky_count/kMaxJifen*2500
        -- cnode["box"..tostring(i)]:setPositionX(x-30)
        -- cnode["boxbg"..tostring(i)]:setPositionX(x-30)
        GameUtil:createBoxEffect(self.m_activity:isBoxCanReceive(i),cnode["box"..tostring(i)],i)

        if not self.m_activity:isBoxCanReceive(i) then
            cnode["box"..tostring(i)]:clearShader()
        end
        if self.m_activity:getBoxHaxReceive(i) then
            cnode["box"..tostring(i)]:setGrayShader()
        else
            cnode["box"..tostring(i)]:clearShader()
        end

    end

end

function This:getIsAnimation()
    return self.m_runningAniantion
end

function This:initTouchEvent()
    local node = self.m_csNode
    node.weigouxuan:setTouchEnded(function()
        if not self.m_runningAniantion then
            local visible = node.gouxuan:isVisible()
            if self.m_showAnimation == not visible then 
                self.m_showAnimation = visible
                node.gouxuan:setVisible(not visible)
            end
        end
    end)

    node.button:setTouchEnded(function()
        local blessTimes = self.m_activity:getBlessTimes()
        if blessTimes <= 0 then
            return
        end
        self.m_activity:onBlessingRequest(2) 
    end)

    node.button10:setTouchEnded(function()
        local blessTimes = self.m_activity:getBlessTimes()
        if blessTimes <= 0 then
            return
        end
        self.m_activity:onBlessingRequest(1) 
    end)
    node.button20:setTouchEnded(function()
        local blessTimes = self.m_activity:getBlessTimes()
        if blessTimes <= 0 then
            return
        end
        self.m_activity:onBlessingRequest(3) 
    end)

    node.button1:setTouchEnded(function()
        local level = GamePlayer:getLevel()
        local limitLevel = ArgConfig["mythical_campaign_refresh_open_level"].arg[1]
        if level >= limitLevel then
            self:startPrizeDraw()      
        else
            g_scene:showTips(Serialize[62069])
        end
    end)

    node.button2:setTouchEnded(function()
        local level = GamePlayer:getLevel()
        local limitLevel = ArgConfig["mythical_campaign_refresh_open_level"].arg[1]
        if level >= limitLevel then
            self:startPrizeDraw(true)      
        else
            g_scene:showTips(Serialize[62069])
        end  
    end)

    node.guize:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize["AnimalsActivity"],node = node.guize})
    end)

    for i = 1,4 do
        node["box"..tostring(i)]:setTouch({
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
                local boxConfig = self.m_activity:getLuckyBoxConfigByIndex(i)
                local hasReward = self.m_activity:getBoxHaxReceive(i)
                local state = 0
                if hasReward then
                    state = RewardState.kHasReward
                elseif self.m_luckNum < boxConfig.lucky_count then
                    state = RewardState.kNotFinish
                else
                    state = RewardState.kCanReward
                end
                local panel = g_scene:openDialog("UI.RewardShowPanel",{
                    title = Serialize[62070],
                    state = state,
                    csFileName = "MenuScene/NodeUI/AnimalsTurnTableBox.csb",
                    text = string.format(Serialize[62071],boxConfig.lucky_count)
                })

        		local rewardList = {}
                for i , info in ipairs(boxConfig.reward) do
                    table.insert(rewardList,{id = info[1],num = info[2]})
                end
                panel:setRewardList(rewardList)
                panel:setHandler(function()
                    self.m_activity:onGetAnimalRewardRequest(i)
                    panel:close()
                end)

                -- if self.m_luckNum < boxConfig.lucky_count or self.m_activity:getBoxHaxReceive(i) then
                --     return
                -- end
            end
        })
    end

    for i = 1 , 3 do
        local roleInfo = self.m_activity:getRoleInfo(i)
        local mNode = self.m_csNode[tostring(i)]
        mNode.icon:setTouch({
            isScale = false, 
            isDynamic = true, 
            onEnded = function()
                g_scene:openDialog("Player.RoleInfoPanel",{id = roleInfo.id})
            end
        })
    end

end

function This:refreshTime()
    local node = self.m_csNode
    local leftTime = self.m_activity:getEndTime() - GamePlayer:getServerTime()
    node.time:setString(GameUtil:formatDTime(Serialize[62072],leftTime))
end

function This:onEnter()
    self.m_activity:addObserver(self)
    self.m_activity:onGetActivityInfoRequest()
end

function This:onExit()
    self.m_activity:removeObserver(self)
end

function This:calculateCostByDType( multiTime )
    local dType = ActPrizeDrawType.kOnce 
    if multiTime then
        dType = ActPrizeDrawType.kTenTimes
    end
    return self:calculateCost(dType, ActPrizeDrawNumType[dType])
end

function This:startPrizeDraw( multiTime )
    if self.m_runningAniantion or not self.m_activity or not self.m_activity:isUpdating() then
       return
    end

    local dType = ActPrizeDrawType.kOnce
    if multiTime then
        dType = ActPrizeDrawType.kTenTimes
    end
    local dNumType = ActPrizeDrawNumType[dType]
    if not self.m_activity:checkHasLeftTimes(dNumType) then
        g_scene:showTips(Serialize[11510])
        return
    end

    local reuestDraw = function ()
        if self.m_showAnimation then
            self.m_runningAniantion = true
            if self.m_dialog then
                self.m_dialog:setIsRunning(true)
            end
            self:runSelectionItem()
        end
        self.m_activity:onActivityPrizeDrawRequest(dNumType)
    end
    local num = dNumType - self.m_activity:getLeftTimes()
    if num <= 0 then
        num = 0
    end
    local costNum = self:calculateCost(dType, num)

    if costNum > 0 then
        local notips = "Activity" .. tostring(self.m_activity:getId()) .. "shuaxin" .. tostring(dNumType)
        local check = function ()
            local result, errorCode = DepotManager:isItemOrResEnough(self.m_keyItemId, costNum)
            if not result then
                if errorCode then
                    g_scene:handleErrorCode(errorCode)
                else
                    g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(self.m_keyItemId)))
                end
            end
            return result
        end
        
        if not CostConfirmPanel:existNoTipsKey(notips) then
            local openDialog = function ()
                local myPanel=g_scene:openDialog("UI.CostConfirmPanel", {
                        fileName = "MenuScene/NodeUI/ActivityHappyRewardVoucher.csb",
                        notips = notips,
                        cost = costNum,
                        costId = self.m_keyItemId,
                        descText = string.format(Serialize[62073],num),
                        okHandler = function()
                            coroutine.resume(self.m_coroutine)
                        end }
                    )
                    myPanel:setBtnString(descText,Serialize[62074])
                    return coroutine.yield()
            end

            self.m_coroutine = coroutine.create(function ()
                openDialog()
                if check() then
                    reuestDraw()
                end
            end)
            coroutine.resume(self.m_coroutine)
        else
            if check() then
                reuestDraw()
            end
        end
    else
        reuestDraw()
    end 
end

function This:runSelectionItem(delay)
    local csNode = self.m_csNode
    local continue = false
    local loopCount = 1

    function runTask( )
        loopCount = loopCount + 1
        if not delay then
            delay = self.m_lott.fDelay
        end

        local count = kMaxRewardNum
        local minIndex = 1
        local maxIndex = kMaxRewardNum

        self.m_lott.iCurRound = self.m_lott.iCurRound + 1
        if self.m_lott.iCurRound < kDecelationNum or self.m_lott.iCurRound > self.m_lott.iMaxRound then
            continue = false
        else
            continue = true
        end

        csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
        self.m_lott.iCurIndex = self.m_lott.iCurIndex >= maxIndex and minIndex or self.m_lott.iCurIndex + 1
        csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(true);


        if self.m_lott.iCurRound > self.m_lott.iMaxRound then
            if self.m_lott.fDelay < self.m_lott.fDelayFactor then
                -- music run
                self.m_lott.fDelay = self.m_lott.fDelay + 0.32 * self.m_lott.fDelayFactor
                self.m_lott.fDelay = self.m_lott.fDelay > self.m_lott.fDelayFactor and self.m_lott.fDelayFactor or self.m_lott.fDelay
                self:runSelectionItem(self.m_lott.fDelay)
            else 
                -- music stop
                if self.m_lott.iCurRound == kMaxRound then
                    -- 移動 kMaxRound 次還沒收到網絡回調,認定為断网的节奏
                    csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    self.m_lott.iCurIndex = minIndex
                    g_scene:showTips(Serialize["act_tip_network_bad"])
                    self.m_runningAniantion = false
                    self.m_lott.bNeedClear = true
                    if self.m_dialog then
                        self.m_dialog:setIsRunning(false)
                    end 
                else
                    csNode["pot" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    csNode["pot" .. self.m_lott.iCurIndex].selected:runAction(cc.Sequence(cc.Blink(self.m_lott.fDelay * 2, kBlinkNum), cc.CallFunc(function ()
                        self:onLotteryResultShow()
                    end)))
                end
                self.m_lott.iCurRound = 0
                self.m_lott.iMaxRound = kMaxRound - kDecelationNum
            end
        elseif self.m_lott.iCurRound < kDecelationNum then
            -- music run
            self.m_lott.fDelay = self.m_lott.fDelay - 0.32
            self:runSelectionItem(self.m_lott.fDelay)

        else
            -- music run
        end

        if continue then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
        end
    end

    if delay then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
    else
        runTask()
    end
end

function This:onLotteryResultShow(noAnimations)
    if self.m_curRewardList then
        if self.m_showAnimation then
            self:runSelectionItem()
            self:setTimer(0.1, function()
                local reward,count = self.m_activity:getShowReward()
                self:onActivityPrizeDraw({id=self.m_activity:getId(),reward=reward,rewardCount=count}) end)
        end
    else
        if not noAnimations then
            g_scene:setIsEnableShowTips(true)
            if self.m_showPrizeRewardList then
                g_scene:openDialog("NewActivity.ActivityPrizeDrawRewardPanel", {
                    mode = self.m_useResMode,
                    keyItemId = self.m_keyItemId, 
                    fileName = self.m_activity:getWarmingUIFileName(),
                    itemList = self.m_showPrizeRewardList, 
                    activity = self.m_activity,
                    dialog = self
                })
            end
        end

        self.m_csNode["pot" .. self.m_lott.iLuckyIndex].selected:setVisible(true)
        self.m_lott.bNeedClear = true
        self.m_runningAniantion = false
        if self.m_dialog then
            self.m_dialog:setIsRunning(false)
        end        
        self:refresh()
    end
end

function This:getCurIndexNum(id,num)
    -- return ((num - 1) % kMaxRewardNum) + 1
    return self.m_activity:getIndexById(id,num)
end

function This:onActivityPrizeDraw( msg )
    if msg.id == self.m_activity:getId() then
        local info = msg.reward
        local infoCount = msg.rewardCount
        if infoCount and not self.m_infoCount then
            self.m_infoCount = infoCount
        end
        if info and not self.m_reward then
            self.m_reward = info
        end

        self.m_curRewardList = info or self.m_curRewardList
        self.m_curRewardIndex = (self.m_curRewardIndex or 0) + 1
        if self.m_curRewardList and self.m_curRewardIndex <= #self.m_curRewardList then
            local index = self:getCurIndexNum(self.m_curRewardList[self.m_curRewardIndex] or 1,infoCount[self.m_curRewardIndex])
            self.m_lott.iLuckyIndex = index 
            self.m_lott.iLuckyID = self.m_itemList[index].item_id

            self.m_lott.bIsGot = ItemUtil.getType(self.m_lott.iLuckyID) == ItemCommonType.kHero and HeroManager:isGotHero(self.m_lott.iLuckyID)

            if self.m_showAnimation then
                if self.m_curRewardIndex == 1 then
                    self.m_lott.iMaxRound = self.m_lott.iLuckyIndex - self.m_lott.iCurIndex
                    self.m_lott.iMaxRound = self.m_lott.iMaxRound > 0 and self.m_lott.iMaxRound or self.m_lott.iMaxRound + kRewardRoundNum
                    self.m_lott.iMaxRound = self.m_lott.iCurRound + self.m_lott.iMaxRound + kRewardRoundNum * 1 - kDecelationNum
                else
                    self.m_lott.iMaxRound = self.m_lott.iLuckyIndex - self.m_lott.iCurIndex
                    self.m_lott.iMaxRound = self.m_lott.iMaxRound > 0 and self.m_lott.iMaxRound or self.m_lott.iMaxRound + kMaxRewardNum
                    self.m_lott.iMaxRound = self.m_lott.iCurRound + self.m_lott.iMaxRound - kDecelationNum            
                end
            else
                self.m_showPrizeRewardList = {}
                self.m_showPrizeRewardTipsList = {}
                local index = 1
                for id, item in ipairs(self.m_curRewardList) do
                    if not self.m_showPrizeRewardTipsList[item] then
                        self.m_showPrizeRewardList[index] = {item_id = item,num =infoCount[id]}
                        self.m_showPrizeRewardTipsList[item] = index
                        index = index + 1
                    else
                        self.m_showPrizeRewardList[self.m_showPrizeRewardTipsList[item]] = {item_id = item,num = self.m_showPrizeRewardList[self.m_showPrizeRewardTipsList[item]].num + infoCount[id]}
                    end
                end
                self.m_curRewardList = nil
                self.m_curRewardIndex = 0
                self:onLotteryResultShow()
            end

            if self.m_curRewardList and self.m_curRewardIndex == #self.m_curRewardList then
                self.m_showPrizeRewardList = {}
                self.m_showPrizeRewardTipsList = {}
                --self.m_showPrizeRewardList = self.m_curRewardList
                local infoCount = msg.rewardCount
                local index = 1
                for id, item in ipairs(self.m_curRewardList) do
                    if not self.m_showPrizeRewardTipsList[item] then
                        self.m_showPrizeRewardList[index] = {item_id = item,num =infoCount[id]}
                        self.m_showPrizeRewardTipsList[item] = index
                        index = index + 1
                    else
                        self.m_showPrizeRewardList[self.m_showPrizeRewardTipsList[item]] = {item_id = item,num = self.m_showPrizeRewardList[self.m_showPrizeRewardTipsList[item]].num + infoCount[id]}
                    end
                end
                self.m_curRewardList = nil
                self.m_curRewardIndex = 0
            end
        elseif not info then
            self.m_curRewardList = nil
            self.m_curRewardIndex = 0
            if msg and msg.err_code ~= 0 then
                self.m_lott.iCurRound = kMaxRound - kMaxRewardNum
            end
        end

    end

end

function This:onRefreshPrizeDraw(id)
    if id == self.m_activity:getId() then
        self:refreshItemNumNode()
    end
end

function This:onUpdateActivity(activity)
    local type = activity:getId()
    if type == NewActivityType.kAnimalTable then
        self:refreshTime()
    end
end