-- @Author: superm
-- @Date:   2018-03-21 16:13:33
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-27 16:30:39
-- @File Name: ContinuePayPanel.lua
-- @Description:

local This , super = class(...,"NodeUI")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

This.csfileName = "MenuScene/NodeUI/ActivityContinuousPay.csb"

function This:initData(queue, dialog)
	self.m_activity = queue
	self.m_dialog = dialog
    self:initCsNode()
end

function This:initCsNode()
	local filename = self.m_activity:getCSFileName()
    self.m_csNode = cc.Node:createFromCSFile(filename or self.csfileName)
    self:addChild(self.m_csNode)
end

function This:initNode()
    local csNode = self.m_csNode

	local total = self.m_activity:getMaxPayTime()
	self.m_boxNodes = {}
	self.m_effect = {}
	self.m_itemNodes = {}
	local w , h =csNode.kedu.jindu2:getContentSize()
	local index = 1
	for i = 1, total do
		if self.m_activity:hasBoxReward(i) then
			local node = self:getBoxNode(i)
		    local width, h = node.bg:getContentSize()
			node:setPosition(i / total * w, - h / 2)
			node:getContentSize(width, h)
			node:setAnchorPoint(0.5, 0.1)
			csNode.box:addChild(node)
			self.m_boxNodes[i] = node

			index = math.min(4, index)
			node.box1:setImage(string.format("MenuScene/boxLock%d.png", index))
			node.box2:setImage(string.format("MenuScene/boxGet%d.png",index))
			index = index + 1
		end
	end

	self:refresh()
end

function This:onEnter()
	super.onEnter(self)
	self.m_activity:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_activity:removeObserver(self)
end

function This:refresh()
    local csNode = self.m_csNode
	local total = self.m_activity:getMaxPayTime()
	local totalPayTime = self.m_activity:getTotalPayTime()
	local curRewardDay = self.m_activity:getCurRewardDay()
	local activityDay = self.m_activity:getActivityDay()

	csNode.kedu.day:setString(string.format(Serialize[62076],totalPayTime))
	csNode.kedu.jindu2:setPercent(totalPayTime / total * 100)

	for i , node in pairs(self.m_boxNodes) do
		self:refreshBoxNode(node,i)
	end
	self:refreshItemNodes()

	self.m_isShowBtn = curRewardDay <= totalPayTime or (curRewardDay > totalPayTime and not self.m_activity:isPay(activityDay))
	local rewardNode = csNode.reward
	rewardNode.button:setVisible(self.m_isShowBtn)
	rewardNode.pickuptext:setVisible(not self.m_isShowBtn)
	if curRewardDay <= totalPayTime then
		rewardNode.button.text1:setString(Serialize[60836])
	else
		rewardNode.button.text1:setString(Serialize[62077])
	end

	local showDay
	if curRewardDay > totalPayTime and self.m_activity:isPay(activityDay)  then
		showDay = activityDay
	else
		showDay = curRewardDay
	end

	if not os.isWin32 then
		rewardNode.tips:setString(string.format(Serialize[62078],showDay))
	else
		local id = self.m_activity.m_configList[showDay].id
		rewardNode.tips:setString(string.format(Serialize[62079], id, showDay))
	end
end

function This:initTouchEvent()
    local csNode = self.m_csNode
	csNode.reward.button:setTouchEnded(function()
		local totalPayTime = self.m_activity:getTotalPayTime()
		local curRewardDay = self.m_activity:getCurRewardDay()
		if curRewardDay <= totalPayTime then
			self.m_activity:onGetRewardActivityRequest()
		else
			g_scene:openDialog("System.RechargeMainUI")
		end
	end)
	for i , boxNode in pairs(self.m_boxNodes) do
		boxNode:setTouch({
			onEnded = function()
				local state = RewardState.kNotFinish
				local isReward = self.m_activity:hasRewardBox(i)
				if isReward then state = RewardState.kHasReward end

				local desc 
				if not os.isWin32 then
					desc = string.format(Serialize[62080],i)
				else
					local id = self.m_activity.m_configList[i].id
					desc = string.format(Serialize[62081], id, i)
				end
				local panel = g_scene:openDialog("UI.RewardShowPanel",{
					title = Serialize[62082],
					state = state,
					complex = Serialize[62083],
					text = desc
				})
				local rewardList = self.m_activity:getBoxRewardList(i)
				panel:setRewardList(rewardList)
			end,
			isScale = false,
			sizeNode = boxNode.bg
		})
	end

	for i = 1 ,  4 do
		local view = self.m_itemNodes[i] 
		if view then
            self:handleHelpTips(view, function ()
                return {fileName = "UI.ItemTips", itemId = view.m_id, itemType = view.m_itemType, index = i}
            end)
		end
	end
end

function This:getBoxNode(index)
	local node = cc.Node:createFromCSFile("MenuScene/NodeUI/ActivityPayBox.csb")
	return node
end

function This:refreshBoxNode(node,index)
	node.day:setString(string.format(Serialize["activity_ActOpen_1_c"],index))
	local continueDay = self.m_activity:getTotalPayTime()
	local i, locked, finished = index, index > continueDay, self.m_activity:hasRewardBox(index)
	node.box1:setVisible(locked)
	node.box2:setVisible(not locked)
	if finished then
		node.box2:setGrayShader()
	else
		node.box2:clearShader()
	end

	if (locked or finished) and self.m_effect[i] then
		self.m_effect[i][1]:removeFromParent()
		self.m_effect[i][2]:removeFromParent()
		self.m_effect[i] = nil
		node:stopAllActions()
	elseif not locked and not finished and not self.m_effect[i] then
		local x , y = node.box2:getPosition()
		self.m_effect[i] = {}
		self.m_effect[i][1] = GameUtil:createEffect("BoxLing",true)
		self.m_effect[i][1]:setPosition(x,y)
		node:addChild(self.m_effect[i][1],-1)
		self.m_effect[i][2] = GameUtil:createEffect("BoxLing2",true)
		self.m_effect[i][2]:setPosition(x,y)
		node:addChild(self.m_effect[i][2],-1)
        node:runActionByID("Roll")  
	end
end

function This:refreshItemNodes()
    local csNode = self.m_csNode
	local curRewardDay = self.m_activity:getCurRewardDay()
	local totalPayTime = self.m_activity:getTotalPayTime()
	local activityDay = self.m_activity:getActivityDay()
	local rewardList = {}
	if curRewardDay > totalPayTime and self.m_activity:isPay(activityDay)  then
		rewardList = self.m_activity:getRewardList(curRewardDay - 1)
	else
		rewardList = self.m_activity:getRewardList(curRewardDay)
	end
	 
	for i , info in ipairs(rewardList) do
		if not self.m_itemNodes[i] then
			self.m_itemNodes[i] = ItemNode:create(info.id,info.num, nil, false)
			self.m_itemNodes[i]:setAnchorPoint(0.5,0.5)
			csNode["item"..i]:addChild(self.m_itemNodes[i])
		else
			self.m_itemNodes[i]:initItem(info.id,info.num)
		end
	end

	local itemCount = #rewardList
	for i = 1 ,  4 do
		csNode["item"..i]:setVisible(itemCount >= i)
	end
end

function This:onGetActivityInfo()
	self:refresh()
end

function This:onGetRewardActivity()
	self:refresh()
end

function This:onUpdateActivity()
    local csNode = self.m_csNode
	local leftTime = self.m_activity:getLeftTime()
	csNode.time:setString(string.format(Serialize[62084],GameUtil:formatDTime(Serialize["activity_BaseTabs_1_c"],leftTime)))
end

function This:handleHelpTips(node, getInfoHandler)
    local tips = nil
    node:setTouch({
        onBegan = function(_,touch)
            local rect = node:getBoundingBox()
            if rect:isContainsPoint(touch) then
                local info = getInfoHandler()
                if info then
                	if info.itemId and info.itemId == 215 then
                		tips = nil
                		return true
                	end

                	if not info.node then
                		info.node = node
                	end
                    tips = g_scene:openDialog(info.fileName or "UI.HelpTips", info)
                    return true
                end
            end
            return false
        end,
        onEnded = function()
            if tips and tips[0] ~= 0 then
                tips:close()
                tips = nil
            else
                local info = getInfoHandler()
                if info and info.itemId and info.itemId == 215 then
				    local reward = self.m_activity:getRewardPackage()
				    g_scene:openDialog("Activity.ActivityPreviewPanel", {reward=reward, rtype = 1, filename="ActivityRewardShow"})
                end
            end
        end,
        isScale = false,
        isDynamic = true,
		isSwallow = false,
        onCanceled = function()
            if tips and tips[0] ~= 0 then
                tips:close()
                tips = nil
            end
        end,
    })
end

--endregion
