-- @Author: superm
-- @Date:   2018-04-21 15:11:00
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-13 14:31:11
-- @File Name: EntireConsumePanel.lua
-- @Description: 

local This , super = class(...,"NodeUI")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

function This:initData(queue, dialog)
    self.m_activity = queue
    self.m_dialog = dialog
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ActivityLabourPays.csb")
    self:addChild(self.m_csNode)
end

function This:initNode()
    local csNode = self.m_csNode
	local total = 7
	local posx, maxNum = csNode.kedu.jindu2:getPositionX(), self.m_activity:getMaxBoxTargetNum()

	self.m_boxNodes = {}
	self.m_effect = {}
	self.m_itemNodes = {}
	local w , h = csNode.kedu.jindu2:getContentSize()
	local index = 1
	for i = 1, total do
		local node = self:getBoxNode(i)
		local width, h = node.bg:getContentSize()
		local num = self.m_activity:getBoxTargetNum(i)
		local nodePosx =  w * num / maxNum
		node:setPosition(nodePosx, - h / 2)
		node:getContentSize(width, h)
		node:setAnchorPoint(0.5, 0.1)
		csNode.box:addChild(node)
		self.m_boxNodes[i] = node

		index = math.min(4, index)
		node.box1:setImage(string.format("MenuScene/boxLock%d.png", index))
		node.box2:setImage(string.format("MenuScene/boxGet%d.png",index))
		index = index + 1

		csNode.kedu["day" .. i]:setString(GameUtil:formatCount(num))
		csNode.kedu["day" .. i]:setPositionX(posx + nodePosx)
	end

	local startTime = self.m_activity:getStartTime()
	local startStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],startTime)
	local endTime = self.m_activity:getEndTime()
	local endStr = GameUtil:formatDate(Serialize["activity_FamilyUI_1_c"],endTime)
	csNode.time:setString(string.format("%s ~ %s",startStr,endStr))
	self:refresh()
end

function This:onEnter()
	super.onEnter(self)
	self.m_activity:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_activity:removeObserver(self)
end

function This:refresh()
    local csNode = self.m_csNode.kedu
	csNode.number:setString(tostring(self.m_activity:getTotalConsumption()))
	csNode.number1:setString(tostring(self.m_activity:getSelfConsumption()))
	csNode.jindu2:setPercent(self.m_activity:getConsumptionProgress() * 100)

	for i , node in pairs(self.m_boxNodes) do
		self:refreshBoxNode(node, i)
	end

	self:refreshItemNodes()
end

function This:initTouchEvent()
    local csNode = self.m_csNode
	csNode.reward.button:setTouchEnded(function()
		if self.m_activity:hasRewardBox(self.m_selectIndex) then
			self.m_activity:onGetRewardActivityRequest(self.m_selectIndex)
		else
			g_scene:showTips(string.format(Serialize[62093], self.m_activity:getBoxTargetNum(self.m_selectIndex)))
		end
	end)

	if csNode.help then
	    csNode.help:setTouchEnded(function()
	        g_scene:openDialog("UI.HelpTips", {text = Serialize["activity_entireconsume_help_tips"], node = self.help})
	    end)
	end

	for i , boxNode in pairs(self.m_boxNodes) do
		boxNode:setTouch({
			onEnded = function()
				self:refreshItemNodes(i)
			end,
			isScale = false,
			sizeNode = boxNode.bg
		})
	end
end

function This:getBoxNode(index)
	local node = cc.Node:createFromCSFile("MenuScene/NodeUI/ActivityPayBox.csb")
	return node
end

function This:refreshBoxNode(node,index)
	-- node.day:setVisible(true)
	-- node.day:setString(self.m_activity:getBoxTargetNum(index))
	local i, hasReward, finished = index, self.m_activity:hasRewardBox(index), self.m_activity:isBoxRewarded(index)
    print("" .. i, tostring(hasReward))
	node.box1:setVisible(not hasReward or finished)
	node.box2:setVisible(hasReward or finished)
	if not hasReward then
		node.box2:setGrayShader()
	else
		node.box2:clearShader()
	end

	if not hasReward and self.m_effect[i] then
		self.m_effect[i][1]:removeFromParent()
		self.m_effect[i][2]:removeFromParent()
		self.m_effect[i] = nil
		node:stopAllActions()
	elseif hasReward and not self.m_effect[i] then
		local x , y = node.box2:getPosition()
		self.m_effect[i] = {}
		self.m_effect[i][1] = GameUtil:createEffect("BoxLing",true)
		self.m_effect[i][1]:setPosition(x,y)
		node:addChild(self.m_effect[i][1],-1)
		self.m_effect[i][2] = GameUtil:createEffect("BoxLing2",true)
		self.m_effect[i][2]:setPosition(x,y)
		node:addChild(self.m_effect[i][2],-1)
        node:runActionByID("Roll")  
	end
end

function This:refreshItemNodes(index)
    local rewardNode = self.m_csNode.reward
	local rewardList, money
	if not index then
		local activity = self.m_activity:getDefaultItem()
        if activity then
		    self.m_selectIndex = activity.id
		    rewardList = activity.reward
		    money = activity.money
        end
	else
		self.m_selectIndex = index
		rewardList = self.m_activity:getBoxRewardList(index)
		money = self.m_activity:getBoxTargetNum(index)
	end
	
    if rewardList and money then
	    local w, count = 10, 0
	    for i , info in ipairs(rewardList) do
		    local node = self.m_itemNodes[i]
		    if not node then
			    node = ItemNode:create(info[1], info[2], nil, true)
			    node:setAnchorPoint(0.5, 0.5)
			    self.m_itemNodes[i] = node
                rewardNode.itemtipslist:addChild(node)
		    else
			    node:initItem(info[1], info[2])
		    end
		    node:setPositionX(w)
		    node:setScale(0.8)
            node:setVisible(true)
            count = count + 1
		    w = w + node:getContentWidth() * 0.8 + 10
	    end
	    rewardNode.itemtipslist:setPositionX(40 - w / 2)

	    local itemCount = #self.m_itemNodes
	    for i = count + 1 ,  itemCount do
		    self.m_itemNodes[i]:setVisible(false)
	    end

	    self.m_isShowBtn = self.m_activity:isBoxRewarded(self.m_selectIndex)
	    rewardNode.button:setVisible(not self.m_isShowBtn)
	    rewardNode.pickuptext:setVisible(self.m_isShowBtn)
	    if self.m_activity:hasRewardBox(self.m_selectIndex) then
		    rewardNode.button.text1:setString(Serialize[60205])
	    else
		    rewardNode.button.text1:setString(Serialize[62094])
	    end

	    rewardNode.tips:setString(string.format(Serialize[62095], GameUtil:formatCount(money)))
    end
end

function This:onGetActivityInfo()
	self:refresh()
end

function This:onGetRewardActivity()
	self:refresh()
end

-- function This:onUpdateActivity()
-- 	local leftTime = self.m_activity:getLeftTime()
-- 	self.time:setString(string.format(Serialize[62084],GameUtil:formatDTime(Serialize["activity_BaseTabs_1_c"],leftTime)))
-- end

--endregion
