-- @Author: superm
-- @Date:   2018-02-06 21:23:44
-- @Last Modified by:   superm
-- @Last Modified time: 2018-02-07 10:07:35
-- @File Name: NewActivityModel.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")

function This:create(info)
	local ret = {}
	setmetatable(ret,self)
	ret:init(info)
	return ret
end

function This:init( info )
	super.init(self)
	self.m_escape = 0
    self.m_id = info.activity_id or 0
    self.m_upStartTime = info.start_time or 0
    self.m_upEndTime = info.end_time or 0	
end

-- 更新活动信息
function This:updateInfo(info)
	if self.m_isUpdating then
		self:endUpdate()
	end

	self:onActivityUpdateInfo(info)
	self.m_state = info.state
	self.m_endTime = info.end_time
	self.m_escape = 0

	local curTime = GamePlayer:getServerTime()
	if self.m_state ~= ActObjectState.kLocked then
		if curTime < self.m_endTime then
			self.m_state = ActObjectState.kNormal
			self:startUpdate()
		elseif curTime >= self.m_endTime then
			self.m_state = ActObjectState.kInvalid
		end
	end
end

function This:startUpdate()
    if not self.m_upIsUpdating then
        self.m_upIsUpdating = true
        cc.Application:addToUpdateList(self)
        self:onActivityStart()
    end
end

function This:endUpdate()
    if self.m_upIsUpdating then
        self.m_upIsUpdating = false
        cc.Application:removeFromUpdateList(self)
        self:onActivityEnd()
    end
end

-- 具体实现更新的代码
function This:onActivityUpdateInfo( info )
end

function This:onActivityStart()
	self:notifyObservers("onActivityStart")
end

function This:onActivityEnd()
	self:notifyObservers("onActivityEnd")
end

function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1.0
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			self:notifyObservers("updateActivityTime")
		else
			self:endUpdate()
		end
	end
end

function This:setView(view)
    self.m_view = view
end

function This:getView()
    return self.m_view
end

-- 领取接口
function This:requestPickUp( ... )
end

-- 领取接口的返回
function This:onRequestPickUpReply( message )
end

-- 请求信息接口
function This:requestActivityInfo( ... )
end

-- 请求信息接口的返回
function This:onActivityInfoReply( message )
end