-- @Author: Su
-- @Date:   2019-05-15 11:12:05
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-04-28 11:17:29
-- @File Name: RedHeroTurnTableNode.lua
-- @Description: 

local This , super = class(...,"NewActivity.AnimalsTurnTableNode")
local NewActivityManager = require("NewActivity.NewActivityManager")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local ViewUtil = require("UI.ViewUtil")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local CostConfirmPanel = require("UI.CostConfirmPanel")
local RedTurnTableInfo = require("NewActivity.RedTurnTableInfo")
local HeroManager = require("Hero.HeroManager")
local AnimalsActivityConfig = require("Metadatas.AnimalsActivityConfig")
local WarfareUtil = require("Warfare.WarfareUtil")
local GodBeastManager = require("GodBeast.GodBeastManager")
local ArgConfig = require("Metadatas.ArgConfig")

local kMaxRewardNum = 12
local kRewardRoundNum = 1 * kMaxRewardNum
local kMaxRound = kMaxRewardNum * 200
local kDecelationNum = 1
local kBlinkNum = 6
local kMaxJifen = AnimalsActivityConfig[1].max_lucky_count
local itemKey = ArgConfig.red_hero_cost_item.arg[1]

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RedHeroTurnTable.csb")
    self:addChild(self.m_csNode)
    -- self.m_csNode:setToCenter()

    local activityType = NewActivityType.kRedHeroTable
    self.m_activityType = activityType
    self.m_activity = NewActivityManager:getActivity(activityType)
    
    self.m_keyItemId = self.m_activity:getActKeyItemId()
    self.m_useResMode = self.m_activity:isUseResMode()
    self.m_itemList = self.m_activity:getDrawItemList()
    local node = self.m_csNode
    for i=1, kMaxRewardNum do
        node["pot" .. tostring(i)].selected:setVisible(false)
        local itemInfo = self.m_itemList[i]
        if itemInfo then
            local itemId, num, isRale = itemInfo.item_id, itemInfo.num, itemInfo.if_rare
            local itemNode = node["item" .. tostring(i)]
            local rareNode = node["dazhetiao"..tostring(i)]
            rareNode:setVisible(isRale==1)
            if itemNode then
                self:initRewardItemNode(itemNode, itemId, num)
                if os.isWin32 then
                    itemNode.num:setString(string.format("id:%d-%s", itemInfo.id, ItemUtil.formatItemCount(num)))
                end
            end               
        end
    end
    self:refreshItemNumNode()
    -- if self.m_useResMode then
    --     self:updateItemNumNode( GamePlayer:getDiamond() )
    -- end
    self:initView()
    self:initTouchEvent()
end

function This:initView()
    local csNode = self.m_csNode
    csNode.number4:setString(self.m_activity:getActKeyItemPrice(ActPrizeDrawType.kOnce))
    csNode.number6:setString(self.m_activity:getActKeyItemPrice(ActPrizeDrawType.kTenTimes)*10)
    csNode.number2:setString(GamePlayer:getDiamond())
    csNode.number1:setString(DepotManager:getItemCount(itemKey))
end

function This:refreshItemNumNode()
    local csNode = self.m_csNode
    GameUtil:createBoxEffect(self.m_activity:hasRewardCanReceive(),csNode.box11,2)
    if self.m_activity:hasReceiveAllReward() then
        csNode.box11:setGrayShader()
    else
        csNode.box11:clearShader()
    end

    local scrollView = csNode.scrollView1
    scrollView:removeAllItems()
    local stringList=self.m_activity:getRewardStringList()
    if stringList then
        for _,rewardString in pairs(stringList) do
            local view = RedTurnTableInfo:create({text = rewardString})
            local w,h = view:getSize()
            scrollView:pushBackCustomItem(view, AppText:isEnglish() and w or 200, h)
        end
    end
end

function This:onEnter()
    self.m_activity:addObserver(self)
    self.m_activity:onGetActivityInfoRequest()
    self:refreshItemNumNode()
    GamePlayer:addObserver(self)
end

function This:onExit()
    self.m_activity:removeObserver(self)
    GamePlayer:removeObserver(self)
end

function This:initTouchEvent()
    local node = self.m_csNode
    node.weigouxuan:setTouch({isDynamic = true,onEnded = function()
        if not self.m_runningAniantion then
            local visible = node.gouxuan:isVisible()
            if self.m_showAnimation == not visible then 
                self.m_showAnimation = visible
                node.gouxuan:setVisible(not visible)
            end
        end
    end})

    node.button1:setTouch({isDynamic = true,onEnded = function()
        -- local level = GamePlayer:getLevel()
        -- local limitLevel = ArgConfig["mythical_campaign_refresh_open_level"].arg[1]
        -- if level >= limitLevel then
        self:startPrizeDraw()      
        -- else
        --     g_scene:showTips(Serialize[62069])
        -- end
    end})

    node.button2:setTouch({isDynamic = true,onEnded = function()
        -- local level = GamePlayer:getLevel()
        -- local limitLevel = ArgConfig["mythical_campaign_refresh_open_level"].arg[1]
        -- if level >= limitLevel then
        self:startPrizeDraw(true)      
        -- else
        --     g_scene:showTips(Serialize[62069])
        -- end  
    end})

    node.jiahaodi2:setTouch({isDynamic = true,onEnded = function ()
        g_scene:openDialog("System.RechargeMainUI")
    end})

    -- node.guize:setTouch({isDynamic = true,onEnded = ffunction()
    --     g_scene:openDialog("UI.HelpTips",{text = Serialize["AnimalsActivity"],node = node.guize})
    -- end})

    node.box11:setTouch({isDynamic = true,onEnded = function ()
        g_scene:openDialog("NewActivity.RedHeroRewardPanel",{activity = self.m_activity})
        
    end})
end

function This:startPrizeDraw( multiTime )
    if self.m_runningAniantion or not self.m_activity or not self.m_activity:isUpdating() then
       return
    end

    local dType = ActPrizeDrawType.kOnce
    if multiTime then
        dType = ActPrizeDrawType.kTenTimes
    end
    local dNumType = ActPrizeDrawNumType[dType]
    if not self.m_activity:checkHasLeftTimes(dNumType) then
        g_scene:showTips(Serialize[11510])
        return
    end

    local reuestDraw = function ()
        if self.m_showAnimation then
            self.m_runningAniantion = true
            if self.m_dialog then
                self.m_dialog:setIsRunning(true)
            end
            self:runSelectionItem()
        end
        self.m_activity:onActivityPrizeDrawRequest(dNumType)
    end
    local num = dNumType - self.m_activity:getLeftTimes()
    if num <= 0 then
        num = 0
    end
    local costNum = self:calculateCost(dType, num)

    if costNum > 0 then
        local notips = "Activity" .. tostring(self.m_activity:getId()) .. "shuaxin" .. tostring(dNumType)
        local check = function ()
            local result, errorCode = DepotManager:isItemOrResEnough(self.m_keyItemId, costNum)
            if not result then
                if errorCode then
                    g_scene:handleErrorCode(errorCode)
                else
                    g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(self.m_keyItemId)))
                end
            end
            return result
        end
        
        if not CostConfirmPanel:existNoTipsKey(notips) then
            local openDialog = function ()
                local myPanel=g_scene:openDialog("UI.CostConfirmPanel", {
                        fileName = "MenuScene/NodeUI/ActivityHappyRewardVoucher.csb",
                        notips = notips,
                        cost = costNum,
                        costId = self.m_keyItemId,
                        descText = string.format(Serialize[62111],num),
                        okHandler = function()
                            coroutine.resume(self.m_coroutine)
                        end }
                    )
                    myPanel:setBtnString(descText,Serialize[62112])
                    return coroutine.yield()
            end

            self.m_coroutine = coroutine.create(function ()
                openDialog()
                if check() then
                    reuestDraw()
                end
            end)
            coroutine.resume(self.m_coroutine)
        else
            if check() then
                reuestDraw()
            end
        end
    else
        reuestDraw()
    end 
end


function This:onUpdateActivity(activity)
    local type = activity:getId()
    if type == NewActivityType.kRedHeroTable then
        self:refreshTime()
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.diamond then
        self.m_csNode.number2:setString(GamePlayer:getDiamond())
    end
    self.m_csNode.number1:setString(DepotManager:getItemCount(itemKey))
end

-- function This:onActivityPrizeDraw(data)
--     if self.m_activity:getId() == data.id then
--         self.m_csNode.number1:setString(DepotManager:getItemCount(itemKey))
--     end
-- end

function This:onRefreshPrizeDraw(id)
    if self.m_activity:getId() == id then
        self:refreshItemNumNode()
    end
end