-- @Author: superm
-- @Date:   2018-08-07 11:34:33
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-09 10:43:51
-- @File Name: ScoreShopTabNode.lua
-- @Description: 

local This, super = class(..., "NodeView")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ItemUtil = require("Bag.ItemUtil")
-- local ViewUtil = require("UI.ViewUtil")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")

local ScoreShopItemView = class(This._NAME ..".ScoreShopItemView", "NewActivity.WestShopItemView")
ScoreShopItemView.fileName = "MenuScene/NodeUI/ActivityHappyShopItem.csb"

function This:initData( queue, dialog )
    self.m_actId = NewActivityType.kScoreShop
	self.m_activity = queue
    self.m_dialog = dialog
	self.m_curViews = {}
end

function This:initNode()
    self.m_fileName = self.m_activity:getUIFileName()
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/" .. self.m_fileName .. ".csb")
	self:addChild(self.m_csNode)
    self.m_csNode.tip:setVisible(false)
    -- ViewUtil:blinkView(self.m_csNode.tip, true)
    self:itemUpdate()
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    GamePlayer:addObserver(self)
    if self.m_activity then
        self.m_activity:addObserver(self)
    end
    self:refresh()
end

function This:onExit()
    if self.m_activity then
        self.m_activity:removeObserver(self)
    end
    GamePlayer:removeObserver(self)
    DepotManager:removeObserver(self)
    super.onExit(self)
end


function This:initTouchEvent()
    if not self.inited then
        self.m_csNode.button:setTouchEnded(function ()
            g_scene:openDialog("System.RechargeMainUI")
        end)
        self.m_inited = true
    end
end


function This:refresh(forceUpdate)
    self:refreshItemList()
    self:itemUpdate()
end

function This:refreshItemList(refresh)
    local scollview = self.m_csNode.scrollView
    if refresh then
        scollview:removeAllItems()
        scollview:setTouchEnabled(true)
        self.m_curViews = {}
    end

    local slots = self.m_activity:getAllSlotItem()
    for index, slot in ipairs(slots) do
        local view = self.m_curViews[index]
        if not view then
            view = ScoreShopItemView:create({item = slot, isContainer = true})
            self.m_curViews[index] = view
            local w , h = view:getContentSize()
            scollview:pushNode(view, w-10, h, 2) 
        else
            view:refresh(refresh)
        end
    end
end

function This:onGetActShopInfo(id)
    if id == self.m_actId then
        self:refresh(true)
    end
end

function This:onBuyActShopItem(id, index)
    if id == self.m_actId then
        if index then 
            local view = self.m_curViews[index]
            if view then
                view:refresh()
            end
            self:itemUpdate()
        else
            self:refresh()
        end
    end
end

function This:formatDTime(format,time)
    local day = math.floor(time / 86400)
    time = time % 86400
    local hour = math.floor(time / 3600)
    time = time % 3600
    local minue = math.floor(time / 60)
    time = time % 60
    local second = time
    if day > 0 then
        format = string.gsub(format,Serialize[62117],string.format(Serialize[62118],day))
    else
        format = string.gsub(format,Serialize[62117],"")
    end

    if hour > 0 then
        format = string.gsub(format,Serialize[62119],string.format(Serialize[62120],hour))
    else
        format = string.gsub(format,Serialize[62119],"")
    end

    format = string.gsub(format,"M",string.format("%d",minue))
    format = string.gsub(format,"S",string.format("%d",second))
    return format
end

function This:itemUpdate()
    self.m_csNode.number:setString(DepotManager:getItemCount(kScoreShopCostItemId))
end