-- @Author: Su
-- @Date:   2019-03-25 10:01:41
-- @Last Modified by:   Su
-- @Last Modified time: 2019-03-25 10:01:41
-- @File Name: SevenDayCelebrationItem.lua

local This , super = class(...,"NodeUI")
local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local MatchingConfig = require("Metadatas.MatchingConfig")


local nameList={
    Serialize[62125],
    Serialize[62126],
    Serialize[62127]
}

local tipsList={
    Serialize[62128],
    Serialize[62129],
    Serialize[62130],
}


function This:initData(info)
    self.m_tabIndex = info.tabIndex
    self.m_data = info.data[info.index] or info.data
    self.m_index = info.index or 0
    self.m_activity = info.activity
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/SevenDayCelebrationItem"..self.m_tabIndex..".csb")
    self:addChild(self.m_csNode)
    self:refresh()
    self:refreshItemList()
end

function This:onEnter()
    super:onEnter(self)
    self:initTouchEvent()
    self.m_activity:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    self.m_activity:removeObserver(self)
end
function This:refresh()
    local config = self.m_data
    local node = self.m_csNode
    if self.m_tabIndex == SevenDayActivityTab.kSpendGold then
        local roleList = self.m_activity:getRoleList()
        if config.mix_ranking == config.max_ranking then
            node.condition:setString(string.format( Serialize[62131],config.mix_ranking))
            node.name1:setVisible(false)
            node.name3:setVisible(false)
            if #roleList > 0 then
                node.name2:setString(roleList[1].name)
            else
                node.name2:setString(Serialize[60243])
            end
        elseif config.max_ranking - config.mix_ranking ==1 then
            node.condition:setString(string.format( Serialize[62132],config.mix_ranking,config.max_ranking))
            local index = 1
            for i =config.mix_ranking,config.max_ranking do
                if roleList[i] then
                    node["name"..index]:setString(roleList[i].name)
                else
                    node["name"..index]:setString(Serialize[60243])
                end
                index = index+1
            end
            -- if roleList[2] then
            --     node.name1:setString(roleList[2].name)
            -- else
            --     node.name1:setString(Serialize[60243])
            -- end
            -- if roleList[3] then
            --     node.name2:setString(roleList[3].name)
            -- else
            --     node.name2:setString(Serialize[60243])
            -- end
            node.name3:setVisible(false)
            node.name1:setPositionY(18)
            node.name2:setPositionY(-7)
        else
            node.condition:setString(string.format( Serialize[62132],config.mix_ranking,config.max_ranking))
            local index = 1
            for i =config.mix_ranking,config.max_ranking do
                if roleList[i] then
                    node["name"..index]:setString(roleList[i].name)
                else
                    node["name"..index]:setString(Serialize[60243])
                end
                index = index+1
            end
            -- if roleList[4] then
            --     node.name1:setString(roleList[4].name)
            -- else
            --     node.name1:setString(Serialize[60243])
            -- end
            -- if roleList[5] then
            --     node.name2:setString(roleList[5].name)
            -- else
            --     node.name2:setString(Serialize[60243])
            -- end
            -- if roleList[6] then
            --     node.name3:setString(roleList[6].name)
            -- else
            --     node.name3:setString(Serialize[60243])

            -- end
        end
    elseif self.m_tabIndex == SevenDayActivityTab.kCumulative then
        config = self.m_data:getConfig()
        node.condition:setString(string.format( Serialize[62133],config.progress,100))
        if config.progress > self.m_activity:getMyConsumption() then
            node.button:setGrayShader()
        else
            node.button:clearShader()
        end
        node.button:setVisible(not self.m_activity:hasTaskComplete(self.m_data:getId()))
        node.pickuptext:setVisible(self.m_activity:hasTaskComplete(self.m_data:getId()))
    elseif self.m_tabIndex == SevenDayActivityTab.kBestKing then
        for i =1,3 do
            local csNode = self.m_csNode["player"..i]
            local roleInfo = self.m_activity:getRoleInfoByIndex(i)
            local text,nameText
            csNode.icon:setVisible(roleInfo ~= nil)
            csNode.name:setVisible(roleInfo ~= nil)
            csNode.button.text:setString(nameList[i])
            if roleInfo == nil then
                text = tipsList[i]..Serialize[60243]
                nameText = Serialize[60243]
            else
                local text2 = self:getText(roleInfo.value,i)
                text =tipsList[i]..text2
                nameText = roleInfo.name
                GameUtil:setPlayerFeltNodeResource(csNode.icon, roleInfo.icon)
            end
            csNode.shangbang:setString(text)
            csNode.name:setString(nameText)
        end
    elseif self.m_tabIndex == SevenDayActivityTab.kDailyRecharge then
        node.condition:setString(string.format( Serialize[62134],config.progress))
        if config.progress > self.m_activity:getMyConsumption() then
            node.button:setGrayShader()
        else
            node.button:clearShader()
        end
        node.button:setVisible(not self.m_activity:hasTaskComplete(self.m_data.id))
        node.pickuptext:setVisible(self.m_activity:hasTaskComplete(self.m_data.id))
    end
end

function This:refreshItemList()
    local scrollView = self.m_csNode.scrollView
    if scrollView then
        scrollView:setDirection("horizontal")
        scrollView.m_direction = "right"
        scrollView:removeAllItems()
        for _,itemConfig in pairs(self.m_data.reward) do
            local item = ItemNode:create(itemConfig[1],itemConfig[2])
            local w , h = item:getContentSize()
            item:setAnchorPoint(0.5,0.5)
            scrollView:pushBackCustomItem(item,w,h,0.5)
        end
    end
end

function This:initTouchEvent()
    local node = self.m_csNode
    if node.button then
        node.button:setTouch({
            isScale = false, 
            isDynamic = true,     
            onEnded =function ()
                print("点击")
                if self.m_data.progress > self.m_activity:getMyConsumption() then
                    return
                end
                self.m_activity:onPickupActRewardRequest(self.m_data.id)
        end})
    end

    if node.wenhao then
        node.wenhao:setTouch({
            isScale = false, 
            isDynamic = true,     
            onEnded =function ()
                g_scene:openDialog("UI.HelpTips",{text = Serialize["activity_bestking_tips"],node = self.m_csNode.wenhao})
        end})
    end

    if node.player1 then
        for i=1,3 do
            node["player"..i].button:setTouch({
                isScale = false, 
                isDynamic = true,     
                onEnded =function ()
                    g_scene:openDialog("NewActivity.SevenDayCelebrationBoxPanel",{index = NewActivityType.kSpendGold + i,name=nameList[i]})
            end})
        end
    end


    
end

function This:getText(value,index)
    if index == 1 then
        return value..Serialize[60187]
    elseif index == 2 then
        return MatchingConfig[value].rank_level
    else
        return value..Serialize[60804]
    end
    
end

--function This:onRefreshCumulative(index)
--    if self.m_tabIndex == index then
--        self:refresh()
--    end
--end
--
--function This:onRefreshDailyRecharge(index)
--    if self.m_tabIndex == index then
--        self:refresh()
--    end
--end

function This:onRefreshkBestKing()
    self:refresh()
end