-- @Author: superm
-- @Date:   2018-03-14 16:50:40
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 16:47:15
-- @File Name: WestShopItem.lua
-- @Description: 


local This , super = class(...,"Base.BaseModel")
local ItemUtil = require("Bag.ItemUtil")
local GamePlayer = require("Player.GamePlayer")
local TerritoryManager = require("Territory.TerritoryManager")
local DepotManager = require("Depot.DepotManager")
local WestShopConfig = require("Metadatas.WestShopConfig")
local Socket = require("Net.Socket")

function This:init( info )
    super.init(self)
    if info then
        self:initSelf( info )
        self:updateState(true, info)
    else
        self.m_ID = 0
        self.m_state = TerrObjectState.kLocked
    end
end

function This:initSelf( info )
    self.m_ID = info.grid_id or 0

    self.m_itemId = info.item_id
    self.m_itemNum = info.item_num
    self.m_costType = info.price_type
    -- self.m_price = info.price_num
    self.m_buyTime = info.cur_buy_times
    self.m_buyTimeLimit = nil
    self.m_configId = info.config_id

    self.m_config = self:getConfigById(info.config_id)
end

function This:getConfigById( id )
    return WestShopConfig[id]
end

function This:updateInfo( info )
    local isDiff = self:isSameType(info)
    self:initSelf( info )
    self:updateState(isDiff, info)
    -- print("槽位" .. tostring(self.m_ID), ItemUtil.getName(self.m_itemId), "购买次数:" .. tostring(self.m_buyTime))
end

function This:updateState(isInit, info)
    if isInit then
        self:initSelf(info)
    end

    local conds = self:getCondition()
    if not conds then
        self.m_state = TerrObjectState.kNormal
    else
        local ret = self:isNotLockedBy(conds)
        self.m_state = ret and TerrObjectState.kNormal or TerrObjectState.kLocked
    end
end

function This:getCondition()
end

function This:isNotLockedBy( conds )
    return true
end

function This:increaseBuyTime()
    if not self:isMaxTime() then
    	self.m_buyTime = self.m_buyTime + 1
    end
end

function This:setId( id )
    self.m_ID = id
end

function This:getId()
    return self.m_ID
end

--道具ID
function This:getConfigId()
    return self.m_configId or self.m_config.id
end

function This:isSameType( info )
    return self.m_ID == info.grid_id and self.m_itemId == info.item_id 
end

function This:isCostEnoughBy()
    local id, num = self:getPriceItemId(), self:getPrice()
    if ItemUtil.getType(id) == ItemCommonType.kGameRes then
        local switch = {
            [101] = function(num)
                return GamePlayer:getGold() >= num, 101
            end,
            [102] = function(num)
                return GamePlayer:getDiamond() >= num , 102
            end,
        }
        return switch[id](num)
    elseif ItemUtil.getType(id) == ItemCommonType.kProps then
        return DepotManager:getItemCount(id) >= num, id
    end
end

function This:isLocked()
    return (self.m_state == TerrObjectState.kLocked) or (not self.m_config)
end

function This:isOpening()
    return self.m_state == TerrObjectState.kNormal
end

function This:getItemId()
    return self.m_itemId or self.m_config.item_id
end

function This:isMaxTime()
    return self.m_buyTime >= self:getMaxBuyTime()
end

function This:getBuyTime()
    return self.m_buyTime
end

function This:getMaxBuyTime()
    return self.m_buyTimeLimit or self.m_config.num
end

function This:setActivityQueue( queue )
    self.m_activityQueue = queue
end

function This:getActivityId()
	return self.m_activityQueue:getId()
end

function This:onBuyActShopItemRequest()
    Socket:sendAndLock(12028, {activity_id = self:getActivityId(), grid_id = self:getId()})
end

function This:requestBuy()
    if self:isMaxTime() then
        g_scene:showTips(Serialize[11512])
        return false
    end

    local state, id = self:isCostEnoughBy() 
    if state then
        if self.m_activityQueue and self.m_activityQueue:canBuy() then
    	    self:onBuyActShopItemRequest()
            return true
        else
            g_scene:showTips(Serialize[11512])
            return false
        end
    else
        if id == 102 then
            GamePlayer:checkDiamond(0, true)
        else
            g_scene:showTips(Serialize[60714] .. ItemUtil.getName(id) .. Serialize[60715])
        end
    end
    return false
end

--获取配置信息region begin

function This:getItemNum()
    return self.m_itemNum or self.m_config.item_num
end

function This:getPriceType()
    return self.m_costType or self.m_config.money_type
end

function This:getPrice()
    return self.m_config.price[self:getBuyTime() + 1]
end

function This:getName()
    return ItemUtil.getName(self:getItemId())
end

function This:getItemImage()
    return ItemUtil.getIconImage(self:getItemId())
end

function This:getItemQulityImage()
    return ItemUtil.getQualityImage(self:getItemId())
end

function This:getQuality()
    return ItemUtil.getQuality(self:getItemId())
end

function This:getItemQulity()
    return ItemUtil.getQuality(self:getItemId())
end

function This:getPriceItemId()
    local priceType = self:getPriceType()
    if priceType == 1 then
        return 102
    elseif priceType == 2 then
        return 124
    end
end

function This:getPriceTypeImage()
    return ItemUtil.getIconImage(self:getPriceItemId())
end
