-- @Author: superm
-- @Date:   2018-03-14 19:37:35
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-04 17:16:38
-- @File Name: WestShopPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local NewActivityManager = require("NewActivity.NewActivityManager")
local ItemUtil = require("Bag.ItemUtil")
local ViewUtil = require("UI.ViewUtil")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")
local WestShopItemView = require("NewActivity.WestShopItemView")

function This:initNode(info)
    self.m_actId = NewActivityType.kWestShop1
    if info and info.activityId > self.m_actId then
        self.m_actId = info.activityId
    end
    self.m_activity = NewActivityManager:getActivity(self.m_actId)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/XiYuShangDian.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    
    self.m_csNode.tips:setVisible(false)
    ViewUtil:blinkView(self.m_csNode.tips, true)
    self:itemUpdate()
    self:initItemList()
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    GamePlayer:addObserver(self)
    if self.m_activity then
        self.m_activity:addObserver(self)
        self.m_activity:onGetActivityInfoRequest()
    end
    self:refresh()
end

function This:onExit()
    if self.m_activity then
        self.m_activity:removeObserver(self)
    end
    GamePlayer:removeObserver(self)
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.help:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize["westshop_help_tips"],node = self.m_csNode.help})
    end)

    self.m_csNode.shuaxin_button:setTouch({
        onEnded = function()
            if not self.m_activity:canBuy() then
                g_scene:showTips(Serialize[62141])
                return
            end

            local hasHighLevItem = false

            for _, view in ipairs(self.m_viewList) do
                hasHighLevItem = view:isHighLevelNotBuy()
                if hasHighLevItem then break end
            end

            local costNum = self.m_activity:getRefreshCost()
            g_scene:openDialog("UI.CostConfirmPanel", {
                cost = costNum,
                -- costId = 102,
                desc = string.format(Serialize[62142], dNumType),  
                note = hasHighLevItem and Serialize[62143] or nil,
                okHandler = function ()
                    self.m_activity:onRefreshWestShopRequest()
                end
            })
        end,
        isAlpha = false,
    })
end

function This:refresh(forceUpdate)
    self.m_hasLeftBuyTime = self.m_activity:canBuy() 
    local items = self.m_activity:getAllSlotItem()
    if items then
        for index, item in ipairs(items) do
            item:updateState()
            self:refreshItem(index, item, forceUpdate)
        end
        if not self.m_viewList then
            self:initItemList()
        end
    end

    self.m_csNode.free.number:setString(tostring(self.m_activity:getLeftTotalBuyTime()))
    self:refreshResetTimes()
end

function This:initItemList()
    self.m_viewList = {}
    local count = self.m_activity:getSlotCount()
    for i=1, count do
        local itemNode = self.m_csNode["item" .. i]
        local item = self.m_activity:getSlotItemAtIndex(i)
        if itemNode then
            local view = WestShopItemView:create({item = item})
            itemNode:addChild(view)
            self.m_viewList[i]=view
        end
    end
end

function This:refreshItem(i, item, forceUpdate)
    local view = self.m_viewList[i]
    if view then
        view:refresh(forceUpdate)
    end
end

function This:refreshResetTimes()
    
end

function This:onGetActShopInfo(id)
    if id == self.m_actId then
        self:refresh(true)
    end
end

function This:onBuyActShopItem(id)
    if id == self.m_actId then
        self:refresh()
    end
end

function This:onUpdateActivity( activity )
    if activity and activity:getId() == self.m_actId then
        self.m_csNode.free.shoptips:setString(Serialize[62144] .. self:formatDTime(Serialize[62145], activity:getLeftTime()))
    end
end

function This:onActivityEnded( activity )
    if activity and activity:getId() == self.m_actId then
        g_scene:showTips(Serialize[61051])
        self:close()
    end
end

function This:onRefreshWestShop( activity )
    if activity and activity:getId() == self.m_actId then
        self:refresh(true)
    end
end

function This:formatDTime(format,time)
    local day = math.floor(time / 86400)
    time = time % 86400
    local hour = math.floor(time / 3600)
    time = time % 3600
    local minue = math.floor(time / 60)
    time = time % 60
    local second = time
    if day > 0 then
        format = string.gsub(format,Serialize[62117],string.format(Serialize[62118],day))
    else
        format = string.gsub(format,Serialize[62117],"")
    end

    if hour > 0 then
        format = string.gsub(format,Serialize[62119],string.format(Serialize[62120],hour))
    else
        format = string.gsub(format,Serialize[62119],"")
    end

    format = string.gsub(format,"M",string.format("%d",minue))
    format = string.gsub(format,"S",string.format("%d",second))
    return format
end

function This:itemUpdate()
    self.m_csNode.text3:setString(DepotManager:getItemCount(124))
    self.m_csNode.text4:setString(DepotManager:getItemOrResCount(102))
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.diamond then
        self.m_csNode.text4:setString(value)
    end
end