
--爵位管理

local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local PlanShopConfig = require("Metadatas.PlanShopConfig")
local NobilityTitleConfig = require("Metadatas.NobilityTitleConfig")
local NobilityAchieve = require("Nobility.NobilityAchieve")
local FunctionNoticeManager = require("Function.FunctionNoticeManager")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")


function This:init()
	super.init(self)
	self.m_isInit = false
	self.m_nobility = 0
	self.m_achieves = {}
	self.m_rewards = {} --已领取奖励
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 11900, self.onGetAchieveInfoReply)
	EventDispatcher:addListener(self, 11901, self.onAchieveChangeNotify)
	EventDispatcher:addListener(self, 14100, self.onNobilityInfoReply)
	EventDispatcher:addListener(self, 14101, self.onNobilityUpgradeReply)
	EventDispatcher:addListener(self, 14102, self.onNobilityRewardReply)
	GamePlayer:addObserver(self)
end

--当前爵位
function This:getNobility()
	return self.m_nobility
end

--最大爵位
function This:getMaxNobility()
	if not self.m_maxNobility then
		local list = {}

		for k, v in pairs(NobilityTitleConfig) do
			table.insert(list, v)
		end

		table.sort(list, function (a, b)
			return a.id < b.id
		end)

		self.m_maxNobility = list[#list].id
	end
	return self.m_maxNobility	
end

--是否达到最大爵位
function This:isMaxNobility()
	local maxId = self:getMaxNobility()
	local curId = self:getNobility()
	return curId >= maxId
end

--检查计策解锁
function This:checkPlanUnlock(planId)
	local config = PlanShopConfig[planId]
	if not config then
		return false
	end 

	return self:getNobility() >= config.unlock_nobilitytitle
end

--爵位特权增加的值
function This:getPrivilegeNumBy(pField)
	local curId = self:getNobility()
	if curId <= 0 then
		return 0
	end

	local key = ""
	if pField == PrivilegeField[Privilege.kBuildTime] then
		key = "privilege1"
	end

	local config = NobilityTitleConfig[curId]
	return config[key] or 0
end

--爵位奖励状态   0不可领，1可领，2已领
function This:getNobRewardState(nobId)
	local curId = self:getNobility()
	if curId < nobId then
		return 0
	end
	return self.m_rewards[nobId] and 2 or 1
end

--是否有可领取的奖励
function This:isHasReward()
	local curId = self:getNobility()
	if curId > 0 then
		for id=1, curId do
			if not self.m_rewards[id] then
				return true
			end
		end
	end
	return false
end

--是否可以晋升爵位
function This:isCanUpgradeNobility()
	if self:isMaxNobility() then
		return false
	end

	return self:isFinishAllAchieves()
end

--是否完成所有条件
function This:isFinishAllAchieves()
	local data = self:getCurAchieves()
	local isAllFinish = true

	for id, achieve in pairs(data) do
		if not achieve:isFinish() then
			isAllFinish = false
			break
		end
	end

	return isAllFinish	
end

function This:onAllRequest()
	if not FunctionNoticeManager:checkOpen(FunctionOpenId.kJueWei) then
		return
	end
	-- print("-------- NobilityManager:onAllRequest")
	self:onNobilityInfoRequest()
	--self:onGetAchieveInfoRequest()
end

--计策解锁爵位
function This:getLockedNobilityName(planId)
	local config = PlanShopConfig[planId]
	if not config then
		return
	end
	return NobilityTitleConfig[config.unlock_nobilitytitle].name
end

function This:getNobilityName(level)
	return NobilityTitleConfig[level].name
end

-------------------------------------------------------------------------------
--爵位任务
-------------------------------------------------------------------------------

function This:onGetAchieveInfoRequest()
	Socket:send(11900,{type = AchieveType.kNobility})
end

function This:onGetAchieveInfoReply(msg)
	if not self:isValidAchieveType(msg.achievement.type) then
		return
	end
	--print_tmp(msg)
	self.m_achieves = {}

	for _, info in ipairs(msg.achievement.progress_list) do
		self.m_achieves[info.id] = NobilityAchieve:create(info)
	end

	self:notifyObservers("onNobilityAchieveChange")
	EventDispatcher:dispatcher("nobility_refresh")
end

function This:onAchieveChangeNotify(msg)
	for i, v in ipairs(msg.progress_list) do
		if self:isValidAchieveType(v.type) then
			local achieve = self.m_achieves[v.id]
			if achieve then
				achieve:setProgress(v.num)
			end
		end
	end
	
	self:notifyObservers("onNobilityAchieveChange")
	EventDispatcher:dispatcher("nobility_refresh")
end

function This:getAchieves()
	return self.m_achieves
end

function This:isValidAchieveType(atype)
	return atype == AchieveType.kNobility
end

function This:getCurAchieves()
	local ret = {}
	local nobId = self:getNobility()
	local config = NobilityTitleConfig[nobId]

	if config then
		local conditions = config.promotion_conditions
		for _, aid in ipairs(conditions) do
			ret[aid] = self.m_achieves[aid]
		end
	end

	return ret
end

---------------------------------------------------------------------------------

--爵位信息
function This:onNobilityInfoRequest()
	Socket:send(14100, {})
end

function This:onNobilityInfoReply(msg)
	-- print("----- onNobilityInfoReply")
	-- print_tmp(msg)
	if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    self.m_nobility = msg.level

    self.m_rewards = {}
    for _, id in ipairs(msg.reward_list) do
    	self.m_rewards[id] = true
    end

    self:updateHeroAttr(self.m_isInit)
    self.m_isInit = true
end

--爵位晋升
function This:onNobilityUpgradeRequest()
	Socket:sendAndLock(14101, {})
end

function This:onNobilityUpgradeReply(msg)
	if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    self.m_nobility = msg.level
    self:updateHeroAttr(true)
    self:onGetAchieveInfoRequest()
    self:notifyObservers("onNobilityUpgradeReply")
end

--领取爵位奖励
function This:onNobilityRewardRequest(id)
	Socket:sendAndLock(14102, {id = id})
end

function This:onNobilityRewardReply(msg)
	if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    local id = msg.id
    self.m_rewards[id] = true
    g_scene:showReward(NobilityTitleConfig[id].reward)
    self:notifyObservers("onNobilityRewardReply", id)
    EventDispatcher:dispatcher("nobility_refresh")
end

function This:onPlayerAttrChanged(field, lastValue, value)
	--主公等级检查
	if field == PlayerAttr.level and lastValue ~= value then
		local lv = FunctionNoticeManager:getOpenLevel(FunctionOpenId.kJueWei)
		if value == lv then
			self:onAllRequest()
		end
	end
end

function This:getNoblityAttr()
	local ret = {}
	local id = self:getNobility()
	local config = NobilityTitleConfig[id]

	if config then
		for i, v in ipairs(config.privilege3) do
			local id = v[1]
			ret[id] = v[2]
		end
	end
	
	return ret
end

function This:updateHeroAttr(isCalcSaint)
	local id = self:getNobility()
	if id <= 0 then
		return
	end

	local attrs = self:getNoblityAttr()
	local heros = HeroManager:getAllHero()

	for id , hero in pairs(heros) do
		if hero:isGot() then
			hero:updateNobilityAttr(attrs)
		end
	end

	if isCalcSaint then
		HeroManager:calcSaintFightPower()	
	end
end