
local This , super = class(...,"NodeUI")
local ItemNode = require("UI.ItemNode")


function This:initData(data)
    self.m_data = data
    self.m_itemList = {}
end

function This:initNode(data)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWayNobilityOverviewItem.csb")
    self:addChild(self.m_csNode)

    local scrollView = self.m_csNode.scrollView
    scrollView:setDirection("horizontal")
    scrollView:setSwallowTouch(false)

    local rewardNode = cc.Node:create()
    rewardNode:setContentSize(1, 78)
    rewardNode:setAnchorPoint(0.5, 0)
    scrollView:pushBackCustomItem(rewardNode, 1, 78, 0)
    self.m_rewardNode = rewardNode

    local w, h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w, h)
end

function This:initTouchEvent()

end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:setModel(data)
    self.m_data = data
    self:refresh()
end

function This:refresh()
    local data = self.m_data
    local csNode = self.m_csNode

    csNode.icon:setImage(data.icon)
    csNode.name:setString(data.name)
    csNode.xuanzhong:setVisible(data.id == data.myNobility)

    for i = 1, 4 do
        local lb = csNode[string.format("text%d", i)]
        local desc = data.desc[i]
        if desc and desc ~= ""  then
            lb:setVisible(true)
            lb:setStringWithColor( string.format(desc, "New18", "New19") )
        else
            lb:setVisible(false)
        end
    end

    --奖励
    for i, info in ipairs(data.reward) do
        if self.m_itemList[i] then
            self.m_itemList[i]:setVisible(true)
            self.m_itemList[i]:setModel({id = info[1], num = info[2]})
        else
            local item = ItemNode:create(info[1], info[2])
            item:setAnchorPoint(0, 0.5)
            item:setScale(0.8)
            self.m_rewardNode:addChild(item)
            table.insert(self.m_itemList, item)        
        end
    end    

    if #self.m_itemList > #data.reward then
        for i = #data.reward+1, #self.m_itemList do
            self.m_itemList[i]:setVisible(false)
        end
    end

    self:updateRewardNodeSize()
end

function This:updateRewardNodeSize()
    local scrollView = self.m_csNode.scrollView
    local minWidth = scrollView:getContentWidth()
    local w = 0
    local h = self.m_rewardNode:getContentHeight()

    for i, node in ipairs(self.m_itemList) do
        if node:isVisible() then
            local box = node:getBoundingBox()
            node:setPosition(w, h/2)
            w = w + box.width + 5
        end
    end

    if w < minWidth then
        w = minWidth
    end
    self.m_rewardNode:setContentSize(w, h)

    self.m_rewardNode.itemWidth = w
    self.m_rewardNode.itemHeight = h
    scrollView.m_itemWidth = w
    scrollView.m_itemHeight = h
    scrollView:refreshPosition()
end

