

local This , super = class(...,"NodeUI")
local RankPageListLayer = require("Rank.RankPageListLayer")
local NobilityTitleConfig = require("Metadatas.NobilityTitleConfig")
local NobilityManager = require("Nobility.NobilityManager")
local DragSelectContainer = require("UI.DragSelectContainer")
local NobilityAchieveItem = require("Nobility.NobilityAchieveItem")
local NobilityNode = require("Nobility.NobilityNode")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")


function This:initData(parent)
    self.m_parent = parent
	self.m_selectIdx = 0
    self.m_nobilityList = {}
	self.m_achieveList = {}

    self:initNobilityList()
	self:initAchieveList()
end

function This:initNobilityList()
    self.m_nobilityList = {}

    for k, v in pairs(NobilityTitleConfig) do
        table.insert(self.m_nobilityList, v.id)
    end

    table.sort(self.m_nobilityList, function (a, b)
        return a < b
    end)
end

function This:initAchieveList()
	local achieves = NobilityManager:getCurAchieves()
	local list = {}

	for k, v in pairs(achieves) do
		table.insert(list, v)
	end

	if #list > 1 then
		table.sort(list, function (a, b)
			local aFinish = a:isFinish() and 1 or 0
			local bFinish = b:isFinish() and 1 or 0
			if aFinish == bFinish then
				return a:getId() < b:getId()
			else
				return aFinish < bFinish
			end
		end)
	end

	self.m_achieveList = list
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingWayNobility.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local x1, y1 = csNode.jiantou1:getPosition()
    local x2, y2 = csNode.jiantou2:getPosition()

    --爵位列表
    self.m_container = NobilityNode:create(curId)
    self.m_container:setAnchorPoint(0.5, 0.5)
    self.m_container:setPosition(x1+0.5*(x2-x1), y1)
    csNode:addChild(self.m_container)

    local curId = NobilityManager:getNobility()
    if curId > 0 then
        self:setSelectById(curId)
    else
        self:setSelectIdx(1)
    end

    -- self.m_container = DragSelectContainer:create({
    --     width = x2-x1-50,
    --     height = 140,
    --     space = 0,
    --     maxScale = 1,
    --     minScale = 0.1,
    --     initSpeed = 300,
    -- })
    -- self.m_container:setAnchorPoint(0.5, 0.5)
    -- self.m_container:setPosition(x1+0.5*(x2-x1), y1)
    -- csNode:addChild(self.m_container)

    -- local curId = NobilityManager:getNobility()
    -- local selectIdx = 1
    -- for i, id in ipairs(self.m_nobilityList) do
    --     local view = NobilityNode:create(id)
    --     view:setSelectHandler(function(isSelect)
    --         if isSelect then
    --             self:setSelectIdx(i)
    --         end
    --     end)
    --     self.m_container:addItemNode(view)

    --     if id == curId then
    --         selectIdx = i
    --     end
    -- end
    -- self.m_container:setLocalZOrder(99)
    -- self:setSelectIdx(selectIdx)


    --条件列表
    local scrollView = csNode.scrollView
	scrollView:setDirection("vertical")

	self.m_listLayer = RankPageListLayer:create({
        itemWidth = 386,
        itemHeight = 62,
        spaceX = 0,
        spaceY = 3,
        scrollView = scrollView,
		dataCounter = function ()	 
	        return #self.m_achieveList
        end,
        viewCreator =  function(index, node) 
        	local data = self.m_achieveList[index]
        	if data then     
			    if not node then
                    node = NobilityAchieveItem:create(data, index)
                    node:setGotoCb(function ()
                        if self.m_parent then
                            self.m_parent:close()
                        end
                    end)
				    return node
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self:refreshAchieve()

end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        local csNode = self.m_csNode

        -- 特权预览
        csNode.button:setTouchEnded(function ()
            g_scene:openDialog("Nobility.NobilityPrivilegePanel")
        end)

        -- 奖励盒子
        csNode.box2:setTouchEnded(function ()
            self:onRewardBoxClick()
        end)

        -- 左箭头
        csNode.jiantou1:setTouchEnded(function ()
            self:setSelectIdx(self.m_selectIdx - 1)
        end)

        -- 右箭头
        csNode.jiantou2:setTouchEnded(function ()
            self:setSelectIdx(self.m_selectIdx + 1)
        end)

        -- 晋升
        csNode.upgrade:setTouchEnded(function ()
            self:onUpgradeClick()
        end)

        -- 帮助
        csNode.help:setTouchEnded(function()
            g_scene:openDialog("UI.HelpTips",{text=Serialize["hero_nobility_tips"], node = csNode.help})
        end)
    end
end

function This:onEnter()
    super.onEnter(self)
    NobilityManager:addObserver(self)
end

function This:onExit()
    NobilityManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
	self:refreshNobilityInfo()
    self:refreshAchieve()
end

function This:refreshNobilityInfo()
    local csNode = self.m_csNode
    local selectId = self:getSelectId()
	local config = NobilityTitleConfig[selectId]

    csNode.name:setString(config.name)

	for i = 1, 4 do
        local lb = csNode[string.format("text%d", i)]
        local desc = config[string.format("privilege_desc%d", i)]
        if desc and desc ~= "" then
            lb:setVisible(true)
            if i == 4 then
                lb:setStringWithColor( string.format(desc, "A5", "Q5") )
            else
                lb:setStringWithColor( string.format(desc, "A5", "New17") )
            end
        else
            lb:setVisible(false)
        end
    end


    local state = NobilityManager:getNobRewardState(selectId)
    
    local box = csNode.box2
    local boxPic = (state == 0) and "MenuScene/boxLock4.png" or "MenuScene/boxGet4.png"
    box:setImage(boxPic)
    box:clearShader()

    local effectList = {"BoxLing","BoxLing2"}

    if state == 0 then
    	--不可领
    	GameUtil:hideNodeList(box, effectList)
    elseif state == 1 then
    	--可领
    	for _, name in ipairs(effectList) do
            if not box[name] then
                local eff = GameUtil:createEffect(name, true, box)
                eff:setScale(1.2)
                box:addChild(eff, -1)
                box[name] = eff
            end
            box[name]:setVisible(true)
        end
    else
    	--已领
    	GameUtil:hideNodeList(box, effectList)
    	box:setGrayShader()
    end

    csNode.jiantou1:setVisible( self.m_selectIdx > 1 )
    csNode.jiantou2:setVisible( self.m_selectIdx < #self.m_nobilityList )
end

function This:refreshAchieve()
	local csNode = self.m_csNode
	self.m_listLayer:clearNode(true, true)
	self.m_listLayer:setDataList()

	local isMax = NobilityManager:isMaxNobility()
	csNode.upgrade:setVisible(not isMax)

    local isFinish = NobilityManager:isFinishAllAchieves()
    if isFinish then
        csNode.upgrade:clearShader()
    else
        csNode.upgrade:setGrayShader()
    end

    csNode.title1:setString(isMax and Serialize[62161] or Serialize[62162])

    
end

function This:setSelectIdx(index)
    if index < 1 or index > #self.m_nobilityList then
        return
    end

    if self.m_selectIdx == index then
        return
    end

    self.m_selectIdx = index
    self.m_container:setId(self.m_nobilityList[index])
    self:refreshNobilityInfo()
end

function This:setSelectById(selectId)
    local index = 0

    for i, id in ipairs(self.m_nobilityList) do
        if id == selectId then
            index = i
            break
        end
    end

    self:setSelectIdx(index)
end

function This:getSelectId()
    return self.m_nobilityList[self.m_selectIdx] or 0
end

--点击奖励盒子
function This:onRewardBoxClick()
    local id = self:getSelectId()
    if id == 0 then
        return
    end

    local state = NobilityManager:getNobRewardState(id)
    if state == 2 then
        return --已领
    end

    local config = NobilityTitleConfig[id]
    local tips = string.format(Serialize[62163], config.name)

    g_scene:openDialog("Function.FunctionRewardBoxPanel", {
        title = Serialize[60659],
        state = state, -- 0不可领取，1可领取
        text = tips,
        reward = config.reward,
        onBtnClick = function ( ... )
            NobilityManager:onNobilityRewardRequest(id)
        end
    })
end

--点击晋升
function This:onUpgradeClick()
    if NobilityManager:isMaxNobility() then
        g_scene:showTips(Serialize[62164])
        return
    end
    
    if not NobilityManager:isFinishAllAchieves() then
        g_scene:showTips(Serialize[62165])
        return
    end

    NobilityManager:onNobilityUpgradeRequest()
end


function This:onNobilityUpgradeReply()
    local curId = NobilityManager:getNobility()
    local selectIdx = 1
    for i, id in ipairs(self.m_nobilityList) do
        if id == curId then
            selectIdx = i
        end
    end
    self:setSelectIdx(selectIdx)
    self:initAchieveList()
    self:refreshAchieve()
end

function This:onNobilityRewardReply(id)
    if self:getSelectId() == id then
        self:refreshNobilityInfo()
    end
end

function This:onNobilityAchieveChange()
    self:initAchieveList()
    self:refreshAchieve()
end