--NodeUI.lua
local This, super = class(..., "cc.Node")
local Timer = require("Timer")
local EventDispatcher = require("EventDispatcher")

function This:create(...)
    if self.fileName then
        local node = super.createFromCSFile(self, self.fileName, self._MNAME)
        node:init(...)
        return node
    else
        return super.create(self, ...)
    end
    return node
end

function This:onEnter()
    super.onEnter(self)
    self:initListener()
    self:initTouchEvent()
    if self.m_tabButtons then
        for name,button in pairs(self.m_tabButtons) do
            if not button.m_touch then
                button:setTouch({
                    onEnded = function ()
                        self:selectTabButton(name)
                    end
                })
            end
        end
    end
end

function This:onExit()
    super.onExit(self)
    EventDispatcher:removeAllListener(self)
end

function This:init(...)
    self:setName(self._NAME)
    self.m_timer = false
    self.m_tabButtons = false
    self.m_selectTabButton = false
    self.m_isRegisterEvent = false
    self:initData(...)
	self:initNode(...)
end

function This:initNode()
    
end

function This:initListener()
    
end

function This:initTouchEvent()
    
end

function This:addListener(key, func)
    EventDispatcher:addListener(self, key, func)
end

function This:initData()
    
end

function This:update(deltaTime)
    if self.m_timer then
        self.m_timer:update(deltaTime)
    end
end

function This:setTimer(delayTime, listener, key, triggerTotal)
    if not self.m_timer then
        self.m_timer = Timer:create(self)
        self:scheduleUpdate()
    end
    self.m_timer:setTimer(delayTime, listener, key, triggerTotal)
end

function This:removeAllTimer()
    if self.m_timer then
        self.m_timer:removeAllTimer()
    end 
end

function This:createTabNode(name)
    return require(name):create(self:getTabNodeData(name))
end

function This:addTabNode(name)
    if not self.m_tabNodes then
        self.m_tabNodes = {}
    end
    if self.m_tabNodes[name] then
        return
    end
    
    --local node = require(name):create(self:getTabNodeData(name))
	local node = self:createTabNode(name)
    node.m_owner = self
    self:addChild(node)
    self.m_tabNodes[name] = node
    self:onAddTabNode(node, name)
    return node
end

function This:getTabNodeData(name)
    
end

function This:onAddTabNode(node, name)
    
end

function This:onTabNodeSwitchIn(node, name)
    
end

function This:onTabNodeSwitchOut(node, name)
    
end

function This:canOpenTabNode(name)
    return true
end

function This:openTabNode(name)
    local result, errorCode = self:canOpenTabNode(name)
    if not result then
        if errorCode then
            g_scene:handleErrorCode(errorCode)
        end
        return false
    end
    if not self.m_tabNodes or not self.m_tabNodes[name] then
        self:addTabNode(name)
    end
    if self.m_selectTabName then
        local lastNode = self.m_tabNodes[self.m_selectTabName]
        if lastNode then
            self:onTabNodeSwitchOut(lastNode, self.m_selectTabName)
        end
    end
    self.m_selectTabName = name
    local curTabNode = self.m_tabNodes[name]
    if curTabNode then
        self:onTabNodeSwitchIn(curTabNode, name)
    end
    for k,tabNode in pairs(self.m_tabNodes) do
        tabNode:setVisible(k == name)
    end
    return self.m_tabNodes[name]
end

function This:addTabButton(name, button, onSelected)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end
    if self.m_tabButtons[name] then
        return
    end
    button.m_touch = false
    self.m_tabButtons[name] = button
    button.onSelected = onSelected
end

function This:selectTabButton(name)
    if self.m_tabButtons then
        if self.m_selectTabButton == name then
            return
        end
        if self.m_tabButtons[name].onSelected() ~= false then
            self:lightTabButton(name)
        end
    end
end

function This:lightTabButton(name)
    self.m_selectTabButton = name
    if self.m_tabButtons then
        for k,node in pairs(self.m_tabButtons) do
            if k == name then
                node.light:setVisible(true)
                node.normal:setVisible(false)
            else
                node.light:setVisible(false)
                node.normal:setVisible(true)
            end
        end
    end
end