-- @Author: zheng
-- @Date:   2018-09-21 14:30:03
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-10-21 17:46:34
-- @File Name: GameOfficialPosition.lua

local OfficialPositionConfig = require("Metadatas.OfficialPositionConfig")
local CountryNameConfig = require("Metadatas.CountryNameConfig")
local PrisonConfig = require("Metadatas.PrisonConfig")
local BubbleConfig = require("Metadatas.BubbleConfig")
local HitPrisonerConfig = require("Metadatas.HitPrisonerConfig")
local FamilyPowerConfig = require("Metadatas.FamilyPowerConfig")

local OfficialManager = require("Official.OfficialManager")

local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")

local This = class(...)

function This:create(id)
	local Morality = {}
	setmetatable(Morality,This)
	Morality:init(id)
	return Morality
end

function This:init(id)
	self.m_id = id
	self.m_config = OfficialPositionConfig[id]
end

function This:getId()
	return self.m_id
end

--获得官俸
function This:getWage(officialId)
	if not officialId then
		officialId = self:getId()
	end
	local temConfig = self:getOfficialConfig(officialId)
	local res = {}
	if temConfig then
		res = temConfig.wages
	end
	return res
end

-- --获取所有职位的官俸
-- function This:getAllWages()
-- 	return self.m_id ==1
-- end

function This:getName(officialId)
	if not officialId then
		officialId = self:getId()
	end
	local res =""
	local temConfig = self:getOfficialConfig(officialId)
	if temConfig then
		res = temConfig.name
	end
	return res
end

--获取官职的文字描述
function This:getOfficialDesc(officialId)
	if not officialId then
		officialId = self:getId()
	end
	local res =""
	local temConfig = self:getOfficialConfig(officialId)
	if temConfig then
		res = temConfig.desc
	end
	return res
end

--禅让花费
function This:getDemiseCost(time)
	local res =0
	if OfficialManager:myIsEmperor() then
		local costs = self.m_config.demise_cost
		local resetCount = #costs
		if time>0 and time<=resetCount then
			local temResetTime = time-1
			local config = costs[resetCount-temResetTime]
			if config then
				res = config
			end
		end
	end
	return res
end

--修改国号花费
function This:getCouuntryCost()
	local countryInfor = OfficialManager:getCountryInfor()
	local resetTimes = countryInfor.contry_code_info.times
	local resetCount = #CountryNameConfig
	if resetTimes>0 and resetTimes<=resetCount then
		local temResetTime = resetTimes-1
		local config = CountryNameConfig[resetCount-temResetTime]
		if config then
			return config.cost,resetTimes
		end
	end
	return 0,resetTimes
end

--天子赏赐
function This:getEmperorAward()
	local res ={}
	if OfficialManager:myIsEmperor() then
		local reward = self.m_config.emperor_award
		for _,itemNode in ipairs(reward) do
			table.insert(res,itemNode[1])
		end
	end
	return res
end

--官职描述
function This:getPowerDesc(officialId)
	if not officialId then
		officialId = self:getId()
	end
	local temConfig = self:getOfficialConfig(officialId)
	local res = ""
	if temConfig then
		res = temConfig.profit_desc
	end
	return res
end

--犯人描述
function This:getPrisonDesc(prisonId)
	local temConfig = self:getPrisonConfig(prisonId)
	local res = ""
	if temConfig then
		res = temConfig.debuff_desc
	end
	return res
end

-- --官职属性加成
-- function This:getPowerAttr(officialId)
	-- if not officialId then
	-- 	officialId = self:getId()
	-- end
	-- local temConfig = self:getOfficialConfig(officialId)
	-- local res = {}
	-- if temConfig then
-- 		-- local profit = self:convertData(temConfig.profit)
-- 		local arg = self:convertData(temConfig.arg)
-- 		-- self:tableAdd(profit, res)
-- 		self:tableAdd(arg, res)
-- 	end
-- 	return res
-- end

-- --自己的罪名属性加成
-- function This:getPrisonAttr()
-- 	local res = {}
-- 	local prison = OfficialManager:getPrisonWithPlayerId()
-- 	if prison.id~=0 then
-- 		local prisonId = prison.id
-- 		local prisonConfig = self:getPrisonConfig(prisonId)
-- 		return self:convertData(prisonConfig.debuff)
-- 	end
-- 	return res
-- end

function This:getHitCost()
	local time = OfficialManager:getHitCount()
	time  = time + 1
	local freeTime = 0
	local res =0
	if time then
		local temCost = nil
		for _,costItem in ipairs(HitPrisonerConfig) do
			local time1 = costItem.times[2]
			local cost = costItem.price
			if costItem.id ==1 then
				freeTime = time1
			end
			if time<= time1 then
				temCost = cost
				break
			end
		end
		if temCost then
			res = temCost
		else
			return res,true,freeTime
		end
	end
	return res,false,freeTime
end

function This:getBubbleDesc(id)
	local item1 = BubbleConfig[id]
	if item1 then
		return item1.desc
	end
	return ""
end

--收监 talk
function This:getPrisonTalks(id)
	local item1 = self:getPrisonConfig(id)
	if item1 then
		return item1.talk
	end
	return {}
end

--作揖,朝拜 talk
function This:getWorshipTalks()
	return self:getWorshipTalkWithOfficial()
end

function This:getWorshipTalkWithOfficial(officialId)
	local temOfficialId = officialId or OfficialType.kType1
	local item1 = self:getOfficialConfig(temOfficialId)
	if item1 then
		return item1.worship_bubble
	end
	if temOfficialId == OfficialType.kType1 then
		return {{},{}}
	end
	return {{}}
end


-- 获取随机的气泡
function This:getRandTalk(srcTable)
	if not srcTable or #srcTable ==0 then
		return "",0,false
	end
	local max = #srcTable
	local rand = math.random(1,max)
	local bubbleId = srcTable[rand]
	local desc = self:getBubbleDesc(bubbleId)
	return desc,bubbleId,true
end

--获得军团职位的名字
function This:getFamilyTitel(id)
	local item1 = FamilyPowerConfig[id]
	if item1 then
		return item1.title
	end
	return ""
end

function This:getPrisonConfig(id)
	local item1 = PrisonConfig[id]
	if item1 then
		return item1
	end
	return nil
end

function This:getOfficialConfig(id)
	local item1 = OfficialPositionConfig[id]
	if item1 then
		return item1
	end
	return nil
end

--获取所有的官职id
function This:getOfficialIds()
	local res ={}
	for officialId,configItem in ipairs(OfficialPositionConfig) do
		res[configItem.sort]= officialId
	end
	return res
end

function This:getOfficialCount()
	return #OfficialPositionConfig
end

--两个表相加
function This:tableAdd( table1,res )
	for k,v in pairs(table1) do
		if res[k] then
			res[k] = res[k] + v
		else
			res[k] = v
		end
	end 
end

-- table转换
function This:convertData( tableData )
	return GameUtil:convertData( tableData )
end

function This:convertDataToStr(srcTable, isSpace)
	local function getKey(data)
		local res = {}
		for k,v in pairs(data) do
			table.insert(res,k)
		end
		return res
	end
	local tableKey =getKey(srcTable)
	table.sort(tableKey,function(a,b)
		return a < b
	end)
	if isSpace then
		local str = ""
		for i,v in ipairs(tableKey) do
			local name,desc = self:getAttrStr(v, srcTable[v])
			str = str .. name .. desc.."\n"
		end
		return str
	else
		local str1 = ""
		local str2 = ""
		for i,v in ipairs(tableKey) do
			local name,desc = self:getAttrStr(v, srcTable[v])
			str2 = str2 .. desc.."\n"
			str1 = str1 .. name .."\n"
		end
		return {str1,str2},#tableKey
	end
end

function This:getAttrStr( key,value )
	local name = AttrName[key]
	local attr = "+%d"
	if AttrInfo[key] and AttrInfo[key].valueType == ValueType.kPercent then
		value = value * 100
		attr = "+%d%%"
	end
	return name,string.format(attr, value)
end

function This:refreshOfficialPawerAttr(itemsNode,officialId,count)
	if not count then
		count = 3
	end
	if officialId ==0 then
		self:refreshItemNode(itemsNode,{},count)
	else
		local rewards = self:getWage(officialId)
		self:refreshItemNode(itemsNode,rewards,count)
	end
end

function This:refreshItemNode( node,scrTable ,itemCount)
    local count = 1
    for i=1,itemCount do
        local item = node["item"..i]
        item:setVisible(false)
    end
    for kIndex,itemConfig in pairs(scrTable) do
        local item = node["item"..count]
        if type(itemConfig) == "number" then
        	item:setVisible(false)
        else
        	item:setVisible(true)
        	self:creatItemNode(item,itemConfig[1],itemConfig[2],count)
        end
        count = count+1
    end
end

function This:creatItemNode(parent,key,needNum,index)
    local name = "ItemNode"
    local node = parent:getChildByName(name)
    if not node then
        node = ItemNode:create(key, 1)
        node:setAnchorPoint(0.5, 0.5)
        parent:addChild(node)
        node:setName(name)
    end
    node:initItem(key,needNum) 
end

--官职特权
function This:refreshOfficialRewards(scrollViewNode,width,height,officialId)
	local strs = self:getPowerDesc(officialId)
	local GameUtil = require("Util.GameUtil")
	GameUtil:resetScoolSize3(scrollViewNode,width,height,strs)
end

function This:refreshPrisonRewards(scrollViewNode,width,height,prisonId)
	local strs = self:getPrisonDesc(prisonId)
	local GameUtil = require("Util.GameUtil")
	GameUtil:resetScoolSize3(scrollViewNode,width,height,strs)
end