

local This, super = class(..., "Dialog")
local ThroneManager = require("Throne.ThroneManager")
local RescuePalaceManager = require("Palace.RescuePalaceManager")
local CompetitionManager = require("Competition.CompetitionManager")
local PalaceTab = {
	kThrone = "tep1",
	kRescue = "tep2",
	kServerCompetition = "tep3",
}

function This:initData(info)
	self.m_initSelectTab = PalaceTab.kThrone

	if info then
		self.m_initSelectTab = info.tab
	end
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingPalace.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    for key, val in pairs(PalaceTab) do
		self:addTabButton(val, self.m_csNode[val], function ( ... )
			return self:onTabButtonSelect(val)
		end)
    end
    self:refreshLock()
    CompetitionManager:onHonorInfoRequest()
end

function This:initTouchEvent()
	if not self.m_inited then
		self.m_inited = true

		self.m_csNode.close:setTouchEnded(function()
        	self:close()
    	end)
	end
end

function This:onEnter()
    super.onEnter(self)
    self:selectTabButton(self.m_initSelectTab)
    RescuePalaceManager:addObserver(self)
end

function This:onExit()
	RescuePalaceManager:removeObserver(self)
    super.onExit(self)
end

function This:onTabButtonSelect(name)
	local isLock, tips = self:isLock(name)
	if isLock then
		g_scene:showTips(tips)
		return false
	end

	self:openTabNode(name)
	return true
end

function This:createTabNode(name)
	if name == PalaceTab.kThrone then
		return require("Throne.ThronePanel"):create()
	elseif name == PalaceTab.kRescue then
		return require("Palace.RescuePalacePanel"):create()
	elseif name == PalaceTab.kServerCompetition then
		return require("Competition.HonorTab"):create()
	end
end

function This:isLock(name)
	local isOpen = true
	local tips = ""

	if name == PalaceTab.kThrone then
		isOpen = ThroneManager:checkOpen()
		tips = ThroneManager:getOpenTips()
	elseif name == PalaceTab.kRescue then
		isOpen = RescuePalaceManager:checkOpen()
		tips = RescuePalaceManager:getOpenTips()		
	end

	return (not isOpen), tips
end

function This:refreshLock()
    for key, name in pairs(PalaceTab) do
    	local isLock = self:isLock(name)
    	local suoNode = self.m_csNode[name].suo
    	if suoNode then
    		suoNode:setVisible(isLock)
    	end
    end	
end

function This:onDayPassNotify()
	self:refreshLock()
end