
local This , super = class(...,"NodeUI")
local ItemNode = require("UI.ItemNode")
local RescuePalaceManager = require("Palace.RescuePalaceManager")


function This:initData(data)
    self.m_data = data
    self.m_itemList = {}
end

function This:initNode(data)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingPalaceBoxItem.csb")
    self:addChild(self.m_csNode)

    local scrollView = self.m_csNode.scrollview
    scrollView:setDirection("horizontal")
    scrollView:setSwallowTouch(false)

    local rewardNode = cc.Node:create()
    rewardNode:setContentSize(1, 78)
    rewardNode:setAnchorPoint(0.5, 0)
    scrollView:pushBackCustomItem(rewardNode, 1, 78, 0)
    self.m_rewardNode = rewardNode

    local w, h = self.m_csNode.beijing.bg:getContentSize()
    self.m_csNode:setPosition(w/2, h/2)
    self:setContentSize(w, h)
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        self.m_csNode.button:setTouch({
            isDynamic = true, 
            isContainer  = true,
            onEnded = function ()
                if self.m_data.state == 1 then
                    RescuePalaceManager:onRescuePalaceRewardRequest(self.m_data.rankType, self.m_data.id)
                end
            end,
        })
    end    
end

function This:onEnter()
    super.onEnter(self)
    self:refresh()
end

function This:setModel(data)
    self.m_data = data
    self:refresh()
end

function This:refresh()
    local data = self.m_data
    local csNode = self.m_csNode

    csNode.numbercishu:setString( string.format(Serialize[62188], data.config.count) )

    csNode.yilingqu:setVisible(data.state == 2)
    csNode.button:setVisible(data.state < 2)
    
    if data.state == 0 then
        csNode.button:setGrayShader()
    else
        csNode.button:clearShader()
    end

    --奖励
    for i, info in ipairs(data.config.reward) do
        if self.m_itemList[i] then
            self.m_itemList[i]:setVisible(true)
            self.m_itemList[i]:setModel({id = info[1], num = info[2]})
        else
            local item = ItemNode:create(info[1], info[2])
            item:setAnchorPoint(0, 0.5)
            item:setScale(0.8)
            self.m_rewardNode:addChild(item)
            table.insert(self.m_itemList, item)        
        end
    end    

    if #self.m_itemList > #data.config.reward then
        for i = #data.reward+1, #self.m_itemList do
            self.m_itemList[i]:setVisible(false)
        end
    end

    self:updateRewardNodeSize()
end

function This:updateRewardNodeSize()
    local scrollView = self.m_csNode.scrollview
    local minWidth = scrollView:getContentWidth()
    local w = 0
    local h = self.m_rewardNode:getContentHeight()

    for i, node in ipairs(self.m_itemList) do
        if node:isVisible() then
            local box = node:getBoundingBox()
            node:setPosition(w, h/2)
            w = w + box.width + 5
        end
    end

    if w < minWidth then
        w = minWidth
    end
    self.m_rewardNode:setContentSize(w, h)

    self.m_rewardNode.itemWidth = w
    self.m_rewardNode.itemHeight = h
    scrollView.m_itemWidth = w
    scrollView.m_itemHeight = h
    scrollView:refreshPosition()
end

