

local This, super = class(...,"Dialog")
local RescuePalaceBoxItemView = require("Palace.RescuePalaceBoxItemView")
local RankPageListLayer = require("Rank.RankPageListLayer")
local RescuePalaceManager = require("Palace.RescuePalaceManager")


function This:initData(itemData)
	self.m_itemData = itemData
	self.m_boxList = {}

	local boxs = itemData:getBoxs()
	for k, v in pairs(boxs) do
		table.insert(self.m_boxList, v)
	end

	table.sort(self.m_boxList, function (a, b)
        local aCanGet = (a.state == 1) and 1 or 0
        local bCanGet = (b.state == 1) and 1 or 0

        if aCanGet ~= bCanGet then
            return aCanGet > bCanGet
        else
            if a.state ~= b.state then
                return a.state < b.state
            end
            return a.id < b.id
        end
	end)
end

function This:initNode(data)
    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingPalaceBox.csb")
    self:addChild(csNode)
    csNode:setToCenter()
    self.m_csNode = csNode

    csNode.title:setString( self.m_itemData:getGiftBagName() )

   	local scrollView = self.m_csNode.scrollView
	scrollView:setDirection("vertical")

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 588,
        itemHeight = 95,
        spaceX = 0,
        spaceY = 3,
        scrollView = scrollView,
		dataCounter = function ()	 
	        return #self.m_boxList
        end,
        viewCreator =  function(index, node) 
        	local data = self.m_boxList[index]
        	if data then     
			    if not node then
				    return RescuePalaceBoxItemView:create(data, index)
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)

    self:refresh()
end

function This:onEnter()
    super.onEnter(self)
    RescuePalaceManager:addObserver(self)
end

function This:onExit()
    RescuePalaceManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
	if not self.m_inited then
        self.m_inited = true

        self.m_csNode.closebg:setTouchEnded(function()
        	self:close()
        end)
    end
end

function This:refresh()
    self.m_listLayer:setDataList()
end

function This:onRescuePalaceRewardReply(data)
    self:initData(data)
    self:refresh()
end