

local This, super = class(...,"Base.BaseModel")
local CompetitionAreaConfig = require("Metadatas.CompetitionAreaConfig")
local CompetitionAreaBoxConfig = require("Metadatas.CompetitionAreaBoxConfig")
local CompetitionMailConfig = require("Metadatas.CompetitionMailConfig")
local LoginManager = require("Login.LoginManager")
local ServerMixConfig = require("Metadatas.ServerMixConfig")


function This:init(data)
	super.init(self)

	local role = data.top_role

	self.m_rankType = data.type

	self.m_roleId = role.id
	self.m_roleName = role.name
	self.m_rankNum = role.rank
	self.m_rankVal = role.value
	self.m_groupName = role.group_name
	self.m_serverId = role.server_id
	self.m_roleIcon = role.icon
	self.m_likes = data.support_value
	self.m_remainLikes = data.left_support_times

	self.m_boxs = {}
	self.m_gotBoxs = {}
end

function This:setGotRewardList(rewardList)
	rewardList = rewardList or {}
	self.m_gotBoxs = {}

	for i, id in ipairs(rewardList) do
		self.m_gotBoxs[id] = true
	end

	self:initBoxs()
end

function This:setGotRewardId(rewardId)
	self.m_gotBoxs[rewardId] = true
	self:initBoxs()
end

function This:initBoxs()
	self.m_boxs = {}

	for id, config in pairs(CompetitionAreaBoxConfig) do
		if config.type_id == self.m_rankType then
			local box = {
				id = id,
				rankType = self.m_rankType,
				config = config,
				state = self:getBoxState(id)
			}
			self.m_boxs[id] = box
		end
	end
end

-- -1不是同一服务器没有资格, 0不可领取，1可领取，2已领取
function This:getBoxState(id)
	local config = CompetitionAreaBoxConfig[id]

	if not self:isMyServer() then
		return -1
	end

	if self.m_gotBoxs[id] then
		return 2
	end

	if self.m_likes >= config.count then
		return 1
	end
	
	return 0
end

function This:isMyServer()
	local server = LoginManager:getLoginServer()
	local id = self:getMyMergeServerId()
	return self.m_serverId == id
end

function This:isLingQuEnabled()
	local isCan = false

	for k, v in pairs(self.m_boxs) do
		if v.state == 1 then
			isCan = true
			break
		end
	end
	
	return isCan
end

function This:isLingQuFinish()
	local isFinish = false
	local curNum = 0
	local allNum = 0

	for k, v in pairs(self.m_boxs) do
		allNum = allNum + 1
		if v.state == 2 then
			curNum = curNum + 1
		end
	end

	if curNum >= allNum then
		isFinish = true
	end

	return isFinish
end

function This:getBoxs()
	return self.m_boxs
end

function This:getRoleIcon()
	return self.m_roleIcon
end

function This:getRoleName()
	return self.m_roleName
end

function This:getServerIndexInGroup()
	local idx = self.m_serverId
	local server = LoginManager:getServer(self.m_serverId)
	if server then
		idx = server:getIndexInGroup()
	end
	return idx
end

function This:getServerName()
	local server = LoginManager:getServer(self.m_serverId)
	if server then
		return server:getName()
	end
    return ""
end

function This:setLikes(likes)
	self.m_likes = likes
end

function This:getLikes()
	return self.m_likes
end

function This:setRemainLikes(remain)
	if remain < 0 then
		remain = 0
	end
	self.m_remainLikes = remain
end

function This:getRemainLikes()
	return self.m_remainLikes
end

function This:getGiftBagName()
	local config = CompetitionMailConfig[self.m_rankType]
	if config then
		return string.format(Serialize[62191], config.title)
	end
	return Serialize[62192]
end

function This:getMyMergeServerId()
	local server = LoginManager:getLoginServer()
	local id = server:getId()

	if ServerMixConfig[id] then
		return ServerMixConfig[id].target_server
	end

	return id
end