
-- 勤王殿

local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local CompetitionAreaConfig = require("Metadatas.CompetitionAreaConfig")
local RescuePalaceItemData = require("Palace.RescuePalaceItemData")
local PlayTypeConfig = require("Metadatas.PlayTypeConfig")
local EntrantManager = require("LegionEntrant.EntrantManager")
local GamePlayer = require("Player.GamePlayer")
local LoginManager = require("Login.LoginManager")
local GameUtil = require("Util.GameUtil")


function This:init()
	super.init(self)

	self.m_ranks = {}
	self.m_areas = nil
	self.m_myAreaId = 0

	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 12058, self.onRescuePalaceInfoReply)
	EventDispatcher:addListener(self, 12059, self.onRescuePalaceLikesReply)
	EventDispatcher:addListener(self, 12060, self.onRescuePalaceRewardReply)
	EventDispatcher:addListener(self,"day_passed_withoutnet", self.onDayPassNotify)
end

function This:initAreas()
	-- B开头: 1-1030, R开头: 1031以后
	local serverList = LoginManager:getServerListWithSort()
	local myServerId = self:getServerId()
	self.m_areas = {}
	self.m_myAreaId = 0

	for id, config in pairs(CompetitionAreaConfig) do
		local area = {}
		local minSid = config.serverid_min
		local maxSid = config.serverid_max

		area.id = id
		area.minServerId = minSid
		area.maxServerId = maxSid
		area.serverList = {}
		self.m_areas[id] = area

		for i, server in ipairs(serverList) do
			local serverId = server:getId()
			if serverId >= minSid and serverId <= maxSid then
				local t = {}
				t.id = serverId
				t.indexInGroup = server:getIndexInGroup()
				t.name = server:getName()
				t.tag = GameUtil:getServerTag(serverId)
				table.insert(area.serverList, t)
			end
		end

		if myServerId >= minSid and myServerId <= maxSid then
			self.m_myAreaId = id
		end
	end
end

function This:getCompetitionAreaInfo(areaId)
	if not self.m_areas then
		self:initAreas()
	end 

	if not areaId then
		areaId = self.m_myAreaId
	end

	return self.m_areas[areaId]
end

function This:getMyAreaId()
	if not self.m_areas then
		self:initAreas()
	end 

	return self.m_myAreaId
end

function This:getServerId()
	local server = LoginManager:getLoginServer()
	return server:getId()
end

function This:isRedDot()
	if self:checkOpen() then
		for rankType, data in pairs(self.m_ranks) do
			if data:isLingQuEnabled() then
				return true
			end
		end
	end
	return false
end

function This:getRank(id)
	return self.m_ranks[id]
end

-- 勤王殿信息
function This:onRescuePalaceInfoRequest()
	Socket:sendAndLock(12058, {})
end

function This:onRescuePalaceInfoReply(msg)
	print_tmp(msg)
    if msg.err_code == 0 then
    	self.m_ranks = {}
    	for i, v in ipairs(msg.rank_list) do
    		local data = RescuePalaceItemData:create(v)
    		self.m_ranks[v.type] = data    		

    		local rewardList = data:isMyServer() and msg.reward_list or {}
    		data:setGotRewardList(rewardList)
    	end
    	self:notifyObservers("onRescuePalaceInfoReply", self.m_ranks)
    	EventDispatcher:dispatcher("rescue_palace_refresh")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 点赞
function This:onRescuePalaceLikesRequest(rankType)
	Socket:sendAndLock(12059, {type = rankType})
end

function This:onRescuePalaceLikesReply(msg)
	print_tmp(msg)
	if msg.err_code == 0 then
		local data = self:getRank(msg.type)
		if data then
			data:setLikes(msg.support_value)
			data:setRemainLikes( data:getRemainLikes()-1 )
			data:initBoxs()
			self:notifyObservers("onRescuePalaceLikesReply", data)
			EventDispatcher:dispatcher("rescue_palace_refresh")
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 领奖
function This:onRescuePalaceRewardRequest(rankType, rewardId)
	Socket:sendAndLock(12060, {type = rankType, reward_id = rewardId})
end

function This:onRescuePalaceRewardReply(msg)
	print_tmp(msg)
	if msg.err_code == 0 then
		local data = self:getRank(msg.type)
		if data then
			data:setGotRewardId(msg.reward_id)
			self:notifyObservers("onRescuePalaceRewardReply", data)
			EventDispatcher:dispatcher("rescue_palace_refresh")

			local CompetitionAreaBoxConfig = require("Metadatas.CompetitionAreaBoxConfig")
			g_scene:showReward(CompetitionAreaBoxConfig[msg.reward_id].reward)
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:checkOpen()
	local openSvrTime = EntrantManager:getOpenServerZeroTime()
	local config =  PlayTypeConfig[GameMapType.kRescueMap]
	local curTime = GamePlayer:getServerTime()
	local openTime = SECONDS_PER_DAY * config.open_day + openSvrTime

	return curTime >= openTime
end

function This:getOpenTips()
	local config = PlayTypeConfig[GameMapType.kRescueMap]
	return string.format("开服第%d天开启", config.open_day)
end

function This:onDayPassNotify()
	self:notifyObservers("onDayPassNotify")
end