

local This , super = class(...,"NodeUI")
local RescuePalaceManager = require("Palace.RescuePalaceManager")
local GameUtil = require("Util.GameUtil")
local CompetitionAreaConfig = require("Metadatas.CompetitionAreaConfig")
local LoginManager = require("Login.LoginManager")


local RankTypeNode = {
	[RescueRankType.kTuLu] = {winer = "winer1", box = "box1"},
	[RescueRankType.kShaDi] = {winer = "winer2", box = "box2"},
	[RescueRankType.kLianZhan] = {winer = "winer3", box = "box3"},
}


function This:initData(info)
	RescuePalaceManager:onRescuePalaceInfoRequest()
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingPalaceRescue.csb")
    self:addChild(self.m_csNode)

    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        local csNode = self.m_csNode

        for rankType, val in pairs(RankTypeNode) do
        	-- 点赞
			csNode[val.winer].xin:setTouchEnded(function ()
				local itemData = RescuePalaceManager:getRank(rankType)
				if not itemData then
					return
				end

				if not itemData:isMyServer() then
					return
				end

				if itemData:getRemainLikes() <= 0 then
					g_scene:showTips(Serialize[62195])
					return
				end

				RescuePalaceManager:onRescuePalaceLikesRequest(rankType)
        	end)

			-- 盒子
			csNode[val.box].box:setTouchEnded(function ()
				local itemData = RescuePalaceManager:getRank(rankType)
				if not itemData then
					return
				end

				if not itemData:isMyServer() then
					return
				end

				g_scene:openDialog("Palace.RescuePalaceBoxPanel", itemData)
        	end)
		end

        -- 帮助
        csNode.wenhao:setTouchEnded(function()
            g_scene:openDialog("UI.HelpTips",{text=Serialize["rescue_palace_tips"], node = csNode.wenhao})
        end)
    end
end

function This:onEnter()
    super.onEnter(self)
    RescuePalaceManager:addObserver(self)
end

function This:onExit()
    RescuePalaceManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
	local csNode = self.m_csNode

	local area = RescuePalaceManager:getCompetitionAreaInfo()
	if not area then
		csNode.qufu:setString(Serialize[60685])
	else
		local serverList = area.serverList
		local minServer = serverList[1]
		local maxServer = serverList[#serverList]
		local str = ""

		if minServer.id == maxServer.id then
			str = string.format(Serialize[60686], minServer.tag, minServer.indexInGroup)
		else
			str = string.format(Serialize[60687], minServer.tag, minServer.indexInGroup, maxServer.tag, maxServer.indexInGroup)
		end

		csNode.qufu:setString(str)
	end

	for rankType, val in pairs(RankTypeNode) do
		local itemData = RescuePalaceManager:getRank(rankType)
		self:refreshWiner(csNode[val.winer], csNode[val.box], itemData)
	end
end

function This:refreshWiner(winerNode, boxNode, itemData)
	local isExist = true

	if not itemData then
		isExist = false
	end

	winerNode.icon:setVisible(isExist)
	winerNode.name:setVisible(isExist)
	winerNode.xin:setVisible(isExist)
	winerNode.qufu:setVisible(isExist)
	winerNode.noone:setVisible(not isExist)
	boxNode:setVisible(isExist)

	if not isExist then
		return
	end

	-- 形象-名字-区服
	GameUtil:setPlayerFeltNodeResource(winerNode.icon, itemData:getRoleIcon())
	winerNode.name:setString(itemData:getRoleName())
	winerNode.qufu:setString( string.format(Serialize[62196], itemData:getServerIndexInGroup(), itemData:getServerName()) )
	
	-- 不是本服的隐藏
	if not itemData:isMyServer() then
		winerNode.xin:setVisible(false)
		boxNode:setVisible(false)
		return
	end

	-- 点赞数
	boxNode.num:setString(itemData:getLikes())

	winerNode.xin.text:setVisible(false)
	if itemData:getRemainLikes() > 0 then
		winerNode.xin:clearShader()
		winerNode.xin.aixin:clearShader()
	else
		winerNode.xin:setGrayShader()
		winerNode.xin.aixin:setGrayShader()
	end

	-- 盒子状态
	local state = 0
	if itemData:isLingQuEnabled() then
		state = 1
	elseif itemData:isLingQuFinish() then
		state = 2
	end

	local boxPic = (state == 0) and "MenuScene/boxLock4.png" or "MenuScene/boxGet4.png"
	local box = boxNode.box
    box:setImage(boxPic)
    box:clearShader()

	local effectList = {"BoxLing","BoxLing2"}
    if state == 0 then
    	--不可领
    	GameUtil:hideNodeList(box, effectList)
    elseif state == 1 then
    	--可领
    	for _, name in ipairs(effectList) do
            if not box[name] then
                local eff = GameUtil:createEffect(name, true, box)
                eff:setScale(1.2)
                box:addChild(eff, -1)
                box[name] = eff
            end
            box[name]:setVisible(true)
        end
    else
    	--已领
    	GameUtil:hideNodeList(box, effectList)
    	box:setGrayShader()
    end

end

function This:onRescuePalaceInfoReply(data)
	self:refresh()
end

function This:onRescuePalaceLikesReply(data)
	self:refresh()	
end

function This:onRescuePalaceRewardReply(data)
	self:refresh()	
end

function This:onDayPassNotify()
	RescuePalaceManager:onRescuePalaceInfoRequest()
end