-- @Author: ZYY
-- @Date:   2018-09-12 16:56:46
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-14 15:13:04
-- @File Name: PeakContestArrayPanel.lua
-- @Desc: 布阵界面
local This, super = class(...,"Dialog")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local PeakContestManager = require("PeakContest.PeakContestManager")
local PeakContestUtil = require("PeakContest.PeakContestUtil") 
local HeroManager = require("Hero.HeroManager")
local UpdateLayer = require("UI.UpdateLayer")

function This:initNode(info)
	self.m_troopType = info.type
	local troopList = PeakContestManager:getMyTroop(info.type)
	self.m_troopList = {}

	if troopList then 
		for i , v in ipairs(troopList) do 
			self.m_troopList[i] = v.card_list
		end
	end
	self.m_cardTroopList = {}
	for i , v in ipairs(self.m_troopList) do 
		for j , uid in ipairs(v) do 
			self.m_cardTroopList[uid] = i
		end
	end

    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PeakContestArray.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
	self.m_heroLayer = UpdateLayer:create({
		itemWidth = 95,
		itemHeight = 85,
		column = 6,
		spaceY = 0,
		spaceX = 0,
		scrollView = self.m_csNode.ScrollView,
	})
	self.m_heroLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_heroLayer:getContentSize()
	self.m_csNode.ScrollView:pushBackCustomItem(self.m_heroLayer, w,h,0)
	self.m_heroLayer:setCascadeOpacityEnabled(true)
	self.m_heroList = PeakContestManager:getHeroList()
	self.m_selectCountry = 5
	self.m_selectTroop = 1
	self.m_troopListItem = {}
	self:refresh()
end

function This:refresh()
	self.m_csNode.save:setVisible(self.m_selectTroop == 3) 
	self.m_csNode.next:setVisible(self.m_selectTroop ~= 3) 
	self:updateTroopList()
	self:updateHeroList()
	self:updateTroopNum()
end

function This:initTouchEvent()
	for i = 1 , 5 do 
		self.m_csNode["botton"..i]:setTouchEnded(function()
			self.m_selectCountry = i
			self:refresh()
		end)
	end

	for i = 1 , 3 do 
		self.m_csNode["troop"..i]:setTouchEnded(function()
			self.m_selectTroop = i
			self:refresh()
		end)
	end

	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.next:setTouchEnded(function()
		if self.m_selectTroop < 3 then 
			self.m_selectTroop = self.m_selectTroop + 1 
			self:refresh()
		end
	end)

	self.m_csNode.save:setTouchEnded(function()
		local function insufficientTroop()
			local t = 0
			for i = 1 , 3 do 
				if not self.m_troopList[i] then 
					return -1
				end
				for j = 1 , 6 do 
					if not self.m_troopList[i][j] then 
						if j == 1 then 
							return -1 
						else
							if t == 0 then
								t = i
							end
						end
					end
				end
			end		
			return t	
		end		
		local troop = insufficientTroop() 
		if troop == -1 then
			g_scene:showTips(Serialize[60618])
		elseif troop ~= 0 then  
			local panel = g_scene:openDialog("UI.ConfirmPanel", {text = string.format(Serialize[60619],troop)})		
			panel:setOkHandler(function()
				cc.Audio:playEffect("TopArenaSave.mp3")
				PeakContestManager:reqSaveTroopData(self.m_troopType,self.m_troopList)
				self:close()
			end)	
		else
			cc.Audio:playEffect("TopArenaSave.mp3")
			PeakContestManager:reqSaveTroopData(self.m_troopType,self.m_troopList)
			self:close()
		end		
	end)
end

function This:addTroopList(hero)
	local heroUid = hero:getUid()
	local troop = self:getCardTroop(heroUid)
	if troop ~= 0 then
		g_scene:showTips(Serialize[62199])
		return 
	end 
	cc.Audio:playEffect("PickHero.mp3")
	self.m_troopList[self.m_selectTroop] = self.m_troopList[self.m_selectTroop] or {}
	table.insert(self.m_troopList[self.m_selectTroop],heroUid)
	self.m_cardTroopList[heroUid] = self.m_selectTroop
	self:refresh()
end

function This:delTroopList(hero)
	if not hero then 
		return 
	end
	local heroUid = hero:getUid()
	self.m_troopList[self.m_selectTroop] = self.m_troopList[self.m_selectTroop] or {}
	for i, uid in ipairs(self.m_troopList[self.m_selectTroop]) do
	    if heroUid == uid then
	        table.remove(self.m_troopList[self.m_selectTroop], i)
	        break
	    end
	end
	self.m_cardTroopList[heroUid] = nil
	self:refresh()
end


function This:updateTroopList()

	for i = 1 , 3 do 
		self.m_csNode["troop"..i].light:setVisible( self.m_selectTroop == i )
	end

	for i , v in ipairs(self.m_troopListItem) do 
		v:setVisible(false)
	end
	local heroList = self.m_troopList[self.m_selectTroop] or {}
	local width = 80
	for i = 1, 6 do  
		local hero = HeroManager:getHeroByUid(heroList[i])
		local awake = false
		if hero then
			awake = hero:isAwakeFull()
		end
		local item = self.m_troopListItem[i] 
		if not item then 
			item = PeakContestTeamItemView:create(hero,i,nil,awake)
			item:setSelectHandler(function(hero)
				self:delTroopList(hero)
			end)
			item:setPosition(width * (i - 3.5), 0 )
			item:setScale(0.78)
			self.m_csNode.team:addChild(item)
		else
			item:setModel(hero,nil,nil,awake)
		end
		item:setVisible(true)
		self.m_troopListItem[i] = item
		item:showX(hero ~= nil)
	end

	local power = 0 
	for i , uid in ipairs(heroList) do 
		local hero = HeroManager:getHeroByUid(uid)
		power = power + hero:getFightPower()
	end
	self.m_csNode.power:setString(power)
end

function This:updateHeroList()
	for i = 1 , 5 do 
		self.m_csNode["botton"..i].light:setVisible( self.m_selectCountry == i )
	end

	local heroList = {}
	if self.m_selectCountry == 5 then 
		heroList = self.m_heroList
	else
		for i , hero in ipairs(self.m_heroList) do 
			local country = hero:getCountry()
			if country == self.m_selectCountry then 
				table.insert(heroList,hero)
			end
		end
	end	
	self.m_heroLayer:setDataList(heroList, function(hero, index)
		local awake = false
		if hero then
			awake = hero:isAwakeFull()
		end
		local item = PeakContestTeamItemView:create(hero,index,self,awake)
		item:setSelectHandler(function(hero)
			local troopList = self.m_troopList[self.m_selectTroop] or {}
			if #troopList < 6 then
				self:addTroopList(hero)
			else
				g_scene:showTips(Serialize[61625])
			end
		end)
		item:setScale(0.78)
		return item
	end)
end

function This:updateTroopNum()
	for i = 1 , 3 do
		local num = self.m_troopList[i] and #self.m_troopList[i] or 0
		self.m_csNode["troop"..i].num:setString(string.format("(%d/%d)",num,6))
	end
end

function This:getCardTroop(uid)
	return self.m_cardTroopList[uid] or 0
end

function This:onTouchMask()

end
