-- @Author: ZYY
-- @Date:   2018-09-19 15:11:29
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-21 17:18:24
-- @File Name: PeakContestBattleInfoListView.lua
-- @Desc: 
local This , super = class(...,"NodeUI")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local ReportManager = require("Report.ReportManager")
local ChatManager = require("Chat.ChatManager")
local GameUtil = require("Util.GameUtil")
local Hero = require("Hero.Hero")
local GodBeastManager = require("GodBeast.GodBeastManager")

This.fileName = "MenuScene/NodeUI/PeakContestBattleInfoList.csb"
local LastShareReportTime = 0
local order = {Serialize[60012],Serialize[60013],Serialize[60014],Serialize[60015],Serialize[60016],Serialize[60097],Serialize[60018],Serialize[60019],Serialize[60020],Serialize[60021]}

function This:initNode(...)
	self.m_myTroopItemList = {}
	self.m_roleTroopItemList = {}
	self:setData(...)
end

function This:refresh()
	self:updateMyTroop()
	self:updateRoleTroop()
end

function This:updateMyTroop()
	local data = self.data.attack or {}
	self.levelNode1.level:setString(data.level or 0)
	GameUtil:setPlayerIconNodeImage(self.pic1.icon, data.icon)
	self.name1:setString(data.name or "")
	self.troops1.num:setString(string.format("%d/%d",data.force_num_result,data.force_num_init))
	self.troops1.pro:setPercent(data.force_num_result / data.force_num_init * 100)
	if self.data.state == 1 then 
		self.result1:setImage("MenuScene/shengli1.png")
	else
		self.result1:setImage("MenuScene/shibai1.png")
	end
	local width = 65

	local temCardList = GodBeastManager:removeListBeastId(data.card_list)
	for i = 1 , 6 do 
		local card = temCardList[i]
		local hero
		if card then  
			hero = Hero:create({
				heroId = card.card_type or 1000,
				starLv = card.star_level or 0
			})
		end
		local item = self.m_myTroopItemList[i]
		-- local awake = card.awake
		if not item then 
			item = PeakContestTeamItemView:create(hero,nil,nil,awake)
			item:setPosition(width * (i - 3.5), 0)
			item:setScale(0.6)
			self.troop1:addChild(item)
		else
			item:setModel(hero,nil,nil,awake)
		end		
		self.m_myTroopItemList[i] = item
	end
end

function This:updateRoleTroop()
	local data = self.data.defence or {}
	self.zhan:setString(string.format(Serialize[60621],order[self.m_index]))
	self.levelNode2.level:setString(data.level or 0)
	GameUtil:setPlayerIconNodeImage(self.pic2.icon, data.icon)
	self.name2:setString(data.name or "")
	self.troops2.num:setString(string.format("%d/%d",data.force_num_result,data.force_num_init))
	self.troops2.pro:setPercent(data.force_num_result / data.force_num_init * 100)
	if self.data.state ~= 1 then 
		self.result2:setImage("MenuScene/shengli1.png")
	else
		self.result2:setImage("MenuScene/shibai1.png")
	end
	local width = 65
	local temCardList = GodBeastManager:removeListBeastId(data.card_list)
	for i = 1 , 6 do 
		local card = temCardList[i]
		local hero
		if card then  
			hero = Hero:create({
				heroId = card.card_type or 1000,
				starLv = card.star_level or 0
			})
		end
		-- local awake = card.awake
		local item = self.m_roleTroopItemList[i]
		if not item then 
			item = PeakContestTeamItemView:create(hero,nil,nil,awake)
			item:setPosition(width * (i - 3.5), 0)
			item:setScale(0.6)
			self.troop2:addChild(item)
		else
			if hero then
				item:setModel(hero,nil,nil,awake)
			end
		end		
		self.m_roleTroopItemList[i] = item
	end
end

function This:onEnter()
	super.onEnter(self)
	ReportManager:addObserver(self)
	self:refresh()
end

function This:onExit()
	ReportManager:removeObserver(self)
	super.onExit(self)
end

function This:initTouchEvent()
	self.videotape:setTouchEnded(function()
		ReportManager:onGetReportDetailRequest(self.data.id)
		ReportManager:setIsRead(self.data.id,true)
	end)

	self.share:setTouchEnded(function()
		local ret , msg = ChatManager:isCanChatInWorld()
		if not ret then
			g_scene:showTips(msg)
			return 
		end
		if os.time() - LastShareReportTime >= 5 then
			local panel = g_scene:openDialog("UI.ConfirmPanel")
			panel:setString(Serialize[62201])
			panel:setOkHandler(function()
				ChatManager:onShareBattleReportRequest(self.data.id)
				LastShareReportTime = os.time()
				ReportManager:setIsRead(self.data.id,true)
			end)
		else
			g_scene:showtTips(string.format(Serialize[62202],os.time() - LastShareReportTime))
		end	
	end)
end

function This:onGetReportDetail(msg)
	cc.SceneManager:switchTo("Report.ReportLoadingScene",nil,msg)
end

function This:setData(data,index)
	self.data = data
	print('ZYY:PeakContestBattleInfoListView.lua [start:142] :data',data)
	self.m_index = index
end