-- @Author: Su
-- @Date:   2018-09-11 20:24:11
-- @Last Modified by: Su
-- @Last Modified time: 2018-11-29 16:44:29
-- @File Name: PeakContestChallengeBoxPanel.lua
-- @Desc: 

local This,super = class(...,"Dialog")
local PeakContestManager = require("PeakContest.PeakContestManager")
local PeakContestChallengeBoxItem = require("PeakContest.PeakContestChallengeBoxItem")
local DailyRewardConfig = require("Metadatas.DailyRewardConfig")
local WeeklyRewardConfig = require("Metadatas.WeeklyRewardConfig")
local GameUtil = require("Util.GameUtil")

local GameWarShopData = require("Map.GameWarShopData")

local BoxImage = 
{
	[1] = {
		"MenuScene/boxLock1.png",
		"MenuScene/boxGet1.png",
	},
	[2] = {
		"MenuScene/boxLock2.png",
		"MenuScene/boxGet2.png",
	},
	[3] = {
		"MenuScene/boxLock3.png",
		"MenuScene/boxGet3.png",
	},
	[4] = {
		"MenuScene/boxLock4.png",
		"MenuScene/boxGet4.png",
	}
}

function This:initNode()
    self.m_itemList = {}
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PeakContestChallengeBox.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self.m_boxNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PeakContestCommon.csb")
    self.m_boxNode:setToCenter()
    self.m_sliderX = self.m_boxNode.jindubg.jindutiao:getScaleX()
    self.m_csNode.box:addChild(self.m_boxNode)
    self.m_boxNode:setPosition(0,0)
    local m_scrollView = self.m_csNode.scrollView
    for _,date in pairs(DailyRewardConfig) do
        local state = PeakContestManager:getTaskState(date.id,PeakContestChallengeBoxRewardType.kDay)
        date.state = state
        local node = PeakContestChallengeBoxItem:create(date)
        table.insert( self.m_itemList, node)
    end
    table.sort( self.m_itemList, function (a, b)
        local state1, state2 = a:getState(), b:getState()
        if state1 ~= state2 then
            return state1 < state2
        end
        return a:getId() < b:getId()
    end )
    for i=1,self.m_maxDayListNum do
        local w,h = self.m_itemList[i].beijing.bg:getContentSize()
        m_scrollView:pushBackCustomItem(self.m_itemList[i],w,h)
    end
    self:refresh()
end

function This:initData()
    self.m_data = PeakContestManager:getMyData()
    self.m_maxDayListNum = #DailyRewardConfig
end

function This:refresh()
    local text = string.format(Serialize["peakContest_Box_1"],self.m_data.day_challenge_times)
    self.m_csNode.number:setString(text)
    local node = self.m_boxNode
    node.weekvitality:setString(self.m_data.week_challenge_times)
    for i=1,4 do
        self.m_boxNode["bg"..i].text:setString(self:getFrequencyById(i))
    end
    local max=PeakContestManager:getMaxfrequency()
    local slider = node.jindubg.jindutiao
    local w = (self.m_data.week_challenge_times/max)*self.m_sliderX
    slider:setScaleX(w)
    self:refreshBoxState()
end


function This:refreshBoxState()
    for i=1,4 do
        local state = PeakContestManager:getTaskState(i,PeakContestChallengeBoxRewardType.kWeek)
        local node,index = self.m_boxNode["boxLock"..i]
        if state == PeakContestChallengeBoxRewardState.kUnReceive then
            index = 1
        elseif state == PeakContestChallengeBoxRewardState.kHasReceive then
            index = 2
            node:setGrayShader()
        else
            index = 2
        end
        GameUtil:createBoxEffect(state==PeakContestChallengeBoxRewardState.kCanReceive,node,i)
        node:setImage(BoxImage[i][index])
    end
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.close:setTouchEnded(function ()
        self:close()
    end)
    csNode.help:setTouchEnded(function ()
        g_scene:openDialog("UI.HelpTips",{text = Serialize["peakContest_Box_2"],node=csNode.help})
    end)
    for i=1,4 do
        self.m_boxNode["boxLock"..i]:setTouch({
            onEnded = function ()
                g_scene:openDialog("PeakContest.PeakContestChallengeBoxReward",{key = i})
            end,
            isScale = false,
        })
    end
end

function This:onEnter()
    super.onEnter(self)
    PeakContestManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    PeakContestManager:removeObserver(self)
end

function This:updateBoxRewardState()
    self:initNode()
end

------------------------------------------------------
function This:getFrequencyById(id)
    local num
    for _,data in ipairs(WeeklyRewardConfig) do
        if id==data.id then
            num = data.frequency
        end
    end
    return num
end

-- -- 测试优化代码
-- function This:getGameWarShopData()
--     if not self.m_ratingData then
--         self.m_ratingData = GameWarShopData:create(0)
--     end
--     return self.m_ratingData
-- end
