-- @Author: ZYY
-- @Date:   2018-09-14 11:49:27
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-21 16:52:37
-- @File Name: PeakContestDefenseFightPanel.lua
-- @Desc: 
local This, super = class(...,"Dialog")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local PeakContestManager = require("PeakContest.PeakContestManager")
local PeakContestUtil = require("PeakContest.PeakContestUtil") 
local TopArenaRankingConfig = require("Metadatas.TopArenaRankingConfig")  
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")

function This:initNode(info)
	self.m_myData = info.myData
	self.m_roleData = info.roleData
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PeakContestDefenseFight.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
	self.m_myTroopItemList = {}
	self.m_roleTroopItemList = {}
	self.m_selectTroop = 0
	self.m_changeTroopList = {}
	self:updateJiantouAction()
	self.m_csNode.changeNode:setVisible(false)
	self:refresh()
end

function This:refresh()
	self:updateRoleTroop(1)
	self:updateMyTroop()
end

function This:updateRoleTroop(troop)
	for i = 1 , 3 do 
		self.m_csNode["troopBtn"..i].light:setVisible(troop == i)
	end

	local function getHide()
		for _ , v in ipairs(TopArenaRankingConfig) do 
			if self.m_roleData.rank_id >= v.top and self.m_roleData.rank_id <= v.lowest then 
				for _ ,t in ipairs(v.pocket_formation) do 
					if troop == t[1] then 
						return true
					end
				end
			end
		end
		return false
	end
	local hide = getHide()
	self.m_csNode.troopBtn.text:setString(troop .. Serialize[62219])
	local cardList = self.m_roleData["card_list".. troop]
	local width = 80
	for i = 1 , 6 do 
		local item = self.m_roleTroopItemList[i]
		local hero
		local awake
		if cardList[i] then 
			hero = Hero:create({
				heroId = cardList[i].card_type,
				starLv = cardList[i].star,
				level = cardList[i].level
				})
			awake = cardList[i].awake
		end
		if not item then 
			-- if hero then
				item = PeakContestTeamItemView:create(hero,nil,nil,awake)
				item:setPosition(width * (i - 3.5), 0)
				item:setScale(0.75)
				self.m_csNode["troop"]:addChild(item)
			-- end
		else
			item:setModel(hero,nil,nil,awake)
			item:setHide(hide)
		end
		self.m_roleTroopItemList[i] = item
	end
end

function This:updateMyTroop()
	local width = 80
	for i = 1 , 3 do 
		local num = 0
		self.m_myTroopItemList[i] = self.m_myTroopItemList[i] or {}
		for j = 1 , 6 do 	
			local item = self.m_myTroopItemList[i][j]
			local uid = 0 
			if self.m_myData.attack_lineup[i] and self.m_myData.attack_lineup[i]["card_list"] then 
				uid = self.m_myData.attack_lineup[i]["card_list"][j] or 0
			end
			if uid ~= 0 then 
				num = num + 1 
			end
			local hero = HeroManager:getHeroByUid(uid)
			local awake = false
			if hero then
				awake = hero:isAwakeFull()
			end
			if not item then 
				item = PeakContestTeamItemView:create(hero,nil,nil,awake)
				item:setPosition(width * (j - 3.5), 0)
				item:setScale(0.75)
				self.m_csNode["troop" .. i]:addChild(item)
			else
				item:setModel(hero,nil,nil,awake)
			end
			self.m_myTroopItemList[i][j] = item
		end
		self.m_csNode["troopBtn"..i].num:setString(string.format("(%d/%d)",num,6))
	end	
end

function This:updateTroopData(type,troopList)
	if type == TroopType.Attack then 
		for i = 1 ,3 do 
			if not self.m_myData.attack_lineup[i] then 
				self.m_myData.attack_lineup[i] = {}
			end
			self.m_myData.attack_lineup[i]["card_list"] = troopList[i]
		end
		print('ZYY:PeakContestDefenseFightPanel.lua [start:110] :data',self.m_myData.attack_lineup)
		self:updateMyTroop()
	end
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.pk:setTouchEnded(function()
		PeakContestManager:reqChallengeRole(self.m_roleData.rank_id,self.m_roleData.role_id)
		self:close()
	end)

	self.m_csNode.changeNode.save:setTouchEnded(function()
		local function insufficientTroop()
			for i = 1 , 3 do 
				for j = 1 , 6 do 
					if not self.m_myData.attack_lineup[i]["card_list"][j] then 
						return i
					end
				end
			end		
			return 0	
		end
		local function noShowChange()
			self.m_csNode.change:setVisible(true)
			self.m_csNode.pk:setVisible(true)
			self.m_csNode.changeNode:setVisible(false)
			self:updateMyTroop()
			local troopList = {}
			for i = 1 , 3 do
				troopList[i] = self.m_myData.attack_lineup[i]["card_list"]
			end
			PeakContestManager:reqSaveTroopData(1,troopList)
			cc.Audio:playEffect("TopArenaSave.mp3")			
		end
		local troop = insufficientTroop() 
		if troop ~= 0 then 
			local panel = g_scene:openDialog("UI.ConfirmPanel", {text = string.format(Serialize[60619],troop)})		
			panel:setOkHandler(function()
				noShowChange()
			end)	
		else
			noShowChange()
		end
	end)

	self.m_csNode.replace:setTouchEnded(function()
		g_scene:openDialog("PeakContest.PeakContestArrayPanel",{type = TroopType.Attack})
	end)

	self.m_csNode.change:setTouchEnded(function()
		self.m_csNode.change:setVisible(false)
		self.m_csNode.pk:setVisible(false)
		self.m_csNode.changeNode:setVisible(true)
		self.m_selectTroop = 0
		for i = 1 , 3 do 
			self.m_csNode.changeNode["kuang"..i]:setVisible(true)
			self.m_csNode.changeNode["jiantou"..i]:setVisible(false)
			self.m_csNode.changeNode["zhezhao"..i]:setVisible(false)					
		end
		self.m_changeTroopList = table.clone(self.m_myData.attack_lineup)
	end)

	self.m_csNode.changeNode.cancelChange:setTouchEnded(function()
		self.m_csNode.change:setVisible(true)
		self.m_csNode.pk:setVisible(true)
		self.m_csNode.changeNode:setVisible(false)
		self.m_myData.attack_lineup = self.m_changeTroopList
		self:updateMyTroop()
	end)

	for i = 1 , 3 do 
		self.m_csNode.changeNode["kuang"..i]:setTouchEnded(function()
			self.m_selectTroop = i
			for j = 1 , 3 do 
				self.m_csNode.changeNode["kuang"..j]:setVisible(false)
				self.m_csNode.changeNode["jiantou"..j]:setVisible(j ~= i)
				self.m_csNode.changeNode["zhezhao"..j]:setVisible(j~=i)					
			end
		end)
	end

	for i = 1 , 3 do 
		self.m_csNode.changeNode["jiantou"..i]:setTouchEnded(function()
			if self.m_selectTroop ~= 0 then 
				local lineup1= self.m_myData.attack_lineup[self.m_selectTroop] or {}
				local lineup2 = self.m_myData.attack_lineup[i] or {}
				local t = table.clone(lineup1)
				self.m_myData.attack_lineup[self.m_selectTroop] = table.clone(lineup2)
				self.m_myData.attack_lineup[i] = table.clone(t) 
				for j = 1 , 3 do 
					self.m_csNode.changeNode["kuang"..j]:setVisible(true)
					self.m_csNode.changeNode["jiantou"..j]:setVisible(false)					
					self.m_csNode.changeNode["zhezhao"..j]:setVisible(false)					
				end
				self.m_selectTroop = 0
				self:updateMyTroop()
				cc.Audio:playEffect("SwitchTeam.mp3")
			end
		end)		
	end

	for i = 1 , 3 do 
		self.m_csNode["troopBtn"..i]:setTouchEnded(function()
			self:updateRoleTroop(i)		
		end)
	end

	self.m_csNode.help:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text = Serialize["peak_contest_att_help_tips"],node = self.m_csNode.help})
	end)
end

function This:onTouchMask()

end

function This:onEnter()
	super.onEnter(self)
	PeakContestManager:addObserver(self)
	self:refresh()
end

function This:onExit()
	PeakContestManager:removeObserver(self)
	super.onExit(self)
end

function This:updateJiantouAction()
	local x = 10
	local time = 0.5
	for i = 1 , 3 do 
		local action = cc.Sequence(cc.MoveBy(time, -x, 0) , cc.MoveBy(time, x, 0))
		self.m_csNode.changeNode["jiantou"..i]:runAction(cc.RepeatForever(action))
	end
end
