-- @Author: ZYY
-- @Date:   2018-09-26 18:03:31
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-14 15:02:30
-- @File Name: PeakContestDefensePanel.lua
-- @Desc: 
local This, super = class(...,"Dialog")
local PeakContestTeamItemView = require("PeakContest.PeakContestTeamItemView")
local PeakContestManager = require("PeakContest.PeakContestManager")
local PeakContestUtil = require("PeakContest.PeakContestUtil") 
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")

function This:initNode(info)
	self.m_troopType = info.type
	local troopList = PeakContestManager:getMyTroop(info.type)
	self.m_troopList = {}
	if troopList then 
		for i , v in ipairs(troopList) do 
			self.m_troopList[i] = v.card_list
		end
	end
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PeakContestDefense.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
	self.m_myData = PeakContestManager:getMyData()
	self.m_beforTroopItemList = {}
	self.m_afterTroopItemList = {}
	self.m_selectTroop = 0
	self.m_csNode.after:setVisible(false)
	self:updateJiantouAction()
	self:refresh()
end

function This:refresh()
	self:updateTroop()
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.after.replace:setTouchEnded(function()
		g_scene:openDialog("PeakContest.PeakContestArrayPanel",{type = self.m_troopType})
	end)

	self.m_csNode.after.save:setTouchEnded(function()
		PeakContestManager:reqSaveTroopData(self.m_troopType,self.m_troopList)	
		cc.Audio:playEffect("TopArenaSave.mp3")
		self:close()		
	end)

	self.m_csNode.befor.replace:setTouchEnded(function()
		g_scene:openDialog("PeakContest.PeakContestArrayPanel",{type = self.m_troopType})
	end)

	for i = 1 , 3 do 
		self.m_csNode.after["kuang"..i]:setTouchEnded(function()
			self.m_selectTroop = i
			for j = 1 , 3 do 
				self.m_csNode.after["kuang"..j]:setVisible(false)
				self.m_csNode.after["jiantou"..j]:setVisible(j ~= i)
				self.m_csNode.after["zhezhao"..j]:setVisible(j ~= i)
			end
		end)
	end
	
	for i = 1 , 3 do 
		self.m_csNode.after["jiantou"..i]:setTouchEnded(function()
			if self.m_selectTroop ~= 0 then 
				local t = table.clone(self.m_troopList[self.m_selectTroop])
				self.m_troopList[self.m_selectTroop] = table.clone(self.m_troopList[i])
				self.m_troopList[i] = table.clone(t) 
				for j = 1 , 3 do 
					self.m_csNode.after["kuang"..j]:setVisible(true)
					self.m_csNode.after["jiantou"..j]:setVisible(false)					
					self.m_csNode.after["zhezhao"..j]:setVisible(false)
				end
				self.m_selectTroop = 0
				self:updateTroop()
				cc.Audio:playEffect("SwitchTeam.mp3")
			end
		end)		
	end

	self.m_csNode.befor.change:setTouchEnded(function()
		self.m_csNode.befor:setVisible(false)
		self.m_csNode.after:setVisible(true)
		for i = 1 , 3 do 
			self.m_csNode.after["kuang"..i]:setVisible(true)
			self.m_csNode.after["jiantou"..i]:setVisible(false)
			self.m_csNode.after["zhezhao"..i]:setVisible(false)		
		end
	end)

	self.m_csNode.after.cancelChange:setTouchEnded(function()
		self.m_csNode.befor:setVisible(true)
		self.m_csNode.after:setVisible(false)	
	end)
end

function This:updateTroop()
	local width = 80
	for i = 1 , 3 do 
		local num = 0
		self.m_beforTroopItemList[i] = self.m_beforTroopItemList[i] or {}
		self.m_afterTroopItemList[i] = self.m_afterTroopItemList[i] or {}
		for j = 1 , 6 do 	
			local uid = 0 
			if self.m_troopList[i] then 
				uid = self.m_troopList[i][j] or 0
			end
			if uid ~= 0 then 
				num = num + 1 
			end
			local hero = HeroManager:getHeroByUid(uid)
			local awake = false
			if hero then
				awake = hero:isAwakeFull()
			end
			local item = self.m_beforTroopItemList[i][j]
			if not item then 
				item = PeakContestTeamItemView:create(hero,nil,nil,awake)
				item:setPosition(width * (j - 3.5), 0)
				item:setScale(0.75)
				self.m_csNode.befor["troop" .. i]:addChild(item)
			else
				item:setModel(hero,nil,nil,awake)
			end
			self.m_beforTroopItemList[i][j] = item
			
			local item2 = self.m_afterTroopItemList[i][j]
			if not item2 then 
				item2 = PeakContestTeamItemView:create(hero,nil,nil,awake)
				item2:setPosition(width * (j - 3.5), 0)
				item2:setScale(0.75)
				self.m_csNode.after["troop" .. i]:addChild(item2)
			else
				item2:setModel(hero,nil,nil,awake)
			end	
			self.m_afterTroopItemList[i][j] = item2
		end
		-- self.m_csNode.befor["troopBtn"..i].num:setString(string.format("(%d/%d)",num,6))
	end		
end

function This:updateTroopData(type,troopList)
	if type == self.m_troopType then 
		self.m_troopList = troopList 
		self:refresh()
	end
end

function This:onTouchMask()

end

function This:onEnter()
	super.onEnter(self)
	PeakContestManager:addObserver(self)
	self:refresh()
end

function This:onExit()
	PeakContestManager:removeObserver(self)
	super.onExit(self)
end

function This:updateJiantouAction()
	local x = 10
	local time = 0.5
	for i = 1 , 3 do 
		local action = cc.Sequence(cc.MoveBy(time, -x, 0) , cc.MoveBy(time, x, 0))
		self.m_csNode.after["jiantou"..i]:runAction(cc.RepeatForever(action))
	end
end