-- @Author: ZYY
-- @Date:   2018-09-11 20:27:18
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 16:48:31
-- @File Name: PeakContestManager.lua
-- @Desc:

local This, super = class(...,"Base.BaseModel")
local TopArenaCommonShopConfig = require("Metadatas.TopArenaCommonShopConfig")
local TopArenaFirstShopConfig = require("Metadatas.TopArenaFirstShopConfig")
local DailyRewardConfig = require("Metadatas.DailyRewardConfig")
local WeeklyRewardConfig = require("Metadatas.WeeklyRewardConfig")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local ReportManager = require("Report.ReportManager")
local PeakContestUtil = require("PeakContest.PeakContestUtil") 
local HeroManager = require("Hero.HeroManager")
local PeakContestData = require("PeakContest.PeakContestData")

PeakContestChallengeBoxRewardState =
{
	kCanReceive = 0,        -- 可领取
	kUnReceive  = 1,        -- 不可领取 
    kHasReceive = 2         -- 已领取
}
PeakContestChallengeBoxRewardType = 
{
	kDay = 1,				-- 日奖励
	kWeek = 2				-- 周奖励
}
local kOpenLevel = 55

function This:init()
	super.init(self)
	self.m_nextRefreshTime = 0
	self.m_data = {}
	self.m_firstGroup = {}
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self,13200,self.ansMyPeakContest)
	EventDispatcher:addListener(self,13201,self.ansUpdateChallenge)
	EventDispatcher:addListener(self,13202,self.ansSaveTroopData)
	EventDispatcher:addListener(self,13203,self.ansBuyChallenge)
	EventDispatcher:addListener(self,13204,self.ansSeeRankData)
	EventDispatcher:addListener(self,13205,self.ansChallengeRole)
	EventDispatcher:addListener(self,13206,self.ansHonorShopData)
	EventDispatcher:addListener(self,13207,self.ansBuyHonorShop)
	EventDispatcher:addListener(self,13208,self.ansPeakShopData)
	EventDispatcher:addListener(self,13209,self.ansBuyPeakShop)
	EventDispatcher:addListener(self,13210,self.ansUpdatePeakShopData)
	EventDispatcher:addListener(self,13211,self.ansReportData)
	EventDispatcher:addListener(self,13212,self.ansCleanChallengeCD)
	EventDispatcher:addListener(self,13213,self.ansFirstGroupData)
	EventDispatcher:addListener(self,13214,self.ansReceiveRoleReward)
	EventDispatcher:addListener(self,13215,self.ansReceiveGroupReward)
	EventDispatcher:addListener(self,13217,self.ansReceiveFightReward)
end

function This:getChallengeCD()
	return self.m_nextRefreshTime - GamePlayer:getServerTime() 
end

function This:getCardTroop(cardId)
	return self.cardTroopList[cardId] or 0
end

function This:getMyRank()
	return self.m_data.rank_id or 0
end

function This:getMyMaxRank()
	return self.m_data.max_rank_id or 0
end

function This:getMyData()
	return self.m_data or {}
end

function This:getFirstGroupName()
	return self.m_firstGroup.name 
end

function This:isOpen()
	local level = GamePlayer:getLevel()
	return level >= kOpenLevel
end

function This:getMaxNum(type)	-- 获取最大任务数
	local maxNum,res = 0
	if type == PeakContestChallengeBoxRewardType.kDay then
		res = DailyRewardConfig
	else
		res = WeeklyRewardConfig
	end
	for _,List in pairs(res) do
		maxNum = maxNum + 1
	end
	return maxNum
end

function This:getTaskState(id,type)		--获取奖励状态
	local state,res,des,num= PeakContestChallengeBoxRewardState.kUnReceive
	if type == PeakContestChallengeBoxRewardType.kDay then
		res = DailyRewardConfig
		des = self.m_data.day_reward_record
		num = self.m_data.day_challenge_times
	else
		res = WeeklyRewardConfig
		des = self.m_data.week_reward_record
		num = self.m_data.week_challenge_times
	end
	-- for _,List in pairs (res) do
	-- 	if id == List.id then
	-- 		if num >= List.frequency then
	-- 			state = PeakContestChallengeBoxRewardState.kCanReceive
	-- 		end
	-- 	end
	-- end
	if num >= res[id].frequency then
		state = PeakContestChallengeBoxRewardState.kCanReceive
	end
	for _,rewardId in pairs(des) do
		if id == rewardId then
			state = PeakContestChallengeBoxRewardState.kHasReceive
		end
	end
	return state
end

function This:getMaxfrequency()			--获取周最大奖励次数
	local num = 0
	for _,data in pairs(WeeklyRewardConfig) do
		if num < data.frequency then
			num = data.frequency
		end
	end
	return num
end

function This:hasFightBoxReward()  		--存在未领取奖励
	local hasReward = false
	if self:hasRewardTool(PeakContestChallengeBoxRewardType.kDay) or self:hasRewardTool(PeakContestChallengeBoxRewardType.kWeek) then
		hasReward = true
	end
	return hasReward
end

function This:hasRewardTool(type)
	local hasReward,res = false
	if type == PeakContestChallengeBoxRewardType.kDay then
		res = DailyRewardConfig
	else
		res = WeeklyRewardConfig
	end
	for _,data in pairs(res) do
		local state =self:getTaskState(data.id,type)
		if state == PeakContestChallengeBoxRewardState.kCanReceive then
			hasReward = true
			break
		end
	end
	return hasReward
end

function This:isDefened()
	local len = 0
	for i , v in ipairs(self.m_data.defened_lineup) do 
		if v.card_list then 
			len = #v.card_list > len and #v.card_list or len
		end
	end
	return len ~= 0
end

function This:getMyTroop(type)
	if type == TroopType.Attack then 
		return table.clone(self.m_data.attack_lineup)
	elseif type == TroopType.Defened then 
		return table.clone(self.m_data.defened_lineup)
	end
end

function This:getHeroList()
	local heros = HeroManager:getAllHeroGotWithSort(function(hero) 
			if hero:isDeputy() then
				return false
			end
			return true
		end
	)
	return heros
end

function This:getPeakContestData()
	if not self.m_ratingData then
		self.m_ratingData = PeakContestData:create(0)
	end
	return self.m_ratingData
end

function This:reqMyPeakContest()
	Socket:sendAndLock(13200,{})
end

function This:ansMyPeakContest(msg)
	-- 本地化机器人名字
	if msg.challenge_member_list then
		for i, v in ipairs(msg.challenge_member_list) do
			v.name = AppText:getServerText(v.name)
		end
	end

	self.m_nextRefreshTime = msg.next_refresh_time
	self.m_data = msg
	local ratingData = self:getPeakContestData()
	ratingData:setChallengeTimes(msg.challenge_times)
	ratingData:setFirstPlayer(msg.previous_day_frist_role)
	self:notifyObservers("setMyPeakContestData",msg)
end

function This:reqUpdateChallenge()
	Socket:sendAndLock(13201,{})
end

function This:ansUpdateChallenge(msg)
	self:notifyObservers("setChallenge",msg.challenge_member_list)
end

function This:reqSaveTroopData(type,troopList)
	Socket:sendAndLock(13202,{type = type , card_list1 = troopList[1] or {} , card_list2 = troopList[2] or {},card_list3 = troopList[3] or {}})
end

function This:ansSaveTroopData(msg)
	if msg.err_code == 0 then 
		if msg.type == TroopType.Attack then 
			self.m_data.attack_lineup = {{card_list = msg.card_list1},{card_list = msg.card_list2},{card_list = msg.card_list3}}
		elseif msg.type == TroopType.Defened then 
			self.m_data.defened_lineup = {{card_list = msg.card_list1},{card_list = msg.card_list2},{card_list = msg.card_list3}}
		end
		self:notifyObservers("updateTroopData",msg.type,{msg.card_list1,msg.card_list2,msg.card_list3})
		g_scene:showTips(Serialize[60673])
	end
end

function This:reqBuyChallenge()
	Socket:sendAndLock(13203,{buy_time = 1})
end

function This:ansBuyChallenge(msg)
	if msg.err_code == 0 then 
		self:notifyObservers("updateChallengeNum",msg.challenge_time,msg.buy_time)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:reqSeeRankData()
	Socket:sendAndLock(13204,{start_index = 1,end_index = 50})
end

function This:ansSeeRankData(msg)
	if msg.err_code == 0 then 
		local function rank_fun(a,b)
			return a.rank_id < b.rank_id
		end
		table.sort(msg.challenge_member_list,rank_fun)
		self:notifyObservers("UpdateRankData",msg.challenge_member_list)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:reqChallengeRole(rank,role)
	Socket:sendAndLock(13205,{rank_id = rank,role_id = role})
end

function This:ansChallengeRole(msg)
	if msg.err_code == 0 then
		-- 本地化机器人名字
		if msg.challenge_member_list then
			for i, v in ipairs(msg.challenge_member_list) do
				v.name = AppText:getServerText(v.name)
			end
		end
		if msg.report then
			if msg.report.defence_player then
				msg.report.defence_player.name = AppText:getServerText(msg.report.defence_player.name)
			end
			for i, v in ipairs(msg.report.report_list) do	
				if v.defence then
					v.defence.name = AppText:getServerText(v.defence.name)
				end
			end
		end

		for i , info in ipairs(msg.report.report_list) do	
			ReportManager:addReport(info)
		end
		self.m_nextRefreshTime = msg.next_refresh_time
		self.m_data.rank_id = msg.rank
		self.m_data.challenge_member_list = msg.challenge_member_list
		self.m_data.challenge_times = msg.challenge_time
		self.m_data.max_rank_id = msg.max_rank
		self.m_data.role_reward = msg.role_reward
		self.m_data.group_reward = msg.group_reward
		self.m_data.day_challenge_times = msg.day_challenge_times
		self.m_data.week_challenge_times = msg.week_challenge_times
		local ratingData = self:getPeakContestData()
		ratingData:setChallengeTimes(msg.challenge_time)
		g_scene:openDialog("PeakContest.PeakContestBattleInfoPanel",msg.report.report_list)
		local function rank_fun(a,b)
			return a.rank_id < b.rank_id
		end
		table.sort(msg.challenge_member_list,rank_fun)
		self:notifyObservers("setMyPeakContestData",self.m_data)

		local win = msg.report.res
		local lose = #msg.report.report_list - win
		if win > lose then 
			g_scene:openDialog("PeakContest.PeakContestVictoryPanel",msg)
		else
			g_scene:openDialog("PeakContest.PeakContestEndPanel",msg)
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:reqHonorShopData()
	Socket:sendAndLock(13206,{})
end

function This:ansHonorShopData(msg)
	self:notifyObservers("UpdateShopItem",msg.queue_list,PeakContestShopType.Honor)
end

function This:reqBuyHonorShop(trench, item_id)
	Socket:sendAndLock(13207,{trench = trench , item_id = item_id })
end

function This:ansBuyHonorShop(msg)
	if msg.err_code == 0 then 
		local goods = TopArenaFirstShopConfig[msg.trench]
		if goods then
			g_scene:showReward({{goods.item_id,goods.item_num}})
		end		
		self:notifyObservers("UpdateShopItem",msg.queue_list,PeakContestShopType.Honor)
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

function This:reqPeakShopData()
	Socket:sendAndLock(13208,{})
end

function This:ansPeakShopData(msg)
	self:notifyObservers("UpdateRefreshTimes",msg.refresh_times)
	self:notifyObservers("UpdateShopItem",msg.queue_list,PeakContestShopType.Peak)
end

function This:reqBuyPeakShop(trench, config_id)
	Socket:sendAndLock(13209,{trench = trench , config_id = config_id})
end

function This:ansBuyPeakShop(msg)
	if msg.err_code == 0 then 
		local goods = TopArenaCommonShopConfig[msg.config_id]
		if goods then
			g_scene:showReward({{goods.item_id,goods.item_num}})
		end
		self:notifyObservers("UpdateShopItem",msg.queue_list,PeakContestShopType.Peak)
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

function This:reqUpdatePeakShopData()
	Socket:sendAndLock(13210,{})
end

function This:ansUpdatePeakShopData(msg)
	if msg.err_code == 0 then 
		self:notifyObservers("UpdateShopItem",msg.queue_list,PeakContestShopType.Peak)
		self:notifyObservers("UpdateRefreshTimes",msg.refresh_times)
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

function This:reqReportData()
	Socket:sendAndLock(13211,{})
end

function This:ansReportData(msg)
	if msg.err_code == 0 then
		-- 本地化机器人名字 
		for _, info in ipairs(msg.report_list) do
			info.defence_player.name = AppText:getServerText(info.defence_player.name)
		end

		local result = {}
		for i , info in ipairs(msg.report_list) do
            local reportInfo = info.report_list[1]
            if reportInfo then
			    local report = ReportManager:addReport(reportInfo)
			    table.insert(result,report)
            end
		end
		table.sort(result,function(a,b)
			return a:getStartTime() > b:getStartTime()
		end)
		g_scene:openDialog("PeakContest.PeakContestReportPanel",{reportList = msg.report_list})
	else
		g_scene:handleErrorCode(msg.err_code)
	end		
end

function This:reqCleanChallengeCD()
	Socket:sendAndLock(13212, {})
end

function This:ansCleanChallengeCD(msg)
	if msg.err_code == 0 then 
		self.m_nextRefreshTime = msg.next_challenge_time
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:reqFirstGroupData()
	Socket:sendAndLock(13213, {})
end

function This:ansFirstGroupData(msg)
	if msg.err_code == 0 then 
		self.m_firstGroup.id = msg.group_id
		self.m_firstGroup.name = msg.group_name
		self:notifyObservers("updateFirstGroup")
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

function This:reqReceiveRoleReward()
	Socket:sendAndLock(13214, {})
end

function This:ansReceiveRoleReward(msg)
	if msg.err_code == 0 then 
		self.m_data.is_get_role_reward = 1
		self:notifyObservers("updateBoxData")
		self:notifyObservers("updateRewardEffect")
		g_scene:showReward(msg.reward_list)
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

function This:reqReceiveGroupReward()
	Socket:sendAndLock(13215, {})
end

function This:ansReceiveGroupReward(msg)
	if msg.err_code == 0 then 
		self.m_data.is_get_group_reward = 1
		self:notifyObservers("updateBoxData")
		self:notifyObservers("updateRewardEffect")
		g_scene:showReward(msg.reward_list)
	else
		g_scene:handleErrorCode(msg.err_code)
	end	
end

function This:reqReceiveFightReward(type, key)
	Socket:sendAndLock(13217,{type=type,id=key})
end

function This:ansReceiveFightReward(msg)
	if msg.err_code == 0 then
		self.m_data.day_reward_record = msg.day_reward_record
		self.m_data.week_reward_record = msg.week_reward_record
		self:notifyObservers("updateBoxRewardState")
		g_scene:showReward(msg.reward_list)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end
