-- @Author: ZYY
-- @Date:   2018-09-11 20:24:11
-- @Last Modified by: ZYY
-- @Last Modified time: 2018-09-29 10:44:29
-- @File Name: PeakContestPanel.lua
-- @Desc: 
local PeakContestPlayerView = require("PeakContest.PeakContestPlayerView")
local PeakContestManager = require("PeakContest.PeakContestManager")
local TopArenaTimesConfig = require("Metadatas.TopArenaTimesConfig")  
local PeakContestUtil = require("PeakContest.PeakContestUtil")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local VipConfig = require("Metadatas.VipConfig")  
local VipManager = require("Vip.VipManager")
local This, super = class(...,"Dialog")

local BoxImage = {
	[1] = {
		"MenuScene/boxLock3.png",
		"MenuScene/boxGet3.png",
	},
	[2] = {
		"MenuScene/boxLock4.png",
		"MenuScene/boxGet4.png",
	}
}

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PeakContest.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)
	self.m_playerItemList = {}
	self.m_myData = {}
	self.m_escape = 0
	self.m_time = 600
	self.m_changeTime = 0
	self.m_csNode.time:setVisible(false)
	PeakContestManager:reqMyPeakContest()
	self:updateRewardEffect()
	self:refresh()
end

function This:refresh()
	self:updateMyRank()
	self:updateChallengeNum()
	self:updateChallengeView()
	self.m_csNode.groudNum:setString(self.m_myData.group_num_on_rank or 0)
	local peakData = PeakContestManager:getPeakContestData()
	local playerID = peakData:getFirstPlayer()
	local playeString
	if playerID == "" then
		playeString =Serialize[60736]
	else
		playeString = playerID
	end
	self.m_csNode.dianfengbazhu:setString(string.format( Serialize[62228],playeString ))
end

function This:updateMyRank()
	local rank = PeakContestManager:getMyRank()
	if not rank or rank == 0 then 
		self.m_csNode.myNoRank:setVisible(true)
		self.m_csNode.myRank:setVisible(false)
	else
		self.m_csNode.myNoRank:setVisible(false)
		self.m_csNode.myRank:setVisible(true)
		self.m_csNode.myRank:setString(rank or 0)
	end
end

function This:updateChallengeView()
	local challengeList = self.m_myData.challenge_member_list or {}
	for i , v in ipairs(self.m_playerItemList) do 
		v:setVisible(false)
	end
	for i , challenge in ipairs(challengeList) do 
		local item = self.m_playerItemList[i] 
		if not item then 
			item = PeakContestPlayerView:create(challenge,self.m_myData)
			self.m_csNode["player"..i]:addChild(item)
		else
			item:setData(challenge,self.m_myData)
		end
		item:setVisible(true)
		self.m_playerItemList[i]  = item
	end
end

function This:updateTime()
	self.m_time = PeakContestManager:getChallengeCD()
	if self.m_time <= 1 then
		self.m_csNode.time:setVisible(false)
	else
		local timeStr = GameUtil:formatTime(Serialize[62229],self.m_time)
		self.m_csNode.time:setStringWithColor(timeStr)
		self.m_csNode.time:setVisible(true)
	end 

	if self.m_changeTime <= 0 then 
		self.m_csNode.change.text:setString(Serialize[62230])
	else
		self.m_csNode.change.text:setString(Serialize[62231].. self.m_changeTime .. Serialize[62232])
		self.m_changeTime = self.m_changeTime - 1
	end
end

function This:updateChallengeNum(challenge,buyTime)
	if challenge then 
		self.m_myData.challenge_times = challenge
	end
	if buyTime then 
		self.m_myData.buy_times = buyTime
	end
	self.m_csNode.num:setString(self.m_myData.challenge_times or 0 )
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.add:setTouchEnded(function()
		if self.m_myData.challenge_times > 0 then
			g_scene:showTips(Serialize["Arena_ArenaPanel_2"])
			return
		end
		local buyTimes = self.m_myData.buy_times or 0
		local vipLevel = VipManager:getMaxMemberLevel() or 0
		local toparenaTimes = 0
		for i = 0 , vipLevel do 
			toparenaTimes = VipConfig[i].toparena_times + toparenaTimes
		end
		if buyTimes >= toparenaTimes then 
			if vipLevel == 3 then 
				g_scene:showTips(Serialize[62233])
			else
			    local panel = g_scene:openDialog("UI.ConfirmPanel", {
			            fileName = "MenuScene/NodeUI/MemberActivation.csb" , 
			            text = Serialize["vip_priv_warming_arena_times"], 
			            okBtnTxt = Serialize[62234],
			            okHandler = function () g_scene:openDialog("Vip.VipPanel") end
			    })				
			end
		else
			local cost = TopArenaTimesConfig[buyTimes + 1].reset_cost
			GamePlayer:costDiamond({
				cost = cost,
				okHandler = function()
					PeakContestManager:reqBuyChallenge()
				end,
				desc = Serialize["Arena_ArenaPanel_3"]
			})
		end

	end)

	self.m_csNode.reward:setTouchEnded(function()
		g_scene:openDialog("PeakContest.PeakContestBoxPanel")
	end)

	self.m_csNode.reward1:setTouchEnded(function ()
		g_scene:openDialog("PeakContest.PeakContestChallengeBoxPanel")
	end)

	self.m_csNode.array:setTouchEnded(function()
		
		if PeakContestManager:isDefened() then 
			g_scene:openDialog("PeakContest.PeakContestDefensePanel",{type = TroopType.Defened})
		else
			g_scene:openDialog("PeakContest.PeakContestArrayPanel",{type = TroopType.Defened})
		end
	end)

	self.m_csNode.change:setTouchEnded(function()
		if self.m_changeTime > 0 then 
		else
			PeakContestManager:reqUpdateChallenge()
			self.m_changeTime = 3
		end
	end)

    self.m_csNode.biaoqian.rule.kuang:setTouch({
        onEnded = function()
	        g_scene:openDialog("PeakContest.PeakContestRulePanel")
        end,
        isAlpha = false,
        isScale = false,
        isSwallow = false,
        isDynamic = true,
    })

    self.m_csNode.biaoqian.rank.kuang:setTouch({
        onEnded = function()
			g_scene:openDialog("PeakContest.PeakContestRankingPanel")
        end,
        isAlpha = false,
        isScale = false,
        isSwallow = false,
        isDynamic = true,
    })

    self.m_csNode.biaoqian.shop.kuang:setTouch({
        onEnded = function()
			g_scene:openDialog("PeakContest.PeakContestShopPanel",{index = 1})
        end,
        isAlpha = false,
        isScale = false,
        isSwallow = false,
        isDynamic = true,
    })

    self.m_csNode.biaoqian.record.kuang:setTouch({
        onEnded = function()
			PeakContestManager:reqReportData()
        end,
        isAlpha = false,
        isScale = false,
        isSwallow = false,
        isDynamic = true,
    })
	self.m_csNode.gantanhao:setTouchEnded(function ()
		g_scene:openDialog("UI.HelpTips",{text = Serialize["peakContest_Box_3"],node=self.m_csNode.gantanhao})
	end)
end

function This:onTouchMask()

end

function This:setMyPeakContestData(data)
	self.m_myData = data
	self:refresh()
	self:updateRewardEffect()
end

function This:setChallenge(challengeList)
	self.m_myData.challenge_member_list = challengeList
	self:updateChallengeView()
end

function This:onEnter()
	super.onEnter(self)
	PeakContestManager:addObserver(self)
	self:refresh()
	self:updateRewardEffect()
end

function This:onExit()
	PeakContestManager:removeObserver(self)
	super.onExit(self)
end

function This:update(dt)
	super.update(self,dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return 
	end
	self.m_escape = self.m_escape - 1
	self:updateTime()
end

function This:updateRewardEffect()
	if not self.m_effect then 
		local w ,h = self.m_csNode.reward:getContentSize()
		self.m_effect = GameUtil:createEffect("BoxLing",true)
		self.m_effect:setPosition(w/2, h/2)
		self.m_csNode.reward:addChild(self.m_effect, -1)
	end
	if not self.m_effect1 then
		local w ,h = self.m_csNode.reward1:getContentSize()
		self.m_effect1 = GameUtil:createEffect("BoxLing",true)
		self.m_effect1:setPosition(w/2, h/2)
		self.m_csNode.reward1:addChild(self.m_effect1, -1)
	end
	local data = PeakContestManager:getMyData()
	if data.is_get_role_reward == 0 or data.is_get_group_reward == 0 then 
		self.m_effect:setVisible(true)
		self.m_csNode.reward:setImage(BoxImage[1][2])
	else
		self.m_effect:setVisible(false)
		self.m_csNode.reward:setImage(BoxImage[1][1])
	end
	if self.m_myData.day_challenge_times then
		if PeakContestManager:hasFightBoxReward() then
			self.m_effect1:setVisible(true)
			self.m_csNode.reward1:setImage(BoxImage[2][2])
		else
			self.m_effect1:setVisible(false)
			self.m_csNode.reward1:setImage(BoxImage[2][1])
		end
	end
end

function This:updateBoxRewardState()
	self:updateRewardEffect()
end