--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local PlanShopUtil = require("PlanShop.PlanShopUtil")
local Application = require("cc.Application")
local WorldMapManager = require("World.WorldMapManager")

local kUsePlanGainResouceNum = 80000

PlanFormula = {
    kSkipShopCdCost = function ( time )     -- 清商店CD的花销公式 
        return math.ceil(time / 600)
    end,
    kSkipCdCost = function ( time )     -- 清单个计策CD的花销公式 
        return math.ceil(time / 60 * 0.5)
    end,
}

function This:init()
	super.init(self)
    self.m_cur_planId = nil
    self.m_escape = 0
    self.m_leftTime = nil
    self.m_isUpdating = false

	self:initListener()
end

function This:getPlanTypeNum()
    return JiCeType.kTotal
end

function This:getLeftTime()
    return self.m_leftTime
end

function This:initListener()
    EventDispatcher:addListener(self, "init_role_info", self.onGetPlanShopRequest)
	EventDispatcher:addListener(self, 11300, self.onGetPlanShopReply)
	EventDispatcher:addListener(self, 11301, self.onStudyPlanReply)
	EventDispatcher:addListener(self, 11302, self.onClearCDReply)
    EventDispatcher:addListener(self, 11303, self.onUsePlanReply)
	EventDispatcher:addListener(self, 11304, self.onClearUseCDReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self,"init_role_info")
	EventDispatcher:removeListener(self,11300)
	EventDispatcher:removeListener(self,11301)
	EventDispatcher:removeListener(self,11302)
    EventDispatcher:removeListener(self,11303)
	EventDispatcher:removeListener(self,11304)
end


function This:startUpdate()
	Application:addToUpdateList(self)
    self:notifyObservers("onPlanListsUpdateStart")
	self.m_isUpdating = true
    EventDispatcher:dispatcher("plan_shop_cd_changed")
end

function This:endUpdate()
    local ret = true
    if self.m_isUpdating then
	    Application:removeFromUpdateList(self)
        self:notifyObservers("onPlanListsUpdateEnd")
	    self.m_isUpdating = false
    else
        local leftTime = self:getLeftTime()
        if leftTime <= 0 then
            self:notifyObservers("onPlanListsUpdateEnd")
        else
            ret = false
            self:notifyObservers("updateTime", leftTime)
        end
    end
    EventDispatcher:dispatcher("plan_shop_cd_changed")
end

function This:getCostForClearCD()
    return PlanFormula.kSkipShopCdCost(self.m_leftTime)
end

function This:setLeftTime(leftTime)
    self.m_leftTime = leftTime
    if leftTime <= 0 then
        self:endUpdate()
        return
    end

	if not self.m_isUpdating then
		self:startUpdate()
    else
        self:notifyObservers("updateTime", leftTime)
	end
	self:notifyObservers("onPlanListsChanged", leftTime < 36000)
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
    	local leftTime = self:getLeftTime()
		self.m_escape = self.m_escape - 1
        leftTime = leftTime - 1
        self:setLeftTime(leftTime)
	    if leftTime >= 0 then
		    self:notifyObservers("updateTime", leftTime)
            if leftTime >= 36000 then
                self:notifyObservers("onPlanListsChanged", false)
            end
	    else
		    self:endUpdate()
	    end
    end
end

function This:onGetPlanShopRequest()
    if not self.m_leftTime then
        Socket:send(11300,{})
    else
        return false, self.m_leftTime < 36000
    end
    return true, false
end

function This:onGetPlanShopReply(msg)
	self:setLeftTime(msg.last_refresh_time)
    self:notifyObservers("onGetPlanInfo")
end

function This:onStudyPlanRequest(planId)
    Socket:sendAndLock(11301,{id = planId})
end

function This:onStudyPlanReply(msg)
    if msg.err_code == 0 then
        self.m_cur_planId = msg.id
		self:setLeftTime(msg.last_refresh_time)
        self:notifyObservers("onStudyPlan", msg.id)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onClearCDRequest()
    Socket:sendAndLock(11302)
end

function This:onClearCDReply(msg)
    if msg.err_code == 0 then
        self.m_cur_planId = nil
		self:setLeftTime(0)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onUsePlanRequest(param, useGold)
    local cost = useGold and (param.num * param.gold) or 0
    local switch = {
        [JiCeType.kCaiYuanGunGun] = function()    -- for case 1
            return {index = JiCeType.kCaiYuanGunGun, item_num = param.num, gold = cost, arg_list={}}
        end,
        [JiCeType.kWuGuFengDeng] = function()    -- for case 2
            return {index = JiCeType.kWuGuFengDeng, item_num = param.num, gold = cost, arg_list={}}
        end,
        [JiCeType.kZhaoBinMaima] = function()    -- for case 3
            return {index = JiCeType.kZhaoBinMaima, item_num = param.num, gold = cost, arg_list={param.argId}}
        end,
        [JiCeType.kXingJunJiaSu] = function()    -- for case 4
            return {index = JiCeType.kXingJunJiaSu, item_num = param.num, gold = cost, arg_list={param.argId}}
        end,
        [JiCeType.kNengGongQiaoJiang] = function()    -- for case 5
            return {index = JiCeType.kNengGongQiaoJiang, item_num = param.num, gold = cost, arg_list={param.argId}}
        end,
        [JiCeType.kLuBanZaiShi] = function()
            return {index = JiCeType.kLuBanZaiShi, item_num = param.num, gold = cost, arg_list={param.argId}}
        end,

		[JiCeType.kQianDuLing] = function()
			return {index = JiCeType.kQianDuLing, item_num = param.num, gold = cost, arg_list={param.argId}}
		end,
		[JiCeType.kQiMenBaGua] = function()
			return {index = JiCeType.kQiMenBaGua, item_num = param.num, gold = cost, arg_list={param.argId}}
		end,
		[JiCeType.kKongChengJi] = function()
			return {index = JiCeType.kKongChengJi, item_num = param.num, gold = cost, arg_list={param.argId}}
		end,
        [JiCeType.kDongRuoGuanHuo] = function()
            return {index = JiCeType.kDongRuoGuanHuo, item_num = param.num, gold = cost, arg_list={param.argId}}
        end,
		[JiCeType.kHanDiShouYu] = function()
			return {index = JiCeType.kHanDiShouYu, item_num = param.num, gold = cost, arg_list={param.argId}}
		end,
    }
    Socket:sendAndLock(11303, switch[param.plan]())
end


function This:onUsePlanReply(msg)
    if msg.err_code ~= 0 then
		g_scene:handleErrorCode(msg.err_code)
        return 
	end

    local planId = PlanShopUtil:getPlanIdByItemId(msg.item_id)
    msg.planId = planId
    if planId == JiCeType.kCaiYuanGunGun then
		local num = WorldMapManager:getValueByUsingJice(planId)
        msg.rewardList = {{id = 108, count = num * msg.item_num}}
    elseif  planId == JiCeType.kWuGuFengDeng then
		local num = WorldMapManager:getValueByUsingJice(planId)
        msg.rewardList = {{id = 109, count = num * msg.item_num}}
    end
    self:notifyObservers("onUsePlan", msg)

end

function This:onClearUseCDRequest(index)
	Socket:sendAndLock(11304,{index = index})
end

function This:onClearUseCDReply(msg)
	if msg.err_code == 0 then
		self:notifyObservers("onClearUseCD",msg.index)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end
--endregion
