--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This, super = class(...,"Dialog")
local ListLayer = require("UI.ListLayer")
local PlanShopItemView = require("PlanShop.PlanShopItemView")
local PlanShopManager = require("PlanShop.PlanShopManager")
local DepotManager = require("Depot.DepotManager")
local NobilityManager = require("Nobility.NobilityManager")

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/PlanShop.csb")
    self.m_csNode:setToCenter()
	self:addChild(self.m_csNode)

    self:refreshItemList()
end

function This:refreshItemList(refresh)
    if not refresh then
        self.m_curViewList = {}
        self.m_csNode.scrollView:removeAllItems()
        self.m_csNode.scrollView:setTouchEnabled(true)
    end

    local count = PlanShopManager:getPlanTypeNum()

    for index=1, count do
        local view = PlanShopItemView:create(index)
        local w , h = view:getContentSize()
        self.m_curViewList[index] = view
        self.m_csNode.scrollView:pushNode(view, w + 11.5, h + 3, 3) 
    end

    local result, hasTimeLeft = PlanShopManager:onGetPlanShopRequest()
    if not result then
        self:onPlanListsChanged(hasTimeLeft)
        self:updateRefreshCD()
    end
end

function This:onPlanListsUpdateEnd()
    self:updateRefreshCD()
end

function This:onPlanListsUpdateStart()
	self:updateRefreshCD()
end

function This:onPlanListsChanged(enable)
    self:refreshList(enable)
end

function This:updateTime(remainSecond)
    local hour = math.floor(remainSecond / 3600)
    local minute = math.floor(remainSecond % 3600 / 60)
    local second = math.floor(remainSecond % 60)
    self.m_csNode.cost_time:setString(string.format("%02d:%02d:%02d", hour, minute, second))
end

function This:onEnter()
	super.onEnter(self)
	PlanShopManager:addObserver(self)
    DepotManager:addObserver(self)
    NobilityManager:addObserver(self)
end

function This:itemUpdate(itemList)
    if itemList and #itemList > 0 then
        g_scene:showReward(itemList)
    end
end

function This:onExit()
	super.onExit(self)
	PlanShopManager:removeObserver(self)
    DepotManager:removeObserver(self)
    NobilityManager:removeObserver(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
		self:close()
    end)
    self.m_csNode.clear_btn:setTouchEnded(function()
        self:onClearBtnClick()
    end)
end

function This:onClearBtnClick()
    local cost = PlanShopManager:getCostForClearCD()
    local panel = g_scene:openDialog("PlanShop.PlanComfirmPanel", { cost = cost })
    panel:setHandler(function()
        PlanShopManager.onClearCDRequest()
    end)
end

function This:updateRefreshCD()
    local remainSecond = PlanShopManager:getLeftTime()
    self:updateTime(remainSecond)

    local enable = remainSecond < 36000
    self:refreshList(enable)

    self.m_csNode.clear_btn_gray:setVisible(remainSecond <= 0)
    self.m_csNode.clear_btn:setVisible(remainSecond > 0)
end

function This:refreshList(isEnable)
    local count = PlanShopManager:getPlanTypeNum()
    for i=1, count do
        local node = self.m_curViewList[i]
        local isLockend = NobilityManager:checkPlanUnlock(node:getId())
        node:setItemEnable(isEnable and isLockend)
    end

    if not isEnable then
        self.m_csNode.cost_time:setColor(255, 0, 0)
    else
        self.m_csNode.cost_time:setColor(self.m_csNode.cost_txt:getColor())
    end
end


--endregion
