---
---
--- Created by aid.
--- DateTime: 2018/12/8 15:40
---
local This , super = class(...,"Dialog")
local GamePlayer = require("Player.GamePlayer")
local TitleConfig = require("Metadatas.TitleConfig") -- 称号配表
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local GameUtil = require("Util.GameUtil")
local VipManager = require("Vip.VipManager")

function This:initData(arguments)
    super.initData(self, arguments)
    self.m_type = arguments.type
    self.m_config = arguments.widgetConfig
    self.m_icon = GamePlayer:getIcon()
    self.m_title = GamePlayer:getTitle()
    self.m_effect = GamePlayer:getEffect()
    if self.m_type == PlayerEffectType.icon then
        self.m_icon = self.m_config.ID
    elseif self.m_type == PlayerEffectType.title then
        self.m_title = self.m_config.ID
    elseif self.m_type == PlayerEffectType.effect then
        self.m_effect = self.m_config.ID
    else
        error("error!")
    end
    self.m_owned = arguments.owned
    self.m_using = arguments.using
    self.m_neededItemId = self.m_config.price[1]
    self.m_neededItemCount = self.m_config.price[2]
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RoleChange.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    -- 设置标题
    if self.m_type == PlayerEffectType.icon then
        self.m_csNode.title:setString(Serialize[62284]) -- TODO 国际化
    elseif self.m_type == PlayerEffectType.title then
        self.m_csNode.title:setString(Serialize[62285]) -- TODO 国际化
    elseif self.m_type == PlayerEffectType.effect then
        self.m_csNode.title:setString(Serialize[62286]) -- TODO 国际化
    else
        error("error!")
    end

    -- 描述
    self.m_csNode.text1:setStringWithColor(self.m_config.unlock_des)

    -- 时限
    local timeStr = Serialize[62287]..self.m_config.text2 -- TODO 国际化
    self.m_csNode.timetext1:setString(timeStr)
    self.m_csNode.timetext2:setString(timeStr)

    -- 设置左侧预览
    local touxiang = self.m_csNode.touxiang
    local icon = touxiang.icon
    GameUtil:setPlayerIconNodeImage(icon, self.m_icon)
    GameUtil:setPlayerEffectNodeTo(icon, self.m_effect)
    local touxiangVisible = false
    if self.m_title ~= 0 then
        touxiangVisible = true
       -- self.m_csNode.touxiang.chenghaobg -- TODO 称号背景
        touxiang.chenghao:setImage(TitleConfig[self.m_title].head)
    end
    touxiang.chenghaobg:setVisible(touxiangVisible)
    touxiang.chenghao:setVisible(touxiangVisible)

    -- 购买消耗
    self:setConsumableStatus(self.m_owned, self.m_neededItemCount)

    -- 按钮状态
    self.m_csNode.xingxiang.bg1:setVisible(false) -- TODO 暂时不使用这个node
    self:setButtonStatus(self.m_using)
end

function This:setButtonStatus(using)
    if using then
        self.m_csNode.xingxiang:setVisible(false)
        self.m_csNode.use:setVisible(true)
    else
        if (not self.m_owned) and self.m_neededItemId == 102 and self.m_neededItemCount ~= 0 then
            self.m_csNode.xingxiang.text:setString(Serialize[62288]) -- TODO 国际化
        else
            self.m_csNode.xingxiang.text:setString(Serialize[62289]) -- TODO 国际化
        end
        self.m_csNode.xingxiang:setVisible(true)
        self.m_csNode.use:setVisible(false)
    end
end

function This:setConsumableStatus(owned, neededCount)
    if owned or neededCount == 0 then
        self.m_csNode.money:setVisible(false)
        self.m_csNode.timetext1:setVisible(true)
        self.m_csNode.timetext2:setVisible(false)
    else
        self.m_csNode.money:setVisible(true)
        self.m_csNode.money.yuanbao:setImage(ItemUtil.getIconImage(self.m_neededItemId))
        self.m_csNode.money.num:setString(self.m_neededItemCount)
        self.m_csNode.timetext1:setVisible(false)
        self.m_csNode.timetext2:setVisible(true)
    end
end

function This:onEnter()
    super.onEnter(self)
    GamePlayer:addObserver(self)
end

function This:onExit()
    GamePlayer:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    -- 关闭按钮
    self.m_csNode.close.close:setTouchEnded(function()
        self:close()
    end)

    -- 使用按钮
    self.m_csNode.xingxiang.bg2:setTouchEnded(function()
        -- 检查是否已拥有
        if self.m_owned then
            self:sendChangeWidgetRequest(self.m_config.ID)
            return
        end

        -- 检查是否能够购买
        if self.m_neededItemId == 102 then
            if not GamePlayer:checkDiamond(self.m_neededItemCount) then
                return
            end
        else
            local ownedCount = DepotManager:getItemCount(self.m_neededItemId)
            if ownedCount < self.m_neededItemCount then
                g_scene:showTips(Serialize[60714] .. ItemUtil.getName(self.m_neededItemId) .. Serialize[60715]) -- TODO 国际化
                return
            end
        end

        self:sendChangeWidgetRequest(self.m_config.ID)
    end)
end

function This:sendChangeWidgetRequest(widgetId)
    if self.m_type == PlayerEffectType.icon then
        GamePlayer:onModifyPlayerIconRequest(widgetId)
    elseif self.m_type == PlayerEffectType.title then
        GamePlayer:onModifyPlayerTitleRequest(widgetId)
    elseif self.m_type == PlayerEffectType.effect then
        GamePlayer:onModifyPlayerEffectRequest(widgetId)
    end
end

function This:onPlayerAttrChanged(attrType, oldId, newId)
    local expectedType
    if self.m_type == PlayerEffectType.icon then
        expectedType = PlayerAttr.icon
    elseif self.m_type == PlayerEffectType.title then
        expectedType = PlayerAttr.title
    elseif self.m_type == PlayerEffectType.effect then
        expectedType = PlayerAttr.effect
    end

    if attrType ~= expectedType then
        return
    end

    if newId == self.m_config.ID then
        self.m_owned = true
        self.m_using = true
        self:setButtonStatus(self.m_using)
        self:setConsumableStatus(self.m_owned, self.m_neededItemCount)
    end
end