--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local GamePlayer = require("Player.GamePlayer")
local FriendManager = require("Friend.FriendManager")
local GameUtil = require("Util.GameUtil")
local GuideUtil = require("Guide.GuideUtil")
local RoleLevelConfig = require("Metadatas.RoleLevelConfig")
local MatchingConfig = require("Metadatas.MatchingConfig")
local MatchingNpcConfig = require("Metadatas.MatchNpcConfig")
local HeroConfig = require("Metadatas.HeroConfig")
local FamilyPowerConfig = require("Metadatas.FamilyPowerConfig")
local LoginManager = require("Login.LoginManager")

function This:init(info)
	self.m_info = info.friend_detail
	self.m_groupInfo = info.role_group

	local npcConfig = MatchingNpcConfig[self.m_info.id ]
	if npcConfig then
		self.m_info.name = npcConfig.name
		self.m_info.level = npcConfig.level
		self.m_info.icon = npcConfig.icon
		self.m_info.hero_list = {}
		for i , heroId in ipairs(npcConfig.card_list) do
			table.insert(self.m_info.hero_list,{
				type_id = heroId,
				level = 1,
				start = 0,
				quality = HeroConfig[heroId].quality,
			})
		end
		self.m_info.battle_info.city_num = #npcConfig.cityid
	end

	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RoleInformation.csb")
	-- print_r(cc.visible.height)
	-- self:setContentSize(cc.visible.width, cc.visible.height)
	self:setAnchorPoint(0.5, 0.5)
	self:addChild(self.m_csNode)
	self:infoBaseInfo()
end

function This:infoBaseInfo()
	local csNode = self.m_csNode
	local battle = self.m_info.battle_info
	self.m_isMy = GamePlayer:getPlayerID() == self.m_info.id

	GameUtil:setPlayerFeltNodeResource(self.m_csNode.icon, self.m_info.icon)
	csNode.tips:setString(self.m_info.name)
	csNode.level:setString(self.m_info.level)

	if self.m_groupInfo and self.m_groupInfo.group_id > 0 then
		csNode.position:setString(self.m_groupInfo.group_name .. FamilyPowerConfig[self.m_groupInfo.group_title].title )
	else
		csNode.position:setString(Serialize[60243])
	end 

	csNode.own:setVisible(self.m_isMy)
	csNode.other:setVisible(not self.m_isMy)
	csNode.dengjitiao:setVisible(not self.m_isMy)

	if self.m_isMy then
		local exp = GamePlayer:getExp()

		if self.m_info.level >= kMaxLevel then
			csNode.own.exp:setString(exp)
		else
			csNode.own.exp:setString(string.format("%d/%d",exp,RoleLevelConfig[self.m_info.level + 1].exp))
			csNode.own.expbar:setPercent(exp / RoleLevelConfig[self.m_info.level + 1].exp * 100)
		end
	else
		self:refreshRelationBtn()
	end

	local rank = battle.cur_rank_level <= 0 and 1 or battle.cur_rank_level
	csNode.maquis:setString(MatchingConfig[rank].rank_level)
	csNode.starnum:setString(string.format("X%d",battle.cur_star))
	for i , title in ipairs(battle.title_num_list) do
		csNode["chenghao"..title.title_id].num:setString(title.title_num)
	end

	if rank < kMaxMatchRankLv then
		csNode.nextoffice:setString(MatchingConfig[rank + 1].rank_level)
	else
		csNode.nextoffice:setString(Serialize[62291])
	end
	csNode.jifen:setString(string.format("%d/%d", self.m_info.battle_info.score,MatchingConfig[rank].score))
	csNode.jifenbar:setPercent(self.m_info.battle_info.score / MatchingConfig[rank].score * 100)

	csNode.score:setString(string.format("%0.2f",battle.pingfen))
	csNode.kas:setString(string.format("%d/%d/%d",battle.kill,battle.assist,battle.city_num))
	csNode.battletimes:setString(battle.match_num)
	csNode.conwin:setString(battle.max_continue_win_num)
	csNode.win:setString(battle.win_num)
	csNode.junzhuwin:setString(battle.master_win_num)
	local maxrankLevel = battle.max_rank_level > 0 and battle.max_rank_level or 1
	csNode.max_maquis:setString(MatchingConfig[maxrankLevel].rank_level)
	csNode.winlv:setString(string.format("%0.2f%%",100 * battle.win_num / (battle.match_num <= 0 and 1 or battle.match_num)))
	local serverName = LoginManager:getServerAreaName(self.m_info.server_id,true)
	csNode.qufuname:setString(serverName)
end

function This:refreshRelationBtn()
	local isBlackFriend = false
    self.m_isMyFriend = nil
	if not self.m_isMy then
		local friendNode = 	FriendManager:getFriend(self.m_info.id)
		if friendNode and friendNode:isFriend() then
			self.m_isMyFriend = friendNode
		end

		if self.m_isMyFriend then
			isBlackFriend = self.m_isMyFriend:isBlackFriend()
			if not isBlackFriend then
				self.m_csNode.other.addfriend.text:setString(Serialize[62292])
			end
		else
			self.m_csNode.other.addfriend.text:setString(Serialize[62293])
		end
	end
	self.m_csNode.other.addfriend:setVisible(not isBlackFriend)
	self.m_csNode.other.addblacklist:setVisible(not isBlackFriend)
end

function This:initTouchEvent()
	self.m_csNode.own.changename:setTouchEnded(function()
		local panel = g_scene:openDialog("List.QueueRenamePanel",{
			defaultStr = "",
			title = Serialize[62294],
			desc = Serialize[62294],
			maxLength = 7,
			cost = 200,
		})
		panel:setRenameHandler(function(str)
			GamePlayer:onModifyPlayerNameRequest(str)
		end)
	end)
	self.m_csNode.own.changeicon:setTouchEnded(function()
		g_scene:openDialog("Player.ChangeIconPanel")
	end)
	self.m_csNode.other.checkherolist:setTouchEnded(function()
		g_scene:openDialog("Player.RoleHeroPanel",self.m_info.hero_list)
	end)
	self.m_csNode.other.addblacklist:setTouchEnded(function()
		FriendManager:onAddBlackFriendRequest(self.m_info.id)
	end)
	self.m_csNode.other.addfriend:setTouchEnded(function()
		if not self.m_isMyFriend or self.m_isMyFriend:isBlackFriend() then
			FriendManager:onApplyAddFriendRequest(self.m_info.id)
		else
			local friendPanel = g_scene:getDialog("FriendPanel")
			if friendPanel then
				g_scene:showTips(Serialize[62295])
				return
			end
			g_scene:openDialog("Friend.FriendPanel", {tabIndex = "friend", defaultFriendId = self.m_isMyFriend:getId()})
			-- g_scene:openDialog("Friend.FriendPanel")
		end
	end)	
	self.m_csNode.chenghao1:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text=Serialize["RoleInformation_evaluate"],node = self.m_csNode.chenghao1})
	end)
	self.m_csNode.chenghao2:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text=Serialize["RoleInformation_evaluate"],node = self.m_csNode.chenghao2})
	end)
	self.m_csNode.chenghao3:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text=Serialize["RoleInformation_evaluate"],node = self.m_csNode.chenghao3})
	end)
	self.m_csNode.chenghao4:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text=Serialize["RoleInformation_evaluate"],node = self.m_csNode.chenghao4})
	end)
end

function This:onEnter()
	super.onEnter(self)
	GamePlayer:addObserver(self)
	FriendManager:addObserver(self)
	self:initTouchEvent()
end

function This:onExit()
	super.onExit(self)
	GamePlayer:removeObserver(self)
	FriendManager:removeObserver(self)
end

function This:onPlayerAttrChanged(attrId,old,value)
	if attrId == PlayerAttr.name then
		self.m_info.name = value
		self.m_csNode.tips:setString(value)
	end
	if attrId == PlayerAttr.icon then
		self.m_info.icon = value
		GameUtil:setPlayerFeltNodeResource(self.m_csNode.icon, value)
	end
end

function This:onAddBlackFriend( friend )
	if not self.m_isMy and self.m_info and self.m_info.id == friend:getId() then
		self:refreshRelationBtn()
    end
end

function This:onRemoveBlackFriend( friend )
	if not self.m_isMy and self.m_info and self.m_info.id == friend:getId() then
		self:refreshRelationBtn()
    end
end

function This:onRemoveFriend( friend )
	if not self.m_isMy and self.m_info and self.m_info.id == friend:getId() then
		self:refreshRelationBtn()
    end 
end

function This:onAddFriend( friend )
	if not self.m_isMy and self.m_info and self.m_info.id == friend:getId() then
		self:refreshRelationBtn()
    end
end

--endregion
