

local This , super = class(...,"Dialog")
local ActivityManager = require("Activity.ActivityManager")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local ItemNode = require("UI.ItemNode")
local EquipManager = require("Equip.EquipManager")
local PopDialogManager = require("UI.PopDialogManager")


FailedJumpType = {
	shengji = 1,
	shengxing = 2,
	peiyang = 3,
	shouchong = 4,
}

local SceneFullPath = {
	["MenuScene"] = "Home.MenuScene",
	["WorldScene"] = "World.WorldScene",
}

function This:initFailedGuide()
	local csNode = self.m_csNode
	local targetScene = self:getJumpToTargetScene(false)

	if not csNode then
		return
	end

	--武将升级
	if csNode.shengji then
		csNode.shengji.yuanbg:setTouchEnded(function()
	       	self:onFailedJump(FailedJumpType.shengji)
	    end)
	end

	--武将升星
	if csNode.shengxing then
		csNode.shengxing.yuanbg:setTouchEnded(function()
	       	self:onFailedJump(FailedJumpType.shengxing)     	
	    end)
	end

	--装备培养
	if csNode.peiyang then
		csNode.peiyang.yuanbg:setTouchEnded(function()
	       	self:onFailedJump(FailedJumpType.peiyang)      	
	    end)	
	end

    --首充
    local shouchong = csNode.shouchong
    if shouchong then
	    local isFirstPay = not ActivityManager:isFirstPayFinish()
	    shouchong:setVisible(isFirstPay)

	    local node = ItemNode:create(10012, 1)
		node:setAnchorPoint(0.5, 0.5)
		shouchong.item:addChild(node)

		shouchong.bg:setTouch({
	  		onEnded = function()
	    		self:onFailedJump(FailedJumpType.shouchong) 
	  		end,
	    	isScale = false
	    })
	end
end

function This:onFailedJump(jumpType)
	local isCanJump = false
	local tips = ""
	local uiName = ""
	local uiArg = nil

	if jumpType == FailedJumpType.shengji then
		local hero, text = self:getJumpTargetHeroForLv()
		isCanJump = hero and true or false
		if hero then
			uiName = "Hero.HeroPanel"
			uiArg = {heroId = hero:getId(), tab="equip"}
		else
			tips = text
		end

	elseif jumpType == FailedJumpType.shengxing then
		local hero, text = self:getJumpTargetHeroForStar()
		isCanJump = hero and true or false
		if hero then
			uiName = "Hero.HeroPanel"
			uiArg = {heroId = hero:getId(), tab="equip"}
		else
			tips = text
		end

	elseif jumpType == FailedJumpType.peiyang then
		local hero, text = self:getJumpTargetHeroForEquip()
		isCanJump = hero and true or false
		if hero then
			uiName = "Hero.HeroPanel"
			uiArg = {heroId = hero:getId(), tab="equip"}
		else
			tips = text
		end

	elseif jumpType == FailedJumpType.shouchong then
		isCanJump = true
		uiName = "Activity.ActivitySeparatedPanel"
		uiArg = {queue = ActivityManager:getQueueById(1)}

	end	

	if not isCanJump then
		print("onFailedJump isCanJump == false")
		if tips ~= "" then
			g_scene:showTips(tips)
		end
		return
	end

	local switchCb = self:getSwitchSceneCallback()

	local jumpFunc = function (name, arg)
		print("Failed jump", name)
		PopDialogManager:addPopDialog({
			bCheck = false,
			name = name, 
			arg = arg,
			level = DialogLevel.kTopLevel
		})

		if type(switchCb) == "function" then
			switchCb()
		end
	end

	local targetName = self:getJumpToTargetScene()
	if targetName ~= g_scene:getName() then
		local path = SceneFullPath[targetName]
		cc.SceneManager:switchTo(path, function ()
			jumpFunc(uiName, uiArg)
		end)
	else
		jumpFunc(uiName, uiArg)
	end
end

function This:getJumpToTargetScene()
	local isInWorld = GamePlayer:isInWorld()
	return isInWorld and "WorldScene" or "MenuScene"
end

function This:getSwitchSceneCallback()
	return nil --根据需要重写
end

function This:getJumpTargetHeroForLv()
	local saintInfo = HeroManager:getSaintInfo()
	local _, heroList = saintInfo:getStrongestGroup()
	local roleLv = GamePlayer:getLevel()
	local selectHero = false
	local tips = ""

    for i, hero in ipairs(heroList) do
        if hero:getLevel() < roleLv then
        	selectHero = hero
        	break
        end
    end

    if not selectHero then
		local list = HeroManager:getAllHeroGotWithSort(nil, false)
		for _, hero in ipairs(list) do
			if hero:getLevel() < roleLv then
	        	selectHero = hero
	        	break
	        end
		end
		tips = Serialize[62307]
    end

    return selectHero, tips
end

function This:getJumpTargetHeroForStar()
	local saintInfo = HeroManager:getSaintInfo()
	local _, heroList = saintInfo:getStrongestGroup()
	local maxLv = 25
	local selectHero = false
	local tips = ""

    for i, hero in ipairs(heroList) do
        if hero:getTotalPhase() < maxLv then
        	selectHero = hero
        	break
        end
    end
    
    if not selectHero then
		local list = HeroManager:getAllHeroGotWithSort(nil, false)
		for _, hero in ipairs(list) do
			if hero:getTotalPhase() < maxLv then
	        	selectHero = hero
	        	break
	        end
		end
		tips = Serialize[62308]
    end

    return selectHero, tips
end

function This:getJumpTargetHeroForEquip()
	local getEquipTotalLev = function (hero)
		local lv = 0
		for i = EquipPos.kWeapons , EquipPos.kHorse do
	        local equipUid = hero:getEquipUid(i)
	        if equipUid > 0 then
	            local equip = EquipManager:getEquip(equipUid)
	            if equip then
	                lv = lv + equip:getStrengthLv()
	            end
	        end
    	end
    	return lv
	end
	
	local saintInfo = HeroManager:getSaintInfo()
	local _, heroList = saintInfo:getStrongestGroup()
	local maxLv = 30
	local selectHero = false
	local tips = ""

    for i, hero in ipairs(heroList) do
        if getEquipTotalLev(hero) < maxLv then
        	selectHero = hero
        	break
        end
    end

    if not selectHero then
    	selectHero = heroList[1]
    end

    return selectHero, tips
end