﻿local This = class(...)
-- local GamePlayer = require("Player.GamePlayer")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local LoginManager = require("Login.LoginManager")
local Application = require("cc.Application")
local MatchManager = require("Match.MatchManager")
local SystemManager = require("System.SystemManager")
local MenuScene = require("Home.MenuScene")
local GuideManager = require("Guide.GuideManager")
local PopDialogManager = require("UI.PopDialogManager")
local SdkManager = require("Sdk.SdkManager")
local HeroManager = require("Hero.HeroManager")
local CityAppearanceConfig = require("Metadatas.CityAppearanceConfig")
local MarchAppearanceConfig = require("Metadatas.MarchAppearanceConfig")
local GameUtil = require("Util.GameUtil")

GamePlayerChanelId ={
	kChuangYu = 1001,--创娱
	k51Gamer = 1002,--51gamer
	kMuZhi = 1003,--拇指
}

kMaxLevel = 100
kMaxPower = 120
PlayerAttr = 
{
	id                 = 1,      
	icon               = 2,	--头像
	name               = 3,
	level              = 4,
	power              = 5,
	reserveExp         = 6,
	exp                = 7,
	silver             = 8,
	gold               = 9,
	diamond            = 10,
	map_id             = 11,
	zhangong           = 12,  --战功
	soul               = 13,  --魂
	power_buy_times    = 14,
	jingjibi           = 15,
	contributionbi     = 16,
	mana               = 17,
	role_create_time   = 18,
	supercompetebi     = 19,
	skill_reel         = 20,
	group_war_map_id   = 23,
	title              = 24, -- 称号
	effect             = 25, -- 头像特效
	jungong            = 26,--军功
	world_chat_num     = 27,--世界聊天条数
	mingwang           = 30,--名望值
	mingwang_level     = 31,--名望等级
	cityicon           = 32,--城池外形
	mounticon          = 33,--坐骑外形（跑马的皮肤）
	system_open_reward = 34,--已领取的系统预告奖励
	animalsoul         = 35,--兽魂
	user_code          = 36,--用户标识符
	equip_stone		   = 37,--寒天石
	runnestone_composite_return = 38,--符石熔炼值
	map_type 		   = 39,--1为争霸，2为皇帝战
	quiz_score		   = 40,--精炼积分
	expedition	   	   = 41,--遗迹币

}

PlayerEffectType ={
	icon             = 1,	--头像
	title            = 2, -- 称号
	effect           = 3, -- 头像特效
	cityicon        = 4,--城池外形
	mounticon       = 5,--坐骑外形（跑马的皮肤）
	kCount 			= 5,
}

PlayerMapType = {
	-- 0-99 争霸赛
	kKing = 100, -- 皇帝战
}

function This:init()
    self.m_account = nil
    self.m_data = {}
	for k , value in pairs(PlayerAttr) do
		self.m_data[value] = 0
	end
	self.m_data[PlayerAttr.level] = 1
    self.m_observers = {}
	self.m_timeDiff = 0
    self.m_itemUseTimes = {}
	self.m_isInit = false

	self:initListener()
	
	self.m_limitTime = 60
	self.m_escape = 0
	self.m_localTime = os.getmillisecond()
	self.m_isEnterGame = false
	self.m_escapeSecends = 0
	
	self.m_ownedIconList = {}
	self.m_ownedTitleList = {}
	self.m_ownedEffectList = {}
	self.m_cityEffectList = {}
	self.m_horseEffectList = {}

	self.m_openServerTime = 0 --开服时间
end

function This:initWithMsg(msg)
	for k , value in pairs(PlayerAttr) do
		self.m_data[value] = msg[k]
		if self.m_data[value] == nil then
			print(string.format("warnning:player attr %s init is nil",k))
			self.m_data[value] = 0
		end
		print(string.format("player attr %s init %s",k,self.m_data[value]))
	end

    SystemManager:setDiamondBuyInfo(msg)
end

function This:initListener()
    EventDispatcher:addListener(self,10200,self.onGetPlayerInfoReply)
    EventDispatcher:addListener(self,10201,self.onPlayerAttrChangedNotify)
    EventDispatcher:addListener(self,10202,self.onGameTimeSynNotify)
    EventDispatcher:addListener(self,11207,self.onBattleSettlementNotify)
    EventDispatcher:addListener(self,10209,self.onGetSeasonInfoReply)
    EventDispatcher:addListener(self,10210,self.onModifyPlayerNameReply)
    EventDispatcher:addListener(self,10212,self.onCodeSubmitReply)
	EventDispatcher:addListener(self,10213,self.onReceivedOwnedRoleWidgetsReply)
    EventDispatcher:addListener(self,10214,self.onModifyPlayerRoleImageReply)
	EventDispatcher:addListener(self,10215,self.onGetMingWangUpReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,10200)
	EventDispatcher:removeListener(self,10201)
	EventDispatcher:removeListener(self,10202)
	EventDispatcher:removeListener(self,11207)
	EventDispatcher:removeListener(self,11209)
	EventDispatcher:removeListener(self,10210)
	EventDispatcher:removeListener(self,10212)
	EventDispatcher:removeListener(self,10213)
    EventDispatcher:removeListener(self,10214)
	EventDispatcher:removeListener(self,10215)
	Application:removeFromUpdateList(self)
end

function This:addObserver(observer, callbacks)
    self.m_observers[observer] = callbacks or observer
end

function This:removeObserver(observer)
    self.m_observers[observer] = nil
end

function This:callObservers(handlerName, ...)
    for observer, callbacks in pairs(self.m_observers) do
        local callback = callbacks[handlerName]
        if callback then
           callback(observer, ...)
        end
    end
end

function This:onGetPlayerInfoRequest()
	Socket:send(10200,{})
end

function This:onGetPlayerInfoReply(msg)
	print('ZYY:GamePlayer.lua [start:115] :msg',msg)
	self:initWithMsg(msg)
	self.m_isInit = true
end

function This:getAccount(account)
	return self.m_account
end

function This:setAccount(account)
	self.m_account = account 
end

function This:isEnterGame()
	return self.m_isEnterGame
end

function This:isInit()
	return self.m_isInit
end

function This:clear()
	self.m_isInit = false
	self.m_isEnterGame = false
end

function This:isMergeServerStart()
	return self:getServerTime() >= (self.m_mergeServTime or 0)
end

-- 不开放返回true
function This:checkOpenlvLimit(configId,lv,showTip)
	local OpenLevelConfig = require("Metadatas.OpenLevelConfig")
	local level = self:getLevel()
	local temLv = lv or 0
	local openConfigItem = OpenLevelConfig[configId] or {}
	if openConfigItem then
		temLv = openConfigItem.open_level or 0
	end
	local res = level< temLv
	if showTip and res then
		g_scene:showTips(Serialize[62310]..temLv..Serialize[60394])
	end
	return res
end

-----------------------------------------------------

function This:convertEffectTypeToPlayerAttrType(effectType)
	local temPlayerAttr={
		[PlayerEffectType.icon] = PlayerAttr.icon ,
		[PlayerEffectType.title] = PlayerAttr.title ,
		[PlayerEffectType.effect] = PlayerAttr.effect ,
		[PlayerEffectType.cityicon] = PlayerAttr.cityicon ,
		[PlayerEffectType.mounticon] = PlayerAttr.mounticon ,
	}
	return temPlayerAttr[effectType]
end

function This:mofifyPlayerAttr(attrType,value)
	if attrType then
		local old = self.m_data[attrType]
		self.m_data[attrType] = value
		self:callObservers("onPlayerAttrChanged",attrType,old,value)
	end
end

function This:getPlayerAttrWithType(attrType,normalValue)
	if attrType then
		if self.m_data[attrType] and self.m_data[attrType] ~= 0 then 
			return self.m_data[attrType] 
		else
			return normalValue or 0
		end
	end
end

function This:setPlayerID(id)
	self:mofifyPlayerAttr(PlayerAttr.id,id)
	-- local old = self.m_data[PlayerAttr.id]
	-- self.m_data[PlayerAttr.id] = id
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.id,old,id)
end

function This:getPlayerID()
    return self.m_data[PlayerAttr.id]
end

function This:getExp()
	return self.m_data[PlayerAttr.exp]
end

function This:setExp(exp)
	self:mofifyPlayerAttr(PlayerAttr.exp,exp)
	-- local old = self.m_data[PlayerAttr.exp]
	-- self.m_data[PlayerAttr.exp] = exp
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.exp,old,exp)
end

function This:getPower()
	return self.m_data[PlayerAttr.power]
end

function This:setPower(power)
	self:mofifyPlayerAttr(PlayerAttr.power,power)	
	-- local old = self.m_data[PlayerAttr.power] 
	-- self.m_data[PlayerAttr.power] = power
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.power,old,power)
end

function This:getLevel()
    return self.m_data[PlayerAttr.level]
end

function This:setLevel(level)
	self:mofifyPlayerAttr(PlayerAttr.level,level)	
	self:callObservers("onPlayerLevelChanged",level)
	-- local old = self.m_data[PlayerAttr.level]
	-- self.m_data[PlayerAttr.level] = level
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.level,old,level)
end

function This:getRoleName()
	return self.m_data[PlayerAttr.name]
end

function This:setRoleName(name)
	self:mofifyPlayerAttr(PlayerAttr.name,name)	
	-- local old = self.m_data[PlayerAttr.name] 
	-- self.m_data[PlayerAttr.name] = name
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.name,old,name)
end

function This:getSpeed()
	return self.m_data[PlayerAttr.level] * 2
end

function This:getReserveExp()
	return self.m_data[PlayerAttr.reserveExp]
end

function This:setReserveExp(exp)
	self:mofifyPlayerAttr(PlayerAttr.reserveExp,exp)	
	-- local old = self.m_data[PlayerAttr.reserveExp] 
	-- self.m_data[PlayerAttr.reserveExp] = exp
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.reserveExp,old,reserveExp)
end

function This:getGold()
	return self.m_data[PlayerAttr.gold]
end

function This:setGold(gold)
	self:mofifyPlayerAttr(PlayerAttr.gold,gold)	
	-- local old = self.m_data[PlayerAttr.gold]
	-- self.m_data[PlayerAttr.gold] = gold
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.gold,old,gold)
end

function This:getStone()
	return self.m_data[PlayerAttr.equip_stone]
end

function This:getMonumentCoin()
	return self.m_data[PlayerAttr.expedition]
end

function This:getSilver()
	return self.m_data[PlayerAttr.silver]
end

function This:setSilver(silver)
	self:mofifyPlayerAttr(PlayerAttr.silver,silver)	
	-- local old = self.m_data[PlayerAttr.silver]
	-- self.m_data[PlayerAttr.silver] = silver
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.diamond,old,silver)
end

function This:getDiamond()
	return self.m_data[PlayerAttr.diamond]
end

function This:setDiamond(diamond)
	self:mofifyPlayerAttr(PlayerAttr.diamond,diamond)	
	-- local old = self.m_data[PlayerAttr.diamond]
	-- self.m_data[PlayerAttr.diamond] = diamond
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.diamond,old,diamond)
end

--名望，皮肤的属性在这里累加了
function This:getMingWangAttr()
	local GameUtil = require("Util.GameUtil")
	local RolePrestigeConfig = require("Metadatas.RolePrestigeConfig")
	local lv = self:getMingWangLv()
	local result = GameUtil:convertData(RolePrestigeConfig[lv].attribute)
	result = GameUtil:tableAdd(self:getPlayerSkinAttr(),result)

	local temAttr = require("GodBeast.GodBeastManager"):getCurentAtrr()
	result = GameUtil:tableAdd(temAttr,result)

	local lordEquipAttr = require("Awake.LordEquipManager"):getLordAttr()
	result = GameUtil:tableAdd(lordEquipAttr,result)
	return result
end

--计算坐骑外形,城池外形获得的全体武将加成
function This:getPlayerSkinAttr()
	local citySkinId = self:getPlayerAttrWithType(PlayerAttr.cityicon,1)
	local horseSkinId = self:getPlayerAttrWithType(PlayerAttr.mounticon,1)
	local idList = {citySkinId,horseSkinId}
	local configLists = {CityAppearanceConfig,MarchAppearanceConfig}
	local result = {}
	for index,configList in ipairs(configLists) do
		local itemId = idList[index]
		local configItem = configList[itemId]
		if configItem then
			local temAttr = GameUtil:convertData(configItem.attribute)
			result = GameUtil:tableAdd(temAttr,result)
		end
	end
	return result
end

function This:checkSkinLeftTime()
	local citySkinId = self:getPlayerAttrWithType(PlayerAttr.cityicon,1)
	local horseSkinId = self:getPlayerAttrWithType(PlayerAttr.mounticon,1)

	-- local citySkinId = self:getPlayerAttrWithType(PlayerAttr.icon,1)
	-- local horseSkinId = self:getPlayerAttrWithType(PlayerAttr.title,101)
	-- local citySkinId = self:getPlayerAttrWithType(PlayerAttr.effect,101)
	local function selectSkinIsHaveleftTime(id,effectList)
		local currentTime = self:getServerTime()
		local skinItem = effectList[id]
		if id==1 then
			return false
		elseif skinItem then
			local endTime = skinItem.end_time
			if endTime ~=-1 and endTime<currentTime then
				return true
			end
		end
		return false
	end
	local result = false
	if selectSkinIsHaveleftTime(citySkinId,self.m_cityEffectList) then
		self.m_cityEffectList[citySkinId] = nil
		result = true
		self:mofifyPlayerAttr(PlayerAttr.cityicon,1)
	end
	if selectSkinIsHaveleftTime(horseSkinId,self.m_horseEffectList) then
		self.m_horseEffectList[horseSkinId] = nil
		result = true
		self:mofifyPlayerAttr(PlayerAttr.mounticon,1)
	end
	
	if result then
		self:callObservers("onCheckSkinLeftTime")
		HeroManager:updateMingWang()
	end
end

function This:getMingWangValue()
	return self.m_data[PlayerAttr.mingwang] or 0
end

function This:getMingWangLv()
	return self.m_data[PlayerAttr.mingwang_level] or 0
end

function This:getIcon()
	return self.m_data[PlayerAttr.icon]
end

function This:setIcon(icon)
	self:mofifyPlayerAttr(PlayerAttr.icon,icon)	
	-- local old = self.m_data[PlayerAttr.icon]
	-- self.m_data[PlayerAttr.icon] = icon
	-- self:callObservers("onPlayerAttrChanged",PlayerAttr.icon,old,icon)
end

function This:getTitle() -- 称号
	return self.m_data[PlayerAttr.title]
end

function This:setTitle(titleId) -- 称号
	self:mofifyPlayerAttr(PlayerAttr.title,titleId)	
	-- local oldTitleId = self.m_data[PlayerAttr.title]
	-- self.m_data[PlayerAttr.title] = titleId
	-- self:callObservers("onPlayerAttrChanged", PlayerAttr.title, oldTitleId, titleId)
end

function This:getEffect() -- 头像特效
	return self.m_data[PlayerAttr.effect]
end

function This:setEffect(effectId) -- 头像特效
	self:mofifyPlayerAttr(PlayerAttr.effect,effectId)	
	-- local oldEffectId = self.m_data[PlayerAttr.effect]
	-- self.m_data[PlayerAttr.effect] = effectId
	-- self:callObservers("onPlayerAttrChanged", PlayerAttr.effect, oldEffectId, effectId)
end

function This:getUserCode()
	return self.m_data[PlayerAttr.user_code] or ""
end

function This:getMapId()
	return self.m_data[PlayerAttr.map_id]
end

function This:getMapType()
	return self.m_data[PlayerAttr.map_type]
end

function This:getLegionMapId()
	return self.m_data[PlayerAttr.group_war_map_id]
end

function This:getZhangong()
	return self.m_data[PlayerAttr.zhangong]
end

function This:getSoul()
	return self.m_data[PlayerAttr.soul]
end

function This:getRefinedScore() --精炼积分
	return self.m_data[PlayerAttr.quiz_score]
end

function This:getAnimalSoul()
	return self.m_data[PlayerAttr.animalsoul]
end

function This:getSkillReel()
	return self.m_data[PlayerAttr.skill_reel]
end

function This:getJingjibi()
	return self.m_data[PlayerAttr.jingjibi]
end

function This:getContributionbi()
	return self.m_data[PlayerAttr.contributionbi]
end

function This:getJunGong()
	return self.m_data[PlayerAttr.jungong]
end

function This:getPeakContestbi()
	return self.m_data[PlayerAttr.supercompetebi]
end

-- 在世界地图（争霸赛和皇帝战）
function This:isInWorld()
	return self:getMapId() > 0
end

-- 在争霸赛
function This:isInMatchMap()
	print( string.format("========== isInMatchMap %s %s", self:getMapId(), self:getMapType()) )
	return self:getMapId() > 0 and self:getMapType() ~= PlayerMapType.kKing
end

-- 在皇帝战
function This:isInKingMap()
	print( string.format("========== isInKingMap %s %s", self:getMapId(), self:getMapType()) )
	return self:getMapId() > 0 and self:getMapType() == PlayerMapType.kKing
end

function This:isInLegionMap()
	return self.m_data[PlayerAttr.group_war_map_id] > 0
end

function This:isEnoughDiamond(diamond)
    assert(diamond >= 0)
    return self.m_data[PlayerAttr.diamond] >= diamond
end

function This:isEnoughGold(gold)
    assert(gold >= 0)
    return self.m_data[PlayerAttr.gold] >= gold
end

function This:getPowBuyTime()
    return self.m_data[PlayerAttr.power_buy_times]
end

function This:setPowBuyTime(time)
    self.m_data[PlayerAttr.power_buy_times] = time
end

function This:getRoleCreateTime()
	return self.m_data[PlayerAttr.role_create_time] or 0
end

function This:setRoleCreateTime(time)
	self.m_data[PlayerAttr.role_create_time] = time
end


function This:getWorldChatNum()
	return self.m_data[PlayerAttr.world_chat_num]
end

function This:setWorldChatNum(num)
	self.m_data[PlayerAttr.world_chat_num] = num
end

function This:getTimeDay(time)
	local day = 0
	if time then
		local date = os.serverdate("*t", time + 3600 * 24)
		date.hour = 0
		date.min = 0
		date.sec = 0
		local time = os.time(date)
		local curTime = self:getServerTime()
		day = math.ceil((curTime - time) / (3600 * 24)) + 1
	end
	return day
end

--开服的天数
function This:getOpenServerDays()
	return self:getTimeDay(self:getOpenServerTime())
end

function This:getRoleCreateDays()
	return self:getTimeDay(self.m_data[PlayerAttr.role_create_time])
end

function This:getServMergeDays()
	return self:getTimeDay(self:getMergeServTime())
end

function This:getMergeServTime()
    return self.m_mergeServTime
end

function This:onPlayerAttrChangedNotify(msg)
    local valueList = msg.value
    for i, attrId in ipairs(msg.type) do
        local value = valueList[i]
        local old = self.m_data[attrId]
        if old ~= value then
            self.m_data[attrId] = value
            print(string.format("player attr update, attrId=%s, value=%s", attrId, value))
            self:callObservers("onPlayerAttrChanged",attrId,old,value)
			EventDispatcher:dispatcher("player_attr_changed",{attrId = attrId, old = old , new = value})
        end
		if attrId == PlayerAttr.map_id then
			if value == 0 then
				print("玩家%s离开大地图...",self.m_data[PlayerAttr.name])
			else
				print("玩家%s进入大地图...",self.m_data[PlayerAttr.name])
			end
		end
    end
end

function This:onGameTimeSynRequest()
	Socket:sendAndLock(10202,{})
end

function This:onGameTimeSynNotify(msg)
	local localtime = os.getmillisecond()
	local servertime = msg.cur_time
	self.m_mergeServTime = msg.merge_server_time
	self.m_timeDiff = servertime - localtime
	print(string.format("服务器时间：%s,本地时间差值：%d",os.date("%Y-%m-%d %H:%M:%S",servertime/1000),self.m_timeDiff))
end

function This:getServerTime()
	local diff = self.m_timeDiff or 0
	-- diff = (self.m_timeDiff or 0) + (16 * SECONDS_PER_HOUR + 13 * SECONDS_PER_MIN) * 1000
	return math.floor((self.m_localTime + diff)/1000)
end

function This:getMilliServerTime()
	local diff = self.m_timeDiff or 0
	return self.m_localTime + diff
end

function This:checkSettleBattle()
    if self.m_settleBattleData then
		if self:getMapId() <= 0  then
			self:openDialog("Match.MatchSettlePanel", self.m_settleBattleData)
			self.m_settleBattleData = nil
			return true
		end
    end
    return false
end

function This:onBattleSettlementNotify(msg)
	local matchReport = MatchManager:getBattleReports()
	local mapId = msg.map_result.map_id
	if matchReport[mapId] then
		print("--------------------------已经有该战局的结算信息！！！--------------------")
		matchReport[mapId] = msg.map_result
	else
		self.m_settleBattleData = msg.map_result
		matchReport[mapId] = msg.map_result
	end

    if not self.m_isEnterGame  then
        return
    end

	if not self.m_settleBattleData then
		cc.SceneManager:switchTo("Home.MenuScene")
		return
	end

	PopDialogManager:addPopDialog({
		name = "Match.MatchBattleEndPanel",
		arg = self.m_settleBattleData,
		level = DialogLevel.kTopLevel,
		bCheck = true
	})

    self.m_settleBattleData = nil   
	EventDispatcher:dispatcher("world_map_end")
end

function This:onGameDataLoadFinishNotify(msg)
	--cc.Application:reload("Full")
	--do return end

	Application:addToUpdateList(self)
	self.m_isEnterGame = true
	--[[if GuideManager:isInGuide() then
		local entryIndex = GuideManager:getProgress() 
		if self.m_settleBattleData then 
			cc.SceneManager:switchTo("Home.MenuScene")
			MenuScene:addDelayDialog({name = "Match.MatchSettlePanel", arg = self.m_settleBattleData})
		elseif entryIndex <= 11 or not self:isInWorld() then
			cc.SceneManager:switchTo("Home.MenuScene")
		else
			cc.SceneManager:switchTo("World.WorldScene")
		end
		return
	end--]]

	print("------------onGameDataLoadFinishNotify-------------")
	if self:isInMatchMap() then
		print("正在进入世界地图...")
		print(string.format("地图ID：%d",self:getMapId()))
		cc.SceneManager:switchTo("World.WorldScene")
	elseif self.m_settleBattleData then 
		print("------------玩家世界地图完结，正在加载结算界面-------------")
		cc.SceneManager:switchTo("Home.MenuScene")
		MenuScene:addDelayDialog({name = "Match.MatchBattleEndPanel", arg = self.m_settleBattleData})
		self.m_settleBattleData = nil
	elseif g_setting.isDebug and (g_setting.isWarTest or g_setting.isWarWithNetTest) then
		if g_setting.isWarTest then
			require("Map.GameMapUtil").getTestor(g_setting.isWarTest, true)
		else
			require("Map.GameMapUtil").enterMap(g_setting.isWarWithNetTest)
		end
	else
		print("------------正在进入主界面-------------")
		cc.SceneManager:switchTo("Home.MenuScene")
	end
	EventDispatcher:dispatcher("enter_game_server")
end
	
function This:onGetMingWangUpRequest()
	Socket:sendAndLock(10215)
end

function This:onGetMingWangUpReply( msg )
    if msg.err_code == 0 then
        self:onPlayerAttrChangedNotify({type ={PlayerAttr.mingwang_level},value = {msg.level or 0}})
        HeroManager:updateMingWang()
        self:callObservers("onGetMingWangUp")
        EventDispatcher:dispatcher("menu_scene_enter")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
-- 更改玩家头像
function This:onModifyPlayerIconRequest(iconId)
	self:onRefreshGamePlayerEffectRequest(PlayerEffectType.icon,iconId)
end

-- 更改玩家称号
function This:onModifyPlayerTitleRequest(titleId)
	self:onRefreshGamePlayerEffectRequest(PlayerEffectType.title,titleId)
end

-- 更改头像特效
function This:onModifyPlayerEffectRequest(effectId)
	self:onRefreshGamePlayerEffectRequest(PlayerEffectType.effect,effectId)
end

function This:onRefreshGamePlayerEffectRequest(type,effectId)
	Socket:sendAndLock(10214,{type = type, id = effectId})
end


function This:onModifyPlayerRoleImageReply(msg)
    if msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

	local function tryAddEffectId(ownedList, id,endTime)
		local isUsed = ownedList[id]
		ownedList[id] = {id = id, end_time = endTime}
		return isUsed
	end
	local effectType = msg.type
	local effectId = msg.id
	local temAttrType = self:convertEffectTypeToPlayerAttrType(effectType)
    -- if GameUtil:findItem({PlayerEffectType.icon,PlayerEffectType.title,PlayerEffectType.effect},effectType)   then
        tryAddEffectId(self:getPlayerEffectListWithType(effectType), effectId,msg.end_time)
        --local a = self:getPlayerEffectListWithType(effectType)
        self:mofifyPlayerAttr(temAttrType,effectId)
    -- elseif effectType == PlayerEffectType.cityicon or effectType == PlayerEffectType.mounticon then
    -- 	self:mofifyPlayerAttr(temAttrType,effectId)
    -- end
	self:callObservers("onModifyGamePlayerSkin",effectType,effectId)
    
end

function This:onGetOwnedRoleWidgetsRequest()
	Socket:sendAndLock(10213,{})
end

--刷新所有的头像皮肤
function This:onReceivedOwnedRoleWidgetsReply(msg)
	if msg.err_code == 0 then
		local function covertTabale(tablec)
			local result ={}
			for _,skinItem in pairs(tablec) do
				local id = skinItem.id
				result[id]= skinItem
			end
			return result
		end
		self.m_ownedIconList = covertTabale(msg.icon_list or {})
		self.m_ownedTitleList = covertTabale(msg.title_list or {})
		self.m_ownedEffectList = covertTabale(msg.effect_list or {})
		self.m_cityEffectList = covertTabale(msg.city_list or {})
		self.m_horseEffectList = covertTabale(msg.mounts_list or {})
		self:callObservers("onRefreshGamePlayerEffect")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--兑换码
function This:onSubmitCodeRequest(code)
	if code == "" then
		g_scene:handleErrorCode(10012)
	else
		local message = {}
		message.code = code
		Socket:sendAndLock(10212, message)
	end
end

-- 更改玩家名字
function This:onModifyPlayerNameRequest(name)
    Socket:sendAndLock(10210,{name = name})
end

function This:onModifyPlayerNameReply(msg)
	if msg.err_code == 0 then
		if self:getRoleName() ~= msg.name then
			self:setRoleName(msg.name)
		end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onCodeSubmitReply(msg)
	if msg.err_code == 0 then
		local itemList = {}
		for i, num in ipairs(msg.item_num_list) do
			local id = msg.item_id_list[i]
			if num > 0 and id and id > 0 then
				table.insert(itemList, {id = msg.item_id_list[i], count = num})
			end
		end

		if itemList and #itemList > 0 then
	        g_scene:showReward(itemList)
	    end
		self:callObservers("onCodeSubmit")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetSeasonInfoRequest()
	Socket:sendAndLock(10209,{})
end

function This:onGetSeasonInfoReply(msg)
	self:callObservers("onGetSeasonInfo",msg.info)
end

-- function This:filterInvalidWidget(widgetList)
-- 	local list = {}
-- 	local currentTime = self:getServerTime()
-- 	for _, widget in ipairs(widgetList) do
-- 		if widget.end_time == -1 or widget.end_time > currentTime then
-- 			table.insert(list, widget)
-- 		end
-- 	end
-- 	if #list == #widgetList then
-- 		return widgetList, false
-- 	else
-- 		return list, true
-- 	end
-- end

-- function This:refreshOwnedRoleWidgetList()
-- 	local newOwnedList, ok = self:filterInvalidWidget(self.m_ownedIconList)
-- 	if ok then
-- 		self.m_ownedIconList = newOwnedList
-- 		self:callObservers("onOwnedIconsChanged")
-- 	end

-- 	newOwnedList, ok = self:filterInvalidWidget(self.m_ownedIconList)
-- 	if ok then
-- 		self.m_ownedTitleList = newOwnedList
-- 		self:callObservers("onOwnedTitlesChanged")
-- 	end

-- 	newOwnedList, ok = self:filterInvalidWidget(self.m_ownedEffectList)
-- 	self.m_ownedEffectList = newOwnedList
-- 	if ok then
-- 		self.m_ownedTitleList = newOwnedList
-- 		self:callObservers("onOwnedEffectsChanged")
-- 	end
-- end

function This:getPlayerEffectListWithType(effectType)
	if effectType == PlayerEffectType.icon then
		return self.m_ownedIconList
	elseif effectType == PlayerEffectType.title then
		return self.m_ownedTitleList
	elseif effectType == PlayerEffectType.effect then
		return self.m_ownedEffectList
	elseif effectType == PlayerEffectType.cityicon then
		return self.m_cityEffectList
	elseif effectType == PlayerEffectType.mounticon then
		return self.m_horseEffectList
	end
end

-- function This:setOwnedIconList(iconList)
-- 	self.m_ownedIconList = self:filterInvalidWidget(iconList)
-- 	self:callObservers("onOwnedIconsChanged")
-- end

-- function This:getOwnedIconList()
-- 	return self:getPlayerEffectListWithType(PlayerEffectType.icon)
-- end

-- function This:setOwnedTitleList(titleList)
-- 	self.m_ownedTitleList = self:filterInvalidWidget(titleList)
-- 	self:callObservers("onOwnedTitlesChanged")
-- end

-- function This:getOwnedTitleList()
-- 	return self:getPlayerEffectListWithType(PlayerEffectType.title)
-- end

-- function This:setOwnedEffectList(effectList)
-- 	self.m_ownedEffectList = self:filterInvalidWidget(effectList)
-- 	self:callObservers("onOwnedEffectsChanged")
-- end

-- function This:getOwnedEffectList()
-- 	return self:getPlayerEffectListWithType(PlayerEffectType.effect)
-- end

----------------------------------------------------

function This:checkDiamond( cost, nocheck )
	if nocheck or not self:isEnoughDiamond(cost) then
		local panel = g_scene:openDialog("Territory.TerrCommonConfirmPanel")
		panel:setString(Serialize[61361])
	 	panel:setOkHandler(function()
             g_scene:openDialog("System.RechargeMainUI")
        end)
		return false
	end
	return true
end

function This:costDiamond(info)
	assert(info.cost)
	local handler = info.okHandler
	info.okHandler = function()
		if self:checkDiamond(info.cost) then
			handler()
		end
	end
	local panel = g_scene:openDialog("UI.CostConfirmPanel",info)
	panel:setLocalZOrder(g_scene:getMaxLocalZOrder() + 1)
end

function This:checkGold(info)
    assert(info)
    assert(info.nocheck or info.cost)

	if info.nocheck or not self:isEnoughGold(info.cost) then
		if self:getLevel() < kOpenGoldChangeLevel then
		    local warmingNote = Serialize[62311]
		    if info.nowarming then
				warmingNote = ""
		    end
			g_scene:showTips(warmingNote .. tostring(kOpenGoldChangeLevel) .. Serialize[62312])
		else
			if info.nowarming then
				g_scene:openDialog("System.GlodChangePanel", info)
			else
				local panel = g_scene:openDialog("Territory.TerrCommonConfirmPanel")
				panel:setString(Serialize[60773])
				panel:setOkHandler(function()
					g_scene:openDialog("System.GlodChangePanel", info)
				end)
			end
		end
		return false
	end
	return true
end

function This:costGold(info)
	assert(info.cost)
	local handler = info.okHandler
	info.okHandler = function()
		if self:checkGold(info) then
			handler()
		end
	end
	info.itemId = 101
	local panel = g_scene:openDialog("UI.CostConfirmPanel",info)
	panel:setLocalZOrder(g_scene:getMaxLocalZOrder() + 1)
end

function This:update(dt)
	self.m_localTime = os.getmillisecond()
	self.m_escape = self.m_escape + dt
	if self.m_escape >= self.m_limitTime then
		self:onGameTimeSynRequest()
		self.m_escape = self.m_escape - self.m_limitTime
	end
	self.m_escapeSecends = self.m_escapeSecends + dt
	if self.m_escapeSecends >= 1.0 then
		self.m_escapeSecends = self.m_escapeSecends - 1
		self:checkSkinLeftTime()
		local curTime = self:getServerTime()	
		local curTimeCalc = curTime % 86400
		if curTimeCalc == 57600 then
			print("*******************onDayPassed*********************")
			self:callObservers("onDayPassed")
			-- EventDispatcher:dispatcher("day_passed")
			EventDispatcher:dispatcher("day_passed_withoutnet")
		elseif curTimeCalc == 57631 then
			-- EventDispatcher:dispatcher("day_passed_delay_31s")
			-- EventDispatcher:dispatcher("day_passed_delay_31s_next")
		end
		if not self.m_isMergeStart and curTime > self.m_mergeServTime then
			print("********************onServerMerge*******************")
			self.m_isMergeStart = true
			self:callObservers("onMergeServer")
			EventDispatcher:dispatcher("merge_server")
		end
	end
end

--审核版本判断
function This:isAuditVersion()
	local ServerConfig = require("Net.ServerConfig")
	return os.isIos and ServerConfig.isAudit
end

function This:isIPhioneX()
	if g_setting.isDebug then
		return true
	end
	
	if os.isIos then
		return os.native_call("isIPhoneX")
	end
	return false
end

function This:checkChanelId(chanelId)
	if not chanelId then
		return false
	end
	-- if os.isIos then
	-- 	return true
	-- end
	local temChannelId = nil
	if not os.isWin32 and not os.isIosSimulator then
		temChannelId = os.native_call("getChannelId")
	end
	if os.isWin32 then
		return false
	end
	if temChannelId and tonumber(temChannelId) ==chanelId then
		return true
	end
	return false
end

function This:isPad()
	if os.isIos then
		return os.native_call("isPad")
	end
end

--开服时间戳
function This:setOpenServerTime(time)
	self.m_openServerTime = time
end

function This:getOpenServerTime()
	return self.m_openServerTime
end

--已领取的功能预告奖励
function This:getSystemOpenReward()
	return self.m_data[PlayerAttr.system_open_reward]
end