local ffi = require("ffi")
ffi.cdef[[
typedef struct FanRange{float minVX;float minVY;float maxVX;float maxVY;float radius;}FanRange;
]]

local rad = math.rad
local cos = math.cos
local sin = math.sin

local FanRange = ffi.typeof("FanRange")
ffi.metaclass("FanRange", {
    create = function (_, minAngle, maxAngle, radius)
        minAngle = rad(minAngle)
        maxAngle = rad(maxAngle)
        return FanRange(cos(minAngle), sin(minAngle), cos(maxAngle), sin(maxAngle), radius)
    end,
    testPoint = function (self, vx, vy)
        if vx * vx + vy * vy <= self.radius * self.radius then
            local cross1 = self.minVX * vy - self.minVY * vx
            local cross2 = self.maxVX * vy - self.maxVY * vx
            if cross1 >= 0 and cross2 <= 0 then
                return 1
            end
            cross1 = self.minVX * vy + self.minVY * vx
            cross2 = self.maxVX * vy + self.maxVY * vx
            if cross1 >= 0 and cross2 <= 0 then
                return -1
            end
        end
        return false
    end,
})