-- @Author: superm
-- @Date:   2017-12-15 14:42:58
-- @Last Modified by:   superm
-- @Last Modified time: 2018-02-26 19:09:44
-- @File Name: RankBaseUI.lua
-- @Description: 

local This, super = class(..., "NodeUI")
This.fileName = "MenuScene/NodeUI/RankingListMainBg.csb"

local RankListLayer = require("Rank.RankListLayer")
local RankOfficeItemView = require("Rank.RankOfficeItemView")
local RankLevelItemView = require("Rank.RankLevelItemView")
local RankOverlordItemView = require("Rank.RankOverlordItemView")
local RankManager = require("Rank.RankManager")

This.titleTextList = { Serialize[64046], Serialize[64047], Serialize[64048], Serialize[64049]}

RankSubType = {
	kAll = 1,
	kThis = 2,
}

function This:initData( rtype )
	self.m_curType = rtype
	self.m_inited = true
end

function This:initNode()
    self.m_listLayer = RankListLayer:create({
        itemWidth = 748,
        itemHeight = 57,
        spaceX = 0,
        spaceY = 2,
        scrollView = self.scrollView,
        scrollHandler = function ( data, event )
        	if event == 1 and not self.m_locked then
        		self.m_locked = true
				RankManager:onGetRankInfoRequest(self.m_curType, self.m_curSubType == RankSubType.kAll)
				return true
        	end
        	return false
        end
    })
    
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self:initTabsInfo()
end

function This:initTabsInfo()
    for _, rankSubType in pairs(RankSubType) do
		local button = self["tab" .. rankSubType]
	    self:addTabButton(tostring(rankSubType), button, function()
	    	if self.m_curSubType == rankSubType then
	    		return false
	    	end
	    	self.m_listLayer:clearNode(true)
	        self.m_curSubType = rankSubType
			RankManager:onGetRankInfoRequest(self.m_curType, self.m_curSubType == RankSubType.kAll)
	        return true
	    end)
	end
end

function This:requestData()
	self:selectTabButton(tostring(RankSubType.kAll))
end

function This:onEnter()
    super.onEnter(self)
	RankManager:addObserver(self)
	self:requestData()
end

function This:onExit()
	RankManager:removeObserver(self)
    super.onExit(self)
end


function This:refresh()
	if self.m_inited then
		self:refreshTitleView(self.m_curType)
		self:refreshSelfRankView(self.m_curType)
		self:refreshListView()
	end
end

function This:refreshTitleView( rankType )
    self.text3:setString(self.titleTextList[rankType] .. Serialize[64050])
	self.teptitle:setString(self.titleTextList[rankType] .. Serialize[64051])
end

function This:refreshSelfRankView( rankType )
	local selfRank, selfRankInfo = self:getSelfRank(), self:getSelfRankInfo()
	local parentNode = self.mine
	self.mine_txt:setVisible(selfRank)
	self.weishangbang:setVisible(not selfRank)
	for name, value in pairs(RankType) do
		local node = parentNode[name]
		if node then
			if rankType == value and selfRank then
    			node:setModel(selfRankInfo, selfRank)
    			node:setVisible(true)
				node:hideBg()
    		else
    			node:setVisible(false)
    		end
		elseif rankType == value and selfRank then
			node = self:getViewByType(rankType, selfRankInfo, selfRank )
			parentNode:addChild(node)
			parentNode[name] = node
			node:hideBg()
		end
	end
end

function This:refreshListView()
    self.m_listLayer:setDataList(self.m_data, function(item, index)
        return self:getViewByType(self.m_curType, item, index)
    end)
end

function This:getSelfRank()
    return self.m_myRank
end

function This:getSelfRankInfo()
	local selfRankInfo = self.m_data[self.m_myRank] or self.m_selfRankInfo
	if not selfRankInfo and self.m_myRank > 0 then
		RankManager:onGetSelfRankInfoRequest(self.m_curType, self.m_curSubType == RankSubType.kAll, self.m_myRank)
	end
	return selfRankInfo
end

function This:getViewByType( rankType, item, index )
	local switch = {  
	    [RankType.Office] = function( ... )  
	        return RankOfficeItemView:create( ... )  
	    end,  
	    [RankType.Overlord] = function( ... )   
	        return RankOverlordItemView:create( ... )  
	    end,  
	    [RankType.Level] = function( ... )  
	        return RankLevelItemView:create( ... )  
	    end,  
	}  
	  
	local fSwitch = switch[rankType] 
	  
	if fSwitch then 
	    return fSwitch(item, index) 
	end  
end

function This:onGetRankInfo(rankType, listData, myRank)
	if self.m_curType == rankType then
		if self.m_myRank ~= myRank then
			self.m_myRank = myRank
		end

		if self.m_data ~= listData then
			self.m_data = listData
			self:refresh()
		else
			self.m_listLayer:updateByDataList()
		end
		self.m_locked = false
	end
end

function This:onGetSelfRankInfo( msg )
	if self.m_curType == msg.type then
		self.m_selfRankInfo = msg.rank_list[1]
		self:refreshSelfRankView(self.m_curType)
	end
end