-- @Author: superm
-- @Date:   2017-12-23 22:04:17
-- @Last Modified by:   superm
-- @Last Modified time: 2018-05-10 11:56:54
-- @File Name: RankListLayer.lua
-- @Description: 

local This , super = class(...,"UI.UpdateLayer")
local EventDispatcher = require("EventDispatcher")

function This:init(info)
	self.m_scrollHandler = info.scrollHandler
	super.init(self, info)
end

function This:setDataList(dataList, createHandler, finishHandler, index)
	self:refreshContentSize(#dataList)
	self.m_dataList = dataList
	if not self.m_isInit then
		self:initItemView(dataList, createHandler, finishHandler, index)
	else
		self:updateItemView(dataList,createHandler)
		if finishHandler then
			finishHandler()
		end
	end
end

function This:initItemView(dataList, handler, finishHandler, index)
	local startIndex = 0
	local dataCount = #dataList

	if index and self.m_isVertical then
		local w , h = self.m_scrollView:getContentSize()
		local showCount = math.floor(h / (self.m_itemHeight + self.m_spaceY)) * self.m_column
		if index > showCount then
			if dataCount - index < showCount then
				self.m_scrollView:setInnerContainerPosition(0, 0)
				startIndex = dataCount - self.m_showCount
			else
				local column = math.floor((index-1) % self.m_column)
				local row = math.floor((index-1) / self.m_column)
				local y = row * (self.m_itemHeight + self.m_spaceY) + h
				self.m_scrollView:setInnerContainerPosition(0, y - self.m_height)
				if dataCount - index >= self.m_showCount then
					startIndex = index - column - 1
				else
					startIndex = dataCount - self.m_showCount
				end
			end
		end
	end

	local count = math.min(dataCount, self.m_showCount)
	for i = 1 , count do
		self.m_timer:setTimer((i-1) * self.m_inteval,function()
			local view = handler(dataList[startIndex + i],startIndex + i)
			if not view then
				return
			end
			local x , y = self:getPosByIndex(startIndex + i)
			view:setPosition(x,y)
			view:setAnchorPoint(0.5,0.5)
			view.index = startIndex + i
			view:setName("updateItem"..i)
			self:addChild(view)
			self.m_views[i] = view
            if i == count then
		        self.m_timer:setTimer(i * self.m_inteval, function()
					if finishHandler then
						finishHandler()
					end
					EventDispatcher:dispatcher("list_finish")
		        end, "finishHandler")
	        end
		end,"init_item"..i)
	end

	self:scheduleUpdate()
	self.m_timer:setTimer(count * self.m_inteval, function()
		self.m_scrollView:addEventListener(function(_,event)
			if not self:onScrollEvent(event) then
				self:updateItemView(self.m_dataList,handler)
			end
		end)
	end)
	self.m_isInit = true
end

function This:onScrollEvent( event )
    if self.m_scrollHandler then
    	return self.m_scrollHandler(self.m_dataList, event)
    end
end

function This:updateByDataList(handler)
	if self.m_dataList then
		self:refreshContentSize(#self.m_dataList)
		self:updateItemView(self.m_dataList, handler)
	end
end