-- @Author: superm
-- @Date:   2017-11-29 14:13:55
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-07 17:30:53
-- @File Name: RankManager.lua
-- @Description: 

--region lua_filename.lua

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local LoginManager = require("Login.LoginManager")
local GamePlayer = require("Player.GamePlayer")
local MatchManager = require("Match.MatchManager")

kRankItemNum = 10
kRankItemStartIndex = 1
kRankItemEndIndex = 10
local kRankCacheExpiration = 600
local kRankAllServerId = 0

RankType = {
	Office = 1,
	Overlord = 2,
	Level = 3,
	Family = 4,
}

local WhoReqType = {
	RankManager = 0,
	MatchManager = 1,
	ThroneManager = 2,
}

function This:init()
	super.init(self)

	self.m_whoReq = 0
	self:clearInfo()
    self:initListener()
end

function This:clearInfo()
    self.m_expiration = 0
    self.m_requestType = RankType.Office
    self.m_requestServerId = kRankAllServe
    self.m_allRankItemList = {}
	self.m_curServerRankItemList = {}
end

function This:initListener()
	EventDispatcher:addListener(self, 11600, self.onGetRankInfoReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,11600)
end

function This:onGetAllRankInfoRequest()
	local server = LoginManager:getLoginServer()
	local servIdList = {kRankAllServerId, server:getId()}
	for _, rtype in pairs(RankType) do
		for i=1, 2 do
    		Socket:send(11600, {start_index = kRankItemStartIndex, end_index= kRankItemEndIndex, server_id = servIdList[i], type = rtype})
		end
    end
    self.m_whoReq = WhoReqType.RankManager
end

function This:onGetRankInfoRequest(rankType, showAll)
	if self.m_requestSelfRank then
		return
	end

	local container = showAll and self.m_allRankItemList or self.m_curServerRankItemList
	local server = LoginManager:getLoginServer()
	local curServerId = server:getId()
    self.m_requestType = rankType
    self.m_requestServerId = showAll and kRankAllServerId or curServerId
	local cache = container[rankType]
    if cache then
		Socket:sendAndLock(11600, {start_index = cache.endIndex + 1, end_index= cache.endIndex + kRankItemNum, server_id = self.m_requestServerId, type = rankType})
    else
	    Socket:sendAndLock(11600, {start_index = kRankItemStartIndex, end_index= kRankItemEndIndex, server_id = self.m_requestServerId, type = rankType})
	end
	self.m_whoReq = WhoReqType.RankManager
end

function This:onGetSelfRankInfoRequest(rankType, showAll, index)
	if self.m_requestSelfRank then
		return
	end
	self.m_requestSelfRank = true
    Socket:sendAndLock(11600, {start_index = index, end_index= index, server_id = self.m_requestServerId, type = rankType})
end

function This:onGetRankInfoReply( msg )
	if self.m_requestSelfRank then
		self.m_requestSelfRank = false
		self:notifyObservers("onGetSelfRankInfo", msg)
		return
	end

	if msg then
		if self.m_whoReq == WhoReqType.RankManager then
			local cache = self:cacheRankInfo( msg.type, msg.server_id == kRankAllServerId, msg.rank_list, msg.self_rank)
			if self.m_requestType == msg.type then
	        	self:notifyObservers("onGetRankInfo", msg.type, cache, msg.self_rank)
	        end

		elseif self.m_whoReq == WhoReqType.MatchManager then
			MatchManager:notifyObservers("onGetRankListInfo", msg)

		elseif self.m_whoReq == WhoReqType.ThroneManager then
			EventDispatcher:dispatcher("throne_rank_info", msg)

		end
	else
		g_scene:handleErrorCode("")
	end

	-- 重置回这个类型
	self.m_whoReq = WhoReqType.RankManager
end

function This:cacheRankInfo( rankType, showAll, data, selfIndex )
	local container = showAll and self.m_allRankItemList or self.m_curServerRankItemList

	local cache = container[rankType]
	if not cache then
		container[rankType] = {}
		cache = container[rankType]
	end

	local newStartIndex = cache.endIndex or kRankItemStartIndex
	local count = cache.endIndex or 0
	for i, v in ipairs(data) do
		count = count + 1
		cache[count] = v
	end
	container[rankType].startIndex = newStartIndex
	container[rankType].endIndex = count
	container[rankType].selfIndex = selfIndex
	return cache
end

function This:onGetRankListRequest(serverIndex)
	Socket:send(11600, {start_index = 1, end_index = 50,server_id = serverIndex, type = RankType.Office})
	self.m_whoReq = WhoReqType.MatchManager
end
MatchManager.onGetRankListRequest = This.onGetRankListRequest

--君王殿
function This:onThroneRankRequest()
 	local server = LoginManager:getLoginServer()
 	Socket:sendAndLock(11600, {start_index = 1, end_index = 30, server_id = server:getId(), type = RankType.Office})
 	self.m_whoReq = WhoReqType.ThroneManager
end