-- @Author: superm
-- @Date:   2017-11-29 11:50:31
-- @Last Modified by:   superm
-- @Last Modified time: 2018-01-08 10:00:28
-- @File Name: RankOfficeItemView.lua
-- @Description: 

local This, super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/RankingListOfficeItem.csb"

local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local MatchManager = require("Match.MatchManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local FamilyManager = require("Family.FamilyManager")

function This:init(info, index)
	self.m_rankItem = info
	self.m_index = index or 0
	local w , h = self.bg:getContentSize()
	self:setContentSize(w,h)
	self:setCascadeOpacityEnabled(true)
end

function This:initNode()
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self:setTouch({
		    onEnded = function()
                self:onViewClick()
            end,
         	isContainer = true,
        	isSwallow = false,
            isScale = false,
        })  
        self.m_inited = true
    end
end

function This:setModel(item, index)
	if not item and not index then
		self:refreshMyInfo()
		return 
    elseif not self.m_rankItem and self.m_selfInfo then
        self.m_index = index
        self.m_rankItem = item
		self:refreshMyInfo()
		return         
	end

	if self.m_index ~= index  then
		self.m_index = index or 0
		local oldData = self.m_rankItem
		self.m_rankItem = item
		self:refresh(oldData)
	end
end

function This:refresh(old)
	self:refreshIndexNode()
	local new = self.m_rankItem
	if not new then
		self:refreshMyInfo()
		return
	end

	self:refreshLevelInfo(new, old)
	self:refreshName(new)
	self:refreshFamilyInfo(new)
	self.bg:setVisible(self.m_index % 2 < 1)
end

function This:hideBg()
    self.bg:setVisible(false)
	self.m_touch.isContainer = false
end

function This:refreshIndexNode()
	local topVisible = self.m_index < 4 and self.m_index > 0
	self.rank:setVisible(not topVisible)
	self.top3:setVisible(topVisible)
	if topVisible then
		self.top3:setImage("MenuScene/top" .. self.m_index .. ".png")
	elseif self.m_index > 0 then
		self.rank:setString(tostring(self.m_index))
	else
		self.rank:setString("未上榜")
	end 
end

function This:refreshMyInfo()
	self.m_selfInfo = true
	self:refreshName()
	self:refreshLevelInfo(self.m_rankItem)
	self:refreshFamilyInfo({group_name = FamilyManager:getMyFamilyName()})
end

function This:refreshIcon( new, old )
	local icon = self.pic.icon
	if new then
		GameUtil:setPlayerIconNodeImage(icon, new.role_icon)
		GameUtil:setPlayerEffectNodeTo(icon, new.role_effect)
	else
		GameUtil:setPlayerIconNodeImage(icon, GamePlayer:getIcon())
		GameUtil:setPlayerEffectNodeTo(icon, GamePlayer:getEffect())
	end
end

function This:refreshName( info )
    self.name:setString( (info and info.role_name) or GamePlayer:getRoleName() )
end

function This:refreshLevelInfo(new, old)
	self:refreshIcon(new, old)
	local rankLev
	if new or old then
		local rankLev = math.floor(new.rank_first_key / 10000)
		if rankLev < 1 then
			rankLev = 1
		end
		if rankLev > kMaxMatchRankLv then
			rankLev = kMaxMatchRankLv
		end

		if old then
			if self.m_rankLev ~= rankLev then
				self.m_rankLev = rankLev
				self:refreshRankLevel( rankLev )
			end
		else
			self:refreshRankLevel( rankLev )
		end
	else
		self:refreshRankLevel( MatchManager:getRankLv() )
	end
end

function This:refreshRankLevel( rankLevel )
	self.marquis:setString(MatchingConfig[rankLevel].rank_level)
	self.marquisicon:setImage(RankIconImage[rankLevel] or RankIconImage[1])
end

function This:refreshFamilyInfo( info )
	if not info or not info.group_name or info.group_name == "" then
		self.family:setString("无")
	else
		self.family:setString(info.group_name)
	end
end

function This:onViewClick()
    if self.m_rankItem and not self.m_selfInfo then
    	g_scene:openDialog("Player.RoleInfoPanel",{id = self.m_rankItem.role_id})
    end
end
