-- @Author: superm
-- @Date:   2018-04-02 16:44:36
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-30 16:20:10
-- @File Name: RankPageListLayer.lua
-- @Description: 

local This , super = class(...,"cc.Node")
local Timer = require("Timer")
local EventDispatcher = require("EventDispatcher")

function This:init(info)
	self.m_itemWidth = info.itemWidth or 100
	self.m_itemHeight = info.itemHeight or 100
	self.m_spaceX = info.spaceX or 20
	self.m_spaceY = info.spaceY or 30
	self.m_column = info.column or 1
	self.m_inteval = info.inteval or 0.02
	self.m_scrollView = info.scrollView
	self.m_isVertical = info.isVertical == nil or false
	self.m_hintIndex = info.hint or false
	self.m_hintGetter = info.hintGetter or false

	self.m_layer = cc.LayerColor:create(cc.Color(255,255,0,0))
	self:addChild(self.m_layer)
	self.m_views = {}

	self:refreshContentSize(info.itemCount or 0)

	self.m_dataCounter = info.dataCounter
	self.m_viewCreator = info.viewCreator
	self.m_finishHandler = info.finishHandler
	self.m_scrollHandler = info.scrollHandler

	local w , h = self.m_scrollView:getContentSize()
	if self.m_isVertical then
		self.m_showCount = math.ceil(h / (self.m_itemHeight+self.m_spaceY) + 1) * self.m_column
	else
		self.m_showCount = math.ceil(w / (self.m_itemWidth+self.m_spaceX) + 1) * self.m_column
	end

	self.m_isInited = false
	self.m_refreshInterval = 0
	self.m_refreshEscape = 0
	self.m_startIndex = 1
	self.m_timer = Timer:create(self)
end

function This:getPosByIndex(index)
	if self.m_isVertical then
		local column = math.floor((index-1) % self.m_column)
		local row = math.floor((index-1) / self.m_column)
		local x = column * (self.m_itemWidth + self.m_spaceX) + (self.m_itemWidth + self.m_spaceX) / 2
		local y = row * (self.m_itemHeight + self.m_spaceY) + (self.m_itemHeight + self.m_spaceY) / 2
		return x , self.m_height - y
	else
		local column = math.floor((index-1) % self.m_column)
		local row = math.floor((index-1) / self.m_column)
		local y = column * (self.m_itemHeight + self.m_spaceY) + (self.m_itemHeight + self.m_spaceY) / 2
		local x = row * (self.m_itemWidth + self.m_spaceX) + (self.m_itemWidth + self.m_spaceX) / 2
		return x , self.m_height - y
	end
end

function This:initItemView()
	local initing = self.m_initing
	if initing then 
		self.m_needReload = true
		return 
	end

	self.m_initing = true
	local startIndex = 0
	local dataCount = self.m_dataCounter()

	if self.m_hintGetter and not self.m_hintIndex then
		self.m_hintIndex = self.m_hintGetter()
	end

	if self.m_hintIndex and self.m_isVertical then
		local w , h = self.m_scrollView:getContentSize()
		local showCount = math.floor(h / (self.m_itemHeight + self.m_spaceY)) * self.m_column

		if self.m_hintIndex > showCount then
			if dataCount - self.m_hintIndex < showCount then
				self.m_scrollView:setInnerContainerPosition(0, 0)
				startIndex = dataCount - self.m_showCount
				if startIndex < 0 then
					startIndex  = 0
				end
			else
				local column = math.floor((self.m_hintIndex-1) % self.m_column)
				local row = math.floor((self.m_hintIndex-1) / self.m_column)
				local y = row * (self.m_itemHeight + self.m_spaceY) + h
				self.m_scrollView:setInnerContainerPosition(0, y - self.m_height)
				if dataCount - self.m_hintIndex >= self.m_showCount then
					startIndex = self.m_hintIndex - column - 1
				else
					startIndex = dataCount - self.m_showCount
					if startIndex < 0 then
						startIndex  = 0
					end
				end
			end
		end
	end

	local count = math.min(dataCount, self.m_showCount)
	for i = 1 , count do
		self.m_timer:setTimer((i-1)*self.m_inteval, function()
			local index = startIndex + i
			local view = self.m_views[i]
			if not view then
				view = self.m_viewCreator(index)
				self:addChild(view)
			end
			local x , y = self:getPosByIndex(index)
			view:setPosition(x,y)
			view:setAnchorPoint(0.5,0.5)
			view.index = index
			view._startIndex = 1
			view:setName("updateItem"..i)
			self.m_views[i] = view
            if i == count then
		        self.m_timer:setTimer(i * self.m_inteval, function()
					if self.m_finishHandler then
						self.m_finishHandler()
					end
					EventDispatcher:dispatcher("list_finish")

					self.m_initing = false
					if self.m_needReload then
						self:updateItemView()
						if self.m_finishHandler then
							self.m_finishHandler()
						end
					end

		        end, "finishHandler")
	        end
		end,"init_item".. i)
	end
	self:scheduleUpdate()
	self.m_timer:setTimer(count * self.m_inteval, function()
		self.m_scrollView:addEventListener(function(_,event)
			if not self:onScrollEvent(event, self.m_startIndex + self.m_showCount * 2) then
				self:updateItemView()
			end
		end)
	end)
	self.m_isInited = true
end

function This:updateItemView()
	self.m_needReload = false
	
	local x , y = self.m_scrollView:getInnerContainerPosition()
	local w , h = self.m_scrollView:getInnerContainerSize()
	local w1 , h1 = self.m_scrollView:getContentSize()

	local startIndex ,endIndex = 0,0
	if self.m_isVertical then
		local pt , pb = h + y - h1, h + y 
		pt = math.max(1,pt)
		pb = math.min(h,pb)
		startIndex = math.floor(pt / (self.m_itemHeight + self.m_spaceY)) * self.m_column + 1
		endIndex = math.ceil(pb / (self.m_itemHeight + self.m_spaceY)) * self.m_column
	else
		local pt , pb = -x, -x + w1
		pt = math.max(1,pt)
		pb = math.min(w,pb)
		startIndex = math.floor(pt / (self.m_itemWidth + self.m_spaceX)) * self.m_column + 1
		endIndex = math.ceil(pb / (self.m_itemWidth + self.m_spaceX)) * self.m_column
	end
	self.m_startIndex = startIndex

	for i = startIndex , endIndex do
		local pos = i - startIndex + 1
		if i <= self.m_dataCounter() and not self.m_views[pos] then
			local view = self.m_viewCreator(i)
			view:setAnchorPoint(0.5,0.5)
			view:setVisible(false)
			view:setName("updateItem"..i)
			self:addChild(view)
			self.m_views[pos] = view
		end
	end

	for i = 1 , #self.m_views do
		local view = self.m_views[i]
		local index = startIndex - 1 + i
		view.index = index
		view._startIndex = startIndex
		if index <= endIndex and self.m_dataCounter() >= index then
			self.m_viewCreator(index, view)
			view:setVisible(true)
			local x , y = self:getPosByIndex(index)
			view:setPosition(x,y)
		else
			view:setVisible(false)
		end
	end
end

-- function This:getViewByIndex(index)
-- 	if index >= self.m_startIndex and index <= #self.m_dataList then
-- 		return self.m_views[index - self.m_startIndex + 1]
-- 	end
-- end

function This:refreshContentSize(itemCount)
	if self.m_itemCount ~= itemCount then
		self.m_itemCount = itemCount
		self.m_row = math.ceil(self.m_itemCount / self.m_column)
		local w , h = self.m_scrollView:getContentSize()

		if self.m_isVertical then
			self.m_width = (self.m_itemWidth + self.m_spaceX ) * self.m_column  
			self.m_height = (self.m_itemHeight + self.m_spaceY) * self.m_row
			if self.m_height < h then
				self.m_height = h
			end
		else
			self.m_width = (self.m_itemWidth + self.m_spaceX ) * self.m_row   
			self.m_height = (self.m_itemHeight + self.m_spaceY) * self.m_column
			if self.m_width < w then
				self.m_width = w
			end
		end

		self:setContentSize(self.m_width,self.m_height)	
		self.m_scrollView:setInnerContainerSize(self.m_width,self.m_height)	
		self.m_layer:setContentSize(self.m_width,self.m_height)
	end
end

function This:refreshPosition()
	for i , view in ipairs(self.m_views) do
		local x , y = self:getPosByIndex(view.index)
		view:setPosition(x,y)
	end
end

function This:setDataList()
	self:refreshContentSize(self.m_dataCounter())
	if not self.m_isInited then
		self:initItemView()
	else
		self:updateItemView()
		if self.m_finishHandler then
			self.m_finishHandler()
		end
	end
end

function This:swapNode(node1,node2)
	local key1 , key2
	for key , node in pairs(self.m_views) do
		if node == node1 then
			key1 = key
		elseif node == node2 then
			key2 = key
		end
	end
	if key1 and key2 then
		local x1 , y1 = self:getPosByIndex(node1.index)
		local x2 , y2 = self:getPosByIndex(node2.index)
		node1:setPosition(x2,y2)
		node2:setPosition(x1,y1)
		local tmp = node1.index
		node1.index = node2.index
		node2.index = tmp
		self.m_views[key1] = node2
		self.m_views[key2] = node1
	else
		assert(false)
	end
end

function This:clearNode(remove, cleanup)
	if not remove then
		for i , view in pairs(self.m_views) do
			view:setVisible(false)
		end
	else
		for i , view in pairs(self.m_views) do
			view:removeFromParent(cleanup)
		end
		self.m_views = {}
	end
	self:refreshContentSize(0)
end

function This:update(dt)
	self.m_timer:update(dt)
end

function This:isInited()
    return self.m_isInited
end

function This:onScrollEvent( event, index )
    if self.m_scrollHandler then
    	return self.m_scrollHandler(event, index)
    end
end

function This:updateByDataList()
	self:refreshContentSize(self.m_dataCounter())
	self:updateItemView()
end

function This:getAllViews()
	return self.m_views
end
--endregion
