--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local DepotManager = require("Depot.DepotManager")
local Application = require("cc.Application")
local GamePlayer = require("Player.GamePlayer")
local RecruitPriceConfig = require("Metadatas.RecruitPriceConfig")
local Socket = require("Net.Socket")
local HeroManager = require("Hero.HeroManager")
local EventDispatcher = require("EventDispatcher")
local CountryConfig = require("Metadatas.CountryConfig")
local PrivilegeManager = require("Vip.PrivilegeManager")
local GameUtil = require("Util.GameUtil")
local RecruitBoxConfig = require("Metadatas.RecruitBoxConfig")

ExtractType = 
{
	kSingleCommon = 1,
	kMultiCommon = 2,

	kSingleSpecial = 3,
	kMultiSpecial = 4,
}

kMaxFavoursValue = 500

function This:init()
	super.init(self)
	self.m_cdDeadline = 0
	self.m_progressTime = 0
	self.m_isUpdating = false
	self.m_escape = 0
	self.m_favours = 0
	self:resetBoxData()
	self:initListener()
end

function This:resetBoxData()
	self.m_boxRewardList ={}
	self.m_boxRewardHistory ={}
	self.m_boxRewardScore =0
	self.m_isCanReward = false
	self.m_minChallengeStarLv = 0

	for index, config in ipairs(RecruitBoxConfig) do
		self.m_boxRewardList[index] = {
                id = config.id, 
                state = RecruitManagerBoxState.kLock, 
                reward = config.reward, 
                integral = config.integral,
            }
	end
end

function This:initListener()
	EventDispatcher:addListener(self,11800,self.onGetRecruitInfoNotify)
	EventDispatcher:addListener(self,11801,self.onRecruitCommonReply)
	EventDispatcher:addListener(self,11802,self.onRecruitSpecialReply)
	EventDispatcher:addListener(self,11803,self.onRecruitFavoursReply)
	EventDispatcher:addListener(self, 11804, self.onGetBoxRewardReply)
	EventDispatcher:addListener(self,"vip_end",self.onVipEndNotify)
	EventDispatcher:addListener(self,"day_passed_withoutnet", self.onDayPassNotify)
end

function This:getRecruitmentOrder()
	return DepotManager:getItemCount(116)
end

function This:getCdDeadline()
	return self.m_cdDeadline
end

function This:setCdDeadline(deadline)
	local curTime = GamePlayer:getServerTime()
	if not self.m_isUpdating and deadline > curTime then
		self:startUpdate()
	elseif self.m_isUpdating and deadline <= curTime then
		self:endUpdate()
	end
	self.m_cdDeadline = deadline
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if self.m_cdDeadline > curTime then
		return self.m_cdDeadline - curTime
	end
	return 0
end

function This:getProgressTime()
	return self.m_progressTime 
end

function This:setProgressTime(time)
	self.m_progressTime = time
end

function This:isCanDiscount(recruitType)
	if recruitType == ExtractType.kMultiSpecial then
		local curTime = GamePlayer:getServerTime()
		local curDate = os.serverdate("*t",curTime)
		local preData = os.serverdate("*t",self.m_lastMutiSpeicalTime)
		if curDate.day ~= preData.day then
			local factor = PrivilegeManager:getPrivilegeNumBy(Privilege.kFiveRecruitCost)
			return factor > 0
		end
	end
	return false
end

function This:getRecruitCost(recruitType)
	if recruitType == ExtractType.kMultiSpecial then
		local priceList = RecruitPriceConfig[recruitType].price
		local itemId , cost =  priceList[1] , priceList[2]
		local curTime = GamePlayer:getServerTime()
		if not GameUtil:isSameDay(curTime,self.m_lastMutiSpeicalTime) then
			local factor = PrivilegeManager:getPrivilegeNumBy(Privilege.kFiveRecruitCost)
			return itemId, cost * (1-factor)
		else
			return itemId, cost
		end
	else
		local priceList = RecruitPriceConfig[recruitType].price
		return  priceList[1] , priceList[2]
	end
end
      
function This:startUpdate()
	Application:addToUpdateList(self)
	self.m_isUpdating = true
	self.m_escape = 0
end

function This:endUpdate()
	Application:removeFromUpdateList(self)
	self.m_isUpdating = false
	EventDispatcher:dispatcher("recruit_zhongjin_cd_changed")
end

function This:backUpHeroInfo()
	local heroList = HeroManager:getAllHero()
	self.m_heroGot = {}
	for i , hero in pairs(heroList) do
		self.m_heroGot[hero:getId()] = hero:isGot()
	end
end

function This:isCountryOpen(country)
	local config = CountryConfig[country]
	if GamePlayer:getLevel() >= config.level then
		return true
	end

	if GamePlayer:getRoleCreateDays() >= config.day then
		return true
	end

	local count = 0

	local heros = HeroManager:getAllHero()
	for i , hero in pairs(heros) do
		if hero:isGot() and hero:getCountry() == config.card_num[1] then
			count = count + 1
		end
	end
	if count >= config.card_num[2] then
		return true
	end
	return false
end

function This:getCountryHero()
	local list = {0,0,0,0}
	local heros = HeroManager:getAllHero()
	for i , hero in pairs(heros) do
		if hero:isGot() then
			local country = hero:getCountry()
			list[country] = list[country] + 1
		end
	end
	return list
end

function This:getFavours()
	return self.m_favours
end

--------------------------------------------------
--获取招募信息

function This:onGetRecruitInfoRequest()
	Socket:sendAndLock(11800, {})
end

function This:onGetRecruitInfoNotify(msg)
	self:setProgressTime(msg.buy_time)
	self:setCdDeadline(msg.refresh_interval)
	self.m_lastMutiSpeicalTime = msg.five_refresh_time
	self.m_favours = msg.favours
	self.m_boxRewardScore = msg.jifen
	self.m_boxRewardHistory = msg.jifen_reward_list
	self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = self:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,RecruitBoxConfig,self.m_boxRewardHistory,"integral")
	self:notifyObservers("onGetRecruitInfo")
	EventDispatcher:dispatcher("recruit_zhongjin_cd_changed")
end

--普通招募
function This:onRecruitCommonRequest(countryId,type)
	g_scene:setIsEnableShowTips(false)
	self:backUpHeroInfo()
	Socket:sendAndLock(11801,{country_id = countryId, type = type})
end

function This:onRecruitCommonReply(msg)
	g_scene:setIsEnableShowTips(true)
	if msg.err_code == 0 then
		local heroList = {}
		local recruitType = nil
		for i , info in ipairs(msg.card_list) do
			for j , heroId in ipairs(info.card_id) do
				table.insert(heroList,{heroId = heroId, isGot = self.m_heroGot[heroId] or false})
				recruitType = info.type
			end
		end

		for i = 1 , #heroList do
			for j = 1 , i - 1 do
				if heroList[j].heroId == heroList[i].heroId then
					heroList[i].isGot = true
				end
			end
		end

		local old = self.m_favours
		self.m_favours = msg.favours
		if self.m_favours > old then
			g_scene:showTips(string.format(Serialize[64070],self.m_favours - old))
		end
		self:notifyObservers("onRecruitCard",heroList,recruitType)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--重金招募
function This:onRecruitSpecialRequest(countryId,type)
	g_scene:setIsEnableShowTips(false)
	self:backUpHeroInfo()
	Socket:sendAndLock(11802,{country_id = countryId, type = type})
end

function This:onRecruitSpecialReply(msg)
	g_scene:setIsEnableShowTips(true)
	if msg.err_code == 0 then
		self:setProgressTime(msg.buy_time)
		self:setCdDeadline(msg.refresh_interval)
		self.m_lastMutiSpeicalTime = msg.five_refresh_time
		local heroList = {}
		local recruitType = nil
		for i , info in ipairs(msg.card_list) do
			for j , heroId in ipairs(info.card_id) do
				table.insert(heroList,{heroId = heroId, isGot = self.m_heroGot[heroId] or false})
				recruitType = info.type
			end
		end
		for i = 1 , #heroList do
			for j = 1 , i - 1 do
				if heroList[j].heroId == heroList[i].heroId then
					heroList[i].isGot = true
				end
			end
		end

		local old = self.m_favours
		self.m_favours = msg.favours
		if self.m_favours > old then
			g_scene:showTips(string.format(Serialize[64070],self.m_favours - old))
		end
		self.m_boxRewardScore = msg.jifen
		self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = self:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,RecruitBoxConfig,self.m_boxRewardHistory,"integral")
		self:notifyObservers("onRecruitCard",heroList,recruitType)
        EventDispatcher:dispatcher("recruit_zhongjin_cd_changed")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onRecruitFavoursRequest(countryId)
	Socket:sendAndLock(11803,{country_id = countryId})
end

function This:onRecruitFavoursReply(msg)
	if msg.err_code == 0 then
		self.m_favours = 0
		g_scene:showReward({{id = msg.drop_id, num = 1}})
		self:notifyObservers("onRecruitFavours",msg.drop_id)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

 	-- rewardTable 本地的奖励列表
    -- curentScore 当前积分
    -- rewardConfig 奖励的配置表,仅支持从1开始的顺序表
    -- rewardReceivedData 已经领取的奖励列表
function This:refreshRewardList(rewardTable,curentScore,rewardConfig,rewardReceivedData,rewardMarkStr)
   return GameUtil:refreshRewardList(rewardTable,curentScore,rewardConfig,rewardReceivedData,rewardMarkStr)
end

function This:onGetBoxRewardRequest(boxId)
	Socket:sendAndLock(11804,{id = boxId})
end

function This:getBoxHistory()
	return self.m_boxRewardHistory
end

function This:getBoxRewardScore()
	return self.m_boxRewardScore
end

function This:getIsCanReward()
	return self.m_isCanReward
end

function This:getMinBoxStarLv()
	return self.m_minChallengeStarLv
end

function This:getRewardList()
	return table.clone(self.m_boxRewardList)
end

function This:onGetBoxRewardReply( msg )
    if msg.err_code == 0 then
    	local boxId = msg.id
    	local boxItem = RecruitBoxConfig[boxId]
    	if boxItem then
	    	local rewardList = boxItem.reward
			g_scene:showReward(rewardList)
    	end

        self.m_boxRewardHistory = msg.jifen_reward_list
        self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = self:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,RecruitBoxConfig,self.m_boxRewardHistory,"integral")
        self:notifyObservers("onGetBoxReward")
        EventDispatcher:dispatcher("recruit_zhongjin_cd_changed")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onVipEndNotify()
	self:onGetRecruitInfoRequest()
end

function This:onDayPassNotify()
	self:resetBoxData()
	self:notifyObservers("onGetBoxReward")
    EventDispatcher:dispatcher("recruit_zhongjin_cd_changed")
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
		local leftTime = self:getLeftTime()
		self:notifyObservers("onRecruitCDUpdate",leftTime)
		if leftTime <= 0 then
			self:endUpdate()
		end
	end
end


--endregion
