local This = {}
local Rect = require("ffi").metaclass("Rect", This)
cc.Rect = Rect

function This.__tostring(rect)
    return string.format("x:%g, y:%g, width:%g, height:%g", rect.x, rect.y, rect.width, rect.height)
end

function This:isContainsPoint(point)
    if point.x >= self.x and point.x <= self.x + self.width and point.y >= self.y and point.y <= self.y + self.height then
        return true
    end
    return false
end

function This:isContainsXY(x, y)
    if x >= self.x and x <= self.x + self.width and y >= self.y and y <= self.y + self.height then
        return true
    end
    return false
end

function This:isIntersectRect(rect)
    if self.x <= rect.x + rect.width and self.y <= rect.y + rect.height and rect.x <= self.x + self.width and rect.y <= self.y + self.height then
        return true
    end
    return false
end

function This.__add(rect, value)
    if isInstance(value, Vec) then
        return Rect(rect.x + value.x, rect.y + value.y, rect.width, rect.height)
    end
end

function This:centerExtend(width, height)
    local halfWidth = width * 0.5
    local halfHeight = height * 0.5
    self.x = self.x - halfWidth
    self.y = self.y - halfHeight
    self.width = self.width + width
    self.height = self.height + height
    return self
end

function This:leftExtend(width, height)
    self.x = self.x - width
    self.width = self.width + width
    if height then
        self.y = self.y - height * 0.5
        self.height = self.height + height
    end
    return self
end

function This:centerScale(scale)
    self:centerExtend(-self.width * (1 - scale), -self.height * (1 - scale))
end

function This:merge(rect)
    local top1    = self.y + self.height
    local left1   = self.x
    local right1  = self.x + self.width
    local bottom1 = self.y
    
    local top2    = rect.y + self.height
    local left2   = rect.x
    local right2  = rect.x + self.width
    local bottom2 = rect.y
    
    self.x = math.min(left1, left2)
    self.y = math.min(bottom1, bottom2)
    self.width = math.max(right1, right2) - self.x
    self.height = math.max(top1, top2) - self.y

    return self
end

function This:getCenterXY()
    return self.x + self.width * 0.5, self.y + self.height * 0.5
end

function This:getMaxX()
    return self.x + self.width
end

function This:getMaxY()
    return self.y + self.height
end

function This:raycast(ax, ay, bx, by)
    local isIntersect, px, py = math.isSegmentIntersect(ax, ay, bx, by, self.x, self.y, self.x + self.width, self.y)
    if isIntersect then
        return px, py, 0, 1
    end

    isIntersect, px, py = math.isSegmentIntersect(ax, ay, bx, by, self.x + self.width, self.y, self.x + self.width, self.y + self.height)
    if isIntersect then
        return px, py, -1, 0
    end

    isIntersect, px, py = math.isSegmentIntersect(ax, ay, bx, by, self.x + self.width, self.y + self.height, self.x, self.y + self.height)
    if isIntersect then
        return px, py, 0, -1
    end

    isIntersect, px, py = math.isSegmentIntersect(ax, ay, bx, by, self.x, self.y + self.height, self.x, self.y)
    if isIntersect then
        return px, py, 1, 0
    end
    return false
end

return Rect