local This, super= class(..., "Base.BaseModel")

local redDotTreeSetting = {
    shop = {
        jianghun = {
            jhRestTimes = false,
            jhRestFreeItem = false,
        },
        zhangong = {
            zgRestTimes = false,
            zgRestFreeItem = false,
        },
    },
    recruit = {
        recruitCanUseItem = {
            recruitItem1 = false,
            recruitItem5 = false,
        },
        recruitHasFreeTimes = false,
    },
    stage = {
        stageReward = false,
        stageVisit = false,
    },
    jice = false,
    task = {
        taskMain = false,
        taskDaily = false,
        taskWeekly = {
            taskWeeklyReward = false,
            taskWeeklyBoxReward = false,
        }
    },
    friend = {
        friendUnreadChat = false,               
        friendNewRequest = false,               
    },
    vip = {
        vipActiveReward = false,
        vipDailyReward = false,
    },
    office = {
        officeActiveReward = false,
        officeDailyReward = false,
    },
    activity = false,
    diplomatic = false,
    email = false,
    prop = false,
    greatEvent = false,
    family = false,
    hero = false,
    zhenfa = false
}

---------------------------------------------------------------------------------
local function createRedDot(scale, parent)
    local node = cc.Sprite:create("Common/redPoint.png")
    if scale then
        node:setScale(scale, scale)
    end
    --node:playCSAnimation("Start", true)
    parent:addChild(node)
    return node
end

local function nodeAlignRightTop(parent, node, xSpace, ySpace)
    xSpace = xSpace or 0
    ySpace = ySpace or 0
    local width, height = parent:getContentSize()
    local scaleX, scaleY = parent:getScale()
    scaleX = scaleX or 1
    scaleY = scaleY or 1
    local rightX = width * scaleX - xSpace
    local topY = height  * scaleY - ySpace
    node:setPosition(rightX, topY)
end

function This:addRedDot(arg, parent, scale, offX, offY)
    local dotState
    if type(arg) == "string" then
        dotState = self:getNodeDot(arg)
    else
        dotState = arg
    end
    if dotState then
        local redDotNode = parent.m_redDotNode
        if not redDotNode then
            redDotNode = createRedDot(scale, parent)
            nodeAlignRightTop(parent, redDotNode, offX, offY)
            parent.m_redDotNode = redDotNode
        end
        return redDotNode
    elseif parent.m_redDotNode then
        parent.m_redDotNode:removeFromParent()
        parent.m_redDotNode = nil
    end
end

---------------------------------------------------------------------------------

local treeNodes
local function createRedDotNode(name, parent, value)
    local node = {name = name, dot = value, parent = parent, childs = {}}
    assert(not treeNodes[name], name)
    treeNodes[name] = node
    return node
end

local function calcNodeState(node)
    for k,child in pairs(node.childs) do
        if child.dot then
            return true
        end
    end
    return false
end
local createRedDotTree
createRedDotTree = function(name, parent, settings)
    local nodes = createRedDotNode(name, parent)
    for name, setting in pairs(settings) do
        if type(setting) == "table" then
            nodes.childs[name] = createRedDotTree(name, nodes, setting)
        else
            nodes.childs[name] = createRedDotNode(name, nodes, setting)
        end
    end
    nodes.dot = calcNodeState(nodes)
    return nodes
end

local rootNode
local function initRedDotTree()
    treeNodes = {}
    rootNode = createRedDotTree("root", nil, redDotTreeSetting)
    redDotTreeSetting = nil
end
initRedDotTree()

local EventDispatcher = require("EventDispatcher")

function dispatchChidrenEvent( curNode )
    if curNode and curNode.childs then
        for k, child in pairs(curNode.childs) do
            EventDispatcher:addMsg("onRedDotChange_" .. child.name, child.dot)
        end
    end  
end

local function onNodeStateUpdate(node, params)
    -- dispatchEvent(node)
    -- local parent = node.parent
    -- while parent do
    --     local lastState = parent.dot
    --     parent.dot = calcNodeState(parent)
    --     if parent.dot ~= lastState then
    --         dispatchEvent(parent)
    --         parent = parent.parent
    --     else
    --         return
    --     end
    -- end

    local curNode, parent = node, node.parent
    if parent then
        This:notifyObservers("onSubRedDotChange", curNode)
        while parent.parent do
            local lastState = parent.dot
            parent.dot = calcNodeState(parent)
            print(parent.name)
            This:notifyObservers("onSubRedDotChange", parent)
            if parent.dot == lastState then
                return
            end
            curNode = parent
            parent = curNode.parent
        end
        print(curNode.name)
        print("[reddot]-------------------" .. curNode.name, tostring(curNode.dot))
        This:notifyObservers("onRedDotChange", curNode)
        if params and params.isAll then
            dispatchChidrenEvent( curNode )
        end
    end
end


local function valueIsEqual(lastValue, newValue)
    if #lastValue == #newValue then
        for i, v in ipairs(lastValue) do
            if newValue[i] ~= v then
                return false
            end
        end
        return true
    else
        return false
    end
end

local function updateNodeState(name, value, op, params)
    local node = treeNodes[name]
    if not node and params and params.parent then
        local parentNode = treeNodes[params.parent]
        if parentNode then
            node = createRedDotNode(name, parentNode)
        end
    elseif not node then
        print(name .. " not exist!!!!!!!")
        return
    end
    local valueType = type(value)
    local isUpdate = false
    if valueType == "table" then
        local lastValue = node.value or {}
        local lastState = node.dot
        if not op then
            node.dot = #value > 0
            node.value = value
            isUpdate = not valueIsEqual(lastValue, value)
        else
            local valueCount = #lastValue
            if valueCount > 0 then
                for _, v in ipairs(value) do
                    if valueCount > 0 then
                        for k, lv in ipairs(lastValue) do
                            if v == lv then
                                valueCount = valueCount - 1
                                table.remove(lastValue, k)
                                break
                            end
                        end
                    else
                        break
                    end
                end
            elseif op == "+" then
                node.value = lastValue
            end

            if op == "+" then
                for _, v in ipairs(value) do
                    table.insert(lastValue, v)
                end                    
            end

            node.dot = #lastValue > 0
            isUpdate = node.dot ~= lastState
        end
    else
        if valueType ~= "boolean" then
            value = nil
        end
        local lastState = node.dot
        node.dot = value
        isUpdate = node.dot ~= lastState
    end

    if isUpdate then
        onNodeStateUpdate(node, params)
    end
end

function This:getNodeDot(name)
    if treeNodes[name] then
        return treeNodes[name].dot
    else
        return nil
    end
end

function This:getNodeValue(name)
    return treeNodes[name].value
end

function This:print()
    print_r(treeNodes)
end

-- EventDispatcher:addListener(This, "common_reddot_multi", function(This, message)
--     print_r(message)
--     local operate = message.operate
--     local params = nil
--     if message.params then
--         params = {
--             isAll = message.params.isAll,
--             parent = message.params.parent,
--         }
--     end
--     message.params = nil
--     message.operate = nil
--     for name,value in pairs(message) do
--         updateNodeState(name, value, operate, params)
--     end
-- end)