--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"NodeUI")
local CityConfig = require("Metadatas.CityConfig")
local GameUtil = require("Util.GameUtil")
local ChatManager = require("Chat.ChatManager")
local ReportManager = require("Report.ReportManager")


local LastShareReportTime = 0

function This:init(report)
	super.init(self)
	self.m_report = report
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/Arenalogitem.csb")
	self:addChild(self.m_csNode)
	self:initBaseInfo()
end

function This:initCityInfo()
	local cityId = self.m_report:getCityId()
	if cityId > 0 then
		self.m_csNode.actionaddress:setString(string.format(Serialize[64087],CityConfig[cityId].name))
	end
	self.m_csNode.actionaddress:setVisible(cityId > 0)
end

function This:initBaseInfo()
	self:initCityInfo()
	self.m_csNode.defence.defense:setVisible(self.m_report:isDefender())
	self.m_csNode.attacker.actiontitle:setVisible(self.m_report:isAttacker())
	self.m_csNode.defence.txt:setVisible(self.m_report:isDefender())
	self.m_csNode.attacker.txt:setVisible(self.m_report:isAttacker())

	local time = self.m_report:getStartTime()
	self.m_csNode.time:setString(GameUtil:formatDate(Serialize[62245],time))
	
	local attInfo = self.m_report:getAttackerInfo()
	local defInfo = self.m_report:getDefencerInfo()

	local uprank = math.abs(tonumber(attInfo.uprankid or 0))
	local lowrank = math.abs(tonumber(defInfo.uprankid or 0))

	if self.m_report:getType() == ReportType.kArena then
		if self.m_report:isAttacker() then
			if uprank > 0 then
				self.m_csNode.paiming:setString(string.format(Serialize[62246],uprank))
			end
			self.m_csNode.paiming:setVisible(uprank > 0)
			self.m_csNode.jiantou:setVisible(uprank > 0)
			self.m_csNode.jiantou2:setVisible(false)
		elseif self.m_report:isDefender() then
			if lowrank > 0 then
				self.m_csNode.paiming:setString(string.format(Serialize[62247],lowrank))
			end
			self.m_csNode.paiming:setVisible(lowrank > 0)
			self.m_csNode.jiantou2:setVisible(lowrank > 0)
			self.m_csNode.jiantou:setVisible(false)
		end
	else
		self.m_csNode.jiantou:setVisible(false)
		self.m_csNode.jiantou2:setVisible(false)
		self.m_csNode.paiming:setVisible(false)
	end

	self.m_csNode.attacker.actionname:setString(attInfo.name)
	if not attInfo.group_name or attInfo.group_name == "" then
		self.m_csNode.attacker.name:setString(Serialize[62248])
	else
		self.m_csNode.attacker.name:setString(attInfo.group_name)
	end

	self.m_csNode.attacker.number:setString(string.format("%d/%d",attInfo.resultHp,attInfo.initHp))
	self.m_csNode.attacker.actionbar:setPercent(attInfo.resultHp / attInfo.initHp * 100)

	self.m_csNode.defence.name:setString(defInfo.name)
	if not defInfo.group_name or defInfo.group_name == "" then
		self.m_csNode.defence.tongmengname:setString(Serialize[62248])
	else
		self.m_csNode.defence.tongmengname:setString(defInfo.group_name)
	end

	self.m_csNode.defence.Text2:setString(string.format("%d/%d",defInfo.resultHp,defInfo.initHp))
	self.m_csNode.defence.actionbar2:setPercent(defInfo.resultHp / defInfo.initHp * 100)

	if self.m_report:isAttacker() then
		self.m_csNode.wintitle:setVisible(self.m_report:isAttackerVictory())
		self.m_csNode.falsetitle:setVisible(not self.m_report:isAttackerVictory())
	elseif self.m_report:isDefender() then
		self.m_csNode.wintitle:setVisible(self.m_report:isDefencerVictory())
		self.m_csNode.falsetitle:setVisible(not self.m_report:isDefencerVictory())
	else
		self.m_csNode.wintitle:setVisible(self.m_report:isAttackerVictory())
		self.m_csNode.falsetitle:setVisible(not self.m_report:isAttackerVictory())
	end
	self:refresh()
end

function This:refresh()
	self.m_csNode.weidu:setVisible(not ReportManager:isRead(self.m_report:getId()))
end

function This:onShareReport()
	local ret , msg = ChatManager:isCanChatInWorld()
	if not ret then
		g_scene:showTips(msg)
		return 
	end
	if os.time() - LastShareReportTime >= 5 then
		local panel = g_scene:openDialog("UI.ConfirmPanel")
		panel:setString(Serialize[62201])
		panel:setOkHandler(function()
			ChatManager:onShareBattleReportRequest(self.m_report:getId())
			LastShareReportTime = os.time()
			ReportManager:setIsRead(self.m_report:getId(),true)
		end)
	else
		g_scene:showtTips(string.format(Serialize[62202],os.time() - LastShareReportTime))
	end
end

function This:initTouchEvent()
	self.m_csNode.videotape:setTouch({
		onEnded = function()
			ReportManager:onGetReportDetailRequest(self.m_report:getId())
			ReportManager:setIsRead(self.m_report:getId(),true)
		end,
		isContainer = true
	})
	self.m_csNode.share:setTouch({
		onEnded = function()
			self:onShareReport()
		end,
		isContainer = true
	})
	self.m_csNode.weidu:setTouch({
		onEnded = function()
			ReportManager:setIsRead(self.m_report:getId(),true)
			self:refresh()
		end,
		isScale = false,
		isContainer = true
	})
end

function This:onEnter()
	super.onEnter(self)
	ChatManager:addObserver(self)
	ReportManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	ChatManager:removeObserver(self)
	ReportManager:removeObserver(self)
end

function This:setModel(report)
	self.m_report = report
	self:initBaseInfo()
end

function This:onGetReportDetail(msg)
	cc.SceneManager:switchTo("Report.ReportLoadingScene",nil,msg)
end


--endregion
