--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local ReportManager = require("Report.ReportManager")
local HeroManager = require("Hero.HeroManager")
local GameUtil = require("Util.GameUtil")

function This:create(info)
	local r = {}
	setmetatable(r,self)
	r:init(info)
	return r
end

function This:init(info)
	self.m_id = info.id
	self.m_result = info.state
	self.m_cityId = info.city_id
	self.m_type = info.type
	self.m_startTime = info.start_time
	self.m_endTime = info.end_time + 3 --(服务器延时导致+1秒)

	local attacker = info.attack
	self.m_attack = {
		id = attacker.id,
		level = attacker.level,
		name = attacker.name,
		icon = attacker.icon,
		speed = attacker.speed,
		initHp = attacker.force_num_init,
		resultHp = attacker.force_num_result,
		rewardList = attacker.reward_list,

	}
	for i , key in ipairs(info.attack.reserve_key) do
		self.m_attack[key] = info.attack.reserve_value[i]
	end

	self.m_attackTroops = {}
	for i , troop in pairs(attacker.card_list) do
		local t = {
			heroId = troop.card_type,
			level = troop.level,
			hp = troop.force_num,
			id = troop.card_id,
			ownerId = troop.owner_id,
			pos = troop.init_pos,
			anger = troop.init_anger,
            star = troop.star_level,
	        quality = troop.quality,
	        forceType = troop.force_type,
		}
		table.insert(self.m_attackTroops,t)
	end

	local defencer = info.defence
	self.m_defence = {
		id = defencer.id,
		level = defencer.level,
		name = AppText:getServerText(defencer.name),
		icon = defencer.icon,
		speed = defencer.speed,
		initHp = defencer.force_num_init,
		resultHp = defencer.force_num_result,
		rewardList = defencer.reward_list,
	}

	for i , key in ipairs(info.defence.reserve_key) do
		self.m_defence[key] = info.defence.reserve_value[i]
	end

	self.m_defenceTroops = {}
	for i , troop in pairs(defencer.card_list) do
		local t = {
			heroId = troop.card_type,
			level = troop.level,
			hp = troop.force_num,
			id = troop.card_id,
			ownerId = troop.owner_id,
			pos = troop.init_pos,
			anger = troop.init_anger,
            star = troop.star_level,
	        quality = troop.quality,
	        forceType = troop.force_type,
		}
		table.insert(self.m_defenceTroops,t)
	end	

	if self:isInFighting() then
		self:fightContinue()
	end
	self.m_escape = 0
end

function This:getId()
	return self.m_id
end

function This:getResult()
	return self.m_result
end

function This:getCityId()
	return self.m_cityId
end

function This:getType()
	return self.m_type
end

function This:getStartTime()
	return self.m_startTime
end

function This:getAttackerInfo()
	return self.m_attack
end

function This:getAttackerName()
	return self.m_attack.name
end

function This:getAttackerReward()
	return self.m_attack.rewardList
end

function This:getDefencerName()
	return self.m_defence.name
end

function This:getDefencerInfo()
	return self.m_defence
end

function This:getDefencerReward()
	return self.m_defence.rewardList
end

function This:getAttackerTroops()
	return self.m_attackTroops
end

function This:getDefencerTroops()
	return self.m_defenceTroops
end

function This:isAttackerVictory()
	return self.m_result == BattleResult.kAttackerVictory
end

function This:isDefencerVictory()
	return self.m_result ~= BattleResult.kAttackerVictory
end

function This:isInFighting()
	if self.m_type == ReportType.kPeakContest or self.m_type == ReportType.kLegion then 
		return false
	end
	local curTime = GamePlayer:getServerTime()
	return self.m_endTime > curTime 
end

function This:isMy()
	return self.m_attack.id == GamePlayer:getPlayerID() or self.m_defence.id == GamePlayer:getPlayerID()
end


function This:isAttacker()
	return self.m_attack.id == GamePlayer:getPlayerID()
end

function This:isDefender()
	return self.m_defence.id == GamePlayer:getPlayerID()
end

function This:getAttackerAttribute(name)
	return self.m_attack[name]
end

function This:getDefencerAttribute(name)
	return self.m_defence[name]
end

function This:getGotReserveExpCount()
	if GamePlayer:getPlayerID() == self.m_attack.id then
		if self.m_attack.rewardList.id == 105 then
			return self.m_attack.rewardList.num
		end
	elseif GamePlayer:getPlayerID() == self.m_defence.id then
		if self.m_defence.rewardList.id == 105 then
			return self.m_defence.rewardList.num
		end
	end
	return 0
end

function This:getMyReward()
	if GamePlayer:getPlayerID() == self.m_attack.id then
		return {self.m_attack.rewardList}
	elseif GamePlayer:getPlayerID() == self.m_defence.id then
		return {self.m_defence.rewardList}
	end
	return {}
end

function This:fightContinue()
	local str = GameUtil:formatTime(Serialize["H_M_S"], self.m_startTime)
	local curTime = GamePlayer:getServerTime()
	local str1 = GameUtil:formatTime(Serialize["H_M_S"], curTime)
	print(string.format("战报%s开始继续，开始时间:%s，本地当前时间:%s",GameUtil:int64ToString(self:getId()),str,str1))
	if not self.m_updating then
		self.m_updating = true
		Application:addToUpdateList(self)
	end
end

function This:fightEnded()
	local curTime = GamePlayer:getServerTime()
	local str = GameUtil:formatTime(Serialize["H_M_S"], curTime)
	print(string.format("战报%s结束，本地结束时间:%s",GameUtil:int64ToString(self:getId()),str))
	if self.m_updating then
		self.m_updating = false
		Application:removeFromUpdateList(self)
	end
	ReportManager:onFightEnded(self)
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		if not self:isInFighting() then
			self:fightEnded()
		end
		local leftTime = self.m_endTime - GamePlayer:getServerTime()
		print("更新报告：" .. self:getId() )
		print(string.format("城市%d战斗剩余时间%d", self.m_cityId, leftTime))
		self.m_escape = self.m_escape - 1
	end
end


--endregion
