
--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local Report = require("Report.Report")
local FightReport = require("Report.FightReport")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local WorldMapManager = require("World.WorldMapManager")
local GameUtil = require("Util.GameUtil")
local BattleDataManager = require("Battle.BattleDataManager")
local MenuScene = require("Home.MenuScene")
local ArenaManager = require("Arena.ArenaManager")
local GameMapManager = require("Map.GameMapManager")
local PeakContestManager = require("PeakContest.PeakContestManager")

ReportType = 
{
	kWorld = 1,
	kStage = 2,
	kArena = 3,
	kPeakContest = 4,  --巅峰竞技场

	kLegion = 7, --军团战
	kEmperor = 8, --皇帝战

	kRescue = 9, --勤王战
}

function This:init()
	super.init(self)
	self.m_reports = {}
	self.m_fightReport = {}
	self.m_cityReport = {}
	self:initListener()
	self.m_readList= cc.FileUtils:loadLuaFile("ReportReaded") or {}
end

function This:initListener()
	EventDispatcher:addListener(self, 10301, self.onGetReportInfoReply)
	EventDispatcher:addListener(self, 10302, self.onGetReportDetailReply)
	EventDispatcher:addListener(self, 10303, self.onGetNewReportNotify)
	EventDispatcher:addListener(self, 11207, self.onBattleSettlementNotify)
	EventDispatcher:addListener(self, 10304, self.onGetBattleSimpleReportReply)
	EventDispatcher:addListener(self, 12210, self.onGetArenaBattleReportReply)
	EventDispatcher:addListener(self, 13026, self.onGetLegionSimpleReportReply)
	EventDispatcher:addListener(self, 10305, self.onGetLegionDetailReportReply)
	EventDispatcher:addListener(self, 14022, self.onGetCityBattleReportReply)
	EventDispatcher:addListener(self, 14030, self.onGetRescueDetailReportReply)
end

function This:clear()
	self.m_reports = {}
	self.m_fightReport = {}
	self.m_readList = {}
	self.m_cityReport = {}
	cc.FileUtils:saveToLua(self.m_readList,"ReportReaded")
end

function This:addReport(info)
	if self.m_reports[info.id] then
		return self.m_reports[info.id]
	end
	local report = Report:create(info)
	self.m_reports[info.id] = report
	
	local cityId = info.city_id
	if cityId > 0 then
		if not self.m_cityReport[cityId] then
			self.m_cityReport[cityId] = {}
		end
		table.insert(self.m_cityReport[cityId],report)
	end
	self:notifyObservers("onAddReport",report)
	return report
end

function This:addFightReport(info)
	if self.m_fightReport[info.id] then
		return self.m_fightReport[info.id]
	end

	local report = FightReport:create(info)
	self.m_fightReport[report:getId()] = report
	return report 
end

function This:getReport(id)
	return self.m_reports[id] or self.m_fightReport[id]
end

function This:getReportInCity(cityId)
	return self.m_cityReport[cityId] or {}
end

function This:setIsRead(id,isRead)
	local key = GameUtil:int64ToString(id)
	if self.m_readList[key] ~= isRead then
		if isRead == false then
			self.m_readList[key] = nil
		elseif isRead == true then
			self.m_readList[key] = true
		end
		self:notifyObservers("onReportReaded",id)
		cc.FileUtils:saveToLua(self.m_readList,"ReportReaded")
	end
end

function This:isRead(id)
	local key = GameUtil:int64ToString(id)
	return self.m_readList[key] or false
end

function This:hasNoRead()
	for id , report in pairs(self.m_reports) do
		if report:isMy() and not report:isInFighting() and not self:isRead(id) and report:getType() ~= ReportType.kArena and report:getType() ~= ReportType.kPeakContest then
			return true
		end
	end
	return false
end

function This:isCityInFighting(cityId)
	local list = self:getReportInCity(cityId)
	for i , report in pairs(list) do
		if report:isInFighting() then
			return true
		end
	end
	return false
end

function This:getReportInfightingInCity(cityId)
	local list = self.m_cityReport[cityId] or {}
	local result = {}
	for i , report in pairs(list) do
		if report:isInFighting() then
			table.insert(result,report)
		end
	end
	table.sort(result,function(a , b)
		return a:getStartTime() > b:getStartTime()
	end)
	return result
end

function This:getReportList(reportType)
	reportType = ReportType.kWorld
	local result = {}
	for i , report in pairs(self.m_reports) do
		if report:getType() == reportType then
			table.insert(result,report)
		end
	end	
	table.sort(result,function(a,b)
		return a:getStartTime() > b:getStartTime()
	end)
	return result
end

function This:getReportInFighting()
	local result = {}
	for i , report in pairs(self.m_reports) do
		if report:isInFighting() then
			table.insert(result,report)
		end
	end
	table.sort(result,function(a,b)
		return a:getStartTime() > b:getStartTime()
	end)
	return result
end

function This:getReportEnded()
	local result = {}
	for i , report in pairs(self.m_reports) do
		if not report:isInFighting() then
			table.insert(result,report)
		end
	end
	table.sort(result,function(a,b)
		return a:getStartTime() < b:getStartTime()
	end)
	return result
end

function This:getReportList(selector)
	local result = {}
	for i , report in pairs(self.m_reports) do
		if selector then
			if selector(report) then
				table.insert(result,report)
			end
		else
			table.insert(result,report)
		end
	end
	table.sort(result,function(a,b)
		return a:getStartTime() > b:getStartTime()
	end)
	return result
end

function This:onGetReportInfoRequest(finishHandler)
	Socket:sendAndLock(10301,{start_index = 1, end_index = 112})
end

function This:onGetReportInfoReply(msg)
	self.m_reports = {}
	self.m_cityReport = {}
	for i , reportList in ipairs(msg.report_list) do
		for j , report in ipairs(reportList.report_list) do
			self:addReport(report)
		end
	end
end

function This:onGetReportDetailRequest(id)
	self.m_isRequesting = true
	Socket:sendAndLock(10302,{report_id = id})
end

function This:onGetReportDetailReply(msg)
	if not self.m_isRequesting then
		return
	end
	if msg.err_code == 0 then
		BattleDataManager:addObserver(self)
		self:notifyObservers("onGetReportDetail",msg)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
	self.m_isRequesting = nil
end

function This:onGetNewReportNotify(msg)
	for i , info in pairs(msg.report_list) do
		local report = self:addReport(info)
		print(string.format("新增战报，城市:%d",report:getCityId()))
		self:notifyObservers("onNewReport",report)
	end
end

--获取简单战报
function This:onGetBattleSimpleReportRequest(reportId)
	Socket:sendAndLock(10304,{report_id = reportId})
end

function This:onGetBattleSimpleReportReply(msg)
	if msg.err_code == 0 then
		local report = self:addReport(msg.report_list)
		self:notifyObservers("onGetBattleSimpleReport",report)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

--竞技场战报记录
function This:onGetArenaBattleReportRequest()
	Socket:sendAndLock(12210,{start_index = 1, end_index = 100})
end

function This:onGetArenaBattleReportReply(msg)
	if msg.err_code == 0 then
		local result = {}
		for i , info in ipairs(msg.report_list) do
            local reportInfo = info.report_list[1]
            if reportInfo then
			    local report = self:addReport(reportInfo)
			    table.insert(result,report)
            end
		end
		table.sort(result,function(a,b)
			return a:getStartTime() > b:getStartTime()
		end)
		self:notifyObservers("onGetArenaBattleReport",result,msg.start_index,msg.end_index)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetLegionSimpleReportRequest()
	Socket:sendAndLock(13026,{})
end

function This:onGetLegionSimpleReportReply(msg)
	for i , info in ipairs(msg.report_list) do
		-- 本地化军团名字（NPC名字才会被转换）
		if info.origin_group then
			info.origin_group = AppText:getServerText(info.origin_group)
		end
		if info.now_group then
			info.now_group = AppText:getServerText(info.now_group)
		end

		local cityId = info.city_id
		for i , reportInfo in ipairs(info.report_list) do
			reportInfo.city_id = cityId
			self:addReport(reportInfo)
		end
	end
	self:notifyObservers("onGetLegionSimpleReport", msg.report_list)
end

function This:onGetCityBattleReportRequest(cityId)
	Socket:sendAndLock(14022,{
		city_id = cityId
	})
end

function This:onGetCityBattleReportReply(msg)
	for i , info in ipairs(msg.fight_list) do
		self:addFightReport(info)
	end
	self:notifyObservers("onGetCityBattleReport", msg)
end

function This:onGetLegionDetailReportRequest(reportId)
	self.m_isRequesting = true
	Socket:sendAndLock(10305,{
		report_id = reportId
	})
end

function This:onGetLegionDetailReportReply(msg)
	if not self.m_isRequesting then
		return
	end
	if msg.err_code == 0 then
		local report = self.m_fightReport[msg.report.id]
		if report then
			report:setRealReportInfo(msg.report)
		end
		BattleDataManager:addObserver(self)
		self:notifyObservers("onGetReportDetail",msg)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
	self.m_isRequesting = nil
end

function This:onGetRescueDetailReportRequest(reportId)
	self.m_isRequesting = true
	Socket:sendAndLock(14030,{
		report_id = reportId
	})	
end

function This:onGetRescueDetailReportReply(msg)
	if not self.m_isRequesting then
		return
	end
	if msg.err_code == 0 then
		local report = self.m_fightReport[msg.report.id]
		if report then
			report:setRealReportInfo(msg.report)
		end
		BattleDataManager:addObserver(self)
		self:notifyObservers("onGetReportDetail",msg)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
	self.m_isRequesting = nil
end

----------------------------------------
function This:onFightEnded(report)
	if report then
		local rType = report:getType()
	    if rType ~= ReportType.kArena and rType ~= ReportType.kLegion  then
		    local city = WorldMapManager:getCity(report:getCityId())
		    print(string.format("战斗停止，城市%d",report:getCityId()))
	    end
	end
	self:notifyObservers("onFightEnded", report)
end

function This:onBattleSettlementNotify()
	self:clear()
end

function This:onBattleSkip()
	local reportId = BattleDataManager:getBattleId()
	local report = self:getReport(reportId)
	if report:getType() == ReportType.kLegion or report:getType() == ReportType.kEmperor then
		if GamePlayer:isInLegionMap() then
			GameMapManager:enterMap(GamePlayer:getLegionMapId())	
		else
			cc.SceneManager:switchTo("Home.MenuScene")
		end
	elseif report:getType() == ReportType.kRescue then
		if GameMapManager:isInMap(GameMapType.kRescueMap) then
			GameMapManager:enterRescueMap()
		else
			cc.SceneManager:switchTo("Home.MenuScene")
		end
		-- print("跳过")
	elseif GamePlayer:isInWorld() then
		if report:getType() == ReportType.kArena  then
			cc.SceneManager:switchTo("Home.MenuScene")
			if ArenaManager:isArenaOpened() then
				MenuScene:addDelayDialog({name = "Arena.ArenaPanel"})
			end
		elseif report:getType() == ReportType.kPeakContest then
			cc.SceneManager:switchTo("Home.MenuScene")
			if PeakContestManager:isOpen() then
				MenuScene:addDelayDialog({name = "PeakContest.PeakContestPanel"})
			end
		else
			cc.SceneManager:switchTo("World.WorldScene")
	 	end
	else
		local reportId = BattleDataManager:getBattleId()
		local report = self:getReport(reportId)
		cc.SceneManager:switchTo("Home.MenuScene")
		if report and report:getType() == ReportType.kArena and ArenaManager:isArenaOpened() then
			MenuScene:addDelayDialog({name = "Arena.ArenaPanel"})
		end
	end
	BattleDataManager:removeObserver(self)
end

function This:onBattleVictory()
	BattleDataManager:removeObserver(self)
	local reportId = BattleDataManager:getBattleId()
	local report = self:getReport(reportId)

	if report:getType() == ReportType.kArena then
		local panel = g_scene:openDialog("Report.ArenaReportEndPanel",report)
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	elseif report:getType() == ReportType.kRescue then
        local panel = g_scene:openDialog("RescueEmperor.RescuBattleEndPanel",{isWin= true})
        panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	elseif report:getType() == ReportType.kPeakContest then 
		local panel = g_scene:openDialog("PeakContest.PeakContestReportEndPanel",report)
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	elseif report:getType() == ReportType.kLegion or report:getType() == ReportType.kEmperor then
		local panel = g_scene:openDialog("Map.LegionBattleEndPanel",{reportId = reportId})
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	else
		local panel = g_scene:openDialog("World.BattleEndPanel",{reportId = reportId})
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	end
end

function This:onBattleFailed(isSkip)
	BattleDataManager:removeObserver(self)
	local reportId = BattleDataManager:getBattleId()
	local report = self:getReport(reportId)
	if report:getType() == ReportType.kArena then
		local panel = g_scene:openDialog("Report.ArenaReportEndPanel",report)
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	elseif report:getType() == ReportType.kRescue then
        local panel = g_scene:openDialog("RescueEmperor.RescuBattleEndPanel",{})
        panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	elseif report:getType() == ReportType.kPeakContest then 
		local panel = g_scene:openDialog("PeakContest.PeakContestReportEndPanel",report)
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	elseif report:getType() == ReportType.kLegion or report:getType() == ReportType.kEmperor then
		local panel = g_scene:openDialog("Map.LegionBattleEndPanel",{reportId = reportId})
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	else
		local panel = g_scene:openDialog("World.BattleEndPanel",{reportId = reportId})
		panel:setLocalZOrder(BattleZorder.kUiLayer + 1)
	end
end

--endregion
