-- @Author: zheng
-- @Date:   2019-03-22 14:57:13
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 16:53:51
-- @File Name: RescuBianDuiEditPanel.lua

local This , super = class(...,"Map.LegionBianDuiEditPanel")
local RescuBianDuiEditItemView = require("RescueEmperor.RescuBianDuiEditItemView")

local kMaxBianDuiCount = 3

function This:initData(info)
    self.m_filename = "RescueEmperor/RescueEmperorTroopsEdit.csb"
    if info then
        self.m_map = info.map
    end
end

function This:initNode(info)
    super.initNode(self,info)
    self.m_csNode.changename:setVisible(false)
    self:refreshLockTxt()
end

function This:refreshLockTxt()
    local csNode = self.m_csNode
    local maxCount = self.m_map:getMaxBianDuiCount()
    for index = 1, kMaxBianDuiCount do
        local suoNode = csNode["tab" .. index].suo
        if suoNode then
            suoNode:setVisible(index > maxCount)
        end
    end
end

function This:createHeroItemView(hero)
    local view = RescuBianDuiEditItemView:create({
        hero = hero,
        isContanier = true
    })

    view:setSelectHandler(function(o)
        local isFind = false
        for i = 1 , 6 do
            if not self.m_curBianDui[i] then
                self:addCardToCurBianDui(i,o:getId())
                isFind = true
                break
            end
        end
        return isFind
    end)

    view:setJudgeIsInAllTeamHandler(function(o)
        local heroId = o:getId()
        return self.m_allBianDuiCards[heroId]
    end)

    view:setJudgeIsInCurTeamHandler(function(o)
        local heroId = o:getId()
        return self.m_bianDuiCards[self.m_index][heroId]
    end)
    return view
end

function This:createCardItemView(hero)
    local view = RescuBianDuiEditItemView:create({
        hero = hero,    
        isSwallow = true,
        isContanier = true
    })
    view:setSelectHandler(function(o)
        local biandui = self.m_map:getBianDui(self.m_index)
        if biandui:getState() ~= LegionTroopState.kIdle then
            g_scene:showTips(Serialize[61688])
            return
        end
        local isFind = false
        for i = 1 , 6 do
            if self.m_curBianDui[i] == o:getId() then
                self:removeCardFromCurBianDui(i,o:getId())
                isFind = true
                break
            end
        end
        return isFind
    end)
    return view
end

function This:selectBianDui(index)
    local maxCount = self.m_map:getMaxBianDuiCount()
    if index and index > maxCount then
        -- g_scene:openDialog("Vip.VipPanel")
        return false
    end

    if self.m_index ~= index then
        self.m_index = index
        self:updateBianDuiData()
        self.m_heroLayer:setDataList(self.m_heroList,function(hero,index)
            local view = self:createHeroItemView(hero)
            return view
        end)
        self:refreshBianDuiCard()
    end
end