-- @Author: zheng
-- @Date:   2019-03-22 14:39:57
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-30 17:15:42
-- @File Name: RescuBianDuiView.lua

local This , super = class(..., "Map.LegionBianDuiView")
local RescueEmperorCityConfig = require("Metadatas.RescueEmperorCityConfig")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

local StateImage = {
	[LegionTroopState.kAddHp] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kBattle] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kDefensive] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kIdle] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kMarch] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kTechAddSpeed] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kTransport] = "MenuScene/zhengzhan.png",
	[LegionTroopState.kReadyBattle] = "MenuScene/zhengzhan.png"
}

local StateColor = 
{
	-- [LegionTroopState.kAddHp] = {255,0,255},
	-- [LegionTroopState.kBattle] = {255,0,255},
	-- [LegionTroopState.kDefensive] = {255,0,255},
	-- [LegionTroopState.kIdle] = {255,0,255},
	-- [LegionTroopState.kMarch] = {255,0,255},
	-- [LegionTroopState.kTechAddSpeed] = {255,0,255},
	-- [LegionTroopState.kTransport] = {255,0,255},
	-- [LegionTroopState.kReadyBattle] = {255,0,255}
	[LegionTroopState.kAddHp] = "<New9>",
	[LegionTroopState.kBattle] = "<New11>",
	[LegionTroopState.kDefensive] = "<New5>",
	[LegionTroopState.kIdle] = "<New9>",
	[LegionTroopState.kMarch] = "<New5>",
	[LegionTroopState.kTechAddSpeed] = "<New5>",
	[LegionTroopState.kTransport] = "<New5>",
	[LegionTroopState.kReadyBattle] = "<New11>"
}


function This:initNode(info)
	self.m_fileName = "RescueEmperor/RescueEmperorPreviewlist.csb"
	self.m_escape = 0
	super.initNode(self, info)
    self:scheduleUpdate()
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.biandui:setTouch({
		onEnded = function()
		    g_scene:openDialog("RescueEmperor.RescuBianDuiEditPanel",{
		        pos = self.m_biandui:getPos(),
		        map = self.m_map,
		    })
		end,
		isDynamic = true
	})

	csNode.node.iconbg:setTouch({
		onEnded = function()
			if self.m_map and self.m_biandui then
				local troop = self.m_biandui:getTroop()
				if troop then
					local state = troop:getState()
					if state == LegionTroopState.kMarch then
						local marchId = troop:getMarchId()
						if marchId and marchId > 0 then
							g_scene:focuseMarch(marchId)
						end
					else
						local cityId = troop:getCityId()
						if cityId and cityId > 0 then
							g_scene:focuseToCity({cityId = cityId})
						end
					end
				end
			end
		end,
		isDynamic = true
	})
end

function This:getCityName(cityId)
    if cityId > 0 then
        return RescueEmperorCityConfig[cityId].city_name
    end
    return ""
end

function This:refreshState(troop)
	local contentNode = self.m_csNode.node

	local state = troop:getState()
	contentNode.pic:setImage(StateImage[state])

	local cityId = troop:getCityId()
	local statestr = RescueTroopStateText[state]
	if cityId > 0 and state ~= LegionTroopState.kIdle then
		 statestr = self:getCityName(cityId) .. statestr
	end

	self.m_countEndTime = false
	if state == LegionTroopState.kMarch or state == LegionTroopState.kReadyBattle then
		self.m_countEndTime = self.m_biandui:getTroop():getMarchEndTime()
	elseif state == LegionTroopState.kAddHp then
		self.m_countEndTime = self.m_biandui:getTroop():getAddHpEndTime()
	end

	-- contentNode.state2:setString(statestr)
	-- contentNode.state2:setColor(StateColor[state][1],StateColor[state][2],StateColor[state][3])
	contentNode.state2:setStringWithColor(StateColor[state]..statestr)
end

function This:update(dt)
	super.update(self,dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return
	end
	self.m_escape = self.m_escape - 1
	if self.m_countEndTime  then
		if self.m_countEndTime > 0 then
			local endTime = self.m_countEndTime
			local curTime = GamePlayer:getServerTime()
			local statestr = RescueTroopStateText[LegionTroopState.kAddHp]
			self.m_csNode.node.state2:setString( statestr .. GameUtil:formatTime(" ".. Serialize["H_M_S"], math.max(0,endTime - curTime)))
		else
			self.m_countEndTime = false
		end
	end
end