-- @Author: zheng
-- @Date:   2019-03-23 10:58:28
-- @Last Modified by:   superm
-- @Last Modified time: 2019-09-26 14:11:49
-- @File Name: RescuCity.lua

local This , super = class(...,"Map.LegionCity")
local RescueEmperorCityConfig = require("Metadatas.RescueEmperorCityConfig")
local ArgConfig = require("Metadatas.ArgConfig")

local CityImage = {
	[1] = "GameScene/city1.png",
	[2] = "GameScene/town1.png",
	[3] = "GameScene/town1.png",
}

function This:initByConfig()
	local initLevel = self.m_map:getCityInitLevel()
	self.m_cityConfig = RescueEmperorCityConfig[self.m_id]
	self:setLevel(initLevel)

	self.m_buildType = LegionBuildType.kNone
	local cityId = self:getId()
	local config = ArgConfig["rescue_emperor_city_distribution"].arg
	if config then
		for _, cityIdList in ipairs(config) do
			if table.contain(cityIdList, cityId) then
				self.m_buildType = LegionBuildType.kMainCity
				break
			end
		end
	end
end

function This:updateInfo(info)
	local oldLegionId = self.m_legionId

	self.m_hpNum = info.force_num
	self.m_oreNum = info.ore_num
	self.m_legionId = info.group_id
	self.m_damage = info.damage
	self.m_state = info.state

	if info.build_type > 0 then
		self.m_buildType = info.build_type
	end

	self:setLevel(info.level)

	self.m_guardTroops = {}
	for i , id in ipairs(info.guard_troop_list) do
		self.m_guardTroops[id] = self.m_map:getTroop(id)
	end
	self.m_attackTroops = {}
	for i , id in ipairs(info.attack_troop_list) do
		self.m_attackTroops[id] = self.m_map:getTroop(id)
	end
	self.m_addHpTroops = {}
	for i , id in ipairs(info.fill_force_troop_list) do
		self.m_addHpTroops[id] = self.m_map:getTroop(id)
	end

	if oldLegionId > 0 then
		local legion = self.m_map:getLegion(oldLegionId)
		if legion then
			legion:removeCity(self)
		end
	end

	if self.m_legionId > 0 then
		local legion = self.m_map:getLegion(self.m_legionId)
		if legion then
			legion:addCity(self)
		end
	end
end

function This:setInvestigateCityInfo(info)
    self.m_troopCount = info.troop_count
	self.m_totalForce = info.total_force
end

function This:getInvestigateCityInfo()
    return self.m_troopCount, self.m_totalForce 
end

function This:isEmptyCity()
end

function This:getImage()
	local type = self:getType()
	return CityImage[type]
end

function This:getBuildName()
    return "ID:" .. self:getId()
end

function This:setLevel(level)
	self.m_level = level
end

function This:updateAddSpeedTroopList( info )
end