-- @Author: superm
-- @Date:   2019-03-27 11:39:15
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-30 15:53:27
-- @File Name: RescuEmbattlePanel.lua
-- @Description: 

local This , super = class(...,"Stage.EmbattleHeroPanel")
local Dialog = require("Dialog")
local RescuPath = require("RescueEmperor.RescuPath")
local GameUtil = require("Util.GameUtil")
local LegionMapUtil = require("Map.LegionMapUtil")
local HeroManager = require("Hero.HeroManager")
local StageHeroItemView = require("Stage.StageHeroItemView")

function This:initNode(info)
	self.m_biandui = info.biandui
	self.m_fromCity = info.fromCity
	self.m_toCity = info.toCity
	self.m_transType = info.transType
	self.m_resList = info.resList
	self.m_map = info.map

	self.m_scene = g_scene

	self.m_heros = {}
	self.m_fetters = {}
	self.m_fetterInfo = {}

	self.m_path = RescuPath:getMapPath(self.m_fromCity:getId(), self.m_toCity:getId(), self.m_map, self.m_transType)
	self.m_isUseJice = false

	self.m_csNode = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorLineUp.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self:initBaseInfo()
    self:initListLayer(info)
	self:initItems()
end

function This:initBaseInfo()
	self.m_csNode.bg3.taget:setString(self.m_toCity:getName())
	local legion = self.m_map:getMyLegion()
	self.m_csNode.bg3.name:setString(legion:getName())

	local maxHp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(maxHp)

	local time = self.m_path:getCostTime(self.m_isUseJice)
	self.m_csNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"],time))
end

function This:getHeroSource()
	local result = {}
	local troop = self.m_biandui:getTroop()
	local cardList = troop:getCardList()
	for _, card in ipairs(cardList) do
		local pos = card:getPos()
		result[pos] = card:getHeroId()
	end
	return result
end

function This:onEnter()
	Dialog.onEnter(self)
	self.m_map:addObserver(self)
end

function This:onExit()
	Dialog.onExit(self)
	self.m_map:removeObserver(self)
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.Battlebutton:setTouchEnded(function()
		self.m_map:onCreateMarchRequest({
			index = self.m_biandui:getPos(),
			res_type = self.m_transType,
			res_list = self.m_resList,
			from_city = self.m_fromCity:getId(),
			to_city = self.m_toCity:getId()
		})
	end)
end

function This:getTotalHp()
	local total = 0
	for i , num in ipairs(self.m_resList) do
		total = total + num
	end
	return total
end

function This:getHeroView(heroId)
	local hp, maxHp = 0, 0
	local cardList = self.m_biandui:getTroop():getCardList()
	for _, card in ipairs(cardList) do
		if card:getHeroId() == heroId then
			hp = self.m_resList[card:getPos()]
			maxHp = card:getMaxHp()
			break
		end
	end
	local node = StageHeroItemView:create(heroId,{
		hp = hp,
		maxHp = maxHp
	})
	return node
end

function This:addHeroView(heroId,pos)
	local node = self:getHeroView(heroId)
	node:setAnchorPoint(0.5,0.5)
	self.m_csNode["hero"..pos]:addChild(node)
	self.m_heros[pos].node = node
end

------------------------------------------------------
function This:onCreateMarch(march)
	self:close()
	self.m_scene:exitSelectMode()
end