-- @Author: superm
-- @Date:   2019-03-25 20:53:12
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-29 11:54:06
-- @File Name: RescuPath.lua
-- @Description: 

local This = class(...,"World.Path")
local ArgConfig = require("Metadatas.ArgConfig")

local kHpTimeFactor = ArgConfig["rescue_emperor_march_time_coefficient"].arg[1]
local kMyDistFactor = ArgConfig["rescue_emperor_owner_city_distance"].arg[1]
local kOtherDistFactor = ArgConfig["rescue_emperor_enemy_city_distance"].arg[1]

function This:init(transType)
	self.m_transType = transType
	self.m_path = {}
	self.m_isJice = false
	self.m_isToBattle= false
	self.m_distance = nil
end

function This:setIsUseJiCe(isUseJice)
	self.m_isJice = isUseJice
end

function This:setIsBattle(isBattle)
	self.m_isToBattle = isBattle
end

function This:getDistance()
	if self.m_distance then
		return self.m_distance
	end
	self.m_distance = 0
	for i = 2 , #self.m_path do
		if self.m_path[i]:isOur() then
			self.m_distance = self.m_distance + kMyDistFactor
		else
			self.m_distance = self.m_distance + kOtherDistFactor
		end
	end
	return self.m_distance
end

--获取运输时间花费
function This:getCostTime(isJice)
	local distance = self:getDistance()
	return distance * kHpTimeFactor
end

--获取粮草花费
function This:getCostFood(resNum,isJice)
	return 0
end

function This:getMapJicePath(fromId,toId,map,transType)
	local openList = {}
	local distList = {}
	local closeList = {}
	local tmp = {}

	local fromCity = map:getCity(fromId)
	local toCity = map:getCity(toId)

	table.insert(openList,fromCity)
	distList[fromCity] = 0
	
	function getDist(c1,c2)
		if c1:isOur() and c2:isOur() then
			return kMyDistFactor
		end
		return kOtherDistFactor
	end

	while #openList > 0 do
		local city = openList[1]
		table.remove(openList,1)
		tmp[city] = nil

		local neighborList = city:getNeighborhood()
		for i , ct in ipairs(neighborList) do
			local neighCity = ct
			local dist = distList[city] + getDist(city,neighCity)
			if not distList[neighCity] or distList[neighCity] > dist then
				distList[neighCity] = dist
				closeList[neighCity] = city
				if not tmp[neighCity] then
					table.insert(openList,neighCity)
					tmp[neighCity] = true
				end
			end
		end
	end
	
	local path = This:create(transType)
	path:setIsUseJiCe(true)
	path:setIsBattle(not toCity:isOur())
	local curCity = toCity
	path:addCity(curCity,true)
	while curCity ~= fromCity do
		curCity = closeList[curCity]
		path:addCity(curCity,true)
	end
	return path
end

function This:getMapPath(fromId,toId,map,transType)
	local openList = {}
	local closeList = {}
	local fromCity = map:getCity(fromId)
	local toCity = map:getCity(toId)

	table.insert(openList,fromCity)
	closeList[fromCity] = fromCity

	while #openList > 0 do
		local city = openList[1]
		table.remove(openList,1)

		if city == toCity then
			break
		end

		local isFind = false
		local neighborList = city:getNeighborhood()
		for i , neighCity in ipairs(neighborList) do
			if neighCity == toCity then
				closeList[neighCity] = city
				isFind = true
				break
			elseif not closeList[neighCity] and neighCity:isOur() then
				table.insert(openList,neighCity)
				closeList[neighCity] = city
			end
		end

		if isFind then
			break
		end
	end
	
	local path = This:create(transType)
	path:setIsBattle(not toCity:isOur())

	local curCity = toCity
	path:addCity(curCity,true)
	while curCity ~= fromCity do
		curCity = closeList[curCity]
		path:addCity(curCity,true)
	end
	return path
end

function This:getMapAroundCity(fromCity,map)
	local openList = {}
	local closeList = {}
	local result = {}
	table.insert(openList,fromCity)
	closeList[fromCity] = true

	while #openList > 0 do
		local city = openList[1]
		table.remove(openList,1)
		local neighborList = city:getNeighborhood()
		for i , neighCity in ipairs(neighborList) do
			if city:isOur() and not closeList[neighCity] then
				local isWalkable = neighCity:isOur() or (not neighCity:isOur() and  not neighCity:isProtected())
				if isWalkable then
					result[neighCity:getId()] = neighCity
					table.insert(openList,neighCity)
				end
			end
		end
		closeList[city] = true
	end
	return result
end

function This:getAroundCityList(map,city,depth,handler)
	local openList = {}
	local closeList = {}
	table.insert(openList,city)
	closeList[city] = 0

	while #openList > 0 do
		local curCity = openList[1]
		table.remove(openList,1)
		if closeList[curCity] and closeList[curCity] >= depth then
			break
		end
		local neighborList = curCity:getNeighborhood()
		for i , neighCity in ipairs(neighborList) do
			if not closeList[neighCity] then
				table.insert(openList,neighCity)
				closeList[neighCity] = closeList[curCity] + 1
			end
		end
	end
	local result = {}
	for ct , dp in pairs(closeList) do
		if not handler or handler(ct) then
			table.insert(result,ct)
		end
	end
	return result
end