-- @Author: superm
-- @Date:   2019-03-25 16:53:14
-- @Last Modified by:   superm
-- @Last Modified time: 2019-04-01 17:33:24
-- @File Name: RescuTroopsView.lua
-- @Description: 

local This , super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")
local LegionCardView = require("Map.LegionCardView")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")

function This:initData(info)
	self.m_biandui = info.biandui
	self.m_city = info.city
	self.m_map = info.map
	self.m_scene = g_scene

	self.m_escape = 0
    self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
	self.m_fileName = "RescueEmperor/RescueEmperorTroopsList.csb"
    self.m_csNode = cc.Node:createFromCSFile(self.m_fileName)
    self:addChild(self.m_csNode)
end

function This:initNode(info)
	local csNode = self.m_csNode
	csNode.team1:setString(self.m_biandui:getName())
	self.m_itemList = {}
	local troop = self.m_biandui:getTroop()
	for _, card in ipairs(troop:getCardList()) do
		local pos = card:getPos()
		local view = LegionCardView:create(card)
		csNode["hero".. pos]:addChild(view)
		self.m_itemList[pos] = view
	end

    self:scheduleUpdate()
end

function This:initTouchEvent()
	if not self.m_inited then
		local csNode = self.m_csNode
		csNode.chuzheng:setTouch({
			onEnded = function()
				local troop = self.m_biandui:getTroop()
				local state = troop:getState()
				if state == LegionTroopState.kIdle then
					print("驻守...")
					self.m_map:onGarrisonRequest(self.m_biandui:getPos(), self.m_city:getId())
				elseif state == LegionTroopState.kDefensive then
					print("出征...")
					self.m_scene:closeAllDialog()
					local cardList = self.m_biandui:getCardList()
					local hpList = {0,0,0,0,0,0}
					local total = 0
					for _, card in ipairs(cardList) do
						local pos = card:getPos()
						hpList[pos] = card:getHp()
						total = total + card:getHp()
					end
					if total <= 0 then
						print("兵量为0")
						return
					end
					self.m_scene:enterSelectMode(self.m_city,self.m_biandui, LegionTransType.kHp, hpList)
				end
			end,
			isScale = false,
			isContainer = true
		})

		self.m_inited = true
	end
end

function This:refresh()
	local csNode = self.m_csNode
	local troop = self.m_biandui:getTroop()
	local state = troop:getState()
	self.m_isAddingHp = (state == LegionTroopState.kAddHp)
	self.m_csNode.Text_1:setVisible(self.m_isAddingHp)
	
	local visible, name = false
	if state == LegionTroopState.kIdle and self.m_city:getId() == self.m_map:getMyLegionCityId() then
		visible = true
		name = Serialize[64155]	
	elseif state == LegionTroopState.kDefensive then
		visible = true
		name = Serialize[64156]
	end
	csNode.chuzheng:setVisible(visible)
	csNode.chuzheng.text:setString(name)

	csNode.state:setString(RescueTroopStateText[state])
	for i , view in ipairs(self.m_itemList) do
		view:refresh()
	end
end

function This:onEnter()
	super.onEnter(self)
	self:refresh()
	self.m_map:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	self.m_map:removeObserver(self)
end

---------------------------------------
function This:update(dt)
	super.update(self,dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1 then
		return
	end
	self.m_escape = self.m_escape - 1
	if self.m_isAddingHp then
		local endTime = self.m_biandui:getTroop():getAddHpEndTime() 
		local curTime = GamePlayer:getServerTime()
		self.m_csNode.Text_1:setString( GameUtil:formatTime(Serialize["H_M_S"],math.max(0,endTime - curTime)))
	end
end

function This:onGarrison(biandui)
	if self.m_biandui == biandui then
		self:refresh()
	end
end




function This:onAddHpFinish(biandui)
	if self.m_biandui == biandui then
		self:refresh()
	end
end