-- @Author: zheng
-- @Date:   2019-03-26 11:21:27
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-03 09:59:02
-- @File Name: RescueBoxManager.lua
RescueBox ={
    kForceKill   = 4,--杀兵
    kTroopKill = 5,--杀敌
    kCombKill  = 6,--连斩
}

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")

local GamePlayer = require("Player.GamePlayer")
local RecruitManager = require("Recruit.RecruitManager")
local GameUtil = require("Util.GameUtil")
local RescueEmperorBoxRewardConfig = require("Metadatas.RescueEmperorBoxRewardConfig")

function This:init()
    super.init(self)
    self:resetBoxData()

    self.m_rankInfor = {}
    self:initListener()
end

function This:initListener()
    EventDispatcher:addListener(self, 14027, self.onGetBoxRewardReply)--杀敌宝箱
    EventDispatcher:addListener(self, 14028, self.onGetRankInforReply)--排行榜
    EventDispatcher:addListener(self, 14029, self.onGetKillScoreReply)--个人击杀信息
end

function This:removeListener()
    EventDispatcher:removeListener(self,14027)
    EventDispatcher:removeListener(self,14028)
    EventDispatcher:removeListener(self,14029)
end

function This:isHaveBoxManager()
    local result = false
    local map = self.m_map
    if map and map:isInit() then
        result = true
    end
    return result
end

function This:getRankInfor(rankType)
    return self.m_rankInfor[rankType]
end

function This:setRankInfor(rankInfor)
    if rankInfor then
       local rankType = rankInfor.type
       local rankList = rankInfor.list or {}
       if rankType then
            table.sort(rankList, function( a, b )
                    local state1, state2 = a.value , b.value
                    return state1>state2
                end)
           local myPlayerId = GamePlayer:getPlayerID()
           for rankIndex,rankItem in ipairs(rankList) do
               local roleId = rankItem.roleid
               rankItem.isMe = myPlayerId == roleId
               rankItem.rank = rankIndex
           end
           self.m_rankInfor[rankType] = rankList
       end
    end
end

function This:getLevel()
    local defaultValue = 1
    local map = self.m_map
    if not self:isHaveBoxManager() then
        return defaultValue
    end

    local temLevel = map:getMapLv()
    if not temLevel or temLevel==0 then
        temLevel = defaultValue
    end
    return temLevel
end

function This:setMapInfo(data)
    local mapInfor = data.mapInfor
    local map = data.map
    self:setRankInfor(mapInfor.force_kill_rank_info or {})
    self:setRankInfor(mapInfor.troop_kill_rank_info or {})
    self:setRankInfor(mapInfor.comb_kill_rank_info or {})
    self:onGetKillScoreRequest()
    self.m_map = map

    self:resetBoxData()
    self.m_boxRewardHistory = mapInfor.geren.reward_list or {}
    local temBoxRewardHistory = self:convertBoxIdToOrder()
    local boxConfigs = self.m_temBoxConfig
    self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = GameUtil:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,boxConfigs,temBoxRewardHistory,"prestige_num")
end

function This:convertBoxIdToOrder()
    local result ={}
    for _,boxConfigId in ipairs(self.m_boxRewardHistory) do
        local boxConfigItem = RescueEmperorBoxRewardConfig[boxConfigId]
        if boxConfigItem then
            local order = boxConfigItem.order
            table.insert(result,order)
        end
    end
    return result
end

function This:resetBoxData()
    self.m_boxRewardList ={}
    self.m_boxRewardHistory ={}
    self.m_boxRewardScore =0
    self.m_isCanReward = false
    self.m_minChallengeStarLv = 0
    self.m_temBoxConfig = {}

    local mapLevel = self:getLevel()
    for index, boxConfigItem in pairs(RescueEmperorBoxRewardConfig) do
        local temMapLevel = boxConfigItem.map_level
        if mapLevel and mapLevel == temMapLevel then
            local order = boxConfigItem.order
            self.m_temBoxConfig[order] = boxConfigItem
            self.m_boxRewardList[order] = {
                    id = boxConfigItem.id, 
                    state = RecruitManagerBoxState.kLock, 
                    reward = boxConfigItem.reward, 
                    prestige_num = boxConfigItem.prestige_num,
                }
        end
    end
end

function This:getBoxHistory()
    return self.m_boxRewardHistory
end

function This:getBoxRewardScore()
    return self.m_boxRewardScore
end

function This:getIsCanReward()
    return self.m_isCanReward
end

function This:getMinBoxStarLv()
    return self.m_minChallengeStarLv
end

function This:getRewardList()
    return table.clone(self.m_boxRewardList)
end
function This:needShowHelpWhenEnterMap()
    return false
end

function This:getBoxManager()
    return self.m_boxManager
end
function This:onGetBoxRewardRequest(boxId)
    Socket:sendAndLock(14027,{box_id = boxId})
end

function This:onGetBoxRewardReply( msg )
    if not self:isHaveBoxManager() then
        return
    end
    if msg.err_code == 0 then
        local boxId = msg.box_id
        local boxItem = RescueEmperorBoxRewardConfig[boxId]
        if boxItem then
            local rewardList = boxItem.reward
            g_scene:showReward(rewardList)
        end
        local isFind = GameUtil:findItem(self.m_boxRewardHistory,boxId)
        if not isFind then
            table.insert(self.m_boxRewardHistory,boxId)
        end
        local temBoxRewardHistory = self:convertBoxIdToOrder()
        local boxConfigs = self.m_temBoxConfig
        self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = GameUtil:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,boxConfigs,temBoxRewardHistory,"prestige_num")
        self:notifyObservers("onGetBoxReward")
        EventDispatcher:dispatcher("recruit_zhongjin_cd_changed")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
    
function This:onGetRankInforRequest(rankType)
    -- Socket:send(14028,{type = rankType})
end

function This:onGetRankInforReply( msg )
    if not self:isHaveBoxManager() then
        return
    end
--    if msg.err_code == 0 then
        self:setRankInfor(msg.rank_info or {})
        self:notifyObservers("onGetRankInfor")
--    else
--        g_scene:handleErrorCode(msg.err_code)
--    end
end

function This:onGetKillScoreRequest()
    Socket:send(14029)
end

function This:onGetKillScoreReply( msg )
    if not self:isHaveBoxManager() then
        return
    end
    -- if msg.err_code == 0 then
        self.m_boxRewardScore = msg.kill_force or 0
        local temBoxRewardHistory = self:convertBoxIdToOrder()
        local boxConfigs = self.m_temBoxConfig
        self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = GameUtil:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,boxConfigs,temBoxRewardHistory,"prestige_num")
        self:notifyObservers("onGetBoxReward")
    -- else
    --     g_scene:handleErrorCode(msg.err_code)
    -- end
end