-- @Author: superm
-- @Date:   2019-03-20 10:58:22
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-27 15:46:47
-- @File Name: RescueEmperorCityView.lua
-- @Description: 

local This , super = class(...,"Map.LegionCityView")
local GameUtil = require("Util.GameUtil")

function This:initCsNode()
	self.m_csNode = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorCity.csb")
	self:addChild(self.m_csNode)
end

function This:initTouchEvent()
	local csNode = self.m_csNode
	csNode.city:setTouch({
		onEnded = function()
			if self.m_touchHandler then
				self.m_touchHandler(self)
			end
		end,
		isDynamic = true,
		isSwallow = false
	})
end

function This:initBarrackNode()
end


function This:refreshCityInfo()
	local legionId = self.m_city:getLegionId()
	local legion
	if legionId > 0 then
		legion = self.m_map:getLegion(legionId)
	end
	
	if legion then 
		self.m_csNode.camp.name:setString(legion:getName())
		self.m_csNode.camp:setVisible(true)
		local node = self.m_csNode.camp.flag
		if node then
			local city = legion:getMainCity()
			local cityId = city:getId()
			node:setVisible(self.m_city:getBuildType() == LegionBuildType.kMainCity and self.m_city:getId() == cityId)
		end
	else
		self.m_csNode.camp:setVisible(false)
	end

	self:refreshCityUI()

	local count , totalHp = self.m_city:getGuardCountAndHp()
	if count > 0 then
		self.m_csNode.armydetails.linenumber:setString(count)
		self.m_csNode.armydetails.hpnumber:setString(GameUtil:formatCount(totalHp))
		self.m_csNode.armydetails:setVisible(true)
	else
		self.m_csNode.armydetails:setVisible(false)
	end
	self.m_csNode.buildname.name:setString(self.m_city:getBuildName())
	self:refreshBarrackNode()
	self:refreshHorse()
end


function This:showSelectMenu(isShow)
	print(tostring(self.m_city:isOur()))
	if isShow and not self.m_menu then
		if self.m_city:isOur() then
			self.m_menu = self:getOurCityMenu()
		else
			self.m_menu = self:getEmptyCityMenu()
		end
		self.m_menu:setLocalZOrder(g_scene:getMaxLocalZOrder() + 1)
		self.m_scene:showMenu(self.m_menu,self.m_city:getId())
	elseif not isShow and self.m_menu then
		self.m_menu:removeFromParent()
		self.m_menu = nil
		self.m_scene:showMenu(nil)
	end
end

function This:getOurCityMenu()
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorMySelect.csb")
	node.ownclick.troops:setTouch({
		onEnded = function()
        	g_scene:openDialog("RescueEmperor.RescuTroopsPanel",{
				map = self.m_map,
				city = self.m_city
			})
		end,
		isScale = false,
		isDynamic = true
	})
	local isMainCity = self.m_city:getId() == self.m_map:getMyLegionCityId()
	node.ownclick.troops:setVisible((not self.m_city:isFighting()) or isMainCity)
	node.ownclick.guanzhan:setTouch({
		onEnded = function()
			if self.m_city:isInFighting() then
				self.m_scene:openDialog("RescueEmperor.RescueReportPanel",{
					map = self.m_map,
					cityId = self.m_city:getId()
				})
			else
				self.m_scene:showTips(Serialize[61704])
			end
		end,
		isScale = false,
		isDynamic = true
	})

	node.ownclick.look:setTouch({
		onEnded = function()
			self.m_map:onInvestigateCityInfoRequest(self.m_city:getId())
		end,
		isScale = false,
		isDynamic = true
	})
	node.ownclick.guanzhan:setVisible(self.m_city:isFighting())
	return node
end

function This:getEmptyCityMenu()
	local node = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorOtherSelect.csb")
	node.otherclick.see:setTouch({
		onEnded = function()
			if self.m_city:isInFighting() then
				self.m_scene:openDialog("RescueEmperor.RescueReportPanel",{
					map = self.m_map,
					cityId = self.m_city:getId()
				})
			else
				self.m_scene:showTips(Serialize[61704])
			end
		end,
		isScale = false,
		isDynamic = true
	})
	
	node.otherclick.look:setTouch({
		onEnded = function()
			self.m_map:onInvestigateCityInfoRequest(self.m_city:getId())
		end,
		isScale = false,
		isDynamic = true
	})
	node.otherclick.see:setVisible(self.m_city:isFighting())
	return node
end

function This:refreshBarrackNode( ... )
end

function This:refreshJunceState( ... )
	self.m_csNode.timebg:setVisible(false)
	self.m_csNode.userbg:setVisible(false)
end

function This:showSelectAnimation(isShow)
	super.showSelectAnimation(self,isShow)
	self:refreshHorse()
end

function This:refreshHorse()
	local count , totalHp = self.m_city:getGuardCountAndHp()
	if totalHp and count and count > 0 and totalHp > 0 then
		self.m_csNode.horse:setVisible(true)
	else
		self.m_csNode.horse:setVisible(false)
	end
end

function This:onInvestigateCity( city )
    if city == self.m_city then
		local count , totalHp = self.m_city:getInvestigateCityInfo()
		if count > 0 and totalHp > 0 then
			self.m_csNode.armydetails.linenumber:setString(count)
			self.m_csNode.armydetails.hpnumber:setString(GameUtil:formatCount(totalHp))
			self.m_csNode.armydetails:setVisible(true)
		else
			g_scene:showTips(Serialize[64111])
			self.m_csNode.armydetails:setVisible(false)
		end
		self:refreshHorse()
    end
end