-- @Author: superm
-- @Date:   2019-03-20 10:54:54
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-08 17:28:58
-- @File Name: RescueEmperorScene.lua
-- @Description: 

local This , super = class(...,"LoadingScene")
local GameEvent = require("Event.GameEvent")
local AsynchEvent = require("Event.AsynchEvent")

local RescueEmperorUI = require("RescueEmperor.RescueEmperorUI")
local RescueEmperorCityView = require("RescueEmperor.RescueEmperorCityView")
local EventDispatcher = require("EventDispatcher")
local SlideScaleContaner = require("Action.SlideScaleContaner")
local LegionMarchPathView = require("Map.LegionMarchPathView")
local RescuPath = require("RescueEmperor.RescuPath")
local LegionArrowView = require("Map.LegionArrowView")
-- local LegionArrowView = require("Map.LegionArrowView")

ShowMode =
{
	kNormal = 1, --正常模式
	kSelect = 2, --选择模式
	-- kBuild = 3, --建筑
}

local LegionMapZorder = 
{
	kMarch = 1,
	kCityView = 2,
	kPaoMa = 700,
	kArrow = 701,
	kMenu = 702
}

function This:init(info)
	super.init(self)
	self:setName("RescueEmperorScene")
	self.m_map = info.map
	self.m_isIniting = false
	self.m_cityView = {}
	self.m_marchViews = {}
	self.m_paomaEffects = {}
	self.m_mode = ShowMode.kNormal
    self:setType(SceneType.kResueScene)
	self:initTask()
end

function This:initTask()
	self:addTask(GameEvent:create(function()
		self:initNode()
	end))

	self:addTask(GameEvent:create(function()
		self:initMarchArmyPath()
	end))

	local startId , endId = self.m_map:getCityRange()
	for i = startId , endId, 3 do
		task = GameEvent:create(function()
			for j = i , i + 2 do
				if j <= endId then
					self:addCityView(j)
				end
			end
		end)
		self:addTask(task)
	end

	self:addTask(GameEvent:create(function()
		self:initUI()
	end))

	task = GameEvent:create(function()
		self:initLegionBox()
	end)
	self:addTask(task)

	self:addTask(GameEvent:create(function()
		self:initAction()
	end))

	self:addTask(GameEvent:create(function()
		local count = collectgarbage("count")
		collectgarbage("collect")
		print("垃圾回收",count)
	end))
	self.m_isIniting = true
end

function This:loadFinish()
	super.loadFinish(self)
	local legion = self.m_map:getMyLegion()
	if legion then
		local mainCity = legion:getMainCity()
		self:focuseToCity({cityId = mainCity:getId()})
	else
		print("ERROR:" .. "legion is null")
	end
	self.m_isIniting = false
	cc.Application:addListener(self)
	self.m_map:addObserver(self)
	EventDispatcher:dispatcher("resue_emperor_init_finish")
end

function This:onExit()
	super.onExit(self)
	cc.Application:removeListener(self)
	self.m_map:removeObserver(self)
end

function This:initNode()
	 local size = {width = 3000, height = 2500}
	 self.m_mapLayer = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorScene.csb")
	 self.m_mapLayer:setAnchorPoint(0,0)
	 self.m_mapLayer:setContentSize(size.width,size.height)
	 self.m_mapLayer:setPosition((cc.visible.width - size.width)/2,(cc.visible.height - size.height)/2)
	 self:addChild(self.m_mapLayer)
end

function This:initUI()
	self.m_ui = RescueEmperorUI:create({
		map = self.m_map
	})
	self:addChild(self.m_ui)
end

function This:initMarchArmyPath()
	self.m_marchViews = {}
	local marchList = self.m_map:getMarchList()
	for i , march in ipairs(marchList) do
		if march:isVisibleOnMap() then
			self:addMarchPathView(march)
		end
	end
end

function This:initLegionBox()
	-- local map = self.m_map
	-- local cityIds = map:getAppearCitys()
	-- for cityIndex,cityId in ipairs(cityIds) do
	-- 	local cityView = self:getCityView(cityId)
	-- 	cityView:showLegionBox(true)
	-- end
end

function This:initAction()
	self.m_mapLayer:setTouch({
		onMoved = function()
			self:onTouchView()
		end,
		onEnded = function()
			self:onTouchView()
		end,
		isSwallow = false,
		isScale = false,
		isAlpha = false
	 })
	local width , height = self.m_mapLayer:getContentSize()
	self.m_slideScaleContaner = SlideScaleContaner:create({
		node = self.m_mapLayer,
		maxWidth = 	width,
		maxHeight = height,
		minWidth = cc.visible.width,
		minHeight = cc.visible.height,
		scale = 0.8
	})
	self.m_slideScaleContaner:setSlideHandler(function(newx,newy)
		local w , h = self.m_mapLayer:getContentSize()
		local x , y = self.m_mapLayer:getPosition()
		local scale = self.m_mapLayer:getScale()
		w = w * scale 
		h = h * scale
		x , y = cc.visible.center.x - x , cc.visible.center.y - y
		self.m_ui:updateMiniMap(x / w, y / h)
	end)
end

function This:getPositionRatio(id)
	local cityNode = self:getCityNode(id)
	local x ,y = cityNode:getPosition()
	local w , h = self.m_mapLayer:getContentSize()
	return x / w , y / h
end

function This:getViewportCenterRatio()
	local x , y = self.m_mapLayer:getPosition()
	local fx , fy = cc.visible.center.x - x , cc.visible.center.y - y
	local w , h = self.m_mapLayer:getContentSize()
	local scale = self.m_mapLayer:getScale()
	w = w * scale 
	h = h * scale
	return fx / w, fy / h
end

function This:focusePoint(ratiox,ratioy)
	local w , h = self.m_mapLayer:getContentSize()
	local x , y = self.m_mapLayer:getPosition()
	local scale = self.m_mapLayer:getScale()
	w = w * scale 
	h = h * scale

	local dx , dy = cc.visible.center.x - (w * ratiox+x) , cc.visible.center.y - (h * ratioy+y)
	x = math.min(0,math.max(cc.visible.width - w,dx + x))
	y = math.min(0,math.max(cc.visible.height - h,dy + y))
	self.m_mapLayer:setPosition(x,y)
	self.m_ui:updateMiniMap(ratiox, ratioy)
end

function This:focuseToCity(info)
	self.m_mapLayer:stopAllActions()

	local cityId = info.cityId
	local node = self:getCityNode(cityId)
	local x , y = node:convertToWorldSpace(0,0)
	local dx , dy = cc.visible.center.x - x , cc.visible.center.y - y
	local lx , ly = self.m_mapLayer:getPosition()
	local boundRect = self.m_mapLayer:getBoundingBox()

	local maxx = 0
	local minx = cc.visible.width - boundRect.width

	if lx + dx > maxx then
		dx = maxx - lx
	elseif lx + dx < minx then
		dx = minx - lx
	end

	local maxy = 0
	local miny = cc.visible.height - boundRect.height

	if ly + dy > maxy then
		dy = maxy - ly
	elseif ly + dy < miny then
		dy = miny - ly
	end

	local x , y = self.m_mapLayer:getPosition()
	self.m_mapLayer:setPosition(x + dx, y + dy)
	local rx , ry = self:getPositionRatio(cityId)
	self.m_ui:updateMiniMap(rx, ry)
end

function This:focuseMarch(marchId)
	local marchNode = self.m_marchViews[marchId]
	if not marchNode then
		return
	end
	
	self.m_mapLayer:stopAllActions()
	local x , y = marchNode:convertToWorldSpace(0,0)
	local dx , dy = cc.visible.center.x - x , cc.visible.center.y - y
	local lx , ly = self.m_mapLayer:getPosition()
	local boundRect = self.m_mapLayer:getBoundingBox()

	local maxx = 0
	local minx = cc.visible.width - boundRect.width

	if lx + dx > maxx then
		dx = maxx - lx
	elseif lx + dx < minx then
		dx = minx - lx
	end

	local maxy = 0
	local miny = cc.visible.height - boundRect.height

	if ly + dy > maxy then
		dy = maxy - ly
	elseif ly + dy < miny then
		dy = miny - ly
	end

	local x , y = self.m_mapLayer:getPosition()
	self.m_mapLayer:setPosition(x + dx, y + dy)
	local x ,y = marchNode:getPaoMaPosition()
	local w , h = self.m_mapLayer:getContentSize()
	local rx, ry = x / w , y / h
	self.m_ui:updateMiniMap(rx, ry) 
end

function This:focuseCity(cityId,handler,bAction)
	self:focuseToCity({
		cityId = cityId,
		bAction = bAction,
		handler = handler
	})
end

function This:addCityView(cityId)
	local city = self.m_map:getCity(cityId)
	local view = RescueEmperorCityView:create({
		city = city,
		map = self.m_map
	})
	-- local node = self.m_mapLayer[tostring(cityId)]
	print("cityId:" .. cityId)
	local node = self:getCityNode(cityId)
	node:addChild(view)
	node:setLocalZOrder(LegionMapZorder.kCityView + cityId)

	view:setTouchHandler(function(cityView)
		self:onTouchView(cityView)
	end)
	self.m_cityView[cityId] = view
end

function This:getCityNode(cityId)
	local nodeId
	if cityId > 1000 then
		nodeId = cityId - 1000
	else
		nodeId = cityId
	end
	local node = self.m_mapLayer[tostring(nodeId)]
	return node
end

function This:getCityNodeRadio(cityId)
	print("cityId:" .. cityId)
	local node = self:getCityNode(cityId)
	local x , y = node:getPosition()
	local w , h = self.m_mapLayer:getContentSize()
	return x / w, y / h
end

function This:getCityView(cityId)
	return self.m_cityView[cityId]
end

function This:getShowMode()
	return self.m_mode
end

function This:showMenu(menu,cityId)
	if menu then
		local node = self:getCityNode(cityId)
		local x , y = node:getPosition()
		menu:setPosition(x,y)
		self.m_mapLayer:addChild(menu)
		menu:setLocalZOrder(LegionMapZorder.kMenu)
	end
	self.m_menu = menu
end

function This:onTouchView(view)
	if self.m_mode == ShowMode.kNormal then
		if view then
			if self.m_curView then
				self.m_curView:setSelected(false)
			end
			self.m_curView = view
			if self.m_curView then
				self.m_curView:setSelected(true)
			end
			self.m_touchCityView = true
		else
			if not self.m_touchCityView and self.m_curView then
				self.m_curView:setSelected(false)
				self.m_curView = nil
			end
			self.m_touchCityView = false
		end
	elseif self.m_mode == ShowMode.kSelect then
		if view then
			local toCity = view:getModel()
			if self.m_selectCity[toCity:getId()] then
				self:openDialog("RescueEmperor.RescuEmbattlePanel",{
					fromCity = self.m_fromCity,
					toCity = toCity,
					biandui = self.m_biandui,
					map = self.m_map,
					transType = self.m_transType,
					resList = self.m_resList
				})
			else
				g_scene:showTips("该城池不在路线中")
			end
		else
			if not self.m_touchCityView and self.m_curView then
				self.m_curView:setSelected(false)
				self.m_curView = nil
			end
			self.m_touchCityView = false
		end
	end
end

function This:enterSelectMode(fromCity,biandui,transType,resList)
	self.m_mode = ShowMode.kSelect
	self.m_fromCity = fromCity
	self.m_transType = transType
	self.m_biandui = biandui
	self.m_resList = resList
	self.m_ui:refreshModelVisible()
	if transType == LegionTransType.kHp then
		self.m_selectCity = RescuPath:getMapAroundCity(fromCity,self.m_map)
	end

	for id , city in pairs(self.m_selectCity) do
		if not city:isInProtected() then
			local view = self.m_cityView[id]
			view:showSelectAnimation(true)
		else
			self.m_selectCity[id] = nil
		end
	end
end

function This:exitSelectMode()
	self.m_mode = ShowMode.kNormal
	self.m_fromCity = nil
	self.m_transType = nil
	self.m_biandui = nil
	self.m_resList = nil
	self.m_ui:refreshModelVisible()
	for i , city in pairs(self.m_selectCity) do
		local view = self.m_cityView[city:getId()]
		view:showSelectAnimation(false)
	end
end

function This:addDangerView(march)
	if march and not march:isOur() and self.m_ui then
		local city = self.m_map:getCity(march:getToCity())
		if city:isOur() then
			self.m_ui:addDangerView({march},city:getCityId())
			self.m_ui:refreshModelVisible()
		end
	end
end

function This:addMarchPathView(march)
	local view = LegionMarchPathView:create({
		march = march,
		map = self.m_map,
	})
	local node = self:getCityNode(march:getFromCity())
	local x , y = node:getPosition()
	view:setPosition(x,y)
	self.m_mapLayer:addChild(view,LegionMapZorder.kMarch)
	self.m_marchViews[march:getId()] = view
	self:addDangerView(march)
end

function This:removeMarchPathView(march)
	local view = self.m_marchViews[march:getId()]
	if view then
		view:stopUpdate()
		view:removeFromParent()
		self:removeMarchPaoMaEffect(march)
		self.m_marchViews[march:getId()] = nil
	end
end

function This:addMarchPaoMaEffect(effect,march)
	self.m_mapLayer:addChild(effect,LegionMapZorder.kPaoMa)
	self.m_paomaEffects[march:getId()] = effect
end

function This:removeMarchPaoMaEffect(march)
	local id = march:getId()
	if self.m_paomaEffects[id] then
		self.m_paomaEffects[id]:removeFromParent()
		self.m_paomaEffects[id] = nil
	end
end

function This:getMarchPaoMaEffect(march)
	return self.m_paomaEffects[march:getId()]
end

function This:addAttackMarchView(city,march)
	local cityNode = self:getCityNode(city:getId())
	local x , y = cityNode:getPosition()
	local paoma = self:getMarchPaoMaEffect(march)
	if not paoma then
		return 
	end
	local x1, y1 = paoma:getPosition()
	if cc.Vec(x1-x,y1-y):length() > 10 then
		local view = LegionArrowView:create({
			startPoint = cc.Vec(x,y),
			march = march,
			speed = 200,
			scene = self
		})
		self.m_mapLayer:addChild(view,LegionMapZorder.kArrow)
	end
end

---------------------------------------------------------
-- function This:onRebuildCity(city)
-- 	local view = self:getCityView(city:getId())
-- 	if view then
-- 		view:refresh()
-- 	end
-- end

function This:onCityInfoChanged(city)
	local view = self:getCityView(city:getId())
	if view then
		view:refresh()
		view:setSelected(false) 
	end
end

function This:onAddMarch(march)
	if not march:isVisibleOnMap() then
		return
	end
	self:addMarchPathView(march)
	local view = self:getCityView(march:getToCity())
	if view then
		view:onAddMarch(march)
	end
end

function This:onUpdateMarch(march)
	if not march:isVisibleOnMap() then
		return
	end
	local id = march:getId()
	local view = self.m_marchViews[id]
	if not view then
		view = LegionMarchPathView:create({
			march = march,
			map = self.m_map,
		})
		self.m_mapLayer:addChild(view,LegionMapZorder.kMarch)
		self.m_marchViews[march:getId()] = view
	else
		view:refresh()
	end
end

function This:onDeleteMarch(march)
	if not march:isVisibleOnMap() then
		return
	end
	self:removeMarchPathView(march)
	local view = self:getCityView(march:getToCity())
	view:onDeleteMarch(march)
end

function This:onLegionExitMap(msg)
	if msg.group_id == self.m_map:getMyLegionId() then
		g_scene:openDialog("Map.LegionExitMapPanel",{
			msg = msg,
			map = self.m_map
		})
	end
end

function This:onRescueMapStartFight()
	for _ , view in pairs(self.m_cityView) do
		view:refreshProtectedState()
	end
end

function This:refreshMapCityUI()
	for _ , view in pairs(self.m_cityView) do
		view:refreshCityUI()
	end
end

function This:onKeyDown(keyCode)
	if keyCode == '4' then

	end
end
