-- @Author: superm
-- @Date:   2019-03-20 12:03:35
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-08 16:24:00
-- @File Name: RescueEmperorUI.lua
-- @Description: 

local This , super = class(...,"Home.HomeUI")
local RescuBianDuiNode = require("RescueEmperor.RescuBianDuiNode")
local LegionMarchView = require("Map.LegionMarchView")
local LegionMarchDangerView = require("Map.LegionMarchDangerView")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local VoiceChatManager = require("VoiceChat")
local RescuBoxAndRank = require("RescueEmperor.RescuBoxAndRank")
local MiniMapNode = require("RescueEmperor.RescueMiniMapNode")

function This:init(info)
	super.init(self, info)
	self.m_map = info.map
end

function This:initUI()
	self.m_scene = g_scene
	self.m_csNode = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorBuildMain.csb")
	self:fixScreen()
	self:addChild(self.m_csNode)
	local info = {node = self,map = self.m_map}
	self.m_boxTip = RescuBoxAndRank:create(info)
	self:addChild(self.m_boxTip)

	self.m_csNode.selectcity:setVisible(false)
	self.m_csNode.smallmap.voice.icon2:setVisible(false)

	self.m_cityNodes = {}
	self.m_cityEffect = {}
	self.m_isInVoiceRoom = false
	self.m_taskIsShow = true
	self.m_rectW , self.m_rectH = 95, 50
end


function This:fixScreen()
	self.m_csNode.smallmap:recoveryToFixedWidth("right","top")
	self.m_csNode.dingbg:recoveryToFixedWidth("left","top")
	self.m_csNode.player_info:recoveryToFixedWidth("left","top")
	self.m_csNode.chat_node:recoveryToFixedWidth("left","bottom")
	self.m_csNode.Dangerous:recoveryToFixedWidth("right","top")
	self.m_csNode.uieffect:recoveryToFixedWidth("right","top")
	self.m_csNode.goto:recoveryToFixedWidth("right","bottom")

	self.m_csNode.biandui:recoveryToFixedWidth("right","")
	-- if GamePlayer:isIPhioneX() then
	-- 	self.m_csNode.march:setPositionX(50)
	-- end
end

function This:onEnter()
	super.onEnter(self)

	self:setName("sceneUI")
	self:initTroopTreamNode()
	-- self:initMarchArmy()
	self:initDangerous()
    self:initMiniMap()
	cc.Application:addListener(self)
end

function This:addMapManager()
	self.m_map:addObserver(self)
end

function This:removeMapManager()
	self.m_map:removeObserver(self)
end

function This:onExit()
	cc.Application:removeListener(self)
	VoiceChatManager:setJoinRoomCallback(function(memeberId)
		if memeberId > 0 then
			self.m_scene:showTips(Serialize["world_WorldUI_1"])
			self.m_isInVoiceRoom = true
			self.m_csNode.smallmap.voice.icon2:setVisible(true)
		else
			self.m_scene:showTips(Serialize["world_WorldUI_2"])
			self.m_isInVoiceRoom = false
			self.m_csNode.smallmap.voice.icon2:setVisible(false)
		end
	end)
	super.onExit(self)
end

function This:initTouchEvent()
	super.initTouchEvent(self)
	local csNode = self.m_csNode
	csNode["goto"]:setTouch({
	    isScale = false, 
		isDynamic = true, 
		onEnded = function()
			cc.SceneManager:switchTo("Home.MenuScene")
	    end,
	})
	local smallMap = csNode.smallmap
	smallMap.map_bg:setTouch({
		onEnded = function()
			self.m_scene:openDialog("Map.LegionMiniMapPanel",{
				mapRadioX = 2.32,
				mapRadioY = 2.32,
				map = self.m_map,
				scene = self.m_scene
			})
		end,
		isAlpha = false,
		isScale = false
	})
	smallMap.help:setTouch({
		onEnded = function()
        	g_scene:openDialog("RescueEmperor.RescueWarRulePanel")
		end,
		isScale = false
	})
	smallMap.search:setTouchEnded(function()
		self.m_scene:openDialog("Map.LegionSearchCityPanel",{
			map = self.m_map
		})
	end)

	csNode.selectcity.close:setTouchEnded(function()
		self.m_scene:exitSelectMode()
	end)
	
	smallMap.voice:setTouchEnded(function()
		if not self.m_isInVoiceRoom then
			local mapId = GamePlayer:getLegionMapId()
			if mapId > 0 then
				local key = "map_" .. GameUtil:int64ToString(GamePlayer:getLegionMapId())
				VoiceChatManager:joinTeamRoom(key)
			end
		else
			VoiceChatManager:exitRoom()
		end
	end)
end

function This:refreshTaskTab(tabType)
    if not tabType then
        return
    end
    self.m_normalTabType = tabType
    for tabIndex=1,LegionUITabName.ktep2 do
        self.m_csNode.target["tep"..tabIndex].light:setVisible(tabType ==tabIndex)
        self.m_csNode.target["item"..tabIndex]:setVisible(tabType ==tabIndex)
    end
end

function This:refreshJtdsVisible()
	local jtdsIsEnd = AchieveManager:getJtdsIsEnd()
	local item2 = self.m_csNode.target.item2
	item2.juntuandashi:setVisible(not jtdsIsEnd)
	item2.zhanlingluoyang:setVisible(jtdsIsEnd)
end

function This:initTroopTreamNode()
	self.m_bianduiNode = RescuBianDuiNode:create({
		map = self.m_map
	})
	self.m_csNode.biandui:addChild(self.m_bianduiNode)
	self.m_bianduiNode:setScale(0.6)
end

-- function This:initMarchArmy()
-- 	local marchList = self.m_map:getMarchList(function(march)
-- 		return march:isMy()
-- 	end)
-- 	for i , march in ipairs(marchList) do
-- 		local view = LegionMarchView:create({
-- 			march = march,
-- 			map = self.m_map
-- 		})
-- 		self.m_csNode.march.ScrollView:pushBackCustomItem(view,150,30,0.5)
-- 	end
-- 	-- self:refreshModelVisible()
-- end

function This:initMiniMap()
	local info = { 
		map = self.m_map, 
		scaleRateX = 195 / 560,
		scaleRateY = 163 / 466,
		mapRadioX = 2.1,
		mapRadioY = 2.1,
		mapNode = self.m_csNode.smallmap.map, 
	}
	local liangDianNode = MiniMapNode:create(info)
	self:addChild(liangDianNode)
	liangDianNode:refresh()
	self.m_liangDianNode = liangDianNode

	
	self.m_rectNode = cc.DrawNode:create()
	self.m_rectNode:drawRect(cc.Rect(0,0,self.m_rectW,self.m_rectH),cc.Color(255,255,0,255))
	self.m_csNode.smallmap.map.img:addChild(self.m_rectNode)

	local rx , ry = self.m_scene:getViewportCenterRatio()
	local w , h = self.m_csNode.smallmap.map.img:getContentSize()
	self.m_rectNode:setPosition(rx * w - self.m_rectW / 2, ry * h - self.m_rectH / 2)
end

function This:initDangerous()
	local marchList = self.m_map:getEnemyMarchArmyToMyCity()
	table.sort(marchList,function(a,b)
		return a:getStartTime() < b:getStartTime()
	end)
	local cityList = {}
	local citys = {}
	for i , march in ipairs(marchList) do
		local toCityId = march:getToCity()
		if not citys[toCityId] then
			table.insert(cityList,toCityId) 
			citys[toCityId] = {march}
		else
			table.insert(citys[toCityId],march)
		end
	end

	self.m_dangerViewList = {}
	local total = #self.m_dangerViewList 
	local fromIndex = total >= MaxDangerNum and total - MaxDangerNum + 1 or 1

	for i = fromIndex , #cityList  do
		local cityId = cityList[i]
		self:addDangerView(citys[cityId],cityId)
	end
end

function This:addDangerView(armyList,cityId,isInit)
	local function refreshContent()
		local w = 0
		for i , view in ipairs(self.m_dangerViewList) do
			view:setPositionX(w)
			w = w - view:getContentWidth()
		end
	end
	local function removeHandler(cityId)
		for i , view in ipairs(self.m_dangerViewList) do
			if view:getCityId() == cityId then
				view:removeFromParent()
				table.remove(self.m_dangerViewList,i)
				refreshContent()
				self:refreshModelVisible()
				break
			end
		end
	end

	local view = LegionMarchDangerView:create({armyList = armyList,map = self.m_map, cityId = cityId })
	view:setAnchorPoint(1.0,0.5)
	view:setRemoveHandler(removeHandler)
	view:setLookedHandler(refreshContent)
	view:setTouchHandler(function(id)
		self.m_scene:focuseCity(id)
	end)
	self.m_csNode.Dangerous:addChild(view)

	if #self.m_dangerViewList >= MaxDangerNum then
		local tmpView = self.m_dangerViewList[1]
		tmpView:removeFromParent()
		table.remove(self.m_dangerViewList,1)
	else
		table.insert(self.m_dangerViewList,view)
	end
	self.m_csNode.Dangerous:setVisible(#self.m_dangerViewList > 0)
	refreshContent()
	return view
end

function This:updateMiniMap(rx,ry)
	local cw, ch = self.m_csNode.smallmap.map:getContentSize()
	local w , h = self.m_csNode.smallmap.map.img:getContentSize()
	local y = ry * h - self.m_rectH / 2
	local topy = y + self.m_rectH
	if topy > ch then
		self.m_csNode.smallmap.map.img:setPositionY(ch - topy)
	end
	local x = rx * w - self.m_rectW / 2
	x = math.min(w - self.m_rectW, math.max(0,x))
	y = math.min(h - self.m_rectH, math.max(0,y))
	self.m_rectNode:setPosition(x,y)
end


function This:refreshModelVisible()
    local sceneType = self.m_scene:getType()
    if sceneType ~= SceneType.kResueScene then
		return    
    end

	local mode = self.m_scene:getShowMode()
	self.m_csNode.player_info:setVisible(mode == ShowMode.kNormal)
	self.m_refreshModelVisible_count = (self.m_refreshModelVisible_count or 0) + 1
	self.m_csNode.chat_node:setVisible(mode == ShowMode.kNormal)
	self.m_csNode.selectcity:setVisible(mode == ShowMode.kSelect)
	self.m_csNode.Dangerous:setVisible(mode == ShowMode.kNormal)
	self.m_csNode["goto"]:setVisible(mode == ShowMode.kNormal)
	self.m_csNode.smallmap:setVisible(mode == ShowMode.kNormal)
	self.m_csNode.biandui:setVisible(mode == ShowMode.kNormal)
	self.m_csNode.Dangerous:setVisible(#self.m_dangerViewList > 0)
	self.m_csNode.kills:setVisible(mode == ShowMode.kNormal)
end

function This:refreshTime()
	-- local curTime = GamePlayer:getServerTime()
	-- local curDate = os.date("*t",curTime)
	-- local fightTime = self.m_map:getFightTime()
	-- if curTime < fightTime then
	-- 	local fightDate = os.date("*t",fightTime)
	-- 	if curDate.day ~= fightDate.day then
	-- 		self.m_csNode.smallmap.timetips.time:setString(string.format("明日%d:%d:%d开战",fightDate.hour,fightDate.min,fightDate.sec))
	-- 	else
	-- 		self.m_csNode.smallmap.timetips.time:setString(string.format("%d:%d:%d开战",fightDate.hour,fightDate.min,fightDate.sec))
	-- 	end
	-- else
	-- 	local time = self.m_map:getEndTime()
	-- 	local str = {"一","二","三","四","五","六","日"}
	-- 	self.m_csNode.smallmap.timetips.time:setString(string.format("周%s%d:%d:%d结束",str[time.wday],time.hour, time.min,time.sec))
	-- end
end

-- function This:refreshBianDui()
-- 	local bainduiNode = self.m_bianduiNode
-- 	if bainduiNode then
		
-- 	end
-- end

-- function This:refresh(field)
-- 	super.refresh(self,field)
-- 	self:refreshBianDui()
-- end

function This:onGetBoxReward()
	local boxTip = self.m_boxTip
	if boxTip then
		boxTip:refreshBoxState()
	end
end
