-- @Author: superm
-- @Date:   2019-03-21 10:52:57
-- @Last Modified by:   superm
-- @Last Modified time: 2019-10-21 22:10:37
-- @File Name: RescueMapManager.lua
-- @Description: 
local This , super = class(...,"Base.BaseModel")
local LegionMapRoomConfig = require("Metadatas.LegionMapRoomConfig")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local RescuCity = require("RescueEmperor.RescuCity")
local LegionTroop = require("Map.LegionTroop")
local LegionGroup = require("Map.LegionGroup")
local LegionMarch = require("Map.LegionMarch")
local GamePlayer = require("Player.GamePlayer")
local LegionBianDui = require("Map.LegionBianDui")
local ArgConfig = require("Metadatas.ArgConfig")
local FamilyManager = require("Family.FamilyManager")
local HeroManager = require("Hero.HeroManager")
local PlayTypeConfig = require("Metadatas.PlayTypeConfig")

local GameMapManager = require("Map.GameMapManager")
local GameUtil = require("Util.GameUtil")

local LegionWarBoxConfig = require("Metadatas.LegionWarBoxConfig")
local RescueEmperorBoxRewardConfig = require("Metadatas.RescueEmperorBoxRewardConfig")
local CeilsConfig = require("RescueEmperor.RescueCeilsConfig")
local RescueEmperorCityConfig = require("Metadatas.RescueEmperorCityConfig")
local ReportManager = require("Report.ReportManager")
local RescueBoxManager = require("RescueEmperor.RescueBoxManager")

local socketSend = function ( cmd, ... )
	if not g_setting.isWarTest then
		Socket:sendAndLock(cmd, ...)
	else
		EventDispatcher:dispatcher(tostring(cmd), ...)
	end
end

function This:getMapLv()
	return self.m_level
end

function This:init(mapType)
	super.init(self)
	self.m_mapType = mapType
	self.m_level = 1
	-- self.m_config = nil
	self.m_citys = {}
	self.m_troops = {} --部队
	self.m_legions = {}--军团信息列表

	self.m_marchs = {}--行军
	self.m_biandui = {}--编队
	-- self.m_warBoxAppearCitys = {}--ciytid{ciytid1,ciytid2,ciytid3}
	-- self.m_warBoxId = 0
	self.m_maxBianDuiCount = 0
	self.m_escape = 0
	self.m_myLegionId = 0
	self.m_isInit = false
	self.m_gameWarShopData = nil
	self.m_showHelpWhenEnterMap = true

	self.m_boxManager = RescueBoxManager
	EventDispatcher:addListener(self, 14000,self.onGetMapInfoReply)
end

function This:initListener()
	EventDispatcher:addListener(self,14001,self.onRefreshTroopNotify)
	EventDispatcher:addListener(self,14002,self.onCityInfoChangeNotify)
	EventDispatcher:addListener(self,14003,self.onUpdateMapRoleNotify)
	EventDispatcher:addListener(self,14004,self.onUpdateBianDuiReply)

	EventDispatcher:addListener(self,14005,self.onGarrisonReply)

	EventDispatcher:addListener(self,14006,self.onUndoAddTroopHpReply)
	EventDispatcher:addListener(self,14007,self.onAddHpSpeedTroopReply)
	EventDispatcher:addListener(self,14012,self.onClearBianDuiHpReply)
	EventDispatcher:addListener(self,14008,self.onAddHpFinishNotify)
	EventDispatcher:addListener(self,14009,self.onCreateMarchReply)
	EventDispatcher:addListener(self,14010,self.onMarchChangedNotify)
	EventDispatcher:addListener(self,14014,self.onClearChengFangCreateTimeReply)
	EventDispatcher:addListener(self,14015,self.onTrainChengFangOnceKeyReply)
	EventDispatcher:addListener(self,14013,self.onTrainChengFangReply)
	EventDispatcher:addListener(self,14016,self.onMarchAddSpeedReply)
	EventDispatcher:addListener(self,14017,self.onRebuildCityReply)
	EventDispatcher:addListener(self,14018,self.onRemoveCityBuildReply)
	EventDispatcher:addListener(self,14019,self.onUpgradeTechReply)
	EventDispatcher:addListener(self,14020,self.onChangeTechStateReply)
	EventDispatcher:addListener(self,14021,self.onTechAddSpeedChangeNotify)
	-- EventDispatcher:addListener(self,14022,self.onGetCityBattleReportReply)
	EventDispatcher:addListener(self,14023,self.onGeRenInfoChangedNotify)
	EventDispatcher:addListener(self,14024,self.onLegionExitMapNotity)
	EventDispatcher:addListener(self,14026,self.onInvestigateCityInfoReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,14000)
	EventDispatcher:removeListener(self,14001)
	EventDispatcher:removeListener(self,14002)
	EventDispatcher:removeListener(self,14003)
	EventDispatcher:removeListener(self,14004)
	EventDispatcher:removeListener(self,14005)
	EventDispatcher:removeListener(self,14006)
	EventDispatcher:removeListener(self,14007)
	EventDispatcher:removeListener(self,14017)
	EventDispatcher:removeListener(self,14008)
	EventDispatcher:removeListener(self,14009)
	EventDispatcher:removeListener(self,14010)
	EventDispatcher:removeListener(self,14011)
	EventDispatcher:removeListener(self,14014)
	EventDispatcher:removeListener(self,14015)
	EventDispatcher:removeListener(self,14016)
	EventDispatcher:removeListener(self,14017)
	EventDispatcher:removeListener(self,14018)
	EventDispatcher:removeListener(self,14019)
	EventDispatcher:removeListener(self,14020)
	EventDispatcher:removeListener(self,14021)
	-- EventDispatcher:removeListener(self,14022)
	EventDispatcher:removeListener(self,14023)
	EventDispatcher:removeListener(self,14026)
end


function This:isInit()
	return self.m_isInit
end

function This:getId()
	return self.m_id
end

function This:getMapType()
	return self.m_mapType
end

function This:getCityCeilConfig()
    return CeilsConfig
end

function This:getCityConfig()
    return RescueEmperorCityConfig
end

function This:getCityInitLevel()
	return 1
end

function This:getCityMaxLevel()
	return 1
end

function This:getCityRange()
	return 1 , 19
end

function This:getCity(cityId)
	if not cityId then
		assert(false)
	end
	local city = self.m_citys[cityId]
	if not city then
		city = RescuCity:create({
			id = cityId,
			map = self
		})
		self.m_citys[cityId] = city
	end
	return city
end

function This:getLegionColor(legionId)
	local colors = {
		cc.Color.ORANGE,
		cc.Color.GREEN,
		cc.Color.BLUE,
		cc.Color.RED
	}

	if not self.m_cityColors then
		self.m_cityColors = {}
	end

	if not self.m_cityColors[legionId] then
		local legions = self:getAllLegions()
		local count = 1
		for id, _ in pairs(legions) do
			if not self.m_cityColors[id] then
				self.m_cityColors[id] = colors[count]
			end
			count = count + 1
		end
	end
	return self.m_cityColors[legionId]

end

function This:getAllCitys()
	return self.m_citys
end

--获取部队
function This:getTroop(troopId)
	return self.m_troops[troopId]
end

--获取编队
function This:getBianDui(index)
	return self.m_biandui[index]
end

function This:getAllBianDui()
	return self.m_biandui
end

--获取最大编队数
function This:getMaxBianDuiCount()
	return self.m_bianduiMaxCount
end

function This:getBianDuiCount()
	local count = 0
	if self.m_biandui then
		for i , biandui in ipairs(self.m_biandui) do
			if not biandui:isEmpty() then
				count = count + 1
			end
		end
	end
	return count
end

function This:getLegion(legionId)
	return self.m_legions[legionId]
end

function This:getAllLegions()
	return self.m_legions
end

function This:getMyLegion()
	if self.m_myLegionId and self.m_myLegionId > 0 then
		return self.m_legions[self.m_myLegionId]
	end
--	assert(false)
	return nil
end

function This:getMyLegionId()
	return self.m_myLegionId
end

function This:getMyLegionCityId()
	return self.m_myLegionCityId
end

function This:getMyLegionCity()
    return self.m_citys[self.m_myLegionCityId]
end

function This:getQuitLegionList()
	local result = {}
	-- for _, info in pairs(self.m_legionsInRank) do
 --        result[info.rank] = info
 --    end
    return result
end

--获取行军
function This:getMarch(marchId)
	return self.m_marchs[marchId]
end

function This:getMarchList(selector)
	local result = {}
	for i , march in pairs(self.m_marchs) do
		if not selector or selector(march) then
			table.insert(result,march)
		end
	end
	return result
end

function This:getEnemyMarchArmyToMyCity()
	return self:getMarchList(function(march)
		if not march:isOur() then
			local city = self:getCity(march:getToCity())
			return city:isOur()
		end
		return false
	end)
end

function This:getRole(id)
	for legionId, legion in pairs(self.m_legions) do
		local role = legion:getRole(id)
		if role then
			return role
		end
	end
end

function This:getStartTime()
	return self.m_fightTime - PlayTypeConfig[self:getMapType()].protect_time
end

function This:getFightTime()
	return self.m_fightTime or 0
end

function This:getJueZhanTime()
	return self.m_juezhanTime or 0
end

--是否到达决胜期
function This:isToJueShengTime()
	local curTime = GamePlayer:getServerTime()
	return curTime >= self.m_juezhanTime
end

function This:getEndTime()
	local curTime = GamePlayer:getServerTime()

	local t = PlayTypeConfig[self:getMapType()].fight_end_time
	return {
		wday = t[1],
		hour = t[2],
		min = t[3],
		sec = t[4]
	}
end

function This:getBoxManager()
	return self.m_boxManager
		end


function This:exit()
	self.m_isInit = false
	self:removeListener()
end

function This:enter()
	local routine = coroutine.create(function(synchro)
    	self.m_routine = synchro
		self:onGetMapInfoRequest()
		local result = coroutine.yield()
		if result then
			cc.SceneManager:switchTo("RescueEmperor.RescueEmperorScene",nil,{
				map = self
			})
		end
	end)
	coroutine.resume(routine, routine)
end

function This:setShowHelpWhenEnterMap(ok)
	self.m_showHelpWhenEnterMap = ok
end

function This:needShowHelpWhenEnterMap()
	return self.m_showHelpWhenEnterMap
end

function This:onGetMapInfoRequest()
	socketSend(14000,{})
end

function This:onGetMapInfoReply(msg)
	if self.m_routine then
		coroutine.resume(self.m_routine, msg.err_code == 0)
		self.m_routine = nil
	end
	-- print("==== onGetMapInfoReply")
	-- print_tmp(msg)
	if msg.err_code ~= 0 then
		self:notifyObservers("onGetMapInfoFail", msg) --界面有特殊处理
		-- g_scene:handleErrorCode(msg.err_code)
		return 
	end
	local map = msg.map
	self.m_legions = {}
	self.m_marchs = {}
	self.m_junces = {}
	self.m_citys = {}
	self.m_id = msg.map.id
	self.m_mapType = msg.map.map_type
	self.m_level = msg.map.map_level or 1
	self.m_config = LegionMapRoomConfig[self.m_mapType * 1000 + self.m_level]

	self.m_fightTime = msg.map.fight_time
	self.m_escape = 0

	local curTime = GamePlayer:getServerTime()
	if self.m_fightTime > curTime then
		cc.Application:addToUpdateList(self)
	end

	self.m_myLegionId = map.geren.group_id

	for _, info in ipairs(map.group_list) do
		local groupId = info.group_id
		local legion = self.m_legions[groupId]
		if not legion then
			legion = LegionGroup:create({
				map = self
			})
			self.m_legions[groupId] = legion
		end
		legion:updateInfo(info)
		if self.m_myLegionId == groupId then
			self.m_myLegionCityId = info.main_city
		end
	end
	print("主城id:" .. self.m_myLegionCityId)
	if map.quit_group_list then
		for _, info in ipairs(map.quit_group_list) do
			local groupId = info.group_id
			if self.m_legions[groupId] then
				self.m_legions[groupId] = nil
			end
		end
	end

	for i , info in ipairs(map.troop_list) do
		local troop = self.m_troops[info.id]
		if not troop then
			troop = LegionTroop:create({
				map = self
			})
			self.m_troops[info.id] = troop
		end
		troop:updateInfo(info)
	end

	for i , info in ipairs(map.city_list) do
		local city = self.m_citys[info.city_id]
		if not city then
			local cityId = info.city_id
			city = RescuCity:create({
				id = cityId,
				map = self
			})
			self.m_citys[info.city_id] = city
		end
		city:updateInfo(info)
	end

	for i , info in ipairs(map.march_list) do
		if info.endtime > curTime then
			local march = self.m_marchs[info.id]
			if not march then
				march = LegionMarch:create({
					map = self
				})
				self.m_marchs[info.id] = march
			end
			march:updateInfo(info)
		end
	end

	self.m_bianduiMaxCount = map.geren.max_count
	self.m_biandui = {}

	for i = 1, self.m_bianduiMaxCount do
		self.m_biandui[i] = LegionBianDui:create({
			map = self,
			pos = i
		})
	end

	for i , info in ipairs(map.geren.biandui_list) do
		local biandui = self.m_biandui[info.pos]
		biandui:updateInfo(info)
	end

	self.m_isInit = true
	self.m_boxManager:setMapInfo({map = self,mapInfor = map})
	self:initListener()
	self:notifyObservers("onGetLegionMapInfo",self)
end

-------------------------------刷新部队-----------------------
function This:onRefreshTroopNotify(msg)
	local updateList = {}
	for i , info in ipairs(msg.update_troop_list) do
		local troop = self.m_troops[info.id]
		if not troop then
			troop = LegionTroop:create({
				map = self
			})
			self.m_troops[info.id] = troop
		end
		troop:updateInfo(info)
		print(string.format("更新部队:%s",GameUtil:int64ToString(info.id)))
		self:notifyObservers("onUpdateTroop",troop)
	end

	for i , id in ipairs(msg.delete_troop_list) do
		local troop = self.m_troops[id]
		if self.m_troops[id] then
			self.m_troops[id] = nil
			self:notifyObservers("onDeleteTroop",troop)
			print(string.format("删除部队:%s",GameUtil:int64ToString(id)))
		end
	end
end

-------------------------------刷新玩家---------------------
function This:onUpdateMapRoleNotify(msg)
	local roles = {}
	for i , info in ipairs(msg.role_list) do
		local legion = self:getLegion(info.group_id)
		local role = legion:getRole(info.base.id)
		role:updateInfo(info)
		roles[info.base.id] = role
	end
	self:notifyObservers("onUpdateMapRole",roles)
end

-------------------------------刷新编队------------------------
function This:onUpdateBianDuiRequest(index,cardList,name)
	local param =	{
		index = index,
		card_list = cardList,
		name = name
	}
	socketSend(14004, param)
end

function This:onUpdateBianDuiReply(msg)
	if msg.err_code == 0 then
		local biandui = self.m_biandui[msg.biandui_info.pos]
		biandui:updateInfo(msg.biandui_info)
		self:notifyObservers("onUpdateBianDuiTroop",msg.biandui_info.pos,biandui)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end
-------------------------编队---------------------------------
function This:onGarrisonRequest( index, cityId )
	local param = {
		index = index,
		city_id = cityId
	}
	socketSend(14005, param)
end

function This:onGarrisonReply( msg )
	if msg.err_code == 0 then
		local biandui = self:getBianDui(msg.index)
		self:notifyObservers("onGarrison", biandui)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- function This:onAddBianDuiHpReply(msg)
-- 	if msg.err_code == 0 then
-- 		local biandui = self:getBianDui(msg.index)
-- 		self:notifyObservers("onAddBianDuiRes",biandui)
-- 	else
-- 		g_scene:handleErrorCode(msg.err_code)
-- 	end
-- end

----------------------------创建行军---------------------------
function This:onCreateMarchRequest(info)
	socketSend(14009,{
		index = info.index,
		res_type = info.res_type,
		res_list = info.res_list,
		from_city = info.from_city,
		to_city = info.to_city
	})
end

function This:onCreateMarchReply(msg)
	if msg.err_code == 0 then
		local march = self.m_marchs[msg.march_id]
		self:notifyObservers("onCreateMarch",march)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-------------------------行军变化通知---------------------------
function This:onMarchChangedNotify(msg)
	local curTime = GamePlayer:getServerTime()
	for i , info in ipairs(msg.update_march_list) do
        if not info.res_type or info.res_type <= 0 then 
            info.res_type = LegionTransType.kHp
        end
		local march = self.m_marchs[info.id]
		if not march then
			if info.endtime < curTime then
				assert(false)
			end
			march = LegionMarch:create({
				map = self
			})
			self.m_marchs[info.id] = march
			march:updateInfo(info)
			self:notifyObservers("onAddMarch",march)
		else
			march:updateInfo(info)
			self:notifyObservers("onUpdateMarch",march)
		end
	end

	for i , id in ipairs(msg.arrive_march_list) do
		local march = self.m_marchs[id]
		if march then
			self.m_marchs[id] = nil
			march:clear()
			self:notifyObservers("onDeleteMarch",march)
		end
	end
end

-------------------------城市信息变化-------------------------
function This:onCityInfoChangeNotify(msg)
	for i , info in ipairs(msg.city_list) do
		local city = self:getCity(info.city_id)
		local oldState = city:getState()
		local oldLegionId = city:getLegionId()

		city:updateInfo(info)
		local newState = city:getState()
		local newLegionId = info.group_id

		if oldState ~= newState then
			if newState == LegionCityState.kFighting then
				self:notifyObservers("onCityStartFight",city)
			elseif oldState ~= LegionCityState.kFighting and newState == LegionCityState.kFighting then
				self:notifyObservers("onCityEndedFight",city)
			end
			print(string.format("***************************city:%d state -> (%d,%d)********************",info.city_id,oldState,newState))
		end

		if oldLegionId ~= newLegionId then
			self:notifyObservers("onCityOwnerChanged",city)
		end

		self:notifyObservers("onCityInfoChanged",city)
	end
end

---------------------------请求城市战报--------------------------
function This:onGetCityBattleReportRequest(cityId)
	ReportManager:onGetCityBattleReportRequest(cityId)
	-- socketSend(14022,{
	-- 	city_id = cityId
	-- })
end

-- function This:onGetCityBattleReportReply(msg)
-- 	if msg.err_code == 0 then

-- 		self:notifyObservers("onGetCityBattleReport",msg)
-- 	else
-- 		g_scene:handleErrorCode(msg.err_code)
-- 	end
-- end

--------------------------个人信息变化------------------------------
function This:onGeRenInfoChangedNotify(msg)
	self.m_bianduiMaxCount = msg.geren.max_count

	for i = 1, self.m_bianduiMaxCount do
		if not self.m_biandui[i] then
			self.m_biandui[i] = LegionBianDui:create({
				map = self,
				pos = i
			})
		end
	end

	for i , info in ipairs(msg.geren.biandui_list) do
		local biandui = self.m_biandui[info.pos]
		biandui:updateInfo(info)
	end

	self.m_myLegionId = msg.geren.group_id
	self.m_maxBianDuiCount = msg.geren.max_count
	self:notifyObservers("onGeRenInfoChanged")
end

--侦查城市
function This:onInvestigateCityInfoRequest(cityId)
	Socket:sendAndLock(14026,{city_id = cityId})
end

function This:onInvestigateCityInfoReply(msg)
	-- print_tmp(msg)
	if msg.err_code == 0 then
		local city = self:getCity(msg.city_id)
		if city then
			city:setInvestigateCityInfo(msg)
		end
		self:notifyObservers("onInvestigateCity",city)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

---------------------------军团退出通知----------------------------
function This:onLegionExitMapNotity(msg)
	local legion = self.m_legions[msg.group_id]
	self:notifyObservers("onLegionExitMap",msg)
	if self.m_myLegionId == msg.group_id then
		self:exit()
	end
	self.m_legions[msg.group_id] = nil
    self:notifyObservers("onLegionExitMapAfter")
end


function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1.0 then
		return
	end
	self.m_escape = self.m_escape - 1
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_fightTime and self.m_fightTime < curTime + 1 then
		cc.Application:removeFromUpdateList(self)
		self:notifyObservers("onRescueMapStartFight")
	end
end
