-- @Author: superm
-- @Date:   2019-03-29 18:26:24
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-30 14:41:20
-- @File Name: RescueMiniMapNode.lua
-- @Description: 

local This , super = class(...,"NodeUI")

local CityFlag = {
    [LegionBuildType.kArrowTower] = "WarScene/town_0.png",
    [LegionBuildType.kBarracks] = "WarScene/town_0.png",
    [LegionBuildType.kBoltTower] = "WarScene/town_0.png",
    [LegionBuildType.kFarmland] = "WarScene/town_0.png",
    [LegionBuildType.kMainCity] = "WarScene/town_0.png",
    [LegionBuildType.kMarket] = "WarScene/town_0.png",
    [LegionBuildType.kMintage] = "WarScene/town_0.png",
    [LegionBuildType.kNone] = "WarScene/town_0.png",
    [LegionBuildType.kWarehouse] = "WarScene/town_0.png",
}

function This:initNode(info)
    if info then
        local map = info.map
        local mapNode = info.mapNode.img
        local scaleRateX = info.scaleRate or info.scaleRateX
        local scaleRateY = info.scaleRate or info.scaleRateY
        local mapRadioX, mapRadioY = info.mapRadioX + 0.22, info.mapRadioY + 0.22

        self.m_scene = g_scene
        self.m_map = map
        self.m_mapNode = mapNode
        self.m_mapContainNode = info.mapNode
        self.m_liangNode = cc.Node:create()
        mapNode:addChild(self.m_liangNode)
        self.m_ceilsNode = cc.Node:create()
        mapNode:addChild(self.m_ceilsNode)
        self.m_scaleRateX = scaleRateX
        self.m_scaleRateY = scaleRateY

        self.m_ceilConfig = map:getCityCeilConfig()
        local w , h = mapNode:getContentSize()
        local dx , dy = 20 * mapRadioX * scaleRateX, 12 * mapRadioY * scaleRateY
        local f = dy / dx
        local nx , ny = math.ceil(w / dx), math.ceil(h / dy)
        self.m_nx = nx
        self.m_ny = ny
        self.m_dx = dx
        self.m_dy = dy
        self.m_f = f
        self.m_vy = cc.Vec(self.m_dx,self.m_dy):normalize()
        self.m_vx = cc.Vec(-self.m_dx,self.m_dy):normalize()
    end
end

function This:refresh()
   self:initCitys()
   self:initCeils()
end

--色块
function This:initCeils()
    local map = self.m_map
    if map then
        self.m_ceilsNode:removeAllChildren()
        self.m_cityNum = {}
        local citys = map:getAllCitys()
        for cityId , city in pairs(citys) do
            local cityConfigId = cityId > 1000 and cityId - 1000 or cityId
            local config = self.m_ceilConfig[cityConfigId]
            local legionId = city:getLegionId()
            local legionColor = self:getLegionColor(legionId)
            if legionId > 0 then
                self.m_cityNum[legionId] = (self.m_cityNum[legionId] or 0) + 1
                for j , ceilKey in ipairs(config.ceilList) do
                    local nx = math.floor(ceilKey / 1000)
                    local ny = ceilKey % 1000
                    local sprite = self:createCeil(nx,ny,legionColor or cc.Color(0,0,0,0))
                end
            end
        end
    end
end

--白点
function This:initCitys()
    local map = self.m_map
    if map then
        local liangNode = self.m_liangNode
        liangNode:removeAllChildren()
        local citys = map:getAllCitys()
        for cityId, city in pairs(citys) do
            local rx , ry = self.m_scene:getCityNodeRadio(cityId)
            local buildType = city:getBuildType()
            local sprite = cc.Sprite:create(CityFlag[buildType])
            liangNode:addChild(sprite)
            sprite:setScale(self.m_scaleRateX, self.m_scaleRateY)
            local w , h = self.m_mapNode:getContentSize()
            sprite:setPosition(w * rx, h * ry)
        end
    end
end

function This:createCeil(nx,ny,color)
    local b = cc.Vec(self.m_dx, self.m_dy):length() / 2
    local t = self.m_vy * b * (ny + 0.5) + self.m_vx * b * (nx + 0.5)
    local layer = cc.Sprite:create("LegionScene/lingxing.png")
    layer:setPosition(t.x,t.y)
    layer:setColor(color.r,color.g,color.b)
    layer:setOpacity(128)
    layer:setScale(self.m_dx / 250)
    
    self.m_ceilsNode:addChild(layer)
    return layer
end

function This:getLegionColor(legionId)
    return self.m_map:getLegionColor(legionId)
end

function This:getLegionCityNums()
    return self.m_cityNum or {0,0,0,0}
end
