-- @Author: Su
-- @Date:   2019-03-22 14:44:07
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-03 11:29:14
-- @File Name: RescueReportPanel.lua

local This , super = class(...,"Dialog")
local RescueReportItemView = require("RescueEmperor.RescueReportItemView")
local GamePlayer = require("Player.GamePlayer")
local ReportManager = require("Report.ReportManager")
function This:initNode(info)
    self.m_cityId = info.cityId
    self.m_map = info.map
    self.m_csNode = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorReport.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    self.m_attackCount = 0--进攻
    self.m_defendCount = 1
    self.m_fightList = {}
    self.m_map:onGetCityBattleReportRequest(self.m_cityId)
--    self:refresh()

end

 function This:onEnter()
     super.onEnter(self)
     self.m_map:addObserver(self)
     ReportManager:addObserver(self)
     
 end

 function This:onExit()
     super.onExit(self)
     self.m_map:removeObserver(self)
     ReportManager:removeObserver(self)
 end

function This:refresh()
    local attackTeam = self.m_attackCount
    local defineTeam = self.m_defendCount
    local node = self.m_csNode
    node.jindu.jindu1:setPercent((attackTeam/(attackTeam+defineTeam))*100)

    node.jinggong.team:setString(string.format("%d%s",attackTeam,Serialize["RescueEmperor_ReportPanel_1"]))
    node.fangshou.team:setString(string.format("%d%s",defineTeam,Serialize["RescueEmperor_ReportPanel_1"]))
    self:refreshItemList()
end

function This:refreshItemList()
    local itemList = self:getListData()
    local scrollview = self.m_csNode.scrollview
    scrollview:removeAllItems()
    for i=1, 2 do
        local itemData = itemList[i]
        if itemData and (itemData.isOur or itemData.isOther) then
            local info = {map = self.m_map,itemData =itemData }
            local item = RescueReportItemView:create(info)
            local w,h = item.bg:getContentSize()
            scrollview:pushBackCustomItem(item, w, h ,0.5)
        end
    end
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.close:setTouchEnded(function()
		self:close()
    end)
    
    csNode.jinggong.gantanhao:setTouchEnded(function()
        info = {listData = {{groupName = "groupName2",num1=21,num2 =122},{groupName = "groupName1",num1=1,num2 =12}}
        ,title = Serialize["RescueEmperor_QueueDetails_1"] }
        g_scene:openDialog("RescueEmperor.RescueQueueDetailsPanel",info)
    end)

    csNode.fangshou.gantanhao:setTouchEnded(function()
        info = {listData = {{groupName = "groupName2",num1=21,num2 =122},{groupName = "groupName1",num1=1,num2 =12}}
        ,title = Serialize["RescueEmperor_QueueDetails_2"] }
        g_scene:openDialog("RescueEmperor.RescueQueueDetailsPanel",info)
    end)

    --刷新
    csNode.bubing:setTouchEnded(function()
        self.m_map:onGetCityBattleReportRequest(self.m_cityId)
    end)

    csNode.more:setTouchEnded(function()
        local info = {id = self.m_cityId,map = self.m_map}
        
        local panel = g_scene:openDialog("RescueEmperor.RescueLogListPanel",info)
                self.m_subView = panel
                self.m_subView:setlistData(self.m_fightList)
                self.m_subView:refresh()
                panel:addCloseCallback(function ()
                    self.m_subView = nil
                end)
        -- self:close()
    end)
end

function This:getListData()
    return self.m_fightList
    -- return {{attackid = 1020,name1= "name1",name2= "name2",group1 = "group1",group2 = "group2",isOur = true,isOther = false},
    --         {attackid = 2222,name1= "name1",name2= "name2",group1 = "group1",group2 = "group2",isOur = false,isOther = true},}
end

function This:onGetReportDetail(msg)
    msg.playerMode = PlayMode.kRescu
    cc.SceneManager:switchTo("Report.ReportLoadingScene",nil,msg)
end

function This:onGetCityBattleReport(info)
    local cityId = info.city_id or 0
    if cityId == self.m_cityId then
        local function getGroupInfo(troopInfo,normalKillNum)
            local groupName = troopInfo.group_name
            local roleName = troopInfo.role_info.name
            local isMy = troopInfo.role_info.id == GamePlayer:getPlayerID()
            local killNum = troopInfo.troop.comb_kill

            local temKillNum = killNum
            if normalKillNum>killNum then
                temKillNum = normalKillNum
            end

            return groupName,roleName,isMy,temKillNum
        end
        self.m_attackCount = info.attack_count--进攻
        self.m_defendCount = info.defend_count
        local temFightList = {}
        local fightList = info.fight_list
        
        local killNum = 0
        for _,fightInfo in ipairs(fightList) do
            local reportId = fightInfo.report_id
            local name1,group1,temIsOur1,temKillNum1 = getGroupInfo(fightInfo.attack_troop,0)
            local name2,group2 ,temIsOur2,temKillNum2 = getGroupInfo(fightInfo.defend_troop,temKillNum1)
            if temKillNum2>killNum then
                killNum = temKillNum2
            end
            local temIsOur = temIsOur1 or temIsOur2
            local item1 = {attackid = reportId,name1 = name1,name2 = name2,group1 = group1,group2 = group2,isOur = temIsOur ,killNum =temKillNum2 }
            table.insert(temFightList,item1)
        end

        local isOther = false
        local isOur = false
        local temFightList2 ={}
        for _,fightItem in ipairs(temFightList) do
            local temKillNum = fightItem.killNum
            local temIsOther = killNum ==temKillNum
            if fightItem.isOur and not isOur then
                isOur = true
                fightItem.isOther = false
                table.insert(temFightList2,1,fightItem)
            elseif temIsOther and not isOther then
                fightItem.isOur = false
                fightItem.isOther = true
                table.insert(temFightList2,1,fightItem)
            else
                fightItem.isOur = false
                fightItem.isOther = false
                table.insert(temFightList2,fightItem)
            end
            isOther = isOther or temIsOther
        end
        self.m_fightList = temFightList2
        if  self.m_subView then
            self.m_subView:setlistData(self.m_fightList)
            self.m_subView:refresh()
        end
        self:refresh()

    end
end
