

-- 勤王战备战界面

local This, super = class(...,"NodeView")
local RankPageListLayer = require("Rank.RankPageListLayer")
local RescueWarPrepareItemView = require("RescueEmperor.RescueWarPrepareItemView")
local GameMapManager = require("Map.GameMapManager")
local EntrantManager = require("LegionEntrant.EntrantManager")

function This:initData(info)
	self.m_mainUI = info.mainUI
	self.m_rescueWarData = info.data
	self.m_isInitListLayer = false
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorQualification.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    csNode.bg1:adapterFullScreen()
	csNode.close:adapterBorder("right", "top", -cc.visible.designWidth/2, -cc.visible.designHeight/2)
	csNode.guize:adapterBorder("right", "bottom", -cc.visible.designWidth/2, -cc.visible.designHeight/2)
	csNode.qinwangdian:adapterBorder("right", "bottom", -cc.visible.designWidth/2, -cc.visible.designHeight/2 )
	csNode.time:setVisible(false)

	-- 关闭
	csNode.close:setTouchEnded(function()
	    self.m_mainUI:close()
	end) 

	-- 规则
	csNode.guize:setTouchEnded(function()
        g_scene:openDialog("RescueEmperor.RescueWarRulePanel")
	end)

	-- 勤王殿
	csNode.qinwangdian:setTouchEnded(function()
        g_scene:openDialog("Palace.PalacePanel", {tab = "tep2"})
	end)

	-- 进入地图
	csNode.buttonjinru:setTouchEnded(function()
        GameMapManager:enterRescueMap()
	end)	

   	local scrollView = self.m_csNode.ScrollView
	scrollView:setDirection("vertical")

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 440,
        itemHeight = 39,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
		dataCounter = function ()
	        local rank = self:getRankData()	 
	        return #rank
        end,
        viewCreator =  function(index, node)
        	local rank = self:getRankData()	   
        	local data = rank[index]
        	if data then     
			    if not node then
				    return RescueWarPrepareItemView:create(data, index)
				else
			        node:setModel(data, index)
			    end
			end
	    end
    })

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
    GameMapManager:getRescueMap():addObserver(self)

    if not self.m_isInitListLayer then
    	self.m_isInitListLayer = true
    	self.m_listLayer:setDataList()
    end
end

function This:onExit()
    EntrantManager:removeObserver(self)
    GameMapManager:getRescueMap():removeObserver(self)
    super.onExit(self)
end

function This:getRankData()
	return self.m_rescueWarData:getGroupList()
end

function This:onRescueResultReply(data)
	self.m_rescueWarData = data

	local resultFlag = data:getResultFlag()
	if resultFlag == 4 then
		return
	end

	self.m_isInitListLayer = true
	self.m_listLayer:setDataList()
end

function This:onGetMapInfoFail(msg)
	EntrantManager:onRescueInfoRequest()
	
	if self.m_rescueWarData:isOut() then
		g_scene:showTips(Serialize[61593])
		return
	end
	
	g_scene:handleErrorCode(msg.err_code)
end