
-- 勤王战主界面

local This, super = class(..., "Dialog")
local EntrantManager = require("LegionEntrant.EntrantManager")

-- 未开始 RescueWarBeforeNode
-- 预告 RescueWarPreviewNode
-- 报名 RescueWarResponseNode
-- 开战倒计时 RescueWarPrepareNode
-- 战斗中 RescueWarFightNode
-- 结果排名 RescueWarResultNode


function This:initData(info)	
	self.m_playType = info.playType
	self.m_mapType = info.mapType
	self.m_nodeView = nil
	self.m_nodeName = ""
	
	self.m_rescueWarData = EntrantManager:getRescueWarData()
	self.m_state = 0
	self.m_endTime = 0
	self.m_hasTimeUpdate = false
	self.m_escape = 0
	self:updateState()
	
	EntrantManager:onRescueInfoRequest()
end

function This:initNode()
	local data = self.m_rescueWarData
	local arg = {mainUI = self, data = data}

	if self.m_state == KingWarBtnState.kNotStart then
		-- 未开始
		self:openNodeView("RescueEmperor.RescueWarBeforeNode", arg)

	elseif self.m_state == KingWarBtnState.kPreView then
		-- 预告
		self:openNodeView("RescueEmperor.RescueWarPreviewNode", arg)

	elseif self.m_state == KingWarBtnState.kSignUp then
		-- 报名
		self:openNodeView("RescueEmperor.RescueWarResponseNode", arg)

	elseif self.m_state == KingWarBtnState.kPrepare then
		-- 备战，可能出现：未报名 / 分组中 / 开战倒计时
		local fightFlag = data:getFightFlag()
		local groupList = data:getGroupList()
		if fightFlag == 0 then
			self:openNodeView("RescueEmperor.RescueWarNotSignUpNode", arg)
		elseif #groupList == 0 then
			self:openNodeView("RescueEmperor.RescueWarGroupNode", arg)
		else
			self:openNodeView("RescueEmperor.RescueWarPrepareNode", arg)
		end

	elseif self.m_state == KingWarBtnState.kFight then
		-- 战斗，可能出现：未报名 / 进入地图 / 提前结束，出战斗结果
		local fightFlag = data:getFightFlag()
		local resultFlag = data:getResultFlag()
		if fightFlag == 0 then
			self:openNodeView("RescueEmperor.RescueWarNotSignUpNode", arg)
		elseif resultFlag == 4 then
			self:openNodeView("RescueEmperor.RescueWarResultNode", arg)
		else
			self:openNodeView("RescueEmperor.RescueWarFightNode", arg)
		end

	elseif self.m_state == KingWarBtnState.kEnd then
		-- 结束，可能出现：未报名 / 战斗结果
		local fightFlag = data:getFightFlag()
		if fightFlag == 0 then
			self:openNodeView("RescueEmperor.RescueWarNotSignUpNode", arg)
		else
			self:openNodeView("RescueEmperor.RescueWarResultNode", arg)
		end

	else
		self:openNodeView("RescueEmperor.RescueWarEmptyNode", arg)

	end
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    EntrantManager:removeObserver(self)
end

function This:openNodeView(name, info)
	if self.m_nodeName == name then
		return
	end
	
	print("openNodeView", name)
	self:closeNodeView()

	local view = require(name):create(info)
	self:addChild(view)
	self.m_nodeView = view
	self.m_nodeName = name
	
	return view
end

function This:closeNodeView()
	if self.m_nodeView then
		self.m_nodeView:removeFromParent(true)
		self.m_nodeView = nil
	end
	self.m_nodeName = ""
end

function This:callNodeViewFunc(funcName, ...)
	if not self.m_nodeView then
		return
	end

	local func = self.m_nodeView[funcName]
    if type(func) == "function" then
		return func(self.m_nodeView, ...)
	end
end

function This:getLeftTime()
	return self.m_rescueWarData:getStateLeftTime()
end

function This:update(dt)
    super.update(self, dt)
    if self.m_hasTimeUpdate then
		self.m_escape = self.m_escape + dt
		if self.m_escape >= 1.0 then
			self.m_escape = self.m_escape - 1
			local leftTime = self:getLeftTime()
			if leftTime > 0 then
				self:callNodeViewFunc("updateLeftTime", leftTime)
			else
				self.m_hasTimeUpdate = false
				self.m_escape = 0
				self:callNodeViewFunc("updateLeftTime", 0)
				EntrantManager:onRescueInfoRequest()
			end
		end
	end
end

-- 勤王战信息
function This:onRescueInfoReply(data)
	self:updateState()
	self:initNode()
end

-- 勤王战结果
function This:onRescueResultReply(data)
	self:initNode()
end

function This:updateState()
	local data = self.m_rescueWarData
	self.m_state = data:getState()
	self.m_endTime = data:getStateEndTime()
	self.m_hasTimeUpdate = (self.m_state < KingWarBtnState.kEnd)
end
