

-- 勤王战排行榜界面

local This , super = class(...,"Dialog")
local EntrantManager = require("LegionEntrant.EntrantManager")
local RescueWarRankItemView = require("RescueEmperor.RescueWarRankItemView")
local UpdateLayer = require("UI.UpdateLayer")
local RankPageListLayer = require("Rank.RankPageListLayer")


local RescueRankTab = {
    kShaDi = RescueRankType.kShaDi, -- 杀敌排行
    kTuLu = RescueRankType.kTuLu, -- 屠戮排行
    kLianZhan = RescueRankType.kLianZhan, -- 连斩排行
}

local ListTitle = {
    [RescueRankTab.kShaDi] = Serialize[64135],
    [RescueRankTab.kTuLu] = Serialize[64136],
    [RescueRankTab.kLianZhan] = Serialize[64137],
}


function This:initData(info)
    self.m_dataType = info.dataType
    self.m_rescueWarData = info.data

    if self.m_dataType == 2 then
        EntrantManager:onRescueAreaRankRequest()
    end
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("RescueEmperor/RescueEmperorRewardRank.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode

    if self.m_dataType == 1 then
        csNode.beijing.title:setString(Serialize[64138])
    else
        csNode.beijing.title:setString(Serialize[64139])
    end
    
    -- 杀敌排行
    self:addTabButton(RescueRankTab.kShaDi, csNode.tabs.tab1, function()
        self:selectTab(RescueRankTab.kShaDi)
    end)
    -- 屠戮排行
    self:addTabButton(RescueRankTab.kTuLu, csNode.tabs.tab2, function()
        self:selectTab(RescueRankTab.kTuLu)
    end)
    -- 连斩排行
    self:addTabButton(RescueRankTab.kLianZhan, csNode.tabs.tab3, function()
        self:selectTab(RescueRankTab.kLianZhan)
    end)

    local scrollView = csNode.scrollView
    -- self.m_listLayer = UpdateLayer:create({
    --     itemWidth = 730,
    --     itemHeight = 55,
    --     spaceX =0,
    --     spaceY =0,
    --     scrollView = scrollView
    -- })

    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 730,
        itemHeight = 55,
        spaceX = 0,
        spaceY = 0,
        scrollView = scrollView,
        dataCounter = function ()
            local rank = self:getRoleRank()
            return #rank
        end,
        viewCreator =  function(index, node)
            local rank = self:getRoleRank()    
            local data = rank[index]
            if data then     
                if not node then
                    return RescueWarRankItemView:create(data, index)
                else
                    node:setModel(data, index)
                end
            end
        end
    })    

    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setAnchorPoint(0.5, 0)
    local w, h = self.m_listLayer:getContentSize()
    scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    

    self:selectTabButton(RescueRankTab.kShaDi)
end

function This:initTouchEvent()
    self.m_csNode.beijing.closebutton:setTouchEnded(function()
        self:close()
    end)
end

function This:onEnter()
    super.onEnter(self)
    EntrantManager:addObserver(self)
end

function This:onExit()
    EntrantManager:removeObserver(self)
    super.onExit(self)
end

function This:selectTab(tab)
    if self.m_curTab == tab then
        return
    end
    self.m_curTab = tab
    self:refresh()
end

function This:refresh()
    local csNode = self.m_csNode
    
    local rank = self:getRoleRank()
    self:refreshList(rank)

    csNode.biaoqian.biaoqian1.bingli:setString( ListTitle[self.m_curTab] or "" )
end

function This:getRoleRank()
    if self.m_dataType == 1 then
        return self.m_rescueWarData:getRoleRankByType(self.m_curTab)
    end

    if self.m_dataType == 2 then
        return self.m_rescueWarData:getAreaRoleRankByType(self.m_curTab)
    end

    return {}
end

function This:refreshList(list)
    -- local createFunc = function (data, i)
    --     return RescueWarRankItemView:create(data)
    -- end
    -- self.m_listLayer:setDataList(list, createFunc)
    self.m_listLayer:setDataList()    
end

function This:onRescueAreaRankReply(data)
    self.m_rescueWarData = data
    self:refresh()
end


