local This = class(...)
g_setting.game = require("GameSetting")

local ResourceInfos = {}
local lastResource = nil
function This:create(sceneType, listener)
    local obj = {}
    local amount = 0
    local resource = self:getResourceInfo(sceneType, listener and listener.sceneArg)
    for name,part in pairs(resource) do
        if type(part) == "table" then
            obj[name] = {}
            for k,v in pairs(part) do
                obj[name][k] = v
                amount = amount + 1
            end
        end
    end
    resource.m_type = sceneType
    obj[0] = obj
    obj.m_resouce = resource
    obj.m_amount = amount
    obj.m_listener = listener
    obj.m_loadedCount = 0
    obj.m_type = sceneType
    setmetatable(obj, self)
    obj:init()
    return obj
end

function This:forceLoad(type)
    local obj = self:create(type)
    obj.m_maxTime = -1
    obj.m_startTime = os.clock() * 1000
    obj:stepLoad()
    obj.m_isFinish = true
    obj.m_costTime = os.clock() * 1000 - obj.m_startTime
    print("forceLoad cost time:", obj.m_costTime)
    return obj
end

function This:scheduleUpdate(owner)
    if owner then
        self.m_refUpdateList = owner.m_updateList
    else
        self.m_refUpdateList = g_scene.m_updateList
    end
    self.m_refUpdateList[self[0]] = self
end

function This:unscheduleUpdate()
    if self.m_refUpdateList then
        self.m_refUpdateList[self[0]] = nil
    end
end

function This:getResourceInfo(type, sceneArg)
    if not ResourceInfos[type] then
        logError("not found resource:", type)
        return {}
    end
    local resource = ResourceInfos[type](sceneArg)
    if resource.m_characters == nil then
        resource.m_characters = {}
    end
    if resource.m_images == nil then
        resource.m_images = {}
    end
    if resource.m_plists == nil then
        resource.m_plists = {}
    end
    for partName,partResource in pairs(ResourceInfos["Common"]) do
        resource[partName] = resource[partName] or {}
        for k,v in pairs(partResource) do
            resource[partName][k] = v
        end
    end

    if resource.m_csFile then
        local getCSNodeFileResource = lib2d.getCSNodeFileResource
        for name,v in pairs(resource.m_csFile) do
            local csResource = getCSNodeFileResource(name)
            for k,v in pairs(csResource.images) do
                -- local _, _, mapKey = string.find(k, "MapProject/(%w*)/")
                -- if mapKey and string.find(k, ".png") then
                --     resource.m_plists[mapKey] = 1
                -- else
                    resource.m_images[k] = resource.m_images[k] or 1
                -- end
            end

            for k,v in pairs(csResource.plists) do
                local _, _, plistKey = string.find(k, "Plist/(%w*)%.plist")
                if plistKey then
                    resource.m_plists[plistKey] = resource.m_plists[plistKey] or 1
                else
                    logError("plist file must in [Plist] folder", k)
                end
            end
        end
        resource.m_csFile = nil
    end

    for id,v in pairs(resource.m_plists) do
        resource.m_images["Plist/" .. id .. ".png"] = 1
    end
    return resource
end

function This:init()
    self.m_stepTime = require("RunTime"):create()
    self.m_maxTime = 30
    self.m_isFinish = false
    self.m_isFinishClear = false
end

local TextureCache = cc.TextureCache
local SpriteFrameCache = cc.SpriteFrameCache
local Audio = cc.Audio
local FontCacheAddFNT = lib2d.ccFontCache.addFNT
local FontCacheAddTTF = lib2d.ccFontCache.addTTF
local function addImage(k, v)
    local image = TextureCache.m_images[k]
end
local function addCharacter(k, v)
    cc.SkeletonNode.loadCharacterData(k)
end
local function addEffect(k, v)
    cc.SkeletonNode.loadEffectData(k)
end
local function addBackgroundSound(k, v)
    Audio:preloadBackground(k)
end
local function addSound(k, v)
    Audio:preloadEffect(k)
end
local function addPlist(k, v)
    SpriteFrameCache:addPlist(k)
end
local function addTTF(k, v)
    fileName = "Fonts/" .. v[1]
    FontCacheAddTTF(fileName, v[2])
end
local function addFNT(k, v)
    fileName = "Fonts/" .. k
    FontCacheAddFNT(fileName)
end

function This:foreach(t, handler)
    if not t then
        return false
    end
    local stepTime = self.m_stepTime
    local maxTime = self.m_maxTime
    for k,v in pairs(t) do
        t[k] = nil
        self.m_loadedCount = self.m_loadedCount + 1
        handler(k, v)
        if stepTime:isTimeOut(maxTime) then
            return true
        end
    end
    return false
end

function This:getLastSceneType()
    if lastResource then
        return lastResource.m_type
    end
end

function This:updateClear()
    if not lastResource then
        self.m_isFinishClear = true
        return
    end
    if lastResource.m_type == self.m_type then
        self.m_isFinishClear = true
        return
    end

    local unremoveEffects = {}
    local unremoveChracters = {}
    for i = 1, #unremoveEffects do
        unremoveEffects[unremoveEffects[i]] = 1
    end
    for i = 1, #unremoveChracters do
        unremoveChracters[unremoveChracters[i]] = 1
    end
    cc.SkeletonNode.clearAllEffectData(unremoveEffects)
    cc.SkeletonNode.clearAllCharacterData(unremoveChracters)
    if lastResource.clear then
        lastResource:clear()
    end
    TextureCache:removeUnusedTextures({"Images/BulletStreak.png", "Images/BulletStreak2.png", "Images/BulletStreak3.png"})
    self.m_isFinishClear = true
end

function This:stepLoad()
    if self:foreach(self.m_images, addImage) then
        return
    end
    if self:foreach(self.m_characters, addCharacter) then
        return
    end
    if self:foreach(self.m_effects, addEffect) then
        return
    end
    if self:foreach(self.m_plists, addPlist) then
        return
    end
    if self:foreach(self.m_backgroundSounds, addBackgroundSound) then
        return
    end
    if self:foreach(self.m_sounds, addSound) then
        return
    end
    if self:foreach(self.m_fonts, addTTF) then
        return
    end
    if self:foreach(self.m_fnts, addFNT) then
        return
    end
end

function This:callListener(eventID, ...)
    if self.m_listener and self.m_listener[eventID] then
        self.m_listener[eventID](...)
    end
end

function This:update()
    if self.m_isFinish then
        return
    end
    self.m_stepTime:reset()
    if not self.m_isFinishClear then
        self:updateClear()
    end

    self:stepLoad()
    self:callListener("onLoading", self:getPercent())
    if self.m_loadedCount >= self.m_amount then
        self.m_isFinish = true
        lastResource = self.m_resouce
        self:unscheduleUpdate()
        self:callListener("onFinish")
    end
end

function This:getPercent()
    if self.m_amount == 0 then
        return 100
    else
        return (self.m_loadedCount / self.m_amount) * 100
    end
end

ResourceInfos["Common"] = {
    m_plists = {
        
    },
    m_fonts = {

    },
    m_fnts = {
    },
    m_characters = {
        
    },
}

ResourceInfos["GameScene"] = function (sceneArg)
    local resource = {
        m_sounds = sounds,
        m_backgroundSounds = {
            
        },
        m_effects = {
            
        },
        m_plists = {
           
        },
        m_characters = {
        },
        m_images = {
            
        },
        m_csFile = {
            
        },
    }
    return resource
end
ResourceInfos["MenuScene"] = function ()
    local resource = {
        m_sounds = {
        },
        m_effects = {
        },
        m_characters = {
        },
        m_plists = {
        },
        m_images = {
        },
        m_csFile = {
        },
    }
    return resource
end