﻿local This = class(...)
local Json = require("Json")
local ServerConfig = require("Net.ServerConfig")
local GameSetting = require("GameSetting")
local GameChannel = require("GameChannel")

kUpdateServerSite = "http://update.51gamer.cn"
UpdateServerSiteConfig = {
	[ChannelFormatId.kMuZhiIos] = "http://getdata.51gamer.cn", --拇指热更地址
}

function This:create(listener)
    local obj = {m_listener = listener}
    setmetatable(obj, self)
    obj:init()
    return obj
end

function This:init()
	local updateUrl = GameSetting:get("UpdateServerSite")
	if updateUrl then
		kUpdateServerSite = updateUrl
	end
    self.m_state = nil
	print("-----------------ResourceUpdate:init------------------------------")
	local cacheMainVer , cacheSubVer = cc.Application:getVersion()
	cacheMainVer , cacheSubVer = cacheMainVer or "1", cacheSubVer or "0"

	local mainVer , subVer , pfId = cacheMainVer , cacheSubVer, 0
	local channelId , gameId = ServerConfig.channelId, ServerConfig.gameId
	if not os.isWin32 and not os.isIosSimulator then
		local version = os.native_call("getVersion") 
		mainVer , subVer = string.match(version,"(%d+).(%d+)")
		channelId = os.native_call("getChannelId")
		gameId = os.native_call("getGameId")
		pfId = os.native_call("getPfId")
	end

	-- 苹果审核包修改热更地址
	local formatId = GameChannel:getFormatId()
	local newUrl = UpdateServerSiteConfig[formatId]
	if newUrl then
		kUpdateServerSite = newUrl
	end
	

	print(string.format("mainVer = %s, subVer = %s, cacheMainVer = %s, cacheSubVer = %s",mainVer,subVer,cacheMainVer,cacheSubVer))
	if cacheMainVer < mainVer then
		local updateDir = cc.FileUtils:getWritablePath().."/Update/"
		if cc.FileUtils:isDirectoryExist(updateDir) then
			print("删除更新包...")
			cc.FileUtils:removeDirectory(updateDir)
			cc.Application:reload("Full")
		end
	end

	self.m_mainVersion = mainVer
	if self.m_mainVersion == cacheMainVer then
		self.m_subVersion = cacheSubVer
	else
		self.m_subVersion = subVer
	end
	self.m_channelId = channelId
	self.m_gameId = gameId
	self.m_pfId = pfId
    self:stepState("CheckEnvironment")
end


function This:callListener(eventName, ...)
    local listener = self.m_listener
    if listener then
        local func = listener[eventName]
        if func then
            func(listener.owner or listener, self, ...)
        end
    end
end

function This:getDownloadFileInfo(config)
	local fileInfo = {
		files = {},
		totalSize = 0,
		maxMainVer = self.m_mainVersion, 
		maxSubVer = self.m_subVersion,
		url = config.downloadUrl
	}

	for version , info in pairs(config.assets) do
		local mainVer , subVer = string.match(version,"(%d+)-(%d+)")
		if self.m_mainVersion == mainVer and self.m_subVersion < subVer then
			fileInfo.totalSize = fileInfo.totalSize + info.size
			table.insert(fileInfo.files,{
				mainVer = mainVer,
				subVer = subVer,
				name = info.name, 
				size = info.size,
			})
			if fileInfo.maxMainVer < mainVer or fileInfo.maxMainVer == mainVer and fileInfo.maxSubVer < subVer then
				fileInfo.maxMainVer = mainVer
				fileInfo.maxSubVer = subVer
			end
		end
	end
	table.sort(fileInfo.files,function(a,b)
		if a.mainVer < b.mainVer then
			return true
		elseif a.mainVer == b.mainVer and a.subVer < b.subVer then
			return true
		end
		return false
	end)
	return fileInfo
end

function This:getConfigDownloadUrl(config)
	local localMac = g_setting.macAddress
	if os.isWin32 or os.isIosSimulator then
		localMac = lib2d.getComputerMacAddress()
	end

	if os.isIos and config.audit then
		local maxAuditVer = config.audit.maxMain
		local bundleVersion = os.native_call("getBundleVersion")
		if maxAuditVer == bundleVersion then
			ServerConfig.isAudit = true
			if config.audit.ip and config.audit.port then
				ServerConfig.loginServerHost = config.audit.ip
				ServerConfig.loginServerPort = tonumber(config.audit.port)
			end
			if config.audit.pathCfgUrl then
				return "normal", kUpdateServerSite .. "/" .. config.audit.pathCfgUrl
			end
			return "normal", config.audit.cfgUrl
		else
			ServerConfig.isAudit = false
		end
	end

	if table.contain(config.testMac,localMac) then
		if config.debug.ip and config.debug.port then
			ServerConfig.loginServerHost = config.debug.ip
			ServerConfig.loginServerPort = tonumber(config.debug.port)
		end
		if config.debug.csinfo and ServerConfig.censoring then
			local csinfo = config.debug.csinfo
			local csver
			if csinfo then
				csver = csinfo.version
				if csver and ServerConfig.censoring then
					if ServerConfig.censoring == csver or csinfo.csforceup then
						self:initCensoringInfo(csinfo, csver)
					end
				end
			end
		end
		if self.m_mainVersion < config.debug.maxMain then
			return "reload", config.debug.pkgUrl
		else
			if config.debug.pathCfgUrl then
				return "normal", kUpdateServerSite .. "/" .. config.debug.pathCfgUrl
			end
			return "normal", config.debug.cfgUrl
		end
	else
		if config.release.ip and config.release.port then
			ServerConfig.loginServerHost = config.release.ip
			ServerConfig.loginServerPort = tonumber(config.release.port)
		end
		if config.release.csinfo and ServerConfig.censoring then
			local csinfo = config.release.csinfo
			local csver
			if csinfo then
				csver = csinfo.version
				if csver and ServerConfig.censoring then
					if ServerConfig.censoring == csver or csinfo.csforceup then
						self:initCensoringInfo(csinfo, csver)
					end
				end
			end
		end
		if self.m_mainVersion < config.release.maxMain then
			return "reload", config.release.pkgUrl
		else
			if config.release.pathCfgUrl then
				return "normal", kUpdateServerSite .. "/" .. config.release.pathCfgUrl
			end
			return "normal", config.release.cfgUrl
		end
	end
end

function This:initCensoringInfo( csinfo, csver )
	print(csinfo)
	local strList = {"loginServerHost"}
	local valueList = {"loginServerPort", "groupDefault", "groupIndex", "groupSep", "groupCount"}
	for _, key in ipairs(strList) do
		local value = csinfo[key .. csver]
		if value then
			ServerConfig[key] = value
		end
	end
	for _, key in ipairs(valueList) do
		local value = csinfo[key .. csver]
		if value then
			ServerConfig[key] = tonumber(value)
		end
	end
end

function This:stepState(state,arg)
	if self.m_state == state then
		return
	end
	lastState = self.m_state

	if state == "CheckEnvironment" then
		local url = ""
		if os.isAndroid or tonumber(self.m_pfId) <= 0 then
			url = string.format(kUpdateServerSite .. "/king/channel-%s-%s/envcfg.txt",self.m_channelId,self.m_gameId)
		else
			url = string.format(kUpdateServerSite .. "/king/channel-%s-%s-%s/envcfg.txt",self.m_channelId,self.m_gameId,self.m_pfId)
		end
		
		cc.Http:sendRequest(url, function(response)
			local config = response:getJsonData()
			if not config then
				self:callListener("onConfigLost")
				return
			end

			self.m_loadType , self.m_downloadUrl = self:getConfigDownloadUrl(config)
			if self.m_loadType == "normal" then
				self:stepState("CheckVersion")
			else
				self:callListener("onInvalidVersion",self.m_downloadUrl)
			end
		end,"get")
	elseif state == "CheckVersion" then
		self:callListener("onCheckVersion")
		cc.Http:sendRequest(self.m_downloadUrl, function(response)
			 local config = response:getJsonData()
			 print(config)
			 if not config then
                self:callListener("onConfigLost")
                return
			 end

			 self.m_downloadInfo = self:getDownloadFileInfo(config)
			 if self.m_downloadInfo.totalSize <= 0 then
				self:stepState("UpdateFinish", "onBestVersion")
				return
			 end

			 local downloadSize = self:getFinalDownloadSize(self.m_downloadInfo.files,self.m_downloadInfo.totalSize)
			 self:callListener("onNewVersion", {
                mainVersion = self.m_mainVersion,
                subVersion = self.m_subVersion,
                maxMainVersion =  self.m_downloadInfo.maxMainVer,
                maxSubVersion =  self.m_downloadInfo.maxSubVer,
                state =  1,
                totalSize = downloadSize,
             })
		end,"get")
	elseif state == "DowloadStart" then
		if not self.m_downloadInfo or #self.m_downloadInfo.files <= 0 then
			self:stepState("UpdateFinish", "onBestVersion")
			return
		end

		local files = {}
		local url = self.m_downloadInfo.url
		for i , info in ipairs(self.m_downloadInfo.files) do
			table.insert(files,info.name)
		end

		local lastState = 0
		lib2d.createPatchUpdate(url, files, function (_, state, index, count, size, totalSize)
			print(string.format("state = %s, index = %s, count = %s, size = %s, totalSize = %s",tostring(state),tostring(index),tostring(count),tostring(size),tostring(totalSize)))
			if lastState == 1 and state == 3 then
				self:callListener("onUnzipStart", files[index])
			end
			if state == 101 then
				self:callListener("onFileLost", files[index], index, count, size, totalSize)
			elseif state == 1 then -- Downloading
				self:callListener("onDownloading", files[index], index, count, size, totalSize)
			elseif state == 3 then -- Unziping
				self:callListener("onUnziping", files[index], index, count, size, totalSize)
			elseif state == 4 then -- Finish
				self:stepState("UpdateFinish", "UnzipAll")
			end
			lastState = state
		end)
        self:callListener("onDowloadStart")
	elseif state == "UpdateFinish" then
		self:callListener("onUpdateFinish", arg)
	end
end

function This:getFinalDownloadSize(downloadList, totalSize)
    totalSize = totalSize
    local dir = cc.FileUtils:getWritablePath() .. "Patch/"
    for i=#downloadList, 1, -1 do
        local fileName = dir .. downloadList[i].name
        totalSize = totalSize - cc.FileUtils:getFileSize(fileName)
    end
    return totalSize
end

function This:downloadFile()
    local lastState = 0
    self:callListener("onDownloadStart", self.m_downloadList[index])
    lib2d.createPatchUpdate(self.m_downloadUrl, self.m_downloadList, function (state, index, count, size, totalSize)
        if lastState == 1 and state == 3 then
            self:callListener("onUnzipStart", self.m_downloadList[index])
        end
        if state == 1 then -- Downloading
            self:callListener("onDownloading", self.m_downloadList[index], index, count, size, totalSize)
        elseif state == 3 then -- Unziping
            self:callListener("onUnziping", self.m_downloadList[index], index, count, size, totalSize)
        elseif state == 4 then -- Finish
            self:stepState("UpdateFinish", "UnzipAll")
        end
        lastState = state
    end)
end
