local This, super = class(..., "cc.Scene")
local Dialog = require("Dialog")
local EventDispatcher = require("EventDispatcher")
local AppConfig = require("AppCofing")
This.delayDialogList = {}

SceneType = 
{
	kHomeScene = 1,--主场景
	kWorldScene = 2, --世界场景
	kLoadingScene = 3, --加载场景
	kLegionScene = 4, --军团战场景
	kResueScene = 5, --军团战场景
}

function This:init(command)
    super.init(self)
	self:setName(self._NAME)
    self.m_dialogList = {}
    self.m_command = command
	self.m_type = 0
	self.m_isEnableShowTips = true
	self.m_isShowing = false
end

function This:onEnter()
	super.onEnter(self)
	EventDispatcher:addListener("connect_lost",self.onConnectLost)
	EventDispatcher:addListener("account_conflict",self.onAccountConflict)
end

function This:onExit()
	EventDispatcher:removeListener(self,"connect_lost")
	EventDispatcher:removeListener(self,"account_conflict")
	if self.m_getPathListener then 
		self:removeEventListener(self.m_getPathListener)
	end
	super.onExit(self)
end

function This:getType()
	return self.m_type
end

function This:setType(type)
	self.m_type = type
end

function This:setIsEnableShowTips(enable)
	self.m_isEnableShowTips = enable
end

function This:getDialog(name)
    return Dialog.getDialog(self, name)
end

function This:getAllDialogs()
    return self.m_dialogList
end

function This:openDialog(name, info)
    local dialog = Dialog.openDialog(self, name, info)
    if dialog then
        self:pauseGame()
    end
    return dialog
end

function This:getTopDialog()
    return Dialog.getTopDialog(self)
end

function This:closeAllDialog()
	for i = #self.m_dialogList , 1 , -1 do
		local dialog = self.m_dialogList[i]
		dialog:close()
	end
	self.m_dialogList = {}
end

function This:removerDialog(dialog)
    if Dialog.removerDialog(self, dialog) then
        self:resumeGame()
    end
end

function This:getMaxLocalZOrder()
	local children = self:getChildren()
	local zorder = 0
	for i , child in ipairs(children) do
		if zorder < child:getLocalZOrder() then
			zorder = child:getLocalZOrder()
		end
	end
	return zorder
end

function This:fixBackgroundScreen(bg)
	local b = bg 
	if not b and self.m_csNode and self.m_csNode.bg then
		b = self.m_csNode.bg
	end
	if b then
		b:setScale(1.0 / AppConfig.heightScale)
	end
end

function This:isShowHeroCardFinish()
	return not self.m_heroList or #self.m_heroList <= 0
end

function This:showCard(heroList)
	if not self.m_isEnableShowTips then
		return
	end
	local function show(index)
		if index <= #self.m_heroList then
			local info = self.m_heroList[index]
			local view = require("Recruit.RecruitView"):create()
			view:showObtainEffect({heroId = info.heroId, isGot = info.isGot})
			self:addChild(view)		
			view:setFinishHandler(function()
				show(index + 1)
			end)
		else
			self.m_heroList = {}
		end
	end
	if self.m_heroList and #self.m_heroList > 0 then
		for i , info in ipairs(heroList) do
			table.insert(self.m_heroList,info)
		end
	else
		self.m_heroList = heroList
		show(1)
	end
end

function This:showReward(itemList)
    if not itemList or #itemList < 1 then
        return
    end
	if #itemList <= 2 then
		local ItemNode = require("UI.ItemNode")
		for i, item in ipairs(itemList) do
			local node = cc.Node:createFromCSFile("MenuScene/NodeUI/RewardTips.csb")
			local itemNode = ItemNode:create(item.id or item[1])
			itemNode:setTouchEnable(false)
			itemNode:setScale(0.8)
			itemNode:setCascadeOpacityEnabled(true)
			itemNode:setAnchorPoint(0.5,0.5)
			node:addChild(itemNode)
			itemNode:setPosition(-83, -45)
			node.name:setString(itemNode:getItemName())
			node.name:setColorByID("Q".. itemNode:getQuality())
			node.count:setString("X " .. (item.count or item[2] or item.num))
			node:setOpacity(0)
			node:setScale(4)
			node:setPosition(cc.visible.center.x, cc.visible.center.y)
			self:addChild(node, 999)
			local action = cc.Sequence(cc.DelayTime(0.5 * (i-1)), cc.CallFunc(function()cc.Audio:playEffect("GetItem.mp3")end),cc.Spawn(cc.FadeIn(0.07), cc.ScaleTo(0.07, 1)), cc.DelayTime(0.45), cc.Spawn(cc.FadeOut(2), cc.MoveBy(1.2, 0, 200, "BackOut")), cc.RemoveAction())
			node:runAction(action)
		end	
	else
		local panel = g_scene:openDialog("UI.RewardPanel",itemList)
		-- panel:setLocalZOrder(999)
	end
end

function This:showHeroInfoInExp(heroId, deltaHp)
	if not self.m_isEnableShowTips then
		return
	end
	self.m_isEnableShowTips = false
	local node = cc.Node:createFromCSFile("MenuScene/NodeUI/RewardTips.csb")
	local ItemNode = require("UI.ItemNode")
	local itemNode = ItemNode:create(heroId)
	itemNode:setTouchEnable(false)
	itemNode:setScale(0.8)
	itemNode:setCascadeOpacityEnabled(true)
	itemNode:setAnchorPoint(0.5,0.5)
	node:addChild(itemNode)
	itemNode:setPosition(-83, -45)
	node.name:setString((deltaHp > 0 and Serialize[60071] or Serialize[60072]) .. deltaHp)
	node.name:setColorByID(deltaHp < 0 and "R0" or "A7")
	node.count:setVisible(false)
	node:setOpacity(0)
	node:setPosition(cc.visible.center.x, cc.visible.center.y -90)
	self:addChild(node, 999)
	local action = cc.Sequence( cc.CallFunc(function() 
		cc.Audio:playEffect("GetItem.mp3") 
		self.m_isEnableShowTips = true 
	end), cc.Spawn( cc.FadeIn(0.14) ), cc.DelayTime(1.0), cc.Spawn(cc.FadeOut(2), cc.MoveBy(1.0, 0, 90, "BackOut")), cc.RemoveAction())
	node:runAction(action)
end

function This:createItemUI(item)
	local node = cc.Node:createFromCSFile("MenuScene/NodeUI/RewardTips.csb")
	local ItemNode = require("UI.ItemNode")
	local itemNode = ItemNode:create(item.id or item[1])
	itemNode:setTouchEnable(false)
	itemNode:setScale(0.8)
	itemNode:setCascadeOpacityEnabled(true)
	itemNode:setAnchorPoint(0.5,0.5)
	node:addChild(itemNode)
	itemNode:setPosition(-83, -45)
	node.name:setString(itemNode:getItemName())
	node.name:setColorByID("Q".. itemNode:getQuality())
	node.count:setString("X " .. (item.count or item[2] or item.num))
	node:setOpacity(0)
	node:setPosition(cc.visible.center.x, cc.visible.center.y -90)
	return node
end

function This:showRewardInExp(itemList)
	if not self.m_isEnableShowTips then
		return
	end
	
	local count = #itemList
    if not itemList or count < 1 then
        return
    end
    self.m_isEnableShowTips = false
	for i, item in ipairs(itemList) do
		local node = self:createItemUI(item)
		self:addChild(node, 999)
		local action = cc.Sequence(
			cc.DelayTime(1.0 * (i-1)), 
			cc.CallFunc(function()
				cc.Audio:playEffect("GetItem.mp3")
				if count == i then
					self.m_isEnableShowTips = true
				end
			end),
			cc.Spawn( cc.FadeIn(0.14) ), 
			cc.DelayTime(1.0), 
			cc.Spawn(cc.FadeOut(2), 
			cc.MoveBy(1.0, 0, 90, "BackOut")
			), 
			cc.RemoveAction())
		node:runAction(action)
	end	
end

function This:showTips(content, params)
	if self.m_isShowing then
		return 
	end
	
    local node = cc.Node:createFromCSFile("GameScene/NodeUI/Tips.csb")
    node:setPosition(cc.visible.center.x, cc.visible.center.y - 100)
    node.content:setString(content)
    self:addChild(node, 100000)
    local width, height = node.content:getContentSize()
	node.bg:setContentSize(width + 50, height + 10)
    node:runAction(cc.Sequence(cc.MoveBy(0.6, 0, 100, "BackOut"), cc.DelayTime(0.5), cc.FadeOut(0.2), cc.RemoveAction()))
    node:runAction(cc.FadeIn(0.2))

    self.m_timer:setTimer(1, function()
        self.m_isShowing = false
    end,"tips_show")
    self.m_isShowing = true

    return node
end

--if os.isWin32 then
	function This:showMoneyTips(content, params)
		local delayFactor, colorId = 1, nil
		if params then
			delayFactor = params.delayFactor or 1
			colorId = params.colorId
		end

	    local node = cc.Node:createFromCSFile("GameScene/NodeUI/Tips.csb")
	    node:setPosition(cc.visible.center.x, cc.visible.center.y -100)
	    node.content:setString(content)
	    if colorId then
	        node.content:setColorByID(colorId)
	    end
	    node:setPosition(cc.visible.center.x, cc.visible.center.y + 50)
	    node:setScale(1)

	    self:addChild(node, 100000)
	    local width, height = node.content:getContentSize()
		node.bg:setContentSize(width + 50, height + 10)
	    node:runAction(cc.Sequence(cc.MoveBy(0.6 * delayFactor, 0, 100, "BackOut"), cc.DelayTime(0.5 * delayFactor), cc.FadeOut(0.2 * delayFactor), cc.RemoveAction()))
	    node:runAction(cc.FadeIn(0.2 * delayFactor))
	    --self.m_timer:setTimer(0 * delayFactor, function()
	        --self.m_isShowing = false
	    --end,"tips_show")
	    --self.m_isShowing = true

	    return node
	end
--end

function This:handleHelpTips(node, getInfoHandler)
    local tips = nil
    node:setTouch({
        onBegan = function(_,touch)
            local rect = node:getBoundingBox()
            if rect:isContainsPoint(touch) then
                local info = getInfoHandler()
                if info then
                	if not info.node then
                		info.node = node
                	end
                    tips = g_scene:openDialog(info.fileName or "UI.HelpTips", info)
                    return true
                end
            end
            return false
        end,
        onEnded = function()
            if tips and tips[0] ~= 0 then
                tips:close()
                tips = nil
            end
        end,
        isScale = false,
        isDynamic = true,
		isSwallow = false,
        onCanceled = function()
            if tips and tips[0] ~= 0 then
                tips:close()
                tips = nil
            end
        end,
    })
end

function This:showVoiceState(state)
	if state == VoiceState.kNone and self.m_voiceStateNode then
		self.m_voiceStateNode:removeFromParent()
		self.m_voiceStateNode = nil
		return
	end

	if not self.m_voiceStateNode then
		self.m_voiceStateNode = require("Chat.VoiceStateView"):create()
		self:addChild(self.m_voiceStateNode,1000)
		self.m_voiceStateNode:setPosition(cc.visible.center.x,cc.visible.center.y)
	end
	self.m_voiceStateNode:setState(state)
end

local catchErrorString = {
    ["powerLack"] = 10703,
    ["diamondLack"] = 3,
    ["goldLack"] = 4,
    ["hadFamily"] = 11117,
    ["foodLack"] = 11102,
    ["copperLack"] = 11103,
}

local catchErrorCode = {  
    [3] = function()  
	    require("Player.GamePlayer"):checkDiamond(0 ,true)
    end,  
    [4] = function()  
        require("Player.GamePlayer"):checkGold({nocheck = true})
    end, 
    [10703] = function(scene, info)  
		if require("System.SystemManager"):getPowerIsFull() then
    		g_scene:showTips(Serialize["System_SystemManager_1"])
		elseif require("System.SystemManager"):canBuyPower() then
			if not info or not info.notips then
	    		scene:showTips(Serialize[10703])
	    	end
            scene:openDialog("System.BuyPowerMain")
        else
            scene:showTips(Serialize[10703])
        end
    end,
    [11102] = function(scene, info)
		if not info or not info.notips then
    		scene:showTips(Serialize[11102])
    	end
		if scene:getType() == SceneType.kLegionScene then
			return
		end
		require("PlanShop.PlanShopUtil"):usePlan({plan=JiCeType.kWuGuFengDeng, num = 1})
    end,
    [11103] = function(scene, info)
		if not info or not info.notips then
    		scene:showTips(Serialize[11103])
    	end
    	
		if scene:getType() == SceneType.kLegionScene then
			return
		end
		require("PlanShop.PlanShopUtil"):usePlan({plan=JiCeType.kCaiYuanGunGun, num = 1})
    end
}

function This:handleErrorCode(errorCode, info)
    if errorCode == nil then
        return
    end
    print("errorCode: " .. errorCode)
    local catchCode = catchErrorString[errorCode]
    if catchCode then
    	errorCode = catchCode
    end

    local ret = true
	local funcHandle = catchErrorCode[errorCode]
	if funcHandle and (not info or info.callback == nil) then --code exists  
	    ret = funcHandle(self, info)
	elseif info then --code not found  
		ret = not info.notips
		local checkHandler = info.check 
		if checkHandler and checkHandler() or info.nocheck then
	    	if info and info.callback then
	    		info.callback()
	    	end
        end
    end

    if ret then
	  	if Serialize[errorCode] then
			self:showTips(Serialize[errorCode])
		else
			self:showTips(Serialize[60073]..errorCode)
		end
	end
end

function This:catchErrorCode( errorCode )
end

function This:pauseGame()
end

function This:resumeGame()
end


function This:addDelayDialog(info)
	for i , dialog in ipairs(This.delayDialogList) do
		if dialog.name == info.name then
			return
		end
	end
	table.insert(This.delayDialogList,info)
end

function This:checkDelayDialog()
	if #This.delayDialogList > 0 then
		local info = This.delayDialogList[1]
		if not info.scene or info.scene == g_scene:getName() then
			self:openDialog(info.name,info.arg)
		end
		table.remove(This.delayDialogList,1)
	end
end

local noticeList = {}
function This:addNoticeBoard(text, priority, style)
    priority = priority or 0
    if priority > 0 then
        local index = 1
        for i = 1, #noticeList do
            local notice = noticeList[i]
            if notice[2] < priority then
                index = i
                break
            end
        end
        table.insert(noticeList, index, {text, priority, style})
    else
        noticeList[#noticeList + 1] = {text, priority, style}
    end

    self:showNoticeBoard()
end

function This:getNodePathByPoint(point)
	local depth = 0
	local target = nil
	local path = ""

	function getNodeContainPoint(node,dp)
		local children = node:getChildren()
		for i = #children , 1 , -1 do
			local child = children[i]
			local name = child:getName()
			if child:isVisible() and child.m_touch and not target then
				local cd = child.m_touch.sizeNode or child 
				local rect = cd:getBoundingBox()
				if rect:isContainsPoint(point) then
					target = child
					depth = dp
					path = name
					return child
				end
			end
			if child:isVisible() then
				local targetNode =  getNodeContainPoint(child,dp + 1)
				if targetNode then
					path = name .. "." .. path
					return targetNode
				end
			end
		end		
	end
	getNodeContainPoint(self,1)
	if target then
		path = g_scene:getName() .. "." .. path
		local w , h = target:getContentSize()
		local px, py = target:convertToWorldSpace(w/2,h/2)
		local label = cc.Label:createNormalTTF(path,18)
		self:addChild(label)
		label:setLocalZOrder(100000)
		
		local w , h = label:getContentSize()
		if px > w then
			label:setAnchorPoint(1,0)
		elseif cc.visible.width - px > w then
			label:setAnchorPoint(0,0)
		else
			label:setAnchorPoint(0,0)
			label:setString(path,cc.visible.width - px)
		end

		label:setColor(255,0,0)
		label:setPosition(px, py)

		lib2d.writeToClipboard(path)
		return label
	end
end

function This:addGetNodePathListener()
	local target = nil
	if g_setting.isDebug and not self.m_getPathListener then
		self.m_getPathListener = cc.TouchListener:create({
			onBegan = function(_,touch)
				if target and target[0] ~= 0 then
					target:removeFromParent()
					target = nil
				end 
				target = self:getNodePathByPoint(touch)
				return target ~= nil
			end,
			onEnded = function(_,touch)
			end,
			isSwallow = false
		})
		self:addEventListener(self.m_getPathListener,-1)
	end
end

function This:createNoticeBoard(content, style)
	local node
	if not style then
    	node = require("Chat.BroadcastView"):create(content)
    else
        node = require("Chat.BroadcastView"):create(content,style)
    end
	local w , h = node:getContentSize()
    node:setPosition(cc.visible.center.x - w/2, cc.visible.top.y - 80)
    self:addChild(node)
    return node
end

function This:showNoticeBoard()
    if not self.m_noticeBoard and #noticeList > 0 then
        self.m_noticeBoard = self:createNoticeBoard(noticeList[1][1], noticeList[1][3])
        table.remove(noticeList, 1)
        self.m_noticeBoard:setFinishHandler(function ()
            self.m_noticeBoard:removeFromParent()
            self.m_noticeBoard = nil
            self:showNoticeBoard()
        end)
    end
end

function This:showVoiceEffect(isShow)
	
end

function This:onConnectLost()
	print("失去连接...")
	local panel = self:openDialog("Login.ReloginUI")
    panel:setToConnectLost()
end

function This:onAccountConflict()
	print("账号冲突...")
	self:openDialog("Login.ReloginUI")
end