--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local GamePlayer = require("Player.GamePlayer")
local LoginManager = require("Login.LoginManager")
local GuideManager = require("Guide.GuideManager")
local VipManager = require("Vip.VipManager")
local Json = require("Json")
local Socket = require("Net.Socket") 
local GameUtil = require("Util.GameUtil")
local PluginEvent = require "xgame.PluginEvent"
local ShopConfig = require("Metadatas.ShopConfig")

local kHasSDK = true
local kUseThirdSDK = kHasSDK and not os.isWin32

local SDKPlayerInfoKey = setmetatable({}, {
    __index = function (s, k)
    	local ret
        if k == "kCreateRole" then
			ret = "createRole"
        elseif k == "kEnterServer" then
			ret = "enterServer"
        elseif k == "kLevelUp" then
			ret = "levelUp"
        elseif k == "kGoldChanged" then
			ret = "goldChanged"
        end

		local ServerConfig = require("Net.ServerConfig")
        if ret and ServerConfig.isSdkNeedPrefix then
        	ret = "ky_" .. ret
		elseif not ret then
			ret = ""
        end
        s[k] = ret
        return s[k]
    end
})

function This:init()
	if kUseThirdSDK then
		self:addPlugin("EjGame")
	end
	self:initListener()
	self.m_sdkIsLogin = false
end

function This:addPlugin(name)
	if not kUseThirdSDK then
		return
	end
	local plugin = require("xgame." .. name):create()
	plugin:addListener(PluginEvent.AUTH_FAILURE, function( _, ... )
		self:onSdkLoginFailedNotify(...)
	end)
	plugin:addListener(PluginEvent.AUTH_SUCCESS, function( _, ... )
		self:onSdkLoginSecceedNotify(...)
	end)
	plugin:addListener(PluginEvent.PAY_CANCEL, function( _, ... )
		self:onPaymentFailedNotify(...)
	end)
	plugin:addListener(PluginEvent.PAY_FAILURE, function( _, ... )
		self:onPaymentFailedNotify(...)
	end)
	plugin:addListener(PluginEvent.PAY_SUCCESS, function( _, ... )
		self:onPaymentSecceedNotify(...)
	end)
	plugin:addListener(PluginEvent.SWITCH_SUCCESS, function( _, ... )
		self:onSdkLogoutNotify(...)
	end)
    plugin:addListener(PluginEvent.KEYBOARD_CHANGE, function( _, high )
        print("PluginEvent.KEYBOARD_CHANGE")
        local param = tonumber(high)
        if param > 0 then
            EventDispatcher:dispatcher("ios_keyboard_show", param)
        else
            EventDispatcher:dispatcher("ios_keyboard_hide")
        end
    end)
	self.plugin = plugin
end

function This:initListener()
	EventDispatcher:addListener(self,"guide_finish", self.onRecordEvent1Notify)
	EventDispatcher:addListener(self,"guide_start", self.onRecordEvent2Notify)
	EventDispatcher:addListener(self,"family_change", self.onRecordEvent4Notify)

	EventDispatcher:addListener(self,"create_role",self.onCreateRoleNotify)
	EventDispatcher:addListener(self,"player_attr_changed",self.onPlayerAttrChangedNotify)
	EventDispatcher:addListener(self,"enter_game_server",self.onEnterGameServerNotify)
	EventDispatcher:addListener(self,"sdk_payment_secceed",self.onPaymentSecceedNotify)
	EventDispatcher:addListener(self,"sdk_payment_failed",self.onPaymentFailedNotify)

	EventDispatcher:addListener(self,"sdk_login_secceed",self.onSdkLoginSecceedNotify)
	EventDispatcher:addListener(self,"sdk_login_failed",self.onSdkLoginFailedNotify)
	EventDispatcher:addListener(self,"sdk_logout",self.onSdkLogoutNotify)
	EventDispatcher:addListener(self,"login_login_server",self.onLoginLoginServerNotify)
	-- EventDispatcher:addListener(self,"jpush_regist", self.onJpushRegistNotify)
	EventDispatcher:addListener(self,10211,self.onApplyBillIDReply)
	EventDispatcher:addListener(self,10007,self.onPushJPushIDToServerReply)
	EventDispatcher:addListener(self,10218,self.onCommitShopItemReply)
end

function This:isLogin()
	return self.m_sdkIsLogin
end

function This:submitPlayerInfo(event)
	if GamePlayer:getPlayerID() <= 0 then
		return
	end
	if GamePlayer:getRoleName() == "" then
		return
	end
	if GamePlayer:getRoleCreateTime() <= 0 then
		return
	end

	if event == SDKPlayerInfoKey.kEnterServer and GamePlayer:getLevel() < 0 then
		return
	end

    print("*****************submitPlayerInfo******************")
	local tmp = {}
	tmp.roleId = GameUtil:int64ToString(GamePlayer:getPlayerID())
	tmp.roleName = GamePlayer:getRoleName()
	tmp.roleLevel = tostring(GamePlayer:getLevel())
	local server = LoginManager:getLoginServer()
	tmp.serverName = server:getName()
	tmp.serverId = tostring(server:getId())

	local index1 , index2 = GuideManager:getProgress()
	tmp.processId = tostring(index1 * 100 + index2)
	tmp.balance = tostring(GamePlayer:getGold())
	tmp.currency = tostring(GamePlayer:getDiamond())
	tmp.vip = tostring(VipManager:getMaxMemberLevel())
	tmp.power = tostring(GamePlayer:getPower())
	tmp.roleTime = tostring(GamePlayer:getRoleCreateTime())
	tmp.scene = event or ""
	tmp.userCode = ""

	if self.m_sdkInfo and self.m_sdkInfo.sdkuid then
		tmp.userCode = self.m_sdkInfo.sdkuid
	end

	local str = Json.encode(tmp)
	print(str)

	if os.isWin32 then
		return
	end
 
	if kUseThirdSDK then
		self.plugin:upload(tmp)
	end
end

function This:warm(warming, handler)
	if not os.isWin32 then
		self.plugin:warm(warming, handler)
	end
end

function This:login()
	if kUseThirdSDK then
		self.plugin:auth()
	else
		g_scene:openDialog("Login.LoginUI")
	end
end

function This:loginOut()
	if kUseThirdSDK then
		self.plugin:logout()
	end
end

function This:recharge(billInfo)
	local order = {}
	order.orderId = billInfo.billId
	order.pid = billInfo.productId
	-- order.sign = billInfo.extra
	order.roleId = GameUtil:int64ToString(GamePlayer:getPlayerID())
	order.name = GamePlayer:getRoleName()
	order.level = tostring(GamePlayer:getLevel())
	local server = LoginManager:getLoginServer()
	order.server = tostring(server:getId())

	local config = ShopConfig[tonumber(order.pid)] or ShopConfig[tonumber(billInfo.pid_src)]
	if config then
		order.price = string.format("%.2f", config.price)
		order.subject = config.name
	end

	if not kUseThirdSDK then return end
	self.plugin:pay(order, true)
end

function This:onApplyBillIDRequest(itemId)
	if kUseThirdSDK then
		local pfId = os.native_call("getPfId")
		Socket:sendAndLock(10211,{item_id = itemId, pf_id = tonumber(pfId or 0)})
	else
		local ChatManager = require("Chat.ChatManager")
		ChatManager:onSendGmRequest("recharge " .. tostring(itemId))
	end
end

function This:onApplyBillIDReply(msg)
	if msg.err_code == 0 then
		local config = require("Metadatas.CorrespondingConfig") or {}
		local pid = tonumber(msg.product_id)
		local pidconfig = config[pid]
		local tmp = {}
		tmp.billId = msg.order_id
		tmp.productId = tostring((pidconfig and pidconfig.corresponding_id or pid) or pid)
		tmp.extra = msg.extra_info
        	tmp.pid_src = pid
		print(string.format("billId = %s, productId = %s, extra = %s",tostring(tmp.billId),tostring(tmp.productId),tostring(tmp.extra)))
		self:recharge(tmp)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onCommitShopItemRequest(billInfo)
	local pfId = os.native_call("getPfId")
	Socket:sendAndLock(10218, {
		product_id = billInfo.productId, 
		pfid = tonumber(pfId or 0),
		order_id = billInfo.billNo,
		billNo = billInfo.orderId,
		buytoken = billInfo.token,
	})
end

function This:onCommitShopItemReply(msg)
	if msg.err_code == 0 then
		print( "訂單兌現成功！正在消耗" )
		self.plugin:consumeProduct(msg.buytoken)
	elseif msg.err_code == 10208 then
		print( "訂單已兌現，尚未消耗！" )
		self.plugin:consumeProduct(msg.buytoken)
	elseif msg.err_code == 10209 then
		print( "10209 訂單token錯誤！" )
		g_scene:showTips("訂單驗證有誤！錯誤碼：209，請聯係客服")
	elseif msg.err_code == 10210 then
		print( "10210 訂單billNo錯誤！" )
		g_scene:showTips("訂單驗證有誤！錯誤碼：210，請聯係客服")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onSdkLoginSecceedNotify(msg)
	self.m_sdkInfo = {}
	print(msg)

	local str = Json.decode(msg)
	self.m_sdkInfo.sdkuid = str.uid
	self.m_sdkInfo.useName = str.useName
	self.m_sdkInfo.token = str.token or ""
	self.m_sdkInfo.mac = str.mac or ""
	self.m_sdkInfo.serial = str.serial or ""
	self.m_sdkInfo.agent = str.agent or ""
	self.m_sdkInfo.imei = str.imei or ""
	self.m_sdkInfo.systemId = str.systemId or ""
	self.m_sdkInfo.systemInfo = str.systemInfo or ""
	self.m_sdkInfo.ip = str.ip or ""
	self.m_sdkInfo.imei2 = ""
	self.m_sdkInfo.idfa = ""
	self.m_sdkInfo.idfv = ""
	self.m_sdkInfo.udid = str.udid or ""
	self.m_sdkInfo.screen = cc.visible.frameWidth .. "X" .. cc.visible.frameHeight
	if os.isWin32 or os.isIosSimulator then
		self.m_sdkInfo.type = "0"
	elseif os.isAndroid then
		self.m_sdkInfo.type = "1"
	elseif os.isIOS then
		self.m_sdkInfo.type = "2"
	end
	self.m_sdkInfo.ver = str.ver or ""

	if os.isAndroid then
		local subchannelId = str.subchannelId
		if not subchannelId then
			subchannelId = ""
		elseif subchannelId ~= "" then
			local scNumber =  tonumber(subchannelId)
			if scNumber then
				local scStr = tostring(scNumber)
				if scStr ~= subchannelId then
					print("子渠道修正：".. subchannelId, scStr)
					subchannelId = scStr
				end
			else
				subchannelId = ""
			end
		end
		self.m_sdkInfo.loginChannel = ((str.channel or str.channelId) .. subchannelId) or "1"
	else
		self.m_sdkInfo.loginChannel = str.channel or str.channelId or "1"
	end
	
	self.m_sdkInfo.gameId = str.gameId or "1"
	self.m_sdkInfo.city = str.city or ""
	self.m_sdkInfo.province = str.province or ""
	self.m_sdkInfo.password = ""
	self.m_sdkInfo.jpushId = str.jpushId
	self.m_sdkInfo.pfId = tonumber(str.pfId or 0)
	LoginManager:onLoginLoginServerRequest(self.m_sdkInfo)
end

function This:onJpushRegistNotify(pushId)
	self.m_sdkInfo = self.m_sdkInfo or {}
	self.m_sdkInfo.jpushId = pushId
	local platform = os.isIos and "ios" or "android"
	Socket:sendAndLock(10007,{jpush_id = pushId, role_id = GamePlayer:getPlayerID(), platform = platform})
end

function This:onPushJPushIDToServerRequest()
	if kUseThirdSDK then
		return
	end

	local pushId = ""
	local platform = os.isIos and "ios" or "android"
	if self.m_sdkInfo.jpushId and self.m_sdkInfo.jpushId ~= "" then
		pushId = self.m_sdkInfo.jpushId
	else
		pushId = os.native_call("getJPushRegisterID")
	end
	print(string.format("----------------pushId = %s-------------",pushId))
	Socket:sendAndLock(10007,{jpush_id = pushId, role_id = GamePlayer:getPlayerID(), platform = platform})
end

function This:onPushJPushIDToServerReply(msg)
	if msg.err_code == 0 then
		local str = GameUtil:int64ToString(GamePlayer:getPlayerID())
		os.native_call("bindAccount",str)
	end
end

----------------------------------------------------
function This:onSdkLoginFailedNotify(msg)
	g_scene:showTips("login fail!")
end

function This:onCreateRoleNotify()
	print("------------onCreateRoleNotify----------")
	self:submitPlayerInfo(SDKPlayerInfoKey.kCreateRole)
end

function This:onPlayerAttrChangedNotify(info)
	print("-----------onPlayerAttrChangedNotify------------")
	if info.attrId == PlayerAttr.level then
		self:submitPlayerInfo(SDKPlayerInfoKey.kLevelUp)
	elseif info.attrId  == PlayerAttr.gold then
		self:submitPlayerInfo(SDKPlayerInfoKey.kGoldChanged)
	end
end

function This:onEnterGameServerNotify()
	print("------------onEnterGameServerNotify-------------")
	self:submitPlayerInfo(SDKPlayerInfoKey.kEnterServer)
end

function This:onPaymentSecceedNotify(billInfo)
	if billInfo then
		print(string.format("订单%s充值成功", billInfo.orderId or ""))
	end
	g_scene:showTips(Serialize["Top-Ups-Tip"]);
end

function This:onPaymentFailedNotify(msg)
	print(string.format("充值失败:%s",msg))
	g_scene:showTips(Serialize["Top-Ups-Tip1"]);
end

function This:onSdkLogoutNotify(logout)
	LoginManager:switchAccount(logout)
end

function This:onLoginLoginServerNotify()
	self.m_sdkIsLogin = true
end

function This:onRecordEvent1Notify()
    self:onRecordEventNotify(1)
end

function This:onRecordEvent2Notify()
    self:onRecordEventNotify(2)
end

function This:onRecordEvent4Notify()
    self:onRecordEventNotify(4)
end

function This:onRecordEventNotify(event, param)
    if kUseThirdSDK then
        self.plugin:record(event, param or "")
    end
end

--endregion
