-- @Author: zheng
-- @Date:   2018-08-13 15:37:02
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-09-03 15:49:27
-- @File Name: GameSingleChapter.lua


local This = class(...)
local TeamCampaignConfig = require("Metadatas.TeamCampaignConfig")
local SingleManager = require("SingleDaily.SingleManager")

local GamePlayer = require("Player.GamePlayer")

function This:create(...)
	local chapter = setmetatable({},self)
	chapter:init(...)
	return chapter
end

function This:init(id)
	self.m_id = id
	self.m_stages = {}
	self.m_config = TeamCampaignConfig[id]
	self.m_preChapter = nil
	self.m_nexChapter = nil
	self.m_challengeTimes = 0 --剩余挑战的次数
	self.m_resetTimes = 0 --剩余购买次数
	self:initSelectIndex()
end

function This:addStage(stage)
	table.insert(self.m_stages,stage)
end

function This:setPreChapter(chapter)
	self.m_preChapter = chapter
end

function This:getPreChapter()
	return self.m_preChapter
end

function This:setNexChapter(chapter)
	self.m_nexChapter = chapter
end

function This:getNexChapter()
	return self.m_nexChapter
end

function This:onStageLoadFinish()
	table.sort(self.m_stages,function(s1,s2)
		return s1:getId() < s2:getId()
	end)
end

function This:getId()
	return self.m_id
end

function This:getStages()
	return self.m_stages
end

function This:getChallengeTimes(  )
	return self.m_challengeTimes>0 ,self.m_challengeTimes
end

function This:setChallengeTimes( challengeTime )
	self.m_challengeTimes = challengeTime
end

function This:setResetTiem(resetTimes)
	self.m_resetTimes = resetTimes
end

function This:getResetTiem()
	return self.m_resetTimes>0 ,self.m_resetTimes
end

function This:getResetCost()
	local res =0
	local cost = self.m_config.change_cost
	local resetCount = #cost
	if self.m_resetTimes>0 and self.m_resetTimes<=resetCount then
		local temResetTime = self.m_resetTimes-1
		local config = cost[resetCount-temResetTime]
		if config then
			res = config
		end
	end
	return res
end

function This:getName()
	return self.m_config.name
end


function This:initSelectIndex()
	self.m_selectIndex =1
	local stages = self:getStages()
	local stagesCount = #stages
	for stageIndex = stagesCount, 1, -1 do
		local stage = stages[stageIndex]
		local islock,lockAttr = stage:isLock()
		if islock then
			self.m_selectIndex = stageIndex
			break
		end
	end
end

function This:getMinSelectStage()
	local stages = self:getStages()
	local stagesCount = #stages
	local minIndex = stages[1]
	for stageIndex = stagesCount, 1, -1 do
		local stage = stages[stageIndex]
		local islock,lockAttr = stage:isLock()
		if islock then
			minIndex = stage
			break
		end
	end
	return minIndex
end

function This:setSelectIndex( selectIndex )
	self.m_selectIndex = selectIndex
end

function This:getSelectIndex( )
	return self.m_selectIndex
end

function This:getSelectStage( selectIndex )
	local temSelectIndex = selectIndex
	if not temSelectIndex then
		temSelectIndex = self.m_selectIndex
	end
	local stages = self:getStages()
	return stages[temSelectIndex]
end

function This:setData(data,isResetTime )
	local left_times = data.left_times or 0
	local reset_times = data.reset_times or 0
	local difficulty_list = data.difficulty_list or 0
	self:setResetTiem(reset_times)
	self:setChallengeTimes(left_times)
	if not isResetTime then
		for k,item in pairs(difficulty_list) do
			local stage = SingleManager:getStage( item.id )
			if stage then
				stage:setData(item)
			end
		end
	end
	
end
