-- @Author: zheng
-- @Date:   2018-08-13 15:37:02
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-09-01 18:13:15
-- @File Name: GameSingleStage.lua

local This = class(...)

local StageConfig = require("Metadatas.StageDifficultyConfig")
local DropConfig = require("Metadatas.DropConfig")
local SingleManager = require("SingleDaily.SingleManager")

local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")

SingleStageLockType ={
	isLv =1,
	isPass =2,
	allPass =3,
}

function This:create(id)
	local stage = {}
	setmetatable(stage,This)
	stage:init(id)
	return stage
end

function This:init(id)
	self.m_id = id
	self.m_config = StageConfig[id]
	self.m_MaxProgress = 0
	self.m_ProgressChange = 0
	self.m_CurentPercent = 0
	self.m_preStage = nil
	self.m_emBattleHeros ={}--所有队列里的英雄
	self.m_Team = {}--所有队列里
end

function This:setPreStage(stage)
	self.m_preStage = stage
end

function This:getPreStage()
	return self.m_preStage
end

function This:getId()
	return self.m_id
end

function This:getChapterId()
	return self.m_config.type
end

function This:getBgId(  )
	return self.m_config.bg_id
end

function This:getTitle()
	return self.m_config.difficulty_des
end

function This:getDes(  )
	return self.m_config.des
end

function This:getRecommend(  )
	return self.m_config.recommend
end

function This:isPassed()
	return self.m_MaxProgress ==100
end

function This:getPlayerLevelLimit()
	return self.m_config.level_limit
end

function This:getMaxProgress()
	return self.m_MaxProgress
end

function This:setMaxProgress(MaxProgress)
	self.m_MaxProgress = MaxProgress
end

function This:setProgressChange( ProgressChange )
	self.m_ProgressChange = ProgressChange
end

function This:getProgressChange()
	return self.m_ProgressChange>0,self.m_ProgressChange
end

function This:setCurentPercent( CurentPercent )
	self.m_CurentPercent = CurentPercent
end

function This:getCurentPercent()
	return self.m_CurentPercent
end

function This:isLock()
	local lvIsCan = GamePlayer:getLevel()>=self:getPlayerLevelLimit()
	if self.m_preStage then
		local isPasse = self.m_preStage:isPassed()
		if lvIsCan and isPasse  then
			return lvIsCan ,SingleStageLockType.allPass
		elseif not lvIsCan then
			return lvIsCan,SingleStageLockType.isLv
		elseif not isPasse then
			return isPasse,SingleStageLockType.isPass
		end
	 else
		return  lvIsCan,SingleStageLockType.isLv
	 end 
end


function This:resetInEmbattle()
	self.m_emBattleHeros ={}
	local function addATeam( team )
		for k,uid in pairs(team) do
			table.insert(self.m_emBattleHeros,uid)
		end
	end
	for k,team in pairs(self.m_Team) do
		addATeam( team )
	end
end

function This:isInEmbattle(heroId)
	for pos , id in pairs(self.m_emBattleHeros) do
		if id == heroId then
			return true
		end
	end
	return false
end

function This:getAllHeros( heros )
	local res = {}
	for i,hero in pairs(heros) do
		local id = hero:getId()
		if not self:isInEmbattle(id) then
			table.insert(res,hero)
		end
	end
	return res
end

function This:getEmBattle(  )
	return self.m_emBattleHeros
end

function This:getMaxTeam()
	return 3
end

function This:getTeamCount(  )
	return self.m_config.ranks
end



function This:setAllTeam( AllTeam )
	self.m_Team = AllTeam
	self:resetInEmbattle()
end

function This:getAllTeam()
	return self.m_Team
end


function This:setTeam( TeamData,teamIndex)
	if self:getTeamCount( )>=teamIndex then
		local team ={}
		for i,uid in ipairs(TeamData) do
			if uid>0 then
				local uhero = HeroManager:getHeroByUid(uid)
				team[i] = uhero:getId()
			end
		end
		self.m_Team[teamIndex] = team
		self:resetInEmbattle()
		SingleManager:onSaveHeroList( self:getId() )
	end
end

function This:setTeamHeroIdList( TeamData,teamIndex )
	if TeamData and self:getTeamCount( )>=teamIndex and table.count(TeamData)>=0 then
		self.m_Team[teamIndex] = TeamData
		self:resetInEmbattle()
		SingleManager:onSaveHeroList( self:getId() )
	end
end

function This:getTeam(teamIndex,isSort)
	local res = {}
	if teamIndex > self:getTeamCount( ) then
		return res
	end
	local temRes = self.m_Team[teamIndex]
	if temRes then
		if isSort then
			local temRes2 ={}
			for k,hroId in pairs(temRes) do
				table.insert(temRes2,hroId)
			end
			return temRes2
		end
		return temRes
	end
	return res
end

function This:getDrops(dropId)
	local result = {}
	local drops = DropConfig[dropId].drop_info
	for i , info in pairs(drops) do
		table.insert(result,{info[1],info[2]})
	end
	return result
end

function This:getRewardList()
	if self.m_reward then
		return self.m_reward
	end
	local res = {}
	local function addReward( rewards )
		for i,item in pairs(rewards) do
			table.insert(res,item)
		end
	end
	local attr1 = self.m_config.drop_id1
	local attr2 = self.m_config.drop_id2
	addReward(attr1)
	for k,config in pairs(attr2) do
		local dropId = config
		local attr = self:getDrops(dropId)
		addReward(attr)
	end
	self.m_reward = res
	return self.m_reward
end


function This:setData(data,isChalenge)
	local score = data.score or 0
	local score2 = score - self:getMaxProgress()
	if score2>0 then
		self:setMaxProgress(score)
	end
	if isChalenge then
		self:setCurentPercent( score )
		self:setProgressChange(0)
		if score2>0 then
			self:setProgressChange(score2)
		end
	end
end
