-- @Author: zheng
-- @Date:   2018-12-13 16:11:42
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-01 01:42:24
-- @File Name: GameTowerData.lua

local This = class(...)
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")
local WarfareManager = require("Warfare.WarfareManager")

local TierRewardConfig = require("Metadatas.TierRewardConfig")
local TowerAssistConfig = require("Metadatas.TowerAssistConfig")
local TowerMasterConfig = require("Metadatas.TowerMasterConfig")
local GameTowerLevelReward = require("SingleDaily.GameTowerLevelReward")
local GameTowerMaster = require("SingleDaily.GameTowerMaster")
local GameTowerDayReward = require("SingleDaily.GameTowerDayReward")

-- local SingleManager = require("SingleDaily.SingleManager")
SingleTowerLimitType ={
    kHelpTime = "tower_help_time", --镇河塔（林夏超）请求协助cd（秒）
    kMasterOpen = "tower_master_open", --镇河塔（林夏超）争夺塔主系统开启条件（关卡）
    kMasterTime = "tower_master_time", --镇河塔（林夏超）争夺塔主清除CD消耗
    kListOpen = "tower_list_open", --镇河塔（林夏超）争夺塔主队列开启条件{第一队列,第二队列，第三队列}(关卡）
}

SingleTowerDayRewardState ={
    kFreeReward = 1,    --免费奖励
    kMoneyReward = 2,   --付费奖励
}
SingleTowerChallengeType ={
    kLevelType = 1,--挑战关卡，包括协助时挑战关卡
    kMasterType = 2,--霸主争夺
}
kSingleTowerCengNum = 5
function This:create(id)
    local o = {}
    setmetatable(o,This)
    o:init(id)
    return o
end

function This:init(id)
    self.m_chapters = {}
    self.m_levels ={}
    for itemId,itemData in ipairs(TierRewardConfig) do
        local chapterId = itemData.layer
        local item1 = self.m_chapters[chapterId]
        if not item1 then
            item1 = {}
            self.m_chapters[chapterId]= item1
        end
        local levelModel = GameTowerLevelReward:create(itemId)
        table.insert(item1,levelModel)
        self.m_levels[itemId] = levelModel
    end
--    for _,chapterData in ipairs(self.m_chapters) do

--        table.sort(chapterData,function(a,b)
--            local idA = a:getId()
--            local idB = a:getId()
--            return idA>idB
--        end)
--    end

    self.m_masters = {}
    for itemId,itemData in ipairs(TowerMasterConfig) do
        local item1 = GameTowerMaster:create(itemId)
        self.m_masters[itemId] = item1
    end
    self.m_yesterdayLevel = 0
    self.m_levelNum = 1
end

function This:getDayReward()
    if not self.m_dayReward then
        self.m_dayReward = GameTowerDayReward:create()
    end
    return self.m_dayReward
end

--互助奖励内容
function This:getHelpReward()
    local item1 = TowerAssistConfig[1]
    return item1.reward
end

--协助次数上限
function This:getHelpLimit()
    local item1 = TowerAssistConfig[1]
    return item1.order
end

function This:updateMasterList(masterList)
    if #masterList>0 then
        for _,master in ipairs(self.m_masters) do
            master:updateInfor()
        end
    end
    for _,masterInfor in ipairs(masterList) do
        local id = masterInfor.level
        local master = self:getMaster(id)
        if master then
            master:updateInfor(masterInfor)
        end
    end
end

function This:getMasterlist(isNotSelect)
    local result = {}
    if isNotSelect then
        for _,masterModel in ipairs(self.m_masters) do
            table.insert(result,masterModel)
        end
        return result
    else
        for _,masterModel in ipairs(self.m_masters) do
            if masterModel:getRoleId()~=0 then
                table.insert(result,masterModel)
            end
        end
        return result
    end
end

--获取霸主争夺所有防守的武将
function This:getMasterAllHeroList()
    local result = {}
    local function addHeros(table1,tableResult)
        for _,heroId in ipairs(table1) do
            table.insert(tableResult,heroId)
        end
    end
    for _,masterModel in ipairs(self.m_masters) do
        if masterModel:getRoleId()~=0 and masterModel:isMyHeroList() then
            local heroList = masterModel:getAllHeroIds()
            addHeros(heroList,result)
        end
    end
    return result
end

--获取自己占领的所有层数，获取最高层的霸主Id
function This:getMyMasterInforAndMaxMasterId()
    local myMasterlist ={}
    local masterList = self:getMasterlist()
    local maxMasterId = 0
    for _,masterModel in pairs(masterList) do
        if masterModel:isMyHeroList() then
            table.insert(myMasterlist,masterModel)
        end
        local masterId = masterModel:getId()
        if masterId >maxMasterId then
            maxMasterId = masterId
        end
    end
    return myMasterlist,maxMasterId
end


function This:getMaster(masterId)
    return self.m_masters[masterId]
end

function This:getLevel(levelId)
    return self.m_levels[levelId]
end

function This:getChaperList(chapterId)
    if not chapterId then
        chapterId = self:getCurentChapter()
    end
    return self.m_chapters[chapterId]
end

function This:getMaxChapterNum()
    return #self.m_chapters
end

function This:getCurentChapter()
    local stageId = self:getLevelNum(true)
--    if stageId>1 then
--        stageId = stageId - 1
--    end
    local chapterId = math.ceil(stageId/kSingleTowerCengNum)--向上取整
    local maxChapter = self:getMaxChapterNum()
    if chapterId>maxChapter then
        chapterId = maxChapter
    end
    return chapterId
end

-- --当前的
-- function This:setStageId(stageId)
--     self.m_stageId = stageId
-- end

-- function This:getStageId()
--     return self.m_stageId or 1
-- end

-- function This:getYesterdayReward()
--     local result = nil
--     if self.m_yesterdayLevel ~=0 then
--         local stageId = self:getStageId()
--         local chapterId = math.ceil(stageId/5)--向上取整
--         if not self.m_yesterdayReward then
--             self.m_yesterdayReward = GameTowerDayReward:create()
--         end
--         self.m_yesterdayReward:init(chapterId)
--         result = self.m_yesterdayReward
--     end
--     return result
-- end

function This:setYesterdayLevel(yesterdayLevel)
    self.m_yesterdayLevel = yesterdayLevel or 0
    if self.m_yesterdayLevel>0 then
        local dayReward = self:getDayReward()
        dayReward:init(yesterdayLevel)
    end
end

function This:getYesterdayLevel()
    return self.m_yesterdayLevel
end

function This:setLevelNum(levelNum)
    if levelNum and levelNum>1 then
        self.m_levelNum = levelNum
    end
end

--比真实的大1
function This:getLevelNum(checkLevelNum)
    if checkLevelNum then
        local maxLevel = self:getMaxLevel()
        if self.m_levelNum>maxLevel then
            return maxLevel
        end
        return self.m_levelNum
    else
        return self.m_levelNum
    end
end

function This:get()
    
end

function This:setLeftTeamCount(leftTeamCount)
    self.m_leftTeamCount = leftTeamCount
end

function This:getLeftTeamCount()
    return self.m_leftTeamCount or 0
end


--自己有队伍，自己不是队长返回true
--其他返回false
function This:myNotTeamLeader()
    -- local result = false
    -- -- GamePlayer:getPlayerID()
    -- if self:getTeamId() ==0 then
        
    -- end
    -- return result
    if self:isNotHaveTeam() then
        return false
    end
    return not WarfareManager:isTeamLeader(TeamType.kTowerTeam)
end

--自己没有队伍 返回 true
function This:isNotHaveTeam()
    -- return self:getTeamId() ==0
    return not WarfareManager:hasTeam(TeamType.kTowerTeam)
end

function This:setTeamId(teamId)
    -- self.m_teamId = teamId
end

--队伍id
function This:getTeamId()
    -- return self.m_teamId
    -- return WarfareManager:hasTeam(TeamType.kTowerTeam)
end

function This:getMaxLevel()
    return #self.m_levels
end

function This:setMasterlog(masterlog)
    self.m_masterlog = masterlog
end

function This:getMasterlog()
    -- self.m_masterlog = {{time =0,level =0,oldmaster_name ="aaaaaaaaaaaaa",newmaster_name ="bbbbbbbbbbbbbbbb"}}
    return self.m_masterlog or {}
end

function This:setRoleLog(roleLog)
    self.m_roleLog = roleLog
end

function This:getRoleLog()
--     self.m_roleLog= {{time =0,level =0,master_name ="1aaaaaaaaaaaaaaaaaa",state =1},
-- {time =0,level =0,master_name ="2aaaaaaaaaaaaaaaaaa",state =2},
-- {time =0,level =0,master_name ="3aaaaaaaaaaaaaaaaaa",state =3}}

    return self.m_roleLog or {}
end

function This:setCoolingTime(coolingTime)
    self.m_coolingTime = coolingTime
end

function This:getCoolingTime()
    return self.m_coolingTime or 0
end

function This:getTowerConfigNum(towerType)
    local result =0
    local item1 = ArgConfig[towerType]
    if item1 then
        local args = item1.arg
        local temNum = args[1]
        if type(temNum) == 'number' and #args==1  then
            return temNum
        end
        result = args
    end
    return result
end