-- @Author: zheng
-- @Date:   2018-08-13 15:37:02
-- @File Name: SingleManager.lua

local This = class(...)

local StageDifficultyConfig = require("Metadatas.StageDifficultyConfig")
local GameChapter = require("SingleDaily.GameSingleChapter")
local GameStage = require("SingleDaily.GameSingleStage")

local Socket = require("Net.Socket")         
local EventDispatcher = require("EventDispatcher")
local Serialize = require("Serialize.Serialize")
local GameUtil = require("Util.GameUtil")

local BattleDataManager = require("Battle.BattleDataManager")
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")

local SingleManagerFileName = "SingleManager"
ChallengeBoxState = {
	kLock = 1,
	kOpen = 2,
	kClose = 0
}

SingleManagerSaveState={
	kWorldMapHanXianDi = "HanXianDi"
}

local strList ={
	cost =Serialize[64167],
    cost1 = Serialize[64168],
	costLimit =Serialize[64169],
}
function This:init()
	self.m_chapter = {}
	self.m_stage = {}
	local chapterIds ={}
	for id , config in pairs(StageDifficultyConfig) do
		local chapterId = config.type
		if not self.m_chapter[chapterId] then
			self.m_chapter[chapterId] = GameChapter:create(chapterId)
			table.insert(chapterIds,chapterId)
		end
	end

	for id , chapter in pairs(self.m_chapter) do
		local preChapter = self.m_chapter[id - 1]
		if preChapter then
			chapter:setPreChapter(preChapter)
		end
		local nexChapter = self.m_chapter[id + 1]
		if nexChapter then
			chapter:setNexChapter(nexChapter)
		end
	end
	
	for id , config in pairs(StageDifficultyConfig) do
		local stage = GameStage:create(id)
		local chapter = self.m_chapter[config.type]
		chapter:addStage(stage)
		self.m_stage[id] = stage
	end

	for id , stage in pairs(self.m_stage) do
		local preStage = self.m_stage[id-1]
		stage:setPreStage(preStage)
	end
	for id , chapter in pairs(self.m_chapter) do
		chapter:onStageLoadFinish()
	end

	self.m_SelectChapter = nil

	self.m_emBattleHeros = {}
	self.m_observers = {}
	self.m_battleResult = {}
    self.m_isEnterRobber = false
	self:initEventListener()

end

function This:loadHeroList(  )
	for id , stage in pairs(self.m_stage) do
		self:onGetheroList(stage:getId())
	end
end

function This:initEventListener()
	EventDispatcher:addListener(self, 12900, self.onGetStateInfoNotify)--单人日常副本-信息(冲锋陷阵、决胜千里、运筹帷幄)
	EventDispatcher:addListener(self, 12901, self.onEmbattleHerosReply)--设置出战顺序
	EventDispatcher:addListener(self, 12902, self.onChallengeStateReply)--挑战
	EventDispatcher:addListener(self, 12903, self.onWarReportReply)
	EventDispatcher:addListener(self, 12905, self.onStageResetReply)--购买次数
end

function This:removeEventListener()
	EventDispatcher:removeListener(self, 12900)--单人日常副本-信息(冲锋陷阵、决胜千里、运筹帷幄)
	EventDispatcher:removeListener(self, 12901)--设置出战顺序
	EventDispatcher:removeListener(self, 12902)--挑战
	EventDispatcher:removeListener(self,12903)
	EventDispatcher:removeListener(self, 12905)--购买次数
end

function This:addObserver(o)
	for i , observer in pairs(self.m_observers) do
		if observer == o then
			return 
		end
	end
	table.insert(self.m_observers,o)
end

function This:removeObserver(o)
	for i , observer in pairs(self.m_observers) do
		if observer == o then
			table.remove(self.m_observers,i)
			return
		end
	end
end

function This:notifyObservers(funcName,...)
	local observers = {}
	for i , o in pairs(self.m_observers) do
		table.insert(observers,o)
	end
	
	for i, observer in pairs(observers) do
		if observer[funcName] then
			observer[funcName](observer,...)
		end
	end
end
-------------------------------------------------

function This:onSaveHeroList( stageId )
	self:notifyObservers("onHeroChange")
	local stage = self:getStage(stageId)
	if stage then
		local value = stage:getAllTeam()
		self:setPalyerActInfo(stageId, value,SingleManagerFileName)
	end
end

function This:onGetheroList(stageId)
	local stage = self:getStage(stageId)
	if stage then
		local value = self:getPalyerActInfo(stageId,{},SingleManagerFileName)
		stage:setAllTeam(value)
	end
end

function This:setIsHave(saveType,timeNum,fileName)
	if saveType  then
		self:setPalyerActInfo(saveType, {time= timeNum or -2},fileName)
	end
end

function This:getIsHave(saveType,timeNum,fileName)
	if saveType and timeNum then
		local value = self:getPalyerActInfo(saveType,{time= -2,have= false},fileName)
		return value.time == timeNum
	end
	return false
end


function This:setPalyerActInfo(key, value,fileName)
	local luaFileName = fileName or SingleManagerFileName
	local GamePlayer = require("Player.GamePlayer")
	local GameUtil = require("Util.GameUtil")
    local info = cc.FileUtils:loadLuaFile(luaFileName) or {}
    local id = GameUtil:int64ToString(GamePlayer:getPlayerID())
    local actinfo = info[id]
    if not actinfo then
    	actinfo = {}
    	info[id] = actinfo
    end
	actinfo[key] = value
    cc.FileUtils:saveToLua(info,luaFileName)
end

function This:getPalyerActInfo(key, defaultValue,fileName)
	local luaFileName = fileName or SingleManagerFileName
    local info = cc.FileUtils:loadLuaFile(luaFileName)
    if not info then
    	return defaultValue
    end
	local GamePlayer = require("Player.GamePlayer")
	local GameUtil = require("Util.GameUtil")
    local id = GameUtil:int64ToString(GamePlayer:getPlayerID())
    local actinfo = info[id]
    if not actinfo then
    	return defaultValue
    end

    local value = actinfo[key]
    if value ~= nil then
    	return value
    else
    	return defaultValue
    end
end

function This:resetBtnCallBack( )
	local chapter = self:getSelectChapter()
	local canReset,resetTime = chapter:getResetTiem()
	if canReset then
		local diamond = chapter:getResetCost()
			-- g_scene:openDialog("Stage.ChallengeConfirmPanel", {
   --      fileName = "MenuScene/NodeUI/ActivityHappyRewardVoucher.csb",
   --      cost = diamond,
   --      notips = "SingleManager",
   --      text1 = strList.cost,
   --      descText = strList.cost1, 
   --      okHandler = function(build)
   --          self:onStageResetRequest(chapter:getId())            
   --      end } )
		g_scene:openDialog("UI.CostConfirmPanel", {
        cost = diamond,
        desc = strList.cost1, 
        okHandler = function(build)
            self:onStageResetRequest(chapter:getId())            
        end } )
	else
		g_scene:showTips(strList.costLimit)
	end
end


function This:getChallengeTimes(chapterIndex)
	local chapter = self:getChapter(chapterIndex)
	if chapter then
		return chapter:getChallengeTimes(  ) 
	end
	return false,0
end

function This:setSelectChapter( SelectChapter )
	self.m_SelectChapter = SelectChapter
end

function This:getSelectChapterId( )
	if self.m_SelectChapter then
		return self.m_SelectChapter
	end
	return 5
end

function This:getSelectChapter()
	return self:getChapter(self.m_SelectChapter)
end

function This:getLastChapter()
	return self.m_lastChapter
end

function This:setLastChapter(chapter)
	self.m_lastChapter = chapter or self.m_chapter[1]
end

function This:getStage(id)
	return self.m_stage[id]
end

function This:getChapter(id)
	return self.m_chapter[id]
end

function This:getChapters()
	return self.m_chapter
end


function This:onBattleSkip(result)
	self:onBattleFailed()
end
----------------------------------------------------
--挑战结果
function This:onBattleVictory()
	g_scene:openDialog("SingleDaily.SingleStageEndPanel",{
		rewards = self.m_battleResult.reward_list,
		hurts = self.m_battleResult.damage_list,
	})
	self:clearFightData()
	BattleDataManager:removeObserver(self)
end

function This:onBattleFailed()
	self:onBattleVictory()
end

function This:enterStage(stageId)
	local stage = self:getStage(stageId)
	local emBattleHeros = stage:getEmBattle(  )
	if emBattleHeros and  next(emBattleHeros) then
		self:onChallengeStageRequest(stageId)
	else
		g_scene:showTips(Serialize["add_embattle_hero"])
	end
end

function This:onChallengeStageRequest(stageId)
	Socket:sendAndLock(12902,{id = stageId})--挑战
	self.m_curStageId = stageId
end
---------------------------------------------------------------
function This:onChallengeStateReply(msg)
	if msg.err_code == 0 then
		self:clearFightData()
		local left_times = msg.challenge_times or 0
		local daily_copy = msg.daily_copy
		local idList = msg.report_id_list
		for _, id in ipairs(idList) do
		    self.m_battles[id] = false
		end
		self.m_battleResult = msg
		self.m_battleResult.report_id_list = idList

		local chapter = self:getSelectChapter()
		chapter:setChallengeTimes(left_times)
		local stage = chapter:getSelectStage()
		stage:setData(daily_copy,true)
		self.m_battleResult.bgId =stage:getBgId()

		self:notifyObservers("onChallengeState")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onEmbattleHerosRequest2(stageId)

end

function This:onEmbattleHerosRequest(stageId,teamIndex)

	local stage = self:getStage(stageId)
	if stage then
		local allHeroList = stage:getEmBattle()
		if #allHeroList>0 then
			local function getHeroList( heros )
				local result = {0,0,0,0,0,0}
				for pos , info in pairs(heros) do
					local hero = HeroManager:getHero(info)
					result[pos] = hero:getUid()
				end
				return result
			end
			local res ={id = stageId}
			local teamCount = stage:getTeamCount()
			for index=1,3 do
				local temIndex = index
				local heros = stage:getTeam(index) or {}
				local herosCount = table.count(heros)
				local result ={}
				if herosCount>0 then
					result = getHeroList(heros)
				end
				res["card_list"..index] = result
			end
			Socket:sendAndLock(12901,res)--设置出战顺序
		end
	end
end

function This:onEmbattleHerosReply(message)
	if message.err_code == 0 then
	else
		g_scene:showTips(Serialize[message.err_code])
	end
end
	
function This:clearFightData()
    self.m_battleResult = {}
    self.m_battles = {}
    self.m_reports ={}
end

function This:startBattle()
    if not self.m_battles then
        return
    end

    local list = {}
    for index, id in pairs(self.m_battleResult.report_id_list) do
        list[index] = self.m_battles[id]
    end
    local params = { mode = PlayMode.kWarSingle, anonymous = true, bgId = self.m_battleResult.bgId, 
        msg = {
            report_list = list,
        },
    }
    BattleDataManager:addObserver(self)
    cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, params)
end

function This:loadBattle()
    local ret = false
    local battles = self.m_battles
    for id, battle in pairs(battles) do
        if not battle then
            battle = self:getReport(id)
            if battle then
                self.m_battles[id] = battle
            else
                ret = false
                break 
            end
        end
        ret = battle ~= nil
    end

    if ret then
        self:startBattle()
    end
    return ret
end

function This:getReport(reportId)
    return self.m_reports[reportId]
end

function This:setReport( reportId,reportInfo )
	self.m_reports[reportId] = reportInfo
end

function This:onWarReportReply( msg )
    	local reportInfo = msg.report
        local reportId = reportInfo.id
        self:setReport(reportId,reportInfo)
        self:loadBattle()
        self:notifyObservers("onGetWarReport")
end

function This:onStageResetRequest(op_type)
	Socket:sendAndLock(12905, {copy_type = op_type})--挑战次数重置--购买次数
end

function This:onStageResetReply( msg )
    if msg.err_code == 0 then
    	local daily_copy = msg.daily_copy or {}
    	local chapterId = daily_copy.copy_type
    	local chapter = self:getChapter(chapterId)
    	if chapter then
    		chapter:setData(daily_copy,true)
    	end
        self:notifyObservers("onStageReset")
        self:notifyObservers("onRefreshChallengeTime")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetStateInfoRequest()
	self:loadHeroList()
	Socket:sendAndLock(12900, {})--单人日常副本-信息(冲锋陷阵、决胜千里、运筹帷幄)
	local str =""
end

function This:onGetStateInfoNotify(msg)
	local daily_copy_list = msg.daily_copy_list or {}
	
	for k,item in pairs(daily_copy_list) do
		local dailyItem = self:getChapter(item.copy_type)
		if dailyItem then
			dailyItem:setData(item)
		end
	end
	self:notifyObservers("OnStateInfo", msg.award_all_list)
end
-----------------------------------------------------------------
