-- @Author: zheng
-- @Date:   2018-08-14 17:04:47
-- @File Name: SingleStagePanel.lua

local This , super = class(...,"Dialog")
local SingleManager = require("SingleDaily.SingleManager")

local ItemNode = require("UI.ItemNode")
local strList ={
    chapter =Serialize[64175],
    resetTime =Serialize[64181],
    upLimit= Serialize[64182],
    upPass= Serialize[64183],
}

local TabName = 
{
    ktep1 = 1,
    ktep2 = 2,
    ktep3 = 3,
    ktep4 = 4,
    ktep5 = 5,
    ktep6 = 6,
    kMax = 6,
}

function This:initNode(info)
	if info then
		if info.chapterId then
			SingleManager:setSelectChapter(info.chapterId)
		end
	end
	SingleManager:onGetStateInfoRequest()
	self:refreshStage(  )
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/SingleDailyStage.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self.m_inited = false
	for tabKey=1,TabName.kMax do
		local item = self.m_csNode.tep["tep"..tabKey]
		self:addTabButton(tabKey,item,function()
			local data = self.m_data
			local stage = self.m_data:getSelectStage(tabKey)
			if self.m_inited then
				local islock,lockAttr = stage:isLock()
				if not islock then
					local str = string.format(strList.upLimit,stage:getPlayerLevelLimit())
					if lockAttr == SingleStageLockType.isPass then
						local temstage = data:getMinSelectStage()
						local title = temstage:getTitle()
						str = string.format(strList.upPass,title)
					end
					g_scene:showTips(str)
					return false
				end
				self.m_data:setSelectIndex(tabKey)
				self:lightTabButton(tabKey)
				self:refreshStage(  )
			end
			self:refresh(  )
	    end)
	end
	self.m_data:initSelectIndex()
	self:selectTabButton(self.m_data:getSelectIndex( ))
	self:refreshLock()
	self:refresh(  )
	self.m_inited = true
end


function This:refreshTab( )
	local data = self.m_data
	local stage = self.m_stage

	local canChalenge, chanlengeTime = data:getChallengeTimes()
	local canReset,resetTime = data:getResetTiem()
	local strFormat = "<A5>"..strList.chapter.."：%d"
	if not canChalenge then
		strFormat = "<A5>"..strList.chapter.."：<New11>%d"
	end

	local csNode = self.m_csNode
	csNode.nandu:setString(stage:getTitle())
	csNode.yitongguan:setVisible(stage:isPassed())
	csNode.numberjindu:setString(string.format("%d%%",stage:getMaxProgress()))
	csNode.xiaoguo:setString(stage:getDes())
	csNode.hero:setString(stage:getRecommend())


	self.m_csNode.numbercishu:setStringWithColor(string.format(strFormat,chanlengeTime))
	self.m_csNode.jiahaodi1:setVisible(not canChalenge)


	csNode.name:setString(data:getName())
end


function This:onEnter()
	super.onEnter(self)
	SingleManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	SingleManager:removeObserver(self)
end

function This:refresh(  )
	self:refreshStage(  )
	self:refreshTab( )
	self:refreshReward(  )	
end

function This:refreshLock()
	local chapter = self.m_data
	local stages = chapter:getStages()
	local stageCount = #stages
	local csNode = self.m_csNode
	for i = 1 , stageCount do
		local stage = stages[i]
		local tep = csNode.tep["tep"..i]
		local islock,lockAttr = stage:isLock()
		tep.suo:setVisible(not islock)
	end
	local nextStage = stageCount+1
	if TabName.kMax > stageCount then
		for i = nextStage , TabName.kMax do
			local tep = csNode.tep["tep"..i]
			tep:setVisible(false)
		end
	end
end

function This:refreshReward(  )
	local data = self.m_data
	local stage = self.m_stage
	local reward = stage:getRewardList()
	self:refreshItemNode( self.m_csNode,reward )
end

function This:refreshItemNode( node,scrTable )
    local count = 1
    for i=1,5 do
        local item = node["item"..i]
        item:setVisible(false)
    end
    for kIndex,itemConfig in pairs(scrTable) do
        local item = node["item"..count]
        if type(itemConfig) == "number" then
        	item:setVisible(false)
        else
        	item:setVisible(true)
        	self:creatItemNode(item,itemConfig[1],itemConfig[2],count)
        end
        count = count+1
    end
end

function This:creatItemNode(parent,key,needNum,index)
    local name = "ItemNode"
    local node = parent:getChildByName(name)
    if not node then
        node = ItemNode:create(key, 1)
        node:setAnchorPoint(0.5, 0.5)
        parent:addChild(node)
        node:setName(name)
    end
    node:initItem(key,needNum) 
    local str =""
    local visible = false
    if index ==1 then
    	local ItemUtil = require("Bag.ItemUtil")
    	visible = true
    	str = string.format("0~%s",ItemUtil.formatItemCount(needNum))
    end
    node:setCustomCount(str,visible)
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function ()
		self:close()
	end)
	self.m_csNode.button:setTouchEnded(function ()--挑  战,扫荡
		local data = self.m_data
		local stage = self.m_stage
		
		local canChalenge, chanlengeTime = data:getChallengeTimes()
		if not canChalenge then
			g_scene:showTips(strList.resetTime)
			return
		end
		--挑  战
		g_scene:openDialog("SingleDaily.SingleStageFightPanel",{stageId = stageId})
	end)
	self.m_csNode.jiahaodi1:setTouchEnded(function ()
		SingleManager:resetBtnCallBack( )
	end)
	self.m_csNode.wenhao:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips", {text = Serialize["SingleDaily_SingleStage_1"],node = self.m_csNode.wenhao})
	end)
end

function This:refreshStage(  )
	self.m_data = SingleManager:getSelectChapter()
	self.m_stage = self.m_data:getSelectStage( )
end

function This:OnStateInfo()--获取挑战关卡信息
	self:refresh()
end

function This:onStageReset(  )--挑战次数重置
	self:refresh()
end
