-- @Author: zheng
-- @Date:   2018-12-19 16:27:26
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-03 15:06:53
-- @File Name: TowerMainItemView.lua

local This , super = class(...,"NodeView")
local TowerManager = require("SingleDaily.TowerManager")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")

function This:initData(info, index)
    if info then
       self.m_masterModel = info.masterModel
       self.m_btnChallengeCallBack = info.btnChallengeCallBack
    end
    self:getData()
    self.m_index = index

    local csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TowerMasterList.csb")
    self:addChild(csNode)
    self.m_csNode = csNode

    local w, h = csNode.bg:getContentSize()
    self:setContentSize(w, h)
    self.m_inited = false
end

function This:initNode(info, index)
    self:refresh()
end

function This:setModel( achieve, index )
    -- self:unscheduleUpdate()
    -- self.m_achieve:setView(nil)
    -- self.m_achieve = achieve
    -- self.m_achieve:setView(self)
    self:refresh(achieve)
end

function This:refresh(info)
    if info then
       self.m_masterModel = info
    end

    local csNode = self.m_csNode

    local data = self.m_data
    local levelNum = data:getLevelNum()
--    levelNum= 26
    local masterModel = self.m_masterModel

    local zhanLi = ""
    local isHaveMaster = masterModel:isHaveTower()
    if isHaveMaster then
        GameUtil:setPlayerIconNodeImage(csNode.icon,masterModel:getIcon())
        zhanLi = Serialize["SingleDaily_TowerMain_3"]..":"..masterModel:getPowerNum()
    end
    local emptyNode = csNode.emptyNode
    emptyNode.name:setString(masterModel:getRoleName())
    emptyNode.zhanlinum:setString(masterModel:getPowerNum())
    emptyNode:setVisible(isHaveMaster)
    csNode.weizhanling:setVisible(not isHaveMaster)
    csNode.bazhu:setString(string.format("%d%s",masterModel:getId(),Serialize["SingleDaily_MasterDetails_1_c"]))

    local rewards = masterModel:getReward()
    local itemIndex = 1
    for itemId,itemValue in pairs(rewards) do
        local jiangli = csNode["jiangli"..itemIndex]
        if jiangli then
            jiangli:setImage(ItemUtil.getIconImage(itemId))
            jiangli.text:setString(itemValue)
        end
        itemIndex = itemIndex + 1
    end

    local tiaojiantextVisble = false
    local zhenRongVisble = false
    local buzhenVisble = false
    local zhengduoVisble = true
    if not (levelNum>masterModel:getLevelLimit()) then
        tiaojiantextVisble = true
        csNode.tiaojiantext:setString(string.format(Serialize["SingleDaily_MainItem_1"],masterModel:getLevelLimit()))
        zhengduoVisble = false
    end
    if isHaveMaster then
        -- if levelNum<masterModel:getLevelLimit() then
        if masterModel:isMyHeroList() then
            buzhenVisble = true
            zhengduoVisble = false
            tiaojiantextVisble= false
        elseif not tiaojiantextVisble then
            
            zhenRongVisble = true
            zhengduoVisble = true
        end
    end
    csNode.zhengduo:setVisible(zhengduoVisble)
    csNode.buzhen:setVisible(buzhenVisble)
    csNode.zhenRong:setVisible(zhenRongVisble)
    csNode.tiaojiantext:setVisible(tiaojiantextVisble)
    -- csNode.zhenRong:setVisible(not zhenRongVisble)
    csNode.zherongxiangqing:setVisible(zhenRongVisble)
end

function This:initTouchEvent()
    print("index:" .. self.m_index)

    if not self.m_inited then
        local csNode = self.m_csNode
        --阵容详情
        csNode.zhenRong:setTouch({
            isContainer = true,
            onEnded = function()
                local masterModel = self.m_masterModel
                if masterModel:isHaveTower() then
                    local masterModel = self.m_masterModel
                    local id = masterModel:getId()
                    g_scene:openDialog("SingleDaily.TowerMasterDetailsPanel",{masterId = id})
                else
                    g_scene:showTips(Serialize["SingleDaily_MainItem_2"])
                end
            end,
            isScale = false, 
            isDynamic = true, 
        })
        --争  夺
        csNode.zhengduo:setTouch({
            isContainer = true,
            onEnded = function()
                if self.m_btnChallengeCallBack then
                    local isCanChalleng,challengeStr = self.m_btnChallengeCallBack()
                    if isCanChalleng then
                        local data = self.m_data
                        local masterModel = self.m_masterModel
                        local id = masterModel:getId()
                        local info = {
                            initHeros = {},
                            stageId = 1,
                            targetStr = csNode.bazhu:getString(),
                            allChoseHero = data:getMasterAllHeroList(),
                            saveHeroCallBack = function (saveHeroList)
                                TowerManager:onChallengeHeroRequest(id,saveHeroList)
                            end
                        }
                        g_scene:openDialog("SingleDaily.TowerEmbattleHeroPanel", info)
                    else
                        g_scene:showTips(challengeStr)
                    end
                end
            end,
            isScale = false, 
            isDynamic = true, 
        })

        --布  阵
        csNode.buzhen:setTouch({
            isScale = false, 
            isDynamic = true, 
            isContainer = true,
            onEnded = function()
                local data = self.m_data
                local masterModel = self.m_masterModel
                local id = masterModel:getId()

                local info = {initHeros= masterModel:getAllHeroIds(),targetStr = csNode.bazhu:getString(),
                allChoseHero = data:getMasterAllHeroList(),stageId =1,
                saveHeroCallBack = function (saveHeroList)
                    TowerManager:onProtectHeroListRequest(id,saveHeroList)
                end}
                g_scene:openDialog("SingleDaily.TowerEmbattleHeroPanel",info)
            end
        })
        self.m_inited = true
    end
end

function This:onEnter()
    super.onEnter(self)
    TowerManager:addObserver(self)
end

function This:onExit()
    TowerManager:removeObserver(self)
    super.onExit(self)
end

function This:getData()
    self.m_data = TowerManager:getTowerData()
end