-- @Author: zheng
-- @Date:   2018-12-18 20:12:35
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-03 15:18:56
-- @File Name: TowerMainPanel.lua

local This , super = class(...,"Dialog")
local TowerManager = require("SingleDaily.TowerManager")
local TowerMainItemView = require("SingleDaily.TowerMainItemView")

local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")

This.fileName = "MenuScene/NodeUI/TowerMain.csb"

local RankPageListLayer = require("Rank.RankPageListLayer")

function This:initNode(info)

    TowerManager:onGetMasterInfoRequest()
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)


    self.m_isCanChallenge = false--是否可以挑战
    self.m_isChallengeStr = ""--不可以挑战时的提示
    self.m_escape = 0
    self:getData()
    
    local scrollview = self.m_csNode.scrollview1
    scrollview:setDirection("horizontal")
    scrollview.m_direction = "right"
    self:initListLayer()
end

function This:initListLayer()
    self.m_listLayer = RankPageListLayer:create({
        itemWidth = 582,
        itemHeight = 100,
        spaceX = 0,
        spaceY = 2,
        hintGetter = function ()
            return #self.m_masterList
        end,
        scrollView = self.m_csNode.scrollView2,
        dataCounter = function ()
            return #self.m_masterList
        end,
        viewCreator =  function(index, node)
            local model = self.m_masterList[index]
            if model then
                if not node then
                    return self:createItemView(model, index)
                else
                    node:setModel(model, index)
                end
            end
        end,
    })

    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.m_csNode.scrollView2:pushBackCustomItem(self.m_listLayer, w, h,0)
end

function This:update(dt)
   super.update(self,dt)
   self.m_escape = self.m_escape + dt
   if self.m_escape < 1 then
       return
   end
   self.m_escape = self.m_escape - 1
   self:refreshTime()
end

function This:refreshTime()
    local cdNode = self.m_csNode.cdNode
    local data = self.m_data
    local coolingTime = data:getCoolingTime()
    if coolingTime>0 then
        coolingTime = coolingTime -1
        data:setCoolingTime(coolingTime)
        cdNode.CD:setString(GameUtil:formatTime(Serialize["M_S"],coolingTime))
    end
    cdNode:setVisible(coolingTime>0)
end

function This:refresh()
    self:refreshTime()
    self:refreshBaseInfo()
    self:refreshScroolView2()
end

function This:initTouchEvent()
 --   item1:setTouchEnded(function()
	-- 	self:btnCallBack(btnType)
	-- end,item1.bg)
	
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    self.m_csNode.rizhi:setTouchEnded(function()--日志
        g_scene:openDialog("SingleDaily.TowerMasterLogPanel")
    end)
    self.m_csNode.guize:setTouchEnded(function()--规则
        g_scene:openDialog("UI.HelpTips", {text = Serialize["SingleDaily_TowerMain_4"],node = self.m_csNode.guize})
    end)
	self.m_csNode.cdNode.qingchu:setTouchEnded(function()--规则
        local data = self.m_data
        local cost = data:getTowerConfigNum(SingleTowerLimitType.kMasterTime)[1]
            g_scene:openDialog("UI.CostConfirmPanel", {
        cost = cost[2],
        costId = cost[1],
        notips = "TowerMainPanel",
        -- text1 = "是否花费",
        desc = Serialize["SingleDaily_TowerMain_7"], 
        okHandler = function(build)
                local coolingTime = data:getCoolingTime()
                if coolingTime>0 then
                    TowerManager:onGetRefreshCoolingTimeRequest()
                end
        end } )
    end)
end

function This:onEnter()
    super.onEnter(self)
    TowerManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    TowerManager:removeObserver(self)
end

function This:refreshBaseInfo()
    local data = self.m_data
    local myMasterlist,maxMasterId = data:getMyMasterInforAndMaxMasterId()
    local masterOpenList = data:getTowerConfigNum(SingleTowerLimitType.kListOpen)
    local levelNum = data:getLevelNum()
    local levelModel = data:getLevel(levelNum)
    table.sort(myMasterlist, function( a, b )
        local state1, state2 = a:getId() , b:getId()
        return state1<state2
    end)

    local queueNum =0
    local rewards = {}

    local csNode = self.m_csNode
    for openIndex,openValue in ipairs(masterOpenList) do
        local troopBtn = csNode["troopBtn"..openIndex]
        if troopBtn then
            local str = "<New13>"..Serialize["SingleDaily_TowerMain_1"]
            if levelNum<openValue then
                str = string.format("<New8>"..Serialize["SingleDaily_TowerMain_2"],openValue)
           end
                local masterItem = myMasterlist[openIndex]
                if masterItem then
                    str = string.format("<New7>%d"..Serialize["SingleDaily_MasterDetails_1_c"],masterItem:getId())
                    rewards = GameUtil:tableAdd(masterItem:getReward(),rewards)
                end
            -- 判断应该有几个队列
            if levelNum>openValue or (levelNum==openValue and levelModel:IsHaveBoxReward()) then
                queueNum = queueNum + 1
            end
            troopBtn.txt:setStringWithColor(str)
           if openIndex >2 then
                troopBtn:setVisible(queueNum>=2)
           end 
        end
    end
    self.m_isCanChallenge = queueNum > #myMasterlist
    if queueNum== #masterOpenList then
        -- self.m_isChallengeStr = Serialize["SingleDaily_TowerMain_6"]
    self.m_isCanChallenge = true
    elseif queueNum == #myMasterlist then
        local temQueueNum = queueNum+1
        self.m_isChallengeStr = string.format(Serialize["SingleDaily_TowerMain_9"],masterOpenList[temQueueNum],temQueueNum)
    end
    self:refreshScroolView1(rewards,#myMasterlist>0)
    
    local maxMasterItem = data:getMaster(maxMasterId)
    local maxMasterName = ""
    if maxMasterItem then
        maxMasterName = maxMasterItem:getRoleName()
    end
    csNode.zhizun.name:setString(maxMasterName)
end

function This:refreshScroolView1(rewards,scroolViewVisble)
    local scrollview = self.m_csNode.scrollview1
    scrollview:removeAllItems()
    if not rewards or not scroolViewVisble then
        return 
    end

    local function createItemView(model)
        local node = ItemNode:create(model[1],model[2])
        node:setAnchorPoint(0.5,0.5)
        return node
    end

    for itemId,itemValue in pairs(rewards) do
        local item = createItemView({itemId,itemValue})
        local scale = 0.9
        item:setScale(scale)
        local w , h = item:getContentSize()
        scrollview:pushBackCustomItem(item,w*scale,h,0.5)
    end
end

function This:refreshScroolView2()
    if not self.m_masterList then
        local data = self.m_data
        local masterList = data:getMasterlist(true)

        table.sort(masterList, function( a, b )
            local state1, state2 = a:getId() , b:getId()
            return state1>state2
        end)
        self.m_masterList = masterList
    end
    self.m_listLayer:setDataList()
end

function This:createItemView( data, index )
    local info = {
        masterModel = data,
        btnChallengeCallBack = function ()
            local coolingTime = self.m_data:getCoolingTime()
            if coolingTime>0 then
                return false,Serialize[13701]
            end
            return self.m_isCanChallenge, self.m_isChallengeStr
        end
    }
    return TowerMainItemView:create(info, index)
end

function This:onGetMasterInfo()
    self:refresh()
end

function This:onGetRefreshCoolingTime()
    self:refreshTime()
end

function This:getData()
    self.m_data = TowerManager:getTowerData()
end