-- @Author: zheng
-- @Date:   2018-12-17 17:19:58
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-29 10:01:26
-- @File Name: TowerManager.lua

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")


local GameTowerData = require("SingleDaily.GameTowerData")

local MenuScene = require("Home.MenuScene")
local BattleDataManager = require("Battle.BattleDataManager")
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")
local GamePlayer = require("Player.GamePlayer")

local SingleManager = require("SingleDaily.SingleManager")
ToweManagerLevelHeroFileName = "ToweManagerLevelHeroFileName"

function This:init()
    super.init(self)
    self.m_towerData = GameTowerData:create()
    self:initListener()
    self:clearChallengeData()
end

function This:getTowerData()
    return self.m_towerData
end

function This:initListener()
    EventDispatcher:addListener(self, 13800, self.onGetMasterInfoReply)--获取塔主相关信息
    EventDispatcher:addListener(self, 13801, self.onChallengeHeroReply)--挑战
    EventDispatcher:addListener(self, 13802, self.onProtectHeroListReply)--设置防守阵容
    EventDispatcher:addListener(self, 13803, self.onGetRefreshCoolingTimeReply)--清空塔主争夺CD
    EventDispatcher:addListener(self, 13900, self.onGetTowerInfoReply)--镇河塔关卡信息
    EventDispatcher:addListener(self, 13901, self.onGetLevelChallengeReply)--关卡直接挑战
    EventDispatcher:addListener(self, 13902, self.onGetDayRewardReply)--领取今日奖励
    EventDispatcher:addListener(self, 13903, self.onGetPassLevelRewardReply)--挑战关卡通过后的奖励
    EventDispatcher:addListener(self, 13904, self.onGetLevelRewardReply)--领取通关额外的宝箱
end

function This:removeListener()
    EventDispatcher:removeListener(self,13800)--获取塔主相关信息
    EventDispatcher:removeListener(self,13801)--挑战
    EventDispatcher:removeListener(self,13802)--设置防守阵容
    EventDispatcher:removeListener(self,13803)--清空塔主争夺CD
    EventDispatcher:removeListener(self,13900)--镇河塔关卡信息
    EventDispatcher:removeListener(self,13901)--关卡直接挑战
    EventDispatcher:removeListener(self,13902)--领取今日奖励
    EventDispatcher:removeListener(self,13903)--挑战关卡通过后的奖励
    EventDispatcher:removeListener(self,13904)--领取通关额外的宝箱
end

function This:isHaveYesterdayReward()
    local data = self:getTowerData()
    local rewardData = data:getDayReward()
    local yesterdayLevel = data:getYesterdayLevel()
    if yesterdayLevel~=0 and rewardData:isCanReward() then
        return true
    end
    return false
end

--设置组队阵容
function This:getLevelTeamHeroList()
    return self.m_teamHeroList or {}--{heroId1,heroId2}
end

function This:setLevelTeamHeroList(heroList)
    if heroList then
        self.m_teamHeroList = heroList
    end
end

function This:getLevelHeroList()
    return SingleManager:getPalyerActInfo("LevelHeroList",{},ToweManagerLevelHeroFileName)
end

function This:setLevelHeroList(heroList)
    if heroList then
        SingleManager:setPalyerActInfo("LevelHeroList", heroList,ToweManagerLevelHeroFileName)
    end
end

function This:clearChallengeData()
    self.m_chalengeType = nil
    -- self.m_challengeResult = {reward ={},isMyResult = false,isWin = false,starLv = 0}
    self.m_challengeResult = {reward ={},isWin = false,starLv = 0}
end

function This:setChallengeState(chalengeState,chalengeType)
    -- self.m_chalengeState = chalengeState
    self.m_chalengeType = chalengeType
    self.m_challengeResult.isWin = chalengeState
    BattleDataManager:addObserver(self)
end

function This:onBattleSkip(result)
    if self.m_challengeResult.isWin then
        self:onBattleVictory()
    else--if foo then
        self:onBattleFailed()
    end
end
----------------------------------------------------------
--挑战结果
function This:onBattleVictory(teamModel)
    if teamModel then
        self.m_chalengeType = SingleTowerChallengeType.kLevelType
    end
    if self.m_chalengeType then
        local function levelCallBack( )
            cc.SceneManager:switchTo("Home.MenuScene")
            MenuScene:addDelayDialog({name ="SingleDaily.TowerPanel" })
        end
        local function towerCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
           MenuScene:addDelayDialog({name ="SingleDaily.TowerPanel", arg = {towerChllengeCallBack = true}}) 
        end
        local function helpCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
        end

        local info = {dropInfo = self.m_challengeResult.reward,touchMask = levelCallBack,
            isRobber= true}
        if self.m_chalengeType== SingleTowerChallengeType.kMasterType then
            info.touchMask = towerCallBack
        elseif teamModel then
             local role = teamModel:getMainRole()
            if not role:isRoomOwner() then
                info.touchMask = helpCallBack
            end
        end
        g_scene:openDialog("SingleDaily.TowerChallengeSucessPanel",info)
    end
    self:clearChallengeData()
    if not teamModel then
        BattleDataManager:removeObserver(self)
    end
end

function This:onBattleFailed(teamModel)
    
    if teamModel then
        self.m_chalengeType = SingleTowerChallengeType.kLevelType
    end
    if self.m_chalengeType then
        local function levelCallBack( )
            cc.SceneManager:switchTo("Home.MenuScene")
            MenuScene:addDelayDialog({name ="SingleDaily.TowerPanel" })
        end
        local function towerCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
           MenuScene:addDelayDialog({name ="SingleDaily.TowerPanel", arg = {towerChllengeCallBack = true}}) 
        end
        local function helpCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
        end

        local info = {dropInfo = self.m_challengeResult.reward,touchMask = levelCallBack}
        if self.m_chalengeType== SingleTowerChallengeType.kMasterType then
            info.touchMask = towerCallBack
        elseif teamModel then
            local role = teamModel:getMainRole()
            if not role:isRoomOwner() then
                info.touchMask = helpCallBack
            end
            -- local towerData = self:getTowerData()
            -- if towerData:isNotHaveTeam() and towerData:myNotTeamLeader() then --自己有队伍，自己不是队长返回true
            -- end
        end
        g_scene:openDialog("SingleDaily.TowerChallengeFailPanel",info)
    end
    self:clearChallengeData()
    if not teamModel then
        BattleDataManager:removeObserver(self)
    end
end

function This:onGetLevelRewardRequest()
    Socket:sendAndLock(13904)--领取通关额外的宝箱
end

function This:onGetLevelRewardReply( msg )
    if msg.err_code == 0 then
        local towerData = self:getTowerData()
        local oldLevelNum = msg.stage_id
        local levelNum = msg.new_stage_id

        local levelModel = towerData:getLevel(oldLevelNum)
        local rewardList = levelModel:getPassReward()
        g_scene:showReward(rewardList)
        towerData:setLevelNum(levelNum)
        self:notifyObservers("onGetLevelReward")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
-- function This:onGetPassLevelRewardRequest()
--     Socket:sendAndLock(13903)--挑战关卡通过后的奖励
-- end

function This:onGetPassLevelRewardReply( msg )
--    if msg.err_code == 0 then
        local towerData = self:getTowerData()
        
        local roleId = msg.role_id or 0
        self.m_challengeResult.reward = msg.reward_list or {}
        if not towerData:myNotTeamLeader() then
            towerData:setLevelNum(msg.new_stage_id or 1)
        end
        -- self.m_challengeResult.isMyResult = GamePlayer:getPlayerID() == roleId
        self:notifyObservers("onGetPassLevelReward")
--    else
--        g_scene:handleErrorCode(msg.err_code)
--    end
end

function This:onGetDayRewardRequest(rewardType)
    Socket:sendAndLock(13902,{reward_idx = rewardType})--领取今日奖励
end

function This:onGetDayRewardReply( msg )
    if msg.err_code == 0 then
        local towerData = self:getTowerData()
        -- print(msg)
        local rewardType = msg.reward_idx
        local dayReward = towerData:getDayReward()
        dayReward:setIsReceiver(rewardType)
        local rewardList = dayReward:getReward()
        if rewardType == SingleTowerDayRewardState.kMoneyReward then
            rewardList = dayReward:getMoneyReward()
        end
        g_scene:showReward(rewardList)
        self:notifyObservers("onGetDayReward")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetLevelChallengeRequest(heroList)
    local cardList = self:getHeroList(heroList)
    Socket:sendAndLock(13901,{card_list= cardList})--关卡直接挑战
end
function This:onGetLevelChallengeReply( msg )
    if msg.err_code == 0 then
        self:clearChallengeData()
        local towerData = self:getTowerData()
        local isWin = msg.is_win or false
        self.m_challengeResult.isWin = isWin
        -- self:setChallengeState(msself:setChallengeState(msg.is_win,SingleTowerChallengeType.kLevelType)g.is_win,SingleTowerChallengeType.kLevelType)
        self:setChallengeState(msg.is_win,SingleTowerChallengeType.kLevelType)
        cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {bgId = 2,msg = msg})
        self:notifyObservers("onGetLevelChallenge")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetTowerInfoRequest()
    Socket:sendAndLock(13900)--镇河塔关卡信息
end

function This:onGetTowerInfoReply( msg )
    if msg.err_code == 0 then
        local levelNum = msg.stage_id
        local temId = msg.team_id
        local yesterdayLevel = msg.prev_stage_id
        local boxState = msg.reward_idx
        local isHaveBox = msg.is_box
        local leftTeamCount = msg.add_team_count
        
        local towerData = self:getTowerData()
        towerData:setLevelNum(levelNum)
        local levelModel = towerData:getLevel(levelNum)
        if levelModel then
            towerData:setTeamId(temId)
            levelModel:setIsHaveBoxReward(isHaveBox)
        end
        towerData:setYesterdayLevel(yesterdayLevel)
        local dayReward = towerData:getDayReward()
        dayReward:init(yesterdayLevel)
        dayReward:setIsReceiver(boxState)
        towerData:setLeftTeamCount(leftTeamCount)
        self:notifyObservers("onGetTowerInfo")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:getHeroList( heros )
    local result = {0,0,0,0,0,0}
    for pos , info in pairs(heros) do
        local hero = HeroManager:getHero(info)
        result[pos] = hero:getUid()
    end
    return result
end

function This:onGetRefreshCoolingTimeRequest()
    Socket:sendAndLock(13803)--清空塔主争夺CD
end

function This:onGetRefreshCoolingTimeReply( msg )
    if msg.err_code == 0 then
        local towerData = self:getTowerData()
        towerData:setCoolingTime(msg.cd_time or 0)
        self:notifyObservers("onGetRefreshCoolingTime")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onProtectHeroListRequest(level,heroList)
    local cardList = self:getHeroList(heroList)
    Socket:sendAndLock(13802,{level = level,card_list= cardList})--设置防守阵容
end

function This:onProtectHeroListReply( msg )
    if msg.err_code == 0 then
        local towerData = self:getTowerData()
        local masterData = towerData:getMaster(msg.level)
        if masterData then
            masterData:setHeroList(msg.card_list)
            masterData:setPowerNum(msg.power)
        end
        self:notifyObservers("onProtectHeroList")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onChallengeHeroRequest(level,heroList)
    local cardList = self:getHeroList(heroList)
    Socket:sendAndLock(13801,{level = level,card_list= cardList})--挑战
end

function This:onChallengeHeroReply( msg )
    if msg.err_code == 0 then
        self:clearChallengeData()
        local towerData = self:getTowerData()
        local report = msg.report
        local state = report.state
        local new_tower_master = msg.new_tower_master
        local new_master_log = msg.new_master_log
        local new_role_log_list = msg.new_role_log_list
        local cd_time = msg.cd_time
        local masterId = msg.level
        if new_master_log.level == masterId then --霸主替换了
            -- local masterModel = towerData:getMaster(masterId)
            -- masterModel:updateInfor(new_tower_master)
            towerData:updateMasterList(msg.tower_master_list)
            towerData:setMasterlog(new_master_log)
        end
        towerData:setRoleLog(new_role_log_list or {})
        towerData:setCoolingTime(cd_time or 0)
        if state ==0 then--没有霸主挑战胜利
            self:notifyObservers("onGetMasterInfo")
        else
            self.m_challengeResult.isWin = state==BattleResult.kAttackerVictory
            self:setChallengeState(self.m_challengeResult.isWin,SingleTowerChallengeType.kMasterType)
			cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {bgId = 2,msg = msg})
        end
        self:notifyObservers("onGetMasterInfo")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetMasterInfoRequest()
    Socket:sendAndLock(13800)--获取塔主相关信息
end

function This:onGetMasterInfoReply( msg )
    if msg then
        local towerData = self:getTowerData()
        towerData:updateMasterList(msg.tower_master_list or {})
        towerData:setMasterlog(msg.master_log_list or {})
        towerData:setRoleLog(msg.role_log_list or {})
        towerData:setCoolingTime(msg.cd_time or 0)
        self:notifyObservers("onGetMasterInfo")
    else
        g_scene:handleErrorCode("")
    end
end