-- @Author: zheng
-- @Date:   2018-12-18 15:16:43
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-12-25 13:46:15
-- @File Name: TowerMasterDetailsPanel.lua

local This , super = class(...,"Dialog")
local TowerManager = require("SingleDaily.TowerManager")
local GameUtil = require("Util.GameUtil")

local ArenaHeroView = require("Arena.ArenaHeroView")
local HeroManager = require("Hero.HeroManager")

This.fileName = "MenuScene/NodeUI/TowerFormationDetails.csb"

function This:initNode(info)
    if info then
       self.m_masterId = info.masterId
    end
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self.m_cardNodes = {}
    for heroIndex=1,6 do
        local view = ArenaHeroView:create(nil)
        local item = self.m_csNode["hero"..heroIndex]
        item:addChild(view)
        self.m_cardNodes[heroIndex] = view
    end
    self:getData()
    self:refresh()
end

function This:refresh()
    self:refreshBaseInfo()
    self:refreshHeroList()
end

function This:refreshBaseInfo()
    local data = self.m_data
    local masterData = data:getMaster(self.m_masterId)
    if masterData then
        local csNode = self.m_csNode
        GameUtil:setPlayerIconNodeImage(csNode.pic.icon,masterData:getIcon())
        csNode.level:setString(masterData:getRoleLevel())
        csNode.name:setString(masterData:getRoleName())
        csNode.rank:setString(string.format("%s%s",string.numberToWord(self.m_masterId),Serialize["SingleDaily_MasterDetails_1_c"]))
        csNode.fightnum:setString(string.format("%s:%d",Serialize["SingleDaily_MasterDetails_2"],masterData:getPowerNum()))
        csNode.family:setString(string.format("%s:%s",Serialize["SingleDaily_MasterDetails_3"],masterData:getGroupName()))
    end
end

function This:refreshHeroList()
    local data = self.m_data
    local masterData = data:getMaster(self.m_masterId)
    if masterData then
        local cardList = masterData:getHeroList()
        for i = 1 , 6 do
            if #cardList >= i then
                local cardInfor = cardList[i]
                local heroUid = cardInfor.card_id
                local heroId = cardInfor.card_type
                local info = {id = heroUid, level = cardInfor.level, star = cardInfor.star, heroId = heroId,awake = cardInfor.awake}
                self.m_cardNodes[i]:setModel(info)
            else
                self.m_cardNodes[i]:setModel(nil)
            end
        end
    end 
end

function This:initTouchEvent()
 --   item1:setTouchEnded(function()
	-- 	self:btnCallBack(btnType)
	-- end,item1.bg)
	
	self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
end

function This:getData()
	self.m_data = TowerManager:getTowerData()
end