-- @Author: zheng
-- @Date:   2018-12-20 16:21:11
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-02-20 14:52:38
-- @File Name: TowerPanel.lua



local This , super = class(...,"Dialog")
local TowerManager = require("SingleDaily.TowerManager")
local TowerItemView = require("SingleDaily.TowerItemView")

local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")

local WarfareManager = require("Warfare.WarfareManager")
local TeamWarRoleView = require("Warfare.TeamWarRoleView")

This.fileName = "MenuScene/NodeUI/TowerMaster.csb"

function This:initNode(info)
    self:getData()
     TowerManager:onGetMasterInfoRequest()
     if info then
         local stageId = info.stageId
         if stageId then
             local data = self.m_data
             data:setLevelNum(stageId)
             data:setYesterdayLevel(0)
             local dayReward = data:getDayReward()
             dayReward:setIsReceiver(0)
         else
             TowerManager:onGetTowerInfoRequest()
         end
         --是否是争夺霸主结束回调
         local towerChllengeCallBack = info.towerChllengeCallBack or false
         if towerChllengeCallBack then
             self:addOpenCallBack(function ()
                 g_scene:openDialog("SingleDaily.TowerMainPanel")
             end)
         end
	 else
	 	TowerManager:onGetTowerInfoRequest()
     end
     self.m_csNode = cc.Node:createFromCSFile(This.fileName)
     self.m_csNode:setToCenter()
     self:addChild(self.m_csNode)
     local scrollview = self.m_csNode.team.scrollview1
     scrollview:setDirection("horizontal")
     scrollview.m_direction = "right"
     self.m_maxRoleCount = 2
    self:refresh()
end


function This:refresh()
    self:refreshBaseInfo()
    self:refreshTeamInfo()
    self:refreshLevels()
end

local function isHaveHeroList()
    local heroList = TowerManager:getLevelHeroList()
    if heroList then
        return table.count(heroList) > 0
    end
end

function This:initTouchEvent()
    local data = self.m_data
    local csNode = self.m_csNode
    local teamNode = csNode.team
    local teamType = TeamType.kTowerTeam

    local function showExitTeamTip()
        if not WarfareManager:hasTeam(teamType) then
            g_scene:openDialog("SingleDaily.TowerMainPanel")
        else
            g_scene:openDialog("UI.CostConfirmPanel", {
                notips = "ChallengeStageManager",
                simple = Serialize["SingleDaily_TowerMain_5"], 
                okHandler = function()
                    WarfareManager:onExitWarTeamRequest(nil, teamType)
                    g_scene:openDialog("SingleDaily.TowerMainPanel")
                end 
            })
        end
    end
    
    csNode.closebutton:setTouchEnded(function()
        if WarfareManager:hasTeam(teamType) then
            self:close()
            return
        end
        self:close()
    end)

    teamNode.zhijiiekaiishi:setTouchEnded(function()--直接开始
        if isHaveHeroList() then
            local heroList = TowerManager:getLevelHeroList()
            TowerManager:onGetLevelChallengeRequest(heroList)
        else
            g_scene:showTips(Serialize["SingleDaily_TowerMain_8"])
        end
    end)

    teamNode.buzhen:setTouchEnded(function() --布  阵
        local levelNum = data:getLevelNum()
        self:openDialog("Territory.TerrBattleHeroPanel", {
            targetType = TerrBattleTarget.kTower,
            heroSource = TowerManager:getLevelHeroList(),
            attacker = tostring(levelNum), 
            heroList = allHeroList,
            battleHandler = function (heros)
                local saveHeroList = {}
                for _, info in ipairs(heros) do
                    saveHeroList[info.pos] = info.heroId
                end
                TowerManager:setLevelHeroList(saveHeroList)
                if WarfareManager:hasTeam(teamType) then
                    WarfareManager:onWarEmbattleHerosRequest(heros, teamType)
                end
                return true
            end
        })
    end)

    teamNode.zhunbei:setTouchEnded(function()--准  备
        if not WarfareManager:hasTeam(teamType) then
            return
        end

        if not WarfareManager:hadEmbattled(teamType) then
            g_scene:showTips(Serialize[64195])
            teamNode.buzhen.m_touch:onEnded()
            return
        end
        
        if not WarfareManager:hasTeam(teamType) then return end
        local team = WarfareManager:getMyTeam(teamType)
        local mainRole = team:getMainRole()
        if not mainRole:isRoomOwner() then
            local value = mainRole:isReady()
            WarfareManager:onPrepareWarRequest(not value, teamType)
        else
            local ret, errCode = team:canChallenge()
            if not ret then
                g_scene:showTips(errCode)
                return
            end
            WarfareManager:onWarfareBattleRequest(self.m_data:getLevelNum(), nil, nil, teamType)
        end
    end)

    teamNode.gou.weigouxuan:setTouchEnded(function()
        local isAutoStar = self:setAutoStarState(nil,true)
        local curentSelect = not isAutoStar
        self:setAutoStarState(curentSelect)
        local team = WarfareManager:getMyTeam(teamType)
        WarfareManager:onSetTeamAttrRequest(team:getId(), TeamOperateType.kAutoStarGame, curentSelect)
    end)

    csNode.tazhu.baZhu:setTouchEnded(function()--争夺
        local data = self.m_data
        if data:myNotTeamLeader() then
            return false
        end
        local levelNum = data:getLevelNum()
        local opentLevel = data:getTowerConfigNum(SingleTowerLimitType.kMasterOpen)
        local levelModel = data:getLevel(levelNum)
        if levelNum>opentLevel or (levelNum ==opentLevel and levelModel:IsHaveBoxReward()) then
            showExitTeamTip()
        else
            g_scene:showTips(string.format(Serialize["SingleDaily_TowerMain_2"],opentLevel))
        end
    end)

    csNode.tazhu.touxiangkuang_66:setTouchEnded(function()--争夺2
        csNode.tazhu.baZhu.m_touch:onEnded()
    end)

    csNode.box4:setTouchEnded(function()--Day奖励
        local data = self.m_data
        if data:myNotTeamLeader() then
            return false
        end
        g_scene:openDialog("SingleDaily.TowerDayRewardPanel")
    end)
    self.m_csNode.wenhaoxin_7:setTouchEnded(function ()
        g_scene:openDialog("UI.HelpTips", {text = Serialize["SingleDaily_TowerMain_10"],node = self.m_csNode.wenhaoxin_7})
    end)
end

function This:setAutoStarState(isVisible,isGet)
    local node = self.m_csNode.team.gou.gouxuan
    if isGet then
        return node:isVisible()
    else
        node:setVisible(isVisible)
    end
end

function This:onEnter()
    super.onEnter(self)
    TowerManager:addObserver(self)
    WarfareManager:addObserver(self)
end

function This:onExit()
    WarfareManager:removeObserver(self)
    TowerManager:removeObserver(self)
    super.onExit(self)
end

function This:refreshBaseInfo()
    local data = self.m_data
    local chapterId = data:getCurentChapter()
    local masterModel= data:getMaster(chapterId)

    local tazhu = self.m_csNode.tazhu
    local teamNode = self.m_csNode.team
    if not masterModel:isMyHeroList() then
        GameUtil:setPlayerIconNodeImage(tazhu.icon,masterModel:getIcon())
    end
    local name = "<New13>".. masterModel:getRoleName()
    if not masterModel:isHaveTower() then
        name = "<New13>".. Serialize["Official_AwardPane_4_c"]
    end
    tazhu.name:setStringWithColor(name)

    local cengFormat = "MenuScene/ceng%d.png"
    local cengImage = self.m_csNode.ceng
    -- chapterId= 11
    if chapterId<=10 and chapterId>0 then
        cengImage:setImage(string.format(cengFormat,chapterId))
    elseif chapterId>10 and chapterId<100 then
        cengImage:setVisible(false)
        local function getNum(num)
            local number2 = num % 10
            local number1 = math.floor(num / 10)
            if number1 == 1 then
                number1 =10
            end
            if number2 ==0 then
                number2 = 10
            end
            return number1,number2
        end
        local number1,number2 = getNum(chapterId)
        local cengNode = self.m_csNode.cengshu
        cengNode:setVisible(true)
        cengNode.ceng1:setImage(string.format(cengFormat,number1))
        cengNode.ceng2:setImage(string.format(cengFormat,number2))
    end

    local levelNum = data:getLevelNum(true)
    local levelModel = data:getLevel(levelNum)
    local isNotHaveTeam = data:isNotHaveTeam()
    local reward = levelModel:getReward()
    local xiezhucishu = teamNode.xiezhucishu
    local xiezhucishuVisible = false
    local str = Serialize["SingleDaily_TowerPanel_2"]..":"
    if not isNotHaveTeam and data:myNotTeamLeader() then
        str = Serialize["SingleDaily_TowerPanel_3"]..":"
        reward = data:getHelpReward()
        xiezhucishu:setString(Serialize["SingleDaily_TowerPanel_4"]..data:getLeftTeamCount())
        xiezhucishuVisible = true
    end
    xiezhucishu:setVisible(xiezhucishuVisible)

    local function createItemView(model)
        local node = ItemNode:create(model[1],model[2])
        node:setAnchorPoint(0.5,0.8)
        return node
    end
    local function refreshLevelReward(rewards,rewardStr)
        -- local team = self.m_csNode.team
        local scrollview = teamNode.scrollview1
        teamNode.jiangli:setString(rewardStr)
        scrollview:removeAllItems()
        if not rewards then
            return 
        end
        for _,itemData in pairs(rewards) do
            local item = createItemView(itemData)
            local scale = 0.7
            item:setScale(scale)
            local w , h = item:getContentSize()
            scrollview:pushBackCustomItem(item,w*scale,h,0.5)
        end
    end
    refreshLevelReward(reward,str)

    local isHaveTeam = true
    if isNotHaveTeam then
        isHaveTeam = false
    end
    teamNode.zhunbei:setVisible(isHaveTeam)
    teamNode.zhijiiekaiishi:setVisible(not isHaveTeam)
    teamNode.gou:setVisible(isHaveTeam and not xiezhucishuVisible)

    local zhunbeiStr = Serialize["SingleDaily_TowerPanel_5"]
    if teamNode.gou:isVisible() then
        zhunbeiStr = Serialize["SingleDaily_TowerPanel_6"]
    end
    teamNode.zhunbei.text:setString(zhunbeiStr)

    local yesterdayLevel = data:getYesterdayLevel()
    local isHaveReward = yesterdayLevel~= 0
    local isGray = false
    local rewardData = data:getDayReward()
    if isHaveReward then
        if not rewardData:isCanReward() then
            isGray = true
            isHaveReward= false
        end
        if data:myNotTeamLeader() then
           isHaveReward= false 
           isGray = false
        end
        GameUtil:createBoxEffect(isHaveReward,self.m_csNode.box4.box,3,isGray)
    end
end

function This:isHaveBox()
    local data = self.m_data
    local levelNum = data:getLevelNum()
    local levelModel = data:getLevel(levelNum)
    local isHaveReward = false
    local curentLv = data:getLevelNum()
    if not levelModel then
        return false
    end
    local myLevel = levelModel:getId()
    local isSelect = myLevel == curentLv
    local boxVisible = #levelModel:getPassReward()>0
    if boxVisible then
        if isSelect and levelModel:IsHaveBoxReward() then
            isHaveReward = true
        end
    end
    return isHaveReward
end


function This:refreshTeamInfo()
    local teamType = TeamType.kTowerTeam
    local team = WarfareManager:getMyTeam(teamType)
    if not team then
        team = WarfareManager:createMyTeam({is_ready = isHaveHeroList()}, teamType, true)
    end

    local csNode = self.m_csNode.team
    if not self.m_viewList then
        self.m_viewList = {}
        local roleList = team:getRoleList()
        for index = 1 , self.m_maxRoleCount do
            local view = TeamWarRoleView:create(roleList[index], team)
            view:setInviteHandler(function (paramRole, callback)
                local isHaveReward = self:isHaveBox()
                if isHaveReward then
                    g_scene:showTips(Serialize[13900])
                    return
                end
                local data = self.m_data
                local levelNum = data:getLevelNum()
                local levelModel = data:getLevel(levelNum)
                if not levelModel then
                    g_scene:showTips(Serialize[64196])
                    return
                end
                if not team then return end
                self.m_coroutine = coroutine.create(function ()
                    if team:getId() <= 0 then
                        WarfareManager:onCreateTeamRequest(self.m_data:getLevelNum(), nil, teamType)
                        local teamId = coroutine.yield()
                        local curTeamId = team:getId()
                        if curTeamId > 0  and teamId == curTeamId then
                            local heros = TowerManager:getLevelHeroList()
                            if heros and heros[1] and heros[1] > 0 then
                                local heroList = {}
                                for i, heroId in ipairs(heros) do
                                    heroList[i] = { pos = i, heroId = heroId }
                                end
                                WarfareManager:onWarEmbattleHerosRequest(heroList, teamType)
                                local embattleResult = coroutine.yield()
                                if not embattleResult then
                                    self.m_coroutine = nil
                                    return 
                                end
                            end
                        end
                    end

                    local myRole = team:getMainRole()
                    if not myRole or not myRole:isRoomOwner() then
                        self.m_coroutine = nil
                        g_scene:handleErrorCode(12504)
                        return 
                    end
                    
                    local ret, leftTime = WarfareManager:onApplyJoinkWarfareRequest(team)
                    if callback then
                        callback(ret, leftTime)
                    end
                end)
                coroutine.resume(self.m_coroutine)
            end)
            csNode["teamlist" .. index]:addChild(view)
            self.m_viewList[index] = view
        end
    else
        local roleList = team:getRoleList()
        for i = 1 , self.m_maxRoleCount do
            local view = self.m_viewList[i] 
            if view then
                view:setModel(roleList[i])
            end
        end
    end
end

function This:refreshLevels()
    local data = self.m_data
    local chapterId = data:getCurentChapter()
    local levelList = data:getChaperList(chapterId)

    local function refreshLight(levelIndex)
        local lightCount = 4
        local maxLevel = 4
        if levelIndex>maxLevel then
            levelIndex = maxLevel
        end
        local endLightIndex = levelIndex*lightCount
        local maxLight = maxLevel*lightCount
        local lightTexture = "MenuScene/towerluxian1.png"
        local grayTexture = "MenuScene/towerluxian2.png"
        for index=1,maxLight do
            local textureStr = lightTexture
            if index>endLightIndex then
                textureStr = grayTexture
            end
            self.m_csNode.dianNode["dian"..index]:setImage(textureStr)
        end
    end
    local function refreshGuanKa(node,itemList)
        node:setVisible(true)
        local maxPass = 0
        for levelIndex,levelModel in ipairs(itemList) do
            local viewParent =node["guan"..levelIndex]
            -- local level = levelModel:getId()
            local name = "TowerItemView"
            local view = viewParent:getChildByName(name)
            if not view then
                view = TowerItemView:create({levelModel= levelModel})
				viewParent:addChild(view)
                view:setName(name)
            else
                view:refresh(levelModel)
            end
            if view:getIsPassed() then
                maxPass = levelIndex
            end
        end
        refreshLight(maxPass)
    end
    local guakanNode = self.m_csNode.guanka1
    refreshGuanKa(guakanNode,levelList)
    
end

function This:onTeamAttrChange()
    local team = WarfareManager:getMyTeam(TeamType.kTowerTeam)
    self:setAutoStarState(team:getAutoJoin())
end

function This:onGetTowerInfo()
    self:refresh()
end

function This:onGetLevelReward()
    self:refresh()
end

function This:getData()
    self.m_data = TowerManager:getTowerData()
end

function This:onCreateTeam( teamType, teamId )
    if teamType == TeamType.kTowerTeam then
        local current = self.m_coroutine
        if current then
            coroutine.resume(current, teamId)
        end
    end
end

function This:onWarEmbattleHeros( teamType, success )
    if teamType == TeamType.kTowerTeam then
        local current = self.m_coroutine
        if current then
            coroutine.resume(current, success)
        end
    end
end

function This:onGetMyTeamInfo(teamType)
    if teamType == TeamType.kTowerTeam then
        self:refresh()
    end 
end
This.onGetDayReward = This.refreshBaseInfo
This.onJoinMatchTeam = This.onGetMyTeamInfo

function This:onExitTowerTeam(teamType, oldRole, myExit)
    if teamType == TeamType.kTowerTeam then
        if oldRole and not oldRole:isRoomOwner() or myExit then
            self:close()
            return
        end
        self:refresh()
    end 
end
