-- @Author: zheng
-- @Date:   2019-11-21 14:42:18
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-03-24 10:18:10
-- @File Name: WarTeamFightPanel.lua

local This , super  = class(...,"Dialog")
local WarTowerManager = require("SingleDaily.WarTowerManager")
local HeroManager = require("Hero.HeroManager")
local WarfareInviteView =  require("Warfare.WarfareInviteView")
local TeamWarRoleView = require("Warfare.TeamWarRoleView")
local WarfareManager = require("Warfare.WarfareManager")
local FamilyManager =  require("Family.FamilyManager")
local FriendManager =  require("Friend.FriendManager")
local ViewUtil = require("UI.ViewUtil")

local TerrTabGroupBehaviour = require("Territory.TerrTabGroupBehaviour")

local maxRoleCount =2
local NodeAniMode = {
    kOpen = 1,
    kClose = 2,
}

function This:initData(info ,dialog)
    self.m_csNode =cc.Node:createFromCSFile("MenuScene/NodeUI/TeamWarRefreshFound.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    -- self.m_war = info.war

end

function This:initNode()
    local csNode = self.m_csNode
    csNode.tips:setVisible(false)
    ViewUtil:blinkView(csNode.tips, true)
    self.m_viewList ={}

    local team = WarfareManager:getMyTeam(TeamType.kWarTeam)
    self.m_team = team
    if team then
        for i =1 , maxRoleCount do
            csNode["hero" .. i]:removeAllChildren()
            local view = TeamWarRoleView:create(nil,team)
            view:setInviteHandler(function ()
                if self.m_inviteMode == NodeAniMode.kOpen then
                    return
                end 

                self:runAnimation(csNode.invite.node,NodeAniMode.kOpen)
            end)
            csNode["hero" .. i]:addChild(view)
            self.m_viewList[i] = view
        end

    end
    self:refresh()
end

function This:onEnter()
    self:addCustomListener()
end

function This:onExit()
    self:removeCustomListener()
end

function This:resetData()
    self:refresh()
end

function This:addCustomListener()
    FamilyManager:addObserver(self)
    FriendManager:addObserver(self)
    WarfareManager:addObserver(self)
end

function This:removeCustomListener()
    FamilyManager:removeObserver(self)
    FriendManager:removeObserver(self)
    WarfareManager:removeObserver(self)
end

function This:refresh()
    local csNode = self.m_csNode
    csNode.invite.node:setPositionX(256)
    self.m_inviteMode = NodeAniMode.kClose

    local roleList = {}
    local isCaptain = false
    local visible = false
    local oneKey = WarTowerManager:getMaxHistoryLv()
    if WarfareManager:hasTeam() then
        local team = WarfareManager:getMyTeam()
        roleList = team:getRoleList()
        local str = Serialize[64202]
        local role = team:getMainRole()
        isCaptain = role:isRoomOwner()
        local isReady = role:isReady()
        if not isCaptain then
            if not isReady then
                str = Serialize[64203]
            else
                str = Serialize[64204]
            end
            visible = not isReady
        else
            local team = WarfareManager:getMyTeam()
            visible = team:canChallenge()
        end

        csNode.battle.tiaozhan:setString(str)
    else
        isCaptain = false
        visible = false
    end
    
    if not self.m_readyEff and visible then
        local function createAnim( parent )
            local GameUtil = require("Util.GameUtil")
            local effect1 = GameUtil:createEffect("Ready", true, parent)
            parent:addChild(effect1)
            return effect1
        end
        self.m_readyEff = createAnim(csNode.battle)
    elseif self.m_readyEff then
        self.m_readyEff:setVisible(visible)
    end

    for i = 1 , maxRoleCount do
        local view = self.m_viewList[i] 
        if view then
            view:setModel(roleList[i])
        end
    end
    
    -- local itemId, itemNum
    -- if isCaptain then
    --  itemId, itemNum = self.m_war:getTicketId(), self.m_war:getTicketCost()
    -- end
    
    -- if itemId and itemNum and itemId > 0 and itemNum > 0 then
    --  csNode.itemicon:setImage(ItemUtil.getIconImage(itemId))
    --  csNode.num:setString(itemNum)
    --  csNode.itemicon:setVisible(true)
    --  csNode.num:setVisible(true)
    -- else
    local team = WarfareManager:getMyTeam()
    roleList = team:getRoleList()
    local role = team:getMainRole()
    isCaptain = role:isRoomOwner()


    csNode.itemicon:setVisible(false)
    csNode.num:setVisible(false)
    self.m_csNode.title:setVisible(self.m_war:getType() ~= WarfareType.kBoss)
    self.m_csNode.jianglinum:setVisible(self.m_war:getType() == WarfareType.kBoss and not isCaptain)
    -- end


    if self.m_war:getType() == WarfareType.kBoss then
        self.m_csNode.beijing.title:setString(Serialize[64205])
        self.m_csNode.title:setString(self.m_war:getName())
        self.m_csNode.jianglinum:setString(string.format( Serialize[64206],TerritoryManager:getBossLeftRewardTimes()))
        self.m_csNode.tips:setString(Serialize[64207])
    end
end

function This:initTouchEvent()
    self.m_csNode.beijing.close:setTouchEnded(function()
        local team = WarfareManager:getMyTeam()
        if team then
            WarfareManager:onExitWarTeamRequest()

        end
        -- self:close()
    end)

    self.m_csNode.exitbtn:setTouch({
        onEnded = function()
            WarfareManager:onExitWarTeamRequest()
            self:close()
        end,
        isScale = false
    })
    
    self.m_csNode.embattle:setTouch({
        onEnded = function()
            local allHeroList
            -- local heroLimitList = self:getHeroLimit()
            -- if heroLimitList then
            --  allHeroList = {}
            --  for _, hero in ipairs(heroLimitList) do
            --      if hero:isGot() and not hero:isDeputy() then
            --          table.insert(allHeroList, hero)
            --      end
            --  end
            -- end

            g_scene:openDialog("Territory.TerrBattleHeroPanel", {
                targetType = TerrBattleTarget.kWarfareNpc,
                heroSource = WarfareManager:getEmBattleHeros(TeamType.kWarTeam),
                attacker = self.m_war, 
                heroList = allHeroList,
                battleHandler = function ( heroList )
                    WarfareManager:onWarEmbattleHerosRequest(heroList)
                    return true
                end
            })
        end,
        isScale = false
    })

    self.m_csNode.battle:setTouch({
        onEnded = function()
            -- 是否已布阵
            if not WarfareManager:hadEmbattled() then
                g_scene:showTips(Serialize[64195])
                self.m_csNode.embattle.m_touch:onEnded()
                return
            end
            
            if not WarfareManager:hasTeam() then
                return
            end

            local team = WarfareManager:getMyTeam()
            local mainRole = team:getMainRole()
            if not mainRole:isRoomOwner() then
                local value = mainRole:isReady()
                WarfareManager:onPrepareWarRequest(not value)
            else
                -- 队长
                local ret, errCode = team:canChallenge()
                if not ret then
                    g_scene:showTips(errCode)
                    return
                end

                self:challenge(team)
            end
        end,
        isScale = false
    })
end

function This:challenge(team)
    -- WarfareManager:onWarfareBattleRequest(team:getWarId(), team:getHard())
    print("挑战")
    WarfareManager:onWarfareBattleRequest(self.m_war:getCopyId(), team:getWarId())

    

end

function This:initInviteNode()
    if not self.m_groupInvite then
        local inviteNode = self.m_csNode.invite.node
        local group = TerrTabGroupBehaviour:create()
        group:addTabButton("friend", inviteNode.friend, function()
            WarfareManager:onGetInviteFriendListRequest()
        end, false, true)

        group:addTabButton("family", inviteNode.family, function()
            if not FamilyManager:hasFamily() then
                g_scene:showTips(Serialize[61508])
                return false
            end

            self:refreshInviteList(true)
        end, false, true)
        
        self.m_groupInvite = group
    end
    self.m_groupInvite:selectTabButton("friend")
    WarfareManager:onGetInviteFriendListRequest()
end

function This:runAnimation(node, mode)
    if not node or self.m_isPlayingAni then
        return
    end

    self.m_isPlayingAni = true
    self.m_inviteMode = mode
    
    local x, name = - 256, "ExponentialIn"
    if mode == NodeAniMode.kClose then
        x, name = 256, "ExponentialIn"
    end
    node:runAction(cc.Sequence(cc.MoveBy(0.2, x, 0, name), cc.CallFunc(function()
        self.m_isPlayingAni = false
        if self.m_inviteMode == NodeAniMode.kOpen then
            self:initInviteNode()
        end
    end)))
end

function This:refreshInviteList(isGroup)
    -- local timer = self.m_dialog.m_timer
    local csNode = self.m_csNode.invite.node
    local scrollView = csNode.scrollView

    scrollView:removeAllItems()
    -- timer:removeAllTimer()
    self.m_inviteViews = {}

    local list = WarfareManager:getCanInviteList(isGroup)
    for i , invite in ipairs(list) do
        -- timer:setTimer(0.1 * (i - 1),function()
            local view = WarfareInviteView:create(invite)
            local w , h = view:getContentSize()
            scrollView:pushBackCustomItem(view,w,h-3,0.5)
            self.m_inviteViews[invite:getId()] = view
        -- end,"invite"..i)
    end
end

function This:onTouchMask()
    if self.m_groupInvite and self.m_inviteMode == NodeAniMode.kOpen then
        self:runAnimation(self.m_csNode.invite.node, NodeAniMode.kClose)
    end
end

function This:onGetWarfareTabData(text,hasTeam)
    if not text == "onExitWarTeam" or hasTeam == true then
        self:initNode()
        self:refreshInviteList()
    end
    if not hasTeam and text ~= "onApplyTeamWar" then
        self:close()
    elseif hasTeam == nil and text == "onExitWarTeam" then
        self:close()
    end
end

function This:onGetInviteFriendList()
    self:refreshInviteList()
end

function This:onApplyTeamWar()
    self:refresh()
end

function This:onGetMyTeamInfo()
    self:refresh()
end

function This:onBossTeamRemove()
    self:close()
end