-- @Author: zheng
-- @Date:   2019-11-21 14:43:53
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-03-24 11:22:50
-- @File Name: WarTowerManager.lua

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
-- local WarTowerMaster = require("SingleDaily.WarTowerMaster")
local DrumTowerTaskConfig = require("Metadatas.DrumTowerTaskConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local ShopConfig = require("Metadatas.ShopConfig")
local DrumTowerConfig = require("Metadatas.DrumTowerConfig")
local SweepBuyConfig = require("Metadatas.SweepBuyConfig")
local PlayerKindConfig = require("Metadatas.PlayerKindConfig")

local MenuScene = require("Home.MenuScene")
local BattleDataManager = require("Battle.BattleDataManager")
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")
-- local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")

local SingleManager = require("SingleDaily.SingleManager")
ToweManagerLevelHeroFileName = "ToweManagerLevelHeroFileName"

function This:init()
    super.init(self)
    -- self.m_towerData = WarTowerMaster:create()
    self.m_teamNum = 0
    self.m_rankList = {}
    self:initListener()
    self:clearChallengeData()
end

-- function This:getTowerData()
--     return self.m_towerData
-- end

function This:initListener()
    EventDispatcher:addListener(self,"day_passed",self.onDayPassed)
    EventDispatcher:addListener(self, 14705, self.onGetInfoReply)--信息总览请求14705会依次返回14705，14706，14707
    EventDispatcher:addListener(self, 14706, self.onGetTowerRankReply)--进度排行榜
    EventDispatcher:addListener(self, 14707, self.onGetMinPassPowerReply)--最低战力通关列表
    EventDispatcher:addListener(self, 14708, self.onGetSaoDangRewardReply)--领取扫荡卷
    EventDispatcher:addListener(self, 14709, self.onGetBoxRewardReply)--领取扫荡成就
    EventDispatcher:addListener(self, 14712, self.onGetSaodangFightReply)--扫荡
    EventDispatcher:addListener(self, 14711, self.onGetOneFigthReply)--单人挑战，（组队例外的协议）
    EventDispatcher:addListener(self, 14710, self.onGetVipRewardReply)--领取VIP奖励
    EventDispatcher:addListener(self, 14713, self.onGetChallengeRewardReply)--关卡挑战奖励
    EventDispatcher:addListener(self, 14714, self.onGetBuySaoDangTimeReply)--购买扫荡卷
    EventDispatcher:addListener(self, 14715, self.onGetOneKeyFightReply)--一键挑战（不能组队）
    
end

function This:removeListener()
    EventDispatcher:removeListener(self,14705)--信息总览请求14705会依次返回14705，14706，14707
    EventDispatcher:removeListener(self,14707)--最低战力通关列表
    EventDispatcher:removeListener(self,14708)--领取扫荡卷
    EventDispatcher:removeListener(self,14709)--领取扫荡成就
    EventDispatcher:removeListener(self,14712)--扫荡
    EventDispatcher:removeListener(self,14711)--单人挑战，（组队例外的协议）
    EventDispatcher:removeListener(self,14710)--领取VIP奖励
    EventDispatcher:removeListener(self,14713)--关卡挑战奖励
    EventDispatcher:removeListener(self,14714)--购买扫荡卷
    EventDispatcher:removeListener(self,14715)--一键挑战（不能组队）
end

function This:getHeroList( heros )
    local result = {0,0,0,0,0,0}
    for pos , info in pairs(heros) do
        local hero = HeroManager:getHero(info)
        result[pos] = hero:getUid()
    end
    return result
end

--设置组队阵容
function This:getLevelTeamHeroList()
    return self.m_teamHeroList or {}--{heroId1,heroId2}
end

function This:setLevelTeamHeroList(heroList)
    if heroList then
        self.m_teamHeroList = heroList
    end
end

function This:getLevelHeroList()
    return SingleManager:getPalyerActInfo("WarLevelHeroList",{},ToweManagerLevelHeroFileName)
end

function This:setLevelHeroList(heroList)
    if heroList then
        SingleManager:setPalyerActInfo("WarLevelHeroList", heroList,ToweManagerLevelHeroFileName)
    end
end

function This:clearChallengeData()
    self.m_chalengeType = nil
    -- self.m_challengeResult = {reward ={},isMyResult = false,isWin = false,starLv = 0}
    self.m_challengeResult = {reward ={},isWin = false,starLv = 0}
end

function This:setChallengeState(chalengeState,chalengeType)
    -- self.m_chalengeState = chalengeState
    self.m_chalengeType = chalengeType
    self.m_challengeResult.isWin = chalengeState
    BattleDataManager:addObserver(self)
end

function This:onBattleSkip(result)
    if self.m_challengeResult.isWin == BattleResult.kAttackerVictory then
        self:onBattleVictory()
    else--if foo then
        self:onBattleFailed()
    end
end

----------------------------------------------------------
--挑战结果
function This:onBattleVictory(teamModel)
    if teamModel then
        self.m_chalengeType = SingleTowerChallengeType.kLevelType
    end
    if self.m_chalengeType then
        local function levelCallBack( )
            cc.SceneManager:switchTo("Home.MenuScene")
            MenuScene:addDelayDialog({name ="SingleDaily.WarTowerMainPanel" })
        end
        local function towerCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
           MenuScene:addDelayDialog({name ="SingleDaily.WarTowerMainPanel", arg = {towerChllengeCallBack = true}}) 
        end
        local function helpCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
        end

        local info = {dropInfo = self.m_challengeResult.reward,touchMask = levelCallBack,
            isRobber= true}
        if self.m_chalengeType== SingleTowerChallengeType.kMasterType then
            info.touchMask = towerCallBack
        elseif teamModel then
             local role = teamModel:getMainRole()
            if not role:isRoomOwner() then
                info.touchMask = helpCallBack
            end
        end
        g_scene:openDialog("SingleDaily.TowerChallengeSucessPanel",info)
    end
    self:clearChallengeData()
    if not teamModel then
        BattleDataManager:removeObserver(self)
    end
end

function This:onBattleFailed(teamModel)
    
    if teamModel then
        self.m_chalengeType = SingleTowerChallengeType.kLevelType
    end
    if self.m_chalengeType then
        local function levelCallBack( )
            cc.SceneManager:switchTo("Home.MenuScene")
            MenuScene:addDelayDialog({name ="SingleDaily.WarTowerMainPanel" })
        end
        local function towerCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
           MenuScene:addDelayDialog({name ="SingleDaily.WarTowerMainPanel", arg = {towerChllengeCallBack = true}}) 
        end
        local function helpCallBack()
            cc.SceneManager:switchTo("Home.MenuScene")
        end

        local info = {dropInfo = self.m_challengeResult.reward,touchMask = levelCallBack}
        if self.m_chalengeType== SingleTowerChallengeType.kMasterType then
            info.touchMask = towerCallBack
        elseif teamModel then
            local role = teamModel:getMainRole()
            if not role:isRoomOwner() then
                info.touchMask = helpCallBack
            end
            -- local towerData = self:getTowerData()
            -- if towerData:isNotHaveTeam() and towerData:myNotTeamLeader() then --自己有队伍，自己不是队长返回true
            -- end
        end
        g_scene:openDialog("SingleDaily.TowerChallengeFailPanel",info)
    end
    self:clearChallengeData()
    if not teamModel then
        BattleDataManager:removeObserver(self)
    end
end

function This:getVipId()
    return 53
end

function This:getAllMaxCeng()
    return #DrumTowerConfig
end

function This:getPassReward(lv)
    local item1 =DrumTowerConfig[lv]
    if item1 then
        return item1.first_reward ,item1.sweep_reward
    end
    return {},{}
end

function This:getVipLeftDay()
    local function getLeftTime()
        local curTime = GamePlayer:getServerTime()
        if curTime >= self.m_vipEdTime then
            return 0
        end
        return self.m_vipEdTime - curTime
    end
    if self.m_isVip then
       local days = math.secondToDay(getLeftTime())
       return days
    end
    return 30
end

function This:getVipConfig()
    return ArgConfig.ex_gift.arg,self:getVipLeftDay(),ShopConfig[self:getVipId()].price
end

function This:getVipInfo()
    return self.m_isVip,self.m_vipEdTime,self.m_haveVipReward
end
--扫荡卷是否领取 0未领取  
function This:isCanGetSaoDang()
    return self.m_haveSaoDang ==0
end

function This:getTeamNum()
    local function getTeamMaxNum()
        local isVip = self:getVipInfo()
        local configItem = PlayerKindConfig[1]
        if isVip then
            configItem = PlayerKindConfig[2]
        end
        return configItem.assistant, configItem.progress_max
    end
    local teamMax,challengeMax = getTeamMaxNum()
    return  teamMax - self.m_teamNum or 0,challengeMax
end

function This:getChallengeNum()
    return self.m_challengeNum or 0
end

function This:getBuySaoDangCost(buyTime)
    local function getPrice(time)
        local res =0
        local temCost = nil
        for _,costItem in ipairs(SweepBuyConfig) do
            local time1 = costItem.times[2]
            local cost = costItem.price
            if time<= time1 or time1 == -1 then
                temCost = cost
                break
            end
        end
        return temCost or res
    end
    local time = self:getSaoDangBuyTime()
    if 1==buyTime then
        return getPrice(time+1)
    else
        local res = 0
        for index=1,buyTime do
            res = res +getPrice(time+index)
        end
        return res
    end
end

function This:checkSaoDangBuy(time,lv)
    if time<=self:getSaoDangNum() then
        self:onGetSaodangFightRequest(lv,time,0)
    else
        local cost = self:getBuySaoDangCost(time)
         local myPanel = g_scene:openDialog("UI.CostConfirmPanel", {
                fileName = "MenuScene/NodeUI/DrumTowerBuy1.csb",
                cost = cost,
                notips = "WarTowerManager"..cost,--"
                descText = string.format(Serialize[64219],time), 
                okHandler = function(build)
                        self:onGetSaodangFightRequest(lv,time,1)
                end } )
        -- if self.m_activityType == NewActivityType.kBingFaRoulette then
        --     myPanel:setBtnString(string.format(strList[self.m_activityType],needjuanCount),strList.bingFaCost)
        -- end
    end
end

function This:getSaoDangBuyTime()
    return self.m_saoDangBuyNum
end
--扫荡次数
function This:getSaoDangTime()
    return self.m_boxRewardScore
end

function This:getSaoDangDayNum()
    return self.m_saoDangDayTime
end

--扫荡卷数量
function This:getSaoDangNum()
    return DepotManager:getItemCount(345)
end

function This:getMyMinPower(index)
    if index then
        local res = self.m_myMinPower[index] or {}
        res.icon = GamePlayer:getIcon()
        res.name = Serialize[64220]
        return res
    end
    return self.m_myMinPower
end

function This:getOtherMinPower(index)
    if index then
        return self.m_otherMinPower[index]
    end
    return self.m_otherMinPower
end

function This:getRankList(index)
    if index then
        local res = self.m_rankList[index]
        return res
    end
    return self.m_rankList
end

function This:getLastLv()
    return self.m_curLv
end

function This:getCurentLv()
    return self.m_curLv
end

function This:getMaxLv()
    return self.m_maxLv
end

function This:getMaxHistoryLv()
    return self.m_historyMaxLv or 0
end

function This:setMaxLv(lv)
   self.m_maxLv = lv or 0 
   if self:getMaxHistoryLv()< self.m_maxLv then
       self.m_historyMaxLv = self.m_maxLv
   end
end

function This:getRewardList()
    if not self.m_boxRewardList then
        self.m_boxRewardList ={}
        for index, config in ipairs(DrumTowerTaskConfig) do
            self.m_boxRewardList[index] = {
                    id = config.id, 
                    state = RecruitManagerBoxState.kLock, 
                    reward = config.reward, 
                    integral = config.time,
                }
        end
    end
    return table.clone(self.m_boxRewardList)
end

function This:isHaveReward()
    return self:isCanGetSaoDang() or self:isHaveSaoDangReward() or self:isHaveVipReward()
end

function This:isHaveSaoDangReward()
    return self.m_isCanReward
end

function This:isHaveVipReward()
    return self.m_isVip and self.m_haveVipReward
end
    
function This:onGetOneKeyFightRequest()
    Socket:sendAndLock(14715)--一键挑战（不能组队）
end

function This:onGetOneKeyFightReply( msg )
    if msg.err_code == 0 then
        self:setMaxLv(msg.max_level)
        self.m_curLv = msg.cur_level or 0
        self.m_challengeNum = msg.challenge_times or 0
        local tmp = {}
        for i , info in ipairs(msg.rewardList or {}) do
            local id = info.id or info[1]
            local num = info.num or info.count or info[2]
            tmp[id] = tmp[id] or 0
            tmp[id] = tmp[id] + num
        end
        local res ={}
        for id,num in pairs(tmp) do
            table.insert(res,{id,num})
        end

        self.m_challengeResult.reward = res
        self.m_chalengeType = SingleTowerChallengeType.kLevelType
        self.m_challengeResult.isWin = true
        self:onBattleVictory()
        self:notifyObservers("onGetOneKeyFight")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetBuySaoDangTimeRequest(num)
    Socket:sendAndLock(14714,{num = num})--购买扫荡卷
end

function This:onGetBuySaoDangTimeReply( msg )
    if msg.err_code == 0 then
        -- print(msg)
        self.m_saoDangBuyNum = msg.saodang_day_buy_times
        g_scene:showReward({{345,1}})
        self:notifyObservers("onGetBuySaoDangTime")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetChallengeRewardReply( msg )
--    if msg.err_code == 0 then
        self.m_challengeResult.reward = msg.reward_list or {}
        self:setMaxLv(msg.max_level)
        self.m_curLv = msg.cur_level or 0
        self.m_challengeNum = msg.challenge_times or 0
        self.m_teamNum =msg.team_times or 0
        self:notifyObservers("onGetChallengeReward")
--    else
--        g_scene:handleErrorCode(msg.err_code)
--    end
end

function This:onGetSaodangFightRequest(lv,num,isBuy)
    Socket:sendAndLock(14712,{level = lv,num =num,user_gold = isBuy})--扫荡
end

function This:onGetSaodangFightReply( msg )
    if msg.err_code == 0 then
        self.m_saoDangBuyNum = msg.saodang_day_buy_times
        self.m_boxRewardScore = msg.saodang_week_times
        self.m_saoDangDayTime = msg.saodang_day_times or 0
        self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = self:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,DrumTowerTaskConfig,self.m_boxRewardHistory,"time")
        -- g_scene:showReward(msg.reward_list or {})
        self:notifyObservers("onGetSaodangFight",msg.reward_list)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
   -- //1为挑战关卡，2为挑战最低战力 
function This:onGetOneFigthRequest(type,level,heroList)
        self:clearChallengeData()
    local cardList = self:getHeroList(heroList)
    Socket:sendAndLock(14711,{type = type,level = level,card_list= cardList})--单人挑战，（组队例外的协议）
end
    
function This:onGetOneFigthReply( msg )
     if msg.err_code == 0 then
        -- print(msg)
        local isWin = msg.win_or_lose or false
        self.m_challengeResult.isWin = isWin
        if isWin then
            -- self.m_challengeResult.reward = msg.rew
            self.m_myMinPower = self:conertMinPower(msg.pass_power_list or {})
        end
        -- self:setChallengeState(msself:setChallengeState(msg.is_win,SingleTowerChallengeType.kLevelType)g.is_win,SingleTowerChallengeType.kLevelType)
        self:setChallengeState(msg.win_or_lose,SingleTowerChallengeType.kLevelType)
        cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {bgId = 2,msg = msg})
        self:notifyObservers("onGetOneFigth")
     else
         g_scene:handleErrorCode(msg.err_code)
     end
end
    
function This:onGetVipRewardRequest()
    Socket:sendAndLock(14710)--领取VIP奖励
end

function This:onGetVipRewardReply( msg )
    if msg.err_code == 0 then
        -- print(msg)
        local rewardList = self:getVipConfig()
        g_scene:showReward(rewardList)
        self.m_haveVipReward = msg.vip_reward ==0
        self:notifyObservers("onGetVipReward")

    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetBoxRewardRequest(boxId)
    Socket:sendAndLock(14709,{reward_id = boxId})--领取扫荡成就
end
    -- rewardTable 本地的奖励列表
    -- curentScore 当前积分
    -- rewardConfig 奖励的配置表,仅支持从1开始的顺序表
    -- rewardReceivedData 已经领取的奖励列表
function This:refreshRewardList(rewardTable,curentScore,rewardConfig,rewardReceivedData,rewardMarkStr)
   return GameUtil:refreshRewardList(rewardTable,curentScore,rewardConfig,rewardReceivedData,rewardMarkStr)
end

function This:onGetBoxRewardReply( msg )
    if msg then
        -- print(msg)
        local boxId = msg.id
        local rewardList = msg.reward_list
        g_scene:showReward(rewardList)
        self:getRewardList()
        self.m_boxRewardHistory = msg.saodang_reward
        self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = self:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,DrumTowerTaskConfig,self.m_boxRewardHistory,"time")
        self:notifyObservers("onGetBoxReward")
        EventDispatcher:dispatcher("recruit_zhongjin_cd_changed")
    else
        g_scene:handleErrorCode("")
    end
end

function This:onGetSaoDangRewardRequest()
    Socket:sendAndLock(14708)--领取扫荡卷
end

function This:onGetSaoDangRewardReply( msg )
    if msg.err_code == 0 then
        -- print(msg)
        self.m_haveSaoDang =1
        g_scene:showReward({{345,1}})
        self:notifyObservers("onGetSaoDangReward")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
function This:onGetMinPassPowerRequest()
    Socket:sendAndLock(14707)--最低战力通关列表
end

function This:onGetMinPassPowerReply( msg )
    -- if msg.err_code == 0 then
        -- print(msg)
        self.m_otherMinPower = self:conertMinPower(msg.pass_power_list or {})
        self:notifyObservers("onGetMinPassPower")
    -- else
    --     g_scene:handleErrorCode(msg.err_code)
    -- end
end
function This:onGetTowerRankReply( msg )
    -- if msg.err_code == 0 then
        -- print(msg)
        self.m_rankList = self:conertMinPower(msg.tower_rank_list or {},"rank")
        self:notifyObservers("onGetTowerRank")
    -- else
    --     g_scene:handleErrorCode(msg.err_code)
    -- end
end

function This:onDayPassed()
    self.m_boxRewardList = nil
    self.m_boxRewardHistory ={}
    self:onGetInfoRequest()
end

function This:onGetInfoRequest()
    Socket:sendAndLock(14705)--信息总览请求14705会依次返回14705，14706，14707
end

function This:conertMinPower(passList,keyStr)
    local res ={}
    for _,item1 in ipairs(passList) do
        local lv = item1[keyStr] or item1.level
        res[lv] = item1
    end
    return res
end

function This:onGetInfoReply( msg )
    if msg.err_code == 0 then
        self.m_boxRewardList = nil
        self.m_boxRewardHistory ={}
        -- print(msg)
        self.m_historyMaxLv = msg.his_max_level or 0
        self:setMaxLv(msg.max_level)
        self.m_curLv = msg.cur_level or 0
        self.m_myMinPower = self:conertMinPower(msg.pass_power_list or {})
        self.m_challengeNum = msg.challenge_times or 0
        self.m_teamNum =msg.team_times or 0 or 0
        self.m_haveSaoDang = msg.saodang_juan or 0
        self.m_saoDangBuyNum = msg.saodang_day_buy_times
        self:getRewardList()
        self.m_boxRewardHistory = msg.saodang_reward
        self.m_boxRewardScore = msg.saodang_week_times or 0
        self.m_boxRewardList,self.m_isCanReward,self.m_minChallengeStarLv = self:refreshRewardList(self.m_boxRewardList,self.m_boxRewardScore,DrumTowerTaskConfig,self.m_boxRewardHistory,"time")        
        self.m_isVip = msg.vip == 2
        self.m_vipEdTime = msg.vip_end_time or 0
        self.m_haveVipReward = msg.vip_reward ==0
        self.m_saoDangDayTime = msg.saodang_day_times or 0
        self:notifyObservers("onGetInfo")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end