-- @Author: zheng
-- @Date:   2019-11-26 15:45:03
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-03-30 14:59:21
-- @File Name: WarTowerTeamPanel.lua
local WarfareManager = require("Warfare.WarfareManager")
local TeamWarRoleView = require("Warfare.TeamWarRoleView")
local WarTowerManager = require("SingleDaily.WarTowerManager")
local TerrTabGroupBehaviour = require("Territory.TerrTabGroupBehaviour")
local FamilyManager =  require("Family.FamilyManager")
local WarfareInviteView =  require("Warfare.WarfareInviteView")
local GameUtil = require("Util.GameUtil")

local This , super = class(...,"Dialog")
-- local FamilyManager = require("Official.FamilyManager")
This.fileName = "MenuScene/NodeUI/DrumTowerTeam.csb"

local NodeAniMode = {
	kOpen = 1,
	kClose = 2,
}

local teamType = TeamType.kWarTowerTeam
function This:initNode(info)
    -- if info then
    --    self.m_family = info.family
    -- end
    self.m_lv = info.lv
    self.m_csNode = cc.Node:createFromCSFile(This.fileName)
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    --self:getData()
    local csNode = self.m_csNode
    csNode.invite.node:setPositionX(256)
	self.m_inviteMode = NodeAniMode.kClose
    self.m_maxRoleCount = 2
     self:refresh()

end

function This:initInviteNode()
	if not self.m_groupInvite then
	    local inviteNode = self.m_csNode.invite.node
	    local group = TerrTabGroupBehaviour:create()
	    group:addTabButton("friend", inviteNode.friend, function()
			WarfareManager:onGetInviteFriendListRequest()
	    end, false, true)

	    group:addTabButton("family", inviteNode.family, function()
	    	if not FamilyManager:hasFamily() then
	    		g_scene:showTips(Serialize[61508])
	    		return false
	    	end

            self:refreshTeamInfo(true)
            self:refreshInviteList(true)

        end, false, true)
	    
	    self.m_groupInvite = group
	end
    self.m_groupInvite:selectTabButton("friend")
    WarfareManager:onGetInviteFriendListRequest()
end

function This:refresh()
    -- local data = self.m_data
    
    self:refreshTeamInfo()
end

local function isHaveHeroList()
    local heroList = WarTowerManager:getLevelHeroList()
    if heroList then
        return table.count(heroList) > 0
    end
end

function This:saveTeamHero()
    local function setHeroInfo(heros)
        if heros then
            local heroList = {}
            for i, heroId in pairs(heros) do
                heroList[i] = { pos = i, heroId = heroId }
            end
            -- local team = WarfareManager:getMyTeam(teamType)
            -- local mainRole = team:getMainRole()
            -- local value = mainRole:isReady()
            -- if mainRole:isRoomOwner() and not value then
            --     WarfareManager:onPrepareWarRequest(true,teamType)
            -- end
            WarfareManager:onWarEmbattleHerosRequest(heroList, teamType)
            return true
        end
        return false
    end
    if WarfareManager:hasTeam(teamType) then
        local heroList = WarTowerManager:getLevelHeroList()
        setHeroInfo(heroList)
    end
end

function This:initTouchEvent()
    local csNode = self.m_csNode
    csNode.beijing.close:setTouchEnded(function()
        if WarfareManager:hasTeam(teamType) then
            WarfareManager:onExitWarTeamRequest(nil, teamType)
            self:close()
            return
        end
        self:close()
    end)
    csNode.automatic:setTouchEnded(function()
        local oneKeyVisible = WarTowerManager:getMaxHistoryLv()>WarTowerManager:getCurentLv()

        if WarfareManager:hasTeam(teamType) then
            g_scene:showTips(Serialize[64228])
            return
        elseif not oneKeyVisible then
            g_scene:showTips(Serialize[64229])
            return
        end
        WarTowerManager:onGetOneKeyFightRequest()
    end)
    csNode.embattle:setTouchEnded(function()--布阵
        self:openDialog("Territory.TerrBattleHeroPanel", {
            targetType = TerrBattleTarget.kTower,
            heroSource = WarTowerManager:getLevelHeroList(),
            attacker = string.format(Serialize["SingleDaily_MinPower_1_c"],self.m_lv), 
            -- heroList = allHeroList,
            battleHandler = function (heros)
                local saveHeroList = {}
                for _, info in ipairs(heros) do
                    saveHeroList[info.pos] = info.heroId
                end
                WarTowerManager:setLevelHeroList(saveHeroList)
                self:saveTeamHero()
                return true
            end
        })
    end)
    csNode.battle:setTouchEnded(function()--开始挑战
        if WarfareManager:hasTeam(teamType) then
            self:saveTeamHero()
            -- 是否已布阵
            if not WarfareManager:hadEmbattled(teamType) then
                g_scene:showTips(Serialize[64195])
                self.m_csNode.embattle.m_touch:onEnded()
                return
            end
            
            if not WarfareManager:hasTeam(teamType) then
                return
            end

            local team = WarfareManager:getMyTeam(teamType)
            local mainRole = team:getMainRole()
            if not mainRole:isRoomOwner() then
                local value = mainRole:isReady()
                WarfareManager:onPrepareWarRequest(not value,teamType)
            else
                -- 队长
                local ret, errCode = team:canChallenge()
                if not ret then
                    -- WarfareManager:onExitWarTeamRequest(nil, teamType)
                    -- local heroList = WarTowerManager:getLevelHeroList()
                    -- WarTowerManager:onGetOneFigthRequest(1,self.m_lv,heroList)
                    g_scene:showTips(errCode)
                    return
                end
                WarfareManager:onWarfareBattleRequest(team:getWarId(), nil, nil, teamType)
            end
        else
            if isHaveHeroList() then
                local heroList = WarTowerManager:getLevelHeroList()
                WarTowerManager:onGetOneFigthRequest(1,self.m_lv,heroList)
            else
                g_scene:showTips(Serialize["SingleDaily_TowerMain_8"])
            end
        end
    end)
    csNode.exitbtn:setTouchEnded(function()--
        if WarfareManager:hasTeam(teamType) then
            WarfareManager:onExitWarTeamRequest(nil, teamType)
            return
        end
    end)
end

function This:refreshTeamInfo()
    local team = WarfareManager:getMyTeam(teamType)
    if not team then
        team = WarfareManager:createMyTeam({is_ready = isHaveHeroList(),teamType = teamType}, teamType, true)
    end

    local csNode = self.m_csNode
    if not self.m_viewList then
        self.m_viewList = {}
        local roleList = team:getRoleList()
        for index = 1 , self.m_maxRoleCount do
            local view = TeamWarRoleView:create(roleList[index], team)
            view:setInviteHandler(function (paramRole, callback)
            WarfareManager:onGetInviteFriendListRequest()
                if self.m_inviteMode == NodeAniMode.kOpen then
                    return
                end 

                self:runAnimation(csNode.invite.node,NodeAniMode.kOpen)
                if not team then return end
                self.m_coroutine = coroutine.create(function ()
                    if team:getId() <= 0 then
                        WarfareManager:onCreateTeamRequest(self.m_lv, nil, teamType)
                        local teamId = coroutine.yield()
                        local curTeamId = team:getId()
                        if curTeamId > 0  and teamId == curTeamId then
                            local heros = WarTowerManager:getLevelHeroList()
                            if heros and heros[1] and heros[1] > 0 then
                                local heroList = {}
                                for i, heroId in ipairs(heros) do
                                    heroList[i] = { pos = i, heroId = heroId }
                                end
                                WarfareManager:onWarEmbattleHerosRequest(heroList, teamType)
                                local embattleResult = coroutine.yield()
                                if not embattleResult then
                                    self.m_coroutine = nil
                                    return 
                                end
                            end
                        end
                        -- WarfareManager:onPrepareWarRequest(true,teamType)
                    end
                    local myRole = team:getMainRole()
                    if not myRole or not myRole:isRoomOwner() then
                        self.m_coroutine = nil
                        g_scene:handleErrorCode(12504)
                        return 
                    end
                end)
                coroutine.resume(self.m_coroutine)
            end)
            csNode["hero" .. index]:addChild(view)
            self.m_viewList[index] = view
        end
    else
        local roleList = team:getRoleList()
        for i = 1 , self.m_maxRoleCount do
            local view = self.m_viewList[i] 
            if view then
                view:setModel(roleList[i])
            end
        end
    end
     local oneKeyVisible = WarTowerManager:getMaxHistoryLv()>WarTowerManager:getCurentLv()
     
    local isHaveTeam = WarfareManager:hasTeam(teamType)
     csNode.automatic:setVisible(true)
     GameUtil:btnGray(csNode.automatic,isHaveTeam or not oneKeyVisible)
    local isCaptain = false
    local visible = false
    if isHaveTeam then
        local team = WarfareManager:getMyTeam(teamType)
        local str = Serialize[64202]
        local role = team:getMainRole()
        isCaptain = role:isRoomOwner()
        local isReady = role:isReady()
        if not isCaptain then
            if not isReady then
                str = Serialize[64203]
            else
                str = Serialize[64204]
            end
            visible = not isReady
        else
            visible = team:canChallenge()
        end

        csNode.battle.tiaozhan:setString(str)
    else
        isCaptain = false
        visible = false
    end
    csNode.exitbtn:setVisible(isHaveTeam)
end

function This:refreshInviteList(isGroup)
    -- local timer = self.m_dialog.m_timer
    local csNode = self.m_csNode.invite.node
    local scrollView = csNode.scrollView

    scrollView:removeAllItems()
    -- timer:removeAllTimer()
    self.m_inviteViews = {}

    local list = WarfareManager:getCanInviteList(isGroup)
    for i , invite in ipairs(list) do
        -- timer:setTimer(0.1 * (i - 1),function()
            local info2 ={
                inviteCallBack = function()
                    self:inviteFriend(invite)
                end,
                teamType =teamType,
            }
            local view = WarfareInviteView:create(invite,info2)
            local w , h = view:getContentSize()
            scrollView:pushBackCustomItem(view,w,h-3,0.5)
            self.m_inviteViews[invite:getId()] = view
        -- end,"invite"..i)
    end
end

function This:inviteFriend(invite)
    local team = WarfareManager:getMyTeam(teamType)
    WarfareManager:onGetTowerInviteRequest(invite:getId(),team:getId())
end

function This:onEnter()
    super.onEnter(self)
    WarfareManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    WarfareManager:removeObserver(self)
end

function This:runAnimation(node, mode)
	if not node or self.m_isPlayingAni then
		return
	end

	self.m_isPlayingAni = true
	self.m_inviteMode = mode
	
	local x, name = - 256, "ExponentialIn"
	if mode == NodeAniMode.kClose then
		x, name = 256, "ExponentialIn"
	end
    node:runAction(cc.Sequence(cc.MoveBy(0.2, x, 0, name), cc.CallFunc(function()
		self.m_isPlayingAni = false
		if self.m_inviteMode == NodeAniMode.kOpen then
			self:initInviteNode()
		end
	end)))
end

function This:onTouchMask()
	if self.m_groupInvite and self.m_inviteMode == NodeAniMode.kOpen then
		self:runAnimation(self.m_csNode.invite.node, NodeAniMode.kClose)
	end
end


function This:onCreateTeam( teamType1, teamId )
    if teamType1 == teamType then
        local current = self.m_coroutine
        if current then
            coroutine.resume(current, teamId)
        end
        self:refresh()
        self:saveTeamHero()
    end
end

function This:onWarEmbattleHeros( teamType1, success )
    if teamType1 == teamType then
        local current = self.m_coroutine
        if current then
            coroutine.resume(current, success)
        end
        self:refresh()
    end
end

function This:onGetMyTeamInfo(teamType1)
    if teamType1 == teamType then
        self:refresh()
    end 
end
-- This.onGetDayReward = This.refreshBaseInfo
This.onJoinMatchTeam = This.onGetMyTeamInfo
This.onPrepareWar = This.refresh

function This:onExitTowerTeam(teamType1, oldRole, myExit)
    if teamType1 == teamType then
        if oldRole and not oldRole:isRoomOwner() or myExit then
            self:close()
            return
        end
        self:refresh()
    end 
end


function This:onGetWarfareTabData(text,hasTeam)
    if not text == "onExitWarTeam" or hasTeam == true then
        self:initNode()
        self:refreshInviteList()
    end
    if not hasTeam and text ~= "onApplyTeamWar" then
        self:close()
    elseif hasTeam == nil and text == "onExitWarTeam" then
        self:close()
    end
end

function This:onGetInviteFriendList()
    self:refreshInviteList()
end
