-- @Author: zheng
-- @Date:   2018-08-14 17:04:47
-- @File Name: ChallengeChapterPanel.lua

local This , super = class(...,"Dialog")
local ChallengeStageManager = require("Stage.ChallengeStageManager")
local GamePlayer = require("Player.GamePlayer")
local strList ={
    chapter =Serialize[64175],
    Sweeping =Serialize[64290],
    challenge =Serialize[64291],
    resetTime =Serialize[64181],
    challengeLimit = Serialize[64292]
}
function This:initNode(info)
	self.m_chapterId = info.chapterId
	local chapter = ChallengeStageManager:getChapter(self.m_chapterId)
	self.m_data = chapter
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TeamWarMapChapter.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	self.m_csNode.scrollView:setDirection("horizontal")
    self.m_csNode.scrollView.m_direction = "right"

	self.m_selectIndex =1
	local stages = chapter:getStages()
	local stagesCount = #stages
	for i = 1 , stagesCount do
		local stage = stages[i]
		if not stage:isPassed() then
			self.m_selectIndex = i
			break
		end
	end
	if stages[stagesCount]:isPassed() then
		self.m_selectIndex =stagesCount
	end
	self:refreshSelected()
	self:refresh(  )
end


function This:onEnter()
	super.onEnter(self)
	ChallengeStageManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	ChallengeStageManager:removeObserver(self)
end

function This:refresh(  )
	self:refreshTest( )
	self:refreshStage(  )	
end

function This:refreshSelected()
	local chapter = self.m_data
	local stages = chapter:getStages()
	for i = 1 , #stages do
		local stage = stages[i]
		local hero = self.m_csNode["hero"..i]
		hero.item.xuanzhong:setVisible(self.m_selectIndex ==i)
	end
end

function This:refreshStage(  )
	local chapter = self.m_data
	local stages = chapter:getStages()
	for i = 1 , #stages do
		local stage = stages[i]
		local hero = self.m_csNode["hero"..i]

		local item = hero.item
		item.hero:setImage(stage:getImageFile())
		item.name:setString(stage:getName(), AppText:isChinese() and 25 or nil)
		item.lock:setVisible(not stage:isLock())
		local starLv = stage:getStarLv()
		for starIndex=1,3 do
			local star = item["star"..starIndex]
			star:setVisible(starLv>=starIndex)
		end
	end
	local stage = stages[self.m_selectIndex]
	local drops = stage:getDrops()
    self.m_csNode.scrollView:removeAllItems()
    local function createItemView(model)
    	local ItemNode = require("UI.ItemNode")
        local node = ItemNode:create(model[1],model[2])
        node:setAnchorPoint(0.5,0.5)
        return node
    end
    for i,itemConfig in ipairs(drops) do
        local item = createItemView(itemConfig)
        local scale = 1
        item:setScale(scale)
        local w , h = item:getContentSize()
        self.m_csNode.scrollView:pushBackCustomItem(item,w*scale,h,0.5)
    end
    local str = strList.challenge
    if stage:isPassed() then
    	str = strList.Sweeping
    end
    self.m_csNode.buttonbg2.text:setString(str)
end

function This:refreshTest( )
	local chapter = self.m_data
	local canChalenge, chanlengeTime = ChallengeStageManager:getChallengeTimes()
	local canReset,resetTime = ChallengeStageManager:getResetTiem()
	local strFormat = "<A5>"..strList.chapter.."：%d"
	if not canChalenge then
		strFormat = "<A5>"..strList.chapter.."：<New11>%d"
	end
	self.m_csNode.title:setString(chapter:getTitle())
	self.m_csNode.numberstar:setString(chapter:getTotalStar())
	self.m_csNode.addbg:setVisible(not canChalenge)
	self.m_csNode.numbercishu:setStringWithColor(string.format(strFormat,chanlengeTime))
	if not self.m_hongdian then
		local parent = self.m_csNode.jiangli
		local sp = cc.Sprite:create("LoginScene/baoman.png")
		parent:addChild(sp)
	    self.m_hongdian = sp
		local w, h = parent:getContentSize()
        sp:setPosition(w-5, h-5)
	end
	self.m_hongdian:setVisible(chapter:isCanReward())
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function ()
		self:close()
	end)
	self.m_csNode.buttonbg2:setTouchEnded(function ()--挑  战,扫荡
		local chapter = self.m_data
		local stages = chapter:getStages()
		local stage = stages[self.m_selectIndex]
		local stageId = stage:getId()
		local canChalenge, chanlengeTime = ChallengeStageManager:getChallengeTimes()
		if not canChalenge then
			g_scene:showTips(strList.resetTime)
			return
		end
		if stage:isPassed() then--扫荡
			ChallengeStageManager:onStageSweepRequest(stageId)
			return
		end
		local levelLimit = chapter:getLockLevel()
		if not (GamePlayer:getLevel() >= levelLimit) then
			g_scene:showTips(string.format(strList.challengeLimit,levelLimit))
			return
		end
		
		--挑  战
		g_scene:openDialog("Stage.ChallengeEmbattleHeroPanel",{stageId = stageId})
	end)
	self.m_csNode.jiangli:setTouchEnded(function ()
		g_scene:openDialog("Stage.ChallengeRewardPanel",{chapterId = self.m_chapterId})
	end)
	self.m_csNode.addbg:setTouchEnded(function ()
		ChallengeStageManager:resetBtnCallBack( )
	end)
	for i=1,6 do
		local hero = self.m_csNode["hero"..i]
		local item = hero.item
		item:setTouchEnded(function()
			local chapter = self.m_data
			local stages = chapter:getStages()
			local stage = stages[i]
			if stage:isLock() then
		        self.m_selectIndex = i
		        self:refreshSelected()
		        self:refreshStage(  )
		    else
		     	g_scene:showTips(Serialize["stage_unlock"])
			end
		end,item.bg)
	end
	self.m_csNode.scrollView:setSwallowTouch(false)
end

function This:OnStateInfo()--获取挑战关卡信息
	self:refresh()
end

function This:onChallengeState(  )--挑战关卡
	self:refresh()
end

function This:onStageReset(  )--挑战次数重置
	self:refresh()
end

function This:onStageSweep(  )--关卡扫荡
	self:refresh()
end

function This:onStarRewardChanged(  )--领取奖励
	self:refresh()
end
