-- @Author: zheng
-- @Date:   2018-08-17 15:07:10
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 17:35:07
-- @File Name: ChallengeMapNode.lua
local ChallengeStageManager = require("Stage.ChallengeStageManager")


local This , super = class(...,"NodeUI")
local fileNames = {
	"MenuScene/NodeUI/TeamWarMap1.csb",
}

local strList ={
    lockLimit =Serialize[60796]
}
local chapterCount =7

function This:init(infor)
	self.m_UiItems ={}--ui节点
	self.m_chapters ={}
	self.m_scrollView = infor
end

function This:onChapterClick( index )
	local chapter = ChallengeStageManager:getChapter(index)
	if chapter:isLock() then
		g_scene:openDialog("Stage.ChallengeChapterPanel",{chapterId = index})
		ChallengeStageManager:setLastChapter(chapter)
	else
		g_scene:showTips(string.format(strList.lockLimit,chapter:getLockLevel()))
	end
end

function This:onEnter()
    super.onEnter(self)
    ChallengeStageManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    ChallengeStageManager:removeObserver(self)
end

function This:initBg()

	local maxCount = 1
	local chapter = ChallengeStageManager:maxChapter()
	maxCount = math.ceil(chapter:getId()/chapterCount)--不小于x的最大整数
	if maxCount<=0 then
		maxCount =1
	end
	local anx =0
	self:setAnchorPoint(anx, anx)
	local x,y = 0
	for i=1,maxCount do
		local item = self.m_UiItems[i]
		if not item then
			item= self:createItem()
			item:setAnchorPoint(0.5, 0)
			self:addChild(item)
			for index=1,chapterCount do
				local chapterIndex =#self.m_chapters +1
				local chapterView = item["chapter"..index]
				self.m_chapters[chapterIndex]= chapterView
				if chapterIndex == chapter:getId() then
					self:chapterAction(chapterView,true)
				end

				chapterView.chapterNode:setTouch({onEnded = function ()
					self:onChapterClick( chapterIndex )
				end,
				sizeNode = chapterView.chapterNode.bg,
				isDynamic = true})


			end
			self.m_UiItems[i] =item
		end
		if i==1 then
			x,y= item.bg:getContentSize()
		end
	end
	local maxWidth = 0
	local swithPosX= 0
	local swithPosY =0
	for i=1,maxCount do
		local item = self.m_UiItems[i]
		swithPosX = x*(i-0.5)
		swithPosY= y/2
		maxWidth = x*i
		item:setPositionX(swithPosX)
		-- item:setPositionY(swithPosY)
	end
	self:setContentSize(maxWidth, y)

	local halfCount = math.ceil(chapterCount/2)--不小于x的最大整数
	local posX = -x*(maxCount-1)-35
	local posLeft = chapter:getId()%chapterCount
	if posLeft>halfCount or posLeft ==0  then
		posX = posX-400
	end
	self.m_posX = posX
end

function This:refreshScroolViewPos()
	if self.m_posX and self.m_scrollView then
		local scrollview = self.m_scrollView
		local innerHeight = scrollview:getInnerContainerHeight()
    	local topY = scrollview:getContentHeight() - innerHeight
		scrollview:setInnerContainerPosition(self.m_posX, topY)
	end
end

function This:createItem()
	local node = cc.Node:createFromCSFile(fileNames[1])
	-- node:fitScreenY()
	return node
end


function This:refresh()
	self:initBg()
	self:refreshScroolViewPos()
	self:refreshChapterItem()
end

function This:chapterAction( chapterView,haveAction)

	local time = 0.5
	local yuan = chapterView.yuan
	local chapterNode = chapterView.chapterNode

	if not self.m_yuanX then
		local x,y = yuan:getPosition()
		self.m_yuanX = x
		self.m_yuanY = y
		x,y = chapterNode:getPosition()
		self.m_chapterNodeX = x
		self.m_chapterNodeY = y
	end
	if self.m_yuanX then
		yuan:setPosition(self.m_yuanX, self.m_yuanY)
		chapterNode:setPosition(self.m_chapterNodeX, self.m_chapterNodeY)
	end
	if haveAction then
		local seqence1 = cc.Sequence(cc.MoveBy(time,0,-10),cc.MoveBy(time,0,10))
		chapterNode:runAction(cc.RepeatForever(seqence1))

		local seqence2 = cc.Sequence(cc.ScaleTo(time,0.8),cc.ScaleTo(time,1))
		yuan:runAction(cc.RepeatForever(seqence2))
	else
		chapterNode:stopAllActions()
		yuan:stopAllActions()
	end
end

function This:refreshChapterItem()
	for index,chapterView in ipairs(self.m_chapters) do
		local chapterItem = ChallengeStageManager:getChapter(index)
		if chapterItem then
			local curentStar = chapterItem:getTotalStar()
			local maxStar = chapterItem:getMaxStar()

			chapterView.chapterNode.hero:setImage(chapterItem:getImageFile())
			chapterView.chapterNode.zhangjie:setString(chapterItem:getId())
			chapterView.chapterNode.number:setString(string.format("%d/%d",curentStar,maxStar))
			chapterView:setVisible(chapterItem:isLock())
			if not chapterView.hongdian then
				local parent = chapterView.chapterNode.hero
				local sp = cc.Sprite:create("LoginScene/baoman.png")
				parent:addChild(sp)
			    chapterView.hongdian = sp
				local w, h = parent:getContentSize()
		        sp:setPosition(w-5, h-5)
			end
			chapterView.hongdian:setVisible(chapterItem:isCanReward())
		else
			chapterView:setVisible(false)
		end
	end
end
