-- @Author: zheng
-- @Date:   2018-08-15 11:07:39
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-08-16 15:10:24
-- @File Name: ChallengeRewarditemView.lua

local ChallengeStageManager = require("Stage.ChallengeStageManager")
local GameChallengeChapter = require("Stage.GameChallengeChapter")
local ItemNode = require("UI.ItemNode")

local This , super = class(...,"NodeUI")

This.fileName = "MenuScene/NodeUI/TeamWarRewardList.csb"

function This:initData(info)
	self.m_inited = false
    self.m_config = info.itemData
	self.m_chapterId = info.chapterId
    local w , h = self.beijing.bg:getContentSize()
    self:setContentSize(w,h)
    self.scrollview:setDirection("horizontal")
    self.scrollview.m_direction = "right"
end

function This:initNode()
    self:refreshItemUI()
    self:refresh()
end

function This:refresh( data )
	if data then
        self.m_config = data
    end
    self.number:setString(self.m_config.starLv)
	self:setItemEnable(self:isOpening())
    self:refreshItemUI()
end

function This:refreshItemUI()
    self.scrollview:removeAllItems()
    local function createItemView(model)
        local node = ItemNode:create(model[1],model[2])
        node:setAnchorPoint(0.5,0.5)
        return node
    end
    local rewardList = self.m_config.reward
    for i,itemConfig in ipairs(rewardList) do
        local item = createItemView(itemConfig)
        local scale = 0.7
        item:setScale(scale)
        local w , h = item:getContentSize()
        self.scrollview:pushBackCustomItem(item,w*scale,h*scale,0.5)
    end
end

function This:initTouchEvent()
    if not self.m_inited then
        self.button:setTouch({
            getRect = function (touch)
                local rect = touch.sizeNode:getBoundingBox()
                return rect
            end,
            onEnded = function()
                self:onPickUpBtnClick()
            end,
            isContainer = true,
            isDynamic = true,
            isScale = false,
        })  
        self.m_inited = true
        self.scrollview:setSwallowTouch(false)
    end
end

function This:onPickUpBtnClick()
    if self:isOpening() then
        ChallengeStageManager:onReceiveStarRewardRequest(1,self.m_config.id,self.m_chapterId)
    end
end


function This:setItemEnable(enble)
    if self:isLocked() then
        self.button:setGrayShader()
        self.button:setVisible(true)
        
        if self.yilingqu then 
            self.yilingqu:setVisible(false)
        end
    else
        if enble then
            self.button:clearShader()
        else
            self.button:setVisible(false)
        end

        if self.yilingqu then 
            self.yilingqu:setVisible(not enble)
        end
    end
end

function This:isOpening()
    return self.m_config and self.m_config.state == ChapterBoxState.kOpen
end

function This:isLocked()
    return self.m_config and self.m_config.state == ChapterBoxState.kLock
end