-- @Author: zheng
-- @Date:   2018-08-13 15:37:02
-- @File Name: ChallengeStageManager.lua

local This = class(...)

local ChallengeStageStarsConfig = require("Metadatas.ChallengeStageStarsConfig")
local ChallengeStageDailyboxConfig = require("Metadatas.ChallengeStageDailyboxConfig")

local StageChapterConfig = require("Metadatas.ChallengeStageChapterConfig")
local StageConfig = require("Metadatas.ChallengeStageStageConfig")
local GameChapter = require("Stage.GameChallengeChapter")
local GameStage = require("Stage.GameChallengeStage")
local TeamCampaignConfig = require("Metadatas.TeamCampaignConfig")

local Socket = require("Net.Socket")         
local EventDispatcher = require("EventDispatcher")
local Serialize = require("Serialize.Serialize")
local GameUtil = require("Util.GameUtil")

local BattleDataManager = require("Battle.BattleDataManager")
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")

ChallengeBoxState = {
	kLock = 1,
	kOpen = 2,
	kClose = 0
}
local strList ={
	-- cost ="是否花费",
    cost1 = Serialize[64168],
	costLimit =Serialize[12409],
}
function This:init()
	self.m_chapter = {}
	self.m_stage = {}
	for id , config in pairs(StageChapterConfig) do
		self.m_chapter[id] = GameChapter:create(id)
	end

	for id , chapter in pairs(self.m_chapter) do
		local preChapter = self.m_chapter[id - 1]
		if preChapter then
			chapter:setPreChapter(preChapter)
		end
		local nexChapter = self.m_chapter[id + 1]
		if nexChapter then
			chapter:setNexChapter(nexChapter)
		end
	end
	
	for id , config in pairs(StageConfig) do
		local stage = GameStage:create(id)
		local chapter = self.m_chapter[config.chapter_id]
		chapter:addStage(stage)
		self.m_stage[id] = stage
	end

	for id , stage in pairs(self.m_stage) do
		local preStage = self.m_stage[id-1]
		stage:setPreStage(preStage)
	end

	for id , chapter in pairs(self.m_chapter) do
		chapter:onStageLoadFinish()
	end

	self.m_isCanReward = false --是否有可以领取的宝箱
	self.m_challengeHistory ={}--已经领取的宝箱列表
	self.m_challengeRewardList ={}
	for index, config in ipairs(ChallengeStageStarsConfig) do
		self.m_challengeRewardList[index] = {
			id = config.id, 
			state = ChallengeBoxState.kLock, 
			reward = config.reward, 
			starLv = config.reward_starlv
		}
	end

	self.m_minChallengeStarLv =0 --可领取的最大小的奖励星星数
	self.m_challengeTimes = 0 --剩余挑战的次数

    self.m_challengedTimesIsCanReward = false --挑战次数奖励列表是否有可以领取的宝箱
    self.m_challengedTimes = 0 --已挑战的次数
    self.m_NextchallengedTimes = 0
    self.m_challengedTimesHistory ={}--已经领取的挑战次数宝箱列表
    self.m_challengedTimesRewardList ={}--挑战次数奖励列表
	for index, config in ipairs(ChallengeStageDailyboxConfig) do
		self.m_challengedTimesRewardList[index] = {
			id = config.id, 
			state = ChallengeBoxState.kLock, 
			reward = config.reward, 
			challengetime = config.daily_times
		}
	end


	self.m_maxChallengeTimes =0
	self.m_resetTimes = 0 --剩余购买次数
	self.m_emBattleHeros = {}
	self.m_lastChapter = self.m_chapter[1]
	self.m_nextCanBattleStage = self.m_stage[1]
	self.m_enterStage = 0
	self.m_observers = {}
	self.m_battleResult = {}
    self.m_isEnterRobber = false
	self:initEventListener()
end


function This:initEventListener()
	EventDispatcher:addListener(self, 12600, self.onGetStateInfoNotify)--获取挑战关卡信息
	EventDispatcher:addListener(self, 12601, self.onChallengeStateReply)--挑战关卡
	EventDispatcher:addListener(self, 12603, self.onReceiveStarRewardReply)--领取奖励
	EventDispatcher:addListener(self, 12602, self.onEmbattleHerosReply)--设置阵容
	EventDispatcher:addListener(self, 12604, self.onSkipBattleReply)--跳过战斗
 
	EventDispatcher:addListener(self, 12605, self.onStageSweepReply)--关卡扫荡
	EventDispatcher:addListener(self, 12606, self.onStageResetReply)--挑战次数重置
    EventDispatcher:addListener(self, 12607, self.onReplyChallngeTimesReward)--领取挑战次数奖励
end

function This:removeEventListener()
	EventDispatcher:removeListener(self, 12600)--获取挑战关卡信息
	EventDispatcher:removeListener(self, 12601)--挑战关卡
	EventDispatcher:removeListener(self, 12603)--领取奖励
	EventDispatcher:removeListener(self, 12602)--设置阵容
	EventDispatcher:removeListener(self, 12604)--跳过战斗
	EventDispatcher:removeListener(self, 12605)--关卡扫荡
	EventDispatcher:removeListener(self, 12606)--挑战次数重置
    EventDispatcher:removeListener(self, 12607)--领取挑战次数奖励
end

function This:addObserver(o)
	for i , observer in pairs(self.m_observers) do
		if observer == o then
			return 
		end
	end
	table.insert(self.m_observers,o)
end

function This:removeObserver(o)
	for i , observer in pairs(self.m_observers) do
		if observer == o then
			table.remove(self.m_observers,i)
			return
		end
	end
end

function This:notifyObservers(funcName,...)
	local observers = {}
	for i , o in pairs(self.m_observers) do
		table.insert(observers,o)
	end
	
	for i, observer in pairs(observers) do
		if observer[funcName] then
			observer[funcName](observer,...)
		end
	end
end

function This:isCanReward()
	return self.m_isCanReward
end

function This:challengedTimesIsCanReward()
	return self.m_challengedTimesIsCanReward
end

--可领取的最小的奖励星星数
function This:getMinChallengeStarLv(  )
	return self.m_minChallengeStarLv
end

function This:getRewardList( )
	return GameUtil:shallowCopy(self.m_challengeRewardList)
end

function This:getChallengeTimeRewardList( )
	return GameUtil:shallowCopy(self.m_challengedTimesRewardList)
end
--所有章节总的星星数量
function This:getTotalStar(  )
	local starCount =0
	for id , chapter in pairs(self.m_chapter) do
		starCount = starCount +chapter:getTotalStar()
	end
	return starCount
end

--刷新章节所有数据
function This:refreChapterData(chapterList)
	if chapterList and table.count(chapterList) then
		for i , info in ipairs(chapterList) do
			local chapter = self:getChapter(info.chapter_id)
			for i , rewardLv in pairs(info.cap_rew_list) do--//章节星级领奖记录
				chapter:setStarRewardReceived(rewardLv,true)
			end
			for i , stageInfo in pairs(info.stage_list) do--章节星级
				local stage = self:getStage(stageInfo.stage_id)
				stage:setStarLv(stageInfo.star)
			end
			chapter:onStageLoadFinish()
		end
	end
end

--刷新总的星级奖励
function This:refreshChallengeReward()
	local rewardHistory = self.m_challengeHistory
	table.sort(rewardHistory,function(a,b)
        return a < b
    end)
	local starCount = self:getTotalStar(  )
    self.m_isCanReward = false
    local maxCanRewardIndex =0
    local minCanRewardIndex = 1
    for i,item in ipairs(ChallengeStageStarsConfig) do
        if item and item.reward_starlv<= starCount then
            maxCanRewardIndex = i
            self.m_isCanReward = true
        end
    end
    for index=1,maxCanRewardIndex do
    	self.m_challengeRewardList[index].state = ChallengeBoxState.kOpen
    end
    for index1=1,#ChallengeStageStarsConfig do
    	local isFind = table.find(rewardHistory, function (index, value)
    	    return index1 == value
    	end)
    	if not isFind then
    	    minCanRewardIndex = index1
    	    break
    	end
    end
    for _,configIndex in ipairs(rewardHistory) do
    	if self.m_challengeRewardList[configIndex] then
    		self.m_challengeRewardList[configIndex].state = ChallengeBoxState.kClose
		elseif maxCanRewardIndex >= configIndex then
			self.m_isCanReward = true
			self.m_challengeRewardList[configIndex].state = ChallengeBoxState.kOpen
		else
			self.m_challengeRewardList[configIndex].state = ChallengeBoxState.kLock
    	end
    end

    if maxCanRewardIndex ==table.count(rewardHistory) then
    	self.m_isCanReward = false
    	if #ChallengeStageStarsConfig == maxCanRewardIndex then
    		minCanRewardIndex = maxCanRewardIndex
    	end
    end
    self.m_minChallengeStarLv = ChallengeStageStarsConfig[minCanRewardIndex].reward_starlv
end

function This:getChallengeTimes(  )
	return self.m_challengeTimes>0 ,self.m_challengeTimes,self.m_maxChallengeTimes
end

function This:getChallengedTimes()
	return self.m_challengedTimes
end

function This:getNextChallengedTimes()
    return self.m_NextchallengedTimes
end

--刷新挑战次数奖励
function This:refreshChallengedTimesReward()
    local rewardNum = 0
    self.m_challengedTimesIsCanReward = false
    for i,item in ipairs(ChallengeStageDailyboxConfig) do
        if self.m_challengedTimes >= item.daily_times then
             self.m_challengedTimesRewardList[i].state = ChallengeBoxState.kOpen
             
             rewardNum = rewardNum+1
        end
    end

    for _,configIndex in ipairs(self.m_challengedTimesHistory) do
        self.m_challengedTimesRewardList[configIndex].state = ChallengeBoxState.kClose
        rewardNum = rewardNum-1
    end
    if rewardNum>0 then
        self.m_challengedTimesIsCanReward = true
    end


    for i,item in ipairs(ChallengeStageDailyboxConfig) do
        if self.m_challengedTimes < item.daily_times then
            self.m_NextchallengedTimes = item.daily_times
            break
        end
        self.m_NextchallengedTimes = item.daily_times
    end
    for i,item in ipairs(ChallengeStageDailyboxConfig) do
        if self.m_challengedTimesRewardList[i].state == ChallengeBoxState.kOpen then
            self.m_NextchallengedTimes = item.daily_times
            break
        end
    end
end

function This:getSelectChapterId( )
	return 4
end

function This:getResetCost()
	local res =0
	local itemConfig = TeamCampaignConfig[4]
	local costs = itemConfig.change_cost or {}
	local resetCount = #costs
	if self.m_resetTimes>0 and self.m_resetTimes<=resetCount then
		local temResetTime = self.m_resetTimes-1
		local config = costs[resetCount-temResetTime]
		if config then
			return config
		end
	end
	return res
end

function This:resetBtnCallBack( )
	local canReset,resetTime = self:getResetTiem()
	if canReset then
		local diamond = self:getResetCost()
			-- g_scene:openDialog("Stage.ChallengeConfirmPanel", {
   --      fileName = "MenuScene/NodeUI/ActivityHappyRewardVoucher.csb",
   --      cost = diamond,
   --      -- notips = "ChallengeStageManager",
   --      text1 = strList.cost,
   --      descText = strList.cost1, 
   --      okHandler = function(build)
   --          self:onStageResetRequest(1)            
   --      end } )
    g_scene:openDialog("UI.CostConfirmPanel", {
        cost = diamond,
        desc = strList.cost1, 
        okHandler = function(build)
            self:onStageResetRequest(1)            
        end } )
	else
		g_scene:showTips(strList.costLimit)
	end
end

function This:getResetTiem(  )
	return self.m_resetTimes>0 ,self.m_resetTimes
end

--最大可能的章节
function This:maxChapter(  )
	local res = self.m_chapter[1]
	for i,chapter in ipairs(self.m_chapter) do
		if not chapter:isPassed() then
			res = chapter
			break
		end
	end
	local chapterCount = #self.m_chapter
	local lastChapter = self.m_chapter[chapterCount]
	if lastChapter:isPassed() then
		res = lastChapter
	end
	return res
end

function This:getLastChapter()
	return self.m_lastChapter
end

function This:setLastChapter(chapter)
	self.m_lastChapter = chapter or self.m_chapter[1]
end

function This:getStage(id)
	return self.m_stage[id]
end

function This:getChapter(id)
	return self.m_chapter[id]
end

function This:getChapters()
	return self.m_chapter
end

function This:passStage(stageId,starLv)
	local stage = self:getStage(stageId)
	stage:setStarLv(starLv)
	local chapter = self:getChapter(stage:getChapterId())
	chapter:updateTotoalStar()
	if self.m_nextCanBattleStage == stage then
		local nextStage = self:getStage(stageId + 1)
		if nextStage then
			self.m_nextCanBattleStage = nextStage
		end
	end
end

function This:clearAllStageDataTimes()
	for id , stage in pairs(self.m_stage) do
		stage:setChallengeTimes(0)
		stage:setResetTimes(0)
	end 
end

function This:isCanFreeVisitHero()
	for i , chapter in pairs(self.m_chapter) do
		local price = chapter:getVisitPriceBy()
		local opening = chapter:isPassAllStages()
		if opening and price == 0 then
			return true
		end
	end
	return false
end



function This:getEmbattleHero()
	return self.m_emBattleHeros
end

function This:isInEmbattle(heroId)
	for pos , id in pairs(self.m_emBattleHeros) do
		if id == heroId then
			return true
		end
	end
	return false
end

function This:unEmbattleHero(heroId)
	local result = {}
	for pos , id in pairs(self.m_emBattleHeros) do
		if heroId ~= id then
			table.insert(result,{pos = pos, heroId = id})
		end
	end
	self:onEmbattleHerosRequest(result)
end

function This:createHeroForEmbattle()
	local posTable = {}
	local heroTable = {}
	for pos , heroId in pairs(self.m_emBattleHeros) do
		local playerHero = HeroManager:getHero(heroId)
		local hero = Hero:create({
			heroId = heroId,
			level = playerHero:getLevel(),
			starLv = playerHero:getStarLv(),
			quality = playerHero:getQuality(),
			deputy = table.clone(playerHero:getDeputyWithPos())
		})
		posTable[pos] = hero
		heroTable[heroId] = hero
	end
	for heroId , hero in pairs(heroTable) do
		hero:addFetterWithHero(heroTable)
	end
	return posTable
end

function This:getNextBattleChapter()
	local chapterId = self.m_nextCanBattleStage:getChapterId()
	return self.m_chapter[chapterId]
end

function This:onBattleSkip(result)
	self:onSkipBattleRequest()
end

--挑战结果
function This:onBattleVictory()
	g_scene:openDialog("Stage.ChallengeVictoryPanel",{
		stageId = self.m_battleResult.stageId,
		starLv = self.m_battleResult.starLv,
		dropInfo = self.m_battleResult.item
	})
	BattleDataManager:removeObserver(self)
end

function This:onBattleFailed()
	g_scene:openDialog("Stage.ChallengeFailedPanel")
	BattleDataManager:removeObserver(self)
end

function This:onChallengeStageRequest(stageId)
	Socket:sendAndLock(12601,{stage_id = stageId})--挑战关卡
	self.m_curStageId = stageId
end

function This:onChallengeStateReply(message)
	if message.err_code == 0 then
		local stage = message.stage
		self.m_battleResult = {}
		self.m_battleResult.isWin = stage.win
		self.m_battleResult.starLv = stage.star
		self.m_battleResult.item = message.item

        self.m_challengedTimes = message.challenged_times or 0 --已挑战挑战的次数
		self.m_challengeTimes = message.challenge_times or 0 --剩余挑战的次数
		message.isWin = stage.win
		message.starLv = stage.star
		BattleDataManager:addObserver(self)
		self:passStage(stage.stage_id,stage.star)

		cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {mode = PlayMode.kWarSingle,bgId = StageConfig[stage.stage_id].bgId, msg = message})
		self:notifyObservers("onChallengeState")
	else
		g_scene:handleErrorCode(message.err_code)
	end
    self:refreshChallengedTimesReward()
end

function This:enterStage(stageId)
	if next(self.m_emBattleHeros) then
		self:onChallengeStageRequest(stageId)
	else
		g_scene:showTips(Serialize["add_embattle_hero"])
	end
end

--rewardLv 领取奖励索引
function This:onReceiveStarRewardRequest(op_type,rewardLv,chapterId)
	if rewardLv then
		Socket:sendAndLock(12603,{chapter_id = chapterId,index = rewardLv,op_type = op_type })--领取奖励
	end
end

function This:onReceiveStarRewardReply(message)
	if message.err_code == 0 then
		if message.op_type == 1 then
			local chapter = self:getChapter(message.chapter_id)
			chapter:setStarRewardReceived(message.index,true)
		elseif message.op_type == 2 then
			self.m_challengeHistory = message.total_rew_list or {}
			self:refreshChallengeReward()
		end
		g_scene:showReward(message.item_list)
		self:notifyObservers("onStarRewardChanged")
	else
		g_scene:handleErrorCode(message.errorCode)
	end
end

-- 领取挑战次数奖励
function This:onRequestGetChallengeTimesReward(Index)
    if Index then
        Socket:sendAndLock(12607,{index = Index})--领取奖励
    end
end
function This:onReplyChallngeTimesReward(message)
    if message.err_code == 0 then
	
		self.m_challengedTimesHistory = message.challenge_rew_list or {}
	
		self:refreshChallengedTimesReward()
		g_scene:showReward(message.item_list)
        self:notifyObservers("onStarRewardChanged")
	else
		g_scene:handleErrorCode(message.errorCode)
	end
end

function This:onEmbattleHerosRequest(heros)
	local result = {0,0,0,0,0,0}
	for i , info in pairs(heros) do
		local hero = HeroManager:getHero(info.heroId)
		result[info.pos] = hero:getUid()
	end
	Socket:sendAndLock(12602,{card_list = result})--设置阵容
end

function This:onEmbattleHerosReply(message)
	if message.err_code == 0 then
		self.m_emBattleHeros = {}
		for i , uid in ipairs(message.card_list) do
			if uid > 0 then
				local uhero = HeroManager:getHeroByUid(uid)
				self.m_emBattleHeros[i] = uhero:getId()
			end
		end
		self:notifyObservers("onEmbattleHeros")
	else
		g_scene:showTips(Serialize[message.err_code])
	end
end

function This:onSkipBattleRequest()
    if not self.m_isEnterRobber then
	    Socket:sendAndLock(12604,{})--跳过战斗
    end
end

function This:onSkipBattleReply(msg)
    if not self.m_isEnterRobber then
	    if msg.err_code == 0 then
		    if self.m_battleResult.isWin then
			    self:onBattleVictory()
		    else
			    self:onBattleFailed()
		    end
			BattleDataManager:clear()
	    else
		    g_scene:handleErrorCode(msg.err_code)
	    end
    end
end

function This:setEnterRobber( value )
    self.m_isEnterRobber = value 
end

function This:isEnterRobber()
    return self.m_isEnterRobber
end

function This:onStageSweepRequest(stageId)
	local res = {stage_id = stageId,times = 1}
	Socket:sendAndLock(12605, res)--关卡扫荡
end

function This:onStageSweepReply( msg )
    if msg.err_code == 0 then
    	if msg.award_all_list[1] and msg.award_all_list[1].award_list then
    		g_scene:showReward(msg.award_all_list[1].award_list)
    	end
    	self.m_challengeTimes = msg.challenge_time or self.m_challengeTimes  --剩余挑战的次数
        self.m_challengedTimes = msg.challenged_times or 0--已挑战的次数
		self:notifyObservers("onStageSweep", msg.award_all_list)
		self:notifyObservers("onRefreshChallengeTime")
        self:refreshChallengedTimesReward()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onStageResetRequest(op_type)
	Socket:sendAndLock(12606, {op_type = op_type})--挑战次数重置
end

function This:onStageResetReply( msg )
    if msg.err_code == 0 then
    	self.m_challengeTimes = msg.challenge_times or 0 --剩余挑战的次数
		self.m_resetTimes = msg.reset_times or 0 --剩余购买次数
		self:notifyObservers("onRefreshChallengeTime")
        self:notifyObservers("onStageReset")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetStateInfoRequest()
	Socket:sendAndLock(12600, {})--获取挑战关卡信息
end

function This:onGetStateInfoNotify(msg)
	self:refreChapterData(msg.chapter_list)
	self.m_challengeTimes = msg.challenge_times or 0 --剩余挑战的次数
	self.m_resetTimes = msg.reset_times or 0 --剩余购买次数
	self.m_maxChallengeTimes = msg.max_times or 0 --最大挑战的次数

    self.m_challengedTimes = msg.challenged_times or 0--已挑战的次数
    self.m_challengedTimesHistory = msg.challenge_rew_list or {}--已经领取的挑战次数宝箱列表
    
	for i, stage in pairs(self.m_stage) do--可以挑战的最大stage
		local preStage = stage:getPreStage()
		if preStage and preStage:getStarLv() <= 0 then
			break
		end

		if not self.m_nextCanBattleStage or not preStage then
			self.m_nextCanBattleStage = stage
		elseif self.m_nextCanBattleStage:getId() < stage:getId() then 
			self.m_nextCanBattleStage = stage
		end
	end

	for i , uid in ipairs(msg.card_list) do
		if uid > 0 then
			local hero = HeroManager:getHeroByUid(uid)
			self.m_emBattleHeros[i] = hero:getId()
		end
	end
	self.m_challengeHistory = msg.total_rew_list or {}
	self:refreshChallengeReward()
	self:notifyObservers("OnStateInfo", msg.award_all_list)
    self:refreshChallengedTimesReward()
end
-----------------------------------------------------------------
