--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local StageManager = require("Stage.GameStageManager")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local StageHeroItemView = require("Stage.StageHeroItemView")
local HeroConfig = require("Metadatas.HeroConfig")
local ListLayer = require("UI.ListLayer")
local GameStageManager = require("Stage.GameStageManager")

function This:initNode(info)
	self.m_stageId = info.stageId

	self.m_heros = {}
	self.m_fetters = {}
	self.m_fetterInfo = {}

	local fileName = info.file or "GameScene/NodeUI/LineUp.csb"
	self.m_csNode = cc.Node:createFromCSFile(fileName)
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

    if self.m_csNode.text100 then
        local timeLabelVisible = self.m_stageId and self.m_stageId > 0
        self.m_csNode.text100:setVisible(not timeLabelVisible)
        self.m_csNode.time:setVisible(not timeLabelVisible)
        self.m_csNode.diban2:setVisible(not timeLabelVisible)
    end

	self:initBaseInfo()
    self:initListLayer(info)

	self:initItems()
end

function This:initListLayer(info)
	if AppText:isChinese() then
		self.m_listLayer = ListLayer:create({
			itemWidth = info.itemWidth or 90,
			itemHeight = info.itemHeight or 30,
			frameCount = info.frameCount or 1,
			spaceY = info.spaceY or 5,
			column = AppText:isChinese() and (info.column or 2) or 1,
	        scrollView = self.m_csNode.FetterScrollView,
		})
	else
		local itemWidth = self.m_csNode.FetterScrollView:getContentSize()
		self.m_listLayer = ListLayer:create({
			itemWidth = itemWidth,
			itemHeight = 30,
			frameCount = 1,
			spaceX = 0,
			spaceY = info.spaceY or 5,
			column = 1,
	        scrollView = self.m_csNode.FetterScrollView,
		})
	end

	local w , h = self.m_listLayer:getContentSize()
	self.m_listLayer:setAnchorPoint(0.5, 0)
	self.m_csNode.FetterScrollView:pushBackCustomItem(self.m_listLayer,w,h,0);
end

function This:getHeroSource()
    return StageManager:getEmbattleHero()
end

function This:initItems()
	local embattleHeros = self:getHeroSource()
	for pos , heroId in pairs(embattleHeros) do
		self:addHero(pos,heroId)
	end	
end

function This:onEnter()
	super.onEnter(self)
	GameStageManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	GameStageManager:removeObserver(self)
end

function This:initBaseInfo()
	self.m_csNode.helpbutton:setVisible(false)
	self.m_csNode.bg3.tagettext:setString(Serialize[64171])
	local stage = StageManager:getStage(self.m_stageId,StageManager:getStageType())
	self.m_csNode.bg3.taget:setString(stage:getName())
	self.m_csNode.bg3.name:setString(GamePlayer:getRoleName())
	
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
	self.m_csNode.bg3.cost:setString(stage:getPowerCost())
end

function This:refreshBaseInfo()
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
end

function This:initTouchEvent()
	for i = 1 , 6 do
		self.m_csNode.bg2["Cardbg"..i].need:setTouch({
			onEnded = function()
				if self.m_heros[i] then
					return
				end

				if self.m_subView then
					return
				else
	        		self.m_subView = true
				end

				local panel = self:openSelectPanel()
				panel:setSelectHandler(function(heroId)
					self:addHero(i,heroId)
					self:embattle(true)
				end)
		        panel:addCloseCallback(function ()
		            self.m_subView = nil
		        end)
			end,
			isScale = false
		})
	end

	self.m_csNode.automaticbutton:setTouchEnded(function()
		self:onEmbattleOnceKey()
	end)

	self.m_csNode.Battlebutton:setTouchEnded(function()
		self:onConfirmBtnClick()
	end)

	self.m_csNode.close:setTouchEnded(function()
		self:onCloseBtnClick()
	end)
	self.m_csNode.helpbutton:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text=Serialize["embattle_help_tips"],node = self.m_csNode.helpbutton})
	end)
end

function This:openSelectPanel()
	local haveSelected = {}
	for i , info in pairs(self.m_heros) do
		haveSelected[info.heroId] = true
	end
	local panel = self:openDialog("Stage.StageSelectHeroPanel",{haveSelected = haveSelected})
	return panel
end

function This:getTotalHp()
	local total = 0
	for i , info in pairs(self.m_heros) do
		local hero = HeroManager:getHero(info.heroId)
		total = total + hero:getAttr(AttrType.kMaxHp)
	end
	return total
end

function This:refreshFetterInfo()
	local tmp = {}
	for i , list in pairs(self.m_fetters) do
		for i , fetter in pairs(list) do
			if fetter:isActive() then
				local fetterId = fetter:getId()
				tmp[fetterId] = fetter
			end
		end
	end

	for i = #self.m_fetterInfo , 1 , -1 do
		local info = self.m_fetterInfo[i]
		if tmp[info.id] == nil then
            print("刪除緣分" .. info.id, "index:" .. i)
			self.m_listLayer:removeNode(i)
			table.remove(self.m_fetterInfo,i)
		end
	end

	for id , fetter in pairs(tmp) do
		local isFind = false
		for j , info in ipairs(self.m_fetterInfo) do
			if info.id == id then
				isFind = true
				break
			end 
		end
		if isFind == false then
			local node = cc.Label:createWithTTF("Fonts/Normal.ttf",fetter:getName(), AppText:isChinese() and 16 or 15)
			node:setColor(253,148,85)
			node:setStringWithColor(string.format( "<New5>%s",fetter:getName()))
            print("增加緣分" .. id, "index:")
			table.insert(self.m_fetterInfo,{id = id,node = node})
			self.m_listLayer:pushNode(node)
		end
	end
end

function This:getHeroView(heroId)
	local hero = HeroManager:getHero(heroId)
	local maxHp = hero:getAttr(AttrType.kMaxHp)
	local node = StageHeroItemView:create(heroId,{
		maxHp = maxHp,
		hp = maxHp,
	})
	return node
end

function This:refreshHeroView(view,heroId)
	local hero = HeroManager:getHero(heroId)
	view:setModel(hero)
end

function This:addHeroView(heroId,pos)
	local node = self:getHeroView(heroId)
	node:setAnchorPoint(0.61,0.51)
	local w , h = self.m_csNode.bg2["Cardbg"..pos]:getContentSize()
	node:setPosition(w/2,h/2)

	self.m_csNode.bg2["Cardbg"..pos]:addChild(node)
	node:setCancelHandler(function(heroId)
		self:removeHero(pos,heroId)
		self:embattle(true)
	end)
	self.m_heros[pos].node = node
end

function This:addHero(pos,heroId)
	for id , list in pairs(self.m_fetters) do
		for i , fetter in pairs(list) do
			fetter:addHero(heroId)
		end
	end
	self.m_fetters[pos] = {}
	for i , fetterId in pairs( HeroConfig[heroId].fetter) do
		local fetter = HeroManager:createFetter(heroId,fetterId)
		for j , info in pairs(self.m_heros) do
			fetter:addHero(info.heroId)
		end
		table.insert(self.m_fetters[pos],fetter)
	end
	
	self.m_heros[pos] = self.m_heros[pos] or {}
	self.m_heros[pos].heroId = heroId

	self:addHeroView(heroId, pos)
	self:refreshFetterInfo()
	self:refreshBaseInfo()
end

function This:removeHero(pos,heroId)
	self.m_fetters[pos] = nil
	for i , list in pairs(self.m_fetters) do
		for id , fetter in pairs(list) do
			fetter:removeHero(heroId)
		end
	end
	if self.m_heros[pos].node then
		self.m_heros[pos].node:removeFromParent()
	end
	self.m_heros[pos] = nil
	self:refreshFetterInfo()
	self:refreshBaseInfo()
end

function This:updateHero(pos,heroId)
	assert(self.m_heros[pos] and self.m_heros[pos].heroId)
	if self.m_heros[pos].heroId == heroId then
		return
	end
	
	local oldHeroId = self.m_heros[pos].heroId
	self.m_heros[pos].heroId = nil
	self.m_fetters[pos] = {}

	for i , list in pairs(self.m_fetters) do
		for id , fetter in pairs(list) do
			fetter:removeHero(oldHeroId)
			fetter:addHero(heroId)
		end
	end

	for i , fetterId in pairs( HeroConfig[heroId].fetter) do
		local fetter = HeroManager:createFetter(heroId,fetterId)
		for j , info in pairs(self.m_heros) do
			if info.heroId then
				fetter:addHero(info.heroId)
			end
		end
		table.insert(self.m_fetters[pos],fetter)
	end

	self.m_heros[pos].heroId = heroId
	if self.m_heros[pos].node then
		self:refreshHeroView(self.m_heros[pos].node,heroId)
	end
	self:refreshFetterInfo()
	self:refreshBaseInfo()
end

function This:getAllHeroList()
	local heros = HeroManager:getAllHeroGotWithSort(function(hero) 
		if hero:isDeputy() then
			return false
		end
		return true
	end)
	table.sort(heros,function(a,b)
		return a:getFightPower() > b:getFightPower()
	end)
	return heros
end

function This:onEmbattleOnceKey()
	local heros = self:getAllHeroList()
	local maxCount = math.min(#heros,6)
	self.m_fetters = {}
	for i , info in pairs(self.m_heros) do
		info.heroId = nil
		if info.node then
			info.node:removeFromParent()
			info.node = nil
		end
	end
	self.m_heros = {}
	for i = 1 , maxCount do
		self:addHero(i,heros[i]:getId())
	end
	self:embattle(true)
end

function This:gotoBattle(heroList, isOnceKey)
	local stage = StageManager:getStage(self.m_stageId,StageManager:getStageType())
	if not stage or stage:getLeftChallengeTimes() > 0 then
		if not isOnceKey and #heroList < 6 then
			local panel = g_scene:openDialog("UI.ConfirmPanel",{text = Serialize[64179]})
			panel:setOkHandler(function()
				StageManager:onEmbattleHerosRequest(heroList)
			end)				
		else
			StageManager:onEmbattleHerosRequest(heroList)
		end
	else
		g_scene:showTips(Serialize[64310])
	end
end

local isOnceKey = false
function This:embattle(onceKey)
	isOnceKey = onceKey
	local tmp = {}
	for i , info in pairs(self.m_heros) do
		table.insert(tmp,{pos = i,heroId = info.heroId})
	end
	self:gotoBattle(tmp,isOnceKey)
end

function This:onConfirmBtnClick()
	self:embattle(false)
end

function This:onCloseBtnClick()
    self:close()
end

function This:onEmbattleHeros()
	if not isOnceKey then
		if next(self.m_heros) then
			StageManager:enterStage(self.m_stageId)
		else
			g_scene:showTips(Serialize["add_embattle_hero"])
		end
	end
end

--endregion
