-- @Author: zheng
-- @Date:   2018-08-13 15:37:02
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 17:41:54
-- @File Name: GameChallengeChapter.lua


local This = class(...)
local GamePlayer = require("Player.GamePlayer")
local ChallengeStageChapterConfig = require("Metadatas.ChallengeStageChapterConfig")
local DropConfig = require("Metadatas.DropConfig")
ChapterBoxState = {
	kLock = 1,
	kOpen = 2,
	kClose = 0
}
local strList ={
    chapter =Serialize[60857],
}

function This:create(...)
	local chapter = setmetatable({},self)
	chapter:init(...)
	return chapter
end

function This:init(id)
	self.m_id = id
	self.m_stages = {}
	self.m_config = ChallengeStageChapterConfig[id]
	self.m_totalStar = 0
	self.m_starReward = {false,false,false}
	self.m_maxStar = 0
	self.m_preChapter = nil
	self.m_nexChapter = nil
end

function This:clear()
	self.m_totalStar = 0
	for i , _ in pairs(self.m_starReward) do
		self.m_starReward[i] = false
	end
end

function This:getImageFile()
	local GameUtil = require("Util.GameUtil")
	return GameUtil:getItemIcon(self.m_config.image)
end

function This:addStage(stage)
	table.insert(self.m_stages,stage)
	self.m_maxStar = self.m_maxStar + 3
end

function This:getId()
	return self.m_id
end

function This:getStages()
	return self.m_stages
end

function This:isPassed()
    return self:getMaxStar() == self:getTotalStar()
end

--是否已经解锁
function This:isLock()
	if self.m_preChapter then
		return self.m_preChapter:isPassed()
	end
    return GamePlayer:getLevel() >= self.m_config.level_limit
end

function This:getLockLevel()
    return self.m_config.level_limit
end

function This:setPreChapter(chapter)
	self.m_preChapter = chapter
end

function This:getPreChapter()
	return self.m_preChapter
end

function This:setNexChapter(chapter)
	self.m_nexChapter = chapter
end

function This:getNexChapter()
	return self.m_nexChapter
end

function This:onStageLoadFinish()
	table.sort(self.m_stages,function(s1,s2)
		return s1:getId() < s2:getId()
	end)
	self:updateTotoalStar()
end

function This:getMaxStar()
	return self.m_maxStar
end

function This:setTotoalStar(star)
	self.m_totalStar = star
end

function This:getTotalStar()
	return self.m_totalStar
end

function This:getTitle()
	return string.format(strList.chapter,self.m_id) --"第" .. tostring(self.m_id) .. "章"
end

function This:isPassAllStages()
	local stage = self.m_stages[#self.m_stages]
	return stage:getStarLv() > 0
end

function This:updateTotoalStar()
	local total = 0
	for i , stage in pairs(self.m_stages) do
		total = total + stage:getStarLv()
	end
	self.m_totalStar = total
end


function This:getMaxRewards(  )
	return #self.m_config.reward_starlv
end

--获取某一个奖励星级限制 rewardLv，1,2,3
function This:getRewardStarLimit(rewardLv)
	return self.m_config.reward_starlv[rewardLv]
end

--获取达到某个星级获取的奖励
function This:getRewardDrops(rewardLv)
	local dropId = self.m_config.drop_id[rewardLv]
	local drops = DropConfig[dropId]
	local result = {}
	for i , info in pairs(drops.drop_info) do
		table.insert(result,{info[1],info[2]})
	end
	return result
end

--检测某个星级的奖励是否已经领取
function This:isStarRewardReceived(rewardLv)
	return self.m_starReward[rewardLv]
end

--设置某个奖励是否领取
function This:setStarRewardReceived(rewardLv,isReceived)
	self.m_starReward[rewardLv] = isReceived
end

--判断当前星级是否可以领取某个奖励
function This:starIsEnough(rewardLv)
	return self.m_totalStar >= self.m_config.reward_starlv[rewardLv]
end

--获取当前是否有可以领取的奖励
function This:isCanReward()
	local res = false
	local maxrewards = self:getMaxRewards()
	for rewardLv=1,maxrewards do
		local starIsEnough = self:starIsEnough(rewardLv)
		local isStarRewardReceived = self:isStarRewardReceived(rewardLv)
		if starIsEnough and not isStarRewardReceived then
			res = true
			break
		end
	end
	return res
end

--获取宝箱奖励
function This:getRewardList()
	local res ={}
	local maxrewards = self:getMaxRewards()
	for rewardLv=1,maxrewards do
		local starIsEnough = self:starIsEnough(rewardLv)
		local isStarRewardReceived = self:isStarRewardReceived(rewardLv)
		local starLv = self:getRewardStarLimit(rewardLv)
		local reward = self:getRewardDrops(rewardLv)
		local state = ChapterBoxState.kLock
		if starIsEnough then
			state = ChapterBoxState.kOpen
		end
		if isStarRewardReceived then
			state = ChapterBoxState.kClose
		end
		res[rewardLv]={
		id = rewardLv,
		starLv =  starLv,
		reward = reward,
		state = state,
	}
	end
	return res
end


