--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local StageManager = require("Stage.GameStageManager")
local VipManager = require("Vip.VipManager")
local PrivilegeManager = require("Vip.PrivilegeManager")
local GamePlayer = require("Player.GamePlayer")
local ItemNode = require("UI.ItemNode")

function This:initNode(stage, from)
	self.m_stage = stage
	print("打开章节" .. tostring(stage:getId()))
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/StageChallenge.csb")
	self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
	self:addChild(self.m_csNode)
	self.m_csNode.challengeBG.title.text:setString(self.m_stage:getName())
	
	local star = self.m_stage:getStarLv()
	for i = 1 , 3 do
		local key = "star1"..i
		self.m_csNode.challengeBG[key]:setVisible(star >= i)
	end
	self.m_csNode.descBG.text:setString(self.m_stage:getPowerCost())
	self:initItems()

	self:refreshSweepButton()
end

function This:initItems()
	local drops = self.m_stage:getDrops()
	local initX = 0
	local count = table.count(drops)
	local scale = count > 4 and 4 / count or 1
	for i , info in pairs(drops) do
		local node = self:getItemNode(info.id,info.num)
		node:setScale(scale)
		node:setPositionX(initX)
		self.m_csNode.descBG.itemList:addChild(node)
		initX = initX + node:getContentWidth()* scale  + 10 * scale
	end
end

function This:getItemNode(itemId,itemNum)
	local node = ItemNode:create(itemId,itemNum)
	node:setCascadeOpacityEnabled(true)
	node:setAnchorPoint(0.5,0.5)
	return node
end

function This:initTouchEvent()
	self.m_csNode.challengeBG.choose:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.array:setTouchEnded(function()
		g_scene:openDialog("Stage.EmbattleHeroPanel",{stageId = self.m_stage:getId()})
	end)

	self:initSweepTouchEvent()
end

function This:onEnter( )
    super.onEnter(self)
    GamePlayer:addObserver(self)
    VipManager:addObserver(self)
    StageManager:addObserver(self)
end

function This:onExit( )
    StageManager:removeObserver(self)
    VipManager:removeObserver(self)
    GamePlayer:removeObserver(self)
    super.onExit(self)
end

-----=================================================================================------
local leftChallengeTimes, totalChallengeTime
local sweepTimesInView
local hasSweepUnlock, sweepPrivVipModel
local canSweep, hasEnoughPower
local totalRestTime
local leftResetTime
function This:refreshSweepData()
    leftChallengeTimes, totalChallengeTime = self.m_stage:getLeftChallengeTimes()
    print( "剩余次数:" .. tostring(leftChallengeTimes))
    sweepTimesInView = 0
	hasSweepUnlock, sweepPrivVipModel = PrivilegeManager:hasPrivilege(Privilege.kStageSweepUnlock)
    canSweep =  self.m_stage:isPassed() and hasSweepUnlock
	totalRestTime = PrivilegeManager:getPrivilegeNumBy(Privilege.kStageFightResetTimes)
	leftResetTime = totalRestTime - self.m_stage:getResetTimes()
    print( "剩余重置次数:" .. tostring(leftResetTime))
    local curValue, cost = GamePlayer:getPower() , self.m_stage:getPowerCost()
    hasEnoughPower = curValue >= cost

    if leftChallengeTimes > 0 then
	    if hasEnoughPower then
	    	sweepTimesInView = math.floor(curValue / cost)
	    	sweepTimesInView = math.min(sweepTimesInView, leftChallengeTimes)
	    	sweepTimesInView = math.min(sweepTimesInView, kSweepMaxTimesOnce)
	    else
            canSweep = false
			sweepTimesInView = math.min(leftChallengeTimes, kSweepMaxTimesOnce)
	    end
	end
end

function This:refreshSweepButtonUI()
    if self.m_csNode.descBG then
	    self.m_csNode.descBG.times:setString(tostring(leftChallengeTimes) .. "/" .. tostring(totalChallengeTime))
    end

    if leftChallengeTimes > 0 then
	    if not canSweep then
	    	self.m_csNode.sweep:setGrayShader()
        else
	    	self.m_csNode.sweep:clearShader()
	    end	
		self.m_csNode.sweep:setVisible(true)
		self.m_csNode.reset:setVisible(false)
		
		self.m_csNode.sweep.text:setString(Serialize[64319] .. tostring(sweepTimesInView) .. Serialize[60532])

	else
		self.m_csNode.sweep:setVisible(false)
		self.m_csNode.reset:setVisible(true)
	end

	if leftResetTime <= 0 then
		self.m_csNode.reset:setGrayShader()
	else
		self.m_csNode.reset:clearShader()
	end 
end

function This:afterStageSweep( rewardList )
    g_scene:openDialog("Stage.StageSweepResultPanel", {stage = self.m_stage, rewardList = rewardList}) 
    self:close()  
end

function This:refreshSweepButton()
	self:refreshSweepData()
	self:refreshSweepButtonUI()
end

function This:initSweepTouchEvent()
	local resetFunc = function()
		if leftChallengeTimes <= 0 then
            local panel = g_scene:openDialog("UI.CostConfirmPanel")
            if leftResetTime > 0 then
            	panel:setCost(self.m_stage:getResetCost())
            	panel:setString(Serialize[64320])
            	panel:setTimeString(Serialize[64321] .. tostring(leftResetTime) .. "/" .. tostring(totalRestTime), {r=255, g=0, b=0} )
	            panel:setOkHandler(function()
	                StageManager:onStageResetRequest(self.m_stage:getId())
	            end)
        	else
        		panel:setSimpleString(Serialize[64322])
        		if not PrivilegeManager:isFullVipMember() then
		            panel:setOkHandler(function()
		                g_scene:openDialog("Vip.VipPanel")
		            end)
        		end
    		end
		end
	end

	self.m_csNode.reset:setTouchEnded(resetFunc)

	self.m_csNode.sweep:setTouchEnded(function()
		if not hasSweepUnlock then
			VipManager:showVipWarming( PrivilegeField[Privilege.kStageSweepUnlock], sweepPrivVipModel:getName() )
		elseif not canSweep then
            if not hasEnoughPower then
                g_scene:handleErrorCode("powerLack")
            else
			    g_scene:showTips(Serialize[64323])
            end
		else
			StageManager:onStageSweepRequest(self.m_stage:getId())
		end
	end)

	if self.m_csNode.challenge then
		self.m_csNode.challenge:setTouchEnded(function()
			if leftChallengeTimes > 0 then
				if sweepTimesInView <= 0 then
					g_scene:handleErrorCode("powerLack")
				else
					local heros = StageManager:getEmbattleHero()
					local count = table.count(heros)
					if count <= 0 then
						g_scene:showTips(Serialize[64324])
						g_scene:openDialog("Stage.EmbattleHeroPanel",{stageId = self.m_stage:getId()})
					elseif table.count(heros) < 6 then
						local panel = g_scene:openDialog("UI.ConfirmPanel",{text = Serialize[64179]})
						panel:setOkHandler(function()
							StageManager:enterStage(self.m_stage:getId(), self.m_from)
						end)
					else
						StageManager:enterStage(self.m_stage:getId(), self.m_from)
					end
				end
			else
				resetFunc()
			end
		end)
	end
end

-----=================================================================================------
function This.compareAward( info1, info2 )
    if info1.index ~= info2.index then
        return info1.times < info2.times
    end
end

function This:onVipActivated( maxLevel )
	self:refreshSweepButton()    
end

function This:onStageSweep( rewardList )
	self:refreshSweepData() 
    table.sort( rewardList, This.compareAward )
    self:afterStageSweep( rewardList )
end


function This:onStageReset( )
    self:refreshSweepButton()  
end

function This:onPlayerAttrChanged(field,oldValue,value)
    if field == PlayerAttr.power then
		self:refreshSweepButton()  
    end
end

function This:onDayPassed()
	StageManager:clearAllStageDataTimes()
    self:refreshSweepButton()  
end

function This:setFrom(from)
	self.m_from = from
end

--endregion
