-- @Author: superm
-- @Date:   2017-10-10 19:55:32
-- @Last Modified by:   superm
-- @Last Modified time: 2017-10-27 14:10:53


local This, super = class(..., "Stage.StageConfirmPanel")
local StageManager = require("Stage.GameStageManager")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local StageSweepResultItemView = require("Stage.StageSweepResultItemView")

function This:initNode(info)
    self.m_stage = info.stage
    self.m_rewardList = info.rewardList
    self.m_rewardNums = {}
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/StageSweepSettlement.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
    self:addChild(self.m_csNode)
end

function This:initTouchEvent()
    self.m_csNode.confirmButton:setTouchEnded(function()
        self:close()
    end)

    self:initSweepTouchEvent()
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
    self.m_csNode.scrollView:removeAllItems()
    self.m_csNode.scrollView:setTouchEnabled(true)
    self:runSelectionItem()
end

function This:refreshSpallNotes()
    for i, info in ipairs(self.m_rewardList) do
        local itemList = info.award_list 
        for _, itemInfo in ipairs(itemList) do
            local itemType = ItemUtil.getType(itemInfo.id)
            if itemType == ItemCommonType.kSpall then
                if not self.m_rewardNums[itemInfo.id] then
                    self.m_rewardNums[itemInfo.id] = 0
                end
                self.m_rewardNums[itemInfo.id] = self.m_rewardNums[itemInfo.id] + itemInfo.num
            end
        end
    end

    local count = 0
    for id, num in pairs(self.m_rewardNums) do
        local item = DepotManager:getItem(id)
        if item and item:isSpall()  then
            count = count + 1
            local label = self.m_csNode["rewardnote" .. tostring(count)]
            if label then
                label:setVisible(true)
                label:setString(item:getName() .. ":" .. item:getCount() .. "/" .. item:getSynthesizeCost() )
            end
        end
    end

    for i= count + 1, 2 do
        self.m_csNode["rewardnote" .. tostring(i)]:setVisible(false)
    end
end

function This:runSelectionItem(delay)
    local continue = false
    local loopCount = 0

    function runTask( )
        loopCount = loopCount + 1
        if not delay then
            delay = 0.8
        end

        local count = #self.m_rewardList
        local minIndex = 1
        local maxIndex = count

        if loopCount > minIndex then
            self.m_csNode.scrollView:jumpToBottom()
        end

        if loopCount <= maxIndex then
            local view = StageSweepResultItemView:create(self.m_rewardList[loopCount].award_list, loopCount)
            local w , h = view:getContentSize()
            self.m_csNode.scrollView:pushNode(view, w, h, nil)
            view:runActionByID("ListAppear", 0.2)
            continue = true
        else
            self:refreshSpallNotes()
            self:refreshSweepButtonUI()
            continue = false
        end

        if continue then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
        end
    end

    if delay then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
    else
        runTask()
    end
end

function This:refreshSweepButtonUI()
    --boby  
    self.m_csNode.confirmButton:setVisible(true)
    super.refreshSweepButtonUI(self)
end

function This:hiddenButtons()
    self.m_csNode.confirmButton:setVisible(false)
    self.m_csNode.reset:setVisible(false)
    self.m_csNode.sweep:setVisible(false)
    self.m_csNode.rewardnote1:setVisible(false)
    self.m_csNode.rewardnote2:setVisible(false)
end

function This:onStageSweep( rewardList )
    self:hiddenButtons()
    self.m_rewardList = rewardList
    self.m_rewardNums = {}    
    super.onStageSweep( self, rewardList )
end

function This:afterStageSweep( rewardList )
    self:refresh()  
end