-- @Author: superm
-- @Date:   2017-10-09 17:53:43
-- @Last Modified by:   superm
-- @Last Modified time: 2020-08-14 12:17:58

local This, super = class(...,"Dialog")
local StageManager = require("Stage.GameStageManager")
local DepotManager = require("Depot.DepotManager")
local Hero = require("Hero.Hero")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")

local kRewardRoundNum = 2 * 3
local kMaxRound = 3 * 200
local kDecelationNum = 1

local kBlinkNum = 3

function This:initNode(ch)
    self.m_lott = {
        iCurIndex = 1,  -- 当前时刻高亮位置
        iMaxRound = kMaxRound - kDecelationNum , -- 高亮移动的次数上限
        iCurRound = 0, -- 高亮移动的次数,由它控制计时器以及落点
        iLuckyID  = 0, -- 中奖奖品id
        iLuckyIndex = 0,
        bIsGot = false,
        bNeedClear = false, -- 需要复位

        fDelay = 0.3,
        fDelayFactor = 0.3,
    }

    self.m_initChapter = ch
    self.m_chapter = nil

    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/StageVisitHero.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
    self:addChild(self.m_csNode)

    self.m_pageView = self.m_csNode.PageView
    self.m_pageNodes = {}
    local chapters, initIndex = self.m_initChapter:getVisitChapterList()
    self.m_visitChapterList = chapters
    for i , chapter in ipairs(chapters) do
        local node = cc.Widget:create()
        node:setAnchorPoint(0, 0)
        self.m_pageView:addPage(node)
        self.m_pageNodes[i - 1] = {}
        self.m_pageNodes[i - 1].isInit = false
        self.m_pageNodes[i - 1].node = node
    end
    self.m_pageView:setCurrentPageIndex(initIndex - 1)
    self:focusePageOnIndex(initIndex - 1)
    self.m_isPlayAnim = false
end

function This:initTouchEvent()
    self.m_csNode.closebg:setTouchEnded(function()
        self:close()
    end)

    self.m_pageView:addEventListener(function (eventType)
        local index = self.m_pageView:getCurrentPageIndex()
        self:focusePageOnIndex(index)
    end)

    self.m_csNode.visitbutton:setTouchEnded(function()
        if self.m_isPlayAnim then
            return
        end
        local price = self.m_chapter:getVisitPriceBy()
        if price == 0 or GamePlayer:checkDiamond(price) then
            local num = self.m_chapter:getVisitHeroTimes()
            if num <= 1 then
                self:runSelectionItem()
                self.m_isPlayAnim = true
            end
            g_scene:setIsEnableShowTips(false)
            StageManager:onVisitHeroRequest(self.m_chapter:getId(), false)
        end
    end)

    self.m_csNode.right:setTouchEnded(function()
        self:onNextChapter()
    end)

    self.m_csNode.left:setTouchEnded(function()
        self:onPreChaper()
    end)
end


function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    StageManager:addObserver(self)
end

function This:onExit()
    StageManager:removeObserver(self)
    DepotManager:removeObserver(self)
    super.onExit(self)
    self.m_isPlayAnim = false
end

function This:refresh()
    g_scene:setIsEnableShowTips(true)
    local index = self.m_pageView:getCurrentPageIndex()
    self:refreshHeroList(self.m_chapter, self.m_page)
    self:refreshVisitStateNode()
    if self.m_lott.bNeedClear then
        self.m_lott = {
            iCurIndex = 1,
            iMaxRound = kMaxRound - kDecelationNum ,
            iCurRound = 0, 
            iLuckyID  = 0, 
            iLuckyIndex = 0,
            bIsGot = false,
            bNeedClear = false,
            fDelay = 0.3,
            fDelayFactor = 0.3,
        }
    end
end

function This:refreshHeroList(chapter, page, initAction)
    local heroList = chapter:getCurVisitHeroList()
    for index, heroInfo in ipairs(heroList) do
        local curNode = page["hero" .. tostring(index)]

        if initAction then
            local hero = chapter:getVisitHeroInstanceBy(heroInfo.heroId)
            if not hero then
                hero = Hero:create({
                    heroId = heroInfo.heroId,
                })
                chapter:setVisitHeroInstance(hero, heroInfo.heroId)
            end

            local quality = hero:getQuality()
            curNode.icon:setImage(hero:getIcon())
            curNode.icon:setVisible(true)

            local country = hero:getCountry()
            curNode.country:setString(CountryName[country])
            local color = SmallCardQualityColor[quality]
            curNode.country:setColor(color.r , color.g, color.b)

            curNode.country:setVisible(true)

            curNode.qualitybg:setImage(QualityBgFile[quality])
            curNode.quality:setImage(QualityFrameFile[quality])
            curNode.name:setString(hero:getName(), AppText:isChinese() and 10 or nil)
            -- curNode.fightPower:setString(tostring(hero:getFightPower()))
            -- curNode.herofightbg:setImage(QualityFightPowerFrameFile[quality])
        end

        curNode.yizhaomu:setVisible(heroInfo.isMine or false)
        curNode.selected:setVisible(false)
    end
end

function This:refreshVisitStateNode( )
    self.m_csNode.stagename:setString(self.m_chapter:getTitle() .. " " .. self.m_chapter:getName())
    local price = self.m_chapter:getVisitPriceBy()
    local opening = self.m_chapter:isPassAllStages()
    self.m_csNode.visitbutton:setVisible(opening and price >= 0)
    self.m_csNode.cost:setVisible(opening and price >= 0)
    self.m_csNode.condition:setVisible(not opening)
    self.m_csNode.mengban:setVisible(not opening)
    if not opening then
        self.m_csNode.condition:setString(Serialize[64342] .. self.m_chapter:getTitle() .. Serialize[64343])
    elseif opening and price >= 0 then
        self.m_csNode.cost.free:setVisible(price <= 0)
        self.m_csNode.cost.price:setVisible(price > 0)
        self.m_csNode.cost.zuanshi:setVisible(price > 0)
        self.m_csNode.cost.price:setString(price)
    else
        self.m_csNode.condition:setVisible(true)
        self.m_csNode.condition:setString(Serialize[64344])
    end
end

function This:focusePageOnIndex(index)
    local count = self.m_pageView:getPageCount()
    local minIndex = index - 1 < 0 and 0 or index - 1
    local maxIndex = index + 1 >= count and count - 1 or index + 1

    if index == minIndex then
        self.m_csNode.left:setGrayShader()
    elseif index == minIndex + 1 then
        self.m_csNode.left:clearShader()
    end

    if index == maxIndex then
        self.m_csNode.right:setGrayShader()
    elseif index == maxIndex - 1 then
        self.m_csNode.right:clearShader()
    end

    for i = minIndex , maxIndex do
        local chapter = self.m_visitChapterList[i + 1]
        if not self.m_pageNodes[i].isInit then
            local node = cc.Node:createFromCSFile("MenuScene/NodeUI/StageVisitHeroUI.csb")
            local page = self.m_pageView:getPage(i)
            page:addChild(node)
            node:setPositionY(self.m_pageView:getContentHeight())

            self:refreshHeroList(chapter, node, true)

            self.m_pageNodes[i].node = node
            self.m_pageNodes[i].isInit = true
            
            local opening = chapter:isPassAllStages()
            node.lock:setVisible(not opening)
        end
        if index == i then
            self:refreshHeroList(chapter, self.m_pageNodes[i].node)
            self.m_chapter = chapter
            self.m_page = self.m_pageNodes[index].node
        end
    end 

    self:refreshVisitStateNode()
end

function This:onPreChaper()
    local index = self.m_pageView:getCurrentPageIndex()
    if index > 0 then
        self.m_pageView:scrollToPage(index - 1)
    end
end

function This:onNextChapter()
    local index = self.m_pageView:getCurrentPageIndex()
    self:setTimer(0, function (self) 
        self.m_pageView:scrollToPage(index + 1)
    end)
end

function This:runSelectionItem(delay)
    local continue = false
    local loopCount = 1

    function runTask( )
        loopCount = loopCount + 1
        if not delay then
            delay = self.m_lott.fDelay
        end

        local count = 3
        local minIndex = 1
        local maxIndex = 3

        self.m_lott.iCurRound = self.m_lott.iCurRound + 1
        if self.m_lott.iCurRound < kDecelationNum or self.m_lott.iCurRound > self.m_lott.iMaxRound then
            continue = false
        else
            continue = true
        end

        self.m_page["hero" .. self.m_lott.iCurIndex].selected:setVisible(false)
        self.m_lott.iCurIndex = self.m_lott.iCurIndex >= maxIndex and minIndex or self.m_lott.iCurIndex + 1
        if self.m_page["hero" .. self.m_lott.iCurIndex].yizhaomu:isVisible() then
            self.m_lott.iCurIndex = self.m_lott.iCurIndex >= maxIndex and minIndex or self.m_lott.iCurIndex + 1
        end
        self.m_page["hero" .. self.m_lott.iCurIndex].selected:setVisible(true);


        if self.m_lott.iCurRound > self.m_lott.iMaxRound then
            if self.m_lott.fDelay < self.m_lott.fDelayFactor then
                -- music run
                self.m_lott.fDelay = self.m_lott.fDelay + 0.32 * self.m_lott.fDelayFactor
                self.m_lott.fDelay = self.m_lott.fDelay > self.m_lott.fDelayFactor and self.m_lott.fDelayFactor or self.m_lott.fDelay
                self:runSelectionItem(self.m_lott.fDelay)
            else 
                -- music stop
                if self.m_lott.iCurRound == kMaxRound then
                    -- 移動 kMaxRound 次還沒收到網絡回調,認定為断网的节奏
                    self.m_page["hero" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    self.m_lott.iCurIndex = minIndex
                    self.m_isPlayAnim = false
                    g_scene:setIsEnableShowTips(true)
                    g_scene:showTips(Serialize[64345])
                else
                    self.m_page["hero" .. self.m_lott.iCurIndex].selected:setVisible(false)
                    self.m_page["hero" .. self.m_lott.iCurIndex].selected:runAction(cc.Sequence(cc.Blink(self.m_lott.fDelay * 2, kBlinkNum), cc.CallFunc(function ()
                        self:onLotteryResultShow()
                    end)))
                end
                self.m_lott.iCurRound = 0
                self.m_lott.iMaxRound = kMaxRound - kDecelationNum
            end
        elseif self.m_lott.iCurRound < kDecelationNum then
            -- music run
            self.m_lott.fDelay = self.m_lott.fDelay - 0.32
            self:runSelectionItem(self.m_lott.fDelay)

        else
            -- music run
        end

        if continue then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
        end
    end

    if delay then
            local curTime = GamePlayer:getServerTime() 
            self:setTimer(delay,function()
                runTask()
            end, tostring(curTime) .. tostring(loopCount))
    else
        runTask()
    end
end

function This:onLotteryResultShow(noAnimations)
    self.m_page["hero" .. self.m_lott.iLuckyIndex].selected:setVisible(true)
    self.m_lott.bNeedClear = true

    if not noAnimations then
        g_scene:setIsEnableShowTips(true)
        g_scene:showCard( {{heroId = self.m_lott.iLuckyID, isGot = self.m_lott.bIsGot}} )
    end
    self.m_isPlayAnim = false

    self:refresh()
end

-------------------------------------
function This:onVisitHero( info )
    if self.m_chapter:getId() == info.copyId then
        self.m_lott.iLuckyID = info.heroId
        self.m_lott.iLuckyIndex = info.heroIndex
        self.m_lott.bIsGot = HeroManager:isGotHero(self.m_lott.iLuckyID)
        if info.time < 3 then
            self.m_lott.iMaxRound = self.m_lott.iLuckyIndex - self.m_lott.iCurIndex
            self.m_lott.iMaxRound = self.m_lott.iMaxRound > 0 and self.m_lott.iMaxRound or self.m_lott.iMaxRound + kRewardRoundNum
            self.m_lott.iMaxRound = self.m_lott.iCurRound + self.m_lott.iMaxRound + kRewardRoundNum * 1 - kDecelationNum
        else
            self:onLotteryResultShow(true)
        end
    end
end