-- @Author: superm
-- @Date:   2017-10-21 10:51:07
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-05-05 17:28:00
-- @File Name: StarBoxPreview.lua

local This, super = class(...,"Dialog")
local StageManager = require("Stage.GameStageManager")
local ItemNode = require("UI.ItemNode")

function This:initNode(info)
	self.m_chapterId = info.chapterId
	self.m_rewardLv = info.rewardLv 
	self.m_chapterConfigs = info.chapterConfigs
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaskBattlefieldBox.csb")
	self:addChild(self.m_csNode)
	self.m_csNode.title:setString(Serialize[64347])
	self.m_csNode:setToCenter()
	self:initItems()
end

function This:initItems()
	local chapter = StageManager:getChapter(self.m_chapterId,StageManager:getStageType())
	local rewardList = chapter:getRewardDrops(self.m_rewardLv)
	self.m_state = chapter:starIsEnough(self.m_rewardLv)

	local csNode = self.m_csNode
	local count = #rewardList
	local scrollView = csNode.scrollView
	local itemtipslist = csNode.itemtipslist
	if count <= 4 then
		local w = 10
		for i , info in ipairs(rewardList) do
			local node = ItemNode:create(info.id, info.num)
			node:setAnchorPoint(0, 0.5)
			node:setPositionX(w)
			w = w + node:getContentWidth() + 10
			itemtipslist:addChild(node)
		end
		itemtipslist:setPositionX(- w / 2)
		itemtipslist:setVisible(true)
		scrollView:setVisible(false)
	else
		scrollView:removeAllChildren()
		scrollView.m_direction = "right"
		for i , info in ipairs(rewardList) do
			local item = ItemNode:create(info.id, info.num)
			item:setAnchorPoint(0.5,0.5)
			local w , h = item:getContentSize()
			scrollView:pushBackCustomItem(item, w, h ,0.5)
		end
		itemtipslist:setVisible(false)
		scrollView:setVisible(true)
	end

	local config = self.m_chapterConfigs[chapter:getId()]
	csNode.condition:setString(string.format(Serialize[64348], config.reward_starlv[self.m_rewardLv]))

	local hasRecieve = chapter:isStarRewardReceived(self.m_rewardLv)
	csNode.condition:setVisible(not self.m_state)
	csNode.yilingqu:setVisible(hasRecieve)
	csNode.lingqu:setVisible(self.m_state and not hasRecieve)
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	self.m_csNode.lingqu:setTouchEnded(function()
		StageManager:onReceiveStarRewardRequest(self.m_chapterId, self.m_rewardLv)
	end)
end

function This:onEnter()
	super.onEnter(self)
	StageManager:addObserver(self)
end

function This:onExit()
	StageManager:removeObserver(self)
	super.onExit(self)
end

function This:onStarRewardChanged(chapterId,rewardLv)
	if self.m_chapterId == chapterId then
		self:close()
	end
end

--endregion
