--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local WorldMapManager = require("World.WorldMapManager")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")

local TabName = 
{
	kCity = 1,
	kArmy = 2
}

function This:initNode()
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/BattleCity.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	local role = WorldMapManager:getRole(GamePlayer:getPlayerID())
	self.m_cityList = role:getCitys()
	self.m_csNode.citynumber:setString(#self.m_cityList)
end

function This:initTouchEvent()
	self:addTabButton(TabName.kCity,self.m_csNode.tab2,function ()
		self:openTabNode(TabName.kCity)
	end)
	self:addTabButton(TabName.kArmy,self.m_csNode.tab1,function()
		print(TabName.kArmy)
		self:openTabNode(TabName.kArmy)
	end)
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self:selectTabButton(TabName.kCity)

	self.m_csNode.dissolve:setTouchEnded(function()
		self:onDissolveAllCityTroops()
	end)
end

function This:createTabNode(tab)
	if tab == TabName.kCity then
		self.m_cityNode = self:createCityNode()
		return self.m_cityNode
	elseif tab == TabName.kArmy then
		self.m_armyNode = self:createArmyNode()
		return self.m_armyNode
	end
end

function This:onEnter()
	super.onEnter(self)
	WorldMapManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	WorldMapManager:removeObserver(self)
end

function This:statisticCity(city)
	local builds = city:getAllBuild()
	local count = 0
	local minju = 0
	local jishi = 0
	local longtian = 0
	local lv = 0

	local tsfLv = city:getBuild(BuildPos.kMainBuild):getLevel()
	for pos , build in pairs(builds) do
		if build:getType() == BuildType.kDwelling then
			minju = minju + 1
		elseif build:getType() == BuildType.kMarket then
			jishi = jishi + 1
		elseif build:getType() == BuildType.kFarmland then
			longtian = longtian + 1
		end
		lv = lv + build:getLevel()
		count = count + 1
	end

	local kongdi = BuildPos.kMainBuild - count
	if count > 0 then
		lv = math.floor(lv / count)
	end
	return kongdi , minju, jishi, longtian, tsfLv, lv
end

function This:createCityItem(city)
	local itemNode = cc.Node:createFromCSFile("GameScene/NodeUI/BattleCityItem.csb")
	itemNode.city:setString(city:getName())
	local kongdi , minju , jishi , longtian , tsfLv , lv = self:statisticCity(city)
	itemNode.space:setString(kongdi)
	itemNode.home:setString(minju)
	itemNode.shop:setString(jishi)
	itemNode.cropland:setString(longtian)
	itemNode.citylevel:setString(tsfLv)
	itemNode.average:setString(lv..Serialize[60187])
	itemNode.city:setTouch({
		onEnded = function()
			g_scene:focuseCity(city:getCityId())
			self:close()
		end,
		isContainer = true,
		isScale = false
	})
	local w , h = itemNode.bg:getContentSize()
	itemNode:setContentSize(w,h)
	return itemNode
end

function This:createCityNode()
	local node = cc.Node:createFromCSFile("GameScene/NodeUI/BattleCityInformation.csb")
	for i , cityId in ipairs(self.m_cityList) do
		local city = WorldMapManager:getCity(cityId)
		local itemNode = self:createCityItem(city)
		node.ScrollView:pushBackCustomItem(itemNode,530,50,1)	
	end
	return node
end

function This:statisticArmy(city)
	local armyList = city:getRoleTroop(GamePlayer:getPlayerID())
	local troop = #armyList
	local reserve = city:getReserveHp()
	local hp = 0
	local reserveProduct = city:getReserveHpProduct()
	for i , tp in pairs(armyList) do
		hp = hp + tp:getHp()
	end
	return reserve, troop, hp, reserveProduct
end

function This:createArmyItem(city)
	local itemNode = cc.Node:createFromCSFile("GameScene/NodeUI/BattleMilitaryItem.csb")
	itemNode.city:setString(city:getName())
	local reserve , troop , hp , reserveProduct = self:statisticArmy(city)
	itemNode.troops:setString( math.floor(reserve))
	itemNode.hero:setString(troop)
	itemNode.herotroops:setString( math.floor(hp))
	itemNode.troopsoutput:setString( math.floor(reserveProduct)..Serialize[64350])
	itemNode.city:setTouch({
		onEnded = function()
			g_scene:focuseCity(city:getCityId())
			self:close()
		end,
		isContainer = true,
		isScale = false
	})
	local w , h = itemNode.bg:getContentSize()
	itemNode:setContentSize(w,h)
	return itemNode
end

function This:createArmyNode()
	local node = cc.Node:createFromCSFile("GameScene/NodeUI/BattleMilitaryInformation.csb")
	for i , cityId in ipairs(self.m_cityList) do
		local city = WorldMapManager:getCity(cityId)
		local itemNode = self:createArmyItem(city)
		node.ScrollView:pushBackCustomItem(itemNode,530,50,1)
	end
	return node
end

function This:refresh()
	if self.m_cityNode then
		self.m_cityNode.ScrollView:removeAllItems()
		for i , cityId in ipairs(self.m_cityList) do
			local city = WorldMapManager:getCity(cityId)
			local itemNode = self:createCityItem(city)
			self.m_cityNode.ScrollView:pushBackCustomItem(itemNode,530,50,1)	
		end
	end
	if self.m_armyNode then
		self.m_armyNode.ScrollView:removeAllItems()
		for i , cityId in ipairs(self.m_cityList) do
			local city = WorldMapManager:getCity(cityId)
			local itemNode = self:createArmyItem(city)
			self.m_armyNode.ScrollView:pushBackCustomItem(itemNode,530,50,1)
		end
	end
end

function This:onDissolveAllCityTroops()
	local bFind = false
	for i , cityId in ipairs(self.m_cityList) do
		local heroList = {}
		local hpList = {}
		local city = WorldMapManager:getCity(cityId)
		local troopList =  city:getRoleTroop(GamePlayer:getPlayerID())
		for pos , troop in pairs(troopList) do
			local hero = HeroManager:getHero(troop:getHeroId())
			table.insert(heroList,hero:getUid())
			table.insert(hpList,0)
		end

		if #heroList > 0 and #hpList > 0 then
			WorldMapManager:onAddHeroHpRequest(city:getCityId(),{heros = heroList,hp = hpList})
			bFind = true
		end
	end

	if not bFind then
		g_scene:showTips(Serialize[64351])
		return
	end
end

function This:onAddHeroHp()
	self:refresh()
	g_scene:showTips(Serialize[64352])
end


--endregion
