---
---
--- Created by aid.
--- DateTime: 2018/11/3 10:57
---
local This , super = class(...,"Dialog")

local Socket = require("Net.Socket")
local ItemNode = require("UI.ItemNode")
local LegionWarShopJifenConfig = require("Metadatas.LegionWarShopJifenConfig")
local GameMapManager = require("Map.GameMapManager")

local FamilyManager = require("Family.FamilyManager")

local function haveGot(allAwardStatus, awardId)
    for _, awardStatus in ipairs(allAwardStatus) do
        if awardStatus.id == awardId then
            return awardStatus.is_get
        end
    end
--    error("error!")
end

-- 参数说明
-- storeNumber：商店编号
-- myScore：我的积分
-- allAwardStatus：奖励领取状况（LegionMap.proto中msgs13060里的role_jifen_rewarded字段）
-- map：地图类型
function This:initData(arguments)
    super.initData(self, arguments)

    self.m_shopNumber = arguments.storeNumber
    self.m_myScore = arguments.myScore
    self.m_map = GameMapManager:getMap(arguments.map)

    self.m_allAwardList = {}
    local index = 1
    local awardId = self.m_shopNumber*100+index
    while LegionWarShopJifenConfig[awardId] do
        local award = LegionWarShopJifenConfig[awardId]

        table.insert(self.m_allAwardList, {
            awardId = awardId,
            index = index,
            needScore = award.jifen, -- 领取该行奖品需要的积分
            awards = award.reward, -- 达到needScore可领取的奖品
            haveGot = haveGot(arguments.allAwardStatus, awardId), -- 领取状态
        })

        index = index +1
        awardId = awardId + 1
    end
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionShopIntegral.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    self.itemViewList = {}

    -- 奖励列表
    self.m_scrollView = self.m_csNode.scrollView
    self.m_scrollView.m_itemMargin = {2, 2}
    self.m_scrollView:setTouchEnabled(true)

    for _, award in ipairs(self.m_allAwardList) do
        local itemView, width, height = self:createItemView(award)
        self.m_scrollView:pushNode(itemView, width, height)
        table.insert(self.itemViewList, itemView)
    end
end

function This:initTouchEvent()
    self.m_csNode.closebg.close:setTouchEnded(function()
        self:close()
    end)
end

function This:onEnter()
    super.onEnter(self)
    FamilyManager:addObserver(self)
    self.m_map:addObserver(self)
end

function This:onExit()
    FamilyManager:removeObserver(self)
    self.m_map:removeObserver(self)
    super.onExit(self)
end

function This:setDataForItemView(itemView, data)
    -- 领取该行奖品需要的积分
    itemView.numbercishu:setString(data.needScore)
    -- 达到该积分的情况下可领取的奖品列表
    itemView.scrollview:removeAllItems()
    for _, awardItem in ipairs(data.awards) do
        local item =  ItemNode:create(awardItem[1], awardItem[2])
        local width, height = item:getContentSize()
        local x, y = item:getPosition()
        item:setAnchorPoint(0.5, 0.5)
        itemView.scrollview:pushBackCustomItem(item, width, height, 0.5)
    end
    -- 领取状态
    if self.m_myScore >= data.needScore then
        if self.m_allAwardList[data.index].haveGot then
            -- 已领取
            itemView.yilingqu:setVisible(true)
            itemView.button:setVisible(false)
        else
            -- 未领取
            itemView.yilingqu:setVisible(false)
            itemView.button:setTouch({
                getRect = function (touch)
                    local rect = touch.sizeNode:getBoundingBox()
                    return rect
                end,
                onEnded = function()
                    self:onGetButtonClick(data.awardId)
                end,
                isContainer = true,
                isDynamic = true,
                isScale = false,
            })
        end
    else
        -- 无法领取
        itemView.yilingqu:setVisible(false)
        itemView.button:setGrayShader()
    end
end

function This:createItemView(award)
    local itemView = cc.Node:createFromCSFile("MenuScene/NodeUI/ActivityHappyTurnTableDayList.csb")
    itemView.beijing.zhuandong:setImage("MenuScene/wordjifen.png")
    itemView.scrollview:setDirection("horizontal")
    itemView.scrollview.m_direction = "right"
    itemView.scrollview:setSwallowTouch(false)
    self:setDataForItemView(itemView, award)
    local width , height = itemView.beijing.bg:getContentSize()
    return itemView, width, height
end

function This:onGetButtonClick(awardId)
    self.m_map:onGetBarracksScoreAwardRequest(awardId)
end

-- 领取奖励的回复
function This:onReceiveBarracksScoreAwards(data)
    -- 处理奖励的领取状态
    for _, awardsStatus in ipairs(data.role_jifen_rewarded) do
        local index = awardsStatus.id%100

        self.m_allAwardList[index].haveGot = awardsStatus.is_get
        self:setDataForItemView(self.itemViewList[index], self.m_allAwardList[index])
    end
end
