---
---
--- Created by aid.
--- DateTime: 2019/1/7 10:24
---
local This , super = class(...,"Dialog")
--local RefineryPriceConfig = require("Metadatas.RefineryPriceConfig")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local ShopMappingConfig = require("Metadatas.ShopMappingConfig")
local ItemShopConfig = require("Metadatas.ItemShopConfig")

function This:initData(arguments)
    super.initData(self, arguments)
    self.m_storeType = arguments.storeType

    self.m_goodsIdToUiIndexMap = {}
    for index, goodsId in ipairs(ShopMappingConfig[self.m_storeType].shop_id) do
        self.m_goodsIdToUiIndexMap[goodsId] = index
    end
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TreasureRefineStone.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    self.m_csNode.beijing.title:setString(ShopMappingConfig[self.m_storeType].title)

    for index, configId in ipairs(ShopMappingConfig[self.m_storeType].shop_id) do
        local config = ItemShopConfig[configId]
        local rootNode = self.m_csNode["item"..index]
        rootNode.name:setString(ItemUtil.getName(config.item_id).."*"..config.num)

        -- 折扣
        local discount = config.discount[1]
        if discount == 10 then
            rootNode.discountbg:setVisible(false)
            rootNode.discount:setVisible(false)
        else
            rootNode.discountbg:setVisible(true)
            rootNode.discount:setVisible(true)
            rootNode.discount:setString(string.format(Serialize[64358], discount))
        end

        rootNode.button.text:setString(config.price[1])
        local itemNode = ItemNode:create(config.item_id, config.num)
        itemNode:setAnchorPoint(0.5,0.5)
        rootNode.icon:addChild(itemNode)

        -- 限购显示
        local limitTimes = config.limit
        if limitTimes == 0 then -- 0表示不限购
            rootNode.xiangou:setVisible(false)
        else
            rootNode.xiangou:setVisible(true)
            rootNode.xiangou:setString(string.format(Serialize[60782], limitTimes, limitTimes))
        end
    end

end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    DepotManager:onGetUniversalStoreItemInfoRequest(self.m_storeType)
end

function This:onExit()
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    -- 关闭按钮
    self.m_csNode.beijing.closebg:setTouchEnded(function()
        self:close()
    end)

    -- 礼包购买
    for index = 1, 3 do
        self.m_csNode["item"..index].button:setTouchEnded(function()
            self:purchase(ShopMappingConfig[self.m_storeType].shop_id[index])
        end)
    end
end

function This:purchase(index)
    DepotManager:onBuyItemInUniversalStoreRequest(self.m_storeType, index)
end

function This:setRemainingPurchaseTimes(data)
    for i, index in ipairs(data.index_list) do
        local rootNode = self.m_csNode["item"..self.m_goodsIdToUiIndexMap[index]]
        local limitTimes = ItemShopConfig[index].limit
        rootNode.xiangou:setString(string.format(Serialize[60782], limitTimes-data.count_list[i], limitTimes))
    end
end

function This:onReceiveUniversalStoreItemInfo(data)
    if data.buy_type == self.m_storeType then
        self:setRemainingPurchaseTimes(data.buy_info)
    end
end

function This:onReceiveUniversalStorePurchaseResult(data)
    if data.buy_type == self.m_storeType then
        local config = ItemShopConfig[data.buy_index]
        g_scene:showReward({{config.item_id, config.num}})
    end
end