-- @Author: superm
-- @Date:   2018-04-09 18:06:58
-- @Last Modified by:   superm
-- @Last Modified time: 2018-07-05 16:02:26
-- @File Name: StoreArenaPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local StoreManager = require("Store.StoreManager")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")
local StoreArenaItemView = require("Store.StoreArenaItemView")
local DepotManager = require("Depot.DepotManager")

local storeTypeList = {
    StoreType.kArena, 
    StoreType.kArenaBaoge
}

function This:initData(info)
    self.m_inited = false
end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/ArenaShop.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    self.m_scrollView = self.m_csNode.ScrollView
    self.m_scrollView.m_itemMargin = {2, 2}

    for i, storeType in ipairs(storeTypeList) do
        self:addTabButton(tostring(storeType), self.m_csNode["tep" .. i], function()
            self.m_curTab = storeType
            self.m_curSubTab = storeType
            self:selectTabByType( self.m_curTab )
            return true
        end)
    end

    self:selectTabButton(tostring(storeTypeList[1]))
end


function This:initTouchEvent()
    if not self.m_inited then
        self.m_csNode.close:setTouchEnded(function()
            self:close()
        end)

        self.m_csNode.add:setTouchEnded(function()
            g_scene:openDialog("System.RechargeMainUI")
        end)

        self.m_csNode.add1:setTouchEnded(function()
            GamePlayer:checkGold({nocheck = true, nowarming = true})
        end)

        self.m_csNode.help:setTouchEnded(function()
            g_scene:openDialog("UI.HelpTips",{text = Serialize["arena_store_help_tips"], node = self.m_csNode.help})
        end)

        self.m_inited = true
    end
end

-- function This:getTabNodeData(name)
--     return self.m_store
-- end

-- function This:createTabNode(name)
--     local store = self:getTabNodeData(name)
--     local uiName = store:getUITypeName()
--     if uiName then
--         local tabNode = require(uiName):create(store)
--         return tabNode
--     end
-- end

function This:onEnter()
    super.onEnter(self)
    StoreManager:addObserver(self)
    DepotManager:addObserver(self)
    GamePlayer:addObserver(self)
    -- self:refresh()
end

function This:onExit()
    self:cleanUp()
    GamePlayer:removeObserver(self)
    DepotManager:removeObserver(self)
    StoreManager:removeObserver(self)
    super.onExit(self)
end

-- function This:refresh()
-- end

function This:cleanUp()
    for __, storeType in ipairs(storeTypeList) do
        local store = StoreManager:getStoreBy(storeType)
        if store then
            store:clearAllSlotView()
        end
    end
end

function This:selectTabByType( storeType )
    self.m_store = StoreManager:getStoreBy(storeType)
    if self.m_store then
        self:refreshStoreNode()
        self:refreshResourceNode()
    else
        StoreManager:onGetStoreInfoRequest(storeType)
    end
end

function This:refreshStoreNode()
    -- local node = self:openTabNode(tostring(storeType))
    if self.m_scrollView then
        self.m_scrollView:removeAllItems()
        self.m_scrollView:setTouchEnabled(true)
        
        local slots = self.m_store:getAllObjects()
        for index, slot in ipairs(slots) do
            local view = slot:getView()
            if not view then
                view = StoreArenaItemView:create(slot)
                slot:setView(view)
            else
                view:refresh()
            end
            local w , h = view:getContentSize()
            self.m_scrollView:pushNode(view, w+12, h, 4)
        end
    end
end

function This:refreshResourceNode()
    self.m_csNode.text1:setString("" .. GamePlayer:getDiamond())
    self.m_csNode.text3:setString("" .. GamePlayer:getJingjibi())
    self.m_csNode.tips:setVisible(self.m_curSubTab == StoreType.kArena)
end

--------------------------------------------------------------------
function This:onGetArenaStoreInfo()
    if self.m_curSubTab == StoreType.kArena then
        self:selectTabByType( StoreType.kArena )
    elseif self.m_curSubTab == StoreType.kArenaBaoge then
        self:selectTabByType( StoreType.kArenaBaoge )
    end
end

function This:onGetArenaBaogeStoreInfo()
    self:onGetArenaStoreInfo()
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.diamond or field == PlayerAttr.jingjibi then
        self.m_csNode.text1:setString("" .. GamePlayer:getDiamond())
        self.m_csNode.text3:setString("" .. GamePlayer:getJingjibi())
    end
end

function This:itemUpdate(itemList)
    if itemList and #itemList > 0 then
        g_scene:showReward(itemList)
    end
end
