--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local JLevelConfig = require("Metadatas.JLevelConfig")
local ZLevelConfig = require("Metadatas.ZLevelConfig")
local BLevelConfig = require("Metadatas.BudoLevelConfig")
local StoreUtil = require("Store.StoreUtil")
local StoreSlotBaseModel = require("Store.StoreSlotBaseModel")

local StoreConfigs = {
    [StoreType.kJianghun] = JLevelConfig,
    [StoreType.kZhangong] = ZLevelConfig,
    [StoreType.kJifen] = BLevelConfig,
    [StoreType.kJianghunMystery] = JLevelConfig,
    [StoreType.kZhangongMystery] = ZLevelConfig,
    [StoreType.kJifenMystery] = BLevelConfig,
    [StoreType.kAnimalShop] =JLevelConfig,
    [StoreType.kHeroEquipShop] = JLevelConfig,
    [StoreType.kAnimalMystery] =JLevelConfig,
    [StoreType.kBattlefieldShop] = JLevelConfig,

}

function This:init( info )
	super.init(self)
    self:initBaseData( info )
    self.m_view = nil
    self.m_config = info.config
    self.m_state = info.state or StoreObjectState.kNormal
	self.m_escape = 0
	self.m_isUpdating = false
	self.m_objects = {}
	self:updateState(true, info)
end

function This:clean()
end

function This:initBaseData( info )
    self.m_storeType = info.shop_type
    self.m_endTime = 0
end

function This:updateBaseData( info )
    self.m_storeType = info.shop_type
end

function This:updateObjects()
end

function This:updateState(isInit, info)
    if isInit then
        self:initBeforeUpdateState(info, true)
    end

    if self:isNeedBuyValid() and not self:needBuy() then
        self.m_state = StoreObjectState.kNormal
    else
        local curTime = GamePlayer:getServerTime()
        if curTime < self.m_endTime then
            self.m_state = StoreObjectState.kUpdating
            self:startUpdate()
            return
        elseif curTime >= self.m_endTime then
            self:endUpdate()
            self:updateStateByConds()
        end
    end
    self:updateObjects()
end

function This:updateStateByConds()
    local conds = self:getCondition()
    if not conds or conds == 0 then
        self.m_state = StoreObjectState.kNormal
    else
        local ret = self:isNotLockedBy(conds)
        self.m_state = ret and StoreObjectState.kNormal or StoreObjectState.kLocked
    end
end

function This:updateSlot(slotInfo)
    if slotInfo and slotInfo.index then 
        local index = slotInfo.index
        local slot = self.m_objects[index]
        if not slot then
            slot = StoreSlotBaseModel:create(slotInfo)
            slot:setStore(self)
            self.m_objects[index] = slot
        else
            slot:updateInfo(slotInfo)
        end
    end    
end

function This:getSlotBy( index )
    return self.m_objects[index] 
end

function This:markDataRefreshTime( time )
    self.m_dataRefreshTime = os.serverdate("*t", time or GamePlayer:getServerTime())
end

function This:isDataValid()
    return true
end

function This:initBeforeUpdateState(info, isInit)
    self.m_favour = info.favour or 0

    if isInit then
        self.m_objects = {}
    end

    if info.config_list then
        for _, slotInfo in ipairs(info.queue_list) do
            local slotConfig = nil
            for _, cfg in ipairs(info.config_list) do
                if cfg.shopping_order == slotInfo.index then
                    slotConfig = cfg
                    break
                end
            end
            slotInfo.config = slotConfig
            slotInfo.shop_type = info.shop_type
            self:updateSlot(slotInfo)
        end
    else
        for _, slotInfo in ipairs(info.queue_list) do
            slotInfo.shop_type = info.shop_type
            self:updateSlot(slotInfo)
        end
    end

    self:initConfigData()
end

function This:initConfigData()
    local level = GamePlayer:getLevel()
    if not self.m_config then
        local nums, count = {}, 0
        for num, cfg in pairs(StoreConfigs[self.m_storeType]) do
            table.insert(nums, {num = num, max_lv = cfg.last_level})
            count = count + 1
        end
        self.m_configNumberList = nums
        self:updateConfig(level)
    elseif level > self.m_config.last_level or level < self.m_config.init_level then
        self:updateConfig(level)
    end
end

function This:updateConfig(level)
    local found, count
    for i, cfg in ipairs(self.m_configNumberList) do
        count = i
        if level <= cfg.max_lv then
            found = true
            break
        end
    end

    local key = self.m_configNumberList[count].num
    self.m_config = StoreConfigs[self.m_storeType][key]
    if found then
        local nextObj = self.m_configNumberList[count + 1]
        if not nextObj then
            self.m_nextConfig = self.m_config
        else
            self.m_nextConfig = StoreConfigs[self.m_storeType][nextObj.num]
        end
    else
        self.m_nextConfig = self.m_config
    end
end

function This:updateInfo(info)
    self:updateBaseData(info)
    self:selfUpdateInfo(info)
    self:updateState()
end

function This:selfUpdateInfo(info)
    self:initBeforeUpdateState(info)
end

function This:isLocked()
    return self.m_state == StoreObjectState.kLocked
end

function This:isOpening()
    return self.m_state == StoreObjectState.kNormal
end

function This:isUpdating()
    return self.m_state == StoreObjectState.kUpdating
end

function This:isEmpty()
    return self.m_objects == nil or table.count(self.m_objects) <= 0
end

function This:isNotLockedBy(condition)
    return true
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return 0
	end
	return self.m_endTime - curTime
end

function This:setEndTime( endTime )
	self.m_endTime = endTime
	local curTime = GamePlayer:getServerTime()
	if curTime < self.m_endTime then
        self.m_state = StoreObjectState.kUpdating
		self:startUpdate()
    else
        self:endUpdate()
	end
	self:notifyObservers("OnStoreInfoChanged", self:getId())
end

function This:startUpdate()
	cc.Application:addToUpdateList(self)
	self:notifyObservers("onStoreUpdateStart", self:getId())
	self.m_isUpdating = true
end

function This:hasNextUpdate()
    return false
end

function This:startNewUpdate()
    self:notifyObservers("onStoreNewUpdateStart", self:getId())
end

function This:endUpdate()
    if self:hasNextUpdate() then
        self:startNewUpdate()
    else
        cc.Application:removeFromUpdateList(self)
        if self.m_endTime > 0 then
           self:notifyObservers("onStoreUpdateEnd", self:getId())
        end
        self.m_isUpdating = false
    end
end

function This:beforeEndUpdate()
end

function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			self:notifyObservers("updateTime", leftTime)
		else
			self:notifyObservers("updateTime", 0)
			self:beforeEndUpdate()
            self:endUpdate()
		end
	end
end

function This:setView(view)
    self.m_view = view
end

function This:getView()
    return self.m_view
end

function This:getId()
	return self.m_storeType
end

function This:isNeedBuyValid()
    return false
end

function This:needBuy()
	return false
end

function This:getAllObjects()
    return self.m_objects
end

function This:getObjectBy(handler)
    for id, obj in pairs(self.m_objects) do
    	if handler and handler(obj) then
    		return obj
    	end
    end
    return nil
end

function This:removeObjectBy(handler)
    local resultId = nil
    for id, obj in pairs(self.m_objects) do
        if handler and handler(obj) then
            resultId = id
        end
    end
    
    if resultId then
        self.m_objects[resultId] = nil
    end
end

function This:getCondition()
    return nil
end

function This:getFavour()
    return self.m_favour, self:getFavourLimit()
end

function This:setFavour(favour)
    self.m_favour = favour
end

function This:getFavourLimit()
    if self.m_config then
        return self.m_config.favour_limit
    end
    return 0
end

function This:getNextFavourLimit()
    if self.m_nextConfig then
        return self.m_nextConfig.favour_limit
    end
    return 0
end

function This:getCommonCostId()
    local switch = {
        [StoreType.kJianghun] = function()
            return 110, GamePlayer:getSoul(), false
        end,
        [StoreType.kZhangong] = function()
            return 107, GamePlayer:getZhangong(), true
        end,
        [StoreType.kJifen] = function()
            return 168, GamePlayer:getSkillReel(), false
        end,
        [StoreType.kJianghunMystery] = function()
            return 110, GamePlayer:getSoul(), false
        end,
        [StoreType.kZhangongMystery] = function()
            return 107, GamePlayer:getZhangong(), true
        end,
        [StoreType.kJifenMystery] = function()
            return 168, GamePlayer:getSkillReel(), true
        end,
        [StoreType.kAnimalShop] = function ()
            return 302, GamePlayer:getAnimalSoul(),false
        end,
        [StoreType.kAnimalMystery] = function ()
            return 302, GamePlayer:getAnimalSoul(),false
        end
    }

    local func = switch[self:getId()]
    if func then
        return func()
    end
end

-- 获得商店类型
function This:getType()
    return self.m_storeType
end

function This:getName()
    local storeId = self:getId()
    if storeId == StoreType.kJianghun then
        return Serialize[60423]
    elseif storeId == StoreType.kZhangong then
        return Serialize[60424]
    elseif storeId == StoreType.kJifen then
        return Serialize[64372]
    elseif storeId == StoreType.kAnimalShop then
        return Serialize[64373]
    elseif storeId == StoreType.kBattlefieldShop then
        return Serialize[64374]
    end
end


--endregion