--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Store.StoreBaseModel")
local DepotManager = require("Depot.DepotManager")
local StoreManager = require("Store.StoreManager")
local GamePlayer = require("Player.GamePlayer")
local StorePriceConfig = require("Metadatas.StorePriceConfig")
local EventDispatcher = require("EventDispatcher")

local kMaxRefreshCostCount = #StorePriceConfig

function This:create(...)
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret
end

function This:initBaseData( info )
    super.initBaseData(self, info)
    local nowTime = GamePlayer:getServerTime()
    self.m_initTime = info.init_time or 0
    self.m_interval = info.refresh_interval or 0
    self.m_lastRefreshTime = info.last_refresh_time or 0
    local offset = math.floor( (nowTime - self.m_initTime) / self.m_interval )
    if offset < 0 then
        offset = 0
    end
      
    self.m_startTime = self.m_initTime + self.m_interval * offset
    self.m_endTime = self.m_startTime + self.m_interval
    self.m_refreshEscape = (nowTime - 57600) % 86400
end

function This:updateBaseData( info )
    super.updateBaseData(self, info)
    if info.refresh_interval then
        local nowTime = GamePlayer:getServerTime()
        self.m_initTime = info.init_time
        self.m_interval = info.refresh_interval
        self.m_lastRefreshTime = info.last_refresh_time or 0
        local offset = math.floor( (nowTime - self.m_initTime) / self.m_interval )
        if offset < 0 then
            offset = 0
        end
        self.m_startTime = self.m_initTime + self.m_interval * offset
        self:setEndTime(self.m_startTime + self.m_interval)
        self.m_refreshEscape = (nowTime - 57600) % 86400
    end
end

function This:initBeforeUpdateState(info, createHandler)
    super.initBeforeUpdateState(self, info, createHandler)
    if info.gold_refresh_time then
        self.m_refreshMaxFreeTime = info.refresh_max_time or 0
        self.m_refreshFreeTime = info.refresh_time or 0
        self.m_refreshChargedTime = info.gold_refresh_time
        if self.m_storeType % 10 == 1 then
            EventDispatcher:addMsg("common_reddot_multi", {zgRestTimes = self.m_refreshFreeTime >= 10})
        else
            EventDispatcher:addMsg("common_reddot_multi", {jhRestTimes = self.m_refreshFreeTime >= 10})
        end
    end
end

function This:hasNextUpdate()
    return self.m_refreshFreeTime < self.m_refreshMaxFreeTime
end

function This:startNewUpdate()
    self.m_startTime = self.m_endTime
    self.m_endTime = self.m_startTime + self.m_interval
    super.startNewUpdate(self)
end

function This:beforeEndUpdate()
    self:increaseRefreshTime()
	EventDispatcher:dispatcher("store_state_changed",self.m_storeType)
end

function This:update( dt )
    super.update(self, dt)
    self.m_refreshEscape = self.m_refreshEscape + dt
    if self.m_refreshEscape >= 86400 then
        self.m_refreshEscape = self.m_refreshEscape - 86400
        self.m_refreshChargedTime = 0
    end
end

function This:increaseRefreshChargedTime()
    self.m_refreshChargedTime = self.m_refreshChargedTime + 1

    local index = self:getId() % 10 + 1
    local times = self.m_refreshChargedTime > #StorePriceConfig and #StorePriceConfig or self.m_refreshChargedTime
    local addFovour = StorePriceConfig[times]["favours" .. index]
    return addFovour
end

function This:increaseRefreshTime()
    self.m_refreshFreeTime = self.m_refreshFreeTime + 1
    if self.m_refreshFreeTime > self.m_refreshMaxFreeTime then
        self.m_refreshFreeTime = self.m_refreshMaxFreeTime
    end
end

function This:discreaseRefreshTime()
    self.m_refreshFreeTime = self.m_refreshFreeTime - 1
    if self.m_refreshFreeTime <= 0 then
        self.m_refreshFreeTime = 0
    end
end

function This:isCostEnoughBy(num)
    local ids = {
        [StoreType.kJianghun] = 118,
        [StoreType.kJianghunBaoge] = 118,
        [StoreType.kZhangong] = 119,
        [StoreType.kJianghunMystery] = 118,
        [StoreType.kZhangongMystery] = 119,
        [StoreType.kRankBaoge] = 102,
        [StoreType.kJifen] = 119,
        [StoreType.kJifenMystery] = 119,	
        [StoreType.kAnimalShop] = 118,
        [StoreType.kAnimalMystery] = 118,
        [StoreType.kBattlefieldShop] = 119,
    }
    local id = ids[self:getId()]
    local curNumber = DepotManager:getItemOrResCount(id)
    return curNumber >= num, id, curNumber
end

function This:getRefreshTimes()
    return self.m_refreshFreeTime, self.m_refreshMaxFreeTime
end

function This:requestRefresh()
    if self.m_refreshFreeTime <= 0 then
        if self:isCostEnoughBy(1) then
            --item
            StoreManager:onRefreshCommonStoreRequest( self.m_storeType )
        else
            local nextTime = self.m_refreshChargedTime + 1
            if nextTime > kMaxRefreshCostCount then
                nextTime = kMaxRefreshCostCount
            end
            local cost = StorePriceConfig[nextTime]["price" .. (self:getId() % 10 + 1)]
            local panel = g_scene:openDialog("UI.CostConfirmPanel", {
                cost = cost,
                notips = "StoreCommon" .. tostring(self:getId()) .. tostring(cost),
                desc = Serialize[60860], 
                okHandler = function(build)
                    StoreManager:onRefreshCommonStoreRequest( self.m_storeType )
                end } )
        end
    else
        --free
        StoreManager:onRefreshCommonStoreRequest( self.m_storeType )
    end
end

function This:getUITypeName()
    return "Store.StoreCommonUI"
end


--endregion