-- @Author: superm
-- @Date:   2018-07-03 20:36:40
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-29 20:34:43
-- @File Name: StoreFamilyModel.lua
-- @Description: 

local This , super = class(...,"Store.StoreBaseModel")
local StoreSlotFamilyLevelModel = require("Store.StoreSlotFamilyLevelModel")
local StoreSlotFamilyContributionModel = require("Store.StoreSlotFamilyContributionModel")
local StoreSlotFamilyShareModel = require("Store.StoreSlotFamilyShareModel")
local StoreSlotFamilyModel = require("Store.StoreSlotFamilyModel")

local StoreManager = require("Store.StoreManager")
local GamePlayer = require("Player.GamePlayer")

function This:create(...)
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret
end

function This:initBeforeUpdateState(info, isInit)
    self:markDataRefreshTime()
    info.queue_list = info.shop_item_list
    super.initBeforeUpdateState(self, info, isInit)
    if info.refresh_time then
        self.m_refreshTime = info.refresh_time or 0
    end
    if info.shop_buy_times then
        self.m_totalBuyTimes = info.shop_buy_times
    end
end

function This:updateObjects()
    if self.m_objects then
        for index, slot in ipairs(self.m_objects) do
            slot:updateState()
        end
    end
end

function This:markDataRefreshTime(time)
    local curTime = time or GamePlayer:getServerTime()
    local curDate = os.serverdate("*t", curTime)
    local endTime = os.time({day = curDate.day, month = curDate.month, year = curDate.year, hour = 21, min = 30, sec = 0})
    if curTime >= endTime then
        endTime = endTime + SECONDS_PER_DAY
    end
    self.m_dataRefreshTime = endTime
end

function This:isDataValid()
    local curTime = GamePlayer:getServerTime()
    return curTime < self.m_dataRefreshTime
end

function This:hasNextUpdate()
    return not self:isDataValid()
end

function This:startNewUpdate()
    print("family store update")
    StoreManager:onGetStoreInfoRequest(self.m_storeType)
end

function This:getLeftBuyTimes()
    if self.m_totalBuyTimes then
        return kStoreShareTotalBuyTime - self.m_totalBuyTimes
    end
end

function This:initConfigData()
end

function This:getUITypeName() 
end

function This:getCSFileName()
    local id = self.m_storeType
    if id == StoreType.kFamilyContribution then
        return "MenuScene/NodeUI/FamilyContributionStore.csb"
    elseif id == StoreType.kFamilyShare then
        return "MenuScene/NodeUI/FamilyShareStore.csb"
    elseif id == StoreType.kFamilyLevel then
        return "MenuScene/NodeUI/FamilyLevelStore.csb"
    elseif id == StoreType.kFamilyFight then
        return "MenuScene/NodeUI/FamilyLegionMerit.csb"
    end   
end

function This:getRefreshTimes()
    return self.m_refreshTime, kStoreFamilyRefreshMaxTime
end

function This:getLeftRefreshTimes()
    return kStoreFamilyRefreshMaxTime - self.m_refreshTime
end

function This:updateSlot(slotInfo)
    if slotInfo and slotInfo.index then 
        local index = slotInfo.index
        local slot = self.m_objects[index]
        if not slot then
            slot = self:createSlot(slotInfo)
            slot:setStore(self)
            self.m_objects[index] = slot
        else
            slot:updateInfo(slotInfo)
        end
    end
end

function This:createSlot(info)
    local id = self.m_storeType
    if id == StoreType.kFamilyContribution then
        return StoreSlotFamilyContributionModel:create(info)
    elseif id == StoreType.kFamilyShare then
        return StoreSlotFamilyShareModel:create(info)
    elseif id == StoreType.kFamilyLevel then
        return StoreSlotFamilyLevelModel:create(info)
     elseif id == StoreType.kFamilyFight then
        return StoreSlotFamilyContributionModel:create(info)
    end    
end
