-- @Author: superm
-- @Date:   2018-10-30 18:49:36
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 17:51:22
-- @File Name: StoreFamilyUI.lua
-- @Description: 

local This , super = class(..., "NodeView")
local GamePlayer = require("Player.GamePlayer")
local StoreManager = require("Store.StoreManager")
local StoreFamilyItemView = require("Store.StoreFamilyItemView")
local ItemUtil = require("Bag.ItemUtil")
local ViewUtil = require("UI.ViewUtil")


function This:initData(store)
	self.m_store = store
	self.m_storeType = self.m_store:getId()
	self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
	local filename = self.m_store:getCSFileName()
    local csNode = cc.Node:createFromCSFile(filename)
    self:addChild(csNode)
	self.m_csNode = csNode
	self.m_scrollView = csNode.ScrollView
end

function This:initNode()
    self.m_scrollView.m_itemMargin = {2, 2}
	local csNode = self.m_csNode
    if self.m_storeType == StoreType.kFamilyFight then
        csNode.zijin.icon1:setImage(ItemUtil.getIconImage(176))
    else
        csNode.zijin.icon1:setImage(ItemUtil.getIconImage(127))
    end
    if self.m_storeType == StoreType.kFamilyLevel then
        csNode.zijin.icon2:setImage(ItemUtil.getIconImage(176))
    else
        csNode.zijin.icon2:setImage(ItemUtil.getIconImage(102))
    end
    csNode.zijin.add:setVisible(self.m_storeType ~= StoreType.kFamilyLevel)
    csNode.zijin.jiahaodi1:setVisible(self.m_storeType ~= StoreType.kFamilyLevel)
    csNode.tip:setVisible(false)
    ViewUtil:blinkView(csNode.tip, true)
end

function This:initTouchEvent()
    if not self.m_inited then
		local csNode = self.m_csNode
        csNode.zijin.add:setTouchEnded(function()
            g_scene:openDialog("System.RechargeMainUI")
        end)

        if csNode.reset then
            csNode.reset:setTouchEnded(function()
                StoreManager:onGetStoreInfoRequest(self.m_storeType)
                g_scene:showTips(Serialize[62153])
            end)
        end
        self.m_inited = true
    end
end

function This:onEnter()
    super.onEnter(self)
    StoreManager:addObserver(self)
    GamePlayer:addObserver(self)
    self:refresh()
end

function This:onExit()
    GamePlayer:removeObserver(self)
    StoreManager:removeObserver(self)
    super.onExit(self)
end

function This:refreshStoreNode()
    print("store refresh")
    if self.m_scrollView then
        self.m_scrollView:removeAllItems()
        self.m_scrollView:setTouchEnabled(true)
        
        local slots = self.m_store:getAllObjects()
        if self.m_storeType == StoreType.kFamilyLevel then
            table.sort(slots,function (a,b)
                if a:getPosition() ~= b:getPosition() then
                    return a:getPosition() < b:getPosition()
                end
                
            end )
        end
        for index, slot in ipairs(slots) do
            local view = slot:getView()
            if not view then
                view = StoreFamilyItemView:create(slot)
                slot:setView(view)
            else
                view:refresh()
            end
            local w , h = view:getContentSize()
            self.m_scrollView:pushNode(view, w, h, 2)
        end
    end

    if self.m_csNode.number then
        self.m_csNode.number:setString(self.m_store:getLeftBuyTimes())
    end
end

function This:refreshResourceNode()
	local csNode = self.m_csNode
    if self.m_storeType == StoreType.kFamilyFight then
         csNode.zijin.number1:setString("" .. GamePlayer:getJunGong())
    else
         csNode.zijin.number1:setString("" .. GamePlayer:getContributionbi())
    end
    if self.m_storeType == StoreType.kFamilyLevel then
        csNode.zijin.number2:setString("" .. GamePlayer:getJunGong())
    else
        csNode.zijin.number2:setString("" .. GamePlayer:getDiamond())
    end
end

function This:refreshRefreshBtn()
	local resetNode = self.m_csNode.reset
    if not resNode then
        return
    end
    self.m_curRefreshTime = self.m_store:getLeftRefreshTimes()
    if self.m_curRefreshTime > 0 then
        resetNode:setVisible(true)
        resetNode.shuaxin:setString(string.format("%s（%d）", Serialize["family_StoreNode_1"], self.m_curRefreshTime))
    else
        resetNode:setVisible(false)
        resetNode.shuaxin:setString(Serialize["family_StoreNode_1"])
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
	local resNode = self.m_csNode.zijin
    if field == PlayerAttr.diamond then
        resNode.number2:setString(value)
    elseif field == PlayerAttr.contributionbi then
        resNode.number1:setString(value)
    end
end

function This:refresh()
    self:refreshStoreNode()
    -- self:refreshRefreshBtn()
    self:refreshResourceNode()
end

function This:onBuyFamilyStoreItem(storeType)
    if storeType == self.m_storeType then
        if self.m_csNode.number then
            self.m_csNode.number:setString(self.m_store:getLeftBuyTimes())
        end
        self:refresh()
    end
end