-- @Author: superm
-- @Date:   2018-12-10 16:44:21
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-11 16:02:11
-- @File Name: StoreLegionWarModel.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local LegionWarPersonalShopConfig = require("Metadatas.LegionWarPersonalShopConfig")
local LegionWarShareShopConfig = require("Metadatas.LegionWarShareShopConfig")
local StoreLegionPersonalSlotModel = require("Store.StoreLegionPersonalSlotModel")
local StoreLegionShareSlotModel = require("Store.StoreLegionShareSlotModel")

function This:create(...)
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret
end

function This:init( info )
	super.init(self)
    self:initBaseData( info )
    self.m_state = info.state or StoreObjectState.kNormal
	self.m_escape = 0
	self.m_isUpdating = false
	self.m_roleSlotList = {}
	self.m_shareSlotList = {}
	self.m_boxesList = {}
	self:updateState(true, info)
end

function This:initBaseData(info)
    local typeConfig = LegionWarShopTypeConfig[info.shop_key]
    self.m_shopId = typeConfig.personal_shop_num
	self.m_jifen = info.jifen or 0
	self.m_jifenReward = info.role_jifen_rewarded or{}

	self.m_roleSlotList = {}
	for slotId, slotConfg in pairs(LegionWarPersonalShopConfig) do
		if slotConfg.shop_num == self.m_shopId then
			self.m_roleSlotList[slotConfg.index] = StoreLegionPersonalSlotModel:create({
				config = slotConfg
			})
		end
	end

	self.m_shareSlotList = {}
	for slotId, slotConfg in pairs(LegionWarShareShopConfig) do
		if slotConfg.shop_num == self.m_shopId then
			self.m_shareSlotList[slotConfg.index] = StoreLegionShareSlotModel:create({
				config = slotConfg
			})
		end
	end

	for _, buyInfo in ipairs(info.role_solt_list) do
		self.m_roleSlotList[buyInfo.slot]:updateInfo(buyInfo)
	end
	for _, buyInfo in ipairs(info.share_solt_list) do
		self.m_shareSlotList[buyInfo.slot]:updateInfo(buyInfo)
	end

	self.m_boxesList = {}
	self.m_boxesList[storeNumber]=self.m_shopId
	self.m_boxesList[myScore]=self.m_jifen
	self.m_boxesList[allAwardStatus]=self.m_jifenReward
-- role_jifen_rewarded

	if info.over_time then
    	self:setEndTime(info.over_time)
    end
end

function This:updateInfo( info )
	self:initBaseData(info)
end

function This:updateState()
    local curTime = GamePlayer:getServerTime()
    if curTime < self.m_endTime then
        self.m_state = StoreObjectState.kUpdating
        self:startUpdate()
        return
    elseif curTime >= self.m_endTime then
        self.m_state = StoreObjectState.kUpdating
        self:endUpdate()
    end
    self:updateObjects()
end


function This:setEndTime( endTime )
	self.m_endTime = endTime
	local curTime = GamePlayer:getServerTime()
	if curTime < self.m_endTime then
        self.m_state = StoreObjectState.kUpdating
		self:startUpdate()
    else
        self:endUpdate()
	end
	self:notifyObservers("OnStoreInfoChanged", self:getId())
end

function This:startUpdate()
	cc.Application:addToUpdateList(self)
	self:notifyObservers("onStoreUpdateStart", self:getId())
	self.m_isUpdating = true
end


function This:endUpdate()
    cc.Application:removeFromUpdateList(self)
    if self.m_endTime > 0 then
       self:notifyObservers("onStoreUpdateEnd", self:getId())
    end
    self.m_isUpdating = false
end

function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			self:notifyObservers("updateTime", leftTime)
		else
			self:notifyObservers("updateTime", 0)
			self:beforeEndUpdate()
            self:endUpdate()
		end
	end
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return 0
	end
	return self.m_endTime - curTime
end

function This:getCSFileName()
    return "MenuScene/NodeUI/QualifyingBaogeBg.csb"
end

function This:getUITypeName()
    return "Store.StoreLegionWarUI"
end

function This:getWarShopId()
    return self.m_shopId or 1
end

function This:getScore()
    return self.m_jifen or 0
end

function This:getSlotList(storeType)
	if storeType == StoreType.kLegionWarPersonal then
    	return self.m_roleSlotList
    else
    	return self.m_shareSlotList
    end
end

