-- @Author: superm
-- @Date:   2018-12-11 15:18:17
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-11 16:08:14
-- @File Name: StoreLegionWarUI.lua
-- @Description: 

local This , super = class(..., "NodeView")
local GamePlayer = require("Player.GamePlayer")
local StoreManager = require("Store.StoreManager")
local StoreFamilyItemView = require("Store.StoreFamilyItemView")
local ItemUtil = require("Bag.ItemUtil")
local ViewUtil = require("UI.ViewUtil")

function This:initData(store, storeType)
	self.m_store = store
	self.m_storeType = storeType
	self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
	local filename = self.m_store:getCSFileName()
    local csNode = cc.Node:createFromCSFile(filename)
    self:addChild(csNode)
	self.m_csNode = csNode
	self.m_scrollView = csNode.ScrollView
end

function This:initNode()
    self.m_scrollView.m_itemMargin = {2, 2}
	local csNode = self.m_csNode
    csNode.zijin.icon1:setImage(ItemUtil.getIconImage(127))

    local texNode = csNode.tex
    if self.m_storeType == StoreType.kLegionWarPersonal then
		texNode:setVisible(false)
	else
		texNode:setVisible(true)
	    local tipNode = texNode.tip
	    tipNode:setVisible(false)
	    ViewUtil:blinkView(tipNode, true)
    end
end

function This:initTouchEvent()
    if not self.m_inited then
		local csNode = self.m_csNode
        csNode.zijin.add:setTouchEnded(function()
            g_scene:openDialog("System.RechargeMainUI")
        end)

        csNode.wenhao:setTouchEnded(function()
            g_scene:openDialog("UI.HelpTips",{text = Serialize["family_warlordShop_2"],node = csNode.wenhao})
        end)

        if csNode.tex then
        	csNode.tex.box:setTouchEnded(function()
	            g_scene:openDialog("Store.BarracksSoreAwardPanel")
	        end)
        end
        self.m_inited = true
    end
end

function This:onEnter()
    super.onEnter(self)
    -- StoreManager:addObserver(self)
    GamePlayer:addObserver(self)
    self:refresh()
end

function This:onExit()
    GamePlayer:removeObserver(self)
    -- StoreManager:removeObserver(self)
    super.onExit(self)
end

function This:refreshStoreNode()
    print("store refresh")
    if self.m_scrollView then
        self.m_scrollView:removeAllItems()
        self.m_scrollView:setTouchEnabled(true)
        
        local slots = self.m_store:getSlotList(self.m_storeType)
        for index, slot in ipairs(slots) do
            local view = slot:getView()
            if not view then
                view = StoreFamilyItemView:create(slot)
                slot:setView(view)
            else
                view:refresh()
            end
            local w , h = view:getContentSize()
            self.m_scrollView:pushNode(view, w, h, 2)
        end
    end

    if self.m_csNode.number then
        self.m_csNode.number:setString(self.m_store:getLeftBuyTimes())
    end
end

function This:refreshResourceNode()
	local csNode = self.m_csNode
    csNode.zijin.number1:setString(GamePlayer:getContributionbi())
    csNode.zijin.number2:setString(GamePlayer:getDiamond())
end

function This:refreshProgressNode()
    if self.m_storeType == StoreType.kLegionWarPersonal then
    	return
    end
	-- 积分进度
    
end

function This:onPlayerAttrChanged(field, oldValue, value)
	local resNode = self.m_csNode.zijin
    if field == PlayerAttr.diamond then
        resNode.number2:setString(value)
    elseif field == PlayerAttr.contributionbi then
        resNode.number1:setString(value)
    end
end

function This:refresh()
    self:refreshStoreNode()
    self:refreshProgressNode()
    self:refreshResourceNode()
end