-- @Author: superm
-- @Date:   2018-06-25 14:59:27
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-07 10:58:40
-- @File Name: StoreManager.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local StoreUtil = require("Store.StoreUtil")
local GameRatingData = require("Map.GameWarShopData")

function This:isInited()
    return self.m_inited
end

function This:init()
	super.init(self)
    self.m_inited = false
	self.m_stores = {}

	self.m_creator = {
		[StoreType.kJianghun] = "Store.StoreCommonModel",
		[StoreType.kJianghunBaoge] = "Store.StoreBaogeModel",
		[StoreType.kJianghunMystery] = "Store.StoreMysteryModel",		
		[StoreType.kZhangong] = "Store.StoreCommonModel",	
		[StoreType.kZhangongMystery] = "Store.StoreMysteryModel",
        [StoreType.kJifen] = "Store.StoreCommonModel",
		[StoreType.kJifenMystery] = "Store.StoreMysteryModel",			
		[StoreType.kRankBaoge] = "Store.StoreBaogeModel",		
		[StoreType.kArenaBaoge] = "Store.StoreArenaModel",		
		[StoreType.kArena] = "Store.StoreArenaModel",	
		[StoreType.kFamilyContribution] = "Store.StoreFamilyModel",
		[StoreType.kFamilyShare] = "Store.StoreFamilyModel",
        [StoreType.kFamilyLevel] = "Store.StoreFamilyModel",
        [StoreType.kFamilyFight] = "Store.StoreFamilyModel",
		[StoreType.kMingWangBaoge] = "Store.StoreBaogeModel",
		[StoreType.kAnimalShop] = "Store.StoreCommonModel",
		[StoreType.kAnimalMystery] = "Store.StoreMysteryModel",
		[StoreType.kHeroEquipShop] ="Store.StoreHeroModel",
		[StoreType.kBattlefieldShop] = "Expedition.ExpeditionStore"
	}

	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 11305, self.onGetCommonStoreInfoReply)
	EventDispatcher:addListener(self, 11306, self.onBuyStoreItemReply)
	EventDispatcher:addListener(self, 10609, self.onBuyStoreItemReply)
	EventDispatcher:addListener(self, 11307, self.onRefreshCommonStoreReply)

	EventDispatcher:addListener(self, 11308, self.onGetMysteryStoreInfoReply)
	EventDispatcher:addListener(self, 11309, self.onBuyStoreItemReply)
	EventDispatcher:addListener(self, 11312, self.onLeaveMysteryStoreReply)

	EventDispatcher:addListener(self, 11310, self.onGetBaogeStoreInfoReply)
	EventDispatcher:addListener(self, 11311, self.onBuyStoreItemReply)
	
	EventDispatcher:addListener(self, 11315, self.onGetJinLianJifenStoreInfoReply)
	EventDispatcher:addListener(self, 11316, self.onBuyJinLianJifenStoreItemReply)

	EventDispatcher:addListener(self, 12206, self.onGetArenaBaogeStoreInfoReply)
	EventDispatcher:addListener(self, 12207, self.onBuyStoreItemReply)

	EventDispatcher:addListener(self, 12208, self.onGetArenaStoreInfoReply)
	EventDispatcher:addListener(self, 12209, self.onBuyStoreItemReply)

	EventDispatcher:addListener(self, 12122, self.onGetFamilyStoreInfoReply)
	EventDispatcher:addListener(self, 12123, self.onBuyStoreItemReply)
	EventDispatcher:addListener(self, 12124, self.onRefreshFamilyStoreReply)

    EventDispatcher:addListener(self, 12144, self.onGetFamilyStoreInfoReply)
    EventDispatcher:addListener(self, 12145, self.onBuyStoreItemReply)

    EventDispatcher:addListener(self, 12142, self.onGetFamilyStoreInfoReply)
    EventDispatcher:addListener(self, 12143, self.onBuyStoreItemReply)

    EventDispatcher:addListener(self, 12157, self.onGetFamilyStoreInfoReply)
    EventDispatcher:addListener(self, 12158, self.onBuyStoreItemReply)

	EventDispatcher:addListener(self, "day_passed",self.onDayPassed)
end

function This:removeListener()
	EventDispatcher:removeListener(self, 11305)
	EventDispatcher:removeListener(self, 11306)
	EventDispatcher:removeListener(self, 11307)
	EventDispatcher:removeListener(self, 11308)
	EventDispatcher:removeListener(self, 11309)
	EventDispatcher:removeListener(self, 11310)
	EventDispatcher:removeListener(self, 11311)
	EventDispatcher:removeListener(self, 11315)
	EventDispatcher:removeListener(self, 11316)
	EventDispatcher:removeListener(self, 12122)
	EventDispatcher:removeListener(self, 12123)
	EventDispatcher:removeListener(self, 12124)
    EventDispatcher:removeListener(self, 12144)
    EventDispatcher:removeListener(self, 12145)
    EventDispatcher:removeListener(self, 12142)
    EventDispatcher:removeListener(self, 12143)
    EventDispatcher:removeListener(self, 10609)
	EventDispatcher:removeListener(self, "day_passed")
end

function This:initDataFromConfig()
end

-- 根据类型获取商店实例
function This:getStoreBy( storeType, isCheckUpdate )
	local ret = self.m_stores[storeType or StoreType.kJianghun]
    if ret and isCheckUpdate  then
    	ret:updateState()
		if not ret:isUpdating() then 
			return nil 
		end
	end
	return ret
end

-- 增加
function This:createStore( storeType, info, slotFactory )
	local className = self.m_creator[storeType]
	if className then
		local classObj = require(className)
		if classObj ~= true then
			return classObj:create(info, slotFactory)
		end
	end
end

-- 删除
function This:removeStore( storeType )
	local ret = self.m_stores[storeType]
	if ret then
		ret:setEndTime(0)
    	ret:updateState()
	end
end

-- 更新商店信息
function This:addStore( info )
    local store = self:getStoreBy(info.shop_type)
	if not store then
		store = self:createStore(info.shop_type, info)
		self.m_stores[info.shop_type] = store
        self:notifyObservers("addStore", store)
	else
		store:updateInfo(info)
        self:notifyObservers("updateStore", store)
	end
	return info.shop_type, store
end

-- 商店红点判断方法
function This:hasStoreRedDot( storeType )
	local mysteryModelType  
	if StoreType.kJianghun == storeType then
		mysteryModelType = StoreType.kJianghunMystery
	else
		mysteryModelType = StoreType.kZhangongMystery
	end
	
	local storeMysteryModel = self:getStoreBy(mysteryModelType)
    if storeMysteryModel then
    	return storeMysteryModel:hasFreeItemNotBought() ~= nil
    end

    local store = self:getStoreBy(storeType)
    if store then
    	return not store:hasNextUpdate()
    end
end

-----------------------------------------------------------------------------------------------
function This:onGetStoreInfoRequest( shopType, quietly )
    local cmd = 11305
	local param = { shop_type = shopType }
	if shopType == StoreType.kZhangongMystery or shopType == StoreType.kJianghunMystery then
		cmd = 11308
	elseif shopType == StoreType.kJianghunBaoge or shopType == StoreType.kRankBaoge or shopType == StoreType.kMingWangBaoge then
		cmd = 11310
	elseif shopType == StoreType.kArenaBaoge then
		cmd = 12206
	elseif shopType == StoreType.kArena then
		cmd = 12208
	elseif shopType == StoreType.kFamilyContribution then
		cmd = 12122
		param.type = shopType
    elseif shopType == StoreType.kFamilyShare then
        cmd = 12144
    elseif shopType == StoreType.kFamilyLevel then
        cmd = 12142
	elseif shopType == StoreType.kFamilyFight then
        cmd = 12157
    end

	if not quietly then
    	Socket:sendAndLock(cmd, param)
    else
    	Socket:send(cmd, param)
    end
end

function This:onBuyStoreItemRequest( shopType, slotIndex, itemId, orderId, priceType, priceNum)
    local cmd = 11306
    local param = { shop_type = shopType, index = slotIndex, award_id = itemId, order = orderId }
	if shopType == StoreType.kZhangongMystery or shopType == StoreType.kJianghunMystery or shopType == StoreType.kJifenMystery or shopType == StoreType.kAnimalMystery then
		cmd = 11309
	elseif shopType == StoreType.kJianghunBaoge or shopType == StoreType.kRankBaoge or shopType == StoreType.kMingWangBaoge then
		cmd = 11311
	elseif shopType == StoreType.kArenaBaoge then
		cmd = 12207
		param.item_id = itemId
	elseif shopType == StoreType.kArena then
		cmd = 12209
	elseif shopType == StoreType.kFamilyContribution then
		cmd = 12123
		param.item_id = itemId
		param.type = shopType
	elseif shopType == StoreType.kFamilyShare then
		cmd = 12145
		param.item_id = itemId
		param.type = shopType
        param.price_type = priceType
        param.price_num = priceNum
	elseif shopType == StoreType.kFamilyLevel then
		cmd = 12143
		param.item_id = itemId
		param.type = shopType
	elseif shopType == StoreType.kFamilyFight then
		cmd = 12158
        param.order = orderId
		param.index = slotIndex
	elseif shopType == StoreType.kHeroEquipShop then
		cmd = 10609
		param.pos = slotIndex
		param.item_id = itemId
	end
	Socket:sendAndLock(cmd, param)
end

-- 购买商店商品
function This:onBuyStoreItemReply( msg )
	if msg.err_code == 0 then
		if msg.queue_list and #msg.queue_list ~= 1 then
			buglyReportLuaException(Serialize[64401],table.tostring(msg))
			return
        end

		if msg.shop_item_list and #msg.shop_item_list ~= 1 then
			buglyReportLuaException(Serialize[64402],table.tostring(msg))
			return
        end

        if msg.cmd == 12207 then
		    msg.shop_type = StoreType.kArenaBaoge 
	    elseif msg.cmd == 12209 then
		    msg.shop_type = StoreType.kArena
	    elseif msg.cmd == 12123 then
		    msg.shop_type = StoreType.kFamilyContribution
        elseif msg.cmd == 12145 then
            msg.shop_type = StoreType.kFamilyShare
        elseif msg.cmd == 12143 then
            msg.shop_type = StoreType.kFamilyLevel
	    elseif msg.cmd == 12158 then
			msg.shop_type = StoreType.kFamilyFight
		elseif msg.cmd == 10609 then
			msg.shop_type = StoreType.kHeroEquipShop
	    end

		local storeType, store = self:addStore(msg)
		if storeType == StoreType.kHeroEquipShop then
			self:notifyObservers("onBuyHeroQuipStoreItem")
			return
		end
		if store then
			if msg.favour then
				store:setFavour(msg.favour)
				self:notifyObservers("onFavourChange", storeType)
			end
		end

        local id = msg.queue_list[1].index
        if storeType == StoreType.kZhangongMystery or storeType == StoreType.kJianghunMystery or storeType == StoreType.kJifenMystery or storeType == StoreType.kAnimalMystery then
	        self:notifyObservers("onBuyMysteryStoreItem", storeType, id)
			EventDispatcher:dispatcher("store_state_changed", storeType)
		elseif storeType == StoreType.kJianghunBaoge or storeType == StoreType.kRankBaoge or storeType == StoreType.kMingWangBaoge then
    		self:notifyObservers("onBuyBaogeStoreItem", storeType, id)
		elseif storeType == StoreType.kArenaBaoge then
    		self:notifyObservers("onBuyArenaBaogeStoreItem", storeType, id)
		elseif storeType == StoreType.kArena then
    		self:notifyObservers("onBuyArenaStoreItem", storeType, id)
		elseif storeType == StoreType.kFamilyContribution or storeType == StoreType.kFamilyLevel or storeType == StoreType.kFamilyShare or storeType == StoreType.kFamilyFight then
			self:notifyObservers("onBuyFamilyStoreItem", storeType, id)
		else
	        self:notifyObservers("onBuyCommonStoreItem", storeType, id)
			EventDispatcher:dispatcher("store_state_changed", storeType)
		end
	else
		g_scene:handleErrorCode( msg.err_code )
	end
end

-- 获取普通商店信息
function This:onGetCommonStoreInfoReply( msg )
	if msg then
		local storeType, store = self:addStore(msg)
		self:notifyObservers("onGetCommonStoreInfo", storeType)
		EventDispatcher:dispatcher("store_state_changed")
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

-- 刷新普通商店
function This:onRefreshCommonStoreRequest( storeType )
	Socket:sendAndLock(11307, { shop_type = storeType})
end

function This:onRefreshCommonStoreReply( msg )
	if msg.err_code == 0 then
		if msg.favours then
			local store = self:getStoreBy( msg.shop_type )
			if store then
                self:onGetStoreInfoRequest(msg.shop_type)
				store:setFavour(msg.favours)
				self:notifyObservers("onFavourChange", msg.shop_type)
			end
			EventDispatcher:dispatcher("store_state_changed",msg.shop_type)
		end
        self:notifyObservers("onRefreshCommonStore", msg.shop_type)
	else
		g_scene:handleErrorCode( msg.err_code )
	end
end

-- 获取神秘商店信息
function This:onGetMysteryStoreInfoReply( msg )
	if msg and (not msg.err_code or msg.err_code == 0) then
		local storeType, store = self:addStore(msg)
		if store and store:getLeftTime() <= 0 then
			self:removeStore(storeType)
			print("请求神秘商店返回数据有误")
			print("时间戳:" .. msg.last_time, ",间隔:" .. msg.left_time)
			return
		end
		self:notifyObservers("onGetMysteryStoreInfo", storeType)
		EventDispatcher:dispatcher("store_state_changed",msg.shop_type)
	else
		g_scene:handleErrorCode(msg.err_code or 1)
	end
end

-- 离开神秘商店的backcall
function This:leaveMysteryStore( storeType )
	self:onLeaveMysteryStoreReply( {err_code = 0, shop_type = storeType} )
end

-- 离开神秘商店
function This:onLeaveMysteryStoreRequset( shopType )
	Socket:sendAndLock(11312, {shop_type = shopType})
end

function This:onLeaveMysteryStoreReply( msg )
	if msg.err_code == 0 then
        if msg.shop_type then
		    self:removeStore( msg.shop_type )
        end
		self:notifyObservers("onLeaveMysteryStore", msg.shop_type)
		EventDispatcher:dispatcher("store_state_changed", msg.shop_type)
	else
		g_scene:handleErrorCode( msg.err_code )
	end
end

-- 获取宝阁商店信息
function This:onGetBaogeStoreInfoReply( msg )
	if msg then
		local storeType, store = self:addStore(msg)
		self:notifyObservers("onGetBaogeStoreInfo", storeType)
	else
		g_scene:handleErrorCode("")
	end
end

-- 获取竞技场荣誉商店
function This:onGetArenaBaogeStoreInfoReply( msg )
	if msg then
		msg.shop_type = StoreType.kArenaBaoge
		local storeType, store = self:addStore(msg)
		self:notifyObservers("onGetArenaBaogeStoreInfo", storeType)
	else
		g_scene:handleErrorCode("")
	end
end

function This:onGetArenaStoreInfoReply( msg )
	if msg then
		msg.shop_type = StoreType.kArena
		local storeType, store = self:addStore(msg)
		self:notifyObservers("onGetArenaStoreInfo", storeType)
	else
		g_scene:handleErrorCode("")
	end
end

-- 军团商店
function This:onGetFamilyStoreInfoReply( msg )
    if msg.err_code == 0 then
	    if msg.cmd == 12122 then
		    msg.shop_type = StoreType.kFamilyContribution
        end
		local storeType, store = self:addStore(msg)
        self:notifyObservers("onGetFamilyStoreInfo", storeType)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 
function This:onRefreshFamilyStoreReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onRefreshFamilyStore", msg.shop_type)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetJinLianJifenStoreInfoRequest()
	Socket:sendAndLock(11315, {})

end

function This:onGetJinLianJifenStoreInfoReply(msg)
    if msg.err_code == 0 then
        self:notifyObservers("onRefreshCompetionStore", msg.queue_list)
	else
        g_scene:handleErrorCode(msg.err_code)		
	end
end

function This:onBuyJinLianJifenStoreItemRequest(id)
	Socket:sendAndLock(11316, {trench = id})
end

function This:onBuyJinLianJifenStoreItemReply(msg)
    if msg.err_code == 0 then
        self:notifyObservers("onRefreshCompetionStore", msg.queue_list)
	else
        g_scene:handleErrorCode(msg.err_code)		
	end

end


function This:onDayPassed()
	self:onGetStoreInfoRequest( StoreType.kArena )
end

-- function This:getGameWarShopData()
-- 	if not self.m_ratingData then
-- 		self.m_ratingData = GameRatingData:create(0)
-- 	end
-- 	return self.m_ratingData
-- end