-- @Author: superm
-- @Date:   2017-11-25 13:21:03
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-11 15:14:02
-- @File Name: StoreMysteryUI.lua
-- @Description: 

local This, super = class(..., "NodeView")

local StoreManager = require("Store.StoreManager")
local StoreMysteryItemView = require("Store.StoreMysteryItemView")
local GameUtil = require("Util.GameUtil")

function This:initData(store)
	self.m_store = store
	self.m_curViews = {}
	self.m_inited = false
    self:initCsNode()
end

function This:initCsNode()
	local csfileName = self.m_store:getCSFileName() 
	local csNode = cc.Node:createFromCSFile(csfileName)
    self.m_csNode = csNode
    self:addChild(csNode)
	csNode.scrollView:setDirection("horizontal")
	csNode.scrollView.m_direction = "right"
end

function This:initNode()
	self:updateTime(self.m_store:getLeftTime())
    self:refreshTipsNode()
    self:refreshFavourNode()
    self:refreshItemList()
    self.m_inited = true
end

function This:refreshFavourNode()
    local csNode = self.m_csNode
    local favour, favourLimit = self.m_store:getFavour()
    csNode.haogandu:setString("" .. favourLimit .. "/" .. favourLimit)
    csNode.jindutiao:setPercent(100)
    csNode.jindutiao:setVisible(true)
end

function This:refreshTipsNode()
    local csNode = self.m_csNode
	csNode.tips:setString(Serialize[64409] .. self.m_store:getNextFavourLimit() .. Serialize[64410])
end

function This:updateTime(leftTime)
    local csNode = self.m_csNode
	csNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"], leftTime))
end

function This:refreshItemList(refresh)
    local csNode = self.m_csNode
	if not refresh then
	    csNode.scrollView:removeAllItems()
	    csNode.scrollView:setTouchEnabled(true)

		local slots = self.m_store:getAllObjects()
		for index, slot in ipairs(slots) do
			local view = StoreMysteryItemView:create(slot)
			self.m_curViews[index] = view
	        local w , h = view:getContentSize()
	        csNode.scrollView:pushBackCustomItem(view, w, h)
		end
	else
		local slots = self.m_store:getAllObjects()
		for index, slot in ipairs(slots) do
			local view = self.m_curViews[index]
			view:refresh()
		end
	end
end

function This:onEnter()
    super.onEnter(self)
    StoreManager:addObserver(self)
    self.m_store:addObserver(self)
end

function This:onExit()
	self.m_store:removeObserver(self)
    StoreManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
	--if not self.m_inited then
	--    local csNode = self.m_csNode
	--	csNode.leave:setTouchEnded(function()
	--		self:showWarming()
	--	end)
	--	self.m_inited = true
	--end
	local csNode = self.m_csNode
	csNode.leave:setTouchEnded(function()
		self:showWarming()
	end)
end

function This:refresh()
	if self.m_inited then
	    self:refreshItemList(true)
	end
end

function This:showWarming()
	if self.m_store:hasItemNotBought() then
		local panel = g_scene:openDialog("UI.ConfirmPanel", {fileName = "MenuScene/NodeUI/QualifyingMysteryShopTip.csb"})
		panel:setString(Serialize[64411] .. self.m_store:getNextFavourLimit() .. Serialize[64412], 560)
		panel:setOkHandler(function()
			StoreManager:onLeaveMysteryStoreRequset(self.m_store:getId())
		end)
	else
		StoreManager:onLeaveMysteryStoreRequset(self.m_store:getId())
	end
end

function This:onStoreUpdateEnd( storeType )
    if self.m_store:getId() == storeType then
        StoreManager:leaveMysteryStore( storeType )
    end
end