-- @Author: Su
-- @Date:   2019-12-09 16:37:15
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-11 19:10:02
-- @File Name: StoreRuinsUI.lua
-- @Description: 

local This, super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/RuinsShopBg.csb"

local StoreManager = require("Store.StoreManager")
local ExpeditionStoreItem = require("Expedition.ExpeditionStoreItem")
local ItemNode = require("UI.ItemNode")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")

function This:initData(store)
	self.m_store = store
	self.m_curViews = {}
	self.m_inited = false
	self.m_curCostItemNum = 0
	self.m_curRefreshFreeTime = 0
	self.m_storeFavour = 0
    -- if  self.m_store.m_storeType == StoreType.kJifen or self.m_store.m_storeType == StoreType.kBattlefieldShop then
    --     self.bg3:setVisible(false)
    -- end
    self:initNode()
end

function This:initNode()
	-- self.title:setString(self.m_store:getName())

    --获取刷新令的数量
    self:refreshCostItem()
    --设置免费次数、免费刷新时间、好感度
    self:refreshTimeNode()
    self:refreshFavourNode()
    self:refreshItemList()
end

function This:onEnter()
    super.onEnter(self)
    StoreManager:addObserver(self)
    self.m_store:addObserver(self)
end

function This:onExit()
	self.m_store:removeObserver(self)
    StoreManager:removeObserver(self)
    if self.m_curViews then
    	for _, view in pairs(self.m_curViews) do
    		if view and view[0] ~= 0 then
    			view:release()
    		end
    	end
    end
    super.onExit(self)
end

function This:initTouchEvent()
	-- if not self.m_inited then
		self.cleanbutton:setTouchEnded(function()
			self.m_store:requestRefresh()
		end)

		-- self.tips:setTouchEnded(function()
		-- end)
	    -- self.m_inited = true
	-- end
end

function This:needCost()
	return self.m_curCostItemNum <= 0 and self.m_curRefreshFreeTime <= 0
end

function This:refreshFavourNode()
end

function This:refreshCostItem()
    local _, id, number = self.m_store:isCostEnoughBy(0)
    self.m_curCostItemNum = number
    return number > 0
end

function This:refreshTimeNode()
    local refreshTimes, refrshMaxTimes = self.m_store:getRefreshTimes()
    self.m_curRefreshFreeTime = refreshTimes
    if refreshTimes >= 0 and refreshTimes < refrshMaxTimes then
		self.free_text:setVisible(true)
	    local leftTime = self.m_store:getLeftTime()
	    self.free_text:setString(GameUtil:formatTime(Serialize[64417], leftTime))
	else
		self.free_text:setVisible(false)
    end

	if refreshTimes > 0 then 
		self.cleanbutton.text:setString(Serialize[62005] .. refreshTimes .. ")")
	else
		local _, id, number = self.m_store:isCostEnoughBy(0)
		local hasCostItem = number > 0
		if hasCostItem then
	    	self.cleanbutton.text:setString(Serialize[64377])
	    else
	    	self.cleanbutton.text:setString(Serialize[62006])
		end
	end
end

function This:updateTime(leftTime)
	self.free_text:setString(GameUtil:formatTime(Serialize[64417], leftTime))
end


function This:refreshItemList(refresh)
    self.ScrollView:removeAllItems()
    self.ScrollView:setTouchEnabled(true)

	local slots = self.m_store:getAllObjects()
	for index, slot in ipairs(slots) do
		local view = self.m_curViews[index]
		if not view then
			view = ExpeditionStoreItem:create(slot)
			self.m_curViews[index] = view
			view:retain()
		else
			view:refresh()
		end
        local w , h = view:getContentSize()
        self.ScrollView:pushNode(view, w, h, 2)
	end
end

function This:refresh()
	if self.m_inited then
	    local ret = self:refreshCostItem()
	    self:refreshTimeNode()
	    self:refreshItemList(true)
        self:refreshFavourNode()
	end
end

----------------------------------------------------------------
function This:onStoreNewUpdateStart( storeType )
    if self.m_store:getId() == storeType then
        self:refreshTimeNode()
    end
end

function This:onStoreUpdateEnd( storeType )
    if self.m_store:getId() == storeType then
        self:refreshTimeNode()
    end
end

function This:onRefreshCommonStore( storeType )
	if self.m_store:getId() == storeType then
        self:refreshCostItem()
    end
end

function This:onGetCommonStoreInfo(storeType)
	if self.m_store:getId() == storeType then
		self:refreshTimeNode()
		self:refreshItemList()

	end
end

function This:onFavourChange(storeType)
	if self.m_store:getId() == storeType then
		self:refreshTimeNode()
		self:refreshItemList()

	end
end

function This:onBuyCommonStoreItem(storeType,id)
	if self.m_store:getId() == storeType then
		local view = self.m_curViews[id]
		local itemId,num = view:getReward()
		local reward = {{itemId,num}}
		g_scene:showReward(reward)
	end
end