-- @Author: superm
-- @Date:   2018-04-04 18:54:57
-- @Last Modified by:   superm
-- @Last Modified time: 2018-04-19 16:37:31
-- @File Name: StoreSlotArenaModel.lua
-- @Description: 

local This , super = class(...,"Store.StoreSlotBaseModel")
local ArenaManager = require("Arena.ArenaManager")
local StoreManager = require("Store.StoreManager")
local ArenaFirstShopConfig = require("Metadatas.ArenaFirstShopConfig")
local ArenaCommonShopConfig = require("Metadatas.ArenaCommonShopConfig")

local StoreArenaSlotConfigs = {
    [StoreType.kArena] = ArenaCommonShopConfig,
    [StoreType.kArenaBaoge] = ArenaFirstShopConfig,
}

function This:initSelf( info )
    self.m_storeType = info.shop_type
    self.m_orderId = info.id
    self.m_itemId = info.item_id
    self.m_ID = info.index 
    self.m_buyTime = info.buy_times or 0
    if info.config then
        self.m_config = info.config 
    end
    if not self.m_config then
        self.m_config = StoreArenaSlotConfigs[self.m_storeType][self.m_ID]
    end
    -- if self.m_config then
    --     self.m_config.id = self.m_ID
    -- end
    self.m_itemNum = info.item_num
    self.m_costId = info.cost_id
    self.m_costNum = info.cost_count
    self.m_discount = info.discount
    if info.rank_id then
        self.m_condType = TerrConditionType.kArenaRank
        self.m_condLevel = info.rank_id
    end
end

function This:isNotLockedBy( cond1, cond2 )
    if cond1 == TerrConditionType.kArenaRank then
        local rankLev =  ArenaManager:getRankIndex() or 0
        return rankLev > 0 and rankLev <= cond2
    elseif cond1 == TerrConditionType.kNone then
        return true
    end
end

function This:getItemNum()
    if self.m_itemNum then
        return self.m_itemNum
    elseif self.m_config then
        return self.m_config.item_num
    end
    return 0
end

function This:getCondition()
    if self.m_condType then
        return self.m_condType, self.m_condLevel
    elseif self.m_config and self.m_config.need then
        return TerrConditionType.kArenaRank, self.m_config.need
    else
        return TerrConditionType.kNone, 0
    end
end

function This:getDiscount()
    if self.m_discount then
        return self.m_discount > 0 and self.m_discount or 10
    elseif self.m_config and self.m_config.discount then
        return self.m_config.discount > 0 and self.m_config.discount or 10
    end
    return 10
end

function This:getPriceType()
    if self.m_costId then
        return self.m_costId
    elseif self.m_config then
        return self.m_config.price_id
    end
    return 0
end

function This:getPrice()
    if self.m_costNum then
        return self.m_costNum
    elseif self.m_config then
        return self.m_config.price_num
    end
    return 0
end

function This:doRequestBuy()
    StoreManager:onBuyStoreItemRequest(self.m_storeType, self.m_ID, self.m_itemId)
end

function This:setView( view )
    if self.m_view and self.m_view ~= view then
        self.m_view:release()
        self.m_view = nil
    end

    if view then
        self.m_view = view
        view:retain()
    end
end

function This:getView()
    return self.m_view
end

function This:showLockedTips()
    local cType, arg = self:getCondition()
    if cType > 0 and arg then 
        g_scene:showTips(string.format(Serialize[64419], arg))
    end
end

