-- @Author: superm
-- @Date:   2018-01-15 15:51:07
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-10 18:00:57
-- @File Name: StoreSlotBaseModel.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local ItemUtil = require("Bag.ItemUtil")
local GamePlayer = require("Player.GamePlayer")
local DepotManager = require("Depot.DepotManager")
local StoreManager = require("Store.StoreManager")
local JRareStoreConfig = require("Metadatas.JRareStoreConfig")
local OfficeStoreConfig = require("Metadatas.OfficeStoreConfig")
local MatchManager = require("Match.MatchManager")
local GameUtil = require("Util.GameUtil")

local StoreSlotConfigs = {
    [StoreType.kJianghun] = nil,
    [StoreType.kJianghunBaoge] = JRareStoreConfig,
    [StoreType.kRankBaoge] = OfficeStoreConfig,
    [StoreType.kZhangong] = nil,
    [StoreType.kJianghunMystery] = nil,
    [StoreType.kZhangongMystery] = nil,
    [StoreType.kJifen] = nil,
	[StoreType.kJifenMystery] = nil,	
}

function This:init( info )
    super.init(self)
    if info then
        self.m_store = nil
        self:initSelf( info )
        self:updateState()
    else
        self.m_ID = 0
        self.m_state = StoreObjectState.kNormal
    end
end

function This:initSelf( info )
    self.m_storeType = info.shop_type
    self.m_orderId = info.order
    self.m_itemId = info.award_id
    self.m_ID = info.index 
    self.m_buyTime = info.buy_times or 0
    if info.config then
        self.m_config = info.config 
    end
    if not self.m_config and StoreSlotConfigs[self.m_storeType] then
        self.m_config = StoreSlotConfigs[self.m_storeType][self.m_ID]
    end
    -- if self.m_config then
    --     self.m_config.id = self.m_ID
    -- end
    self.m_itemNum = info.count
    self.m_costId = info.cost_id
    self.m_costNum = info.cost_count
    self.m_favour = info.favour
    self.m_discount = info.discount
    if info.cond_type then
        self.m_condType = info.cond_type
        self.m_condLevel = info.cond_level
    end
end

function This:updateInfo( info )
    self:updateState(true, info)
end

function This:updateState(isInit, info)
    local ret = false
    if isInit then
        self:initSelf(info)
        ret = true
    end
    return self:updateStateByConds()
end

function This:updateStateByConds()
    local oldState = self.m_state
    local condType, level = self:getCondition()
    if not condType then
        self.m_state = StoreObjectState.kNormal
    else
        local ret = self:isNotLockedBy(condType, level)
        self.m_state = ret and StoreObjectState.kNormal or StoreObjectState.kLocked
    end
     
    return oldState ~= self.m_state
end

function This:getCondition()
    if self.m_condType then
        return self.m_condType, self.m_condLevel
    elseif self.m_config then
        return self.m_config.cond_type, self.m_config.cond_level
    else
        return TerrConditionType.kNone, 0
    end
end

function This:getBuyCondition()
    return TerrConditionType.kNone, 0
end

function This:setId( id )
    self.m_ID = id
end

function This:getId()
    return self.m_ID
end

-- function This:getOrderId()
--     return self.m_orderId
-- end

--道具ID
function This:getConfigId()
    return self.m_itemId
end

function This:isNotLockedByCondList(condlist)
    if not condlist or condlist == 0 then
        return true
    else
        local ret = true
        if not isNestedObject(condlist) then
            if #condlist >= 2 then
                ret = self:isNotLockedBy(condlist[1], condlist[2])
            end
        else
            for i, cond in ipairs(condlist) do
                ret = ret and self:isNotLockedBy(cond[1], cond[2])
                if not ret then
                    break
                end
            end
        end
        return ret
    end
end

function This:isNotLockedBy( cond1, cond2 )
    if cond1 == TerrConditionType.kPlayerLev then
        local level = GamePlayer:getLevel()
        return level >= cond2
    elseif cond1 == TerrConditionType.kTerrMainBuildLev then
        local level = TerrBuildManager:getBuildLevel()
        return level >= cond2
    elseif cond1 == TerrConditionType.kRankLev then
        local level = MatchManager:getMaxRankLvInHistory() or MatchManager:getRankLv()
        return level >= cond2
    elseif cond1 == TerrConditionType.kComplex then
        return self:isNotLockedByCondList(cond2)
    elseif cond1 == TerrConditionType.kFamilyTitle then
        local FamilyManager = require("Family.FamilyManager")
        local value = FamilyManager:getMySelfTitle()
        if not value or value <= 0 then
            return
        end
        local vtype = type(cond2)
        if vtype == "number" then
            return value == cond2
        elseif vtype == "table" then
            return table.contain(cond2, value)
        end
        return
    elseif cond1 == TerrConditionType.kNone then
        return true
    end
end

function This:isSameType( info )
    return self.m_ID == info.index and self.m_itemId == info.award_id
end

function This:isCostEnoughBy()
    local id = self:getPriceType()
    local num = self:getDiscount() / 10 * self:getPrice() 
    return DepotManager:getItemOrResCount(id) >= num, id, num
end

function This:isLocked()
    return (self.m_state == StoreObjectState.kLocked) or (not self.m_itemId)
end

function This:isOpening()
    return self.m_state == StoreObjectState.kNormal
end

function This:isBuyLocked()
    return false
end

function This:isMaxTime()
    local maxValue = self:getMaxBuyTime()
    if maxValue > 0 then
        return self.m_buyTime >= self:getMaxBuyTime()
    else
        return false
    end
end

function This:getLeftBuyTimes()
    local maxValue = self:getMaxBuyTime()
    if maxValue > 0 then
        local ret = self:getMaxBuyTime() - self.m_buyTime
        if ret < 0 then
            return 0
        end
        return ret
    end
end

function This:getBuyTime()
    return self.m_buyTime
end

function This:getMaxBuyTime()
    if self.m_config and self.m_config.buy_times then
        return self.m_config.buy_times
    end
    return 1
end

function This:requestBuy()
    if self:isLocked() then
        self:showLockedTips()
        return 
    end

    if self:isBuyLocked() then
        self:showBuyLockedTips()
        return 
    end

    local store = self:getStore()
    local state, id, num = self:isCostEnoughBy() 
    if state then
        if id ~= 102 then
            self:doRequestBuy()
        else
            g_scene:openDialog("UI.CostConfirmPanel", {
                cost = num,
                desc = Serialize[60495] .. self:getName() , 
                okHandler = function()
                    self:doRequestBuy()
                end })
        end
    else
        if id == 102 then
            g_scene:handleErrorCode("diamondLack")
        elseif id == 101 then
            g_scene:handleErrorCode("goldLack")
        else
            g_scene:showTips(Serialize[60714] .. ItemUtil.getName(id) .. Serialize[60715])
        end
    end
end

function This:doRequestBuy()
    StoreManager:onBuyStoreItemRequest(self.m_storeType, self.m_ID, self.m_itemId, self.m_orderId)
end

function This:getStoreType()
    return self.m_storeType
end

function This:setStore( store )
    self.m_store = store
    if store then
        self.m_storeType = store:getId()
    end
end

function This:getStore()
    return self.m_store
end

function This:isSpall()
    local itype = ItemUtil.getType(self.m_itemId)
    return itype == ItemCommonType.kSpall
end

function This:getNumInDepot()
    local item = DepotManager:getItem(self.m_itemId)
    if not item then
        item = ItemUtil.createItemModel({type_id=self.m_itemId, num=0})
    end
    if item:isSpall()  then
        return item:getCount(), item:getSynthesizeCost()
    end
    return item:getCount(), 0
    
end


--获取配置信息region begin
function This:getDiscount()
    if self.m_discount then
        return self.m_discount > 0 and self.m_discount or 10
    elseif self.m_config then
        return self.m_config.discount > 0 and self.m_config.discount or 10
    end
    return 10
end

function This:getItemNum()
    if self.m_itemNum then
        return self.m_itemNum
    elseif self.m_config then
        return self.m_config.count
    end
    return 0
end

function This:getFavour()
    if self.m_favour then
        return self.m_favour
    elseif self.m_config then
        return self.m_config.favour
    end
    return 0
end

function This:getPriceType()
    if self.m_costId then
        return self.m_costId
    elseif self.m_config then
        return self.m_config.cost_id
    end
    return 0
end

function This:getPrice()
    if self.m_costNum then
        return self.m_costNum
    elseif self.m_config then
        return self.m_config.cost_count
    end
    return 0
end

function This:getItemId()
    return self.m_itemId
end

function This:getName()
    return ItemUtil.getName(self:getConfigId())
end

function This:getPriceImage()
    local id = self:getPriceType()
    return ItemUtil.getIconImage(id)
end

function This:getItemImage()
    return ItemUtil.getIconImage(self:getConfigId())
end

function This:getItemQulityImage()
    return ItemUtil.getQualityImage(self:getConfigId())
end

function This:showLockedTips()
end

function This:showBuyLockedTips()
end



--获取配置信息region end

--endregion