-- @Author: superm
-- @Date:   2018-07-03 20:41:07
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-03 11:56:43
-- @File Name: StoreSlotFamilyModel.lua
-- @Description: 

local This , super = class(...,"Store.StoreSlotBaseModel")
local StoreManager = require("Store.StoreManager")
local FamilyBuildManager = require("Family.FamilyBuildManager")
local ContributionShopConfig = require("Metadatas.ContributionShopConfig")
local LevelShopConfig = require("Metadatas.LevelShopConfig")
local CommonShopConfig = require("Metadatas.CommonShopConfig")
local MilitaryShopConfig = require("Metadatas.MilitaryShopConfig")

local StoreFamilySlotConfigs = {
    [StoreType.kFamilyContribution] = ContributionShopConfig,
    [StoreType.kFamilyLevel] = LevelShopConfig,
    [StoreType.kFamilyShare] = CommonShopConfig,
    [StoreType.kFamilyFight] = MilitaryShopConfig,
}

function This:initSelf( info )
    self.m_storeType = info.shop_type
    self.m_orderId = info.order
    self.m_itemId = info.item_id
    self.m_ID = info.index 
    self.m_buyTime = info.buy_times or 0
    self.m_config = nil
    if info.config then
        self.m_config = info.config 
    end
    if not self.m_config then
        self.m_config = StoreFamilySlotConfigs[self.m_storeType][self.m_orderId]
    end
    self.m_itemNum = info.item_num
    self.m_costId = info.cost_id
    self.m_costNum = info.cost_count
    self.m_discount = info.discount
end

function This:updateState(isInit, info)
    local ret = super.updateState(self, isInit, info)
    local oldState = self.m_buyCondType
    self.m_buyCondType, self.m_buyCondValue = TerrConditionType.kNone, 0
    local cType, cValue = self:getBuyCondition()
    if cType and cType ~= TerrConditionType.kNone then
        if cType ~= TerrConditionType.kComplex then
            if not self:isNotLockedBy(cType, cValue) then
                self.m_buyCondType, self.m_buyCondValue = cType, cValue
            end
        else
            for _, cond in ipairs(cValue) do
                local subCType, subCValue = cond[1], cond[2]
                if not self:isNotLockedBy(subCType, subCValue) then
                    self.m_buyCondType, self.m_buyCondValue = subCType, subCValue
                    break 
                end
            end
        end
    end
    return ret or oldState ~= self.m_buyCondType
end

function This:getCurBuyCondition()
    return self.m_buyCondType, self.m_buyCondValue
end

function This:isNotLockedBy( cond1, cond2 )
    if cond1 == TerrConditionType.kFamilySelfBuildLev then
        local level =  FamilyBuildManager:getStoreSlotNum(self.m_storeType) or 0
        return level >= cond2
    elseif cond1 == TerrConditionType.kFamilyLevel then
        local level = FamilyBuildManager:getBuildingLevel(FamilyBuildType.kYiShiTing)
        return level >= cond2
    elseif cond1 == TerrConditionType.kNone then
        return true
    else
        return super.isNotLockedBy(self, cond1, cond2)
    end
end

function This:getItemNum()
    if self.m_itemNum then
        return self.m_itemNum
    elseif self.m_config then
        return self.m_config.item_num
    end
    return 0
end

function This:getDiscount()
    if self.m_discount then
        return self.m_discount > 0 and self.m_discount or 10
    elseif self.m_config and self.m_config.discount then
        return self.m_config.discount > 0 and self.m_config.discount or 10
    end
    return 10
end

function This:getPriceType()
    if self.m_costId then
        return self.m_costId
    elseif self.m_config then
        return self.m_config.money_id
    end
    return 0
end

function This:getPrice()
    if self.m_costNum then
        return self.m_costNum
    elseif self.m_config then
        return self.m_config.price[self.m_buyTime + 1]
    end
    return 0
end

function This:getPosition()
    return self.m_config.position
end

function This:getMaxBuyTime()
    if self.m_config and self.m_config.num then
        return self.m_config.num
    end
    return 1
end

function This:isShared()
    if self.m_config and self.m_config.common then
        return self.m_config.common == 1
    end
end

function This:requestBuy()
    if not FamilyBuildManager:isInNormalState() then
        return g_scene:showTips(Serialize[64427])
    end
    super.requestBuy(self)
end

function This:doRequestBuy()
    StoreManager:onBuyStoreItemRequest(self.m_storeType, self.m_ID, self.m_itemId, self.m_orderId)
end

function This:setView( view )
    if self.m_view and self.m_view ~= view then
        self.m_view:release()
        self.m_view = nil
    end

    if view then
        self.m_view = view
        view:retain()
    end
end

function This:getView()
    return self.m_view
end

function This:showLockedTips()
end


