-- @Author: superm
-- @Date:   2017-10-16 19:59:38
-- @Last Modified by:   superm
-- @Last Modified time: 2018-01-31 10:57:58
-- @File Name: GlodChangePanel.lua

local This, super = class(..., "Dialog")
local SystemManager = require("System.SystemManager")
local VipManager = require("Vip.VipManager")
local PrivilegeManager = require("Vip.PrivilegeManager")
-- local GameUtil = require("Util.GameUtil")

function This:initData(info)
    self.m_gain = SystemManager:getCurCoinHilt()
    self.m_count = 0
end

function This:onEnter()
    super.onEnter(self)
    SystemManager:addObserver(self)
    VipManager:addObserver(self)
    self:setLocalZOrder(g_scene:getMaxLocalZOrder() + 1)
end

function This:onExit()
    VipManager:removeObserver(self)
    SystemManager:removeObserver(self)
    super.onExit(self)
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/GoldExchangeMain.csb")
    self.m_csNode:setPosition(cc.visible.center.x, cc.visible.center.y)
    self:addChild(self.m_csNode)
    self:refresh() 
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.normal.exchange:setTouchEnded(function()
        SystemManager:onUseGlodChangeRequest()
    end)

    self.m_csNode.normal.multiExchange:setTouchEnded(function()
        local times = self.m_leftTime
        if times >= 5 then
            times = 5
        end
        self:openDialog("System.GlodMutilChangePanel", {times = times, gain = self.m_gain * times, cost = SystemManager:getGoldChangeCost(times)})
    end)

    self.m_csNode.vip:setTouchEnded(function()
        self:openDialog("Vip.VipPanel")
    end)
end

function This:refresh()
    local cost
    cost, self.m_curTimes = SystemManager:getGoldChangeCost()

    if cost then
        self.m_totalTimes  = PrivilegeManager:getPrivilegeNumBy(Privilege.kGlodChangeTimes)
        self.m_leftTime = self.m_totalTimes - self.m_curTimes
        local visible = self.m_leftTime > 0
        self.m_csNode.normal:setVisible(visible)
        self.m_csNode.note:setVisible(not visible)
        self.m_csNode.vip:setVisible(not visible)
        self.m_csNode.limit:setVisible(visible)
        if visible then
            self.m_csNode.normal.scount:setString(tostring(cost))
            self.m_csNode.normal.tcount:setString(tostring(self.m_gain)) 
        else
            self.m_leftTime = 0
        end
        self.m_csNode.limit:setString(Serialize[64434] .. tostring(self.m_leftTime) .. "/" .. tostring(self.m_totalTimes) .. "）")
    end
end

function This:onGetGlodChangeInfo()
    self:refresh()
end

local bgWidth
function This:addTxtItemView( diamond, gold, crit )
    if not bgWidth then
        bgWidth = self.m_csNode.record:getContentWidth()
    end

    local node = cc.Node:createFromCSFile("MenuScene/NodeUI/GoldExchangeItem.csb")
    
    node.number1:setString(tostring(diamond))
    node.number2:setString(tostring(gold))
    if crit > 1 then
        node.Crit:setImage("MenuScene/BaoJiX" .. tostring(crit) .. ".png")
        node.Crit:setVisible(true)
    end
    -- GameUtil:linearLayoutNodes(2, {node.text1, node.number1, node.icon1, node.text2, node.icon2, node.number2}, false, bgWidth)
    self.m_csNode.scrollView:pushNode(node, bgWidth, 23)
end

function This:playRewardAni(msg, i)
    local rewardList = {}
    table.insert(rewardList, {id = 101, num = msg.add_coin[i]})
    g_scene:showReward(rewardList)

    if msg.beilv[i] > 1 then
        local node = cc.Sprite:create("MenuScene/BaoJiX" .. tostring(msg.beilv[i]) .. ".png")
        node:setScale(2)
        node:setPosition(cc.visible.center.x, cc.visible.center.y)
        g_scene:addChild(node, 100001)
        local action = cc.Sequence(
            cc.DelayTime(0.1), 
            cc.CallFunc(function()
                self:addTxtItemView(msg.gold[i], msg.add_coin[i], msg.beilv[i]) 
                self.m_csNode.scrollView:scrollToBottom(0.5, false)
            end), 
            cc.Spawn(cc.FadeIn(0.07), 
            cc.ScaleTo(0.07, 1)),
            cc.DelayTime(0.2), 
            cc.Spawn(cc.FadeOut(0.6), cc.MoveBy(0.6, 0, 80, "BackOut")),
            cc.RemoveAction())
        node:runAction(action)
    else
        self:addTxtItemView(msg.gold[i], msg.add_coin[i], msg.beilv[i]) 
        self.m_csNode.scrollView:scrollToBottom(0.5, false)
    end
    print(string.format("第%d次花了%d元宝得到%d金， 倍数为%d", i, msg.gold[i], msg.add_coin[i], msg.beilv[i]))
end

function This:onUseGlodChange(msg)
    self:refresh()
    self.m_csNode.scrollView:setVisible(true)
    self.m_csNode.record:setVisible(true)
    self:playRewardAni(msg, 1)
    local times = msg.times
    for i=2, times do
        self:setTimer(0.6 * ( i - 1 ), function()
            self:playRewardAni(msg, i)
        end , "GoldExchangeItem" .. tostring(self.m_count + i) )
    end
    self.m_count = self.m_count + times
end


function This:onVipActivated( maxLevel )
    self:refresh()    
end
