local This, super = class(..., "Dialog")
This.openSound = "Open_Ui.mp3"
local SdkManager = require("Sdk.SdkManager")
local SystemManager = require("System.SystemManager")
local ViewUtil = require("UI.ViewUtil")
local GameUtil = require("Util.GameUtil")

function This:initData()
    self.m_effectRecharge = {}
end

function This:initNode()
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/RechargeMain.csb")
    self.m_csNode:setPosition(cc.visible.center.x, cc.visible.center.y)
    self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
    self.m_csNode:playCSAnimation(nil,true)
    self:refreshDiamondBuyReward()
    self:updateItemScrollView()
end

function This:initListener()
    SystemManager:addObserver(self)
end

function This:onExit()
    SystemManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.gantanhao:setTouchEnded(function()
        local text
        if self.m_showBuyReward then
            text = string.format(Serialize[64437], self.m_showBuyReward, self.m_showBuyReward)
        else
            text = Serialize[64438]
        end
        
        g_scene:openDialog("UI.HelpTips", {
            node = self.m_csNode.gantanhao, 
            text = text,
        })
    end)
end

function This:refreshDiamondBuyReward()
    local csNode = self.m_csNode
    local reward, progress, target = SystemManager:getCurDiamondBuyReward()
    self.m_showBuyReward = reward
    self.m_buyReward = false
    csNode.tips:stopAllActions()
    csNode.tips:setOpacity(255)
    csNode.ef:setVisible(false)
    csNode.text:setVisible(true)
    csNode.tips:setVisible(true)
    csNode.sanbei:setVisible(false)
    if reward then
        csNode.text:setString(tostring(progress) .. " / " .. target)
        if progress >= target then
            csNode.tips:setString(string.format(Serialize[64439], reward))
            csNode.tips:setVisible(false)
            ViewUtil:blinkView(csNode.tips, true)
            csNode.jindu2:setPercent(100)
            self.m_buyReward = true
            csNode.text:setVisible(false)
            csNode.ef:setVisible(true)
        else
            csNode.jindu2:setPercent(progress / target * 100)
            csNode.tips:setString(string.format(Serialize[64440], target - progress, reward))
        end
    elseif csNode.ef and SystemManager:hasHighPhaseBuyReward() then
        csNode.ef:setVisible(false)
        csNode.jindu1:setVisible(false)
        csNode.jindu2:setVisible(false)
        csNode.tips:setVisible(false)
        csNode.gantanhao:setVisible(false)
        csNode.text:setVisible(false)
        csNode.sanbei:setVisible(true)
    else
        csNode.text:setString("MAX")
        csNode.tips:setString(Serialize[64441])
    end
end

function This:updateItemScrollView()
    self.m_effectRecharge = {}
    self.m_csNode.scrollView:removeAllItems()
    self.m_csNode.scrollView:setTouchEnabled(true)
    local itemList = SystemManager:getShopItemList()
    for i, item in ipairs(itemList) do
        local node = cc.Node:createFromCSFile("MenuScene/NodeUI/RechargeItem.csb")
        node:setTouch({
            isContainer = true,
            onEnded = function()
                if self.m_buyReward then
                    g_scene:openDialog("UI.ConfirmPanel", {
                        fileName = "MenuScene/NodeUI/RechargeTips.csb",
                        title = string.format(Serialize[64442], self.m_showBuyReward),
                        richText = string.format(Serialize[64443], self.m_showBuyReward),
                        okBtnTxt = Serialize[64444],
                        okHandler = function ()
				            SdkManager:onApplyBillIDRequest(node.item.id)
                        end
                    })
                else
                    SdkManager:onApplyBillIDRequest(node.item.id)
                end
            end,
        })
        self:resetShopItem(node, item)
        self.m_csNode.scrollView:pushNode(node, 260, 200, 3)
    end
    -- self.m_csNode.scrollView:runItemAppearAction()
end

local function resetShopItem(node, item)
    if item then
        node.item = item
    else
        item = node.item
    end
    node.temp:setVisible(false and item.isDouble)
    node.double:setVisible(item.isDouble)
    if item.isDouble then
        node.double.text:setString(tostring(item.remainDoubleCount))
    end
    node.name:setString(item.name)
    node.tips:setString(item.tips)
    node.icon:setImage("Image/Recharge/" .. item.id .. ".png")
    node.price:setString(Serialize[60195] .. item.price)
end

function This:resetShopItem(node, item)
    local id = item.id
    node.reset = resetShopItem
    node:reset(item)
    local startIndex, count = 1, 0
    if self.m_buyReward then
        startIndex, count = 3, 1
        node.sanbei:setVisible(true)
        node.jindu:setVisible(false)        
    elseif not self.m_showBuyReward and item.tripleCount then
        local curTime = SystemManager:getLastPhaseRechargeTime(id)
        if curTime > item.tripleCount then
            curTime = ((curTime - 1) % item.tripleCount) + 1
        end
        local visible = curTime >= item.tripleCount
        node.sanbei:setVisible(visible)
        node.jindu:setVisible(not visible)
        if not visible then
            local percent = curTime / item.tripleCount * 100
            node.jindu.jindutiao:setPercent(percent)
            node.jindu.text:setString(string.format("%d/%d", curTime, item.tripleCount))
        else
            startIndex, count = 1, 2
        end
    else
        node.sanbei:setVisible(false)
        node.jindu:setVisible(false)
    end

    local hasEffect = node.sanbei:isVisible()
    local effectList = self.m_effectRecharge[id]
    if hasEffect and not effectList then
        local effectArray = {}
        for index=startIndex, startIndex + count - 1 do
            local effect = GameUtil:createEffect("SanBeiCZ" .. index, true, node.effect)
            node.effect:addChild(effect)
            effectArray[index] = effect
        end
        effectList = effectArray         
    elseif not hasEffect and effectList then
        for _, effect in ipairs(effectList) do
            effect:removeFromParent()
        end
        effectList = nil
    end
    if self.m_effectRecharge[id] ~= effectList then
        self.m_effectRecharge[id] = effectList
    end
end

function This:onBuyShopItem( itemList )
    self:refreshDiamondBuyReward()
    self:updateItemScrollView()
    if itemList then
        g_scene:showReward(itemList)
    end
end