local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local VigorConfig = require("Metadatas.VigorConfig")
local ShopConfig = require("Metadatas.ShopConfig")
local CoinConfig = require("Metadatas.CoinConfig")
local CoinPriceConfig = require("Metadatas.CoinPriceConfig")
local ExcessConfig = require("Metadatas.ExcessConfig")
local GamePlayer = require("Player.GamePlayer")
local SdkManager = require("Sdk.SdkManager")

local kMaxBuyPowerTimes = #VigorConfig
local kMaxGoldChangeTimes = #CoinPriceConfig
local kBuyPowerPerTime = 60
local kDiamondItemId = 102

function This:init()
    super.init(self)
    self.m_diamondBuyTimeLists = {}
    self:initListener()
end

function This:initListener()
    EventDispatcher:addListener(self, 10204, self.onBuyPowerReply)
    EventDispatcher:addListener(self, 10205, self.onBuyShopItemReply)
    EventDispatcher:addListener(self, 11313, self.onGetGlodChangeInfoReply)
    EventDispatcher:addListener(self, 11314, self.onUseGlodChangeReply)
end

function This:getBuyPower()
    return kBuyPowerPerTime
end

--体力购买次数有没有超过限制
function This:canBuyPower()
    return GamePlayer:getPowBuyTime() < kMaxBuyPowerTimes
end

--体力是否已满
function This:getPowerIsFull()
    local res = GamePlayer:getPower() >= kMaxPower and self:getBuyPowerDiamondCost()==0
    return res
end

function This:getBuyPowerDiamondCost()
    local count = GamePlayer:getPowBuyTime() + 1
    count = math.min(count, kMaxBuyPowerTimes)
    return VigorConfig[count].diamond
end

function This:getDiamondBuyTotalTimes()
    return self.m_diamondBuyTotalTimes
end

function This:setDiamondBuyInfo(msg)
    self.m_diamondBuyTotalTimes = 0
    for i, id in ipairs(msg.diamond_id) do
        local num = msg.diamond_buy_times[i]
        self.m_diamondBuyTimeLists[id] = num
        self.m_diamondBuyTotalTimes = self.m_diamondBuyTotalTimes + num
    end
    print_tmp(msg)
    self:setDiamondBuyRewardInfo(msg)
    EventDispatcher:dispatcher("init_role_info")
end

function This:getDiamondBuyTime(itemId)
    if not self.m_diamondBuyTimeLists then
        return 0
    end
    return self.m_diamondBuyTimeLists[itemId] or 0
end

function This:setDiamondBuyTime(itemId)
    if not self.m_diamondBuyTimeLists then
        self.m_diamondBuyTimeLists={}
    end
    self.m_diamondBuyTimeLists[itemId] = time
end

function This:getCurDiamondBuyReward()
    local progress = 0
    local totalRecharge, lastRecharge = self.m_totalRecharge, self.m_lastRecharge
    local reward, target

    local maxId, totalTarget = 1, 0
    for id, config in ipairs(ExcessConfig) do
        maxId = id
        progress = totalRecharge - totalTarget
        if progress <= config.min_price or progress - lastRecharge < config.min_price then
            reward = config.multipleCount
            target = config.min_price
            break
        -- else
        --     totalTarget = totalTarget + config.price
        end
    end

    -- if not reward then
    --     local config = ExcessConfig[maxId]
    --     reward = config.multipleCount
    --     target = config.min_price
    -- end
    return reward, progress, target
end

function This:hasBuyReward( ... )
    local reward, progress, target = self:getCurDiamondBuyReward()
    self.m_hasLowPhaseBuyReward = reward
    if not reward then
        for _, config in ipairs(ShopConfig) do
            local tripleCount = config.tripleCount
            local curTime = self:getLastPhaseRechargeTime(config.id)
            if tripleCount and curTime > tripleCount then
                curTime = ((curTime - 1) % tripleCount) + 1
            end
            if tripleCount and curTime >= tripleCount then
                return true
            end
        end
    elseif progress >= target then
        return true
    end
end

function This:hasHighPhaseBuyReward()
    local ret = false
    for _, config in ipairs(ShopConfig) do
        if config.tripleCount and config.tripleCount > 0 then
            ret = true
            break
        end
    end
    return ret
end

function This:setDiamondBuyRewardInfo( msg )
    if msg then
        self.m_totalRecharge = msg.total_recharge_num or 0 
        self.m_lastRecharge = msg.last_recharge_num or 0
        if msg.charge_list then
            if not self.m_lastPhaseRechargeTimes then
                self.m_lastPhaseRechargeTimes = {}
            end 
            for _, info in ipairs(msg.charge_list) do
                self.m_lastPhaseRechargeTimes[info.id] = info.times or 0
            end
        end
        if self.m_totalRecharge == 0 and self.m_lastRecharge == 0 and self:getDiamondBuyTime(1) > 0 then
            self.m_totalRecharge = 0.99
            self.m_lastRecharge = 0.99
        end

    else
        self.m_lastPhaseRechargeTimes = {}
        self.m_totalRecharge = 0
        self.m_lastRecharge = 0
    end
end

function This:getLastPhaseRechargeTime(itemId)
    if not self.m_lastPhaseRechargeTimes then
        return 0
    end
    return self.m_lastPhaseRechargeTimes[itemId] or 0
end

function This:diamondBuyTimeAddOne(itemId)
    if not self.m_diamondBuyTimeLists then
        self.m_diamondBuyTimeLists = {}
    end

    local count = self.m_diamondBuyTimeLists[itemId] or 0
    self.m_diamondBuyTimeLists[itemId] = count + 1

    if not self.m_hasLowPhaseBuyReward then
        if not self.m_lastPhaseRechargeTimes then
            self.m_lastPhaseRechargeTimes = {}
        end
        local lcount  = self.m_lastPhaseRechargeTimes[itemId] or 0
        self.m_lastPhaseRechargeTimes[itemId] = lcount + 1
    end
end

function This:getShopItemList()
    local itemList = {}
    for id, config in ipairs(ShopConfig) do
        local item = {}
        item.id = id
        item.name = config.name
        item.icon = false
        item.remainDoubleCount = config.doubleCount - self:getDiamondBuyTime(id)
        item.isDouble = item.remainDoubleCount > 0
        item.price = config.price
        item.tripleCount = config.tripleCount
        local more = item.isDouble and config.value or config.more
        item.tips = string.format(Serialize[64446], more)
        table.insert(itemList, item)
    end
    return itemList
end

function This:getCurCoinHilt()
    local config = self.m_coinHiltConfig
    local updateConfig = function(level)
        local count
        for i, cfg in ipairs(CoinConfig) do
            count = i
            if level <= cfg.max_lv then
                break
            end
        end
        config = CoinConfig[count]
        self.m_coinHiltConfig = config
    end

    local level = GamePlayer:getLevel()
    if not config then
        updateConfig(level)
    elseif level > config.max_lv or level < config.min_lv then
        updateConfig(level)
    end
    return config.coin
end

function This:getGoldChangeCost(times)
    if self.m_goldChangeTimes then
        if self.m_goldChangeRefreshTime == os.date("*t", GamePlayer:getServerTime()).yday then
            local cost = 0
            times = times or 1
            for i=1, times do
                local nextTimes = self.m_goldChangeTimes + i
                nextTimes = nextTimes >= kMaxGoldChangeTimes and kMaxGoldChangeTimes or nextTimes
                cost = cost + CoinPriceConfig[nextTimes].gold
            end

            return cost, self.m_goldChangeTimes
        end
    end
    self:onGetGlodChangeInfoRequest()
    return
end

---=============================================================---
function This:onBuyPowerRequest()
    if self:canBuyPower() and not self:getPowerIsFull() then
        local diamond = self:getBuyPowerDiamondCost()
        if not GamePlayer:isEnoughDiamond(diamond) then
            g_scene:handleErrorCode("diamondLack")
            return
        end
        Socket:sendAndLock(10204)
    end
end

function This:onBuyPowerReply(message)
    if message.err_code ~= 0 then
        g_scene:handleErrorCode(message.err_code)
        return
    end
    GamePlayer:setPowBuyTime(message.buy_times)
    self:notifyObservers("onBuyPowerReply")
end

function This:onBuyShopItemRequest(id)
    Socket:sendAndLock(10205, {id = id})
end

function This:onBuyShopItemReply( msg )
    if msg.err_code == 0 then
        local config = ShopConfig[msg.id]
        EventDispatcher:dispatcher("after_recharge", config)
        local itemLists = nil
        if msg.buy_diamondNum > 0 then
            if config and config.heap > 0 then 
                self:diamondBuyTimeAddOne(msg.id)
                if config.price <= msg.total_recharge_num then
                    self.m_totalRecharge = msg.total_recharge_num
                else
                    self.m_totalRecharge = (self.m_totalRecharge or 0) + config.price
                end
                self.m_lastRecharge = config.price
                itemLists = {{ id = kDiamondItemId, count = msg.buy_diamondNum} }
            end
        end
        self:notifyObservers("onBuyShopItem", itemLists,msg.id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetGlodChangeInfoRequest()
    Socket:sendAndLock(11313)
end

function This:onGetGlodChangeInfoReply( msg )
    if msg.err_code == 0 then
        self.m_goldChangeTimes = msg.cur_times
        self.m_goldChangeRefreshTime = os.date("*t", GamePlayer:getServerTime()).yday
        self:notifyObservers("onGetGlodChangeInfo", msg.cur_times)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onUseGlodChangeRequest(times)
    Socket:sendAndLock(11314, {times = times or 1})
end

function This:onUseGlodChangeReply( msg )
    if msg.err_code == 0 then
        self.m_goldChangeTimes = self.m_goldChangeTimes + msg.times
        self:notifyObservers("onUseGlodChange", msg)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end