---
---
--- Created by aid.
--- DateTime: 2018/12/18 16:47
---
local M = {}
package.loaded[...] = M

function M:new()
    self.__index = self
    local instance = {}
    setmetatable(instance, self)
    return instance
end

local NodeType = {
    Tab = 1,
    Content = 2,
}

local function highlightTab(tabNode)
    tabNode.light:setVisible(true)
    tabNode.normal:setVisible(false)
end

local function normalizeTab(tabNode)
    tabNode.light:setVisible(false)
    tabNode.normal:setVisible(true)
end

-- contentNode: 可选参数，如果传递了这个参数，那么contentNode的可见性由tabNode的是否选中决定；
-- 同时，如果contentNode没有父node，tabNode将会作为它的父node
function M:addTabNode(key, tabNode, contentNode)
    if not self.m_tabNodes then
        self.m_tabNodes = {}
    end

    tabNode:setTouch({
        onEnded = function ()
            self:selectTab(key)
        end
    })
    normalizeTab(tabNode)

    if contentNode then
        contentNode:setVisible(false)
        if not contentNode:getParent() then
            tabNode:addChild(contentNode)
        end
    end

    self.m_tabNodes[key] = {
        [NodeType.Tab] = tabNode,
        [NodeType.Content] = contentNode,
    }
end

-- guardian: 函数类型，原型为func(sourceTabKey, targetTabKey)->bool
function M:setTabChangeGuardian(guardian)
    self.m_guardian = guardian
end

-- listener: 函数类型，原型为func(currentTabKey)->void
function M:setOnCurrentTabChangedListener(listener)
    self.m_onCurrentTabChangedListener = listener
end

function M:selectTab(targetTabKey)
    local sourceTabKey = self:getCurrentTabKey()
    if sourceTabKey == targetTabKey then
        return
    end

    if self.m_guardian and (not self.m_guardian(sourceTabKey, targetTabKey)) then
        return -- 不允许切换，所以直接返回
    end

    local nodes
    local contentNode

    if sourceTabKey then
        nodes = self.m_tabNodes[sourceTabKey]
        normalizeTab(nodes[NodeType.Tab])
        contentNode = nodes[NodeType.Content]
        if contentNode then
            contentNode:setVisible(false)
        end
    end

    nodes = self.m_tabNodes[targetTabKey]
    highlightTab(nodes[NodeType.Tab])
    contentNode = nodes[NodeType.Content]
    if contentNode then
        contentNode:setVisible(true)
    end
    self.m_currentTabKey = targetTabKey

    if self.m_onCurrentTabChangedListener then
        self.m_onCurrentTabChangedListener(self.m_currentTabKey)
    end
end

function M:getCurrentTabKey()
    return self.m_currentTabKey
end