local This = class(...)
local packageLoaded = package.loaded
local pairs = pairs

function This:init()
    if self.m_init then
        return
    end
    self.m_init = true
    local templates = self.m_templates
    for id,template in pairs(templates) do
        self:initInherit(id, template)
    end
    return self
end

function This:initTemplate(template)
    
end

function This:getClass(typeName)
    if self.type then
        return require(self.type .. "." .. typeName)
    else
        return require(typeName)
    end
end

function This:initInherit(id, template)
    template.id = id
    self:initTemplate(template)
    if template.type then
        template.class = self:getClass(template.type)
    end
    if id == 0 then
        return
    end
    local baseID = template.baseID or 0
    local baseTemplate = self.m_templates[baseID]
    if baseTemplate then
        if baseID == id then
            logError(Serialize[60077] .. id .. ", " .. self._MNAME .. "!!!")
        end
        if not baseTemplate.id then
            self:initInherit(baseID, baseTemplate)
        end
        template.super = baseTemplate
        for k,v in pairs(baseTemplate) do
            if template[k] == nil then
                template[k] = v
            end
        end
    end
end

function This:throwError(id)
    logError("no templateID:" .. tostring(id) .. " in " .. self._MNAME .. " Template!!!")
end

function This:get(id)
    if not self.m_init then
        self:init()
    end
    local template = self.m_templates[id]
    if template == nil then
        self:throwError(id)
    end
    return template
end

function This:getOrCreateTemplate(id)
    local template = self.m_templates[id]
    if template == nil then
        template = {}
        self.m_templates[id] = template
        self:initInherit(id, template)
        return template
    else
        return self:getTemplate(id)
    end
end