-- @Author: superm
-- @Date:   2018-06-29 11:13:35
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 17:52:48
-- @File Name: TerrAltarItemView.lua
-- @Description: 

local This , super = class(..., "NodeView")
This.fileName = "MenuScene/NodeUI/TaiXueYuan10.csb"

local ItemNode = require("UI.ItemNode")
local GameUtil = require("Util.GameUtil")
local MatchingConfig = require("Metadatas.MatchingConfig")

function This:initData(info)
    if info then
        self.m_altarItem = info.item
        self.m_build = info.build
        self.m_isContainer = info.isContainer
        self.m_inited = false
    end
    
    local w , h = self.bg:getContentSize()
    self:setContentSize(w,h)
end

function This:onEnter()
    super.onEnter(self)
    self.m_altarItem:addObserver(self)
end

function This:onExit()
    self.m_altarItem:removeObserver(self)
    super.onExit(self)
end

function This:initNode()
    self:refreshItemUI()
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.buy.button:setTouch({
		    onEnded = function()
                self:onBuyBtnClick()
            end,
            isScale = false,
            isContainer = self.m_isContainer
        })  
        self.m_inited = true
    end
end

function This:refreshItemUI()
    local node = self.item.itemNode
    local itemId, num = 102, 0
    if not self:isLocked() then
        itemId, num = self.m_altarItem:getConfigId(), self.m_altarItem:getItemNum()
        self.itemName:setString(self.m_altarItem:getName())
        self.itemName:setColorByID("Q".. self.m_altarItem:getQuality())
    end
    if not node then
        node = ItemNode:create(itemId, num)
        node:setAnchorPoint(0.5, 0.5)
        self.item:addChild(node)
        self.item.itemNode = node
    else
        node:init(itemId, num)
    end
    self:initStateUI()
end

function This:initStateUI()
	local needRank = self.m_altarItem:getCondition()
    self.kong_bg.tips:setString(MatchingConfig[needRank].rank_level .. Serialize[60201])
    self:refreshStateUI()
end

function This:refreshStateUI()
	local leftTime, ret = 0, false

	if not self:isLocked() then
	    maxTime = self.m_altarItem:getMaxBuyTime()
        if maxTime > 0 then
    	    leftTime = maxTime - self.m_altarItem:getBuyTime()
        end

        if maxTime <= 0 or leftTime > 0 then
            ret = true
            self.buy.icon:setImage(self.m_altarItem:getPriceTypeImage())
            self.buy.number:setVisible(maxTime > 0)
            self.buy.number:setString("(" .. leftTime .. "/" .. maxTime .. ")")
            self.buy.price:setString(self.m_altarItem:getPrice())
        end
	end
    self:setItemEnable(ret)
end

function This:setItemEnable(enble)
	local isLocked = self:isLocked()
    self.buy:setVisible(not isLocked)
    self.kong_bg:setVisible(isLocked)
    self.item:setVisible(not isLocked)
    self.itemName:setVisible(not isLocked)

    if not isLocked then
        local maxTime = self.m_altarItem:getMaxBuyTime()
        self.buy.tips:setVisible(not enble)
        self.buy.button:setVisible(enble)
        self.buy.icon:setVisible(enble)
        self.buy.number:setVisible(enble and maxTime > 0)
        self.buy.price:setVisible(enble)
        self.buy.bg3:setVisible(enble)
    end

end

function This:isLocked()
    return not self.m_altarItem or self.m_altarItem:isLocked()
end

function This:refresh(forceUpdate)
    if forceUpdate then
        self:refreshItemUI()
    end
    self:setItemEnable(not self.m_altarItem:isLocked())
    self:refreshStateUI()
end

function This:onBuyBtnClick()
    if self.m_altarItem then
        self.m_altarItem:requestBuy(self.m_build)
    end
end

function This:showRewardList()
	local itemList = self.m_altarItem:getRewardList(1)
	if itemList and #itemList > 0 then
		g_scene:showReward(itemList)
	end
end

function This:isHighLevelNotBuy()
    if self:isLocked() then
        return false
    end

    local maxTime = self.m_altarItem:getMaxBuyTime()
    return self.m_altarItem:getQuality() > 3 and maxTime <= 0 or maxTime > self.m_altarItem:getBuyTime()
end

--endregion