-- @Author: superm
-- @Date:   2017-08-22 10:57:32
-- @Last Modified by:   superm
-- @Last Modified time: 2018-07-24 10:30:33

--region TerrAltarPanel.lua

local This, super = class(...,"Dialog")
-- local TerritoryManager = require("Territory.TerritoryManager")
local VipManager = require("Vip.VipManager")
local PrivilegeManager = require("Vip.PrivilegeManager")
local ItemUtil = require("Bag.ItemUtil")
local TerrAltarItemView = require("Territory.TerrAltarItemView")

function This:initNode(build)
    self.m_build = build
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaiXueYuan11.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    
    local itemId, num = self.m_build:getProduction()
    self.m_csNode.free.icon:setImage(ItemUtil.getIconImage(itemId))
    self.m_csNode.free.number:setString("+" .. num .. kTerrBuildProductIntervalDesc)

    self.m_hasHighLevItem = false

    self:onVipActivated()

    self.m_viewList = {}
    local count = self.m_build:getAltarItemCount()
    for i=1, count do
        local itemNode = self.m_csNode["item" .. i]
        local item = self.m_build:getTerrAltarItemBy(i)
        if itemNode then
            local view = TerrAltarItemView:create({item = item, build = self.m_build})
            itemNode:addChild(view)
            self.m_viewList[i]=view
        end
    end
end

function This:onEnter()
    super.onEnter(self)
    VipManager:addObserver(self)
    self.m_build:addObserver(self)
    self:refresh()
end

function This:itemUpdate(itemList)
    if itemList and #itemList > 0 then
        g_scene:showReward(itemList)
    end
end

function This:onExit()
    self.m_build:removeObserver(self)
    VipManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.vip.buy:setTouchEnded(function()
        g_scene:openDialog("Vip.VipPanel")
    end)

    self.m_csNode.vip.reset:setTouch({
        onEnded = function()
            local curTimes = self.m_build:getDataTimes()
            if PrivilegeManager:isPrivilegeNumEnough(Privilege.kAltarRefreshTimes, curTimes, true)  then
                if self.m_hasHighLevItem then
                    local panel = g_scene:openDialog("UI.ConfirmPanel", {fileName = "MenuScene/NodeUI/QualifyingMysteryShopTip.csb"})
                    panel:setString(Serialize[64450], 500)
                    panel:setOkHandler(function()
                        self.m_build:onRefreshAltarRequest()
                    end)                
                else
                    self.m_build:onRefreshAltarRequest()
                end
            end
        end,
        isAlpha = false,
    })
end

function This:refresh(forceUpdate)
    self.m_hasHighLevItem = false
    local items = self.m_build:getAllTerrAltarItem()
    if items then
        for index, item in ipairs(items) do
            item:updateState()
            if not self.m_hasHighLevItem and item:isOpening() then
                self.m_hasHighLevItem = not item:isMaxTime() and item:getQuality() > 3
            end
            self:refreshItem(index, item, forceUpdate)
        end
    end
end

function This:refreshResetButton( times )
    -- if VipManager:getMaxMemberLevel() <= VipType.kNone then
    --     self.m_csNode.vip:setVisible(false)
    --     self.m_csNode.free:setVisible(true)
    --     return
    -- end

    local totalTimes = PrivilegeManager:getPrivilegeNumBy(Privilege.kAltarRefreshTimes)
    local ltime = totalTimes - times
    if VipManager:isFullMember() then
        if ltime > 0 then
            self.m_csNode.vip:setVisible(true)
            self.m_csNode.vip.buy:setVisible(false)
            self.m_csNode.vip.tip:setVisible(false)
            self.m_csNode.free:setVisible(false)
            self.m_csNode.vip.reset:clearShader()
            self.m_csNode.vip.reset.text:setColor(255, 255, 255)
            self.m_csNode.vip.reset.text:setString(Serialize[64451] .. ltime .. "）")
        else
            self.m_csNode.vip:setVisible(false)
            self.m_csNode.free:setVisible(true)
        end
    else
        self.m_csNode.vip:setVisible(true)
        self.m_csNode.vip.buy:setVisible(true)
        self.m_csNode.vip.tip:setVisible(true)
        self.m_csNode.free:setVisible(false)
        if ltime > 0 then
            self.m_csNode.vip.reset:clearShader()
            self.m_csNode.vip.reset.text:setColor(255, 255, 255)
            self.m_csNode.vip.reset.text:setString(Serialize[64451] .. ltime .. "）")
        else
            self.m_csNode.vip.reset:setGrayShader()
            self.m_csNode.vip.reset.text:setColor(0, 0, 0)
            self.m_csNode.vip.reset.text:setString(Serialize[62006])
        end
    end
end

function This:refreshItem(i, item, forceUpdate)
    local view = self.m_viewList[i]
    if view then
        view:refresh(forceUpdate)
    end
end

function This:onGetAltarInfo()
    self:refresh(true)
end

function This:onBuyAltarItem()
    self:refresh()
end

function This:onRefreshAltar(times)
    self:refreshResetButton(times)
end

function This:onVipActivated( maxLevel )
    self:refreshResetButton(self.m_build:getDataTimes())    
end


--endregion
