-- @Author: superm
-- @Date:   2017-11-25 13:21:07
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-28 15:07:57
-- @File Name: TerrBattleHeroPanel.lua
-- @Description: 

local This , super = class(..., "Stage.EmbattleHeroPanel")
local HeroManager = require("Hero.HeroManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")

function This:initNode(info)
	self.m_battleHandler = info.battleHandler
	self.m_embattleHandler = info.emBattleHandler
	self.m_attacker = info.attacker
	self.m_target = info.targetType

	if info.heroList then
		self.m_allHeroList = info.heroList
		self.m_allHeroIds = {}
		for _, hero in ipairs(self.m_allHeroList) do
			self.m_allHeroIds[hero:getId()] = true
		end
	end

	self.m_heroSource = info.heroSource
	self.m_heros = {}
    super.initNode(self, info)
end

function This:getHeroSource()
	return self.m_heroSource or {}
end

function This:getAllHeroList()
	if not self.m_allHeroList then
		return super.getAllHeroList(self)
    end
    return self.m_allHeroList
end

function This:initBaseInfo()
	if self.m_target == TerrBattleTarget.kRobber then
		self:initRobberInfo()
	elseif self.m_target == TerrBattleTarget.kWarfareNpc then
		self:initWarfareNpcInfo()
	elseif self.m_target == TerrBattleTarget.kTower then
		self:initTowerNpcInfo()
	elseif self.m_target == TerrBattleTarget.kLegionAboss then
		self:initLegionAbossInfo()
	end
end

function This:initRobberInfo()
	self.m_csNode.helpbutton:setVisible(false)
	self.m_csNode.bg3.tagettext:setString(Serialize[64461])
	self.m_csNode.bg3.taget:setString("" .. self.m_attacker:getHardDesc())
	self.m_csNode.bg3.consume:setString(Serialize[64462])
	self.m_csNode.bg3.cost:setString("1")
	self.m_csNode.bg3.name:setString(GamePlayer:getRoleName())
	self.m_csNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"], TerrRobberFightWaitTime))
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
end

function This:initWarfareNpcInfo()
	self.m_csNode.helpbutton:setVisible(false)
	self.m_csNode.bg3.tagettext:setString(Serialize[64463])
	self.m_csNode.bg3.taget:setString(self.m_attacker:getName())
	self.m_csNode.bg3.name:setString(GamePlayer:getRoleName())
	
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
	self.m_csNode.bg3.consume:setVisible(false)
	self.m_csNode.bg3.cost:setVisible(false)

	self.m_csNode.text100:setVisible(false)
	self.m_csNode.time:setVisible(false)
	self.m_csNode.diban2:setVisible(false)

	self.m_csNode.Battlebutton.text:setString(Serialize[64464])
end

function This:initTowerNpcInfo()
	self.m_csNode.helpbutton:setVisible(false)
	self.m_csNode.bg3.tagettext:setString(Serialize[64171])
	self.m_csNode.bg3.taget:setString(self.m_attacker)
	self.m_csNode.bg3.name:setString(GamePlayer:getRoleName())
	
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
	self.m_csNode.bg3.consume:setVisible(false)
	self.m_csNode.bg3.cost:setVisible(false)

	self.m_csNode.text100:setVisible(false)
	self.m_csNode.time:setVisible(false)
	self.m_csNode.diban2:setVisible(false)

	self.m_csNode.Battlebutton.text:setString(Serialize[64464])
end

function This:initLegionAbossInfo()
	self.m_csNode.helpbutton:setVisible(false)
	self.m_csNode.bg3.tagettext:setString(Serialize[64463])
	self.m_csNode.bg3.taget:setString(Serialize[64465])
	self.m_csNode.bg3.name:setString(GamePlayer:getRoleName())
	
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
	self.m_csNode.bg3.consume:setVisible(false)
	self.m_csNode.bg3.cost:setVisible(false)

	self.m_csNode.text100:setVisible(false)
	self.m_csNode.time:setVisible(false)
	self.m_csNode.diban2:setVisible(false)
end

function This:openSelectPanel()
	local haveSelected = {}
	for i , info in pairs(self.m_heros) do
		haveSelected[info.heroId] = true
	end
	local panel = self:openDialog("Warfare.WarfareSelectHeroPanel",{haveSelected = haveSelected, pick = self.m_allHeroIds})
	return panel
end

function This:refreshBaseInfo()
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
end

local isOnceKey = false
function This:embattle(onceKey)
	isOnceKey = onceKey
	local tmp = {}
	for i , info in pairs(self.m_heros) do
		table.insert(tmp,{pos = i,heroId = info.heroId})
	end
	self:gotoBattle(tmp,isOnceKey)
end

function This:gotoBattle(heroList, onceKey)
	local count = #heroList
	if count <= 0 then
		return
	end 

	if not onceKey then
		if count < 6 then
			local panel = g_scene:openDialog("UI.ConfirmPanel",{text = Serialize[64179]})
			panel:setOkHandler(function()
				if self.m_battleHandler then
					local result = self.m_battleHandler(heroList)
					if result then
						self:close()
					end
				end
			end)				
		elseif self.m_battleHandler then
			local result = self.m_battleHandler(heroList)
			if result then
				self:close()
			end
		end
	elseif onceKey and self.m_embattleHandler then
		self.m_embattleHandler(heroList)
		return
    end
end

--endregion
