-- @Author: superm
-- @Date:   2018-08-21 15:00:28
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-29 17:32:23
-- @File Name: TerrBattleLoadingScene.lua
-- @Description: 

local This , super = class(...,"Stage.StageLoadingScene")
local LoadingScene = require("LoadingScene")
local GameEvent = require("Event.GameEvent")
local BattleDataManager = require("Battle.BattleDataManager")
local StageConfig = require("Metadatas.StageConfig")
local WarMarchArmy = require("Warfare.WarMarchArmy")

function This:init(info)
	LoadingScene.init(self)
	local stageId = info.stageId
	self.m_bgId = info.bgId or StageConfig[stageId].bgId
	self.m_playMode = info.mode or PlayMode.kStage
	self.m_anonymous = info.anonymous

	self.m_msg = info.msg
	local report = info.msg.report
	local reportList = info.msg.report_list
	if not report and reportList then
		local playerList = info.msg.fight_player
		self:initByWarfareMode( playerList, reportList )
	else
		self.m_battleReport = report
	end

	self.m_attackerHeroAttrList = info.msg.attack_list or {}
	self.m_defenderHeroAttrList = info.msg.defence_list or {}
	self:initTask()
end

function This:initByWarfareMode( playerList, reportList )
	local _, curReport = next(reportList)
	self.m_battleReport = curReport

	local players = {}
	local allAttackQueue = {}
	local count = #reportList
	local roleCount = count
	for index, reportInfo in ipairs(reportList) do
		local playerInfo = reportInfo.attack
		if playerInfo then
			local army = WarMarchArmy:create(playerInfo, true, index, self.m_anonymous)
			table.insert(allAttackQueue, army)
			players[army:getId()] = army
		end
	end

	local function addBattle(index, reportInfo)
		local attackQueue = {}
		for i = index, roleCount do
			table.insert(attackQueue, allAttackQueue[i])
		end

		if index > 1 then
			local defenceQueue = {}
			table.insert(defenceQueue, WarMarchArmy:create(reportInfo.defence))
			BattleDataManager:addBattleForWarfare(reportInfo.id, defenceQueue, attackQueue, reportInfo)		
		else
			BattleDataManager:setAttackQueue(allAttackQueue)
		end
	end

	if playerList then 
		roleCount = #playerList
		for _, playerInfo in ipairs(playerList) do
			if not players[playerInfo.id] then
				local army = WarMarchArmy:create(playerInfo, true)
				table.insert(allAttackQueue, army)
				players[army:getId()] = army
			end
		end
	end

	for index, reportInfo in ipairs(reportList) do
		addBattle(index, reportInfo)
	end
end

function This:addBattlePrepareTask()
	local task10 = GameEvent:create(function()
		BattleDataManager:prepareBattle(true, self.m_playMode, self.m_bgId)
	end)
	self:addTask(task10)
end
