-- @Author: superm
-- @Date:   2018-06-25 10:26:26
-- @Last Modified by:   superm
-- @Last Modified time: 2018-07-02 16:21:12
-- @File Name: TerrBlackSmithHelpDetailPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local TerrBuildManager = require("Territory.TerrBuildManager")

local HelpInfoItemView, NodeView = class(This._NAME .. ".HelpInfoItemView", "NodeView")
HelpInfoItemView.fileName = "MenuScene/NodeUI/BlackSmithDetailList.csb"

local HelpConfig = require("Metadatas.HelpConfig")
local RankPageListLayer = require("Rank.RankPageListLayer")

function This:initNode(info)
	if not self.m_build then
		self.m_build = info.build
		self.m_build:onGetHelpInfoRequest()
	end
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/BlackSmithDetail.csb")
	self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    self.m_count = 0
    self.m_csNode.number:setString(string.format("%d/%d", self.m_count, TerrPolyMaxHelpTime))

	self.m_layerLayer = RankPageListLayer:create({
		itemWidth = 480,
		itemHeight = 35,
        spaceX = 0,
        spaceY = 1,
        scrollView = self.m_csNode.scrollView,
		dataCounter = function ()
	        return self.m_count
        end,
        viewCreator =  function(index, node)
        	if self.m_count < index then
        		return
        	end

        	local data = self.m_itemList[index]
	        if data then
				-- print(index, data.role_name)
		        if not node then
			        return HelpInfoItemView:create(data, index)
			    else
		        	node:setModel(data, index)
		        end
		    end
	    end,
    })
	self.m_layerLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_layerLayer:getContentSize()
	self.m_csNode.scrollView:pushBackCustomItem(self.m_layerLayer, w, h,0)
end

function This:onEnter()
	if self.m_build then
		self.m_build:addObserver(self)
	end
    super.onEnter(self)
end

function This:onExit()
	if self.m_build then
		self.m_build:removeObserver(self)
	end
    super.onExit(self)
end

function This:refresh()
    self.m_csNode.number:setString(string.format("%d/%d", self.m_count, TerrPolyMaxHelpTime))
	self.m_layerLayer:setDataList()
end

function This:initTouchEvent()    
    self.m_csNode.closebutton:setTouchEnded(function()
        self:close()
    end)
end

function This:onGetBuildHelpInfo()
 	self.m_itemList = self.m_build:getReceiveHelpList()
    self.m_count = #self.m_itemList
    if self.m_count > TerrPolyMaxHelpTime then
    	self.m_count = TerrPolyMaxHelpTime
    end
    self:refresh()
end

----------------------------------------------------------------------------
function HelpInfoItemView:initData(data, index)
	if data and index then
	    self:setData(data, index)
	    self.m_dirty = true
	end
end

function HelpInfoItemView:setData( data, index )
	if self.m_index ~= index or self.m_id ~= data.timestamp then
	    self.m_index = index
	    self.m_id = data.timestamp
	    self.m_roleName = data.role_name
	    self.m_number = HelpConfig[data.type].lucky[2]
	    self.m_dirty = true
	end
end

function HelpInfoItemView:initNode()
    self:refresh()
end

function HelpInfoItemView:refresh()
	if self.m_dirty then
		self.name:setString(self.m_roleName)
		self.number:setString(" + " .. self.m_number)
		self.m_dirty = false
		self.bg:setVisible(self.m_index % 2 > 0)
	end
end

function HelpInfoItemView:setModel( data, index )
    self:setData(data, index)
    self:refresh()
end

