-- @Author: superm
-- @Date:   2018-06-26 14:27:12
-- @Last Modified by:   superm
-- @Last Modified time: 2018-07-18 13:52:04
-- @File Name: TerrBlackSmithHelpPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local TerrBuildManager = require("Territory.TerrBuildManager")

local HelpInfoItemView, NodeView = class(This._NAME .. ".HelpInfoItemView", "NodeView")
HelpInfoItemView.fileName = "MenuScene/NodeUI/BlackSmithFriend.csb"

local RankPageListLayer = require("Rank.RankPageListLayer")
local GameUtil = require("Util.GameUtil")

function This:initNode(info)
	if not self.m_build then
		self.m_build = info.build
        self:addOpenCallBack(function ()
            if self.m_build then    
                self.m_build:onGetHelpInfoRequest()
            end
        end)
        
        self:addCloseCallback(function ()
            if self.m_build then    
                self.m_build:onGetHelpInfoRequest()
            end
        end)
	end
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/BlackSmithHelpList.csb")
	self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()
    self.m_count = 0

	self.m_layerLayer = RankPageListLayer:create({
		itemWidth = 530,
		itemHeight = 98,
        spaceX = 0,
        spaceY = 5,
        scrollView = self.m_csNode.scrollView,
		dataCounter = function ()
	        return self.m_count
        end,
        viewCreator =  function(index, node)
        	if self.m_count < index then
        		return
        	end

        	local data = self.m_itemList[index]
	        if data then
		        if not node then
			        return HelpInfoItemView:create(data, index, function (id, type, role)
                        self.m_build:onHelpForAccerlationRequest(id, type, role)
                    end)
			    else
		        	node:setModel(data, index)
		        end
		    end
	    end,
    })
	self.m_layerLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_layerLayer:getContentSize()
	self.m_csNode.scrollView:pushBackCustomItem(self.m_layerLayer, w, h,0)
end

function This:onEnter()
	if self.m_build then
		self.m_build:addObserver(self)
	end
    super.onEnter(self)
end

function This:onExit()
	if self.m_build then
		self.m_build:removeObserver(self)
	end
    super.onExit(self)
end

function This:refresh()
	local count = self.m_count > 0 and self.m_count or 0
    if count > TerrPolyMaxHelpTime then
    	count = TerrPolyMaxHelpTime
    end
 	table.sort(self.m_itemList, function (a, b)
 		if a.invalid ~= b.invalid then
 			return not a.invalid
 		end
 		return a.timestamp > b.timestamp
 	end)
	self.m_layerLayer:setDataList()
end

function This:initTouchEvent()    
    self.m_csNode.closebutton:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.helpbutton:setTouchEnded(function()
        if self.m_count <= 0 then
        	g_scene:showTips(Serialize[64470])
        	return
        end

        self.m_build:onPolyHelpAllRequest()
    end)
end

function This:onGetBuildHelpInfo()
 	self.m_itemList = self.m_build:getNeedHelpList()
    self.m_count = #self.m_itemList
    self:refresh()
end

function This:onPolyHelpOne()
    g_scene:showTips(Serialize[64471])
    self.m_count = self.m_count - 1
    self:refresh()
end

function This:onPolyHelpAll()
    self.m_helpAll = true
    g_scene:showTips(Serialize[64471])
    self:close()
end

----------------------------------------------------------------------------
function HelpInfoItemView:initData(data, index, handler)
    if data and index then
        -- self.m_index = index
        self.m_clickHandler = handler
        self:setData(data, index)
        self.m_dirty = true
    end
end

function HelpInfoItemView:setData(data, index)
	if self.m_index ~= index or self.m_id ~= data.timestamp then
        self.m_index = index
	    self.m_id = data.timestamp
	    self.m_role = data.role_id
	    self.m_type = data.type
	    self.m_icon = data.icon --GameUtil:getPlayerIcon(data.icon)
	    self.m_data = data
		if self.m_type == 1 then
		    self.m_roleName = data.role_name .. Serialize[64472]
		else
		    self.m_roleName = data.role_name .. Serialize[64473]
        end
        self.m_dirty = true
	end

end

function HelpInfoItemView:initNode()
    self:refresh()
end

function HelpInfoItemView:refresh()
    if self.m_dirty then
        self.name:setString(self.m_roleName)
        GameUtil:setPlayerIconNodeImage(self.icon, self.m_icon)
	    self.button:setVisible(not self.m_data.invalid)
        self.m_dirty = false
    end
end

function HelpInfoItemView:setModel( data, index )
    self:setData(data, index)
    self:refresh()
end

function HelpInfoItemView:initTouchEvent()
    if not self.m_inited then
        self.button:setTouch({
            getRect = function (touch)
                local rect = touch.sizeNode:getBoundingBox()
                return rect
            end,
		    onEnded = function()
                self:onHelpBtnClick()
            end,
            isContainer = true,
            isDynamic = true,
            isScale = false,
        })  
        self.m_inited = true
    end
end

function HelpInfoItemView:onHelpBtnClick()
    if not self.m_data.invalid then
    	self.m_data.invalid = true
    	self.button:setVisible(false)
        if self.m_clickHandler then
    	   self.m_clickHandler(self.m_id, self.m_type, self.m_role)
        end
    end
end


---------------------------------------------------------