-- @Author: superm
-- @Date:   2018-06-25 13:37:34
-- @Last Modified by:   superm
-- @Last Modified time: 2018-06-29 16:30:53
-- @File Name: TerrBlackSmithMWenPolyInfo.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local ItemUtil = require("Bag.ItemUtil")
local WorkerPriceConfig = require("Metadatas.WorkerPriceConfig");
local ArgConfig = require("Metadatas.ArgConfig");
local PloyQueueState = 
{
	kFree = 0,  
	kRunning = 1, --正开启
	kLock = 2, 
}

function This:create(...)
	local o = {}
	setmetatable(o,self)
	o:init(...)
	return o
end

function This:init(info)
	super.init(self)
	if info then
		self.m_id = info.timestamp or 0
		self.m_endTime = info.end_time
		self.m_ployItemList = info.mix_item_id_list
		self.m_accerlated = info.is_accelerate ~= 0
		if info.reward_id > 0 then
			self.m_result = info.reward_id
			self.m_resultNum = info.reward_count or 1
		end
        self.m_particular_reward = info.particular_reward
		self.m_escape = 0
		self.m_isUpdating = false
		local curTime = GamePlayer:getServerTime()
		if curTime < self.m_endTime then
			self.m_state = PloyQueueState.kRunning
			self:startUpdate()
		else 
			self.m_state = PloyQueueState.kFree
			if self.m_result then
				self.m_state = PloyQueueState.kLock
			end
		end

        self.m_fortune = info.fortune_value or 0
	else
		self:clear()
	end
end

function This:clear()
	if self.m_isUpdating then
		self:endUpdate()
	end
	local id = self.m_id
	local result = self.m_result
	self.m_id = 0
	self.m_endTime = 0
	self.m_accerlated = false
	self.m_escape = 0
	self.m_ployItemList = nil
    self.m_particular_reward = nil
	self.m_result = nil
	self.m_resultNum = nil
	if self.m_object and result then
		self.m_object:notifyObservers("onQueueClearInfo", id)
	end
	self.m_state = PloyQueueState.kFree
    self.m_fortune = 0
end

function This:setObject( obj )
    self.m_object = obj
end


-- table: 0x1e3dceb8 {
--   #[1] => table: 0x1e3dceb8 {
--            [err_code] => 0
--            [cmd] => 10453
--            [mix_data] => table: 0x1e3dca60 {
--                            [timestamp] => 1562919437
--                            [end_time] => 1562962637
--                            [scroll_typeid] => 0
--                            [is_accelerate] => 0
--                            [mix_item_id_list] => table: 0x1e3dcac0 {
--                                                    #[1] => 120011
--                                                    #[2] => 120011
--                                                    #[3] => 120011
--                                                  }
--                          }
--          }
-- }

function This:updateInfo( info )

	if info.end_time == 0 then
		self.m_state = PloyQueueState.kFree
		self.m_endTime = 0
		if self.m_isUpdating then
			self:endUpdate()
		end
		return
	end
	if self.m_id ~= 0 and info.timestamp ~= self.m_id then
		self.m_id = info.timestamp
	end
	print("info=-========")
	print_tmp(info)

	self.m_id = info.timestamp
	self.m_state = info.state
	self.m_endTime = info.end_time
	self.m_ployItemList = info.mix_item_id_list
	self.m_polyItemForPage = info.scroll_typeid
	self.m_accerlated = info.is_accelerate ~= 0
	-- if info.reward_id and info.reward_id > 0 then
	-- 	self.m_result = info.reward_id
	-- 	self.m_resultNum = info.reward_count or 1
	-- end
    -- self.m_particular_reward = info.particular_reward
	local curTime = GamePlayer:getServerTime()
	if curTime < self.m_endTime then
		self.m_state = PloyQueueState.kRunning
	elseif curTime >= self.m_endTime then
		self.m_state = PloyQueueState.kFree
		if #self.m_ployItemList > 0 then
			self.m_state = PloyQueueState.kLock
		end
	end
	
	if self.m_isUpdating and self.m_state == PloyQueueState.kFree then
		self:endUpdate()
	elseif not self.m_isUpdating and self.m_state == PloyQueueState.kRunning then
		self:startUpdate()
	end

    self.m_fortune = info.fortune_value
end

function This:startUpdate()
	Application:addToUpdateList(self)
	if self.m_object then
		self.m_object:notifyObservers("onQueueUpdateStart", self:getId(),true)
	end
	self:notifyObservers("onQueueUpdateStart",self,true)
	self.m_isUpdating = true
end

function This:endUpdate()
	Application:removeFromUpdateList(self)
	self.m_state = PloyQueueState.kFree
	if self.m_result then
		self.m_state = PloyQueueState.kLock
	end
	if self.m_object then
		self.m_object:notifyObservers("onQueueUpdateEnd", self:getId(),true)
	end
	self:notifyObservers("onQueueUpdateEnd",self,true)
	self.m_isUpdating = false
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return -1
	end
	return self.m_endTime - curTime
end

function This:getCostByLeftTime()
    local curTime = GamePlayer:getServerTime()
    local times = 0;
    if self.m_accerlated then
        times = 1
    end
 
    local timescale = (curTime - self.m_id + times* WorkerPriceConfig[1].derate)/(self.m_endTime - self.m_id + times* WorkerPriceConfig[1].derate)
    -- local startdata = WorkerPriceConfig[1].initialization_price[2]
    local startdata = ArgConfig["inscription_position_price"].arg[1]
    return math.ceil(startdata*(1 - (timescale/(timescale+1))) - startdata/2*timescale)
end

function This:getMixSkillFortuneValue()
     return self.m_fortune 
end
function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
		local leftTime = self:getLeftTime()
		if self.m_object then
			self.m_object:notifyObservers("updateQueueTime", {time = leftTime, id = self:getId()})
		end
		self:notifyObservers("updateQueueTime", {time = leftTime, id = self:getId()})
		if leftTime <= 0 then
			self:endUpdate()
		end
	end
end

function This:getId()
	return self.m_id
end

function This:getState()
	return self.m_state
end

function This:isRunning()
	return self.m_state == PloyQueueState.kRunning
end

function This:hasAccerlated()
    return self.m_accerlated
end

function This:canAccerlate()
    return self:isRunning() and not self:hasAccerlated()
end

function This:getPolyingitemList()
    return self.m_ployItemList
end

function This:getPolyItemForPage()
	return self.m_polyItemForPage
end

function This:setPolyItemForPage(time)
	self.m_polyItemForPage = time
end

function This:getParticular_reward()
    return self.m_particular_reward
end

function This:isFree()
    return self.m_state == PloyQueueState.kFree
end

function This:isLocked()
	return self.m_state == PloyQueueState.kLock
end

function This:getReslut()
    return 333
end

function This:getFuncItemImage()
    if self:isRunning() then
        return "MenuScene/ronghename.png", 0
    end

    local result = self:getReslut()
    if result and result > 0 then
        return ItemUtil.getIconImage(result), self.m_resultNum
    end
end

function This:onFunctionEvent()
	if not self.m_object then
		return
	end

	if self:getLeftTime() > 0 then
    	-- g_scene:openDialog("Territory.TerrBlackSmithPanel", self.m_object) LordEquipMakePanel
    	local dialog = g_scene:openDialog("Awake.LordEquipMakePanel", self.m_object)
    	dialog:selectTabButton(LordEquipMakePanelTabName.kMingWenMake)
    else
    	-- self.m_object:onPickupResultRequest()
    	self.m_object:onPickUpResultRequestForEquip()
    end
end