-- @Author: superm
-- @Date:   2018-06-22 14:22:26
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 17:55:47
-- @File Name: TerrBlackSmithPanel.lua
-- @Description: 

local This, super = class(...,"NodeUI")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrBlackSmithPolyNode = require("Territory.TerrBlackSmithPolyNode")
local TerrBlackSmithWokerNode = require("Territory.TerrBlackSmithWokerNode")
local GameUtil = require("Util.GameUtil")

PolyState = {
	kNotBegin = 1,
	kUpdating =2,
	kFinish =3,

	kPrepare = 11,
}

local sendCdTime = 5 * 60

function This:initNode(build)
    self.m_build = build or TerrBuildManager:getBuildByType(TerrBuildType.kTieJiangPu)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/BlackSmithSkill.csb")
	self:addChild(self.m_csNode)
    -- self.m_csNode:setToCenter()


    self.m_polyNode = TerrBlackSmithPolyNode:create({
    	node = self.m_csNode, 
    	build = self.m_build,
    })

    self.m_workerNode = TerrBlackSmithWokerNode:create({
    	node = self.m_csNode.rongheshi, 
    	build = self.m_build,
    })
    self:refreshFortuneNode()
end

function This:onEnter()
    super.onEnter(self)
    self.m_polyNode:onEnter()
    self.m_workerNode:onEnter()
	if self.m_build then
		self.m_build:addObserver(self)
	end
end

function This:onExit()
	if self.m_build then
		self.m_build:removeObserver(self)
	end
    self.m_workerNode:onExit()
    self.m_polyNode:onExit()
    super.onExit(self)
end

function This:refresh()
    self.m_polyNode:refresh()
    self.m_workerNode:refresh()
    self:refreshFortuneNode()
end

-- 运势 
function This:refreshFortuneNode()
    local cls = self.m_build:getFortuneClass()
    local resource = self.m_build:getFortuneResource()
	local isFull = self.m_build:isFullFortuneClass()
	if isFull and not self.m_fullClassEffect then
		self.m_fullClassEffect = GameUtil:createEffect("YunShiMax", true, self.m_csNode.yunshi.lianlu)
		self.m_csNode.yunshi.lianlu:addChild(self.m_fullClassEffect)
	elseif not isFull and self.m_fullClassEffect then
		self.m_fullClassEffect:removeFromParent()
		self.m_fullClassEffect = nil
	end
	
    self.m_csNode.yunshi.yunshiqidi:setImage("MenuScene/bg_yunshiqidi" .. resource .. ".png")
    self.m_csNode.yunshi.yunshiqidi.yunshizi:setImage("MenuScene/yunshi" .. resource .. ".png")

    local fortune = self.m_build:getFortune()
    self.m_csNode.yunshi.value:setString(Serialize[60368] .. fortune)
end

function This:initTouchEvent()
    --boby  
	if self.m_csNode.help then
        self.m_csNode.help:setTouchEnded(function ()
            g_scene:openDialog("UI.HelpTips", {text = Serialize["black_smith_help"], node = self.m_csNode.help})
        end)
	end
	
	if self.m_csNode.closebg then
		self.m_csNode.closebg:setTouchEnded(function()
			self:close()
		end)
	end

	self.m_csNode.yunshi.xiangqingdi:setTouchEnded(function()
    	g_scene:openDialog("Territory.TerrBlackSmithHelpDetailPanel", {build = self.m_build})
	end)

	self.m_csNode.yunshi.jiantou:setTouchEnded(function()
    	g_scene:openDialog("Territory.TerrFortunePanel", {build = self.m_build})
	end)

	self.m_csNode.rongheshi.weiguyong.jiahao_rongheshi:setTouchEnded(function()
		g_scene:openDialog("Territory.TerrBlackSmithWorkerPanel", {build = self.m_build})
	end)

	---------一键放入 
	self.m_csNode.ronghe.quickput:setTouchEnded(function()
		self.m_polyNode:quicklyPutIn()
	end)

	---------开始融合
	self.m_csNode.ronghe.poly:setTouchEnded(function()
		self.m_polyNode:startPoly()
	end)

    ---------融合师加速
	self.m_csNode.ronghe.accerlate:setTouchEnded(function()
		self.m_polyNode:startAccerlate()
	end)

    ---------立即完成
	self.m_csNode.ronghe.finish:setTouchEnded(function()
		self.m_polyNode:finishPoly()
	end)

	self.m_csNode.ronghe.pickup:setTouchEnded(function()
		self.m_build:onPickupResultRequest()
	end)
end

------------------------------------------------------
function This:onPolymerize()
    self.m_polyNode:refresh()
end

function This:onPloyAccerlate(oldtime)
    self.m_polyNode:playAccAnimation(oldtime, 3)
end

function This:onHirePolymerization()
    self.m_workerNode:refresh()
    self.m_polyNode:refresh()
end

function This:onPloyFortuneChange( oldValue, newValue )
    self:refreshFortuneNode()
end

function This:onPolyPickupResult()
    self.m_polyNode:refresh(true)
end