-- @Author: superm
-- @Date:   2018-06-22 15:43:33
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-18 22:57:03
-- @File Name: TerrBlackSmithPolyNode.lua
-- @Description: 

local This = class(...)
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local PropChoosePanel = require("Depot.PropChoosePanel")
local InosculateSkillConfig = require("Metadatas.InosculateSkillConfig")
local PreviewRewardConfig = require("Metadatas.PreviewRewardConfig")
local LuckyConfig = require("Metadatas.LuckyConfig");
local GameUtil = require("Util.GameUtil")
local ViewUtil = require("UI.ViewUtil")
local GamePlayer = require("Player.GamePlayer")

local kMaxPanelItemCount = 5

function This:create(...)
	local o = {}
	setmetatable(o,self)
	o:init(...)
	return o
end

function This:init(info)
    self.m_parentNode = info.node
    self.m_csNode = info.node.ronghe
    self.m_build = info.build
    self.m_polyInfo = self.m_build:getPolyInfo()

	self.m_configs = {}
	for i, cfg in ipairs(InosculateSkillConfig) do
		self.m_configs[cfg.skill_quality] = cfg
	end
	self:initNode()
	self:initTouchEvent()
end

function This:onEnter()
	self.m_polyInfo:addObserver(self)
end

function This:onExit()
	self.m_polyInfo:removeObserver(self)
end

function This:initTouchEvent()
    if not self.m_inited then
		for i=1, kMaxPanelItemCount do
			self.m_csNode.itemtipslist["need" .. i].add:setTouch({
				onEnded = function()
					if self.m_subView then
						return
					else
		        		self.m_subView = true
					end
					local panel = self:chooseProp(i)
			        panel:addCloseCallback(function ()
			            self.m_subView = nil
			        end)
				end,
				isDynamic = true,
				isScale = false,
			})

			self.m_csNode.itemtipslist["item" .. i].close:setTouch({
				onEnded = function()
					if self.m_state ~= PolyState.kUpdating then 
						self:revokeChosen(i)
					end
				end,
				isDynamic = true,
				isScale = false,
				isAlpha = false,
			})
		end
        self.m_inited = true
    end
end

function This:initNode()
	self:initEffect()
	self:getCostInfo()

	for i = 1, kMaxPanelItemCount do
		self.m_csNode.itemtipslist["item" .. i]:setVisible(false)
	end
	local viewCount = self.m_costPropCount
	local itemWidth = self.m_csNode.itemtipslist.need1:getContentWidth() * self.m_csNode.itemtipslist.need1:getScaleX()
	local w = itemWidth / 2
	for i= 1, viewCount do
		local needNode = self.m_csNode.itemtipslist["need" .. i]
		local node = self.m_csNode.itemtipslist["item" .. i]
		needNode:setVisible(true)
		needNode:setPositionX(w)
		node:setVisible(false)
		node:setPositionX(w)
		w = w + itemWidth + 10
	end

	self.m_csNode.itemtipslist:setPositionX( 200 - (itemWidth + 10) * viewCount / 2 )

	for i= viewCount + 1, kMaxPanelItemCount do
		self.m_csNode.itemtipslist["need" .. i]:setVisible(false)
	end

	self:setStateAndRefresh()
end

function This:initEffect()
	if not self.m_effectJinDu then
		local node = self.m_csNode.jindutiao
		local posx, posy = node.jindutiao:getPosition()
		self.m_sourcePosX = posx
		self.m_sourceWidth = node.jindutiao:getContentWidth()
		self.m_effectJinDu = GameUtil:createEffect("RongHeJiaSu", true)
		self.m_effectJinDu:setVisible(false)
		node:addChild(self.m_effectJinDu)
		self.m_effectJinDu:setPosition(posx, posy)
	end
end

function This:getCostInfo()
	self.m_costPropCount = 3
	self.m_fileterList = {}
	self.m_fileterInfoList = {}
	for i=1, kMaxPanelItemCount do
		self.m_fileterList[i] = 0
		self.m_fileterInfoList[i] = { quality = 0, uid = 0 }
	end
end

function This:refresh(cleanup)
	self.m_quicklyPutIn = nil
	--self:getCostInfo()
	self:setStateAndRefresh(cleanup)
end

function This:quicklyPutIn()
	local indexList = {}
	local slotCount = self.m_costPropCount
	for i, uid in ipairs(self.m_fileterList) do
		if i > slotCount then
			break
		end

		if uid <= 0 then
			table.insert(indexList, i)
		end
	end

	local emptyCount = #indexList
	if emptyCount > 0 then
		local propList, _, count = PropChoosePanel:getItemList({
			filter = self.m_fileterList, 
			checkHandler = function ( item )
				return item:isPassiveSkill()
			end,
			alone = true
		})
		if emptyCount > count then
			if count <= 0 then
				g_scene:showTips(Serialize[64488])
			end
			self.m_quicklyPutIn = false
		end
		for i, index in ipairs(indexList) do
			local newProp = propList[i]
			if newProp then
				self:putInChosen(newProp.item, index)
			else
				return
			end
		end
	else
		g_scene:showTips(Serialize[60430])
	end
	self.m_quicklyPutIn = true

	self:setStateAndRefresh()
end

function This:putInChosen(item, pos)
	local node = self.m_csNode.itemtipslist["item" .. pos]
	if item then
		self.m_fileterList[pos] = item:getId()
		self.m_fileterInfoList[pos].quality = item:getQuality()
		self.m_fileterInfoList[pos].uid = item:getUid()

		self.m_csNode.itemtipslist["need" .. pos]:setVisible(false)
		if not node.itemNode then
			local itemNode = ItemNode:create(item:getId(), nil, false, true)
			itemNode:setAnchorPoint(0.5, 0.5)
			node:addChild(itemNode)
			node.itemNode = itemNode
			node.close:setLocalZOrder(10)
			node.jianhao:setLocalZOrder(11)
		else
			node.itemNode:initItem(item:getId(), nil, false)
		end 
		node:setVisible(true)
	else
		node:setVisible(false)
	end

	if node.effect then
		node.effect:removeFromParent()	
		node.effect = nil	
	end	
	node.close:setVisible(self.m_state ~= PolyState.kUpdating)
	node.jianhao:setVisible(self.m_state ~= PolyState.kUpdating)
end

function This:putInChosenByItemId(itemId, pos)
	local node = self.m_csNode.itemtipslist["item" .. pos]
	if itemId then
		self.m_fileterList[pos] = itemId
		self.m_fileterInfoList[pos].quality = ItemUtil.getQuality(itemId)
        self.m_fileterInfoList[pos].uid = itemId
		self.m_csNode.itemtipslist["need" .. pos]:setVisible(false)
		if not node.itemNode then
			local itemNode = ItemNode:create(itemId, nil, false, true)
			itemNode:setAnchorPoint(0.5, 0.5)
			node:addChild(itemNode)
			node.itemNode = itemNode
			node.close:setLocalZOrder(10)
			node.jianhao:setLocalZOrder(11)
		else
			node.itemNode:initItem(itemId, nil, false)
		end

		if node.effect then
			node.effect:removeFromParent()	
			node.effect = nil	
		end
        local posx, posy = node.itemNode:getPosition()
		node.effect = GameUtil:createEffect("RongHeZhong", true, node.itemNode)
		node:addChild(node.effect)
		node.effect:setPosition(posx, posy)
		node.effect:setLocalZOrder(12)
		node:setVisible(true)
	else
		if node.effect then
			node.effect:removeFromParent()		
			node.effect = nil
		end
		node:setVisible(false)
	end
	node.close:setVisible(false)
	node.jianhao:setVisible(false)
end

function This:revokeChosen(pos)
	if pos <= self.m_costPropCount then
		self.m_quicklyPutIn = false

	    self.m_fileterList[pos] = 0
		self.m_fileterInfoList[pos] = { quality = 0, uid = 0 }
	    self.m_csNode.itemtipslist["item" .. pos]:setVisible(false)
	    self.m_csNode.itemtipslist["need" .. pos]:setVisible(true)
		self:setStateAndRefresh()
	end   	
end


function This:chooseProp(pos)
	local panel = g_scene:openDialog("Depot.PropChoosePanel",{
		filter = self.m_fileterList, 
		checkHandler = function ( item )
			return item:isPassiveSkill()
		end
	})
	panel:setSelectHandler(function (item)
		self:putInChosen(item, pos)
		self:setStateAndRefresh()
		return true
	end)
	return panel
end

function This:getState()
	local state = nil
	if not self.m_polyInfo:isFree() then
		state = self.m_polyInfo:isRunning() and PolyState.kUpdating or PolyState.kFinish
	else
		if self.m_quicklyPutIn then
			state = PolyState.kPrepare
		else
			state = PolyState.kPrepare
			for i, uid in ipairs(self.m_fileterList) do
				if i > self.m_costPropCount then
					break
				end

				if uid <= 0 then
					state = PolyState.kNotBegin
					break
				end
			end
		end
	end
    return state
end

function This:setStateAndRefresh(cleanup)
	self:refreshStateUI(cleanup)
	self:refreshPriceUI(cleanup)
    local fortuneValue = self.m_polyInfo:getMixSkillFortuneValue()
    if fortuneValue == 0 then
        fortuneValue = self.m_build:getFortune()
    end
    self:refreshPreviewUI(fortuneValue)
end

function This:refreshStateUI(cleanup)
	local state = self:getState()
	local stateIsDirty = false
	local oldstate = self.m_state
	if not state or self.m_state ~= state then
		if state then
			for _, v in pairs(PolyState) do
				self.m_csNode["ronghebiaoti" .. v]:setVisible(false)
			end
			self.m_state = state
			self.m_csNode["ronghebiaoti" .. state]:setVisible(true)
		end
		stateDirty = true
	end

	if stateDirty then
		if self.m_state == PolyState.kNotBegin or self.m_state == PolyState.kFinish then
			self.m_csNode.jindutiao:setVisible(false)
			self.m_csNode.ronghetime:setVisible(false)
			self.m_csNode.rhtime:setVisible(false)

			local canPickup = self.m_state == PolyState.kFinish
			self.m_csNode.quickput:setVisible(not canPickup)
			self.m_csNode.poly:setVisible(not canPickup)
			self.m_csNode.itemtipslist:setVisible(not canPickup)
		else
			local visible = self.m_state == PolyState.kPrepare
			self.m_csNode.ronghetime:setVisible(true)
			self.m_csNode.rhtime:setVisible(true)
			self.m_csNode.itemtipslist:setVisible(true)

			self.m_csNode.quickput:setVisible(visible)
			self.m_csNode.jindutiao:setVisible(not visible)
			self.m_csNode.poly:setVisible(visible)
		end

		self:refreshResultUI(cleanup)
		stateDirty = false	
	end
	self:refreshButtonUI()
end

function This:refreshResultUI(cleanup)
	local node = self.m_csNode.result
	if self.m_state == PolyState.kFinish then
        local particularReward = self.m_polyInfo:getParticular_reward()
        local itemList = {}
		local itemNumList = {}
		itemList[1] = self.m_polyInfo:getReslut()
		itemNumList[1] = 1
        if particularReward and particularReward[1].num > 0 then
            itemList[2] = particularReward[1].id
			itemNumList[2]=particularReward[1].num
        end
        local posx1, posy1 = node:getPosition()
        if not node.itemNode then
            node.itemNode = {}
        end
		for i=1,2 do
            if itemList[i] then
		        if not node.itemNode[i] then
			        local itemNode = ItemNode:create(itemList[i], itemNumList[i], false, true)
			        itemNode:setAnchorPoint(0.5, 0.5)
			        node:addChild(itemNode)
					node.itemNode[i] = itemNode
					if particularReward and particularReward[1].num > 0 then
						itemNode:setPosition(-150+100*i, 0)
					else
						itemNode:setPosition(-100+100*i, 0)
					end
		        else
					node.itemNode[i]:initItem(itemList[i], itemNumList[i], false)
					if particularReward and particularReward[1].num > 0 then
						node.itemNode[i]:setPosition(-150+100*i, 0)
					else
						node.itemNode[i]:setPosition(-100+100*i, 0)
					end
		        end 
                node.itemNode[i]:setVisible(true)
            else
                if node.itemNode[i] then
                    node.itemNode[i]:setVisible(false)
                end
            end
--		    if node.effect then
--			    node.effect:removeFromParent()	
--			    node.effect = nil	
--		    end
--            local posx, posy = node.itemNode:getPosition()
--		    node.effect = GameUtil:createEffect("RongHeWanCheng", true, node.itemNode)
--		    node:addChild(node.effect)
--		    node.effect:setPosition(posx, posy)
--		    node.effect:setLocalZOrder(12)
        end
		node:setVisible(true)
		self.m_csNode.pickup:setVisible(true)
	else
		node:setVisible(false)
		self.m_csNode.pickup:setVisible(false)

		if cleanup and self.m_state == PolyState.kNotBegin then
			local viewCount = self.m_costPropCount
			local itemListNode = self.m_csNode.itemtipslist
			for i= 1, viewCount do
				local needNode = itemListNode["need" .. i]
				local itemNode = itemListNode["item" .. i]
				needNode:setVisible(true)
				itemNode:setVisible(false)
			end
		elseif self.m_state == PolyState.kUpdating then
			local polyingItemList = self.m_polyInfo:getPolyingitemList()
			if polyingItemList then
				for i, id in ipairs(polyingItemList) do
					self:putInChosenByItemId(id, i)
				end
			end
		end
	end
end

function This:refreshButtonUI()
	if self.m_state == PolyState.kUpdating then
		local canAccerlate = self.m_build:canPolyAccerlate()
		self.m_csNode.accerlate:setVisible(canAccerlate)
		self.m_csNode.finish:setVisible(not canAccerlate)
		self.m_csNode.jinbi:setVisible(not canAccerlate)
	else
		self.m_csNode.jinbi:setVisible(false)
		self.m_csNode.accerlate:setVisible(false)
		self.m_csNode.finish:setVisible(false)
	end
end

function This:refreshPriceUI(cleanup)
	local costModeIsDirty = false
	local costMode = nil
    if cleanup and self.m_costMode then
        self.m_costMode = nil
    end

	if self.m_state == PolyState.kPrepare or self.m_state == PolyState.kUpdating then
		costMode = 0
		for i, info in ipairs(self.m_fileterInfoList) do
			costMode = info.quality + costMode
		end

        if costMode <= 0 then
            costMode = nil
        end
	end

	if costMode and self.m_costMode ~= costMode then
		if costMode then
			self.m_costMode = costMode
		end
		costModeIsDirty = true
	end

	if costModeIsDirty then
		local time = self.m_configs[costMode].time
		self.m_totalCDTime = time
		if self.m_state == PolyState.kUpdating then
			time = self.m_polyInfo:getLeftTime()
		end
		self:updatePloyTime(time)
		costModeIsDirty = false
	end
end

-----------------------------------------------------------------------
function This:startPoly()
	-- if self.m_build:getFortune() <= 0 then
	-- 	g_scene:openDialog("Territory.TerrFortunePanel", {build = self.m_build})
	-- 	g_scene:showTips(Serialize[64489])
	-- 	return
	-- end

	local paramList = {}
	-- local needWarming = false
	for i, info in ipairs(self.m_fileterInfoList) do
		if i > self.m_costPropCount then
			break
		end
		
		-- if not needWarming then
		-- 	needWarming = info.quality > 3
		-- end

		if info.uid <= 0 then
			g_scene:showTips(Serialize[64477])
			return 
		end
		table.insert(paramList, info.uid)
	end

	-- if not needWarming then
	self.m_build:onPolymerizeRequest(paramList)
	-- else
	-- 	g_scene:openDialog("UI.ConfirmPanel", {
    --         title = Serialize[64479],
    --         richText = Serialize[64480],
    --         okHandler = function ()
	-- 			self.m_build:onPolymerizeRequest(paramList)
    --         end
    --     })
	-- end
end

function This:startAccerlate()
	if not self.m_polyInfo:isRunning() or not self.m_build:canPolyAccerlate() then
		return
	end
	self.m_build:onPloyAccerlateRequest()
end

function This:finishPoly()
	if not self.m_polyInfo:isRunning() or self.m_build:canPolyAccerlate() then
		return
	end

	GamePlayer:costDiamond({
		title = Serialize[64479],
		desc = Serialize[64481], 
		cost = self.m_polyInfo:getCostByLeftTime(), 
		okHandler = function()
			self.m_build:onFinishPolyRequest()
		end
	})
end

function This:onQueueUpdateEnd()
    self:getCostInfo()
    self:refresh(true)
end

function This:onWorkerUpdateEnd()
    self:refreshButtonUI()
end

function This:updateQueueTime( info )
	if not self.m_isRunningAnimate and  info and info.time > 0 then
    	self:updatePloyTime(info.time)
    end
end

function This:updatePloyTime(leftTime)
    if leftTime > 0 then
    	local percent = (self.m_totalCDTime - leftTime) / self.m_totalCDTime
		self.m_csNode.rhtime:setString(GameUtil:formatTime(Serialize["H_M_S"], leftTime))
		self.m_csNode.jindutiao.jindutiao:setPercent(math.ceil(percent * 100))
		self.m_csNode.jinbi.number_jinbi:setString(self.m_polyInfo:getCostByLeftTime())
		return percent
    end
    return 1
end

function This:playAccAnimation(startNum, detalTime, finishHandler)
	self.m_isRunningAnimate = true
	self.m_effectJinDu:setVisible(true)
	local info = {startNum=startNum, endNum= self.m_polyInfo:getLeftTime(), interval= detalTime}
	ViewUtil:runCountDownAction(
		self.m_csNode.rhtime, 
		info, 
		function (node, showNum)
			local percent = self:updatePloyTime(showNum)
			self.m_effectJinDu:setPositionX(self.m_sourcePosX + self.m_sourceWidth * percent)
		end, 
		function()
			self:refresh()
			self.m_effectJinDu:setVisible(false)
			self.m_isRunningAnimate = false
			local ret, worker = self.m_build:hasPloyWorker()
			if ret and worker then
				worker:notifyObservers("speakRandom", startNum)
			end
		end
	)
end

function This:refreshPreviewUI(fortune)
       
       local itemNum = 0;
       local quality = 0;
       local weightlist = {};
       for i, cfg in ipairs(PreviewRewardConfig) do
            weightlist[cfg.id] = { preview = cfg.preview,weight = 0}
       end

       for i, info in ipairs(self.m_fileterInfoList) do
            if i > 3 then
			    break
		    end 
            if info.uid > 0 then
                itemNum = itemNum + 1
                quality = quality+ info.quality
            end
       end
       if itemNum < 3 then
            self.m_parentNode.yulan.tips:setVisible(true)
            self.m_parentNode.yulan.tips:setString(Serialize[64482])
            self.m_parentNode.yulan.itemlist:setVisible(false)
            return
       else
            self.m_parentNode.yulan.tips:setVisible(false)
            self.m_parentNode.yulan.itemlist:setVisible(true)  

               for i, cfg in ipairs(InosculateSkillConfig) do
		            if quality == cfg.skill_quality then 
                        
                          for id,weight in ipairs(cfg.weight) do
                               itemType =  weight[1]
                               weightlist[itemType].weight = weightlist[itemType].weight + weight[2] 
                          end
                          for id,weight in ipairs(cfg.particular) do
                               itemType =  weight[1]
                               weightlist[itemType].weight = weightlist[itemType].weight + weight[2] 
                          end

                        break
                    end
	           end 

                for i, cfg in ipairs(LuckyConfig) do
		            if fortune >= cfg.min_num and fortune <= cfg.max_num then 
                        
                          for id,weight in ipairs(cfg.arise_weight) do
                               itemType =  weight[1]
                               weightlist[itemType].weight = weightlist[itemType].weight + weight[2] 
                          end
                          for id,weight in ipairs(cfg.arise_particular) do
                               itemType =  weight[1]
                               weightlist[itemType].weight = weightlist[itemType].weight + weight[2] 
                          end

                        break
                    end
	           end 
               local allwight1 = 0
               local allwight2 = 0;
               for i, info in ipairs(weightlist) do
                    if i <= 4 then
                        allwight1 = allwight1 + info.weight
                    else
                        allwight2 = allwight2 + info.weight
                    end
               end
               local itemListNode = self.m_parentNode.yulan.itemlist

               for i = 1,5 do
                    itemListNode["need" .. i]:setVisible(false) 
               end 

               local index = 1
               for i, info in ipairs(weightlist) do
                    if info.preview > 0 and info.weight > 0 then
                           if i > 4 then
                                 index = 5 --最后一格固定显示特效奖励
                           end

                           local needNode = itemListNode["need" .. index]
                           
                           needNode:setVisible(true) 
                           local node = needNode.item
                           if not node.itemNode then
			                    local itemNode = ItemNode:create(info.preview, nil, false, true)
			                    itemNode:setAnchorPoint(0.5, 0.5)
                                itemNode:setScale(1.2,1.2)
			                    node:addChild(itemNode)
			                    node.itemNode = itemNode
		                    else
			                    node.itemNode:initItem(info.preview, nil, false)
		                    end 
                            local num = 0
                            if i<= 4 then
                                num = string.format("%.2f",info.weight/allwight1*100)
                               
                            else
                                num = string.format("%.2f",info.weight/allwight2*100)
                            end
                            needNode.num:setString(num .."%")
                            index = index+1
                    end  
               end  
       end
end