-- @Author: superm
-- @Date:   2018-06-23 14:20:58
-- @Last Modified by:   superm
-- @Last Modified time: 2018-06-25 13:41:03
-- @File Name: TerrBlackSmithWorker.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local WorkerPriceConfig = require("Metadatas.WorkerPriceConfig")
      
local WorkerState = 
{
	kClose = 0,  
	kOpening = 1, --正开启
	kLock = 2, 
}

function This:create(...)
	local o = {}
	setmetatable(o,self)
	o:init(...)
	return o
end

function This:init(info)
	super.init(self)
	self.m_id = info.id
	self.m_state = info.state
	self.m_endTime = info.end_time
	self.m_config = WorkerPriceConfig[self.m_id]
	self.m_escape = 0
	self.m_isUpdating = false
	local curTime = GamePlayer:getServerTime()
	if self.m_state ~= WorkerState.kLock then
		if curTime < self.m_endTime then
			self.m_state = WorkerState.kOpening
			self:startUpdate()
		else 
			self.m_state = WorkerState.kClose
		end
	end
end

function This:updateInfo( info )
	if info.id ~= self.m_id then
		return
	end

	self.m_id = info.id
	self.m_state = info.state
	self.m_endTime = info.end_time
	self.m_config = WorkerPriceConfig[self.m_id]

	local curTime = GamePlayer:getServerTime()
	if self.m_state ~= WorkerState.kLock then
		if curTime < self.m_endTime then
			self.m_state = WorkerState.kOpening
		elseif curTime >= self.m_endTime then
			self.m_state = WorkerState.kClose
		end
	end 
	
	if self.m_isUpdating and self.m_state == BoxState.kClose then
		self:endUpdate()
	elseif not self.m_isUpdating and self.m_state == BoxState.kOpening then
		self:startUpdate()
	end
end

function This:startUpdate()
	Application:addToUpdateList(self)
	self:notifyObservers("onWorkerUpdateStart",self)
	self.m_isUpdating = true
end

function This:endUpdate()
	Application:removeFromUpdateList(self)
	self.m_state = WorkerState.kClose
	self:notifyObservers("onWorkerUpdateEnd",self)
	self.m_isUpdating = false
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_endTime then
		return 0
	end
	return self.m_endTime - curTime
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			self:notifyObservers("updateWorkerTime",leftTime)
		else
			self:notifyObservers("updateWorkerTime",0)
			self:endUpdate()
		end
	end
end

function This:getId()
	return self.m_id
end

function This:getState()
	return self.m_state
end

function This:isOpening()
	return self.m_state == WorkerState.kOpening
end

function This:getConfig()
	return self.m_config
end

function This:getCDTotalTime()
	return self.m_config.time
end

function This:getDerateTime()
	return self.m_config.derate
end

function This:getPrice()
    return self.m_config.price
end

function This:getName()
   return self.m_config.name
end
