-- @Author: Su
-- @Date:   2019-06-29 14:51:06
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-29 14:51:06
-- @File Name: TerrBossRobber.lua
-- @Description:

-- local This , super = class(..., "Territory.TerrRobber")
local This , super = BaseClass( ... )
local PersonalBossConfig = require("Metadatas.PersonalBossConfig")
local BattleDataManager = require("Battle.BattleDataManager")
local GamePlayer = require("Player.GamePlayer")
local TerritoryManager = require("Territory.TerritoryManager")
local Report = require("Report.Report")

function This:create(...)
    local ret = This.New(...)
    return ret
end

function This:addObserver(o)
    for i , observer in pairs(self.m_observers) do
        if observer == o then
            return 
        end
    end
    table.insert(self.m_observers,o)
end

function This:removeObserver(o)
    for i , observer in pairs(self.m_observers) do
        if observer == o then
            table.remove(self.m_observers,i)
            return
        end
    end
end

function This:notifyObservers(funcName,...)
    local observers = {}
    for i , o in pairs(self.m_observers) do
        table.insert(observers,o)
    end
    
    for i, observer in pairs(observers) do
        if observer[funcName] then
            observer[funcName](observer,...)
        end
    end
end

function This:__init( info, createHandler )
    self.m_observers = {}
    self.m_copy = nil
    self:initBaseData( info )
    self.m_view = nil
    self.m_config = info.config
    self.m_state = info.state or TerrObjectState.kNormal
    self.m_escape = 0
    self.m_upIsUpdating = false
    self:updateState(true, info, createHandler)
    -- if info.report and info.report.id and info.report.id > 0 then
    --     print("贼寇pos_id = " .. self.m_ID .. ", id = " .. self.m_robberId .. ", " .. "有报告" .. info.report.id)
    -- end
end

function This:__before_delete()

    if self.m_reportInfo then
        self.m_reportId = 0
        self.m_reportInfo = nil
    end

    if self.m_view and self.m_view[0] ~= 0 then
        print("【删除】贼寇pos_id = " .. self.m_ID .. ", id = " .. self.m_robberId .. ", " .. "有报告" .. self.m_reportId)
        self.m_view:removeFromParent(false)
    end

    TerritoryManager:getBossRobberBy( self:getId(), -1 )
    self.m_view = nil
end

function This:initBaseData( info )
    self.m_ID = info.pos_id or 0
    self.m_robberId = info.id or 0
    self.m_upStartTime = info.walk_start_time or 0
    self.m_upEndTime = info.walk_end_time or 0
end

function This:updateBaseData( info )
    self.m_ID = info.pos_id or 0
    self.m_robberId = info.id or 0
    self.m_upStartTime = info.walk_start_time or 0
    self:setEndTime(info.walk_end_time or 0)
end

function This:updateState(isInit, info, notUpdateAll)
    if isInit then
        self:initBeforeUpdateState(info, notUpdateAll)
    end

    local curTime = GamePlayer:getServerTime()
    if curTime < self.m_upEndTime then
        self.m_state = TerrObjectState.kUpdating
        self:startUpdate()
        return
    elseif curTime >= self.m_upEndTime then
        self:endUpdate()
        if self.m_state ~= TerrObjectState.kLocked then
            self.m_state = TerrObjectState.kNormal
        end
    end

    self:updateObjects()
end

function This:updateObjects()

end

function This:initBeforeUpdateState(info, notUpdateAll)
    if not self.m_config then
        self.m_config = PersonalBossConfig[self.m_robberId]
    end
    self.m_cardLists = info.card_list 
    self.m_defenderLists = info.zeikou_list 
    self.m_lastTime = info.last_zeikou_time or -1
    self.m_rewardList = info.item
    
    if not notUpdateAll then
        if info.report and info.report.id and info.report.id > 0 then
            self.m_reportInfo = info.report
            self.m_reportId = info.report.id
        else
            -- print("贼寇pos_id = " .. self.m_ID .. ", id = " .. self.m_robberId .. ", " ..  "没报告")
            self.m_reportInfo = nil
            self.m_reportId = 0
        end
    end
end

function This:updateInfo(info, notUpdateAll)
    self:updateBaseData( info )
    self:updateState(true, info, notUpdateAll)
end

function This:isLocked()
    return self.m_state == TerrObjectState.kLocked
end

function This:isOpening()
    return self.m_state == TerrObjectState.kNormal
end

function This:isUpdating()
    return self.m_state == TerrObjectState.kUpdating
end

function This:isEmpty()
    return false
end

function This:isNotLockedBy(condition)
    return true
end

function This:getLeftTime()
    local curTime = GamePlayer:getServerTime()
    if curTime >= self.m_upEndTime then
        return 0
    end
    return self.m_upEndTime - curTime
end

function This:setEndTime( endTime )
    self.m_upEndTime = endTime
    local curTime = GamePlayer:getServerTime()
    if curTime < self.m_upEndTime then
        self.m_state = TerrObjectState.kUpdating
        self:startUpdate()
    else
        self:endUpdate()
    end
    self:notifyObservers("onBossRobberInfoChanged", self:getId())
end

function This:startUpdate()
    cc.Application:addToUpdateList(self)
    self:notifyObservers("onBossRobberUpdateStart", self:getId())
    self.m_upIsUpdating = true
end

function This:endUpdate()
    cc.Application:removeFromUpdateList(self)
    if self.m_upEndTime > 0 then
       self:notifyObservers("onBossRobberUpdateEnd", self:getId())
    end
    self.m_upIsUpdating = false
end

function This:update( dt )
    self.m_escape = self.m_escape + dt
    if self.m_escape >= 1.0 then
        self.m_escape = self.m_escape - 1
        local leftTime = self:getLeftTime()
        if leftTime > 0 then
            self:notifyObservers("updateBossRobberTime", {time = leftTime, id = self:getId()})
        else
            self:notifyObservers("updateBossRobberTime", {time = 0, id = self:getId()})
            self:endUpdate()
        end
    end
end

function This:setView(view)
    self.m_view = view
end

function This:getView()
    return self.m_view
end

function This:setCopy( object )
    self.m_copy = object
end

function This:getCopy()
    return self.m_copy
end

function This:hasCopy()
    return self.m_copy ~= nil
end

function This:getId()
    return self.m_ID
end

function This:getRobberId()
    return self.m_robberId
end

function This:getObjectBy(handler)
    for id, obj in pairs(self.m_objects) do
        if handler and handler(obj) then
            return obj
        end
    end
    return nil
end

function This:removeObjectBy(handler)
    local resultId = nil
    for id, obj in pairs(self.m_objects) do
        if handler and handler(obj) then
            resultId = id
        end
    end
    
    if resultId then
        self.m_objects[resultId] = nil
    end
end

function This:hasReportInFight()
    local curTime = GamePlayer:getServerTime()
    if self.m_reportInfo and self.m_reportInfo.end_time >= curTime then
        return true
    end

    return false
end

function This:isAttackerVictory()
    return self.m_reportInfo.state == BattleResult.kAttackerVictory
end

function This:getReportId()
    if self.m_reportId and self.m_reportId > 0 then
        return self.m_reportId
    end
    return 0
end

function This:getReportInfo()
    return self.m_reportInfo
end

function This:readReport()
    -- TerritoryManager:setRobberCopy(self)
    BattleDataManager:addObserver(TerritoryManager)

    local params = {
        stageId = self:getBattleId(), 
        msg = { report = self:getReportInfo() }
    }
    cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, params)
end
--获取配置信息region begin

function This:getHard()
    return self.m_config.difficulty
end

function This:getConfig()
    return self.m_config
end

function This:getHardDesc()
    return TerrRobberNanduTxt[self:getHard()]
end

function This:getName()
    return self.m_config.name
end

function This:getReward()
    return self.m_config.reward
end

function This:getOffsetPostion()
    return TerrRobberPosOffset[self:getHard()]
end

function This:getBattleId()
    return self.m_config.scene or 2
end

function This:getRobberList()
    return self.m_config.npc_deploy
end

function This:getContentSize()
    return TerrRobberContentSize[self:getHard()]()
end

function This:getEffectOffset()
    return TerrRobberEffect[self:getHard()]()
end

function This:getModelNum()
    return self:getHard() * 2 + 1
end

function This:getBattleId()
    return self.m_config.scene
end

function This:getRobberList()
    return self.m_config.zeikou_list
end

function This:getRobberReward()
    return self.m_rewardList
end

--获取配置信息region end

--endregion