-- @Author: Su
-- @Date:   2019-07-10 20:29:31
-- @Last Modified by:   Su
-- @Last Modified time: 2019-07-10 20:29:31
-- @File Name: TerrBossRobberPanel.lua
-- @Description:

local This, super = class(...,"Dialog")
local TerritoryManager = require("Territory.TerritoryManager")
local DepotManager = require("Depot.DepotManager")
local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")
local WarfareManager = require("Warfare.WarfareManager")

function This:initNode(robber)
    self.m_robber = robber
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/Building10.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
    self:addChild(self.m_csNode)

    -- self.m_csNode.nandu:setString(Serialize[64500] .. self.m_robber:getHardDesc())


    self.m_csNode.title:setString(self.m_robber:getName())
    if self.m_robber:getReportId() > 0 and self.m_robber:isAttackerVictory() then
        self.m_csNode.button.text:setString(Serialize[61310])
    else
        self.m_csNode.button.text:setString(Serialize[64501])
    end
end

function This:onEnter()
    super.onEnter(self)
    DepotManager:addObserver(self)
    TerritoryManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    super.onExit(self)
    TerritoryManager:removeObserver(self)
    DepotManager:removeObserver(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    -- self.m_csNode.xzk.add:setTouchEnded(function()
    --     g_scene:openDialog("Vip.VipPanel")
    -- end)

    self.m_csNode.button:setTouchEnded(function()
        if self.m_robber:getReportId() > 0 and self.m_robber:isAttackerVictory() then
            TerritoryManager:onBattleVictory()
            self:close()
        else
            if not WarfareManager:hasTeam() then
                WarfareManager:onCreateTeamRequest(12, self.m_robber:getId())
            else
                local team = WarfareManager:getMyTeam()
                local war = team:getWar()
                if war:getType() ~= WarfareType.kBoss then
                    g_scene:showTips(Serialize[64502])
                    return
                end
            end
        end
    end)
end


function This:refresh()
    local items = self.m_robber:getReward()
    for index, item in ipairs(items) do
         self:refreshItem(index, item)
    end
end


function This:refreshItem(i, item)
    local curNode = self.m_csNode["reward" .. i]
    local node = curNode.itemNode
    if not node then
        node = ItemNode:create(item[1], item[2])
        node:setAnchorPoint(0.5, 0.5)
        curNode:addChild(node)
    else
        node:init(item[1], item[2])
    end
end