-- @Author: Su
-- @Date:   2019-06-29 14:51:06
-- @Last Modified by:   Su
-- @Last Modified time: 2019-06-29 14:51:06
-- @File Name: TerrBossRobberView.lua
-- @Description:
local This , super = class(...,"Territory.TerrRobberView")
local Timer = require("Timer")
local BossTeamWar = require("Warfare.BossTeamWar")
local TerritoryManager = require("Territory.TerritoryManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local WarfareManager = require("Warfare.WarfareManager")
local TerrBuildManager = require("Territory.TerrBuildManager")


function This:init(robber)
    self.m_robber = robber
    -- self.m_war= BossTeamWar:create(self.m_robber)
    self.m_build = TerrBuildManager:getBuildByType(TerrBuildType.kJiaoChang)
    self.m_war = self.m_build:getAllWarfareList(5)[robber:getId()]
    self.m_soilders = {}
    self:setContentSize(self.m_robber:getContentSize())
    self.m_timer = Timer:create(self)
    self:scheduleUpdate()
    self.m_inited = false
end

function This:onEnter()
    super.onEnter(self)
    WarfareManager:addObserver(self)
end

function This:onExit()
    super.onExit(self)
    WarfareManager:removeObserver(self)
end

function This:initTouchEvent()
    if not self.m_inited then
        self:setTouch({
            onEnded = function()
                -- if not WarfareManager:hasTeam() then
                --     WarfareManager:onCreateTeamRequest(12, self.m_robber:getId())
                -- else
                --     local team = WarfareManager:getMyTeam()
                --     local war = team:getWar()
                --     if war:getType() ~= WarfareType.kBoss then
                --         g_scene:showTips("已有其他队伍")
                --         return
                --     end
                -- end
                g_scene:openDialog("Territory.TerrBossRobberPanel", self.m_robber)
            end,
            isDynamic = true,
            isSwallow = false,
			name = "BossRobber"
        })
        self.m_inited = true
    end
end

function This:onCreateBossTeam(team)
    if team:getBossWar():getPosId() == self.m_robber:getId() then
        g_scene:openDialog("Warfare.TeamWarfareFundPanel",{war = self.m_war})
    end
end

function This:onExitAnimalTeam(team,iswin)
    if team:getBossWar()and team:getBossWar():getPosId() == self.m_robber:getId() then
        WarfareManager:onRemoveTeamRequest()
        self.m_war:setIsFight(iswin)
        self:refresh()
    end
end

function This:refresh(delay)

    local function update() 
        local soilderList= self.m_soilders
        self.m_soilders = {}
        for i, soilder in ipairs(soilderList) do
            soilder:removeFromParent()
        end

        -- local modelName = "ShuGuo_GongBing"BuBing3
        local modelName = "BuBing3"

        local count = self.m_robber:getModelNum()
        local soilderConfig = self.m_robber:getOffsetPostion()
        for i = 1 , count do
            local skeletonNode = cc.SkeletonNode:createCharacter(modelName)
            skeletonNode:scheduleUpdate()
            -- skeletonNode:playAnimation("animation", true)
            skeletonNode:playAnimation(StateName.STAND, true)
            if i == count then
                skeletonNode:setAnimationListener(self)
            end

            self:addChild(skeletonNode)
            skeletonNode:setScale(TerrBossRobberSize)
            self.m_soilders[i] = skeletonNode

            skeletonNode:setLocalZOrder(i)
            skeletonNode:setPosition(soilderConfig[i].x, soilderConfig[i].y)
        end

        if not self.m_effect then
            local info = self.m_robber:getEffectOffset()
            local effInfo = info.eff
            self.m_effect = GameUtil:createEffect("HongQuan",true,self)
            self.m_effect:setPosition(effInfo.position.x, effInfo.position.y)
            self.m_effect:setScale(effInfo.scale.x, effInfo.scale.y)
            self.m_effect:setAngle(effInfo.angle)
            self:addChild(self.m_effect, -1)

            local nameInfo = info.name
            self.m_nameBgSprite = cc.Sprite:create("Common/tuijianneirongdi.png")
            self.m_nameBgSprite:setPosition(nameInfo.position.x, nameInfo.position.y)
            self.m_nameBgSprite:setScale(nameInfo.scale.x, nameInfo.scale.y)
            self.m_nameBgSprite:setAngle(nameInfo.angle)
            self:addChild(self.m_nameBgSprite, 10)

            self.m_nameLabel = cc.Label:createNormalTTF(self.m_robber:getName(), nameInfo.font_size)
            self.m_nameLabel:setPosition(nameInfo.position.x, nameInfo.position.y)
            self.m_nameLabel:setAnchorPoint(0.5, 0.5)
            self.m_nameLabel:setColor(nameInfo.color.r, nameInfo.color.g, nameInfo.color.b)
            self.m_nameLabel:setAngle(nameInfo.angle)
            self:addChild(self.m_nameLabel, 11)
        else
            self.m_nameLabel:setString(self.m_robber:getName())
        end

        self:setContentSize(self.m_robber:getContentSize())
    end

    if delay then
        self.m_timer:setTimer(delay,function()
            update()
        end, "robber" .. tostring(self.m_robber:getId()))
    else
        update()
    end
end

function This:updateInfo(robber)
    self.m_robber = robber
    self:refresh()
end