-- @Author: superm
-- @Date:   2017-10-21 10:51:06
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-12 16:22:57
-- @File Name: TerrBuild.lua
-- @Description: 

local This , super = class(..., "Territory.TerrUpgradeModel")
local GamePlayer = require("Player.GamePlayer")
local TerrBuildManager = require("Territory.TerrBuildManager")
local PrivilegeManager = require("Vip.PrivilegeManager")
local TerrBuildView = require("Territory.TerrBuildView")
local TerrBuildConfig = require("Metadatas.TerrBuildConfig")

function This.createBaseBuildings()
    local TerrBuildTypeConfig = require("Metadatas.TerrBuildTypeConfig")
    for _, cfg in ipairs(TerrBuildTypeConfig) do
        if not TerrBuildMap[cfg.build_type] then
            TerrBuildMap[cfg.build_type] = {}
        end
        TerrBuildMap[cfg.build_type][cfg.index] = cfg.uid
        if cfg.build_type > 0 and cfg.build_type < 999 then
            local building = TerrBuildManager:getBuildBy(cfg.uid)
            if not building then
                local build = nil
                if not cfg.sec_func then
                    build = This:create()
                else
                    build = require("Territory.TerrBuilding" .. TerrBuildType[cfg.build_type]):create()
                end
                local info = { buildId = cfg.config_id, descConfig = cfg }
                build:initByConfig(info)
                TerrBuildManager:addBuild(nil, build)
                build:hasProduction()
            end
        elseif cfg.build_type > 999 then
            local info = { buildId = cfg.config_id, descConfig = cfg }
            local build = require("Territory.TerrBuilding" .. cfg.type_name):create()
            build:initByConfig(info)
            TerrBuildManager:addFakeBuild(build)
        end
    end
end

----=========================================================---

function This:create(...)
	local ret = {}
	setmetatable(ret,self)
	ret:init(...)
	return ret
end

function This:hasListener()
end

function This:initListener()
end

function This:removeListener()
end

function This:createView()
    return TerrBuildView:create(self)
end

function This:initIdByInfo(info)
    if info then
        -- self.m_descConfig = TerrBuildTypeConfig[info.id]
        local maxNum = self.m_descConfig.max_level
        if maxNum < info.level then
            info.level = maxNum
        end
        self:setId(10000 + info.id * 100 + info.level)
        self.m_config = TerrBuildConfig[self:getId()]
    end
end

function This:initSelf( info )
    super.initSelf(self, info)
    self.m_nextConfig = TerrBuildConfig[self:getId() + 1]
    if info then
        self:updateReward( info.reward_item_id, info.reward_item_num )
    end
end

function This:updateSelf( info )
    super.updateSelf(self, info)
    if info then
        self:updateReward( info.reward_item_id, info.reward_item_num )
    end
end

function This:updateReward( idList, numList )
    self.m_rewardList = nil
    if numList and #numList > 0 then
        self.m_rewardList = {}
        for i, id in ipairs(idList) do
            self.m_rewardList[id] = numList[i]
        end
    end
end

function This:updateOfflineReward( idList, numList )
    self.m_offlineRewardList = nil
    if numList and #numList > 0 then
        self.m_offlineRewardList = {}
        for i, id in ipairs(idList) do
            self.m_offlineRewardList[id] = numList[i]
        end

        -- if self:hasOfflineReward() then
        --     print("建筑" .. self:getName() .. "存在离线收益，" .. "id:" .. self:getId())
        --     for id, num in pairs(self.m_offlineRewardList) do
        --         print("收益有：" .. require("Bag.ItemUtil").getName(id), num)
        --     end
        -- end
    end
end

function This:initByConfig( info )
    super.initByConfig(self, info)
    self.m_config = TerrBuildConfig[info.buildId]
    self.m_nextConfig = TerrBuildConfig[info.buildId + 1]
    self:setId(info.buildId)
    self.m_level = info.level or 0
    self:updateState(true)
end

function This:isNotLockedBy( cond1, cond2 )
    local switch = {
        [TerrConditionType.kNone] = function()
            return true
        end,
        [TerrConditionType.kPlayerLev] = function(arg)
            return  arg <= GamePlayer:getLevel()
        end,
        [TerrConditionType.kTerrMainBuildLev] = function(arg)   
            return arg <= TerrBuildManager:getBuildLevel()
        end,
    }

    return switch[cond1](cond2)
end

function This:getAccerlateCost( time )
    local freeTime = PrivilegeManager:getPrivilegeNumBy(Privilege.kBuildTime)
    if time <= freeTime then
        return 0
    end
    local ret = TerrFormula.kBuildAccSkipCost(time - freeTime)
    return ret
end

function This:isInNoDisturbMode(mode)
    return false
end

function This:isSameType( info )
    return self.m_level == info.level and self.m_state == info.state
end

function This:isInFreeTime()
    return self:getLeftTime() <= PrivilegeManager:getPrivilegeNumBy(Privilege.kBuildTime)
end

function This:hasOfflineReward()
    return self.m_offlineRewardList ~= nil and table.count( self.m_offlineRewardList ) > 0
end


function This:hasFunctions()
    return false
end

function This:hasProduction()
    local ret = TerrBuildHasProduct[self:getType()]
    if ret then
        return ret
    else
        ret = false
        local obj = self.m_config.build_num
        if obj and type(obj) == "table" then
            if obj[1] then
                local firstObj = type(obj[1]) == "table" and obj[1] or obj
                ret = #firstObj > 1 and firstObj[1] > 0 and firstObj[2] > 0
            end
        end
        TerrBuildHasProduct[self:getType()] = ret
        return ret
    end
end

function This:getProductTimeIntervalNum()
    return math.floor(self:getProductTime() / kTerrBuildProductIntervalInView) / kProductRefreshTimeFactor
end

function This:getCurProduction()
    if self:getLevel() <= 0 then
        return 0, 0
    end

    local itemId, itemNum = self:getProduction()
    if not self.m_rewardList then
        return itemId, math.floor(itemNum * self:getProductTimeIntervalNum())
    else
        for id, number in pairs(self.m_rewardList) do
            if id == itemId then
                local retNumber = number 
                return id, retNumber
            end
        end
    end
    return 0, 0
end

function This.getConfigIdBy( buildType, idx )
    return TerrBuildMap[buildType][idx or kTerrMinBuildId]
end

function This:getType()
    return self.m_descConfig.build_type
end

function This:getTypeName()
    return self.m_descConfig.type_name
end

function This:getIndex()
    return self.m_descConfig.index
end

function This:getSelfFuc()
    return self.m_descConfig.sec_func
end

function This:getTypeBy( buildId )
    for type, ids in ipairs(TerrBuildMap) do
        for idx, id in pairs(ids) do
            if id == buildId then
                return type, idx
            end
        end
    end
    return 0
end

function This:getProduction()
    if self:hasProduction() then
        return self.m_config.build_num[1],  self.m_config.build_num[2] * kProductFactor
    else
        return nil, nil
    end
end

function This:getNextLevProduction()
    if self.m_nextConfig.build_num ~= nil and self.m_nextConfig.build_num ~= 0 then
        return self.m_nextConfig.build_num[1],  self.m_nextConfig.build_num[2] * kProductFactor
    else
        return nil, nil
    end
end

function This:getImage()
    return self.m_descConfig.pic or ("GameScene/building" .. self:getType() .. ".png")
end

function This:getBuildAtrrNum()
    local ret = {}
    if self:getLevel() > 0 and self.m_config and self.m_config.atrr_id then
        for index, attrId in ipairs(self.m_config.atrr_id) do
            ret[attrId] = self.m_config.atrr_num[index] or 0
        end
    end
    return ret
end

function This:getRequest(mode)
    return  function( delay )
                if self:isUpgrading() then
                    self:requestAccerlate()
                else
                    if not delay then
                        return g_scene:openDialog("Territory.TerrBuildUpgradePanel", self)
                    else
                        local view = self:getView()
                        if view then
                            view:setTimer(0.3, function()
                                g_scene:openDialog("Territory.TerrBuildUpgradePanel", self)
                            end, "open_dialog")
                        end
                    end
                end
            end
end

function This:getRequestName( )
    return self:getRequestNameBy(self:getType())
end

function This:getRequestNameBy( buildType )
    if not buildType then
        if not self:isUpgrading() then
            return TerrSpecialName[self:getType()] or kTerrDefaultFuncionName
        else
            return kTerrAccFuncionName
        end
    end

    return TerrFunctionName[buildType] or kTerrDefaultFuncionName
end

function This:hasFuncQueue()
    return false
end

function This:hasHelpList()
    return false
end

function This:getHelpListCount()
    return 0
end

function This:onHelpFuncionEvent()

end

--请求
function This:requestUpgrade( skipMode, dialog ,opType)
    if opType and opType == TerrUpType.kNow then
        TerrBuildManager:onGetBuildUpgradeNowRequest(self:getType(), self:getIndex(), self:getConfigId())
        return
    end
    local hasQueue, state, queueId = TerrBuildManager:hasEmptyBuildQueue()
    local closePanel = false
    if hasQueue then
        TerrBuildManager:onBuildUpgradeRequest(self:getType(), self:getIndex(), self:getConfigId())
        if skipMode then
            TerrBuildManager:onBuildUpAccelerateRequest(queueId)
            if skipMode == TerrAccelerateMode.kImmFinish then--加速走了新的协议
                TerrBuildManager:onBuildUpAccelerateRequest(queueId)
            end
        end
    else
        local ret = self:requestAddBuildQueue(state, queueId, dialog)
        closePanel = not ret
    end
    return hasQueue, closePanel
end

function This:requestAddBuildQueue( state, queueId, dialog )
    dialog = dialog or g_scene
    if state == TerrAccelerateMode.kAddQueue then
        local panel = dialog:openDialog("Territory.TerrBuildConfirmPanel", {mode = state})
        panel:setCancelHandler(function()
            if PrivilegeManager:hasPrivilege( Privilege.kBuildQueueUnlock, true ) then
                local emptyQueue = TerrBuildManager:getTempEmptyBuildQueue(queueId)
                local newPanel = g_scene:openDialog("Territory.TerrBuildQueueUnlockPanel", emptyQueue)
                newPanel:setUnlockHandler(function(days)
                    TerrBuildManager:onBuildQueueBuyRequest(days)
                end)
            end
        end)

        panel:setOkHandler(function()
            local build, isInFreeTime = TerrBuildManager:getUpgradingBuild()
            local mode = isInFreeTime and TerrAccelerateSkipMode.kInFreeTime or TerrAccelerateSkipMode.kNone
            build:requestAccerlate(mode)
        end)

        return true
    else
        local upgradingBuild, isInFreeTime = TerrBuildManager:getUpgradingBuild()
        if upgradingBuild and isInFreeTime then
            g_scene:showTips(Serialize[64505] .. upgradingBuild:getName() .. Serialize[64506])
            return false
        end
        local panel = dialog:openDialog("Territory.TerrBuildConfirmPanel", {mode = state, build = upgradingBuild})
        panel:setOkHandler(function(build)
            build:requestAccerlate()
        end)
        return true
    end
end

local function buildAccerlate(build, skipMode)
    if not skipMode then
        skipMode = TerrAccelerateSkipMode.kNone
    end
    local canAccerlate, state = build:canAccerlate(skipMode == TerrAccelerateSkipMode.kPreView)
    if canAccerlate then
        if skipMode == TerrAccelerateSkipMode.kNone then
            TerrBuildManager:onBuildUpAccelerateRequest(build:getQueueId())
        elseif skipMode == TerrAccelerateSkipMode.kNoTips then
            TerrBuildManager:onBuildUpAccelerateRequest(build:getQueueId())
            TerrBuildManager:onBuildUpAccelerateRequest(build:getQueueId())
        end
    else
        local switch = {
            [TerrResponse.kFailByTimeEnd] = function()
                g_scene:showTips(Serialize[64507])
            end,
            [TerrResponse.kFailByCost] = function()
                GamePlayer:checkDiamond(0, true)
                -- g_scene:showTips(Serialize[64508])
            end,
            [TerrResponse.kFailByStateErr] = function()
                g_scene:showTips(Serialize[64509])
            end,
        }
        switch[state]()
    end
    return canAccerlate
end

function This:requestAccerlate( skipMode, needJump, dialog )
    if skipMode then 
        if skipMode == TerrAccelerateSkipMode.kInFreeTime then
            TerrBuildManager:onBuildUpAccelerateRequest(self:getQueueId())
            return true
        elseif skipMode == TerrAccelerateSkipMode.kPreView then
            return buildAccerlate(self, skipMode)
        end
    end

    if self:isInFreeTime() then
        TerrBuildManager:onBuildUpAccelerateRequest(self:getQueueId())
        return true
    else
        if needJump then
            local panel =  g_scene:openDialog("Territory.TerrBuildConfirmPanel", {mode = TerrAccelerateMode.kImmFinish, build = self})
            panel:setVisible(false)
            panel:setOkHandler(buildAccerlate)
            return true, panel  
        else
            dialog = dialog or g_scene
            local panel =  dialog:openDialog("Territory.TerrBuildConfirmPanel", {mode = TerrAccelerateMode.kImmFinish, build = self})
            panel:setOkHandler(buildAccerlate) 
            return true
        end
    end
end

function This:requestonGetReward(needLock)
    if self:hasProduction() then
        TerrBuildManager:onGetBuildGeneralRewardRequest(self:getType(), self:getIndex(), self:getConfigId(), needLock)
    end
end

--请求结束

--endregion
