-- @Author: superm
-- @Date:   2017-10-09 17:53:43
-- @Last Modified by:   superm
-- @Last Modified time: 2018-01-31 10:52:40
-- @File Name: TerrBuildConfirmPanel.lua

local This , super = class(...,"Dialog")
local TerrBuildManager = require("Territory.TerrBuildManager")
local VipManager = require("Vip.VipManager")
local PrivilegeManager = require("Vip.PrivilegeManager")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

function This:initNode(info)
	self.m_build = info.build
    if self.m_build then
	    self.m_totalTime = self.m_build:getUpgradeTime()
    end

	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/Building3.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	for i=TerrAccelerateMode.kImmFinish, TerrAccelerateMode.kQueueFull do
		self.m_csNode["mode" .. i]:setVisible(false)
	end

	self.m_mode = info.mode or TerrAccelerateMode.kImmFinish
	self.m_price = 0
    self.m_freeTime = PrivilegeManager:getPrivilegeNumBy(Privilege.kBuildTime)
	self.m_uiNode = self.m_csNode["mode" .. self.m_mode]
	self.m_uiNode:setVisible(true)

	if self.m_mode == TerrAccelerateMode.kImmFinish then
		self:setTitle(self.m_build:getName() .. Serialize[64511])
		self:refreshFreeTime()
	-- elseif self.m_mode == TerrAccelerateMode.kAddQueue then
	elseif self.m_mode == TerrAccelerateMode.kQueueFull then
		local time = self.m_build:getLeftTime()
		self:updateTimeNode( time )
	end
end

function This:onEnter()
    super.onEnter(self)
    VipManager:addObserver(self)
    if self.m_build then
    	self.m_build:addObserver(self)
    	self.m_build:updateState()
    end
end

function This:onExit()
    if self.m_build then
    	self.m_build:removeObserver(self)
    end
    VipManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)

	if self.m_uiNode.button1 then
		self.m_uiNode.button1:setTouchEnded(function()
			if self.m_cancelHandler then
				self.m_cancelHandler()
			end
			self:close()
		end)
	end
	
	if self.m_uiNode.button2 then
		self.m_uiNode.button2:setTouchEnded(function()
			if self.m_okHandler then
				if self.m_mode == TerrAccelerateMode.kImmFinish then
					GamePlayer:costDiamond({
		            	cost = self.m_price, 
		            	desc = Serialize[64512], 
		            	okHandler = function()
			            	if self.m_price > 0 then
								self.m_okHandler(self.m_build, TerrAccelerateSkipMode.kNoTips)
							else
								self.m_okHandler(self.m_build)
							end
							self:close()
			            end
			        })
				else
					self.m_okHandler(self.m_build)
					self:close()
				end
			else
				self:close()
			end	
		end)
	end

	if self.m_uiNode.xzk then
		self.m_uiNode.xzk:setTouchEnded(function()
			if self.m_otherHandler then
				self.m_otherHandler()
				self:close()
			end	
			self:openDialog("Vip.VipPanel")
		end)
	end
end

function This:setCancelHandler(handler)
	self.m_cancelHandler = handler
end

function This:setOkHandler(handler)
	self.m_okHandler = handler
end

function This:setOtherHandler(handler)
	self.m_otherHandler = handler
end

function This:setTitle(str)
	self.m_uiNode.title:setString(str)
end

function This:setText(str)
	self.m_uiNode.text:setString(str, 547)
end

function This:refreshFreeTime()
	if not VipManager:isFullMember() then
		self.m_uiNode.xzk:setVisible(true)
		self.m_uiNode.vip:setVisible(true)
	else
		self.m_uiNode.xzk:setVisible(false)
		self.m_uiNode.vip:setVisible(false)
	end

	local time = self.m_build:getLeftTime()
	-- local freeTime = PrivilegeManager:getPrivilegeNumBy(Privilege.kBuildTime)
    local accerlateTime = time - self.m_freeTime
	self:updateTimeNode( time, accerlateTime, self.m_freeTime )
end

function This:updateTime(time)
	if self.m_mode == TerrAccelerateMode.kImmFinish then
		-- local freeTime = PrivilegeManager:getPrivilegeNumBy(Privilege.kBuildTime)
	    local accerlateTime = time - self.m_freeTime
		if accerlateTime > 0 then
			self:updateTimeNode( time, accerlateTime, self.m_freeTime )
		else
			g_scene:showTips(Serialize[64513])
			self:close()
		end
	-- elseif self.m_mode == TerrAccelerateMode.kAddQueue then
	elseif self.m_mode == TerrAccelerateMode.kQueueFull then
		self:updateTimeNode( time )
	end
end

function This:updateTimeNode( time, accerlateTime, freeTime )
	if self.m_mode == TerrAccelerateMode.kImmFinish then
		self.m_price = TerrFormula.kBuildAccSkipCost(accerlateTime)
		self:setText(Serialize[64514] .. GameUtil:formatTime(Serialize["H_M_S"], accerlateTime) .. Serialize[64515] .. GameUtil:formatTime(Serialize["H_M_S"], freeTime) .. ")")
		self.m_uiNode.button2.price:setString( tostring(self.m_price) )
		self.m_uiNode.time:setString(Serialize[60249] .. GameUtil:formatTime(Serialize["H_M_S"], time))
		self.m_uiNode.jindubg.jindu:setPercent( (self.m_totalTime - time) / self.m_totalTime * 100)
	-- elseif self.m_mode == TerrAccelerateMode.kAddQueue then
	elseif self.m_mode == TerrAccelerateMode.kQueueFull then
		self:setText(Serialize[64516] .. self.m_build:getName() .. Serialize[64517] .. GameUtil:formatTime(Serialize["H_M_S"], time) .. Serialize[64518])
	end
end

-- function This:onTouchMask()
--     if self.m_csNode.close then
-- 	    self:close()
--     end
-- end

------------------------------------------------------

function This:onGetVipInfo( )
    self:refreshFreeTime()
end

--endregion
