-- @Author: superm
-- @Date:   2017-10-09 17:05:00
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-12 17:09:56
-- @File name: TerrBuildManager.lua

local This , super = class(..., "Base.BaseModel")
local TerrUtil = require("Territory.TerrUtil")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GuideManager = require("Guide.GuideManager")
local TerrBuild = require("Territory.TerrBuild")
local TerrBuildQueue = require("Territory.TerrBuildQueue")
local TerrBuildQueueConfig = require("Metadatas.TerrBuildQueueConfig")
local GameUtil = require("Util.GameUtil")
local PrivilegeManager = require("Vip.PrivilegeManager")
local HeroManager = require("Hero.HeroManager")
local PopDialogManager = require("UI.PopDialogManager")

function This:isInited()
    return self.m_inited
end

function This:init()
	super.init(self)

    self.m_builds = {}
    self.m_fakeBuilds = {}
    self.m_priorityList = {}
    self.m_buildQueues = {}

    self.m_tmpBuildQueues = {}

    self:initOffilineData()

    --封地建筑
    TerrBuild.createBaseBuildings()

    for id, cfg in pairs(TerrBuildQueueConfig) do
        local info = {index = id, config = cfg}
        self.m_tmpBuildQueues[id] = TerrBuildQueue:create(info)
    end

    if table.count(self.m_buildQueues) <= 0 then
        self.m_buildQueues[1] = self.m_tmpBuildQueues[1]
    end

    self:initListener()
    self.m_inited = true
end

function This:initOffilineData()
    self.m_offlineRewardTimeLimit = 0
    self:resetOffReward()
end

function This:resetOffReward()
    self.m_offlineReward = {
        name = {},
        image = {},
        list = {},
        state = {},
        time = 0,
        timeLimit = self.m_offlineRewardTimeLimit,
    }
end

function This:setOffRewardTimeLimit( num )
    self.m_offlineRewardTimeLimit = num
    if not self.m_offlineReward then
        self.m_offlineReward = { timeLimit = self.m_offlineRewardTimeLimit }
    end
end

function This:initBuildingListener()
    if not self.m_buildInited then
        self.m_buildInited = true
        for id, build in pairs(self.m_builds) do
            build:initListener()
        end

        for id, build in pairs(self.m_fakeBuilds) do
            if build:hasListener() then
                build:initListener()
            end
        end
    end
end

function This:removeBuildingListener()
    for id, build in pairs(self.m_builds) do
        build:removeListener()
    end

    for id, build in pairs(self.m_fakeBuilds) do
        if build:hasListener() then
            build:removeListener()
        end
    end
end

function This:initListener()
	EventDispatcher:addListener(self,"init_role_info", self.initBuildingListener)
    EventDispatcher:addListener(self,11500, self.onGetTerritoryBuildInfoReply)
	EventDispatcher:addListener(self,11501, self.onBuildUpgradeReply)
	EventDispatcher:addListener(self,11502, self.onBuildUpAccelerateReply)
	EventDispatcher:addListener(self,11503, self.onBuildQueueBuyReply)
	EventDispatcher:addListener(self,11506, self.onBuildUpFinishReply)

    EventDispatcher:addListener(self,11507, self.onGetBuildGeneralRewardReply)
    EventDispatcher:addListener(self,11517, self.onGetOfflineRewardReply)
	EventDispatcher:addListener(self,11521, self.onBuyOfflineTimeLimitReply)
    EventDispatcher:addListener(self,11524, self.onGetBuildUpgradeNowReply)
    -- self:initBuildingListener()

end
function This:removeListener()
    self:removeBuildingListener()
    EventDispatcher:removeListener(self,11500)
    EventDispatcher:removeListener(self,11501)
    EventDispatcher:removeListener(self,11502)
    EventDispatcher:removeListener(self,11503)
    EventDispatcher:removeListener(self,11506)
    
    EventDispatcher:removeListener(self,11507)
    EventDispatcher:removeListener(self,11517)
    EventDispatcher:removeListener(self,11521)
    EventDispatcher:removeListener(self,11524)
end

function This:addBuild( info, baseBuild )
    local build = baseBuild
    if not build then
        build = self:getBuildBy(info.id)
    	if not build then
    		build = TerrBuild:create(info)
            self.m_builds[build:getConfigId()] = build
            self:notifyObservers("addBuild", build)
        else
            build:updateInfo(info)
            self:notifyObservers("updateBuild", build)
        end
    else
        self.m_builds[build:getConfigId()] = build
        self:notifyObservers("initBuild", build)
    end
	return build:getConfigId(), build
end

function This:addFakeBuild( baseBuild )
    local build = baseBuild
    if build then
        self.m_fakeBuilds[build:getConfigId()] = build
    end
    return build:getConfigId(), build
end

function This:setBuildCount(value)
    self.m_allBuildCount = value
end

function This:getBuildCount()
    return self.m_allBuildCount
end

function This:addBuildQueue( info )
	local buildFactory = function( buildInfo, queue )
        buildInfo.last_auto_time = nil
		local id, build = self:addBuild(buildInfo)
		build:setQueue(queue)
		return id, build 
	end
    
	local queue = self:getBuildQueueBy(info.index)
    if not queue then
        queue = TerrBuildQueue:create(info, buildFactory)
    	self.m_buildQueues[queue:getId()] = queue
    	self:notifyObservers("addBuildQueue", queue)
    else
    	queue:updateInfo(info, buildFactory)
    	self:notifyObservers("updateBuildQueue", queue)
    end
    return queue
end

function This:getBuildLevel(buildType, idx)
    local btype = buildType or TerrBuildType.kGuanfu
    local buildConfigId = TerrBuildMap[btype][idx or 1]
    local build = nil
    if buildConfigId > 0 and buildConfigId < 11 then
        build = self.m_builds[buildConfigId]
    else
        return 0
    end

    if not build then
        return 0
    end
    return build:getLevel()
end

function This:getAllBuildings()
    if not self.m_allBuildList then
        local buildList = {}
        self.m_allBuildList = buildList
        for _, v in pairs(self.m_builds) do
            table.insert(buildList, v)
        end
        for _, v in pairs(self.m_fakeBuilds) do
            table.insert(buildList, v)
        end
    end
    return self.m_allBuildList
end

function This:getAllBuildingsUpdateState()
	for id, build in pairs(self.m_builds) do
        if id ~= TerrBuildType.kGuanfu then
            build:updateState()
        end
    end
end

function This:setUpgradeList( id, value )
    self.m_priorityRecord = nil
    self.m_priorityList[id] = value 
end

function This:getRecommendUpgradeBuild()
    if not next(self.m_priorityList) then
        return
    end

    if not self.m_priorityRecord then
        local tmpList = GameUtil:shallowChange(self.m_priorityList)
        table.sort(tmpList, function(a, b)
            return a > b
        end )
        local num = kTerrMaxBuildId - tmpList[1] % 100
        if num > 0 then
            self.m_priorityRecord = self.m_builds[num]
            return num, self.m_priorityRecord
        end
    else
        return self.m_priorityRecord:getConfigId(), self.m_priorityRecord
    end          
end

function This:getBuildBy( buildConfigId )
    if buildConfigId >= kTerrMinBuildId and buildConfigId <= kTerrMaxBuildId then
        return self.m_builds[buildConfigId]
    else
        return self.m_fakeBuilds[buildConfigId]
    end
end

function This:getBuildByType( buildType, idx )
    local buildConfigId = TerrBuild.getConfigIdBy(buildType, idx)
    return self:getBuildBy( buildConfigId )
end

function This:getAllBuildQueues()
    return self.m_buildQueues
end

function This:getAllTempEmptyBuildQueue()
	return self.m_tmpBuildQueues
end

function This:getBuildQueueBy( index )
    return self.m_buildQueues[index]
end

function This:getTempEmptyBuildQueue( index )
	return self.m_tmpBuildQueues[index]
end

function This:getUpgradingBuild()
    for _, queue in pairs(self.m_buildQueues) do
    	local build = queue:getUpgradingBuildInFreeTime()
    	if build then
    		return build, true
    	end
    end

    for _, queue in pairs(self.m_buildQueues) do
        local build = queue:getUpgradingBuild()
        if build then
            return build, false
        end
    end
    return nil, false
end

function This:hasEmptyBuildQueue(isUnlockAction)
    if isUnlockAction then
        if kTerrBuildQueuesNum <= table.count(self.m_buildQueues) then
            return false, TerrAccelerateMode.kQueueFull, 0
        end
    else
        local list = GameUtil:shallowChange(self.m_buildQueues)
        table.sort( list, function ( queueA, queueB )
            local timeA, timeB = queueA:getStartTime(), queueB:getStartTime() 
            if timeA ~= timeB then
                return timeA < timeB
            else
                return queueA:getId() < queueB:getId() 
            end
        end )
    	local found = false
    	for _, queue in ipairs(list) do
    		found = (not queue:isLocked()) and queue:isEmpty()
    		if found then
    			return true, TerrAccelerateMode.kImmFinish, queue:getId() 
    		end
    	end
    end

    local foundLocked = false
    for id, queue in pairs(self.m_buildQueues) do
        foundLocked = queue:isLocked() and queue:isEmpty()
        if foundLocked then
            return false, TerrAccelerateMode.kAddQueue, id
        end
    end


	for id, _ in pairs(TerrBuildQueueConfig) do
		if not self.m_buildQueues[id] then
			return false, TerrAccelerateMode.kAddQueue, id
		end
	end
	return false, TerrAccelerateMode.kQueueFull, 0

end

-----------------------------------------------------------------------------------

function This:onGetTerritoryBuildInfoRequest()
	Socket:send(11500)
end

function This:onGetTerritoryBuildInfoReply( msg )
	if msg then
        local ret = false
        for _, info in ipairs(msg.building_list) do
			if not ret and info.type == TerrBuildType.kGuanfu then
                ret = true
                self:addBuild(info)
            end

		end

    	for _, info in ipairs(msg.building_list) do
            if info.type ~= TerrBuildType.kGuanfu then
                local _, build = self:addBuild(info)
				if build:getType() == TerrBuildType.kBingYing then
					local index = build:getIndex()
					HeroManager:updateSoilderAttr(index)
				end
				HeroManager:updateBuildAttr(build)
            end
		end

		for _, info in ipairs(msg.queue_list) do
			self:addBuildQueue(info)
		end

        self:setOffRewardTimeLimit(msg.buy_limit or 0)
		HeroManager:updateAllSoilderAttr()

        EventDispatcher:dispatcher("terr_build_info")
	else
		g_scene:handleErrorCode("")
	end
end

function This:onBuildUpgradeRequest( buildType, idx, buildId )
	Socket:sendAndLock(11501, { type = buildType, index = idx, id = buildId})
end

function This:onBuildUpgradeReply(msg)
	if msg.err_code == 0 then
		local queue = self:addBuildQueue(msg.queue)
        self:notifyObservers("onBuildUpgrade", queue:getId())
		local build = queue:getBuild()
		EventDispatcher:dispatcher("terr_build_upgrade_start",build)
	else
		g_scene:handleErrorCode( msg.err_code )
	end
end

function This:onBuildUpAccelerateRequest( queueId )
	Socket:sendAndLock(11502, { queue = queueId })
end

function This:onBuildUpAccelerateReply(msg)
	if msg.err_code == 0 then
		local id, build = self:addBuild(msg.building)
		local queue = self:addBuildQueue(msg.queue)
        if TerrUtil:isObjectInvalid(g_scene) then
            if build:isUpgrading() then
                g_scene:showTips(Serialize[64550])
            else
                g_scene:showTips(build:getName() .. Serialize[61257])
            end
        end
        HeroManager:updateBuildAttr(build)
        self:notifyObservers("onBuildUpAccelerate", id)
        self:notifyObservers("onBuildUpAccelerateForQueue", queue:getId())
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onBuildQueueBuyRequest( costDays )
	Socket:sendAndLock(11503, {days = costDays})
end

function This:onBuildQueueBuyReply( msg )
	if msg.err_code == 0 then
		-- optional int32 days = 2;
		local queue = self:addBuildQueue(msg.queue)
        self:notifyObservers("onBuildQueueBuy")
        g_scene:showTips(Serialize[64551])
        if msg.queue.index <= 1 then
            print("后端数据有误, msg.queue.index=" ..  msg.queue.index)
            print(msg.queue)
            print("---------------------------------")
        end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onBuildUpFinishReply( msg )	
	if msg.building and msg.queue then
		local id, build = self:addBuild(msg.building)
        if id and id == TerrBuildType.kGuanfu then
            --更新其他建筑信息
            self:getAllBuildingsUpdateState()
        end
		if build:getType() == TerrBuildType.kBingYing then
			local index = build:getIndex()
			HeroManager:updateSoilderAttr(index)
		end
        HeroManager:updateBuildAttr(build)

        self:notifyObservers("onBuildUpFinish", id)
        if msg.queue.index ~= 0 then
            local queue = self:addBuildQueue(msg.queue)
            self:notifyObservers("onBuildUpFinishForQueue", queue:getId())
        end
		EventDispatcher:dispatcher("terr_build_level_changed",id)
	end
end 

function This:onGetBuildGeneralRewardRequest( buildType, buildIndex, buildId, needLock )
    print("获取建筑id=" .. buildId .. "的收益")
    if not needLock then
       Socket:send(11507, {type = buildType, index = buildIndex, id = buildId} )
    else
	   Socket:sendAndLock(11507, {type = buildType, index = buildIndex, id = buildId} )
    end
end

function This:onGetBuildGeneralRewardReply( msg )
	if msg and msg.err_code == 0 then
        local id, build = self:addBuild(msg.building)
        local itemList = {}
        if msg.item_id and #msg.item_id > 0 then
            for i, number in ipairs(msg.item_num) do
                if number > 0 then
                    table.insert(itemList,{id = msg.item_id[i], num = number})
                end
            end
        end
        self:notifyObservers("onGetBuildGeneralReward", id)
        if TerrUtil:isObjectInvalid(g_scene) then
            g_scene:showReward(itemList)
        end
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end


--離綫收益
function This:onGetOfflineRewardReply( msg )
    if msg.reward then
        self:resetOffReward()

        for _, reward in ipairs(msg.reward) do
            if TerrBuildHasProduct[reward.type] then
                local buildId, buildIndex, buildType = reward.id, reward.index, reward.type
                local build = self:getBuildBy(buildId)
                if build then
                    build:updateOfflineReward(reward.item_id, reward.item_num)
                    self.m_offlineReward.time = build:getProductTime()
                end

                local productionId, _ = build:getProduction()
                local itemList = self.m_offlineReward.list[buildType]
                if not itemList then
                    itemList = {}
                    itemList[productionId] = 0
                    self.m_offlineReward.name[buildType] = build:getTypeName()
                    self.m_offlineReward.image[buildType] = build:getImage()
                    self.m_offlineReward.list[buildType] = itemList
                    self.m_offlineReward.state[buildType] = false
                end
                if #reward.item_num > 0 then
                    self.m_offlineReward.state[buildType] = true
                    for i, itemId in ipairs(reward.item_id) do
                        local itemNum = itemList[itemId] or 0
                        itemNum = itemNum + reward.item_num[i]
                        if itemNum > 0 then
                            itemList[itemId] = itemNum
                        end
                    end
                end
            end
        end

        for buildType, value in pairs(TerrBuildHasProduct) do
            if value and not self.m_offlineReward.list[buildType] then
                local build = self:getBuildByType(buildType)
                self.m_offlineReward.name[buildType] = build:getName()
                self.m_offlineReward.image[buildType] = build:getImage()
                local productionId, _ = build:getProduction()
                local itemList = self.m_offlineReward.list[buildType]
                if not itemList then
                    itemList = {}
                    itemList[productionId] = 0
                    self.m_offlineReward.list[buildType] = itemList
                    self.m_offlineReward.state[buildType] = false
                end
            end
        end
    else
        print("悄悄告诉你，没有离线收益哦")
    end
end


function This:pickUpOfflineReward()
    for id, build in ipairs(self.m_builds) do
        if build and build:hasOfflineReward() then
            print("建筑id=" .. build:getConfigId() .. "有离线收益")
            local unaryPredicate = function(msg)
                if not msg or not msg.building then
                    return false
                end
                if msg.building.id == build:getConfigId() then
                    build:updateOfflineReward()
                    return true
                end
                return false
            end

            build:requestonGetReward()
        end
    end
    self:resetOffReward()
end


function This:showOfflineReward()
    if self.m_offlineReward.state then
        local canOpen = false

        for _, state in pairs(self.m_offlineReward.state) do
            if state then
                canOpen = true
            end
        end

        if canOpen and GuideManager:isInGuide() then
            canOpen = false
            self:pickUpOfflineReward()
        end

		if canOpen then
			self.m_offlineReward.pickUpHandler = function()
				self:pickUpOfflineReward()
			end
            PopDialogManager:addPopDialog({
                bCheck = true,
                name = "Territory.TerrOfflineIncomePanel", 
                arg = self.m_offlineReward, 
                level = DialogLevel.kSecLevel
            })
            return true
		end
    end
    return false
end

function This:onBuyOfflineTimeLimitRequest(  )
    Socket:sendAndLock(11521)
end

function This:onBuyOfflineTimeLimitReply( msg )
    if msg.err_code == 0 then
        if TerrUtil:isObjectInvalid(g_scene) then
            g_scene:showTips(Serialize[64552])
        end
        self:setOffRewardTimeLimit(self.m_offlineRewardTimeLimit + 1)
        self:notifyObservers("onBuyOfflineTimeLimit", self.m_offlineRewardTimeLimit)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
--建筑的立刻升级，走了新的协议
function This:onGetBuildUpgradeNowRequest(buildType, idx, buildId)
    Socket:sendAndLock(11524,{ type = buildType, index = idx, id = buildId})
end

function This:onGetBuildUpgradeNowReply( msg )
    if msg.err_code == 0 then
        local id, build = self:addBuild(msg.building)
        -- local queue = self:addBuildQueue(msg.queue)
        if TerrUtil:isObjectInvalid(g_scene) then
            if build:isUpgrading() then
                g_scene:showTips(Serialize[64550])
            else
                g_scene:showTips(build:getName() .. Serialize[61257])
            end
        end
        if build:getType() == TerrBuildType.kBingYing then
            local index = build:getIndex()
            HeroManager:updateSoilderAttr(index)
        end
        HeroManager:updateBuildAttr(build)
        self:notifyObservers("onBuildUpAccelerate", id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end