-- @Author: superm
-- @Date:   2017-11-27 16:32:53
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-20 20:36:57
-- @File Name: TerrBuildQueueView.lua
-- @Description: 

local This , super = class(...,"cc.Node")
local Timer = require("Timer")
local TerrBuildManager = require("Territory.TerrBuildManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local PrivilegeManager = require("Vip.PrivilegeManager")

local function print()
end

function This:init(queue)
    self.m_queue = queue
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/Building2.csb")
    self.m_effectList = {}
    self.m_doingAction = false

    self:addChild(self.m_csNode)
    self.m_timer = Timer:create(self)
    self:scheduleUpdate()
    self:setScale(0.75)
    self.m_csNode.mode3.time:setLocalZOrder(10)
    self.m_csNode.mode2.bg:setVisible(false)
end

function This:setPositionBy( view )
    local posX, posY = view:getPosition()
    self:setPosition(posX, posY - 64.0)
end

function This:onEnter()
    super.onEnter(self)
    if self.m_queue then 
        self.m_queue:setView(self)
        self.m_queue:addObserver(self)
    end
    TerrBuildManager:addObserver(self)
    self:initTouchEvent()
    self:refresh()
end

function This:onExit()
    super.onExit(self)
    if self.m_queue then 
        self.m_queue:setView(nil)
        self.m_queue:removeObserver(self)
    end

    self:unloadEffect()
    TerrBuildManager:removeObserver(self)
end


function This:initTouchEvent()
    self.m_csNode.bg2:setTouch({
        onEnded = function()
            if self.m_csNode.mode1:isVisible() then
                if self.m_queue:isVipBuildQueue()  then
                    PrivilegeManager:hasPrivilege( Privilege.kBuildQueueUnlock, true )
                else
                    local newPanel = g_scene:openDialog("Territory.TerrBuildQueueUnlockPanel", self.m_queue)
                    newPanel:setUnlockHandler(function(days)
                        TerrBuildManager:onBuildQueueBuyRequest(days)
                    end)
                end
            elseif self.m_csNode.mode3:isVisible() then
                if self.m_queue and not self.m_queue:isEmpty() then
                    local build = self.m_queue:getUpgradingBuild()
                    if build and not self.m_isBlocking then
                        self.m_isBlocking = true
                        if build:isInFreeTime() then
                            g_scene:focuseBuild(build:getConfigId(), true, function()
                                build:requestAccerlate(TerrAccelerateSkipMode.kInFreeTime)
                                self.m_isBlocking = false
                            end)
                        else
                            local ret, panel = build:requestAccerlate(TerrAccelerateSkipMode.kNone, true)
                            g_scene:focuseBuild(build:getConfigId(), true, function()
                                panel:setVisible(true)
                                self.m_isBlocking = false
                            end)
                        end
                    else
                        print("加速队列有错，没有在升级建筑")
                    end
                else
                    print("加速队列有错，队列为空")
                end
            else
                local leftTime = self.m_queue:getLeftTime()
                if not self.m_doingAction and leftTime > 0 then
                    self.m_doingAction = true
                    local node = self.m_csNode.mode2.info
                    node1 = self.m_csNode.mode2.bg
                    node:setString(Serialize[60249] .. string.secondToDay(self.m_queue:getLeftTime()))
                    node:setVisible(true)
                    node1:setVisible(true)
                    node1:runAction(cc.Sequence(cc.FadeIn(0.5), cc.DelayTime(1.25), cc.FadeOut(2.5), cc.Hide(), cc.CallFunc(function ()
                        self.m_doingAction = false
                        end)))
                    node:runAction(cc.Sequence(cc.FadeIn(0.5), cc.DelayTime(1.25), cc.FadeOut(2.5), cc.Hide(), cc.CallFunc(function ()
                        self.m_doingAction = false
                    end)))
                end

                local recommendId, build = TerrBuildManager:getRecommendUpgradeBuild()
                if recommendId and recommendId > 0 then
                    local request = build:getRequest(TerrBuildRequestMode.kUpgrade)
                    if request then
                        local panel = request()
                        if panel then
                            panel:setVisible(false)
                            g_scene:focuseBuild(recommendId, true, function()
                                panel:setVisible(true)
                            end)
                        end
                    end
                else
                    g_scene:showTips(Serialize[64560])
                end
            end
        end,
        isAlpha = false,
        isScale = false
    })
end

function This:loadEffect(name, index, scale)
    local effect = self.m_effectList[name]
    if not effect then
        print("loading ... ", name, tostring(self.m_queue:getId()))
        effect = GameUtil:createEffect(name, true, self)
        effect:setScale(scale or 1.0)
        self.m_effectList[name] = effect
        self.m_csNode["mode" .. index]:addChild(effect)            
    end
end

function This:unloadEffect(name)
    if name then
        print("unloadEffect ... ", name, tostring(self.m_queue:getId()))
        local effect = self.m_effectList[name]
        if effect then
            effect:removeFromParent()
            self.m_effectList[name] = nil
        end
    else
        for _, effect in pairs(self.m_effectList) do
            effect:removeFromParent()
        end
        self.m_effectList = nil
    end
end

function This:refresh(delay)
    -- print("----queue---".. self.m_queue:getId() .. "----------------------------refresh(delay)")
    local function update() 
        self.m_csNode.mode1:setVisible(false)
        self.m_csNode.mode2:setVisible(false)
        self.m_csNode.mode3:setVisible(false)
        if not self.m_queue or self.m_queue:isLocked() and self.m_queue:isEmpty() then
            self.m_csNode.mode1:setVisible(true)
            self:unloadEffect("Building")
            self.m_csNode.mode3.jiantou:setVisible(true)
            self.m_csNode.mode3.free:setVisible(false)
            self:unloadEffect("KongXian1")

        elseif not self.m_queue:isEmpty() then
            self.m_csNode.mode3:setVisible(true)
            local build = self.m_queue:getBuild()
            if build then
                -- self.m_csNode.mode3.level:setString(Serialize[64048] .. build:getLevel() .. " " .. build:getName())
                local leftTime = build:getLeftTime()
                self:updateTime(leftTime, build:getProgress())
                local visible = leftTime > PrivilegeManager:getPrivilegeNumBy(Privilege.kBuildTime)
                self.m_csNode.mode3.jiantou:setVisible(visible)
                self.m_csNode.mode3.free:setVisible(not visible)
                self:loadEffect("Building", 3, 1.7)
                self:unloadEffect("KongXian1")
            else
                print("加速队列有错，没有在升级建筑")
            end
        else
            self:unloadEffect("Building")
            self:loadEffect("KongXian1", 2)
            self.m_csNode.mode2:setVisible(true)
        end
    end

    if delay then
        self.m_timer:setTimer(delay,function()
            update()
        end, "build_queue" .. (self.m_queue and tostring(self.m_queue:getId()) or "empty"))
    else
        update()
    end
end

function This:updateTime(leftTime, progress)
    if leftTime > 0 then
        self.m_csNode.mode3.time:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
        self.m_csNode.mode3.jindutiao:setPercent(100 - progress * 100.0)
    else
        self:unloadEffect("Building")
        self:refresh(1)
    end
end

function This:update(dt)
    self.m_timer:update(dt)
end

function This:onBuildUpFinishForQueue( queueId )
    if self.m_queue and self.m_queue:getId() == queueId then
        self:refresh()
    end
end

function This:onBuildUpgrade( queueId )
    if self.m_queue and self.m_queue:getId() == queueId then
        self:refresh()
    end
end

function This:onQueueUpdateEnd( queueId )
    if self.m_queue and self.m_queue:getId() == queueId then
        local leftTime = self.m_queue:getBuildLeftTime()
        self.m_timer:setTimer(leftTime + 0.1, function()
            self.m_queue:updateState()
            self:refresh()
        end, "build_queue_end" .. (self.m_queue and tostring(self.m_queue:getId()) or "empty"))
    end
end

function This:onUpgradeEnd(queueId)
    if self.m_queue and self.m_queue:getId() == queueId then
        self.m_queue:updateState()
        self:refresh(0.1)
    end
end

function This:onQueueUpgradeFree()
    if self.m_queue then
        self.m_csNode.mode3.free:setVisible(true)
        self.m_csNode.mode3.jiantou:setVisible(false)
    end
end
--endregion