--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This, super = class(..., "Dialog")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrUtil = require("Territory.TerrUtil")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

function This:initNode(build)
    if type(build) == "number" and build > 0 then
        build = TerrBuildManager:getBuildBy(build)
    end

    self.m_build = build
    self.m_build:updateState()

	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/Manor1.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

    self:refresh()
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    self.m_csNode.open.levelup:setTouchEnded(function()
        self:upgradeBuilding(false,TerrUpType.kNormal)
    end)

    self.m_csNode.open.atonce:setTouchEnded(function()
        self:upgradeBuilding(true,TerrUpType.kNow)
    end)

    self.m_csNode.goto1:setTouchEnded(function()

		if not self.m_mainBuild then
			self:openDialog("UI.HelpPanel", {index = HelpPanelTabName.kLevelUp})
			return
		end

        if not self.m_mainBuild:isUpgrading() then
            self:setVisible(false)
            g_scene:focuseBuild(1, true, function()
                self:setVisible(true)
                self.m_build = self.m_mainBuild 
                self:refresh()
                self.m_mainBuild = nil
            end)

        else
            self.m_mainBuild:requestAccerlate(nil, nil, self)
        end
    end)

    self.m_csNode.open.goto2:setTouchEnded(function()
        GamePlayer:checkGold({nocheck = true, nowarming = true})
    end)
end

function This:quickUpgradeBuilding( accerlate, cost ,opType)
    local accerlateMode
    if accerlate then
        accerlateMode = TerrAccelerateMode.kNormal
        if cost then
            accerlateMode = TerrAccelerateMode.kImmFinish
        end
    end
    local _, response = self.m_build:requestUpgrade( accerlateMode, self ,opType)
    if response then
        self:close()
    end
end

function This:upgradeBuilding( accerlate ,opType)
    local canUpgrade, state, errCode = self.m_build:canUpgrade()
    if not canUpgrade then
        local switch = {
            [TerrResponse.kFailByCost] = function()
                if errCode then
                    g_scene:handleErrorCode(errCode)
                else
                    g_scene:showTips(Serialize[64562])
                end
            end,
            [TerrResponse.kFailByLocked] = function()
                g_scene:showTips(Serialize[64563])
            end,
        }
        switch[state]()
    else
        local costTime = self.m_build:getUpgradeTime()
        local accerlateCost = self.m_build:getAccerlateCost(costTime)
        if accerlateCost ==0 then
            self:quickUpgradeBuilding(accerlate, true,opType)
            return
        end

        -- if accerlate then
        --     local state = self.m_build:requestAccerlate(TerrAccelerateSkipMode.kPreView)
        --     if not state then
        --         return
        --     end
        -- end

        if accerlate and self.m_csNode.open.price:isVisible() then
            self:openDialog("UI.CostConfirmPanel", {
                cost = self.m_csNode.open.price:getString(),
                desc = Serialize[64564], 
                okHandler = function()
                    self:quickUpgradeBuilding(accerlate, true,opType)
                end 
            })
        else
            self:quickUpgradeBuilding(accerlate, false,opType)
        end
    end
end

function This:onEnter()
	super.onEnter(self)
    TerrBuildManager:addObserver(self)
    GamePlayer:addObserver(self)
end

function This:onExit()
    GamePlayer:removeObserver(self)
    TerrBuildManager:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
    local curBuildLev = self.m_build:getLevel()
    local isMaxLev = self.m_build:isMaxLevel()
    self.m_csNode.title:setString(self.m_build:getName())

    local switch = {
        [TerrConditionType.kPlayerLev] = Serialize[64565],
        [TerrConditionType.kTerrMainBuildLev] = Serialize[64566],
    }
    local cond1, cond2 = self.m_build:getCondition()
    if cond1 == TerrConditionType.kTerrMainBuildLev then
        self.m_mainBuild = TerrBuildManager:getBuildByType(TerrBuildType.kGuanfu)
    else
        self.m_mainBuild = nil
    end

    self.m_csNode.leveluptext:setVisible( false )
    self.m_csNode.full:setVisible(false)
    local visible = curBuildLev <= 0 or isMaxLev
    self.m_csNode.lock:setVisible(visible)
    self.m_csNode.open:setVisible(not visible)

    if visible then
        if not isMaxLev then
            if self.m_mainBuild and self.m_mainBuild:isUpgrading() then
                self.m_csNode.leveluptext:setVisible( true )
                self.m_csNode.condition1:setVisible( false )
                self.m_csNode.goto1.text:setString( Serialize[64567] )
            else
                self.m_csNode.goto1.text:setString( Serialize[64568] )
                TerrUtil:colorCondText(self.m_csNode, 1, false, switch[cond1], tostring(cond2) .. Serialize[60187])
            end

            self.m_csNode.lock:setString( Serialize[64569] .. switch[cond1] .. cond2 .. Serialize[60187])
        else
            TerrUtil:hideCondText(self.m_csNode)
            self.m_csNode.lock:setVisible(false)
            self.m_csNode.full:setVisible(true)
            self.m_csNode.open:setVisible(false)
        end
    else
        local costItemId, costNum = self.m_build:getUpgradeCost()
        local costTime = self.m_build:getUpgradeTime()
        local accerlateCost = self.m_build:getAccerlateCost(costTime)
        self.m_csNode.open.time:setString(GameUtil:formatTime(Serialize["H_M_S"], costTime))
        if accerlateCost > 0 then
            self.m_csNode.open.price:setVisible(true)
            self.m_csNode.open.price:setString(tostring(accerlateCost))
            self.m_csNode.open.icon:setVisible(true)
            self.m_csNode.open.free:setVisible(false)
        else
            self.m_csNode.open.icon:setVisible(false)
            self.m_csNode.open.price:setVisible(false)
            self.m_csNode.open.free:setVisible(true)
        end

        local canUpgrade, state = self.m_build:canUpgrade(true)
        local stateLevel, stateCost = false, false
        if not canUpgrade then
            if TerrResponse.kFailByCost == state then
                stateLevel = true
            elseif TerrResponse.kFailByLocked == state then
                stateCost = true
            end
        else
            stateLevel, stateCost = true, true
        end

        if stateLevel then
            TerrUtil:colorCondText(self.m_csNode, 1, true, switch[cond1], tostring(cond2) .. Serialize[60187])
        elseif self.m_mainBuild and self.m_mainBuild:isUpgrading() then
            self.m_csNode.leveluptext:setVisible( true )
            self.m_csNode.condition1:setVisible( false )
            self.m_csNode.goto1.text:setString( Serialize[64567] )
        else 
            self.m_csNode.goto1.text:setString( Serialize[64568] )
            TerrUtil:colorCondText(self.m_csNode, 1, false, switch[cond1], tostring(cond2) .. Serialize[60187])
        end
        TerrUtil:colorCondText(self.m_csNode.open, 2, stateCost, ItemUtil.getName(costItemId), tostring(costNum))
    end

    self.m_csNode.buildingIcon:setImage(self.m_build:getImage())
    self.m_csNode.levelText:setString(Serialize[64048] .. curBuildLev)
    if not isMaxLev then
        self.m_csNode.nextLevel:setString(curBuildLev + 1)
    else
        self.m_csNode.jiantou:setVisible(false)
        self.m_csNode.nextLevel:setVisible(false)
    end

    self:refreshDescr(curBuildLev, isMaxLev)
end

function This:refreshDescr( curBuildLev, isMaxLev )
    local attrList = {}
    local attr = self.m_build:getDescription()
    local itemId, num = self.m_build:getProduction()
    if attr ~= nil and attr ~= 0 and attr ~= "" and attr ~= "0" then
        table.insert(attrList, attr)
    end
    if itemId then
        if not isMaxLev then
            local delta
            if curBuildLev > 0 then
                local _, nextNum = self.m_build:getNextLevProduction()
                delta = nextNum - num
            else
                delta = num
                num = 0
            end
            table.insert(attrList, string.format(Serialize[64570], ItemUtil.getName(itemId), num, delta))
        else
            table.insert(attrList, string.format(Serialize[64571], ItemUtil.getName(itemId), num))
        end
    end
    for i=1, 2 do
        local txt = attrList[i]
        if txt then
            local describe = self.m_csNode["describe" .. i]
            describe:setVisible(true)
            describe:setStringWithColor(attrList[i], 340)
        else
            self.m_csNode["describe" .. i]:setVisible(false)
        end
    end  
end

function This:onBuildUpAccelerate(id)
    if self.m_build:getConfigId() == id then
        self:refresh()
    end
end

function This:onBuildUpgrade(queueId)
    if self.m_build:getQueueId() == queueId then
        self:close()
    end
end

function This:onBuildUpFinish( buildId )
    if not self.m_mainBuild then
        return
    end

    if buildId == TerrBuildType.kGuanfu then
        self:refresh(0.3)
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.gold and not self.m_build:isUpgrading() then
        local costItemId, costNum = self.m_build:getUpgradeCost()
        TerrUtil:colorCondText(self.m_csNode.open, 2, value >= costNum, ItemUtil.getName(costItemId), tostring(costNum))
    end
end

--endregion
