-- @Author: superm
-- @Date:   2017-11-21 16:51:53
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-05 14:49:43
-- @File Name: TerrBuildViewWithFunc.lua
-- @Description: 

local This , super = class(...,"Territory.TerrBuildView")
local GameUtil = require("Util.GameUtil")

function This:init(build)
    super.init(self, build)
    self.isEquipMake = nil
    if not self.m_clippingNode then
        local node = self.m_csNode.roll.func
        local clippingNode = cc.ClippingNode:create(node.bg)
        clippingNode:setAlphaThreshold(node.bg)
        node.bg:switchParent(clippingNode)
        node.icon:switchParent(clippingNode)
        self.m_clippingNode = clippingNode
        node:addChild(clippingNode)
        node.text:setLocalZOrder(10)
    end

    if build then
        local ret, image = build:hasHelpList()
        if ret and image then
            local helpNode = self.m_csNode.roll.help
            helpNode.icon:setImage(image)
        end
    end
end

function This:initTouchEvent()
    local rollNode = self.m_csNode.roll
    super.initTouchEvent(self)
    rollNode.func:setTouch({
        onEnded = function()
            if self.m_funcQueueIndex then
                local queue = self.m_build:getFuncQueueBy(self.m_funcQueueIndex)
                if self.isEquipMake then
                    queue = self.m_build:getFuncQueueByEquip()
                end
                if queue then
                    queue:onFunctionEvent(self.m_build, self.m_funcQueueIndex)
                end
            end
        end,
        isDynamic = true,
        isScale = false
    })    

    rollNode.help:setTouch({
        onEnded = function()
            if self.m_build:getHelpListCount() > 0 then
        		self.m_build:onHelpFuncionEvent()
        	end
        end,
        isDynamic = true,
        isScale = false
    })    
end

function This:updateQueueTime( info )
	if self.m_hasFuncQueue and self.m_funcQueueIndex == info.id then
		self.m_csNode.roll.func.text:setString(GameUtil:formatTime(Serialize["H_M_S"], info.time))
	end
end

function This:checkFuncReward()
    local funcNode = self.m_csNode.roll.func
    if self.m_hasFuncQueue then
        if self.m_funcQueueIndex then
            funcNode:setVisible(true)
            return true
        end

        local idx, image, num = self.m_build:getFuncQueueItem()
        if self.isEquipMake then
            idx, image, num = self.m_build:getFuncQueueItemForEquip()
        end
        if image then
            self.m_funcQueueIndex = idx
            funcNode:setVisible(true)
            funcNode.icon:setImage(image)
            if num then
                funcNode.text:setString(tostring(num))
            end
            return true
        end
    end
    funcNode:setVisible(false)
    return false
end

function This:checkHelpList()
    local helpNode = self.m_csNode.roll.help
	if self.m_hasHelpList then
        local count = self.m_build:getHelpListCount()

        if count > 0 then
            helpNode:setVisible(true)
            return true
        end
	end
	helpNode:setVisible(false)
	return false
end

function This:onQueueUpdateStart(idx,isEquipMake)
    self.isEquipMake = isEquipMake
    if self.m_hasFuncQueue then
        if not self.m_funcQueueIndex then
            self.m_funcQueueIndex = idx
            local queue = self.m_build:getFuncQueueBy(idx)
            if isEquipMake then
                queue = self.m_build:getFuncQueueByEquip(idx)
            end
            if queue then
            	local image = queue:getFuncItemImage()
                if image then
				    self.m_csNode.roll.func.icon:setImage(image)
                end
            end
        else
            self.m_funcQueueIndex = nil
        end
        self:refreshViewEffect(true)
    end
end

function This:onQueueUpdateEnd(idx,isEquipMake)
    self.isEquipMake = isEquipMake
    if self.m_funcQueueIndex == idx then
        local queue = self.m_build:getFuncQueueBy(idx)
        if isEquipMake then
            queue = self.m_build:getFuncQueueByEquip(idx)
        end
        if queue and queue.getFuncItemNum then
            local num = queue:getFuncItemNum()
            self.m_csNode.roll.func.text:setString(tostring(num))
        else
            self.m_funcQueueIndex = nil
			self.m_timer:setTimer(1, function (self)
                self:refreshViewEffect(true)
            end, "onQueueClearInfo")
        end
    end
end

function This:onStateChange(idx, curState, oldState)
    if self.m_hasFuncQueue then
        if curState then
            self:loadFreeEffect()
        else
            self:refreshFreeEffect()
        end
    end
end

function This:onQueueClearInfo(idx)
    if self.m_funcQueueIndex == idx then
    	local queue = self.m_build:getFuncQueueBy(idx)
        if self.isEquipMake then
            queue = self.m_build:getFuncQueueByEquip(idx)
        end
    	if queue then
    		self.m_funcQueueIndex = nil
			self.m_timer:setTimer(0.1, function (self)
                self:refreshViewEffect(true)
            end, "onQueueClearInfo")
    	end
    end
end

function This:onGetBuildHelpInfo( buildId )
    if self.m_build:getId() == buildId then
        self:refreshViewEffect(true)
    end
end

function This:onPolyHelpOne( buildId )
    if self.m_build:getId() == buildId then
        self:refreshViewEffect(true)
    end
end

function This:onPolyHelpAll( buildId )
    if self.m_build:getId() == buildId then
        self:refreshViewEffect(true)
    end
end
function This:refreshFreeEffect()
    if self.m_hasFuncQueue then
        local state = self.m_build:hasEmptyFuncQueue()
        if self.isEquipMake then
            state = self.m_build:hasEmptyFuncQueueForEquip()
        end
        if state then
            self:loadFreeEffect()
        elseif self.m_effectFree then
            self.m_effectFree:removeFromParent()
            self.m_effectFree = nil
        end
    end
end

function This:loadFreeEffect()
    if not self.m_effectFree then
        self.m_effectFree = GameUtil:createEffect("KongXian2", true, self)
        self.m_csNode:addChild(self.m_effectFree)         
    end
end
