-- @Author: superm
-- @Date:   2017-11-21 17:40:05
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-03 10:10:27
-- @File Name: TerrBuildingJiTan.lua
-- @Description: 

local This , super = class(..., "Territory.TerrBuild")
local TerrAltarShopSlotConfig = require("Metadatas.TerrAltarShopSlotConfig")
local TerrAltarItem = require("Territory.TerrAltarItem")

local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")

function This:init( ... )
    super.init( self, ... )
    self.m_altarItemList = {}
    for slotId, cfg in ipairs(TerrAltarShopSlotConfig) do
        local info = { index = slotId, slotConfig = cfg.need_rank }
        self:addTerrAltarItem(info)
    end
end

function This:hasFunctions()
    return true
end

function This:getRequest(mode)
    local requestMode = mode or TerrBuildRequestMode.kOther
    if requestMode == TerrBuildRequestMode.kUpgrade then
        return super.getRequest(self, mode)
    else

        if not self.m_clickFunc then
            self.m_clickFunc = function()
                g_scene:openDialog("Territory.TerrAltarPanel", self)
            end
        end
        return self.m_clickFunc
    end
end

function This:getAltarItemCount()
    return #self.m_altarItemList
end

function This:getAllTerrAltarItem()
    return self.m_altarItemList
end

function This:getTerrAltarItemBy(index)
    return self.m_altarItemList[index]
end

function This:addTerrAltarItem( info )
    local item = self.m_altarItemList[ info.index ]
    if not item then
        item = TerrAltarItem:create(info)
    else
        item:updateInfo(info)
    end
    self.m_altarItemList[item:getId()] = item
    self:notifyObservers("addTerrAltarItem", item)
    return item
end

function This:markDataTimes(times)
    self.m_dataRefreshTimes = times
end

function This:getDataTimes()
    return self.m_dataRefreshTimes or 0
end

--===============================================================================--
function This:initListener()
    EventDispatcher:addListener(self, 11519, self.onGetAltarInfoReply)
    EventDispatcher:addListener(self, 11520, self.onBuyAltarItemReply)
    EventDispatcher:addListener(self, 11522, self.onRefreshAltarReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self,11519)
    EventDispatcher:removeListener(self,11520)
    EventDispatcher:removeListener(self,11522)
end

function This:onGetAltarInfoRequest()
    Socket:send(11519)
end

function This:onGetAltarInfoReply( msg )
    if msg then
        if msg.queue_list and #msg.queue_list > 0 then
            -- self:markDataTimeBy(msg.last_refresh_time)
            self:markDataTimes(msg.refresh_tieme)
            local curLev = msg.queue_list[1].max_level
            if curLev > 0 then
                for _, info in ipairs(msg.queue_list) do
                    self:addTerrAltarItem(info)
                end
            end
        end
        self:notifyObservers("onGetAltarInfo")
    else
        g_scene:handleErrorCode("")
    end
end

function This:onBuyAltarItemRequest(slotId, itemId)
    Socket:sendAndLock(11520, {trench_id = slotId, prop_id = itemId})
end

function This:onBuyAltarItemReply( msg )
    if msg.err_code == 0 then
        if msg.queue_list and #msg.queue_list > 0 then
            local curLev = msg.queue_list[1].max_level
            if curLev > 0 then
                local rewardList = {}
                for _, info in ipairs(msg.queue_list) do
                    local item = self:addTerrAltarItem(info)
                    table.insert(rewardList, item)
                end
                for i, item in ipairs(rewardList) do
                    local id, num =  item:getConfigId(), item:getItemNum()
                    if num > 0 then
                        rewardList[i] = {id = item:getConfigId(), num = item:getItemNum()}
                    end
                end
                g_scene:showReward(rewardList)
            end
        end
        self:notifyObservers("onBuyAltarItem")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onRefreshAltarRequest()
    Socket:sendAndLock(11522)
end

function This:onRefreshAltarReply( msg )
    if msg.err_code == 0 then
        self:onGetAltarInfoReply( msg )
        self:notifyObservers("onRefreshAltar", msg.refresh_tieme)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

