-- @Author: superm
-- @Date:   2017-11-21 17:26:42
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-03 10:09:52
-- @File Name: TerrBuildingJiangJunFu.lua
-- @Description: 

local This , super = class(..., "Territory.TerrBuild")
local TerrPatrolConfig = require("Metadatas.TerrPatrolConfig")
local TerrBuildViewWithFunc = require("Territory.TerrBuildViewWithFunc")
local TerrPatrolQueue = require("Territory.TerrPatrolQueue")
local HeroManager = require("Hero.HeroManager")
local ItemUtil = require("Bag.ItemUtil")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")

function This:init( ... )
    super.init( self, ... )
    self.m_settings = {}
    self.m_patrolQueues = {}
    for idx, cfg in ipairs(TerrPatrolConfig) do
        local info = { index = idx, config = cfg }
        self:addPatrolQueue(info)
    end
end

function This:createView()
    return TerrBuildViewWithFunc:create(self)
end

function This:hasFunctions()
    return true
end

function This:updateInfo( info )
    super.updateInfo(self, info)
    if self:getLevel() > 0 then
        for index, queue in ipairs(self.m_patrolQueues) do
            if queue:isLocked() then
                queue:updateState()
            end
        end
    end
end

function This:setView(view)
    super.setView(self, view)
    for index, queue in ipairs(self.m_patrolQueues) do
        queue:setView(view)
    end

    if view and self:hasEmptyFuncQueue() then
        self:notifyObservers("onStateChange", 0, true, false)
    end
end

function This:getRequest(mode)
    local requestMode = mode or TerrBuildRequestMode.kOther
    if requestMode == TerrBuildRequestMode.kUpgrade then
        return super.getRequest(self, mode)
    else
        if not self.m_clickFunc then
            self.m_clickFunc = function()
                g_scene:openDialog("Territory.TerrPatrolPanel")
            end
        end
        return self.m_clickFunc
    end
end

function This:addPatrolQueue( info )
    local heroFactory = function( heroId )
        return HeroManager:getHeroByUid(heroId), self
    end

    local queue = self.m_patrolQueues[info.index]
    if not queue then
        queue = TerrPatrolQueue:create(info, heroFactory)
        self.m_patrolQueues[info.index] = queue
    else
        queue:updateInfo(info, heroFactory)
    end
    self:notifyObservers("addPatrolQueue", queue)
end

function This:getSettings()
    return self.m_settings
end

function This:clearSettings()
    self.m_settings = {}
end

function This:getPatrolQueues()
    return self.m_patrolQueues
end

function This:getPatrolQueueBy( index )
    return self.m_patrolQueues[index]
end

function This:hasFuncQueue()
    return true
end

This.getFuncQueueBy = This.getPatrolQueueBy
function This:getFuncQueue()
	local ret, time
    for index, queue in ipairs(self.m_patrolQueues) do
        if not queue:isEmpty() then
            local curTime = queue:getLeftTime()
            if not time or time > curTime then
                ret = queue
                time = curTime
            end
        elseif not ret and queue:isOpening() then
            ret = queue
    	end
    end
    return ret
end

function This:getFuncQueueItem()
    local queue = self:getFuncQueue()
    if not queue then
        return
    end

    if queue:isOpening() then
        local list = queue:getRewardList()
        if not list then return end

        local _, ret = next(list)
        if not ret then 
            return queue:getId()
        end
        return queue:getId(), ItemUtil.getIconImage(ret.itemId), ret.num
    else
        return queue:getId(), ItemUtil.getIconImage(queue:getHeroId())
    end
end

function This:hasEmptyFuncQueue()
    for _, queue in ipairs(self.m_patrolQueues) do
        local state = queue:isEmpty() and queue:isOpening()
        if state then
            return true
        end
    end
    return false
end


--=============================================================================--
function This:requestHeroSelect( queueId )
    if queueId then
        local heroId = self.m_settings[queueId].heroId
        local timeType = self.m_settings[queueId].timeType or kTerrDefaultPatrolTimeType
        local patrolType = self.m_settings[queueId].patrolType or kTerrDefaultPatrolTimeType
        if heroId and heroId > 0 then
            local hero = HeroManager:getHero(heroId)
            self:onPatrolHeroSelectRequest( hero:getUid(), queueId, timeType, patrolType )
        end
    end
end

function This:requestReward( queue )
    if queue then
        local queueId = queue:getId()    
        self:onGetPatrolRewardRequest(queueId)
    end
end

function This:initListener()
    EventDispatcher:addListener(self, 11504, self.onPatrolHeroSelectReply)
    EventDispatcher:addListener(self, 11508, self.onGetPatrolQueueReply)
    EventDispatcher:addListener(self, 11509, self.onGetPatrolRewardReply)
    EventDispatcher:addListener(self, 11523, self.onBuyPatrolQueueReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self,11504)
    EventDispatcher:removeListener(self,11508)
    EventDispatcher:removeListener(self,11509)
    EventDispatcher:removeListener(self,11523)
end

-- 选中武将，开始巡逻 
function This:onPatrolHeroSelectRequest( cardId, idx, timeType, patrolType )
    Socket:sendAndLock(11504, {card_id = cardId, index = idx, type = timeType, reward_level = patrolType})
end

function This:onPatrolHeroSelectReply( msg )
    if msg.err_code == 0 and msg.index > 0 then
        local queue = self:getPatrolQueueBy(msg.index)
        if queue then
            self:addPatrolQueue(msg)
            self:notifyObservers("onPatrolHeroSelect", msg.index)
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 获取巡逻武将的信息 
function This:onGetPatrolQueueRequest()
    Socket:sendAndLock(11508)
end

function This:onGetPatrolQueueReply( msg )
    if msg then
        for _, info in ipairs(msg.queue_list) do
            self:addPatrolQueue(info)
        end
        self:notifyObservers("onGetPatrolQueue")
    else
        g_scene:handleErrorCode("")
    end
end

-- 领取巡逻奖励 
function This:onGetPatrolRewardRequest(idx)
    Socket:sendAndLock(11509, {index = idx})
end

function This:onGetPatrolRewardReply( msg )
    if msg.err_code == 0 then
        if msg.index > 0 then
            local queue = self:getPatrolQueueBy(msg.index)
            queue:clearInfo()
        end
        self:notifyObservers("onGetPatrolReward")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 购买巡逻队列
function This:onBuyPatrolQueueRequest(idx)
    Socket:sendAndLock(11523, {index = idx})
end

function This:onBuyPatrolQueueReply( msg )
    if msg.err_code == 0 then
        local queue = self:getPatrolQueueBy(msg.index)
        queue:increaseBuyTime()
        queue:updateState()
        self:notifyObservers("onBuyPatrolQueue", msg.index)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
