-- @Author: superm
-- @Date:   2018-07-19 20:11:51
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-03-02 10:55:40
-- @File Name: TerrBuildingJiaoChang.lua
-- @Description: 

local This , super = class(..., "Territory.TerrBuild")
local TeamCampaignConfig = require("Metadatas.TeamCampaignConfig")
local CampaignBoxConfig = require("Metadatas.CampaignBoxConfig")
local MythicalCampaignConfig = require("Metadatas.MythicalCampaignConfig")
local TeamWar = require("Warfare.TeamWar")
local BossTeamWar = require("Warfare.BossTeamWar")
local AnimalTeamWar = require("Warfare.AnimalTeamWar")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local WarfareManager = require("Warfare.WarfareManager")
local TerrBuildViewWithFunc = require("Territory.TerrBuildViewWithFunc")
local TerritoryManager = require("Territory.TerritoryManager")
local TerrBossRobber = require("Territory.TerrBossRobber")


function This:init( ... )
    super.init( self, ... )
    self.m_settings = {}
    self.m_buttonList = {}

    self.m_boxRewardList = {}
    self.m_warfareList = {}
    self.m_warfareTypeList = {}
    self.m_needHelpList = {}

    self.m_animalWarFightTimes = 0
    self.m_animalWarIdList={}

    local warfareBoxesList = {}
    for _, config in ipairs(CampaignBoxConfig) do
        local warId = config.campaign_id
        local boxId = config.id

        local boxList = warfareBoxesList[warId]
        if not boxList then
            boxList = {}
            warfareBoxesList[warId] = boxList
        end
        local box = {
            id = boxId, 
            state = SaintBoxState.kLock, 
            warId = warId,
            reward = config.reward, 
            jifen = config.jifen
        }
        self.m_boxRewardList[boxId] = box
        table.insert(boxList, box)
    end

    for _, config in ipairs(TeamCampaignConfig) do
        local wtype = config.tpye
        local order = config.display_order
        local warList = self.m_warfareTypeList[wtype]
        if not warList then
            warList = {}
            self.m_warfareTypeList[wtype] = warList
        end
        local warfare = TeamWar:create({config = config, boxlist = warfareBoxesList[config.id]})
        warList[order] = warfare
        self.m_warfareList[config.id] = warfare
    end
end

function This:createView()
    self:onGetWarfareInfoRequest(true)
    return TerrBuildViewWithFunc:create(self)
end

function This:hasFunctions()
    return true
end

function This:hasHelpList()
    return not self:isLocked(), "MenuScene/wordketiaozhan1.png"
end

function This:getHelpListCount()
    if self.m_needHelpList then
        return #self.m_needHelpList
    end
    return 0
end

function This:onHelpFuncionEvent()
    local func = self:getRequest()
    if func then
        func()
    end
end

function This:getRequest(mode)
    local requestMode = mode or TerrBuildRequestMode.kOther
    if requestMode == TerrBuildRequestMode.kUpgrade then
        return super.getRequest(self, mode)
    else
        if not self.m_clickFunc then
            self.m_clickFunc = function ()
                WarfareManager:loadCacheBattle()
                g_scene:openDialog("Warfare.WarfarePanel", {build = self})
            end
        end
        return self.m_clickFunc
    end
end

function This:getAllWarfareList(warType)
    return self.m_warfareTypeList[warType] or {}
end

function This:getWarfare(warfareId)
    if not self.m_warfareList[warfareId] then
        local config = MythicalCampaignConfig[warfareId]
        if config then
            local warfare = AnimalTeamWar:create({config = config})
            return warfare
        end
    end
    return self.m_warfareList[warfareId]
end

function This:getBossWar(warId)
    if not self.m_warfareTypeList[WarfareType.kBoss][warId] then
        local bossRobber = TerrBossRobber:create({pos_id=warId,id=1})
        bossRobber.name = "boss"
        local bossWar = BossTeamWar:create(bossRobber)
        return bossWar
    else
        return self.m_warfareTypeList[WarfareType.kBoss][warId]
    end
end

function This:getAnimalWar()
    return self.m_animalWarIdList
end

function This:getAnimalWarFightTime()
    return self.m_animalWarFightTimes
end

function This:setAnimalWarData()
    local animalWarList = self:getAnimalWar()
    self.m_warfareTypeList[WarfareType.kAnimal] = {}
    local warList = self.m_warfareTypeList[WarfareType.kAnimal]
    local index = 1
    for _,warId in pairs(animalWarList) do
        local config = MythicalCampaignConfig[warId.id]
        local warfare = AnimalTeamWar:create({config = config})
        warfare:setIsFight(warId.is_challenge)
        warList[index] = warfare
        index = index + 1
        self.m_warfareList[config.id] = warfare
    end
    table.sort(warList,function(a,b)
        return a:getNextHard() >= b:getNextHard()
    end)

end

function This:onGetRobbersInfo(bossRobberList)
    if not bossRobberList then
        return
    end
    self.m_warfareTypeList[WarfareType.kBoss] = {}
    local warList = self.m_warfareTypeList[WarfareType.kBoss]
    local index = 1
    for _,bossRobber in pairs(bossRobberList) do
        local war = BossTeamWar:create(bossRobber)
        war:setIsFight(false)
        warList[bossRobber:getId()] =war
        index = index + 1
    end


end
-- function This:getBox(boxId)
--     return self.m_boxRewardList[boxId]
-- end

function This:initListener()
    EventDispatcher:addListener(self, "day_passed", self.onDayPassed)
    EventDispatcher:addListener(self, 12500, self.onGetWarfareInfoReply)
    EventDispatcher:addListener(self, 12517, self.onPickupWarBoxRewardReply)
    EventDispatcher:addListener(self, 12522, self.onRefreshAnimalWarReply)
    TerritoryManager:addObserver(self)
end

function This:removeListener()
    EventDispatcher:removeListener(self,12500)
    EventDispatcher:removeListener(self,12517)
    EventDispatcher:removeListener(self,12522)
    EventDispatcher:removeListener(self, "day_passed")
    TerritoryManager:removeObserver(self)
end

-------------------------------------------------------------------------------
function This:onGetWarfareInfoRequest(slient)
    if not slient then
        Socket:sendAndLock(12500)
    else
        Socket:send(12500)
    end
end

function This:onGetWarfareInfoReply( msg )
    if msg then
        self.m_needHelpList = {}
        for _, info in ipairs(msg.copy_progress) do
            local warfareId = info.copy_id
            if info.copy_id == 11 then
                self.m_animalWarIdList = info.shenshu_list
                self.m_animalWarFightTimes = info.captain_reward_times
                self:setAnimalWarData()
            end
            local warfare = self.m_warfareList[warfareId]
            if warfare then
                warfare:updateInfo(info)
                if warfare:isOpening() then
                    local time = warfare:getLeftRewardTimes()
                    if time > 0 then
                        table.insert(self.m_needHelpList, warfare)
                    end
                end
            end
        end
        local singleWarList = self.m_warfareTypeList[WarfareType.kSingle]
        if singleWarList then
            for _, warfare in ipairs(singleWarList) do
                if warfare:isOpening() then
                    local time = warfare:getLeftRewardTimes()
                    if time > 0 then
                        table.insert(self.m_needHelpList, warfare)
                    end
                end
            end
        end
        self:notifyObservers("onGetBuildHelpInfo", self:getId())
        self:notifyObservers("onGetWarfareInfo")
    end
end

function This:onPickupWarBoxRewardRequest(boxId)
    Socket:sendAndLock(12517, {reward_id = boxId})
end

function This:onPickupWarBoxRewardReply( msg )
    if msg.err_code == 0 then
        local box = self.m_boxRewardList[msg.reward_id]
        if box then
            g_scene:showReward(box.reward)
            local warfare = self.m_warfareList[box.warId]
            if warfare then
                warfare:updateInfo(msg)
            end
        end

        self:notifyObservers("onPickupWarBoxReward")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onDayPassed()
    self:onGetWarfareInfoRequest(true)
end


function This:onRefreshAnimalWarRequest(id)
    Socket:sendAndLock(12522, {type = id})
end

function This:onRefreshAnimalWarReply( msg )

    if msg.err_code == 0 then
        self.m_animalWarIdList = msg.shenshu_list
        self:setAnimalWarData()
        self:notifyObservers("onGetWarfareInfo")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end