-- @Author: superm
-- @Date:   2017-11-21 17:26:23
-- @Last Modified by:   superm
-- @Last Modified time: 2018-01-09 17:10:09
-- @File Name: TerrBuildingTaiXueYuan.lua
-- @Description: 

local This , super = class(..., "Territory.TerrBuild")
local TerrTechnManager = require("Territory.TerrTechnManager")
local PrivilegeManager = require("Vip.PrivilegeManager")
local TerrTechnQueue = require("Territory.TerrTechnQueue")
local TerrBuildViewWithFunc = require("Territory.TerrBuildViewWithFunc")

function This:init( ... )
    super.init( self, ... )
    self.m_tmpTechnQueues = {}
    for i=1, kTerrMaxTechnQueueNum do
        self.m_tmpTechnQueues[i] = TerrTechnQueue:create({index = i})
    end

    self.m_technQueues = {}
end

function This:createView()
    return TerrBuildViewWithFunc:create(self)
end

function This:setView(view)
    super.setView(self, view)
    for index, queue in ipairs(self.m_tmpTechnQueues) do
        queue:setView(view)
    end

    if view and self:hasEmptyFuncQueue() then
        self:notifyObservers("onStateChange", 0, true, false)
    end
end

function This:isInNoDisturbMode(mode)
    local requestMode = mode or TerrBuildRequestMode.kOther
    local isBuildUpgrading = self:isUpgrading()
    if requestMode == TerrBuildRequestMode.kOther then
        if not isBuildUpgrading then
            return false
        end
    else
        if self:hasEmptyTechnQueue() then
            return false
        end
    end

    if not PrivilegeManager:hasPrivilege(Privilege.kTerrTechnUpdate, true) then
        return true
    end
    return false
end

function This:hasFunctions()
    return true
end

function This:getRequest(mode)
    local requestMode = mode or TerrBuildRequestMode.kOther
    if requestMode == TerrBuildRequestMode.kUpgrade then
        if self:isUpgrading() then
        	return function ()
    	       self:requestAccerlate()
            end
        elseif not self:isInNoDisturbMode(TerrBuildRequestMode.kUpgrade) then
            return super.getRequest(self, mode)
        end
	else
		return function()
	        if not self:isInNoDisturbMode() then
	            g_scene:openDialog("Territory.TerrTechnPanel", self)
	        end
	    end
    end
end

function This:hasFuncQueue()
    return true
end

function This:getTechnQueueBy( index )
    local queue = self.m_technQueues[index] 
    if not queue then
        _, queue = next(self.m_tmpTechnQueues)
    end
    return queue
end
This.getFuncQueueBy = This.getTechnQueueBy

function This:getTechnQueueList()
    return self.m_tmpTechnQueues
end

function This:addTechnQueue(info, techn)
    local factory = function(technInfo, queue)
        if techn then
            self.m_technQueues[technInfo.id] = queue
            techn:setQueue(queue)
            return techn, self 
        else
            self.m_technQueues[technInfo.id] = nil
            return
        end
    end
    
    local queue = self:getTechnQueueBy(info.id)
    if queue then
        queue:updateInfo(info, factory)
        self:notifyObservers("updateTechnQueue", queue)
    end
    return queue
end

function This:getFuncQueue()
    local ret, time
    for id, queue in pairs(self.m_technQueues) do
        if not queue:isEmpty() then
            local curTime = queue:getLeftTime()
            if not time or time > curTime then
                ret = queue
                time = curTime
            end
        elseif not ret and queue:isOpening() then
            ret = queue
        end
    end
    return ret
end

function This:getFuncQueueItem()
    local queue = self:getFuncQueue()
    if not queue then
        return
    end

    if queue:isUpdating() then
        return queue:getId(), queue:getFuncItemImage()
    end
end

function This:hasEmptyTechnQueue()
    local ret, technId
    for _, queue in ipairs(self.m_tmpTechnQueues) do
        if (not queue:isLocked()) and queue:isEmpty() then
            ret = queue
            break
        elseif not technId then
            technId = queue:getTechnId()
        end
    end

    if ret then
        return true, TerrAccelerateMode.kImmFinish, 0
    else
        return false, TerrAccelerateMode.kQueueFull, technId or 0
    end
end
This.hasEmptyFuncQueue = This.hasEmptyTechnQueue
