-- @Author: superm
-- @Date:   2018-06-19 10:35:13
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-17 16:12:23
-- @File Name: TerrBuildingTieJiangPu.lua
-- @Description: 

local This , super = class(..., "Territory.TerrBuild")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local GamePlayer = require("Player.GamePlayer")
local TerrBuildViewWithFunc = require("Territory.TerrBuildViewWithFunc")
local TerrBlackSmithWorker = require("Territory.TerrBlackSmithWorker")
local TerrBlackSmithPolyInfo = require("Territory.TerrBlackSmithPolyInfo")
local TerrBlackSmithMWenPolyInfo = require("Territory.TerrBlackSmithMWenPolyInfo")

local LuckyConfig = require("Metadatas.LuckyConfig")
local LuckyItemConfig = require("Metadatas.LuckyItemConfig")
local DrugPriceConfig = require("Metadatas.DrugPriceConfig")
local ItemUtil = require("Bag.ItemUtil")

local kPolyWorkerNum = 1
function This:init( ... )
    super.init( self, ... )

    self.m_polyinfo = TerrBlackSmithPolyInfo:create()
    self.m_polyinfo:setObject(self)

    self.m_WenPloyInfo = TerrBlackSmithMWenPolyInfo:create()
    self.m_WenPloyInfo:setObject(self)

    self.m_polyWokerList = {}
    self.m_fortuneBuyInfo = {}

    self.m_sendHelpTimes = 0
    self.m_lastSendHelpTime = 0

    self.m_hire_npc_times = 0

    self.m_hire_npc_timesForEquip = 0

    for i=1, kPolyWorkerNum do
        local work = TerrBlackSmithWorker:create({id = i, end_time = -1})
        self.m_polyWokerList[i] = work
    end
    
    local minConfig = LuckyConfig[1]
    self.m_minFortuneClass = minConfig.id
    self.m_minFortuneResId = minConfig.res_id
    local config = LuckyConfig[#LuckyConfig]
    self.m_maxFortune = config.max_num
    self.m_maxFortuneClass = config.id
    self.m_maxFortuneResId = config.res_id
end

function This:createView()
    self:onGetPloyInfoRequest()
    return TerrBuildViewWithFunc:create(self)
end
function This:hasFunctions()
    return true
end

function This:getFuncItemNum()
    return 0
end

function This:getFuncQueueBy()
    return self.m_polyinfo
end

function This:getFuncQueueByEquip()
    return self.m_WenPloyInfo
end

function This:hasEmptyFuncQueueForEquip()
    return self.m_WenPloyInfo:isFree()
end

function This:hasEmptyFuncQueue()
    return self.m_polyinfo:isFree()
end

function This:hasFuncQueue()
    return true
end

function This:getFuncQueueItem()
    local id = self.m_polyinfo:getId()
    local image, count = self.m_polyinfo:getFuncItemImage()
    if image then
        return id, image, count
    end
end

function This:getFuncQueueItemForEquip()
    local id = self.m_WenPloyInfo:getId()
    local image, count = self.m_polyinfo:getFuncItemImage()
    if image then
        return id, image, count
    end
end

function This:hasHelpList()
    return true
end

function This:getHelpListCount()
    if self.m_needHelpList then
        return #self.m_needHelpList
    end
    return 0
end

function This:onHelpFuncionEvent()
    g_scene:openDialog("Territory.TerrBlackSmithHelpPanel", {build = self})
end

function This:getRequest(mode)
    local requestMode = mode or TerrBuildRequestMode.kOther
    if requestMode == TerrBuildRequestMode.kUpgrade then
        return super.getRequest(self, mode)
    else
        if not self.m_clickFunc then
            if GamePlayer:getLevel() < 35 then
                g_scene:showTips(Serialize[64529])
                return
            end
            -- g_scene:openDialog("Territory.TerrBlackSmithPanel", self)
            g_scene:openDialog("Awake.LordEquipMakePanel", self)
        end
        return self.m_clickFunc
    end
end

function This:getHireNpcTime()
     return self.m_hire_npc_times or 0
end

function This:hasPloyWorker()
   if self.m_hire_npc_times > 0 then
        return true
   end
   return false
end

function This:getHireNpcTimeForEquip()
     return self.m_hire_npc_timesForEquip or 0
end

function This:hasPloyWorkerForEquip()
   if self.m_hire_npc_timesForEquip > 0 then
        return true
   end
   return false
end

function This:canPolyAccerlate()
    local ret = self:hasPloyWorker()
    return ret and self.m_polyinfo:canAccerlate()
end

function This:canEquipAccerlate()
    local ret = self:hasPloyWorkerForEquip()
    return ret and self.m_WenPloyInfo:canAccerlate()
end

function This:getWorker(id)
    return self.m_polyWokerList[id]
end

function This:getSentHelpTime()
    return self.m_sendHelpTimes, TerrPolyMaxBegTime
end

function This:getPolyInfo()
    return self.m_polyinfo
end

function This:getMwPolyInfo()
    return self.m_WenPloyInfo
end

function This:isFullFortune()
    return self.m_fortune >= self.m_maxFortune
end

function This:isFullFortuneClass()
    return self:getFortuneClass() > self.m_minFortuneClass
    -- return self:getFortuneClass() >= self.m_maxFortuneClass
end

function This:getFortune()
    return self.m_fortune
end

function This:getFortuneResource()
    if not self.m_fortuneResource then
        self:getFortuneClass()
    end
    return self.m_fortuneResource
end

function This:getFortuneClass()
    local value = self.m_fortune or 0
    if self.m_fortuneRecord ~= value then
        self.m_fortuneClass = self.m_minFortuneClass
        self.m_fortuneResource = self.m_minFortuneResId
        for _, config in ipairs(LuckyConfig) do
            if config.min_num <= value and config.max_num >= value then
                self.m_fortuneClass = config.id
                self.m_fortuneResource = config.res_id
                found = true 
                break
            end
        end

        if not found and value > self.m_maxFortune then
            value = self.m_maxFortune
            self.m_fortuneClass = self.m_maxFortuneClass
            self.m_fortuneResource = self.m_maxFortuneResId
            self.m_fortune = value
        end
        self.m_fortuneRecord = value
    end
    return self.m_fortuneClass
end

function This:getLeftSendCDTime()
    local offset = self.m_lastSendHelpTime + TerrSendHelpCDTime - GamePlayer:getServerTime()
    if offset < 0 then
        return 0
    end
    return offset
end

function This:getReceiveHelpList()
    return self.m_receiveHelpList
end

function This:getNeedHelpList()
    return self.m_needHelpList
end

function This:getFortuneItemInfo(itemId, index)
    local curTime = self.m_fortuneBuyInfo[itemId] or 0
    local config = DrugPriceConfig[curTime + 1]
    if not config then
        local maxNum = LuckyItemConfig[index].buy_num
        if maxNum > curTime then
            config = DrugPriceConfig[#DrugPriceConfig]
        end
    end
    return curTime, config and config.price[index] or 0
end

--=============================================================================--
function This:initListener()
    EventDispatcher:addListener(self, "day_passed", self.onDayPassed)
    EventDispatcher:addListener(self, 10424, self.onPolymerizeReply)
    EventDispatcher:addListener(self, 10425, self.onHirePolymerizationReply)
    EventDispatcher:addListener(self, 10426, self.onPickupResultReply)
    EventDispatcher:addListener(self, 10427, self.onBegAccerlateReply)
    EventDispatcher:addListener(self, 10428, self.onHelpForAccerlationReply)
    EventDispatcher:addListener(self, 10429, self.onGetPloyInfoReply)
    EventDispatcher:addListener(self, 10430, self.onFinishPolyReply)
    EventDispatcher:addListener(self, 10431, self.onPloyAccerlateReply)
    EventDispatcher:addListener(self, 10432, self.onGetHelpInfoReply)
    EventDispatcher:addListener(self, 10433, self.onPolyHelpAllReply)
    EventDispatcher:addListener(self, 10434, self.onUseFortuneItemReply)
    EventDispatcher:addListener(self, 10435, self.onBuyFortuneItemReply)
    EventDispatcher:addListener(self, 10436, self.onBuyHireTimesReply)
    EventDispatcher:addListener(self, 10453, self.onMakeEquipReply)
    EventDispatcher:addListener(self, 10452, self.onCheckMakingEquipReply)
    EventDispatcher:addListener(self, 10454, self.onBuyHireTimesForEquipReply)
    EventDispatcher:addListener(self,10457,self.onCheckHireInfoReply)
    EventDispatcher:addListener(self,10455,self.onPickupMwenReply)
    EventDispatcher:addListener(self,10456,self.onQuickGetMWenReply)


end

function This:removeListener()
    EventDispatcher:removeListener(self,10424)
    EventDispatcher:removeListener(self,10425)
    EventDispatcher:removeListener(self,10428)
    EventDispatcher:removeListener(self,10427)
    EventDispatcher:removeListener(self,10426)
    EventDispatcher:removeListener(self,10429)
    EventDispatcher:removeListener(self,10430)
    EventDispatcher:removeListener(self,10431)
    EventDispatcher:removeListener(self,10432)
    EventDispatcher:removeListener(self,10433)
    EventDispatcher:removeListener(self,10434)
    EventDispatcher:removeListener(self,10435)
    EventDispatcher:removeListener(self,10436)
    EventDispatcher:removeListener(self,10453)
    EventDispatcher:removeListener(self,10452)
    EventDispatcher:removeListener(self,10454)
    EventDispatcher:removeListener(self,10457)    
    EventDispatcher:removeListener(self,10455)    
    EventDispatcher:removeListener(self,10456)    
    EventDispatcher:removeListener(self, "day_passed")
end

function This:onPloyFortuneChangeReply( fortuneValue )
    local oldFortune = self.m_fortune
    self.m_fortune = fortuneValue or 0
    if oldFortune and self.m_fortune ~= oldFortune then
        local delta = self.m_fortune - oldFortune
        if delta > 0 then
            g_scene:showTips(Serialize[64530] .. delta)
        else
            g_scene:showTips(Serialize[64531] .. -delta)
        end
        self:notifyObservers("onPloyFortuneChange", oldFortune, self.m_fortune)
    end
end

-- 技能融合
function This:onPolymerizeRequest(itemIds)
    Socket:sendAndLock(10424, {need_item_list = itemIds})
end

function This:onPolymerizeReply( msg )
    if msg.err_code == 0 then
        self:onPloyFortuneChangeReply(msg.fortune_value)
        self.m_polyinfo:updateInfo(msg.mix_data)
        --self:notifyObservers("onStateChange", nil, self.m_polyinfo:isFree())
        self:notifyObservers("onPolymerize")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 雇佣融合师
function This:onHirePolymerizationRequest(id)
    Socket:sendAndLock(10425, {id= id})
end

function This:onHirePolymerizationReply( msg )
    if msg.err_code == 0 then
        if msg.end_time and msg.end_time > 0 then
            local info = {end_time = msg.end_time, id = 1}
            self.m_polyWokerList[1]:updateInfo(info)
            self:notifyObservers("onHirePolymerization")
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 领取技能融合结果
function This:onPickupResultRequest()
    Socket:sendAndLock(10426, {timestamp = self.m_polyinfo:getId()})
end

--领取打造结果
function This:onPickUpResultRequestForEquip()
    Socket:sendAndLock(10455)
end

function This:onPickupResultReply( msg )
    if msg.err_code == 0 then
        g_scene:showReward({{msg.new_skill_id, msg.reward_count}})
        if msg.item_num > 0 then
             g_scene:showReward({{msg.item_id, msg.item_num}})
        end
        self.m_polyinfo:clear()
        self:notifyObservers("onPolyPickupResult")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 请求加速融合的帮助
function This:onBegAccerlateRequest(mode)
    Socket:sendAndLock(10427, {type= mode})
end

function This:onBegAccerlateReply( msg )
    if msg.err_code == 0 then
        self.m_sendHelpTimes = self.m_sendHelpTimes + 1
        self.m_lastSendHelpTime = GamePlayer:getServerTime()
        self:notifyObservers("onBegAccerlate")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 给与加速融合帮助
function This:onHelpForAccerlationRequest(time, mode, role)
    Socket:sendAndLock(10428, {timestamp = time, type= mode, roleid = role})
end

function This:onHelpForAccerlationReply( msg )
    if msg then
        self:notifyObservers("onPolyHelpOne")
    end
end

function This:onGetPloyInfoRequest()
    -- Socket:sendAndLock(10452, {})
    Socket:sendAndLock(10429, {})
    Socket:send(10432, {})
end

function This:onGetPloyInfoReply( msg )
    if msg then
        if msg.hire_npc_id and msg.hire_npc_id > 0 then
            local info = {end_time = msg.hire_end_time, id = msg.hire_npc_id}
            self.m_polyWokerList[msg.hire_npc_id]:updateInfo(info)
        end
        self.m_sendHelpTimes = msg.send_help_times

        self.m_HelpTimes = msg.send_help_times
        self.m_lastSendHelpTime = msg.last_send_help_time

        self.m_polyinfo:updateInfo(msg.mix_data)
        self.m_hire_npc_times = msg.hire_npc_times
        for _, buyInfo in ipairs(msg.buy_fortune_data) do
            self.m_fortuneBuyInfo[buyInfo.type_id] = buyInfo.buy_times
        end
        self:onPloyFortuneChangeReply(msg.fortune_value)
        self:notifyObservers("onGetPloyInfo")
    else
        g_scene:handleErrorCode("")
    end
end

function This:onFinishPolyRequest()
    Socket:sendAndLock(10430, {timestamp = self.m_polyinfo:getId()})
end

function This:onFinishPolyReply( msg )
    if msg.err_code == 0 then
        self:onPickupResultReply(msg.receive_data)
        self:notifyObservers("onFinishPoly")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onPloyAccerlateRequest()
    Socket:sendAndLock(10431, {timestamp = self.m_polyinfo:getId(), hire_npc_id = 1})
end

function This:onPloyAccerlateReply( msg )
    if msg.err_code == 0 then
        local oldValue = self.m_polyinfo:getLeftTime()
        self.m_polyinfo:updateInfo(msg.mix_data)
        self:notifyObservers("onPloyAccerlate", oldValue)
        self.m_hire_npc_times = msg.hire_npc_times
        self:notifyObservers("onHirePolymerization")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetHelpInfoRequest()
    Socket:sendAndLock(10432)
end

function This:onGetHelpInfoReply( msg )
    if msg then
        self.m_receiveHelpList = msg.receive_help_list
        self.m_needHelpList = msg.need_help_list
        self:notifyObservers("onGetBuildHelpInfo", self:getId())
    end
end

function This:onUseFortuneItemRequest(uid, useNum, itemId)
    Socket:sendAndLock(10434, {id=uid, num=useNum, type_id=itemId})
end

function This:onUseFortuneItemReply( msg )
    if msg.err_code == 0 then
        self:onPloyFortuneChangeReply(msg.fortune_value)
        self:notifyObservers("onUseFortuneItem")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onPolyHelpAllRequest()
    Socket:sendAndLock(10433)
end

function This:onPolyHelpAllReply( msg )
    if msg then
        self.m_needHelpList = nil
        self:notifyObservers("onPolyHelpAll")
    else
        g_scene:handleErrorCode("")
    end
end

function This:onBuyFortuneItemRequest(itemId, useNum)
    Socket:sendAndLock(10435, {type_id= itemId, buy_count= useNum})
end

function This:onBuyFortuneItemReply( msg )
    if msg.err_code == 0 then
        self.m_fortuneBuyInfo[msg.type_id] = msg.buy_times
        self:notifyObservers("onBuyFortuneItem")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end


function This:onToMakeEquip(paramList,scroll_id)
    Socket:sendAndLock(10453 ,{material_list = paramList,scroll_id = scroll_id})
end

function This:onCheckMakingEquip()
    Socket:sendAndLock(10452)
end

function This:onMakeEquipReply(msg)
    if msg.err_code == 0 then
        -- self.m_fortuneBuyInfo[msg.type_id] = msg.buy_times
        self.m_WenPloyInfo:updateInfo(msg.mix_data)
        self:notifyObservers("onMakeEquipItem",msg)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
    -- optional int32 hire_npc_id = 1;             // 雇佣的NPCID
    -- optional int32 hire_end_time = 2;           // 雇佣结束时间
    -- optional int32 hire_npc_times = 3;          // 加速次数
    -- optional alabaster_mix_msg mix_data = 4;    // 正在打造的信息
function This:onCheckMakingEquipReply(msg)
    local info = msg.mix_data
    self.m_hire_npc_timesForEquip = msg.hire_npc_times
    self.m_WenPloyInfo:updateInfo(msg.mix_data)
    self:notifyObservers("onMakingEquipItem",msg)
end
--
function This:onBuyHireTimesRequest(typeID)
    Socket:sendAndLock(10436,{type_id = typeID})
end
--雇佣融合师
function This:onBuyHireTimesRequestForEquip(typeID)
    Socket:sendAndLock(10454,{id = typeID})
end
--雇佣师加速
function This:onCheckHireInfo()
    Socket:sendAndLock(10457)
end
--立即完成
function This:onQuickGetMWen()
    Socket:sendAndLock(10456)
end
--领取結果
function This:onPickUpMwen()
    Socket:sendAndLock(10455)
end

function This:onBuyHireTimesReply( msg )
    if msg.err_code == 0 then
        self.m_hire_npc_times = msg.hire_npc_times
        self:notifyObservers("onHirePolymerization")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end


function This:onBuyHireTimesForEquipReply( msg )
    if msg.err_code == 0 then
        -- self.m_hire_npc_times = msg.hire_npc_times
        self.m_hire_npc_timesForEquip = msg.end_time or msg.hire_npc_times or 0
        self:notifyObservers("onHirePolymerization")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onCheckHireInfoReply(msg)
    local oldValue = self.m_WenPloyInfo:getLeftTime()
    self.m_hire_npc_timesForEquip = msg.hire_npc_times or 0
    self.m_WenPloyInfo:updateInfo(msg.mix_data)
    self:notifyObservers("onCheckHireInfoForEquip", oldValue)
    if msg.mix_data.end_time - GamePlayer:getServerTime() <= 2 then
        self:notifyObservers("onCanFinishToGet")
        self.m_WenPloyInfo:clear()
    else
        self:notifyObservers("onMakeEquipItem",msg)
    end
end

--立即完成
function This:onQuickGetMWenReply( msg )
    if msg.err_code == 0 then
        -- self.m_hire_npc_times = msg.hire_npc_times
        g_scene:showReward({{msg.reward_id, 1}})
        self:notifyObservers("onQuickGetMWenForEquip")
        self.m_WenPloyInfo:endUpdate()
        self.m_WenPloyInfo:clear()
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
--领取打造结果
function This:onPickupMwenReply(msg)
    if msg.err_code == 0 then
        -- self.m_hire_npc_times = msg.hire_npc_times
        g_scene:showReward({{msg.reward_id, 1}})
        self:notifyObservers("onQuickGetMWenForEquip")
    else
        if msg.err_code == 1 then
            return
        end
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onDayPassed()
    self:onGetPloyInfoRequest()
end