-- @Author: su
-- @Date:   2019-02-28 16:14:00
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-11-19 15:30:51
-- @File Name: TerrBuildingXiYuYouShang.lua
-- @Description: 
local This , super = class(...,"Territory.TerrBuild")
local ArgConfig = require("Metadatas.ArgConfig")
local NewActivityManager = require("NewActivity.NewActivityManager")
local GameUtil = require("Util.GameUtil")

local TerrBuildViewWithFunc = require("Territory.TerrBuildViewWithFunc")

function This:createView()
    local view = TerrBuildViewWithFunc:create(self)
    view:runAction(cc.RepeatForever(cc.Sequence( cc.DelayTime(10), cc.CallFunc(function ()
            self:notifyObservers("onGetBuildHelpInfo", self:getId())
        end))))
    return view
end

function This:hasFunctions()
    return false
end

function This:hasHelpList()
    -- self:startUpdate()
    return true, "MenuScene/wordyikaiqi.png"
end


function This:getHelpListCount()
    self:refreshActivity()
    local state = self.m_activity:getState()
    if state == NewActivityState.kNotOpen then
    elseif state == NewActivityState.kNotStart then
    elseif state == NewActivityState.kEnded then
    else
        return 1
    end
    return 0
end

function This:onHelpFuncionEvent()
    local func = self:getRequest()
    if func then
        func()
    end
end


function This:getCondition()
    local conds = self:getConditions()
    if conds then
        return conds[1], conds[2]
    end
    return nil, nil
end

function This:getConditions()
    return {1,self.m_descConfig.open_level}
end

function This:hasProduction()
	return false
end

function This:getName()
    return self.m_descConfig.name
end

function This:getRequest(mode)
    local requestMode = mode or TerrBuildRequestMode.kOther
    if requestMode == TerrBuildRequestMode.kUpgrade then
    	local _, level = self:getCondition()
    	g_scene:showTips(Serialize[64521] .. level .. Serialize[60394]) 
        return
    else
        if not self.m_clickFunc then
            self.m_clickFunc = function()
                self:refreshActivity()
                local state = self.m_activity:getState()
                if state == NewActivityState.kNotOpen then
                    -- print(self.m_activity:getId(), "NotOpen", GameUtil:formatDate(Serialize["y_m_d_H_M_S"], self.m_activity:getStartTime()))
                    g_scene:showTips(Serialize[64541])
                elseif state == NewActivityState.kNotStart then
                    -- print(self.m_activity:getId(), "NotStart", GameUtil:formatDate(Serialize["y_m_d_H_M_S"], self.m_activity:getStartTime()))
                    g_scene:showTips(GameUtil:formatDate(Serialize[64542], self.m_activity:getStartTime()))
                elseif state == NewActivityState.kEnded then
                    -- print(self.m_activity:getId(), "Ended", GameUtil:formatDate(Serialize["y_m_d_H_M_S"], self.m_activity:getEndTime()))
                    g_scene:showTips(GameUtil:formatDate(Serialize[64543], self.m_activity:getEndTime()))
                else
                    self.m_activity:onGetActivityInfoRequest()
                    local panel = g_scene:openDialog("NewActivity.WestShopPanel", {activityId = self.m_activity:getId()})
                    self.m_subView = panel
                    panel:addCloseCallback(function ()
                        self.m_subView = nil
                    end)
                end
            end
        end
        return self.m_clickFunc
    end
end

function This:refreshActivity()
    if self.m_activity and self.m_inited then
        self.m_activity:removeObserver(self)
        self.m_activity = nil
    end

    local idList = {NewActivityType.kWestShop1, NewActivityType.kWestShop2, NewActivityType.kWestShop3}
    local backup, endedAct, lockedAct
    for _, i in ipairs(idList) do
        local act = NewActivityManager:getActivity(i)
        if act and act:isLevelOpen() then
            local state = act:getState() 
            if not lockedAct and state == NewActivityState.kNotOpen then
                lockedAct = act
            elseif not backup and state == NewActivityState.kNotStart then
                backup = act
            elseif not endedAct and state == NewActivityState.kEnded then
                endedAct = act
            elseif state == NewActivityState.kUpdating then
                self.m_activity = act
                break
            end
        end
    end
    
    if not self.m_activity then
        self.m_activity = backup or endedAct or lockedAct
    end

    if self.m_activity and self.m_inited then
        self.m_activity:addObserver(self)
        self.m_activity:updateState()
    end
end