-- @Author: superm
-- @Date:   2018-03-10 17:30:56
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-11-06 16:43:35
-- @File Name: TerrBusinessManView.lua
-- @Description: 

local This , super = class(...,"cc.Node")
local Timer = require("Timer")
local NewActivityManager = require("NewActivity.NewActivityManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local EventDispatcher = require("EventDispatcher")

function This:init()
    self:refreshActivity()
    self.m_soilder = nil
    self:setContentSize(60, 80)
    self.m_timer = Timer:create(self)
    self:scheduleUpdate()
    self.m_inited = false
end

function This:setPositionBy( view )
    local posX, posY = view:getPosition()
    self:setPosition(posX, posY)
end

function This:setClickView( view )
    self.m_clickView = view
end

function This:onEnter()
    super.onEnter(self)
    if self.m_activity then
        self.m_activity:addObserver(self)
        self.m_activity:updateState()
        self:refresh()
    end
    self:initTouchEvent()
    EventDispatcher:addListener(self, "day_passed",self.refreshActivity)
end

function This:onExit()
    EventDispatcher:removeListener(self, "day_passed")
    super.onExit(self)
    if self.m_activity then
        self.m_activity:removeObserver(self)
    end
end

function This:initTouchEvent()
    -- if not self.m_inited then
    --     self:setTouch({
    --         onEnded = function()
    --             local state = self.m_activity:getState()
    --             if state == NewActivityState.kNotOpen then
    --                 -- print(self.m_activity:getId(), "NotOpen", GameUtil:formatDate(Serialize["y_m_d_H_M_S"], self.m_activity:getStartTime()))
    --                 g_scene:showTips(Serialize[64541])
    --             elseif state == NewActivityState.kNotStart then
    --                 -- print(self.m_activity:getId(), "NotStart", GameUtil:formatDate(Serialize["y_m_d_H_M_S"], self.m_activity:getStartTime()))
    --                 g_scene:showTips(GameUtil:formatDate(Serialize[64542], self.m_activity:getStartTime()))
    --             elseif state == NewActivityState.kEnded then
    --                 -- print(self.m_activity:getId(), "Ended", GameUtil:formatDate(Serialize["y_m_d_H_M_S"], self.m_activity:getEndTime()))
    --                 g_scene:showTips(GameUtil:formatDate(Serialize[64543], self.m_activity:getEndTime()))
    --             else
    --                 self.m_activity:onGetActivityInfoRequest()
    --                 local panel = g_scene:openDialog("NewActivity.WestShopPanel", {activityId = self.m_activity:getId()})
    --                 self.m_subView = panel
    --                 panel:addCloseCallback(function ()
    --                     self.m_subView = nil
    --                 end)
    --             end
    --         end,
    --         isDynamic = true,
    --         isSwallow = false,
    --         name = "shangren"
    --     })
    --     self.m_inited = true
    -- end
end

function This:refreshActivity()
    if self.m_activity and self.m_inited then
        self.m_activity:removeObserver(self)
        self.m_activity = nil
    end

    local idList = {NewActivityType.kWestShop1, NewActivityType.kWestShop2, NewActivityType.kWestShop3}
    local backup, endedAct, lockedAct
    for _, i in ipairs(idList) do
        local act = NewActivityManager:getActivity(i)
        if act and act:isLevelOpen() then
            local state = act:getState() 
            if not lockedAct and state == NewActivityState.kNotOpen then
                lockedAct = act
            elseif not backup and state == NewActivityState.kNotStart then
                backup = act
            elseif not endedAct and state == NewActivityState.kEnded then
                endedAct = act
            elseif state == NewActivityState.kUpdating then
                self.m_activity = act
                break
            end
        end
    end
    
    if not self.m_activity then
        self.m_activity = backup or endedAct or lockedAct
    end

    if self.m_activity and self.m_inited then
        self.m_activity:addObserver(self)
        self.m_activity:updateState()
    end
end

function This:refresh(delay)

    local function update() 
        if self.m_soilder then
            self.m_soilder:removeFromParent()
            self.m_soilder = nil
        end
       

        local modelName = "ShangRen"
        local skeletonNode = cc.SkeletonNode:createCharacter(modelName)
        skeletonNode:scheduleUpdate()
        skeletonNode:playAnimation("animation", true)
        skeletonNode:setAnimationListener(self)

        -- self:addChild(skeletonNode)
        self.m_soilder = skeletonNode
        self.m_soilder:setPosition(20, 40)

        -- if not self.m_effect then
        --     local info =  {
        --         name = {
        --             position = {x = 40, y = 10}, 
        --             color = { r =255, g = 255, b =153},
        --             scale={x = 0.6, y = 0.8}, 
        --             font_size = 18,
        --             angle = 1
        --         }
        --     }
        --     local nameInfo = info.name
        --     self.m_nameBgSprite = cc.Sprite:create("Common/tuijianneirongdi.png")
        --     self.m_nameBgSprite:setPosition(nameInfo.position.x, nameInfo.position.y)
        --     self.m_nameBgSprite:setScale(nameInfo.scale.x, nameInfo.scale.y)
        --     self.m_nameBgSprite:setAngle(nameInfo.angle)
        --     self:addChild(self.m_nameBgSprite, 10)

        --     self.m_nameLabel = cc.Label:createNormalTTF(Serialize[64576], nameInfo.font_size)
        --     self.m_nameLabel:setPosition(nameInfo.position.x, nameInfo.position.y)
        --     self.m_nameLabel:setAnchorPoint(0.5, 0.5)
        --     self.m_nameLabel:setColor(nameInfo.color.r, nameInfo.color.g, nameInfo.color.b)
        --     self.m_nameLabel:setAngle(nameInfo.angle)
        --     self:addChild(self.m_nameLabel, 11)
        -- end

        self.m_infoBg = cc.Scale9Sprite:create("MenuScene/kuafu_touming_bg.png", cc.Rect(6,6,6,6))
        self.m_infoBg:setContentSize(320, 40)
        self.m_infoBg:setPosition(190, 60)
        self.m_infoBg:setFlippedX(true)
        self:addChild(self.m_infoBg)

        self.m_infoLabel = cc.Label:createNormalTTF(Serialize[64577], 17)
        self.m_infoLabel:setAnchorPoint(0, 0.5)
        self.m_infoLabel:setPosition(55, 60)
        self:addChild(self.m_infoLabel)
        self.m_infoLabel:setVisible(false)
        self.m_infoBg:setVisible(false)

        self:setContentSize(60, 80)
        if self.m_activity and self.m_activity:isUpdating() then
            self:showEffect()
        end
    end

    if delay then
        self.m_timer:setTimer(delay,function()
            update()
        end, "shangeren")
    else
        update()
    end
end

function This:updateTime(leftTime)
    if leftTime <= 0 then
        self:refresh()
    end
end

function This:update(dt)
    self.m_timer:update(dt)
end

function This:showEffect()
    -- local node, node1 = self.m_infoLabel, self.m_infoBg
    -- if node then
    --     node:setVisible(true)
    --     node:runAction(cc.RepeatForever( cc.Sequence(cc.FadeIn(1), cc.DelayTime(2.25), cc.FadeOut(2.5), cc.DelayTime(5)) ))
    -- end

    -- if node1 then 
    --     node1:setVisible(true)
    --     node1:runAction(cc.RepeatForever( cc.Sequence(cc.FadeIn(1), cc.DelayTime(2.25), cc.FadeOut(2.5), cc.DelayTime(5)) ))
    -- end
end

function This:onActivityEnded( activity )
    if activity and activity:getId() == self.m_activity:getId() then
        local node, node1 = self.m_infoLabel, self.m_infoBg    
        if node then
            node:stopAllActions()
            node:setVisible(false)
        end

        if node1 then
            node1:stopAllActions()
            node1:setVisible(false)
        end
        
        if self.m_subView then
            self.m_subView:close()
        end
    end
end

function This:onActivityStart( ... )
    if activity and activity:getId() == self.m_activity:getId() then
        self:showEffect()
    end 
end