-- @Author: superm
-- @Date:   2018-06-23 15:53:24
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-30 19:16:28
-- @File Name: TerrFortunePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local DepotManager = require("Depot.DepotManager")
local FamilyManager = require("Family.FamilyManager")
local LuckyItemConfig = require("Metadatas.LuckyItemConfig")
local GamePlayer = require("Player.GamePlayer")

function This:initNode(info)
    self.m_build = info.build
    self.isEquipMake = info.isEquipMake
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/BlackSmithLuck.csb")
    self:addChild(self.m_csNode)
    self.m_csNode:setToCenter()

    if self.isEquipMake then
        self.m_csNode.shuoming:setString(Serialize[64580])
    end

    self:refresh()
    self:initItemList()
end

function This:refresh()
    self.m_cdEscape = self.m_build:getLeftSendCDTime()
    self.m_hasCD = self.m_cdEscape > 0
    self.m_nodeEscape = 0

    self:refreshHelpNode()
    self:refreshHelpBtns()
end

function This:initItemList()
    local node = self.m_csNode
    for i, info in ipairs(LuckyItemConfig) do
        local itemId = info.item_id
        local itemName = ItemUtil.getName(itemId)
        local curItem = DepotManager:getItem(itemId)
        local count = curItem and curItem:getCount() or 0
        
        local itemRootNode = node["item" .. i]
        node["itemname" .. i]:setString(itemName)
        node["tishengyunqi" .. i]:setString(ItemUtil.getComment(itemId), 350)
        node["button" .. i]:setTouchEnded(function ()
            if self.m_build:isFullFortune() then
                g_scene:showTips(Serialize[64581])
                return
            end

            local item = DepotManager:getItem(itemId)
            local useNum = info.use_num
            if not item or item:getCount() < useNum then
                local maxTime = info.buy_num
                local curTime, costNum = self.m_build:getFortuneItemInfo(itemId, i)
                if curTime >= maxTime then
                    g_scene:showTips(Serialize[60800])
                    return 
                end

                GamePlayer:costDiamond({
                    -- notips = "BuyItem" .. itemId .. "." .. useNum,
                    cost = costNum * useNum, 
                    desc = string.format(Serialize[64582], itemName, useNum), 
                    note = Serialize[64583] .. (maxTime - curTime), 
                    okHandler = function()
                        self.m_build:onBuyFortuneItemRequest(itemId, useNum)
                end})
                return
            end

            self.m_build:onUseFortuneItemRequest(item:getUid(), useNum, item:getId())
            if itemRootNode and not itemRootNode.effect then
                local effect = GameUtil:createEffect("TiHuan2", false, nil,{
                    onAnimationComplete = function()
                        if itemRootNode.effect then
                            itemRootNode.effect:removeFromParent()
                            itemRootNode.effect = nil
                        end
                    end
                })
                itemRootNode.effect = effect
                itemRootNode:addChild(effect)
            end
        end)

        if not itemRootNode.itemNode then
            local itemNode = ItemNode:create(itemId, count)
            itemNode:setAnchorPoint(0.5, 0.5)
            itemRootNode:setScale(0.9)
            itemRootNode:addChild(itemNode)
            itemRootNode.itemNode = itemNode
        else
            itemRootNode.itemNode:initItem(itemId, count)
        end
        itemRootNode.itemNode:showNeedNum(info.use_num)
    end
end

function This:refreshItemList()
    local node = self.m_csNode
    for i, info in ipairs(LuckyItemConfig) do
        local curItem = DepotManager:getItem(info.item_id)
        local count = curItem and curItem:getCount() or 0
        local itemRootNode = node["item" .. i]
        if itemRootNode.itemNode then
            itemRootNode.itemNode:initItem(info.item_id, count)
            itemRootNode.itemNode:showNeedNum(info.use_num)
        end
    end
end

function This:refreshHelpNode()
    self.m_curTime, self.m_maxTime = self.m_build:getSentHelpTime()
    self.m_csNode.number:setString(string.format("%d/%d", self.m_maxTime - self.m_curTime, self.m_maxTime))
end

function This:refreshHelpBtns()
    if self.m_hasCD then
        self.m_csNode.haoyoupindao:setGrayShader()
        self.m_csNode.juntuanpindao:setGrayShader()
        self.m_csNode.haoyoupindao.hypd:setString(string.format(Serialize[61501],math.ceil(self.m_cdEscape)))
        self.m_csNode.juntuanpindao.jtpd:setString(string.format(Serialize[61501],math.ceil(self.m_cdEscape)))
    else
        self.m_csNode.haoyoupindao.hypd:setString(Serialize[64584])
        self.m_csNode.juntuanpindao.jtpd:setString(Serialize[64585])
        self.m_csNode.haoyoupindao:clearShader()
        self.m_csNode.juntuanpindao:clearShader()
    end
end

function This:updateHelpBtns()
    if self.m_hasCD then
        self.m_csNode.haoyoupindao.hypd:setString(string.format(Serialize[61501],math.ceil(self.m_cdEscape)))
        self.m_csNode.juntuanpindao.jtpd:setString(string.format(Serialize[61501],math.ceil(self.m_cdEscape)))
    else
        self:refreshHelpBtns()
    end
end

function This:initTouchEvent()
    self.m_csNode.haoyoupindao:setTouchEnded(function ()
        if self.m_hasCD then
            g_scene:showTips(self.m_csNode.haoyoupindao.hypd:getString() .. Serialize[64586])
            return
        end

        if self.m_curTime >= self.m_maxTime then
            g_scene:showTips(Serialize[62233])
            return
        end

        g_scene:openDialog("UI.ConfirmPanel", {
            text = Serialize[64587],
            fileName = "MenuScene/NodeUI/BlackSmithHelp.csb",
            okHandler = function ()
                self.m_build:onBegAccerlateRequest(1)
            end
        })
    end)

    self.m_csNode.juntuanpindao:setTouchEnded(function ()
        if self.m_hasCD then
            g_scene:showTips(self.m_csNode.juntuanpindao.jtpd:getString() .. Serialize[64586])
            return
        end

        if not FamilyManager:hasFamily() then
            g_scene:showTips(Serialize[64588])
            return
        end

        if self.m_curTime >= self.m_maxTime then
            g_scene:showTips(Serialize[62233])
            return
        end

        g_scene:openDialog("UI.ConfirmPanel", {
            text = Serialize[64589],
            fileName = "MenuScene/NodeUI/BlackSmithHelp.csb",
            okHandler = function ()
                self.m_build:onBegAccerlateRequest(2)
            end
        })
    end)
    
    self.m_csNode.closebutton:setTouchEnded(function()
        self:close()
    end)
end

function This:onEnter()
    super.onEnter(self)
    if self.m_build then
        self.m_build:addObserver(self)
    end
    DepotManager:addObserver(self)
end

function This:onExit()
    if self.m_build then
        self.m_build:removeObserver(self)
    end
    DepotManager:removeObserver(self)
    super.onExit(self)
end

function This:itemUpdate( itemList )
    self:refreshItemList()
end

function This:onBegAccerlate()
    g_scene:showTips(Serialize[64590])
    self.m_cdEscape = self.m_build:getLeftSendCDTime()
    self.m_hasCD = self.m_cdEscape > 0
    self.m_nodeEscape = 0
    self:refreshHelpNode()
    self.m_csNode.haoyoupindao:setGrayShader()
    self.m_csNode.juntuanpindao:setGrayShader()
end

function This:update(dt)
    super.update(self, dt)
    if self.m_hasCD then
        self.m_nodeEscape = self.m_nodeEscape + dt
        if self.m_nodeEscape >= 1 then
            self.m_nodeEscape = self.m_nodeEscape - 1
            self.m_cdEscape = self.m_cdEscape - 1
            self.m_hasCD = self.m_cdEscape > 0
            self:updateHelpBtns()
        end
    end
end