-- @Author: superm
-- @Date:   2018-06-19 14:08:52
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-07-31 11:53:45
-- @File Name: TerrPatrolHeroView.lua
-- @Description: 

local This , super = class(...,"NodeView")
This.fileName = "MenuScene/NodeUI/Manor3.csb"

local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")
local HeroManager = require("Hero.HeroManager")
local MatchingConfig = require("Metadatas.MatchingConfig")
local TerrPatrolTypeConfig = require("Metadatas.TerrPatrolTypeConfig")

function This:initData(queue, dialog)
    if queue then
        self.m_Id = queue:getId()
        
        self.m_queue = queue
        self.m_dialog = dialog
        self.m_hero = self.m_queue:getHeroId()
        self.m_settings = self.m_queue:getBuilding():getSettings()
        self:addHero(self.m_Id, self.m_hero)
        self.m_patrolType = self.m_queue:getPatrolType() 
        self.m_patrolTimeType = self.m_queue:getPatrolTimeType()
        self.m_inited = false
    end
end

function This:onEnter()
    super.onEnter(self)
    self.m_queue:updateState()
    self.m_queue:addObserver(self)
    self:refresh()
end

function This:onExit()
    self.m_dialog = nil
    self.m_queue:removeObserver(self)
    super.onExit(self)
end

function This:isSelected()
    return self.m_selected
end

function This:setIsSelected(value)
    self.m_selected = value
    if value and self.m_touch then
        self.m_touch:onEnded()
    end
end

function This:hasHero()
    return self.m_hero and self.m_hero > 0
end

function This:getHeroId()
    return self.m_hero
end

function This:isEmpty()
    return self.m_queue:isEmpty() and not self:hasHero()
end

function This:addHero(pos,heroId)
    self.m_settings[pos] = self.m_settings[pos] or {}
    self.m_settings[pos].heroId = heroId
end

function This:removeHero(pos)
    if self.m_settings[pos] then
        self.m_settings[pos].heroId = nil
    end
end

function This:openSelectPanel()
    local haveSelected = {}
    for i , info in pairs(self.m_settings) do
        if info.heroId and info.heroId > 0 then
            haveSelected[info.heroId] = true
        end
    end
    local panel = self.m_dialog:openDialog("Stage.StageSelectHeroPanel",{haveSelected = haveSelected,excludeRedHero = true})
    return panel
end

function This:initTouchEvent()
    if not self.m_inited then
        self.add:setTouch({
            onEnded = function()
                if not self:isEmpty() then
                    return
                end

                if self.m_dialog then
                    if self.m_dialog.m_selectPanel then
                        return
                    end
                    self.m_dialog.m_selectPanel = true
                end

                if not self:isSelected() then
                    self:setIsSelected(true)
                end

                local panel = self:openSelectPanel()
                panel:setSelectHandler(function(heroId)
                	self.m_hero = heroId
                    self:refresh()
                    self:addHero(self.m_Id, heroId)
                    self:onSelected()
                end)
                panel:addCloseCallback(function ()
                    if self.m_dialog then
                        self.m_dialog.m_selectPanel = nil
                    end
                end)
            end,
            isScale = false,
        })

        self.close:setTouchEnded(function()
            if not self.m_hero then
                return
            end

            self.m_hero = nil
            self:removeHero(self.m_Id)
            self:refresh()
            if not self:isSelected() then
                self:setIsSelected(true)
            else
                self:onSelected()
            end
        end)

        self.button:setTouchEnded(function()
            g_scene:openDialog("UI.CostConfirmPanel", {
                cost = self.m_queue:getBuyCost(),
                desc = Serialize[64600], 
                okHandler = function()
                	self.m_queue:getBuilding():onBuyPatrolQueueRequest(self.m_Id)
                end 
            })
            if not self:isSelected() then
                self:setIsSelected(true)
            end
        end)        
        self.m_inited = true
    end
end


function This:refresh()
    self.bg.xunLuo_pos:setString(self.m_queue:getName())
                
    if not self:isEmpty() then
        local showCloseBtn = true
        self.Quality:setImage(ItemUtil.getQualityImage(self.m_hero))
        self.heroIcon:setImage(ItemUtil.getIconImage(self.m_hero))
        self.Quality:setVisible(true)
        self.heroIcon:setVisible(true)
        self.add:setVisible(false)
        self.lock:setVisible(false)
        self.lockbg:setVisible(false)
        self.button:setVisible(false)
        self.tips:setVisible(false)
        if self.m_queue:isEmpty() then
            self.xunLuo_statue:setVisible(false)
        else
            showCloseBtn = false
            self.xunLuo_statue:setVisible(true)
            local isUpdating = self.m_queue:isUpdating()
            self.xunLuo_statue:setString(isUpdating and Serialize[64601] or Serialize[64602])
        end
        self.close:setVisible(showCloseBtn)
    else 
        self.xunLuo_statue:setVisible(false)
        self.Quality:setVisible(false)
        self.heroIcon:setVisible(false)
        self.close:setVisible(false)
        if not self.m_queue:isLocked() then
            self.add:setVisible(true)
            self.lock:setVisible(false)
            self.lockbg:setVisible(false)
            self.button:setVisible(false)
            self.tips:setVisible(false)
        else
            self.add:setVisible(false)
            self.lock:setVisible(true)
            self.lockbg:setVisible(true)
            self.button:setVisible(true)
            self.tips:setVisible(true)
            self.tips:setString(self:getConditionDesc())
        end
    end
end

function This:getConditionDesc()
    local conds = self.m_queue:getCondition()
    local ctype = conds[1]
    if ctype == TerrConditionType.kTerrSelfBuildLev then   
        return Serialize[64603] .. conds[2] .. Serialize[60759]
    elseif ctype == TerrConditionType.kRankLev then
        return MatchingConfig[conds[2]].rank_level .. Serialize[60201]
    else
       	return ""
    end
end

function This:setPatrolType(ptype)
    self.m_patrolType = ptype
    self.m_settings[self.m_Id] = self.m_settings[self.m_Id] or {}
    self.m_settings[self.m_Id].patrolType = ptype
end

function This:setPatrolTimeType(ttype)
    self.m_patrolTimeType = ttype
    self.m_settings[self.m_Id] = self.m_settings[self.m_Id] or {}
    self.m_settings[self.m_Id].timeType = ttype
end

function This:getPatrolType()
    if not self.m_patrolType then
        local setting = self.m_settings[self.m_Id]
        local patrolType = kTerrDefaultPatrolType
        if setting and setting.patrolType then
            patrolType = setting.patrolType
        end
        self.m_patrolType = patrolType
    end
    return self.m_patrolType
end

function This:getPatrolTimeType()
    if not self.m_patrolTimeType then
        local setting = self.m_settings[self.m_Id]
        local timeType = kTerrDefaultPatrolTimeType
        if setting and setting.timeType then
            timeType = setting.timeType
        end
        self.m_patrolTimeType = timeType
    end
    return self.m_patrolTimeType
end

function This:getPatrolPrice()
    local patrolType = self:getPatrolType()
    return TerrPatrolTypeConfig[patrolType].cost  
end

function This:onQueueUpdateStart(idx)
	if idx == self.m_queue:getId() then
    	self.xunLuo_statue:setString(Serialize[64601])
    end
end

function This:onQueueUpdateEnd(idx)
	if idx == self.m_queue:getId() then
    	self.xunLuo_statue:setString(Serialize[64602])
    end
end

function This:updateQueueTime( info )
    if self:isSelected() and self.m_queue:getId() == info.id then
        -- self.m_csNode.GuoCheng.time:setString(GameUtil:formatTime(Serialize["H_M_S"], info.time))
    end
end