-- @Author: superm
-- @Date:   2017-10-21 10:51:06
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-28 20:37:29
-- @File Name: TerrPatrolPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrPatrolQueue =  require("Territory.TerrPatrolQueue")
local VipManager = require("Vip.VipManager")
local TerrPatrolTimeConfig = require("Metadatas.TerrPatrolTimeConfig")
local TerrPatrolHeroView =  require("Territory.TerrPatrolHeroView")
local TerrTabGroupBehaviour =  require("Territory.TerrTabGroupBehaviour")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local ItemNode = require("UI.ItemNode")

local kDefaultSelectIndex = 1
local kQueueTotalNum = 4
local kItemTotalNum = 4

function This:initNode(info)
    self.m_build = TerrBuildManager:getBuildByType(TerrBuildType.kJiangJunFu)
    self.m_build:clearSettings()

    local paramIndex = kDefaultSelectIndex
    local paramPt = kTerrDefaultPatrolTimeType
    if info and info.patrolIndex then
        paramIndex = info.patrolIndex
        paramPt = info.patrolTimeType or paramPt
    end

    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/Manor2.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    self.m_heroViewList = {}
    self.m_groupHero = TerrTabGroupBehaviour:create()
    self.m_groupTime = TerrTabGroupBehaviour:create()
    self.m_selectIdx = paramIndex

    self.m_settings = self.m_build:getSettings()

    for i, v in ipairs(TerrPatrolTimeConfig) do
        self.m_csNode.ZhunBei["time" .. i]:setString(Serialize[64605] .. v.patrol_time .. Serialize[64596])
        local view = self.m_csNode.ZhunBei["hours_bg" .. i]
        self.m_groupTime:addTabButton(tostring(i), view, function()
            local view = self.m_heroViewList[self.m_selectIdx]
            local timeType = view:getPatrolTimeType()
            if VipManager:getMaxMemberLevel() < i - 1 then
                g_scene:showTips(VipName[i-1] .. Serialize[61525])
                view:setPatrolTimeType(kTerrDefaultPatrolTimeType)
                return false
            end

            if timeType ~= i then
                view:setPatrolTimeType(i)
                self:refreshPriceNode(view)
            end
            return true
        end)
    end

    for i = 1, kQueueTotalNum do
        local queue = self:getPatrolQueueByPos(i)
        local view = TerrPatrolHeroView:create(queue, self)
        self.m_heroViewList[i] = view
        self.m_csNode["xunLuo" .. i]:addChild(view)
        self.m_groupHero:addTabButton(tostring(i), view, function()
            if queue:isLocked() then
                return false
            end

            self.m_selectIdx = i
            self:refresh()
            return true
        end, false)
    end
    self.m_groupHero:selectTabButton(tostring(paramIndex))
    self.m_groupTime:selectTabButton(tostring(paramPt))
end

function This:refresh( delay )
    local update = function()
        for i, view in ipairs(self.m_heroViewList) do
            view:refresh()
        end

        self:refreshDescriptionNode()
        self:refreshStateNode(self.m_selectIdx)
    end

    if delay then
        self:setTimer(delay,function()
            update()
        end, "TerrPatrolPanel")
    else
        update()
    end
end

function This:updateQueueTime(info)
    if self.m_selectIdx == info.id then
        self.m_csNode.GuoCheng.time:setString(GameUtil:formatTime(Serialize["H_M_S"], info.time))
    end
end

function This:onQueueEventChanged( queueId )
    if self.m_selectIdx == queueId then
        self:refreshStateNode(self.m_selectIdx)
    end
end

function This:onEnter()
	super.onEnter(self)
    self:refresh()
	self.m_build:addObserver(self)
    HeroManager:addObserver(self)
end

function This:onExit()
    HeroManager:removeObserver(self)
	self.m_build:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
		self:close()
    end)

    self.m_csNode.GuoCheng.button:setTouch({
        onEnded = function()
            local queue = self:getPatrolQueueByPos()
            if queue:isOpening() and not queue:isEmpty() then
                self.m_build:requestReward(queue)
            end
        end,
        isContainer = false,
        isScale = false,
    })

    self.m_csNode.ZhunBei.kaiShi:setTouchEnded(function()
        local view = self.m_heroViewList[self.m_selectIdx]
        if view:isEmpty() then
            g_scene:showTips(Serialize[64606])
            return 
        end

        local timeType = view:getPatrolTimeType()
        if not timeType or timeType == 0 then
            g_scene:showTips(Serialize[64607])
            return
        else
            local config = TerrPatrolTimeConfig[timeType]
            local needVip = config and config.need_vip or 0
            if VipManager:getMaxMemberLevel() < needVip  then
                g_scene:showTips(Serialize[64608] .. VipName[needVip])
                return
            end
        end

        local price = view:getPatrolPrice()
        if GamePlayer:getDiamond() < price then
            GamePlayer:checkDiamond(0, true)
            return
        end

        local queue = self:getPatrolQueueByPos(self.m_selectIdx)
        if queue and not queue:isLocked() then
            self.m_build:requestHeroSelect(queue:getId())
        else
            g_scene:showTips(Serialize[64609])
            return
        end
    end)

    self.m_csNode.ZhunBei.xlfs_Btn:setTouchEnded(function()
        local view = self.m_heroViewList[self.m_selectIdx]
        local callback = function( patrolType, index )
            view:setPatrolType(patrolType)
            self.m_csNode.ZhunBei.xlfs_Text:setString(TerrPatrolType[patrolType].name)
            self:refreshPriceNode(view)
        end

        -- local patrolType = view:getPatrolType() 
        local patrolTimeType = view:getPatrolTimeType()
        self:openDialog("Territory.TerrPatrolSelectModePanel", {
            config = TerrPatrolTimeConfig[patrolTimeType],
            callback = callback,
            queueId = self.m_selectIdx, 
            -- mode = patrolType
        })
    end)
end

function This:refreshStateNode( selIndex )
    if self.m_selectIdx ~= selIndex then
        self.m_selectIdx = selIndex
    end

    local queue = self:getPatrolQueueByPos()
    if not queue:isEmpty() then
        self.m_csNode.ZhunBei:setVisible(false)
        self.m_csNode.GuoCheng:setVisible(true)
        local rewardList = queue:getRewardList()
        for i=1, kItemTotalNum do
            local itemNode = self.m_csNode.GuoCheng["reward" .. i]
            local itemId, num, visible = 0, 0, false
            if rewardList and rewardList[i] then
                itemId = rewardList[i].itemId
                num = rewardList[i].num
                visible = true
                local node = itemNode.node
                if not node then
                    node = ItemNode:create(itemId, num)
                    node:setAnchorPoint(0.5, 0.5)
                    node:setScale(0.8)
                    itemNode:addChild(node)
                    itemNode.node = node
                else
                    node:init(itemId, num)
                end
            end
            itemNode:setVisible(visible)
        end

        if queue:isUpdating() then
            self.m_csNode.GuoCheng.button:setGrayShader()
            self.m_csNode.GuoCheng.text_Btn:setColor(0, 0, 0)
            self.m_csNode.GuoCheng.tips:setString(Serialize[64601])
        else
            self.m_csNode.GuoCheng.time:setString("")
            self.m_csNode.GuoCheng.button:clearShader()
            self.m_csNode.GuoCheng.text_Btn:setColor(255, 255, 255)
            self.m_csNode.GuoCheng.tips:setString(Serialize[64602])
        end

    else
        local view = self.m_heroViewList[self.m_selectIdx]
        local ttype = view:getPatrolTimeType()
        self.m_groupTime:selectTabButton(tostring(ttype))
        local ptype = view:getPatrolType()
        self.m_csNode.ZhunBei.xlfs_Text:setString(TerrPatrolType[ptype].name)
        self:refreshPriceNode(view)

        local price = view:getPatrolPrice()
        if not view:hasHero() then
            for i=1, kItemTotalNum do
                local itemNode = self.m_csNode.ZhunBei["item" .. i]
                itemNode:setVisible(false)
            end

            self.m_csNode.ZhunBei.bd_title:setVisible(false)
        else
            local heroId = view:getHeroId()
            local rewardList = queue:getConfigRewardList(heroId)
            for i=1, kItemTotalNum do
                local itemNode = self.m_csNode.ZhunBei["item" .. i]
                local node = itemNode.node
                local itemId = rewardList[i]
                if not node then
                    node = ItemNode:create(itemId)
                    node:setAnchorPoint(0.5, 0.5)
                    itemNode:addChild(node)
                    itemNode.node = node
                else
                    node:init(itemId)
                end
                itemNode:setVisible(true)
            end

            self.m_csNode.ZhunBei.bd_title:setVisible(true)
        end
        self.m_csNode.ZhunBei:setVisible(true)
        self.m_csNode.GuoCheng:setVisible(false)
    end
end

function This:refreshPriceNode(view)
    if view and view:hasHero() then
        local price = view:getPatrolPrice()
        self.m_csNode.ZhunBei.price:setString(price)
        self.m_csNode.ZhunBei.icon:setVisible(price > 0)
        self.m_csNode.ZhunBei.price:setVisible(price > 0)
        self.m_csNode.ZhunBei.free:setVisible(price <= 0)
    else
        self.m_csNode.ZhunBei.icon:setVisible(false)
        self.m_csNode.ZhunBei.price:setVisible(false)
        self.m_csNode.ZhunBei.free:setVisible(false)
    end
end

function This:refreshHeorItemBy(index)
    local view = self.m_heroViewList[index]
    if view then
        view:refresh()        
    end
end

function This:getPatrolQueueByPos(index)
    if not index then
        index = self.m_selectIdx
    end
    return self.m_build:getPatrolQueueBy(index)
end

function This:refreshDescriptionNode()
    self.m_csNode.GuoCheng.bg1.scrollView:removeAllItems()
    local curEventList = {}
    for index=1, kQueueTotalNum do
        local queue = self:getPatrolQueueByPos(index)
        local eventList = queue:getEventList()
        if eventList then
            for i, pEvent in ipairs(eventList) do
                table.insert(curEventList, pEvent)
            end
        end
    end

    table.sort( curEventList, function(a, b)
        return a.time > b.time
    end)

    for i, pEvent in ipairs(curEventList) do
        local label = cc.Label:createNormalTTF("", 16)
        label:setString(pEvent.event, 410)
        local _, h = label:getContentSize()
        self.m_csNode.GuoCheng.bg1.scrollView:pushNode(label, 410, h + 10)
        if i > 20 then break end
    end
end

function This:onPatrolHeroSelect(index)
    self:refreshStateNode(index)
end

function This:onQueueUpdateStart(idx)
    self:refresh()
end

function This:onQueueUpdateEnd(idx)
    self:setTimer(0.3, function()
        local queue = self:getPatrolQueueByPos(idx)
        if queue then
            queue:updateState()
        end

        if self.m_selectIdx == idx then
            self.m_csNode.GuoCheng.time:setString("")
            self.m_csNode.GuoCheng.button:clearShader()
            self.m_csNode.GuoCheng.text_Btn:setColor(255, 255, 255)
            self.m_csNode.GuoCheng.tips:setString(Serialize[64602])
        end
    end, "onQueueUpdateEnd" .. idx)
end

function This:onGetPatrolReward(idx)
    local queue = self:getPatrolQueueByPos()
    if queue then
        self:refresh()
    end
end

function This:onBuyPatrolQueue( idx )
    g_scene:showTips(Serialize[64610])
    self:refreshHeorItemBy(idx)
end

--endregion