--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Territory.TerrQueue")
local GamePlayer = require("Player.GamePlayer")
local TerrPatrolConfig = require("Metadatas.TerrPatrolConfig")
-- local TerritoryManager = require("Territory.TerritoryManager")
local TerrBuildManager = require("Territory.TerrBuildManager")
local MatchManager = require("Match.MatchManager")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")

This.defalutState = TerrObjectState.kNormal

function This:create(...)
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret
end

function This:update( dt )
    super.update(self, dt)
    if self.m_eventUnReadIndexList and #self.m_eventUnReadIndexList > 0 then
        self.m_eventUnReadIndexList = {}
        self:notifyObservers("onQueueEventChanged", self:getId())
    end
end

function This:isNeedBuyValid()
    return self.m_hero ~= nil
end

function This:needBuy()
    local curTime = GamePlayer:getServerTime()
    return curTime < self.m_upEndTime
end

function This:isVisibleQueue()
    return true
end

function This:updateBaseData( info )
    self.m_index = info.index or 0
    if info.start_time then
        self.m_upStartTime = info.start_time
    end

    if info.end_time then
        self.m_upEndTime = info.end_time
    end
end

function This:initBeforeUpdateState(info, createHandler)
    self.m_object = nil
    self.m_buyTimes = info.buy_time or 0
    self.m_cardId = info.card_id or 0
    self.m_lastRewardTime = info.last_reward_time or 0
    self.m_patrolType = info.reward_level 
    if info.type and info.type < 3 and info.type > 0 then
        self.m_patrolTimeType = info.type
    end
    self.m_itemList = {}
    self.m_eventList = self.m_eventList or {}

    self.m_eventIndexList = self.m_eventIndexList or {}
    self.m_eventUnReadIndexList = self.m_eventUnReadIndexList or {}

    if info.item_id then
        for idx, id in ipairs(info.item_id) do
            table.insert(self.m_itemList, {itemId = id, num = info.item_num[idx]})
        end
    end

    if info.info then
        for idx, patrolEvent in ipairs(info.info) do
            local infoTime = info.info_time[idx]
            local pass = GamePlayer:getServerTime() >= infoTime
            if not self.m_eventIndexList[infoTime] then
                self.m_eventIndexList[infoTime] = #self.m_eventList + 1
                table.insert(self.m_eventUnReadIndexList, infoTime)
                table.insert(self.m_eventList, {event = patrolEvent, time = infoTime, isHistory = pass})
            end
        end
    end

    if self.m_cardId > 0 then
        self.m_hero, self.m_object = createHandler( self.m_cardId )
    else
        self.m_hero = nil
    end

    if not self.m_config then
        for index, cfg in ipairs(TerrPatrolConfig) do
            if index == self.m_index then
                self.m_config = cfg
            end
        end
    end
end

function This:getBuilding()
    return TerrBuildManager:getBuildByType(TerrBuildType.kJiangJunFu)
end

function This:updateObjects( isInit, oldState )
    if self.m_state ~= oldState then
        local view = self:getView()
        if view then
            view:onStateChange(self:getId(), self:isEmpty(), oldState)
        end
    end
end

function This:selfUpdateInfo(info, createHandler)
    self:initBeforeUpdateState(info, createHandler)
end

function This:isNotLockedBy( conds )
    local switch = {
        [TerrConditionType.kNone] = function()
            return true
        end,
        [TerrConditionType.kTerrSelfBuildLev] = function(arg)   
            return arg <= TerrBuildManager:getBuildLevel(TerrBuildType.kJiangJunFu)
        end,
        [TerrConditionType.kRankLev] = function(arg)   
            return arg <= MatchManager:getMaxRankLvInHistory()
        end,
    }

    return switch[conds[1]](conds[2])
end

function This:setPatrolType( type )
    self.m_patrolType = type
end

function This:getPatrolType()
    return self.m_patrolType 
end

function This:setPatrolTimeType( type )
    self.m_patrolTimeType = type
end

function This:getPatrolTimeType()
    return self.m_patrolTimeType 
end

function This:setHero( hero )
    self.m_hero = hero
end

function This:getHero()
    return self.m_hero
end

function This:getHeroId()
    return self.m_hero and self.m_hero:getId() or 0
end

function This:clearInfo()
    if self.m_object then
        self.m_object:notifyObservers("onQueueClearInfo", self:getId())
        self.m_object = nil
        if self.m_itemList then
            local itemList = {}
            for _, info in ipairs(self.m_itemList) do
                table.insert(itemList, {id = info.itemId , num = info.num })
            end
            g_scene:showReward(itemList)
            self.m_itemList = nil
        end

        self.m_patrolTimeType = nil
        self.m_patrolType = nil
        self.m_cardId = 0
        self.m_hero = nil
        self:updateState()
    end
end

function This:getFuncItemImage()
    return ItemUtil.getIconImage(self:getHeroId())
end

function This:getFuncItemNum()
    if not self:isEmpty() and self:isOpening() then
        local list = self:getRewardList()
        if not list then return 0 end

        local _, ret = next(list)
        if not ret then return 0 end
        return ret.num
    else
        return 0
    end
end

function This:onFunctionEvent(build, index)
    if not self.m_upIsUpdating and not self:isEmpty() and self:getLeftTime() <= 0 then
        if build then
            build:onGetPatrolRewardRequest(self:getId())
        end
    else
        g_scene:showTips(Serialize[64612])
        g_scene:openDialog("Territory.TerrPatrolPanel", {patrolIndex = self:getId(), patrolTimeType = self:getPatrolTimeType()})

    end
end

function This:increaseBuyTime()
    self.m_buyTimes = (self.m_buyTimes or 0) + 1
end

function This:isEmpty()
    return self.m_hero == nil
end

function This:getRewardList()
    return self.m_itemList
end

function This:getEventList()
    return self.m_eventList
end

--获取配置信息region begin

function This:getCondition()
    if not self.m_buyTimes or self.m_buyTimes <= 0 then
        return super.getCondition(self)
    end
end

function This:getBuyCost()
    return self.m_config.cost
end

function This:getName()
    return self.m_config.name
end

function This:getConfigRewardList(heroId)
    local ret = {[1] = heroId + 10000}
    return GameUtil:shallowChange(self.m_config.prop_type, ret)
end

--获取配置信息region end



--endregion