--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")

This.defalutState = TerrObjectState.kLocked

function This:create(...)
	local ret = {}
	setmetatable(ret,self)
	ret:init(...)
	return ret
end

function This:init( info, createHandler )
	super.init(self)
    self:initBaseData( info )
    self.m_view = nil
    self.m_config = info.config
    self.m_state = info.state or TerrObjectState.kLocked
	self.m_escape = 0
    self.m_upIsUpdating = false
	self.m_hasTimeUpdate = false
	self.m_object = nil
    self.m_objects = {}
	self:updateState(true, info, createHandler)
end

function This:initBaseData( info )

    self.m_index = info.index or 0
    self.m_upStartTime = info.start_time or 0
    self.m_upEndTime = info.end_time or 0
end

function This:updateBaseData( info )

    self.m_index = info.index or 0
    self.m_upStartTime = info.start_time or 0
    self:setEndTime(info.end_time or 0)
end

function This:getStartTime()
    return self.m_upStartTime
end

function This:getEndTime()
    return self.m_upEndTime
end

function This:updateObjects(isInit, oldState)
end

function This:updateState(isInit, info, createHandler)
    if isInit then
        self:initBeforeUpdateState(info, createHandler)
    end
    local oldState = self.m_state
    if self:isNeedBuyValid() and not self:needBuy() then
        self.m_state = TerrObjectState.kNormal
    else
        local curTime = GamePlayer:getServerTime()
        if curTime < self.m_upEndTime then
            self.m_state = TerrObjectState.kUpdating
            if not self.m_hasTimeUpdate then
                self:startUpdate()
            end
        elseif curTime >= self.m_upEndTime then
            if self.m_hasTimeUpdate then
                self:endUpdate()
            end
            local conds = self:getCondition()
            if not conds or conds == 0 then
                self.m_state = self.defalutState
            else
                local ret = self:isNotLockedBy(conds)
                self.m_state = ret and TerrObjectState.kNormal or TerrObjectState.kLocked
            end
        end
    end
    self:updateObjects(isInit, oldState)
end

function This:initBeforeUpdateState(info, createHandler)
end


function This:updateInfo(info, createHandler)
    self:updateBaseData( info )
    self:selfUpdateInfo(info, createHandler)
    self:updateState()
end

function This:selfUpdateInfo(info, createHandler)
end

function This:isLocked()
    return self.m_state == TerrObjectState.kLocked
end

function This:isOpening()
    return self.m_state == TerrObjectState.kNormal
end

function This:isUpdating()
    return self.m_state == TerrObjectState.kUpdating
end

function This:getState()
    return self.m_state
end

function This:isEmpty()
    return (self.m_object == nil) and (self.m_objects == nil or table.count(self.m_objects) <= 0)
end

function This:isNotLockedBy(condition)
    return true
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	if curTime >= self.m_upEndTime then
		return 0
	end
	return self.m_upEndTime - curTime
end

function This:setEndTime( endTime )
	self.m_upEndTime = endTime
	local curTime = GamePlayer:getServerTime()
	if curTime < self.m_upEndTime then
        self.m_state = TerrObjectState.kUpdating
        if not self.m_hasTimeUpdate then
		    self:startUpdate()
        end
    else
        if self.m_hasTimeUpdate then
            self:endUpdate()
        end
	end
	self:notifyObservers("onQueueInfoChanged", self:getId())
    if self:isVisibleQueue() and self.m_object then
        self.m_object:notifyObservers("onQueueInfoChanged", self:getId())
    end
end

function This:startUpdate()
    if not self.m_upIsUpdating then
        cc.Application:addToUpdateList(self)
        self:notifyObservers("onQueueUpdateStart", self:getId())
        if self:isVisibleQueue() and self.m_object then
            self.m_object:notifyObservers("onQueueUpdateStart", self:getId())
        end
        self.m_upIsUpdating = true
    end
	self.m_hasTimeUpdate = true
end

function This:endUpdate()
    if self.m_upIsUpdating then
        cc.Application:removeFromUpdateList(self)
        if self.m_upEndTime > 0 then
            self:notifyObservers("onQueueUpdateEnd", self:getId())
            if self:isVisibleQueue() and self.m_object then
                self.m_object:notifyObservers("onQueueUpdateEnd", self:getId())
            end
        end
        self.m_upIsUpdating = false
    end
	self.m_hasTimeUpdate = false
end

function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			self:notifyObservers("updateQueueTime", {time = leftTime, id = self:getId()})
            if self:isVisibleQueue() and self.m_object then
                self.m_object:notifyObservers("updateQueueTime", {time = leftTime, id = self:getId()})
            end
		else
            self:notifyObservers("updateQueueTime", {time = 0, id = self:getId()})
            if self:isVisibleQueue() and self.m_object then
                self.m_object:notifyObservers("updateQueueTime", {time = 0, id = self:getId()})
            end
			self:endUpdate()
		end
	end
end

function This:setView(view)
    self.m_view = view
end

function This:getView()
    return self.m_view
end

function This:getId()
	return self.m_index
end

function This:isNeedBuyValid()
    return false
end

function This:needBuy()
	return true
end

function This:isVisibleQueue()
    return false
end

function This:getAllObjects()
    return self.m_objects
end

function This:getObjectIdListBy(handler)
    local ret = {}
    for id, obj in pairs(self.m_objects) do
        if handler and handler(obj) then
            table.insert(ret, id)
        end
    end
    return ret 
end

function This:getObjectBy(handler)
    for id, obj in pairs(self.m_objects) do
    	if handler and handler(obj) then
    		return obj
    	end
    end
    return nil
end

function This:removeObjectBy(handler)
    local resultId = nil
    for id, obj in pairs(self.m_objects) do
        if handler and handler(obj) then
            resultId = id
        end
    end
    
    if resultId then
        self.m_objects[resultId] = nil
    end
end

function This:removeAllObjects()
    self.m_objects = {}
    self.m_object = nil
end

function This:refreshAllObjectsBy( predicateHandler, refreshHandler)
    local ret = false
    for id, obj in pairs(self.m_objects) do
        if predicateHandler(obj) then
            ret = true
            refreshHandler(obj)
        end
    end 
    return ret
end

function This:getCondition()
    return self.m_config.upcondition
end



--endregion