local This , super = class(...,"cc.Node")
local Timer = require("Timer")
local TerritoryManager = require("Territory.TerritoryManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")

function This:init(robber)
    self.m_robber = robber
    self.m_soilders = {}
    self:setContentSize(self.m_robber:getContentSize())
    self.m_timer = Timer:create(self)
    self:scheduleUpdate()
    self.m_inited = false
end

function This:setPositionBy( view )
    local posX, posY = view:getPosition()
    self:setPosition(posX, posY)
end

function This:setClickView( view )
    self.m_clickView = view
end

function This:adjustZOrder( isToppest )
    self:setLocalZOrder(self.m_robber:getId() + 20)         
end

function This:setSelectHandler( handler )
    self.m_selectHandler = handler
end

function This:setSelect( isSelected )

    self:adjustZOrder(isSelected)
end

function This:onEnter()
    super.onEnter(self)
    -- self:adjustZOrder()
    self.m_robber:setView(self)
    self.m_robber:addObserver(self)
    self.m_robber:updateState()

    TerritoryManager:addObserver(self)
    self:initTouchEvent()
    self:refresh()
end

function This:onExit()
    super.onExit(self)
    -- self:setClickView(nil)
    self.m_robber:setView(nil)
    self.m_robber:removeObserver(self)
    TerritoryManager:removeObserver(self)
end


function This:initTouchEvent()
    if not self.m_inited then
        self:setTouch({
            onEnded = function()
                local copy = TerritoryManager:setRobberCopy(self.m_robber)
                g_scene:openDialog("Territory.TerrRobberPanel", copy)
            end,
            isDynamic = true,
            isSwallow = false,
			name = "robber"
        })
        self.m_inited = true
    end

end

function This:refresh(delay)

    local function update() 
        local soilderList= self.m_soilders
        self.m_soilders = {}
        for i, soilder in ipairs(soilderList) do
            soilder:removeFromParent()
        end

        local modelName = "HuangJinJun"
        local count = self.m_robber:getModelNum()
        local soilderConfig = self.m_robber:getOffsetPostion()
        for i = 1 , count do
            local skeletonNode = cc.SkeletonNode:createCharacter(modelName)
            skeletonNode:scheduleUpdate()
            skeletonNode:playAnimation("animation", true)
            if i == count then
                skeletonNode:setAnimationListener(self)
            end

            self:addChild(skeletonNode)
            skeletonNode:setScale(TerrRobberSize)
            self.m_soilders[i] = skeletonNode

            skeletonNode:setLocalZOrder(i)
            skeletonNode:setPosition(soilderConfig[i].x, soilderConfig[i].y)
        end

        if not self.m_effect then
            local info = self.m_robber:getEffectOffset()
            local effInfo = info.eff
            self.m_effect = GameUtil:createEffect("HongQuan",true,self)
            self.m_effect:setPosition(effInfo.position.x, effInfo.position.y)
            self.m_effect:setScale(effInfo.scale.x, effInfo.scale.y)
            self.m_effect:setAngle(effInfo.angle)
            self:addChild(self.m_effect, -1)

            local nameInfo = info.name
            self.m_nameBgSprite = cc.Sprite:create("Common/tuijianneirongdi.png")
            self.m_nameBgSprite:setPosition(nameInfo.position.x, nameInfo.position.y)
            self.m_nameBgSprite:setScale(nameInfo.scale.x, nameInfo.scale.y)
            self.m_nameBgSprite:setAngle(nameInfo.angle)
            self:addChild(self.m_nameBgSprite, 10)

            self.m_nameLabel = cc.Label:createNormalTTF(self.m_robber:getName(), nameInfo.font_size)
            self.m_nameLabel:setPosition(nameInfo.position.x, nameInfo.position.y)
            self.m_nameLabel:setAnchorPoint(0.5, 0.5)
            self.m_nameLabel:setColor(nameInfo.color.r, nameInfo.color.g, nameInfo.color.b)
            self.m_nameLabel:setAngle(nameInfo.angle)
            self:addChild(self.m_nameLabel, 11)
        end

        self:setContentSize(self.m_robber:getContentSize())
    end

    if delay then
        self.m_timer:setTimer(delay,function()
            update()
        end, "robber" .. tostring(self.m_robber:getId()))
    else
        update()
    end
end

function This:updateTime(leftTime)
    if leftTime <= 0 then
        self:refresh()
    end
end

function This:update(dt)
    self.m_timer:update(dt)
end