-- @Author: superm
-- @Date:   2018-09-05 15:19:21
-- @Last Modified by:   superm
-- @Last Modified time: 2019-02-27 18:03:47
-- @File Name: TerrRollView.lua
-- @Description: 

local This , super = class(..., "cc.Node")
local ArenaManager = require("Arena.ArenaManager")
local GameUtil = require("Util.GameUtil")

function This:create(snode)
    if not snode then
        return 
    end

    local snodeId = snode[0]
    local ret = cc.registerObject(snodeId, self.getAutoChildClass(snodeId, self._MNAME), {[0]=snodeId})
    if ret:getName() == "" then
        ret:setName(self._NAME)
    end

    if ret.init then
        ret:init()
    end
    return ret
end

function This:init()
end

function This:onEnter()
    super.onEnter(self)
    self:initTouchEvent()
	ArenaManager:addObserver(self)
end

function This:onExit()
	ArenaManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    if self.help then
        self.help:setTouch({
            onEnded = function()
                if ArenaManager:isArenaOpened(true) then
                    g_scene:openDialog("Arena.ArenaPanel")
                end
            end,
            isDynamic = true,
            isScale = false,
            isSwallow = false,
        })
    end
end

function This:refresh()
	local player = ArenaManager:getFirstPlayer()
	if player then
		self.name:setString(player:getName())
		self.name:setVisible(true)
		self.Image_1:setVisible(true)
	else
		self.name:setVisible(false)
		self.Image_1:setVisible(false)
	end

   self:refreshRollEffect()
end

function This:refreshRollEffect()
    if not self.help then
        return
    end
    
    local visible = ArenaManager:isArenaOpened() and ArenaManager:getLeftChallengeTimes() > 0
    self.help:setVisible(visible)
    if visible then
        self:loadProductEffect()
    else
        self:unloadProductEffect()
    end
end
This.onRefreshArenaInfo = This.refreshRollEffect
This.onBuyChallengeTimes = This.refreshRollEffect
This.onGetArenaPlayerInfo = This.refresh

function This:loadProductEffect()
    if not self.m_effectProduct then
        self.m_effectProduct = GameUtil:createEffect("Build1", true, self.help)
        local posx, posy = 43, 56.8
        self.m_effectProduct:setPosition(posx, posy)
        self.m_effectProduct:setScale(1.15)
        self.help:addChild(self.m_effectProduct)
        self.help:runActionByID("Roll")  
    end
end

function This:unloadProductEffect()
    if self.m_effectProduct then
        self:stopAllActions()
        self.m_effectProduct:removeFromParent()
        self.m_effectProduct = nil
    end
end