-- @Author: superm
-- @Date:   2018-06-21 12:00:04
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-09 17:08:54
-- @File Name: TerrTabGroupBehaviour.lua
-- @Description: 

local This = class(...)

function This:create(...)
	local o = {}
	setmetatable(o, self)
	o:init(...)
	return o
end

function This:init(info)
    self.m_visible = true
    if info then
        self.m_innerHeight = info.inner_height
        local tabsInfo = info.tabs
        if tabsInfo and tabsInfo.nodes then
            self.m_buttonHeight = tabsInfo.height
            local tabNodeInfo = tabsInfo.nodes
            local isScale, isDynamic = tabsInfo.isScale, tabsInfo.isDynamic

            local nameList = tabsInfo.names
            local nameCount = tabsInfo.count
            local callback = tabsInfo.func
            local rootNode = tabsInfo.pnode
            local defaultTabName = tabNodeInfo.default
            local useName = tabNodeInfo.use_name
            local usePrefix = tabNodeInfo.use_regular_prefix
            local pindice = tabNodeInfo.pindice
            local useNodeList = tabNodeInfo[1]
            local tabNodeList
            if useNodeList then
                tabNodeList = {}
                for _, node in ipairs(useNodeList) do
                    table.insert(tabNodeList, node)
                end
            elseif usePrefix then
                tabNodeList = {}
                local useIndice = tabNodeInfo.use_indice
                local useRootIndice = tabNodeInfo.use_pindice
                local nameListCount = #nameList
                for indice=1, nameListCount do
                    local key = usePrefix
                    if useRootIndice then
                        key = key .. pindice
                    end
                    if useIndice then
                        key = key .. indice
                    end
                    local tabNode = rootNode[key] 
                    if tabNode and indice <= nameCount then
                        table.insert(tabNodeList, tabNode)
                    elseif tabNode then
                        tabNode:setVisible(false)
                    end
                end
            end

            for i=1, nameCount do
                local name = nameList[i]
                local subtabInfo, useSubIndice, defaultSubIndice, subtabNodeInfo
                if tabNodeInfo.subtabs then
                    subtabInfo = tabNodeInfo.subtabs[i]
                    if subtabInfo and subtabInfo.tabs and subtabInfo.tabs.nodes then
                        subtabNodeInfo = subtabInfo.tabs.nodes
                        defaultSubIndice = subtabNodeInfo.default
                        useSubIndice = subtabNodeInfo.use_pindice
                    end
                end

                local key = useName and name or i
                if subtabInfo then
                    if tabNodeList[i] then
                        tabNodeList[i]:setVisible(true)
                    end
                    self:addTabButton(key, tabNodeList[i], function(subGroup)
                        if subGroup then
                            subGroup:selectTabButton(defaultSubIndice, true)
                        end
                        if callback then
                            return callback(i, name, pindice)
                        end
                        return true
                    end, isScale, isDynamic)
                    if useSubIndice then
                        subtabNodeInfo.pindice = i
                    end
                    self:createSubGroup(key, subtabInfo)
                else
                    if tabNodeList[i] then
                        tabNodeList[i]:setVisible(true)
                    end
                    self:addTabButton(key, tabNodeList[i], function()
                        if callback then
                            return callback(i, name, pindice)
                        end
                        return true
                    end, isScale, isDynamic)        
                end
            end

            if tabNodeInfo.select_default then
                self:selectTabButton(tabNodeInfo.default)
            end
        end
    end
end

function This:addTabButton(name, button, onSelected, isScale, isDynamic)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end

    if self.m_tabButtons[name] then
        return
    end

    self.m_tabButtons[name] = button
    if not button.m_touch then
        button:setTouch({
            isScale = isScale,
        	isDynamic = isDynamic,
            onEnded = function ()
                self:selectTabButton(name)
            end
        })
    end
    button.onSelected = onSelected
end

function This:getCurSelectTab()
    return self.m_selectTabButton
end

function This:selectTabButton(name, force)
    if self.m_tabButtons then
        if not force and self.m_selectTabButton == name then
            return
        end
        local subButton = self:getSubGroup(name)
        if self.m_tabButtons[name].onSelected(subButton) ~= false then
            self:lightTabButton(name)
        end
        self:layout()
    end
end

function This:lightTabButton(name)
    self.m_selectTabButton = name
    if self.m_tabButtons then
        for k, node in pairs(self.m_tabButtons) do
            local visible = k == name
            node.m_selected = visible
            if node.light then
                node.light:setVisible(visible)
            end
            if node.normal then
                node.normal:setVisible(not visible)
            end
        end
    end
    if self.m_tabSubButtons then
        for k, node in pairs(self.m_tabSubButtons) do
            local visible = k == name
            node:setVisible(visible)
        end
    end
end

function This:layout(height)
    if not height then
        if not self.m_innerHeight then
            return
        end
        height = self.m_innerHeight
    end

    local y = height
    for index, node in ipairs(self.m_tabButtons) do
        local itemHeight = self.m_buttonHeight
        y = y - itemHeight
        node:setPositionY(y)
        print(index, y, tostring(node.m_selected), itemHeight)
        if node.m_selected and self.m_tabSubButtons then
            local subGroup = self.m_tabSubButtons[index]
            if subGroup then
                y = subGroup:layout(y - 12)
            end
        end
    end
    return y
end

function This:createSubGroup(name, info)
    if not self.m_tabButtons[name] then
        return
    end

    if not self.m_tabSubButtons then
        self.m_tabSubButtons = {}
    end

    if self.m_tabSubButtons[name] then
        return
    end
    local group = This:create(info)
    self.m_tabSubButtons[name] = group
    return group
end

function This:getSubGroup(name)
    if not self.m_tabButtons[name] or not self.m_tabSubButtons or not self.m_tabSubButtons[name] then
        return
    end
    return self.m_tabSubButtons[name]
end

function This:setVisible(value)
    if self.m_tabButtons then
        self.m_visible = value
        for _, node in pairs(self.m_tabButtons) do
            node:setVisible(value)
        end
    end
end