--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Territory.TerrUpgradeModel")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrTechnManager = require("Territory.TerrTechnManager")
local TerrTechnQueueConfig = require("Metadatas.TerrTechnQueueConfig")
local TerrTechnsConfig = require("Metadatas.TerrTechnsConfig")
local GamePlayer = require("Player.GamePlayer")

local kTechnIdPrefix = 50000000

This.s_layerInstanceMap={}
This.s_layerCountMap={}
This.s_layerNumMap={}

function This.createBaseTechns( container )
    for technId, cfg in pairs(TerrTechnsConfig) do
        local techn = nil
        if not TerrTechnManager:getTechnBy(technId) then
            local info = { id = technId, descConfig = cfg }
            techn = This:create()
            techn:initByConfig(info)
            container[technId] = techn
        end
        -----============================================------
        local technType = cfg.type
        local layerIndex = cfg.layer
        if not This.s_layerInstanceMap[technType] then
            This.s_layerNumMap[technType] = {}
            This.s_layerInstanceMap[technType] = {}
        end
        if not This.s_layerInstanceMap[technType][layerIndex] then
            This.s_layerInstanceMap[technType][layerIndex] = {}
        end
        table.insert(This.s_layerInstanceMap[technType][layerIndex], techn)
        This.s_layerNumMap[technType][layerIndex] = ( This.s_layerNumMap[technType][layerIndex] or 0 ) + 1

        local maxLayerIndex = This.s_layerCountMap[technType] or 0
        if maxLayerIndex < layerIndex then
            This.s_layerCountMap[technType] = layerIndex
        end
    end
end

function This.getLineNumBy( paramType )
    return This.s_layerCountMap[paramType] or 0
end

function This.getNumByLayerIdx( paramType, paramIndex )
    return This.s_layerNumMap[paramType][paramIndex] or 0
end

function This.getTechnByFilter( paramType, paramIndex )
    return This.s_layerInstanceMap[paramType][paramIndex]
end

function This.getTotalLevelsBy( paramType, paramIndex )
    local ret = 0
    local techns = This.getTechnByFilter( paramType, paramIndex )
    for index, techn in ipairs(techns) do
        ret = ret + techn:getLevel()
    end
    return ret
end

function This:create(...)
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret
end

function This:initIdByInfo(info)
    if info then
        if not info.descConfig then
            info.descConfig = TerrTechnsConfig[info.id]
        end
        local maxNum = info.descConfig.max_level
        if info.level and maxNum < info.level then
            info.level = maxNum
        end
        self:initByConfig( info )
    end
end

function This:initSelf( info )
    super.initSelf(self, info)
    self.m_nextConfig = TerrTechnQueueConfig[self:getId() + 1]
    if self.m_nextConfig then
        self.m_nextConfig.id = self:getId() + 1
    end
end

function This:initByConfig( info )
    super.initByConfig(self, info)
    self.m_level = info.level or 0
    local id = kTechnIdPrefix + info.id * 100 + self.m_level
    self:setId(id)
    self.m_config = TerrTechnQueueConfig[id]
    self.m_nextConfig = TerrTechnQueueConfig[id + 1]
    self.m_configId = info.id
end

function This:getConfigId()
    if not self.m_configId then
        local config = self.m_config or self.m_nextConfig
        return config.tech_id
    end
    return self.m_configId
end

function This:isNotLockedBy( cond1, cond2 )
    local switch = {
        [TerrTechnUpCondition.kTerrTechnBuildLev] = function(arg)
            local build = TerrBuildManager:getBuildByType(TerrBuildType.kTaiXueYuan)
            return arg <= build:getLevel()
        end,
        [TerrTechnUpCondition.kTerrPreTechnLev] = function(numArg, layerIndexArg)
            local levels = This.getTotalLevelsBy(self:getType(), layerIndexArg)
            return numArg <= levels 
        end,
    }

    local condition = TerrTechnUpCondition.kTerrTechnBuildLev
    if cond1 ~= condition then
        condition = TerrTechnUpCondition.kTerrPreTechnLev
    end

    return switch[condition](cond2, cond1)
end

function This:getAccerlateCost( time )
    if time <= 0 then
        return 0
    end
    local ret = TerrFormula.kTechnAccSkipCost(time)
    return ret
end

function This:isRelativeWith( infoTechnId )
    local cond1, cond2 = self:getCondition()
    if not cond1 then
        return false
    end

    local str = tostring(infoTechnId)
    local technType = tonumber(string.sub(str, 1, 1))
    local layerIndex = tonumber(string.sub(str, 2, 3))
    if technType ~= self:getType() then
        return false
    end

    local switch = {
        [TerrTechnUpCondition.kTerrTechnBuildLev] = function(...)
            return false
        end,
        [TerrTechnUpCondition.kTerrPreTechnLev] = function(layerIndexArg)
            return layerIndexArg == layerIndex
        end,
    }

    local condition = TerrTechnUpCondition.kTerrTechnBuildLev
    if cond1 ~= condition then
        condition = TerrTechnUpCondition.kTerrPreTechnLev
    end
    return switch[condition](cond1)
end

function This:getConditions()
    if self:getLevel() == 0 then
        local t = {}
        table.insert(t, self.m_config.upcondition)
        table.insert(t, self.m_nextConfig.upcondition)
        return t
    else
        local cfg = self:getConfig()
        return cfg.upcondition
    end
end

function This:getType()
    return self.m_descConfig.type
end

function This:getLayerIndex()
    return self.m_descConfig.layer
end

function This:getIndex()
    return self.m_descConfig.index
end

function This:getTypeBy( fullTechnId )
    local str = tostring(fullTechnId)
    return tonumber(string.sub(str, 2, 2)), tonumber(string.sub(str, 3, 4)), tonumber(string.sub(str, 5, 6))
end

function This:getImage()
    return "Icons/Techn/" .. tostring(self.m_descConfig.icon) .. ".png"
end

function This:isSameType( info )
    local str = tostring(info.id)
    local index = tonumber(string.sub(str, -2))
    local layerIndex = tonumber(string.sub(str, 2, 3))
    local technType = tonumber(string.sub(str, 1, 1))
    return self.m_level == info.level and self:getType() == technType and self:getLayerIndex() == layerIndex and self:getIndex() == index and self.m_state == info.state
end

function This:getAttrDesc()
    local level = self:getLevel() + 1
    local attrType = self.m_descConfig.attr_type
    local attr, attrNext = self.m_descConfig.attr[level], self.m_descConfig.attr[level + 1]
    if attrNext then
        if attrType ~= TerrTechnAttrType.kPercent then
            return tostring(attr), tostring(attrNext)
        else
            return tostring(attr * 100) .. "%", tostring(attrNext * 100) .. "%"
        end
    else
        if attrType ~= TerrTechnAttrType.kPercent then
            return tostring(attr)
        else
            return tostring(attr * 100) .. "%"
        end
    end
end

function This:getAttrList()
    local level = self:getLevel() + 1
    if self:getType() == TerrTechnType.kMilitary and level > 1 then
        if self.m_descConfig.attr_type == TerrTechnAttrType.kRange then
            return nil
        end
        local attrList = {}
        local sType = self.m_descConfig.affect_type
        local ret = {[self.m_descConfig.affect_id] = self.m_descConfig.attr[level]}
        if sType ~= 0 then
            attrList[sType] = ret
        else
            attrList[SoilderType.kShield] = ret
            attrList[SoilderType.kCavalry] = ret
            attrList[SoilderType.kArcher] = ret
            attrList[SoilderType.kSapperCar] = ret
        end
        return attrList
    end
    return nil
end

function This:hasLevelZero()
    return true
end

function This:isQueueModel()
    return false
end

function This:getDescription()
    return self.m_descConfig.desp
end

--请求
function This:requestUpgrade(accerlate,opType)
    if opType and opType == TerrUpType.kNow then
        local id = self:getConfigId()
        TerrTechnManager:onGetBuildUpgradeNowRequest(id)
        return
    end
    local build = TerrBuildManager:getBuildByType(TerrBuildType.kTaiXueYuan)
    local hasQueue, state = build:hasEmptyTechnQueue()
    if hasQueue then
        local id = self:getConfigId()
        TerrTechnManager:onGetTechnUpgradeRequest(id)
        if accerlate then
            TerrTechnManager:onAccerlateTechnUpRequest(id)
        end
    else
        g_scene:showTips(Serialize[64623])
    end
    return hasQueue
end

function This:requestAccerlate(preview)
    local canAccerlate, state = self:canAccerlate(preview)
    if canAccerlate then
        if not preview then
            local id = self:getConfigId()
            TerrTechnManager:onAccerlateTechnUpRequest(id)
        end
        return true
    else
        local switch = {
            [TerrResponse.kFailByTimeEnd] = function()
                g_scene:showTips(Serialize[64624])
            end,
            [TerrResponse.kFailByCost] = function()
                GamePlayer:checkDiamond(0, true)
            end,
            [TerrResponse.kFailByStateErr] = function()
                g_scene:showTips(Serialize[64625])
            end,
        }
        switch[state]()
    end
end


--请求结束

--endregion

