-- @Author: superm
-- @Date:   2017-10-09 16:55:04
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-11 20:54:08
-- @File name: TerrTechnManager.lua

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local TerrTechn = require("Territory.TerrTechn")
local TerrUtil = require("Territory.TerrUtil")
local TerrBuildManager = require("Territory.TerrBuildManager")
local HeroManager = require("Hero.HeroManager")

This.getLineNumBy = TerrTechn.getLineNumBy 
This.getNumByLayerIdx = TerrTechn.getNumByLayerIdx 
This.getTechnByFilter = TerrTechn.getTechnByFilter 

function This:init()
	super.init(self)
    self.m_techns = {}

    TerrTechn.createBaseTechns(self.m_techns)
    self:initListener()
    self.m_inited = true
end

function This:initListener()
	EventDispatcher:addListener(self,11511, self.onGetTechnUpgradeReply)
	EventDispatcher:addListener(self,11512, self.onGetTechnListInfoReply)
	EventDispatcher:addListener(self,11513, self.onGetTechnInfoChangeReply)
	EventDispatcher:addListener(self,11514, self.onAccerlateTechnUpReply)
    EventDispatcher:addListener(self, 11525, self.onGetBuildUpgradeNowReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self,11511)
    EventDispatcher:removeListener(self,11512)
    EventDispatcher:removeListener(self,11513)
    EventDispatcher:removeListener(self,11514)
    EventDispatcher:removeListener(self,11525)
end

--太学院科技
function This:getTechnBy( technId )
    return self.m_techns[technId]
end

function This:addTechn( info )
    local queue
    local techn = self:getTechnBy(info.id)
    if not techn then
        techn = TerrTechn:create(info)
        self.m_techns[info.id] = techn
    else
        queue = techn:getQueue()
        techn:updateInfo(info)
    end

    if techn:isUpgrading() then
        local build = TerrBuildManager:getBuildByType(TerrBuildType.kTaiXueYuan)
        build:addTechnQueue(info, techn)
    elseif queue then
        local build = TerrBuildManager:getBuildByType(TerrBuildType.kTaiXueYuan)
        build:addTechnQueue(info, nil)
        techn:setQueue(nil)
    end
    return techn:getConfigId(), techn
end

function This:getAllTechnAttrAdditionList()
    local attrAdditionList = {            
        [SoilderType.kShield] = {},
        [SoilderType.kCavalry] = {},
        [SoilderType.kArcher] = {},
        [SoilderType.kSapperCar] = {},
    }
    for technId, techn in pairs(self.m_techns) do
        local attrList = techn:getAttrList()
        if attrList then
            for soilderId, info in pairs(attrList) do
                local soilderInfo = attrAdditionList[soilderId]
                for attrId, attrValue in pairs(info) do
                    local rvalue = soilderInfo[attrId] or 0
                    soilderInfo[attrId] = rvalue + attrValue
                end
            end
        end
    end
    return attrAdditionList
end

--------------------------------------------------------------------------------------

function This:onGetTechnUpgradeRequest(technId)
    Socket:sendAndLock(11511, {tech_id = technId})
end

function This:onGetTechnUpgradeReply(msg)
    if msg.err_code == 0 then
        self:notifyObservers("onGetTechnUpgrade", msg.tech_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetTechnListInfoRequest()
    Socket:sendAndLock(11512)
end

function This:onGetTechnListInfoReply(msg)
    if msg then        
        for _, info in ipairs(msg.tech_list) do
            self:addTechn(info)
        end
        HeroManager:updateTechnAttr(self:getAllTechnAttrAdditionList())
        self:notifyObservers("onGetTechnListInfo")
    else
        g_scene:handleErrorCode("")
    end
end

function This:onGetTechnInfoChangeReply(msg)
    if msg then
        for _, info in ipairs(msg.tech_list) do
            self:addTechn(info)
        end
        HeroManager:updateTechnAttr(self:getAllTechnAttrAdditionList())
        self:notifyObservers("onGetTechnInfoChange")
    else
        g_scene:handleErrorCode("")
    end
end

function This:onAccerlateTechnUpRequest(technId)
    Socket:sendAndLock(11514, {tech_id = technId})
end

function This:onAccerlateTechnUpReply(msg)
    if msg.err_code == 0 then
        g_scene:showTips(Serialize[64628])
        self:notifyObservers("onAccerlateTechnUp", msg.tech_id)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetBuildUpgradeNowRequest(technId)
    Socket:sendAndLock(11525, {tech_id = technId})
end

function This:onGetBuildUpgradeNowReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onGetTechnUpgrade", msg.tech_id)
        -- self:notifyObservers("onGetBuildUpgradeNow")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end