-- @Author: superm
-- @Date:   2017-10-21 10:51:06
-- @Last Modified by:   superm
-- @Last Modified time: 2017-11-28 14:44:16
-- @File Name: TerrTechnPanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ListLayer = require("UI.ListLayer")
local TerrTechnManager = require("Territory.TerrTechnManager")
local TerrTechnItemView = require("Territory.TerrTechnItemView")

local kPageNum = 3

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaiXueYuan2.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self.m_scrollView = self.m_csNode.scrollView

    self.m_curTab = "TerrTechnType.kMilitary"
    self.m_curTabType = TerrTechnType.kMilitary
    self.m_curLockedLayerIndex = 0
    self.m_curTabViews = {}

    self:loadTabViews()
end

function This:loadTabViews()
    TerrTechnItemView.createAllTechnViews()
    self.m_scrollView:setVisible(false)
    for i, v in pairs(TerrTechnType) do
        self:selectTab(v)
    end
    self.m_scrollView:removeAllItems()
    self.m_scrollView:setTouchEnabled(true)
    self.m_scrollView:setVisible(true)
end

function This:onEnter()
    super.onEnter(self)
    TerrTechnManager:addObserver(self)
end

function This:onExit()
    for _, views in ipairs(self.m_curTabViews) do
        for _, view in ipairs(views) do
            view:release()
        end
    end
    TerrTechnItemView.releaseAllTechnViews()
    TerrTechnManager:removeObserver(self)
    super.onExit(self)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)

    for i, v in pairs(TerrTechnType) do
        self:addTabButton("TerrTechnType." .. i, self.m_csNode["button" .. v], function()
            self.m_curTabType = v
            self:selectTab(v, true)
        end)
    end

    self:selectTabButton(self.m_curTab)
    -- self:selectTab( TerrTechnType.kMilitary )
end

function This:selectTab( technType, refresh )
    if refresh then
        self.m_scrollView:removeAllItems()
        self.m_scrollView:setTouchEnabled(true)
    end

    local views = self.m_curTabViews[technType]
    local num = TerrTechnManager.getLineNumBy(technType)
    if not views then
        views = {}
        for i=1, num do
            local view = self:addLine(technType, i, nil, refresh)
            view:retain()
            table.insert(views, view)
        end
        self.m_curTabViews[technType] = views
    else
        for i=1, num do
            self:addLine(technType, i, views[i], refresh)
        end
    end

    if refresh then
        self:refreshLines(views)
    end
end

function This:showTechnBy( techn, delay )
    if self.m_curLockedLayerIndex > kPageNum then
        local layerIndex = techn:getLayerIndex()
        self.m_scrollView:scrollToPercentVertical((layerIndex - 1) / (self.m_curLockedLayerIndex - kPageNum) * 100, delay)
        return true
    end
end

function This:addLine(technType, index, node, refresh)
    local view = node or TerrTechnItemView:create(self)
    if refresh then
        view:refreshList(technType, index)
    end
    view:setAnchorPoint(0.5, 0)
    return view
end

function This:refreshLines( views )
    views = views or self.m_curTabViews[self.m_curTabType]
    for index, view in ipairs(views) do
        local w , h = view:getContentSize()
        self.m_scrollView:pushNode(view, w, h)
        self.m_curLockedLayerIndex = index
        if view:isAllTechnsLocked() and index >= KDefaultVisiableLayerNum then
            break
        end
    end
end

function This:onAccerlateTechnUp( technId )

    local str = tostring(technId)
    if self.m_curTabType ~= tonumber(string.sub(str, 1, 1)) then
        return
    end

    local ret = false
    local techns = TerrTechnManager.getTechnByFilter( self.m_curTabType, self.m_curLockedLayerIndex )
    for _, techn in ipairs(techns) do
        ret = techn:isRelativeWith(technId)
        if ret then
            break
        end
    end

    if ret then
        local views = self.m_curTabViews[self.m_curTabType]
        for index, view in ipairs(views) do
            if index  == self.m_curLockedLayerIndex + 1 and not view:isAllTechnsLocked() then
                self.m_curLockedLayerIndex = index
                local w , h = view:getContentSize()
                self.m_scrollView:pushNode(view, w, h)
                break
            end
        end
    end
end




--endregion
