-- @Author: superm
-- @Date:   2017-10-21 10:51:06
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-11 18:02:04
-- @File Name: TerrTechnUpgradePanel.lua
-- @Description: 

local This, super = class(..., "Dialog")
local TerrTechnManager = require("Territory.TerrTechnManager")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrTechnView = require("Territory.TerrTechnView")
local TerrUtil = require("Territory.TerrUtil")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")

function This:initNode(techn)
    self.m_techn = techn
    self.m_techn:updateState()

	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaiXueYuan4.csb")
    self.m_csNode:setPosition(cc.visible.center.x,cc.visible.center.y)
    self.m_techView = TerrTechnView:create(self.m_techn, self.m_csNode.skill)
    self.m_csNode:addChild(self.m_techView)
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

    self:refresh()
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
    self.m_csNode.zhuangtai1.levelup:setTouchEnded(function()
        self:upgradeTechn(false,TerrUpType.kNormal)
    end)

    self.m_csNode.zhuangtai1.atonce:setTouchEnded(function()
        self:upgradeTechn(true,TerrUpType.kNow)--立即完成
    end)

    self.m_csNode.zhuangtai1.goto1:setTouchEnded(function()
        if self.m_mainBuild then
            self:addCloseCallback(function ()
                local request = self.m_mainBuild:getRequest(TerrBuildRequestMode.kUpgrade)
                if request then
                    request()
                end
            end)
            g_scene:closeAllDialog()
        else
            if not self.m_mainTechn:isUpgrading() then
                local view = self.m_mainTechn:getView()
                if view then
                    local node = g_scene:getDialog("TerrTechnPanel")
                    if node then 
                        self:setVisible(false)
                        local delay = 0.5
                        local hasAction = node:showTechnBy(self.m_mainTechn, delay) 
                        if not hasAction then
                            self:changeCurrentTechn()
                            return
                        end
                        self:runAction(cc.Sequence(cc.DelayTime(delay), cc.CallFunc(function ()
                            self:changeCurrentTechn()
                        end)))
                    end
                end
            else
                self:changeCurrentTechn()
            end
        end
    end)

    self.m_csNode.zhuangtai1.goto2:setTouchEnded(function()
        GamePlayer:checkGold({nocheck = true, nowarming = true})
    end)
    
    self.m_csNode.zhuangtai2.atonce:setTouchEnded(function()
        self.m_techn:requestAccerlate()
    end)
end

function This:changeCurrentTechn()
    self.m_techn:removeObserver(self)
    self.m_techView:setTechn( self.m_mainTechn )
    self.m_techn = self.m_mainTechn
    self.m_mainTechn = nil
    self:refresh(0.1)
    self:setVisible(true)  
end

function This:upgradeTechn( accerlate ,opType)
    local canUpgrade, state, errCode = self.m_techn:canUpgrade()
    if not canUpgrade then
        local switch = {
            [TerrResponse.kFailByCost] = function()
                if errCode then
                    g_scene:handleErrorCode(errCode)
                else
                    g_scene:showTips(Serialize[64562])
                end
            end,
            [TerrResponse.kFailByLocked] = function()
                g_scene:showTips(Serialize[64563])
            end,
        }
        switch[state]()
    else
        -- if accerlate then
        --     local canAccerlate = self.m_techn:requestAccerlate(true)
        --     if not canAccerlate then
        --         return
        --     end
        -- end
        self.m_techn:requestUpgrade(accerlate,opType)
    end
end

function This:onEnter()
	super.onEnter(self)
    self.m_techn:addObserver(self)
    TerrTechnManager:addObserver(self)
    GamePlayer:addObserver(self)
end

function This:onExit()
    GamePlayer:removeObserver(self)
    TerrTechnManager:removeObserver(self)
    self.m_techn:removeObserver(self)
    super.onExit(self)
end

function This:refresh()
    local curTechnType = self.m_techn:getType()
    local curTechnLev = self.m_techn:getLevel()
    local isMaxLev = self.m_techn:isMaxLevel()
    self.m_csNode.title:setString( self.m_techn:getName() )

    local description = self.m_techn:getDescription()
    local curAttr, nextAttr = self.m_techn:getAttrDesc()
    self.m_csNode.descript:setString(description)

    if isMaxLev then
        self.m_csNode.zhuangtai1:setVisible(false)
        self.m_csNode.zhuangtai2:setVisible(false)
        self.m_csNode.bg6:setVisible(false)
        self.m_csNode.zhuangtai3:setVisible(true)
        self.m_csNode.next_txt:setVisible(false)
        self.m_csNode.now:setString(curAttr)
    else
        self.m_csNode.zhuangtai3:setVisible(false)
        self.m_csNode.next_txt:setVisible(true)
        self.m_csNode.now:setString(curAttr)
        self.m_csNode.next:setString(nextAttr)
        local showNode, costTime, accerlateCost 
        local isOpening = self.m_techn:isOpening()
        local costItemId, costNum = self.m_techn:getUpgradeCost()
        self.m_upgradeTime = self.m_techn:getUpgradeTime()
        if not self.m_techn:isUpgrading() then
            local switch = {
                [TerrTechnUpCondition.kTerrPreTechnLev] = function(arg, technType)
                    local num  = TerrTechnManager.getNumByLayerIdx(technType, arg)
                    if num > 1 then
                        return Serialize[60247] .. arg .. Serialize[64632] 
                    else
                        if not self.m_mainTechn then
                            local technList  = TerrTechnManager.getTechnByFilter(technType, arg)
                            self.m_mainTechn = technList[1]
                        end
                        return self.m_mainTechn:getName() 
                    end
                end,
                [TerrTechnUpCondition.kTerrTechnBuildLev] = function()
                    return Serialize[64633]
                end,
            }
            showNode = self.m_csNode.zhuangtai1
            self.m_csNode.zhuangtai2:setVisible(false)
            costTime = self.m_upgradeTime 

            local stateLevel, stateCost = false, false
            local cond1, cond2 = self.m_techn:getCondition()
            local canUpgrade, state = self.m_techn:canUpgrade(true)
            if not canUpgrade then
                if TerrResponse.kFailByCost == state then
                    stateLevel = true
                elseif TerrResponse.kFailByLocked == state then
                    stateCost = true
                end
            else
                stateLevel, stateCost = true, true
            end

            local condition = cond1 <= 0 and TerrTechnUpCondition.kTerrTechnBuildLev or TerrTechnUpCondition.kTerrPreTechnLev
            if stateLevel then
                TerrUtil:colorCondText(showNode, 1, true, switch[condition](cond1, curTechnType), tostring(cond2) .. Serialize[60187])
                showNode.leveluptext:setVisible( false )
            else
                local dependence
                if cond1 > 0 then
                    if not self.m_mainTechn then
                        local technList  = TerrTechnManager.getTechnByFilter(curTechnType, cond1)
                        self.m_mainTechn = technList[1]
                    end
                    dependence = self.m_mainTechn
                else
                    if not self.m_mainBuild then
                        self.m_mainBuild = TerrBuildManager:getBuildByType(TerrBuildType.kTaiXueYuan)
                    end
                    dependence = self.m_mainBuild
                end

                if dependence:isUpgrading() then
                    showNode.leveluptext:setString(dependence:getName() .. Serialize[64634])
                    showNode.leveluptext:setVisible( true )
                    showNode.condition1:setVisible( false )
                    showNode.finish1:setVisible( false )
                    showNode.unfinish1:setVisible( true )
                    showNode.goto1.text:setString( Serialize[64567] )                    
                else
                    showNode.goto1.text:setString( Serialize[64568] )
                    TerrUtil:colorCondText(showNode, 1, false, switch[condition](cond1, curTechnType), tostring(cond2) .. Serialize[60187])
                end
            end
            TerrUtil:colorCondText(showNode, 2, stateCost, ItemUtil.getName(costItemId), tostring(costNum))
        else
            showNode = self.m_csNode.zhuangtai2
            self.m_csNode.zhuangtai1:setVisible(false)
            costTime = self.m_techn:getLeftTime()
        end
        accerlateCost = self.m_techn:getAccerlateCost(costTime)
        
        showNode.price:setString(tostring(accerlateCost))
        showNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"], costTime))
        showNode:setVisible(true)
    end
end

function This:updateTime(leftTime)
    if leftTime > 0 then
        self.m_csNode.zhuangtai2.price:setString(tostring(math.ceil(leftTime / 60.0)))
        self.m_csNode.zhuangtai2.time:setString(GameUtil:formatTime(Serialize["H_M_S"], leftTime))
        self.m_csNode.zhuangtai2.jindubg.jindu:setPercent(100 * leftTime / self.m_upgradeTime)
        if leftTime > 1 then
            self:setTimer(1, function()
                self.m_techn:updateState()
                self:refresh()
            end, "tachn_updateTime")
        end
    end
end

function This:onAccerlateTechnUp( technId )
    local str = tostring(technId)
    if self.m_techn:getType() ~= tonumber(string.sub(str, 1, 1)) then
        return
    end

    if self.m_techn:getConfigId() == technId then
        self.m_techn:updateState()
        self:refresh()
    elseif self.m_techn:isLocked() and self.m_techn:getLevel() <= 0 and self.m_techn:isRelativeWith(technId) then
        self.m_techn:updateState()
        self:refresh()
    end
    self:close()
end

function This:onGetTechnUpgrade( technId )
    local str = tostring(technId)
    if self.m_techn:getType() ~= tonumber(string.sub(str, 1, 1)) then
        return
    end

    if self.m_techn:getConfigId() == technId then
        g_scene:showTips(self.m_techn:getName() .. Serialize[64635])
        self.m_techn:updateState()
        self:refresh()
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.gold and not self.m_techn:isUpgrading() then
        local costItemId, costNum = self.m_techn:getUpgradeCost()
        TerrUtil:colorCondText(self.m_csNode.zhuangtai1, 2, costNum >= value, ItemUtil.getName(costItemId), tostring(costNum))
    end
end

--endregion
