--region *.lua
--Date
--此文件由[BabeLua]插件自动生成
local This , super = class(...,"cc.Node")
local Timer = require("Timer")
local TerrTechnManager = require("Territory.TerrTechnManager")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")

This.s_technViews = {}
function This.getAllTechnViews()
    return This.s_technViews
end

function This.createAllTechnViews()
    for _, technType in pairs(TerrTechnType) do
        local num = TerrTechnManager.getLineNumBy(technType)
        for layerIndex=1, num do
            local techns = This.s_technViews[technType * 100 + layerIndex]
            if not techns or table.count(techns) <= 0 then
                if not techns then
                    This.s_technViews[technType * 100 + layerIndex] = {}
                    techns = This.s_technViews[technType * 100 + layerIndex]
                end
                local models = TerrTechnManager.getTechnByFilter(technType, layerIndex)
                if models then
                    for _, model in ipairs(models) do
                        local view = This:create(model)
                        techns[model:getConfigId()] = view
                        view:retain()
                    end
                end
            end
        end
    end
end

function This.releaseAllTechnViews()
    for _, views in pairs(This.s_technViews) do
        for _, view in pairs(views) do
            view:release()
        end
    end
    This.s_technViews={}
end

function This:init(techn, node)
    self.m_techn = techn
    self.m_inited = false
    if not node then
        self.m_selected = false
        self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TaiXueYuan1.csb")
        self:addChild(self.m_csNode)
    else
        self.m_selected = true
        local posx, posy = node:getPosition()
        node:retain()
        node:removeFromParent(false)
        node:setPosition(0, 0)
        self:addChild(node)
        node:release()
        self.m_csNode = node
        self:setPosition(posx, posy)
    end

    self:refreshProgress()
    self.m_csNode.name:setString(self.m_techn:getName())
    self.m_timer = Timer:create(self)
    self:scheduleUpdate()
end


function This:onEnter()
    super.onEnter(self)
    self.m_techn:updateState()
    if not self.m_selected then
        self.m_techn:setView(self)
    end
    self.m_techn:addObserver(self)
    TerrTechnManager:addObserver(self)
    GamePlayer:addObserver(self)
    self:initTouchEvent()
    self:refresh()
end

function This:onExit()
    GamePlayer:removeObserver(self)
    TerrTechnManager:removeObserver(self)
    self.m_techn:removeObserver(self)
    if not self.m_selected then
        self.m_techn:setView(nil)
    end
    self:unloadUpgradeEff()
    super.onExit(self)
end


function This:initTouchEvent()
    if not self.m_selected and not self.m_inited then
        -- print("-----------------------------------" .. self.m_techn:getName())
        self.m_csNode.bg:setTouch({
            onEnded = function()
                if not self.m_selected then
                    if not self.m_techn:isCompleteLocked() then
                        g_scene:openDialog("Territory.TerrTechnUpgradePanel", self.m_techn)
                    else
                        self:showLockedTips()
                    end
                elseif self.m_selectHandler then
                    self.m_selectHandler()
                end
            end,
            isContainer = true,
            isScale = false
        })
    end
    self.m_inited = true
end

function This:setPositionBy(parent)
    local technType, layerIndex = self.m_techn:getType(), self.m_techn:getLayerIndex()
    local index, num = self.m_techn:getIndex(), TerrTechnManager.getNumByLayerIdx(technType, layerIndex)
    local parentLenght, parentHigh = parent:getContentSize()
    local unitLength = parentLenght / (num + 1)
    local posX = index * unitLength 
    local posY = parentHigh * 0.5
    self:setPosition(posX, posY)
end

function This:setSelectHandler(handler)
    self.m_selectHandler = handler
end

function This:setSelect(isSelected)
    self.m_selected  = isSelected
end

function This:showLockedTips()
    local cond1, cond2 = nil, nil
    local conds = self.m_techn:getConditions()

    if conds and isNestedObject(conds) then
        for _, cond in ipairs(conds) do
            local isSuitable = self.m_techn:isNotLockedBy(cond[1], cond[2])
            if not isSuitable then
                cond1, cond2 = cond[1], cond[2]
                break
            end
        end

        if not cond1 or not cond2 then
            return
        end
    else
        return
    end

    local switch = {
        [TerrTechnUpCondition.kTerrPreTechnLev] = function(arg, num)
            return Serialize[60247] .. arg .. Serialize[64632] .. num .. Serialize[60759]
        end,
        [TerrTechnUpCondition.kTerrTechnBuildLev] = function(arg, num)
            return Serialize[64637] .. num .. Serialize[60759]
        end,
    }
    
    local condition = TerrTechnUpCondition.kTerrTechnBuildLev
    if cond1 > 0 then
        condition = TerrTechnUpCondition.kTerrPreTechnLev 
    end
    g_scene:showTips(switch[condition](cond1, cond2)) 
end

function This:refreshProgress()
    if self.m_techn:isUpgrading() then
        self.m_csNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"], self.m_techn:getLeftTime()))
        self.m_csNode.time:setVisible(true)
    else
        self.m_csNode.time:setVisible(false)
    end
end

function This:loadUpgradeEff()
    if not self.m_effect then
        self.m_effect = GameUtil:createEffect("YanJiuZhong", true)
        self.m_csNode:addChild(self.m_effect)         
    end
end

function This:unloadUpgradeEff()
    if self.m_effect then
        self.m_effect:removeFromParent()
        self.m_effect = nil
    end
end

function This:refresh(delay)
    -- print("----m_techn---".. self.m_techn:getConfigId() .. "----------------------------refresh(delay)")
    local function update() 
        self.m_csNode.level:setVisible(true)
        self.m_csNode.icon:setVisible(true)
        self.m_csNode.name:setVisible(true)
        self.m_csNode.timebg:setVisible(false)
        if not self.m_techn:isLocked() or self.m_techn:getLevel() > 0 then
            self.m_csNode.mask:setVisible(false)
            self.m_csNode.lock:setVisible(false)
            self.m_csNode.jiantou:setVisible(self.m_techn:canUpgrade())
            local isUpgrading = self.m_techn:isUpgrading()
            if isUpgrading then
                self.m_csNode.jiantou:setVisible(false)
                self.m_csNode.mask:setVisible(false)
                self.m_csNode.timebg:setVisible(true)
                self:loadUpgradeEff()
            else
                self:unloadUpgradeEff()
            end
        else
            self:unloadUpgradeEff()
            if self.m_techn:isCompleteLocked() then
                self.m_csNode.mask:setVisible(true)
                self.m_csNode.lock:setVisible(true)
            else
                self.m_csNode.mask:setVisible(false)
                self.m_csNode.lock:setVisible(false)
            end
            self.m_csNode.jiantou:setVisible(false)
        end
        self:refreshProgress()
        self.m_csNode.name:setString("" .. self.m_techn:getName())
        self.m_csNode.icon:setImage(self.m_techn:getImage())
        self.m_csNode.level:setString("" .. self.m_techn:getLevel() .. "/" .. self.m_techn:getMaxLevel())
    end

    if delay then
        self.m_timer:setTimer(delay,function()
            update()
        end,"techn"..self.m_techn:getId())
    else
        update()
    end
end

function This:updateTime(leftTime)
    self.m_csNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"], leftTime))
end

function This:setTechn( techn )
    if self.m_selected then
        self.m_techn:removeObserver(self)
        self.m_techn = techn
        techn:updateState()
        techn:addObserver(self)
        self:refresh(0.1)
    end
end

----===================================================================================----

function This:update(dt)
    self.m_timer:update(dt)
end

function This:onUpgradeStart( techn )
    if self.m_techn == techn then
        self:refresh()
    end
end

function This:onUpgradeEnd( techn )
    if self.m_techn == techn then
        self:refresh()
    end
end

function This:onAccerlateTechnUp( technId )
    local str = tostring(technId)
    if self.m_techn:getType() ~= tonumber(string.sub(str, 1, 1)) then
        return
    end

    if self.m_techn:getLevel() <= 0 and self.m_techn:isLocked() and self.m_techn:isRelativeWith(technId) then
        self.m_techn:updateState()
        self:refresh()
    end
end

function This:onPlayerAttrChanged(field, oldValue, value)
    if field == PlayerAttr.gold and not self.m_techn:isUpgrading() then
        self.m_techn:updateState()
        self:refresh()
    end
end

--endregion
