-- @Author: superm
-- @Date:   2017-10-21 10:51:06
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-25 21:40:47
-- @File Name: TerrUpgradeModel.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")

local function isTable( obj )
    if obj and type(obj) == "table" then
        return true
    end
    return false
end

function isNestedObject( obj )
    if isTable(obj) and isTable(obj[1]) then
        return true
    end
    return false
end

-- local function formatDate(format,time)
--  return require("Util.GameUtil"):formatDate(format,time)
-- end

function This:init( info )
	super.init(self)

    self.m_view = nil
	if info then
        self:initSelf( info )

        if not self:hasLevelZero() and info.level == 0 then
            print("后端数据有误, id=" .. info.id)
            info.level = 1
        end

        if info.state == TerrObjectState.kUpdating and info.start_time == info.end_time then
            print("后端数据有误，升級進程本應該結束, id=" .. info.id)
            print(info)
            print("---------------------------------")
        end

        if info.end_time > 0 and info.state == TerrObjectState.kNormal and GamePlayer:getServerTime() < info.end_time then
            -- print("升級進程結束, id=" .. info.id .. ", end_time:" .. formatDate(Serialize["y_m_d_H_M_S"],info.end_time))
            info.end_time = -1
        end

        self.m_level = info.level
        self.m_state = info.state
        self.m_queue = nil       
        self.m_upStartTime = info.start_time
        self.m_upEndTime = info.end_time
        self.m_upEscape = 0
        self.m_upIsUpdating = false

        self.m_lastAutoTime = info.last_auto_time
        self:updateState(true, info)
    else
        self.m_ID = 0
        self.m_level = 0
        self.m_upStartTime = -1
        self.m_upEndTime = -1
        self.m_upEscape = 0
        self.m_upIsUpdating = false

        self.m_lastAutoTime = -1
        self.m_queue = nil
        self.m_state = TerrObjectState.kNormal
    end
end

function This:initIdByInfo( info )
end

function This:initSelf( info )
    self:initIdByInfo(info)
end

function This:updateSelf( info )
end

function This:initByConfig( info )
    self.m_descConfig = info.descConfig
end

function This:updateInfo( info )
	local isDiff = not self:isSameType(info) 

    if not self:hasLevelZero() and info.level == 0 then
        print("后端数据有误, id=" .. info.id)
        info.level = 1
    end


    if info.state == TerrObjectState.kUpdating and info.start_time == info.end_time then
        print("后端数据有误，升級進程本應該結束, id=" .. info.id)
        print(info)
        print("---------------------------------")
    end

    if info.end_time > 0 and info.state == TerrObjectState.kNormal and GamePlayer:getServerTime() < info.end_time then
        -- print("升級進程結束, id=" .. info.id .. ", end_time:" .. formatDate(Serialize["y_m_d_H_M_S"],info.end_time))
        info.end_time = -1
    end

    self:setLevel(info.level)
    self.m_state = info.state
    self.m_upStartTime = info.start_time or 0
    self:setEndTime(info.end_time or 0)
    if info.last_auto_time then
        self.m_lastAutoTime = info.last_auto_time
    end
    self:updateSelf( info )
    self:updateState(isDiff, info)
end

function This:updateState(isInit, info)
    local isChanged = false
    local oldState = self.m_state
    if isInit then
        self:initSelf(info)
        isChanged = true
    end

	local curTime = GamePlayer:getServerTime()
    if curTime < self.m_upEndTime then
	    self.m_state = TerrObjectState.kUpdating
	    self:startUpdate()
    elseif curTime >= self.m_upEndTime then
        self:endUpdate()

        local conds = self:getConditions()
        if (not conds) or ( type(conds) == "number" and conds <= 0 ) then
            self.m_state = TerrObjectState.kNormal
        else
            local ret   = true
            local hadDone   = false
            local isCompleteLocked  = false
            if not isNestedObject(conds) then
            	if #conds >= 2 then
                	ret = self:isNotLockedBy(conds[1], conds[2])
                end
            else
                local condIndex = 1
                local condsCount = table.count(conds)
                for i, cond in ipairs(conds) do
                    condIndex = i
                    ret = ret and self:isNotLockedBy(cond[1], cond[2])
                    if not ret then
                        self.m_state = TerrObjectState.kLocked
                        hadDone   = true
                        break
                    end
                end
                
                if not ret and condsCount > 1 and condIndex ~= condsCount then
                    self.m_state = TerrObjectState.kCompleteLocked
                    hadDone   = true
                end
            end

            if not hadDone then
                self.m_state = ret and TerrObjectState.kNormal or TerrObjectState.kLocked
            end
        end
    end

    if not isChanged and self.m_state ~= oldState then
        isChanged = true
    end

    if not self.m_level or self.m_level == 0 then
        local oleLevel = self.m_level

        if info and info.level and info.level > 0 then
            self:setLevel(info.level)
        elseif not self:isLocked() then
            self.m_level = self.m_ID % 100
        end

        if not isChanged and self.m_level ~= oleLevel then
            isChanged = true
            if self.m_lastAutoTime <= 0 then
                self.m_lastAutoTime = GamePlayer:getServerTime()
            end
        end
    end
    return isChanged
end

function This:increaseLevel()
    self.m_level = self.m_level + 1
end

function This:setEndTime( endTime )
	self.m_upEndTime = endTime
	local curTime = GamePlayer:getServerTime()
	if curTime < self.m_upEndTime then
		self.m_state = TerrObjectState.kUpdating
		self:startUpdate()
	end
	self:notifyObservers("onInfoChanged", self)
    if self:isQueueModel() and self.m_queue then
        self.m_queue:notifyObservers("onInfoChanged", self)
    end
end

function This:setView(view)
    self.m_view = view
end

function This:getView()
    return self.m_view
end

function This:getQueue()
    return self.m_queue
end

function This:getQueueId()
    if self.m_queue then
        return self.m_queue:getId()
    end
    return 0
end

function This:isQueueModel()
    return true
end

function This:setQueue( queue )
    if queue and self.m_state == TerrObjectState.kUpdating then
        self.m_queue = queue
    elseif not queue then
        self.m_queue = nil
    end
end

function This:getLevel()
    return self.m_level
end

function This:setLevel( level )
    if self.m_descConfig then
        local max = self:getMaxLevel()
        if level > max then
            self.m_level = max
            return
        end
    end
    self.m_level = level
end

function This:setId( id )
    self.m_ID = id
end

function This:getId()
    return self.m_ID
end

function This:getConfigId()
    return self.m_descConfig.uid
end

function This:getLeftTime()
    local curTime = GamePlayer:getServerTime()
    if curTime >= self.m_upEndTime then
        return 0
    end
    return self.m_upEndTime - curTime
end

function This:getProductTime()
    if self.m_lastAutoTime > 0 and self:getLevel() > 0 then
	    local curTime = GamePlayer:getServerTime()
	    return curTime - self.m_lastAutoTime
    else
        return 0
    end
end

function This:getProgress()
    local ltime = self:getLeftTime()
    if not self:isUpgrading() or ltime <= 0 then
        return 0
    end

    local totalTime = self.m_upEndTime - self.m_upStartTime
    return ltime / totalTime, ltime, totalTime
end

function This:isNotLockedBy( cond1, cond2 )
    return false
end

function This:isCostEnoughBy( id, num )
    if ItemUtil.getType(id) == ItemCommonType.kGameRes then
        local switch = {
            [101] = function(num)
                return GamePlayer:getGold() >= num, "goldLack"
            end,
            [102] = function(num)
                return GamePlayer:getDiamond() >= num, "diamondLack"
            end,
        }
        return switch[id](num)
    else
        return true
    end
end

function This:isCompleteLocked()
    return self.m_state == TerrObjectState.kCompleteLocked
end

function This:isLocked()
    return self.m_state == TerrObjectState.kLocked or self.m_state == TerrObjectState.kCompleteLocked
end

function This:isOpening()
    return self.m_state == TerrObjectState.kNormal
end

function This:isUpgrading()
    return self.m_state == TerrObjectState.kUpdating
end

function This:isSameType( info )
	return true
end

function This:isQualified()
    local costList = self.m_nextConfig.upcost
    if not costList or costList == 0 then
        return true
    end

    local ret = true
    local errCode
    if not isNestedObject(costList) then
        if #costList >= 2 then
            ret, errCode = self:isCostEnoughBy(costList[1], costList[2])
        end
    else
        for _, cond in ipairs(costList) do
            local result, reason = self:isCostEnoughBy(cond[1], cond[2])
            ret = ret and result
            if not ret then
                return false, reason
            end
        end
    end
    return ret, errCode
end


function This:isMaxLevel()
    local level = self:getLevel()
    if level > 0 then
        return level >= self:getMaxLevel() or not self.m_nextConfig 
    else
        return level >= self:getMaxLevel()
    end
end

function This:getMaxLevel()
    if not self.m_descConfig then
        return 0 
    end

    return self.m_descConfig.max_level
end

function This:isAccQualified( time )
    local cost = self:getAccerlateCost( time or self:getLeftTime() )
    if cost <= 0 then
        return true
    end
    return GamePlayer:getDiamond() >= cost
end

function This:canUpgrade(returnState)
    if self:isMaxLevel() then
        return false, TerrResponse.kFailByMaxLevel
    end

    if not self:isOpening() then
        if returnState then
            local ret, condition = self:isQualified()
            if not ret then
                return false, TerrResponse.kFailByLocked + TerrResponse.kFailByCost, condition
            end            
        end
        return false, TerrResponse.kFailByLocked
    end

    local ret, condition = self:isQualified()
    if not ret then
        return false, TerrResponse.kFailByCost, condition
    end

    return true, TerrResponse.kOK
end

function This:canAccerlate(preView)
    if not preView then
        if not self:isUpgrading() then
            return false, TerrResponse.kFailByStateErr
        end

        if self:getUpgradeTime() <= 0 or self:getLeftTime() <= 0 then
            return false, TerrResponse.kFailByTimeEnd
        end

        if self:isMaxLevel() then
            return false, TerrResponse.kFailByMaxLevel
        end

        if not self:isAccQualified() then
            return false, TerrResponse.kFailByCost
        end
    else
        local time = self:getUpgradeTime()
        if time <= 0 then
            return false, TerrResponse.kFailByTimeEnd
        end

        if self:isMaxLevel() then
            return false, TerrResponse.kFailByMaxLevel
        end

        local time = self:getUpgradeTime()
        if not self:isAccQualified(time) then
            return false, TerrResponse.kFailByCost
        end
    end
    return true, TerrResponse.kOK
end

function This:startUpdate()
	cc.Application:addToUpdateList(self)
	self:notifyObservers("onUpgradeStart", self)
    if self:isQueueModel() and self.m_queue then
        self.m_queue:notifyObservers("onUpgradeStart", self.m_queue:getId())
    end
	self.m_upIsUpdating = true
end

function This:endUpdate()
	cc.Application:removeFromUpdateList(self)
	self:notifyObservers("onUpgradeEnd", self)
    if self.m_queue then
        if self:isQueueModel() then
            self.m_queue:notifyObservers("onUpgradeEnd", self.m_queue:getId())
        end
        self:removeFromQueue()
    end
	self.m_upIsUpdating = false
    
end

function This:removeFromQueue()
    if self.m_queue then
        self.m_queue:removeAllObjects()
        self.m_queue = nil
    end
end

function This:update( dt )
	self.m_upEscape = self.m_upEscape + dt
	if self.m_upEscape >= 1.0 then
		self.m_upEscape = self.m_upEscape - 1
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			self:notifyObservers("updateTime", leftTime)
            if self:isQueueModel() and self.m_queue then
                self.m_queue:notifyObservers("updateTime", leftTime, self:getProgress())
            end
		else
        	self:endUpdate()
            self:updateState()
			self:notifyObservers("updateTime", 0)
            if self:isQueueModel() and self.m_queue then
                self.m_queue:notifyObservers("updateTime", 0)
            end
		end
	end
end

--获取配置信息region begin

function This:getConfig()
    local cfg = nil
    if not self:hasLevelZero() then
        cfg = (self:getLevel() > 0 and not self:isMaxLevel()) and self.m_nextConfig or self.m_config
    else
        cfg = not self:isMaxLevel() and self.m_nextConfig or self.m_config
    end
    return cfg
end

function This:getCondition()
    local cfg = self:getConfig()
    local conds = cfg.upcondition
    if conds then
        return conds[1], conds[2]
    end
    return nil, nil
end

function This:getConditions()
    local cfg = self:getConfig()
    return cfg.upcondition
end

function This:getDescription()
    return self.m_config.desp
end

function This:getDescrNextLev()
    return self.m_nextConfig.desp
end

function This:hasLevelZero()
    return false
end

function This:getName()
    local cfg = nil
    if not self:hasLevelZero() then
        cfg = (self:getLevel() > 0 and not self:isMaxLevel()) and self.m_nextConfig or self.m_config
    else
        cfg = not self:isMaxLevel() and self.m_nextConfig or self.m_config
    end
    return cfg.name
end

-- return 秒数
function This:getUpgradeTime()
    local cfg = self:getConfig()
    return cfg.uptime
end

function This:getUpgradeCost()
    local cfg = self:getConfig()
    
    local costList = cfg.upcost
    if not costList or costList == 0 then
        return 0, 0
    end
    return costList[1], costList[2]
end

--获取配置信息region end

--endregion