-- @Author: superm
-- @Date:   2017-10-08 15:12:11
-- @Last Modified by:   superm
-- @Last Modified time: 2019-12-20 17:39:15
local This = class(...)
local ArgConfig = require("Metadatas.ArgConfig")

SECONDS_PER_MIN = 60
MINS_PER_HOUR = 60
SECONDS_PER_HOUR = 3600
SECONDS_PER_DAY = 86400

-- 公式
TerrFormula = {
    kBuildAccSkipCost = function ( time )
        -- [封地建筑加速] 跳过花销系数（跟时间挂钩，单位秒），倒数（目前60秒一元宝）
        local x = time / SECONDS_PER_DAY
        if x > 0.5 then
            x = 0.5
        end
        local num = math.ceil(math.ceil(time / SECONDS_PER_MIN) * (1 - x))
        local maxNum = ArgConfig.territory_building_max_cost.arg[1]
        if num > maxNum then
            num = maxNum
        end
        return num
    end,
    kTechnAccSkipCost = function ( time )
        -- [封地科技加速] 跳过花销系数（跟时间挂钩，单位秒），倒数（目前60秒一元宝）
        return math.ceil(time / SECONDS_PER_MIN)
    end,
    kPolyAccSkipCost = function ( time )
        -- [封地融合] 跳过花销系数（跟时间挂钩，单位秒），倒数（目前60秒一元宝）
        return math.ceil(time / SECONDS_PER_MIN)
    end
}

-- [封地] 通用状态码（内部使用）
TerrObjectState = {
    kNormal = 0,
    kUpdating = 1,
    kLocked = 2,    --锁定
    kCompleteLocked = 3,    --完全锁定
}

-- [封地] 通用响应码（内部使用）
TerrResponse = {
    kOK = 0,
    kFailByMaxLevel = 1, --已经满级
    kFailByCost = 2,
    kFailByLocked = 3,    --锁定
    kFailByStateErr = 20,    --出错
    kFailByTimeEnd = 21,     --倒计时时间不足（可能系出错）
}

-- [封地建筑] 配置的最小ID
kTerrMinBuildId = 1

-- [封地建筑] 配置的最大ID
kTerrMaxBuildId = 14

-- [封地建筑] 配置的类型
TerrBuildType = {
    kGuanfu = 1,    
    kMingJu = 2,
    kBingYing= 3,
    kTaiXueYuan = 4,    
    kJiangJunFu= 5,
    kJiaoChang = 6,
    kJiTan = 7,
    kTieJiangPu = 8,
    kJingJiDianFeng = 1001,
    kZhenHeTa = 1002,
    kWuDaoShangDian = 1005,
    kZhanYiTa = 1008,

    [1] = "Guanfu",
    [2] = "MingJu",
    [3] = "BingYing",
    [4] = "TaiXueYuan",
    [5] = "JiangJunFu",
    [6] = "JiaoChang",
    [7] = "JiTan",
    [8] = "TieJiangPu",
}

-- [封地建筑] 配置的ID
TerrBuildMap = { 
}

TerrBuildRequestMode = { 
    kOther = 0,
    kUpgrade = 100,
}

-- [封地建筑] 是否存在产出默认值
TerrBuildHasProduct = {
    [TerrBuildType.kGuanfu] = nil, 
    [TerrBuildType.kMingJu] = nil,
    [TerrBuildType.kBingYing]= nil,
    [TerrBuildType.kTaiXueYuan] = nil,    
    [TerrBuildType.kJiangJunFu]= nil,
    [TerrBuildType.kJiaoChang] = nil,
    [TerrBuildType.kJiTan] = nil,
    [TerrBuildType.kTieJiangPu] = nil,
}

-- [封地建筑] 显示次序
TerrBuildZOrder = {6, 7, 7, 5, 8, 6, 7, 7, 9, 4, 5, 5, 5, 4}

-- [封地建筑] 显示次序最大上限
kTerrBuildMaxZOrder = 10

-- [封地建筑] 菜单显示位置
TerrBuildViewPosition = {
    [1] = {
        offset = {x = -55, y = 120},    --整体偏移
        name = {x = -20, y = 115},
        menu = {x = 0, y = -78},
        roll = {x = -20, y = 145}
    }, 
    [2] = {
        offset = {x = 15, y = 75},
        name = {x = -10, y = -5},
        menu = {x = 0, y = -38},
        roll = {x = -10, y = 25}
    }, 
    [3]= {
        offset = {x = 95, y = 30},
        name = {x = 0, y = -40},
        menu = {x = 0, y = -38},
        roll = {x = 0, y = -10}
    }, 
    [4] = {
        offset = {x = 0, y = 60},
        name = {x = 10, y = 10},
        menu = {x = 10, y = -120},
        roll = {x = 10, y = 40}
    },
    [5] = {
        offset = {x = -5, y =65},
        name = {x = 10, y = -5},
        menu = {x = 0, y = -8},
        roll = {x = 10, y = 25}
    },
    [6] = {
        offset = {x = 10, y = 55},
        name = {x = -5, y = 55},
        menu = {x = -5, y = -95},
        roll = {x = -5, y = 80}
    },
    [7] = {
        offset = {x = 15, y = 80},
        name = {x = 15, y = 0},
        menu = {x = 15, y = -120},
        roll = {x = 15, y = 30}
    },
    [8] = {
        offset = {x = 10, y = 70},
        name = {x = -10, y = -10},
        menu = {x = 0, y = -38},
        roll = {x = -10, y = 20}
    },
    [9] = {
        offset = {x = 5, y = 55},
        name = {x = 5, y = -10},
        menu = {x = 5, y = -85},
        roll = {x = 5, y = 20}
    },
    [10] = {
        offset = {x = 5, y = 80},
        name = {x = 0, y = -25},
        menu = {x = 0, y = -38},
        roll = {x = 0, y = 5}
    },
    [11] = {
        offset = {x = 10, y = 70},
        name = {x = 0, y = -10},
        menu = {x = 0, y = -18},
        roll = {x = 0, y = 20}
    },
    [12] = {
        offset = {x = 10, y = 70},
        name = {x = 0, y = -10},
        menu = {x = 0, y = -38},
        roll = {x = 0, y = 20}
    },
    [13] = {
        offset = {x = 15, y = 70},
        name = {x = -20, y = -20},
        menu = {x = 0, y = -38},
        roll = {x = -20, y = 10}
    },
    [14] = {
        offset = {x = 0, y = 75},
        name = {x = 0, y = 0},
        menu = {x = 0, y = -80},
        roll = {x = 0, y = 30}
    },
    [1001] = {
        offset = {x = 5, y = 65},
        name = {x = 0, y = 10},
        menu = {x = 0, y = -38},
        roll = {x = 0, y = 40}
    },
    [1002] = {
        offset = {x = 5, y = 85},
        name = {x = 0, y = 10},
        menu = {x = 0, y = -38},
        roll = {x = 0, y = 40}
    },
    [1003] = {
        offset = {x = 35, y = 70},
        name = {x = -20, y = -20},
        menu = {x = 0, y = -38},
        roll = {x = -20, y = 10}
    },
    [1004] = {
        offset = {x = 35, y = 70},
        name = {x = -20, y = -20},
        menu = {x = 0, y = -38},
        roll = {x = -20, y = 10}
    },
    [1005] = {
        offset = {x = 35, y = 75},
        name = {x = -20, y = -20},
        menu = {x = 0, y = -38},
        roll = {x = -20, y = 10}
    },
    [1006] = {
        offset = {x = 20, y = 60},
        name = {x = -20, y = -20},
        menu = {x = 0, y = -38},
        roll = {x = -20, y = 10}
    },
    [1007] = {
        offset = {x = 20, y = 60},
        name = {x = -20, y = -20},
        menu = {x = 0, y = -38},
        roll = {x = -20, y = 10}
    },
    [1008] = {
        offset = {x = 5, y = 85},
        name = {x = 0, y = 10},
        menu = {x = 0, y = -38},
        roll = {x = 0, y = 40}
    },
}

-- [封地建筑升级] 条件类型 0无条件 1要求玩家等级 2要求主建筑（官府）等级 3要求爵位等级 4关联入口等级 5.竞技场排名 6.VIP等级（内部使用）
TerrConditionType = {
    kNone = 0,  
    kPlayerLev = 1,
    kTerrMainBuildLev = 2,
    kRankLev = 3,
    kTerrSelfBuildLev = 4,
    kArenaRank = 5,
    kVipLevel = 6,
    kFamilySelfBuildLev = 7,
    kFamilyLevel = 8,
    kComplex = 9,
    kFamilyTitle = 10,
}

-- [封地建筑升级] 模式类型 0,升级（正常升级只需要花费时间）1立即升级（立即升级可能需要花费元宝）
--和太学院的升级状态重用了
TerrUpType = {
    kNormal = 0,
    kNow = 1,
}

-- [封地建筑加速] 模式类型 1立即加速 2提示添加队列 3提示队列已满 （内部使用）
TerrAccelerateMode = {
    kNormal = 0,
    kImmFinish = 1,
    kAddQueue = 2,
    kQueueFull = 3,
}

-- [封地建筑加速] 跳过模式类型 0不跳过 1免费加速模式 2无提示加速 （内部使用）
TerrAccelerateSkipMode = {
    kNone = 0,
    kInFreeTime = 1,
    kNoTips = 2,
    kPreView = 3,
}

-- [封地建筑产出] 配置的时间间隔(单位:分钟)
kProductIntervalMinute = 10

-- [封地建筑产出] 显示的时间间隔(单位:分钟)
kProductIntervalMinuteInView = 60

-- [封地建筑产出] 显示倍数
kProductFactor = 6 -- kProductIntervalMinuteInView / kProductIntervalMinute

-- [封地建筑产出] 时间间隔(单位:秒) ，自动计算（内部使用）
kTerrBuildProductInterval = kProductIntervalMinuteInView * SECONDS_PER_MIN

-- [封地建筑产出] 时间间隔 显示的文字格式
kTerrBuildProductIntervalDesc = "/" .. kTerrBuildProductInterval / SECONDS_PER_HOUR .. Serialize[64596]

-- [封地建筑产出] 刷新的时间间隔(单位:秒)
kTerrBuildProductIntervalInView = kProductIntervalMinute * SECONDS_PER_MIN

-- [封地建筑产出] 刷新的时间倍数
kProductRefreshTimeFactor = 6 -- kTerrBuildProductInterval / kTerrBuildProductIntervalInView


-- [封地建筑功能] 名字的图片
TerrFunctionName = {
    [TerrBuildType.kTaiXueYuan] = "MenuScene/wenzi2.png",
    [TerrBuildType.kJiangJunFu] = "MenuScene/wenzi1.png",
    [TerrBuildType.kJiaoChang] = "MenuScene/wordzhanyi.png",
    [TerrBuildType.kJiTan] = "MenuScene/wenzi3.png",
    [TerrBuildType.kTieJiangPu] = "MenuScene/wenzi8.png",
}

TerrSpecialName = {
    [TerrBuildType.kTieJiangPu] = "MenuScene/wenzi7.png",
}

-- [封地建筑功能] 名字的默认图片(升级)
kTerrDefaultFuncionName = "MenuScene/wenzi4.png"

-- [封地建筑功能] 名字的默认图片（加速）
kTerrAccFuncionName = "MenuScene/wenzi5.png"

kTerrBuildQueuesNum = 3

kTerrVipBuildQueueId = 3

kTerrOfflineTimeLimitBuyTimesLimit = 18

kTerrOfflineTimeBuyOnce = SECONDS_PER_MIN * MINS_PER_HOUR * 1
-----------------------------------------------------------------------------------

-- [封地巡逻] 读表使用的辅助工具
kTerrDefaultPatrolType = 1
kTerrDefaultPatrolTimeType = 1

kTerrPatrolTypeCount = 3
TerrPatrolType = {
    [1] = {field = "commond", name = Serialize[64640]},
    [2] = {field = "middle", name = Serialize[64641]}, 
    [3] = {field = "high", name = Serialize[64642]},
}

-- [封地巡逻] 等级类型描述
TerrPatrolTypeDesc = {
    Serialize[64643], 
    Serialize[64644],
}

------------------------------------------------------------------------------------

-- [封地科技] 同时升级数的上限
kTerrMaxTechnQueueNum = 1

-- [封地科技] 大类类型
TerrTechnType = {
    kMilitary = 1,    
    kResource = 2,
    kDefence = 3,
}
TerrTechnAttrType = {
    kRange = 1,    
    kNumber = 2,
    kPercent = 3,
}

-- [封地科技] 升级条件类型 0要求太学院等级 1要求前层级科技等级
TerrTechnUpCondition = {
    kTerrTechnBuildLev = 0,
    kTerrPreTechnLev = 1,
}


-- [封地科技] 默认显示未解锁的科技层数（刚好一页）
KDefaultVisiableLayerNum = 3

------------------------------------------------------------------------------------
-- 鐵匠鋪
-- 最大求助次数
TerrPolyMaxBegTime = 5

-- 最大被帮助次数
TerrPolyMaxHelpTime = 20

-- 求助cd
TerrSendHelpCDTime = 3 * SECONDS_PER_MIN

-- 消耗运势
TerrPolyCostFortun = 10

------------------------------------------------------------------------------------
-- [封地贼寇] 等待时间
TerrRobberFightWaitTime = 0.5 * SECONDS_PER_MIN


-- [封地贼寇] 显示缩放比
TerrRobberSize = 0.65

-- [封地叛军] 显示缩放比
TerrBossRobberSize = 0.65

-- [封地贼寇] 名字,索引为难度
TerrRobberName = {
    [1] = Serialize[64645],--1-1-1
    [2] = Serialize[64646],--2-3
    [3] = Serialize[64647],--2-3-2
}

-- [封地贼寇] 难度描述,索引为难度
TerrRobberNanduTxt = {
	[1] = Serialize[64648],
	[2] = Serialize[64649],
	[3] = Serialize[64650],
}

-- [封地贼寇] 显示阵列位置,索引为难度
TerrRobberPosOffset = {
    [1] = {[1] = {x = 22, y = 20}, [2] = {x = 7, y = 5}, [3] = {x =-8, y = 20}},
    [2] = {[1] = {x = -15, y = 18}, [2] = {x =15, y = 18}, [3] = {x = 45, y = 18}, [4] = {x = 0, y =3}, [5] = {x = 30, y = 3}},
    [3] = {[1] = {x = -15, y = 18}, [2] = {x =15, y = 18}, [3] = {x = 48, y = 18}, [4] = {x = 0, y = 3}, [5] = {x = 30, y = 3}, [6] = {x = 0, y = 33}, [7] = {x = 30, y = 33}},
}

-- [封地贼寇] 点击范围大小,索引为难度
TerrRobberContentSize = {
    [1] = function (  )
        return 40, 40
    end ,
    [2] = function (  )
        return 53, 40
    end ,
    [3] = function (  )
        return 53, 50
    end ,
}

TerrRobberEffect= {
    [1] = function (  )
        return {
            eff = {
                position = {x = 10, y = 13}, 
                scale={x = 1.1, y = 1.1}, 
                angle = 1
            }, 
            name = {
                color = { r =255, g = 255, b =153},
                position = {x = 10, y = -18}, 
                scale={x = 0.6, y = 0.8}, 
                font_size = 13,
                angle = 1
            }
        }
    end ,
    [2] = function (  )
        return {
            eff = {
                position = {x = 17, y = 17}, 
                scale={x = 1.5, y = 1.5}, 
                angle = 1
            }, 
            name = {
                color = { r =255, g = 255, b =153},
                position = {x = 15, y = -18}, 
                scale={x = 0.8, y = 0.8}, 
                font_size = 13,
                angle = 1
            }
        }
    end ,
    [3] = function (  )
        return {
            eff = {
                position = {x = 17, y = 17}, 
                scale={x = 1.5, y = 1.5}, 
                angle = 1,
            }, 
            name = {
                color = { r =255, g = 255, b =153},
                position = {x = 15, y = -18}, 
                scale={x = 0.8, y = 0.8}, 
                font_size = 13,
                angle = 1
            }
        }
    end ,
}

TerrBattleTarget = {
    kRobber = 1,
    kWarfareNpc = 2,
    kTower = 3,
    kLegionAboss = 4,
}

function This:colorCondText( node, index, value, text1, text2  )
    node["condition" .. index]:setStringWithColor( "<A5>" .. text1 .. "<A0>" .. (value and " : <A1>" or " : <R0>") .. text2)
    node["condition" .. index]:setVisible( true )
    node["goto" .. index]:setVisible( not value )
    node["unfinish" .. index]:setVisible( not value )
    node["finish" .. index]:setVisible( value )
end

function This:hideCondText( node, index )
    if not index then
        index = 1
        node.condition:setVisible( false )
        node.bg4:setVisible( false )
    end
    node["condition" .. index]:setVisible( false )
    node["unfinish" .. index]:setVisible( false )
    node["finish" .. index]:setVisible( false )
    node["goto" .. index]:setVisible( false )
end

function This:reload( name )
    local reload = require "reload"
    -- reload.print = print_tmp
    local co = coroutine.create(function ()
        reload.reload { name }
    end)
    coroutine.resume(co)
end

function This:find2Realod( moduleName )
    local nameList = {}
    for name, _ in pairs(package.loaded) do
        local pos = string.find(name, moduleName)
        if pos and pos > 0 then
            table.insert(nameList, name)
        end
    end
    print_tmp(nameList)
    if #nameList > 0 then
        local reload = require "reload"
        local co = coroutine.create(function ()
            reload.reload(nameList)
        end)
        coroutine.resume(co)
    end
end

function This:isObjectInvalid(obj)
    return obj and obj[0] ~= 0
end

if g_setting.isDebug then
    function table2json(t)  
        local function serialize(tbl)  
                local tmp = {}  
                for k, v in pairs(tbl) do  
                        local k_type = type(k)  
                        local v_type = type(v)  
                        local key = (k_type == "string" and "\"" .. k .. "\":")  
                            or (k_type == "number" and "")  
                        local value = (v_type == "table" and serialize(v))  
                            or (v_type == "boolean" and tostring(v))  
                            or (v_type == "string" and "\"" .. v .. "\"")  
                            or (v_type == "number" and v)  
                        tmp[#tmp + 1] = key and value and tostring(key) .. tostring(value) or nil  
                end  
                if table.maxn(tbl) == 0 then  
                        return "{" .. table.concat(tmp, ",") .. "}"  
                else  
                        return "[" .. table.concat(tmp, ",") .. "]"  
                end  
        end  
        assert(type(t) == "table")  
        return serialize(t)  
    end

	function print_r ( t ) 
	    local print_r_cache={}
        local function sub_print_r(t,indent)
            if (print_r_cache[tostring(t)]) then
                print(indent.."*"..tostring(t))
            else
                print_r_cache[tostring(t)]=true
                if (type(t)=="table") then
                    local tLen = #t
                    for i = 1, tLen do
                        local val = t[i]
                        if (type(val)=="table") then
                            print(indent.."#["..i.."] => "..tostring(t).." {")
                            sub_print_r(val,indent..string.rep(" ",string.len(i)+8))
                            print(indent..string.rep(" ",string.len(i)+6).."}")
                        elseif (type(val)=="string") then
                            print(indent.."#["..i..'] => "'..val..'"')
                        else
                            print(indent.."#["..i.."] => "..tostring(val))
                        end
                    end
                    for pos,val in pairs(t) do
                        if type(pos) ~= "number" or math.floor(pos) ~= pos or (pos < 1 or pos > tLen) then
                            if (type(val)=="table") then
                                print(indent.."["..pos.."] => "..tostring(t).." {")
                                sub_print_r(val,indent..string.rep(" ",string.len(pos)+8))
                                print(indent..string.rep(" ",string.len(pos)+6).."}")
                            elseif (type(val)=="string") then
                                print(indent.."["..pos..'] => "'..val..'"')
                            else
                                print(indent.."["..pos.."] => "..tostring(val))
                            end
                        end
                    end
                else
                    print(indent..tostring(t))
                end
            end
        end

        if This.open_debug or This.allways_debug then
            if (type(t)=="table") then
                print(tostring(t).." {")
                sub_print_r(t,"  ")
                print("}")
            else
                sub_print_r(t,"  ")
            end
        end

    end
    
    function print_tmp( ... )
        This.open_debug = true
        print_r( {...} )
        This.open_debug = false
    end

    function print_keys( t )
        This.open_debug = true
        if (type(t)=="table") then
            print_r(table.getKeyList(t))
        else
            print_r(t)
        end
        This.open_debug = false
    end
else
    function table2json()
    end

    function print_r()
    end

    function print_tmp()
    end

    function print_keys()
    end
end
