-- @Author: superm
-- @Date:   2017-10-21 10:51:06
-- @Last Modified by:   superm
-- @Last Modified time: 2018-06-23 17:14:43
-- @File Name: TerritoryManager.lua
-- @Description: 封地总管理类, 建筑和科技涉及了其他模块进行了拆分， 其余均由具体建筑管理

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local BattleDataManager = require("Battle.BattleDataManager")
local GameStageManager = require("Stage.GameStageManager")
local HeroManager = require("Hero.HeroManager")
local Hero = require("Hero.Hero")
local TerrRobber = require("Territory.TerrRobber")
local TerrBossRobber = require("Territory.TerrBossRobber")
local TerrUtil = require("Territory.TerrUtil")
local TerrBuildManager = require("Territory.TerrBuildManager")
local TerrTechnManager = require("Territory.TerrTechnManager")

function This:init()
	super.init(self)

    self.m_dataRefreshTime = {}
    self.m_dataRefreshTimes = {}

    TerrBuildManager:init()
    TerrTechnManager:init()

    self.m_robbers = {}
    self.m_bossRobbers={}
    self.m_yards = {}

    self.m_emBattleHeros = {}
    self.m_robberInBattle = nil
    self.m_bossLeftRewardTime = 0

    self:initListener()
    self.m_inited = true
end

function This:initListener()
    EventDispatcher:addListener(self,11515, self.onGetRobbersInfoReply)
    EventDispatcher:addListener(self,11516, self.onFightRobbersReply)
    EventDispatcher:addListener(self,10704, self.onSkipBattleReply)
end

function This:removeListener()
    EventDispatcher:removeListener(self,11515)
    EventDispatcher:removeListener(self,11516)
    EventDispatcher:removeListener(self,10704)
end

--贼寇
function This:getAllRobbers()
    return self.m_robbers
end

function This:getAllBossRobbers()
    return self.m_bossRobbers
end

function This:getBossLeftRewardTimes()
    return self.m_bossLeftRewardTime
end

function This:addRobber( info )
    local robber = self:getRobberBy(info.pos_id, info.id)
    if not robber then
        robber = TerrRobber:create(info)
        self.m_robbers[info.pos_id] = robber
    else
        robber:updateInfo(info, true)
    end
    return info.pos_id, robber
end

function This:addBossRobber(info)
    local bossRobber = self:getBossRobberBy(info.pos_id,info.id)
    if not bossRobber then
        bossRobber = TerrBossRobber:create(info)
        bossRobber.name = "boss"
        self.m_bossRobbers[info.pos_id] = bossRobber
    else
        bossRobber:updateInfo(info, true)
    end
end

function This:getRobberBy( posId, robberId )
    if not posId and not robberId then
        return next(self.m_robbers)
    end

    local robber = self.m_robbers[posId]
    if ( robber ) and ( not robberId or robber:getRobberId() == robberId ) then
        return robber
    elseif robber and robberId then
        self.m_robbers[posId] = nil
    else
        return nil
    end
end

function This:getBossRobberBy(posId, robberId)
    if not posId and not robberId then
        return next(self.m_bossRobbers)
    end
    local bossRobber = self.m_bossRobbers[posId]
    if ( bossRobber ) and ( not robberId or bossRobber:getRobberId() == robberId ) then
        return bossRobber
    elseif bossRobber and robberId then
        self.m_bossRobbers[posId] = nil
    else
        return nil
    end
end

function This:onGetRobbersInfoRequest()
    Socket:sendAndLock(11515)
end

function This:onGetRobbersInfoReply(msg)
    if msg then
--贼寇
        local posIndexList = {}
        local bossPosIndexList = {}
        for pos, robber in pairs(self.m_robbers) do
            local found = table.find(msg.zeikou, function ( index, info )
                return info.pos_id == pos and info.id ~= 0
            end)

            if not found then
                robber:DeleteMe()
                table.insert(posIndexList, pos)
            end
        end

        for _, pos in ipairs(posIndexList) do
            self.m_robbers[pos] = nil
        end

        for _, info in ipairs(msg.zeikou) do
            if info and info.id and info.id > 0 then
                self:addRobber(info)
            end
        end
-- BOSS
        for pos, bossRobber in pairs(self.m_bossRobbers) do
            local found = table.find(msg.boss_list, function ( index, info )
                return info.pos_id == pos and info.id ~= 0
            end)

            if not found then
                bossRobber:DeleteMe()
                table.insert(bossPosIndexList, pos)
            end
        end

        for _, pos in ipairs(bossPosIndexList) do
            self.m_bossRobbers[pos] = nil
        end

        for _, info in ipairs(msg.boss_list) do
            if info and info.id and info.id > 0 then
                self:addBossRobber(info)
            end
        end
        self.m_bossLeftRewardTime = msg.reward_times

        self:notifyObservers("onGetRobbersInfo",self.m_bossRobbers)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end


function This:getEmbattleHero()
    return self.m_emBattleHeros
end

function This:isInEmbattle(heroId)
    for pos , id in pairs(self.m_emBattleHeros) do
        if id == heroId then
            return true
        end
    end
    return false
end

function This:createHeroForEmbattle()
    local posTable = {}
    local heroTable = {}
    for pos , heroId in pairs(self.m_emBattleHeros) do
        local playerHero = HeroManager:getHero(heroId)
        local hero = Hero:create({
            heroId = heroId,
            level = playerHero:getLevel(),
            starLv = playerHero:getStarLv(),
            quality = playerHero:getQuality(),
            deputy = table.clone(playerHero:getDeputyWithPos())
        })
        posTable[pos] = hero
        heroTable[heroId] = hero
    end
    for heroId , hero in pairs(heroTable) do
        hero:addFetterWithHero(heroTable)
    end
    return posTable
end

function This:onFightRobbersRequest(heros, robberId)
    if #heros > 0 then
        local result = {0,0,0,0,0,0}
        for i , info in pairs(heros) do
            local hero = HeroManager:getHero(info.heroId)
            result[info.pos] = hero:getUid()
        end
        Socket:sendAndLock(11516, {card_list = result, pos_id = robberId})
    else
        g_scene:showTips(Serialize["add_embattle_hero"])
    end
end

function This:setRobberCopy(object)
    if object then
        self.m_robberInBattle = object
    end
    return self.m_robberInBattle
end

function This:hasRobberCopy()
    return self.m_robberInBattle ~= nil
end

function This:onFightRobbersReply(message)
    if message.err_code == 0 then
        self.m_emBattleHeros = {}
        local heros = HeroManager.m_heros
        for i , hero in pairs(heros) do
            -- print("hero = ",hero:getUid())
        end

        local count = 0
        for i , hero in pairs(HeroManager.m_heroUid) do
            -- print("hero uid = ", i)
            count = count + 1
        end
        -- print("count = ", count)

        for i , uid in ipairs(message.card_list) do
            -- print("uid = ",uid)
            if uid > 0 then
                local uhero = HeroManager:getHeroByUid(uid)
                self.m_emBattleHeros[i] = uhero:getId()
            end
        end

        if self.m_robberInBattle and self.m_robberInBattle:getId() == message.pos_id then
            -- self.m_robbers[ message.pos_id ] = nil
            self.m_robberInBattle:updateInfo(message, false)
        end
        self:notifyObservers("onFightRobbers", message.pos_id)
    else
        g_scene:handleErrorCode(message.err_code)
    end
end

function This:onSkipBattleRequest()
    if self.m_robberInBattle then
        GameStageManager:setEnterRobber(true)
        Socket:sendAndLock(10704,{})
    end
end

function This:onSkipBattleReply(msg)
    if self.m_robberInBattle and GameStageManager:isEnterRobber() then
        if msg.err_code == 0 then
            local report = self.m_robberInBattle:getReportInfo()
            if report and self.m_robberInBattle:isAttackerVictory() then
                self:onBattleVictory()
            else
                self:onBattleFailed()
            end
            BattleDataManager:clear()
        else
            g_scene:handleErrorCode(msg.err_code)
        end
    end
end

function This:onBattleSkip(result)
    self:onSkipBattleRequest()
end

function This:onBattleVictory()
    g_scene:openDialog("Stage.StageVictoryPanel",{
        stageId = self.m_robberInBattle:getBattleId(),
        isRobber = true,
        dropInfo = self.m_robberInBattle:getRobberReward(),
        callback = function ()
            if self.m_robberInBattle then
                local posId = self.m_robberInBattle:getId()
                self.m_robberInBattle:DeleteMe()
                if self.m_robbers[posId] then
                    self.m_robbers[posId] = nil
                end
                self.m_robberInBattle = nil
            end
            BattleDataManager:removeObserver(self)
            GameStageManager:setEnterRobber(false)
            -- self:onGetRobbersInfoRequest()
        end
    })
    BattleDataManager:removeObserver(self)
end

function This:onBattleFailed()
    g_scene:openDialog("Stage.StageFailedPanel", {
        isRobber = true,
        callback = function ()
            BattleDataManager:removeObserver(self)
            GameStageManager:setEnterRobber(false)
            -- self:onGetRobbersInfoRequest()
        end
    })
end
