
-- 君王殿排行榜数据

local This, super = class(...,"Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")
local LoginManager = require("Login.LoginManager")


function This:init()
	super.init(self)

	self.m_requestServerId = curServerId
	self.m_rankList = {}
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 11600, self.onThroneRankInfo)
	EventDispatcher:addListener(self,"day_passed_withoutnet", self.onDayPassNotify)
end

function This:removeListener()
	EventDispatcher:removeListener(self,11600)
	EventDispatcher:removeListener(self,"day_passed_withoutnet")

end

function This:getRankList()
	return self.m_rankList
end

function This:onThroneRankInfoRequest()
	local server = LoginManager:getLoginServer()
	Socket:sendAndLock(11600, {start_index = 1, end_index= 30, server_id = server:getId(), type = RankType.Office})
end

function This:onThroneRankInfo(msg)
	local ThroneRankItemData = require("Throne.ThroneRankItemData")
	self.m_rankList = {}

	for rankNum, info in ipairs(msg.rank_list) do
		local matchLv = math.floor(info.rank_first_key / 10000)
		if matchLv < 25 then
			break
		end

		local item = ThroneRankItemData:create(rankNum, info)
		table.insert(self.m_rankList, item)
	end
	
	self:notifyObservers("onThroneRankInfo", self.m_rankList)
end

function This:checkOpen()
	local needDay = ArgConfig["throne_open_time"].arg[1]
	local curDay = GamePlayer:getOpenServerDays()
	return curDay >= needDay
end

function This:onDayPassNotify()
	self:notifyObservers("onDayPassNotify")
end

function This:getOpenTips()
	local needDay = ArgConfig["throne_open_time"].arg[1]
	return string.format("开服第%d天开启", needDay)
end