
local This, super = class(..., "NodeUI")
local GameUtil = require("Util.GameUtil")
local ThroneManager = require("Throne.ThroneManager")
local ThroneRankItemView = require("Throne.ThroneRankItemView")

function This:initData(info)

end

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/KingPalaceKing.csb")
    self:addChild(self.m_csNode)
    --self.m_csNode:setToCenter()

    self.m_rankList = {{},{},{},{},{},{}}
end

function This:refreshRankByIndex(index)
    local ScrollView = self.m_csNode.ScrollView
    self.m_roleItemList = {}
    ScrollView:removeAllItems()
    -- ScrollView:setAnchorPoint(0.5,0.5)
    local rankList = self.m_rankList[index]
    local num,data = 1
    for rankNum,rankData in ipairs(rankList) do
        if num == 1 then
            data = rankData
        end
        local item = ThroneRankItemView:create(rankData)
        item:setClick(function()
            self:selectRoleByData(index,rankData)
            for _,roleItem in pairs(self.m_roleItemList) do
                roleItem:showLight(roleItem:getRankNum() == rankData:getRankNum())
            end
        end)
        local w,h = item:getSize()
        ScrollView:pushBackCustomItem(item,w,h-5,0.5)
        self.m_roleItemList[num] = item
        num = num + 1

    end
    self:selectRoleByData(index,data)
    local roleItem = self.m_roleItemList[1]
    if roleItem then
        roleItem:showLight(roleItem:getRankNum() == (index-1)*5+1)
    end
end

function This:selectRoleByData(index,data)
    local node = self.m_csNode
    if data then
        GameUtil:setPlayerFeltNodeResource(node.renwutu, data:getIcon())
        node.name:setString(data:getName())
        self.m_selectData = data
    else
        node.name:setString(Serialize[64656])
    end
end

function This:refresh()
    local node = self.m_csNode
    for i = 1,6 do
        node["dian"..i].light:setVisible(i == self.m_index)
    end
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_inited = true

        local csNode = self.m_csNode

        -- csNode.close:setTouchEnded(function()
        --     self:close()
        -- end)
       
        csNode.jiantou1:setTouchEnded(function()
            local rankList = self.m_rankList[self.m_index-1]
            if rankList and #rankList == 0 then
                g_scene:showTips(Serialize[64657])
                return 
            end
            if self.m_index > 1 then
                self.m_index = self.m_index - 1
                self:refreshRankByIndex(self.m_index)
                self:refresh()
            else
                return
            end
            
        end)

        csNode.jiantou2:setTouchEnded(function()
            local rankList = self.m_rankList[self.m_index+1]
            if rankList and #rankList == 0 then
                g_scene:showTips(Serialize[64657])
                return 
            end
            if self.m_index < 6 then
                self.m_index = self.m_index + 1
                self:refreshRankByIndex(self.m_index)
                self:refresh()
            else
                return
            end
            
        end)

        csNode.xiangqingdi:setTouchEnded(function()
            local friendPanel = g_scene:getDialog("RoleInfoPanel")
            if friendPanel then
                g_scene:showTips(Serialize[64658])
                return
            end
            if self.m_selectData then
                g_scene:openDialog("Player.RoleInfoPanel",{id =self.m_selectData:getRoleId()})
            else
                g_scene:showTips(Serialize[64659])
            end
            
        end)

        csNode.wenhao:setTouchEnded(function()
            g_scene:openDialog("UI.HelpTips",{text = Serialize["throne_tips"],node=csNode.wenhao})
            
            
        end)
    end
end

function This:onEnter()
    super.onEnter(self)
    ThroneManager:onThroneRankInfoRequest()
    ThroneManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    ThroneManager:removeObserver(self)
    super.onExit(self)
end

function This:onThroneRankInfo(ranlist)
    local rankList = ThroneManager:getRankList()
    self.m_index = 1
    for rankNum,rankData in ipairs(rankList) do
        table.insert(self.m_rankList[math.ceil(rankNum/5)],rankData )
    end

    self:refreshRankByIndex(self.m_index)
    self:refresh()
end