local This = class(...)
local pairs = pairs

function This:create(owner)
    local obj = {
        m_timers = {},
        m_keyIndex = 1,
        m_count = 0,
        m_isRunning = true,
        m_owner = owner,
        getTimer = self.getTimer,
        setTimer = self.setTimer,
        update = self.update,
        removeTimer = self.removeTimer,
        removeAllTimer = self.removeAllTimer,
        callTimer = self.callTimer,
    }
    return obj
end

function This:getTimer(key)
    return self.m_timers[key]
end

function This:setOrderTimer(delayTime, listener, key, triggerTotal)
    assert(key)
    for i , timer in ipairs(m_timerList) do
        if key == timer.key then
            timer[1] = 0
            timer[2] = delayTime
            timer[3] = 0
            timer[4] = triggerTotal or 1
            timer[5] = listener 
            timer.key = key
            return timer
        end
    end
    local timer = {0, delayTime, 0, triggerTotal or 1, listener}
    table.insert(self.m_timerList,1,timer)
    self.m_count = self.m_count + 1
    return timer
end


function This:setTimer(delayTime, listener, key, triggerTotal)
    local timer = nil
    if key then
        timer = self.m_timers[key]
    else
        key = self.m_keyIndex
        self.m_keyIndex = self.m_keyIndex + 1
    end
    if not timer then
        timer = {0, delayTime, 0, triggerTotal or 1, listener}
        self.m_timers[key] = timer
        self.m_count = self.m_count + 1
    else
        timer[1] = 0
        timer[2] = delayTime
        timer[3] = 0
        timer[4] = triggerTotal or 1
        timer[5] = listener
    end
    return timer
end

function This:update(delta)
    if not self.m_isRunning or self.m_count == 0 then
        return
    end

    for key, timer in pairs(self.m_timers) do
		local lastPassTime = timer[1]
        local passTime = timer[1]
        passTime = passTime  + delta
        timer[1] = passTime
        if lastPassTime > 0 and passTime >= timer[2] then
            timer[3] = timer[3] + 1
            timer[1] = 0
            if timer[4] > 0 and timer[4] >= timer[3] then
                self.m_timers[key] = nil
				self.m_count = self.m_count - 1
            end

            --保护调用
            local ret, err = pcall(function ()
                timer[5](self.m_owner)
            end)
            if not ret then
                logError(err)
            end
        end
    end
end

function This:removeTimer(key)
    if self.m_timers[key] then
        self.m_timers[key] = nil
        self.m_count = self.m_count - 1
    end
end

function This:removeAllTimer()
    self.m_timers = {}
    self.m_count = 0
    self.m_keyIndex = 1
end

function This:callTimer(key)
    local timer = self.m_timers[key]
    if timer then
        timer.passTime = timer.time
    end
end