-- @Last Modified time: 2018-08-09 11:23:22
-- @File Name: CostConfirmPanel.lua
-- @Description: 

local This , super = class(...,"Dialog")
local ItemUtil = require("Bag.ItemUtil")
local GameUtil = require("Util.GameUtil")

local mapCostNoTipsValue = {}

function This:initNode(info)
	if not info or not info.fileName then
		self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/QuestConfirmUI.csb")
	else
		self.m_csNode = cc.Node:createFromCSFile(info.fileName)
	end

    if self.m_csNode.time then
	    self.m_csNode.time:setVisible(false)
    end
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()
	if info then
		if info.cost then
			self:setCost(tostring(info.cost), info.costId)
		end
	    if info.simple then
	        self:setSimpleString(info.simple, info.width)
	    end
	    if info.desc then
	        self:setString(info.desc)
	    elseif info.descText then
	        self:setDescString(info.descText)
	    end

	    if info.note then
	        self:setTimeString(info.note)
	    end
	    
	    if info.okHandler then
	    	self:setOkHandler(info.okHandler)
	    end

	    if info.cancelHandler then
	    	self:setCancelHandler(info.cancelHandler)
	    end

	    if info.notips then
	    	self:setNoTipsString(info.notips)
	    end
	    if info.itemId then
	    	self.m_csNode.icon:setImage(ItemUtil.getIconImage(info.itemId))
	    end
		if info.text then
			self.m_csNode.text2:setString(info.text)
		end
		if info.icon and info.icon ~= "" then
			self.m_csNode.icon:setImage(info.icon)
		end
    else
		self.m_text = ""
	end
end

function This:onEnter()
	if self:existNoTipsKey(self.m_noTipsKey) then
    	if self.m_okHandler then
	        self.m_okHandler(true)
        end	
		self:close()
	end 
	super.onEnter(self)
end

function This:existNoTipsKey(str)
    if mapCostNoTipsValue[str] then
    	return true
    end
end

function This:setString(str, width)
	self.m_text = str
	self.m_csNode.text2:setString(str, width)
	GameUtil:linearLayoutNodes(3, {self.m_csNode.text1, self.m_csNode.cost, self.m_csNode.icon, self.m_csNode.text2}, true, self.m_csNode.bg:getContentWidth())
end

function This:setDescString(str, width)
	self.m_text = str
	self.m_csNode.desc:setString(str, width)
	GameUtil:linearLayoutNodes(3, {self.m_csNode.text1, self.m_csNode.cost, self.m_csNode.icon}, true, self.m_csNode.bg:getContentWidth())
end

function This:setSimpleString(str, width)
	self.m_text = str
    self.m_csNode.icon:setVisible(false) 
    self.m_csNode.text1:setVisible(false) 
    self.m_csNode.text2:setVisible(false) 
	self.m_csNode.cost:setVisible(false) 
	self.m_csNode.desc:setString(str, width)
end

function This:setTimeString(str, color)
	self.m_csNode.time:setString(str)
	self.m_csNode.time:setVisible(true)
	if color then
		self.m_csNode.time:setColor(color.r or 0, color.g or 0, color.b or 0)
	end
end

function This:setNoTipsString(key)
    self.m_csNode.gou:setVisible(false) 
    self.m_csNode.kuang:setVisible(true) 
	self.m_csNode.time:setString(Serialize[64670])
	self.m_csNode.time:setVisible(true)
	self.m_noTipsKey = key
	self.m_noTipsValue = false
end

function This:setCost(num, itemId)
	self.m_csNode.cost:setString("" .. num)
	if itemId then
		self.m_csNode.icon:setImage(ItemUtil.getIconImage(itemId))
	end
end

function This:setHandler(okHandler, cancelHandler)
    self.m_okHandler = okHandler
    self.m_cancelHandler = cancelHandler
end

function This:setCancelHandler(handler)
	self.m_cancelHandler = handler
end

function This:setOkHandler(handler)
	self.m_okHandler = handler
end

function This:setBtnString(cancel,ok)
	self.m_csNode.text2:setString(cancel)
	self.m_csNode.text1:setString(ok)
end

function This:setButtonGray(cancel, ok)
	if cancel then
		self.m_csNode.cancelButton:setGrayShader()
	end

	if ok then
		self.m_csNode.confirmButton:setGrayShader() 
	end
end

function This:setButtonNormal(cancel, ok)
	if cancel then
		self.m_csNode.cancelButton:clearShader()
	end

	if ok then
		self.m_csNode.confirmButton:clearShader() 
	end
end

function This:initTouchEvent()
	self.m_csNode.cancelButton:setTouchEnded(function()
		if self.m_cancelHandler then
			self.m_cancelHandler()
		end
		self:close()
	end)

	self.m_csNode.confirmButton:setTouchEnded(function()
		if self.m_okHandler then
			self.m_okHandler()
		end	
		if self.m_noTipsValue then
			mapCostNoTipsValue[self.m_noTipsKey] = true
		end
		self:close()
	end)

	if self.m_csNode.kuang then
		self.m_csNode.kuang:setTouchEnded(function()
	        local visible = self.m_csNode.gou:isVisible()
	        self.m_csNode.gou:setVisible(not visible)
	        self.m_noTipsValue = not visible
		end)
	end
end

-- function This:onTouchMask()
-- 	self:close()
-- end