local This, super = class(..., "cc.Node")
local GameUtil = require("Util.GameUtil")
local ItemUtil = require("Bag.ItemUtil")
local DepotManager = require("Depot.DepotManager")
local GamePlayer = require("Player.GamePlayer")
local LordEquipManager = require("Awake.LordEquipManager")
local TacticsConfig = require("Metadatas.TacticsConfig")
local ArgConfig = require("Metadatas.ArgConfig")
local ccSprite = cc.Sprite
local ccLabel = cc.Label

local scaleFactor = 1.0

function This:init(id, count, isCompleteCount, isTouchable,isShowGray)
    self.m_isTouchable = true
	if isTouchable~=nil then
		self.m_isTouchable = isTouchable
	end
	self.m_isShowGray = isShowGray
    if self.m_id == id and self.m_count == count then
        return
    end
	self.m_id = id
	self.m_count = count
	self.m_isCompleteCount = isCompleteCount
	self.m_itemType = ItemUtil.getType(id)
	self:initItem(id, count, isCompleteCount)
end

function This:setModel(info)
	if info and info.id then
		self.m_id = info.id
		self.m_count = info.num
		self:initItem(self.m_id, self.m_count)
	end
end

function This:initItem(id, count, isCompleteCount)
	self.m_id = id 
	self.m_count = count
	self.m_itemType = ItemUtil.getType(id)
	local qualityBg = self.m_qualityBg
	if not qualityBg then
		qualityBg = ccSprite:create(ItemUtil.getQualityImage(id))
		self:addChild(qualityBg)
		self.m_qualityBg = qualityBg
	else
		qualityBg:setImage(ItemUtil.getQualityImage(id))
	end

	if self.m_isShowGray then
		GameUtil:btnGray(qualityBg,true)
	end

	local w , h = qualityBg:getContentSize()
	w , h = w * scaleFactor,  h * scaleFactor
	self:setContentSize(w, h)
	qualityBg:setPosition(w/2, h/2)

	if not self.m_icon then
		self.m_icon = ccSprite:create(ItemUtil.getIconImage(id))
		self:addChild(self.m_icon)
	else
		self.m_icon:setImage(ItemUtil.getIconImage(id))
	end
	self.m_icon:setPosition(w/2, h/2)

	if self.m_isShowGray then
		GameUtil:btnGray(self.m_icon,true)
	end

	if count then
		if not self.m_menban then
			self.m_menban = ccSprite:create("Common/daojushuliangdi.png")
			self:addChild(self.m_menban, 0)
			self.m_menban:setPosition(w/2, 18)
		end

		local str = "X".. tostring(count)
		if not isCompleteCount then
			str = ItemUtil.formatItemCount(count)
		end

		local countLabel = self.m_countLabel
		if not countLabel then
			countLabel = ccLabel:createNormalTTF(str, 18)
			countLabel:setAnchorPoint(1.0,0)
			countLabel:enableOutline(26, 26, 26, 255, 1)
			self:addChild(countLabel, 2)
			self.m_countLabel = countLabel
		else
			countLabel:setString(str)
		end
		countLabel:setPosition(w-8,5)
	end

	self:setTacticsViewVisible(false)
	
	if self.m_itemType == ItemCommonType.kSpall then
		self:setToSpall()
	elseif self.m_itemType == ItemCommonType.kTactics then
		self:setToTactics()
	else
		self:setToNormal()
	end
	self:setCascadeOpacityEnabled(true)
	self:refreshQualityEffect()
end

function This:onEnter()
	super.onEnter(self)

	--检测铭文
	local isInscription = LordEquipManager:isPerfectItemId(self.m_id)
	if isInscription then
		self:setTouch({
	        onEnded = function()            
	        	g_scene:openDialog("Awake.LordEquipMasterNameplate",{ node= self, itemId = self.m_id, itemType = self.m_itemType})
	        end,
	        isScale = false,
	        isDynamic = true,
			isSwallow = false,
	    })
	else
		if self.m_isTouchable and not self.m_touch then
		    g_scene:handleHelpTips(self, function ()
		    	return {fileName = "UI.ItemTips", itemId = self.m_id, itemType = self.m_itemType}
		    end)
		end
	end

    self:refreshQualityEffect()
end

function This:onExit()
    if self.m_effectQuality then
    	self.m_effectQuality:unscheduleUpdate()
        -- self.m_effectQuality:removeFromParent()
        -- self.m_effectQuality = nil
    end
    super.onExit(self)
end

function This:setTouchEnable(isEnable)
	if self.m_touch and not isEnable then
		self:removeEventListener(self.m_touch)
	elseif not self.m_touch and isEnable then
		g_scene:handleHelpTips(self, function ()
	    	return {fileName = "UI.ItemTips", itemId = self.m_id, itemType = self.m_itemType}
	    end)
	end
    self.m_isTouchable = isEnable
end


function This:setToNormal()
	if self.m_strengthLvLabel then
		self.m_strengthLvLabel:setVisible(false)
	end
	if self.m_spallIcon then
		self.m_spallIcon:setVisible(false)
	end
end

function This:showNeedNum(num)
	local total = 0
	local info = {itemUid = 0}
	total = DepotManager:getItemOrResCount(self.m_id, info)
	num = num or self.m_count
	local enough = total >= num
	local format = "%d<A0>/%d"
	local temNum,temTotal = num,total
	if self.m_isCompleteCount then
		format = "%s<A0>/%s"
		total = ItemUtil.formatItemCount(total)
		num = ItemUtil.formatItemCount(num)
	end
	self.m_countLabel:setStringWithColor(string.format( (enough and "<A0>" or "<R0>") .. format, total, num))
	self.m_countLabel:setScale(0.8)
	return temTotal, temNum, info.itemUid
end

-- countStr 里面的数量需要通过str = ItemUtil.formatItemCount(count) 处理
function This:setCustomCount( countStr,isShowCountBg )
	if self.m_countLabel and countStr then
		self.m_countLabel:setString(countStr)
		self.m_countLabel:setScale(0.8)
		local isVisible = false
		if isNotShowCountBg then
			isVisible = true
		end
		self.m_menban:setVisible(isVisible)
	end
end

local function showFabaoExtraViews(fabaoExtraViews, bool)
	fabaoExtraViews.starView:setVisible(bool)
	fabaoExtraViews.levelView:setVisible(bool)
	fabaoExtraViews.refineDegreeView:setVisible(bool)
	fabaoExtraViews.starLevelView:setVisible(bool)
end

function This:setToSpall()
	if self.m_strengthLvLabel then
		self.m_strengthLvLabel:setVisible(false)
	end
	if self.m_fabaoExtraViews then
		showFabaoExtraViews(self.m_fabaoExtraViews, false)
	end
	if not self.m_spallIcon then
		local w , h = self.m_qualityBg:getContentSize()
		h = h * scaleFactor
		local spallIcon = ccSprite:create("GameScene/equipSpall.png")
		spallIcon:setAnchorPoint(0, 1.0)
		spallIcon:setPosition(5, h - 5)
		self:addChild(spallIcon)
		self.m_spallIcon = spallIcon
	else
		self.m_spallIcon:setVisible(true)
	end 
end

function This:setToEquip(lv,level)
	if self.m_spallIcon then
		self.m_spallIcon:setVisible(false)
	end
	if self.m_fabaoExtraViews then
		showFabaoExtraViews(self.m_fabaoExtraViews, false)
	end
	local w , h = self:getContentSize()
	local str = string.format("+%d",lv)
	if not self.m_strengthLvLabel then
		local lvLabel = ccLabel:createNormalTTF(str, 16)
		lvLabel:setAnchorPoint(1.0,1.0)
		local w , h = self:getContentSize()
		lvLabel:setPosition(w-10 ,h-6)
		self:addChild(lvLabel)
		self.m_strengthLvLabel = lvLabel
	else
		self.m_strengthLvLabel:setString(str)
		self.m_strengthLvLabel:setVisible(true)
	end
	str = string.format("%d",level)
	if not self.m_upLevelLable then
		local levelView = ccLabel:createNormalTTF(str, 16)
		levelView:setAnchorPoint(1.0, 0)
		levelView:setPosition(w-10, 10)
		self:addChild(levelView)
		self.m_upLevelLable = levelView
	else
		self.m_upLevelLable:setString(str)
		self.m_upLevelLable:setVisible(true)
	end
	self.m_strengthLv = lv
	self.m_upLevel = level
end

function This:setToFabao(level, refineDegree, starLevel)
	if self.m_spallIcon then
		self.m_spallIcon:setVisible(false)
	end
	if self.m_strengthLvLabel then
		self.m_strengthLvLabel:setVisible(false)
	end

	if not self.m_fabaoExtraViews then
		self.m_fabaoExtraViews = {}
		local w , h = self:getContentSize()
		-- 星星图标
		local starView = ccSprite:create("MenuScene/star_2.png")
		starView:setAnchorPoint(1.0, 1.0)
		starView:setPosition(w-6 ,h-6)
		starView:setScale(0.3)
		self:addChild(starView)
		self.m_fabaoExtraViews.starView = starView
		-- 星级
		local starLevelView = ccLabel:createNormalTTF("", 16)
		starLevelView:setAnchorPoint(1.0, 1.0)
		starLevelView:setPosition(w-20, h-2.5)
		self:addChild(starLevelView)
		self.m_fabaoExtraViews.starLevelView = starLevelView
		-- 等级
		local levelView = ccLabel:createNormalTTF("", 16)
		levelView:setAnchorPoint(1.0, 0)
		levelView:setPosition(w-10, 10)
		self:addChild(levelView)
		self.m_fabaoExtraViews.levelView = levelView
		-- 精炼程度
		local refineDegreeView = ccLabel:createNormalTTF("", 16)
		refineDegreeView:setAnchorPoint(0, 0)
		refineDegreeView:setPosition(10, 10)
		self:addChild(refineDegreeView)
		self.m_fabaoExtraViews.refineDegreeView = refineDegreeView
	end
	self.m_fabaoExtraViews.levelView:setStringWithColor("<FabaoLevel>"..level)
	self.m_fabaoExtraViews.refineDegreeView:setStringWithColor("<FabaoRefine>+"..refineDegree)
	self.m_fabaoExtraViews.starLevelView:setStringWithColor("<FabaoStar>"..starLevel)
	showFabaoExtraViews(self.m_fabaoExtraViews, true)
end

function This:refreshQualityEffect(  )
	local hasEffect = self:getQuality() > 4 -- 橙色和红色品质道具有特效
	local isIniting = false
	if hasEffect and not self.m_effectQuality then
		if self.m_itemType == ItemCommonType.kFaBao then
			self.m_effectQuality = GameUtil:createEffect("FaBaoEffect", true)
			local x , y = self.m_qualityBg:getPosition()
			self.m_effectQuality:setPosition(x, y)
		else
			self.m_effectQuality = GameUtil:createEffect("Itemnb", true)
			local x , y = self.m_qualityBg:getPosition()
			self.m_effectQuality:setPosition(x, y + 11)
		end
        self:addChild(self.m_effectQuality)
        isIniting = true
    end


    if not hasEffect and self.m_effectQuality then
        self.m_effectQuality:removeFromParent()
        self.m_effectQuality = nil
    end

	if self.m_effectQuality and not isIniting then
		self.m_effectQuality:scheduleUpdate()
	end
end

function This:showName()
    if not self.m_nameLabel then
		local w , h = self:getContentSize()
        local label = ccLabel:createNormalTTF(ItemUtil.getName(self.m_id, 24))
        label:setPosition(w/2,h/2)
        self:addChild(label)
        self.m_nameLabel = label
    else
        self.m_nameLabel:setString(ItemUtil.getName(self.m_id))
    end
	self.m_nameLabel:setColorByID("Q".. item:getQuality())
end

function This:getItemName()
	return ItemUtil.getName(self.m_id)
end


function This:getItemCount()
	local info = {itemUid = 0}
	local total = DepotManager:getItemOrResCount(self.m_id, info)
	return total
end

function This:getQuality()  
	return ItemUtil.getQuality(self.m_id)
end

-- 战法
function This:setToTactics()
	self:setToNormal()

	self.m_tacticsViews = self.m_tacticsViews or {}

	local config = TacticsConfig[self.m_id]
	local curStar = config.star
	local maxStar = ArgConfig.tactics_starlevel.arg[1]

	local w, h = self:getContentSize()
	local starBg = ccSprite:create("Common/toumingdi.png")
	starBg:setPosition(w/2, 18)
	self:addChild(starBg)
	self.m_tacticsViews["tacStarBg"] = starBg

	for i=1, maxStar do
		local name = string.format("tacStar%d", i)
		local node = self.m_tacticsViews[name]
		if not node then
			node = ccSprite:create("MenuScene/star_2.png")
			node:setScale(0.25)
			node:setPosition(17+(i-1)*15, 18)
			self:addChild(node, 1)
			self.m_tacticsViews[name] = node
		end
		node:setVisible(i <= curStar)
	end
end

function This:setTacticsViewVisible(visible)
	if self.m_tacticsViews then
		for k, node in pairs(self.m_tacticsViews) do
			node:setVisible(visible)
		end
	end
end

function This:setShowGray(isGray)
	self.m_isShowGray = isGray

	if self.m_icon then
		GameUtil:btnGray(self.m_icon, isGray)
	end

	if self.m_qualityBg then
		GameUtil:btnGray(self.m_qualityBg, isGray)
	end

	if self.m_effectQuality then
		self.m_effectQuality:setVisible(not isGray)
	end
end