--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local Timer = require("Timer")
local EventDispatcher = require("EventDispatcher")

function This:init(info)
	self.m_itemWidth = info.itemWidth or 100
	self.m_itemHeight = info.itemHeight or 100
	self.m_spaceX = info.spaceX or 20
	self.m_spaceY = info.spaceY or 30
	self.m_column = info.column or 1
	self.m_itemCount = info.itemCount or 0
	self.m_frameCount = info.frameCount or 1
	self.m_row = math.ceil(self.m_itemCount / self.m_column)
	self.m_isDelayDelete = info.isDelayDelete or false
	self.m_scrollView = info.scrollView
	
	self.m_width = (self.m_itemWidth + self.m_spaceX ) * self.m_column  
	self.m_height = (self.m_itemHeight + self.m_spaceY) * self.m_row
	self:setContentSize(self.m_width,self.m_height)
	
	self.m_layer = cc.LayerColor:create(cc.Color(255,255,0,128))
	self.m_layer:setContentSize(self.m_width,self.m_height)
	--self:addChild(self.m_layer)

	self.m_nodes = {}
	self.m_task = {}
	self.m_isListing = false
	self.m_timer = Timer:create(self)
end

function This:onExit()
	super.onExit(self)
	self.m_scrollView = nil
end

function This:getNode(index)
	return self.m_nodes[index]
end

function This:isListing()
	return self.m_isListing
end

function This:getAllItem()
	return self.m_nodes
end

function This:getItemCount()
	return #self.m_nodes
end

function This:getPosByIndex(index)
	local column = math.floor((index-1) % self.m_column)
	local row = math.floor((index-1) / self.m_column)
	local x = column * (self.m_itemWidth + self.m_spaceX) + (self.m_itemWidth + self.m_spaceX) / 2
	local y = row * (self.m_itemHeight + self.m_spaceY) + (self.m_itemHeight + self.m_spaceY) / 2
	return x , self.m_height - y
end

function This:listNode(node,index)
	self:addChild(node)
	node:setCascadeOpacityEnabled(true)
	node:setName("listNode"..index)
	local x , y = self:getPosByIndex(index)
	node:setAnchorPoint(0.5,0.5)
	node:setPosition(x,y)
	self.m_nodes[index] = node
end

function This:starList(handler,finishHandler)
	assert(handler)
	self.m_task = {}
	local countPerFrame = math.ceil(self.m_itemCount / self.m_frameCount)

	local listFunc = function (index)
		local node = handler(index)
		self:listNode(node,index)

		if index >= self.m_itemCount then
			if finishHandler then
				finishHandler()
				EventDispatcher:dispatcher("list_finish")
			end
		end
	end

	for i = 1, self.m_frameCount do
		local t = function()
			for j = 1 ,countPerFrame do
				local index = (i-1)*countPerFrame + j
				if index <= self.m_itemCount then
					listFunc(index)
				end
			end
		end
		table.insert(self.m_task,t)
	end
	self:scheduleUpdate()
	self.m_isListing = true
end

function This:resetListNodeCount(count)
	self.m_itemCount = count
	self:refreshContentSize(true,count)
end

function This:refreshContentSize(isToTop,count)
	local itemCount = count or 0
	if itemCount <= 0 and self.m_nodes and #self.m_nodes > 0 then
		itemCount = #self.m_nodes + 1
	end
	 
	local newRow = math.ceil(itemCount / self.m_column)
	if self.m_row ~= newRow then
		local oldRow = self.m_row
		self.m_row = newRow
		self.m_width = (self.m_itemWidth + self.m_spaceX ) * self.m_column  
		self.m_height = (self.m_itemHeight + self.m_spaceY) * self.m_row
		self:setContentSize(self.m_width,self.m_height)

		if self.m_scrollView then
			local w , h = self.m_scrollView:getContentSize()
			if self.m_height > h then
				self.m_scrollView:setInnerContainerSize(self.m_width,self.m_height)
				local ax , ay = self:getAnchorPoint()
				self:setPosition(ax * self.m_width, ay * self.m_height)
				if isToTop then
					self.m_scrollView:setInnerContainerPosition(0,h - self.m_height)
				else
					if self.m_row > oldRow then
						self.m_scrollView:setInnerContainerPosition(0,0)
					end
				end
			else
				self.m_scrollView:setInnerContainerSize(w,h)
				local ax , ay = self:getAnchorPoint()
				self:setPosition(ax * self.m_width, h - self.m_height)
			end
			--self.m_layer:setContentSize(self.m_width,self.m_height)
			self:refreshPosition()
		end

	end
end

function This:refreshPosition()
	for i , node in ipairs(self.m_nodes) do
		local x , y = self:getPosByIndex(i)
		node:setPosition(x,y)
	end
end

function This:swapNode(aIndex,bIndex)
	local ax , ay = self:getPosByIndex(aIndex)
	local bx , by = self:getPosByIndex(bIndex)
	self.m_nodes[aIndex]:setPosition(bx,by)
	self.m_nodes[bIndex]:setPosition(ax,ay)
	local tnode = self.m_nodes[aIndex]
	self.m_nodes[aIndex] = self.m_nodes[bIndex]
	self.m_nodes[bIndex] = tnode
end

function This:forward(index,count)
	local front = math.max(index - count,1)
	count = index - front
	local itemCount = #self.m_nodes

	for i = itemCount , index , -1 do
		local x , y = self.m_nodes[i-count]:getPosition()
		self.m_nodes[i]:setPosition(x,y)
	end
	for i = front , itemCount do
		if i < index then
			self.m_nodes[i]:removeFromParent()
			self.m_nodes[i] = self.m_nodes[i+count]
		elseif i <= itemCount - count then
			self.m_nodes[i] = self.m_nodes[i+count]
		else
			self.m_nodes[i] = nil
		end
	end
	self:refreshContentSize()
end

function This:pushNode(node,isToTop)
	self:refreshContentSize(isToTop)
	self:listNode(node,#self.m_nodes + 1)
end

function This:deleteNode(node)
	for i , listNode in pairs(self.m_nodes) do
		if listNode == node then
			self:removeNode(i)
			break
		end
	end
end

function This:insertNode(index,node)
	local itemCount = #self.m_nodes
	if index >= itemCount then
		self:listNode(node,itemCount + 1)
	else
		itemCount= itemCount + 1
		self:listNode(node,itemCount)
		for i = itemCount-1 , index + 1, -1 do
			self:swapNode(i,i+1)
		end
	end
	self:refreshContentSize()
end

function This:removeNode(index,isToTop)
	local itemCount = #self.m_nodes
	if index > itemCount then
		return
	end
	if index >= itemCount then
		self.m_nodes[index]:removeFromParent()
		self.m_nodes[index] = nil
	else
		for i = index + 1 , itemCount do
			self:swapNode(i-1,i)
		end
		self.m_nodes[itemCount]:removeFromParent()
		self.m_nodes[itemCount] = nil
	end
	self:refreshContentSize(isToTop)
end

function This:clearNode()
	if self.m_isDelayDelete then
		for i , node in pairs(self.m_nodes) do
			self.m_timer:setTimer(0.01* (i - 1),function()
				node:removeFromParent()
			end,"listlayer"..i)
			self.m_nodes[i] = nil
		end
	else
		self:unscheduleUpdate()
		self.m_isListing = false
		for i , node in pairs(self.m_nodes) do
			node:removeFromParent()
			self.m_nodes[i] = nil
		end
	end
	self.m_task = {}
end

function This:removeAllNode()
	for i , node in pairs(self.m_nodes) do
		node:removeFromParent()
		self.m_nodes[i] = nil
	end
	if self.m_isListing then
		self.m_timer:removeAllTimer()
		self.m_isListing = false
	end
end

function This:endList()
	if not self.m_isDelayDelete then
		self:unscheduleUpdate()
	end
	self.m_timer:removeAllTimer()
end

function This:update(dt)
	if #self.m_task <= 0 and self.m_isListing then
		self:endList()
		self.m_isListing=false
	elseif self.m_isListing then
		self.m_task[1]()
		table.remove(self.m_task,1)
	end
	self.m_timer:update(dt)
end

--endregion
