

local This , super = class(...,"cc.Node")
local Timer = require("Timer")


function This:init(info)
	self.m_itemWidth = info.itemWidth or 100
	self.m_itemHeight = info.itemHeight or 100
	self.m_spaceX = info.spaceX or 10
	self.m_spaceY = info.spaceY or 10
	self.m_column = info.column or 1 --水平无效
	self.m_scrollView = info.scrollView
	self.m_isVertical = (info.isVertical and true) or false
	self.m_onItemUpdate = info.onItemUpdate --onItemUpdate(idx, node)

	self.m_views = {}
	self.m_recycle = {}
	self.m_dataList = {}
	self.m_isDirty = true

	-- self.m_layer = cc.LayerColor:create(cc.Color(255,0,0,100))
	-- self:addChild(self.m_layer)

	self:setCascadeOpacityEnabled(true)
	self:refreshContentSize()

	local w, h = self:getContentSize()

	if self.m_isVertical then
		self.m_scrollView:setDirection("vertical")
		self.m_scrollView.m_direction = "down"
		self:setAnchorPoint(0.5, 0)
		self.m_scrollView:pushBackCustomItem(self, w, h, 0)
	else
		self.m_scrollView:setDirection("horizontal")
		self.m_scrollView.m_direction = "right"
		self:setAnchorPoint(0, 0.5)
		self.m_scrollView:pushBackCustomItem(self, w, h, 0)
	end

	self.m_scrollView:addEventListener(function(_,event)
		self.m_isDirty = true
	end)
	
	self:scheduleUpdate()	
end

function This:setDataList(dataList)
	self.m_dataList = dataList or {}
	self:setItemDirty(true)
	self:refreshContentSize()
end

function This:setItemDirty(isDirty)
	self.m_isDirty = isDirty
	for idx, node in pairs(self.m_views) do
		node.__isDirty__ = isDirty
	end
end

function This:refreshContentSize()
	local w, h = 0, 0
	local sw, sh = self.m_scrollView:getContentSize()
	local itemCount = #self.m_dataList

	if self.m_isVertical then
		local row = math.ceil(itemCount / self.m_column)
		w = (self.m_itemWidth + self.m_spaceX) * self.m_column - self.m_spaceX
		h = (self.m_itemHeight + self.m_spaceY) * row - self.m_spaceY 
	else
		w = (self.m_itemWidth + self.m_spaceX) * itemCount - self.m_spaceX
		h = self.m_itemHeight
	end

	if w < sw then
		w = sw
	end

	if h < sh then
		h = sh
	end

	self:setContentSize(w, h)	
	self.m_scrollView:setInnerContainerSize(w, h)	
	if self.m_layer then
		self.m_layer:setContentSize(w, h)
	end

	-- print("refreshContentSize", w, h)
end

function This:onItemViewUpdate()
	local startIndex, endIndex = self:calcIndex()
	
	-- 回收超出范围的节点
	for idx, node in pairs(self.m_views) do
		if idx < startIndex or idx > endIndex then
			node:setVisible(false)
			table.insert(self.m_recycle, node)
			self.m_views[idx] = nil
		end
	end 

	if #self.m_dataList == 0 then
		self.m_isDirty = false
		return
	end

	local count = 0
	for idx = startIndex, endIndex do
		local item = self.m_views[idx]
		count = count + 1

		if item then
			if item.__isDirty__ then
				-- print("update item", idx)
				item.__index__ = idx
				item.__isDirty__ = false
				self.m_onItemUpdate(idx, item)
				break
			end
		else
			local x, y = self:getPosByIndex(idx)
			item = self:getFreeItem()

			if item then
				-- print("update free item", idx)
				self.m_onItemUpdate(idx, item)
			else
				-- print("create item", idx)
				item = self.m_onItemUpdate(idx)
				if item then
					self:addChild(item)
				end
			end

			if item then
				-- print("item pos", x, y)
				item.__index__ = idx
				item.__isDirty__ = false
				item:setName(string.format("item%d", idx))
				item:setAnchorPoint(0.5, 0.5)
				item:setPosition(x, y)
				self.m_views[idx] = item
			end

			break
		end
	end

	if count >= endIndex - startIndex + 1 then
		self.m_isDirty = false
	end
end

function This:calcIndex()
	local startIndex, endIndex = 0, 0
	local innerW, innerH = self.m_scrollView:getInnerContainerSize()
	local innerX, innerY = self.m_scrollView:getInnerContainerPosition()
	local sw, sh = self.m_scrollView:getContentSize()

	if self.m_isVertical then
		local p1 = innerH + innerY - sh
		local p2 = innerH + innerY
		p1 = math.max(1, p1)
		p2 = math.min(innerH, p2)
		startIndex = math.floor(p1 / (self.m_itemHeight + self.m_spaceY)) * self.m_column + 1
		endIndex = math.ceil(p2 / (self.m_itemHeight + self.m_spaceY)) * self.m_column
	else
		local p1 = -innerX
		local p2 = -innerX + sw
		p1 = math.max(1, p1)
		p2 = math.min(innerW, p2)
		startIndex = math.floor(p1 / (self.m_itemWidth + self.m_spaceX)) + 1
		endIndex = math.ceil(p2 / (self.m_itemWidth + self.m_spaceX))
	end

	local count = #self.m_dataList
	if startIndex > count then
		startIndex = count
	end
	if endIndex > count then
		endIndex = count
	end

	return startIndex, endIndex
end

function This:getPosByIndex(index)
	local width, height = self:getContentSize()

	if self.m_isVertical then
		local column = math.floor((index-1) % self.m_column)
		local row = math.floor((index-1) / self.m_column)
		local x = column * (self.m_itemWidth + self.m_spaceX) + self.m_itemWidth / 2
		local y = row * (self.m_itemHeight + self.m_spaceY) + self.m_itemHeight / 2
		return x, height - y
	else
		local x = (index - 1) * (self.m_itemWidth + self.m_spaceX) + self.m_itemWidth / 2
		local y = height / 2
		return x, y
	end
end

function This:getFreeItem()
	local item = nil
	if #self.m_recycle > 0 then
		item = table.remove(self.m_recycle, 1)
		item:setVisible(true)
	end
	return item
end

function This:update(dt)
	if self.m_isDirty then
		self:onItemViewUpdate()
	end
end

function This:getItemList()
	local list = {}

	for idx, node in pairs(self.m_views) do
		node.__index__ = idx
		table.insert(list, node)
	end

	table.sort(list, function (a, b)
		return a.__index__ < b.__index__
	end)

	return list
end