-- @Author: yuchao
-- @Date:   2018-01-11 21:41:52
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-28 11:53:48
-- @File Name: PopDialogManager.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local Application = require("cc.Application")
local EventDispatcher = require("EventDispatcher")

g_popMgr = This

DialogLevel = 
{
	kTopLevel = 1,  --ûֱӵ
	kSecLevel = 2,  --иԼĲ
	kThrLevel = 3,  --Լŵ
	kForLevel = 4,  --ûд򿪵dialogŵ
}

LvDlgZorder = 
{
	[DialogLevel.kTopLevel] = 1004,
	[DialogLevel.kSecLevel] = 1003,
	[DialogLevel.kThrLevel] = 1002,
	[DialogLevel.kForLevel] = 1001,
}

function This:init()
	super.init(self)
	self.m_lvDlgList = {}
	for key , level in pairs(DialogLevel) do
		self.m_lvDlgList[level] = {}
	end
    EventDispatcher:addListener(self, "dialog_close", self.check)
    EventDispatcher:addListener(self, "menu_scene_enter", function ()
		self:check()
	end)
    EventDispatcher:addListener(self, "world_scene_enter", self.check)
    EventDispatcher:addListener(self, "legion_init_finish", self.check)
end

function This:getDelayTime(level)
	for i = DialogLevel.kTopLevel , level - 1 do
		if #self.m_lvDlgList[i] > 0 then
			return 0
		end
	end
	return level * 0.1
end

function This:addPopDialog(info)
	info.level = info.level or DialogLevel.kForLevel
	local queue = self.m_lvDlgList[info.level]
	table.insert(queue,info)
	if info.bCheck or info.level == DialogLevel.kTopLevel then
		local delay = self:getDelayTime(info.level)
		local timer = cc.Application:getTimer()
		timer:setTimer(delay,function()
			self:check()
		end,"add_pop_dialog")
	end
end

function This:openDialog(info)
	local panel = nil
	if not info.func then
		panel = g_scene:openDialog(info.name, info.arg)
	else
		panel = info.func(info.arg)
	end
	if info.level < DialogLevel.kForLevel and panel[0] ~= 0 then
		panel:setLocalZOrder(LvDlgZorder[info.level])
		panel:setLevel(info.level)
	end
	return panel
end

function This:check()
	if g_scene:getType() == SceneType.kLoadingScene then
		return
	end

	local function handler(info , level)
		local panel = g_scene:getTopDialog()
		if level == DialogLevel.kTopLevel then
			return self:openDialog(info)
		elseif level == DialogLevel.kSecLevel then
			local panel = g_scene:getTopDialog()
			if not panel or level < panel:getLevel() then
				return self:openDialog(info)
			end
		elseif level == DialogLevel.kThrLevel then
			local panel = g_scene:getTopDialog()
			if not panel or level <= panel:getLevel()  then
				return self:openDialog(info)
			end
		elseif level == DialogLevel.kForLevel then
			if not panel then
				return self:openDialog(info)
			end
		end
	end

	for i = DialogLevel.kTopLevel , DialogLevel.kForLevel do
		for j , info in ipairs(self.m_lvDlgList[i]) do
			if not info.scene or info.scene and g_scene:getName() == info.scene then
				if handler(info,i) then
					table.remove(self.m_lvDlgList[i],j)
					return
				end
			end
		end
	end
end
--endregion
