

--皇帝战/军团战 规则界面

local This , super = class(...,"Dialog")
local LegionWarRuleConfig = require("Metadatas.LegionWarRuleConfig")

RulesType = {
    kKing = 1, -- 皇帝战
    kLegion = 2, -- 军团战
}

local RulesRewardPage = {
    [RulesType.kKing] = "LegionEntrant.KingWarRuleRewardNode",
    [RulesType.kLegion] = "LegionEntrant.LegionWarRuleRewardNode",    
}

function This:initData(info)
    self.m_rulesType = info.rulesType
    self.m_rulesList = {}
    self.m_rewardView = nil

    for k, v in pairs(LegionWarRuleConfig) do
        if v.type == self.m_rulesType then
            table.insert(self.m_rulesList, v)
        end
    end

    table.sort(self.m_rulesList, function (a, b)
        return a.id < b.id
    end)

    --最后一个是奖励页面
    local id = #self.m_rulesList
    table.insert(self.m_rulesList, {
        id = id,
        paging_id = id,
        paging_name = Serialize[60659],
        paging_text = "",
        button_name = Serialize[60659]
    })
end

function This:createTabButton(text)
    local node = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionHelpTabBtn.csb")

    local w, h = node.bg:getContentSize()
    node.bg:setPosition(w/2, h/2)
    node:setContentSize(w, h)
    node:setAnchorPoint(0.5, 0.5)

    node.normal = node.bg.normal
    node.light = node.bg.light
    node.normal:setString(text)
    node.light.light:setString(text)
    
    return node
end

function This:addTabButton(name, button, onSelected)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end
    if self.m_tabButtons[name] then
        return
    end
    self.m_tabButtons[name] = button
    if not button.m_touch then
        button:setTouch({
            isContainer = true,
            isDynamic = true,
            isScale = false,
            onEnded = function ()
                self:selectTabButton(name)
            end
        })
    end
    button.onSelected = onSelected
end

function This:initNode(info)
    local csNode = cc.Node:createFromCSFile("LegionScene/NodeUI/LegionHelpMain.csb")
    csNode:setToCenter()
    self:addChild(csNode)
    self.m_csNode = csNode


    for i, config in ipairs(self.m_rulesList) do
        local btn = self:createTabButton(config.button_name)
        local w, h = btn:getContentSize()
        csNode.btnList:pushNode(btn, w, h) 

        self:addTabButton(i, btn, function()
            self:selectTab(i)
        end)
    end

    self:selectTabButton(1)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
end

function This:onEnter()
    super.onEnter(self)
end

function This:onExit()
    super.onExit(self)
end

function This:selectTab(tab)
    if self.m_curTab == tab then
        return
    end
    self.m_curTab = tab

    local csNode = self.m_csNode

    -- 小标题
    local config = self.m_rulesList[tab]
    csNode.teptitle:setString(config.paging_name)

    -- 内容
    csNode.scrollview:removeAllItems()

    if tab == #self.m_rulesList then
        if not self.m_rewardView then
            self.m_rewardView = require(RulesRewardPage[self.m_rulesType]):create()
            self.m_rewardView:setPosition( csNode.scrollview:getPosition() )
            self.m_csNode:addChild(self.m_rewardView)
        end
        self.m_rewardView:setVisible(true)
        csNode.scrollview:setVisible(false)
    else
        local text = cc.Label:createNormalTTF("", 16)
        text:setStringWithColor(config.paging_text, 740)
        text:setAnchorPoint(0.5,0)
        text:setCascadeOpacityEnabled(true)
        local w , h = text:getContentSize()
        csNode.scrollview:pushBackCustomItem(text, w, h, 0)
        csNode.scrollview:setVisible(true)
        if self.m_rewardView then
            self.m_rewardView:setVisible(false)
        end
    end
end
