--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"UI.UpdateLayer")
local TouchListener = require("cc.TouchListener")
local Vec = require("Vec")

function This:init(info)
	super.init(self,info)
	self.m_eventListener = nil
	self.m_curNode = nil
	self.m_curIndex = 0
	self.m_isTouching = false
	self.m_isSwaping = false
	self.m_escape = 0
	self.m_timeLimit = 1.0
	self.m_scrollView = info.scrollView
	--[[local layer = cc.LayerColor:create(cc.Color(0,255,255,128))
	local w , h =self:getContentSize()
	layer:setContentSize(w,h)
	self:addChild(layer)]]
end

function This:setSwapHandler(handler,judgeHandler)
	self.m_swapHandler = handler
	self.m_judgeHandler = judgeHandler
end

function This:setDragSwapNode(isEnable)
	if isEnable and not self.m_eventListener then
		self.m_eventListener = TouchListener:create({
			onBegan = function(_,touch)
				return self:onTouchBegan(touch)
			end,

			onMoved = function(_,touch)
				self:onTouchMoved(touch)
			end,

			onEnded = function(_,touch)
				self:onTouchEnded(touch)
			end,
			isSwallow = false,
		})
		self:addEventListener(self.m_eventListener)
		if self.m_scrollView then
			self.m_scrollView:setSwallowTouch(false)
		end
	elseif self.m_eventListener then
		self:removeEventListener(self.m_eventListener)
		self.m_eventListener = nil
	end
end

function This:onTouchBegan(touch)
	for index, node in pairs(self.m_views) do
		local nodex , nodey = node:getPosition()
		local w, h = node:getContentSize()
		local boundBox = node:getBoundingBox()
		if boundBox:isContainsXY(touch.x,touch.y) then
			self.m_curNode = node
			self.m_isTouching = true
			self.m_prePoint = Vec(touch.x , touch.y)
			self.m_initPoint = Vec(touch.x, touch.y)
			return true
		end
	end
	return false
end

function This:onTouchMoved(touch)
	if self.m_isSwaping then
		local dx , dy = touch.x - self.m_prePoint.x , touch.y - self.m_prePoint.y
		local x , y = self.m_curNode:getPosition()
		self.m_curNode:setPosition(x+dx,y+dy)
		self.m_prePoint = Vec(touch.x, touch.y)
	end
end

function This:onTouchEnded(touch)
	if self.m_scrollView and not self.m_scrollView:isTouchEnabled() then
		self.m_scrollView:setTouchEnabled(true)
		self.m_scrollView:setSwallowTouch(false)
	end
	if not self.m_curNode then
		self.m_isSwaping = false
		self.m_escape = 0
		self.m_isTouching = false
		return
	end

	if self.m_escape < self.m_timeLimit then
		local dist = Vec.getDistance(self.m_initPoint,Vec(touch.x,touch.y))
		if dist < 20 and self.m_curNode.onClicked then
			self.m_curNode:onClicked()
		end
	else
		local targetNode = nil
		for index, node in pairs(self.m_views) do
			if node ~= self.m_curNode then
				local boundBox = node:getBoundingBox()
				if boundBox:isContainsXY(touch.x, touch.y) then
					targetNode = node
				end
			end
			if node.index == nil then
				assert(false)
			end
		end
		local dist = 0xffff
		if targetNode then
			local x1 , y1 = targetNode:getPosition()
			local x2 , y2 = self.m_curNode:getPosition()
			dist = math.sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2))
		end
		if dist < 50 and self.m_judgeHandler and self.m_judgeHandler(self.m_curNode.index,targetNode.index) then
			self:swap(self.m_curNode,targetNode)
		else
			self:cancelSwap()
		end
	end
	self.m_escape = 0
	self.m_isTouching = false
end

function This:swap(curNode,targetNode)
	local x1 , y1 = self:getPosByIndex(curNode.index)
	local x2 , y2 = self:getPosByIndex(targetNode.index)
	curNode:runAction(cc.Spawn(cc.MoveTo(0.5,x2,y2,"SineInOut"),cc.ScaleTo(0.5,1.0 * self.m_curNode.initScale,"SineOut")))
	targetNode:runAction(cc.Sequence(cc.MoveTo(0.52,x1,y1,"SineInOut"),cc.CallFunc(function()
		self:swapNode(curNode,targetNode)
		self.m_isSwaping = false
		curNode:setLocalZOrder(0)
		targetNode:setLocalZOrder(0)
		if self.m_swapHandler then
			self.m_swapHandler(curNode.index,targetNode.index)
		end
		if self.m_scrollView and not self.m_scrollView:isTouchEnabled() then
			self.m_scrollView:setTouchEnabled(true)
			self.m_scrollView:setSwallowTouch(false)
		end
	end)))
	targetNode:setLocalZOrder(1)
end

function This:cancelSwap()
	local x , y = self:getPosByIndex(self.m_curNode.index)
	self.m_curNode:runAction(cc.Sequence(cc.Spawn(cc.MoveTo(0.5,x,y,"SineInOut"),cc.ScaleTo(0.5,1.0 * self.m_curNode.initScale or 1,"SineOut")),cc.CallFunc(function()
		self.m_isSwaping = false
		self.m_curNode:setLocalZOrder(0)
	end)))
	if self.m_scrollView and not self.m_scrollView:isTouchEnabled() then
		self.m_scrollView:setTouchEnabled(true)
		self.m_scrollView:setSwallowTouch(false)
	end
end

function This:startSwap()
	local initScale = self.m_curNode:getScale()
	self.m_curNode:runAction(cc.ScaleTo(0.3,1.1 * initScale,"SineIn"))
	self.m_curNode.initScale = initScale

	self.m_isSwaping = true
	self.m_curNode:setLocalZOrder(2)
	if self.m_scrollView and self.m_scrollView:isTouchEnabled() then
		self.m_scrollView:setTouchEnabled(false)
	end
end

function This:update(dt)
	super.update(self,dt)
	if self.m_isTouching then
		self.m_escape =  self.m_escape + dt
		if self.m_escape > 1.0 and not self.m_isSwaping then
			self:startSwap()
		end
	end
end

--endregion
