-- @Author: superm
-- @Date:   2017-12-12 15:00:47
-- @Last Modified by:   superm
-- @Last Modified time: 2017-12-20 16:23:03
-- @File Name: UIBehaviour.lua
-- @Description: 

local This = class(...)
local ccTouchListener = require("cc.TouchListener")

function This:init(parentView)
	self.m_parentView = parentView
	self.m_hasSetup = false
	self.m_ctrlHandlerList = {}
end

function This:setup(...)
	-- 安装行为
	if self.m_hasSetup then
		self:clear()
	end
	
	self:doSetup(...)

	self.m_hasSetup = true
end

function This:clear()
	-- 清除行为
	if self.m_hasSetup then
		self:doClear()
		self:clearListener()
		self.m_viewMap = {}
		self.m_hasSetup = false
	end
end

function This:addTouchListener(touch)
    local listener = ccTouchListener:create(touch)
    self:addEventListener(listener)
    return listener
end

function This:addListener(control, func, event)
	local handler = control:setTouch({onEnded = func, isScale = false, isDynamic = true})
	self.m_ctrlHandlerList[#self.m_ctrlHandlerList + 1] = {
		control=control, event=event, handler=handler
	}
end

function This:removeListener(control, event)
	for index, t in pairs(self.m_ctrlHandlerList) do
		if t.control == control and t.event == event then
			t.control:removeEventListener(t.handler)
			self.m_ctrlHandlerList[index] = nil
		end
	end
end

function This:clearListener()
	for _, t in pairs(self.m_ctrlHandlerList) do
		t.control:removeEventListener(t.handler)
	end
	self.m_ctrlHandlerList = {}
end

function This:getSubView(name)
	if type(name) == "string" then
		return self.m_parentView[name]
	else
		return name
	end
end

function This:getSubViewList(nameList)
	for k, v in pairs(nameList) do
		if type(v) == "string" then
			nameList[k] = self.m_parentView[v]
		end
	end
	return nameList
end
