-- @Author: superm
-- @Date:   2017-12-12 15:19:24
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-03-30 18:15:29
-- @File Name: UIButtonListBehaviour.lua
-- @Description: 

local This, super = class(..., "UI.UIBehaviour")

function This:create(...)
    local ret = {}
    setmetatable(ret,self)
    ret:init(...)
    return ret
end

function This:init(parentView)
	super.init(self, parentView)
	self.m_visibleCount = 0
	self.m_buttonList = {}
	self.m_buttonPosList = {}
end

function This:doSetup(btnList, info)
	self.m_buttonList = btnList

	self.m_flexible = info and info.flexible or false
	self.m_adjustable = nil
	if info and info.adjustable then
		self.m_adjustable = self.m_parentView[info.adjustable]
	end
	if info and info.switch then
		self.m_switch = info.switch
		self.m_switchCount = info.switchCount or 1
		self.m_switch:setBehaviour(self)
		local x, y = self.m_switch:getPosition()
		self.m_switchPosition = {x = x, y = y}
	end

	for i, node in ipairs(self.m_buttonList) do
		local x, y = node:getPosition()
		self.m_buttonPosList[i] = {x = x, y = y}
		if self.m_flexible or node:hasGuideTouch() then
			node:setBehaviour(self)
		end
	end

	if info and info.container then
		self.m_container = self:getSubView(info.container)
		self.m_style = info.style
		local w , h = self.m_container:getContentSize()
		self.m_containerSize = {width = w, height = h}
	end
end

function This:doClear()
	self.m_buttonList = {}
	self.m_buttonPosList = {}
end

function This:getButtonList()
	return self.m_buttonList
end

function This:getButton(index)
	return self.m_buttonList[index]
end

function This:setButtonVisible(index, visible)
    if visible == nil then
	    visible = true
    end

	local button = self.m_buttonList[index]
	if not button then
		return
	end

	local curVisible = button:isVisible()
	if visible == curVisible then
		return
	-- else
	-- 	if visible then
	-- 		print(button:getNoteName() .. " 要求被强开, index:" .. index)
	-- 	else
	-- 		print(button:getNoteName() .. " 要求被强关, index:" .. index)
	-- 	end
	end

	if self.m_isRunning or self.m_isRunningAnimation or not button or button:isPlaying() then
		self.m_isDirty = true
		return
	end
	button:setNodeVisible(visible)

	local posIndex = 1
	for i = 1, #self.m_buttonList do
		if self.m_buttonList[i]:isVisible() then
			local position = self.m_buttonPosList[posIndex]
			self.m_buttonList[i]:setPosition(position.x, position.y)
			posIndex = posIndex + 1
		end
	end
	self:doResizeContainer(posIndex - 1, false)
end

function This:setButtonsVisible(visibleList)
	local posIndex = 1
	for i = 1, #self.m_buttonList do
		local btn = self.m_buttonList[i]
		if visibleList[i] then
			local position = self.m_buttonPosList[posIndex]
		    btn:setPosition(position.x, position.y)
			posIndex = posIndex + 1
		else
			btn:setNodeVisible(false)
		end
	end
	self:doResizeContainer(posIndex - 1, false)
end

function This:refreshButtonsVisible()
	local posIndex = 1
	local count = #self.m_buttonList
	for i = 1, count do
		local btn = self.m_buttonList[i]
		if btn:getCurVisible() then
			local position = self.m_buttonPosList[posIndex]
			btn:setPosition(position.x, position.y)
			btn:setNodeVisible(true)
			posIndex = posIndex + 1
		else
			btn:setNodeVisible(false)
		end
	end
	self:doResizeContainer(posIndex - 1)
end

-- function This:refreshWithAnimate(isInited)
-- 	if self.m_isRunning or self.m_isRunningAnimation then
-- 		self.m_isDirty = true
-- 		return
-- 	end
-- 	self.m_isRunning = true

-- 	local actionList = {}

-- 	local posIndex = 1
-- 	local animationCount, animationIndex = 0, 0
-- 	local count = #self.m_buttonList
-- 	for i = 1, count do
-- 		local btn 		= self.m_buttonList[i]
-- 		local position 	= {x=0, y=0}
-- 		local visible 	= btn:getCurVisible()
-- 		local hasAnimation, hasEffect = visible, false

-- 		if visible then
-- 			hasAnimation = not isInited
-- 			position = self.m_buttonPosList[posIndex]
-- 			hasEffect = not btn:isVisible()
-- 			if hasAnimation and hasEffect then
-- 				animationIndex = i
-- 				animationCount = animationCount + 1
-- 			-- 	print(btn:getNoteName() .. "有开放动画， index:" .. i)
-- 			-- elseif hasAnimation then
-- 			-- 	print(btn:getNoteName() .. "有移动动画， index:" .. i)
-- 			end
-- 		else
-- 			--print(btn:getNoteName() .. "没开放， index:" .. i)
-- 			-- position.x, position.y = btn:getParent():convertToNodeSpace(cc.visible.center.x, cc.visible.center.y)
-- 			btn:setNodeVisible(false)
-- 		end

-- 		actionList[i] = {pos = position, animate = hasAnimation, eff = hasEffect}
-- 		if visible then
-- 			posIndex = posIndex + 1
-- 		end

-- 		if self.m_isDirty then
-- 			self.m_isDirty = false
-- 			i = 1
-- 			posIndex = 1
-- 			animationCount = 0
-- 			animationIndex = 0
-- 		end
-- 	end
	
-- 	self.m_isRunning = false
-- 	local doubleDelay = false
-- 	local callbackIndex = count
-- 	if animationCount > 0 then
-- 		self.m_isRunningAnimation = true
-- 		callbackIndex = animationIndex
-- 		if self.m_switch and self.m_switch:getSwitchState() == false then
-- 			doubleDelay = true
-- 		end
-- 	end
-- 	actionList[callbackIndex].callback = function ()
-- 		self.m_isRunningAnimation = false
-- 		if self.m_isDirty then
-- 			if not doubleDelay then
-- 				self:refreshButtonsVisible()
-- 			else 
-- 				self.m_isDirty = false
-- 			end
-- 		end
-- 	end

-- 	if not doubleDelay then
-- 		for i = 1, count do
-- 			local btn = self.m_buttonList[i]
-- 			if btn then
-- 				btn:adjustPosition(actionList[i])
-- 			end
-- 		end
-- 		self:doResizeContainer(posIndex - 1, false)
-- 	else
-- 		local touchHandler = self.m_switch.m_touch
-- 		if touchHandler then
-- 			touchHandler:onEnded(nil, function()
-- 				for i = 1, count do
-- 					local btn = self.m_buttonList[i]
-- 					if btn then
-- 						btn:adjustPosition(actionList[i])
-- 					end
-- 				end
-- 				self:doResizeContainer(posIndex - 1, false)				
-- 			end)
-- 		end
-- 	end
-- end

function This:playSwitchAnimation( value, callbackList, animate )
	if self.m_isPlaySwitchAnimation then
		return
	end
	self.m_isPlaySwitchAnimation = true

	local isPosAnimation = animate ~= false

    local factor = 0.3
	if self.m_buttonList and self.m_switch then 
		local nodeHandler = callbackList.node_handler
		local otherHandler = callbackList.other_handler
		local infoList = {}
		local startIndex = callbackList.start_index or 1
		local posIndex = 1

	    for i, button in ipairs(self.m_buttonList) do
	    	local button = self.m_buttonList[i]
			local parameter = nil
			local isVisible = button:isVisible()
			if not value and isVisible then
                parameter = {animate = true, button = button, factor = factor, visible = false}
                if isPosAnimation then
                	parameter.visible = nil
                	parameter.pos = {}
		    		local x, y = self.m_switchPosition.x, self.m_switchPosition.y
					parameter.pos.x, parameter.pos.y = button:getParent():convertToNodeSpace(x, y)
				end
			elseif value then
				local curVisible = button:getCurVisible()
				local ret = false
				if otherHandler and curVisible and isVisible == curVisible then
					ret = true
				elseif not otherHandler and curVisible then
					ret = true
				end

				if ret then
	                parameter = {pos = {}, animate = true, button = button, factor = factor, visible = true}
					parameter.pos = self.m_buttonPosList[posIndex]
					posIndex = posIndex + 1
				end
			end
			
            if parameter then
                table.insert(infoList, parameter)
            end
	    end

        local count = #infoList
        if not value and nodeHandler then
            local lastInfo = infoList[count]
            if lastInfo then
                lastInfo.callback = function ()
					self.m_isPlaySwitchAnimation = false
            		nodeHandler()
                end 
            end
        elseif nodeHandler then
            nodeHandler()
            local lastInfo = infoList[count]
            if lastInfo then
                lastInfo.callback = function ()
					self.m_isPlaySwitchAnimation = false
					if otherHandler then
						otherHandler()
					end
                end
            end
        end
		self:playSwitchMoveAnimation(posIndex -1, true)

	    for i = startIndex, count do
		    local info = infoList[i]
		    if info and info.button then
			    info.button:adjustPosition(info)
		    end
	    end
	end
end

function This:setSwitchVisible( value )
	if self.m_switch then
		self.m_switch:setNodeVisible( value and self.m_visibleCount > self.m_switchCount)
	end
end

function This:doResizeContainer(visibleCount, animate)
	if self.m_switch then
		if self.m_visibleCount ~= visibleCount then
			self.m_visibleCount = visibleCount
			self:playSwitchMoveAnimation(visibleCount, animate)
		end
		self.m_switch:setNodeVisible(visibleCount > self.m_switchCount)
	end

	if self.m_container then 
		if visibleCount >= #self.m_buttonList then
			self.m_container:setContentSize(self.m_containerSize)
			return
		end
	end
end

function This:playSwitchMoveAnimation( visibleCount, animate )
	if not animate and self.m_switch:getSwitchState() == false then
		return
	end

	if self.m_flexible then
		local curSwitchPos = {x= self.m_switchPosition.x, y= self.m_switchPosition.y}
		local huodongSwitch ={
			[SceneType.kHomeScene] = 7,
			[SceneType.kWorldScene] = 6,
			[SceneType.kLegionScene] = 6,
		}		
		local sceneType = g_scene:getType()
		local customLength = huodongSwitch[sceneType] or 7
		local destNode = self.m_buttonList[customLength]:getParent()
		if visibleCount > 0 then
			local posLeft = {x=0, y=0} 
			local numberPerLine, leftIndex = customLength, math.min(visibleCount, customLength)
			-- if visibleCount > numberPerLine * 2 then
			-- 	numberPerLine, leftIndex = 8, 16
			-- end 
			local x, y = destNode:convertToWorldSpace(self.m_buttonPosList[leftIndex].x, self.m_buttonPosList[leftIndex].y)
			posLeft.x, posLeft.y = self.m_switch:getParent():convertToNodeSpace(x, y)
			curSwitchPos.x = posLeft.x - 50
			curSwitchPos.y = posLeft.y
		end
		self.m_switch:adjustPosition({pos = curSwitchPos, animate = animate, factor=0.3})
	end

	if self.m_adjustable and self.m_adjustable.bg2 and visibleCount > 0 then
		local bgNode = self.m_adjustable.bg2
		local numberPerLine = 7
		local count = visibleCount - numberPerLine
		if count > 0 then
			local width, high = 70, 78
			bgNode:setVisible(true)
			bgNode:setContentSize(width * count + 25, high)
		else
			bgNode:setVisible(false)
		end
	end
end