--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local Timer = require("Timer")
local EventDispatcher = require("EventDispatcher")

function This:init(info)
	self.m_itemWidth = info.itemWidth or 100
	self.m_itemHeight = info.itemHeight or 100
	self.m_spaceX = info.spaceX or 20
	self.m_spaceY = info.spaceY or 30
	self.m_column = info.column or 1
	self.m_inteval = info.inteval or 0.02
	self.m_scrollView = info.scrollView
	self.m_isVertical = info.isVertical == nil or false

	self.m_layer = cc.LayerColor:create(cc.Color(255,255,0,0))
	self:addChild(self.m_layer)
	self.m_views = {}

	self:refreshContentSize(info.itemCount or 0)
	
	local w , h = self.m_scrollView:getContentSize()
	if self.m_isVertical then
		self.m_showCount = math.ceil(h / (self.m_itemHeight+self.m_spaceY) + 1) * self.m_column
	else
		self.m_showCount = math.ceil(w / (self.m_itemWidth+self.m_spaceX) + 1) * self.m_column
	end

	self.m_isInit = false
	self.m_refreshInterval = 0
	self.m_refreshEscape = 0
	self.m_startIndex = 1
	self.m_timer = Timer:create(self)
	self.m_dataList = {}
end

function This:getPosByIndex(index)
	if self.m_isVertical then
		local column = math.floor((index-1) % self.m_column)
		local row = math.floor((index-1) / self.m_column)
		local x = column * (self.m_itemWidth + self.m_spaceX) + (self.m_itemWidth + self.m_spaceX) / 2
		local y = row * (self.m_itemHeight + self.m_spaceY) + (self.m_itemHeight + self.m_spaceY) / 2
		return x , self.m_height - y
	else
		local column = math.floor((index-1) % self.m_column)
		local row = math.floor((index-1) / self.m_column)
		local y = column * (self.m_itemHeight + self.m_spaceY) + (self.m_itemHeight + self.m_spaceY) / 2
		local x = row * (self.m_itemWidth + self.m_spaceX) + (self.m_itemWidth + self.m_spaceX) / 2
		return x , self.m_height - y
	end
end

function This:initItemView(dataList, handler, finishHandler)
	local count = math.min(#dataList,self.m_showCount)
	for i = 1 , count do
		self.m_timer:setTimer((i-1)*self.m_inteval,function()
			local view = handler(dataList[i], i)
			local x , y = self:getPosByIndex(i)
			view:setPosition(x,y)
			view:setAnchorPoint(0.5,0.5)
			view.index = i
			view:setName("updateItem"..i)
			self:addChild(view)
			self.m_views[i] = view
            if i == count then
		        self.m_timer:setTimer(i * self.m_inteval, function()
					if finishHandler then
						finishHandler()
					end
					EventDispatcher:dispatcher("list_finish")
		        end, "finishHandler")
	        end
		end,"init_item"..i)
	end

	self:scheduleUpdate()
	self.m_timer:setTimer(count * self.m_inteval, function()
		self.m_scrollView:addEventListener(function(_,event)
			self:updateItemView(self.m_dataList,handler)
		end)
	end)
	self.m_isInit = true
end

function This:updateItemView(dataList,handler)
	local x , y = self.m_scrollView:getInnerContainerPosition()
	local w , h = self.m_scrollView:getInnerContainerSize()
	local w1 , h1 = self.m_scrollView:getContentSize()

	local startIndex ,endIndex = 0,0
	if self.m_isVertical then
		local pt , pb = h + y - h1, h + y 
		pt = math.max(1,pt)
		pb = math.min(h,pb)
		startIndex = math.floor(pt / (self.m_itemHeight + self.m_spaceY)) * self.m_column + 1
		endIndex = math.ceil(pb / (self.m_itemHeight + self.m_spaceY)) * self.m_column
	else
		local pt , pb = -x, -x + w1
		pt = math.max(1,pt)
		pb = math.min(w,pb)
		startIndex = math.floor(pt / (self.m_itemWidth + self.m_spaceX)) * self.m_column + 1
		endIndex = math.ceil(pb / (self.m_itemWidth + self.m_spaceX)) * self.m_column
	end
	self.m_startIndex = startIndex

	for i = startIndex , endIndex do
		local pos = i - startIndex + 1
		if i <= #dataList and not self.m_views[pos] then
			local view = handler(dataList[i], i)
			view:setAnchorPoint(0.5,0.5)
			view:setVisible(false)
			view:setName("updateItem"..i)
			self:addChild(view)
			self.m_views[pos] = view
		end
	end

	for i = 1 , #self.m_views do
		local view = self.m_views[i]
		local index = startIndex - 1 + i
		view.index = index
		if index <= endIndex and dataList[index] then
			view:setModel(dataList[index],index)
			view:setVisible(true)
			local x , y = self:getPosByIndex(index)
			view:setPosition(x,y)
		else
			view:setVisible(false)
		end
	end
end

function This:getViewByIndex(index)
	if index >= self.m_startIndex and index <= #self.m_dataList then
		return self.m_views[index - self.m_startIndex + 1]
	end
end

function This:refreshContentSize(itemCount)
	if self.m_itemCount ~= itemCount then
		self.m_itemCount = itemCount
		self.m_row = math.ceil(self.m_itemCount / self.m_column)
		local w , h = self.m_scrollView:getContentSize()

		if self.m_isVertical then
			self.m_width = (self.m_itemWidth + self.m_spaceX ) * self.m_column  
			self.m_height = (self.m_itemHeight + self.m_spaceY) * self.m_row
			if self.m_height < h then
				self.m_height = h
			end
		else
			self.m_width = (self.m_itemWidth + self.m_spaceX ) * self.m_row   
			self.m_height = (self.m_itemHeight + self.m_spaceY) * self.m_column
			if self.m_width < w then
				self.m_width = w
			end
		end

		self:setContentSize(self.m_width,self.m_height)	
		self.m_scrollView:setInnerContainerSize(self.m_width,self.m_height)	
		self.m_layer:setContentSize(self.m_width,self.m_height)
	end
end

function This:refreshPosition()
	for i , view in ipairs(self.m_views) do
		local x , y = self:getPosByIndex(view.index)
		view:setPosition(x,y)
	end
end

function This:setDataList(dataList, createHandler, finishHandler)
	self:refreshContentSize(#dataList)
	self.m_dataList = dataList
	if not self.m_isInit then
		self:initItemView(dataList, createHandler, finishHandler)
	else
		self:updateItemView(dataList,createHandler)
		if finishHandler then
			finishHandler()
		end
	end
end

function This:swapNode(node1,node2)
	local key1 , key2
	for key , node in pairs(self.m_views) do
		if node == node1 then
			key1 = key
		elseif node == node2 then
			key2 = key
		end
	end
	if key1 and key2 then
		local x1 , y1 = self:getPosByIndex(node1.index)
		local x2 , y2 = self:getPosByIndex(node2.index)
		node1:setPosition(x2,y2)
		node2:setPosition(x1,y1)
		local tmp = node1.index
		node1.index = node2.index
		node2.index = tmp
		self.m_views[key1] = node2
		self.m_views[key2] = node1
	else
		assert(false)
	end
end

function This:clearNode(remove, cleanup)
	if not remove then
		for i , view in pairs(self.m_views) do
			view:setVisible(false)
		end
	else
		for i , view in pairs(self.m_views) do
			view:removeFromParent(cleanup)
		end
		self.m_views = {}
	end
	self.m_dataList = {}
	self:refreshContentSize(0)
end

function This:update(dt)
	self.m_timer:update(dt)
end

function This:isInited()
    return self.m_isInit
end

function This:getAllViews()
	return self.m_views
end

--endregion
