local This = {}
local Vec = require("ffi").metaclass("Vec", This)
cc.Vec = Vec
This.ZERO = Vec()
local sqrt = math.sqrt
local atan2 = math.atan2
local toDegree = math.deg
local toRadian = math.rad
local acos = math.acos

function This.__tostring(v)
    return string.format("x:%g, y:%g", v.x, v.y)
end

function This.__add(v1, v2)
    return Vec(v1.x + v2.x, v1.y + v2.y)
end

function This.__sub(v1, v2)
    return Vec(v1.x - v2.x, v1.y - v2.y)
end

function This.__mul(v1, v2)
    return Vec(v1.x * v2, v1.y * v2)
end

function This.__unm(v)
    return Vec(-v.x, -v.y)
end

function This.__div(v1, v2)
    return Vec(v1.x / v2.x, v1.y / v2.y)
end

function This:sub(v)
    return Vec(self.x - v.x, self.y - v.y)
end

function This:mul(v)
    self.x = self.x * v
    self.y = self.y * v
    return self
end

function This.createByRadian(radian)
    return Vec(math.cos(radian), math.sin(radian))
end
function This.createByDegree(degree)
    return This.createByRadian(toRadian(degree))
end

local function normalizeXY(x, y)
    local n = x * x + y * y
    if n == 1.0 or n < 0.00001 then
        return 0, 0
    end
    n = 1.0 / sqrt(n)
    return x * n, y * n
end
This.normalizeXY = normalizeXY
function This:normalize()
    local x, y = normalizeXY(self.x, self.y)
    if x then
        self.x = x
        self.y = y
    end
    return self
end

function This:getNormalize()
    return This.normalize(Vec(self.x, self.y))
end

function This:dot(v)
    return self.x * v.x + self.y * v.y
end

function This:dotXY(x, y)
    return self.x * x + self.y * y
end

function This:cross(v)
    return self.x * v.y - self.y * v.x
end

function This:rotateBy(v)
    return Vec(self.x * v.x - self.y * v.y , self.x * v.y + self.y * v.x)
end    

function This:getLPerp()
    return Vec(-self.y, self.x)
end

function This:getRPerp()
    return Vec(self.y, -self.x)
end

function This:getLPerpXY()
    return -self.y, self.x
end

function This:getRPerpXY()
    return self.y, -self.x
end

function This:length()
    local n = self:dot(self)
    if n == 1.0 or n < 0.0000001 then
        return 0
    end
    
    return sqrt(n)
end

function This:length2()
    return self.x * self.x + self.y * self.y
end

function This:getRadian()
    return atan2(self.y, self.x)
end

function This:getDegree()
    return toDegree(self:getRadian())
end

function This:getDegreeByVec(v)   --a * b = /a//b/cos0
    return toDegree(self:getRadianByVec(v))
end

function This:getRadianByVec(v)   --a * b = /a//b/cos0
    return acos((self:dot(v)) / (self:length() * v:length()))
end


function This:localToWorld(relativeVec)
    return self:rotateBy(relativeVec)
end

function This:getXY()
    return self.x, self.y
end

function This.getDistance2(v1, v2)
    local x = v2.x - v1.x
    local y = v2.y - v1.y
    return x * x + y * y
end

function This.getDistance(v1, v2)
    local x = v2.x - v1.x
    local y = v2.y - v1.y
    local d2 =  x * x + y * y
    if d2 > 0.1 then
        return sqrt(d2)
    else
        return 0
    end
end


function This:getDirection(t)
    return normalizeXY(t.x - self.x, t.y - self.y)
end

function This:getDirectionXY(x, y)
    return normalizeXY(x - self.x, y - self.y)
end

function This:isInCricle(targetXY, radius)
    local x = targetXY.x - self.x
    local y = targetXY.y - self.y
    local distance2 = x * x + y * y
    if distance2 <= radius * radius then
        return true
    end
    return false
end

return Vec