-- @Author: superm
-- @Date:   2018-01-09 16:17:04
-- @Last Modified by:   zheng
-- @Last Modified time: 2018-12-08 17:44:46
-- @File Name: PrivilegeManager.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")

local VipManager = require("Vip.VipManager")
local MatchManager = require("Match.MatchManager")
local GamePlayer = require("Player.GamePlayer")
local NobilityManager = require("Nobility.NobilityManager")

function This:isPrivilegeScopeType( ptype, stype )
    local scope = PrivlScopeMap[ptype]
    return bit.band(scope, stype) == stype
end

function This:isMixScopeType( ptype )
    return bit.band(PrivlScopeMap[ptype], PrivlScopeType.kMIX) == PrivlScopeType.kMIX
end

function This:hasPrivilege( privilegeType, withWarming )
    local continue = self:isMixScopeType(privilegeType) 
    local result, model = false, nil

    if self:isPrivilegeScopeType(privilegeType, PrivlScopeType.kVIP) then
        result, model = VipManager:hasPrivilege( PrivilegeField[privilegeType], withWarming )
        if not continue then
            return result, model
        elseif result then
            return true
        end
    end

    if self:isPrivilegeScopeType(privilegeType, PrivlScopeType.kOffice) then
        result = MatchManager:getPrivilegeNumBy( PrivilegeField[privilegeType] ) > 0
        if not continue then
            return result, model
        elseif result then
            return true
        end
    end

    if self:isPrivilegeScopeType(privilegeType, PrivlScopeType.kLevel) then
        local level = PrivlLevelMap[privilegeType] 
        if level then
            result = GamePlayer:getLevel() >= level
            model = model or level
        end

        if withWarming then
            withWarming = not result
            if not result then
                local des = string.format(Serialize["lev_priv_warming_" .. PrivilegeField[privilegeType]], level)
                g_scene:showTips(des)
            end
        end
    end
    return result, model
end

function This:getPrivilegeNumBy( privilegeType )
    if privilegeType == Privilege.kBuildTime then
        local vipAdd = VipManager:getPrivilegeNumBy( PrivilegeField[privilegeType] )
        local nobAdd = NobilityManager:getPrivilegeNumBy( PrivilegeField[privilegeType] )
        return vipAdd + nobAdd
    end


	local continue = self:isMixScopeType(privilegeType) 
    if not continue then
        if self:isPrivilegeScopeType(privilegeType, PrivlScopeType.kOffice) then
           return MatchManager:getPrivilegeNumBy( PrivilegeField[privilegeType] )
        end

        if self:isPrivilegeScopeType(privilegeType, PrivlScopeType.kVIP) then
            return VipManager:getPrivilegeNumBy( PrivilegeField[privilegeType] )
        end
    else
        local level = PrivlLevelMap[privilegeType] 
        if level and GamePlayer:getLevel() >= level then
            return 0
        end

        if self:isPrivilegeScopeType(privilegeType, PrivlScopeType.kVIP) then
            return VipManager:getPrivilegeNumBy( PrivilegeField[privilegeType] )
        end
    end
end

function This:isPrivilegeNumEnough( privilegeType, targetNum, withWarming )
    local ptype = PrivilegeField[privilegeType]
    return VipManager:isPrivilegeNumEnough( ptype, targetNum, withWarming )
end

function This:isFullVipMember()
    return VipManager:isFullMember()
end