local This, super = class(...,"Dialog")
local VipManager = require("Vip.VipManager")
local SystemManager = require("System.SystemManager")
local HeroManager = require("Hero.HeroManager")
local GameUtil = require("Util.GameUtil")
local ItemNode = require("UI.ItemNode")
local ViewUtil = require("UI.ViewUtil")

local kVipDefaultDays = 30

function This:initNode(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/MemberMain.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    local models = VipManager:getMemberModels()
    for i=1, VipType.kMax do
        self:initItemList(i, models[i])
    end   
end

function This:initTouchEvent()
    self.m_csNode.closebg:setTouchEnded(function()
        self:close()
    end)

    local models = VipManager:getMemberModels()
    for i=1, VipType.kMax do
        self:initItemTouch(i, models[i])
    end   
end


function This:onEnter()
    super.onEnter(self)
    SystemManager:addObserver(self)
    VipManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    local models = VipManager:getMemberModels()
    for i=1, VipType.kMax do
        models[i]:removeObserver(self)
    end 
    VipManager:removeObserver(self)
    SystemManager:removeObserver(self)
    -- g_scene:setIsEnableShowTips(true)
    super.onExit(self)
end

function This:refresh()
    local models = VipManager:getMemberModels()
    for i=1, VipType.kMax do
        models[i]:removeObserver(self)
        if models[i]:isUpdating() and models[i]:getDays() < 1 then
            models[i]:addObserver(self)
        end
        self:refreshItemList(i, models[i])
    end 

    self:refreshProgress()
end

function This:initItemList(index, model)
    local price = model:getConfigPrice()

    local beginPosX = self.m_csNode.recharge:getPositionX()
    local curMoneyNode = self.m_csNode["money" .. index]
    local width = self.m_csNode.recharge:getContentWidth() * self.m_csNode.recharge:getScaleX() - 10
    local progress = price / model:getMaxPrice()
    curMoneyNode:setPositionX(beginPosX + width * progress) 
    curMoneyNode.text:setString(tostring(price))

    local curNode = self.m_csNode[VipName[index] .. "bg"]

    -- curNode.title:setString(VipName[index])
    -- curNode.icon:setImage(model:getIcon())
    -- curNode.condition:setString(Serialize[64820] .. model:getConfigPrice() .. Serialize[64821])
    curNode.condition:setString(string.format(Serialize[64820],model:getConfigPrice()))
end

function This:initItemTouch(index, model)
    local curNode = self.m_csNode[VipName[index] .. "bg"]
    curNode.receive:setTouchEnded(function()
        if model:isUpdating() then
            local hasFirstReward = model:hasFirstReward()
            local hasDailyReward = model:hasDailyReward()
            local hasRenewReward = model:hasRenewReward()
            if hasFirstReward  then
                -- g_scene:setIsEnableShowTips(false)
                VipManager:onGetVipRewardRequest(model:getId(), VipReward.kActive)
            elseif hasRenewReward then
                VipManager:onGetVipRewardRequest(model:getId(), VipReward.kRenew)
            elseif hasDailyReward then
                VipManager:onGetVipRewardRequest(model:getId(), VipReward.kDaily)
            end
        end
    end)

    curNode.activation:setTouchEnded(function()
        if not model:isUpdating() then
            g_scene:openDialog("System.RechargeMainUI")
        end
    end)

    curNode.tips:setTouchEnded(function()
        g_scene:openDialog("Vip.VipPrivilegePanel")
    end)
end

function This:refreshProgress( )
    local models = VipManager:getMemberModels()
    local progress = VipManager:getCurPrice() / models[1]:getMaxPrice()
    self.m_csNode.recharge:setPercent(progress * 100)
    self.m_csNode.recharge:setVisible(true)
end

function This:getExpression(expression, tRet, fRet)
    if expression then
        return tRet
    else
        return fRet
    end
end

function This:refreshItemList(index, model)
    local curNode = self.m_csNode[VipName[index] .. "bg"]
    local curConfig = model:getConfig()

    local hasFirstReward = model:hasFirstReward()
    local hasDailyReward = model:hasDailyReward()
    local hasRenewReward = model:hasRenewReward()
    local hasVip = model:isUpdating()
    local hasFirstRecharge = model:hasFirstRecharge()

    local showFirstReward = hasFirstReward
    local showRenewReward = (not hasFirstReward) and self:getExpression(hasVip, hasRenewReward, not hasFirstRecharge)

    curNode.tips:setVisible(false)
    curNode.dailyReward:setString(showRenewReward and Serialize[64822] or VipName[index] .. Serialize[64823])
    if hasVip then
        curNode.days:setString("（" .. tostring(model:getDays()) .. Serialize[64824])
        curNode.notactive:setVisible(false)
        curNode.condition:setVisible(false)
        curNode.activation:setVisible(false)

        curNode.activated:setVisible(true)
        if hasFirstReward or hasDailyReward or hasRenewReward then
            curNode.firstReward:setVisible(showFirstReward)
            curNode.dailyReward:setVisible(not showFirstReward)

            curNode.icon:setVisible(showFirstReward)
            curNode.desc:setVisible(not showFirstReward)
            ViewUtil:blinkView(curNode.tips, showFirstReward)
            curNode.receive:setVisible(true)
            curNode.received:setVisible(false)
        else
            curNode.firstReward:setVisible(false)
            curNode.dailyReward:setVisible(true)
            curNode.icon:setVisible(false)
            curNode.desc:setVisible(true)
            ViewUtil:blinkView(curNode.tips, false)
            curNode.receive:setVisible(false)
            curNode.received:setVisible(true)
        end
    else
        curNode.days:setString("（" .. kVipDefaultDays .. Serialize[64824])
        curNode.notactive:setVisible(true)
        curNode.firstReward:setVisible(true)
        curNode.icon:setVisible(true)
        curNode.condition:setVisible(true)
        curNode.activation:setVisible(true)

        curNode.activated:setVisible(false)
        curNode.dailyReward:setVisible(false)
        curNode.desc:setVisible(false)
        curNode.received:setVisible(false)
        curNode.receive:setVisible(false)
        ViewUtil:blinkView(curNode.tips, true)
        if hasFirstReward or hasDailyReward or hasRenewReward or showRenewReward then
            curNode.firstReward:setVisible(showFirstReward)
            curNode.dailyReward:setVisible(not showFirstReward)
            curNode.icon:setVisible(showFirstReward)
            curNode.desc:setVisible(not showFirstReward)
            ViewUtil:blinkView(curNode.tips, showFirstReward)
        else
            showFirstReward = true
        end
    end

    GameUtil:linearLayoutNodes(3, {curNode.title, curNode.days}, true, nil, curNode.tips:getContentWidth())
    print_r({
        hasVip = hasVip,
        hasFirstRecharge = hasFirstRecharge,
        hasFirstReward = hasFirstReward, 
        hasDailyReward = hasDailyReward, 
        hasRenewReward = hasRenewReward,
        showFirstReward = showFirstReward,
        showRenewReward = showRenewReward,
    })
    local reward
    if showFirstReward then
        reward = curConfig.active_reward 
    elseif showRenewReward then
        reward = curConfig.continue_reward
    else
        reward = curConfig.daily_reward
    end
    local rewardCount = #reward
    local curRewardNode = curNode.scrollview
    curRewardNode:setDirection("horizontal")
    curRewardNode.m_direction="right"
    curRewardNode:removeAllItems()
    for i=1, rewardCount do
        local item = reward[i]
        -- local curRewardNode = curNode["reward" .. i]
        if curRewardNode then
            local node = curRewardNode.node
            if item then
                if not node then
                    node = ItemNode:create(item[1], item[2])
                    node:setAnchorPoint(0.5, 0.5)
                    node:setScale(0.8)
                    local w, h = node:getContentSize()
                    -- node:setPosition(w * 0.5, h * 0.5)
                    -- curRewardNode:addChild(node)
                    -- curRewardNode.node = node
                    curRewardNode:pushBackCustomItem(node,w-20,h,0.5)
                else
                    node:init(item[1], item[2])
                    node:setVisible(true)
                end
            else
                if node then
                    node:setVisible(false)
                end
            end
        end
    end

end

-------------------------------------
function This:onGetVipInfo( )
    self:refresh()
end

function This:onBuyShopItem(itemLists)
    if itemLists and #itemLists > 0 then
        self:refreshProgress()
    end
end

function This:onGetVipReward( msg, model )
    local rewardType = msg.type
    local curConfig = model:getConfig()
    local reward 

    if rewardType == VipReward.kActive then
        reward = curConfig.active_reward 
    elseif rewardType == VipReward.kRenew then
        reward = curConfig.continue_reward
    else
        reward = curConfig.daily_reward
    end

    local itemList = {}
    for i , info in ipairs(reward) do
        table.insert(itemList,{id = info[1], num = info[2]})
    end
    if itemList and #itemList > 0 then
        g_scene:showReward(itemList)
    end

    local index = msg.id
    if index > VipType.kNone and index <= VipType.kMax and model then
        self:refreshItemList( index, model )
    end
end

function This:updateVipTime( info )
    if info then
        local curNode = self.m_csNode[VipName[info.id] .. "bg"]
        if info.time > 0 then
            curNode.days:setString(GameUtil:formatTime("（" .. Serialize["H_M_S"] .. "）", info.time))
        else
            curNode.days:setString("（" .. tostring(kVipDefaultDays) .. Serialize[64824])   
        end
    end
end