-- @Author: Su
-- @Date:   2019-05-06 15:29:31
-- @Last Modified by:   Su
-- @Last Modified time: 2019-05-06 15:29:31
-- @File Name: AnimalTeamWarRefreshView.lua
-- @Description:

local This , super = class(...,"NodeView")
local WarfareManager = require("Warfare.WarfareManager")
local GamePlayer = require("Player.GamePlayer")
local ArgConfig = require("Metadatas.ArgConfig")
local VipManager = require("Vip.VipManager")
local VipConfig = require("Metadatas.VipConfig")  

function This:initData(info)
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TeamWarRefresh.csb")
    self.m_isInit = false
    self:addChild(self.m_csNode)
    self.m_build = info.build
    local w, h = self.m_csNode.bg:getContentSize()
    self:setContentSize(w-15, h + 40)
    self.m_fightLimit = ArgConfig.mythical_campaign_refresh_times.arg[1]
    self.m_refreshPrice = ArgConfig.mythical_campaign_refresh_cost.arg[1]
    self.m_orangeRefreshPrice = ArgConfig.mythical_campaign_refresh_orange_cost.arg[1]
end

function This:onEnter()
    super.onEnter(self)
    self.m_build:addObserver(self)
    WarfareManager:addObserver(self)
end

function This:onExit()
    WarfareManager:removeObserver(self)
    self.m_build:removeObserver(self)
    super.onExit(self)
end

function This:initNode()
    self:refresh()
end

function This:initTouchEvent()
    self.m_csNode.Button:setTouchEnded(function()
        print("刷新战役") 
        g_scene:openDialog("UI.CostConfirmPanel", {
            cost = self.m_refreshPrice,
            desc = Serialize[64829],
            okHandler = function()
                if GamePlayer:isEnoughDiamond(self.m_refreshPrice) then
                    self.m_build:onRefreshAnimalWarRequest(1)                    
                else
                    g_scene:showTips(Serialize[60153])
                end
            end
        })
    end)

    self.m_csNode.yijian:setTouchEnded(function()
        if not self:isLock() then
            g_scene:showTips(Serialize[64830])
            return
        end
        g_scene:openDialog("UI.CostConfirmPanel", {
            cost = self.m_orangeRefreshPrice,
            desc = Serialize[64829],
            okHandler = function()
                if GamePlayer:isEnoughDiamond(self.m_orangeRefreshPrice) then
                    self.m_build:onRefreshAnimalWarRequest(2)                    
                else
                    g_scene:showTips(Serialize[60153])
                end
            end
        })
    end)

    self.m_csNode.wenhao:setTouchEnded(function()
        g_scene:openDialog("UI.HelpTips",{text = Serialize[64831],node=self.m_csNode.wenhao})
    end)
end

function This:refresh()
    self.m_csNode.text:setString(self.m_refreshPrice)
    local fightTimes = self.m_build:getAnimalWarFightTime()
    local num = self.m_fightLimit - fightTimes
    if num > 0 then
        self.m_csNode.cishu:setStringWithColor(string.format(Serialize[64832],num))
    else
        self.m_csNode.cishu:setStringWithColor(string.format(Serialize[64833],0))
    end
end

function This:isLock()
    local vipLevel = VipManager:getMaxMemberLevel() or 0
    return VipConfig[vipLevel].refresh_unlock == 1
end
