-- @Author: superm
-- @Date:   2018-07-21 17:32:30
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-12-19 16:02:06
-- @File Name: InviteWarfarePopPanel.lua
-- @Description: 
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")

local This , super = class(...,"Match.InviteMatchPopPanel")
local WarfareManager = require("Warfare.WarfareManager")
local TowerManager = require("SingleDaily.TowerManager")
local WarTowerManager = require("SingleDaily.WarTowerManager")
local ViewUtil = require("UI.ViewUtil")

function This:initData( info )
    if info then
		self.m_team = WarfareManager:createTeam(info.teamInfo, info.teamType)
		self.m_role = self.m_team:getOwner()
    end
end

function This:refreshContent(info)
	self.m_csNode.type:setString(self.m_team:getWarFullName())
	self.m_csNode.text:setString(Serialize[64838])
end

function This:addCustomListener()
	WarfareManager:addObserver(self)
end

function This:removeCustomListener()
	WarfareManager:removeObserver(self)
end

function This:reponseRequest( value )
	if not value then
		return self:close()
	end

	local teamType = self.m_team:getType() 
    if teamType == TeamType.kTowerTeam then
		local openLv = ArgConfig["tower_open_level"].arg[1]
		local curentLv = GamePlayer:getLevel()
		if curentLv < openLv then
			return g_scene:showTips(string.format(Serialize["achieve_MainPanel_1_c"],openLv))
		end
	end
	if WarfareManager:hasTeam() then
		g_scene:showTips(Serialize[64547])
		self:close()
		return
	elseif WarfareManager:hasTeam(TeamType.kTowerTeam) then
		g_scene:showTips(Serialize[64839])
		self:close()
		return
	elseif WarfareManager:hasTeam(TeamType.kWarTowerTeam) then
		g_scene:showTips(Serialize[64840])
		self:close()
		return
	end

	WarfareManager:onJoinMatchTeamRequest(self.m_team:getId(), self.m_team:getType())
end

function This:onGetWarfareTabData(str)
	if str == "onJoinMatchTeam" then
		if WarfareManager:hasTeam() then
			local team = WarfareManager:getMyTeam()
			local war = team:getWar()
			local params = {
				tab = 2, 
				args = { id = team:getWarId() }
			}
			if war:getType() == WarfareType.kAnimal then
				ViewUtil:jumpToPanel("TeamWarfarePanel", {tab = 4,args = {war = war}})
			else
				ViewUtil:jumpToPanel("TeamWarfarePanel", params)
			end
		else
			ViewUtil:jumpToPanel("TeamWarfarePanel", {tab = 2})
		end
		self:close()
	end
end

function This:onJoinMatchTeam(teamType)
    if self.m_team:getType() == teamType then
    	local function setHeroInfo(heros)
    		if heros then
                local heroList = {}
                for i, heroId in pairs(heros) do
                    heroList[i] = { pos = i, heroId = heroId }
                end
                WarfareManager:onWarEmbattleHerosRequest(heroList, teamType)
                return true
            end
            return false
    	end
    	if teamType == TeamType.kTowerTeam then
			local heros = TowerManager:getLevelHeroList()
            if setHeroInfo(heros) then
            else
				local team = WarfareManager:getMyTeam(teamType)
	        	local info = {stageId= team:getWarId()}
				if WarfareManager:hasTeam() then
					g_scene:showTips(Serialize[64547])
					self:close()
					return 
				end
	        	g_scene:openDialog("SingleDaily.TowerPanel",testRunAttr)
				self:close()            	
            end
	    elseif teamType == TeamType.kWarTowerTeam then
			local team = WarfareManager:getMyTeam(teamType)
			local heros = WarTowerManager:getLevelHeroList()
            if setHeroInfo(heros) then
            	-- WarfareManager:onPrepareWarRequest(true, teamType)
            	g_scene:openDialog("SingleDaily.WarTowerTeamPanel",{lv =team:getWarId()})
				self:close()            	
            else
				if WarfareManager:hasTeam() then
					g_scene:showTips(Serialize[64547])
					self:close()
					return 
				end
				g_scene:openDialog("SingleDaily.WarTowerTeamPanel",{lv =team:getWarId()})
				self:close()            	
            end
	    end
    end
end

function This:onWarEmbattleHeros( teamType, success )
    if teamType == TeamType.kTowerTeam then
		local team = WarfareManager:getMyTeam(teamType)
    	local info = {stageId= team:getWarId()}
		-- testRunStr ="SingleDaily.TowerPanel"
    	g_scene:openDialog("SingleDaily.TowerPanel",testRunAttr)
        self:close()
    end
end

function This:onJoinMatchTeamFail()
	self:close()
end