-- @Author: superm
-- @Date:   2018-07-20 15:33:33
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-20 12:11:34
-- @File Name: TeamWar.lua
-- @Description: 

local This , super = class(...,"Base.BaseModel")
local TeamCampaignConfig = require("Metadatas.TeamCampaignConfig")
local TeamNornalCampaignNpcConfig = require("Metadatas.TeamNornalCampaignNpcConfig")
local TeamVipCampaignNpcConfig = require("Metadatas.TeamVipCampaignNpcConfig")
local GamePlayer = require("Player.GamePlayer")
local ItemUtil = require("Bag.ItemUtil")
local VipManager = require("Vip.VipManager")

function This:init(info)
	super.init(self)
	if info then
		if info.config then
			self.m_config = info.config
			self.m_id = self.m_config.id
		else
			self.m_id = info.id
			self.m_config = TeamCampaignConfig[self.m_id]
		end
		self.m_type = self.m_config.tpye
	end
	self.m_challengeTimes = 0
	self.m_challengeCaptainTimes = 0
	self.m_maxHard = 1
	
	self.m_escape = 0
	self.m_startTime = 0
	self.m_endTime = 0
	self.m_isUpdating = false

	self.m_jifen = 0
	self.m_boxList = info.boxlist or {}
	if info.reward_record then
		self.m_rewardedIdList = {}
	    for _, id in ipairs(info.reward_record) do
	     	self.m_rewardedIdList[id] = true
	    end
	end
end

function This:updateInfo(info)
    if info.scoring then
		self.m_jifen = info.scoring
	end

	if info.reward_times then
		self.m_challengeTimes = info.reward_times
		self.m_challengeCaptainTimes = info.captain_reward_times
	end

	if info.copy_level then
		self.m_maxHard = info.copy_level
	end

	if info.reward_record then
		self.m_rewardedIdList = {}
	    for _, id in ipairs(info.reward_record) do
	     	self.m_rewardedIdList[id] = true
	    end
	end
	
    for _, box in ipairs(self.m_boxList) do
    	if self.m_rewardedIdList[box.id] then
    		box.state = SaintBoxState.kClose
    	elseif self.m_jifen >= box.jifen then
    		box.state = SaintBoxState.kOpen
    	else
    		box.state = SaintBoxState.kLock
    	end
    end

	self:updateState()
end

function This:updateState()
	self:updateTimeInfo()
	self.m_state = self:getState()
end

function This:updateTimeInfo()
	local curTime = GamePlayer:getServerTime()
    local curDate = os.serverdate("*t", curTime)

    local stime, etime = unpack(self.m_config.time)
    if stime < 0 and etime < 0 then
		self.m_startTime = -1
		self.m_endTime = -1
		return
    end
	local zeroTime = os.time({day = curDate.day, month = curDate.month, year = curDate.year, hour = 0, min = 0, sec = 0})
	self.m_startTime = zeroTime + stime or 0
	self.m_endTime = zeroTime + etime or 0

	if self.m_id > 0 and self.m_startTime > 0 then
		local curTime = GamePlayer:getServerTime()
		if curTime < self.m_endTime then
			if not self.m_isUpdating then
				self:startUpdate()
			end
			return
		end
	end

	if self.m_isUpdating then
		self:endUpdate()
	end  
end

function This:getState()
	if self.m_startTime <= 0 then
		return TeamWarState.kNotOpen
	end

	if not self:isOpening() then
		return TeamWarState.kNotOpen
	end

	local curTime = GamePlayer:getServerTime()
	if curTime > self.m_endTime then
		return TeamWarState.kFinish
	elseif curTime < self.m_startTime then
		return TeamWarState.kNotStart
	end

	return TeamWarState.kActive
end

function This:getName()
    return self.m_config.name
end

function This:getTicketCost()
	local times = self.m_challengeCaptainTimes
	local config = self.m_config.cost
    return config[times + 1] or config[#config]
end

function This:getTicketId()
    return self.m_config.ticket_ID
end

function This:getCostDesc()
	local id = self:getTicketId()
	if id <= 0 then
		return Serialize[60243]
	end

	local costNum = self:getTicketCost()
	if costNum <= 0 then
		return Serialize[60243]
	end
    return ItemUtil.getName(id) .. " X " .. costNum
end

function This:convertChallengeCost(cost)
    cost = cost or self:getTicketCost()
	local times = self.m_challengeCaptainTimes
	local config = self.m_config.change_cost
    local change = config[times + 1] or config[#config]
    return cost * change
end

function This:getMaxChallengeTime()
    return self.m_config.cost
end

function This:getConfig()
    return self.m_config
end

function This:isPreparing()
	return self.m_state == TeamWarState.kNotStart
end

local order = {Serialize[60012],Serialize[60013],Serialize[60014],Serialize[60015],Serialize[60016],Serialize[60097],Serialize[60978]}
function This:isOpening()
	if not self:isLevelOpen() then
		return false, string.format(Serialize[60304], self.m_config.level)
	end

	-- if not self:isVipOpen() then
	-- 	return false, Serialize[64847]
	-- end

	local curTime = GamePlayer:getServerTime()
    local curDay = os.serverdate("*t", curTime).wday
    local wday = (curDay + 5) % 7 + 1
	local validDayList = self.m_config.day
	if not table.contain(validDayList, wday) then
		local desc = {}
		for i,v in ipairs(validDayList) do
			desc[i] = order[v]
		end
		return false, string.format(Serialize[64848], table.concat(desc, "、"))
	end

	return true
end

function This:isLevelOpen()
	local level = GamePlayer:getLevel()
    return level >= self.m_config.level
end

function This:isVipOpen()
	if self:doNeedVip() then
		local vipList = self.m_config.vip
		local vipState = VipManager:getMemberState()
		local ret = false
		for i , vip in ipairs(vipList) do
			if vipState[vip] then
				ret = true
				break
			end
		end
		return ret
	end
	return true
end

function This:getAvailableHeroList()
	if self.m_config.hero and next(self.m_config.hero) then
    	return self.m_config.hero
    end
end

function This:getLeftRewardTimes(isCaptain)
	local value, max
	local wtype = self:getType()
    if wtype == WarfareType.kSingle then
        local ChallengeStageManager = require("Stage.ChallengeStageManager")
        local SingleManager = require("SingleDaily.SingleManager")

        local isCan ,challengeTimes 
        local id = self:getId()
        if id ~= 4 then
            isCan, challengeTimes = SingleManager:getChallengeTimes(id)
        else
        	isCan, challengeTimes = ChallengeStageManager:getChallengeTimes()
        end
        max = self.m_config.reward_times 
        value = challengeTimes
    else
		if not isCaptain then
			value, max = self.m_config.reward_times - self.m_challengeTimes, self.m_config.reward_times 
		else
		 	value, max = self.m_config.captain_times - self.m_challengeCaptainTimes, self.m_config.captain_times
		end
    end

	if value < 0 then
		value = 0
	end
    return value, max 
end

function This:getId()
    return self.m_id
end

function This:getType()
    return self.m_type
end

function This:doNeedVip()
	local vipList = self.m_config.vip
	if not vipList or table.contain(vipList, 0) then
		return false
	end
	return true
end

function This:getRewardList()
    return self.m_config.reward_list
end

function This:getBoxRewardList()
    return self.m_boxList
end

function This:hasBoxReward()
	for _, box in ipairs(self.m_boxList) do
		if box.state == SaintBoxState.kOpen then
			return true
		end
	end
	return false
end

function This:getScore()
    return self.m_jifen
end

function This:getNextHard()
    return self.m_maxHard
end

----------------------------
function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
	    local curTime = GamePlayer:getServerTime()
	    local leftOpenTime = self.m_startTime - curTime
	    if leftOpenTime > 0 then
	    	self:notifyObservers("updateOpenTime", leftOpenTime)
	    else
	    	if self:isPreparing() then
	    		self:updateState()
	    		self:notifyObservers("onWarOpenning", self)
	    	end

	    	local leftTime = self.m_endTime - curTime
	    	if leftTime > 0 then
				self:notifyObservers("updateTime", leftTime)
			else
	            self:updateState()
	        end
		end
	end
end

function This:startUpdate()
	cc.Application:addToUpdateList(self)
	self.m_isUpdating = true
	self:notifyObservers("onWarUpdateStart", self)
end

function This:endUpdate()
	cc.Application:removeFromUpdateList(self)
	self.m_isUpdating = false
	self:notifyObservers("onWarUpdateEnd", self)
end

function This:getLeftTime()
    local curTime = GamePlayer:getServerTime()
    local offset = self.m_endTime - curTime
    if offset < 0 then
        return 0
    end
    return offset
end

function This:getReawrdWithoutTeam( hard, level )
	local id = self:getId() * 10000 + (hard or 1) * 1000 + level
	local config 
	if self:getType() ~= WarfareType.kVip then
		config = TeamNornalCampaignNpcConfig[id]
	else
		config = TeamVipCampaignNpcConfig[id]
	end
	return config
end