-- @Author: superm
-- @Date:   2018-08-18 15:26:33
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-12-02 14:48:36
-- @File Name: TeamWarRoleView.lua
-- @Description: 

local This , super = class(..., "NodeView")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local WarfareManager = require("Warfare.WarfareManager")
local WarTowerManager = require("SingleDaily.WarTowerManager")

local TeamViewFile = {
    [TeamType.kWarTeam] = "MenuScene/NodeUI/TeamWarFoundList.csb",
    [TeamType.kTowerTeam] = "MenuScene/NodeUI/TowerTeamList.csb",
    [TeamType.kWarTowerTeam] = "MenuScene/NodeUI/DrumTowerTeamList.csb",
}

function This:initData(role, team)
    self.m_role = role
    self.m_team = team 
    self.m_escape = 0
    self.m_leftTime = 0
	if team then
		self.m_teamType = team:getType()
		if self.m_teamType == TeamType.kWarTeam then
			self.m_war = self.m_team:getWar()
		end
		self.m_csNode = cc.Node:createFromCSFile(TeamViewFile[self.m_teamType])
		self:addChild(self.m_csNode)
		local w , h = self.m_csNode.bg:getContentSize()
		self:setContentSize(w, h)
	end
    self.m_inited = false
end

function This:onEnter()
    super.onEnter(self)
    if self.m_role then
    	self.m_role:setView(self)
   	end
end

function This:onExit()
    if self.m_role then
	    self.m_role:setView(nil)
	end
    super.onExit(self)
end

function This:initNode()
    self:refresh()
end

function This:refresh()
	if  not self.m_csNode and WarfareManager:hasTeam() then
		local team = WarfareManager:getMyTeam()
		self.m_teamType = team:getType()
		if self.m_teamType == TeamType.kWarTeam then
			self.m_war = self.m_team:getWar()
			self.m_bossWar = self.m_team:getBossWar()
		end

		self.m_csNode = cc.Node:createFromCSFile(TeamViewFile[self.m_teamType])
		self:addChild(self.m_csNode)
		local w , h = self.m_csNode.bg:getContentSize()
		self:setContentSize(w, h)
	end
	local csNode = self.m_csNode
	local ctNode = csNode.content
	local yqNode = csNode.yaoqing

	local valid = self.m_team and  self.m_team:getId() ~= 0
	local visible = self.m_role ~= nil
	if ctNode then
		ctNode:setVisible(visible)
		yqNode:setVisible(not visible)
	end
	local playerId = GamePlayer:getPlayerID()
	local isMyself = visible and self.m_role:getId() == playerId
	csNode.self:setVisible(isMyself)
	if visible and ctNode then

		local ownerId = self.m_team:getOwnerId()
		local isPlayerOnwer = ownerId == playerId
		local isRoleOwner = self.m_role:isRoomOwner()

		GameUtil:setPlayerIconNodeImage(ctNode.icon, self.m_role:getIcon())
		ctNode.level:setString("Lv." .. self.m_role:getLevel())
		ctNode.name:setString(self.m_role:getName())
		ctNode.zhanlibg:setVisible(valid)
		ctNode.zhanlinumber:setVisible(valid)
		ctNode.zuiqiangzhanli:setVisible(valid)
		if valid then
			ctNode.zhanlinumber:setString(self.m_role:getFightPower())
		end
		ctNode.duizhang:setVisible(isRoleOwner)
		ctNode.yizhunbei:setVisible(self.m_role:isReady())

		local showByOnwerPower = isPlayerOnwer and not isMyself
		if ctNode.setbutton then
			ctNode.setbutton:setVisible(showByOnwerPower)
		end
		if ctNode.tuichu then
			ctNode.tuichu:setVisible(valid and isMyself)
		end
		if yqNode.juntuan then
			yqNode.juntuan:setVisible(showByOnwerPower)
		end
		ctNode.close:setVisible(showByOnwerPower)
	end
	if yqNode.juntuan then
		local _,isGray = self:getTeamLeftTime()
		GameUtil:btnGray(yqNode.juntuan,isGray)
	end
	if yqNode.xiezhu then
		local xiezhuNum,isGray = self:getTeamLeftTime()
		yqNode.xiezhu.text:setString(string.format(Serialize[64863],xiezhuNum))
	end
	if self.m_war then
		if self.m_war and self.m_war:getType() == WarfareType.kAnimal or self.m_war:getType() == WarfareType.kBoss then
			yqNode.shijie:setVisible(false)
			ctNode.setbutton:setVisible(false)
		else
			if yqNode.shijie then
				yqNode.shijie:setVisible(true)
			end
		end
	end
end

function This:getTeamLeftTime()
	local res1,res2 =1, false
	if self.m_teamType and self.m_teamType == TeamType.kWarTowerTeam then
		res1= WarTowerManager:getTeamNum()
		res2= res1<=0
	end
	return res1,res2
end

function This:refreshByReady(cleanup)
	local visible = self.m_role ~= nil
	local valid = self.m_team:getId() ~= 0
	local ctNode = self.m_csNode.content
	if visible then
		ctNode.yizhunbei:setVisible(self.m_role:isReady())
		ctNode.zhanlibg:setVisible(valid)
		ctNode.zhanlinumber:setVisible(valid)
		ctNode.zuiqiangzhanli:setVisible(valid)
		if valid then
			ctNode.zhanlinumber:setString(self.m_role:getFightPower())
		end
		if cleanup then
			local playerId = GamePlayer:getPlayerID()
			local isMyself = visible and self.m_role:getId() == playerId
			local isRoleOwner = self.m_role:isRoomOwner()
			local ownerId = self.m_team:getOwnerId()
			local isPlayerOnwer = ownerId == playerId
			ctNode.duizhang:setVisible(isRoleOwner)
			local showByOnwerPower = isPlayerOnwer and not isMyself
			if ctNode.setbutton then
				ctNode.setbutton:setVisible(showByOnwerPower)
			end
			if ctNode.tuichu then
				ctNode.tuichu:setVisible(valid and isMyself)
			end
			local yqNode = self.m_csNode.yaoqing
			if yqNode.juntuan then
				yqNode.juntuan:setVisible(showByOnwerPower)
			end
			ctNode.close:setVisible(showByOnwerPower)
		end
	end
end

function This:initTouchEvent()
	local ctNode = self.m_csNode.content
	local yqNode = self.m_csNode.yaoqing
	ctNode.close:setTouchEnded(function()
		WarfareManager:onExitWarTeamRequest(self.m_role:getId(), self.m_teamType)
	end)

	if ctNode.tuichu then
		ctNode.tuichu:setTouchEnded(function()
			WarfareManager:onExitWarTeamRequest(self.m_role:getId(), self.m_teamType)
		end)
	end

	if ctNode.setbutton then
		ctNode.setbutton:setTouchEnded(function()
			-- 后端检测受让方VIP等级、和副本可挑战最大等级
			WarfareManager:onChangeTeamOwnerRequest(self.m_role:getId(), self.m_teamType)
		end)
	end

	if yqNode then
		if yqNode.haoyou then
			yqNode.haoyou:setTouchEnded(function()
				if self.m_team and self.m_team:getId() > 0 then
					local playerId = GamePlayer:getPlayerID()
					local myRole = self.m_team:getMainRole()
					if not myRole or not myRole:isRoomOwner(playerId) then
						g_scene:handleErrorCode(12504)
						return
					end

					if self.m_inviteHandler then
						self.m_inviteHandler(self.m_role)
					end
				end
			end)
		end

		if yqNode.haoyou and yqNode.shijie then
			yqNode.shijie.line:setTouchEnded(function()
				if self.m_team and self.m_team:getId() > 0 then
					local myRole = self.m_team:getMainRole()
					local playerId = GamePlayer:getPlayerID()
					if not myRole or not myRole:isRoomOwner(playerId) then
						g_scene:handleErrorCode(12504)
						return
					end
					WarfareManager:onApplyJoinkWarfareRequest(self.m_team)
				end
			end)
		end

		if yqNode.juntuan then
			if yqNode.juntuan.time then
				yqNode.juntuan.time:setString("")
			end
			yqNode.juntuan:setTouchEnded(function()
				local _,isGray = self:getTeamLeftTime()
				if isGray then
					return
				end
				if self.m_inviteHandler and self.m_leftTime <= 0 then
					self.m_inviteHandler(self.m_role, function(ret, leftTime)
						if ret and self.m_teamType and self.m_teamType == TeamType.kTowerTeam then
							self.m_leftTime = leftTime
							self.m_escape = 0
							self:scheduleUpdate()
						end
					end)
				end
			end)
		end
	end
end

function This:getModel()
	return self.m_role
end

function This:setModel(role)
	local changed = self.m_role ~= role
	if not self.m_team or changed then
		self.m_team = WarfareManager:getMyTeam(self.m_teamType)
	end

	if changed then
		if self.m_role then
			self.m_role:setView(nil)
		end
		self.m_role = role
		if role then 
			role:setView(self)
		end
	end
    self:refresh()
end

function This:setInviteHandler( handler )
	self.m_inviteHandler = handler
end

function This:update(dt)
	super.update(self,dt)
	if self.m_leftTime > 0 then
		if self.m_escape < 1.0 then
			self.m_escape = self.m_escape + dt
		else
			self:refreshSendBtn()
			self.m_escape = self.m_escape - 1.0
			self.m_leftTime = self.m_leftTime - 1.0
		end
	else
		self:unscheduleUpdate()
		self:refreshSendBtn()
	end
end

function This:refreshSendBtn()
	local node = self.m_csNode.yaoqing.juntuan
	if not node then return end
	if self.m_leftTime > 0 then
		node:setGrayShader()
		local str = string.format(Serialize["chat_ChatPanel_12_c"], math.ceil(self.m_leftTime))
		if node.time then
			node.time:setString(str)
		end
	else
		node:clearShader()
		if node.time then
			node.time:setString("")
		end
	end
end