-- @Author: superm
-- @Date:   2018-08-08 17:53:31
-- @Last Modified by:   superm
-- @Last Modified time: 2018-10-26 17:43:10
-- @File Name: TeamWarView.lua
-- @Description: 

local This , super = class(..., "NodeView")
local ItemNode = require("UI.ItemNode")
local WarfareManager = require("Warfare.WarfareManager")


function This:initData(war)
    self.m_war = war
    self.m_inited = false
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TeamWarListItem.csb")
    self:addChild(self.m_csNode)
    local w, h = self.m_csNode.kuang:getContentSize()
    self:setContentSize(w-15, h + 40)
end

function This:onEnter()
    super.onEnter(self)
    if self.m_war then
        self.m_war:updateState()
        self.m_war:addObserver(self)
    end
end

function This:onExit()
    if self.m_war then
        self.m_war:removeObserver(self)
    end
    super.onExit(self)
end

function This:initNode()
    self:refresh()
end

function This:initTouchEvent()
    if not self.m_inited then
        self.m_csNode.kuang:setTouch({
            onEnded = function(touch, point, params)
                if not self.m_war then
                    return
                end

                local ret, errorStr = self.m_war:isOpening()
                if not ret then
                    return g_scene:showTips(errorStr)
                end 

                if self.m_war:getLeftTime() > 0 then
                    local wtype = self.m_war:getType()
                    if wtype == WarfareType.kSingle then
                        local id = self.m_war:getId()
                        if id == 4 then
                            print("单人副本")
                            g_scene:openDialog("Stage.ChallengeMapPanel", params)
                        else
                            g_scene:openDialog("SingleDaily.SingleStagePanel",{chapterId = id })
                        end
                    elseif wtype == WarfareType.kTeam then
                        if WarfareManager:hasTeam(TeamType.kTowerTeam) then
                            g_scene:showTips(Serialize[64839])
                            return 
                        end
                        local myTeam = WarfareManager:getMyTeam()
                        if not WarfareManager:isTeamLeader(TeamType.kWarTeam) then
                            if myTeam and myTeam:getWarId() ~= 0 and  myTeam:getWarId() ~= self.m_war:getId() then 
                                g_scene:showTips(Serialize[64865])
                                return
                            end
                        end
                        if myTeam and myTeam:getWar() and myTeam:getWar():getType() == WarfareType.kAnimal then
                            g_scene:showTips(Serialize[64866])
                            return 
                        end

                        print("组队副本")
                        g_scene:openDialog("Warfare.TeamWarfarePanel", {war = self.m_war})

                    elseif wtype == WarfareType.kVip then
                        if WarfareManager:hasTeam(TeamType.kTowerTeam) then
                            g_scene:showTips(Serialize[64839])
                            return 
                        end
                        print("Vip副本")
                        g_scene:openDialog("Warfare.TeamWarfarePanel", {war = self.m_war})
                    end
                end
            end,
            isContainer = true,
            isScale = false
        })
        self.m_inited = true
    end
end

function This:onClick(args)
    self.m_csNode.kuang.m_touch:onEnded(nil,args)
end

function This:setModel(war)
    if self.m_war ~= war then
        if self.m_war then
            self.m_war:removeObserver(self)
        end
        self.m_war = war
        
        if war then 
            war:addObserver(self)
        end
    end
    self:refresh()
end

function This:refresh()
    local csNode = self.m_csNode
    local descNode = csNode.desc
    local rewardNode = csNode.rewardn

    local ret = not self.m_war
    csNode.kuang2:setVisible(ret)
    csNode.wordjqqd:setVisible(ret)
    csNode.pic:setVisible(not ret)
    csNode.name:setString(Serialize[64867])
    descNode:setVisible(not ret)
    rewardNode:setVisible(not ret)
    if ret then
        return
    end

    local config = self.m_war:getConfig()
    csNode.pic:setImage(config.entrance_bg)
    csNode.name:setString(self.m_war:getName())

    descNode:setVisible(config.des)
    if config.des then
        descNode.xiaoguo:setString(config.des)
    end

    ret = not self.m_war:isLevelOpen()
    rewardNode:setVisible(not ret)
    if ret then
        csNode.kuang:setGrayShader()
        csNode.pic:setGrayShader()
        csNode.namebg:setGrayShader()
        return
    else
        csNode.kuang:clearShader()
        csNode.pic:clearShader()
        csNode.namebg:clearShader()
    end

    local rewardList = self.m_war:getRewardList()
    for i=1, 3 do
        local pnode = rewardNode["item" .. i]
        local itemId = rewardList[i]
        if itemId and pnode then
            if not pnode.node then
                pnode.node = ItemNode:create(itemId)
                pnode.node:setAnchorPoint(0.5, 0.5)
                pnode:addChild(pnode.node)
            else
                pnode.node:init(itemId)
            end
        end
    end
    local times, max = self.m_war:getLeftRewardTimes()
    rewardNode.cishu:setString(string.format(Serialize[64868], times, max))
end
