-- @Author: superm
-- @Date:   2018-08-16 20:21:37
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-03 21:03:23
-- @File Name: TeamWarfareBaseNode.lua
-- @Description: 


local This , super = class(..., "NodeView")
local HeroManager = require("Hero.HeroManager")
local WarfareManager = require("Warfare.WarfareManager")

function This:initData(war, dialog)
    self.m_dialog = dialog
    self.m_csNode = cc.Node:createFromCSFile(self.csFileName)
    self:addChild(self.m_csNode)
    self.m_inited = false
    self:setWar(war)
end

function This:setWar( war )
    if self.m_war ~= war then
        self.m_war = war
        self.m_warId = self.m_war:getId()

        local settings = self.m_dialog.m_settings
        local setting = settings[self.m_warId]
        if not setting then
            self.m_setting = {}
            self:setDefaultSetting(self.m_setting)
            settings[self.m_warId] = self.m_setting
        else
            if WarfareManager:hasTeam() then
                local team = WarfareManager:getMyTeam()
                setting.hard = team:getHard()
                setting.auto_quick = team:getAutoJoin()
            end
            self.m_setting = setting
        end

        self.m_heroList = nil
    
        self.m_isAuto = self.m_setting.auto_quick
        self:setHardLevel(self.m_setting.hard)
        if self.m_inited then
            self:refresh()
        end
    else
        local setting = self.m_setting
        if not setting then
            local settings = self.m_dialog.m_settings
            self.m_setting = {}
            self:setDefaultSetting(self.m_setting)
            settings[self.m_warId] = self.m_setting
        else
            if WarfareManager:hasTeam() then
                local team = WarfareManager:getMyTeam()
                setting.hard = team:getHard()
                setting.auto_quick = team:getAutoJoin()
            end
        end
        self.m_isAuto = self.m_setting.auto_quick
        self:setHardLevel(self.m_setting.hard)
    end
end

function This:setDefaultSetting(setting)
    if WarfareManager:hasTeam() then
        local team = WarfareManager:getMyTeam()
        setting.hard = team:getHard()
        setting.auto_quick = team:getAutoJoin()
    else
        setting.hard = 1
        setting.auto_quick = true
    end
end

function This:clearSetting(cleanup)
    if cleanup then
        self.m_heroList = nil
    end
    self:setDefaultSetting(self.m_setting)
end

function This:onEnter()
    super.onEnter(self)
    self.m_war:updateState()
    self.m_war:addObserver(self)
    self:addCustomListener()
end

function This:onExit()
    self:removeCustomListener()
    self.m_war:removeObserver(self)
    super.onExit(self)
end

function This:resetData()
    local war
    if WarfareManager:hasTeam() then
        local team = WarfareManager:getMyTeam() 
        local warId = team:getWarId()
        war = self.m_dialog.m_build:getWarfare(warId)
        if war then
            self.m_dialog:setWar(war)
        end
    end
    if not war then
        war = self.m_dialog:getTabNodeData()
    end

    self:setWar(war)
end

function This:getHeroLimit()
    if not self.m_setting.heroLimit then
        local heroIdList = self.m_war:getAvailableHeroList()
        if heroIdList then
            local heroList = {}
            for i, id in ipairs(heroIdList) do
                local hero = HeroManager:getHero(id)
                table.insert(heroList, hero)
            end
            self.m_setting.heroLimit = heroList
        end
    end
    return self.m_setting.heroLimit
end

function This:requestTabNodeData()
end

function This:refresh()
end

function This:setHardLevel( hard, name )
    self.m_hardLevel = hard
end

function This:addCustomListener()
end

function This:removeCustomListener()
end
