-- @Author: superm
-- @Date:   2018-08-16 14:59:55
-- @Last Modified by:   superm
-- @Last Modified time: 2018-09-06 14:37:04
-- @File Name: TeamWarfareEasyNode.lua
-- @Description: 
local This , super = class(..., "Warfare.TeamWarfareBaseNode")
local ListLayer = require("UI.ListLayer")
local RankPageListLayer = require("Rank.RankPageListLayer")
local TeamListItemView = require("Warfare.TeamListItemView")
local ViewUtil = require("UI.ViewUtil")
local WarfareUtil = require("Warfare.WarfareUtil")
local WarfareManager = require("Warfare.WarfareManager")
local GamePlayer = require("Player.GamePlayer")
This.csFileName = "MenuScene/NodeUI/TeamWarOrganize.csb"

function This:initNode()
	self.m_markTimeList = {}
	local csNode = self.m_csNode
    csNode.tips:setVisible(false)
    ViewUtil:blinkView(csNode.tips, true)
    self:initListLayer()
    self:initTeamInfoList()
    self:refresh()
end

function This:initListLayer()
	local selectNode = self.m_csNode.select
	self.m_listLayer = ListLayer:create({
		itemWidth =  100,
		itemHeight =  30,
		frameCount = kTeamWarMaxHard,
		itemCount = kTeamWarMaxHard,
		spaceX = 0,
		spaceY = 0,
        scrollView = selectNode.scrollView,
	})
	local w , h = self.m_listLayer:getContentSize()
	self.m_listLayer:setAnchorPoint(0.5, 0)
	selectNode.scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
	self.m_listLayer:starList(function (index)
		local name = WarfareUtil.getHardDesc(index)
		local bg = cc.Scale9Sprite:create("GameScene/xialakuangbiaoti.png")
		bg:setContentSize(130, 31)
		local text = cc.Label:createWithTTF("Fonts/Normal.ttf", name, 15)
		text:setPosition(65 , 16)
		bg:addChild(text)
		bg:setTouch({
			onEnded = function ()
				self:setHardLevel(index, name, true)
				self:refreshTeamList()
			end,
			isContainer = true,
			isScale = false,
		})
		return bg
	end)
end

function This:initTeamInfoList()
	self.m_teamListView = RankPageListLayer:create({
        itemWidth = 612,
        itemHeight = 42,
        spaceX = 0,
        spaceY = 1,
        scrollView = self.m_csNode.ScrollView,
		dataCounter = function ()
	        return #self.m_teamListInfo
        end,
        viewCreator =  function(index, node)
			local curList = self.m_teamListInfo
        	if not curList then return end
        	local info = curList[index]
	        if info then
		        if not node then
			        return TeamListItemView:create(info, index)
			    else
		        	node:setModel(info, index)
		        end
		    end
	    end,
    })
    self.m_teamListView:setAnchorPoint(0.5,0)
    local w , h = self.m_teamListView:getContentSize()
    self.m_csNode.ScrollView:pushBackCustomItem(self.m_teamListView, w, h, 0)
    self.m_teamListView:setCascadeOpacityEnabled(true)
end

function This:initTouchEvent()
	if not self.m_inited  then
		local csNode = self.m_csNode
		csNode.listbtnxiala:setTouch({
			onEnded = function()
				local visible = csNode.select:isVisible()
				csNode.select:setVisible(not visible)
				csNode.listbtn:setAngle(visible and 180 or 0)
			end,
			isScale = false,
		})

		csNode.shuaxin:setTouch({
			onEnded = function()
				local markTime = self.m_markTimeList[self.m_warId]
				local time = markTime or 0
			    local curTime = GamePlayer:getServerTime()
				if (curTime - time) > 5 then
					self.m_markTimeList[self.m_warId] = curTime
			    	self:requestTabNodeData()
			    else
			    	g_scene:showTips(Serialize[64851])
			    end
			end,
			isScale = false,
		})

	    csNode.info:setTouch({
	        onEnded = function()
	        	self.m_dialog:openTabNode("Warfare.TeamWarfareInfoNode")
	        end,
	        isScale = false
	    })

	    csNode.new:setTouch({
	        onEnded = function()
	        	if not self.m_war:isVipOpen() then
					return g_scene:showTips(Serialize[64847])
				end
                if not WarfareManager:hasTeam() then
                    local maxHard = self.m_war:getNextHard()
                    if maxHard < self.m_hardLevel then
                        return g_scene:showTips(Serialize[64852] .. WarfareUtil.getHardDesc(maxHard))
                    end
                    self.m_setting.hard = self.m_hardLevel
                end
	        	self.m_dialog:openTabNode("Warfare.TeamWarfareFoundNode")
	        end,
	        isScale = false
	    })

	    csNode.quick:setTouch({
	        onEnded = function()
                if WarfareManager:hasTeam() then
                    return g_scene:showTips(Serialize[64844])
                end
    			WarfareManager:onJoinTeamQuicklyRequest(self.m_warId)
	        end,
	        isScale = false
	    })
	    self.m_inited = true
	end
end

function This:requestTabNodeData()
	WarfareManager:onGetTeamListInfoRequest(self.m_warId)
end

function This:setHardLevel( hard, name, skip )
	if self.m_hardLevel == hard then
		return
	end
	self.m_hardLevel = hard
	if not skip then
		self.m_setting.hard = hard
	end
	local csNode = self.m_csNode
	csNode.select:setVisible(false)
	csNode.listbtn:setAngle(0)
	name = name or WarfareUtil.getHardDesc(hard)
	csNode.name:setString( name )
end

function This:onGetTeamListInfo( list )
	self.m_allTeamListInfo = list
	self:refreshTeamList()
end

function This:refreshTeamList()
	if self.m_allTeamListInfo then 
		self.m_teamListInfo = {}
		for _, team in ipairs(self.m_allTeamListInfo) do
			if self.m_hardLevel == team:getHard() then
				table.insert( self.m_teamListInfo, team )
			end
		end
		self.m_teamListView:setDataList()
	end
end

function This:onJoinMatchTeam()
	self.m_dialog:openTabNode("Warfare.TeamWarfareFoundNode")
end