-- @Author: superm
-- @Date:   2018-08-16 15:00:07
-- @Last Modified by:   superm
-- @Last Modified time: 2018-12-26 17:52:36
-- @File Name: TeamWarfareFoundNode.lua
-- @Description: 

local This , super = class(..., "Warfare.TeamWarfareBaseNode")
local TeamWarRoleView = require("Warfare.TeamWarRoleView")
local WarfareInviteView =  require("Warfare.WarfareInviteView")
local TerrTabGroupBehaviour =  require("Territory.TerrTabGroupBehaviour")

local ViewUtil = require("UI.ViewUtil")
local ItemUtil = require("Bag.ItemUtil")
local WarfareUtil = require("Warfare.WarfareUtil")
local DepotManager = require("Depot.DepotManager")
local FamilyManager =  require("Family.FamilyManager")
local FriendManager =  require("Friend.FriendManager")
local WarfareManager = require("Warfare.WarfareManager")

This.csFileName = "MenuScene/NodeUI/TeamWarFound.csb"
This.hasTouchMask = true

local NodeAniMode = {
	kOpen = 1,
	kClose = 2,
}

function This:initNode()
	local csNode = self.m_csNode
    csNode.tips:setVisible(false)
    ViewUtil:blinkView(csNode.tips, true)

    self.m_viewList = {}
    self.m_maxRoleCount = 3

    local team = WarfareManager:getMyTeam(TeamType.kWarTeam)
	self.m_team = team
	if team then
		for i = 1 , self.m_maxRoleCount do
			local view = TeamWarRoleView:create(nil, team)
			view:setInviteHandler(function ()
				if self.m_inviteMode == NodeAniMode.kOpen then
					return
				end

				self:runAnimation(csNode.invite.node, NodeAniMode.kOpen)
			end)
			csNode["hero" .. i]:addChild(view)
			self.m_viewList[i] = view
		end

	end
end

function This:addCustomListener()
	FamilyManager:addObserver(self)
	FriendManager:addObserver(self)
	WarfareManager:addObserver(self)
end

function This:removeCustomListener()
	FamilyManager:removeObserver(self)
	FriendManager:removeObserver(self)
	WarfareManager:removeObserver(self)
end

function This:initTouchEvent()
	if not self.m_inited  then
		local csNode = self.m_csNode
		csNode.weigouxuan:setTouch({
			onEnded = function()
				local team = WarfareManager:getMyTeam()
				local myRole = team:getMainRole()
				if not myRole or not myRole:isRoomOwner() then
					g_scene:handleErrorCode(12504)
					return
				end
				self.m_isAuto = not self.m_isAuto
				self.m_csNode.gouxuan:setVisible(self.m_isAuto)
				self.m_setting.auto_quick = self.m_isAuto
				WarfareManager:onSetAutoQuicklyJoinRequest(self.m_isAuto)
			end,
			isScale = false,
		})

	    csNode.info:setTouch({
	        onEnded = function()
	        	self.m_dialog:openTabNode("Warfare.TeamWarfareInfoNode")
	        end,
	        isScale = false
	    })

	    csNode.embattle:setTouch({
	        onEnded = function()
	        	local allHeroList
            	local heroLimitList = self:getHeroLimit()
	        	if heroLimitList then
	        		allHeroList = {}
		        	for _, hero in ipairs(heroLimitList) do
		        		if hero:isGot() and not hero:isDeputy() then
		        			table.insert(allHeroList, hero)
		        		end
		        	end
		        end

	            self.m_dialog:openDialog("Territory.TerrBattleHeroPanel", {
	                targetType = TerrBattleTarget.kWarfareNpc,
	                heroSource = WarfareManager:getEmBattleHeros(TeamType.kWarTeam),
	                attacker = self.m_war, 
	                heroList = allHeroList,
	                battleHandler = function ( heroList )
	                    WarfareManager:onWarEmbattleHerosRequest(heroList)
	                    return true
	                end
	            })
	        end,
	        isScale = false
	    })

	    csNode.battle:setTouch({
	        onEnded = function()
				-- 是否已布阵
				if not WarfareManager:hadEmbattled() then
					g_scene:showTips(Serialize[64195])
					csNode.embattle.m_touch:onEnded()
					return
				end
				
        	    if not WarfareManager:hasTeam() then
        	    	return
				end

			    local team = WarfareManager:getMyTeam()
				local mainRole = team:getMainRole()
				if not mainRole:isRoomOwner() then
					local value = mainRole:isReady()
					WarfareManager:onPrepareWarRequest(not value)
				else
					-- 队长
				    local ret, errCode = team:canChallenge()
				    if not ret then
				    	g_scene:showTips(errCode)
				    	return
				    end

				    self:challenge(team)
				end
	        end,
	        isScale = false
	    })

	    csNode.exitbtn:setTouch({
	        onEnded = function()
	        	self:clearSetting()
	        	WarfareManager:onExitWarTeamRequest()
	        end,
	        isScale = false
	    })
	    self.m_inited = true
	end
end


function This:challenge(team)
	local itemId, itemNum = self.m_war:getTicketId(), self.m_war:getTicketCost()
	local cstate, errCode
	if itemId and itemId > 0 then
		cstate, errCode = DepotManager:isItemOrResEnough(itemId, itemNum)
	else
		cstate = true	
	end
	
	if not cstate then
		if errCode then
			g_scene:handleErrorCode(errorStr)
			return
		else
			local name = ItemUtil.getName(itemId)
			g_scene:showTips(Serialize[60714] .. name .. Serialize[60715])
			self.m_coroutine = coroutine.create(function ()
				local diamond = self.m_war:convertChallengeCost(itemNum)
				g_scene:openDialog("UI.CostConfirmPanel", {
	                cost = diamond,
	                costId = 102,
	                desc = Serialize[60495] .. name .. "X" .. itemNum, 
	                okHandler = function()
	                    coroutine.resume(self.m_coroutine, true)
	                end,
			        cancelHandler = function()
			            coroutine.resume(self.m_coroutine, false)
			        end,
                })

				local ret = coroutine.yield()
				if not ret then
					return
				else
					ret, errCode = DepotManager:isItemOrResEnough(102, diamond)
					if not ret then
						if errCode then
							g_scene:handleErrorCode(errorStr)
						end
						return
					end
				end
				WarfareManager:onWarfareBattleRequest(team:getWarId(), team:getHard(), ret)
			end)
			coroutine.resume(self.m_coroutine)
		end
	else
		WarfareManager:onWarfareBattleRequest(team:getWarId(), team:getHard())
	end
end

function This:initInviteNode()
	if not self.m_groupInvite then
	    local inviteNode = self.m_csNode.invite.node
	    local group = TerrTabGroupBehaviour:create()
	    group:addTabButton("friend", inviteNode.friend, function()
			WarfareManager:onGetInviteFriendListRequest()
	    end, false, true)

	    group:addTabButton("family", inviteNode.family, function()
	    	if not FamilyManager:hasFamily() then
	    		g_scene:showTips(Serialize[61508])
	    		return false
	    	end

	    	self:refreshInviteList(true)
	    end, false, true)
	    
	    self.m_groupInvite = group
	end
    self.m_groupInvite:selectTabButton("friend")
    WarfareManager:onGetInviteFriendListRequest()
end

function This:requestTabNodeData()
    if not WarfareManager:hasTeam() then
		WarfareManager:onCreateTeamRequest(self.m_warId, self.m_hardLevel)
	end
end

function This:refresh()
    super.refresh(self)

	local csNode = self.m_csNode
	csNode.gouxuan:setVisible( self.m_isAuto )

	local warName = string.gsub(self.m_war:getName(), "\n", "")
	csNode.title:setString(string.format(Serialize[64854], warName, WarfareUtil.getHardDesc(self.m_hardLevel)))
	csNode.invite.node:setPositionX(256)
	self.m_inviteMode = NodeAniMode.kClose

	local roleList = {}
	local isCaptain = false
    local visible = false
    if WarfareManager:hasTeam() then
	    local team = WarfareManager:getMyTeam()
	    roleList = team:getRoleList()
	    local str = Serialize[64202]
		local role = team:getMainRole()
		isCaptain = role:isRoomOwner()
		local isReady = role:isReady()
		if not isCaptain then
			if not isReady then
				str = Serialize[64203]
			else
				str = Serialize[64204]
			end
			visible = not isReady
		else
		    local team = WarfareManager:getMyTeam()
			visible = team:canChallenge()
		end

		csNode.battle.tiaozhan:setString(str)
	else
		isCaptain = false
		visible = false
	end

	if not self.m_readyEff and visible then
		local function createAnim( parent )
	        local GameUtil = require("Util.GameUtil")
	        local effect1 = GameUtil:createEffect("Ready", true, parent)
	        parent:addChild(effect1)
	        return effect1
	    end
		self.m_readyEff = createAnim(csNode.battle)
	elseif self.m_readyEff then
		self.m_readyEff:setVisible(visible)
	end

	for i = 1 , self.m_maxRoleCount do
		local view = self.m_viewList[i] 
		if view then
			view:setModel(roleList[i])
		end
	end
	
	local itemId, itemNum
	if isCaptain then
		itemId, itemNum = self.m_war:getTicketId(), self.m_war:getTicketCost()
	end
	
	if itemId and itemNum and itemId > 0 and itemNum > 0 then
		csNode.itemicon:setImage(ItemUtil.getIconImage(itemId))
		csNode.num:setString(itemNum)
		csNode.itemicon:setVisible(true)
		csNode.num:setVisible(true)
	else
		csNode.itemicon:setVisible(false)
		csNode.num:setVisible(false)
	end
end

function This:refreshInviteList(isGroup)
	local timer = self.m_dialog.m_timer
	local csNode = self.m_csNode.invite.node
	local scrollView = csNode.scrollView

	scrollView:removeAllItems()
	timer:removeAllTimer()
	self.m_inviteViews = {}

	local list = WarfareManager:getCanInviteList(isGroup)
	for i , invite in ipairs(list) do
		timer:setTimer(0.1 * (i - 1),function()
			local view = WarfareInviteView:create(invite)
			local w , h = view:getContentSize()
			scrollView:pushBackCustomItem(view,w,h-3,0.5)
			self.m_inviteViews[invite:getId()] = view
		end,"invite"..i)
	end
end

function This:runAnimation(node, mode)
	if not node or self.m_isPlayingAni then
		return
	end

	self.m_isPlayingAni = true
	self.m_inviteMode = mode
	
	local x, name = - 256, "ExponentialIn"
	if mode == NodeAniMode.kClose then
		x, name = 256, "ExponentialIn"
	end
    node:runAction(cc.Sequence(cc.MoveBy(0.2, x, 0, name), cc.CallFunc(function()
		self.m_isPlayingAni = false
		if self.m_inviteMode == NodeAniMode.kOpen then
			self:initInviteNode()
		end
	end)))
end

function This:onTouchMask()
	if self.m_groupInvite and self.m_inviteMode == NodeAniMode.kOpen then
		self:runAnimation(self.m_csNode.invite.node, NodeAniMode.kClose)
	end
end

function This:onGetFamilyInfoChange( hasFamily )
    if not self.m_groupInvite or self.m_inviteMode == NodeAniMode.kClose then
    	return
    end

	if self.m_groupInvite:getCurSelectTab() == "family" and not hasFamily then
		self.m_groupInvite:selectTabButton("friend")
	end
end

function This:onFriendStateChanged(friend)
    if not self.m_groupInvite or self.m_inviteMode == NodeAniMode.kClose then
    	return
    end

	if self.m_groupInvite:getCurSelectTab() == "friend" then
		
	end
end

function This:onApplyTeamWar()
    self:refresh()
end

function This:onExitWarTeam( hasTeam )
    if hasTeam then
		self:refresh()
	else
    	self.m_dialog:openTabNode("Warfare.TeamWarfareEasyNode")
	end
end

function This:onGetInviteFriendList()
	self:refreshInviteList()
end

function This:onGetWarfareTabData()
	if not self.m_team then
		self:initNode()
		self:refresh()
	end
end