-- @Author: superm
-- @Date:   2018-08-16 14:37:37
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-06-30 17:10:49
-- @File Name: TeamWarfareInfoNode.lua
-- @Description: 

local This , super = class(..., "Warfare.TeamWarfareBaseNode")
local TerrTabGroupBehaviour =  require("Territory.TerrTabGroupBehaviour")
local ItemNode =  require("UI.ItemNode")
local HeroConfig =  require("Metadatas.HeroConfig")
local GamePlayer = require("Player.GamePlayer")
local WarfareUtil = require("Warfare.WarfareUtil")
local WarfareManager = require("Warfare.WarfareManager")

This.csFileName = "MenuScene/NodeUI/TeamWarChoice.csb"

function This:initNode()
    self.m_groupHard = TerrTabGroupBehaviour:create()
    local hardNode = self.m_csNode.nandu
    for i=1, kTeamWarMaxHard do
        local view = hardNode["nandu" .. i]
        self.m_groupHard:addTabButton(tostring(i), view, function()
            local isCaptain = false
            if WarfareManager:hasTeam() then
                local team = WarfareManager:getMyTeam()
                isCaptain = team:isCaptain()
            end
            if isCaptain then                
                local maxHard = self.m_war:getNextHard()
                if maxHard < i then
                    g_scene:showTips(Serialize[64852] .. WarfareUtil.getHardDesc(maxHard))
                    return false
                end
                if self.m_hardLevel == i then
                    return false
                end
                self.m_hardLevel = i
                self.m_setting.hard = i
                WarfareManager:onSwitchWarfareRequest(self.m_warId, i)
            else
                self.m_hardLevel = i
            end

            local reward = self.m_war:getReawrdWithoutTeam(self.m_hardLevel, GamePlayer:getLevel())
            self:refreshRewardInfo(self.m_csNode.reward, reward)
        end)
    end

    for i= kTeamWarMaxHard + 1, kTeamWarUIMaxHard do
        local view = hardNode["nandu" .. i]
        if view then
            view:setVisible(false)
        end
    end
    local maxHard = self.m_war:getNextHard()
    self.m_setting.hard = maxHard
    self:setHardLevel(maxHard)
end

function This:initTouchEvent()
	if not self.m_inited  then
        local csNode = self.m_csNode
	    csNode.easy:setTouch({
	        onEnded = function()
                if WarfareManager:hasTeam() then
                    g_scene:showTips(Serialize[64844])
                    return
                end
                self.m_dialog:openTabNode("Warfare.TeamWarfareEasyNode")
	        end,
	        isScale = false
	    })

	    csNode.new:setTouch({
	        onEnded = function()
                if not self.m_war:isVipOpen() then
                    g_scene:showTips(Serialize[64847])
                    return false
                end
                if not WarfareManager:hasTeam() then
                    local maxHard = self.m_war:getNextHard()
                    if maxHard < self.m_hardLevel then
                        g_scene:showTips(Serialize[64852] .. WarfareUtil.getHardDesc(maxHard))
                        return false
                    end
                    self.m_setting.hard = self.m_hardLevel
                end
	        	self.m_dialog:openTabNode("Warfare.TeamWarfareFoundNode")
	        end,
	        isScale = false
	    })

        csNode.wenhao:setTouchEnded(function()
            self.m_dialog:openDialog("UI.HelpTips", {
                text = Serialize["Warfare_WarfareInfo_1"],
                node = self.m_csNode.wenhao
            })
        end)
        
	    csNode.box:setTouchEnded(function()
            self.m_dialog:openDialog("Warfare.WarfareBoxRewardPanel", {war = self.m_war})
        end)
	    self.m_inited = true
	end
end

function This:refresh()
    super.refresh(self)
    local nameList = {}
    if not self.m_heroList then
        self.m_heroList = self:getHeroLimit()
    end
    local csNode = self.m_csNode
    if self.m_heroList then
        self:setHeroNameList(nameList, self.m_heroList)
        csNode.heroname:setStringWithColor(table.concat(nameList, "、"), 500)
    else
        csNode.heroname:setString(Serialize[64857])
    end

    local rvalue, rmax = self.m_war:getLeftRewardTimes()
    local cvalue, cmax = self.m_war:getLeftRewardTimes(true)
    local rewardNode = csNode.reward
    rewardNode.reward:setString(string.format("%d/%d", rvalue, rmax))
    rewardNode.captainreward:setString(string.format("%d/%d", cvalue, cmax))
    csNode.new.chuangdui:setString(not WarfareManager:hasTeam() and Serialize[64858] or Serialize[64859])

    local needVip = self.m_war:doNeedVip()
    csNode.box:setVisible(not needVip)
    csNode.numberjifen:setVisible(not needVip)
    if not needVip then
        csNode.numberjifen:setString(Serialize[64860] .. self.m_war:getScore())
    end

    self.m_hardLevel = self.m_setting.hard or 1
    self.m_effectList = self.m_effectList or {}
    WarfareUtil.createBoxEffect(csNode.box, self.m_war:hasBoxReward(), self.m_effectList)
    
    local reward = self.m_war:getReawrdWithoutTeam(self.m_hardLevel, GamePlayer:getLevel())
    self:refreshRewardInfo(csNode.reward, reward)
end

function This:refreshRewardInfo(csNode, info)
    if not csNode then return end
    csNode.sv:removeAllItems()
    csNode.capsv:removeAllItems()
    if info then
        self.m_rewards = info.rewardlist or {}
        self.m_extRewards = info.extra_rewardlist or info.extra_rewardlist1 or {}
    else
        self.m_rewards = {}
        self.m_extRewards = {}
        return
    end
    local function createItemView(model)
        local node = ItemNode:create(model[1],model[2])
        node:setAnchorPoint(0.5,0.5)
        return node
    end
    local function refreshsv( sv, reward )
        sv:setDirection("horizontal")
        sv.m_direction = "right"
        csNode:setCascadeOpacityEnabled(true)
        if reward then
            for i, model in ipairs(reward) do
                local view = createItemView(model)
                sv:pushBackCustomItem(view, 95, 90, 0.5) 
            end
        end
    end
    refreshsv(csNode.sv, self.m_rewards)
    refreshsv(csNode.capsv, self.m_extRewards)
end

function This:setHeroNameList(nameList, heroList)
    local isGot
    for i, hero in ipairs(heroList) do
        local name = hero:getName()
        local ret = hero:isGot()
        if isGot ~= ret then
            table.insert(nameList, (ret and "<A2>" or "<A5>").. name)
            isGot = ret
        else
            table.insert(nameList, name)
        end
    end
end

function This:setHardLevel( hard, name )
    if self.m_groupHard then
        local hard = self.m_setting.hard or 1
        if WarfareManager:hasTeam() then
            local team = WarfareManager:getMyTeam()
            if hard == team:getHard() then
                self.m_hardLevel = hard
                return
            end
        end
        self.m_groupHard:selectTabButton(tostring(hard))
    end
end