-- @Author: superm
-- @Date:   2018-07-20 14:39:57
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-01-14 21:18:12
-- @File Name: TeamWarfarePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local TerrBuildManager = require("Territory.TerrBuildManager")
local WarfareManager = require("Warfare.WarfareManager")
local FriendManager = require("Friend.FriendManager")

local TeamBrowerTab = {
    kInfo = "info",
    kEasy = "easy",
    kNew = "new",
}

function This:initNode(info)
    FriendManager:onInitFriendInfoRequest()
    self.m_settings = {}
    self.m_war = info.war
    self.m_build = info.build or TerrBuildManager:getBuildByType(TerrBuildType.kJiaoChang)

    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TeamWar.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)

    local bgNode = self.m_csNode.typeBg
    local bgPosx, bgPosy = bgNode:getPosition()
    local bgw, bgh = bgNode:getContentSize()
    self.m_bgNodeInfo = {
        width = bgw,
        high = bgh,
        posx = bgPosx,
        posy = bgPosy,
    }
    
    self.m_btnListView = self.m_csNode.buttonList
    if self.m_btnListView then
        self.m_btnListView:setSwallowTouch(false)
    end
    self.m_buttonList = {}
end

function This:refresh()
    local copyList = self.m_build:getAllWarfareList(self.m_war:getType())
    self:addTabButtonList(copyList, self.m_war)
end

function This:initTouchEvent()
    self.m_csNode.close:setTouchEnded(function()
        self:close()
    end)
end

function This:onEnter()
    super.onEnter(self)
    if self.m_build then
        self.m_build:addObserver(self)
    end
    WarfareManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    WarfareManager:removeObserver(self)
    if self.m_build then
        self.m_build:removeObserver(self)
    end
    super.onExit(self)
end

function This:addTabButton(name, button, onSelected)
    if not self.m_tabButtons then
        self.m_tabButtons = {}
    end
    if self.m_tabButtons[name] then
        return
    end
    self.m_tabButtons[name] = button
    if not button.m_touch then
        button:setTouch({
            isContainer = true,
            isDynamic = true,
            isScale = false,
            onEnded = function ()
                self:selectTabButton(name)
            end
        })
    end
    button.onSelected = onSelected
end

function This:addTabButtonList(orderList, defaultCopy)
    local function createSp(file, pnode, posx, posy, name)
        local sp = cc.Sprite:create(file) 
        sp:setAnchorPoint(0.5, 0.5)
        sp:setPosition(posx, posy)
        if name then
            sp:setName(name)
            pnode[name] = sp
        end
        pnode:addChild(sp)
        return sp
    end

    local function createLabel(text, pnode, posx, posy, name)
        local label = cc.Label:createWithBMFont("Fonts/title.fnt", text, "center", 200)
        label:setAnchorPoint(0.5, 0.5)
        label:setPosition(posx, posy)
        label:setScale(0.75)
        if name then
            label:setName(name)
            pnode[name] = label
        end
        pnode:addChild(label)
        return label
    end

    local function addActTabBtn( index, copy )
        local copyName = copy:getName()
        -- local state = copy:isOpening()
        local btnName = "war" .. tostring(copy:getId())
        local rootNode = self.m_buttonList[btnName]
        local w, h = 130, 100
        if not rootNode then
            local ax, ay = 0.5, 0.5
            local aw, ah = w * ax, h * ay
            rootNode = cc.Node:create()
            rootNode:setAnchorPoint(ax, ay)
            rootNode:setContentSize(w, h)
            rootNode:retain()
            self.m_buttonList[btnName] = rootNode

            createSp("Common/zuofenyeweidianji1.png", rootNode, aw, ah, "normal")
            createSp("Common/zuofenyetuan1.png", rootNode, aw, ah)
            local lsprite = createSp("Common/zuofenyedianji1.png", rootNode, aw, ah, "light")
            createSp("Common/bgjianbian28.png", rootNode, aw, ah)
            createSp("Common/jiantou15.png", lsprite, w - 8, ah)
            createSp("Common/jiantou15.png", lsprite, 8, ah):setFlippedX(true)
            createLabel(copyName, rootNode, aw, ah, "name")
        else
            rootNode.name:setString(copyName)
        end

        -- if not state then
        --     rootNode.normal:setGrayShader()
        --     rootNode.name:setGrayShader()
        --     rootNode.bgk:setGrayShader()
        --     rootNode.bgz:setGrayShader()
        -- else
        --     rootNode.normal:clearShader()
        --     rootNode.name:clearShader()
        --     rootNode.bgk:clearShader()
        --     rootNode.bgz:clearShader()
        -- end

        if self.m_btnListView then
            self.m_btnListView:pushNode(rootNode, w, h)
        end
        rootNode:setName(btnName)
        rootNode:setVisible(true)
        return rootNode
    end

    if self.m_tabButtons then
        for _, view in pairs(self.m_tabButtons) do
            view:setVisible(false)
        end
    end

    local defaultCopyId
    if defaultCopy then
        defaultCopyId = defaultCopy:getId()
    end

    if not defaultCopyId then
        local _, defaultCopyInfo = next(orderList)
        local defaultCopyId = defaultCopyInfo:getId()
    end

    for index, war in ipairs(orderList) do
        local copyId = war:getId()
        local tabName = tostring(copyId)
        local btn = addActTabBtn( index, war )
        if btn then
            self:addTabButton(tostring(copyId), btn, function()
                local ret, errorStr = war:isOpening()
                if not ret then
                    g_scene:showTips(errorStr)
                    return false
                end 

                if war:getLeftTime() <= 0 then
                    --不在开放时间
                    g_scene:handleErrorCode(12507)

                    return false
                end

                self.m_curCopyId = copyId
                self.m_war = war
                self:selectTabById(copyId, true)
                return true
            end)
        end
    end
    self:selectTabButton(tostring(defaultCopyId))
end

function This:selectTabById( copyId, forceRequest )
    if WarfareManager:hasTeam() then
        local team = WarfareManager:getMyTeam() 
        if team then
            local warId = team:getWarId()
            if warId > 0 and warId ~= copyId then
                WarfareManager:onSwitchWarfareRequest(copyId)
                return
            else
                self:openTabNode("Warfare.TeamWarfareFoundNode")
            end
        end
    else
        self:openTabNode("Warfare.TeamWarfareInfoNode")
    end
end

function This:setWar(war)
    if self.m_war ~= war then
        if war and war:isOpening() and war:getLeftTime() > 0 then
            self.m_war = war
            if self.m_tabButtons then
                local name = tostring(war:getId())
                if self.m_selectTabButton == name then
                    return
                end
                self.m_selectTabButton = name
                self:lightTabButton(name)
            end
        end
    end
end

function This:getTabNodeData( name )
    return self.m_war, self
end

function This:onTabNodeSwitchIn( node, name )
    if node then
        node:resetData()
        node:requestTabNodeData()
        node:refresh()
    end
end

function This:onTouchMask()
    if self.m_tabNodes then
        for _, node in pairs(self.m_tabNodes) do
            if node.hasTouchMask then
                node:onTouchMask()
            end
        end
    end
end

function This:onGetWarfareTabData(funcName, ...)
    if self.m_tabNodes then
        local curNode = self.m_tabNodes[name]
        for name, observer in pairs(self.m_tabNodes) do
            if observer[funcName] then
                observer[funcName](observer, ...)
            end
        end
    end
end

function This:onSwitchWarfare()
    self:openSubTabByName("Warfare.TeamWarfareFoundNode")
end

function This:onGetMyTeamInfo(teamType)
    if teamType == TeamType.kWarTeam then
        self:openSubTabByName("Warfare.TeamWarfareFoundNode", true)
    end
end

function This:openSubTabByName( name, forFresh )
    if self.m_tabNodes then
        if self.m_selectTabName and self.m_selectTabName == name then
            local curNode = self.m_tabNodes[name]
            if curNode then
                curNode:resetData()
                curNode:refresh()
            end
        elseif not forFresh then
            self:openTabNode(name)
        end
    elseif not forFresh then
        self:openTabNode(name)
    end
end
