-- @Author: superm
-- @Date:   2018-12-24 17:14:34
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-11-30 14:55:10
-- @File Name: TowerTeam.lua
-- @Description: 

local This , super = class(...,"Warfare.MatchTeam")
local GamePlayer = require("Player.GamePlayer")
local FamilyManager = require("Family.FamilyManager")
local TeamRole = require("Warfare.TeamRole")
local GameUtil = require("Util.GameUtil")

function This:createOfflineTeam(info)
	local o = {}
	setmetatable(o,self)
	o:initOffline(info)
	return o
end

function This:initOffline(info)
    self.m_roleRemoved = {}
	self.m_roleList = {}
	self.m_matching = false
	self.m_teamType = info.teamType
	local teamId = info.team_id or 0
	self.m_id = tonumber(teamId)
	self.m_ownerId = tonumber(info.captain_id)
	local role = self:addRoleByMySelf(info)
end

function This:initInfo(info)
	self.m_roleRemoved = {}
	self.m_roleList = {}
	self.m_matching = false

	local teamId = info.team_id or 0
	self.m_id = tonumber(teamId)
	self.m_type = info.team_type
	self.m_ownerId = tonumber(info.captain_id)
	if info.member_list then
		for i , roleInfo in ipairs(info.member_list) do
			self:addRole(roleInfo)
		end
	end
	
	self:setOwnerId(self.m_ownerId)
	self:updateTeamAttrInfo(info)
	self:updateWarfareInfo(info)
end

function This:updateInfo(info)
	self.m_warConfig = nil
	self.m_id = info.team_id 
	self.m_type = info.team_type
	self.m_ownerId = info.captain_id

	local updateList = {}
	for i , roleInfo in ipairs(info.member_list) do
		updateList[roleInfo.role_id] = roleInfo
	end
	for i = #self.m_roleList, 1 , -1 do
		local role = self.m_roleList[i]
		if not updateList[role:getId()] then
			table.remove(self.m_roleList,i)
			self.m_roleRemoved[role:getId()] = role
		end
	end
	for i , roleInfo in ipairs(info.member_list) do
		local role = self:getRole(roleInfo.role_id)
		if role then
			role:updateInfo(roleInfo)
		else
			self:addRole(roleInfo)
		end
	end

	if info.captain_id then
		self:setOwnerId(self.m_ownerId)
	end
	self:updateTeamAttrInfo(info)
	self:updateWarfareInfo(info)
end

function This:addRole(roleInfo)
	local role = TeamRole:create(roleInfo)
	table.insert(self.m_roleList,role)
	role:setIsRoomOwner(roleInfo.role_id == self.m_ownerId)
end

function This:addRoleByMySelf(info)
	local roleInfo = {
		role_id = GamePlayer:getPlayerID(),	
		role_name = GamePlayer:getRoleName(),
		role_icon = GamePlayer:getIcon(), 
		role_level = GamePlayer:getLevel(), 
		group_id = 0,
		group_name = "",
		group_title = 0,
		fight_power = 0, 
		vip_level = 0,	
		is_ready = info.is_ready,				
	}

	local familyModel = FamilyManager:getMyFamily()
	if familyModel then
		local memberInfo = familyModel:getSelfMemberInfo()
		if memberInfo then
			roleInfo.group_id = memberInfo:getId()
			roleInfo.group_name = memberInfo:getName()
			roleInfo.group_title = memberInfo:getTitle()
		end
	end
	local role = TeamRole:create(roleInfo)
	table.insert(self.m_roleList,role)
	return role
end

function This:updateWarfareInfo(info)
	self.m_warfareId = info.def_team_id
end

function This:updateTeamAttrInfo( info )
	self.m_autoFight = info.is_auto_fight or true
end

function This:getPeople()
	local curNum, minNum = #self.m_roleList, 2
	return curNum,minNum
end

function This:canChallenge()
    local curNum, minNum = #self.m_roleList, 2
    if curNum < minNum then
    	return false, Serialize[64871]
    end

    for _, role in ipairs(self.m_roleList) do
    	if role and not role:isRoomOwner() and not role:isReady() then
    		return false, Serialize[64872]
    	end
    end

    return true
end

function This:tostring()
	local owner = self:getOwner()
	local ret = { 
		team_id = GameUtil:int64ToString(self:getId()),
		team_type = self:getType(),
		captain_id = GameUtil:int64ToString(owner.m_id),
		def_team_id = self.m_warfareId,
		war_full_name = Serialize[64873] .. self.m_warfareId,
		member_list = {
			[1] = {
				role_id = GameUtil:int64ToString(owner.m_id), 
				role_name = owner.m_name, 
				role_icon = owner.m_icon, 
				role_level = owner.m_level, 
				is_ready = owner.m_isReady, 
				rank_level = owner.m_rankLv, 
			}	
		}
	}
	return table.tostring(ret)
end

function This:getWarId()
    return self.m_warfareId
end

function This:getType()
    return self:getRelationType() or self.m_teamType or TeamType.kTowerTeam
end

function This:getChatType()
    return ChatType.kApplyTower
end

function This:getWarFullName()
    return Serialize[64874] .. self.m_warfareId
end
