-- @Author: superm
-- @Date:   2018-12-25 13:00:17
-- @Last Modified by:   superm
-- @Last Modified time: 2019-01-02 19:15:52
-- @File Name: WarfareBattleInfo.lua
-- @Description: 

local This = class(...)
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local BattleDataManager = require("Battle.BattleDataManager")

function This:create(info)
	local o = {}
	setmetatable(o,self)
	o:init(info)
	return o
end

function This:init(info)
	local msg = info.msg
	self.m_team = info.team
    self:updateBattleInfo(msg)
end

function This:setTeam( team )
    self.m_team = team
end

function This:updateBattleInfo(msg)
    self.m_reportList = {}
    self.m_reportIdList = {}
    self.m_fightPlayerList = msg.fight_player
    self.m_isWin = msg.iswin

    for index, id in ipairs(msg.report_id_list) do
        local key = GameUtil:int64ToString(id)
        self.m_reportIdList[index] = key
        self.m_reportList[key] = false
    end
    
    self.m_rewardList = msg.raward_list
    self.m_rewardListForCaptain = msg.captain_raward_list
end

function This:clear()
    self.m_reportIdList = nil
    self.m_reportList = nil
    self.m_fightPlayerList = nil
end

function This:getConfig()
    return self.m_team:getWarfareConfig()
end

function This:isCaptain()
    return self.m_team:isCaptain()
end

function This:getRewardList()
    return self.m_rewardList, self.m_rewardListForCaptain  
end

function This:isWin()
    return self.m_isWin
end

function This:loadBattle(selectHandler, needCheck)
    local ret = false
    if self.m_reportList then
        for id, battle in pairs(self.m_reportList) do
            if not battle then
                local report = selectHandler(id)
                if report then
                    self.m_reportList[id] = report
                else
                    print("获取战报失败, id=" .. id)
                    ret = false
                    break 
                end
            end
            ret = battle ~= nil
        end

        if ret then
            self:startBattle(needCheck)
        end
    end
    return ret
end

function This:startBattle(needCheck)
    print("开始准备战斗...")
    print("检查战报")
    if not self.m_reportList then
        print("检查战报失败, 战斗中止")
        return
    end

    local list = {}
    if not needCheck then
        for index, id in pairs(self.m_reportIdList) do
            list[index] = self.m_reportList[id]
        end
    else
        for index, id in pairs(self.m_reportIdList) do
            local report = self.m_reportList[id]
            if report then
                local endTime = report.end_time + 2 --(服务器延时导致+1秒)
                local curTime = GamePlayer:getServerTime()
                if endTime > curTime then
                    table.insert(list, report)
                end
            end
        end

        if #list < 1 then
            print("检查战报数量小于1, 战斗中止")
            self:clear()
            return
        end 
    end

    local params = { mode = PlayMode.kWarfare, bgId = 1, 
        msg = {
            report_list = list,
            fight_player = self.m_fightPlayerList,
        },
    }
    print("检查结束 , 战斗正式开启")
    BattleDataManager:addObserver(self)
    cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, params)
end

function This:onShowBattleResult(victory)
    if self.m_reportList then
        BattleDataManager:removeObserver(self)
        if self.m_team and self.m_team:getWarId() == 12 then
            if victory then
                local reward
                if table.count(self.m_rewardListForCaptain ) > 0 then
                    reward = self.m_rewardListForCaptain
                else
                    reward = self.m_rewardList
                end
                g_scene:openDialog("Stage.StageVictoryPanel",{
                    -- stageId = 1,
                    isRobber = true,
                    dropInfo = reward
                })
            else
                g_scene:openDialog("Stage.StageFailedPanel", {
                    isRobber = true,
                })
            end
        else
            g_scene:openDialog("Warfare.WarfareOverPanel", self)
        end
        self:clear()
    end
end

function This:onBattleVictory()
    self:onShowBattleResult(true)
end

function This:onBattleFailed()
    self:onShowBattleResult()
end
