-- @Author: superm
-- @Date:   2018-08-11 10:55:30
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-12-03 15:37:37
-- @File Name: WarfareManager.lua
-- @Description: 

local This , super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local MatchBaseTeam = require("Warfare.MatchBaseTeam")
local WarfareInvite = require("Warfare.WarfareInvite")
local WarfareUtil = require("Warfare.WarfareUtil")
local GamePlayer = require("Player.GamePlayer")
local HeroManager = require("Hero.HeroManager")
local FamilyManager = require("Family.FamilyManager")
local GameUtil = require("Util.GameUtil")
local LruTable = require("Base.lrutbl")
local ChatManager = require("Chat.ChatManager")
local ArgConfig = require("Metadatas.ArgConfig")

function This:isInited()
    return self.m_inited
end

function This:init()
    super.init(self)
    self.m_inited = false
    self.m_teams = {}
    self.m_teamIdMap = {}

    self.m_teamCacheList = {} 
    self.m_emBattleHeros = {}

    self.m_battleInfoList = {}

    self.m_friendInvitesRoles = {}
    self.m_invationList = {}
    self.m_markTimeList = {}
    self.m_reports = LruTable.new(50)
    self.m_battleList = {}

    self.m_teamCommandMap = {}
    self:initListener()
end

function This:initListener()
    EventDispatcher:addListener(self, "init_role_info", self.onInitAllWarfareInfo)

    EventDispatcher:addListener(self, 12505, self.onRecieveInvaiotnNotify)
    EventDispatcher:addListener(self, 13709, self.onRecieveTowerInvaiotnNotify)
    
    EventDispatcher:addListener(self, 12509, self.onWarInviteReply)
    EventDispatcher:addListener(self, 12510, self.onSwitchWarfareReply)
    EventDispatcher:addListener(self, 12512, self.onGetTeamListInfoReply)
    EventDispatcher:addListener(self, 12514, self.onRemoveTeamReply)

    EventDispatcher:addListener(self, 12515, self.onGetInviteFriendListReply)
    EventDispatcher:addListener(self, 12520, self.onGetWarExistInfoReply)
    EventDispatcher:addListener(self, 12521, self.onGetTeamRoleChangeNotify)

    local allCommands = {}
    for teamType, opGroup in pairs(TeamOperateCommand) do
        local commandMap = {}
        for operation, command in pairs(opGroup) do
            commandMap[command] = operation
            allCommands[command] = true
        end
        self.m_teamCommandMap[teamType] = commandMap
    end

    for command in pairs(allCommands) do
        if command > 0 then
            EventDispatcher:addListener(self, command, self.onOperateTeamReply)
        end
    end
end

function This:removeListener()
    EventDispatcher:removeListener(self, "init_role_info")
    EventDispatcher:removeListener(self,12501)
    EventDispatcher:removeListener(self,12502)
    EventDispatcher:removeListener(self,12503)
    EventDispatcher:removeListener(self,12505)
    EventDispatcher:removeListener(self,12504)
    EventDispatcher:removeListener(self,12506)
    EventDispatcher:removeListener(self,12507)
    EventDispatcher:removeListener(self,12508)
    EventDispatcher:removeListener(self,12509)
    EventDispatcher:removeListener(self,12510)
    EventDispatcher:removeListener(self,12512)
    EventDispatcher:removeListener(self,12513)
    EventDispatcher:removeListener(self,12514)
    EventDispatcher:removeListener(self,12515)
    EventDispatcher:removeListener(self,12516)
    EventDispatcher:removeListener(self,12518)
    EventDispatcher:removeListener(self,12519)
    EventDispatcher:removeListener(self,12520)
    EventDispatcher:removeListener(self,12521)

    EventDispatcher:removeListener(self,13701)
    EventDispatcher:removeListener(self,13702)
    EventDispatcher:removeListener(self,13703)
    EventDispatcher:removeListener(self,13704)
    EventDispatcher:removeListener(self,13705)
    EventDispatcher:removeListener(self,13706)
    EventDispatcher:removeListener(self,13707)
    EventDispatcher:removeListener(self,13709)
end

function This:onOperateTeamReply(msg)
    if not msg then
        g_scene:handleErrorCode("")
        return
    elseif msg.err_code and msg.err_code ~= 0 then
        g_scene:handleErrorCode(msg.err_code)
        return
    end

    local command, curTeamType, curOperation = msg.cmd, nil, nil
    for teamType, commandMap in pairs(self.m_teamCommandMap) do
        local operation = commandMap[command]
        if operation then
            curTeamType = teamType --default, but not correct
            curOperation = operation
            break
        end
    end

    if not curOperation or not curTeamType  then
        print("没找到相关操作")
        print_tmp(msg)
        return
    elseif msg.team_type then
        curTeamType = msg.team_type
    elseif msg.team_id then
        local ttype = self.m_teamIdMap[msg.team_id]
        if ttype then
            curTeamType = ttype
        end
    end

    if not self.m_replyMethodList then
        local callbackList = {
            [TeamOperateType.kGetTeamInfo] = self.onGetMyTeamInfoReply,
            [TeamOperateType.kCreateTeam] = self.onCreateTeamReply,
            [TeamOperateType.kJoinTeam] = self.onJoinMatchTeamReply,
            [TeamOperateType.kPrepare] = self.onPrepareWarReply,
            [TeamOperateType.kSelectHero ] = self.onWarEmbattleHerosReply,
            [TeamOperateType.kBattle] = self.onWarfareBattleReply,
            [TeamOperateType.kExitTeam] = self.onExitWarTeamReply,
            [TeamOperateType.kDismissTeam] = self.onExitWarTeamReply,
            [TeamOperateType.kChangeOwner] = self.onChangeTeamOwnerReply,
            [TeamOperateType.kQuickJoin] = self.onJoinTeamQuicklyReply,
            [TeamOperateType.kBattleReport] = self.onGetWarReportReply,
            [TeamOperateType.kModifyAttr] = self.onModifyAttrReply,
        }
        self.m_replyMethodList = callbackList
    end

    -- 服务端纠正
    if msg.op_type and msg.op_type ~= curOperation then
        curOperation = msg.op_type
    end
    if curTeamType ==TeamType.kTowerTeam and msg.cmd ==13706 then
        local report = msg.report
        if report and self.m_battleRepotIdTeamIdMap  then
           local teamId = self.m_battleRepotIdTeamIdMap[report.id]
           if teamId then
                local ttype = self.m_teamIdMap[teamId]
                if ttype then
                    curTeamType = ttype
                end
           end
        end
    end
    print("curOperation:" .. curOperation)
    local callback = self.m_replyMethodList[curOperation]
    if callback then
        callback(self, msg, curTeamType)
    end
    self:notifyObservers("onOperateTeamReply", curTeamType, curOperation)
end

function This:getNowWarState()
    return self.m_battleList
end

function This:getFightData()
    return self.m_battleInfoList
end

function This:getMyTeam(teamType)
    return self.m_teams[teamType or TeamType.kWarTeam]
end

function This:createMyTeam(info, teamType, offline)
    local team = self.m_teams[teamType]
    if not team then 
        team = self:createTeam(info, teamType, offline)
        self.m_teams[teamType] = team
    end
    return team
end

function This:createTeamBattleInfo(msg,  team)
    if not teamType or teamType == TeamType.kWarTeam then
        local WarfareBattleInfo = require("Warfare.WarfareBattleInfo")
        return WarfareBattleInfo:create({msg = msg, team = team})
    else

    end
end

function This:createTeam(info, teamType, offline)
    if not teamType or teamType == TeamType.kWarTeam then
        local WarfareTeam = require("Warfare.WarfareTeam")
        return WarfareTeam:create(info)
    else
        local TowerTeam = require("Warfare.TowerTeam")
        if teamType == TeamType.kWarTowerTeam then
            TowerTeam = require("Warfare.WarTowerTeam")
        end
        if info and not offline then
            return TowerTeam:create(info)
        else
            return TowerTeam:createOfflineTeam(info)
        end
    end
end

function This:updateMyTeamInfo(info, teamType)
    if not info then return end
    local team = self:getMyTeam(teamType)
    if not team then
        team = self:createMyTeam(info, teamType)
    else
        team:updateInfo(info)
    end
    self.m_teamIdMap[team:getId()] = team:getType()
    return team
end

function This:hasTeam(teamType)
    local team = self:getMyTeam(teamType)
    return team and team:getId() > 0
end

function This:isTeamLeader(teamType)
    local team = self:getMyTeam(teamType)
    if not team or team:getId() <= 0 then return false end
    local role = team:getMainRole()
    if not role then return false end
    return role:isRoomOwner()
end

function This:getTeamList( teamListInfo )
    if not teamListInfo then
        return
    end

    function addTeamInfo( info )
        local id = info.team_id 
        local team = self.m_teamCacheList[id]
        if not team then
            team = MatchBaseTeam:create(info)
            self.m_teamCacheList[id] = team
        else
            team:updateInfo(info)
        end
        return team
    end

    local ret = {}
    for i, info in ipairs(teamListInfo) do
        ret[i] = addTeamInfo(info)
    end
    return ret
end

function This:getEmBattleHeros(teamType)
    if not teamType then
        teamType = TeamType.kWarTeam
    end
    return self.m_emBattleHeros[teamType]
end

function This:hadEmbattled(teamType)
    local heros = self:getEmBattleHeros(teamType)
    if not heros then return false end

    for _, id in pairs(heros) do
        if id >= 0 then
            return true
        end
    end
    return false
end

function This:getCanInviteList( isGroup )
    local result = {}
    local roleSource
    if not isGroup then
        roleSource = self.m_friendInvitesRoles
    else
        self:getInviteGroupList()
        roleSource = self.m_groupInvitesRoles
    end

    for id, invite in pairs(roleSource) do
        if invite:isValid() and invite:isOnline() and id ~= GamePlayer:getPlayerID() then
            table.insert(result,invite)
        end
    end

    table.sort(result,function(a,b)
        if a:getState() ~= b:getState() then
            return a:getState() < b:getState()
        else
            return a:getId() < b:getId()
        end
        return false
    end)
    return result
end

function This:getHasInvite()
    return self.m_hasInvite or false
end

function This:setHasInvite(info)
    self.m_hasInvite = info
end

function This:getRole(id, teamType)
    if not self:hasTeam(teamType) then
        return
    end
    return self:getMyTeam(teamType):getRole(id)
end

function This:clearFightData(teamType)
    self.m_battleInfoList[teamType] = nil
end

function This:dismissTeam(teamType)
    local team = self.m_teams[teamType]
    if team then
        self.m_teamIdMap[team:getId()] = nil
        self.m_teams[teamType] = nil
    end

    self:clearFightData(teamType)
    if teamType ~= TeamType.kTowerTeam then
        self.m_friendInvitesRoles = {}
        self.m_invationList = {}
    end
    return team
end

function This:getReport(reportId)
    return self.m_reports:get(reportId)
end

function This:getInviteGroupList()
    if FamilyManager:hasFamily() and not self.m_groupMemList then
        local family = FamilyManager:getMyFamily()
        self.m_groupMemList = family.m_memberList
    end

    if self.m_groupMemList then
        local list = self.m_groupMemList:getAllNodeList()
        local ret = {}
        for id, model in ipairs(list) do
            --table.insert(ret, WarfareInvite:createByMember(model))
            ret[model.m_id]=WarfareInvite:createByMember(model)
        end
        self.m_groupInvitesRoles = ret
    end
end

------------------------------------------------------------
function This:onGetMyTeamInfoRequest(teamType, slient)
    local command, params
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12516
    else--if teamType >= TeamType.kTowerTeam then
        command = 13701
        params = {op_type = TeamOperateType.kGetTeamInfo, team_type = teamType, team_id = 0}
    end

    if not slient then
        Socket:sendAndLock(command, params)
    else
        Socket:send(command, params)
    end
end

function This:updateTeamInfo(msg, teamType)
    print("cmd:" .. msg.cmd)
    local teamList = msg.team_list
    if not teamList then
        return self:updateMyTeamInfo(msg.team or msg.team_info, teamType)
    else
        return self:updateMyTeamInfo(teamList[1], teamType)
    end
end

function This:onGetMyTeamInfoReply(msg, teamType)
    if msg then
        self:updateTeamInfo(msg, teamType)
        self:notifyObservers("onGetMyTeamInfo", teamType)
    else
        g_scene:handleErrorCode("")
    end
    
    if not self:isInited() then
        self.m_inited = true
        self:onGetWarExistInfoRequest(true)
    end
end

function This:onInitAllWarfareInfo()
    self:onGetMyTeamInfoRequest(TeamType.kWarTeam,true)
    self:onGetMyTeamInfoRequest(TeamType.kTowerTeam,true)
end

-- 13701 
function This:onCreateTeamRequest(id, hard, teamType)
    local command, params
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12501
        params = {copy_id = id, copy_level = hard}
    else--if teamType == TeamType.kTowerTeam then
        command = 13701
        params = {op_type = TeamOperateType.kCreateTeam, team_id = 0, team_type = teamType, def_team_id = id}
    end
    Socket:sendAndLock(command, params)
end

function This:onCreateTeamReply( msg, teamType )
    if msg.err_code == 0 then
        local team = self:updateTeamInfo(msg, teamType)
        if not teamType or teamType == TeamType.kWarTeam then
            self:notifyObservers("onGetWarfareTabData", "onApplyTeamWar")
            if team:getWar():getType() == WarfareType.kAnimal then
                self:notifyObservers("onCreateAnimalTeam",team)
            elseif team:getBossWar():getType() == WarfareType.kBoss and msg.team_info.copy_id == 12 then
                self:notifyObservers("onCreateBossTeam",team)

            end
        else
            self:notifyObservers("onCreateTeam", teamType, team:getId())
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 13701
function This:onJoinMatchTeamRequest(teamId, teamType)
    local command, params
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12502
        params = {team_id = teamId}
    elseif teamType >= TeamType.kTowerTeam then
        command = 13701
        params = {op_type = TeamOperateType.kJoinTeam, team_id = teamId}
    end
    Socket:sendAndLock(command, params)
end

function This:onJoinMatchTeamReply( msg, teamType )
    if msg.err_code == 0 then
        if not teamType or teamType == TeamType.kWarTeam then
            self:notifyObservers("onGetWarfareTabData", "onJoinMatchTeam")
        elseif teamType >= TeamType.kTowerTeam then
            self:updateTeamInfo(msg, teamType)
            self:notifyObservers("onJoinMatchTeam", teamType)
        end
    else
        self:notifyObservers("onJoinMatchTeamFail", teamType)
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 13702
function This:onPrepareWarRequest(isReady, teamType)
    local command
    local params = {is_ready = isReady}
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12503
    else
        local team = self:getMyTeam(teamType)
        if team then
            params.team_id = team:getId()
        end
        command = 13702
    end
    Socket:sendAndLock(command, params)
end

function This:onPrepareWarReply( msg, teamType )
    if msg.err_code == 0 then
        if teamType >= TeamType.kTowerTeam then
            local team = self:getMyTeam(teamType)
            if msg.team_id == team:getId() then
                local role = team:getRole(msg.role_id)
                if role then
                    role:updateInfo(msg)
                end
            end
        end
        self:notifyObservers("onPrepareWar", teamType)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 13705
function This:onWarfareBattleRequest(id, hard, userGold, teamType)
    local command, params
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12504
        params = {copy_id = id, copy_difficulty = hard or 1, user_gold = userGold}
    elseif teamType >= TeamType.kTowerTeam then
        command = 13705
        local team = self:getMyTeam(teamType)
        if team then
            params = {team_id = team:getId(), def_team_id = id}
        end
    end
    Socket:sendAndLock(command, params)
end

function This:onWarfareBattleReply( msg, teamType )
    if not msg.err_code or msg.err_code == 0 then
        self.m_battleInfoList[teamType] = nil

        local teamId = msg.team_id
        self.m_battleRepotIdTeamIdMap = {}
        if teamId then
            local idList = msg.report_id_list or {}
            for _,id in ipairs(idList) do
                self.m_battleRepotIdTeamIdMap[id] =teamId
             end 
        end

        self:playBattle(TeamBattleState.kInit, teamType, msg)
        return true
    elseif msg.err_code then
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:getMyTeamBattleInfo(msg, team)
    if team then
        local teamType = team:getType()
        if teamType == TeamType.kWarTeam then
            local WarfareBattleInfo = require("Warfare.WarfareBattleInfo")
            return WarfareBattleInfo:create({msg = msg, team = team})
        elseif teamType == TeamType.kTowerTeam then
            local TowerBattleInfo = require("Warfare.TowerBattleInfo")
            return TowerBattleInfo:create({msg = msg, team = team})
        elseif teamType >= TeamType.kWarTowerTeam then
            local TowerBattleInfo = require("Warfare.WarTowerBattleInfo")
            return TowerBattleInfo:create({msg = msg, team = team})
        end
    end
end

function This:playBattle(state, teamType, msg)
    print("playBattle", state, teamType)
    if state == TeamBattleState.kInit then
        local battleInfo = self.m_battleInfoList[teamType]
        if not battleInfo then
            local team = self:getMyTeam(teamType)
            local config = team:getWarfareConfig()
            battleInfo = self:getMyTeamBattleInfo(msg, team)
            self.m_battleInfoList[teamType] = battleInfo
            if config and (config.id == 11 or config.id == 12) then
                self:notifyObservers("onExitAnimalTeam",team,msg.iswin)
            end
        else
            table.insert(self.m_battleList, msg)
        end
    elseif state == TeamBattleState.kLoad then
        local reportInfo = msg.report
        local key = GameUtil:int64ToString(reportInfo.id)
        print("战报id=" .. key)
        self.m_reports:set(key, reportInfo)
        local battleInfo = self.m_battleInfoList[teamType]
        if battleInfo then
            battleInfo:loadBattle(function ( id )
                return self.m_reports:get(id)
            end)
        end
        self:notifyObservers("onExitInfo",reportInfo)

    elseif state == TeamBattleState.kLoadCache then
        local battleInfo = self.m_battleInfoList[teamType]
        if not battleInfo then
            local _, message = next(self.m_battleList)
            if message then
                if self:onWarfareBattleReply(message, teamType) then
                    if not battleInfo then
                        battleInfo= self.m_battleInfoList[teamType]
                    end
                    battleInfo:loadBattle(function ( id )
                        return self.m_reports:get(id)
                    end, true)
                    table.remove(self.m_battleList,1)
                end
            end
        end
    end
    self:notifyObservers("onFightStateChange")
end  

function This:loadCacheBattle(teamType)
    self:playBattle(TeamBattleState.kLoadCache, teamType or TeamType.kWarTeam)
end

-- 13704
function This:onExitWarTeamRequest(roleId, teamType)
    if not roleId then
        roleId = GamePlayer:getPlayerID()
    end

    local command, params
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12506
        params = {target_id = roleId}
    elseif teamType >= TeamType.kTowerTeam then
        command = 13704
        local team = self:getMyTeam(teamType)
        params = {op_type = TeamOperateType.kExitTeam, team_id = team:getId(), role_id = roleId}
    end
    Socket:sendAndLock(command, params)
end

function This:onExitWarTeamReply( msg, teamType )
    if msg.err_code == 0 then
        local roleId = msg.target_id or msg.role_id
        local team, role
        local myExit = false
        if roleId and roleId > 0 and GamePlayer:getPlayerID() ~= roleId then
            if self:hasTeam(teamType) then
                team = self:getMyTeam(teamType)
                if team then
                    role = team:removeRole(roleId)
                    --if team:getWar():getType() == WarfareType.kAnimal and team:getOwnerId() ~= GamePlayer:getPlayerID() then
                    -- if team:getWar():getType() == WarfareType.kAnimal then

                    --     -- self:onExitWarTeamRequest(GamePlayer:getPlayerID(), 1)
                    -- end
                end
            end
        else
            team = self:dismissTeam(teamType)
            if team then
                role = team:getRole(roleId)
                myExit = true
            end
        end
        if not teamType or teamType == TeamType.kWarTeam then
            self:notifyObservers("onGetWarfareTabData", "onExitWarTeam", self:hasTeam())
        elseif teamType >= TeamType.kTowerTeam then
            self:notifyObservers("onExitTowerTeam", teamType, role, myExit)
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onRemoveTeamRequest()
    Socket:sendAndLock(12514)
end

function This:onRemoveTeamReply(msg)
    if msg.err_code == 0 then
        local team = self:dismissTeam(TeamType.kWarTeam)
        if team then
           local role = team:getRole(GamePlayer:getPlayerID())
        end
        self:notifyObservers("onBossTeamRemove")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 13704
function This:onChangeTeamOwnerRequest(roleId)
    local command, params
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12507
        params = {target_id = roleId}
    elseif teamType >= TeamType.kTowerTeam then
        command = 13704
        local team = self:getMyTeam(teamType)
        params = {op_type = TeamOperateType.kChangeOwner, team_id = team:getId(), role_id = roleId}
    end
    Socket:sendAndLock(command, params)
end

function This:onChangeTeamOwnerReply(msg, teamType)
    if msg.err_code == 0 then
        self:notifyObservers("onChangeTeamOwner", msg.target_id or msg.role_id, teamType)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onWarEmbattleHerosRequest(heros, teamType)
    local result = {0,0,0,0,0,0}
    for i , info in pairs(heros) do
        local hero = HeroManager:getHero(info.heroId)
        result[info.pos] = hero:getUid()
    end

    local command, params
    if not teamType or teamType == TeamType.kWarTeam then
        command = 12508
        params = {card_list = result}
    elseif teamType >= TeamType.kTowerTeam then
        command = 13703
        local team = self:getMyTeam(teamType)
        params = {op_type = TeamOperateType.kSelectHero, team_id = team:getId(), card_ids = result}
    end
    Socket:sendAndLock(command, params)
end

function This:onWarEmbattleHerosReply( msg, teamType )
    if msg.err_code == 0 then
        local roleId = msg.role_id
        if not roleId or roleId == GamePlayer:getPlayerID() then
            local heros = {}
            self.m_emBattleHeros[teamType] = heros
            local cardList = msg.card_list or msg.card_ids
            for i , uid in ipairs(cardList) do
                if uid > 0 then
                    local uhero = HeroManager:getHeroByUid(uid)
                    heros[i] = uhero:getId()
                end
            end
        end
        
        local team = self:getMyTeam(teamType)
        if team then
            local role = team:getRole(roleId)
            if role then
                role:updateInfo(msg)
            end
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
    self:notifyObservers("onWarEmbattleHeros", teamType, msg.err_code == 0)
end

function This:onRecieveTowerInvaiotnNotify( msg )
    if msg then
        if self.m_Dialog then
            self.m_Dialog:close()
        end
        local temInfo =msg.team_info
        self.m_Dialog = g_scene:openDialog("Warfare.InviteWarfarePopPanel", {
            teamInfo = temInfo,
            teamType = temInfo.team_type or TeamType.kWarTowerTeam,
            time = 30
        })
        self:notifyObservers("onRecieveTowerInvaiotn", team)
    end
end

function This:onRecieveInvaiotnNotify( msg )
    if msg then
        if self.m_Dialog then
            self.m_Dialog:close()
        end
        self.m_Dialog = g_scene:openDialog("Warfare.InviteWarfarePopPanel", {
            teamInfo = msg.team_info,
            teamType = TeamType.kWarTeam,
            time = 30
        })
        self:notifyObservers("onRecieveInvaiotn", team)
    end
end
    
function This:onGetTowerInviteRequest(friendId,teamId)
    Socket:sendAndLock(13708,{friend_id=friendId,team_id= teamId})
end

function This:onWarInviteRequest(roleId)
    Socket:sendAndLock(12509, {friend_id = roleId})
end

function This:onWarInviteReply( msg )
    if msg.err_code == 0 then
        local roleId = msg.friend_id
        self.m_invationList[roleId] = true
        local invite = self.m_friendInvitesRoles[roleId]
        local view = invite:getView()
        if not view then
            invite = self.m_groupInvitesRoles[roleId]
        end
        if invite then
            invite:startUpdate()
        end
        self:notifyObservers("onWarInvite", roleId)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onSwitchWarfareRequest(warfareId, hard)
    Socket:sendAndLock(12510, {copy_id = warfareId, copy_difficulty = hard or 1})
end

function This:onSwitchWarfareReply( msg )
    if msg.err_code == 0 then
        self.m_emBattleHeros = {}
        self:notifyObservers("onSwitchWarfare")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetTeamListInfoRequest(warfareId)
    Socket:sendAndLock(12512, {copy_id = warfareId})
end

function This:onGetTeamListInfoReply( msg )
    if msg.err_code == 0 then
        local infoList = self:getTeamList(msg.team_list)
        self:notifyObservers("onGetWarfareTabData", "onGetTeamListInfo", infoList)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onJoinTeamQuicklyRequest(warfareId)
    Socket:sendAndLock(12513, {copy_id = warfareId})
end

function This:onJoinTeamQuicklyReply( msg )
    if msg.err_code == 0 then
        self:notifyObservers("onGetWarfareTabData", "onJoinTeamQuickly")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetInviteFriendListRequest()
    Socket:sendAndLock(12515)
end

function This:onGetInviteFriendListReply( msg )
    if msg then
        for _, invite in pairs(self.m_friendInvitesRoles) do
            invite:setIsValid(false)
        end
        for i , info in ipairs(msg.friend_list) do
            local id = info.friend_id
            local invite = self.m_friendInvitesRoles[id]
            if not invite then 
                invite = WarfareInvite:create(info)
                self.m_friendInvitesRoles[id] = invite
            else
                invite:updateInfo(info)
            end
        end
        self:notifyObservers("onGetInviteFriendList", "onGetWarfareTabData")
    end
end

function This:onSetAutoQuicklyJoinRequest(autoJoin)
    Socket:sendAndLock(12518, {can_auto_join = autoJoin})
end

function This:onSetTeamAttrRequest(teamId,opType,opState)
    Socket:sendAndLock(13707, {team_id = teamId,cfg_type= {opType},is_open ={opState} })
end

function This:onModifyAttrReply( msg, teamType )
    if msg.err_code == 0 then
        if not teamType or teamType == TeamType.kWarTeam then
            self:getMyTeam(teamType):setAutoJoin(msg.can_auto_join)
            self:notifyObservers("onGetWarfareTabData", "onSetAutoQuicklyJoin")
        else
            local opTypeList = msg.cfg_type
            if opTypeList then
                local opType = opTypeList[1]
                local isOpen = msg.is_open[1]
                if opType == TeamOperateType.kAutoStarGame then
                    self:getMyTeam(teamType):setAutoJoin(isOpen)
                    self:notifyObservers("onTeamAttrChange")
                end
            end
        end
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

-- 12519 
-- 13706
function This:onGetWarReportReply( msg, teamType )
    if not msg.err_code or msg.err_code == 0 then
        self:playBattle(TeamBattleState.kLoad, teamType, msg)
        self:notifyObservers("onGetWarReport")
    elseif msg.err_code then
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onGetWarExistInfoRequest()
    Socket:send(12520)
end

function This:onGetWarExistInfoReply( msg )
    if msg and msg.report_id_list and #msg.report_id_list > 0 then
        table.insert(self.m_battleList, msg)
        self:notifyObservers("onGetWarExistInfo")
    end
end

function This:onApplyJoinkWarfareRequest(team)
    if not team then
        return
    end

    local ret, errStr = ChatManager:isCanChatInWorld()
    if not ret then
        if errStr then
            g_scene:showTips(errStr)
        end
        return
    end
        
    local teamId = team:getType()
    local timeLimit
    if teamId >= TeamType.kTowerTeam then
        if not FamilyManager:hasFamily() then 
            g_scene:showTips(Serialize[61508])
            return 
        end
        timeLimit = ArgConfig["tower_help_time"].arg[1]
    else
        timeLimit = 180
    end

    local curTime = GamePlayer:getServerTime()
    local time = self.m_markTimeList[teamId] or 0
    local leftTime = curTime - time
    if leftTime > timeLimit then
        self.m_markTimeList[teamId] = curTime
        ChatManager:onApplyJoinkWarfareRequest(team)
        return true, timeLimit
    else
        g_scene:showTips(tostring(timeLimit - leftTime) .. Serialize[64881])
        return true, timeLimit - leftTime
    end
end

function This:onGetTeamRoleChangeNotify( msg )
    if msg.err_code == 0 then
        local role = self:getRole(msg.role_id)
        if role then
            role:updateInfo(msg)
        end
        self:notifyObservers("onGetTeamRoleChange")
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end
-----------------------------------------


--endregion 
