-- @Author: superm
-- @Date:   2018-08-08 17:28:54
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-04-15 17:29:54
-- @File Name: WarfarePanel.lua
-- @Description: 

local This, super = class(...,"Dialog")
local ViewUtil = require("UI.ViewUtil")
local TeamWarView = require("Warfare.TeamWarView")
local ArgConfig = require("Metadatas.ArgConfig")
local GamePlayer = require("Player.GamePlayer")
local AnimalTeamWarRefreshView = require("Warfare.AnimalTeamWarRefreshView")
local AnimalTeamWarView = require("Warfare.AnimalTeamWarView")
local TerrBuildManager = require("Territory.TerrBuildManager")
local ChallengeStageManager = require("Stage.ChallengeStageManager")
local SingleManager = require("SingleDaily.SingleManager")

function This:initNode(info)
    self.m_openIndex = info.tabIndex or 1
    self.m_openArg = info.arg

    self.m_build = info.build or TerrBuildManager:getBuildByType(TerrBuildType.kJiaoChang)

    self.m_viewList = {}
    self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/TeamWarList.csb")
    self.m_csNode:setToCenter()
    self:addChild(self.m_csNode)
    self.m_warTypeList = {WarfareType.kSingle, WarfareType.kTeam, WarfareType.kVip, WarfareType.kAnimal}

	local csNode = self.m_csNode
	csNode.ScrollView:setDirection("horizontal")
	csNode.ScrollView.m_direction = "right"

    csNode.tips:setVisible(false)
    ViewUtil:blinkView(csNode.tips, true)

    for i, wtype in ipairs(self.m_warTypeList) do
        self.m_viewList[wtype] = {}
        self:addTabButton(tostring(wtype), csNode["tep" .. i], function()
            if wtype == WarfareType.kAnimal then
                local limitLevel = ArgConfig.mythical_campaign_refresh_open_level.arg[1]
                if GamePlayer:getLevel() < limitLevel then
                    g_scene:showTips(string.format( Serialize[60304],limitLevel))
                    return false
                end
            end
            self.m_curTab = wtype
            self:selectTabByType( self.m_curTab )
            return true
        end)
    end
end

function This:initTouchEvent()
	if not self.m_inited then
		local csNode = self.m_csNode
	    csNode.close:setTouchEnded(function()
	        self:close()
	    end)

	    csNode.wenhao:setTouchEnded(function()
	    	g_scene:openDialog("UI.HelpTips", {text = Serialize[64884], node = csNode.wenhao})
	    end)

    	csNode.ScrollView:setSwallowTouch(false)
	end
end

function This:onEnter()
    super.onEnter(self)
    -- if self.m_build then
        self.m_build:addObserver(self)
    -- end
    SingleManager:addObserver(self)
    ChallengeStageManager:addObserver(self)

    self:refresh()
end

function This:onExit()
    super.onExit(self)
    -- if self.m_build then
        self.m_build:removeObserver(self)
    -- end
    ChallengeStageManager:removeObserver(self)
    SingleManager:removeObserver(self)
end

function This:refresh()
	self:selectTabButton(tostring(self.m_warTypeList[self.m_openIndex]))
end

function This:selectTabByType( tab )
    self:refreshWarViewList(tab, true, self.m_openArg)
    ViewUtil:blinkView(self.m_csNode.tips, tab ~= WarfareType.kAnimal)
    self.m_csNode.bgjianbian:setVisible( tab ~= WarfareType.kAnimal)
    self.m_openArg = nil
end

function This:refreshWarViewList(wtab, cleanup, clickArg)
	local scrollView = self.m_csNode.ScrollView
	if cleanup then
	    scrollView:removeAllItems()
	    self.m_viewList[wtab] = {}
	end
    local foundId, result
    if clickArg then
        foundId = clickArg.id
    end

    local warList = self.m_build:getAllWarfareList(wtab)
    local viewList = self.m_viewList[wtab]
    for index=1, math.max(#warList, 4) do
        local war = warList[index]
        local view = viewList[index]
        if not war and wtab == WarfareType.kAnimal then
            break
        end
        if not view then
            if war and war:getType() == WarfareType.kAnimal then
                view = AnimalTeamWarView:create(war)
            else
                view = TeamWarView:create(war)
            end
            view:retain()
        	viewList[index] = view
            local w , h = view:getContentSize()
            scrollView:pushBackCustomItem(view, w, h, 0, 15) 
        else
        	view:setModel(war)
        end

        if war and foundId and not result and foundId == war:getId() then
            result = view
        end
    end

    if wtab == WarfareType.kAnimal then
        if not self.m_animalNode then
            self.m_animalNode = AnimalTeamWarRefreshView:create({build = self.m_build})
            self.m_csNode:addChild(self.m_animalNode)
        end
    else
        if self.m_animalNode then
            self.m_animalNode:removeFromParent()
            self.m_animalNode = nil
        end
    end

    if result then
        if clickArg then
            result:onClick(clickArg.childArg)
        else
            result:onClick()
        end
    end
end

function This:onGetWarfareInfo()
    self:selectTabByType(self.m_curTab)
    if self.m_curTab == WarfareType.kAnimal then
        g_scene:showTips(Serialize[62153])
    end
end

function This:onRefreshChallengeTime()
    if self.m_curTab and WarfareType.kSingle ==  self.m_curTab then
        self:selectTabByType(self.m_curTab)
    end
end
