-- @Author: superm
-- @Date:   2018-12-24 17:14:57
-- @Last Modified by:   superm
-- @Last Modified time: 2019-03-02 15:22:29
-- @File Name: WarfareTeam.lua
-- @Description: 

local This , super = class(..., "Warfare.MatchTeam")
local TeamRole = require("Warfare.TeamRole")
local GameUtil = require("Util.GameUtil")
local TeamCampaignConfig

function This:addRole(roleInfo)
	local role = TeamRole:create(roleInfo)
	table.insert(self.m_roleList,role)
	role:setIsRoomOwner(roleInfo.role_id == self.m_ownerId)
end

function This:updateTeamAttrInfo( info )
	self:setAutoJoin(info.can_auto_join or true)
end

function This:updateWarfareInfo( info )
	if info.copy_id == 11 then
		self.m_warfareId = info.copy_level
		self.m_warfareHard = 5

	-- elseif info.copy_id == 12 then
	-- 	self.m_warfareId = info.copy_id
	-- 	self.m_warfareHard = info.copy_level
	-- 	--未写完
	else
		self.m_warfareHard = info.copy_level
		self.m_warfareId = info.copy_id
	end
    if info.copy_state then
    	self.m_inBattle = info.copy_state == 3
    end
end

function This:getHard()
    return self.m_warfareHard or 1
end

function This:getWarfareConfig()
	if not TeamCampaignConfig then
		TeamCampaignConfig = require("Metadatas.TeamCampaignConfig")
	end
	local config = self.m_warConfig
	if not config then
		if self.m_warfareId >=100 then
			config = TeamCampaignConfig[11]
		else
			config = TeamCampaignConfig[self.m_warfareId]
		end
	    self.m_warConfig = config
	end
    return config
end

function This:getWarName()
	local config = self:getWarfareConfig()
    return config.name
end

function This:getWarFullName()
	local WarfareUtil = require("Warfare.WarfareUtil")
	local config = self:getWarfareConfig()
	if config.id == 12 then
		return Serialize[64887]
	else
		return string.format("%s（%s）", config.name, WarfareUtil.getHardDesc(self:getHard()))
	end
end

function This:getWarId()
    return self.m_warfareId
end

function This:canChallenge()
	local config = self:getWarfareConfig()
    local curNum, minNum = #self.m_roleList, config.number[1]	
    if curNum < minNum then
    	return false, Serialize[64871]
    end

    for _, role in ipairs(self.m_roleList) do
    	if role and not role:isRoomOwner() and not role:isReady() then
    		return false, Serialize[64872]
		end
		if role and role:getLevel() < config.level then
			return false, Serialize[64888]
		end
    end
    return true
end


function This:tostring()
	local owner = self:getOwner()
	local ret = { 
		team_id = GameUtil:int64ToString(self:getId()),
		owner_id = GameUtil:int64ToString(self.m_ownerId),
		copy_id = self.m_warfareId,
		copy_level = self.m_warfareHard,
		war_full_name = self:getWarFullName(),
		role_list = {
			[1] = {
				role_id = GameUtil:int64ToString(owner.m_id), 
				role_name = owner.m_name, 
				role_icon = owner.m_icon, 
				role_level = owner.m_level, 
				is_ready = owner.m_isReady, 
				rank_level = owner.m_rankLv, 
			}	
		}
	}
	return table.tostring(ret)
end

function This:getWar()
	if not self.m_build then
		local TerrBuildManager = require("Territory.TerrBuildManager")
    	self.m_build = TerrBuildManager:getBuildByType(TerrBuildType.kJiaoChang)
    end

	return self.m_build:getWarfare(self.m_warfareId)
end

function This:getBossWar()
	return self.m_build:getBossWar(self.m_warfareHard)
end

function This:canJoin()
    if not self.m_autoJoin then
    	return false
    end

	local number, max = self:getPeople()
	if number >= max then
		return false
	end

	local war = self:getWar()
	if not war or not war:isOpening() then
		return false
	end

    return true
end

function This:getType()
    return TeamType.kWarTeam
end

function This:getChatType()
    return ChatType.kApplyTeamWarfare
end


