--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local Application = require("cc.Application")
local GamePlayer = require("Player.GamePlayer")
local ResourceConfig = require("Metadatas.ResourceConfig")
local WorldMapManager = require("World.WorldMapManager")
local CustomsConfig = require("Metadatas.CustomsConfig")
local WorldUtil = require("World.WorldUtil")
local EventDispatcher = require("EventDispatcher")

local BuildDesc = 
{
	[BuildType.kDwelling] = Serialize["world_Building_1"],
	[BuildType.kFarmland] = Serialize["world_Building_2"],
	[BuildType.kMarket] = Serialize["world_Building_3"],
	[BuildType.kMainBuild] = Serialize["world_Building_4"],
}

local kCityMainBuildMaxLv = 5
local kTownMainBuildMaxLv = 3
local kCommonBuildMaxLv = 10

function This:create(info,city)
	local b = {}
	setmetatable(b,self)
	b:init(info,city)
	return b
end

function This:init(info,city)
	self.m_city = city
	self.m_pos = info.pos
	self.m_level = info.level
	self.m_type = info.building_type
	self.m_lvUpStartTime = info.level_up_start_time-1
	self.m_state = info.level_up_state 
	self.m_escape = 1
	self.m_view = nil
	if self.m_state == BuildState.kUpgrading then
		self:upgradeStart(info)
	end
end

function This:getType()
	return self.m_type
end

function This:getLevel()
	return self.m_level
end

function This:setLevel(level)
	self.m_level = level
	if self.m_view then
		self.m_view:refresh()
	end
end

function This:getCity()
	return self.m_city
end

function This:getPos()
	return self.m_pos
end

function This:getName()
	if self.m_type == BuildType.kDwelling then
		return Serialize["world_Building_5"]
	elseif self.m_type == BuildType.kFarmland then
		return Serialize["world_Building_6"]
	elseif self.m_type == BuildType.kMarket then
		return Serialize["world_Building_7"]
	elseif self.m_type == BuildType.kMainBuild then
		return Serialize["world_Building_8"]
	end
end

function This:getDesc()
	local format = BuildDesc[self.m_type]
	if self.m_type == BuildType.kDwelling or self.m_type == BuildType.kMainBuild then
		return string.format(format,self:getReserveHpProduct())
	elseif self.m_type == BuildType.kFarmland then
		return string.format(format,self:getFoodProduct())
	elseif self.m_type == BuildType.kMarket then
		return string.format(format,self:getCopperProduct())
	end
end

function This:getState()
	return self.m_state
end

function This:setState(state)
	self.m_state = state
end

function This:setView(view)
	self.m_view = view
end

function This:getView()
	return self.m_view
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	local endTime = 0
	local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kBuildTime)
	if self.m_type == BuildType.kMainBuild then
		if CustomsConfig[self.m_level + 1] then
			local time = CustomsConfig[self.m_level + 1].time
			endTime = self.m_lvUpStartTime + time + value + time * percent
		else
			return 0
		end
	else
		if ResourceConfig[self.m_level + 1] then
			local time = ResourceConfig[self.m_level + 1].time
			endTime = self.m_lvUpStartTime + time + value + time * percent
		else
			return 0
		end
	end
	return endTime - curTime
end

function This:getMaxLv()
	if self.m_type == BuildType.kMainBuild then
		if self.m_city:getType() == CityType.kCity then
			return kCityMainBuildMaxLv
		else
			return kTownMainBuildMaxLv
		end
	else
		return kCommonBuildMaxLv
	end
end

function This:isCanUpgrade()
	local maxLv = self:getMaxLv()
	if self.m_level >= maxLv then
		return false
	end
	if self.m_state ~= BuildState.kFree then
		return false
	end
	return true
end

function This:getLvUpCost()
	if self.m_type == BuildType.kMainBuild then
		if self.m_level < self:getMaxLv() then
			local cost = CustomsConfig[self.m_level + 1].cost
			local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kBuildCost)
			return cost + value + percent * cost
		end
	elseif self.m_level < self:getMaxLv() then
		local cost = ResourceConfig[self.m_level + 1].cost
		local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kBuildCost)
		return cost + value + percent * cost
	end
	return 0
end

function This:getLvUpTime()
	if self.m_level >= self:getMaxLv() then
		return 0
	end
	if self.m_type == BuildType.kMainBuild then
		local cost = CustomsConfig[self.m_level + 1].time 
		local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kBuildTime)
		cost = cost + value + percent * cost
		return math.max(cost,0)
	else
		local cost = ResourceConfig[self.m_level + 1].time
		local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kBuildTime)
		cost = cost + value + percent * cost
		return math.max(cost,0)
	end
	return 0
end

function This:setUpgradeStartTime(time)
	self.m_lvUpStartTime = time
end

function This:getUpgradeStartTime()
	return self.m_lvUpStartTime
end

function This:updateInfo(info)
	local oldState = self.m_state
	local oldLevel = self.m_level
	self.m_pos = info.pos
	self.m_level = info.level
	self.m_type = info.building_type
	self.m_lvUpStartTime = info.level_up_start_time
	self.m_state = info.level_up_state 
	if oldState ~= BuildState.kUpgrading and self.m_state == BuildState.kUpgrading then
		self:upgradeStart(info)
	elseif oldState == BuildState.kUpgrading and self.m_state == BuildState.kFree then
		self:upgradeEnd()
	elseif oldState == BuildState.kUpgrading and self.m_state == BuildState.kUpgrading and oldLevel ~= self.m_level then
		if self.m_view then
			self.m_view:refresh()
		end	
		WorldMapManager:onBuildUpgrade(self.m_city, self)
	end
end

function This:getReserveHpProduct()
	if self.m_level <= 0 then
		return 0
	end
	local product = 0
	if self.m_type == BuildType.kMainBuild then
		product = CustomsConfig[self.m_level].force_num
	elseif self.m_type == BuildType.kDwelling then
		product = ResourceConfig[self.m_level].force_num
	end
	local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kProduct)
	return product + value + product*percent
end

function This:getCopperProduct()
	if self.m_level <= 0 then
		return 0
	end
	if self.m_type == BuildType.kMarket then
		local product =  ResourceConfig[self.m_level].coin_num
		local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kProduct)
		return product + value + product*percent 
	end
	return 0
end

function This:getFoodProduct()
	if self.m_level <= 0 then
		return 0
	end
	if self.m_type == BuildType.kFarmland then
		local product = ResourceConfig[self.m_level].food_num
		local percent,value = WorldMapManager:getBuildAddValue(self.m_type,BuildAddType.kProduct)
		return product + value + product*percent 
	end
	return 0
end

function This:getRemoveCost()
	if self.m_level <= 0 then
		return 0
	end
    return ResourceConfig[self.m_level].remove_cost
end

function This:upgradeImmediately()
	self:upgradeEnd()
end

function This:upgradeStart(info)
	self.m_lvUpStartTime = info.level_up_start_time
	self.m_state = info.level_up_state
	Application:addToUpdateList(self)
	if self.m_view then
		self.m_view:refresh(info.delay)
	end
	EventDispatcher:dispatcher("build_upgrade_start",city,build)
end

function This:upgradeEnd()
	Application:removeFromUpdateList(self)
	self.m_state = BuildState.kFree
	self.m_level = self.m_level + 1
	if self.m_view then
		self.m_view:refresh()
	end	
	WorldMapManager:onBuildUpgrade(self.m_city, self)
	EventDispatcher:dispatcher("build_upgrade_end",city,build)
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1 then
		local leftTime = self:getLeftTime()
		if leftTime > 0 then
			if self.m_view then
				self.m_view:refreshLeftTime(leftTime)
			end
		else
			self:upgradeEnd()
		end
		self.m_escape = self.m_escape - 1
	end
end

function This:clear()
	if self.m_state == BuildState.kUpgrading then
		self:upgradeEnd()
	end
end

--endregion
