--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local AchieveManager = require("Achieve.AchieveManager")
local GuideManager = require("Guide.GuideManager")
local EventDispatcher = require("EventDispatcher")
local GuideUtil = require("Guide.GuideUtil")

function This:init()
	self.m_isRunning = false
	self.m_timer = g_scene.m_timer
	self.m_timerKey = "chapter_guide"
	self.m_escape = 0
	self.m_events = {"dialog_open", "dialog_close", "guide_next_step", "guide_start", "world_scene_enter","enter_city","exit_city","anc_guide_changed","task_switch_click"}
end

function This:onEnter()
	super.onEnter(self)
	self:initNodeInfo()
	self:scheduleUpdate()
	for i , event in ipairs(self.m_events) do
		EventDispatcher:addListener(self, event, function(obj, isShow)
			if event == "task_switch_click" then
				if not isShow then
					self:cancel()
				end
				return
			end

			local ret = self:check()
			if self.m_isRunning and not ret then
				self:cancel()
			end
			if not ret then
				self.m_escape = 0
			end
		end)
	end
end

function This:onExit()
	super.onExit(self)
	self.m_timer:removeTimer(self.m_timerKey)
	for i , event in ipairs(self.m_events) do
		EventDispatcher:removeListener(self,event)
	end
end

function This:check()
	local entry = GuideManager:getCurGuideEntry()
	if not entry then
		self:finish()
		return
	end

	if entry:isRunning()  then
		self:cancel()
		return false
	end

	if GuideManager:hasAncGuider() then
		return false
	end
	
	if not entry:isRunning() then
		if g_scene:getName() ~= "WorldScene" or #g_scene:getAllDialogs() > 0 then
			return false
		end
		if g_scene:getShowMode() ~= ShowMode.kNormal then
			return false
		end
		local node = g_scene:getUi():getTaskNode()
		if not node:isVisible() or not g_scene:getUi():isVisible() then
			return false
		end
	end

	local mainAchieve = AchieveManager:getCurMainAchieve()
	if mainAchieve and mainAchieve:isCycleCheck() and mainAchieve:getState() ~= AchieveState.kHaseReward then
		return true
	end

	if mainAchieve then
		local subAchieves = AchieveManager:getMainSubAchieveWithoutSort(mainAchieve:getId())
		for i ,achieve in pairs(subAchieves) do
			if achieve:isCycleCheck() and achieve:getState() ~= AchieveState.kHaseReward then
				return true
			end
		end
	end

	return false
end

function This:refresh()
	local ret = self:check()
	if self.m_isRunning and not ret then
		self:cancel()
	elseif not self.m_isRunning and ret then
		self:guide()
	end
end

function This:guide()
	if not self:check() then
		self:cancel()
		return
	end

	GuideUtil:showRecommend()

	if not self.m_finger then
		local x, y = self:getTouchCenterXY()
		self.m_finger = cc.Node:createFromCSFile("Common/NodeUI/Finger.csb")
		self.m_finger:playCSAnimation()
		self.m_finger:setPosition(x, y)
		self:addChild(self.m_finger)
	else
		self.m_finger:setVisible(true)
	end
	self.m_isRunning = true
end

function This:cancel()
	if self.m_finger then
		self.m_finger:setVisible(false)
	end
	self.m_isRunning = false
	self.m_escape = 0
end

function This:finish()
	for i , event in ipairs(self.m_events) do
		EventDispatcher:removeListener(self,event)
	end
	self:cancel()
	self:unscheduleUpdate()
end

function This:update(dt)
	self.m_escape = self.m_escape  + dt
	if self.m_escape > 5.0 then
		local ret = self:check()
		if not self.m_isRunning and ret then
			self:guide()
		end
		self.m_escape = self.m_escape - 5.0
	end
end

function This:initNodeInfo()
	local path = "WorldScene.sceneUI.m_csNode.Recommend.main.bg"
	self.m_node = GuideUtil:getNodeByPath(path, true)

	if self.m_node then
		local rect = self.m_node:getBoundingBox()
		if rect.width == 0 or rect.height == 0 then
			local width , height = 60,60
			self.m_touchRect = cc.Rect(rect.x - width/2, rect.y - height/2, width, height)
		else
			self.m_touchRect = rect
		end
	end
end

function This:getTouchCenterXY()
	if self.m_touchRect then
		return self.m_touchRect:getCenterXY()
	end
	return 120, 370
end

--endregion
