
--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local CityConfig = require("Metadatas.CityConfig")
local Building = require("World.Building")
local WorldMapManager = require("World.WorldMapManager")
local Troop = require("World.Troop")
local GamePlayer = require("Player.GamePlayer")
local WorldUtil = require("World.WorldUtil")
local RoomConfig = require("Metadatas.RoomConfig")
local ResourceConfig = require("Metadatas.ResourceConfig")
local ReportManager = require("Report.ReportManager")
local HeroManager = require("Hero.HeroManager")
local MarchArmy = require("World.MarchArmy")
local GameUtil = require("Util.GameUtil")
local TreasureBox = require("World.TreasureBox")
local DepotManager = require("Depot.DepotManager")

CityState = 
{
	kNormal = 0,
	kInFighting = 1,
	kProtected = 2,
}

function This:create(...)
	local c = {}
	setmetatable(c,self)
	c:init(...)
	return c
end

function This:init(info)
	self.m_id = info.city_id or 0
	self.m_config = CityConfig[self.m_id]

	if self.m_config == nil then
		print("城市信息出错",info)
		assert(false)
	end

	self.m_ownerId = info.owner_id or 0
	self.m_totalHp = info.force_num or 0
	self.m_reserveHp = info.reserve_force_num or 0
	self.m_level = info.level or 0
	self.m_reportId = info.report_id
	self.m_autoUpgradeTimes = info.auto_times
	self:updateState(info.fighting_state)

	self.m_buildings = {}
	for _, build in pairs(info.building_list or {}) do
		self:addBuild(build)
	end

	self.m_troops = {}
	for _, army in pairs(info.army_list or {}) do
		self:addTroop(army)
	end

	self.m_defenceQueue = {}
	self.m_defenceArmys = {}
	for _, info in pairs(info.guard_list or {}) do
		self:addDefenceArmy(info)
	end

	self.m_attackQueue = {}
	self.m_attackArmys = {}
	for _, info in pairs(info.attack_list or {}) do
		self:addAttackArmy(info)
	end

	self.m_treasuerBoxs = {} --藏宝图宝箱
	for _, info in pairs(info.treasure_box_list or {}) do
		self:addTreasureBox(info)
	end

	self.m_stgList = {}
	self.m_investInfo = {}
	self.m_isInvestigated = false
	self.m_isAutoAddHp = info.auto_change_force
	self.m_isImprisoned = false --禁锢状态
	self.m_isAdvanceInvestigated = false
end

function This:getCityId()
	return self.m_id
end

function This:getId()
	return self.m_id
end

function This:getName()
	return self.m_config.name
end

function This:getType()
	return self.m_config.type
end

function This:isHanXianDi()
	return WorldMapManager:isHanXianDi(self.m_id)
end

function This:getState()
	return self.m_state
end

function This:setState(state)
	self.m_state = state
end

function This:getReportId()
	return self.m_reportId
end

function This:getInitLevel()
	return self.m_config.begin_level
end

function This:getMaxLevel()
	return self.m_config.Max_level
end

function This:getNeighborhood()
	return self.m_config.neighborhood
end

function This:buildUpgradeImmediately(posList)
	for i , pos in ipairs(posList) do
		local build = self:getBuild(pos)
		build:upgradeImmediately()
	end
end

function This:getOwnerId()
	return self.m_ownerId
end

function This:isVisibleInGuide()
	local mapType = WorldMapManager:getMapType()

	-- 策划要求的,皇帝战的改成31
	if mapType == 100 then
		mapType = 31
	end

	return #CityConfig[self.m_id]["interval"..mapType] > 0
end

function This:getStgList()
	return self.m_stgList
end

--自己是否能对该城池使用献帝手谕
function This:isUsedOnCity()
	local res = false
	local hdsyItem=nil
	for _,info in pairs(self.m_stgList) do
		if info.state == JiCeState.kHDSY then
			hdsyItem = info
			break
		end
	end
	if hdsyItem then
		local userList = hdsyItem.user_list or {}
		local playerId = GamePlayer:getPlayerID()
		for _,id in pairs(userList) do
			if playerId == id then
				res = true
			end
		end
	end
	return res,hdsyItem
end


function This:isHide()
	return false
end

function This:isWildCity()
	return self.m_ownerId <= 0
end 

function This:isMyMainCity()
	local id = GamePlayer:getPlayerID()
	local role = WorldMapManager:getRole(id)
	if role then
		local cityId = role:getMainCity()
		return self.m_id == cityId
	end
end

function This:isFriendMainCity()
	local role = WorldMapManager:getRole(self.m_ownerId)
	if role and role:getRelationWithMe() > 0 and role:getMainCity() == self.m_id then
		return true
	end
end

function This:isMainCity()
	if self.m_ownerId > 0 then
		local role = WorldMapManager:getRole(self.m_ownerId)
		return role and self.m_id == role:getMainCity()
	end
	return false
end

function This:isProtected()
	return self.m_state == CityState.kProtected
end

function This:isFighting()
	return self.m_state == CityState.kInFighting
end

function This:isImprisoned()
	return self.m_isImprisoned
end

function This:setIsImprisoned(isImprisoned)
	self.m_isImprisoned = isImprisoned
end

function This:getTotalHp()
	return self.m_totalHp
end

function This:setTotalHp(totalHp)
	self.m_totalHp = totalHp
end

function This:getReserveHp()
	return self.m_reserveHp
end

function This:getTroopTotalHp()
	local total = 0
	for id , troop in pairs(self.m_troops) do
		total = total + troop:getHp()
	end
	return total
end

function This:getAllTroopsTotalHp()
    local total = 0
    for i , army in ipairs(self.m_defenceQueue) do
		local troops = army:getAllTroop()
		for id , troop in pairs(troops) do
			total = total + troop:getHp()	
		end
	end
	for i ,troop in pairs(self.m_troops) do
		total = total + troop:getHp()
	end
	return total
end

function This:setReserveHp(reserveHp)
	self.m_reserveHp = reserveHp
end

function This:getMaxReserveHp()
	local index = WorldMapManager:getMapType()
	local config = RoomConfig[index]
	return config.init_force_num
end

function This:getLevel()
	return self.m_level
end

function This:setLevel(level)
	self.m_level = level
	--[[local build = self.m_buildings[BuildPos.kMainBuild]
	if build then
		build:setLevel(level)
	end]]
end

function This:isInvestigated()
	return self.m_isInvestigated
end

function This:setInvestTime(time)
	self.m_isInvestigated = time > 0
	self.m_investInfo.time = time
end

function This:isAdvanceInvestigated()
	return self.m_isAdvanceInvestigated
end

function This:setIsAdvanceInvestigated(isAdvance)
	self.m_isAdvanceInvestigated = isAdvance
end

function This:getInvestTime()
	return self.m_investInfo.time
end

function This:isAutoAddHp()
	return self.m_isAutoAddHp
end

function This:setIsAutoAddHp(isAuto)
	self.m_isAutoAddHp = isAuto
end

function This:getAutoUpgradeTimes()
	return self.m_autoUpgradeTimes 
end

function This:setAutoUpgradeTimes(times)
	self.m_autoUpgradeTimes = times
end

function This:getOwnerTroop()
	return self:getRoleTroop(self.m_ownerId)
end

function This:getFriendTroop()
	local result = {}
	for i ,army in ipairs(self.m_defenceQueue) do
		if army:getOwnerId() ~= GamePlayer:getPlayerID() then
			local troops = army:getAllTroop()
			for id , troop in pairs(troops) do
				table.insert(result,army)		
			end
		end
	end
	for i ,troop in pairs(self.m_troops) do
		if troop:getOwnerId() ~= GamePlayer:getPlayerID() then
			table.insert(result,troop)
		end
	end
	return result
end

function This:getBuild(pos)
	return self.m_buildings[pos]
end

function This:getAllBuild()
	return self.m_buildings
end

function This:getTroop(id)
	return self.m_troops[id]
end

function This:getAllTroop()
	return self.m_troops
end

function This:getReserveHpProduct()
	local total = 0
	for i , build in pairs(self.m_buildings) do
		total = total + build:getReserveHpProduct()
	end
	return total
end

function This:getCopperProduct()
	local total = 0
	for i , build in pairs(self.m_buildings) do
		total = total + build:getCopperProduct()
	end
	return total
end

function This:getFoodProduct()
	local total = 0
	for i , build in pairs(self.m_buildings) do
		total = total + build:getFoodProduct()
	end
	return total
end

function This:updateInfo(info)
	if self.m_ownerId ~= info.owner_id then
		local role = WorldMapManager:getRole(self.m_ownerId)
		if role then
			role:removeCity(self.m_id)
		end
		self.m_ownerId = info.owner_id
		role = WorldMapManager:getRole(self.m_ownerId)
		if role then
			role:addCity(self.m_id)
		end
	end

	local troopUpdateList = {}
	for i , troop in pairs(info.army_list) do
		troopUpdateList[troop.id] = troop
	end
	for id , troop in pairs(self.m_troops) do
		if not troopUpdateList[id] then
			self:removeTroop(id)
		end 
	end
	for id , troop in pairs(troopUpdateList) do
		if not self.m_troops[id] then
			self:addTroop(troop)
		else
			self:updateTroop(troop)
		end
	end

	local updateDefenceList = {}
	for i , army in pairs(info.guard_list) do
		updateDefenceList[army.id] = army
	end
	for i , army in pairs(self.m_defenceQueue) do
		if not updateDefenceList[army:getId()] then
			self:removeDefenceArmy(army:getId())
		end
	end
	for i , army in pairs(info.guard_list) do
		if not self.m_defenceArmys[army.id] then
			self:addDefenceArmy(army)
		else
			self:updateDefenceArmy(army)
		end
	end

	print("更新防守部队数量:")
	for id , army in pairs(self.m_defenceArmys) do
		print("-------------m_defenceArmys--------------",GameUtil:int64ToString(id))
	end

	local updateAttackList = {}
	for i , army in pairs(info.attack_list) do
		updateAttackList[army.id] = army
	end
	for id , army in pairs(self.m_attackArmys) do
		if not updateAttackList[army:getId()] then
			self:removeAttackArmy(army:getId())
		end
	end
	for i , army in ipairs(info.attack_list) do
		if not self.m_attackArmys[army.id] then
			self:addAttackArmy(army)
		else
			self:updateAttackArmy(army)
		end
	end
	print("更新进攻部队数量:")
	for id , army in pairs(self.m_attackArmys) do
		print("-------m_attackArmys------------",GameUtil:int64ToString(id))
	end

	local buildUpdateList = {}
	for i , build in pairs(info.building_list) do
		buildUpdateList[build.pos] = build
	end
	for pos, build in pairs(self.m_buildings) do
		if not buildUpdateList[pos] then
			self:removeBuild(pos)
		end
	end
	for pos , build in pairs(buildUpdateList) do
		if self.m_buildings[pos] then
			self.m_buildings[pos]:updateInfo(build)
		else
			self:addBuild(build)
		end
	end

	local treasureBoxList = {}
	for i , info in pairs(info.treasure_box_list) do
		treasureBoxList[info.treasure_id] = info
	end
	for i , box in pairs(self.m_treasuerBoxs) do
		local id = box:getId()
		if not treasureBoxList[id] then
			self:removeTreasureBox(id)
		end
	end
	for i , info in pairs(treasureBoxList) do
		self:addTreasureBox(info)
	end

	self.m_totalHp = info.force_num
	self.m_reserveHp = info.reserve_force_num
	self.m_level = info.level
	self.m_autoUpgradeTimes = info.auto_times
	self.m_reportId = info.report_id
	self:updateState(info.fighting_state)
end

function This:getRoleTroop(ownerId)
	local troopsList = {}
	if ownerId == self.m_ownerId then
		for i , troop in pairs(self.m_troops) do
			table.insert(troopsList,troop)
		end
		table.sort(troopsList,function(a,b)
			return a:getPos() < b:getPos()
		end)
	else
		local armyList = {}
		for i , army in pairs(self.m_attackQueue) do
			if army:getOwnerId() == ownerId then
				table.insert(armyList,army)
			end
		end
		for i , army in pairs(self.m_defenceQueue) do
			if army:getOwnerId() == ownerId then
				table.insert(armyList,army)
			end
		end

		for i , army in pairs(armyList) do
			local troops = army:getAllTroop()
			for i , troop in pairs(troops) do
				table.insert(troopsList,troop)
			end
		end
		table.sort(troopsList,function(a,b)
			return a:getPos() < b:getPos()
		end)
	end
	return troopsList
end

function This:isInCity(ownerId)
	if self.m_ownerId == ownerId then
		return true
	else
		for i , army in pairs(self.m_attackArmys) do
			if army:getOwnerId() == ownerId then
				return true
			end
		end
		for i , army in pairs(self.m_defenceArmys) do
			if army:getOwnerId() == ownerId then
				return true
			end
		end
	end
	return false
end

function This:setIsInvestigated(isInvestigated)
	if GamePlayer:getPlayerID() ~= self.m_ownerId then
		self.m_isInvestigated = isInvestigated
	end
end

function This:getInvestigateInfo()
	return self.m_investInfo
end

function This:getInvestRoleTroop(roleId)
    local armyList = {}
	for _, army in ipairs(self.m_investInfo.troopList) do
        if army.m_ownerId == roleId then
            table.insert(armyList, army)
        end
    end
    return armyList
end

function This:setInvestigateInfo(info)
	local forceNum = 0
	local totalHp = 0
	local troopNum = 0
	local tmp = {}
	self.m_investInfo.troopList = {}

	local function parse(armyList)
		for i , troop in ipairs(armyList) do
			if not tmp[troop.owner_id] then
				tmp[troop.owner_id] = true
				forceNum = forceNum + 1
			end
			totalHp = totalHp + troop.cur_hp
			troopNum = troopNum + 1
			local tp = Troop:create(troop)
			table.insert(self.m_investInfo.troopList,tp)
		end		
	end

	parse(info.army_list)
	if info.guard_list then
		for _, guard in ipairs(info.guard_list) do
            if guard.army_list then
                parse(guard.army_list)
            end
        end
	end

	self.m_investInfo.forceNum = forceNum
	self.m_investInfo.totalHp = totalHp
	self.m_investInfo.troopCount = troopNum
	self.m_investInfo.reserveHp = info.reserve_force_num
	self.m_investInfo.time = 0
	self.m_investInfo.ownerId = info.owner_id
	self.m_isInvestigated = true
end

function This:isMy()
	return GamePlayer:getPlayerID() == self.m_ownerId
end

function This:isEmptyCity()
	return self.m_isEmptyCity
end

function This:isOur()
	if self:isMy() then
		return true
	end
	local role = WorldMapManager:getRole(self.m_ownerId)
	if role and role:getRelationWithMe() > 0 then
		return true
	end
	return false
end

function This:addBatchBuild(infoList)
	local buildList = {}
	for i, info in ipairs(infoList) do
		local build = Building:create(info,self)
		self.m_buildings[info.pos] = build
		table.insert(buildList,build)
	end
	WorldMapManager:onAddBuild(self,buildList)
end

function This:addBuild(info)
	if not self.m_buildings[info.pos]  then
		local build = Building:create(info,self)
		self.m_buildings[info.pos] = build
		WorldMapManager:onAddBuild(self,{build})
		return build
	else
		assert(false)
	end
end

function This:removeBuild(pos)
	local build = self.m_buildings[pos]
	if build then
		self.m_buildings[pos] = nil
		WorldMapManager:onRemoveBuild(self,build)
	else
		assert(false)
	end
end

function This:addTroop(info)
	info.city_id = self.m_id
	local troop = Troop:create(info)
	self.m_troops[info.id] = troop

	local ownerId = info.owner_id
	if ownerId == GamePlayer:getPlayerID() then
		local hero = HeroManager:getHero(troop:getHeroId())
		hero:setState(HeroState.kInGarrison)
	end
	return troop
end

function This:removeTroop(id)
	local troop = self.m_troops[id]
	if troop then
		local ownerId = troop:getOwnerId()
		self.m_troops[id] = nil
		if ownerId == GamePlayer:getPlayerID() then
			local hero = HeroManager:getHero(troop:getHeroId())
			hero:setState(HeroState.kInFree)
		end
		return troop
	end
end

function This:updateTroop(info)
	info.city_id = self.m_id
	local troop = self.m_troops[info.id]
	if troop then
		troop:updateInfo(info)
	end
end

function This:updateTroopList(roleId,infoList)
	local roleTroop = self.m_troops
	local updateList = {}

	for i , info in pairs(infoList) do
		if info.cur_hp <= 0 then
			self:removeTroop(info.id)
			infoList[i] = nil
		end
	end

	for i , info in pairs(infoList) do
		if self.m_troops[info.id] then
			info.city_id = self.m_id
			self.m_troops[info.id]:updateInfo(info)
		else
			self:addTroop(info)
		end
	end
end

function This:getMyAttackArmy()
	local result = {}
	for id , army in pairs(self.m_attackArmys) do
		print(string.format("攻击部队:%s,拥有者ID:%s,我的ID:%s",GameUtil:int64ToString(army:getId()),
			GameUtil:int64ToString(army:getOwnerId()),GameUtil:int64ToString(GamePlayer:getPlayerID())))
		if army:isMy() then
			table.insert(result,army)
		end
	end
	return result
end

function This:getMyDefenceArmy()
	local result = {}
	for id , army in pairs(self.m_defenceArmys) do
		if army:isMy() then
			table.insert(result,army)
		end
	end
	return result
end

function This:getDefenceQueue()
	return self.m_defenceQueue
end

function This:addDefenceArmy(info)
	local army = MarchArmy:create(info)
	self.m_defenceArmys[info.id] = army
	table.insert(self.m_defenceQueue,army)
end

function This:removeDefenceArmy(armyId)
	local army = self.m_defenceArmys[armyId]
	if army then
		for i , ay in ipairs(self.m_defenceQueue) do
			if army == ay then
				table.remove(self.m_defenceQueue,i)
				break
			end
		end
		self.m_defenceArmys[armyId] = nil
	end
end

function This:updateDefenceArmy(info)
	local army = self.m_defenceArmys[info.id]
	if army then
		army:updateInfo(info)
	end	
end

function This:getAttackQueue()
	return self.m_attackQueue
end

function This:addAttackArmy(info)
	local role
	if info.owner_id and info.owner_id > 0 then
		role = WorldMapManager:getRole(info.owner_id)
	end

	if not role then return end

	local army = MarchArmy:create(info)
	self.m_attackArmys[info.id] = army
	table.insert(self.m_attackQueue,army)
end

function This:removeAttackArmy(armyId)
	local army = self.m_attackArmys[armyId]
	if army then
		for i , ay in ipairs(self.m_attackQueue) do
			if army == ay then
				table.remove(self.m_attackQueue,i)
				break
			end
		end
		self.m_attackArmys[armyId] = nil
	end
end

function This:updateAttackArmy(armyId)
	local army = self.m_attackArmys[armyId]
	if army then
		army:updateInfo(info)
	end
end

function This:getRoleCount()
	local count = 0
	local ownerList = {}
	--[[if next(self.m_troops) then
		count = count + 1
		ownerList[GamePlayer:getPlayerID()] = true
	end]]
	
	for i , troop in pairs(self.m_troops) do
		if not ownerList[troop:getOwnerId()] then
			count = count + 1
			ownerList[troop:getOwnerId()] = true
		end
	end

	for id , army in pairs(self.m_defenceArmys) do
		local ownerId = army:getOwnerId()
		if not ownerList[ownerId] then
			count = count + 1
			ownerList[ownerId] = true
		end
	end
	for id , army in pairs(self.m_attackArmys) do
		local ownerId = army:getOwnerId()
		if not ownerList[ownerId] then
			count = count + 1
			ownerList[ownerId] = true
		end
	end
	return count
end

function This:getTroopCount()
	local count = table.count(self.m_troops)
	for ownerId , army in pairs(self.m_defenceQueue) do
		count = count + table.count(army:getAllTroop())
	end
	for ownerId , army in pairs(self.m_attackQueue) do
		count = count + table.count(army:getAllTroop())
	end
	return count
end

function This:updateTroopPos(troopList)
	for i , info in pairs(troopList) do
		local troop = self:getTroop(info.id)
		troop:setPos(info.pos)
	end
end

function This:updateState(state)
	if self.m_config.type == CityType.kCity and GamePlayer:getServerTime() <= WorldMapManager:getNextFightTime() and WorldMapManager:getMapType() ~= 0 then
		self.m_state = CityState.kProtected
		--print(string.format("城市%d状态更新为保护状态",self.m_id))
	elseif state then
		self.m_state = state
		--print(string.format("城市%d状态更新为%d",self.m_id,state))
	elseif self.m_state ~= CityState.kInFighting then
		self.m_state = CityState.kNormal
		--print(string.format("城市%d状态更新为正常",self.m_id))
	end
end

function This:getLeftMinTimeStg()
	return self.m_minStg
end

function This:updateStgState(stateList,isUpdateDRGH)
	local tmp = {}
	for i , info in ipairs(stateList) do
		tmp[info.state] = true
	end

	if self.m_state ~= CityState.kProtected and tmp[JiCeState.kKCJ] then
		self.m_state = CityState.kProtected
		self.m_isEmptyCity = true
	elseif self.m_state == CityState.kProtected and not tmp[JiCeState.kKCJ] then
		self:updateState(CityState.kNormal)
		self.m_isEmptyCity = false
	end
	if isUpdateDRGH then
		if tmp[JiCeState.kDRGH] then
			self.m_isAdvanceInvestigated = true
		else
			self.m_isAdvanceInvestigated = false
		end
	end
	self.m_isImprisoned = tmp[JiCeState.kQMBG] or false

	
	local tmp = {}
	for i , info in ipairs(self.m_stgList) do
		tmp[info.state] = info
	end

	local minTime = 0xffffff
	for i , info in ipairs(stateList) do
		local stg = WorldMapManager:getStgItem(JiCeToState[info.state])
		if stg then
			stg:addCity(self.m_id,info.start_time,info.end_time)
			local t = stg:getLeftHoldCD(self.m_id)
			if t > 0 and minTime > t then
				minTime = t
				self.m_minStg = stg
			end
		end
		tmp[info.state] = nil
	end

	for i , info in pairs(tmp) do
		local stg = WorldMapManager:getStgItem(JiCeToState[info.state])
		if stg then
			stg:removeCity(self:getCityId())
		end
	end

	self.m_stgList = stateList
	--print(string.format("城市:%d --> 空城计:%s,奇门八卦:%s",self.m_id,tostring(self.m_state == CityState.kProtected),tostring(self.m_isImprisoned)))
end

function This:getTreasureMapUidList()
	local lowList = {}
	local itemId = 50000 + TreasureMapType.kLow * 1000 + self:getId() 
	local itemList = DepotManager:getItemList(itemId)
	for i , item in ipairs(itemList) do
		table.insert(lowList,{uid = item:getUid(), id = item:getId()})
	end

	local highList = {}
	itemId = 50000 + TreasureMapType.kHigh * 1000 + self:getId();
	itemList = DepotManager:getItemList(itemId)
	for i , item in ipairs(itemList) do
		table.insert(highList,{uid = item:getUid(), id = item:getId()})
	end

	return lowList, highList
end

function This:addTreasureBox(info)
	local box = self.m_treasuerBoxs[info.treasure_id]
	if not box then
		box = TreasureBox:create({ id = info.treasure_id, cityId = self:getCityId(), pos = info.pos})
		self.m_treasuerBoxs[info.treasure_id] = box
	end
	box:setPos(info.pos)
	box:setTime(info.start_time,info.end_time)
end

function This:removeTreasureBox(id)
	local box = self.m_treasuerBoxs[id]
	box:endUpdate()
	self.m_treasuerBoxs[id] = nil
end

function This:getTreasureBox(id)
	return self.m_treasuerBoxs[id]
end

function This:hasTreasureBox()
	return next(self.m_treasuerBoxs) ~= nil
end

function This:getAllTreasureBox()
	return self.m_treasuerBoxs
end

function This:clear()
	for i , build in pairs(self.m_buildings) do
		build:clear()
	end
end

--endregion
