-- @Author: zheng
-- @Date:   2019-05-31 15:02:47
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-06-20 10:36:07
-- @File Name: CityHanXianDiView.lua

local This , super = class(...,"World.CityView")
local WorldMapManager = require("World.WorldMapManager")
local Vec = require("Vec")
local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local EventDispatcher = require("EventDispatcher")

function This:init(id)
	self.m_scene = g_scene
	self.m_csNode = cc.Node:createFromCSFile("WarScene/NodeUI/CityKing.csb")
	self:addChild(self.m_csNode)
	self.m_city = WorldMapManager:getCity(id)
	self.m_csNode.kingnode.name:setString(self.m_city:getName())
	self.m_csNode.choose:setVisible(false)

	self.m_skeletes = {}
	self.m_escape = 0
    self.m_count =0
    self:scheduleUpdate()
	self:refreshRebelsNodes()
end

function This:update(dt)
    self.m_escape = self.m_escape + dt
    if self.m_escape >= 1 then
        self.m_escape = self.m_escape - 1
        local hanxiandiVisible = WorldMapManager:isHanXianDiAppearTime()
        local isHaveHanXianDi = self.m_city:isHide()
        if isHaveHanXianDi or hanxiandiVisible then
        	self:unscheduleUpdate()
        	self:refresh()
        	self:refreshHanXianDiApearTime()
        	if not isHaveHanXianDi then
        		WorldMapManager:notifyObservers("onCityAppear",self.m_city)
        	end
        end
        if not hanxiandiVisible then
        	self:refreshHanXianDiApearTime()
        end
    end
end

function This:checkOpenHanXianDi()
	local MatchingRoomConfig = require("Metadatas.MatchingRoomConfig")
	local mapType = WorldMapManager:getMapType()
	local configItem = MatchingRoomConfig[mapType]
	if configItem and configItem.type[1]==1 then
		return true
	end
	return false
end

function This:initTouchEvent()
	self.m_csNode.kingnode.kingEffect.click:setTouch({
		onEnded = function()
			if self.m_touchHandler then
				self.m_touchHandler(self)
			end
		end,
		isDynamic = true,
		isSwallow = false,
		isAlpha = false
	})
	self.m_csNode.jitan:setTouch({
		onEnded = function()
			if not self:checkOpenHanXianDi() and not self.m_city:isHide() and not WorldMapManager:isHanXianDiAppearTime() then
				g_scene:showTips(Serialize["world_CityHanXianDi_1"])
			end
		end,
		isDynamic = true,
		isSwallow = false,
		isAlpha = false,
		isScale = false,
	})
end

function This:onEnter()
	cc.Node.onEnter(self)
	self:initTouchEvent()
	self:refresh()
end

function This:onExit()
	cc.Node.onExit(self)
	self:unscheduleUpdate()
end

function This:showSelectAnimation(isShow)
	if isShow and not self.m_selectAnimation then
		local str = ""
		if self.m_city:isOur() then
			str = "WarScene/jiantou1.png"
		else
			str = "WarScene/jiantou2.png"
		end
		local sprite = cc.Sprite:create(str)
		local initPos = Vec(0,50)
		local upPos = Vec(initPos.x,initPos.y + 10)
		sprite:setPosition(initPos.x,initPos.y)
		sprite:runAction(cc.RepeatForever(cc.Sequence(cc.MoveTo(0.5,upPos.x,upPos.y,"SineOut"),cc.MoveTo(0.5,initPos.x,initPos.y,"SineOut"))))
		self.m_csNode.kingnode.kingEffect:addChild(sprite)
		self.m_selectAnimation = sprite
	else
		self.m_selectAnimation:removeFromParent()
		self.m_selectAnimation = nil
	end
end

function This:setSelectHandler(handler)
	self.m_touchHandler = handler
end

function This:setSelect(isSelect)
	if isSelect and not self.m_menu then
		if self.m_city:isMy() then
			self:showMyCityMenu()
        elseif self.m_city:isOur() then
            self:showAllyCityMenu()
		else
			self:showOtherCityMenu()
		end
	elseif not isSelect and self.m_menu then
		self.m_menu:removeFromParent()
		self.m_menu = nil
		self.m_scene:addMenuAtMapLayer(nil)
		self:refreshStgHoldTime()
		EventDispatcher:dispatcher("city_nemu_close")
	end
	self.m_csNode.choose:setVisible(isSelect)
end

function This:refreshRebelsNodes()
	
end

--侦查的按钮
function This:onInvestigateBtnClicked()
	g_scene:openDialog("World.RebelsHanXianDiPanel",self.m_city)
end

function This:refresh()
	if not self.m_city:isHide() then
		self:setVisible(true)
		-- self:refreshRebelsNodes()
		self:refreshFightingState()
	else
		self:setVisible(false)
	end
	local csNode = self.m_csNode
	local kingNode = csNode.kingnode
	local hanxiandiVisible = WorldMapManager:isHanXianDiAppearTime()
	if WorldMapManager:getMapType() == 0 then
		self:setVisible(false)
	else--if hanxiandiVisible then
		local kingEffect = kingNode.kingEffect
		if hanxiandiVisible then
			self:refreshStgHoldTime()
		else
			self:refreshHanXianDiApearTime()
		end
		self:addHanXianDiEffect(kingEffect,hanxiandiVisible)
		local name = self.m_city:getName()
		local roleWin = WorldMapManager:getPlayerWinHanXianDi()
		if roleWin then
			kingEffect.effect:removeAllChildren()
			name = WorldMapManager:getPlayerWinHanXianDiName()
			self:setSelect(false)
		else
			
		end
		kingNode.name:setString(name)
		kingEffect:setVisible(hanxiandiVisible and not roleWin)
	end
	kingNode:setVisible(hanxiandiVisible)
end

function This:addHanXianDiEffect(kingEffect,isAdd)
	local function createAnim( parent,anmStr )
        local skeletonNode = cc.SkeletonNode:createCharacter(anmStr)
        skeletonNode:scheduleUpdate()
        skeletonNode:playAnimation("animation", true)
        parent:addChild(skeletonNode)
        return skeletonNode
    end
	self.m_hanxaindiEffect = createAnim(kingEffect,"MoDi2")
end

function This:refreshHanXianDiApearTime()
	local timeBg = self.m_csNode.timebg
	local leftTime = WorldMapManager:getHanXianDiApearTime() - GamePlayer:getServerTime()
	if leftTime>0 then
		local str = GameUtil:getFormatDtimeStr(leftTime)
		timeBg.time:setString(str..Serialize[64906])
	end
	timeBg:setVisible(leftTime>0 and self:checkOpenHanXianDi())
end

function This:refreshFightingState()
	local state = self.m_city:getState()
	if state == CityState.kInFighting and not self.m_fightEffect then
		self.m_fireEffect =  GameUtil:createEffect("Sha",true,nil,nil,false)
		self:addChild(self.m_fireEffect)
		self.m_fightEffect =  GameUtil:createEffect("Engagement",true,nil,nil,false)
		self:addChild(self.m_fightEffect)
		print(string.format("刷新城市%d战斗状态,添加特效",self.m_city:getCityId()))
	elseif state ~= CityState.kInFighting and self.m_fightEffect then
		self.m_fireEffect:removeFromParent()
		self.m_fireEffect = nil
		self.m_fightEffect:removeFromParent()
		self.m_fightEffect = nil
		print(string.format("刷新城市%d战斗状态,删除特效",self.m_city:getCityId()))
	end
end

--endregion
