--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local TouchListener = require("cc.TouchListener")
local BuildView = require("World.BuildView")
local WorldMapManager = require("World.WorldMapManager")
local WorldUtil = require("World.WorldUtil")
local GameUtil = require("Util.GameUtil")
local Timer = require("Timer")
local PlanShopUtil = require("PlanShop.PlanShopUtil")
local PlanShopManager = require("PlanShop.PlanShopManager") 
local ResourceConfig = require("Metadatas.ResourceConfig")
local SlideScaleContaner = require("Action.SlideScaleContaner")
local EventDispatcher = require("EventDispatcher")
local ItemConfig = require("Metadatas.ItemConfig")
local TreasureBoxView = require("World.TreasureBoxView")
local DepotManager = require("Depot.DepotManager")

local BuildCoordinate =

{
  [1] = {x = 313, y = 476},
  [2] = {x = 440, y = 532},
  [3] = {x = 361, y = 385},
  [4] = {x = 467, y = 437},
  [5] = {x = 576, y = 488},
  [6] = {x = 595, y = 279},
  [7] = {x = 649, y = 355},
  [8] = {x = 759, y = 403},
  [9] = {x = 765, y = 290},
  [10] ={x =872, y = 345},
  [11] ={x = 907, y = 585},
}

local BuildZorder = 
{
	[1] = 9,
	[2] = 7,
	[3] = 5,
	[4] = 3,
	[5] = 10,
	[6] = 8,
	[7] = 6,
	[8] = 4,
	[9] = 2,
	[10] = 1,
	[11] = 11
}

local BuildMainOffsetPoint = {x = 50, y =120}

function This:init(city)
	self.m_city = city
	self.m_isInit = false
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/BulidBgLayer.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setPosition(-180, -185)

	local build = self.m_city:getBuild(BuildPos.kMainBuild)
	self.m_csNode.des:setString(build:getName())
	self.m_isCreating = false
	self.m_builds = {}
	self.m_treasureBoxList = {}
	self:initBuilds()
	self:refreshTreasureBox()
	self:refreshTreasureMap()
	self:refresh()
	self.m_timer = Timer:create(self)
end

function This:onEnter()
	super.onEnter(self)
	self:initTouchEvent()
	WorldMapManager:addObserver(self)
    PlanShopManager:addObserver(self)
	local build = self.m_city:getBuild(BuildPos.kMainBuild)
	build:setView(self)
	self:scheduleUpdate()
end

function This:onExit()
	super.onExit(self)
	WorldMapManager:removeObserver(self)
    PlanShopManager:removeObserver(self)
	local build = self.m_city:getBuild(BuildPos.kMainBuild)
	build:setView(nil)
end

function This:getModel()
	return self.m_city
end

function This:initAction()
	if not self.m_isInit then
	    local width , height = self.m_csNode.Main:getContentSize()
	    width , height = width *0.55 , height *0.55
	    self.m_csNode:setContentSize(width , height)
	    self.m_slideScaleContaner = SlideScaleContaner:create({
	        node = self.m_csNode,
	        maxWidth =  width,
	        maxHeight = height,
	        minWidth = cc.visible.width,
	        minHeight = cc.visible.height,
	    })
	    self.m_isInit = true
	end
	self:moveToCenter()
end

function This:initTouchEvent()	
	for i = 1 , BuildPos.kMainBuild do
		self.m_csNode["click"..i]:setTouch({
			onEnded = function()
				if not self.m_boxIsOpening then
					self:onTouchBuild(i)
				end
			end,
			isAlpha = false,
			isScale = false,
			isDynamic = true,
		})
	end

	self.m_csNode.use.click_use:setTouch({
		onEnded = function()
			if #self.m_lowList > 0 then
				local uid = self.m_lowList[1].uid
				local id = self.m_lowList[1].id
				g_scene:openDialog("World.TreasureMapUsePanel",{city = self.m_city,itemUid = uid, itemId = id})
			end
		end,
		isScale = false,
		isAlpha = false,
		isDynamic = true,
	})

	self.m_csNode.use1.click_use:setTouch({
		onEnded = function()
			if #self.m_highList > 0 then
				local uid = self.m_highList[1].uid
				local id = self.m_highList[1].id
				g_scene:openDialog("World.TreasureMapUsePanel",{city = self.m_city,itemUid = uid, itemId = id})
			end
		end,
		isScale = false,
		isAlpha = false,
		isDynamic = true,
	})

    self:initAction()
end

function This:initBuilds()
	for i , pos in pairs(BuildPos) do
		local build = self.m_city:getBuild(pos)
		if build and pos ~= BuildPos.kMainBuild then
			local view = BuildView:create(build)
			view:setPosition(BuildCoordinate[pos].x, BuildCoordinate[pos].y)
			self.m_csNode:addChild(view)
			view:setLocalZOrder(BuildZorder[pos])
			self.m_builds[pos] = view
		end
	end
end

function This:getTreasureBoxView(box)
	local view = TreasureBoxView:create(box)
	view:setOpenHandler(function(isOpening)
		self.m_boxIsOpening = isOpening
	end)
	return view
end

function This:refreshTreasureBox()
	local boxes = self.m_city:getAllTreasureBox()
	local posList = {}
	for i = 1 , 10 do
		posList[i] = i
	end

	for i , box in pairs(boxes) do
		local pos = box:getPos()
		if self.m_treasureBoxList[pos] then
			self.m_treasureBoxList[pos]:setModel(box)
		else
			local view = self:getTreasureBoxView(box)
			self.m_csNode["box"..pos]:addChild(view)
			self.m_treasureBoxList[pos] = view
		end
		posList[pos] = nil
	end
	for i , pos in pairs(posList) do
		if self.m_treasureBoxList[pos] then
			self.m_treasureBoxList[pos]:removeFromParent()
			self.m_treasureBoxList[pos] = nil
		end
	end
end

function This:refreshTreasureMap()
	self.m_lowList , self.m_highList = self.m_city:getTreasureMapUidList()
	if #self.m_lowList > 0 then
		if not self.m_lowEffect then
			self.m_lowEffect = GameUtil:createEffect("LiangDian",true)
			self.m_csNode.use:addChild(self.m_lowEffect)
		end
		if not self.m_lowHammerEffect then
			self.m_lowHammerEffect = GameUtil:createEffect("TreasureDig",true)
			self.m_csNode.use:addChild(self.m_lowHammerEffect)
		end
		self.m_csNode.use.baotubg:setVisible(true)
		self.m_csNode.use.baotutime:setVisible(true)
	elseif #self.m_lowList <= 0 then
		if self.m_lowEffect then
			self.m_lowEffect:removeFromParent()
			self.m_lowEffect = nil
		end
		if self.m_lowHammerEffect then
			self.m_lowHammerEffect:removeFromParent()
			self.m_lowHammerEffect = nil
		end
		self.m_csNode.use.baotubg:setVisible(false)
		self.m_csNode.use.baotutime:setVisible(false)
	end

	if #self.m_highList > 0 then
		if not self.m_highEffect then
			self.m_highEffect = GameUtil:createEffect("TiXing",true)
			self.m_csNode.use1:addChild(self.m_highEffect) 
		end
		if not self.m_highHammerEffect then
			self.m_highHammerEffect = GameUtil:createEffect("TreasureDig",true)
			self.m_csNode.use1:addChild(self.m_highHammerEffect)
		end
		self.m_csNode.use1.baotubg:setVisible(true)
		self.m_csNode.use1.baotutime:setVisible(true)
	elseif #self.m_highList <= 0 then
		if self.m_highEffect then
			self.m_highEffect:removeFromParent()
			self.m_highEffect = nil
		end
		if self.m_highHammerEffect then
			self.m_highHammerEffect:removeFromParent()
			self.m_highHammerEffect = nil
		end
		self.m_csNode.use1.baotubg:setVisible(false)
		self.m_csNode.use1.baotutime:setVisible(false)
	end
end

function This:moveToCenter()
	local w , h = self.m_csNode:getContentSize()
	self.m_csNode:setPosition((cc.visible.width - w) / 2, (cc.visible.height - h)/2 )
end

function This:refreshLeftTime(leftTime)
	self.m_csNode.leveluptime:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
end

function This:refresh()
	local build = self.m_city:getBuild(BuildPos.kMainBuild)
	self.m_csNode.level:setString(Serialize[64048]..build:getLevel())
	self.m_csNode.leveluptime:setVisible(build:getState() == BuildState.kUpgrading)
	if build:getState() == BuildState.kUpgrading and not self.m_effect then
		self.m_effect = GameUtil:createEffect("BuildImpact",true)
		self.m_effect:setPosition(BuildCoordinate[BuildPos.kMainBuild].x + BuildMainOffsetPoint.x, BuildCoordinate[BuildPos.kMainBuild].y + BuildMainOffsetPoint.y)
		self.m_csNode:addChild(self.m_effect)
	elseif build:getState() ~= BuildState.kUpgrading and self.m_effect then
		self.m_effect:removeFromParent()
		self.m_effect = nil
	end
end

function This:onCityLvChanged(city)
	if self.m_city == city then
		self:refresh()
	end
end

function This:onAddBuild(city,buildList)
	if self.m_city ~= city then
		return
	end
	self.m_isCreating = true
	table.sort(buildList,function(a,b)
		return a:getPos() < b:getPos()
	end)

	for i , build in ipairs(buildList) do
		self.m_timer:setTimer(i * 0.1,function()
			local pos = build:getPos()
			local view = BuildView:create(build)
			view:setPosition(BuildCoordinate[pos].x,BuildCoordinate[pos].y)
			self.m_csNode:addChild(view)
			view:setLocalZOrder(BuildZorder[pos])
			self.m_builds[pos] = view	
			if i >= #buildList then
				self.m_isCreating = false
			end		
			EventDispatcher:dispatcher("city_create_build", build)
		end)
	end
end

function This:onRemoveBuild(city,build)
	if self.m_city ~= city then
		return 
	end
	local pos = build:getPos()
	local view = self.m_builds[pos]
	if pos ~= BuildPos.kMainBuild and view then
		view:removeFromParent()
	end
	self.m_builds[pos] = nil
end

function This:onBuildUpgrade(city,build)
	if self.m_city == city then
		local pos = build:getPos()
		local effect = GameUtil:createEffect("ActionDone11", false, false)
		effect:setPosition(BuildCoordinate[pos].x, BuildCoordinate[pos].y)
		self.m_csNode:addChild(effect)
	end
end

function This:createBuild(pos,buildType,isBatch)
	if not isBatch then
		local posList = {}
		local typeList = {}
		table.insert(posList,pos)
		table.insert(typeList,buildType)
		WorldMapManager:onCreateBuildRequest(self.m_city:getCityId(),posList,typeList)	
	else
		local posList = {}
		local typeList = {}
		local isNotEnough = false
		local copper = WorldMapManager:getCopper()
		local cost = ResourceConfig[1].cost
		local offsetCost = 0
		for p = 1 , BuildPos.kMainBuild do
			local build = self.m_city:getBuild(p)
			if not build and p ~= BuildPos.kMainBuild then
				if copper >= cost then
					table.insert(posList,p)
					table.insert(typeList,buildType)
					copper = copper - cost
				else
					isNotEnough = true
					offsetCost = offsetCost + cost
				end
			end
		end
		if #posList > 0 then
			WorldMapManager:onCreateBuildRequest(self.m_city:getCityId(),posList,typeList)	
		end

		if isNotEnough then
			PlanShopUtil:needUsePlan( JiCeType.kCaiYuanGunGun, 0, offsetCost )
		end
	end
end

function This:upgradeBuild(pos,isBatch)
	if not isBatch then
		local posList = {}
		local build = self.m_city:getBuild(pos)
		if build:getLevel() < build:getMaxLv() then
			table.insert(posList,pos)
		end
		if #posList > 0 then
			WorldMapManager:onUpgradeBuildRequest(self.m_city:getCityId(),posList)
		end
		if build:getLevel() >= build:getMaxLv() then
			g_scene:handleErrorCode("to_max_level")
		end
	else
		local b = self.m_city:getBuild(pos)
		local buildType = b:getType()
		local posList = {}
		local buildList = {}

		for p = 1 , BuildPos.kMainBuild do
			local build = self.m_city:getBuild(p)
			if build and build:getType() == buildType and build:isCanUpgrade() then
				table.insert(buildList,build)
			end
		end

		table.sort(buildList,function(a,b)
			return a:getLevel() < b:getLevel()
		end)

		local isNotEnough = false
		local isToMaxLv = buildList and #buildList > 0
		local copper = WorldMapManager:getCopper()
		local offsetCost = 0
		for i , build in pairs(buildList) do
			local cost = build:getLvUpCost()
			if copper >= cost then
				table.insert(posList,build:getPos())
				copper = copper - cost
			else
				isNotEnough = true
				offsetCost = offsetCost + cost
			end

			if isToMaxLv and build:getLevel() < build:getMaxLv() then
				isToMaxLv = false
			end
		end

		if #posList > 0 then
			WorldMapManager:onUpgradeBuildRequest(self.m_city:getCityId(),posList)
		end
		if isNotEnough then
			PlanShopUtil:needUsePlan( JiCeType.kCaiYuanGunGun, 0, offsetCost )
		elseif isToMaxLv then
			g_scene:handleErrorCode("to_max_level")
		end
	end	
end

function This:onTouchBuild(pos)
	if self.m_isCreating then
		return
	end
	if pos ~= BuildPos.kMainBuild and self.m_builds[pos] == nil then
		local panel = g_scene:openDialog("World.BuildCreatePanel")
		panel:setCreateHandle(function(buildType,isBatch)
			self:createBuild(pos,buildType,isBatch)
		end)
	else
		local build = self.m_city:getBuild(pos)
		if build:getState() ~= BuildState.kUpgrading then
			local panel = g_scene:openDialog("World.BuildLvUpPanel",build)
			panel:setLvUpHandler(function(isBatch)
				self:upgradeBuild(pos,isBatch)
			end)
			panel:setRemoveHandler(function()
				WorldMapManager:onRemoveBuildRequest(self.m_city:getCityId(),pos)
			end)
		else
            local planId = build:getPos() == BuildPos.kMainBuild and JiCeType.kLuBanZaiShi or JiCeType.kNengGongQiaoJiang
			PlanShopUtil:usePlan({plan=planId, num = 1, argId=build:getCity():getCityId(), object=self, callback=self.onUsePlan})
		end
	end
end

function This:update(dt)
	self.m_timer:update(dt)
    if self.m_isInit then
        self.m_slideScaleContaner:update(dt)
    end
end

function This:onUsePlan(msg)
	local planId = msg.planId
    if planId == JiCeType.kNengGongQiaoJiang then
        local buildings = self.m_city:getAllBuild()
        local pos_list = {}
        for _, building in pairs(buildings) do
            if building:getState() == BuildState.kUpgrading then
                local pos = building:getPos()
                if building:getPos() ~= BuildPos.kMainBuild then
                    table.insert(pos_list, pos)
                end
            end
        end
        self.m_city:buildUpgradeImmediately(pos_list)
    elseif planId == JiCeType.kLuBanZaiShi then
        if self.m_city:getBuild(BuildPos.kMainBuild):getState() ~= BuildState.kUpgrading then
            return
        end
        local cityId = msg.arg_list[1]
        local pos_list = {BuildPos.kMainBuild}
        self.m_city:buildUpgradeImmediately(pos_list)
    end
end

function This:onBuildAddValue()
	self:refresh()
end

function This:onUseTreasureMap()
	self:refreshTreasureMap()
end

function This:onOpenTreasureBox()
	self:refreshTreasureBox()
end

function This:onCityInfoChanged(city)
	if city:getId() == self.m_city:getId() then
		self:refreshTreasureBox()
	end
end


--endregion
