--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This = class(...)
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local Troop = require("World.Troop")
local HeroManager = require("Hero.HeroManager")
local WorldMapManager = require("World.WorldMapManager")

TransType = 
{
	kTransHp = 1,  --运输兵力
	kGoToBattle = 2, --出征
	kGarrison = 3, --驻防
}

function This:create(...)
	local a = {}
	setmetatable(a,self)
	a:init(...)
	return a
end

function This:init(info)
	self.m_id = info.id
	self.m_ownerId = info.owner_id

	self.m_troops = {}
	local count  = 0
	for i , troop in pairs(info.army_list) do
		self:addTroop(troop)
		count = count + 1
	end

	if info.to_city > 0 then
		local city = WorldMapManager:getCity(info.to_city)
		local cityOwner = WorldMapManager:getRole(city:getOwnerId())
		local armyOwner = WorldMapManager:getRole(self.m_ownerId)
		if not cityOwner then
			self.m_transType = TransType.kGoToBattle
		elseif cityOwner == armyOwner then
			self.m_transType = TransType.kTransHp
		elseif cityOwner:getRelationWithOther(armyOwner) > 0 then
			self.m_transType = TransType.kGarrison
		else
			self.m_transType = TransType.kGoToBattle
		end
	end

	self.m_totalHp = info.force_num
	self.m_fromCity = info.from_city
	self.m_toCity = info.to_city
	self.m_startTime = info.start_time
	self.m_endTime = info.end_time
	self.m_firstEndTime = info.end_time
	self.m_speedUpTime = info.speed_up_time

	self.m_observers = {}
	if self.m_endTime > GamePlayer:getServerTime() then
		self:updateTripStart()
	end
end

function This:updateData()
	if self.m_toCity > 0 then
		local city = WorldMapManager:getCity(self.m_toCity)
		local cityOwner = WorldMapManager:getRole(city:getOwnerId())
		local armyOwner = WorldMapManager:getRole(self.m_ownerId)
		if not cityOwner then
			self.m_transType = TransType.kGoToBattle
		elseif cityOwner == armyOwner then
			self.m_transType = TransType.kTransHp
		elseif cityOwner:getRelationWithOther(armyOwner) > 0 then
			self.m_transType = TransType.kGarrison
		else
			self.m_transType = TransType.kGoToBattle
		end
	end
end

function This:addObserver(o)
	for i , observer in pairs(self.m_observers) do
		if observer == o then
			return 
		end
	end
	table.insert(self.m_observers,o)
end

function This:removeObserver(o)
	for i , observer in pairs(self.m_observers) do
		if observer == o then
			table.remove(self.m_observers,i)
			return
		end
	end
end

function This:notifyObservers(funcName,...)
	for i, observer in pairs(self.m_observers) do
		if observer[funcName] then
			observer[funcName](observer,...)
		end
	end
end

function This:addTroop(info)
	local troop = Troop:create(info)
	self.m_troops[info.id] = troop
end 

function This:removeTroop(id)
	self.m_troops[id] = nil
end

function This:getId()
	return self.m_id
end

function This:getTroop(id)
	return self.m_troops[id]
end

function This:getAllTroop()
	return self.m_troops
end

function This:getTroopList()
	local result = {}
	for i , troop in pairs(self.m_troops) do
		table.insert(result,troop)
	end
	table.sort(result,function(a,b)
		return a:getPos() < b:getPos()
	end)
	return result
end

function This:getStartTime()
	return self.m_startTime
end

function This:getOwnerId()
	return self.m_ownerId
end

function This:getTotalHp()
	return self.m_totalHp
end

function This:getFromCity()
	return self.m_fromCity
end

function This:getToCity()
	return self.m_toCity
end

function This:isVisible()
	if GamePlayer:getPlayerID() == self.m_ownerId then
		return true
	end

	local role = WorldMapManager:getRole(self.m_ownerId)
	if not role then
		return false
	end

	local city = WorldMapManager:getCity(self.m_toCity)
	if not city then
		return false
	end

	if role:getRelationWithMe() > 0 then
		return true
	end

	return self.m_transType == TransType.kGoToBattle
end

function This:isMy()
	return self.m_ownerId == GamePlayer:getPlayerID()
end

function This:isOur()
	if self.m_ownerId == GamePlayer:getPlayerID() then
		return true
	else
		local role = WorldMapManager:getRole(self.m_ownerId)
		if role then
			return role:getRelationWithMe() > 0
		end
	end
end

function This:getEndTime()
	return self.m_endTime
end

function This:getTransportType()
	return self.m_transType
end

function This:getSpeedUpTime()
	return self.m_speedUpTime
end

function This:updateTripStart()
	Application:addToUpdateList(self)
end

function This:updateTripEnd()
	Application:removeFromUpdateList(self)
end

function This:getLeftTime()
	local curTime = GamePlayer:getServerTime()
	local leftTime = self.m_endTime - curTime
	if leftTime <= 0 then
		return 0
	end
	return leftTime
end

function This:updateInfo(info)
	assert(info.id == self.m_id)
	self.m_ownerId = info.owner_id

	local updateList = {}
	for i , troop in pairs(info.army_list) do
		updateList[troop.id] = troop
	end

	for id , troop in pairs(self.m_troops) do
		if not updateList[id] then
			self.m_troops[id] = nil
		end
	end

	for id , troop in pairs(updateList) do
		if not self.m_troops[id] then
			self:addTroop(troop)
		else
			self.m_troops[id]:updateInfo(troop)
		end
	end

	self.m_totalHp = info.force_num
	self.m_fromCity = info.from_city
	self.m_toCity = info.to_city
	self.m_startTime = info.start_time
	self.m_endTime = info.end_time
	self.m_speedUpTime = info.speed_up_time
end

function This:update(dt)
	local curTime = GamePlayer:getServerTime()
	local leftTime = self.m_endTime - curTime
	local maxTime = self.m_firstEndTime - self.m_startTime
	if leftTime > 0 then
		self:notifyObservers("onUpdateTrip",leftTime,maxTime)
	else
		self:updateTripEnd()
	end
end

function This:clear()
	self:updateTripEnd()
end

--endregion
