--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"cc.Node")
local GameUtil = require("Util.GameUtil")
local MarchArmyView = require("World.MarchArmyView")
local WorldMapManager = require("World.WorldMapManager")
local PlanShopUtil = require("PlanShop.PlanShopUtil")
local PlanShopManager = require("PlanShop.PlanShopManager") 

function This:init(info)
	self.m_army = info.army
	self.m_city = info.cityInfo
	self.m_isCanSpeed = info.isCanSpeed--是否可以加速
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/MarchingListItem.csb")
	local w , h = self.m_csNode.bg:getContentSize()
	self:setCascadeOpacityEnabled(true)
	self:setContentSize(w,h)
	self:addChild(self.m_csNode)
	self:initInfo()
end

function This:initInfo()
	self.m_csNode.state:setImage(MarchFlag[self.m_army:getTransportType()])
	local fromCity = WorldMapManager:getCity(self.m_army:getFromCity())
	self.m_csNode.start:setString(fromCity:getName())
	local toCity = WorldMapManager:getCity(self.m_army:getToCity())
	self.m_csNode.target:setString(toCity:getName())
	local troopList = self.m_army:getTroopList()

	if #troopList > 0 then
		for i , troop in ipairs(troopList) do
			local quality = troop:getQuality()
			self.m_csNode["hero"..i]:setString(troop:getName())
			self.m_csNode["hero"..i]:setColor(QualityColor[quality].r,QualityColor[quality].g,QualityColor[quality].b)
			self.m_csNode["hp"..i]:setString(troop:getHp())
		end
		for i = #troopList + 1, 6 do
			self.m_csNode["hero"..i]:setVisible(false)
			self.m_csNode["hp"..i]:setVisible(false)
		end
	else
		self.m_csNode["hero1"]:setString(Serialize["world_MarchArmy_1"])
		self.m_csNode["hp1"]:setString(self.m_army:getTotalHp())
		for i = 2 , 6 do
			self.m_csNode["hero"..i]:setVisible(false)
			self.m_csNode["hp"..i]:setVisible(false)
		end
	end

	local leftTime = self.m_army:getLeftTime()
	self.m_csNode.timecost:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
	self.m_csNode.gotowar:setVisible(self.m_city ~= nil)
	self.m_csNode.accelerate:setVisible(self.m_city == nil)
	self.m_csNode.timecost:setVisible(self.m_city == nil)
	self.m_csNode.jindutiao:setVisible(self.m_city == nil)

    self:refreshUI()
end

function This:getMarchArmy()
	return self.m_army
end

function This:getCity()
	return self.m_city
end

function This:initTouchEvent()
	self.m_csNode.accelerate:setTouch({
		onEnded = function()
			self:onAddSpeed()
		end,
		isContainer = true,
        isScale = false
	})
	self.m_csNode.gotowar:setTouch({
		onEnded = function()
			self:onEnterBattle()
		end,
		isContainer = true
	})
	self.m_csNode.start:setTouch({
		onEnded = function()
			g_scene:focuseCity(self.m_army:getFromCity())
		end,
		isContainer = true
	})
	self.m_csNode.target:setTouch({
		onEnded = function()
			g_scene:focuseCity(self.m_army:getToCity())
		end,
		isContainer = true
	})
end

function This:onEnter()
	super.onEnter(self)
	self:initTouchEvent()
    PlanShopManager:addObserver(self)
	self.m_army:addObserver(self)
end

function This:onExit()
	super.onExit(self)
    PlanShopManager:removeObserver(self)
	self.m_army:removeObserver(self)
end

function This:onUpdateTrip(leftTime,maxTime)
	self.m_csNode.timecost:setString(GameUtil:formatTime(Serialize["H_M_S"],leftTime))
	self.m_csNode.jindutiao:setPercent(leftTime/maxTime*100)
end

function This:onEnterBattle()
	print("进入战场...")
	if self.m_city then
		cc.SceneManager:switchTo("World.WorldLoadingScene",nil,self.m_city:getCityId())
	end
end

function This:onAddSpeed()
	if not self.m_isCanSpeed then
		g_scene:showTips(Serialize["world_MarchArmy_2"])
		return
	end
    if self.m_army:getSpeedUpTime() < kMaxSpeedUpTime then 
    	local curTime = kMaxSpeedUpTime - self.m_army:getSpeedUpTime()
		PlanShopUtil:usePlan({plan=JiCeType.kXingJunJiaSu, num=1, argId=self.m_army:getId(), object=self, callback=self.onUsePlan,curAddSpeed =curTime })
    end
end

function This:onUsePlan(msg)
    if msg.planId == JiCeType.kXingJunJiaSu then
        self:refreshUI()
    end
end

function This:refreshUI()
    local curTime = kMaxSpeedUpTime - self.m_army:getSpeedUpTime()
    self.m_csNode.accelerate.text:setString(Serialize[64922].. curTime .."/".. kMaxSpeedUpTime .. ")")
    if curTime <= 0 or not self.m_isCanSpeed then  
        self.m_csNode.accelerate:setGrayShader()
    end
end

--endregion
