-- @Author: zheng
-- @Date:   2019-03-25 14:38:06
-- @Last Modified by:   zheng
-- @Last Modified time: 2019-03-29 11:03:41
-- @File Name: MarchPaoTuView.lua

local This , super = class(...,"NodeUI")
local MarchAppearanceConfig = require("Metadatas.MarchAppearanceConfig")

local Vec = require("Vec")

local GameUtil = require("Util.GameUtil")
local GamePlayer = require("Player.GamePlayer")
local PaoMaEffect = 
{
    [1] = {name = "zuo", isFlipX = true,rotation =270},
    [2] = {name = "xieshang", isFlipX = true,rotation =315},
    [3] = {name = "shang", isFlipX = false,rotation =0},
    [4] = {name = "xieshang", isFlipX = false,rotation =45},
    [5] = {name = "zuo", isFlipX = false,rotation =90},
    [6] = {name = "xiexia", isFlipX = false,rotation =135},
    [7] = {name = "xia", isFlipX = false,rotation =180},
    [8] = {name = "xiexia", isFlipX = true,rotation =225},
}


function This:initNode(info)
    self.m_startTime = info.startTime
    self.m_endTime = info.endTime
    self.m_fromNode = info.fromNode
    self.m_toNode = info.toNode
    self.m_configId = info.configId or 1
    local x1,y1 = self.m_fromNode:getPosition()
    local x2,y2 = self.m_toNode:getPosition()
    self:createPaoTu(x1,y1,x2,y2)
    if info.runAnim then
        self:paoMaAction(x1,y1,x2,y2)
    end
end

function This:getAnimConfig()
    return MarchAppearanceConfig[self.m_configId].house_arg
    -- return {{"PaoTu",0,0,1},{"XingJun1",70,70,1}}
end

function This:paoMaAction(x1,y1,x2,y2)
    local v = cc.Vec(x2 - x1,y2 - y1)
    local speed = 50
    local l = v:length()
    self.m_startPos = cc.Vec(x1,y1)
    self.m_endPos = cc.Vec(x2,y2)

    self:setPosition(self.m_startPos.x,self.m_startPos.y)
    self:runAction(cc.RepeatForever(cc.Sequence(
        cc.MoveTo(l/speed,x2,y2),
        cc.CallFunc(function()
            self:setPosition(self.m_startPos.x,self.m_startPos.y)
        end)
    )))

end

function This:createPaoTu(x1,y1,x2,y2)
    -- PaoMaEffectId =3
    local radio = self:getPaoMaRadio()
    local v = cc.Vec(x2 - x1, y2 - y1)
    local v1 = v * radio
    local degree = v:getDegree()
    local config,temDegree = self:getConfigWithDegree(degree)
    -- local degreeOffSet = 90
    local degreeOffSets={
        [270] = {90,-90},
        [315] = {45,-45},
        [0] = {0,0},
        -- [45] = {-45,45},
        -- [90] = {-90,90},
        [45] = {315,-315},
        [90] = {270,-270},
        [135] = {225,-225},
        [180] = {180,-180},
        [225] = {135,-135},
    }
    -- print("temDegree",temDegree)
    local function createAnim(animConfig,myRotation)
        local animName = animConfig[1]
        local x = animConfig[2]
        local y= animConfig[3]
        local scale = animConfig[4] or 1

        local paoMa = GameUtil:createEffect(animName,true)
        paoMa:setFlippedX(config.isFlipX)
        paoMa:setPosition(x,y)
         -- print("myRotation",myRotation)
        paoMa:setRotation(myRotation)
        paoMa:setScale(scale)
        paoMa:playAnimation(config.name,true)
        return paoMa
    end
    local animList = self:getAnimConfig()

    self:setPosition(x1 + v1.x, y1 + v1.y)
    local degreeOffSetItem= degreeOffSets[temDegree]
    local animRotation = degreeOffSetItem[2] or (-temDegree)
    for animIndex,animConfig in ipairs(animList) do
        local anim = createAnim(animConfig,animRotation)
        self:addChild(anim)
    end
    local paretRotation = degreeOffSetItem[1] or temDegree
    -- print("paretRotation",paretRotation)
    self:setRotation(paretRotation)
end


function This:getPaoMaRadio()
    local curTime = GamePlayer:getServerTime()
    local radio = (curTime - self.m_startTime) / (self.m_endTime - self.m_startTime)
    return radio
end


function This:getRotationABS(degree)
    if degree < 0 then
        degree = 360 + degree
    end
    if degree >=360 then
        degree = degree-360
    end
    return degree
end

--获取节点旋转角度
function This:getConfigWithDegree(degree)
     degree = self:getRotationABS(degree)
     for i , config in ipairs(PaoMaEffect) do
         if degree <= i * 45 - 22.5 then
             local temDegree = config.rotation
             -- print(i,"节点旋转  ",temDegree)
             return config,temDegree
         end
     end
     -- local config = PaoMaEffect[1]
     local config = PaoMaEffect[1]
     local temDegree = config.rotation
     -- print("节点旋转  ",temDegree)
     return config,temDegree
end