
--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Stage.EmbattleHeroPanel")
local WorldMapManager = require("World.WorldMapManager")
local GamePlayer = require("Player.GamePlayer")
local Path = require("World.Path")
local HeroManager = require("Hero.HeroManager")
local StageHeroItemView = require("Stage.StageHeroItemView")
local GameUtil = require("Util.GameUtil")
local PlanShopUtil = require("PlanShop.PlanShopUtil")
local TroopView = require("World.TroopView")
local DailyAchieveManager = require("Achieve.DailyAchieveManager")

function This:initNode(info)
	self.m_fromCity = info.fromCity
	self.m_toCity = info.toCity
	if self.m_fromCity:isMy() then
		local troopList = self.m_fromCity:getRoleTroop(GamePlayer:getPlayerID())
		self.m_troops = {}
		for i , troop in pairs(troopList) do
			self.m_troops[troop:getHeroId()] = troop
		end
	else
		local troopList = info.troopList
		self.m_troops = {}
		for i , troop in pairs(troopList) do
			self:addHero(troop:getHeroId(),troop:getPos())
		end

	end
	super.initNode(self,info)
end

function This:initItems()
end

function This:onEnter()
	super.onEnter(self)
	WorldMapManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	WorldMapManager:removeObserver(self)
end

function This:initBaseInfo()
	self.m_csNode.bg3.taget:setString(self.m_toCity:getName())
	self.m_csNode.bg3.tagettext:setString(Serialize["world_TroopEmbattle_1"])
	local ownerId =  self.m_toCity:getOwnerId()
	if ownerId > 0 then
		local role = WorldMapManager:getRole(ownerId)
		self.m_csNode.bg3.name:setString(role:getName())
	else
		self.m_csNode.bg3.name:setString(string.format(Serialize["world_CityIntelligence_1_c"],self.m_toCity:getName()))
	end

	self.m_path = Path:getPath(self.m_fromCity:getCityId(),self.m_toCity:getCityId())
	local time = self.m_path:getCostTime()
	--行军时间
	local totalHp = self:getTotalHp()
	local foodCost = 0

	if not self.m_toCity:isOur() then
		foodCost = self.m_path:getCostFoodForGoFight(totalHp,time)
	else
		foodCost = self.m_path:getCostFoodForTansport(totalHp,time)
	end
	if self.m_toCity:isUsedOnCity() then
		local foodRadio = DailyAchieveManager:getArgConfigLv("zb_plan_food_coefficient")
		local timeRadio = DailyAchieveManager:getArgConfigLv("zb_plan_time_coefficient")
		foodCost = foodCost*foodRadio
		time = time*timeRadio
	end
	self.m_csNode.bg3.cost:setString(math.floor(foodCost))
	self.m_csNode.bg3.totalhp:setString(math.floor(totalHp))
	self.m_csNode.bg3.consume:setString(Serialize["world_TroopEmbattle_2"])
	self.m_csNode.time:setString(GameUtil:formatTime(Serialize["H_M_S"],time * 60))
end

function This:refreshBaseInfo()
	local hp = self:getTotalHp()
	self.m_csNode.bg3.totalhp:setString(string.format("%d",hp))
	local time = self.m_path:getCostTime()
	local totalHp = self:getTotalHp()
	
	local foodCost = 0
	if not self.m_toCity:isOur() then
		foodCost = self.m_path:getCostFoodForGoFight(totalHp,time)
	else
		foodCost = self.m_path:getCostFoodForTansport(totalHp,time)
	end
	self.m_csNode.bg3.cost:setString(math.floor(foodCost))
end

function This:getTotalHp()
	local total = 0
	for i , info in pairs(self.m_heros) do
		local troop = self.m_troops[info.heroId]
		if troop then
			total = total + troop:getHp()
		end
	end
	return total
end

function This:getHeroView(heroId)
	local troop = self.m_troops[heroId]
	local hero = HeroManager:getHero(heroId)
	local node = StageHeroItemView:create(heroId,{
		maxHp = hero:getAttr(AttrType.kMaxHp),
		hp = troop:getHp(),
	})
	return node
end

function This:refreshHeroView(view,heroId)
	local troop = self.m_troops[heroId]
	view:setModel(troop)
end

function This:getAllHeroList()
	local result = {}
	local troopList = {}
	for i , troop in pairs(self.m_troops) do
		table.insert(troopList,troop)
	end 
	table.sort(troopList,function(a,b)
		return a:getPos() < b:getPos()
	end)
	for i , troop in ipairs(troopList) do
		local hero = HeroManager:getHero(troop:getHeroId())
		table.insert(result,hero)
	end
	return result
end

function This:openSelectPanel()
	local haveSelected = {}
	for i , info in pairs(self.m_heros) do
		haveSelected[info.heroId] = true
	end
	local panel = g_scene:openDialog("World.TroopSelectPanel",{troops = self.m_troops, haveSelected = haveSelected})
	return panel
end

function This:gotoBattle(heroList,isOnceKey)
	if isOnceKey then
		return
	end

	if self.m_fromCity:isFighting() then
		g_scene:showTips(Serialize["world_TroopEmbattle_3"])
		return
	end

	if #heroList <= 0 then
		g_scene:handleErrorCode("please_add_hp_first")
		return
	end

    local time = self.m_path:getCostTime()
	local totalHp = self:getTotalHp()
	local foodCost = self.m_path:getCostFoodForGoFight(totalHp,time)
	if PlanShopUtil:needUsePlan(JiCeType.kWuGuFengDeng, foodCost) then
		return
	end

	local troopList = {}
	local posList = {}
	for i , info in pairs(heroList) do
		local hero = HeroManager:getHero(info.heroId)
		table.insert(troopList,hero:getUid())
		table.insert(posList,info.pos)
	end
	WorldMapManager:onMarchArmyRequest(self.m_fromCity:getCityId(),self.m_toCity:getCityId(),troopList,posList,0)
	if not self.m_toCity:isOur() then
		cc.Audio:playEffect("Chuzheng.mp3",false)
	else
		cc.Audio:playEffect("Yunshubingli.mp3",false)
	end
end

function This:onMarchArmy(troopInfo)
	if troopInfo.from_city == self.m_fromCity:getCityId() and troopInfo.to_city == self.m_toCity:getCityId() then
		self:close()
	end
end

--endregion
