--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Dialog")
local HeroManager = require("Hero.HeroManager")
local TroopView = require("World.TroopView")
local GamePlayer = require("Player.GamePlayer")
local ListLayer = require("UI.ListLayer")
local Troop = require("World.Troop")
local WorldMapManager = require("World.WorldMapManager")
local SwapLayer = require("UI.SwapLayer")
local PlanShopUtil =  require("PlanShop.PlanShopUtil")
local PlanShopManager = require("PlanShop.PlanShopManager") 
local GuideManager = require("Guide.GuideManager")
local HeroListManager = require("List.HeroListManager")
local UpdateLayer = require("UI.UpdateLayer")

local curQueueSelectedIndex = 0 

function This:initNode(city)
	self.m_city = city
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TroopsMain.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self.m_isRequesting = true
	self:refreshInfo()
	
	self.m_csNode.creepsbutton:setVisible(self.m_city:isMy())
	self.m_csNode.dissolutionbutton:setVisible(self.m_city:isMy())
	self.m_csNode.transportationbutton:setVisible(self.m_city:isMy())
	--self.m_csNode.buytroopsbutton:setVisible(GuideManager:getGuideStep() >= GuideNodeVisible.ZhaoBingMaiMaBtn)
	self.m_csNode.queue_list:setVisible(false)

	HeroManager:onUpdateHeroStateRequest()
end

function This:initItems()
	self.m_troopList = self:getTroopList()
	self.m_listLayer = SwapLayer:create({
		itemWidth = 130,
		itemHeight = 170,
		itemCount = #self.m_troopList,
		frameCount = #self.m_troopList,
		column = 6,
		scrollView = self.m_csNode.heroList,
	})
	self.m_listLayer:setAnchorPoint(0.5,0)
	local w , h = self.m_listLayer:getContentSize()
	self.m_csNode.heroList:pushBackCustomItem(self.m_listLayer, w,h,0)

	self.m_listLayer:setDragSwapNode(true)
	self.m_listLayer:setSwapHandler(function(pos1,pos2)
		local tmp = self.m_troopList[pos1]
		self.m_troopList[pos1] = self.m_troopList[pos2]
		self.m_troopList[pos2] = tmp

		local heroList = {}
		local hpList = {}
		for i , troop in ipairs(self.m_troopList) do
			if troop:getHp() <= 0 then
				break
			end
			table.insert(heroList,troop:getId())
			table.insert(hpList,troop:getHp())
		end

		WorldMapManager:onAddHeroHpRequest(self.m_city:getCityId(),{heros = heroList,hp = hpList})
	end,function(pos1,pos2)
		if self.m_city:isFighting() then
			return false
		end
		local troop1 = self.m_troopList[pos1]
		local troop2 = self.m_troopList[pos2]
		if not troop1 or not troop2 then
			assert(false)
		end
		return troop1:getHp() > 0 and troop2:getHp() > 0
	end)
	self.m_listLayer:setCascadeOpacityEnabled(true)
	self:refreshHeroListBtns(false)
	self:refreshItems()
end

function This:initTouchEvent()
	self.m_csNode.close:setTouchEnded(function()
		self:close()
	end)
	self.m_csNode.buytroopsbutton:setTouchEnded(function()
		self:onBuyTroopBtnClicked()
	end)
	self.m_csNode.creepsbutton:setTouchEnded(function()
		self:onOnceKeyAddHpBtnClicked()
	end)
	self.m_csNode.dissolutionbutton:setTouchEnded(function()
		self:onOnceKeyDissolveBtnClicked()
	end)
	self.m_csNode.transportationbutton:setTouchEnded(function()
		self:onTransportHpBtnClicked()
	end)
	self.m_csNode.battlebutton:setTouchEnded(function()
		self:onGotoBattleBtnClicked()
	end)
	self.m_csNode.listbtn:setTouchEnded(function()
		print("队列...")
	end)
	self.m_csNode.listbtnxiala:setTouchEnded(function()
		self:refreshHeroListBtns(true)
	end)
	for i = 0 , 6 do
		self.m_csNode.queue_list["selectbg"..i]:setTouch({
			onEnded = function()
				self:refreshItemsByQueueIndex(i)
				self:refreshHeroListBtns(false)
			end,
			isScale = false
		})
	end
end

function This:initMaskLayer()
	self.m_maskLayer = cc.LayerColor:create(cc.Color(0, 0, 0, 50))
    self:addChild(self.m_maskLayer)
    self.m_maskLayer:setTouchForMask({
		onBegan = function ()
			print("initMaskLayer---------- began")
			return true
        end,
        onEnded = function ()
            self:onTouchMask()
			print("initMaskLayer---------- ended")
        end
    })
end

function This:onEnter()
	super.onEnter(self)
	WorldMapManager:addObserver(self)
	HeroManager:addObserver(self)
    PlanShopManager:addObserver(self)
end

function This:onExit()
	super.onExit(self)
	WorldMapManager:removeObserver(self)
	HeroManager:removeObserver(self)
    PlanShopManager:removeObserver(self)
end

function This:getTroopList()
	local troopList = {}
	local index = 1
	local roleTroop = self.m_city:getRoleTroop(GamePlayer:getPlayerID())
	local queue = HeroListManager:getList(curQueueSelectedIndex)


	for i , tp in ipairs(roleTroop) do
		local heroId = tp:getHeroId()
		local hero = HeroManager:getHero(heroId)
		local troop = Troop:create({
			id = tp:getId(),
			pos = index,
			card_type = tp:getHeroId(),
			city_id = tp:getCityId(),
			owner_id = tp:getOwnerId(),
			level = tp:getLevel(),
			cur_hp = tp:getHp(),
			max_hp = tp:getMaxHp(),
			star = tp:getStarLv(),
			quality = tp:getQuality(),
			fight_power = tp:getFightPower(),
		})
		table.insert(troopList,troop)
		index = index + 1
	end

	if not self.m_city:isMy() then
		return troopList
	end

	if curQueueSelectedIndex > 0 then
		local list = queue:getHeroList()
		for i , heroId in ipairs(list) do
			local hero = HeroManager:getHero(heroId)
			if hero:isGot() and not hero:isDeputy() and hero:getState() == HeroState.kInFree then
				local troop = Troop:create({
					id = 0,
					pos = index,
					city_id = self.m_city:getCityId(),
					owner_id = GamePlayer:getPlayerID(),
					level = hero:getLevel(),
					card_type = heroId,
					cur_hp = 0,
					max_hp = hero:getAttr(AttrType.kMaxHp),
					star = hero:getStarLv(),
					quality = hero:getQuality(),
					fight_power = hero:getFightPower(),
				})
				table.insert(troopList,troop)
				index = index + 1
			end
		end
	else
		local heroList = HeroManager:getAllHeroGotWithSort(function(hero)
			if hero:isGot() and not hero:isDeputy() and hero:getState() == HeroState.kInFree then
				return true
			end
		end)
		for i , hero in ipairs(heroList) do
			local heroId = hero:getId()
			if curQueueSelectedIndex == 0 or (queue and queue:hasHero(heroId)) then
				local troop = Troop:create({
					id = 0,
					pos = index,
					city_id = self.m_city:getCityId(),
					owner_id = GamePlayer:getPlayerID(),
					level = hero:getLevel(),
					card_type = heroId,
					cur_hp = 0,
					max_hp = hero:getAttr(AttrType.kMaxHp),
					star = hero:getStarLv(),
					quality = hero:getQuality(),
					fight_power = hero:getFightPower(),
				})
				table.insert(troopList,troop)
				index = index + 1
			end
		end
	end
	return troopList
end

function This:refreshInfo()
    local canUsePlan, useNum, maxNum = WorldMapManager:canUseItem(JiCeType.kZhaoBinMaima)
	self.m_csNode.buytroopsbutton.num:setString(maxNum - useNum)
    self.m_csNode.buytroopsbutton.troopslimit:setString("/" ..maxNum)

	local reserveHp = self.m_city:getReserveHp()
	local maxReserveHp = self.m_city:getMaxReserveHp()
	self.m_csNode.num:setString(reserveHp)
	self.m_csNode.troops:setString(string.format("/%d",maxReserveHp))
end

function This:refreshItems()
	self.m_troopList = self:getTroopList()
	self.m_listLayer:setDataList(self.m_troopList,function(model)
		local view = TroopView:create({troop = model,showLegion = true,city = self.m_city})
		view:setClickHandler(function(tp)
			local panel = g_scene:openDialog("World.TroopAdjustPanel",{troopList = self.m_troopList,tp = tp})
			panel:setShowHeroHandler(function()
				local heroList = {}
				for i , troop in ipairs(self.m_troopList) do
					table.insert(heroList,troop:getHeroId())
				end
				g_scene:openDialog("Hero.HeroPanel",{heroId = tp:getHeroId(),heroList = heroList})
			end)
		end)
		view:setCascadeOpacityEnabled(true)
		return view
	end)
end

function This:refreshItemsByQueueIndex(index)
	if curQueueSelectedIndex ~= index then
		curQueueSelectedIndex = index
		self.m_troopList = self:getTroopList()
		self:refreshItems()
	end
end

function This:refreshHeroListBtns(isShowBtn)
	if isShowBtn then
		self.m_csNode.queue_list:setVisible(true)
		for i = 1 , 6 do
			local queue = HeroListManager:getList(i)
			local node = self.m_csNode.queue_list["selectbg"..i]
			node:setVisible(queue:isOpen())
			node.teamname:setString(queue:getName())
		end
	else
		self.m_csNode.queue_list:setVisible(false)
	end
	
	if curQueueSelectedIndex == 0 then
		self.m_csNode.name:setString(Serialize["world_TroopPanel_1"])
	else
		local queue = HeroListManager:getList(curQueueSelectedIndex)
		self.m_csNode.name:setString(queue:getName())
	end
end

function This:onTouchMask()
	print("-----------")
end

function This:onBuyTroopBtnClicked()
	if self.m_city:isFighting() then
		g_scene:showTips(Serialize["world_TroopPanel_2"])
		return
	end
	if self.m_city:isImprisoned() then
		g_scene:showTips(Serialize["world_TroopPanel_6_c"])
		return
	end
    local canUseZBMM= WorldMapManager:canUseItem(JiCeType.kZhaoBinMaima)
	if canUseZBMM then
        PlanShopUtil:usePlan({plan=JiCeType.kZhaoBinMaima, num = 1, argId = self.m_city:getCityId(), object=self, callback=self.onUsePlan})
    else
        g_scene:showTips(Serialize["planshop_force_num_limit"])
    end
end

function This:onOnceKeyAddHpBtnClicked()
	if self.m_city:isFighting() then
		g_scene:showTips(Serialize["world_TroopPanel_5_c"])
		return
	end
	if self.m_city:isImprisoned() then
		g_scene:showTips(Serialize["world_TroopPanel_6_c"])
		return
	end
	local heroList = {}
	local hpList = {}
	local reserveHp = self.m_city:getReserveHp()

	if reserveHp <= 0 then
		g_scene:showTips(Serialize["world_TroopPanel_3"])
		self:onBuyTroopBtnClicked()
		return
	end

	local count = 0
	local copper = WorldMapManager:getCopper()

	local breakFlag, calcCount = false, 0

	for i = 1 , #self.m_troopList do
		local troop = self.m_troopList[i]
		local hero = HeroManager:getHero(troop:getHeroId())
		local maxHp = math.floor(hero:getAttr(AttrType.kMaxHp))
		local hp = troop:getHp()

		if count >= 6 or reserveHp <= 0 or calcCount >= 6 then
			break
		end

		if hp < maxHp then
			local left = math.min(reserveHp, maxHp - hp)
			local soilder = hero:getCurSoilder()	
			local cost =  soilder:getRecruitCost() * left
			if not breakFlag then
				if copper < soilder:getRecruitCost() then
					copper = 0
					breakFlag = true
				elseif not breakFlag then
					if copper < cost then
						left = math.floor(copper / soilder:getRecruitCost())
						cost = soilder:getRecruitCost() * left
					end
					table.insert(heroList,hero:getUid())
					table.insert(hpList, hp + left)
					count = count + 1
				end
			end
            calcCount = calcCount + 1
			reserveHp = reserveHp - left
			copper = copper - cost
		else
			table.insert(heroList,hero:getUid())
			table.insert(hpList, hp)
		end
	end

	if count > 0 then
		WorldMapManager:onAddHeroHpRequest(self.m_city:getCityId(),{heros = heroList,hp = hpList})
		cc.Audio:playEffect("Yijianbubing.mp3",false)
	else
		PlanShopUtil:needUsePlan( JiCeType.kCaiYuanGunGun, 0, - copper )
	end
end

function This:onOnceKeyDissolveBtnClicked()
	if self.m_city:isFighting() then
		g_scene:showTips(Serialize["world_TroopPanel_5_c"])
		return
	end
	if self.m_city:isImprisoned() then
		g_scene:showTips(Serialize["world_TroopPanel_6_c"])
		return
	end
	local heroList = {}
	local hpList = {}
	local troopList = self.m_city:getRoleTroop(GamePlayer:getPlayerID())
	for pos , troop in pairs(troopList) do
		local hero = HeroManager:getHero(troop:getHeroId())
		table.insert(heroList,hero:getUid())
		table.insert(hpList,0)
	end

	if #heroList <= 0 then
		g_scene:showTips(Serialize["world_TroopPanel_4"])
		return
	end

	if #heroList > 0 and #hpList > 0 then
		WorldMapManager:onAddHeroHpRequest(self.m_city:getCityId(),{heros = heroList,hp = hpList})
	end
end

function This:onTransportHpBtnClicked()
	if self.m_city:isFighting() then
		g_scene:showTips(Serialize["world_TroopPanel_5_c"])
		return
	end
	if self.m_city:isImprisoned() then
		g_scene:showTips(Serialize["world_TroopPanel_6_c"])
		return
	end
	g_scene:enterSelectMode(self.m_city,false)
	self:close()
end

function This:onGotoBattleBtnClicked()
	if self.m_city:isFighting() then
		g_scene:showTips(Serialize["world_TroopPanel_5_c"])
		return
	end
	if self.m_city:isImprisoned() then
		g_scene:showTips(Serialize["world_TroopPanel_6_c"])
		return
	end
	if #self.m_troopList>0 and self.m_troopList[1]:getHp() <= 0 then
		g_scene:handleErrorCode("please_add_hp_first")
		return
	end

	if self.m_city:isMy() then
		g_scene:enterSelectMode(self.m_city,true)
	else
		
	end
	self:close()
end

function This:onAddHeroHp(city)
	if self.m_city == city then
		self:refreshItems()
		self:refreshInfo()
	end
end

function This:onTroopPosChanged(city)
	if self.m_city == city then
		self:refreshItems()
	end
end

function This:onUpdateHeroState()
	if self.m_isRequesting then
		self:initItems()
		self.m_isRequesting = nil
	end
end

function This:onUsePlan(msg)
    if msg.planId == JiCeType.kZhaoBinMaima then
    	g_scene:showTips(Serialize[64949] .. WorldMapManager:getValueByUsingJice(msg.planId)*msg.item_num)
		self:refreshInfo()
    end
end

--endregion
