--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This, super = class(...,"cc.Node")
local HeroManager = require("Hero.HeroManager")
local HeroConfig = require("Metadatas.HeroConfig")
local GamePlayer = require("Player.GamePlayer")
local GameUtil = require("Util.GameUtil")

function This:init(info)
	self.m_troop = info.troop
	self.m_city = info.city
	self.m_showLegion = info.showLegion
	self.m_csNode = cc.Node:createFromCSFile("GameScene/NodeUI/TroopsHeroItem.csb")
	self:addChild(self.m_csNode)
	local w, h = self.m_csNode.qualitybg:getContentSize()
	local scale = self.m_csNode.qualitybg:getScale()
	w = w * scale
	h = h * scale
	self:setContentSize(w,h)
end

function This:onEnter()
	super.onEnter(self)
	self:initNode()
	if self.m_troop:isMy() then
		HeroManager:addObserver(self)
	end
end

function This:onExit()
	super.onExit(self)
	if self.m_troop:isMy() then
		HeroManager:removeObserver(self)
	end
end

function This:initNode()
	local hero = HeroManager:getHero(self.m_troop:getHeroId())
	local country = hero:getCountry()
	local quality = self.m_troop:getQuality()

	self.m_csNode.country:setString(CountryName[country])
	local color = SmallCardQualityColor[quality]
	self.m_csNode.country:setColor(color.r , color.g, color.b)
	self.m_csNode.name:setString(hero:getName(), AppText:isChinese() and 10 or nil)
	self.m_csNode.icon:setImage(hero:getIcon())

	self.m_csNode.level:setString(self.m_troop:getLevel())
	self.m_csNode.qualitybg:setImage(QualityBgFile[quality])
	self.m_csNode.quality:setImage(QualityFrameFile[quality])
	self.m_csNode.hpbg:setImage(QualityFightPowerFrameFile[quality])
	
	for i = 1, 5 do
		self.m_csNode["star"..i]:setVisible(self.m_troop:getStarLv() >= i)
	end

	if self.m_showLegion and GamePlayer:getPlayerID() == self.m_troop:getOwnerId() then
		local leginName = hero:getLegionName()
		if leginName then
			self.m_csNode.corps:setString(leginName)
		else
			self.m_csNode.corps:setVisible(false)
			self.m_csNode.corpsbg:setVisible(false)
		end
	else
		self.m_csNode.corps:setVisible(false)
		self.m_csNode.corpsbg:setVisible(false)
	end
	self.m_csNode.fightPower:setString(math.floor(self.m_troop:getFightPower()))
	self:refresh()
end

function This:refresh()
	local hp = self.m_troop:getHp()
	local maxHp = self.m_troop:getMaxHp()
	self.m_csNode.hp:setString(string.format("%d/%d",hp,maxHp))
	self.m_csNode.hpbar:setPercent(hp / maxHp * 100)
	self:refreshAwak(self.m_csNode)
end

function This:refreshAwak(node)
	local isMy = false
	if GamePlayer:getPlayerID() == self.m_troop:getOwnerId() then
		isMy = true
	end
	local hero = HeroManager:getHero(self.m_troop:getHeroId())
	local heroEffect = false
	if hero:isAwaken() then
		node.xingdian:setVisible(true)
		local curentLv = hero:getAwakeLevel()
		if not isMy then
			curentLv = self.m_troop:getAwake() or 0
		end
		if curentLv==0 then
			node.xingdian.light:setVisible(false)
		else
			node.xingdian.light:setVisible(true)
			for i=1,AwakdValue.ParagraphLimit do
				local item = node.xingdian.light["dian"..i]
				item:setVisible(curentLv>=i)
			end
		end
		heroEffect = hero:isAwakeFull()
		if not isMy then
			heroEffect = self.m_troop:isAwakeFull()
		end
	else
		node.xingdian:setVisible(false)
	end

	self:createWeakEffect( node.quality, heroEffect )
end

function This:createWeakEffect( node,isAdd,heroBg )
	local function createAnim( parent,anmStr )
        local GameUtil = require("Util.GameUtil")
        local effect1 = GameUtil:createEffect(anmStr, true)
        local scale =0.564
        effect1:setScale(scale)
        local x = 56
        local y = 54
        effect1:setPosition(x,y)
        print("xy",x,"  ",y,"scale",scale)
        parent:addChild(effect1)
        return effect1
    end
    local effectName = "JueXing5"
    node:removeAllChildren()
    if isAdd then
    	local effect = createAnim( node,effectName )
    end
end

function This:setModel(troop)
	if self.m_troop:isMy() then
		HeroManager:removeObserver(self.m_troop)
	end
	if self.m_troop:getHeroId() ~= troop:getHeroId() then	
		self.m_troop = troop
		self:initNode()
	elseif self.m_troop:getHp() ~= troop:getHp() then
		self.m_troop = troop
		self:refresh()
	end	
	if self.m_troop:isMy() then
		HeroManager:removeObserver(self.m_troop)
	end
end

function This:setClickHandler(handler)
	self.m_clickHandler = handler
end

function This:setSelectHandler(handler,isContainer)
	self:setTouch({
		isContainer = isContainer or false,
		isDynamic = true,
		isSwallow = false,
		isScale = false,
		sizeNode = self.m_csNode.qualitybg,
		onEnded = function()
			handler(self.m_troop:getHeroId())
		end
	})
end

function This:getTroop()
	return self.m_troop
end

function This:onClicked()
	if self.m_clickHandler then
		if self.m_city then
			if self.m_city:isMy() then
				self.m_clickHandler(self.m_troop)
			end
		else
			self.m_clickHandler(self.m_troop)
		end
	end
end

function This:onUpgradeLevel(heroId)
	if self.m_troop:getHeroId() == heroId then
		self:initNode()
	end
end

function This:onUpgradeStarLv(heroId)
	if self.m_troop:getHeroId() == heroId then
		self:initNode()
	end
end

function This:onUpgradeQuality(heroId)
	if self.m_troop:getHeroId() == heroId then
		self:initNode()
	end
end

function This:onDeputyChanged(heroId)
	if self.m_troop:getHeroId() == heroId then
		self:initNode()
	end
end

function This:onEquipChanged(heroId)
	if self.m_troop:getHeroId() == heroId then
		self:initNode()
	end	
end



--endregion
