--region *.lua
--Date
--此文件由[BabeLua]插件自动生成

local This , super = class(...,"Base.BaseModel")
local PlanShopUtil = require("PlanShop.PlanShopUtil")
local WorldMapManager = require("World.WorldMapManager")
local PlanShopConfig = require("Metadatas.PlanShopConfig")
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local PlanShopManager = require("PlanShop.PlanShopManager")
local EventDispatcher = require("EventDispatcher")
local UpdateList = require("Base.UpdateList")
local ItemConfig = require("Metadatas.ItemConfig")
local GameUtil = require("Util.GameUtil")
local NobilityManager = require("Nobility.NobilityManager")

function This:init(info)
	super.init(self)
	self.m_id = info.id
	self.m_useCount = info.useCount or 0
	self.m_lastUseTime = info.lastUseTime or 0
	self.m_config = PlanShopConfig[self.m_id]
	self.m_cityList = UpdateList:create()
	self.m_isUpdating = false
	self.m_isUseCDUpdating = false
	self.m_escape = 0
end

function This:initListener()
	EventDispatcher:addListener(self,11303,self.onUseJiceReply)
end

function This:removeListener()
	EventDispatcher:removeListener(self,11303)
end

function This:clear()
	if self.m_isUpdating or self.m_isUseCDUpdating then
		self:endUpdate()
	end
	self:removeListener()
	self.m_cityList:clear()
end

function This:getId()
	return self.m_id
end

function This:getConfig()
	return self.m_config
end

function This:getName()
	return self.m_config.name
end

function This:hasUseCD()
	return self.m_config.usetime > 0
end

function This:hasHoldCD()
	return self.m_config.holdtime > 0
end

function This:getUseCDTime()
	return self.m_config.usetime
end

function This:getHoldCDTime()
	return self.m_config.holdtime
end

function This:isHoldUpdating()
	return self.m_isUpdating
end

function This:isUseUpdating()
	return self.m_isUseCDUpdating
end

function This:getIcon()
	local itemId = self.m_config.item_id
	local icon = ItemConfig[itemId].icon
	return GameUtil:getItemIcon(icon)
end

function This:getLeftUseCD()
	if not self.m_isUseCDUpdating then
		return 0
	end
	local curTime = GamePlayer:getServerTime()
	local leftTime = self.m_lastUseTime - curTime
	return math.max(0,leftTime)
end

function This:getLeftHoldCD(cityId)
	if not self.m_isUpdating then
		return 0
	end
	local curTime = GamePlayer:getServerTime()
	local info = self.m_cityList:getNode(cityId)
	if info then
		local leftTime = info.endTime - curTime 
		return math.max(0,leftTime)
	end
	return 0
end

function This:addCity(cityId,startTime,endTime)
	startTime = startTime or 0
	endTime = endTime or 0

	local curTime = GamePlayer:getServerTime()
	if self.m_cityList:hasNode(cityId) then
		local t = self.m_cityList:getNode(cityId)
		t.startTime = startTime
		t.endTime = endTime 
		self.m_isUpdating = true
	elseif curTime < endTime then
		local t = { cityId = cityId, startTime = startTime, endTime = endTime}
		self.m_cityList:addNode(t,cityId)
		if not self.m_isUpdating and not self.m_isUseCDUpdating then
			self:startUpdate()
		end
		self.m_isUpdating = true
	end
end

function This:removeCity(cityId)
	self.m_cityList:removeNode(cityId)
	self.m_escape = self.m_escape + 1
end

function This:hasCity(cityId)
	return self.m_cityList:hasNode(cityId)
end

function This:getCityList()
	local result = self.m_cityList:getNodeList()
	local cityList = {}
	for i , info in ipairs(result) do
		table.insert(cityList,info.cityId)
	end
	return cityList
end

function This:setUseCount(count)
	self.m_useCount = count
end

function This:getUseCount()
	return self.m_useCount
end

function This:getMaxUseCount()
	if not self.m_maxUseCount then
		self.m_maxUseCount = WorldMapManager:getMaxUseItemNum(self:getId())
	end
	return self.m_maxUseCount
end

function This:getLeftUseCount()
	return self:getMaxUseCount() - self.m_useCount
end

function This:setLastUseTime(lastTime)
	local curTime = GamePlayer:getServerTime()
	self.m_lastUseTime = lastTime + self:getUseCDTime()

	if self.m_lastUseTime <= curTime then
		self.m_lastUseTime = 0
		if self.m_escape < 1 then
			self.m_escape = self.m_escape + 1
		end
	else
		if not (self.m_isUpdating or self.m_isUseCDUpdating) then
			self:startUpdate()
		end
		self.m_isUseCDUpdating = true
	end
end

function This:getLastUseTime()
	return self.m_lastUseTime
end

function This:isCanUseWithoutCD(city)
	local curentTime = GamePlayer:getServerTime()
	local startTimeOffset = curentTime - WorldMapManager:getWorldStartTime()
	local fightTimeOffset = curentTime - WorldMapManager:getNextFightTime()
	local function checkTime(timeType,timeLimit)
		if timeType==1 then
			return timeLimit - startTimeOffset
		elseif timeType ==2 then
			return timeLimit - fightTimeOffset
		end
	end
	local temStartTime = checkTime(self.m_config.start_time_type,self.m_config.start_time)
	local temfightTime = checkTime(self.m_config.end_time_type,self.m_config.end_time)
	if temStartTime>0 then
		local str = GameUtil:getFormatDtimeStr(temStartTime)
		return false, str..Serialize["world_StgHDSY_2"]
	end
	if temfightTime<0 then
		return false,Serialize["world_StgHDSY_3"]
	end
	if self:getLeftUseCount() <= 0 then
		return false, Serialize["world_Stratagem_1_c"]
	end
	if city:isHanXianDi() and self:getId()~= JiCeType.kHanDiShouYu then
		return false, Serialize["world_WorldStratagem_1"]
	end

	if not NobilityManager:checkPlanUnlock(self:getId()) then
		return false,string.format(Serialize["Warfare_WarfareInfo_54"],NobilityManager:getLockedNobilityName(self:getId()))
	end
	return true
end

function This:isCanUse(city)
	if self:getLeftUseCount() <= 0 then
		return false, Serialize["world_Stratagem_1_c"]
	end
	if self:getLeftUseCD() > 0 then
		return false, Serialize["world_StgCM_7_c"]
	end
	return true
end

function This:use(city)
	PlanShopManager:onUsePlanRequest({
		num = 1,
		plan = self.m_id,
		gold = self.m_config.gold,
		argId = city:getCityId(),
	},true)
end

function This:startUpdate()
	Application:addToUpdateList(self)
	self.m_escape = 1
end

function This:endUpdate()
	self.m_isUpdating = false
	self.m_isUseCDUpdating = false
	Application:removeFromUpdateList(self)
	EventDispatcher:dispatcher("stg_end_update")
end

function This:onUseJiceReply(msg)
	
end

function This:update(dt)
	self.m_escape = self.m_escape + dt
	if self.m_escape < 1.0 then
		return
	end
	self.m_escape = self.m_escape - 1.0
	
	local curTime = GamePlayer:getServerTime()
	for key , info in self.m_cityList:getNextIterator() do
		if curTime <= info.endTime then
			self:notifyObservers("onStgHoldUpdate",info.cityId, self.m_id,info.endTime - curTime)
		else
			self:notifyObservers("onStgHoldUpdate",info.cityId, self.m_id,0)
			self:removeCity(key)
		end
	end
	self.m_isUpdating = self.m_cityList:getSize() > 0

	if self.m_isUseCDUpdating then
		if curTime <= self.m_lastUseTime then
			self:notifyObservers("onStgUseUpdate", self.m_id, self.m_lastUseTime - curTime)
		else
			self:notifyObservers("onStgUseUpdate", self.m_id,0)
			self.m_isUseCDUpdating = false
		end
	end

	if not self.m_isUseCDUpdating and not self.m_isUpdating then
		self:endUpdate()
	end
end




--endregion
