-- @Author: superm
-- @Date:   2018-07-24 16:18:33
-- @Last Modified by:   superm
-- @Last Modified time: 2018-11-20 12:12:06
-- @File Name: WorldBoss.lua
-- @Description: 

local This ,super = class(...,"Base.BaseModel")
local WorldBossConfig = require("Metadatas.WorldBossConfig")
local GamePlayer = require("Player.GamePlayer")
local Application = require("cc.Application")
local WorldBossRewardConfig = require("Metadatas.WorldBossRewardConfig")
local WorldBossLevelConfig = require("Metadatas.WorldBossLevelConfig")
local HeroManager = require("Hero.HeroManager")
local BattleDataManager = require("Battle.BattleDataManager")

function This:create(info)
	local ret = {}
	setmetatable(ret, self)
	ret:init(info)
	return ret
end

function This:init(info)
	super.init(self)
	self.m_id = info.boss_id
	
	self.m_escape = 0
	self.m_startTime = 0
	self.m_endTime = 0
	self.m_isUpdating = false

	self.m_exp 				    = 0
	self.m_level 			    = 0
	self.m_challengeTimes 	    = 0
    self.m_buyTimes 		    = 0
	self.m_totalForce 		    = 0
	self.m_curForce 		    = self.m_totalForc 
	if self.m_id > 0 then
		self.m_config = WorldBossConfig[info.boss_id]
	end
	self.m_state = self:getState()
	self.m_rewardStateList = {}

    self.m_weekchallengeTimes 	= 0
    self.m_weekRewardInfos = {}
end

function This:isUpdating()
    return self.m_isUpdating
end

function This:updateState()
	self:updateTimeInfo()
	self.m_state = self:getState()
end

function This:updateTimeInfo()
	if self.m_id > 0 and self.m_startTime > 0 then
		local curTime = GamePlayer:getServerTime()
		if curTime < self.m_endTime then
			if not self.m_isUpdating then
				self:startUpdate()
			end
			return
		end
	end

	if self.m_isUpdating then
		self:endUpdate()
	end  
end

function This:isNotEnd()
    return self:isActive() or self:isFighting() or self:isDeath()
end

function This:isExist()
    return self.m_state ~= WorldBossState.kNone
end

function This:isNotOpen()
	local curTime = GamePlayer:getServerTime()
	local tmp = os.serverdate("*t",curTime)
	local wday = (tmp.wday + 5) % 7 + 1
	if not table.contain(self.m_config.week, wday) then
		return true
	end
	return false
end

function This:getState()
	if self.m_startTime <= 0 then
		return WorldBossState.kNone
	end

	if self:isNotOpen() then
		return WorldBossState.kNone
	end

	local curTime = GamePlayer:getServerTime()
	if curTime > self.m_endTime then
		return WorldBossState.kInvisible
	elseif curTime < self.m_startTime then
		return WorldBossState.kPrepare
	end

	if self.m_curForce <= 0 or self.m_slainer > 0 then
		return WorldBossState.kDeath
	end
	return not self.m_battleInfo and WorldBossState.kActive or WorldBossState.kFighting
end

function This:update( dt )
	self.m_escape = self.m_escape + dt
	if self.m_escape >= 1.0 then
		self.m_escape = self.m_escape - 1
	    local curTime = GamePlayer:getServerTime()
	    local leftOpenTime = self.m_startTime - curTime
	    if leftOpenTime > 0 then
	    	self:notifyObservers("updateOpenTime", leftOpenTime)
	    else
	    	if self:isPreparing() then
	    		self:updateState()
	    		self:notifyObservers("onBossOpenning", self)
	    	end

	    	local leftTime = self.m_endTime - curTime
	    	if leftTime > 0 then
				self:notifyObservers("updateTime", leftTime)
			else
	            self:updateState()
	        end
		end
	end
end

function This:reset()
	self.m_escape = 0
	self.m_startTime = 0
	self.m_endTime = 0
	self.m_isUpdating = false

	self.m_exp 				= 0
	self.m_level 			= 0
	self.m_challengeTimes 	= 0
    self.m_buyTimes 		= 0
	self.m_totalForce 		= 0
	self.m_curForce 		= 0

	if self.m_isUpdating then
		cc.Application:removeFromUpdateList(self)
		self.m_isUpdating = false
	end
	self.m_state = self:getState()
	self.m_rewardStateList = {}
	self.m_bossRewardInfos = {}
	self.m_challengeCards = {}

  --  self.m_weekchallengeTimes 	= 0
   -- self.m_weekRewardInfos = {}
end

function This:startUpdate()
	cc.Application:addToUpdateList(self)
	self.m_isUpdating = true
	self:notifyObservers("onBossUpdateStart", self)
end

function This:endUpdate()
	cc.Application:removeFromUpdateList(self)
	self.m_isUpdating = false
	self:notifyObservers("onBossUpdateEnd", self)
end

function This:getLeftTime()
    local curTime = GamePlayer:getServerTime()
    local offset = self.m_endTime - curTime
    if offset < 0 then
        return 0
    end
    return offset
end

function This:isPreparing()
	return self.m_state == WorldBossState.kPrepare
end

function This:isActive()
	return self.m_state == WorldBossState.kActive
end

function This:isFighting()
	return self.m_state == WorldBossState.kFighting
end

function This:isDeath()
    return self.m_state == WorldBossState.kDeath
end

function This:getValueMap(t, map)
	local ret = map or {}
	for _, v in ipairs(t) do
		ret[v] = true
	end
	return ret
end

function This:updateInfo( info )
	-- print_tmp(info)
	self.m_startTime 	= info.start_time or 0
	self.m_endTime 		= info.end_time or 0
	self.m_exp 			= info.exp or 0
	self.m_level 		= info.level
	self.m_totalForce 	= info.init_force_num
	self.m_curForce 	= info.force_num
	self.m_slainer 		= info.kill_boss_role_id

	if self.m_startTime > 0 and self.m_endTime > 0 then
		local curTime = GamePlayer:getServerTime()
		local endDate = os.serverdate("*t", self.m_endTime)
	    local curDate = os.serverdate("*t", curTime)
	    if endDate.year == curDate.year then
	    	-- print_tmp({endDate_yday = endDate.yday, curDate_yday = curDate.yday})
		    if endDate.yday < curDate.yday then
		    	-- 过期信息处理
		    	local zeroTime = os.time({day = curDate.day, month = curDate.month, year = curDate.year, hour = 0, min = 0, sec = 0})
				self.m_startTime 	= zeroTime + self.m_config.open_time
				self.m_endTime 		= zeroTime + self.m_config.close_time
				self.m_totalForce 	= info.init_force_num
				self.m_curForce 	= info.init_force_num
				self.m_slainer 		= 0
			elseif endDate.yday == curDate.yday and self.m_slainer > 0 then
		    	-- 提前结束
				self.m_endTime 		= curTime - 1
			end
	    end
	elseif self.m_startTime == 0 and self.m_endTime == 0 and self.m_totalForce > 0 then
    	-- 首次初始化
		local curTime = GamePlayer:getServerTime()
	    local curDate = os.serverdate("*t", curTime)
    	local zeroTime = os.time({day = curDate.day, month = curDate.month, year = curDate.year, hour = 0, min = 0, sec = 0})
		self.m_startTime 	= zeroTime + self.m_config.open_time
		self.m_endTime 		= zeroTime + self.m_config.close_time
		if self.m_totalForce ~= self.m_curForce then
			self.m_totalForce 	= info.init_force_num
			self.m_curForce 	= info.init_force_num
		end
	end

    self:updateRankInfo(info.boss_rank_list)
    self:updateState()
end

function This:updateChallengeInfo(info)
	self.m_slain 			    = info.is_reward
    self.m_damage 			    = info.damage
	self.m_challengeTimes 	    = info.challenge_time or 0
    self.m_buyTimes 		    = info.buy_time or 0
    self.m_weekchallengeTimes 	= info.weekchallenge_time
    self:updateWeekRewardInfo(info.week_reward_list)
    self:updateRewardInfo(info.reward_list)
    self:updateFightedCardinfo(info.card_id_list)
end

function This:updateRewardInfo(info)
    self.m_bossRewardInfos = self:getValueMap(info)
end

function This:updateWeekRewardInfo(info)
    self.m_weekRewardInfos = self:getValueMap(info)
end

function This:updateFightedCardinfo(info)
    self.m_challengeCards = self:getValueMap(info)
end

function This:updateRankInfo(info)
    self.m_bossRankInfoList = info

    self.m_myRank = 0
    local roleId = GamePlayer:getPlayerID()
    for i, rankInfo in ipairs(info) do
    	if rankInfo.role_id == roleId then
    		self.m_myRank = i
			break
    	end
    end
end

function This:setBuyTimes( times )
    self.m_buyTimes = times
end

function This:getBuyCost()
	if self.m_buyTimes >= self.m_config.buy_times then
		return 
	end
	return self.m_config.buy_times_cost[self.m_buyTimes + 1]
end

function This:getBuySkillCost()
	return self.m_config.buff_price
end

function This:getBuySkillId()
	return self.m_config.inspire_skill
end

function This:requestFighting(cardList, buyBuff)
    self.m_tmpCardList = cardList
    self.m_buyBuff = buyBuff
    return true
end

function This:clearFightReuest()
    self.m_tmpCardList = nil
    self.m_buyBuff = nil
end

function This:clearFightData()
    self:clearFightReuest()
	self.m_battleInfo = nil
	self.m_state = self:getState()
end

function This:onFight(info)
	if self.m_state > WorldBossState.kNone and self.m_state < WorldBossState.kFighting then
		self.m_state = WorldBossState.kFighting
		self.m_battleInfo = info
		self:updateFightedCardinfo(info.card_id_list)
		cc.SceneManager:switchTo("Territory.TerrBattleLoadingScene", nil, {
			msg = info,
			bgId = self.m_config.bgId,
			mode = PlayMode.kWorldBoss,
		})
		BattleDataManager:addObserver(self)
	end
end

function This:onBattleVictory()
	if self.m_battleInfo then
		BattleDataManager:removeObserver(self)
		g_scene:openDialog("WorldBoss.WorldBossOverPanel", self.m_battleInfo)
		self:updateState(true)
		self:clearFightData()
	end
end

function This:isCardCanChallenge(uid)
	if not self.m_challengeCards then
		return true
	end
	
	return not self.m_challengeCards[uid]
end

function This:getCanEmbattleHeroList()
	local heroList = HeroManager:getAllHeroGotWithSort(function(hero)
		if not self:isCardCanChallenge(hero:getUid()) then
			return false
		end
		return true
	end)
	return heroList
end

function This:onBattleFailed()
	self:onBattleVictory()
end

function This:getCurHp()
	if self.m_totalForce <= 0 then 
		return 0, 0, 0
	end
    return self.m_curForce, self.m_totalForce, self.m_curForce / self.m_totalForce 
end

function This:getLeftChallengeTimes()
    return self.m_config.free_times - self.m_challengeTimes + self.m_buyTimes
end

function This:getWeekChallengeTimes()
    return self.m_weekchallengeTimes
end

function This:getId()
	return self.m_id
end

function This:getConfig()
	return self.m_config
end

function This:getName()
    return self.m_config.name
end

function This:getIcon()
    return self.m_config.image
end

function This:getLevel()
    return self.m_level
end

function This:getExpPercent()
	local id = self.m_id * 1000 + self.m_level
	local config = WorldBossLevelConfig[id]
	if not config then
		return 0
	end
    return self.m_exp / config.level_exp
end

function This:getRankInfo()
    return self.m_bossRankInfoList
end

function This:getBattleLevel()
	local bLevel = self.m_config.open_level
    return bLevel, bLevel <= GamePlayer:getLevel() 
end

function This:getAllRewardList()
    if not self.m_allRewardList then
    	self.m_allRewardList = {}
    	local id = self:getId()
		for _, config in pairs(WorldBossRewardConfig) do
			if config.boss_id == id then
				table.insert(self.m_allRewardList, config)
			end
		end

		table.sort( self.m_allRewardList, function ( a, b )
		    return a.id < b.id
		end )
    end
   	return self.m_allRewardList
end

function This:getMyRankInfo()
	if not self.m_bossRankInfoList then
		return 0
	end
    return self.m_myRank, self.m_damage
end

function This:getKillerReward()
    return self.m_config.kill_reward
end

function This:isSlainer()
	if not self.m_slainer or self.m_slainer < 0 then
		return false
	end
    return self.m_slainer == GamePlayer:getPlayerID()
end

local kWBMaxRewardRankLevel = 100
function This:hasReweard()
	if self.m_challengeTimes <= 0 and self.m_myRank <= 0 then
		return
	end 

	local rankIndex = self.m_myRank
	local retId, lastId
	for index, info in pairs(self.m_allRewardList) do
        local curId = index
		if not retId and info.min > kWBMaxRewardRankLevel then
			retId = curId
			lastId = curId
			break
		end

		if info.max <= rankIndex and info.min >= rankIndex then
			retId = curId
			break
		end
	end

	local ret = retId or lastId
	return ret, self:isSlainer()
end

function This:getRankReward(index)
	local index = self:hasReweard()
	local reward = self.m_allRewardList[index]
	if reward then
		return reward.rewardlist
	end
end

-- 1- 排名奖励 2- 击杀奖励
function This:hasPickUpReweard()
	if self.m_challengeTimes <= 0 then
		return false, false
	end

	if not self.m_bossRewardInfos then
		return false, false
	end 
	return self.m_bossRewardInfos[1], self.m_bossRewardInfos[2]
end

-- 1- 排名奖励 2- 击杀奖励
function This:showRewardByType(rtype)
	local rewardlist = nil 
    if rtype == 1 then
    	rewardlist = self:getRankReward()
    else
    	rewardlist = self:getKillerReward()
    end

    if rewardlist then
		g_scene:showReward(rewardlist)
	end
end