-- @Author: superm
-- @Date:   2018-07-31 11:27:05
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-03 01:47:11
-- @File Name: WorldBossBtn.lua
-- @Description: 

local This, super = class(..., "Home.MenuButton")
local WorldMonsterManager = require("WorldBoss.WorldMonsterManager")


function This:init(config)
    super.init(self, config)
    self.m_autoClose = config.uiclass.auto_close
    self.m_queueList = WorldMonsterManager:getAllBosses()
end

function This:initTouchEvent()
    self:setTouchEnded(function()
        local panel = g_scene:openDialog("WorldBoss.WorldBossPanel", {
            queueList = self.m_queueList,
        })
        self.m_subView = panel
        panel:addCloseCallback(function ()
            self.m_subView = nil
        end)
    end) 
end

function This:addCustomListener()
    super.addCustomListener(self)
    if self.m_autoClose then
        WorldMonsterManager:addObserver(self)
    	if self.m_queueIdList then
    		for _, queue in pairs(self.m_queueList) do
                if queue then
                    queue:addObserver(self)
                end
            end
        end
    end
end

function This:removeCustomListener()
    if self.m_queueIdList then
        for _, queue in pairs(self.m_queueList) do
            if queue then
                queue:removeObserver(self)
            end
		end
	end
    WorldMonsterManager:removeObserver(self)
    super.removeCustomListener(self)
end


function This:getDefaultVisible()
    if self.m_queueList then
        for _, queue in pairs(self.m_queueList) do
            if queue and queue:isExist() then
                return true
            end
        end
    end

    return super.getDefaultVisible(self)
end

function This:onBossUpdateStart( boss )
    if self.m_autoClose then
    	local bossId = boss:getId()
        for _, queue in pairs(self.m_queueList) do
            if queue and queue:getId() == bossId then
                self:open()
                break
            end
        end
    end 
end

function This:onBossUpdateEnd( boss )
    if self.m_autoClose then
        local isAllClose = true 
        local bossId = boss:getId()
        for _, queue in pairs(self.m_queueList) do
            if queue:getId() ~= bossId and queue:isExist() then
                isAllClose = false
                break
            end
        end

        if isAllClose then
            self:close()
        end
    end 
end

function This:close()
    if self.m_subView and self.m_subView[0] ~= 0 then
        self.m_subView:close()
    end
    self:setVisible(false)
end

function This:open()
    self:setVisible(true)
end

function This:onRequestAllBossInfo()
    local visible = self:getCurVisible()
    self:setVisible(visible)
end