-- @Author: superm
-- @Date:   2018-07-24 19:21:33
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-23 17:12:48
-- @File Name: WorldBossPanel.lua
-- @Description: 

local This , super = class(..., "Dialog")
local RankPageListLayer = require("Rank.RankPageListLayer")
local WorldMonsterManager = require("WorldBoss.WorldMonsterManager")
local WorldBossRankItemView = require("WorldBoss.WorldBossRankItemView")
local DepotManager = require("Depot.DepotManager") 
local GameUtil = require("Util.GameUtil")

function This:initNode(info)
	self.m_csNode = cc.Node:createFromCSFile("MenuScene/NodeUI/WorldBoss.csb")
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

	self.m_bossList = WorldMonsterManager:getBossesList()
	self.m_selectIndex = 1
	self.m_bossCount = #self.m_bossList
	local boss = self.m_bossList[self.m_selectIndex]
	self:setCurBoss( boss )
	
    self.m_progressTimer = cc.ProgressTimer:create(self.m_csNode.numberbossjindu)
    self.m_csNode.yuanbg:addChild(self.m_progressTimer)
    self.m_progressTimer:setScaleY(-1)
    local w , h = self.m_csNode.yuanbg:getContentSize()
    self.m_progressTimer:setPosition(w / 2, h / 2)
	self:refreshBossView()

	self:initRankInfo()
    self:refresh()
end

function This:refresh()
    self.m_csNode.beijing.jiantou1:setVisible(self.m_selectIndex > 1)
	self.m_csNode.beijing.jiantou2:setVisible(self.m_selectIndex < self.m_bossCount)
	self.m_csNode.button2:setVisible(self.m_selectBoss:isActive())

	self:requestBossInfo()
end

function This:initRankInfo()
	self.m_listLayer = RankPageListLayer:create({
        itemWidth = 448,
        itemHeight = 36,
        spaceX = 0,
        spaceY = 1,
        scrollView = self.m_csNode.ScrollView,
		dataCounter = function ()
			local info = self.m_selectBoss:getRankInfo()
	        return #info
        end,
        viewCreator =  function(index, node)
			local curRankList = self.m_selectBoss:getRankInfo()
        	if not curRankList then
        		return
        	end

        	local info = curRankList[index]
	        if info then
		        if not node then
		        	local view = WorldBossRankItemView:create(info, index)
			        return view
			    else
		        	node:setModel(info, index)
		        end
		    end
	    end,
    })
    
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.m_csNode.ScrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)
end

function This:initTouchEvent()
	local rootNode = self.m_csNode
    rootNode.button1:setTouchEnded(function()
		if not self.m_selectBoss then
            return
    	end
		
		if self.m_selectBoss:getLeftTime() <= 0 then
			if self.m_requesting then
				return
			end
			self:requestBossInfo()
		end

        self:openDialog("WorldBoss.WorldBossRewardPanel", {
    		reward = self.m_selectBoss:getAllRewardList(),
    		target = self.m_selectBoss
    	})
    end)

    rootNode.button2:setTouchEnded(function()
		if not self.m_selectBoss:isDeath() and not self.m_selectBoss:isFighting() then
			self.m_challengeCoroutine = coroutine.create(function ()
				self:challenge()
				self.m_challengeCoroutine = nil
			end)
			coroutine.resume(self.m_challengeCoroutine)
		end
    end)


    local bgNode = rootNode.beijing
    bgNode.close:setTouchEnded(function()
		self:close()
    end)

    bgNode.jiantou1:setTouchEnded(function()
    	if self.m_selectIndex <= 1 then
    		return
    	end
    	self.m_selectIndex = self.m_selectIndex - 1
    	local boss = self.m_bossList[self.m_selectIndex]
    	self:setCurBoss( boss )
    	self:refresh()
    end)

    bgNode.jiantou2:setTouchEnded(function()
    	if self.m_selectIndex >= self.m_bossCount then
    		return
    	end
    	self.m_selectIndex = self.m_selectIndex + 1
    	local boss = self.m_bossList[self.m_selectIndex]
    	self:setCurBoss( boss )
    	self:refresh()
    end)

	self.m_csNode.wenhao:setTouchEnded(function()
		g_scene:openDialog("UI.HelpTips",{text = Serialize["world_boss_help_tips"], node = self.m_csNode.button1})
	end)

    self.m_csNode.box:setTouchEnded(function()
           self:openDialog("WorldBoss.WorldBossBoxRewardPanel",{boss = self.m_selectBoss})
        end)
end

function This:onEnter()
    super.onEnter(self)  
    WorldMonsterManager:addObserver(self)
end

function This:onExit()
	self:setCurBoss(nil)
    WorldMonsterManager:removeObserver(self)
    super.onExit(self)  
end

function This:setCurBoss( boss )
	if self.m_selectBoss then
		self.m_selectBoss:removeObserver(self)
	end

	self.m_selectBoss = boss
	if boss then
		boss:addObserver(self)
	end  
end

function This:checkItem(itemId, costNum)
    local result, errorCode = DepotManager:isItemOrResEnough(itemId, costNum)
    if not result then
        if errorCode then
            g_scene:handleErrorCode(errorCode)
        else
            g_scene:showTips(string.format(Serialize[60334], ItemUtil.getName(itemId)))
        end
    end
    return result
end

function This:challenge()
	local level, canFight = self.m_selectBoss:getBattleLevel()
	if not canFight then
		g_scene:showTips(string.format(Serialize[60692], level))
		return
	end

	self.m_challengeTimes = self.m_selectBoss:getLeftChallengeTimes()
    if self.m_challengeTimes <= 0 then
		local costNum = self.m_selectBoss:getBuyCost()
    	if not costNum then
			g_scene:showTips(Serialize[60693])
    		return
    	end

	    if self.m_selectBoss:isDeath() then
	    	g_scene:showTips(Serialize[64975])
	    	return
	    end

	    self:openDialog("UI.CostConfirmPanel",{
	    	cost = costNum,
	    	desc = Serialize[64976],
	        okHandler = function()
	        	if self:checkItem(102, costNum) then
		        	WorldMonsterManager:onBuyChallengeTimesRequest(self.m_selectBoss:getId())
		        end
	        end,
	        cancelHandler = function()
	            coroutine.resume(self.m_challengeCoroutine, 0)
	        end,
	    })
    	self.m_challengeTimes = coroutine.yield()
    end

	local cardList, buyBuff
	if self.m_challengeTimes > 0 then
		local panel = self:openDialog("WorldBoss.WorldBossEmbattlePanel",{
			battleHandler = function(param)
            	coroutine.resume(self.m_challengeCoroutine, param)
			end,
			stage = self.m_selectBoss
		})
    	cardList = coroutine.yield()
    else
    	return
	end

    if self.m_selectBoss:isDeath() then
    	g_scene:showTips(Serialize[64975])
    	return
    end

	if cardList then
		local costNum = self.m_selectBoss:getBuySkillCost()
	    self:openDialog("UI.CostConfirmPanel",{
	    	fileName = "MenuScene/NodeUI/WorldBossMorale.csb",
	    	cost = costNum,
	        okHandler = function()
	        	if self:checkItem(102, costNum) then
	        	    coroutine.resume(self.m_challengeCoroutine, true)
	        	end
	        end,
	        cancelHandler = function()
	            coroutine.resume(self.m_challengeCoroutine, false)
	        end
	    })
	    buyBuff = coroutine.yield()

	    if self.m_selectBoss:isDeath() then
	    	g_scene:showTips(Serialize[64975])
	    	return
	    end

		WorldMonsterManager:onChallengeWorldBossRequest(self.m_selectBoss:getId(), cardList, buyBuff)
	end
end

function This:refreshBossView()
	local isPreparing = self.m_selectBoss:isPreparing() 
	local isUpdating = self.m_selectBoss:isUpdating() 


    self.m_csNode.overtime:setVisible(isUpdating and not isPreparing)
    self.m_csNode.overtime_note:setVisible(isUpdating and not isPreparing)
    self.m_csNode.over:setVisible(not isUpdating and not isPreparing)
    self.m_csNode.opentime:setVisible(isUpdating and isPreparing)

    self.m_csNode.bosstu:setImage(self.m_selectBoss:getIcon())
    self.m_csNode.bossname:setString(self.m_selectBoss:getName())
    self.m_csNode.numberboss:setString(self.m_selectBoss:getLevel())

    self.m_progressTimer:fromToTimer(1, 0, self.m_selectBoss:getExpPercent())

    local curHp, totalHp, percent = self.m_selectBoss:getCurHp()
    self.m_csNode.jindutiao:setPercent(percent * 100)
    self.m_csNode.blood:setString(string.format("%d/%d", curHp, totalHp))
    self.m_csNode.number:setString(Serialize[64484] .. self.m_selectBoss:getLeftChallengeTimes())

    self.m_csNode.number1:setString(self.m_selectBoss:getWeekChallengeTimes())

    self:updateTime(self.m_selectBoss:getLeftTime())
end

function This:refreshRankInfo()
	local node = self.m_csNode.me
	local rank, damage = self.m_selectBoss:getMyRankInfo()
	node.hurt:setString(string.format(Serialize[60530], damage))
	local visible = rank > 0
	node.weishangbang:setVisible(not visible)
	node.name:setVisible(visible)
	if visible then
		node.name:setString(string.format(Serialize[60689], rank))
	end
end

function This:updateTime( leftTime )
	local time = leftTime > 0 and leftTime or 0
	self.m_csNode.overtime:setString(GameUtil:formatDTime(Serialize[64977], time))
end

function This:updateOpenTime( leftTime )
	local time = leftTime > 0 and leftTime or 0
	self.m_csNode.opentime:setString(GameUtil:formatDTime(Serialize[64978], time))
end

function This:onBossOpenning( boss )
    if self.m_selectBoss:getId() == boss:getId() then
    	self:refresh()
    end
end

function This:onBossUpdateEnd( boss )
    if self.m_selectBoss:getId() == boss:getId() then
    	self:refresh()
    end
end

function This:requestBossInfo()
	if self.m_requesting then
		return
	end

	local boss = self.m_selectBoss
    if not boss or boss:isNotOpen() then
        return
    end

	local bossId = boss:getId()

	self.m_requesting = true
	self.m_coroutine = coroutine.create(function ()
        WorldMonsterManager:onGetWorldBossInfoRequest(bossId, true)
	    coroutine.yield(bossId)

        WorldMonsterManager:onGetBossChallengeInfoRequest(bossId, true)
        self.m_requesting = false
	end)
    coroutine.resume(self.m_coroutine)
end

function This:onBuyChallengeTimes(bossId)
	local boss = self.m_selectBoss
	if boss and bossId then
		if bossId == boss:getId() and self.m_challengeCoroutine then
			local times = boss:getLeftChallengeTimes()
    		coroutine.resume(self.m_challengeCoroutine, times)
    	end
    end
end

function This:onRefreshWorldBoss(bossId)
	if self.m_requesting then
		if self.m_selectBoss and self.m_selectBoss:getId() == bossId then
			if self.m_coroutine then
				coroutine.resume(self.m_coroutine, boss)
			end
		end
	end
end		

function This:onGetBossChallengeInfo(bossId)
	if self.m_selectBoss and self.m_selectBoss:getId() == bossId then
		self:refreshBossView()
		-- print_r(self.m_selectBoss)
		self.m_listLayer:setDataList()
		self:refreshRankInfo()
	end
end

function This:onWorldBossChange( bossId )
    if self.m_selectBoss and self.m_selectBoss:getId() == bossId then
		self:refreshBossView()
    	self:refresh()
    end
end

function This:onRequestAllBossInfo()
	self.m_bossList = WorldMonsterManager:getBossesList()
	self.m_selectIndex = 1
	self.m_bossCount = #self.m_bossList
	if self.m_bossCount <= 0 then
		self:close()
		return
	end

	local boss = self.m_bossList[self.m_selectIndex]
	self:setCurBoss( boss )
    self:refresh()
end