-- @Author: superm
-- @Date:   2018-07-24 19:23:06
-- @Last Modified by:   zheng
-- @Last Modified time: 2020-07-03 18:34:45
-- @File Name: WorldBossRewardPanel.lua
-- @Description: 

local This , super = class(..., "Dialog")
local RankPageListLayer = require("Rank.RankPageListLayer")
local WorldBossRewardItemView = require("WorldBoss.WorldBossRewardItemView")
local WorldMonsterManager = require("WorldBoss.WorldMonsterManager")
local ItemNode = require("UI.ItemNode")

function This:initNode(info)
	self.m_filename = info.filename or "MenuScene/NodeUI/WorldBossReward.csb"

	self.m_rewardList = info.reward
	self.m_target = info.target

	self.m_csNode = cc.Node:createFromCSFile(self.m_filename)
	self:addChild(self.m_csNode)
	self.m_csNode:setToCenter()

    local node = self.m_csNode.me.yincang.scrollview
	node:setDirection("horizontal")
	node.m_direction = "right"
	node:setSwallowTouch(false)

	node = self.m_csNode.mejisha.scrollview
	node:setDirection("horizontal")
	node.m_direction = "right"
	node:setSwallowTouch(false)
end

function This:onEnter()
    super.onEnter(self)  
    WorldMonsterManager:addObserver(self)
    self:refresh()
end

function This:onExit()
    WorldMonsterManager:removeObserver(self)
    super.onExit(self)  
end

function This:refresh()
	self:initRewardInfo()
	self:initRewardList()
end

function This:initRewardInfo()
	local rankRewardIndex, killerReward = self.m_target:hasReweard()
	local pkRankReward, pkKillerReward = self.m_target:hasPickUpReweard()

	local topReward = self.m_target:getKillerReward()
	self:refreshItemList(self.m_csNode.mejisha, topReward, killerReward, pkKillerReward)

	local node = self.m_csNode.me
	local rank, damage = self.m_target:getMyRankInfo()
	-- node.hurt:setString(string.format("伤害量：%d", damage))
	local visible = rank > 0
	node.weishangbang:setVisible(not visible)
	node.number:setVisible(visible)
	node.number_0:setVisible(visible)
	if visible then
		node.number:setString(string.format(Serialize[60698], rank))
	end

	if self.m_target:getLeftTime() > 0 then
		return
	end

	node.yincang:setVisible(true)
	node.tips:setVisible(false)
	local rankReward = self.m_target:getRankReward(rankRewardIndex)
	self:refreshItemList(node.yincang, rankReward, rankRewardIndex, pkRankReward)
end

function This:refreshItemList(node, rewardList, hasReweard, hasPickUpReweard)

	node.button:setVisible(hasReweard and not hasPickUpReweard)
	node.yilingqu:setVisible(hasReweard and hasPickUpReweard)
	node.scrollview:removeAllItems()

	if not rewardList then
		return
	end
	for i , info in ipairs(rewardList) do
		local item = ItemNode:create(info[1],info[2])
		local w , h = item:getContentSize()
		item:setAnchorPoint(0.5,0.5)
		node.scrollview:pushBackCustomItem(item, w, h, 0.5)
	end
end

function This:initTouchEvent()
    self.m_csNode.beijing.closebg:setTouchEnded(function()
        self:close()
    end)

    self.m_csNode.mejisha.button:setTouchEnded(function()
		local _, hasPickUpReweard = self.m_target:hasPickUpReweard()
        if not hasPickUpReweard then
			WorldMonsterManager:onPickUpWorldBossRewardRequest(self.m_target:getId(), 2)
        end
    end)

    self.m_csNode.me.yincang.button:setTouchEnded(function()
		local hasPickUpReweard = self.m_target:hasPickUpReweard()
        if not hasPickUpReweard then
			local rank = self.m_target:getMyRankInfo()
			WorldMonsterManager:onPickUpWorldBossRewardRequest(self.m_target:getId(), 1, rank)        	
        end
    end)
end

function This:initRewardList()
	self.m_listLayer = RankPageListLayer:create({
        itemWidth = 608,
        itemHeight = 75,
        spaceX = 0,
        spaceY = 3,
        scrollView = self.m_csNode.scrollView,
		dataCounter = function ()
	        return #self.m_rewardList
        end,
        viewCreator =  function(index, node)
        	local info = self.m_rewardList[index]
	        if info then
		        if not node then
		        	local view = WorldBossRewardItemView:create(info, index)
			        return view
			    else
		        	node:setModel(info, index)
		        end
		    end
	    end,
    })
    
    self.m_listLayer:setAnchorPoint(0.5,0)
    local w , h = self.m_listLayer:getContentSize()
    self.m_csNode.scrollView:pushBackCustomItem(self.m_listLayer, w, h, 0)
    self.m_listLayer:setCascadeOpacityEnabled(true)
    self.m_listLayer:setDataList()
end

function This:onPickUpWorldBossReward( bossId, rtype )
	if not self.m_target or self.m_target:getId() ~= bossId then
		return
	end

	if self.m_target:getLeftTime() > 0 then
		return
	end

	local node
	if rtype == 1 then
		node = self.m_csNode.me.yincang
	else
		node = self.m_csNode.mejisha
	end
	node.button:setVisible(false)
	node.yilingqu:setVisible(true)
end