-- @Author: superm
-- @Date:   2018-07-25 15:30:53
-- @Last Modified by:   superm
-- @Last Modified time: 2018-08-22 17:56:21
-- @File Name: WorldMonsterManager.lua
-- @Description: 

local This, super = class(..., "Base.BaseModel")
local EventDispatcher = require("EventDispatcher")
local Socket = require("Net.Socket")
local WorldBoss = require("WorldBoss.WorldBoss")
local WorldBossUtil = require("WorldBoss.WorldBossUtil")
local WorldBossConfig = require("Metadatas.WorldBossConfig")

function This:init()
	super.init(self)
	self.m_bossList = nil
	self:initListener()
end

function This:initListener()
	EventDispatcher:addListener(self, 12400, self.onGetWorldBossInfoReply)
	EventDispatcher:addListener(self, 12401, self.onGetBossChallengeInfoReply)
	EventDispatcher:addListener(self, 12402, self.onChallengeWorldBossReply)
	EventDispatcher:addListener(self, 12403, self.onBuyChallengeTimesReply)
	EventDispatcher:addListener(self, 12404, self.onPickUpWorldBossRewardReply)
	EventDispatcher:addListener(self, 12405, self.onWorldBossChangeNotify)
    EventDispatcher:addListener(self, 12406, self.onGetWorldBossWeekRewardReply)
	EventDispatcher:addListener(self, "init_role_info", self.onInitManagerNotify)
	EventDispatcher:addListener(self, "day_passed", self.onDayPassNotify)
end


function This:removeListener()
	EventDispatcher:removeListener(self, 12400)
	EventDispatcher:removeListener(self, 12401)
	EventDispatcher:removeListener(self, 12402)
	EventDispatcher:removeListener(self, 12403)
	EventDispatcher:removeListener(self, 12404)
	EventDispatcher:removeListener(self, 12405)
	EventDispatcher:removeListener(self, "init_role_info")
	EventDispatcher:removeListener(self, "day_passed")
end

function This:getAllBosses()
	return self.m_bossList
end

function This:getBossesList()
    local ret = {}
    for _, boss in pairs(self.m_bossList) do
    	if boss:isExist() then
    		table.insert(ret, boss)
    	end
    end

    table.sort(ret, function ( bossA , bossB )
    	local stateA, stateB = bossA:getState(), bossB:getState()
    	if stateA ~= stateB then
    		return stateA > stateB
    	end
        return bossA:getId() < bossB:getId()
    end)

    return ret
end

function This:getBoss( bossId )
	local boss = self.m_bossList[bossId]
	if not boss then
		boss = WorldBoss:create({boss_id = bossId})
		self.m_bossList[bossId] = boss
	end
	return boss
end

function This:clearAllFightingRequest()
    for _, boss in pairs(self.m_bossList) do
    	boss:clearFightReuest()
    end
end
-----------------------------------------------------------------
function This:onGetWorldBossInfoRequest(bossId, slient)
	if not slient then
		Socket:sendAndLock(12400,{boss_id = bossId})
	else
		Socket:send(12400,{boss_id = bossId})
	end
end

function This:onGetWorldBossInfoReply(msg)
	if msg.err_code == 0 then
		local bossId = msg.boss_info.boss_id
		local boss = self:getBoss(bossId)
		if boss then
			boss:updateInfo(msg.boss_info)
		end

		if self.m_coroutine then
			coroutine.resume(self.m_coroutine, boss)
		end
		self:notifyObservers("onRefreshWorldBoss", bossId)
	else

		if msg.err_code == 12405 then
			print("boss没出现")
			if self.m_coroutine then
				coroutine.resume(self.m_coroutine)
			end
			return 
		end
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onGetWorldBossWeekRewardReply(msg)
    if msg.err_code == 0 then
        local boss = self:getBoss(msg.boss_id)
        if boss then
               boss:updateWeekRewardInfo(msg.week_reward_list)
               self:notifyObservers("onPickupWarBoxReward")
               g_scene:showReward(msg.reward_list)
        end
    else
         g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onWorldBossWeekRewardRequest(bossId, rewardId)
	Socket:sendAndLock(12406,{boss_id = bossId,reward_id = rewardId})
end


function This:onGetBossChallengeInfoRequest(bossId, slient)
	Socket:sendAndLock(12401,{boss_id = bossId})
end

function This:onGetBossChallengeInfoReply( msg )
    if msg.err_code == 0 then
		local bossId = msg.role_boss_info.boss_id
		local boss = self:getBoss(bossId)
		if boss then
			boss:updateChallengeInfo(msg.role_boss_info)
		end
        self:notifyObservers("onGetBossChallengeInfo", bossId)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onChallengeWorldBossRequest(bossId, cardList, buyBuff)
	local boss = self:getBoss(bossId)
    if boss and boss:requestFighting(cardList, buyBuff) then
		Socket:sendAndLock(12402, {boss_id = bossId, card_id_list = cardList, buy_buff = not buyBuff and 0 or 1})
    end
end

function This:onChallengeWorldBossReply(msg)
	if msg.err_code == 0 then
		local boss = self:getBoss(msg.boss_id)
		if boss then
			boss:onFight(msg)
		end
		self:notifyObservers("onChallengeWorldBoss")
	else
		self:clearAllFightingRequest()
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onBuyChallengeTimesRequest(bossId)
	Socket:sendAndLock(12403, {boss_id = bossId})
end

function This:onBuyChallengeTimesReply(msg)
	if msg.err_code == 0 then
		local boss = self:getBoss(msg.boss_id)
        if boss then
        	boss:setBuyTimes(msg.buy_time)
        end
		self:notifyObservers("onBuyChallengeTimes", msg.boss_id)
	else
		g_scene:handleErrorCode(msg.err_code)
	end
end

function This:onPickUpWorldBossRewardRequest(bossId, rtype, rank)
	Socket:sendAndLock(12404, {boss_id = bossId, type= rtype, rank_id = rank})
end

function This:onPickUpWorldBossRewardReply( msg )
    if msg.err_code == 0 then
		local boss = self:getBoss(msg.boss_id)
        if boss then
        	boss:showRewardByType(msg.type)
        	boss:updateRewardInfo(msg.reward_list)
        end
        self:notifyObservers("onPickUpWorldBossReward", msg.boss_id, msg.type)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:onWorldBossChangeNotify( msg )
    if msg then
		local bossId = msg.boss_info.boss_id
		local boss = self:getBoss(bossId)
		if boss then
			boss:updateInfo(msg.boss_info)
		end
		self:notifyObservers("onWorldBossChange", bossId)
    else
        g_scene:handleErrorCode(msg.err_code)
    end
end

function This:requestAllBossInfo(force)
	if self.m_requesting then
		return
	end

    if not self.m_bossList or force then
        if not self.m_bossList then
    	    self.m_bossList = {}
        end

        local openBossIdList = {}
		for bossId, _ in pairs(WorldBossConfig) do
			local boss = self:getBoss(bossId)
			boss:reset()
            if not boss:isNotOpen() then
                table.insert(openBossIdList, boss:getId())
            end
		end

		if #openBossIdList > 0 then
			self.m_requesting = true
			self.m_coroutine = coroutine.create(function ()
	            for i, bossId in ipairs(openBossIdList) do
	                self:onGetWorldBossInfoRequest(bossId, true)
				    local ret = coroutine.yield(bossId)
				    if not ret then
				    	break
				    end
	            end
	            self.m_requesting = false
	            self:notifyObservers("onRequestAllBossInfo", true)
			end)
			coroutine.resume(self.m_coroutine)
		else
			self:notifyObservers("onRequestAllBossInfo", false)
		end
	end
end


function This:onInitManagerNotify()
    self:requestAllBossInfo()
end

function This:onDayPassNotify()
    self:requestAllBossInfo(true)
end