local This = class(...)
b2.Physics = This
local lib_physics = lib2d.b2Physics
local b2Body = b2.Body
local b2Contact = b2.Contact
local setmetatable = setmetatable
local physicsList = {}
local contact = {}
for k,v in pairs(b2Contact) do
    contact[k] = v
end
local b2Contact_getShapeAB = lib2d.b2Contact.getShapeAB
lib_physics.setCallback("BeginContact", function (this, contactPtr)
    local contact = contact
    local self = physicsList[this]
    contact[0] = contactPtr
    local shapeA, shapeB = b2Contact_getShapeAB(contactPtr)
    contact.m_shapeA = self.m_shapes[shapeA]
    contact.m_shapeB = self.m_shapes[shapeB]
    local isContact = self.m_owner:onBeginContact(contact)
    contact.m_shapeA = false
    contact.m_shapeB = false
    contact.m_shape = false
    contact.m_worldManifold = false
    return isContact
end)

lib_physics.setCallback("PreSolve", function (this, contactPtr)
    local contact = contact
    local self = physicsList[this]
    contact[0] = contactPtr
    local shapeA, shapeB = b2Contact_getShapeAB(contactPtr)
    contact.m_shapeA = self.m_shapes[shapeA]
    contact.m_shapeB = self.m_shapes[shapeB]
    self.m_owner:onPreSolve(contact)
    contact.m_shapeA = false
    contact.m_shapeB = false
    contact.m_worldManifold = false
end)

function This:create(gravityX, gravityY, pixelPerMeter)
    local obj = {[0] = lib_physics.create(gravityX, gravityY, pixelPerMeter)}
    setmetatable(obj, self)
    physicsList[obj[0]] = obj
    obj:init()
    return obj
end

function This:init()
    self.m_bodies = {}
    self.m_shapes = {}
end

function This:destroy()
    physicsList[self[0]] = nil
    self.m_bodies = nil
    self.m_shapes = nil
    lib_physics.destroy(self[0])
end

local lib_createBody = lib_physics.createBody
function This:createBody(bodyDef)
    local bodyPtr = lib_createBody(self[0], bodyDef)
    local body = {
        [0] = bodyPtr,
        m_owner = false,
        m_physics = self,
        m_shapes = {},
        m_type = bodyDef.type,
        setVelocity = b2Body.setVelocity,
        getVelocity = b2Body.getVelocity,
        getPosition = b2Body.getPosition,
        setXY = b2Body.setXY,
        addXY = b2Body.addXY,
        activate = b2Body.activate,
        setGravity = b2Body.setGravity,
        update = b2Body.update,
    }
    setmetatable(body, b2Body)
    --assert(self.m_bodies[bodyPtr] == nil)
    self.m_bodies[bodyPtr] = body
    return body
end

local lib_destroyBody = lib_physics.destroyBody
function This:destroyBody(body)
    self.m_bodies[body[0]] = nil
    lib_destroyBody(self[0], body[0])
end

local lib_step = lib_physics.step
function This:step(dt, velocityIterations, positionIterations)
    if dt > 0.06 then
        dt = 0.06
    end
    lib_step(self[0], dt, velocityIterations, positionIterations)
end

local lib_isLocked = lib_physics.isLocked
function This:isLocked()
    return lib_isLocked(self[0])
end

function This:setIsContinuousPhysics(isContinuous)
    lib_physics.setIsContinuousPhysics(self[0], isContinuous)
end

function This:isContinuousPhysics()
    return lib_physics.isContinuousPhysics(self[0])
end

function This:setIsAllowSleeping(isAllowSleeping)
    lib_physics.setIsAllowSleeping(self[0], isAllowSleeping)
end

function This:isAllowSleeping()
    return lib_physics.isAllowSleeping(self[0])
end