local ffi = require("ffi")
ffi.cdef[[
typedef struct b2ShapeDef
{
    void* shape;
    void* userData;
    float32 friction;
    float32 restitution;
    float32 density;
    int32 isGround;
    int32 isSensor;
    int32 categoryBits;
    int32 maskBits;
    int32 groupIndex;
    int32 disableCallback;
}b2ShapeDef;
]]
local This = class(...)
local Vec = require("Vec")
b2.Shape = This
local b2ShapeDef = ffi.typeof("b2ShapeDef")
local function ShapeDef(t)
    if t then
        return b2ShapeDef(t)
    else
        local def = b2ShapeDef()
        def.friction = 0.2
        def.categoryBits = 0x0001
        def.maskBits = 0xFFFF
        return def
    end
end
b2.ShapeDef = ShapeDef

local lib_shape = lib2d.b2Shape

local lib_getType = lib_shape.getType
function This:getType()
    lib_getType(self[0])
end

local lib_setIsSensor = lib_shape.setIsSensor
function This:setIsSensor(isSensor)
    lib_setIsSensor(self[0], isSensor)
end

local lib_isSensor = lib_shape.isSensor
function This:isSensor()
    return lib_isSensor(self[0])
end

local lib_setFilter = lib_shape.setFilter
function This:setFilter(categoryBits, maskBits, groupIndex)
    lib_setFilter(self[0], categoryBits, maskBits, groupIndex)
end

local lib_setFilterMaskBit = lib_shape.setFilterMaskBit
function This:setFilterMaskBitOr(...)
    lib_setFilterMaskBit(self[0], b2.getMaskBitOr(...))
end

function This:setFilterMaskBitXOr(...)
    lib_setFilterMaskBit(self[0], b2.getMaskBitXOr(self:getFilterMaskBit(), ...))
end

function This:setFilterMaskBit(bit)
    lib_setFilterMaskBit(self[0], bit)
end

local lib_getFilterMaskBit = lib_shape.getFilterMaskBit
function This:getFilterMaskBit()
    return lib_getFilterMaskBit(self[0])
end

local lib_setFilter = lib_shape.setFilter
function This:setFilter(categoryBits, maskBits, groupIndex)
    lib_setFilter(self[0], categoryBits, maskBits, groupIndex)
end

local lib_getFilter = lib_shape.getFilter
function This:getFilter()
    return lib_getFilter(self[0])
end

local lib_refilter = lib_shape.refilter
function This:refilter()
    lib_refilter(self[0])
end

local lib_setDensity =lib_shape.setDensity
function This:setDensity(density)
    lib_setDensity(self[0], density)
end

local lib_getDensity = lib_shape.getDensity
function This:getDensity()
    return lib_getDensity(self[0])
end

local lib_setFriction = lib_shape.setFriction
function This:setFriction(friction)
    lib_setFriction(self[0], friction)
end

local lib_getFriction = lib_shape.getFriction
function This:getFriction()
    return lib_getFriction(self[0])
end

local lib_setRestitution = lib_shape.setRestitution
function This:setRestitution(restitution)
    lib_setRestitution(self[0], restitution)
end

local lib_getRestitution = lib_shape.getRestitution
function This:getRestitution()
    return lib_getRestitution(self[0])
end

local lib_getAABB = lib_shape.getAABB
-- function This:getAABB()
function This:getAABB(childIndex)
    return lib_getAABB(self[0], childIndex or 0)
end

local lib_setIsGround = lib_shape.setIsGround
function This:setIsGround(isGround)
    lib_setIsGround(self[0], isGround)
end

local lib_getIsGround = lib_shape.getIsGround
function This:isGround()
    lib_getIsGround(self[0])
end

local lib_setDisableCallback = lib_shape.setDisableCallback
function This:disableCallback()
    lib_setDisableCallback(self[0], true)
end

function This:enableCallback()
    lib_setDisableCallback(self[0], false)
end

local lib_rayCast = lib_shape.rayCast
function This:rayCast(x1, y1, x2, y2)
    return lib_rayCast(self[0], x1, y1, x2, y2)
end

function This:testPoint(x, y)
    return lib_shape.testPoint(self[0], x, y)
end

function This:removeFromBody()
    self.m_body:removeShape(self)
end
function This:destroy()
    self.m_body:removeShape(self)
end

local ANCHOR_CENTER = cc.Vec(0.5,0.5)
local VEC_ZEOR = cc.Vec(0, 0)
function This:resetBox(width, height, anchor, offset)
    anchor = anchor or ANCHOR_CENTER
    offset = offset or VEC_ZEOR
    lib_shape.resetBox(self[0], width, height, anchor.x, anchor.y, offset.x, offset.y)
end

function This:resetPolygon(pointList)
    return lib_shape.resetPolygon(self[0], b2.PointList(pointList), #pointList / 2)
end

function This:isValidVertices(pointList)
    return lib_shape.isValidVertices(self[0], b2.PointList(pointList), #pointList / 2)
end

function This:resetEdge(x1, y1, x2, y2)
    return lib_shape.resetEdge(self[0], x1, y1, x2, y2)
end

function This:resetCircle(radius, centerX, centerY)
    return lib_shape.resetCircle(self[0], radius, centerX, centerY)
end

function This:getShapeInfo()
    local shapeType, info = lib_shape.getShapeInfo(self[0])
    if shapeType == 0 then
        shapeType = "Circle"
    elseif shapeType == 1 then
        shapeType = "Edge"
    elseif shapeType == 2 then
        shapeType = "Polygon"
        local vertexList = {}
        for i = 1, #info, 2 do
            table.insert(vertexList, Vec(info[i], info[i + 1]))
        end
        info = vertexList
    elseif shapeType == 3 then
        shapeType = "Chain"
    end
    return shapeType, info
end

local onBeginContact = function (self, target, contact)
    local passWay = self.m_passWay
    if passWay == 12 then
        return false
    end
    local nx, ny = contact:getNormal(target.m_body)
    if passWay == 1 then
        if ny >= 0 then
            return false
        end
    elseif passWay == -1 then
        if ny <= 0 then
            return false
        end
    elseif passWay == 2 then
        if nx >= 0 then
            return false
        end
    elseif passWay == -2 then
        if nx <= 0 then
            return false
        end
    end
    return true
end

function This:setPassWay(way)
    self.m_passWay = way
    if way == 0 then
        self.m_owner = false
        self:disableCallback()
        self.onBeginContact = false
    else
        self:enableCallback()
        self.m_owner = self
        self.onBeginContact = onBeginContact
    end
end