local This = class(...)
cc.Application = This
local lib_application = lib2d.ccApplication

if not os.isWin32 then
    local cacheMainVer , cacheSubVer = lib_application.getVersion(lib_application.getInstance())
    cacheMainVer , cacheSubVer = cacheMainVer or "1", cacheSubVer or "0"

    local version = os.native_call("getVersion") 
    local mainVer , subVer = string.match(version,"(%d+).(%d+)")

    if cacheMainVer < mainVer then
        local updateDir = cc.FileUtils:getWritablePath().."/Update/"
        if cc.FileUtils:isDirectoryExist(updateDir) then
            print("删除更新包...")
            cc.FileUtils:removeDirectory(updateDir)
        end
    end
end
print("---------require(Proto.Protocal)----------")

local isPrintIntoFile = true
local oldprint = print
local isFirstWrite = true

--弱引用表用于测试
collectgarbage("setpause", 200)
collectgarbage("setstepmul", 5000)

g_weakTable = {}
setmetatable(g_weakTable, {__mode = "kv"})    

if os.isWin32 then
	print = function (...)
 		local t = {...}
 		local log = ""
 		for i , str in ipairs(t) do
 			if type(str) == "table" then
 				log = log .. table.tostring(str)
 			elseif i ~= #t then
 				log = log ..str.. ","
 			else
 				log = log .. str
 			end
 		end
 		if isPrintIntoFile then
 			local writePath = cc.FileUtils:getWritablePath()
 			if not cc.FileUtils:isDirectoryExist(writePath.."Data") then
 				cc.FileUtils:createDirectory(writePath.."Data")
 			end
 			writePath = writePath .. "Data/log.txt"
 			local file = nil
 			if isFirstWrite then
 				file = io.open(writePath,"w+")
 			else
 				file = io.open(writePath,"a+")
 			end
 			if file then
 				file:write(log.."\n")
 				io.close(file)
 			end
 			isFirstWrite = false
 		end
 		oldprint(log)
	 end
end

function This:init()
    
    cc.SkeletonNode.clearAllCharacterData()
    cc.SkeletonNode.clearAllEffectData()
    lib2d.clearCSNodeCache()
    cc.SpriteFrameCache:removeAllPlist()
    cc.TextureCache:releaseAllTextures()
    lib2d.ccFontCache.clear()

	cc.FileUtils:createDirectory(cc.FileUtils:getWritablePath() .. "Data/")
	print(string.format("writeable path = %s",cc.FileUtils:getWritablePath()))

    math.randomseed(os.time() + (os.clock() * 1000 % 1000))
    self[0] = lib_application.getInstance()
    cc.objectRegistry[self[0]] = self
    self.m_updateList = require("Base.UpdateList"):create()
    self.m_timer = require("Timer"):create(self)
    self.m_escape = 0

    self.m_listenerList = {}
    self.m_pressKeys = {}
    self.m_isRunning = true

	cc.Director:getOpenGLView():initDesignResolutionSize()
    cc.Director:getOpenGLView():initVisible()
    cc.Director:setDisplayStats(os.isWin32 and g_setting.isDebug)
    cc.Director:setAnimationInterval(1.0 / 60.0)
    if os.isWin32 then
        lib2d.ccHttp.initcurl(0)
    end
	self:initConfigArg()
	cc.SceneManager:runWithSceneName(g_setting.sceneName)
end

function This:initGame()
	cc.Audio:init(self)
	require("Util.GMUtil")
    require("Player.GamePlayer"):init()
    require("Login.LoginManager"):init()
    require("Hero.HeroManager"):init()
    require("Friend.FriendManager"):init()
    require("Stage.GameStageManager"):init()
    require("Stage.ChallengeStageManager"):init()
    require("SingleDaily.SingleManager"):init()
    require("Mail.MailManager"):init()
    require("Chat.ChatManager"):init()
    require("Depot.DepotManager"):init()
    require("Equip.EquipManager"):init()
    require("Fabao.FabaoManager"):init()
    require("System.SystemManager"):init()
    require("Battle.BattleDataManager"):init()
    require("Report.ReportManager"):init()
    require("World.WorldMapManager"):init()
    require("Match.MatchManager"):init()
    require("PlanShop.PlanShopManager"):init()
    require("Guide.GuideManager"):init()
    require("Expedition.ExpeditionManager"):init()
    require("SingleDaily.TowerManager"):init()
    require("SingleDaily.WarTowerManager"):init()
    require("List.HeroListManager"):init()
    require("Store.StoreManager"):init()
    require("Vip.VipManager"):init()
    require("Sdk.SdkManager"):init()
    require("Recruit.RecruitManager"):init()
    require("Achieve.AchieveManager"):init()
    require("Achieve.VitalityManager"):init()
    require("Activity.ActivityManager"):init()
    require("Rank.RankManager"):init()
    require("Family.FamilyManager"):init()
    require("Achieve.DailyAchieveManager"):init()
    require("UI.PopDialogManager"):init()
    require("NewActivity.NewActivityManager"):init()
    require("Arena.ArenaManager"):init()
    require("Territory.TerritoryManager"):init()
    require("EventDispatcher"):init(self)
    require("RedDot.RedDotManager"):init()
    require("WorldBoss.WorldMonsterManager"):init()
    require("Warfare.WarfareManager"):init()
    require("Awake.GameKingWayManager"):init()
    require("PeakContest.PeakContestManager"):init()
	require("Map.GameMapManager"):init()
    require("Official.OfficialManager"):init()
    require("Hero.ArrayManager"):init()
    require("Function.FunctionNoticeManager"):init()
    require("RescueEmperor.RescueBoxManager"):init()
    require("Nobility.NobilityManager"):init()
    require("GodBeast.GodBeastManager"):init()
    require("ArmyTactical.ArmyTacticalManager"):init()
    require("Throne.ThroneManager"):init()
    require("Awake.LordEquipManager"):init()
    require("Palace.RescuePalaceManager"):init()
    require("Boss.PrincipalBossManager"):init()
    require("Competition.CompetitionManager"):init()
    require("LegionAboss.LegionAbossManager"):init()
    require("CrossServerBoss.CrossServerBossManager"):init()
    require("Invite.InviteManager"):init()
    require("Drum.DrumManager"):init()
    print("------------initGame-------------")
end

function This:addListener(listener)
    self.m_listenerList[#self.m_listenerList + 1] = listener
    return listener
end

function This:removeListener(listener)
    local list = self.m_listenerList
    for i = #list, 1, -1 do
        if list[i] == listener then
            table.remove(list, i)
            break
        end
    end
end

function This:callListener(eventType, ...)
    for _,listener in ipairs(self.m_listenerList) do
        local callback = listener[eventType]
        if callback then
            callback(listener, ...)
        end
    end
end

function This:getTimer()
    return self.m_timer
end

function This:onEnterBackground()
	cc.Audio:pauseBackground()
	cc.Audio:pauseAllEffect()
    self:callListener("onEnterBackground")
end

function This:onEnterForeground()
	cc.Audio:resumeBackground()
	cc.Audio:resumeAllEffect()
    self:callListener("onEnterForeground")
end

function This:addToUpdateList(listener)
	if not self.m_updateList:hasNode(listener) then
		self.m_updateList:addNode(listener)
	end
end

function This:removeFromUpdateList(listener)
	self.m_updateList:removeNode(listener)
end

function This:getServerTime()
    local GamePlayer = require("Player.GamePlayer")
    if GamePlayer:isEnterGame() then
        return GamePlayer:getServerTime()
    end
    return 0
end

function This:update(delta)
    if not self.m_isRunning then
        return false
    end
    if self.m_timer then
        self.m_timer:update(delta)
    end
    local second = nil
    -- self.m_escape = self.m_escape + delta
    -- if self.m_escape > 1 then
    --     second = self:getServerTime() % 86400
    --     if second ==0 then
    --         second = nil
    --     end
    --     self.m_escape = self.m_escape - 1
    -- end
	self.m_updateList:update(delta,second)
end

function This:onReload() 
    if package.loaded["Net.Socket"] then
        package.loaded["Net.Socket"]:close()
    end
    if package.loaded["Audio"] then
        package.loaded["Audio"]:stop()
    end
    
    cc.SkeletonNode.clearAllCharacterData()
    cc.SkeletonNode.clearAllEffectData()
    lib2d.clearCSNodeCache()
    
    if self:getReloadType() == "Full" then
        cc.SpriteFrameCache:removeAllPlist()
        cc.TextureCache:releaseAllTextures()
        lib2d.ccFontCache.clear()
    end
    if os.isWin32 then
        os.system_cls()
    end
	cc.SceneManager:switchTo("LoginScene")
end

function This:getReloadType()
    return lib_application.getReloadType(self[0])
end

function This:initConfigArg()
    local str = lib_application.getInitConfigArg(self[0])
    local ret = loadstring(str)
    print("initConfigArg , str = ",str)
    if ret then
        local arg = ret()
        for k,v in pairs(arg) do
            g_setting[k] = v
        end
    end
end

function This:getVersion()
    return lib_application.getVersion(self[0])
end

function This:getEnterCount()
    return lib_application.getEnterCount(self[0])
end

function This:reload(reloadType)
    if self.m_isReloading then
        return
    end
    lib_application.reload(self[0], reloadType or "nil")
end

function This:exit()
    if self.m_isExit then
        return
    end
    self.m_isExit = true
    
    lib_application.exit(self[0])
end

function This:onKeyUp(keyCode)
    self.m_pressKeys[keyCode] = nil
    if keyCode == "F5" then
        self:reload("Full")
        return
    elseif keyCode == "F8" then
        os.system_ex('python -c "import sys;sys.dont_write_bytecode = True;sys.path.append(\'数值源数据\');import ExportToLua;ExportToLua.main()"')
    elseif keyCode == "C" then
        if not lib2d.isControlDown() then
            os.system_cls()
        end
    elseif keyCode == "R" then
        if lib2d.isControlDown() then
            require("TestDebug")
            package.loaded["TestDebug"] = nil
        end
    elseif keyCode == "F4" then
		-- TODO
    elseif keyCode == "P" then
        print(cc.TextureCache:getCachedInfo())
    elseif keyCode == "ESC" then
        -- self:exit()
    elseif keyCode == "F12" then
		g_scene:addGetNodePathListener()
    elseif string.sub(keyCode, 1, 3) == "GM_" then
        local commond = string.sub(keyCode, 4, -1)
        ChatManager:onSendGmRequest(commond)
    elseif keyCode == "Q" then 

    end

    self:callListener("onKeyUp", keyCode)
end

function This:onKeyDown(keyCode)
    self:callListener("onKeyPress", keyCode)

    if self.m_pressKeys[keyCode] == 1 then
        return
    end
    self.m_pressKeys[keyCode] = 1
    
    self:callListener("onKeyDown", keyCode)
end

function This:dispatchEvent(event, args)
    if os.isAndroid then
        local xruntime = require "xgame.xruntime"
        xruntime.dispatchEvent(event, args)
    end
end
