local This = class(...)
cc.Audio = This
local audio = require("AudioEngineEx")
--[[if os.isAndroid then
    audio = require("cc.AndroidAudio")
elseif os.isWin32 then
    audio = require("cc.OpenAudio")
else
    audio = require("cc.SimpleAudio")
end]]

-- 某一时间段内最多能播放的次数
local defaultLimitInfo = {0.3, 2}
local limitAudios = {
    ["bomb.mp3"] = {0.2, 3},
    ["npc_shoot1.mp3"] = {0.2, 4},
    ["npc_attack2.mp3"] = {0.3, 2},
    ["skill2_baojian.mp3"] = {0.2, 3},
    ["boss8_skill4_1.mp3"] = {0.3, 1},
    ["npc_shoot3.mp3"] = {0.3, 3},
    ["npc_shoot5.mp3"] = {0.2, 2},
    ["npc_shoot8.mp3"] = {0.2, 4},
    ["npc_shoot2.mp3"] = {0.4, 1},
    ["boss_shoot.mp3"] = {0.15, 3},
    ["boss7_skill1.mp3"] = {5, 1},
    ["boss7_skill2.mp3"] = {5, 1},
    ["boss7_skill5.mp3"] = {5, 1},
    ["skill2002.mp3"] = {0.3, 3},
    ["skill2009.mp3"] = {0.3, 3},
    ["petskill4.mp3"] = {0.2, 2},
    ["npc_boom1.mp3"] = {0.4, 2},
}

local preLoadSound = 
{
	"Cg.mp3",
	"Man1-1.mp3",
	"Man1-3.mp3",
	"Man1-4.mp3",
	"Man1-5.mp3",
	"Man1-10.mp3",
	"Man1-11.mp3",
	"Man1-12.mp3",
    "Women-3.mp3",
	"Women-11.mp3",
    "Women-21.mp3",
    "Women-24.mp3",
    "Women-27.mp3",
	
}

function This:playEffectLimit(name, volume, rate)
    if not self.m_isOpenSound then
        return
    end
    if not name then
        return
    end
    local count = self.m_playingAudios[name] or 0
    local limitInfo = limitAudios[name] or defaultLimitInfo
    if count >= limitInfo[2] then
        return
    end
    count = count + 1
    self.m_playingAudios[name] = count
    local id = audio:playEffect(name, false, volume, rate)
    self.m_timer:setTimer(limitInfo[1], function ()
        self.m_playingAudios[name] = self.m_playingAudios[name] - 1
    end)
    return id
end

function This:init(application)
    audio:init()
    application:addToUpdateList(self)
    application:addListener(self)
    self.m_timer = require("Timer"):create()
    self.m_playingAudios = {}
    self.m_loopAudios = {}
    self.m_isOpenMusic = require("GameSetting").isOpenMusic
    self.m_isOpenSound = require("GameSetting").isOpenSound
    self.m_pauseSoundCount = 0
    self.m_pauseMusicCount = 0
	self.m_playTimes = {}
end

function This:preLoad()
    --[[for i , sound in ipairs(preLoadSound) do
		self:preloadEffect(sound)
	end	]]
end

function This:update(deltaTime)
    self.m_timer:update(deltaTime)
end

function This:playBackground(name)
    self.m_lastBackground = name
    if not self.m_isOpenMusic then
        return 
    end
    if self.m_background == name then
        return
    end
    if self.m_background then
        self:stopBackground()
    end
    self.m_background = name
    self.m_backgroundID = audio:playBackground(name)
end

function This:stopBackground()
    if self.m_backgroundID then
        audio:stopBackground(self.m_backgroundID)
        self.m_backgroundID = nil
        self.m_background = nil
    end
end

function This:pauseBackground()
    if not self.m_isOpenMusic then
        return
    end
    if self.m_backgroundID then
        audio:pauseBackground(self.m_backgroundID)
    end
end

function This:resumeBackground()
    if not self.m_isOpenMusic then
        return
    end
    if self.m_backgroundID then
        audio:resumeBackground(self.m_backgroundID)
    end
end

function This:preloadBackground(name)
    if not self.m_isOpenMusic then
        return
    end
    audio:preloadBackground(name)
end

function This:openBackground()
    self.m_isOpenMusic = true
    if self.m_lastBackground then
        self:playBackground(self.m_lastBackground)
    end
end

function This:closeBackground()
    self:stopBackground()
    self.m_isOpenMusic = false
end

function This:playEffect(name, isLoop, rate,num)
    if not name then
        return
    end
    if not self.m_isOpenSound then
        return 
    end

	num = num or 1
	self.m_playTimes[name] = self.m_playTimes[name] or 0

	if self.m_playTimes[name] < num then
		local id = audio:playEffect(name, isLoop or false, rate or 1)
		self.m_playTimes[name] = self.m_playTimes[name] + 1
		self.m_timer:setTimer(0.5,function()
			self.m_playTimes[name] = self.m_playTimes[name] - 1
		end,name)
        return id
	end
end

function This:stopEffect(id)
    if not id then
        return
    end
    audio:stopEffect(id)
end

function This:openAllEffect()
    self.m_isOpenSound = true
    audio:resumeAllEffect()
end

function This:closeAllEffect()
    self.m_isOpenSound = false
    audio:pauseAllEffect()
    --self:stopAllEffect()
end

function This:stopAllEffect()
    self:stopAllLoopEffect()
end

function This:preloadEffect(name)
    if not name then
        return
    end
    if self.m_isOpenSound then
        audio:preloadEffect(name)
    end
end

function This:unloadEffect(name)
    
end

function This:playLoopEffect(name)
    if not self.m_isOpenSound then
        return
    end
    if not name then
        return
    end
    local effectID = self.m_loopAudios[name]
    if effectID then
        return
    end
    self.m_loopAudios[name] = audio:playEffect(name, true)
end

function This:stopLoopEffect(name)
    if not name then
        return
    end
    local effectID = self.m_loopAudios[name]
    if not effectID then
        return
    end
    audio:stopEffect(effectID)
    self.m_loopAudios[name] = nil
end

function This:pauseLoopEffect(name)
    if not name then
        return
    end
    if not self.m_isOpenSound then
        return
    end
    local effectID = self.m_loopAudios[name]
    if not effectID then
        return
    end
    audio:pauseEffect(effectID)
end

function This:resumeLoopEffect(name)
    if not name then
        return
    end
    if not self.m_isOpenSound then
        return
    end
    local effectID = self.m_loopAudios[name]
    if not effectID then
        return
    end
    audio:resumeEffect(effectID)
end

function This:stopAllLoopEffect()
    for name,effectID in pairs(self.m_loopAudios) do
        self:stopLoopEffect(name)
    end
end

function This:pauseAllEffect()
    audio:pauseAllEffect()
end

function This:resumeAllEffect()
    audio:resumeAllEffect()
end

function This:pauseAllLoopEffect()
    if not self.m_isOpenSound then
        return
    end
    for name,effectID in pairs(self.m_loopAudios) do
        self:pauseLoopEffect(name)
    end
end

function This:resumeAllLoopEffect()
    if not self.m_isOpenSound then
        return
    end
    for name,effectID in pairs(self.m_loopAudios) do
        self:resumeLoopEffect(name)
    end
end

function This:pauseAll()
    self:pauseBackground()
    self:pauseAllEffect()
end

function This:resumeAll()
    self:resumeBackground()
    self:resumeAllEffect()
end

function This:clear()
    
end

function This:stop()
    self:stopBackground()
    self:stopAllEffect()
end