local This, super = class(..., "cc.ScrollView")

function This:onEnter()
    super.onEnter(self)
    self:addEventListener(function (self, eventType)
        if self.m_isListenerEnabled then
            self:checkVertical()
        end
    end)
end

function This:setInnerContainerSize(width, height)
    self.m_isListenerEnabled = false
    super.setInnerContainerSize(self, width, height)
    self.m_isListenerEnabled = true
end

function This:getInnerContainerHeight()
    local _, height = self:getInnerContainerSize()
    return height
end

function This:setInnerContainerPosition(x, y)
    self.m_isListenerEnabled = false
    super.setInnerContainerPosition(self, x, y)
    self.m_isListenerEnabled = true
end

function This:reset(onCreateCacheNode, onIndexUpdate)
    self.m_innerNode = self:getInnerContainer()
    self.m_contentWidth, self.m_contentHeight = self:getContentSize()
    self.m_itemMargin = self.m_itemMargin or 0
    self.m_cacheCount = self.m_cacheCount or 10
    self.m_isListenerEnabled = true
    self.m_onIndexUpdate = onIndexUpdate
    local innerHeight = (self.m_itemHeight + self.m_itemMargin) * self.m_itemAmount
    innerHeight = math.max(innerHeight, self.m_contentHeight)
    self:setInnerContainerSize(self.m_contentWidth, innerHeight)
    self.m_itemList = {}
    for i = 1, self.m_cacheCount do
        local node = onCreateCacheNode(i)
        node:setVisible(false)
        node:runActionByID("ListAppear", i * 0.1)
        self:pushItem(node)
        self:updateItem(node)
    end
    
end

function This:resetAmount(amount)
    self.m_itemAmount = amount
    self:stopAutoScoll()
    local innerHeight = (self.m_itemHeight + self.m_itemMargin) * self.m_itemAmount
    if innerHeight < self.m_contentHeight then
        innerHeight = self.m_contentHeight
        self:setInnerContainerSize(self.m_contentWidth, innerHeight)
        self:setInnerContainerPosition(0, 0)
        local topY = innerHeight - self.m_itemHeight * 0.5
        for i, node in ipairs(self.m_itemList) do
            local x, y = node:getPosition()
            node.index = i
            node:setPosition(x, topY - (self.m_itemHeight + self.m_itemMargin) * (i - 1))
            node:setVisible(i <= amount)
        end
    else
        local deltaHeight = innerHeight - self:getInnerContainerHeight()
        local innerX, innerY = self:getInnerContainerPosition()
        self:setInnerContainerSize(self.m_contentWidth, innerHeight)
        self:setInnerContainerPosition(0, -(innerHeight - self.m_contentHeight))
        local innerY2 = math.min(innerY - deltaHeight, 0)
        for i, node in ipairs(self.m_itemList) do
            local x, y = node:getPosition()
            node:setPosition(x, y + deltaHeight)
        end
        self:setInnerContainerPosition(innerX, innerY2)
    end
end

function This:jumpToIndex(index)
    local innerWidth, innerHeight = self:getInnerContainerSize()
    local bottomY = -(innerHeight - self.m_contentHeight)
    local y = bottomY + (self.m_itemHeight + self.m_itemMargin) * (index - 1)
    y = math.min(y, 0)
    self:setInnerContainerPosition(0, y)
end

function This:updateItem(node)
    if node.index <= self.m_itemAmount then
        node:setVisible(true)
        if self.m_onIndexUpdate then
            self.m_onIndexUpdate(node, node.index)
        end
    else
        node:setVisible(false)
    end
end

function This:updateAllItem()
    local onIndexUpdate = self.m_onIndexUpdate
    self.m_onIndexUpdate = false
    self:checkVertical()
    self.m_onIndexUpdate = onIndexUpdate
    for i, node in ipairs(self.m_itemList) do
        self:updateItem(node)
    end
end

function This:removeAllItems()
    self:stopAutoScoll()
    self:jumpToTop()
    self.m_itemList = {}
    self.m_cacheCount = false
    self.m_itemAmount = false
    self:removeAllChildren()
end

function This:checkVertical()
    if not self.m_itemList then
        return
    end
    if #self.m_itemList == 0 then
        return
    end
    local totalHeight = self.m_cacheCount * (self.m_itemHeight + self.m_itemMargin)

    while self:isTopItemOutBoundary() do
        local topItem = self.m_itemList[1]
        local x, y = topItem:getPosition()
        topItem:setPosition(x, y - totalHeight)
        table.remove(self.m_itemList, 1)
        table.insert(self.m_itemList, topItem)
        topItem.index = topItem.index + self.m_cacheCount
        self:updateItem(topItem)
    end

    while self:isBottomItemOutBoundary() do
        local bottomItem = self.m_itemList[#self.m_itemList]
        local x, y = bottomItem:getPosition()
        bottomItem:setPosition(x, y + totalHeight)
        table.remove(self.m_itemList, #self.m_itemList)
        table.insert(self.m_itemList, 1, bottomItem)
        bottomItem.index = bottomItem.index - self.m_cacheCount
        self:updateItem(bottomItem)
    end
end

function This:isTopItemOutBoundary()
    -- print("isTopItemOutBoundary")
    local topItem = self.m_itemList[1]
    if not topItem then
        return false
    end
    -- if topItem.index + self.m_cacheCount > self.m_itemAmount then
    --     return false
    -- end
    local maxY = self.m_contentHeight + self.m_itemHeight * 0.5 + self.m_itemMargin
    if self:getItemWorldY(topItem) < maxY then
        return false
    end
    return true
end

function This:isBottomItemOutBoundary()
    local bottomItem = self.m_itemList[#self.m_itemList]
    if not bottomItem then
        return false
    end
    if bottomItem.index - self.m_cacheCount < 1 then
        return false
    end
    local minY = -self.m_itemHeight * 0.5 - self.m_itemMargin
    if self:getItemWorldY(bottomItem) > minY then
        return false
    end
    return true
end

function This:getItemWorldY(node)
    local x, y = node:getPosition()
    local innerX, innerY = self:getInnerContainerPosition()
    return y + innerY
end

function This:pushItem(node)
    local index = #self.m_itemList + 1
    node.index = index
    self.m_itemList[index] = node
    local innerWidth, innerHeight = self:getInnerContainerSize()
    local margin = self.m_itemMargin
    local itemHeight = self.m_itemHeight
    local topY = innerHeight - itemHeight * 0.5
    node:setPosition(self.m_itemWidth * 0.5, topY - (itemHeight + margin) * (index - 1))
    self:addChild(node)
end