local This = class(...)
cc.GLProgramState = This
-- 0 ccPositionTextureColor_vert
-- 1 ccPositionTextureColor_noMVP_vert
local fshList = {}
function fshList.Gray()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

void main(void)
{
    vec4 c = texture2D(CC_Texture0, v_texCoord);
    gl_FragColor.xyz = vec3(0.2126*c.r + 0.7152*c.g + 0.0722*c.b);
    gl_FragColor.w = c.w;
    gl_FragColor *= v_fragmentColor;
}
]]
end
function fshList.Light()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

void main(void)
{
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    gl_FragColor.rgb = color.rgb * 1.5;
    gl_FragColor.a = color.a;
}
]]
end

function fshList.Light2()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

void main(void)
{
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    gl_FragColor = vec4(2.0*color.r, 2.0*color.g, 1.0*color.b, color.a);
}
]]
end

function fshList.hue()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
uniform vec3 u_rgb;


void main(void)
{
    vec3 rgb = u_rgb;
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    mat3 hue = mat3(rgb[0], rgb[1], rgb[2], rgb[1], rgb[2], rgb[0], rgb[2], rgb[0], rgb[1]);
    gl_FragColor.rgb = color.rgb * hue;
    gl_FragColor.a = color.a;
    gl_FragColor *= v_fragmentColor.a;
}
]]
end

function fshList.hueLight()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
uniform vec3 u_rgb;


void main(void)
{
    vec3 rgb = u_rgb;
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    mat3 hue = mat3(rgb[0], rgb[1], rgb[2], rgb[1], rgb[2], rgb[0], rgb[2], rgb[0], rgb[1]);
    gl_FragColor.rgb = color.rgb * hue * 1.5;
    gl_FragColor.a = color.a;
    gl_FragColor *= v_fragmentColor.a;
}
]]
end

function fshList.setColor()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
uniform vec3 u_rgb;


void main(void)
{
    vec3 rgb = u_rgb;
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    gl_FragColor.rgb = color.rgb * u_rgb;
    gl_FragColor.a = color.a;
    gl_FragColor *= v_fragmentColor.a;
}
]]
end

function fshList.setColorLight()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
uniform vec3 u_rgb;


void main(void)
{
    vec3 rgb = u_rgb;
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    gl_FragColor.rgb = color.rgb * u_rgb * 1.5;
    gl_FragColor.a = color.a;
    gl_FragColor *= v_fragmentColor.a;
}
]]
end

function fshList.iceColor()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

void main(void)
{
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    color *= vec4(0.1, 0.7, 1, 1);
    color.b += color.a * 0.4;
    color *= v_fragmentColor.a;
    gl_FragColor = color;
}
]]
end

function fshList.iceColorLight()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;


void main(void)
{
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    color *= vec4(0.1, 0.7, 1, 1);
    color.b += color.a * 0.4;
    color *= v_fragmentColor.a * 1.5;
    gl_FragColor = color;
}
]]
end

function fshList.highLight()
return [[
varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

void main()
{
    vec4 normal = texture2D(CC_Texture0, v_texCoord);
    float t = CC_Time[1] / 1.5;
    float flag = tan(t);
    float line = abs(v_texCoord.x - flag);
    if (line < 0.2){
    float flag2 = 1 - line;
    //vec2 rb = normal.rb;
    normal *= max(pow(flag2, 8) * 5, 1);
    //normal.gb = rb;
    }
    gl_FragColor = normal;
}

]]
end

function fshList.RadialBlur()
return [[
#ifdef GL_ES
precision mediump float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
uniform float u_factor;
void main()
{
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    vec2 dir = vec2(0.5, 0.5) - v_texCoord.xy;
    float dist = length(dir);
    if(dist > 0.005)
    {
        vec4 sumColor = vec4(0);
        dir = normalize(dir.xy);
        float step = 0.0;
        for (int i = 0; i < 28; ++i)
        {
            float unit = 1.0/400.0 * (dist / 0.5) * u_factor;
            step += unit;
            if(step >= dist)
            {
                step = dist;
            }
            sumColor += texture2D(CC_Texture0, v_texCoord + dir * step);
        }
        gl_FragColor = sumColor / 28.0;
    }
    else
    {
        gl_FragColor = color;
    }
}
]]
end

function fshList.Blur()
return [[
varying vec2 v_texCoord;
varying vec4 v_fragmentColor;
uniform float u_blur_x;
uniform float u_blur_y;
vec2 blurSize;
void main() {
    blurSize=vec2(u_blur_x,u_blur_y);
    vec4 sum = vec4(0.0);
    sum += texture2D(CC_Texture0, v_texCoord -4.0 * blurSize) * 0.05;
    sum += texture2D(CC_Texture0, v_texCoord -3.0 * blurSize) * 0.09;
    sum += texture2D(CC_Texture0, v_texCoord -2.0 * blurSize) * 0.12;
    sum += texture2D(CC_Texture0, v_texCoord -1.0 * blurSize) * 0.15;
    sum += texture2D(CC_Texture0, v_texCoord                 ) *0.16;
    sum += texture2D(CC_Texture0, v_texCoord +1.0 * blurSize) * 0.15;
    sum += texture2D(CC_Texture0, v_texCoord +2.0 * blurSize) * 0.12;
    sum += texture2D(CC_Texture0, v_texCoord +3.0 * blurSize) * 0.09;
    sum += texture2D(CC_Texture0, v_texCoord +4.0 * blurSize) * 0.05;
    sum.a = 1.0;
    gl_FragColor = sum * v_fragmentColor;
}]]
end

function fshList.Blur1()
return [[
varying vec2 v_texCoord;
varying vec4 v_fragmentColor;
uniform float blurRadius;

void main() {
    vec4 sum = vec4(0.0);
    float step = blurRadius / 10;
    float count = 0;
    for(float x = -blurRadius ; x <= blurRadius; x += step)
    {
        for(float y = -blurRadius ; y <= blurRadius; y += step)
        {
            vec2 v = v_texCoord + vec2(x,y);
            float weight = (blurRadius - abs(x)) * (blurRadius - abs(y));
            sum += texture2D(CC_Texture0, v) * weight;
            count += weight;
        }
    }
    sum = sum / count;
    gl_FragColor = sum * v_fragmentColor;
}]]
end

function fshList.Blur2()
return [[
varying vec2 v_texCoord;
varying vec4 v_fragmentColor;
void main() {
    float dist = abs(v_texCoord.x - 0.5) + abs(v_texCoord.y - 0.5);
    gl_FragColor = (vec4(1,0,0,1) * dist + texture2D(CC_Texture0,v_texCoord) * (1- dist)) * v_fragmentColor;
}]]
end

local ccColor = require("cc.Color")
local templates = {
    [1] =                   {vsh = 0, fsh = "Gray"},
    [2] =                   {vsh = 1, fsh = "Gray"},
    [3] =                   {vsh = 0, fsh = "Light"},
    ["GoldenLight"] =       {vsh = 1, fsh = "Light2"},
    ["NormalLight"] =       {vsh = 1, fsh = "Light"},
    ["Hue"] =               {vsh = 0, fsh = "hue"},
    ["SetColor"] =          {vsh = 0, fsh = "setColor"},
    ["PoisonColor"] =       {vsh = 0, fsh = "setColor",      rgb = ccColor(90, 255, 0), lightID = "PoisonColorLight"},
    ["PoisonColorLight"] =  {vsh = 0, fsh = "setColorLight", rgb = ccColor(90, 255, 0)},
    ["FireColor"] =         {vsh = 0, fsh = "setColor",      rgb = ccColor(255, 0, 0),  lightID = "FireColorLight"},
    ["FireColorLight"] =    {vsh = 0, fsh = "setColorLight", rgb = ccColor(255, 0, 0)},
    ["IceColor"] =          {vsh = 0, fsh = "iceColor", lightID = "IceColorLight"},
    ["IceColorLight"] =     {vsh = 0, fsh = "iceColorLight"},
    ["HighLight"]   =       {vsh = 1, fsh = "highLight"},
    ["RadialBlur"]    =     {vsh = 1, fsh = "RadialBlur", factor = 1},
    ["Blur"]    =           {vsh = 1, fsh = "Blur", blurx = 0.001, blury = 0.001},
    ["Blur1"] =             {vsh = 1, fsh = "Blur1", blurRadius = 0.01},
    ["Blur2"] =             {vsh = 1, fsh = "Blur2"},
}

function This:init()
    for id,template in pairs(templates) do
        template.id = id
    end
    self.m_shaderList = {}
end

local lib_glProgram = lib2d.ccGLProgram
local Action = cc.Action
local Shader = {
    create = function (self, template)
        local obj = {
            [0] = lib_glProgram.addGLprogramState(template.vsh, fshList[template.fsh]()),
            m_template = template,
            m_id = template.id,
            setUniformRGB = self.setUniformRGB,
            setUniformFloat = self.setUniformFloat,
            runAction = self.runAction,
            updateAction = self.updateAction,
        }
        self.init(obj)
        return obj
    end,
    init = function (self)
        local template = self.m_template
        if template.rgb then
            self:setUniformRGB("u_rgb", template.rgb)
        end
        if template.factor then
            self.setFactor = function (self, factor)
                lib_glProgram.setUniformFloat(self[0], "u_factor", factor)
            end
            self:setFactor(template.factor)
        end
        if template.blurx then
            self.setBlurx = function(self,blurx)
                lib_glProgram.setUniformFloat(self[0],"u_blur_x",blurx)
            end
            self:setBlurx(template.blurx)
        end

        if template.blury then
            self.setBlury = function(self,blury)
                lib_glProgram.setUniformFloat(self[0],"u_blur_y",blury)
            end
            self:setBlury(template.blury)
        end

        if template.blurRadius then
            self.setBlurRadius = function(self,radius)
                lib_glProgram.setUniformFloat(self[0],"blurRadius",radius)
            end
            self:setBlurRadius(template.blurRadius)
        end

        if template.lightID then
            self.m_light = This:getShader(template.lightID)
        end
    end,
    setUniformRGB = function (self, name, rgb)
        lib_glProgram.setUniformVec3(self[0], name, rgb.r / 255, rgb.g / 255, rgb.b / 255)
    end,
    setUniformFloat = function (self, name, value)
        lib_glProgram.setUniformFloat(self[0], name, value)
    end,
    setLightShader = function (self, lightID)
        self.m_light = This:getShader(lightID)
    end,
    runAction = function (self, type, easingID, duration, ...)
        self.m_action = Action:create(type, easingID, duration, ...)
    end,
    updateAction = function (self, deltaTime)
        local action = self.m_action
        if action then
            if Action.update(action, self, deltaTime) then
                self.m_action = false
            end
        end
    end,
}


function This:getShader(shaderID)
    local shader = self.m_shaderList[shaderID]
    if not shader then
        shader = Shader:create(templates[shaderID])
        self.m_shaderList[shaderID] = shader
    end
    return shader
end

function This:addHueShader(id, rgb)
    local shader = self.m_shaderList[id]
    if not shader then
        shader = Shader:create({id = id, vsh = 0, fsh = "hue", rgb = rgb})
        shader.m_light = Shader:create({id = "Light" .. id, vsh = 0, fsh = "hueLight", rgb = rgb})
        self.m_shaderList[id] = shader
    end
    return shader
end

local lib_node = lib2d.ccNode
function This:getNodeShader(node)
    return {
        [0] = lib_node.getGLProgramState(node[0]),
        m_light = This:getShader("NormalLight"),
        setUniformRGB = Shader.setUniformRGB,
        setLightShader = Shader.setLightShader,
    }
end

This:init()