local This = class(...)
local lib_glview = lib2d.ccGLView
local Vec = cc.Vec
local Rect = require("Rect")

function This:createWithRect(name, rect, frameZoomFactor)
    local glview = {[0] = lib_glview.createWithRect(name, rect[1], rect[2], rect[3], rect[4], frameZoomFactor or 1)}
    setmetatable(glview, self)
    return glview
end

function This:setDesignResolutionSize(width, height, resolutionPolicy)
    if resolutionPolicy == "ExactFit" then
        resolutionPolicy = 0
    elseif resolutionPolicy == "NoBorder" then
        resolutionPolicy = 1
    elseif resolutionPolicy == "ShowAll" then
        resolutionPolicy = 2
    elseif resolutionPolicy == "FixedHeight" then
        resolutionPolicy = 3
    elseif resolutionPolicy == "FixedWidth" then
        resolutionPolicy = 4
    elseif resolutionPolicy == "Unknown" then
        resolutionPolicy = 5
    else
        resolutionPolicy = 2
    end
    lib_glview.setDesignResolutionSize(self[0], width, height, resolutionPolicy)
end

function This:initDesignResolutionSize()
	--[[local appconfig = require("AppCofing")
	if os.isIos then
		local isPad = os.native_call("isPad")
		if isPad then
			print("-----------------isPad-------------------")
			appconfig.resolutionPolicy = "ShowAll"
			self:setDesignResolutionSize(appconfig.designSize[1],appconfig.designSize[2],appconfig.resolutionPolicy)
		end
	end]]
end

function This:initVisible()
    local appconfig = require("AppCofing")
    local x, y, width, height = lib_glview.getVisibleRect(self[0])
    local visible = {}
    cc.visible = visible
    visible.left = Vec(x, y + height / 2)
    visible.right = Vec(x + width, y + height / 2)
    visible.top = Vec(x + width / 2, y + height)
    visible.bottom = Vec(x + width / 2, y)
    visible.center = Vec(x + width / 2, y + height / 2)
    visible.leftTop = Vec(x, y + height)
    visible.rightTop = Vec(x + width, y + height)
    visible.leftBottom = Vec(x, y)
    visible.rightBottom = Vec(x + width, y)
    visible.width = width
    visible.height = height
    visible.x = x
    visible.y = y
    visible.viewScale = 1
    visible.frameWidth, visible.frameHeight = self:getFrameSize()
    visible.scaleX = self:getScaleX()
    visible.scaleY = self:getScaleY()
    visible.rect = Rect(0, 0, width, height)
    visible.worldRect = Rect(0, 0, width, height)
    visible.getSize = function (self)
        return self.width, self.height
    end
    print("glsize", width, height)
    visible.getCenterXY = function (self)
        return self.center.x, self.center.y
    end
    local winWidth, winHeight = cc.Director:getWinSize()
    visible.winWidth = winWidth
    visible.winHeight = winHeight
    visible.designWidth = appconfig.designSize[1]
    visible.designHeight = appconfig.designSize[2]
end

function This:getScaleX()
    return lib_glview.getScaleX(self[0])
end

function This:getScaleY()
    return lib_glview.getScaleY(self[0])
end

function This:getFrameSize()
    return lib_glview.getFrameSize(self[0])
end

function This:setFrameSize(width, height)
    return lib_glview.setFrameSize(self[0], width, height)
end