local This, super = class(..., "cc.Node")
cc.addCppNodeClass(This)
cc.Label = This

local TextAlignment = {
    left = 0,
    center = 1,
    right = 2,
    top = 0,
    center = 1,
    bottom = 2,
}

local VEC_ZERO = cc.Vec.ZERO
local SIZE_ZERO = {width = 0, height = 0}
local lib_label = lib2d.ccLabel
local registerObject = cc.registerObject
local tostring = tostring
local lib_createWithBMFont = lib_label.createWithBMFont

-- function This:createWithBMFont(fontName, text)
function This:createWithBMFont(fontName, text, hAlignment, maxLineWidth, imageOffset)
    if hAlignment == nil then
        hAlignment = 0
    else
        hAlignment = TextAlignment[hAlignment] 
    end

    if maxLineWidth == nil then
        maxLineWidth = 0
    end

    if imageOffset == nil then
        imageOffset = VEC_ZERO
    end
    
    local label = lib_createWithBMFont(fontName, tostring(text), hAlignment, maxLineWidth, imageOffset.x, imageOffset.y)
    return registerObject(label, self)
end

local ccLabelAtlas = class("cc.LabelAtlas", "cc.Node")
local lib_labelAtlas_createWithCharMap = lib2d.ccLabel.createWithCharMap
-- function This:createWithBMFont(imageName, text)
function This:createWithCharMap(imageName, text)
    fontName = "Fonts/" .. imageName

    local label = lib_labelAtlas_createWithCharMap(fontName, 48, tostring(text))
    label = registerObject(label, ccLabelAtlas)
    return label
end

local lib_createWithTTF = lib_label.createWithTTF
-- function This:createWithTTF(fontName, text, fontSize)
function This:createWithTTF(fontName, text, fontSize, dimensions, hAlignment, vAlignment)
    if hAlignment == nil then
        hAlignment = 0
    else
        hAlignment = TextAlignment[hAlignment] 
    end

    if vAlignment == nil then
        vAlignment = 0
    else
        vAlignment = TextAlignment[vAlignment] 
    end
    if dimensions == nil then
        dimensions = SIZE_ZERO
    end
    local label = lib_createWithTTF(fontName, tostring(text), fontSize, dimensions.width, dimensions.height, hAlignment, vAlignment)
    label = registerObject(label, self)
    return label
end

-- function This:createNormalTTF(text)
-- function This:createNormalTTF(text, fontSize)
function This:createNormalTTF(text, fontSize, dimensions, hAlignment, vAlignment)
    return self:createWithTTF("Fonts/Normal.ttf", text, fontSize or 24, dimensions, hAlignment, vAlignment)
end

-- function This:createEditorTTF(text, fontSize)
function This:createEditorTTF(text, fontSize, dimensions, hAlignment, vAlignment)
    return self:createWithTTF("Editor/Fonts/fzht.ttf", text, fontSize or 24, dimensions, hAlignment, vAlignment)
end

function This.hasLetter(text)
    local fontName = "Fonts/Normal.ttf"
    return lib2d.ccFontCache.hasLetter(fontName, text)
end


local lib_setString = lib_label.setString
function This:setString(text, maxLineWidth)
    lib_setString(self[0], tostring(text), maxLineWidth)
end

function This:setStringWithColor(text,maxLineWidth)
    lib_label.setStringWithColor(self[0], tostring(text), maxLineWidth)
end

local lib_getString = lib_label.getString
function This:getString() 
    return lib_getString(self[0])
end

local lib_getNextFontPosition = lib_label.getNextFontPosition
function This:getNextFontPosition() 
    return lib_getNextFontPosition(self[0])
end

local lib_enableOutline = lib_label.enableOutline
function This:enableOutline(r, g, b, a, size)
    lib_enableOutline(self[0], r or 0, g or 0, b or 0, a or 255, size or 1)
end

local lib_enableShadow = lib_label.enableShadow
-- function This:enableShadow()
function This:enableShadow(r, g, b, a, offsetX, offsetY, radius)
    lib_enableShadow(self[0], r or 110, g or 110, b or 110, a or 255, offsetX or 2, offsetY or -2, radius or 0)
end

local lib_labelAtlas_setString = lib2d.ccLabelAtlas.setString
function ccLabelAtlas:setString(text)
    lib_labelAtlas_setString(self[0], tostring(text))
end

local lib_labelAtlas_getString = lib2d.ccLabelAtlas.getString
function ccLabelAtlas:getString(text)
    return lib_labelAtlas_getString(self[0])
end
