﻿local This = class(...)
local cc = cc
cc.addCppNodeClass(This)
cc.Node = This

This.update = false
This._schedule = false
This._actions = false
local ccTouchListener = require("cc.TouchListener")
local Vec = cc.Vec
local Rect = cc.Rect
local lib_node = lib2d.ccNode
local autoChildClassList = {}
local objectRegistry = cc.objectRegistry
local registerObject = cc.registerObject
local createFromCSFile = lib_node.createFromCSFile
local getActionTimeline = lib_node.getActionTimeline
local getChildByName = lib_node.getChildByName
local getObjectTypeID = lib2d.getObjectTypeID
local cppNodeClassList = cc.cppNodeClassList
local lib_ref = lib2d.ccRef
local lib_action = lib2d.ccAction

local function getAutoChildClass(node, className)
    className = className or lib_node.getCustomClassName(node)
    if className and cc[className] then
        return cc[className]
    end
    local typeID = className or getObjectTypeID(node)
    local autoChildClass = autoChildClassList[typeID]
    if autoChildClass == nil then
        local nodeClass = nil
        if className then
            nodeClass =  require(className)
        else
            nodeClass = cppNodeClassList[typeID]
        end
        if not nodeClass then
            error("could not found cocos2d class:" .. lib2d.getObjectTypeName(node))
        end
        autoChildClass = {_MNAME = nodeClass._MNAME, _THIS = nodeClass, super = nodeClass.super, init = nodeClass.init}
        autoChildClassList[typeID] = autoChildClass
        autoChildClass.__index = function (self, k)
            local v = nodeClass[k]
            if v ~= nil then
                return v
            end
            local child = getChildByName(self[0], k)
            if child ~= nil then
                obj = objectRegistry[child]
                if obj == nil then
                    local childClass = getAutoChildClass(child)
                    child = registerObject(child, childClass, {m_actionTimeline = getActionTimeline(child), })
                else
                    child = obj
                end
                self[k] = child
                return child
            end
            return nil
        end
    end
    return autoChildClass
end
This.getAutoChildClass = getAutoChildClass

-- function This:create()
function This:create(...)
    local node = self:createNode(...)
    node = registerObject(node, self, {[0] = 0, _schedule = false, _actions = false})
	if node:getName() == "" then
        node:setName(self._NAME)
    end
    if node.init then
        node:init(...)
    end
    return node
end

function This:createFromCSFile(fileName, className)
    -- print(fileName)
    local node, customNodeList = createFromCSFile(fileName)
    if className == -1 then
        node = registerObject(node, self)
    else
        if customNodeList then
            for i = 1, #customNodeList do
                local customNode = customNodeList[i]
                if customNode ~= node then
                    local customNodeClass = getAutoChildClass(customNode)
                    customNodeList[i] = registerObject(customNode, customNodeClass, {[0]=0, m_actionTimeline = getActionTimeline(customNode)})
                end
            end
            for i = 1, #customNodeList do
                local customNode = customNodeList[i]
                if customNode ~= node and customNode.init then
                    customNode:init()
                end
            end
        end
        node = registerObject(node, getAutoChildClass(node, className), {[0]=0, m_actionTimeline = getActionTimeline(node)})
    end
    node:setName("m_csNode")
    return node
end

function This:fitScreen(mode)
    if mode == 1 then
        self:setContentSize(cc.visible.width, cc.visible.height)
        lib2d.ccui.doLayout(self[0])
    else
        local w, h = self:getContentSize()
        local scale = math.max(cc.visible.frameWidth / w, cc.visible.frameHeight / h) / cc.visible.scaleX
        self:setScale(scale, scale)
    end
end

function This:fitScreenX()
    local w , h = self:getContentSize()
    local scale = cc.visible.width / w
    self:setScaleX(scale)
end

function This:fitScreenY()
	local w , h = self:getContentSize()
	local scale = cc.visible.height / h
	self:setScale(scale)
end

local lib_create = lib_node.create
function This:createNode()
    return lib_create()
end

function This:retain()
    lib_node.retain(self[0])
end

function This:release()
    lib_node.release(self[0])
end

local lib_getPosition = lib_node.getPosition
function This:getPosition()
    return lib_getPosition(self[0])
end

local lib_getPositionX = lib_node.getPositionX
function This:getPositionX()
    return lib_getPositionX(self[0])
end

local lib_getPositionY = lib_node.getPositionY
function This:getPositionY()
    return lib_getPositionY(self[0])
end

local lib_setPosition = lib_node.setPosition
function This:setPosition(x, y)
    lib_setPosition(self[0], x, y)
end

local lib_addPosition = lib_node.addPosition
function This:addPosition(x, y)
    lib_addPosition(self[0], x, y)
end

local lib_setPositionX = lib_node.setPositionX
function This:setPositionX(x)
    lib_setPositionX(self[0], x)
end

local lib_setPositionY = lib_node.setPositionY
function This:setPositionY(y)
    lib_setPositionY(self[0], y)
end

function This:onEnter()
end

function This:onExit()
    if not self then return end 
    if self._schedule then
        self._schedule[self[0]] = nil
        self._schedule = nil
    end
    if self._actions then
        local _actions = self._actions
        self._actions = false
        for i = 1, #_actions do
            lib_ref.release(_actions[i])
        end
    end
end

-- function This:addChild(node)
local lib_addChild = lib_node.addChild
function This:addChild(node, zOrder, name)
    lib_addChild(self[0], node[0], zOrder or 0, name)
    -- if lib_node.isRunning(self[0]) then
    --     node:onEnter()
    -- end
end

-- function This:addChildToCenter(node)
function This:addChildToCenter(node, zOrder, name)
    lib_addChild(self[0], node[0], zOrder or 0, name)
    local w, h = self:getContentSize()
    node:setPosition(w * 0.5, h * 0.5)
end

-- function This:removeFromParent()
function This:removeFromParent(isCleanUp)
    if isCleanUp == nil then
        isCleanUp = true
    end
    -- if lib_node.isRunning(self[0]) then
    --     self:onExit()
    -- end
    lib_node.removeFromParent(self[0], isCleanUp)
    if isCleanUp then
        self[0] = 0
    end
end


local function castNode(node, isAutoChild)
    if node == nil then
        return nil
    end
    obj = objectRegistry[node]
    if obj == nil then
		local key = getObjectTypeID(node)
        local classObject = cppNodeClassList[key]
        if classObject == nil then
            error("could not found cocos2d class:" .. lib2d.getObjectTypeName(node))
        end
        if isAutoChild then
            classObject = getAutoChildClass(node)
        end
        return registerObject(node, classObject)
    end
    return obj
end
This.castNode = castNode
This.castTempNode = function (node)
    return setmetatable({[0] = node}, cppNodeClassList[getObjectTypeID(node)])
end

function This:getChildByName(name)
    local child = getChildByName(self[0], name)
    if child ~= nil then
        return castNode(child)
    end
    return nil
end

function This:getChildByTag(tag)
    local child = lib_node.getChildByTag(self[0], tag)
    if child ~= nil then
        return castNode(child)
    end
    return nil
end

function This:getChildrenCount()
    return lib_node.getChildrenCount(self[0])
end

-- function This:getChildren()
function This:getChildren(isAutoChild)
    local children = lib_node.getChildren(self[0])
    for i = 1, #children do
        children[i] = castNode(children[i], isAutoChild)
    end
    return children
end

function This:getChildrenNames()
    local children = lib_node.getChildrenNames(self[0])
    for k,v in pairs(children) do
        children[k] = castNode(v)
    end
    return children
end

local lib_setRotation = lib_node.setRotation
function This:setRotation(rotation)
    lib_setRotation(self[0], rotation)
end

local lib_getRotation = lib_node.getRotation
function This:getRotation()
    return lib_getRotation(self[0])
end

local lib_setScale = lib_node.setScale
local lib_setScaleXY = lib_node.setScaleXY

function This:setScale(scaleX, scaleY)
    if scaleY then
        lib_setScaleXY(self[0], scaleX, scaleY)
    else
        lib_setScale(self[0], scaleX)
    end
end

local lib_setScaleX = lib_node.setScaleX
function This:setScaleX(scale)
    lib_setScaleX(self[0], scale)
end

local lib_setScaleY = lib_node.setScaleY
function This:setScaleY(scale)
    lib_setScaleY(self[0], scale)
end

local lib_getScaleX = lib_node.getScaleX
function This:getScaleX()
    return lib_getScaleX(self[0])
end

local lib_getScaleY = lib_node.getScaleY
function This:getScaleY()
    return lib_getScaleY(self[0])
end

local lib_getScale = lib_node.getScale
function This:getScale()
    return lib_getScale(self[0])
end

local lib_runAction = lib_node.runAction
function This:runAction(action)
    lib_runAction(self[0], action)
end

local lib_stopAction = lib_node.stopAction
function This:stopAction(action)
    lib_stopAction(self[0], action)
end

local lib_stopAllActions = lib_node.stopAllActions
function This:stopAllActions()
    lib_stopAllActions(self[0])
end

local lib_addEventListener = lib_node.addEventListener
--function This:addEventListener(listener)
function This:addEventListener(listener, priority)
    lib_addEventListener(self[0], listener[0], priority)
end

local lib_removeEventListener = lib_node.removeEventListener
function This:removeEventListener(listener)
    lib_removeEventListener(self[0], listener[0])
end

function This:addTouchListener(touch)
    local listener = ccTouchListener:create(touch)
    self:addEventListener(listener)
    return listener
end

function This:setTouchForMask(touch)
    local listener = ccTouchListener:create(touch)
    self:addEventListener(listener)
    return listener
end

function This:setTouch(touch)
    if os.isWin32 then
        assert(not rawget(self, "m_touch"), "could not set touch again")
    end 
    if touch.sound == nil then
        touch.sound = "Btn.mp3"
    end
    local width, height = self:getContentSize(self[0])
    if not touch.sizeNode and (width < 5 or height < 5) then
        touch.sizeNode = self.bg or self.normal
        if not touch.sizeNode and (touch.rect or touch.getRect) then
            touch.sizeNode = self
        end
        if not touch.sizeNode then
            local parent = self:getParent()
            local nodeTreeName = self:getName()
            while parent do
                nodeTreeName = nodeTreeName .. "<-" .. parent:getName()
                parent = parent:getParent()
            end
            error("cc.Node not size:" .. nodeTreeName)
            touch.sizeNode = self
        end
    end
    if not touch.sizeNode then
        touch.sizeNode = self
    end
    touch.node = self
    local listener = ccTouchListener:create(touch)
    self:addEventListener(listener)
    self.m_touch = listener
    return listener
end

function This:setTouchEnded(onEnded, sizeNode)
    return self:setTouch({onEnded = onEnded, sizeNode = sizeNode})
end

function This:setAngle(angle)
    self:setRotation(-angle)
end

local lib_getName = lib_node.getName
function This:getName()
    return lib_getName(self[0])
end

local lib_setName = lib_node.setName
function This:setName(name)
    lib_setName(self[0], name)
end

local lib_getParent = lib_node.getParent
function This:getParent()
    local parent = lib_getParent(self[0])
    if parent ~= nil then
        return castNode(parent)
    end
    return nil
end

local lib_convertToNodeSpace = lib_node.convertToNodeSpace
function This:convertToNodeSpace(x, y)
    return lib_convertToNodeSpace(self[0], x or 0, y or 0)
end

local lib_convertToWorldSpace = lib_node.convertToWorldSpace
function This:convertToWorldSpace(x, y)
    return lib_convertToWorldSpace(self[0], x or 0, y or 0)
end

local lib_convertToNodeSpaceAR = lib_node.convertToNodeSpaceAR
function This:convertToNodeSpaceAR(x, y)
    return lib_convertToNodeSpaceAR(self[0], x or 0, y or 0)
end

local lib_convertToWorldSpaceAR = lib_node.convertToWorldSpaceAR
function This:convertToWorldSpaceAR(x, y)
    return lib_convertToWorldSpaceAR(self[0], x or 0, y or 0)
end

function This:pause()
    lib_node.pause(self[0])
end

function This:resume()
    lib_node.resume(self[0])
end

local lib_ignoreAnchorPointForPosition = lib_node.ignoreAnchorPointForPosition
function This:ignoreAnchorPointForPosition(isIgnore)
    lib_ignoreAnchorPointForPosition(self[0], isIgnore)
end

local lib_setLocalZOrder = lib_node.setLocalZOrder
function This:setLocalZOrder(zorder)
    lib_setLocalZOrder(self[0], zorder)
end

local lib_getLocalZOrder = lib_node.getLocalZOrder
function This:getLocalZOrder()
    return lib_getLocalZOrder(self[0])
end

function This:setTag(tag)
    lib_node.setTag(self[0], tag)
end

function This:getTag(tag)
    lib_node.getTag(self[0])
end

function This:setChildrenVisible(isVisible)
    lib_node.setChildrenVisible(self[0], isVisible)
end

local lib_setVisible = lib_node.setVisible
function This:setVisible(isVisible)
    lib_setVisible(self[0], isVisible)
end

local lib_isVisible = lib_node.isVisible
function This:isVisible()
    return lib_isVisible(self[0])
end

function This:scheduleUpdate(owner)
    local updateList = owner and owner.m_updateList or g_scene.m_updateList
    if updateList ~= self._schedule then
        self:unscheduleUpdate()
        updateList[self[0]] = self
        self._schedule = updateList
    end
end

function This:unscheduleUpdate()
    if self._schedule then
        self._schedule[self[0]] = nil
        self._schedule = false
    end
end

local lib_isRunning = lib_node.isRunning
function This:isRunning()
    return lib_isRunning(self[0])
end

local lib_getContentSize = lib_node.getContentSize
function This:getContentSize()
    return lib_getContentSize(self[0])
end

function This:getContentWidth()
    local w = lib_getContentSize(self[0])
    return w
end

function This:getContentHeight()
    local _,h = lib_getContentSize(self[0])
    return h
end

local lib_setContentSize = lib_node.setContentSize
function This:setContentSize(width, height)
    lib_setContentSize(self[0], width, height)
end

local lib_setOpacity = lib_node.setOpacity
function This:setOpacity(alpha)
    lib_setOpacity(self[0], alpha)
end

local lib_getOpacity = lib_node.getOpacity
function This:getOpacity()
    return lib_getOpacity(self[0])
end

local lib_setCascadeOpacityEnabled = lib_node.setCascadeOpacityEnabled
function This:setCascadeOpacityEnabled(isEnabled)
    lib_setCascadeOpacityEnabled(self[0], isEnabled)
end

function This:getColor()
    return lib_node.getColor(self[0])
end

local lib_setColor = lib_node.setColor
function This:setColor(r, g, b)
    lib_setColor(self[0], r or 255, g or 255, b or 255)
end

function This:setColorHex(str)
    local r, g, b = cc.Color.hexToRGB(str)
    lib_setColor(self[0], r, g, b)
end

function This:setColorHex2(str)
    if type(str) == "string" then 
        str = string.gsub(str,"#","")
        self:setColorHex(tonumber(str,16))
    else
        self:setColorHex(str)
    end
end

function This:setColorByID(id)
    local color = cc.ColorList[id]
    lib_setColor(self[0], color.r or 255, color.g or 255, color.b or 255)
end

local lib_getAnchorPoint = lib_node.getAnchorPoint
function This:getAnchorPoint()
    return lib_getAnchorPoint(self[0])
end

local lib_setAnchorPoint = lib_node.setAnchorPoint
function This:setAnchorPoint(x, y) 
    lib_setAnchorPoint(self[0], x, y)
end

local lib_applyAnchorPoint = lib_node.applyAnchorPoint
function This:applyAnchorPoint(x, y)
    lib_applyAnchorPoint(self[0], x, y)
end

local lib_timeline_play = lib2d.ccActionTimeline.play
local lib_timeline_gotoFrameAndPlay = lib2d.ccActionTimeline.gotoFrameAndPlay
local lib_timeline_gotoFrameAndPause = lib2d.ccActionTimeline.gotoFrameAndPlay
local lib_timeline_setEndListener = lib2d.ccActionTimeline.setEndListener
-- function This:playCSAnimation()
function This:playCSAnimation(name, isLoop, onEndListener)
    if isLoop == nil then
        isLoop = true
    end
    local actionTimeline = self.m_actionTimeline
    if actionTimeline ~= nil then
        if name then
            lib_timeline_play(actionTimeline, name, isLoop)
        else
            lib_timeline_gotoFrameAndPlay(actionTimeline, 0, isLoop)
        end
        
         lib_timeline_setEndListener(actionTimeline, onEndListener)
    end
end

function This:gotoFrameAndPause(index)
    local actionTimeline = self.m_actionTimeline
    if actionTimeline then
        lib_timeline_gotoFrameAndPause(actionTimeline, index)
    end
end

function This:removeAllChildren()
    lib_node.removeAllChildren(self[0])
end

local GLProgramState = require("cc.GLProgramState")
local lib_setGLProgramState = lib_node.setGLProgramState
local function setShader(self, shader)
    self.m_glProgramState = shader[0]
    lib_setGLProgramState(self[0], shader[0])
    self.m_shader = shader
end
-- function This:setShader(shader)
This.setShader = setShader

function This:setShaderByID(shaderID)
    if not shaderID then
        self:clearShader()
        return
    end

    if not self.m_normalShader then
        self.m_normalShader = self:getNomalShader()
    end

    local shader = self.m_shader
    if shader.m_id ~= shaderID then
        setShader(self, GLProgramState:getShader(shaderID))
        return self.m_shader
    end
end

function This:getNomalShader()
    if not self.m_normalShader then
        self.m_normalShader = GLProgramState:getNodeShader(self)
        self.m_shader = self.m_normalShader
        self.m_glProgramState = self.m_normalShader[0]
    end
    return self.m_normalShader
end

function This:resetToNormalShader()
    if not self.m_normalShader then
        return
    end
    local shader = self.m_shader
    local isLight = shader and shader.m_light and self.m_glProgramState == shader.m_light[0]
    self:setShader(self.m_normalShader)
    if isLight then
        self:setGLProgramState(self.m_normalShader.m_light[0])
    end
end

function This:showLightShader()
    if not self.m_shader then
        self:getNomalShader()
    end
    self:setGLProgramState(self.m_shader.m_light[0])
end

function This:setHueAsNormalShader(id, rbg)
    local shader = GLProgramState:addHueShader(id, rbg)
    self.m_normalShader = shader
    self:setShader(shader)
end

function This:clearShader()
    if self.m_normalShader then
        setShader(self, self.m_normalShader)
    end
end

function This:setGLProgramState(glProgramState)
    print("glProgramState:" ..tostring(glProgramState))
    if self.m_glProgramState ~= glProgramState then
        self.m_glProgramState = glProgramState
        lib_setGLProgramState(self[0], glProgramState)
    end
end

function This:setGrayShader()
    self:setShaderByID(1)
end

local lib_isRootVisible = lib_node.isRootVisible
function This:isRootVisible()
    return lib_isRootVisible(self[0])
end

local lib_getBoundingBox = lib_node.getBoundingBox
function This:getBoundingBox()
    return Rect(lib_getBoundingBox(self[0]))
end

local lib_getWorldRect = lib_node.getWorldRect
function This:getWorldRect(targetName)
    return Rect(lib_getWorldRect(self[0], targetName))
end

local lib_getWorldScale = lib_node.getWorldScale
function This:getWorldScale()
    return lib_getWorldScale(self[0])
end

function This:setCSAnimationSpeed(speed)
    lib2d.ccActionTimeline.setTimeSpeed(self.m_actionTimeline, speed)
end

local clamp = math.clamp
local random = math.random
local function updateShake(self, deltaTime)
    if not self.m_shakeList then
        return
    end
    local maxX = -1
    local maxY = -1
    for k,shake in pairs(self.m_shakeList) do
        shake[3] = shake[3] + deltaTime
        if shake[3] >= shake[4] then
            self.m_shakeList[k] = nil
            self.m_shakeCount = self.m_shakeCount -1
        else
            local ratio = clamp((shake[4] - shake[3]) / shake[4], 0, 1)
            local shakeX, shakeY = shake[1], shake[2]
            local randX = shakeX * ratio
            local randY = shakeY * ratio
            if randX >= maxX then
                maxX = randX
            end
            if randY >= maxY then
                maxY = randY
            end
        end
    end
    if maxX > 0 and maxY > 0 then
        self:setPosition(self.m_initX + random(-maxX, maxX), self.m_initY + random(-maxY, maxY))
    end
end


function This:shake(x, y, shakeTime)
    if not self.m_shakeList then
        self.m_shakeList = {}
        self.m_shakeKey = 0
        self.m_shakeCount = 0
        self.m_initX, self.m_initY = self:getPosition()
        local originUpdate = self.update
        if originUpdate then
            self.update = function (self, delta)
                if originUpdate then
                    originUpdate(self, delta)
                end
                updateShake(self, delta)
            end
        else
            self.update = updateShake
        end
        self:scheduleUpdate() 
    end
    self.m_shakeCount = self.m_shakeCount + 1
    self.m_shakeKey = self.m_shakeKey + 1
    local shakeInfo = {x, y, 0, shakeTime}
    self.m_shakeList[self.m_shakeKey] = shakeInfo
end

function This:setSelected(isSelected)
    self.m_isSelected = isSelected
    self.selected:setVisible(isSelected)
end

function This:switchParent(parent)
    self:retain()
    local ptr = self[0]
    self:removeFromParent(false)
    self[0] = ptr
    parent:addChild(self)
    self:release()
end

function This:setToCenter(sizew,sizeh)
    self:setPosition(cc.visible.center.x, cc.visible.center.y)
	local scale = math.max(960 / cc.visible.width, 540 / cc.visible.height)
	self:setScale(1/scale)
end

local lib_visit = lib_node.visit
function This:visit()
    lib_visit(self[0])
end

function This:runActionByID(actionID, arg)
    if actionID == "ZoomInOut" then
        local scale = arg or 1
        self:stopAllActions()
        self:setVisible(true)
        self:setScale(scale)
        self:setOpacity(0)
        self:runAction(cc.FadeIn(0.2))
        self:runAction(cc.RepeatForever(cc.Sequence(cc.ScaleTo(0.5, scale - 0.1), cc.DelayTime(0.1), cc.ScaleTo(0.5, scale), cc.DelayTime(0.15))))
    elseif actionID == "Appear" then
        self:stopAllActions()
        self:setVisible(true)
        self:setOpacity(0)
        self:setScale(0.1)
        self:runAction(cc.FadeIn(0.2))
        self:runAction(cc.ScaleTo(0.6, arg or 1, "BackOut"))
    elseif actionID == "Appear2" then
        self:stopAllActions()
        self:setVisible(true)
        self:setOpacity(50)
        self:setScale(1.2)
        self:runAction(cc.FadeIn(0.2))
        self:runAction(cc.ScaleTo(0.2, arg or 1, "BackIn"))
    elseif actionID == "AppearThenDisappear" then
        self:stopAllActions()
        self:setVisible(true)
        self:setOpacity(0)
        self:setScale(0.1)
        self:runAction(cc.FadeIn(0.2))
        self:runAction(cc.Sequence(cc.ScaleTo(0.3, 1, "BackOut"), cc.DelayTime(arg or 1), cc.Spawn(cc.ScaleTo(0.3, 1, "BackIn"), cc.FadeOut(0.2)), cc.RemoveAction()))
    elseif actionID == "Select" then
        self:stopAllActions()
        self:setVisible(true)
        self:setOpacity(50)
        self:setScale(1.2)
        self:runAction(cc.Sequence(cc.Spawn(cc.FadeIn(0.2), cc.ScaleTo(0.2, arg or 1, "BackIn")), cc.DelayTime(0.1), cc.CallFunc(function ()
            self:runAction(cc.RepeatForever(cc.Sequence(cc.ScaleTo(0.5, 1.1), cc.DelayTime(0.1), cc.ScaleTo(0.5, 1), cc.DelayTime(0.1))))
        end)))
    elseif actionID == "Blink" then
        self:stopAllActions()
        self:setVisible(true)
        self:runAction(cc.Blink(0.3, 3))
    elseif actionID == "BlinkThenDisappear" then
        self:stopAllActions()
        self:setVisible(true)
        self:runAction(cc.Sequence(cc.DelayTime(arg or 0.3), cc.FadeIn(0.2), cc.Spawn(cc.Blink(0.3, 3), cc.FadeOut(0.2)), cc.CallFunc(function() self:setVisible(false) end)))
    elseif actionID == "ListAppear" then
        self:stopAllActions()
        self:setVisible(true)
        self:setOpacity(0)
        self:setScale(0.1)
        self:runAction(cc.Sequence(cc.DelayTime(arg or 0.1), cc.Spawn(cc.FadeIn(0.2), cc.ScaleTo(0.3, 1, "BackOut"))))
    elseif actionID == "Rock" then
        self:stopAllActions()
        self:setVisible(true)
        local scale = self:getScale()
        self:runAction(cc.RepeatForever(cc.Sequence(cc.ScaleTo(0.5, scale + 0.05), cc.DelayTime(0.1), cc.ScaleTo(0.5, scale), cc.DelayTime(0.1))))
        self:runAction(cc.RepeatForever(cc.Sequence(cc.RotateTo(0.1, 8), cc.RotateTo(0.1, 10), cc.RotateTo(0.1, 0), cc.RotateTo(0.1, 10), cc.RotateTo(0.1, 8), cc.DelayTime(0.1))))
    elseif actionID == "Smash" then
        self:stopAllActions()
        self:setVisible(true)
        self:setOpacity(50)
        self:setScale(3)
        self:runAction(cc.FadeIn(0.2))
        self:runAction(cc.ScaleTo(0.3, 1, "BackIn"))
    elseif actionID == "WaftUpDown" then
        self:stopAllActions()
        self:setVisible(true)
        self:runAction(cc.RepeatForever(cc.Sequence(cc.DelayTime(0.05), cc.MoveBy(1, 0, 14), cc.DelayTime(0.05), cc.MoveBy(1, 0, -14), cc.DelayTime(0.05))))
    elseif actionID == "Blink1" then
        self:stopAllActions()
        self:setVisible(true)
        self:runAction(cc.Sequence(cc.Blink(1.2, 5),cc.CallFunc(function() self:setVisible(false) end)))
    elseif actionID == "Roll" then
        self:stopAllActions()
        self:setVisible(true)
        local action = cc.Sequence(
            cc.RotateTo(0.1, -8,"SineInOut"), cc.RotateTo(0.1, -10,"SineInOut"), cc.RotateTo(0.2, 0,"SineInOut"),
            cc.RotateTo(0.1, 8,"SineInOut"), cc.RotateTo(0.1, 10,"SineInOut"), cc.RotateTo(0.2, 0,"SineInOut"), 
            cc.RotateTo(0.1, -8,"SineInOut"), cc.RotateTo(0.1, -10,"SineInOut"), cc.RotateTo(0.2, 0,"SineInOut"),
            cc.DelayTime(1))

        self:runAction(cc.RepeatForever(action))
    end
end

local lib_clone = lib_node.clone
function This:clone()
    local node = lib_clone(self[0])
    return registerObject(node, getmetatable(self))
end

local lib_setOriginScale = lib_node.setOriginScale
-- function This:setOriginScale(scale)
function This:setOriginScale(scaleX, scaleY)
    lib_setOriginScale(self[0], scaleX, scaleY)
end

local function _runAction(self, action)
    local actions = self._actions
    if not actions then
        actions = {}
        self._actions = actions
    end
    actions[#actions + 1] = action
    lib_ref.retain(action)
    lib_action.startWithTarget(action, self[0])
end

local function _updateAction(self, deltaTime)
    if self.__update then
        self:__update(deltaTime)
    end
    local actions = self._actions
    if not actions then
        return
    end
    local i, n = 1, #actions
    while n > 0 and self._actions do
        if lib_action.step(actions[i], deltaTime) and self._actions then
            lib_ref.release(actions[i])
            table.remove(actions, i)
        else
            i = i + 1
        end
        n = n - 1
    end
end
function This:setActionRunWith(target)
    self.__update = self.update
    self.runAction = _runAction
    self.update = _updateAction
    self:scheduleUpdate(target)
end

function This:fitToScreen(bounderx,boundery)
    local parent = self:getParent()
    if not parent then
        return
    end

	local x , y = self:getPosition()
	local dx , dy = cc.visible.frameWidth / cc.visible.scaleX - cc.visible.designWidth  , cc.visible.frameHeight / cc.visible.scaleY - cc.visible.designHeight
	if bounderx == "right" then
		x = x + dx
	end
    if bounderx == "center" then
        x = parent:convertToNodeSpace(cc.visible.winWidth/2,0)
    end
    if boundery == "center" then
        _ , y = parent:convertToNodeSpace(0,cc.visible,winHeight/2)
    end
	self:setPosition(x, y)
end

function This:recoveryToFixedWidth(bx,by)
    local hScale = cc.visible.frameHeight / cc.visible.designHeight
    local wScale = cc.visible.frameWidth / cc.visible.designWidth

    local x , y = self:getPosition()
    local scale = 1 / hScale * wScale

    if hScale > wScale then    
        self:setScale(scale)
        local wx , wy = self:getParent():convertToWorldSpace(x, y)
        if bx == "left" then
            self:setPositionX( x * scale)
        elseif bx == "right" then
            self:setPositionX(cc.visible.width - (cc.visible.designWidth- wx) * scale)
        end
        if by == "top" then
            self:setPositionY(cc.visible.height - (cc.visible.designHeight- wy) * scale)
        elseif by == "bottom" then
            self:setPositionY(y * scale)
        end
    else
        if bx == "right" then
            x , y = self:getParent():convertToWorldSpace(x, y)
            self:setPositionX(cc.visible.width - (cc.visible.designWidth- x))
        elseif bx == "center" then
            self:setPositionX((cc.visible.width - (cc.visible.designWidth- x))/2)
        end
    end
end

-- 适配边界
-- bx = "left"/"right", by = "top"/"bottom", (designOx, designOy) = csd上的原点坐标
function This:adapterBorder(bx, by, designOx, designOy)
    local hScale = cc.visible.frameHeight / cc.visible.designHeight
    local wScale = cc.visible.frameWidth / cc.visible.designWidth
    local scale = 1
    if hScale > wScale then
        scale = 1 / hScale * wScale
    end

    local x, y = self:getPosition()
    local wx, wy = self:getParent():convertToWorldSpace(x, y)
    local toLeft = x - designOx
    local toRight =  designOx + cc.visible.designWidth - x
    local toTop = designOy + cc.visible.designHeight - y
    local toBottom = y - designOy

    if bx == "left" then
        wx = toLeft * scale
    elseif bx == "right" then
        wx = cc.visible.width - toRight * scale
    end

    if by == "top" then
        wy = cc.visible.height - toTop * scale
    elseif by == "bottom" then
        wy = toBottom * scale
    end

    x, y = self:getParent():convertToNodeSpace(wx, wy)
    self:setPosition(x, y)
    self:setScale(self:getScale() * scale)
end

-- 等比例拉伸到全屏大小
function This:adapterFullScreen()
    local x, y = self:getPosition()
    local w, h = self:getContentSize()
    local anchorX, anchorY = self:getAnchorPoint()    
    self:setAnchorPoint(0.5, 0.5)
    self:setPosition(x+(0.5-anchorX)*w, y+(0.5-anchorY)*h)

    local box = self:getBoundingBox()
    local scaleX = cc.visible.width/box.width
    local scaleY = cc.visible.height/box.height
    local scale = (scaleX > scaleY) and scaleX or scaleY
    self:setScale(self:getScale() * scale)
end