local This, super = class("cc.PhysicsDrawNode", "cc.Node")
cc.PhysicsDrawNode = This


function This:createNode(world)
    return lib2d.b2PhysicsDrawNode.create(world[0])
end

function This:init()
    self.m_shapeList = {}
end

function This:drawPhysics()
    lib2d.b2PhysicsDrawNode.drawPhysics(self[0])
end

function This:onDraw()
    for _, shape in ipairs(self.m_shapeList) do
        if shape.m_zorder == -1 then
            shape:onDraw()
        end
    end
    self:drawPhysics()
    for _, shape in ipairs(self.m_shapeList) do
        if shape.m_zorder ~= -1 then
            shape:onDraw()
        end
    end
end

function This:drawSolidPolygon(pointList, rgba, outlineWidth, outlineRBGA)
    lib2d.b2PhysicsDrawNode.drawSolidPolygon(self[0], pointList, rgba, outlineWidth, outlineRBGA)
end

local ColorList = {
    [1] = {0x0033CCB4, 0x0033CCDD, 0x0033CCDD, 0xAA69AAFF},
    [2] = {0x333388B4, 0x333388DD, 0x333388DD, 0xAA69AAFF},
}
local BoxShape = {
    _type = "BoxShape",
    colorID = 1,
    onDraw = function (self)
        local pointList = self.m_pointList
        local ax, ay = self.m_anchor[1], self.m_anchor[2]
        local left = self.m_x - self.m_width * ax
        local bottom = self.m_y - self.m_height * ay
        pointList[1] = left
        pointList[2] = bottom
        pointList[3] = left + self.m_width
        pointList[4] = bottom
        pointList[5] = left + self.m_width
        pointList[6] = bottom + self.m_height
        pointList[7] = left
        pointList[8] = bottom + self.m_height
        local colors = ColorList[self.colorID]
        if self.m_isHightLight then
            self.m_node:drawSolidPolygon(pointList, colors[3], 3, colors[4])
        else
            self.m_node:drawSolidPolygon(pointList, colors[1], 2, colors[2])
        end
    end,
    remove = function (self)
        self.m_node:removeShape(self)
    end,
    testPoint = function (self, px, py)
        local ax, ay = self.m_anchor[1], self.m_anchor[2]
        local minX = self.m_x - self.m_width * ax
        local minY = self.m_y - self.m_height * ay
        local maxX = minX + self.m_width
        local maxY = minY + self.m_height
        return px >= minX and px <= maxX and py >= minY and py <= maxY
    end,
}
BoxShape.__index = BoxShape

function This:addBox(x, y, width, height, anchor)
    local shape = {
        m_x = x,
        m_y = y,
        m_width = width,
        m_height = height,
        m_anchor = anchor or {0.5, 0.5},
        m_pointList = {},
    }
    setmetatable(shape, BoxShape)
    self:addShape(shape)
    return shape
end

function This:addShape(shape)
    local id = #self.m_shapeList + 1
    shape.m_id = id
    shape.m_node = self
    self.m_shapeList[id] = shape
end

function This:removeShape(shape)
    local id = shape.m_id
    for i = 1, #self.m_shapeList do
        if id == i then
            table.remove(self.m_shapeList, i)
            break
        end
    end
end