local This, super = class(..., "cc.Node")
cc.ProgressTimer = This
local lib2d = lib2d
local lib_create = lib2d.ccProgressTimer.create
local registerObject = cc.registerObject

function This:create(sprite, text)
    local scale = sprite:getScale()
    local node = lib_create(sprite[0])
    node = registerObject(node, self)
    if text then
        node.m_text = text
    end
    sprite:setVisible(false)
    node:setScale(scale)
    node:setPosition(sprite:getPosition())
    node:init()
    return node
end

function This:init()
    self:scheduleUpdate()
    self:setReverseProgress(true)
    self:setPercent(0)
    --self:setVisible(false)
    self.m_scene = g_scene
    if self.m_text then
        self.m_text:setVisible(false)
    end
    self.m_isPlay = false
end

local lib_setProgressType = lib2d.ccProgressTimer.setType
function This:setProgressType(type)
    lib_setProgressType(self[0], type)
end

local lib_getProgressType = lib2d.ccProgressTimer.getType
function This:getProgressType()
    return lib_getProgressType(self[0])
end

local lib_setPercent = lib2d.ccProgressTimer.setPercent
function This:setPercent(percent)
    lib_setPercent(self[0], percent)
end

function This:setRatio(ratio)
    lib_setPercent(self[0], ratio * 100)
end

local lib_getPercent = lib2d.ccProgressTimer.getPercent
function This:getPercent()
    return lib_getPercent(self[0])
end

local lib_setReverseProgress = lib2d.ccProgressTimer.setReverseProgress
function This:setReverseProgress(isReverse)
	if self.m_isReverseProgress ~= isReverse then
		self.m_isReverseProgress = isReverse
		lib_setReverseProgress(self[0], isReverse)
	end
end

local lib_setReverseDirection = lib2d.ccProgressTimer.setReverseDirection
function This:setReverseDirection(isReverse)
    if self.m_isReverseDir ~= isReverse then
        lib_setReverseDirection(self[0], isReverse)
        self.m_isReverseDir = isReverse
    end
end

local lib_isReverseDirection = lib2d.ccProgressTimer.isReverseDirection
function This:isReverseDirection()
    return lib_isReverseDirection(self[0])
end

function This:startTimer(time,passTime)
    self.m_isPlay = true
    self.m_time = time
    self.m_passTime = passTime or 0
    self:setPercent(0)
    self:setVisible(true)
    if self.m_text then
        self.m_text:setVisible(true)
        self.m_text:setString(time)
    end
    if self.m_isReverseProgress then
        self.m_fromPercent = 1
        self.m_toPercent = 0
    else
        self.m_fromPercent = 0
        self.m_toPercent = 1
    end
end

function This:fromToTimer(time,from, to)
    self.m_isPlay = true
    self.m_fromPercent = from
    self.m_toPercent = to
    self.m_time = time
    self.m_passTime = 0
    self:setVisible(true)
    if self.m_text then
        self.m_text:setVisible(true)
        self.m_text:setString(time)
    end
end

function This:endTimer()
    self.m_isPlay = false
    --self:setVisible(false)
    if self.m_text then
        self.m_text:setVisible(false)
    end
end

function This:update(deltaTime)
    if not self.m_isPlay then
        return
    end
    
    self.m_passTime = self.m_passTime + deltaTime
    if self.m_passTime >= self.m_time then
        self.m_passTime = self.m_time
        self:endTimer()
    end

    local percent = self.m_passTime / self.m_time
    percent = self.m_fromPercent + (self.m_toPercent - self.m_fromPercent) * percent  

    if self.m_text then
        self.m_text:setString(string.format("%.1f", self.m_time - self.m_passTime))
    end
    self:setPercent(percent * 100)
end

function This:setGrayShader()
    self:setShaderByID(1)
end

function This:isPlaying()
    return self.m_isPlay
end