local This, super = class(..., "cc.Node")
cc.RichText = This

function This:init(info)
    self.m_items = {}
    self:setAnchorPoint(0.5, 0.5)
    if info then
        self.m_fontSize = info.fontSize or 24
        if info.scale then
            self:setScale(info.scale)
        end
        if info.anchor then
            self:setAnchorPoint(info.anchor[1] or 0.5,info.anchor[2] or 0.5)
        end
        if info.text then
            self:setString(info.text, info.lineWidth or 0)
        end
    end
end

function This:setString(text, lineWidth)
    for i, node in ipairs(self.m_items) do
        node:removeFromParent()
    end
    self.m_items = {}
    local textList = {}
    local lp = 1
    for i = 1,10000 do
        local p = string.find(text, "<#", lp)
        if not p then
            break
        end
        if p ~= 1 then
            textList[#textList + 1] = string.sub(text, lp-1, p-1)
        end
        lp = p + 1
    end
    local lineNum = 1
    textList[#textList + 1] = string.sub(text, lp-1, -1)
    for _, str in ipairs(textList) do
        local colorID = "A0"
        if string.sub(str, 1, 2) == "<#" then
            local p = string.find(str, '>')
            colorID = string.sub(str, 3, p-1)
            str = string.sub(str, p+1, -1)
        end
        local lineList = string.split(str, "\n")
        for lineIndex, line in ipairs(lineList) do
            local label
            if lineWidth and lineWidth > 0 then
                label = cc.Label:createNormalTTF(line, self.m_fontSize, {width = lineWidth, height = 0})
            else
                label = cc.Label:createNormalTTF(line, self.m_fontSize)
            end
            label:setAnchorPoint(0, 0)
            label:setColorByID(colorID)
            label:getContentSize()
            label.m_lineNum = lineNum
            label.m_endX, label.m_endY = label:getNextFontPosition()
            self.m_items[#self.m_items + 1] = label
            self:addChild(label)
            if #lineList > 1 and lineIndex ~= #lineList then
                lineNum = lineNum+1
            end
        end
        if string.byte(str, -1) == 10 then
            lineNum = lineNum+1
        end
    end
    self:updateItemPosition()
end

function This:updateItemPosition()
    local totalWidth = 0
    local totalHeight = 0
    local lineIndex = 1
    local lineX = 0
    for i = 1, #self.m_items do
        local node = self.m_items[i]
        if node.m_lineNum ~= lineIndex then
            lineIndex = node.m_lineNum
            lineX = 0
            local width, height = node:getContentSize()
            for j = 1, i-1 do
                self.m_items[j]:addPosition(nil, height)
            end
        else
            if i ~= 1 then
                lineX = lineX + self.m_items[i - 1].m_endX
                node:setPosition(lineX,0)
            end
        end
    end
    for i, node in ipairs(self.m_items) do
        local x, y = node:getPosition()
        local width, height = node:getContentSize()
        if x + width > totalWidth then
            totalWidth = x + width
        end
        if y + height > totalHeight then
            totalHeight = y + height
        end
    end
    self:setContentSize(totalWidth, totalHeight)
end