local This, super = class(..., "cc.Node")
cc.Scene = This
cc.addCppNodeClass(This)
local Timer = require("Timer")
local registerObject = cc.registerObject
local lib_scene = lib2d.ccScene
local lib_create = lib_scene.create
local EventListeners = require("EventDispatcher").m_listeners
local NetWaitView = require("Net.NetWaitView")

function This:create(...)
    local scene = registerObject(lib_create(self), self:getClass())
    scene:init(...)
    return scene
end

function This:getClass()
    return self
end

function This:init()
	g_scene = self
    self.m_timer = Timer:create(self)
    self.m_updateList = {}
    self.m_lockTouch = 0
    self.m_lockGuide = 0
	self.m_blurNode = nil
    self:scheduleUpdate()
end

function This:onEnter()
    g_weakTable[self._NAME] = self
end

function This:lockTouchWithoutWaiting()
	self.m_lockTouch = self.m_lockTouch + 1
    if self.m_lockTouch > 0 then
        if not self.m_maskTouch then
            self.m_maskTouch = cc.TouchListener:create({})
            self:addEventListener(self.m_maskTouch, -1)
        end
    end
end

function This:lockTouch()
    self.m_lockTouch = self.m_lockTouch + 1
    if self.m_lockTouch > 0 then
        if not self.m_maskTouch then
            self.m_maskTouch = cc.TouchListener:create({})
            self:addEventListener(self.m_maskTouch, -1)
			self.m_waitNode = NetWaitView:create()
			self:addChild(self.m_waitNode)
			self.m_waitNode:showWaiting()
        end
    end
end

function This:unlockTouch(delayTime)
    if self.m_maskTouch then
        if not delayTime or delayTime == 0 then
            self:onUnlockTouch()
        else
            self:setTimer(delayTime, self.onUnlockTouch)
        end
    end
end

function This:onUnlockTouch()
    self.m_lockTouch = self.m_lockTouch - 1
    if self.m_lockTouch <= 0 then
        if self.m_maskTouch then
            self:removeEventListener(self.m_maskTouch)
            self.m_maskTouch = false
			if self.m_waitNode then
				self.m_waitNode:hideWaiting()
                self.m_waitNode:removeFromParent(true)
                self.m_waitNode = false
			end
        end
    end
end

function This:onExit()
    self.m_updateList = nil
    local maskTouch = self.m_maskTouch
    if maskTouch and maskTouch[0] ~= 0 then
        self:removeEventListener(maskTouch)
        self.m_maskTouch = nil
    end
    self.m_timer:removeAllTimer()
    EventListeners[self] = nil
end

function This:addListener(key, func)
    EventListeners[self][key]=func
end

function This:removeListener(key)
    EventListeners[self][key] = nil
end

-- function This:setTimer(delayTime, listener)
function This:setTimer(delayTime, listener, key, triggerTotal)
    self.m_timer:setTimer(delayTime, listener, key, triggerTotal)
end

function This:updateNodeList(delta)
    for id, node in pairs(self.m_updateList) do
        if type(node) == "table" then
            --保护调用
            local ret, err = pcall(function ()
                node:update(delta)
            end)
            if not ret then
                logError(err)
            end
        end
    end
end

function This:update(delta)
    if delta < 1.0 / 60.0 then
        delta = 1.0 / 60.0
    end

    if delta > 0.1 then
        delta = 0.1
    end
    self.m_timer:update(delta)
    self:updateNodeList(delta)
end

function This:scheduleUpdate()
    lib_scene.scheduleUpdate(self[0])
end

function This:unscheduleUpdate()
    lib_scene.unscheduleUpdate(self[0])
end

function This:addMouseListener(listener)
    listener = require("cc.MouseListener"):create(listener)
    self:addEventListener(listener)
    return listener
end

function This:addToUpdateList(node)
    self.m_updateList[node[0]] = node
end

function This:removeFromUpdateList(node)
    self.m_updateList[node[0]] = nil
end

function This:setChildrenVisible(isVisible)
    lib_scene.setChildrenVisible(self[0], isVisible)
end

function This:setIsBlur(isBlur)
	if isBlur then
		if self.m_blurNode then
			self.m_blurNode:setVisible(false)
		else
			self.m_blurNode = cc.RenderTexture:create(cc.visible.width,cc.visible.height,1)
			self.m_blurNode:setPosition(cc.visible.center.x,cc.visible.center.y)
			self:addChild(self.m_blurNode)
		end
	elseif not isBlur and self.m_blurNode then
		self.m_blurNode:setVisible(false)
	end
end

local castNode = cc.Node.castNode
function This:getChildren()
    local children = lib_scene.getChildren(self[0])
    for i = 1, #children do
        children[i] = castNode(children[i])
    end
    return children
end