local This = class(...)
cc.SceneManager = This
local director = require("cc.Director")

function This:init()
    self.m_runningScene = nil
    self.m_isSwitching = false
    self.m_sceneName = ""
end

function This:createScene(sceneName, ...)
    self.m_sceneName = sceneName
    return require(sceneName):create(...)
end

function This:runWithSceneName(sceneName, ...)
    local scene = self:createScene(sceneName, ...)
    if scene then
        self:runScene(scene)
    end
    return scene
end

function This:runScene(scene)
	if self.m_runningScene and self.m_runningScene[0]~= 0 and self.m_runningScene._NAME == "WorldScene" then
		self.m_lastScene = self.m_runningScene
		self.m_lastScene:retain()
	end
    self.m_runningScene = scene
    
    if director:hasRunningScene() then
        director:replaceScene(scene)
    else
        director:runWithScene(scene)
    end
end

function This:switchTo(sceneName, finishHandler, ...)
    if self.m_isSwitching then
        return
    end
    local runeTime = require("RunTime"):create()
    self.m_isSwitching = true
    local scene = self:createScene(sceneName,...)
    if not scene or scene[0] == 0 then
        print("scene not found:" .. sceneName)
        self.m_isSwitching = false
        return
    end
	g_scene = scene

    print("---------------------------------",runeTime:getTimeMS())
	runeTime:reset()
    self:runScene(scene)

    local layer = cc.LayerColor:create(cc.Color(0, 0, 0, 0))
    layer:setTouchForMask({})
    scene:addChild(layer, 1000000)

    layer:runAction(cc.Sequence(
		cc.CallFunc(function()
			print("---------------------------------",runeTime:getTimeMS())
		end),
		cc.FadeOut(scene.fadeTime or 0.03), 
		cc.CallFunc(function ()
			layer:removeFromParent()
			if finishHandler then
				finishHandler()
			end
			if self.m_lastScene then
				self.m_lastScene:release()
				self.m_lastScene = nil
			end
			self.m_isSwitching = false
		end)
	))
end

function This:getSceneName()
    return self.m_sceneName
end

This:init()