local This, super = class(..., "cc.Node")
This.IsContainer = true
cc.addCppNodeClass(This)
cc.ScrollView = This

local lib_scrollView = lib2d.ccScrollView
local castNode = cc.Node.castNode

function This:createNode()
    return lib_scrollView.create()
end

function This:init()

end

function This:getDirection()
    return lib_scrollView.getDirection(self[0])
end

function This:setDirection(direction)
    if direction == "vertical" then
        direction = 1
    elseif direction == "horizontal" then
        direction = 2
    elseif direction == "both" then
        direction = 3
    else
        direction = 0
    end
    return lib_scrollView.setDirection(self[0], direction)
end

function This:setInnerContainerSize(width, height)
    lib_scrollView.setInnerContainerSize(self[0], width, height)
end

function This:setInnerContainerHeight(height)
    local width, _ = lib_scrollView.getInnerContainerSize(self[0])
    lib_scrollView.setInnerContainerSize(self[0], width, height)
end

function This:getInnerContainerSize()
    return lib_scrollView.getInnerContainerSize(self[0])
end

function This:getInnerContainerHeight()
    local width, height = lib_scrollView.getInnerContainerSize(self[0])
    return height
end

function This:setBounceEnabled(isEnabled)
    lib_scrollView.setBounceEnabled(self[0], isEnabled)
end

function This:getInnerContainer()
    return castNode(lib_scrollView.getInnerContainer(self[0]))
end

function This:getInnerContainerPosition()
    return lib_scrollView.getInnerContainerPosition(self[0])
end

function This:setInnerContainerPosition(x, y)
    lib_scrollView.setInnerContainerPosition(self[0], x, y)
end

function This:addEventListener(listener)
    self.m_eventListener = listener
    lib_scrollView.addEventListener(self[0])
end

function This:stopAutoScoll()
    lib_scrollView.stopAutoScroll(self[0])
end

function This:setTouchEnabled(isEnabled)
    lib_scrollView.setTouchEnabled(self[0], isEnabled)
end

function This:isTouchEnabled()
    return lib_scrollView.isTouchEnabled(self[0])
end

function This:jumpToTop()
    lib_scrollView.jumpToTop(self[0])
end

function This:jumpToBottom()
    lib_scrollView.jumpToBottom(self[0])
end

function This:jumpToLeft()
    lib_scrollView.jumpToLeft(self[0])
end

function This:jumpToRight()
    lib_scrollView.jumpToRight(self[0])
end

function This:getTotalHeight()
    local height = 0
    local count = #self.m_itemList
    for i = 1, count do
        local itemNode = self.m_itemList[i]
        height = height + itemNode.itemHeight
        if i ~= count then
            height = height + self.m_itemMargin
        end
    end
    return height
end

function This:getTotalWidth()
    local width = 0
    local count = #self.m_itemList
    for i = 1, count do
        local itemNode = self.m_itemList[i]
        width = width + itemNode.itemWidth
        if i ~= count then
            width = width + self.m_itemMargin
        end
    end
    return width
end

function This:getItemList()
    return self.m_itemList or {}
end

function This:pushBackCustomItem(node, width, height, anchoY, itemMargin)
    if not self.m_itemList then
        self.m_itemList = {}
    end
    if not self.m_direction then
        self.m_direction = "down"
    end
    if not self.m_contentWidth then
        self.m_contentWidth, self.m_contentHeight = self:getContentSize()
    end
    --node:setCascadeOpacityEnabled(true)
    node.itemAnchoY = anchoY or 0.5
    node.itemWidth = width
    node.itemHeight = height
    self.m_itemWidth = width
    self.m_itemHeight = height
    self.m_itemMargin = itemMargin or 5
    node:setName("scrollNode"..#self.m_itemList)
    self:addChild(node)
    self.m_itemList[#self.m_itemList + 1] = node
    self:refreshPosition()
end

function This:removeItem(index)
    local itemNode = self.m_itemList[index]
    table.remove(self.m_itemList, index)
    itemNode:removeFromParent()
    self:refreshPosition()
end

function This:addHeight(deltaHeight)
    local innerWidth, innerHeight = self:getInnerContainerSize()
    self:setInnerContainerSize(innerWidth, innerHeight + deltaHeight)
    for i, node in ipairs(self.m_itemList) do
        node:setPositionY(node:getPositionY() + deltaHeight)
    end
    self:jumpToBottom()
end

function This:refreshPosition()
    if not self.m_itemList then
        return
    end
    if #self.m_itemList <= 0 then
        return
    end

    if self.m_direction == "down" then
	    local innerHeight = self:getTotalHeight()
		if innerHeight < self.m_contentHeight then
			innerHeight = self.m_contentHeight
		end
		self:setInnerContainerSize(self.m_contentWidth, innerHeight)
        local y = innerHeight
        for i, node in ipairs(self.m_itemList) do
            y = y - (1 - node.itemAnchoY) * node.itemHeight
            node:setPosition(node.itemWidth * 0.5, y)
            y = y - node.itemAnchoY * node.itemHeight - self.m_itemMargin
        end
    elseif self.m_direction == "up" then
		local innerHeight = self:getTotalHeight()
		if innerHeight < self.m_contentHeight then
			innerHeight = self.m_contentHeight
		end
		self:setInnerContainerSize(self.m_contentWidth, innerHeight)
        local y = 0
        for i, node in ipairs(self.m_itemList) do
            y = y + node.itemAnchoY * node.itemHeight
            node:setPosition(node.itemWidth * 0.5, y)
            y = y + (1 - node.itemAnchoY) * node.itemHeight + self.m_itemMargin
        end
	elseif self.m_direction == "right" then
		local innerWidth = self:getTotalWidth()
		if innerWidth < self.m_contentWidth then
			innerWidth = self.m_contentWidth
		end
		self:setInnerContainerSize(innerWidth, self.m_contentHeight)
        local x = 0
        for i, node in ipairs(self.m_itemList) do
            x = x + node.itemAnchoY * node.itemWidth
            node:setPosition(x, node.itemHeight * 0.5)
            x = x + (1 - node.itemAnchoY) * node.itemWidth + self.m_itemMargin
        end
    end
end

function This:getContentHeight()
    local width, height = self:getContentSize()
    return height
end

function This:removeAllItems()
    self:signCurrentY()
    self:removeAllChildren()
    self:stopAutoScoll()
    self:jumpToTop()
    self.m_itemList = {}
	self.m_itemMargin = nil
end

function This:signCurrentY()
    local x, y = self:getInnerContainerPosition()
    self.m_offsetY = self:getInnerContainerHeight() - self:getContentHeight() + y
end

function This:jumpToSign()
    local topY = self:getContentHeight() - self:getInnerContainerHeight()
    local y = topY + self.m_offsetY
    if y > 0 then
        y = 0
    end
    self:setInnerContainerPosition(0, y)
end

function This:resetAmount(amount)
    if #self.m_itemList < amount then
        for i = 1, amount - #self.m_itemList do
            self:pushBackCustomItem(self.m_itemCreateFunc(), self.m_itemWidth, self.m_itemHeight)
        end
        self:refreshPosition()
    elseif #self.m_itemList > amount then
        for i = #self.m_itemList, amount + 1, -1 do
            self.m_itemList[i]:removeFromParent()
            table.remove(self.m_itemList, i)
        end
        self:refreshPosition()
    end
end

function This:jumpToAtInnerPosY(posy)
    local innerHeight = self:getInnerContainerHeight()
    local topY = self:getContentHeight() - innerHeight
    if topY < 0 then
        local dy = math.min(innerHeight - posy, -topY)
        self:setInnerContainerPosition(0, topY + dy)
    end
end

function This:scrollToBottom(time, attenuated)
    lib_scrollView.scrollToBottom(self[0], time, attenuated)
end

function This:scrollToTop(time, attenuated)
    lib_scrollView.scrollToTop(self[0], time, attenuated)
end

function This:scrollToLeft(time, attenuated)
    lib_scrollView.scrollToLeft(self[0], time, attenuated)
end

function This:scrollToRight(time, attenuated)
    lib_scrollView.scrollToRight(self[0], time, attenuated)
end

function This:scrollToPercentHorizontal(percent,time,attenuated)
    percent = math.min(100,math.max(0,percent))
    lib_scrollView.scrollToPercentHorizontal(self[0], percent,time or 0.03, attenuated or false)
end

function This:scrollToPercentVertical(percent,time,attenuated)
    lib_scrollView.scrollToPercentVertical(self[0], percent , time or 0.03, attenuated or false)
end

function This:pushNode(node, width, height, limit)
    if not self.m_itemList then
        self.m_itemList = {}
    end
    if not self.m_direction then
        self.m_direction = -1
    end
    if not self.m_contentWidth then
        self.m_contentWidth, self.m_contentHeight = self:getContentSize()
    end
    if not self.m_itemMargin then
        self.m_itemMargin = {5, 5}
    end
    if not self.m_leftTopSpace then
        self.m_leftTopSpace = {0, 0}
    end
    limit = limit or 1
    self:addChild(node)
    self.m_itemList[#self.m_itemList + 1] = node
    node:setName("scrollNode"..#self.m_itemList)

    local marginX, marginY = self.m_itemMargin[1], self.m_itemMargin[2]
    local startX, startY = width * 0.5 + self.m_leftTopSpace[1], -height * 0.5 - self.m_leftTopSpace[2]
    local x, y = startX, startY
    local index = 1
    local line = 1
    for i, node in ipairs(self.m_itemList) do
        if index > limit then
            index = 1
            line = line + 1
            x = startX
            y = y - height - marginY
        end
        node:setPosition(x, y)
        x = x + width + marginX
        index = index + 1
    end

    local totalHeight = line * height + (line - 1) * marginY
    local width, height = self.m_contentWidth, self.m_contentHeight
    self:setTouchEnabled(height < totalHeight)
    local innerHeight = height > totalHeight and height or totalHeight
    self:setInnerContainerSize(width, innerHeight)
    for k,node in ipairs(self.m_itemList) do
        node:addPosition(0, innerHeight)
    end
end

function This:runItemAppearAction()
    if not rawget(self, "m_itemList") then
        return
    end
    for i, node in ipairs(self.m_itemList) do
        node:runActionByID("ListAppear", i * 0.1)
    end
end

function This:setSwallowTouch(isSwallow)
    local eventListener = lib_scrollView.getTouchListener(self[0])
    lib2d.ccTouchListener.setSwallowTouches(eventListener, isSwallow)
end