local This, super = class(..., "cc.Node")
cc.Sprite = This
local spriteFrameCache = require("cc.SpriteFrameCache")
local textureCache = require("cc.TextureCache")
cc.addCppNodeClass(This)

local lib_sprite = lib2d.ccSprite
local Size = Size


local lib_createWithSpriteFrame = lib_sprite.createWithSpriteFrame
local lib_createWithTexture = lib_sprite.createWithTexture
local ccRegister = cc.registerObject
function This:create(fileName)
    local frame = spriteFrameCache.m_frames[fileName]
    local node = frame and lib_createWithSpriteFrame(frame) or lib_createWithTexture(textureCache.m_images[fileName])
    node = ccRegister(node, self, {[0] = 0})
    return node
end

function This:createWithSpriteFrame(frame)
    return ccRegister(lib_createWithSpriteFrame(frame), self, {[0] = 0})
end

local lib_isFlippedX = lib_sprite.isFlippedX
function This:isFlippedX()
    return lib_isFlippedX(self[0])
end

local lib_setFlippedX = lib_sprite.setFlippedX
function This:setFlippedX(isFlippedX)
    lib_setFlippedX(self[0], isFlippedX)
end

local lib_isFlippedY = lib_sprite.isFlippedY
function This:isFlippedY()
    return lib_isFlippedY(self[0])
end

local lib_setFlippedY = lib_sprite.setFlippedY
function This:setFlippedY(isFlippedY)
    lib_setFlippedY(self[0], isFlippedY)
end

local lib_setSpriteFrame = lib_sprite.setSpriteFrame
function This:setSpriteFrame(frame)
    lib_setSpriteFrame(self[0], frame)
end

local lib_setSpriteFrameByName = lib_sprite.setSpriteFrameByName
function This:setSpriteFrameName(frameName)
    lib_setSpriteFrameByName(self[0], frameName)
end

local lib_setTextureByName = lib_sprite.setTextureByName
local lib_setTexture = lib_sprite.setTexture
local textureCache = cc.TextureCache
function This:setImage(fileName)
    local spriteFrame = spriteFrameCache.m_frames[fileName]
    if spriteFrame then
        lib_setSpriteFrame(self[0], spriteFrame)
    else
        lib_setTexture(self[0], textureCache.m_images[fileName])
    end
end

function This:setTexture(fileName)
    lib_setTextureByName(self[0], fileName)
end

function This:setAngle(angle)
    if self:isFlippedX() then
        self:setRotation(angle)
    else
        self:setRotation(-angle)
    end
end

local lib_getTextureRect = lib_sprite.getTextureRect
function This:getTextureRect()
    return cc.Rect(lib_getTextureRect(self[0]))
end

local lib_setTextureRect = lib_sprite.setTextureRect
-- function This:setTextureRect(rect)
function This:setTextureRect(rect, isRoate, width, height)
    if isRoate == nil then
        isRoate = false
    end
    if not width then
        width = rect.width
    end
    if not height then
        height = rect.height
    end
    lib_setTextureRect(self[0], rect.x, rect.y , rect.width, rect.height, isRoate, width, height)
end

local lib_isTextureRectRotated = lib_sprite.isTextureRectRotated
function This:isTextureRectRotated()
    return lib_isTextureRectRotated(self[0])
end

function This:setGrayShader()
    self:setShaderByID(2)
end

function This:setRatio(ratio)
    self:setPercent(ratio * 100)
end

function This:setPercent(percent,isHeight)
    if percent >= 100 then
        percent = 100
    end
    if percent <= 0 then
        percent = 0
    end
    if not self.m_percent then
        self.m_barRect = self:getTextureRect()
		self.m_rx , self.m_ry = self.m_barRect.x,self.m_barRect.y
        self.m_barWidth, self.m_barHeight = self:getContentSize()
        self.m_isTextureRotated = self:isTextureRectRotated()
        self.m_percent = 100
    end
    if self.m_percent == percent then
        return
    end
    local rect = self.m_barRect
    self.m_percent = percent
    if not isHeight then
        rect.width = self.m_barWidth * percent / 100
        if self:getPercentStyle() == 1 then
            rect.x = self.m_barWidth - rect.width
        end
    else
        rect.y =  self.m_ry + self.m_barHeight * (100-percent) / 100
		rect.height = self.m_barHeight * percent / 100
    end
    self:setTextureRect(rect, self.m_isTextureRotated, self.m_barRect.width, self.m_barRect.height)
    --self:setContentSize(self.m_barWidth, self.m_barHeight)
end

function This:runPercentAction(percent, velocity,isHeight)
    if not self.m_actionVelocity then
        if not self.m_percent then
            self:setPercent(0)
        end
        if not self.m_refUpdateList then
            self:scheduleUpdate()
        end
        self.update = function (self, deltaTime)
            if not self.m_isRunning then
                return
            end
            local percent = self.m_percent + self.m_actionDirection * self.m_actionVelocity * deltaTime
            if self.m_actionDirection > 0 then
                if percent > self.m_targetPercent then
                    percent = self.m_targetPercent
                    self.m_isRunning = false
                end
            elseif self.m_actionDirection < 0 then
                if percent < self.m_targetPercent then
                    percent = self.m_targetPercent
                    self.m_isRunning = false
                end
            end
            self:setPercent(percent,isHeight)
        end
    end
    self.m_isRunning = true
    self.m_actionDirection = percent > self.m_percent and 1 or -1
    self.m_targetPercent = percent
    self.m_actionVelocity = velocity
end

function This:getPercentX()
    if not self.m_percent then
        return 0
    end
    local anchorPoint = self:getAnchorPoint()
    local x, y = self:getPosition()
    return x - (self.m_barSize.width * (anchorPoint.x - self.m_percent / 100))
end

function This:setAsButton(normalName, onBegan, onEnded, sound)
    local normalImage = normalName .. "1.png"
    local pressImage = normalName .. "2.png"
    local touch = {
        onBegan = function ()
            self:setImage(pressImage)
            onBegan()
        end,
        onEnded = function ()
            self:setImage(normalImage)
            onEnded()
        end,
        sound = sound,
    }
    touch.onCanceled = touch.onEnded
    self:setTouch(touch)
end

local GL_ONE = 1
local GL_ZERO = 0
local GL_SRC_ALPHA = 0x0302
local GL_ONE_MINUS_SRC_ALPHA = 0x0303
local GL_DST_ALPHA = 0x0304
local GL_ONE_MINUS_DST_ALPHA = 0x0305
local GL_DST_COLOR = 0x0306
local BlendFuncs = {
    ["Additive"] = {GL_SRC_ALPHA, GL_ONE},
}
local lib_setBlendFunc = lib_sprite.setBlendFunc
function This:setBlendID(id)
    local b = BlendFuncs[id]
    if BlendFuncs[id] then
        lib_setBlendFunc(self[0], b[1], b[2])
    end
end

--0从左到右，1从右到左，默认0
function This:setPercentStyle(style)
    self.m_percentStyle = style
end

function This:getPercentStyle()
    return self.m_percentStyle or 0
end