local This, super = class(..., "cc.Node")
local Vec = cc.Vec
cc.cppNodeClassList[lib2d.ccTMXLayer.getClassID()] = cc.Node

local lib_tiledMap = lib2d.ccTiledMap
function This:createNode(fileName)
    return lib_tiledMap.create(fileName)
end

function This:init()
    self:ignoreAnchorPointForPosition(true)
end

function This:getGroupList()
    return lib_tiledMap.getGroupList(self[0])
end

function This:getMapSize()
    return lib_tiledMap.getMapSize(self[0])
end

function This:getTileSize()
    return lib_tiledMap.getTileSize(self[0])
end


local function convertToMapObject(self)
    self.name = tonumber(self.name) or self.name
    if self.points then
        -- convertPointsToCCW begin
        local points = self.points
        local vec1 = Vec(points[3] - points[1], points[4] - points[2])
        local vec2 = Vec(points[5] - points[3], points[6] - points[4])
        if vec1:cross(vec2) < 0 then
            local newPoints = {points[1], points[2]}
            local count = #points
            for i = 3, count, 2 do
                newPoints[i] = points[count - i + 2]
                newPoints[i+1] = points[count - i + 3]
            end
            self.points = newPoints
        end
        -- convertPointsToCCW end

        -- initWidthAndHeight begin
        local minX, minY = points[1], points[2]
        local maxX, maxY = minX, minY
        for i=1, #points, 2 do
            local x = points[i]
            local y = points[i + 1]
            if maxX < x then
                maxX = x
            end
            if minX > x then
                minX = x
            end
            if maxY < y then
                maxY = y
            end
            if minY > y then
                minY = y
            end
        end

        self.width = maxX - minX
        self.height = maxY - minY
        -- initWidthAndHeight end

        -- convertPointsToWorld begin
        for i = 1, #points, 2 do
            points[i] = points[i] + self.x
            points[i + 1] = points[i + 1] + self.y
        end
        -- convertPointsToWorld end
    end
    return self
end

function This:generateGroup()
    local group = {}
    for i, layer in pairs(lib_tiledMap.getGroupList(self[0])) do
        local name = layer.name
        local objectList = {}
        local objectListByName = {}
        local objectListByID = {}
        for i, objectValue in ipairs(layer.objectList) do
            local mapObject = convertToMapObject(objectValue)
            objectList[i] = mapObject
            objectListByID[mapObject.id] = mapObject
            if mapObject.name then
                objectListByName[mapObject.name] = mapObject
            end
        end
        group[name] = {
            objectList = objectList,
            objectListByName = objectListByName,
            objectListByID = objectListByID,
            getObjectByName = function (layer, name)
                return layer.objectListByName[name]
            end,
            getObjectByID = function (layer, id)
                return layer.objectListByName[id]
            end,
        }
    end
    group.getObjectByName = function (group, layerName, objName)
        local layer = group[layerName]
        if layer then
            return layer.objectListByName[objName]
        end
    end
    group.getObjectByID = function (group, layerName, objID)
        local layer = group[layerName]
        if layer then
            return layer.objectListByID[objID]
        end
    end
    return group
end