local This = class(...)
cc.TouchListener = This
local lib_touchListener = lib2d.ccTouchListener
local Vec = cc.Vec
local objectRegistry = cc.objectRegistry
local registerObject = cc.registerObject

local normalAlpha = 255
local pressAlpha = 200
local function setIsPressState(self, isPress)
    if self.isScale ~= false then
        local scaleX = isPress and (self.normalScaleX + self.deltaScale) or self.normalScaleX
        local scaleY = isPress and (self.normalScaleY + self.deltaScale) or self.normalScaleY
        self.node:setScale(scaleX, scaleY)
    end
    if self.isAlpha ~= false then
        self.node:setOpacity(isPress and pressAlpha or normalAlpha)
    end
end

local function onTouchBegan(self, touchPoint)
    if self.isEnabled == false then
        return false
    end

    if not self.node:isRootVisible() then
        return false
    end
    if not self.rect or self.isDynamic then
        self.rect = self:getTouchRect()
    end

    if self.isContainer then
        local container = self.node
        while container and not(container.IsContainer) do
            container = container:getParent()
        end
        self.isCheckMoving = container:isTouchEnabled()
        -- if not self.containerRect then
        self.containerRect = container:getBoundingBox()
        -- end

        if not self.containerRect:isContainsPoint(touchPoint) then
            return false
        end
        self.startX = touchPoint.x
        self.startY = touchPoint.y
    end
    if self.rect:isContainsPoint(touchPoint) then
        if self.canTouch and not self:canTouch(touchPoint) then
            return false
        end
        local result = true
        if self.onBegan then
           local ret = self:onBegan(touchPoint)
           if ret ~= nil then
               result = ret
           end
        end
        if result then
            setIsPressState(self, true)
        end
        return result
    end

    return false
end

local function onTouchMoved(self, touchPoint)
    if self.isCheckMoving then
        local deltaX = touchPoint.x - self.startX
        local deltaY = touchPoint.y - self.startY
        if deltaX * deltaX + deltaY * deltaY > 225 then
            setIsPressState(self, false)
            self.isMoving = true
        end
    end
    local onMoved = self.onMoved
    if onMoved then
        onMoved(self, self.deltaXY, self.location)
    end
end

local function onTouchEnded(self, touchPoint)
    setIsPressState(self, false)
    if self.isMoving or self.m_scene.m_lockTouch > 0 then
        self.isMoving = false
        if self.onCanceled then
           self:onCanceled()
        end
    elseif self.rect:isContainsPoint(touchPoint) then
        if self.onEnded then
            self:onEnded(touchPoint)
            if self.sound then
                cc.Audio:playEffect(self.sound)
            end
        end
    else
        if self.onCanceled then
           self:onCanceled()
        end
    end
end

lib_touchListener.setCallback("OnBegan", function (this, x, y, id)
    local self = objectRegistry[this]
    self.location.x = x
    self.location.y = y
    self.touchID = id
	self.startX = x
	self.startY = y
    if self.m_scene.m_lockGuide > 0 and self.m_unlock == 0 then
        return false
    end
    local result = true
    if self.node then
        result = onTouchBegan(self, self.location)
    else
        if self.isEnabled == false then
            result = false
        else
            local onBegan = self.onBegan
            if onBegan then
                local rt = onBegan(self, self.location)
                if rt ~= nil then
                    result = rt
                end
            end
        end
    end
    return result
end)

lib_touchListener.setCallback("OnEnded", function (this, x, y, id)
    local self = objectRegistry[this]
    self.location.x = x
    self.location.y = y
    self.touchID = id
    if self.node then
        onTouchEnded(self, self.location)
    else
        local onEnded = self.onEnded
        if onEnded then
            onEnded(self, self.location)
        end
    end
end)

lib_touchListener.setCallback("OnMoved", function (this, x, y, dx, dy, id)
    local self = objectRegistry[this]
    self.location.x = x
    self.location.y = y
    self.deltaXY.x = x
    self.deltaXY.y = y
    self.touchID = id
    if self.node then
        onTouchMoved(self, self.location)
    else
        local onMoved = self.onMoved
        if onMoved then
            onMoved(self, self.deltaXY, self.location)
        end
    end
end)

local function getTouchRect(self)
    if self.getRect then
        return self:getRect()
    else
        return self.sizeNode:getBoundingBox()
    end
end

local function getStartTouchLocation(self)
	return self.startX , self.startY
end

function This:create(touch)
    touch.deltaScale = touch.deltaScale or 0.05
    local listener = lib_touchListener.create()
    touch.getTouchRect = getTouchRect
    touch.m_scene = g_scene
    touch.m_unlock = 0
    if touch.node then
        if touch.normalScaleX == nil then
            touch.normalScaleX = touch.node:getScaleX()
            touch.normalScaleY = touch.node:getScaleY()
        end
        
        if not touch.rect then
            touch.rect = touch:getTouchRect()
        end
        if touch.isContainer then
            touch.isDynamic = true
            if touch.isSwallow == nil then
                touch.isSwallow = false
            end
        end
    end
    touch.setEnabled = self.setEnabled
	touch.getStartTouchLocation = getStartTouchLocation
    listener = registerObject(listener, nil, touch)
    if touch.isSwallow == false then
        lib_touchListener.setSwallowTouches(listener[0], false)
    else
        lib_touchListener.setSwallowTouches(listener[0], true)
    end
    listener.location = Vec()
    listener.deltaXY = Vec()
    listener._NAME = "TouchListener"
	g_weakTable[tostring(listener)] = listener
    return listener
end

function This:setEnabled(isEnabled)
    lib_touchListener.setEnabled(self[0], isEnabled)
end
