local spriteFrameCache = require("cc.SpriteFrameCache")
local castNode = cc.Node.castNode
local objectRegistry = cc.objectRegistry
local rawget = rawget
local function onEventCallback(this, eventType)
    local self = objectRegistry[this]
    local listener = rawget(self, "m_eventListener")
    if listener then
        listener(self, eventType)
    end
end
lib2d.ccui.setCallback("EditBox", onEventCallback)
lib2d.ccui.setCallback("CheckBox", onEventCallback)
lib2d.ccui.setCallback("PageView", onEventCallback)
lib2d.ccui.setCallback("ListView", onEventCallback)
lib2d.ccui.setCallback("ScrollView", onEventCallback)
lib2d.ccui.setCallback("VideoPlayer", onEventCallback)

-- ccWidget begin
local ccWidget = class("cc.Widget", "cc.Node")
cc.addCppNodeClass(ccWidget)
cc.Widget = ccWidget
local lib_widget = lib2d.ccWidget
function ccWidget:createNode()
    return lib_widget.create()
end
-- ccWidget end


-- ccText begin
local ccText = class("cc.Text", "cc.Node")
cc.addCppNodeClass(ccText)
cc.Text = ccText
local lib_text = lib2d.ccText
function ccText:setString(text)
    lib_text.setString(self[0], tostring(text))
end
function ccText:getString(text)
    return lib_text.getString(self[0])
end
function ccText:setFlippedX(isFlipped)
    return lib_text.setFlippedX(self[0], isFlipped)
end
function ccText:setFlippedY(isFlipped)
    return lib_text.setFlippedY(self[0], isFlipped)
end
-- ccText end

local ccRichTextEx = class("cc.RichTextEx","cc.Node")
cc.addCppNodeClass(ccRichTextEx)
cc.RichTextEx = ccRichTextEx
local lib_richText = lib2d.ccRichTextEx

local TxtLbl = {"color","fontName","fontSize","outLine","underLine"}
local function parseTextLabel(text,pos,label)
	local result = {}
	local curpos = pos
	local firstpos = pos
	while curpos <= #text do
		local c = string.sub(text,curpos,curpos)
		if c == "<" then
			local endsub = string.sub(text, curpos , curpos + 2)
			if endsub == "</>" then
				label.str = string.sub(text,firstpos,curpos - 1)
				table.insert(result,label)
				curpos = curpos + 3
				return result , curpos
			else
				local endpos = curpos + 1
				while endpos <= #text do
					if string.sub(text,endpos,endpos) == ">" then
						break
					end
					endpos = endpos + 1
				end

				if string.sub(text,endpos,endpos) == ">" then
					local t = {}
					local sub = string.sub(text,curpos+1,endpos-1) .. ","
					for i , tbl in ipairs(TxtLbl) do
						local pattern = string.format("%s%%s*=%%s*(.-)%%s*,",tbl)
						local _,_,value = string.find(sub,pattern,1)
						t[tbl]=value
					end

					if firstpos < curpos and label then
						local l = table.clone(label)
						l.str = string.sub(text,firstpos,curpos - 1)
						table.insert(result,l)
					end

					local ret , newpos = parseTextLabel(text,endpos + 1, t)
					for i , info in ipairs(ret) do
						table.insert(result,info)
					end
					curpos = newpos
					firstpos = newpos
				end
			end
		else
			curpos = curpos + 1
		end
	end

	if firstpos < curpos then
		table.insert(result,{str = string.sub(text,firstpos,curpos - 1)})
	end	

	return result, pos
end

function ccRichTextEx:createNode()
    return lib_richText.create()
end

function ccRichTextEx:insertElment(str)

end

function ccRichTextEx:insertText(str)
	local result = parseTextLabel(str,1)
	local Color = require("cc.Color")
	for i , info in ipairs(result) do
		local color = cc.ColorList[info.color or "A0"]
		lib_richText.insertText(self[0],info.str,info.fontName or "Fonts/Normal.ttf", info.fontSize or 16,color.r,color.g,color.b,info.outLine or 0,info.underLine or false)
	end
end
function ccRichTextEx:insertImage(filePath)
    assert(cc.FileUtils:isFileExist(filePath))
    lib_richText.insertImage(self[0],filePath)
end
function ccRichTextEx:setMultiLineMode(b,width)
    if b then 
        assert(width)
        self:setContentSize(width,0)
    end
    lib_richText.ignoreContentAdaptWithSize(self[0],not b)
end
function ccRichTextEx:formatText()
    lib_richText.formatText(self[0])
end
function ccRichTextEx:clear()
    lib_richText.clear(self[0])
end



-- ccTextField begin
local ccTextField = class("cc.TextField", "cc.Node")
cc.addCppNodeClass(ccTextField)
cc.TextField = ccTextField
local lib_textField = lib2d.ccTextField
function ccTextField:setString(text)
    lib_textField.setString(self[0], tostring(text))
end
function ccTextField:getString(text)
    return lib_textField.getString(self[0])
end
function ccTextField:setFlippedX(isFlipped)
    return lib_textField.setFlippedX(self[0], isFlipped)
end
function ccTextField:setFlippedY(isFlipped)
    return lib_textField.setFlippedY(self[0], isFlipped)
end
function ccTextField:setMaxLength(len)
    return lib_textField.setMaxLength(self[0],len)
end

-- ccTextField end


-- ccTextBMFont begin
local ccTextBMFont = class("cc.TextBMFont", "cc.Node")
cc.addCppNodeClass(ccTextBMFont)
cc.TextBMFont = ccTextBMFont
local lib_textBMFont = lib2d.ccTextBMFont
function ccTextBMFont:setString(text)
    lib_textBMFont.setString(self[0], tostring(text))
end
function ccTextBMFont:getString(text)
    return lib_textBMFont.getString(self[0])
end
-- ccTextBMFont end


-- ccEditBox begin
local ccEditBox = class("cc.EditBox", "cc.Node")
cc.addCppNodeClass(ccEditBox)
cc.EditBox = ccEditBox
local lib_editBox = lib2d.ccEditBox
function ccEditBox:createNode(width, height, backgroundFile)
    return lib_editBox.create(width, height, backgroundFile)
end
function ccEditBox:setText(text)
    lib_editBox.setText(self[0], tostring(text))
end
function ccEditBox:getText(text)
    return lib_editBox.getText(self[0])
end
function ccEditBox:setString(text)
    lib_editBox.setText(self[0], tostring(text))
end
function ccEditBox:getString()
    local text = lib_editBox.getText(self[0])
    if text then
        return tonumber(text) or text
    end
end
function ccEditBox:setFont(fontName, fontSize)
    return lib_editBox.setFont(self[0], fontName, fontSize)
end
function ccEditBox:setEventListener(listener)
    self.m_eventListener = listener
end
-- ccEditBox end


-- ccCheckBox begin
local ccCheckBox = class("cc.CheckBox", "cc.Node")
cc.addCppNodeClass(ccCheckBox)
cc.CheckBox = ccCheckBox
local lib_checkBox = lib2d.ccCheckBox
function ccCheckBox:createNode(backGround, backGroundSeleted, cross, backGroundDisabled, frontCrossDisabled)
    return lib_checkBox.create(backGround, backGroundSeleted, cross, backGroundDisabled, frontCrossDisabled)
end
function ccCheckBox:setSelected(isSelected)
    lib_checkBox.setSelected(self[0], isSelected)
end
function ccCheckBox:isSelected(text)
    return lib_checkBox.isSelected(self[0])
end
function ccCheckBox:addEventListener(listener)
    self.m_eventListener = listener
    lib_checkBox.addEventListener(self[0])
end
-- ccCheckBox end


-- ccCheckBox begin
local ccImageView = class("cc.ImageView", "cc.Node")
cc.addCppNodeClass(ccImageView)
cc.ImageView = ccImageView
local lib_imageView = lib2d.ccImageView
function ccImageView:createNode(imageFileName)
    return lib_imageView.create(imageFileName, 0)
end
function ccImageView:setFlippedX(isFlipped)
    return lib_imageView.setFlippedX(self[0], isFlipped)
end
function ccImageView:setFlippedY(isFlipped)
    return lib_imageView.setFlippedY(self[0], isFlipped)
end
function ccImageView:loadTexture(fileName, textureType)
    return lib_imageView.loadTexture(self[0], textureType)
end
function ccImageView:setImage(fileName)
    local spriteFrame = spriteFrameCache.m_frames[fileName]
    if spriteFrame then
        lib_imageView.loadTexture(self[0], fileName, 1)
    else
        lib_imageView.loadTexture(self[0], fileName, 0)
    end
end
-- ccCheckBox end


-- ccLayout begin
local ccLayout = class("cc.Layout", "cc.Node")
cc.addCppNodeClass(ccLayout)
cc.Layout = ccLayout
local lib_layout = lib2d.ccLayout
function ccLayout:createNode()
    return lib_layout.create()
end
function ccLayout:setBackGroundImage(fileName, textureType)
    lib_layout.setBackGroundImage(self[0], fileName, textureType or 0)
end
-- ccLayout end


-- ccPageView begin
local ccPageView = class("cc.PageView", "cc.Node")
ccPageView.IsContainer = true
cc.addCppNodeClass(ccPageView)
cc.PageView = ccPageView
local lib_pageView = lib2d.ccPageView
function ccPageView:createNode()
    return lib_pageView.create()
end
function ccPageView:addEventListener(listener)
    self.m_eventListener = listener 
    lib_pageView.addEventListener(self[0])
end
function ccPageView:setTouchEnabled(isEnabled)
    lib_pageView.setTouchEnabled(self[0], isEnabled)
end
function ccPageView:isTouchEnabled()
    return lib_pageView.isTouchEnabled(self[0])
end
function ccPageView:addPage(page)
    lib_pageView.addPage(self[0], page[0])
end
function ccPageView:getPage(index)
    return castNode(lib2d.ccPageView.getPage(self[0], index), true)
end
function ccPageView:getPageCount()
    return lib_pageView.getPageCount(self[0])
end
function ccPageView:getPages()
    local pages = lib_pageView.getPages(self[0])
    for i = 1, #pages do
        pages[i] = castNode(pages[i], true)
    end
    return pages
end
function ccPageView:removeAllPages()
    lib_pageView.removeAllPages(self[0])
end
function ccPageView:scrollToPage(index)
    lib_pageView.scrollToPage(self[0], index)
end
function ccPageView:setBackGroundImage(fileName, textureType)
    lib_pageView.setBackGroundImage(self[0], fileName, texType or 0)
end
function ccPageView:removeBackGroundImage()
    lib_pageView.removeBackGroundImage(self[0])
end
function ccPageView:getCurrentPageIndex()
    return lib_pageView.getCurPageIndex(self[0])
end
function ccPageView:setCurrentPageIndex(pageIndex)
    lib_pageView.setCurrentPageIndex(self[0], pageIndex)
end
-- ccPageView end


-- ccVideoPlayer begin
local ccVideoPlayer = class("cc.VideoPlayer", "cc.Node")
cc.addCppNodeClass(ccVideoPlayer)
cc.VideoPlayer = ccVideoPlayer
local lib_videoPlayer = lib2d.ccVideoPlayer
function ccVideoPlayer:createNode(fileName)
    return lib_videoPlayer.create(fileName)
end
function ccVideoPlayer:onEnter()
    cc.Node.onEnter(self)
    self:setAnchorPoint(0.5, 0.5)
    self:setKeepAspectRatioEnabled(true)
    self:setFullScreenEnabled(true)
    self:setContentSize(cc.visible.width, cc.visible.height)
end
function ccVideoPlayer:setKeepAspectRatioEnabled(isEnabled)
    return lib_videoPlayer.setKeepAspectRatioEnabled(self[0], isEnabled)
end
function ccVideoPlayer:setFullScreenEnabled(isEnabled)
    return lib_videoPlayer.setFullScreenEnabled(self[0],isEnabled)
end
function ccVideoPlayer:play()
    return lib_videoPlayer.play(self[0])
end
function ccVideoPlayer:stop()
    return lib_videoPlayer.stop(self[0])
end
function ccVideoPlayer:pause()
    return lib_videoPlayer.pause(self[0])
end
function ccVideoPlayer:resume()
    return lib_videoPlayer.resume(self[0])
end
function ccVideoPlayer:addEventListener(listener)
    self.m_eventListener = listener
    return lib_videoPlayer.addEventListener(self[0])
end
-- ccVideoPlayer end


-- ccLayerColor begin
local ccLayerColor = class("cc.LayerColor", "cc.Node")
cc.addCppNodeClass(ccLayerColor)
cc.LayerColor = ccLayerColor
local lib_layerColor = lib2d.ccLayerColor
local BLACK_COLOR = cc.Color.BLACK
function ccLayerColor:createNode(color, width, height)
    if not color then
        color = BLACK_COLOR
    end
    
    if not width or not height then
        width = width or cc.visible.width
        height = height or cc.visible.height
    end
    return lib_layerColor.create(color.r, color.g, color.b, color.a, width, height)
end

------------------------WebView-----------------------

local ccWebView = class("cc.WebView","cc.Node")
cc.addCppNodeClass(ccWebView)
cc.WebView = ccWebView
local lib_webView = lib2d.ccWebView

function ccWebView:createNode()
	return lib_webView.create()
end

function ccWebView:setJavascriptInterfaceScheme(str)
	lib_webView.setJavascriptInterfaceScheme(self[0],str)
end

function ccWebView:loadHTMLString(str,url)
	lib_webView.loadHTMLString(self[0],str,url)
end

function ccWebView:loadURL(url)
	print("load url " .. url)
	lib_webView.loadURL(self[0],url)
end

function ccWebView:loadFile(filename)
	lib_webView.loadFile(self[0],filename)
end

function ccWebView:stopLoading()
	lib_webView.stopLoading(self[0])
end

function ccWebView:reload()
	lib_webView.reload(self[0])
end

function ccWebView:canGoBack()
	return lib_webView.canGoBack(self[0])
end

function ccWebView:canGoForward()
	return lib_webView.canGoForward(self[0])
end

function ccWebView:goBack()
	lib_webView.goBack(self[0])
end

function ccWebView:goForward()
	lib_webView.goForward(self[0])
end

function ccWebView:setScalesPageToFit(bFit)
	bFit = bFit or false
	lib_webView.setScalesPageToFit(self[0],bFit);
end

function ccWebView:setBackgroundColor(color)
	color = color or "#00000000"
	lib_webView.setBackgroundColor(self[0],color)
end

-- ccLayerColor end

