local This, super   = class(..., "xgame.Dispatcher")
local PluginEvent   = require "xgame.PluginEvent"
local Json          = require("Json")

local impl

function This:init()
    super.init(self);
    local function listen(event)
        local xruntime = require "xgame.xruntime"
        xruntime.on(event, function (...)
            print(event)
            self:dispatch(event, ...)
        end)
    end
    listen(PluginEvent.SWITCH_SUCCESS)
    listen(PluginEvent.KEYBOARD_CHANGE)
    
    impl:setDispatcher(function (...)
        self:_didResponse(...)
    end)
end

function This:pay(order, isTest)
    impl:pay(
    	tostring(assert(order.pid, "no product id")),
        tostring(assert(order.roleId, "no role id")),
        tostring(assert(order.name, "no name")),
        tostring(assert(order.level, "no level")),
        tostring(assert(order.server, "no server")),
        tostring(assert(order.orderId, "no order id")),
        tostring(assert(order.price, "no price")),
        tostring(assert(order.subject, "no subject"))
    )
end

function This:upload(roleInfo)
    if os.isIos then
        impl:upload(
            tostring(assert(roleInfo.roleId, "no role id")),
            tostring(assert(roleInfo.roleName, "no name")),
            tostring(assert(roleInfo.roleLevel, "no level")),
            tostring(assert(roleInfo.serverId, "no server")),
            tostring(assert(roleInfo.serverName, "no server name"))
        )
    else
        impl:upload(
	        tostring(assert(roleInfo.roleId, "no role id")),
	        tostring(assert(roleInfo.roleName, "no name")),
	        tostring(assert(roleInfo.roleLevel, "no level")),
	        tostring(assert(roleInfo.serverId, "no server")),
	        tostring(assert(roleInfo.serverName, "no server name")),
	        tostring(assert(roleInfo.serverName, "no balance")),
	        tostring(assert(roleInfo.vip, "no vip level")),
	        tostring(assert(roleInfo.roleTime, "no time")),
	        tostring(assert(roleInfo.scene, "no scene id"))
	    )
    end
end

function This:record(event, param)
    impl:record(event, param)
end

function This:auth()
    impl:auth()
end

function This:logout(order)
    impl:logout()
end

function This:checkBill()
    impl:checkBill()
end

function This:consumeProduct(token)
    impl:consumeProduct(
        tostring(assert(token, "no token"))
    )
end

function This:warm(text, callback)
    impl:alert(text, callback)
end

function This:_didResponse(action, infostr)
    print("response: " ..  action, infostr)
    local response = Json.decode(infostr)
    if action == 'auth' then
        if response.status == 0 then
            self:dispatch(PluginEvent.AUTH_SUCCESS, infostr)
        else
            self:dispatch(PluginEvent.AUTH_FAILURE)
        end
    elseif action == 'pay' then
        if response.status == 0 then
            self:dispatch(PluginEvent.PAY_SUCCESS, response)
        elseif response.status == 60000 or response.status == -1 then
            self:dispatch(PluginEvent.PAY_CANCEL)
        else
            self:dispatch(PluginEvent.PAY_FAILURE, response.status)
        end
    end
end

if os.isAndroid then
    local xluaj = require "xgame.xluaj"
    local xruntime = require "xgame.xruntime"

    local inst = xluaj.new("kernel/plugins/ejyx/EjGame")
    impl = {}
    
    function impl:setDispatcher(callback)
        impl.callback = callback
    end

    function impl:auth()
        local status = xruntime.getPermissionStatus('android.permission.READ_PHONE_STATE')
        if status == xruntime.PermissionStatus.DENIED or status == xruntime.PermissionStatus.RESTRICTED then
            xruntime.alert( '温馨提示', 
                '没有权限访问您的手机状态，请前往权限管理进行设置。', 
                '去设置', 
                '取消', 
                function (status) if status then os.native_call("openUrl", 'app-settings') end end)
            return
        elseif status == xruntime.PermissionStatus.NOT_DETERMINED then
            impl:requestPermission(function( result )
                if result then
                    impl:auth()
                end
            end)
            return
        end

	    inst.login(function (...)
	        impl.callback("auth", ...)
	    end)

    end

    function impl:pay(productId, roleId, name, level, server, orderId, price, subject)
        inst.pay(productId, roleId, name, level, server, orderId, price, subject, function (...)
            impl.callback("pay", ...)
        end)
    end

    function impl:upload(roleId, name, level, server, serverName, balance, vipLevel, time, scene)
        inst.upload(roleId, name, level, server, serverName, balance, vipLevel, time, scene)
    end

    function impl:record(eventId, param)
        inst.record(eventId, param)
    end

    function impl:consumeProduct(token)
        inst.consumableProduct(token)
    end

    function impl:checkBill()
        inst.checkObtainPurchases("", function (...)
            impl.callback("pay", ...)
        end)
    end

    function impl:requestPermission(callback)
        xruntime.requestPermission('android.permission.READ_PHONE_STATE', function (status)
            if status == xruntime.PermissionStatus.AUTHORIZED then
                callback(true)
            else
                local alertCallback = function (status)
                    if status then
                        os.native_call("openUrl", 'app-settings')
                    else
                        callback(false)
                    end
                end
                xruntime.alert(
                    '温馨提示',
                    '没有权限访问您的手机状态，请前往权限管理进行设置。',
                    '去设置',
                    '取消', alertCallback)
            end
        end)
    end

    function impl:logout()
        inst.logout()
    end

    function impl:setRoleInfo(server, serverId, roleId, roleName)
        inst.setRoleInfo(server, serverId, roleId, roleName)
    end

    function impl:alert(text, callback)
        print("---======== ANDROID ALERT ===============---")
        xruntime.alert('温馨提示', text, '确定', '取消', callback)
    end
else
    impl = require "kernel.plugins.ejyx"
    impl:init("39", "1ece53b9be2de3a5da4e03bf4d4a72b8", "zxhmzzios")
    function impl:alert(text, callback)
        local xruntime = require "xgame.xruntime"
        xruntime.alert('温馨提示', text, '确定', '取消', callback)
    end
end