local This, super   = class(..., "xgame.Dispatcher")
local PluginEvent   = require "xgame.PluginEvent"
local Json          = require("Json")

local impl

function This:init()
    super.init(self);
    local function listen(event)
        if os.isAndroid then
            local xruntime = require "xgame.xruntime"
            xruntime.on(event, function (...)
                self:dispatch(event, ...)
            end)
        end
    end
    listen(PluginEvent.SWITCH_SUCCESS)
    listen(PluginEvent.PAY_SUCCESS)

    impl:setDispatcher(function (...)
        self:_didResponse(...)
    end)
end

function This:pay(order)
    impl:pay(
    	tostring(assert(order.pid, "no product id")),
        tostring(assert(order.orderId, "no order id"))
    )
end

function This:auth()
    impl:auth()
end

function This:logout(order)
    impl:logout()
end

function This:checkBill()
    impl:checkBill()
end

function This:consumeProduct(token)
    impl:consumeProduct(
        tostring(assert(token, "no token"))
    )
end

function This:warm(text, callback)
    impl:alert(text, callback)
end

function This:_didResponse(action, infostr)
    print("response: " ..  action, infostr)
    local response = Json.decode(infostr)
    if action == 'auth' then
        if response.status == 0 then
            self:dispatch(PluginEvent.AUTH_SUCCESS, infostr)
        else
            self:dispatch(PluginEvent.AUTH_FAILURE)
        end
    elseif action == 'pay' then
        if response.status == 0 then
            self:dispatch(PluginEvent.PAY_SUCCESS, response)
        elseif response.status == 60000 then
            self:dispatch(PluginEvent.PAY_CANCEL)
        else
            self:dispatch(PluginEvent.PAY_FAILURE, response.status)
        end
    end
end

if os.isAndroid then
    local xluaj = require "xgame.xluaj"
    local xruntime = require "xgame.xruntime"

    local inst = xluaj.new("kernel/plugins/huawei/Huawei")
    impl = {}

    function impl:setDispatcher(callback)
        impl.callback = callback
    end

    function impl:auth()
        local status = xruntime.getPermissionStatus('android.permission.READ_PHONE_STATE')
        if status == xruntime.PermissionStatus.DENIED or status == xruntime.PermissionStatus.RESTRICTED then
            xruntime.alert( '温馨提示', 
                '没有权限访问您的手机状态，请前往权限管理进行设置。', 
                '去设置', 
                '取消', 
                function (status) if status then os.native_call("openUrl", 'app-settings') end end)
            return
        end

	    inst.login(function (...)
	        impl.callback("auth", ...)
	    end)

    end

    function impl:pay(pid, orderNo)
        inst.pay(pid, orderNo, function (...)
            impl.callback("pay", ...)
        end)
    end

    function impl:consumeProduct(token)
        inst.consumableProduct(token)
    end

    function impl:checkBill()
        inst.checkObtainPurchases("", function (...)
            impl.callback("pay", ...)
        end)
    end

    function impl:requestPermission(callback)
        xruntime.requestPermission('android.permission.READ_PHONE_STATE', function (status)
            if status == xruntime.PermissionStatus.AUTHORIZED then
                callback(true)
            else
                local alertCallback = function (status)
                    if status then
                        os.native_call("openUrl", 'app-settings')
                    else
                        callback(false)
                    end
                end
                xruntime.alert(
                    '温馨提示',
                    '没有权限访问您的手机状态，请前往权限管理进行设置。',
                    '去设置',
                    '取消', alertCallback)
            end
        end)
    end

    function impl:logout()
        inst.logout()
    end

    function impl:setRoleInfo(server, serverId, roleId, roleName)
        inst.setRoleInfo(server, serverId, roleId, roleName)
    end

    function impl:alert(text, callback)
        xruntime.alert('温馨提示', text, '确定', '取消', callback)
    end

else
    impl = setmetatable({}, {__index = function (_, func)
        return function ()
            trace("function 'kernel.plugins.huawei.%s' not supported", func)
        end
    end})
end